/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:00:25.982090
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62,x63,x64,x65,x66,x67,x68,x69,x70,x71,x72,x73;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.0600000000000000))*(x5));
x15=((IkReal(0.0450000000000000))*(x1));
x16=((IkReal(1.00000000000000))*(x5));
x17=((IkReal(0.0600000000000000))*(x9));
x18=((IkReal(0.300000000000000))*(x1));
x19=((IkReal(0.0600000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x11));
x21=((IkReal(0.0450000000000000))*(x5));
x22=((IkReal(1.00000000000000))*(x1));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(0.0600000000000000))*(x8));
x25=((IkReal(1.00000000000000))*(x6));
x26=((x0)*(x4));
x27=((x0)*(x2));
x28=((x2)*(x7));
x29=((x3)*(x4));
x30=((x0)*(x7));
x31=((x2)*(x3));
x32=((x3)*(x7));
x33=((IkReal(1.00000000000000))*(x29));
x34=((IkReal(0.0450000000000000))*(x29));
x35=((IkReal(0.0450000000000000))*(x26));
x36=((x22)*(x6));
x37=((x32)*(x6));
x38=((x4)*(x7)*(x9));
x39=((x16)*(x30));
x40=((x25)*(x30));
x41=((x15)*(x27));
x42=((x16)*(x32));
x43=((x25)*(x32));
x44=((x15)*(x31));
x45=((((IkReal(-1.00000000000000))*(x33)))+(((x1)*(x27))));
x46=((x26)+(((x1)*(x31))));
x47=((x27)+(((IkReal(-1.00000000000000))*(x22)*(x29))));
x48=((((IkReal(-1.00000000000000))*(x36)))+(((x28)*(x5))));
x49=((x33)+(((IkReal(-1.00000000000000))*(x22)*(x27))));
x50=((((IkReal(-1.00000000000000))*(x31)))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x51=((((IkReal(-1.00000000000000))*(x26)))+(((IkReal(-1.00000000000000))*(x22)*(x31))));
x52=((((x1)*(x16)))+(((x25)*(x28))));
x53=((IkReal(-1.00000000000000))*(x52));
x54=((x35)+(x44));
x55=((x46)*(x6));
x56=((x11)*(x48));
x57=((x45)*(x6));
x58=((x5)*(x51));
x59=((x50)*(x9));
x60=((x53)*(x8));
x61=((((IkReal(-1.00000000000000))*(x39)))+(x57));
x62=((((IkReal(-1.00000000000000))*(x42)))+(x55));
x63=((((IkReal(-1.00000000000000))*(x40)))+(((x49)*(x5))));
x64=((((IkReal(-1.00000000000000))*(x43)))+(x58));
x65=((((x4)*(x7)*(x8)))+(((x52)*(x9))));
x66=((x38)+(x60));
x67=((x61)*(x8));
x68=((x11)*(x64));
x69=((((x47)*(x9)))+(((x62)*(x8))));
x70=((((x47)*(x8)))+(((x9)*(((((IkReal(-1.00000000000000))*(x25)*(x46)))+(x42))))));
x71=((x59)+(x67));
x72=((x12)*(x69));
x73=((((x50)*(x8)))+(((x9)*(((x39)+(((IkReal(-1.00000000000000))*(x57))))))));
eerot[0]=((((x10)*(x73)))+(((x13)*(((((x11)*(x63)))+(((x12)*(x71))))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x63)))+(((IkReal(-1.00000000000000))*(x23)*(x71)))))))+(((x13)*(x73))));
eerot[2]=((((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x49)))+(x40)))))+(((x11)*(x71))));
eetrans[0]=((IkReal(0.220000000000000))+(((IkReal(-1.00000000000000))*(x34)))+(((x12)*(((((x19)*(x30)))+(((IkReal(-1.00000000000000))*(x14)*(x49)))))))+(((x11)*(((((x17)*(x50)))+(((x24)*(x61)))))))+(((x21)*(x30)))+(((IkReal(0.300000000000000))*(x30)*(x6)))+(((IkReal(0.550000000000000))*(x30)))+(x41)+(((x5)*(((((x18)*(x27)))+(((IkReal(-0.300000000000000))*(x29)))))))+(((x6)*(((((IkReal(-1.00000000000000))*(x41)))+(x34))))));
eerot[3]=((((x10)*(x70)))+(((x13)*(((x72)+(x68))))));
eerot[4]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x64)))+(((IkReal(-1.00000000000000))*(x23)*(x69)))))))+(((x13)*(x70))));
eerot[5]=((((x11)*(x69)))+(((x12)*(((x43)+(((IkReal(-1.00000000000000))*(x16)*(x51))))))));
eetrans[1]=((IkReal(0.140000000000000))+(((x5)*(((((x18)*(x31)))+(((IkReal(0.300000000000000))*(x26)))))))+(((IkReal(-1.00000000000000))*(x54)*(x6)))+(((x21)*(x32)))+(((IkReal(0.550000000000000))*(x32)))+(x54)+(((x12)*(((((x19)*(x32)))+(((IkReal(-1.00000000000000))*(x14)*(x51)))))))+(((x11)*(((((x17)*(x47)))+(((x24)*(x62)))))))+(((IkReal(0.300000000000000))*(x37))));
eerot[6]=((((x13)*(((((x12)*(x66)))+(x56)))))+(((x10)*(x65))));
eerot[7]=((((x13)*(x65)))+(((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x48)))+(((IkReal(-1.00000000000000))*(x23)*(x66))))))));
eerot[8]=((((x11)*(x66)))+(((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x28)))+(x36))))));
IkReal x74=((IkReal(0.0450000000000000))*(x28));
eetrans[2]=((IkReal(0.346000000000000))+(((x18)*(x6)))+(((x11)*(((((x24)*(x53)))+(((x17)*(x4)*(x7)))))))+(((IkReal(-0.300000000000000))*(x28)*(x5)))+(((IkReal(-1.00000000000000))*(x74)))+(((x12)*(((((x1)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x28)))))))+(((x6)*(x74)))+(((IkReal(0.550000000000000))*(x1)))+(((x15)*(x5))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {6}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij6[0] = -1; _ij6[1] = -1; _nj6 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j6=pfree[0]; cj6=cos(pfree[0]); sj6=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((IkReal(-0.220000000000000))+(((IkReal(-0.0600000000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((IkReal(-0.140000000000000))+(py)+(((IkReal(-0.0600000000000000))*(r12))));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.346000000000000))+(((IkReal(-0.0600000000000000))*(r22)))+(pz));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x75=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j3array[0]=((IkReal(1.80132261653346))+(((IkReal(1.00000000000000))*(x75))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(4.94291527012326))+(((IkReal(-1.00000000000000))*(x75))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(20.0000000000000))*((npz)*(npz))))+(((IkReal(20.0000000000000))*((cj6)*(cj6))*((npx)*(npx))))+(((IkReal(-40.0000000000000))*(cj6)*(npx)*(npy)*(sj6)))+(((IkReal(20.0000000000000))*((npy)*(npy))*((sj6)*(sj6))))));
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((gconst2)*(((((IkReal(17.0000000000000))*(npy)*(sj6)))+(((IkReal(-17.0000000000000))*(cj6)*(npx))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(17.0000000000000))*(gconst2)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst2)*(((((IkReal(17.0000000000000))*(npy)*(sj6)))+(((IkReal(-17.0000000000000))*(cj6)*(npx)))))), ((IkReal(17.0000000000000))*(gconst2)*(npz)));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x76=IKsin(j5);
IkReal x77=IKcos(j5);
IkReal x78=((npy)*(sj6));
IkReal x79=((cj6)*(npx));
IkReal x80=((IkReal(1.00000000000000))*(x76));
evalcond[0]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(x79)*(x80)))+(((x76)*(x78)))+(((npz)*(x77))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x77)*(x79)))+(((IkReal(-1.00000000000000))*(npz)*(x80)))+(((x77)*(x78))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x81=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x81));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x81))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x82=IKsin(j4);
IkReal x83=IKcos(j4);
IkReal x84=((IkReal(1.00000000000000))*(x83));
IkReal x85=((IkReal(1.00000000000000))*(x82));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x84)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x84)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x85)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x85)))+(((cj5)*(npy)*(sj6)*(x82))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((((IkReal(20.0000000000000))*((npz)*(npz))))+(((IkReal(20.0000000000000))*((cj6)*(cj6))*((npx)*(npx))))+(((IkReal(-40.0000000000000))*(cj6)*(npx)*(npy)*(sj6)))+(((IkReal(20.0000000000000))*((npy)*(npy))*((sj6)*(sj6))))));
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((gconst3)*(((((IkReal(17.0000000000000))*(npy)*(sj6)))+(((IkReal(-17.0000000000000))*(cj6)*(npx))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(17.0000000000000))*(gconst3)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst3)*(((((IkReal(17.0000000000000))*(npy)*(sj6)))+(((IkReal(-17.0000000000000))*(cj6)*(npx)))))), ((IkReal(17.0000000000000))*(gconst3)*(npz)));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x159=IKsin(j5);
IkReal x160=IKcos(j5);
IkReal x161=((npy)*(sj6));
IkReal x162=((cj6)*(npx));
IkReal x163=((IkReal(1.00000000000000))*(x159));
evalcond[0]=((IkReal(-0.850000000000000))+(((npz)*(x160)))+(((IkReal(-1.00000000000000))*(x162)*(x163)))+(((x159)*(x161))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x160)*(x162)))+(((IkReal(-1.00000000000000))*(npz)*(x163)))+(((x160)*(x161))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x164=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x164));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x164))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x165=IKsin(j4);
IkReal x166=IKcos(j4);
IkReal x167=((IkReal(1.00000000000000))*(x166));
IkReal x168=((IkReal(1.00000000000000))*(x165));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)*(x168)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x167)))+(((cj5)*(npy)*(sj6)*(x165)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x168)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x167))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
sj4array[0]=((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(6.15202020000000e+16))*(cj6)*(npx)*(npy)*(sj6)))+(((IkReal(-3.07601010000000e+16))*((npy)*(npy))*((sj6)*(sj6))))+(((IkReal(-3.07601010000000e+16))*((cj6)*(cj6))*((npx)*(npx))))+(((IkReal(-3.07601010000000e+16))*((npz)*(npz))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((npy)*(npy))*((sj6)*(sj6))))+(((IkReal(-1.00000000000000))*((npz)*(npz))))+(((IkReal(-1.00000000000000))*((cj6)*(cj6))*((npx)*(npx))))+(((IkReal(2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x169=((npy)*(sj6));
IkReal x170=((IkReal(74227.0000000000))*(cj4));
IkReal x171=((cj6)*(npx));
if( IKabs(((gconst4)*(((((npz)*(x170)))+(((IkReal(7.69002526201833e+15))*(x169)))+(((IkReal(-7.69002526201833e+15))*(x171))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(7.69002526201833e+15))*(npz)))+(((x170)*(x171)))+(((IkReal(-1.00000000000000))*(x169)*(x170))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst4)*(((((npz)*(x170)))+(((IkReal(7.69002526201833e+15))*(x169)))+(((IkReal(-7.69002526201833e+15))*(x171)))))), ((gconst4)*(((((IkReal(7.69002526201833e+15))*(npz)))+(((x170)*(x171)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x172=IKsin(j5);
IkReal x173=IKcos(j5);
IkReal x174=((npx)*(sj4));
IkReal x175=((cj6)*(npy));
IkReal x176=((IkReal(1.00000000000000))*(npx));
IkReal x177=((cj4)*(x176));
IkReal x178=((cj6)*(x173));
IkReal x179=((npy)*(sj6)*(x173));
IkReal x180=((IkReal(1.00000000000000))*(npz)*(x172));
evalcond[0]=((IkReal(0.250000000390712))+(((npz)*(x173)))+(((IkReal(-1.00000000000000))*(cj6)*(x172)*(x176)))+(((npy)*(sj6)*(x172))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x180)))+(((IkReal(-2.41309350707268e-12))*(cj4)))+(x179)+(((IkReal(-1.00000000000000))*(x176)*(x178))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj4)*(x175)))+(((sj4)*(x179)))+(((IkReal(-1.00000000000000))*(x174)*(x178)))+(((IkReal(-1.00000000000000))*(sj6)*(x177)))+(((IkReal(-1.00000000000000))*(sj4)*(x180))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((cj4)*(x179)))+(((IkReal(-1.00000000000000))*(cj4)*(x180)))+(((IkReal(-1.00000000000000))*(x177)*(x178)))+(((sj4)*(x175)))+(((sj6)*(x174))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
sj4array[0]=((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))))):(IkReal)1.0e30))*(((((IkReal(1.00000000000000))*(npx)*(sj6)))+(((IkReal(1.00000000000000))*(cj6)*(npy))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign((((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6)))));
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x181=((IkReal(0.550000000000000))*(cj3));
IkReal x182=((cj6)*(npx));
IkReal x183=((npy)*(sj6));
IkReal x184=((IkReal(0.0450000000000000))*(sj3));
IkReal x185=((IkReal(0.0450000000000000))*(cj4));
IkReal x186=((IkReal(0.0450000000000000))*(x183));
IkReal x187=((npz)*(x185));
IkReal x188=((IkReal(0.550000000000000))*(cj4)*(sj3));
if( IKabs(((gconst0)*(((((IkReal(0.300000000000000))*(x183)))+(((IkReal(-1.00000000000000))*(x187)))+(((IkReal(-1.00000000000000))*(npz)*(x188)))+(((cj3)*(x187)))+(((IkReal(-0.300000000000000))*(x182)))+(((x183)*(x184)))+(((IkReal(-1.00000000000000))*(x182)*(x184)))+(((x181)*(x183)))+(((IkReal(-1.00000000000000))*(x181)*(x182))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((npz)*(x184)))+(((npz)*(x181)))+(((IkReal(0.300000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(cj3)*(x183)*(x185)))+(((cj3)*(x182)*(x185)))+(((x183)*(x185)))+(((x183)*(x188)))+(((IkReal(-1.00000000000000))*(x182)*(x185)))+(((IkReal(-1.00000000000000))*(x182)*(x188))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst0)*(((((IkReal(0.300000000000000))*(x183)))+(((IkReal(-1.00000000000000))*(x187)))+(((IkReal(-1.00000000000000))*(npz)*(x188)))+(((cj3)*(x187)))+(((IkReal(-0.300000000000000))*(x182)))+(((x183)*(x184)))+(((IkReal(-1.00000000000000))*(x182)*(x184)))+(((x181)*(x183)))+(((IkReal(-1.00000000000000))*(x181)*(x182)))))), ((gconst0)*(((((npz)*(x184)))+(((npz)*(x181)))+(((IkReal(0.300000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(cj3)*(x183)*(x185)))+(((cj3)*(x182)*(x185)))+(((x183)*(x185)))+(((x183)*(x188)))+(((IkReal(-1.00000000000000))*(x182)*(x185)))+(((IkReal(-1.00000000000000))*(x182)*(x188)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x189=IKsin(j5);
IkReal x190=IKcos(j5);
IkReal x191=((IkReal(0.0450000000000000))*(cj3));
IkReal x192=((IkReal(0.550000000000000))*(sj3));
IkReal x193=((IkReal(1.00000000000000))*(cj6));
IkReal x194=((npx)*(sj4));
IkReal x195=((cj4)*(npx));
IkReal x196=((npy)*(sj6)*(x190));
IkReal x197=((IkReal(1.00000000000000))*(npz)*(x189));
evalcond[0]=((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3)))+(((npy)*(sj6)*(x189)))+(((IkReal(-1.00000000000000))*(npx)*(x189)*(x193)))+(((npz)*(x190))));
evalcond[1]=((((cj4)*(x191)))+(((IkReal(-1.00000000000000))*(cj4)*(x192)))+(((IkReal(-1.00000000000000))*(x197)))+(x196)+(((IkReal(-0.0450000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(npx)*(x190)*(x193))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x190)*(x193)*(x194)))+(((IkReal(-1.00000000000000))*(cj4)*(npy)*(x193)))+(((IkReal(-1.00000000000000))*(sj6)*(x195)))+(((sj4)*(x196)))+(((IkReal(-1.00000000000000))*(sj4)*(x197))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((cj4)*(x196)))+(((IkReal(-1.00000000000000))*(x190)*(x193)*(x195)))+(((IkReal(-1.00000000000000))*(cj4)*(x197)))+(((IkReal(-1.00000000000000))*(x192)))+(x191)+(((sj6)*(x194)))+(((cj6)*(npy)*(sj4))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x198=((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))));
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(x198) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x199=((IkReal(1.00000000000000))*(IKatan2(npz, x198)));
if( ((((x198)*(x198))+((npz)*(npz)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x198)*(x198))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x198)*(x198))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x198)*(x198))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x198)*(x198))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x200=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x198)*(x198))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x198)*(x198))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3)))))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x199)))+(((IkReal(-1.00000000000000))*(x200))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x199)))+(((IkReal(1.00000000000000))*(x200))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x201=((IkReal(1.00000000000000))*(sj5));
IkReal x202=((IkReal(1.00000000000000))*(cj6));
IkReal x203=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x201)))+(((cj5)*(npz)))+(((sj5)*(x203))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x202)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
evalcond[4]=((((cj5)*(x203)))+(((IkReal(-1.00000000000000))*(npz)*(x201)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x202))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x204=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x204));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x204))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x205=IKsin(j4);
IkReal x206=IKcos(j4);
IkReal x207=((IkReal(1.00000000000000))*(x206));
IkReal x208=((IkReal(1.00000000000000))*(x205));
evalcond[0]=((((cj5)*(npy)*(sj6)*(x205)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x208)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x207)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x207)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x208))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x209=((IkReal(1.00000000000000))*(sj5));
IkReal x210=((IkReal(1.00000000000000))*(cj6));
IkReal x211=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj5)*(x211)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x209)))+(((cj5)*(npz))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x210)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
evalcond[4]=((((cj5)*(x211)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x210)))+(((IkReal(-1.00000000000000))*(npz)*(x209))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x212=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x212));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x212))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x213=IKsin(j4);
IkReal x214=IKcos(j4);
IkReal x215=((IkReal(1.00000000000000))*(x214));
IkReal x216=((IkReal(1.00000000000000))*(x213));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x215)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x215)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x216)))+(((cj5)*(npy)*(sj6)*(x213)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x216))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(0.250000000390712))+(((npy)*(sj5)*(sj6)))+(((cj5)*(npz)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(sj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x217=((IkReal(414405822679.079))*(cj5));
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)*(x217)))+(((IkReal(-414405822679.079))*(npz)*(sj5)))+(((npy)*(sj6)*(x217))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj6)*(npx)*(x217)))+(((IkReal(-414405822679.079))*(npz)*(sj5)))+(((npy)*(sj6)*(x217)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((((IkReal(-1.00000000000000))*(cj6)*(npx)*(x217)))+(((IkReal(-414405822679.079))*(npz)*(sj5)))+(((npy)*(sj6)*(x217)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x218=IKsin(j4);
IkReal x219=IKcos(j4);
IkReal x220=((IkReal(1.00000000000000))*(cj6));
IkReal x221=((cj5)*(sj6));
IkReal x222=((cj5)*(npx));
IkReal x223=((IkReal(1.00000000000000))*(npz)*(sj5));
IkReal x224=((npx)*(x218));
IkReal x225=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x226=((npy)*(x218));
IkReal x227=((npy)*(x219));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x218)))+(((IkReal(-1.00000000000000))*(x225)))+(((IkReal(-1.00000000000000))*(npy)*(x220))));
evalcond[1]=((((IkReal(-2.41309350707268e-12))*(x219)))+(((npy)*(x221)))+(((IkReal(-1.00000000000000))*(x220)*(x222)))+(((IkReal(-1.00000000000000))*(x223))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x218)*(x220)*(x222)))+(((IkReal(-1.00000000000000))*(x219)*(x225)))+(((IkReal(-1.00000000000000))*(x220)*(x227)))+(((IkReal(-1.00000000000000))*(x218)*(x223)))+(((x221)*(x226))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((sj6)*(x224)))+(((cj6)*(x226)))+(((IkReal(-1.00000000000000))*(x219)*(x220)*(x222)))+(((IkReal(-1.00000000000000))*(x219)*(x223)))+(((x221)*(x227))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst1)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x228=IKsin(j4);
IkReal x229=IKcos(j4);
IkReal x230=((IkReal(0.0450000000000000))*(cj3));
IkReal x231=((IkReal(1.00000000000000))*(cj6));
IkReal x232=((cj5)*(npx));
IkReal x233=((cj5)*(sj6));
IkReal x234=((IkReal(0.550000000000000))*(sj3));
IkReal x235=((npy)*(x228));
IkReal x236=((IkReal(1.00000000000000))*(npz)*(sj5));
IkReal x237=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x238=((npy)*(x229));
evalcond[0]=((((x228)*(x234)))+(((IkReal(-1.00000000000000))*(npy)*(x231)))+(((IkReal(0.0450000000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x237)))+(((IkReal(-1.00000000000000))*(x228)*(x230))));
evalcond[1]=((((x229)*(x230)))+(((IkReal(-1.00000000000000))*(x236)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((IkReal(-0.0450000000000000))*(x229)))+(((IkReal(-1.00000000000000))*(x229)*(x234)))+(((npy)*(x233))));
evalcond[2]=((((x233)*(x235)))+(((IkReal(-1.00000000000000))*(x231)*(x238)))+(((IkReal(-1.00000000000000))*(x229)*(x237)))+(((IkReal(-1.00000000000000))*(x228)*(x231)*(x232)))+(((IkReal(-1.00000000000000))*(x228)*(x236))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((cj6)*(x235)))+(((npx)*(sj6)*(x228)))+(((x233)*(x238)))+(((IkReal(-1.00000000000000))*(x234)))+(x230)+(((IkReal(-1.00000000000000))*(x229)*(x236)))+(((IkReal(-1.00000000000000))*(x229)*(x231)*(x232))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x239=((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))));
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(x239) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x240=((IkReal(1.00000000000000))*(IKatan2(npz, x239)));
if( ((((x239)*(x239))+((npz)*(npz)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x239)*(x239))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x239)*(x239))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x239)*(x239))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x239)*(x239))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x241=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x239)*(x239))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x239)*(x239))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.550000000000000))*(cj3)))))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x240)))+(((IkReal(-1.00000000000000))*(x241))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(1.00000000000000))*(x241)))+(((IkReal(-1.00000000000000))*(x240))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x242=((IkReal(1.00000000000000))*(sj5));
IkReal x243=((IkReal(1.00000000000000))*(cj6));
IkReal x244=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj5)*(x244)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x242)))+(((cj5)*(npz))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x243)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
evalcond[4]=((((IkReal(-1.00000000000000))*(npz)*(x242)))+(((cj5)*(x244)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x243))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x245=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x245));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x245))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x246=IKsin(j4);
IkReal x247=IKcos(j4);
IkReal x248=((IkReal(1.00000000000000))*(x247));
IkReal x249=((IkReal(1.00000000000000))*(x246));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)*(x249)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x248)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x249)))+(((cj5)*(npy)*(sj6)*(x246)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x248))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x250=((IkReal(1.00000000000000))*(sj5));
IkReal x251=((IkReal(1.00000000000000))*(cj6));
IkReal x252=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((cj5)*(npz)))+(((sj5)*(x252)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x250))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x251)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x251)))+(((cj5)*(x252)))+(((IkReal(-1.00000000000000))*(npz)*(x250))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x253=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x253));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x253))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x254=IKsin(j4);
IkReal x255=IKcos(j4);
IkReal x256=((IkReal(1.00000000000000))*(x255));
IkReal x257=((IkReal(1.00000000000000))*(x254));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)*(x257)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x257)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x256)))+(((cj5)*(npy)*(sj6)*(x254)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x256))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(0.250000000390712))+(((npy)*(sj5)*(sj6)))+(((cj5)*(npz)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(sj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x258=((IkReal(414405822679.079))*(cj5));
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-414405822679.079))*(npz)*(sj5)))+(((npy)*(sj6)*(x258)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x258))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((((IkReal(-414405822679.079))*(npz)*(sj5)))+(((npy)*(sj6)*(x258)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x258)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((((IkReal(-414405822679.079))*(npz)*(sj5)))+(((npy)*(sj6)*(x258)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x258)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x259=IKsin(j4);
IkReal x260=IKcos(j4);
IkReal x261=((IkReal(1.00000000000000))*(cj6));
IkReal x262=((cj5)*(sj6));
IkReal x263=((cj5)*(npx));
IkReal x264=((IkReal(1.00000000000000))*(npz)*(sj5));
IkReal x265=((npx)*(x259));
IkReal x266=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x267=((npy)*(x259));
IkReal x268=((npy)*(x260));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x261)))+(((IkReal(2.41309350707268e-12))*(x259)))+(((IkReal(-1.00000000000000))*(x266))));
evalcond[1]=((((IkReal(-2.41309350707268e-12))*(x260)))+(((IkReal(-1.00000000000000))*(x261)*(x263)))+(((npy)*(x262)))+(((IkReal(-1.00000000000000))*(x264))));
evalcond[2]=((((x262)*(x267)))+(((IkReal(-1.00000000000000))*(x259)*(x264)))+(((IkReal(-1.00000000000000))*(x261)*(x268)))+(((IkReal(-1.00000000000000))*(x259)*(x261)*(x263)))+(((IkReal(-1.00000000000000))*(x260)*(x266))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x262)*(x268)))+(((sj6)*(x265)))+(((cj6)*(x267)))+(((IkReal(-1.00000000000000))*(x260)*(x264)))+(((IkReal(-1.00000000000000))*(x260)*(x261)*(x263))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst1)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x269=IKsin(j4);
IkReal x270=IKcos(j4);
IkReal x271=((IkReal(0.0450000000000000))*(cj3));
IkReal x272=((IkReal(1.00000000000000))*(cj6));
IkReal x273=((cj5)*(npx));
IkReal x274=((cj5)*(sj6));
IkReal x275=((IkReal(0.550000000000000))*(sj3));
IkReal x276=((npy)*(x269));
IkReal x277=((IkReal(1.00000000000000))*(npz)*(sj5));
IkReal x278=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x279=((npy)*(x270));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x272)))+(((IkReal(-1.00000000000000))*(x269)*(x271)))+(((IkReal(-1.00000000000000))*(x278)))+(((IkReal(0.0450000000000000))*(x269)))+(((x269)*(x275))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x272)*(x273)))+(((IkReal(-0.0450000000000000))*(x270)))+(((x270)*(x271)))+(((npy)*(x274)))+(((IkReal(-1.00000000000000))*(x277)))+(((IkReal(-1.00000000000000))*(x270)*(x275))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x272)*(x279)))+(((x274)*(x276)))+(((IkReal(-1.00000000000000))*(x269)*(x277)))+(((IkReal(-1.00000000000000))*(x269)*(x272)*(x273)))+(((IkReal(-1.00000000000000))*(x270)*(x278))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x270)*(x272)*(x273)))+(((x274)*(x279)))+(x271)+(((IkReal(-1.00000000000000))*(x275)))+(((IkReal(-1.00000000000000))*(x270)*(x277)))+(((cj6)*(x276)))+(((npx)*(sj6)*(x269))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x86=((IkReal(1.00000000000000))*(cj3));
IkReal x87=((IkReal(1.00000000000000))*(sj6));
IkReal x88=((IkReal(1.00000000000000))*(sj3));
IkReal x89=((IkReal(1.00000000000000))*(cj5));
IkReal x90=((IkReal(-1.00000000000000))*(sj4));
IkReal x91=((IkReal(-1.00000000000000))*(cj5));
IkReal x92=((((cj6)*(r00)))+(((IkReal(-1.00000000000000))*(r01)*(x87))));
IkReal x93=((((r00)*(sj6)))+(((cj6)*(r01))));
IkReal x94=((((IkReal(-1.00000000000000))*(r11)*(x87)))+(((cj6)*(r10))));
IkReal x95=((((r10)*(sj6)))+(((cj6)*(r11))));
IkReal x96=((((cj6)*(r20)))+(((IkReal(-1.00000000000000))*(r21)*(x87))));
IkReal x97=((((cj6)*(r21)))+(((r20)*(sj6))));
IkReal x98=((sj5)*(x94));
IkReal x99=((sj5)*(x96));
IkReal x100=((sj4)*(x97));
IkReal x101=((((r02)*(sj5)))+(((cj5)*(x92))));
IkReal x102=((((IkReal(-1.00000000000000))*(r02)*(x89)))+(((sj5)*(x92))));
IkReal x103=((((r12)*(sj5)))+(((cj5)*(x94))));
IkReal x104=((((r22)*(sj5)))+(((cj5)*(x96))));
IkReal x105=((cj4)*(x103));
IkReal x106=((cj4)*(x104));
IkReal x107=((IkReal(1.00000000000000))*(x102));
IkReal x108=((((x90)*(x93)))+(((cj4)*(x101))));
new_r00=((((cj3)*(x108)))+(((IkReal(-1.00000000000000))*(x102)*(x88))));
new_r01=((((sj4)*(x101)))+(((cj4)*(x93))));
new_r02=((((IkReal(-1.00000000000000))*(x102)*(x86)))+(((IkReal(-1.00000000000000))*(x108)*(x88))));
new_r10=((((cj3)*(((((IkReal(-1.00000000000000))*(sj4)*(x95)))+(x105)))))+(((IkReal(-1.00000000000000))*(x88)*(((((r12)*(x91)))+(x98))))));
new_r11=((((sj4)*(x103)))+(((cj4)*(x95))));
new_r12=((((IkReal(-1.00000000000000))*(x88)*(((x105)+(((x90)*(x95)))))))+(((IkReal(-1.00000000000000))*(x86)*(((((IkReal(-1.00000000000000))*(r12)*(x89)))+(x98))))));
new_r20=((((cj3)*(((x106)+(((IkReal(-1.00000000000000))*(x100)))))))+(((IkReal(-1.00000000000000))*(x88)*(((x99)+(((r22)*(x91))))))));
new_r21=((((sj4)*(x104)))+(((cj4)*(x97))));
new_r22=((((IkReal(-1.00000000000000))*(x86)*(((((IkReal(-1.00000000000000))*(r22)*(x89)))+(x99)))))+(((IkReal(-1.00000000000000))*(x88)*(((x106)+(((x90)*(x97))))))));
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
cj1array[0]=new_r22;
if( cj1array[0] >= -1-IKFAST_SINCOS_THRESH && cj1array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j1valid[0] = j1valid[1] = true;
    j1array[0] = IKacos(cj1array[0]);
    sj1array[0] = IKsin(j1array[0]);
    cj1array[1] = cj1array[0];
    j1array[1] = -j1array[0];
    sj1array[1] = -sj1array[0];
}
else if( isnan(cj1array[0]) )
{
    // probably any value will work
    j1valid[0] = true;
    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
}
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x109=IKatan2(new_r02, new_r12);
j0array[0]=((IkReal(-1.00000000000000))*(x109));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x109))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j0))))+(((new_r12)*(IKcos(j0)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj0)))+(((cj0)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))))+IKsqr(((((new_r10)*(sj0)))+(((cj0)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))), ((((new_r10)*(sj0)))+(((cj0)*(new_r00)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x110=IKsin(j2);
IkReal x111=((IkReal(1.00000000000000))*(sj0));
IkReal x112=((IkReal(1.00000000000000))*(IKcos(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x111)))+(((IkReal(-1.00000000000000))*(x110)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x111)))+(((IkReal(-1.00000000000000))*(x112)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(x110)+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(((IkReal(-1.00000000000000))*(x112)))+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x113=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x113;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x113;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x114=IKatan2(new_r02, new_r12);
j0array[0]=((IkReal(-1.00000000000000))*(x114));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x114))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j0))))+(((new_r12)*(IKcos(j0)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((new_r11)*(sj0)))+(((cj0)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))), ((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x115=IKcos(j2);
IkReal x116=((IkReal(1.00000000000000))*(sj0));
IkReal x117=((IkReal(1.00000000000000))*(IKsin(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x116)))+(((IkReal(-1.00000000000000))*(x117)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x116)))+(((IkReal(-1.00000000000000))*(x115)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(((IkReal(-1.00000000000000))*(x117)))+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(x115)+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x118=((gconst5)*(sj1));
if( IKabs(((new_r12)*(x118))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x118))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r12)*(x118)), ((new_r02)*(x118)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[6];
IkReal x119=IKcos(j0);
IkReal x120=IKsin(j0);
IkReal x121=((IkReal(1.00000000000000))*(cj1));
IkReal x122=((IkReal(1.00000000000000))*(sj1));
IkReal x123=((new_r12)*(x120));
IkReal x124=((new_r02)*(x119));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(x120)))+(((new_r12)*(x119))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x122)))+(x124)+(x123));
evalcond[2]=((((cj1)*(x124)))+(((cj1)*(x123)))+(((IkReal(-1.00000000000000))*(new_r22)*(x122))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r10)*(x120)*(x122)))+(((IkReal(-1.00000000000000))*(new_r00)*(x119)*(x122)))+(((IkReal(-1.00000000000000))*(new_r20)*(x121))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r21)*(x121)))+(((IkReal(-1.00000000000000))*(new_r01)*(x119)*(x122)))+(((IkReal(-1.00000000000000))*(new_r11)*(x120)*(x122))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x121)))+(((IkReal(-1.00000000000000))*(x122)*(x123)))+(((IkReal(-1.00000000000000))*(x122)*(x124))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x125=((((IkReal(-1.00000000000000))*(new_r02)*(sj0)))+(((cj0)*(new_r12))));
IkReal x126=((((new_r12)*(sj0)))+(((cj0)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x125;
evalcond[5]=x125;
evalcond[6]=x126;
evalcond[7]=x126;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj0)))+(((cj0)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))))+IKsqr(((((new_r10)*(sj0)))+(((cj0)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))), ((((new_r10)*(sj0)))+(((cj0)*(new_r00)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x127=IKsin(j2);
IkReal x128=((IkReal(1.00000000000000))*(sj0));
IkReal x129=((IkReal(1.00000000000000))*(IKcos(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x128)))+(((IkReal(-1.00000000000000))*(x127)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x128)))+(((IkReal(-1.00000000000000))*(x129)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(x127)+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(((IkReal(-1.00000000000000))*(x129)))+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x130=((IkReal(1.00000000000000))+(new_r22));
IkReal x131=((new_r12)*(sj0));
IkReal x132=((cj0)*(new_r02));
IkReal x133=((((IkReal(-1.00000000000000))*(new_r02)*(sj0)))+(((cj0)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x130;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x133;
evalcond[5]=x133;
evalcond[6]=((x131)+(x132));
evalcond[7]=((((IkReal(-1.00000000000000))*(x131)))+(((IkReal(-1.00000000000000))*(x132))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x130;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((new_r11)*(sj0)))+(((cj0)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))), ((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x134=IKcos(j2);
IkReal x135=((IkReal(1.00000000000000))*(sj0));
IkReal x136=((IkReal(1.00000000000000))*(IKsin(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x135)))+(((IkReal(-1.00000000000000))*(x136)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x135)))+(((IkReal(-1.00000000000000))*(x134)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(((IkReal(-1.00000000000000))*(x136)))+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(x134)+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst7)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst7)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst7)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst7)*(new_r20)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[8];
IkReal x137=IKsin(j2);
IkReal x138=IKcos(j2);
IkReal x139=((IkReal(1.00000000000000))*(sj0));
IkReal x140=((IkReal(1.00000000000000))*(sj1));
IkReal x141=((cj0)*(new_r01));
IkReal x142=((cj0)*(new_r00));
IkReal x143=((new_r11)*(sj0));
IkReal x144=((new_r10)*(sj0));
IkReal x145=((IkReal(1.00000000000000))*(x138));
evalcond[0]=((new_r20)+(((sj1)*(x138))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x137)*(x140)))+(new_r21));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x139)))+(((IkReal(-1.00000000000000))*(x137)))+(((cj0)*(new_r10))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x145)))+(((IkReal(-1.00000000000000))*(new_r01)*(x139)))+(((cj0)*(new_r11))));
evalcond[4]=((x141)+(x143)+(((cj1)*(x137))));
evalcond[5]=((x142)+(x144)+(((IkReal(-1.00000000000000))*(cj1)*(x145))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r21)*(x140)))+(x137)+(((cj1)*(x143)))+(((cj1)*(x141))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x145)))+(((IkReal(-1.00000000000000))*(new_r20)*(x140)))+(((cj1)*(x142)))+(((cj1)*(x144))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst6)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst6)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst6)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst6)*(new_r20)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
evalcond[0]=((((sj1)*(IKcos(j2))))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj1)*(IKsin(j2))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst8;
gconst8=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x146=((gconst8)*(sj1));
if( IKabs(((new_r12)*(x146))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x146))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r12)*(x146)), ((new_r02)*(x146)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[12];
IkReal x147=IKcos(j0);
IkReal x148=IKsin(j0);
IkReal x149=((IkReal(1.00000000000000))*(cj2));
IkReal x150=((IkReal(1.00000000000000))*(sj1));
IkReal x151=((IkReal(1.00000000000000))*(cj1));
IkReal x152=((new_r10)*(x148));
IkReal x153=((new_r01)*(x147));
IkReal x154=((new_r00)*(x147));
IkReal x155=((IkReal(1.00000000000000))*(x148));
IkReal x156=((new_r11)*(x148));
IkReal x157=((new_r12)*(x148));
IkReal x158=((new_r02)*(x147));
evalcond[0]=((((new_r12)*(x147)))+(((IkReal(-1.00000000000000))*(new_r02)*(x155))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x150)))+(x157)+(x158));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj2)))+(((new_r10)*(x147)))+(((IkReal(-1.00000000000000))*(new_r00)*(x155))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x149)))+(((new_r11)*(x147)))+(((IkReal(-1.00000000000000))*(new_r01)*(x155))));
evalcond[4]=((((cj1)*(sj2)))+(x156)+(x153));
evalcond[5]=((x154)+(x152)+(((IkReal(-1.00000000000000))*(cj1)*(x149))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r22)*(x150)))+(((cj1)*(x157)))+(((cj1)*(x158))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x151)))+(((IkReal(-1.00000000000000))*(x150)*(x154)))+(((IkReal(-1.00000000000000))*(x150)*(x152))));
evalcond[8]=((((IkReal(-1.00000000000000))*(x150)*(x153)))+(((IkReal(-1.00000000000000))*(x150)*(x156)))+(((IkReal(-1.00000000000000))*(new_r21)*(x151))));
evalcond[9]=((sj2)+(((IkReal(-1.00000000000000))*(new_r21)*(x150)))+(((cj1)*(x156)))+(((cj1)*(x153))));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x150)*(x157)))+(((IkReal(-1.00000000000000))*(x150)*(x158)))+(((IkReal(-1.00000000000000))*(new_r22)*(x151))));
evalcond[11]=((((IkReal(-1.00000000000000))*(x149)))+(((IkReal(-1.00000000000000))*(new_r20)*(x150)))+(((cj1)*(x152)))+(((cj1)*(x154))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3238550ee83defc84c543b45c2db9325"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
