/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:45:59.074704
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.182500000000000))*(x5));
x15=((IkReal(0.182500000000000))*(x8));
x16=((IkReal(0.276500000000000))*(x6));
x17=((IkReal(0.182500000000000))*(x6));
x18=((IkReal(1.00000000000000))*(x5));
x19=((IkReal(1.00000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x4));
x21=((IkReal(0.276500000000000))*(x0));
x22=((IkReal(0.182500000000000))*(x9));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(1.00000000000000))*(x3));
x25=((IkReal(1.00000000000000))*(x8));
x26=((IkReal(0.276500000000000))*(x3));
x27=((IkReal(1.00000000000000))*(x11));
x28=((IkReal(1.00000000000000))*(x9));
x29=((x3)*(x7));
x30=((x2)*(x7));
x31=((x1)*(x2));
x32=((x0)*(x7));
x33=((x20)*(x3));
x34=((x18)*(x32));
x35=((x18)*(x29));
x36=((x20)*(x7)*(x9));
x37=((((IkReal(-1.00000000000000))*(x33)))+(((x0)*(x31))));
x38=((((x0)*(x4)))+(((x3)*(x31))));
x39=((((IkReal(-1.00000000000000))*(x1)*(x33)))+(((x0)*(x2))));
x40=((((IkReal(-1.00000000000000))*(x18)*(x30)))+(((x1)*(x6))));
x41=((((x1)*(x5)))+(((x30)*(x6))));
x42=((((IkReal(-1.00000000000000))*(x0)*(x31)))+(x33));
x43=((((IkReal(-1.00000000000000))*(x0)*(x1)*(x20)))+(((IkReal(-1.00000000000000))*(x2)*(x24))));
x44=((((IkReal(-1.00000000000000))*(x0)*(x20)))+(((IkReal(-1.00000000000000))*(x24)*(x31))));
x45=((x41)*(x8));
x46=((x37)*(x6));
x47=((x44)*(x5));
x48=((x43)*(x9));
x49=((((IkReal(-1.00000000000000))*(x34)))+(x46));
x50=((((IkReal(-1.00000000000000))*(x35)))+(((x38)*(x6))));
x51=((((IkReal(-1.00000000000000))*(x36)))+(x45));
x52=((((x42)*(x5)))+(((IkReal(-1.00000000000000))*(x19)*(x32))));
x53=((((IkReal(-1.00000000000000))*(x19)*(x29)))+(x47));
x54=((((x9)*(((((IkReal(-1.00000000000000))*(x1)*(x18)))+(((IkReal(-1.00000000000000))*(x19)*(x30)))))))+(((IkReal(-1.00000000000000))*(x20)*(x7)*(x8))));
x55=((x50)*(x8));
x56=((x11)*(x52));
x57=((x11)*(x53));
x58=((x55)+(((x39)*(x9))));
x59=((((x9)*(((((IkReal(-1.00000000000000))*(x19)*(x38)))+(x35)))))+(((x39)*(x8))));
x60=((((x49)*(x8)))+(x48));
x61=((x12)*(x58));
x62=((((x9)*(((((IkReal(-1.00000000000000))*(x46)))+(x34)))))+(((x43)*(x8))));
eerot[0]=((((x13)*(((((x12)*(x60)))+(x56)))))+(((x10)*(x62))));
eerot[1]=((((x13)*(x62)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x52)))+(((IkReal(-1.00000000000000))*(x23)*(x60))))))));
eerot[2]=((((x12)*(x52)))+(((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x49)))+(((IkReal(-1.00000000000000))*(x28)*(x43))))))));
IkReal x63=((IkReal(1.00000000000000))*(x32));
eetrans[0]=((((IkReal(-0.328000000000000))*(x32)))+(((IkReal(-1.00000000000000))*(x16)*(x63)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x49)))+(((IkReal(-1.00000000000000))*(x22)*(x43)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x63)))+(((x14)*(x42)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x31)))+(((x26)*(x4))))))));
eerot[3]=((((x13)*(((x57)+(x61)))))+(((x10)*(x59))));
eerot[4]=((((x13)*(x59)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x53)))+(((IkReal(-1.00000000000000))*(x23)*(x58))))))));
eerot[5]=((((x12)*(x53)))+(((x11)*(((((IkReal(-1.00000000000000))*(x28)*(x39)))+(((IkReal(-1.00000000000000))*(x25)*(x50))))))));
IkReal x64=((IkReal(1.00000000000000))*(x29));
eetrans[1]=((((IkReal(-1.00000000000000))*(x16)*(x64)))+(((x11)*(((((IkReal(-1.00000000000000))*(x22)*(x39)))+(((IkReal(-1.00000000000000))*(x15)*(x50)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x4)))+(((IkReal(-1.00000000000000))*(x26)*(x31)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x64)))+(((x14)*(x44)))))))+(((IkReal(-0.328000000000000))*(x29))));
eerot[6]=((((x13)*(((((x12)*(x51)))+(((x11)*(x40)))))))+(((x10)*(x54))));
eerot[7]=((((x13)*(x54)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x40)))+(((IkReal(-1.00000000000000))*(x23)*(x51))))))));
eerot[8]=((((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x41)))+(x36)))))+(((x12)*(x40))));
eetrans[2]=((IkReal(0.300000000000000))+(((x1)*(x16)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x41)))+(((x22)*(x4)*(x7)))))))+(((IkReal(-0.276500000000000))*(x30)*(x5)))+(((IkReal(0.328000000000000))*(x1)))+(((x12)*(((((x1)*(x17)))+(((IkReal(-1.00000000000000))*(x14)*(x30))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {6}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij6[0] = -1; _ij6[1] = -1; _nj6 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j6=pfree[0]; cj6=cos(pfree[0]); sj6=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((((IkReal(-0.182500000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((((IkReal(-0.182500000000000))*(r12)))+(py));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.300000000000000))+(pz)+(((IkReal(-0.182500000000000))*(r22))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
cj3array[0]=((IkReal(-1.01462229303577))+(((IkReal(5.51316543906850))*(pp))));
if( cj3array[0] >= -1-IKFAST_SINCOS_THRESH && cj3array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j3valid[0] = j3valid[1] = true;
    j3array[0] = IKacos(cj3array[0]);
    sj3array[0] = IKsin(j3array[0]);
    cj3array[1] = cj3array[0];
    j3array[1] = -j3array[0];
    sj3array[1] = -sj3array[0];
}
else if( isnan(cj3array[0]) )
{
    // probably any value will work
    j3valid[0] = true;
    cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
}
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((((cj6)*(cj6))*((npx)*(npx))))+((((npy)*(npy))*((sj6)*(sj6))))+((npz)*(npz))+(((IkReal(-2.00000000000000))*(cj6)*(npx)*(npy)*(sj6))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x65=((((IkReal(-0.553000000000000))*(npy)*(sj6)))+(((IkReal(0.553000000000000))*(cj6)*(npx))));
if( IKabs(((IkReal(0.553000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(x65) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x66=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.553000000000000))*(npz)), x65)));
if( (((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65)))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65))))))):(IkReal)1.0e30))*(((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(pp))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65)))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65))))))):(IkReal)1.0e30))*(((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(pp))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x67=IKasin(((((IKabs(IKabs(IKsqrt(((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65)))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.305809000000000))*((npz)*(npz))))+((x65)*(x65))))))):(IkReal)1.0e30))*(((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(pp)))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x66)))+(((IkReal(-1.00000000000000))*(x67))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(x67)+(((IkReal(-1.00000000000000))*(x66))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x68=IKsin(j5);
evalcond[0]=((IkReal(-0.276500000000000))+(((cj6)*(npx)*(x68)))+(((npz)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(npy)*(sj6)*(x68)))+(((IkReal(-0.328000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x69=((IkReal(1.00000000000000))*(pp));
IkReal x70=((IkReal(1.00000000000000))*(sj5));
IkReal x71=((npy)*(sj6));
IkReal x72=((cj5)*(npz));
IkReal x73=((IkReal(0.553000000000000))*(sj5));
IkReal x74=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x69))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(((IkReal(-1.00000000000000))*(x70)*(x71)))+(((sj5)*(x74)))+(x72));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(x72)))+(((IkReal(-1.00000000000000))*(x71)*(x73)))+(((x73)*(x74)))+(((IkReal(-1.00000000000000))*(x69))));
evalcond[5]=((((cj5)*(x74)))+(((IkReal(-1.00000000000000))*(npz)*(x70)))+(((IkReal(-1.00000000000000))*(cj5)*(x71))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x75=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x75)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x75)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x76=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x75)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x75)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x76));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x76))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x77=IKcos(j4);
IkReal x78=IKsin(j4);
IkReal x79=((IkReal(1.00000000000000))*(x77));
IkReal x80=((IkReal(1.00000000000000))*(x78));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x80)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x79)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x80)))+(((cj5)*(cj6)*(npx)*(x77)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x79))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x155=((IkReal(1.00000000000000))*(pp));
IkReal x156=((IkReal(1.00000000000000))*(sj5));
IkReal x157=((npy)*(sj6));
IkReal x158=((cj5)*(npz));
IkReal x159=((IkReal(0.553000000000000))*(sj5));
IkReal x160=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x155))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(x158)+(((IkReal(-1.00000000000000))*(x156)*(x157)))+(((sj5)*(x160))));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(x157)*(x159)))+(((IkReal(0.553000000000000))*(x158)))+(((x159)*(x160))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x157)))+(((cj5)*(x160)))+(((IkReal(-1.00000000000000))*(npz)*(x156))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x161=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x161))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x161)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x162=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x161)))), ((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x161)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x162));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x162))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x163=IKcos(j4);
IkReal x164=IKsin(j4);
IkReal x165=((IkReal(1.00000000000000))*(x163));
IkReal x166=((IkReal(1.00000000000000))*(x164));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)*(x165)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x166)))+(((cj5)*(cj6)*(npx)*(x163)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x165)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x166))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x167=((IkReal(125.000000000000))*(cj6));
IkReal x168=((IkReal(125.000000000000))*(sj6));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x167)))+(((IkReal(-1.00000000000000))*(npx)*(x168))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(npx)*(x167)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x168)))+(((IkReal(-125.000000000000))*(npz)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x167)))+(((IkReal(-1.00000000000000))*(npx)*(x168)))))), ((gconst0)*(((((cj5)*(npx)*(x167)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x168)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x169=IKsin(j4);
IkReal x170=IKcos(j4);
IkReal x171=((IkReal(0.328000000000000))*(sj3));
IkReal x172=((npz)*(sj5));
IkReal x173=((cj5)*(npx));
IkReal x174=((IkReal(1.00000000000000))*(npy));
IkReal x175=((npx)*(sj6));
IkReal x176=((cj5)*(sj6));
IkReal x177=((cj6)*(x170));
IkReal x178=((IkReal(1.00000000000000))*(x170));
IkReal x179=((cj6)*(x169));
evalcond[0]=((((cj6)*(npy)))+(((x169)*(x171)))+(x175));
evalcond[1]=((((IkReal(-1.00000000000000))*(x174)*(x176)))+(((cj6)*(x173)))+(((IkReal(-1.00000000000000))*(x172)))+(((IkReal(-1.00000000000000))*(x170)*(x171))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x175)*(x178)))+(((IkReal(-1.00000000000000))*(x174)*(x177)))+(((x169)*(x172)))+(((npy)*(x169)*(x176)))+(((IkReal(-1.00000000000000))*(x173)*(x179))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x174)*(x179)))+(((IkReal(-1.00000000000000))*(x171)))+(((IkReal(-1.00000000000000))*(x169)*(x175)))+(((x173)*(x177)))+(((IkReal(-1.00000000000000))*(x172)*(x178)))+(((IkReal(-1.00000000000000))*(x170)*(x174)*(x176))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x180=((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))));
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(x180) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x181=((IkReal(1.00000000000000))*(IKatan2(npz, x180)));
if( ((((npz)*(npz))+((x180)*(x180)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((npz)*(npz))+((x180)*(x180)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npz)*(npz))+((x180)*(x180))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((IkReal(-0.328000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((npz)*(npz))+((x180)*(x180)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npz)*(npz))+((x180)*(x180))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((IkReal(-0.328000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x182=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((npz)*(npz))+((x180)*(x180)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npz)*(npz))+((x180)*(x180))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((IkReal(-0.328000000000000))*(cj3)))))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x182)))+(((IkReal(-1.00000000000000))*(x181))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x181)))+(((IkReal(1.00000000000000))*(x182))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
evalcond[0]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(pp)))+(((((((IkReal(-0.553000000000000))*(npy)*(sj6)))+(((IkReal(0.553000000000000))*(cj6)*(npx)))))*(IKsin(j5))))+(((IkReal(0.553000000000000))*(npz)*(IKcos(j5)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x183=((IkReal(1.00000000000000))*(pp));
IkReal x184=((IkReal(1.00000000000000))*(sj5));
IkReal x185=((npy)*(sj6));
IkReal x186=((cj5)*(npz));
IkReal x187=((IkReal(0.553000000000000))*(sj5));
IkReal x188=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x183))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(x186)+(((IkReal(-1.00000000000000))*(x184)*(x185)))+(((sj5)*(x188))));
evalcond[4]=((IkReal(0.0311317500000000))+(((x187)*(x188)))+(((IkReal(-1.00000000000000))*(x183)))+(((IkReal(0.553000000000000))*(x186)))+(((IkReal(-1.00000000000000))*(x185)*(x187))));
evalcond[5]=((((cj5)*(x188)))+(((IkReal(-1.00000000000000))*(cj5)*(x185)))+(((IkReal(-1.00000000000000))*(npz)*(x184))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x189=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x189)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x189)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x190=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x189)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x189)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x190));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x190))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x191=IKcos(j4);
IkReal x192=IKsin(j4);
IkReal x193=((IkReal(1.00000000000000))*(x191));
IkReal x194=((IkReal(1.00000000000000))*(x192));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x194)))+(((cj5)*(cj6)*(npx)*(x191)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x193)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x193)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x194))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x195=((IkReal(1.00000000000000))*(pp));
IkReal x196=((IkReal(1.00000000000000))*(sj5));
IkReal x197=((npy)*(sj6));
IkReal x198=((cj5)*(npz));
IkReal x199=((IkReal(0.553000000000000))*(sj5));
IkReal x200=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x195))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(((IkReal(-1.00000000000000))*(x196)*(x197)))+(x198)+(((sj5)*(x200))));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x195)))+(((IkReal(-1.00000000000000))*(x197)*(x199)))+(((IkReal(0.553000000000000))*(x198)))+(((x199)*(x200))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x197)))+(((IkReal(-1.00000000000000))*(npz)*(x196)))+(((cj5)*(x200))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x201=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x201)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x201))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x202=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x201)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x201)))));
j4array[0]=((IkReal(-1.00000000000000))*(x202));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x202))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x203=IKcos(j4);
IkReal x204=IKsin(j4);
IkReal x205=((IkReal(1.00000000000000))*(x203));
IkReal x206=((IkReal(1.00000000000000))*(x204));
evalcond[0]=((((cj5)*(cj6)*(npx)*(x203)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x205)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x206)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x206)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x205))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x207=((IkReal(125.000000000000))*(cj6));
IkReal x208=((IkReal(125.000000000000))*(sj6));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x207)))+(((IkReal(-1.00000000000000))*(npx)*(x208))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(npx)*(x207)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x208)))+(((IkReal(-125.000000000000))*(npz)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x207)))+(((IkReal(-1.00000000000000))*(npx)*(x208)))))), ((gconst0)*(((((cj5)*(npx)*(x207)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x208)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x209=IKsin(j4);
IkReal x210=IKcos(j4);
IkReal x211=((IkReal(0.328000000000000))*(sj3));
IkReal x212=((npz)*(sj5));
IkReal x213=((cj5)*(npx));
IkReal x214=((IkReal(1.00000000000000))*(npy));
IkReal x215=((npx)*(sj6));
IkReal x216=((cj5)*(sj6));
IkReal x217=((cj6)*(x210));
IkReal x218=((IkReal(1.00000000000000))*(x210));
IkReal x219=((cj6)*(x209));
evalcond[0]=((((cj6)*(npy)))+(((x209)*(x211)))+(x215));
evalcond[1]=((((cj6)*(x213)))+(((IkReal(-1.00000000000000))*(x210)*(x211)))+(((IkReal(-1.00000000000000))*(x212)))+(((IkReal(-1.00000000000000))*(x214)*(x216))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x213)*(x219)))+(((x209)*(x212)))+(((npy)*(x209)*(x216)))+(((IkReal(-1.00000000000000))*(x214)*(x217)))+(((IkReal(-1.00000000000000))*(x215)*(x218))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x209)*(x215)))+(((IkReal(-1.00000000000000))*(x210)*(x214)*(x216)))+(((x213)*(x217)))+(((IkReal(-1.00000000000000))*(x214)*(x219)))+(((IkReal(-1.00000000000000))*(x212)*(x218)))+(((IkReal(-1.00000000000000))*(x211))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x220=((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))));
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(x220) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x221=((IkReal(1.00000000000000))*(IKatan2(npz, x220)));
if( ((((x220)*(x220))+((npz)*(npz)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x220)*(x220))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x220)*(x220))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((IkReal(-0.328000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x220)*(x220))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x220)*(x220))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((IkReal(-0.328000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x222=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x220)*(x220))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x220)*(x220))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((IkReal(-0.328000000000000))*(cj3)))))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x221)))+(((IkReal(-1.00000000000000))*(x222))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(1.00000000000000))*(x222)))+(((IkReal(-1.00000000000000))*(x221))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x223=IKsin(j5);
IkReal x224=((IkReal(0.553000000000000))*(x223));
evalcond[0]=((IkReal(0.0311317500000000))+(((cj6)*(npx)*(x224)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)*(x224)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.553000000000000))*(npz)*(IKcos(j5)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x225=((IkReal(1.00000000000000))*(pp));
IkReal x226=((IkReal(1.00000000000000))*(sj5));
IkReal x227=((npy)*(sj6));
IkReal x228=((cj5)*(npz));
IkReal x229=((IkReal(0.553000000000000))*(sj5));
IkReal x230=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x225))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(x228)+(((IkReal(-1.00000000000000))*(x226)*(x227)))+(((sj5)*(x230))));
evalcond[4]=((IkReal(0.0311317500000000))+(((x229)*(x230)))+(((IkReal(-1.00000000000000))*(x227)*(x229)))+(((IkReal(0.553000000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x225))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x227)))+(((cj5)*(x230)))+(((IkReal(-1.00000000000000))*(npz)*(x226))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x231=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x231)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x231)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x232=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x231)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x231)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x232));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x232))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x233=IKcos(j4);
IkReal x234=IKsin(j4);
IkReal x235=((IkReal(1.00000000000000))*(x233));
IkReal x236=((IkReal(1.00000000000000))*(x234));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x235)))+(((cj5)*(cj6)*(npx)*(x233)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x235)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x236)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x236))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x237=((IkReal(1.00000000000000))*(pp));
IkReal x238=((IkReal(1.00000000000000))*(sj5));
IkReal x239=((npy)*(sj6));
IkReal x240=((cj5)*(npz));
IkReal x241=((IkReal(0.553000000000000))*(sj5));
IkReal x242=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x237))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(((sj5)*(x242)))+(((IkReal(-1.00000000000000))*(x238)*(x239)))+(x240));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(x240)))+(((IkReal(-1.00000000000000))*(x237)))+(((x241)*(x242)))+(((IkReal(-1.00000000000000))*(x239)*(x241))));
evalcond[5]=((((cj5)*(x242)))+(((IkReal(-1.00000000000000))*(cj5)*(x239)))+(((IkReal(-1.00000000000000))*(npz)*(x238))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x243=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x243)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x243)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x244=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x243)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x243)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x244));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x244))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x245=IKcos(j4);
IkReal x246=IKsin(j4);
IkReal x247=((IkReal(1.00000000000000))*(x245));
IkReal x248=((IkReal(1.00000000000000))*(x246));
evalcond[0]=((((cj5)*(cj6)*(npx)*(x245)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x247)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x248)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x247)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x248))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x249=((IkReal(125.000000000000))*(cj6));
IkReal x250=((IkReal(125.000000000000))*(sj6));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x249)))+(((IkReal(-1.00000000000000))*(npx)*(x250))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(x250)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))+(((cj5)*(npx)*(x249))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x249)))+(((IkReal(-1.00000000000000))*(npx)*(x250)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(x250)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))+(((cj5)*(npx)*(x249)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x251=IKsin(j4);
IkReal x252=IKcos(j4);
IkReal x253=((IkReal(0.328000000000000))*(sj3));
IkReal x254=((npz)*(sj5));
IkReal x255=((cj5)*(npx));
IkReal x256=((IkReal(1.00000000000000))*(npy));
IkReal x257=((npx)*(sj6));
IkReal x258=((cj5)*(sj6));
IkReal x259=((cj6)*(x252));
IkReal x260=((IkReal(1.00000000000000))*(x252));
IkReal x261=((cj6)*(x251));
evalcond[0]=((((cj6)*(npy)))+(((x251)*(x253)))+(x257));
evalcond[1]=((((IkReal(-1.00000000000000))*(x252)*(x253)))+(((cj6)*(x255)))+(((IkReal(-1.00000000000000))*(x256)*(x258)))+(((IkReal(-1.00000000000000))*(x254))));
evalcond[2]=((((npy)*(x251)*(x258)))+(((x251)*(x254)))+(((IkReal(-1.00000000000000))*(x256)*(x259)))+(((IkReal(-1.00000000000000))*(x255)*(x261)))+(((IkReal(-1.00000000000000))*(x257)*(x260))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x251)*(x257)))+(((IkReal(-1.00000000000000))*(x252)*(x256)*(x258)))+(((x255)*(x259)))+(((IkReal(-1.00000000000000))*(x256)*(x261)))+(((IkReal(-1.00000000000000))*(x254)*(x260)))+(((IkReal(-1.00000000000000))*(x253))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x81=((cj6)*(sj4));
IkReal x82=((IkReal(1.00000000000000))*(sj5));
IkReal x83=((cj4)*(cj5));
IkReal x84=((IkReal(1.00000000000000))*(sj3));
IkReal x85=((IkReal(1.00000000000000))*(sj6));
IkReal x86=((IkReal(-1.00000000000000))*(cj3));
IkReal x87=((sj4)*(x85));
IkReal x88=((((IkReal(-1.00000000000000))*(sj3)*(x82)))+(((cj3)*(x83))));
IkReal x89=((((cj4)*(cj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x87))));
IkReal x90=((((cj4)*(sj6)))+(((cj5)*(x81))));
IkReal x91=((((cj3)*(sj5)))+(((sj3)*(x83))));
IkReal x92=((((cj3)*(cj5)))+(((IkReal(-1.00000000000000))*(cj4)*(sj3)*(x82))));
IkReal x93=((cj6)*(x88));
IkReal x94=((sj6)*(x88));
IkReal x95=((((IkReal(-1.00000000000000))*(cj5)*(x84)))+(((IkReal(-1.00000000000000))*(cj3)*(cj4)*(x82))));
IkReal x96=((x93)+(((IkReal(-1.00000000000000))*(cj3)*(x87))));
IkReal x97=((((cj6)*(x91)))+(((IkReal(-1.00000000000000))*(sj4)*(sj6)*(x84))));
IkReal x98=((((IkReal(-1.00000000000000))*(x81)*(x84)))+(((IkReal(-1.00000000000000))*(x85)*(x91))));
IkReal x99=((((x81)*(x86)))+(((IkReal(-1.00000000000000))*(x94))));
new_r00=((((r00)*(((x93)+(((sj4)*(sj6)*(x86)))))))+(((r01)*(((((IkReal(-1.00000000000000))*(cj3)*(x81)))+(((IkReal(-1.00000000000000))*(x85)*(x88)))))))+(((r02)*(x95))));
new_r01=((((r12)*(x95)))+(((r11)*(x99)))+(((r10)*(x96))));
new_r02=((((r21)*(x99)))+(((r20)*(x96)))+(((r22)*(x95))));
new_r10=((((r00)*(x90)))+(((r01)*(x89)))+(((IkReal(-1.00000000000000))*(r02)*(sj4)*(x82))));
new_r11=((((r11)*(x89)))+(((r10)*(x90)))+(((IkReal(-1.00000000000000))*(r12)*(sj4)*(x82))));
new_r12=((((r21)*(x89)))+(((IkReal(-1.00000000000000))*(r22)*(sj4)*(x82)))+(((r20)*(x90))));
new_r20=((((r01)*(x98)))+(((r00)*(x97)))+(((r02)*(x92))));
new_r21=((((r12)*(x92)))+(((r11)*(x98)))+(((r10)*(x97))));
new_r22=((((r21)*(x98)))+(((r20)*(x97)))+(((r22)*(x92))));
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
cj1array[0]=new_r22;
if( cj1array[0] >= -1-IKFAST_SINCOS_THRESH && cj1array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j1valid[0] = j1valid[1] = true;
    j1array[0] = IKacos(cj1array[0]);
    sj1array[0] = IKsin(j1array[0]);
    cj1array[1] = cj1array[0];
    j1array[1] = -j1array[0];
    sj1array[1] = -sj1array[0];
}
else if( isnan(cj1array[0]) )
{
    // probably any value will work
    j1valid[0] = true;
    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
}
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-1.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(-1.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(-1.00000000000000))*((new_r12)*(new_r12)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
IkReal x100=((IkReal(-1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x100;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x100;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
if( IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x101=IKatan2(new_r12, new_r02);
j2array[0]=((IkReal(-1.00000000000000))*(x101));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x101))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r12)*(IKsin(j2))))+(((new_r02)*(IKcos(j2)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r01)*(sj2)))+(((cj2)*(new_r11))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))))+IKsqr(((((new_r01)*(sj2)))+(((cj2)*(new_r11)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))), ((((new_r01)*(sj2)))+(((cj2)*(new_r11)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x102=IKsin(j0);
IkReal x103=((IkReal(1.00000000000000))*(sj2));
IkReal x104=((IkReal(1.00000000000000))*(IKcos(j0)));
evalcond[0]=((((new_r00)*(sj2)))+(x102)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((IkReal(-1.00000000000000))*(x104)))+(((cj2)*(new_r11))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x103)))+(((IkReal(-1.00000000000000))*(x104)))+(((cj2)*(new_r00))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x103)))+(((cj2)*(new_r01)))+(((IkReal(-1.00000000000000))*(x102))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
if( IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x105=IKatan2(new_r12, new_r02);
j2array[0]=((IkReal(-1.00000000000000))*(x105));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x105))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r12)*(IKsin(j2))))+(((new_r02)*(IKcos(j2)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x106=((IkReal(1.00000000000000))*(new_r00));
if( IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x106)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x106))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x106)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10)))))+IKsqr(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x106)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj2)*(x106)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10)))), ((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x106)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x107=IKcos(j0);
IkReal x108=IKsin(j0);
IkReal x109=((IkReal(1.00000000000000))*(sj2));
evalcond[0]=((((new_r00)*(sj2)))+(x108)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((cj2)*(new_r11)))+(((IkReal(-1.00000000000000))*(x107))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x109)))+(x107)+(((cj2)*(new_r00))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x109)))+(x108)+(((cj2)*(new_r01))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x110=((gconst2)*(sj1));
if( IKabs(((new_r12)*(x110))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r02)*(x110))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((new_r12)*(x110)), ((IkReal(-1.00000000000000))*(new_r02)*(x110)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[6];
IkReal x111=IKcos(j2);
IkReal x112=IKsin(j2);
IkReal x113=((IkReal(1.00000000000000))*(sj1));
IkReal x114=((sj1)*(x111));
IkReal x115=((IkReal(1.00000000000000))*(x112));
IkReal x116=((new_r02)*(x111));
evalcond[0]=((((new_r02)*(x112)))+(((new_r12)*(x111))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x113)))+(x116)+(((IkReal(-1.00000000000000))*(new_r12)*(x115))));
evalcond[2]=((((cj1)*(new_r20)))+(((new_r00)*(x114)))+(((IkReal(-1.00000000000000))*(new_r10)*(x112)*(x113))));
evalcond[3]=((((cj1)*(new_r21)))+(((IkReal(-1.00000000000000))*(new_r11)*(x112)*(x113)))+(((new_r01)*(x114))));
evalcond[4]=((((cj1)*(x116)))+(((IkReal(-1.00000000000000))*(new_r22)*(x113)))+(((IkReal(-1.00000000000000))*(cj1)*(new_r12)*(x115))));
evalcond[5]=((IkReal(-1.00000000000000))+(((cj1)*(new_r22)))+(((new_r02)*(x114)))+(((IkReal(-1.00000000000000))*(new_r12)*(x112)*(x113))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x117=((IkReal(-1.00000000000000))+(new_r22));
IkReal x118=((IkReal(1.00000000000000))*(new_r12));
IkReal x119=((new_r02)*(sj2));
IkReal x120=((((cj2)*(new_r02)))+(((IkReal(-1.00000000000000))*(sj2)*(x118))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x117;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x119)+(((cj2)*(new_r12))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj2)*(x118)))+(((IkReal(-1.00000000000000))*(x119))));
evalcond[6]=x120;
evalcond[7]=new_r20;
evalcond[8]=new_r21;
evalcond[9]=x120;
evalcond[10]=x117;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r01)*(sj2)))+(((cj2)*(new_r11))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))))+IKsqr(((((new_r01)*(sj2)))+(((cj2)*(new_r11)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))), ((((new_r01)*(sj2)))+(((cj2)*(new_r11)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x121=IKsin(j0);
IkReal x122=((IkReal(1.00000000000000))*(sj2));
IkReal x123=((IkReal(1.00000000000000))*(IKcos(j0)));
evalcond[0]=((((new_r00)*(sj2)))+(x121)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((IkReal(-1.00000000000000))*(x123)))+(((cj2)*(new_r11))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x122)))+(((IkReal(-1.00000000000000))*(x123)))+(((cj2)*(new_r00))));
evalcond[3]=((((cj2)*(new_r01)))+(((IkReal(-1.00000000000000))*(x121)))+(((IkReal(-1.00000000000000))*(new_r11)*(x122))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x124=((new_r12)*(sj2));
IkReal x125=((new_r02)*(sj2));
IkReal x126=((cj2)*(new_r12));
IkReal x127=((cj2)*(new_r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x126)+(x125));
evalcond[5]=((((IkReal(-1.00000000000000))*(x125)))+(((IkReal(-1.00000000000000))*(x126))));
evalcond[6]=((x127)+(((IkReal(-1.00000000000000))*(x124))));
evalcond[7]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[9]=((x124)+(((IkReal(-1.00000000000000))*(x127))));
evalcond[10]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x128=((IkReal(1.00000000000000))*(new_r00));
if( IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x128)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x128))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x128)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10)))))+IKsqr(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x128)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj2)*(x128)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10)))), ((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x128)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x129=IKcos(j0);
IkReal x130=IKsin(j0);
IkReal x131=((IkReal(1.00000000000000))*(sj2));
evalcond[0]=((((new_r00)*(sj2)))+(x130)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((cj2)*(new_r11)))+(((IkReal(-1.00000000000000))*(x129))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x131)))+(x129)+(((cj2)*(new_r00))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x131)))+(x130)+(((cj2)*(new_r01))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x132=((IkReal(-1.00000000000000))*(gconst3));
if( IKabs(((new_r21)*(x132))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r20)*(x132))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r21)*(x132)), ((new_r20)*(x132)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[8];
IkReal x133=IKsin(j0);
IkReal x134=IKcos(j0);
IkReal x135=((IkReal(1.00000000000000))*(cj1));
IkReal x136=((new_r10)*(sj2));
IkReal x137=((cj2)*(new_r01));
IkReal x138=((IkReal(1.00000000000000))*(sj1));
IkReal x139=((new_r11)*(sj2));
IkReal x140=((cj2)*(new_r00));
IkReal x141=((IkReal(1.00000000000000))*(x134));
evalcond[0]=((new_r20)+(((sj1)*(x134))));
evalcond[1]=((new_r21)+(((sj1)*(x133))));
evalcond[2]=((((new_r00)*(sj2)))+(x133)+(((cj2)*(new_r10))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x141)))+(((new_r01)*(sj2)))+(((cj2)*(new_r11))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x134)*(x135)))+(x140)+(((IkReal(-1.00000000000000))*(x136))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x133)*(x135)))+(x137)+(((IkReal(-1.00000000000000))*(x139))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x141)))+(((IkReal(-1.00000000000000))*(x135)*(x136)))+(((IkReal(-1.00000000000000))*(new_r20)*(x138)))+(((cj1)*(x140))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x135)*(x139)))+(((IkReal(-1.00000000000000))*(new_r21)*(x138)))+(((IkReal(-1.00000000000000))*(x133)))+(((cj1)*(x137))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x142=((IkReal(-1.00000000000000))*(gconst1));
if( IKabs(((new_r21)*(x142))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r20)*(x142))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r21)*(x142)), ((new_r20)*(x142)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
evalcond[0]=((((sj1)*(IKcos(j0))))+(new_r20));
evalcond[1]=((((sj1)*(IKsin(j0))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(-1.00000000000000))*(new_r00)*(new_r12)))+(((new_r02)*(new_r10)))));
dummyeval[0]=((((IkReal(-1.00000000000000))*(new_r00)*(new_r12)))+(((new_r02)*(new_r10))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x143=((gconst4)*(sj0));
if( IKabs(((new_r12)*(x143))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r02)*(x143))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((new_r12)*(x143)), ((IkReal(-1.00000000000000))*(new_r02)*(x143)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[12];
IkReal x144=IKsin(j2);
IkReal x145=IKcos(j2);
IkReal x146=((IkReal(1.00000000000000))*(cj0));
IkReal x147=((IkReal(1.00000000000000))*(sj1));
IkReal x148=((IkReal(1.00000000000000))*(cj1));
IkReal x149=((new_r10)*(x144));
IkReal x150=((new_r12)*(x144));
IkReal x151=((new_r01)*(x145));
IkReal x152=((new_r00)*(x145));
IkReal x153=((new_r02)*(x145));
IkReal x154=((IkReal(1.00000000000000))*(new_r11)*(x144));
evalcond[0]=((((new_r12)*(x145)))+(((new_r02)*(x144))));
evalcond[1]=((sj0)+(((new_r00)*(x144)))+(((new_r10)*(x145))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x146)))+(((new_r01)*(x144)))+(((new_r11)*(x145))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x147)))+(x153)+(((IkReal(-1.00000000000000))*(x150))));
evalcond[4]=((x152)+(((IkReal(-1.00000000000000))*(x149)))+(((IkReal(-1.00000000000000))*(cj1)*(x146))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x154)))+(((IkReal(-1.00000000000000))*(sj0)*(x148)))+(x151));
evalcond[6]=((((cj1)*(new_r20)))+(((sj1)*(x152)))+(((IkReal(-1.00000000000000))*(x147)*(x149))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r11)*(x144)*(x147)))+(((cj1)*(new_r21)))+(((sj1)*(x151))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r22)*(x147)))+(((IkReal(-1.00000000000000))*(x148)*(x150)))+(((cj1)*(x153))));
evalcond[9]=((IkReal(-1.00000000000000))+(((cj1)*(new_r22)))+(((sj1)*(x153)))+(((IkReal(-1.00000000000000))*(x147)*(x150))));
evalcond[10]=((((IkReal(-1.00000000000000))*(x146)))+(((IkReal(-1.00000000000000))*(new_r20)*(x147)))+(((IkReal(-1.00000000000000))*(x148)*(x149)))+(((cj1)*(x152))));
evalcond[11]=((((IkReal(-1.00000000000000))*(sj0)))+(((IkReal(-1.00000000000000))*(new_r11)*(x144)*(x148)))+(((IkReal(-1.00000000000000))*(new_r21)*(x147)))+(((cj1)*(x151))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "bcb25302dbec7575df10994dd89abd19"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
