/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:46:15.519503
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.182500000000000))*(x5));
x15=((IkReal(0.182500000000000))*(x8));
x16=((IkReal(0.276500000000000))*(x6));
x17=((IkReal(0.182500000000000))*(x6));
x18=((IkReal(1.00000000000000))*(x5));
x19=((IkReal(1.00000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x4));
x21=((IkReal(0.276500000000000))*(x0));
x22=((IkReal(0.182500000000000))*(x9));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(1.00000000000000))*(x3));
x25=((IkReal(1.00000000000000))*(x8));
x26=((IkReal(0.276500000000000))*(x3));
x27=((IkReal(1.00000000000000))*(x11));
x28=((IkReal(1.00000000000000))*(x9));
x29=((x3)*(x7));
x30=((x2)*(x7));
x31=((x1)*(x2));
x32=((x0)*(x7));
x33=((x20)*(x3));
x34=((x18)*(x32));
x35=((x18)*(x29));
x36=((x20)*(x7)*(x9));
x37=((((IkReal(-1.00000000000000))*(x33)))+(((x0)*(x31))));
x38=((((x0)*(x4)))+(((x3)*(x31))));
x39=((((IkReal(-1.00000000000000))*(x1)*(x33)))+(((x0)*(x2))));
x40=((((IkReal(-1.00000000000000))*(x18)*(x30)))+(((x1)*(x6))));
x41=((((x1)*(x5)))+(((x30)*(x6))));
x42=((((IkReal(-1.00000000000000))*(x0)*(x31)))+(x33));
x43=((((IkReal(-1.00000000000000))*(x0)*(x1)*(x20)))+(((IkReal(-1.00000000000000))*(x2)*(x24))));
x44=((((IkReal(-1.00000000000000))*(x0)*(x20)))+(((IkReal(-1.00000000000000))*(x24)*(x31))));
x45=((x41)*(x8));
x46=((x37)*(x6));
x47=((x44)*(x5));
x48=((x43)*(x9));
x49=((((IkReal(-1.00000000000000))*(x34)))+(x46));
x50=((((IkReal(-1.00000000000000))*(x35)))+(((x38)*(x6))));
x51=((((IkReal(-1.00000000000000))*(x36)))+(x45));
x52=((((x42)*(x5)))+(((IkReal(-1.00000000000000))*(x19)*(x32))));
x53=((((IkReal(-1.00000000000000))*(x19)*(x29)))+(x47));
x54=((((x9)*(((((IkReal(-1.00000000000000))*(x1)*(x18)))+(((IkReal(-1.00000000000000))*(x19)*(x30)))))))+(((IkReal(-1.00000000000000))*(x20)*(x7)*(x8))));
x55=((x50)*(x8));
x56=((x11)*(x52));
x57=((x11)*(x53));
x58=((x55)+(((x39)*(x9))));
x59=((((x9)*(((((IkReal(-1.00000000000000))*(x19)*(x38)))+(x35)))))+(((x39)*(x8))));
x60=((((x49)*(x8)))+(x48));
x61=((x12)*(x58));
x62=((((x9)*(((((IkReal(-1.00000000000000))*(x46)))+(x34)))))+(((x43)*(x8))));
eerot[0]=((((x13)*(((((x12)*(x60)))+(x56)))))+(((x10)*(x62))));
eerot[1]=((((x13)*(x62)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x52)))+(((IkReal(-1.00000000000000))*(x23)*(x60))))))));
eerot[2]=((((x12)*(x52)))+(((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x49)))+(((IkReal(-1.00000000000000))*(x28)*(x43))))))));
IkReal x63=((IkReal(1.00000000000000))*(x32));
eetrans[0]=((((IkReal(-0.328000000000000))*(x32)))+(((IkReal(-1.00000000000000))*(x16)*(x63)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x49)))+(((IkReal(-1.00000000000000))*(x22)*(x43)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x63)))+(((x14)*(x42)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x31)))+(((x26)*(x4))))))));
eerot[3]=((((x13)*(((x57)+(x61)))))+(((x10)*(x59))));
eerot[4]=((((x13)*(x59)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x53)))+(((IkReal(-1.00000000000000))*(x23)*(x58))))))));
eerot[5]=((((x12)*(x53)))+(((x11)*(((((IkReal(-1.00000000000000))*(x28)*(x39)))+(((IkReal(-1.00000000000000))*(x25)*(x50))))))));
IkReal x64=((IkReal(1.00000000000000))*(x29));
eetrans[1]=((((IkReal(-1.00000000000000))*(x16)*(x64)))+(((x11)*(((((IkReal(-1.00000000000000))*(x22)*(x39)))+(((IkReal(-1.00000000000000))*(x15)*(x50)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x4)))+(((IkReal(-1.00000000000000))*(x26)*(x31)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x64)))+(((x14)*(x44)))))))+(((IkReal(-0.328000000000000))*(x29))));
eerot[6]=((((x13)*(((((x12)*(x51)))+(((x11)*(x40)))))))+(((x10)*(x54))));
eerot[7]=((((x13)*(x54)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x40)))+(((IkReal(-1.00000000000000))*(x23)*(x51))))))));
eerot[8]=((((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x41)))+(x36)))))+(((x12)*(x40))));
eetrans[2]=((IkReal(0.300000000000000))+(((x1)*(x16)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x41)))+(((x22)*(x4)*(x7)))))))+(((IkReal(-0.276500000000000))*(x30)*(x5)))+(((IkReal(0.328000000000000))*(x1)))+(((x12)*(((((x1)*(x17)))+(((IkReal(-1.00000000000000))*(x14)*(x30))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {5}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j6,cj6,sj6,htj6,j5,cj5,sj5,htj5,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij6[2], _nj6,_ij5[2], _nj5;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1;  _ij5[0] = -1; _ij5[1] = -1; _nj5 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j5=pfree[0]; cj5=cos(pfree[0]); sj5=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((((IkReal(-0.182500000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((((IkReal(-0.182500000000000))*(r12)))+(py));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.300000000000000))+(pz)+(((IkReal(-0.182500000000000))*(r22))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
cj3array[0]=((IkReal(-1.01462229303577))+(((IkReal(5.51316543906850))*(pp))));
if( cj3array[0] >= -1-IKFAST_SINCOS_THRESH && cj3array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j3valid[0] = j3valid[1] = true;
    j3array[0] = IKacos(cj3array[0]);
    sj3array[0] = IKsin(j3array[0]);
    cj3array[1] = cj3array[0];
    j3array[1] = -j3array[0];
    sj3array[1] = -sj3array[0];
}
else if( isnan(cj3array[0]) )
{
    // probably any value will work
    j3valid[0] = true;
    cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
}
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
IkReal x65=(sj5)*(sj5);
dummyeval[0]=((((x65)*((npy)*(npy))))+(((x65)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x66=(sj5)*(sj5);
dummyeval[0]=((((x66)*((npx)*(npx))))+(((x66)*((npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x67=(sj5)*(sj5);
dummyeval[0]=((((x67)*((npx)*(npx))))+(((x67)*((npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x68=((IkReal(1.00000000000000))*(pp));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.184036250000000))+(((IkReal(0.181384000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x68))));
evalcond[2]=((IkReal(-0.276500000000000))+(npz)+(((IkReal(-0.328000000000000))*(cj3))));
evalcond[3]=((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x68))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-9.29506246281975))*(npy)*((npx)*(npx))))+(((npy)*((sj3)*(sj3))))+(((IkReal(-9.29506246281975))*((npy)*(npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x69=(sj3)*(sj3);
IkReal x70=(npy)*(npy);
IkReal x71=(npx)*(npx)*(npx);
IkReal x72=((npx)*(x70));
IkReal x73=((npx)*(x69));
IkReal x74=((IkReal(0.430336000000000))*(npy)*(x69));
IkReal x75=((((IkReal(4.00000000000000))*(npy)*((npx)*(npx))))+(((IkReal(4.00000000000000))*((npy)*(npy)*(npy)))));
IkReal x76=((((IkReal(-1.00000000000000))*(x75)))+(x74));
IkReal x77=((((IkReal(-0.860672000000000))*(x73)))+(((IkReal(8.00000000000000))*(x71)))+(((IkReal(8.00000000000000))*(x72))));
IkReal x78=((((IkReal(-1.00000000000000))*(x74)))+(x75));
op[0]=x76;
op[1]=x77;
op[2]=x76;
op[3]=((((IkReal(-1.72134400000000))*(x73)))+(((IkReal(16.0000000000000))*(x72)))+(((IkReal(16.0000000000000))*(x71))));
op[4]=x78;
op[5]=x77;
op[6]=x78;
polyroots6(op,zeror,numroots);
IkReal j6array[6], cj6array[6], sj6array[6], tempj6array[1];
int numsolutions = 0;
for(int ij6 = 0; ij6 < numroots; ++ij6)
{
IkReal htj6 = zeror[ij6];
tempj6array[0]=((IkReal(2.00000000000000))*(atan(htj6)));
for(int kj6 = 0; kj6 < 1; ++kj6)
{
j6array[numsolutions] = tempj6array[kj6];
if( j6array[numsolutions] > IKPI )
{
    j6array[numsolutions]-=IK2PI;
}
else if( j6array[numsolutions] < -IKPI )
{
    j6array[numsolutions]+=IK2PI;
}
sj6array[numsolutions] = IKsin(j6array[numsolutions]);
cj6array[numsolutions] = IKcos(j6array[numsolutions]);
numsolutions++;
}
}
bool j6valid[6]={true,true,true,true,true,true};
_nj6 = 6;
for(int ij6 = 0; ij6 < numsolutions; ++ij6)
    {
if( !j6valid[ij6] )
{
    continue;
}
    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
htj6 = IKtan(j6/2);

_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < numsolutions; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x79=((IkReal(1.00000000000000))*(pp));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x79))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(npz));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x79)))+(((IkReal(0.553000000000000))*(npz))));
evalcond[5]=((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x80=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x80))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(npx)*(x80))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x81=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x80)))), ((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(npx)*(x80)))));
j4array[0]=((IkReal(-1.00000000000000))*(x81));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x81))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x82=IKcos(j4);
IkReal x83=IKsin(j4);
IkReal x84=((IkReal(1.00000000000000))*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x82)*(x84)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x83)))+(((cj6)*(npx)*(x82)))+(((IkReal(-1.00000000000000))*(npx)*(x83)*(x84))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x159=((IkReal(1.00000000000000))*(pp));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x159))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(npz));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x159)))+(((IkReal(0.553000000000000))*(npz))));
evalcond[5]=((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x160=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x160))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(npx)*(x160))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x161=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x160)))), ((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(npx)*(x160)))));
j4array[0]=((IkReal(-1.00000000000000))*(x161));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x161))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x162=IKcos(j4);
IkReal x163=IKsin(j4);
IkReal x164=((IkReal(1.00000000000000))*(sj6));
evalcond[0]=((((cj6)*(npx)*(x162)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x163)))+(((IkReal(-1.00000000000000))*(npx)*(x163)*(x164)))+(((IkReal(-1.00000000000000))*(npy)*(x162)*(x164))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x165=((IkReal(125.000000000000))*(npy));
IkReal x166=((IkReal(125.000000000000))*(npx));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(cj6)*(x165)))+(((IkReal(-1.00000000000000))*(sj6)*(x166))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((cj6)*(x166)))+(((IkReal(-1.00000000000000))*(sj6)*(x165))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(cj6)*(x165)))+(((IkReal(-1.00000000000000))*(sj6)*(x166)))))), ((gconst1)*(((((cj6)*(x166)))+(((IkReal(-1.00000000000000))*(sj6)*(x165)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x167=IKsin(j4);
IkReal x168=IKcos(j4);
IkReal x169=((IkReal(0.328000000000000))*(sj3));
IkReal x170=((IkReal(1.00000000000000))*(npx));
IkReal x171=((cj6)*(npx));
IkReal x172=((IkReal(1.00000000000000))*(npy));
IkReal x173=((sj6)*(x172));
IkReal x174=((cj6)*(x167));
IkReal x175=((sj6)*(x167));
evalcond[0]=((((cj6)*(npy)))+(((npx)*(sj6)))+(((x167)*(x169))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x173)))+(x171)+(((IkReal(-1.00000000000000))*(x168)*(x169))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)*(x168)*(x170)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((IkReal(-1.00000000000000))*(cj6)*(x168)*(x172)))+(((npy)*(x175))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x169)))+(((x168)*(x171)))+(((IkReal(-1.00000000000000))*(x170)*(x175)))+(((IkReal(-1.00000000000000))*(x168)*(x173)))+(((IkReal(-1.00000000000000))*(x172)*(x174))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
IkReal x176=((IkReal(1.00000000000000))*(pp));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.184036250000000))+(((IkReal(-1.00000000000000))*(x176)))+(((IkReal(0.181384000000000))*(cj3))));
evalcond[2]=((IkReal(-0.276500000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-0.328000000000000))*(cj3))));
evalcond[3]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x176)))+(((IkReal(-0.553000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-9.29506246281975))*(npy)*((npx)*(npx))))+(((npy)*((sj3)*(sj3))))+(((IkReal(-9.29506246281975))*((npy)*(npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x177=(sj3)*(sj3);
IkReal x178=(npy)*(npy);
IkReal x179=(npx)*(npx)*(npx);
IkReal x180=((npx)*(x178));
IkReal x181=((npx)*(x177));
IkReal x182=((IkReal(0.430336000000000))*(npy)*(x177));
IkReal x183=((((IkReal(4.00000000000000))*(npy)*((npx)*(npx))))+(((IkReal(4.00000000000000))*((npy)*(npy)*(npy)))));
IkReal x184=((((IkReal(-1.00000000000000))*(x183)))+(x182));
IkReal x185=((((IkReal(-0.860672000000000))*(x181)))+(((IkReal(8.00000000000000))*(x179)))+(((IkReal(8.00000000000000))*(x180))));
IkReal x186=((((IkReal(-1.00000000000000))*(x182)))+(x183));
op[0]=x184;
op[1]=x185;
op[2]=x184;
op[3]=((((IkReal(-1.72134400000000))*(x181)))+(((IkReal(16.0000000000000))*(x180)))+(((IkReal(16.0000000000000))*(x179))));
op[4]=x186;
op[5]=x185;
op[6]=x186;
polyroots6(op,zeror,numroots);
IkReal j6array[6], cj6array[6], sj6array[6], tempj6array[1];
int numsolutions = 0;
for(int ij6 = 0; ij6 < numroots; ++ij6)
{
IkReal htj6 = zeror[ij6];
tempj6array[0]=((IkReal(2.00000000000000))*(atan(htj6)));
for(int kj6 = 0; kj6 < 1; ++kj6)
{
j6array[numsolutions] = tempj6array[kj6];
if( j6array[numsolutions] > IKPI )
{
    j6array[numsolutions]-=IK2PI;
}
else if( j6array[numsolutions] < -IKPI )
{
    j6array[numsolutions]+=IK2PI;
}
sj6array[numsolutions] = IKsin(j6array[numsolutions]);
cj6array[numsolutions] = IKcos(j6array[numsolutions]);
numsolutions++;
}
}
bool j6valid[6]={true,true,true,true,true,true};
_nj6 = 6;
for(int ij6 = 0; ij6 < numsolutions; ++ij6)
    {
if( !j6valid[ij6] )
{
    continue;
}
    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
htj6 = IKtan(j6/2);

_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < numsolutions; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x187=((IkReal(1.00000000000000))*(pp));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x187))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x187)))+(((IkReal(-0.553000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x188=((IkReal(1.00000000000000))*(npy));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x188))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(sj6)*(x188))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x189=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x188)))), ((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(sj6)*(x188)))));
j4array[0]=((IkReal(-1.00000000000000))*(x189));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x189))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x190=IKcos(j4);
IkReal x191=IKsin(j4);
IkReal x192=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj6)*(x191)*(x192)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x191)))+(((IkReal(-1.00000000000000))*(cj6)*(x190)*(x192)))+(((npy)*(sj6)*(x190))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x193=((IkReal(1.00000000000000))*(pp));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x193))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x193)))+(((IkReal(-0.553000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x194=((IkReal(1.00000000000000))*(npy));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x194))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(sj6)*(x194))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x195=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x194)))), ((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(sj6)*(x194)))));
j4array[0]=((IkReal(-1.00000000000000))*(x195));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x195))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x196=IKcos(j4);
IkReal x197=IKsin(j4);
IkReal x198=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(x196)*(x198)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x197)))+(((IkReal(-1.00000000000000))*(sj6)*(x197)*(x198)))+(((npy)*(sj6)*(x196))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x199=((IkReal(125.000000000000))*(npy));
IkReal x200=((IkReal(125.000000000000))*(npx));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(sj6)*(x200)))+(((IkReal(-1.00000000000000))*(cj6)*(x199))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((sj6)*(x199)))+(((IkReal(-1.00000000000000))*(cj6)*(x200))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(sj6)*(x200)))+(((IkReal(-1.00000000000000))*(cj6)*(x199)))))), ((gconst2)*(((((sj6)*(x199)))+(((IkReal(-1.00000000000000))*(cj6)*(x200)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x201=IKsin(j4);
IkReal x202=IKcos(j4);
IkReal x203=((IkReal(0.328000000000000))*(sj3));
IkReal x204=((cj6)*(npx));
IkReal x205=((npy)*(sj6));
IkReal x206=((npx)*(sj6));
IkReal x207=((cj6)*(npy));
IkReal x208=((IkReal(1.00000000000000))*(x201));
IkReal x209=((IkReal(1.00000000000000))*(x202));
evalcond[0]=((((x201)*(x203)))+(x207)+(x206));
evalcond[1]=((((IkReal(-1.00000000000000))*(x202)*(x203)))+(x205)+(((IkReal(-1.00000000000000))*(x204))));
evalcond[2]=((((x201)*(x204)))+(((IkReal(-1.00000000000000))*(x206)*(x209)))+(((IkReal(-1.00000000000000))*(x207)*(x209)))+(((IkReal(-1.00000000000000))*(x205)*(x208))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x204)*(x209)))+(((IkReal(-1.00000000000000))*(x206)*(x208)))+(((IkReal(-1.00000000000000))*(x203)))+(((IkReal(-1.00000000000000))*(x207)*(x208)))+(((x202)*(x205))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x210=(sj5)*(sj5);
IkReal x211=((IkReal(0.305809000000000))*(x210));
if( IKabs(((IkReal(0.553000000000000))*(npx)*(sj5))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.553000000000000))*(npy)*(sj5))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x212=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.553000000000000))*(npx)*(sj5)), ((IkReal(-0.553000000000000))*(npy)*(sj5)))));
if( (((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(pp))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(pp))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x213=IKasin(((((IKabs(IKabs(IKsqrt(((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x211)*((npy)*(npy))))+(((x211)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(pp)))))));
j6array[0]=((((IkReal(-1.00000000000000))*(x213)))+(((IkReal(-1.00000000000000))*(x212))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(x213)+(((IkReal(-1.00000000000000))*(x212))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
evalcond[0]=((IkReal(-0.276500000000000))+(((IkReal(-1.00000000000000))*(npy)*(sj5)*(IKsin(j6))))+(((cj5)*(npz)))+(((npx)*(sj5)*(IKcos(j6))))+(((IkReal(-0.328000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x214=((IkReal(1.00000000000000))*(pp));
IkReal x215=((IkReal(1.00000000000000))*(sj5));
IkReal x216=((npy)*(sj6));
IkReal x217=((cj5)*(npz));
IkReal x218=((IkReal(0.553000000000000))*(sj5));
IkReal x219=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x214))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(((sj5)*(x219)))+(x217)+(((IkReal(-1.00000000000000))*(x215)*(x216))));
evalcond[4]=((IkReal(0.0311317500000000))+(((x218)*(x219)))+(((IkReal(0.553000000000000))*(x217)))+(((IkReal(-1.00000000000000))*(x214)))+(((IkReal(-1.00000000000000))*(x216)*(x218))));
evalcond[5]=((((cj5)*(x219)))+(((IkReal(-1.00000000000000))*(npz)*(x215)))+(((IkReal(-1.00000000000000))*(cj5)*(x216))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x220=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x220))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x220)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x221=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x220)))), ((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x220)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x221));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x221))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x222=IKcos(j4);
IkReal x223=IKsin(j4);
IkReal x224=((IkReal(1.00000000000000))*(x222));
IkReal x225=((IkReal(1.00000000000000))*(x223));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)*(x224)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x224)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x225)))+(((cj5)*(cj6)*(npx)*(x222)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x225))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x226=((IkReal(1.00000000000000))*(pp));
IkReal x227=((IkReal(1.00000000000000))*(sj5));
IkReal x228=((npy)*(sj6));
IkReal x229=((cj5)*(npz));
IkReal x230=((IkReal(0.553000000000000))*(sj5));
IkReal x231=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x226))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(((IkReal(-1.00000000000000))*(x227)*(x228)))+(x229)+(((sj5)*(x231))));
evalcond[4]=((IkReal(0.0311317500000000))+(((x230)*(x231)))+(((IkReal(0.553000000000000))*(x229)))+(((IkReal(-1.00000000000000))*(x228)*(x230)))+(((IkReal(-1.00000000000000))*(x226))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x228)))+(((cj5)*(x231)))+(((IkReal(-1.00000000000000))*(npz)*(x227))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x232=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x232)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x232)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x233=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x232)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((IkReal(-1.00000000000000))*(cj5)*(npx)*(x232)))+(((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x233));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x233))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x234=IKcos(j4);
IkReal x235=IKsin(j4);
IkReal x236=((IkReal(1.00000000000000))*(x234));
IkReal x237=((IkReal(1.00000000000000))*(x235));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x236)))+(((cj5)*(cj6)*(npx)*(x234)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x236)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x237)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x237))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x238=((IkReal(125.000000000000))*(cj6));
IkReal x239=((IkReal(125.000000000000))*(sj6));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x239)))+(((IkReal(-1.00000000000000))*(npy)*(x238))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(npx)*(x238)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x239)))+(((IkReal(-125.000000000000))*(npz)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x239)))+(((IkReal(-1.00000000000000))*(npy)*(x238)))))), ((gconst0)*(((((cj5)*(npx)*(x238)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x239)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x240=IKsin(j4);
IkReal x241=IKcos(j4);
IkReal x242=((IkReal(0.328000000000000))*(sj3));
IkReal x243=((npz)*(sj5));
IkReal x244=((cj5)*(npx));
IkReal x245=((IkReal(1.00000000000000))*(npy));
IkReal x246=((npx)*(sj6));
IkReal x247=((cj5)*(sj6));
IkReal x248=((cj6)*(x241));
IkReal x249=((IkReal(1.00000000000000))*(x241));
IkReal x250=((cj6)*(x240));
evalcond[0]=((((cj6)*(npy)))+(((x240)*(x242)))+(x246));
evalcond[1]=((((IkReal(-1.00000000000000))*(x243)))+(((IkReal(-1.00000000000000))*(x241)*(x242)))+(((IkReal(-1.00000000000000))*(x245)*(x247)))+(((cj6)*(x244))));
evalcond[2]=((((x240)*(x243)))+(((npy)*(x240)*(x247)))+(((IkReal(-1.00000000000000))*(x244)*(x250)))+(((IkReal(-1.00000000000000))*(x245)*(x248)))+(((IkReal(-1.00000000000000))*(x246)*(x249))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x243)*(x249)))+(((x244)*(x248)))+(((IkReal(-1.00000000000000))*(x245)*(x250)))+(((IkReal(-1.00000000000000))*(x240)*(x246)))+(((IkReal(-1.00000000000000))*(x242)))+(((IkReal(-1.00000000000000))*(x241)*(x245)*(x247))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x251=(sj5)*(sj5);
if( IKabs(((npx)*(sj5))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npy)*(sj5))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x252=((IkReal(1.00000000000000))*(IKatan2(((npx)*(sj5)), ((IkReal(-1.00000000000000))*(npy)*(sj5)))));
if( (((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((cj5)*(npz)))+(((IkReal(-0.328000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((cj5)*(npz)))+(((IkReal(-0.328000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x253=IKasin(((((IKabs(IKabs(IKsqrt(((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x251)*((npy)*(npy))))+(((x251)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((cj5)*(npz)))+(((IkReal(-0.328000000000000))*(cj3)))))));
j6array[0]=((((IkReal(-1.00000000000000))*(x253)))+(((IkReal(-1.00000000000000))*(x252))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(x253)+(((IkReal(-1.00000000000000))*(x252))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
IkReal x254=((IkReal(0.553000000000000))*(sj5));
evalcond[0]=((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(npy)*(x254)*(IKsin(j6))))+(((npx)*(x254)*(IKcos(j6)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x255=((IkReal(1.00000000000000))*(pp));
IkReal x256=((IkReal(1.00000000000000))*(sj5));
IkReal x257=((npy)*(sj6));
IkReal x258=((cj5)*(npz));
IkReal x259=((IkReal(0.553000000000000))*(sj5));
IkReal x260=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x255))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(((sj5)*(x260)))+(((IkReal(-1.00000000000000))*(x256)*(x257)))+(x258));
evalcond[4]=((IkReal(0.0311317500000000))+(((x259)*(x260)))+(((IkReal(-1.00000000000000))*(x257)*(x259)))+(((IkReal(0.553000000000000))*(x258)))+(((IkReal(-1.00000000000000))*(x255))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x257)))+(((cj5)*(x260)))+(((IkReal(-1.00000000000000))*(npz)*(x256))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x261=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x261)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x261)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x262=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x261)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x261)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x262));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x262))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x263=IKcos(j4);
IkReal x264=IKsin(j4);
IkReal x265=((IkReal(1.00000000000000))*(x263));
IkReal x266=((IkReal(1.00000000000000))*(x264));
evalcond[0]=((((cj5)*(cj6)*(npx)*(x263)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x265)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x266)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x265)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x266))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x267=((IkReal(1.00000000000000))*(pp));
IkReal x268=((IkReal(1.00000000000000))*(sj5));
IkReal x269=((npy)*(sj6));
IkReal x270=((cj5)*(npz));
IkReal x271=((IkReal(0.553000000000000))*(sj5));
IkReal x272=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x267))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(((sj5)*(x272)))+(x270)+(((IkReal(-1.00000000000000))*(x268)*(x269))));
evalcond[4]=((IkReal(0.0311317500000000))+(((x271)*(x272)))+(((IkReal(0.553000000000000))*(x270)))+(((IkReal(-1.00000000000000))*(x269)*(x271)))+(((IkReal(-1.00000000000000))*(x267))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x269)))+(((IkReal(-1.00000000000000))*(npz)*(x268)))+(((cj5)*(x272))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x273=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npy)*(x273)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x273)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x274=IKatan2(((((IkReal(-1.00000000000000))*(npy)*(x273)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))), ((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x273)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x274));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x274))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x275=IKcos(j4);
IkReal x276=IKsin(j4);
IkReal x277=((IkReal(1.00000000000000))*(x275));
IkReal x278=((IkReal(1.00000000000000))*(x276));
evalcond[0]=((((cj5)*(cj6)*(npx)*(x275)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x277)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x278)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x277)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x278))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x279=((IkReal(125.000000000000))*(cj6));
IkReal x280=((IkReal(125.000000000000))*(sj6));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x279)))+(((IkReal(-1.00000000000000))*(npx)*(x280))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(x280)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))+(((cj5)*(npx)*(x279))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npy)*(x279)))+(((IkReal(-1.00000000000000))*(npx)*(x280)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(x280)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))+(((cj5)*(npx)*(x279)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x281=IKsin(j4);
IkReal x282=IKcos(j4);
IkReal x283=((IkReal(0.328000000000000))*(sj3));
IkReal x284=((npz)*(sj5));
IkReal x285=((cj5)*(npx));
IkReal x286=((IkReal(1.00000000000000))*(npy));
IkReal x287=((npx)*(sj6));
IkReal x288=((cj5)*(sj6));
IkReal x289=((cj6)*(x282));
IkReal x290=((IkReal(1.00000000000000))*(x282));
IkReal x291=((cj6)*(x281));
evalcond[0]=((((cj6)*(npy)))+(x287)+(((x281)*(x283))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x282)*(x283)))+(((IkReal(-1.00000000000000))*(x284)))+(((cj6)*(x285)))+(((IkReal(-1.00000000000000))*(x286)*(x288))));
evalcond[2]=((((npy)*(x281)*(x288)))+(((IkReal(-1.00000000000000))*(x287)*(x290)))+(((IkReal(-1.00000000000000))*(x285)*(x291)))+(((IkReal(-1.00000000000000))*(x286)*(x289)))+(((x281)*(x284))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x284)*(x290)))+(((x285)*(x289)))+(((IkReal(-1.00000000000000))*(x286)*(x291)))+(((IkReal(-1.00000000000000))*(x281)*(x287)))+(((IkReal(-1.00000000000000))*(x283)))+(((IkReal(-1.00000000000000))*(x282)*(x286)*(x288))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x292=(sj5)*(sj5);
if( IKabs(((npx)*(sj5))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npy)*(sj5))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x293=((IkReal(1.00000000000000))*(IKatan2(((npx)*(sj5)), ((IkReal(-1.00000000000000))*(npy)*(sj5)))));
if( (((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((cj5)*(npz)))+(((IkReal(-0.328000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((cj5)*(npz)))+(((IkReal(-0.328000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x294=IKasin(((((IKabs(IKabs(IKsqrt(((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x292)*((npy)*(npy))))+(((x292)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(-0.276500000000000))+(((cj5)*(npz)))+(((IkReal(-0.328000000000000))*(cj3)))))));
j6array[0]=((((IkReal(-1.00000000000000))*(x294)))+(((IkReal(-1.00000000000000))*(x293))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x293)))+(x294));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
IkReal x295=((IkReal(0.553000000000000))*(sj5));
evalcond[0]=((IkReal(0.0311317500000000))+(((IkReal(0.553000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(npy)*(x295)*(IKsin(j6))))+(((npx)*(x295)*(IKcos(j6))))+(((IkReal(-1.00000000000000))*(pp))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x296=((IkReal(1.00000000000000))*(pp));
IkReal x297=((IkReal(1.00000000000000))*(sj5));
IkReal x298=((npy)*(sj6));
IkReal x299=((cj5)*(npz));
IkReal x300=((IkReal(0.553000000000000))*(sj5));
IkReal x301=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.365420250000000))+(((IkReal(-1.00000000000000))*(x296))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(-0.604500000000000))+(((sj5)*(x301)))+(((IkReal(-1.00000000000000))*(x297)*(x298)))+(x299));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x298)*(x300)))+(((IkReal(-1.00000000000000))*(x296)))+(((IkReal(0.553000000000000))*(x299)))+(((x300)*(x301))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x298)))+(((IkReal(-1.00000000000000))*(npz)*(x297)))+(((cj5)*(x301))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x302=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x302))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x302))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x303=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x302)))), ((((npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x302)))));
j4array[0]=((IkReal(-1.00000000000000))*(x303));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x303))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x304=IKcos(j4);
IkReal x305=IKsin(j4);
IkReal x306=((IkReal(1.00000000000000))*(x304));
IkReal x307=((IkReal(1.00000000000000))*(x305));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x307)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x306)))+(((cj5)*(cj6)*(npx)*(x304)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x306)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x307))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x308=((IkReal(1.00000000000000))*(pp));
IkReal x309=((IkReal(1.00000000000000))*(sj5));
IkReal x310=((npy)*(sj6));
IkReal x311=((cj5)*(npz));
IkReal x312=((IkReal(0.553000000000000))*(sj5));
IkReal x313=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.00265225000000000))+(((IkReal(-1.00000000000000))*(x308))));
evalcond[2]=((((cj6)*(npy)))+(((npx)*(sj6))));
evalcond[3]=((IkReal(0.0515000000000000))+(x311)+(((sj5)*(x313)))+(((IkReal(-1.00000000000000))*(x309)*(x310))));
evalcond[4]=((IkReal(0.0311317500000000))+(((IkReal(-1.00000000000000))*(x310)*(x312)))+(((x312)*(x313)))+(((IkReal(0.553000000000000))*(x311)))+(((IkReal(-1.00000000000000))*(x308))));
evalcond[5]=((((cj5)*(x313)))+(((IkReal(-1.00000000000000))*(cj5)*(x310)))+(((IkReal(-1.00000000000000))*(npz)*(x309))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x314=((IkReal(1.00000000000000))*(cj6));
if( IKabs(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x314))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x314)))+(((cj5)*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x315=IKatan2(((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x314)))), ((((npz)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(npx)*(x314)))+(((cj5)*(npy)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x315));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x315))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x316=IKcos(j4);
IkReal x317=IKsin(j4);
IkReal x318=((IkReal(1.00000000000000))*(x316));
IkReal x319=((IkReal(1.00000000000000))*(x317));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x319)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x318)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x319)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x318)))+(((cj5)*(cj6)*(npx)*(x316))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x320=((IkReal(125.000000000000))*(cj6));
IkReal x321=((IkReal(125.000000000000))*(sj6));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x321)))+(((IkReal(-1.00000000000000))*(npy)*(x320))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(npx)*(x320)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x321)))+(((IkReal(-125.000000000000))*(npz)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x321)))+(((IkReal(-1.00000000000000))*(npy)*(x320)))))), ((gconst0)*(((((cj5)*(npx)*(x320)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(x321)))+(((IkReal(-125.000000000000))*(npz)*(sj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x322=IKsin(j4);
IkReal x323=IKcos(j4);
IkReal x324=((IkReal(0.328000000000000))*(sj3));
IkReal x325=((npz)*(sj5));
IkReal x326=((cj5)*(npx));
IkReal x327=((IkReal(1.00000000000000))*(npy));
IkReal x328=((npx)*(sj6));
IkReal x329=((cj5)*(sj6));
IkReal x330=((cj6)*(x323));
IkReal x331=((IkReal(1.00000000000000))*(x323));
IkReal x332=((cj6)*(x322));
evalcond[0]=((((x322)*(x324)))+(((cj6)*(npy)))+(x328));
evalcond[1]=((((IkReal(-1.00000000000000))*(x327)*(x329)))+(((IkReal(-1.00000000000000))*(x325)))+(((IkReal(-1.00000000000000))*(x323)*(x324)))+(((cj6)*(x326))));
evalcond[2]=((((x322)*(x325)))+(((IkReal(-1.00000000000000))*(x328)*(x331)))+(((IkReal(-1.00000000000000))*(x326)*(x332)))+(((npy)*(x322)*(x329)))+(((IkReal(-1.00000000000000))*(x327)*(x330))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x322)*(x328)))+(((IkReal(-1.00000000000000))*(x324)))+(((x326)*(x330)))+(((IkReal(-1.00000000000000))*(x323)*(x327)*(x329)))+(((IkReal(-1.00000000000000))*(x327)*(x332)))+(((IkReal(-1.00000000000000))*(x325)*(x331))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x85=((cj6)*(sj4));
IkReal x86=((IkReal(1.00000000000000))*(sj5));
IkReal x87=((cj4)*(cj5));
IkReal x88=((IkReal(1.00000000000000))*(sj3));
IkReal x89=((IkReal(1.00000000000000))*(sj6));
IkReal x90=((IkReal(-1.00000000000000))*(cj3));
IkReal x91=((sj4)*(x89));
IkReal x92=((((IkReal(-1.00000000000000))*(sj3)*(x86)))+(((cj3)*(x87))));
IkReal x93=((((cj4)*(cj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x91))));
IkReal x94=((((cj4)*(sj6)))+(((cj5)*(x85))));
IkReal x95=((((cj3)*(sj5)))+(((sj3)*(x87))));
IkReal x96=((((cj3)*(cj5)))+(((IkReal(-1.00000000000000))*(cj4)*(sj3)*(x86))));
IkReal x97=((cj6)*(x92));
IkReal x98=((sj6)*(x92));
IkReal x99=((((IkReal(-1.00000000000000))*(cj5)*(x88)))+(((IkReal(-1.00000000000000))*(cj3)*(cj4)*(x86))));
IkReal x100=((x97)+(((IkReal(-1.00000000000000))*(cj3)*(x91))));
IkReal x101=((((cj6)*(x95)))+(((IkReal(-1.00000000000000))*(sj4)*(sj6)*(x88))));
IkReal x102=((((IkReal(-1.00000000000000))*(x85)*(x88)))+(((IkReal(-1.00000000000000))*(x89)*(x95))));
IkReal x103=((((x85)*(x90)))+(((IkReal(-1.00000000000000))*(x98))));
new_r00=((((r01)*(((((IkReal(-1.00000000000000))*(cj3)*(x85)))+(((IkReal(-1.00000000000000))*(x89)*(x92)))))))+(((r00)*(((x97)+(((sj4)*(sj6)*(x90)))))))+(((r02)*(x99))));
new_r01=((((r12)*(x99)))+(((r10)*(x100)))+(((r11)*(x103))));
new_r02=((((r21)*(x103)))+(((r20)*(x100)))+(((r22)*(x99))));
new_r10=((((r01)*(x93)))+(((r00)*(x94)))+(((IkReal(-1.00000000000000))*(r02)*(sj4)*(x86))));
new_r11=((((r11)*(x93)))+(((r10)*(x94)))+(((IkReal(-1.00000000000000))*(r12)*(sj4)*(x86))));
new_r12=((((r21)*(x93)))+(((IkReal(-1.00000000000000))*(r22)*(sj4)*(x86)))+(((r20)*(x94))));
new_r20=((((r01)*(x102)))+(((r00)*(x101)))+(((r02)*(x96))));
new_r21=((((r12)*(x96)))+(((r10)*(x101)))+(((r11)*(x102))));
new_r22=((((r21)*(x102)))+(((r20)*(x101)))+(((r22)*(x96))));
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
cj1array[0]=new_r22;
if( cj1array[0] >= -1-IKFAST_SINCOS_THRESH && cj1array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j1valid[0] = j1valid[1] = true;
    j1array[0] = IKacos(cj1array[0]);
    sj1array[0] = IKsin(j1array[0]);
    cj1array[1] = cj1array[0];
    j1array[1] = -j1array[0];
    sj1array[1] = -sj1array[0];
}
else if( isnan(cj1array[0]) )
{
    // probably any value will work
    j1valid[0] = true;
    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
}
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(-1.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(-1.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(-1.00000000000000))*((new_r12)*(new_r12)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
IkReal x104=((IkReal(-1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x104;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x104;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
if( IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x105=IKatan2(new_r12, new_r02);
j2array[0]=((IkReal(-1.00000000000000))*(x105));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x105))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r12)*(IKsin(j2))))+(((new_r02)*(IKcos(j2)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r01)*(sj2)))+(((cj2)*(new_r11))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))))+IKsqr(((((new_r01)*(sj2)))+(((cj2)*(new_r11)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))), ((((new_r01)*(sj2)))+(((cj2)*(new_r11)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x106=IKsin(j0);
IkReal x107=((IkReal(1.00000000000000))*(sj2));
IkReal x108=((IkReal(1.00000000000000))*(IKcos(j0)));
evalcond[0]=((((new_r00)*(sj2)))+(x106)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((IkReal(-1.00000000000000))*(x108)))+(((cj2)*(new_r11))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x107)))+(((IkReal(-1.00000000000000))*(x108)))+(((cj2)*(new_r00))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x107)))+(((cj2)*(new_r01)))+(((IkReal(-1.00000000000000))*(x106))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
if( IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x109=IKatan2(new_r12, new_r02);
j2array[0]=((IkReal(-1.00000000000000))*(x109));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x109))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r12)*(IKsin(j2))))+(((new_r02)*(IKcos(j2)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x110=((IkReal(1.00000000000000))*(new_r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(sj2)*(x110))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x110))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(sj2)*(x110)))))+IKsqr(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x110)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(sj2)*(x110)))), ((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x110)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x111=IKcos(j0);
IkReal x112=IKsin(j0);
IkReal x113=((IkReal(1.00000000000000))*(sj2));
evalcond[0]=((((new_r00)*(sj2)))+(x112)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((cj2)*(new_r11)))+(((IkReal(-1.00000000000000))*(x111))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x113)))+(x111)+(((cj2)*(new_r00))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x113)))+(x112)+(((cj2)*(new_r01))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x114=((gconst4)*(sj1));
if( IKabs(((new_r12)*(x114))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r02)*(x114))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((new_r12)*(x114)), ((IkReal(-1.00000000000000))*(new_r02)*(x114)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[6];
IkReal x115=IKcos(j2);
IkReal x116=IKsin(j2);
IkReal x117=((IkReal(1.00000000000000))*(sj1));
IkReal x118=((sj1)*(x115));
IkReal x119=((IkReal(1.00000000000000))*(x116));
IkReal x120=((new_r02)*(x115));
evalcond[0]=((((new_r02)*(x116)))+(((new_r12)*(x115))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x117)))+(x120)+(((IkReal(-1.00000000000000))*(new_r12)*(x119))));
evalcond[2]=((((cj1)*(new_r20)))+(((new_r00)*(x118)))+(((IkReal(-1.00000000000000))*(new_r10)*(x116)*(x117))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x116)*(x117)))+(((cj1)*(new_r21)))+(((new_r01)*(x118))));
evalcond[4]=((((cj1)*(x120)))+(((IkReal(-1.00000000000000))*(new_r22)*(x117)))+(((IkReal(-1.00000000000000))*(cj1)*(new_r12)*(x119))));
evalcond[5]=((IkReal(-1.00000000000000))+(((cj1)*(new_r22)))+(((IkReal(-1.00000000000000))*(new_r12)*(x116)*(x117)))+(((new_r02)*(x118))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x121=((IkReal(-1.00000000000000))+(new_r22));
IkReal x122=((IkReal(1.00000000000000))*(new_r12));
IkReal x123=((new_r02)*(sj2));
IkReal x124=((((IkReal(-1.00000000000000))*(sj2)*(x122)))+(((cj2)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x121;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x123)+(((cj2)*(new_r12))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj2)*(x122)))+(((IkReal(-1.00000000000000))*(x123))));
evalcond[6]=x124;
evalcond[7]=new_r20;
evalcond[8]=new_r21;
evalcond[9]=x124;
evalcond[10]=x121;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r01)*(sj2)))+(((cj2)*(new_r11))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))))+IKsqr(((((new_r01)*(sj2)))+(((cj2)*(new_r11)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj2)))), ((((new_r01)*(sj2)))+(((cj2)*(new_r11)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x125=IKsin(j0);
IkReal x126=((IkReal(1.00000000000000))*(sj2));
IkReal x127=((IkReal(1.00000000000000))*(IKcos(j0)));
evalcond[0]=((((new_r00)*(sj2)))+(x125)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((IkReal(-1.00000000000000))*(x127)))+(((cj2)*(new_r11))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x126)))+(((IkReal(-1.00000000000000))*(x127)))+(((cj2)*(new_r00))));
evalcond[3]=((((cj2)*(new_r01)))+(((IkReal(-1.00000000000000))*(x125)))+(((IkReal(-1.00000000000000))*(new_r11)*(x126))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x128=((new_r12)*(sj2));
IkReal x129=((new_r02)*(sj2));
IkReal x130=((cj2)*(new_r12));
IkReal x131=((cj2)*(new_r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x130)+(x129));
evalcond[5]=((((IkReal(-1.00000000000000))*(x130)))+(((IkReal(-1.00000000000000))*(x129))));
evalcond[6]=((x131)+(((IkReal(-1.00000000000000))*(x128))));
evalcond[7]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[9]=((x128)+(((IkReal(-1.00000000000000))*(x131))));
evalcond[10]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x132=((IkReal(1.00000000000000))*(new_r00));
if( IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x132)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x132))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x132)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10)))))+IKsqr(((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x132)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j0array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj2)*(x132)))+(((IkReal(-1.00000000000000))*(cj2)*(new_r10)))), ((((new_r10)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x132)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x133=IKcos(j0);
IkReal x134=IKsin(j0);
IkReal x135=((IkReal(1.00000000000000))*(sj2));
evalcond[0]=((((new_r00)*(sj2)))+(x134)+(((cj2)*(new_r10))));
evalcond[1]=((((new_r01)*(sj2)))+(((cj2)*(new_r11)))+(((IkReal(-1.00000000000000))*(x133))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x135)))+(x133)+(((cj2)*(new_r00))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x135)))+(x134)+(((cj2)*(new_r01))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x136=((IkReal(-1.00000000000000))*(gconst5));
if( IKabs(((new_r21)*(x136))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r20)*(x136))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r21)*(x136)), ((new_r20)*(x136)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[8];
IkReal x137=IKsin(j0);
IkReal x138=IKcos(j0);
IkReal x139=((IkReal(1.00000000000000))*(cj1));
IkReal x140=((new_r10)*(sj2));
IkReal x141=((cj2)*(new_r01));
IkReal x142=((IkReal(1.00000000000000))*(sj1));
IkReal x143=((new_r11)*(sj2));
IkReal x144=((cj2)*(new_r00));
IkReal x145=((IkReal(1.00000000000000))*(x138));
evalcond[0]=((new_r20)+(((sj1)*(x138))));
evalcond[1]=((new_r21)+(((sj1)*(x137))));
evalcond[2]=((((new_r00)*(sj2)))+(x137)+(((cj2)*(new_r10))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x145)))+(((new_r01)*(sj2)))+(((cj2)*(new_r11))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x138)*(x139)))+(x144)+(((IkReal(-1.00000000000000))*(x140))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x137)*(x139)))+(x141)+(((IkReal(-1.00000000000000))*(x143))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x145)))+(((IkReal(-1.00000000000000))*(new_r20)*(x142)))+(((IkReal(-1.00000000000000))*(x139)*(x140)))+(((cj1)*(x144))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r21)*(x142)))+(((IkReal(-1.00000000000000))*(x139)*(x143)))+(((IkReal(-1.00000000000000))*(x137)))+(((cj1)*(x141))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x146=((IkReal(-1.00000000000000))*(gconst3));
if( IKabs(((new_r21)*(x146))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r20)*(x146))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r21)*(x146)), ((new_r20)*(x146)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
evalcond[0]=((((sj1)*(IKcos(j0))))+(new_r20));
evalcond[1]=((((sj1)*(IKsin(j0))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(((((IkReal(-1.00000000000000))*(new_r00)*(new_r12)))+(((new_r02)*(new_r10)))));
dummyeval[0]=((((IkReal(-1.00000000000000))*(new_r00)*(new_r12)))+(((new_r02)*(new_r10))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x147=((gconst6)*(sj0));
if( IKabs(((new_r12)*(x147))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r02)*(x147))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((new_r12)*(x147)), ((IkReal(-1.00000000000000))*(new_r02)*(x147)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[12];
IkReal x148=IKsin(j2);
IkReal x149=IKcos(j2);
IkReal x150=((IkReal(1.00000000000000))*(cj0));
IkReal x151=((IkReal(1.00000000000000))*(sj1));
IkReal x152=((IkReal(1.00000000000000))*(cj1));
IkReal x153=((new_r10)*(x148));
IkReal x154=((new_r12)*(x148));
IkReal x155=((new_r01)*(x149));
IkReal x156=((new_r00)*(x149));
IkReal x157=((new_r02)*(x149));
IkReal x158=((IkReal(1.00000000000000))*(new_r11)*(x148));
evalcond[0]=((((new_r12)*(x149)))+(((new_r02)*(x148))));
evalcond[1]=((sj0)+(((new_r00)*(x148)))+(((new_r10)*(x149))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x150)))+(((new_r01)*(x148)))+(((new_r11)*(x149))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x151)))+(x157)+(((IkReal(-1.00000000000000))*(x154))));
evalcond[4]=((x156)+(((IkReal(-1.00000000000000))*(x153)))+(((IkReal(-1.00000000000000))*(cj1)*(x150))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x158)))+(((IkReal(-1.00000000000000))*(sj0)*(x152)))+(x155));
evalcond[6]=((((cj1)*(new_r20)))+(((IkReal(-1.00000000000000))*(x151)*(x153)))+(((sj1)*(x156))));
evalcond[7]=((((cj1)*(new_r21)))+(((sj1)*(x155)))+(((IkReal(-1.00000000000000))*(new_r11)*(x148)*(x151))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r22)*(x151)))+(((IkReal(-1.00000000000000))*(x152)*(x154)))+(((cj1)*(x157))));
evalcond[9]=((IkReal(-1.00000000000000))+(((cj1)*(new_r22)))+(((IkReal(-1.00000000000000))*(x151)*(x154)))+(((sj1)*(x157))));
evalcond[10]=((((IkReal(-1.00000000000000))*(new_r20)*(x151)))+(((IkReal(-1.00000000000000))*(x150)))+(((IkReal(-1.00000000000000))*(x152)*(x153)))+(((cj1)*(x156))));
evalcond[11]=((((IkReal(-1.00000000000000))*(sj0)))+(((IkReal(-1.00000000000000))*(new_r21)*(x151)))+(((cj1)*(x155)))+(((IkReal(-1.00000000000000))*(new_r11)*(x148)*(x152))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}static inline void polyroots6(IkReal rawcoeffs[6+1], IkReal rawroots[6], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[6];
    const int maxsteps = 110;
    for(int i = 0; i < 6; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[6];
    IkReal err[6];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 6; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 6; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 6; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 6; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[6] = {false};
    for(int i = 0; i < 6; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 6; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "bcb25302dbec7575df10994dd89abd19"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
