/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:44:08.935599
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.182500000000000))*(x5));
x15=((IkReal(0.182500000000000))*(x8));
x16=((IkReal(0.276500000000000))*(x6));
x17=((IkReal(0.182500000000000))*(x6));
x18=((IkReal(1.00000000000000))*(x5));
x19=((IkReal(1.00000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x4));
x21=((IkReal(0.276500000000000))*(x0));
x22=((IkReal(0.182500000000000))*(x9));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(1.00000000000000))*(x3));
x25=((IkReal(1.00000000000000))*(x8));
x26=((IkReal(0.276500000000000))*(x3));
x27=((IkReal(1.00000000000000))*(x11));
x28=((IkReal(1.00000000000000))*(x9));
x29=((x3)*(x7));
x30=((x2)*(x7));
x31=((x1)*(x2));
x32=((x0)*(x7));
x33=((x20)*(x3));
x34=((x18)*(x32));
x35=((x18)*(x29));
x36=((x20)*(x7)*(x9));
x37=((((IkReal(-1.00000000000000))*(x33)))+(((x0)*(x31))));
x38=((((x0)*(x4)))+(((x3)*(x31))));
x39=((((IkReal(-1.00000000000000))*(x1)*(x33)))+(((x0)*(x2))));
x40=((((IkReal(-1.00000000000000))*(x18)*(x30)))+(((x1)*(x6))));
x41=((((x1)*(x5)))+(((x30)*(x6))));
x42=((((IkReal(-1.00000000000000))*(x0)*(x31)))+(x33));
x43=((((IkReal(-1.00000000000000))*(x0)*(x1)*(x20)))+(((IkReal(-1.00000000000000))*(x2)*(x24))));
x44=((((IkReal(-1.00000000000000))*(x0)*(x20)))+(((IkReal(-1.00000000000000))*(x24)*(x31))));
x45=((x41)*(x8));
x46=((x37)*(x6));
x47=((x44)*(x5));
x48=((x43)*(x9));
x49=((((IkReal(-1.00000000000000))*(x34)))+(x46));
x50=((((IkReal(-1.00000000000000))*(x35)))+(((x38)*(x6))));
x51=((((IkReal(-1.00000000000000))*(x36)))+(x45));
x52=((((x42)*(x5)))+(((IkReal(-1.00000000000000))*(x19)*(x32))));
x53=((((IkReal(-1.00000000000000))*(x19)*(x29)))+(x47));
x54=((((x9)*(((((IkReal(-1.00000000000000))*(x1)*(x18)))+(((IkReal(-1.00000000000000))*(x19)*(x30)))))))+(((IkReal(-1.00000000000000))*(x20)*(x7)*(x8))));
x55=((x50)*(x8));
x56=((x11)*(x52));
x57=((x11)*(x53));
x58=((x55)+(((x39)*(x9))));
x59=((((x9)*(((((IkReal(-1.00000000000000))*(x19)*(x38)))+(x35)))))+(((x39)*(x8))));
x60=((((x49)*(x8)))+(x48));
x61=((x12)*(x58));
x62=((((x9)*(((((IkReal(-1.00000000000000))*(x46)))+(x34)))))+(((x43)*(x8))));
eerot[0]=((((x13)*(((((x12)*(x60)))+(x56)))))+(((x10)*(x62))));
eerot[1]=((((x13)*(x62)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x52)))+(((IkReal(-1.00000000000000))*(x23)*(x60))))))));
eerot[2]=((((x12)*(x52)))+(((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x49)))+(((IkReal(-1.00000000000000))*(x28)*(x43))))))));
IkReal x63=((IkReal(1.00000000000000))*(x32));
eetrans[0]=((((IkReal(-0.328000000000000))*(x32)))+(((IkReal(-1.00000000000000))*(x16)*(x63)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x49)))+(((IkReal(-1.00000000000000))*(x22)*(x43)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x63)))+(((x14)*(x42)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x31)))+(((x26)*(x4))))))));
eerot[3]=((((x13)*(((x57)+(x61)))))+(((x10)*(x59))));
eerot[4]=((((x13)*(x59)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x53)))+(((IkReal(-1.00000000000000))*(x23)*(x58))))))));
eerot[5]=((((x12)*(x53)))+(((x11)*(((((IkReal(-1.00000000000000))*(x28)*(x39)))+(((IkReal(-1.00000000000000))*(x25)*(x50))))))));
IkReal x64=((IkReal(1.00000000000000))*(x29));
eetrans[1]=((((IkReal(-1.00000000000000))*(x16)*(x64)))+(((x11)*(((((IkReal(-1.00000000000000))*(x22)*(x39)))+(((IkReal(-1.00000000000000))*(x15)*(x50)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x4)))+(((IkReal(-1.00000000000000))*(x26)*(x31)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x64)))+(((x14)*(x44)))))))+(((IkReal(-0.328000000000000))*(x29))));
eerot[6]=((((x13)*(((((x12)*(x51)))+(((x11)*(x40)))))))+(((x10)*(x54))));
eerot[7]=((((x13)*(x54)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x40)))+(((IkReal(-1.00000000000000))*(x23)*(x51))))))));
eerot[8]=((((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x41)))+(x36)))))+(((x12)*(x40))));
eetrans[2]=((IkReal(0.300000000000000))+(((x1)*(x16)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x41)))+(((x22)*(x4)*(x7)))))))+(((IkReal(-0.276500000000000))*(x30)*(x5)))+(((IkReal(0.328000000000000))*(x1)))+(((x12)*(((((x1)*(x17)))+(((IkReal(-1.00000000000000))*(x14)*(x30))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j1,cj1,sj1,htj1,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((((IkReal(-0.182500000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((((IkReal(-0.182500000000000))*(r12)))+(py));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.300000000000000))+(pz)+(((IkReal(-0.182500000000000))*(r22))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal dummyeval[1];
IkReal x65=(sj1)*(sj1);
dummyeval[0]=((((x65)*((px)*(px))))+(((x65)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x66=(sj1)*(sj1);
dummyeval[0]=((((x66)*((px)*(px))))+(((x66)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x67=(sj1)*(sj1);
IkReal x68=((IkReal(0.430336000000000))*(x67));
IkReal x69=((IkReal(-0.656000000000000))*(sj1));
if( IKabs(((px)*(x69))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((py)*(x69))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x70=((IkReal(1.00000000000000))*(IKatan2(((px)*(x69)), ((py)*(x69)))));
if( (((((x68)*((py)*(py))))+(((x68)*((px)*(px)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x68)*((py)*(py))))+(((x68)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x68)*((py)*(py))))+(((x68)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x68)*((py)*(py))))+(((x68)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x68)*((py)*(py))))+(((x68)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x71=IKasin(((((IKabs(IKabs(IKsqrt(((((x68)*((py)*(py))))+(((x68)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x68)*((py)*(py))))+(((x68)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x70)))+(((IkReal(-1.00000000000000))*(x71))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x70)))+(x71));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x72=((IkReal(1.18625678119349))*(sj1));
cj5array[0]=((((cj0)*(r02)*(x72)))+(((r12)*(sj0)*(x72)))+(((IkReal(3.61663652802893))*(npz)))+(((IkReal(-1.18625678119349))*(cj1)*(r22))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x73=((IkReal(0.328000000000000))*(cj1));
IkReal x74=((sj0)*(sj1));
IkReal x75=((IkReal(0.328000000000000))*(cj0)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.656000000000000))*(py)*(x74)))+(((IkReal(-0.656000000000000))*(cj0)*(px)*(sj1))));
evalcond[2]=((IkReal(-0.276500000000000))+(((IkReal(-1.00000000000000))*(r22)*(x73)))+(((IkReal(0.328000000000000))*(r12)*(x74)))+(npz)+(((r02)*(x75))));
evalcond[3]=((((IkReal(0.328000000000000))*(r10)*(x74)))+(npx)+(((IkReal(-1.00000000000000))*(r20)*(x73)))+(((r00)*(x75))));
evalcond[4]=((((IkReal(0.328000000000000))*(r11)*(x74)))+(npy)+(((IkReal(-1.00000000000000))*(r21)*(x73)))+(((r01)*(x75))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x76=((IkReal(0.328000000000000))*(cj1));
IkReal x77=((IkReal(0.328000000000000))*(sj1));
if( IKabs(((npx)+(((cj0)*(r00)*(x77)))+(((IkReal(-1.00000000000000))*(r20)*(x76)))+(((r10)*(sj0)*(x77))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r21)*(x76)))+(((IkReal(-1.00000000000000))*(npy)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x77)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x77))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x78=IKatan2(((npx)+(((cj0)*(r00)*(x77)))+(((IkReal(-1.00000000000000))*(r20)*(x76)))+(((r10)*(sj0)*(x77)))), ((((r21)*(x76)))+(((IkReal(-1.00000000000000))*(npy)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x77)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x77)))));
j6array[0]=((IkReal(-1.00000000000000))*(x78));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x78))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
IkReal x79=IKcos(j6);
IkReal x80=IKsin(j6);
IkReal x81=((IkReal(0.328000000000000))*(cj1));
IkReal x82=((IkReal(0.328000000000000))*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(r00)*(x80)*(x82)))+(((IkReal(-1.00000000000000))*(npy)*(x79)))+(((IkReal(-1.00000000000000))*(npx)*(x80)))+(((IkReal(-1.00000000000000))*(r10)*(sj0)*(x80)*(x82)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x79)*(x82)))+(((r20)*(x80)*(x81)))+(((r21)*(x79)*(x81)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x79)*(x82))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x157=((IkReal(0.328000000000000))*(cj1));
IkReal x158=((sj0)*(sj1));
IkReal x159=((IkReal(0.328000000000000))*(cj0)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.656000000000000))*(cj0)*(px)*(sj1)))+(((IkReal(-0.656000000000000))*(py)*(x158))));
evalcond[2]=((IkReal(0.276500000000000))+(npz)+(((IkReal(0.328000000000000))*(r12)*(x158)))+(((r02)*(x159)))+(((IkReal(-1.00000000000000))*(r22)*(x157))));
evalcond[3]=((((r00)*(x159)))+(npx)+(((IkReal(0.328000000000000))*(r10)*(x158)))+(((IkReal(-1.00000000000000))*(r20)*(x157))));
evalcond[4]=((((r01)*(x159)))+(npy)+(((IkReal(0.328000000000000))*(r11)*(x158)))+(((IkReal(-1.00000000000000))*(r21)*(x157))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x160=((IkReal(0.328000000000000))*(cj1));
IkReal x161=((IkReal(0.328000000000000))*(sj1));
if( IKabs(((((cj0)*(r00)*(x161)))+(((r10)*(sj0)*(x161)))+(npx)+(((IkReal(-1.00000000000000))*(r20)*(x160))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(npy)))+(((r21)*(x160)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x161)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x161))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x162=IKatan2(((((cj0)*(r00)*(x161)))+(((r10)*(sj0)*(x161)))+(npx)+(((IkReal(-1.00000000000000))*(r20)*(x160)))), ((((IkReal(-1.00000000000000))*(npy)))+(((r21)*(x160)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x161)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x161)))));
j6array[0]=((IkReal(-1.00000000000000))*(x162));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x162))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
IkReal x163=IKcos(j6);
IkReal x164=IKsin(j6);
IkReal x165=((IkReal(0.328000000000000))*(cj1));
IkReal x166=((IkReal(0.328000000000000))*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(r00)*(x164)*(x166)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x163)*(x166)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x163)*(x166)))+(((r20)*(x164)*(x165)))+(((r21)*(x163)*(x165)))+(((IkReal(-1.00000000000000))*(r10)*(sj0)*(x164)*(x166)))+(((IkReal(-1.00000000000000))*(npx)*(x164)))+(((IkReal(-1.00000000000000))*(npy)*(x163))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x167=((IkReal(656.000000000000))*(sj1));
IkReal x168=((IkReal(656.000000000000))*(cj1));
if( IKabs(((gconst0)*(((((r21)*(x168)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x167)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x167)))+(((IkReal(-2000.00000000000))*(npy))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj0)*(r00)*(x167)))+(((r10)*(sj0)*(x167)))+(((IkReal(2000.00000000000))*(npx)))+(((IkReal(-1.00000000000000))*(r20)*(x168))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(((((r21)*(x168)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x167)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x167)))+(((IkReal(-2000.00000000000))*(npy)))))), ((gconst0)*(((((cj0)*(r00)*(x167)))+(((r10)*(sj0)*(x167)))+(((IkReal(2000.00000000000))*(npx)))+(((IkReal(-1.00000000000000))*(r20)*(x168)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x169=IKcos(j6);
IkReal x170=IKsin(j6);
IkReal x171=((IkReal(0.328000000000000))*(sj1));
IkReal x172=((r10)*(sj0));
IkReal x173=((IkReal(0.328000000000000))*(cj1));
IkReal x174=((r11)*(sj0));
IkReal x175=((IkReal(0.276500000000000))*(sj5));
IkReal x176=((cj0)*(x170));
IkReal x177=((IkReal(1.00000000000000))*(x170));
IkReal x178=((cj0)*(x169));
IkReal x179=((x170)*(x173));
evalcond[0]=((((cj0)*(r00)*(x171)))+(npx)+(((x171)*(x172)))+(((IkReal(-1.00000000000000))*(r20)*(x173)))+(((IkReal(-1.00000000000000))*(x169)*(x175))));
evalcond[1]=((((cj0)*(r01)*(x171)))+(npy)+(((x171)*(x174)))+(((x170)*(x175)))+(((IkReal(-1.00000000000000))*(r21)*(x173))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x171)*(x178)))+(((IkReal(-1.00000000000000))*(x170)*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(x169)*(x171)*(x174)))+(((r20)*(x179)))+(((IkReal(-1.00000000000000))*(npx)*(x177)))+(((IkReal(-1.00000000000000))*(npy)*(x169)))+(((r21)*(x169)*(x173)))+(((IkReal(-1.00000000000000))*(r00)*(x171)*(x176))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(x171)*(x176)))+(((IkReal(-1.00000000000000))*(r20)*(x169)*(x173)))+(((IkReal(-1.00000000000000))*(x170)*(x171)*(x174)))+(((IkReal(-1.00000000000000))*(x175)))+(((r21)*(x179)))+(((IkReal(-1.00000000000000))*(npy)*(x177)))+(((r00)*(x171)*(x178)))+(((npx)*(x169)))+(((x169)*(x171)*(x172))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x180=(sj1)*(sj1);
IkReal x181=((IkReal(0.430336000000000))*(x180));
IkReal x182=((IkReal(-0.656000000000000))*(sj1));
if( IKabs(((px)*(x182))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((py)*(x182))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x183=((IkReal(1.00000000000000))*(IKatan2(((px)*(x182)), ((py)*(x182)))));
if( (((((x181)*((px)*(px))))+(((x181)*((py)*(py)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x181)*((px)*(px))))+(((x181)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x181)*((px)*(px))))+(((x181)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x181)*((px)*(px))))+(((x181)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x181)*((px)*(px))))+(((x181)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x184=IKasin(((((IKabs(IKabs(IKsqrt(((((x181)*((px)*(px))))+(((x181)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x181)*((px)*(px))))+(((x181)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x183)))+(((IkReal(-1.00000000000000))*(x184))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x183)))+(x184));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x185=((IkReal(1.18625678119349))*(sj1));
cj5array[0]=((((cj0)*(r02)*(x185)))+(((IkReal(3.61663652802893))*(npz)))+(((r12)*(sj0)*(x185)))+(((IkReal(-1.18625678119349))*(cj1)*(r22))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x186=((IkReal(0.328000000000000))*(cj1));
IkReal x187=((sj0)*(sj1));
IkReal x188=((IkReal(0.328000000000000))*(cj0)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.656000000000000))*(py)*(x187)))+(((IkReal(-0.656000000000000))*(cj0)*(px)*(sj1))));
evalcond[2]=((IkReal(-0.276500000000000))+(((IkReal(0.328000000000000))*(r12)*(x187)))+(((IkReal(-1.00000000000000))*(r22)*(x186)))+(npz)+(((r02)*(x188))));
evalcond[3]=((((r00)*(x188)))+(npx)+(((IkReal(0.328000000000000))*(r10)*(x187)))+(((IkReal(-1.00000000000000))*(r20)*(x186))));
evalcond[4]=((((IkReal(0.328000000000000))*(r11)*(x187)))+(((r01)*(x188)))+(npy)+(((IkReal(-1.00000000000000))*(r21)*(x186))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x189=((IkReal(0.328000000000000))*(cj1));
IkReal x190=((IkReal(0.328000000000000))*(sj1));
if( IKabs(((((cj0)*(r00)*(x190)))+(((r10)*(sj0)*(x190)))+(npx)+(((IkReal(-1.00000000000000))*(r20)*(x189))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(r01)*(x190)))+(((IkReal(-1.00000000000000))*(npy)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x190)))+(((r21)*(x189))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x191=IKatan2(((((cj0)*(r00)*(x190)))+(((r10)*(sj0)*(x190)))+(npx)+(((IkReal(-1.00000000000000))*(r20)*(x189)))), ((((IkReal(-1.00000000000000))*(cj0)*(r01)*(x190)))+(((IkReal(-1.00000000000000))*(npy)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x190)))+(((r21)*(x189)))));
j6array[0]=((IkReal(-1.00000000000000))*(x191));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x191))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
IkReal x192=IKcos(j6);
IkReal x193=IKsin(j6);
IkReal x194=((IkReal(0.328000000000000))*(cj1));
IkReal x195=((IkReal(0.328000000000000))*(sj1));
evalcond[0]=((((r20)*(x193)*(x194)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x192)*(x195)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x192)*(x195)))+(((IkReal(-1.00000000000000))*(npx)*(x193)))+(((r21)*(x192)*(x194)))+(((IkReal(-1.00000000000000))*(npy)*(x192)))+(((IkReal(-1.00000000000000))*(r10)*(sj0)*(x193)*(x195)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x193)*(x195))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x196=((IkReal(0.328000000000000))*(cj1));
IkReal x197=((sj0)*(sj1));
IkReal x198=((IkReal(0.328000000000000))*(cj0)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0311317500000000))+(((IkReal(0.656000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.656000000000000))*(cj0)*(px)*(sj1)))+(((IkReal(-0.656000000000000))*(py)*(x197))));
evalcond[2]=((IkReal(0.276500000000000))+(((IkReal(0.328000000000000))*(r12)*(x197)))+(((IkReal(-1.00000000000000))*(r22)*(x196)))+(npz)+(((r02)*(x198))));
evalcond[3]=((((r00)*(x198)))+(npx)+(((IkReal(0.328000000000000))*(r10)*(x197)))+(((IkReal(-1.00000000000000))*(r20)*(x196))));
evalcond[4]=((((IkReal(0.328000000000000))*(r11)*(x197)))+(((r01)*(x198)))+(npy)+(((IkReal(-1.00000000000000))*(r21)*(x196))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x199=((IkReal(0.328000000000000))*(cj1));
IkReal x200=((IkReal(0.328000000000000))*(sj1));
if( IKabs(((((cj0)*(r00)*(x200)))+(npx)+(((r10)*(sj0)*(x200)))+(((IkReal(-1.00000000000000))*(r20)*(x199))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(r01)*(x200)))+(((IkReal(-1.00000000000000))*(npy)))+(((r21)*(x199)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x200))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x201=IKatan2(((((cj0)*(r00)*(x200)))+(npx)+(((r10)*(sj0)*(x200)))+(((IkReal(-1.00000000000000))*(r20)*(x199)))), ((((IkReal(-1.00000000000000))*(cj0)*(r01)*(x200)))+(((IkReal(-1.00000000000000))*(npy)))+(((r21)*(x199)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x200)))));
j6array[0]=((IkReal(-1.00000000000000))*(x201));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x201))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[1];
IkReal x202=IKcos(j6);
IkReal x203=IKsin(j6);
IkReal x204=((IkReal(0.328000000000000))*(cj1));
IkReal x205=((IkReal(0.328000000000000))*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(r10)*(sj0)*(x203)*(x205)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x203)*(x205)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x202)*(x205)))+(((IkReal(-1.00000000000000))*(npx)*(x203)))+(((r21)*(x202)*(x204)))+(((r20)*(x203)*(x204)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x202)*(x205)))+(((IkReal(-1.00000000000000))*(npy)*(x202))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x206=((IkReal(656.000000000000))*(sj1));
IkReal x207=((IkReal(656.000000000000))*(cj1));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(cj0)*(r01)*(x206)))+(((r21)*(x207)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x206)))+(((IkReal(-2000.00000000000))*(npy))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj0)*(r00)*(x206)))+(((IkReal(2000.00000000000))*(npx)))+(((IkReal(-1.00000000000000))*(r20)*(x207)))+(((r10)*(sj0)*(x206))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(cj0)*(r01)*(x206)))+(((r21)*(x207)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x206)))+(((IkReal(-2000.00000000000))*(npy)))))), ((gconst0)*(((((cj0)*(r00)*(x206)))+(((IkReal(2000.00000000000))*(npx)))+(((IkReal(-1.00000000000000))*(r20)*(x207)))+(((r10)*(sj0)*(x206)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x208=IKcos(j6);
IkReal x209=IKsin(j6);
IkReal x210=((IkReal(0.328000000000000))*(sj1));
IkReal x211=((r10)*(sj0));
IkReal x212=((IkReal(0.328000000000000))*(cj1));
IkReal x213=((r11)*(sj0));
IkReal x214=((IkReal(0.276500000000000))*(sj5));
IkReal x215=((cj0)*(x209));
IkReal x216=((IkReal(1.00000000000000))*(x209));
IkReal x217=((cj0)*(x208));
IkReal x218=((x209)*(x212));
evalcond[0]=((((IkReal(-1.00000000000000))*(r20)*(x212)))+(((x210)*(x211)))+(npx)+(((IkReal(-1.00000000000000))*(x208)*(x214)))+(((cj0)*(r00)*(x210))));
evalcond[1]=((((x210)*(x213)))+(npy)+(((x209)*(x214)))+(((cj0)*(r01)*(x210)))+(((IkReal(-1.00000000000000))*(r21)*(x212))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x208)*(x210)*(x213)))+(((IkReal(-1.00000000000000))*(npx)*(x216)))+(((IkReal(-1.00000000000000))*(r01)*(x210)*(x217)))+(((IkReal(-1.00000000000000))*(r00)*(x210)*(x215)))+(((r21)*(x208)*(x212)))+(((IkReal(-1.00000000000000))*(npy)*(x208)))+(((r20)*(x218)))+(((IkReal(-1.00000000000000))*(x209)*(x210)*(x211))));
evalcond[3]=((((r21)*(x218)))+(((IkReal(-1.00000000000000))*(npy)*(x216)))+(((IkReal(-1.00000000000000))*(r01)*(x210)*(x215)))+(((r00)*(x210)*(x217)))+(((IkReal(-1.00000000000000))*(r20)*(x208)*(x212)))+(((npx)*(x208)))+(((IkReal(-1.00000000000000))*(x214)))+(((x208)*(x210)*(x211)))+(((IkReal(-1.00000000000000))*(x209)*(x210)*(x213))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}
}
}

}

}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x83=((IkReal(1.00000000000000))*(sj5));
IkReal x84=((IkReal(1.00000000000000))*(sj0));
IkReal x85=((cj5)*(cj6));
IkReal x86=((r01)*(sj6));
IkReal x87=((IkReal(1.00000000000000))*(cj5));
IkReal x88=((IkReal(1.00000000000000))*(sj1));
IkReal x89=((r21)*(sj6));
IkReal x90=((r11)*(sj6));
IkReal x91=((cj6)*(sj5));
IkReal x92=((((r10)*(sj6)))+(((cj6)*(r11))));
IkReal x93=((((r00)*(sj6)))+(((cj6)*(r01))));
IkReal x94=((((cj6)*(r21)))+(((r20)*(sj6))));
IkReal x95=((((cj5)*(r12)))+(((IkReal(-1.00000000000000))*(x83)*(x90)))+(((r10)*(x91))));
IkReal x96=((((IkReal(-1.00000000000000))*(x83)*(x86)))+(((r00)*(x91)))+(((cj5)*(r02))));
IkReal x97=((((cj5)*(r22)))+(((IkReal(-1.00000000000000))*(x83)*(x89)))+(((r20)*(x91))));
IkReal x98=((((IkReal(-1.00000000000000))*(r12)*(x83)))+(((r10)*(x85)))+(((IkReal(-1.00000000000000))*(x87)*(x90))));
IkReal x99=((((IkReal(-1.00000000000000))*(r02)*(x83)))+(((r00)*(x85)))+(((IkReal(-1.00000000000000))*(x86)*(x87))));
IkReal x100=((((r20)*(x85)))+(((IkReal(-1.00000000000000))*(x87)*(x89)))+(((IkReal(-1.00000000000000))*(r22)*(x83))));
IkReal x101=((((cj0)*(x93)))+(((sj0)*(x92))));
IkReal x102=((((cj0)*(x96)))+(((sj0)*(x95))));
IkReal x103=((((cj0)*(x99)))+(((sj0)*(x98))));
new_r00=((((sj1)*(x100)))+(((cj1)*(x103))));
new_r01=((((cj0)*(x98)))+(((IkReal(-1.00000000000000))*(x84)*(x99))));
new_r02=((((cj1)*(x100)))+(((IkReal(-1.00000000000000))*(x103)*(x88))));
new_r10=((((cj1)*(x101)))+(((sj1)*(x94))));
new_r11=((((cj0)*(x92)))+(((IkReal(-1.00000000000000))*(x84)*(x93))));
new_r12=((((IkReal(-1.00000000000000))*(x101)*(x88)))+(((cj1)*(x94))));
new_r20=((((cj1)*(x102)))+(((sj1)*(x97))));
new_r21=((((cj0)*(x95)))+(((IkReal(-1.00000000000000))*(x84)*(x96))));
new_r22=((((IkReal(-1.00000000000000))*(x102)*(x88)))+(((cj1)*(x97))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
cj3array[0]=new_r22;
if( cj3array[0] >= -1-IKFAST_SINCOS_THRESH && cj3array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j3valid[0] = j3valid[1] = true;
    j3array[0] = IKacos(cj3array[0]);
    sj3array[0] = IKsin(j3array[0]);
    cj3array[1] = cj3array[0];
    j3array[1] = -j3array[0];
    sj3array[1] = -sj3array[0];
}
else if( isnan(cj3array[0]) )
{
    // probably any value will work
    j3valid[0] = true;
    cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
}
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(-1.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(-1.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(-1.00000000000000))*((new_r12)*(new_r12)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
IkReal x104=((IkReal(-1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=x104;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x104;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x105=IKatan2(new_r12, new_r02);
j4array[0]=((IkReal(-1.00000000000000))*(x105));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x105))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
evalcond[0]=((((new_r02)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(new_r12)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj4)*(new_r11)))+(((new_r01)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)))))+IKsqr(((((cj4)*(new_r11)))+(((new_r01)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)))), ((((cj4)*(new_r11)))+(((new_r01)*(sj4)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x106=IKsin(j2);
IkReal x107=((IkReal(1.00000000000000))*(sj4));
IkReal x108=((IkReal(1.00000000000000))*(IKcos(j2)));
evalcond[0]=((((cj4)*(new_r10)))+(((new_r00)*(sj4)))+(x106));
evalcond[1]=((((cj4)*(new_r11)))+(((new_r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x108))));
evalcond[2]=((((cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(x107)))+(((IkReal(-1.00000000000000))*(x108))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x107)))+(((cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(x106))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x109=IKatan2(new_r12, new_r02);
j4array[0]=((IkReal(-1.00000000000000))*(x109));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x109))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
evalcond[0]=((((new_r02)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(new_r12)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x110=((IkReal(1.00000000000000))*(new_r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(sj4)*(x110))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x110))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(sj4)*(x110)))))+IKsqr(((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x110)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(sj4)*(x110)))), ((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x110)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x111=IKcos(j2);
IkReal x112=IKsin(j2);
IkReal x113=((IkReal(1.00000000000000))*(sj4));
evalcond[0]=((((cj4)*(new_r10)))+(((new_r00)*(sj4)))+(x112));
evalcond[1]=((((cj4)*(new_r11)))+(((new_r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x111))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x113)))+(((cj4)*(new_r00)))+(x111));
evalcond[3]=((((cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(x113)))+(x112));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x114=((gconst4)*(sj3));
if( IKabs(((new_r12)*(x114))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r02)*(x114))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((new_r12)*(x114)), ((IkReal(-1.00000000000000))*(new_r02)*(x114)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x115=IKcos(j4);
IkReal x116=IKsin(j4);
IkReal x117=((IkReal(1.00000000000000))*(sj3));
IkReal x118=((new_r02)*(x115));
IkReal x119=((sj3)*(x115));
IkReal x120=((IkReal(1.00000000000000))*(new_r12)*(x116));
evalcond[0]=((((new_r02)*(x116)))+(((new_r12)*(x115))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x120)))+(x118));
evalcond[2]=((((new_r00)*(x119)))+(((IkReal(-1.00000000000000))*(new_r10)*(x116)*(x117)))+(((cj3)*(new_r20))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x116)*(x117)))+(((new_r01)*(x119)))+(((cj3)*(new_r21))));
evalcond[4]=((((cj3)*(x118)))+(((IkReal(-1.00000000000000))*(cj3)*(x120)))+(((IkReal(-1.00000000000000))*(new_r22)*(x117))));
evalcond[5]=((IkReal(-1.00000000000000))+(((sj3)*(x118)))+(((IkReal(-1.00000000000000))*(new_r12)*(x116)*(x117)))+(((cj3)*(new_r22))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj3);
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x121=((IkReal(-1.00000000000000))+(new_r22));
IkReal x122=((IkReal(1.00000000000000))*(new_r12));
IkReal x123=((new_r02)*(sj4));
IkReal x124=((((cj4)*(new_r02)))+(((IkReal(-1.00000000000000))*(sj4)*(x122))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=x121;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((((cj4)*(new_r12)))+(x123));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj4)*(x122)))+(((IkReal(-1.00000000000000))*(x123))));
evalcond[6]=x124;
evalcond[7]=new_r20;
evalcond[8]=new_r21;
evalcond[9]=x124;
evalcond[10]=x121;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj4)*(new_r11)))+(((new_r01)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)))))+IKsqr(((((cj4)*(new_r11)))+(((new_r01)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)))), ((((cj4)*(new_r11)))+(((new_r01)*(sj4)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x125=IKsin(j2);
IkReal x126=((IkReal(1.00000000000000))*(sj4));
IkReal x127=((IkReal(1.00000000000000))*(IKcos(j2)));
evalcond[0]=((((cj4)*(new_r10)))+(((new_r00)*(sj4)))+(x125));
evalcond[1]=((((cj4)*(new_r11)))+(((new_r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x127))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x126)))+(((cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(x127))));
evalcond[3]=((((cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(x125)))+(((IkReal(-1.00000000000000))*(new_r11)*(x126))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x128=((cj4)*(new_r02));
IkReal x129=((IkReal(1.00000000000000))*(new_r12));
IkReal x130=((new_r02)*(sj4));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((((cj4)*(new_r12)))+(x130));
evalcond[5]=((((IkReal(-1.00000000000000))*(x130)))+(((IkReal(-1.00000000000000))*(cj4)*(x129))));
evalcond[6]=((x128)+(((IkReal(-1.00000000000000))*(sj4)*(x129))));
evalcond[7]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[9]=((((new_r12)*(sj4)))+(((IkReal(-1.00000000000000))*(x128))));
evalcond[10]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x131=((IkReal(1.00000000000000))*(new_r00));
if( IKabs(((((IkReal(-1.00000000000000))*(sj4)*(x131)))+(((IkReal(-1.00000000000000))*(cj4)*(new_r10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x131))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj4)*(x131)))+(((IkReal(-1.00000000000000))*(cj4)*(new_r10)))))+IKsqr(((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x131)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj4)*(x131)))+(((IkReal(-1.00000000000000))*(cj4)*(new_r10)))), ((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x131)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x132=IKcos(j2);
IkReal x133=IKsin(j2);
IkReal x134=((IkReal(1.00000000000000))*(sj4));
evalcond[0]=((((cj4)*(new_r10)))+(((new_r00)*(sj4)))+(x133));
evalcond[1]=((((cj4)*(new_r11)))+(((new_r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x132))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r10)*(x134)))+(((cj4)*(new_r00)))+(x132));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r11)*(x134)))+(((cj4)*(new_r01)))+(x133));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x135=((IkReal(-1.00000000000000))*(gconst5));
if( IKabs(((new_r21)*(x135))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r20)*(x135))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((new_r21)*(x135)), ((new_r20)*(x135)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[8];
IkReal x136=IKsin(j2);
IkReal x137=IKcos(j2);
IkReal x138=((cj4)*(new_r00));
IkReal x139=((cj4)*(new_r01));
IkReal x140=((IkReal(1.00000000000000))*(cj3));
IkReal x141=((IkReal(1.00000000000000))*(sj3));
IkReal x142=((new_r10)*(sj4));
IkReal x143=((new_r11)*(sj4));
IkReal x144=((IkReal(1.00000000000000))*(x137));
evalcond[0]=((((sj3)*(x137)))+(new_r20));
evalcond[1]=((((sj3)*(x136)))+(new_r21));
evalcond[2]=((((cj4)*(new_r10)))+(((new_r00)*(sj4)))+(x136));
evalcond[3]=((((IkReal(-1.00000000000000))*(x144)))+(((cj4)*(new_r11)))+(((new_r01)*(sj4))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x137)*(x140)))+(x138)+(((IkReal(-1.00000000000000))*(x142))));
evalcond[5]=((x139)+(((IkReal(-1.00000000000000))*(x143)))+(((IkReal(-1.00000000000000))*(x136)*(x140))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x144)))+(((IkReal(-1.00000000000000))*(x140)*(x142)))+(((IkReal(-1.00000000000000))*(new_r20)*(x141)))+(((cj3)*(x138))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x140)*(x143)))+(((cj3)*(x139)))+(((IkReal(-1.00000000000000))*(new_r21)*(x141)))+(((IkReal(-1.00000000000000))*(x136))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x145=((IkReal(-1.00000000000000))*(gconst3));
if( IKabs(((new_r21)*(x145))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r20)*(x145))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((new_r21)*(x145)), ((new_r20)*(x145)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
evalcond[0]=((((sj3)*(IKcos(j2))))+(new_r20));
evalcond[1]=((((sj3)*(IKsin(j2))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(((((IkReal(-1.00000000000000))*(new_r00)*(new_r12)))+(((new_r02)*(new_r10)))));
dummyeval[0]=((((IkReal(-1.00000000000000))*(new_r00)*(new_r12)))+(((new_r02)*(new_r10))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x146=((gconst6)*(sj2));
if( IKabs(((new_r12)*(x146))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r02)*(x146))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((new_r12)*(x146)), ((IkReal(-1.00000000000000))*(new_r02)*(x146)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[12];
IkReal x147=IKcos(j4);
IkReal x148=IKsin(j4);
IkReal x149=((IkReal(1.00000000000000))*(cj2));
IkReal x150=((IkReal(1.00000000000000))*(cj3));
IkReal x151=((IkReal(1.00000000000000))*(sj3));
IkReal x152=((IkReal(1.00000000000000))*(x148));
IkReal x153=((new_r02)*(x147));
IkReal x154=((cj3)*(x147));
IkReal x155=((new_r00)*(x147));
IkReal x156=((new_r01)*(x147));
evalcond[0]=((((new_r12)*(x147)))+(((new_r02)*(x148))));
evalcond[1]=((sj2)+(((new_r00)*(x148)))+(((new_r10)*(x147))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x149)))+(((new_r01)*(x148)))+(((new_r11)*(x147))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x151)))+(x153)+(((IkReal(-1.00000000000000))*(new_r12)*(x152))));
evalcond[4]=((x155)+(((IkReal(-1.00000000000000))*(new_r10)*(x152)))+(((IkReal(-1.00000000000000))*(cj3)*(x149))));
evalcond[5]=((x156)+(((IkReal(-1.00000000000000))*(new_r11)*(x152)))+(((IkReal(-1.00000000000000))*(sj2)*(x150))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r10)*(x148)*(x151)))+(((sj3)*(x155)))+(((cj3)*(new_r20))));
evalcond[7]=((((sj3)*(x156)))+(((cj3)*(new_r21)))+(((IkReal(-1.00000000000000))*(new_r11)*(x148)*(x151))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r12)*(x148)*(x150)))+(((cj3)*(x153)))+(((IkReal(-1.00000000000000))*(new_r22)*(x151))));
evalcond[9]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r12)*(x148)*(x151)))+(((sj3)*(x153)))+(((cj3)*(new_r22))));
evalcond[10]=((((IkReal(-1.00000000000000))*(x149)))+(((IkReal(-1.00000000000000))*(new_r20)*(x151)))+(((IkReal(-1.00000000000000))*(new_r10)*(x148)*(x150)))+(((new_r00)*(x154))));
evalcond[11]=((((IkReal(-1.00000000000000))*(sj2)))+(((new_r01)*(x154)))+(((IkReal(-1.00000000000000))*(new_r21)*(x151)))+(((IkReal(-1.00000000000000))*(new_r11)*(x148)*(x150))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "bcb25302dbec7575df10994dd89abd19"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
