/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:20:14.730938
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16;
x0=IKsin(j[2]);
x1=IKsin(j[3]);
x2=IKsin(j[1]);
x3=IKcos(j[3]);
x4=IKcos(j[1]);
x5=IKcos(j[2]);
x6=IKcos(j[0]);
x7=IKsin(j[0]);
x8=((IkReal(0.0450000000000000))*(x0));
x9=((IkReal(0.350000000000000))*(x4));
x10=((IkReal(0.0450000000000000))*(x1));
x11=((IkReal(0.0450000000000000))*(x5));
x12=((x1)*(x5));
x13=((x2)*(x3));
x14=((x3)*(x8));
x15=((x8)+(((IkReal(0.350000000000000))*(x0)*(x1))));
IkReal x17=((x11)*(x4));
x16=((((IkReal(-1.00000000000000))*(x17)*(x3)))+(x17)+(((x2)*(((IkReal(0.550000000000000))+(x10)))))+(((x12)*(x9)))+(((IkReal(0.350000000000000))*(x13))));
eetrans[0]=((IkReal(0.220000000000000))+(((x7)*(((((IkReal(-1.00000000000000))*(x15)))+(x14)))))+(((x16)*(x6))));
eetrans[1]=((IkReal(0.140000000000000))+(((x16)*(x7)))+(((x6)*(((((IkReal(-1.00000000000000))*(x14)))+(x15))))));
eetrans[2]=((IkReal(0.346000000000000))+(((IkReal(-1.00000000000000))*(x11)*(x2)))+(((IkReal(-0.350000000000000))*(x12)*(x2)))+(((x11)*(x13)))+(((x3)*(x9)))+(((x10)*(x4)))+(((IkReal(0.550000000000000))*(x4))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 4; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x33000003; }

class IKSolver {
public:
IkReal j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j0,cj0,sj0,htj0,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij0[2], _nj0;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1;  _ij0[0] = -1; _ij0[1] = -1; _nj0 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

IkReal x18=IKsin(j0);
IkReal x19=IKcos(j0);
new_px=((((py)*(x18)))+(((px)*(x19)))+(((IkReal(-0.220000000000000))*(x19)))+(((IkReal(-0.140000000000000))*(x18))));
new_py=((IkReal(0.346000000000000))+(((IkReal(-1.00000000000000))*(pz))));
IkReal x20=IKcos(j0);
IkReal x21=IKsin(j0);
new_pz=((((py)*(x20)))+(((IkReal(-1.00000000000000))*(px)*(x21)))+(((IkReal(-0.140000000000000))*(x20)))+(((IkReal(0.220000000000000))*(x21))));
px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x22=IKasin(((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp)))));
j3array[0]=((IkReal(-1.36129006610583))+(((IkReal(-1.00000000000000))*(x22))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(1.78030258748397))+(x22));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((py)*(py)*(py)*(py))+((((px)*(px))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x23=(py)*(py);
if( IKabs(((IkReal(-1.10000000000000))*(x23))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.10000000000000))*(px)*(py))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x24=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.10000000000000))*(x23)), ((IkReal(1.10000000000000))*(px)*(py)))));
if( (((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(0.0315000000000000))*(py)*(sj3)))+(((IkReal(-0.00405000000000000))*(cj3)*(py)))+(((IkReal(-0.175950000000000))*(py))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(0.0315000000000000))*(py)*(sj3)))+(((IkReal(-0.00405000000000000))*(cj3)*(py)))+(((IkReal(-0.175950000000000))*(py))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x25=IKasin(((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x23)*((px)*(px))))+(((IkReal(1.21000000000000))*((x23)*(x23))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(0.0315000000000000))*(py)*(sj3)))+(((IkReal(-0.00405000000000000))*(cj3)*(py)))+(((IkReal(-0.175950000000000))*(py)))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x24)))+(((IkReal(-1.00000000000000))*(x25))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x24)))+(x25));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x26=IKsin(j1);
IkReal x27=IKcos(j1);
IkReal x28=((py)*(x27));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((px)*(x26)))+(((IkReal(-1.00000000000000))*(x28))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(0.0315000000000000))*(px)*(sj3)))+(((IkReal(1.10000000000000))*(x26)*((px)*(px))))+(((IkReal(-0.00405000000000000))*(cj3)*(px)))+(((IkReal(-1.10000000000000))*(px)*(x28)))+(((IkReal(-0.175950000000000))*(px))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x29=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x29)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(px)*(x29))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x30=((cj1)*(px));
IkReal x31=((py)*(sj1));
IkReal x32=((IkReal(100.000000000000))*(pp));
IkReal x33=((IkReal(110.000000000000))*(pz));
IkReal x34=((IkReal(110.000000000000))*(cj1)*(sj1));
IkReal x35=((IkReal(110.000000000000))*(px)*(py));
if( IKabs(((gconst1)*(((((px)*(sj1)*(x33)))+(((IkReal(-1.00000000000000))*(pz)*(x32)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x33))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-110.000000000000))*(cj1)*(py)*(x31)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x30)))+(((IkReal(-1.00000000000000))*(x30)*(x32)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x31)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x30)))+(((IkReal(-1.00000000000000))*(x31)*(x32)))+(((IkReal(-18.0000000000000))*(x31)))+(((IkReal(-18.0000000000000))*(x30))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((px)*(sj1)*(x33)))+(((IkReal(-1.00000000000000))*(pz)*(x32)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x33)))))), ((gconst1)*(((((IkReal(-110.000000000000))*(cj1)*(py)*(x31)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x30)))+(((IkReal(-1.00000000000000))*(x30)*(x32)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x31)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x30)))+(((IkReal(-1.00000000000000))*(x31)*(x32)))+(((IkReal(-18.0000000000000))*(x31)))+(((IkReal(-18.0000000000000))*(x30)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x36=IKsin(j2);
IkReal x37=IKcos(j2);
IkReal x38=((cj1)*(px));
IkReal x39=((IkReal(1.00000000000000))*(pz));
IkReal x40=((py)*(sj1));
IkReal x41=((IkReal(0.0900000000000000))*(x37));
IkReal x42=((IkReal(1.00000000000000))*(x37));
evalcond[0]=((((IkReal(-1.00000000000000))*(x37)*(x39)))+(((x36)*(x40)))+(((x36)*(x38))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x36)*(x39)))+(((IkReal(-1.00000000000000))*(x38)*(x42)))+(((IkReal(-1.00000000000000))*(x40)*(x42))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x40)*(x41)))+(((IkReal(0.0900000000000000))*(pz)*(x36)))+(((x38)*(x41)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x43=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x43)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(px)*(x43))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x44=((cj1)*(px));
IkReal x45=((IkReal(100.000000000000))*(pp));
IkReal x46=((py)*(sj1));
IkReal x47=((IkReal(110.000000000000))*(sj1));
IkReal x48=((IkReal(110.000000000000))*(py));
if( IKabs(((gconst2)*(((((px)*(pz)*(x47)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x45)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x48))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((px)*(x46)*(x47)))+(((IkReal(-1.00000000000000))*(cj1)*(x44)*(x48)))+(((IkReal(-1.00000000000000))*(x45)*(x46)))+(((IkReal(-1.00000000000000))*(x44)*(x45)))+(((IkReal(-1.00000000000000))*(cj1)*(x46)*(x48)))+(((px)*(x44)*(x47)))+(((IkReal(-18.0000000000000))*(x46)))+(((IkReal(-18.0000000000000))*(x44))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((px)*(pz)*(x47)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x45)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x48)))))), ((gconst2)*(((((px)*(x46)*(x47)))+(((IkReal(-1.00000000000000))*(cj1)*(x44)*(x48)))+(((IkReal(-1.00000000000000))*(x45)*(x46)))+(((IkReal(-1.00000000000000))*(x44)*(x45)))+(((IkReal(-1.00000000000000))*(cj1)*(x46)*(x48)))+(((px)*(x44)*(x47)))+(((IkReal(-18.0000000000000))*(x46)))+(((IkReal(-18.0000000000000))*(x44)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x49=IKsin(j2);
IkReal x50=IKcos(j2);
IkReal x51=((cj1)*(px));
IkReal x52=((IkReal(1.00000000000000))*(pz));
IkReal x53=((py)*(sj1));
IkReal x54=((IkReal(0.0900000000000000))*(x50));
IkReal x55=((IkReal(1.00000000000000))*(x50));
evalcond[0]=((((x49)*(x51)))+(((x49)*(x53)))+(((IkReal(-1.00000000000000))*(x50)*(x52))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x53)*(x55)))+(((IkReal(-1.00000000000000))*(x51)*(x55)))+(((IkReal(-1.00000000000000))*(x49)*(x52))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x49)))+(((IkReal(-1.00000000000000))*(pp)))+(((x53)*(x54)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x51)*(x54))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((IkReal(-1.00000000000000))*(cj1)*(py)))+(((px)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((IkReal(1052631578.94737))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1052631578.94737))*(pz)))+IKsqr(((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((IkReal(1052631578.94737))*(pz)), ((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x56=IKsin(j2);
IkReal x57=IKcos(j2);
IkReal x58=((cj1)*(px));
IkReal x59=((IkReal(1.00000000000000))*(pz));
IkReal x60=((py)*(sj1));
IkReal x61=((IkReal(0.0900000000000000))*(x57));
IkReal x62=((IkReal(1.00000000000000))*(x57));
evalcond[0]=((((x56)*(x60)))+(((x56)*(x58)))+(((IkReal(-1.00000000000000))*(x57)*(x59))));
evalcond[1]=((((IkReal(9.50000000000000e-10))*(x56)))+(((IkReal(-1.00000000000000))*(x59))));
evalcond[2]=((((IkReal(9.50000000000000e-10))*(x57)))+(((IkReal(-1.00000000000000))*(x58)))+(((IkReal(-1.00000000000000))*(x60))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((IkReal(-1.00000000000000))*(x56)*(x59)))+(((IkReal(-1.00000000000000))*(x58)*(x62)))+(((IkReal(-1.00000000000000))*(x60)*(x62))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x60)*(x61)))+(((x58)*(x61)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x56)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(px)))+(((py)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj1)*(px)))+(((py)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x63=IKsin(j2);
IkReal x64=IKcos(j2);
IkReal x65=((cj1)*(px));
IkReal x66=((py)*(sj1));
IkReal x67=((IkReal(0.0450000000000000))*(cj3));
IkReal x68=((IkReal(1.00000000000000))*(pz));
IkReal x69=((IkReal(0.350000000000000))*(sj3));
IkReal x70=((IkReal(0.0900000000000000))*(x64));
IkReal x71=((IkReal(1.00000000000000))*(x64));
evalcond[0]=((((IkReal(-1.00000000000000))*(x64)*(x68)))+(((x63)*(x65)))+(((x63)*(x66))));
evalcond[1]=((((IkReal(0.0450000000000000))*(x63)))+(((x63)*(x69)))+(((IkReal(-1.00000000000000))*(x68)))+(((IkReal(-1.00000000000000))*(x63)*(x67))));
evalcond[2]=((((IkReal(0.0450000000000000))*(x64)))+(((IkReal(-1.00000000000000))*(x64)*(x67)))+(((x64)*(x69)))+(((IkReal(-1.00000000000000))*(x65)))+(((IkReal(-1.00000000000000))*(x66))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x66)*(x71)))+(x69)+(((IkReal(-1.00000000000000))*(x67)))+(((IkReal(-1.00000000000000))*(x63)*(x68)))+(((IkReal(-1.00000000000000))*(x65)*(x71))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x63)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x66)*(x70)))+(((x65)*(x70))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x72=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x73=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x72)))+(((IkReal(-1.00000000000000))*(x73))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x72)))+(((IkReal(1.00000000000000))*(x73))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x74=IKsin(j1);
IkReal x75=IKcos(j1);
IkReal x76=((IkReal(0.00405000000000000))*(cj3));
IkReal x77=((IkReal(1.00000000000000))*(pp));
IkReal x78=((IkReal(0.0315000000000000))*(sj3));
IkReal x79=((IkReal(1.10000000000000))*(px)*(py));
evalcond[0]=((((py)*(x78)))+(((IkReal(-1.00000000000000))*(py)*(x76)))+(((IkReal(-1.00000000000000))*(py)*(x77)))+(((x74)*(x79)))+(((IkReal(-0.175950000000000))*(py)))+(((IkReal(-1.10000000000000))*(x75)*((py)*(py)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(px)*(x77)))+(((IkReal(-1.00000000000000))*(px)*(x76)))+(((IkReal(1.10000000000000))*(x74)*((px)*(px))))+(((px)*(x78)))+(((IkReal(-0.175950000000000))*(px)))+(((IkReal(-1.00000000000000))*(x75)*(x79))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x80=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x80)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(px)*(x80))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x81=((cj1)*(px));
IkReal x82=((py)*(sj1));
IkReal x83=((IkReal(100.000000000000))*(pp));
IkReal x84=((IkReal(110.000000000000))*(pz));
IkReal x85=((IkReal(110.000000000000))*(cj1)*(sj1));
IkReal x86=((IkReal(110.000000000000))*(px)*(py));
if( IKabs(((gconst1)*(((((IkReal(-18.0000000000000))*(pz)))+(((px)*(sj1)*(x84)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x84)))+(((IkReal(-1.00000000000000))*(pz)*(x83))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x81)*(x83)))+(((IkReal(-18.0000000000000))*(x82)))+(((IkReal(-18.0000000000000))*(x81)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x82)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x81)))+(((IkReal(-1.00000000000000))*(x82)*(x83)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x81)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x82))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-18.0000000000000))*(pz)))+(((px)*(sj1)*(x84)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x84)))+(((IkReal(-1.00000000000000))*(pz)*(x83)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x81)*(x83)))+(((IkReal(-18.0000000000000))*(x82)))+(((IkReal(-18.0000000000000))*(x81)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x82)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x81)))+(((IkReal(-1.00000000000000))*(x82)*(x83)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x81)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x82)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x87=IKsin(j2);
IkReal x88=IKcos(j2);
IkReal x89=((cj1)*(px));
IkReal x90=((IkReal(1.00000000000000))*(pz));
IkReal x91=((py)*(sj1));
IkReal x92=((IkReal(0.0900000000000000))*(x88));
IkReal x93=((IkReal(1.00000000000000))*(x88));
evalcond[0]=((((x87)*(x91)))+(((IkReal(-1.00000000000000))*(x88)*(x90)))+(((x87)*(x89))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x91)*(x93)))+(((IkReal(-1.00000000000000))*(x87)*(x90)))+(((IkReal(-1.00000000000000))*(x89)*(x93))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x89)*(x92)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((IkReal(0.0900000000000000))*(pz)*(x87)))+(((x91)*(x92))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x94=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((px)*(sj1)))+(((IkReal(-1.00000000000000))*(py)*(x94))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(px)*(x94))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x95=((cj1)*(px));
IkReal x96=((IkReal(100.000000000000))*(pp));
IkReal x97=((py)*(sj1));
IkReal x98=((IkReal(110.000000000000))*(sj1));
IkReal x99=((IkReal(110.000000000000))*(py));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x96)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x99)))+(((px)*(pz)*(x98))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x95)*(x96)))+(((px)*(x97)*(x98)))+(((IkReal(-1.00000000000000))*(cj1)*(x97)*(x99)))+(((px)*(x95)*(x98)))+(((IkReal(-1.00000000000000))*(x96)*(x97)))+(((IkReal(-1.00000000000000))*(cj1)*(x95)*(x99)))+(((IkReal(-18.0000000000000))*(x95)))+(((IkReal(-18.0000000000000))*(x97))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x96)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x99)))+(((px)*(pz)*(x98)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x95)*(x96)))+(((px)*(x97)*(x98)))+(((IkReal(-1.00000000000000))*(cj1)*(x97)*(x99)))+(((px)*(x95)*(x98)))+(((IkReal(-1.00000000000000))*(x96)*(x97)))+(((IkReal(-1.00000000000000))*(cj1)*(x95)*(x99)))+(((IkReal(-18.0000000000000))*(x95)))+(((IkReal(-18.0000000000000))*(x97)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x100=IKsin(j2);
IkReal x101=IKcos(j2);
IkReal x102=((cj1)*(px));
IkReal x103=((IkReal(1.00000000000000))*(pz));
IkReal x104=((py)*(sj1));
IkReal x105=((IkReal(0.0900000000000000))*(x101));
IkReal x106=((IkReal(1.00000000000000))*(x101));
evalcond[0]=((((IkReal(-1.00000000000000))*(x101)*(x103)))+(((x100)*(x102)))+(((x100)*(x104))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x104)*(x106)))+(((IkReal(-1.00000000000000))*(x100)*(x103)))+(((IkReal(-1.00000000000000))*(x102)*(x106))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x100)))+(((x102)*(x105)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x104)*(x105))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((IkReal(-1.00000000000000))*(cj1)*(py)))+(((px)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((IkReal(1052631578.94737))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1052631578.94737))*(pz)))+IKsqr(((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((IkReal(1052631578.94737))*(pz)), ((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x107=IKsin(j2);
IkReal x108=IKcos(j2);
IkReal x109=((cj1)*(px));
IkReal x110=((IkReal(1.00000000000000))*(pz));
IkReal x111=((py)*(sj1));
IkReal x112=((IkReal(0.0900000000000000))*(x108));
IkReal x113=((IkReal(1.00000000000000))*(x108));
evalcond[0]=((((x107)*(x109)))+(((IkReal(-1.00000000000000))*(x108)*(x110)))+(((x107)*(x111))));
evalcond[1]=((((IkReal(9.50000000000000e-10))*(x107)))+(((IkReal(-1.00000000000000))*(x110))));
evalcond[2]=((((IkReal(9.50000000000000e-10))*(x108)))+(((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x109))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((IkReal(-1.00000000000000))*(x111)*(x113)))+(((IkReal(-1.00000000000000))*(x107)*(x110)))+(((IkReal(-1.00000000000000))*(x109)*(x113))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x111)*(x112)))+(((IkReal(0.0900000000000000))*(pz)*(x107)))+(((x109)*(x112)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(px)))+(((py)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj1)*(px)))+(((py)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x114=IKsin(j2);
IkReal x115=IKcos(j2);
IkReal x116=((cj1)*(px));
IkReal x117=((py)*(sj1));
IkReal x118=((IkReal(0.0450000000000000))*(cj3));
IkReal x119=((IkReal(1.00000000000000))*(pz));
IkReal x120=((IkReal(0.350000000000000))*(sj3));
IkReal x121=((IkReal(0.0900000000000000))*(x115));
IkReal x122=((IkReal(1.00000000000000))*(x115));
evalcond[0]=((((x114)*(x116)))+(((x114)*(x117)))+(((IkReal(-1.00000000000000))*(x115)*(x119))));
evalcond[1]=((((x114)*(x120)))+(((IkReal(-1.00000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(x114)*(x118)))+(((IkReal(0.0450000000000000))*(x114))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x115)*(x118)))+(((IkReal(0.0450000000000000))*(x115)))+(((x115)*(x120)))+(((IkReal(-1.00000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x116))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x116)*(x122)))+(((IkReal(-1.00000000000000))*(x118)))+(((IkReal(-1.00000000000000))*(x117)*(x122)))+(x120)+(((IkReal(-1.00000000000000))*(x114)*(x119))));
evalcond[4]=((IkReal(-0.180000000000000))+(((x117)*(x121)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x114)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x116)*(x121))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x123=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x124=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x123)))+(((IkReal(-1.00000000000000))*(x124))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x123)))+(((IkReal(1.00000000000000))*(x124))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x125=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x125)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(px)*(x125)))+(((IkReal(-1.00000000000000))*(py)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x126=((cj1)*(px));
IkReal x127=((py)*(sj1));
IkReal x128=((IkReal(100.000000000000))*(pp));
IkReal x129=((IkReal(110.000000000000))*(pz));
IkReal x130=((IkReal(110.000000000000))*(cj1)*(sj1));
IkReal x131=((IkReal(110.000000000000))*(px)*(py));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(pz)*(x128)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x129)))+(((px)*(sj1)*(x129))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-18.0000000000000))*(x126)))+(((IkReal(-18.0000000000000))*(x127)))+(((IkReal(-1.00000000000000))*(x126)*(x128)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x126)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x127)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x127)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x126)))+(((IkReal(-1.00000000000000))*(x127)*(x128))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(pz)*(x128)))+(((IkReal(-18.0000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x129)))+(((px)*(sj1)*(x129)))))), ((gconst1)*(((((IkReal(-18.0000000000000))*(x126)))+(((IkReal(-18.0000000000000))*(x127)))+(((IkReal(-1.00000000000000))*(x126)*(x128)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x126)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x127)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x127)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x126)))+(((IkReal(-1.00000000000000))*(x127)*(x128)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x132=IKsin(j2);
IkReal x133=IKcos(j2);
IkReal x134=((cj1)*(px));
IkReal x135=((IkReal(1.00000000000000))*(pz));
IkReal x136=((py)*(sj1));
IkReal x137=((IkReal(0.0900000000000000))*(x133));
IkReal x138=((IkReal(1.00000000000000))*(x133));
evalcond[0]=((((IkReal(-1.00000000000000))*(x133)*(x135)))+(((x132)*(x136)))+(((x132)*(x134))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x134)*(x138)))+(((IkReal(-1.00000000000000))*(x136)*(x138)))+(((IkReal(-1.00000000000000))*(x132)*(x135))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x132)))+(((x136)*(x137)))+(((IkReal(-1.00000000000000))*(pp)))+(((x134)*(x137)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x139=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x139)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(px)*(x139)))+(((IkReal(-1.00000000000000))*(py)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x140=((cj1)*(px));
IkReal x141=((IkReal(100.000000000000))*(pp));
IkReal x142=((py)*(sj1));
IkReal x143=((IkReal(110.000000000000))*(sj1));
IkReal x144=((IkReal(110.000000000000))*(py));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(pz)*(x144)))+(((IkReal(-1.00000000000000))*(pz)*(x141)))+(((IkReal(-18.0000000000000))*(pz)))+(((px)*(pz)*(x143))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(x142)*(x144)))+(((IkReal(-1.00000000000000))*(x140)*(x141)))+(((px)*(x140)*(x143)))+(((IkReal(-1.00000000000000))*(cj1)*(x140)*(x144)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))+(((px)*(x142)*(x143)))+(((IkReal(-18.0000000000000))*(x140)))+(((IkReal(-18.0000000000000))*(x142))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(pz)*(x144)))+(((IkReal(-1.00000000000000))*(pz)*(x141)))+(((IkReal(-18.0000000000000))*(pz)))+(((px)*(pz)*(x143)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(x142)*(x144)))+(((IkReal(-1.00000000000000))*(x140)*(x141)))+(((px)*(x140)*(x143)))+(((IkReal(-1.00000000000000))*(cj1)*(x140)*(x144)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))+(((px)*(x142)*(x143)))+(((IkReal(-18.0000000000000))*(x140)))+(((IkReal(-18.0000000000000))*(x142)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x145=IKsin(j2);
IkReal x146=IKcos(j2);
IkReal x147=((cj1)*(px));
IkReal x148=((IkReal(1.00000000000000))*(pz));
IkReal x149=((py)*(sj1));
IkReal x150=((IkReal(0.0900000000000000))*(x146));
IkReal x151=((IkReal(1.00000000000000))*(x146));
evalcond[0]=((((x145)*(x149)))+(((x145)*(x147)))+(((IkReal(-1.00000000000000))*(x146)*(x148))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x147)*(x151)))+(((IkReal(-1.00000000000000))*(x145)*(x148)))+(((IkReal(-1.00000000000000))*(x149)*(x151))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.0900000000000000))*(pz)*(x145)))+(((x149)*(x150)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x147)*(x150))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((IkReal(-1.00000000000000))*(cj1)*(py)))+(((px)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((IkReal(1052631578.94737))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1052631578.94737))*(pz)))+IKsqr(((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((IkReal(1052631578.94737))*(pz)), ((((IkReal(1052631578.94737))*(py)*(sj1)))+(((IkReal(1052631578.94737))*(cj1)*(px)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x152=IKsin(j2);
IkReal x153=IKcos(j2);
IkReal x154=((cj1)*(px));
IkReal x155=((IkReal(1.00000000000000))*(pz));
IkReal x156=((py)*(sj1));
IkReal x157=((IkReal(0.0900000000000000))*(x153));
IkReal x158=((IkReal(1.00000000000000))*(x153));
evalcond[0]=((((IkReal(-1.00000000000000))*(x153)*(x155)))+(((x152)*(x154)))+(((x152)*(x156))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x155)))+(((IkReal(9.50000000000000e-10))*(x152))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x156)))+(((IkReal(-1.00000000000000))*(x154)))+(((IkReal(9.50000000000000e-10))*(x153))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((IkReal(-1.00000000000000))*(x154)*(x158)))+(((IkReal(-1.00000000000000))*(x152)*(x155)))+(((IkReal(-1.00000000000000))*(x156)*(x158))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(-1.00000000000000))*(pp)))+(((x154)*(x157)))+(((x156)*(x157)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((IkReal(0.0900000000000000))*(pz)*(x152))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(px)))+(((py)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj1)*(px)))+(((py)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x159=IKsin(j2);
IkReal x160=IKcos(j2);
IkReal x161=((cj1)*(px));
IkReal x162=((py)*(sj1));
IkReal x163=((IkReal(0.0450000000000000))*(cj3));
IkReal x164=((IkReal(1.00000000000000))*(pz));
IkReal x165=((IkReal(0.350000000000000))*(sj3));
IkReal x166=((IkReal(0.0900000000000000))*(x160));
IkReal x167=((IkReal(1.00000000000000))*(x160));
evalcond[0]=((((IkReal(-1.00000000000000))*(x160)*(x164)))+(((x159)*(x161)))+(((x159)*(x162))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x164)))+(((IkReal(-1.00000000000000))*(x159)*(x163)))+(((IkReal(0.0450000000000000))*(x159)))+(((x159)*(x165))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x160)*(x163)))+(((IkReal(-1.00000000000000))*(x161)))+(((IkReal(-1.00000000000000))*(x162)))+(((IkReal(0.0450000000000000))*(x160)))+(((x160)*(x165))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x163)))+(((IkReal(-1.00000000000000))*(x159)*(x164)))+(((IkReal(-1.00000000000000))*(x161)*(x167)))+(x165)+(((IkReal(-1.00000000000000))*(x162)*(x167))));
evalcond[4]=((IkReal(-0.180000000000000))+(((x161)*(x166)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x162)*(x166)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((IkReal(0.0900000000000000))*(pz)*(x159))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "4382fadc0284794e13f46b1dd48b52ef"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
