/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 08:57:55.213823
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42;
x0=IKsin(j[0]);
x1=IKcos(j[0]);
x2=IKcos(j[3]);
x3=IKsin(j[3]);
x4=IKcos(j[1]);
x5=IKcos(j[2]);
x6=IKsin(j[1]);
x7=IKsin(j[2]);
x8=IKsin(j[4]);
x9=IKcos(j[4]);
x10=IKsin(j[5]);
x11=IKcos(j[5]);
x12=((IkReal(0.331000000000000))*(x5));
x13=((IkReal(1.00000000000000))*(x9));
x14=((IkReal(1.00000000000000))*(x7));
x15=((IkReal(1.00000000000000))*(x0));
x16=((IkReal(1.00000000000000))*(x8));
x17=((IkReal(1.00000000000000))*(x5));
x18=((x1)*(x3));
x19=((x0)*(x4));
x20=((x1)*(x4));
x21=((x5)*(x6));
x22=((x1)*(x6));
x23=((x4)*(x7));
x24=((x17)*(x4));
x25=((IkReal(0.331000000000000))*(x6)*(x7));
x26=((x0)*(x14)*(x6));
x27=((x17)*(x20));
x28=((x21)+(x23));
x29=((((IkReal(-1.00000000000000))*(x24)))+(((x6)*(x7))));
x30=((((IkReal(-1.00000000000000))*(x14)*(x6)))+(x24));
x31=((x2)*(x29));
x32=((((x19)*(x5)))+(((IkReal(-1.00000000000000))*(x26))));
x33=((((x22)*(x7)))+(((IkReal(-1.00000000000000))*(x27))));
x34=((x3)*(x30)*(x9));
x35=((((IkReal(-1.00000000000000))*(x15)*(x21)))+(((IkReal(-1.00000000000000))*(x14)*(x19))));
x36=((((x14)*(x20)))+(((x17)*(x22))));
x37=((x32)*(x8));
x38=((x33)*(x8));
x39=((x2)*(x35));
x40=((((x1)*(x2)))+(((x3)*(x35))));
x41=((((x0)*(x2)))+(((x3)*(x36))));
x42=((((x2)*(x36)))+(((IkReal(-1.00000000000000))*(x15)*(x3))));
eerot[0]=((((x11)*(((x37)+(((x40)*(x9)))))))+(((x10)*(((((IkReal(-1.00000000000000))*(x18)))+(x39))))));
eerot[1]=((((x9)*(((x26)+(((IkReal(-1.00000000000000))*(x15)*(x4)*(x5)))))))+(((x40)*(x8))));
eerot[2]=((((x11)*(((((IkReal(-1.00000000000000))*(x18)))+(x39)))))+(((x10)*(((((IkReal(-1.00000000000000))*(x16)*(x32)))+(((IkReal(-1.00000000000000))*(x13)*(x40))))))));
eetrans[0]=((IkReal(0.0454160000000000))+(((IkReal(-0.100150000000000))*(x1)))+(((x0)*(x25)))+(((IkReal(-1.00000000000000))*(x12)*(x19)))+(((IkReal(-0.405000000000000))*(x19))));
eerot[3]=((((x10)*(x31)))+(((x11)*(((((IkReal(-1.00000000000000))*(x16)*(x28)))+(((IkReal(-1.00000000000000))*(x13)*(x3)*(x30))))))));
eerot[4]=((((x29)*(x3)*(x8)))+(((x28)*(x9))));
eerot[5]=((((x11)*(x31)))+(((x10)*(((x34)+(((x28)*(x8))))))));
eetrans[1]=((IkReal(0.302408000000000))+(((IkReal(0.331000000000000))*(x23)))+(((IkReal(0.405000000000000))*(x6)))+(((x12)*(x6))));
eerot[6]=((((x11)*(((((x41)*(x9)))+(x38)))))+(((x10)*(x42))));
eerot[7]=((((x41)*(x8)))+(((x9)*(((((IkReal(-1.00000000000000))*(x14)*(x22)))+(x27))))));
eerot[8]=((((x11)*(x42)))+(((x10)*(((((IkReal(-1.00000000000000))*(x16)*(x33)))+(((IkReal(-1.00000000000000))*(x13)*(x41))))))));
eetrans[2]=((IkReal(0.467052000000000))+(((IkReal(-0.100150000000000))*(x0)))+(((IkReal(-0.331000000000000))*(x22)*(x7)))+(((IkReal(0.405000000000000))*(x20)))+(((x12)*(x20))));
}

IKFAST_API int GetNumFreeParameters() { return 0; }
IKFAST_API int* GetFreeParameters() { return NULL; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r02;
new_r02=((IkReal(-1.00000000000000))*(r01));
new_px=((IkReal(-0.0454160000000000))+(px));
new_r10=r20;
new_r11=r22;
new_r12=((IkReal(-1.00000000000000))*(r21));
new_py=((IkReal(-0.467052000000000))+(pz));
new_r20=((IkReal(-1.00000000000000))*(r10));
new_r21=((IkReal(-1.00000000000000))*(r12));
new_r22=r11;
new_pz=((IkReal(0.302408000000000))+(((IkReal(-1.00000000000000))*(py))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
cj2array[0]=((IkReal(-1.05783455484689))+(((IkReal(3.72981239043676))*(pp))));
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.49711119832787))+((sj2)*(sj2))+((cj2)*(cj2))+(((IkReal(2.44712990936556))*(cj2))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x43=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2))));
if( IKabs(((IkReal(0.331000000000000))*(sj2))) < IKFAST_ATAN2_MAGTHRESH && IKabs(x43) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x44=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.331000000000000))*(sj2)), x43)));
if( ((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x45=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x43)*(x43))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x45)))+(((IkReal(-1.00000000000000))*(x44))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x44)))+(x45));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x46=((IkReal(20000.0000000000))*(sj1));
gconst0=IKsign(((((x46)*((px)*(px))))+(((x46)*((py)*(py))))));
dummyeval[0]=((((sj1)*((py)*(py))))+(((sj1)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x47=((pz)+(((IkReal(0.331000000000000))*(sj2))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=x47;
evalcond[3]=x47;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-20000.0000000000))*((py)*(py))))+(((IkReal(-20000.0000000000))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x48=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x48)))+(((IkReal(2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x48)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x48)))+(((IkReal(2003.00000000000))*(py)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x48)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x49=IKsin(j0);
IkReal x50=IKcos(j0);
IkReal x51=((px)*(x49));
IkReal x52=((py)*(x50));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x49)))+(((px)*(x50))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2)))+(x51)+(((IkReal(-1.00000000000000))*(x52))));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x51)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.810000000000000))*(x52))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x114=((IkReal(0.331000000000000))*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x114)))+(pz));
evalcond[3]=((x114)+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(20000.0000000000))*((px)*(px))))+(((IkReal(20000.0000000000))*((py)*(py))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x115=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x115)))+(((IkReal(-2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-8100.00000000000))*(py)))+(((IkReal(-2003.00000000000))*(px)))+(((IkReal(-1.00000000000000))*(py)*(x115))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x115)))+(((IkReal(-2003.00000000000))*(py)))))), ((gconst2)*(((((IkReal(-8100.00000000000))*(py)))+(((IkReal(-2003.00000000000))*(px)))+(((IkReal(-1.00000000000000))*(py)*(x115)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x116=IKcos(j0);
IkReal x117=IKsin(j0);
IkReal x118=((px)*(x117));
IkReal x119=((py)*(x116));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x117)))+(((px)*(x116))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2)))+(x119)+(((IkReal(-1.00000000000000))*(x118))));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.810000000000000))*(x118))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x120=((IkReal(6620.00000000000))*(sj2));
IkReal x121=((IkReal(2003.00000000000))*(sj1));
IkReal x122=((IkReal(20000.0000000000))*(cj1)*(pz));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x121)))+(((px)*(x120)))+(((px)*(x122))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x120)))+(((IkReal(-1.00000000000000))*(py)*(x122)))+(((IkReal(-1.00000000000000))*(px)*(x121))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x121)))+(((px)*(x120)))+(((px)*(x122)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x120)))+(((IkReal(-1.00000000000000))*(py)*(x122)))+(((IkReal(-1.00000000000000))*(px)*(x121)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x123=IKcos(j0);
IkReal x124=IKsin(j0);
IkReal x125=((IkReal(0.810000000000000))*(cj1));
IkReal x126=((IkReal(0.331000000000000))*(sj2));
IkReal x127=((pz)*(sj1));
IkReal x128=((IkReal(0.331000000000000))*(cj2));
IkReal x129=((px)*(x124));
IkReal x130=((py)*(x123));
IkReal x131=((IkReal(1.00000000000000))*(x130));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x124)))+(((px)*(x123))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj1)*(x129)))+(x126)+(((cj1)*(pz)))+(((sj1)*(x130))));
evalcond[2]=((IkReal(0.405000000000000))+(((cj1)*(x129)))+(x127)+(x128)+(((IkReal(-1.00000000000000))*(cj1)*(x131))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x127)))+(((IkReal(-1.00000000000000))*(pp)))+(((x125)*(x130)))+(((IkReal(-1.00000000000000))*(x125)*(x129))));
evalcond[4]=((((cj1)*(x128)))+(((IkReal(-1.00000000000000))*(sj1)*(x126)))+(((IkReal(0.405000000000000))*(cj1)))+(x129)+(((IkReal(-1.00000000000000))*(x131))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x132=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x133=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x133))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x133)+(((IkReal(-1.00000000000000))*(x132))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x134=((cj0)*(py));
IkReal x135=((px)*(sj0));
IkReal x136=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-331.000000000000))*(pz)*(sj2)))+(((IkReal(-1.00000000000000))*(x134)*(x136)))+(((x135)*(x136)))+(((IkReal(405.000000000000))*(x135)))+(((IkReal(-405.000000000000))*(x134)))));
IkReal x137=((px)*(sj0));
IkReal x138=((cj0)*(py));
dummyeval[0]=((((cj2)*(x137)))+(((IkReal(-1.22356495468278))*(x138)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(1.22356495468278))*(x137)))+(((IkReal(-1.00000000000000))*(cj2)*(x138))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x139=((IkReal(331.000000000000))*(sj2));
IkReal x140=((cj0)*(py));
IkReal x141=((px)*(sj0));
IkReal x142=((IkReal(1000.00000000000))*(pz));
IkReal x143=((cj2)*(pz));
IkReal x144=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141)))))):(IkReal)1.0e30))*(((((x140)*(x142)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x141)*(x142))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141)))))):(IkReal)1.0e30))*(((((x140)*(x142)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141)))))):(IkReal)1.0e30))*(((((x140)*(x142)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x145=IKsin(j1);
IkReal x146=IKcos(j1);
IkReal x147=((IkReal(0.331000000000000))*(sj2));
IkReal x148=((cj0)*(py));
IkReal x149=((px)*(sj0));
IkReal x150=((IkReal(0.331000000000000))*(cj2));
IkReal x151=((IkReal(0.810000000000000))*(x146));
IkReal x152=((pz)*(x145));
evalcond[0]=((((IkReal(0.405000000000000))*(x145)))+(((x145)*(x150)))+(pz)+(((x146)*(x147))));
evalcond[1]=((((pz)*(x146)))+(((x145)*(x148)))+(x147)+(((IkReal(-1.00000000000000))*(x145)*(x149))));
evalcond[2]=((IkReal(0.405000000000000))+(x152)+(x150)+(((IkReal(-1.00000000000000))*(x146)*(x148)))+(((x146)*(x149))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x152)))+(((IkReal(-1.00000000000000))*(pp)))+(((x148)*(x151)))+(((IkReal(-1.00000000000000))*(x149)*(x151))));
evalcond[4]=((((IkReal(0.405000000000000))*(x146)))+(x149)+(((x146)*(x150)))+(((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(x145)*(x147))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x153=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((cj0)*(py)*(x153)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x153)))+(((IkReal(109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))+(((pz)*(x153))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((cj0)*(py)*(x153)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x153)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))+(((pz)*(x153)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x154=IKsin(j1);
IkReal x155=IKcos(j1);
IkReal x156=((IkReal(0.331000000000000))*(sj2));
IkReal x157=((cj0)*(py));
IkReal x158=((px)*(sj0));
IkReal x159=((IkReal(0.331000000000000))*(cj2));
IkReal x160=((IkReal(0.810000000000000))*(x155));
IkReal x161=((pz)*(x154));
evalcond[0]=((((IkReal(0.405000000000000))*(x154)))+(((x155)*(x156)))+(((x154)*(x159)))+(pz));
evalcond[1]=((x156)+(((x154)*(x157)))+(((IkReal(-1.00000000000000))*(x154)*(x158)))+(((pz)*(x155))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x155)*(x157)))+(x161)+(x159)+(((x155)*(x158))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((x157)*(x160)))+(((IkReal(-1.00000000000000))*(x158)*(x160)))+(((IkReal(-0.810000000000000))*(x161)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x154)*(x156)))+(((IkReal(0.405000000000000))*(x155)))+(x158)+(((x155)*(x159)))+(((IkReal(-1.00000000000000))*(x157))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x162=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x163=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x162)))+(((IkReal(-1.00000000000000))*(x163))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x162)))+(x163));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x164=((cj0)*(py));
IkReal x165=((px)*(sj0));
IkReal x166=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-331.000000000000))*(pz)*(sj2)))+(((IkReal(-405.000000000000))*(x164)))+(((IkReal(405.000000000000))*(x165)))+(((x165)*(x166)))+(((IkReal(-1.00000000000000))*(x164)*(x166)))));
IkReal x167=((px)*(sj0));
IkReal x168=((cj0)*(py));
dummyeval[0]=((((IkReal(-1.22356495468278))*(x168)))+(((IkReal(1.22356495468278))*(x167)))+(((cj2)*(x167)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x168))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x169=((IkReal(331.000000000000))*(sj2));
IkReal x170=((cj0)*(py));
IkReal x171=((px)*(sj0));
IkReal x172=((IkReal(1000.00000000000))*(pz));
IkReal x173=((cj2)*(pz));
IkReal x174=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(405000.000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((x171)*(x174)))+(((IkReal(331000.000000000))*(x173))))) != 0)?((IkReal)1/(((((IkReal(405000.000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((x171)*(x174)))+(((IkReal(331000.000000000))*(x173)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((x169)*(x171)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(331.000000000000))*(x173))))) != 0)?((IkReal)1/(((((x169)*(x171)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(331.000000000000))*(x173)))))):(IkReal)1.0e30))*(((((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x171)*(x172)))+(((x170)*(x172))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(405000.000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((x171)*(x174)))+(((IkReal(331000.000000000))*(x173))))) != 0)?((IkReal)1/(((((IkReal(405000.000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((x171)*(x174)))+(((IkReal(331000.000000000))*(x173)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((x169)*(x171)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(331.000000000000))*(x173))))) != 0)?((IkReal)1/(((((x169)*(x171)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(331.000000000000))*(x173)))))):(IkReal)1.0e30))*(((((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x171)*(x172)))+(((x170)*(x172)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(405000.000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((x171)*(x174)))+(((IkReal(331000.000000000))*(x173))))) != 0)?((IkReal)1/(((((IkReal(405000.000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((x171)*(x174)))+(((IkReal(331000.000000000))*(x173)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((x169)*(x171)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(331.000000000000))*(x173))))) != 0)?((IkReal)1/(((((x169)*(x171)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(331.000000000000))*(x173)))))):(IkReal)1.0e30))*(((((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x171)*(x172)))+(((x170)*(x172)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x175=IKsin(j1);
IkReal x176=IKcos(j1);
IkReal x177=((IkReal(0.331000000000000))*(sj2));
IkReal x178=((cj0)*(py));
IkReal x179=((px)*(sj0));
IkReal x180=((IkReal(0.331000000000000))*(cj2));
IkReal x181=((IkReal(0.810000000000000))*(x176));
IkReal x182=((pz)*(x175));
evalcond[0]=((((IkReal(0.405000000000000))*(x175)))+(((x176)*(x177)))+(((x175)*(x180)))+(pz));
evalcond[1]=((((pz)*(x176)))+(((x175)*(x178)))+(x177)+(((IkReal(-1.00000000000000))*(x175)*(x179))));
evalcond[2]=((IkReal(0.405000000000000))+(((x176)*(x179)))+(x180)+(x182)+(((IkReal(-1.00000000000000))*(x176)*(x178))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((x178)*(x181)))+(((IkReal(-0.810000000000000))*(x182)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x179)*(x181))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x175)*(x177)))+(((x176)*(x180)))+(((IkReal(0.405000000000000))*(x176)))+(((IkReal(-1.00000000000000))*(x178)))+(x179));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x183=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((cj0)*(py)*(x183)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x183)))+(((IkReal(109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x183)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((cj0)*(py)*(x183)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x183)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x183)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x184=IKsin(j1);
IkReal x185=IKcos(j1);
IkReal x186=((IkReal(0.331000000000000))*(sj2));
IkReal x187=((cj0)*(py));
IkReal x188=((px)*(sj0));
IkReal x189=((IkReal(0.331000000000000))*(cj2));
IkReal x190=((IkReal(0.810000000000000))*(x185));
IkReal x191=((pz)*(x184));
evalcond[0]=((((IkReal(0.405000000000000))*(x184)))+(((x185)*(x186)))+(((x184)*(x189)))+(pz));
evalcond[1]=((((pz)*(x185)))+(((IkReal(-1.00000000000000))*(x184)*(x188)))+(x186)+(((x184)*(x187))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x185)*(x187)))+(((x185)*(x188)))+(x191)+(x189));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x188)*(x190)))+(((x187)*(x190)))+(((IkReal(-0.810000000000000))*(x191)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[4]=((((IkReal(0.405000000000000))*(x185)))+(((x185)*(x189)))+(((IkReal(-1.00000000000000))*(x187)))+(x188)+(((IkReal(-1.00000000000000))*(x184)*(x186))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x53=((cj0)*(r10));
IkReal x54=((IkReal(1.00000000000000))*(sj0));
IkReal x55=((cj0)*(r11));
IkReal x56=((IkReal(1.00000000000000))*(cj2));
IkReal x57=((cj1)*(sj2));
IkReal x58=((cj0)*(r12));
IkReal x59=((((sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x56))));
IkReal x60=((((cj2)*(sj1)))+(x57));
IkReal x61=((cj0)*(x59));
IkReal x62=((((IkReal(-1.00000000000000))*(sj1)*(x56)))+(((IkReal(-1.00000000000000))*(x57))));
new_r00=((((IkReal(-1.00000000000000))*(r00)*(x54)*(x62)))+(((x53)*(x62)))+(((r20)*(x59))));
new_r01=((((r21)*(x59)))+(((IkReal(-1.00000000000000))*(r01)*(x54)*(x62)))+(((x55)*(x62))));
new_r02=((((r22)*(x59)))+(((x58)*(x62)))+(((IkReal(-1.00000000000000))*(r02)*(x54)*(x62))));
new_r10=((((r10)*(sj0)))+(((cj0)*(r00))));
new_r11=((((r11)*(sj0)))+(((cj0)*(r01))));
new_r12=((((r12)*(sj0)))+(((cj0)*(r02))));
new_r20=((((IkReal(-1.00000000000000))*(r00)*(x54)*(x59)))+(((r20)*(x60)))+(((x53)*(x59))));
new_r21=((((r21)*(x60)))+(((IkReal(-1.00000000000000))*(r01)*(x54)*(x59)))+(((x55)*(x59))));
new_r22=((((r22)*(x60)))+(((x58)*(x59)))+(((IkReal(-1.00000000000000))*(r02)*(x54)*(x59))));
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
cj4array[0]=new_r22;
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
IkReal x63=((IkReal(-1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=x63;
evalcond[2]=new_r21;
evalcond[3]=new_r20;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x63;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x64=IKatan2(new_r02, new_r12);
j3array[0]=((IkReal(-1.00000000000000))*(x64));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x64))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j3))))+(((new_r12)*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x65=((IkReal(1.00000000000000))*(cj3));
IkReal x66=((IkReal(1.00000000000000))*(sj3));
if( IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(x66)))+(((IkReal(-1.00000000000000))*(new_r00)*(x65))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(x66)))+(((IkReal(-1.00000000000000))*(new_r01)*(x65))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(x66)))+(((IkReal(-1.00000000000000))*(new_r00)*(x65)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(x66)))+(((IkReal(-1.00000000000000))*(new_r01)*(x65)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r10)*(x66)))+(((IkReal(-1.00000000000000))*(new_r00)*(x65)))), ((((IkReal(-1.00000000000000))*(new_r11)*(x66)))+(((IkReal(-1.00000000000000))*(new_r01)*(x65)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x67=IKsin(j5);
IkReal x68=IKcos(j5);
IkReal x69=((IkReal(1.00000000000000))*(sj3));
evalcond[0]=((((new_r10)*(sj3)))+(x67)+(((cj3)*(new_r00))));
evalcond[1]=((((new_r11)*(sj3)))+(x68)+(((cj3)*(new_r01))));
evalcond[2]=((x67)+(((cj3)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x69))));
evalcond[3]=((((cj3)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x69)))+(((IkReal(-1.00000000000000))*(x68))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r21;
evalcond[3]=new_r20;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x70=IKatan2(new_r02, new_r12);
j3array[0]=((IkReal(-1.00000000000000))*(x70));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x70))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j3))))+(((new_r12)*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x71=((IkReal(1.00000000000000))*(cj3));
IkReal x72=((IkReal(1.00000000000000))*(sj3));
if( IKabs(((((IkReal(-1.00000000000000))*(new_r00)*(x71)))+(((IkReal(-1.00000000000000))*(new_r10)*(x72))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r01)*(x71)))+(((IkReal(-1.00000000000000))*(new_r11)*(x72))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r00)*(x71)))+(((IkReal(-1.00000000000000))*(new_r10)*(x72)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r01)*(x71)))+(((IkReal(-1.00000000000000))*(new_r11)*(x72)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r00)*(x71)))+(((IkReal(-1.00000000000000))*(new_r10)*(x72)))), ((((IkReal(-1.00000000000000))*(new_r01)*(x71)))+(((IkReal(-1.00000000000000))*(new_r11)*(x72)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x73=IKsin(j5);
IkReal x74=IKcos(j5);
IkReal x75=((IkReal(1.00000000000000))*(sj3));
evalcond[0]=((((new_r10)*(sj3)))+(x73)+(((cj3)*(new_r00))));
evalcond[1]=((((new_r11)*(sj3)))+(x74)+(((cj3)*(new_r01))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r01)*(x75)))+(((IkReal(-1.00000000000000))*(x73)))+(((cj3)*(new_r11))));
evalcond[3]=((x74)+(((IkReal(-1.00000000000000))*(new_r00)*(x75)))+(((cj3)*(new_r10))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x76=((gconst4)*(sj4));
if( IKabs(((new_r02)*(x76))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r12)*(x76))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((new_r02)*(x76)), ((IkReal(-1.00000000000000))*(new_r12)*(x76)));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[6];
IkReal x77=IKcos(j3);
IkReal x78=IKsin(j3);
IkReal x79=((IkReal(1.00000000000000))*(sj4));
IkReal x80=((new_r12)*(x77));
IkReal x81=((sj4)*(x78));
IkReal x82=((IkReal(1.00000000000000))*(new_r02)*(x78));
evalcond[0]=((((new_r02)*(x77)))+(((new_r12)*(x78))));
evalcond[1]=((sj4)+(x80)+(((IkReal(-1.00000000000000))*(x82))));
evalcond[2]=((((new_r22)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x82)))+(((cj4)*(x80))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r10)*(x77)*(x79)))+(((cj4)*(new_r20)))+(((new_r00)*(x81))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r11)*(x77)*(x79)))+(((cj4)*(new_r21)))+(((new_r01)*(x81))));
evalcond[5]=((IkReal(-1.00000000000000))+(((cj4)*(new_r22)))+(((IkReal(-1.00000000000000))*(x79)*(x80)))+(((new_r02)*(x81))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x83=((IkReal(-1.00000000000000))+(new_r22));
IkReal x84=((((new_r12)*(sj3)))+(((cj3)*(new_r02))));
IkReal x85=((((IkReal(-1.00000000000000))*(new_r02)*(sj3)))+(((cj3)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=x83;
evalcond[2]=new_r21;
evalcond[3]=new_r20;
evalcond[4]=x84;
evalcond[5]=x84;
evalcond[6]=x85;
evalcond[7]=x85;
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x83;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x86=((IkReal(1.00000000000000))*(cj3));
IkReal x87=((IkReal(1.00000000000000))*(sj3));
if( IKabs(((((IkReal(-1.00000000000000))*(new_r00)*(x86)))+(((IkReal(-1.00000000000000))*(new_r10)*(x87))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(x87)))+(((IkReal(-1.00000000000000))*(new_r01)*(x86))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r00)*(x86)))+(((IkReal(-1.00000000000000))*(new_r10)*(x87)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(x87)))+(((IkReal(-1.00000000000000))*(new_r01)*(x86)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r00)*(x86)))+(((IkReal(-1.00000000000000))*(new_r10)*(x87)))), ((((IkReal(-1.00000000000000))*(new_r11)*(x87)))+(((IkReal(-1.00000000000000))*(new_r01)*(x86)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x88=IKsin(j5);
IkReal x89=IKcos(j5);
IkReal x90=((IkReal(1.00000000000000))*(sj3));
evalcond[0]=((((new_r10)*(sj3)))+(x88)+(((cj3)*(new_r00))));
evalcond[1]=((((new_r11)*(sj3)))+(x89)+(((cj3)*(new_r01))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r01)*(x90)))+(x88)+(((cj3)*(new_r11))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x89)))+(((cj3)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x90))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x91=((cj3)*(new_r12));
IkReal x92=((new_r02)*(sj3));
IkReal x93=((((new_r12)*(sj3)))+(((cj3)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(1.00000000000000))+(new_r22));
evalcond[2]=new_r21;
evalcond[3]=new_r20;
evalcond[4]=x93;
evalcond[5]=x93;
evalcond[6]=((x91)+(((IkReal(-1.00000000000000))*(x92))));
evalcond[7]=((x92)+(((IkReal(-1.00000000000000))*(x91))));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x94=((IkReal(1.00000000000000))*(cj3));
IkReal x95=((IkReal(1.00000000000000))*(sj3));
if( IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(x95)))+(((IkReal(-1.00000000000000))*(new_r00)*(x94))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(x95)))+(((IkReal(-1.00000000000000))*(new_r01)*(x94))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(x95)))+(((IkReal(-1.00000000000000))*(new_r00)*(x94)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(x95)))+(((IkReal(-1.00000000000000))*(new_r01)*(x94)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r10)*(x95)))+(((IkReal(-1.00000000000000))*(new_r00)*(x94)))), ((((IkReal(-1.00000000000000))*(new_r11)*(x95)))+(((IkReal(-1.00000000000000))*(new_r01)*(x94)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x96=IKsin(j5);
IkReal x97=IKcos(j5);
IkReal x98=((IkReal(1.00000000000000))*(sj3));
evalcond[0]=((((new_r10)*(sj3)))+(x96)+(((cj3)*(new_r00))));
evalcond[1]=((((new_r11)*(sj3)))+(x97)+(((cj3)*(new_r01))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r01)*(x98)))+(((cj3)*(new_r11)))+(((IkReal(-1.00000000000000))*(x96))));
evalcond[3]=((x97)+(((cj3)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x98))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(-1.00000000000000))*(gconst6)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst6)*(new_r21)), ((gconst6)*(new_r20)));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[8];
IkReal x99=IKsin(j5);
IkReal x100=IKcos(j5);
IkReal x101=((IkReal(1.00000000000000))*(sj3));
IkReal x102=((cj3)*(cj4));
IkReal x103=((IkReal(1.00000000000000))*(x100));
evalcond[0]=((((sj4)*(x99)))+(new_r21));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)*(x103)))+(new_r20));
evalcond[2]=((((new_r10)*(sj3)))+(x99)+(((cj3)*(new_r00))));
evalcond[3]=((((new_r11)*(sj3)))+(x100)+(((cj3)*(new_r01))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r01)*(x101)))+(((cj4)*(x99)))+(((cj3)*(new_r11))));
evalcond[5]=((((IkReal(-1.00000000000000))*(new_r00)*(x101)))+(((IkReal(-1.00000000000000))*(cj4)*(x103)))+(((cj3)*(new_r10))));
evalcond[6]=((((IkReal(-1.00000000000000))*(cj4)*(new_r01)*(x101)))+(x99)+(((new_r21)*(sj4)))+(((new_r11)*(x102))));
evalcond[7]=((((new_r10)*(x102)))+(((new_r20)*(sj4)))+(((IkReal(-1.00000000000000))*(cj4)*(new_r00)*(x101)))+(((IkReal(-1.00000000000000))*(x103))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(-1.00000000000000))*(gconst5)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst5)*(new_r21)), ((gconst5)*(new_r20)));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((((sj4)*(IKsin(j5))))+(new_r21));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)*(IKcos(j5))))+(new_r20));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign(((((new_r00)*(new_r12)))+(((IkReal(-1.00000000000000))*(new_r02)*(new_r10)))));
dummyeval[0]=((((new_r00)*(new_r12)))+(((IkReal(-1.00000000000000))*(new_r02)*(new_r10))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x104=((gconst7)*(sj5));
if( IKabs(((new_r02)*(x104))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(new_r12)*(x104))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((new_r02)*(x104)), ((IkReal(-1.00000000000000))*(new_r12)*(x104)));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[12];
IkReal x105=IKsin(j3);
IkReal x106=IKcos(j3);
IkReal x107=((IkReal(1.00000000000000))*(cj5));
IkReal x108=((new_r11)*(x106));
IkReal x109=((cj4)*(x106));
IkReal x110=((sj4)*(x105));
IkReal x111=((IkReal(1.00000000000000))*(x105));
IkReal x112=((IkReal(1.00000000000000))*(sj4)*(x106));
IkReal x113=((cj4)*(x111));
evalcond[0]=((((new_r02)*(x106)))+(((new_r12)*(x105))));
evalcond[1]=((((new_r10)*(x105)))+(sj5)+(((new_r00)*(x106))));
evalcond[2]=((cj5)+(((new_r01)*(x106)))+(((new_r11)*(x105))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r02)*(x111)))+(sj4)+(((new_r12)*(x106))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r01)*(x111)))+(x108)+(((cj4)*(sj5))));
evalcond[5]=((((new_r10)*(x106)))+(((IkReal(-1.00000000000000))*(new_r00)*(x111)))+(((IkReal(-1.00000000000000))*(cj4)*(x107))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r02)*(x113)))+(((new_r22)*(sj4)))+(((new_r12)*(x109))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r10)*(x112)))+(((cj4)*(new_r20)))+(((new_r00)*(x110))));
evalcond[8]=((((IkReal(-1.00000000000000))*(sj4)*(x108)))+(((cj4)*(new_r21)))+(((new_r01)*(x110))));
evalcond[9]=((IkReal(-1.00000000000000))+(((cj4)*(new_r22)))+(((new_r02)*(x110)))+(((IkReal(-1.00000000000000))*(new_r12)*(x112))));
evalcond[10]=((sj5)+(((IkReal(-1.00000000000000))*(new_r01)*(x113)))+(((cj4)*(x108)))+(((new_r21)*(sj4))));
evalcond[11]=((((new_r10)*(x109)))+(((IkReal(-1.00000000000000))*(new_r00)*(x113)))+(((new_r20)*(sj4)))+(((IkReal(-1.00000000000000))*(x107))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
