/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:50:32.318320
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.182500000000000))*(x5));
x15=((IkReal(0.182500000000000))*(x8));
x16=((IkReal(0.276500000000000))*(x6));
x17=((IkReal(0.182500000000000))*(x6));
x18=((IkReal(1.00000000000000))*(x5));
x19=((IkReal(1.00000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x4));
x21=((IkReal(0.276500000000000))*(x0));
x22=((IkReal(0.182500000000000))*(x9));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(1.00000000000000))*(x3));
x25=((IkReal(1.00000000000000))*(x8));
x26=((IkReal(0.276500000000000))*(x3));
x27=((IkReal(1.00000000000000))*(x11));
x28=((IkReal(1.00000000000000))*(x9));
x29=((x3)*(x7));
x30=((x2)*(x7));
x31=((x1)*(x2));
x32=((x0)*(x7));
x33=((x20)*(x3));
x34=((x18)*(x32));
x35=((x18)*(x29));
x36=((x20)*(x7)*(x9));
x37=((((IkReal(-1.00000000000000))*(x33)))+(((x0)*(x31))));
x38=((((x0)*(x4)))+(((x3)*(x31))));
x39=((((IkReal(-1.00000000000000))*(x1)*(x33)))+(((x0)*(x2))));
x40=((((IkReal(-1.00000000000000))*(x18)*(x30)))+(((x1)*(x6))));
x41=((((x1)*(x5)))+(((x30)*(x6))));
x42=((((IkReal(-1.00000000000000))*(x0)*(x31)))+(x33));
x43=((((IkReal(-1.00000000000000))*(x0)*(x1)*(x20)))+(((IkReal(-1.00000000000000))*(x2)*(x24))));
x44=((((IkReal(-1.00000000000000))*(x0)*(x20)))+(((IkReal(-1.00000000000000))*(x24)*(x31))));
x45=((x41)*(x8));
x46=((x37)*(x6));
x47=((x44)*(x5));
x48=((x43)*(x9));
x49=((((IkReal(-1.00000000000000))*(x34)))+(x46));
x50=((((IkReal(-1.00000000000000))*(x35)))+(((x38)*(x6))));
x51=((((IkReal(-1.00000000000000))*(x36)))+(x45));
x52=((((x42)*(x5)))+(((IkReal(-1.00000000000000))*(x19)*(x32))));
x53=((((IkReal(-1.00000000000000))*(x19)*(x29)))+(x47));
x54=((((x9)*(((((IkReal(-1.00000000000000))*(x1)*(x18)))+(((IkReal(-1.00000000000000))*(x19)*(x30)))))))+(((IkReal(-1.00000000000000))*(x20)*(x7)*(x8))));
x55=((x50)*(x8));
x56=((x11)*(x52));
x57=((x11)*(x53));
x58=((x55)+(((x39)*(x9))));
x59=((((x9)*(((((IkReal(-1.00000000000000))*(x19)*(x38)))+(x35)))))+(((x39)*(x8))));
x60=((((x49)*(x8)))+(x48));
x61=((x12)*(x58));
x62=((((x9)*(((((IkReal(-1.00000000000000))*(x46)))+(x34)))))+(((x43)*(x8))));
eerot[0]=((((x13)*(((((x12)*(x60)))+(x56)))))+(((x10)*(x62))));
eerot[1]=((((x13)*(x62)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x52)))+(((IkReal(-1.00000000000000))*(x23)*(x60))))))));
eerot[2]=((((x12)*(x52)))+(((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x49)))+(((IkReal(-1.00000000000000))*(x28)*(x43))))))));
IkReal x63=((IkReal(1.00000000000000))*(x32));
eetrans[0]=((((IkReal(-0.328000000000000))*(x32)))+(((IkReal(-1.00000000000000))*(x16)*(x63)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x49)))+(((IkReal(-1.00000000000000))*(x22)*(x43)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x63)))+(((x14)*(x42)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x31)))+(((x26)*(x4))))))));
eerot[3]=((((x13)*(((x57)+(x61)))))+(((x10)*(x59))));
eerot[4]=((((x13)*(x59)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x53)))+(((IkReal(-1.00000000000000))*(x23)*(x58))))))));
eerot[5]=((((x12)*(x53)))+(((x11)*(((((IkReal(-1.00000000000000))*(x28)*(x39)))+(((IkReal(-1.00000000000000))*(x25)*(x50))))))));
IkReal x64=((IkReal(1.00000000000000))*(x29));
eetrans[1]=((((IkReal(-1.00000000000000))*(x16)*(x64)))+(((x11)*(((((IkReal(-1.00000000000000))*(x22)*(x39)))+(((IkReal(-1.00000000000000))*(x15)*(x50)))))))+(((x5)*(((((IkReal(-1.00000000000000))*(x21)*(x4)))+(((IkReal(-1.00000000000000))*(x26)*(x31)))))))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x64)))+(((x14)*(x44)))))))+(((IkReal(-0.328000000000000))*(x29))));
eerot[6]=((((x13)*(((((x12)*(x51)))+(((x11)*(x40)))))))+(((x10)*(x54))));
eerot[7]=((((x13)*(x54)))+(((x10)*(((((IkReal(-1.00000000000000))*(x27)*(x40)))+(((IkReal(-1.00000000000000))*(x23)*(x51))))))));
eerot[8]=((((x11)*(((((IkReal(-1.00000000000000))*(x25)*(x41)))+(x36)))))+(((x12)*(x40))));
eetrans[2]=((IkReal(0.300000000000000))+(((x1)*(x16)))+(((x11)*(((((IkReal(-1.00000000000000))*(x15)*(x41)))+(((x22)*(x4)*(x7)))))))+(((IkReal(-0.276500000000000))*(x30)*(x5)))+(((IkReal(0.328000000000000))*(x1)))+(((x12)*(((((x1)*(x17)))+(((IkReal(-1.00000000000000))*(x14)*(x30))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {3}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j3,cj3,sj3,htj3,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij3[2], _nj3;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1;  _ij3[0] = -1; _ij3[1] = -1; _nj3 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j3=pfree[0]; cj3=cos(pfree[0]); sj3=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((((IkReal(-0.182500000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((((IkReal(-0.182500000000000))*(r12)))+(py));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.300000000000000))+(pz)+(((IkReal(-0.182500000000000))*(r22))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(1.01462229303577))*(sj3)))+(((IkReal(-5.51316543906850))*(pp)*(sj3)))+(((cj3)*(sj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x65=((pz)*(sj3));
cj1array[0]=((((IKabs(((((IkReal(-0.328000000000000))*(pp)*(sj3)))+(((IkReal(0.0594939520000000))*(cj3)*(sj3)))+(((IkReal(0.0603638900000000))*(sj3))))) != 0)?((IkReal)1/(((((IkReal(-0.328000000000000))*(pp)*(sj3)))+(((IkReal(0.0594939520000000))*(cj3)*(sj3)))+(((IkReal(0.0603638900000000))*(sj3)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(x65)))+(((IkReal(0.181384000000000))*(cj3)*(x65)))+(((IkReal(0.184036250000000))*(x65))))));
if( cj1array[0] >= -1-IKFAST_SINCOS_THRESH && cj1array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j1valid[0] = j1valid[1] = true;
    j1array[0] = IKacos(cj1array[0]);
    sj1array[0] = IKsin(j1array[0]);
    cj1array[1] = cj1array[0];
    j1array[1] = -j1array[0];
    sj1array[1] = -sj1array[0];
}
else if( isnan(cj1array[0]) )
{
    // probably any value will work
    j1valid[0] = true;
    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
}
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal x66=((sj1)*(sj3));
dummyeval[0]=((((IkReal(-32.1215479373951))*(pp)*(x66)))+(x66));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj1;
dummyeval[1]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
IkReal x67=((IKabs(sj1) != 0)?((IkReal)1/(sj1)):(IkReal)1.0e30);
IkReal x68=((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30);
IkReal x69=((cj1)*(x67)*(x68));
cj2array[0]=((((IkReal(-3.61663652802893))*(pz)*(x67)*(x68)))+(((IkReal(1.18625678119349))*(x69)))+(((cj3)*(x69))));
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
IkReal x70=IKcos(j2);
IkReal x71=((cj1)*(cj3));
IkReal x72=((sj1)*(sj3)*(x70));
evalcond[0]=((((IkReal(-0.656000000000000))*(cj3)*(pz)))+(((IkReal(0.184036250000000))*(x71)))+(((IkReal(0.181384000000000))*(cj1)))+(((IkReal(0.0311317500000000))*(x72)))+(((IkReal(-0.553000000000000))*(pz)))+(((pp)*(x71)))+(((IkReal(-1.00000000000000))*(pp)*(x72))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((((IkReal(2000.00000000000))*((py)*(py))))+(((IkReal(2000.00000000000))*((px)*(px))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x73=((px)*(sj1));
IkReal x74=((IkReal(553.000000000000))*(cj3));
IkReal x75=((py)*(sj1));
IkReal x76=((cj1)*(cj2));
IkReal x77=((IkReal(553.000000000000))*(py)*(sj3));
IkReal x78=((IkReal(553.000000000000))*(px)*(sj3));
if( IKabs(((gconst0)*(((((IkReal(-656.000000000000))*(x75)))+(((sj2)*(x78)))+(((IkReal(-1.00000000000000))*(x74)*(x75)))+(((IkReal(-1.00000000000000))*(x76)*(x77))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-656.000000000000))*(x73)))+(((IkReal(-1.00000000000000))*(sj2)*(x77)))+(((IkReal(-1.00000000000000))*(x73)*(x74)))+(((IkReal(-1.00000000000000))*(x76)*(x78))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-656.000000000000))*(x75)))+(((sj2)*(x78)))+(((IkReal(-1.00000000000000))*(x74)*(x75)))+(((IkReal(-1.00000000000000))*(x76)*(x77)))))), ((gconst0)*(((((IkReal(-656.000000000000))*(x73)))+(((IkReal(-1.00000000000000))*(sj2)*(x77)))+(((IkReal(-1.00000000000000))*(x73)*(x74)))+(((IkReal(-1.00000000000000))*(x76)*(x78)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
IkReal x79=IKcos(j0);
IkReal x80=IKsin(j0);
IkReal x81=((IkReal(1.00000000000000))*(px));
IkReal x82=((IkReal(0.276500000000000))*(sj3));
evalcond[0]=((((py)*(x79)))+(((sj2)*(x82)))+(((IkReal(-1.00000000000000))*(x80)*(x81))));
evalcond[1]=((((IkReal(-0.328000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(py)*(x80)))+(((IkReal(-1.00000000000000))*(x79)*(x81)))+(((IkReal(-0.276500000000000))*(cj3)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x82))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal x83=(sj2)*(sj2);
IkReal x84=(cj1)*(cj1);
IkReal x85=(sj1)*(sj1);
IkReal x86=(cj2)*(cj2);
IkReal x87=((r11)*(sj0));
IkReal x88=((cj0)*(r01));
IkReal x89=((IkReal(1.00000000000000))*(sj3));
IkReal x90=((cj0)*(r11));
IkReal x91=((cj1)*(cj2));
IkReal x92=((IkReal(1.00000000000000))*(cj3)*(sj1));
IkReal x93=((sj2)*(x85));
IkReal x94=((cj1)*(cj3)*(r21));
IkReal x95=((r01)*(sj0)*(sj3));
IkReal x96=((sj2)*(x84));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x88)*(x89)*(x91)))+(((IkReal(-1.00000000000000))*(x86)*(x88)*(x92)))+(((IkReal(-1.00000000000000))*(x86)*(x87)*(x92)))+(((IkReal(-1.00000000000000))*(cj2)*(r21)*(sj1)*(x89)))+(((x86)*(x94)))+(((IkReal(-1.00000000000000))*(x83)*(x87)*(x92)))+(((x93)*(x95)))+(((x95)*(x96)))+(((x83)*(x94)))+(((IkReal(-1.00000000000000))*(x89)*(x90)*(x96)))+(((IkReal(-1.00000000000000))*(x89)*(x90)*(x93)))+(((IkReal(-1.00000000000000))*(x87)*(x89)*(x91)))+(((IkReal(-1.00000000000000))*(x83)*(x88)*(x92))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x97=(sj2)*(sj2);
IkReal x98=(cj2)*(cj2);
IkReal x99=(cj1)*(cj1);
IkReal x100=(sj1)*(sj1);
IkReal x101=((cj1)*(cj3));
IkReal x102=((r00)*(sj3));
IkReal x103=((IkReal(16.0000000000000))*(sj0));
IkReal x104=((IkReal(8.00000000000000))*(cj0));
IkReal x105=((cj1)*(cj2));
IkReal x106=((r10)*(sj3));
IkReal x107=((IkReal(16.0000000000000))*(cj0));
IkReal x108=((IkReal(4.00000000000000))*(sj3));
IkReal x109=((cj2)*(sj1));
IkReal x110=((r11)*(sj0));
IkReal x111=((cj3)*(sj1));
IkReal x112=((IkReal(8.00000000000000))*(sj0));
IkReal x113=((cj0)*(r11));
IkReal x114=((r20)*(sj3));
IkReal x115=((IkReal(16.0000000000000))*(x97));
IkReal x116=((IkReal(4.00000000000000))*(x98));
IkReal x117=((cj0)*(x111));
IkReal x118=((sj2)*(x99));
IkReal x119=((r00)*(x98));
IkReal x120=((IkReal(8.00000000000000))*(x97));
IkReal x121=((sj2)*(x100));
IkReal x122=((IkReal(8.00000000000000))*(x98));
IkReal x123=((IkReal(4.00000000000000))*(x97));
IkReal x124=((r01)*(sj0)*(x108));
IkReal x125=((r10)*(sj0)*(x111));
IkReal x126=((((r21)*(x101)*(x116)))+(((x121)*(x124)))+(((r21)*(x101)*(x123)))+(((x118)*(x124))));
IkReal x127=((((r01)*(x117)*(x123)))+(((r01)*(x116)*(x117)))+(((x108)*(x113)*(x121)))+(((x110)*(x111)*(x116)))+(((x110)*(x111)*(x123)))+(((cj0)*(r01)*(x105)*(x108)))+(((r21)*(x108)*(x109)))+(((x105)*(x108)*(x110)))+(((x108)*(x113)*(x118))));
IkReal x128=((((IkReal(-1.00000000000000))*(x127)))+(x126));
IkReal x129=((((x104)*(x106)*(x121)))+(((x104)*(x106)*(x118)))+(((x104)*(x111)*(x119)))+(((x105)*(x106)*(x112)))+(((x102)*(x104)*(x105)))+(((IkReal(-1.00000000000000))*(x102)*(x112)*(x121)))+(((IkReal(-1.00000000000000))*(x102)*(x112)*(x118)))+(((IkReal(-1.00000000000000))*(r20)*(x101)*(x120)))+(((IkReal(-1.00000000000000))*(r20)*(x101)*(x122)))+(((r00)*(x104)*(x111)*(x97)))+(((IkReal(8.00000000000000))*(x109)*(x114)))+(((r10)*(x111)*(x112)*(x98)))+(((r10)*(x111)*(x112)*(x97))));
IkReal x130=((((IkReal(-1.00000000000000))*(x126)))+(x127));
op[0]=x128;
op[1]=x129;
op[2]=x128;
op[3]=((((r00)*(x107)*(x111)*(x97)))+(((IkReal(-1.00000000000000))*(x102)*(x103)*(x121)))+(((IkReal(16.0000000000000))*(x109)*(x114)))+(((IkReal(-1.00000000000000))*(x102)*(x103)*(x118)))+(((x103)*(x105)*(x106)))+(((x102)*(x105)*(x107)))+(((x106)*(x107)*(x118)))+(((IkReal(-1.00000000000000))*(r20)*(x101)*(x115)))+(((x106)*(x107)*(x121)))+(((x107)*(x111)*(x119)))+(((r10)*(x103)*(x111)*(x98)))+(((r10)*(x103)*(x111)*(x97)))+(((IkReal(-16.0000000000000))*(r20)*(x101)*(x98))));
op[4]=x130;
op[5]=x129;
op[6]=x130;
polyroots6(op,zeror,numroots);
IkReal j6array[6], cj6array[6], sj6array[6], tempj6array[1];
int numsolutions = 0;
for(int ij6 = 0; ij6 < numroots; ++ij6)
{
IkReal htj6 = zeror[ij6];
tempj6array[0]=((IkReal(2.00000000000000))*(atan(htj6)));
for(int kj6 = 0; kj6 < 1; ++kj6)
{
j6array[numsolutions] = tempj6array[kj6];
if( j6array[numsolutions] > IKPI )
{
    j6array[numsolutions]-=IK2PI;
}
else if( j6array[numsolutions] < -IKPI )
{
    j6array[numsolutions]+=IK2PI;
}
sj6array[numsolutions] = IKsin(j6array[numsolutions]);
cj6array[numsolutions] = IKcos(j6array[numsolutions]);
numsolutions++;
}
}
bool j6valid[6]={true,true,true,true,true,true};
_nj6 = 6;
for(int ij6 = 0; ij6 < numsolutions; ++ij6)
    {
if( !j6valid[ij6] )
{
    continue;
}
    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
htj6 = IKtan(j6/2);

_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < numsolutions; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x131=((IkReal(1.00000000000000))*(cj3)*(sj1));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x131)*((cj2)*(cj2))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x131)*((sj2)*(sj2))))));
IkReal x132=((IkReal(1.00000000000000))*(cj3)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x132)*((sj2)*(sj2))))+(((IkReal(-1.00000000000000))*(x132)*((cj2)*(cj2)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x133=((cj6)*(sj0));
IkReal x134=((sj0)*(sj6));
IkReal x135=((cj0)*(cj6));
IkReal x136=((cj0)*(sj6));
IkReal x137=((IkReal(1.00000000000000))*(r22));
gconst2=IKsign(((((r00)*(r22)*(x133)))+(((r11)*(r22)*(x136)))+(((IkReal(-1.00000000000000))*(r02)*(r20)*(x133)))+(((IkReal(-1.00000000000000))*(r12)*(r21)*(x136)))+(((r02)*(r21)*(x134)))+(((IkReal(-1.00000000000000))*(r01)*(x134)*(x137)))+(((IkReal(-1.00000000000000))*(r10)*(x135)*(x137)))+(((r12)*(r20)*(x135)))));
IkReal x138=((cj6)*(sj0));
IkReal x139=((sj0)*(sj6));
IkReal x140=((cj0)*(cj6));
IkReal x141=((cj0)*(sj6));
IkReal x142=((IkReal(1.00000000000000))*(r22));
dummyeval[0]=((((r00)*(r22)*(x138)))+(((IkReal(-1.00000000000000))*(r12)*(r21)*(x141)))+(((IkReal(-1.00000000000000))*(r02)*(r20)*(x138)))+(((IkReal(-1.00000000000000))*(r10)*(x140)*(x142)))+(((r02)*(r21)*(x139)))+(((r11)*(r22)*(x141)))+(((r12)*(r20)*(x140)))+(((IkReal(-1.00000000000000))*(r01)*(x139)*(x142))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x143=((sj2)*(sj3));
IkReal x144=((cj0)*(r12));
IkReal x145=((cj1)*(cj3));
IkReal x146=((r02)*(sj0));
IkReal x147=((IkReal(1.00000000000000))*(cj6));
IkReal x148=((cj0)*(r10));
IkReal x149=((cj6)*(r00)*(sj0));
IkReal x150=((cj0)*(r11)*(sj6));
IkReal x151=((cj2)*(sj1)*(sj3));
IkReal x152=((r01)*(sj0)*(sj6));
IkReal x153=((IkReal(1.00000000000000))*(x151));
if( IKabs(((gconst2)*(((((x144)*(x145)))+(((IkReal(-1.00000000000000))*(x144)*(x153)))+(((x146)*(x151)))+(((IkReal(-1.00000000000000))*(x145)*(x146)))+(((r22)*(x143))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((r21)*(sj6)*(x143)))+(((IkReal(-1.00000000000000))*(x150)*(x153)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(x147)*(x151)))+(((x145)*(x149)))+(((x151)*(x152)))+(((x145)*(x150)))+(((IkReal(-1.00000000000000))*(x145)*(x152)))+(((IkReal(-1.00000000000000))*(x145)*(x147)*(x148)))+(((IkReal(-1.00000000000000))*(r20)*(x143)*(x147)))+(((cj6)*(x148)*(x151))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst2)*(((((x144)*(x145)))+(((IkReal(-1.00000000000000))*(x144)*(x153)))+(((x146)*(x151)))+(((IkReal(-1.00000000000000))*(x145)*(x146)))+(((r22)*(x143)))))), ((gconst2)*(((((r21)*(sj6)*(x143)))+(((IkReal(-1.00000000000000))*(x150)*(x153)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(x147)*(x151)))+(((x145)*(x149)))+(((x151)*(x152)))+(((x145)*(x150)))+(((IkReal(-1.00000000000000))*(x145)*(x152)))+(((IkReal(-1.00000000000000))*(x145)*(x147)*(x148)))+(((IkReal(-1.00000000000000))*(r20)*(x143)*(x147)))+(((cj6)*(x148)*(x151)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x154=IKsin(j5);
IkReal x155=IKcos(j5);
IkReal x156=((IkReal(1.00000000000000))*(sj1));
IkReal x157=((cj2)*(sj3));
IkReal x158=((IkReal(1.00000000000000))*(cj0));
IkReal x159=((sj6)*(x154));
IkReal x160=((IkReal(1.00000000000000))*(sj0)*(x155));
IkReal x161=((cj6)*(r10)*(x154));
IkReal x162=((IkReal(1.00000000000000))*(cj6)*(r00)*(x154));
evalcond[0]=((((IkReal(-1.00000000000000))*(r22)*(x155)))+(((IkReal(-1.00000000000000))*(cj6)*(r20)*(x154)))+(((r21)*(x159)))+(((IkReal(-1.00000000000000))*(x156)*(x157)))+(((cj1)*(cj3))));
evalcond[1]=((((cj0)*(x161)))+(((sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(r02)*(x160)))+(((cj0)*(r12)*(x155)))+(((r01)*(sj0)*(x159)))+(((IkReal(-1.00000000000000))*(r11)*(x158)*(x159)))+(((IkReal(-1.00000000000000))*(sj0)*(x162))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r12)*(x160)))+(((IkReal(-1.00000000000000))*(sj0)*(x161)))+(((r11)*(sj0)*(x159)))+(((IkReal(-1.00000000000000))*(cj1)*(x157)))+(((IkReal(-1.00000000000000))*(cj3)*(x156)))+(((IkReal(-1.00000000000000))*(r02)*(x155)*(x158)))+(((IkReal(-1.00000000000000))*(cj6)*(r00)*(x154)*(x158)))+(((cj0)*(r01)*(x159))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x163=((IkReal(1.00000000000000))*(cj3)*(sj1));
gconst3=IKsign(((((IkReal(-1.00000000000000))*(x163)*((cj2)*(cj2))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x163)*((sj2)*(sj2))))));
IkReal x164=((IkReal(1.00000000000000))*(cj3)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x164)*((cj2)*(cj2))))+(((IkReal(-1.00000000000000))*(x164)*((sj2)*(sj2)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x165=((cj2)*(cj3));
IkReal x166=((cj6)*(r21));
IkReal x167=((r10)*(sj6));
IkReal x168=((r20)*(sj6));
IkReal x169=((cj3)*(sj2));
IkReal x170=((IkReal(1.00000000000000))*(sj2));
IkReal x171=((cj0)*(sj1));
IkReal x172=((cj1)*(sj0)*(sj3));
IkReal x173=((cj6)*(r11)*(x171));
IkReal x174=((r00)*(sj0)*(sj1)*(sj6));
IkReal x175=((IkReal(1.00000000000000))*(cj0)*(cj1)*(sj3));
IkReal x176=((cj6)*(r01)*(sj0)*(sj1));
if( IKabs(((gconst3)*(((((sj2)*(x173)))+(((cj2)*(x166)))+(((cj2)*(x168)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((IkReal(-1.00000000000000))*(x170)*(x176)))+(((sj2)*(x167)*(x171))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((r00)*(sj6)*(x172)))+(((IkReal(-1.00000000000000))*(x165)*(x173)))+(((x168)*(x169)))+(((IkReal(-1.00000000000000))*(x165)*(x167)*(x171)))+(((x166)*(x169)))+(((x165)*(x176)))+(((x165)*(x174)))+(((cj6)*(r01)*(x172)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x175)))+(((IkReal(-1.00000000000000))*(x167)*(x175))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((sj2)*(x173)))+(((cj2)*(x166)))+(((cj2)*(x168)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((IkReal(-1.00000000000000))*(x170)*(x176)))+(((sj2)*(x167)*(x171)))))), ((gconst3)*(((((r00)*(sj6)*(x172)))+(((IkReal(-1.00000000000000))*(x165)*(x173)))+(((x168)*(x169)))+(((IkReal(-1.00000000000000))*(x165)*(x167)*(x171)))+(((x166)*(x169)))+(((x165)*(x176)))+(((x165)*(x174)))+(((cj6)*(r01)*(x172)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x175)))+(((IkReal(-1.00000000000000))*(x167)*(x175)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x177=IKsin(j4);
IkReal x178=IKcos(j4);
IkReal x179=((cj0)*(cj6));
IkReal x180=((IkReal(1.00000000000000))*(r01));
IkReal x181=((IkReal(1.00000000000000))*(sj1));
IkReal x182=((IkReal(1.00000000000000))*(sj0));
IkReal x183=((r00)*(sj6));
IkReal x184=((cj5)*(r10));
IkReal x185=((cj0)*(sj6));
IkReal x186=((IkReal(1.00000000000000))*(cj5));
IkReal x187=((r02)*(sj5));
IkReal x188=((IkReal(1.00000000000000))*(cj1));
IkReal x189=((cj5)*(r01));
IkReal x190=((IkReal(1.00000000000000))*(cj2));
IkReal x191=((IkReal(1.00000000000000))*(cj0));
IkReal x192=((sj0)*(sj6));
IkReal x193=((r12)*(sj5));
IkReal x194=((sj3)*(x178));
IkReal x195=((sj2)*(x177));
IkReal x196=((sj3)*(x177));
IkReal x197=((sj2)*(x178));
IkReal x198=((cj2)*(cj3)*(x177));
IkReal x199=((cj2)*(cj3)*(x178));
evalcond[0]=((((IkReal(-1.00000000000000))*(x188)*(x196)))+(((IkReal(-1.00000000000000))*(cj6)*(r21)))+(((IkReal(-1.00000000000000))*(x181)*(x197)))+(((IkReal(-1.00000000000000))*(x181)*(x198)))+(((IkReal(-1.00000000000000))*(r20)*(sj6))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x178)*(x190)))+(((cj3)*(x195)))+(((IkReal(-1.00000000000000))*(cj6)*(sj0)*(x180)))+(((r10)*(x185)))+(((IkReal(-1.00000000000000))*(x182)*(x183)))+(((r11)*(x179))));
evalcond[2]=((((cj5)*(r21)*(sj6)))+(((sj1)*(x199)))+(((r22)*(sj5)))+(((IkReal(-1.00000000000000))*(cj6)*(r20)*(x186)))+(((cj1)*(x194)))+(((IkReal(-1.00000000000000))*(x181)*(x195))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj6)*(r11)*(x182)))+(((IkReal(-1.00000000000000))*(x188)*(x198)))+(((IkReal(-1.00000000000000))*(x188)*(x197)))+(((sj1)*(x196)))+(((IkReal(-1.00000000000000))*(x179)*(x180)))+(((IkReal(-1.00000000000000))*(x183)*(x191)))+(((IkReal(-1.00000000000000))*(r10)*(sj6)*(x182))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj3)*(x197)))+(((IkReal(-1.00000000000000))*(x191)*(x193)))+(((sj0)*(x187)))+(((x179)*(x184)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(r00)*(x182)))+(((x189)*(x192)))+(((IkReal(-1.00000000000000))*(x177)*(x190)))+(((IkReal(-1.00000000000000))*(r11)*(x185)*(x186))));
evalcond[5]=((((cj0)*(x187)))+(((IkReal(-1.00000000000000))*(x188)*(x195)))+(((cj5)*(r11)*(x192)))+(((sj0)*(x193)))+(((x185)*(x189)))+(((IkReal(-1.00000000000000))*(r00)*(x179)*(x186)))+(((cj1)*(x199)))+(((IkReal(-1.00000000000000))*(cj6)*(x182)*(x184)))+(((IkReal(-1.00000000000000))*(x181)*(x194))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x200=((cj2)*(cj3));
IkReal x201=((cj6)*(r21));
IkReal x202=((r10)*(sj6));
IkReal x203=((r20)*(sj6));
IkReal x204=((cj3)*(sj2));
IkReal x205=((IkReal(1.00000000000000))*(sj2));
IkReal x206=((cj0)*(sj1));
IkReal x207=((cj1)*(sj0)*(sj3));
IkReal x208=((cj6)*(r11)*(x206));
IkReal x209=((r00)*(sj0)*(sj1)*(sj6));
IkReal x210=((IkReal(1.00000000000000))*(cj0)*(cj1)*(sj3));
IkReal x211=((cj6)*(r01)*(sj0)*(sj1));
if( IKabs(((gconst1)*(((((cj2)*(x201)))+(((cj2)*(x203)))+(((sj2)*(x202)*(x206)))+(((IkReal(-1.00000000000000))*(x205)*(x211)))+(((IkReal(-1.00000000000000))*(x205)*(x209)))+(((sj2)*(x208))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((cj6)*(r01)*(x207)))+(((x200)*(x209)))+(((x201)*(x204)))+(((IkReal(-1.00000000000000))*(x200)*(x202)*(x206)))+(((x203)*(x204)))+(((r00)*(sj6)*(x207)))+(((IkReal(-1.00000000000000))*(x202)*(x210)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x210)))+(((IkReal(-1.00000000000000))*(x200)*(x208)))+(((x200)*(x211))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((((cj2)*(x201)))+(((cj2)*(x203)))+(((sj2)*(x202)*(x206)))+(((IkReal(-1.00000000000000))*(x205)*(x211)))+(((IkReal(-1.00000000000000))*(x205)*(x209)))+(((sj2)*(x208)))))), ((gconst1)*(((((cj6)*(r01)*(x207)))+(((x200)*(x209)))+(((x201)*(x204)))+(((IkReal(-1.00000000000000))*(x200)*(x202)*(x206)))+(((x203)*(x204)))+(((r00)*(sj6)*(x207)))+(((IkReal(-1.00000000000000))*(x202)*(x210)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x210)))+(((IkReal(-1.00000000000000))*(x200)*(x208)))+(((x200)*(x211)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x212=IKsin(j4);
IkReal x213=IKcos(j4);
IkReal x214=((IkReal(1.00000000000000))*(cj1));
IkReal x215=((cj0)*(cj6));
IkReal x216=((IkReal(1.00000000000000))*(r01));
IkReal x217=((IkReal(1.00000000000000))*(sj6));
IkReal x218=((IkReal(1.00000000000000))*(cj6));
IkReal x219=((sj3)*(x212));
IkReal x220=((cj3)*(x212));
IkReal x221=((IkReal(1.00000000000000))*(x213));
evalcond[0]=((((IkReal(-1.00000000000000))*(r20)*(x217)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x220)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x221)))+(((IkReal(-1.00000000000000))*(x214)*(x219)))+(((IkReal(-1.00000000000000))*(r21)*(x218))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(sj0)*(x217)))+(((sj2)*(x220)))+(((r11)*(x215)))+(((cj0)*(r10)*(sj6)))+(((IkReal(-1.00000000000000))*(cj2)*(x221)))+(((IkReal(-1.00000000000000))*(cj6)*(sj0)*(x216))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj2)*(x213)*(x214)))+(((IkReal(-1.00000000000000))*(r10)*(sj0)*(x217)))+(((sj1)*(x219)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x217)))+(((IkReal(-1.00000000000000))*(cj2)*(x214)*(x220)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x218)))+(((IkReal(-1.00000000000000))*(x215)*(x216))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(2.00000000000000))*(cj6)*(r20)*(r21)*(sj6)))+(((IkReal(-1.00000000000000))*((r21)*(r21))*((sj6)*(sj6))))+(((IkReal(-1.00000000000000))*((r22)*(r22))))+(((IkReal(-1.00000000000000))*((cj6)*(cj6))*((r20)*(r20))))));
dummyeval[0]=((((IkReal(2.00000000000000))*(cj6)*(r20)*(r21)*(sj6)))+(((IkReal(-1.00000000000000))*((r21)*(r21))*((sj6)*(sj6))))+(((IkReal(-1.00000000000000))*((r22)*(r22))))+(((IkReal(-1.00000000000000))*((cj6)*(cj6))*((r20)*(r20)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x222=((IkReal(1.00000000000000))*(sj1));
IkReal x223=((cj6)*(r20));
IkReal x224=((sj2)*(sj4));
IkReal x225=((r21)*(sj6));
IkReal x226=((cj2)*(sj3));
IkReal x227=((r22)*(sj1));
IkReal x228=((cj2)*(cj3)*(cj4));
IkReal x229=((IkReal(1.00000000000000))*(cj1)*(cj3));
IkReal x230=((cj1)*(cj4)*(sj3));
if( IKabs(((gconst4)*(((((x227)*(x228)))+(((IkReal(-1.00000000000000))*(x222)*(x225)*(x226)))+(((IkReal(-1.00000000000000))*(r22)*(x222)*(x224)))+(((sj1)*(x223)*(x226)))+(((IkReal(-1.00000000000000))*(x223)*(x229)))+(((cj1)*(cj3)*(x225)))+(((r22)*(x230))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x223)*(x230)))+(((IkReal(-1.00000000000000))*(x222)*(x224)*(x225)))+(((IkReal(-1.00000000000000))*(x222)*(x223)*(x228)))+(((sj1)*(x223)*(x224)))+(((x225)*(x230)))+(((sj1)*(x225)*(x228)))+(((x226)*(x227)))+(((IkReal(-1.00000000000000))*(r22)*(x229))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst4)*(((((x227)*(x228)))+(((IkReal(-1.00000000000000))*(x222)*(x225)*(x226)))+(((IkReal(-1.00000000000000))*(r22)*(x222)*(x224)))+(((sj1)*(x223)*(x226)))+(((IkReal(-1.00000000000000))*(x223)*(x229)))+(((cj1)*(cj3)*(x225)))+(((r22)*(x230)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x223)*(x230)))+(((IkReal(-1.00000000000000))*(x222)*(x224)*(x225)))+(((IkReal(-1.00000000000000))*(x222)*(x223)*(x228)))+(((sj1)*(x223)*(x224)))+(((x225)*(x230)))+(((sj1)*(x225)*(x228)))+(((x226)*(x227)))+(((IkReal(-1.00000000000000))*(r22)*(x229)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x231=IKsin(j5);
IkReal x232=IKcos(j5);
IkReal x233=((IkReal(1.00000000000000))*(cj6));
IkReal x234=((r01)*(sj6));
IkReal x235=((r11)*(sj0));
IkReal x236=((IkReal(1.00000000000000))*(r12));
IkReal x237=((cj6)*(r10));
IkReal x238=((cj3)*(sj1));
IkReal x239=((cj1)*(cj4));
IkReal x240=((IkReal(1.00000000000000))*(r11));
IkReal x241=((IkReal(1.00000000000000))*(cj4));
IkReal x242=((cj0)*(x231));
IkReal x243=((sj6)*(x231));
IkReal x244=((IkReal(1.00000000000000))*(cj2)*(sj3));
IkReal x245=((sj6)*(x232));
IkReal x246=((sj0)*(x231));
IkReal x247=((sj0)*(x232));
IkReal x248=((IkReal(1.00000000000000))*(sj2)*(sj4));
IkReal x249=((cj0)*(x232));
IkReal x250=((IkReal(1.00000000000000))*(r02)*(x232));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj1)*(x244)))+(((IkReal(-1.00000000000000))*(r22)*(x232)))+(((r21)*(x243)))+(((IkReal(-1.00000000000000))*(r20)*(x231)*(x233)))+(((cj1)*(cj3))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj1)*(x248)))+(((r21)*(x245)))+(((cj2)*(cj4)*(x238)))+(((IkReal(-1.00000000000000))*(r20)*(x232)*(x233)))+(((sj3)*(x239)))+(((r22)*(x231))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x247)))+(((sj2)*(sj3)))+(((x234)*(x246)))+(((x237)*(x242)))+(((IkReal(-1.00000000000000))*(r00)*(x233)*(x246)))+(((IkReal(-1.00000000000000))*(sj6)*(x240)*(x242)))+(((r12)*(x249))));
evalcond[3]=((((r02)*(x246)))+(((x234)*(x247)))+(((x237)*(x249)))+(((IkReal(-1.00000000000000))*(cj2)*(sj4)))+(((IkReal(-1.00000000000000))*(r00)*(x233)*(x247)))+(((IkReal(-1.00000000000000))*(x236)*(x242)))+(((IkReal(-1.00000000000000))*(cj0)*(x240)*(x245)))+(((IkReal(-1.00000000000000))*(cj3)*(sj2)*(x241))));
evalcond[4]=((((IkReal(-1.00000000000000))*(r02)*(x249)))+(((x234)*(x242)))+(((IkReal(-1.00000000000000))*(cj1)*(x244)))+(((IkReal(-1.00000000000000))*(r00)*(x233)*(x242)))+(((IkReal(-1.00000000000000))*(x236)*(x247)))+(((IkReal(-1.00000000000000))*(r10)*(x233)*(x246)))+(((IkReal(-1.00000000000000))*(x238)))+(((x235)*(x243))));
evalcond[5]=((((IkReal(-1.00000000000000))*(sj1)*(sj3)*(x241)))+(((cj2)*(cj3)*(x239)))+(((r02)*(x242)))+(((x234)*(x249)))+(((IkReal(-1.00000000000000))*(cj1)*(x248)))+(((IkReal(-1.00000000000000))*(r00)*(x233)*(x249)))+(((r12)*(x246)))+(((IkReal(-1.00000000000000))*(r10)*(x233)*(x247)))+(((x235)*(x245))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
    }

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
IkReal x251=((IkReal(1.00000000000000))*(pp));
IkReal x252=((sj1)*(sj3));
IkReal x253=((IKabs(((((IkReal(-1.00000000000000))*(x251)*(x252)))+(((IkReal(0.0311317500000000))*(x252))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x251)*(x252)))+(((IkReal(0.0311317500000000))*(x252)))))):(IkReal)1.0e30);
IkReal x254=((cj1)*(x253));
IkReal x255=((cj3)*(x253));
IkReal x256=((pz)*(x253));
IkReal x257=((cj3)*(x254));
cj2array[0]=((((IkReal(0.656000000000000))*(pz)*(x255)))+(((IkReal(-1.00000000000000))*(cj1)*(cj3)*(x251)*(((IKabs(((((IkReal(-1.00000000000000))*(pp)*(x252)))+(((IkReal(0.0311317500000000))*(x252))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(pp)*(x252)))+(((IkReal(0.0311317500000000))*(x252)))))):(IkReal)1.0e30))))+(((IkReal(-0.184036250000000))*(x257)))+(((IkReal(0.553000000000000))*(x256)))+(((IkReal(-0.181384000000000))*(x254))));
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(0.328000000000000))*(cj1)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-0.276500000000000))*(sj1)*(sj3)*(IKcos(j2))))+(((IkReal(0.276500000000000))*(cj1)*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((((IkReal(2000.00000000000))*((py)*(py))))+(((IkReal(2000.00000000000))*((px)*(px))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x258=((px)*(sj1));
IkReal x259=((IkReal(553.000000000000))*(cj3));
IkReal x260=((py)*(sj1));
IkReal x261=((cj1)*(cj2));
IkReal x262=((IkReal(553.000000000000))*(py)*(sj3));
IkReal x263=((IkReal(553.000000000000))*(px)*(sj3));
if( IKabs(((gconst0)*(((((sj2)*(x263)))+(((IkReal(-1.00000000000000))*(x259)*(x260)))+(((IkReal(-656.000000000000))*(x260)))+(((IkReal(-1.00000000000000))*(x261)*(x262))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(sj2)*(x262)))+(((IkReal(-1.00000000000000))*(x261)*(x263)))+(((IkReal(-1.00000000000000))*(x258)*(x259)))+(((IkReal(-656.000000000000))*(x258))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((sj2)*(x263)))+(((IkReal(-1.00000000000000))*(x259)*(x260)))+(((IkReal(-656.000000000000))*(x260)))+(((IkReal(-1.00000000000000))*(x261)*(x262)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(sj2)*(x262)))+(((IkReal(-1.00000000000000))*(x261)*(x263)))+(((IkReal(-1.00000000000000))*(x258)*(x259)))+(((IkReal(-656.000000000000))*(x258)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
IkReal x264=IKcos(j0);
IkReal x265=IKsin(j0);
IkReal x266=((IkReal(1.00000000000000))*(px));
IkReal x267=((IkReal(0.276500000000000))*(sj3));
evalcond[0]=((((sj2)*(x267)))+(((IkReal(-1.00000000000000))*(x265)*(x266)))+(((py)*(x264))));
evalcond[1]=((((IkReal(-0.328000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(py)*(x265)))+(((IkReal(-0.276500000000000))*(cj3)*(sj1)))+(((IkReal(-1.00000000000000))*(x264)*(x266)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x267))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal x268=(sj2)*(sj2);
IkReal x269=(cj1)*(cj1);
IkReal x270=(sj1)*(sj1);
IkReal x271=(cj2)*(cj2);
IkReal x272=((r11)*(sj0));
IkReal x273=((cj0)*(r01));
IkReal x274=((IkReal(1.00000000000000))*(sj3));
IkReal x275=((cj0)*(r11));
IkReal x276=((cj1)*(cj2));
IkReal x277=((IkReal(1.00000000000000))*(cj3)*(sj1));
IkReal x278=((sj2)*(x270));
IkReal x279=((cj1)*(cj3)*(r21));
IkReal x280=((r01)*(sj0)*(sj3));
IkReal x281=((sj2)*(x269));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x273)*(x274)*(x276)))+(((x271)*(x279)))+(((x278)*(x280)))+(((IkReal(-1.00000000000000))*(x268)*(x272)*(x277)))+(((IkReal(-1.00000000000000))*(x274)*(x275)*(x281)))+(((IkReal(-1.00000000000000))*(cj2)*(r21)*(sj1)*(x274)))+(((IkReal(-1.00000000000000))*(x272)*(x274)*(x276)))+(((x280)*(x281)))+(((IkReal(-1.00000000000000))*(x268)*(x273)*(x277)))+(((IkReal(-1.00000000000000))*(x271)*(x272)*(x277)))+(((IkReal(-1.00000000000000))*(x271)*(x273)*(x277)))+(((IkReal(-1.00000000000000))*(x274)*(x275)*(x278)))+(((x268)*(x279))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x282=(sj2)*(sj2);
IkReal x283=(cj2)*(cj2);
IkReal x284=(cj1)*(cj1);
IkReal x285=(sj1)*(sj1);
IkReal x286=((cj1)*(cj3));
IkReal x287=((r00)*(sj3));
IkReal x288=((IkReal(16.0000000000000))*(sj0));
IkReal x289=((IkReal(8.00000000000000))*(cj0));
IkReal x290=((cj1)*(cj2));
IkReal x291=((r10)*(sj3));
IkReal x292=((IkReal(16.0000000000000))*(cj0));
IkReal x293=((IkReal(4.00000000000000))*(sj3));
IkReal x294=((cj2)*(sj1));
IkReal x295=((r11)*(sj0));
IkReal x296=((cj3)*(sj1));
IkReal x297=((IkReal(8.00000000000000))*(sj0));
IkReal x298=((cj0)*(r11));
IkReal x299=((r20)*(sj3));
IkReal x300=((IkReal(16.0000000000000))*(x282));
IkReal x301=((IkReal(4.00000000000000))*(x283));
IkReal x302=((cj0)*(x296));
IkReal x303=((sj2)*(x284));
IkReal x304=((r00)*(x283));
IkReal x305=((IkReal(8.00000000000000))*(x282));
IkReal x306=((sj2)*(x285));
IkReal x307=((IkReal(8.00000000000000))*(x283));
IkReal x308=((IkReal(4.00000000000000))*(x282));
IkReal x309=((r01)*(sj0)*(x293));
IkReal x310=((r10)*(sj0)*(x296));
IkReal x311=((((x303)*(x309)))+(((r21)*(x286)*(x308)))+(((r21)*(x286)*(x301)))+(((x306)*(x309))));
IkReal x312=((((x295)*(x296)*(x308)))+(((x295)*(x296)*(x301)))+(((r01)*(x302)*(x308)))+(((x293)*(x298)*(x306)))+(((x293)*(x298)*(x303)))+(((x290)*(x293)*(x295)))+(((r21)*(x293)*(x294)))+(((cj0)*(r01)*(x290)*(x293)))+(((r01)*(x301)*(x302))));
IkReal x313=((x311)+(((IkReal(-1.00000000000000))*(x312))));
IkReal x314=((((IkReal(8.00000000000000))*(x294)*(x299)))+(((x289)*(x291)*(x303)))+(((x289)*(x291)*(x306)))+(((r00)*(x282)*(x289)*(x296)))+(((x287)*(x289)*(x290)))+(((x290)*(x291)*(x297)))+(((r10)*(x283)*(x296)*(x297)))+(((IkReal(-1.00000000000000))*(r20)*(x286)*(x305)))+(((IkReal(-1.00000000000000))*(r20)*(x286)*(x307)))+(((r10)*(x282)*(x296)*(x297)))+(((x289)*(x296)*(x304)))+(((IkReal(-1.00000000000000))*(x287)*(x297)*(x306)))+(((IkReal(-1.00000000000000))*(x287)*(x297)*(x303))));
IkReal x315=((x312)+(((IkReal(-1.00000000000000))*(x311))));
op[0]=x313;
op[1]=x314;
op[2]=x313;
op[3]=((((IkReal(-1.00000000000000))*(x287)*(x288)*(x303)))+(((IkReal(-1.00000000000000))*(x287)*(x288)*(x306)))+(((r00)*(x282)*(x292)*(x296)))+(((IkReal(16.0000000000000))*(x294)*(x299)))+(((r10)*(x282)*(x288)*(x296)))+(((IkReal(-16.0000000000000))*(r20)*(x283)*(x286)))+(((r10)*(x283)*(x288)*(x296)))+(((x288)*(x290)*(x291)))+(((x292)*(x296)*(x304)))+(((x287)*(x290)*(x292)))+(((IkReal(-1.00000000000000))*(r20)*(x286)*(x300)))+(((x291)*(x292)*(x306)))+(((x291)*(x292)*(x303))));
op[4]=x315;
op[5]=x314;
op[6]=x315;
polyroots6(op,zeror,numroots);
IkReal j6array[6], cj6array[6], sj6array[6], tempj6array[1];
int numsolutions = 0;
for(int ij6 = 0; ij6 < numroots; ++ij6)
{
IkReal htj6 = zeror[ij6];
tempj6array[0]=((IkReal(2.00000000000000))*(atan(htj6)));
for(int kj6 = 0; kj6 < 1; ++kj6)
{
j6array[numsolutions] = tempj6array[kj6];
if( j6array[numsolutions] > IKPI )
{
    j6array[numsolutions]-=IK2PI;
}
else if( j6array[numsolutions] < -IKPI )
{
    j6array[numsolutions]+=IK2PI;
}
sj6array[numsolutions] = IKsin(j6array[numsolutions]);
cj6array[numsolutions] = IKcos(j6array[numsolutions]);
numsolutions++;
}
}
bool j6valid[6]={true,true,true,true,true,true};
_nj6 = 6;
for(int ij6 = 0; ij6 < numsolutions; ++ij6)
    {
if( !j6valid[ij6] )
{
    continue;
}
    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
htj6 = IKtan(j6/2);

_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < numsolutions; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x316=((IkReal(1.00000000000000))*(cj3)*(sj1));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x316)*((sj2)*(sj2))))+(((IkReal(-1.00000000000000))*(x316)*((cj2)*(cj2))))));
IkReal x317=((IkReal(1.00000000000000))*(cj3)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x317)*((sj2)*(sj2))))+(((IkReal(-1.00000000000000))*(x317)*((cj2)*(cj2)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x318=((cj6)*(sj0));
IkReal x319=((sj0)*(sj6));
IkReal x320=((cj0)*(cj6));
IkReal x321=((cj0)*(sj6));
IkReal x322=((IkReal(1.00000000000000))*(r22));
gconst2=IKsign(((((IkReal(-1.00000000000000))*(r01)*(x319)*(x322)))+(((r00)*(r22)*(x318)))+(((r02)*(r21)*(x319)))+(((IkReal(-1.00000000000000))*(r02)*(r20)*(x318)))+(((r12)*(r20)*(x320)))+(((r11)*(r22)*(x321)))+(((IkReal(-1.00000000000000))*(r10)*(x320)*(x322)))+(((IkReal(-1.00000000000000))*(r12)*(r21)*(x321)))));
IkReal x323=((cj6)*(sj0));
IkReal x324=((sj0)*(sj6));
IkReal x325=((cj0)*(cj6));
IkReal x326=((cj0)*(sj6));
IkReal x327=((IkReal(1.00000000000000))*(r22));
dummyeval[0]=((((IkReal(-1.00000000000000))*(r10)*(x325)*(x327)))+(((r12)*(r20)*(x325)))+(((r11)*(r22)*(x326)))+(((IkReal(-1.00000000000000))*(r02)*(r20)*(x323)))+(((r00)*(r22)*(x323)))+(((IkReal(-1.00000000000000))*(r12)*(r21)*(x326)))+(((r02)*(r21)*(x324)))+(((IkReal(-1.00000000000000))*(r01)*(x324)*(x327))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x328=((sj2)*(sj3));
IkReal x329=((cj0)*(r12));
IkReal x330=((cj1)*(cj3));
IkReal x331=((r02)*(sj0));
IkReal x332=((IkReal(1.00000000000000))*(cj6));
IkReal x333=((cj0)*(r10));
IkReal x334=((cj6)*(r00)*(sj0));
IkReal x335=((cj0)*(r11)*(sj6));
IkReal x336=((cj2)*(sj1)*(sj3));
IkReal x337=((r01)*(sj0)*(sj6));
IkReal x338=((IkReal(1.00000000000000))*(x336));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x330)*(x331)))+(((x331)*(x336)))+(((IkReal(-1.00000000000000))*(x329)*(x338)))+(((r22)*(x328)))+(((x329)*(x330))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((x336)*(x337)))+(((x330)*(x334)))+(((x330)*(x335)))+(((IkReal(-1.00000000000000))*(x330)*(x337)))+(((r21)*(sj6)*(x328)))+(((IkReal(-1.00000000000000))*(r20)*(x328)*(x332)))+(((cj6)*(x333)*(x336)))+(((IkReal(-1.00000000000000))*(x330)*(x332)*(x333)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(x332)*(x336)))+(((IkReal(-1.00000000000000))*(x335)*(x338))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(x330)*(x331)))+(((x331)*(x336)))+(((IkReal(-1.00000000000000))*(x329)*(x338)))+(((r22)*(x328)))+(((x329)*(x330)))))), ((gconst2)*(((((x336)*(x337)))+(((x330)*(x334)))+(((x330)*(x335)))+(((IkReal(-1.00000000000000))*(x330)*(x337)))+(((r21)*(sj6)*(x328)))+(((IkReal(-1.00000000000000))*(r20)*(x328)*(x332)))+(((cj6)*(x333)*(x336)))+(((IkReal(-1.00000000000000))*(x330)*(x332)*(x333)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(x332)*(x336)))+(((IkReal(-1.00000000000000))*(x335)*(x338)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x339=IKsin(j5);
IkReal x340=IKcos(j5);
IkReal x341=((IkReal(1.00000000000000))*(sj1));
IkReal x342=((cj2)*(sj3));
IkReal x343=((IkReal(1.00000000000000))*(cj0));
IkReal x344=((sj6)*(x339));
IkReal x345=((IkReal(1.00000000000000))*(sj0)*(x340));
IkReal x346=((cj6)*(r10)*(x339));
IkReal x347=((IkReal(1.00000000000000))*(cj6)*(r00)*(x339));
evalcond[0]=((((r21)*(x344)))+(((IkReal(-1.00000000000000))*(cj6)*(r20)*(x339)))+(((IkReal(-1.00000000000000))*(x341)*(x342)))+(((IkReal(-1.00000000000000))*(r22)*(x340)))+(((cj1)*(cj3))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x345)))+(((sj2)*(sj3)))+(((cj0)*(r12)*(x340)))+(((IkReal(-1.00000000000000))*(sj0)*(x347)))+(((r01)*(sj0)*(x344)))+(((cj0)*(x346)))+(((IkReal(-1.00000000000000))*(r11)*(x343)*(x344))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj3)*(x341)))+(((IkReal(-1.00000000000000))*(cj1)*(x342)))+(((cj0)*(r01)*(x344)))+(((IkReal(-1.00000000000000))*(r12)*(x345)))+(((IkReal(-1.00000000000000))*(r02)*(x340)*(x343)))+(((IkReal(-1.00000000000000))*(cj6)*(r00)*(x339)*(x343)))+(((IkReal(-1.00000000000000))*(sj0)*(x346)))+(((r11)*(sj0)*(x344))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x348=((IkReal(1.00000000000000))*(cj3)*(sj1));
gconst3=IKsign(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x348)*((cj2)*(cj2))))+(((IkReal(-1.00000000000000))*(x348)*((sj2)*(sj2))))));
IkReal x349=((IkReal(1.00000000000000))*(cj3)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(x349)*((cj2)*(cj2))))+(((IkReal(-1.00000000000000))*(x349)*((sj2)*(sj2)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x350=((cj2)*(cj3));
IkReal x351=((cj6)*(r21));
IkReal x352=((r10)*(sj6));
IkReal x353=((r20)*(sj6));
IkReal x354=((cj3)*(sj2));
IkReal x355=((IkReal(1.00000000000000))*(sj2));
IkReal x356=((cj0)*(sj1));
IkReal x357=((cj1)*(sj0)*(sj3));
IkReal x358=((cj6)*(r11)*(x356));
IkReal x359=((r00)*(sj0)*(sj1)*(sj6));
IkReal x360=((IkReal(1.00000000000000))*(cj0)*(cj1)*(sj3));
IkReal x361=((cj6)*(r01)*(sj0)*(sj1));
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x355)*(x361)))+(((sj2)*(x358)))+(((sj2)*(x352)*(x356)))+(((cj2)*(x353)))+(((cj2)*(x351)))+(((IkReal(-1.00000000000000))*(x355)*(x359))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((x350)*(x361)))+(((cj6)*(r01)*(x357)))+(((IkReal(-1.00000000000000))*(x352)*(x360)))+(((r00)*(sj6)*(x357)))+(((x353)*(x354)))+(((x351)*(x354)))+(((IkReal(-1.00000000000000))*(x350)*(x358)))+(((IkReal(-1.00000000000000))*(x350)*(x352)*(x356)))+(((x350)*(x359)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x360))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(x355)*(x361)))+(((sj2)*(x358)))+(((sj2)*(x352)*(x356)))+(((cj2)*(x353)))+(((cj2)*(x351)))+(((IkReal(-1.00000000000000))*(x355)*(x359)))))), ((gconst3)*(((((x350)*(x361)))+(((cj6)*(r01)*(x357)))+(((IkReal(-1.00000000000000))*(x352)*(x360)))+(((r00)*(sj6)*(x357)))+(((x353)*(x354)))+(((x351)*(x354)))+(((IkReal(-1.00000000000000))*(x350)*(x358)))+(((IkReal(-1.00000000000000))*(x350)*(x352)*(x356)))+(((x350)*(x359)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x360)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x362=IKsin(j4);
IkReal x363=IKcos(j4);
IkReal x364=((cj0)*(cj6));
IkReal x365=((IkReal(1.00000000000000))*(r01));
IkReal x366=((IkReal(1.00000000000000))*(sj1));
IkReal x367=((IkReal(1.00000000000000))*(sj0));
IkReal x368=((r00)*(sj6));
IkReal x369=((cj5)*(r10));
IkReal x370=((cj0)*(sj6));
IkReal x371=((IkReal(1.00000000000000))*(cj5));
IkReal x372=((r02)*(sj5));
IkReal x373=((IkReal(1.00000000000000))*(cj1));
IkReal x374=((cj5)*(r01));
IkReal x375=((IkReal(1.00000000000000))*(cj2));
IkReal x376=((IkReal(1.00000000000000))*(cj0));
IkReal x377=((sj0)*(sj6));
IkReal x378=((r12)*(sj5));
IkReal x379=((sj3)*(x363));
IkReal x380=((sj2)*(x362));
IkReal x381=((sj3)*(x362));
IkReal x382=((sj2)*(x363));
IkReal x383=((cj2)*(cj3)*(x362));
IkReal x384=((cj2)*(cj3)*(x363));
evalcond[0]=((((IkReal(-1.00000000000000))*(x373)*(x381)))+(((IkReal(-1.00000000000000))*(cj6)*(r21)))+(((IkReal(-1.00000000000000))*(x366)*(x383)))+(((IkReal(-1.00000000000000))*(x366)*(x382)))+(((IkReal(-1.00000000000000))*(r20)*(sj6))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x363)*(x375)))+(((cj3)*(x380)))+(((IkReal(-1.00000000000000))*(cj6)*(sj0)*(x365)))+(((IkReal(-1.00000000000000))*(x367)*(x368)))+(((r11)*(x364)))+(((r10)*(x370))));
evalcond[2]=((((cj5)*(r21)*(sj6)))+(((r22)*(sj5)))+(((IkReal(-1.00000000000000))*(cj6)*(r20)*(x371)))+(((cj1)*(x379)))+(((sj1)*(x384)))+(((IkReal(-1.00000000000000))*(x366)*(x380))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x373)*(x382)))+(((IkReal(-1.00000000000000))*(x373)*(x383)))+(((IkReal(-1.00000000000000))*(x368)*(x376)))+(((IkReal(-1.00000000000000))*(r10)*(sj6)*(x367)))+(((IkReal(-1.00000000000000))*(x364)*(x365)))+(((sj1)*(x381)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x367))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x362)*(x375)))+(((x364)*(x369)))+(((sj0)*(x372)))+(((IkReal(-1.00000000000000))*(cj3)*(x382)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(r00)*(x367)))+(((IkReal(-1.00000000000000))*(x376)*(x378)))+(((IkReal(-1.00000000000000))*(r11)*(x370)*(x371)))+(((x374)*(x377))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x373)*(x380)))+(((x370)*(x374)))+(((cj1)*(x384)))+(((IkReal(-1.00000000000000))*(r00)*(x364)*(x371)))+(((sj0)*(x378)))+(((cj5)*(r11)*(x377)))+(((IkReal(-1.00000000000000))*(x366)*(x379)))+(((IkReal(-1.00000000000000))*(cj6)*(x367)*(x369)))+(((cj0)*(x372))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x385=((cj2)*(cj3));
IkReal x386=((cj6)*(r21));
IkReal x387=((r10)*(sj6));
IkReal x388=((r20)*(sj6));
IkReal x389=((cj3)*(sj2));
IkReal x390=((IkReal(1.00000000000000))*(sj2));
IkReal x391=((cj0)*(sj1));
IkReal x392=((cj1)*(sj0)*(sj3));
IkReal x393=((cj6)*(r11)*(x391));
IkReal x394=((r00)*(sj0)*(sj1)*(sj6));
IkReal x395=((IkReal(1.00000000000000))*(cj0)*(cj1)*(sj3));
IkReal x396=((cj6)*(r01)*(sj0)*(sj1));
if( IKabs(((gconst1)*(((((cj2)*(x386)))+(((cj2)*(x388)))+(((sj2)*(x393)))+(((sj2)*(x387)*(x391)))+(((IkReal(-1.00000000000000))*(x390)*(x396)))+(((IkReal(-1.00000000000000))*(x390)*(x394))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x387)*(x395)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x395)))+(((x385)*(x396)))+(((x385)*(x394)))+(((x386)*(x389)))+(((cj6)*(r01)*(x392)))+(((r00)*(sj6)*(x392)))+(((x388)*(x389)))+(((IkReal(-1.00000000000000))*(x385)*(x393)))+(((IkReal(-1.00000000000000))*(x385)*(x387)*(x391))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((((cj2)*(x386)))+(((cj2)*(x388)))+(((sj2)*(x393)))+(((sj2)*(x387)*(x391)))+(((IkReal(-1.00000000000000))*(x390)*(x396)))+(((IkReal(-1.00000000000000))*(x390)*(x394)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x387)*(x395)))+(((IkReal(-1.00000000000000))*(cj6)*(r11)*(x395)))+(((x385)*(x396)))+(((x385)*(x394)))+(((x386)*(x389)))+(((cj6)*(r01)*(x392)))+(((r00)*(sj6)*(x392)))+(((x388)*(x389)))+(((IkReal(-1.00000000000000))*(x385)*(x393)))+(((IkReal(-1.00000000000000))*(x385)*(x387)*(x391)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x397=IKsin(j4);
IkReal x398=IKcos(j4);
IkReal x399=((IkReal(1.00000000000000))*(cj1));
IkReal x400=((cj0)*(cj6));
IkReal x401=((IkReal(1.00000000000000))*(r01));
IkReal x402=((IkReal(1.00000000000000))*(sj6));
IkReal x403=((IkReal(1.00000000000000))*(cj6));
IkReal x404=((sj3)*(x397));
IkReal x405=((cj3)*(x397));
IkReal x406=((IkReal(1.00000000000000))*(x398));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x405)))+(((IkReal(-1.00000000000000))*(r20)*(x402)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x406)))+(((IkReal(-1.00000000000000))*(r21)*(x403)))+(((IkReal(-1.00000000000000))*(x399)*(x404))));
evalcond[1]=((((r11)*(x400)))+(((IkReal(-1.00000000000000))*(cj2)*(x406)))+(((sj2)*(x405)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(x402)))+(((cj0)*(r10)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(sj0)*(x401))));
evalcond[2]=((((sj1)*(x404)))+(((IkReal(-1.00000000000000))*(sj2)*(x398)*(x399)))+(((IkReal(-1.00000000000000))*(cj2)*(x399)*(x405)))+(((IkReal(-1.00000000000000))*(x400)*(x401)))+(((IkReal(-1.00000000000000))*(r11)*(sj0)*(x403)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x402)))+(((IkReal(-1.00000000000000))*(r10)*(sj0)*(x402))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(2.00000000000000))*(cj6)*(r20)*(r21)*(sj6)))+(((IkReal(-1.00000000000000))*((r21)*(r21))*((sj6)*(sj6))))+(((IkReal(-1.00000000000000))*((r22)*(r22))))+(((IkReal(-1.00000000000000))*((cj6)*(cj6))*((r20)*(r20))))));
dummyeval[0]=((((IkReal(2.00000000000000))*(cj6)*(r20)*(r21)*(sj6)))+(((IkReal(-1.00000000000000))*((r21)*(r21))*((sj6)*(sj6))))+(((IkReal(-1.00000000000000))*((r22)*(r22))))+(((IkReal(-1.00000000000000))*((cj6)*(cj6))*((r20)*(r20)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x407=((IkReal(1.00000000000000))*(sj1));
IkReal x408=((cj6)*(r20));
IkReal x409=((sj2)*(sj4));
IkReal x410=((r21)*(sj6));
IkReal x411=((cj2)*(sj3));
IkReal x412=((r22)*(sj1));
IkReal x413=((cj2)*(cj3)*(cj4));
IkReal x414=((IkReal(1.00000000000000))*(cj1)*(cj3));
IkReal x415=((cj1)*(cj4)*(sj3));
if( IKabs(((gconst4)*(((((sj1)*(x408)*(x411)))+(((cj1)*(cj3)*(x410)))+(((r22)*(x415)))+(((x412)*(x413)))+(((IkReal(-1.00000000000000))*(x408)*(x414)))+(((IkReal(-1.00000000000000))*(x407)*(x410)*(x411)))+(((IkReal(-1.00000000000000))*(r22)*(x407)*(x409))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x407)*(x408)*(x413)))+(((sj1)*(x408)*(x409)))+(((sj1)*(x410)*(x413)))+(((x411)*(x412)))+(((x410)*(x415)))+(((IkReal(-1.00000000000000))*(x407)*(x409)*(x410)))+(((IkReal(-1.00000000000000))*(x408)*(x415)))+(((IkReal(-1.00000000000000))*(r22)*(x414))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst4)*(((((sj1)*(x408)*(x411)))+(((cj1)*(cj3)*(x410)))+(((r22)*(x415)))+(((x412)*(x413)))+(((IkReal(-1.00000000000000))*(x408)*(x414)))+(((IkReal(-1.00000000000000))*(x407)*(x410)*(x411)))+(((IkReal(-1.00000000000000))*(r22)*(x407)*(x409)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x407)*(x408)*(x413)))+(((sj1)*(x408)*(x409)))+(((sj1)*(x410)*(x413)))+(((x411)*(x412)))+(((x410)*(x415)))+(((IkReal(-1.00000000000000))*(x407)*(x409)*(x410)))+(((IkReal(-1.00000000000000))*(x408)*(x415)))+(((IkReal(-1.00000000000000))*(r22)*(x414)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x416=IKsin(j5);
IkReal x417=IKcos(j5);
IkReal x418=((IkReal(1.00000000000000))*(cj6));
IkReal x419=((r01)*(sj6));
IkReal x420=((r11)*(sj0));
IkReal x421=((IkReal(1.00000000000000))*(r12));
IkReal x422=((cj6)*(r10));
IkReal x423=((cj3)*(sj1));
IkReal x424=((cj1)*(cj4));
IkReal x425=((IkReal(1.00000000000000))*(r11));
IkReal x426=((IkReal(1.00000000000000))*(cj4));
IkReal x427=((cj0)*(x416));
IkReal x428=((sj6)*(x416));
IkReal x429=((IkReal(1.00000000000000))*(cj2)*(sj3));
IkReal x430=((sj6)*(x417));
IkReal x431=((sj0)*(x416));
IkReal x432=((sj0)*(x417));
IkReal x433=((IkReal(1.00000000000000))*(sj2)*(sj4));
IkReal x434=((cj0)*(x417));
IkReal x435=((IkReal(1.00000000000000))*(r02)*(x417));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj1)*(x429)))+(((r21)*(x428)))+(((IkReal(-1.00000000000000))*(r20)*(x416)*(x418)))+(((IkReal(-1.00000000000000))*(r22)*(x417)))+(((cj1)*(cj3))));
evalcond[1]=((((cj2)*(cj4)*(x423)))+(((IkReal(-1.00000000000000))*(sj1)*(x433)))+(((r22)*(x416)))+(((r21)*(x430)))+(((sj3)*(x424)))+(((IkReal(-1.00000000000000))*(r20)*(x417)*(x418))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x432)))+(((r12)*(x434)))+(((sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(sj6)*(x425)*(x427)))+(((x419)*(x431)))+(((IkReal(-1.00000000000000))*(r00)*(x418)*(x431)))+(((x422)*(x427))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x421)*(x427)))+(((IkReal(-1.00000000000000))*(cj3)*(sj2)*(x426)))+(((r02)*(x431)))+(((IkReal(-1.00000000000000))*(cj2)*(sj4)))+(((IkReal(-1.00000000000000))*(cj0)*(x425)*(x430)))+(((x419)*(x432)))+(((IkReal(-1.00000000000000))*(r00)*(x418)*(x432)))+(((x422)*(x434))));
evalcond[4]=((((IkReal(-1.00000000000000))*(r02)*(x434)))+(((x420)*(x428)))+(((IkReal(-1.00000000000000))*(x423)))+(((IkReal(-1.00000000000000))*(cj1)*(x429)))+(((IkReal(-1.00000000000000))*(x421)*(x432)))+(((IkReal(-1.00000000000000))*(r10)*(x418)*(x431)))+(((x419)*(x427)))+(((IkReal(-1.00000000000000))*(r00)*(x418)*(x427))));
evalcond[5]=((((r12)*(x431)))+(((IkReal(-1.00000000000000))*(cj1)*(x433)))+(((IkReal(-1.00000000000000))*(r10)*(x418)*(x432)))+(((r02)*(x427)))+(((x419)*(x434)))+(((cj2)*(cj3)*(x424)))+(((IkReal(-1.00000000000000))*(r00)*(x418)*(x434)))+(((x420)*(x430)))+(((IkReal(-1.00000000000000))*(sj1)*(sj3)*(x426))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
    }

}

}
}
}

}

}
}
}

}

}
}
}

}

}
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots6(IkReal rawcoeffs[6+1], IkReal rawroots[6], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[6];
    const int maxsteps = 110;
    for(int i = 0; i < 6; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[6];
    IkReal err[6];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 6; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 6; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 6; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 6; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[6] = {false};
    for(int i = 0; i < 6; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 6; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "bcb25302dbec7575df10994dd89abd19"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
