/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 06:59:21.690565
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[3]);
x3=IKcos(j[3]);
x4=IKsin(j[1]);
x5=IKcos(j[4]);
x6=IKsin(j[2]);
x7=IKsin(j[4]);
x8=IKcos(j[6]);
x9=IKcos(j[5]);
x10=IKsin(j[5]);
x11=IKsin(j[6]);
x12=IKsin(j[0]);
x13=IKcos(j[0]);
x14=((IkReal(0.0450000000000000))*(x6));
x15=((IkReal(0.0450000000000000))*(x1));
x16=((IkReal(1.00000000000000))*(x11));
x17=((IkReal(1.00000000000000))*(x8));
x18=((IkReal(1.00000000000000))*(x10));
x19=((IkReal(1.00000000000000))*(x9));
x20=((IkReal(0.300000000000000))*(x1));
x21=((IkReal(1.00000000000000))*(x1));
x22=((IkReal(0.0600000000000000))*(x1));
x23=((x5)*(x6));
x24=((x0)*(x2));
x25=((x1)*(x7));
x26=((x2)*(x6));
x27=((x0)*(x3));
x28=((x2)*(x4));
x29=((x3)*(x4));
x30=((IkReal(1.00000000000000))*(x28));
x31=((IkReal(0.300000000000000))*(x26));
x32=((x14)*(x3));
x33=((IkReal(1.00000000000000))*(x6)*(x7));
x34=((IkReal(0.0600000000000000))*(x6)*(x7));
x35=((x18)*(x26));
x36=((IkReal(0.0600000000000000))*(x26)*(x9));
x37=((x33)*(x4));
x38=((((x1)*(x5)))+(((IkReal(-1.00000000000000))*(x3)*(x33))));
x39=((((x23)*(x3)))+(x25));
x40=((((IkReal(-1.00000000000000))*(x30)))+(((x1)*(x27))));
x41=((((IkReal(-1.00000000000000))*(x21)*(x28)))+(x27));
x42=((((x1)*(x29)))+(x24));
x43=((((IkReal(-1.00000000000000))*(x21)*(x24)))+(((IkReal(-1.00000000000000))*(x29))));
x44=((((x21)*(x24)))+(((IkReal(1.00000000000000))*(x29))));
x45=((((x22)*(x7)))+(((IkReal(0.0600000000000000))*(x23)*(x3))));
x46=((x42)*(x5));
x47=((x40)*(x5));
x48=((x31)+(x36)+(x14));
x49=((((IkReal(-1.00000000000000))*(x0)*(x33)))+(x47));
x50=((((IkReal(-1.00000000000000))*(x35)))+(((x39)*(x9))));
x51=((((IkReal(-1.00000000000000))*(x37)))+(x46));
x52=((((IkReal(-1.00000000000000))*(x0)*(x23)))+(((x7)*(((((IkReal(-1.00000000000000))*(x21)*(x27)))+(x30))))));
x53=((((IkReal(-1.00000000000000))*(x19)*(x39)))+(x35));
x54=((x51)*(x9));
x55=((((IkReal(1.00000000000000))*(x23)*(x4)))+(((x7)*(((((IkReal(1.00000000000000))*(x24)))+(((IkReal(1.00000000000000))*(x21)*(x29))))))));
x56=((x11)*(x53));
x57=((((x10)*(x49)))+(((x44)*(x9))));
x58=((((x11)*(x52)))+(((x8)*(((((x10)*(x43)))+(((x49)*(x9))))))));
x59=((((x11)*(((((IkReal(-1.00000000000000))*(x18)*(x43)))+(((IkReal(-1.00000000000000))*(x19)*(x49)))))))+(((x52)*(x8))));
x60=((((IkReal(-1.00000000000000))*(x15)*(x27)))+(((x0)*(x15)))+(((IkReal(0.300000000000000))*(x29)))+(((x10)*(((((IkReal(-1.00000000000000))*(x0)*(x34)))+(((IkReal(0.0600000000000000))*(x47)))))))+(((x9)*(((((x22)*(x24)))+(((IkReal(0.0600000000000000))*(x29)))))))+(((x20)*(x24)))+(((IkReal(0.0450000000000000))*(x28)))+(((IkReal(0.550000000000000))*(x4))));
eerot[0]=((((x13)*(x58)))+(((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x38)))+(((IkReal(-1.00000000000000))*(x17)*(x50))))))));
eerot[1]=((((x13)*(x59)))+(((x12)*(((((IkReal(-1.00000000000000))*(x17)*(x38)))+(((IkReal(-1.00000000000000))*(x16)*(x53))))))));
eerot[2]=((((x13)*(x57)))+(((x12)*(((((IkReal(-1.00000000000000))*(x18)*(x39)))+(((IkReal(-1.00000000000000))*(x19)*(x26))))))));
eetrans[0]=((IkReal(0.220000000000000))+(((x13)*(x60)))+(((x12)*(((((IkReal(-1.00000000000000))*(x48)))+(((IkReal(-1.00000000000000))*(x18)*(x45)))+(x32))))));
eerot[3]=((((x12)*(x58)))+(((x13)*(((((x50)*(x8)))+(((x11)*(x38))))))));
eerot[4]=((((x12)*(x59)))+(((x13)*(((x56)+(((x38)*(x8))))))));
eerot[5]=((((x12)*(x57)))+(((x13)*(((((x10)*(x39)))+(((x26)*(x9))))))));
eetrans[1]=((IkReal(0.140000000000000))+(((x12)*(x60)))+(((x13)*(((((IkReal(-1.00000000000000))*(x32)))+(((x10)*(x45)))+(x48))))));
eerot[6]=((((x11)*(x55)))+(((x8)*(((((IkReal(-1.00000000000000))*(x18)*(x41)))+(((IkReal(-1.00000000000000))*(x19)*(x51))))))));
eerot[7]=((((x11)*(((((x10)*(x41)))+(x54)))))+(((x55)*(x8))));
eerot[8]=((((x41)*(x9)))+(((x10)*(((((IkReal(-1.00000000000000))*(x46)))+(x37))))));
IkReal x61=((IkReal(1.00000000000000))*(x28));
eetrans[2]=((IkReal(0.346000000000000))+(((x9)*(((((IkReal(0.0600000000000000))*(x27)))+(((IkReal(-1.00000000000000))*(x22)*(x61)))))))+(((IkReal(-1.00000000000000))*(x20)*(x61)))+(((IkReal(0.300000000000000))*(x27)))+(((IkReal(-1.00000000000000))*(x15)*(x4)))+(((x10)*(((((x34)*(x4)))+(((IkReal(-0.0600000000000000))*(x46)))))))+(((IkReal(0.0450000000000000))*(x24)))+(((IkReal(0.550000000000000))*(x0)))+(((x15)*(x29))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j0,cj0,sj0,htj0,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij0[2], _nj0;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1;  _ij0[0] = -1; _ij0[1] = -1; _nj0 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=((((r00)*(IKcos(j0))))+(((r10)*(IKsin(j0)))));
new_r01=((((r01)*(IKcos(j0))))+(((r11)*(IKsin(j0)))));
new_r02=((((r12)*(IKsin(j0))))+(((r02)*(IKcos(j0)))));
IkReal x62=IKcos(j0);
IkReal x63=IKsin(j0);
new_px=((((IkReal(-0.0600000000000000))*(r12)*(x63)))+(((IkReal(-0.0600000000000000))*(r02)*(x62)))+(((IkReal(-0.220000000000000))*(x62)))+(((IkReal(-0.140000000000000))*(x63)))+(((px)*(x62)))+(((py)*(x63))));
new_r10=((IkReal(-1.00000000000000))*(r20));
new_r11=((IkReal(-1.00000000000000))*(r21));
new_r12=((IkReal(-1.00000000000000))*(r22));
new_py=((IkReal(0.346000000000000))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(0.0600000000000000))*(r22))));
new_r20=((((IkReal(-1.00000000000000))*(r00)*(IKsin(j0))))+(((r10)*(IKcos(j0)))));
new_r21=((((IkReal(-1.00000000000000))*(r01)*(IKsin(j0))))+(((r11)*(IKcos(j0)))));
new_r22=((((IkReal(-1.00000000000000))*(r02)*(IKsin(j0))))+(((r12)*(IKcos(j0)))));
IkReal x64=IKsin(j0);
IkReal x65=IKcos(j0);
new_pz=((((IkReal(-0.0600000000000000))*(r12)*(x65)))+(((IkReal(-0.140000000000000))*(x65)))+(((IkReal(-1.00000000000000))*(px)*(x64)))+(((IkReal(0.220000000000000))*(x64)))+(((py)*(x65)))+(((IkReal(0.0600000000000000))*(r02)*(x64))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x66=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j3array[0]=((IkReal(-1.34027003705633))+(((IkReal(-1.00000000000000))*(x66))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(1.80132261653346))+(x66));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((py)*(py)*(py)*(py))+((((px)*(px))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x67=(py)*(py);
if( IKabs(((IkReal(-1.10000000000000))*(x67))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.10000000000000))*(px)*(py))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x68=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.10000000000000))*(x67)), ((IkReal(1.10000000000000))*(px)*(py)))));
if( (((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(-0.00405000000000000))*(cj3)*(py)))+(((IkReal(0.0270000000000000))*(py)*(sj3)))+(((IkReal(-0.208450000000000))*(py))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(-0.00405000000000000))*(cj3)*(py)))+(((IkReal(0.0270000000000000))*(py)*(sj3)))+(((IkReal(-0.208450000000000))*(py))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x69=IKasin(((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*((x67)*(x67))))+(((IkReal(1.21000000000000))*(x67)*((px)*(px))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(-0.00405000000000000))*(cj3)*(py)))+(((IkReal(0.0270000000000000))*(py)*(sj3)))+(((IkReal(-0.208450000000000))*(py)))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x68)))+(((IkReal(-1.00000000000000))*(x69))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(x69)+(((IkReal(-1.00000000000000))*(x68))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x70=IKsin(j1);
IkReal x71=IKcos(j1);
IkReal x72=((py)*(x71));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x72)))+(((px)*(x70))));
evalcond[1]=((((IkReal(1.10000000000000))*(x70)*((px)*(px))))+(((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(-1.10000000000000))*(px)*(x72)))+(((IkReal(0.0270000000000000))*(px)*(sj3)))+(((IkReal(-0.00405000000000000))*(cj3)*(px)))+(((IkReal(-0.208450000000000))*(px))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x73=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((px)*(sj1)))+(((IkReal(-1.00000000000000))*(py)*(x73))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(px)*(x73)))+(((IkReal(-1.00000000000000))*(py)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x74=((py)*(sj1));
IkReal x75=((IkReal(100.000000000000))*(pp));
IkReal x76=((IkReal(110.000000000000))*(pz));
IkReal x77=((cj1)*(px));
IkReal x78=((IkReal(110.000000000000))*(cj1)*(sj1));
IkReal x79=((IkReal(110.000000000000))*(px)*(py));
if( IKabs(((gconst1)*(((((px)*(sj1)*(x76)))+(((IkReal(-21.2500000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x76)))+(((IkReal(-1.00000000000000))*(pz)*(x75))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-110.000000000000))*(cj1)*(py)*(x74)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x77)))+(((IkReal(-1.00000000000000))*(x74)*(x75)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x77)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x74)))+(((IkReal(-1.00000000000000))*(x75)*(x77)))+(((IkReal(-21.2500000000000))*(x77)))+(((IkReal(-21.2500000000000))*(x74))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((px)*(sj1)*(x76)))+(((IkReal(-21.2500000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x76)))+(((IkReal(-1.00000000000000))*(pz)*(x75)))))), ((gconst1)*(((((IkReal(-110.000000000000))*(cj1)*(py)*(x74)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x77)))+(((IkReal(-1.00000000000000))*(x74)*(x75)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x77)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x74)))+(((IkReal(-1.00000000000000))*(x75)*(x77)))+(((IkReal(-21.2500000000000))*(x77)))+(((IkReal(-21.2500000000000))*(x74)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x80=IKsin(j2);
IkReal x81=IKcos(j2);
IkReal x82=((cj1)*(px));
IkReal x83=((IkReal(1.00000000000000))*(pz));
IkReal x84=((py)*(sj1));
IkReal x85=((IkReal(0.0900000000000000))*(x81));
IkReal x86=((IkReal(1.00000000000000))*(x81));
evalcond[0]=((((IkReal(-1.00000000000000))*(x81)*(x83)))+(((x80)*(x82)))+(((x80)*(x84))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x82)*(x86)))+(((IkReal(-1.00000000000000))*(x80)*(x83)))+(((IkReal(-1.00000000000000))*(x84)*(x86))));
evalcond[2]=((IkReal(-0.212500000000000))+(((x84)*(x85)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x82)*(x85)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((IkReal(0.0900000000000000))*(pz)*(x80))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x150=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(py)*(x150)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(px)*(x150)))+(((IkReal(-1.00000000000000))*(py)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x151=((cj1)*(px));
IkReal x152=((py)*(sj1));
IkReal x153=((IkReal(100.000000000000))*(pp));
IkReal x154=((IkReal(110.000000000000))*(sj1));
IkReal x155=((IkReal(110.000000000000))*(py));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x153)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x155)))+(((IkReal(-21.2500000000000))*(pz)))+(((px)*(pz)*(x154))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(x151)*(x155)))+(((IkReal(-1.00000000000000))*(x151)*(x153)))+(((IkReal(-21.2500000000000))*(x151)))+(((IkReal(-21.2500000000000))*(x152)))+(((px)*(x151)*(x154)))+(((px)*(x152)*(x154)))+(((IkReal(-1.00000000000000))*(x152)*(x153)))+(((IkReal(-1.00000000000000))*(cj1)*(x152)*(x155))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x153)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x155)))+(((IkReal(-21.2500000000000))*(pz)))+(((px)*(pz)*(x154)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(x151)*(x155)))+(((IkReal(-1.00000000000000))*(x151)*(x153)))+(((IkReal(-21.2500000000000))*(x151)))+(((IkReal(-21.2500000000000))*(x152)))+(((px)*(x151)*(x154)))+(((px)*(x152)*(x154)))+(((IkReal(-1.00000000000000))*(x152)*(x153)))+(((IkReal(-1.00000000000000))*(cj1)*(x152)*(x155)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x156=IKsin(j2);
IkReal x157=IKcos(j2);
IkReal x158=((cj1)*(px));
IkReal x159=((IkReal(1.00000000000000))*(pz));
IkReal x160=((py)*(sj1));
IkReal x161=((IkReal(0.0900000000000000))*(x157));
IkReal x162=((IkReal(1.00000000000000))*(x157));
evalcond[0]=((((IkReal(-1.00000000000000))*(x157)*(x159)))+(((x156)*(x160)))+(((x156)*(x158))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x158)*(x162)))+(((IkReal(-1.00000000000000))*(x160)*(x162)))+(((IkReal(-1.00000000000000))*(x156)*(x159))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x158)*(x161)))+(((x160)*(x161)))+(((IkReal(0.0900000000000000))*(pz)*(x156))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((IkReal(-1.00000000000000))*(cj1)*(py)))+(((px)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((IkReal(1111111111.11111))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1111111111.11111))*(pz)))+IKsqr(((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((IkReal(1111111111.11111))*(pz)), ((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x163=IKsin(j2);
IkReal x164=IKcos(j2);
IkReal x165=((cj1)*(px));
IkReal x166=((IkReal(1.00000000000000))*(pz));
IkReal x167=((py)*(sj1));
IkReal x168=((IkReal(0.0900000000000000))*(x164));
IkReal x169=((IkReal(1.00000000000000))*(x164));
evalcond[0]=((((x163)*(x167)))+(((x163)*(x165)))+(((IkReal(-1.00000000000000))*(x164)*(x166))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x166)))+(((IkReal(9.00000000000000e-10))*(x163))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x167)))+(((IkReal(-1.00000000000000))*(x165)))+(((IkReal(9.00000000000000e-10))*(x164))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x163)*(x166)))+(((IkReal(-1.00000000000000))*(x165)*(x169)))+(((IkReal(-1.00000000000000))*(x167)*(x169))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x163)))+(((IkReal(-1.00000000000000))*(pp)))+(((x167)*(x168)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x165)*(x168))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(px)))+(((py)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj1)*(px)))+(((py)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x170=IKsin(j2);
IkReal x171=IKcos(j2);
IkReal x172=((cj1)*(px));
IkReal x173=((py)*(sj1));
IkReal x174=((IkReal(0.0450000000000000))*(cj3));
IkReal x175=((IkReal(1.00000000000000))*(pz));
IkReal x176=((IkReal(0.300000000000000))*(sj3));
IkReal x177=((IkReal(0.0900000000000000))*(x171));
IkReal x178=((IkReal(1.00000000000000))*(x171));
evalcond[0]=((((IkReal(-1.00000000000000))*(x171)*(x175)))+(((x170)*(x173)))+(((x170)*(x172))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x175)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((IkReal(0.0450000000000000))*(x170)))+(((x170)*(x176))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x173)))+(((IkReal(-1.00000000000000))*(x172)))+(((x171)*(x176)))+(((IkReal(0.0450000000000000))*(x171)))+(((IkReal(-1.00000000000000))*(x171)*(x174))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x174)))+(x176)+(((IkReal(-1.00000000000000))*(x170)*(x175)))+(((IkReal(-1.00000000000000))*(x172)*(x178)))+(((IkReal(-1.00000000000000))*(x173)*(x178))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(pz)*(x170)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x172)*(x177)))+(((x173)*(x177)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x179=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x180=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x179)))+(((IkReal(-1.00000000000000))*(x180))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x179)))+(((IkReal(1.00000000000000))*(x180))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x181=IKsin(j1);
IkReal x182=IKcos(j1);
IkReal x183=((IkReal(0.0270000000000000))*(sj3));
IkReal x184=((IkReal(0.00405000000000000))*(cj3));
IkReal x185=((IkReal(1.00000000000000))*(pp));
IkReal x186=((IkReal(1.10000000000000))*(px)*(py));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x184)))+(((IkReal(-1.00000000000000))*(py)*(x185)))+(((IkReal(-1.10000000000000))*(x182)*((py)*(py))))+(((py)*(x183)))+(((x181)*(x186)))+(((IkReal(-0.208450000000000))*(py))));
evalcond[1]=((((IkReal(-1.00000000000000))*(px)*(x185)))+(((IkReal(-1.00000000000000))*(px)*(x184)))+(((px)*(x183)))+(((IkReal(1.10000000000000))*(x181)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x182)*(x186)))+(((IkReal(-0.208450000000000))*(px))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x187=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(py)*(x187)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(px)*(x187))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x188=((py)*(sj1));
IkReal x189=((IkReal(100.000000000000))*(pp));
IkReal x190=((IkReal(110.000000000000))*(pz));
IkReal x191=((cj1)*(px));
IkReal x192=((IkReal(110.000000000000))*(cj1)*(sj1));
IkReal x193=((IkReal(110.000000000000))*(px)*(py));
if( IKabs(((gconst1)*(((((px)*(sj1)*(x190)))+(((IkReal(-1.00000000000000))*(pz)*(x189)))+(((IkReal(-21.2500000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x190))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x188)*(x189)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x191)))+(((IkReal(-1.00000000000000))*(x189)*(x191)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x188)))+(((IkReal(-21.2500000000000))*(x191)))+(((IkReal(-21.2500000000000))*(x188)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x191)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x188))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((px)*(sj1)*(x190)))+(((IkReal(-1.00000000000000))*(pz)*(x189)))+(((IkReal(-21.2500000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x190)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x188)*(x189)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x191)))+(((IkReal(-1.00000000000000))*(x189)*(x191)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x188)))+(((IkReal(-21.2500000000000))*(x191)))+(((IkReal(-21.2500000000000))*(x188)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x191)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x188)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x194=IKsin(j2);
IkReal x195=IKcos(j2);
IkReal x196=((cj1)*(px));
IkReal x197=((IkReal(1.00000000000000))*(pz));
IkReal x198=((py)*(sj1));
IkReal x199=((IkReal(0.0900000000000000))*(x195));
IkReal x200=((IkReal(1.00000000000000))*(x195));
evalcond[0]=((((IkReal(-1.00000000000000))*(x195)*(x197)))+(((x194)*(x198)))+(((x194)*(x196))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x194)*(x197)))+(((IkReal(-1.00000000000000))*(x198)*(x200)))+(((IkReal(-1.00000000000000))*(x196)*(x200))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x198)*(x199)))+(((x196)*(x199)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((IkReal(0.0900000000000000))*(pz)*(x194))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x201=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(py)*(x201)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(px)*(x201)))+(((IkReal(-1.00000000000000))*(py)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x202=((cj1)*(px));
IkReal x203=((py)*(sj1));
IkReal x204=((IkReal(100.000000000000))*(pp));
IkReal x205=((IkReal(110.000000000000))*(sj1));
IkReal x206=((IkReal(110.000000000000))*(py));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x204)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x206)))+(((IkReal(-21.2500000000000))*(pz)))+(((px)*(pz)*(x205))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(x202)*(x206)))+(((IkReal(-1.00000000000000))*(x203)*(x204)))+(((IkReal(-1.00000000000000))*(x202)*(x204)))+(((px)*(x202)*(x205)))+(((IkReal(-1.00000000000000))*(cj1)*(x203)*(x206)))+(((IkReal(-21.2500000000000))*(x202)))+(((IkReal(-21.2500000000000))*(x203)))+(((px)*(x203)*(x205))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x204)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x206)))+(((IkReal(-21.2500000000000))*(pz)))+(((px)*(pz)*(x205)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(cj1)*(x202)*(x206)))+(((IkReal(-1.00000000000000))*(x203)*(x204)))+(((IkReal(-1.00000000000000))*(x202)*(x204)))+(((px)*(x202)*(x205)))+(((IkReal(-1.00000000000000))*(cj1)*(x203)*(x206)))+(((IkReal(-21.2500000000000))*(x202)))+(((IkReal(-21.2500000000000))*(x203)))+(((px)*(x203)*(x205)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x207=IKsin(j2);
IkReal x208=IKcos(j2);
IkReal x209=((cj1)*(px));
IkReal x210=((IkReal(1.00000000000000))*(pz));
IkReal x211=((py)*(sj1));
IkReal x212=((IkReal(0.0900000000000000))*(x208));
IkReal x213=((IkReal(1.00000000000000))*(x208));
evalcond[0]=((((x207)*(x211)))+(((IkReal(-1.00000000000000))*(x208)*(x210)))+(((x207)*(x209))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x207)*(x210)))+(((IkReal(-1.00000000000000))*(x211)*(x213)))+(((IkReal(-1.00000000000000))*(x209)*(x213))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x211)*(x212)))+(((x209)*(x212)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x207)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((IkReal(-1.00000000000000))*(cj1)*(py)))+(((px)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((IkReal(1111111111.11111))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1111111111.11111))*(pz)))+IKsqr(((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((IkReal(1111111111.11111))*(pz)), ((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x214=IKsin(j2);
IkReal x215=IKcos(j2);
IkReal x216=((cj1)*(px));
IkReal x217=((IkReal(1.00000000000000))*(pz));
IkReal x218=((py)*(sj1));
IkReal x219=((IkReal(0.0900000000000000))*(x215));
IkReal x220=((IkReal(1.00000000000000))*(x215));
evalcond[0]=((((x214)*(x218)))+(((x214)*(x216)))+(((IkReal(-1.00000000000000))*(x215)*(x217))));
evalcond[1]=((((IkReal(9.00000000000000e-10))*(x214)))+(((IkReal(-1.00000000000000))*(x217))));
evalcond[2]=((((IkReal(9.00000000000000e-10))*(x215)))+(((IkReal(-1.00000000000000))*(x218)))+(((IkReal(-1.00000000000000))*(x216))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x216)*(x220)))+(((IkReal(-1.00000000000000))*(x218)*(x220)))+(((IkReal(-1.00000000000000))*(x214)*(x217))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x218)*(x219)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((x216)*(x219)))+(((IkReal(0.0900000000000000))*(pz)*(x214)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(px)))+(((py)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj1)*(px)))+(((py)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x221=IKsin(j2);
IkReal x222=IKcos(j2);
IkReal x223=((cj1)*(px));
IkReal x224=((py)*(sj1));
IkReal x225=((IkReal(0.0450000000000000))*(cj3));
IkReal x226=((IkReal(1.00000000000000))*(pz));
IkReal x227=((IkReal(0.300000000000000))*(sj3));
IkReal x228=((IkReal(0.0900000000000000))*(x222));
IkReal x229=((IkReal(1.00000000000000))*(x222));
evalcond[0]=((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((x221)*(x223)))+(((x221)*(x224))));
evalcond[1]=((((IkReal(0.0450000000000000))*(x221)))+(((x221)*(x227)))+(((IkReal(-1.00000000000000))*(x221)*(x225)))+(((IkReal(-1.00000000000000))*(x226))));
evalcond[2]=((((x222)*(x227)))+(((IkReal(-1.00000000000000))*(x222)*(x225)))+(((IkReal(0.0450000000000000))*(x222)))+(((IkReal(-1.00000000000000))*(x223)))+(((IkReal(-1.00000000000000))*(x224))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x224)*(x229)))+(x227)+(((IkReal(-1.00000000000000))*(x223)*(x229)))+(((IkReal(-1.00000000000000))*(x221)*(x226)))+(((IkReal(-1.00000000000000))*(x225))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x224)*(x228)))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x221)))+(((x223)*(x228)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x230=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x231=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x230)))+(((IkReal(-1.00000000000000))*(x231))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x230)))+(((IkReal(1.00000000000000))*(x231))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x232=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(py)*(x232)))+(((px)*(sj1))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(px)*(x232)))+(((IkReal(-1.00000000000000))*(py)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x233=((py)*(sj1));
IkReal x234=((IkReal(100.000000000000))*(pp));
IkReal x235=((IkReal(110.000000000000))*(pz));
IkReal x236=((cj1)*(px));
IkReal x237=((IkReal(110.000000000000))*(cj1)*(sj1));
IkReal x238=((IkReal(110.000000000000))*(px)*(py));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(pz)*(x234)))+(((px)*(sj1)*(x235)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x235)))+(((IkReal(-21.2500000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x233)*(x234)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x236)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x233)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x233)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x236)))+(((IkReal(-1.00000000000000))*(x234)*(x236)))+(((IkReal(-21.2500000000000))*(x236)))+(((IkReal(-21.2500000000000))*(x233))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(pz)*(x234)))+(((px)*(sj1)*(x235)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x235)))+(((IkReal(-21.2500000000000))*(pz)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x233)*(x234)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x236)))+(((IkReal(-110.000000000000))*(cj1)*(py)*(x233)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x233)))+(((IkReal(110.000000000000))*(px)*(sj1)*(x236)))+(((IkReal(-1.00000000000000))*(x234)*(x236)))+(((IkReal(-21.2500000000000))*(x236)))+(((IkReal(-21.2500000000000))*(x233)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x239=IKsin(j2);
IkReal x240=IKcos(j2);
IkReal x241=((cj1)*(px));
IkReal x242=((IkReal(1.00000000000000))*(pz));
IkReal x243=((py)*(sj1));
IkReal x244=((IkReal(0.0900000000000000))*(x240));
IkReal x245=((IkReal(1.00000000000000))*(x240));
evalcond[0]=((((x239)*(x243)))+(((x239)*(x241)))+(((IkReal(-1.00000000000000))*(x240)*(x242))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x243)*(x245)))+(((IkReal(-1.00000000000000))*(x241)*(x245)))+(((IkReal(-1.00000000000000))*(x239)*(x242))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x239)))+(((x241)*(x244)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x243)*(x244))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x246=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((px)*(sj1)))+(((IkReal(-1.00000000000000))*(py)*(x246))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(px)*(x246))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-18.0000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj1)*(sj1))))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((cj1)*(cj1))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj1)*(cj1))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x247=((cj1)*(px));
IkReal x248=((py)*(sj1));
IkReal x249=((IkReal(100.000000000000))*(pp));
IkReal x250=((IkReal(110.000000000000))*(sj1));
IkReal x251=((IkReal(110.000000000000))*(py));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x249)))+(((px)*(pz)*(x250)))+(((IkReal(-21.2500000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x251))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x248)*(x249)))+(((IkReal(-21.2500000000000))*(x247)))+(((IkReal(-21.2500000000000))*(x248)))+(((px)*(x247)*(x250)))+(((IkReal(-1.00000000000000))*(cj1)*(x247)*(x251)))+(((px)*(x248)*(x250)))+(((IkReal(-1.00000000000000))*(cj1)*(x248)*(x251)))+(((IkReal(-1.00000000000000))*(x247)*(x249))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(pz)*(x249)))+(((px)*(pz)*(x250)))+(((IkReal(-21.2500000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x251)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x248)*(x249)))+(((IkReal(-21.2500000000000))*(x247)))+(((IkReal(-21.2500000000000))*(x248)))+(((px)*(x247)*(x250)))+(((IkReal(-1.00000000000000))*(cj1)*(x247)*(x251)))+(((px)*(x248)*(x250)))+(((IkReal(-1.00000000000000))*(cj1)*(x248)*(x251)))+(((IkReal(-1.00000000000000))*(x247)*(x249)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x252=IKsin(j2);
IkReal x253=IKcos(j2);
IkReal x254=((cj1)*(px));
IkReal x255=((IkReal(1.00000000000000))*(pz));
IkReal x256=((py)*(sj1));
IkReal x257=((IkReal(0.0900000000000000))*(x253));
IkReal x258=((IkReal(1.00000000000000))*(x253));
evalcond[0]=((((x252)*(x256)))+(((x252)*(x254)))+(((IkReal(-1.00000000000000))*(x253)*(x255))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x252)*(x255)))+(((IkReal(-1.00000000000000))*(x256)*(x258)))+(((IkReal(-1.00000000000000))*(x254)*(x258))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x252)))+(((x256)*(x257)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x254)*(x257))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((IkReal(-1.00000000000000))*(cj1)*(py)))+(((px)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((IkReal(1111111111.11111))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1111111111.11111))*(pz)))+IKsqr(((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((IkReal(1111111111.11111))*(pz)), ((((IkReal(1111111111.11111))*(py)*(sj1)))+(((IkReal(1111111111.11111))*(cj1)*(px)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x259=IKsin(j2);
IkReal x260=IKcos(j2);
IkReal x261=((cj1)*(px));
IkReal x262=((IkReal(1.00000000000000))*(pz));
IkReal x263=((py)*(sj1));
IkReal x264=((IkReal(0.0900000000000000))*(x260));
IkReal x265=((IkReal(1.00000000000000))*(x260));
evalcond[0]=((((x259)*(x263)))+(((x259)*(x261)))+(((IkReal(-1.00000000000000))*(x260)*(x262))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x262)))+(((IkReal(9.00000000000000e-10))*(x259))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x263)))+(((IkReal(-1.00000000000000))*(x261)))+(((IkReal(9.00000000000000e-10))*(x260))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x263)*(x265)))+(((IkReal(-1.00000000000000))*(x259)*(x262)))+(((IkReal(-1.00000000000000))*(x261)*(x265))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(0.0900000000000000))*(pz)*(x259)))+(((x263)*(x264)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x261)*(x264))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(px)))+(((py)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj1)*(px)))+(((py)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x266=IKsin(j2);
IkReal x267=IKcos(j2);
IkReal x268=((cj1)*(px));
IkReal x269=((py)*(sj1));
IkReal x270=((IkReal(0.0450000000000000))*(cj3));
IkReal x271=((IkReal(1.00000000000000))*(pz));
IkReal x272=((IkReal(0.300000000000000))*(sj3));
IkReal x273=((IkReal(0.0900000000000000))*(x267));
IkReal x274=((IkReal(1.00000000000000))*(x267));
evalcond[0]=((((x266)*(x268)))+(((x266)*(x269)))+(((IkReal(-1.00000000000000))*(x267)*(x271))));
evalcond[1]=((((x266)*(x272)))+(((IkReal(-1.00000000000000))*(x271)))+(((IkReal(-1.00000000000000))*(x266)*(x270)))+(((IkReal(0.0450000000000000))*(x266))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x269)))+(((IkReal(-1.00000000000000))*(x268)))+(((IkReal(0.0450000000000000))*(x267)))+(((IkReal(-1.00000000000000))*(x267)*(x270)))+(((x267)*(x272))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x269)*(x274)))+(x272)+(((IkReal(-1.00000000000000))*(x268)*(x274)))+(((IkReal(-1.00000000000000))*(x270)))+(((IkReal(-1.00000000000000))*(x266)*(x271))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(px)*(sj1)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(cj1)*(py)))+(((x269)*(x273)))+(((IkReal(0.0900000000000000))*(pz)*(x266)))+(((x268)*(x273))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x87=((sj2)*(sj3));
IkReal x88=((cj3)*(sj1));
IkReal x89=((cj3)*(sj2));
IkReal x90=((cj1)*(cj3));
IkReal x91=((sj1)*(sj3));
IkReal x92=((IkReal(1.00000000000000))*(sj2));
IkReal x93=((cj1)*(sj3));
IkReal x94=((((cj2)*(x88)))+(x93));
IkReal x95=((((cj2)*(x90)))+(((IkReal(-1.00000000000000))*(x91))));
IkReal x96=((((cj2)*(x91)))+(((IkReal(-1.00000000000000))*(x90))));
IkReal x97=((((cj2)*(x93)))+(x88));
new_r00=((((r10)*(x94)))+(((r00)*(x95)))+(((r20)*(x89))));
new_r01=((((r21)*(x89)))+(((r11)*(x94)))+(((r01)*(x95))));
new_r02=((((r12)*(x94)))+(((r22)*(x89)))+(((r02)*(x95))));
new_r10=((((IkReal(-1.00000000000000))*(r10)*(sj1)*(x92)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x92)))+(((cj2)*(r20))));
new_r11=((((IkReal(-1.00000000000000))*(cj1)*(r01)*(x92)))+(((IkReal(-1.00000000000000))*(r11)*(sj1)*(x92)))+(((cj2)*(r21))));
new_r12=((((IkReal(-1.00000000000000))*(cj1)*(r02)*(x92)))+(((cj2)*(r22)))+(((IkReal(-1.00000000000000))*(r12)*(sj1)*(x92))));
new_r20=((((r10)*(x96)))+(((r00)*(x97)))+(((r20)*(x87))));
new_r21=((((r21)*(x87)))+(((r11)*(x96)))+(((r01)*(x97))));
new_r22=((((r12)*(x96)))+(((r22)*(x87)))+(((r02)*(x97))));
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
cj5array[0]=new_r22;
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x98=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x98));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x98))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
evalcond[0]=((((new_r12)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj4)*(new_r00)))+(((new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))))+IKsqr(((((cj4)*(new_r00)))+(((new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))), ((((cj4)*(new_r00)))+(((new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x99=IKsin(j6);
IkReal x100=((IkReal(1.00000000000000))*(sj4));
IkReal x101=((IkReal(1.00000000000000))*(IKcos(j6)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x100)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x99))));
evalcond[1]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x100)))+(((IkReal(-1.00000000000000))*(x101))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(x99));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x101))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x102=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x102;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x102;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x103=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x103));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x103))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
evalcond[0]=((((new_r12)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((cj4)*(new_r01)))+(((new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))), ((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x104=IKcos(j6);
IkReal x105=((IkReal(1.00000000000000))*(sj4));
IkReal x106=((IkReal(1.00000000000000))*(IKsin(j6)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x105)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x106))));
evalcond[1]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x105)))+(((IkReal(-1.00000000000000))*(x104))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(((IkReal(-1.00000000000000))*(x106))));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(x104));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x107=((gconst3)*(sj5));
if( IKabs(((new_r12)*(x107))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x107))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((new_r12)*(x107)), ((new_r02)*(x107)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x108=IKcos(j4);
IkReal x109=IKsin(j4);
IkReal x110=((IkReal(1.00000000000000))*(sj5));
IkReal x111=((IkReal(1.00000000000000))*(cj5));
IkReal x112=((new_r12)*(x109));
IkReal x113=((new_r02)*(x108));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(x109)))+(((new_r12)*(x108))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x110)))+(x113)+(x112));
evalcond[2]=((((cj5)*(x113)))+(((cj5)*(x112)))+(((IkReal(-1.00000000000000))*(new_r22)*(x110))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r10)*(x109)*(x110)))+(((IkReal(-1.00000000000000))*(new_r00)*(x108)*(x110)))+(((IkReal(-1.00000000000000))*(new_r20)*(x111))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r01)*(x108)*(x110)))+(((IkReal(-1.00000000000000))*(new_r11)*(x109)*(x110)))+(((IkReal(-1.00000000000000))*(new_r21)*(x111))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x110)*(x112)))+(((IkReal(-1.00000000000000))*(x110)*(x113)))+(((IkReal(-1.00000000000000))*(new_r22)*(x111))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x114=((((IkReal(-1.00000000000000))*(new_r02)*(sj4)))+(((cj4)*(new_r12))));
IkReal x115=((((new_r12)*(sj4)))+(((cj4)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x114;
evalcond[5]=x114;
evalcond[6]=x115;
evalcond[7]=x115;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj4)*(new_r00)))+(((new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))))+IKsqr(((((cj4)*(new_r00)))+(((new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))), ((((cj4)*(new_r00)))+(((new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x116=IKsin(j6);
IkReal x117=((IkReal(1.00000000000000))*(sj4));
IkReal x118=((IkReal(1.00000000000000))*(IKcos(j6)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x117)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x116))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x117)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x118))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(x116));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x118))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x119=((IkReal(1.00000000000000))+(new_r22));
IkReal x120=((cj4)*(new_r02));
IkReal x121=((new_r12)*(sj4));
IkReal x122=((((IkReal(-1.00000000000000))*(new_r02)*(sj4)))+(((cj4)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x119;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x122;
evalcond[5]=x122;
evalcond[6]=((x120)+(x121));
evalcond[7]=((((IkReal(-1.00000000000000))*(x120)))+(((IkReal(-1.00000000000000))*(x121))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x119;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((cj4)*(new_r01)))+(((new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))), ((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x123=IKcos(j6);
IkReal x124=((IkReal(1.00000000000000))*(sj4));
IkReal x125=((IkReal(1.00000000000000))*(IKsin(j6)));
evalcond[0]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x124)))+(((IkReal(-1.00000000000000))*(x125))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x124)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x123))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(((IkReal(-1.00000000000000))*(x125))));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(x123));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst5)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst5)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst5)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst5)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x126=IKsin(j6);
IkReal x127=IKcos(j6);
IkReal x128=((cj4)*(new_r01));
IkReal x129=((IkReal(1.00000000000000))*(sj5));
IkReal x130=((new_r10)*(sj4));
IkReal x131=((new_r11)*(sj4));
IkReal x132=((cj4)*(new_r00));
IkReal x133=((IkReal(1.00000000000000))*(sj4));
IkReal x134=((IkReal(1.00000000000000))*(x127));
evalcond[0]=((((sj5)*(x127)))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(x126)*(x129)))+(new_r21));
evalcond[2]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x133)))+(((IkReal(-1.00000000000000))*(x126))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x133)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x134))));
evalcond[4]=((x131)+(x128)+(((cj5)*(x126))));
evalcond[5]=((x130)+(x132)+(((IkReal(-1.00000000000000))*(cj5)*(x134))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r21)*(x129)))+(x126)+(((cj5)*(x131)))+(((cj5)*(x128))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x134)))+(((IkReal(-1.00000000000000))*(new_r20)*(x129)))+(((cj5)*(x130)))+(((cj5)*(x132))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst4)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst4)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst4)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst4)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[2];
evalcond[0]=((new_r20)+(((sj5)*(IKcos(j6)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(IKsin(j6))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x135=((gconst6)*(sj5));
if( IKabs(((new_r12)*(x135))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x135))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((new_r12)*(x135)), ((new_r02)*(x135)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[12];
IkReal x136=IKcos(j4);
IkReal x137=IKsin(j4);
IkReal x138=((IkReal(1.00000000000000))*(cj6));
IkReal x139=((IkReal(1.00000000000000))*(sj5));
IkReal x140=((IkReal(1.00000000000000))*(cj5));
IkReal x141=((new_r00)*(x136));
IkReal x142=((cj5)*(x137));
IkReal x143=((new_r11)*(x137));
IkReal x144=((cj5)*(x136));
IkReal x145=((new_r02)*(x136));
IkReal x146=((new_r12)*(x137));
IkReal x147=((new_r10)*(x137));
IkReal x148=((IkReal(1.00000000000000))*(x137));
IkReal x149=((new_r01)*(x136));
evalcond[0]=((((new_r12)*(x136)))+(((IkReal(-1.00000000000000))*(new_r02)*(x148))));
evalcond[1]=((x145)+(x146)+(((IkReal(-1.00000000000000))*(x139))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)))+(((new_r10)*(x136)))+(((IkReal(-1.00000000000000))*(new_r00)*(x148))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x148)))+(((IkReal(-1.00000000000000))*(x138)))+(((new_r11)*(x136))));
evalcond[4]=((((cj5)*(sj6)))+(x143)+(x149));
evalcond[5]=((x141)+(x147)+(((IkReal(-1.00000000000000))*(cj5)*(x138))));
evalcond[6]=((((new_r12)*(x142)))+(((new_r02)*(x144)))+(((IkReal(-1.00000000000000))*(new_r22)*(x139))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x140)))+(((IkReal(-1.00000000000000))*(x139)*(x141)))+(((IkReal(-1.00000000000000))*(x139)*(x147))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r21)*(x140)))+(((IkReal(-1.00000000000000))*(x139)*(x143)))+(((IkReal(-1.00000000000000))*(x139)*(x149))));
evalcond[9]=((sj6)+(((new_r01)*(x144)))+(((new_r11)*(x142)))+(((IkReal(-1.00000000000000))*(new_r21)*(x139))));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x140)))+(((IkReal(-1.00000000000000))*(x139)*(x145)))+(((IkReal(-1.00000000000000))*(x139)*(x146))));
evalcond[11]=((((cj5)*(x141)))+(((new_r10)*(x142)))+(((IkReal(-1.00000000000000))*(new_r20)*(x139)))+(((IkReal(-1.00000000000000))*(x138))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3238550ee83defc84c543b45c2db9325"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
