/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:45:21.444227
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62,x63,x64,x65,x66,x67,x68,x69,x70,x71,x72,x73,x74,x75,x76,x77,x78,x79,x80,x81,x82;
x0=IKsin(j[1]);
x1=IKcos(j[1]);
x2=IKcos(j[3]);
x3=IKcos(j[2]);
x4=IKsin(j[3]);
x5=IKcos(j[4]);
x6=IKsin(j[2]);
x7=IKsin(j[4]);
x8=IKcos(j[6]);
x9=IKsin(j[6]);
x10=IKcos(j[5]);
x11=IKsin(j[5]);
x12=IKcos(j[0]);
x13=IKsin(j[0]);
x14=((IkReal(0.707105171781653))*(x11));
x15=((IkReal(1.00000000000000))*(x12));
x16=((IkReal(0.707108390587779))*(x13));
x17=((IkReal(0.707108390587779))*(x11));
x18=((IkReal(0.177950000000000))*(x0));
x19=((IkReal(1.00000000000000))*(x3));
x20=((IkReal(0.707108390587779))*(x10));
x21=((IkReal(1.00000000000000))*(x1));
x22=((IkReal(1.00000000000000))*(x13));
x23=((IkReal(0.707105171781653))*(x10));
x24=((IkReal(1.00000000000000))*(x7));
x25=((IkReal(1.00000000000000))*(x0));
x26=((IkReal(0.707105171781653))*(x9));
x27=((IkReal(0.707105171781653))*(x12));
x28=((IkReal(0.707105171781653))*(x13));
x29=((IkReal(0.0450000000000000))*(x6));
x30=((IkReal(0.177950000000000))*(x2));
x31=((IkReal(0.707108390587779))*(x9));
x32=((IkReal(0.707108390587779))*(x12));
x33=((IkReal(0.0318197327301744))*(x3));
x34=((IkReal(0.707108390587779))*(x8));
x35=((IkReal(0.707105171781653))*(x8));
x36=((x5)*(x6));
x37=((x0)*(x4));
x38=((x6)*(x7));
x39=((x3)*(x7));
x40=((x1)*(x3));
x41=((x1)*(x4));
x42=((x4)*(x6));
x43=((x0)*(x2));
x44=((x1)*(x2));
x45=((x25)*(x4));
x46=((IkReal(0.0318198775764501))*(x0)*(x3));
x47=((x10)*(x42));
x48=((x2)*(x40));
x49=((x25)*(x36));
x50=((x2)*(x24)*(x6));
x51=((x21)*(x36));
x52=((x19)*(x37));
x53=((((x2)*(x36)))+(x39));
x54=((((x3)*(x43)))+(x41));
x55=((((IkReal(-1.00000000000000))*(x45)))+(x48));
x56=((((IkReal(-1.00000000000000))*(x19)*(x44)))+(x45));
x57=((((IkReal(-1.00000000000000))*(x2)*(x21)))+(x52));
x58=((((IkReal(-1.00000000000000))*(x1)*(x30)))+(((x18)*(x3)*(x4))));
x59=((((IkReal(-1.00000000000000))*(x21)*(x4)))+(((IkReal(-1.00000000000000))*(x19)*(x43))));
x60=((((x19)*(x41)))+(((x2)*(x25))));
x61=((x5)*(x54));
x62=((x5)*(x55));
x63=((x59)*(x7));
x64=((((x10)*(x53)))+(((IkReal(-1.00000000000000))*(x11)*(x42))));
x65=((((IkReal(-1.00000000000000))*(x0)*(x24)*(x6)))+(x61));
x66=((x62)+(((IkReal(-1.00000000000000))*(x21)*(x38))));
x67=((((x11)*(x53)))+(x47));
x68=((((IkReal(-1.00000000000000))*(x18)*(x38)))+(((IkReal(0.177950000000000))*(x61))));
x69=((((IkReal(-1.00000000000000))*(x49)))+(x63));
x70=((x11)*(x65));
x71=((x12)*(x67));
x72=((x49)+(((IkReal(-1.00000000000000))*(x24)*(x59))));
x73=((((x10)*(x65)))+(((x11)*(((x44)+(((IkReal(-1.00000000000000))*(x52))))))));
x74=((((x9)*(((((x3)*(x5)))+(((IkReal(-1.00000000000000))*(x50)))))))+(((x64)*(x8))));
x75=((((x8)*(((((IkReal(-1.00000000000000))*(x19)*(x5)))+(x50)))))+(((x64)*(x9))));
x76=((((x11)*(((((x30)*(x36)))+(((IkReal(0.177950000000000))*(x39)))))))+(((IkReal(-1.00000000000000))*(x2)*(x29)))+(x29)+(((IkReal(0.177950000000000))*(x47)))+(((IkReal(0.300000000000000))*(x42))));
x77=((((IkReal(-1.00000000000000))*(x11)*(x60)))+(((x10)*(x66))));
x78=((((x11)*(x66)))+(((x10)*(x60))));
x79=((x12)*(x75));
x80=((((x77)*(x8)))+(((x9)*(((((x56)*(x7)))+(((IkReal(-1.00000000000000))*(x51))))))));
x81=((((x77)*(x9)))+(((x8)*(((x51)+(((IkReal(-1.00000000000000))*(x24)*(x56))))))));
IkReal x83=((x4)*(x40));
x82=((((IkReal(0.300000000000000))*(x83)))+(((IkReal(-0.0450000000000000))*(x48)))+(((x11)*(((((IkReal(-0.177950000000000))*(x1)*(x38)))+(((IkReal(0.177950000000000))*(x62)))))))+(((IkReal(0.0450000000000000))*(x37)))+(((IkReal(0.300000000000000))*(x43)))+(((x10)*(((((x18)*(x2)))+(((IkReal(0.177950000000000))*(x83)))))))+(((IkReal(0.0450000000000000))*(x40)))+(((IkReal(0.550000000000000))*(x0))));
eerot[0]=((((IkReal(-1.00000000000000))*(x27)*(x75)))+(((x34)*(x72)))+(((x31)*(x73)))+(((IkReal(-1.00000000000000))*(x28)*(x81))));
eerot[1]=((((IkReal(-1.00000000000000))*(x27)*(x74)))+(((x31)*(x69)))+(((x34)*(x73)))+(((IkReal(-1.00000000000000))*(x28)*(x80))));
eerot[2]=((((x20)*(x57)))+(((x17)*(x65)))+(((IkReal(-1.00000000000000))*(x27)*(x67)))+(((IkReal(-1.00000000000000))*(x28)*(x78))));
eetrans[0]=((IkReal(-0.350177000000000))+(((IkReal(-0.212132517176334))*(x44)))+(((x20)*(x58)))+(((IkReal(-1.00000000000000))*(x27)*(x76)))+(((IkReal(-0.388909614823279))*(x1)))+(((x17)*(x68)))+(((IkReal(0.212132517176334))*(x3)*(x37)))+(x46)+(((IkReal(-0.0318198775764501))*(x3)*(x43)))+(((IkReal(-0.0318198775764501))*(x41)))+(((IkReal(-1.00000000000000))*(x28)*(x82))));
eerot[3]=((((x32)*(x75)))+(((x16)*(x81)))+(((x26)*(x73)))+(((x35)*(x72))));
eerot[4]=((((x26)*(x69)))+(((x32)*(x74)))+(((x16)*(x80)))+(((x35)*(x73))));
eerot[5]=((((x14)*(x65)))+(((x16)*(x78)))+(((x32)*(x67)))+(((x23)*(x57))));
eetrans[1]=((IkReal(-0.350177000000000))+(((x32)*(x76)))+(((x14)*(x68)))+(((x16)*(x82)))+(((IkReal(-0.212131551534496))*(x44)))+(((IkReal(-0.388907844479909))*(x1)))+(((IkReal(0.212131551534496))*(x3)*(x37)))+(((x0)*(x33)))+(((IkReal(-1.00000000000000))*(x33)*(x43)))+(((x23)*(x58)))+(((IkReal(-0.0318197327301744))*(x41))));
eerot[6]=((((x15)*(x81)))+(((IkReal(-1.00000000000000))*(x22)*(x75))));
eerot[7]=((((x15)*(x80)))+(((IkReal(-1.00000000000000))*(x22)*(x74))));
eerot[8]=((((x15)*(x78)))+(((IkReal(-1.00000000000000))*(x22)*(x67))));
eetrans[2]=((IkReal(0.378361000000000))+(((x15)*(x82)))+(((IkReal(-1.00000000000000))*(x22)*(x76))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j7,cj7,sj7,htj7,j8,cj8,sj8,htj8,j9,cj9,sj9,htj9,j10,cj10,sj10,htj10,j4,cj4,sj4,htj4,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij5[2], _nj5,_ij6[2], _nj6,_ij7[2], _nj7,_ij8[2], _nj8,_ij9[2], _nj9,_ij10[2], _nj10,_ij4[2], _nj4;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1; j7=numeric_limits<IkReal>::quiet_NaN(); _ij7[0] = -1; _ij7[1] = -1; _nj7 = -1; j8=numeric_limits<IkReal>::quiet_NaN(); _ij8[0] = -1; _ij8[1] = -1; _nj8 = -1; j9=numeric_limits<IkReal>::quiet_NaN(); _ij9[0] = -1; _ij9[1] = -1; _nj9 = -1; j10=numeric_limits<IkReal>::quiet_NaN(); _ij10[0] = -1; _ij10[1] = -1; _nj10 = -1;  _ij4[0] = -1; _ij4[1] = -1; _nj4 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j4=pfree[0]; cj4=cos(pfree[0]); sj4=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

IkReal x84=IKsin(j4);
new_r00=((((IkReal(1.00000000000000))*(r21)*(IKcos(j4))))+(((IkReal(-0.707105171781653))*(r01)*(x84)))+(((IkReal(0.707108390587779))*(r11)*(x84))));
IkReal x85=IKsin(j4);
new_r01=((((IkReal(-0.707108390587779))*(r10)*(x85)))+(((IkReal(0.707105171781653))*(r00)*(x85)))+(((IkReal(-1.00000000000000))*(r20)*(IKcos(j4)))));
IkReal x86=IKsin(j4);
new_r02=((((IkReal(-0.707105171781653))*(r02)*(x86)))+(((IkReal(0.707108390587779))*(r12)*(x86)))+(((IkReal(1.00000000000000))*(r22)*(IKcos(j4)))));
IkReal x87=IKsin(j4);
IkReal x88=IKcos(j4);
new_px=((((IkReal(-0.707105171781653))*(px)*(x87)))+(((IkReal(1.00000000000000))*(pz)*(x88)))+(((IkReal(1.12715187294832e-6))*(x87)))+(((IkReal(0.125829365318545))*(r02)*(x87)))+(((IkReal(0.707108390587779))*(py)*(x87)))+(((IkReal(-0.378361000000000))*(x88)))+(((IkReal(-0.177950000000000))*(r22)*(x88)))+(((IkReal(-0.125829938105095))*(r12)*(x87))));
new_r10=((((IkReal(0.707108390587779))*(r01)))+(((IkReal(0.707105171781653))*(r11))));
new_r11=((((IkReal(-0.707108390587779))*(r00)))+(((IkReal(-0.707105171781653))*(r10))));
new_r12=((((IkReal(0.707108390587779))*(r02)))+(((IkReal(0.707105171781653))*(r12))));
new_py=((IkReal(0.495225062629841))+(((IkReal(0.707105171781653))*(py)))+(((IkReal(0.707108390587779))*(px)))+(((IkReal(-0.125829938105095))*(r02)))+(((IkReal(-0.125829365318545))*(r12))));
IkReal x89=IKcos(j4);
new_r20=((((IkReal(-1.00000000000000))*(r21)*(IKsin(j4))))+(((IkReal(-0.707105171781653))*(r01)*(x89)))+(((IkReal(0.707108390587779))*(r11)*(x89))));
IkReal x90=IKcos(j4);
new_r21=((((IkReal(-0.707108390587779))*(r10)*(x90)))+(((IkReal(1.00000000000000))*(r20)*(IKsin(j4))))+(((IkReal(0.707105171781653))*(r00)*(x90))));
IkReal x91=IKcos(j4);
new_r22=((((IkReal(-0.707105171781653))*(r02)*(x91)))+(((IkReal(-1.00000000000000))*(r22)*(IKsin(j4))))+(((IkReal(0.707108390587779))*(r12)*(x91))));
IkReal x92=IKcos(j4);
IkReal x93=IKsin(j4);
new_pz=((((IkReal(-0.707105171781653))*(px)*(x92)))+(((IkReal(0.378361000000000))*(x93)))+(((IkReal(0.707108390587779))*(py)*(x92)))+(((IkReal(0.125829365318545))*(r02)*(x92)))+(((IkReal(1.12715187294832e-6))*(x92)))+(((IkReal(-1.00000000000000))*(pz)*(x93)))+(((IkReal(0.177950000000000))*(r22)*(x93)))+(((IkReal(-0.125829938105095))*(r12)*(x92))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j7array[2], cj7array[2], sj7array[2];
bool j7valid[2]={false};
_nj7 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x94=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j7array[0]=((IkReal(-1.34027003705633))+(((IkReal(-1.00000000000000))*(x94))));
sj7array[0]=IKsin(j7array[0]);
cj7array[0]=IKcos(j7array[0]);
j7array[1]=((IkReal(1.80132261653346))+(x94));
sj7array[1]=IKsin(j7array[1]);
cj7array[1]=IKcos(j7array[1]);
if( j7array[0] > IKPI )
{
    j7array[0]-=IK2PI;
}
else if( j7array[0] < -IKPI )
{    j7array[0]+=IK2PI;
}
j7valid[0] = true;
if( j7array[1] > IKPI )
{
    j7array[1]-=IK2PI;
}
else if( j7array[1] < -IKPI )
{    j7array[1]+=IK2PI;
}
j7valid[1] = true;
for(int ij7 = 0; ij7 < 2; ++ij7)
{
if( !j7valid[ij7] )
{
    continue;
}
_ij7[0] = ij7; _ij7[1] = -1;
for(int iij7 = ij7+1; iij7 < 2; ++iij7)
{
if( j7valid[iij7] && IKabs(cj7array[ij7]-cj7array[iij7]) < IKFAST_SOLUTION_THRESH && IKabs(sj7array[ij7]-sj7array[iij7]) < IKFAST_SOLUTION_THRESH )
{
    j7valid[iij7]=false; _ij7[1] = iij7; break; 
}
}
j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((py)*(py)*(py)*(py))+((((px)*(px))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x95=(py)*(py);
if( IKabs(((IkReal(-1.10000000000000))*(x95))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.10000000000000))*(px)*(py))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x96=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.10000000000000))*(x95)), ((IkReal(1.10000000000000))*(px)*(py)))));
if( (((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(0.0270000000000000))*(py)*(sj7)))+(((IkReal(-0.00405000000000000))*(cj7)*(py)))+(((IkReal(-0.208450000000000))*(py))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(0.0270000000000000))*(py)*(sj7)))+(((IkReal(-0.00405000000000000))*(cj7)*(py)))+(((IkReal(-0.208450000000000))*(py))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x97=IKasin(((((IKabs(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(1.21000000000000))*(x95)*((px)*(px))))+(((IkReal(1.21000000000000))*((x95)*(x95))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(py)))+(((IkReal(0.0270000000000000))*(py)*(sj7)))+(((IkReal(-0.00405000000000000))*(cj7)*(py)))+(((IkReal(-0.208450000000000))*(py)))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x97)))+(((IkReal(-1.00000000000000))*(x96))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(x97)+(((IkReal(-1.00000000000000))*(x96))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x98=IKsin(j5);
IkReal x99=IKcos(j5);
IkReal x100=((py)*(x99));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((px)*(x98)))+(((IkReal(-1.00000000000000))*(x100))));
evalcond[1]=((((IkReal(-1.10000000000000))*(px)*(x100)))+(((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(1.10000000000000))*(x98)*((px)*(px))))+(((IkReal(0.0270000000000000))*(px)*(sj7)))+(((IkReal(-0.00405000000000000))*(cj7)*(px)))+(((IkReal(-0.208450000000000))*(px))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x101=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x101)))+(((px)*(sj5))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x101)))+(((IkReal(-1.00000000000000))*(cj5)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-9.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-18.0000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-2.00000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-1.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x102=((cj5)*(px));
IkReal x103=((IkReal(110.000000000000))*(px));
IkReal x104=((py)*(sj5));
IkReal x105=((IkReal(100.000000000000))*(pp));
IkReal x106=((IkReal(110.000000000000))*(cj5)*(sj5));
if( IKabs(((gconst1)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x105)))+(((IkReal(-21.2500000000000))*(pz)))+(((pz)*(sj5)*(x103))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x104)*(x105)))+(((IkReal(-21.2500000000000))*(x102)))+(((IkReal(-21.2500000000000))*(x104)))+(((sj5)*(x103)*(x104)))+(((sj5)*(x102)*(x103)))+(((IkReal(-1.00000000000000))*(x102)*(x105)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x104)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x102))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x105)))+(((IkReal(-21.2500000000000))*(pz)))+(((pz)*(sj5)*(x103)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x104)*(x105)))+(((IkReal(-21.2500000000000))*(x102)))+(((IkReal(-21.2500000000000))*(x104)))+(((sj5)*(x103)*(x104)))+(((sj5)*(x102)*(x103)))+(((IkReal(-1.00000000000000))*(x102)*(x105)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x104)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x102)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x107=IKsin(j6);
IkReal x108=IKcos(j6);
IkReal x109=((py)*(sj5));
IkReal x110=((IkReal(1.00000000000000))*(pz));
IkReal x111=((cj5)*(px));
IkReal x112=((IkReal(0.0900000000000000))*(x108));
IkReal x113=((IkReal(1.00000000000000))*(x108));
evalcond[0]=((((x107)*(x109)))+(((IkReal(-1.00000000000000))*(x108)*(x110)))+(((x107)*(x111))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x111)*(x113)))+(((IkReal(-1.00000000000000))*(x107)*(x110)))+(((IkReal(-1.00000000000000))*(x109)*(x113))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x111)*(x112)))+(((IkReal(0.0900000000000000))*(pz)*(x107)))+(((x109)*(x112)))+(((IkReal(-1.00000000000000))*(pp))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x175=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x175)))+(((px)*(sj5))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x175)))+(((IkReal(-1.00000000000000))*(cj5)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-9.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-18.0000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-2.00000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-1.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x176=((cj5)*(px));
IkReal x177=((IkReal(100.000000000000))*(pp));
IkReal x178=((IkReal(110.000000000000))*(px));
IkReal x179=((py)*(sj5));
IkReal x180=((IkReal(110.000000000000))*(cj5)*(sj5));
if( IKabs(((gconst2)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((pz)*(sj5)*(x178)))+(((IkReal(-1.00000000000000))*(pz)*(x177)))+(((IkReal(-21.2500000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((sj5)*(x178)*(x179)))+(((IkReal(-21.2500000000000))*(x179)))+(((IkReal(-21.2500000000000))*(x176)))+(((IkReal(-1.00000000000000))*(x177)*(x179)))+(((IkReal(-1.00000000000000))*(x176)*(x177)))+(((sj5)*(x176)*(x178)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x179)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x176))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((pz)*(sj5)*(x178)))+(((IkReal(-1.00000000000000))*(pz)*(x177)))+(((IkReal(-21.2500000000000))*(pz)))))), ((gconst2)*(((((sj5)*(x178)*(x179)))+(((IkReal(-21.2500000000000))*(x179)))+(((IkReal(-21.2500000000000))*(x176)))+(((IkReal(-1.00000000000000))*(x177)*(x179)))+(((IkReal(-1.00000000000000))*(x176)*(x177)))+(((sj5)*(x176)*(x178)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x179)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x176)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x181=IKsin(j6);
IkReal x182=IKcos(j6);
IkReal x183=((py)*(sj5));
IkReal x184=((IkReal(1.00000000000000))*(pz));
IkReal x185=((cj5)*(px));
IkReal x186=((IkReal(0.0900000000000000))*(x182));
IkReal x187=((IkReal(1.00000000000000))*(x182));
evalcond[0]=((((IkReal(-1.00000000000000))*(x182)*(x184)))+(((x181)*(x183)))+(((x181)*(x185))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x185)*(x187)))+(((IkReal(-1.00000000000000))*(x183)*(x187)))+(((IkReal(-1.00000000000000))*(x181)*(x184))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(pz)*(x181)))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x185)*(x186)))+(((IkReal(-1.00000000000000))*(pp)))+(((x183)*(x186))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((px)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(py))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((IkReal(1111111111.11111))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1111111111.11111))*(pz)))+IKsqr(((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((IkReal(1111111111.11111))*(pz)), ((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x188=IKsin(j6);
IkReal x189=IKcos(j6);
IkReal x190=((py)*(sj5));
IkReal x191=((IkReal(1.00000000000000))*(pz));
IkReal x192=((cj5)*(px));
IkReal x193=((IkReal(0.0900000000000000))*(x189));
IkReal x194=((IkReal(1.00000000000000))*(x189));
evalcond[0]=((((IkReal(-1.00000000000000))*(x189)*(x191)))+(((x188)*(x190)))+(((x188)*(x192))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x191)))+(((IkReal(9.00000000000000e-10))*(x188))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x192)))+(((IkReal(-1.00000000000000))*(x190)))+(((IkReal(9.00000000000000e-10))*(x189))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x188)*(x191)))+(((IkReal(-1.00000000000000))*(x192)*(x194)))+(((IkReal(-1.00000000000000))*(x190)*(x194))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(pz)*(x188)))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((IkReal(-1.00000000000000))*(pp)))+(((x192)*(x193)))+(((x190)*(x193))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(px)))+(((py)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj5)*(px)))+(((py)*(sj5)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x195=IKsin(j6);
IkReal x196=IKcos(j6);
IkReal x197=((cj5)*(px));
IkReal x198=((IkReal(1.00000000000000))*(pz));
IkReal x199=((IkReal(0.0450000000000000))*(cj7));
IkReal x200=((py)*(sj5));
IkReal x201=((IkReal(0.300000000000000))*(sj7));
IkReal x202=((IkReal(0.0900000000000000))*(x196));
IkReal x203=((IkReal(1.00000000000000))*(x196));
evalcond[0]=((((IkReal(-1.00000000000000))*(x196)*(x198)))+(((x195)*(x200)))+(((x195)*(x197))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x195)*(x199)))+(((x195)*(x201)))+(((IkReal(-1.00000000000000))*(x198)))+(((IkReal(0.0450000000000000))*(x195))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x196)*(x199)))+(((x196)*(x201)))+(((IkReal(-1.00000000000000))*(x197)))+(((IkReal(0.0450000000000000))*(x196)))+(((IkReal(-1.00000000000000))*(x200))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x200)*(x203)))+(((IkReal(-1.00000000000000))*(x195)*(x198)))+(((IkReal(-1.00000000000000))*(x197)*(x203)))+(((IkReal(-1.00000000000000))*(x199)))+(x201));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x200)*(x202)))+(((x197)*(x202)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x195))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x204=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x205=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x205)))+(((IkReal(-1.00000000000000))*(x204))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x204)))+(((IkReal(1.00000000000000))*(x205))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x206=IKsin(j5);
IkReal x207=IKcos(j5);
IkReal x208=((IkReal(0.0270000000000000))*(sj7));
IkReal x209=((IkReal(1.00000000000000))*(pp));
IkReal x210=((IkReal(0.00405000000000000))*(cj7));
IkReal x211=((px)*(py));
IkReal x212=((IkReal(1.10000000000000))*(x207));
IkReal x213=((IkReal(1.10000000000000))*(x206));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x210)))+(((IkReal(-1.00000000000000))*(py)*(x209)))+(((x211)*(x213)))+(((py)*(x208)))+(((IkReal(-1.00000000000000))*(x212)*((py)*(py))))+(((IkReal(-0.208450000000000))*(py))));
evalcond[1]=((((IkReal(-1.00000000000000))*(px)*(x209)))+(((IkReal(-1.00000000000000))*(px)*(x210)))+(((x213)*((px)*(px))))+(((px)*(x208)))+(((IkReal(-1.00000000000000))*(x211)*(x212)))+(((IkReal(-0.208450000000000))*(px))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x214=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x214)))+(((px)*(sj5))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x214)))+(((IkReal(-1.00000000000000))*(cj5)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-9.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-18.0000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-2.00000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-1.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x215=((cj5)*(px));
IkReal x216=((IkReal(110.000000000000))*(px));
IkReal x217=((py)*(sj5));
IkReal x218=((IkReal(100.000000000000))*(pp));
IkReal x219=((IkReal(110.000000000000))*(cj5)*(sj5));
if( IKabs(((gconst1)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x218)))+(((pz)*(sj5)*(x216)))+(((IkReal(-21.2500000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-21.2500000000000))*(x215)))+(((IkReal(-21.2500000000000))*(x217)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x215)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x217)))+(((IkReal(-1.00000000000000))*(x217)*(x218)))+(((sj5)*(x215)*(x216)))+(((sj5)*(x216)*(x217)))+(((IkReal(-1.00000000000000))*(x215)*(x218))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x218)))+(((pz)*(sj5)*(x216)))+(((IkReal(-21.2500000000000))*(pz)))))), ((gconst1)*(((((IkReal(-21.2500000000000))*(x215)))+(((IkReal(-21.2500000000000))*(x217)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x215)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x217)))+(((IkReal(-1.00000000000000))*(x217)*(x218)))+(((sj5)*(x215)*(x216)))+(((sj5)*(x216)*(x217)))+(((IkReal(-1.00000000000000))*(x215)*(x218)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x220=IKsin(j6);
IkReal x221=IKcos(j6);
IkReal x222=((py)*(sj5));
IkReal x223=((IkReal(1.00000000000000))*(pz));
IkReal x224=((cj5)*(px));
IkReal x225=((IkReal(0.0900000000000000))*(x221));
IkReal x226=((IkReal(1.00000000000000))*(x221));
evalcond[0]=((((x220)*(x224)))+(((x220)*(x222)))+(((IkReal(-1.00000000000000))*(x221)*(x223))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x224)*(x226)))+(((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(-1.00000000000000))*(x220)*(x223))));
evalcond[2]=((IkReal(-0.212500000000000))+(((x224)*(x225)))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x222)*(x225)))+(((IkReal(0.0900000000000000))*(pz)*(x220)))+(((IkReal(-1.00000000000000))*(pp))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x227=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x227)))+(((px)*(sj5))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x227)))+(((IkReal(-1.00000000000000))*(cj5)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-9.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-18.0000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-2.00000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-1.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x228=((cj5)*(px));
IkReal x229=((IkReal(100.000000000000))*(pp));
IkReal x230=((IkReal(110.000000000000))*(px));
IkReal x231=((py)*(sj5));
IkReal x232=((IkReal(110.000000000000))*(cj5)*(sj5));
if( IKabs(((gconst2)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((pz)*(sj5)*(x230)))+(((IkReal(-1.00000000000000))*(pz)*(x229)))+(((IkReal(-21.2500000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((sj5)*(x228)*(x230)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x228)))+(((IkReal(-1.00000000000000))*(x228)*(x229)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x231)))+(((IkReal(-21.2500000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x229)*(x231)))+(((IkReal(-21.2500000000000))*(x231)))+(((sj5)*(x230)*(x231))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((pz)*(sj5)*(x230)))+(((IkReal(-1.00000000000000))*(pz)*(x229)))+(((IkReal(-21.2500000000000))*(pz)))))), ((gconst2)*(((((sj5)*(x228)*(x230)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x228)))+(((IkReal(-1.00000000000000))*(x228)*(x229)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x231)))+(((IkReal(-21.2500000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x229)*(x231)))+(((IkReal(-21.2500000000000))*(x231)))+(((sj5)*(x230)*(x231)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x233=IKsin(j6);
IkReal x234=IKcos(j6);
IkReal x235=((py)*(sj5));
IkReal x236=((IkReal(1.00000000000000))*(pz));
IkReal x237=((cj5)*(px));
IkReal x238=((IkReal(0.0900000000000000))*(x234));
IkReal x239=((IkReal(1.00000000000000))*(x234));
evalcond[0]=((((x233)*(x237)))+(((x233)*(x235)))+(((IkReal(-1.00000000000000))*(x234)*(x236))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x233)*(x236)))+(((IkReal(-1.00000000000000))*(x235)*(x239)))+(((IkReal(-1.00000000000000))*(x237)*(x239))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((x237)*(x238)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((IkReal(-1.00000000000000))*(pp)))+(((x235)*(x238)))+(((IkReal(0.0900000000000000))*(pz)*(x233))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((px)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(py))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((IkReal(1111111111.11111))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1111111111.11111))*(pz)))+IKsqr(((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((IkReal(1111111111.11111))*(pz)), ((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x240=IKsin(j6);
IkReal x241=IKcos(j6);
IkReal x242=((py)*(sj5));
IkReal x243=((IkReal(1.00000000000000))*(pz));
IkReal x244=((cj5)*(px));
IkReal x245=((IkReal(0.0900000000000000))*(x241));
IkReal x246=((IkReal(1.00000000000000))*(x241));
evalcond[0]=((((x240)*(x242)))+(((x240)*(x244)))+(((IkReal(-1.00000000000000))*(x241)*(x243))));
evalcond[1]=((((IkReal(9.00000000000000e-10))*(x240)))+(((IkReal(-1.00000000000000))*(x243))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x244)))+(((IkReal(-1.00000000000000))*(x242)))+(((IkReal(9.00000000000000e-10))*(x241))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x242)*(x246)))+(((IkReal(-1.00000000000000))*(x240)*(x243)))+(((IkReal(-1.00000000000000))*(x244)*(x246))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x244)*(x245)))+(((IkReal(-1.00000000000000))*(pp)))+(((x242)*(x245)))+(((IkReal(0.0900000000000000))*(pz)*(x240))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(px)))+(((py)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj5)*(px)))+(((py)*(sj5)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x247=IKsin(j6);
IkReal x248=IKcos(j6);
IkReal x249=((cj5)*(px));
IkReal x250=((IkReal(1.00000000000000))*(pz));
IkReal x251=((IkReal(0.0450000000000000))*(cj7));
IkReal x252=((py)*(sj5));
IkReal x253=((IkReal(0.300000000000000))*(sj7));
IkReal x254=((IkReal(0.0900000000000000))*(x248));
IkReal x255=((IkReal(1.00000000000000))*(x248));
evalcond[0]=((((IkReal(-1.00000000000000))*(x248)*(x250)))+(((x247)*(x249)))+(((x247)*(x252))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x247)*(x251)))+(((IkReal(0.0450000000000000))*(x247)))+(((IkReal(-1.00000000000000))*(x250)))+(((x247)*(x253))));
evalcond[2]=((((x248)*(x253)))+(((IkReal(-1.00000000000000))*(x249)))+(((IkReal(-1.00000000000000))*(x248)*(x251)))+(((IkReal(0.0450000000000000))*(x248)))+(((IkReal(-1.00000000000000))*(x252))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x252)*(x255)))+(((IkReal(-1.00000000000000))*(x247)*(x250)))+(x253)+(((IkReal(-1.00000000000000))*(x249)*(x255)))+(((IkReal(-1.00000000000000))*(x251))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x249)*(x254)))+(((IkReal(-1.00000000000000))*(pp)))+(((x252)*(x254)))+(((IkReal(0.0900000000000000))*(pz)*(x247))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x256=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x257=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))))))));
j5array[0]=((((IkReal(-1.00000000000000))*(x257)))+(((IkReal(-1.00000000000000))*(x256))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(1.00000000000000))*(x257)))+(((IkReal(-1.00000000000000))*(x256))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x258=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x258)))+(((px)*(sj5))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x258)))+(((IkReal(-1.00000000000000))*(cj5)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-9.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-18.0000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-2.00000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-1.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x259=((cj5)*(px));
IkReal x260=((IkReal(110.000000000000))*(px));
IkReal x261=((py)*(sj5));
IkReal x262=((IkReal(100.000000000000))*(pp));
IkReal x263=((IkReal(110.000000000000))*(cj5)*(sj5));
if( IKabs(((gconst1)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-21.2500000000000))*(pz)))+(((pz)*(sj5)*(x260)))+(((IkReal(-1.00000000000000))*(pz)*(x262))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-21.2500000000000))*(x261)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x261)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x259)))+(((IkReal(-1.00000000000000))*(x259)*(x262)))+(((IkReal(-1.00000000000000))*(x261)*(x262)))+(((IkReal(-21.2500000000000))*(x259)))+(((sj5)*(x260)*(x261)))+(((sj5)*(x259)*(x260))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-21.2500000000000))*(pz)))+(((pz)*(sj5)*(x260)))+(((IkReal(-1.00000000000000))*(pz)*(x262)))))), ((gconst1)*(((((IkReal(-21.2500000000000))*(x261)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x261)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x259)))+(((IkReal(-1.00000000000000))*(x259)*(x262)))+(((IkReal(-1.00000000000000))*(x261)*(x262)))+(((IkReal(-21.2500000000000))*(x259)))+(((sj5)*(x260)*(x261)))+(((sj5)*(x259)*(x260)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x264=IKsin(j6);
IkReal x265=IKcos(j6);
IkReal x266=((py)*(sj5));
IkReal x267=((IkReal(1.00000000000000))*(pz));
IkReal x268=((cj5)*(px));
IkReal x269=((IkReal(0.0900000000000000))*(x265));
IkReal x270=((IkReal(1.00000000000000))*(x265));
evalcond[0]=((((x264)*(x268)))+(((x264)*(x266)))+(((IkReal(-1.00000000000000))*(x265)*(x267))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x268)*(x270)))+(((IkReal(-1.00000000000000))*(x266)*(x270)))+(((IkReal(-1.00000000000000))*(x264)*(x267))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x266)*(x269)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x264)))+(((x268)*(x269))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x271=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x271)))+(((px)*(sj5))));
evalcond[3]=((IkReal(-1.00000000000000))*(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj5)*(px)))+(((IkReal(-1.00000000000000))*(sj5)*(x271))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-9.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-18.0000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-9.00000000000000))*((pz)*(pz))))+(((IkReal(-9.00000000000000))*((py)*(py))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-2.00000000000000))*(cj5)*(px)*(py)*(sj5)))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-1.00000000000000))*((cj5)*(cj5))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x272=((cj5)*(px));
IkReal x273=((IkReal(100.000000000000))*(pp));
IkReal x274=((IkReal(110.000000000000))*(px));
IkReal x275=((py)*(sj5));
IkReal x276=((IkReal(110.000000000000))*(cj5)*(sj5));
if( IKabs(((gconst2)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x273)))+(((pz)*(sj5)*(x274)))+(((IkReal(-21.2500000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x272)*(x273)))+(((IkReal(-1.00000000000000))*(x273)*(x275)))+(((sj5)*(x274)*(x275)))+(((IkReal(-21.2500000000000))*(x272)))+(((IkReal(-21.2500000000000))*(x275)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x275)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x272)))+(((sj5)*(x272)*(x274))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((IkReal(-110.000000000000))*(cj5)*(py)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x273)))+(((pz)*(sj5)*(x274)))+(((IkReal(-21.2500000000000))*(pz)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x272)*(x273)))+(((IkReal(-1.00000000000000))*(x273)*(x275)))+(((sj5)*(x274)*(x275)))+(((IkReal(-21.2500000000000))*(x272)))+(((IkReal(-21.2500000000000))*(x275)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x275)))+(((IkReal(-110.000000000000))*(cj5)*(py)*(x272)))+(((sj5)*(x272)*(x274)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x277=IKsin(j6);
IkReal x278=IKcos(j6);
IkReal x279=((py)*(sj5));
IkReal x280=((IkReal(1.00000000000000))*(pz));
IkReal x281=((cj5)*(px));
IkReal x282=((IkReal(0.0900000000000000))*(x278));
IkReal x283=((IkReal(1.00000000000000))*(x278));
evalcond[0]=((((x277)*(x279)))+(((x277)*(x281)))+(((IkReal(-1.00000000000000))*(x278)*(x280))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x277)*(x280)))+(((IkReal(-1.00000000000000))*(x281)*(x283)))+(((IkReal(-1.00000000000000))*(x279)*(x283))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(pz)*(x277)))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x279)*(x282)))+(((IkReal(-1.00000000000000))*(pp)))+(((x281)*(x282))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((px)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(py))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((IkReal(1111111111.11111))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1111111111.11111))*(pz)))+IKsqr(((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((IkReal(1111111111.11111))*(pz)), ((((IkReal(1111111111.11111))*(py)*(sj5)))+(((IkReal(1111111111.11111))*(cj5)*(px)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x284=IKsin(j6);
IkReal x285=IKcos(j6);
IkReal x286=((py)*(sj5));
IkReal x287=((IkReal(1.00000000000000))*(pz));
IkReal x288=((cj5)*(px));
IkReal x289=((IkReal(0.0900000000000000))*(x285));
IkReal x290=((IkReal(1.00000000000000))*(x285));
evalcond[0]=((((x284)*(x286)))+(((x284)*(x288)))+(((IkReal(-1.00000000000000))*(x285)*(x287))));
evalcond[1]=((((IkReal(9.00000000000000e-10))*(x284)))+(((IkReal(-1.00000000000000))*(x287))));
evalcond[2]=((((IkReal(9.00000000000000e-10))*(x285)))+(((IkReal(-1.00000000000000))*(x288)))+(((IkReal(-1.00000000000000))*(x286))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x286)*(x290)))+(((IkReal(-1.00000000000000))*(x284)*(x287)))+(((IkReal(-1.00000000000000))*(x288)*(x290))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((x286)*(x289)))+(((IkReal(-1.00000000000000))*(pp)))+(((x288)*(x289)))+(((IkReal(0.0900000000000000))*(pz)*(x284))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst0)*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj5)*(px)))+(((py)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(pz)), ((gconst0)*(((((cj5)*(px)))+(((py)*(sj5)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x291=IKsin(j6);
IkReal x292=IKcos(j6);
IkReal x293=((cj5)*(px));
IkReal x294=((IkReal(1.00000000000000))*(pz));
IkReal x295=((IkReal(0.0450000000000000))*(cj7));
IkReal x296=((py)*(sj5));
IkReal x297=((IkReal(0.300000000000000))*(sj7));
IkReal x298=((IkReal(0.0900000000000000))*(x292));
IkReal x299=((IkReal(1.00000000000000))*(x292));
evalcond[0]=((((x291)*(x296)))+(((x291)*(x293)))+(((IkReal(-1.00000000000000))*(x292)*(x294))));
evalcond[1]=((((x291)*(x297)))+(((IkReal(-1.00000000000000))*(x291)*(x295)))+(((IkReal(0.0450000000000000))*(x291)))+(((IkReal(-1.00000000000000))*(x294))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x296)))+(((IkReal(-1.00000000000000))*(x293)))+(((IkReal(0.0450000000000000))*(x292)))+(((x292)*(x297)))+(((IkReal(-1.00000000000000))*(x292)*(x295))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x291)*(x294)))+(((IkReal(-1.00000000000000))*(x295)))+(((IkReal(-1.00000000000000))*(x296)*(x299)))+(((IkReal(-1.00000000000000))*(x293)*(x299)))+(x297));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.10000000000000))*(cj5)*(py)))+(((IkReal(1.10000000000000))*(px)*(sj5)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(pz)*(x291)))+(((x296)*(x298)))+(((x293)*(x298))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x114=((cj6)*(cj7));
IkReal x115=((cj5)*(sj7));
IkReal x116=((IkReal(1.00000000000000))*(sj6));
IkReal x117=((cj7)*(sj6));
IkReal x118=((sj6)*(sj7));
IkReal x119=((sj5)*(sj7));
IkReal x120=((((cj5)*(x114)))+(((IkReal(-1.00000000000000))*(x119))));
IkReal x121=((((sj5)*(x114)))+(x115));
IkReal x122=((((cj6)*(x119)))+(((IkReal(-1.00000000000000))*(cj5)*(cj7))));
IkReal x123=((((cj6)*(x115)))+(((cj7)*(sj5))));
new_r00=((((r00)*(x120)))+(((r20)*(x117)))+(((r10)*(x121))));
new_r01=((((r01)*(x120)))+(((r21)*(x117)))+(((r11)*(x121))));
new_r02=((((r22)*(x117)))+(((r02)*(x120)))+(((r12)*(x121))));
new_r10=((((IkReal(-1.00000000000000))*(cj5)*(r00)*(x116)))+(((cj6)*(r20)))+(((IkReal(-1.00000000000000))*(r10)*(sj5)*(x116))));
new_r11=((((cj6)*(r21)))+(((IkReal(-1.00000000000000))*(r11)*(sj5)*(x116)))+(((IkReal(-1.00000000000000))*(cj5)*(r01)*(x116))));
new_r12=((((cj6)*(r22)))+(((IkReal(-1.00000000000000))*(r12)*(sj5)*(x116)))+(((IkReal(-1.00000000000000))*(cj5)*(r02)*(x116))));
new_r20=((((r00)*(x123)))+(((r20)*(x118)))+(((r10)*(x122))));
new_r21=((((r01)*(x123)))+(((r21)*(x118)))+(((r11)*(x122))));
new_r22=((((r22)*(x118)))+(((r02)*(x123)))+(((r12)*(x122))));
{
IkReal j9array[2], cj9array[2], sj9array[2];
bool j9valid[2]={false};
_nj9 = 2;
cj9array[0]=new_r22;
if( cj9array[0] >= -1-IKFAST_SINCOS_THRESH && cj9array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j9valid[0] = j9valid[1] = true;
    j9array[0] = IKacos(cj9array[0]);
    sj9array[0] = IKsin(j9array[0]);
    cj9array[1] = cj9array[0];
    j9array[1] = -j9array[0];
    sj9array[1] = -sj9array[0];
}
else if( isnan(cj9array[0]) )
{
    // probably any value will work
    j9valid[0] = true;
    cj9array[0] = 1; sj9array[0] = 0; j9array[0] = 0;
}
for(int ij9 = 0; ij9 < 2; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 2; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj9);
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x124=IKatan2(new_r02, new_r12);
j8array[0]=((IkReal(-1.00000000000000))*(x124));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x124))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j8))))+(((new_r12)*(IKcos(j8)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj8)))+(((cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))))+IKsqr(((((new_r10)*(sj8)))+(((cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))), ((((new_r10)*(sj8)))+(((cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x125=IKsin(j10);
IkReal x126=((IkReal(1.00000000000000))*(sj8));
IkReal x127=((IkReal(1.00000000000000))*(IKcos(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x126)))+(((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x125))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x126)))+(((IkReal(-1.00000000000000))*(x127)))+(((cj8)*(new_r11))));
evalcond[2]=((((new_r11)*(sj8)))+(x125)+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(x127)))+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x128=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=x128;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x128;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x129=IKatan2(new_r02, new_r12);
j8array[0]=((IkReal(-1.00000000000000))*(x129));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x129))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j8))))+(((new_r12)*(IKcos(j8)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((new_r11)*(sj8)))+(((cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))), ((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x130=IKcos(j10);
IkReal x131=((IkReal(1.00000000000000))*(sj8));
IkReal x132=((IkReal(1.00000000000000))*(IKsin(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x131)))+(((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x132))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x131)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x130))));
evalcond[2]=((((new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(x132)))+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(x130)+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x133=((gconst3)*(sj9));
if( IKabs(((new_r12)*(x133))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x133))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((new_r12)*(x133)), ((new_r02)*(x133)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[6];
IkReal x134=IKsin(j8);
IkReal x135=IKcos(j8);
IkReal x136=((IkReal(1.00000000000000))*(cj9));
IkReal x137=((IkReal(1.00000000000000))*(sj9));
IkReal x138=((IkReal(1.00000000000000))*(x134));
IkReal x139=((new_r12)*(x134));
IkReal x140=((new_r02)*(x135));
evalcond[0]=((((new_r12)*(x135)))+(((IkReal(-1.00000000000000))*(new_r02)*(x138))));
evalcond[1]=((x140)+(x139)+(((IkReal(-1.00000000000000))*(x137))));
evalcond[2]=((((cj9)*(x139)))+(((cj9)*(x140)))+(((IkReal(-1.00000000000000))*(new_r22)*(x137))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r00)*(x135)*(x137)))+(((IkReal(-1.00000000000000))*(new_r10)*(x134)*(x137)))+(((IkReal(-1.00000000000000))*(new_r20)*(x136))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r21)*(x136)))+(((IkReal(-1.00000000000000))*(new_r01)*(x135)*(x137)))+(((IkReal(-1.00000000000000))*(new_r11)*(x134)*(x137))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x137)*(x140)))+(((IkReal(-1.00000000000000))*(x137)*(x139)))+(((IkReal(-1.00000000000000))*(new_r22)*(x136))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj9);
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x141=((((IkReal(-1.00000000000000))*(new_r02)*(sj8)))+(((cj8)*(new_r12))));
IkReal x142=((((new_r12)*(sj8)))+(((cj8)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x141;
evalcond[5]=x141;
evalcond[6]=x142;
evalcond[7]=x142;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj8)))+(((cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))))+IKsqr(((((new_r10)*(sj8)))+(((cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))), ((((new_r10)*(sj8)))+(((cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x143=IKsin(j10);
IkReal x144=((IkReal(1.00000000000000))*(sj8));
IkReal x145=((IkReal(1.00000000000000))*(IKcos(j10)));
evalcond[0]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x143)))+(((IkReal(-1.00000000000000))*(new_r00)*(x144))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x144)))+(((IkReal(-1.00000000000000))*(x145)))+(((cj8)*(new_r11))));
evalcond[2]=((((new_r11)*(sj8)))+(x143)+(((cj8)*(new_r01))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x145)))+(((new_r10)*(sj8)))+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x146=((IkReal(1.00000000000000))+(new_r22));
IkReal x147=((IkReal(1.00000000000000))*(new_r02));
IkReal x148=((new_r12)*(sj8));
IkReal x149=((((IkReal(-1.00000000000000))*(sj8)*(x147)))+(((cj8)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=x146;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x149;
evalcond[5]=x149;
evalcond[6]=((x148)+(((cj8)*(new_r02))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(cj8)*(x147))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x146;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((new_r11)*(sj8)))+(((cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))), ((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x150=IKcos(j10);
IkReal x151=((IkReal(1.00000000000000))*(sj8));
IkReal x152=((IkReal(1.00000000000000))*(IKsin(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(x152)))+(((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x151))));
evalcond[1]=((((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x150)))+(((IkReal(-1.00000000000000))*(new_r01)*(x151))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x152)))+(((new_r11)*(sj8)))+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(x150)+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((gconst5)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst5)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst5)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst5)*(new_r20)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[8];
IkReal x153=IKsin(j10);
IkReal x154=IKcos(j10);
IkReal x155=((IkReal(1.00000000000000))*(sj9));
IkReal x156=((new_r10)*(sj8));
IkReal x157=((cj8)*(cj9));
IkReal x158=((new_r11)*(sj8));
IkReal x159=((IkReal(1.00000000000000))*(sj8));
IkReal x160=((IkReal(1.00000000000000))*(x154));
evalcond[0]=((new_r20)+(((sj9)*(x154))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x153)*(x155)))+(new_r21));
evalcond[2]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x153)))+(((IkReal(-1.00000000000000))*(new_r00)*(x159))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x160)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x159))));
evalcond[4]=((x158)+(((cj8)*(new_r01)))+(((cj9)*(x153))));
evalcond[5]=((x156)+(((cj8)*(new_r00)))+(((IkReal(-1.00000000000000))*(cj9)*(x160))));
evalcond[6]=((((new_r01)*(x157)))+(((IkReal(-1.00000000000000))*(new_r21)*(x155)))+(x153)+(((cj9)*(x158))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x155)))+(((IkReal(-1.00000000000000))*(x160)))+(((new_r00)*(x157)))+(((cj9)*(x156))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((gconst4)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst4)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst4)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst4)*(new_r20)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[2];
evalcond[0]=((((sj9)*(IKcos(j10))))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj9)*(IKsin(j10))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x161=((gconst6)*(sj9));
if( IKabs(((new_r12)*(x161))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x161))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((new_r12)*(x161)), ((new_r02)*(x161)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[12];
IkReal x162=IKsin(j8);
IkReal x163=IKcos(j8);
IkReal x164=((IkReal(1.00000000000000))*(cj10));
IkReal x165=((IkReal(1.00000000000000))*(sj9));
IkReal x166=((IkReal(1.00000000000000))*(cj9));
IkReal x167=((IkReal(1.00000000000000))*(x162));
IkReal x168=((new_r01)*(x163));
IkReal x169=((new_r11)*(x162));
IkReal x170=((cj9)*(x163));
IkReal x171=((new_r00)*(x163));
IkReal x172=((new_r02)*(x163));
IkReal x173=((new_r10)*(x162));
IkReal x174=((new_r12)*(x162));
evalcond[0]=((((new_r12)*(x163)))+(((IkReal(-1.00000000000000))*(new_r02)*(x167))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x165)))+(x172)+(x174));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x167)))+(((new_r10)*(x163)))+(((IkReal(-1.00000000000000))*(sj10))));
evalcond[3]=((((new_r11)*(x163)))+(((IkReal(-1.00000000000000))*(x164)))+(((IkReal(-1.00000000000000))*(new_r01)*(x167))));
evalcond[4]=((x168)+(x169)+(((cj9)*(sj10))));
evalcond[5]=((x171)+(x173)+(((IkReal(-1.00000000000000))*(cj9)*(x164))));
evalcond[6]=((((new_r02)*(x170)))+(((IkReal(-1.00000000000000))*(new_r22)*(x165)))+(((cj9)*(x174))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x166)))+(((IkReal(-1.00000000000000))*(x165)*(x173)))+(((IkReal(-1.00000000000000))*(x165)*(x171))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r21)*(x166)))+(((IkReal(-1.00000000000000))*(x165)*(x168)))+(((IkReal(-1.00000000000000))*(x165)*(x169))));
evalcond[9]=((((IkReal(-1.00000000000000))*(new_r21)*(x165)))+(sj10)+(((cj9)*(x169)))+(((cj9)*(x168))));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x166)))+(((IkReal(-1.00000000000000))*(x165)*(x172)))+(((IkReal(-1.00000000000000))*(x165)*(x174))));
evalcond[11]=((((new_r00)*(x170)))+(((IkReal(-1.00000000000000))*(x164)))+(((IkReal(-1.00000000000000))*(new_r20)*(x165)))+(((cj9)*(x173))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ba32f85ca3c87502739320e9e26a5905"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
