/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:21:16.440621
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21;
x0=IKcos(j[0]);
x1=IKsin(j[1]);
x2=IKcos(j[3]);
x3=IKsin(j[3]);
x4=IKcos(j[1]);
x5=IKcos(j[2]);
x6=IKsin(j[0]);
x7=IKsin(j[2]);
x8=((IkReal(0.0450000000000000))*(x3));
x9=((IkReal(0.350000000000000))*(x4));
x10=((IkReal(0.350000000000000))*(x7));
x11=((IkReal(0.0450000000000000))*(x6));
x12=((IkReal(0.350000000000000))*(x2));
x13=((x0)*(x1));
x14=((x1)*(x5));
x15=((x0)*(x5));
x16=((x1)*(x6));
x17=((x11)*(x7));
x18=((IkReal(0.0450000000000000))*(x0)*(x7));
x19=((IkReal(0.0450000000000000))*(x15)*(x4));
x20=((x11)*(x4)*(x5));
x21=((x18)+(x20));
eetrans[0]=((IkReal(0.220000000000000))+(((IkReal(0.550000000000000))*(x13)))+(((IkReal(-1.00000000000000))*(x17)))+(x19)+(((x12)*(x13)))+(((x2)*(((((IkReal(-1.00000000000000))*(x19)))+(x17)))))+(((x3)*(((((IkReal(-1.00000000000000))*(x10)*(x6)))+(((x15)*(x9)))))))+(((x13)*(x8))));
eetrans[1]=((IkReal(0.140000000000000))+(((IkReal(0.550000000000000))*(x16)))+(((IkReal(-1.00000000000000))*(x2)*(x21)))+(x21)+(((x3)*(((((x0)*(x10)))+(((x5)*(x6)*(x9)))))))+(((x16)*(x8)))+(((x12)*(x16))));
IkReal x22=((IkReal(0.0450000000000000))*(x14));
eetrans[2]=((IkReal(0.346000000000000))+(((IkReal(-1.00000000000000))*(x22)))+(((x4)*(x8)))+(((x2)*(x22)))+(((x2)*(x9)))+(((IkReal(-0.350000000000000))*(x14)*(x3)))+(((IkReal(0.550000000000000))*(x4))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int GetNumJoints() { return 4; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x33000003; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j3,cj3,sj3,htj3,j2,cj2,sj2,htj2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij3[2], _nj3,_ij2[2], _nj2;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1;  _ij2[0] = -1; _ij2[1] = -1; _nj2 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j2=pfree[0]; cj2=cos(pfree[0]); sj2=sin(pfree[0]);
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_px=((IkReal(-0.220000000000000))+(px));
new_py=((IkReal(-0.140000000000000))+(py));
new_pz=((IkReal(-0.346000000000000))+(pz));
px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x23=IKasin(((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp)))));
j3array[0]=((IkReal(-1.36129006610583))+(((IkReal(-1.00000000000000))*(x23))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(1.78030258748397))+(x23));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x24=(sj3)*(sj3);
IkReal x25=(cj2)*(cj2);
IkReal x26=(cj3)*(cj3);
IkReal x27=((IkReal(15.5555555555556))*(sj3));
IkReal x28=((cj3)*(x25));
dummyeval[0]=((IkReal(149.382716049383))+(((IkReal(-2.00000000000000))*(x28)))+(((IkReal(190.123456790123))*(cj3)))+(((IkReal(60.4938271604938))*(x24)*(x25)))+(((IkReal(-1.00000000000000))*(x27)*(x28)))+(((IkReal(24.4444444444444))*(sj3)))+(((IkReal(60.4938271604938))*(x26)))+(x24)+(x25)+(((cj3)*(x27)))+(((x25)*(x26)))+(((x25)*(x27))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x29=((IkReal(0.0450000000000000))*(cj2));
IkReal x30=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3))));
IkReal x31=((((IkReal(-1.00000000000000))*(cj3)*(x29)))+(x29)+(((IkReal(0.350000000000000))*(cj2)*(sj3))));
if( IKabs(x30) < IKFAST_ATAN2_MAGTHRESH && IKabs(x31) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x32=((IkReal(1.00000000000000))*(IKatan2(x30, x31)));
if( ((((x30)*(x30))+((x31)*(x31)))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x30)*(x30))+((x31)*(x31)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x30)*(x30))+((x31)*(x31))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x30)*(x30))+((x31)*(x31)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x30)*(x30))+((x31)*(x31))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x33=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x30)*(x30))+((x31)*(x31)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x30)*(x30))+((x31)*(x31))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x33)))+(((IkReal(-1.00000000000000))*(x32))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x32)))+(x33));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((((sj1)*((py)*(py))))+(((sj1)*((px)*(px))))));
dummyeval[0]=((((sj1)*((py)*(py))))+(((sj1)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x34=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(pz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=x34;
evalcond[3]=x34;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x35=((IkReal(1.00000000000000))*(sj2));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x35)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x35)*((px)*(px))))));
IkReal x36=((IkReal(1.00000000000000))*(sj2));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x36)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x36)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x37=x34;
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=x37;
evalcond[3]=x37;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x38=((IkReal(0.350000000000000))*(sj3));
IkReal x39=((IkReal(0.0450000000000000))*(px));
IkReal x40=((IkReal(0.0450000000000000))*(py));
if( IKabs(((gconst2)*(((((py)*(x38)))+(((IkReal(-1.00000000000000))*(cj3)*(x40)))+(x40))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(cj3)*(x39)))+(((px)*(x38)))+(x39))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((py)*(x38)))+(((IkReal(-1.00000000000000))*(cj3)*(x40)))+(x40)))), ((gconst2)*(((((IkReal(-1.00000000000000))*(cj3)*(x39)))+(((px)*(x38)))+(x39)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x41=IKcos(j0);
IkReal x42=IKsin(j0);
IkReal x43=((IkReal(1.00000000000000))*(py));
IkReal x44=((px)*(x41));
evalcond[0]=((((IkReal(-1.00000000000000))*(x41)*(x43)))+(((px)*(x42))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x44)))+(((IkReal(-1.00000000000000))*(x42)*(x43)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.0900000000000000))*(x44)))+(((IkReal(0.0900000000000000))*(py)*(x42)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x45=x34;
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=x45;
evalcond[3]=x45;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x46=((IkReal(0.0450000000000000))*(px));
IkReal x47=((IkReal(0.350000000000000))*(sj3));
IkReal x48=((IkReal(0.0450000000000000))*(py));
if( IKabs(((gconst3)*(((((py)*(x47)))+(((IkReal(-1.00000000000000))*(cj3)*(x48)))+(x48))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(cj3)*(x46)))+(x46)+(((px)*(x47))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst3)*(((((py)*(x47)))+(((IkReal(-1.00000000000000))*(cj3)*(x48)))+(x48)))), ((gconst3)*(((((IkReal(-1.00000000000000))*(cj3)*(x46)))+(x46)+(((px)*(x47)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x49=IKsin(j0);
IkReal x50=IKcos(j0);
IkReal x51=((px)*(x50));
IkReal x52=((py)*(x49));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x50)))+(((px)*(x49))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(x51)+(x52));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(x52)))+(((IkReal(-0.0900000000000000))*(x51))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x53=(sj2)*(sj2);
IkReal x54=((cj2)*(sj2));
IkReal x55=((cj3)*(py));
IkReal x56=((IkReal(0.0450000000000000))*(cj3)*(px));
IkReal x57=((IkReal(0.350000000000000))*(px)*(sj3));
IkReal x58=((IkReal(0.0450000000000000))*(x53));
IkReal x59=((IkReal(0.350000000000000))*(py)*(sj3));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x53)*(x56)))+(((IkReal(0.0450000000000000))*(x54)*(x55)))+(((px)*(x58)))+(((IkReal(-1.00000000000000))*(x54)*(x59)))+(((x53)*(x57)))+(((IkReal(-0.0450000000000000))*(py)*(x54))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x53)*(x59)))+(((IkReal(-0.0450000000000000))*(px)*(x54)))+(((IkReal(-1.00000000000000))*(x54)*(x57)))+(((IkReal(-1.00000000000000))*(py)*(x58)))+(((x55)*(x58)))+(((x54)*(x56))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x53)*(x56)))+(((IkReal(0.0450000000000000))*(x54)*(x55)))+(((px)*(x58)))+(((IkReal(-1.00000000000000))*(x54)*(x59)))+(((x53)*(x57)))+(((IkReal(-0.0450000000000000))*(py)*(x54)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x53)*(x59)))+(((IkReal(-0.0450000000000000))*(px)*(x54)))+(((IkReal(-1.00000000000000))*(x54)*(x57)))+(((IkReal(-1.00000000000000))*(py)*(x58)))+(((x55)*(x58)))+(((x54)*(x56)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x60=IKcos(j0);
IkReal x61=IKsin(j0);
IkReal x62=((IkReal(0.0450000000000000))*(cj2));
IkReal x63=((IkReal(0.0900000000000000))*(sj2));
IkReal x64=((IkReal(0.0450000000000000))*(cj3));
IkReal x65=((IkReal(0.0900000000000000))*(cj2));
IkReal x66=((IkReal(0.350000000000000))*(sj3));
IkReal x67=((IkReal(1.00000000000000))*(cj2));
IkReal x68=((px)*(x61));
IkReal x69=((px)*(x60));
IkReal x70=((py)*(x60));
IkReal x71=((py)*(x61));
evalcond[0]=((((sj2)*(x66)))+(((IkReal(0.0450000000000000))*(sj2)))+(x68)+(((IkReal(-1.00000000000000))*(x70)))+(((IkReal(-1.00000000000000))*(sj2)*(x64))));
evalcond[1]=((((sj2)*(x69)))+(((sj2)*(x71)))+(((IkReal(-1.00000000000000))*(x67)*(x70)))+(((cj2)*(x68))));
evalcond[2]=((IkReal(0.0450000000000000))+(((sj2)*(x68)))+(((IkReal(-1.00000000000000))*(x67)*(x69)))+(x66)+(((IkReal(-1.00000000000000))*(x67)*(x71)))+(((IkReal(-1.00000000000000))*(sj2)*(x70)))+(((IkReal(-1.00000000000000))*(x64))));
evalcond[3]=((x62)+(((IkReal(-1.00000000000000))*(x71)))+(((IkReal(-1.00000000000000))*(cj3)*(x62)))+(((cj2)*(x66)))+(((IkReal(-1.00000000000000))*(x69))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((x63)*(x70)))+(((IkReal(-1.00000000000000))*(x63)*(x68)))+(((x65)*(x71)))+(((x65)*(x69))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x72=((IkReal(0.0450000000000000))*(sj3));
IkReal x73=((IkReal(0.350000000000000))*(cj3));
IkReal x74=((x73)+(x72));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x74)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[3]=((IkReal(0.550000000000000))+(pz)+(x74));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x75=((cj2)*(py));
IkReal x76=((IkReal(0.0450000000000000))*(cj3));
IkReal x77=((IkReal(0.350000000000000))*(sj3));
IkReal x78=((py)*(sj2));
IkReal x79=((px)*(sj2));
IkReal x80=((cj2)*(px));
if( IKabs(((gconst4)*(((((IkReal(-0.0450000000000000))*(x79)))+(((IkReal(-0.0450000000000000))*(x75)))+(((x75)*(x76)))+(((x76)*(x79)))+(((IkReal(-1.00000000000000))*(x77)*(x79)))+(((IkReal(-1.00000000000000))*(x75)*(x77))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(0.0450000000000000))*(x78)))+(((IkReal(-1.00000000000000))*(x77)*(x80)))+(((x76)*(x80)))+(((IkReal(-1.00000000000000))*(x76)*(x78)))+(((x77)*(x78)))+(((IkReal(-0.0450000000000000))*(x80))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst4)*(((((IkReal(-0.0450000000000000))*(x79)))+(((IkReal(-0.0450000000000000))*(x75)))+(((x75)*(x76)))+(((x76)*(x79)))+(((IkReal(-1.00000000000000))*(x77)*(x79)))+(((IkReal(-1.00000000000000))*(x75)*(x77)))))), ((gconst4)*(((((IkReal(0.0450000000000000))*(x78)))+(((IkReal(-1.00000000000000))*(x77)*(x80)))+(((x76)*(x80)))+(((IkReal(-1.00000000000000))*(x76)*(x78)))+(((x77)*(x78)))+(((IkReal(-0.0450000000000000))*(x80)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x81=IKsin(j0);
IkReal x82=IKcos(j0);
IkReal x83=((IkReal(0.0450000000000000))*(cj2));
IkReal x84=((IkReal(0.0900000000000000))*(sj2));
IkReal x85=((IkReal(0.0450000000000000))*(cj3));
IkReal x86=((IkReal(1.00000000000000))*(sj2));
IkReal x87=((IkReal(0.0900000000000000))*(cj2));
IkReal x88=((IkReal(0.350000000000000))*(sj3));
IkReal x89=((px)*(x81));
IkReal x90=((px)*(x82));
IkReal x91=((py)*(x82));
IkReal x92=((py)*(x81));
evalcond[0]=((((sj2)*(x88)))+(((IkReal(-1.00000000000000))*(sj2)*(x85)))+(((IkReal(0.0450000000000000))*(sj2)))+(x89)+(((IkReal(-1.00000000000000))*(x91))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(x91)))+(((cj2)*(x89)))+(((IkReal(-1.00000000000000))*(x86)*(x90)))+(((IkReal(-1.00000000000000))*(x86)*(x92))));
evalcond[2]=((IkReal(0.0450000000000000))+(((sj2)*(x89)))+(((cj2)*(x92)))+(((cj2)*(x90)))+(((IkReal(-1.00000000000000))*(x86)*(x91)))+(x88)+(((IkReal(-1.00000000000000))*(x85))));
evalcond[3]=((((cj3)*(x83)))+(((IkReal(-1.00000000000000))*(x92)))+(((IkReal(-1.00000000000000))*(x90)))+(((IkReal(-1.00000000000000))*(x83)))+(((IkReal(-1.00000000000000))*(cj2)*(x88))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(x87)*(x92)))+(((IkReal(-1.00000000000000))*(x87)*(x90)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x84)*(x89)))+(((x84)*(x91))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x93=((IkReal(0.350000000000000))*(sj3));
IkReal x94=((IkReal(0.0450000000000000))*(px));
IkReal x95=((IkReal(0.350000000000000))*(cj3));
IkReal x96=((sj1)*(sj2));
IkReal x97=((py)*(x96));
IkReal x98=((IkReal(1.00000000000000))*(cj1)*(pz));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(x94)*(x96)))+(((IkReal(-1.00000000000000))*(px)*(x93)*(x96)))+(((IkReal(0.0450000000000000))*(py)*(sj3)))+(((cj3)*(x94)*(x96)))+(((py)*(x95)))+(((IkReal(0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x98))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((sj3)*(x94)))+(((IkReal(-0.0450000000000000))*(cj3)*(x97)))+(((IkReal(0.0450000000000000))*(x97)))+(((x93)*(x97)))+(((px)*(x95)))+(((IkReal(-1.00000000000000))*(px)*(x98)))+(((IkReal(0.550000000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(x94)*(x96)))+(((IkReal(-1.00000000000000))*(px)*(x93)*(x96)))+(((IkReal(0.0450000000000000))*(py)*(sj3)))+(((cj3)*(x94)*(x96)))+(((py)*(x95)))+(((IkReal(0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x98)))))), ((gconst0)*(((((sj3)*(x94)))+(((IkReal(-0.0450000000000000))*(cj3)*(x97)))+(((IkReal(0.0450000000000000))*(x97)))+(((x93)*(x97)))+(((px)*(x95)))+(((IkReal(-1.00000000000000))*(px)*(x98)))+(((IkReal(0.550000000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[6];
IkReal x99=IKsin(j0);
IkReal x100=IKcos(j0);
IkReal x101=((cj1)*(sj2));
IkReal x102=((IkReal(0.0900000000000000))*(py));
IkReal x103=((IkReal(0.350000000000000))*(cj3));
IkReal x104=((IkReal(0.0450000000000000))*(cj3));
IkReal x105=((IkReal(1.10000000000000))*(sj1));
IkReal x106=((IkReal(0.350000000000000))*(sj3));
IkReal x107=((cj1)*(cj2));
IkReal x108=((IkReal(1.00000000000000))*(py));
IkReal x109=((IkReal(0.0450000000000000))*(sj3));
IkReal x110=((cj1)*(pz));
IkReal x111=((px)*(x99));
IkReal x112=((px)*(x100));
IkReal x113=((sj2)*(x100));
IkReal x114=((py)*(x99));
IkReal x115=((cj2)*(pz)*(sj1));
evalcond[0]=((IkReal(-0.550000000000000))+(((sj1)*(x114)))+(((sj1)*(x112)))+(((IkReal(-1.00000000000000))*(x109)))+(((IkReal(-1.00000000000000))*(x103)))+(x110));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj2)*(x104)))+(((sj2)*(x106)))+(((IkReal(-1.00000000000000))*(x100)*(x108)))+(((IkReal(0.0450000000000000))*(sj2)))+(x111));
evalcond[2]=((((cj2)*(x111)))+(((IkReal(-1.00000000000000))*(cj2)*(x100)*(x108)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x101)*(x114)))+(((x101)*(x112))));
evalcond[3]=((IkReal(0.0450000000000000))+(((sj2)*(x111)))+(((IkReal(-1.00000000000000))*(x104)))+(x115)+(x106)+(((IkReal(-1.00000000000000))*(x108)*(x113)))+(((IkReal(-1.00000000000000))*(x107)*(x112)))+(((IkReal(-1.00000000000000))*(x107)*(x108)*(x99))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x104)*(x107)))+(((sj1)*(x103)))+(((sj1)*(x109)))+(((x106)*(x107)))+(((IkReal(0.0450000000000000))*(x107)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(x108)*(x99)))+(((IkReal(-1.00000000000000))*(x112))));
evalcond[5]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(sj2)*(x111)))+(((IkReal(-0.0900000000000000))*(x115)))+(((x102)*(x113)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(x107)*(x112)))+(((IkReal(1.10000000000000))*(x110)))+(((x102)*(x107)*(x99)))+(((x105)*(x112)))+(((x105)*(x114))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x116=((IkReal(0.0450000000000000))*(sj2));
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x117=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x116)))+(x116)+(((IkReal(0.350000000000000))*(sj2)*(sj3))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x116)))+(x116)+(((IkReal(0.350000000000000))*(sj2)*(sj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x118=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x116)))+(x116)+(((IkReal(0.350000000000000))*(sj2)*(sj3)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x118))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x117)))+(x118));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
IkReal x119=(py)*(py);
IkReal x120=(cj2)*(cj2);
IkReal x121=(sj2)*(sj2);
IkReal x122=IKcos(j0);
IkReal x123=IKsin(j0);
IkReal x124=(px)*(px);
IkReal x125=((py)*(sj2));
IkReal x126=((IkReal(0.0450000000000000))*(cj3));
IkReal x127=((IkReal(0.350000000000000))*(sj3));
IkReal x128=((px)*(sj2));
IkReal x129=((IkReal(1.00000000000000))*(x124));
IkReal x130=((px)*(py)*(x121));
IkReal x131=((px)*(py)*(x120));
evalcond[0]=((((x122)*(((((x119)*(x121)))+(((x119)*(x120)))))))+(((x125)*(x126)))+(((IkReal(-1.00000000000000))*(x125)*(x127)))+(((x123)*(((((IkReal(-1.00000000000000))*(x131)))+(((IkReal(-1.00000000000000))*(x130)))))))+(((IkReal(-0.0450000000000000))*(x125))));
evalcond[1]=((((x126)*(x128)))+(((x122)*(((x131)+(x130)))))+(((IkReal(-1.00000000000000))*(x127)*(x128)))+(((x123)*(((((IkReal(-1.00000000000000))*(x120)*(x129)))+(((IkReal(-1.00000000000000))*(x121)*(x129)))))))+(((IkReal(-0.0450000000000000))*(x128))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x132=((cj0)*(px));
IkReal x133=((cj2)*(pz));
IkReal x134=((py)*(sj0));
IkReal x135=((IkReal(0.350000000000000))*(cj3));
IkReal x136=((IkReal(0.0450000000000000))*(sj3));
gconst5=IKsign(((((IkReal(-0.0450000000000000))*(x133)))+(((IkReal(-0.550000000000000))*(x134)))+(((IkReal(-0.550000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x134)*(x135)))+(((IkReal(-1.00000000000000))*(x134)*(x136)))+(((IkReal(-1.00000000000000))*(x132)*(x135)))+(((IkReal(-1.00000000000000))*(x132)*(x136)))+(((IkReal(-0.350000000000000))*(sj3)*(x133)))+(((IkReal(0.0450000000000000))*(cj3)*(x133)))));
IkReal x137=((py)*(sj0));
IkReal x138=((IkReal(7.77777777777778))*(cj3));
IkReal x139=((cj2)*(pz));
IkReal x140=((cj0)*(px));
IkReal x141=((IkReal(1.00000000000000))*(sj3));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x137)*(x141)))+(((IkReal(-1.00000000000000))*(x138)*(x140)))+(((IkReal(-12.2222222222222))*(x137)))+(((IkReal(-1.00000000000000))*(x140)*(x141)))+(((cj3)*(x139)))+(((IkReal(-1.00000000000000))*(x137)*(x138)))+(((IkReal(-7.77777777777778))*(sj3)*(x139)))+(((IkReal(-1.00000000000000))*(x139)))+(((IkReal(-12.2222222222222))*(x140))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x142=(cj3)*(cj3);
IkReal x143=(sj3)*(sj3);
IkReal x144=((cj3)*(sj3));
IkReal x145=((IkReal(0.0157500000000000))*(cj2));
IkReal x146=((IkReal(1.00000000000000))*(pz));
if( IKabs(((gconst5)*(((IkReal(-0.302500000000000))+(((IkReal(-0.0315000000000000))*(x144)))+(((IkReal(-0.385000000000000))*(cj3)))+(((IkReal(-0.00202500000000000))*(x143)))+(((IkReal(-0.0495000000000000))*(sj3)))+((pz)*(pz))+(((IkReal(-0.122500000000000))*(x142))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(0.00900000000000000))*(cj2)*(cj3)))+(((x142)*(x145)))+(((IkReal(-1.00000000000000))*(x143)*(x145)))+(((IkReal(-0.120475000000000))*(cj2)*(x144)))+(((IkReal(-0.0247500000000000))*(cj2)))+(((IkReal(-0.194525000000000))*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x146)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(x146))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst5)*(((IkReal(-0.302500000000000))+(((IkReal(-0.0315000000000000))*(x144)))+(((IkReal(-0.385000000000000))*(cj3)))+(((IkReal(-0.00202500000000000))*(x143)))+(((IkReal(-0.0495000000000000))*(sj3)))+((pz)*(pz))+(((IkReal(-0.122500000000000))*(x142)))))), ((gconst5)*(((((IkReal(0.00900000000000000))*(cj2)*(cj3)))+(((x142)*(x145)))+(((IkReal(-1.00000000000000))*(x143)*(x145)))+(((IkReal(-0.120475000000000))*(cj2)*(x144)))+(((IkReal(-0.0247500000000000))*(cj2)))+(((IkReal(-0.194525000000000))*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x146)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(x146)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[6];
IkReal x147=IKsin(j1);
IkReal x148=IKcos(j1);
IkReal x149=((px)*(sj2));
IkReal x150=((IkReal(0.0450000000000000))*(cj2));
IkReal x151=((IkReal(0.0900000000000000))*(sj0));
IkReal x152=((cj2)*(px));
IkReal x153=((py)*(sj0));
IkReal x154=((IkReal(1.00000000000000))*(cj2));
IkReal x155=((IkReal(0.350000000000000))*(cj2));
IkReal x156=((IkReal(0.0450000000000000))*(sj3));
IkReal x157=((cj0)*(py));
IkReal x158=((IkReal(1.00000000000000))*(sj2));
IkReal x159=((cj0)*(px));
IkReal x160=((cj0)*(x148));
IkReal x161=((cj3)*(x147));
IkReal x162=((sj3)*(x147));
IkReal x163=((cj3)*(x148));
IkReal x164=((pz)*(x148));
IkReal x165=((pz)*(x147));
IkReal x166=((IkReal(1.10000000000000))*(x147));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.350000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x156)))+(x164)+(((x147)*(x159)))+(((x147)*(x153))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x154)*(x157)))+(((IkReal(-1.00000000000000))*(x158)*(x165)))+(((x149)*(x160)))+(((sj0)*(x152)))+(((sj2)*(x148)*(x153))));
evalcond[2]=((((IkReal(-0.350000000000000))*(x163)))+(((IkReal(-1.00000000000000))*(x150)*(x161)))+(pz)+(((IkReal(-1.00000000000000))*(x148)*(x156)))+(((x155)*(x162)))+(((x147)*(x150)))+(((IkReal(-0.550000000000000))*(x148))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x157)*(x158)))+(((IkReal(-1.00000000000000))*(x152)*(x160)))+(((sj0)*(x149)))+(((cj2)*(x165)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(x148)*(x153)*(x154)))+(((IkReal(-0.0450000000000000))*(cj3))));
evalcond[4]=((((sj3)*(x148)*(x155)))+(((IkReal(-1.00000000000000))*(x150)*(x163)))+(((IkReal(0.550000000000000))*(x147)))+(((IkReal(-1.00000000000000))*(x159)))+(((IkReal(-1.00000000000000))*(x153)))+(((x147)*(x156)))+(((x148)*(x150)))+(((IkReal(0.350000000000000))*(x161))));
evalcond[5]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(cj2)*(x165)))+(((IkReal(1.10000000000000))*(x164)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(sj2)*(x157)))+(((IkReal(0.0900000000000000))*(x152)*(x160)))+(((cj2)*(py)*(x148)*(x151)))+(((IkReal(-1.00000000000000))*(x149)*(x151)))+(((x159)*(x166)))+(((x153)*(x166))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x167=((IkReal(0.0450000000000000))*(sj2));
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x168=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((x167)+(((IkReal(0.350000000000000))*(sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x167))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((x167)+(((IkReal(0.350000000000000))*(sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x167))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x169=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((x167)+(((IkReal(0.350000000000000))*(sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x167)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x168)))+(((IkReal(-1.00000000000000))*(x169))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x168)))+(x169));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x170=((cj0)*(px));
IkReal x171=((cj2)*(pz));
IkReal x172=((py)*(sj0));
IkReal x173=((IkReal(0.350000000000000))*(cj3));
IkReal x174=((IkReal(0.0450000000000000))*(sj3));
gconst5=IKsign(((((IkReal(0.0450000000000000))*(cj3)*(x171)))+(((IkReal(-0.350000000000000))*(sj3)*(x171)))+(((IkReal(-1.00000000000000))*(x170)*(x174)))+(((IkReal(-1.00000000000000))*(x170)*(x173)))+(((IkReal(-0.0450000000000000))*(x171)))+(((IkReal(-0.550000000000000))*(x172)))+(((IkReal(-0.550000000000000))*(x170)))+(((IkReal(-1.00000000000000))*(x172)*(x173)))+(((IkReal(-1.00000000000000))*(x172)*(x174)))));
IkReal x175=((py)*(sj0));
IkReal x176=((IkReal(7.77777777777778))*(cj3));
IkReal x177=((cj2)*(pz));
IkReal x178=((cj0)*(px));
IkReal x179=((IkReal(1.00000000000000))*(sj3));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x175)*(x176)))+(((IkReal(-1.00000000000000))*(x175)*(x179)))+(((IkReal(-1.00000000000000))*(x177)))+(((cj3)*(x177)))+(((IkReal(-7.77777777777778))*(sj3)*(x177)))+(((IkReal(-1.00000000000000))*(x176)*(x178)))+(((IkReal(-1.00000000000000))*(x178)*(x179)))+(((IkReal(-12.2222222222222))*(x175)))+(((IkReal(-12.2222222222222))*(x178))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x180=(cj3)*(cj3);
IkReal x181=(sj3)*(sj3);
IkReal x182=((cj3)*(sj3));
IkReal x183=((IkReal(0.0157500000000000))*(cj2));
IkReal x184=((IkReal(1.00000000000000))*(pz));
if( IKabs(((gconst5)*(((IkReal(-0.302500000000000))+(((IkReal(-0.385000000000000))*(cj3)))+(((IkReal(-0.0315000000000000))*(x182)))+(((IkReal(-0.00202500000000000))*(x181)))+(((IkReal(-0.0495000000000000))*(sj3)))+((pz)*(pz))+(((IkReal(-0.122500000000000))*(x180))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(0.00900000000000000))*(cj2)*(cj3)))+(((x180)*(x183)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(x184)))+(((IkReal(-0.0247500000000000))*(cj2)))+(((IkReal(-0.194525000000000))*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x184)))+(((IkReal(-1.00000000000000))*(x181)*(x183)))+(((IkReal(-0.120475000000000))*(cj2)*(x182))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst5)*(((IkReal(-0.302500000000000))+(((IkReal(-0.385000000000000))*(cj3)))+(((IkReal(-0.0315000000000000))*(x182)))+(((IkReal(-0.00202500000000000))*(x181)))+(((IkReal(-0.0495000000000000))*(sj3)))+((pz)*(pz))+(((IkReal(-0.122500000000000))*(x180)))))), ((gconst5)*(((((IkReal(0.00900000000000000))*(cj2)*(cj3)))+(((x180)*(x183)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(x184)))+(((IkReal(-0.0247500000000000))*(cj2)))+(((IkReal(-0.194525000000000))*(cj2)*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x184)))+(((IkReal(-1.00000000000000))*(x181)*(x183)))+(((IkReal(-0.120475000000000))*(cj2)*(x182)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[6];
IkReal x185=IKsin(j1);
IkReal x186=IKcos(j1);
IkReal x187=((px)*(sj2));
IkReal x188=((IkReal(0.0450000000000000))*(cj2));
IkReal x189=((IkReal(0.0900000000000000))*(sj0));
IkReal x190=((cj2)*(px));
IkReal x191=((py)*(sj0));
IkReal x192=((IkReal(1.00000000000000))*(cj2));
IkReal x193=((IkReal(0.350000000000000))*(cj2));
IkReal x194=((IkReal(0.0450000000000000))*(sj3));
IkReal x195=((cj0)*(py));
IkReal x196=((IkReal(1.00000000000000))*(sj2));
IkReal x197=((cj0)*(px));
IkReal x198=((cj0)*(x186));
IkReal x199=((cj3)*(x185));
IkReal x200=((sj3)*(x185));
IkReal x201=((cj3)*(x186));
IkReal x202=((pz)*(x186));
IkReal x203=((pz)*(x185));
IkReal x204=((IkReal(1.10000000000000))*(x185));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.350000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x194)))+(((x185)*(x191)))+(((x185)*(x197)))+(x202));
evalcond[1]=((((IkReal(-1.00000000000000))*(x192)*(x195)))+(((x187)*(x198)))+(((sj0)*(x190)))+(((IkReal(-1.00000000000000))*(x196)*(x203)))+(((sj2)*(x186)*(x191))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x188)*(x199)))+(((IkReal(-0.350000000000000))*(x201)))+(((IkReal(-1.00000000000000))*(x186)*(x194)))+(((x185)*(x188)))+(pz)+(((IkReal(-0.550000000000000))*(x186)))+(((x193)*(x200))));
evalcond[3]=((IkReal(0.0450000000000000))+(((cj2)*(x203)))+(((IkReal(-1.00000000000000))*(x195)*(x196)))+(((sj0)*(x187)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x186)*(x191)*(x192)))+(((IkReal(-1.00000000000000))*(x190)*(x198))));
evalcond[4]=((((x186)*(x188)))+(((IkReal(-1.00000000000000))*(x191)))+(((IkReal(-1.00000000000000))*(x197)))+(((x185)*(x194)))+(((IkReal(0.550000000000000))*(x185)))+(((sj3)*(x186)*(x193)))+(((IkReal(-1.00000000000000))*(x188)*(x201)))+(((IkReal(0.350000000000000))*(x199))));
evalcond[5]=((IkReal(-0.180000000000000))+(((IkReal(0.0900000000000000))*(sj2)*(x195)))+(((IkReal(-1.00000000000000))*(x187)*(x189)))+(((x191)*(x204)))+(((x197)*(x204)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj2)*(py)*(x186)*(x189)))+(((IkReal(0.0900000000000000))*(x190)*(x198)))+(((IkReal(1.10000000000000))*(x202)))+(((IkReal(-0.0900000000000000))*(cj2)*(x203))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "4382fadc0284794e13f46b1dd48b52ef"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
