/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 06:59:17.218849
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22;
x0=IKcos(j[0]);
x1=IKsin(j[1]);
x2=IKsin(j[2]);
x3=IKcos(j[1]);
x4=IKcos(j[2]);
x5=IKsin(j[0]);
x6=IKcos(j[4]);
x7=IKsin(j[4]);
x8=IKsin(j[3]);
x9=IKcos(j[3]);
x10=((IkReal(1.00000000000000))*(x5));
x11=((IkReal(1.00000000000000))*(x1));
x12=((IkReal(0.433100000000000))*(x5));
x13=((IkReal(0.431800000000000))*(x3));
x14=((IkReal(0.0203000000000000))*(x2));
x15=((IkReal(0.433100000000000))*(x1));
x16=((x3)*(x4));
x17=((x0)*(x4));
x18=((x1)*(x4));
x19=((x0)*(x2));
x20=((x2)*(x3));
x21=((IkReal(0.0203000000000000))*(x16));
x22=((IkReal(1.00000000000000))*(x20));
eetrans[0]=((((x0)*(x13)))+(((IkReal(-1.00000000000000))*(x0)*(x21)))+(((x0)*(x1)*(x14)))+(((IkReal(0.433100000000000))*(x19)*(x3)))+(((IkReal(-0.150100000000000))*(x5)))+(((x15)*(x17))));
eetrans[1]=((((IkReal(-1.00000000000000))*(x21)*(x5)))+(((IkReal(0.150100000000000))*(x0)))+(((x1)*(x14)*(x5)))+(((x12)*(x20)))+(((x12)*(x18)))+(((x13)*(x5))));
eetrans[2]=((IkReal(1.37100000000000))+(((IkReal(-0.431800000000000))*(x1)))+(((IkReal(0.433100000000000))*(x16)))+(((x14)*(x3)))+(((IkReal(-1.00000000000000))*(x15)*(x2)))+(((IkReal(0.0203000000000000))*(x18))));
IkReal x23=((IkReal(1.00000000000000))*(x19));
eerot[0]=((((x7)*(((((IkReal(-1.00000000000000))*(x10)*(x8)))+(((x9)*(((((x0)*(x16)))+(((IkReal(-1.00000000000000))*(x11)*(x23)))))))))))+(((x6)*(((((x23)*(x3)))+(((x11)*(x17))))))));
eerot[1]=((((x6)*(((((x10)*(x20)))+(((x10)*(x18)))))))+(((x7)*(((((x0)*(x8)))+(((x9)*(((((IkReal(-1.00000000000000))*(x1)*(x10)*(x2)))+(((x16)*(x5))))))))))));
IkReal x24=((IkReal(1.00000000000000))*(x11));
eerot[2]=((((x7)*(x9)*(((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x24)*(x4)))))))+(((x6)*(((((IkReal(1.00000000000000))*(x16)))+(((IkReal(-1.00000000000000))*(x2)*(x24))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {5}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1;  _ij5[0] = -1; _ij5[1] = -1; _nj5 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j5=pfree[0]; cj5=cos(pfree[0]); sj5=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((((IkReal(-0.0940000000000000))*(r00)))+(px));
new_r01=r01;
new_py=((((IkReal(-0.0940000000000000))*(r01)))+(py));
new_r02=r02;
new_pz=((IkReal(-1.37100000000000))+(((IkReal(-0.0940000000000000))*(r02)))+(pz));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
if( (((IkReal(-1.06017897616850))+(((IkReal(2.67068488900329))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.06017897616850))+(((IkReal(2.67068488900329))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x25=IKasin(((IkReal(-1.06017897616850))+(((IkReal(2.67068488900329))*(pp)))));
j2array[0]=((IkReal(0.0468371131132389))+(((IkReal(1.00000000000000))*(x25))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.18842976670303))+(((IkReal(-1.00000000000000))*(x25))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(10.6354679802956))+(((IkReal(10.7231100422792))*((cj2)*(cj2))))+(((IkReal(-1.00000000000000))*(cj2)))+(((IkReal(21.3349753694581))*(sj2)))+(((IkReal(10.7231100422792))*((sj2)*(sj2)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x26=((((IkReal(-0.433100000000000))*(cj2)))+(((IkReal(-0.0203000000000000))*(sj2))));
IkReal x27=((IkReal(0.431800000000000))+(((IkReal(-0.0203000000000000))*(cj2)))+(((IkReal(0.433100000000000))*(sj2))));
if( IKabs(x26) < IKFAST_ATAN2_MAGTHRESH && IKabs(x27) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x28=((IkReal(1.00000000000000))*(IKatan2(x26, x27)));
if( ((((x26)*(x26))+((x27)*(x27)))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+((x27)*(x27)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+((x27)*(x27))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+((x27)*(x27)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+((x27)*(x27))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x29=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+((x27)*(x27)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+((x27)*(x27))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x28)))+(((IkReal(-1.00000000000000))*(x29))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x28)))+(x29));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x30=((IkReal(10000.0000000000))*(sj1));
gconst0=IKsign(((((x30)*((py)*(py))))+(((x30)*((px)*(px))))));
dummyeval[0]=((((sj1)*((py)*(py))))+(((sj1)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x31=((((IkReal(-0.433100000000000))*(cj2)))+(pz)+(((IkReal(-0.0203000000000000))*(sj2))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396968950000000))+(((IkReal(-0.0175310800000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.374025160000000))*(sj2))));
evalcond[2]=x31;
evalcond[3]=x31;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-21590000.0000000))*((px)*(px))))+(((IkReal(-21590000.0000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x32=((IkReal(25000000.0000000))*(pp));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x32)))+(((IkReal(601661.750000000))*(py)))+(((IkReal(3240659.00000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-3240659.00000000))*(py)))+(((IkReal(601661.750000000))*(px)))+(((IkReal(-1.00000000000000))*(px)*(x32))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x32)))+(((IkReal(601661.750000000))*(py)))+(((IkReal(3240659.00000000))*(px)))))), ((gconst1)*(((((IkReal(-3240659.00000000))*(py)))+(((IkReal(601661.750000000))*(px)))+(((IkReal(-1.00000000000000))*(px)*(x32)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x33=IKsin(j0);
IkReal x34=IKcos(j0);
IkReal x35=((px)*(x34));
IkReal x36=((py)*(x33));
evalcond[0]=((IkReal(0.150100000000000))+(((IkReal(-1.00000000000000))*(py)*(x34)))+(((px)*(x33))));
evalcond[1]=((IkReal(0.0240664700000000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.863600000000000))*(x36)))+(((IkReal(0.863600000000000))*(x35))));
evalcond[2]=((IkReal(0.431800000000000))+(((IkReal(-1.00000000000000))*(x36)))+(((IkReal(-1.00000000000000))*(x35)))+(((IkReal(-0.0203000000000000))*(cj2)))+(((IkReal(0.433100000000000))*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x37=((IkReal(1.00000000000000))*(sj1));
cj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x37)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(sj2)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x37)))+(((cj1)*(cj2)*(r02))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x38=((r02)*(sj0));
IkReal x39=((cj2)*(sj1));
IkReal x40=((cj1)*(sj2));
IkReal x41=((cj0)*(r00));
IkReal x42=((cj1)*(cj2));
IkReal x43=((cj0)*(r02));
IkReal x44=((IkReal(1.00000000000000))*(sj0));
IkReal x45=((sj1)*(sj2));
if( IKabs(((gconst4)*(((((x38)*(x39)))+(((r00)*(sj0)*(x42)))+(((x38)*(x40)))+(((IkReal(-1.00000000000000))*(r00)*(x44)*(x45)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x41)*(x42)))+(((IkReal(-1.00000000000000))*(r01)*(x44)))+(((x40)*(x43)))+(((x39)*(x43)))+(((IkReal(-1.00000000000000))*(x41)*(x45))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x38)*(x39)))+(((r00)*(sj0)*(x42)))+(((x38)*(x40)))+(((IkReal(-1.00000000000000))*(r00)*(x44)*(x45)))+(((cj0)*(r01)))))), ((gconst4)*(((((x41)*(x42)))+(((IkReal(-1.00000000000000))*(r01)*(x44)))+(((x40)*(x43)))+(((x39)*(x43)))+(((IkReal(-1.00000000000000))*(x41)*(x45)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x46=((IkReal(1.00000000000000))*(r02));
IkReal x47=((sj0)*(sj1));
IkReal x48=((cj1)*(sj2));
IkReal x49=((IkReal(1.00000000000000))*(cj0));
IkReal x50=((r00)*(sj2));
IkReal x51=((cj0)*(sj1));
IkReal x52=((cj1)*(cj2)*(r00));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(x46)*(x51)))+(((sj4)*(IKcos(j3))))+(((r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x46)*(x48)))+(((IkReal(-1.00000000000000))*(x49)*(x52)))+(((x50)*(x51))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj0)*(x46)*(x48)))+(((IkReal(-1.00000000000000))*(r01)*(x49)))+(((IkReal(-1.00000000000000))*(sj0)*(x52)))+(((x47)*(x50)))+(((sj4)*(IKsin(j3))))+(((IkReal(-1.00000000000000))*(cj2)*(x46)*(x47))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
IkReal x53=((IkReal(0.0203000000000000))*(sj2));
IkReal x54=((IkReal(0.433100000000000))*(cj2));
IkReal x55=((x54)+(x53));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396968950000000))+(((IkReal(-0.0175310800000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.374025160000000))*(sj2))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x55))));
evalcond[3]=((pz)+(x55));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(21590000.0000000))*((px)*(px))))+(((IkReal(21590000.0000000))*((py)*(py))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x56=((IkReal(25000000.0000000))*(pp));
if( IKabs(((gconst2)*(((((IkReal(-3240659.00000000))*(px)))+(((IkReal(601661.750000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x56))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(px)*(x56)))+(((IkReal(601661.750000000))*(px)))+(((IkReal(3240659.00000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(-3240659.00000000))*(px)))+(((IkReal(601661.750000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x56)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(px)*(x56)))+(((IkReal(601661.750000000))*(px)))+(((IkReal(3240659.00000000))*(py)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x57=IKsin(j0);
IkReal x58=IKcos(j0);
IkReal x59=((px)*(x58));
IkReal x60=((py)*(x57));
evalcond[0]=((IkReal(0.150100000000000))+(((IkReal(-1.00000000000000))*(py)*(x58)))+(((px)*(x57))));
evalcond[1]=((IkReal(0.0240664700000000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.863600000000000))*(x60)))+(((IkReal(-0.863600000000000))*(x59))));
evalcond[2]=((IkReal(0.431800000000000))+(x59)+(x60)+(((IkReal(-0.0203000000000000))*(cj2)))+(((IkReal(0.433100000000000))*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x61=((IkReal(1.00000000000000))*(sj1));
cj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x61)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(sj2)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x61)))+(((cj1)*(cj2)*(r02))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x62=((r02)*(sj0));
IkReal x63=((cj2)*(sj1));
IkReal x64=((cj1)*(sj2));
IkReal x65=((cj0)*(r00));
IkReal x66=((cj1)*(cj2));
IkReal x67=((cj0)*(r02));
IkReal x68=((IkReal(1.00000000000000))*(sj0));
IkReal x69=((sj1)*(sj2));
if( IKabs(((gconst4)*(((((x62)*(x64)))+(((x62)*(x63)))+(((r00)*(sj0)*(x66)))+(((IkReal(-1.00000000000000))*(r00)*(x68)*(x69)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x65)*(x69)))+(((IkReal(-1.00000000000000))*(r01)*(x68)))+(((x63)*(x67)))+(((x64)*(x67)))+(((x65)*(x66))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x62)*(x64)))+(((x62)*(x63)))+(((r00)*(sj0)*(x66)))+(((IkReal(-1.00000000000000))*(r00)*(x68)*(x69)))+(((cj0)*(r01)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x65)*(x69)))+(((IkReal(-1.00000000000000))*(r01)*(x68)))+(((x63)*(x67)))+(((x64)*(x67)))+(((x65)*(x66)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x70=((IkReal(1.00000000000000))*(r02));
IkReal x71=((sj0)*(sj1));
IkReal x72=((cj1)*(sj2));
IkReal x73=((IkReal(1.00000000000000))*(cj0));
IkReal x74=((r00)*(sj2));
IkReal x75=((cj0)*(sj1));
IkReal x76=((cj1)*(cj2)*(r00));
evalcond[0]=((((sj4)*(IKcos(j3))))+(((r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj2)*(x70)*(x75)))+(((IkReal(-1.00000000000000))*(cj0)*(x70)*(x72)))+(((IkReal(-1.00000000000000))*(x73)*(x76)))+(((x74)*(x75))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(x70)*(x71)))+(((IkReal(-1.00000000000000))*(r01)*(x73)))+(((IkReal(-1.00000000000000))*(sj0)*(x70)*(x72)))+(((IkReal(-1.00000000000000))*(sj0)*(x76)))+(((sj4)*(IKsin(j3))))+(((x71)*(x74))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x77=((IkReal(203.000000000000))*(sj2));
IkReal x78=((IkReal(4331.00000000000))*(cj2));
IkReal x79=((IkReal(1501.00000000000))*(sj1));
IkReal x80=((IkReal(10000.0000000000))*(cj1)*(pz));
if( IKabs(((gconst0)*(((((py)*(x77)))+(((py)*(x78)))+(((IkReal(-1.00000000000000))*(py)*(x80)))+(((IkReal(-1.00000000000000))*(px)*(x79))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((py)*(x79)))+(((px)*(x77)))+(((px)*(x78)))+(((IkReal(-1.00000000000000))*(px)*(x80))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((py)*(x77)))+(((py)*(x78)))+(((IkReal(-1.00000000000000))*(py)*(x80)))+(((IkReal(-1.00000000000000))*(px)*(x79)))))), ((gconst0)*(((((py)*(x79)))+(((px)*(x77)))+(((px)*(x78)))+(((IkReal(-1.00000000000000))*(px)*(x80)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x81=IKsin(j0);
IkReal x82=IKcos(j0);
IkReal x83=((IkReal(0.433100000000000))*(sj2));
IkReal x84=((IkReal(0.433100000000000))*(cj2));
IkReal x85=((pz)*(sj1));
IkReal x86=((IkReal(0.0203000000000000))*(cj2));
IkReal x87=((IkReal(0.863600000000000))*(cj1));
IkReal x88=((IkReal(1.00000000000000))*(cj1));
IkReal x89=((IkReal(0.0203000000000000))*(sj2));
IkReal x90=((px)*(x82));
IkReal x91=((py)*(x81));
evalcond[0]=((IkReal(0.150100000000000))+(((IkReal(-1.00000000000000))*(py)*(x82)))+(((px)*(x81))));
evalcond[1]=((IkReal(0.0240664700000000))+(((x87)*(x90)))+(((x87)*(x91)))+(((IkReal(-0.863600000000000))*(x85)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((cj1)*(pz)))+(((sj1)*(x90)))+(((sj1)*(x91)))+(((IkReal(-1.00000000000000))*(x89)))+(((IkReal(-1.00000000000000))*(x84))));
evalcond[3]=((IkReal(0.431800000000000))+(x83)+(x85)+(((IkReal(-1.00000000000000))*(x86)))+(((IkReal(-1.00000000000000))*(x88)*(x90)))+(((IkReal(-1.00000000000000))*(x88)*(x91))));
evalcond[4]=((((IkReal(0.431800000000000))*(cj1)))+(((sj1)*(x84)))+(((sj1)*(x89)))+(((IkReal(-1.00000000000000))*(x91)))+(((IkReal(-1.00000000000000))*(x90)))+(((IkReal(-1.00000000000000))*(cj1)*(x86)))+(((cj1)*(x83))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x92=((IkReal(1.00000000000000))*(sj1));
cj4array[0]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x92)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x92)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(sj2)))+(((cj1)*(cj2)*(r02))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x93=((r02)*(sj0));
IkReal x94=((cj2)*(sj1));
IkReal x95=((cj1)*(sj2));
IkReal x96=((cj0)*(r00));
IkReal x97=((cj1)*(cj2));
IkReal x98=((cj0)*(r02));
IkReal x99=((IkReal(1.00000000000000))*(sj0));
IkReal x100=((sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(r00)*(x100)*(x99)))+(((x93)*(x94)))+(((x93)*(x95)))+(((r00)*(sj0)*(x97)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x96)*(x97)))+(((IkReal(-1.00000000000000))*(x100)*(x96)))+(((x95)*(x98)))+(((IkReal(-1.00000000000000))*(r01)*(x99)))+(((x94)*(x98))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(r00)*(x100)*(x99)))+(((x93)*(x94)))+(((x93)*(x95)))+(((r00)*(sj0)*(x97)))+(((cj0)*(r01)))))), ((gconst4)*(((((x96)*(x97)))+(((IkReal(-1.00000000000000))*(x100)*(x96)))+(((x95)*(x98)))+(((IkReal(-1.00000000000000))*(r01)*(x99)))+(((x94)*(x98)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x101=((IkReal(1.00000000000000))*(r02));
IkReal x102=((sj0)*(sj1));
IkReal x103=((cj1)*(sj2));
IkReal x104=((IkReal(1.00000000000000))*(cj0));
IkReal x105=((r00)*(sj2));
IkReal x106=((cj0)*(sj1));
IkReal x107=((cj1)*(cj2)*(r00));
evalcond[0]=((((IkReal(-1.00000000000000))*(x104)*(x107)))+(((sj4)*(IKcos(j3))))+(((r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x101)*(x103)))+(((IkReal(-1.00000000000000))*(cj2)*(x101)*(x106)))+(((x105)*(x106))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj0)*(x107)))+(((IkReal(-1.00000000000000))*(r01)*(x104)))+(((x102)*(x105)))+(((IkReal(-1.00000000000000))*(cj2)*(x101)*(x102)))+(((IkReal(-1.00000000000000))*(sj0)*(x101)*(x103)))+(((sj4)*(IKsin(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x108=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.150100000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.150100000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x109=IKasin(((IkReal(0.150100000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x108)))+(((IkReal(-1.00000000000000))*(x109))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x108)))+(x109));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((((IkReal(2159.00000000000))*((pz)*(pz))))+(((IkReal(4318.00000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(2159.00000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2159.00000000000))*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=(((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x110=((IkReal(43.8277000000000))*(sj2));
IkReal x111=((cj0)*(px));
IkReal x112=((py)*(sj0));
IkReal x113=((IkReal(2500.00000000000))*(pp));
IkReal x114=((IkReal(935.062900000000))*(cj2));
if( IKabs(((gconst3)*(((((x110)*(x112)))+(((x110)*(x111)))+(((x112)*(x114)))+(((x111)*(x114)))+(((IkReal(60.1661750000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x113))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((x112)*(x113)))+(((x111)*(x113)))+(((pz)*(x114)))+(((pz)*(x110)))+(((IkReal(-60.1661750000000))*(x111)))+(((IkReal(-60.1661750000000))*(x112))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((x110)*(x112)))+(((x110)*(x111)))+(((x112)*(x114)))+(((x111)*(x114)))+(((IkReal(60.1661750000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x113)))))), ((gconst3)*(((((x112)*(x113)))+(((x111)*(x113)))+(((pz)*(x114)))+(((pz)*(x110)))+(((IkReal(-60.1661750000000))*(x111)))+(((IkReal(-60.1661750000000))*(x112)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x115=IKcos(j1);
IkReal x116=IKsin(j1);
IkReal x117=((IkReal(0.433100000000000))*(sj2));
IkReal x118=((cj0)*(px));
IkReal x119=((IkReal(0.433100000000000))*(cj2));
IkReal x120=((py)*(sj0));
IkReal x121=((IkReal(0.0203000000000000))*(cj2));
IkReal x122=((IkReal(0.0203000000000000))*(sj2));
IkReal x123=((IkReal(1.00000000000000))*(x115));
IkReal x124=((IkReal(0.0203000000000000))*(x116));
IkReal x125=((pz)*(x116));
IkReal x126=((IkReal(0.863600000000000))*(x115));
evalcond[0]=((IkReal(0.0240664700000000))+(((IkReal(-1.00000000000000))*(pp)))+(((x120)*(x126)))+(((IkReal(-0.863600000000000))*(x125)))+(((x118)*(x126))));
evalcond[1]=((((pz)*(x115)))+(((IkReal(-1.00000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(x122)))+(((x116)*(x120)))+(((x116)*(x118))));
evalcond[2]=((IkReal(0.431800000000000))+(((IkReal(-1.00000000000000))*(x118)*(x123)))+(((IkReal(-1.00000000000000))*(x120)*(x123)))+(((IkReal(-1.00000000000000))*(x121)))+(x125)+(x117));
evalcond[3]=((((IkReal(-1.00000000000000))*(x116)*(x121)))+(((IkReal(-1.00000000000000))*(x115)*(x122)))+(((IkReal(-1.00000000000000))*(x115)*(x119)))+(pz)+(((IkReal(0.431800000000000))*(x116)))+(((x116)*(x117))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x115)*(x121)))+(((IkReal(0.431800000000000))*(x115)))+(((x116)*(x122)))+(((x116)*(x119)))+(((IkReal(-1.00000000000000))*(x120)))+(((x115)*(x117)))+(((IkReal(-1.00000000000000))*(x118))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x127=((IkReal(1.00000000000000))*(sj1));
cj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x127)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(sj2)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x127))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x128=((r02)*(sj0));
IkReal x129=((cj2)*(sj1));
IkReal x130=((cj1)*(sj2));
IkReal x131=((cj0)*(r00));
IkReal x132=((cj1)*(cj2));
IkReal x133=((cj0)*(r02));
IkReal x134=((IkReal(1.00000000000000))*(sj0));
IkReal x135=((sj1)*(sj2));
if( IKabs(((gconst4)*(((((r00)*(sj0)*(x132)))+(((IkReal(-1.00000000000000))*(r00)*(x134)*(x135)))+(((x128)*(x130)))+(((x128)*(x129)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(r01)*(x134)))+(((IkReal(-1.00000000000000))*(x131)*(x135)))+(((x129)*(x133)))+(((x131)*(x132)))+(((x130)*(x133))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((r00)*(sj0)*(x132)))+(((IkReal(-1.00000000000000))*(r00)*(x134)*(x135)))+(((x128)*(x130)))+(((x128)*(x129)))+(((cj0)*(r01)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(r01)*(x134)))+(((IkReal(-1.00000000000000))*(x131)*(x135)))+(((x129)*(x133)))+(((x131)*(x132)))+(((x130)*(x133)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x136=((IkReal(1.00000000000000))*(r02));
IkReal x137=((sj0)*(sj1));
IkReal x138=((cj1)*(sj2));
IkReal x139=((IkReal(1.00000000000000))*(cj0));
IkReal x140=((r00)*(sj2));
IkReal x141=((cj0)*(sj1));
IkReal x142=((cj1)*(cj2)*(r00));
evalcond[0]=((((x140)*(x141)))+(((sj4)*(IKcos(j3))))+(((r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj2)*(x136)*(x141)))+(((IkReal(-1.00000000000000))*(x139)*(x142)))+(((IkReal(-1.00000000000000))*(cj0)*(x136)*(x138))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x139)))+(((x137)*(x140)))+(((IkReal(-1.00000000000000))*(sj0)*(x142)))+(((IkReal(-1.00000000000000))*(cj2)*(x136)*(x137)))+(((sj4)*(IKsin(j3))))+(((IkReal(-1.00000000000000))*(sj0)*(x136)*(x138))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x143=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.150100000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.150100000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x144=IKasin(((IkReal(0.150100000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x143)))+(((IkReal(-1.00000000000000))*(x144))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x143)))+(x144));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((((IkReal(2159.00000000000))*((pz)*(pz))))+(((IkReal(4318.00000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(2159.00000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2159.00000000000))*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=(((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x145=((IkReal(43.8277000000000))*(sj2));
IkReal x146=((cj0)*(px));
IkReal x147=((py)*(sj0));
IkReal x148=((IkReal(2500.00000000000))*(pp));
IkReal x149=((IkReal(935.062900000000))*(cj2));
if( IKabs(((gconst3)*(((((IkReal(60.1661750000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x148)))+(((x145)*(x147)))+(((x145)*(x146)))+(((x147)*(x149)))+(((x146)*(x149))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((pz)*(x149)))+(((pz)*(x145)))+(((IkReal(-60.1661750000000))*(x146)))+(((IkReal(-60.1661750000000))*(x147)))+(((x147)*(x148)))+(((x146)*(x148))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((IkReal(60.1661750000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x148)))+(((x145)*(x147)))+(((x145)*(x146)))+(((x147)*(x149)))+(((x146)*(x149)))))), ((gconst3)*(((((pz)*(x149)))+(((pz)*(x145)))+(((IkReal(-60.1661750000000))*(x146)))+(((IkReal(-60.1661750000000))*(x147)))+(((x147)*(x148)))+(((x146)*(x148)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x150=IKcos(j1);
IkReal x151=IKsin(j1);
IkReal x152=((IkReal(0.433100000000000))*(sj2));
IkReal x153=((cj0)*(px));
IkReal x154=((IkReal(0.433100000000000))*(cj2));
IkReal x155=((py)*(sj0));
IkReal x156=((IkReal(0.0203000000000000))*(cj2));
IkReal x157=((IkReal(0.0203000000000000))*(sj2));
IkReal x158=((IkReal(1.00000000000000))*(x150));
IkReal x159=((IkReal(0.0203000000000000))*(x151));
IkReal x160=((pz)*(x151));
IkReal x161=((IkReal(0.863600000000000))*(x150));
evalcond[0]=((IkReal(0.0240664700000000))+(((IkReal(-0.863600000000000))*(x160)))+(((IkReal(-1.00000000000000))*(pp)))+(((x155)*(x161)))+(((x153)*(x161))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x157)))+(((IkReal(-1.00000000000000))*(x154)))+(((x151)*(x153)))+(((x151)*(x155)))+(((pz)*(x150))));
evalcond[2]=((IkReal(0.431800000000000))+(((IkReal(-1.00000000000000))*(x156)))+(((IkReal(-1.00000000000000))*(x153)*(x158)))+(x160)+(x152)+(((IkReal(-1.00000000000000))*(x155)*(x158))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x151)*(x156)))+(((IkReal(-1.00000000000000))*(x150)*(x154)))+(((IkReal(-1.00000000000000))*(x150)*(x157)))+(((x151)*(x152)))+(pz)+(((IkReal(0.431800000000000))*(x151))));
evalcond[4]=((((x150)*(x152)))+(((IkReal(-1.00000000000000))*(x150)*(x156)))+(((x151)*(x157)))+(((x151)*(x154)))+(((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(x153)))+(((IkReal(0.431800000000000))*(x150))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x162=((IkReal(1.00000000000000))*(sj1));
cj4array[0]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x162)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x162)))+(((cj1)*(cj2)*(r02))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x163=((r02)*(sj0));
IkReal x164=((cj2)*(sj1));
IkReal x165=((cj1)*(sj2));
IkReal x166=((cj0)*(r00));
IkReal x167=((cj1)*(cj2));
IkReal x168=((cj0)*(r02));
IkReal x169=((IkReal(1.00000000000000))*(sj0));
IkReal x170=((sj1)*(sj2));
if( IKabs(((gconst4)*(((((r00)*(sj0)*(x167)))+(((IkReal(-1.00000000000000))*(r00)*(x169)*(x170)))+(((x163)*(x165)))+(((x163)*(x164)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(r01)*(x169)))+(((IkReal(-1.00000000000000))*(x166)*(x170)))+(((x164)*(x168)))+(((x166)*(x167)))+(((x165)*(x168))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((r00)*(sj0)*(x167)))+(((IkReal(-1.00000000000000))*(r00)*(x169)*(x170)))+(((x163)*(x165)))+(((x163)*(x164)))+(((cj0)*(r01)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(r01)*(x169)))+(((IkReal(-1.00000000000000))*(x166)*(x170)))+(((x164)*(x168)))+(((x166)*(x167)))+(((x165)*(x168)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x171=((IkReal(1.00000000000000))*(r02));
IkReal x172=((sj0)*(sj1));
IkReal x173=((cj1)*(sj2));
IkReal x174=((IkReal(1.00000000000000))*(cj0));
IkReal x175=((r00)*(sj2));
IkReal x176=((cj0)*(sj1));
IkReal x177=((cj1)*(cj2)*(r00));
evalcond[0]=((((sj4)*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(x174)*(x177)))+(((r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj2)*(x171)*(x176)))+(((x175)*(x176)))+(((IkReal(-1.00000000000000))*(cj0)*(x171)*(x173))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj0)*(x171)*(x173)))+(((IkReal(-1.00000000000000))*(sj0)*(x177)))+(((IkReal(-1.00000000000000))*(cj2)*(x171)*(x172)))+(((x172)*(x175)))+(((IkReal(-1.00000000000000))*(r01)*(x174)))+(((sj4)*(IKsin(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "560fcf56964ec92e4e01b8b6c12313af"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
