/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:04:15.552882
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25;
x0=IKsin(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[1]);
x3=IKcos(j[2]);
x4=IKsin(j[2]);
x5=IKcos(j[0]);
x6=IKcos(j[3]);
x7=IKsin(j[3]);
x8=IKcos(j[5]);
x9=IKsin(j[5]);
x10=IKsin(j[4]);
x11=IKcos(j[4]);
x12=((IkReal(0.331000000000000))*(x0));
x13=((IkReal(1.00000000000000))*(x11));
x14=((IkReal(1.00000000000000))*(x0));
x15=((IkReal(1.00000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x10));
x17=((IkReal(0.405000000000000))*(x1));
x18=((IkReal(0.331000000000000))*(x5));
x19=((x1)*(x3));
x20=((x2)*(x4));
x21=((x2)*(x3));
x22=((x1)*(x4));
x23=((IkReal(1.00000000000000))*(x19));
x24=((x14)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x22))))));
x25=((x15)*(((x21)+(x22))));
eetrans[0]=((IkReal(0.0454160000000000))+(((IkReal(-0.100150000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x12)*(x19)))+(((IkReal(-1.00000000000000))*(x0)*(x17)))+(((x12)*(x20))));
eetrans[1]=((IkReal(0.302408000000000))+(((IkReal(0.331000000000000))*(x21)))+(((IkReal(0.331000000000000))*(x22)))+(((IkReal(0.405000000000000))*(x2))));
eetrans[2]=((IkReal(0.467052000000000))+(((x18)*(x19)))+(((IkReal(-0.100150000000000))*(x0)))+(((IkReal(-1.00000000000000))*(x18)*(x20)))+(((x17)*(x5))));
eerot[0]=((((x8)*(((((IkReal(-1.00000000000000))*(x5)*(x7)))+(((x24)*(x6)))))))+(((IkReal(-1.00000000000000))*(x9)*(((((x13)*(((((x5)*(x6)))+(((x24)*(x7)))))))+(((x16)*(((((x0)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x20))))))))))));
eerot[1]=((((x9)*(((((x11)*(x7)*(((((IkReal(-1.00000000000000))*(x20)))+(x23)))))+(((x10)*(((x21)+(x22)))))))))+(((x6)*(x8)*(((((IkReal(-1.00000000000000))*(x23)))+(x20))))));
eerot[2]=((((IkReal(-1.00000000000000))*(x9)*(((((x16)*(((((IkReal(-1.00000000000000))*(x15)*(x19)))+(((x20)*(x5)))))))+(((x13)*(((((x0)*(x6)))+(((x25)*(x7)))))))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x14)*(x7)))+(((x25)*(x6))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {3}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j3,cj3,sj3,htj3,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij4[2], _nj4,_ij5[2], _nj5,_ij3[2], _nj3;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij3[0] = -1; _ij3[1] = -1; _nj3 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j3=pfree[0]; cj3=cos(pfree[0]); sj3=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((IkReal(-0.0454160000000000))+(px));
new_r01=r02;
new_py=((IkReal(-0.467052000000000))+(pz));
new_r02=((IkReal(-1.00000000000000))*(r01));
new_pz=((IkReal(0.302408000000000))+(((IkReal(-1.00000000000000))*(py))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
cj2array[0]=((IkReal(-1.05783455484689))+(((IkReal(3.72981239043676))*(pp))));
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.49711119832787))+((sj2)*(sj2))+((cj2)*(cj2))+(((IkReal(2.44712990936556))*(cj2))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x26=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2))));
if( IKabs(((IkReal(0.331000000000000))*(sj2))) < IKFAST_ATAN2_MAGTHRESH && IKabs(x26) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x27=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.331000000000000))*(sj2)), x26)));
if( ((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x28=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x27)))+(((IkReal(-1.00000000000000))*(x28))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x27)))+(x28));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x29=((IkReal(20000.0000000000))*(sj1));
gconst0=IKsign(((((x29)*((py)*(py))))+(((x29)*((px)*(px))))));
dummyeval[0]=((((sj1)*((py)*(py))))+(((sj1)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x30=((pz)+(((IkReal(0.331000000000000))*(sj2))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=x30;
evalcond[3]=x30;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-20000.0000000000))*((py)*(py))))+(((IkReal(-20000.0000000000))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x31=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x31)))+(((IkReal(2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x31)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x31)))+(((IkReal(2003.00000000000))*(py)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x31)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x32=IKsin(j0);
IkReal x33=IKcos(j0);
IkReal x34=((px)*(x32));
IkReal x35=((py)*(x33));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x32)))+(((px)*(x33))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x35)))+(((IkReal(0.331000000000000))*(cj2)))+(x34));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(0.810000000000000))*(x35)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x34))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x36=((cj1)*(cj2));
IkReal x37=((r00)*(sj3));
IkReal x38=((cj3)*(r02));
IkReal x39=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((sj1)*(sj2)*(x37)))+(((IkReal(-1.00000000000000))*(sj2)*(x38)*(x39)))+(((x36)*(x38)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x39)))+(((IkReal(-1.00000000000000))*(x36)*(x37)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x40=((IkReal(1.00000000000000))*(cj0));
IkReal x41=((r02)*(sj3));
IkReal x42=((cj2)*(sj1));
IkReal x43=((r00)*(sj3));
IkReal x44=((cj3)*(r02));
IkReal x45=((cj1)*(sj2));
IkReal x46=((IkReal(1.00000000000000))*(sj0));
IkReal x47=((cj3)*(r00));
IkReal x48=((r01)*(sj1)*(sj2));
IkReal x49=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((cj0)*(x44)*(x45)))+(((IkReal(-1.00000000000000))*(x40)*(x43)*(x45)))+(((IkReal(-1.00000000000000))*(x41)*(x46)))+(((IkReal(-1.00000000000000))*(x40)*(x48)))+(((cj0)*(x42)*(x44)))+(((cj0)*(x49)))+(((IkReal(-1.00000000000000))*(x46)*(x47)))+(((IkReal(-1.00000000000000))*(x40)*(x42)*(x43))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x44)*(x45)*(x46)))+(((sj0)*(x43)*(x45)))+(((IkReal(-1.00000000000000))*(x42)*(x44)*(x46)))+(((sj0)*(x42)*(x43)))+(((IkReal(-1.00000000000000))*(x40)*(x47)))+(((IkReal(-1.00000000000000))*(x40)*(x41)))+(((sj0)*(x48)))+(((IkReal(-1.00000000000000))*(x46)*(x49))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj0)*(x44)*(x45)))+(((IkReal(-1.00000000000000))*(x40)*(x43)*(x45)))+(((IkReal(-1.00000000000000))*(x41)*(x46)))+(((IkReal(-1.00000000000000))*(x40)*(x48)))+(((cj0)*(x42)*(x44)))+(((cj0)*(x49)))+(((IkReal(-1.00000000000000))*(x46)*(x47)))+(((IkReal(-1.00000000000000))*(x40)*(x42)*(x43)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x44)*(x45)*(x46)))+(((sj0)*(x43)*(x45)))+(((IkReal(-1.00000000000000))*(x42)*(x44)*(x46)))+(((sj0)*(x42)*(x43)))+(((IkReal(-1.00000000000000))*(x40)*(x47)))+(((IkReal(-1.00000000000000))*(x40)*(x41)))+(((sj0)*(x48)))+(((IkReal(-1.00000000000000))*(x46)*(x49)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x50=((IkReal(1.00000000000000))*(cj0));
IkReal x51=((r02)*(sj3));
IkReal x52=((cj3)*(r00));
IkReal x53=((IkReal(1.00000000000000))*(sj5));
IkReal x54=((cj3)*(r02));
IkReal x55=((cj1)*(sj2));
IkReal x56=((IkReal(1.00000000000000))*(sj0));
IkReal x57=((cj2)*(sj1));
IkReal x58=((r00)*(sj3));
IkReal x59=((r01)*(sj1)*(sj2));
IkReal x60=((cj1)*(cj2)*(r01));
IkReal x61=((x57)*(x58));
evalcond[0]=((((IkReal(-1.00000000000000))*(x56)*(x60)))+(((sj0)*(x61)))+(((IkReal(-1.00000000000000))*(x54)*(x56)*(x57)))+(((sj0)*(x59)))+(((sj0)*(x55)*(x58)))+(((IkReal(-1.00000000000000))*(x53)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x54)*(x55)*(x56)))+(((IkReal(-1.00000000000000))*(x50)*(x52)))+(((IkReal(-1.00000000000000))*(x50)*(x51))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x52)*(x56)))+(((IkReal(-1.00000000000000))*(x51)*(x56)))+(((IkReal(-1.00000000000000))*(x50)*(x55)*(x58)))+(((cj0)*(x60)))+(((IkReal(-1.00000000000000))*(x53)*(IKsin(j4))))+(((cj0)*(x54)*(x55)))+(((cj0)*(x54)*(x57)))+(((IkReal(-1.00000000000000))*(x50)*(x59)))+(((IkReal(-1.00000000000000))*(x50)*(x61))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
IkReal x62=((IkReal(0.331000000000000))*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=((pz)+(((IkReal(-1.00000000000000))*(x62))));
evalcond[3]=((((IkReal(-1.00000000000000))*(pz)))+(x62));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(20000.0000000000))*((px)*(px))))+(((IkReal(20000.0000000000))*((py)*(py))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x63=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x63)))+(((IkReal(-2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-8100.00000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x63)))+(((IkReal(-2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x63)))+(((IkReal(-2003.00000000000))*(py)))))), ((gconst2)*(((((IkReal(-8100.00000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x63)))+(((IkReal(-2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x64=IKcos(j0);
IkReal x65=IKsin(j0);
IkReal x66=((px)*(x65));
IkReal x67=((py)*(x64));
evalcond[0]=((IkReal(0.100150000000000))+(((px)*(x64)))+(((py)*(x65))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2)))+(x67)+(((IkReal(-1.00000000000000))*(x66))));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.810000000000000))*(x66)))+(((IkReal(-0.810000000000000))*(x67))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x68=((cj1)*(cj2));
IkReal x69=((r00)*(sj3));
IkReal x70=((cj3)*(r02));
IkReal x71=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((sj1)*(sj2)*(x69)))+(((x68)*(x70)))+(((IkReal(-1.00000000000000))*(sj2)*(x70)*(x71)))+(((IkReal(-1.00000000000000))*(x68)*(x69)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x71)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x72=((IkReal(1.00000000000000))*(cj0));
IkReal x73=((r02)*(sj3));
IkReal x74=((cj2)*(sj1));
IkReal x75=((r00)*(sj3));
IkReal x76=((cj3)*(r02));
IkReal x77=((cj1)*(sj2));
IkReal x78=((IkReal(1.00000000000000))*(sj0));
IkReal x79=((cj3)*(r00));
IkReal x80=((r01)*(sj1)*(sj2));
IkReal x81=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x72)*(x80)))+(((IkReal(-1.00000000000000))*(x78)*(x79)))+(((cj0)*(x76)*(x77)))+(((cj0)*(x81)))+(((cj0)*(x74)*(x76)))+(((IkReal(-1.00000000000000))*(x73)*(x78)))+(((IkReal(-1.00000000000000))*(x72)*(x75)*(x77)))+(((IkReal(-1.00000000000000))*(x72)*(x74)*(x75))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((sj0)*(x75)*(x77)))+(((sj0)*(x74)*(x75)))+(((IkReal(-1.00000000000000))*(x78)*(x81)))+(((sj0)*(x80)))+(((IkReal(-1.00000000000000))*(x74)*(x76)*(x78)))+(((IkReal(-1.00000000000000))*(x76)*(x77)*(x78)))+(((IkReal(-1.00000000000000))*(x72)*(x79)))+(((IkReal(-1.00000000000000))*(x72)*(x73))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x72)*(x80)))+(((IkReal(-1.00000000000000))*(x78)*(x79)))+(((cj0)*(x76)*(x77)))+(((cj0)*(x81)))+(((cj0)*(x74)*(x76)))+(((IkReal(-1.00000000000000))*(x73)*(x78)))+(((IkReal(-1.00000000000000))*(x72)*(x75)*(x77)))+(((IkReal(-1.00000000000000))*(x72)*(x74)*(x75)))))), ((gconst4)*(((((sj0)*(x75)*(x77)))+(((sj0)*(x74)*(x75)))+(((IkReal(-1.00000000000000))*(x78)*(x81)))+(((sj0)*(x80)))+(((IkReal(-1.00000000000000))*(x74)*(x76)*(x78)))+(((IkReal(-1.00000000000000))*(x76)*(x77)*(x78)))+(((IkReal(-1.00000000000000))*(x72)*(x79)))+(((IkReal(-1.00000000000000))*(x72)*(x73)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x82=((IkReal(1.00000000000000))*(cj0));
IkReal x83=((r02)*(sj3));
IkReal x84=((cj3)*(r00));
IkReal x85=((IkReal(1.00000000000000))*(sj5));
IkReal x86=((cj3)*(r02));
IkReal x87=((cj1)*(sj2));
IkReal x88=((IkReal(1.00000000000000))*(sj0));
IkReal x89=((cj2)*(sj1));
IkReal x90=((r00)*(sj3));
IkReal x91=((r01)*(sj1)*(sj2));
IkReal x92=((cj1)*(cj2)*(r01));
IkReal x93=((x89)*(x90));
evalcond[0]=((((IkReal(-1.00000000000000))*(x86)*(x88)*(x89)))+(((sj0)*(x87)*(x90)))+(((IkReal(-1.00000000000000))*(x86)*(x87)*(x88)))+(((IkReal(-1.00000000000000))*(x85)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x82)*(x83)))+(((IkReal(-1.00000000000000))*(x82)*(x84)))+(((sj0)*(x93)))+(((sj0)*(x91)))+(((IkReal(-1.00000000000000))*(x88)*(x92))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x82)*(x93)))+(((IkReal(-1.00000000000000))*(x82)*(x91)))+(((IkReal(-1.00000000000000))*(x83)*(x88)))+(((IkReal(-1.00000000000000))*(x85)*(IKsin(j4))))+(((cj0)*(x92)))+(((IkReal(-1.00000000000000))*(x84)*(x88)))+(((cj0)*(x86)*(x89)))+(((cj0)*(x86)*(x87)))+(((IkReal(-1.00000000000000))*(x82)*(x87)*(x90))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x94=((IkReal(6620.00000000000))*(sj2));
IkReal x95=((IkReal(2003.00000000000))*(sj1));
IkReal x96=((IkReal(20000.0000000000))*(cj1)*(pz));
if( IKabs(((gconst0)*(((((px)*(x94)))+(((px)*(x96)))+(((IkReal(-1.00000000000000))*(py)*(x95))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x95)))+(((IkReal(-1.00000000000000))*(py)*(x94)))+(((IkReal(-1.00000000000000))*(py)*(x96))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((px)*(x94)))+(((px)*(x96)))+(((IkReal(-1.00000000000000))*(py)*(x95)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x95)))+(((IkReal(-1.00000000000000))*(py)*(x94)))+(((IkReal(-1.00000000000000))*(py)*(x96)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x97=IKcos(j0);
IkReal x98=IKsin(j0);
IkReal x99=((IkReal(0.810000000000000))*(cj1));
IkReal x100=((IkReal(0.331000000000000))*(sj2));
IkReal x101=((pz)*(sj1));
IkReal x102=((IkReal(0.331000000000000))*(cj2));
IkReal x103=((px)*(x98));
IkReal x104=((py)*(x97));
IkReal x105=((IkReal(1.00000000000000))*(x104));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x98)))+(((px)*(x97))));
evalcond[1]=((((sj1)*(x104)))+(((IkReal(-1.00000000000000))*(sj1)*(x103)))+(x100)+(((cj1)*(pz))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x105)))+(((cj1)*(x103)))+(x101)+(x102));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x103)*(x99)))+(((IkReal(-0.810000000000000))*(x101)))+(((IkReal(-1.00000000000000))*(pp)))+(((x104)*(x99))));
evalcond[4]=((((cj1)*(x102)))+(((IkReal(0.405000000000000))*(cj1)))+(((IkReal(-1.00000000000000))*(x105)))+(x103)+(((IkReal(-1.00000000000000))*(sj1)*(x100))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x106=((cj1)*(cj2));
IkReal x107=((r00)*(sj3));
IkReal x108=((cj3)*(r02));
IkReal x109=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((sj1)*(sj2)*(x107)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x109)))+(((x106)*(x108)))+(((IkReal(-1.00000000000000))*(sj2)*(x108)*(x109)))+(((IkReal(-1.00000000000000))*(x106)*(x107)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x110=((IkReal(1.00000000000000))*(cj0));
IkReal x111=((r02)*(sj3));
IkReal x112=((cj2)*(sj1));
IkReal x113=((r00)*(sj3));
IkReal x114=((cj3)*(r02));
IkReal x115=((cj1)*(sj2));
IkReal x116=((IkReal(1.00000000000000))*(sj0));
IkReal x117=((cj3)*(r00));
IkReal x118=((r01)*(sj1)*(sj2));
IkReal x119=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x111)*(x116)))+(((IkReal(-1.00000000000000))*(x110)*(x113)*(x115)))+(((cj0)*(x112)*(x114)))+(((IkReal(-1.00000000000000))*(x110)*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x116)*(x117)))+(((IkReal(-1.00000000000000))*(x110)*(x118)))+(((cj0)*(x119)))+(((cj0)*(x114)*(x115))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((sj0)*(x118)))+(((sj0)*(x113)*(x115)))+(((IkReal(-1.00000000000000))*(x116)*(x119)))+(((IkReal(-1.00000000000000))*(x110)*(x117)))+(((IkReal(-1.00000000000000))*(x110)*(x111)))+(((IkReal(-1.00000000000000))*(x114)*(x115)*(x116)))+(((sj0)*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)*(x116))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x111)*(x116)))+(((IkReal(-1.00000000000000))*(x110)*(x113)*(x115)))+(((cj0)*(x112)*(x114)))+(((IkReal(-1.00000000000000))*(x110)*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x116)*(x117)))+(((IkReal(-1.00000000000000))*(x110)*(x118)))+(((cj0)*(x119)))+(((cj0)*(x114)*(x115)))))), ((gconst4)*(((((sj0)*(x118)))+(((sj0)*(x113)*(x115)))+(((IkReal(-1.00000000000000))*(x116)*(x119)))+(((IkReal(-1.00000000000000))*(x110)*(x117)))+(((IkReal(-1.00000000000000))*(x110)*(x111)))+(((IkReal(-1.00000000000000))*(x114)*(x115)*(x116)))+(((sj0)*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)*(x116)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x120=((IkReal(1.00000000000000))*(cj0));
IkReal x121=((r02)*(sj3));
IkReal x122=((cj3)*(r00));
IkReal x123=((IkReal(1.00000000000000))*(sj5));
IkReal x124=((cj3)*(r02));
IkReal x125=((cj1)*(sj2));
IkReal x126=((IkReal(1.00000000000000))*(sj0));
IkReal x127=((cj2)*(sj1));
IkReal x128=((r00)*(sj3));
IkReal x129=((r01)*(sj1)*(sj2));
IkReal x130=((cj1)*(cj2)*(r01));
IkReal x131=((x127)*(x128));
evalcond[0]=((((IkReal(-1.00000000000000))*(x120)*(x122)))+(((IkReal(-1.00000000000000))*(x120)*(x121)))+(((IkReal(-1.00000000000000))*(x124)*(x126)*(x127)))+(((IkReal(-1.00000000000000))*(x126)*(x130)))+(((sj0)*(x125)*(x128)))+(((IkReal(-1.00000000000000))*(x124)*(x125)*(x126)))+(((IkReal(-1.00000000000000))*(x123)*(IKcos(j4))))+(((sj0)*(x129)))+(((sj0)*(x131))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x120)*(x125)*(x128)))+(((IkReal(-1.00000000000000))*(x120)*(x129)))+(((cj0)*(x124)*(x125)))+(((cj0)*(x124)*(x127)))+(((IkReal(-1.00000000000000))*(x120)*(x131)))+(((cj0)*(x130)))+(((IkReal(-1.00000000000000))*(x123)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x121)*(x126)))+(((IkReal(-1.00000000000000))*(x122)*(x126))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x132=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x133=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x133))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x133)+(((IkReal(-1.00000000000000))*(x132))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x134=((cj0)*(py));
IkReal x135=((px)*(sj0));
IkReal x136=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-331.000000000000))*(pz)*(sj2)))+(((IkReal(-1.00000000000000))*(x134)*(x136)))+(((x135)*(x136)))+(((IkReal(405.000000000000))*(x135)))+(((IkReal(-405.000000000000))*(x134)))));
IkReal x137=((px)*(sj0));
IkReal x138=((cj0)*(py));
dummyeval[0]=((((cj2)*(x137)))+(((IkReal(-1.22356495468278))*(x138)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(1.22356495468278))*(x137)))+(((IkReal(-1.00000000000000))*(cj2)*(x138))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x139=((IkReal(331.000000000000))*(sj2));
IkReal x140=((cj0)*(py));
IkReal x141=((px)*(sj0));
IkReal x142=((IkReal(1000.00000000000))*(pz));
IkReal x143=((cj2)*(pz));
IkReal x144=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141)))))):(IkReal)1.0e30))*(((((x140)*(x142)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x141)*(x142))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141)))))):(IkReal)1.0e30))*(((((x140)*(x142)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x140)*(x144)))+(((IkReal(405000.000000000))*(pz)))+(((x141)*(x144)))+(((IkReal(331000.000000000))*(x143)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x139)*(x140)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(331.000000000000))*(x143)))+(((x139)*(x141)))))):(IkReal)1.0e30))*(((((x140)*(x142)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x145=IKsin(j1);
IkReal x146=IKcos(j1);
IkReal x147=((IkReal(0.331000000000000))*(sj2));
IkReal x148=((cj0)*(py));
IkReal x149=((px)*(sj0));
IkReal x150=((IkReal(0.331000000000000))*(cj2));
IkReal x151=((IkReal(0.810000000000000))*(x146));
IkReal x152=((pz)*(x145));
evalcond[0]=((((IkReal(0.405000000000000))*(x145)))+(((x145)*(x150)))+(pz)+(((x146)*(x147))));
evalcond[1]=((((pz)*(x146)))+(((x145)*(x148)))+(x147)+(((IkReal(-1.00000000000000))*(x145)*(x149))));
evalcond[2]=((IkReal(0.405000000000000))+(x152)+(x150)+(((IkReal(-1.00000000000000))*(x146)*(x148)))+(((x146)*(x149))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x152)))+(((IkReal(-1.00000000000000))*(pp)))+(((x148)*(x151)))+(((IkReal(-1.00000000000000))*(x149)*(x151))));
evalcond[4]=((((IkReal(0.405000000000000))*(x146)))+(x149)+(((x146)*(x150)))+(((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(x145)*(x147))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x153=((cj1)*(cj2));
IkReal x154=((r00)*(sj3));
IkReal x155=((cj3)*(r02));
IkReal x156=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((x153)*(x155)))+(((sj1)*(sj2)*(x154)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x156)))+(((IkReal(-1.00000000000000))*(x153)*(x154)))+(((IkReal(-1.00000000000000))*(sj2)*(x155)*(x156)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x157=((IkReal(1.00000000000000))*(cj0));
IkReal x158=((r02)*(sj3));
IkReal x159=((cj2)*(sj1));
IkReal x160=((r00)*(sj3));
IkReal x161=((cj3)*(r02));
IkReal x162=((cj1)*(sj2));
IkReal x163=((IkReal(1.00000000000000))*(sj0));
IkReal x164=((cj3)*(r00));
IkReal x165=((r01)*(sj1)*(sj2));
IkReal x166=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((cj0)*(x166)))+(((IkReal(-1.00000000000000))*(x157)*(x165)))+(((IkReal(-1.00000000000000))*(x158)*(x163)))+(((IkReal(-1.00000000000000))*(x157)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(x163)*(x164)))+(((cj0)*(x159)*(x161)))+(((IkReal(-1.00000000000000))*(x157)*(x160)*(x162)))+(((cj0)*(x161)*(x162))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x157)*(x164)))+(((IkReal(-1.00000000000000))*(x157)*(x158)))+(((sj0)*(x165)))+(((IkReal(-1.00000000000000))*(x163)*(x166)))+(((IkReal(-1.00000000000000))*(x161)*(x162)*(x163)))+(((sj0)*(x160)*(x162)))+(((sj0)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(x159)*(x161)*(x163))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj0)*(x166)))+(((IkReal(-1.00000000000000))*(x157)*(x165)))+(((IkReal(-1.00000000000000))*(x158)*(x163)))+(((IkReal(-1.00000000000000))*(x157)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(x163)*(x164)))+(((cj0)*(x159)*(x161)))+(((IkReal(-1.00000000000000))*(x157)*(x160)*(x162)))+(((cj0)*(x161)*(x162)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x157)*(x164)))+(((IkReal(-1.00000000000000))*(x157)*(x158)))+(((sj0)*(x165)))+(((IkReal(-1.00000000000000))*(x163)*(x166)))+(((IkReal(-1.00000000000000))*(x161)*(x162)*(x163)))+(((sj0)*(x160)*(x162)))+(((sj0)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(x159)*(x161)*(x163)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x167=((IkReal(1.00000000000000))*(cj0));
IkReal x168=((r02)*(sj3));
IkReal x169=((cj3)*(r00));
IkReal x170=((IkReal(1.00000000000000))*(sj5));
IkReal x171=((cj3)*(r02));
IkReal x172=((cj1)*(sj2));
IkReal x173=((IkReal(1.00000000000000))*(sj0));
IkReal x174=((cj2)*(sj1));
IkReal x175=((r00)*(sj3));
IkReal x176=((r01)*(sj1)*(sj2));
IkReal x177=((cj1)*(cj2)*(r01));
IkReal x178=((x174)*(x175));
evalcond[0]=((((sj0)*(x178)))+(((sj0)*(x176)))+(((IkReal(-1.00000000000000))*(x170)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x171)*(x173)*(x174)))+(((IkReal(-1.00000000000000))*(x171)*(x172)*(x173)))+(((sj0)*(x172)*(x175)))+(((IkReal(-1.00000000000000))*(x173)*(x177)))+(((IkReal(-1.00000000000000))*(x167)*(x169)))+(((IkReal(-1.00000000000000))*(x167)*(x168))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x167)*(x172)*(x175)))+(((cj0)*(x177)))+(((IkReal(-1.00000000000000))*(x168)*(x173)))+(((IkReal(-1.00000000000000))*(x169)*(x173)))+(((cj0)*(x171)*(x174)))+(((cj0)*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(x170)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x167)*(x178)))+(((IkReal(-1.00000000000000))*(x167)*(x176))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x179=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((cj0)*(py)*(x179)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x179)))+(((IkReal(109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x179)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((cj0)*(py)*(x179)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x179)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x179)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x180=IKsin(j1);
IkReal x181=IKcos(j1);
IkReal x182=((IkReal(0.331000000000000))*(sj2));
IkReal x183=((cj0)*(py));
IkReal x184=((px)*(sj0));
IkReal x185=((IkReal(0.331000000000000))*(cj2));
IkReal x186=((IkReal(0.810000000000000))*(x181));
IkReal x187=((pz)*(x180));
evalcond[0]=((((IkReal(0.405000000000000))*(x180)))+(((x180)*(x185)))+(pz)+(((x181)*(x182))));
evalcond[1]=((((pz)*(x181)))+(((x180)*(x183)))+(x182)+(((IkReal(-1.00000000000000))*(x180)*(x184))));
evalcond[2]=((IkReal(0.405000000000000))+(x185)+(x187)+(((IkReal(-1.00000000000000))*(x181)*(x183)))+(((x181)*(x184))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x187)))+(((IkReal(-1.00000000000000))*(x184)*(x186)))+(((IkReal(-1.00000000000000))*(pp)))+(((x183)*(x186))));
evalcond[4]=((((IkReal(0.405000000000000))*(x181)))+(((IkReal(-1.00000000000000))*(x183)))+(((IkReal(-1.00000000000000))*(x180)*(x182)))+(x184)+(((x181)*(x185))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x188=((cj1)*(cj2));
IkReal x189=((r00)*(sj3));
IkReal x190=((cj3)*(r02));
IkReal x191=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((sj1)*(sj2)*(x189)))+(((x188)*(x190)))+(((IkReal(-1.00000000000000))*(x188)*(x189)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x191)))+(((IkReal(-1.00000000000000))*(sj2)*(x190)*(x191)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x192=((IkReal(1.00000000000000))*(cj0));
IkReal x193=((r02)*(sj3));
IkReal x194=((cj2)*(sj1));
IkReal x195=((r00)*(sj3));
IkReal x196=((cj3)*(r02));
IkReal x197=((cj1)*(sj2));
IkReal x198=((IkReal(1.00000000000000))*(sj0));
IkReal x199=((cj3)*(r00));
IkReal x200=((r01)*(sj1)*(sj2));
IkReal x201=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x192)*(x200)))+(((IkReal(-1.00000000000000))*(x192)*(x195)*(x197)))+(((IkReal(-1.00000000000000))*(x192)*(x194)*(x195)))+(((cj0)*(x194)*(x196)))+(((IkReal(-1.00000000000000))*(x198)*(x199)))+(((cj0)*(x201)))+(((cj0)*(x196)*(x197)))+(((IkReal(-1.00000000000000))*(x193)*(x198))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((sj0)*(x200)))+(((IkReal(-1.00000000000000))*(x192)*(x193)))+(((IkReal(-1.00000000000000))*(x192)*(x199)))+(((IkReal(-1.00000000000000))*(x194)*(x196)*(x198)))+(((sj0)*(x195)*(x197)))+(((sj0)*(x194)*(x195)))+(((IkReal(-1.00000000000000))*(x196)*(x197)*(x198)))+(((IkReal(-1.00000000000000))*(x198)*(x201))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x192)*(x200)))+(((IkReal(-1.00000000000000))*(x192)*(x195)*(x197)))+(((IkReal(-1.00000000000000))*(x192)*(x194)*(x195)))+(((cj0)*(x194)*(x196)))+(((IkReal(-1.00000000000000))*(x198)*(x199)))+(((cj0)*(x201)))+(((cj0)*(x196)*(x197)))+(((IkReal(-1.00000000000000))*(x193)*(x198)))))), ((gconst4)*(((((sj0)*(x200)))+(((IkReal(-1.00000000000000))*(x192)*(x193)))+(((IkReal(-1.00000000000000))*(x192)*(x199)))+(((IkReal(-1.00000000000000))*(x194)*(x196)*(x198)))+(((sj0)*(x195)*(x197)))+(((sj0)*(x194)*(x195)))+(((IkReal(-1.00000000000000))*(x196)*(x197)*(x198)))+(((IkReal(-1.00000000000000))*(x198)*(x201)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x202=((IkReal(1.00000000000000))*(cj0));
IkReal x203=((r02)*(sj3));
IkReal x204=((cj3)*(r00));
IkReal x205=((IkReal(1.00000000000000))*(sj5));
IkReal x206=((cj3)*(r02));
IkReal x207=((cj1)*(sj2));
IkReal x208=((IkReal(1.00000000000000))*(sj0));
IkReal x209=((cj2)*(sj1));
IkReal x210=((r00)*(sj3));
IkReal x211=((r01)*(sj1)*(sj2));
IkReal x212=((cj1)*(cj2)*(r01));
IkReal x213=((x209)*(x210));
evalcond[0]=((((IkReal(-1.00000000000000))*(x205)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x202)*(x204)))+(((IkReal(-1.00000000000000))*(x202)*(x203)))+(((IkReal(-1.00000000000000))*(x206)*(x208)*(x209)))+(((sj0)*(x213)))+(((sj0)*(x211)))+(((sj0)*(x207)*(x210)))+(((IkReal(-1.00000000000000))*(x208)*(x212)))+(((IkReal(-1.00000000000000))*(x206)*(x207)*(x208))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x204)*(x208)))+(((cj0)*(x212)))+(((IkReal(-1.00000000000000))*(x203)*(x208)))+(((IkReal(-1.00000000000000))*(x205)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x202)*(x207)*(x210)))+(((cj0)*(x206)*(x209)))+(((cj0)*(x206)*(x207)))+(((IkReal(-1.00000000000000))*(x202)*(x213)))+(((IkReal(-1.00000000000000))*(x202)*(x211))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x214=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x215=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x215)))+(((IkReal(-1.00000000000000))*(x214))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x215)+(((IkReal(-1.00000000000000))*(x214))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x216=((cj0)*(py));
IkReal x217=((px)*(sj0));
IkReal x218=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(405.000000000000))*(x217)))+(((IkReal(-331.000000000000))*(pz)*(sj2)))+(((IkReal(-405.000000000000))*(x216)))+(((x217)*(x218)))+(((IkReal(-1.00000000000000))*(x216)*(x218)))));
IkReal x219=((px)*(sj0));
IkReal x220=((cj0)*(py));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj2)*(x220)))+(((IkReal(-1.22356495468278))*(x220)))+(((cj2)*(x219)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(1.22356495468278))*(x219))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x221=((IkReal(331.000000000000))*(sj2));
IkReal x222=((cj0)*(py));
IkReal x223=((px)*(sj0));
IkReal x224=((IkReal(1000.00000000000))*(pz));
IkReal x225=((cj2)*(pz));
IkReal x226=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(405000.000000000))*(pz)))+(((x223)*(x226)))+(((IkReal(331000.000000000))*(x225))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(405000.000000000))*(pz)))+(((x223)*(x226)))+(((IkReal(331000.000000000))*(x225)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(331.000000000000))*(x225)))+(((IkReal(405.000000000000))*(pz)))+(((x221)*(x223)))+(((IkReal(-1.00000000000000))*(x221)*(x222))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x225)))+(((IkReal(405.000000000000))*(pz)))+(((x221)*(x223)))+(((IkReal(-1.00000000000000))*(x221)*(x222)))))):(IkReal)1.0e30))*(((((x222)*(x224)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x223)*(x224))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(405000.000000000))*(pz)))+(((x223)*(x226)))+(((IkReal(331000.000000000))*(x225))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(405000.000000000))*(pz)))+(((x223)*(x226)))+(((IkReal(331000.000000000))*(x225)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(331.000000000000))*(x225)))+(((IkReal(405.000000000000))*(pz)))+(((x221)*(x223)))+(((IkReal(-1.00000000000000))*(x221)*(x222))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x225)))+(((IkReal(405.000000000000))*(pz)))+(((x221)*(x223)))+(((IkReal(-1.00000000000000))*(x221)*(x222)))))):(IkReal)1.0e30))*(((((x222)*(x224)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x223)*(x224)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(405000.000000000))*(pz)))+(((x223)*(x226)))+(((IkReal(331000.000000000))*(x225))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x222)*(x226)))+(((IkReal(405000.000000000))*(pz)))+(((x223)*(x226)))+(((IkReal(331000.000000000))*(x225)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(331.000000000000))*(x225)))+(((IkReal(405.000000000000))*(pz)))+(((x221)*(x223)))+(((IkReal(-1.00000000000000))*(x221)*(x222))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x225)))+(((IkReal(405.000000000000))*(pz)))+(((x221)*(x223)))+(((IkReal(-1.00000000000000))*(x221)*(x222)))))):(IkReal)1.0e30))*(((((x222)*(x224)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(x223)*(x224)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x227=IKsin(j1);
IkReal x228=IKcos(j1);
IkReal x229=((IkReal(0.331000000000000))*(sj2));
IkReal x230=((cj0)*(py));
IkReal x231=((px)*(sj0));
IkReal x232=((IkReal(0.331000000000000))*(cj2));
IkReal x233=((IkReal(0.810000000000000))*(x228));
IkReal x234=((pz)*(x227));
evalcond[0]=((((x228)*(x229)))+(pz)+(((IkReal(0.405000000000000))*(x227)))+(((x227)*(x232))));
evalcond[1]=((((pz)*(x228)))+(x229)+(((x227)*(x230)))+(((IkReal(-1.00000000000000))*(x227)*(x231))));
evalcond[2]=((IkReal(0.405000000000000))+(((x228)*(x231)))+(x232)+(x234)+(((IkReal(-1.00000000000000))*(x228)*(x230))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x234)))+(((x230)*(x233)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x231)*(x233))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x227)*(x229)))+(((x228)*(x232)))+(x231)+(((IkReal(0.405000000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x230))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x235=((cj1)*(cj2));
IkReal x236=((r00)*(sj3));
IkReal x237=((cj3)*(r02));
IkReal x238=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((sj1)*(sj2)*(x236)))+(((IkReal(-1.00000000000000))*(x235)*(x236)))+(((x235)*(x237)))+(((IkReal(-1.00000000000000))*(sj2)*(x237)*(x238)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x238)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x239=((IkReal(1.00000000000000))*(cj0));
IkReal x240=((r02)*(sj3));
IkReal x241=((cj2)*(sj1));
IkReal x242=((r00)*(sj3));
IkReal x243=((cj3)*(r02));
IkReal x244=((cj1)*(sj2));
IkReal x245=((IkReal(1.00000000000000))*(sj0));
IkReal x246=((cj3)*(r00));
IkReal x247=((r01)*(sj1)*(sj2));
IkReal x248=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((cj0)*(x248)))+(((cj0)*(x243)*(x244)))+(((IkReal(-1.00000000000000))*(x239)*(x241)*(x242)))+(((cj0)*(x241)*(x243)))+(((IkReal(-1.00000000000000))*(x239)*(x242)*(x244)))+(((IkReal(-1.00000000000000))*(x240)*(x245)))+(((IkReal(-1.00000000000000))*(x245)*(x246)))+(((IkReal(-1.00000000000000))*(x239)*(x247))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((sj0)*(x242)*(x244)))+(((sj0)*(x241)*(x242)))+(((IkReal(-1.00000000000000))*(x245)*(x248)))+(((IkReal(-1.00000000000000))*(x239)*(x246)))+(((IkReal(-1.00000000000000))*(x239)*(x240)))+(((sj0)*(x247)))+(((IkReal(-1.00000000000000))*(x243)*(x244)*(x245)))+(((IkReal(-1.00000000000000))*(x241)*(x243)*(x245))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj0)*(x248)))+(((cj0)*(x243)*(x244)))+(((IkReal(-1.00000000000000))*(x239)*(x241)*(x242)))+(((cj0)*(x241)*(x243)))+(((IkReal(-1.00000000000000))*(x239)*(x242)*(x244)))+(((IkReal(-1.00000000000000))*(x240)*(x245)))+(((IkReal(-1.00000000000000))*(x245)*(x246)))+(((IkReal(-1.00000000000000))*(x239)*(x247)))))), ((gconst4)*(((((sj0)*(x242)*(x244)))+(((sj0)*(x241)*(x242)))+(((IkReal(-1.00000000000000))*(x245)*(x248)))+(((IkReal(-1.00000000000000))*(x239)*(x246)))+(((IkReal(-1.00000000000000))*(x239)*(x240)))+(((sj0)*(x247)))+(((IkReal(-1.00000000000000))*(x243)*(x244)*(x245)))+(((IkReal(-1.00000000000000))*(x241)*(x243)*(x245)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x249=((IkReal(1.00000000000000))*(cj0));
IkReal x250=((r02)*(sj3));
IkReal x251=((cj3)*(r00));
IkReal x252=((IkReal(1.00000000000000))*(sj5));
IkReal x253=((cj3)*(r02));
IkReal x254=((cj1)*(sj2));
IkReal x255=((IkReal(1.00000000000000))*(sj0));
IkReal x256=((cj2)*(sj1));
IkReal x257=((r00)*(sj3));
IkReal x258=((r01)*(sj1)*(sj2));
IkReal x259=((cj1)*(cj2)*(r01));
IkReal x260=((x256)*(x257));
evalcond[0]=((((sj0)*(x254)*(x257)))+(((IkReal(-1.00000000000000))*(x253)*(x255)*(x256)))+(((IkReal(-1.00000000000000))*(x255)*(x259)))+(((IkReal(-1.00000000000000))*(x252)*(IKcos(j4))))+(((sj0)*(x258)))+(((IkReal(-1.00000000000000))*(x249)*(x250)))+(((IkReal(-1.00000000000000))*(x249)*(x251)))+(((sj0)*(x260)))+(((IkReal(-1.00000000000000))*(x253)*(x254)*(x255))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x250)*(x255)))+(((IkReal(-1.00000000000000))*(x251)*(x255)))+(((cj0)*(x253)*(x256)))+(((cj0)*(x253)*(x254)))+(((IkReal(-1.00000000000000))*(x249)*(x260)))+(((IkReal(-1.00000000000000))*(x249)*(x254)*(x257)))+(((cj0)*(x259)))+(((IkReal(-1.00000000000000))*(x249)*(x258)))+(((IkReal(-1.00000000000000))*(x252)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x261=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((cj0)*(py)*(x261)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x261)))+(((IkReal(109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x261)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((cj0)*(py)*(x261)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x261)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x261)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x262=IKsin(j1);
IkReal x263=IKcos(j1);
IkReal x264=((IkReal(0.331000000000000))*(sj2));
IkReal x265=((cj0)*(py));
IkReal x266=((px)*(sj0));
IkReal x267=((IkReal(0.331000000000000))*(cj2));
IkReal x268=((IkReal(0.810000000000000))*(x263));
IkReal x269=((pz)*(x262));
evalcond[0]=((((IkReal(0.405000000000000))*(x262)))+(((x262)*(x267)))+(((x263)*(x264)))+(pz));
evalcond[1]=((((x262)*(x265)))+(((IkReal(-1.00000000000000))*(x262)*(x266)))+(x264)+(((pz)*(x263))));
evalcond[2]=((IkReal(0.405000000000000))+(((x263)*(x266)))+(x267)+(x269)+(((IkReal(-1.00000000000000))*(x263)*(x265))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x269)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x266)*(x268)))+(((x265)*(x268))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x262)*(x264)))+(((IkReal(-1.00000000000000))*(x265)))+(((IkReal(0.405000000000000))*(x263)))+(((x263)*(x267)))+(x266));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x270=((cj1)*(cj2));
IkReal x271=((r00)*(sj3));
IkReal x272=((cj3)*(r02));
IkReal x273=((IkReal(1.00000000000000))*(sj1));
cj5array[0]=((((IkReal(-1.00000000000000))*(cj2)*(r01)*(x273)))+(((IkReal(-1.00000000000000))*(sj2)*(x272)*(x273)))+(((x270)*(x272)))+(((sj1)*(sj2)*(x271)))+(((IkReal(-1.00000000000000))*(x270)*(x271)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x274=((IkReal(1.00000000000000))*(cj0));
IkReal x275=((r02)*(sj3));
IkReal x276=((cj2)*(sj1));
IkReal x277=((r00)*(sj3));
IkReal x278=((cj3)*(r02));
IkReal x279=((cj1)*(sj2));
IkReal x280=((IkReal(1.00000000000000))*(sj0));
IkReal x281=((cj3)*(r00));
IkReal x282=((r01)*(sj1)*(sj2));
IkReal x283=((cj1)*(cj2)*(r01));
if( IKabs(((gconst4)*(((((cj0)*(x283)))+(((cj0)*(x276)*(x278)))+(((IkReal(-1.00000000000000))*(x280)*(x281)))+(((IkReal(-1.00000000000000))*(x274)*(x277)*(x279)))+(((IkReal(-1.00000000000000))*(x274)*(x276)*(x277)))+(((cj0)*(x278)*(x279)))+(((IkReal(-1.00000000000000))*(x275)*(x280)))+(((IkReal(-1.00000000000000))*(x274)*(x282))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((sj0)*(x277)*(x279)))+(((IkReal(-1.00000000000000))*(x274)*(x275)))+(((IkReal(-1.00000000000000))*(x276)*(x278)*(x280)))+(((IkReal(-1.00000000000000))*(x280)*(x283)))+(((sj0)*(x276)*(x277)))+(((sj0)*(x282)))+(((IkReal(-1.00000000000000))*(x278)*(x279)*(x280)))+(((IkReal(-1.00000000000000))*(x274)*(x281))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj0)*(x283)))+(((cj0)*(x276)*(x278)))+(((IkReal(-1.00000000000000))*(x280)*(x281)))+(((IkReal(-1.00000000000000))*(x274)*(x277)*(x279)))+(((IkReal(-1.00000000000000))*(x274)*(x276)*(x277)))+(((cj0)*(x278)*(x279)))+(((IkReal(-1.00000000000000))*(x275)*(x280)))+(((IkReal(-1.00000000000000))*(x274)*(x282)))))), ((gconst4)*(((((sj0)*(x277)*(x279)))+(((IkReal(-1.00000000000000))*(x274)*(x275)))+(((IkReal(-1.00000000000000))*(x276)*(x278)*(x280)))+(((IkReal(-1.00000000000000))*(x280)*(x283)))+(((sj0)*(x276)*(x277)))+(((sj0)*(x282)))+(((IkReal(-1.00000000000000))*(x278)*(x279)*(x280)))+(((IkReal(-1.00000000000000))*(x274)*(x281)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x284=((IkReal(1.00000000000000))*(cj0));
IkReal x285=((r02)*(sj3));
IkReal x286=((cj3)*(r00));
IkReal x287=((IkReal(1.00000000000000))*(sj5));
IkReal x288=((cj3)*(r02));
IkReal x289=((cj1)*(sj2));
IkReal x290=((IkReal(1.00000000000000))*(sj0));
IkReal x291=((cj2)*(sj1));
IkReal x292=((r00)*(sj3));
IkReal x293=((r01)*(sj1)*(sj2));
IkReal x294=((cj1)*(cj2)*(r01));
IkReal x295=((x291)*(x292));
evalcond[0]=((((IkReal(-1.00000000000000))*(x290)*(x294)))+(((sj0)*(x295)))+(((sj0)*(x293)))+(((IkReal(-1.00000000000000))*(x287)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x288)*(x289)*(x290)))+(((IkReal(-1.00000000000000))*(x284)*(x285)))+(((IkReal(-1.00000000000000))*(x284)*(x286)))+(((sj0)*(x289)*(x292)))+(((IkReal(-1.00000000000000))*(x288)*(x290)*(x291))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x284)*(x293)))+(((IkReal(-1.00000000000000))*(x284)*(x295)))+(((cj0)*(x288)*(x289)))+(((IkReal(-1.00000000000000))*(x284)*(x289)*(x292)))+(((IkReal(-1.00000000000000))*(x285)*(x290)))+(((IkReal(-1.00000000000000))*(x286)*(x290)))+(((cj0)*(x288)*(x291)))+(((IkReal(-1.00000000000000))*(x287)*(IKsin(j4))))+(((cj0)*(x294))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
