/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:12:27.584462
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25;
x0=IKsin(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[1]);
x3=IKcos(j[2]);
x4=IKsin(j[2]);
x5=IKcos(j[0]);
x6=IKcos(j[3]);
x7=IKsin(j[3]);
x8=IKcos(j[5]);
x9=IKsin(j[5]);
x10=IKsin(j[4]);
x11=IKcos(j[4]);
x12=((IkReal(0.331000000000000))*(x0));
x13=((IkReal(1.00000000000000))*(x11));
x14=((IkReal(1.00000000000000))*(x0));
x15=((IkReal(1.00000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x10));
x17=((IkReal(0.405000000000000))*(x1));
x18=((IkReal(0.331000000000000))*(x5));
x19=((x1)*(x3));
x20=((x2)*(x4));
x21=((x2)*(x3));
x22=((x1)*(x4));
x23=((IkReal(1.00000000000000))*(x19));
x24=((x14)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x22))))));
x25=((x15)*(((x21)+(x22))));
eetrans[0]=((IkReal(0.0454160000000000))+(((IkReal(-0.100150000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x12)*(x19)))+(((IkReal(-1.00000000000000))*(x0)*(x17)))+(((x12)*(x20))));
eetrans[1]=((IkReal(0.302408000000000))+(((IkReal(0.331000000000000))*(x21)))+(((IkReal(0.331000000000000))*(x22)))+(((IkReal(0.405000000000000))*(x2))));
eetrans[2]=((IkReal(0.467052000000000))+(((x18)*(x19)))+(((IkReal(-0.100150000000000))*(x0)))+(((IkReal(-1.00000000000000))*(x18)*(x20)))+(((x17)*(x5))));
eerot[0]=((((x8)*(((((IkReal(-1.00000000000000))*(x5)*(x7)))+(((x24)*(x6)))))))+(((IkReal(-1.00000000000000))*(x9)*(((((x13)*(((((x5)*(x6)))+(((x24)*(x7)))))))+(((x16)*(((((x0)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x20))))))))))));
eerot[1]=((((x9)*(((((x11)*(x7)*(((((IkReal(-1.00000000000000))*(x20)))+(x23)))))+(((x10)*(((x21)+(x22)))))))))+(((x6)*(x8)*(((((IkReal(-1.00000000000000))*(x23)))+(x20))))));
eerot[2]=((((IkReal(-1.00000000000000))*(x9)*(((((x16)*(((((IkReal(-1.00000000000000))*(x15)*(x19)))+(((x20)*(x5)))))))+(((x13)*(((((x0)*(x6)))+(((x25)*(x7)))))))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x14)*(x7)))+(((x25)*(x6))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j2,cj2,sj2,htj2,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij2[2], _nj2;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij2[0] = -1; _ij2[1] = -1; _nj2 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j2=pfree[0]; cj2=cos(pfree[0]); sj2=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((IkReal(-0.0454160000000000))+(px));
new_r01=r02;
new_py=((IkReal(-0.467052000000000))+(pz));
new_r02=((IkReal(-1.00000000000000))*(r01));
new_pz=((IkReal(0.302408000000000))+(((IkReal(-1.00000000000000))*(py))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal Noneeval[1];
IkReal x26=((IkReal(3.02114803625378))*(cj1));
IkReal x27=((cj0)*(py));
IkReal x28=((px)*(sj0));
IkReal x29=((IkReal(3.02114803625378))*(sj1)*(sj2));
Noneeval[0]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x26)*(x27)))+(((IkReal(1.22356495468278))*(cj2)))+(((pz)*(sj2)*(x26)))+(((cj2)*(x26)*(x28)))+(((IkReal(3.02114803625378))*(cj2)*(pz)*(sj1)))+(((IkReal(-1.00000000000000))*(x28)*(x29)))+(((x27)*(x29))));
if( IKabs(Noneeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x30=(cj2)*(cj2);
IkReal x31=(pz)*(pz);
IkReal x32=(sj2)*(sj2);
IkReal x33=(pp)*(pp);
IkReal x34=((px)*(r01));
IkReal x35=((py)*(r00));
IkReal x36=((IkReal(13.1383542509501))*(pp));
IkReal x37=((IkReal(3.79063415822223))*(cj2));
IkReal x38=((IkReal(40.3787728292733))*(pp));
IkReal x39=((IkReal(24.9251684131317))*(x33));
IkReal x40=((cj2)*(pz)*(sj2));
IkReal x41=((IkReal(13.3653738064895))*(cj2)*(pp));
IkReal x42=((IkReal(11.4879527832565))*(pz)*(sj2));
IkReal x43=((x30)*(x35));
IkReal x44=((IkReal(17.3534029958557))*(x31)*(x32));
dummyeval[0]=((((IkReal(1.90125618562895))*(x35)))+(((IkReal(-1.00000000000000))*(x36)*(x43)))+(((x34)*(x42)))+(((x34)*(x41)))+(((x35)*(x37)))+(((x35)*(x44)))+(((IkReal(-1.00000000000000))*(x43)*((px)*(px))))+(((x30)*(x34)*((py)*(py))))+(((IkReal(-1.89537089867521))*(x30)*(x34)))+(((x39)*(x43)))+(((IkReal(-11.4520669432696))*(x35)*(x40)))+(((IkReal(-1.00000000000000))*(x43)*((py)*(py))))+(((IkReal(-1.90125618562895))*(x34)))+(((IkReal(1.89537089867521))*(x43)))+(((IkReal(-1.00000000000000))*(x34)*(x44)))+(((IkReal(-1.00000000000000))*(x34)*(x38)*(x40)))+(((IkReal(11.4520669432696))*(x34)*(x40)))+(((IkReal(-1.00000000000000))*(x35)*(x42)))+(((IkReal(-1.00000000000000))*(x35)*(x41)))+(((IkReal(-1.00000000000000))*(x34)*(x37)))+(((IkReal(-1.00000000000000))*(x30)*(x34)*(x39)))+(((x30)*(x34)*((px)*(px))))+(((x30)*(x34)*(x36)))+(((x35)*(x38)*(x40))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal j1evalpoly[2];
IkReal x45=(cj2)*(cj2);
IkReal x46=(pp)*(pp);
IkReal x47=(pz)*(pz);
IkReal x48=(sj2)*(sj2);
IkReal x49=(py)*(py);
IkReal x50=(px)*(px)*(px);
IkReal x51=(py)*(py)*(py);
IkReal x52=(px)*(px);
IkReal x53=((px)*(r01));
IkReal x54=((IkReal(0.608322334339800))*(cj2));
IkReal x55=((py)*(r00));
IkReal x56=((cj2)*(sj2));
IkReal x57=((IkReal(25.9200000000000))*(pp));
IkReal x58=((IkReal(0.276663877959600))*(sj2));
IkReal x59=((cj2)*(pz));
IkReal x60=((IkReal(1.83783182580000))*(pz));
IkReal x61=((IkReal(3.18499128000000))*(pp));
IkReal x62=((IkReal(0.436216777061400))*(cj2));
IkReal x63=((IkReal(12.9600000000000))*(pp));
IkReal x64=((IkReal(0.234784722234120))*(x53));
IkReal x65=((IkReal(0.234784722234120))*(x55));
IkReal x66=((IkReal(0.305114277121960))*(x55));
IkReal x67=((IkReal(0.305114277121960))*(x53));
IkReal x68=((IkReal(0.641921440000000))*(x48));
IkReal x69=((pz)*(x48));
IkReal x70=((r01)*(x50));
IkReal x71=((IkReal(16.0000000000000))*(x46));
IkReal x72=((IkReal(0.160480360000000))*(x45));
IkReal x73=((IkReal(1.84359079832000))*(pz)*(sj2));
IkReal x74=((pz)*(x45));
IkReal x75=((IkReal(0.0665516630116881))*(x48));
IkReal x76=((IkReal(0.641921440000000))*(x47));
IkReal x77=((IkReal(2.14488000000000))*(cj2)*(pp));
IkReal x78=((r00)*(x51));
IkReal x79=((IkReal(0.198878639956922))*(x45));
IkReal x80=((IkReal(6.48000000000000))*(pp)*(pz));
IkReal x81=((IkReal(0.304169804152922))*(x45));
IkReal x82=((IkReal(4.28976000000000))*(pp)*(sj2));
IkReal x83=((x53)*(x62));
IkReal x84=((x55)*(x62));
IkReal x85=((x55)*(x58));
IkReal x86=((x53)*(x58));
IkReal x87=((x54)*(x55));
IkReal x88=((x53)*(x54));
IkReal x89=((IkReal(2.10844782000000))*(pp)*(x45));
IkReal x90=((x47)*(x48));
IkReal x91=((IkReal(2.06380872000000))*(pp)*(x48));
IkReal x92=((x53)*(x73));
IkReal x93=((x55)*(x77));
IkReal x94=((x55)*(x73));
IkReal x95=((x53)*(x77));
IkReal x96=((x55)*(x75));
IkReal x97=((x53)*(x79));
IkReal x98=((x55)*(x79));
IkReal x99=((x53)*(x75));
IkReal x100=((IkReal(0.271964562286312))*(x55)*(x56));
IkReal x101=((x55)*(x82));
IkReal x102=((IkReal(0.212475996640000))*(x55)*(x59));
IkReal x103=((IkReal(0.212475996640000))*(x53)*(x59));
IkReal x104=((x53)*(x82));
IkReal x105=((IkReal(0.271964562286312))*(x53)*(x56));
IkReal x106=((x55)*(x81));
IkReal x107=((x53)*(x81));
IkReal x108=((IkReal(4.00000000000000))*(x45)*(x46));
IkReal x109=((x55)*(x89));
IkReal x110=((x72)*(x78));
IkReal x111=((x53)*(x56)*(x60));
IkReal x112=((x70)*(x72));
IkReal x113=((x53)*(x89));
IkReal x114=((x55)*(x56)*(x60));
IkReal x115=((x68)*(x70));
IkReal x116=((x55)*(x91));
IkReal x117=((x68)*(x78));
IkReal x118=((x53)*(x91));
IkReal x119=((x55)*(x56)*(x61));
IkReal x120=((IkReal(0.641921440000000))*(x56)*(x78));
IkReal x121=((IkReal(0.259978183200000))*(x55)*(x74));
IkReal x122=((IkReal(0.835842531600000))*(x55)*(x69));
IkReal x123=((x53)*(x56)*(x61));
IkReal x124=((IkReal(0.835842531600000))*(x53)*(x69));
IkReal x125=((IkReal(0.641921440000000))*(x56)*(x70));
IkReal x126=((IkReal(0.259978183200000))*(x53)*(x74));
IkReal x127=((x108)*(x53));
IkReal x128=((x55)*(x56)*(x80));
IkReal x129=((x52)*(x55)*(x72));
IkReal x130=((x108)*(x55));
IkReal x131=((x49)*(x53)*(x72));
IkReal x132=((x53)*(x56)*(x80));
IkReal x133=((x45)*(x55)*(x76));
IkReal x134=((x49)*(x53)*(x68));
IkReal x135=((x48)*(x55)*(x71));
IkReal x136=((IkReal(7.71271964000000))*(x55)*(x90));
IkReal x137=((x48)*(x53)*(x71));
IkReal x138=((IkReal(7.71271964000000))*(x53)*(x90));
IkReal x139=((x52)*(x55)*(x68));
IkReal x140=((x45)*(x53)*(x76));
IkReal x141=((x53)*(x56)*(x71));
IkReal x142=((x55)*(x56)*(x76));
IkReal x143=((x55)*(x63)*(x69));
IkReal x144=((IkReal(0.641921440000000))*(x52)*(x55)*(x56));
IkReal x145=((x55)*(x56)*(x71));
IkReal x146=((IkReal(0.641921440000000))*(x49)*(x53)*(x56));
IkReal x147=((x53)*(x56)*(x76));
IkReal x148=((x53)*(x63)*(x69));
IkReal x149=((IkReal(2.78488036000000))*(x55)*(x90));
IkReal x150=((IkReal(2.78488036000000))*(x53)*(x90));
IkReal x151=((x128)+(x111)+(x92));
IkReal x152=((x132)+(x114)+(x94));
IkReal x153=((x150)+(x107)+(x67)+(x88));
IkReal x154=((x149)+(x106)+(x66)+(x87));
IkReal x155=((x148)+(x126)+(x124)+(x103));
IkReal x156=((x143)+(x122)+(x121)+(x102));
IkReal x157=((x127)+(x129)+(x110)+(x109)+(x93));
IkReal x158=((x131)+(x130)+(x113)+(x112)+(x95));
IkReal x159=((x145)+(x146)+(x147)+(x125)+(x123)+(x104)+(x105)+(x86));
IkReal x160=((x141)+(x142)+(x144)+(x120)+(x119)+(x100)+(x101)+(x85));
IkReal x161=((x140)+(x137)+(x139)+(x138)+(x117)+(x118)+(x65)+(x84)+(x99)+(x98));
IkReal x162=((x135)+(x134)+(x136)+(x133)+(x116)+(x115)+(x64)+(x83)+(x97)+(x96));
op[0]=((((IkReal(-1.00000000000000))*(x157)))+(((IkReal(-1.00000000000000))*(x152)))+(((IkReal(-1.00000000000000))*(x153)))+(x154)+(x151)+(x158));
op[1]=((((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(x160)))+(x156)+(x159));
op[2]=((((IkReal(-1.00000000000000))*(x152)))+(((IkReal(-1.00000000000000))*(x158)))+(((IkReal(-1.00000000000000))*(x161)))+(x162)+(x157)+(x151));
op[3]=((((x55)*(x57)*(x69)))+(((IkReal(-0.424951993280000))*(x53)*(x59)))+(((IkReal(0.519956366400000))*(x55)*(x74)))+(((IkReal(-1.00000000000000))*(x53)*(x57)*(x69)))+(((IkReal(0.424951993280000))*(x55)*(x59)))+(((IkReal(-1.67168506320000))*(x53)*(x69)))+(((IkReal(-0.519956366400000))*(x53)*(x74)))+(((IkReal(1.67168506320000))*(x55)*(x69))));
op[4]=((((IkReal(-1.00000000000000))*(x151)))+(((IkReal(-1.00000000000000))*(x158)))+(((IkReal(-1.00000000000000))*(x161)))+(x162)+(x157)+(x152));
op[5]=((((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(x159)))+(x160)+(x156));
op[6]=((((IkReal(-1.00000000000000))*(x157)))+(((IkReal(-1.00000000000000))*(x151)))+(((IkReal(-1.00000000000000))*(x153)))+(x154)+(x152)+(x158));
polyroots6(op,zeror,numroots);
IkReal j1array[6], cj1array[6], sj1array[6], tempj1array[1];
int numsolutions = 0;
for(int ij1 = 0; ij1 < numroots; ++ij1)
{
IkReal htj1 = zeror[ij1];
tempj1array[0]=((IkReal(2.00000000000000))*(atan(htj1)));
for(int kj1 = 0; kj1 < 1; ++kj1)
{
j1array[numsolutions] = tempj1array[kj1];
if( j1array[numsolutions] > IKPI )
{
    j1array[numsolutions]-=IK2PI;
}
else if( j1array[numsolutions] < -IKPI )
{
    j1array[numsolutions]+=IK2PI;
}
sj1array[numsolutions] = IKsin(j1array[numsolutions]);
cj1array[numsolutions] = IKcos(j1array[numsolutions]);
numsolutions++;
}
}
bool j1valid[6]={true,true,true,true,true,true};
_nj1 = 6;
for(int ij1 = 0; ij1 < numsolutions; ++ij1)
    {
if( !j1valid[ij1] )
{
    continue;
}
    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
htj1 = IKtan(j1/2);

j1evalpoly[0]=((((IkReal(-0.814435596450000))*(cj2)*(py)*(r00)))+(((IkReal(0.107404866000000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.265197200000000))*(cj2)*(r02)*(sj2)*((py)*(py))))+(((IkReal(0.113616578613500))*(cj2)*(py)*(r01)))+(((IkReal(0.265197200000000))*(cj2)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(py)*(r00)))+((((htj1)*(htj1))*(((((IkReal(-1.29794400000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.107404866000000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(-1.32598600000000))*(cj2)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.113616578613500))*(cj2)*(py)*(r01)))+(((IkReal(1.29794400000000))*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.684446504850000))*(cj2)*(py)*(r00)))+(((IkReal(5.29600000000000))*(pp)*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(px)*(r01)))+(((IkReal(0.400600000000000))*(cj2)*(pp)*(px)*(r00)))+(((IkReal(-1.29794400000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-0.801200000000000))*(cj2)*(r00)*((px)*(px)*(px))))+(((IkReal(0.400600000000000))*(cj2)*(pp)*(py)*(r01)))+(((IkReal(-5.29600000000000))*(pp)*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(-1.39244018000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(0.341560343160000))*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(0.113616578613500))*(cj2)*(px)*(r00)))+(((IkReal(-1.07244000000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(-0.801200000000000))*(cj2)*(pz)*(r02)*((px)*(px))))+(((IkReal(0.107404866000000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.684446504850000))*(cj2)*(px)*(r01)))+(((IkReal(0.354659701260000))*(py)*(r00)))+(((IkReal(-0.348948114210000))*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.341560343160000))*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(0.348948114210000))*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(1.32400000000000))*(pp)*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-1.32598600000000))*(cj2)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-0.354659701260000))*(px)*(r01)))+(((IkReal(1.29794400000000))*(r02)*(sj2)*((py)*(py))))+(((IkReal(-0.801200000000000))*(cj2)*(pz)*(r02)*((py)*(py))))+(((IkReal(1.32598600000000))*(cj2)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(py)*(r00)))+(((IkReal(1.39244018000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(-0.801200000000000))*(cj2)*(px)*(r00)*((py)*(py))))+(((IkReal(-0.801200000000000))*(cj2)*(py)*(r01)*((px)*(px))))+(((IkReal(-1.32400000000000))*(pp)*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(1.32598600000000))*(cj2)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-0.801200000000000))*(cj2)*(r01)*((py)*(py)*(py))))+(((IkReal(1.07244000000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))))))+(((IkReal(0.400600000000000))*(cj2)*(pp)*(px)*(r00)))+(((IkReal(-0.801200000000000))*(cj2)*(r00)*((px)*(px)*(px))))+(((IkReal(0.460897699580000))*(px)*(r01)))+(((IkReal(0.400600000000000))*(cj2)*(pp)*(py)*(r01)))+(((IkReal(-1.39244018000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(-0.348948114210000))*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(0.113616578613500))*(cj2)*(px)*(r00)))+((((htj1)*(htj1)*(htj1)*(htj1))*(((((IkReal(-1.29794400000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-1.32598600000000))*(cj2)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(1.29794400000000))*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.684446504850000))*(cj2)*(py)*(r00)))+(((IkReal(-0.107404866000000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(0.801200000000000))*(cj2)*(pz)*(r02)*((px)*(px))))+(((IkReal(0.801200000000000))*(cj2)*(r01)*((py)*(py)*(py))))+(((IkReal(5.29600000000000))*(pp)*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.107404866000000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(px)*(r01)))+(((IkReal(-0.113616578613500))*(cj2)*(px)*(r00)))+(((IkReal(-1.29794400000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-1.39244018000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(1.39244018000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(-5.29600000000000))*(pp)*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(0.341560343160000))*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(0.801200000000000))*(cj2)*(px)*(r00)*((py)*(py))))+(((IkReal(-0.400600000000000))*(cj2)*(pp)*(px)*(r00)))+(((IkReal(-0.684446504850000))*(cj2)*(px)*(r01)))+(((IkReal(0.354659701260000))*(py)*(r00)))+(((IkReal(-0.348948114210000))*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(0.801200000000000))*(cj2)*(pz)*(r02)*((py)*(py))))+(((IkReal(-0.341560343160000))*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(0.348948114210000))*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(1.32400000000000))*(pp)*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-1.32598600000000))*(cj2)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-0.354659701260000))*(px)*(r01)))+(((IkReal(1.29794400000000))*(r02)*(sj2)*((py)*(py))))+(((IkReal(1.32598600000000))*(cj2)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(py)*(r00)))+(((IkReal(1.07244000000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(0.801200000000000))*(cj2)*(py)*(r01)*((px)*(px))))+(((IkReal(-1.07244000000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))+(((IkReal(-1.32400000000000))*(pp)*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(1.32598600000000))*(cj2)*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.801200000000000))*(cj2)*(r00)*((px)*(px)*(px))))+(((IkReal(-0.113616578613500))*(cj2)*(py)*(r01)))+(((IkReal(-0.400600000000000))*(cj2)*(pp)*(py)*(r01)))))))+(((IkReal(-0.460897699580000))*(py)*(r00)))+(((IkReal(-1.07244000000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(1.32400000000000))*(pp)*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.801200000000000))*(cj2)*(pz)*(r02)*((px)*(px))))+(((IkReal(0.107404866000000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(0.814435596450000))*(cj2)*(px)*(r01)))+(((htj1)*(((((IkReal(3.24000000000000))*(pp)*(px)*(r01)*(sj2)))+(((IkReal(-0.648972000000000))*(cj2)*(r02)*((py)*(py))))+(((IkReal(0.648972000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(2.14488000000000))*(py)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.648972000000000))*(cj2)*(r02)*((px)*(px))))+(((IkReal(-5.29600000000000))*(cj2)*(pp)*(py)*(r00)*(sj2)))+(((IkReal(0.530394400000000))*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-3.24000000000000))*(pp)*(py)*(r00)*(sj2)))+(((IkReal(0.500994647100000))*(py)*(r00)*(sj2)))+(((IkReal(1.60240000000000))*(r00)*(sj2)*((px)*(px)*(px))))+(((IkReal(-0.530394400000000))*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.500994647100000))*(px)*(r01)*(sj2)))+(((IkReal(-0.530394400000000))*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(pp)*(px)*(r00)*(sj2)))+(((IkReal(-0.530394400000000))*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(0.530394400000000))*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(1.60240000000000))*(r01)*(sj2)*((py)*(py)*(py))))+(((IkReal(0.648972000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(-0.214809732000000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(0.530394400000000))*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(-0.227233157227000))*(py)*(r01)*(sj2)))+(((IkReal(-0.527116056840000))*(cj2)*(px)*(r01)*(sj2)))+(((IkReal(5.29600000000000))*(cj2)*(pp)*(px)*(r01)*(sj2)))+(((IkReal(0.160480360000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(0.527116056840000))*(cj2)*(py)*(r00)*(sj2)))+(((IkReal(1.60240000000000))*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(1.60240000000000))*(pz)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-0.160480360000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(-0.530394400000000))*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(1.60240000000000))*(px)*(r00)*(sj2)*((py)*(py))))+(((IkReal(0.530394400000000))*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(pp)*(py)*(r01)*(sj2)))+(((IkReal(-0.214809732000000))*(cj2)*(px)*(r00)*(sj2)))+(((IkReal(-2.14488000000000))*(px)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(py)*(r01)*(sj2)*((px)*(px))))+(((IkReal(-0.227233157227000))*(px)*(r00)*(sj2)))))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(px)*(r01)))+(((IkReal(0.265197200000000))*(cj2)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-0.801200000000000))*(cj2)*(pz)*(r02)*((py)*(py))))+((((htj1)*(htj1)*(htj1))*(((((IkReal(3.20480000000000))*(py)*(r01)*(sj2)*((px)*(px))))+(((IkReal(-1.60240000000000))*(pp)*(py)*(r01)*(sj2)))+(((IkReal(3.20480000000000))*(pz)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-0.320960720000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(3.20480000000000))*(px)*(r00)*(sj2)*((py)*(py))))+(((IkReal(4.28976000000000))*(py)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.454466314454000))*(py)*(r01)*(sj2)))+(((IkReal(3.20480000000000))*(r00)*(sj2)*((px)*(px)*(px))))+(((IkReal(3.20480000000000))*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.320960720000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(-0.429619464000000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(-0.454466314454000))*(px)*(r00)*(sj2)))+(((IkReal(-1.60240000000000))*(pp)*(px)*(r00)*(sj2)))+(((IkReal(3.20480000000000))*(r01)*(sj2)*((py)*(py)*(py))))+(((IkReal(-4.28976000000000))*(px)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-0.429619464000000))*(cj2)*(px)*(r00)*(sj2)))))))+((((htj1)*(htj1)*(htj1)*(htj1)*(htj1))*(((((IkReal(0.648972000000000))*(cj2)*(r02)*((py)*(py))))+(((IkReal(0.530394400000000))*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(-0.648972000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(2.14488000000000))*(py)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.530394400000000))*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(-3.24000000000000))*(pp)*(px)*(r01)*(sj2)))+(((IkReal(-0.530394400000000))*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-5.29600000000000))*(cj2)*(pp)*(px)*(r01)*(sj2)))+(((IkReal(0.530394400000000))*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(0.530394400000000))*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(r00)*(sj2)*((px)*(px)*(px))))+(((IkReal(-0.648972000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.530394400000000))*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(5.29600000000000))*(cj2)*(pp)*(py)*(r00)*(sj2)))+(((IkReal(-0.801200000000000))*(pp)*(px)*(r00)*(sj2)))+(((IkReal(1.60240000000000))*(r01)*(sj2)*((py)*(py)*(py))))+(((IkReal(0.500994647100000))*(px)*(r01)*(sj2)))+(((IkReal(-0.214809732000000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(-0.227233157227000))*(py)*(r01)*(sj2)))+(((IkReal(-0.500994647100000))*(py)*(r00)*(sj2)))+(((IkReal(0.160480360000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(1.60240000000000))*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.530394400000000))*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(1.60240000000000))*(pz)*(r02)*(sj2)*((py)*(py))))+(((IkReal(0.648972000000000))*(cj2)*(r02)*((px)*(px))))+(((IkReal(-0.160480360000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(1.60240000000000))*(px)*(r00)*(sj2)*((py)*(py))))+(((IkReal(0.527116056840000))*(cj2)*(px)*(r01)*(sj2)))+(((IkReal(-0.530394400000000))*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(3.24000000000000))*(pp)*(py)*(r00)*(sj2)))+(((IkReal(-0.801200000000000))*(pp)*(py)*(r01)*(sj2)))+(((IkReal(-0.214809732000000))*(cj2)*(px)*(r00)*(sj2)))+(((IkReal(-0.527116056840000))*(cj2)*(py)*(r00)*(sj2)))+(((IkReal(-2.14488000000000))*(px)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(py)*(r01)*(sj2)*((px)*(px))))+(((IkReal(-0.227233157227000))*(px)*(r00)*(sj2)))))))+(((IkReal(1.39244018000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(0.348948114210000))*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.801200000000000))*(cj2)*(px)*(r00)*((py)*(py))))+(((IkReal(-0.265197200000000))*(cj2)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-0.801200000000000))*(cj2)*(py)*(r01)*((px)*(px))))+(((IkReal(-0.801200000000000))*(cj2)*(r01)*((py)*(py)*(py))))+(((IkReal(1.07244000000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))+(((IkReal(-1.32400000000000))*(pp)*(px)*(r01)*((cj2)*(cj2))))+((((htj1)*(htj1)*(htj1)*(htj1)*(htj1)*(htj1))*(((((IkReal(-0.814435596450000))*(cj2)*(py)*(r00)))+(((IkReal(-0.265197200000000))*(cj2)*(r02)*(sj2)*((py)*(py))))+(((IkReal(0.265197200000000))*(cj2)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-0.107404866000000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(0.801200000000000))*(cj2)*(pz)*(r02)*((px)*(px))))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(py)*(r00)))+(((IkReal(0.801200000000000))*(cj2)*(r01)*((py)*(py)*(py))))+(((IkReal(-0.107404866000000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.113616578613500))*(cj2)*(px)*(r00)))+(((IkReal(-1.39244018000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(1.39244018000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(0.460897699580000))*(px)*(r01)))+(((IkReal(-0.348948114210000))*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(0.801200000000000))*(cj2)*(px)*(r00)*((py)*(py))))+(((IkReal(-0.460897699580000))*(py)*(r00)))+(((IkReal(-0.400600000000000))*(cj2)*(pp)*(px)*(r00)))+(((IkReal(1.32400000000000))*(pp)*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(0.814435596450000))*(cj2)*(px)*(r01)))+(((IkReal(0.801200000000000))*(cj2)*(pz)*(r02)*((py)*(py))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(px)*(r01)))+(((IkReal(0.265197200000000))*(cj2)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.348948114210000))*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(1.07244000000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(0.801200000000000))*(cj2)*(py)*(r01)*((px)*(px))))+(((IkReal(-0.265197200000000))*(cj2)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-1.07244000000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))+(((IkReal(0.801200000000000))*(cj2)*(r00)*((px)*(px)*(px))))+(((IkReal(-0.113616578613500))*(cj2)*(py)*(r01)))+(((IkReal(-1.32400000000000))*(pp)*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.400600000000000))*(cj2)*(pp)*(py)*(r01))))))));
j1evalpoly[1]=((((IkReal(-1.05422391000000))*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(0.459457956450000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(0.265197200000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(0.801200000000000))*(cj2)*(px)*(r01)*(sj2)*((py)*(py))))+((((htj1)*(htj1)*(htj1))*(((((IkReal(6.48000000000000))*(pp)*(py)*(r01)*((sj2)*(sj2))))+(((IkReal(1.29794400000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(0.106237998320000))*(cj2)*(px)*(r00)))+(((IkReal(-12.9600000000000))*(pz)*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(0.129989091600000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(0.106237998320000))*(cj2)*(py)*(r01)))+(((IkReal(6.48000000000000))*(pp)*(px)*(r00)*((sj2)*(sj2))))+(((IkReal(12.9600000000000))*(py)*(r01)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(0.417921265800000))*(px)*(r00)*((sj2)*(sj2))))+(((IkReal(0.129989091600000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(1.06078880000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(0.417921265800000))*(py)*(r01)*((sj2)*(sj2))))+(((IkReal(12.9600000000000))*(px)*(r00)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(-1.06078880000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(-1.29794400000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))+(((IkReal(-12.9600000000000))*(pz)*(r02)*((px)*(px))*((sj2)*(sj2))))))))+(((htj1)*(((((IkReal(-1.60240000000000))*(px)*(r01)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(-1.91345636000000))*(cj2)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-1.60240000000000))*(r01)*((px)*(px)*(px))*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(px)*(r01)*((cj2)*(cj2))*((pz)*(pz))))+(((IkReal(6.48000000000000))*(py)*(r01)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(-6.48000000000000))*(pz)*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(pp)*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.0516726108270000))*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(-1.60240000000000))*(r00)*((cj2)*(cj2))*((py)*(py)*(py))))+(((IkReal(0.0531189991600000))*(cj2)*(px)*(r00)))+(((IkReal(16.0000000000000))*(cj2)*(pp)*(r02)*(sj2)*((py)*(py))))+(((IkReal(0.530394400000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(1.91345636000000))*(cj2)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.208960632900000))*(py)*(r01)*((sj2)*(sj2))))+(((IkReal(0.0649945458000000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(0.801200000000000))*(pp)*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(0.648972000000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(2.14488000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.0649945458000000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(0.0531189991600000))*(cj2)*(py)*(r01)))+(((IkReal(-0.211161049173000))*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(1.60240000000000))*(r00)*((py)*(py)*(py))*((sj2)*(sj2))))+(((IkReal(-16.0000000000000))*(cj2)*(pp)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(2.14488000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-16.0000000000000))*(cj2)*(pp)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.0516726108270000))*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(py)*(r00)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(6.48000000000000))*(px)*(r00)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(3.24000000000000))*(pp)*(py)*(r01)*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(px)*(r01)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(-0.801200000000000))*(pp)*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(-2.14488000000000))*(r02)*(sj2)*((px)*(px))))+(((IkReal(1.60240000000000))*(r01)*((cj2)*(cj2))*((px)*(px)*(px))))+(((IkReal(-0.530394400000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(16.0000000000000))*(cj2)*(pp)*(r02)*(sj2)*((px)*(px))))+(((IkReal(1.60240000000000))*(py)*(r00)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(0.211161049173000))*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-6.48000000000000))*(pz)*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(-1.60240000000000))*(py)*(r00)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(0.801200000000000))*(pp)*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(0.214809732000000))*(cj2)*(px)*(r01)))+(((IkReal(-2.14488000000000))*(r02)*(sj2)*((py)*(py))))+(((IkReal(0.208960632900000))*(px)*(r00)*((sj2)*(sj2))))+(((IkReal(-1.60240000000000))*(py)*(r00)*((cj2)*(cj2))*((pz)*(pz))))+(((IkReal(-1.91345636000000))*(cj2)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-1.60240000000000))*(px)*(r01)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(-0.214809732000000))*(cj2)*(py)*(r00)))+(((IkReal(1.91345636000000))*(cj2)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(3.24000000000000))*(pp)*(px)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.648972000000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))))))+(((IkReal(-3.24000000000000))*(cj2)*(py)*(r01)*(sj2)*((pz)*(pz))))+(((IkReal(0.801200000000000))*(cj2)*(px)*(r01)*(sj2)*((pz)*(pz))))+(((IkReal(-1.05422391000000))*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(4.00000000000000))*(pp)*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.265197200000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(-0.324486000000000))*(py)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.400600000000000))*(cj2)*(pp)*(px)*(r01)*(sj2)))+(((IkReal(-0.801200000000000))*(cj2)*(py)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((py)*(py))))+((((htj1)*(htj1)*(htj1)*(htj1)*(htj1))*(((((IkReal(1.91345636000000))*(cj2)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-1.60240000000000))*(py)*(r00)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(px)*(r01)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(6.48000000000000))*(py)*(r01)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(16.0000000000000))*(cj2)*(pp)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-6.48000000000000))*(pz)*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(0.0516726108270000))*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(-16.0000000000000))*(cj2)*(pp)*(r02)*(sj2)*((py)*(py))))+(((IkReal(2.14488000000000))*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.0531189991600000))*(cj2)*(px)*(r00)))+(((IkReal(0.530394400000000))*(py)*(pz)*(r00)*(sj2)))+(((IkReal(-0.0516726108270000))*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(0.208960632900000))*(py)*(r01)*((sj2)*(sj2))))+(((IkReal(-16.0000000000000))*(cj2)*(pp)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-2.14488000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(16.0000000000000))*(cj2)*(pp)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.0649945458000000))*(py)*(r01)*((cj2)*(cj2))))+(((IkReal(0.648972000000000))*(cj2)*(py)*(pz)*(r00)*(sj2)))+(((IkReal(1.91345636000000))*(cj2)*(r02)*(sj2)*((py)*(py))))+(((IkReal(1.60240000000000))*(py)*(r00)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(0.214809732000000))*(cj2)*(py)*(r00)))+(((IkReal(1.60240000000000))*(r00)*((cj2)*(cj2))*((py)*(py)*(py))))+(((IkReal(-1.60240000000000))*(r01)*((cj2)*(cj2))*((px)*(px)*(px))))+(((IkReal(0.0649945458000000))*(px)*(r00)*((cj2)*(cj2))))+(((IkReal(0.801200000000000))*(pp)*(py)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(pp)*(px)*(r01)*((sj2)*(sj2))))+(((IkReal(0.0531189991600000))*(cj2)*(py)*(r01)))+(((IkReal(1.60240000000000))*(r01)*((px)*(px)*(px))*((sj2)*(sj2))))+(((IkReal(6.48000000000000))*(px)*(r00)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(3.24000000000000))*(pp)*(py)*(r01)*((sj2)*(sj2))))+(((IkReal(1.60240000000000))*(px)*(r01)*((pz)*(pz))*((sj2)*(sj2))))+(((IkReal(-1.91345636000000))*(cj2)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(1.60240000000000))*(py)*(r00)*((cj2)*(cj2))*((pz)*(pz))))+(((IkReal(-1.60240000000000))*(px)*(r01)*((cj2)*(cj2))*((pz)*(pz))))+(((IkReal(0.801200000000000))*(pp)*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.530394400000000))*(px)*(pz)*(r01)*(sj2)))+(((IkReal(-0.211161049173000))*(px)*(r01)*((cj2)*(cj2))))+(((IkReal(-1.60240000000000))*(r00)*((py)*(py)*(py))*((sj2)*(sj2))))+(((IkReal(-6.48000000000000))*(pz)*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(-0.214809732000000))*(cj2)*(px)*(r01)))+(((IkReal(-1.60240000000000))*(py)*(r00)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(-1.60240000000000))*(px)*(r01)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(-2.14488000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-1.91345636000000))*(cj2)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.208960632900000))*(px)*(r00)*((sj2)*(sj2))))+(((IkReal(3.24000000000000))*(pp)*(px)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(pp)*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(2.14488000000000))*(r02)*(sj2)*((py)*(py))))+(((IkReal(0.211161049173000))*(py)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.648972000000000))*(cj2)*(px)*(pz)*(r01)*(sj2)))))))+(((IkReal(0.460897699580000))*(py)*(r01)*(sj2)))+((((htj1)*(htj1))*(((((IkReal(-0.710943640000000))*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(0.429619464000000))*(py)*(r00)*(sj2)))+(((IkReal(0.459457956450000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(0.265197200000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(-0.429619464000000))*(px)*(r01)*(sj2)))+(((IkReal(-4.00600000000000))*(cj2)*(r01)*(sj2)*((px)*(px)*(px))))+(((IkReal(1.37518463000000))*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(2.00300000000000))*(cj2)*(pp)*(px)*(r01)*(sj2)))+(((IkReal(-3.24000000000000))*(cj2)*(py)*(r01)*(sj2)*((pz)*(pz))))+(((IkReal(-0.265197200000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(-16.0000000000000))*(pp)*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(-0.324486000000000))*(py)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(16.0000000000000))*(pp)*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-1.13446409000000))*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-0.710943640000000))*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(4.00600000000000))*(cj2)*(py)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(0.460897699580000))*(py)*(r01)*(sj2)))+(((IkReal(4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(0.460897699580000))*(px)*(r00)*(sj2)))+(((IkReal(0.396485792932500))*(cj2)*(py)*(r00)*(sj2)))+(((IkReal(-4.00000000000000))*(pp)*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(4.00600000000000))*(cj2)*(r00)*(sj2)*((py)*(py)*(py))))+(((IkReal(-3.24000000000000))*(cj2)*(px)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(0.459457956450000))*(cj2)*(px)*(r00)*(sj2)))+(((IkReal(3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(16.0000000000000))*(pp)*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-4.00600000000000))*(cj2)*(px)*(r01)*(sj2)*((py)*(py))))+(((IkReal(1.07244000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(px)*(r00)*(sj2)))+(((IkReal(4.88826418000000))*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-4.00600000000000))*(cj2)*(px)*(r01)*(sj2)*((pz)*(pz))))+(((IkReal(0.324486000000000))*(px)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(4.88826418000000))*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-1.07244000000000))*(cj2)*(r02)*((py)*(py))))+(((IkReal(4.00600000000000))*(cj2)*(py)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-1.13446409000000))*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(1.37518463000000))*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-1.07244000000000))*(cj2)*(r02)*((px)*(px))))+(((IkReal(-0.396485792932500))*(cj2)*(px)*(r01)*(sj2)))+(((IkReal(-2.00300000000000))*(cj2)*(pp)*(py)*(r00)*(sj2)))+(((IkReal(-16.0000000000000))*(pp)*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(py)*(r01)*(sj2)))+(((IkReal(-4.00000000000000))*(pp)*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(1.07244000000000))*(cj2)*(py)*(pz)*(r01)))))))+(((IkReal(1.07244000000000))*(cj2)*(r02)*((py)*(py))))+(((IkReal(0.801200000000000))*(cj2)*(r01)*(sj2)*((px)*(px)*(px))))+(((IkReal(1.39244018000000))*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-1.07244000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(0.460897699580000))*(px)*(r00)*(sj2)))+(((IkReal(4.00000000000000))*(pp)*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.801200000000000))*(cj2)*(r00)*(sj2)*((py)*(py)*(py))))+(((IkReal(-3.24000000000000))*(cj2)*(px)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(0.459457956450000))*(cj2)*(px)*(r00)*(sj2)))+(((IkReal(0.0258363054135000))*(cj2)*(py)*(r00)*(sj2)))+(((IkReal(-0.0258363054135000))*(cj2)*(px)*(r01)*(sj2)))+(((IkReal(3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(px)*(r00)*(sj2)))+(((IkReal(-1.07244000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(0.324486000000000))*(px)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(1.13446409000000))*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(1.13446409000000))*(r02)*((cj2)*(cj2))*((py)*(py))))+((((htj1)*(htj1)*(htj1)*(htj1))*(((((IkReal(-0.460897699580000))*(py)*(r01)*(sj2)))+(((IkReal(3.24000000000000))*(cj2)*(px)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(-0.710943640000000))*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(0.429619464000000))*(py)*(r00)*(sj2)))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(py)*(r01)*(sj2)))+(((IkReal(-0.429619464000000))*(px)*(r01)*(sj2)))+(((IkReal(-4.00600000000000))*(cj2)*(r01)*(sj2)*((px)*(px)*(px))))+(((IkReal(1.37518463000000))*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(-3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(2.00300000000000))*(cj2)*(pp)*(px)*(r01)*(sj2)))+(((IkReal(-0.460897699580000))*(px)*(r00)*(sj2)))+(((IkReal(-16.0000000000000))*(pp)*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(16.0000000000000))*(pp)*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-0.265197200000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(-1.13446409000000))*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(-0.710943640000000))*(r02)*((py)*(py))*((sj2)*(sj2))))+(((IkReal(4.00600000000000))*(cj2)*(py)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(-0.324486000000000))*(px)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(0.324486000000000))*(py)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(0.396485792932500))*(cj2)*(py)*(r00)*(sj2)))+(((IkReal(-4.00000000000000))*(pp)*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((py)*(py))))+(((IkReal(4.00600000000000))*(cj2)*(r00)*(sj2)*((py)*(py)*(py))))+(((IkReal(-0.459457956450000))*(cj2)*(px)*(r00)*(sj2)))+(((IkReal(16.0000000000000))*(pp)*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-4.00600000000000))*(cj2)*(px)*(r01)*(sj2)*((py)*(py))))+(((IkReal(1.07244000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(4.88826418000000))*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-4.00600000000000))*(cj2)*(px)*(r01)*(sj2)*((pz)*(pz))))+(((IkReal(4.88826418000000))*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-1.07244000000000))*(cj2)*(r02)*((py)*(py))))+(((IkReal(4.00600000000000))*(cj2)*(py)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-1.13446409000000))*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(1.37518463000000))*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.459457956450000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(-1.07244000000000))*(cj2)*(r02)*((px)*(px))))+(((IkReal(-0.396485792932500))*(cj2)*(px)*(r01)*(sj2)))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(px)*(r00)*(sj2)))+(((IkReal(-2.00300000000000))*(cj2)*(pp)*(py)*(r00)*(sj2)))+(((IkReal(0.265197200000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(-16.0000000000000))*(pp)*(r02)*((px)*(px))*((sj2)*(sj2))))+(((IkReal(-4.00000000000000))*(pp)*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(1.07244000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(3.24000000000000))*(cj2)*(py)*(r01)*(sj2)*((pz)*(pz))))))))+(((IkReal(0.400600000000000))*(cj2)*(pp)*(py)*(r00)*(sj2)))+(((IkReal(1.39244018000000))*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(cj2)*(py)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-1.62000000000000))*(cj2)*(pp)*(py)*(r01)*(sj2)))+(((IkReal(1.07244000000000))*(cj2)*(r02)*((px)*(px))))+(((IkReal(-4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((py)*(py))))+((((htj1)*(htj1)*(htj1)*(htj1)*(htj1)*(htj1))*(((((IkReal(-1.05422391000000))*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(-0.460897699580000))*(py)*(r01)*(sj2)))+(((IkReal(3.24000000000000))*(cj2)*(px)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(py)*(r01)*(sj2)))+(((IkReal(0.801200000000000))*(cj2)*(px)*(r01)*(sj2)*((py)*(py))))+(((IkReal(-3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((px)*(px))))+(((IkReal(0.801200000000000))*(cj2)*(px)*(r01)*(sj2)*((pz)*(pz))))+(((IkReal(-1.05422391000000))*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(4.00000000000000))*(pp)*(py)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(-0.460897699580000))*(px)*(r00)*(sj2)))+(((IkReal(-0.400600000000000))*(cj2)*(pp)*(px)*(r01)*(sj2)))+(((IkReal(-0.801200000000000))*(cj2)*(py)*(r00)*(sj2)*((pz)*(pz))))+(((IkReal(-0.265197200000000))*(cj2)*(px)*(pz)*(r01)))+(((IkReal(-0.324486000000000))*(px)*(pz)*(r01)*((cj2)*(cj2))))+(((IkReal(0.324486000000000))*(py)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(1.07244000000000))*(cj2)*(r02)*((py)*(py))))+(((IkReal(0.801200000000000))*(cj2)*(r01)*(sj2)*((px)*(px)*(px))))+(((IkReal(1.39244018000000))*(py)*(pz)*(r01)*((sj2)*(sj2))))+(((IkReal(-1.07244000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(4.00000000000000))*(pp)*(px)*(pz)*(r00)*((cj2)*(cj2))))+(((IkReal(-3.24000000000000))*(cj2)*(pz)*(r02)*(sj2)*((py)*(py))))+(((IkReal(-0.801200000000000))*(cj2)*(r00)*(sj2)*((py)*(py)*(py))))+(((IkReal(0.0258363054135000))*(cj2)*(py)*(r00)*(sj2)))+(((IkReal(-0.0258363054135000))*(cj2)*(px)*(r01)*(sj2)))+(((IkReal(-0.459457956450000))*(cj2)*(px)*(r00)*(sj2)))+(((IkReal(-1.07244000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(1.13446409000000))*(r02)*((cj2)*(cj2))*((px)*(px))))+(((IkReal(1.13446409000000))*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(-0.459457956450000))*(cj2)*(py)*(r01)*(sj2)))+(((IkReal(1.62000000000000))*(cj2)*(pp)*(px)*(r00)*(sj2)))+(((IkReal(0.265197200000000))*(cj2)*(py)*(pz)*(r00)))+(((IkReal(0.400600000000000))*(cj2)*(pp)*(py)*(r00)*(sj2)))+(((IkReal(1.39244018000000))*(px)*(pz)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.801200000000000))*(cj2)*(py)*(r00)*(sj2)*((px)*(px))))+(((IkReal(1.07244000000000))*(cj2)*(r02)*((px)*(px))))+(((IkReal(-4.00000000000000))*(pp)*(r02)*((cj2)*(cj2))*((py)*(py))))+(((IkReal(3.24000000000000))*(cj2)*(py)*(r01)*(sj2)*((pz)*(pz)))))))));
if( IKabs(j1evalpoly[0]) > 0.0000001000000000  || IKabs(j1evalpoly[1]) > 0.0000001000000000  )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < numsolutions; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x163=(py)*(py);
IkReal x164=(px)*(px);
IkReal x165=((cj1)*(cj2));
IkReal x166=((sj1)*(sj2));
IkReal x167=((IkReal(0.200300000000000))*(x163));
IkReal x168=((IkReal(0.200300000000000))*(x164));
gconst1=IKsign(((((x166)*(x167)))+(((x166)*(x168)))+(((IkReal(-1.00000000000000))*(x165)*(x167)))+(((IkReal(-1.00000000000000))*(x165)*(x168)))));
IkReal x169=(px)*(px);
IkReal x170=(py)*(py);
IkReal x171=((sj1)*(sj2));
IkReal x172=((IkReal(1.00000000000000))*(cj1)*(cj2));
dummyeval[0]=((((x169)*(x171)))+(((IkReal(-1.00000000000000))*(x170)*(x172)))+(((x170)*(x171)))+(((IkReal(-1.00000000000000))*(x169)*(x172))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x173=(cj1)*(cj1);
IkReal x174=(sj1)*(sj1);
IkReal x175=((IkReal(0.0811215000000000))*(cj2));
IkReal x176=((cj2)*(py));
IkReal x177=((IkReal(0.263555977500000))*(cj1));
IkReal x178=((IkReal(1.00000000000000))*(pp));
IkReal x179=((IkReal(0.200300000000000))*(pz));
IkReal x180=((IkReal(0.268110000000000))*(cj1));
IkReal x181=((cj2)*(px));
IkReal x182=((cj1)*(pp));
IkReal x183=((cj1)*(sj2));
IkReal x184=((px)*(sj1)*(sj2));
IkReal x185=((py)*(sj1)*(sj2));
IkReal x186=((IkReal(0.810000000000000))*(pz)*(sj2));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x178)*(x185)))+(((x176)*(x182)))+(((IkReal(-1.00000000000000))*(py)*(x180)))+(((sj1)*(x179)*(x181)))+(((IkReal(0.0662993000000000))*(px)))+(((px)*(x175)))+(((px)*(x179)*(x183)))+(((IkReal(-1.00000000000000))*(py)*(x173)*(x186)))+(((IkReal(-1.00000000000000))*(x176)*(x177)))+(((IkReal(-0.0644940225000000))*(x185)))+(((IkReal(-1.00000000000000))*(py)*(x174)*(x186))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x178)*(x184)))+(((IkReal(-1.00000000000000))*(py)*(x179)*(x183)))+(((IkReal(-1.00000000000000))*(px)*(x174)*(x186)))+(((IkReal(-1.00000000000000))*(py)*(x175)))+(((IkReal(-1.00000000000000))*(x177)*(x181)))+(((IkReal(-0.0662993000000000))*(py)))+(((IkReal(-1.00000000000000))*(px)*(x180)))+(((IkReal(-0.0644940225000000))*(x184)))+(((IkReal(-1.00000000000000))*(sj1)*(x176)*(x179)))+(((IkReal(-1.00000000000000))*(px)*(x173)*(x186)))+(((x181)*(x182))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x178)*(x185)))+(((x176)*(x182)))+(((IkReal(-1.00000000000000))*(py)*(x180)))+(((sj1)*(x179)*(x181)))+(((IkReal(0.0662993000000000))*(px)))+(((px)*(x175)))+(((px)*(x179)*(x183)))+(((IkReal(-1.00000000000000))*(py)*(x173)*(x186)))+(((IkReal(-1.00000000000000))*(x176)*(x177)))+(((IkReal(-0.0644940225000000))*(x185)))+(((IkReal(-1.00000000000000))*(py)*(x174)*(x186)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x178)*(x184)))+(((IkReal(-1.00000000000000))*(py)*(x179)*(x183)))+(((IkReal(-1.00000000000000))*(px)*(x174)*(x186)))+(((IkReal(-1.00000000000000))*(py)*(x175)))+(((IkReal(-1.00000000000000))*(x177)*(x181)))+(((IkReal(-0.0662993000000000))*(py)))+(((IkReal(-1.00000000000000))*(px)*(x180)))+(((IkReal(-0.0644940225000000))*(x184)))+(((IkReal(-1.00000000000000))*(sj1)*(x176)*(x179)))+(((IkReal(-1.00000000000000))*(px)*(x173)*(x186)))+(((x181)*(x182)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x187=IKcos(j0);
IkReal x188=IKsin(j0);
IkReal x189=((cj1)*(sj2));
IkReal x190=((IkReal(1.00000000000000))*(px));
IkReal x191=((IkReal(0.331000000000000))*(sj1));
IkReal x192=((py)*(r00));
IkReal x193=((cj2)*(sj1));
IkReal x194=((cj2)*(r02));
IkReal x195=((IkReal(0.662000000000000))*(pz));
IkReal x196=((px)*(sj1));
IkReal x197=((IkReal(1.00000000000000))*(py));
IkReal x198=((IkReal(0.662000000000000))*(cj2));
IkReal x199=((py)*(sj1));
IkReal x200=((sj1)*(sj2));
IkReal x201=((cj2)*(r01));
IkReal x202=((IkReal(1.00000000000000))*(pz));
IkReal x203=((r00)*(x188));
IkReal x204=((sj2)*(x187));
IkReal x205=((cj1)*(x188));
IkReal x206=((r01)*(x188));
IkReal x207=((sj2)*(x188));
IkReal x208=((cj1)*(x187));
IkReal x209=((IkReal(0.100150000000000))*(x187));
IkReal x210=((px)*(x205));
evalcond[0]=((IkReal(0.331000000000000))+(((pz)*(x189)))+(((IkReal(0.405000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x188)*(x190)*(x200)))+(((cj2)*(x210)))+(((IkReal(-1.00000000000000))*(cj2)*(x197)*(x208)))+(((x199)*(x204)))+(((pz)*(x193))));
evalcond[1]=((((IkReal(-0.100150000000000))*(cj1)*(x194)))+(((IkReal(-1.00000000000000))*(r01)*(x193)*(x209)))+(((pz)*(x201)*(x205)))+(((IkReal(-1.00000000000000))*(x190)*(x194)*(x208)))+(((cj2)*(pz)*(r00)*(x208)))+(((x189)*(x192)))+(((IkReal(-1.00000000000000))*(r01)*(x189)*(x190)))+(((IkReal(-1.00000000000000))*(r01)*(x189)*(x209)))+(((r02)*(x199)*(x207)))+(((x192)*(x193)))+(((IkReal(-1.00000000000000))*(x194)*(x197)*(x205)))+(((IkReal(-1.00000000000000))*(x200)*(x202)*(x206)))+(((IkReal(-1.00000000000000))*(r00)*(x187)*(x200)*(x202)))+(((r02)*(x196)*(x204)))+(((IkReal(-0.405000000000000))*(r00)*(x204)))+(((IkReal(0.100150000000000))*(x193)*(x203)))+(((IkReal(0.100150000000000))*(x189)*(x203)))+(((IkReal(0.100150000000000))*(r02)*(x200)))+(((IkReal(-1.00000000000000))*(r01)*(x190)*(x193)))+(((IkReal(-0.405000000000000))*(sj2)*(x206))));
evalcond[2]=((((IkReal(2.00000000000000))*(px)*(py)*(r01)*(sj1)*(sj2)*(IKsin(j0))))+(((IkReal(2.00000000000000))*(px)*(pz)*(r02)*(sj1)*(sj2)*(IKsin(j0))))+(((IkReal(-0.263555977500000))*(cj2)*(r02)*(sj1)))+(((IkReal(-2.00000000000000))*(py)*(pz)*(r02)*(sj1)*(sj2)*(IKcos(j0))))+(((IkReal(-0.810000000000000))*(cj2)*(pz)*(r02)))+(((IkReal(-0.0811215000000000))*(cj2)*(r00)*(IKcos(j0))))+(((IkReal(-0.662000000000000))*(pz)*(r02)))+(((IkReal(-0.0662993000000000))*(r00)*(IKcos(j0))))+(((IkReal(0.200300000000000))*(cj2)*(py)*(r02)*(sj1)*(IKsin(j0))))+(((IkReal(-0.263555977500000))*(cj1)*(cj2)*(r00)*(IKsin(j0))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r02)*(sj2)*(IKsin(j0))))+(((pp)*(r01)*(sj1)*(sj2)*(IKcos(j0))))+(((IkReal(-0.810000000000000))*(cj2)*(px)*(r00)))+(((IkReal(-2.00000000000000))*(cj1)*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pp)*(r01)*(IKcos(j0))))+(((IkReal(-0.200300000000000))*(cj1)*(pz)*(r00)*(sj2)*(IKcos(j0))))+(((IkReal(-0.810000000000000))*(py)*(r02)*(sj2)*(IKcos(j0))))+(((IkReal(-2.00000000000000))*(r01)*(sj1)*(sj2)*((py)*(py))*(IKcos(j0))))+(((IkReal(2.00000000000000))*(cj1)*(cj2)*(r01)*((py)*(py))*(IKcos(j0))))+(((IkReal(-1.00000000000000))*(pp)*(r00)*(sj1)*(sj2)*(IKsin(j0))))+(((IkReal(0.0644940225000000))*(r01)*(sj1)*(sj2)*(IKcos(j0))))+(((IkReal(-0.200300000000000))*(cj1)*(cj2)*(px)*(r01)))+(((IkReal(-0.0811215000000000))*(cj2)*(r01)*(IKsin(j0))))+(((IkReal(0.810000000000000))*(pz)*(r01)*(sj2)*(IKcos(j0))))+(((IkReal(-2.00000000000000))*(cj2)*(py)*(pz)*(r01)*(sj1)))+(((cj1)*(pp)*(r02)*(sj2)))+(((IkReal(-0.662000000000000))*(py)*(r01)))+(((IkReal(0.200300000000000))*(px)*(r01)*(sj1)*(sj2)))+(((IkReal(-2.00000000000000))*(cj1)*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-0.200300000000000))*(py)*(r00)*(sj1)*(sj2)))+(((IkReal(-2.00000000000000))*(px)*(py)*(r00)*(sj1)*(sj2)*(IKcos(j0))))+(((cj2)*(pp)*(r02)*(sj1)))+(((cj1)*(cj2)*(pp)*(r00)*(IKsin(j0))))+(((IkReal(-0.200300000000000))*(cj1)*(pz)*(r01)*(sj2)*(IKsin(j0))))+(((IkReal(-2.00000000000000))*(cj2)*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(0.268110000000000))*(cj1)*(r01)*(IKcos(j0))))+(((IkReal(-0.662000000000000))*(px)*(r00)))+(((IkReal(0.0644940225000000))*(cj1)*(r02)*(sj2)))+(((IkReal(-0.0662993000000000))*(r01)*(IKsin(j0))))+(((IkReal(-0.810000000000000))*(pz)*(r00)*(sj2)*(IKsin(j0))))+(((IkReal(2.00000000000000))*(r00)*(sj1)*(sj2)*((px)*(px))*(IKsin(j0))))+(((IkReal(-0.268110000000000))*(r02)*(sj1)))+(((IkReal(0.200300000000000))*(cj2)*(px)*(r02)*(sj1)*(IKcos(j0))))+(((IkReal(2.00000000000000))*(cj1)*(cj2)*(py)*(pz)*(r02)*(IKcos(j0))))+(((IkReal(-2.00000000000000))*(cj2)*(px)*(pz)*(r00)*(sj1)))+(((IkReal(0.263555977500000))*(cj1)*(cj2)*(r01)*(IKcos(j0))))+(((IkReal(0.200300000000000))*(cj1)*(px)*(r02)*(sj2)*(IKcos(j0))))+(((IkReal(0.200300000000000))*(cj1)*(cj2)*(py)*(r00)))+(((IkReal(-2.00000000000000))*(cj1)*(cj2)*(px)*(py)*(r01)*(IKsin(j0))))+(((IkReal(2.00000000000000))*(cj1)*(cj2)*(px)*(py)*(r00)*(IKcos(j0))))+(((IkReal(-2.00000000000000))*(cj1)*(cj2)*(px)*(pz)*(r02)*(IKsin(j0))))+(((IkReal(0.810000000000000))*(px)*(r02)*(sj2)*(IKsin(j0))))+(((IkReal(-0.200300000000000))*(cj2)*(pz)*(r00)*(sj1)*(IKcos(j0))))+(((IkReal(-0.268110000000000))*(cj1)*(r00)*(IKsin(j0))))+(((IkReal(-0.810000000000000))*(cj2)*(py)*(r01)))+(((IkReal(-2.00000000000000))*(cj1)*(cj2)*(r00)*((px)*(px))*(IKsin(j0))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r00)*(sj2)))+(((IkReal(-0.0644940225000000))*(r00)*(sj1)*(sj2)*(IKsin(j0))))+(((IkReal(-0.200300000000000))*(cj2)*(pz)*(r01)*(sj1)*(IKsin(j0)))));
evalcond[3]=((IkReal(0.283616022500000))+(((x193)*(x195)))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.662000000000000))*(x199)*(x204)))+(((IkReal(-1.00000000000000))*(py)*(x198)*(x208)))+(((x189)*(x195)))+(((x198)*(x210)))+(((IkReal(0.200300000000000))*(py)*(x188)))+(((IkReal(0.200300000000000))*(px)*(x187)))+(pp)+(((IkReal(0.810000000000000))*(x210)))+(((IkReal(-0.810000000000000))*(py)*(x208)))+(((IkReal(-0.662000000000000))*(x196)*(x207)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[4]=((((IkReal(0.100150000000000))*(x206)))+(((IkReal(-0.405000000000000))*(r01)*(x208)))+(((IkReal(0.331000000000000))*(cj1)*(cj2)*(x203)))+(((px)*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj1)))+(((r00)*(x209)))+(((IkReal(-0.331000000000000))*(x201)*(x208)))+(((IkReal(0.331000000000000))*(r02)*(x189)))+(((x191)*(x194)))+(((IkReal(-1.00000000000000))*(sj2)*(x191)*(x203)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(cj1)*(x203)))+(((r01)*(x191)*(x204))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x211=((cj1)*(sj2));
IkReal x212=((cj2)*(sj1));
IkReal x213=((cj0)*(py));
IkReal x214=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x211)*(x213)))+(((x212)*(x213)))+(((IkReal(-1.00000000000000))*(x211)*(x214)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x212)*(x214))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x215=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x211)*(x213)))+(((x212)*(x213)))+(((IkReal(-1.00000000000000))*(x211)*(x214)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x212)*(x214)))));
j3array[0]=((IkReal(-1.00000000000000))*(x215));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x215))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x216=IKsin(j3);
IkReal x217=IKcos(j3);
IkReal x218=((cj2)*(sj1));
IkReal x219=((cj1)*(sj2));
IkReal x220=((sj2)*(x217));
IkReal x221=((px)*(sj0)*(x217));
IkReal x222=((IkReal(1.00000000000000))*(cj0)*(py)*(x217));
evalcond[0]=((((cj0)*(px)*(x216)))+(((py)*(sj0)*(x216)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x217)))+(((IkReal(-1.00000000000000))*(x219)*(x222)))+(((x219)*(x221)))+(((IkReal(-1.00000000000000))*(x218)*(x222)))+(((IkReal(0.100150000000000))*(x216)))+(((x218)*(x221)))+(((IkReal(0.405000000000000))*(x220)))+(((pz)*(sj1)*(x220))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x223=((IkReal(1.00000000000000))*(sj3));
IkReal x224=((cj2)*(cj3));
IkReal x225=((cj0)*(r01));
IkReal x226=((cj3)*(sj2));
IkReal x227=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((IkReal(-1.00000000000000))*(sj1)*(x224)*(x227)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x223)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x223)))+(((cj1)*(x225)*(x226)))+(((cj1)*(r02)*(x224)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x226)))+(((sj1)*(x224)*(x225)))+(((IkReal(-1.00000000000000))*(cj1)*(x226)*(x227))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x228=((sj1)*(sj3));
IkReal x229=((IkReal(1.00000000000000))*(cj0));
IkReal x230=((r02)*(sj2));
IkReal x231=((IkReal(1.00000000000000))*(cj1));
IkReal x232=((sj1)*(sj2));
IkReal x233=((cj2)*(r01));
IkReal x234=((cj2)*(r02));
IkReal x235=((r00)*(sj0));
IkReal x236=((cj2)*(x235));
IkReal x237=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x230)*(x231)))+(((x232)*(x235)))+(((IkReal(-1.00000000000000))*(r01)*(x229)*(x232)))+(((IkReal(-1.00000000000000))*(sj1)*(x234)))+(((cj0)*(cj1)*(x233)))+(((IkReal(-1.00000000000000))*(x231)*(x236))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((x228)*(x236)))+(((x228)*(x230)))+(((IkReal(-1.00000000000000))*(r01)*(x229)*(x237)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x229)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x235)*(x237)))+(((IkReal(-1.00000000000000))*(x228)*(x229)*(x233)))+(((IkReal(-1.00000000000000))*(sj3)*(x231)*(x234))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(x230)*(x231)))+(((x232)*(x235)))+(((IkReal(-1.00000000000000))*(r01)*(x229)*(x232)))+(((IkReal(-1.00000000000000))*(sj1)*(x234)))+(((cj0)*(cj1)*(x233)))+(((IkReal(-1.00000000000000))*(x231)*(x236)))))), ((gconst2)*(((((x228)*(x236)))+(((x228)*(x230)))+(((IkReal(-1.00000000000000))*(r01)*(x229)*(x237)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x229)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x235)*(x237)))+(((IkReal(-1.00000000000000))*(x228)*(x229)*(x233)))+(((IkReal(-1.00000000000000))*(sj3)*(x231)*(x234)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x238=((cj2)*(sj1));
IkReal x239=((cj1)*(sj2));
IkReal x240=((cj0)*(r01));
IkReal x241=((sj1)*(sj2));
IkReal x242=((r00)*(sj0));
IkReal x243=((cj1)*(cj2));
IkReal x244=((r02)*(sj3));
IkReal x245=((IkReal(1.00000000000000))*(sj3)*(x242));
evalcond[0]=((((r02)*(x238)))+(((r02)*(x239)))+(((x240)*(x241)))+(((IkReal(-1.00000000000000))*(x241)*(x242)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x240)*(x243)))+(((x242)*(x243))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x241)*(x244)))+(((sj5)*(IKcos(j4))))+(((sj3)*(x239)*(x240)))+(((IkReal(-1.00000000000000))*(x238)*(x245)))+(((sj3)*(x238)*(x240)))+(((IkReal(-1.00000000000000))*(x239)*(x245)))+(((x243)*(x244)))+(((cj0)*(cj3)*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
    }

}

}

} else
{
continue;

}

}
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots6(IkReal rawcoeffs[6+1], IkReal rawroots[6], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[6];
    const int maxsteps = 110;
    for(int i = 0; i < 6; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[6];
    IkReal err[6];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 6; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 6; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 6; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 6; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[6] = {false};
    for(int i = 0; i < 6; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 6; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
