/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:02:32.992885
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27;
x0=IKcos(j[1]);
x1=IKsin(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[2]);
x4=IKcos(j[0]);
x5=IKsin(j[0]);
x6=IKcos(j[3]);
x7=IKcos(j[4]);
x8=IKsin(j[5]);
x9=IKcos(j[5]);
x10=IKsin(j[3]);
x11=IKsin(j[4]);
x12=((IkReal(0.331000000000000))*(x1));
x13=((IkReal(1.00000000000000))*(x10));
x14=((IkReal(1.00000000000000))*(x6));
x15=((IkReal(1.00000000000000))*(x11));
x16=((IkReal(1.00000000000000))*(x3));
x17=((IkReal(1.00000000000000))*(x2));
x18=((IkReal(0.331000000000000))*(x0));
x19=((x6)*(x9));
x20=((x12)*(x3));
x21=((x0)*(x17));
x22=((((x18)*(x2)))+(((IkReal(0.405000000000000))*(x0))));
x23=((((x1)*(x16)))+(((IkReal(-1.00000000000000))*(x21))));
x24=((((x0)*(x16)))+(((x1)*(x17))));
x25=((((IkReal(-1.00000000000000))*(x14)*(x7)*(x8)))+(((IkReal(-1.00000000000000))*(x13)*(x9))));
x26=((((IkReal(-1.00000000000000))*(x13)*(x24)*(x7)))+(((x15)*(((((IkReal(-1.00000000000000))*(x1)*(x3)))+(x21))))));
x27=((x26)*(x8));
eetrans[0]=((IkReal(0.0454160000000000))+(((x5)*(((((IkReal(-1.00000000000000))*(x22)))+(x20)))))+(((IkReal(-0.100150000000000))*(x4))));
eetrans[1]=((IkReal(0.302408000000000))+(((x18)*(x3)))+(((IkReal(0.405000000000000))*(x1)))+(((x12)*(x2))));
eetrans[2]=((IkReal(0.467052000000000))+(((IkReal(-0.100150000000000))*(x5)))+(((x4)*(((((IkReal(-1.00000000000000))*(x20)))+(x22))))));
eerot[0]=((((x5)*(((((IkReal(-1.00000000000000))*(x27)))+(((IkReal(-1.00000000000000))*(x14)*(x24)*(x9)))))))+(((x25)*(x4))));
eerot[1]=((((x19)*(x23)))+(((IkReal(-1.00000000000000))*(x8)*(((((IkReal(-1.00000000000000))*(x15)*(x24)))+(((x13)*(x23)*(x7))))))));
eerot[2]=((((x4)*(((((x19)*(x24)))+(x27)))))+(((x25)*(x5))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j0,cj0,sj0,htj0,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij0[2], _nj0;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij0[0] = -1; _ij0[1] = -1; _nj0 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=((((IkReal(-1.00000000000000))*(r00)*(IKsin(j0))))+(((r02)*(IKcos(j0)))));
IkReal x28=IKsin(j0);
IkReal x29=IKcos(j0);
new_px=((((IkReal(-1.00000000000000))*(px)*(x28)))+(((pz)*(x29)))+(((IkReal(-0.467052000000000))*(x29)))+(((IkReal(0.0454160000000000))*(x28))));
new_r01=r01;
new_py=((IkReal(-0.302408000000000))+(py));
new_r02=((((IkReal(-1.00000000000000))*(r02)*(IKsin(j0))))+(((IkReal(-1.00000000000000))*(r00)*(IKcos(j0)))));
IkReal x30=IKsin(j0);
IkReal x31=IKcos(j0);
new_pz=((IkReal(-0.130150000000000))+(((IkReal(-1.00000000000000))*(pz)*(x30)))+(((IkReal(0.0454160000000000))*(x31)))+(((IkReal(0.467052000000000))*(x30)))+(((IkReal(-1.00000000000000))*(px)*(x31))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal Noneeval[1];
IkReal x32=((IkReal(3.02114803625378))*(sj1));
IkReal x33=((IkReal(3.02114803625378))*(cj1));
Noneeval[0]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x32)))+(((px)*(sj2)*(x32)))+(((IkReal(1.22356495468278))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x33)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x33))));
if( IKabs(Noneeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((py)*(py)*(py)*(py))+((((px)*(px))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((((px)*(px))*((py)*(py))))+((px)*(px)*(px)*(px)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[1];
evalcond[0]=IKabs(py);
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=px;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[1];
evalcond[0]=IKabs(px);
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
cj2array[0]=IkReal(-0.817283950617284);
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
evalcond[0]=((IkReal(0.274486000000000))+(((IkReal(0.268110000000000))*(IKcos(j2))))+(pp)+(((IkReal(0.0600000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x34=(r01)*(r01);
IkReal x35=(r00)*(r00);
IkReal x36=((IkReal(0.405000000000000))*(pp));
IkReal x37=((IkReal(0.108584550000000))*(cj2));
IkReal x38=((IkReal(0.0243000000000000))*(pz));
IkReal x39=((sj2)*(x34));
IkReal x40=((sj2)*(x35));
gconst2=IKsign(((((x36)*(x40)))+(((x37)*(x39)))+(((x36)*(x39)))+(((x37)*(x40)))+(((x38)*(x39)))+(((x38)*(x40)))+(((IkReal(0.111166830000000))*(x40)))+(((IkReal(0.111166830000000))*(x39)))));
IkReal x41=(r00)*(r00);
IkReal x42=(r01)*(r01);
IkReal x43=((IkReal(16.6666666666667))*(pp));
IkReal x44=((IkReal(4.46850000000000))*(cj2));
IkReal x45=((sj2)*(x41));
IkReal x46=((sj2)*(x42));
dummyeval[0]=((((pz)*(x45)))+(((pz)*(x46)))+(((x43)*(x45)))+(((x43)*(x46)))+(((IkReal(4.57476666666667))*(x45)))+(((IkReal(4.57476666666667))*(x46)))+(((x44)*(x46)))+(((x44)*(x45))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=IkReal(0.736000000000000);
evalcond[2]=((IkReal(0.542596000000000))+(pp)+(((IkReal(0.0600000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(pz)*(r01)))+(((IkReal(-0.0300000000000000))*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(0.0300000000000000))*(r00)))+(((pz)*(r00))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x47=IKatan2(((((IkReal(-1.00000000000000))*(pz)*(r01)))+(((IkReal(-0.0300000000000000))*(r01)))), ((((IkReal(0.0300000000000000))*(r00)))+(((pz)*(r00)))));
j1array[0]=((IkReal(-1.00000000000000))*(x47));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x47))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x48=IKsin(j1);
IkReal x49=IKcos(j1);
IkReal x50=((IkReal(0.0600000000000000))*(pz));
IkReal x51=((pz)*(r02));
IkReal x52=((IkReal(1.00000000000000))*(pp));
IkReal x53=((r01)*(x48));
IkReal x54=((r00)*(x49));
evalcond[0]=((((IkReal(-0.0441600000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x52)*(x53)))+(((IkReal(-1.00000000000000))*(x52)*(x54)))+(((IkReal(-1.47200000000000))*(x51)))+(((IkReal(-1.00000000000000))*(x50)*(x53)))+(((IkReal(-1.00000000000000))*(x50)*(x54)))+(((IkReal(0.540796000000000))*(x53)))+(((IkReal(0.540796000000000))*(x54))));
evalcond[1]=((x51)+(((IkReal(0.0300000000000000))*(r02)))+(((IkReal(-0.736000000000000))*(x54)))+(((IkReal(-0.736000000000000))*(x53))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x55=IKatan2(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz)))), ((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x55));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x55))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x56=IKcos(j3);
IkReal x57=IKsin(j3);
IkReal x58=((IkReal(1.00000000000000))*(px));
IkReal x59=((sj2)*(x56));
IkReal x60=((cj2)*(x56));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(sj1)*(x59)))+(((IkReal(0.405000000000000))*(x59)))+(((cj1)*(py)*(x60)))+(((IkReal(-1.00000000000000))*(cj1)*(x58)*(x59)))+(((IkReal(-1.00000000000000))*(sj1)*(x58)*(x60)))+(((IkReal(-1.00000000000000))*(pz)*(x57)))+(((IkReal(-0.0300000000000000))*(x57))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x61=((cj3)*(r01));
IkReal x62=((cj3)*(r00));
cj5array[0]=((((sj1)*(sj2)*(x61)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x61)))+(((r02)*(sj3)))+(((cj2)*(sj1)*(x62)))+(((cj1)*(sj2)*(x62))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x63=((cj1)*(r00));
IkReal x64=((r01)*(sj1));
IkReal x65=((cj2)*(sj3));
IkReal x66=((cj1)*(r01));
IkReal x67=((IkReal(1.00000000000000))*(sj2)*(sj3));
IkReal x68=((IkReal(1.00000000000000))*(r00)*(sj1));
if( IKabs(((gconst3)*(((((sj2)*(x66)))+(((IkReal(-1.00000000000000))*(sj2)*(x68)))+(((cj2)*(x64)))+(((cj2)*(x63))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x64)*(x67)))+(((IkReal(-1.00000000000000))*(x65)*(x68)))+(((IkReal(-1.00000000000000))*(x63)*(x67)))+(((x65)*(x66))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((sj2)*(x66)))+(((IkReal(-1.00000000000000))*(sj2)*(x68)))+(((cj2)*(x64)))+(((cj2)*(x63)))))), ((gconst3)*(((((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x64)*(x67)))+(((IkReal(-1.00000000000000))*(x65)*(x68)))+(((IkReal(-1.00000000000000))*(x63)*(x67)))+(((x65)*(x66)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x69=((cj1)*(sj2));
IkReal x70=((IkReal(1.00000000000000))*(r01));
IkReal x71=((r00)*(sj1));
IkReal x72=((cj2)*(sj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x70)))+(((sj2)*(x71)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(x69)*(x70))));
evalcond[1]=((((r01)*(sj1)*(sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(cj1)*(x70)*(x72)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(cj3)*(r02)))+(((r00)*(sj3)*(x69)))+(((x71)*(x72))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=IkReal(-0.0740000000000000);
evalcond[2]=((IkReal(0.00637600000000000))+(pp)+(((IkReal(0.0600000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
if( IKabs(((((IkReal(0.0300000000000000))*(r01)))+(((pz)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(pz)*(r00)))+(((IkReal(-0.0300000000000000))*(r00))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x73=IKatan2(((((IkReal(0.0300000000000000))*(r01)))+(((pz)*(r01)))), ((((IkReal(-1.00000000000000))*(pz)*(r00)))+(((IkReal(-0.0300000000000000))*(r00)))));
j1array[0]=((IkReal(-1.00000000000000))*(x73));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x73))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x74=IKcos(j1);
IkReal x75=IKsin(j1);
IkReal x76=((IkReal(0.0600000000000000))*(pz));
IkReal x77=((pz)*(r02));
IkReal x78=((r01)*(x75));
IkReal x79=((r00)*(x74));
evalcond[0]=((((IkReal(-0.00457600000000000))*(x79)))+(((IkReal(-0.00457600000000000))*(x78)))+(((IkReal(0.148000000000000))*(x77)))+(((x76)*(x79)))+(((x76)*(x78)))+(((IkReal(0.00444000000000000))*(r02)))+(((pp)*(x79)))+(((pp)*(x78))));
evalcond[1]=((((IkReal(-0.0740000000000000))*(x79)))+(((IkReal(-0.0740000000000000))*(x78)))+(x77)+(((IkReal(0.0300000000000000))*(r02))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x80=IKatan2(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz)))), ((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x80));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x80))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x81=IKcos(j3);
IkReal x82=IKsin(j3);
IkReal x83=((IkReal(1.00000000000000))*(px));
IkReal x84=((sj2)*(x81));
IkReal x85=((cj2)*(x81));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj1)*(x83)*(x85)))+(((cj1)*(py)*(x85)))+(((IkReal(-1.00000000000000))*(pz)*(x82)))+(((IkReal(-1.00000000000000))*(cj1)*(x83)*(x84)))+(((IkReal(-0.0300000000000000))*(x82)))+(((IkReal(0.405000000000000))*(x84)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x84))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x86=((cj3)*(r01));
IkReal x87=((cj3)*(r00));
cj5array[0]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x86)))+(((sj1)*(sj2)*(x86)))+(((r02)*(sj3)))+(((cj2)*(sj1)*(x87)))+(((cj1)*(sj2)*(x87))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x88=((cj1)*(r00));
IkReal x89=((r01)*(sj1));
IkReal x90=((cj2)*(sj3));
IkReal x91=((cj1)*(r01));
IkReal x92=((IkReal(1.00000000000000))*(sj2)*(sj3));
IkReal x93=((IkReal(1.00000000000000))*(r00)*(sj1));
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(sj2)*(x93)))+(((cj2)*(x88)))+(((cj2)*(x89)))+(((sj2)*(x91))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x90)*(x93)))+(((x90)*(x91)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x88)*(x92)))+(((IkReal(-1.00000000000000))*(x89)*(x92))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(sj2)*(x93)))+(((cj2)*(x88)))+(((cj2)*(x89)))+(((sj2)*(x91)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x90)*(x93)))+(((x90)*(x91)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x88)*(x92)))+(((IkReal(-1.00000000000000))*(x89)*(x92)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x94=((cj1)*(sj2));
IkReal x95=((IkReal(1.00000000000000))*(r01));
IkReal x96=((r00)*(sj1));
IkReal x97=((cj2)*(sj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(x94)*(x95)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((sj2)*(x96)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x95))));
evalcond[1]=((((x96)*(x97)))+(((r01)*(sj1)*(sj2)*(sj3)))+(((sj5)*(IKcos(j4))))+(((r00)*(sj3)*(x94)))+(((IkReal(-1.00000000000000))*(cj3)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x95)*(x97))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x98=(pz)*(pz);
IkReal x99=(cj2)*(cj2);
IkReal x100=((pz)*(r02));
IkReal x101=((pp)*(r00));
IkReal x102=((r01)*(sj2));
IkReal x103=((IkReal(0.0300000000000000))*(r02));
IkReal x104=((cj2)*(r01));
IkReal x105=((IkReal(0.00804330000000000))*(r02));
IkReal x106=((r00)*(sj2));
IkReal x107=((IkReal(0.00823458000000000))*(r02));
IkReal x108=((IkReal(0.268110000000000))*(cj2));
IkReal x109=((r00)*(x99));
IkReal x110=((cj2)*(r00)*(r02));
IkReal x111=((IkReal(0.0600000000000000))*(x98));
IkReal x112=((r02)*(x106));
IkReal x113=((r01)*(x99));
if( IKabs(((gconst2)*(((((x110)*(x111)))+(((cj2)*(x102)*(x105)))+(((IkReal(0.268110000000000))*(x100)*(x109)))+(((x100)*(x102)*(x108)))+(((cj2)*(x100)*(x101)))+(((x102)*(x107)))+(((pp)*(x100)*(x102)))+(((cj2)*(x101)*(x103)))+(((cj2)*(r00)*(x107)))+(((IkReal(0.276286000000000))*(cj2)*(r00)*(x100)))+(((pp)*(x102)*(x103)))+(((IkReal(0.276286000000000))*(x100)*(x102)))+(((x105)*(x109)))+(((r02)*(x102)*(x111))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x104)*(x107)))+(((IkReal(-0.276286000000000))*(x100)*(x104)))+(((x106)*(x107)))+(((x111)*(x112)))+(((IkReal(-1.00000000000000))*(pp)*(x103)*(x104)))+(((x100)*(x106)*(x108)))+(((sj2)*(x100)*(x101)))+(((IkReal(-0.268110000000000))*(x100)*(x113)))+(((sj2)*(x101)*(x103)))+(((IkReal(-1.00000000000000))*(pp)*(x100)*(x104)))+(((IkReal(0.276286000000000))*(x100)*(x106)))+(((cj2)*(x105)*(x106)))+(((IkReal(-1.00000000000000))*(x105)*(x113)))+(((IkReal(-1.00000000000000))*(r02)*(x104)*(x111))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst2)*(((((x110)*(x111)))+(((cj2)*(x102)*(x105)))+(((IkReal(0.268110000000000))*(x100)*(x109)))+(((x100)*(x102)*(x108)))+(((cj2)*(x100)*(x101)))+(((x102)*(x107)))+(((pp)*(x100)*(x102)))+(((cj2)*(x101)*(x103)))+(((cj2)*(r00)*(x107)))+(((IkReal(0.276286000000000))*(cj2)*(r00)*(x100)))+(((pp)*(x102)*(x103)))+(((IkReal(0.276286000000000))*(x100)*(x102)))+(((x105)*(x109)))+(((r02)*(x102)*(x111)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x104)*(x107)))+(((IkReal(-0.276286000000000))*(x100)*(x104)))+(((x106)*(x107)))+(((x111)*(x112)))+(((IkReal(-1.00000000000000))*(pp)*(x103)*(x104)))+(((x100)*(x106)*(x108)))+(((sj2)*(x100)*(x101)))+(((IkReal(-0.268110000000000))*(x100)*(x113)))+(((sj2)*(x101)*(x103)))+(((IkReal(-1.00000000000000))*(pp)*(x100)*(x104)))+(((IkReal(0.276286000000000))*(x100)*(x106)))+(((cj2)*(x105)*(x106)))+(((IkReal(-1.00000000000000))*(x105)*(x113)))+(((IkReal(-1.00000000000000))*(r02)*(x104)*(x111)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[3];
IkReal x114=IKsin(j1);
IkReal x115=IKcos(j1);
IkReal x116=((cj2)*(r02));
IkReal x117=((IkReal(0.0300000000000000))*(r00));
IkReal x118=((IkReal(0.331000000000000))*(cj2));
IkReal x119=((IkReal(1.00000000000000))*(pp));
IkReal x120=((IkReal(0.272686000000000))*(cj2));
IkReal x121=((pz)*(r02));
IkReal x122=((IkReal(0.0600000000000000))*(pz));
IkReal x123=((pz)*(r00));
IkReal x124=((sj2)*(x115));
IkReal x125=((r01)*(x114));
IkReal x126=((r00)*(x115));
IkReal x127=((cj2)*(x114));
IkReal x128=((r00)*(sj2)*(x114));
IkReal x129=((cj2)*(r01)*(x115));
evalcond[0]=((((x117)*(x127)))+(((x117)*(x124)))+(((IkReal(-0.0300000000000000))*(x129)))+(((IkReal(0.405000000000000))*(r02)*(sj2)))+(((x123)*(x127)))+(((x123)*(x124)))+(((IkReal(-1.00000000000000))*(pz)*(x129)))+(((pz)*(sj2)*(x125)))+(((IkReal(0.0300000000000000))*(sj2)*(x125))));
evalcond[1]=((((IkReal(-0.662000000000000))*(x121)))+(((IkReal(-1.00000000000000))*(r01)*(x119)*(x124)))+(((IkReal(-0.0553640000000000))*(r01)*(x124)))+(((IkReal(0.0553640000000000))*(x128)))+(((IkReal(-0.0243000000000000))*(x116)))+(((IkReal(-0.0198600000000000))*(r02)))+(((x122)*(x128)))+(((x120)*(x125)))+(((x120)*(x126)))+(((pp)*(x128)))+(((IkReal(-1.00000000000000))*(r01)*(x122)*(x124)))+(((IkReal(-1.00000000000000))*(cj2)*(x122)*(x125)))+(((IkReal(-1.00000000000000))*(cj2)*(x122)*(x126)))+(((IkReal(0.268110000000000))*(x125)))+(((IkReal(0.268110000000000))*(x126)))+(((IkReal(-1.00000000000000))*(cj2)*(x119)*(x125)))+(((IkReal(-1.00000000000000))*(cj2)*(x119)*(x126)))+(((IkReal(-0.810000000000000))*(pz)*(x116))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x118)*(x125)))+(((IkReal(-1.00000000000000))*(x118)*(x126)))+(((IkReal(-0.405000000000000))*(x126)))+(((IkReal(-0.405000000000000))*(x125)))+(x121)+(((IkReal(0.331000000000000))*(x128)))+(((IkReal(0.0300000000000000))*(r02)))+(((IkReal(-0.331000000000000))*(r01)*(x124))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x130=IKatan2(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz)))), ((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x130));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x130))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x131=IKcos(j3);
IkReal x132=IKsin(j3);
IkReal x133=((IkReal(1.00000000000000))*(px));
IkReal x134=((sj2)*(x131));
IkReal x135=((cj2)*(x131));
evalcond[0]=((((IkReal(-0.0300000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(cj1)*(x133)*(x134)))+(((IkReal(-1.00000000000000))*(sj1)*(x133)*(x135)))+(((IkReal(-1.00000000000000))*(pz)*(x132)))+(((cj1)*(py)*(x135)))+(((IkReal(0.405000000000000))*(x134)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x134))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x136=((cj3)*(r01));
IkReal x137=((cj3)*(r00));
cj5array[0]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x136)))+(((sj1)*(sj2)*(x136)))+(((r02)*(sj3)))+(((cj2)*(sj1)*(x137)))+(((cj1)*(sj2)*(x137))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x138=((cj1)*(r00));
IkReal x139=((r01)*(sj1));
IkReal x140=((cj2)*(sj3));
IkReal x141=((cj1)*(r01));
IkReal x142=((IkReal(1.00000000000000))*(sj2)*(sj3));
IkReal x143=((IkReal(1.00000000000000))*(r00)*(sj1));
if( IKabs(((gconst3)*(((((cj2)*(x139)))+(((cj2)*(x138)))+(((IkReal(-1.00000000000000))*(sj2)*(x143)))+(((sj2)*(x141))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((x140)*(x141)))+(((IkReal(-1.00000000000000))*(x138)*(x142)))+(((IkReal(-1.00000000000000))*(x140)*(x143)))+(((IkReal(-1.00000000000000))*(x139)*(x142)))+(((cj3)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((cj2)*(x139)))+(((cj2)*(x138)))+(((IkReal(-1.00000000000000))*(sj2)*(x143)))+(((sj2)*(x141)))))), ((gconst3)*(((((x140)*(x141)))+(((IkReal(-1.00000000000000))*(x138)*(x142)))+(((IkReal(-1.00000000000000))*(x140)*(x143)))+(((IkReal(-1.00000000000000))*(x139)*(x142)))+(((cj3)*(r02)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x144=((cj1)*(sj2));
IkReal x145=((IkReal(1.00000000000000))*(r01));
IkReal x146=((r00)*(sj1));
IkReal x147=((cj2)*(sj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x145)))+(((IkReal(-1.00000000000000))*(x144)*(x145)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((sj2)*(x146))));
evalcond[1]=((((r01)*(sj1)*(sj2)*(sj3)))+(((r00)*(sj3)*(x144)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(cj3)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x145)*(x147)))+(((x146)*(x147))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
cj1array[0]=((IkReal(-1.23456790123457))*(((IKabs(px) != 0)?((IkReal)1/((px)*(px))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(-0.0600000000000000))*(px)*(pz)))+(((IkReal(-0.0553640000000000))*(px))))));
if( cj1array[0] >= -1-IKFAST_SINCOS_THRESH && cj1array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j1valid[0] = j1valid[1] = true;
    j1array[0] = IKacos(cj1array[0]);
    sj1array[0] = IKsin(j1array[0]);
    cj1array[1] = cj1array[0];
    j1array[1] = -j1array[0];
    sj1array[1] = -sj1array[0];
}
else if( isnan(cj1array[0]) )
{
    // probably any value will work
    j1valid[0] = true;
    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
}
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x148=((IkReal(331.000000000000))*(px)*(r01));
gconst1=IKsign(((((IkReal(-134.055000000000))*(cj1)*(r01)))+(((x148)*((cj1)*(cj1))))+(((x148)*((sj1)*(sj1))))+(((IkReal(134.055000000000))*(r00)*(sj1)))));
IkReal x149=((IkReal(2.46913580246914))*(px)*(r01));
dummyeval[0]=((((r00)*(sj1)))+(((x149)*((cj1)*(cj1))))+(((IkReal(-1.00000000000000))*(cj1)*(r01)))+(((x149)*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x150=(px)*(px);
IkReal x151=((cj1)*(px));
IkReal x152=((r00)*(sj1));
IkReal x153=((r01)*(sj1));
IkReal x154=((cj1)*(r00));
IkReal x155=((px)*(sj1));
IkReal x156=((IkReal(30.0000000000000))*(r02));
IkReal x157=((IkReal(405.000000000000))*(px));
IkReal x158=((IkReal(1000.00000000000))*(pz)*(r02));
IkReal x159=((IkReal(1000.00000000000))*(x150));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(r00)*(x157)))+(((x151)*(x156)))+(((x151)*(x158)))+(((x154)*(x159)))+(((IkReal(54.4640000000000))*(x153)))+(((IkReal(54.4640000000000))*(x154)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(x151)*(x154)))+(((IkReal(-405.000000000000))*(x151)*(x153)))+(((IkReal(-12.1500000000000))*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-405.000000000000))*(x153)*(x155)))+(((IkReal(-109.561000000000))*(x152)))+(((x152)*(x159)))+(((x155)*(x156)))+(((x155)*(x158)))+(((IkReal(-405.000000000000))*(x151)*(x152)))+(((IkReal(109.561000000000))*(cj1)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(r00)*(x157)))+(((x151)*(x156)))+(((x151)*(x158)))+(((x154)*(x159)))+(((IkReal(54.4640000000000))*(x153)))+(((IkReal(54.4640000000000))*(x154)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(x151)*(x154)))+(((IkReal(-405.000000000000))*(x151)*(x153)))+(((IkReal(-12.1500000000000))*(r02)))))), ((gconst1)*(((((IkReal(-405.000000000000))*(x153)*(x155)))+(((IkReal(-109.561000000000))*(x152)))+(((x152)*(x159)))+(((x155)*(x156)))+(((x155)*(x158)))+(((IkReal(-405.000000000000))*(x151)*(x152)))+(((IkReal(109.561000000000))*(cj1)*(r01)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x160=IKcos(j2);
IkReal x161=IKsin(j2);
IkReal x162=(px)*(px);
IkReal x163=((cj1)*(pz));
IkReal x164=((r01)*(sj1));
IkReal x165=((pp)*(sj1));
IkReal x166=((IkReal(0.0600000000000000))*(pz));
IkReal x167=((pz)*(r02));
IkReal x168=((cj1)*(px));
IkReal x169=((IkReal(0.0300000000000000))*(cj1));
IkReal x170=((px)*(r00));
IkReal x171=((cj1)*(r00));
IkReal x172=((IkReal(1.00000000000000))*(pp));
IkReal x173=((px)*(sj1));
IkReal x174=((pz)*(x161));
IkReal x175=((IkReal(0.810000000000000))*(x160));
IkReal x176=((r02)*(x161));
IkReal x177=((r00)*(x160));
IkReal x178=((r00)*(x161));
IkReal x179=((IkReal(2.00000000000000))*(x162));
IkReal x180=((IkReal(1.00000000000000))*(x160));
IkReal x181=((r02)*(x160));
IkReal x182=((r01)*(x161));
IkReal x183=((r01)*(x180));
IkReal x184=((cj1)*(x182));
evalcond[0]=((IkReal(0.331000000000000))+(((x161)*(x173)))+(((IkReal(-1.00000000000000))*(x168)*(x180)))+(((IkReal(0.405000000000000))*(x160))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x163)*(x183)))+(((IkReal(0.405000000000000))*(x176)))+(((IkReal(0.0300000000000000))*(x161)*(x164)))+(((x169)*(x178)))+(((pz)*(sj1)*(x177)))+(((IkReal(-1.00000000000000))*(x168)*(x176)))+(((IkReal(0.0300000000000000))*(sj1)*(x177)))+(((IkReal(-1.00000000000000))*(r01)*(x160)*(x169)))+(((x164)*(x174)))+(((IkReal(-1.00000000000000))*(r02)*(x173)*(x180)))+(((x163)*(x178))));
evalcond[2]=((((IkReal(-0.0600000000000000))*(x173)*(x176)))+(((IkReal(-0.0243000000000000))*(x181)))+(((x160)*(x171)*(x179)))+(((IkReal(-1.00000000000000))*(x160)*(x171)*(x172)))+(((IkReal(-0.662000000000000))*(x167)))+(((IkReal(0.810000000000000))*(px)*(x182)))+(((IkReal(-0.662000000000000))*(x170)))+(((IkReal(-0.0198600000000000))*(r02)))+(((IkReal(0.268110000000000))*(x171)))+(((IkReal(0.268110000000000))*(x164)))+(((IkReal(-1.00000000000000))*(x170)*(x175)))+(((IkReal(0.0600000000000000))*(x168)*(x181)))+(((IkReal(-0.0600000000000000))*(x163)*(x182)))+(((IkReal(-2.00000000000000))*(x161)*(x167)*(x173)))+(((x165)*(x178)))+(((IkReal(-0.0553640000000000))*(x184)))+(((IkReal(0.272686000000000))*(x160)*(x171)))+(((IkReal(0.272686000000000))*(x160)*(x164)))+(((IkReal(-0.0600000000000000))*(x163)*(x177)))+(((IkReal(-1.00000000000000))*(x160)*(x164)*(x166)))+(((IkReal(-1.00000000000000))*(x172)*(x184)))+(((IkReal(0.0553640000000000))*(sj1)*(x178)))+(((IkReal(2.00000000000000))*(px)*(x163)*(x181)))+(((IkReal(-1.00000000000000))*(x160)*(x164)*(x172)))+(((sj1)*(x166)*(x178)))+(((IkReal(-1.00000000000000))*(sj1)*(x178)*(x179)))+(((IkReal(-1.00000000000000))*(x167)*(x175))));
evalcond[3]=((IkReal(0.274486000000000))+(((IkReal(-0.662000000000000))*(x160)*(x168)))+(((IkReal(-0.810000000000000))*(x168)))+(x166)+(((IkReal(0.268110000000000))*(x160)))+(pp)+(((IkReal(0.662000000000000))*(x161)*(x173))));
evalcond[4]=((((IkReal(-0.405000000000000))*(x164)))+(((IkReal(-0.331000000000000))*(x160)*(x164)))+(((IkReal(-0.331000000000000))*(x184)))+(((IkReal(0.331000000000000))*(sj1)*(x178)))+(x170)+(x167)+(((IkReal(0.0300000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(x171)))+(((IkReal(-0.331000000000000))*(x160)*(x171))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x185=IKatan2(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz)))), ((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x185));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x185))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x186=IKcos(j3);
IkReal x187=IKsin(j3);
IkReal x188=((IkReal(1.00000000000000))*(px));
IkReal x189=((sj2)*(x186));
IkReal x190=((cj2)*(x186));
evalcond[0]=((((IkReal(0.405000000000000))*(x189)))+(((cj1)*(py)*(x190)))+(((IkReal(-0.0300000000000000))*(x187)))+(((IkReal(-1.00000000000000))*(pz)*(x187)))+(((IkReal(-1.00000000000000))*(sj1)*(x188)*(x190)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x189)))+(((IkReal(-1.00000000000000))*(cj1)*(x188)*(x189))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x191=((cj3)*(r01));
IkReal x192=((cj3)*(r00));
cj5array[0]=((((cj1)*(sj2)*(x192)))+(((sj1)*(sj2)*(x191)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x191)))+(((cj2)*(sj1)*(x192))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x193=((cj1)*(r00));
IkReal x194=((r01)*(sj1));
IkReal x195=((cj2)*(sj3));
IkReal x196=((cj1)*(r01));
IkReal x197=((IkReal(1.00000000000000))*(sj2)*(sj3));
IkReal x198=((IkReal(1.00000000000000))*(r00)*(sj1));
if( IKabs(((gconst3)*(((((sj2)*(x196)))+(((cj2)*(x194)))+(((cj2)*(x193)))+(((IkReal(-1.00000000000000))*(sj2)*(x198))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x194)*(x197)))+(((IkReal(-1.00000000000000))*(x195)*(x198)))+(((x195)*(x196)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x193)*(x197))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((sj2)*(x196)))+(((cj2)*(x194)))+(((cj2)*(x193)))+(((IkReal(-1.00000000000000))*(sj2)*(x198)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x194)*(x197)))+(((IkReal(-1.00000000000000))*(x195)*(x198)))+(((x195)*(x196)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x193)*(x197)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x199=((cj1)*(sj2));
IkReal x200=((IkReal(1.00000000000000))*(r01));
IkReal x201=((r00)*(sj1));
IkReal x202=((cj2)*(sj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x200)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(x199)*(x200)))+(((sj2)*(x201))));
evalcond[1]=((((r01)*(sj1)*(sj2)*(sj3)))+(((r00)*(sj3)*(x199)))+(((x201)*(x202)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(cj3)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x200)*(x202))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x203=(px)*(px);
if( IKabs(((IkReal(0.810000000000000))*(x203))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.810000000000000))*(px)*(py))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x204=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.810000000000000))*(x203)), ((IkReal(0.810000000000000))*(px)*(py)))));
if( (((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203)))))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(-0.0600000000000000))*(px)*(pz)))+(((IkReal(-0.0553640000000000))*(px))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(-0.0600000000000000))*(px)*(pz)))+(((IkReal(-0.0553640000000000))*(px))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x205=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x203)*((py)*(py))))+(((IkReal(0.656100000000000))*((x203)*(x203))))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(-0.0600000000000000))*(px)*(pz)))+(((IkReal(-0.0553640000000000))*(px)))))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x205)))+(((IkReal(-1.00000000000000))*(x204))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x204)))+(((IkReal(1.00000000000000))*(x205))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(0.0600000000000000))*(py)*(pz)))+(((IkReal(-0.810000000000000))*(px)*(py)*(IKcos(j1))))+(((IkReal(-0.810000000000000))*((py)*(py))*(IKsin(j1))))+(((IkReal(0.0553640000000000))*(py)))+(((pp)*(py))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x206=(sj1)*(sj1);
IkReal x207=(cj1)*(cj1);
IkReal x208=((py)*(r00));
IkReal x209=((px)*(r01));
IkReal x210=((IkReal(331.000000000000))*(x207));
IkReal x211=((IkReal(331.000000000000))*(x206));
gconst0=IKsign(((((IkReal(-134.055000000000))*(cj1)*(r01)))+(((x209)*(x210)))+(((x209)*(x211)))+(((IkReal(134.055000000000))*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(x208)*(x210)))+(((IkReal(-1.00000000000000))*(x208)*(x211)))));
IkReal x212=(sj1)*(sj1);
IkReal x213=(cj1)*(cj1);
IkReal x214=((py)*(r00));
IkReal x215=((px)*(r01));
IkReal x216=((IkReal(2.46913580246914))*(x212));
IkReal x217=((IkReal(2.46913580246914))*(x213));
dummyeval[0]=((((r00)*(sj1)))+(((x215)*(x216)))+(((x215)*(x217)))+(((IkReal(-1.00000000000000))*(x214)*(x216)))+(((IkReal(-1.00000000000000))*(x214)*(x217)))+(((IkReal(-1.00000000000000))*(cj1)*(r01))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x218=(cj1)*(cj1);
IkReal x219=(px)*(px);
IkReal x220=(py)*(py);
IkReal x221=(sj1)*(sj1);
IkReal x222=((IkReal(1000.00000000000))*(sj1));
IkReal x223=((cj1)*(r01));
IkReal x224=((IkReal(405.000000000000))*(py));
IkReal x225=((py)*(r02));
IkReal x226=((IkReal(30.0000000000000))*(sj1));
IkReal x227=((IkReal(405.000000000000))*(px));
IkReal x228=((IkReal(1000.00000000000))*(cj1));
IkReal x229=((px)*(py));
IkReal x230=((cj1)*(r00));
IkReal x231=((px)*(r02));
IkReal x232=((IkReal(30.0000000000000))*(cj1));
IkReal x233=((r01)*(x220));
IkReal x234=((r00)*(x218));
IkReal x235=((pz)*(x231));
IkReal x236=((r01)*(x221));
IkReal x237=((r00)*(x219));
if( IKabs(((gconst0)*(((((x225)*(x226)))+(((pz)*(x222)*(x225)))+(((IkReal(-1.00000000000000))*(sj1)*(x223)*(x227)))+(((IkReal(-1.00000000000000))*(x227)*(x234)))+(((IkReal(-1.00000000000000))*(x224)*(x236)))+(((x228)*(x235)))+(((x228)*(x237)))+(((IkReal(-1.00000000000000))*(r01)*(x224)))+(((x222)*(x233)))+(((IkReal(54.4640000000000))*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(x227)))+(((IkReal(1000.00000000000))*(x223)*(x229)))+(((IkReal(54.4640000000000))*(x230)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-12.1500000000000))*(r02)))+(((IkReal(-1.00000000000000))*(sj1)*(x224)*(x230)))+(((x231)*(x232)))+(((r00)*(x222)*(x229))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(x225)*(x232)))+(((IkReal(-1.00000000000000))*(x227)*(x236)))+(((r01)*(x222)*(x229)))+(((IkReal(-1000.00000000000))*(x220)*(x223)))+(((IkReal(109.561000000000))*(x223)))+(((IkReal(-1.00000000000000))*(r00)*(x228)*(x229)))+(((x226)*(x231)))+(((IkReal(-109.561000000000))*(r00)*(sj1)))+(((sj1)*(x223)*(x224)))+(((x222)*(x237)))+(((x222)*(x235)))+(((x224)*(x234)))+(((IkReal(-1.00000000000000))*(sj1)*(x227)*(x230)))+(((IkReal(-1.00000000000000))*(pz)*(x225)*(x228))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((x225)*(x226)))+(((pz)*(x222)*(x225)))+(((IkReal(-1.00000000000000))*(sj1)*(x223)*(x227)))+(((IkReal(-1.00000000000000))*(x227)*(x234)))+(((IkReal(-1.00000000000000))*(x224)*(x236)))+(((x228)*(x235)))+(((x228)*(x237)))+(((IkReal(-1.00000000000000))*(r01)*(x224)))+(((x222)*(x233)))+(((IkReal(54.4640000000000))*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(x227)))+(((IkReal(1000.00000000000))*(x223)*(x229)))+(((IkReal(54.4640000000000))*(x230)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-12.1500000000000))*(r02)))+(((IkReal(-1.00000000000000))*(sj1)*(x224)*(x230)))+(((x231)*(x232)))+(((r00)*(x222)*(x229)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(x225)*(x232)))+(((IkReal(-1.00000000000000))*(x227)*(x236)))+(((r01)*(x222)*(x229)))+(((IkReal(-1000.00000000000))*(x220)*(x223)))+(((IkReal(109.561000000000))*(x223)))+(((IkReal(-1.00000000000000))*(r00)*(x228)*(x229)))+(((x226)*(x231)))+(((IkReal(-109.561000000000))*(r00)*(sj1)))+(((sj1)*(x223)*(x224)))+(((x222)*(x237)))+(((x222)*(x235)))+(((x224)*(x234)))+(((IkReal(-1.00000000000000))*(sj1)*(x227)*(x230)))+(((IkReal(-1.00000000000000))*(pz)*(x225)*(x228)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x238=IKsin(j2);
IkReal x239=IKcos(j2);
IkReal x240=(py)*(py);
IkReal x241=(px)*(px);
IkReal x242=((cj1)*(pz));
IkReal x243=((r01)*(sj1));
IkReal x244=((IkReal(0.0600000000000000))*(r02));
IkReal x245=((pp)*(sj1));
IkReal x246=((IkReal(2.00000000000000))*(px));
IkReal x247=((IkReal(1.00000000000000))*(cj1));
IkReal x248=((IkReal(0.0600000000000000))*(pz));
IkReal x249=((cj1)*(r01));
IkReal x250=((IkReal(0.810000000000000))*(r01));
IkReal x251=((cj1)*(px));
IkReal x252=((pz)*(r02));
IkReal x253=((px)*(r00));
IkReal x254=((IkReal(0.662000000000000))*(sj1));
IkReal x255=((cj1)*(r00));
IkReal x256=((py)*(r01));
IkReal x257=((r02)*(sj1));
IkReal x258=((py)*(x239));
IkReal x259=((py)*(x238));
IkReal x260=((pz)*(x238));
IkReal x261=((IkReal(2.00000000000000))*(x240));
IkReal x262=((r02)*(x238));
IkReal x263=((r00)*(x239));
IkReal x264=((r00)*(x238));
IkReal x265=((IkReal(2.00000000000000))*(x241));
IkReal x266=((px)*(x239));
IkReal x267=((r01)*(x238));
IkReal x268=((px)*(x238));
IkReal x269=((r02)*(x239));
IkReal x270=((IkReal(1.00000000000000))*(r01)*(x239));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(x258)))+(((IkReal(0.405000000000000))*(x239)))+(((IkReal(-1.00000000000000))*(x247)*(x259)))+(((IkReal(-1.00000000000000))*(x247)*(x266)))+(((sj1)*(x268))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x242)*(x270)))+(((IkReal(0.0300000000000000))*(x238)*(x243)))+(((pz)*(sj1)*(x263)))+(((IkReal(0.0300000000000000))*(x238)*(x255)))+(((IkReal(0.405000000000000))*(x262)))+(((IkReal(-0.0300000000000000))*(x239)*(x249)))+(((IkReal(0.0300000000000000))*(sj1)*(x263)))+(((IkReal(-1.00000000000000))*(x257)*(x259)))+(((x242)*(x264)))+(((cj1)*(r02)*(x258)))+(((IkReal(-1.00000000000000))*(px)*(x247)*(x262)))+(((x243)*(x260)))+(((IkReal(-1.00000000000000))*(x257)*(x266))));
evalcond[2]=((((IkReal(-0.0553640000000000))*(x238)*(x249)))+(((IkReal(-0.0243000000000000))*(x269)))+(((IkReal(-1.00000000000000))*(x250)*(x258)))+(((IkReal(0.272686000000000))*(x239)*(x255)))+(((IkReal(-0.810000000000000))*(x239)*(x252)))+(((IkReal(-0.810000000000000))*(x239)*(x253)))+(((x239)*(x255)*(x265)))+(((x238)*(x249)*(x261)))+(((IkReal(-1.00000000000000))*(sj1)*(x264)*(x265)))+(((IkReal(-0.0198600000000000))*(r02)))+(((x246)*(x255)*(x259)))+(((IkReal(-1.00000000000000))*(pp)*(x247)*(x267)))+(((IkReal(-1.00000000000000))*(pp)*(x247)*(x263)))+(((IkReal(0.268110000000000))*(x255)))+(((IkReal(-0.662000000000000))*(x256)))+(((IkReal(-0.662000000000000))*(x252)))+(((IkReal(-0.662000000000000))*(x253)))+(((IkReal(-1.00000000000000))*(sj1)*(x238)*(x246)*(x252)))+(((x239)*(x243)*(x261)))+(((x239)*(x244)*(x251)))+(((IkReal(-0.810000000000000))*(r00)*(x259)))+(((x246)*(x249)*(x258)))+(((IkReal(-0.0600000000000000))*(x242)*(x263)))+(((IkReal(-0.0600000000000000))*(x242)*(x267)))+(((IkReal(0.0553640000000000))*(sj1)*(x264)))+(((sj1)*(x248)*(x264)))+(((IkReal(2.00000000000000))*(r02)*(x242)*(x259)))+(((IkReal(-1.00000000000000))*(pp)*(x239)*(x243)))+(((IkReal(0.272686000000000))*(x239)*(x243)))+(((IkReal(0.268110000000000))*(x243)))+(((cj1)*(x244)*(x259)))+(((IkReal(2.00000000000000))*(sj1)*(x252)*(x258)))+(((x250)*(x268)))+(((IkReal(-1.00000000000000))*(x239)*(x243)*(x248)))+(((sj1)*(x244)*(x258)))+(((r00)*(sj1)*(x246)*(x258)))+(((x245)*(x264)))+(((IkReal(-1.00000000000000))*(x243)*(x246)*(x259)))+(((x242)*(x246)*(x269)))+(((IkReal(-1.00000000000000))*(sj1)*(x244)*(x268))));
evalcond[3]=((IkReal(0.274486000000000))+(((IkReal(0.268110000000000))*(x239)))+(((x254)*(x268)))+(((IkReal(-1.00000000000000))*(x254)*(x258)))+(((IkReal(-0.662000000000000))*(cj1)*(x259)))+(x248)+(((IkReal(-0.662000000000000))*(x239)*(x251)))+(pp)+(((IkReal(-0.810000000000000))*(py)*(sj1)))+(((IkReal(-0.810000000000000))*(x251))));
evalcond[4]=((((IkReal(-0.405000000000000))*(x243)))+(x256)+(x252)+(x253)+(((IkReal(0.0300000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(x255)))+(((IkReal(-0.331000000000000))*(x239)*(x255)))+(((IkReal(-0.331000000000000))*(x238)*(x249)))+(((IkReal(0.331000000000000))*(sj1)*(x264)))+(((IkReal(-0.331000000000000))*(x239)*(x243))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x271=IKatan2(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz)))), ((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x271));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x271))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x272=IKcos(j3);
IkReal x273=IKsin(j3);
IkReal x274=((IkReal(1.00000000000000))*(px));
IkReal x275=((sj2)*(x272));
IkReal x276=((cj2)*(x272));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj1)*(x274)*(x275)))+(((IkReal(0.405000000000000))*(x275)))+(((IkReal(-1.00000000000000))*(sj1)*(x274)*(x276)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x275)))+(((cj1)*(py)*(x276)))+(((IkReal(-1.00000000000000))*(pz)*(x273)))+(((IkReal(-0.0300000000000000))*(x273))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x277=((cj3)*(r01));
IkReal x278=((cj3)*(r00));
cj5array[0]=((((cj2)*(sj1)*(x278)))+(((cj1)*(sj2)*(x278)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x277)))+(((sj1)*(sj2)*(x277))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x279=((cj1)*(r00));
IkReal x280=((r01)*(sj1));
IkReal x281=((cj2)*(sj3));
IkReal x282=((cj1)*(r01));
IkReal x283=((IkReal(1.00000000000000))*(sj2)*(sj3));
IkReal x284=((IkReal(1.00000000000000))*(r00)*(sj1));
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(sj2)*(x284)))+(((sj2)*(x282)))+(((cj2)*(x280)))+(((cj2)*(x279))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x280)*(x283)))+(((IkReal(-1.00000000000000))*(x281)*(x284)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x279)*(x283)))+(((x281)*(x282))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(sj2)*(x284)))+(((sj2)*(x282)))+(((cj2)*(x280)))+(((cj2)*(x279)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x280)*(x283)))+(((IkReal(-1.00000000000000))*(x281)*(x284)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x279)*(x283)))+(((x281)*(x282)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x285=((cj1)*(sj2));
IkReal x286=((IkReal(1.00000000000000))*(r01));
IkReal x287=((r00)*(sj1));
IkReal x288=((cj2)*(sj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x286)))+(((sj2)*(x287)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(x285)*(x286))));
evalcond[1]=((((r01)*(sj1)*(sj2)*(sj3)))+(((IkReal(-1.00000000000000))*(cj1)*(x286)*(x288)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(cj3)*(r02)))+(((x287)*(x288)))+(((r00)*(sj3)*(x285))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x289=(py)*(py);
if( IKabs(((IkReal(-0.810000000000000))*(px)*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.810000000000000))*(x289))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x290=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-0.810000000000000))*(px)*(py)), ((IkReal(-0.810000000000000))*(x289)))));
if( (((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289))))))))):(IkReal)1.0e30))*(((((IkReal(0.0600000000000000))*(py)*(pz)))+(((IkReal(0.0553640000000000))*(py)))+(((pp)*(py))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289))))))))):(IkReal)1.0e30))*(((((IkReal(0.0600000000000000))*(py)*(pz)))+(((IkReal(0.0553640000000000))*(py)))+(((pp)*(py))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x291=IKasin(((((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x289)*((px)*(px))))+(((IkReal(0.656100000000000))*((x289)*(x289))))))))):(IkReal)1.0e30))*(((((IkReal(0.0600000000000000))*(py)*(pz)))+(((IkReal(0.0553640000000000))*(py)))+(((pp)*(py)))))));
j1array[0]=((((IkReal(-1.00000000000000))*(x291)))+(((IkReal(-1.00000000000000))*(x290))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x290)))+(x291));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(0.810000000000000))*((px)*(px))*(IKcos(j1))))+(((IkReal(-1.00000000000000))*(pp)*(px)))+(((IkReal(-0.0600000000000000))*(px)*(pz)))+(((IkReal(-0.0553640000000000))*(px)))+(((IkReal(0.810000000000000))*(px)*(py)*(IKsin(j1)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x292=(sj1)*(sj1);
IkReal x293=(cj1)*(cj1);
IkReal x294=((py)*(r00));
IkReal x295=((px)*(r01));
IkReal x296=((IkReal(331.000000000000))*(x293));
IkReal x297=((IkReal(331.000000000000))*(x292));
gconst0=IKsign(((((IkReal(-134.055000000000))*(cj1)*(r01)))+(((IkReal(134.055000000000))*(r00)*(sj1)))+(((x295)*(x297)))+(((x295)*(x296)))+(((IkReal(-1.00000000000000))*(x294)*(x297)))+(((IkReal(-1.00000000000000))*(x294)*(x296)))));
IkReal x298=(sj1)*(sj1);
IkReal x299=(cj1)*(cj1);
IkReal x300=((py)*(r00));
IkReal x301=((px)*(r01));
IkReal x302=((IkReal(2.46913580246914))*(x298));
IkReal x303=((IkReal(2.46913580246914))*(x299));
dummyeval[0]=((((x301)*(x302)))+(((x301)*(x303)))+(((r00)*(sj1)))+(((IkReal(-1.00000000000000))*(x300)*(x302)))+(((IkReal(-1.00000000000000))*(x300)*(x303)))+(((IkReal(-1.00000000000000))*(cj1)*(r01))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x304=(cj1)*(cj1);
IkReal x305=(px)*(px);
IkReal x306=(py)*(py);
IkReal x307=(sj1)*(sj1);
IkReal x308=((IkReal(1000.00000000000))*(sj1));
IkReal x309=((cj1)*(r01));
IkReal x310=((IkReal(405.000000000000))*(py));
IkReal x311=((py)*(r02));
IkReal x312=((IkReal(30.0000000000000))*(sj1));
IkReal x313=((IkReal(405.000000000000))*(px));
IkReal x314=((IkReal(1000.00000000000))*(cj1));
IkReal x315=((px)*(py));
IkReal x316=((cj1)*(r00));
IkReal x317=((px)*(r02));
IkReal x318=((IkReal(30.0000000000000))*(cj1));
IkReal x319=((r01)*(x306));
IkReal x320=((r00)*(x304));
IkReal x321=((pz)*(x317));
IkReal x322=((r01)*(x307));
IkReal x323=((r00)*(x305));
if( IKabs(((gconst0)*(((((r00)*(x308)*(x315)))+(((x317)*(x318)))+(((IkReal(54.4640000000000))*(x316)))+(((IkReal(-1.00000000000000))*(r01)*(x310)))+(((IkReal(-1.00000000000000))*(r00)*(x313)))+(((x308)*(x319)))+(((IkReal(-1.00000000000000))*(sj1)*(x309)*(x313)))+(((x314)*(x321)))+(((x314)*(x323)))+(((IkReal(54.4640000000000))*(r01)*(sj1)))+(((x311)*(x312)))+(((IkReal(-1.00000000000000))*(sj1)*(x310)*(x316)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-12.1500000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x310)*(x322)))+(((IkReal(-1.00000000000000))*(x313)*(x320)))+(((pz)*(x308)*(x311)))+(((IkReal(1000.00000000000))*(x309)*(x315))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((r01)*(x308)*(x315)))+(((x312)*(x317)))+(((x308)*(x323)))+(((x308)*(x321)))+(((x310)*(x320)))+(((IkReal(-1.00000000000000))*(r00)*(x314)*(x315)))+(((IkReal(-1.00000000000000))*(pz)*(x311)*(x314)))+(((IkReal(-1000.00000000000))*(x306)*(x309)))+(((IkReal(-109.561000000000))*(r00)*(sj1)))+(((sj1)*(x309)*(x310)))+(((IkReal(-1.00000000000000))*(sj1)*(x313)*(x316)))+(((IkReal(109.561000000000))*(x309)))+(((IkReal(-1.00000000000000))*(x313)*(x322)))+(((IkReal(-1.00000000000000))*(x311)*(x318))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((r00)*(x308)*(x315)))+(((x317)*(x318)))+(((IkReal(54.4640000000000))*(x316)))+(((IkReal(-1.00000000000000))*(r01)*(x310)))+(((IkReal(-1.00000000000000))*(r00)*(x313)))+(((x308)*(x319)))+(((IkReal(-1.00000000000000))*(sj1)*(x309)*(x313)))+(((x314)*(x321)))+(((x314)*(x323)))+(((IkReal(54.4640000000000))*(r01)*(sj1)))+(((x311)*(x312)))+(((IkReal(-1.00000000000000))*(sj1)*(x310)*(x316)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-12.1500000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x310)*(x322)))+(((IkReal(-1.00000000000000))*(x313)*(x320)))+(((pz)*(x308)*(x311)))+(((IkReal(1000.00000000000))*(x309)*(x315)))))), ((gconst0)*(((((r01)*(x308)*(x315)))+(((x312)*(x317)))+(((x308)*(x323)))+(((x308)*(x321)))+(((x310)*(x320)))+(((IkReal(-1.00000000000000))*(r00)*(x314)*(x315)))+(((IkReal(-1.00000000000000))*(pz)*(x311)*(x314)))+(((IkReal(-1000.00000000000))*(x306)*(x309)))+(((IkReal(-109.561000000000))*(r00)*(sj1)))+(((sj1)*(x309)*(x310)))+(((IkReal(-1.00000000000000))*(sj1)*(x313)*(x316)))+(((IkReal(109.561000000000))*(x309)))+(((IkReal(-1.00000000000000))*(x313)*(x322)))+(((IkReal(-1.00000000000000))*(x311)*(x318)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x324=IKsin(j2);
IkReal x325=IKcos(j2);
IkReal x326=(py)*(py);
IkReal x327=(px)*(px);
IkReal x328=((cj1)*(pz));
IkReal x329=((r01)*(sj1));
IkReal x330=((IkReal(0.0600000000000000))*(r02));
IkReal x331=((pp)*(sj1));
IkReal x332=((IkReal(2.00000000000000))*(px));
IkReal x333=((IkReal(1.00000000000000))*(cj1));
IkReal x334=((IkReal(0.0600000000000000))*(pz));
IkReal x335=((cj1)*(r01));
IkReal x336=((IkReal(0.810000000000000))*(r01));
IkReal x337=((cj1)*(px));
IkReal x338=((pz)*(r02));
IkReal x339=((px)*(r00));
IkReal x340=((IkReal(0.662000000000000))*(sj1));
IkReal x341=((cj1)*(r00));
IkReal x342=((py)*(r01));
IkReal x343=((r02)*(sj1));
IkReal x344=((py)*(x325));
IkReal x345=((py)*(x324));
IkReal x346=((pz)*(x324));
IkReal x347=((IkReal(2.00000000000000))*(x326));
IkReal x348=((r02)*(x324));
IkReal x349=((r00)*(x325));
IkReal x350=((r00)*(x324));
IkReal x351=((IkReal(2.00000000000000))*(x327));
IkReal x352=((px)*(x325));
IkReal x353=((r01)*(x324));
IkReal x354=((px)*(x324));
IkReal x355=((r02)*(x325));
IkReal x356=((IkReal(1.00000000000000))*(r01)*(x325));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(x344)))+(((IkReal(-1.00000000000000))*(x333)*(x352)))+(((IkReal(0.405000000000000))*(x325)))+(((sj1)*(x354)))+(((IkReal(-1.00000000000000))*(x333)*(x345))));
evalcond[1]=((((x328)*(x350)))+(((IkReal(-0.0300000000000000))*(x325)*(x335)))+(((IkReal(0.0300000000000000))*(sj1)*(x349)))+(((IkReal(0.0300000000000000))*(x324)*(x341)))+(((pz)*(sj1)*(x349)))+(((x329)*(x346)))+(((IkReal(0.0300000000000000))*(x324)*(x329)))+(((IkReal(-1.00000000000000))*(px)*(x333)*(x348)))+(((IkReal(-1.00000000000000))*(x343)*(x345)))+(((IkReal(-1.00000000000000))*(x343)*(x352)))+(((cj1)*(r02)*(x344)))+(((IkReal(0.405000000000000))*(x348)))+(((IkReal(-1.00000000000000))*(x328)*(x356))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj1)*(x350)*(x351)))+(((IkReal(2.00000000000000))*(r02)*(x328)*(x345)))+(((IkReal(-0.0600000000000000))*(x328)*(x353)))+(((x328)*(x332)*(x355)))+(((IkReal(0.268110000000000))*(x329)))+(((IkReal(-0.0243000000000000))*(x355)))+(((IkReal(-0.0600000000000000))*(x328)*(x349)))+(((IkReal(0.0553640000000000))*(sj1)*(x350)))+(((IkReal(-1.00000000000000))*(x336)*(x344)))+(((IkReal(-0.662000000000000))*(x339)))+(((IkReal(-0.662000000000000))*(x338)))+(((IkReal(-0.0553640000000000))*(x324)*(x335)))+(((IkReal(-1.00000000000000))*(sj1)*(x330)*(x354)))+(((x325)*(x329)*(x347)))+(((x331)*(x350)))+(((x332)*(x335)*(x344)))+(((IkReal(-0.0198600000000000))*(r02)))+(((x324)*(x335)*(x347)))+(((IkReal(-1.00000000000000))*(pp)*(x325)*(x329)))+(((IkReal(0.272686000000000))*(x325)*(x329)))+(((x336)*(x354)))+(((IkReal(-0.810000000000000))*(r00)*(x345)))+(((x325)*(x341)*(x351)))+(((IkReal(0.268110000000000))*(x341)))+(((IkReal(-1.00000000000000))*(pp)*(x333)*(x353)))+(((IkReal(0.272686000000000))*(x325)*(x341)))+(((IkReal(-1.00000000000000))*(pp)*(x333)*(x349)))+(((IkReal(-1.00000000000000))*(x325)*(x329)*(x334)))+(((IkReal(-0.810000000000000))*(x325)*(x339)))+(((IkReal(-0.810000000000000))*(x325)*(x338)))+(((cj1)*(x330)*(x345)))+(((IkReal(-1.00000000000000))*(sj1)*(x324)*(x332)*(x338)))+(((IkReal(-1.00000000000000))*(x329)*(x332)*(x345)))+(((r00)*(sj1)*(x332)*(x344)))+(((x332)*(x341)*(x345)))+(((sj1)*(x334)*(x350)))+(((sj1)*(x330)*(x344)))+(((x325)*(x330)*(x337)))+(((IkReal(2.00000000000000))*(sj1)*(x338)*(x344)))+(((IkReal(-0.662000000000000))*(x342))));
evalcond[3]=((IkReal(0.274486000000000))+(((IkReal(0.268110000000000))*(x325)))+(((IkReal(-0.662000000000000))*(cj1)*(x345)))+(x334)+(((IkReal(-0.662000000000000))*(x325)*(x337)))+(((x340)*(x354)))+(pp)+(((IkReal(-0.810000000000000))*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(x340)*(x344)))+(((IkReal(-0.810000000000000))*(x337))));
evalcond[4]=((((IkReal(-0.405000000000000))*(x329)))+(((IkReal(-0.331000000000000))*(x324)*(x335)))+(((IkReal(-0.331000000000000))*(x325)*(x329)))+(x339)+(x338)+(x342)+(((IkReal(-0.331000000000000))*(x325)*(x341)))+(((IkReal(0.331000000000000))*(sj1)*(x350)))+(((IkReal(0.0300000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(x341))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( IKabs(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x357=IKatan2(((IkReal(-0.0300000000000000))+(((IkReal(-1.00000000000000))*(pz)))), ((((IkReal(-0.405000000000000))*(sj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x357));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x357))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x358=IKcos(j3);
IkReal x359=IKsin(j3);
IkReal x360=((IkReal(1.00000000000000))*(px));
IkReal x361=((sj2)*(x358));
IkReal x362=((cj2)*(x358));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj1)*(x360)*(x362)))+(((IkReal(-1.00000000000000))*(pz)*(x359)))+(((IkReal(-1.00000000000000))*(cj1)*(x360)*(x361)))+(((IkReal(-0.0300000000000000))*(x359)))+(((cj1)*(py)*(x362)))+(((IkReal(0.405000000000000))*(x361)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x361))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x363=((cj3)*(r01));
IkReal x364=((cj3)*(r00));
cj5array[0]=((((cj2)*(sj1)*(x364)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x363)))+(((r02)*(sj3)))+(((sj1)*(sj2)*(x363)))+(((cj1)*(sj2)*(x364))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x365=((cj1)*(r00));
IkReal x366=((r01)*(sj1));
IkReal x367=((cj2)*(sj3));
IkReal x368=((cj1)*(r01));
IkReal x369=((IkReal(1.00000000000000))*(sj2)*(sj3));
IkReal x370=((IkReal(1.00000000000000))*(r00)*(sj1));
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(sj2)*(x370)))+(((sj2)*(x368)))+(((cj2)*(x365)))+(((cj2)*(x366))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x367)*(x370)))+(((IkReal(-1.00000000000000))*(x366)*(x369)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x365)*(x369)))+(((x367)*(x368))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(sj2)*(x370)))+(((sj2)*(x368)))+(((cj2)*(x365)))+(((cj2)*(x366)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x367)*(x370)))+(((IkReal(-1.00000000000000))*(x366)*(x369)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x365)*(x369)))+(((x367)*(x368)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x371=((cj1)*(sj2));
IkReal x372=((IkReal(1.00000000000000))*(r01));
IkReal x373=((r00)*(sj1));
IkReal x374=((cj2)*(sj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x372)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((sj2)*(x373))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x372)*(x374)))+(((r01)*(sj1)*(sj2)*(sj3)))+(((r00)*(sj3)*(x371)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(cj3)*(r02)))+(((x373)*(x374))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
continue;

}

}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
