/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:25:39.341673
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {

// necessary for local/global translation3d
eerot[0] = eerot[4] = eerot[8] = 0;
IkReal r00 = 0, r11 = 0, r22 = 0;
IkReal x0,x1,x2,x3,x4,x5,x6,x7;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[1]);
x3=IKsin(j[2]);
x4=((IkReal(0.0232000000000000))*(x1));
x5=((IkReal(1.00000000000000))*(x3));
x6=((r22)*(x1));
x7=((IkReal(0.0980000000000000))*(x3));
eetrans[0]=((IkReal(-0.0670700000000000))+(((r00)*(x2)))+(((x0)*(x6)))+(((x0)*(x7)))+(((x0)*(x4)))+(((IkReal(0.0680000000000000))*(x0)))+(((IkReal(-0.0300000000000000))*(x2)))+(((IkReal(-1.00000000000000))*(r11)*(x0)*(x5))));
eetrans[1]=((((IkReal(-1.00000000000000))*(r11)*(x2)*(x5)))+(((IkReal(-1.00000000000000))*(r00)*(x0)))+(((IkReal(0.0680000000000000))*(x2)))+(((IkReal(0.0300000000000000))*(x0)))+(((x2)*(x4)))+(((x2)*(x7)))+(((x2)*(x6))));
eetrans[2]=((IkReal(1.12112500000000))+(((IkReal(0.0980000000000000))*(x1)))+(((IkReal(-1.00000000000000))*(r11)*(x1)))+(j[0])+(((IkReal(-0.0232000000000000))*(x3)))+(((IkReal(-1.00000000000000))*(r22)*(x5))));
}

IKFAST_API int GetNumFreeParameters() { return 0; }
IKFAST_API int* GetFreeParameters() { return NULL; }
IKFAST_API int GetNumJoints() { return 3; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x3600000a; }

class IKSolver {
public:
IkReal j12,cj12,sj12,htj12,j13,cj13,sj13,htj13,j14,cj14,sj14,htj14,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij12[2], _nj12,_ij13[2], _nj13,_ij14[2], _nj14;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j12=numeric_limits<IkReal>::quiet_NaN(); _ij12[0] = -1; _ij12[1] = -1; _nj12 = -1; j13=numeric_limits<IkReal>::quiet_NaN(); _ij13[0] = -1; _ij13[1] = -1; _nj13 = -1; j14=numeric_limits<IkReal>::quiet_NaN(); _ij14[0] = -1; _ij14[1] = -1; _nj14 = -1; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
r00 = eerot[0];
r11 = eerot[4];
r22 = eerot[8];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_px=((IkReal(0.0670700000000000))+(px));
new_py=py;
new_pz=((IkReal(-1.12112500000000))+(pz));
px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j13array[2], cj13array[2], sj13array[2];
bool j13valid[2]={false};
_nj13 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x8=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(0.0300000000000000))+(((IkReal(-1.00000000000000))*(r00))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(0.0300000000000000))+(((IkReal(-1.00000000000000))*(r00))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x9=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(0.0300000000000000))+(((IkReal(-1.00000000000000))*(r00)))))));
j13array[0]=((((IkReal(-1.00000000000000))*(x9)))+(((IkReal(-1.00000000000000))*(x8))));
sj13array[0]=IKsin(j13array[0]);
cj13array[0]=IKcos(j13array[0]);
j13array[1]=((IkReal(3.14159265358979))+(x9)+(((IkReal(-1.00000000000000))*(x8))));
sj13array[1]=IKsin(j13array[1]);
cj13array[1]=IKcos(j13array[1]);
if( j13array[0] > IKPI )
{
    j13array[0]-=IK2PI;
}
else if( j13array[0] < -IKPI )
{    j13array[0]+=IK2PI;
}
j13valid[0] = true;
if( j13array[1] > IKPI )
{
    j13array[1]-=IK2PI;
}
else if( j13array[1] < -IKPI )
{    j13array[1]+=IK2PI;
}
j13valid[1] = true;
for(int ij13 = 0; ij13 < 2; ++ij13)
{
if( !j13valid[ij13] )
{
    continue;
}
_ij13[0] = ij13; _ij13[1] = -1;
for(int iij13 = ij13+1; iij13 < 2; ++iij13)
{
if( j13valid[iij13] && IKabs(cj13array[ij13]-cj13array[iij13]) < IKFAST_SOLUTION_THRESH && IKabs(sj13array[ij13]-sj13array[iij13]) < IKFAST_SOLUTION_THRESH )
{
    j13valid[iij13]=false; _ij13[1] = iij13; break; 
}
}
j13 = j13array[ij13]; cj13 = cj13array[ij13]; sj13 = sj13array[ij13];

{
IkReal j12array[2], cj12array[2], sj12array[2];
bool j12valid[2]={false};
_nj12 = 2;
if( (((IkReal(0.00641824000000000))+(((IkReal(0.136000000000000))*(cj13)*(px)))+(((IkReal(-0.196000000000000))*(r11)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+((pz)*(pz))+(((IkReal(-0.0600000000000000))*(r00)))+(((IkReal(0.0464000000000000))*(r22)))+(((IkReal(0.136000000000000))*(py)*(sj13))))) < (IkReal)-0.00001 )
    continue;
IkReal x10=IKsqrt(((IkReal(0.00641824000000000))+(((IkReal(0.136000000000000))*(cj13)*(px)))+(((IkReal(-0.196000000000000))*(r11)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+((pz)*(pz))+(((IkReal(-0.0600000000000000))*(r00)))+(((IkReal(0.0464000000000000))*(r22)))+(((IkReal(0.136000000000000))*(py)*(sj13)))));
j12array[0]=((pz)+(x10));
sj12array[0]=IKsin(j12array[0]);
cj12array[0]=IKcos(j12array[0]);
j12array[1]=((((IkReal(-1.00000000000000))*(x10)))+(pz));
sj12array[1]=IKsin(j12array[1]);
cj12array[1]=IKcos(j12array[1]);
j12valid[0] = true;
j12valid[1] = true;
for(int ij12 = 0; ij12 < 2; ++ij12)
{
if( !j12valid[ij12] )
{
    continue;
}
_ij12[0] = ij12; _ij12[1] = -1;
for(int iij12 = ij12+1; iij12 < 2; ++iij12)
{
if( j12valid[iij12] && IKabs(cj12array[ij12]-cj12array[iij12]) < IKFAST_SOLUTION_THRESH && IKabs(sj12array[ij12]-sj12array[iij12]) < IKFAST_SOLUTION_THRESH )
{
    j12valid[iij12]=false; _ij12[1] = iij12; break; 
}
}
j12 = j12array[ij12]; cj12 = cj12array[ij12]; sj12 = sj12array[ij12];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(-0.0101422400000000))+(((IkReal(0.196000000000000))*(r11)))+(((IkReal(-1.00000000000000))*((r11)*(r11))))+(((IkReal(-1.00000000000000))*((r22)*(r22))))+(((IkReal(-0.0464000000000000))*(r22)))));
dummyeval[0]=((IkReal(-1.00000000000000))+(((IkReal(19.3251195002287))*(r11)))+(((IkReal(-4.57492624903374))*(r22)))+(((IkReal(-98.5975484705548))*((r22)*(r22))))+(((IkReal(-98.5975484705548))*((r11)*(r11)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j14array[1], cj14array[1], sj14array[1];
bool j14valid[1]={false};
_nj14 = 1;
IkReal x11=((cj13)*(px));
IkReal x12=((IkReal(1.00000000000000))*(r22));
IkReal x13=((py)*(sj13));
if( IKabs(((gconst0)*(((IkReal(0.00666400000000000))+(((IkReal(0.0232000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(j12)*(x12)))+(((IkReal(-0.0232000000000000))*(j12)))+(((IkReal(-0.0680000000000000))*(r11)))+(((IkReal(-0.0980000000000000))*(x11)))+(((IkReal(-0.0980000000000000))*(x13)))+(((pz)*(r22)))+(((r11)*(x13)))+(((r11)*(x11))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((IkReal(0.00157760000000000))+(((IkReal(-0.0232000000000000))*(x13)))+(((IkReal(-0.0232000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(j12)*(r11)))+(((IkReal(-0.0980000000000000))*(pz)))+(((IkReal(0.0680000000000000))*(r22)))+(((IkReal(0.0980000000000000))*(j12)))+(((pz)*(r11)))+(((IkReal(-1.00000000000000))*(x11)*(x12)))+(((IkReal(-1.00000000000000))*(x12)*(x13))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j14array[0]=IKatan2(((gconst0)*(((IkReal(0.00666400000000000))+(((IkReal(0.0232000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(j12)*(x12)))+(((IkReal(-0.0232000000000000))*(j12)))+(((IkReal(-0.0680000000000000))*(r11)))+(((IkReal(-0.0980000000000000))*(x11)))+(((IkReal(-0.0980000000000000))*(x13)))+(((pz)*(r22)))+(((r11)*(x13)))+(((r11)*(x11)))))), ((gconst0)*(((IkReal(0.00157760000000000))+(((IkReal(-0.0232000000000000))*(x13)))+(((IkReal(-0.0232000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(j12)*(r11)))+(((IkReal(-0.0980000000000000))*(pz)))+(((IkReal(0.0680000000000000))*(r22)))+(((IkReal(0.0980000000000000))*(j12)))+(((pz)*(r11)))+(((IkReal(-1.00000000000000))*(x11)*(x12)))+(((IkReal(-1.00000000000000))*(x12)*(x13)))))));
sj14array[0]=IKsin(j14array[0]);
cj14array[0]=IKcos(j14array[0]);
if( j14array[0] > IKPI )
{
    j14array[0]-=IK2PI;
}
else if( j14array[0] < -IKPI )
{    j14array[0]+=IK2PI;
}
j14valid[0] = true;
for(int ij14 = 0; ij14 < 1; ++ij14)
{
if( !j14valid[ij14] )
{
    continue;
}
_ij14[0] = ij14; _ij14[1] = -1;
for(int iij14 = ij14+1; iij14 < 1; ++iij14)
{
if( j14valid[iij14] && IKabs(cj14array[ij14]-cj14array[iij14]) < IKFAST_SOLUTION_THRESH && IKabs(sj14array[ij14]-sj14array[iij14]) < IKFAST_SOLUTION_THRESH )
{
    j14valid[iij14]=false; _ij14[1] = iij14; break; 
}
}
j14 = j14array[ij14]; cj14 = cj14array[ij14]; sj14 = sj14array[ij14];
{
IkReal evalcond[5];
IkReal x14=IKsin(j14);
IkReal x15=IKcos(j14);
IkReal x16=((IkReal(1.00000000000000))*(py));
IkReal x17=((IkReal(1.00000000000000))*(cj13));
IkReal x18=((IkReal(0.0980000000000000))*(x14));
IkReal x19=((IkReal(0.0232000000000000))*(x15));
IkReal x20=((r22)*(x15));
IkReal x21=((IkReal(1.00000000000000))*(r11)*(x14));
evalcond[0]=((((IkReal(-1.00000000000000))*(j12)))+(((r22)*(x14)))+(((IkReal(0.0232000000000000))*(x14)))+(pz)+(((IkReal(-0.0980000000000000))*(x15)))+(((r11)*(x15))));
evalcond[1]=((IkReal(0.0680000000000000))+(((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(sj13)*(x16)))+(x19)+(x18)+(x20)+(((IkReal(-1.00000000000000))*(px)*(x17))));
evalcond[2]=((((cj13)*(x20)))+(((cj13)*(x18)))+(((cj13)*(x19)))+(((IkReal(-0.0300000000000000))*(sj13)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.0680000000000000))*(cj13)))+(((r00)*(sj13)))+(((IkReal(-1.00000000000000))*(r11)*(x14)*(x17))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x16)))+(((IkReal(-1.00000000000000))*(sj13)*(x21)))+(((sj13)*(x20)))+(((sj13)*(x19)))+(((sj13)*(x18)))+(((IkReal(0.0680000000000000))*(sj13)))+(((IkReal(-1.00000000000000))*(r00)*(x17)))+(((IkReal(0.0300000000000000))*(cj13))));
evalcond[4]=((IkReal(0.0156662400000000))+(((IkReal(-0.196000000000000))*(r11)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0133280000000000))*(x14)))+(((IkReal(2.00000000000000))*(j12)*(pz)))+(((IkReal(0.00315520000000000))*(x15)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0600000000000000))*(r00)))+(((IkReal(-1.00000000000000))*((j12)*(j12))))+(((IkReal(0.136000000000000))*(x20)))+(((IkReal(0.0464000000000000))*(r22)))+(((IkReal(-0.136000000000000))*(r11)*(x14))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(3);
vinfos[0].jointtype = 17;
vinfos[0].foffset = j12;
vinfos[0].indices[0] = _ij12[0];
vinfos[0].indices[1] = _ij12[1];
vinfos[0].maxsolutions = _nj12;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j13;
vinfos[1].indices[0] = _ij13[0];
vinfos[1].indices[1] = _ij13[1];
vinfos[1].maxsolutions = _nj13;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j14;
vinfos[2].indices[0] = _ij14[0];
vinfos[2].indices[1] = _ij14[1];
vinfos[2].maxsolutions = _nj14;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}

} else
{
{
IkReal j13array[2], cj13array[2], sj13array[2];
bool j13valid[2]={false};
_nj13 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x22=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(0.0300000000000000))+(((IkReal(-1.00000000000000))*(r00))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(0.0300000000000000))+(((IkReal(-1.00000000000000))*(r00))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x23=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((IkReal(0.0300000000000000))+(((IkReal(-1.00000000000000))*(r00)))))));
j13array[0]=((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x23))));
sj13array[0]=IKsin(j13array[0]);
cj13array[0]=IKcos(j13array[0]);
j13array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x22)))+(x23));
sj13array[1]=IKsin(j13array[1]);
cj13array[1]=IKcos(j13array[1]);
if( j13array[0] > IKPI )
{
    j13array[0]-=IK2PI;
}
else if( j13array[0] < -IKPI )
{    j13array[0]+=IK2PI;
}
j13valid[0] = true;
if( j13array[1] > IKPI )
{
    j13array[1]-=IK2PI;
}
else if( j13array[1] < -IKPI )
{    j13array[1]+=IK2PI;
}
j13valid[1] = true;
for(int ij13 = 0; ij13 < 2; ++ij13)
{
if( !j13valid[ij13] )
{
    continue;
}
_ij13[0] = ij13; _ij13[1] = -1;
for(int iij13 = ij13+1; iij13 < 2; ++iij13)
{
if( j13valid[iij13] && IKabs(cj13array[ij13]-cj13array[iij13]) < IKFAST_SOLUTION_THRESH && IKabs(sj13array[ij13]-sj13array[iij13]) < IKFAST_SOLUTION_THRESH )
{
    j13valid[iij13]=false; _ij13[1] = iij13; break; 
}
}
j13 = j13array[ij13]; cj13 = cj13array[ij13]; sj13 = sj13array[ij13];

{
IkReal j12array[2], cj12array[2], sj12array[2];
bool j12valid[2]={false};
_nj12 = 2;
if( (((IkReal(0.00641824000000000))+(((IkReal(0.136000000000000))*(cj13)*(px)))+(((IkReal(-0.196000000000000))*(r11)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+((pz)*(pz))+(((IkReal(-0.0600000000000000))*(r00)))+(((IkReal(0.0464000000000000))*(r22)))+(((IkReal(0.136000000000000))*(py)*(sj13))))) < (IkReal)-0.00001 )
    continue;
IkReal x24=IKsqrt(((IkReal(0.00641824000000000))+(((IkReal(0.136000000000000))*(cj13)*(px)))+(((IkReal(-0.196000000000000))*(r11)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+((pz)*(pz))+(((IkReal(-0.0600000000000000))*(r00)))+(((IkReal(0.0464000000000000))*(r22)))+(((IkReal(0.136000000000000))*(py)*(sj13)))));
j12array[0]=((pz)+(x24));
sj12array[0]=IKsin(j12array[0]);
cj12array[0]=IKcos(j12array[0]);
j12array[1]=((((IkReal(-1.00000000000000))*(x24)))+(pz));
sj12array[1]=IKsin(j12array[1]);
cj12array[1]=IKcos(j12array[1]);
j12valid[0] = true;
j12valid[1] = true;
for(int ij12 = 0; ij12 < 2; ++ij12)
{
if( !j12valid[ij12] )
{
    continue;
}
_ij12[0] = ij12; _ij12[1] = -1;
for(int iij12 = ij12+1; iij12 < 2; ++iij12)
{
if( j12valid[iij12] && IKabs(cj12array[ij12]-cj12array[iij12]) < IKFAST_SOLUTION_THRESH && IKabs(sj12array[ij12]-sj12array[iij12]) < IKFAST_SOLUTION_THRESH )
{
    j12valid[iij12]=false; _ij12[1] = iij12; break; 
}
}
j12 = j12array[ij12]; cj12 = cj12array[ij12]; sj12 = sj12array[ij12];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(-0.0101422400000000))+(((IkReal(0.196000000000000))*(r11)))+(((IkReal(-1.00000000000000))*((r11)*(r11))))+(((IkReal(-1.00000000000000))*((r22)*(r22))))+(((IkReal(-0.0464000000000000))*(r22)))));
dummyeval[0]=((IkReal(-1.00000000000000))+(((IkReal(19.3251195002287))*(r11)))+(((IkReal(-4.57492624903374))*(r22)))+(((IkReal(-98.5975484705548))*((r22)*(r22))))+(((IkReal(-98.5975484705548))*((r11)*(r11)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j14array[1], cj14array[1], sj14array[1];
bool j14valid[1]={false};
_nj14 = 1;
IkReal x25=((cj13)*(px));
IkReal x26=((IkReal(1.00000000000000))*(r22));
IkReal x27=((py)*(sj13));
if( IKabs(((gconst0)*(((IkReal(0.00666400000000000))+(((IkReal(0.0232000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(j12)*(x26)))+(((IkReal(-0.0232000000000000))*(j12)))+(((IkReal(-0.0680000000000000))*(r11)))+(((pz)*(r22)))+(((IkReal(-0.0980000000000000))*(x25)))+(((IkReal(-0.0980000000000000))*(x27)))+(((r11)*(x27)))+(((r11)*(x25))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((IkReal(0.00157760000000000))+(((IkReal(-0.0232000000000000))*(x27)))+(((IkReal(-0.0232000000000000))*(x25)))+(((IkReal(-1.00000000000000))*(j12)*(r11)))+(((IkReal(-1.00000000000000))*(x25)*(x26)))+(((IkReal(-0.0980000000000000))*(pz)))+(((IkReal(0.0680000000000000))*(r22)))+(((IkReal(-1.00000000000000))*(x26)*(x27)))+(((IkReal(0.0980000000000000))*(j12)))+(((pz)*(r11))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j14array[0]=IKatan2(((gconst0)*(((IkReal(0.00666400000000000))+(((IkReal(0.0232000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(j12)*(x26)))+(((IkReal(-0.0232000000000000))*(j12)))+(((IkReal(-0.0680000000000000))*(r11)))+(((pz)*(r22)))+(((IkReal(-0.0980000000000000))*(x25)))+(((IkReal(-0.0980000000000000))*(x27)))+(((r11)*(x27)))+(((r11)*(x25)))))), ((gconst0)*(((IkReal(0.00157760000000000))+(((IkReal(-0.0232000000000000))*(x27)))+(((IkReal(-0.0232000000000000))*(x25)))+(((IkReal(-1.00000000000000))*(j12)*(r11)))+(((IkReal(-1.00000000000000))*(x25)*(x26)))+(((IkReal(-0.0980000000000000))*(pz)))+(((IkReal(0.0680000000000000))*(r22)))+(((IkReal(-1.00000000000000))*(x26)*(x27)))+(((IkReal(0.0980000000000000))*(j12)))+(((pz)*(r11)))))));
sj14array[0]=IKsin(j14array[0]);
cj14array[0]=IKcos(j14array[0]);
if( j14array[0] > IKPI )
{
    j14array[0]-=IK2PI;
}
else if( j14array[0] < -IKPI )
{    j14array[0]+=IK2PI;
}
j14valid[0] = true;
for(int ij14 = 0; ij14 < 1; ++ij14)
{
if( !j14valid[ij14] )
{
    continue;
}
_ij14[0] = ij14; _ij14[1] = -1;
for(int iij14 = ij14+1; iij14 < 1; ++iij14)
{
if( j14valid[iij14] && IKabs(cj14array[ij14]-cj14array[iij14]) < IKFAST_SOLUTION_THRESH && IKabs(sj14array[ij14]-sj14array[iij14]) < IKFAST_SOLUTION_THRESH )
{
    j14valid[iij14]=false; _ij14[1] = iij14; break; 
}
}
j14 = j14array[ij14]; cj14 = cj14array[ij14]; sj14 = sj14array[ij14];
{
IkReal evalcond[5];
IkReal x28=IKsin(j14);
IkReal x29=IKcos(j14);
IkReal x30=((IkReal(1.00000000000000))*(py));
IkReal x31=((IkReal(1.00000000000000))*(cj13));
IkReal x32=((IkReal(0.0980000000000000))*(x28));
IkReal x33=((IkReal(0.0232000000000000))*(x29));
IkReal x34=((r22)*(x29));
IkReal x35=((IkReal(1.00000000000000))*(r11)*(x28));
evalcond[0]=((((IkReal(-1.00000000000000))*(j12)))+(((r22)*(x28)))+(((IkReal(0.0232000000000000))*(x28)))+(pz)+(((IkReal(-0.0980000000000000))*(x29)))+(((r11)*(x29))));
evalcond[1]=((IkReal(0.0680000000000000))+(((IkReal(-1.00000000000000))*(sj13)*(x30)))+(((IkReal(-1.00000000000000))*(x35)))+(x33)+(x32)+(x34)+(((IkReal(-1.00000000000000))*(px)*(x31))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r11)*(x28)*(x31)))+(((IkReal(-0.0300000000000000))*(sj13)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.0680000000000000))*(cj13)))+(((r00)*(sj13)))+(((cj13)*(x34)))+(((cj13)*(x32)))+(((cj13)*(x33))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj13)*(x35)))+(((sj13)*(x33)))+(((sj13)*(x32)))+(((sj13)*(x34)))+(((IkReal(-1.00000000000000))*(r00)*(x31)))+(((IkReal(0.0680000000000000))*(sj13)))+(((IkReal(-1.00000000000000))*(x30)))+(((IkReal(0.0300000000000000))*(cj13))));
evalcond[4]=((IkReal(0.0156662400000000))+(((IkReal(-0.196000000000000))*(r11)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0133280000000000))*(x28)))+(((IkReal(2.00000000000000))*(j12)*(pz)))+(((IkReal(0.00315520000000000))*(x29)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0600000000000000))*(r00)))+(((IkReal(-1.00000000000000))*((j12)*(j12))))+(((IkReal(0.136000000000000))*(x34)))+(((IkReal(0.0464000000000000))*(r22)))+(((IkReal(-0.136000000000000))*(r11)*(x28))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(3);
vinfos[0].jointtype = 17;
vinfos[0].foffset = j12;
vinfos[0].indices[0] = _ij12[0];
vinfos[0].indices[1] = _ij12[1];
vinfos[0].maxsolutions = _nj12;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j13;
vinfos[1].indices[0] = _ij13[0];
vinfos[1].indices[1] = _ij13[1];
vinfos[1].maxsolutions = _nj13;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j14;
vinfos[2].indices[0] = _ij14[0];
vinfos[2].indices[1] = _ij14[1];
vinfos[2].maxsolutions = _nj14;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}

}

}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "2640ae411e0c87b03f56bf289296f9d8"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
