/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:00:56.965743
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62,x63,x64,x65,x66,x67,x68,x69,x70,x71,x72,x73;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.0600000000000000))*(x5));
x15=((IkReal(0.0450000000000000))*(x1));
x16=((IkReal(1.00000000000000))*(x5));
x17=((IkReal(0.0600000000000000))*(x9));
x18=((IkReal(0.300000000000000))*(x1));
x19=((IkReal(0.0600000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x11));
x21=((IkReal(0.0450000000000000))*(x5));
x22=((IkReal(1.00000000000000))*(x1));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(0.0600000000000000))*(x8));
x25=((IkReal(1.00000000000000))*(x6));
x26=((x0)*(x4));
x27=((x0)*(x2));
x28=((x2)*(x7));
x29=((x3)*(x4));
x30=((x0)*(x7));
x31=((x2)*(x3));
x32=((x3)*(x7));
x33=((IkReal(1.00000000000000))*(x29));
x34=((IkReal(0.0450000000000000))*(x29));
x35=((IkReal(0.0450000000000000))*(x26));
x36=((x22)*(x6));
x37=((x32)*(x6));
x38=((x4)*(x7)*(x9));
x39=((x16)*(x30));
x40=((x25)*(x30));
x41=((x15)*(x27));
x42=((x16)*(x32));
x43=((x25)*(x32));
x44=((x15)*(x31));
x45=((((IkReal(-1.00000000000000))*(x33)))+(((x1)*(x27))));
x46=((x26)+(((x1)*(x31))));
x47=((x27)+(((IkReal(-1.00000000000000))*(x22)*(x29))));
x48=((((IkReal(-1.00000000000000))*(x36)))+(((x28)*(x5))));
x49=((x33)+(((IkReal(-1.00000000000000))*(x22)*(x27))));
x50=((((IkReal(-1.00000000000000))*(x31)))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x51=((((IkReal(-1.00000000000000))*(x26)))+(((IkReal(-1.00000000000000))*(x22)*(x31))));
x52=((((x1)*(x16)))+(((x25)*(x28))));
x53=((IkReal(-1.00000000000000))*(x52));
x54=((x35)+(x44));
x55=((x46)*(x6));
x56=((x11)*(x48));
x57=((x45)*(x6));
x58=((x5)*(x51));
x59=((x50)*(x9));
x60=((x53)*(x8));
x61=((((IkReal(-1.00000000000000))*(x39)))+(x57));
x62=((((IkReal(-1.00000000000000))*(x42)))+(x55));
x63=((((IkReal(-1.00000000000000))*(x40)))+(((x49)*(x5))));
x64=((((IkReal(-1.00000000000000))*(x43)))+(x58));
x65=((((x4)*(x7)*(x8)))+(((x52)*(x9))));
x66=((x38)+(x60));
x67=((x61)*(x8));
x68=((x11)*(x64));
x69=((((x47)*(x9)))+(((x62)*(x8))));
x70=((((x47)*(x8)))+(((x9)*(((((IkReal(-1.00000000000000))*(x25)*(x46)))+(x42))))));
x71=((x59)+(x67));
x72=((x12)*(x69));
x73=((((x50)*(x8)))+(((x9)*(((x39)+(((IkReal(-1.00000000000000))*(x57))))))));
eerot[0]=((((x10)*(x73)))+(((x13)*(((((x11)*(x63)))+(((x12)*(x71))))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x63)))+(((IkReal(-1.00000000000000))*(x23)*(x71)))))))+(((x13)*(x73))));
eerot[2]=((((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x49)))+(x40)))))+(((x11)*(x71))));
eetrans[0]=((IkReal(0.220000000000000))+(((IkReal(-1.00000000000000))*(x34)))+(((x12)*(((((x19)*(x30)))+(((IkReal(-1.00000000000000))*(x14)*(x49)))))))+(((x11)*(((((x17)*(x50)))+(((x24)*(x61)))))))+(((x21)*(x30)))+(((IkReal(0.300000000000000))*(x30)*(x6)))+(((IkReal(0.550000000000000))*(x30)))+(x41)+(((x5)*(((((x18)*(x27)))+(((IkReal(-0.300000000000000))*(x29)))))))+(((x6)*(((((IkReal(-1.00000000000000))*(x41)))+(x34))))));
eerot[3]=((((x10)*(x70)))+(((x13)*(((x72)+(x68))))));
eerot[4]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x64)))+(((IkReal(-1.00000000000000))*(x23)*(x69)))))))+(((x13)*(x70))));
eerot[5]=((((x11)*(x69)))+(((x12)*(((x43)+(((IkReal(-1.00000000000000))*(x16)*(x51))))))));
eetrans[1]=((IkReal(0.140000000000000))+(((x5)*(((((x18)*(x31)))+(((IkReal(0.300000000000000))*(x26)))))))+(((IkReal(-1.00000000000000))*(x54)*(x6)))+(((x21)*(x32)))+(((IkReal(0.550000000000000))*(x32)))+(x54)+(((x12)*(((((x19)*(x32)))+(((IkReal(-1.00000000000000))*(x14)*(x51)))))))+(((x11)*(((((x17)*(x47)))+(((x24)*(x62)))))))+(((IkReal(0.300000000000000))*(x37))));
eerot[6]=((((x13)*(((((x12)*(x66)))+(x56)))))+(((x10)*(x65))));
eerot[7]=((((x13)*(x65)))+(((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x48)))+(((IkReal(-1.00000000000000))*(x23)*(x66))))))));
eerot[8]=((((x11)*(x66)))+(((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x28)))+(x36))))));
IkReal x74=((IkReal(0.0450000000000000))*(x28));
eetrans[2]=((IkReal(0.346000000000000))+(((x18)*(x6)))+(((x11)*(((((x24)*(x53)))+(((x17)*(x4)*(x7)))))))+(((IkReal(-0.300000000000000))*(x28)*(x5)))+(((IkReal(-1.00000000000000))*(x74)))+(((x12)*(((((x1)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x28)))))))+(((x6)*(x74)))+(((IkReal(0.550000000000000))*(x1)))+(((x15)*(x5))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {5}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j6,cj6,sj6,htj6,j5,cj5,sj5,htj5,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij6[2], _nj6,_ij5[2], _nj5;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1;  _ij5[0] = -1; _ij5[1] = -1; _nj5 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j5=pfree[0]; cj5=cos(pfree[0]); sj5=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((IkReal(-0.220000000000000))+(((IkReal(-0.0600000000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((IkReal(-0.140000000000000))+(py)+(((IkReal(-0.0600000000000000))*(r12))));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.346000000000000))+(((IkReal(-0.0600000000000000))*(r22)))+(pz));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x75=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j3array[0]=((IkReal(1.80132261653346))+(((IkReal(1.00000000000000))*(x75))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(4.94291527012326))+(((IkReal(-1.00000000000000))*(x75))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
IkReal x76=(sj5)*(sj5);
dummyeval[0]=((((x76)*((npx)*(npx))))+(((x76)*((npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x77=(sj5)*(sj5);
dummyeval[0]=((((x77)*((npy)*(npy))))+(((x77)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(sj5)))+(((IkReal(-12.2222222222222))*(sj3)*(sj5)))+(((cj3)*(sj5))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x78=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj3))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=x78;
evalcond[3]=x78;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x79=((cj3)*(npy));
IkReal x80=((IkReal(24.4444444444444))*(sj3));
IkReal x81=((IkReal(1.00000000000000))*(npy));
dummyeval[0]=((((x79)*(x80)))+(((IkReal(-149.382716049383))*(npy)*((sj3)*(sj3))))+(((IkReal(-1.00000000000000))*(cj3)*(x79)))+(((IkReal(493.827160493827))*(npy)*((npx)*(npx))))+(((IkReal(-1.00000000000000))*(npy)*(x80)))+(((IkReal(2.00000000000000))*(x79)))+(((IkReal(493.827160493827))*((npy)*(npy)*(npy))))+(((IkReal(-1.00000000000000))*(x81))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x82=(sj3)*(sj3);
IkReal x83=(cj3)*(cj3);
IkReal x84=(npy)*(npy);
IkReal x85=(npx)*(npx)*(npx);
IkReal x86=((IkReal(0.00810000000000000))*(npy));
IkReal x87=((IkReal(0.0162000000000000))*(npx));
IkReal x88=((IkReal(0.0324000000000000))*(npx));
IkReal x89=((npx)*(x84));
IkReal x90=((IkReal(0.396000000000000))*(npx)*(sj3));
IkReal x91=((npx)*(x82));
IkReal x92=((IkReal(0.792000000000000))*(npx)*(sj3));
IkReal x93=((IkReal(0.198000000000000))*(npy)*(sj3));
IkReal x94=((x86)+(x93)+(((x83)*(x86)))+(((IkReal(1.21000000000000))*(npy)*(x82))));
IkReal x95=((((cj3)*(x93)))+(((IkReal(4.00000000000000))*(npy)*((npx)*(npx))))+(((IkReal(0.0162000000000000))*(cj3)*(npy)))+(((IkReal(4.00000000000000))*((npy)*(npy)*(npy)))));
IkReal x96=((x95)+(((IkReal(-1.00000000000000))*(x94))));
IkReal x97=((((IkReal(2.42000000000000))*(x91)))+(x87)+(x90)+(((IkReal(-8.00000000000000))*(x85)))+(((IkReal(-8.00000000000000))*(x89)))+(((x83)*(x87)))+(((IkReal(-1.00000000000000))*(cj3)*(x88)))+(((IkReal(-1.00000000000000))*(cj3)*(x90))));
IkReal x98=((x94)+(((IkReal(-1.00000000000000))*(x95))));
op[0]=x96;
op[1]=x97;
op[2]=x96;
op[3]=((((IkReal(-16.0000000000000))*(x89)))+(((IkReal(-16.0000000000000))*(x85)))+(((IkReal(4.84000000000000))*(x91)))+(((IkReal(-0.0648000000000000))*(cj3)*(npx)))+(x88)+(x92)+(((x83)*(x88)))+(((IkReal(-1.00000000000000))*(cj3)*(x92))));
op[4]=x98;
op[5]=x97;
op[6]=x98;
polyroots6(op,zeror,numroots);
IkReal j6array[6], cj6array[6], sj6array[6], tempj6array[1];
int numsolutions = 0;
for(int ij6 = 0; ij6 < numroots; ++ij6)
{
IkReal htj6 = zeror[ij6];
tempj6array[0]=((IkReal(2.00000000000000))*(atan(htj6)));
for(int kj6 = 0; kj6 < 1; ++kj6)
{
j6array[numsolutions] = tempj6array[kj6];
if( j6array[numsolutions] > IKPI )
{
    j6array[numsolutions]-=IK2PI;
}
else if( j6array[numsolutions] < -IKPI )
{
    j6array[numsolutions]+=IK2PI;
}
sj6array[numsolutions] = IKsin(j6array[numsolutions]);
cj6array[numsolutions] = IKcos(j6array[numsolutions]);
numsolutions++;
}
}
bool j6valid[6]={true,true,true,true,true,true};
_nj6 = 6;
for(int ij6 = 0; ij6 < numsolutions; ++ij6)
    {
if( !j6valid[ij6] )
{
    continue;
}
    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
htj6 = IKtan(j6/2);

_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < numsolutions; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x99=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x100=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)*(x100)))+(((IkReal(-1.00000000000000))*(cj6)*(npy))));
evalcond[3]=x99;
evalcond[4]=x99;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj6)*(x100)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x101=IKatan2(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x101));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x101))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x102=IKcos(j4);
IkReal x103=IKsin(j4);
IkReal x104=((IkReal(1.00000000000000))*(x102));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npx)*(x103)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x104)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x104)))+(((npy)*(sj6)*(x103))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x178=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x179=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x179))));
evalcond[3]=x178;
evalcond[4]=x178;
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x179))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x180=IKatan2(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x180));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x180))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x181=IKcos(j4);
IkReal x182=IKsin(j4);
IkReal x183=((IkReal(1.00000000000000))*(x181));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npx)*(x182)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x183)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x183)))+(((npy)*(sj6)*(x182))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x184=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x184;
evalcond[3]=x184;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x185=((IkReal(414405822679.079))*(npx));
IkReal x186=((IkReal(414405822679.079))*(npy));
if( IKabs(((((cj6)*(x186)))+(((sj6)*(x185))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj6)*(x186)))+(((IkReal(-1.00000000000000))*(cj6)*(x185))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj6)*(x186)))+(((sj6)*(x185)))))+IKsqr(((((sj6)*(x186)))+(((IkReal(-1.00000000000000))*(cj6)*(x185)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((cj6)*(x186)))+(((sj6)*(x185)))), ((((sj6)*(x186)))+(((IkReal(-1.00000000000000))*(cj6)*(x185)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x187=IKsin(j4);
IkReal x188=IKcos(j4);
IkReal x189=((IkReal(1.00000000000000))*(cj6));
IkReal x190=((npy)*(sj6));
IkReal x191=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x192=((npx)*(x187));
evalcond[0]=((((IkReal(-1.00000000000000))*(x191)))+(((IkReal(2.41309350707268e-12))*(x187)))+(((IkReal(-1.00000000000000))*(npy)*(x189))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x188)*(x191)))+(((x187)*(x190)))+(((IkReal(-1.00000000000000))*(x189)*(x192)))+(((IkReal(-1.00000000000000))*(npy)*(x188)*(x189))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((x188)*(x190)))+(((sj6)*(x192)))+(((IkReal(-1.00000000000000))*(npx)*(x188)*(x189)))+(((cj6)*(npy)*(x187))));
evalcond[3]=((((IkReal(-2.41309350707268e-12))*(x188)))+(x190)+(((IkReal(-1.00000000000000))*(npx)*(x189))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst7)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst7)*(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst7)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst7)*(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x193=IKsin(j4);
IkReal x194=IKcos(j4);
IkReal x195=((IkReal(0.550000000000000))*(sj3));
IkReal x196=((IkReal(0.0450000000000000))*(cj3));
IkReal x197=((npy)*(sj6));
IkReal x198=((IkReal(1.00000000000000))*(npx));
IkReal x199=((IkReal(1.00000000000000))*(cj6)*(npy));
IkReal x200=((cj6)*(x193));
evalcond[0]=((((x193)*(x195)))+(((IkReal(-1.00000000000000))*(x199)))+(((IkReal(-1.00000000000000))*(sj6)*(x198)))+(((IkReal(0.0450000000000000))*(x193)))+(((IkReal(-1.00000000000000))*(x193)*(x196))));
evalcond[1]=((((x193)*(x197)))+(((IkReal(-1.00000000000000))*(x194)*(x199)))+(((IkReal(-1.00000000000000))*(x198)*(x200)))+(((IkReal(-1.00000000000000))*(sj6)*(x194)*(x198))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(cj6)*(x194)*(x198)))+(((IkReal(-1.00000000000000))*(x195)))+(x196)+(((x194)*(x197)))+(((npy)*(x200)))+(((npx)*(sj6)*(x193))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x194)*(x195)))+(x197)+(((x194)*(x196)))+(((IkReal(-0.0450000000000000))*(x194)))+(((IkReal(-1.00000000000000))*(cj6)*(x198))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
IkReal x201=((IkReal(0.0450000000000000))*(sj3));
IkReal x202=((IkReal(0.550000000000000))*(cj3));
IkReal x203=((x202)+(x201));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x203));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x203))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x204=((cj3)*(npy));
IkReal x205=((IkReal(24.4444444444444))*(sj3));
IkReal x206=((IkReal(1.00000000000000))*(npy));
dummyeval[0]=((((IkReal(2.00000000000000))*(x204)))+(((IkReal(-1.00000000000000))*(npy)*(x205)))+(((IkReal(-149.382716049383))*(npy)*((sj3)*(sj3))))+(((IkReal(-1.00000000000000))*(x206)))+(((x204)*(x205)))+(((IkReal(493.827160493827))*(npy)*((npx)*(npx))))+(((IkReal(493.827160493827))*((npy)*(npy)*(npy))))+(((IkReal(-1.00000000000000))*(cj3)*(x204))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x207=(sj3)*(sj3);
IkReal x208=(cj3)*(cj3);
IkReal x209=(npy)*(npy);
IkReal x210=(npx)*(npx)*(npx);
IkReal x211=((IkReal(0.00810000000000000))*(npy));
IkReal x212=((IkReal(0.0162000000000000))*(npx));
IkReal x213=((IkReal(0.0324000000000000))*(npx));
IkReal x214=((npx)*(x209));
IkReal x215=((IkReal(0.396000000000000))*(npx)*(sj3));
IkReal x216=((npx)*(x207));
IkReal x217=((IkReal(0.792000000000000))*(npx)*(sj3));
IkReal x218=((IkReal(0.198000000000000))*(npy)*(sj3));
IkReal x219=((((x208)*(x211)))+(((IkReal(1.21000000000000))*(npy)*(x207)))+(x211)+(x218));
IkReal x220=((((IkReal(4.00000000000000))*(npy)*((npx)*(npx))))+(((IkReal(0.0162000000000000))*(cj3)*(npy)))+(((cj3)*(x218)))+(((IkReal(4.00000000000000))*((npy)*(npy)*(npy)))));
IkReal x221=((x220)+(((IkReal(-1.00000000000000))*(x219))));
IkReal x222=((((IkReal(2.42000000000000))*(x216)))+(((IkReal(-1.00000000000000))*(cj3)*(x213)))+(((IkReal(-1.00000000000000))*(cj3)*(x215)))+(((x208)*(x212)))+(x212)+(x215)+(((IkReal(-8.00000000000000))*(x214)))+(((IkReal(-8.00000000000000))*(x210))));
IkReal x223=((x219)+(((IkReal(-1.00000000000000))*(x220))));
op[0]=x221;
op[1]=x222;
op[2]=x221;
op[3]=((((IkReal(-1.00000000000000))*(cj3)*(x217)))+(((x208)*(x213)))+(((IkReal(-16.0000000000000))*(x214)))+(((IkReal(-16.0000000000000))*(x210)))+(((IkReal(-0.0648000000000000))*(cj3)*(npx)))+(x213)+(x217)+(((IkReal(4.84000000000000))*(x216))));
op[4]=x223;
op[5]=x222;
op[6]=x223;
polyroots6(op,zeror,numroots);
IkReal j6array[6], cj6array[6], sj6array[6], tempj6array[1];
int numsolutions = 0;
for(int ij6 = 0; ij6 < numroots; ++ij6)
{
IkReal htj6 = zeror[ij6];
tempj6array[0]=((IkReal(2.00000000000000))*(atan(htj6)));
for(int kj6 = 0; kj6 < 1; ++kj6)
{
j6array[numsolutions] = tempj6array[kj6];
if( j6array[numsolutions] > IKPI )
{
    j6array[numsolutions]-=IK2PI;
}
else if( j6array[numsolutions] < -IKPI )
{
    j6array[numsolutions]+=IK2PI;
}
sj6array[numsolutions] = IKsin(j6array[numsolutions]);
cj6array[numsolutions] = IKcos(j6array[numsolutions]);
numsolutions++;
}
}
bool j6valid[6]={true,true,true,true,true,true};
_nj6 = 6;
for(int ij6 = 0; ij6 < numsolutions; ++ij6)
    {
if( !j6valid[ij6] )
{
    continue;
}
    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
htj6 = IKtan(j6/2);

_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < numsolutions; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst8;
gconst8=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x224=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x224))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x224))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x225=IKatan2(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x225));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x225))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x226=IKsin(j4);
IkReal x227=IKcos(j4);
IkReal x228=((IkReal(1.00000000000000))*(x227));
evalcond[0]=((((cj6)*(npx)*(x226)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)*(x226)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x228)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x228))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x229=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x229))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x229))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x230=IKatan2(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x230));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x230))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x231=IKsin(j4);
IkReal x232=IKcos(j4);
IkReal x233=((IkReal(1.00000000000000))*(x232));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(sj6)*(x231)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x233)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x233)))+(((cj6)*(npx)*(x231))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(npz));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x234=((IkReal(414405822679.079))*(npx));
IkReal x235=((IkReal(414405822679.079))*(npy));
if( IKabs(((((cj6)*(x235)))+(((sj6)*(x234))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(x234)))+(((IkReal(-1.00000000000000))*(sj6)*(x235))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj6)*(x235)))+(((sj6)*(x234)))))+IKsqr(((((cj6)*(x234)))+(((IkReal(-1.00000000000000))*(sj6)*(x235)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((cj6)*(x235)))+(((sj6)*(x234)))), ((((cj6)*(x234)))+(((IkReal(-1.00000000000000))*(sj6)*(x235)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x236=IKsin(j4);
IkReal x237=IKcos(j4);
IkReal x238=((cj6)*(npx));
IkReal x239=((cj6)*(npy));
IkReal x240=((IkReal(1.00000000000000))*(npy)*(sj6));
IkReal x241=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x236)))+(((IkReal(-1.00000000000000))*(x241)))+(((IkReal(-1.00000000000000))*(x239))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x237)*(x241)))+(((IkReal(-1.00000000000000))*(x237)*(x239)))+(((IkReal(-1.00000000000000))*(x236)*(x240)))+(((x236)*(x238))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((x237)*(x238)))+(((IkReal(-1.00000000000000))*(x237)*(x240)))+(((npx)*(sj6)*(x236)))+(((x236)*(x239))));
evalcond[3]=((((npy)*(sj6)))+(((IkReal(2.41309350707268e-12))*(x237)))+(((IkReal(-1.00000000000000))*(x238))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst8)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst8)*(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst8)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst8)*(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x242=IKsin(j4);
IkReal x243=IKcos(j4);
IkReal x244=((IkReal(0.550000000000000))*(sj3));
IkReal x245=((IkReal(0.0450000000000000))*(cj3));
IkReal x246=((cj6)*(npx));
IkReal x247=((cj6)*(npy));
IkReal x248=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x249=((sj6)*(x242));
IkReal x250=((IkReal(1.00000000000000))*(npy)*(x243));
evalcond[0]=((((IkReal(-1.00000000000000))*(x242)*(x245)))+(((IkReal(-1.00000000000000))*(x247)))+(((x242)*(x244)))+(((IkReal(-1.00000000000000))*(x248)))+(((IkReal(0.0450000000000000))*(x242))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npy)*(x249)))+(((IkReal(-1.00000000000000))*(x243)*(x248)))+(((x242)*(x246)))+(((IkReal(-1.00000000000000))*(x243)*(x247))));
evalcond[2]=((IkReal(-0.0450000000000000))+(x245)+(((x242)*(x247)))+(((IkReal(-1.00000000000000))*(x244)))+(((IkReal(-1.00000000000000))*(sj6)*(x250)))+(((x243)*(x246)))+(((npx)*(x249))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x246)))+(((IkReal(-1.00000000000000))*(x243)*(x245)))+(((npy)*(sj6)))+(((x243)*(x244)))+(((IkReal(0.0450000000000000))*(x243))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.850000000000000))*(cj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst9;
gconst9=IKsign(((((IkReal(20.0000000000000))*((npy)*(npy))))+(((IkReal(20.0000000000000))*((npx)*(npx))))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x251=((gconst9)*(sj5));
if( IKabs(((IkReal(17.0000000000000))*(npy)*(x251))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-17.0000000000000))*(npx)*(x251))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((IkReal(17.0000000000000))*(npy)*(x251)), ((IkReal(-17.0000000000000))*(npx)*(x251)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x252=IKcos(j6);
IkReal x253=IKsin(j6);
IkReal x254=((IkReal(1.00000000000000))*(npx));
IkReal x255=((npy)*(x253));
evalcond[0]=((((IkReal(-1.00000000000000))*(x253)*(x254)))+(((IkReal(-1.00000000000000))*(npy)*(x252))));
evalcond[1]=((IkReal(0.850000000000000))+(((npx)*(sj5)*(x252)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(sj5)*(x255))));
evalcond[2]=((((IkReal(-0.850000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(x252)*(x254)))+(x255));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x256=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x256));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x256))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x257=IKsin(j4);
IkReal x258=IKcos(j4);
IkReal x259=((IkReal(1.00000000000000))*(x258));
IkReal x260=((IkReal(1.00000000000000))*(x257));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)*(x260)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x259)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x260)))+(((cj5)*(npy)*(sj6)*(x257)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x259))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x261=((IkReal(0.0450000000000000))*(sj5));
cj4array[0]=((((IKabs(((((IkReal(-0.550000000000000))*(sj3)*(sj5)))+(((IkReal(-1.00000000000000))*(x261)))+(((cj3)*(x261))))) != 0)?((IkReal)1/(((((IkReal(-0.550000000000000))*(sj3)*(sj5)))+(((IkReal(-1.00000000000000))*(x261)))+(((cj3)*(x261)))))):(IkReal)1.0e30))*(((((IkReal(1.00000000000000))*(npz)))+(((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(((IkReal(-0.0450000000000000))*(cj5)*(sj3))))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((((sj5)*((npy)*(npy))))+(((sj5)*((npx)*(npx))))));
dummyeval[0]=((((sj5)*((npy)*(npy))))+(((sj5)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x262=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj3))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=x262;
evalcond[3]=x262;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign((((npx)*(npx))+((npy)*(npy))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x263=((IkReal(0.0450000000000000))*(cj3));
IkReal x264=((npx)*(sj4));
IkReal x265=((cj4)*(npy));
IkReal x266=((IkReal(0.550000000000000))*(sj3));
IkReal x267=((npy)*(sj4));
IkReal x268=((cj4)*(npx));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x263)*(x264)))+(((IkReal(-1.00000000000000))*(x263)*(x265)))+(((x264)*(x266)))+(((IkReal(0.0450000000000000))*(x264)))+(((IkReal(0.0450000000000000))*(x265)))+(((x265)*(x266))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x263)*(x267)))+(((x266)*(x267)))+(((x263)*(x268)))+(((IkReal(-0.0450000000000000))*(x268)))+(((IkReal(-1.00000000000000))*(x266)*(x268)))+(((IkReal(0.0450000000000000))*(x267))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x263)*(x264)))+(((IkReal(-1.00000000000000))*(x263)*(x265)))+(((x264)*(x266)))+(((IkReal(0.0450000000000000))*(x264)))+(((IkReal(0.0450000000000000))*(x265)))+(((x265)*(x266)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x263)*(x267)))+(((x266)*(x267)))+(((x263)*(x268)))+(((IkReal(-0.0450000000000000))*(x268)))+(((IkReal(-1.00000000000000))*(x266)*(x268)))+(((IkReal(0.0450000000000000))*(x267)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x269=IKsin(j6);
IkReal x270=IKcos(j6);
IkReal x271=((IkReal(0.550000000000000))*(sj3));
IkReal x272=((IkReal(0.0450000000000000))*(cj3));
IkReal x273=((IkReal(1.00000000000000))*(npx));
IkReal x274=((npy)*(x270));
IkReal x275=((cj4)*(x269));
IkReal x276=((npy)*(x269));
evalcond[0]=((((IkReal(-1.00000000000000))*(x274)))+(((IkReal(-1.00000000000000))*(sj4)*(x272)))+(((IkReal(0.0450000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(x269)*(x273)))+(((sj4)*(x271))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x273)*(x275)))+(((IkReal(-1.00000000000000))*(cj4)*(x274)))+(((IkReal(-1.00000000000000))*(sj4)*(x270)*(x273)))+(((sj4)*(x276))));
evalcond[2]=((IkReal(-0.0450000000000000))+(x272)+(((npy)*(x275)))+(((npx)*(sj4)*(x269)))+(((IkReal(-1.00000000000000))*(x271)))+(((sj4)*(x274)))+(((IkReal(-1.00000000000000))*(cj4)*(x270)*(x273))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj4)*(x271)))+(((cj4)*(x272)))+(((IkReal(-0.0450000000000000))*(cj4)))+(x276)+(((IkReal(-1.00000000000000))*(x270)*(x273))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x277=((IkReal(0.0450000000000000))*(sj3));
IkReal x278=((IkReal(0.550000000000000))*(cj3));
IkReal x279=((x277)+(x278));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x279));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x279))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign((((npx)*(npx))+((npy)*(npy))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x280=((cj4)*(npy));
IkReal x281=((IkReal(0.550000000000000))*(sj3));
IkReal x282=((npy)*(sj4));
IkReal x283=((IkReal(0.0450000000000000))*(cj3));
IkReal x284=((cj4)*(npx));
IkReal x285=((npx)*(sj4));
IkReal x286=((sj4)*(x283));
if( IKabs(((gconst2)*(((((IkReal(-0.0450000000000000))*(x280)))+(((IkReal(-1.00000000000000))*(x280)*(x281)))+(((x280)*(x283)))+(((IkReal(0.0450000000000000))*(x285)))+(((x281)*(x285)))+(((IkReal(-1.00000000000000))*(x283)*(x285))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x282)*(x283)))+(((IkReal(0.0450000000000000))*(x284)))+(((IkReal(0.0450000000000000))*(x282)))+(((x281)*(x282)))+(((x281)*(x284)))+(((IkReal(-1.00000000000000))*(x283)*(x284))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((IkReal(-0.0450000000000000))*(x280)))+(((IkReal(-1.00000000000000))*(x280)*(x281)))+(((x280)*(x283)))+(((IkReal(0.0450000000000000))*(x285)))+(((x281)*(x285)))+(((IkReal(-1.00000000000000))*(x283)*(x285)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x282)*(x283)))+(((IkReal(0.0450000000000000))*(x284)))+(((IkReal(0.0450000000000000))*(x282)))+(((x281)*(x282)))+(((x281)*(x284)))+(((IkReal(-1.00000000000000))*(x283)*(x284)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x287=IKcos(j6);
IkReal x288=IKsin(j6);
IkReal x289=((IkReal(0.550000000000000))*(sj3));
IkReal x290=((IkReal(0.0450000000000000))*(cj3));
IkReal x291=((IkReal(1.00000000000000))*(npy));
IkReal x292=((sj4)*(x287));
IkReal x293=((sj4)*(x288));
IkReal x294=((npx)*(x287));
IkReal x295=((IkReal(1.00000000000000))*(npx)*(x288));
evalcond[0]=((((IkReal(-1.00000000000000))*(x295)))+(((IkReal(0.0450000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(x287)*(x291)))+(((IkReal(-1.00000000000000))*(sj4)*(x290)))+(((sj4)*(x289))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x291)*(x293)))+(((IkReal(-1.00000000000000))*(cj4)*(x295)))+(((IkReal(-1.00000000000000))*(cj4)*(x287)*(x291)))+(((npx)*(x292))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((cj4)*(x294)))+(x290)+(((npx)*(x293)))+(((IkReal(-1.00000000000000))*(cj4)*(x288)*(x291)))+(((IkReal(-1.00000000000000))*(x289)))+(((npy)*(x292))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x294)))+(((IkReal(0.0450000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj4)*(x290)))+(((npy)*(x288)))+(((cj4)*(x289))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x296=((IkReal(0.550000000000000))*(cj3));
IkReal x297=((IkReal(0.550000000000000))*(sj3));
IkReal x298=((cj5)*(npz));
IkReal x299=((IkReal(0.0450000000000000))*(sj3));
IkReal x300=((IkReal(0.0450000000000000))*(cj3));
IkReal x301=((npy)*(sj4)*(sj5));
IkReal x302=((npx)*(sj4)*(sj5));
if( IKabs(((gconst0)*(((((IkReal(0.300000000000000))*(npy)))+(((x297)*(x302)))+(((IkReal(0.0450000000000000))*(x302)))+(((IkReal(-1.00000000000000))*(npy)*(x298)))+(((IkReal(-1.00000000000000))*(x300)*(x302)))+(((npy)*(x299)))+(((npy)*(x296))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x296)))+(((IkReal(-1.00000000000000))*(npx)*(x299)))+(((npx)*(x298)))+(((x297)*(x301)))+(((IkReal(0.0450000000000000))*(x301)))+(((IkReal(-1.00000000000000))*(x300)*(x301)))+(((IkReal(-0.300000000000000))*(npx))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(((((IkReal(0.300000000000000))*(npy)))+(((x297)*(x302)))+(((IkReal(0.0450000000000000))*(x302)))+(((IkReal(-1.00000000000000))*(npy)*(x298)))+(((IkReal(-1.00000000000000))*(x300)*(x302)))+(((npy)*(x299)))+(((npy)*(x296)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x296)))+(((IkReal(-1.00000000000000))*(npx)*(x299)))+(((npx)*(x298)))+(((x297)*(x301)))+(((IkReal(0.0450000000000000))*(x301)))+(((IkReal(-1.00000000000000))*(x300)*(x301)))+(((IkReal(-0.300000000000000))*(npx)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x303=IKcos(j6);
IkReal x304=IKsin(j6);
IkReal x305=((IkReal(1.00000000000000))*(npz));
IkReal x306=((cj4)*(cj5));
IkReal x307=((IkReal(0.0450000000000000))*(sj4));
IkReal x308=((IkReal(0.550000000000000))*(sj3));
IkReal x309=((IkReal(0.550000000000000))*(cj3));
IkReal x310=((IkReal(0.0450000000000000))*(cj3));
IkReal x311=((cj5)*(sj4));
IkReal x312=((IkReal(0.0450000000000000))*(sj3));
IkReal x313=((IkReal(1.00000000000000))*(cj4));
IkReal x314=((npy)*(x304));
IkReal x315=((npx)*(x304));
IkReal x316=((npy)*(x303));
IkReal x317=((npx)*(x303));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj3)*(x307)))+(x307)+(((sj4)*(x308)))+(((IkReal(-1.00000000000000))*(x316)))+(((IkReal(-1.00000000000000))*(x315))));
evalcond[1]=((IkReal(0.300000000000000))+(x309)+(x312)+(((IkReal(-1.00000000000000))*(cj5)*(x305)))+(((sj5)*(x317)))+(((IkReal(-1.00000000000000))*(sj5)*(x314))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x313)*(x316)))+(((IkReal(-1.00000000000000))*(x313)*(x315)))+(((IkReal(-1.00000000000000))*(sj4)*(sj5)*(x305)))+(((x311)*(x314)))+(((IkReal(-1.00000000000000))*(x311)*(x317))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((sj4)*(x315)))+(((sj4)*(x316)))+(x310)+(((x306)*(x314)))+(((IkReal(-1.00000000000000))*(x306)*(x317)))+(((IkReal(-1.00000000000000))*(cj4)*(sj5)*(x305)))+(((IkReal(-1.00000000000000))*(x308))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x312)))+(x314)+(((IkReal(-1.00000000000000))*(x306)*(x308)))+(((x306)*(x310)))+(((IkReal(-0.0450000000000000))*(x306)))+(((IkReal(-1.00000000000000))*(x317)))+(((IkReal(-0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x309))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x318=(sj5)*(sj5);
if( IKabs(((npx)*(sj5))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npy)*(sj5))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x319=((IkReal(1.00000000000000))*(IKatan2(((npx)*(sj5)), ((IkReal(-1.00000000000000))*(npy)*(sj5)))));
if( (((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(0.550000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(0.550000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x320=IKasin(((((IKabs(IKabs(IKsqrt(((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x318)*((npy)*(npy))))+(((x318)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(0.550000000000000))*(cj3)))))));
j6array[0]=((((IkReal(-1.00000000000000))*(x320)))+(((IkReal(-1.00000000000000))*(x319))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(x320)+(((IkReal(-1.00000000000000))*(x319))));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];

{
IkReal dummyeval[2];
IkReal x321=((IkReal(12.2222222222222))*(sj3));
dummyeval[0]=((IkReal(1.00000000000000))+(x321)+(((IkReal(-1.00000000000000))*(cj3))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x321)))+(((cj3)*(sj5))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x322=((IkReal(12.2222222222222))*(sj3));
dummyeval[0]=((IkReal(1.00000000000000))+(x322)+(((IkReal(-1.00000000000000))*(cj3))));
dummyeval[1]=((((cj3)*(cj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x322)))+(((IkReal(-1.00000000000000))*(cj5))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x323=((IkReal(12.2222222222222))*(sj3));
IkReal x324=((npx)*(sj6));
IkReal x325=((cj6)*(npy));
dummyeval[0]=((IkReal(1.00000000000000))+(x323)+(((IkReal(-1.00000000000000))*(cj3))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(x325)))+(((IkReal(-1.00000000000000))*(x324)))+(((IkReal(-1.00000000000000))*(x323)*(x325)))+(((IkReal(-1.00000000000000))*(x323)*(x324)))+(((cj3)*(x324)))+(((cj3)*(x325))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x326=((IkReal(1.00000000000000))*(npy));
IkReal x327=((IkReal(1.00000000000000))*(npx));
IkReal x328=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(x326)))+(((IkReal(-1.00000000000000))*(sj6)*(x327))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x328)))+(((cj6)*(npx)*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(sj6)*(x326))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x328)))+(((IkReal(0.850000000000000))*(cj5))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj6)*(x327)))+(((IkReal(-0.850000000000000))*(sj5)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x329=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x329));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x329))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x330=IKsin(j4);
IkReal x331=IKcos(j4);
IkReal x332=((IkReal(1.00000000000000))*(x331));
IkReal x333=((IkReal(1.00000000000000))*(x330));
evalcond[0]=((((cj5)*(npy)*(sj6)*(x330)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x332)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x333)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x333)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x332))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x334=((IkReal(1.00000000000000))*(npy));
IkReal x335=((IkReal(1.00000000000000))*(npx));
IkReal x336=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(x334)))+(((IkReal(-1.00000000000000))*(sj6)*(x335))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(sj5)*(sj6)*(x334)))+(((IkReal(-1.00000000000000))*(cj5)*(x336)))+(((cj6)*(npx)*(sj5))));
evalcond[4]=((((IkReal(0.850000000000000))*(cj5)))+(((IkReal(-1.00000000000000))*(x336))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(cj6)*(x335)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x337=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x337));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x337))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x338=IKsin(j4);
IkReal x339=IKcos(j4);
IkReal x340=((IkReal(1.00000000000000))*(x339));
IkReal x341=((IkReal(1.00000000000000))*(x338));
evalcond[0]=((((cj5)*(npy)*(sj6)*(x338)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x341)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x340)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x340)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x341))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(npy)*(sj5)*(sj6)))+(((cj6)*(npx)*(sj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x342=((npy)*(sj6));
IkReal x343=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(x343)+(((IkReal(-1.00000000000000))*(x342))));
evalcond[3]=((IkReal(0.250000000390712))+(x342)+(((IkReal(-1.00000000000000))*(x343))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(-414405822679.079))*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x344=IKsin(j4);
IkReal x345=IKcos(j4);
IkReal x346=((IkReal(1.00000000000000))*(npz));
IkReal x347=((cj6)*(npy));
IkReal x348=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x348)))+(((IkReal(2.41309350707268e-12))*(x344)))+(((IkReal(-1.00000000000000))*(x347))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x344)*(x346)))+(((IkReal(-1.00000000000000))*(x345)*(x348)))+(((IkReal(-1.00000000000000))*(x345)*(x347))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x345)))+(((IkReal(-1.00000000000000))*(x346))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x344)*(x347)))+(((npx)*(sj6)*(x344)))+(((IkReal(-1.00000000000000))*(x345)*(x346))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x349=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x349;
evalcond[3]=x349;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(414405822679.079))*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x350=IKsin(j4);
IkReal x351=IKcos(j4);
IkReal x352=((cj6)*(npy));
IkReal x353=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x353)))+(((IkReal(-1.00000000000000))*(x352)))+(((IkReal(2.41309350707268e-12))*(x350))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x351)*(x353)))+(((npz)*(x350))));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((IkReal(2.41309350707268e-12))*(x351))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x350)*(x352)))+(((npx)*(sj6)*(x350)))+(((npz)*(x351))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x354=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x354;
evalcond[3]=x354;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x355=((IkReal(414405822679.079))*(npx));
IkReal x356=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj6)*(x355)))+(((cj6)*(x356))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj6)*(x355)))+(((sj6)*(x356))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj6)*(x355)))+(((cj6)*(x356)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj6)*(x355)))+(((sj6)*(x356)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((sj6)*(x355)))+(((cj6)*(x356)))), ((((IkReal(-1.00000000000000))*(cj6)*(x355)))+(((sj6)*(x356)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x357=IKsin(j4);
IkReal x358=IKcos(j4);
IkReal x359=((IkReal(1.00000000000000))*(cj6));
IkReal x360=((npy)*(sj6));
IkReal x361=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x362=((npx)*(x357));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x359)))+(((IkReal(-1.00000000000000))*(x361)))+(((IkReal(2.41309350707268e-12))*(x357))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x358)*(x361)))+(((IkReal(-1.00000000000000))*(x359)*(x362)))+(((x357)*(x360)))+(((IkReal(-1.00000000000000))*(npy)*(x358)*(x359))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((x358)*(x360)))+(((IkReal(-1.00000000000000))*(npx)*(x358)*(x359)))+(((cj6)*(npy)*(x357)))+(((sj6)*(x362))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npx)*(x359)))+(x360)+(((IkReal(-2.41309350707268e-12))*(x358))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(npz));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x363=((IkReal(414405822679.079))*(npx));
IkReal x364=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj6)*(x363)))+(((cj6)*(x364))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(x363)))+(((IkReal(-1.00000000000000))*(sj6)*(x364))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj6)*(x363)))+(((cj6)*(x364)))))+IKsqr(((((cj6)*(x363)))+(((IkReal(-1.00000000000000))*(sj6)*(x364)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((sj6)*(x363)))+(((cj6)*(x364)))), ((((cj6)*(x363)))+(((IkReal(-1.00000000000000))*(sj6)*(x364)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x365=IKsin(j4);
IkReal x366=IKcos(j4);
IkReal x367=((cj6)*(npx));
IkReal x368=((cj6)*(npy));
IkReal x369=((IkReal(1.00000000000000))*(npy)*(sj6));
IkReal x370=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x368)))+(((IkReal(2.41309350707268e-12))*(x365)))+(((IkReal(-1.00000000000000))*(x370))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x366)*(x368)))+(((x365)*(x367)))+(((IkReal(-1.00000000000000))*(x366)*(x370)))+(((IkReal(-1.00000000000000))*(x365)*(x369))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((npx)*(sj6)*(x365)))+(((x366)*(x367)))+(((x365)*(x368)))+(((IkReal(-1.00000000000000))*(x366)*(x369))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x367)))+(((IkReal(2.41309350707268e-12))*(x366)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x371=((IkReal(2.28323001692700e+21))*(cj5));
IkReal x372=((cj6)*(npx));
IkReal x373=((npy)*(sj6));
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((x371)*(x373))))) != 0)?((IkReal)1/(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((x371)*(x373)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj6)*(sj6))))+(((IkReal(-1.89236762706040e+33))*(x372)*(x373)))+(((IkReal(-9.46183813530201e+32))*((cj6)*(cj6))*((npy)*(npy)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((((IKabs(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((x371)*(x373))))) != 0)?((IkReal)1/(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((x371)*(x373)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj6)*(sj6))))+(((IkReal(-1.89236762706040e+33))*(x372)*(x373)))+(((IkReal(-9.46183813530201e+32))*((cj6)*(cj6))*((npy)*(npy))))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((((IKabs(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((x371)*(x373))))) != 0)?((IkReal)1/(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((x371)*(x373)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj6)*(sj6))))+(((IkReal(-1.89236762706040e+33))*(x372)*(x373)))+(((IkReal(-9.46183813530201e+32))*((cj6)*(cj6))*((npy)*(npy))))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x374=IKsin(j4);
IkReal x375=IKcos(j4);
IkReal x376=((IkReal(1.00000000000000))*(cj6));
IkReal x377=((cj5)*(npx));
IkReal x378=((cj5)*(npy));
IkReal x379=((IkReal(1.00000000000000))*(npz));
IkReal x380=((IkReal(1.00000000000000))*(npx));
IkReal x381=((IkReal(2.41309350707268e-12))*(x375));
IkReal x382=((sj6)*(x375));
IkReal x383=((sj6)*(x374));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x374)))+(((IkReal(-1.00000000000000))*(npy)*(x376)))+(((IkReal(-1.00000000000000))*(sj6)*(x380))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x374)*(x379)))+(((x378)*(x383)))+(((IkReal(-1.00000000000000))*(x380)*(x382)))+(((IkReal(-1.00000000000000))*(x374)*(x376)*(x377)))+(((IkReal(-1.00000000000000))*(npy)*(x375)*(x376))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x381)))+(((IkReal(-1.00000000000000))*(x379)))+(((IkReal(-0.250000000390712))*(cj5))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x378)*(x382)))+(((npx)*(x383)))+(((IkReal(-1.00000000000000))*(sj5)*(x375)*(x379)))+(((cj6)*(npy)*(x374)))+(((IkReal(-1.00000000000000))*(x375)*(x376)*(x377))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj5)))+(((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x381)))+(((IkReal(-1.00000000000000))*(npx)*(x376))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(cj6)*(npx)))+(((IkReal(7.69002526201833e+15))*(sj5)))+(((IkReal(3.07601010000000e+16))*(npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(cj6)*(npx)))+(((IkReal(7.69002526201833e+15))*(sj5)))+(((IkReal(3.07601010000000e+16))*(npy)*(sj6)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(1.34721866706185e-5))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(cj6)*(npx)))+(((IkReal(7.69002526201833e+15))*(sj5)))+(((IkReal(3.07601010000000e+16))*(npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x384=IKsin(j4);
IkReal x385=IKcos(j4);
IkReal x386=((IkReal(1.00000000000000))*(cj6));
IkReal x387=((cj5)*(npx));
IkReal x388=((cj5)*(npy));
IkReal x389=((IkReal(1.00000000000000))*(npz));
IkReal x390=((IkReal(1.00000000000000))*(npx));
IkReal x391=((IkReal(2.41309350707268e-12))*(x385));
IkReal x392=((sj6)*(x385));
IkReal x393=((sj6)*(x384));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x386)))+(((IkReal(-1.00000000000000))*(sj6)*(x390)))+(((IkReal(2.41309350707268e-12))*(x384))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x384)*(x389)))+(((IkReal(-1.00000000000000))*(npy)*(x385)*(x386)))+(((IkReal(-1.00000000000000))*(x384)*(x386)*(x387)))+(((IkReal(-1.00000000000000))*(x390)*(x392)))+(((x388)*(x393))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x391)))+(((IkReal(-1.00000000000000))*(x389)))+(((IkReal(-0.250000000390712))*(cj5))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(sj5)*(x385)*(x389)))+(((IkReal(-1.00000000000000))*(x385)*(x386)*(x387)))+(((npx)*(x393)))+(((cj6)*(npy)*(x384)))+(((x388)*(x392))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj5)*(x391)))+(((IkReal(0.250000000390712))*(sj5)))+(((IkReal(-1.00000000000000))*(npx)*(x386)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj5)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(1.34721866706185e-5))*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x394=IKsin(j4);
IkReal x395=IKcos(j4);
IkReal x396=((IkReal(1.00000000000000))*(cj6));
IkReal x397=((cj5)*(npx));
IkReal x398=((cj5)*(npy));
IkReal x399=((IkReal(1.00000000000000))*(npz));
IkReal x400=((IkReal(1.00000000000000))*(npx));
IkReal x401=((IkReal(2.41309350707268e-12))*(x395));
IkReal x402=((sj6)*(x395));
IkReal x403=((sj6)*(x394));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x394)))+(((IkReal(-1.00000000000000))*(sj6)*(x400)))+(((IkReal(-1.00000000000000))*(npy)*(x396))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npy)*(x395)*(x396)))+(((IkReal(-1.00000000000000))*(x400)*(x402)))+(((x398)*(x403)))+(((IkReal(-1.00000000000000))*(sj5)*(x394)*(x399)))+(((IkReal(-1.00000000000000))*(x394)*(x396)*(x397))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x401)))+(((IkReal(-1.00000000000000))*(x399)))+(((IkReal(-0.250000000390712))*(cj5))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(sj5)*(x395)*(x399)))+(((cj6)*(npy)*(x394)))+(((npx)*(x403)))+(((x398)*(x402)))+(((IkReal(-1.00000000000000))*(x395)*(x396)*(x397))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj5)))+(((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x401)))+(((IkReal(-1.00000000000000))*(npx)*(x396))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x404=((IkReal(0.0450000000000000))*(sj3));
IkReal x405=((IkReal(0.550000000000000))*(cj3));
IkReal x406=((cj6)*(npx));
IkReal x407=((npy)*(sj6));
IkReal x408=((x405)+(x404));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(x406)+(x408)+(((IkReal(-1.00000000000000))*(x407))));
evalcond[3]=((IkReal(-0.300000000000000))+(x407)+(((IkReal(-1.00000000000000))*(x406)))+(((IkReal(-1.00000000000000))*(x408))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(-1.00000000000000))+(cj3)+(((IkReal(-12.2222222222222))*(sj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x409=((IkReal(1.00000000000000))*(cj6));
IkReal x410=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x409))));
evalcond[3]=((IkReal(0.850000000000000))+(((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(x410))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x409)))+(x410));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x411=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x411));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x411))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x412=IKcos(j4);
IkReal x413=((IkReal(1.00000000000000))*(x412));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x413)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x413)))+(((IkReal(-1.00000000000000))*(npz)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x414=((npy)*(sj6));
IkReal x415=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(6.11991262292145))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(x415)+(((IkReal(-1.00000000000000))*(x414))));
evalcond[3]=((IkReal(0.250000000390712))+(x414)+(((IkReal(-1.00000000000000))*(x415))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(-414405822679.079))*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x416=IKsin(j4);
IkReal x417=IKcos(j4);
IkReal x418=((IkReal(1.00000000000000))*(npz));
IkReal x419=((cj6)*(npy));
IkReal x420=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x420)))+(((IkReal(2.41309350707268e-12))*(x416)))+(((IkReal(-1.00000000000000))*(x419))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x417)*(x419)))+(((IkReal(-1.00000000000000))*(x416)*(x418)))+(((IkReal(-1.00000000000000))*(x417)*(x420))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x417)))+(((IkReal(-1.00000000000000))*(x418))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x417)*(x418)))+(((x416)*(x419)))+(((npx)*(sj6)*(x416))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x421=((IkReal(1.00000000000000))*(cj6));
IkReal x422=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x421)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(x422)))+(((cj6)*(npx))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x421)))+(x422));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x423=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x423));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x423))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x424=IKcos(j4);
IkReal x425=((IkReal(1.00000000000000))*(x424));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x425)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x425)))+(((IkReal(-1.00000000000000))*(npz)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))))), ((gconst3)*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x426=IKsin(j4);
IkReal x427=IKcos(j4);
IkReal x428=((IkReal(0.550000000000000))*(sj3));
IkReal x429=((IkReal(1.00000000000000))*(npz));
IkReal x430=((IkReal(0.0450000000000000))*(cj3));
IkReal x431=((IkReal(1.00000000000000))*(cj6)*(npy));
IkReal x432=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((x426)*(x428)))+(((IkReal(-1.00000000000000))*(x432)))+(((IkReal(-1.00000000000000))*(x431)))+(((IkReal(-1.00000000000000))*(x426)*(x430)))+(((IkReal(0.0450000000000000))*(x426))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x427)*(x432)))+(((IkReal(-1.00000000000000))*(x427)*(x431)))+(((IkReal(-1.00000000000000))*(x426)*(x429))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x429)))+(((IkReal(-1.00000000000000))*(x427)*(x428)))+(((IkReal(-0.0450000000000000))*(x427)))+(((x427)*(x430))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x428)))+(((npx)*(sj6)*(x426)))+(((IkReal(-1.00000000000000))*(x427)*(x429)))+(((cj6)*(npy)*(x426)))+(x430));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x433=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))+(((IkReal(0.550000000000000))*(cj3))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=x433;
evalcond[3]=x433;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x434=((IkReal(1.00000000000000))*(npx));
IkReal x435=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj6)*(x434)))+(((npy)*(sj6))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x434))));
evalcond[3]=x435;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x435;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x436=IKatan2(npz, ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x436));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x436))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x437=IKcos(j4);
IkReal x438=((IkReal(1.00000000000000))*(x437));
evalcond[0]=((((npz)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x438)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x438))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x439=((IkReal(1.00000000000000))*(npx));
IkReal x440=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj6)*(x439)))+(((npy)*(sj6))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x439))));
evalcond[3]=x440;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x440;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x441=IKatan2(npz, ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x441));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x441))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x442=IKcos(j4);
IkReal x443=((IkReal(1.00000000000000))*(x442));
evalcond[0]=((((npz)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x443)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x443))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst4)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst4)*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x444=IKsin(j4);
IkReal x445=IKcos(j4);
IkReal x446=((IkReal(0.550000000000000))*(sj3));
IkReal x447=((IkReal(0.0450000000000000))*(cj3));
IkReal x448=((npx)*(sj6));
IkReal x449=((IkReal(1.00000000000000))*(cj6)*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(x449)))+(((IkReal(-1.00000000000000))*(x444)*(x447)))+(((IkReal(-1.00000000000000))*(x448)))+(((x444)*(x446)))+(((IkReal(0.0450000000000000))*(x444))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x445)*(x449)))+(((npz)*(x444)))+(((IkReal(-1.00000000000000))*(x445)*(x448))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x445)*(x447)))+(((IkReal(-1.00000000000000))*(npz)))+(((x445)*(x446)))+(((IkReal(0.0450000000000000))*(x445))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x446)))+(((cj6)*(npy)*(x444)))+(((npz)*(x445)))+(x447)+(((x444)*(x448))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x450=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj3))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=x450;
evalcond[3]=x450;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x451=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x452=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x452))));
evalcond[3]=x451;
evalcond[4]=x451;
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x452))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x453=IKatan2(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x453));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x453))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x454=IKcos(j4);
IkReal x455=IKsin(j4);
IkReal x456=((IkReal(1.00000000000000))*(x454));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x456)))+(((npy)*(sj6)*(x455)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x455)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x456))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x457=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x458=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x458))));
evalcond[3]=x457;
evalcond[4]=x457;
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x458))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x459=IKatan2(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x459));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x459))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x460=IKcos(j4);
IkReal x461=IKsin(j4);
IkReal x462=((IkReal(1.00000000000000))*(x460));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x462)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x461)))+(((npy)*(sj6)*(x461)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x462))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst5)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst5)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst5)*(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x463=IKsin(j4);
IkReal x464=IKcos(j4);
IkReal x465=((IkReal(0.550000000000000))*(sj3));
IkReal x466=((IkReal(0.0450000000000000))*(cj3));
IkReal x467=((npy)*(sj6));
IkReal x468=((IkReal(1.00000000000000))*(npx));
IkReal x469=((IkReal(1.00000000000000))*(cj6)*(npy));
IkReal x470=((cj6)*(x463));
evalcond[0]=((((IkReal(-1.00000000000000))*(x463)*(x466)))+(((x463)*(x465)))+(((IkReal(-1.00000000000000))*(x469)))+(((IkReal(0.0450000000000000))*(x463)))+(((IkReal(-1.00000000000000))*(sj6)*(x468))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj6)*(x464)*(x468)))+(((IkReal(-1.00000000000000))*(x468)*(x470)))+(((x463)*(x467)))+(((IkReal(-1.00000000000000))*(x464)*(x469))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((npy)*(x470)))+(((IkReal(-1.00000000000000))*(x465)))+(((npx)*(sj6)*(x463)))+(x466)+(((IkReal(-1.00000000000000))*(cj6)*(x464)*(x468)))+(((x464)*(x467))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x464)*(x465)))+(x467)+(((IkReal(-0.0450000000000000))*(x464)))+(((IkReal(-1.00000000000000))*(cj6)*(x468)))+(((x464)*(x466))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x471=((IkReal(0.0450000000000000))*(sj3));
IkReal x472=((IkReal(0.550000000000000))*(cj3));
IkReal x473=((x471)+(x472));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x473));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x473))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x474=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x474))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x474))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x475=IKatan2(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x475));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x475))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x476=IKsin(j4);
IkReal x477=IKcos(j4);
IkReal x478=((IkReal(1.00000000000000))*(x477));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(sj6)*(x476)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x478)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x478)))+(((cj6)*(npx)*(x476))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x479=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x479))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x479))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x480=IKatan2(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x480));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x480))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x481=IKsin(j4);
IkReal x482=IKcos(j4);
IkReal x483=((IkReal(1.00000000000000))*(x482));
evalcond[0]=((((cj6)*(npx)*(x481)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)*(x481)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x483)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x483))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst6)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst6)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst6)*(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x484=IKsin(j4);
IkReal x485=IKcos(j4);
IkReal x486=((IkReal(0.550000000000000))*(sj3));
IkReal x487=((IkReal(0.0450000000000000))*(cj3));
IkReal x488=((cj6)*(npx));
IkReal x489=((cj6)*(npy));
IkReal x490=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x491=((sj6)*(x484));
IkReal x492=((IkReal(1.00000000000000))*(npy)*(x485));
evalcond[0]=((((IkReal(-1.00000000000000))*(x484)*(x487)))+(((x484)*(x486)))+(((IkReal(-1.00000000000000))*(x490)))+(((IkReal(-1.00000000000000))*(x489)))+(((IkReal(0.0450000000000000))*(x484))));
evalcond[1]=((((x484)*(x488)))+(((IkReal(-1.00000000000000))*(x485)*(x489)))+(((IkReal(-1.00000000000000))*(npy)*(x491)))+(((IkReal(-1.00000000000000))*(x485)*(x490))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((x485)*(x488)))+(((x484)*(x489)))+(((npx)*(x491)))+(((IkReal(-1.00000000000000))*(x486)))+(((IkReal(-1.00000000000000))*(sj6)*(x492)))+(x487));
evalcond[3]=((((x485)*(x486)))+(((IkReal(-1.00000000000000))*(x488)))+(((npy)*(sj6)))+(((IkReal(0.0450000000000000))*(x485)))+(((IkReal(-1.00000000000000))*(x485)*(x487))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x493=((IkReal(0.0450000000000000))*(cj3));
IkReal x494=((npx)*(sj6));
IkReal x495=((IkReal(0.550000000000000))*(sj3));
IkReal x496=((cj6)*(npy));
IkReal x497=((npz)*(sj5));
IkReal x498=((cj5)*(npx)*(npy));
IkReal x499=((cj5)*(cj6)*(sj6));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(x495))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(x495)))):(IkReal)1.0e30))*(((x496)+(x494))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x494)*(x495)))+(((IkReal(-0.0450000000000000))*(x496)))+(((IkReal(-0.0450000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((x493)*(x494)))+(((x493)*(x496))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x494)*(x495)))+(((IkReal(-0.0450000000000000))*(x496)))+(((IkReal(-0.0450000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((x493)*(x494)))+(((x493)*(x496)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x496)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x494)))+(((x496)*(x497)))+(((x494)*(x497)))+(((cj5)*(cj6)*(npx)*(x494)))+(((cj5)*(cj6)*(npx)*(x496))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(x495))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(x495)))):(IkReal)1.0e30))*(((x496)+(x494)))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x494)*(x495)))+(((IkReal(-0.0450000000000000))*(x496)))+(((IkReal(-0.0450000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((x493)*(x494)))+(((x493)*(x496))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x494)*(x495)))+(((IkReal(-0.0450000000000000))*(x496)))+(((IkReal(-0.0450000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((x493)*(x494)))+(((x493)*(x496)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x496)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x494)))+(((x496)*(x497)))+(((x494)*(x497)))+(((cj5)*(cj6)*(npx)*(x494)))+(((cj5)*(cj6)*(npx)*(x496)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(x495))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(x495)))):(IkReal)1.0e30))*(((x496)+(x494)))), ((((IKabs(((((IkReal(-1.00000000000000))*(x494)*(x495)))+(((IkReal(-0.0450000000000000))*(x496)))+(((IkReal(-0.0450000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((x493)*(x494)))+(((x493)*(x496))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x494)*(x495)))+(((IkReal(-0.0450000000000000))*(x496)))+(((IkReal(-0.0450000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((x493)*(x494)))+(((x493)*(x496)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x496)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x494)))+(((x496)*(x497)))+(((x494)*(x497)))+(((cj5)*(cj6)*(npx)*(x494)))+(((cj5)*(cj6)*(npx)*(x496)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x500=IKsin(j4);
IkReal x501=IKcos(j4);
IkReal x502=((IkReal(0.550000000000000))*(cj3));
IkReal x503=((npy)*(sj6));
IkReal x504=((IkReal(0.0450000000000000))*(cj3));
IkReal x505=((IkReal(1.00000000000000))*(npx));
IkReal x506=((cj6)*(npy));
IkReal x507=((cj5)*(sj3));
IkReal x508=((cj5)*(cj6));
IkReal x509=((IkReal(1.00000000000000))*(npz));
IkReal x510=((IkReal(0.550000000000000))*(sj3));
IkReal x511=((IkReal(0.0450000000000000))*(x501));
IkReal x512=((sj5)*(x501));
IkReal x513=((cj5)*(x501));
evalcond[0]=((((IkReal(0.0450000000000000))*(x500)))+(((x500)*(x510)))+(((IkReal(-1.00000000000000))*(sj6)*(x505)))+(((IkReal(-1.00000000000000))*(x506)))+(((IkReal(-1.00000000000000))*(x500)*(x504))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x500)*(x505)*(x508)))+(((IkReal(-1.00000000000000))*(sj6)*(x501)*(x505)))+(((IkReal(-1.00000000000000))*(x501)*(x506)))+(((cj5)*(x500)*(x503)))+(((IkReal(-1.00000000000000))*(sj5)*(x500)*(x509))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x509)))+(((IkReal(0.0450000000000000))*(x507)))+(((x504)*(x512)))+(((IkReal(-1.00000000000000))*(sj5)*(x511)))+(((cj5)*(x502)))+(((IkReal(0.300000000000000))*(cj5)))+(((IkReal(-1.00000000000000))*(x510)*(x512))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((x500)*(x506)))+(((IkReal(-1.00000000000000))*(x501)*(x505)*(x508)))+(((x503)*(x513)))+(x504)+(((IkReal(-1.00000000000000))*(x510)))+(((npx)*(sj6)*(x500)))+(((IkReal(-1.00000000000000))*(x509)*(x512))));
evalcond[4]=((((IkReal(-0.550000000000000))*(x501)*(x507)))+(((x504)*(x513)))+(((IkReal(-1.00000000000000))*(cj6)*(x505)))+(x503)+(((IkReal(-1.00000000000000))*(cj5)*(x511)))+(((IkReal(-0.0450000000000000))*(sj3)*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x502)))+(((IkReal(-0.300000000000000))*(sj5))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x514=((IkReal(0.0450000000000000))*(cj5));
IkReal x515=((IkReal(0.550000000000000))*(sj3));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x515))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x515)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj3)*(x514)))+(((IkReal(-1.00000000000000))*(cj5)*(x515)))+(((IkReal(-1.00000000000000))*(x514))))) != 0)?((IkReal)1/(((((cj3)*(x514)))+(((IkReal(-1.00000000000000))*(cj5)*(x515)))+(((IkReal(-1.00000000000000))*(x514)))))):(IkReal)1.0e30))*(((((cj6)*(npx)))+(((IkReal(0.550000000000000))*(cj3)*(sj5)))+(((IkReal(0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))+(((IkReal(0.0450000000000000))*(sj3)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x515))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x515)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))))+IKsqr(((((IKabs(((((cj3)*(x514)))+(((IkReal(-1.00000000000000))*(cj5)*(x515)))+(((IkReal(-1.00000000000000))*(x514))))) != 0)?((IkReal)1/(((((cj3)*(x514)))+(((IkReal(-1.00000000000000))*(cj5)*(x515)))+(((IkReal(-1.00000000000000))*(x514)))))):(IkReal)1.0e30))*(((((cj6)*(npx)))+(((IkReal(0.550000000000000))*(cj3)*(sj5)))+(((IkReal(0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))+(((IkReal(0.0450000000000000))*(sj3)*(sj5)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x515))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x515)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((((IKabs(((((cj3)*(x514)))+(((IkReal(-1.00000000000000))*(cj5)*(x515)))+(((IkReal(-1.00000000000000))*(x514))))) != 0)?((IkReal)1/(((((cj3)*(x514)))+(((IkReal(-1.00000000000000))*(cj5)*(x515)))+(((IkReal(-1.00000000000000))*(x514)))))):(IkReal)1.0e30))*(((((cj6)*(npx)))+(((IkReal(0.550000000000000))*(cj3)*(sj5)))+(((IkReal(0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))+(((IkReal(0.0450000000000000))*(sj3)*(sj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x516=IKsin(j4);
IkReal x517=IKcos(j4);
IkReal x518=((IkReal(0.550000000000000))*(cj3));
IkReal x519=((npy)*(sj6));
IkReal x520=((IkReal(0.0450000000000000))*(cj3));
IkReal x521=((IkReal(1.00000000000000))*(npx));
IkReal x522=((cj6)*(npy));
IkReal x523=((cj5)*(sj3));
IkReal x524=((cj5)*(cj6));
IkReal x525=((IkReal(1.00000000000000))*(npz));
IkReal x526=((IkReal(0.550000000000000))*(sj3));
IkReal x527=((IkReal(0.0450000000000000))*(x517));
IkReal x528=((sj5)*(x517));
IkReal x529=((cj5)*(x517));
evalcond[0]=((((IkReal(-1.00000000000000))*(x522)))+(((IkReal(-1.00000000000000))*(sj6)*(x521)))+(((x516)*(x526)))+(((IkReal(-1.00000000000000))*(x516)*(x520)))+(((IkReal(0.0450000000000000))*(x516))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj6)*(x517)*(x521)))+(((IkReal(-1.00000000000000))*(x517)*(x522)))+(((IkReal(-1.00000000000000))*(x516)*(x521)*(x524)))+(((IkReal(-1.00000000000000))*(sj5)*(x516)*(x525)))+(((cj5)*(x516)*(x519))));
evalcond[2]=((((cj5)*(x518)))+(((IkReal(-1.00000000000000))*(x526)*(x528)))+(((IkReal(0.0450000000000000))*(x523)))+(((IkReal(-1.00000000000000))*(sj5)*(x527)))+(((IkReal(-1.00000000000000))*(x525)))+(((x520)*(x528)))+(((IkReal(0.300000000000000))*(cj5))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((x519)*(x529)))+(((IkReal(-1.00000000000000))*(x525)*(x528)))+(((npx)*(sj6)*(x516)))+(((x516)*(x522)))+(((IkReal(-1.00000000000000))*(x526)))+(x520)+(((IkReal(-1.00000000000000))*(x517)*(x521)*(x524))));
evalcond[4]=((((IkReal(-0.550000000000000))*(x517)*(x523)))+(((IkReal(-1.00000000000000))*(sj5)*(x518)))+(((IkReal(-1.00000000000000))*(cj6)*(x521)))+(((x520)*(x529)))+(x519)+(((IkReal(-0.0450000000000000))*(sj3)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x527)))+(((IkReal(-0.300000000000000))*(sj5))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x530=((IkReal(0.550000000000000))*(sj3));
IkReal x531=((IkReal(0.0450000000000000))*(cj3));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x531)))+(x530))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x531)))+(x530)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x530)))+(((sj5)*(x531))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x530)))+(((sj5)*(x531)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(npz)+(((IkReal(-0.0450000000000000))*(cj5)*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x531)))+(x530))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x531)))+(x530)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))))+IKsqr(((((IKabs(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x530)))+(((sj5)*(x531))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x530)))+(((sj5)*(x531)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(npz)+(((IkReal(-0.0450000000000000))*(cj5)*(sj3)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x531)))+(x530))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x531)))+(x530)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((((IKabs(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x530)))+(((sj5)*(x531))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x530)))+(((sj5)*(x531)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(npz)+(((IkReal(-0.0450000000000000))*(cj5)*(sj3)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x532=IKsin(j4);
IkReal x533=IKcos(j4);
IkReal x534=((IkReal(0.550000000000000))*(cj3));
IkReal x535=((npy)*(sj6));
IkReal x536=((IkReal(0.0450000000000000))*(cj3));
IkReal x537=((IkReal(1.00000000000000))*(npx));
IkReal x538=((cj6)*(npy));
IkReal x539=((cj5)*(sj3));
IkReal x540=((cj5)*(cj6));
IkReal x541=((IkReal(1.00000000000000))*(npz));
IkReal x542=((IkReal(0.550000000000000))*(sj3));
IkReal x543=((IkReal(0.0450000000000000))*(x533));
IkReal x544=((sj5)*(x533));
IkReal x545=((cj5)*(x533));
evalcond[0]=((((IkReal(-1.00000000000000))*(x532)*(x536)))+(((IkReal(0.0450000000000000))*(x532)))+(((IkReal(-1.00000000000000))*(x538)))+(((IkReal(-1.00000000000000))*(sj6)*(x537)))+(((x532)*(x542))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x532)*(x541)))+(((cj5)*(x532)*(x535)))+(((IkReal(-1.00000000000000))*(x532)*(x537)*(x540)))+(((IkReal(-1.00000000000000))*(x533)*(x538)))+(((IkReal(-1.00000000000000))*(sj6)*(x533)*(x537))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x542)*(x544)))+(((cj5)*(x534)))+(((IkReal(0.0450000000000000))*(x539)))+(((IkReal(-1.00000000000000))*(sj5)*(x543)))+(((IkReal(-1.00000000000000))*(x541)))+(((x536)*(x544)))+(((IkReal(0.300000000000000))*(cj5))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x541)*(x544)))+(((IkReal(-1.00000000000000))*(x533)*(x537)*(x540)))+(((IkReal(-1.00000000000000))*(x542)))+(x536)+(((x535)*(x545)))+(((x532)*(x538)))+(((npx)*(sj6)*(x532))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj5)*(x534)))+(((IkReal(-1.00000000000000))*(cj6)*(x537)))+(((x536)*(x545)))+(((IkReal(-0.550000000000000))*(x533)*(x539)))+(x535)+(((IkReal(-0.0450000000000000))*(sj3)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x543)))+(((IkReal(-0.300000000000000))*(sj5))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[2], cj6array[2], sj6array[2];
bool j6valid[2]={false};
_nj6 = 2;
IkReal x546=(sj5)*(sj5);
if( IKabs(((npx)*(sj5))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npy)*(sj5))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x547=((IkReal(1.00000000000000))*(IKatan2(((npx)*(sj5)), ((IkReal(-1.00000000000000))*(npy)*(sj5)))));
if( (((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(0.550000000000000))*(cj3))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(0.550000000000000))*(cj3))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x548=IKasin(((((IKabs(IKabs(IKsqrt(((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x546)*((npx)*(npx))))+(((x546)*((npy)*(npy))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(0.550000000000000))*(cj3)))))));
j6array[0]=((((IkReal(-1.00000000000000))*(x548)))+(((IkReal(-1.00000000000000))*(x547))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
j6array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x547)))+(x548));
sj6array[1]=IKsin(j6array[1]);
cj6array[1]=IKcos(j6array[1]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
if( j6array[1] > IKPI )
{
    j6array[1]-=IK2PI;
}
else if( j6array[1] < -IKPI )
{    j6array[1]+=IK2PI;
}
j6valid[1] = true;
for(int ij6 = 0; ij6 < 2; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 2; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];

{
IkReal dummyeval[2];
IkReal x549=((IkReal(12.2222222222222))*(sj3));
dummyeval[0]=((IkReal(1.00000000000000))+(x549)+(((IkReal(-1.00000000000000))*(cj3))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x549)))+(((cj3)*(sj5))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x550=((IkReal(12.2222222222222))*(sj3));
dummyeval[0]=((IkReal(1.00000000000000))+(x550)+(((IkReal(-1.00000000000000))*(cj3))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(cj5)*(x550)))+(((cj3)*(cj5)))+(((IkReal(-1.00000000000000))*(cj5))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x551=((IkReal(12.2222222222222))*(sj3));
IkReal x552=((npx)*(sj6));
IkReal x553=((cj6)*(npy));
dummyeval[0]=((IkReal(1.00000000000000))+(x551)+(((IkReal(-1.00000000000000))*(cj3))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(x553)))+(((IkReal(-1.00000000000000))*(x552)))+(((IkReal(-1.00000000000000))*(x551)*(x553)))+(((IkReal(-1.00000000000000))*(x551)*(x552)))+(((cj3)*(x552)))+(((cj3)*(x553))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x554=((IkReal(1.00000000000000))*(npy));
IkReal x555=((IkReal(1.00000000000000))*(npx));
IkReal x556=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(x554)))+(((IkReal(-1.00000000000000))*(sj6)*(x555))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x556)))+(((cj6)*(npx)*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(sj6)*(x554))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x556)))+(((IkReal(0.850000000000000))*(cj5))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(cj6)*(x555)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x557=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x557));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x557))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x558=IKsin(j4);
IkReal x559=IKcos(j4);
IkReal x560=((IkReal(1.00000000000000))*(x559));
IkReal x561=((IkReal(1.00000000000000))*(x558));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x560)))+(((cj5)*(npy)*(sj6)*(x558)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x560)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x561)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x561))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x562=((IkReal(1.00000000000000))*(npy));
IkReal x563=((IkReal(1.00000000000000))*(npx));
IkReal x564=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)*(x563)))+(((IkReal(-1.00000000000000))*(cj6)*(x562))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(sj5)*(sj6)*(x562)))+(((IkReal(-1.00000000000000))*(cj5)*(x564)))+(((cj6)*(npx)*(sj5))));
evalcond[4]=((((IkReal(0.850000000000000))*(cj5)))+(((IkReal(-1.00000000000000))*(x564))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj5)))+(((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x563))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x565=IKatan2(((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x565));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x565))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x566=IKsin(j4);
IkReal x567=IKcos(j4);
IkReal x568=((IkReal(1.00000000000000))*(x567));
IkReal x569=((IkReal(1.00000000000000))*(x566));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x568)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x568)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx)*(x569)))+(((IkReal(-1.00000000000000))*(npz)*(sj5)*(x569)))+(((cj5)*(npy)*(sj6)*(x566))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(cj5)*(npz)))+(((IkReal(-1.00000000000000))*(npy)*(sj5)*(sj6)))+(((cj6)*(npx)*(sj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(npz)*(sj5)))+(((cj5)*(npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(cj6)*(npx))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x570=((npy)*(sj6));
IkReal x571=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(x570)))+(x571));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(x571)))+(x570));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(-414405822679.079))*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x572=IKsin(j4);
IkReal x573=IKcos(j4);
IkReal x574=((IkReal(1.00000000000000))*(npz));
IkReal x575=((cj6)*(npy));
IkReal x576=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x572)))+(((IkReal(-1.00000000000000))*(x575)))+(((IkReal(-1.00000000000000))*(x576))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x573)*(x576)))+(((IkReal(-1.00000000000000))*(x573)*(x575)))+(((IkReal(-1.00000000000000))*(x572)*(x574))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x574)))+(((IkReal(-2.41309350707268e-12))*(x573))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((npx)*(sj6)*(x572)))+(((IkReal(-1.00000000000000))*(x573)*(x574)))+(((x572)*(x575))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x577=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x577;
evalcond[3]=x577;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(414405822679.079))*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x578=IKsin(j4);
IkReal x579=IKcos(j4);
IkReal x580=((cj6)*(npy));
IkReal x581=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x580)))+(((IkReal(2.41309350707268e-12))*(x578)))+(((IkReal(-1.00000000000000))*(x581))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x579)*(x581)))+(((npz)*(x578)))+(((IkReal(-1.00000000000000))*(x579)*(x580))));
evalcond[2]=((((IkReal(2.41309350707268e-12))*(x579)))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x578)*(x580)))+(((npx)*(sj6)*(x578)))+(((npz)*(x579))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x582=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x582;
evalcond[3]=x582;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x583=((IkReal(414405822679.079))*(npx));
IkReal x584=((IkReal(414405822679.079))*(npy));
if( IKabs(((((cj6)*(x584)))+(((sj6)*(x583))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj6)*(x583)))+(((sj6)*(x584))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj6)*(x584)))+(((sj6)*(x583)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj6)*(x583)))+(((sj6)*(x584)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((cj6)*(x584)))+(((sj6)*(x583)))), ((((IkReal(-1.00000000000000))*(cj6)*(x583)))+(((sj6)*(x584)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x585=IKsin(j4);
IkReal x586=IKcos(j4);
IkReal x587=((IkReal(1.00000000000000))*(cj6));
IkReal x588=((npy)*(sj6));
IkReal x589=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x590=((npx)*(x585));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x585)))+(((IkReal(-1.00000000000000))*(npy)*(x587)))+(((IkReal(-1.00000000000000))*(x589))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x587)*(x590)))+(((IkReal(-1.00000000000000))*(x586)*(x589)))+(((x585)*(x588)))+(((IkReal(-1.00000000000000))*(npy)*(x586)*(x587))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(npx)*(x586)*(x587)))+(((sj6)*(x590)))+(((x586)*(x588)))+(((cj6)*(npy)*(x585))));
evalcond[3]=((x588)+(((IkReal(-1.00000000000000))*(npx)*(x587)))+(((IkReal(-2.41309350707268e-12))*(x586))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(npz));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x591=((IkReal(414405822679.079))*(npx));
IkReal x592=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj6)*(x591)))+(((cj6)*(x592))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj6)*(x592)))+(((cj6)*(x591))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj6)*(x591)))+(((cj6)*(x592)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj6)*(x592)))+(((cj6)*(x591)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((sj6)*(x591)))+(((cj6)*(x592)))), ((((IkReal(-1.00000000000000))*(sj6)*(x592)))+(((cj6)*(x591)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x593=IKsin(j4);
IkReal x594=IKcos(j4);
IkReal x595=((cj6)*(npx));
IkReal x596=((cj6)*(npy));
IkReal x597=((IkReal(1.00000000000000))*(npy)*(sj6));
IkReal x598=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x596)))+(((IkReal(-1.00000000000000))*(x598)))+(((IkReal(2.41309350707268e-12))*(x593))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x594)*(x596)))+(((IkReal(-1.00000000000000))*(x593)*(x597)))+(((IkReal(-1.00000000000000))*(x594)*(x598)))+(((x593)*(x595))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((x594)*(x595)))+(((IkReal(-1.00000000000000))*(x594)*(x597)))+(((npx)*(sj6)*(x593)))+(((x593)*(x596))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x595)))+(((npy)*(sj6)))+(((IkReal(2.41309350707268e-12))*(x594))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x599=((IkReal(2.28323001692700e+21))*(cj5));
IkReal x600=((cj6)*(npx));
IkReal x601=((npy)*(sj6));
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((x599)*(x601))))) != 0)?((IkReal)1/(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((x599)*(x601)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj6)*(sj6))))+(((IkReal(-1.89236762706040e+33))*(x600)*(x601)))+(((IkReal(-9.46183813530201e+32))*((cj6)*(cj6))*((npy)*(npy)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((((IKabs(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((x599)*(x601))))) != 0)?((IkReal)1/(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((x599)*(x601)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj6)*(sj6))))+(((IkReal(-1.89236762706040e+33))*(x600)*(x601)))+(((IkReal(-9.46183813530201e+32))*((cj6)*(cj6))*((npy)*(npy))))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((((IKabs(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((x599)*(x601))))) != 0)?((IkReal)1/(((((IkReal(-2.28323001692700e+21))*(npz)*(sj5)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((x599)*(x601)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj6)*(sj6))))+(((IkReal(-1.89236762706040e+33))*(x600)*(x601)))+(((IkReal(-9.46183813530201e+32))*((cj6)*(cj6))*((npy)*(npy))))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x602=IKsin(j4);
IkReal x603=IKcos(j4);
IkReal x604=((IkReal(1.00000000000000))*(cj6));
IkReal x605=((cj5)*(npx));
IkReal x606=((cj5)*(npy));
IkReal x607=((IkReal(1.00000000000000))*(npz));
IkReal x608=((IkReal(1.00000000000000))*(npx));
IkReal x609=((IkReal(2.41309350707268e-12))*(x603));
IkReal x610=((sj6)*(x603));
IkReal x611=((sj6)*(x602));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x602)))+(((IkReal(-1.00000000000000))*(sj6)*(x608)))+(((IkReal(-1.00000000000000))*(npy)*(x604))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x602)*(x607)))+(((IkReal(-1.00000000000000))*(x602)*(x604)*(x605)))+(((x606)*(x611)))+(((IkReal(-1.00000000000000))*(npy)*(x603)*(x604)))+(((IkReal(-1.00000000000000))*(x608)*(x610))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x609)))+(((IkReal(-0.250000000390712))*(cj5)))+(((IkReal(-1.00000000000000))*(x607))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(sj5)*(x603)*(x607)))+(((cj6)*(npy)*(x602)))+(((IkReal(-1.00000000000000))*(x603)*(x604)*(x605)))+(((npx)*(x611)))+(((x606)*(x610))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj5)))+(((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x609)))+(((IkReal(-1.00000000000000))*(npx)*(x604))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(cj6)*(npx)))+(((IkReal(7.69002526201833e+15))*(sj5)))+(((IkReal(3.07601010000000e+16))*(npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(cj6)*(npx)))+(((IkReal(7.69002526201833e+15))*(sj5)))+(((IkReal(3.07601010000000e+16))*(npy)*(sj6)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(1.34721866706185e-5))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(cj6)*(npx)))+(((IkReal(7.69002526201833e+15))*(sj5)))+(((IkReal(3.07601010000000e+16))*(npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x612=IKsin(j4);
IkReal x613=IKcos(j4);
IkReal x614=((IkReal(1.00000000000000))*(cj6));
IkReal x615=((cj5)*(npx));
IkReal x616=((cj5)*(npy));
IkReal x617=((IkReal(1.00000000000000))*(npz));
IkReal x618=((IkReal(1.00000000000000))*(npx));
IkReal x619=((IkReal(2.41309350707268e-12))*(x613));
IkReal x620=((sj6)*(x613));
IkReal x621=((sj6)*(x612));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj6)*(x618)))+(((IkReal(2.41309350707268e-12))*(x612)))+(((IkReal(-1.00000000000000))*(npy)*(x614))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x612)*(x617)))+(((x616)*(x621)))+(((IkReal(-1.00000000000000))*(x612)*(x614)*(x615)))+(((IkReal(-1.00000000000000))*(x618)*(x620)))+(((IkReal(-1.00000000000000))*(npy)*(x613)*(x614))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x619)))+(((IkReal(-1.00000000000000))*(x617)))+(((IkReal(-0.250000000390712))*(cj5))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(sj5)*(x613)*(x617)))+(((npx)*(x621)))+(((cj6)*(npy)*(x612)))+(((x616)*(x620)))+(((IkReal(-1.00000000000000))*(x613)*(x614)*(x615))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj5)))+(((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x619)))+(((IkReal(-1.00000000000000))*(npx)*(x614))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj5)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(1.34721866706185e-5))*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x622=IKsin(j4);
IkReal x623=IKcos(j4);
IkReal x624=((IkReal(1.00000000000000))*(cj6));
IkReal x625=((cj5)*(npx));
IkReal x626=((cj5)*(npy));
IkReal x627=((IkReal(1.00000000000000))*(npz));
IkReal x628=((IkReal(1.00000000000000))*(npx));
IkReal x629=((IkReal(2.41309350707268e-12))*(x623));
IkReal x630=((sj6)*(x623));
IkReal x631=((sj6)*(x622));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj6)*(x628)))+(((IkReal(2.41309350707268e-12))*(x622)))+(((IkReal(-1.00000000000000))*(npy)*(x624))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x628)*(x630)))+(((IkReal(-1.00000000000000))*(x622)*(x624)*(x625)))+(((IkReal(-1.00000000000000))*(sj5)*(x622)*(x627)))+(((IkReal(-1.00000000000000))*(npy)*(x623)*(x624)))+(((x626)*(x631))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(sj5)*(x629)))+(((IkReal(-0.250000000390712))*(cj5))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((npx)*(x631)))+(((IkReal(-1.00000000000000))*(x623)*(x624)*(x625)))+(((IkReal(-1.00000000000000))*(sj5)*(x623)*(x627)))+(((cj6)*(npy)*(x622)))+(((x626)*(x630))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj5)))+(((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj5)*(x629)))+(((IkReal(-1.00000000000000))*(npx)*(x624))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x632=((IkReal(0.0450000000000000))*(sj3));
IkReal x633=((IkReal(0.550000000000000))*(cj3));
IkReal x634=((cj6)*(npx));
IkReal x635=((npy)*(sj6));
IkReal x636=((x632)+(x633));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(x636)+(x634)+(((IkReal(-1.00000000000000))*(x635))));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(x636)))+(x635)+(((IkReal(-1.00000000000000))*(x634))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(-1.00000000000000))+(cj3)+(((IkReal(-12.2222222222222))*(sj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x637=((IkReal(1.00000000000000))*(cj6));
IkReal x638=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x637))));
evalcond[3]=((IkReal(0.850000000000000))+(((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(x638))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x637)))+(x638));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x639=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x639));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x639))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x640=IKcos(j4);
IkReal x641=((IkReal(1.00000000000000))*(x640));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x641)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x641)))+(((IkReal(-1.00000000000000))*(npz)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x642=((npy)*(sj6));
IkReal x643=((cj6)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(6.11991262292145))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(x642)))+(x643));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(x643)))+(x642));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IkReal(414405822679.079))*(npx)*(sj6)))+(((IkReal(414405822679.079))*(cj6)*(npy)))), ((IkReal(-414405822679.079))*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x644=IKsin(j4);
IkReal x645=IKcos(j4);
IkReal x646=((IkReal(1.00000000000000))*(npz));
IkReal x647=((cj6)*(npy));
IkReal x648=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((IkReal(-1.00000000000000))*(x647)))+(((IkReal(-1.00000000000000))*(x648)))+(((IkReal(2.41309350707268e-12))*(x644))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x645)*(x647)))+(((IkReal(-1.00000000000000))*(x644)*(x646)))+(((IkReal(-1.00000000000000))*(x645)*(x648))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x645)))+(((IkReal(-1.00000000000000))*(x646))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x644)*(x647)))+(((npx)*(sj6)*(x644)))+(((IkReal(-1.00000000000000))*(x645)*(x646))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x649=((IkReal(1.00000000000000))*(cj6));
IkReal x650=((npy)*(sj6));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npx)*(sj6)))+(((IkReal(-1.00000000000000))*(npy)*(x649))));
evalcond[3]=((IkReal(0.850000000000000))+(((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(x650))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(x650)+(((IkReal(-1.00000000000000))*(npx)*(x649))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x651=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x651));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x651))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x652=IKcos(j4);
IkReal x653=((IkReal(1.00000000000000))*(x652));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x653)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x653)))+(((IkReal(-1.00000000000000))*(npz)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)))))), ((gconst3)*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x654=IKsin(j4);
IkReal x655=IKcos(j4);
IkReal x656=((IkReal(0.550000000000000))*(sj3));
IkReal x657=((IkReal(1.00000000000000))*(npz));
IkReal x658=((IkReal(0.0450000000000000))*(cj3));
IkReal x659=((IkReal(1.00000000000000))*(cj6)*(npy));
IkReal x660=((IkReal(1.00000000000000))*(npx)*(sj6));
evalcond[0]=((((x654)*(x656)))+(((IkReal(-1.00000000000000))*(x659)))+(((IkReal(-1.00000000000000))*(x660)))+(((IkReal(0.0450000000000000))*(x654)))+(((IkReal(-1.00000000000000))*(x654)*(x658))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x655)*(x659)))+(((IkReal(-1.00000000000000))*(x655)*(x660)))+(((IkReal(-1.00000000000000))*(x654)*(x657))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x655)*(x656)))+(((IkReal(-1.00000000000000))*(x657)))+(((x655)*(x658)))+(((IkReal(-0.0450000000000000))*(x655))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((cj6)*(npy)*(x654)))+(((IkReal(-1.00000000000000))*(x655)*(x657)))+(((IkReal(-1.00000000000000))*(x656)))+(((npx)*(sj6)*(x654)))+(x658));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x661=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))+(((IkReal(0.550000000000000))*(cj3))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=x661;
evalcond[3]=x661;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x662=((IkReal(1.00000000000000))*(npx));
IkReal x663=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj6)*(x662)))+(((npy)*(sj6))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)*(x662)))+(((IkReal(-1.00000000000000))*(cj6)*(npy))));
evalcond[3]=x663;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x663;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x664=IKatan2(npz, ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x664));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x664))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x665=IKcos(j4);
IkReal x666=((IkReal(1.00000000000000))*(x665));
evalcond[0]=((((npz)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x666)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x666))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x667=((IkReal(1.00000000000000))*(npx));
IkReal x668=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj6)*(x667)))+(((npy)*(sj6))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)*(x667)))+(((IkReal(-1.00000000000000))*(cj6)*(npy))));
evalcond[3]=x668;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x668;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x669=IKatan2(npz, ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x669));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x669))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x670=IKcos(j4);
IkReal x671=((IkReal(1.00000000000000))*(x670));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x671)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x671)))+(((npz)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst4)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst4)*(npz)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x672=IKsin(j4);
IkReal x673=IKcos(j4);
IkReal x674=((IkReal(0.550000000000000))*(sj3));
IkReal x675=((IkReal(0.0450000000000000))*(cj3));
IkReal x676=((npx)*(sj6));
IkReal x677=((IkReal(1.00000000000000))*(cj6)*(npy));
evalcond[0]=((((IkReal(0.0450000000000000))*(x672)))+(((x672)*(x674)))+(((IkReal(-1.00000000000000))*(x676)))+(((IkReal(-1.00000000000000))*(x672)*(x675)))+(((IkReal(-1.00000000000000))*(x677))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x673)*(x676)))+(((npz)*(x672)))+(((IkReal(-1.00000000000000))*(x673)*(x677))));
evalcond[2]=((((x673)*(x674)))+(((IkReal(0.0450000000000000))*(x673)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x673)*(x675))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((cj6)*(npy)*(x672)))+(((x672)*(x676)))+(x675)+(((npz)*(x673)))+(((IkReal(-1.00000000000000))*(x674))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x678=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj3))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=x678;
evalcond[3]=x678;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x679=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x680=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x680))));
evalcond[3]=x679;
evalcond[4]=x679;
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x680))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x681=IKatan2(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x681));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x681))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x682=IKcos(j4);
IkReal x683=IKsin(j4);
IkReal x684=((IkReal(1.00000000000000))*(x682));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj6)*(x684)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x683)))+(((npy)*(sj6)*(x683)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x684))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x685=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x686=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x686))));
evalcond[3]=x685;
evalcond[4]=x685;
evalcond[5]=((((npy)*(sj6)))+(((IkReal(-1.00000000000000))*(cj6)*(x686))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x687=IKatan2(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x687));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x687))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x688=IKcos(j4);
IkReal x689=IKsin(j4);
IkReal x690=((IkReal(1.00000000000000))*(x688));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj6)*(npy)*(x690)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x690)))+(((IkReal(-1.00000000000000))*(cj6)*(npx)*(x689)))+(((npy)*(sj6)*(x689))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst5)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst5)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst5)*(((((IkReal(-1.00000000000000))*(cj6)*(npx)))+(((npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x691=IKsin(j4);
IkReal x692=IKcos(j4);
IkReal x693=((IkReal(0.550000000000000))*(sj3));
IkReal x694=((IkReal(0.0450000000000000))*(cj3));
IkReal x695=((npy)*(sj6));
IkReal x696=((IkReal(1.00000000000000))*(npx));
IkReal x697=((IkReal(1.00000000000000))*(cj6)*(npy));
IkReal x698=((cj6)*(x691));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj6)*(x696)))+(((x691)*(x693)))+(((IkReal(0.0450000000000000))*(x691)))+(((IkReal(-1.00000000000000))*(x697)))+(((IkReal(-1.00000000000000))*(x691)*(x694))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj6)*(x692)*(x696)))+(((IkReal(-1.00000000000000))*(x696)*(x698)))+(((x691)*(x695)))+(((IkReal(-1.00000000000000))*(x692)*(x697))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((npx)*(sj6)*(x691)))+(((x692)*(x695)))+(((IkReal(-1.00000000000000))*(cj6)*(x692)*(x696)))+(x694)+(((IkReal(-1.00000000000000))*(x693)))+(((npy)*(x698))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj6)*(x696)))+(((x692)*(x694)))+(((IkReal(-1.00000000000000))*(x692)*(x693)))+(x695)+(((IkReal(-0.0450000000000000))*(x692))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x699=((IkReal(0.0450000000000000))*(sj3));
IkReal x700=((IkReal(0.550000000000000))*(cj3));
IkReal x701=((x699)+(x700));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj3)))+(((IkReal(-0.325950000000000))*(cj3)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x701));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x701))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.550000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x702=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x702))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj6)*(x702)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x703=IKatan2(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x703));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x703))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x704=IKsin(j4);
IkReal x705=IKcos(j4);
IkReal x706=((IkReal(1.00000000000000))*(x705));
evalcond[0]=((((cj6)*(npx)*(x704)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x706)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x706)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)*(x704))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x707=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj6)*(npy)))+(((IkReal(-1.00000000000000))*(sj6)*(x707))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj6)*(x707)))+(((npy)*(sj6))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj6)*(npy)))+(((npx)*(sj6))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x708=IKatan2(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))), ((((cj6)*(npy)))+(((npx)*(sj6)))));
j4array[0]=((IkReal(-1.00000000000000))*(x708));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x708))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
IkReal x709=IKsin(j4);
IkReal x710=IKcos(j4);
IkReal x711=((IkReal(1.00000000000000))*(x710));
evalcond[0]=((((cj6)*(npx)*(x709)))+(((IkReal(-1.00000000000000))*(cj6)*(npy)*(x711)))+(((IkReal(-1.00000000000000))*(npx)*(sj6)*(x711)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)*(x709))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
if( IKabs(((gconst6)*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst6)*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((gconst6)*(((((cj6)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x712=IKsin(j4);
IkReal x713=IKcos(j4);
IkReal x714=((IkReal(0.550000000000000))*(sj3));
IkReal x715=((IkReal(0.0450000000000000))*(cj3));
IkReal x716=((cj6)*(npx));
IkReal x717=((cj6)*(npy));
IkReal x718=((IkReal(1.00000000000000))*(npx)*(sj6));
IkReal x719=((sj6)*(x712));
IkReal x720=((IkReal(1.00000000000000))*(npy)*(x713));
evalcond[0]=((((IkReal(-1.00000000000000))*(x717)))+(((IkReal(0.0450000000000000))*(x712)))+(((IkReal(-1.00000000000000))*(x718)))+(((IkReal(-1.00000000000000))*(x712)*(x715)))+(((x712)*(x714))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npy)*(x719)))+(((IkReal(-1.00000000000000))*(x713)*(x717)))+(((IkReal(-1.00000000000000))*(x713)*(x718)))+(((x712)*(x716))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x714)))+(((IkReal(-1.00000000000000))*(sj6)*(x720)))+(x715)+(((x712)*(x717)))+(((x713)*(x716)))+(((npx)*(x719))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x716)))+(((npy)*(sj6)))+(((IkReal(0.0450000000000000))*(x713)))+(((IkReal(-1.00000000000000))*(x713)*(x715)))+(((x713)*(x714))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x721=((IkReal(0.0450000000000000))*(cj3));
IkReal x722=((npx)*(sj6));
IkReal x723=((IkReal(0.550000000000000))*(sj3));
IkReal x724=((cj6)*(npy));
IkReal x725=((npz)*(sj5));
IkReal x726=((cj5)*(npx)*(npy));
IkReal x727=((cj5)*(cj6)*(sj6));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x721)))+(x723))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x721)))+(x723)))):(IkReal)1.0e30))*(((x722)+(x724))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((x721)*(x724)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-0.0450000000000000))*(x724)))+(((IkReal(-0.0450000000000000))*(x722)))+(((IkReal(-1.00000000000000))*(x722)*(x723))))) != 0)?((IkReal)1/(((((x721)*(x724)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-0.0450000000000000))*(x724)))+(((IkReal(-0.0450000000000000))*(x722)))+(((IkReal(-1.00000000000000))*(x722)*(x723)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x722)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x724)))+(((cj5)*(cj6)*(npx)*(x724)))+(((cj5)*(cj6)*(npx)*(x722)))+(((x722)*(x725)))+(((x724)*(x725))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x721)))+(x723))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x721)))+(x723)))):(IkReal)1.0e30))*(((x722)+(x724)))))+IKsqr(((((IKabs(((((x721)*(x724)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-0.0450000000000000))*(x724)))+(((IkReal(-0.0450000000000000))*(x722)))+(((IkReal(-1.00000000000000))*(x722)*(x723))))) != 0)?((IkReal)1/(((((x721)*(x724)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-0.0450000000000000))*(x724)))+(((IkReal(-0.0450000000000000))*(x722)))+(((IkReal(-1.00000000000000))*(x722)*(x723)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x722)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x724)))+(((cj5)*(cj6)*(npx)*(x724)))+(((cj5)*(cj6)*(npx)*(x722)))+(((x722)*(x725)))+(((x724)*(x725)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x721)))+(x723))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x721)))+(x723)))):(IkReal)1.0e30))*(((x722)+(x724)))), ((((IKabs(((((x721)*(x724)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-0.0450000000000000))*(x724)))+(((IkReal(-0.0450000000000000))*(x722)))+(((IkReal(-1.00000000000000))*(x722)*(x723))))) != 0)?((IkReal)1/(((((x721)*(x724)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-0.0450000000000000))*(x724)))+(((IkReal(-0.0450000000000000))*(x722)))+(((IkReal(-1.00000000000000))*(x722)*(x723)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x722)))+(((IkReal(-1.00000000000000))*(cj5)*(npy)*(sj6)*(x724)))+(((cj5)*(cj6)*(npx)*(x724)))+(((cj5)*(cj6)*(npx)*(x722)))+(((x722)*(x725)))+(((x724)*(x725)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x728=IKsin(j4);
IkReal x729=IKcos(j4);
IkReal x730=((IkReal(0.550000000000000))*(cj3));
IkReal x731=((npy)*(sj6));
IkReal x732=((IkReal(0.0450000000000000))*(cj3));
IkReal x733=((IkReal(1.00000000000000))*(npx));
IkReal x734=((cj6)*(npy));
IkReal x735=((cj5)*(sj3));
IkReal x736=((cj5)*(cj6));
IkReal x737=((IkReal(1.00000000000000))*(npz));
IkReal x738=((IkReal(0.550000000000000))*(sj3));
IkReal x739=((IkReal(0.0450000000000000))*(x729));
IkReal x740=((sj5)*(x729));
IkReal x741=((cj5)*(x729));
evalcond[0]=((((x728)*(x738)))+(((IkReal(-1.00000000000000))*(x734)))+(((IkReal(-1.00000000000000))*(sj6)*(x733)))+(((IkReal(0.0450000000000000))*(x728)))+(((IkReal(-1.00000000000000))*(x728)*(x732))));
evalcond[1]=((((cj5)*(x728)*(x731)))+(((IkReal(-1.00000000000000))*(sj6)*(x729)*(x733)))+(((IkReal(-1.00000000000000))*(x728)*(x733)*(x736)))+(((IkReal(-1.00000000000000))*(sj5)*(x728)*(x737)))+(((IkReal(-1.00000000000000))*(x729)*(x734))));
evalcond[2]=((((cj5)*(x730)))+(((IkReal(-1.00000000000000))*(x738)*(x740)))+(((IkReal(-1.00000000000000))*(x737)))+(((IkReal(0.0450000000000000))*(x735)))+(((x732)*(x740)))+(((IkReal(0.300000000000000))*(cj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x739))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((x728)*(x734)))+(((IkReal(-1.00000000000000))*(x737)*(x740)))+(((x731)*(x741)))+(((IkReal(-1.00000000000000))*(x729)*(x733)*(x736)))+(((IkReal(-1.00000000000000))*(x738)))+(((npx)*(sj6)*(x728)))+(x732));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj6)*(x733)))+(((x732)*(x741)))+(((IkReal(-0.0450000000000000))*(sj3)*(sj5)))+(((IkReal(-0.550000000000000))*(x729)*(x735)))+(((IkReal(-1.00000000000000))*(cj5)*(x739)))+(x731)+(((IkReal(-1.00000000000000))*(sj5)*(x730)))+(((IkReal(-0.300000000000000))*(sj5))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x742=((IkReal(0.0450000000000000))*(cj5));
IkReal x743=((IkReal(0.550000000000000))*(sj3));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x743))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x743)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj3)*(x742)))+(((IkReal(-1.00000000000000))*(cj5)*(x743)))+(((IkReal(-1.00000000000000))*(x742))))) != 0)?((IkReal)1/(((((cj3)*(x742)))+(((IkReal(-1.00000000000000))*(cj5)*(x743)))+(((IkReal(-1.00000000000000))*(x742)))))):(IkReal)1.0e30))*(((((cj6)*(npx)))+(((IkReal(0.550000000000000))*(cj3)*(sj5)))+(((IkReal(0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))+(((IkReal(0.0450000000000000))*(sj3)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x743))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x743)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))))+IKsqr(((((IKabs(((((cj3)*(x742)))+(((IkReal(-1.00000000000000))*(cj5)*(x743)))+(((IkReal(-1.00000000000000))*(x742))))) != 0)?((IkReal)1/(((((cj3)*(x742)))+(((IkReal(-1.00000000000000))*(cj5)*(x743)))+(((IkReal(-1.00000000000000))*(x742)))))):(IkReal)1.0e30))*(((((cj6)*(npx)))+(((IkReal(0.550000000000000))*(cj3)*(sj5)))+(((IkReal(0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))+(((IkReal(0.0450000000000000))*(sj3)*(sj5)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x743))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(x743)))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((((IKabs(((((cj3)*(x742)))+(((IkReal(-1.00000000000000))*(cj5)*(x743)))+(((IkReal(-1.00000000000000))*(x742))))) != 0)?((IkReal)1/(((((cj3)*(x742)))+(((IkReal(-1.00000000000000))*(cj5)*(x743)))+(((IkReal(-1.00000000000000))*(x742)))))):(IkReal)1.0e30))*(((((cj6)*(npx)))+(((IkReal(0.550000000000000))*(cj3)*(sj5)))+(((IkReal(0.300000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(npy)*(sj6)))+(((IkReal(0.0450000000000000))*(sj3)*(sj5)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x744=IKsin(j4);
IkReal x745=IKcos(j4);
IkReal x746=((IkReal(0.550000000000000))*(cj3));
IkReal x747=((npy)*(sj6));
IkReal x748=((IkReal(0.0450000000000000))*(cj3));
IkReal x749=((IkReal(1.00000000000000))*(npx));
IkReal x750=((cj6)*(npy));
IkReal x751=((cj5)*(sj3));
IkReal x752=((cj5)*(cj6));
IkReal x753=((IkReal(1.00000000000000))*(npz));
IkReal x754=((IkReal(0.550000000000000))*(sj3));
IkReal x755=((IkReal(0.0450000000000000))*(x745));
IkReal x756=((sj5)*(x745));
IkReal x757=((cj5)*(x745));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj6)*(x749)))+(((IkReal(0.0450000000000000))*(x744)))+(((x744)*(x754)))+(((IkReal(-1.00000000000000))*(x744)*(x748)))+(((IkReal(-1.00000000000000))*(x750))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x744)*(x753)))+(((cj5)*(x744)*(x747)))+(((IkReal(-1.00000000000000))*(x745)*(x750)))+(((IkReal(-1.00000000000000))*(sj6)*(x745)*(x749)))+(((IkReal(-1.00000000000000))*(x744)*(x749)*(x752))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x755)))+(((IkReal(0.0450000000000000))*(x751)))+(((IkReal(-1.00000000000000))*(x754)*(x756)))+(((IkReal(0.300000000000000))*(cj5)))+(((cj5)*(x746)))+(((IkReal(-1.00000000000000))*(x753)))+(((x748)*(x756))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((npx)*(sj6)*(x744)))+(((x747)*(x757)))+(((x744)*(x750)))+(((IkReal(-1.00000000000000))*(x745)*(x749)*(x752)))+(((IkReal(-1.00000000000000))*(x753)*(x756)))+(x748)+(((IkReal(-1.00000000000000))*(x754))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj6)*(x749)))+(((IkReal(-1.00000000000000))*(sj5)*(x746)))+(((IkReal(-1.00000000000000))*(cj5)*(x755)))+(((IkReal(-0.0450000000000000))*(sj3)*(sj5)))+(x747)+(((IkReal(-0.550000000000000))*(x745)*(x751)))+(((x748)*(x757)))+(((IkReal(-0.300000000000000))*(sj5))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x758=((IkReal(0.550000000000000))*(sj3));
IkReal x759=((IkReal(0.0450000000000000))*(cj3));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(x758)+(((IkReal(-1.00000000000000))*(x759))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(x758)+(((IkReal(-1.00000000000000))*(x759)))))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((sj5)*(x759))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((sj5)*(x759)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(npz)+(((IkReal(-0.0450000000000000))*(cj5)*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(x758)+(((IkReal(-1.00000000000000))*(x759))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(x758)+(((IkReal(-1.00000000000000))*(x759)))))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))))+IKsqr(((((IKabs(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((sj5)*(x759))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((sj5)*(x759)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(npz)+(((IkReal(-0.0450000000000000))*(cj5)*(sj3)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j4array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(x758)+(((IkReal(-1.00000000000000))*(x759))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(x758)+(((IkReal(-1.00000000000000))*(x759)))))):(IkReal)1.0e30))*(((((cj6)*(npy)))+(((npx)*(sj6)))))), ((((IKabs(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((sj5)*(x759))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((sj5)*(x759)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj5)))+(((IkReal(-0.550000000000000))*(cj3)*(cj5)))+(npz)+(((IkReal(-0.0450000000000000))*(cj5)*(sj3)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x760=IKsin(j4);
IkReal x761=IKcos(j4);
IkReal x762=((IkReal(0.550000000000000))*(cj3));
IkReal x763=((npy)*(sj6));
IkReal x764=((IkReal(0.0450000000000000))*(cj3));
IkReal x765=((IkReal(1.00000000000000))*(npx));
IkReal x766=((cj6)*(npy));
IkReal x767=((cj5)*(sj3));
IkReal x768=((cj5)*(cj6));
IkReal x769=((IkReal(1.00000000000000))*(npz));
IkReal x770=((IkReal(0.550000000000000))*(sj3));
IkReal x771=((IkReal(0.0450000000000000))*(x761));
IkReal x772=((sj5)*(x761));
IkReal x773=((cj5)*(x761));
evalcond[0]=((((IkReal(0.0450000000000000))*(x760)))+(((IkReal(-1.00000000000000))*(x766)))+(((IkReal(-1.00000000000000))*(sj6)*(x765)))+(((IkReal(-1.00000000000000))*(x760)*(x764)))+(((x760)*(x770))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x761)*(x766)))+(((cj5)*(x760)*(x763)))+(((IkReal(-1.00000000000000))*(sj6)*(x761)*(x765)))+(((IkReal(-1.00000000000000))*(sj5)*(x760)*(x769)))+(((IkReal(-1.00000000000000))*(x760)*(x765)*(x768))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj5)*(x771)))+(((x764)*(x772)))+(((IkReal(-1.00000000000000))*(x769)))+(((cj5)*(x762)))+(((IkReal(0.0450000000000000))*(x767)))+(((IkReal(0.300000000000000))*(cj5)))+(((IkReal(-1.00000000000000))*(x770)*(x772))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((npx)*(sj6)*(x760)))+(((x760)*(x766)))+(((IkReal(-1.00000000000000))*(x761)*(x765)*(x768)))+(((IkReal(-1.00000000000000))*(x769)*(x772)))+(((x763)*(x773)))+(x764)+(((IkReal(-1.00000000000000))*(x770))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj6)*(x765)))+(((x764)*(x773)))+(((IkReal(-1.00000000000000))*(cj5)*(x771)))+(((IkReal(-0.0450000000000000))*(sj3)*(sj5)))+(x763)+(((IkReal(-1.00000000000000))*(sj5)*(x762)))+(((IkReal(-0.300000000000000))*(sj5)))+(((IkReal(-0.550000000000000))*(x761)*(x767))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x105=((IkReal(1.00000000000000))*(cj3));
IkReal x106=((IkReal(1.00000000000000))*(sj6));
IkReal x107=((IkReal(1.00000000000000))*(sj3));
IkReal x108=((IkReal(1.00000000000000))*(cj5));
IkReal x109=((IkReal(-1.00000000000000))*(sj4));
IkReal x110=((IkReal(-1.00000000000000))*(cj5));
IkReal x111=((((IkReal(-1.00000000000000))*(r01)*(x106)))+(((cj6)*(r00))));
IkReal x112=((((r00)*(sj6)))+(((cj6)*(r01))));
IkReal x113=((((IkReal(-1.00000000000000))*(r11)*(x106)))+(((cj6)*(r10))));
IkReal x114=((((r10)*(sj6)))+(((cj6)*(r11))));
IkReal x115=((((IkReal(-1.00000000000000))*(r21)*(x106)))+(((cj6)*(r20))));
IkReal x116=((((cj6)*(r21)))+(((r20)*(sj6))));
IkReal x117=((sj5)*(x113));
IkReal x118=((sj5)*(x115));
IkReal x119=((sj4)*(x116));
IkReal x120=((((r02)*(sj5)))+(((cj5)*(x111))));
IkReal x121=((((sj5)*(x111)))+(((IkReal(-1.00000000000000))*(r02)*(x108))));
IkReal x122=((((r12)*(sj5)))+(((cj5)*(x113))));
IkReal x123=((((r22)*(sj5)))+(((cj5)*(x115))));
IkReal x124=((cj4)*(x122));
IkReal x125=((cj4)*(x123));
IkReal x126=((IkReal(1.00000000000000))*(x121));
IkReal x127=((((x109)*(x112)))+(((cj4)*(x120))));
new_r00=((((cj3)*(x127)))+(((IkReal(-1.00000000000000))*(x107)*(x121))));
new_r01=((((cj4)*(x112)))+(((sj4)*(x120))));
new_r02=((((IkReal(-1.00000000000000))*(x107)*(x127)))+(((IkReal(-1.00000000000000))*(x105)*(x121))));
new_r10=((((cj3)*(((((IkReal(-1.00000000000000))*(sj4)*(x114)))+(x124)))))+(((IkReal(-1.00000000000000))*(x107)*(((x117)+(((r12)*(x110))))))));
new_r11=((((cj4)*(x114)))+(((sj4)*(x122))));
new_r12=((((IkReal(-1.00000000000000))*(x105)*(((((IkReal(-1.00000000000000))*(r12)*(x108)))+(x117)))))+(((IkReal(-1.00000000000000))*(x107)*(((((x109)*(x114)))+(x124))))));
new_r20=((((IkReal(-1.00000000000000))*(x107)*(((((r22)*(x110)))+(x118)))))+(((cj3)*(((x125)+(((IkReal(-1.00000000000000))*(x119))))))));
new_r21=((((cj4)*(x116)))+(((sj4)*(x123))));
new_r22=((((IkReal(-1.00000000000000))*(x107)*(((((x109)*(x116)))+(x125)))))+(((IkReal(-1.00000000000000))*(x105)*(((x118)+(((IkReal(-1.00000000000000))*(r22)*(x108))))))));
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
cj1array[0]=new_r22;
if( cj1array[0] >= -1-IKFAST_SINCOS_THRESH && cj1array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j1valid[0] = j1valid[1] = true;
    j1array[0] = IKacos(cj1array[0]);
    sj1array[0] = IKsin(j1array[0]);
    cj1array[1] = cj1array[0];
    j1array[1] = -j1array[0];
    sj1array[1] = -sj1array[0];
}
else if( isnan(cj1array[0]) )
{
    // probably any value will work
    j1valid[0] = true;
    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
}
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst11;
gconst11=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst10;
gconst10=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x128=IKatan2(new_r02, new_r12);
j0array[0]=((IkReal(-1.00000000000000))*(x128));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x128))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j0))))+(((new_r12)*(IKcos(j0)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj0)))+(((cj0)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))))+IKsqr(((((new_r10)*(sj0)))+(((cj0)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))), ((((new_r10)*(sj0)))+(((cj0)*(new_r00)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x129=IKsin(j2);
IkReal x130=((IkReal(1.00000000000000))*(sj0));
IkReal x131=((IkReal(1.00000000000000))*(IKcos(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x130)))+(((IkReal(-1.00000000000000))*(x129)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x130)))+(((IkReal(-1.00000000000000))*(x131)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(x129)+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(((IkReal(-1.00000000000000))*(x131)))+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x132=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x132;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x132;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x133=IKatan2(new_r02, new_r12);
j0array[0]=((IkReal(-1.00000000000000))*(x133));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x133))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j0))))+(((new_r12)*(IKcos(j0)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((new_r11)*(sj0)))+(((cj0)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))), ((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x134=IKcos(j2);
IkReal x135=((IkReal(1.00000000000000))*(sj0));
IkReal x136=((IkReal(1.00000000000000))*(IKsin(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x135)))+(((IkReal(-1.00000000000000))*(x136)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x135)))+(((IkReal(-1.00000000000000))*(x134)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(((IkReal(-1.00000000000000))*(x136)))+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(x134)+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x137=((gconst10)*(sj1));
if( IKabs(((new_r12)*(x137))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x137))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r12)*(x137)), ((new_r02)*(x137)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[6];
IkReal x138=IKcos(j0);
IkReal x139=IKsin(j0);
IkReal x140=((IkReal(1.00000000000000))*(cj1));
IkReal x141=((IkReal(1.00000000000000))*(sj1));
IkReal x142=((new_r12)*(x139));
IkReal x143=((new_r02)*(x138));
evalcond[0]=((((new_r12)*(x138)))+(((IkReal(-1.00000000000000))*(new_r02)*(x139))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x141)))+(x142)+(x143));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r22)*(x141)))+(((cj1)*(x142)))+(((cj1)*(x143))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r20)*(x140)))+(((IkReal(-1.00000000000000))*(new_r10)*(x139)*(x141)))+(((IkReal(-1.00000000000000))*(new_r00)*(x138)*(x141))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r01)*(x138)*(x141)))+(((IkReal(-1.00000000000000))*(new_r21)*(x140)))+(((IkReal(-1.00000000000000))*(new_r11)*(x139)*(x141))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x140)))+(((IkReal(-1.00000000000000))*(x141)*(x143)))+(((IkReal(-1.00000000000000))*(x141)*(x142))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst12;
gconst12=IKsign(sj1);
dummyeval[0]=sj1;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x144=((((IkReal(-1.00000000000000))*(new_r02)*(sj0)))+(((cj0)*(new_r12))));
IkReal x145=((((new_r12)*(sj0)))+(((cj0)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x144;
evalcond[5]=x144;
evalcond[6]=x145;
evalcond[7]=x145;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj0)))+(((cj0)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))))+IKsqr(((((new_r10)*(sj0)))+(((cj0)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj0)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj0)))), ((((new_r10)*(sj0)))+(((cj0)*(new_r00)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x146=IKsin(j2);
IkReal x147=((IkReal(1.00000000000000))*(sj0));
IkReal x148=((IkReal(1.00000000000000))*(IKcos(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(x146)))+(((cj0)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x147))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x147)))+(((IkReal(-1.00000000000000))*(x148)))+(((cj0)*(new_r11))));
evalcond[2]=((((new_r11)*(sj0)))+(x146)+(((cj0)*(new_r01))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x148)))+(((new_r10)*(sj0)))+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x149=((IkReal(1.00000000000000))+(new_r22));
IkReal x150=((new_r12)*(sj0));
IkReal x151=((cj0)*(new_r02));
IkReal x152=((((IkReal(-1.00000000000000))*(new_r02)*(sj0)))+(((cj0)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x149;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x152;
evalcond[5]=x152;
evalcond[6]=((x151)+(x150));
evalcond[7]=((((IkReal(-1.00000000000000))*(x151)))+(((IkReal(-1.00000000000000))*(x150))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x149;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((((new_r11)*(sj0)))+(((cj0)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((new_r11)*(sj0)))+(((cj0)*(new_r01)))), ((((IkReal(-1.00000000000000))*(cj0)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj0)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[4];
IkReal x153=IKcos(j2);
IkReal x154=((IkReal(1.00000000000000))*(sj0));
IkReal x155=((IkReal(1.00000000000000))*(IKsin(j2)));
evalcond[0]=((((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(new_r00)*(x154)))+(((cj0)*(new_r10))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x153)))+(((cj0)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x154))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x155)))+(((new_r11)*(sj0)))+(((cj0)*(new_r01))));
evalcond[3]=((((new_r10)*(sj0)))+(x153)+(((cj0)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst12)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst12)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst12)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst12)*(new_r20)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[8];
IkReal x156=IKsin(j2);
IkReal x157=IKcos(j2);
IkReal x158=((IkReal(1.00000000000000))*(sj0));
IkReal x159=((IkReal(1.00000000000000))*(sj1));
IkReal x160=((cj0)*(new_r01));
IkReal x161=((cj0)*(new_r00));
IkReal x162=((new_r11)*(sj0));
IkReal x163=((new_r10)*(sj0));
IkReal x164=((IkReal(1.00000000000000))*(x157));
evalcond[0]=((((sj1)*(x157)))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(x156)*(x159)))+(new_r21));
evalcond[2]=((((IkReal(-1.00000000000000))*(x156)))+(((IkReal(-1.00000000000000))*(new_r00)*(x158)))+(((cj0)*(new_r10))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x164)))+(((cj0)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x158))));
evalcond[4]=((x162)+(x160)+(((cj1)*(x156))));
evalcond[5]=((x163)+(x161)+(((IkReal(-1.00000000000000))*(cj1)*(x164))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r21)*(x159)))+(x156)+(((cj1)*(x162)))+(((cj1)*(x160))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x159)))+(((IkReal(-1.00000000000000))*(x164)))+(((cj1)*(x163)))+(((cj1)*(x161))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst11)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst11)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst11)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst11)*(new_r20)));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
evalcond[0]=((((sj1)*(IKcos(j2))))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj1)*(IKsin(j2))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst13;
gconst13=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x165=((gconst13)*(sj1));
if( IKabs(((new_r12)*(x165))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x165))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((new_r12)*(x165)), ((new_r02)*(x165)));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[12];
IkReal x166=IKcos(j0);
IkReal x167=IKsin(j0);
IkReal x168=((IkReal(1.00000000000000))*(cj2));
IkReal x169=((IkReal(1.00000000000000))*(sj1));
IkReal x170=((IkReal(1.00000000000000))*(cj1));
IkReal x171=((new_r10)*(x167));
IkReal x172=((new_r01)*(x166));
IkReal x173=((new_r00)*(x166));
IkReal x174=((IkReal(1.00000000000000))*(x167));
IkReal x175=((new_r11)*(x167));
IkReal x176=((new_r12)*(x167));
IkReal x177=((new_r02)*(x166));
evalcond[0]=((((new_r12)*(x166)))+(((IkReal(-1.00000000000000))*(new_r02)*(x174))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x169)))+(x177)+(x176));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj2)))+(((new_r10)*(x166)))+(((IkReal(-1.00000000000000))*(new_r00)*(x174))));
evalcond[3]=((((new_r11)*(x166)))+(((IkReal(-1.00000000000000))*(x168)))+(((IkReal(-1.00000000000000))*(new_r01)*(x174))));
evalcond[4]=((((cj1)*(sj2)))+(x172)+(x175));
evalcond[5]=((x171)+(x173)+(((IkReal(-1.00000000000000))*(cj1)*(x168))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r22)*(x169)))+(((cj1)*(x177)))+(((cj1)*(x176))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x170)))+(((IkReal(-1.00000000000000))*(x169)*(x171)))+(((IkReal(-1.00000000000000))*(x169)*(x173))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r21)*(x170)))+(((IkReal(-1.00000000000000))*(x169)*(x175)))+(((IkReal(-1.00000000000000))*(x169)*(x172))));
evalcond[9]=((sj2)+(((IkReal(-1.00000000000000))*(new_r21)*(x169)))+(((cj1)*(x172)))+(((cj1)*(x175))));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x170)))+(((IkReal(-1.00000000000000))*(x169)*(x176)))+(((IkReal(-1.00000000000000))*(x169)*(x177))));
evalcond[11]=((((IkReal(-1.00000000000000))*(x168)))+(((IkReal(-1.00000000000000))*(new_r20)*(x169)))+(((cj1)*(x173)))+(((cj1)*(x171))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}static inline void polyroots6(IkReal rawcoeffs[6+1], IkReal rawroots[6], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[6];
    const int maxsteps = 110;
    for(int i = 0; i < 6; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[6];
    IkReal err[6];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 6; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 6; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 6; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 6; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[6] = {false};
    for(int i = 0; i < 6; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 6; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3238550ee83defc84c543b45c2db9325"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
