/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:51:17.116063
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[2]);
x3=IKsin(j[3]);
x4=IKcos(j[3]);
x5=IKsin(j[1]);
x6=IKsin(j[5]);
x7=IKsin(j[4]);
x8=IKcos(j[4]);
x9=IKcos(j[5]);
x10=((IkReal(0.707106781186548))*(IKcos(j[0])));
x11=((IkReal(0.707106781186548))*(IKsin(j[0])));
x12=((IkReal(0.707106781186548))*(x6));
x13=((IkReal(0.707106781186548))*(x9));
x14=((IkReal(0.500001138018525))*(x9));
x15=((IkReal(0.499998861978885))*(x9));
x16=((IkReal(0.499998861978885))*(x6));
x17=((IkReal(1.00000000000000))*(x1));
x18=((IkReal(0.500001138018525))*(x6));
x19=((IkReal(1.00000000000000))*(x3));
x20=((x0)*(x1));
x21=((x0)*(x2));
x22=((x4)*(x5));
x23=((IkReal(-1.00000000000000))*(x2));
x24=((x3)*(x5));
x25=((IkReal(-1.00000000000000))*(x3));
x26=((((IkReal(-1.00000000000000))*(x10)))+(x11));
x27=((((IkReal(-1.00000000000000))*(x11)))+(x10));
x28=((((IkReal(-1.00000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(x10))));
x29=((x11)+(x10));
x30=((x26)*(x5));
x31=((IkReal(-1.00000000000000))*(x27));
x32=((x2)*(x27));
x33=((((IkReal(1.00000000000000))*(x0)*(x4)))+(((IkReal(-1.00000000000000))*(x17)*(x24))));
x34=((x33)*(x9));
x35=((((x8)*(((((IkReal(-1.00000000000000))*(x0)*(x19)))+(((IkReal(-1.00000000000000))*(x17)*(x22)))))))+(((x2)*(x5)*(x7))));
x36=((x35)*(x6));
x37=((((IkReal(-1.00000000000000))*(x22)*(x28)))+(((x19)*(((((x0)*(x17)*(x29)))+(((IkReal(1.00000000000000))*(x32))))))));
x38=((((x25)*(((((x20)*(x31)))+(((x23)*(x28)))))))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x39=((x37)*(x9));
x40=((((x8)*(((((x4)*(((((x2)*(x28)))+(((x20)*(x27)))))))+(((x24)*(x26)))))))+(((x7)*(((((x21)*(x26)))+(((x1)*(x28))))))));
x41=((((x8)*(((((x4)*(((x32)+(((x20)*(x29)))))))+(((x24)*(x28)))))))+(((x7)*(((((x21)*(x28)))+(((x1)*(x27))))))));
eerot[0]=((((((((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))+(((IkReal(0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))))*(IKcos(j[6])))));
eerot[1]=((((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707108390587779))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6])))));
eerot[2]=((((IkReal(-0.707108390587779))*(x36)))+(((IkReal(-0.707108390587779))*(x34)))+(((x16)*(x40)))+(((x15)*(x38)))+(((IkReal(-1.00000000000000))*(x16)*(x41)))+(((IkReal(-1.00000000000000))*(x15)*(x37))));
eetrans[0]=((IkReal(-0.350177000000000))+(((IkReal(0.212132517176334))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKcos(j[0]))))+(((IkReal(-0.106065775767248))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318198775764501))*(IKcos(j[2]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKsin(j[0]))))+(((IkReal(0.106065775767248))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(-0.388909614823279))*(IKcos(j[1]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((IkReal(-0.212132517176334))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(-0.125829938105095))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829938105095))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.125829938105095))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707108390587779))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1])))));
eerot[3]=((((((((IkReal(0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6])))));
eerot[4]=((((((((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6])))));
eerot[5]=((((IkReal(-0.707105171781653))*(x34)))+(((IkReal(-0.707105171781653))*(x36)))+(((IkReal(-1.00000000000000))*(x18)*(x40)))+(((x18)*(x41)))+(((IkReal(-1.00000000000000))*(x14)*(x38)))+(((x14)*(((((x25)*(((((x23)*(x27)))+(((IkReal(-1.00000000000000))*(x20)*(x29)))))))+(((IkReal(-1.00000000000000))*(x22)*(x28))))))));
eetrans[1]=((IkReal(-0.350177000000000))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318197327301744))*(IKcos(j[2]))*(IKsin(j[1]))))+(((IkReal(0.212131551534496))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829365318545))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.212131551534496))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKcos(j[0]))))+(((IkReal(-0.0159099387882250))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.388907844479909))*(IKcos(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(-0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3])))));
eerot[6]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6])))));
eerot[7]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6])))));
eerot[8]=((((x12)*(x40)))+(((x12)*(x41)))+(((x13)*(x37)))+(((x13)*(x38))));
eetrans[2]=((IkReal(0.378361000000000))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.275000000000000))*(IKcos(j[0]))))+(((IkReal(0.275000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.150000000000000))*(IKcos(j[0]))))+(((IkReal(0.150000000000000))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.275000000000000))*(IKsin(j[0]))))+(((IkReal(0.275000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.150000000000000))*(IKsin(j[0]))))+(((IkReal(0.150000000000000))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1])))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {6}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j7,cj7,sj7,htj7,j8,cj8,sj8,htj8,j9,cj9,sj9,htj9,j10,cj10,sj10,htj10,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij7[2], _nj7,_ij8[2], _nj8,_ij9[2], _nj9,_ij10[2], _nj10;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1; j7=numeric_limits<IkReal>::quiet_NaN(); _ij7[0] = -1; _ij7[1] = -1; _nj7 = -1; j8=numeric_limits<IkReal>::quiet_NaN(); _ij8[0] = -1; _ij8[1] = -1; _nj8 = -1; j9=numeric_limits<IkReal>::quiet_NaN(); _ij9[0] = -1; _ij9[1] = -1; _nj9 = -1;  _ij10[0] = -1; _ij10[1] = -1; _nj10 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j10=pfree[0]; cj10=cos(pfree[0]); sj10=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=((((IkReal(-0.500001138018525))*(r11)))+(((IkReal(0.499998861978885))*(r01)))+(((IkReal(0.707106781186548))*(r21))));
new_r01=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(0.500001138018525))*(r10)))+(((IkReal(-0.499998861978885))*(r00))));
new_r02=((((IkReal(-0.500001138018525))*(r12)))+(((IkReal(0.499998861978885))*(r02)))+(((IkReal(0.707106781186548))*(r22))));
new_px=((IkReal(-0.267542425853256))+(((IkReal(0.499998861978885))*(px)))+(((IkReal(-0.500001138018525))*(py)))+(((IkReal(-0.0889747974891426))*(r02)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.0889752025103964))*(r12))));
new_r10=((((IkReal(0.500001138018525))*(r11)))+(((IkReal(0.707106781186548))*(r21)))+(((IkReal(-0.499998861978885))*(r01))));
new_r11=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(-0.500001138018525))*(r10)))+(((IkReal(0.499998861978885))*(r00))));
new_r12=((((IkReal(0.500001138018525))*(r12)))+(((IkReal(0.707106781186548))*(r22)))+(((IkReal(-0.499998861978885))*(r02))));
new_py=((IkReal(-0.267540831819791))+(((IkReal(-0.499998861978885))*(px)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.500001138018525))*(py)))+(((IkReal(0.0889747974891426))*(r02)))+(((IkReal(-0.0889752025103964))*(r12))));
new_r20=((((IkReal(-0.707108390587779))*(r01)))+(((IkReal(-0.707105171781653))*(r11))));
new_r21=((((IkReal(0.707108390587779))*(r00)))+(((IkReal(0.707105171781653))*(r10))));
new_r22=((((IkReal(-0.707108390587779))*(r02)))+(((IkReal(-0.707105171781653))*(r12))));
new_pz=((IkReal(-0.495225062629841))+(((IkReal(0.125829365318545))*(r12)))+(((IkReal(-0.707105171781653))*(py)))+(((IkReal(0.125829938105095))*(r02)))+(((IkReal(-0.707108390587779))*(px))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j7array[2], cj7array[2], sj7array[2];
bool j7valid[2]={false};
_nj7 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x42=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j7array[0]=((IkReal(1.80132261653346))+(((IkReal(1.00000000000000))*(x42))));
sj7array[0]=IKsin(j7array[0]);
cj7array[0]=IKcos(j7array[0]);
j7array[1]=((IkReal(4.94291527012326))+(((IkReal(-1.00000000000000))*(x42))));
sj7array[1]=IKsin(j7array[1]);
cj7array[1]=IKcos(j7array[1]);
if( j7array[0] > IKPI )
{
    j7array[0]-=IK2PI;
}
else if( j7array[0] < -IKPI )
{    j7array[0]+=IK2PI;
}
j7valid[0] = true;
if( j7array[1] > IKPI )
{
    j7array[1]-=IK2PI;
}
else if( j7array[1] < -IKPI )
{    j7array[1]+=IK2PI;
}
j7valid[1] = true;
for(int ij7 = 0; ij7 < 2; ++ij7)
{
if( !j7valid[ij7] )
{
    continue;
}
_ij7[0] = ij7; _ij7[1] = -1;
for(int iij7 = ij7+1; iij7 < 2; ++iij7)
{
if( j7valid[iij7] && IKabs(cj7array[ij7]-cj7array[iij7]) < IKFAST_SOLUTION_THRESH && IKabs(sj7array[ij7]-sj7array[iij7]) < IKFAST_SOLUTION_THRESH )
{
    j7valid[iij7]=false; _ij7[1] = iij7; break; 
}
}
j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];

{
IkReal dummyeval[1];
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(20.0000000000000))*((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-40.0000000000000))*(cj10)*(npx)*(npy)*(sj10)))+(((IkReal(20.0000000000000))*((npz)*(npz))))+(((IkReal(20.0000000000000))*((cj10)*(cj10))*((npx)*(npx))))));
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[1], cj9array[1], sj9array[1];
bool j9valid[1]={false};
_nj9 = 1;
if( IKabs(((gconst2)*(((((IkReal(-17.0000000000000))*(cj10)*(npx)))+(((IkReal(17.0000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(17.0000000000000))*(gconst2)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j9array[0]=IKatan2(((gconst2)*(((((IkReal(-17.0000000000000))*(cj10)*(npx)))+(((IkReal(17.0000000000000))*(npy)*(sj10)))))), ((IkReal(17.0000000000000))*(gconst2)*(npz)));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
for(int ij9 = 0; ij9 < 1; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 1; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];
{
IkReal evalcond[2];
IkReal x43=IKsin(j9);
IkReal x44=IKcos(j9);
IkReal x45=((npy)*(sj10));
IkReal x46=((cj10)*(npx));
IkReal x47=((IkReal(1.00000000000000))*(x43));
evalcond[0]=((IkReal(-0.850000000000000))+(((npz)*(x44)))+(((x43)*(x45)))+(((IkReal(-1.00000000000000))*(x46)*(x47))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x44)*(x46)))+(((x44)*(x45)))+(((IkReal(-1.00000000000000))*(npz)*(x47))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x48=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x48));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x48))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x49=IKsin(j8);
IkReal x50=IKcos(j8);
IkReal x51=((IkReal(1.00000000000000))*(x50));
IkReal x52=((IkReal(1.00000000000000))*(x49));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj9)*(x52)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x51)))+(((cj9)*(npy)*(sj10)*(x49)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x51)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x52))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((((IkReal(20.0000000000000))*((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-40.0000000000000))*(cj10)*(npx)*(npy)*(sj10)))+(((IkReal(20.0000000000000))*((npz)*(npz))))+(((IkReal(20.0000000000000))*((cj10)*(cj10))*((npx)*(npx))))));
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[1], cj9array[1], sj9array[1];
bool j9valid[1]={false};
_nj9 = 1;
if( IKabs(((gconst3)*(((((IkReal(-17.0000000000000))*(cj10)*(npx)))+(((IkReal(17.0000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(17.0000000000000))*(gconst3)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j9array[0]=IKatan2(((gconst3)*(((((IkReal(-17.0000000000000))*(cj10)*(npx)))+(((IkReal(17.0000000000000))*(npy)*(sj10)))))), ((IkReal(17.0000000000000))*(gconst3)*(npz)));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
for(int ij9 = 0; ij9 < 1; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 1; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];
{
IkReal evalcond[2];
IkReal x287=IKsin(j9);
IkReal x288=IKcos(j9);
IkReal x289=((npy)*(sj10));
IkReal x290=((cj10)*(npx));
IkReal x291=((IkReal(1.00000000000000))*(x287));
evalcond[0]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(x290)*(x291)))+(((npz)*(x288)))+(((x287)*(x289))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x288)*(x290)))+(((IkReal(-1.00000000000000))*(npz)*(x291)))+(((x288)*(x289))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x292=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x292));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x292))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x293=IKsin(j8);
IkReal x294=IKcos(j8);
IkReal x295=((IkReal(1.00000000000000))*(x294));
IkReal x296=((IkReal(1.00000000000000))*(x293));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(npy)*(x295)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x295)))+(((cj9)*(npy)*(sj10)*(x293)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x296)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x296))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
sj8array[0]=((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))));
if( sj8array[0] >= -1-IKFAST_SINCOS_THRESH && sj8array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j8valid[0] = j8valid[1] = true;
    j8array[0] = IKasin(sj8array[0]);
    cj8array[0] = IKcos(j8array[0]);
    sj8array[1] = sj8array[0];
    j8array[1] = j8array[0] > 0 ? (IKPI-j8array[0]) : (-IKPI-j8array[0]);
    cj8array[1] = -cj8array[0];
}
else if( isnan(sj8array[0]) )
{
    // probably any value will work
    j8valid[0] = true;
    cj8array[0] = 1; sj8array[0] = 0; j8array[0] = 0;
}
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(-3.07601010000000e+16))*((cj10)*(cj10))*((npx)*(npx))))+(((IkReal(-3.07601010000000e+16))*((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(6.15202020000000e+16))*(cj10)*(npx)*(npy)*(sj10)))+(((IkReal(-3.07601010000000e+16))*((npz)*(npz))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((npz)*(npz))))+(((IkReal(-1.00000000000000))*((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+(((IkReal(-1.00000000000000))*((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[1], cj9array[1], sj9array[1];
bool j9valid[1]={false};
_nj9 = 1;
IkReal x297=((npy)*(sj10));
IkReal x298=((cj10)*(npx));
IkReal x299=((IkReal(74227.0000000000))*(cj8));
if( IKabs(((gconst4)*(((((IkReal(7.69002526201833e+15))*(x297)))+(((IkReal(-7.69002526201833e+15))*(x298)))+(((npz)*(x299))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x297)*(x299)))+(((x298)*(x299)))+(((IkReal(7.69002526201833e+15))*(npz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j9array[0]=IKatan2(((gconst4)*(((((IkReal(7.69002526201833e+15))*(x297)))+(((IkReal(-7.69002526201833e+15))*(x298)))+(((npz)*(x299)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x297)*(x299)))+(((x298)*(x299)))+(((IkReal(7.69002526201833e+15))*(npz)))))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
for(int ij9 = 0; ij9 < 1; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 1; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];
{
IkReal evalcond[4];
IkReal x300=IKsin(j9);
IkReal x301=IKcos(j9);
IkReal x302=((cj10)*(npy));
IkReal x303=((IkReal(1.00000000000000))*(cj8));
IkReal x304=((npx)*(sj10));
IkReal x305=((IkReal(1.00000000000000))*(x300));
IkReal x306=((npy)*(sj10)*(x301));
IkReal x307=((IkReal(1.00000000000000))*(cj10)*(npx)*(x301));
evalcond[0]=((IkReal(0.250000000390712))+(((npy)*(sj10)*(x300)))+(((IkReal(-1.00000000000000))*(cj10)*(npx)*(x305)))+(((npz)*(x301))));
evalcond[1]=((((IkReal(-2.41309350707268e-12))*(cj8)))+(x306)+(((IkReal(-1.00000000000000))*(npz)*(x305)))+(((IkReal(-1.00000000000000))*(x307))));
evalcond[2]=((((sj8)*(x306)))+(((IkReal(-1.00000000000000))*(npz)*(sj8)*(x305)))+(((IkReal(-1.00000000000000))*(x303)*(x304)))+(((IkReal(-1.00000000000000))*(x302)*(x303)))+(((IkReal(-1.00000000000000))*(sj8)*(x307))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((sj8)*(x304)))+(((sj8)*(x302)))+(((cj8)*(x306)))+(((IkReal(-1.00000000000000))*(cj10)*(npx)*(x301)*(x303)))+(((IkReal(-1.00000000000000))*(npz)*(x300)*(x303))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
sj8array[0]=((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))))):(IkReal)1.0e30))*(((((IkReal(1.00000000000000))*(cj10)*(npy)))+(((IkReal(1.00000000000000))*(npx)*(sj10))))));
if( sj8array[0] >= -1-IKFAST_SINCOS_THRESH && sj8array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j8valid[0] = j8valid[1] = true;
    j8array[0] = IKasin(sj8array[0]);
    cj8array[0] = IKcos(j8array[0]);
    sj8array[1] = sj8array[0];
    j8array[1] = j8array[0] > 0 ? (IKPI-j8array[0]) : (-IKPI-j8array[0]);
    cj8array[1] = -cj8array[0];
}
else if( isnan(sj8array[0]) )
{
    // probably any value will work
    j8valid[0] = true;
    cj8array[0] = 1; sj8array[0] = 0; j8array[0] = 0;
}
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign((((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx))))));
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[1], cj9array[1], sj9array[1];
bool j9valid[1]={false};
_nj9 = 1;
IkReal x308=((cj7)*(cj8));
IkReal x309=((IkReal(0.550000000000000))*(cj7));
IkReal x310=((cj10)*(npx));
IkReal x311=((npy)*(sj10));
IkReal x312=((IkReal(0.0450000000000000))*(npz));
IkReal x313=((IkReal(0.0450000000000000))*(x311));
IkReal x314=((IkReal(0.550000000000000))*(cj8)*(sj7));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npz)*(x314)))+(((IkReal(-0.300000000000000))*(x310)))+(((IkReal(-0.0450000000000000))*(sj7)*(x310)))+(((x308)*(x312)))+(((sj7)*(x313)))+(((x309)*(x311)))+(((IkReal(-1.00000000000000))*(cj8)*(x312)))+(((IkReal(-1.00000000000000))*(x309)*(x310)))+(((IkReal(0.300000000000000))*(x311))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(0.0450000000000000))*(x308)*(x310)))+(((IkReal(-1.00000000000000))*(x310)*(x314)))+(((IkReal(0.300000000000000))*(npz)))+(((sj7)*(x312)))+(((IkReal(-1.00000000000000))*(x308)*(x313)))+(((cj8)*(x313)))+(((IkReal(-0.0450000000000000))*(cj8)*(x310)))+(((x311)*(x314)))+(((npz)*(x309))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j9array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npz)*(x314)))+(((IkReal(-0.300000000000000))*(x310)))+(((IkReal(-0.0450000000000000))*(sj7)*(x310)))+(((x308)*(x312)))+(((sj7)*(x313)))+(((x309)*(x311)))+(((IkReal(-1.00000000000000))*(cj8)*(x312)))+(((IkReal(-1.00000000000000))*(x309)*(x310)))+(((IkReal(0.300000000000000))*(x311)))))), ((gconst0)*(((((IkReal(0.0450000000000000))*(x308)*(x310)))+(((IkReal(-1.00000000000000))*(x310)*(x314)))+(((IkReal(0.300000000000000))*(npz)))+(((sj7)*(x312)))+(((IkReal(-1.00000000000000))*(x308)*(x313)))+(((cj8)*(x313)))+(((IkReal(-0.0450000000000000))*(cj8)*(x310)))+(((x311)*(x314)))+(((npz)*(x309)))))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
for(int ij9 = 0; ij9 < 1; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 1; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];
{
IkReal evalcond[4];
IkReal x315=IKsin(j9);
IkReal x316=IKcos(j9);
IkReal x317=((IkReal(0.0450000000000000))*(cj7));
IkReal x318=((IkReal(0.550000000000000))*(sj7));
IkReal x319=((npy)*(sj8));
IkReal x320=((npx)*(sj10));
IkReal x321=((cj8)*(npy));
IkReal x322=((sj10)*(x316));
IkReal x323=((IkReal(1.00000000000000))*(cj10)*(npx));
IkReal x324=((IkReal(1.00000000000000))*(npz)*(x315));
evalcond[0]=((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(x315)*(x323)))+(((npz)*(x316)))+(((npy)*(sj10)*(x315))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x324)))+(((IkReal(-0.0450000000000000))*(cj8)))+(((cj8)*(x317)))+(((IkReal(-1.00000000000000))*(x316)*(x323)))+(((npy)*(x322)))+(((IkReal(-1.00000000000000))*(cj8)*(x318))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj8)*(x316)*(x323)))+(((IkReal(-1.00000000000000))*(sj8)*(x324)))+(((IkReal(-1.00000000000000))*(cj8)*(x320)))+(((IkReal(-1.00000000000000))*(cj10)*(x321)))+(((x319)*(x322))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(cj8)*(x324)))+(x317)+(((IkReal(-1.00000000000000))*(x318)))+(((cj10)*(x319)))+(((sj8)*(x320)))+(((x321)*(x322)))+(((IkReal(-1.00000000000000))*(cj8)*(x316)*(x323))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j9array[2], cj9array[2], sj9array[2];
bool j9valid[2]={false};
_nj9 = 2;
IkReal x325=((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))));
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(x325) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x326=((IkReal(1.00000000000000))*(IKatan2(npz, x325)));
if( ((((x325)*(x325))+((npz)*(npz)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x325)*(x325))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x325)*(x325))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x325)*(x325))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x325)*(x325))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x327=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x325)*(x325))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x325)*(x325))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)))))))));
j9array[0]=((((IkReal(-1.00000000000000))*(x327)))+(((IkReal(-1.00000000000000))*(x326))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
j9array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x326)))+(((IkReal(1.00000000000000))*(x327))));
sj9array[1]=IKsin(j9array[1]);
cj9array[1]=IKcos(j9array[1]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
if( j9array[1] > IKPI )
{
    j9array[1]-=IK2PI;
}
else if( j9array[1] < -IKPI )
{    j9array[1]+=IK2PI;
}
j9valid[1] = true;
for(int ij9 = 0; ij9 < 2; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 2; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];

{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x328=((npy)*(sj10));
IkReal x329=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj9)*(x328)))+(((cj9)*(npz)))+(((IkReal(-1.00000000000000))*(npx)*(sj9)*(x329))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x329)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[4]=((((cj9)*(x328)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(npx)*(x329))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x330=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x330));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x330))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x331=IKsin(j8);
IkReal x332=IKcos(j8);
IkReal x333=((IkReal(1.00000000000000))*(x332));
IkReal x334=((IkReal(1.00000000000000))*(x331));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(npy)*(x333)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x333)))+(((cj9)*(npy)*(sj10)*(x331)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x334)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x334))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x335=((npy)*(sj10));
IkReal x336=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj9)*(x335)))+(((cj9)*(npz)))+(((IkReal(-1.00000000000000))*(npx)*(sj9)*(x336))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x336)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[4]=((((cj9)*(x335)))+(((IkReal(-1.00000000000000))*(cj9)*(npx)*(x336)))+(((IkReal(-1.00000000000000))*(npz)*(sj9))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x337=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x337));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x337))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x338=IKsin(j8);
IkReal x339=IKcos(j8);
IkReal x340=((IkReal(1.00000000000000))*(x339));
IkReal x341=((IkReal(1.00000000000000))*(x338));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x341)))+(((cj9)*(npy)*(sj10)*(x338)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x340)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x341)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x340))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(0.250000000390712))+(((npy)*(sj10)*(sj9)))+(((IkReal(-1.00000000000000))*(cj10)*(npx)*(sj9)))+(((cj9)*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x342=((IkReal(414405822679.079))*(npx));
IkReal x343=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj10)*(x342)))+(((cj10)*(x343))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-414405822679.079))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(x342)))+(((cj9)*(sj10)*(x343))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj10)*(x342)))+(((cj10)*(x343)))))+IKsqr(((((IkReal(-414405822679.079))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(x342)))+(((cj9)*(sj10)*(x343)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((sj10)*(x342)))+(((cj10)*(x343)))), ((((IkReal(-414405822679.079))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(x342)))+(((cj9)*(sj10)*(x343)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x344=IKsin(j8);
IkReal x345=IKcos(j8);
IkReal x346=((IkReal(1.00000000000000))*(cj10));
IkReal x347=((cj9)*(npx));
IkReal x348=((cj9)*(npy)*(sj10));
IkReal x349=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x350=((IkReal(1.00000000000000))*(npz)*(sj9));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x346)))+(((IkReal(-1.00000000000000))*(x349)))+(((IkReal(2.41309350707268e-12))*(x344))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x350)))+(x348)+(((IkReal(-1.00000000000000))*(x346)*(x347)))+(((IkReal(-2.41309350707268e-12))*(x345))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x344)*(x346)*(x347)))+(((IkReal(-1.00000000000000))*(npy)*(x345)*(x346)))+(((x344)*(x348)))+(((IkReal(-1.00000000000000))*(x344)*(x350)))+(((IkReal(-1.00000000000000))*(x345)*(x349))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x345)*(x350)))+(((cj10)*(npy)*(x344)))+(((x345)*(x348)))+(((IkReal(-1.00000000000000))*(x345)*(x346)*(x347)))+(((npx)*(sj10)*(x344))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst1)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst1)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x351=IKsin(j8);
IkReal x352=IKcos(j8);
IkReal x353=((npy)*(sj10));
IkReal x354=((IkReal(1.00000000000000))*(cj10));
IkReal x355=((cj9)*(npx));
IkReal x356=((IkReal(0.0450000000000000))*(cj7));
IkReal x357=((IkReal(0.550000000000000))*(sj7));
IkReal x358=((cj9)*(x352));
IkReal x359=((IkReal(1.00000000000000))*(npz)*(sj9));
IkReal x360=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(0.0450000000000000))*(x351)))+(((IkReal(-1.00000000000000))*(npy)*(x354)))+(((IkReal(-1.00000000000000))*(x360)))+(((x351)*(x357)))+(((IkReal(-1.00000000000000))*(x351)*(x356))));
evalcond[1]=((((cj9)*(x353)))+(((IkReal(-0.0450000000000000))*(x352)))+(((IkReal(-1.00000000000000))*(x359)))+(((x352)*(x356)))+(((IkReal(-1.00000000000000))*(x354)*(x355)))+(((IkReal(-1.00000000000000))*(x352)*(x357))));
evalcond[2]=((((cj9)*(x351)*(x353)))+(((IkReal(-1.00000000000000))*(x352)*(x360)))+(((IkReal(-1.00000000000000))*(npy)*(x352)*(x354)))+(((IkReal(-1.00000000000000))*(x351)*(x359)))+(((IkReal(-1.00000000000000))*(x351)*(x354)*(x355))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((cj10)*(npy)*(x351)))+(((IkReal(-1.00000000000000))*(x357)))+(((IkReal(-1.00000000000000))*(x352)*(x354)*(x355)))+(((x353)*(x358)))+(x356)+(((IkReal(-1.00000000000000))*(x352)*(x359)))+(((npx)*(sj10)*(x351))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j9array[2], cj9array[2], sj9array[2];
bool j9valid[2]={false};
_nj9 = 2;
IkReal x361=((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))));
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(x361) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x362=((IkReal(1.00000000000000))*(IKatan2(npz, x361)));
if( ((((x361)*(x361))+((npz)*(npz)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x361)*(x361))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x361)*(x361))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x361)*(x361))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x361)*(x361))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x363=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x361)*(x361))+((npz)*(npz)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x361)*(x361))+((npz)*(npz))))))):(IkReal)1.0e30))*(((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)))))))));
j9array[0]=((((IkReal(-1.00000000000000))*(x363)))+(((IkReal(-1.00000000000000))*(x362))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
j9array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x362)))+(((IkReal(1.00000000000000))*(x363))));
sj9array[1]=IKsin(j9array[1]);
cj9array[1]=IKcos(j9array[1]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
if( j9array[1] > IKPI )
{
    j9array[1]-=IK2PI;
}
else if( j9array[1] < -IKPI )
{    j9array[1]+=IK2PI;
}
j9valid[1] = true;
for(int ij9 = 0; ij9 < 2; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 2; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];

{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x364=((npy)*(sj10));
IkReal x365=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((cj9)*(npz)))+(((sj9)*(x364)))+(((IkReal(-1.00000000000000))*(npx)*(sj9)*(x365))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x365)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj9)*(npx)*(x365)))+(((cj9)*(x364)))+(((IkReal(-1.00000000000000))*(npz)*(sj9))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x366=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x366));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x366))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x367=IKsin(j8);
IkReal x368=IKcos(j8);
IkReal x369=((IkReal(1.00000000000000))*(x368));
IkReal x370=((IkReal(1.00000000000000))*(x367));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj9)*(x370)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x369)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x369)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x370)))+(((cj9)*(npy)*(sj10)*(x367))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x371=((npy)*(sj10));
IkReal x372=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(sj9)*(x372)))+(((cj9)*(npz)))+(((sj9)*(x371))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npy)*(x372)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj9)*(npx)*(x372)))+(((cj9)*(x371)))+(((IkReal(-1.00000000000000))*(npz)*(sj9))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x373=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x373));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x373))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x374=IKsin(j8);
IkReal x375=IKcos(j8);
IkReal x376=((IkReal(1.00000000000000))*(x375));
IkReal x377=((IkReal(1.00000000000000))*(x374));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(sj9)*(x377)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x376)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x376)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x377)))+(((cj9)*(npy)*(sj10)*(x374))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(0.250000000390712))+(((npy)*(sj10)*(sj9)))+(((IkReal(-1.00000000000000))*(cj10)*(npx)*(sj9)))+(((cj9)*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x378=((IkReal(414405822679.079))*(npx));
IkReal x379=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj10)*(x378)))+(((cj10)*(x379))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-414405822679.079))*(npz)*(sj9)))+(((cj9)*(sj10)*(x379)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(x378))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj10)*(x378)))+(((cj10)*(x379)))))+IKsqr(((((IkReal(-414405822679.079))*(npz)*(sj9)))+(((cj9)*(sj10)*(x379)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(x378)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((sj10)*(x378)))+(((cj10)*(x379)))), ((((IkReal(-414405822679.079))*(npz)*(sj9)))+(((cj9)*(sj10)*(x379)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(x378)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x380=IKsin(j8);
IkReal x381=IKcos(j8);
IkReal x382=((IkReal(1.00000000000000))*(cj10));
IkReal x383=((cj9)*(npx));
IkReal x384=((cj9)*(npy)*(sj10));
IkReal x385=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x386=((IkReal(1.00000000000000))*(npz)*(sj9));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x382)))+(((IkReal(2.41309350707268e-12))*(x380)))+(((IkReal(-1.00000000000000))*(x385))));
evalcond[1]=((((IkReal(-2.41309350707268e-12))*(x381)))+(x384)+(((IkReal(-1.00000000000000))*(x386)))+(((IkReal(-1.00000000000000))*(x382)*(x383))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x380)*(x386)))+(((IkReal(-1.00000000000000))*(x381)*(x385)))+(((x380)*(x384)))+(((IkReal(-1.00000000000000))*(npy)*(x381)*(x382)))+(((IkReal(-1.00000000000000))*(x380)*(x382)*(x383))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((npx)*(sj10)*(x380)))+(((IkReal(-1.00000000000000))*(x381)*(x386)))+(((IkReal(-1.00000000000000))*(x381)*(x382)*(x383)))+(((cj10)*(npy)*(x380)))+(((x381)*(x384))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst1)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst1)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x387=IKsin(j8);
IkReal x388=IKcos(j8);
IkReal x389=((npy)*(sj10));
IkReal x390=((IkReal(1.00000000000000))*(cj10));
IkReal x391=((cj9)*(npx));
IkReal x392=((IkReal(0.0450000000000000))*(cj7));
IkReal x393=((IkReal(0.550000000000000))*(sj7));
IkReal x394=((cj9)*(x388));
IkReal x395=((IkReal(1.00000000000000))*(npz)*(sj9));
IkReal x396=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(x387)*(x392)))+(((IkReal(-1.00000000000000))*(x396)))+(((IkReal(0.0450000000000000))*(x387)))+(((x387)*(x393)))+(((IkReal(-1.00000000000000))*(npy)*(x390))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x395)))+(((cj9)*(x389)))+(((IkReal(-1.00000000000000))*(x388)*(x393)))+(((IkReal(-0.0450000000000000))*(x388)))+(((IkReal(-1.00000000000000))*(x390)*(x391)))+(((x388)*(x392))));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x388)*(x390)))+(((IkReal(-1.00000000000000))*(x387)*(x395)))+(((IkReal(-1.00000000000000))*(x387)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(x388)*(x396)))+(((cj9)*(x387)*(x389))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x393)))+(((npx)*(sj10)*(x387)))+(((IkReal(-1.00000000000000))*(x388)*(x395)))+(((cj10)*(npy)*(x387)))+(x392)+(((IkReal(-1.00000000000000))*(x388)*(x390)*(x391)))+(((x389)*(x394))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x53=((IkReal(1.00000000000000))*(sj7));
IkReal x54=((IkReal(1.00000000000000))*(sj10));
IkReal x55=((IkReal(1.00000000000000))*(cj7));
IkReal x56=((IkReal(1.00000000000000))*(cj9));
IkReal x57=((IkReal(-1.00000000000000))*(cj9));
IkReal x58=((((IkReal(-1.00000000000000))*(r01)*(x54)))+(((cj10)*(r00))));
IkReal x59=((((r00)*(sj10)))+(((cj10)*(r01))));
IkReal x60=((((IkReal(-1.00000000000000))*(r11)*(x54)))+(((cj10)*(r10))));
IkReal x61=((((r10)*(sj10)))+(((cj10)*(r11))));
IkReal x62=((((IkReal(-1.00000000000000))*(r21)*(x54)))+(((cj10)*(r20))));
IkReal x63=((((r20)*(sj10)))+(((cj10)*(r21))));
IkReal x64=((sj9)*(x58));
IkReal x65=((sj9)*(x60));
IkReal x66=((sj9)*(x62));
IkReal x67=((sj8)*(x63));
IkReal x68=((sj8)*(x59));
IkReal x69=((sj8)*(x61));
IkReal x70=((((cj9)*(x58)))+(((r02)*(sj9))));
IkReal x71=((((cj9)*(x60)))+(((r12)*(sj9))));
IkReal x72=((((cj9)*(x62)))+(((r22)*(sj9))));
IkReal x73=((cj8)*(x70));
IkReal x74=((cj8)*(x71));
IkReal x75=((cj8)*(x72));
new_r00=((((IkReal(-1.00000000000000))*(x53)*(((((IkReal(-1.00000000000000))*(r02)*(x56)))+(x64)))))+(((cj7)*(((x73)+(((IkReal(-1.00000000000000))*(x68))))))));
new_r01=((((cj8)*(x59)))+(((sj8)*(x70))));
new_r02=((((IkReal(-1.00000000000000))*(x55)*(((((r02)*(x57)))+(x64)))))+(((IkReal(-1.00000000000000))*(x53)*(((x73)+(((IkReal(-1.00000000000000))*(x68))))))));
new_r10=((((cj7)*(((x74)+(((IkReal(-1.00000000000000))*(x69)))))))+(((IkReal(-1.00000000000000))*(x53)*(((((IkReal(-1.00000000000000))*(r12)*(x56)))+(x65))))));
new_r11=((((cj8)*(x61)))+(((sj8)*(x71))));
new_r12=((((IkReal(-1.00000000000000))*(x53)*(((x74)+(((IkReal(-1.00000000000000))*(x69)))))))+(((IkReal(-1.00000000000000))*(x55)*(((x65)+(((r12)*(x57))))))));
new_r20=((((IkReal(-1.00000000000000))*(x53)*(((x66)+(((IkReal(-1.00000000000000))*(r22)*(x56)))))))+(((cj7)*(((x75)+(((IkReal(-1.00000000000000))*(x67))))))));
new_r21=((((cj8)*(x63)))+(((sj8)*(x72))));
new_r22=((((IkReal(-1.00000000000000))*(x53)*(((x75)+(((IkReal(-1.00000000000000))*(x67)))))))+(((IkReal(-1.00000000000000))*(x55)*(((((r22)*(x57)))+(x66))))));
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
cj5array[0]=new_r22;
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((((IkReal(2.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(2.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x76=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x76));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x76))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x77=IKcos(j4);
IkReal x78=IKsin(j4);
IkReal x79=((IkReal(0.707106781186548))*(new_r02));
IkReal x80=((new_r12)*(x77));
IkReal x81=((IkReal(0.707106781186548))*(new_r12)*(x78));
IkReal x82=((x77)*(x79));
IkReal x83=((IkReal(0.707106781186548))*(x80));
IkReal x84=((x78)*(x79));
IkReal x85=((x82)+(x81));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(x78)))+(x80));
evalcond[1]=((x83)+(((IkReal(-1.00000000000000))*(x84)))+(((IkReal(-1.00000000000000))*(x85))));
evalcond[2]=((x83)+(x85)+(((IkReal(-1.00000000000000))*(x84))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x86=((IkReal(0.707106781186548))*(sj4));
IkReal x87=((IkReal(0.707106781186548))*(cj4));
IkReal x88=((new_r10)*(x86));
IkReal x89=((new_r00)*(x87));
IkReal x90=((new_r11)*(x86));
IkReal x91=((new_r01)*(x87));
IkReal x92=((x91)+(x90));
IkReal x93=((x88)+(x89));
if( IKabs(((((IkReal(-1.00000000000000))*(x93)))+(((IkReal(-1.00000000000000))*(x92))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x93)+(((IkReal(-1.00000000000000))*(x92))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x93)))+(((IkReal(-1.00000000000000))*(x92)))))+IKsqr(((x93)+(((IkReal(-1.00000000000000))*(x92)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x93)))+(((IkReal(-1.00000000000000))*(x92)))), ((x93)+(((IkReal(-1.00000000000000))*(x92)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x94=IKsin(j6);
IkReal x95=IKcos(j6);
IkReal x96=((cj4)*(new_r00));
IkReal x97=((cj4)*(new_r01));
IkReal x98=((IkReal(0.707106781186548))*(sj4));
IkReal x99=((IkReal(0.707106781186548))*(cj4));
IkReal x100=((IkReal(1.00000000000000))*(sj4));
IkReal x101=((IkReal(0.707106781186548))*(x95));
IkReal x102=((IkReal(0.707106781186548))*(x94));
IkReal x103=((new_r11)*(x98));
IkReal x104=((IkReal(0.707106781186548))*(x97));
IkReal x105=((new_r11)*(x99));
IkReal x106=((new_r01)*(x98));
IkReal x107=((new_r10)*(x98));
IkReal x108=((IkReal(0.707106781186548))*(x96));
IkReal x109=((new_r10)*(x99));
IkReal x110=((new_r00)*(x98));
IkReal x111=((IkReal(1.00000000000000))*(x95));
IkReal x112=((x101)+(x102));
IkReal x113=((x108)+(x107));
IkReal x114=((x104)+(x103));
evalcond[0]=((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x101)))+(x102)+(x96));
evalcond[1]=((((new_r11)*(sj4)))+(x112)+(x97));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x100)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x112))));
evalcond[3]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x100)))+(((IkReal(-1.00000000000000))*(x101)))+(x102));
evalcond[4]=((((IkReal(-1.00000000000000))*(x113)))+(((IkReal(-1.00000000000000))*(x110)))+(x109)+(((IkReal(-1.00000000000000))*(x94))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x106)))+(((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x114)))+(x105));
evalcond[6]=((((IkReal(-1.00000000000000))*(x106)))+(x114)+(x105)+(x94));
evalcond[7]=((((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x110)))+(x113)+(x109));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x115=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x115;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x115;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x116=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x116));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x116))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x117=IKcos(j4);
IkReal x118=IKsin(j4);
IkReal x119=((IkReal(0.707106781186548))*(new_r02));
IkReal x120=((new_r12)*(x117));
IkReal x121=((x118)*(x119));
IkReal x122=((IkReal(0.707106781186548))*(new_r12)*(x118));
IkReal x123=((x117)*(x119));
IkReal x124=((IkReal(0.707106781186548))*(x120));
IkReal x125=((x122)+(x123));
evalcond[0]=((x120)+(((IkReal(-1.00000000000000))*(new_r02)*(x118))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x121)))+(((IkReal(-1.00000000000000))*(x125)))+(x124));
evalcond[2]=((((IkReal(-1.00000000000000))*(x125)))+(((IkReal(-1.00000000000000))*(x124)))+(x121));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x126=((IkReal(0.707106781186548))*(sj4));
IkReal x127=((IkReal(0.707106781186548))*(cj4));
IkReal x128=((new_r11)*(x126));
IkReal x129=((new_r10)*(x126));
IkReal x130=((new_r00)*(x127));
IkReal x131=((new_r01)*(x127));
IkReal x132=((x131)+(x128));
IkReal x133=((x130)+(x129));
if( IKabs(((x132)+(((IkReal(-1.00000000000000))*(x133))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x133))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x132)+(((IkReal(-1.00000000000000))*(x133)))))+IKsqr(((((IkReal(-1.00000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x133)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((x132)+(((IkReal(-1.00000000000000))*(x133)))), ((((IkReal(-1.00000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x133)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x134=IKsin(j6);
IkReal x135=IKcos(j6);
IkReal x136=((cj4)*(new_r00));
IkReal x137=((cj4)*(new_r01));
IkReal x138=((IkReal(0.707106781186548))*(sj4));
IkReal x139=((IkReal(0.707106781186548))*(cj4));
IkReal x140=((IkReal(1.00000000000000))*(sj4));
IkReal x141=((IkReal(0.707106781186548))*(x135));
IkReal x142=((IkReal(0.707106781186548))*(x134));
IkReal x143=((new_r01)*(x138));
IkReal x144=((new_r11)*(x138));
IkReal x145=((IkReal(0.707106781186548))*(x137));
IkReal x146=((new_r11)*(x139));
IkReal x147=((new_r00)*(x138));
IkReal x148=((IkReal(1.00000000000000))*(x135));
IkReal x149=((new_r10)*(x138));
IkReal x150=((IkReal(0.707106781186548))*(x136));
IkReal x151=((new_r10)*(x139));
IkReal x152=((x150)+(x149));
IkReal x153=((x144)+(x145));
IkReal x154=((x141)+(x142));
evalcond[0]=((((new_r10)*(sj4)))+(x154)+(x136));
evalcond[1]=((((IkReal(-1.00000000000000))*(x142)))+(((new_r11)*(sj4)))+(x141)+(x137));
evalcond[2]=((((IkReal(-1.00000000000000))*(x142)))+(((cj4)*(new_r10)))+(x141)+(((IkReal(-1.00000000000000))*(new_r00)*(x140))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x140)))+(((IkReal(-1.00000000000000))*(x154)))+(((cj4)*(new_r11))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x147)))+(((IkReal(-1.00000000000000))*(x152)))+(x151)+(((IkReal(-1.00000000000000))*(x134))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x143)))+(((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(x153)))+(x146));
evalcond[6]=((((IkReal(-1.00000000000000))*(x146)))+(((IkReal(-1.00000000000000))*(x153)))+(x143)+(x134));
evalcond[7]=((((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(x151)))+(((IkReal(-1.00000000000000))*(x152)))+(x147));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x155=((IkReal(1.41421356237310))*(sj5));
IkReal x156=((new_r12)*(x155));
IkReal x157=((new_r02)*(x155));
if( IKabs(((gconst5)*(((((IkReal(-1.00000000000000))*(x157)))+(x156))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((x157)+(x156))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst5)*(((((IkReal(-1.00000000000000))*(x157)))+(x156)))), ((gconst5)*(((x157)+(x156)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[7];
IkReal x158=IKcos(j4);
IkReal x159=((IkReal(0.707106781186548))*(sj5));
IkReal x160=IKsin(j4);
IkReal x161=((IkReal(1.00000000000000))*(cj5));
IkReal x162=((IkReal(0.707106781186548))*(cj5));
IkReal x163=((new_r02)*(x158));
IkReal x164=((new_r12)*(x158));
IkReal x165=((new_r12)*(x160));
IkReal x166=((new_r02)*(x160));
IkReal x167=((IkReal(0.707106781186548))*(x164));
evalcond[0]=((((IkReal(-1.00000000000000))*(x159)))+(x163)+(x165));
evalcond[1]=((((IkReal(-1.00000000000000))*(x159)))+(x164)+(((IkReal(-1.00000000000000))*(x166))));
evalcond[2]=((x167)+(((IkReal(-0.707106781186548))*(x163)))+(((IkReal(-0.707106781186548))*(x166)))+(((IkReal(-0.707106781186548))*(x165))));
evalcond[3]=((((x162)*(x163)))+(((x162)*(x164)))+(((x162)*(x165)))+(((IkReal(-1.00000000000000))*(x162)*(x166)))+(((IkReal(-1.00000000000000))*(new_r22)*(sj5))));
evalcond[4]=((((new_r00)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(new_r20)*(x161)))+(((IkReal(-1.00000000000000))*(new_r10)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(new_r10)*(x158)*(x159)))+(((IkReal(-1.00000000000000))*(new_r00)*(x158)*(x159))));
evalcond[5]=((((new_r01)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(new_r21)*(x161)))+(((IkReal(-1.00000000000000))*(new_r11)*(x158)*(x159)))+(((IkReal(-1.00000000000000))*(new_r11)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(new_r01)*(x158)*(x159))));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x159)*(x165)))+(((IkReal(-1.00000000000000))*(x159)*(x164)))+(((IkReal(-1.00000000000000))*(x159)*(x163)))+(((IkReal(-1.00000000000000))*(new_r22)*(x161)))+(((x159)*(x166))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x168=((cj4)*(new_r02));
IkReal x169=((cj4)*(new_r12));
IkReal x170=((new_r02)*(sj4));
IkReal x171=((new_r12)*(sj4));
IkReal x172=((IkReal(0.707106781186548))*(x168));
IkReal x173=((IkReal(0.707106781186548))*(x171));
IkReal x174=((IkReal(0.707106781186548))*(x169));
IkReal x175=((IkReal(0.707106781186548))*(x170));
IkReal x176=((x173)+(x172));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x171)+(x168));
evalcond[5]=((((IkReal(-1.00000000000000))*(x170)))+(x169));
evalcond[6]=((((IkReal(-1.00000000000000))*(x176)))+(((IkReal(-1.00000000000000))*(x175)))+(x174));
evalcond[7]=((((IkReal(-1.00000000000000))*(x175)))+(x174)+(x176));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x177=((IkReal(0.707106781186548))*(sj4));
IkReal x178=((IkReal(0.707106781186548))*(cj4));
IkReal x179=((new_r10)*(x177));
IkReal x180=((new_r00)*(x178));
IkReal x181=((new_r11)*(x177));
IkReal x182=((new_r01)*(x178));
IkReal x183=((x181)+(x182));
IkReal x184=((x179)+(x180));
if( IKabs(((((IkReal(-1.00000000000000))*(x184)))+(((IkReal(-1.00000000000000))*(x183))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x183)))+(x184))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x184)))+(((IkReal(-1.00000000000000))*(x183)))))+IKsqr(((((IkReal(-1.00000000000000))*(x183)))+(x184)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x184)))+(((IkReal(-1.00000000000000))*(x183)))), ((((IkReal(-1.00000000000000))*(x183)))+(x184)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x185=IKsin(j6);
IkReal x186=IKcos(j6);
IkReal x187=((cj4)*(new_r00));
IkReal x188=((cj4)*(new_r01));
IkReal x189=((IkReal(0.707106781186548))*(sj4));
IkReal x190=((IkReal(0.707106781186548))*(cj4));
IkReal x191=((IkReal(1.00000000000000))*(sj4));
IkReal x192=((IkReal(0.707106781186548))*(x186));
IkReal x193=((IkReal(0.707106781186548))*(x185));
IkReal x194=((new_r11)*(x189));
IkReal x195=((IkReal(0.707106781186548))*(x188));
IkReal x196=((new_r11)*(x190));
IkReal x197=((new_r01)*(x189));
IkReal x198=((new_r10)*(x189));
IkReal x199=((IkReal(0.707106781186548))*(x187));
IkReal x200=((new_r10)*(x190));
IkReal x201=((new_r00)*(x189));
IkReal x202=((IkReal(1.00000000000000))*(x186));
IkReal x203=((x193)+(x192));
IkReal x204=((x199)+(x198));
IkReal x205=((x195)+(x194));
evalcond[0]=((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x192)))+(x193)+(x187));
evalcond[1]=((((new_r11)*(sj4)))+(x188)+(x203));
evalcond[2]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x203)))+(((IkReal(-1.00000000000000))*(new_r00)*(x191))));
evalcond[3]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x192)))+(x193)+(((IkReal(-1.00000000000000))*(new_r01)*(x191))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x185)))+(((IkReal(-1.00000000000000))*(x201)))+(((IkReal(-1.00000000000000))*(x204)))+(x200));
evalcond[5]=((((IkReal(-1.00000000000000))*(x197)))+(x196)+(((IkReal(-1.00000000000000))*(x205)))+(((IkReal(-1.00000000000000))*(x202))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x197)))+(x196)+(x185)+(x205));
evalcond[7]=((((IkReal(-1.00000000000000))*(x201)))+(((IkReal(-1.00000000000000))*(x202)))+(x204)+(x200));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x206=((IkReal(1.00000000000000))+(new_r22));
IkReal x207=((cj4)*(new_r02));
IkReal x208=((cj4)*(new_r12));
IkReal x209=((new_r02)*(sj4));
IkReal x210=((new_r12)*(sj4));
IkReal x211=((IkReal(0.707106781186548))*(x209));
IkReal x212=((IkReal(0.707106781186548))*(x207));
IkReal x213=((IkReal(0.707106781186548))*(x210));
IkReal x214=((IkReal(0.707106781186548))*(x208));
IkReal x215=((x212)+(x213));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x206;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x210)+(x207));
evalcond[5]=((x208)+(((IkReal(-1.00000000000000))*(x209))));
evalcond[6]=((x214)+(((IkReal(-1.00000000000000))*(x211)))+(((IkReal(-1.00000000000000))*(x215))));
evalcond[7]=((x211)+(((IkReal(-1.00000000000000))*(x215)))+(((IkReal(-1.00000000000000))*(x214))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x206;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x216=((IkReal(0.707106781186548))*(sj4));
IkReal x217=((IkReal(0.707106781186548))*(cj4));
IkReal x218=((new_r11)*(x216));
IkReal x219=((new_r10)*(x216));
IkReal x220=((new_r00)*(x217));
IkReal x221=((new_r01)*(x217));
IkReal x222=((x218)+(x221));
IkReal x223=((x219)+(x220));
if( IKabs(((x222)+(((IkReal(-1.00000000000000))*(x223))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x223)))+(((IkReal(-1.00000000000000))*(x222))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x222)+(((IkReal(-1.00000000000000))*(x223)))))+IKsqr(((((IkReal(-1.00000000000000))*(x223)))+(((IkReal(-1.00000000000000))*(x222)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((x222)+(((IkReal(-1.00000000000000))*(x223)))), ((((IkReal(-1.00000000000000))*(x223)))+(((IkReal(-1.00000000000000))*(x222)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x224=IKsin(j6);
IkReal x225=IKcos(j6);
IkReal x226=((cj4)*(new_r00));
IkReal x227=((cj4)*(new_r01));
IkReal x228=((IkReal(0.707106781186548))*(sj4));
IkReal x229=((IkReal(0.707106781186548))*(cj4));
IkReal x230=((IkReal(1.00000000000000))*(sj4));
IkReal x231=((IkReal(0.707106781186548))*(x225));
IkReal x232=((IkReal(0.707106781186548))*(x224));
IkReal x233=((new_r01)*(x228));
IkReal x234=((new_r11)*(x228));
IkReal x235=((IkReal(0.707106781186548))*(x227));
IkReal x236=((new_r11)*(x229));
IkReal x237=((new_r00)*(x228));
IkReal x238=((IkReal(1.00000000000000))*(x225));
IkReal x239=((new_r10)*(x228));
IkReal x240=((IkReal(0.707106781186548))*(x226));
IkReal x241=((new_r10)*(x229));
IkReal x242=((x239)+(x240));
IkReal x243=((x234)+(x235));
IkReal x244=((x232)+(x231));
evalcond[0]=((((new_r10)*(sj4)))+(x226)+(x244));
evalcond[1]=((((new_r11)*(sj4)))+(((IkReal(-1.00000000000000))*(x232)))+(x227)+(x231));
evalcond[2]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x232)))+(x231)+(((IkReal(-1.00000000000000))*(new_r00)*(x230))));
evalcond[3]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x244)))+(((IkReal(-1.00000000000000))*(new_r01)*(x230))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x237)))+(x241)+(((IkReal(-1.00000000000000))*(x242)))+(((IkReal(-1.00000000000000))*(x224))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x238)))+(((IkReal(-1.00000000000000))*(x233)))+(x236)+(((IkReal(-1.00000000000000))*(x243))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x236)))+(x224)+(x233)+(((IkReal(-1.00000000000000))*(x243))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x238)))+(x237)+(((IkReal(-1.00000000000000))*(x241)))+(((IkReal(-1.00000000000000))*(x242))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst7)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst7)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst7)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst7)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[10];
IkReal x245=IKsin(j6);
IkReal x246=IKcos(j6);
IkReal x247=((IkReal(1.00000000000000))*(sj5));
IkReal x248=((IkReal(0.707106781186548))*(cj4));
IkReal x249=((IkReal(1.00000000000000))*(sj4));
IkReal x250=((cj5)*(new_r01));
IkReal x251=((IkReal(0.707106781186548))*(cj5));
IkReal x252=((new_r11)*(sj4));
IkReal x253=((IkReal(0.707106781186548))*(sj4));
IkReal x254=((new_r10)*(sj4));
IkReal x255=((IkReal(0.707106781186548))*(x245));
IkReal x256=((IkReal(0.707106781186548))*(x246));
IkReal x257=((IkReal(1.00000000000000))*(x246));
IkReal x258=((x246)*(x251));
IkReal x259=((x245)*(x251));
evalcond[0]=((((sj5)*(x246)))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(x245)*(x247)))+(new_r21));
evalcond[2]=((((cj4)*(new_r00)))+(x254)+(x255)+(((IkReal(-1.00000000000000))*(x258))));
evalcond[3]=((((cj4)*(new_r01)))+(x259)+(x256)+(x252));
evalcond[4]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x249)))+(((IkReal(-1.00000000000000))*(x258)))+(((IkReal(-1.00000000000000))*(x255))));
evalcond[5]=((((IkReal(-1.00000000000000))*(new_r01)*(x249)))+(((cj4)*(new_r11)))+(x259)+(((IkReal(-1.00000000000000))*(x256))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r10)*(x253)))+(((IkReal(-1.00000000000000))*(x245)))+(((IkReal(-1.00000000000000))*(new_r00)*(x248)))+(((IkReal(-1.00000000000000))*(new_r00)*(x253)))+(((new_r10)*(x248))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r01)*(x253)))+(((IkReal(-1.00000000000000))*(new_r01)*(x248)))+(((IkReal(-0.707106781186548))*(x252)))+(((new_r11)*(x248)))+(((IkReal(-1.00000000000000))*(x257))));
evalcond[8]=((((x248)*(x250)))+(((IkReal(-1.00000000000000))*(x250)*(x253)))+(((x251)*(x252)))+(((cj5)*(new_r11)*(x248)))+(((IkReal(-1.00000000000000))*(new_r21)*(x247)))+(x245));
evalcond[9]=((((cj5)*(new_r10)*(x248)))+(((x251)*(x254)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)*(x251)))+(((IkReal(-1.00000000000000))*(new_r20)*(x247)))+(((cj5)*(new_r00)*(x248)))+(((IkReal(-1.00000000000000))*(x257))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst6)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst6)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst6)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst6)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[2];
evalcond[0]=((new_r20)+(((sj5)*(IKcos(j6)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(IKsin(j6))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst8;
gconst8=IKsign(((((IkReal(2.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(2.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x260=((IkReal(1.41421356237310))*(sj5));
IkReal x261=((new_r12)*(x260));
IkReal x262=((new_r02)*(x260));
if( IKabs(((gconst8)*(((x261)+(((IkReal(-1.00000000000000))*(x262))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst8)*(((x261)+(x262))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst8)*(((x261)+(((IkReal(-1.00000000000000))*(x262)))))), ((gconst8)*(((x261)+(x262)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[15];
IkReal x263=IKcos(j4);
IkReal x264=((IkReal(0.707106781186548))*(sj5));
IkReal x265=IKsin(j4);
IkReal x266=((IkReal(0.707106781186548))*(sj6));
IkReal x267=((IkReal(0.707106781186548))*(cj6));
IkReal x268=((IkReal(1.00000000000000))*(cj6));
IkReal x269=((IkReal(0.707106781186548))*(cj5));
IkReal x270=((IkReal(1.00000000000000))*(cj5));
IkReal x271=((IkReal(1.00000000000000))*(sj5));
IkReal x272=((cj5)*(x267));
IkReal x273=((cj5)*(x266));
IkReal x274=((new_r10)*(x263));
IkReal x275=((new_r12)*(x263));
IkReal x276=((new_r11)*(x265));
IkReal x277=((new_r02)*(x265));
IkReal x278=((new_r00)*(x263));
IkReal x279=((IkReal(0.707106781186548))*(x265));
IkReal x280=((new_r02)*(x263));
IkReal x281=((new_r11)*(x263));
IkReal x282=((new_r00)*(x265));
IkReal x283=((new_r12)*(x265));
IkReal x284=((new_r01)*(x265));
IkReal x285=((new_r10)*(x265));
IkReal x286=((new_r01)*(x263));
evalcond[0]=((x283)+(x280)+(((IkReal(-1.00000000000000))*(x264))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x277)))+(x275)+(((IkReal(-1.00000000000000))*(x264))));
evalcond[2]=((x266)+(x278)+(x285)+(((IkReal(-1.00000000000000))*(x272))));
evalcond[3]=((x267)+(x276)+(x273)+(x286));
evalcond[4]=((x274)+(((IkReal(-1.00000000000000))*(x282)))+(((IkReal(-1.00000000000000))*(x272)))+(((IkReal(-1.00000000000000))*(x266))));
evalcond[5]=((x273)+(x281)+(((IkReal(-1.00000000000000))*(x284)))+(((IkReal(-1.00000000000000))*(x267))));
evalcond[6]=((((IkReal(-0.707106781186548))*(x277)))+(((IkReal(-1.00000000000000))*(new_r12)*(x279)))+(((IkReal(0.707106781186548))*(x275)))+(((IkReal(-0.707106781186548))*(x280))));
evalcond[7]=((((IkReal(-1.00000000000000))*(sj6)))+(((IkReal(-0.707106781186548))*(x278)))+(((IkReal(-1.00000000000000))*(new_r00)*(x279)))+(((IkReal(0.707106781186548))*(x274)))+(((IkReal(-1.00000000000000))*(new_r10)*(x279))));
evalcond[8]=((((IkReal(-0.707106781186548))*(x276)))+(((IkReal(-1.00000000000000))*(new_r01)*(x279)))+(((IkReal(0.707106781186548))*(x281)))+(((IkReal(-1.00000000000000))*(x268)))+(((IkReal(-0.707106781186548))*(x286))));
evalcond[9]=((((x269)*(x283)))+(((x269)*(x280)))+(((IkReal(-1.00000000000000))*(new_r22)*(x271)))+(((IkReal(-1.00000000000000))*(x269)*(x277)))+(((x269)*(x275))));
evalcond[10]=((((x264)*(x282)))+(((IkReal(-1.00000000000000))*(new_r20)*(x270)))+(((IkReal(-1.00000000000000))*(x264)*(x285)))+(((IkReal(-1.00000000000000))*(x264)*(x274)))+(((IkReal(-1.00000000000000))*(x264)*(x278))));
evalcond[11]=((((x264)*(x284)))+(((IkReal(-1.00000000000000))*(new_r21)*(x270)))+(((IkReal(-1.00000000000000))*(x264)*(x281)))+(((IkReal(-1.00000000000000))*(x264)*(x286)))+(((IkReal(-1.00000000000000))*(x264)*(x276))));
evalcond[12]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x270)))+(((x264)*(x277)))+(((IkReal(-1.00000000000000))*(x264)*(x280)))+(((IkReal(-1.00000000000000))*(x264)*(x283)))+(((IkReal(-1.00000000000000))*(x264)*(x275))));
evalcond[13]=((sj6)+(((x269)*(x286)))+(((x269)*(x281)))+(((IkReal(-1.00000000000000))*(new_r21)*(x271)))+(((IkReal(-1.00000000000000))*(x269)*(x284)))+(((x269)*(x276))));
evalcond[14]=((((x269)*(x285)))+(((IkReal(-1.00000000000000))*(new_r20)*(x271)))+(((IkReal(-1.00000000000000))*(x269)*(x282)))+(((IkReal(-1.00000000000000))*(x268)))+(((x269)*(x274)))+(((x269)*(x278))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  || IKabs(evalcond[12]) > 0.000001  || IKabs(evalcond[13]) > 0.000001  || IKabs(evalcond[14]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ba32f85ca3c87502739320e9e26a5905"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
