/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:50:02.422312
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[2]);
x3=IKsin(j[3]);
x4=IKcos(j[3]);
x5=IKsin(j[1]);
x6=IKsin(j[5]);
x7=IKsin(j[4]);
x8=IKcos(j[4]);
x9=IKcos(j[5]);
x10=((IkReal(0.707106781186548))*(IKcos(j[0])));
x11=((IkReal(0.707106781186548))*(IKsin(j[0])));
x12=((IkReal(0.707106781186548))*(x6));
x13=((IkReal(0.707106781186548))*(x9));
x14=((IkReal(0.500001138018525))*(x9));
x15=((IkReal(0.499998861978885))*(x9));
x16=((IkReal(0.499998861978885))*(x6));
x17=((IkReal(1.00000000000000))*(x1));
x18=((IkReal(0.500001138018525))*(x6));
x19=((IkReal(1.00000000000000))*(x3));
x20=((x0)*(x1));
x21=((x0)*(x2));
x22=((x4)*(x5));
x23=((IkReal(-1.00000000000000))*(x2));
x24=((x3)*(x5));
x25=((IkReal(-1.00000000000000))*(x3));
x26=((((IkReal(-1.00000000000000))*(x10)))+(x11));
x27=((((IkReal(-1.00000000000000))*(x11)))+(x10));
x28=((((IkReal(-1.00000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(x10))));
x29=((x11)+(x10));
x30=((x26)*(x5));
x31=((IkReal(-1.00000000000000))*(x27));
x32=((x2)*(x27));
x33=((((IkReal(1.00000000000000))*(x0)*(x4)))+(((IkReal(-1.00000000000000))*(x17)*(x24))));
x34=((x33)*(x9));
x35=((((x8)*(((((IkReal(-1.00000000000000))*(x0)*(x19)))+(((IkReal(-1.00000000000000))*(x17)*(x22)))))))+(((x2)*(x5)*(x7))));
x36=((x35)*(x6));
x37=((((IkReal(-1.00000000000000))*(x22)*(x28)))+(((x19)*(((((x0)*(x17)*(x29)))+(((IkReal(1.00000000000000))*(x32))))))));
x38=((((x25)*(((((x20)*(x31)))+(((x23)*(x28)))))))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x39=((x37)*(x9));
x40=((((x8)*(((((x4)*(((((x2)*(x28)))+(((x20)*(x27)))))))+(((x24)*(x26)))))))+(((x7)*(((((x21)*(x26)))+(((x1)*(x28))))))));
x41=((((x8)*(((((x4)*(((x32)+(((x20)*(x29)))))))+(((x24)*(x28)))))))+(((x7)*(((((x21)*(x28)))+(((x1)*(x27))))))));
eerot[0]=((((((((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))+(((IkReal(0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))))*(IKcos(j[6])))));
eerot[1]=((((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707108390587779))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6])))));
eerot[2]=((((IkReal(-0.707108390587779))*(x36)))+(((IkReal(-0.707108390587779))*(x34)))+(((x16)*(x40)))+(((x15)*(x38)))+(((IkReal(-1.00000000000000))*(x16)*(x41)))+(((IkReal(-1.00000000000000))*(x15)*(x37))));
eetrans[0]=((IkReal(-0.350177000000000))+(((IkReal(0.212132517176334))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKcos(j[0]))))+(((IkReal(-0.106065775767248))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318198775764501))*(IKcos(j[2]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKsin(j[0]))))+(((IkReal(0.106065775767248))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(-0.388909614823279))*(IKcos(j[1]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((IkReal(-0.212132517176334))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(-0.125829938105095))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829938105095))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.125829938105095))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707108390587779))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1])))));
eerot[3]=((((((((IkReal(0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6])))));
eerot[4]=((((((((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6])))));
eerot[5]=((((IkReal(-0.707105171781653))*(x34)))+(((IkReal(-0.707105171781653))*(x36)))+(((IkReal(-1.00000000000000))*(x18)*(x40)))+(((x18)*(x41)))+(((IkReal(-1.00000000000000))*(x14)*(x38)))+(((x14)*(((((x25)*(((((x23)*(x27)))+(((IkReal(-1.00000000000000))*(x20)*(x29)))))))+(((IkReal(-1.00000000000000))*(x22)*(x28))))))));
eetrans[1]=((IkReal(-0.350177000000000))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318197327301744))*(IKcos(j[2]))*(IKsin(j[1]))))+(((IkReal(0.212131551534496))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829365318545))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.212131551534496))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKcos(j[0]))))+(((IkReal(-0.0159099387882250))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.388907844479909))*(IKcos(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(-0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3])))));
eerot[6]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6])))));
eerot[7]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6])))));
eerot[8]=((((x12)*(x40)))+(((x12)*(x41)))+(((x13)*(x37)))+(((x13)*(x38))));
eetrans[2]=((IkReal(0.378361000000000))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.275000000000000))*(IKcos(j[0]))))+(((IkReal(0.275000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.150000000000000))*(IKcos(j[0]))))+(((IkReal(0.150000000000000))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.275000000000000))*(IKsin(j[0]))))+(((IkReal(0.275000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.150000000000000))*(IKsin(j[0]))))+(((IkReal(0.150000000000000))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1])))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {4}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j7,cj7,sj7,htj7,j9,cj9,sj9,htj9,j10,cj10,sj10,htj10,j8,cj8,sj8,htj8,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij7[2], _nj7,_ij9[2], _nj9,_ij10[2], _nj10,_ij8[2], _nj8;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1; j7=numeric_limits<IkReal>::quiet_NaN(); _ij7[0] = -1; _ij7[1] = -1; _nj7 = -1; j9=numeric_limits<IkReal>::quiet_NaN(); _ij9[0] = -1; _ij9[1] = -1; _nj9 = -1; j10=numeric_limits<IkReal>::quiet_NaN(); _ij10[0] = -1; _ij10[1] = -1; _nj10 = -1;  _ij8[0] = -1; _ij8[1] = -1; _nj8 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j8=pfree[0]; cj8=cos(pfree[0]); sj8=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=((((IkReal(-0.500001138018525))*(r11)))+(((IkReal(0.499998861978885))*(r01)))+(((IkReal(0.707106781186548))*(r21))));
new_r01=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(0.500001138018525))*(r10)))+(((IkReal(-0.499998861978885))*(r00))));
new_r02=((((IkReal(-0.500001138018525))*(r12)))+(((IkReal(0.499998861978885))*(r02)))+(((IkReal(0.707106781186548))*(r22))));
new_px=((IkReal(-0.267542425853256))+(((IkReal(0.499998861978885))*(px)))+(((IkReal(-0.500001138018525))*(py)))+(((IkReal(-0.0889747974891426))*(r02)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.0889752025103964))*(r12))));
new_r10=((((IkReal(0.500001138018525))*(r11)))+(((IkReal(0.707106781186548))*(r21)))+(((IkReal(-0.499998861978885))*(r01))));
new_r11=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(-0.500001138018525))*(r10)))+(((IkReal(0.499998861978885))*(r00))));
new_r12=((((IkReal(0.500001138018525))*(r12)))+(((IkReal(0.707106781186548))*(r22)))+(((IkReal(-0.499998861978885))*(r02))));
new_py=((IkReal(-0.267540831819791))+(((IkReal(-0.499998861978885))*(px)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.500001138018525))*(py)))+(((IkReal(0.0889747974891426))*(r02)))+(((IkReal(-0.0889752025103964))*(r12))));
new_r20=((((IkReal(-0.707108390587779))*(r01)))+(((IkReal(-0.707105171781653))*(r11))));
new_r21=((((IkReal(0.707108390587779))*(r00)))+(((IkReal(0.707105171781653))*(r10))));
new_r22=((((IkReal(-0.707108390587779))*(r02)))+(((IkReal(-0.707105171781653))*(r12))));
new_pz=((IkReal(-0.495225062629841))+(((IkReal(0.125829365318545))*(r12)))+(((IkReal(-0.707105171781653))*(py)))+(((IkReal(0.125829938105095))*(r02)))+(((IkReal(-0.707108390587779))*(px))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j7array[2], cj7array[2], sj7array[2];
bool j7valid[2]={false};
_nj7 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x42=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j7array[0]=((IkReal(1.80132261653346))+(((IkReal(1.00000000000000))*(x42))));
sj7array[0]=IKsin(j7array[0]);
cj7array[0]=IKcos(j7array[0]);
j7array[1]=((IkReal(4.94291527012326))+(((IkReal(-1.00000000000000))*(x42))));
sj7array[1]=IKsin(j7array[1]);
cj7array[1]=IKcos(j7array[1]);
if( j7array[0] > IKPI )
{
    j7array[0]-=IK2PI;
}
else if( j7array[0] < -IKPI )
{    j7array[0]+=IK2PI;
}
j7valid[0] = true;
if( j7array[1] > IKPI )
{
    j7array[1]-=IK2PI;
}
else if( j7array[1] < -IKPI )
{    j7array[1]+=IK2PI;
}
j7valid[1] = true;
for(int ij7 = 0; ij7 < 2; ++ij7)
{
if( !j7valid[ij7] )
{
    continue;
}
_ij7[0] = ij7; _ij7[1] = -1;
for(int iij7 = ij7+1; iij7 < 2; ++iij7)
{
if( j7valid[iij7] && IKabs(cj7array[ij7]-cj7array[iij7]) < IKFAST_SOLUTION_THRESH && IKabs(sj7array[ij7]-sj7array[iij7]) < IKFAST_SOLUTION_THRESH )
{
    j7valid[iij7]=false; _ij7[1] = iij7; break; 
}
}
j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];

{
IkReal dummyeval[1];
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x43=(sj7)*(sj7);
IkReal x44=(cj8)*(cj8);
IkReal x45=(cj7)*(cj7);
IkReal x46=((IkReal(24.4444444444444))*(sj7));
IkReal x47=((cj7)*(x44));
dummyeval[0]=((IkReal(44.4444444444444))+(((IkReal(13.3333333333333))*(sj7)))+(((IkReal(149.382716049383))*(x43)*(x44)))+(((IkReal(-2.00000000000000))*(x47)))+(((cj7)*(x46)))+(x44)+(x43)+(((x44)*(x46)))+(((x44)*(x45)))+(((IkReal(149.382716049383))*(x45)))+(((IkReal(162.962962962963))*(cj7)))+(((IkReal(-1.00000000000000))*(x46)*(x47))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[2], cj9array[2], sj9array[2];
bool j9valid[2]={false};
_nj9 = 2;
IkReal x48=((IkReal(0.0450000000000000))*(cj8));
IkReal x49=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7))));
IkReal x50=((((IkReal(-1.00000000000000))*(x48)))+(((IkReal(-0.550000000000000))*(cj8)*(sj7)))+(((cj7)*(x48))));
if( IKabs(x49) < IKFAST_ATAN2_MAGTHRESH && IKabs(x50) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x51=((IkReal(1.00000000000000))*(IKatan2(x49, x50)));
if( ((((x50)*(x50))+((x49)*(x49)))) < (IkReal)-0.00001 )
    continue;
if( (((npz)*(((IKabs(IKabs(IKsqrt((((x50)*(x50))+((x49)*(x49)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x50)*(x50))+((x49)*(x49))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((npz)*(((IKabs(IKabs(IKsqrt((((x50)*(x50))+((x49)*(x49)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x50)*(x50))+((x49)*(x49))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x52=IKasin(((npz)*(((IKabs(IKabs(IKsqrt((((x50)*(x50))+((x49)*(x49)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x50)*(x50))+((x49)*(x49))))))):(IkReal)1.0e30))));
j9array[0]=((x52)+(((IkReal(-1.00000000000000))*(x51))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
j9array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x51)))+(((IkReal(-1.00000000000000))*(x52))));
sj9array[1]=IKsin(j9array[1]);
cj9array[1]=IKcos(j9array[1]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
if( j9array[1] > IKPI )
{
    j9array[1]-=IK2PI;
}
else if( j9array[1] < -IKPI )
{    j9array[1]+=IK2PI;
}
j9valid[1] = true;
for(int ij9 = 0; ij9 < 2; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 2; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((((sj9)*((npy)*(npy))))+(((sj9)*((npx)*(npx))))));
dummyeval[0]=((((sj9)*((npy)*(npy))))+(((sj9)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x53=((IkReal(0.0450000000000000))*(sj7));
IkReal x54=((IkReal(0.550000000000000))*(cj7));
IkReal x55=((x54)+(x53));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(-0.300000000000000))+(npz)+(((IkReal(-1.00000000000000))*(x55))));
evalcond[3]=((IkReal(0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(x55));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign((((npx)*(npx))+((npy)*(npy))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x56=((IkReal(0.0450000000000000))*(cj8));
IkReal x57=((IkReal(0.0450000000000000))*(sj8));
IkReal x58=((cj7)*(npy));
IkReal x59=((IkReal(0.550000000000000))*(sj7));
IkReal x60=((cj7)*(npx));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x56)*(x58)))+(((npx)*(x57)))+(((npy)*(x56)))+(((cj8)*(npy)*(x59)))+(((npx)*(sj8)*(x59)))+(((IkReal(-1.00000000000000))*(x57)*(x60))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((npy)*(x57)))+(((IkReal(-1.00000000000000))*(npx)*(x56)))+(((npy)*(sj8)*(x59)))+(((x56)*(x60)))+(((IkReal(-1.00000000000000))*(cj8)*(npx)*(x59)))+(((IkReal(-1.00000000000000))*(x57)*(x58))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x56)*(x58)))+(((npx)*(x57)))+(((npy)*(x56)))+(((cj8)*(npy)*(x59)))+(((npx)*(sj8)*(x59)))+(((IkReal(-1.00000000000000))*(x57)*(x60)))))), ((gconst1)*(((((npy)*(x57)))+(((IkReal(-1.00000000000000))*(npx)*(x56)))+(((npy)*(sj8)*(x59)))+(((x56)*(x60)))+(((IkReal(-1.00000000000000))*(cj8)*(npx)*(x59)))+(((IkReal(-1.00000000000000))*(x57)*(x58)))))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[2];
IkReal x61=IKsin(j10);
IkReal x62=IKcos(j10);
IkReal x63=((IkReal(0.0450000000000000))*(cj7));
IkReal x64=((IkReal(0.550000000000000))*(sj7));
IkReal x65=((IkReal(1.00000000000000))*(x62));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj8)*(x63)))+(((IkReal(-1.00000000000000))*(npy)*(x65)))+(((IkReal(0.0450000000000000))*(sj8)))+(((sj8)*(x64)))+(((IkReal(-1.00000000000000))*(npx)*(x61))));
evalcond[1]=((((npy)*(x61)))+(((IkReal(-1.00000000000000))*(cj8)*(x64)))+(((cj8)*(x63)))+(((IkReal(-1.00000000000000))*(npx)*(x65)))+(((IkReal(-0.0450000000000000))*(cj8))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x300=((IkReal(-0.300000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x300;
evalcond[3]=x300;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign((((npx)*(npx))+((npy)*(npy))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x301=((IkReal(0.0450000000000000))*(npy));
IkReal x302=((cj7)*(cj8));
IkReal x303=((npx)*(sj8));
IkReal x304=((IkReal(0.550000000000000))*(sj7));
IkReal x305=((cj8)*(npx));
if( IKabs(((gconst2)*(((((x301)*(x302)))+(((IkReal(-1.00000000000000))*(cj8)*(npy)*(x304)))+(((x303)*(x304)))+(((IkReal(0.0450000000000000))*(x303)))+(((IkReal(-1.00000000000000))*(cj8)*(x301)))+(((IkReal(-0.0450000000000000))*(cj7)*(x303))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((sj8)*(x301)))+(((npy)*(sj8)*(x304)))+(((IkReal(-0.0450000000000000))*(npx)*(x302)))+(((IkReal(-1.00000000000000))*(cj7)*(sj8)*(x301)))+(((IkReal(0.0450000000000000))*(x305)))+(((x304)*(x305))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst2)*(((((x301)*(x302)))+(((IkReal(-1.00000000000000))*(cj8)*(npy)*(x304)))+(((x303)*(x304)))+(((IkReal(0.0450000000000000))*(x303)))+(((IkReal(-1.00000000000000))*(cj8)*(x301)))+(((IkReal(-0.0450000000000000))*(cj7)*(x303)))))), ((gconst2)*(((((sj8)*(x301)))+(((npy)*(sj8)*(x304)))+(((IkReal(-0.0450000000000000))*(npx)*(x302)))+(((IkReal(-1.00000000000000))*(cj7)*(sj8)*(x301)))+(((IkReal(0.0450000000000000))*(x305)))+(((x304)*(x305)))))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[2];
IkReal x306=IKcos(j10);
IkReal x307=IKsin(j10);
IkReal x308=((IkReal(1.00000000000000))*(npy));
IkReal x309=((IkReal(0.0450000000000000))*(cj7));
IkReal x310=((IkReal(0.550000000000000))*(sj7));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(x307)))+(((IkReal(0.0450000000000000))*(sj8)))+(((IkReal(-1.00000000000000))*(x306)*(x308)))+(((sj8)*(x310)))+(((IkReal(-1.00000000000000))*(sj8)*(x309))));
evalcond[1]=((((cj8)*(x309)))+(((IkReal(-0.0450000000000000))*(cj8)))+(((IkReal(-1.00000000000000))*(x307)*(x308)))+(((IkReal(-1.00000000000000))*(cj8)*(x310)))+(((npx)*(x306))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x311=((IkReal(0.550000000000000))*(npy));
IkReal x312=((sj8)*(sj9));
IkReal x313=((cj9)*(npz));
IkReal x314=((IkReal(0.0450000000000000))*(cj7));
IkReal x315=((npx)*(sj7));
IkReal x316=((IkReal(0.0450000000000000))*(npy));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x312)*(x314)))+(((IkReal(0.300000000000000))*(npy)))+(((sj7)*(x316)))+(((IkReal(-1.00000000000000))*(npy)*(x313)))+(((IkReal(0.0450000000000000))*(npx)*(x312)))+(((IkReal(0.550000000000000))*(x312)*(x315)))+(((cj7)*(x311))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-0.550000000000000))*(cj7)*(npx)))+(((x312)*(x316)))+(((IkReal(-1.00000000000000))*(npy)*(x312)*(x314)))+(((sj7)*(x311)*(x312)))+(((IkReal(-0.0450000000000000))*(x315)))+(((npx)*(x313)))+(((IkReal(-0.300000000000000))*(npx))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(npx)*(x312)*(x314)))+(((IkReal(0.300000000000000))*(npy)))+(((sj7)*(x316)))+(((IkReal(-1.00000000000000))*(npy)*(x313)))+(((IkReal(0.0450000000000000))*(npx)*(x312)))+(((IkReal(0.550000000000000))*(x312)*(x315)))+(((cj7)*(x311)))))), ((gconst0)*(((((IkReal(-0.550000000000000))*(cj7)*(npx)))+(((x312)*(x316)))+(((IkReal(-1.00000000000000))*(npy)*(x312)*(x314)))+(((sj7)*(x311)*(x312)))+(((IkReal(-0.0450000000000000))*(x315)))+(((npx)*(x313)))+(((IkReal(-0.300000000000000))*(npx)))))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x317=IKsin(j10);
IkReal x318=IKcos(j10);
IkReal x319=((IkReal(0.0450000000000000))*(cj8));
IkReal x320=((IkReal(0.0450000000000000))*(sj8));
IkReal x321=((IkReal(0.0450000000000000))*(sj7));
IkReal x322=((IkReal(0.550000000000000))*(cj7));
IkReal x323=((IkReal(1.00000000000000))*(x318));
IkReal x324=((npy)*(x317));
IkReal x325=((IkReal(0.550000000000000))*(cj8)*(sj7));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x323)))+(x320)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(npx)*(x317)))+(((IkReal(-1.00000000000000))*(cj7)*(x320))));
evalcond[1]=((IkReal(-0.300000000000000))+(((sj9)*(x324)))+(((IkReal(-1.00000000000000))*(x322)))+(((IkReal(-1.00000000000000))*(x321)))+(((cj9)*(npz)))+(((IkReal(-1.00000000000000))*(npx)*(sj9)*(x323))));
evalcond[2]=((((cj9)*(x324)))+(((IkReal(-1.00000000000000))*(x325)))+(((IkReal(-1.00000000000000))*(x319)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(npx)*(x323)))+(((cj7)*(x319))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj9)*(x322)))+(((IkReal(-1.00000000000000))*(sj9)*(x321)))+(((IkReal(-1.00000000000000))*(npx)*(x323)))+(((IkReal(-1.00000000000000))*(cj9)*(x325)))+(((cj7)*(cj9)*(x319)))+(x324)+(((IkReal(-1.00000000000000))*(cj9)*(x319)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[2], cj10array[2], sj10array[2];
bool j10valid[2]={false};
_nj10 = 2;
IkReal x326=((IkReal(0.0450000000000000))*(sj8));
if( IKabs(((IkReal(-1.00000000000000))*(npy))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npx))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x327=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(npy)), ((IkReal(-1.00000000000000))*(npx)))));
if( ((((npx)*(npx))+((npy)*(npy)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy))))))):(IkReal)1.0e30))*(((x326)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(cj7)*(x326))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy))))))):(IkReal)1.0e30))*(((x326)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(cj7)*(x326))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x328=IKasin(((((IKabs(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy))))))):(IkReal)1.0e30))*(((x326)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(cj7)*(x326)))))));
j10array[0]=((((IkReal(-1.00000000000000))*(x327)))+(((IkReal(-1.00000000000000))*(x328))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
j10array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x327)))+(x328));
sj10array[1]=IKsin(j10array[1]);
cj10array[1]=IKcos(j10array[1]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
if( j10array[1] > IKPI )
{
    j10array[1]-=IK2PI;
}
else if( j10array[1] < -IKPI )
{    j10array[1]+=IK2PI;
}
j10valid[1] = true;
for(int ij10 = 0; ij10 < 2; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 2; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx))))));
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[1], cj9array[1], sj9array[1];
bool j9valid[1]={false};
_nj9 = 1;
IkReal x329=((cj7)*(cj8));
IkReal x330=((IkReal(0.550000000000000))*(cj7));
IkReal x331=((cj10)*(npx));
IkReal x332=((npy)*(sj10));
IkReal x333=((IkReal(0.0450000000000000))*(npz));
IkReal x334=((IkReal(0.0450000000000000))*(x332));
IkReal x335=((IkReal(0.550000000000000))*(cj8)*(sj7));
if( IKabs(((gconst3)*(((((IkReal(0.300000000000000))*(x332)))+(((x330)*(x332)))+(((IkReal(-0.300000000000000))*(x331)))+(((IkReal(-0.0450000000000000))*(sj7)*(x331)))+(((IkReal(-1.00000000000000))*(x330)*(x331)))+(((x329)*(x333)))+(((IkReal(-1.00000000000000))*(cj8)*(x333)))+(((IkReal(-1.00000000000000))*(npz)*(x335)))+(((sj7)*(x334))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(0.0450000000000000))*(x329)*(x331)))+(((cj8)*(x334)))+(((IkReal(0.300000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x329)*(x334)))+(((x332)*(x335)))+(((npz)*(x330)))+(((IkReal(-1.00000000000000))*(x331)*(x335)))+(((sj7)*(x333)))+(((IkReal(-0.0450000000000000))*(cj8)*(x331))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j9array[0]=IKatan2(((gconst3)*(((((IkReal(0.300000000000000))*(x332)))+(((x330)*(x332)))+(((IkReal(-0.300000000000000))*(x331)))+(((IkReal(-0.0450000000000000))*(sj7)*(x331)))+(((IkReal(-1.00000000000000))*(x330)*(x331)))+(((x329)*(x333)))+(((IkReal(-1.00000000000000))*(cj8)*(x333)))+(((IkReal(-1.00000000000000))*(npz)*(x335)))+(((sj7)*(x334)))))), ((gconst3)*(((((IkReal(0.0450000000000000))*(x329)*(x331)))+(((cj8)*(x334)))+(((IkReal(0.300000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x329)*(x334)))+(((x332)*(x335)))+(((npz)*(x330)))+(((IkReal(-1.00000000000000))*(x331)*(x335)))+(((sj7)*(x333)))+(((IkReal(-0.0450000000000000))*(cj8)*(x331)))))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
for(int ij9 = 0; ij9 < 1; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 1; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];
{
IkReal evalcond[4];
IkReal x336=IKsin(j9);
IkReal x337=IKcos(j9);
IkReal x338=((IkReal(0.0450000000000000))*(cj8));
IkReal x339=((IkReal(1.00000000000000))*(npz));
IkReal x340=((npy)*(sj10));
IkReal x341=((IkReal(0.0450000000000000))*(sj7));
IkReal x342=((IkReal(1.00000000000000))*(cj10)*(npx));
IkReal x343=((cj7)*(x336));
IkReal x344=((IkReal(0.550000000000000))*(cj8)*(sj7));
IkReal x345=((cj7)*(x337));
evalcond[0]=((IkReal(-0.300000000000000))+(((x336)*(x340)))+(((IkReal(-1.00000000000000))*(x336)*(x342)))+(((IkReal(-0.550000000000000))*(cj7)))+(((npz)*(x337)))+(((IkReal(-1.00000000000000))*(x341))));
evalcond[1]=((((x337)*(x340)))+(((IkReal(-1.00000000000000))*(x337)*(x342)))+(((cj7)*(x338)))+(((IkReal(-1.00000000000000))*(x338)))+(((IkReal(-1.00000000000000))*(x344)))+(((IkReal(-1.00000000000000))*(x336)*(x339))));
evalcond[2]=((((IkReal(0.300000000000000))*(x337)))+(((x337)*(x341)))+(((IkReal(-1.00000000000000))*(x336)*(x344)))+(((IkReal(0.550000000000000))*(x345)))+(((IkReal(-1.00000000000000))*(x339)))+(((x338)*(x343)))+(((IkReal(-1.00000000000000))*(x336)*(x338))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x337)*(x338)))+(((IkReal(-1.00000000000000))*(x336)*(x341)))+(((IkReal(-1.00000000000000))*(x337)*(x344)))+(((IkReal(-0.300000000000000))*(x336)))+(((IkReal(-0.550000000000000))*(x343)))+(x340)+(((x338)*(x345)))+(((IkReal(-1.00000000000000))*(x342))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[2], cj10array[2], sj10array[2];
bool j10valid[2]={false};
_nj10 = 2;
IkReal x346=((IkReal(0.0450000000000000))*(sj8));
if( IKabs(((IkReal(-1.00000000000000))*(npy))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npx))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x347=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(npy)), ((IkReal(-1.00000000000000))*(npx)))));
if( ((((npx)*(npx))+((npy)*(npy)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy))))))):(IkReal)1.0e30))*(((x346)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(cj7)*(x346))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy))))))):(IkReal)1.0e30))*(((x346)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(cj7)*(x346))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x348=IKasin(((((IKabs(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((npx)*(npx))+((npy)*(npy))))))):(IkReal)1.0e30))*(((x346)+(((IkReal(0.550000000000000))*(sj7)*(sj8)))+(((IkReal(-1.00000000000000))*(cj7)*(x346)))))));
j10array[0]=((((IkReal(-1.00000000000000))*(x347)))+(((IkReal(-1.00000000000000))*(x348))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
j10array[1]=((IkReal(3.14159265358979))+(x348)+(((IkReal(-1.00000000000000))*(x347))));
sj10array[1]=IKsin(j10array[1]);
cj10array[1]=IKcos(j10array[1]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
if( j10array[1] > IKPI )
{
    j10array[1]-=IK2PI;
}
else if( j10array[1] < -IKPI )
{    j10array[1]+=IK2PI;
}
j10valid[1] = true;
for(int ij10 = 0; ij10 < 2; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 2; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];

{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx))))));
dummyeval[0]=(((((npy)*(npy))*((sj10)*(sj10))))+(((IkReal(-2.00000000000000))*(cj10)*(npx)*(npy)*(sj10)))+((npz)*(npz))+((((cj10)*(cj10))*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j9array[1], cj9array[1], sj9array[1];
bool j9valid[1]={false};
_nj9 = 1;
IkReal x349=((cj7)*(cj8));
IkReal x350=((IkReal(0.550000000000000))*(cj7));
IkReal x351=((cj10)*(npx));
IkReal x352=((npy)*(sj10));
IkReal x353=((IkReal(0.0450000000000000))*(npz));
IkReal x354=((IkReal(0.0450000000000000))*(x352));
IkReal x355=((IkReal(0.550000000000000))*(cj8)*(sj7));
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(npz)*(x355)))+(((IkReal(-0.300000000000000))*(x351)))+(((IkReal(-1.00000000000000))*(cj8)*(x353)))+(((IkReal(0.300000000000000))*(x352)))+(((x349)*(x353)))+(((x350)*(x352)))+(((sj7)*(x354)))+(((IkReal(-0.0450000000000000))*(sj7)*(x351)))+(((IkReal(-1.00000000000000))*(x350)*(x351))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(0.0450000000000000))*(x349)*(x351)))+(((IkReal(0.300000000000000))*(npz)))+(((cj8)*(x354)))+(((x352)*(x355)))+(((IkReal(-1.00000000000000))*(x349)*(x354)))+(((sj7)*(x353)))+(((IkReal(-1.00000000000000))*(x351)*(x355)))+(((npz)*(x350)))+(((IkReal(-0.0450000000000000))*(cj8)*(x351))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j9array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(npz)*(x355)))+(((IkReal(-0.300000000000000))*(x351)))+(((IkReal(-1.00000000000000))*(cj8)*(x353)))+(((IkReal(0.300000000000000))*(x352)))+(((x349)*(x353)))+(((x350)*(x352)))+(((sj7)*(x354)))+(((IkReal(-0.0450000000000000))*(sj7)*(x351)))+(((IkReal(-1.00000000000000))*(x350)*(x351)))))), ((gconst3)*(((((IkReal(0.0450000000000000))*(x349)*(x351)))+(((IkReal(0.300000000000000))*(npz)))+(((cj8)*(x354)))+(((x352)*(x355)))+(((IkReal(-1.00000000000000))*(x349)*(x354)))+(((sj7)*(x353)))+(((IkReal(-1.00000000000000))*(x351)*(x355)))+(((npz)*(x350)))+(((IkReal(-0.0450000000000000))*(cj8)*(x351)))))));
sj9array[0]=IKsin(j9array[0]);
cj9array[0]=IKcos(j9array[0]);
if( j9array[0] > IKPI )
{
    j9array[0]-=IK2PI;
}
else if( j9array[0] < -IKPI )
{    j9array[0]+=IK2PI;
}
j9valid[0] = true;
for(int ij9 = 0; ij9 < 1; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 1; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];
{
IkReal evalcond[4];
IkReal x356=IKsin(j9);
IkReal x357=IKcos(j9);
IkReal x358=((IkReal(0.0450000000000000))*(cj8));
IkReal x359=((IkReal(1.00000000000000))*(npz));
IkReal x360=((npy)*(sj10));
IkReal x361=((IkReal(0.0450000000000000))*(sj7));
IkReal x362=((IkReal(1.00000000000000))*(cj10)*(npx));
IkReal x363=((cj7)*(x356));
IkReal x364=((IkReal(0.550000000000000))*(cj8)*(sj7));
IkReal x365=((cj7)*(x357));
evalcond[0]=((IkReal(-0.300000000000000))+(((IkReal(-0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(x361)))+(((IkReal(-1.00000000000000))*(x356)*(x362)))+(((x356)*(x360)))+(((npz)*(x357))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x364)))+(((IkReal(-1.00000000000000))*(x358)))+(((IkReal(-1.00000000000000))*(x356)*(x359)))+(((cj7)*(x358)))+(((IkReal(-1.00000000000000))*(x357)*(x362)))+(((x357)*(x360))));
evalcond[2]=((((x358)*(x363)))+(((IkReal(-1.00000000000000))*(x359)))+(((IkReal(-1.00000000000000))*(x356)*(x358)))+(((IkReal(-1.00000000000000))*(x356)*(x364)))+(((IkReal(0.550000000000000))*(x365)))+(((IkReal(0.300000000000000))*(x357)))+(((x357)*(x361))));
evalcond[3]=((((x358)*(x365)))+(((IkReal(-1.00000000000000))*(x357)*(x358)))+(((IkReal(-1.00000000000000))*(x362)))+(((IkReal(-0.300000000000000))*(x356)))+(((IkReal(-1.00000000000000))*(x356)*(x361)))+(x360)+(((IkReal(-0.550000000000000))*(x363)))+(((IkReal(-1.00000000000000))*(x357)*(x364))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x66=((IkReal(1.00000000000000))*(sj7));
IkReal x67=((IkReal(1.00000000000000))*(sj10));
IkReal x68=((IkReal(1.00000000000000))*(cj7));
IkReal x69=((IkReal(1.00000000000000))*(cj9));
IkReal x70=((IkReal(-1.00000000000000))*(cj9));
IkReal x71=((((IkReal(-1.00000000000000))*(r01)*(x67)))+(((cj10)*(r00))));
IkReal x72=((((r00)*(sj10)))+(((cj10)*(r01))));
IkReal x73=((((IkReal(-1.00000000000000))*(r11)*(x67)))+(((cj10)*(r10))));
IkReal x74=((((r10)*(sj10)))+(((cj10)*(r11))));
IkReal x75=((((IkReal(-1.00000000000000))*(r21)*(x67)))+(((cj10)*(r20))));
IkReal x76=((((r20)*(sj10)))+(((cj10)*(r21))));
IkReal x77=((sj9)*(x71));
IkReal x78=((sj9)*(x73));
IkReal x79=((sj9)*(x75));
IkReal x80=((sj8)*(x76));
IkReal x81=((sj8)*(x72));
IkReal x82=((sj8)*(x74));
IkReal x83=((((r02)*(sj9)))+(((cj9)*(x71))));
IkReal x84=((((r12)*(sj9)))+(((cj9)*(x73))));
IkReal x85=((((r22)*(sj9)))+(((cj9)*(x75))));
IkReal x86=((cj8)*(x83));
IkReal x87=((cj8)*(x84));
IkReal x88=((cj8)*(x85));
new_r00=((((IkReal(-1.00000000000000))*(x66)*(((((IkReal(-1.00000000000000))*(r02)*(x69)))+(x77)))))+(((cj7)*(((x86)+(((IkReal(-1.00000000000000))*(x81))))))));
new_r01=((((sj8)*(x83)))+(((cj8)*(x72))));
new_r02=((((IkReal(-1.00000000000000))*(x68)*(((((r02)*(x70)))+(x77)))))+(((IkReal(-1.00000000000000))*(x66)*(((x86)+(((IkReal(-1.00000000000000))*(x81))))))));
new_r10=((((cj7)*(((x87)+(((IkReal(-1.00000000000000))*(x82)))))))+(((IkReal(-1.00000000000000))*(x66)*(((((IkReal(-1.00000000000000))*(r12)*(x69)))+(x78))))));
new_r11=((((sj8)*(x84)))+(((cj8)*(x74))));
new_r12=((((IkReal(-1.00000000000000))*(x66)*(((x87)+(((IkReal(-1.00000000000000))*(x82)))))))+(((IkReal(-1.00000000000000))*(x68)*(((x78)+(((r12)*(x70))))))));
new_r20=((((cj7)*(((x88)+(((IkReal(-1.00000000000000))*(x80)))))))+(((IkReal(-1.00000000000000))*(x66)*(((x79)+(((IkReal(-1.00000000000000))*(r22)*(x69))))))));
new_r21=((((sj8)*(x85)))+(((cj8)*(x76))));
new_r22=((((IkReal(-1.00000000000000))*(x68)*(((((r22)*(x70)))+(x79)))))+(((IkReal(-1.00000000000000))*(x66)*(((x88)+(((IkReal(-1.00000000000000))*(x80))))))));
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
cj5array[0]=new_r22;
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((IkReal(2.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(2.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x89=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x89));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x89))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x90=IKcos(j4);
IkReal x91=IKsin(j4);
IkReal x92=((IkReal(0.707106781186548))*(new_r02));
IkReal x93=((new_r12)*(x90));
IkReal x94=((IkReal(0.707106781186548))*(new_r12)*(x91));
IkReal x95=((x90)*(x92));
IkReal x96=((IkReal(0.707106781186548))*(x93));
IkReal x97=((x91)*(x92));
IkReal x98=((x95)+(x94));
evalcond[0]=((x93)+(((IkReal(-1.00000000000000))*(new_r02)*(x91))));
evalcond[1]=((x96)+(((IkReal(-1.00000000000000))*(x97)))+(((IkReal(-1.00000000000000))*(x98))));
evalcond[2]=((x98)+(x96)+(((IkReal(-1.00000000000000))*(x97))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x99=((IkReal(0.707106781186548))*(sj4));
IkReal x100=((IkReal(0.707106781186548))*(cj4));
IkReal x101=((new_r10)*(x99));
IkReal x102=((new_r00)*(x100));
IkReal x103=((new_r11)*(x99));
IkReal x104=((new_r01)*(x100));
IkReal x105=((x104)+(x103));
IkReal x106=((x101)+(x102));
if( IKabs(((((IkReal(-1.00000000000000))*(x106)))+(((IkReal(-1.00000000000000))*(x105))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x105)))+(x106))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x106)))+(((IkReal(-1.00000000000000))*(x105)))))+IKsqr(((((IkReal(-1.00000000000000))*(x105)))+(x106)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x106)))+(((IkReal(-1.00000000000000))*(x105)))), ((((IkReal(-1.00000000000000))*(x105)))+(x106)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x107=IKsin(j6);
IkReal x108=IKcos(j6);
IkReal x109=((cj4)*(new_r00));
IkReal x110=((cj4)*(new_r01));
IkReal x111=((IkReal(0.707106781186548))*(sj4));
IkReal x112=((IkReal(0.707106781186548))*(cj4));
IkReal x113=((IkReal(1.00000000000000))*(sj4));
IkReal x114=((IkReal(0.707106781186548))*(x108));
IkReal x115=((IkReal(0.707106781186548))*(x107));
IkReal x116=((new_r11)*(x111));
IkReal x117=((IkReal(0.707106781186548))*(x110));
IkReal x118=((new_r11)*(x112));
IkReal x119=((new_r01)*(x111));
IkReal x120=((new_r10)*(x111));
IkReal x121=((IkReal(0.707106781186548))*(x109));
IkReal x122=((new_r10)*(x112));
IkReal x123=((new_r00)*(x111));
IkReal x124=((IkReal(1.00000000000000))*(x108));
IkReal x125=((x115)+(x114));
IkReal x126=((x120)+(x121));
IkReal x127=((x117)+(x116));
evalcond[0]=((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x114)))+(x115)+(x109));
evalcond[1]=((((new_r11)*(sj4)))+(x125)+(x110));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x113)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x125))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x113)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x114)))+(x115));
evalcond[4]=((((IkReal(-1.00000000000000))*(x123)))+(((IkReal(-1.00000000000000))*(x126)))+(x122)+(((IkReal(-1.00000000000000))*(x107))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(x124)))+(((IkReal(-1.00000000000000))*(x127)))+(x118));
evalcond[6]=((((IkReal(-1.00000000000000))*(x119)))+(x127)+(x118)+(x107));
evalcond[7]=((((IkReal(-1.00000000000000))*(x123)))+(((IkReal(-1.00000000000000))*(x124)))+(x126)+(x122));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x128=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x128;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x128;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x129=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x129));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x129))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x130=IKcos(j4);
IkReal x131=IKsin(j4);
IkReal x132=((IkReal(0.707106781186548))*(new_r02));
IkReal x133=((new_r12)*(x130));
IkReal x134=((x131)*(x132));
IkReal x135=((IkReal(0.707106781186548))*(new_r12)*(x131));
IkReal x136=((x130)*(x132));
IkReal x137=((IkReal(0.707106781186548))*(x133));
IkReal x138=((x135)+(x136));
evalcond[0]=((x133)+(((IkReal(-1.00000000000000))*(new_r02)*(x131))));
evalcond[1]=((x137)+(((IkReal(-1.00000000000000))*(x134)))+(((IkReal(-1.00000000000000))*(x138))));
evalcond[2]=((x134)+(((IkReal(-1.00000000000000))*(x137)))+(((IkReal(-1.00000000000000))*(x138))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x139=((IkReal(0.707106781186548))*(sj4));
IkReal x140=((IkReal(0.707106781186548))*(cj4));
IkReal x141=((new_r11)*(x139));
IkReal x142=((new_r10)*(x139));
IkReal x143=((new_r00)*(x140));
IkReal x144=((new_r01)*(x140));
IkReal x145=((x141)+(x144));
IkReal x146=((x142)+(x143));
if( IKabs(((((IkReal(-1.00000000000000))*(x146)))+(x145))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x146)))+(((IkReal(-1.00000000000000))*(x145))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x146)))+(x145)))+IKsqr(((((IkReal(-1.00000000000000))*(x146)))+(((IkReal(-1.00000000000000))*(x145)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x146)))+(x145)), ((((IkReal(-1.00000000000000))*(x146)))+(((IkReal(-1.00000000000000))*(x145)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x147=IKsin(j6);
IkReal x148=IKcos(j6);
IkReal x149=((cj4)*(new_r00));
IkReal x150=((cj4)*(new_r01));
IkReal x151=((IkReal(0.707106781186548))*(sj4));
IkReal x152=((IkReal(0.707106781186548))*(cj4));
IkReal x153=((IkReal(1.00000000000000))*(sj4));
IkReal x154=((IkReal(0.707106781186548))*(x148));
IkReal x155=((IkReal(0.707106781186548))*(x147));
IkReal x156=((new_r01)*(x151));
IkReal x157=((new_r11)*(x151));
IkReal x158=((IkReal(0.707106781186548))*(x150));
IkReal x159=((new_r11)*(x152));
IkReal x160=((new_r00)*(x151));
IkReal x161=((IkReal(1.00000000000000))*(x148));
IkReal x162=((new_r10)*(x151));
IkReal x163=((IkReal(0.707106781186548))*(x149));
IkReal x164=((new_r10)*(x152));
IkReal x165=((x162)+(x163));
IkReal x166=((x157)+(x158));
IkReal x167=((x155)+(x154));
evalcond[0]=((((new_r10)*(sj4)))+(x167)+(x149));
evalcond[1]=((((IkReal(-1.00000000000000))*(x155)))+(((new_r11)*(sj4)))+(x154)+(x150));
evalcond[2]=((((IkReal(-1.00000000000000))*(x155)))+(((cj4)*(new_r10)))+(x154)+(((IkReal(-1.00000000000000))*(new_r00)*(x153))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x167)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x153))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x165)))+(((IkReal(-1.00000000000000))*(x160)))+(x164)+(((IkReal(-1.00000000000000))*(x147))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x156)))+(((IkReal(-1.00000000000000))*(x166)))+(((IkReal(-1.00000000000000))*(x161)))+(x159));
evalcond[6]=((((IkReal(-1.00000000000000))*(x159)))+(((IkReal(-1.00000000000000))*(x166)))+(x156)+(x147));
evalcond[7]=((((IkReal(-1.00000000000000))*(x165)))+(((IkReal(-1.00000000000000))*(x164)))+(((IkReal(-1.00000000000000))*(x161)))+(x160));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x168=((IkReal(1.41421356237310))*(sj5));
IkReal x169=((new_r12)*(x168));
IkReal x170=((new_r02)*(x168));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x170)))+(x169))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((x170)+(x169))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x170)))+(x169)))), ((gconst4)*(((x170)+(x169)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[7];
IkReal x171=IKcos(j4);
IkReal x172=((IkReal(0.707106781186548))*(sj5));
IkReal x173=IKsin(j4);
IkReal x174=((IkReal(1.00000000000000))*(cj5));
IkReal x175=((IkReal(0.707106781186548))*(cj5));
IkReal x176=((new_r02)*(x171));
IkReal x177=((new_r12)*(x171));
IkReal x178=((new_r12)*(x173));
IkReal x179=((new_r02)*(x173));
IkReal x180=((IkReal(0.707106781186548))*(x177));
evalcond[0]=((((IkReal(-1.00000000000000))*(x172)))+(x178)+(x176));
evalcond[1]=((((IkReal(-1.00000000000000))*(x172)))+(((IkReal(-1.00000000000000))*(x179)))+(x177));
evalcond[2]=((x180)+(((IkReal(-0.707106781186548))*(x179)))+(((IkReal(-0.707106781186548))*(x178)))+(((IkReal(-0.707106781186548))*(x176))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x175)*(x179)))+(((x175)*(x176)))+(((x175)*(x177)))+(((x175)*(x178)))+(((IkReal(-1.00000000000000))*(new_r22)*(sj5))));
evalcond[4]=((((new_r00)*(x172)*(x173)))+(((IkReal(-1.00000000000000))*(new_r20)*(x174)))+(((IkReal(-1.00000000000000))*(new_r10)*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(new_r10)*(x172)*(x173)))+(((IkReal(-1.00000000000000))*(new_r00)*(x171)*(x172))));
evalcond[5]=((((IkReal(-1.00000000000000))*(new_r01)*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(new_r21)*(x174)))+(((IkReal(-1.00000000000000))*(new_r11)*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(new_r11)*(x172)*(x173)))+(((new_r01)*(x172)*(x173))));
evalcond[6]=((IkReal(1.00000000000000))+(((x172)*(x179)))+(((IkReal(-1.00000000000000))*(new_r22)*(x174)))+(((IkReal(-1.00000000000000))*(x172)*(x177)))+(((IkReal(-1.00000000000000))*(x172)*(x176)))+(((IkReal(-1.00000000000000))*(x172)*(x178))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x181=((cj4)*(new_r02));
IkReal x182=((cj4)*(new_r12));
IkReal x183=((new_r02)*(sj4));
IkReal x184=((new_r12)*(sj4));
IkReal x185=((IkReal(0.707106781186548))*(x181));
IkReal x186=((IkReal(0.707106781186548))*(x184));
IkReal x187=((IkReal(0.707106781186548))*(x182));
IkReal x188=((IkReal(0.707106781186548))*(x183));
IkReal x189=((x185)+(x186));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x184)+(x181));
evalcond[5]=((((IkReal(-1.00000000000000))*(x183)))+(x182));
evalcond[6]=((((IkReal(-1.00000000000000))*(x189)))+(((IkReal(-1.00000000000000))*(x188)))+(x187));
evalcond[7]=((((IkReal(-1.00000000000000))*(x188)))+(x187)+(x189));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x190=((IkReal(0.707106781186548))*(sj4));
IkReal x191=((IkReal(0.707106781186548))*(cj4));
IkReal x192=((new_r10)*(x190));
IkReal x193=((new_r00)*(x191));
IkReal x194=((new_r11)*(x190));
IkReal x195=((new_r01)*(x191));
IkReal x196=((x195)+(x194));
IkReal x197=((x193)+(x192));
if( IKabs(((((IkReal(-1.00000000000000))*(x196)))+(((IkReal(-1.00000000000000))*(x197))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x196)))+(x197))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x196)))+(((IkReal(-1.00000000000000))*(x197)))))+IKsqr(((((IkReal(-1.00000000000000))*(x196)))+(x197)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x196)))+(((IkReal(-1.00000000000000))*(x197)))), ((((IkReal(-1.00000000000000))*(x196)))+(x197)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x198=IKsin(j6);
IkReal x199=IKcos(j6);
IkReal x200=((cj4)*(new_r00));
IkReal x201=((cj4)*(new_r01));
IkReal x202=((IkReal(0.707106781186548))*(sj4));
IkReal x203=((IkReal(0.707106781186548))*(cj4));
IkReal x204=((IkReal(1.00000000000000))*(sj4));
IkReal x205=((IkReal(0.707106781186548))*(x199));
IkReal x206=((IkReal(0.707106781186548))*(x198));
IkReal x207=((new_r11)*(x202));
IkReal x208=((IkReal(0.707106781186548))*(x201));
IkReal x209=((new_r11)*(x203));
IkReal x210=((new_r01)*(x202));
IkReal x211=((new_r10)*(x202));
IkReal x212=((IkReal(0.707106781186548))*(x200));
IkReal x213=((new_r10)*(x203));
IkReal x214=((new_r00)*(x202));
IkReal x215=((IkReal(1.00000000000000))*(x199));
IkReal x216=((x206)+(x205));
IkReal x217=((x211)+(x212));
IkReal x218=((x207)+(x208));
evalcond[0]=((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x205)))+(x206)+(x200));
evalcond[1]=((((new_r11)*(sj4)))+(x216)+(x201));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x204)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x216))));
evalcond[3]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x205)))+(x206)+(((IkReal(-1.00000000000000))*(new_r01)*(x204))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x198)))+(x213)+(((IkReal(-1.00000000000000))*(x217)))+(((IkReal(-1.00000000000000))*(x214))));
evalcond[5]=((x209)+(((IkReal(-1.00000000000000))*(x210)))+(((IkReal(-1.00000000000000))*(x218)))+(((IkReal(-1.00000000000000))*(x215))));
evalcond[6]=((x198)+(x218)+(x209)+(((IkReal(-1.00000000000000))*(x210))));
evalcond[7]=((x213)+(x217)+(((IkReal(-1.00000000000000))*(x215)))+(((IkReal(-1.00000000000000))*(x214))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x219=((IkReal(1.00000000000000))+(new_r22));
IkReal x220=((cj4)*(new_r02));
IkReal x221=((cj4)*(new_r12));
IkReal x222=((new_r02)*(sj4));
IkReal x223=((new_r12)*(sj4));
IkReal x224=((IkReal(0.707106781186548))*(x222));
IkReal x225=((IkReal(0.707106781186548))*(x220));
IkReal x226=((IkReal(0.707106781186548))*(x223));
IkReal x227=((IkReal(0.707106781186548))*(x221));
IkReal x228=((x225)+(x226));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x219;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x220)+(x223));
evalcond[5]=((x221)+(((IkReal(-1.00000000000000))*(x222))));
evalcond[6]=((x227)+(((IkReal(-1.00000000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x224))));
evalcond[7]=((x224)+(((IkReal(-1.00000000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x227))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x219;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x229=((IkReal(0.707106781186548))*(sj4));
IkReal x230=((IkReal(0.707106781186548))*(cj4));
IkReal x231=((new_r11)*(x229));
IkReal x232=((new_r10)*(x229));
IkReal x233=((new_r00)*(x230));
IkReal x234=((new_r01)*(x230));
IkReal x235=((x231)+(x234));
IkReal x236=((x232)+(x233));
if( IKabs(((((IkReal(-1.00000000000000))*(x236)))+(x235))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x235)))+(((IkReal(-1.00000000000000))*(x236))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x236)))+(x235)))+IKsqr(((((IkReal(-1.00000000000000))*(x235)))+(((IkReal(-1.00000000000000))*(x236)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x236)))+(x235)), ((((IkReal(-1.00000000000000))*(x235)))+(((IkReal(-1.00000000000000))*(x236)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x237=IKsin(j6);
IkReal x238=IKcos(j6);
IkReal x239=((cj4)*(new_r00));
IkReal x240=((cj4)*(new_r01));
IkReal x241=((IkReal(0.707106781186548))*(sj4));
IkReal x242=((IkReal(0.707106781186548))*(cj4));
IkReal x243=((IkReal(1.00000000000000))*(sj4));
IkReal x244=((IkReal(0.707106781186548))*(x238));
IkReal x245=((IkReal(0.707106781186548))*(x237));
IkReal x246=((new_r01)*(x241));
IkReal x247=((new_r11)*(x241));
IkReal x248=((IkReal(0.707106781186548))*(x240));
IkReal x249=((new_r11)*(x242));
IkReal x250=((new_r00)*(x241));
IkReal x251=((IkReal(1.00000000000000))*(x238));
IkReal x252=((new_r10)*(x241));
IkReal x253=((IkReal(0.707106781186548))*(x239));
IkReal x254=((new_r10)*(x242));
IkReal x255=((x252)+(x253));
IkReal x256=((x247)+(x248));
IkReal x257=((x245)+(x244));
evalcond[0]=((((new_r10)*(sj4)))+(x257)+(x239));
evalcond[1]=((((new_r11)*(sj4)))+(x240)+(x244)+(((IkReal(-1.00000000000000))*(x245))));
evalcond[2]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x243)))+(x244)+(((IkReal(-1.00000000000000))*(x245))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x243)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x257))));
evalcond[4]=((x254)+(((IkReal(-1.00000000000000))*(x237)))+(((IkReal(-1.00000000000000))*(x250)))+(((IkReal(-1.00000000000000))*(x255))));
evalcond[5]=((x249)+(((IkReal(-1.00000000000000))*(x246)))+(((IkReal(-1.00000000000000))*(x251)))+(((IkReal(-1.00000000000000))*(x256))));
evalcond[6]=((x237)+(x246)+(((IkReal(-1.00000000000000))*(x249)))+(((IkReal(-1.00000000000000))*(x256))));
evalcond[7]=((x250)+(((IkReal(-1.00000000000000))*(x251)))+(((IkReal(-1.00000000000000))*(x255)))+(((IkReal(-1.00000000000000))*(x254))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst6)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst6)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst6)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst6)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[10];
IkReal x258=IKsin(j6);
IkReal x259=IKcos(j6);
IkReal x260=((IkReal(1.00000000000000))*(sj5));
IkReal x261=((IkReal(0.707106781186548))*(cj4));
IkReal x262=((IkReal(1.00000000000000))*(sj4));
IkReal x263=((cj5)*(new_r01));
IkReal x264=((IkReal(0.707106781186548))*(cj5));
IkReal x265=((new_r11)*(sj4));
IkReal x266=((IkReal(0.707106781186548))*(sj4));
IkReal x267=((new_r10)*(sj4));
IkReal x268=((IkReal(0.707106781186548))*(x258));
IkReal x269=((IkReal(0.707106781186548))*(x259));
IkReal x270=((IkReal(1.00000000000000))*(x259));
IkReal x271=((x259)*(x264));
IkReal x272=((x258)*(x264));
evalcond[0]=((((sj5)*(x259)))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(x258)*(x260)))+(new_r21));
evalcond[2]=((((cj4)*(new_r00)))+(x267)+(x268)+(((IkReal(-1.00000000000000))*(x271))));
evalcond[3]=((((cj4)*(new_r01)))+(x265)+(x269)+(x272));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r00)*(x262)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x271)))+(((IkReal(-1.00000000000000))*(x268))));
evalcond[5]=((((IkReal(-1.00000000000000))*(new_r01)*(x262)))+(((cj4)*(new_r11)))+(x272)+(((IkReal(-1.00000000000000))*(x269))));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r00)*(x266)))+(((IkReal(-1.00000000000000))*(new_r00)*(x261)))+(((IkReal(-1.00000000000000))*(new_r10)*(x266)))+(((new_r10)*(x261)))+(((IkReal(-1.00000000000000))*(x258))));
evalcond[7]=((((IkReal(-0.707106781186548))*(x265)))+(((IkReal(-1.00000000000000))*(new_r01)*(x266)))+(((IkReal(-1.00000000000000))*(new_r01)*(x261)))+(((IkReal(-1.00000000000000))*(x270)))+(((new_r11)*(x261))));
evalcond[8]=((((IkReal(-1.00000000000000))*(x263)*(x266)))+(((cj5)*(new_r11)*(x261)))+(((x264)*(x265)))+(((IkReal(-1.00000000000000))*(new_r21)*(x260)))+(x258)+(((x261)*(x263))));
evalcond[9]=((((cj5)*(new_r10)*(x261)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)*(x264)))+(((x264)*(x267)))+(((IkReal(-1.00000000000000))*(new_r20)*(x260)))+(((cj5)*(new_r00)*(x261)))+(((IkReal(-1.00000000000000))*(x270))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst5)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst5)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst5)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst5)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[2];
evalcond[0]=((new_r20)+(((sj5)*(IKcos(j6)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(IKsin(j6))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign(((((IkReal(2.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(2.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x273=((IkReal(1.41421356237310))*(sj5));
IkReal x274=((new_r12)*(x273));
IkReal x275=((new_r02)*(x273));
if( IKabs(((gconst7)*(((x274)+(((IkReal(-1.00000000000000))*(x275))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst7)*(((x274)+(x275))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst7)*(((x274)+(((IkReal(-1.00000000000000))*(x275)))))), ((gconst7)*(((x274)+(x275)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[15];
IkReal x276=IKcos(j4);
IkReal x277=((IkReal(0.707106781186548))*(sj5));
IkReal x278=IKsin(j4);
IkReal x279=((IkReal(0.707106781186548))*(sj6));
IkReal x280=((IkReal(0.707106781186548))*(cj6));
IkReal x281=((IkReal(1.00000000000000))*(cj6));
IkReal x282=((IkReal(0.707106781186548))*(cj5));
IkReal x283=((IkReal(1.00000000000000))*(cj5));
IkReal x284=((IkReal(1.00000000000000))*(sj5));
IkReal x285=((cj5)*(x280));
IkReal x286=((cj5)*(x279));
IkReal x287=((new_r10)*(x276));
IkReal x288=((new_r12)*(x276));
IkReal x289=((new_r11)*(x278));
IkReal x290=((new_r02)*(x278));
IkReal x291=((new_r00)*(x276));
IkReal x292=((IkReal(0.707106781186548))*(x278));
IkReal x293=((new_r02)*(x276));
IkReal x294=((new_r11)*(x276));
IkReal x295=((new_r00)*(x278));
IkReal x296=((new_r12)*(x278));
IkReal x297=((new_r01)*(x278));
IkReal x298=((new_r10)*(x278));
IkReal x299=((new_r01)*(x276));
evalcond[0]=((x293)+(x296)+(((IkReal(-1.00000000000000))*(x277))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x290)))+(x288)+(((IkReal(-1.00000000000000))*(x277))));
evalcond[2]=((x298)+(x291)+(x279)+(((IkReal(-1.00000000000000))*(x285))));
evalcond[3]=((x299)+(x289)+(x286)+(x280));
evalcond[4]=((((IkReal(-1.00000000000000))*(x295)))+(x287)+(((IkReal(-1.00000000000000))*(x279)))+(((IkReal(-1.00000000000000))*(x285))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x297)))+(x294)+(x286)+(((IkReal(-1.00000000000000))*(x280))));
evalcond[6]=((((IkReal(-0.707106781186548))*(x290)))+(((IkReal(-0.707106781186548))*(x293)))+(((IkReal(-1.00000000000000))*(new_r12)*(x292)))+(((IkReal(0.707106781186548))*(x288))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r10)*(x292)))+(((IkReal(-1.00000000000000))*(sj6)))+(((IkReal(-0.707106781186548))*(x291)))+(((IkReal(0.707106781186548))*(x287)))+(((IkReal(-1.00000000000000))*(new_r00)*(x292))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r01)*(x292)))+(((IkReal(-0.707106781186548))*(x299)))+(((IkReal(-1.00000000000000))*(x281)))+(((IkReal(-0.707106781186548))*(x289)))+(((IkReal(0.707106781186548))*(x294))));
evalcond[9]=((((IkReal(-1.00000000000000))*(new_r22)*(x284)))+(((x282)*(x293)))+(((x282)*(x296)))+(((x282)*(x288)))+(((IkReal(-1.00000000000000))*(x282)*(x290))));
evalcond[10]=((((IkReal(-1.00000000000000))*(x277)*(x287)))+(((x277)*(x295)))+(((IkReal(-1.00000000000000))*(x277)*(x298)))+(((IkReal(-1.00000000000000))*(x277)*(x291)))+(((IkReal(-1.00000000000000))*(new_r20)*(x283))));
evalcond[11]=((((IkReal(-1.00000000000000))*(x277)*(x289)))+(((x277)*(x297)))+(((IkReal(-1.00000000000000))*(x277)*(x299)))+(((IkReal(-1.00000000000000))*(x277)*(x294)))+(((IkReal(-1.00000000000000))*(new_r21)*(x283))));
evalcond[12]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x277)*(x288)))+(((IkReal(-1.00000000000000))*(new_r22)*(x283)))+(((x277)*(x290)))+(((IkReal(-1.00000000000000))*(x277)*(x293)))+(((IkReal(-1.00000000000000))*(x277)*(x296))));
evalcond[13]=((sj6)+(((x282)*(x294)))+(((x282)*(x299)))+(((IkReal(-1.00000000000000))*(new_r21)*(x284)))+(((x282)*(x289)))+(((IkReal(-1.00000000000000))*(x282)*(x297))));
evalcond[14]=((((x282)*(x291)))+(((x282)*(x298)))+(((IkReal(-1.00000000000000))*(x281)))+(((IkReal(-1.00000000000000))*(new_r20)*(x284)))+(((x282)*(x287)))+(((IkReal(-1.00000000000000))*(x282)*(x295))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  || IKabs(evalcond[12]) > 0.000001  || IKabs(evalcond[13]) > 0.000001  || IKabs(evalcond[14]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ba32f85ca3c87502739320e9e26a5905"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
