/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:50:34.261953
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[2]);
x3=IKsin(j[3]);
x4=IKcos(j[3]);
x5=IKsin(j[1]);
x6=IKsin(j[5]);
x7=IKsin(j[4]);
x8=IKcos(j[4]);
x9=IKcos(j[5]);
x10=((IkReal(0.707106781186548))*(IKcos(j[0])));
x11=((IkReal(0.707106781186548))*(IKsin(j[0])));
x12=((IkReal(0.707106781186548))*(x6));
x13=((IkReal(0.707106781186548))*(x9));
x14=((IkReal(0.500001138018525))*(x9));
x15=((IkReal(0.499998861978885))*(x9));
x16=((IkReal(0.499998861978885))*(x6));
x17=((IkReal(1.00000000000000))*(x1));
x18=((IkReal(0.500001138018525))*(x6));
x19=((IkReal(1.00000000000000))*(x3));
x20=((x0)*(x1));
x21=((x0)*(x2));
x22=((x4)*(x5));
x23=((IkReal(-1.00000000000000))*(x2));
x24=((x3)*(x5));
x25=((IkReal(-1.00000000000000))*(x3));
x26=((((IkReal(-1.00000000000000))*(x10)))+(x11));
x27=((((IkReal(-1.00000000000000))*(x11)))+(x10));
x28=((((IkReal(-1.00000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(x10))));
x29=((x11)+(x10));
x30=((x26)*(x5));
x31=((IkReal(-1.00000000000000))*(x27));
x32=((x2)*(x27));
x33=((((IkReal(1.00000000000000))*(x0)*(x4)))+(((IkReal(-1.00000000000000))*(x17)*(x24))));
x34=((x33)*(x9));
x35=((((x8)*(((((IkReal(-1.00000000000000))*(x0)*(x19)))+(((IkReal(-1.00000000000000))*(x17)*(x22)))))))+(((x2)*(x5)*(x7))));
x36=((x35)*(x6));
x37=((((IkReal(-1.00000000000000))*(x22)*(x28)))+(((x19)*(((((x0)*(x17)*(x29)))+(((IkReal(1.00000000000000))*(x32))))))));
x38=((((x25)*(((((x20)*(x31)))+(((x23)*(x28)))))))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x39=((x37)*(x9));
x40=((((x8)*(((((x4)*(((((x2)*(x28)))+(((x20)*(x27)))))))+(((x24)*(x26)))))))+(((x7)*(((((x21)*(x26)))+(((x1)*(x28))))))));
x41=((((x8)*(((((x4)*(((x32)+(((x20)*(x29)))))))+(((x24)*(x28)))))))+(((x7)*(((((x21)*(x28)))+(((x1)*(x27))))))));
eerot[0]=((((((((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))+(((IkReal(0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))))*(IKcos(j[6])))));
eerot[1]=((((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707108390587779))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6])))));
eerot[2]=((((IkReal(-0.707108390587779))*(x36)))+(((IkReal(-0.707108390587779))*(x34)))+(((x16)*(x40)))+(((x15)*(x38)))+(((IkReal(-1.00000000000000))*(x16)*(x41)))+(((IkReal(-1.00000000000000))*(x15)*(x37))));
eetrans[0]=((IkReal(-0.350177000000000))+(((IkReal(0.212132517176334))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKcos(j[0]))))+(((IkReal(-0.106065775767248))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318198775764501))*(IKcos(j[2]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKsin(j[0]))))+(((IkReal(0.106065775767248))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(-0.388909614823279))*(IKcos(j[1]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((IkReal(-0.212132517176334))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(-0.125829938105095))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829938105095))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.125829938105095))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707108390587779))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1])))));
eerot[3]=((((((((IkReal(0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6])))));
eerot[4]=((((((((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6])))));
eerot[5]=((((IkReal(-0.707105171781653))*(x34)))+(((IkReal(-0.707105171781653))*(x36)))+(((IkReal(-1.00000000000000))*(x18)*(x40)))+(((x18)*(x41)))+(((IkReal(-1.00000000000000))*(x14)*(x38)))+(((x14)*(((((x25)*(((((x23)*(x27)))+(((IkReal(-1.00000000000000))*(x20)*(x29)))))))+(((IkReal(-1.00000000000000))*(x22)*(x28))))))));
eetrans[1]=((IkReal(-0.350177000000000))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318197327301744))*(IKcos(j[2]))*(IKsin(j[1]))))+(((IkReal(0.212131551534496))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829365318545))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.212131551534496))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKcos(j[0]))))+(((IkReal(-0.0159099387882250))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.388907844479909))*(IKcos(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(-0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3])))));
eerot[6]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6])))));
eerot[7]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6])))));
eerot[8]=((((x12)*(x40)))+(((x12)*(x41)))+(((x13)*(x37)))+(((x13)*(x38))));
eetrans[2]=((IkReal(0.378361000000000))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.275000000000000))*(IKcos(j[0]))))+(((IkReal(0.275000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.150000000000000))*(IKcos(j[0]))))+(((IkReal(0.150000000000000))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.275000000000000))*(IKsin(j[0]))))+(((IkReal(0.275000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.150000000000000))*(IKsin(j[0]))))+(((IkReal(0.150000000000000))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1])))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j4,cj4,sj4,htj4,j6,cj6,sj6,htj6,j7,cj7,sj7,htj7,j8,cj8,sj8,htj8,j9,cj9,sj9,htj9,j10,cj10,sj10,htj10,j5,cj5,sj5,htj5,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij4[2], _nj4,_ij6[2], _nj6,_ij7[2], _nj7,_ij8[2], _nj8,_ij9[2], _nj9,_ij10[2], _nj10,_ij5[2], _nj5;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1; j7=numeric_limits<IkReal>::quiet_NaN(); _ij7[0] = -1; _ij7[1] = -1; _nj7 = -1; j8=numeric_limits<IkReal>::quiet_NaN(); _ij8[0] = -1; _ij8[1] = -1; _nj8 = -1; j9=numeric_limits<IkReal>::quiet_NaN(); _ij9[0] = -1; _ij9[1] = -1; _nj9 = -1; j10=numeric_limits<IkReal>::quiet_NaN(); _ij10[0] = -1; _ij10[1] = -1; _nj10 = -1;  _ij5[0] = -1; _ij5[1] = -1; _nj5 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j5=pfree[0]; cj5=cos(pfree[0]); sj5=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=((((IkReal(-0.500001138018525))*(r11)))+(((IkReal(0.499998861978885))*(r01)))+(((IkReal(0.707106781186548))*(r21))));
new_r01=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(0.500001138018525))*(r10)))+(((IkReal(-0.499998861978885))*(r00))));
new_r02=((((IkReal(-0.500001138018525))*(r12)))+(((IkReal(0.499998861978885))*(r02)))+(((IkReal(0.707106781186548))*(r22))));
new_px=((IkReal(-0.267542425853256))+(((IkReal(0.499998861978885))*(px)))+(((IkReal(-0.500001138018525))*(py)))+(((IkReal(-0.0889747974891426))*(r02)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.0889752025103964))*(r12))));
new_r10=((((IkReal(0.500001138018525))*(r11)))+(((IkReal(0.707106781186548))*(r21)))+(((IkReal(-0.499998861978885))*(r01))));
new_r11=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(-0.500001138018525))*(r10)))+(((IkReal(0.499998861978885))*(r00))));
new_r12=((((IkReal(0.500001138018525))*(r12)))+(((IkReal(0.707106781186548))*(r22)))+(((IkReal(-0.499998861978885))*(r02))));
new_py=((IkReal(-0.267540831819791))+(((IkReal(-0.499998861978885))*(px)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.500001138018525))*(py)))+(((IkReal(0.0889747974891426))*(r02)))+(((IkReal(-0.0889752025103964))*(r12))));
new_r20=((((IkReal(-0.707108390587779))*(r01)))+(((IkReal(-0.707105171781653))*(r11))));
new_r21=((((IkReal(0.707108390587779))*(r00)))+(((IkReal(0.707105171781653))*(r10))));
new_r22=((((IkReal(-0.707108390587779))*(r02)))+(((IkReal(-0.707105171781653))*(r12))));
new_pz=((IkReal(-0.495225062629841))+(((IkReal(0.125829365318545))*(r12)))+(((IkReal(-0.707105171781653))*(py)))+(((IkReal(0.125829938105095))*(r02)))+(((IkReal(-0.707108390587779))*(px))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j7array[2], cj7array[2], sj7array[2];
bool j7valid[2]={false};
_nj7 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x42=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j7array[0]=((IkReal(-1.34027003705633))+(((IkReal(-1.00000000000000))*(x42))));
sj7array[0]=IKsin(j7array[0]);
cj7array[0]=IKcos(j7array[0]);
j7array[1]=((IkReal(1.80132261653346))+(x42));
sj7array[1]=IKsin(j7array[1]);
cj7array[1]=IKcos(j7array[1]);
if( j7array[0] > IKPI )
{
    j7array[0]-=IK2PI;
}
else if( j7array[0] < -IKPI )
{    j7array[0]+=IK2PI;
}
j7valid[0] = true;
if( j7array[1] > IKPI )
{
    j7array[1]-=IK2PI;
}
else if( j7array[1] < -IKPI )
{    j7array[1]+=IK2PI;
}
j7valid[1] = true;
for(int ij7 = 0; ij7 < 2; ++ij7)
{
if( !j7valid[ij7] )
{
    continue;
}
_ij7[0] = ij7; _ij7[1] = -1;
for(int iij7 = ij7+1; iij7 < 2; ++iij7)
{
if( j7valid[iij7] && IKabs(cj7array[ij7]-cj7array[iij7]) < IKFAST_SOLUTION_THRESH && IKabs(sj7array[ij7]-sj7array[iij7]) < IKFAST_SOLUTION_THRESH )
{
    j7valid[iij7]=false; _ij7[1] = iij7; break; 
}
}
j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];

{
IkReal dummyeval[1];
IkReal x43=(sj5)*(sj5);
dummyeval[0]=((((x43)*((py)*(py))))+(((x43)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x44=(sj5)*(sj5);
dummyeval[0]=((((x44)*((py)*(py))))+(((x44)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x45=(sj5)*(sj5);
IkReal x46=((IkReal(2.00000000000000))*(x45));
dummyeval[0]=((((x45)*((px)*(px)*(px)*(px))))+(((x46)*((px)*(px))*((py)*(py))))+(((px)*(x46)*((py)*(py)*(py))))+(((py)*(x46)*((px)*(px)*(px))))+(((x45)*((py)*(py)*(py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj7))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst4;
gconst4=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst5;
gconst5=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst6;
gconst6=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst7;
gconst7=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal gconst8;
gconst8=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst9;
gconst9=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst10;
gconst10=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst11;
gconst11=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst12;
gconst12=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal gconst13;
gconst13=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst14;
gconst14=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst15;
gconst15=((((IkReal(-1.41421356237310))*(py)))+(((IkReal(1.41421356237310))*(px))));
IkReal gconst16;
gconst16=((((IkReal(-1.41421356237310))*(py)))+(((IkReal(1.41421356237310))*(px))));
IkReal gconst17;
gconst17=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst18;
gconst18=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst19;
gconst19=((((IkReal(-1.41421356237310))*(py)))+(((IkReal(1.41421356237310))*(px))));
IkReal gconst20;
gconst20=((((IkReal(-1.41421356237310))*(py)))+(((IkReal(1.41421356237310))*(px))));
IkReal gconst21;
gconst21=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst22;
gconst22=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst23;
gconst23=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst24;
gconst24=((IkReal(0.0450000000000000))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7))));
IkReal gconst25;
gconst25=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal gconst26;
gconst26=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst27;
gconst27=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst28;
gconst28=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst29;
gconst29=((IkReal(0.0450000000000000))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7))));
IkReal gconst30;
gconst30=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal x47=((IkReal(1.00000000000000))*(gconst29));
IkReal x48=((gconst23)*(gconst24));
IkReal x49=((gconst26)*(gconst30));
IkReal x50=((gconst21)*(gconst25));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x48)))+(((gconst28)*(gconst29)*(x50)))+(((IkReal(-1.00000000000000))*(x49)*(x50)))+(((x48)*(x49)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst25)*(gconst27)*(x47))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x51=((IkReal(1.00000000000000))*(gconst13));
IkReal x52=((gconst12)*(gconst8));
IkReal x53=((gconst3)*(gconst7));
IkReal x54=((gconst11)*(gconst18));
IkReal x55=((gconst28)*(gconst29));
IkReal x56=((IkReal(1.00000000000000))*(gconst23));
IkReal x57=((gconst14)*(gconst15));
IkReal x58=((gconst20)*(gconst24));
IkReal x59=((gconst14)*(gconst26));
IkReal x60=((gconst25)*(gconst3));
IkReal x61=((IkReal(1.00000000000000))*(gconst17));
IkReal x62=((gconst25)*(gconst30));
IkReal x63=((gconst16)*(gconst3));
IkReal x64=((gconst17)*(gconst23));
IkReal x65=((gconst21)*(gconst29));
IkReal x66=((IkReal(1.00000000000000))*(gconst26));
IkReal x67=((gconst5)*(gconst6));
IkReal x68=((gconst21)*(gconst7));
IkReal x69=((gconst26)*(gconst30));
IkReal x70=((gconst23)*(gconst24));
IkReal x71=((gconst19)*(gconst28));
IkReal x72=((gconst23)*(gconst26));
IkReal x73=((gconst15)*(gconst20));
IkReal x74=((gconst13)*(gconst16));
IkReal x75=((gconst10)*(gconst11));
IkReal x76=((gconst16)*(gconst21));
IkReal x77=((gconst10)*(gconst19));
IkReal x78=((gconst13)*(gconst7));
IkReal x79=((gconst10)*(gconst29));
IkReal x80=((gconst23)*(gconst6));
IkReal x81=((gconst11)*(gconst28));
IkReal x82=((gconst21)*(gconst25));
IkReal x83=((gconst24)*(gconst30));
IkReal x84=((gconst14)*(gconst17));
IkReal x85=((gconst24)*(gconst5));
IkReal x86=((gconst18)*(gconst29));
IkReal x87=((gconst14)*(gconst6));
IkReal x88=((gconst13)*(gconst25));
IkReal x89=((gconst12)*(gconst26));
IkReal x90=((gconst30)*(gconst8));
IkReal x91=((gconst25)*(gconst27));
IkReal x92=((IkReal(1.00000000000000))*(gconst22));
IkReal x93=((gconst25)*(gconst9));
IkReal x94=((gconst18)*(gconst19));
IkReal x95=((gconst16)*(gconst19));
IkReal x96=((gconst20)*(gconst8));
IkReal x97=((gconst15)*(gconst5));
IkReal x98=((IkReal(1.00000000000000))*(gconst4));
IkReal x99=((IkReal(1.00000000000000))*(x75));
IkReal x100=((gconst12)*(gconst17)*(gconst5));
IkReal x101=((gconst20)*(x82));
IkReal x102=((IkReal(1.00000000000000))*(x96));
IkReal x103=((gconst11)*(x92));
IkReal x104=((IkReal(1.00000000000000))*(x97));
IkReal x105=((gconst9)*(x98));
IkReal x106=((IkReal(1.00000000000000))*(gconst14)*(gconst24));
IkReal x107=((gconst27)*(gconst29)*(x92));
IkReal x108=((gconst7)*(x105));
op[0]=((((x55)*(x82)))+(((IkReal(-1.00000000000000))*(gconst21)*(x62)*(x66)))+(((x69)*(x70)))+(((IkReal(-1.00000000000000))*(gconst29)*(x91)*(x92)))+(((IkReal(-1.00000000000000))*(gconst24)*(x55)*(x56))));
op[1]=((((IkReal(-1.00000000000000))*(gconst15)*(x55)*(x56)))+(((x55)*(x88)))+(((x64)*(x83)))+(((x58)*(x72)))+(((IkReal(-1.00000000000000))*(gconst30)*(x66)*(x76)))+(((IkReal(-1.00000000000000))*(gconst19)*(x91)*(x92)))+(((IkReal(-1.00000000000000))*(gconst21)*(x61)*(x62)))+(((IkReal(-1.00000000000000))*(gconst16)*(x107)))+(((IkReal(-1.00000000000000))*(gconst24)*(x56)*(x86)))+(((IkReal(-1.00000000000000))*(gconst24)*(x56)*(x71)))+(((IkReal(-1.00000000000000))*(x106)*(x55)))+(((x55)*(x76)))+(((gconst18)*(gconst25)*(x65)))+(((IkReal(-1.00000000000000))*(x101)*(x66)))+(((gconst15)*(gconst23)*(x69)))+(((IkReal(-1.00000000000000))*(gconst26)*(x51)*(x62)))+(((x59)*(x83)))+(((x71)*(x82))));
op[2]=((((IkReal(-1.00000000000000))*(gconst29)*(x92)*(x93)))+(((IkReal(-1.00000000000000))*(gconst15)*(x56)*(x86)))+(((IkReal(-1.00000000000000))*(x103)*(x91)))+(((gconst16)*(gconst18)*(x65)))+(((x82)*(x94)))+(((IkReal(-1.00000000000000))*(gconst15)*(x56)*(x71)))+(((x58)*(x59)))+(((IkReal(-1.00000000000000))*(gconst7)*(x107)))+(((x81)*(x82)))+(((x58)*(x64)))+(((IkReal(-1.00000000000000))*(gconst17)*(x51)*(x62)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x62)))+(((gconst15)*(gconst30)*(x64)))+(((x70)*(x90)))+(((IkReal(-1.00000000000000))*(gconst27)*(x92)*(x95)))+(((x86)*(x88)))+(((x70)*(x89)))+(((IkReal(-1.00000000000000))*(gconst6)*(x55)*(x56)))+(((IkReal(-1.00000000000000))*(x55)*(x85)))+(((IkReal(-1.00000000000000))*(gconst30)*(x61)*(x76)))+(((IkReal(-1.00000000000000))*(gconst16)*(x51)*(x69)))+(((IkReal(-1.00000000000000))*(gconst24)*(x56)*(x94)))+(((gconst10)*(gconst25)*(x65)))+(((IkReal(-1.00000000000000))*(gconst12)*(x66)*(x82)))+(((IkReal(-1.00000000000000))*(x106)*(x71)))+(((IkReal(-1.00000000000000))*(gconst24)*(x56)*(x81)))+(((x57)*(x69)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst25)*(gconst26)*(x51)))+(((x69)*(x85)))+(((x69)*(x80)))+(((IkReal(-1.00000000000000))*(gconst24)*(x56)*(x79)))+(((IkReal(-1.00000000000000))*(gconst30)*(x60)*(x66)))+(((IkReal(-1.00000000000000))*(x55)*(x57)))+(((IkReal(-1.00000000000000))*(gconst29)*(x91)*(x98)))+(((x83)*(x84)))+(((x55)*(x74)))+(((x72)*(x73)))+(((IkReal(-1.00000000000000))*(gconst20)*(x66)*(x76)))+(((IkReal(-1.00000000000000))*(x101)*(x61)))+(((IkReal(-1.00000000000000))*(gconst30)*(x66)*(x68)))+(((IkReal(-1.00000000000000))*(x106)*(x86)))+(((x71)*(x76)))+(((x55)*(x60)))+(((x55)*(x68)))+(((x71)*(x88))));
op[3]=((((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst22)*(gconst25)*(gconst9)))+(((gconst19)*(gconst25)*(gconst28)*(gconst3)))+(((gconst17)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst28)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst25)*(gconst30)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst24)*(gconst28)*(gconst5)))+(((gconst13)*(gconst16)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst25)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst22)*(gconst27)))+(((gconst14)*(gconst26)*(gconst30)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst30)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst15)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst21)*(gconst30)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst13)*(gconst25)*(gconst26)))+(((gconst14)*(gconst17)*(gconst20)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst18)*(gconst19)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst3)*(gconst30)))+(((gconst12)*(gconst15)*(gconst23)*(gconst26)))+(((gconst13)*(gconst16)*(gconst19)*(gconst28)))+(((gconst20)*(gconst24)*(gconst26)*(gconst5)))+(((gconst13)*(gconst18)*(gconst19)*(gconst25)))+(((gconst10)*(gconst19)*(gconst21)*(gconst25)))+(((gconst11)*(gconst16)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst20)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst19)*(gconst24)))+(((gconst12)*(gconst14)*(gconst24)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst23)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst21)*(gconst25)))+(((gconst20)*(gconst23)*(gconst26)*(gconst6)))+(((gconst18)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst23)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst7)))+(((gconst20)*(gconst23)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst25)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst19)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst22)*(gconst27)*(gconst7)))+(((gconst13)*(gconst28)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst14)*(gconst24)*(gconst28)))+(((gconst14)*(gconst15)*(gconst17)*(gconst30)))+(((gconst18)*(gconst25)*(gconst29)*(gconst3)))+(((gconst10)*(gconst13)*(gconst25)*(gconst29)))+(((gconst14)*(gconst24)*(gconst30)*(gconst8)))+(((gconst12)*(gconst17)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst17)*(gconst30)))+(((gconst11)*(gconst18)*(gconst21)*(gconst25)))+(((gconst15)*(gconst23)*(gconst30)*(gconst8)))+(((gconst16)*(gconst28)*(gconst29)*(gconst3)))+(((gconst15)*(gconst17)*(gconst20)*(gconst23)))+(((gconst11)*(gconst13)*(gconst25)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst26)*(gconst30)*(gconst7)))+(((gconst19)*(gconst21)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst3)*(gconst30)))+(((gconst17)*(gconst23)*(gconst30)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst29)*(gconst9)))+(((gconst10)*(gconst16)*(gconst21)*(gconst29)))+(((gconst15)*(gconst26)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst20)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst27)*(gconst29)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst19)*(gconst23)*(gconst24)))+(((gconst14)*(gconst15)*(gconst20)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst29)*(gconst5)))+(((gconst16)*(gconst18)*(gconst19)*(gconst21))));
op[4]=((((IkReal(-1.00000000000000))*(gconst11)*(gconst14)*(gconst15)*(gconst28)))+(((gconst11)*(gconst21)*(gconst28)*(gconst7)))+(((gconst15)*(gconst20)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst13)*(gconst17)*(gconst25)))+(((gconst10)*(gconst13)*(gconst19)*(gconst25)))+(((gconst11)*(gconst13)*(gconst16)*(gconst28)))+(((gconst10)*(gconst11)*(gconst21)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst22)*(gconst27)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst28)*(gconst5)))+(((gconst14)*(gconst17)*(gconst30)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst13)*(gconst16)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst19)*(gconst22)*(gconst9)))+(((gconst11)*(gconst13)*(gconst18)*(gconst25)))+(((gconst11)*(gconst25)*(gconst28)*(gconst3)))+(((gconst11)*(gconst16)*(gconst18)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst21)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst29)*(gconst6)))+(((gconst12)*(gconst24)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst23)*(gconst29)*(gconst6)))+(((gconst14)*(gconst20)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst19)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst30)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst19)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst25)*(gconst26)*(gconst3)))+(((gconst13)*(gconst19)*(gconst28)*(gconst7)))+(((gconst13)*(gconst16)*(gconst18)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst15)*(gconst29)))+(((gconst23)*(gconst30)*(gconst6)*(gconst8)))+(((gconst14)*(gconst20)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst19)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst17)*(gconst21)))+(((gconst12)*(gconst14)*(gconst17)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst22)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst18)*(gconst23)))+(((gconst16)*(gconst18)*(gconst29)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst28)*(gconst29)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst21)*(gconst26)*(gconst7)))+(((gconst13)*(gconst18)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst30)*(gconst7)))+(((gconst16)*(gconst19)*(gconst28)*(gconst3)))+(((gconst12)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst21)*(gconst25)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst30)*(gconst7)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst20)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst17)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst19)*(gconst28)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst25)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst17)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst26)*(gconst3)))+(((gconst24)*(gconst30)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst18)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst15)*(gconst19)*(gconst23)))+(((gconst15)*(gconst17)*(gconst30)*(gconst5)))+(((gconst28)*(gconst29)*(gconst3)*(gconst7)))+(((gconst12)*(gconst23)*(gconst24)*(gconst8)))+(((gconst14)*(gconst15)*(gconst30)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst18)*(gconst29)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst3)))+(((gconst10)*(gconst16)*(gconst19)*(gconst21)))+(((gconst17)*(gconst20)*(gconst23)*(gconst6)))+(((gconst15)*(gconst20)*(gconst23)*(gconst8)))+(((gconst14)*(gconst15)*(gconst17)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst23)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst19)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst20)*(gconst25)*(gconst8)))+(((gconst26)*(gconst30)*(gconst5)*(gconst6)))+(((gconst12)*(gconst14)*(gconst15)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst20)*(gconst25)*(gconst3)))+(((gconst17)*(gconst20)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst29)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst19)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst3)*(gconst30)*(gconst8)))+(((gconst12)*(gconst15)*(gconst17)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst20)*(gconst26)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst14)*(gconst18)*(gconst24)))+(((gconst18)*(gconst19)*(gconst21)*(gconst7)))+(((gconst10)*(gconst21)*(gconst29)*(gconst7)))+(((gconst10)*(gconst13)*(gconst16)*(gconst29)))+(((gconst18)*(gconst19)*(gconst25)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst11)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst30)*(gconst7))));
op[5]=((((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst25)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst19)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst11)*(gconst15)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst13)*(gconst16)*(gconst17)))+(((gconst13)*(gconst18)*(gconst19)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst14)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst29)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst15)*(gconst19)))+(((gconst12)*(gconst17)*(gconst24)*(gconst5)))+(((gconst19)*(gconst28)*(gconst3)*(gconst7)))+(((gconst12)*(gconst15)*(gconst23)*(gconst8)))+(((gconst11)*(gconst18)*(gconst21)*(gconst7)))+(((gconst11)*(gconst16)*(gconst28)*(gconst3)))+(((gconst10)*(gconst13)*(gconst29)*(gconst7)))+(((gconst14)*(gconst30)*(gconst6)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst20)*(gconst8)))+(((gconst10)*(gconst19)*(gconst21)*(gconst7)))+(((gconst12)*(gconst14)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst15)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst25)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst18)*(gconst19)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst29)*(gconst4)*(gconst9)))+(((gconst20)*(gconst26)*(gconst5)*(gconst6)))+(((gconst15)*(gconst17)*(gconst20)*(gconst5)))+(((gconst11)*(gconst13)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst11)*(gconst14)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst20)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst3)*(gconst30)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst28)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst7)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst21)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst27)*(gconst4)*(gconst7)))+(((gconst12)*(gconst14)*(gconst26)*(gconst6)))+(((gconst15)*(gconst30)*(gconst5)*(gconst8)))+(((gconst10)*(gconst16)*(gconst29)*(gconst3)))+(((gconst11)*(gconst18)*(gconst25)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst19)*(gconst24)*(gconst5)))+(((gconst10)*(gconst19)*(gconst25)*(gconst3)))+(((gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst18)*(gconst29)*(gconst3)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst17)*(gconst20)*(gconst3)))+(((gconst20)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst22)*(gconst9)))+(((gconst10)*(gconst11)*(gconst16)*(gconst21)))+(((gconst14)*(gconst15)*(gconst20)*(gconst8)))+(((gconst20)*(gconst23)*(gconst6)*(gconst8)))+(((gconst16)*(gconst18)*(gconst19)*(gconst3)))+(((gconst12)*(gconst14)*(gconst15)*(gconst17)))+(((gconst17)*(gconst30)*(gconst5)*(gconst6)))+(((gconst10)*(gconst11)*(gconst13)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst30)*(gconst7)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst3)*(gconst30)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst22)*(gconst7)*(gconst9)))+(((gconst12)*(gconst15)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst13)*(gconst26)*(gconst7)))+(((gconst14)*(gconst17)*(gconst20)*(gconst6)))+(((gconst10)*(gconst13)*(gconst16)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst28)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst19)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst13)*(gconst25)*(gconst8)))+(((gconst11)*(gconst13)*(gconst16)*(gconst18))));
op[6]=((((IkReal(-1.00000000000000))*(x104)*(x54)))+(((gconst17)*(gconst20)*(x67)))+(((IkReal(-1.00000000000000))*(gconst7)*(x51)*(x96)))+(((x87)*(x96)))+(((IkReal(-1.00000000000000))*(x67)*(x79)))+(((gconst12)*(gconst6)*(x84)))+(((IkReal(-1.00000000000000))*(gconst20)*(x53)*(x61)))+(((x53)*(x94)))+(((x53)*(x81)))+(((IkReal(-1.00000000000000))*(gconst12)*(x53)*(x66)))+(((IkReal(-1.00000000000000))*(gconst29)*(x108)))+(((IkReal(-1.00000000000000))*(x52)*(x60)))+(((IkReal(-1.00000000000000))*(x52)*(x68)))+(((IkReal(-1.00000000000000))*(gconst16)*(x51)*(x52)))+(((IkReal(-1.00000000000000))*(x53)*(x90)))+(((x68)*(x75)))+(((x67)*(x89)))+(((IkReal(-1.00000000000000))*(gconst6)*(x56)*(x75)))+(((IkReal(-1.00000000000000))*(x105)*(x95)))+(((IkReal(-1.00000000000000))*(x57)*(x99)))+(((IkReal(-1.00000000000000))*(x54)*(x87)))+(((gconst5)*(gconst8)*(x73)))+(((x67)*(x90)))+(((x60)*(x75)))+(((IkReal(-1.00000000000000))*(x77)*(x87)))+(((x54)*(x78)))+(((IkReal(-1.00000000000000))*(gconst7)*(gconst9)*(x103)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst7)*(x51)))+(((x52)*(x57)))+(((gconst12)*(gconst17)*(x97)))+(((x63)*(x77)))+(((x52)*(x80)))+(((x52)*(x85)))+(((x53)*(x79)))+(((IkReal(-1.00000000000000))*(gconst12)*(x61)*(x63)))+(((IkReal(-1.00000000000000))*(x67)*(x81)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst27)*(gconst7)*(x98)))+(((IkReal(-1.00000000000000))*(x102)*(x63)))+(((IkReal(-1.00000000000000))*(x67)*(x94)))+(((IkReal(-1.00000000000000))*(x104)*(x77)))+(((x74)*(x75)))+(((x77)*(x78)))+(((IkReal(-1.00000000000000))*(gconst11)*(x93)*(x98)))+(((x54)*(x63)))+(((IkReal(-1.00000000000000))*(x85)*(x99))));
op[7]=((((IkReal(-1.00000000000000))*(x67)*(x77)))+(((IkReal(-1.00000000000000))*(gconst7)*(x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst12)*(x53)*(x61)))+(((IkReal(-1.00000000000000))*(x52)*(x63)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(x105)))+(((IkReal(-1.00000000000000))*(x87)*(x99)))+(((x67)*(x96)))+(((x75)*(x78)))+(((x53)*(x54)))+(((IkReal(-1.00000000000000))*(gconst19)*(x108)))+(((x63)*(x75)))+(((x52)*(x87)))+(((x53)*(x77)))+(((x52)*(x97)))+(((IkReal(-1.00000000000000))*(x54)*(x67)))+(((IkReal(-1.00000000000000))*(x102)*(x53)))+(((IkReal(-1.00000000000000))*(x97)*(x99)))+(((gconst12)*(gconst17)*(x67))));
op[8]=((((IkReal(-1.00000000000000))*(gconst11)*(x108)))+(((IkReal(-1.00000000000000))*(x52)*(x53)))+(((x52)*(x67)))+(((x53)*(x75)))+(((IkReal(-1.00000000000000))*(x67)*(x99))));
polyroots8(op,zeror,numroots);
IkReal j4array[8], cj4array[8], sj4array[8], tempj4array[1];
int numsolutions = 0;
for(int ij4 = 0; ij4 < numroots; ++ij4)
{
IkReal htj4 = zeror[ij4];
tempj4array[0]=((IkReal(2.00000000000000))*(atan(htj4)));
for(int kj4 = 0; kj4 < 1; ++kj4)
{
j4array[numsolutions] = tempj4array[kj4];
if( j4array[numsolutions] > IKPI )
{
    j4array[numsolutions]-=IK2PI;
}
else if( j4array[numsolutions] < -IKPI )
{
    j4array[numsolutions]+=IK2PI;
}
sj4array[numsolutions] = IKsin(j4array[numsolutions]);
cj4array[numsolutions] = IKcos(j4array[numsolutions]);
numsolutions++;
}
}
bool j4valid[8]={true,true,true,true,true,true,true,true};
_nj4 = 8;
for(int ij4 = 0; ij4 < numsolutions; ++ij4)
    {
if( !j4valid[ij4] )
{
    continue;
}
    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
htj4 = IKtan(j4/2);

_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < numsolutions; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x109=((IkReal(0.707106781186548))*(cj4));
IkReal x110=((IkReal(0.707106781186548))*(sj4));
IkReal x111=((px)*(x110));
IkReal x112=((py)*(x109));
IkReal x113=((px)*(x109));
IkReal x114=((py)*(x110));
IkReal x115=((x113)+(x114));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x112)))+(x111)+(x115));
evalcond[3]=((IkReal(-0.850000000000000))+(pz));
evalcond[4]=((((IkReal(-1.00000000000000))*(x112)))+(((IkReal(-1.00000000000000))*(x115)))+(x111));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x116=(py)*(py);
IkReal x117=(sj4)*(sj4);
IkReal x118=(cj4)*(cj4);
IkReal x119=(px)*(px);
IkReal x120=((IkReal(0.0900000000000000))*(x118));
IkReal x121=((IkReal(0.0900000000000000))*(x117));
gconst48=IKsign(((((x116)*(x121)))+(((x116)*(x120)))+(((x119)*(x121)))+(((x119)*(x120)))));
IkReal x122=(px)*(px);
IkReal x123=(sj4)*(sj4);
IkReal x124=(cj4)*(cj4);
IkReal x125=(py)*(py);
dummyeval[0]=((((x122)*(x124)))+(((x122)*(x123)))+(((x123)*(x125)))+(((x124)*(x125))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x126=((IkReal(0.150260191002141))*(py));
IkReal x127=((cj4)*(py));
IkReal x128=((IkReal(0.777817459305202))*(pz));
IkReal x129=((cj4)*(px));
IkReal x130=((IkReal(0.707106781186548))*(pp));
IkReal x131=((px)*(sj4));
IkReal x132=((py)*(sj4));
IkReal x133=((cj4)*(x126));
IkReal x134=((IkReal(0.150260191002141))*(x129));
IkReal x135=((sj4)*(x126));
IkReal x136=((IkReal(0.150260191002141))*(x131));
IkReal x137=((x130)*(x132));
IkReal x138=((x128)*(x131));
IkReal x139=((x127)*(x130));
IkReal x140=((x129)*(x130));
IkReal x141=((x130)*(x131));
IkReal x142=((x127)*(x128));
IkReal x143=((x128)*(x132));
IkReal x144=((x128)*(x129));
IkReal x145=((x143)+(x144));
IkReal x146=((x133)+(x139)+(x138));
IkReal x147=((x141)+(x142)+(x136));
IkReal x148=((x140)+(x135)+(x134)+(x137));
if( IKabs(((gconst48)*(((((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(x147)))+(x145)+(x146))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst48)*(((((IkReal(-1.00000000000000))*(x147)))+(((IkReal(-1.00000000000000))*(x145)))+(x146)+(x148))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst48)*(((((IkReal(-1.00000000000000))*(x148)))+(((IkReal(-1.00000000000000))*(x147)))+(x145)+(x146)))), ((gconst48)*(((((IkReal(-1.00000000000000))*(x147)))+(((IkReal(-1.00000000000000))*(x145)))+(x146)+(x148)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x149=IKcos(j6);
IkReal x150=IKsin(j6);
IkReal x151=((py)*(sj4));
IkReal x152=((cj4)*(px));
IkReal x153=((px)*(sj4));
IkReal x154=((cj4)*(py));
IkReal x155=((IkReal(0.707106781186548))*(x149));
IkReal x156=((IkReal(0.0636396103067893))*(x150));
IkReal x157=((IkReal(0.0636396103067893))*(x149));
IkReal x158=((IkReal(0.707106781186548))*(x150));
IkReal x159=((x153)*(x155));
IkReal x160=((x153)*(x158));
IkReal x161=((x151)*(x158));
IkReal x162=((x152)*(x158));
IkReal x163=((x154)*(x155));
IkReal x164=((x151)*(x155));
IkReal x165=((x152)*(x155));
IkReal x166=((x154)*(x158));
IkReal x167=((x166)+(x164)+(x165));
IkReal x168=((x162)+(x161)+(x159));
evalcond[0]=((((IkReal(-1.00000000000000))*(x160)))+(((IkReal(-1.00000000000000))*(x163)))+(x168)+(x167));
evalcond[1]=((((IkReal(-1.00000000000000))*(x167)))+(((IkReal(-1.00000000000000))*(x163)))+(x168)+(x160));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x153)*(x156)))+(((IkReal(-1.00000000000000))*(x153)*(x157)))+(((IkReal(-1.00000000000000))*(x151)*(x156)))+(((x152)*(x157)))+(((x151)*(x157)))+(((IkReal(-1.00000000000000))*(x152)*(x156)))+(((IkReal(-1.00000000000000))*(pp)))+(((x154)*(x157)))+(((x154)*(x156)))+(((IkReal(1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x254=((IkReal(0.707106781186548))*(cj4));
IkReal x255=((IkReal(0.707106781186548))*(sj4));
IkReal x256=((px)*(x255));
IkReal x257=((py)*(x254));
IkReal x258=((px)*(x254));
IkReal x259=((py)*(x255));
IkReal x260=((x258)+(x259));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((x256)+(x260)+(((IkReal(-1.00000000000000))*(x257))));
evalcond[3]=((IkReal(-0.850000000000000))+(pz));
evalcond[4]=((x256)+(((IkReal(-1.00000000000000))*(x260)))+(((IkReal(-1.00000000000000))*(x257))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst49;
IkReal x261=(py)*(py);
IkReal x262=(sj4)*(sj4);
IkReal x263=(cj4)*(cj4);
IkReal x264=(px)*(px);
IkReal x265=((IkReal(0.0900000000000000))*(x263));
IkReal x266=((IkReal(0.0900000000000000))*(x262));
gconst49=IKsign(((((x264)*(x266)))+(((x264)*(x265)))+(((x261)*(x265)))+(((x261)*(x266)))));
IkReal x267=(px)*(px);
IkReal x268=(sj4)*(sj4);
IkReal x269=(cj4)*(cj4);
IkReal x270=(py)*(py);
dummyeval[0]=((((x267)*(x268)))+(((x267)*(x269)))+(((x269)*(x270)))+(((x268)*(x270))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x271=((py)*(sj4));
IkReal x272=((IkReal(0.707106781186548))*(pp));
IkReal x273=((IkReal(0.150260191002141))*(cj4));
IkReal x274=((cj4)*(py));
IkReal x275=((IkReal(0.777817459305202))*(pz));
IkReal x276=((px)*(sj4));
IkReal x277=((py)*(x273));
IkReal x278=((px)*(x273));
IkReal x279=((IkReal(0.150260191002141))*(x271));
IkReal x280=((IkReal(0.150260191002141))*(x276));
IkReal x281=((px)*(x275));
IkReal x282=((x271)*(x272));
IkReal x283=((x275)*(x276));
IkReal x284=((x272)*(x274));
IkReal x285=((cj4)*(px)*(x272));
IkReal x286=((x272)*(x276));
IkReal x287=((x274)*(x275));
IkReal x288=((x271)*(x275));
IkReal x289=((cj4)*(x281));
IkReal x290=((x289)+(x288));
IkReal x291=((x277)+(x284)+(x283));
IkReal x292=((x287)+(x286)+(x280));
IkReal x293=((x278)+(x279)+(x285)+(x282));
if( IKabs(((gconst49)*(((((IkReal(-1.00000000000000))*(x293)))+(((IkReal(-1.00000000000000))*(x292)))+(x290)+(x291))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst49)*(((((IkReal(-1.00000000000000))*(x292)))+(((IkReal(-1.00000000000000))*(x290)))+(x291)+(x293))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst49)*(((((IkReal(-1.00000000000000))*(x293)))+(((IkReal(-1.00000000000000))*(x292)))+(x290)+(x291)))), ((gconst49)*(((((IkReal(-1.00000000000000))*(x292)))+(((IkReal(-1.00000000000000))*(x290)))+(x291)+(x293)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x294=IKcos(j6);
IkReal x295=IKsin(j6);
IkReal x296=((py)*(sj4));
IkReal x297=((cj4)*(px));
IkReal x298=((px)*(sj4));
IkReal x299=((cj4)*(py));
IkReal x300=((IkReal(0.707106781186548))*(x294));
IkReal x301=((IkReal(0.0636396103067893))*(x295));
IkReal x302=((IkReal(0.0636396103067893))*(x294));
IkReal x303=((IkReal(0.707106781186548))*(x295));
IkReal x304=((x298)*(x300));
IkReal x305=((x298)*(x303));
IkReal x306=((x296)*(x303));
IkReal x307=((x297)*(x303));
IkReal x308=((x299)*(x300));
IkReal x309=((x296)*(x300));
IkReal x310=((x297)*(x300));
IkReal x311=((x299)*(x303));
IkReal x312=((x309)+(x311)+(x310));
IkReal x313=((x306)+(x307)+(x304));
evalcond[0]=((x313)+(x312)+(((IkReal(-1.00000000000000))*(x308)))+(((IkReal(-1.00000000000000))*(x305))));
evalcond[1]=((x305)+(x313)+(((IkReal(-1.00000000000000))*(x312)))+(((IkReal(-1.00000000000000))*(x308))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x298)*(x301)))+(((IkReal(-1.00000000000000))*(x298)*(x302)))+(((IkReal(-1.00000000000000))*(x296)*(x301)))+(((IkReal(-1.00000000000000))*(x297)*(x301)))+(((x299)*(x302)))+(((x299)*(x301)))+(((IkReal(-1.00000000000000))*(pp)))+(((x297)*(x302)))+(((IkReal(1.10000000000000))*(pz)))+(((x296)*(x302))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x314=((IkReal(785674201.318386))*(cj4));
IkReal x315=((IkReal(785674201.318386))*(sj4));
IkReal x316=((py)*(x315));
IkReal x317=((px)*(x314));
IkReal x318=((py)*(x314));
IkReal x319=((px)*(x315));
IkReal x320=((x317)+(x316));
if( IKabs(((((IkReal(-1.00000000000000))*(x320)))+(x318)+(((IkReal(-1.00000000000000))*(x319))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x320)+(x318)+(((IkReal(-1.00000000000000))*(x319))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x320)))+(x318)+(((IkReal(-1.00000000000000))*(x319)))))+IKsqr(((x320)+(x318)+(((IkReal(-1.00000000000000))*(x319)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x320)))+(x318)+(((IkReal(-1.00000000000000))*(x319)))), ((x320)+(x318)+(((IkReal(-1.00000000000000))*(x319)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x321=IKcos(j6);
IkReal x322=IKsin(j6);
IkReal x323=((py)*(sj4));
IkReal x324=((cj4)*(px));
IkReal x325=((px)*(sj4));
IkReal x326=((cj4)*(py));
IkReal x327=((IkReal(0.707106781186548))*(x325));
IkReal x328=((IkReal(0.707106781186548))*(x326));
IkReal x329=((IkReal(0.707106781186548))*(x324));
IkReal x330=((IkReal(0.707106781186548))*(x323));
IkReal x331=((IkReal(0.0636396103067893))*(x322));
IkReal x332=((IkReal(0.707106781186548))*(x321));
IkReal x333=((IkReal(0.707106781186548))*(x322));
IkReal x334=((IkReal(0.0636396103067893))*(x321));
IkReal x335=((x321)*(x327));
IkReal x336=((x322)*(x327));
IkReal x337=((x322)*(x330));
IkReal x338=((x322)*(x329));
IkReal x339=((x321)*(x328));
IkReal x340=((x321)*(x330));
IkReal x341=((x321)*(x329));
IkReal x342=((x322)*(x328));
IkReal x343=((x330)+(x329));
IkReal x344=((x342)+(x340)+(x341));
IkReal x345=((x338)+(x337)+(x335));
evalcond[0]=((((IkReal(9.00000000000000e-10))*(x322)))+(((IkReal(-1.00000000000000))*(x328)))+(x327)+(x343));
evalcond[1]=((((IkReal(9.00000000000000e-10))*(x321)))+(((IkReal(-1.00000000000000))*(x328)))+(x327)+(((IkReal(-1.00000000000000))*(x343))));
evalcond[2]=((x344)+(x345)+(((IkReal(-1.00000000000000))*(x339)))+(((IkReal(-1.00000000000000))*(x336))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(x336)+(x345)+(((IkReal(-1.00000000000000))*(x339)))+(((IkReal(-1.00000000000000))*(x344))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x324)*(x334)))+(((IkReal(-1.00000000000000))*(x324)*(x331)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x323)*(x331)))+(((x323)*(x334)))+(((x326)*(x334)))+(((x326)*(x331)))+(((IkReal(-1.00000000000000))*(x325)*(x331)))+(((IkReal(-1.00000000000000))*(x325)*(x334))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x346=((IkReal(0.707106781186548))*(sj4));
IkReal x347=((IkReal(0.707106781186548))*(cj4));
IkReal x348=((py)*(x347));
IkReal x349=((px)*(x347));
IkReal x350=((py)*(x346));
IkReal x351=((px)*(x346));
IkReal x352=((x350)+(x349));
if( IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x351)))+(((IkReal(-1.00000000000000))*(x352)))+(x348))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x351)))+(x352)+(x348))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst47)*(((((IkReal(-1.00000000000000))*(x351)))+(((IkReal(-1.00000000000000))*(x352)))+(x348)))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x351)))+(x352)+(x348)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x353=IKsin(j6);
IkReal x354=IKcos(j6);
IkReal x355=((py)*(sj4));
IkReal x356=((cj4)*(px));
IkReal x357=((px)*(sj4));
IkReal x358=((IkReal(0.0450000000000000))*(cj7));
IkReal x359=((cj4)*(py));
IkReal x360=((IkReal(0.300000000000000))*(sj7));
IkReal x361=((IkReal(0.707106781186548))*(x357));
IkReal x362=((IkReal(0.707106781186548))*(x359));
IkReal x363=((IkReal(0.707106781186548))*(x356));
IkReal x364=((IkReal(0.707106781186548))*(x355));
IkReal x365=((IkReal(0.707106781186548))*(x354));
IkReal x366=((IkReal(0.0636396103067893))*(x353));
IkReal x367=((IkReal(0.707106781186548))*(x353));
IkReal x368=((IkReal(0.0636396103067893))*(x354));
IkReal x369=((x354)*(x361));
IkReal x370=((x353)*(x361));
IkReal x371=((x353)*(x364));
IkReal x372=((x353)*(x363));
IkReal x373=((x354)*(x362));
IkReal x374=((x354)*(x364));
IkReal x375=((x354)*(x363));
IkReal x376=((x353)*(x362));
IkReal x377=((x363)+(x364));
IkReal x378=((x376)+(x375)+(x374));
IkReal x379=((x372)+(x371)+(x369));
evalcond[0]=((((IkReal(0.0450000000000000))*(x353)))+(((IkReal(-1.00000000000000))*(x362)))+(((x353)*(x360)))+(x377)+(x361)+(((IkReal(-1.00000000000000))*(x353)*(x358))));
evalcond[1]=((((IkReal(0.0450000000000000))*(x354)))+(((IkReal(-1.00000000000000))*(x362)))+(((IkReal(-1.00000000000000))*(x377)))+(x361)+(((x354)*(x360)))+(((IkReal(-1.00000000000000))*(x354)*(x358))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x373)))+(((IkReal(-1.00000000000000))*(x370)))+(x379)+(x378));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x358)))+(((IkReal(-1.00000000000000))*(x373)))+(((IkReal(-1.00000000000000))*(x378)))+(x379)+(x370)+(x360));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x355)*(x366)))+(((x355)*(x368)))+(((IkReal(-1.00000000000000))*(x356)*(x366)))+(((x359)*(x366)))+(((x359)*(x368)))+(((x356)*(x368)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x357)*(x368)))+(((IkReal(-1.00000000000000))*(x357)*(x366))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj7))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst50;
gconst50=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst51;
gconst51=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst52;
gconst52=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst53;
gconst53=((IkReal(0.0450000000000000))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7))));
IkReal gconst54;
gconst54=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal gconst55;
gconst55=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst56;
gconst56=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst57;
gconst57=((((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.707106781186548))*(py))));
IkReal gconst58;
gconst58=((IkReal(0.0450000000000000))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7))));
IkReal gconst59;
gconst59=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(0.707106781186548))*(py)))+(((IkReal(0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal gconst60;
gconst60=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst61;
gconst61=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst62;
gconst62=((((IkReal(-1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst63;
gconst63=((((IkReal(-1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst64;
gconst64=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst65;
gconst65=((((IkReal(1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst66;
gconst66=((((IkReal(-1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst67;
gconst67=((((IkReal(-1.41421356237310))*(px)))+(((IkReal(1.41421356237310))*(py))));
IkReal gconst68;
gconst68=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst69;
gconst69=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst70;
gconst70=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst71;
gconst71=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst72;
gconst72=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal gconst73;
gconst73=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst74;
gconst74=((IkReal(0.0900000000000000))+(((IkReal(0.600000000000000))*(sj7)))+(((IkReal(-0.0900000000000000))*(cj7))));
IkReal gconst75;
gconst75=((((IkReal(0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst76;
gconst76=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px))));
IkReal gconst77;
gconst77=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-0.707106781186548))*(py)))+(((IkReal(-0.707106781186548))*(px)))+(((IkReal(-0.300000000000000))*(sj7))));
IkReal x380=((IkReal(1.00000000000000))*(gconst72));
IkReal x381=((gconst73)*(gconst77));
IkReal x382=((gconst70)*(gconst71));
IkReal x383=((gconst75)*(gconst76));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst76)*(x380)))+(((IkReal(-1.00000000000000))*(x382)*(x383)))+(((IkReal(-1.00000000000000))*(gconst68)*(x380)*(x381)))+(((gconst68)*(gconst72)*(x383)))+(((x381)*(x382))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x384=((gconst54)*(gconst59));
IkReal x385=((gconst72)*(gconst77));
IkReal x386=((gconst53)*(gconst61));
IkReal x387=((gconst59)*(gconst64));
IkReal x388=((gconst54)*(gconst66));
IkReal x389=((gconst58)*(gconst72));
IkReal x390=((gconst71)*(gconst73));
IkReal x391=((gconst50)*(gconst65));
IkReal x392=((gconst54)*(gconst58));
IkReal x393=((gconst75)*(gconst76));
IkReal x394=((gconst68)*(gconst72));
IkReal x395=((gconst55)*(gconst59));
IkReal x396=((IkReal(1.00000000000000))*(gconst50));
IkReal x397=((gconst52)*(gconst71));
IkReal x398=((gconst58)*(gconst63));
IkReal x399=((gconst54)*(gconst77));
IkReal x400=((gconst50)*(gconst57));
IkReal x401=((IkReal(1.00000000000000))*(gconst62));
IkReal x402=((IkReal(1.00000000000000))*(gconst76));
IkReal x403=((gconst72)*(gconst74));
IkReal x404=((gconst66)*(gconst75));
IkReal x405=((gconst60)*(gconst72));
IkReal x406=((gconst52)*(gconst77));
IkReal x407=((gconst61)*(gconst77));
IkReal x408=((gconst63)*(gconst68));
IkReal x409=((gconst65)*(gconst76));
IkReal x410=((gconst70)*(gconst71));
IkReal x411=((IkReal(1.00000000000000))*(gconst60));
IkReal x412=((gconst65)*(gconst66));
IkReal x413=((gconst57)*(gconst58));
IkReal x414=((gconst56)*(gconst69));
IkReal x415=((gconst58)*(gconst75));
IkReal x416=((gconst54)*(gconst76));
IkReal x417=((gconst58)*(gconst65));
IkReal x418=((gconst57)*(gconst66));
IkReal x419=((IkReal(1.00000000000000))*(gconst74));
IkReal x420=((gconst55)*(gconst67));
IkReal x421=((gconst52)*(gconst62));
IkReal x422=((IkReal(1.00000000000000))*(gconst73));
IkReal x423=((gconst52)*(gconst53));
IkReal x424=((gconst63)*(gconst66));
IkReal x425=((gconst63)*(gconst77));
IkReal x426=((gconst61)*(gconst71));
IkReal x427=((gconst69)*(gconst74));
IkReal x428=((IkReal(1.00000000000000))*(gconst64));
IkReal x429=((gconst53)*(gconst70));
IkReal x430=((gconst57)*(gconst60));
IkReal x431=((gconst62)*(gconst73));
IkReal x432=((gconst64)*(gconst67));
IkReal x433=((gconst67)*(gconst70));
IkReal x434=((IkReal(1.00000000000000))*(gconst55)*(gconst68));
IkReal x435=((gconst77)*(x410));
IkReal x436=((IkReal(1.00000000000000))*(gconst51)*(gconst56));
IkReal x437=((gconst68)*(x422));
IkReal x438=((gconst62)*(gconst70)*(gconst77));
IkReal x439=((IkReal(1.00000000000000))*(x423));
op[0]=((((IkReal(-1.00000000000000))*(x393)*(x410)))+(((IkReal(-1.00000000000000))*(x385)*(x437)))+(((IkReal(-1.00000000000000))*(gconst69)*(x402)*(x403)))+(((gconst70)*(gconst77)*(x390)))+(((x393)*(x394))));
op[1]=((((IkReal(-1.00000000000000))*(gconst66)*(gconst69)*(x403)))+(((gconst70)*(gconst77)*(x431)))+(((x390)*(x407)))+(((IkReal(-1.00000000000000))*(gconst73)*(x385)*(x411)))+(((IkReal(-1.00000000000000))*(gconst68)*(x385)*(x428)))+(((x393)*(x408)))+(((x393)*(x405)))+(((IkReal(-1.00000000000000))*(gconst65)*(x402)*(x410)))+(((gconst64)*(x435)))+(((IkReal(-1.00000000000000))*(gconst63)*(x402)*(x427)))+(((IkReal(-1.00000000000000))*(gconst77)*(x408)*(x422)))+(((x390)*(x433)))+(((IkReal(-1.00000000000000))*(x393)*(x426)))+(((IkReal(-1.00000000000000))*(gconst70)*(x393)*(x401)))+(((IkReal(-1.00000000000000))*(gconst67)*(x394)*(x422)))+(((IkReal(-1.00000000000000))*(x404)*(x410)))+(((x394)*(x409)))+(((x394)*(x404))));
op[2]=((((IkReal(-1.00000000000000))*(gconst65)*(x402)*(x426)))+(((IkReal(-1.00000000000000))*(gconst69)*(x419)*(x424)))+(((gconst57)*(gconst76)*(x394)))+(((x405)*(x409)))+(((IkReal(-1.00000000000000))*(gconst64)*(x385)*(x411)))+(((x390)*(x406)))+(((x404)*(x408)))+(((x404)*(x405)))+(((IkReal(-1.00000000000000))*(gconst73)*(x411)*(x425)))+(((IkReal(-1.00000000000000))*(gconst54)*(x402)*(x427)))+(((gconst54)*(gconst68)*(x393)))+(((IkReal(-1.00000000000000))*(gconst72)*(x402)*(x414)))+(((IkReal(-1.00000000000000))*(x410)*(x415)))+(((IkReal(-1.00000000000000))*(x410)*(x412)))+(((IkReal(-1.00000000000000))*(gconst51)*(x402)*(x403)))+(((IkReal(-1.00000000000000))*(gconst57)*(x402)*(x410)))+(((x410)*(x432)))+(((gconst64)*(x438)))+(((IkReal(-1.00000000000000))*(gconst59)*(x394)*(x422)))+(((x408)*(x409)))+(((gconst55)*(x435)))+(((IkReal(-1.00000000000000))*(gconst77)*(x408)*(x428)))+(((gconst68)*(gconst75)*(x389)))+(((IkReal(-1.00000000000000))*(gconst69)*(x389)*(x419)))+(((IkReal(-1.00000000000000))*(x399)*(x437)))+(((IkReal(-1.00000000000000))*(x393)*(x429)))+(((IkReal(-1.00000000000000))*(x385)*(x434)))+(((gconst50)*(gconst72)*(x393)))+(((gconst60)*(gconst63)*(x393)))+(((IkReal(-1.00000000000000))*(gconst70)*(x401)*(x409)))+(((IkReal(-1.00000000000000))*(gconst70)*(x401)*(x404)))+(((IkReal(-1.00000000000000))*(gconst67)*(x408)*(x422)))+(((x431)*(x433)))+(((x394)*(x412)))+(((x407)*(x431)))+(((IkReal(-1.00000000000000))*(gconst67)*(x394)*(x428)))+(((IkReal(-1.00000000000000))*(gconst73)*(x385)*(x396)))+(((gconst64)*(gconst71)*(x407)))+(((IkReal(-1.00000000000000))*(x404)*(x426)))+(((IkReal(-1.00000000000000))*(gconst67)*(x405)*(x422)))+(((gconst73)*(gconst77)*(x429)))+(((gconst61)*(gconst67)*(x390)))+(((gconst59)*(gconst70)*(x390)))+(((IkReal(-1.00000000000000))*(gconst61)*(x393)*(x401)))+(((IkReal(-1.00000000000000))*(x393)*(x397))));
op[3]=((((IkReal(-1.00000000000000))*(gconst57)*(gconst66)*(gconst70)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst62)*(gconst75)*(gconst76)))+(((gconst58)*(gconst63)*(gconst68)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst67)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst62)*(gconst70)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst67)*(gconst68)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst60)*(gconst72)*(gconst77)))+(((gconst54)*(gconst66)*(gconst68)*(gconst75)))+(((gconst50)*(gconst63)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst69)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst66)*(gconst69)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst60)*(gconst72)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst67)*(gconst68)))+(((gconst60)*(gconst63)*(gconst65)*(gconst76)))+(((gconst60)*(gconst63)*(gconst66)*(gconst75)))+(((gconst54)*(gconst65)*(gconst68)*(gconst76)))+(((gconst57)*(gconst60)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst70)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst63)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst67)*(gconst72)*(gconst73)))+(((gconst61)*(gconst64)*(gconst67)*(gconst71)))+(((gconst59)*(gconst62)*(gconst70)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst68)*(gconst72)))+(((gconst52)*(gconst62)*(gconst73)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst66)*(gconst71)*(gconst75)))+(((gconst54)*(gconst60)*(gconst75)*(gconst76)))+(((gconst57)*(gconst66)*(gconst68)*(gconst72)))+(((gconst59)*(gconst64)*(gconst70)*(gconst71)))+(((gconst59)*(gconst61)*(gconst71)*(gconst73)))+(((gconst52)*(gconst64)*(gconst71)*(gconst77)))+(((gconst55)*(gconst61)*(gconst71)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst65)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst65)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst65)*(gconst70)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst63)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst61)*(gconst75)*(gconst76)))+(((gconst58)*(gconst60)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst66)*(gconst75)))+(((gconst52)*(gconst67)*(gconst71)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst61)*(gconst71)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst63)*(gconst68)*(gconst77)))+(((gconst62)*(gconst64)*(gconst67)*(gconst70)))+(((gconst63)*(gconst65)*(gconst66)*(gconst68)))+(((gconst61)*(gconst62)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst63)*(gconst73)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst63)*(gconst69)*(gconst76)))+(((gconst57)*(gconst63)*(gconst68)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst65)*(gconst66)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst66)*(gconst72)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst71)*(gconst76)))+(((gconst53)*(gconst61)*(gconst73)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst62)*(gconst70)*(gconst75)))+(((gconst50)*(gconst66)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst67)*(gconst68)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst64)*(gconst68)*(gconst77)))+(((gconst53)*(gconst67)*(gconst70)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst66)*(gconst69)*(gconst72)))+(((gconst55)*(gconst62)*(gconst70)*(gconst77)))+(((gconst50)*(gconst65)*(gconst72)*(gconst76)))+(((gconst58)*(gconst65)*(gconst68)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst64)*(gconst72)*(gconst77)))+(((gconst55)*(gconst67)*(gconst70)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst68)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst60)*(gconst73)*(gconst77)))+(((gconst60)*(gconst65)*(gconst66)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst66)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst65)*(gconst66)*(gconst70)))+(((gconst53)*(gconst64)*(gconst70)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst63)*(gconst67)*(gconst73)))+(((gconst61)*(gconst62)*(gconst67)*(gconst73))));
op[4]=((((IkReal(-1.00000000000000))*(gconst55)*(gconst63)*(gconst67)*(gconst68)))+(((gconst54)*(gconst60)*(gconst65)*(gconst76)))+(((gconst50)*(gconst63)*(gconst65)*(gconst76)))+(((gconst55)*(gconst61)*(gconst67)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst59)*(gconst68)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst58)*(gconst71)*(gconst75)))+(((gconst52)*(gconst64)*(gconst67)*(gconst71)))+(((gconst59)*(gconst61)*(gconst64)*(gconst71)))+(((gconst57)*(gconst60)*(gconst66)*(gconst72)))+(((gconst54)*(gconst65)*(gconst66)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst58)*(gconst72)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst61)*(gconst65)*(gconst76)))+(((gconst54)*(gconst57)*(gconst68)*(gconst76)))+(((gconst50)*(gconst65)*(gconst66)*(gconst72)))+(((gconst60)*(gconst63)*(gconst65)*(gconst66)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst62)*(gconst76)))+(((gconst55)*(gconst59)*(gconst70)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst64)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst59)*(gconst68)*(gconst73)))+(((gconst52)*(gconst53)*(gconst73)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst60)*(gconst67)*(gconst72)))+(((gconst57)*(gconst60)*(gconst63)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst56)*(gconst69)*(gconst76)))+(((gconst57)*(gconst58)*(gconst68)*(gconst72)))+(((gconst54)*(gconst58)*(gconst68)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst64)*(gconst67)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst57)*(gconst71)*(gconst76)))+(((gconst52)*(gconst62)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst62)*(gconst66)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst61)*(gconst62)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst63)*(gconst67)*(gconst73)))+(((gconst58)*(gconst60)*(gconst63)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst55)*(gconst68)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst65)*(gconst66)))+(((gconst53)*(gconst61)*(gconst67)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst53)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst63)*(gconst64)*(gconst67)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst62)*(gconst65)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst57)*(gconst70)*(gconst76)))+(((gconst53)*(gconst64)*(gconst67)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst65)*(gconst66)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst54)*(gconst73)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst58)*(gconst69)*(gconst74)))+(((gconst58)*(gconst63)*(gconst65)*(gconst68)))+(((gconst55)*(gconst61)*(gconst62)*(gconst77)))+(((gconst55)*(gconst62)*(gconst67)*(gconst70)))+(((gconst50)*(gconst57)*(gconst72)*(gconst76)))+(((gconst50)*(gconst58)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst61)*(gconst66)*(gconst75)))+(((gconst58)*(gconst60)*(gconst65)*(gconst72)))+(((gconst53)*(gconst61)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst61)*(gconst65)*(gconst71)))+(((gconst59)*(gconst61)*(gconst62)*(gconst73)))+(((gconst61)*(gconst62)*(gconst64)*(gconst67)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst65)*(gconst66)*(gconst71)))+(((gconst52)*(gconst55)*(gconst71)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst66)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst62)*(gconst65)*(gconst76)))+(((gconst52)*(gconst59)*(gconst71)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst55)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst63)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst59)*(gconst72)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst60)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst58)*(gconst70)*(gconst71)))+(((gconst53)*(gconst59)*(gconst70)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst60)*(gconst63)*(gconst77)))+(((gconst57)*(gconst63)*(gconst66)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst60)*(gconst64)*(gconst72)))+(((gconst52)*(gconst62)*(gconst67)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst63)*(gconst66)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst62)*(gconst66)*(gconst75)))+(((gconst50)*(gconst63)*(gconst66)*(gconst75)))+(((gconst50)*(gconst54)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst60)*(gconst67)*(gconst73)))+(((gconst53)*(gconst55)*(gconst70)*(gconst77)))+(((gconst59)*(gconst62)*(gconst64)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst54)*(gconst74)*(gconst76)))+(((gconst54)*(gconst60)*(gconst66)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst64)*(gconst67)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst58)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst58)*(gconst69)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst63)*(gconst66)*(gconst69)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst56)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst60)*(gconst63)*(gconst73))));
op[5]=((((IkReal(-1.00000000000000))*(gconst54)*(gconst59)*(gconst64)*(gconst68)))+(((gconst52)*(gconst53)*(gconst67)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst58)*(gconst63)*(gconst74)))+(((gconst50)*(gconst58)*(gconst65)*(gconst72)))+(((gconst53)*(gconst55)*(gconst61)*(gconst77)))+(((gconst54)*(gconst60)*(gconst65)*(gconst66)))+(((gconst54)*(gconst57)*(gconst66)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst62)*(gconst66)))+(((gconst52)*(gconst55)*(gconst62)*(gconst77)))+(((gconst57)*(gconst60)*(gconst63)*(gconst66)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst61)*(gconst65)*(gconst66)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst58)*(gconst62)*(gconst75)))+(((gconst50)*(gconst54)*(gconst66)*(gconst75)))+(((gconst50)*(gconst54)*(gconst65)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst54)*(gconst64)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst58)*(gconst61)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst53)*(gconst65)*(gconst76)))+(((gconst50)*(gconst58)*(gconst63)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst61)*(gconst62)*(gconst65)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst57)*(gconst61)*(gconst76)))+(((gconst52)*(gconst62)*(gconst64)*(gconst67)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst58)*(gconst61)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst58)*(gconst62)*(gconst70)))+(((gconst53)*(gconst59)*(gconst61)*(gconst73)))+(((gconst50)*(gconst57)*(gconst66)*(gconst72)))+(((gconst50)*(gconst57)*(gconst63)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst59)*(gconst60)*(gconst73)))+(((gconst58)*(gconst60)*(gconst63)*(gconst65)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst55)*(gconst63)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst59)*(gconst63)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst53)*(gconst66)*(gconst75)))+(((gconst53)*(gconst55)*(gconst67)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst56)*(gconst63)*(gconst76)))+(((gconst53)*(gconst59)*(gconst64)*(gconst70)))+(((gconst54)*(gconst58)*(gconst65)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst54)*(gconst66)*(gconst74)))+(((gconst52)*(gconst53)*(gconst64)*(gconst77)))+(((gconst52)*(gconst59)*(gconst64)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst57)*(gconst66)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst57)*(gconst62)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst55)*(gconst60)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst59)*(gconst64)*(gconst72)))+(((gconst55)*(gconst61)*(gconst62)*(gconst67)))+(((gconst57)*(gconst58)*(gconst60)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst55)*(gconst67)*(gconst68)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst59)*(gconst63)*(gconst73)))+(((gconst53)*(gconst61)*(gconst64)*(gconst67)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst54)*(gconst67)*(gconst73)))+(((gconst54)*(gconst58)*(gconst60)*(gconst75)))+(((gconst59)*(gconst61)*(gconst62)*(gconst64)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst58)*(gconst63)*(gconst69)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst58)*(gconst65)*(gconst70)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst59)*(gconst60)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst56)*(gconst66)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst58)*(gconst65)*(gconst71)))+(((gconst52)*(gconst59)*(gconst62)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst62)*(gconst65)*(gconst66)))+(((IkReal(-1.00000000000000))*(gconst55)*(gconst60)*(gconst63)*(gconst67)))+(((gconst54)*(gconst57)*(gconst60)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst52)*(gconst57)*(gconst66)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst56)*(gconst66)*(gconst69)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst63)*(gconst64)*(gconst67)))+(((IkReal(-1.00000000000000))*(gconst50)*(gconst55)*(gconst67)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst54)*(gconst60)*(gconst64)*(gconst67)))+(((gconst55)*(gconst59)*(gconst62)*(gconst70)))+(((gconst52)*(gconst55)*(gconst67)*(gconst71)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst60)*(gconst63)*(gconst64)))+(((gconst55)*(gconst59)*(gconst61)*(gconst71)))+(((gconst57)*(gconst58)*(gconst63)*(gconst68)))+(((gconst50)*(gconst63)*(gconst65)*(gconst66))));
op[6]=((((x398)*(x430)))+(((x420)*(x421)))+(((IkReal(-1.00000000000000))*(gconst64)*(x384)*(x411)))+(((IkReal(-1.00000000000000))*(x392)*(x414)))+(((IkReal(-1.00000000000000))*(gconst52)*(x401)*(x418)))+(((IkReal(-1.00000000000000))*(gconst52)*(x401)*(x417)))+(((IkReal(-1.00000000000000))*(x397)*(x413)))+(((IkReal(-1.00000000000000))*(gconst51)*(gconst54)*(gconst56)*(x402)))+(((IkReal(-1.00000000000000))*(gconst63)*(x395)*(x411)))+(((x386)*(x420)))+(((gconst53)*(gconst55)*(x406)))+(((IkReal(-1.00000000000000))*(gconst57)*(x402)*(x423)))+(((IkReal(-1.00000000000000))*(x413)*(x429)))+(((x387)*(x421)))+(((x395)*(x397)))+(((x395)*(x429)))+(((IkReal(-1.00000000000000))*(gconst55)*(x396)*(x399)))+(((IkReal(-1.00000000000000))*(x384)*(x434)))+(((x386)*(x387)))+(((IkReal(-1.00000000000000))*(gconst54)*(x396)*(x432)))+(((gconst57)*(gconst68)*(x392)))+(((x389)*(x400)))+(((IkReal(-1.00000000000000))*(gconst54)*(x411)*(x420)))+(((IkReal(-1.00000000000000))*(gconst63)*(x396)*(x420)))+(((x391)*(x398)))+(((IkReal(-1.00000000000000))*(x424)*(x436)))+(((IkReal(-1.00000000000000))*(gconst51)*(x392)*(x419)))+(((x423)*(x432)))+(((IkReal(-1.00000000000000))*(x389)*(x436)))+(((IkReal(-1.00000000000000))*(gconst61)*(x401)*(x413)))+(((gconst59)*(gconst73)*(x423)))+(((gconst61)*(gconst62)*(x395)))+(((IkReal(-1.00000000000000))*(gconst72)*(x395)*(x396)))+(((gconst60)*(gconst65)*(x392)))+(((IkReal(-1.00000000000000))*(x415)*(x439)))+(((IkReal(-1.00000000000000))*(gconst73)*(x384)*(x396)))+(((gconst50)*(gconst75)*(x392)))+(((x388)*(x430)))+(((IkReal(-1.00000000000000))*(x386)*(x417)))+(((IkReal(-1.00000000000000))*(x386)*(x418)))+(((x400)*(x416)))+(((IkReal(-1.00000000000000))*(gconst63)*(x387)*(x396)))+(((IkReal(-1.00000000000000))*(x412)*(x439)))+(((x388)*(x391)))+(((x400)*(x424))));
op[7]=((((x420)*(x423)))+(((IkReal(-1.00000000000000))*(gconst64)*(x384)*(x396)))+(((IkReal(-1.00000000000000))*(gconst52)*(x401)*(x413)))+(((x387)*(x423)))+(((x395)*(x421)))+(((IkReal(-1.00000000000000))*(gconst63)*(x395)*(x396)))+(((x391)*(x392)))+(((x388)*(x400)))+(((x392)*(x430)))+(((IkReal(-1.00000000000000))*(gconst54)*(x396)*(x420)))+(((IkReal(-1.00000000000000))*(x398)*(x436)))+(((IkReal(-1.00000000000000))*(x388)*(x436)))+(((IkReal(-1.00000000000000))*(x417)*(x439)))+(((x398)*(x400)))+(((IkReal(-1.00000000000000))*(gconst55)*(x384)*(x411)))+(((IkReal(-1.00000000000000))*(x386)*(x413)))+(((x386)*(x395)))+(((IkReal(-1.00000000000000))*(x418)*(x439))));
op[8]=((((IkReal(-1.00000000000000))*(x413)*(x439)))+(((IkReal(-1.00000000000000))*(gconst55)*(x384)*(x396)))+(((x392)*(x400)))+(((IkReal(-1.00000000000000))*(x392)*(x436)))+(((x395)*(x423))));
polyroots8(op,zeror,numroots);
IkReal j4array[8], cj4array[8], sj4array[8], tempj4array[1];
int numsolutions = 0;
for(int ij4 = 0; ij4 < numroots; ++ij4)
{
IkReal htj4 = zeror[ij4];
tempj4array[0]=((IkReal(2.00000000000000))*(atan(htj4)));
for(int kj4 = 0; kj4 < 1; ++kj4)
{
j4array[numsolutions] = tempj4array[kj4];
if( j4array[numsolutions] > IKPI )
{
    j4array[numsolutions]-=IK2PI;
}
else if( j4array[numsolutions] < -IKPI )
{
    j4array[numsolutions]+=IK2PI;
}
sj4array[numsolutions] = IKsin(j4array[numsolutions]);
cj4array[numsolutions] = IKcos(j4array[numsolutions]);
numsolutions++;
}
}
bool j4valid[8]={true,true,true,true,true,true,true,true};
_nj4 = 8;
for(int ij4 = 0; ij4 < numsolutions; ++ij4)
    {
if( !j4valid[ij4] )
{
    continue;
}
    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
htj4 = IKtan(j4/2);

_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < numsolutions; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst94;
gconst94=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x440=((IkReal(0.707106781186548))*(sj4));
IkReal x441=((IkReal(0.707106781186548))*(cj4));
IkReal x442=((py)*(x441));
IkReal x443=((px)*(x441));
IkReal x444=((py)*(x440));
IkReal x445=((px)*(x440));
IkReal x446=((x444)+(x443));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x442)))+(x445)+(x446));
evalcond[3]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x445)))+(x446)+(x442));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst95;
IkReal x447=(py)*(py);
IkReal x448=(sj4)*(sj4);
IkReal x449=(cj4)*(cj4);
IkReal x450=(px)*(px);
IkReal x451=((IkReal(0.0900000000000000))*(x449));
IkReal x452=((IkReal(0.0900000000000000))*(x448));
gconst95=IKsign(((((x447)*(x451)))+(((x447)*(x452)))+(((x450)*(x451)))+(((x450)*(x452)))));
IkReal x453=(px)*(px);
IkReal x454=(sj4)*(sj4);
IkReal x455=(cj4)*(cj4);
IkReal x456=(py)*(py);
dummyeval[0]=((((x454)*(x456)))+(((x455)*(x456)))+(((x453)*(x455)))+(((x453)*(x454))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x457=((IkReal(0.150260191002141))*(py));
IkReal x458=((cj4)*(px));
IkReal x459=((IkReal(0.707106781186548))*(pp));
IkReal x460=((cj4)*(py));
IkReal x461=((IkReal(0.777817459305202))*(pz));
IkReal x462=((px)*(sj4));
IkReal x463=((py)*(sj4));
IkReal x464=((IkReal(0.150260191002141))*(x462));
IkReal x465=((cj4)*(x457));
IkReal x466=((IkReal(0.150260191002141))*(x458));
IkReal x467=((sj4)*(x457));
IkReal x468=((x459)*(x462));
IkReal x469=((x461)*(x462));
IkReal x470=((x459)*(x463));
IkReal x471=((x460)*(x461));
IkReal x472=((x461)*(x463));
IkReal x473=((x458)*(x461));
IkReal x474=((x459)*(x460));
IkReal x475=((x458)*(x459));
IkReal x476=((x469)+(x468)+(x464));
IkReal x477=((x465)+(x474)+(x471));
IkReal x478=((x467)+(x466)+(x475)+(x470)+(x472)+(x473));
if( IKabs(((gconst95)*(((((IkReal(-1.00000000000000))*(x476)))+(((IkReal(-1.00000000000000))*(x478)))+(x477))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst95)*(((((IkReal(-1.00000000000000))*(x477)))+(((IkReal(-1.00000000000000))*(x478)))+(x476))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst95)*(((((IkReal(-1.00000000000000))*(x476)))+(((IkReal(-1.00000000000000))*(x478)))+(x477)))), ((gconst95)*(((((IkReal(-1.00000000000000))*(x477)))+(((IkReal(-1.00000000000000))*(x478)))+(x476)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x479=IKcos(j6);
IkReal x480=IKsin(j6);
IkReal x481=((py)*(sj4));
IkReal x482=((cj4)*(px));
IkReal x483=((px)*(sj4));
IkReal x484=((cj4)*(py));
IkReal x485=((IkReal(0.707106781186548))*(x479));
IkReal x486=((IkReal(0.0636396103067893))*(x480));
IkReal x487=((IkReal(0.0636396103067893))*(x479));
IkReal x488=((IkReal(0.707106781186548))*(x480));
IkReal x489=((x484)*(x485));
IkReal x490=((x481)*(x485));
IkReal x491=((x482)*(x485));
IkReal x492=((x483)*(x488));
IkReal x493=((x481)*(x488));
IkReal x494=((x482)*(x488));
IkReal x495=((x483)*(x485));
IkReal x496=((x484)*(x488));
IkReal x497=((x489)+(x493)+(x494));
IkReal x498=((x492)+(x490)+(x491));
evalcond[0]=((((IkReal(-1.00000000000000))*(x496)))+(((IkReal(-1.00000000000000))*(x497)))+(x498)+(x495));
evalcond[1]=((((IkReal(-1.00000000000000))*(x496)))+(((IkReal(-1.00000000000000))*(x495)))+(x498)+(x497));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x484)*(x487)))+(((x484)*(x486)))+(((IkReal(-1.00000000000000))*(x483)*(x486)))+(((x483)*(x487)))+(((IkReal(-1.00000000000000))*(x481)*(x486)))+(((IkReal(-1.00000000000000))*(x481)*(x487)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x482)*(x487)))+(((IkReal(-1.00000000000000))*(x482)*(x486))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x499=((IkReal(0.707106781186548))*(sj4));
IkReal x500=((IkReal(0.707106781186548))*(cj4));
IkReal x501=((py)*(x500));
IkReal x502=((px)*(x500));
IkReal x503=((py)*(x499));
IkReal x504=((px)*(x499));
IkReal x505=((x502)+(x503));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x501)))+(x504)+(x505));
evalcond[3]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x504)))+(x501)+(x505));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst96;
IkReal x506=(py)*(py);
IkReal x507=(sj4)*(sj4);
IkReal x508=(cj4)*(cj4);
IkReal x509=(px)*(px);
IkReal x510=((IkReal(0.0900000000000000))*(x508));
IkReal x511=((IkReal(0.0900000000000000))*(x507));
gconst96=IKsign(((((x506)*(x510)))+(((x506)*(x511)))+(((x509)*(x510)))+(((x509)*(x511)))));
IkReal x512=(px)*(px);
IkReal x513=(sj4)*(sj4);
IkReal x514=(cj4)*(cj4);
IkReal x515=(py)*(py);
dummyeval[0]=((((x514)*(x515)))+(((x513)*(x515)))+(((x512)*(x514)))+(((x512)*(x513))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x516=((py)*(sj4));
IkReal x517=((IkReal(0.707106781186548))*(pp));
IkReal x518=((cj4)*(px));
IkReal x519=((cj4)*(py));
IkReal x520=((IkReal(0.777817459305202))*(pz));
IkReal x521=((px)*(sj4));
IkReal x522=((IkReal(0.150260191002141))*(x521));
IkReal x523=((IkReal(0.150260191002141))*(x519));
IkReal x524=((IkReal(0.150260191002141))*(x518));
IkReal x525=((IkReal(0.150260191002141))*(x516));
IkReal x526=((x517)*(x521));
IkReal x527=((x520)*(x521));
IkReal x528=((x516)*(x517));
IkReal x529=((x519)*(x520));
IkReal x530=((x516)*(x520));
IkReal x531=((x518)*(x520));
IkReal x532=((x517)*(x519));
IkReal x533=((x517)*(x518));
IkReal x534=((x522)+(x526)+(x527));
IkReal x535=((x532)+(x523)+(x529));
IkReal x536=((x531)+(x530)+(x533)+(x524)+(x525)+(x528));
if( IKabs(((gconst96)*(((((IkReal(-1.00000000000000))*(x536)))+(((IkReal(-1.00000000000000))*(x534)))+(x535))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst96)*(((((IkReal(-1.00000000000000))*(x536)))+(((IkReal(-1.00000000000000))*(x535)))+(x534))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst96)*(((((IkReal(-1.00000000000000))*(x536)))+(((IkReal(-1.00000000000000))*(x534)))+(x535)))), ((gconst96)*(((((IkReal(-1.00000000000000))*(x536)))+(((IkReal(-1.00000000000000))*(x535)))+(x534)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x537=IKcos(j6);
IkReal x538=IKsin(j6);
IkReal x539=((py)*(sj4));
IkReal x540=((cj4)*(px));
IkReal x541=((px)*(sj4));
IkReal x542=((cj4)*(py));
IkReal x543=((IkReal(0.707106781186548))*(x537));
IkReal x544=((IkReal(0.0636396103067893))*(x538));
IkReal x545=((IkReal(0.0636396103067893))*(x537));
IkReal x546=((IkReal(0.707106781186548))*(x538));
IkReal x547=((x542)*(x543));
IkReal x548=((x539)*(x543));
IkReal x549=((x540)*(x543));
IkReal x550=((x541)*(x546));
IkReal x551=((x539)*(x546));
IkReal x552=((x540)*(x546));
IkReal x553=((x541)*(x543));
IkReal x554=((x542)*(x546));
IkReal x555=((x552)+(x551)+(x547));
IkReal x556=((x550)+(x548)+(x549));
evalcond[0]=((((IkReal(-1.00000000000000))*(x554)))+(((IkReal(-1.00000000000000))*(x555)))+(x553)+(x556));
evalcond[1]=((((IkReal(-1.00000000000000))*(x553)))+(((IkReal(-1.00000000000000))*(x554)))+(x556)+(x555));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x541)*(x544)))+(((IkReal(-1.00000000000000))*(x542)*(x545)))+(((x541)*(x545)))+(((IkReal(-1.00000000000000))*(x539)*(x545)))+(((IkReal(-1.00000000000000))*(x539)*(x544)))+(((x542)*(x544)))+(((IkReal(-1.00000000000000))*(x540)*(x544)))+(((IkReal(-1.00000000000000))*(x540)*(x545)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x557=((IkReal(785674201.318386))*(cj4));
IkReal x558=((IkReal(785674201.318386))*(sj4));
IkReal x559=((px)*(x558));
IkReal x560=((py)*(x558));
IkReal x561=((px)*(x557));
IkReal x562=((py)*(x557));
IkReal x563=((x560)+(x561));
if( IKabs(((((IkReal(-1.00000000000000))*(x559)))+(x562)+(((IkReal(-1.00000000000000))*(x563))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x559)+(((IkReal(-1.00000000000000))*(x562)))+(((IkReal(-1.00000000000000))*(x563))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x559)))+(x562)+(((IkReal(-1.00000000000000))*(x563)))))+IKsqr(((x559)+(((IkReal(-1.00000000000000))*(x562)))+(((IkReal(-1.00000000000000))*(x563)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x559)))+(x562)+(((IkReal(-1.00000000000000))*(x563)))), ((x559)+(((IkReal(-1.00000000000000))*(x562)))+(((IkReal(-1.00000000000000))*(x563)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x564=IKcos(j6);
IkReal x565=IKsin(j6);
IkReal x566=((py)*(sj4));
IkReal x567=((cj4)*(px));
IkReal x568=((px)*(sj4));
IkReal x569=((cj4)*(py));
IkReal x570=((IkReal(0.707106781186548))*(x569));
IkReal x571=((IkReal(0.707106781186548))*(x567));
IkReal x572=((IkReal(0.707106781186548))*(x566));
IkReal x573=((IkReal(0.707106781186548))*(x568));
IkReal x574=((IkReal(0.0636396103067893))*(x565));
IkReal x575=((IkReal(0.707106781186548))*(x564));
IkReal x576=((IkReal(0.707106781186548))*(x565));
IkReal x577=((IkReal(0.0636396103067893))*(x564));
IkReal x578=((x564)*(x570));
IkReal x579=((x564)*(x572));
IkReal x580=((x564)*(x571));
IkReal x581=((x565)*(x573));
IkReal x582=((x565)*(x572));
IkReal x583=((x565)*(x571));
IkReal x584=((x564)*(x573));
IkReal x585=((x565)*(x570));
IkReal x586=((x571)+(x572));
IkReal x587=((x578)+(x582)+(x583));
IkReal x588=((x579)+(x580)+(x581));
evalcond[0]=((((IkReal(9.00000000000000e-10))*(x565)))+(((IkReal(-1.00000000000000))*(x570)))+(x573)+(x586));
evalcond[1]=((((IkReal(9.00000000000000e-10))*(x564)))+(((IkReal(-1.00000000000000))*(x573)))+(x570)+(x586));
evalcond[2]=((x584)+(x588)+(((IkReal(-1.00000000000000))*(x587)))+(((IkReal(-1.00000000000000))*(x585))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(x587)+(x588)+(((IkReal(-1.00000000000000))*(x585)))+(((IkReal(-1.00000000000000))*(x584))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x569)*(x577)))+(((IkReal(-1.00000000000000))*(x566)*(x577)))+(((IkReal(-1.00000000000000))*(x566)*(x574)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x567)*(x574)))+(((IkReal(-1.00000000000000))*(x567)*(x577)))+(((x569)*(x574)))+(((x568)*(x577)))+(((IkReal(-1.00000000000000))*(x568)*(x574))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x589=((IkReal(0.707106781186548))*(sj4));
IkReal x590=((IkReal(0.707106781186548))*(cj4));
IkReal x591=((px)*(x589));
IkReal x592=((py)*(x590));
IkReal x593=((px)*(x590));
IkReal x594=((py)*(x589));
IkReal x595=((x594)+(x593));
if( IKabs(((gconst94)*(((((IkReal(-1.00000000000000))*(x591)))+(((IkReal(-1.00000000000000))*(x595)))+(x592))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst94)*(((((IkReal(-1.00000000000000))*(x592)))+(((IkReal(-1.00000000000000))*(x595)))+(x591))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst94)*(((((IkReal(-1.00000000000000))*(x591)))+(((IkReal(-1.00000000000000))*(x595)))+(x592)))), ((gconst94)*(((((IkReal(-1.00000000000000))*(x592)))+(((IkReal(-1.00000000000000))*(x595)))+(x591)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x596=IKsin(j6);
IkReal x597=IKcos(j6);
IkReal x598=((py)*(sj4));
IkReal x599=((cj4)*(px));
IkReal x600=((px)*(sj4));
IkReal x601=((IkReal(0.0450000000000000))*(cj7));
IkReal x602=((cj4)*(py));
IkReal x603=((IkReal(0.300000000000000))*(sj7));
IkReal x604=((IkReal(0.707106781186548))*(x602));
IkReal x605=((IkReal(0.707106781186548))*(x599));
IkReal x606=((IkReal(0.707106781186548))*(x598));
IkReal x607=((IkReal(0.707106781186548))*(x600));
IkReal x608=((IkReal(0.707106781186548))*(x597));
IkReal x609=((IkReal(0.0636396103067893))*(x596));
IkReal x610=((IkReal(0.707106781186548))*(x596));
IkReal x611=((IkReal(0.0636396103067893))*(x597));
IkReal x612=((x597)*(x604));
IkReal x613=((x597)*(x606));
IkReal x614=((x597)*(x605));
IkReal x615=((x596)*(x607));
IkReal x616=((x596)*(x606));
IkReal x617=((x596)*(x605));
IkReal x618=((x597)*(x607));
IkReal x619=((x596)*(x604));
IkReal x620=((x606)+(x605));
IkReal x621=((x616)+(x617)+(x612));
IkReal x622=((x614)+(x615)+(x613));
evalcond[0]=((((IkReal(-1.00000000000000))*(x596)*(x601)))+(((IkReal(0.0450000000000000))*(x596)))+(((x596)*(x603)))+(x607)+(x620)+(((IkReal(-1.00000000000000))*(x604))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x597)*(x601)))+(((IkReal(0.0450000000000000))*(x597)))+(((x597)*(x603)))+(x604)+(x620)+(((IkReal(-1.00000000000000))*(x607))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x621)))+(((IkReal(-1.00000000000000))*(x619)))+(x618)+(x622));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x619)))+(((IkReal(-1.00000000000000))*(x618)))+(x603)+(x621)+(x622)+(((IkReal(-1.00000000000000))*(x601))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x600)*(x611)))+(((IkReal(-1.00000000000000))*(x599)*(x611)))+(((IkReal(-1.00000000000000))*(x599)*(x609)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x598)*(x609)))+(((IkReal(-1.00000000000000))*(x598)*(x611)))+(((IkReal(-1.00000000000000))*(x600)*(x609)))+(((x602)*(x609)))+(((IkReal(-1.00000000000000))*(x602)*(x611))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x623=((IkReal(0.550000000000000))*(sj5));
IkReal x624=((IkReal(0.0190918830920368))*(sj7));
IkReal x625=((IkReal(0.00286378246380552))*(cj7));
IkReal x626=((IkReal(0.707106781186548))*(pp));
IkReal x627=((IkReal(0.777817459305202))*(cj5)*(pz));
IkReal x628=((x623)*((px)*(px)));
IkReal x629=((x623)*((py)*(py)));
IkReal x630=((((IkReal(-1.00000000000000))*(x629)))+(x628));
IkReal x631=((((IkReal(-1.00000000000000))*(x628)))+(((IkReal(-1.00000000000000))*(x629)))+(((IkReal(-1.10000000000000))*(px)*(py)*(sj5))));
if( IKabs(x631) < IKFAST_ATAN2_MAGTHRESH && IKabs(x630) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x632=((IkReal(1.00000000000000))*(IKatan2(x631, x630)));
if( ((((x631)*(x631))+((x630)*(x630)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x631)*(x631))+((x630)*(x630)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x631)*(x631))+((x630)*(x630))))))):(IkReal)1.0e30))*(((((py)*(x626)))+(((py)*(x625)))+(((IkReal(-1.00000000000000))*(px)*(x624)))+(((IkReal(-1.00000000000000))*(px)*(x627)))+(((px)*(x625)))+(((px)*(x626)))+(((IkReal(0.147396408538336))*(px)))+(((IkReal(0.147396408538336))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x627)))+(((IkReal(-1.00000000000000))*(py)*(x624))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x631)*(x631))+((x630)*(x630)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x631)*(x631))+((x630)*(x630))))))):(IkReal)1.0e30))*(((((py)*(x626)))+(((py)*(x625)))+(((IkReal(-1.00000000000000))*(px)*(x624)))+(((IkReal(-1.00000000000000))*(px)*(x627)))+(((px)*(x625)))+(((px)*(x626)))+(((IkReal(0.147396408538336))*(px)))+(((IkReal(0.147396408538336))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x627)))+(((IkReal(-1.00000000000000))*(py)*(x624))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x633=IKasin(((((IKabs(IKabs(IKsqrt((((x631)*(x631))+((x630)*(x630)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x631)*(x631))+((x630)*(x630))))))):(IkReal)1.0e30))*(((((py)*(x626)))+(((py)*(x625)))+(((IkReal(-1.00000000000000))*(px)*(x624)))+(((IkReal(-1.00000000000000))*(px)*(x627)))+(((px)*(x625)))+(((px)*(x626)))+(((IkReal(0.147396408538336))*(px)))+(((IkReal(0.147396408538336))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x627)))+(((IkReal(-1.00000000000000))*(py)*(x624)))))));
j4array[0]=((((IkReal(-1.00000000000000))*(x632)))+(((IkReal(-1.00000000000000))*(x633))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x632)))+(x633));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x634=IKsin(j4);
IkReal x635=IKcos(j4);
IkReal x636=(py)*(py);
IkReal x637=(px)*(px);
IkReal x638=(cj5)*(cj5);
IkReal x639=((IkReal(0.147396408538336))*(px));
IkReal x640=((IkReal(0.0190918830920368))*(sj7));
IkReal x641=((IkReal(0.00286378246380552))*(cj7));
IkReal x642=((cj5)*(py));
IkReal x643=((IkReal(0.707106781186548))*(pp));
IkReal x644=((IkReal(0.777817459305202))*(pz));
IkReal x645=((cj5)*(px));
IkReal x646=((IkReal(0.707106781186548))*(sj5));
IkReal x647=((py)*(x646));
IkReal x648=((px)*(x646));
IkReal x649=((IkReal(0.147396408538336))*(x642));
IkReal x650=((cj5)*(x639));
IkReal x651=((IkReal(1.10000000000000))*(px)*(sj5));
IkReal x652=((IkReal(0.550000000000000))*(sj5)*(x636));
IkReal x653=((IkReal(0.550000000000000))*(sj5)*(x637));
IkReal x654=((x640)*(x642));
IkReal x655=((x640)*(x645));
IkReal x656=((x641)*(x642));
IkReal x657=((x641)*(x645));
IkReal x658=((x642)*(x643));
IkReal x659=((x643)*(x645));
IkReal x660=((cj5)*(x652));
IkReal x661=((cj5)*(x653));
IkReal x662=((py)*(x638)*(x644));
IkReal x663=((px)*(x638)*(x644));
IkReal x664=((x642)*(x651));
IkReal x665=((((IkReal(-1.00000000000000))*(x661)))+(x660));
IkReal x666=((x655)+(x663));
IkReal x667=((x654)+(x662));
IkReal x668=((x661)+(x660));
IkReal x669=((x649)+(x656)+(x658));
IkReal x670=((x650)+(x657)+(x659));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((x634)*(((((IkReal(-1.00000000000000))*(x648)))+(x647)))))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz)))+(((x635)*(((x647)+(x648))))));
evalcond[1]=((((x635)*(((((IkReal(-1.00000000000000))*(x653)))+(x652)))))+(((IkReal(-1.00000000000000))*(px)*(x640)))+(((IkReal(-0.147396408538336))*(py)))+(((py)*(x640)))+(((x634)*(((x652)+(x653)+(((IkReal(-1.00000000000000))*(py)*(x651)))))))+(((px)*(x643)))+(((px)*(x641)))+(((IkReal(-1.00000000000000))*(x644)*(x645)))+(x639)+(((IkReal(-1.00000000000000))*(py)*(x641)))+(((IkReal(-1.00000000000000))*(py)*(x643)))+(((x642)*(x644))));
evalcond[2]=((((x635)*(x665)))+(((IkReal(-1.00000000000000))*(x666)))+(((IkReal(-1.00000000000000))*(x669)))+(x667)+(x670)+(((x634)*(((((IkReal(-1.00000000000000))*(x664)))+(x668))))));
evalcond[3]=((((x635)*(((x668)+(x664)))))+(((IkReal(-1.00000000000000))*(x669)))+(((x634)*(x665)))+(x667)+(x666)+(((IkReal(-1.00000000000000))*(x670))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x671=((IkReal(0.707106781186548))*(sj4));
IkReal x672=((cj5)*(px));
IkReal x673=((py)*(sj5));
IkReal x674=((IkReal(0.707106781186548))*(cj4));
IkReal x675=((px)*(sj5));
IkReal x676=((cj5)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((py)*(x671)))+(((IkReal(-1.00000000000000))*(py)*(x674)))+(((px)*(x674)))+(((px)*(x671))));
evalcond[3]=((IkReal(-0.850000000000000))+(((x673)*(x674)))+(((x671)*(x673)))+(((x674)*(x675)))+(((cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x671)*(x675))));
evalcond[4]=((((x671)*(x672)))+(((IkReal(-1.00000000000000))*(x671)*(x676)))+(((IkReal(-1.00000000000000))*(x672)*(x674)))+(((pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x674)*(x676))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x677=(cj5)*(cj5);
IkReal x678=(py)*(py);
IkReal x679=(cj4)*(cj4);
IkReal x680=(px)*(px);
IkReal x681=(sj4)*(sj4);
IkReal x682=((IkReal(0.0450000000000000))*(x679));
IkReal x683=((IkReal(0.0900000000000000))*(px)*(py));
IkReal x684=((IkReal(0.0900000000000000))*(cj4)*(sj4));
IkReal x685=((IkReal(0.0450000000000000))*(x681));
IkReal x686=((IkReal(0.127279220613579))*(cj5)*(pz)*(sj5));
IkReal x687=((x677)*(x678));
IkReal x688=((x677)*(x680));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x677)*(x681)*(x683)))+(((x678)*(x682)))+(((x678)*(x685)))+(((x682)*(x688)))+(((x682)*(x687)))+(((x685)*(x687)))+(((x685)*(x688)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x686)))+(((IkReal(-1.00000000000000))*(x679)*(x683)))+(((IkReal(-1.00000000000000))*(x678)*(x684)))+(((IkReal(-1.00000000000000))*(x684)*(x688)))+(((x681)*(x683)))+(((x680)*(x684)))+(((x680)*(x685)))+(((x680)*(x682)))+(((px)*(sj4)*(x686)))+(((x684)*(x687)))+(((x677)*(x679)*(x683)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x686)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x686)))));
IkReal x689=(cj5)*(cj5);
IkReal x690=(py)*(py);
IkReal x691=(sj4)*(sj4);
IkReal x692=(cj4)*(cj4);
IkReal x693=(px)*(px);
IkReal x694=((IkReal(2.00000000000000))*(px)*(py));
IkReal x695=((IkReal(2.00000000000000))*(cj4)*(sj4));
IkReal x696=((IkReal(2.82842712474619))*(cj5)*(pz)*(sj5));
IkReal x697=((x689)*(x690));
IkReal x698=((x689)*(x693));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj4)*(px)*(x696)))+(((x695)*(x697)))+(((x690)*(x691)))+(((x690)*(x692)))+(((px)*(sj4)*(x696)))+(((x691)*(x694)))+(((x691)*(x697)))+(((x691)*(x698)))+(((x691)*(x693)))+(((x692)*(x697)))+(((x692)*(x693)))+(((x692)*(x698)))+(((x693)*(x695)))+(((IkReal(-1.00000000000000))*(x692)*(x694)))+(((IkReal(-1.00000000000000))*(x689)*(x691)*(x694)))+(((IkReal(-1.00000000000000))*(x695)*(x698)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x696)))+(((IkReal(-1.00000000000000))*(x690)*(x695)))+(((x689)*(x692)*(x694)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x696)))+(((IkReal(2.00000000000000))*((pz)*(pz))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x699=(px)*(px);
IkReal x700=(sj4)*(sj4);
IkReal x701=(cj4)*(cj4);
IkReal x702=(py)*(py);
IkReal x703=(sj5)*(sj5);
IkReal x704=(cj5)*(cj5);
IkReal x705=((px)*(sj4));
IkReal x706=((IkReal(0.777817459305202))*(pz));
IkReal x707=((IkReal(0.550000000000000))*(sj5));
IkReal x708=((cj5)*(py));
IkReal x709=((IkReal(0.150260191002141))*(sj4));
IkReal x710=((cj4)*(px));
IkReal x711=((pz)*(sj5));
IkReal x712=((IkReal(1.10000000000000))*(sj5));
IkReal x713=((IkReal(0.707106781186548))*(pp));
IkReal x714=((IkReal(0.150260191002141))*(cj5));
IkReal x715=((cj4)*(py));
IkReal x716=((cj4)*(sj4));
IkReal x717=((py)*(sj4));
IkReal x718=((cj5)*(x702));
IkReal x719=((x713)*(x717));
IkReal x720=((x699)*(x701));
IkReal x721=((x699)*(x700));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x709)))+(((IkReal(2.20000000000000))*(sj5)*(x705)*(x715)))+(((IkReal(0.150260191002141))*(x715)))+(((IkReal(-1.00000000000000))*(cj4)*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x701)*(x702)*(x707)))+(((cj5)*(x706)*(x710)))+(((IkReal(-1.00000000000000))*(x710)*(x713)))+(((x707)*(x720)))+(((cj5)*(x705)*(x706)))+(((sj4)*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x719)))+(((x700)*(x702)*(x707)))+(((IkReal(-1.00000000000000))*(x705)*(x713)))+(((IkReal(-1.00000000000000))*(x707)*(x721)))+(((x713)*(x715)))+(((IkReal(-0.150260191002141))*(x710)))+(((IkReal(-0.150260191002141))*(x705))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x700)*(x707)*(x718)))+(((IkReal(-0.212500000000000))*(x711)))+(((IkReal(-1.00000000000000))*(x712)*(x716)*(x718)))+(((IkReal(-1.00000000000000))*(x701)*(x707)*(x718)))+(((cj5)*(x710)*(x713)))+(((cj5)*(x699)*(x712)*(x716)))+(((IkReal(-1.00000000000000))*(px)*(x701)*(x708)*(x712)))+(((px)*(x700)*(x708)*(x712)))+(((IkReal(-1.00000000000000))*(cj5)*(x707)*(x720)))+(((IkReal(-1.00000000000000))*(cj5)*(x707)*(x721)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x711)))+(((IkReal(0.150260191002141))*(cj4)*(x708)))+(((cj4)*(x708)*(x713)))+(((IkReal(-1.00000000000000))*(pp)*(x711)))+(((IkReal(-1.00000000000000))*(cj5)*(x705)*(x713)))+(((x710)*(x714)))+(((sj4)*(x708)*(x713)))+(((IkReal(-1.00000000000000))*(x705)*(x714)))+(((IkReal(-1.00000000000000))*(x704)*(x706)*(x717)))+(((IkReal(-1.00000000000000))*(x704)*(x706)*(x715)))+(((IkReal(-1.00000000000000))*(x704)*(x706)*(x710)))+(((x704)*(x705)*(x706)))+(((x703)*(x706)*(x715)))+(((x703)*(x706)*(x717)))+(((x703)*(x706)*(x710)))+(((x708)*(x709)))+(((IkReal(-1.00000000000000))*(x703)*(x705)*(x706))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x709)))+(((IkReal(2.20000000000000))*(sj5)*(x705)*(x715)))+(((IkReal(0.150260191002141))*(x715)))+(((IkReal(-1.00000000000000))*(cj4)*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x701)*(x702)*(x707)))+(((cj5)*(x706)*(x710)))+(((IkReal(-1.00000000000000))*(x710)*(x713)))+(((x707)*(x720)))+(((cj5)*(x705)*(x706)))+(((sj4)*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x719)))+(((x700)*(x702)*(x707)))+(((IkReal(-1.00000000000000))*(x705)*(x713)))+(((IkReal(-1.00000000000000))*(x707)*(x721)))+(((x713)*(x715)))+(((IkReal(-0.150260191002141))*(x710)))+(((IkReal(-0.150260191002141))*(x705)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x700)*(x707)*(x718)))+(((IkReal(-0.212500000000000))*(x711)))+(((IkReal(-1.00000000000000))*(x712)*(x716)*(x718)))+(((IkReal(-1.00000000000000))*(x701)*(x707)*(x718)))+(((cj5)*(x710)*(x713)))+(((cj5)*(x699)*(x712)*(x716)))+(((IkReal(-1.00000000000000))*(px)*(x701)*(x708)*(x712)))+(((px)*(x700)*(x708)*(x712)))+(((IkReal(-1.00000000000000))*(cj5)*(x707)*(x720)))+(((IkReal(-1.00000000000000))*(cj5)*(x707)*(x721)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x711)))+(((IkReal(0.150260191002141))*(cj4)*(x708)))+(((cj4)*(x708)*(x713)))+(((IkReal(-1.00000000000000))*(pp)*(x711)))+(((IkReal(-1.00000000000000))*(cj5)*(x705)*(x713)))+(((x710)*(x714)))+(((sj4)*(x708)*(x713)))+(((IkReal(-1.00000000000000))*(x705)*(x714)))+(((IkReal(-1.00000000000000))*(x704)*(x706)*(x717)))+(((IkReal(-1.00000000000000))*(x704)*(x706)*(x715)))+(((IkReal(-1.00000000000000))*(x704)*(x706)*(x710)))+(((x704)*(x705)*(x706)))+(((x703)*(x706)*(x715)))+(((x703)*(x706)*(x717)))+(((x703)*(x706)*(x710)))+(((x708)*(x709)))+(((IkReal(-1.00000000000000))*(x703)*(x705)*(x706)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x722=IKsin(j6);
IkReal x723=IKcos(j6);
IkReal x724=((py)*(sj4));
IkReal x725=((cj4)*(px));
IkReal x726=((pz)*(sj5));
IkReal x727=((px)*(sj4));
IkReal x728=((cj4)*(py));
IkReal x729=((IkReal(0.777817459305202))*(sj5));
IkReal x730=((IkReal(0.707106781186548))*(x723));
IkReal x731=((IkReal(0.707106781186548))*(x722));
IkReal x732=((IkReal(0.0636396103067893))*(x722));
IkReal x733=((IkReal(0.0636396103067893))*(cj5)*(x723));
evalcond[0]=((((cj5)*(x728)*(x731)))+(((x727)*(x730)))+(((IkReal(-1.00000000000000))*(cj5)*(x727)*(x731)))+(((IkReal(-1.00000000000000))*(x722)*(x726)))+(((cj5)*(x724)*(x731)))+(((cj5)*(x725)*(x731)))+(((x724)*(x730)))+(((x725)*(x730)))+(((IkReal(-1.00000000000000))*(x728)*(x730))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x725)*(x730)))+(((x727)*(x731)))+(((IkReal(-1.00000000000000))*(cj5)*(x728)*(x730)))+(((x723)*(x726)))+(((x724)*(x731)))+(((IkReal(-1.00000000000000))*(cj5)*(x724)*(x730)))+(((x725)*(x731)))+(((IkReal(-1.00000000000000))*(x728)*(x731)))+(((cj5)*(x727)*(x730))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x727)*(x729)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((x728)*(x729)))+(((IkReal(-1.00000000000000))*(x727)*(x733)))+(((IkReal(-1.00000000000000))*(x727)*(x732)))+(((x728)*(x733)))+(((x728)*(x732)))+(((IkReal(-1.00000000000000))*(x724)*(x732)))+(((IkReal(-1.00000000000000))*(x725)*(x732)))+(((IkReal(-1.00000000000000))*(pp)))+(((x724)*(x733)))+(((x725)*(x729)))+(((x725)*(x733)))+(((x724)*(x729)))+(((IkReal(-0.0900000000000000))*(x723)*(x726))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x734=((IkReal(0.707106781186548))*(sj4));
IkReal x735=((cj5)*(px));
IkReal x736=((py)*(sj5));
IkReal x737=((IkReal(0.707106781186548))*(cj4));
IkReal x738=((px)*(sj5));
IkReal x739=((cj5)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((px)*(x737)))+(((px)*(x734)))+(((py)*(x734)))+(((IkReal(-1.00000000000000))*(py)*(x737))));
evalcond[3]=((IkReal(-0.850000000000000))+(((x736)*(x737)))+(((x734)*(x736)))+(((cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x734)*(x738)))+(((x737)*(x738))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x735)*(x737)))+(((x734)*(x735)))+(((IkReal(-1.00000000000000))*(x734)*(x739)))+(((IkReal(-1.00000000000000))*(x737)*(x739)))+(((pz)*(sj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x740=(cj5)*(cj5);
IkReal x741=(py)*(py);
IkReal x742=(cj4)*(cj4);
IkReal x743=(px)*(px);
IkReal x744=(sj4)*(sj4);
IkReal x745=((IkReal(0.0450000000000000))*(x742));
IkReal x746=((IkReal(0.0900000000000000))*(px)*(py));
IkReal x747=((IkReal(0.0900000000000000))*(cj4)*(sj4));
IkReal x748=((IkReal(0.0450000000000000))*(x744));
IkReal x749=((IkReal(0.127279220613579))*(cj5)*(pz)*(sj5));
IkReal x750=((x740)*(x741));
IkReal x751=((x740)*(x743));
gconst2=IKsign(((((x745)*(x750)))+(((x745)*(x751)))+(((x747)*(x750)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x749)))+(((x744)*(x746)))+(((IkReal(-1.00000000000000))*(x742)*(x746)))+(((px)*(sj4)*(x749)))+(((x740)*(x742)*(x746)))+(((IkReal(-1.00000000000000))*(x740)*(x744)*(x746)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((x741)*(x748)))+(((x741)*(x745)))+(((IkReal(-1.00000000000000))*(x747)*(x751)))+(((x743)*(x747)))+(((x743)*(x745)))+(((x743)*(x748)))+(((IkReal(-1.00000000000000))*(x741)*(x747)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x749)))+(((x748)*(x750)))+(((x748)*(x751)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x749)))));
IkReal x752=(cj5)*(cj5);
IkReal x753=(py)*(py);
IkReal x754=(sj4)*(sj4);
IkReal x755=(cj4)*(cj4);
IkReal x756=(px)*(px);
IkReal x757=((IkReal(2.00000000000000))*(px)*(py));
IkReal x758=((IkReal(2.00000000000000))*(cj4)*(sj4));
IkReal x759=((IkReal(2.82842712474619))*(cj5)*(pz)*(sj5));
IkReal x760=((x752)*(x753));
IkReal x761=((x752)*(x756));
dummyeval[0]=((((x756)*(x758)))+(((IkReal(-1.00000000000000))*(x758)*(x761)))+(((px)*(sj4)*(x759)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x759)))+(((IkReal(-1.00000000000000))*(x752)*(x754)*(x757)))+(((IkReal(-1.00000000000000))*(x755)*(x757)))+(((IkReal(-1.00000000000000))*(x753)*(x758)))+(((x758)*(x760)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x759)))+(((x755)*(x756)))+(((x753)*(x755)))+(((x753)*(x754)))+(((x752)*(x755)*(x757)))+(((x755)*(x761)))+(((x755)*(x760)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x759)))+(((IkReal(2.00000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((x754)*(x761)))+(((x754)*(x760)))+(((x754)*(x757)))+(((x754)*(x756))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x762=(px)*(px);
IkReal x763=(sj4)*(sj4);
IkReal x764=(cj4)*(cj4);
IkReal x765=(py)*(py);
IkReal x766=(sj5)*(sj5);
IkReal x767=(cj5)*(cj5);
IkReal x768=((px)*(sj4));
IkReal x769=((IkReal(0.777817459305202))*(pz));
IkReal x770=((IkReal(0.550000000000000))*(sj5));
IkReal x771=((cj5)*(py));
IkReal x772=((IkReal(0.150260191002141))*(sj4));
IkReal x773=((cj4)*(px));
IkReal x774=((pz)*(sj5));
IkReal x775=((IkReal(1.10000000000000))*(sj5));
IkReal x776=((IkReal(0.707106781186548))*(pp));
IkReal x777=((IkReal(0.150260191002141))*(cj5));
IkReal x778=((cj4)*(py));
IkReal x779=((cj4)*(sj4));
IkReal x780=((py)*(sj4));
IkReal x781=((cj5)*(x765));
IkReal x782=((x776)*(x780));
IkReal x783=((x762)*(x764));
IkReal x784=((x762)*(x763));
if( IKabs(((gconst2)*(((((x776)*(x778)))+(((IkReal(-1.00000000000000))*(x768)*(x776)))+(((IkReal(-1.00000000000000))*(x770)*(x784)))+(((x770)*(x783)))+(((IkReal(-1.00000000000000))*(x782)))+(((IkReal(0.150260191002141))*(x778)))+(((IkReal(2.20000000000000))*(sj5)*(x768)*(x778)))+(((IkReal(-1.00000000000000))*(x764)*(x765)*(x770)))+(((IkReal(-1.00000000000000))*(py)*(x772)))+(((sj4)*(x769)*(x771)))+(((x763)*(x765)*(x770)))+(((IkReal(-0.150260191002141))*(x768)))+(((IkReal(-1.00000000000000))*(x773)*(x776)))+(((cj5)*(x769)*(x773)))+(((IkReal(-0.150260191002141))*(x773)))+(((IkReal(-1.00000000000000))*(cj4)*(x769)*(x771)))+(((cj5)*(x768)*(x769))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((cj5)*(x762)*(x775)*(x779)))+(((IkReal(-1.00000000000000))*(x768)*(x777)))+(((IkReal(-1.00000000000000))*(x764)*(x770)*(x781)))+(((IkReal(-1.00000000000000))*(cj5)*(x768)*(x776)))+(((IkReal(-1.00000000000000))*(x775)*(x779)*(x781)))+(((IkReal(0.150260191002141))*(cj4)*(x771)))+(((cj5)*(x773)*(x776)))+(((x771)*(x772)))+(((IkReal(-1.00000000000000))*(x767)*(x769)*(x780)))+(((IkReal(-1.00000000000000))*(x766)*(x768)*(x769)))+(((x767)*(x768)*(x769)))+(((IkReal(-1.00000000000000))*(cj5)*(x770)*(x784)))+(((IkReal(-1.00000000000000))*(cj5)*(x770)*(x783)))+(((x773)*(x777)))+(((x766)*(x769)*(x773)))+(((x766)*(x769)*(x778)))+(((IkReal(-1.00000000000000))*(x763)*(x770)*(x781)))+(((IkReal(-1.00000000000000))*(px)*(x764)*(x771)*(x775)))+(((IkReal(-1.00000000000000))*(x767)*(x769)*(x773)))+(((IkReal(-1.00000000000000))*(x767)*(x769)*(x778)))+(((sj4)*(x771)*(x776)))+(((x766)*(x769)*(x780)))+(((px)*(x763)*(x771)*(x775)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x774)))+(((IkReal(-0.212500000000000))*(x774)))+(((IkReal(-1.00000000000000))*(pp)*(x774)))+(((cj4)*(x771)*(x776))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((x776)*(x778)))+(((IkReal(-1.00000000000000))*(x768)*(x776)))+(((IkReal(-1.00000000000000))*(x770)*(x784)))+(((x770)*(x783)))+(((IkReal(-1.00000000000000))*(x782)))+(((IkReal(0.150260191002141))*(x778)))+(((IkReal(2.20000000000000))*(sj5)*(x768)*(x778)))+(((IkReal(-1.00000000000000))*(x764)*(x765)*(x770)))+(((IkReal(-1.00000000000000))*(py)*(x772)))+(((sj4)*(x769)*(x771)))+(((x763)*(x765)*(x770)))+(((IkReal(-0.150260191002141))*(x768)))+(((IkReal(-1.00000000000000))*(x773)*(x776)))+(((cj5)*(x769)*(x773)))+(((IkReal(-0.150260191002141))*(x773)))+(((IkReal(-1.00000000000000))*(cj4)*(x769)*(x771)))+(((cj5)*(x768)*(x769)))))), ((gconst2)*(((((cj5)*(x762)*(x775)*(x779)))+(((IkReal(-1.00000000000000))*(x768)*(x777)))+(((IkReal(-1.00000000000000))*(x764)*(x770)*(x781)))+(((IkReal(-1.00000000000000))*(cj5)*(x768)*(x776)))+(((IkReal(-1.00000000000000))*(x775)*(x779)*(x781)))+(((IkReal(0.150260191002141))*(cj4)*(x771)))+(((cj5)*(x773)*(x776)))+(((x771)*(x772)))+(((IkReal(-1.00000000000000))*(x767)*(x769)*(x780)))+(((IkReal(-1.00000000000000))*(x766)*(x768)*(x769)))+(((x767)*(x768)*(x769)))+(((IkReal(-1.00000000000000))*(cj5)*(x770)*(x784)))+(((IkReal(-1.00000000000000))*(cj5)*(x770)*(x783)))+(((x773)*(x777)))+(((x766)*(x769)*(x773)))+(((x766)*(x769)*(x778)))+(((IkReal(-1.00000000000000))*(x763)*(x770)*(x781)))+(((IkReal(-1.00000000000000))*(px)*(x764)*(x771)*(x775)))+(((IkReal(-1.00000000000000))*(x767)*(x769)*(x773)))+(((IkReal(-1.00000000000000))*(x767)*(x769)*(x778)))+(((sj4)*(x771)*(x776)))+(((x766)*(x769)*(x780)))+(((px)*(x763)*(x771)*(x775)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x774)))+(((IkReal(-0.212500000000000))*(x774)))+(((IkReal(-1.00000000000000))*(pp)*(x774)))+(((cj4)*(x771)*(x776)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x785=IKsin(j6);
IkReal x786=IKcos(j6);
IkReal x787=((py)*(sj4));
IkReal x788=((cj4)*(px));
IkReal x789=((pz)*(sj5));
IkReal x790=((px)*(sj4));
IkReal x791=((cj4)*(py));
IkReal x792=((IkReal(0.777817459305202))*(sj5));
IkReal x793=((IkReal(0.707106781186548))*(x786));
IkReal x794=((IkReal(0.707106781186548))*(x785));
IkReal x795=((IkReal(0.0636396103067893))*(x785));
IkReal x796=((IkReal(0.0636396103067893))*(cj5)*(x786));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x790)*(x794)))+(((IkReal(-1.00000000000000))*(x785)*(x789)))+(((x790)*(x793)))+(((x788)*(x793)))+(((cj5)*(x787)*(x794)))+(((x787)*(x793)))+(((IkReal(-1.00000000000000))*(x791)*(x793)))+(((cj5)*(x791)*(x794)))+(((cj5)*(x788)*(x794))));
evalcond[1]=((((x786)*(x789)))+(((IkReal(-1.00000000000000))*(cj5)*(x791)*(x793)))+(((x790)*(x794)))+(((IkReal(-1.00000000000000))*(cj5)*(x787)*(x793)))+(((x788)*(x794)))+(((x787)*(x794)))+(((IkReal(-1.00000000000000))*(x791)*(x794)))+(((cj5)*(x790)*(x793)))+(((IkReal(-1.00000000000000))*(cj5)*(x788)*(x793))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x788)*(x795)))+(((IkReal(-0.0900000000000000))*(x786)*(x789)))+(((x788)*(x792)))+(((x788)*(x796)))+(((x787)*(x792)))+(((x787)*(x796)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x787)*(x795)))+(((IkReal(-1.00000000000000))*(x790)*(x795)))+(((IkReal(-1.00000000000000))*(x790)*(x792)))+(((IkReal(-1.00000000000000))*(x790)*(x796)))+(((x791)*(x795)))+(((x791)*(x792)))+(((x791)*(x796))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x797=((IkReal(0.707106781186548))*(sj5));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((py)*(sj4)*(x797)))+(((cj5)*(pz)))+(((cj4)*(py)*(x797)))+(((cj4)*(px)*(x797)))+(((IkReal(-1.00000000000000))*(px)*(sj4)*(x797))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x798=((IkReal(785674201.318386))*(sj4));
IkReal x799=((cj5)*(py));
IkReal x800=((IkReal(785674201.318386))*(cj4));
IkReal x801=((cj5)*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(px)*(x800)))+(((IkReal(-1.00000000000000))*(px)*(x798)))+(((py)*(x800)))+(((IkReal(-1.00000000000000))*(py)*(x798))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x798)*(x801)))+(((x800)*(x801)))+(((x799)*(x800)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((x798)*(x799))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(px)*(x800)))+(((IkReal(-1.00000000000000))*(px)*(x798)))+(((py)*(x800)))+(((IkReal(-1.00000000000000))*(py)*(x798)))))+IKsqr(((((IkReal(-1.00000000000000))*(x798)*(x801)))+(((x800)*(x801)))+(((x799)*(x800)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((x798)*(x799)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(px)*(x800)))+(((IkReal(-1.00000000000000))*(px)*(x798)))+(((py)*(x800)))+(((IkReal(-1.00000000000000))*(py)*(x798)))), ((((IkReal(-1.00000000000000))*(x798)*(x801)))+(((x800)*(x801)))+(((x799)*(x800)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((x798)*(x799)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x802=IKcos(j6);
IkReal x803=IKsin(j6);
IkReal x804=((cj5)*(px));
IkReal x805=((IkReal(0.707106781186548))*(sj4));
IkReal x806=((pz)*(sj5));
IkReal x807=((IkReal(0.707106781186548))*(px));
IkReal x808=((IkReal(0.707106781186548))*(cj5));
IkReal x809=((IkReal(0.0636396103067893))*(sj4));
IkReal x810=((IkReal(0.707106781186548))*(cj4));
IkReal x811=((cj5)*(py));
IkReal x812=((cj4)*(py));
IkReal x813=((IkReal(0.777817459305202))*(sj5));
IkReal x814=((cj4)*(x802));
IkReal x815=((py)*(x802));
IkReal x816=((px)*(x803));
IkReal x817=((cj4)*(x803));
IkReal x818=((py)*(x803));
IkReal x819=((sj4)*(x813));
IkReal x820=((x803)*(x812));
evalcond[0]=((((cj4)*(x807)))+(((IkReal(-1.00000000000000))*(py)*(x810)))+(((px)*(x805)))+(((py)*(x805)))+(((IkReal(9.00000000000000e-10))*(x803))));
evalcond[1]=((((x804)*(x805)))+(((IkReal(-1.00000000000000))*(x808)*(x812)))+(((IkReal(-1.00000000000000))*(x804)*(x810)))+(x806)+(((IkReal(-1.00000000000000))*(x805)*(x811)))+(((IkReal(9.00000000000000e-10))*(x802))));
evalcond[2]=((((x803)*(x805)*(x811)))+(((IkReal(-1.00000000000000))*(x803)*(x806)))+(((IkReal(-1.00000000000000))*(x803)*(x804)*(x805)))+(((px)*(x802)*(x805)))+(((IkReal(-1.00000000000000))*(x810)*(x815)))+(((x807)*(x814)))+(((x808)*(x820)))+(((x805)*(x815)))+(((x803)*(x804)*(x810))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x810)*(x818)))+(((x807)*(x817)))+(((IkReal(-1.00000000000000))*(x802)*(x804)*(x810)))+(((IkReal(-1.00000000000000))*(x802)*(x805)*(x811)))+(((x802)*(x804)*(x805)))+(((x805)*(x816)))+(((x805)*(x818)))+(((x802)*(x806)))+(((IkReal(-1.00000000000000))*(x802)*(x808)*(x812))));
evalcond[4]=((IkReal(-0.212500000000000))+(((cj4)*(px)*(x813)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((py)*(x819)))+(((IkReal(0.0636396103067893))*(x804)*(x814)))+(((IkReal(-1.00000000000000))*(x802)*(x804)*(x809)))+(((x812)*(x813)))+(((IkReal(-1.00000000000000))*(x809)*(x818)))+(((IkReal(-1.00000000000000))*(x809)*(x816)))+(((IkReal(0.0636396103067893))*(x811)*(x814)))+(((IkReal(-0.0900000000000000))*(x802)*(x806)))+(((IkReal(0.0636396103067893))*(x820)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0636396103067893))*(cj4)*(x816)))+(((IkReal(-1.00000000000000))*(px)*(x819)))+(((x802)*(x809)*(x811))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x821=((IkReal(0.707106781186548))*(sj4));
IkReal x822=((cj5)*(px));
IkReal x823=((cj5)*(py));
IkReal x824=((IkReal(0.707106781186548))*(cj4));
if( IKabs(((gconst0)*(((((py)*(x824)))+(((IkReal(-1.00000000000000))*(py)*(x821)))+(((IkReal(-1.00000000000000))*(px)*(x824)))+(((IkReal(-1.00000000000000))*(px)*(x821))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((x821)*(x823)))+(((IkReal(-1.00000000000000))*(x821)*(x822)))+(((x822)*(x824)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)))+(((x823)*(x824))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(((((py)*(x824)))+(((IkReal(-1.00000000000000))*(py)*(x821)))+(((IkReal(-1.00000000000000))*(px)*(x824)))+(((IkReal(-1.00000000000000))*(px)*(x821)))))), ((gconst0)*(((((x821)*(x823)))+(((IkReal(-1.00000000000000))*(x821)*(x822)))+(((x822)*(x824)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)))+(((x823)*(x824)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x825=IKsin(j6);
IkReal x826=IKcos(j6);
IkReal x827=((IkReal(0.707106781186548))*(sj4));
IkReal x828=((cj5)*(px));
IkReal x829=((pz)*(sj5));
IkReal x830=((IkReal(0.707106781186548))*(px));
IkReal x831=((IkReal(0.707106781186548))*(cj5));
IkReal x832=((IkReal(0.300000000000000))*(sj7));
IkReal x833=((py)*(sj4));
IkReal x834=((IkReal(0.707106781186548))*(cj4));
IkReal x835=((IkReal(0.777817459305202))*(sj5));
IkReal x836=((IkReal(0.0636396103067893))*(sj4));
IkReal x837=((IkReal(0.0636396103067893))*(cj4));
IkReal x838=((cj5)*(py));
IkReal x839=((cj4)*(py));
IkReal x840=((py)*(x826));
IkReal x841=((cj4)*(x826));
IkReal x842=((px)*(x825));
IkReal x843=((IkReal(0.0450000000000000))*(x826));
IkReal x844=((cj4)*(x825));
IkReal x845=((IkReal(0.0450000000000000))*(x825));
IkReal x846=((x825)*(x839));
evalcond[0]=((((py)*(x827)))+(((IkReal(-1.00000000000000))*(py)*(x834)))+(((px)*(x827)))+(((cj4)*(x830)))+(((IkReal(-1.00000000000000))*(cj7)*(x845)))+(x845)+(((x825)*(x832))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x828)*(x834)))+(((IkReal(-1.00000000000000))*(x827)*(x838)))+(((x826)*(x832)))+(((IkReal(-1.00000000000000))*(cj7)*(x843)))+(x829)+(x843)+(((x827)*(x828)))+(((IkReal(-1.00000000000000))*(x831)*(x839))));
evalcond[2]=((((x825)*(x827)*(x838)))+(((IkReal(-1.00000000000000))*(x825)*(x827)*(x828)))+(((x825)*(x828)*(x834)))+(((x827)*(x840)))+(((IkReal(-1.00000000000000))*(x834)*(x840)))+(((x830)*(x841)))+(((IkReal(-1.00000000000000))*(x825)*(x829)))+(((px)*(x826)*(x827)))+(((x831)*(x846))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x826)*(x827)*(x838)))+(((x827)*(x842)))+(((IkReal(-1.00000000000000))*(x826)*(x831)*(x839)))+(((py)*(x825)*(x827)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((x830)*(x844)))+(((IkReal(-1.00000000000000))*(x826)*(x828)*(x834)))+(x832)+(((IkReal(-1.00000000000000))*(py)*(x825)*(x834)))+(((x826)*(x827)*(x828)))+(((x826)*(x829))));
evalcond[4]=((IkReal(-0.212500000000000))+(((cj4)*(px)*(x835)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(-0.0900000000000000))*(x826)*(x829)))+(((IkReal(-0.0636396103067893))*(x825)*(x833)))+(((IkReal(-1.00000000000000))*(x836)*(x842)))+(((x833)*(x835)))+(((x835)*(x839)))+(((IkReal(0.0636396103067893))*(cj5)*(x826)*(x833)))+(((x826)*(x828)*(x837)))+(((IkReal(-1.00000000000000))*(x837)*(x842)))+(((py)*(x825)*(x837)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x826)*(x828)*(x836)))+(((x826)*(x837)*(x838)))+(((IkReal(-1.00000000000000))*(px)*(sj4)*(x835))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x847=((IkReal(0.707106781186548))*(sj5));
IkReal x848=((py)*(x847));
IkReal x849=((px)*(x847));
IkReal x850=((((IkReal(-1.00000000000000))*(x849)))+(x848));
IkReal x851=((x849)+(x848));
if( IKabs(x851) < IKFAST_ATAN2_MAGTHRESH && IKabs(x850) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x852=((IkReal(1.00000000000000))*(IKatan2(x851, x850)));
if( ((((x851)*(x851))+((x850)*(x850)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x851)*(x851))+((x850)*(x850)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x851)*(x851))+((x850)*(x850))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x851)*(x851))+((x850)*(x850)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x851)*(x851))+((x850)*(x850))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x853=IKasin(((((IKabs(IKabs(IKsqrt((((x851)*(x851))+((x850)*(x850)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x851)*(x851))+((x850)*(x850))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz)))))));
j4array[0]=((((IkReal(-1.00000000000000))*(x852)))+(((IkReal(-1.00000000000000))*(x853))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x852)))+(x853));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[4];
IkReal x854=IKsin(j4);
IkReal x855=IKcos(j4);
IkReal x856=((IkReal(0.147396408538336))*(px));
IkReal x857=((IkReal(0.147396408538336))*(py));
IkReal x858=(py)*(py);
IkReal x859=(px)*(px);
IkReal x860=(cj5)*(cj5);
IkReal x861=((IkReal(0.550000000000000))*(sj5));
IkReal x862=((IkReal(0.0190918830920368))*(sj7));
IkReal x863=((IkReal(0.00286378246380552))*(cj7));
IkReal x864=((cj5)*(py));
IkReal x865=((IkReal(0.777817459305202))*(pz));
IkReal x866=((cj5)*(px));
IkReal x867=((IkReal(0.707106781186548))*(pp));
IkReal x868=((px)*(x863));
IkReal x869=((py)*(x862));
IkReal x870=((px)*(x867));
IkReal x871=((px)*(x862));
IkReal x872=((py)*(x863));
IkReal x873=((py)*(x867));
IkReal x874=((cj5)*(x857));
IkReal x875=((cj5)*(x856));
IkReal x876=((IkReal(1.10000000000000))*(px)*(sj5));
IkReal x877=((x859)*(x861));
IkReal x878=((x858)*(x861));
IkReal x879=((x864)*(x865));
IkReal x880=((py)*(x876));
IkReal x881=((x865)*(x866));
IkReal x882=((x862)*(x864));
IkReal x883=((x862)*(x866));
IkReal x884=((x863)*(x864));
IkReal x885=((x863)*(x866));
IkReal x886=((x864)*(x867));
IkReal x887=((x866)*(x867));
IkReal x888=((cj5)*(x878));
IkReal x889=((cj5)*(x877));
IkReal x890=((py)*(x860)*(x865));
IkReal x891=((px)*(x860)*(x865));
IkReal x892=((x864)*(x876));
IkReal x893=((x869)+(x879));
IkReal x894=((x871)+(x881));
IkReal x895=((x856)+(x868)+(x870));
IkReal x896=((x857)+(x872)+(x873));
IkReal x897=((x878)+(x877));
IkReal x898=((((IkReal(-1.00000000000000))*(x889)))+(x888));
IkReal x899=((x883)+(x891));
IkReal x900=((x882)+(x890));
IkReal x901=((x889)+(x888));
IkReal x902=((x874)+(x884)+(x886));
IkReal x903=((x875)+(x885)+(x887));
evalcond[0]=((((x855)*(((((IkReal(-1.00000000000000))*(x897)))+(((IkReal(-1.00000000000000))*(x880)))))))+(((IkReal(-1.00000000000000))*(x894)))+(((IkReal(-1.00000000000000))*(x893)))+(x896)+(x895)+(((x854)*(((x877)+(((IkReal(-1.00000000000000))*(x878))))))));
evalcond[1]=((((x854)*(((((IkReal(-1.00000000000000))*(x880)))+(x897)))))+(((x855)*(((x878)+(((IkReal(-1.00000000000000))*(x877)))))))+(((IkReal(-1.00000000000000))*(x894)))+(((IkReal(-1.00000000000000))*(x896)))+(x895)+(x893));
evalcond[2]=((((x854)*(((((IkReal(-1.00000000000000))*(x892)))+(x901)))))+(((IkReal(-1.00000000000000))*(x902)))+(((IkReal(-1.00000000000000))*(x899)))+(x900)+(x903)+(((x855)*(x898))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x903)))+(((IkReal(-1.00000000000000))*(x902)))+(((x854)*(x898)))+(((x855)*(((x892)+(x901)))))+(x899)+(x900));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x904=((IkReal(0.707106781186548))*(sj4));
IkReal x905=((cj5)*(px));
IkReal x906=((py)*(sj5));
IkReal x907=((IkReal(0.707106781186548))*(cj4));
IkReal x908=((px)*(sj5));
IkReal x909=((cj5)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((px)*(x904)))+(((px)*(x907)))+(((IkReal(-1.00000000000000))*(py)*(x907)))+(((py)*(x904))));
evalcond[3]=((IkReal(-0.850000000000000))+(((x907)*(x908)))+(((cj5)*(pz)))+(((x906)*(x907)))+(((IkReal(-1.00000000000000))*(x904)*(x908)))+(((x904)*(x906))));
evalcond[4]=((((pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x904)*(x909)))+(((IkReal(-1.00000000000000))*(x905)*(x907)))+(((IkReal(-1.00000000000000))*(x907)*(x909)))+(((x904)*(x905))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x910=(cj5)*(cj5);
IkReal x911=(py)*(py);
IkReal x912=(cj4)*(cj4);
IkReal x913=(px)*(px);
IkReal x914=(sj4)*(sj4);
IkReal x915=((IkReal(0.0450000000000000))*(x912));
IkReal x916=((IkReal(0.0900000000000000))*(px)*(py));
IkReal x917=((IkReal(0.0900000000000000))*(cj4)*(sj4));
IkReal x918=((IkReal(0.0450000000000000))*(x914));
IkReal x919=((IkReal(0.127279220613579))*(cj5)*(pz)*(sj5));
IkReal x920=((x910)*(x911));
IkReal x921=((x910)*(x913));
gconst1=IKsign(((((x918)*(x921)))+(((x918)*(x920)))+(((IkReal(-1.00000000000000))*(x910)*(x914)*(x916)))+(((x915)*(x921)))+(((x915)*(x920)))+(((px)*(sj4)*(x919)))+(((x914)*(x916)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x919)))+(((x911)*(x915)))+(((x911)*(x918)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x919)))+(((x917)*(x920)))+(((IkReal(-1.00000000000000))*(x912)*(x916)))+(((x910)*(x912)*(x916)))+(((x913)*(x917)))+(((x913)*(x915)))+(((x913)*(x918)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x919)))+(((IkReal(-1.00000000000000))*(x911)*(x917)))+(((IkReal(-1.00000000000000))*(x917)*(x921)))));
IkReal x922=(cj5)*(cj5);
IkReal x923=(py)*(py);
IkReal x924=(sj4)*(sj4);
IkReal x925=(cj4)*(cj4);
IkReal x926=(px)*(px);
IkReal x927=((IkReal(2.00000000000000))*(px)*(py));
IkReal x928=((IkReal(2.00000000000000))*(cj4)*(sj4));
IkReal x929=((IkReal(2.82842712474619))*(cj5)*(pz)*(sj5));
IkReal x930=((x922)*(x923));
IkReal x931=((x922)*(x926));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x925)*(x927)))+(((x924)*(x930)))+(((x924)*(x931)))+(((x924)*(x926)))+(((x924)*(x927)))+(((x922)*(x925)*(x927)))+(((IkReal(-1.00000000000000))*(x922)*(x924)*(x927)))+(((IkReal(-1.00000000000000))*(x928)*(x931)))+(((x928)*(x930)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x929)))+(((px)*(sj4)*(x929)))+(((x923)*(x925)))+(((x923)*(x924)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x929)))+(((x925)*(x931)))+(((x925)*(x930)))+(((x925)*(x926)))+(((x926)*(x928)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x929)))+(((IkReal(-1.00000000000000))*(x923)*(x928)))+(((IkReal(2.00000000000000))*((pz)*(pz))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x932=(px)*(px);
IkReal x933=(sj4)*(sj4);
IkReal x934=(cj4)*(cj4);
IkReal x935=(py)*(py);
IkReal x936=(sj5)*(sj5);
IkReal x937=(cj5)*(cj5);
IkReal x938=((px)*(sj4));
IkReal x939=((IkReal(0.777817459305202))*(pz));
IkReal x940=((IkReal(0.550000000000000))*(sj5));
IkReal x941=((cj5)*(py));
IkReal x942=((IkReal(0.150260191002141))*(sj4));
IkReal x943=((cj4)*(px));
IkReal x944=((pz)*(sj5));
IkReal x945=((IkReal(1.10000000000000))*(sj5));
IkReal x946=((IkReal(0.707106781186548))*(pp));
IkReal x947=((IkReal(0.150260191002141))*(cj5));
IkReal x948=((cj4)*(py));
IkReal x949=((cj4)*(sj4));
IkReal x950=((py)*(sj4));
IkReal x951=((cj5)*(x935));
IkReal x952=((x946)*(x950));
IkReal x953=((x932)*(x934));
IkReal x954=((x932)*(x933));
if( IKabs(((gconst1)*(((((cj5)*(x939)*(x943)))+(((IkReal(2.20000000000000))*(sj5)*(x938)*(x948)))+(((IkReal(-1.00000000000000))*(py)*(x942)))+(((x946)*(x948)))+(((x940)*(x953)))+(((IkReal(-1.00000000000000))*(x940)*(x954)))+(((IkReal(-1.00000000000000))*(x938)*(x946)))+(((IkReal(-0.150260191002141))*(x943)))+(((IkReal(-1.00000000000000))*(cj4)*(x939)*(x941)))+(((IkReal(-1.00000000000000))*(x943)*(x946)))+(((IkReal(-0.150260191002141))*(x938)))+(((cj5)*(x938)*(x939)))+(((x933)*(x935)*(x940)))+(((IkReal(-1.00000000000000))*(x952)))+(((IkReal(-1.00000000000000))*(x934)*(x935)*(x940)))+(((IkReal(0.150260191002141))*(x948)))+(((sj4)*(x939)*(x941))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(cj5)*(x938)*(x946)))+(((IkReal(-1.00000000000000))*(x936)*(x938)*(x939)))+(((IkReal(-1.00000000000000))*(x937)*(x939)*(x950)))+(((IkReal(-1.00000000000000))*(px)*(x934)*(x941)*(x945)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x944)))+(((IkReal(0.150260191002141))*(cj4)*(x941)))+(((IkReal(-1.00000000000000))*(x938)*(x947)))+(((x936)*(x939)*(x943)))+(((x936)*(x939)*(x948)))+(((x941)*(x942)))+(((x943)*(x947)))+(((IkReal(-1.00000000000000))*(x937)*(x939)*(x943)))+(((IkReal(-1.00000000000000))*(x937)*(x939)*(x948)))+(((cj5)*(x943)*(x946)))+(((sj4)*(x941)*(x946)))+(((IkReal(-0.212500000000000))*(x944)))+(((cj4)*(x941)*(x946)))+(((x937)*(x938)*(x939)))+(((IkReal(-1.00000000000000))*(pp)*(x944)))+(((IkReal(-1.00000000000000))*(x933)*(x940)*(x951)))+(((IkReal(-1.00000000000000))*(x934)*(x940)*(x951)))+(((cj5)*(x932)*(x945)*(x949)))+(((x936)*(x939)*(x950)))+(((px)*(x933)*(x941)*(x945)))+(((IkReal(-1.00000000000000))*(cj5)*(x940)*(x954)))+(((IkReal(-1.00000000000000))*(cj5)*(x940)*(x953)))+(((IkReal(-1.00000000000000))*(x945)*(x949)*(x951))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((cj5)*(x939)*(x943)))+(((IkReal(2.20000000000000))*(sj5)*(x938)*(x948)))+(((IkReal(-1.00000000000000))*(py)*(x942)))+(((x946)*(x948)))+(((x940)*(x953)))+(((IkReal(-1.00000000000000))*(x940)*(x954)))+(((IkReal(-1.00000000000000))*(x938)*(x946)))+(((IkReal(-0.150260191002141))*(x943)))+(((IkReal(-1.00000000000000))*(cj4)*(x939)*(x941)))+(((IkReal(-1.00000000000000))*(x943)*(x946)))+(((IkReal(-0.150260191002141))*(x938)))+(((cj5)*(x938)*(x939)))+(((x933)*(x935)*(x940)))+(((IkReal(-1.00000000000000))*(x952)))+(((IkReal(-1.00000000000000))*(x934)*(x935)*(x940)))+(((IkReal(0.150260191002141))*(x948)))+(((sj4)*(x939)*(x941)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(cj5)*(x938)*(x946)))+(((IkReal(-1.00000000000000))*(x936)*(x938)*(x939)))+(((IkReal(-1.00000000000000))*(x937)*(x939)*(x950)))+(((IkReal(-1.00000000000000))*(px)*(x934)*(x941)*(x945)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x944)))+(((IkReal(0.150260191002141))*(cj4)*(x941)))+(((IkReal(-1.00000000000000))*(x938)*(x947)))+(((x936)*(x939)*(x943)))+(((x936)*(x939)*(x948)))+(((x941)*(x942)))+(((x943)*(x947)))+(((IkReal(-1.00000000000000))*(x937)*(x939)*(x943)))+(((IkReal(-1.00000000000000))*(x937)*(x939)*(x948)))+(((cj5)*(x943)*(x946)))+(((sj4)*(x941)*(x946)))+(((IkReal(-0.212500000000000))*(x944)))+(((cj4)*(x941)*(x946)))+(((x937)*(x938)*(x939)))+(((IkReal(-1.00000000000000))*(pp)*(x944)))+(((IkReal(-1.00000000000000))*(x933)*(x940)*(x951)))+(((IkReal(-1.00000000000000))*(x934)*(x940)*(x951)))+(((cj5)*(x932)*(x945)*(x949)))+(((x936)*(x939)*(x950)))+(((px)*(x933)*(x941)*(x945)))+(((IkReal(-1.00000000000000))*(cj5)*(x940)*(x954)))+(((IkReal(-1.00000000000000))*(cj5)*(x940)*(x953)))+(((IkReal(-1.00000000000000))*(x945)*(x949)*(x951)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x955=IKsin(j6);
IkReal x956=IKcos(j6);
IkReal x957=((py)*(sj4));
IkReal x958=((cj4)*(px));
IkReal x959=((pz)*(sj5));
IkReal x960=((px)*(sj4));
IkReal x961=((cj4)*(py));
IkReal x962=((IkReal(0.777817459305202))*(sj5));
IkReal x963=((IkReal(0.707106781186548))*(x956));
IkReal x964=((IkReal(0.707106781186548))*(x955));
IkReal x965=((IkReal(0.0636396103067893))*(x955));
IkReal x966=((IkReal(0.0636396103067893))*(cj5)*(x956));
evalcond[0]=((((IkReal(-1.00000000000000))*(x961)*(x963)))+(((x960)*(x963)))+(((x957)*(x963)))+(((IkReal(-1.00000000000000))*(cj5)*(x960)*(x964)))+(((cj5)*(x958)*(x964)))+(((cj5)*(x961)*(x964)))+(((x958)*(x963)))+(((IkReal(-1.00000000000000))*(x955)*(x959)))+(((cj5)*(x957)*(x964))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x958)*(x963)))+(((IkReal(-1.00000000000000))*(x961)*(x964)))+(((IkReal(-1.00000000000000))*(cj5)*(x957)*(x963)))+(((x960)*(x964)))+(((IkReal(-1.00000000000000))*(cj5)*(x961)*(x963)))+(((x956)*(x959)))+(((x957)*(x964)))+(((x958)*(x964)))+(((cj5)*(x960)*(x963))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-0.0900000000000000))*(x956)*(x959)))+(((IkReal(-1.00000000000000))*(x960)*(x962)))+(((IkReal(-1.00000000000000))*(x960)*(x966)))+(((IkReal(-1.00000000000000))*(x960)*(x965)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((x961)*(x966)))+(((x961)*(x965)))+(((x961)*(x962)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x958)*(x965)))+(((x957)*(x966)))+(((x957)*(x962)))+(((IkReal(-1.00000000000000))*(x957)*(x965)))+(((x958)*(x962)))+(((x958)*(x966))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x967=((IkReal(0.707106781186548))*(sj4));
IkReal x968=((cj5)*(px));
IkReal x969=((py)*(sj5));
IkReal x970=((IkReal(0.707106781186548))*(cj4));
IkReal x971=((px)*(sj5));
IkReal x972=((cj5)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((px)*(x967)))+(((IkReal(-1.00000000000000))*(py)*(x970)))+(((py)*(x967)))+(((px)*(x970))));
evalcond[3]=((IkReal(-0.850000000000000))+(((x969)*(x970)))+(((cj5)*(pz)))+(((x967)*(x969)))+(((IkReal(-1.00000000000000))*(x967)*(x971)))+(((x970)*(x971))));
evalcond[4]=((((x967)*(x968)))+(((IkReal(-1.00000000000000))*(x967)*(x972)))+(((IkReal(-1.00000000000000))*(x970)*(x972)))+(((pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x968)*(x970))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x973=(cj5)*(cj5);
IkReal x974=(py)*(py);
IkReal x975=(cj4)*(cj4);
IkReal x976=(px)*(px);
IkReal x977=(sj4)*(sj4);
IkReal x978=((IkReal(0.0450000000000000))*(x975));
IkReal x979=((IkReal(0.0900000000000000))*(px)*(py));
IkReal x980=((IkReal(0.0900000000000000))*(cj4)*(sj4));
IkReal x981=((IkReal(0.0450000000000000))*(x977));
IkReal x982=((IkReal(0.127279220613579))*(cj5)*(pz)*(sj5));
IkReal x983=((x973)*(x974));
IkReal x984=((x973)*(x976));
gconst2=IKsign(((((x981)*(x983)))+(((x981)*(x984)))+(((x977)*(x979)))+(((IkReal(-1.00000000000000))*(x973)*(x977)*(x979)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x982)))+(((x976)*(x980)))+(((x976)*(x981)))+(((IkReal(-1.00000000000000))*(x974)*(x980)))+(((x973)*(x975)*(x979)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((x976)*(x978)))+(((px)*(sj4)*(x982)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x982)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x982)))+(((x974)*(x981)))+(((x974)*(x978)))+(((IkReal(-1.00000000000000))*(x980)*(x984)))+(((x980)*(x983)))+(((IkReal(-1.00000000000000))*(x975)*(x979)))+(((x978)*(x984)))+(((x978)*(x983)))));
IkReal x985=(cj5)*(cj5);
IkReal x986=(py)*(py);
IkReal x987=(sj4)*(sj4);
IkReal x988=(cj4)*(cj4);
IkReal x989=(px)*(px);
IkReal x990=((IkReal(2.00000000000000))*(px)*(py));
IkReal x991=((IkReal(2.00000000000000))*(cj4)*(sj4));
IkReal x992=((IkReal(2.82842712474619))*(cj5)*(pz)*(sj5));
IkReal x993=((x985)*(x986));
IkReal x994=((x985)*(x989));
dummyeval[0]=((((x989)*(x991)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x992)))+(((x991)*(x993)))+(((IkReal(-1.00000000000000))*(x985)*(x987)*(x990)))+(((x988)*(x993)))+(((x988)*(x994)))+(((x988)*(x989)))+(((IkReal(-1.00000000000000))*(x991)*(x994)))+(((x986)*(x988)))+(((x986)*(x987)))+(((x985)*(x988)*(x990)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x992)))+(((x987)*(x993)))+(((x987)*(x990)))+(((x987)*(x994)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x992)))+(((x987)*(x989)))+(((IkReal(-1.00000000000000))*(x986)*(x991)))+(((px)*(sj4)*(x992)))+(((IkReal(-1.00000000000000))*(x988)*(x990)))+(((IkReal(2.00000000000000))*((pz)*(pz))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x995=(px)*(px);
IkReal x996=(sj4)*(sj4);
IkReal x997=(cj4)*(cj4);
IkReal x998=(py)*(py);
IkReal x999=(sj5)*(sj5);
IkReal x1000=(cj5)*(cj5);
IkReal x1001=((px)*(sj4));
IkReal x1002=((IkReal(0.777817459305202))*(pz));
IkReal x1003=((IkReal(0.550000000000000))*(sj5));
IkReal x1004=((cj5)*(py));
IkReal x1005=((IkReal(0.150260191002141))*(sj4));
IkReal x1006=((cj4)*(px));
IkReal x1007=((pz)*(sj5));
IkReal x1008=((IkReal(1.10000000000000))*(sj5));
IkReal x1009=((IkReal(0.707106781186548))*(pp));
IkReal x1010=((IkReal(0.150260191002141))*(cj5));
IkReal x1011=((cj4)*(py));
IkReal x1012=((cj4)*(sj4));
IkReal x1013=((py)*(sj4));
IkReal x1014=((cj5)*(x998));
IkReal x1015=((x1009)*(x1013));
IkReal x1016=((x995)*(x997));
IkReal x1017=((x995)*(x996));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x1003)*(x997)*(x998)))+(((x1009)*(x1011)))+(((IkReal(-1.00000000000000))*(cj4)*(x1002)*(x1004)))+(((x1003)*(x1016)))+(((IkReal(-1.00000000000000))*(py)*(x1005)))+(((IkReal(-1.00000000000000))*(x1015)))+(((IkReal(2.20000000000000))*(sj5)*(x1001)*(x1011)))+(((sj4)*(x1002)*(x1004)))+(((x1003)*(x996)*(x998)))+(((IkReal(-1.00000000000000))*(x1006)*(x1009)))+(((IkReal(-0.150260191002141))*(x1006)))+(((IkReal(-0.150260191002141))*(x1001)))+(((cj5)*(x1002)*(x1006)))+(((IkReal(-1.00000000000000))*(x1003)*(x1017)))+(((IkReal(-1.00000000000000))*(x1001)*(x1009)))+(((cj5)*(x1001)*(x1002)))+(((IkReal(0.150260191002141))*(x1011))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(px)*(x1004)*(x1008)*(x997)))+(((IkReal(-0.212500000000000))*(x1007)))+(((x1002)*(x1006)*(x999)))+(((sj4)*(x1004)*(x1009)))+(((IkReal(-1.00000000000000))*(x1003)*(x1014)*(x996)))+(((IkReal(-1.00000000000000))*(x1003)*(x1014)*(x997)))+(((IkReal(-1.00000000000000))*(x1001)*(x1002)*(x999)))+(((IkReal(-1.00000000000000))*(pp)*(x1007)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)*(x1013)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)*(x1011)))+(((IkReal(-1.00000000000000))*(cj5)*(x1003)*(x1017)))+(((IkReal(-1.00000000000000))*(cj5)*(x1003)*(x1016)))+(((x1000)*(x1001)*(x1002)))+(((IkReal(-1.00000000000000))*(cj5)*(x1001)*(x1009)))+(((cj4)*(x1004)*(x1009)))+(((x1004)*(x1005)))+(((x1002)*(x1011)*(x999)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)*(x1006)))+(((IkReal(0.150260191002141))*(cj4)*(x1004)))+(((x1006)*(x1010)))+(((IkReal(-1.00000000000000))*(x1008)*(x1012)*(x1014)))+(((IkReal(-1.00000000000000))*(x1001)*(x1010)))+(((px)*(x1004)*(x1008)*(x996)))+(((cj5)*(x1006)*(x1009)))+(((cj5)*(x1008)*(x1012)*(x995)))+(((x1002)*(x1013)*(x999)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x1007))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(x1003)*(x997)*(x998)))+(((x1009)*(x1011)))+(((IkReal(-1.00000000000000))*(cj4)*(x1002)*(x1004)))+(((x1003)*(x1016)))+(((IkReal(-1.00000000000000))*(py)*(x1005)))+(((IkReal(-1.00000000000000))*(x1015)))+(((IkReal(2.20000000000000))*(sj5)*(x1001)*(x1011)))+(((sj4)*(x1002)*(x1004)))+(((x1003)*(x996)*(x998)))+(((IkReal(-1.00000000000000))*(x1006)*(x1009)))+(((IkReal(-0.150260191002141))*(x1006)))+(((IkReal(-0.150260191002141))*(x1001)))+(((cj5)*(x1002)*(x1006)))+(((IkReal(-1.00000000000000))*(x1003)*(x1017)))+(((IkReal(-1.00000000000000))*(x1001)*(x1009)))+(((cj5)*(x1001)*(x1002)))+(((IkReal(0.150260191002141))*(x1011)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(px)*(x1004)*(x1008)*(x997)))+(((IkReal(-0.212500000000000))*(x1007)))+(((x1002)*(x1006)*(x999)))+(((sj4)*(x1004)*(x1009)))+(((IkReal(-1.00000000000000))*(x1003)*(x1014)*(x996)))+(((IkReal(-1.00000000000000))*(x1003)*(x1014)*(x997)))+(((IkReal(-1.00000000000000))*(x1001)*(x1002)*(x999)))+(((IkReal(-1.00000000000000))*(pp)*(x1007)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)*(x1013)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)*(x1011)))+(((IkReal(-1.00000000000000))*(cj5)*(x1003)*(x1017)))+(((IkReal(-1.00000000000000))*(cj5)*(x1003)*(x1016)))+(((x1000)*(x1001)*(x1002)))+(((IkReal(-1.00000000000000))*(cj5)*(x1001)*(x1009)))+(((cj4)*(x1004)*(x1009)))+(((x1004)*(x1005)))+(((x1002)*(x1011)*(x999)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)*(x1006)))+(((IkReal(0.150260191002141))*(cj4)*(x1004)))+(((x1006)*(x1010)))+(((IkReal(-1.00000000000000))*(x1008)*(x1012)*(x1014)))+(((IkReal(-1.00000000000000))*(x1001)*(x1010)))+(((px)*(x1004)*(x1008)*(x996)))+(((cj5)*(x1006)*(x1009)))+(((cj5)*(x1008)*(x1012)*(x995)))+(((x1002)*(x1013)*(x999)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x1007)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x1018=IKsin(j6);
IkReal x1019=IKcos(j6);
IkReal x1020=((py)*(sj4));
IkReal x1021=((cj4)*(px));
IkReal x1022=((pz)*(sj5));
IkReal x1023=((px)*(sj4));
IkReal x1024=((cj4)*(py));
IkReal x1025=((IkReal(0.777817459305202))*(sj5));
IkReal x1026=((IkReal(0.707106781186548))*(x1019));
IkReal x1027=((IkReal(0.707106781186548))*(x1018));
IkReal x1028=((IkReal(0.0636396103067893))*(x1018));
IkReal x1029=((IkReal(0.0636396103067893))*(cj5)*(x1019));
evalcond[0]=((((cj5)*(x1020)*(x1027)))+(((IkReal(-1.00000000000000))*(cj5)*(x1023)*(x1027)))+(((cj5)*(x1021)*(x1027)))+(((x1023)*(x1026)))+(((IkReal(-1.00000000000000))*(x1018)*(x1022)))+(((x1021)*(x1026)))+(((x1020)*(x1026)))+(((cj5)*(x1024)*(x1027)))+(((IkReal(-1.00000000000000))*(x1024)*(x1026))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x1021)*(x1026)))+(((IkReal(-1.00000000000000))*(cj5)*(x1024)*(x1026)))+(((x1023)*(x1027)))+(((cj5)*(x1023)*(x1026)))+(((x1021)*(x1027)))+(((x1019)*(x1022)))+(((x1020)*(x1027)))+(((IkReal(-1.00000000000000))*(x1024)*(x1027)))+(((IkReal(-1.00000000000000))*(cj5)*(x1020)*(x1026))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-0.0900000000000000))*(x1019)*(x1022)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x1023)*(x1025)))+(((IkReal(-1.00000000000000))*(x1023)*(x1028)))+(((IkReal(-1.00000000000000))*(x1023)*(x1029)))+(((IkReal(-1.00000000000000))*(x1021)*(x1028)))+(((IkReal(-1.00000000000000))*(x1020)*(x1028)))+(((IkReal(-1.00000000000000))*(pp)))+(((x1021)*(x1025)))+(((x1021)*(x1029)))+(((x1020)*(x1025)))+(((x1020)*(x1029)))+(((x1024)*(x1025)))+(((x1024)*(x1029)))+(((x1024)*(x1028))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x1030=((IkReal(0.707106781186548))*(sj5));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((cj4)*(py)*(x1030)))+(((py)*(sj4)*(x1030)))+(((cj5)*(pz)))+(((cj4)*(px)*(x1030)))+(((IkReal(-1.00000000000000))*(px)*(sj4)*(x1030))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x1031=((IkReal(785674201.318386))*(sj4));
IkReal x1032=((cj5)*(py));
IkReal x1033=((IkReal(785674201.318386))*(cj4));
IkReal x1034=((cj5)*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(py)*(x1031)))+(((py)*(x1033)))+(((IkReal(-1.00000000000000))*(px)*(x1031)))+(((IkReal(-1.00000000000000))*(px)*(x1033))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1031)*(x1032)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x1031)*(x1034)))+(((x1033)*(x1034)))+(((x1032)*(x1033))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(py)*(x1031)))+(((py)*(x1033)))+(((IkReal(-1.00000000000000))*(px)*(x1031)))+(((IkReal(-1.00000000000000))*(px)*(x1033)))))+IKsqr(((((x1031)*(x1032)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x1031)*(x1034)))+(((x1033)*(x1034)))+(((x1032)*(x1033)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(py)*(x1031)))+(((py)*(x1033)))+(((IkReal(-1.00000000000000))*(px)*(x1031)))+(((IkReal(-1.00000000000000))*(px)*(x1033)))), ((((x1031)*(x1032)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x1031)*(x1034)))+(((x1033)*(x1034)))+(((x1032)*(x1033)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x1035=IKcos(j6);
IkReal x1036=IKsin(j6);
IkReal x1037=((cj5)*(px));
IkReal x1038=((IkReal(0.707106781186548))*(sj4));
IkReal x1039=((pz)*(sj5));
IkReal x1040=((IkReal(0.707106781186548))*(px));
IkReal x1041=((IkReal(0.707106781186548))*(cj5));
IkReal x1042=((IkReal(0.0636396103067893))*(sj4));
IkReal x1043=((IkReal(0.707106781186548))*(cj4));
IkReal x1044=((cj5)*(py));
IkReal x1045=((cj4)*(py));
IkReal x1046=((IkReal(0.777817459305202))*(sj5));
IkReal x1047=((cj4)*(x1035));
IkReal x1048=((py)*(x1035));
IkReal x1049=((px)*(x1036));
IkReal x1050=((cj4)*(x1036));
IkReal x1051=((py)*(x1036));
IkReal x1052=((sj4)*(x1046));
IkReal x1053=((x1036)*(x1045));
evalcond[0]=((((px)*(x1038)))+(((py)*(x1038)))+(((IkReal(-1.00000000000000))*(py)*(x1043)))+(((IkReal(9.00000000000000e-10))*(x1036)))+(((cj4)*(x1040))));
evalcond[1]=((x1039)+(((IkReal(-1.00000000000000))*(x1038)*(x1044)))+(((IkReal(-1.00000000000000))*(x1041)*(x1045)))+(((IkReal(-1.00000000000000))*(x1037)*(x1043)))+(((IkReal(9.00000000000000e-10))*(x1035)))+(((x1037)*(x1038))));
evalcond[2]=((((x1038)*(x1048)))+(((x1040)*(x1047)))+(((IkReal(-1.00000000000000))*(x1036)*(x1037)*(x1038)))+(((x1041)*(x1053)))+(((IkReal(-1.00000000000000))*(x1043)*(x1048)))+(((x1036)*(x1038)*(x1044)))+(((x1036)*(x1037)*(x1043)))+(((px)*(x1035)*(x1038)))+(((IkReal(-1.00000000000000))*(x1036)*(x1039))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((x1038)*(x1049)))+(((x1038)*(x1051)))+(((IkReal(-1.00000000000000))*(x1043)*(x1051)))+(((x1040)*(x1050)))+(((x1035)*(x1037)*(x1038)))+(((IkReal(-1.00000000000000))*(x1035)*(x1038)*(x1044)))+(((IkReal(-1.00000000000000))*(x1035)*(x1041)*(x1045)))+(((x1035)*(x1039)))+(((IkReal(-1.00000000000000))*(x1035)*(x1037)*(x1043))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(0.0636396103067893))*(x1037)*(x1047)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(0.0636396103067893))*(x1044)*(x1047)))+(((IkReal(0.0636396103067893))*(x1053)))+(((IkReal(-1.00000000000000))*(px)*(x1052)))+(((IkReal(-1.00000000000000))*(x1042)*(x1051)))+(((py)*(x1052)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x1042)*(x1049)))+(((IkReal(-0.0900000000000000))*(x1035)*(x1039)))+(((cj4)*(px)*(x1046)))+(((x1035)*(x1042)*(x1044)))+(((IkReal(-0.0636396103067893))*(cj4)*(x1049)))+(((x1045)*(x1046)))+(((IkReal(-1.00000000000000))*(x1035)*(x1037)*(x1042))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x1054=((IkReal(0.707106781186548))*(sj4));
IkReal x1055=((cj5)*(px));
IkReal x1056=((cj5)*(py));
IkReal x1057=((IkReal(0.707106781186548))*(cj4));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x1054)))+(((IkReal(-1.00000000000000))*(px)*(x1057)))+(((py)*(x1057)))+(((IkReal(-1.00000000000000))*(py)*(x1054))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((x1054)*(x1056)))+(((x1055)*(x1057)))+(((x1056)*(x1057)))+(((IkReal(-1.00000000000000))*(x1054)*(x1055)))+(((IkReal(-1.00000000000000))*(pz)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x1054)))+(((IkReal(-1.00000000000000))*(px)*(x1057)))+(((py)*(x1057)))+(((IkReal(-1.00000000000000))*(py)*(x1054)))))), ((gconst0)*(((((x1054)*(x1056)))+(((x1055)*(x1057)))+(((x1056)*(x1057)))+(((IkReal(-1.00000000000000))*(x1054)*(x1055)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x1058=IKsin(j6);
IkReal x1059=IKcos(j6);
IkReal x1060=((IkReal(0.707106781186548))*(sj4));
IkReal x1061=((cj5)*(px));
IkReal x1062=((pz)*(sj5));
IkReal x1063=((IkReal(0.707106781186548))*(px));
IkReal x1064=((IkReal(0.707106781186548))*(cj5));
IkReal x1065=((IkReal(0.300000000000000))*(sj7));
IkReal x1066=((py)*(sj4));
IkReal x1067=((IkReal(0.707106781186548))*(cj4));
IkReal x1068=((IkReal(0.777817459305202))*(sj5));
IkReal x1069=((IkReal(0.0636396103067893))*(sj4));
IkReal x1070=((IkReal(0.0636396103067893))*(cj4));
IkReal x1071=((cj5)*(py));
IkReal x1072=((cj4)*(py));
IkReal x1073=((py)*(x1059));
IkReal x1074=((cj4)*(x1059));
IkReal x1075=((px)*(x1058));
IkReal x1076=((IkReal(0.0450000000000000))*(x1059));
IkReal x1077=((cj4)*(x1058));
IkReal x1078=((IkReal(0.0450000000000000))*(x1058));
IkReal x1079=((x1058)*(x1072));
evalcond[0]=((x1078)+(((py)*(x1060)))+(((IkReal(-1.00000000000000))*(py)*(x1067)))+(((px)*(x1060)))+(((IkReal(-1.00000000000000))*(cj7)*(x1078)))+(((cj4)*(x1063)))+(((x1058)*(x1065))));
evalcond[1]=((x1076)+(x1062)+(((x1060)*(x1061)))+(((IkReal(-1.00000000000000))*(x1061)*(x1067)))+(((IkReal(-1.00000000000000))*(cj7)*(x1076)))+(((x1059)*(x1065)))+(((IkReal(-1.00000000000000))*(x1064)*(x1072)))+(((IkReal(-1.00000000000000))*(x1060)*(x1071))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1058)*(x1060)*(x1061)))+(((x1060)*(x1073)))+(((x1058)*(x1060)*(x1071)))+(((x1058)*(x1061)*(x1067)))+(((x1063)*(x1074)))+(((IkReal(-1.00000000000000))*(x1067)*(x1073)))+(((px)*(x1059)*(x1060)))+(((IkReal(-1.00000000000000))*(x1058)*(x1062)))+(((x1064)*(x1079))));
evalcond[3]=((IkReal(0.0450000000000000))+(x1065)+(((x1059)*(x1060)*(x1061)))+(((x1060)*(x1075)))+(((IkReal(-1.00000000000000))*(x1059)*(x1064)*(x1072)))+(((IkReal(-1.00000000000000))*(x1059)*(x1061)*(x1067)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((x1063)*(x1077)))+(((py)*(x1058)*(x1060)))+(((IkReal(-1.00000000000000))*(x1059)*(x1060)*(x1071)))+(((x1059)*(x1062)))+(((IkReal(-1.00000000000000))*(py)*(x1058)*(x1067))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x1068)*(x1072)))+(((IkReal(-0.0900000000000000))*(x1059)*(x1062)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x1070)*(x1075)))+(((x1066)*(x1068)))+(((x1059)*(x1061)*(x1070)))+(((IkReal(-1.00000000000000))*(x1059)*(x1061)*(x1069)))+(((IkReal(-1.00000000000000))*(pp)))+(((py)*(x1058)*(x1070)))+(((IkReal(0.0636396103067893))*(cj5)*(x1059)*(x1066)))+(((cj4)*(px)*(x1068)))+(((x1059)*(x1070)*(x1071)))+(((IkReal(-1.00000000000000))*(x1069)*(x1075)))+(((IkReal(-0.0636396103067893))*(x1058)*(x1066)))+(((IkReal(-1.00000000000000))*(px)*(sj4)*(x1068))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x1080=((IkReal(0.707106781186548))*(sj5));
IkReal x1081=((py)*(x1080));
IkReal x1082=((px)*(x1080));
IkReal x1083=((x1081)+(((IkReal(-1.00000000000000))*(x1082))));
IkReal x1084=((x1082)+(x1081));
if( IKabs(x1084) < IKFAST_ATAN2_MAGTHRESH && IKabs(x1083) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x1085=((IkReal(1.00000000000000))*(IKatan2(x1084, x1083)));
if( ((((x1083)*(x1083))+((x1084)*(x1084)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x1083)*(x1083))+((x1084)*(x1084)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x1083)*(x1083))+((x1084)*(x1084))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x1083)*(x1083))+((x1084)*(x1084)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x1083)*(x1083))+((x1084)*(x1084))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x1086=IKasin(((((IKabs(IKabs(IKsqrt((((x1083)*(x1083))+((x1084)*(x1084)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x1083)*(x1083))+((x1084)*(x1084))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(((cj5)*(pz)))))));
j4array[0]=((((IkReal(-1.00000000000000))*(x1086)))+(((IkReal(-1.00000000000000))*(x1085))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(x1086)+(((IkReal(-1.00000000000000))*(x1085))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.300000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x1087=((IkReal(0.707106781186548))*(sj4));
IkReal x1088=((cj5)*(px));
IkReal x1089=((py)*(sj5));
IkReal x1090=((IkReal(0.707106781186548))*(cj4));
IkReal x1091=((px)*(sj5));
IkReal x1092=((cj5)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(x1090)))+(((py)*(x1087)))+(((px)*(x1087)))+(((px)*(x1090))));
evalcond[3]=((IkReal(-0.850000000000000))+(((x1089)*(x1090)))+(((cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x1087)*(x1091)))+(((x1087)*(x1089)))+(((x1090)*(x1091))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x1088)*(x1090)))+(((IkReal(-1.00000000000000))*(x1090)*(x1092)))+(((IkReal(-1.00000000000000))*(x1087)*(x1092)))+(((x1087)*(x1088)))+(((pz)*(sj5))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x1093=(cj5)*(cj5);
IkReal x1094=(py)*(py);
IkReal x1095=(cj4)*(cj4);
IkReal x1096=(px)*(px);
IkReal x1097=(sj4)*(sj4);
IkReal x1098=((IkReal(0.0450000000000000))*(x1095));
IkReal x1099=((IkReal(0.0900000000000000))*(px)*(py));
IkReal x1100=((IkReal(0.0900000000000000))*(cj4)*(sj4));
IkReal x1101=((IkReal(0.0450000000000000))*(x1097));
IkReal x1102=((IkReal(0.127279220613579))*(cj5)*(pz)*(sj5));
IkReal x1103=((x1093)*(x1094));
IkReal x1104=((x1093)*(x1096));
gconst1=IKsign(((((x1094)*(x1101)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x1102)))+(((x1093)*(x1095)*(x1099)))+(((x1098)*(x1103)))+(((x1098)*(x1104)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x1102)))+(((px)*(sj4)*(x1102)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x1102)))+(((x1094)*(x1098)))+(((x1101)*(x1104)))+(((x1101)*(x1103)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*(x1095)*(x1099)))+(((IkReal(-1.00000000000000))*(x1094)*(x1100)))+(((IkReal(-1.00000000000000))*(x1100)*(x1104)))+(((x1096)*(x1100)))+(((x1096)*(x1101)))+(((x1100)*(x1103)))+(((IkReal(-1.00000000000000))*(x1093)*(x1097)*(x1099)))+(((x1096)*(x1098)))+(((x1097)*(x1099)))));
IkReal x1105=(cj5)*(cj5);
IkReal x1106=(py)*(py);
IkReal x1107=(sj4)*(sj4);
IkReal x1108=(cj4)*(cj4);
IkReal x1109=(px)*(px);
IkReal x1110=((IkReal(2.00000000000000))*(px)*(py));
IkReal x1111=((IkReal(2.00000000000000))*(cj4)*(sj4));
IkReal x1112=((IkReal(2.82842712474619))*(cj5)*(pz)*(sj5));
IkReal x1113=((x1105)*(x1106));
IkReal x1114=((x1105)*(x1109));
dummyeval[0]=((((x1109)*(x1111)))+(((x1108)*(x1109)))+(((x1108)*(x1113)))+(((x1108)*(x1114)))+(((x1111)*(x1113)))+(((x1105)*(x1108)*(x1110)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x1112)))+(((px)*(sj4)*(x1112)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x1112)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x1112)))+(((IkReal(-1.00000000000000))*(x1108)*(x1110)))+(((IkReal(-1.00000000000000))*(x1106)*(x1111)))+(((x1107)*(x1109)))+(((IkReal(-1.00000000000000))*(x1111)*(x1114)))+(((x1107)*(x1113)))+(((x1107)*(x1110)))+(((x1107)*(x1114)))+(((IkReal(2.00000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((x1106)*(x1108)))+(((x1106)*(x1107)))+(((IkReal(-1.00000000000000))*(x1105)*(x1107)*(x1110))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x1115=(px)*(px);
IkReal x1116=(sj4)*(sj4);
IkReal x1117=(cj4)*(cj4);
IkReal x1118=(py)*(py);
IkReal x1119=(sj5)*(sj5);
IkReal x1120=(cj5)*(cj5);
IkReal x1121=((px)*(sj4));
IkReal x1122=((IkReal(0.777817459305202))*(pz));
IkReal x1123=((IkReal(0.550000000000000))*(sj5));
IkReal x1124=((cj5)*(py));
IkReal x1125=((IkReal(0.150260191002141))*(sj4));
IkReal x1126=((cj4)*(px));
IkReal x1127=((pz)*(sj5));
IkReal x1128=((IkReal(1.10000000000000))*(sj5));
IkReal x1129=((IkReal(0.707106781186548))*(pp));
IkReal x1130=((IkReal(0.150260191002141))*(cj5));
IkReal x1131=((cj4)*(py));
IkReal x1132=((cj4)*(sj4));
IkReal x1133=((py)*(sj4));
IkReal x1134=((cj5)*(x1118));
IkReal x1135=((x1129)*(x1133));
IkReal x1136=((x1115)*(x1117));
IkReal x1137=((x1115)*(x1116));
if( IKabs(((gconst1)*(((((IkReal(2.20000000000000))*(sj5)*(x1121)*(x1131)))+(((IkReal(-0.150260191002141))*(x1121)))+(((IkReal(-0.150260191002141))*(x1126)))+(((x1129)*(x1131)))+(((IkReal(-1.00000000000000))*(x1121)*(x1129)))+(((IkReal(-1.00000000000000))*(x1126)*(x1129)))+(((IkReal(-1.00000000000000))*(x1123)*(x1137)))+(((cj5)*(x1121)*(x1122)))+(((IkReal(-1.00000000000000))*(py)*(x1125)))+(((IkReal(-1.00000000000000))*(x1135)))+(((IkReal(-1.00000000000000))*(cj4)*(x1122)*(x1124)))+(((x1123)*(x1136)))+(((cj5)*(x1122)*(x1126)))+(((IkReal(0.150260191002141))*(x1131)))+(((IkReal(-1.00000000000000))*(x1117)*(x1118)*(x1123)))+(((sj4)*(x1122)*(x1124)))+(((x1116)*(x1118)*(x1123))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((x1126)*(x1130)))+(((x1120)*(x1121)*(x1122)))+(((IkReal(-1.00000000000000))*(x1119)*(x1121)*(x1122)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x1127)))+(((IkReal(-0.212500000000000))*(x1127)))+(((IkReal(-1.00000000000000))*(cj5)*(x1123)*(x1136)))+(((IkReal(-1.00000000000000))*(cj5)*(x1123)*(x1137)))+(((IkReal(-1.00000000000000))*(x1121)*(x1130)))+(((IkReal(-1.00000000000000))*(x1117)*(x1123)*(x1134)))+(((x1119)*(x1122)*(x1126)))+(((IkReal(-1.00000000000000))*(cj5)*(x1121)*(x1129)))+(((x1119)*(x1122)*(x1133)))+(((x1119)*(x1122)*(x1131)))+(((x1124)*(x1125)))+(((cj4)*(x1124)*(x1129)))+(((sj4)*(x1124)*(x1129)))+(((cj5)*(x1126)*(x1129)))+(((IkReal(-1.00000000000000))*(x1128)*(x1132)*(x1134)))+(((IkReal(0.150260191002141))*(cj4)*(x1124)))+(((cj5)*(x1115)*(x1128)*(x1132)))+(((IkReal(-1.00000000000000))*(pp)*(x1127)))+(((px)*(x1116)*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(px)*(x1117)*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(x1116)*(x1123)*(x1134)))+(((IkReal(-1.00000000000000))*(x1120)*(x1122)*(x1133)))+(((IkReal(-1.00000000000000))*(x1120)*(x1122)*(x1131)))+(((IkReal(-1.00000000000000))*(x1120)*(x1122)*(x1126))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst1)*(((((IkReal(2.20000000000000))*(sj5)*(x1121)*(x1131)))+(((IkReal(-0.150260191002141))*(x1121)))+(((IkReal(-0.150260191002141))*(x1126)))+(((x1129)*(x1131)))+(((IkReal(-1.00000000000000))*(x1121)*(x1129)))+(((IkReal(-1.00000000000000))*(x1126)*(x1129)))+(((IkReal(-1.00000000000000))*(x1123)*(x1137)))+(((cj5)*(x1121)*(x1122)))+(((IkReal(-1.00000000000000))*(py)*(x1125)))+(((IkReal(-1.00000000000000))*(x1135)))+(((IkReal(-1.00000000000000))*(cj4)*(x1122)*(x1124)))+(((x1123)*(x1136)))+(((cj5)*(x1122)*(x1126)))+(((IkReal(0.150260191002141))*(x1131)))+(((IkReal(-1.00000000000000))*(x1117)*(x1118)*(x1123)))+(((sj4)*(x1122)*(x1124)))+(((x1116)*(x1118)*(x1123)))))), ((gconst1)*(((((x1126)*(x1130)))+(((x1120)*(x1121)*(x1122)))+(((IkReal(-1.00000000000000))*(x1119)*(x1121)*(x1122)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x1127)))+(((IkReal(-0.212500000000000))*(x1127)))+(((IkReal(-1.00000000000000))*(cj5)*(x1123)*(x1136)))+(((IkReal(-1.00000000000000))*(cj5)*(x1123)*(x1137)))+(((IkReal(-1.00000000000000))*(x1121)*(x1130)))+(((IkReal(-1.00000000000000))*(x1117)*(x1123)*(x1134)))+(((x1119)*(x1122)*(x1126)))+(((IkReal(-1.00000000000000))*(cj5)*(x1121)*(x1129)))+(((x1119)*(x1122)*(x1133)))+(((x1119)*(x1122)*(x1131)))+(((x1124)*(x1125)))+(((cj4)*(x1124)*(x1129)))+(((sj4)*(x1124)*(x1129)))+(((cj5)*(x1126)*(x1129)))+(((IkReal(-1.00000000000000))*(x1128)*(x1132)*(x1134)))+(((IkReal(0.150260191002141))*(cj4)*(x1124)))+(((cj5)*(x1115)*(x1128)*(x1132)))+(((IkReal(-1.00000000000000))*(pp)*(x1127)))+(((px)*(x1116)*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(px)*(x1117)*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(x1116)*(x1123)*(x1134)))+(((IkReal(-1.00000000000000))*(x1120)*(x1122)*(x1133)))+(((IkReal(-1.00000000000000))*(x1120)*(x1122)*(x1131)))+(((IkReal(-1.00000000000000))*(x1120)*(x1122)*(x1126)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x1138=IKsin(j6);
IkReal x1139=IKcos(j6);
IkReal x1140=((py)*(sj4));
IkReal x1141=((cj4)*(px));
IkReal x1142=((pz)*(sj5));
IkReal x1143=((px)*(sj4));
IkReal x1144=((cj4)*(py));
IkReal x1145=((IkReal(0.777817459305202))*(sj5));
IkReal x1146=((IkReal(0.707106781186548))*(x1139));
IkReal x1147=((IkReal(0.707106781186548))*(x1138));
IkReal x1148=((IkReal(0.0636396103067893))*(x1138));
IkReal x1149=((IkReal(0.0636396103067893))*(cj5)*(x1139));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1138)*(x1142)))+(((x1140)*(x1146)))+(((x1143)*(x1146)))+(((cj5)*(x1144)*(x1147)))+(((x1141)*(x1146)))+(((IkReal(-1.00000000000000))*(cj5)*(x1143)*(x1147)))+(((cj5)*(x1141)*(x1147)))+(((IkReal(-1.00000000000000))*(x1144)*(x1146)))+(((cj5)*(x1140)*(x1147))));
evalcond[1]=((((x1140)*(x1147)))+(((x1143)*(x1147)))+(((IkReal(-1.00000000000000))*(cj5)*(x1140)*(x1146)))+(((x1141)*(x1147)))+(((IkReal(-1.00000000000000))*(cj5)*(x1144)*(x1146)))+(((cj5)*(x1143)*(x1146)))+(((x1139)*(x1142)))+(((IkReal(-1.00000000000000))*(x1144)*(x1147)))+(((IkReal(-1.00000000000000))*(cj5)*(x1141)*(x1146))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x1140)*(x1148)))+(((x1140)*(x1149)))+(((x1140)*(x1145)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x1143)*(x1148)))+(((IkReal(-1.00000000000000))*(x1143)*(x1149)))+(((IkReal(-1.00000000000000))*(x1143)*(x1145)))+(((IkReal(-1.00000000000000))*(x1141)*(x1148)))+(((x1141)*(x1149)))+(((x1141)*(x1145)))+(((IkReal(-1.00000000000000))*(pp)))+(((x1144)*(x1149)))+(((x1144)*(x1148)))+(((x1144)*(x1145)))+(((IkReal(-0.0900000000000000))*(x1139)*(x1142))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x1150=((IkReal(0.707106781186548))*(sj4));
IkReal x1151=((cj5)*(px));
IkReal x1152=((py)*(sj5));
IkReal x1153=((IkReal(0.707106781186548))*(cj4));
IkReal x1154=((px)*(sj5));
IkReal x1155=((cj5)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(x1153)))+(((px)*(x1153)))+(((px)*(x1150)))+(((py)*(x1150))));
evalcond[3]=((IkReal(-0.850000000000000))+(((cj5)*(pz)))+(((x1152)*(x1153)))+(((x1153)*(x1154)))+(((x1150)*(x1152)))+(((IkReal(-1.00000000000000))*(x1150)*(x1154))));
evalcond[4]=((((x1150)*(x1151)))+(((pz)*(sj5)))+(((IkReal(-1.00000000000000))*(x1153)*(x1155)))+(((IkReal(-1.00000000000000))*(x1151)*(x1153)))+(((IkReal(-1.00000000000000))*(x1150)*(x1155))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x1156=(cj5)*(cj5);
IkReal x1157=(py)*(py);
IkReal x1158=(cj4)*(cj4);
IkReal x1159=(px)*(px);
IkReal x1160=(sj4)*(sj4);
IkReal x1161=((IkReal(0.0450000000000000))*(x1158));
IkReal x1162=((IkReal(0.0900000000000000))*(px)*(py));
IkReal x1163=((IkReal(0.0900000000000000))*(cj4)*(sj4));
IkReal x1164=((IkReal(0.0450000000000000))*(x1160));
IkReal x1165=((IkReal(0.127279220613579))*(cj5)*(pz)*(sj5));
IkReal x1166=((x1156)*(x1157));
IkReal x1167=((x1156)*(x1159));
gconst2=IKsign(((((px)*(sj4)*(x1165)))+(((x1159)*(x1161)))+(((x1159)*(x1164)))+(((x1159)*(x1163)))+(((x1164)*(x1167)))+(((x1164)*(x1166)))+(((IkReal(-1.00000000000000))*(cj4)*(py)*(x1165)))+(((x1163)*(x1166)))+(((IkReal(-1.00000000000000))*(x1158)*(x1162)))+(((IkReal(-1.00000000000000))*(x1163)*(x1167)))+(((x1156)*(x1158)*(x1162)))+(((IkReal(-1.00000000000000))*(x1156)*(x1160)*(x1162)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((x1157)*(x1161)))+(((x1157)*(x1164)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x1165)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x1165)))+(((IkReal(-1.00000000000000))*(x1157)*(x1163)))+(((x1160)*(x1162)))+(((x1161)*(x1166)))+(((x1161)*(x1167)))));
IkReal x1168=(cj5)*(cj5);
IkReal x1169=(py)*(py);
IkReal x1170=(sj4)*(sj4);
IkReal x1171=(cj4)*(cj4);
IkReal x1172=(px)*(px);
IkReal x1173=((IkReal(2.00000000000000))*(px)*(py));
IkReal x1174=((IkReal(2.00000000000000))*(cj4)*(sj4));
IkReal x1175=((IkReal(2.82842712474619))*(cj5)*(pz)*(sj5));
IkReal x1176=((x1168)*(x1169));
IkReal x1177=((x1168)*(x1172));
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj4)*(py)*(x1175)))+(((x1174)*(x1176)))+(((x1168)*(x1171)*(x1173)))+(((IkReal(-1.00000000000000))*(x1171)*(x1173)))+(((x1172)*(x1174)))+(((IkReal(-1.00000000000000))*(cj4)*(px)*(x1175)))+(((IkReal(-1.00000000000000))*(x1168)*(x1170)*(x1173)))+(((IkReal(-1.00000000000000))*(py)*(sj4)*(x1175)))+(((IkReal(-1.00000000000000))*(x1169)*(x1174)))+(((x1170)*(x1172)))+(((x1170)*(x1173)))+(((x1170)*(x1176)))+(((x1170)*(x1177)))+(((IkReal(2.00000000000000))*((pz)*(pz))*((sj5)*(sj5))))+(((px)*(sj4)*(x1175)))+(((x1169)*(x1171)))+(((x1169)*(x1170)))+(((IkReal(-1.00000000000000))*(x1174)*(x1177)))+(((x1171)*(x1177)))+(((x1171)*(x1176)))+(((x1171)*(x1172))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x1178=(px)*(px);
IkReal x1179=(sj4)*(sj4);
IkReal x1180=(cj4)*(cj4);
IkReal x1181=(py)*(py);
IkReal x1182=(sj5)*(sj5);
IkReal x1183=(cj5)*(cj5);
IkReal x1184=((px)*(sj4));
IkReal x1185=((IkReal(0.777817459305202))*(pz));
IkReal x1186=((IkReal(0.550000000000000))*(sj5));
IkReal x1187=((cj5)*(py));
IkReal x1188=((IkReal(0.150260191002141))*(sj4));
IkReal x1189=((cj4)*(px));
IkReal x1190=((pz)*(sj5));
IkReal x1191=((IkReal(1.10000000000000))*(sj5));
IkReal x1192=((IkReal(0.707106781186548))*(pp));
IkReal x1193=((IkReal(0.150260191002141))*(cj5));
IkReal x1194=((cj4)*(py));
IkReal x1195=((cj4)*(sj4));
IkReal x1196=((py)*(sj4));
IkReal x1197=((cj5)*(x1181));
IkReal x1198=((x1192)*(x1196));
IkReal x1199=((x1178)*(x1180));
IkReal x1200=((x1178)*(x1179));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x1189)*(x1192)))+(((x1186)*(x1199)))+(((IkReal(-0.150260191002141))*(x1189)))+(((IkReal(-0.150260191002141))*(x1184)))+(((cj5)*(x1185)*(x1189)))+(((IkReal(-1.00000000000000))*(py)*(x1188)))+(((IkReal(-1.00000000000000))*(x1198)))+(((IkReal(-1.00000000000000))*(x1184)*(x1192)))+(((IkReal(-1.00000000000000))*(cj4)*(x1185)*(x1187)))+(((IkReal(-1.00000000000000))*(x1180)*(x1181)*(x1186)))+(((IkReal(-1.00000000000000))*(x1186)*(x1200)))+(((x1192)*(x1194)))+(((cj5)*(x1184)*(x1185)))+(((IkReal(0.150260191002141))*(x1194)))+(((IkReal(2.20000000000000))*(sj5)*(x1184)*(x1194)))+(((sj4)*(x1185)*(x1187)))+(((x1179)*(x1181)*(x1186))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((cj4)*(x1187)*(x1192)))+(((x1187)*(x1188)))+(((x1182)*(x1185)*(x1189)))+(((x1183)*(x1184)*(x1185)))+(((IkReal(-1.00000000000000))*(px)*(x1180)*(x1187)*(x1191)))+(((x1182)*(x1185)*(x1196)))+(((x1182)*(x1185)*(x1194)))+(((px)*(x1179)*(x1187)*(x1191)))+(((IkReal(-0.212500000000000))*(x1190)))+(((IkReal(-1.00000000000000))*(cj5)*(x1186)*(x1200)))+(((cj5)*(x1189)*(x1192)))+(((IkReal(-1.00000000000000))*(cj5)*(x1186)*(x1199)))+(((x1189)*(x1193)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x1190)))+(((IkReal(-1.00000000000000))*(x1184)*(x1193)))+(((sj4)*(x1187)*(x1192)))+(((IkReal(-1.00000000000000))*(cj5)*(x1184)*(x1192)))+(((IkReal(-1.00000000000000))*(x1183)*(x1185)*(x1189)))+(((IkReal(0.150260191002141))*(cj4)*(x1187)))+(((cj5)*(x1178)*(x1191)*(x1195)))+(((IkReal(-1.00000000000000))*(x1179)*(x1186)*(x1197)))+(((IkReal(-1.00000000000000))*(x1183)*(x1185)*(x1194)))+(((IkReal(-1.00000000000000))*(x1183)*(x1185)*(x1196)))+(((IkReal(-1.00000000000000))*(x1191)*(x1195)*(x1197)))+(((IkReal(-1.00000000000000))*(x1180)*(x1186)*(x1197)))+(((IkReal(-1.00000000000000))*(pp)*(x1190)))+(((IkReal(-1.00000000000000))*(x1182)*(x1184)*(x1185))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(x1189)*(x1192)))+(((x1186)*(x1199)))+(((IkReal(-0.150260191002141))*(x1189)))+(((IkReal(-0.150260191002141))*(x1184)))+(((cj5)*(x1185)*(x1189)))+(((IkReal(-1.00000000000000))*(py)*(x1188)))+(((IkReal(-1.00000000000000))*(x1198)))+(((IkReal(-1.00000000000000))*(x1184)*(x1192)))+(((IkReal(-1.00000000000000))*(cj4)*(x1185)*(x1187)))+(((IkReal(-1.00000000000000))*(x1180)*(x1181)*(x1186)))+(((IkReal(-1.00000000000000))*(x1186)*(x1200)))+(((x1192)*(x1194)))+(((cj5)*(x1184)*(x1185)))+(((IkReal(0.150260191002141))*(x1194)))+(((IkReal(2.20000000000000))*(sj5)*(x1184)*(x1194)))+(((sj4)*(x1185)*(x1187)))+(((x1179)*(x1181)*(x1186)))))), ((gconst2)*(((((cj4)*(x1187)*(x1192)))+(((x1187)*(x1188)))+(((x1182)*(x1185)*(x1189)))+(((x1183)*(x1184)*(x1185)))+(((IkReal(-1.00000000000000))*(px)*(x1180)*(x1187)*(x1191)))+(((x1182)*(x1185)*(x1196)))+(((x1182)*(x1185)*(x1194)))+(((px)*(x1179)*(x1187)*(x1191)))+(((IkReal(-0.212500000000000))*(x1190)))+(((IkReal(-1.00000000000000))*(cj5)*(x1186)*(x1200)))+(((cj5)*(x1189)*(x1192)))+(((IkReal(-1.00000000000000))*(cj5)*(x1186)*(x1199)))+(((x1189)*(x1193)))+(((IkReal(1.10000000000000))*(cj5)*(pz)*(x1190)))+(((IkReal(-1.00000000000000))*(x1184)*(x1193)))+(((sj4)*(x1187)*(x1192)))+(((IkReal(-1.00000000000000))*(cj5)*(x1184)*(x1192)))+(((IkReal(-1.00000000000000))*(x1183)*(x1185)*(x1189)))+(((IkReal(0.150260191002141))*(cj4)*(x1187)))+(((cj5)*(x1178)*(x1191)*(x1195)))+(((IkReal(-1.00000000000000))*(x1179)*(x1186)*(x1197)))+(((IkReal(-1.00000000000000))*(x1183)*(x1185)*(x1194)))+(((IkReal(-1.00000000000000))*(x1183)*(x1185)*(x1196)))+(((IkReal(-1.00000000000000))*(x1191)*(x1195)*(x1197)))+(((IkReal(-1.00000000000000))*(x1180)*(x1186)*(x1197)))+(((IkReal(-1.00000000000000))*(pp)*(x1190)))+(((IkReal(-1.00000000000000))*(x1182)*(x1184)*(x1185)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[3];
IkReal x1201=IKsin(j6);
IkReal x1202=IKcos(j6);
IkReal x1203=((py)*(sj4));
IkReal x1204=((cj4)*(px));
IkReal x1205=((pz)*(sj5));
IkReal x1206=((px)*(sj4));
IkReal x1207=((cj4)*(py));
IkReal x1208=((IkReal(0.777817459305202))*(sj5));
IkReal x1209=((IkReal(0.707106781186548))*(x1202));
IkReal x1210=((IkReal(0.707106781186548))*(x1201));
IkReal x1211=((IkReal(0.0636396103067893))*(x1201));
IkReal x1212=((IkReal(0.0636396103067893))*(cj5)*(x1202));
evalcond[0]=((((x1204)*(x1209)))+(((cj5)*(x1204)*(x1210)))+(((x1203)*(x1209)))+(((cj5)*(x1207)*(x1210)))+(((IkReal(-1.00000000000000))*(x1201)*(x1205)))+(((IkReal(-1.00000000000000))*(cj5)*(x1206)*(x1210)))+(((x1206)*(x1209)))+(((cj5)*(x1203)*(x1210)))+(((IkReal(-1.00000000000000))*(x1207)*(x1209))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1207)*(x1210)))+(((IkReal(-1.00000000000000))*(cj5)*(x1204)*(x1209)))+(((IkReal(-1.00000000000000))*(cj5)*(x1207)*(x1209)))+(((x1204)*(x1210)))+(((x1202)*(x1205)))+(((x1206)*(x1210)))+(((cj5)*(x1206)*(x1209)))+(((x1203)*(x1210)))+(((IkReal(-1.00000000000000))*(cj5)*(x1203)*(x1209))));
evalcond[2]=((IkReal(-0.212500000000000))+(((x1204)*(x1208)))+(((IkReal(-0.0900000000000000))*(x1202)*(x1205)))+(((IkReal(-1.00000000000000))*(x1204)*(x1211)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((IkReal(-1.00000000000000))*(x1206)*(x1212)))+(((IkReal(-1.00000000000000))*(x1206)*(x1211)))+(((IkReal(-1.00000000000000))*(x1203)*(x1211)))+(((x1204)*(x1212)))+(((x1207)*(x1211)))+(((x1207)*(x1212)))+(((IkReal(-1.00000000000000))*(pp)))+(((x1203)*(x1208)))+(((x1203)*(x1212)))+(((IkReal(-1.00000000000000))*(x1206)*(x1208)))+(((x1207)*(x1208))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x1213=((IkReal(0.707106781186548))*(sj5));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((cj4)*(py)*(x1213)))+(((cj5)*(pz)))+(((py)*(sj4)*(x1213)))+(((IkReal(-1.00000000000000))*(px)*(sj4)*(x1213)))+(((cj4)*(px)*(x1213))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x1214=((IkReal(785674201.318386))*(sj4));
IkReal x1215=((cj5)*(py));
IkReal x1216=((IkReal(785674201.318386))*(cj4));
IkReal x1217=((cj5)*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(px)*(x1216)))+(((IkReal(-1.00000000000000))*(px)*(x1214)))+(((py)*(x1216)))+(((IkReal(-1.00000000000000))*(py)*(x1214))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1214)*(x1215)))+(((IkReal(-1.00000000000000))*(x1214)*(x1217)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((x1216)*(x1217)))+(((x1215)*(x1216))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(px)*(x1216)))+(((IkReal(-1.00000000000000))*(px)*(x1214)))+(((py)*(x1216)))+(((IkReal(-1.00000000000000))*(py)*(x1214)))))+IKsqr(((((x1214)*(x1215)))+(((IkReal(-1.00000000000000))*(x1214)*(x1217)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((x1216)*(x1217)))+(((x1215)*(x1216)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(px)*(x1216)))+(((IkReal(-1.00000000000000))*(px)*(x1214)))+(((py)*(x1216)))+(((IkReal(-1.00000000000000))*(py)*(x1214)))), ((((x1214)*(x1215)))+(((IkReal(-1.00000000000000))*(x1214)*(x1217)))+(((IkReal(-1111111111.11111))*(pz)*(sj5)))+(((x1216)*(x1217)))+(((x1215)*(x1216)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x1218=IKcos(j6);
IkReal x1219=IKsin(j6);
IkReal x1220=((cj5)*(px));
IkReal x1221=((IkReal(0.707106781186548))*(sj4));
IkReal x1222=((pz)*(sj5));
IkReal x1223=((IkReal(0.707106781186548))*(px));
IkReal x1224=((IkReal(0.707106781186548))*(cj5));
IkReal x1225=((IkReal(0.0636396103067893))*(sj4));
IkReal x1226=((IkReal(0.707106781186548))*(cj4));
IkReal x1227=((cj5)*(py));
IkReal x1228=((cj4)*(py));
IkReal x1229=((IkReal(0.777817459305202))*(sj5));
IkReal x1230=((cj4)*(x1218));
IkReal x1231=((py)*(x1218));
IkReal x1232=((px)*(x1219));
IkReal x1233=((cj4)*(x1219));
IkReal x1234=((py)*(x1219));
IkReal x1235=((sj4)*(x1229));
IkReal x1236=((x1219)*(x1228));
evalcond[0]=((((py)*(x1221)))+(((cj4)*(x1223)))+(((IkReal(-1.00000000000000))*(py)*(x1226)))+(((px)*(x1221)))+(((IkReal(9.00000000000000e-10))*(x1219))));
evalcond[1]=((x1222)+(((IkReal(-1.00000000000000))*(x1224)*(x1228)))+(((IkReal(-1.00000000000000))*(x1220)*(x1226)))+(((IkReal(-1.00000000000000))*(x1221)*(x1227)))+(((x1220)*(x1221)))+(((IkReal(9.00000000000000e-10))*(x1218))));
evalcond[2]=((((x1219)*(x1221)*(x1227)))+(((x1223)*(x1230)))+(((IkReal(-1.00000000000000))*(x1226)*(x1231)))+(((IkReal(-1.00000000000000))*(x1219)*(x1222)))+(((IkReal(-1.00000000000000))*(x1219)*(x1220)*(x1221)))+(((x1224)*(x1236)))+(((px)*(x1218)*(x1221)))+(((x1221)*(x1231)))+(((x1219)*(x1220)*(x1226))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x1218)*(x1221)*(x1227)))+(((x1223)*(x1233)))+(((IkReal(-1.00000000000000))*(x1226)*(x1234)))+(((IkReal(-1.00000000000000))*(x1218)*(x1220)*(x1226)))+(((IkReal(-1.00000000000000))*(x1218)*(x1224)*(x1228)))+(((x1218)*(x1220)*(x1221)))+(((x1221)*(x1232)))+(((x1221)*(x1234)))+(((x1218)*(x1222))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x1225)*(x1234)))+(((IkReal(-1.00000000000000))*(x1225)*(x1232)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((x1218)*(x1225)*(x1227)))+(((IkReal(-1.00000000000000))*(px)*(x1235)))+(((py)*(x1235)))+(((IkReal(0.0636396103067893))*(x1220)*(x1230)))+(((IkReal(-1.00000000000000))*(x1218)*(x1220)*(x1225)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0636396103067893))*(x1236)))+(((cj4)*(px)*(x1229)))+(((IkReal(0.0636396103067893))*(x1227)*(x1230)))+(((IkReal(-0.0900000000000000))*(x1218)*(x1222)))+(((x1228)*(x1229)))+(((IkReal(-0.0636396103067893))*(cj4)*(x1232))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x1237=((IkReal(0.707106781186548))*(sj4));
IkReal x1238=((cj5)*(px));
IkReal x1239=((cj5)*(py));
IkReal x1240=((IkReal(0.707106781186548))*(cj4));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x1240)))+(((IkReal(-1.00000000000000))*(px)*(x1237)))+(((py)*(x1240)))+(((IkReal(-1.00000000000000))*(py)*(x1237))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((x1239)*(x1240)))+(((IkReal(-1.00000000000000))*(x1237)*(x1238)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)))+(((x1238)*(x1240)))+(((x1237)*(x1239))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x1240)))+(((IkReal(-1.00000000000000))*(px)*(x1237)))+(((py)*(x1240)))+(((IkReal(-1.00000000000000))*(py)*(x1237)))))), ((gconst0)*(((((x1239)*(x1240)))+(((IkReal(-1.00000000000000))*(x1237)*(x1238)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)))+(((x1238)*(x1240)))+(((x1237)*(x1239)))))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[5];
IkReal x1241=IKsin(j6);
IkReal x1242=IKcos(j6);
IkReal x1243=((IkReal(0.707106781186548))*(sj4));
IkReal x1244=((cj5)*(px));
IkReal x1245=((pz)*(sj5));
IkReal x1246=((IkReal(0.707106781186548))*(px));
IkReal x1247=((IkReal(0.707106781186548))*(cj5));
IkReal x1248=((IkReal(0.300000000000000))*(sj7));
IkReal x1249=((py)*(sj4));
IkReal x1250=((IkReal(0.707106781186548))*(cj4));
IkReal x1251=((IkReal(0.777817459305202))*(sj5));
IkReal x1252=((IkReal(0.0636396103067893))*(sj4));
IkReal x1253=((IkReal(0.0636396103067893))*(cj4));
IkReal x1254=((cj5)*(py));
IkReal x1255=((cj4)*(py));
IkReal x1256=((py)*(x1242));
IkReal x1257=((cj4)*(x1242));
IkReal x1258=((px)*(x1241));
IkReal x1259=((IkReal(0.0450000000000000))*(x1242));
IkReal x1260=((cj4)*(x1241));
IkReal x1261=((IkReal(0.0450000000000000))*(x1241));
IkReal x1262=((x1241)*(x1255));
evalcond[0]=((x1261)+(((cj4)*(x1246)))+(((py)*(x1243)))+(((px)*(x1243)))+(((x1241)*(x1248)))+(((IkReal(-1.00000000000000))*(py)*(x1250)))+(((IkReal(-1.00000000000000))*(cj7)*(x1261))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj7)*(x1259)))+(x1245)+(x1259)+(((IkReal(-1.00000000000000))*(x1247)*(x1255)))+(((IkReal(-1.00000000000000))*(x1244)*(x1250)))+(((x1242)*(x1248)))+(((x1243)*(x1244)))+(((IkReal(-1.00000000000000))*(x1243)*(x1254))));
evalcond[2]=((((x1246)*(x1257)))+(((x1241)*(x1244)*(x1250)))+(((x1247)*(x1262)))+(((IkReal(-1.00000000000000))*(x1241)*(x1243)*(x1244)))+(((x1243)*(x1256)))+(((px)*(x1242)*(x1243)))+(((x1241)*(x1243)*(x1254)))+(((IkReal(-1.00000000000000))*(x1250)*(x1256)))+(((IkReal(-1.00000000000000))*(x1241)*(x1245))));
evalcond[3]=((IkReal(0.0450000000000000))+(x1248)+(((x1246)*(x1260)))+(((IkReal(-1.00000000000000))*(x1242)*(x1244)*(x1250)))+(((x1243)*(x1258)))+(((IkReal(-1.00000000000000))*(py)*(x1241)*(x1250)))+(((py)*(x1241)*(x1243)))+(((IkReal(-0.0450000000000000))*(cj7)))+(((x1242)*(x1245)))+(((IkReal(-1.00000000000000))*(x1242)*(x1243)*(x1254)))+(((IkReal(-1.00000000000000))*(x1242)*(x1247)*(x1255)))+(((x1242)*(x1243)*(x1244))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-0.0900000000000000))*(x1242)*(x1245)))+(((IkReal(0.0636396103067893))*(cj5)*(x1242)*(x1249)))+(((py)*(x1241)*(x1253)))+(((IkReal(-0.0636396103067893))*(x1241)*(x1249)))+(((IkReal(1.10000000000000))*(cj5)*(pz)))+(((cj4)*(px)*(x1251)))+(((IkReal(-1.00000000000000))*(x1242)*(x1244)*(x1252)))+(((IkReal(-1.00000000000000))*(x1253)*(x1258)))+(((x1242)*(x1253)*(x1254)))+(((IkReal(-1.00000000000000))*(x1252)*(x1258)))+(((x1242)*(x1244)*(x1253)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(px)*(sj4)*(x1251)))+(((x1251)*(x1255)))+(((x1249)*(x1251))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x169=((IkReal(0.707106781186548))*(cj6));
IkReal x170=((sj5)*(sj6));
IkReal x171=((IkReal(0.707106781186548))*(sj6));
IkReal x172=((cj5)*(cj7));
IkReal x173=((cj7)*(sj5));
IkReal x174=((IkReal(1.00000000000000))*(cj6));
IkReal x175=((IkReal(1.00000000000000))*(sj4));
IkReal x176=((cj5)*(sj7));
IkReal x177=((sj5)*(sj7));
IkReal x178=((cj7)*(x171));
IkReal x179=((IkReal(0.707106781186548))*(x177));
IkReal x180=((cj5)*(x171));
IkReal x181=((IkReal(0.707106781186548))*(x173));
IkReal x182=((sj7)*(x171));
IkReal x183=((IkReal(-1.00000000000000))*(sj4));
IkReal x184=((x169)*(x172));
IkReal x185=((((IkReal(-1.00000000000000))*(x180)))+(x169));
IkReal x186=((x169)*(x176));
IkReal x187=((((IkReal(-1.00000000000000))*(x169)))+(((IkReal(-1.00000000000000))*(x180))));
IkReal x188=((((IkReal(-1.00000000000000))*(x174)*(x177)))+(x172));
IkReal x189=((cj4)*(x187));
IkReal x190=((((IkReal(-1.00000000000000))*(x176)))+(((IkReal(-1.00000000000000))*(x173)*(x174))));
IkReal x191=((x186)+(x181));
IkReal x192=((((IkReal(-1.00000000000000))*(x179)))+(x178)+(x184));
IkReal x193=((((IkReal(-1.00000000000000))*(x178)))+(((IkReal(-1.00000000000000))*(x179)))+(x184));
IkReal x194=((((IkReal(-1.00000000000000))*(x182)))+(x191));
IkReal x195=((x191)+(x182));
IkReal x196=((cj4)*(x194));
IkReal x197=((((sj4)*(x187)))+(((cj4)*(x185))));
IkReal x198=((x189)+(((IkReal(-1.00000000000000))*(x175)*(x185))));
IkReal x199=((((cj4)*(x192)))+(((sj4)*(x193))));
IkReal x200=((((cj4)*(x193)))+(((x183)*(x192))));
IkReal x201=((((cj4)*(x195)))+(((sj4)*(x194))));
IkReal x202=((x196)+(((x183)*(x195))));
new_r00=((((r20)*(x190)))+(((r00)*(x200)))+(((r10)*(x199))));
new_r01=((((r01)*(x200)))+(((r21)*(x190)))+(((r11)*(x199))));
new_r02=((((r12)*(x199)))+(((r22)*(x190)))+(((r02)*(x200))));
new_r10=((((r00)*(x198)))+(((r20)*(x170)))+(((r10)*(x197))));
new_r11=((((r21)*(x170)))+(((r11)*(x197)))+(((r01)*(((x189)+(((x183)*(x185))))))));
new_r12=((((r22)*(x170)))+(((r12)*(x197)))+(((r02)*(x198))));
new_r20=((((r20)*(x188)))+(((r00)*(x202)))+(((r10)*(x201))));
new_r21=((((r11)*(x201)))+(((r21)*(x188)))+(((r01)*(((x196)+(((IkReal(-1.00000000000000))*(x175)*(x195))))))));
new_r22=((((r12)*(x201)))+(((r22)*(x188)))+(((r02)*(x202))));
{
IkReal j9array[2], cj9array[2], sj9array[2];
bool j9valid[2]={false};
_nj9 = 2;
cj9array[0]=new_r22;
if( cj9array[0] >= -1-IKFAST_SINCOS_THRESH && cj9array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j9valid[0] = j9valid[1] = true;
    j9array[0] = IKacos(cj9array[0]);
    sj9array[0] = IKsin(j9array[0]);
    cj9array[1] = cj9array[0];
    j9array[1] = -j9array[0];
    sj9array[1] = -sj9array[0];
}
else if( isnan(cj9array[0]) )
{
    // probably any value will work
    j9valid[0] = true;
    cj9array[0] = 1; sj9array[0] = 0; j9array[0] = 0;
}
for(int ij9 = 0; ij9 < 2; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 2; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];

{
IkReal dummyeval[1];
IkReal gconst98;
gconst98=IKsign(sj9);
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst97;
gconst97=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x203=IKatan2(new_r02, new_r12);
j8array[0]=((IkReal(-1.00000000000000))*(x203));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x203))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j8))))+(((new_r12)*(IKcos(j8)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj8)))+(((cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))))+IKsqr(((((new_r10)*(sj8)))+(((cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))), ((((new_r10)*(sj8)))+(((cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x204=IKsin(j10);
IkReal x205=((IkReal(1.00000000000000))*(sj8));
IkReal x206=((IkReal(1.00000000000000))*(IKcos(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x205)))+(((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x204))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x206)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x205))));
evalcond[2]=((((new_r11)*(sj8)))+(x204)+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(x206)))+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x207=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=x207;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x207;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x208=IKatan2(new_r02, new_r12);
j8array[0]=((IkReal(-1.00000000000000))*(x208));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x208))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j8))))+(((new_r12)*(IKcos(j8)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((new_r11)*(sj8)))+(((cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))), ((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x209=IKcos(j10);
IkReal x210=((IkReal(1.00000000000000))*(sj8));
IkReal x211=((IkReal(1.00000000000000))*(IKsin(j10)));
evalcond[0]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x211)))+(((IkReal(-1.00000000000000))*(new_r00)*(x210))));
evalcond[1]=((((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x210)))+(((IkReal(-1.00000000000000))*(x209))));
evalcond[2]=((((new_r11)*(sj8)))+(((cj8)*(new_r01)))+(((IkReal(-1.00000000000000))*(x211))));
evalcond[3]=((((new_r10)*(sj8)))+(x209)+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x212=((gconst97)*(sj9));
if( IKabs(((new_r12)*(x212))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x212))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((new_r12)*(x212)), ((new_r02)*(x212)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[6];
IkReal x213=IKsin(j8);
IkReal x214=IKcos(j8);
IkReal x215=((IkReal(1.00000000000000))*(cj9));
IkReal x216=((IkReal(1.00000000000000))*(sj9));
IkReal x217=((IkReal(1.00000000000000))*(x213));
IkReal x218=((new_r12)*(x213));
IkReal x219=((new_r02)*(x214));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(x217)))+(((new_r12)*(x214))));
evalcond[1]=((x218)+(x219)+(((IkReal(-1.00000000000000))*(x216))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r22)*(x216)))+(((cj9)*(x219)))+(((cj9)*(x218))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r20)*(x215)))+(((IkReal(-1.00000000000000))*(new_r10)*(x213)*(x216)))+(((IkReal(-1.00000000000000))*(new_r00)*(x214)*(x216))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r21)*(x215)))+(((IkReal(-1.00000000000000))*(new_r01)*(x214)*(x216)))+(((IkReal(-1.00000000000000))*(new_r11)*(x213)*(x216))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x215)))+(((IkReal(-1.00000000000000))*(x216)*(x219)))+(((IkReal(-1.00000000000000))*(x216)*(x218))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst99;
gconst99=IKsign(sj9);
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x220=((((IkReal(-1.00000000000000))*(new_r02)*(sj8)))+(((cj8)*(new_r12))));
IkReal x221=((((new_r12)*(sj8)))+(((cj8)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x220;
evalcond[5]=x220;
evalcond[6]=x221;
evalcond[7]=x221;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj8)))+(((cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))))+IKsqr(((((new_r10)*(sj8)))+(((cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))), ((((new_r10)*(sj8)))+(((cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x222=IKsin(j10);
IkReal x223=((IkReal(1.00000000000000))*(sj8));
IkReal x224=((IkReal(1.00000000000000))*(IKcos(j10)));
evalcond[0]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x223)))+(((IkReal(-1.00000000000000))*(x222))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x223)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x224))));
evalcond[2]=((((new_r11)*(sj8)))+(x222)+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(((cj8)*(new_r00)))+(((IkReal(-1.00000000000000))*(x224))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x225=((IkReal(1.00000000000000))+(new_r22));
IkReal x226=((IkReal(1.00000000000000))*(new_r02));
IkReal x227=((new_r12)*(sj8));
IkReal x228=((((IkReal(-1.00000000000000))*(sj8)*(x226)))+(((cj8)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=x225;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x228;
evalcond[5]=x228;
evalcond[6]=((x227)+(((cj8)*(new_r02))));
evalcond[7]=((((IkReal(-1.00000000000000))*(cj8)*(x226)))+(((IkReal(-1.00000000000000))*(x227))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x225;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((new_r11)*(sj8)))+(((cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))), ((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x229=IKcos(j10);
IkReal x230=((IkReal(1.00000000000000))*(sj8));
IkReal x231=((IkReal(1.00000000000000))*(IKsin(j10)));
evalcond[0]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x231)))+(((IkReal(-1.00000000000000))*(new_r00)*(x230))));
evalcond[1]=((((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x230)))+(((IkReal(-1.00000000000000))*(x229))));
evalcond[2]=((((new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(x231)))+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(x229)+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((gconst99)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst99)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst99)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst99)*(new_r20)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[8];
IkReal x232=IKsin(j10);
IkReal x233=IKcos(j10);
IkReal x234=((IkReal(1.00000000000000))*(sj9));
IkReal x235=((new_r10)*(sj8));
IkReal x236=((cj8)*(cj9));
IkReal x237=((new_r11)*(sj8));
IkReal x238=((IkReal(1.00000000000000))*(sj8));
IkReal x239=((IkReal(1.00000000000000))*(x233));
evalcond[0]=((((sj9)*(x233)))+(new_r20));
evalcond[1]=((new_r21)+(((IkReal(-1.00000000000000))*(x232)*(x234))));
evalcond[2]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x232)))+(((IkReal(-1.00000000000000))*(new_r00)*(x238))));
evalcond[3]=((((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x239)))+(((IkReal(-1.00000000000000))*(new_r01)*(x238))));
evalcond[4]=((((cj9)*(x232)))+(x237)+(((cj8)*(new_r01))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj9)*(x239)))+(x235)+(((cj8)*(new_r00))));
evalcond[6]=((((cj9)*(x237)))+(((IkReal(-1.00000000000000))*(new_r21)*(x234)))+(((new_r01)*(x236)))+(x232));
evalcond[7]=((((cj9)*(x235)))+(((IkReal(-1.00000000000000))*(new_r20)*(x234)))+(((new_r00)*(x236)))+(((IkReal(-1.00000000000000))*(x239))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((gconst98)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst98)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst98)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst98)*(new_r20)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[2];
evalcond[0]=((((sj9)*(IKcos(j10))))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj9)*(IKsin(j10))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst100;
gconst100=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x240=((gconst100)*(sj9));
if( IKabs(((new_r12)*(x240))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x240))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((new_r12)*(x240)), ((new_r02)*(x240)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[12];
IkReal x241=IKsin(j8);
IkReal x242=IKcos(j8);
IkReal x243=((IkReal(1.00000000000000))*(cj10));
IkReal x244=((IkReal(1.00000000000000))*(sj9));
IkReal x245=((IkReal(1.00000000000000))*(cj9));
IkReal x246=((IkReal(1.00000000000000))*(x241));
IkReal x247=((new_r01)*(x242));
IkReal x248=((new_r11)*(x241));
IkReal x249=((cj9)*(x242));
IkReal x250=((new_r00)*(x242));
IkReal x251=((new_r02)*(x242));
IkReal x252=((new_r10)*(x241));
IkReal x253=((new_r12)*(x241));
evalcond[0]=((((new_r12)*(x242)))+(((IkReal(-1.00000000000000))*(new_r02)*(x246))));
evalcond[1]=((x251)+(x253)+(((IkReal(-1.00000000000000))*(x244))));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x246)))+(((IkReal(-1.00000000000000))*(sj10)))+(((new_r10)*(x242))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x246)))+(((IkReal(-1.00000000000000))*(x243)))+(((new_r11)*(x242))));
evalcond[4]=((((cj9)*(sj10)))+(x247)+(x248));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj9)*(x243)))+(x250)+(x252));
evalcond[6]=((((cj9)*(x253)))+(((IkReal(-1.00000000000000))*(new_r22)*(x244)))+(((new_r02)*(x249))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x244)*(x252)))+(((IkReal(-1.00000000000000))*(x244)*(x250)))+(((IkReal(-1.00000000000000))*(new_r20)*(x245))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r21)*(x245)))+(((IkReal(-1.00000000000000))*(x244)*(x248)))+(((IkReal(-1.00000000000000))*(x244)*(x247))));
evalcond[9]=((((cj9)*(x247)))+(((cj9)*(x248)))+(((IkReal(-1.00000000000000))*(new_r21)*(x244)))+(sj10));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x245)))+(((IkReal(-1.00000000000000))*(x244)*(x253)))+(((IkReal(-1.00000000000000))*(x244)*(x251))));
evalcond[11]=((((cj9)*(x252)))+(((new_r00)*(x249)))+(((IkReal(-1.00000000000000))*(new_r20)*(x244)))+(((IkReal(-1.00000000000000))*(x243))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ba32f85ca3c87502739320e9e26a5905"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
