/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:05:02.724350
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25;
x0=IKsin(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[1]);
x3=IKcos(j[2]);
x4=IKsin(j[2]);
x5=IKcos(j[0]);
x6=IKcos(j[3]);
x7=IKsin(j[3]);
x8=IKcos(j[5]);
x9=IKsin(j[5]);
x10=IKsin(j[4]);
x11=IKcos(j[4]);
x12=((IkReal(0.331000000000000))*(x0));
x13=((IkReal(1.00000000000000))*(x11));
x14=((IkReal(1.00000000000000))*(x0));
x15=((IkReal(1.00000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x10));
x17=((IkReal(0.405000000000000))*(x1));
x18=((IkReal(0.331000000000000))*(x5));
x19=((x1)*(x3));
x20=((x2)*(x4));
x21=((x2)*(x3));
x22=((x1)*(x4));
x23=((IkReal(1.00000000000000))*(x19));
x24=((x14)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x22))))));
x25=((x15)*(((x21)+(x22))));
eetrans[0]=((IkReal(0.0454160000000000))+(((IkReal(-0.100150000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x12)*(x19)))+(((IkReal(-1.00000000000000))*(x0)*(x17)))+(((x12)*(x20))));
eetrans[1]=((IkReal(0.302408000000000))+(((IkReal(0.331000000000000))*(x21)))+(((IkReal(0.331000000000000))*(x22)))+(((IkReal(0.405000000000000))*(x2))));
eetrans[2]=((IkReal(0.467052000000000))+(((x18)*(x19)))+(((IkReal(-0.100150000000000))*(x0)))+(((IkReal(-1.00000000000000))*(x18)*(x20)))+(((x17)*(x5))));
eerot[0]=((((x8)*(((((IkReal(-1.00000000000000))*(x5)*(x7)))+(((x24)*(x6)))))))+(((IkReal(-1.00000000000000))*(x9)*(((((x13)*(((((x5)*(x6)))+(((x24)*(x7)))))))+(((x16)*(((((x0)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x20))))))))))));
eerot[1]=((((x9)*(((((x11)*(x7)*(((((IkReal(-1.00000000000000))*(x20)))+(x23)))))+(((x10)*(((x21)+(x22)))))))))+(((x6)*(x8)*(((((IkReal(-1.00000000000000))*(x23)))+(x20))))));
eerot[2]=((((IkReal(-1.00000000000000))*(x9)*(((((x16)*(((((IkReal(-1.00000000000000))*(x15)*(x19)))+(((x20)*(x5)))))))+(((x13)*(((((x0)*(x6)))+(((x25)*(x7)))))))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x14)*(x7)))+(((x25)*(x6))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {5}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1;  _ij5[0] = -1; _ij5[1] = -1; _nj5 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j5=pfree[0]; cj5=cos(pfree[0]); sj5=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((IkReal(-0.0454160000000000))+(px));
new_r01=r02;
new_py=((IkReal(-0.467052000000000))+(pz));
new_r02=((IkReal(-1.00000000000000))*(r01));
new_pz=((IkReal(0.302408000000000))+(((IkReal(-1.00000000000000))*(py))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
cj2array[0]=((IkReal(-1.05783455484689))+(((IkReal(3.72981239043676))*(pp))));
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.49711119832787))+((sj2)*(sj2))+((cj2)*(cj2))+(((IkReal(2.44712990936556))*(cj2))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x26=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2))));
if( IKabs(((IkReal(0.331000000000000))*(sj2))) < IKFAST_ATAN2_MAGTHRESH && IKabs(x26) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x27=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.331000000000000))*(sj2)), x26)));
if( ((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x28=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x27)))+(((IkReal(-1.00000000000000))*(x28))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x27)))+(x28));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x29=((IkReal(20000.0000000000))*(sj1));
gconst0=IKsign(((((x29)*((py)*(py))))+(((x29)*((px)*(px))))));
dummyeval[0]=((((sj1)*((py)*(py))))+(((sj1)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x30=((pz)+(((IkReal(0.331000000000000))*(sj2))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=x30;
evalcond[3]=x30;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-20000.0000000000))*((py)*(py))))+(((IkReal(-20000.0000000000))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x31=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x31)))+(((IkReal(2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x31)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x31)))+(((IkReal(2003.00000000000))*(py)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x31)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x32=IKsin(j0);
IkReal x33=IKcos(j0);
IkReal x34=((px)*(x32));
IkReal x35=((py)*(x33));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x32)))+(((px)*(x33))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x35)))+(((IkReal(0.331000000000000))*(cj2)))+(x34));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(0.810000000000000))*(x35)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x34))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x36=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x37=((r00)*(x36));
IkReal x38=((IkReal(1.00000000000000))*(r02)*(x36));
sj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x38)))+(((cj1)*(cj2)*(x37)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x37)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x38))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x39=((cj0)*(cj5));
IkReal x40=((IkReal(1.00000000000000))*(r01));
IkReal x41=((cj4)*(sj0)*(sj5));
IkReal x42=((cj2)*(r00)*(sj1));
IkReal x43=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x44=((cj0)*(cj4)*(sj5));
IkReal x45=((cj1)*(r00)*(sj2));
IkReal x46=((cj1)*(cj2)*(r02));
IkReal x47=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x41)*(x46)))+(((x41)*(x42)))+(((x41)*(x45)))+(((r01)*(x44)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x40)))+(((IkReal(-1.00000000000000))*(x39)*(x47)))+(((IkReal(-1.00000000000000))*(x41)*(x47)))+(((x39)*(x45)))+(((x39)*(x42)))+(((x39)*(x46))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x42)*(x43)))+(((IkReal(-1.00000000000000))*(x39)*(x40)))+(((IkReal(-1.00000000000000))*(x40)*(x41)))+(((IkReal(-1.00000000000000))*(x43)*(x46)))+(((IkReal(-1.00000000000000))*(x43)*(x45)))+(((IkReal(-1.00000000000000))*(x44)*(x47)))+(((x42)*(x44)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x44)*(x46)))+(((x44)*(x45))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x41)*(x46)))+(((x41)*(x42)))+(((x41)*(x45)))+(((r01)*(x44)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x40)))+(((IkReal(-1.00000000000000))*(x39)*(x47)))+(((IkReal(-1.00000000000000))*(x41)*(x47)))+(((x39)*(x45)))+(((x39)*(x42)))+(((x39)*(x46)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x42)*(x43)))+(((IkReal(-1.00000000000000))*(x39)*(x40)))+(((IkReal(-1.00000000000000))*(x40)*(x41)))+(((IkReal(-1.00000000000000))*(x43)*(x46)))+(((IkReal(-1.00000000000000))*(x43)*(x45)))+(((IkReal(-1.00000000000000))*(x44)*(x47)))+(((x42)*(x44)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x44)*(x46)))+(((x44)*(x45)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x48=IKsin(j3);
IkReal x49=IKcos(j3);
IkReal x50=((IkReal(1.00000000000000))*(cj0));
IkReal x51=((cj0)*(cj1));
IkReal x52=((cj2)*(r02));
IkReal x53=((r00)*(sj2));
IkReal x54=((IkReal(1.00000000000000))*(sj0));
IkReal x55=((cj4)*(sj5));
IkReal x56=((r02)*(sj1)*(sj2));
IkReal x57=((cj2)*(r00)*(sj1));
IkReal x58=((IkReal(1.00000000000000))*(x49));
evalcond[0]=((((IkReal(-1.00000000000000))*(x55)*(x58)))+(((IkReal(-1.00000000000000))*(cj5)*(x48)))+(((cj0)*(x57)))+(((IkReal(-1.00000000000000))*(r01)*(x54)))+(((x51)*(x52)))+(((x51)*(x53)))+(((IkReal(-1.00000000000000))*(x50)*(x56))));
evalcond[1]=((((sj0)*(x56)))+(((IkReal(-1.00000000000000))*(cj5)*(x58)))+(((IkReal(-1.00000000000000))*(x54)*(x57)))+(((IkReal(-1.00000000000000))*(cj1)*(x53)*(x54)))+(((IkReal(-1.00000000000000))*(cj1)*(x52)*(x54)))+(((x48)*(x55)))+(((IkReal(-1.00000000000000))*(r01)*(x50))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x59=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x59)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x59)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x60=((cj0)*(cj5));
IkReal x61=((IkReal(1.00000000000000))*(r01));
IkReal x62=((cj4)*(sj0)*(sj5));
IkReal x63=((cj2)*(r00)*(sj1));
IkReal x64=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x65=((cj0)*(cj4)*(sj5));
IkReal x66=((cj1)*(r00)*(sj2));
IkReal x67=((cj1)*(cj2)*(r02));
IkReal x68=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x60)*(x66)))+(((x60)*(x67)))+(((x60)*(x63)))+(((x62)*(x66)))+(((x62)*(x67)))+(((x62)*(x63)))+(((r01)*(x65)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x61)))+(((IkReal(-1.00000000000000))*(x60)*(x68)))+(((IkReal(-1.00000000000000))*(x62)*(x68))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x61)*(x62)))+(((IkReal(-1.00000000000000))*(x64)*(x67)))+(((IkReal(-1.00000000000000))*(x64)*(x66)))+(((x63)*(x65)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x65)*(x68)))+(((IkReal(-1.00000000000000))*(x60)*(x61)))+(((IkReal(-1.00000000000000))*(x63)*(x64)))+(((x65)*(x67)))+(((x65)*(x66))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x60)*(x66)))+(((x60)*(x67)))+(((x60)*(x63)))+(((x62)*(x66)))+(((x62)*(x67)))+(((x62)*(x63)))+(((r01)*(x65)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x61)))+(((IkReal(-1.00000000000000))*(x60)*(x68)))+(((IkReal(-1.00000000000000))*(x62)*(x68)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x61)*(x62)))+(((IkReal(-1.00000000000000))*(x64)*(x67)))+(((IkReal(-1.00000000000000))*(x64)*(x66)))+(((x63)*(x65)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x65)*(x68)))+(((IkReal(-1.00000000000000))*(x60)*(x61)))+(((IkReal(-1.00000000000000))*(x63)*(x64)))+(((x65)*(x67)))+(((x65)*(x66)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x69=IKsin(j3);
IkReal x70=IKcos(j3);
IkReal x71=((IkReal(1.00000000000000))*(cj0));
IkReal x72=((cj0)*(cj1));
IkReal x73=((cj2)*(r02));
IkReal x74=((r00)*(sj2));
IkReal x75=((IkReal(1.00000000000000))*(sj0));
IkReal x76=((cj4)*(sj5));
IkReal x77=((r02)*(sj1)*(sj2));
IkReal x78=((cj2)*(r00)*(sj1));
IkReal x79=((IkReal(1.00000000000000))*(x70));
evalcond[0]=((((cj0)*(x78)))+(((IkReal(-1.00000000000000))*(r01)*(x75)))+(((IkReal(-1.00000000000000))*(x71)*(x77)))+(((IkReal(-1.00000000000000))*(cj5)*(x69)))+(((IkReal(-1.00000000000000))*(x76)*(x79)))+(((x72)*(x73)))+(((x72)*(x74))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x71)))+(((x69)*(x76)))+(((IkReal(-1.00000000000000))*(cj5)*(x79)))+(((sj0)*(x77)))+(((IkReal(-1.00000000000000))*(cj1)*(x73)*(x75)))+(((IkReal(-1.00000000000000))*(cj1)*(x74)*(x75)))+(((IkReal(-1.00000000000000))*(x75)*(x78))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
IkReal x80=((IkReal(0.331000000000000))*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=((pz)+(((IkReal(-1.00000000000000))*(x80))));
evalcond[3]=((((IkReal(-1.00000000000000))*(pz)))+(x80));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(20000.0000000000))*((px)*(px))))+(((IkReal(20000.0000000000))*((py)*(py))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x81=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x81)))+(((IkReal(-2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(py)*(x81)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(-2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x81)))+(((IkReal(-2003.00000000000))*(py)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(py)*(x81)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(-2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x82=IKcos(j0);
IkReal x83=IKsin(j0);
IkReal x84=((px)*(x83));
IkReal x85=((py)*(x82));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x83)))+(((px)*(x82))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2)))+(x85)+(((IkReal(-1.00000000000000))*(x84))));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x85)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.810000000000000))*(x84))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x86=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x87=((r00)*(x86));
IkReal x88=((IkReal(1.00000000000000))*(r02)*(x86));
sj4array[0]=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x87)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x88)))+(((cj1)*(cj2)*(x87)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x88))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x89=((cj0)*(cj5));
IkReal x90=((IkReal(1.00000000000000))*(r01));
IkReal x91=((cj4)*(sj0)*(sj5));
IkReal x92=((cj2)*(r00)*(sj1));
IkReal x93=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x94=((cj0)*(cj4)*(sj5));
IkReal x95=((cj1)*(r00)*(sj2));
IkReal x96=((cj1)*(cj2)*(r02));
IkReal x97=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x90)))+(((x89)*(x96)))+(((x89)*(x92)))+(((x89)*(x95)))+(((IkReal(-1.00000000000000))*(x91)*(x97)))+(((r01)*(x94)))+(((IkReal(-1.00000000000000))*(x89)*(x97)))+(((x91)*(x96)))+(((x91)*(x92)))+(((x91)*(x95))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x94)*(x97)))+(((x92)*(x94)))+(((IkReal(-1.00000000000000))*(x90)*(x91)))+(((IkReal(-1.00000000000000))*(x92)*(x93)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x93)*(x95)))+(((IkReal(-1.00000000000000))*(x93)*(x96)))+(((x94)*(x96)))+(((x94)*(x95)))+(((IkReal(-1.00000000000000))*(x89)*(x90))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x90)))+(((x89)*(x96)))+(((x89)*(x92)))+(((x89)*(x95)))+(((IkReal(-1.00000000000000))*(x91)*(x97)))+(((r01)*(x94)))+(((IkReal(-1.00000000000000))*(x89)*(x97)))+(((x91)*(x96)))+(((x91)*(x92)))+(((x91)*(x95)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x94)*(x97)))+(((x92)*(x94)))+(((IkReal(-1.00000000000000))*(x90)*(x91)))+(((IkReal(-1.00000000000000))*(x92)*(x93)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x93)*(x95)))+(((IkReal(-1.00000000000000))*(x93)*(x96)))+(((x94)*(x96)))+(((x94)*(x95)))+(((IkReal(-1.00000000000000))*(x89)*(x90)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x98=IKsin(j3);
IkReal x99=IKcos(j3);
IkReal x100=((IkReal(1.00000000000000))*(cj0));
IkReal x101=((cj0)*(cj1));
IkReal x102=((cj2)*(r02));
IkReal x103=((r00)*(sj2));
IkReal x104=((IkReal(1.00000000000000))*(sj0));
IkReal x105=((cj4)*(sj5));
IkReal x106=((r02)*(sj1)*(sj2));
IkReal x107=((cj2)*(r00)*(sj1));
IkReal x108=((IkReal(1.00000000000000))*(x99));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x104)))+(((IkReal(-1.00000000000000))*(x100)*(x106)))+(((IkReal(-1.00000000000000))*(cj5)*(x98)))+(((cj0)*(x107)))+(((x101)*(x102)))+(((x101)*(x103)))+(((IkReal(-1.00000000000000))*(x105)*(x108))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x104)*(x107)))+(((IkReal(-1.00000000000000))*(r01)*(x100)))+(((IkReal(-1.00000000000000))*(cj5)*(x108)))+(((IkReal(-1.00000000000000))*(cj1)*(x103)*(x104)))+(((x105)*(x98)))+(((IkReal(-1.00000000000000))*(cj1)*(x102)*(x104)))+(((sj0)*(x106))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x109=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x109)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x109)))+(((cj1)*(cj2)*(r00))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x110=((cj0)*(cj5));
IkReal x111=((IkReal(1.00000000000000))*(r01));
IkReal x112=((cj4)*(sj0)*(sj5));
IkReal x113=((cj2)*(r00)*(sj1));
IkReal x114=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x115=((cj0)*(cj4)*(sj5));
IkReal x116=((cj1)*(r00)*(sj2));
IkReal x117=((cj1)*(cj2)*(r02));
IkReal x118=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x112)*(x118)))+(((x110)*(x113)))+(((x110)*(x117)))+(((x110)*(x116)))+(((x112)*(x117)))+(((x112)*(x116)))+(((x112)*(x113)))+(((IkReal(-1.00000000000000))*(x110)*(x118)))+(((r01)*(x115)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x111))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x113)*(x114)))+(((x113)*(x115)))+(((IkReal(-1.00000000000000))*(x115)*(x118)))+(((IkReal(-1.00000000000000))*(x110)*(x111)))+(((IkReal(-1.00000000000000))*(x114)*(x116)))+(((IkReal(-1.00000000000000))*(x114)*(x117)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x115)*(x116)))+(((x115)*(x117))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x112)*(x118)))+(((x110)*(x113)))+(((x110)*(x117)))+(((x110)*(x116)))+(((x112)*(x117)))+(((x112)*(x116)))+(((x112)*(x113)))+(((IkReal(-1.00000000000000))*(x110)*(x118)))+(((r01)*(x115)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x111)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x113)*(x114)))+(((x113)*(x115)))+(((IkReal(-1.00000000000000))*(x115)*(x118)))+(((IkReal(-1.00000000000000))*(x110)*(x111)))+(((IkReal(-1.00000000000000))*(x114)*(x116)))+(((IkReal(-1.00000000000000))*(x114)*(x117)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x115)*(x116)))+(((x115)*(x117)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x119=IKsin(j3);
IkReal x120=IKcos(j3);
IkReal x121=((IkReal(1.00000000000000))*(cj0));
IkReal x122=((cj0)*(cj1));
IkReal x123=((cj2)*(r02));
IkReal x124=((r00)*(sj2));
IkReal x125=((IkReal(1.00000000000000))*(sj0));
IkReal x126=((cj4)*(sj5));
IkReal x127=((r02)*(sj1)*(sj2));
IkReal x128=((cj2)*(r00)*(sj1));
IkReal x129=((IkReal(1.00000000000000))*(x120));
evalcond[0]=((((cj0)*(x128)))+(((IkReal(-1.00000000000000))*(x126)*(x129)))+(((x122)*(x124)))+(((x122)*(x123)))+(((IkReal(-1.00000000000000))*(x121)*(x127)))+(((IkReal(-1.00000000000000))*(cj5)*(x119)))+(((IkReal(-1.00000000000000))*(r01)*(x125))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x124)*(x125)))+(((IkReal(-1.00000000000000))*(cj1)*(x123)*(x125)))+(((IkReal(-1.00000000000000))*(x125)*(x128)))+(((IkReal(-1.00000000000000))*(cj5)*(x129)))+(((x119)*(x126)))+(((sj0)*(x127)))+(((IkReal(-1.00000000000000))*(r01)*(x121))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x130=((IkReal(6620.00000000000))*(sj2));
IkReal x131=((IkReal(2003.00000000000))*(sj1));
IkReal x132=((IkReal(20000.0000000000))*(cj1)*(pz));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x131)))+(((px)*(x132)))+(((px)*(x130))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x132)))+(((IkReal(-1.00000000000000))*(py)*(x130)))+(((IkReal(-1.00000000000000))*(px)*(x131))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x131)))+(((px)*(x132)))+(((px)*(x130)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x132)))+(((IkReal(-1.00000000000000))*(py)*(x130)))+(((IkReal(-1.00000000000000))*(px)*(x131)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x133=IKcos(j0);
IkReal x134=IKsin(j0);
IkReal x135=((IkReal(0.810000000000000))*(cj1));
IkReal x136=((IkReal(0.331000000000000))*(sj2));
IkReal x137=((pz)*(sj1));
IkReal x138=((IkReal(0.331000000000000))*(cj2));
IkReal x139=((px)*(x134));
IkReal x140=((py)*(x133));
IkReal x141=((IkReal(1.00000000000000))*(x140));
evalcond[0]=((IkReal(0.100150000000000))+(((px)*(x133)))+(((py)*(x134))));
evalcond[1]=((((sj1)*(x140)))+(((IkReal(-1.00000000000000))*(sj1)*(x139)))+(x136)+(((cj1)*(pz))));
evalcond[2]=((IkReal(0.405000000000000))+(x137)+(x138)+(((IkReal(-1.00000000000000))*(cj1)*(x141)))+(((cj1)*(x139))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x135)*(x139)))+(((IkReal(-0.810000000000000))*(x137)))+(((x135)*(x140)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x141)))+(((IkReal(-1.00000000000000))*(sj1)*(x136)))+(((IkReal(0.405000000000000))*(cj1)))+(x139)+(((cj1)*(x138))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x142=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x143=((r00)*(x142));
IkReal x144=((IkReal(1.00000000000000))*(r02)*(x142));
sj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x144)))+(((cj1)*(cj2)*(x143)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x143)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x144))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x145=((cj0)*(cj5));
IkReal x146=((IkReal(1.00000000000000))*(r01));
IkReal x147=((cj4)*(sj0)*(sj5));
IkReal x148=((cj2)*(r00)*(sj1));
IkReal x149=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x150=((cj0)*(cj4)*(sj5));
IkReal x151=((cj1)*(r00)*(sj2));
IkReal x152=((cj1)*(cj2)*(r02));
IkReal x153=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x146)))+(((r01)*(x150)))+(((x145)*(x148)))+(((x145)*(x151)))+(((x145)*(x152)))+(((IkReal(-1.00000000000000))*(x147)*(x153)))+(((x147)*(x148)))+(((IkReal(-1.00000000000000))*(x145)*(x153)))+(((x147)*(x152)))+(((x147)*(x151))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x150)*(x152)))+(((x150)*(x151)))+(((IkReal(-1.00000000000000))*(x150)*(x153)))+(((IkReal(-1.00000000000000))*(x148)*(x149)))+(((IkReal(-1.00000000000000))*(x146)*(x147)))+(((x148)*(x150)))+(((IkReal(-1.00000000000000))*(x145)*(x146)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x149)*(x152)))+(((IkReal(-1.00000000000000))*(x149)*(x151))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x146)))+(((r01)*(x150)))+(((x145)*(x148)))+(((x145)*(x151)))+(((x145)*(x152)))+(((IkReal(-1.00000000000000))*(x147)*(x153)))+(((x147)*(x148)))+(((IkReal(-1.00000000000000))*(x145)*(x153)))+(((x147)*(x152)))+(((x147)*(x151)))))), ((gconst4)*(((((x150)*(x152)))+(((x150)*(x151)))+(((IkReal(-1.00000000000000))*(x150)*(x153)))+(((IkReal(-1.00000000000000))*(x148)*(x149)))+(((IkReal(-1.00000000000000))*(x146)*(x147)))+(((x148)*(x150)))+(((IkReal(-1.00000000000000))*(x145)*(x146)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x149)*(x152)))+(((IkReal(-1.00000000000000))*(x149)*(x151)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x154=IKsin(j3);
IkReal x155=IKcos(j3);
IkReal x156=((IkReal(1.00000000000000))*(cj0));
IkReal x157=((cj0)*(cj1));
IkReal x158=((cj2)*(r02));
IkReal x159=((r00)*(sj2));
IkReal x160=((IkReal(1.00000000000000))*(sj0));
IkReal x161=((cj4)*(sj5));
IkReal x162=((r02)*(sj1)*(sj2));
IkReal x163=((cj2)*(r00)*(sj1));
IkReal x164=((IkReal(1.00000000000000))*(x155));
evalcond[0]=((((cj0)*(x163)))+(((IkReal(-1.00000000000000))*(cj5)*(x154)))+(((x157)*(x158)))+(((x157)*(x159)))+(((IkReal(-1.00000000000000))*(r01)*(x160)))+(((IkReal(-1.00000000000000))*(x161)*(x164)))+(((IkReal(-1.00000000000000))*(x156)*(x162))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(cj5)*(x164)))+(((sj0)*(x162)))+(((IkReal(-1.00000000000000))*(x160)*(x163)))+(((IkReal(-1.00000000000000))*(r01)*(x156)))+(((x154)*(x161)))+(((IkReal(-1.00000000000000))*(cj1)*(x158)*(x160))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x165=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x165)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x165))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x166=((cj0)*(cj5));
IkReal x167=((IkReal(1.00000000000000))*(r01));
IkReal x168=((cj4)*(sj0)*(sj5));
IkReal x169=((cj2)*(r00)*(sj1));
IkReal x170=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x171=((cj0)*(cj4)*(sj5));
IkReal x172=((cj1)*(r00)*(sj2));
IkReal x173=((cj1)*(cj2)*(r02));
IkReal x174=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x167)))+(((r01)*(x171)))+(((x168)*(x172)))+(((x168)*(x173)))+(((x168)*(x169)))+(((x166)*(x173)))+(((x166)*(x172)))+(((x166)*(x169)))+(((IkReal(-1.00000000000000))*(x168)*(x174)))+(((IkReal(-1.00000000000000))*(x166)*(x174))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x166)*(x167)))+(((x169)*(x171)))+(((IkReal(-1.00000000000000))*(x170)*(x172)))+(((IkReal(-1.00000000000000))*(x170)*(x173)))+(((x171)*(x173)))+(((x171)*(x172)))+(((IkReal(-1.00000000000000))*(x171)*(x174)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(-1.00000000000000))*(x167)*(x168))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x167)))+(((r01)*(x171)))+(((x168)*(x172)))+(((x168)*(x173)))+(((x168)*(x169)))+(((x166)*(x173)))+(((x166)*(x172)))+(((x166)*(x169)))+(((IkReal(-1.00000000000000))*(x168)*(x174)))+(((IkReal(-1.00000000000000))*(x166)*(x174)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x166)*(x167)))+(((x169)*(x171)))+(((IkReal(-1.00000000000000))*(x170)*(x172)))+(((IkReal(-1.00000000000000))*(x170)*(x173)))+(((x171)*(x173)))+(((x171)*(x172)))+(((IkReal(-1.00000000000000))*(x171)*(x174)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x169)*(x170)))+(((IkReal(-1.00000000000000))*(x167)*(x168)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x175=IKsin(j3);
IkReal x176=IKcos(j3);
IkReal x177=((IkReal(1.00000000000000))*(cj0));
IkReal x178=((cj0)*(cj1));
IkReal x179=((cj2)*(r02));
IkReal x180=((r00)*(sj2));
IkReal x181=((IkReal(1.00000000000000))*(sj0));
IkReal x182=((cj4)*(sj5));
IkReal x183=((r02)*(sj1)*(sj2));
IkReal x184=((cj2)*(r00)*(sj1));
IkReal x185=((IkReal(1.00000000000000))*(x176));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x181)))+(((cj0)*(x184)))+(((x178)*(x180)))+(((x178)*(x179)))+(((IkReal(-1.00000000000000))*(cj5)*(x175)))+(((IkReal(-1.00000000000000))*(x177)*(x183)))+(((IkReal(-1.00000000000000))*(x182)*(x185))));
evalcond[1]=((((sj0)*(x183)))+(((IkReal(-1.00000000000000))*(cj5)*(x185)))+(((IkReal(-1.00000000000000))*(r01)*(x177)))+(((x175)*(x182)))+(((IkReal(-1.00000000000000))*(cj1)*(x179)*(x181)))+(((IkReal(-1.00000000000000))*(cj1)*(x180)*(x181)))+(((IkReal(-1.00000000000000))*(x181)*(x184))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x186=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x187=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x186)))+(((IkReal(-1.00000000000000))*(x187))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x186)))+(x187));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x188=((cj0)*(py));
IkReal x189=((px)*(sj0));
IkReal x190=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-1.00000000000000))*(x188)*(x190)))+(((IkReal(-331.000000000000))*(pz)*(sj2)))+(((x189)*(x190)))+(((IkReal(-405.000000000000))*(x188)))+(((IkReal(405.000000000000))*(x189)))));
IkReal x191=((px)*(sj0));
IkReal x192=((cj0)*(py));
dummyeval[0]=((((IkReal(-1.22356495468278))*(x192)))+(((IkReal(1.22356495468278))*(x191)))+(((cj2)*(x191)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x192))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x193=((IkReal(331.000000000000))*(sj2));
IkReal x194=((cj0)*(py));
IkReal x195=((px)*(sj0));
IkReal x196=((IkReal(1000.00000000000))*(pz));
IkReal x197=((cj2)*(pz));
IkReal x198=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x194)*(x198)))+(((IkReal(405000.000000000))*(pz)))+(((x195)*(x198)))+(((IkReal(331000.000000000))*(x197))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x194)*(x198)))+(((IkReal(405000.000000000))*(pz)))+(((x195)*(x198)))+(((IkReal(331000.000000000))*(x197)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(331.000000000000))*(x197)))+(((x193)*(x195)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x193)*(x194))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x197)))+(((x193)*(x195)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x193)*(x194)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x195)*(x196)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x194)*(x196))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x194)*(x198)))+(((IkReal(405000.000000000))*(pz)))+(((x195)*(x198)))+(((IkReal(331000.000000000))*(x197))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x194)*(x198)))+(((IkReal(405000.000000000))*(pz)))+(((x195)*(x198)))+(((IkReal(331000.000000000))*(x197)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(331.000000000000))*(x197)))+(((x193)*(x195)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x193)*(x194))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x197)))+(((x193)*(x195)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x193)*(x194)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x195)*(x196)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x194)*(x196)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x194)*(x198)))+(((IkReal(405000.000000000))*(pz)))+(((x195)*(x198)))+(((IkReal(331000.000000000))*(x197))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x194)*(x198)))+(((IkReal(405000.000000000))*(pz)))+(((x195)*(x198)))+(((IkReal(331000.000000000))*(x197)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(331.000000000000))*(x197)))+(((x193)*(x195)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x193)*(x194))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x197)))+(((x193)*(x195)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x193)*(x194)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x195)*(x196)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x194)*(x196)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x199=IKsin(j1);
IkReal x200=IKcos(j1);
IkReal x201=((IkReal(0.331000000000000))*(sj2));
IkReal x202=((cj0)*(py));
IkReal x203=((px)*(sj0));
IkReal x204=((IkReal(0.331000000000000))*(cj2));
IkReal x205=((IkReal(0.810000000000000))*(x200));
IkReal x206=((pz)*(x199));
evalcond[0]=((((IkReal(0.405000000000000))*(x199)))+(((x200)*(x201)))+(pz)+(((x199)*(x204))));
evalcond[1]=((((pz)*(x200)))+(x201)+(((IkReal(-1.00000000000000))*(x199)*(x203)))+(((x199)*(x202))));
evalcond[2]=((IkReal(0.405000000000000))+(((x200)*(x203)))+(x206)+(x204)+(((IkReal(-1.00000000000000))*(x200)*(x202))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x203)*(x205)))+(((IkReal(-0.810000000000000))*(x206)))+(((IkReal(-1.00000000000000))*(pp)))+(((x202)*(x205))));
evalcond[4]=((((x200)*(x204)))+(((IkReal(0.405000000000000))*(x200)))+(x203)+(((IkReal(-1.00000000000000))*(x199)*(x201)))+(((IkReal(-1.00000000000000))*(x202))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x207=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x208=((r00)*(x207));
IkReal x209=((IkReal(1.00000000000000))*(r02)*(x207));
sj4array[0]=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x208)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x209)))+(((cj1)*(cj2)*(x208)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x209))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x210=((cj0)*(cj5));
IkReal x211=((IkReal(1.00000000000000))*(r01));
IkReal x212=((cj4)*(sj0)*(sj5));
IkReal x213=((cj2)*(r00)*(sj1));
IkReal x214=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x215=((cj0)*(cj4)*(sj5));
IkReal x216=((cj1)*(r00)*(sj2));
IkReal x217=((cj1)*(cj2)*(r02));
IkReal x218=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x210)*(x217)))+(((x210)*(x216)))+(((x210)*(x213)))+(((IkReal(-1.00000000000000))*(x210)*(x218)))+(((x212)*(x213)))+(((x212)*(x216)))+(((x212)*(x217)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x211)))+(((r01)*(x215)))+(((IkReal(-1.00000000000000))*(x212)*(x218))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x213)*(x215)))+(((x215)*(x216)))+(((x215)*(x217)))+(((IkReal(-1.00000000000000))*(x210)*(x211)))+(((IkReal(-1.00000000000000))*(x211)*(x212)))+(((IkReal(-1.00000000000000))*(x213)*(x214)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x214)*(x216)))+(((IkReal(-1.00000000000000))*(x214)*(x217)))+(((IkReal(-1.00000000000000))*(x215)*(x218))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x210)*(x217)))+(((x210)*(x216)))+(((x210)*(x213)))+(((IkReal(-1.00000000000000))*(x210)*(x218)))+(((x212)*(x213)))+(((x212)*(x216)))+(((x212)*(x217)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x211)))+(((r01)*(x215)))+(((IkReal(-1.00000000000000))*(x212)*(x218)))))), ((gconst4)*(((((x213)*(x215)))+(((x215)*(x216)))+(((x215)*(x217)))+(((IkReal(-1.00000000000000))*(x210)*(x211)))+(((IkReal(-1.00000000000000))*(x211)*(x212)))+(((IkReal(-1.00000000000000))*(x213)*(x214)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x214)*(x216)))+(((IkReal(-1.00000000000000))*(x214)*(x217)))+(((IkReal(-1.00000000000000))*(x215)*(x218)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x219=IKsin(j3);
IkReal x220=IKcos(j3);
IkReal x221=((IkReal(1.00000000000000))*(cj0));
IkReal x222=((cj0)*(cj1));
IkReal x223=((cj2)*(r02));
IkReal x224=((r00)*(sj2));
IkReal x225=((IkReal(1.00000000000000))*(sj0));
IkReal x226=((cj4)*(sj5));
IkReal x227=((r02)*(sj1)*(sj2));
IkReal x228=((cj2)*(r00)*(sj1));
IkReal x229=((IkReal(1.00000000000000))*(x220));
evalcond[0]=((((x222)*(x223)))+(((x222)*(x224)))+(((IkReal(-1.00000000000000))*(r01)*(x225)))+(((cj0)*(x228)))+(((IkReal(-1.00000000000000))*(cj5)*(x219)))+(((IkReal(-1.00000000000000))*(x221)*(x227)))+(((IkReal(-1.00000000000000))*(x226)*(x229))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x229)))+(((IkReal(-1.00000000000000))*(x225)*(x228)))+(((sj0)*(x227)))+(((IkReal(-1.00000000000000))*(r01)*(x221)))+(((x219)*(x226)))+(((IkReal(-1.00000000000000))*(cj1)*(x224)*(x225)))+(((IkReal(-1.00000000000000))*(cj1)*(x223)*(x225))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x230=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x230)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x230)))+(((cj1)*(cj2)*(r00))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x231=((cj0)*(cj5));
IkReal x232=((IkReal(1.00000000000000))*(r01));
IkReal x233=((cj4)*(sj0)*(sj5));
IkReal x234=((cj2)*(r00)*(sj1));
IkReal x235=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x236=((cj0)*(cj4)*(sj5));
IkReal x237=((cj1)*(r00)*(sj2));
IkReal x238=((cj1)*(cj2)*(r02));
IkReal x239=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x233)*(x239)))+(((x233)*(x237)))+(((x233)*(x234)))+(((x233)*(x238)))+(((r01)*(x236)))+(((IkReal(-1.00000000000000))*(x231)*(x239)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x232)))+(((x231)*(x234)))+(((x231)*(x237)))+(((x231)*(x238))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x234)*(x236)))+(((IkReal(-1.00000000000000))*(x235)*(x237)))+(((IkReal(-1.00000000000000))*(x235)*(x238)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((IkReal(-1.00000000000000))*(x236)*(x239)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x234)*(x235)))+(((IkReal(-1.00000000000000))*(x232)*(x233)))+(((x236)*(x238)))+(((x236)*(x237))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x233)*(x239)))+(((x233)*(x237)))+(((x233)*(x234)))+(((x233)*(x238)))+(((r01)*(x236)))+(((IkReal(-1.00000000000000))*(x231)*(x239)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x232)))+(((x231)*(x234)))+(((x231)*(x237)))+(((x231)*(x238)))))), ((gconst4)*(((((x234)*(x236)))+(((IkReal(-1.00000000000000))*(x235)*(x237)))+(((IkReal(-1.00000000000000))*(x235)*(x238)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((IkReal(-1.00000000000000))*(x236)*(x239)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x234)*(x235)))+(((IkReal(-1.00000000000000))*(x232)*(x233)))+(((x236)*(x238)))+(((x236)*(x237)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x240=IKsin(j3);
IkReal x241=IKcos(j3);
IkReal x242=((IkReal(1.00000000000000))*(cj0));
IkReal x243=((cj0)*(cj1));
IkReal x244=((cj2)*(r02));
IkReal x245=((r00)*(sj2));
IkReal x246=((IkReal(1.00000000000000))*(sj0));
IkReal x247=((cj4)*(sj5));
IkReal x248=((r02)*(sj1)*(sj2));
IkReal x249=((cj2)*(r00)*(sj1));
IkReal x250=((IkReal(1.00000000000000))*(x241));
evalcond[0]=((((cj0)*(x249)))+(((IkReal(-1.00000000000000))*(x242)*(x248)))+(((IkReal(-1.00000000000000))*(cj5)*(x240)))+(((IkReal(-1.00000000000000))*(r01)*(x246)))+(((IkReal(-1.00000000000000))*(x247)*(x250)))+(((x243)*(x244)))+(((x243)*(x245))));
evalcond[1]=((((x240)*(x247)))+(((IkReal(-1.00000000000000))*(cj5)*(x250)))+(((IkReal(-1.00000000000000))*(r01)*(x242)))+(((IkReal(-1.00000000000000))*(cj1)*(x245)*(x246)))+(((IkReal(-1.00000000000000))*(cj1)*(x244)*(x246)))+(((sj0)*(x248)))+(((IkReal(-1.00000000000000))*(x246)*(x249))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x251=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x251)))+(((cj0)*(py)*(x251)))+(((IkReal(109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x251)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x251)))+(((cj0)*(py)*(x251)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x251)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x252=IKsin(j1);
IkReal x253=IKcos(j1);
IkReal x254=((IkReal(0.331000000000000))*(sj2));
IkReal x255=((cj0)*(py));
IkReal x256=((px)*(sj0));
IkReal x257=((IkReal(0.331000000000000))*(cj2));
IkReal x258=((IkReal(0.810000000000000))*(x253));
IkReal x259=((pz)*(x252));
evalcond[0]=((((x253)*(x254)))+(pz)+(((IkReal(0.405000000000000))*(x252)))+(((x252)*(x257))));
evalcond[1]=((((pz)*(x253)))+(((IkReal(-1.00000000000000))*(x252)*(x256)))+(x254)+(((x252)*(x255))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x253)*(x255)))+(x259)+(x257)+(((x253)*(x256))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x256)*(x258)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x259)))+(((x255)*(x258))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x252)*(x254)))+(x256)+(((x253)*(x257)))+(((IkReal(0.405000000000000))*(x253)))+(((IkReal(-1.00000000000000))*(x255))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x260=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x261=((r00)*(x260));
IkReal x262=((IkReal(1.00000000000000))*(r02)*(x260));
sj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x262)))+(((cj1)*(cj2)*(x261)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x262)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x261))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x263=((cj0)*(cj5));
IkReal x264=((IkReal(1.00000000000000))*(r01));
IkReal x265=((cj4)*(sj0)*(sj5));
IkReal x266=((cj2)*(r00)*(sj1));
IkReal x267=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x268=((cj0)*(cj4)*(sj5));
IkReal x269=((cj1)*(r00)*(sj2));
IkReal x270=((cj1)*(cj2)*(r02));
IkReal x271=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x263)*(x271)))+(((x263)*(x266)))+(((x263)*(x269)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x264)))+(((x263)*(x270)))+(((IkReal(-1.00000000000000))*(x265)*(x271)))+(((x265)*(x270)))+(((r01)*(x268)))+(((x265)*(x266)))+(((x265)*(x269))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x263)*(x264)))+(((x266)*(x268)))+(((IkReal(-1.00000000000000))*(x268)*(x271)))+(((IkReal(-1.00000000000000))*(x267)*(x269)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x266)*(x267)))+(((IkReal(-1.00000000000000))*(x267)*(x270)))+(((IkReal(-1.00000000000000))*(x264)*(x265)))+(((x268)*(x269)))+(((x268)*(x270))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x263)*(x271)))+(((x263)*(x266)))+(((x263)*(x269)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x264)))+(((x263)*(x270)))+(((IkReal(-1.00000000000000))*(x265)*(x271)))+(((x265)*(x270)))+(((r01)*(x268)))+(((x265)*(x266)))+(((x265)*(x269)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x263)*(x264)))+(((x266)*(x268)))+(((IkReal(-1.00000000000000))*(x268)*(x271)))+(((IkReal(-1.00000000000000))*(x267)*(x269)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x266)*(x267)))+(((IkReal(-1.00000000000000))*(x267)*(x270)))+(((IkReal(-1.00000000000000))*(x264)*(x265)))+(((x268)*(x269)))+(((x268)*(x270)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x272=IKsin(j3);
IkReal x273=IKcos(j3);
IkReal x274=((IkReal(1.00000000000000))*(cj0));
IkReal x275=((cj0)*(cj1));
IkReal x276=((cj2)*(r02));
IkReal x277=((r00)*(sj2));
IkReal x278=((IkReal(1.00000000000000))*(sj0));
IkReal x279=((cj4)*(sj5));
IkReal x280=((r02)*(sj1)*(sj2));
IkReal x281=((cj2)*(r00)*(sj1));
IkReal x282=((IkReal(1.00000000000000))*(x273));
evalcond[0]=((((x275)*(x277)))+(((x275)*(x276)))+(((IkReal(-1.00000000000000))*(cj5)*(x272)))+(((cj0)*(x281)))+(((IkReal(-1.00000000000000))*(r01)*(x278)))+(((IkReal(-1.00000000000000))*(x279)*(x282)))+(((IkReal(-1.00000000000000))*(x274)*(x280))));
evalcond[1]=((((x272)*(x279)))+(((IkReal(-1.00000000000000))*(cj1)*(x276)*(x278)))+(((IkReal(-1.00000000000000))*(cj1)*(x277)*(x278)))+(((IkReal(-1.00000000000000))*(r01)*(x274)))+(((IkReal(-1.00000000000000))*(cj5)*(x282)))+(((sj0)*(x280)))+(((IkReal(-1.00000000000000))*(x278)*(x281))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x283=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x283)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x283)))+(((cj1)*(cj2)*(r00))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x284=((cj0)*(cj5));
IkReal x285=((IkReal(1.00000000000000))*(r01));
IkReal x286=((cj4)*(sj0)*(sj5));
IkReal x287=((cj2)*(r00)*(sj1));
IkReal x288=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x289=((cj0)*(cj4)*(sj5));
IkReal x290=((cj1)*(r00)*(sj2));
IkReal x291=((cj1)*(cj2)*(r02));
IkReal x292=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x284)*(x292)))+(((x284)*(x287)))+(((r01)*(x289)))+(((x286)*(x287)))+(((IkReal(-1.00000000000000))*(x286)*(x292)))+(((x284)*(x291)))+(((x284)*(x290)))+(((x286)*(x290)))+(((x286)*(x291)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x285))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x289)*(x290)))+(((x289)*(x291)))+(((IkReal(-1.00000000000000))*(x287)*(x288)))+(((IkReal(-1.00000000000000))*(x289)*(x292)))+(((IkReal(-1.00000000000000))*(x284)*(x285)))+(((IkReal(-1.00000000000000))*(x285)*(x286)))+(((IkReal(-1.00000000000000))*(x288)*(x290)))+(((IkReal(-1.00000000000000))*(x288)*(x291)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x287)*(x289))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x284)*(x292)))+(((x284)*(x287)))+(((r01)*(x289)))+(((x286)*(x287)))+(((IkReal(-1.00000000000000))*(x286)*(x292)))+(((x284)*(x291)))+(((x284)*(x290)))+(((x286)*(x290)))+(((x286)*(x291)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x285)))))), ((gconst4)*(((((x289)*(x290)))+(((x289)*(x291)))+(((IkReal(-1.00000000000000))*(x287)*(x288)))+(((IkReal(-1.00000000000000))*(x289)*(x292)))+(((IkReal(-1.00000000000000))*(x284)*(x285)))+(((IkReal(-1.00000000000000))*(x285)*(x286)))+(((IkReal(-1.00000000000000))*(x288)*(x290)))+(((IkReal(-1.00000000000000))*(x288)*(x291)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x287)*(x289)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x293=IKsin(j3);
IkReal x294=IKcos(j3);
IkReal x295=((IkReal(1.00000000000000))*(cj0));
IkReal x296=((cj0)*(cj1));
IkReal x297=((cj2)*(r02));
IkReal x298=((r00)*(sj2));
IkReal x299=((IkReal(1.00000000000000))*(sj0));
IkReal x300=((cj4)*(sj5));
IkReal x301=((r02)*(sj1)*(sj2));
IkReal x302=((cj2)*(r00)*(sj1));
IkReal x303=((IkReal(1.00000000000000))*(x294));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x293)))+(((IkReal(-1.00000000000000))*(x295)*(x301)))+(((cj0)*(x302)))+(((x296)*(x297)))+(((x296)*(x298)))+(((IkReal(-1.00000000000000))*(x300)*(x303)))+(((IkReal(-1.00000000000000))*(r01)*(x299))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x298)*(x299)))+(((IkReal(-1.00000000000000))*(cj1)*(x297)*(x299)))+(((IkReal(-1.00000000000000))*(cj5)*(x303)))+(((x293)*(x300)))+(((IkReal(-1.00000000000000))*(x299)*(x302)))+(((sj0)*(x301)))+(((IkReal(-1.00000000000000))*(r01)*(x295))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x304=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x305=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x305)))+(((IkReal(-1.00000000000000))*(x304))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x305)+(((IkReal(-1.00000000000000))*(x304))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x306=((cj0)*(py));
IkReal x307=((px)*(sj0));
IkReal x308=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-331.000000000000))*(pz)*(sj2)))+(((IkReal(405.000000000000))*(x307)))+(((IkReal(-1.00000000000000))*(x306)*(x308)))+(((IkReal(-405.000000000000))*(x306)))+(((x307)*(x308)))));
IkReal x309=((px)*(sj0));
IkReal x310=((cj0)*(py));
dummyeval[0]=((((IkReal(-1.22356495468278))*(x310)))+(((IkReal(1.22356495468278))*(x309)))+(((IkReal(-1.00000000000000))*(cj2)*(x310)))+(((cj2)*(x309)))+(((IkReal(-1.00000000000000))*(pz)*(sj2))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x311=((IkReal(331.000000000000))*(sj2));
IkReal x312=((cj0)*(py));
IkReal x313=((px)*(sj0));
IkReal x314=((IkReal(1000.00000000000))*(pz));
IkReal x315=((cj2)*(pz));
IkReal x316=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x312)*(x316)))+(((IkReal(331000.000000000))*(x315)))+(((IkReal(405000.000000000))*(pz)))+(((x313)*(x316))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x312)*(x316)))+(((IkReal(331000.000000000))*(x315)))+(((IkReal(405000.000000000))*(pz)))+(((x313)*(x316)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(331.000000000000))*(x315)))+(((x311)*(x313)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x311)*(x312))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x315)))+(((x311)*(x313)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x311)*(x312)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x313)*(x314)))+(((x312)*(x314)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x312)*(x316)))+(((IkReal(331000.000000000))*(x315)))+(((IkReal(405000.000000000))*(pz)))+(((x313)*(x316))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x312)*(x316)))+(((IkReal(331000.000000000))*(x315)))+(((IkReal(405000.000000000))*(pz)))+(((x313)*(x316)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(331.000000000000))*(x315)))+(((x311)*(x313)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x311)*(x312))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x315)))+(((x311)*(x313)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x311)*(x312)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x313)*(x314)))+(((x312)*(x314)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x312)*(x316)))+(((IkReal(331000.000000000))*(x315)))+(((IkReal(405000.000000000))*(pz)))+(((x313)*(x316))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x312)*(x316)))+(((IkReal(331000.000000000))*(x315)))+(((IkReal(405000.000000000))*(pz)))+(((x313)*(x316)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(331.000000000000))*(x315)))+(((x311)*(x313)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x311)*(x312))))) != 0)?((IkReal)1/(((((IkReal(331.000000000000))*(x315)))+(((x311)*(x313)))+(((IkReal(405.000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x311)*(x312)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x313)*(x314)))+(((x312)*(x314)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x317=IKsin(j1);
IkReal x318=IKcos(j1);
IkReal x319=((IkReal(0.331000000000000))*(sj2));
IkReal x320=((cj0)*(py));
IkReal x321=((px)*(sj0));
IkReal x322=((IkReal(0.331000000000000))*(cj2));
IkReal x323=((IkReal(0.810000000000000))*(x318));
IkReal x324=((pz)*(x317));
evalcond[0]=((((IkReal(0.405000000000000))*(x317)))+(pz)+(((x317)*(x322)))+(((x318)*(x319))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x317)*(x321)))+(x319)+(((x317)*(x320)))+(((pz)*(x318))));
evalcond[2]=((IkReal(0.405000000000000))+(((x318)*(x321)))+(x324)+(x322)+(((IkReal(-1.00000000000000))*(x318)*(x320))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x324)))+(((IkReal(-1.00000000000000))*(x321)*(x323)))+(((x320)*(x323)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[4]=((((x318)*(x322)))+(((IkReal(0.405000000000000))*(x318)))+(((IkReal(-1.00000000000000))*(x320)))+(x321)+(((IkReal(-1.00000000000000))*(x317)*(x319))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x325=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x326=((r00)*(x325));
IkReal x327=((IkReal(1.00000000000000))*(r02)*(x325));
sj4array[0]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x327)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x326)))+(((cj1)*(cj2)*(x326)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x327))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x328=((cj0)*(cj5));
IkReal x329=((IkReal(1.00000000000000))*(r01));
IkReal x330=((cj4)*(sj0)*(sj5));
IkReal x331=((cj2)*(r00)*(sj1));
IkReal x332=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x333=((cj0)*(cj4)*(sj5));
IkReal x334=((cj1)*(r00)*(sj2));
IkReal x335=((cj1)*(cj2)*(r02));
IkReal x336=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((r01)*(x333)))+(((x330)*(x334)))+(((x330)*(x335)))+(((x330)*(x331)))+(((IkReal(-1.00000000000000))*(x328)*(x336)))+(((IkReal(-1.00000000000000))*(x330)*(x336)))+(((x328)*(x335)))+(((x328)*(x334)))+(((x328)*(x331)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x329))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x333)*(x336)))+(((x331)*(x333)))+(((IkReal(-1.00000000000000))*(x329)*(x330)))+(((x333)*(x335)))+(((x333)*(x334)))+(((IkReal(-1.00000000000000))*(x331)*(x332)))+(((IkReal(-1.00000000000000))*(x332)*(x335)))+(((IkReal(-1.00000000000000))*(x332)*(x334)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x328)*(x329))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((r01)*(x333)))+(((x330)*(x334)))+(((x330)*(x335)))+(((x330)*(x331)))+(((IkReal(-1.00000000000000))*(x328)*(x336)))+(((IkReal(-1.00000000000000))*(x330)*(x336)))+(((x328)*(x335)))+(((x328)*(x334)))+(((x328)*(x331)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x329)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x333)*(x336)))+(((x331)*(x333)))+(((IkReal(-1.00000000000000))*(x329)*(x330)))+(((x333)*(x335)))+(((x333)*(x334)))+(((IkReal(-1.00000000000000))*(x331)*(x332)))+(((IkReal(-1.00000000000000))*(x332)*(x335)))+(((IkReal(-1.00000000000000))*(x332)*(x334)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x328)*(x329)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x337=IKsin(j3);
IkReal x338=IKcos(j3);
IkReal x339=((IkReal(1.00000000000000))*(cj0));
IkReal x340=((cj0)*(cj1));
IkReal x341=((cj2)*(r02));
IkReal x342=((r00)*(sj2));
IkReal x343=((IkReal(1.00000000000000))*(sj0));
IkReal x344=((cj4)*(sj5));
IkReal x345=((r02)*(sj1)*(sj2));
IkReal x346=((cj2)*(r00)*(sj1));
IkReal x347=((IkReal(1.00000000000000))*(x338));
evalcond[0]=((((IkReal(-1.00000000000000))*(x344)*(x347)))+(((IkReal(-1.00000000000000))*(r01)*(x343)))+(((cj0)*(x346)))+(((x340)*(x341)))+(((x340)*(x342)))+(((IkReal(-1.00000000000000))*(cj5)*(x337)))+(((IkReal(-1.00000000000000))*(x339)*(x345))));
evalcond[1]=((((x337)*(x344)))+(((IkReal(-1.00000000000000))*(r01)*(x339)))+(((IkReal(-1.00000000000000))*(cj1)*(x341)*(x343)))+(((IkReal(-1.00000000000000))*(cj5)*(x347)))+(((IkReal(-1.00000000000000))*(cj1)*(x342)*(x343)))+(((sj0)*(x345)))+(((IkReal(-1.00000000000000))*(x343)*(x346))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x348=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x348)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x348))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x349=((cj0)*(cj5));
IkReal x350=((IkReal(1.00000000000000))*(r01));
IkReal x351=((cj4)*(sj0)*(sj5));
IkReal x352=((cj2)*(r00)*(sj1));
IkReal x353=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x354=((cj0)*(cj4)*(sj5));
IkReal x355=((cj1)*(r00)*(sj2));
IkReal x356=((cj1)*(cj2)*(r02));
IkReal x357=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((r01)*(x354)))+(((IkReal(-1.00000000000000))*(x349)*(x357)))+(((x351)*(x355)))+(((x351)*(x352)))+(((x351)*(x356)))+(((x349)*(x356)))+(((x349)*(x352)))+(((x349)*(x355)))+(((IkReal(-1.00000000000000))*(x351)*(x357)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x350))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x352)*(x354)))+(((IkReal(-1.00000000000000))*(x349)*(x350)))+(((x354)*(x356)))+(((x354)*(x355)))+(((IkReal(-1.00000000000000))*(x353)*(x356)))+(((IkReal(-1.00000000000000))*(x353)*(x355)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x354)*(x357)))+(((IkReal(-1.00000000000000))*(x352)*(x353)))+(((IkReal(-1.00000000000000))*(x350)*(x351))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((r01)*(x354)))+(((IkReal(-1.00000000000000))*(x349)*(x357)))+(((x351)*(x355)))+(((x351)*(x352)))+(((x351)*(x356)))+(((x349)*(x356)))+(((x349)*(x352)))+(((x349)*(x355)))+(((IkReal(-1.00000000000000))*(x351)*(x357)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x350)))))), ((gconst4)*(((((x352)*(x354)))+(((IkReal(-1.00000000000000))*(x349)*(x350)))+(((x354)*(x356)))+(((x354)*(x355)))+(((IkReal(-1.00000000000000))*(x353)*(x356)))+(((IkReal(-1.00000000000000))*(x353)*(x355)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x354)*(x357)))+(((IkReal(-1.00000000000000))*(x352)*(x353)))+(((IkReal(-1.00000000000000))*(x350)*(x351)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x358=IKsin(j3);
IkReal x359=IKcos(j3);
IkReal x360=((IkReal(1.00000000000000))*(cj0));
IkReal x361=((cj0)*(cj1));
IkReal x362=((cj2)*(r02));
IkReal x363=((r00)*(sj2));
IkReal x364=((IkReal(1.00000000000000))*(sj0));
IkReal x365=((cj4)*(sj5));
IkReal x366=((r02)*(sj1)*(sj2));
IkReal x367=((cj2)*(r00)*(sj1));
IkReal x368=((IkReal(1.00000000000000))*(x359));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x358)))+(((x361)*(x363)))+(((x361)*(x362)))+(((IkReal(-1.00000000000000))*(r01)*(x364)))+(((IkReal(-1.00000000000000))*(x360)*(x366)))+(((cj0)*(x367)))+(((IkReal(-1.00000000000000))*(x365)*(x368))));
evalcond[1]=((((x358)*(x365)))+(((IkReal(-1.00000000000000))*(r01)*(x360)))+(((IkReal(-1.00000000000000))*(cj5)*(x368)))+(((sj0)*(x366)))+(((IkReal(-1.00000000000000))*(cj1)*(x363)*(x364)))+(((IkReal(-1.00000000000000))*(cj1)*(x362)*(x364)))+(((IkReal(-1.00000000000000))*(x364)*(x367))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x369=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x369)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))+(((cj0)*(py)*(x369))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x369)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((IkReal(134.055000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x369)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))+(((cj0)*(py)*(x369)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x369)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x370=IKsin(j1);
IkReal x371=IKcos(j1);
IkReal x372=((IkReal(0.331000000000000))*(sj2));
IkReal x373=((cj0)*(py));
IkReal x374=((px)*(sj0));
IkReal x375=((IkReal(0.331000000000000))*(cj2));
IkReal x376=((IkReal(0.810000000000000))*(x371));
IkReal x377=((pz)*(x370));
evalcond[0]=((((x370)*(x375)))+(pz)+(((IkReal(0.405000000000000))*(x370)))+(((x371)*(x372))));
evalcond[1]=((((x370)*(x373)))+(x372)+(((pz)*(x371)))+(((IkReal(-1.00000000000000))*(x370)*(x374))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x371)*(x373)))+(x377)+(x375)+(((x371)*(x374))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-0.810000000000000))*(x377)))+(((IkReal(-1.00000000000000))*(x374)*(x376)))+(((IkReal(-1.00000000000000))*(pp)))+(((x373)*(x376))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x373)))+(((IkReal(-1.00000000000000))*(x370)*(x372)))+(x374)+(((IkReal(0.405000000000000))*(x371)))+(((x371)*(x375))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x378=((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30);
IkReal x379=((r00)*(x378));
IkReal x380=((IkReal(1.00000000000000))*(r02)*(x378));
sj4array[0]=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x379)))+(((cj1)*(cj2)*(x379)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x380)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x380))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x381=((cj0)*(cj5));
IkReal x382=((IkReal(1.00000000000000))*(r01));
IkReal x383=((cj4)*(sj0)*(sj5));
IkReal x384=((cj2)*(r00)*(sj1));
IkReal x385=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x386=((cj0)*(cj4)*(sj5));
IkReal x387=((cj1)*(r00)*(sj2));
IkReal x388=((cj1)*(cj2)*(r02));
IkReal x389=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x381)*(x389)))+(((x383)*(x387)))+(((x383)*(x384)))+(((x383)*(x388)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x382)))+(((x381)*(x388)))+(((x381)*(x384)))+(((x381)*(x387)))+(((IkReal(-1.00000000000000))*(x383)*(x389)))+(((r01)*(x386))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x384)*(x386)))+(((IkReal(-1.00000000000000))*(x381)*(x382)))+(((x386)*(x388)))+(((x386)*(x387)))+(((IkReal(-1.00000000000000))*(x384)*(x385)))+(((IkReal(-1.00000000000000))*(x386)*(x389)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x385)*(x388)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(-1.00000000000000))*(x382)*(x383))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x381)*(x389)))+(((x383)*(x387)))+(((x383)*(x384)))+(((x383)*(x388)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x382)))+(((x381)*(x388)))+(((x381)*(x384)))+(((x381)*(x387)))+(((IkReal(-1.00000000000000))*(x383)*(x389)))+(((r01)*(x386)))))), ((gconst4)*(((((x384)*(x386)))+(((IkReal(-1.00000000000000))*(x381)*(x382)))+(((x386)*(x388)))+(((x386)*(x387)))+(((IkReal(-1.00000000000000))*(x384)*(x385)))+(((IkReal(-1.00000000000000))*(x386)*(x389)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x385)*(x388)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(-1.00000000000000))*(x382)*(x383)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x390=IKsin(j3);
IkReal x391=IKcos(j3);
IkReal x392=((IkReal(1.00000000000000))*(cj0));
IkReal x393=((cj0)*(cj1));
IkReal x394=((cj2)*(r02));
IkReal x395=((r00)*(sj2));
IkReal x396=((IkReal(1.00000000000000))*(sj0));
IkReal x397=((cj4)*(sj5));
IkReal x398=((r02)*(sj1)*(sj2));
IkReal x399=((cj2)*(r00)*(sj1));
IkReal x400=((IkReal(1.00000000000000))*(x391));
evalcond[0]=((((cj0)*(x399)))+(((IkReal(-1.00000000000000))*(cj5)*(x390)))+(((IkReal(-1.00000000000000))*(r01)*(x396)))+(((IkReal(-1.00000000000000))*(x397)*(x400)))+(((IkReal(-1.00000000000000))*(x392)*(x398)))+(((x393)*(x394)))+(((x393)*(x395))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x395)*(x396)))+(((IkReal(-1.00000000000000))*(cj1)*(x394)*(x396)))+(((sj0)*(x398)))+(((IkReal(-1.00000000000000))*(r01)*(x392)))+(((IkReal(-1.00000000000000))*(x396)*(x399)))+(((IkReal(-1.00000000000000))*(cj5)*(x400)))+(((x390)*(x397))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x401=((IkReal(1.00000000000000))*(r02));
sj4array[0]=((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x401)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x401))))));
if( sj4array[0] >= -1-IKFAST_SINCOS_THRESH && sj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKasin(sj4array[0]);
    cj4array[0] = IKcos(j4array[0]);
    sj4array[1] = sj4array[0];
    j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
    cj4array[1] = -cj4array[0];
}
else if( isnan(sj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x402=((cj0)*(cj5));
IkReal x403=((IkReal(1.00000000000000))*(r01));
IkReal x404=((cj4)*(sj0)*(sj5));
IkReal x405=((cj2)*(r00)*(sj1));
IkReal x406=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x407=((cj0)*(cj4)*(sj5));
IkReal x408=((cj1)*(r00)*(sj2));
IkReal x409=((cj1)*(cj2)*(r02));
IkReal x410=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x404)*(x408)))+(((x404)*(x409)))+(((x404)*(x405)))+(((IkReal(-1.00000000000000))*(x402)*(x410)))+(((r01)*(x407)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x403)))+(((IkReal(-1.00000000000000))*(x404)*(x410)))+(((x402)*(x409)))+(((x402)*(x408)))+(((x402)*(x405))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x407)*(x408)))+(((x407)*(x409)))+(((x405)*(x407)))+(((IkReal(-1.00000000000000))*(x406)*(x409)))+(((IkReal(-1.00000000000000))*(x406)*(x408)))+(((IkReal(-1.00000000000000))*(x403)*(x404)))+(((IkReal(-1.00000000000000))*(x402)*(x403)))+(((IkReal(-1.00000000000000))*(x407)*(x410)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x405)*(x406))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x404)*(x408)))+(((x404)*(x409)))+(((x404)*(x405)))+(((IkReal(-1.00000000000000))*(x402)*(x410)))+(((r01)*(x407)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x403)))+(((IkReal(-1.00000000000000))*(x404)*(x410)))+(((x402)*(x409)))+(((x402)*(x408)))+(((x402)*(x405)))))), ((gconst4)*(((((x407)*(x408)))+(((x407)*(x409)))+(((x405)*(x407)))+(((IkReal(-1.00000000000000))*(x406)*(x409)))+(((IkReal(-1.00000000000000))*(x406)*(x408)))+(((IkReal(-1.00000000000000))*(x403)*(x404)))+(((IkReal(-1.00000000000000))*(x402)*(x403)))+(((IkReal(-1.00000000000000))*(x407)*(x410)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x405)*(x406)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x411=IKsin(j3);
IkReal x412=IKcos(j3);
IkReal x413=((IkReal(1.00000000000000))*(cj0));
IkReal x414=((cj0)*(cj1));
IkReal x415=((cj2)*(r02));
IkReal x416=((r00)*(sj2));
IkReal x417=((IkReal(1.00000000000000))*(sj0));
IkReal x418=((cj4)*(sj5));
IkReal x419=((r02)*(sj1)*(sj2));
IkReal x420=((cj2)*(r00)*(sj1));
IkReal x421=((IkReal(1.00000000000000))*(x412));
evalcond[0]=((((x414)*(x415)))+(((x414)*(x416)))+(((cj0)*(x420)))+(((IkReal(-1.00000000000000))*(x418)*(x421)))+(((IkReal(-1.00000000000000))*(r01)*(x417)))+(((IkReal(-1.00000000000000))*(x413)*(x419)))+(((IkReal(-1.00000000000000))*(cj5)*(x411))));
evalcond[1]=((((sj0)*(x419)))+(((IkReal(-1.00000000000000))*(cj1)*(x416)*(x417)))+(((x411)*(x418)))+(((IkReal(-1.00000000000000))*(cj5)*(x421)))+(((IkReal(-1.00000000000000))*(x417)*(x420)))+(((IkReal(-1.00000000000000))*(cj1)*(x415)*(x417)))+(((IkReal(-1.00000000000000))*(r01)*(x413))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
