/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:08:40.078031
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[1]);
x4=IKsin(j[2]);
x5=IKsin(j[0]);
x6=IKsin(j[4]);
x7=IKsin(j[3]);
x8=IKcos(j[3]);
x9=IKcos(j[4]);
x10=IKcos(j[5]);
x11=IKsin(j[5]);
x12=((IkReal(0.0900000000000000))*(x5));
x13=((IkReal(0.0900000000000000))*(x0));
x14=((IkReal(0.295000000000000))*(x5));
x15=((IkReal(0.0800000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x0));
x17=((IkReal(0.270000000000000))*(x3));
x18=((IkReal(1.00000000000000))*(x5));
x19=((IkReal(0.295000000000000))*(x0));
x20=((IkReal(0.0800000000000000))*(x0));
x21=((IkReal(0.0800000000000000))*(x8));
x22=((x2)*(x3));
x23=((x3)*(x4));
x24=((x1)*(x2));
x25=((IkReal(-1.00000000000000))*(x8));
x26=((x0)*(x7));
x27=((x1)*(x4));
x28=((IkReal(-1.00000000000000))*(x6));
x29=((((IkReal(-1.00000000000000))*(x24)))+(x23));
x30=((x16)*(((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x27))))));
x31=((x18)*(((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x27))))));
x32=((x30)*(x8));
IkReal x33=((IkReal(1.00000000000000))*(x23));
eetrans[0]=((((x0)*(x17)))+(((x19)*(x24)))+(((x6)*(((((x21)*(x30)))+(((IkReal(-1.00000000000000))*(x15)*(x7)))))))+(((IkReal(0.0750000000000000))*(x0)))+(((x9)*(((((IkReal(-1.00000000000000))*(x20)*(x33)))+(((x20)*(x24)))))))+(((IkReal(-1.00000000000000))*(x19)*(x33)))+(((x13)*(x22)))+(((x13)*(x27))));
IkReal x34=((IkReal(1.00000000000000))*(x12));
IkReal x35=((IkReal(1.00000000000000))*(x24));
eetrans[1]=((((IkReal(-1.00000000000000))*(x22)*(x34)))+(((x9)*(((((IkReal(-1.00000000000000))*(x15)*(x35)))+(((x15)*(x23)))))))+(((IkReal(-1.00000000000000))*(x27)*(x34)))+(((x14)*(x23)))+(((IkReal(-0.0750000000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x17)*(x5)))+(((x6)*(((((IkReal(-1.00000000000000))*(x21)*(x31)))+(((IkReal(-1.00000000000000))*(x20)*(x7)))))))+(((IkReal(-1.00000000000000))*(x14)*(x35))));
eetrans[2]=((IkReal(0.335000000000000))+(((x9)*(((((IkReal(-0.0800000000000000))*(x22)))+(((IkReal(-0.0800000000000000))*(x27)))))))+(((IkReal(0.270000000000000))*(x1)))+(((IkReal(-0.0900000000000000))*(x23)))+(((IkReal(-0.295000000000000))*(x22)))+(((IkReal(-0.295000000000000))*(x27)))+(((x6)*(x8)*(((((IkReal(-0.0800000000000000))*(x24)))+(((IkReal(0.0800000000000000))*(x23)))))))+(((IkReal(0.0900000000000000))*(x24))));
eerot[0]=((((x11)*(((((x30)*(x7)))+(((x5)*(x8)))))))+(((x10)*(((((x6)*(((((x0)*(x24)))+(((IkReal(-1.00000000000000))*(x16)*(x23)))))))+(((x9)*(((((x18)*(x7)))+(((IkReal(-1.00000000000000))*(x32))))))))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x9)*(((((IkReal(-1.00000000000000))*(x26)))+(((x25)*(x31)))))))+(((x28)*(((((IkReal(-1.00000000000000))*(x18)*(x23)))+(((x24)*(x5)))))))))))+(((x11)*(((((x16)*(x8)))+(((IkReal(-1.00000000000000))*(x31)*(x7))))))));
eerot[2]=((((x11)*(x29)*(x7)))+(((x10)*(((((x25)*(x29)*(x9)))+(((x28)*(((x27)+(x22))))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j2,cj2,sj2,htj2,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij2[2], _nj2;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij2[0] = -1; _ij2[1] = -1; _nj2 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j2=pfree[0]; cj2=cos(pfree[0]); sj2=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=px;
new_r01=((IkReal(-1.00000000000000))*(r01));
new_py=((IkReal(-1.00000000000000))*(py));
new_r02=((IkReal(-1.00000000000000))*(r02));
new_pz=((IkReal(0.335000000000000))+(((IkReal(-1.00000000000000))*(pz))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.270000000000000))*(r02)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(cj2)*(r00)))+(((IkReal(-0.295000000000000))*(r02)*(sj2))));
IkReal gconst1;
gconst1=((((IkReal(-0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(0.590000000000000))*(r00)*(sj2)))+(((IkReal(-0.590000000000000))*(cj2)*(r02)))+(((IkReal(-0.540000000000000))*(r00))));
IkReal gconst2;
gconst2=((((IkReal(0.295000000000000))*(r02)*(sj2)))+(((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.295000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(-0.270000000000000))*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02))));
IkReal gconst3;
gconst3=((IkReal(0.167250000000000))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj2)))+(((IkReal(0.540000000000000))*(pz)))+(((IkReal(-0.145800000000000))*(sj2)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(0.0928500000000000))*(cj2)))+(((IkReal(-0.590000000000000))*(cj2)*(px))));
IkReal gconst4;
gconst4=((IkReal(0.0810000000000000))+(((IkReal(-0.0885000000000000))*(sj2)))+(((IkReal(1.18000000000000))*(px)*(sj2)))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(-0.360000000000000))*(cj2)*(px)))+(((IkReal(-1.08000000000000))*(px)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.18000000000000))*(cj2)*(pz))));
IkReal gconst5;
gconst5=((IkReal(0.167250000000000))+(((IkReal(0.590000000000000))*(cj2)*(px)))+(((IkReal(-0.172800000000000))*(sj2)))+(((IkReal(0.00435000000000000))*(cj2)))+(((IkReal(-0.150000000000000))*(px)))+(pp)+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(0.590000000000000))*(pz)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.540000000000000))*(pz))));
IkReal gconst6;
gconst6=((((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.270000000000000))*(r02)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(cj2)*(r00)))+(((IkReal(-0.295000000000000))*(r02)*(sj2))));
IkReal gconst7;
gconst7=((((IkReal(-0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(0.590000000000000))*(r00)*(sj2)))+(((IkReal(-0.590000000000000))*(cj2)*(r02)))+(((IkReal(-0.540000000000000))*(r00))));
IkReal gconst8;
gconst8=((((IkReal(0.295000000000000))*(r02)*(sj2)))+(((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.295000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(-0.270000000000000))*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02))));
IkReal gconst9;
gconst9=((IkReal(0.167250000000000))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj2)))+(((IkReal(0.540000000000000))*(pz)))+(((IkReal(-0.145800000000000))*(sj2)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(0.0928500000000000))*(cj2)))+(((IkReal(-0.590000000000000))*(cj2)*(px))));
IkReal gconst10;
gconst10=((IkReal(0.0810000000000000))+(((IkReal(-0.0885000000000000))*(sj2)))+(((IkReal(1.18000000000000))*(px)*(sj2)))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(-0.360000000000000))*(cj2)*(px)))+(((IkReal(-1.08000000000000))*(px)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.18000000000000))*(cj2)*(pz))));
IkReal gconst11;
gconst11=((IkReal(0.167250000000000))+(((IkReal(0.590000000000000))*(cj2)*(px)))+(((IkReal(-0.172800000000000))*(sj2)))+(((IkReal(0.00435000000000000))*(cj2)))+(((IkReal(-0.150000000000000))*(px)))+(pp)+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(0.590000000000000))*(pz)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.540000000000000))*(pz))));
IkReal gconst12;
gconst12=((((IkReal(-0.180000000000000))*(r01)*(sj2)))+(((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.590000000000000))*(cj2)*(r01))));
IkReal gconst13;
gconst13=((((IkReal(-1.08000000000000))*(r01)))+(((IkReal(-0.360000000000000))*(cj2)*(r01)))+(((IkReal(1.18000000000000))*(r01)*(sj2))));
IkReal gconst14;
gconst14=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(0.590000000000000))*(cj2)*(r01)))+(((IkReal(0.180000000000000))*(r01)*(sj2))));
IkReal gconst15;
gconst15=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-0.360000000000000))*(py)*(sj2)))+(((IkReal(-1.18000000000000))*(cj2)*(py))));
IkReal gconst16;
gconst16=((((IkReal(2.36000000000000))*(py)*(sj2)))+(((IkReal(-0.720000000000000))*(cj2)*(py)))+(((IkReal(-2.16000000000000))*(py))));
IkReal gconst17;
gconst17=((((IkReal(0.360000000000000))*(py)*(sj2)))+(((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.18000000000000))*(cj2)*(py))));
IkReal gconst18;
gconst18=((((IkReal(-0.180000000000000))*(r01)*(sj2)))+(((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.590000000000000))*(cj2)*(r01))));
IkReal gconst19;
gconst19=((((IkReal(-1.08000000000000))*(r01)))+(((IkReal(-0.360000000000000))*(cj2)*(r01)))+(((IkReal(1.18000000000000))*(r01)*(sj2))));
IkReal gconst20;
gconst20=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(0.590000000000000))*(cj2)*(r01)))+(((IkReal(0.180000000000000))*(r01)*(sj2))));
IkReal gconst21;
gconst21=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-0.360000000000000))*(py)*(sj2)))+(((IkReal(-1.18000000000000))*(cj2)*(py))));
IkReal gconst22;
gconst22=((((IkReal(2.36000000000000))*(py)*(sj2)))+(((IkReal(-0.720000000000000))*(cj2)*(py)))+(((IkReal(-2.16000000000000))*(py))));
IkReal gconst23;
gconst23=((((IkReal(0.360000000000000))*(py)*(sj2)))+(((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.18000000000000))*(cj2)*(py))));
IkReal gconst24;
gconst24=((((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.295000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(0.270000000000000))*(r02)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(r02)*(sj2))));
IkReal gconst25;
gconst25=((((IkReal(-0.590000000000000))*(r00)*(sj2)))+(((IkReal(0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(0.540000000000000))*(r00)))+(((IkReal(-0.590000000000000))*(cj2)*(r02))));
IkReal gconst26;
gconst26=((((IkReal(0.295000000000000))*(r02)*(sj2)))+(((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(0.0750000000000000))*(r00)))+(((IkReal(-0.270000000000000))*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02)))+(((IkReal(-0.295000000000000))*(cj2)*(r00))));
IkReal gconst27;
gconst27=((IkReal(0.167250000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(0.590000000000000))*(cj2)*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj2)))+(((IkReal(0.540000000000000))*(pz)))+(((IkReal(-0.145800000000000))*(sj2)))+(pp)+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(0.0928500000000000))*(cj2))));
IkReal gconst28;
gconst28=((IkReal(0.0810000000000000))+(((IkReal(-0.0885000000000000))*(sj2)))+(((IkReal(1.08000000000000))*(px)))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(-1.18000000000000))*(px)*(sj2)))+(((IkReal(0.360000000000000))*(cj2)*(px)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.18000000000000))*(cj2)*(pz))));
IkReal gconst29;
gconst29=((IkReal(0.167250000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(-0.172800000000000))*(sj2)))+(((IkReal(0.00435000000000000))*(cj2)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(0.590000000000000))*(pz)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.590000000000000))*(cj2)*(px)))+(((IkReal(-0.540000000000000))*(pz))));
IkReal gconst30;
gconst30=((((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.295000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(0.270000000000000))*(r02)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(r02)*(sj2))));
IkReal gconst31;
gconst31=((((IkReal(-0.590000000000000))*(r00)*(sj2)))+(((IkReal(0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(0.540000000000000))*(r00)))+(((IkReal(-0.590000000000000))*(cj2)*(r02))));
IkReal gconst32;
gconst32=((((IkReal(0.295000000000000))*(r02)*(sj2)))+(((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(0.0750000000000000))*(r00)))+(((IkReal(-0.270000000000000))*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02)))+(((IkReal(-0.295000000000000))*(cj2)*(r00))));
IkReal gconst33;
gconst33=((IkReal(0.167250000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(0.590000000000000))*(cj2)*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj2)))+(((IkReal(0.540000000000000))*(pz)))+(((IkReal(-0.145800000000000))*(sj2)))+(pp)+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(0.0928500000000000))*(cj2))));
IkReal gconst34;
gconst34=((IkReal(0.0810000000000000))+(((IkReal(-0.0885000000000000))*(sj2)))+(((IkReal(1.08000000000000))*(px)))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(-1.18000000000000))*(px)*(sj2)))+(((IkReal(0.360000000000000))*(cj2)*(px)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.18000000000000))*(cj2)*(pz))));
IkReal gconst35;
gconst35=((IkReal(0.167250000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(-0.172800000000000))*(sj2)))+(((IkReal(0.00435000000000000))*(cj2)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(0.590000000000000))*(pz)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.590000000000000))*(cj2)*(px)))+(((IkReal(-0.540000000000000))*(pz))));
IkReal x36=((IkReal(1.00000000000000))*(gconst26));
IkReal x37=((gconst30)*(gconst34));
IkReal x38=((gconst31)*(gconst33));
IkReal x39=((IkReal(1.00000000000000))*(gconst17));
IkReal x40=((gconst2)*(gconst6));
IkReal x41=((gconst11)*(gconst3));
IkReal x42=((gconst25)*(gconst29));
IkReal x43=((gconst14)*(gconst28));
IkReal x44=((IkReal(1.00000000000000))*(gconst14));
IkReal x45=((gconst21)*(gconst32));
IkReal x46=((gconst10)*(gconst6));
IkReal x47=((gconst21)*(gconst5));
IkReal x48=((gconst0)*(gconst8));
IkReal x49=((gconst13)*(gconst5));
IkReal x50=((IkReal(1.00000000000000))*(gconst9));
IkReal x51=((gconst30)*(gconst35));
IkReal x52=((gconst26)*(gconst27));
IkReal x53=((gconst22)*(gconst30));
IkReal x54=((IkReal(1.00000000000000))*(gconst12));
IkReal x55=((gconst2)*(gconst3));
IkReal x56=((gconst1)*(gconst5));
IkReal x57=((gconst19)*(gconst33));
IkReal x58=((gconst26)*(gconst28));
IkReal x59=((gconst11)*(gconst6));
IkReal x60=((gconst1)*(gconst7));
IkReal x61=((gconst18)*(gconst2));
IkReal x62=((gconst18)*(gconst34));
IkReal x63=((gconst24)*(gconst29));
IkReal x64=((IkReal(1.00000000000000))*(gconst23));
IkReal x65=((gconst20)*(gconst33));
IkReal x66=((gconst32)*(gconst33));
IkReal x67=((gconst0)*(gconst5));
IkReal x68=((IkReal(1.00000000000000))*(gconst18));
IkReal x69=((gconst21)*(gconst31));
IkReal x70=((gconst5)*(gconst9));
IkReal x71=((gconst4)*(gconst9));
IkReal x72=((gconst13)*(gconst29));
IkReal x73=((gconst14)*(gconst6));
IkReal x74=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x75=((gconst27)*(x66));
IkReal x76=((gconst2)*(gconst4)*(gconst7));
op[0]=((((x38)*(x58)))+(((IkReal(-1.00000000000000))*(x51)*(x63)))+(((x37)*(x42)))+(((IkReal(-1.00000000000000))*(x36)*(x75)))+(((x63)*(x66)))+(((x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst28)*(x36)*(x37)))+(((IkReal(-1.00000000000000))*(x38)*(x42))));
op[1]=((((gconst23)*(gconst30)*(x52)))+(((IkReal(-1.00000000000000))*(x44)*(x75)))+(((IkReal(-1.00000000000000))*(gconst15)*(x36)*(x66)))+(((IkReal(-1.00000000000000))*(gconst35)*(x63)*(x68)))+(((gconst16)*(gconst26)*(x38)))+(((gconst12)*(gconst29)*(x66)))+(((IkReal(-1.00000000000000))*(gconst28)*(x36)*(x62)))+(((IkReal(-1.00000000000000))*(x42)*(x69)))+(((x58)*(x69)))+(((IkReal(-1.00000000000000))*(gconst28)*(x36)*(x53)))+(((x45)*(x63)))+(((IkReal(-1.00000000000000))*(x42)*(x57)))+(((IkReal(-1.00000000000000))*(x38)*(x72)))+(((IkReal(-1.00000000000000))*(gconst16)*(x36)*(x37)))+(((x38)*(x43)))+(((gconst15)*(gconst26)*(x51)))+(((IkReal(-1.00000000000000))*(gconst25)*(x38)*(x39)))+(((gconst17)*(gconst25)*(x37)))+(((x42)*(x53)))+(((x57)*(x58)))+(((gconst14)*(gconst27)*(x51)))+(((gconst17)*(gconst24)*(x66)))+(((x42)*(x62)))+(((IkReal(-1.00000000000000))*(gconst27)*(x36)*(x65)))+(((IkReal(-1.00000000000000))*(x37)*(x43)))+(((x63)*(x65)))+(((gconst18)*(gconst35)*(x52)))+(((x37)*(x72)))+(((IkReal(-1.00000000000000))*(gconst27)*(x36)*(x45)))+(((IkReal(-1.00000000000000))*(gconst29)*(x51)*(x54)))+(((IkReal(-1.00000000000000))*(gconst30)*(x63)*(x64)))+(((IkReal(-1.00000000000000))*(gconst24)*(x39)*(x51))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((IkReal(-1.00000000000000))*(x47)*(x60)))+(((IkReal(-1.00000000000000))*(gconst19)*(x50)*(x56)))+(((IkReal(-1.00000000000000))*(gconst6)*(x64)*(x67)))+(((gconst10)*(gconst18)*(x56)))+(((gconst22)*(gconst6)*(x56)))+(((IkReal(-1.00000000000000))*(gconst20)*(x50)*(x55)))+(((IkReal(-1.00000000000000))*(gconst9)*(x39)*(x60)))+(((IkReal(-1.00000000000000))*(gconst0)*(x39)*(x59)))+(((x47)*(x48)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x40)))+(((x41)*(x73)))+(((gconst12)*(gconst8)*(x70)))+(((x46)*(x49)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x55)))+(((gconst19)*(gconst2)*(x71)))+(((gconst11)*(gconst15)*(x40)))+(((IkReal(-1.00000000000000))*(gconst11)*(x67)*(x68)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x44)))+(((IkReal(-1.00000000000000))*(x61)*(x74)))+(((gconst14)*(gconst7)*(x71)))+(((IkReal(-1.00000000000000))*(gconst7)*(x49)*(x50)))+(((gconst21)*(x76)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x50)))+(((gconst20)*(gconst9)*(x67)))+(((IkReal(-1.00000000000000))*(gconst4)*(x44)*(x46)))+(((x41)*(x61)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((gconst17)*(gconst9)*(x48)))+(((gconst23)*(gconst3)*(x40)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x40)))+(((gconst1)*(gconst17)*(x46)))+(((IkReal(-1.00000000000000))*(gconst5)*(x54)*(x59))));
op[8]=((((x40)*(x41)))+(((IkReal(-1.00000000000000))*(x59)*(x67)))+(((x48)*(x70)))+(((IkReal(-1.00000000000000))*(gconst8)*(x50)*(x55)))+(((IkReal(-1.00000000000000))*(gconst7)*(x50)*(x56)))+(((x46)*(x56)))+(((gconst2)*(gconst7)*(x71)))+(((IkReal(-1.00000000000000))*(x40)*(x74))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x77=(sj2)*(sj2);
IkReal x78=(cj2)*(cj2);
IkReal x79=((IkReal(0.0142687500000000))*(r02));
IkReal x80=((IkReal(0.318600000000000))*(sj2));
IkReal x81=((IkReal(0.0972000000000000))*(cj2));
IkReal x82=((py)*(r02)*(sj0));
IkReal x83=((IkReal(0.190250000000000))*(x78));
IkReal x84=((pz)*(r01)*(sj0));
IkReal x85=((cj0)*(px)*(r02));
IkReal x86=((IkReal(0.190250000000000))*(x77));
IkReal x87=((cj0)*(pz)*(r00));
gconst52=IKsign(((((IkReal(-1.00000000000000))*(x81)*(x87)))+(((IkReal(-1.00000000000000))*(x81)*(x84)))+(((IkReal(-1.00000000000000))*(x80)*(x85)))+(((IkReal(-1.00000000000000))*(x80)*(x82)))+(((IkReal(-0.145800000000000))*(x87)))+(((IkReal(-0.145800000000000))*(x84)))+(((IkReal(-1.00000000000000))*(x83)*(x87)))+(((IkReal(-1.00000000000000))*(x83)*(x84)))+(((x82)*(x86)))+(((x82)*(x83)))+(((x81)*(x82)))+(((x81)*(x85)))+(((IkReal(-1.00000000000000))*(x78)*(x79)))+(((x85)*(x86)))+(((IkReal(-0.0109350000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x86)*(x87)))+(((IkReal(-1.00000000000000))*(x84)*(x86)))+(((x83)*(x85)))+(((IkReal(-0.00729000000000000))*(cj2)*(r02)))+(((x80)*(x84)))+(((x80)*(x87)))+(((IkReal(0.0238950000000000))*(r02)*(sj2)))+(((IkReal(0.145800000000000))*(x82)))+(((IkReal(0.145800000000000))*(x85)))+(((IkReal(-1.00000000000000))*(x77)*(x79)))));
IkReal x88=(sj2)*(sj2);
IkReal x89=(cj2)*(cj2);
IkReal x90=((IkReal(1.95730452674897))*(r02));
IkReal x91=((cj2)*(r02));
IkReal x92=((IkReal(26.0973936899863))*(sj0));
IkReal x93=((py)*(r02));
IkReal x94=((pz)*(r01));
IkReal x95=((cj0)*(px));
IkReal x96=((r02)*(sj2));
IkReal x97=((IkReal(13.3333333333333))*(cj2));
IkReal x98=((IkReal(20.0000000000000))*(sj0));
IkReal x99=((cj0)*(pz)*(r00));
IkReal x100=((IkReal(26.0973936899863))*(x88));
IkReal x101=((IkReal(26.0973936899863))*(x89));
IkReal x102=((IkReal(43.7037037037037))*(sj0)*(sj2));
dummyeval[0]=((((r02)*(x101)*(x95)))+(((IkReal(3.27777777777778))*(x96)))+(((x88)*(x92)*(x93)))+(((IkReal(-1.00000000000000))*(x102)*(x93)))+(((IkReal(-1.00000000000000))*(x94)*(x98)))+(((IkReal(-1.00000000000000))*(x89)*(x92)*(x94)))+(((IkReal(43.7037037037037))*(sj2)*(x99)))+(((IkReal(13.3333333333333))*(py)*(sj0)*(x91)))+(((x102)*(x94)))+(((x89)*(x92)*(x93)))+(((x93)*(x98)))+(((IkReal(-43.7037037037037))*(x95)*(x96)))+(((IkReal(-1.00000000000000))*(sj0)*(x94)*(x97)))+(((IkReal(-1.00000000000000))*(x101)*(x99)))+(((IkReal(-1.00000000000000))*(x91)))+(((IkReal(20.0000000000000))*(r02)*(x95)))+(((IkReal(-20.0000000000000))*(x99)))+(((IkReal(-1.00000000000000))*(x88)*(x92)*(x94)))+(((IkReal(-1.00000000000000))*(x100)*(x99)))+(((IkReal(13.3333333333333))*(x91)*(x95)))+(((IkReal(-1.00000000000000))*(x88)*(x90)))+(((r02)*(x100)*(x95)))+(((IkReal(-1.00000000000000))*(x97)*(x99)))+(((IkReal(-1.50000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x89)*(x90))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
if( IKabs(((gconst52)*(((((IkReal(0.0469935000000000))*(r02)*((sj2)*(sj2))))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(0.590000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0405000000000000))*(cj0)*(px)*(r02)))+(((IkReal(-0.0143370000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.295000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.0143370000000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(-0.0405000000000000))*(py)*(r02)*(sj0)))+(((IkReal(-0.540000000000000))*(r02)*((pz)*(pz))))+(((IkReal(-0.0442500000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.0460200000000000))*(cj2)*(r01)*(sj0)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(0.590000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.590000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(-0.540000000000000))*(py)*(pz)*(r01)))+(((IkReal(0.0405000000000000))*(pz)*(r01)*(sj0)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(0.590000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.540000000000000))*(px)*(pz)*(r00)))+(((IkReal(0.590000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.0426195000000000))*(cj2)*(r01)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(0.270000000000000))*(pp)*(r02)))+(((IkReal(0.590000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.0140400000000000))*(r01)*(sj0)*(sj2)))+(((IkReal(-0.0442500000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(-0.0460200000000000))*(cj0)*(cj2)*(r00)))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(0.0442500000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(0.590000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(-0.0286740000000000))*(cj2)*(r02)*(sj2)))+(((IkReal(0.0281745000000000))*(cj2)*(r02)))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj2)))+(((IkReal(0.590000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(-0.295000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(-0.0442500000000000))*(cj2)*(pz)*(r02)))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(-0.0442500000000000))*(cj2)*(py)*(r01)))+(((IkReal(0.0143370000000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(-0.0442500000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.00437400000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.0923497500000000))*(r02)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(0.0442500000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(-0.0143370000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.0140400000000000))*(cj0)*(r00)*(sj2)))+(((IkReal(-0.0135000000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(-0.0135000000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(0.0405000000000000))*(cj0)*(pz)*(r00)))+(((IkReal(0.0426195000000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(-0.295000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(0.0451575000000000))*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.0286740000000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(0.0442500000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0143370000000000))*(r02)*((sj2)*(sj2))))+(((IkReal(0.0150525000000000))*(r02)*(sj2)))+(((IkReal(0.0493387500000000))*(cj2)*(r02)))+(((IkReal(0.590000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(-0.295000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(-0.590000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(-0.590000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.540000000000000))*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(0.0405000000000000))*(px)*(r00)))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(0.0421200000000000))*(cj0)*(r00)))+(((IkReal(-0.0890310000000000))*(cj0)*(r00)*(sj2)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(0.590000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(-0.0426195000000000))*(cj2)*(r02)*(sj2)))+(((IkReal(-0.540000000000000))*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.00437400000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.0900000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(0.0442500000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.0469935000000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(0.0271620000000000))*(cj2)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(0.590000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(-0.0442500000000000))*(pz)*(r02)*(sj2)))+(((IkReal(0.0405000000000000))*(py)*(r01)))+(((IkReal(0.0135000000000000))*(cj2)*(py)*(r01)))+(((IkReal(0.0469935000000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.540000000000000))*(cj0)*(px)*(pz)*(r02)))+(((IkReal(-0.295000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.180000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.590000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.0442500000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(0.590000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(0.00437400000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(-0.540000000000000))*(cj0)*(r00)*((px)*(px))))+(((IkReal(-0.590000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(0.0271620000000000))*(cj0)*(cj2)*(r00)))+(((IkReal(0.590000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-0.0442500000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(0.0405000000000000))*(pz)*(r02)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(0.295000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(-0.0442500000000000))*(py)*(r01)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r02)))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(0.0421200000000000))*(r01)*(sj0)))+(((IkReal(0.0143370000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.270000000000000))*(cj0)*(pp)*(r00)))+(((IkReal(-0.540000000000000))*(cj0)*(px)*(py)*(r01)))+(((IkReal(-0.0890310000000000))*(r01)*(sj0)*(sj2)))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(-0.0442500000000000))*(px)*(r00)*(sj2)))+(((IkReal(-0.540000000000000))*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.270000000000000))*(pp)*(r01)*(sj0)))+(((IkReal(-0.0286740000000000))*(cj2)*(r01)*(sj0)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst52)*(((((IkReal(0.0469935000000000))*(r02)*((sj2)*(sj2))))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(0.590000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0405000000000000))*(cj0)*(px)*(r02)))+(((IkReal(-0.0143370000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.295000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.0143370000000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(-0.0405000000000000))*(py)*(r02)*(sj0)))+(((IkReal(-0.540000000000000))*(r02)*((pz)*(pz))))+(((IkReal(-0.0442500000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.0460200000000000))*(cj2)*(r01)*(sj0)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(0.590000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.590000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(-0.540000000000000))*(py)*(pz)*(r01)))+(((IkReal(0.0405000000000000))*(pz)*(r01)*(sj0)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(0.590000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.540000000000000))*(px)*(pz)*(r00)))+(((IkReal(0.590000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.0426195000000000))*(cj2)*(r01)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(0.270000000000000))*(pp)*(r02)))+(((IkReal(0.590000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.0140400000000000))*(r01)*(sj0)*(sj2)))+(((IkReal(-0.0442500000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(-0.0460200000000000))*(cj0)*(cj2)*(r00)))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(0.0442500000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(0.590000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(-0.0286740000000000))*(cj2)*(r02)*(sj2)))+(((IkReal(0.0281745000000000))*(cj2)*(r02)))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj2)))+(((IkReal(0.590000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(-0.295000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(-0.0442500000000000))*(cj2)*(pz)*(r02)))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(-0.0442500000000000))*(cj2)*(py)*(r01)))+(((IkReal(0.0143370000000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(-0.0442500000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.00437400000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.0923497500000000))*(r02)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(0.0442500000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(-0.0143370000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.0140400000000000))*(cj0)*(r00)*(sj2)))+(((IkReal(-0.0135000000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(-0.0135000000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(0.0405000000000000))*(cj0)*(pz)*(r00)))+(((IkReal(0.0426195000000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(-0.295000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(0.0451575000000000))*(r02)))))), ((gconst52)*(((((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.0286740000000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(0.0442500000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0143370000000000))*(r02)*((sj2)*(sj2))))+(((IkReal(0.0150525000000000))*(r02)*(sj2)))+(((IkReal(0.0493387500000000))*(cj2)*(r02)))+(((IkReal(0.590000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(-0.295000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(-0.590000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(-0.590000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.540000000000000))*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(0.0405000000000000))*(px)*(r00)))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(0.0421200000000000))*(cj0)*(r00)))+(((IkReal(-0.0890310000000000))*(cj0)*(r00)*(sj2)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(0.590000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(-0.0426195000000000))*(cj2)*(r02)*(sj2)))+(((IkReal(-0.540000000000000))*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.00437400000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.0900000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(0.0442500000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.0469935000000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(0.0271620000000000))*(cj2)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(0.590000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(-0.0442500000000000))*(pz)*(r02)*(sj2)))+(((IkReal(0.0405000000000000))*(py)*(r01)))+(((IkReal(0.0135000000000000))*(cj2)*(py)*(r01)))+(((IkReal(0.0469935000000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.540000000000000))*(cj0)*(px)*(pz)*(r02)))+(((IkReal(-0.295000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.180000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.590000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.0442500000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(0.590000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(0.00437400000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(-0.540000000000000))*(cj0)*(r00)*((px)*(px))))+(((IkReal(-0.590000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(0.0271620000000000))*(cj0)*(cj2)*(r00)))+(((IkReal(0.590000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-0.0442500000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(0.0405000000000000))*(pz)*(r02)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(0.295000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(-0.0442500000000000))*(py)*(r01)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r02)))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(0.0421200000000000))*(r01)*(sj0)))+(((IkReal(0.0143370000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.270000000000000))*(cj0)*(pp)*(r00)))+(((IkReal(-0.540000000000000))*(cj0)*(px)*(py)*(r01)))+(((IkReal(-0.0890310000000000))*(r01)*(sj0)*(sj2)))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(-0.0442500000000000))*(px)*(r00)*(sj2)))+(((IkReal(-0.540000000000000))*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.270000000000000))*(pp)*(r01)*(sj0)))+(((IkReal(-0.0286740000000000))*(cj2)*(r01)*(sj0)*(sj2)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x103=IKcos(j1);
IkReal x104=IKsin(j1);
IkReal x105=((IkReal(0.180000000000000))*(sj2));
IkReal x106=((py)*(sj0));
IkReal x107=((IkReal(0.0900000000000000))*(cj2));
IkReal x108=((cj0)*(px));
IkReal x109=((IkReal(0.295000000000000))*(sj2));
IkReal x110=((r01)*(sj0));
IkReal x111=((cj0)*(r00));
IkReal x112=((cj2)*(x103));
IkReal x113=((pz)*(x104));
IkReal x114=((IkReal(0.540000000000000))*(x104));
IkReal x115=((r02)*(x103));
IkReal x116=((cj2)*(x104));
IkReal x117=((sj2)*(x103));
IkReal x118=((sj2)*(x104));
IkReal x119=((x104)*(x111));
IkReal x120=((x104)*(x110));
IkReal x121=((IkReal(0.590000000000000))*(x118));
evalcond[0]=((IkReal(0.167250000000000))+(((IkReal(-0.0442500000000000))*(x118)))+(((IkReal(0.540000000000000))*(pz)*(x103)))+(((IkReal(0.0405000000000000))*(x104)))+(((IkReal(-1.00000000000000))*(x103)*(x105)*(x106)))+(((IkReal(-1.00000000000000))*(x103)*(x105)*(x108)))+(((IkReal(-0.590000000000000))*(x106)*(x112)))+(((x106)*(x121)))+(((IkReal(0.0442500000000000))*(x112)))+(((x108)*(x121)))+(((IkReal(-1.00000000000000))*(x108)*(x114)))+(((IkReal(-1.00000000000000))*(x106)*(x114)))+(((IkReal(-0.180000000000000))*(x106)*(x116)))+(((IkReal(0.0135000000000000))*(x117)))+(((IkReal(0.0135000000000000))*(x116)))+(pp)+(((IkReal(-0.150000000000000))*(x106)))+(((IkReal(-0.150000000000000))*(x108)))+(((IkReal(-1.00000000000000))*(x105)*(x113)))+(((IkReal(-0.590000000000000))*(x108)*(x112)))+(((IkReal(0.0486000000000000))*(cj2)))+(((IkReal(-0.180000000000000))*(x108)*(x116)))+(((IkReal(-0.590000000000000))*(pz)*(x117)))+(((IkReal(0.180000000000000))*(pz)*(x112)))+(((IkReal(-0.590000000000000))*(cj2)*(x113)))+(((IkReal(-0.159300000000000))*(sj2))));
evalcond[1]=((((IkReal(-0.0900000000000000))*(x111)*(x117)))+(((IkReal(-0.295000000000000))*(x111)*(x112)))+(((IkReal(-0.0900000000000000))*(x110)*(x117)))+(((px)*(r00)))+(((IkReal(-0.270000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(x107)*(x119)))+(((IkReal(-0.0750000000000000))*(x110)))+(((IkReal(-0.0750000000000000))*(x111)))+(((IkReal(-0.270000000000000))*(x120)))+(((x109)*(x119)))+(((IkReal(0.270000000000000))*(x115)))+(((IkReal(-0.0900000000000000))*(r02)*(x118)))+(((IkReal(-1.00000000000000))*(x107)*(x120)))+(((IkReal(-0.295000000000000))*(r02)*(x116)))+(((IkReal(-1.00000000000000))*(x109)*(x115)))+(((IkReal(-0.295000000000000))*(x110)*(x112)))+(((x109)*(x120)))+(((pz)*(r02)))+(((py)*(r01)))+(((x107)*(x115))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x122=((cj1)*(cj2));
IkReal x123=((sj1)*(sj2));
IkReal x124=((IkReal(12.5000000000000))*(pz));
IkReal x125=((IkReal(12.5000000000000))*(cj0)*(px));
IkReal x126=((IkReal(12.5000000000000))*(py)*(sj0));
cj4array[0]=((IkReal(-3.68750000000000))+(((IkReal(-0.937500000000000))*(x122)))+(((IkReal(-1.00000000000000))*(x123)*(x126)))+(((IkReal(-1.00000000000000))*(x123)*(x125)))+(((IkReal(0.937500000000000))*(x123)))+(((IkReal(3.37500000000000))*(sj2)))+(((x122)*(x126)))+(((x122)*(x125)))+(((cj1)*(sj2)*(x124)))+(((cj2)*(sj1)*(x124))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x127=((pz)*(sj2));
IkReal x128=((IkReal(1.00000000000000))*(cj1));
IkReal x129=((cj2)*(sj0));
IkReal x130=((py)*(sj1));
IkReal x131=((cj0)*(sj1));
IkReal x132=((cj2)*(px));
IkReal x133=((sj0)*(sj2));
IkReal x134=((IkReal(0.0750000000000000))*(cj2));
IkReal x135=((px)*(sj2));
IkReal x136=((IkReal(0.0750000000000000))*(sj2));
IkReal x137=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x128)))+(((r01)*(sj1)*(x133)))+(((IkReal(-1.00000000000000))*(r01)*(x128)*(x129)))+(((r00)*(sj2)*(x131)))+(((IkReal(-1.00000000000000))*(r02)*(x137)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x128))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x129)*(x130)))+(((IkReal(-1.00000000000000))*(sj1)*(x134)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x135)))+(((x131)*(x132)))+(((cj1)*(py)*(x133)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x128)))+(((IkReal(-1.00000000000000))*(cj1)*(x136)))+(((sj1)*(x127))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x137)))+(((IkReal(-1.00000000000000))*(py)*(x128)*(x129)))+(((IkReal(-1.00000000000000))*(sj1)*(x136)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x128)*(x132)))+(((x131)*(x135)))+(((x130)*(x133)))+(((IkReal(-1.00000000000000))*(x127)*(x128)))+(((cj1)*(x134))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst63;
gconst63=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst64;
gconst64=IkReal(-2.00000000000000);
IkReal gconst65;
gconst65=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst66;
IkReal x138=((cj1)*(sj2));
IkReal x139=((r01)*(sj0));
IkReal x140=((cj0)*(r00));
IkReal x141=((cj2)*(sj1));
gconst66=((IkReal(-1.00000000000000))+(((x138)*(x139)))+(((x140)*(x141)))+(((x138)*(x140)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x139)*(x141))));
IkReal gconst67;
IkReal x142=((cj1)*(sj2));
IkReal x143=((r01)*(sj0));
IkReal x144=((cj0)*(r00));
IkReal x145=((cj2)*(sj1));
gconst67=((IkReal(1.00000000000000))+(((x142)*(x143)))+(((x142)*(x144)))+(((x144)*(x145)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x143)*(x145))));
IkReal gconst68;
gconst68=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst69;
gconst69=IkReal(-2.00000000000000);
IkReal gconst70;
gconst70=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst71;
IkReal x146=((cj1)*(sj2));
IkReal x147=((r01)*(sj0));
IkReal x148=((cj0)*(r00));
IkReal x149=((cj2)*(sj1));
gconst71=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x147)*(x149)))+(((x148)*(x149)))+(((x146)*(x148)))+(((x146)*(x147)))+(((r02)*(sj1)*(sj2))));
IkReal gconst72;
IkReal x150=((cj1)*(sj2));
IkReal x151=((r01)*(sj0));
IkReal x152=((cj0)*(r00));
IkReal x153=((cj2)*(sj1));
gconst72=((IkReal(1.00000000000000))+(((x150)*(x152)))+(((x150)*(x151)))+(((x152)*(x153)))+(((x151)*(x153)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(4.00000000000000);
IkReal gconst76;
gconst76=IkReal(-2.00000000000000);
IkReal gconst77;
gconst77=IkReal(2.00000000000000);
IkReal gconst78;
gconst78=IkReal(4.00000000000000);
IkReal gconst79;
gconst79=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst80;
gconst80=IkReal(2.00000000000000);
IkReal gconst81;
gconst81=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst82;
IkReal x154=((cj1)*(sj2));
IkReal x155=((r01)*(sj0));
IkReal x156=((cj0)*(r00));
IkReal x157=((cj2)*(sj1));
gconst82=((IkReal(1.00000000000000))+(((x155)*(x157)))+(((x154)*(x156)))+(((x154)*(x155)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x156)*(x157)))+(((r02)*(sj1)*(sj2))));
IkReal gconst83;
IkReal x158=((cj1)*(sj2));
IkReal x159=((r01)*(sj0));
IkReal x160=((cj0)*(r00));
IkReal x161=((cj2)*(sj1));
gconst83=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x158)*(x159)))+(((x158)*(x160)))+(((r02)*(sj1)*(sj2)))+(((x159)*(x161)))+(((x160)*(x161))));
IkReal gconst84;
gconst84=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst85;
gconst85=IkReal(2.00000000000000);
IkReal gconst86;
gconst86=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst87;
IkReal x162=((cj1)*(sj2));
IkReal x163=((r01)*(sj0));
IkReal x164=((cj0)*(r00));
IkReal x165=((cj2)*(sj1));
gconst87=((IkReal(1.00000000000000))+(((x162)*(x163)))+(((x162)*(x164)))+(((x164)*(x165)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x163)*(x165))));
IkReal gconst88;
IkReal x166=((cj1)*(sj2));
IkReal x167=((r01)*(sj0));
IkReal x168=((cj0)*(r00));
IkReal x169=((cj2)*(sj1));
gconst88=((IkReal(-1.00000000000000))+(((x168)*(x169)))+(((x167)*(x169)))+(((x166)*(x167)))+(((x166)*(x168)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal x170=((IkReal(1.00000000000000))*(gconst87));
IkReal x171=((gconst81)*(gconst82));
IkReal x172=((gconst84)*(gconst88));
IkReal x173=((gconst79)*(gconst83));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst80)*(gconst83)*(gconst85)*(x170)))+(((x171)*(x172)))+(((gconst86)*(gconst87)*(x173)))+(((IkReal(-1.00000000000000))*(x172)*(x173)))+(((IkReal(-1.00000000000000))*(gconst86)*(x170)*(x171))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x174=((IkReal(1.00000000000000))*(gconst87));
IkReal x175=((gconst66)*(gconst70));
IkReal x176=((gconst71)*(gconst83));
IkReal x177=((gconst80)*(gconst85));
IkReal x178=((gconst77)*(gconst79));
IkReal x179=((gconst67)*(gconst79));
IkReal x180=((gconst63)*(gconst83));
IkReal x181=((gconst72)*(gconst84));
IkReal x182=((gconst65)*(gconst66));
IkReal x183=((gconst73)*(gconst83));
IkReal x184=((gconst77)*(gconst87));
IkReal x185=((gconst74)*(gconst82));
IkReal x186=((gconst68)*(gconst88));
IkReal x187=((gconst63)*(gconst77));
IkReal x188=((gconst72)*(gconst76));
IkReal x189=((gconst78)*(gconst84));
IkReal x190=((gconst80)*(gconst83));
IkReal x191=((IkReal(1.00000000000000))*(gconst67));
IkReal x192=((gconst64)*(gconst85));
IkReal x193=((gconst64)*(gconst69));
IkReal x194=((gconst70)*(gconst87));
IkReal x195=((gconst65)*(gconst75));
IkReal x196=((gconst69)*(gconst87));
IkReal x197=((gconst71)*(gconst86));
IkReal x198=((gconst84)*(gconst88));
IkReal x199=((IkReal(1.00000000000000))*(gconst81));
IkReal x200=((gconst81)*(gconst82));
IkReal x201=((gconst76)*(gconst88));
IkReal x202=((gconst65)*(gconst82));
IkReal x203=((gconst70)*(gconst71));
IkReal x204=((gconst85)*(gconst87));
IkReal x205=((gconst79)*(gconst83));
IkReal x206=((gconst68)*(gconst78));
IkReal x207=((gconst64)*(gconst83));
IkReal x208=((gconst66)*(gconst74));
IkReal x209=((gconst76)*(gconst78));
IkReal x210=((gconst67)*(gconst80));
IkReal x211=((gconst67)*(gconst87));
IkReal x212=((gconst73)*(gconst86));
IkReal x213=((gconst75)*(gconst81));
IkReal x214=((gconst66)*(gconst81));
IkReal x215=((gconst71)*(gconst85));
IkReal x216=((gconst63)*(gconst86));
IkReal x217=((gconst67)*(gconst73));
IkReal x218=((gconst64)*(gconst67));
IkReal x219=((gconst69)*(gconst71));
IkReal x220=((gconst74)*(gconst75));
IkReal x221=((gconst63)*(gconst67));
IkReal x222=((gconst86)*(gconst87));
IkReal x223=((IkReal(1.00000000000000))*(gconst71));
IkReal x224=((gconst68)*(gconst72));
IkReal x225=((gconst71)*(gconst77));
IkReal x226=((gconst67)*(gconst71));
IkReal x227=((IkReal(1.00000000000000))*(x224));
IkReal x228=((IkReal(1.00000000000000))*(x205));
IkReal x229=((gconst82)*(x224));
IkReal x230=((gconst77)*(x223));
op[0]=((((x205)*(x222)))+(((x198)*(x200)))+(((IkReal(-1.00000000000000))*(gconst86)*(x174)*(x200)))+(((IkReal(-1.00000000000000))*(x198)*(x228)))+(((IkReal(-1.00000000000000))*(gconst83)*(x174)*(x177))));
op[1]=((((IkReal(-1.00000000000000))*(gconst86)*(x174)*(x185)))+(((x204)*(x213)))+(((gconst83)*(gconst87)*(x178)))+(((x200)*(x201)))+(((x189)*(x190)))+(((x185)*(x198)))+(((IkReal(-1.00000000000000))*(x183)*(x198)))+(((IkReal(-1.00000000000000))*(x201)*(x228)))+(((x183)*(x222)))+(((IkReal(-1.00000000000000))*(gconst77)*(x174)*(x200))));
op[2]=((((x198)*(x202)))+(((IkReal(-1.00000000000000))*(x186)*(x228)))+(((IkReal(-1.00000000000000))*(gconst83)*(x174)*(x192)))+(((x194)*(x205)))+(((IkReal(-1.00000000000000))*(gconst75)*(x189)*(x199)))+(((IkReal(-1.00000000000000))*(gconst77)*(x174)*(x185)))+(((x185)*(x201)))+(((x190)*(x209)))+(((x180)*(x222)))+(((x198)*(x214)))+(((x204)*(x220)))+(((IkReal(-1.00000000000000))*(x181)*(x228)))+(((IkReal(-1.00000000000000))*(gconst82)*(x197)*(x199)))+(((IkReal(-1.00000000000000))*(x180)*(x198)))+(((gconst79)*(gconst86)*(x176)))+(((IkReal(-1.00000000000000))*(gconst86)*(x174)*(x214)))+(((IkReal(-1.00000000000000))*(gconst86)*(x174)*(x202)))+(((IkReal(-1.00000000000000))*(gconst70)*(x174)*(x200)))+(((IkReal(-1.00000000000000))*(gconst67)*(x174)*(x177)))+(((x179)*(x222)))+(((x181)*(x200)))+(((IkReal(-1.00000000000000))*(x176)*(x177)))+(((IkReal(-1.00000000000000))*(x183)*(x201)))+(((x183)*(x184)))+(((IkReal(-1.00000000000000))*(gconst69)*(x174)*(x190)))+(((x186)*(x200)))+(((IkReal(-1.00000000000000))*(x179)*(x198))));
op[3]=((((x198)*(x208)))+(((IkReal(-1.00000000000000))*(x189)*(x220)))+(((IkReal(-1.00000000000000))*(gconst75)*(x199)*(x209)))+(((IkReal(-1.00000000000000))*(x185)*(x197)))+(((x180)*(x184)))+(((x176)*(x178)))+(((x213)*(x215)))+(((IkReal(-1.00000000000000))*(gconst70)*(x174)*(x185)))+(((x195)*(x204)))+(((x201)*(x202)))+(((x189)*(x207)))+(((x190)*(x206)))+(((x185)*(x186)))+(((x211)*(x212)))+(((x176)*(x212)))+(((x188)*(x200)))+(((IkReal(-1.00000000000000))*(x188)*(x228)))+(((x196)*(x213)))+(((IkReal(-1.00000000000000))*(x183)*(x186)))+(((IkReal(-1.00000000000000))*(gconst77)*(x174)*(x202)))+(((x189)*(x210)))+(((IkReal(-1.00000000000000))*(x181)*(x183)))+(((IkReal(-1.00000000000000))*(gconst86)*(x174)*(x208)))+(((IkReal(-1.00000000000000))*(gconst77)*(x174)*(x214)))+(((IkReal(-1.00000000000000))*(x180)*(x201)))+(((IkReal(-1.00000000000000))*(x179)*(x201)))+(((IkReal(-1.00000000000000))*(gconst82)*(x199)*(x225)))+(((x178)*(x211)))+(((x181)*(x185)))+(((x183)*(x194)))+(((IkReal(-1.00000000000000))*(gconst73)*(x191)*(x198)))+(((x201)*(x214))));
op[4]=((((IkReal(-1.00000000000000))*(gconst63)*(x191)*(x198)))+(((IkReal(-1.00000000000000))*(x205)*(x227)))+(((x184)*(x217)))+(((IkReal(-1.00000000000000))*(gconst86)*(x174)*(x182)))+(((x196)*(x220)))+(((IkReal(-1.00000000000000))*(gconst75)*(x199)*(x206)))+(((IkReal(-1.00000000000000))*(gconst83)*(x174)*(x193)))+(((x179)*(x197)))+(((x179)*(x194)))+(((x180)*(x194)))+(((x215)*(x220)))+(((IkReal(-1.00000000000000))*(x185)*(x230)))+(((gconst70)*(gconst79)*(x176)))+(((x201)*(x208)))+(((x181)*(x214)))+(((x185)*(x188)))+(((IkReal(-1.00000000000000))*(x189)*(x195)))+(((x211)*(x216)))+(((IkReal(-1.00000000000000))*(gconst73)*(x191)*(x201)))+(((IkReal(-1.00000000000000))*(x197)*(x202)))+(((IkReal(-1.00000000000000))*(gconst69)*(x174)*(x210)))+(((x176)*(x216)))+(((IkReal(-1.00000000000000))*(x180)*(x186)))+(((IkReal(-1.00000000000000))*(x180)*(x181)))+(((x209)*(x210)))+(((IkReal(-1.00000000000000))*(gconst82)*(x199)*(x203)))+(((gconst73)*(gconst77)*(x176)))+(((IkReal(-1.00000000000000))*(gconst67)*(x174)*(x192)))+(((IkReal(-1.00000000000000))*(x183)*(x188)))+(((x186)*(x214)))+(((IkReal(-1.00000000000000))*(x176)*(x192)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst80)*(x176)))+(((IkReal(-1.00000000000000))*(gconst77)*(x174)*(x208)))+(((IkReal(-1.00000000000000))*(gconst66)*(x197)*(x199)))+(((x200)*(x224)))+(((IkReal(-1.00000000000000))*(gconst70)*(x174)*(x202)))+(((x181)*(x202)))+(((IkReal(-1.00000000000000))*(x179)*(x186)))+(((IkReal(-1.00000000000000))*(x179)*(x181)))+(((x182)*(x198)))+(((IkReal(-1.00000000000000))*(gconst71)*(x177)*(x191)))+(((IkReal(-1.00000000000000))*(x209)*(x220)))+(((IkReal(-1.00000000000000))*(gconst81)*(x174)*(x175)))+(((x186)*(x202)))+(((x207)*(x209))));
op[5]=((((x185)*(x224)))+(((x176)*(x187)))+(((x184)*(x221)))+(((x197)*(x217)))+(((IkReal(-1.00000000000000))*(x206)*(x220)))+(((x213)*(x219)))+(((x206)*(x210)))+(((IkReal(-1.00000000000000))*(x202)*(x230)))+(((IkReal(-1.00000000000000))*(gconst77)*(x174)*(x182)))+(((gconst70)*(gconst73)*(x176)))+(((IkReal(-1.00000000000000))*(x197)*(x208)))+(((x188)*(x202)))+(((IkReal(-1.00000000000000))*(x180)*(x188)))+(((IkReal(-1.00000000000000))*(x183)*(x227)))+(((IkReal(-1.00000000000000))*(gconst73)*(x186)*(x191)))+(((x195)*(x196)))+(((IkReal(-1.00000000000000))*(gconst74)*(x174)*(x175)))+(((IkReal(-1.00000000000000))*(x185)*(x203)))+(((IkReal(-1.00000000000000))*(x195)*(x209)))+(((x189)*(x218)))+(((IkReal(-1.00000000000000))*(gconst73)*(x181)*(x191)))+(((IkReal(-1.00000000000000))*(gconst66)*(x199)*(x225)))+(((x188)*(x214)))+(((x178)*(x226)))+(((IkReal(-1.00000000000000))*(gconst63)*(x191)*(x201)))+(((x182)*(x201)))+(((x181)*(x208)))+(((IkReal(-1.00000000000000))*(x179)*(x188)))+(((x194)*(x217)))+(((x206)*(x207)))+(((x186)*(x208)))+(((x195)*(x215))));
op[6]=((((x194)*(x221)))+(((IkReal(-1.00000000000000))*(x180)*(x227)))+(((IkReal(-1.00000000000000))*(gconst63)*(x186)*(x191)))+(((IkReal(-1.00000000000000))*(gconst65)*(x174)*(x175)))+(((x214)*(x224)))+(((x202)*(x224)))+(((IkReal(-1.00000000000000))*(x179)*(x227)))+(((x188)*(x208)))+(((x209)*(x218)))+(((gconst63)*(gconst70)*(x176)))+(((x219)*(x220)))+(((IkReal(-1.00000000000000))*(gconst73)*(x188)*(x191)))+(((IkReal(-1.00000000000000))*(x208)*(x230)))+(((IkReal(-1.00000000000000))*(gconst67)*(x174)*(x193)))+(((x179)*(x203)))+(((IkReal(-1.00000000000000))*(gconst71)*(x175)*(x199)))+(((IkReal(-1.00000000000000))*(gconst63)*(x181)*(x191)))+(((IkReal(-1.00000000000000))*(x202)*(x203)))+(((IkReal(-1.00000000000000))*(x176)*(x193)))+(((IkReal(-1.00000000000000))*(x195)*(x206)))+(((x217)*(x225)))+(((x197)*(x221)))+(((IkReal(-1.00000000000000))*(gconst80)*(x191)*(x219)))+(((IkReal(-1.00000000000000))*(x182)*(x197)))+(((IkReal(-1.00000000000000))*(gconst71)*(x191)*(x192)))+(((x182)*(x186)))+(((x181)*(x182))));
op[7]=((((IkReal(-1.00000000000000))*(gconst74)*(x175)*(x223)))+(((x206)*(x218)))+(((IkReal(-1.00000000000000))*(gconst63)*(x188)*(x191)))+(((x208)*(x224)))+(((x203)*(x217)))+(((x187)*(x226)))+(((IkReal(-1.00000000000000))*(x182)*(x230)))+(((x182)*(x188)))+(((IkReal(-1.00000000000000))*(gconst73)*(x191)*(x224)))+(((x195)*(x219))));
op[8]=((((x203)*(x221)))+(((IkReal(-1.00000000000000))*(gconst63)*(x191)*(x224)))+(((IkReal(-1.00000000000000))*(gconst65)*(x175)*(x223)))+(((x182)*(x224)))+(((IkReal(-1.00000000000000))*(gconst71)*(x191)*(x193))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst105;
gconst105=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x231=((cj3)*(sj1));
IkReal x232=((r02)*(sj2));
IkReal x233=((r01)*(sj0));
IkReal x234=((IkReal(1.00000000000000))*(cj3));
IkReal x235=((cj1)*(sj2));
IkReal x236=((IkReal(1.00000000000000))*(sj3));
IkReal x237=((cj0)*(r00));
IkReal x238=((cj0)*(r01));
IkReal x239=((r00)*(sj0));
IkReal x240=((cj2)*(x237));
IkReal x241=((sj1)*(x236));
IkReal x242=((cj1)*(cj2)*(r02));
if( IKabs(((gconst105)*(((((IkReal(-1.00000000000000))*(x233)*(x235)*(x236)))+(((cj3)*(x239)))+(((sj3)*(x242)))+(((IkReal(-1.00000000000000))*(x232)*(x241)))+(((IkReal(-1.00000000000000))*(x235)*(x236)*(x237)))+(((IkReal(-1.00000000000000))*(x240)*(x241)))+(((IkReal(-1.00000000000000))*(x234)*(x238)))+(((IkReal(-1.00000000000000))*(cj2)*(x233)*(x241))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst105)*(((((IkReal(-1.00000000000000))*(x234)*(x242)))+(((cj2)*(x231)*(x233)))+(((x231)*(x240)))+(((cj3)*(x235)*(x237)))+(((IkReal(-1.00000000000000))*(x236)*(x238)))+(((x231)*(x232)))+(((cj3)*(x233)*(x235)))+(((sj3)*(x239))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst105)*(((((IkReal(-1.00000000000000))*(x233)*(x235)*(x236)))+(((cj3)*(x239)))+(((sj3)*(x242)))+(((IkReal(-1.00000000000000))*(x232)*(x241)))+(((IkReal(-1.00000000000000))*(x235)*(x236)*(x237)))+(((IkReal(-1.00000000000000))*(x240)*(x241)))+(((IkReal(-1.00000000000000))*(x234)*(x238)))+(((IkReal(-1.00000000000000))*(cj2)*(x233)*(x241)))))), ((gconst105)*(((((IkReal(-1.00000000000000))*(x234)*(x242)))+(((cj2)*(x231)*(x233)))+(((x231)*(x240)))+(((cj3)*(x235)*(x237)))+(((IkReal(-1.00000000000000))*(x236)*(x238)))+(((x231)*(x232)))+(((cj3)*(x233)*(x235)))+(((sj3)*(x239)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x243=IKsin(j5);
IkReal x244=IKcos(j5);
IkReal x245=((IkReal(1.00000000000000))*(cj3));
IkReal x246=((cj1)*(sj2));
IkReal x247=((r01)*(sj0));
IkReal x248=((cj0)*(r00));
IkReal x249=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x244)))+(((IkReal(-1.00000000000000))*(x243)*(x245)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x248)*(x249)))+(((sj3)*(x243)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x246)*(x248)))+(((x246)*(x247)))+(((x247)*(x249)))+(((IkReal(-1.00000000000000))*(x244)*(x245))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x250=((pz)*(sj2));
IkReal x251=((IkReal(1.00000000000000))*(cj1));
IkReal x252=((cj2)*(sj0));
IkReal x253=((py)*(sj1));
IkReal x254=((cj0)*(sj1));
IkReal x255=((cj2)*(px));
IkReal x256=((sj0)*(sj2));
IkReal x257=((IkReal(0.0750000000000000))*(cj2));
IkReal x258=((px)*(sj2));
IkReal x259=((IkReal(0.0750000000000000))*(sj2));
IkReal x260=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x251)))+(((IkReal(-1.00000000000000))*(r02)*(x260)))+(((r01)*(sj1)*(x256)))+(((IkReal(-1.00000000000000))*(r01)*(x251)*(x252)))+(((r00)*(sj2)*(x254)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x251))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x250)))+(((cj1)*(py)*(x256)))+(((IkReal(-1.00000000000000))*(cj1)*(x259)))+(((IkReal(-1.00000000000000))*(sj1)*(x257)))+(((x252)*(x253)))+(((x254)*(x255)))+(((cj0)*(cj1)*(x258)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x251))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(py)*(x251)*(x252)))+(((IkReal(-1.00000000000000))*(x250)*(x251)))+(((cj1)*(x257)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x253)*(x256)))+(((IkReal(-1.00000000000000))*(cj0)*(x251)*(x255)))+(((IkReal(-1.00000000000000))*(sj1)*(x259)))+(((x254)*(x258)))+(((IkReal(-1.00000000000000))*(pz)*(x260))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst106;
gconst106=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst107;
gconst107=IkReal(-2.00000000000000);
IkReal gconst108;
gconst108=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst109;
IkReal x261=((cj1)*(sj2));
IkReal x262=((r01)*(sj0));
IkReal x263=((cj0)*(r00));
IkReal x264=((cj2)*(sj1));
gconst109=((IkReal(1.00000000000000))+(((x262)*(x264)))+(((x263)*(x264)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x261)*(x263)))+(((x261)*(x262)))+(((r02)*(sj1)*(sj2))));
IkReal gconst110;
IkReal x265=((cj1)*(sj2));
IkReal x266=((r01)*(sj0));
IkReal x267=((cj0)*(r00));
IkReal x268=((cj2)*(sj1));
gconst110=((IkReal(-1.00000000000000))+(((x267)*(x268)))+(((x266)*(x268)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x265)*(x266)))+(((x265)*(x267))));
IkReal gconst111;
gconst111=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst112;
gconst112=IkReal(-2.00000000000000);
IkReal gconst113;
gconst113=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst114;
IkReal x269=((cj1)*(sj2));
IkReal x270=((r01)*(sj0));
IkReal x271=((cj0)*(r00));
IkReal x272=((cj2)*(sj1));
gconst114=((IkReal(1.00000000000000))+(((x271)*(x272)))+(((x270)*(x272)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x269)*(x270)))+(((x269)*(x271)))+(((r02)*(sj1)*(sj2))));
IkReal gconst115;
IkReal x273=((cj1)*(sj2));
IkReal x274=((r01)*(sj0));
IkReal x275=((cj0)*(r00));
IkReal x276=((cj2)*(sj1));
gconst115=((IkReal(-1.00000000000000))+(((x273)*(x274)))+(((x273)*(x275)))+(((x275)*(x276)))+(((x274)*(x276)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal gconst116;
gconst116=IkReal(2.00000000000000);
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=IkReal(4.00000000000000);
IkReal gconst119;
gconst119=IkReal(2.00000000000000);
IkReal gconst120;
gconst120=IkReal(-2.00000000000000);
IkReal gconst121;
gconst121=IkReal(4.00000000000000);
IkReal gconst122;
gconst122=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst123;
gconst123=IkReal(2.00000000000000);
IkReal gconst124;
gconst124=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst125;
IkReal x277=((cj1)*(sj2));
IkReal x278=((r01)*(sj0));
IkReal x279=((cj0)*(r00));
IkReal x280=((cj2)*(sj1));
gconst125=((IkReal(-1.00000000000000))+(((x279)*(x280)))+(((x278)*(x280)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x277)*(x279)))+(((x277)*(x278)))+(((r02)*(sj1)*(sj2))));
IkReal gconst126;
IkReal x281=((cj1)*(sj2));
IkReal x282=((r01)*(sj0));
IkReal x283=((cj0)*(r00));
IkReal x284=((cj2)*(sj1));
gconst126=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x281)*(x282)))+(((x281)*(x283)))+(((x282)*(x284)))+(((x283)*(x284))));
IkReal gconst127;
gconst127=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst128;
gconst128=IkReal(2.00000000000000);
IkReal gconst129;
gconst129=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst130;
IkReal x285=((cj1)*(sj2));
IkReal x286=((r01)*(sj0));
IkReal x287=((cj0)*(r00));
IkReal x288=((cj2)*(sj1));
gconst130=((IkReal(-1.00000000000000))+(((x285)*(x287)))+(((x285)*(x286)))+(((x286)*(x288)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x287)*(x288))));
IkReal gconst131;
IkReal x289=((cj1)*(sj2));
IkReal x290=((r01)*(sj0));
IkReal x291=((cj0)*(r00));
IkReal x292=((cj2)*(sj1));
gconst131=((IkReal(1.00000000000000))+(((x289)*(x290)))+(((x289)*(x291)))+(((x291)*(x292)))+(((x290)*(x292)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal x293=((gconst129)*(gconst130));
IkReal x294=((gconst122)*(gconst126));
IkReal x295=((gconst124)*(gconst125));
IkReal x296=((gconst127)*(gconst131));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x293)*(x295)))+(((IkReal(-1.00000000000000))*(x294)*(x296)))+(((x295)*(x296)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst126)*(gconst128)*(gconst130)))+(((x293)*(x294))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x297=((IkReal(1.00000000000000))*(gconst127));
IkReal x298=((gconst106)*(gconst110));
IkReal x299=((gconst118)*(gconst124));
IkReal x300=((gconst109)*(gconst115));
IkReal x301=((gconst119)*(gconst124));
IkReal x302=((gconst110)*(gconst116));
IkReal x303=((gconst111)*(gconst131));
IkReal x304=((IkReal(1.00000000000000))*(gconst130));
IkReal x305=((gconst123)*(gconst126));
IkReal x306=((gconst111)*(gconst117));
IkReal x307=((gconst108)*(gconst119));
IkReal x308=((gconst114)*(gconst120));
IkReal x309=((IkReal(1.00000000000000))*(gconst109));
IkReal x310=((gconst125)*(gconst129));
IkReal x311=((gconst109)*(gconst113));
IkReal x312=((gconst110)*(gconst122));
IkReal x313=((gconst109)*(gconst131));
IkReal x314=((gconst108)*(gconst127));
IkReal x315=((gconst120)*(gconst130));
IkReal x316=((gconst106)*(gconst126));
IkReal x317=((gconst124)*(gconst125));
IkReal x318=((gconst110)*(gconst112));
IkReal x319=((gconst113)*(gconst130));
IkReal x320=((gconst122)*(gconst126));
IkReal x321=((gconst128)*(gconst130));
IkReal x322=((gconst111)*(gconst121));
IkReal x323=((gconst118)*(gconst121));
IkReal x324=((gconst129)*(gconst130));
IkReal x325=((gconst124)*(gconst127));
IkReal x326=((gconst108)*(gconst125));
IkReal x327=((gconst126)*(gconst128));
IkReal x328=((gconst108)*(gconst118));
IkReal x329=((gconst112)*(gconst130));
IkReal x330=((gconst114)*(gconst128));
IkReal x331=((gconst125)*(gconst131));
IkReal x332=((IkReal(1.00000000000000))*(gconst115));
IkReal x333=((gconst114)*(gconst129));
IkReal x334=((gconst113)*(gconst114));
IkReal x335=((gconst112)*(gconst114));
IkReal x336=((gconst110)*(gconst123));
IkReal x337=((gconst121)*(gconst127));
IkReal x338=((gconst115)*(gconst125));
IkReal x339=((gconst116)*(gconst126));
IkReal x340=((gconst117)*(gconst119));
IkReal x341=((gconst117)*(gconst125));
IkReal x342=((gconst109)*(gconst117));
IkReal x343=((gconst112)*(gconst126));
IkReal x344=((gconst109)*(gconst124));
IkReal x345=((IkReal(1.00000000000000))*(gconst114));
IkReal x346=((gconst117)*(gconst118));
IkReal x347=((gconst111)*(gconst115));
IkReal x348=((gconst107)*(gconst126));
IkReal x349=((gconst107)*(gconst110));
IkReal x350=((gconst117)*(gconst127));
IkReal x351=((gconst119)*(gconst121));
IkReal x352=((gconst108)*(gconst109));
IkReal x353=((gconst107)*(x345));
IkReal x354=((IkReal(1.00000000000000))*(gconst119)*(gconst131));
op[0]=((((gconst127)*(gconst131)*(x317)))+(((x320)*(x324)))+(((IkReal(-1.00000000000000))*(gconst131)*(x297)*(x320)))+(((IkReal(-1.00000000000000))*(gconst128)*(x304)*(x305)))+(((IkReal(-1.00000000000000))*(gconst124)*(x304)*(x310))));
op[1]=((((IkReal(-1.00000000000000))*(gconst131)*(x297)*(x339)))+(((x331)*(x350)))+(((x324)*(x339)))+(((IkReal(-1.00000000000000))*(x320)*(x354)))+(((IkReal(-1.00000000000000))*(gconst117)*(x304)*(x310)))+(((x305)*(x337)))+(((x315)*(x320)))+(((x299)*(x321)))+(((x301)*(x331)))+(((IkReal(-1.00000000000000))*(gconst120)*(x304)*(x317))));
op[2]=((((IkReal(-1.00000000000000))*(gconst121)*(x297)*(x299)))+(((gconst115)*(gconst127)*(x317)))+(((IkReal(-1.00000000000000))*(gconst108)*(x304)*(x310)))+(((IkReal(-1.00000000000000))*(x303)*(x320)))+(((IkReal(-1.00000000000000))*(gconst112)*(x304)*(x305)))+(((x305)*(x351)))+(((x313)*(x325)))+(((IkReal(-1.00000000000000))*(gconst128)*(x304)*(x336)))+(((x312)*(x324)))+(((IkReal(-1.00000000000000))*(x339)*(x354)))+(((x331)*(x340)))+(((IkReal(-1.00000000000000))*(gconst124)*(x310)*(x345)))+(((IkReal(-1.00000000000000))*(gconst113)*(x304)*(x317)))+(((x320)*(x333)))+(((x315)*(x339)))+(((x314)*(x331)))+(((IkReal(-1.00000000000000))*(gconst131)*(x297)*(x316)))+(((IkReal(-1.00000000000000))*(gconst131)*(x297)*(x312)))+(((x316)*(x324)))+(((IkReal(-1.00000000000000))*(x305)*(x330)))+(((x303)*(x317)))+(((IkReal(-1.00000000000000))*(gconst129)*(x304)*(x344)))+(((IkReal(-1.00000000000000))*(gconst107)*(x304)*(x327)))+(((x321)*(x346)))+(((x319)*(x320)))+(((IkReal(-1.00000000000000))*(gconst115)*(x297)*(x320)))+(((IkReal(-1.00000000000000))*(gconst120)*(x304)*(x341))));
op[3]=((((IkReal(-1.00000000000000))*(x308)*(x317)))+(((IkReal(-1.00000000000000))*(x299)*(x351)))+(((x338)*(x350)))+(((IkReal(-1.00000000000000))*(gconst117)*(x297)*(x323)))+(((x336)*(x337)))+(((IkReal(-1.00000000000000))*(gconst115)*(x297)*(x339)))+(((x337)*(x348)))+(((IkReal(-1.00000000000000))*(gconst119)*(x320)*(x332)))+(((x312)*(x315)))+(((IkReal(-1.00000000000000))*(gconst113)*(x304)*(x341)))+(((x308)*(x320)))+(((IkReal(-1.00000000000000))*(gconst131)*(x297)*(x302)))+(((x313)*(x350)))+(((x302)*(x324)))+(((x301)*(x313)))+(((x305)*(x322)))+(((x319)*(x339)))+(((x315)*(x316)))+(((IkReal(-1.00000000000000))*(gconst120)*(x304)*(x326)))+(((IkReal(-1.00000000000000))*(gconst117)*(x310)*(x345)))+(((x333)*(x339)))+(((x299)*(x330)))+(((IkReal(-1.00000000000000))*(x316)*(x354)))+(((x299)*(x329)))+(((x307)*(x331)))+(((x303)*(x341)))+(((IkReal(-1.00000000000000))*(gconst129)*(x304)*(x342)))+(((x301)*(x338)))+(((IkReal(-1.00000000000000))*(x303)*(x339)))+(((x321)*(x328)))+(((IkReal(-1.00000000000000))*(x312)*(x354)))+(((IkReal(-1.00000000000000))*(gconst120)*(x304)*(x344))));
op[4]=((((x302)*(x315)))+(((IkReal(-1.00000000000000))*(x302)*(x354)))+(((x312)*(x319)))+(((IkReal(-1.00000000000000))*(x299)*(x322)))+(((IkReal(-1.00000000000000))*(gconst108)*(x297)*(x323)))+(((IkReal(-1.00000000000000))*(x330)*(x336)))+(((IkReal(-1.00000000000000))*(gconst128)*(x304)*(x349)))+(((x329)*(x346)))+(((IkReal(-1.00000000000000))*(gconst108)*(x310)*(x345)))+(((IkReal(-1.00000000000000))*(gconst115)*(x297)*(x316)))+(((IkReal(-1.00000000000000))*(gconst115)*(x297)*(x312)))+(((IkReal(-1.00000000000000))*(gconst129)*(x304)*(x352)))+(((x303)*(x326)))+(((IkReal(-1.00000000000000))*(x308)*(x341)))+(((IkReal(-1.00000000000000))*(gconst124)*(x309)*(x333)))+(((x316)*(x333)))+(((IkReal(-1.00000000000000))*(x303)*(x316)))+(((IkReal(-1.00000000000000))*(x303)*(x312)))+(((x336)*(x351)))+(((x308)*(x339)))+(((IkReal(-1.00000000000000))*(x327)*(x353)))+(((x320)*(x334)))+(((IkReal(-1.00000000000000))*(gconst131)*(x297)*(x298)))+(((IkReal(-1.00000000000000))*(gconst111)*(x320)*(x332)))+(((x313)*(x340)))+(((IkReal(-1.00000000000000))*(x317)*(x334)))+(((x298)*(x324)))+(((IkReal(-1.00000000000000))*(gconst124)*(x304)*(x311)))+(((IkReal(-1.00000000000000))*(gconst113)*(x304)*(x326)))+(((x314)*(x338)))+(((x348)*(x351)))+(((x338)*(x340)))+(((IkReal(-1.00000000000000))*(x305)*(x335)))+(((x303)*(x344)))+(((x312)*(x333)))+(((IkReal(-1.00000000000000))*(gconst123)*(x304)*(x318)))+(((x330)*(x346)))+(((x300)*(x325)))+(((IkReal(-1.00000000000000))*(gconst119)*(x332)*(x339)))+(((x317)*(x347)))+(((IkReal(-1.00000000000000))*(gconst107)*(x304)*(x343)))+(((x316)*(x319)))+(((IkReal(-1.00000000000000))*(gconst120)*(x304)*(x342)))+(((x313)*(x314)))+(((IkReal(-1.00000000000000))*(x323)*(x340))));
op[5]=((((IkReal(-1.00000000000000))*(gconst115)*(x297)*(x302)))+(((x302)*(x319)))+(((IkReal(-1.00000000000000))*(x306)*(x323)))+(((x334)*(x339)))+(((IkReal(-1.00000000000000))*(x302)*(x303)))+(((IkReal(-1.00000000000000))*(x307)*(x323)))+(((IkReal(-1.00000000000000))*(x298)*(x354)))+(((x337)*(x349)))+(((x328)*(x329)))+(((IkReal(-1.00000000000000))*(gconst119)*(x316)*(x332)))+(((IkReal(-1.00000000000000))*(gconst120)*(x304)*(x352)))+(((IkReal(-1.00000000000000))*(x334)*(x341)))+(((x308)*(x316)))+(((x308)*(x312)))+(((IkReal(-1.00000000000000))*(gconst117)*(x304)*(x311)))+(((x298)*(x315)))+(((IkReal(-1.00000000000000))*(gconst111)*(x332)*(x339)))+(((IkReal(-1.00000000000000))*(gconst119)*(x312)*(x332)))+(((x302)*(x333)))+(((x307)*(x313)))+(((x299)*(x335)))+(((x328)*(x330)))+(((x322)*(x348)))+(((x307)*(x338)))+(((IkReal(-1.00000000000000))*(x308)*(x326)))+(((x303)*(x342)))+(((x300)*(x301)))+(((x322)*(x336)))+(((x306)*(x338)))+(((IkReal(-1.00000000000000))*(gconst124)*(x308)*(x309)))+(((x300)*(x350)))+(((IkReal(-1.00000000000000))*(gconst117)*(x309)*(x333))));
op[6]=((((IkReal(-1.00000000000000))*(x322)*(x328)))+(((x335)*(x346)))+(((IkReal(-1.00000000000000))*(gconst123)*(x318)*(x345)))+(((IkReal(-1.00000000000000))*(gconst119)*(x302)*(x332)))+(((IkReal(-1.00000000000000))*(gconst108)*(x304)*(x311)))+(((x326)*(x347)))+(((x300)*(x314)))+(((IkReal(-1.00000000000000))*(gconst108)*(x309)*(x333)))+(((x300)*(x340)))+(((gconst111)*(gconst124)*(x300)))+(((IkReal(-1.00000000000000))*(x335)*(x348)))+(((x298)*(x319)))+(((IkReal(-1.00000000000000))*(gconst124)*(x309)*(x334)))+(((x316)*(x334)))+(((x303)*(x352)))+(((IkReal(-1.00000000000000))*(gconst117)*(x308)*(x309)))+(((IkReal(-1.00000000000000))*(gconst107)*(x304)*(x318)))+(((IkReal(-1.00000000000000))*(gconst111)*(x316)*(x332)))+(((IkReal(-1.00000000000000))*(x298)*(x303)))+(((x302)*(x308)))+(((x349)*(x351)))+(((IkReal(-1.00000000000000))*(x326)*(x334)))+(((x298)*(x333)))+(((IkReal(-1.00000000000000))*(gconst115)*(x297)*(x298)))+(((x312)*(x334)))+(((IkReal(-1.00000000000000))*(gconst111)*(x312)*(x332)))+(((IkReal(-1.00000000000000))*(x330)*(x349))));
op[7]=((((IkReal(-1.00000000000000))*(gconst108)*(x308)*(x309)))+(((IkReal(-1.00000000000000))*(gconst119)*(x298)*(x332)))+(((x302)*(x334)))+(((x298)*(x308)))+(((x328)*(x335)))+(((x322)*(x349)))+(((x300)*(x307)))+(((x300)*(x306)))+(((IkReal(-1.00000000000000))*(gconst111)*(x302)*(x332)))+(((IkReal(-1.00000000000000))*(gconst117)*(x309)*(x334))));
op[8]=((((IkReal(-1.00000000000000))*(gconst108)*(x309)*(x334)))+(((IkReal(-1.00000000000000))*(x318)*(x353)))+(((gconst108)*(gconst111)*(x300)))+(((IkReal(-1.00000000000000))*(gconst111)*(x298)*(x332)))+(((x298)*(x334))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst148;
gconst148=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x355=((cj1)*(sj2));
IkReal x356=((cj2)*(sj1));
IkReal x357=((IkReal(1.00000000000000))*(sj3));
IkReal x358=((cj0)*(r00));
IkReal x359=((r01)*(sj0));
IkReal x360=((cj0)*(r01));
IkReal x361=((IkReal(1.00000000000000))*(cj3));
IkReal x362=((r00)*(sj0));
IkReal x363=((cj1)*(cj2)*(r02));
IkReal x364=((r02)*(sj1)*(sj2));
IkReal x365=((x359)*(x361));
if( IKabs(((gconst148)*(((((IkReal(-1.00000000000000))*(x360)*(x361)))+(((IkReal(-1.00000000000000))*(x356)*(x357)*(x359)))+(((IkReal(-1.00000000000000))*(x356)*(x357)*(x358)))+(((sj3)*(x363)))+(((IkReal(-1.00000000000000))*(x355)*(x357)*(x358)))+(((IkReal(-1.00000000000000))*(x355)*(x357)*(x359)))+(((IkReal(-1.00000000000000))*(x357)*(x364)))+(((cj3)*(x362))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst148)*(((((IkReal(-1.00000000000000))*(x356)*(x358)*(x361)))+(((IkReal(-1.00000000000000))*(x355)*(x358)*(x361)))+(((IkReal(-1.00000000000000))*(x361)*(x364)))+(((IkReal(-1.00000000000000))*(x355)*(x365)))+(((IkReal(-1.00000000000000))*(x356)*(x365)))+(((sj3)*(x360)))+(((IkReal(-1.00000000000000))*(x357)*(x362)))+(((cj3)*(x363))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst148)*(((((IkReal(-1.00000000000000))*(x360)*(x361)))+(((IkReal(-1.00000000000000))*(x356)*(x357)*(x359)))+(((IkReal(-1.00000000000000))*(x356)*(x357)*(x358)))+(((sj3)*(x363)))+(((IkReal(-1.00000000000000))*(x355)*(x357)*(x358)))+(((IkReal(-1.00000000000000))*(x355)*(x357)*(x359)))+(((IkReal(-1.00000000000000))*(x357)*(x364)))+(((cj3)*(x362)))))), ((gconst148)*(((((IkReal(-1.00000000000000))*(x356)*(x358)*(x361)))+(((IkReal(-1.00000000000000))*(x355)*(x358)*(x361)))+(((IkReal(-1.00000000000000))*(x361)*(x364)))+(((IkReal(-1.00000000000000))*(x355)*(x365)))+(((IkReal(-1.00000000000000))*(x356)*(x365)))+(((sj3)*(x360)))+(((IkReal(-1.00000000000000))*(x357)*(x362)))+(((cj3)*(x363)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x366=IKsin(j5);
IkReal x367=IKcos(j5);
IkReal x368=((cj1)*(sj2));
IkReal x369=((r01)*(sj0));
IkReal x370=((cj0)*(r00));
IkReal x371=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj3)*(x366)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((sj3)*(x367))));
evalcond[1]=((((x370)*(x371)))+(((x369)*(x371)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((sj3)*(x366)))+(((x368)*(x370)))+(((r02)*(sj1)*(sj2)))+(((cj3)*(x367)))+(((x368)*(x369))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x372=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x373=((cj1)*(x372));
IkReal x374=((cj0)*(r00));
IkReal x375=((sj1)*(x372));
IkReal x376=((IkReal(1.00000000000000))*(sj2));
IkReal x377=((r01)*(sj0));
cj5array[0]=((((cj2)*(r02)*(x375)))+(((IkReal(-1.00000000000000))*(x374)*(x375)*(x376)))+(((IkReal(-1.00000000000000))*(x375)*(x376)*(x377)))+(((cj2)*(x373)*(x374)))+(((cj2)*(x373)*(x377)))+(((r02)*(sj2)*(x373))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[3];
dummyeval[0]=sj4;
dummyeval[1]=cj4;
dummyeval[2]=cj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  || IKabs(dummyeval[2]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x378=((pz)*(sj2));
IkReal x379=((IkReal(1.00000000000000))*(cj1));
IkReal x380=((cj2)*(sj0));
IkReal x381=((py)*(sj1));
IkReal x382=((cj0)*(sj1));
IkReal x383=((cj2)*(px));
IkReal x384=((sj0)*(sj2));
IkReal x385=((IkReal(0.0750000000000000))*(cj2));
IkReal x386=((px)*(sj2));
IkReal x387=((IkReal(0.0750000000000000))*(sj2));
IkReal x388=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x384)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x379)))+(((IkReal(-1.00000000000000))*(r01)*(x379)*(x380)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x379)))+(((r00)*(sj2)*(x382)))+(((IkReal(-1.00000000000000))*(r02)*(x388))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x384)))+(((IkReal(-1.00000000000000))*(cj1)*(x387)))+(((sj1)*(x378)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x386)))+(((x382)*(x383)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x379)))+(((x380)*(x381)))+(((IkReal(-1.00000000000000))*(sj1)*(x385))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(py)*(x379)*(x380)))+(((cj1)*(x385)))+(((IkReal(-1.00000000000000))*(x378)*(x379)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x382)*(x386)))+(((IkReal(-1.00000000000000))*(sj1)*(x387)))+(((x381)*(x384)))+(((IkReal(-1.00000000000000))*(cj0)*(x379)*(x383)))+(((IkReal(-1.00000000000000))*(pz)*(x388))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x389=((cj1)*(cj5));
IkReal x390=((IkReal(1.00000000000000))*(cj2));
IkReal x391=((r00)*(sj0));
IkReal x392=((IkReal(1.00000000000000))*(sj5));
IkReal x393=((cj0)*(r01));
IkReal x394=((r01)*(sj0));
IkReal x395=((sj1)*(sj5));
IkReal x396=((cj0)*(r00));
IkReal x397=((sj2)*(x396));
IkReal x398=((cj2)*(cj5)*(sj1));
IkReal x399=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x390)*(x395)*(x396)))+(((IkReal(-1.00000000000000))*(x390)*(x394)*(x395)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x392)*(x394)))+(((cj5)*(x391)))+(((IkReal(-1.00000000000000))*(cj1)*(x392)*(x397)))+(((IkReal(-1.00000000000000))*(cj5)*(x393)))+(((IkReal(-1.00000000000000))*(x392)*(x399)))+(((cj1)*(cj2)*(r02)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((sj2)*(x389)*(x394)))+(((cj5)*(x399)))+(((IkReal(-1.00000000000000))*(r02)*(x389)*(x390)))+(((x394)*(x398)))+(((x396)*(x398)))+(((sj5)*(x391)))+(((x389)*(x397)))+(((IkReal(-1.00000000000000))*(x392)*(x393))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(x390)*(x395)*(x396)))+(((IkReal(-1.00000000000000))*(x390)*(x394)*(x395)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x392)*(x394)))+(((cj5)*(x391)))+(((IkReal(-1.00000000000000))*(cj1)*(x392)*(x397)))+(((IkReal(-1.00000000000000))*(cj5)*(x393)))+(((IkReal(-1.00000000000000))*(x392)*(x399)))+(((cj1)*(cj2)*(r02)*(sj5)))))), ((gconst55)*(((((sj2)*(x389)*(x394)))+(((cj5)*(x399)))+(((IkReal(-1.00000000000000))*(r02)*(x389)*(x390)))+(((x394)*(x398)))+(((x396)*(x398)))+(((sj5)*(x391)))+(((x389)*(x397)))+(((IkReal(-1.00000000000000))*(x392)*(x393)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x400=IKsin(j3);
IkReal x401=IKcos(j3);
IkReal x402=((r01)*(sj0));
IkReal x403=((cj1)*(sj2));
IkReal x404=((cj2)*(sj1));
IkReal x405=((cj0)*(r00));
IkReal x406=((IkReal(1.00000000000000))*(x401));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x406)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj5)*(x400))));
evalcond[1]=((((x404)*(x405)))+(((sj5)*(x400)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj5)*(x406)))+(((r02)*(sj1)*(sj2)))+(((x403)*(x405)))+(((x402)*(x403)))+(((x402)*(x404))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x407=((pz)*(sj2));
IkReal x408=((IkReal(1.00000000000000))*(cj1));
IkReal x409=((cj2)*(sj0));
IkReal x410=((py)*(sj1));
IkReal x411=((cj0)*(sj1));
IkReal x412=((cj2)*(px));
IkReal x413=((sj0)*(sj2));
IkReal x414=((IkReal(0.0750000000000000))*(cj2));
IkReal x415=((px)*(sj2));
IkReal x416=((IkReal(0.0750000000000000))*(sj2));
IkReal x417=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x413)))+(((r00)*(sj2)*(x411)))+(((IkReal(-1.00000000000000))*(r02)*(x417)))+(((IkReal(-1.00000000000000))*(r01)*(x408)*(x409)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x408)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x408))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x413)))+(((sj1)*(x407)))+(((x411)*(x412)))+(((IkReal(-1.00000000000000))*(cj1)*(x416)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x409)*(x410)))+(((IkReal(-1.00000000000000))*(sj1)*(x414)))+(((cj0)*(cj1)*(x415)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x408))));
evalcond[4]=((IkReal(0.215000000000000))+(((x411)*(x415)))+(((x410)*(x413)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x408)*(x412)))+(((IkReal(-1.00000000000000))*(x407)*(x408)))+(((IkReal(-1.00000000000000))*(pz)*(x417)))+(((IkReal(-1.00000000000000))*(sj1)*(x416)))+(((cj1)*(x414)))+(((IkReal(-1.00000000000000))*(py)*(x408)*(x409))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x418=((cj1)*(cj5));
IkReal x419=((cj0)*(r00));
IkReal x420=((cj5)*(sj0));
IkReal x421=((cj2)*(sj1));
IkReal x422=((r01)*(sj5));
IkReal x423=((cj1)*(sj5));
IkReal x424=((sj0)*(sj2));
IkReal x425=((IkReal(1.00000000000000))*(cj2)*(r02));
IkReal x426=((r02)*(sj1)*(sj2));
if( IKabs(((gconst56)*(((((sj2)*(x419)*(x423)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((r00)*(x420)))+(((sj0)*(x421)*(x422)))+(((sj5)*(x426)))+(((sj5)*(x419)*(x421)))+(((IkReal(-1.00000000000000))*(x423)*(x425)))+(((cj1)*(x422)*(x424))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((cj5)*(x419)*(x421)))+(((r01)*(x418)*(x424)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((cj0)*(x422)))+(((cj5)*(x426)))+(((IkReal(-1.00000000000000))*(x418)*(x425)))+(((r01)*(x420)*(x421)))+(((sj2)*(x418)*(x419))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((sj2)*(x419)*(x423)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((r00)*(x420)))+(((sj0)*(x421)*(x422)))+(((sj5)*(x426)))+(((sj5)*(x419)*(x421)))+(((IkReal(-1.00000000000000))*(x423)*(x425)))+(((cj1)*(x422)*(x424)))))), ((gconst56)*(((((cj5)*(x419)*(x421)))+(((r01)*(x418)*(x424)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((cj0)*(x422)))+(((cj5)*(x426)))+(((IkReal(-1.00000000000000))*(x418)*(x425)))+(((r01)*(x420)*(x421)))+(((sj2)*(x418)*(x419)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x427=IKsin(j3);
IkReal x428=IKcos(j3);
IkReal x429=((cj1)*(sj2));
IkReal x430=((r01)*(sj0));
IkReal x431=((cj0)*(r00));
IkReal x432=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((cj5)*(x427)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(sj5)*(x428))));
evalcond[1]=((((x429)*(x430)))+(((x429)*(x431)))+(((cj5)*(x428)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x430)*(x432)))+(((sj5)*(x427)))+(((x431)*(x432)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x433=((cj1)*(cj2));
IkReal x434=((IkReal(1.00000000000000))*(cj0));
IkReal x435=((sj1)*(sj2));
IkReal x436=((IkReal(1.00000000000000))*(sj0));
IkReal x437=((sj0)*(x435));
IkReal x438=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x439=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x433)*(x436)))+(((cj0)*(r00)*(x435)))+(cj5)+(((IkReal(-1.00000000000000))*(r00)*(x433)*(x434)))+(((IkReal(-1.00000000000000))*(r02)*(x439)))+(((IkReal(-1.00000000000000))*(r02)*(x438)))+(((r01)*(x437))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(py)*(x433)*(x436)))+(((IkReal(-1.00000000000000))*(px)*(x433)*(x434)))+(((IkReal(-0.0750000000000000))*(x435)))+(((py)*(x437)))+(((IkReal(0.0750000000000000))*(x433)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj0)*(px)*(x435)))+(((IkReal(-1.00000000000000))*(pz)*(x439)))+(((IkReal(-1.00000000000000))*(pz)*(x438))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x440=((cj1)*(sj2));
IkReal x441=((cj0)*(r00));
IkReal x442=((IkReal(1.00000000000000))*(pz));
IkReal x443=((sj1)*(sj2));
IkReal x444=((r01)*(sj0));
IkReal x445=((IkReal(1.00000000000000))*(r02));
IkReal x446=((cj1)*(cj2));
IkReal x447=((cj2)*(sj1));
IkReal x448=((IkReal(1.00000000000000))*(cj0));
IkReal x449=((IkReal(1.00000000000000))*(sj0)*(x446));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x448))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x445)*(x447)))+(((IkReal(-1.00000000000000))*(x444)*(x446)))+(((x441)*(x443)))+(((IkReal(-1.00000000000000))*(x441)*(x446)))+(((x443)*(x444)))+(((IkReal(-1.00000000000000))*(x440)*(x445))));
evalcond[3]=((((r02)*(x443)))+(((IkReal(-1.00000000000000))*(x445)*(x446)))+(((x440)*(x444)))+(((x440)*(x441)))+(((x441)*(x447)))+(((x444)*(x447))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(px)*(x446)*(x448)))+(((IkReal(-0.0750000000000000))*(x443)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x446)))+(((IkReal(-1.00000000000000))*(py)*(x449)))+(((cj0)*(px)*(x443)))+(((IkReal(-1.00000000000000))*(x440)*(x442)))+(((py)*(sj0)*(x443)))+(((IkReal(-1.00000000000000))*(x442)*(x447))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x450=((IkReal(12.5000000000000))*(cj1));
IkReal x451=((py)*(sj0));
IkReal x452=((cj0)*(px));
IkReal x453=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x450)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(sj2)*(x450)*(x452)))+(((IkReal(-1.00000000000000))*(x452)*(x453)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x450)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(sj2)*(x450)*(x452)))+(((IkReal(-1.00000000000000))*(x452)*(x453)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((cj2)*(pz)*(x450)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(sj2)*(x450)*(x452)))+(((IkReal(-1.00000000000000))*(x452)*(x453)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x454=((cj1)*(sj2));
IkReal x455=((cj2)*(sj1));
IkReal x456=((cj0)*(px));
IkReal x457=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x454)*(x457)))+(((x454)*(x456)))+(((x455)*(x457)))+(((x455)*(x456)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0750000000000000))*(x455)))+(((IkReal(-0.0750000000000000))*(x454)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x458=((cj1)*(sj2));
IkReal x459=((cj0)*(r00));
IkReal x460=((IkReal(1.00000000000000))*(pz));
IkReal x461=((sj1)*(sj2));
IkReal x462=((r01)*(sj0));
IkReal x463=((IkReal(1.00000000000000))*(r02));
IkReal x464=((cj1)*(cj2));
IkReal x465=((cj2)*(sj1));
IkReal x466=((IkReal(1.00000000000000))*(cj0));
IkReal x467=((IkReal(1.00000000000000))*(sj0)*(x464));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x466))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x459)*(x464)))+(((IkReal(-1.00000000000000))*(x462)*(x464)))+(((x461)*(x462)))+(((IkReal(-1.00000000000000))*(x463)*(x465)))+(((IkReal(-1.00000000000000))*(x458)*(x463)))+(((x459)*(x461))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x463)*(x464)))+(((r02)*(x461)))+(((x459)*(x465)))+(((x458)*(x459)))+(((x458)*(x462)))+(((x462)*(x465))));
evalcond[4]=((IkReal(0.295000000000000))+(((py)*(sj0)*(x461)))+(((IkReal(-1.00000000000000))*(x460)*(x465)))+(((IkReal(0.0750000000000000))*(x464)))+(((IkReal(-1.00000000000000))*(py)*(x467)))+(((cj0)*(px)*(x461)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x464)*(x466)))+(((IkReal(-0.0750000000000000))*(x461)))+(((IkReal(-1.00000000000000))*(x458)*(x460))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x468=((IkReal(12.5000000000000))*(cj1));
IkReal x469=((py)*(sj0));
IkReal x470=((cj0)*(px));
IkReal x471=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x468)))+(((IkReal(-1.00000000000000))*(x469)*(x471)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x468)*(x469)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x468)*(x470)))+(((IkReal(-1.00000000000000))*(x470)*(x471)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x468)))+(((IkReal(-1.00000000000000))*(x469)*(x471)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x468)*(x469)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x468)*(x470)))+(((IkReal(-1.00000000000000))*(x470)*(x471)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((cj2)*(pz)*(x468)))+(((IkReal(-1.00000000000000))*(x469)*(x471)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x468)*(x469)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x468)*(x470)))+(((IkReal(-1.00000000000000))*(x470)*(x471)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x472=((cj1)*(sj2));
IkReal x473=((cj2)*(sj1));
IkReal x474=((cj0)*(px));
IkReal x475=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x472)*(x475)))+(((x472)*(x474)))+(((x473)*(x474)))+(((x473)*(x475)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0750000000000000))*(x473)))+(((IkReal(-0.0750000000000000))*(x472)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x476=IKsin(j3);
IkReal x477=IKcos(j3);
IkReal x478=((sj1)*(sj2));
IkReal x479=((cj2)*(sj1));
IkReal x480=((py)*(sj0));
IkReal x481=((IkReal(1.00000000000000))*(cj0));
IkReal x482=((r01)*(sj0));
IkReal x483=((cj1)*(sj2));
IkReal x484=((cj0)*(x483));
IkReal x485=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(0.0800000000000000))*(x476)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x481))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x477)))+(((IkReal(-1.00000000000000))*(r01)*(x481))));
evalcond[2]=((((sj5)*(x476)))+(((cj0)*(r00)*(x479)))+(((r02)*(x478)))+(((r00)*(x484)))+(((x482)*(x483)))+(((x479)*(x482)))+(((IkReal(-1.00000000000000))*(r02)*(x485))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x485)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x479)))+(((IkReal(-0.0750000000000000))*(x483)))+(((px)*(x484)))+(((x480)*(x483)))+(((pz)*(x478)))+(((IkReal(-0.0750000000000000))*(x479)))+(((IkReal(0.0800000000000000))*(x477)))+(((x479)*(x480))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x486=((IkReal(1.00000000000000))*(sj2));
IkReal x487=((IkReal(1.00000000000000))*(r01));
IkReal x488=((cj0)*(r00));
IkReal x489=((cj2)*(sj1));
if( IKabs(((gconst57)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x486)))+(((IkReal(-1.00000000000000))*(cj1)*(x486)*(x488)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x486)))+(((IkReal(-1.00000000000000))*(x488)*(x489)))+(((IkReal(-1.00000000000000))*(sj0)*(x487)*(x489)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x487))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x486)))+(((IkReal(-1.00000000000000))*(cj1)*(x486)*(x488)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x486)))+(((IkReal(-1.00000000000000))*(x488)*(x489)))+(((IkReal(-1.00000000000000))*(sj0)*(x487)*(x489)))+(((cj1)*(cj2)*(r02)))))), ((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x487)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x490=IKsin(j3);
IkReal x491=IKcos(j3);
IkReal x492=((sj1)*(sj2));
IkReal x493=((cj2)*(sj1));
IkReal x494=((py)*(sj0));
IkReal x495=((IkReal(1.00000000000000))*(cj0));
IkReal x496=((r01)*(sj0));
IkReal x497=((cj1)*(sj2));
IkReal x498=((cj0)*(x497));
IkReal x499=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x495)))+(((IkReal(0.0800000000000000))*(x490))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x495)))+(((IkReal(-1.00000000000000))*(sj5)*(x491))));
evalcond[2]=((((r02)*(x492)))+(((IkReal(-1.00000000000000))*(r02)*(x499)))+(((x496)*(x497)))+(((x493)*(x496)))+(((r00)*(x498)))+(((sj5)*(x490)))+(((cj0)*(r00)*(x493))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((pz)*(x492)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x497)))+(((IkReal(-0.0750000000000000))*(x493)))+(((x493)*(x494)))+(((x494)*(x497)))+(((px)*(x498)))+(((cj0)*(px)*(x493)))+(((IkReal(-1.00000000000000))*(pz)*(x499)))+(((IkReal(0.0800000000000000))*(x491))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x500=((sj1)*(sj2));
IkReal x501=((cj1)*(cj2));
IkReal x502=((IkReal(1.00000000000000))*(sj0));
IkReal x503=((sj0)*(x500));
IkReal x504=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x505=((IkReal(1.00000000000000))*(cj2)*(sj1));
IkReal x506=((IkReal(1.00000000000000))*(cj0)*(x501));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x501)*(x502)))+(((cj0)*(r00)*(x500)))+(((r01)*(x503)))+(((IkReal(-1.00000000000000))*(r00)*(x506)))+(((IkReal(-1.00000000000000))*(cj5)))+(((IkReal(-1.00000000000000))*(r02)*(x505)))+(((IkReal(-1.00000000000000))*(r02)*(x504))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x505)))+(((IkReal(-1.00000000000000))*(pz)*(x504)))+(((IkReal(-1.00000000000000))*(py)*(x501)*(x502)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj0)*(px)*(x500)))+(((py)*(x503)))+(((IkReal(-1.00000000000000))*(px)*(x506)))+(((IkReal(-0.0750000000000000))*(x500)))+(((IkReal(0.0750000000000000))*(x501))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x507=((cj1)*(sj2));
IkReal x508=((cj0)*(r00));
IkReal x509=((IkReal(1.00000000000000))*(pz));
IkReal x510=((sj1)*(sj2));
IkReal x511=((r01)*(sj0));
IkReal x512=((IkReal(1.00000000000000))*(r02));
IkReal x513=((cj1)*(cj2));
IkReal x514=((cj2)*(sj1));
IkReal x515=((IkReal(1.00000000000000))*(cj0));
IkReal x516=((IkReal(1.00000000000000))*(sj0)*(x513));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x515))));
evalcond[2]=((IkReal(-1.00000000000000))+(((x510)*(x511)))+(((IkReal(-1.00000000000000))*(x508)*(x513)))+(((IkReal(-1.00000000000000))*(x512)*(x514)))+(((x508)*(x510)))+(((IkReal(-1.00000000000000))*(x507)*(x512)))+(((IkReal(-1.00000000000000))*(x511)*(x513))));
evalcond[3]=((((x511)*(x514)))+(((x507)*(x511)))+(((x507)*(x508)))+(((IkReal(-1.00000000000000))*(x512)*(x513)))+(((r02)*(x510)))+(((x508)*(x514))));
evalcond[4]=((IkReal(0.295000000000000))+(((cj0)*(px)*(x510)))+(((IkReal(-1.00000000000000))*(x507)*(x509)))+(((IkReal(0.0750000000000000))*(x513)))+(((IkReal(-0.0750000000000000))*(x510)))+(((IkReal(-0.270000000000000))*(sj2)))+(((py)*(sj0)*(x510)))+(((IkReal(-1.00000000000000))*(py)*(x516)))+(((IkReal(-1.00000000000000))*(x509)*(x514)))+(((IkReal(-1.00000000000000))*(px)*(x513)*(x515))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x517=((IkReal(12.5000000000000))*(sj2));
IkReal x518=((py)*(sj0));
IkReal x519=((cj2)*(sj1));
IkReal x520=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(-1.00000000000000))*(py)*(x520)))+(((IkReal(12.5000000000000))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x517)*(x518)))+(((pz)*(sj1)*(x517)))+(((IkReal(-0.937500000000000))*(x519)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x519)*(x520)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x517)))+(((IkReal(12.5000000000000))*(x518)*(x519))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(py)*(x520)))+(((IkReal(12.5000000000000))*(px)*(sj0)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x517)*(x518)))+(((pz)*(sj1)*(x517)))+(((IkReal(-0.937500000000000))*(x519)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x519)*(x520)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x517)))+(((IkReal(12.5000000000000))*(x518)*(x519)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(py)*(x520)))+(((IkReal(12.5000000000000))*(px)*(sj0)))), ((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x517)*(x518)))+(((pz)*(sj1)*(x517)))+(((IkReal(-0.937500000000000))*(x519)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x519)*(x520)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x517)))+(((IkReal(12.5000000000000))*(x518)*(x519)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x521=((cj1)*(sj2));
IkReal x522=((cj2)*(sj1));
IkReal x523=((cj0)*(px));
IkReal x524=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x522)*(x524)))+(((x522)*(x523)))+(((IkReal(-0.0750000000000000))*(x522)))+(((IkReal(-0.0750000000000000))*(x521)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((x521)*(x524)))+(((x521)*(x523)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x525=((cj1)*(sj2));
IkReal x526=((cj0)*(r00));
IkReal x527=((IkReal(1.00000000000000))*(pz));
IkReal x528=((sj1)*(sj2));
IkReal x529=((r01)*(sj0));
IkReal x530=((IkReal(1.00000000000000))*(r02));
IkReal x531=((cj1)*(cj2));
IkReal x532=((cj2)*(sj1));
IkReal x533=((IkReal(1.00000000000000))*(cj0));
IkReal x534=((IkReal(1.00000000000000))*(sj0)*(x531));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x533))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x525)*(x530)))+(((IkReal(-1.00000000000000))*(x529)*(x531)))+(((IkReal(-1.00000000000000))*(x530)*(x532)))+(((x526)*(x528)))+(((x528)*(x529)))+(((IkReal(-1.00000000000000))*(x526)*(x531))));
evalcond[3]=((((r02)*(x528)))+(((IkReal(-1.00000000000000))*(x530)*(x531)))+(((x526)*(x532)))+(((x525)*(x526)))+(((x525)*(x529)))+(((x529)*(x532))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x528)))+(((IkReal(-1.00000000000000))*(x525)*(x527)))+(((IkReal(-1.00000000000000))*(px)*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(x527)*(x532)))+(((cj0)*(px)*(x528)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x531)))+(((py)*(sj0)*(x528)))+(((IkReal(-1.00000000000000))*(py)*(x534))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x535=((IkReal(12.5000000000000))*(sj2));
IkReal x536=((py)*(sj0));
IkReal x537=((cj2)*(sj1));
IkReal x538=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(-1.00000000000000))*(py)*(x538)))+(((IkReal(12.5000000000000))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-0.937500000000000))*(x537)))+(((pz)*(sj1)*(x535)))+(((px)*(x537)*(x538)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj1)*(x535)*(x536)))+(((cj0)*(cj1)*(px)*(x535)))+(((IkReal(12.5000000000000))*(x536)*(x537))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(py)*(x538)))+(((IkReal(12.5000000000000))*(px)*(sj0)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-0.937500000000000))*(x537)))+(((pz)*(sj1)*(x535)))+(((px)*(x537)*(x538)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj1)*(x535)*(x536)))+(((cj0)*(cj1)*(px)*(x535)))+(((IkReal(12.5000000000000))*(x536)*(x537)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(py)*(x538)))+(((IkReal(12.5000000000000))*(px)*(sj0)))), ((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-0.937500000000000))*(x537)))+(((pz)*(sj1)*(x535)))+(((px)*(x537)*(x538)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj1)*(x535)*(x536)))+(((cj0)*(cj1)*(px)*(x535)))+(((IkReal(12.5000000000000))*(x536)*(x537)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x539=((cj1)*(sj2));
IkReal x540=((cj2)*(sj1));
IkReal x541=((cj0)*(px));
IkReal x542=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x539)*(x542)))+(((x539)*(x541)))+(((IkReal(-0.0750000000000000))*(x539)))+(((IkReal(-0.0750000000000000))*(x540)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x540)*(x541)))+(((x540)*(x542)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-12.5000000000000))*(cj0)*(py)))))+IKsqr(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-12.5000000000000))*(cj0)*(py)))), ((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x543=IKsin(j3);
IkReal x544=IKcos(j3);
IkReal x545=((sj1)*(sj2));
IkReal x546=((cj2)*(sj1));
IkReal x547=((py)*(sj0));
IkReal x548=((IkReal(1.00000000000000))*(cj0));
IkReal x549=((r01)*(sj0));
IkReal x550=((cj1)*(sj2));
IkReal x551=((cj0)*(x550));
IkReal x552=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-0.0800000000000000))*(x543)))+(((IkReal(-1.00000000000000))*(py)*(x548)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x544)))+(((IkReal(-1.00000000000000))*(r01)*(x548))));
evalcond[2]=((((r02)*(x545)))+(((IkReal(-1.00000000000000))*(r02)*(x552)))+(((sj5)*(x543)))+(((cj0)*(r00)*(x546)))+(((x549)*(x550)))+(((r00)*(x551)))+(((x546)*(x549))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x546)))+(((IkReal(-0.0750000000000000))*(x550)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x546)))+(((IkReal(-0.0800000000000000))*(x544)))+(((px)*(x551)))+(((x547)*(x550)))+(((IkReal(-1.00000000000000))*(pz)*(x552)))+(((pz)*(x545)))+(((x546)*(x547))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x553=((IkReal(1.00000000000000))*(sj2));
IkReal x554=((IkReal(1.00000000000000))*(r01));
IkReal x555=((cj0)*(r00));
IkReal x556=((cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(x553)*(x555)))+(((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x553)))+(((IkReal(-1.00000000000000))*(sj0)*(x554)*(x556)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x553))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x554))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(x553)*(x555)))+(((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x553)))+(((IkReal(-1.00000000000000))*(sj0)*(x554)*(x556)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x553)))))), ((gconst58)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x554)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x557=IKsin(j3);
IkReal x558=IKcos(j3);
IkReal x559=((sj1)*(sj2));
IkReal x560=((cj2)*(sj1));
IkReal x561=((py)*(sj0));
IkReal x562=((IkReal(1.00000000000000))*(cj0));
IkReal x563=((r01)*(sj0));
IkReal x564=((cj1)*(sj2));
IkReal x565=((cj0)*(x564));
IkReal x566=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-0.0800000000000000))*(x557)))+(((IkReal(-1.00000000000000))*(py)*(x562)))+(((px)*(sj0))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x558)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x562))));
evalcond[2]=((((r02)*(x559)))+(((r00)*(x565)))+(((sj5)*(x557)))+(((cj0)*(r00)*(x560)))+(((x563)*(x564)))+(((x560)*(x563)))+(((IkReal(-1.00000000000000))*(r02)*(x566))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(px)*(x560)))+(((pz)*(x559)))+(((IkReal(-0.0800000000000000))*(x558)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x561)*(x564)))+(((IkReal(-1.00000000000000))*(pz)*(x566)))+(((x560)*(x561)))+(((IkReal(-0.0750000000000000))*(x560)))+(((IkReal(-0.0750000000000000))*(x564)))+(((px)*(x565))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x567=((cj1)*(cj2));
IkReal x568=((IkReal(1.00000000000000))*(cj0));
IkReal x569=((sj1)*(sj2));
IkReal x570=((IkReal(1.00000000000000))*(sj0));
IkReal x571=((sj0)*(x569));
IkReal x572=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x573=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r01)*(x571)))+(((IkReal(-1.00000000000000))*(r01)*(x567)*(x570)))+(((cj0)*(r00)*(x569)))+(((IkReal(-1.00000000000000))*(r00)*(x567)*(x568)))+(((IkReal(-1.00000000000000))*(r02)*(x572)))+(((IkReal(-1.00000000000000))*(r02)*(x573))));
evalcond[2]=((IkReal(0.295000000000000))+(((cj0)*(px)*(x569)))+(((py)*(x571)))+(((IkReal(0.0750000000000000))*(x567)))+(((IkReal(-1.00000000000000))*(px)*(x567)*(x568)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x572)))+(((IkReal(-1.00000000000000))*(pz)*(x573)))+(((IkReal(-0.0750000000000000))*(x569)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x567)*(x570))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x574=((IkReal(1.00000000000000))*(sj2));
IkReal x575=((IkReal(1.00000000000000))*(r01));
IkReal x576=((cj0)*(r00));
IkReal x577=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(x574)*(x576)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x574)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x574)))+(((IkReal(-1.00000000000000))*(sj0)*(x575)*(x577)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj0)*(x575)))+(((r00)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(x574)*(x576)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x574)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x574)))+(((IkReal(-1.00000000000000))*(sj0)*(x575)*(x577)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj0)*(x575)))+(((r00)*(sj0)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(x574)*(x576)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x574)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x574)))+(((IkReal(-1.00000000000000))*(sj0)*(x575)*(x577)))+(((cj1)*(cj2)*(r02)))), ((((IkReal(-1.00000000000000))*(cj0)*(x575)))+(((r00)*(sj0)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x578=IKsin(j3);
IkReal x579=IKcos(j3);
IkReal x580=((sj1)*(sj2));
IkReal x581=((IkReal(0.0800000000000000))*(sj4));
IkReal x582=((cj2)*(sj1));
IkReal x583=((py)*(sj0));
IkReal x584=((IkReal(1.00000000000000))*(cj0));
IkReal x585=((r01)*(sj0));
IkReal x586=((cj1)*(sj2));
IkReal x587=((cj0)*(x586));
IkReal x588=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x578)*(x581)))+(((IkReal(-1.00000000000000))*(py)*(x584)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x579)))+(((IkReal(-1.00000000000000))*(r01)*(x584))));
evalcond[2]=((((cj0)*(r00)*(x582)))+(((IkReal(-1.00000000000000))*(r02)*(x588)))+(((x582)*(x585)))+(((r02)*(x580)))+(((x585)*(x586)))+(x578)+(((r00)*(x587))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(px)*(x582)))+(((x583)*(x586)))+(((x579)*(x581)))+(((IkReal(-0.0750000000000000))*(x582)))+(((IkReal(-0.0750000000000000))*(x586)))+(((px)*(x587)))+(((x582)*(x583)))+(((IkReal(-1.00000000000000))*(pz)*(x588)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(x580))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x589=((cj1)*(cj2));
IkReal x590=((IkReal(1.00000000000000))*(cj0));
IkReal x591=((sj1)*(sj2));
IkReal x592=((IkReal(1.00000000000000))*(sj0));
IkReal x593=((sj0)*(x591));
IkReal x594=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x595=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x589)*(x592)))+(((cj0)*(r00)*(x591)))+(((r01)*(x593)))+(((IkReal(-1.00000000000000))*(r00)*(x589)*(x590)))+(((IkReal(-1.00000000000000))*(r02)*(x594)))+(((IkReal(-1.00000000000000))*(r02)*(x595))));
evalcond[2]=((IkReal(0.295000000000000))+(((py)*(x593)))+(((IkReal(-1.00000000000000))*(px)*(x589)*(x590)))+(((IkReal(0.0750000000000000))*(x589)))+(((IkReal(-1.00000000000000))*(pz)*(x595)))+(((IkReal(-1.00000000000000))*(pz)*(x594)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x591)))+(((cj0)*(px)*(x591)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x589)*(x592))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x596=((cj1)*(sj2));
IkReal x597=((r01)*(sj0));
IkReal x598=((cj0)*(r00));
IkReal x599=((cj2)*(sj1));
if( IKabs(((((x598)*(x599)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x596)*(x597)))+(((x596)*(x598)))+(((x597)*(x599)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x598)*(x599)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x596)*(x597)))+(((x596)*(x598)))+(((x597)*(x599)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((x598)*(x599)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x596)*(x597)))+(((x596)*(x598)))+(((x597)*(x599)))+(((r02)*(sj1)*(sj2)))), ((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x600=IKsin(j3);
IkReal x601=IKcos(j3);
IkReal x602=((sj1)*(sj2));
IkReal x603=((IkReal(0.0800000000000000))*(sj4));
IkReal x604=((cj2)*(sj1));
IkReal x605=((py)*(sj0));
IkReal x606=((IkReal(1.00000000000000))*(cj0));
IkReal x607=((r01)*(sj0));
IkReal x608=((cj1)*(sj2));
IkReal x609=((cj0)*(x608));
IkReal x610=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x600)*(x603)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x606))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x606)))+(x601));
evalcond[2]=((((r00)*(x609)))+(((x604)*(x607)))+(((IkReal(-1.00000000000000))*(x600)))+(((cj0)*(r00)*(x604)))+(((x607)*(x608)))+(((r02)*(x602)))+(((IkReal(-1.00000000000000))*(r02)*(x610))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x605)*(x608)))+(((x601)*(x603)))+(((x604)*(x605)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x609)))+(((IkReal(-0.0750000000000000))*(x604)))+(((IkReal(-0.0750000000000000))*(x608)))+(((cj0)*(px)*(x604)))+(((IkReal(-1.00000000000000))*(pz)*(x610)))+(((pz)*(x602))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x611=((cj1)*(cj2));
IkReal x612=((IkReal(1.00000000000000))*(cj0));
IkReal x613=((sj1)*(sj2));
IkReal x614=((IkReal(1.00000000000000))*(sj0));
IkReal x615=((sj0)*(x613));
IkReal x616=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x617=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r01)*(x615)))+(sj4)+(((IkReal(-1.00000000000000))*(r01)*(x611)*(x614)))+(((cj0)*(r00)*(x613)))+(((IkReal(-1.00000000000000))*(r00)*(x611)*(x612)))+(((IkReal(-1.00000000000000))*(r02)*(x617)))+(((IkReal(-1.00000000000000))*(r02)*(x616))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(px)*(x611)*(x612)))+(((IkReal(-0.0750000000000000))*(x613)))+(((cj0)*(px)*(x613)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x616)))+(((IkReal(-1.00000000000000))*(pz)*(x617)))+(((py)*(x615)))+(((IkReal(-1.00000000000000))*(py)*(x611)*(x614)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(0.0750000000000000))*(x611))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x618=((pz)*(sj2));
IkReal x619=((IkReal(1.00000000000000))*(cj1));
IkReal x620=((cj2)*(sj0));
IkReal x621=((py)*(sj1));
IkReal x622=((cj0)*(sj1));
IkReal x623=((cj2)*(px));
IkReal x624=((sj0)*(sj2));
IkReal x625=((IkReal(0.0750000000000000))*(cj2));
IkReal x626=((px)*(sj2));
IkReal x627=((IkReal(0.0750000000000000))*(sj2));
IkReal x628=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x619)))+(((IkReal(-1.00000000000000))*(r02)*(x628)))+(((IkReal(-1.00000000000000))*(r01)*(x619)*(x620)))+(((r00)*(sj2)*(x622)))+(((r01)*(sj1)*(x624)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x619))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x627)))+(((cj1)*(py)*(x624)))+(((sj1)*(x618)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x626)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x619)))+(((IkReal(-1.00000000000000))*(sj1)*(x625)))+(((x622)*(x623)))+(((x620)*(x621))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x628)))+(((IkReal(-1.00000000000000))*(py)*(x619)*(x620)))+(((IkReal(-1.00000000000000))*(x618)*(x619)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x621)*(x624)))+(((IkReal(-1.00000000000000))*(sj1)*(x627)))+(((IkReal(-1.00000000000000))*(cj0)*(x619)*(x623)))+(((x622)*(x626)))+(((cj1)*(x625))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x629=((cj1)*(sj2));
IkReal x630=((r01)*(sj0));
IkReal x631=((cj0)*(r00));
IkReal x632=((cj2)*(sj1));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x629)*(x631)))+(((x629)*(x630)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x631)*(x632)))+(((x630)*(x632))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))+IKsqr(((((x629)*(x631)))+(((x629)*(x630)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x631)*(x632)))+(((x630)*(x632)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))), ((((x629)*(x631)))+(((x629)*(x630)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x631)*(x632)))+(((x630)*(x632)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x633=((cj1)*(sj2));
IkReal x634=((r01)*(sj0));
IkReal x635=((cj0)*(r00));
IkReal x636=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j3)))));
evalcond[1]=((((x634)*(x636)))+(((x635)*(x636)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(IKcos(j3))))+(((r02)*(sj1)*(sj2)))+(((x633)*(x634)))+(((x633)*(x635))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x637=((pz)*(sj2));
IkReal x638=((IkReal(1.00000000000000))*(cj1));
IkReal x639=((cj2)*(sj0));
IkReal x640=((py)*(sj1));
IkReal x641=((cj0)*(sj1));
IkReal x642=((cj2)*(px));
IkReal x643=((sj0)*(sj2));
IkReal x644=((IkReal(0.0750000000000000))*(cj2));
IkReal x645=((px)*(sj2));
IkReal x646=((IkReal(0.0750000000000000))*(sj2));
IkReal x647=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r00)*(sj2)*(x641)))+(((IkReal(-1.00000000000000))*(r02)*(x647)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x638)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x638)))+(((IkReal(-1.00000000000000))*(r01)*(x638)*(x639)))+(((r01)*(sj1)*(x643))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(cj1)*(x645)))+(((IkReal(-1.00000000000000))*(cj1)*(x646)))+(((cj1)*(py)*(x643)))+(((x639)*(x640)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x644)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x638)))+(((x641)*(x642)))+(((sj1)*(x637))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(x638)*(x642)))+(((IkReal(-1.00000000000000))*(x637)*(x638)))+(((IkReal(-1.00000000000000))*(sj1)*(x646)))+(((cj1)*(x644)))+(((x640)*(x643)))+(((IkReal(-1.00000000000000))*(py)*(x638)*(x639)))+(((x641)*(x645)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x647))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x648=((IkReal(1.00000000000000))*(sj2));
IkReal x649=((cj0)*(r00));
IkReal x650=((r01)*(sj0));
IkReal x651=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(x648)*(x649)))+(((IkReal(-1.00000000000000))*(x650)*(x651)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x648)))+(((IkReal(-1.00000000000000))*(x649)*(x651)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x648)*(x650))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(x648)*(x649)))+(((IkReal(-1.00000000000000))*(x650)*(x651)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x648)))+(((IkReal(-1.00000000000000))*(x649)*(x651)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x648)*(x650)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(cj1)*(x648)*(x649)))+(((IkReal(-1.00000000000000))*(x650)*(x651)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x648)))+(((IkReal(-1.00000000000000))*(x649)*(x651)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x648)*(x650)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x652=((cj1)*(sj2));
IkReal x653=((r01)*(sj0));
IkReal x654=((cj0)*(r00));
IkReal x655=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j3))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x652)*(x654)))+(((x652)*(x653)))+(((x653)*(x655)))+(((x654)*(x655)))+(IKcos(j3))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x656=((cj1)*(sj2));
IkReal x657=((r01)*(sj0));
IkReal x658=((cj0)*(r00));
IkReal x659=((cj2)*(sj1));
if( IKabs(((IkReal(0.500000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x656)*(x657)))+(((x656)*(x658)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x657)*(x659)))+(((r02)*(sj1)*(sj2)))+(((x658)*(x659))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.500000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x656)*(x657)))+(((x656)*(x658)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x657)*(x659)))+(((r02)*(sj1)*(sj2)))+(((x658)*(x659)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(0.500000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x656)*(x657)))+(((x656)*(x658)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x657)*(x659)))+(((r02)*(sj1)*(sj2)))+(((x658)*(x659)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x660=IKsin(j3);
IkReal x661=IKcos(j3);
IkReal x662=((sj1)*(sj2));
IkReal x663=((IkReal(0.0800000000000000))*(sj4));
IkReal x664=((cj2)*(sj1));
IkReal x665=((py)*(sj0));
IkReal x666=((IkReal(1.00000000000000))*(cj0));
IkReal x667=((IkReal(1.00000000000000))*(cj4));
IkReal x668=((r01)*(sj0));
IkReal x669=((cj1)*(sj2));
IkReal x670=((cj0)*(x669));
IkReal x671=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x660)*(x663)))+(((IkReal(-1.00000000000000))*(py)*(x666)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x666)))+(((IkReal(-1.00000000000000))*(x660)*(x667))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x671)))+(((x664)*(x668)))+(((x668)*(x669)))+(((r00)*(x670)))+(((cj0)*(r00)*(x664)))+(((IkReal(-1.00000000000000))*(x661)*(x667)))+(((r02)*(x662))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x665)*(x669)))+(((x664)*(x665)))+(((IkReal(-0.0750000000000000))*(x669)))+(((IkReal(-0.0750000000000000))*(x664)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x664)))+(((IkReal(-1.00000000000000))*(pz)*(x671)))+(((pz)*(x662)))+(((x661)*(x663)))+(((px)*(x670))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x672=((cj1)*(sj2));
IkReal x673=((r01)*(sj0));
IkReal x674=((cj0)*(r00));
IkReal x675=((cj2)*(sj1));
if( IKabs(((gconst59)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((x673)*(x675)))+(((x672)*(x674)))+(((x672)*(x673)))+(((x674)*(x675)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst59)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((gconst59)*(((((x673)*(x675)))+(((x672)*(x674)))+(((x672)*(x673)))+(((x674)*(x675)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x676=IKsin(j3);
IkReal x677=IKcos(j3);
IkReal x678=((sj1)*(sj2));
IkReal x679=((IkReal(0.0800000000000000))*(sj4));
IkReal x680=((cj2)*(sj1));
IkReal x681=((py)*(sj0));
IkReal x682=((IkReal(1.00000000000000))*(cj0));
IkReal x683=((IkReal(1.00000000000000))*(cj4));
IkReal x684=((r01)*(sj0));
IkReal x685=((cj1)*(sj2));
IkReal x686=((cj0)*(x685));
IkReal x687=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x676)*(x679)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x682))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x682)))+(((IkReal(-1.00000000000000))*(x676)*(x683))));
evalcond[2]=((((r02)*(x678)))+(((IkReal(-1.00000000000000))*(r02)*(x687)))+(((cj0)*(r00)*(x680)))+(((x680)*(x684)))+(((IkReal(-1.00000000000000))*(x677)*(x683)))+(((r00)*(x686)))+(((x684)*(x685))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((px)*(x686)))+(((pz)*(x678)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x677)*(x679)))+(((IkReal(-0.0750000000000000))*(x685)))+(((IkReal(-0.0750000000000000))*(x680)))+(((x681)*(x685)))+(((x680)*(x681)))+(((IkReal(-1.00000000000000))*(pz)*(x687)))+(((cj0)*(px)*(x680))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x688=((cj1)*(cj2));
IkReal x689=((IkReal(1.00000000000000))*(cj0));
IkReal x690=((sj1)*(sj2));
IkReal x691=((IkReal(1.00000000000000))*(sj0));
IkReal x692=((sj0)*(x690));
IkReal x693=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x694=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x688)*(x689)))+(((IkReal(-1.00000000000000))*(sj4)))+(((cj0)*(r00)*(x690)))+(((IkReal(-1.00000000000000))*(r01)*(x688)*(x691)))+(((IkReal(-1.00000000000000))*(r02)*(x693)))+(((IkReal(-1.00000000000000))*(r02)*(x694)))+(((r01)*(x692))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x690)))+(((IkReal(-1.00000000000000))*(px)*(x688)*(x689)))+(((py)*(x692)))+(((IkReal(0.0750000000000000))*(x688)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x693)))+(((IkReal(-1.00000000000000))*(pz)*(x694)))+(((cj0)*(px)*(x690)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x688)*(x691))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x695=((IkReal(1.00000000000000))*(sj2));
IkReal x696=((cj0)*(r00));
IkReal x697=((r01)*(sj0));
IkReal x698=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(x696)*(x698)))+(((IkReal(-1.00000000000000))*(x697)*(x698)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x695)))+(((IkReal(-1.00000000000000))*(cj1)*(x695)*(x696)))+(((IkReal(-1.00000000000000))*(cj1)*(x695)*(x697)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst60)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst60)*(((((IkReal(-1.00000000000000))*(x696)*(x698)))+(((IkReal(-1.00000000000000))*(x697)*(x698)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x695)))+(((IkReal(-1.00000000000000))*(cj1)*(x695)*(x696)))+(((IkReal(-1.00000000000000))*(cj1)*(x695)*(x697)))+(((cj1)*(cj2)*(r02)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x699=IKsin(j3);
IkReal x700=IKcos(j3);
IkReal x701=((sj1)*(sj2));
IkReal x702=((cj2)*(sj1));
IkReal x703=((IkReal(0.0800000000000000))*(sj4));
IkReal x704=((py)*(sj0));
IkReal x705=((IkReal(1.00000000000000))*(cj0));
IkReal x706=((r01)*(sj0));
IkReal x707=((cj1)*(sj2));
IkReal x708=((cj0)*(x707));
IkReal x709=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x705)))+(((px)*(sj0)))+(((x699)*(x703))));
evalcond[1]=((((r00)*(sj0)))+(((cj4)*(x699)))+(((IkReal(-1.00000000000000))*(r01)*(x705))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x709)))+(((x702)*(x706)))+(((cj4)*(x700)))+(((x706)*(x707)))+(((r00)*(x708)))+(((cj0)*(r00)*(x702)))+(((r02)*(x701))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x709)))+(((IkReal(-0.0750000000000000))*(x702)))+(((IkReal(-0.0750000000000000))*(x707)))+(((x702)*(x704)))+(((x700)*(x703)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x702)))+(((px)*(x708)))+(((x704)*(x707)))+(((pz)*(x701))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x710=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x711=((r01)*(sj0));
IkReal x712=((IkReal(0.500000000000000))*(x710));
IkReal x713=((cj0)*(r00));
IkReal x714=((IkReal(25.0000000000000))*(cj0)*(py));
IkReal x715=((IkReal(25.0000000000000))*(px)*(sj0));
IkReal x716=((IkReal(2.00000000000000))*(cj2)*(sj4));
IkReal x717=((IkReal(2.00000000000000))*(cj1)*(sj2)*(sj4));
if( IKabs(((x712)*(((((IkReal(-1.00000000000000))*(x715)))+(x714))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x712)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((sj1)*(x713)*(x716)))+(((x711)*(x717)))+(((sj5)*(x714)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x716)))+(((IkReal(-1.00000000000000))*(sj5)*(x715)))+(((IkReal(2.00000000000000))*(r02)*(sj1)*(sj2)*(sj4)))+(((sj1)*(x711)*(x716)))+(((x713)*(x717))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x712)*(((((IkReal(-1.00000000000000))*(x715)))+(x714)))))+IKsqr(((x712)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((sj1)*(x713)*(x716)))+(((x711)*(x717)))+(((sj5)*(x714)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x716)))+(((IkReal(-1.00000000000000))*(sj5)*(x715)))+(((IkReal(2.00000000000000))*(r02)*(sj1)*(sj2)*(sj4)))+(((sj1)*(x711)*(x716)))+(((x713)*(x717)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x712)*(((((IkReal(-1.00000000000000))*(x715)))+(x714)))), ((x712)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((sj1)*(x713)*(x716)))+(((x711)*(x717)))+(((sj5)*(x714)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x716)))+(((IkReal(-1.00000000000000))*(sj5)*(x715)))+(((IkReal(2.00000000000000))*(r02)*(sj1)*(sj2)*(sj4)))+(((sj1)*(x711)*(x716)))+(((x713)*(x717)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x718=IKsin(j3);
IkReal x719=IKcos(j3);
IkReal x720=((sj1)*(sj2));
IkReal x721=((IkReal(0.0800000000000000))*(sj4));
IkReal x722=((cj2)*(sj1));
IkReal x723=((py)*(sj0));
IkReal x724=((IkReal(1.00000000000000))*(cj0));
IkReal x725=((r01)*(sj0));
IkReal x726=((cj1)*(sj2));
IkReal x727=((cj0)*(x726));
IkReal x728=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x729=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x718)*(x721)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x724))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x719)))+(((IkReal(-1.00000000000000))*(x718)*(x728)))+(((IkReal(-1.00000000000000))*(r01)*(x724))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x729)))+(((sj5)*(x718)))+(((x722)*(x725)))+(((r00)*(x727)))+(((cj0)*(r00)*(x722)))+(((r02)*(x720)))+(((x725)*(x726)))+(((IkReal(-1.00000000000000))*(x719)*(x728))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x722)))+(((IkReal(-0.0750000000000000))*(x726)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x723)*(x726)))+(((x722)*(x723)))+(((cj0)*(px)*(x722)))+(((px)*(x727)))+(((x719)*(x721)))+(((pz)*(x720)))+(((IkReal(-1.00000000000000))*(pz)*(x729))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x730=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x731=((IkReal(2.00000000000000))*(sj4));
IkReal x732=((cj4)*(cj5));
IkReal x733=((IkReal(0.500000000000000))*(x730));
IkReal x734=((IkReal(25.0000000000000))*(cj0)*(py));
IkReal x735=((IkReal(25.0000000000000))*(px)*(sj0));
if( IKabs(((x733)*(((((IkReal(-1.00000000000000))*(x735)))+(x734))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x733)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x732)*(x734)))+(((r00)*(sj0)*(x731)))+(((x732)*(x735)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x731))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x733)*(((((IkReal(-1.00000000000000))*(x735)))+(x734)))))+IKsqr(((x733)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x732)*(x734)))+(((r00)*(sj0)*(x731)))+(((x732)*(x735)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x731)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x733)*(((((IkReal(-1.00000000000000))*(x735)))+(x734)))), ((x733)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x732)*(x734)))+(((r00)*(sj0)*(x731)))+(((x732)*(x735)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x731)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x736=IKsin(j3);
IkReal x737=IKcos(j3);
IkReal x738=((sj1)*(sj2));
IkReal x739=((IkReal(0.0800000000000000))*(sj4));
IkReal x740=((cj2)*(sj1));
IkReal x741=((py)*(sj0));
IkReal x742=((IkReal(1.00000000000000))*(cj0));
IkReal x743=((r01)*(sj0));
IkReal x744=((cj1)*(sj2));
IkReal x745=((cj0)*(x744));
IkReal x746=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x747=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x736)*(x739)))+(((IkReal(-1.00000000000000))*(py)*(x742)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x737)))+(((IkReal(-1.00000000000000))*(r01)*(x742)))+(((IkReal(-1.00000000000000))*(x736)*(x746))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x737)*(x746)))+(((x740)*(x743)))+(((sj5)*(x736)))+(((IkReal(-1.00000000000000))*(r02)*(x747)))+(((r00)*(x745)))+(((x743)*(x744)))+(((r02)*(x738)))+(((cj0)*(r00)*(x740))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(px)*(x740)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x744)))+(((IkReal(-0.0750000000000000))*(x740)))+(((px)*(x745)))+(((x740)*(x741)))+(((x741)*(x744)))+(((IkReal(-1.00000000000000))*(pz)*(x747)))+(((pz)*(x738)))+(((x737)*(x739))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x748=((cj4)*(cj5));
IkReal x749=((r00)*(sj0));
IkReal x750=((cj0)*(r00));
IkReal x751=((r01)*(sj0));
IkReal x752=((IkReal(1.00000000000000))*(cj2));
IkReal x753=((cj1)*(r02));
IkReal x754=((sj1)*(sj5));
IkReal x755=((cj1)*(sj2));
IkReal x756=((cj2)*(sj1));
IkReal x757=((r02)*(sj2));
IkReal x758=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x759=((IkReal(1.00000000000000))*(sj5)*(x755));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x750)*(x759)))+(((IkReal(-1.00000000000000))*(x751)*(x759)))+(((IkReal(-1.00000000000000))*(x754)*(x757)))+(((cj2)*(sj5)*(x753)))+(((IkReal(-1.00000000000000))*(x751)*(x752)*(x754)))+(((IkReal(-1.00000000000000))*(x750)*(x752)*(x754)))+(((IkReal(-1.00000000000000))*(x748)*(x758)))+(((x748)*(x749))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((x748)*(x751)*(x756)))+(((x748)*(x751)*(x755)))+(((x748)*(x750)*(x755)))+(((x748)*(x750)*(x756)))+(((sj1)*(x748)*(x757)))+(((IkReal(-1.00000000000000))*(x748)*(x752)*(x753)))+(((sj5)*(x749))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(x750)*(x759)))+(((IkReal(-1.00000000000000))*(x751)*(x759)))+(((IkReal(-1.00000000000000))*(x754)*(x757)))+(((cj2)*(sj5)*(x753)))+(((IkReal(-1.00000000000000))*(x751)*(x752)*(x754)))+(((IkReal(-1.00000000000000))*(x750)*(x752)*(x754)))+(((IkReal(-1.00000000000000))*(x748)*(x758)))+(((x748)*(x749)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(sj5)*(x758)))+(((x748)*(x751)*(x756)))+(((x748)*(x751)*(x755)))+(((x748)*(x750)*(x755)))+(((x748)*(x750)*(x756)))+(((sj1)*(x748)*(x757)))+(((IkReal(-1.00000000000000))*(x748)*(x752)*(x753)))+(((sj5)*(x749)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x760=IKsin(j3);
IkReal x761=IKcos(j3);
IkReal x762=((sj1)*(sj2));
IkReal x763=((IkReal(0.0800000000000000))*(sj4));
IkReal x764=((cj2)*(sj1));
IkReal x765=((py)*(sj0));
IkReal x766=((IkReal(1.00000000000000))*(cj0));
IkReal x767=((r01)*(sj0));
IkReal x768=((cj1)*(sj2));
IkReal x769=((cj0)*(x768));
IkReal x770=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x771=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x766)))+(((x760)*(x763)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x766)))+(((IkReal(-1.00000000000000))*(sj5)*(x761)))+(((IkReal(-1.00000000000000))*(x760)*(x770))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x771)))+(((r02)*(x762)))+(((cj0)*(r00)*(x764)))+(((x767)*(x768)))+(((IkReal(-1.00000000000000))*(x761)*(x770)))+(((x764)*(x767)))+(((r00)*(x769)))+(((sj5)*(x760))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(px)*(x764)))+(((IkReal(-0.0750000000000000))*(x768)))+(((IkReal(-0.0750000000000000))*(x764)))+(((px)*(x769)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x761)*(x763)))+(((pz)*(x762)))+(((IkReal(-1.00000000000000))*(pz)*(x771)))+(((x765)*(x768)))+(((x764)*(x765))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x772=((IkReal(25.0000000000000))*(cj0));
IkReal x773=((IkReal(25.0000000000000))*(sj0));
IkReal x774=((cj1)*(sj2));
IkReal x775=((cj2)*(sj1));
IkReal x776=((IkReal(25.0000000000000))*(pz));
if( IKabs(((gconst53)*(((((IkReal(-1.00000000000000))*(px)*(x773)))+(((py)*(x772))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x776)))+(((IkReal(-1.00000000000000))*(py)*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(py)*(x773)*(x775)))+(((IkReal(1.87500000000000))*(x774)))+(((IkReal(1.87500000000000))*(x775)))+(((IkReal(-1.00000000000000))*(px)*(x772)*(x774)))+(((IkReal(-1.00000000000000))*(px)*(x772)*(x775)))+(((IkReal(6.75000000000000))*(cj2)))+(((cj1)*(cj2)*(x776))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst53)*(((((IkReal(-1.00000000000000))*(px)*(x773)))+(((py)*(x772)))))), ((gconst53)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x776)))+(((IkReal(-1.00000000000000))*(py)*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(py)*(x773)*(x775)))+(((IkReal(1.87500000000000))*(x774)))+(((IkReal(1.87500000000000))*(x775)))+(((IkReal(-1.00000000000000))*(px)*(x772)*(x774)))+(((IkReal(-1.00000000000000))*(px)*(x772)*(x775)))+(((IkReal(6.75000000000000))*(cj2)))+(((cj1)*(cj2)*(x776)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x777=((cj1)*(sj2));
IkReal x778=((IkReal(0.0800000000000000))*(sj4));
IkReal x779=((cj2)*(sj1));
IkReal x780=((cj0)*(px));
IkReal x781=((py)*(sj0));
evalcond[0]=((((x778)*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((x778)*(IKcos(j3))))+(((IkReal(-0.0750000000000000))*(x779)))+(((IkReal(-0.0750000000000000))*(x777)))+(((pz)*(sj1)*(sj2)))+(((x779)*(x781)))+(((x779)*(x780)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((x777)*(x781)))+(((x777)*(x780))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x782=(sj3)*(sj3);
IkReal x783=(cj3)*(cj3);
dummyeval[0]=((x782)+(x783));
dummyeval[1]=((((cj4)*(x783)))+(((cj4)*(x782))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x784=((pz)*(sj2));
IkReal x785=((IkReal(1.00000000000000))*(cj1));
IkReal x786=((cj2)*(sj1));
IkReal x787=((py)*(sj0));
IkReal x788=((cj0)*(px));
IkReal x789=((sj1)*(sj2));
IkReal x790=((IkReal(0.0750000000000000))*(cj1));
IkReal x791=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(0.0800000000000000))*(sj4)))+(((x786)*(x788)))+(((x786)*(x787)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x784)))+(((x788)*(x791)))+(((x787)*(x791)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x785)))+(((IkReal(-1.00000000000000))*(sj2)*(x790)))+(((IkReal(-0.0750000000000000))*(x786))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x785)*(x787)))+(((IkReal(-1.00000000000000))*(cj2)*(x785)*(x788)))+(((x788)*(x789)))+(((x787)*(x789)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x784)*(x785)))+(((IkReal(-1.00000000000000))*(pz)*(x786)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x790)))+(((IkReal(-0.0750000000000000))*(x789))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x792=((pz)*(sj2));
IkReal x793=((IkReal(1.00000000000000))*(cj1));
IkReal x794=((cj2)*(sj0));
IkReal x795=((py)*(sj1));
IkReal x796=((cj0)*(sj1));
IkReal x797=((cj2)*(px));
IkReal x798=((sj0)*(sj2));
IkReal x799=((IkReal(0.0750000000000000))*(cj2));
IkReal x800=((px)*(sj2));
IkReal x801=((IkReal(0.0750000000000000))*(sj2));
IkReal x802=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x798)))+(((IkReal(-1.00000000000000))*(r02)*(x802)))+(((IkReal(-1.00000000000000))*(r01)*(x793)*(x794)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x793)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x793)))+(((r00)*(sj2)*(x796))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x796)*(x797)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x800)))+(((IkReal(-1.00000000000000))*(cj1)*(x801)))+(((sj1)*(x792)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x793)))+(((cj1)*(py)*(x798)))+(((x794)*(x795)))+(((IkReal(-1.00000000000000))*(sj1)*(x799))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(x792)*(x793)))+(((IkReal(-1.00000000000000))*(sj1)*(x801)))+(((cj1)*(x799)))+(((IkReal(-1.00000000000000))*(cj0)*(x793)*(x797)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x793)*(x794)))+(((IkReal(-1.00000000000000))*(pz)*(x802)))+(((x796)*(x800)))+(((x795)*(x798))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x803=((cj1)*(sj2));
IkReal x804=((r01)*(sj0));
IkReal x805=((cj0)*(r00));
IkReal x806=((cj2)*(sj1));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x804)*(x806)))+(((x805)*(x806)))+(((x803)*(x805)))+(((x803)*(x804)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))+IKsqr(((((x804)*(x806)))+(((x805)*(x806)))+(((x803)*(x805)))+(((x803)*(x804)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))), ((((x804)*(x806)))+(((x805)*(x806)))+(((x803)*(x805)))+(((x803)*(x804)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x807=((cj1)*(sj2));
IkReal x808=((r01)*(sj0));
IkReal x809=((cj0)*(r00));
IkReal x810=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((x809)*(x810)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((x808)*(x810)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x807)*(x808)))+(((x807)*(x809))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x811=((pz)*(sj2));
IkReal x812=((IkReal(1.00000000000000))*(cj1));
IkReal x813=((cj2)*(sj0));
IkReal x814=((py)*(sj1));
IkReal x815=((cj0)*(sj1));
IkReal x816=((cj2)*(px));
IkReal x817=((sj0)*(sj2));
IkReal x818=((IkReal(0.0750000000000000))*(cj2));
IkReal x819=((px)*(sj2));
IkReal x820=((IkReal(0.0750000000000000))*(sj2));
IkReal x821=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x821)))+(((r01)*(sj1)*(x817)))+(((IkReal(-1.00000000000000))*(r01)*(x812)*(x813)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x812)))+(((r00)*(sj2)*(x815)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x812))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x815)*(x816)))+(((sj1)*(x811)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x813)*(x814)))+(((cj1)*(py)*(x817)))+(((cj0)*(cj1)*(x819)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x812)))+(((IkReal(-1.00000000000000))*(sj1)*(x818)))+(((IkReal(-1.00000000000000))*(cj1)*(x820))));
evalcond[4]=((IkReal(0.215000000000000))+(((x815)*(x819)))+(((IkReal(-1.00000000000000))*(pz)*(x821)))+(((IkReal(-1.00000000000000))*(x811)*(x812)))+(((x814)*(x817)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x820)))+(((cj1)*(x818)))+(((IkReal(-1.00000000000000))*(py)*(x812)*(x813)))+(((IkReal(-1.00000000000000))*(cj0)*(x812)*(x816))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x822=((IkReal(1.00000000000000))*(sj1));
IkReal x823=((IkReal(1.00000000000000))*(cj0));
IkReal x824=((r01)*(sj0));
IkReal x825=((cj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(r01)*(x823)))+(((r00)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x822)))+(((IkReal(-1.00000000000000))*(r00)*(x823)*(x825)))+(((IkReal(-1.00000000000000))*(cj2)*(x822)*(x824)))+(((IkReal(-1.00000000000000))*(x824)*(x825)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x822)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r01)*(x823)))+(((r00)*(sj0)))))+IKsqr(((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x822)))+(((IkReal(-1.00000000000000))*(r00)*(x823)*(x825)))+(((IkReal(-1.00000000000000))*(cj2)*(x822)*(x824)))+(((IkReal(-1.00000000000000))*(x824)*(x825)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x822)))+(((cj1)*(cj2)*(r02)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r01)*(x823)))+(((r00)*(sj0)))), ((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x822)))+(((IkReal(-1.00000000000000))*(r00)*(x823)*(x825)))+(((IkReal(-1.00000000000000))*(cj2)*(x822)*(x824)))+(((IkReal(-1.00000000000000))*(x824)*(x825)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x822)))+(((cj1)*(cj2)*(r02)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x826=((cj1)*(sj2));
IkReal x827=((r01)*(sj0));
IkReal x828=((cj0)*(r00));
IkReal x829=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((IKcos(j5))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x828)*(x829)))+(((r02)*(sj1)*(sj2)))+(((x827)*(x829)))+(((x826)*(x827)))+(((x826)*(x828))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x830=((pz)*(sj2));
IkReal x831=((IkReal(1.00000000000000))*(cj1));
IkReal x832=((cj2)*(sj1));
IkReal x833=((py)*(sj0));
IkReal x834=((cj0)*(px));
IkReal x835=((sj1)*(sj2));
IkReal x836=((IkReal(0.0750000000000000))*(cj1));
IkReal x837=((r01)*(sj0));
IkReal x838=((cj1)*(sj2));
IkReal x839=((cj0)*(x838));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((x832)*(x837)))+(((r02)*(x835)))+(((cj0)*(r00)*(x832)))+(((r00)*(x839)))+(((x837)*(x838)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x831))));
evalcond[3]=((IkReal(-0.0100000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x832)))+(((x832)*(x833)))+(((x832)*(x834)))+(((x833)*(x838)))+(((x834)*(x838)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x831)))+(((sj1)*(x830)))+(((IkReal(-1.00000000000000))*(sj2)*(x836))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x831)*(x833)))+(((IkReal(-1.00000000000000))*(cj2)*(x831)*(x834)))+(((IkReal(-0.0750000000000000))*(x835)))+(((x833)*(x835)))+(((IkReal(-1.00000000000000))*(x830)*(x831)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x834)*(x835)))+(((IkReal(-1.00000000000000))*(pz)*(x832)))+(((cj2)*(x836))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x840=((IkReal(1.00000000000000))*(cj0));
IkReal x841=((cj1)*(cj2));
IkReal x842=((sj1)*(sj2));
IkReal x843=((r01)*(sj0));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x840))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x842)*(x843)))+(((IkReal(-1.00000000000000))*(r00)*(x840)*(x842)))+(((cj0)*(r00)*(x841)))+(((cj2)*(r02)*(sj1)))+(((x841)*(x843)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x840)))))+IKsqr(((((IkReal(-1.00000000000000))*(x842)*(x843)))+(((IkReal(-1.00000000000000))*(r00)*(x840)*(x842)))+(((cj0)*(r00)*(x841)))+(((cj2)*(r02)*(sj1)))+(((x841)*(x843)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x840)))), ((((IkReal(-1.00000000000000))*(x842)*(x843)))+(((IkReal(-1.00000000000000))*(r00)*(x840)*(x842)))+(((cj0)*(r00)*(x841)))+(((cj2)*(r02)*(sj1)))+(((x841)*(x843)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x844=((r01)*(sj0));
IkReal x845=((sj1)*(sj2));
IkReal x846=((IkReal(1.00000000000000))*(cj0));
IkReal x847=((cj1)*(cj2));
IkReal x848=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x846)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((cj0)*(r00)*(x845)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x848)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x848)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(r00)*(x846)*(x847)))+(((IkReal(-1.00000000000000))*(x844)*(x847)))+(((x844)*(x845))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x849=((pz)*(sj2));
IkReal x850=((IkReal(1.00000000000000))*(cj1));
IkReal x851=((cj2)*(sj1));
IkReal x852=((py)*(sj0));
IkReal x853=((cj0)*(px));
IkReal x854=((sj1)*(sj2));
IkReal x855=((IkReal(0.0750000000000000))*(cj1));
IkReal x856=((r01)*(sj0));
IkReal x857=((cj1)*(sj2));
IkReal x858=((cj0)*(x857));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r00)*(x858)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x850)))+(((x856)*(x857)))+(((x851)*(x856)))+(((r02)*(x854)))+(((cj0)*(r00)*(x851))));
evalcond[3]=((IkReal(-0.170000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x850)))+(((IkReal(-1.00000000000000))*(sj2)*(x855)))+(((x851)*(x852)))+(((x851)*(x853)))+(((x852)*(x857)))+(((IkReal(-0.0750000000000000))*(x851)))+(((x853)*(x857)))+(((sj1)*(x849))));
evalcond[4]=((IkReal(0.295000000000000))+(((cj2)*(x855)))+(((IkReal(-1.00000000000000))*(x849)*(x850)))+(((IkReal(-1.00000000000000))*(pz)*(x851)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x850)*(x853)))+(((IkReal(-1.00000000000000))*(cj2)*(x850)*(x852)))+(((x852)*(x854)))+(((IkReal(-0.0750000000000000))*(x854)))+(((x853)*(x854))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x859=((r01)*(sj0));
IkReal x860=((sj1)*(sj2));
IkReal x861=((IkReal(1.00000000000000))*(cj0));
IkReal x862=((cj1)*(cj2));
IkReal x863=((IkReal(1.00000000000000))*(r02));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x861))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(x861)*(x862)))+(((x859)*(x860)))+(((cj0)*(r00)*(x860)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x863)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x863)))+(((IkReal(-1.00000000000000))*(x859)*(x862))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x861)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(x861)*(x862)))+(((x859)*(x860)))+(((cj0)*(r00)*(x860)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x863)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x863)))+(((IkReal(-1.00000000000000))*(x859)*(x862)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x861)))), ((((IkReal(-1.00000000000000))*(r00)*(x861)*(x862)))+(((x859)*(x860)))+(((cj0)*(r00)*(x860)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x863)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x863)))+(((IkReal(-1.00000000000000))*(x859)*(x862)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x864=((r01)*(sj0));
IkReal x865=((sj1)*(sj2));
IkReal x866=((IkReal(1.00000000000000))*(cj0));
IkReal x867=((cj1)*(cj2));
IkReal x868=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x866)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((cj0)*(r00)*(x865)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x868)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x868)))+(((IkReal(-1.00000000000000))*(x864)*(x867)))+(((IkReal(-1.00000000000000))*(r00)*(x866)*(x867)))+(((x864)*(x865))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x869=((IkReal(1.00000000000000))*(cj0));
IkReal x870=((cj1)*(cj2));
IkReal x871=((sj1)*(sj2));
IkReal x872=((r01)*(sj0));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x869))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x870)*(x872)))+(((IkReal(-1.00000000000000))*(x871)*(x872)))+(((IkReal(-1.00000000000000))*(r00)*(x869)*(x871)))+(((cj0)*(r00)*(x870)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x869)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x870)*(x872)))+(((IkReal(-1.00000000000000))*(x871)*(x872)))+(((IkReal(-1.00000000000000))*(r00)*(x869)*(x871)))+(((cj0)*(r00)*(x870)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x869)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x870)*(x872)))+(((IkReal(-1.00000000000000))*(x871)*(x872)))+(((IkReal(-1.00000000000000))*(r00)*(x869)*(x871)))+(((cj0)*(r00)*(x870)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x873=IKcos(j5);
IkReal x874=((r01)*(sj0));
IkReal x875=((sj1)*(sj2));
IkReal x876=((IkReal(1.00000000000000))*(cj0));
IkReal x877=((cj1)*(cj2));
IkReal x878=((cj1)*(sj2));
IkReal x879=((cj0)*(r00));
IkReal x880=((cj2)*(sj1));
IkReal x881=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x876)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((x874)*(x875)))+(((IkReal(-1.00000000000000))*(x874)*(x877)))+(((sj4)*(x873)))+(((x875)*(x879)))+(((IkReal(-1.00000000000000))*(r00)*(x876)*(x877)))+(((IkReal(-1.00000000000000))*(x880)*(x881)))+(((IkReal(-1.00000000000000))*(x878)*(x881))));
evalcond[2]=((((x874)*(x880)))+(((x879)*(x880)))+(((x878)*(x879)))+(((x874)*(x878)))+(((IkReal(-1.00000000000000))*(x877)*(x881)))+(((r02)*(x875)))+(((IkReal(-1.00000000000000))*(cj4)*(x873))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x882=((cj1)*(sj2));
IkReal x883=((r01)*(sj0));
IkReal x884=((cj0)*(r00));
IkReal x885=((cj2)*(sj1));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x883)*(x885)))+(((x882)*(x884)))+(((x882)*(x883)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x884)*(x885))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x883)*(x885)))+(((x882)*(x884)))+(((x882)*(x883)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x884)*(x885)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x883)*(x885)))+(((x882)*(x884)))+(((x882)*(x883)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x884)*(x885)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x886=IKcos(j5);
IkReal x887=((r01)*(sj0));
IkReal x888=((sj1)*(sj2));
IkReal x889=((IkReal(1.00000000000000))*(cj0));
IkReal x890=((cj1)*(cj2));
IkReal x891=((cj1)*(sj2));
IkReal x892=((cj0)*(r00));
IkReal x893=((cj2)*(sj1));
IkReal x894=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((IkReal(-1.00000000000000))*(r01)*(x889))));
evalcond[1]=((((x887)*(x888)))+(((sj4)*(x886)))+(((IkReal(-1.00000000000000))*(x887)*(x890)))+(((IkReal(-1.00000000000000))*(r00)*(x889)*(x890)))+(((IkReal(-1.00000000000000))*(x893)*(x894)))+(((x888)*(x892)))+(((IkReal(-1.00000000000000))*(x891)*(x894))));
evalcond[2]=((((r02)*(x888)))+(((IkReal(-1.00000000000000))*(cj4)*(x886)))+(((x892)*(x893)))+(((x891)*(x892)))+(((IkReal(-1.00000000000000))*(x890)*(x894)))+(((x887)*(x891)))+(((x887)*(x893))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x895=((pz)*(sj2));
IkReal x896=((IkReal(1.00000000000000))*(cj1));
IkReal x897=((cj2)*(sj1));
IkReal x898=((py)*(sj0));
IkReal x899=((cj0)*(px));
IkReal x900=((sj1)*(sj2));
IkReal x901=((IkReal(0.0750000000000000))*(cj1));
IkReal x902=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x897)))+(((sj1)*(x895)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x901)))+(((x897)*(x898)))+(((x897)*(x899)))+(((x899)*(x902)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x896)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((x898)*(x902))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x895)*(x896)))+(((cj2)*(x901)))+(((IkReal(-1.00000000000000))*(cj2)*(x896)*(x899)))+(((IkReal(-1.00000000000000))*(cj2)*(x896)*(x898)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x899)*(x900)))+(((IkReal(-1.00000000000000))*(pz)*(x897)))+(((IkReal(-0.0750000000000000))*(x900)))+(((IkReal(0.0800000000000000))*(cj4)))+(((x898)*(x900))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x903=((pz)*(sj2));
IkReal x904=((IkReal(1.00000000000000))*(cj1));
IkReal x905=((cj2)*(sj1));
IkReal x906=((py)*(sj0));
IkReal x907=((cj0)*(px));
IkReal x908=((sj1)*(sj2));
IkReal x909=((IkReal(0.0750000000000000))*(cj1));
IkReal x910=((r01)*(sj0));
IkReal x911=((cj1)*(sj2));
IkReal x912=((cj0)*(x911));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((x910)*(x911)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x904)))+(((r02)*(x908)))+(((x905)*(x910)))+(((r00)*(x912)))+(((cj0)*(r00)*(x905))));
evalcond[3]=((IkReal(-0.170000000000000))+(((x905)*(x906)))+(((x905)*(x907)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x909)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x904)))+(((x906)*(x911)))+(((x907)*(x911)))+(((IkReal(-0.0750000000000000))*(x905)))+(((sj1)*(x903))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x903)*(x904)))+(((IkReal(-1.00000000000000))*(cj2)*(x904)*(x906)))+(((IkReal(-1.00000000000000))*(cj2)*(x904)*(x907)))+(((cj2)*(x909)))+(((x907)*(x908)))+(((x906)*(x908)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x908)))+(((IkReal(-1.00000000000000))*(pz)*(x905))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x913=((cj0)*(r00));
IkReal x914=((cj1)*(cj2));
IkReal x915=((r01)*(sj0));
IkReal x916=((IkReal(1.00000000000000))*(sj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x913)*(x916)))+(((x914)*(x915)))+(((cj2)*(r02)*(sj1)))+(((x913)*(x914)))+(((IkReal(-1.00000000000000))*(x915)*(x916)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(x913)*(x916)))+(((x914)*(x915)))+(((cj2)*(r02)*(sj1)))+(((x913)*(x914)))+(((IkReal(-1.00000000000000))*(x915)*(x916)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(x913)*(x916)))+(((x914)*(x915)))+(((cj2)*(r02)*(sj1)))+(((x913)*(x914)))+(((IkReal(-1.00000000000000))*(x915)*(x916)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x917=((r01)*(sj0));
IkReal x918=((sj1)*(sj2));
IkReal x919=((IkReal(1.00000000000000))*(cj0));
IkReal x920=((cj1)*(cj2));
IkReal x921=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x919)))+(IKsin(j5)));
evalcond[1]=((((x917)*(x918)))+(((cj0)*(r00)*(x918)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(r00)*(x919)*(x920)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x921)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x921)))+(((IkReal(-1.00000000000000))*(x917)*(x920))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x922=((pz)*(sj2));
IkReal x923=((IkReal(1.00000000000000))*(cj1));
IkReal x924=((cj2)*(sj1));
IkReal x925=((py)*(sj0));
IkReal x926=((cj0)*(px));
IkReal x927=((sj1)*(sj2));
IkReal x928=((IkReal(0.0750000000000000))*(cj1));
IkReal x929=((r01)*(sj0));
IkReal x930=((cj1)*(sj2));
IkReal x931=((cj0)*(x930));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((x924)*(x929)))+(((cj0)*(r00)*(x924)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x923)))+(((r00)*(x931)))+(((x929)*(x930)))+(((r02)*(x927))));
evalcond[3]=((IkReal(-0.0100000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x923)))+(((IkReal(-1.00000000000000))*(sj2)*(x928)))+(((x924)*(x926)))+(((x924)*(x925)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x922)))+(((IkReal(-0.0750000000000000))*(x924)))+(((x925)*(x930)))+(((x926)*(x930))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x923)*(x926)))+(((IkReal(-1.00000000000000))*(cj2)*(x923)*(x925)))+(((IkReal(-0.0750000000000000))*(x927)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj2)*(x928)))+(((IkReal(-1.00000000000000))*(pz)*(x924)))+(((x925)*(x927)))+(((x926)*(x927)))+(((IkReal(-1.00000000000000))*(x922)*(x923))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x932=((r01)*(sj0));
IkReal x933=((sj1)*(sj2));
IkReal x934=((IkReal(1.00000000000000))*(cj1));
IkReal x935=((cj0)*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj2)*(x932)*(x934)))+(((IkReal(-1.00000000000000))*(cj2)*(x934)*(x935)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x934)))+(((x933)*(x935)))+(((x932)*(x933))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(x932)*(x934)))+(((IkReal(-1.00000000000000))*(cj2)*(x934)*(x935)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x934)))+(((x933)*(x935)))+(((x932)*(x933)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(cj2)*(x932)*(x934)))+(((IkReal(-1.00000000000000))*(cj2)*(x934)*(x935)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x934)))+(((x933)*(x935)))+(((x932)*(x933)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x936=((r01)*(sj0));
IkReal x937=((sj1)*(sj2));
IkReal x938=((IkReal(1.00000000000000))*(cj0));
IkReal x939=((cj1)*(cj2));
IkReal x940=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(r01)*(x938))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x938)*(x939)))+(((x936)*(x937)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x940)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x936)*(x939)))+(((cj0)*(r00)*(x937)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x940))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x941=((pz)*(sj2));
IkReal x942=((IkReal(1.00000000000000))*(cj1));
IkReal x943=((cj2)*(sj0));
IkReal x944=((py)*(sj1));
IkReal x945=((cj0)*(sj1));
IkReal x946=((cj2)*(px));
IkReal x947=((sj0)*(sj2));
IkReal x948=((IkReal(0.0750000000000000))*(cj2));
IkReal x949=((px)*(sj2));
IkReal x950=((IkReal(0.0750000000000000))*(sj2));
IkReal x951=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x942)*(x943)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x942)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x942)))+(((r00)*(sj2)*(x945)))+(((r01)*(sj1)*(x947)))+(((IkReal(-1.00000000000000))*(r02)*(x951))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x942)))+(((IkReal(-1.00000000000000))*(cj1)*(x950)))+(((x945)*(x946)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x941)))+(((cj0)*(cj1)*(x949)))+(((x943)*(x944)))+(((IkReal(-1.00000000000000))*(sj1)*(x948))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x951)))+(((x945)*(x949)))+(((IkReal(-1.00000000000000))*(x941)*(x942)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj1)*(x948)))+(((IkReal(-1.00000000000000))*(cj0)*(x942)*(x946)))+(((IkReal(-1.00000000000000))*(sj1)*(x950)))+(((x944)*(x947)))+(((IkReal(-1.00000000000000))*(py)*(x942)*(x943))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x952=((IkReal(1.00000000000000))*(sj2));
IkReal x953=((cj0)*(r00));
IkReal x954=((r01)*(sj0));
IkReal x955=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(x952)*(x953)))+(((IkReal(-1.00000000000000))*(cj1)*(x952)*(x954)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x952)))+(((IkReal(-1.00000000000000))*(x953)*(x955)))+(((IkReal(-1.00000000000000))*(x954)*(x955)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(x952)*(x953)))+(((IkReal(-1.00000000000000))*(cj1)*(x952)*(x954)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x952)))+(((IkReal(-1.00000000000000))*(x953)*(x955)))+(((IkReal(-1.00000000000000))*(x954)*(x955)))+(((cj1)*(cj2)*(r02)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(cj1)*(x952)*(x953)))+(((IkReal(-1.00000000000000))*(cj1)*(x952)*(x954)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x952)))+(((IkReal(-1.00000000000000))*(x953)*(x955)))+(((IkReal(-1.00000000000000))*(x954)*(x955)))+(((cj1)*(cj2)*(r02)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x956=((cj1)*(sj2));
IkReal x957=((r01)*(sj0));
IkReal x958=((cj0)*(r00));
IkReal x959=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((IKcos(j5))+(((x956)*(x957)))+(((x956)*(x958)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x957)*(x959)))+(((x958)*(x959)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x960=((pz)*(sj2));
IkReal x961=((IkReal(1.00000000000000))*(cj1));
IkReal x962=((cj2)*(sj0));
IkReal x963=((py)*(sj1));
IkReal x964=((cj0)*(sj1));
IkReal x965=((cj2)*(px));
IkReal x966=((sj0)*(sj2));
IkReal x967=((IkReal(0.0750000000000000))*(cj2));
IkReal x968=((px)*(sj2));
IkReal x969=((IkReal(0.0750000000000000))*(sj2));
IkReal x970=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x961)))+(((IkReal(-1.00000000000000))*(r01)*(x961)*(x962)))+(((r01)*(sj1)*(x966)))+(((IkReal(-1.00000000000000))*(r02)*(x970)))+(((r00)*(sj2)*(x964)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x961))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x966)))+(((IkReal(-1.00000000000000))*(cj1)*(x969)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x968)))+(((x962)*(x963)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x961)))+(((x964)*(x965)))+(((sj1)*(x960)))+(((IkReal(-1.00000000000000))*(sj1)*(x967))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(x960)*(x961)))+(((IkReal(-1.00000000000000))*(pz)*(x970)))+(((cj1)*(x967)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x964)*(x968)))+(((x963)*(x966)))+(((IkReal(-1.00000000000000))*(sj1)*(x969)))+(((IkReal(-1.00000000000000))*(cj0)*(x961)*(x965)))+(((IkReal(-1.00000000000000))*(py)*(x961)*(x962))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x971=((cj1)*(sj2));
IkReal x972=((r01)*(sj0));
IkReal x973=((cj0)*(r00));
IkReal x974=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x973)*(x974)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x971)*(x972)))+(((x971)*(x973)))+(((r02)*(sj1)*(sj2)))+(((x972)*(x974))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((x973)*(x974)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x971)*(x972)))+(((x971)*(x973)))+(((r02)*(sj1)*(sj2)))+(((x972)*(x974)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((x973)*(x974)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x971)*(x972)))+(((x971)*(x973)))+(((r02)*(sj1)*(sj2)))+(((x972)*(x974)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x975=((cj1)*(sj2));
IkReal x976=((r01)*(sj0));
IkReal x977=((cj0)*(r00));
IkReal x978=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x975)*(x976)))+(((x975)*(x977)))+(((x977)*(x978)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((x976)*(x978)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x979=((IkReal(1.00000000000000))*(sj2));
IkReal x980=((cj0)*(r00));
IkReal x981=((r01)*(sj0));
IkReal x982=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(x979)*(x981)))+(((IkReal(-1.00000000000000))*(cj1)*(x979)*(x980)))+(((IkReal(-1.00000000000000))*(x980)*(x982)))+(((IkReal(-1.00000000000000))*(x981)*(x982)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x979)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(x979)*(x981)))+(((IkReal(-1.00000000000000))*(cj1)*(x979)*(x980)))+(((IkReal(-1.00000000000000))*(x980)*(x982)))+(((IkReal(-1.00000000000000))*(x981)*(x982)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x979)))+(((cj1)*(cj2)*(r02)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(x979)*(x981)))+(((IkReal(-1.00000000000000))*(cj1)*(x979)*(x980)))+(((IkReal(-1.00000000000000))*(x980)*(x982)))+(((IkReal(-1.00000000000000))*(x981)*(x982)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x979)))+(((cj1)*(cj2)*(r02)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x983=IKcos(j5);
IkReal x984=((r01)*(sj0));
IkReal x985=((sj1)*(sj2));
IkReal x986=((IkReal(1.00000000000000))*(cj0));
IkReal x987=((cj1)*(cj2));
IkReal x988=((cj1)*(sj2));
IkReal x989=((cj0)*(r00));
IkReal x990=((cj2)*(sj1));
IkReal x991=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x986)))+(IKsin(j5)));
evalcond[1]=((((IkReal(-1.00000000000000))*(x984)*(x987)))+(((IkReal(-1.00000000000000))*(x990)*(x991)))+(((sj4)*(x983)))+(((IkReal(-1.00000000000000))*(r00)*(x986)*(x987)))+(((x984)*(x985)))+(((x985)*(x989)))+(((IkReal(-1.00000000000000))*(x988)*(x991))));
evalcond[2]=((((cj4)*(x983)))+(((x989)*(x990)))+(((x988)*(x989)))+(((x984)*(x988)))+(((r02)*(x985)))+(((IkReal(-1.00000000000000))*(x987)*(x991)))+(((x984)*(x990))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x992=((cj0)*(r00));
IkReal x993=((cj1)*(cj2));
IkReal x994=((r01)*(sj0));
IkReal x995=((IkReal(1.00000000000000))*(sj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x993)*(x994)))+(((IkReal(-1.00000000000000))*(x994)*(x995)))+(((x992)*(x993)))+(((IkReal(-1.00000000000000))*(x992)*(x995)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x993)*(x994)))+(((IkReal(-1.00000000000000))*(x994)*(x995)))+(((x992)*(x993)))+(((IkReal(-1.00000000000000))*(x992)*(x995)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x993)*(x994)))+(((IkReal(-1.00000000000000))*(x994)*(x995)))+(((x992)*(x993)))+(((IkReal(-1.00000000000000))*(x992)*(x995)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x996=IKcos(j5);
IkReal x997=((r01)*(sj0));
IkReal x998=((sj1)*(sj2));
IkReal x999=((IkReal(1.00000000000000))*(cj0));
IkReal x1000=((cj1)*(cj2));
IkReal x1001=((cj1)*(sj2));
IkReal x1002=((cj0)*(r00));
IkReal x1003=((cj2)*(sj1));
IkReal x1004=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(r01)*(x999))));
evalcond[1]=((((sj4)*(x996)))+(((x997)*(x998)))+(((IkReal(-1.00000000000000))*(r00)*(x1000)*(x999)))+(((x1002)*(x998)))+(((IkReal(-1.00000000000000))*(x1001)*(x1004)))+(((IkReal(-1.00000000000000))*(x1000)*(x997)))+(((IkReal(-1.00000000000000))*(x1003)*(x1004))));
evalcond[2]=((((cj4)*(x996)))+(((x1003)*(x997)))+(((r02)*(x998)))+(((x1001)*(x997)))+(((x1002)*(x1003)))+(((IkReal(-1.00000000000000))*(x1000)*(x1004)))+(((x1001)*(x1002))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1005=((pz)*(sj2));
IkReal x1006=((IkReal(1.00000000000000))*(cj1));
IkReal x1007=((cj2)*(sj0));
IkReal x1008=((py)*(sj1));
IkReal x1009=((cj0)*(sj1));
IkReal x1010=((cj2)*(px));
IkReal x1011=((sj0)*(sj2));
IkReal x1012=((IkReal(0.0750000000000000))*(cj2));
IkReal x1013=((px)*(sj2));
IkReal x1014=((IkReal(0.0750000000000000))*(sj2));
IkReal x1015=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x1011)))+(((IkReal(-1.00000000000000))*(r02)*(x1015)))+(((r00)*(sj2)*(x1009)))+(((IkReal(-1.00000000000000))*(r01)*(x1006)*(x1007)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1006)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1006))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1009)*(x1010)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1006)))+(((sj1)*(x1005)))+(((IkReal(-1.00000000000000))*(cj1)*(x1014)))+(((cj1)*(py)*(x1011)))+(((x1007)*(x1008)))+(((IkReal(-1.00000000000000))*(sj1)*(x1012)))+(((cj0)*(cj1)*(x1013))));
evalcond[4]=((IkReal(0.375000000000000))+(((x1009)*(x1013)))+(((IkReal(-1.00000000000000))*(pz)*(x1015)))+(((IkReal(-1.00000000000000))*(py)*(x1006)*(x1007)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1005)*(x1006)))+(((IkReal(-1.00000000000000))*(cj0)*(x1006)*(x1010)))+(((cj1)*(x1012)))+(((IkReal(-1.00000000000000))*(sj1)*(x1014)))+(((x1008)*(x1011))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1016=((cj3)*(sj1));
IkReal x1017=((r02)*(sj2));
IkReal x1018=((r01)*(sj0));
IkReal x1019=((IkReal(1.00000000000000))*(cj3));
IkReal x1020=((cj1)*(sj2));
IkReal x1021=((IkReal(1.00000000000000))*(sj3));
IkReal x1022=((cj0)*(r00));
IkReal x1023=((cj0)*(r01));
IkReal x1024=((r00)*(sj0));
IkReal x1025=((cj2)*(x1022));
IkReal x1026=((sj1)*(x1021));
IkReal x1027=((cj1)*(cj2)*(r02));
if( IKabs(((gconst61)*(((((IkReal(-1.00000000000000))*(x1019)*(x1023)))+(((IkReal(-1.00000000000000))*(x1020)*(x1021)*(x1022)))+(((IkReal(-1.00000000000000))*(x1018)*(x1020)*(x1021)))+(((IkReal(-1.00000000000000))*(cj2)*(x1018)*(x1026)))+(((IkReal(-1.00000000000000))*(x1017)*(x1026)))+(((IkReal(-1.00000000000000))*(x1025)*(x1026)))+(((sj3)*(x1027)))+(((cj3)*(x1024))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst61)*(((((x1016)*(x1017)))+(((cj3)*(x1020)*(x1022)))+(((IkReal(-1.00000000000000))*(x1019)*(x1027)))+(((IkReal(-1.00000000000000))*(x1021)*(x1023)))+(((cj2)*(x1016)*(x1018)))+(((cj3)*(x1018)*(x1020)))+(((sj3)*(x1024)))+(((x1016)*(x1025))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst61)*(((((IkReal(-1.00000000000000))*(x1019)*(x1023)))+(((IkReal(-1.00000000000000))*(x1020)*(x1021)*(x1022)))+(((IkReal(-1.00000000000000))*(x1018)*(x1020)*(x1021)))+(((IkReal(-1.00000000000000))*(cj2)*(x1018)*(x1026)))+(((IkReal(-1.00000000000000))*(x1017)*(x1026)))+(((IkReal(-1.00000000000000))*(x1025)*(x1026)))+(((sj3)*(x1027)))+(((cj3)*(x1024)))))), ((gconst61)*(((((x1016)*(x1017)))+(((cj3)*(x1020)*(x1022)))+(((IkReal(-1.00000000000000))*(x1019)*(x1027)))+(((IkReal(-1.00000000000000))*(x1021)*(x1023)))+(((cj2)*(x1016)*(x1018)))+(((cj3)*(x1018)*(x1020)))+(((sj3)*(x1024)))+(((x1016)*(x1025)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1028=IKsin(j5);
IkReal x1029=IKcos(j5);
IkReal x1030=((IkReal(1.00000000000000))*(cj3));
IkReal x1031=((cj1)*(sj2));
IkReal x1032=((r01)*(sj0));
IkReal x1033=((cj0)*(r00));
IkReal x1034=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1028)*(x1030)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(sj3)*(x1029))));
evalcond[1]=((((x1031)*(x1032)))+(((x1031)*(x1033)))+(((IkReal(-1.00000000000000))*(x1029)*(x1030)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1033)*(x1034)))+(((r02)*(sj1)*(sj2)))+(((x1032)*(x1034)))+(((sj3)*(x1028))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1035=((pz)*(sj2));
IkReal x1036=((IkReal(1.00000000000000))*(cj1));
IkReal x1037=((cj2)*(sj0));
IkReal x1038=((py)*(sj1));
IkReal x1039=((cj0)*(sj1));
IkReal x1040=((cj2)*(px));
IkReal x1041=((sj0)*(sj2));
IkReal x1042=((IkReal(0.0750000000000000))*(cj2));
IkReal x1043=((px)*(sj2));
IkReal x1044=((IkReal(0.0750000000000000))*(sj2));
IkReal x1045=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x1045)))+(((r00)*(sj2)*(x1039)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1036)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1036)))+(((IkReal(-1.00000000000000))*(r01)*(x1036)*(x1037)))+(((r01)*(sj1)*(x1041))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1036)))+(((IkReal(-1.00000000000000))*(cj1)*(x1044)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1039)*(x1040)))+(((cj1)*(py)*(x1041)))+(((sj1)*(x1035)))+(((IkReal(-1.00000000000000))*(sj1)*(x1042)))+(((cj0)*(cj1)*(x1043)))+(((x1037)*(x1038))));
evalcond[4]=((IkReal(0.215000000000000))+(((x1038)*(x1041)))+(((IkReal(-1.00000000000000))*(cj0)*(x1036)*(x1040)))+(((IkReal(-1.00000000000000))*(x1035)*(x1036)))+(((x1039)*(x1043)))+(((cj1)*(x1042)))+(((IkReal(-1.00000000000000))*(sj1)*(x1044)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1045)))+(((IkReal(-1.00000000000000))*(py)*(x1036)*(x1037))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst62;
gconst62=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1046=((cj1)*(sj2));
IkReal x1047=((cj2)*(sj1));
IkReal x1048=((r01)*(sj3));
IkReal x1049=((IkReal(1.00000000000000))*(sj0));
IkReal x1050=((cj1)*(cj2)*(r02));
IkReal x1051=((IkReal(1.00000000000000))*(cj0)*(cj3));
IkReal x1052=((cj3)*(r01)*(x1049));
IkReal x1053=((IkReal(1.00000000000000))*(cj0)*(r00)*(sj3));
IkReal x1054=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst62)*(((((IkReal(-1.00000000000000))*(r01)*(x1051)))+(((IkReal(-1.00000000000000))*(x1046)*(x1053)))+(((sj3)*(x1050)))+(((IkReal(-1.00000000000000))*(sj3)*(x1054)))+(((cj3)*(r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1046)*(x1048)*(x1049)))+(((IkReal(-1.00000000000000))*(x1047)*(x1048)*(x1049)))+(((IkReal(-1.00000000000000))*(x1047)*(x1053))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst62)*(((((IkReal(-1.00000000000000))*(x1046)*(x1052)))+(((cj0)*(x1048)))+(((cj3)*(x1050)))+(((IkReal(-1.00000000000000))*(r00)*(sj3)*(x1049)))+(((IkReal(-1.00000000000000))*(r00)*(x1047)*(x1051)))+(((IkReal(-1.00000000000000))*(r00)*(x1046)*(x1051)))+(((IkReal(-1.00000000000000))*(x1047)*(x1052)))+(((IkReal(-1.00000000000000))*(cj3)*(x1054))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst62)*(((((IkReal(-1.00000000000000))*(r01)*(x1051)))+(((IkReal(-1.00000000000000))*(x1046)*(x1053)))+(((sj3)*(x1050)))+(((IkReal(-1.00000000000000))*(sj3)*(x1054)))+(((cj3)*(r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1046)*(x1048)*(x1049)))+(((IkReal(-1.00000000000000))*(x1047)*(x1048)*(x1049)))+(((IkReal(-1.00000000000000))*(x1047)*(x1053)))))), ((gconst62)*(((((IkReal(-1.00000000000000))*(x1046)*(x1052)))+(((cj0)*(x1048)))+(((cj3)*(x1050)))+(((IkReal(-1.00000000000000))*(r00)*(sj3)*(x1049)))+(((IkReal(-1.00000000000000))*(r00)*(x1047)*(x1051)))+(((IkReal(-1.00000000000000))*(r00)*(x1046)*(x1051)))+(((IkReal(-1.00000000000000))*(x1047)*(x1052)))+(((IkReal(-1.00000000000000))*(cj3)*(x1054)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1055=IKsin(j5);
IkReal x1056=IKcos(j5);
IkReal x1057=((cj1)*(sj2));
IkReal x1058=((r01)*(sj0));
IkReal x1059=((cj0)*(r00));
IkReal x1060=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((sj3)*(x1056)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj3)*(x1055))));
evalcond[1]=((((sj3)*(x1055)))+(((x1057)*(x1059)))+(((x1057)*(x1058)))+(((cj3)*(x1056)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1059)*(x1060)))+(((r02)*(sj1)*(sj2)))+(((x1058)*(x1060))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1061=((pz)*(sj2));
IkReal x1062=((IkReal(1.00000000000000))*(cj1));
IkReal x1063=((cj2)*(sj1));
IkReal x1064=((py)*(sj0));
IkReal x1065=((cj0)*(px));
IkReal x1066=((sj1)*(sj2));
IkReal x1067=((IkReal(0.0750000000000000))*(cj1));
IkReal x1068=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1067)))+(((x1063)*(x1064)))+(((x1063)*(x1065)))+(((sj1)*(x1061)))+(((IkReal(-0.0750000000000000))*(x1063)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1062)))+(((x1064)*(x1068)))+(((x1065)*(x1068))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1063)))+(((cj2)*(x1067)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1061)*(x1062)))+(((IkReal(-0.0750000000000000))*(x1066)))+(((IkReal(-1.00000000000000))*(cj2)*(x1062)*(x1064)))+(((IkReal(-1.00000000000000))*(cj2)*(x1062)*(x1065)))+(((IkReal(0.0800000000000000))*(cj4)))+(((x1064)*(x1066)))+(((x1065)*(x1066))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x1069=((pz)*(sj2));
IkReal x1070=((IkReal(1.00000000000000))*(cj1));
IkReal x1071=((cj2)*(sj0));
IkReal x1072=((py)*(sj1));
IkReal x1073=((cj0)*(sj1));
IkReal x1074=((cj2)*(px));
IkReal x1075=((sj0)*(sj2));
IkReal x1076=((IkReal(0.0750000000000000))*(cj2));
IkReal x1077=((px)*(sj2));
IkReal x1078=((IkReal(0.0750000000000000))*(sj2));
IkReal x1079=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x1075)))+(((IkReal(-1.00000000000000))*(r02)*(x1079)))+(((IkReal(-1.00000000000000))*(r01)*(x1070)*(x1071)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1070)))+(((r00)*(sj2)*(x1073)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1070))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1071)*(x1072)))+(((IkReal(-1.00000000000000))*(cj1)*(x1078)))+(((cj1)*(py)*(x1075)))+(((IkReal(-1.00000000000000))*(sj1)*(x1076)))+(((x1073)*(x1074)))+(((sj1)*(x1069)))+(((cj0)*(cj1)*(x1077)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1070))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(x1070)*(x1074)))+(((IkReal(-1.00000000000000))*(pz)*(x1079)))+(((IkReal(-1.00000000000000))*(sj1)*(x1078)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj1)*(x1076)))+(((x1073)*(x1077)))+(((IkReal(-1.00000000000000))*(py)*(x1070)*(x1071)))+(((x1072)*(x1075)))+(((IkReal(-1.00000000000000))*(x1069)*(x1070))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1080=((IkReal(1.00000000000000))*(sj2));
IkReal x1081=((IkReal(1.00000000000000))*(r01));
IkReal x1082=((cj0)*(r00));
IkReal x1083=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x1082)*(x1083)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1080)))+(((IkReal(-1.00000000000000))*(cj1)*(x1080)*(x1082)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1080)))+(((IkReal(-1.00000000000000))*(sj0)*(x1081)*(x1083)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1081))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x1082)*(x1083)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1080)))+(((IkReal(-1.00000000000000))*(cj1)*(x1080)*(x1082)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1080)))+(((IkReal(-1.00000000000000))*(sj0)*(x1081)*(x1083)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1081)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x1082)*(x1083)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1080)))+(((IkReal(-1.00000000000000))*(cj1)*(x1080)*(x1082)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1080)))+(((IkReal(-1.00000000000000))*(sj0)*(x1081)*(x1083)))+(((cj1)*(cj2)*(r02)))), ((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1081)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1084=((cj1)*(sj2));
IkReal x1085=((r01)*(sj0));
IkReal x1086=((cj0)*(r00));
IkReal x1087=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((IKsin(j5))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1086)*(x1087)))+(((r02)*(sj1)*(sj2)))+(((x1085)*(x1087)))+(((x1084)*(x1085)))+(((x1084)*(x1086))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1088=((pz)*(sj2));
IkReal x1089=((IkReal(1.00000000000000))*(cj1));
IkReal x1090=((cj2)*(sj0));
IkReal x1091=((py)*(sj1));
IkReal x1092=((cj0)*(sj1));
IkReal x1093=((cj2)*(px));
IkReal x1094=((sj0)*(sj2));
IkReal x1095=((IkReal(0.0750000000000000))*(cj2));
IkReal x1096=((px)*(sj2));
IkReal x1097=((IkReal(0.0750000000000000))*(sj2));
IkReal x1098=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r00)*(sj2)*(x1092)))+(((IkReal(-1.00000000000000))*(r01)*(x1089)*(x1090)))+(((IkReal(-1.00000000000000))*(r02)*(x1098)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1089)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1089)))+(((r01)*(sj1)*(x1094))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x1095)))+(((IkReal(-1.00000000000000))*(cj1)*(x1097)))+(((x1092)*(x1093)))+(((sj1)*(x1088)))+(((cj1)*(py)*(x1094)))+(((x1090)*(x1091)))+(((cj0)*(cj1)*(x1096)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1089))));
evalcond[4]=((IkReal(0.215000000000000))+(((cj1)*(x1095)))+(((IkReal(-1.00000000000000))*(pz)*(x1098)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1088)*(x1089)))+(((IkReal(-1.00000000000000))*(sj1)*(x1097)))+(((IkReal(-1.00000000000000))*(py)*(x1089)*(x1090)))+(((x1092)*(x1096)))+(((IkReal(-1.00000000000000))*(cj0)*(x1089)*(x1093)))+(((x1091)*(x1094))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1099=((IkReal(1.00000000000000))*(sj2));
IkReal x1100=((cj0)*(r00));
IkReal x1101=((r01)*(sj0));
IkReal x1102=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(x1099)*(x1100)))+(((IkReal(-1.00000000000000))*(cj1)*(x1099)*(x1101)))+(((IkReal(-1.00000000000000))*(x1100)*(x1102)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1099)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1101)*(x1102))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(x1099)*(x1100)))+(((IkReal(-1.00000000000000))*(cj1)*(x1099)*(x1101)))+(((IkReal(-1.00000000000000))*(x1100)*(x1102)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1099)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1101)*(x1102)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(x1099)*(x1100)))+(((IkReal(-1.00000000000000))*(cj1)*(x1099)*(x1101)))+(((IkReal(-1.00000000000000))*(x1100)*(x1102)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1099)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1101)*(x1102)))), ((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1103=((cj1)*(sj2));
IkReal x1104=((r01)*(sj0));
IkReal x1105=((cj0)*(r00));
IkReal x1106=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(IKcos(j5)));
evalcond[1]=((((x1105)*(x1106)))+(IKsin(j5))+(((x1104)*(x1106)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1103)*(x1105)))+(((x1103)*(x1104)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1107=((pz)*(sj2));
IkReal x1108=((IkReal(1.00000000000000))*(cj1));
IkReal x1109=((cj2)*(sj1));
IkReal x1110=((py)*(sj0));
IkReal x1111=((cj0)*(px));
IkReal x1112=((sj1)*(sj2));
IkReal x1113=((IkReal(1.00000000000000))*(cj0));
IkReal x1114=((IkReal(0.0750000000000000))*(cj1));
IkReal x1115=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1113)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1113)))+(((r00)*(sj0))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1109)*(x1110)))+(((x1109)*(x1111)))+(((x1111)*(x1115)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1110)*(x1115)))+(((IkReal(-1.00000000000000))*(sj2)*(x1114)))+(((IkReal(-0.0750000000000000))*(x1109)))+(((sj1)*(x1107)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1108))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x1108)*(x1110)))+(((IkReal(-1.00000000000000))*(cj2)*(x1108)*(x1111)))+(((x1111)*(x1112)))+(((x1110)*(x1112)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1107)*(x1108)))+(((IkReal(-0.0750000000000000))*(x1112)))+(((cj2)*(x1114)))+(((IkReal(-1.00000000000000))*(pz)*(x1109))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1116=((r02)*(sj1));
IkReal x1117=((IkReal(1.00000000000000))*(sj2));
IkReal x1118=((IkReal(1.00000000000000))*(cj2));
IkReal x1119=((cj1)*(r02));
IkReal x1120=((cj0)*(cj1)*(r00));
IkReal x1121=((cj0)*(r00)*(sj1));
IkReal x1122=((r01)*(sj0)*(sj1));
IkReal x1123=((cj1)*(r01)*(sj0));
if( IKabs(((((IkReal(-1.00000000000000))*(x1117)*(x1123)))+(((IkReal(-1.00000000000000))*(x1117)*(x1120)))+(((IkReal(-1.00000000000000))*(x1116)*(x1117)))+(((cj2)*(x1119)))+(((IkReal(-1.00000000000000))*(x1118)*(x1122)))+(((IkReal(-1.00000000000000))*(x1118)*(x1121))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj2)*(x1119)))+(((cj2)*(x1123)))+(((cj2)*(x1120)))+(((IkReal(-1.00000000000000))*(x1117)*(x1121)))+(((IkReal(-1.00000000000000))*(x1117)*(x1122)))+(((cj2)*(x1116))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x1117)*(x1123)))+(((IkReal(-1.00000000000000))*(x1117)*(x1120)))+(((IkReal(-1.00000000000000))*(x1116)*(x1117)))+(((cj2)*(x1119)))+(((IkReal(-1.00000000000000))*(x1118)*(x1122)))+(((IkReal(-1.00000000000000))*(x1118)*(x1121)))))+IKsqr(((((sj2)*(x1119)))+(((cj2)*(x1123)))+(((cj2)*(x1120)))+(((IkReal(-1.00000000000000))*(x1117)*(x1121)))+(((IkReal(-1.00000000000000))*(x1117)*(x1122)))+(((cj2)*(x1116)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x1117)*(x1123)))+(((IkReal(-1.00000000000000))*(x1117)*(x1120)))+(((IkReal(-1.00000000000000))*(x1116)*(x1117)))+(((cj2)*(x1119)))+(((IkReal(-1.00000000000000))*(x1118)*(x1122)))+(((IkReal(-1.00000000000000))*(x1118)*(x1121)))), ((((sj2)*(x1119)))+(((cj2)*(x1123)))+(((cj2)*(x1120)))+(((IkReal(-1.00000000000000))*(x1117)*(x1121)))+(((IkReal(-1.00000000000000))*(x1117)*(x1122)))+(((cj2)*(x1116)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1124=((IkReal(1.00000000000000))*(cj2));
IkReal x1125=((r01)*(sj0));
IkReal x1126=((cj2)*(sj1));
IkReal x1127=((cj1)*(r02));
IkReal x1128=((r02)*(sj1));
IkReal x1129=((cj0)*(r00));
IkReal x1130=((sj2)*(x1125));
IkReal x1131=((cj1)*(x1129));
evalcond[0]=((((sj1)*(sj2)*(x1129)))+(((IkReal(-1.00000000000000))*(cj1)*(x1124)*(x1125)))+(((sj1)*(x1130)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(sj2)*(x1127)))+(((IkReal(-1.00000000000000))*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(x1124)*(x1131))));
evalcond[1]=((IKsin(j5))+(((x1125)*(x1126)))+(((sj2)*(x1131)))+(((sj2)*(x1128)))+(((cj1)*(x1130)))+(((IkReal(-1.00000000000000))*(x1124)*(x1127)))+(((x1126)*(x1129))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1132=((pz)*(sj2));
IkReal x1133=((IkReal(1.00000000000000))*(cj1));
IkReal x1134=((cj2)*(sj1));
IkReal x1135=((py)*(sj0));
IkReal x1136=((cj0)*(px));
IkReal x1137=((sj1)*(sj2));
IkReal x1138=((IkReal(1.00000000000000))*(cj0));
IkReal x1139=((IkReal(0.0750000000000000))*(cj1));
IkReal x1140=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1138)))+(((px)*(sj0))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1138))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x1139)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x1132)))+(((x1134)*(x1135)))+(((x1134)*(x1136)))+(((IkReal(-0.0750000000000000))*(x1134)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1133)))+(((x1135)*(x1140)))+(((x1136)*(x1140))));
evalcond[4]=((IkReal(0.295000000000000))+(((cj2)*(x1139)))+(((x1136)*(x1137)))+(((x1135)*(x1137)))+(((IkReal(-1.00000000000000))*(cj2)*(x1133)*(x1135)))+(((IkReal(-1.00000000000000))*(cj2)*(x1133)*(x1136)))+(((IkReal(-0.0750000000000000))*(x1137)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1134)))+(((IkReal(-1.00000000000000))*(x1132)*(x1133))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1141=((sj1)*(sj2));
IkReal x1142=((IkReal(1.00000000000000))*(r02));
IkReal x1143=((r01)*(sj0));
IkReal x1144=((cj0)*(r00));
IkReal x1145=((cj1)*(sj2));
IkReal x1146=((cj2)*(sj1));
IkReal x1147=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((IkReal(-1.00000000000000))*(x1144)*(x1145)))+(((IkReal(-1.00000000000000))*(x1144)*(x1146)))+(((IkReal(-1.00000000000000))*(x1141)*(x1142)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1143)*(x1146)))+(((IkReal(-1.00000000000000))*(x1143)*(x1145))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1143)*(x1147)))+(((x1141)*(x1144)))+(((x1141)*(x1143)))+(((IkReal(-1.00000000000000))*(x1144)*(x1147)))+(((IkReal(-1.00000000000000))*(x1142)*(x1145)))+(((IkReal(-1.00000000000000))*(x1142)*(x1146))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x1144)*(x1145)))+(((IkReal(-1.00000000000000))*(x1144)*(x1146)))+(((IkReal(-1.00000000000000))*(x1141)*(x1142)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1143)*(x1146)))+(((IkReal(-1.00000000000000))*(x1143)*(x1145)))))+IKsqr(((((IkReal(-1.00000000000000))*(x1143)*(x1147)))+(((x1141)*(x1144)))+(((x1141)*(x1143)))+(((IkReal(-1.00000000000000))*(x1144)*(x1147)))+(((IkReal(-1.00000000000000))*(x1142)*(x1145)))+(((IkReal(-1.00000000000000))*(x1142)*(x1146)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x1144)*(x1145)))+(((IkReal(-1.00000000000000))*(x1144)*(x1146)))+(((IkReal(-1.00000000000000))*(x1141)*(x1142)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1143)*(x1146)))+(((IkReal(-1.00000000000000))*(x1143)*(x1145)))), ((((IkReal(-1.00000000000000))*(x1143)*(x1147)))+(((x1141)*(x1144)))+(((x1141)*(x1143)))+(((IkReal(-1.00000000000000))*(x1144)*(x1147)))+(((IkReal(-1.00000000000000))*(x1142)*(x1145)))+(((IkReal(-1.00000000000000))*(x1142)*(x1146)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1148=((IkReal(1.00000000000000))*(cj2));
IkReal x1149=((r01)*(sj0));
IkReal x1150=((cj2)*(sj1));
IkReal x1151=((cj1)*(r02));
IkReal x1152=((r02)*(sj1));
IkReal x1153=((cj0)*(r00));
IkReal x1154=((sj2)*(x1149));
IkReal x1155=((cj1)*(x1153));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj1)*(x1148)*(x1149)))+(((sj1)*(sj2)*(x1153)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x1148)*(x1152)))+(((IkReal(-1.00000000000000))*(x1148)*(x1155)))+(((sj1)*(x1154)))+(((IkReal(-1.00000000000000))*(sj2)*(x1151))));
evalcond[1]=((((x1149)*(x1150)))+(((sj2)*(x1155)))+(((sj2)*(x1152)))+(IKsin(j5))+(((x1150)*(x1153)))+(((cj1)*(x1154)))+(((IkReal(-1.00000000000000))*(x1148)*(x1151))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1156=((r02)*(sj1));
IkReal x1157=((IkReal(1.00000000000000))*(sj2));
IkReal x1158=((IkReal(1.00000000000000))*(cj2));
IkReal x1159=((cj1)*(r02));
IkReal x1160=((cj0)*(cj1)*(r00));
IkReal x1161=((cj0)*(r00)*(sj1));
IkReal x1162=((r01)*(sj0)*(sj1));
IkReal x1163=((cj1)*(r01)*(sj0));
if( IKabs(((((cj2)*(x1159)))+(((IkReal(-1.00000000000000))*(x1158)*(x1161)))+(((IkReal(-1.00000000000000))*(x1158)*(x1162)))+(((IkReal(-1.00000000000000))*(x1156)*(x1157)))+(((IkReal(-1.00000000000000))*(x1157)*(x1163)))+(((IkReal(-1.00000000000000))*(x1157)*(x1160))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj2)*(x1156)))+(((sj2)*(x1159)))+(((cj2)*(x1160)))+(((cj2)*(x1163)))+(((IkReal(-1.00000000000000))*(x1157)*(x1162)))+(((IkReal(-1.00000000000000))*(x1157)*(x1161))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(x1159)))+(((IkReal(-1.00000000000000))*(x1158)*(x1161)))+(((IkReal(-1.00000000000000))*(x1158)*(x1162)))+(((IkReal(-1.00000000000000))*(x1156)*(x1157)))+(((IkReal(-1.00000000000000))*(x1157)*(x1163)))+(((IkReal(-1.00000000000000))*(x1157)*(x1160)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj2)*(x1156)))+(((sj2)*(x1159)))+(((cj2)*(x1160)))+(((cj2)*(x1163)))+(((IkReal(-1.00000000000000))*(x1157)*(x1162)))+(((IkReal(-1.00000000000000))*(x1157)*(x1161)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(x1159)))+(((IkReal(-1.00000000000000))*(x1158)*(x1161)))+(((IkReal(-1.00000000000000))*(x1158)*(x1162)))+(((IkReal(-1.00000000000000))*(x1156)*(x1157)))+(((IkReal(-1.00000000000000))*(x1157)*(x1163)))+(((IkReal(-1.00000000000000))*(x1157)*(x1160)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj2)*(x1156)))+(((sj2)*(x1159)))+(((cj2)*(x1160)))+(((cj2)*(x1163)))+(((IkReal(-1.00000000000000))*(x1157)*(x1162)))+(((IkReal(-1.00000000000000))*(x1157)*(x1161)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1164=IKcos(j5);
IkReal x1165=((r01)*(sj0));
IkReal x1166=((sj1)*(sj2));
IkReal x1167=((IkReal(1.00000000000000))*(cj0));
IkReal x1168=((IkReal(1.00000000000000))*(cj1));
IkReal x1169=((cj1)*(sj2));
IkReal x1170=((cj0)*(r00));
IkReal x1171=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj4)*(x1164)))+(((IkReal(-1.00000000000000))*(r01)*(x1167))));
evalcond[1]=((((x1165)*(x1166)))+(((x1166)*(x1170)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)*(x1167)))+(((IkReal(-1.00000000000000))*(cj2)*(x1165)*(x1168)))+(((IkReal(-1.00000000000000))*(r02)*(x1171)))+(((sj4)*(x1164)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1168))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1168)))+(((x1165)*(x1169)))+(((r02)*(x1166)))+(((x1165)*(x1171)))+(IKsin(j5))+(((x1170)*(x1171)))+(((x1169)*(x1170))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1172=((IkReal(1.00000000000000))*(sj2));
IkReal x1173=((IkReal(1.00000000000000))*(r01));
IkReal x1174=((cj0)*(r00));
IkReal x1175=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1172)))+(((IkReal(-1.00000000000000))*(sj0)*(x1173)*(x1175)))+(((IkReal(-1.00000000000000))*(x1174)*(x1175)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1172)))+(((IkReal(-1.00000000000000))*(cj1)*(x1172)*(x1174)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1173))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1172)))+(((IkReal(-1.00000000000000))*(sj0)*(x1173)*(x1175)))+(((IkReal(-1.00000000000000))*(x1174)*(x1175)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1172)))+(((IkReal(-1.00000000000000))*(cj1)*(x1172)*(x1174)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1173)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1172)))+(((IkReal(-1.00000000000000))*(sj0)*(x1173)*(x1175)))+(((IkReal(-1.00000000000000))*(x1174)*(x1175)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1172)))+(((IkReal(-1.00000000000000))*(cj1)*(x1172)*(x1174)))+(((cj1)*(cj2)*(r02)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1173)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1176=IKcos(j5);
IkReal x1177=((r01)*(sj0));
IkReal x1178=((sj1)*(sj2));
IkReal x1179=((IkReal(1.00000000000000))*(cj0));
IkReal x1180=((IkReal(1.00000000000000))*(cj1));
IkReal x1181=((cj1)*(sj2));
IkReal x1182=((cj0)*(r00));
IkReal x1183=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj4)*(x1176)))+(((IkReal(-1.00000000000000))*(r01)*(x1179))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1180)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)*(x1179)))+(((x1178)*(x1182)))+(((IkReal(-1.00000000000000))*(cj2)*(x1177)*(x1180)))+(((x1177)*(x1178)))+(((sj4)*(x1176)))+(((IkReal(-1.00000000000000))*(r02)*(x1183))));
evalcond[2]=((((r02)*(x1178)))+(((x1182)*(x1183)))+(((x1177)*(x1181)))+(((x1177)*(x1183)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1180)))+(((x1181)*(x1182))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1184=((pz)*(sj2));
IkReal x1185=((IkReal(1.00000000000000))*(cj1));
IkReal x1186=((cj2)*(sj1));
IkReal x1187=((py)*(sj0));
IkReal x1188=((cj0)*(px));
IkReal x1189=((sj1)*(sj2));
IkReal x1190=((IkReal(0.0750000000000000))*(cj1));
IkReal x1191=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x1190)))+(((x1186)*(x1188)))+(((x1186)*(x1187)))+(((x1187)*(x1191)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1188)*(x1191)))+(((sj1)*(x1184)))+(((IkReal(-0.0750000000000000))*(x1186)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1185))));
evalcond[3]=((IkReal(0.295000000000000))+(((x1187)*(x1189)))+(((x1188)*(x1189)))+(((IkReal(-0.0750000000000000))*(x1189)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1184)*(x1185)))+(((IkReal(-1.00000000000000))*(pz)*(x1186)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x1190)))+(((IkReal(-1.00000000000000))*(cj2)*(x1185)*(x1188)))+(((IkReal(-1.00000000000000))*(cj2)*(x1185)*(x1187))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x1192=((pz)*(sj2));
IkReal x1193=((IkReal(1.00000000000000))*(cj1));
IkReal x1194=((cj2)*(sj0));
IkReal x1195=((py)*(sj1));
IkReal x1196=((cj0)*(sj1));
IkReal x1197=((cj2)*(px));
IkReal x1198=((sj0)*(sj2));
IkReal x1199=((IkReal(0.0750000000000000))*(cj2));
IkReal x1200=((px)*(sj2));
IkReal x1201=((IkReal(0.0750000000000000))*(sj2));
IkReal x1202=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1193)))+(((r01)*(sj1)*(x1198)))+(((IkReal(-1.00000000000000))*(r02)*(x1202)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1193)))+(((IkReal(-1.00000000000000))*(r01)*(x1193)*(x1194)))+(((r00)*(sj2)*(x1196))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x1198)))+(((x1194)*(x1195)))+(((cj0)*(cj1)*(x1200)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x1201)))+(((sj1)*(x1192)))+(((x1196)*(x1197)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1193)))+(((IkReal(-1.00000000000000))*(sj1)*(x1199))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1202)))+(((IkReal(-1.00000000000000))*(sj1)*(x1201)))+(((x1195)*(x1198)))+(((IkReal(-1.00000000000000))*(py)*(x1193)*(x1194)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x1193)*(x1197)))+(((IkReal(-1.00000000000000))*(x1192)*(x1193)))+(((cj1)*(x1199)))+(((x1196)*(x1200))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1203=((cj1)*(sj2));
IkReal x1204=((r01)*(sj0));
IkReal x1205=((cj0)*(r00));
IkReal x1206=((cj2)*(sj1));
if( IKabs(((((x1204)*(x1206)))+(((x1203)*(x1204)))+(((x1203)*(x1205)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1205)*(x1206)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1204)*(x1206)))+(((x1203)*(x1204)))+(((x1203)*(x1205)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1205)*(x1206)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1204)*(x1206)))+(((x1203)*(x1204)))+(((x1203)*(x1205)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1205)*(x1206)))+(((r02)*(sj1)*(sj2)))), ((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1207=((cj1)*(sj2));
IkReal x1208=((r01)*(sj0));
IkReal x1209=((cj0)*(r00));
IkReal x1210=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(IKcos(j5)));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1209)*(x1210)))+(((x1208)*(x1210)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((r02)*(sj1)*(sj2)))+(((x1207)*(x1209)))+(((x1207)*(x1208))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1211=((pz)*(sj2));
IkReal x1212=((IkReal(1.00000000000000))*(cj1));
IkReal x1213=((cj2)*(sj0));
IkReal x1214=((py)*(sj1));
IkReal x1215=((cj0)*(sj1));
IkReal x1216=((cj2)*(px));
IkReal x1217=((sj0)*(sj2));
IkReal x1218=((IkReal(0.0750000000000000))*(cj2));
IkReal x1219=((px)*(sj2));
IkReal x1220=((IkReal(0.0750000000000000))*(sj2));
IkReal x1221=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1212)))+(((r01)*(sj1)*(x1217)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1212)))+(((IkReal(-1.00000000000000))*(r01)*(x1212)*(x1213)))+(((r00)*(sj2)*(x1215)))+(((IkReal(-1.00000000000000))*(r02)*(x1221))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1213)*(x1214)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1212)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x1211)))+(((cj0)*(cj1)*(x1219)))+(((IkReal(-1.00000000000000))*(sj1)*(x1218)))+(((IkReal(-1.00000000000000))*(cj1)*(x1220)))+(((cj1)*(py)*(x1217)))+(((x1215)*(x1216))));
evalcond[4]=((IkReal(0.215000000000000))+(((x1214)*(x1217)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1221)))+(((IkReal(-1.00000000000000))*(x1211)*(x1212)))+(((IkReal(-1.00000000000000))*(py)*(x1212)*(x1213)))+(((IkReal(-1.00000000000000))*(sj1)*(x1220)))+(((IkReal(-1.00000000000000))*(cj0)*(x1212)*(x1216)))+(((cj1)*(x1218)))+(((x1215)*(x1219))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1222=((cj1)*(sj2));
IkReal x1223=((r01)*(sj0));
IkReal x1224=((cj0)*(r00));
IkReal x1225=((cj2)*(sj1));
if( IKabs(((((x1223)*(x1225)))+(((x1224)*(x1225)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1222)*(x1224)))+(((x1222)*(x1223)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1223)*(x1225)))+(((x1224)*(x1225)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1222)*(x1224)))+(((x1222)*(x1223)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1223)*(x1225)))+(((x1224)*(x1225)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1222)*(x1224)))+(((x1222)*(x1223)))+(((r02)*(sj1)*(sj2)))), ((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1226=((cj1)*(sj2));
IkReal x1227=((r01)*(sj0));
IkReal x1228=((cj0)*(r00));
IkReal x1229=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1226)*(x1228)))+(((x1226)*(x1227)))+(((x1227)*(x1229)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((x1228)*(x1229)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1230=((pz)*(sj2));
IkReal x1231=((IkReal(1.00000000000000))*(cj1));
IkReal x1232=((cj2)*(sj1));
IkReal x1233=((py)*(sj0));
IkReal x1234=((cj0)*(px));
IkReal x1235=((sj1)*(sj2));
IkReal x1236=((IkReal(1.00000000000000))*(cj0));
IkReal x1237=((IkReal(0.0750000000000000))*(cj1));
IkReal x1238=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1236)))+(((px)*(sj0))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1236))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((sj1)*(x1230)))+(((IkReal(-0.0750000000000000))*(x1232)))+(((x1234)*(x1238)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1233)*(x1238)))+(((IkReal(-1.00000000000000))*(sj2)*(x1237)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1231)))+(((x1232)*(x1233)))+(((x1232)*(x1234))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x1235)))+(((x1234)*(x1235)))+(((cj2)*(x1237)))+(((IkReal(-1.00000000000000))*(x1230)*(x1231)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1232)))+(((IkReal(-1.00000000000000))*(cj2)*(x1231)*(x1234)))+(((IkReal(-1.00000000000000))*(cj2)*(x1231)*(x1233)))+(((x1233)*(x1235))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1239=((r01)*(sj0));
IkReal x1240=((cj1)*(sj2));
IkReal x1241=((sj1)*(sj2));
IkReal x1242=((cj2)*(sj1));
IkReal x1243=((cj0)*(r00));
IkReal x1244=((cj1)*(cj2));
IkReal x1245=((cj1)*(x1243));
if( IKabs(((((x1239)*(x1242)))+(((x1239)*(x1240)))+(((x1240)*(x1243)))+(((x1242)*(x1243)))+(((r02)*(x1241)))+(((IkReal(-1.00000000000000))*(r02)*(x1244))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1239)*(x1244)))+(((IkReal(-1.00000000000000))*(x1239)*(x1241)))+(((r02)*(x1240)))+(((r02)*(x1242)))+(((x1243)*(x1244)))+(((IkReal(-1.00000000000000))*(x1241)*(x1243))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1239)*(x1242)))+(((x1239)*(x1240)))+(((x1240)*(x1243)))+(((x1242)*(x1243)))+(((r02)*(x1241)))+(((IkReal(-1.00000000000000))*(r02)*(x1244)))))+IKsqr(((((x1239)*(x1244)))+(((IkReal(-1.00000000000000))*(x1239)*(x1241)))+(((r02)*(x1240)))+(((r02)*(x1242)))+(((x1243)*(x1244)))+(((IkReal(-1.00000000000000))*(x1241)*(x1243)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1239)*(x1242)))+(((x1239)*(x1240)))+(((x1240)*(x1243)))+(((x1242)*(x1243)))+(((r02)*(x1241)))+(((IkReal(-1.00000000000000))*(r02)*(x1244)))), ((((x1239)*(x1244)))+(((IkReal(-1.00000000000000))*(x1239)*(x1241)))+(((r02)*(x1240)))+(((r02)*(x1242)))+(((x1243)*(x1244)))+(((IkReal(-1.00000000000000))*(x1241)*(x1243)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1246=((IkReal(1.00000000000000))*(cj2));
IkReal x1247=((r01)*(sj0));
IkReal x1248=((cj2)*(sj1));
IkReal x1249=((cj1)*(r02));
IkReal x1250=((r02)*(sj1));
IkReal x1251=((cj0)*(r00));
IkReal x1252=((sj2)*(x1247));
IkReal x1253=((cj1)*(x1251));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj2)*(x1249)))+(((sj1)*(x1252)))+(((IkReal(-1.00000000000000))*(x1246)*(x1253)))+(((IkReal(-1.00000000000000))*(x1246)*(x1250)))+(IKcos(j5))+(((sj1)*(sj2)*(x1251)))+(((IkReal(-1.00000000000000))*(cj1)*(x1246)*(x1247))));
evalcond[1]=((((x1247)*(x1248)))+(((cj1)*(x1252)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((x1248)*(x1251)))+(((IkReal(-1.00000000000000))*(x1246)*(x1249)))+(((sj2)*(x1250)))+(((sj2)*(x1253))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1254=((pz)*(sj2));
IkReal x1255=((IkReal(1.00000000000000))*(cj1));
IkReal x1256=((cj2)*(sj1));
IkReal x1257=((py)*(sj0));
IkReal x1258=((cj0)*(px));
IkReal x1259=((sj1)*(sj2));
IkReal x1260=((IkReal(1.00000000000000))*(cj0));
IkReal x1261=((IkReal(0.0750000000000000))*(cj1));
IkReal x1262=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0800000000000000))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1260))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1260))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1258)*(x1262)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x1254)))+(((x1256)*(x1257)))+(((x1256)*(x1258)))+(((IkReal(-1.00000000000000))*(sj2)*(x1261)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1255)))+(((x1257)*(x1262)))+(((IkReal(-0.0750000000000000))*(x1256))));
evalcond[4]=((IkReal(0.295000000000000))+(((x1258)*(x1259)))+(((IkReal(-1.00000000000000))*(pz)*(x1256)))+(((cj2)*(x1261)))+(((IkReal(-1.00000000000000))*(cj2)*(x1255)*(x1258)))+(((IkReal(-1.00000000000000))*(cj2)*(x1255)*(x1257)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1254)*(x1255)))+(((x1257)*(x1259)))+(((IkReal(-0.0750000000000000))*(x1259))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1263=((IkReal(1.00000000000000))*(cj2));
IkReal x1264=((r02)*(sj2));
IkReal x1265=((r01)*(sj0));
IkReal x1266=((cj2)*(sj1));
IkReal x1267=((cj0)*(r00));
IkReal x1268=((sj2)*(x1265));
IkReal x1269=((cj1)*(x1267));
if( IKabs(((((sj1)*(x1264)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1263)))+(((x1266)*(x1267)))+(((cj1)*(x1268)))+(((x1265)*(x1266)))+(((sj2)*(x1269))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj1)*(x1268)))+(((IkReal(-1.00000000000000))*(x1263)*(x1269)))+(((IkReal(-1.00000000000000))*(cj1)*(x1264)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1263)))+(((sj1)*(sj2)*(x1267)))+(((IkReal(-1.00000000000000))*(cj1)*(x1263)*(x1265))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj1)*(x1264)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1263)))+(((x1266)*(x1267)))+(((cj1)*(x1268)))+(((x1265)*(x1266)))+(((sj2)*(x1269)))))+IKsqr(((((sj1)*(x1268)))+(((IkReal(-1.00000000000000))*(x1263)*(x1269)))+(((IkReal(-1.00000000000000))*(cj1)*(x1264)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1263)))+(((sj1)*(sj2)*(x1267)))+(((IkReal(-1.00000000000000))*(cj1)*(x1263)*(x1265)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((sj1)*(x1264)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1263)))+(((x1266)*(x1267)))+(((cj1)*(x1268)))+(((x1265)*(x1266)))+(((sj2)*(x1269)))), ((((sj1)*(x1268)))+(((IkReal(-1.00000000000000))*(x1263)*(x1269)))+(((IkReal(-1.00000000000000))*(cj1)*(x1264)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1263)))+(((sj1)*(sj2)*(x1267)))+(((IkReal(-1.00000000000000))*(cj1)*(x1263)*(x1265)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1270=((IkReal(1.00000000000000))*(cj2));
IkReal x1271=((r01)*(sj0));
IkReal x1272=((cj2)*(sj1));
IkReal x1273=((cj1)*(r02));
IkReal x1274=((r02)*(sj1));
IkReal x1275=((cj0)*(r00));
IkReal x1276=((sj2)*(x1271));
IkReal x1277=((cj1)*(x1275));
evalcond[0]=((((sj1)*(x1276)))+(((IkReal(-1.00000000000000))*(x1270)*(x1277)))+(((IkReal(-1.00000000000000))*(x1270)*(x1274)))+(((IkReal(-1.00000000000000))*(cj1)*(x1270)*(x1271)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((sj1)*(sj2)*(x1275)))+(((IkReal(-1.00000000000000))*(sj2)*(x1273))));
evalcond[1]=((((sj2)*(x1277)))+(((sj2)*(x1274)))+(((IkReal(-1.00000000000000))*(x1270)*(x1273)))+(((x1272)*(x1275)))+(((cj1)*(x1276)))+(((x1271)*(x1272)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1278=((r01)*(sj0));
IkReal x1279=((cj1)*(sj2));
IkReal x1280=((sj1)*(sj2));
IkReal x1281=((cj2)*(sj1));
IkReal x1282=((cj0)*(r00));
IkReal x1283=((cj1)*(cj2));
IkReal x1284=((cj1)*(x1282));
if( IKabs(((((x1279)*(x1282)))+(((r02)*(x1280)))+(((x1278)*(x1281)))+(((x1278)*(x1279)))+(((IkReal(-1.00000000000000))*(r02)*(x1283)))+(((x1281)*(x1282))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((r02)*(x1281)))+(((x1278)*(x1283)))+(((IkReal(-1.00000000000000))*(x1280)*(x1282)))+(((r02)*(x1279)))+(((IkReal(-1.00000000000000))*(x1278)*(x1280)))+(((x1282)*(x1283))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1279)*(x1282)))+(((r02)*(x1280)))+(((x1278)*(x1281)))+(((x1278)*(x1279)))+(((IkReal(-1.00000000000000))*(r02)*(x1283)))+(((x1281)*(x1282)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((r02)*(x1281)))+(((x1278)*(x1283)))+(((IkReal(-1.00000000000000))*(x1280)*(x1282)))+(((r02)*(x1279)))+(((IkReal(-1.00000000000000))*(x1278)*(x1280)))+(((x1282)*(x1283)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1279)*(x1282)))+(((r02)*(x1280)))+(((x1278)*(x1281)))+(((x1278)*(x1279)))+(((IkReal(-1.00000000000000))*(r02)*(x1283)))+(((x1281)*(x1282)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((r02)*(x1281)))+(((x1278)*(x1283)))+(((IkReal(-1.00000000000000))*(x1280)*(x1282)))+(((r02)*(x1279)))+(((IkReal(-1.00000000000000))*(x1278)*(x1280)))+(((x1282)*(x1283)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1285=IKcos(j5);
IkReal x1286=((r01)*(sj0));
IkReal x1287=((sj1)*(sj2));
IkReal x1288=((IkReal(1.00000000000000))*(cj0));
IkReal x1289=((cj1)*(cj2));
IkReal x1290=((cj1)*(sj2));
IkReal x1291=((cj0)*(r00));
IkReal x1292=((cj2)*(sj1));
IkReal x1293=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((cj4)*(x1285)))+(((IkReal(-1.00000000000000))*(r01)*(x1288))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x1288)*(x1289)))+(((x1287)*(x1291)))+(((sj4)*(x1285)))+(((IkReal(-1.00000000000000))*(x1286)*(x1289)))+(((IkReal(-1.00000000000000))*(x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(x1290)*(x1293)))+(((x1286)*(x1287))));
evalcond[2]=((((r02)*(x1287)))+(((x1286)*(x1292)))+(((x1286)*(x1290)))+(((IkReal(-1.00000000000000))*(x1289)*(x1293)))+(((x1290)*(x1291)))+(((x1291)*(x1292)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1294=((cj1)*(sj2));
IkReal x1295=((r01)*(sj0));
IkReal x1296=((cj0)*(r00));
IkReal x1297=((cj2)*(sj1));
if( IKabs(((((x1296)*(x1297)))+(((x1295)*(x1297)))+(((x1294)*(x1295)))+(((x1294)*(x1296)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1296)*(x1297)))+(((x1295)*(x1297)))+(((x1294)*(x1295)))+(((x1294)*(x1296)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1296)*(x1297)))+(((x1295)*(x1297)))+(((x1294)*(x1295)))+(((x1294)*(x1296)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1298=IKcos(j5);
IkReal x1299=((r01)*(sj0));
IkReal x1300=((sj1)*(sj2));
IkReal x1301=((IkReal(1.00000000000000))*(cj0));
IkReal x1302=((cj1)*(cj2));
IkReal x1303=((cj1)*(sj2));
IkReal x1304=((cj0)*(r00));
IkReal x1305=((cj2)*(sj1));
IkReal x1306=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1301)))+(((cj4)*(x1298))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x1301)*(x1302)))+(((x1299)*(x1300)))+(((x1300)*(x1304)))+(((IkReal(-1.00000000000000))*(x1303)*(x1306)))+(((IkReal(-1.00000000000000))*(x1299)*(x1302)))+(((IkReal(-1.00000000000000))*(x1305)*(x1306)))+(((sj4)*(x1298))));
evalcond[2]=((((x1299)*(x1303)))+(((x1299)*(x1305)))+(((x1303)*(x1304)))+(((x1304)*(x1305)))+(((r02)*(x1300)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((IkReal(-1.00000000000000))*(x1302)*(x1306))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1307=((pz)*(sj2));
IkReal x1308=((IkReal(1.00000000000000))*(cj1));
IkReal x1309=((cj2)*(sj1));
IkReal x1310=((py)*(sj0));
IkReal x1311=((cj0)*(px));
IkReal x1312=((sj1)*(sj2));
IkReal x1313=((IkReal(0.0750000000000000))*(cj1));
IkReal x1314=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((x1310)*(x1314)))+(((x1311)*(x1314)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1309)*(x1311)))+(((x1309)*(x1310)))+(((IkReal(-1.00000000000000))*(sj2)*(x1313)))+(((IkReal(-0.0750000000000000))*(x1309)))+(((IkReal(0.0800000000000000))*(cj3)))+(((sj1)*(x1307)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1308))));
evalcond[3]=((IkReal(0.295000000000000))+(((x1310)*(x1312)))+(((x1311)*(x1312)))+(((IkReal(-1.00000000000000))*(x1307)*(x1308)))+(((IkReal(-1.00000000000000))*(cj2)*(x1308)*(x1310)))+(((IkReal(-1.00000000000000))*(cj2)*(x1308)*(x1311)))+(((IkReal(-1.00000000000000))*(pz)*(x1309)))+(((IkReal(-0.0750000000000000))*(x1312)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj2)*(x1313))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1315=((cj0)*(r00));
IkReal x1316=((cj1)*(cj2));
IkReal x1317=((cj2)*(sj1));
IkReal x1318=((r01)*(sj0));
IkReal x1319=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x1320=((IkReal(1.00000000000000))*(cj1)*(sj2));
if( IKabs(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1317)*(x1318)))+(((IkReal(-1.00000000000000))*(x1315)*(x1320)))+(((IkReal(-1.00000000000000))*(x1315)*(x1317)))+(((IkReal(-1.00000000000000))*(r02)*(x1319)))+(((r02)*(x1316)))+(((IkReal(-1.00000000000000))*(x1318)*(x1320))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1315)*(x1319)))+(((IkReal(-1.00000000000000))*(x1318)*(x1319)))+(((x1316)*(x1318)))+(((r02)*(x1317)))+(((x1315)*(x1316)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1317)*(x1318)))+(((IkReal(-1.00000000000000))*(x1315)*(x1320)))+(((IkReal(-1.00000000000000))*(x1315)*(x1317)))+(((IkReal(-1.00000000000000))*(r02)*(x1319)))+(((r02)*(x1316)))+(((IkReal(-1.00000000000000))*(x1318)*(x1320)))))))+IKsqr(((((IkReal(-1.00000000000000))*(x1315)*(x1319)))+(((IkReal(-1.00000000000000))*(x1318)*(x1319)))+(((x1316)*(x1318)))+(((r02)*(x1317)))+(((x1315)*(x1316)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1317)*(x1318)))+(((IkReal(-1.00000000000000))*(x1315)*(x1320)))+(((IkReal(-1.00000000000000))*(x1315)*(x1317)))+(((IkReal(-1.00000000000000))*(r02)*(x1319)))+(((r02)*(x1316)))+(((IkReal(-1.00000000000000))*(x1318)*(x1320)))))), ((((IkReal(-1.00000000000000))*(x1315)*(x1319)))+(((IkReal(-1.00000000000000))*(x1318)*(x1319)))+(((x1316)*(x1318)))+(((r02)*(x1317)))+(((x1315)*(x1316)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1321=IKsin(j5);
IkReal x1322=((r01)*(sj0));
IkReal x1323=((sj1)*(sj2));
IkReal x1324=((IkReal(1.00000000000000))*(cj0));
IkReal x1325=((cj1)*(cj2));
IkReal x1326=((cj1)*(sj2));
IkReal x1327=((cj0)*(r00));
IkReal x1328=((cj2)*(sj1));
IkReal x1329=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1324)))+(((IkReal(-1.00000000000000))*(cj3)*(x1321))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1322)*(x1325)))+(((IkReal(-1.00000000000000))*(r00)*(x1324)*(x1325)))+(((x1323)*(x1327)))+(((x1322)*(x1323)))+(((IkReal(-1.00000000000000))*(x1328)*(x1329)))+(((IkReal(-1.00000000000000))*(x1326)*(x1329)))+(IKcos(j5)));
evalcond[2]=((((sj3)*(x1321)))+(((x1322)*(x1328)))+(((x1322)*(x1326)))+(((x1327)*(x1328)))+(((x1326)*(x1327)))+(((IkReal(-1.00000000000000))*(x1325)*(x1329)))+(((r02)*(x1323))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1330=((IkReal(1.00000000000000))*(r01));
IkReal x1331=((cj0)*(r00));
IkReal x1332=((cj1)*(cj2));
IkReal x1333=((sj1)*(sj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1330))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1331)*(x1332)))+(((r01)*(sj0)*(x1332)))+(((IkReal(-1.00000000000000))*(sj0)*(x1330)*(x1333)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x1331)*(x1333)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1330)))))))+IKsqr(((((x1331)*(x1332)))+(((r01)*(sj0)*(x1332)))+(((IkReal(-1.00000000000000))*(sj0)*(x1330)*(x1333)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x1331)*(x1333)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1330)))))), ((((x1331)*(x1332)))+(((r01)*(sj0)*(x1332)))+(((IkReal(-1.00000000000000))*(sj0)*(x1330)*(x1333)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x1331)*(x1333)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1334=IKsin(j5);
IkReal x1335=((r01)*(sj0));
IkReal x1336=((sj1)*(sj2));
IkReal x1337=((IkReal(1.00000000000000))*(cj0));
IkReal x1338=((cj1)*(cj2));
IkReal x1339=((cj1)*(sj2));
IkReal x1340=((cj0)*(r00));
IkReal x1341=((cj2)*(sj1));
IkReal x1342=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj3)*(x1334)))+(((IkReal(-1.00000000000000))*(r01)*(x1337))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1341)*(x1342)))+(((x1335)*(x1336)))+(((IkReal(-1.00000000000000))*(x1335)*(x1338)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(x1339)*(x1342)))+(((IkReal(-1.00000000000000))*(r00)*(x1337)*(x1338)))+(((x1336)*(x1340))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1338)*(x1342)))+(((x1339)*(x1340)))+(((x1335)*(x1339)))+(((x1340)*(x1341)))+(((sj3)*(x1334)))+(((x1335)*(x1341)))+(((r02)*(x1336))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1343=((pz)*(sj2));
IkReal x1344=((IkReal(1.00000000000000))*(cj1));
IkReal x1345=((cj2)*(sj1));
IkReal x1346=((py)*(sj0));
IkReal x1347=((cj0)*(px));
IkReal x1348=((sj1)*(sj2));
IkReal x1349=((IkReal(0.0750000000000000))*(cj1));
IkReal x1350=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((sj1)*(x1343)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1349)))+(((IkReal(-0.0750000000000000))*(x1345)))+(((x1345)*(x1346)))+(((x1345)*(x1347)))+(((x1346)*(x1350)))+(((x1347)*(x1350)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1344)))+(((IkReal(-0.0800000000000000))*(cj3))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x1348)))+(((x1346)*(x1348)))+(((x1347)*(x1348)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1343)*(x1344)))+(((IkReal(-1.00000000000000))*(cj2)*(x1344)*(x1346)))+(((IkReal(-1.00000000000000))*(cj2)*(x1344)*(x1347)))+(((cj2)*(x1349)))+(((IkReal(-1.00000000000000))*(pz)*(x1345))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1351=((sj1)*(sj2));
IkReal x1352=((IkReal(1.00000000000000))*(r02));
IkReal x1353=((r01)*(sj0));
IkReal x1354=((cj0)*(r00));
IkReal x1355=((cj1)*(sj2));
IkReal x1356=((cj2)*(sj1));
IkReal x1357=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1353)*(x1356)))+(((IkReal(-1.00000000000000))*(x1353)*(x1355)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1351)*(x1352)))+(((IkReal(-1.00000000000000))*(x1354)*(x1356)))+(((IkReal(-1.00000000000000))*(x1354)*(x1355))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1352)*(x1355)))+(((IkReal(-1.00000000000000))*(x1352)*(x1356)))+(((IkReal(-1.00000000000000))*(x1354)*(x1357)))+(((x1351)*(x1353)))+(((x1351)*(x1354)))+(((IkReal(-1.00000000000000))*(x1353)*(x1357))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1353)*(x1356)))+(((IkReal(-1.00000000000000))*(x1353)*(x1355)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1351)*(x1352)))+(((IkReal(-1.00000000000000))*(x1354)*(x1356)))+(((IkReal(-1.00000000000000))*(x1354)*(x1355)))))))+IKsqr(((((IkReal(-1.00000000000000))*(x1352)*(x1355)))+(((IkReal(-1.00000000000000))*(x1352)*(x1356)))+(((IkReal(-1.00000000000000))*(x1354)*(x1357)))+(((x1351)*(x1353)))+(((x1351)*(x1354)))+(((IkReal(-1.00000000000000))*(x1353)*(x1357)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1353)*(x1356)))+(((IkReal(-1.00000000000000))*(x1353)*(x1355)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1351)*(x1352)))+(((IkReal(-1.00000000000000))*(x1354)*(x1356)))+(((IkReal(-1.00000000000000))*(x1354)*(x1355)))))), ((((IkReal(-1.00000000000000))*(x1352)*(x1355)))+(((IkReal(-1.00000000000000))*(x1352)*(x1356)))+(((IkReal(-1.00000000000000))*(x1354)*(x1357)))+(((x1351)*(x1353)))+(((x1351)*(x1354)))+(((IkReal(-1.00000000000000))*(x1353)*(x1357)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1358=IKsin(j5);
IkReal x1359=((r01)*(sj0));
IkReal x1360=((sj1)*(sj2));
IkReal x1361=((IkReal(1.00000000000000))*(cj0));
IkReal x1362=((cj1)*(cj2));
IkReal x1363=((cj1)*(sj2));
IkReal x1364=((cj0)*(r00));
IkReal x1365=((cj2)*(sj1));
IkReal x1366=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1361)))+(((IkReal(-1.00000000000000))*(cj3)*(x1358))));
evalcond[1]=((((x1360)*(x1364)))+(((x1359)*(x1360)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(r00)*(x1361)*(x1362)))+(((IkReal(-1.00000000000000))*(x1359)*(x1362)))+(((IkReal(-1.00000000000000))*(x1365)*(x1366)))+(((IkReal(-1.00000000000000))*(x1363)*(x1366))));
evalcond[2]=((((x1363)*(x1364)))+(((r02)*(x1360)))+(((x1359)*(x1363)))+(((x1359)*(x1365)))+(((IkReal(-1.00000000000000))*(x1362)*(x1366)))+(((x1364)*(x1365)))+(((sj3)*(x1358))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1367=((r01)*(sj0));
IkReal x1368=((sj1)*(sj2));
IkReal x1369=((IkReal(1.00000000000000))*(cj0));
IkReal x1370=((cj1)*(cj2));
IkReal x1371=((IkReal(1.00000000000000))*(r02));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1369))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1367)*(x1368)))+(((cj0)*(r00)*(x1368)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1371)))+(((IkReal(-1.00000000000000))*(x1367)*(x1370)))+(((IkReal(-1.00000000000000))*(r00)*(x1369)*(x1370)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1371))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1369)))))))+IKsqr(((((x1367)*(x1368)))+(((cj0)*(r00)*(x1368)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1371)))+(((IkReal(-1.00000000000000))*(x1367)*(x1370)))+(((IkReal(-1.00000000000000))*(r00)*(x1369)*(x1370)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1371)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1369)))))), ((((x1367)*(x1368)))+(((cj0)*(r00)*(x1368)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1371)))+(((IkReal(-1.00000000000000))*(x1367)*(x1370)))+(((IkReal(-1.00000000000000))*(r00)*(x1369)*(x1370)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1371)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1372=IKsin(j5);
IkReal x1373=((r01)*(sj0));
IkReal x1374=((sj1)*(sj2));
IkReal x1375=((IkReal(1.00000000000000))*(cj0));
IkReal x1376=((cj1)*(cj2));
IkReal x1377=((cj1)*(sj2));
IkReal x1378=((cj0)*(r00));
IkReal x1379=((cj2)*(sj1));
IkReal x1380=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1375)))+(((IkReal(-1.00000000000000))*(cj3)*(x1372))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1373)*(x1376)))+(((IkReal(-1.00000000000000))*(x1377)*(x1380)))+(((x1374)*(x1378)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(r00)*(x1375)*(x1376)))+(((IkReal(-1.00000000000000))*(x1379)*(x1380)))+(((x1373)*(x1374))));
evalcond[2]=((((sj3)*(x1372)))+(((IkReal(-1.00000000000000))*(x1376)*(x1380)))+(((x1377)*(x1378)))+(((x1378)*(x1379)))+(((r02)*(x1374)))+(((x1373)*(x1379)))+(((x1373)*(x1377))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1381=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1382=((r01)*(sj0));
IkReal x1383=((r02)*(sj1));
IkReal x1384=((cj1)*(cj2));
IkReal x1385=((cj3)*(cj4));
IkReal x1386=((cj0)*(r00));
IkReal x1387=((IkReal(1.00000000000000))*(sj2)*(sj4));
IkReal x1388=((cj1)*(x1386));
IkReal x1389=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x1390=((cj1)*(r02)*(sj2));
IkReal x1391=((IkReal(1.00000000000000))*(cj2)*(sj1)*(sj4));
if( IKabs(((x1381)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((x1384)*(x1385)*(x1386)))+(((IkReal(-1.00000000000000))*(x1383)*(x1387)))+(((IkReal(-1.00000000000000))*(x1382)*(x1385)*(x1389)))+(((IkReal(-1.00000000000000))*(x1382)*(x1391)))+(((r02)*(sj4)*(x1384)))+(((IkReal(-1.00000000000000))*(cj1)*(x1382)*(x1387)))+(((x1382)*(x1384)*(x1385)))+(((x1385)*(x1390)))+(((cj2)*(x1383)*(x1385)))+(((IkReal(-1.00000000000000))*(x1385)*(x1386)*(x1389)))+(((IkReal(-1.00000000000000))*(x1387)*(x1388)))+(((IkReal(-1.00000000000000))*(x1386)*(x1391))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1381)*(((x1390)+(((cj2)*(x1383)))+(((IkReal(-1.00000000000000))*(x1382)*(x1389)))+(((x1384)*(x1386)))+(((IkReal(-1.00000000000000))*(x1386)*(x1389)))+(((x1382)*(x1384))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1381)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((x1384)*(x1385)*(x1386)))+(((IkReal(-1.00000000000000))*(x1383)*(x1387)))+(((IkReal(-1.00000000000000))*(x1382)*(x1385)*(x1389)))+(((IkReal(-1.00000000000000))*(x1382)*(x1391)))+(((r02)*(sj4)*(x1384)))+(((IkReal(-1.00000000000000))*(cj1)*(x1382)*(x1387)))+(((x1382)*(x1384)*(x1385)))+(((x1385)*(x1390)))+(((cj2)*(x1383)*(x1385)))+(((IkReal(-1.00000000000000))*(x1385)*(x1386)*(x1389)))+(((IkReal(-1.00000000000000))*(x1387)*(x1388)))+(((IkReal(-1.00000000000000))*(x1386)*(x1391)))))))+IKsqr(((x1381)*(((x1390)+(((cj2)*(x1383)))+(((IkReal(-1.00000000000000))*(x1382)*(x1389)))+(((x1384)*(x1386)))+(((IkReal(-1.00000000000000))*(x1386)*(x1389)))+(((x1382)*(x1384)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1381)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((x1384)*(x1385)*(x1386)))+(((IkReal(-1.00000000000000))*(x1383)*(x1387)))+(((IkReal(-1.00000000000000))*(x1382)*(x1385)*(x1389)))+(((IkReal(-1.00000000000000))*(x1382)*(x1391)))+(((r02)*(sj4)*(x1384)))+(((IkReal(-1.00000000000000))*(cj1)*(x1382)*(x1387)))+(((x1382)*(x1384)*(x1385)))+(((x1385)*(x1390)))+(((cj2)*(x1383)*(x1385)))+(((IkReal(-1.00000000000000))*(x1385)*(x1386)*(x1389)))+(((IkReal(-1.00000000000000))*(x1387)*(x1388)))+(((IkReal(-1.00000000000000))*(x1386)*(x1391)))))), ((x1381)*(((x1390)+(((cj2)*(x1383)))+(((IkReal(-1.00000000000000))*(x1382)*(x1389)))+(((x1384)*(x1386)))+(((IkReal(-1.00000000000000))*(x1386)*(x1389)))+(((x1382)*(x1384)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1392=IKcos(j5);
IkReal x1393=IKsin(j5);
IkReal x1394=((r01)*(sj0));
IkReal x1395=((sj1)*(sj2));
IkReal x1396=((IkReal(1.00000000000000))*(cj0));
IkReal x1397=((cj1)*(cj2));
IkReal x1398=((cj1)*(sj2));
IkReal x1399=((cj0)*(r00));
IkReal x1400=((cj2)*(sj1));
IkReal x1401=((IkReal(1.00000000000000))*(cj3));
IkReal x1402=((IkReal(1.00000000000000))*(r02));
IkReal x1403=((cj4)*(x1392));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1403)))+(((IkReal(-1.00000000000000))*(r01)*(x1396)))+(((IkReal(-1.00000000000000))*(x1393)*(x1401))));
evalcond[1]=((((sj4)*(x1392)))+(((IkReal(-1.00000000000000))*(x1398)*(x1402)))+(((x1394)*(x1395)))+(((x1395)*(x1399)))+(((IkReal(-1.00000000000000))*(r00)*(x1396)*(x1397)))+(((IkReal(-1.00000000000000))*(x1400)*(x1402)))+(((IkReal(-1.00000000000000))*(x1394)*(x1397))));
evalcond[2]=((((x1398)*(x1399)))+(((x1394)*(x1398)))+(((x1394)*(x1400)))+(((sj3)*(x1393)))+(((x1399)*(x1400)))+(((IkReal(-1.00000000000000))*(x1401)*(x1403)))+(((IkReal(-1.00000000000000))*(x1397)*(x1402)))+(((r02)*(x1395))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1404=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1405=((r01)*(sj0));
IkReal x1406=((r02)*(sj1));
IkReal x1407=((IkReal(1.00000000000000))*(sj2));
IkReal x1408=((cj4)*(sj3));
IkReal x1409=((cj1)*(r02));
IkReal x1410=((sj1)*(sj2));
IkReal x1411=((cj0)*(r00));
IkReal x1412=((cj1)*(x1411));
IkReal x1413=((IkReal(1.00000000000000))*(cj2)*(x1408));
if( IKabs(((x1404)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1412)*(x1413)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1407)*(x1408)*(x1409)))+(((IkReal(-1.00000000000000))*(cj1)*(x1405)*(x1413)))+(((IkReal(-1.00000000000000))*(x1406)*(x1413)))+(((x1408)*(x1410)*(x1411)))+(((x1405)*(x1408)*(x1410))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1404)*(((((cj2)*(x1412)))+(((IkReal(-1.00000000000000))*(sj1)*(x1407)*(x1411)))+(((sj2)*(x1409)))+(((IkReal(-1.00000000000000))*(sj1)*(x1405)*(x1407)))+(((cj1)*(cj2)*(x1405)))+(((cj2)*(x1406))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1404)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1412)*(x1413)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1407)*(x1408)*(x1409)))+(((IkReal(-1.00000000000000))*(cj1)*(x1405)*(x1413)))+(((IkReal(-1.00000000000000))*(x1406)*(x1413)))+(((x1408)*(x1410)*(x1411)))+(((x1405)*(x1408)*(x1410)))))))+IKsqr(((x1404)*(((((cj2)*(x1412)))+(((IkReal(-1.00000000000000))*(sj1)*(x1407)*(x1411)))+(((sj2)*(x1409)))+(((IkReal(-1.00000000000000))*(sj1)*(x1405)*(x1407)))+(((cj1)*(cj2)*(x1405)))+(((cj2)*(x1406)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1404)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1412)*(x1413)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1407)*(x1408)*(x1409)))+(((IkReal(-1.00000000000000))*(cj1)*(x1405)*(x1413)))+(((IkReal(-1.00000000000000))*(x1406)*(x1413)))+(((x1408)*(x1410)*(x1411)))+(((x1405)*(x1408)*(x1410)))))), ((x1404)*(((((cj2)*(x1412)))+(((IkReal(-1.00000000000000))*(sj1)*(x1407)*(x1411)))+(((sj2)*(x1409)))+(((IkReal(-1.00000000000000))*(sj1)*(x1405)*(x1407)))+(((cj1)*(cj2)*(x1405)))+(((cj2)*(x1406)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1414=IKcos(j5);
IkReal x1415=IKsin(j5);
IkReal x1416=((r01)*(sj0));
IkReal x1417=((sj1)*(sj2));
IkReal x1418=((IkReal(1.00000000000000))*(cj0));
IkReal x1419=((cj1)*(cj2));
IkReal x1420=((cj1)*(sj2));
IkReal x1421=((cj0)*(r00));
IkReal x1422=((cj2)*(sj1));
IkReal x1423=((IkReal(1.00000000000000))*(cj3));
IkReal x1424=((IkReal(1.00000000000000))*(r02));
IkReal x1425=((cj4)*(x1414));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1415)*(x1423)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1425)))+(((IkReal(-1.00000000000000))*(r01)*(x1418))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1416)*(x1419)))+(((IkReal(-1.00000000000000))*(x1420)*(x1424)))+(((x1416)*(x1417)))+(((sj4)*(x1414)))+(((IkReal(-1.00000000000000))*(x1422)*(x1424)))+(((IkReal(-1.00000000000000))*(r00)*(x1418)*(x1419)))+(((x1417)*(x1421))));
evalcond[2]=((((x1420)*(x1421)))+(((IkReal(-1.00000000000000))*(x1423)*(x1425)))+(((IkReal(-1.00000000000000))*(x1419)*(x1424)))+(((r02)*(x1417)))+(((x1421)*(x1422)))+(((sj3)*(x1415)))+(((x1416)*(x1422)))+(((x1416)*(x1420))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1426=(sj3)*(sj3);
IkReal x1427=(cj3)*(cj3);
IkReal x1428=((cj2)*(cj3));
IkReal x1429=((cj1)*(r02));
IkReal x1430=((r01)*(sj0));
IkReal x1431=((cj3)*(sj2));
IkReal x1432=((r02)*(sj1));
IkReal x1433=((cj0)*(r00));
IkReal x1434=((r00)*(sj0));
IkReal x1435=((sj1)*(x1433));
IkReal x1436=((IkReal(1.00000000000000))*(cj2)*(sj3));
IkReal x1437=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x1438=((IkReal(1.00000000000000))*(cj1)*(sj2)*(sj3));
if( IKabs(((((IKabs(((x1427)+(x1426))) != 0)?((IkReal)1/(((x1427)+(x1426)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj1)*(x1430)*(x1436)))+(((IkReal(-1.00000000000000))*(x1435)*(x1436)))+(((IkReal(-1.00000000000000))*(cj3)*(x1437)))+(((cj2)*(sj3)*(x1429)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1432)))+(((IkReal(-1.00000000000000))*(x1430)*(x1438)))+(((cj3)*(x1434)))+(((IkReal(-1.00000000000000))*(x1433)*(x1438))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1426)))+(((cj4)*(x1427))))) != 0)?((IkReal)1/(((((cj4)*(x1426)))+(((cj4)*(x1427)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x1437)))+(((x1428)*(x1435)))+(((cj1)*(x1431)*(x1433)))+(((sj3)*(x1434)))+(((cj1)*(x1430)*(x1431)))+(((IkReal(-1.00000000000000))*(x1428)*(x1429)))+(((x1431)*(x1432)))+(((sj1)*(x1428)*(x1430))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x1427)+(x1426))) != 0)?((IkReal)1/(((x1427)+(x1426)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj1)*(x1430)*(x1436)))+(((IkReal(-1.00000000000000))*(x1435)*(x1436)))+(((IkReal(-1.00000000000000))*(cj3)*(x1437)))+(((cj2)*(sj3)*(x1429)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1432)))+(((IkReal(-1.00000000000000))*(x1430)*(x1438)))+(((cj3)*(x1434)))+(((IkReal(-1.00000000000000))*(x1433)*(x1438)))))))+IKsqr(((((IKabs(((((cj4)*(x1426)))+(((cj4)*(x1427))))) != 0)?((IkReal)1/(((((cj4)*(x1426)))+(((cj4)*(x1427)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x1437)))+(((x1428)*(x1435)))+(((cj1)*(x1431)*(x1433)))+(((sj3)*(x1434)))+(((cj1)*(x1430)*(x1431)))+(((IkReal(-1.00000000000000))*(x1428)*(x1429)))+(((x1431)*(x1432)))+(((sj1)*(x1428)*(x1430)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x1427)+(x1426))) != 0)?((IkReal)1/(((x1427)+(x1426)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj1)*(x1430)*(x1436)))+(((IkReal(-1.00000000000000))*(x1435)*(x1436)))+(((IkReal(-1.00000000000000))*(cj3)*(x1437)))+(((cj2)*(sj3)*(x1429)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1432)))+(((IkReal(-1.00000000000000))*(x1430)*(x1438)))+(((cj3)*(x1434)))+(((IkReal(-1.00000000000000))*(x1433)*(x1438)))))), ((((IKabs(((((cj4)*(x1426)))+(((cj4)*(x1427))))) != 0)?((IkReal)1/(((((cj4)*(x1426)))+(((cj4)*(x1427)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x1437)))+(((x1428)*(x1435)))+(((cj1)*(x1431)*(x1433)))+(((sj3)*(x1434)))+(((cj1)*(x1430)*(x1431)))+(((IkReal(-1.00000000000000))*(x1428)*(x1429)))+(((x1431)*(x1432)))+(((sj1)*(x1428)*(x1430)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1439=IKcos(j5);
IkReal x1440=IKsin(j5);
IkReal x1441=((r01)*(sj0));
IkReal x1442=((sj1)*(sj2));
IkReal x1443=((IkReal(1.00000000000000))*(cj0));
IkReal x1444=((cj1)*(cj2));
IkReal x1445=((cj1)*(sj2));
IkReal x1446=((cj0)*(r00));
IkReal x1447=((cj2)*(sj1));
IkReal x1448=((IkReal(1.00000000000000))*(cj3));
IkReal x1449=((IkReal(1.00000000000000))*(r02));
IkReal x1450=((cj4)*(x1439));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1450)))+(((IkReal(-1.00000000000000))*(r01)*(x1443)))+(((IkReal(-1.00000000000000))*(x1440)*(x1448))));
evalcond[1]=((((x1442)*(x1446)))+(((IkReal(-1.00000000000000))*(x1445)*(x1449)))+(((x1441)*(x1442)))+(((IkReal(-1.00000000000000))*(r00)*(x1443)*(x1444)))+(((IkReal(-1.00000000000000))*(x1441)*(x1444)))+(((IkReal(-1.00000000000000))*(x1447)*(x1449)))+(((sj4)*(x1439))));
evalcond[2]=((((x1445)*(x1446)))+(((x1441)*(x1447)))+(((x1441)*(x1445)))+(((IkReal(-1.00000000000000))*(x1448)*(x1450)))+(((r02)*(x1442)))+(((sj3)*(x1440)))+(((IkReal(-1.00000000000000))*(x1444)*(x1449)))+(((x1446)*(x1447))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "f39e879836615ea489f5611d5593ce3c"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
