/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:11:35.434926
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25;
x0=IKsin(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[1]);
x3=IKcos(j[2]);
x4=IKsin(j[2]);
x5=IKcos(j[0]);
x6=IKcos(j[3]);
x7=IKsin(j[3]);
x8=IKcos(j[5]);
x9=IKsin(j[5]);
x10=IKsin(j[4]);
x11=IKcos(j[4]);
x12=((IkReal(0.331000000000000))*(x0));
x13=((IkReal(1.00000000000000))*(x11));
x14=((IkReal(1.00000000000000))*(x0));
x15=((IkReal(1.00000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x10));
x17=((IkReal(0.405000000000000))*(x1));
x18=((IkReal(0.331000000000000))*(x5));
x19=((x1)*(x3));
x20=((x2)*(x4));
x21=((x2)*(x3));
x22=((x1)*(x4));
x23=((IkReal(1.00000000000000))*(x19));
x24=((x14)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x22))))));
x25=((x15)*(((x21)+(x22))));
eetrans[0]=((IkReal(0.0454160000000000))+(((IkReal(-0.100150000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x12)*(x19)))+(((IkReal(-1.00000000000000))*(x0)*(x17)))+(((x12)*(x20))));
eetrans[1]=((IkReal(0.302408000000000))+(((IkReal(0.331000000000000))*(x21)))+(((IkReal(0.331000000000000))*(x22)))+(((IkReal(0.405000000000000))*(x2))));
eetrans[2]=((IkReal(0.467052000000000))+(((x18)*(x19)))+(((IkReal(-0.100150000000000))*(x0)))+(((IkReal(-1.00000000000000))*(x18)*(x20)))+(((x17)*(x5))));
eerot[0]=((((x8)*(((((IkReal(-1.00000000000000))*(x5)*(x7)))+(((x24)*(x6)))))))+(((IkReal(-1.00000000000000))*(x9)*(((((x13)*(((((x5)*(x6)))+(((x24)*(x7)))))))+(((x16)*(((((x0)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x20))))))))))));
eerot[1]=((((x9)*(((((x11)*(x7)*(((((IkReal(-1.00000000000000))*(x20)))+(x23)))))+(((x10)*(((x21)+(x22)))))))))+(((x6)*(x8)*(((((IkReal(-1.00000000000000))*(x23)))+(x20))))));
eerot[2]=((((IkReal(-1.00000000000000))*(x9)*(((((x16)*(((((IkReal(-1.00000000000000))*(x15)*(x19)))+(((x20)*(x5)))))))+(((x13)*(((((x0)*(x6)))+(((x25)*(x7)))))))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x14)*(x7)))+(((x25)*(x6))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j1,cj1,sj1,htj1,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((IkReal(-0.0454160000000000))+(px));
new_r01=r02;
new_py=((IkReal(-0.467052000000000))+(pz));
new_r02=((IkReal(-1.00000000000000))*(r01));
new_pz=((IkReal(0.302408000000000))+(((IkReal(-1.00000000000000))*(py))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal Noneeval[1];
IkReal x26=((IkReal(3.02114803625378))*(cj1));
IkReal x27=((cj0)*(py));
IkReal x28=((px)*(sj0));
IkReal x29=((IkReal(3.02114803625378))*(sj1)*(sj2));
Noneeval[0]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x26)*(x27)))+(((IkReal(1.22356495468278))*(cj2)))+(((pz)*(sj2)*(x26)))+(((cj2)*(x26)*(x28)))+(((IkReal(3.02114803625378))*(cj2)*(pz)*(sj1)))+(((IkReal(-1.00000000000000))*(x28)*(x29)))+(((x27)*(x29))));
if( IKabs(Noneeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x30=(sj1)*(sj1);
IkReal x31=(px)*(px)*(px)*(px);
IkReal x32=(cj1)*(cj1);
IkReal x33=(px)*(px);
IkReal x34=(py)*(py);
IkReal x35=((IkReal(16.3534029958557))*(x32));
IkReal x36=((x30)*(x31));
IkReal x37=((x30)*(x33)*(x34));
dummyeval[0]=((((x35)*(x37)))+(((x35)*(x36)))+(x37)+(x36));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x38=(sj1)*(sj1);
IkReal x39=(py)*(py)*(py)*(py);
IkReal x40=(cj1)*(cj1);
IkReal x41=(px)*(px);
IkReal x42=(py)*(py);
IkReal x43=((x41)*(x42));
IkReal x44=((IkReal(16.3534029958557))*(x38)*(x40));
dummyeval[0]=((((x43)*(x44)))+(((x38)*(x39)))+(((x38)*(x43)))+(((x39)*(x44))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x45=(cj1)*(cj1);
IkReal x46=(px)*(px);
IkReal x47=(py)*(py);
IkReal x48=(cj1)*(cj1)*(cj1)*(cj1);
IkReal x49=(py)*(py)*(py)*(py);
IkReal x50=((IkReal(16.3534029958557))*(x48));
IkReal x51=((x46)*(x47));
dummyeval[0]=((((x45)*(x51)))+(((x50)*(x51)))+(((x45)*(x49)))+(((x49)*(x50))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[1];
evalcond[0]=IKabs(px);
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x52=(py)*(py)*(py)*(py);
dummyeval[0]=((((x52)*((cj1)*(cj1))))+(((IkReal(16.3534029958557))*(x52)*((cj1)*(cj1)*(cj1)*(cj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x53=(py)*(py)*(py)*(py);
IkReal x54=(sj1)*(sj1);
IkReal x55=((x53)*(x54));
dummyeval[0]=((x55)+(((IkReal(16.3534029958557))*(x55)*((cj1)*(cj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[1];
evalcond[0]=IKabs(py);
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x56=(pz)*(pz);
IkReal x57=((IkReal(6.09663161103490))*(x56));
dummyeval[0]=((IkReal(1.00000000000000))+(((x57)*((sj1)*(sj1))))+(((IkReal(4.93827160493827))*(pz)*(sj1)))+(((x57)*((cj1)*(cj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x58=(pz)*(pz);
IkReal x59=((IkReal(6.09663161103490))*(x58));
dummyeval[0]=((IkReal(1.00000000000000))+(((x59)*((cj1)*(cj1))))+(((IkReal(4.93827160493827))*(pz)*(sj1)))+(((x59)*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x60=(pz)*(pz);
IkReal x61=((IkReal(6.09663161103490))*(x60));
dummyeval[0]=((IkReal(1.00000000000000))+(((x61)*((cj1)*(cj1))))+(((IkReal(4.93827160493827))*(pz)*(sj1)))+(((x61)*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
IkReal x62=((IkReal(0.662000000000000))*(pz));
IkReal x63=((IkReal(0.268110000000000))+(((sj1)*(x62))));
if( IKabs(x63) < IKFAST_ATAN2_MAGTHRESH && IKabs(((cj1)*(x62))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x64=((IkReal(1.00000000000000))*(IKatan2(x63, ((cj1)*(x62)))));
if( ((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30))*(((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(pp))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30))*(((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x65=IKasin(((((IKabs(IKabs(IKsqrt((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x63)*(x63))+(((IkReal(0.438244000000000))*((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30))*(((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(pp)))));
j2array[0]=((((IkReal(-1.00000000000000))*(x64)))+(((IkReal(-1.00000000000000))*(x65))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(x65)+(((IkReal(-1.00000000000000))*(x64))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
IkReal x66=IKcos(j2);
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(0.405000000000000))*(x66)))+(((cj1)*(pz)*(IKsin(j2))))+(((pz)*(sj1)*(x66))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x67=(r00)*(r00);
IkReal x68=(cj1)*(cj1);
IkReal x69=(sj2)*(sj2);
IkReal x70=(cj2)*(cj2);
IkReal x71=(r01)*(r01);
IkReal x72=(sj1)*(sj1);
IkReal x73=((pz)*(sj1));
IkReal x74=((IkReal(0.331000000000000))*(pz));
IkReal x75=((IkReal(0.0100300225000000))*(cj2));
IkReal x76=((sj1)*(x67));
IkReal x77=((x68)*(x70));
IkReal x78=((x69)*(x71));
IkReal x79=((cj1)*(sj2)*(x67));
IkReal x80=((cj1)*(sj2)*(x71));
IkReal x81=((cj2)*(x80));
IkReal x82=((IkReal(0.405000000000000))*(cj2)*(pz)*(x68));
gconst3=IKsign(((((IkReal(-0.174055022500000))*(x80)))+(((x71)*(x74)*(x77)))+(((IkReal(-0.134055000000000))*(x81)))+(((IkReal(-0.662000000000000))*(x73)*(x81)))+(((IkReal(-0.662000000000000))*(cj2)*(x73)*(x79)))+(((x67)*(x74)*(x77)))+(((x67)*(x82)))+(((IkReal(-1.00000000000000))*(sj1)*(x71)*(x75)))+(((IkReal(-0.405000000000000))*(x73)*(x79)))+(((IkReal(0.134055000000000))*(x69)*(x76)))+(((IkReal(0.134055000000000))*(sj1)*(x78)))+(((IkReal(-0.134055000000000))*(cj2)*(x79)))+(((IkReal(-0.405000000000000))*(x73)*(x80)))+(((x72)*(x74)*(x78)))+(((x67)*(x69)*(x72)*(x74)))+(((IkReal(-0.174055022500000))*(x79)))+(((IkReal(-1.00000000000000))*(x75)*(x76)))+(((x71)*(x82)))));
IkReal x83=(r01)*(r01);
IkReal x84=(cj1)*(cj1);
IkReal x85=(r00)*(r00);
IkReal x86=(cj2)*(cj2);
IkReal x87=(sj1)*(sj1);
IkReal x88=(sj2)*(sj2);
IkReal x89=((IkReal(40.3787728292733))*(pz));
IkReal x90=((cj1)*(sj2));
IkReal x91=((IkReal(66.0018459579727))*(pz));
IkReal x92=((IkReal(33.0009229789863))*(pz));
IkReal x93=((sj1)*(x83));
IkReal x94=((IkReal(13.3653738064895))*(x88));
IkReal x95=((cj2)*(x85));
IkReal x96=((cj2)*(x83));
IkReal x97=((sj1)*(x85));
IkReal x98=((x87)*(x88));
IkReal x99=((x84)*(x86));
dummyeval[0]=((((IkReal(-17.3534029958557))*(x85)*(x90)))+(((IkReal(-1.00000000000000))*(cj2)*(x93)))+(((IkReal(-13.3653738064895))*(x90)*(x95)))+(((IkReal(-13.3653738064895))*(x90)*(x96)))+(((IkReal(-1.00000000000000))*(cj2)*(x90)*(x91)*(x93)))+(((x85)*(x92)*(x98)))+(((x85)*(x92)*(x99)))+(((IkReal(-1.00000000000000))*(sj1)*(x90)*(x91)*(x95)))+(((IkReal(-17.3534029958557))*(x83)*(x90)))+(((x93)*(x94)))+(((x83)*(x92)*(x98)))+(((x83)*(x92)*(x99)))+(((x84)*(x89)*(x96)))+(((x84)*(x89)*(x95)))+(((IkReal(-1.00000000000000))*(sj1)*(x95)))+(((IkReal(-1.00000000000000))*(x89)*(x90)*(x97)))+(((IkReal(-1.00000000000000))*(x89)*(x90)*(x93)))+(((x94)*(x97))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x100=(sj2)*(sj2);
IkReal x101=(sj1)*(sj1);
IkReal x102=(cj2)*(cj2);
IkReal x103=(cj1)*(cj1);
IkReal x104=(pz)*(pz);
IkReal x105=((r01)*(r02));
IkReal x106=((pz)*(sj1));
IkReal x107=((IkReal(0.100150000000000))*(cj1));
IkReal x108=((r00)*(r02));
IkReal x109=((IkReal(0.331000000000000))*(cj2));
IkReal x110=((sj1)*(sj2));
IkReal x111=((IkReal(0.0405607500000000))*(cj2));
IkReal x112=((pz)*(sj2));
IkReal x113=((cj1)*(cj2));
IkReal x114=((IkReal(0.100150000000000))*(cj2));
IkReal x115=((IkReal(1.00000000000000))*(x104));
IkReal x116=((IkReal(0.0331496500000000))*(x101));
IkReal x117=((IkReal(0.405000000000000))*(x101));
IkReal x118=((IkReal(0.0331496500000000))*(x103));
IkReal x119=((x108)*(x113));
IkReal x120=((x108)*(x112));
IkReal x121=((IkReal(0.134055000000000))*(cj2)*(x110));
IkReal x122=((IkReal(0.331000000000000))*(cj1)*(x100));
IkReal x123=((IkReal(0.134055000000000))*(cj1)*(x100));
IkReal x124=((IkReal(0.331000000000000))*(cj1)*(x102));
if( IKabs(((gconst3)*(((((x104)*(x108)*(x110)))+(((x117)*(x120)))+(((x103)*(x105)*(x111)))+(((x105)*(x106)*(x114)))+(((x102)*(x105)*(x116)))+(((x102)*(x105)*(x118)))+(((x105)*(x107)*(x112)))+(((x108)*(x123)))+(((x108)*(x121)))+(((IkReal(-1.00000000000000))*(x103)*(x109)*(x120)))+(((IkReal(-1.00000000000000))*(x115)*(x119)))+(((IkReal(-0.0100300225000000))*(x119)))+(((IkReal(0.174055022500000))*(x108)*(x110)))+(((x101)*(x105)*(x111)))+(((IkReal(-1.00000000000000))*(x106)*(x108)*(x124)))+(((x100)*(x105)*(x116)))+(((x100)*(x105)*(x118)))+(((x106)*(x108)*(x122)))+(((IkReal(0.405000000000000))*(x120)))+(((x101)*(x109)*(x120)))+(((IkReal(-0.405000000000000))*(x106)*(x119))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x105)*(x112)*(x117)))+(((IkReal(0.0100300225000000))*(x105)*(x113)))+(((IkReal(-0.405000000000000))*(x105)*(x112)))+(((x104)*(x105)*(x113)))+(((x105)*(x106)*(x124)))+(((x103)*(x108)*(x111)))+(((IkReal(-1.00000000000000))*(x101)*(x105)*(x109)*(x112)))+(((x103)*(x105)*(x109)*(x112)))+(((x101)*(x108)*(x111)))+(((x102)*(x108)*(x118)))+(((x102)*(x108)*(x116)))+(((IkReal(-0.174055022500000))*(x105)*(x110)))+(((IkReal(-1.00000000000000))*(x105)*(x121)))+(((IkReal(-1.00000000000000))*(x105)*(x123)))+(((x100)*(x108)*(x118)))+(((x100)*(x108)*(x116)))+(((x107)*(x120)))+(((x106)*(x108)*(x114)))+(((IkReal(0.405000000000000))*(x105)*(x106)*(x113)))+(((IkReal(-1.00000000000000))*(x105)*(x106)*(x122)))+(((IkReal(-1.00000000000000))*(x105)*(x110)*(x115))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst3)*(((((x104)*(x108)*(x110)))+(((x117)*(x120)))+(((x103)*(x105)*(x111)))+(((x105)*(x106)*(x114)))+(((x102)*(x105)*(x116)))+(((x102)*(x105)*(x118)))+(((x105)*(x107)*(x112)))+(((x108)*(x123)))+(((x108)*(x121)))+(((IkReal(-1.00000000000000))*(x103)*(x109)*(x120)))+(((IkReal(-1.00000000000000))*(x115)*(x119)))+(((IkReal(-0.0100300225000000))*(x119)))+(((IkReal(0.174055022500000))*(x108)*(x110)))+(((x101)*(x105)*(x111)))+(((IkReal(-1.00000000000000))*(x106)*(x108)*(x124)))+(((x100)*(x105)*(x116)))+(((x100)*(x105)*(x118)))+(((x106)*(x108)*(x122)))+(((IkReal(0.405000000000000))*(x120)))+(((x101)*(x109)*(x120)))+(((IkReal(-0.405000000000000))*(x106)*(x119)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x105)*(x112)*(x117)))+(((IkReal(0.0100300225000000))*(x105)*(x113)))+(((IkReal(-0.405000000000000))*(x105)*(x112)))+(((x104)*(x105)*(x113)))+(((x105)*(x106)*(x124)))+(((x103)*(x108)*(x111)))+(((IkReal(-1.00000000000000))*(x101)*(x105)*(x109)*(x112)))+(((x103)*(x105)*(x109)*(x112)))+(((x101)*(x108)*(x111)))+(((x102)*(x108)*(x118)))+(((x102)*(x108)*(x116)))+(((IkReal(-0.174055022500000))*(x105)*(x110)))+(((IkReal(-1.00000000000000))*(x105)*(x121)))+(((IkReal(-1.00000000000000))*(x105)*(x123)))+(((x100)*(x108)*(x118)))+(((x100)*(x108)*(x116)))+(((x107)*(x120)))+(((x106)*(x108)*(x114)))+(((IkReal(0.405000000000000))*(x105)*(x106)*(x113)))+(((IkReal(-1.00000000000000))*(x105)*(x106)*(x122)))+(((IkReal(-1.00000000000000))*(x105)*(x110)*(x115)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x125=IKsin(j0);
IkReal x126=IKcos(j0);
IkReal x127=(pz)*(pz);
IkReal x128=((r00)*(sj1));
IkReal x129=((cj1)*(r01));
IkReal x130=((IkReal(0.200300000000000))*(pz));
IkReal x131=((pp)*(sj1));
IkReal x132=((cj2)*(r02));
IkReal x133=((IkReal(1.00000000000000))*(pp));
IkReal x134=((IkReal(0.405000000000000))*(r00));
IkReal x135=((IkReal(0.0811215000000000))*(cj2));
IkReal x136=((cj1)*(r00));
IkReal x137=((IkReal(0.331000000000000))*(sj1));
IkReal x138=((pz)*(r02));
IkReal x139=((r02)*(sj2));
IkReal x140=((IkReal(0.100150000000000))*(sj1));
IkReal x141=((cj2)*(pz));
IkReal x142=((pz)*(r01));
IkReal x143=((r02)*(sj1));
IkReal x144=((IkReal(0.810000000000000))*(pz));
IkReal x145=((sj2)*(x125));
IkReal x146=((sj2)*(x126));
IkReal x147=((r00)*(x126));
IkReal x148=((r01)*(x125));
IkReal x149=((cj2)*(x126));
IkReal x150=((IkReal(2.00000000000000))*(x127));
IkReal x151=((cj2)*(x125)*(x136));
evalcond[0]=((((x125)*(x129)*(x141)))+(((IkReal(0.100150000000000))*(cj2)*(x125)*(x128)))+(((IkReal(-0.100150000000000))*(x129)*(x146)))+(((IkReal(-1.00000000000000))*(x134)*(x146)))+(((IkReal(-1.00000000000000))*(pz)*(x128)*(x146)))+(((IkReal(-1.00000000000000))*(sj1)*(x142)*(x145)))+(((IkReal(-0.405000000000000))*(r01)*(x145)))+(((IkReal(-1.00000000000000))*(r01)*(x140)*(x149)))+(((x126)*(x136)*(x141)))+(((IkReal(-0.100150000000000))*(cj1)*(x132)))+(((x139)*(x140)))+(((IkReal(0.100150000000000))*(x136)*(x145))));
evalcond[1]=((((r01)*(x131)*(x146)))+(((IkReal(-0.662000000000000))*(x138)))+(((IkReal(-1.00000000000000))*(cj1)*(x139)*(x150)))+(((IkReal(0.268110000000000))*(x126)*(x129)))+(((IkReal(-1.00000000000000))*(x130)*(x136)*(x146)))+(((IkReal(-1.00000000000000))*(x132)*(x144)))+(((IkReal(-1.00000000000000))*(r00)*(x144)*(x145)))+(((IkReal(-0.268110000000000))*(x125)*(x136)))+(((IkReal(0.0644940225000000))*(cj1)*(x139)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x130)*(x148)))+(((IkReal(-1.00000000000000))*(x128)*(x130)*(x149)))+(((IkReal(-0.263555977500000))*(sj1)*(x132)))+(((IkReal(0.263555977500000))*(x129)*(x149)))+(((IkReal(-0.0644940225000000))*(x128)*(x145)))+(((IkReal(-0.0662993000000000))*(x148)))+(((IkReal(-0.0662993000000000))*(x147)))+(((IkReal(0.810000000000000))*(x142)*(x146)))+(((x131)*(x132)))+(((IkReal(-1.00000000000000))*(x129)*(x133)*(x149)))+(((IkReal(-1.00000000000000))*(sj1)*(x132)*(x150)))+(((IkReal(-0.263555977500000))*(x151)))+(((cj1)*(pp)*(x139)))+(((IkReal(0.0644940225000000))*(r01)*(sj1)*(x146)))+(((pp)*(x151)))+(((IkReal(-0.268110000000000))*(x143)))+(((IkReal(-1.00000000000000))*(x129)*(x130)*(x145)))+(((IkReal(-1.00000000000000))*(x135)*(x147)))+(((IkReal(-1.00000000000000))*(x135)*(x148)))+(((IkReal(-1.00000000000000))*(x128)*(x133)*(x145))));
evalcond[2]=((((IkReal(0.405000000000000))*(x143)))+(((cj1)*(x125)*(x134)))+(((IkReal(-0.331000000000000))*(x129)*(x149)))+(x138)+(((IkReal(0.331000000000000))*(cj1)*(x139)))+(((IkReal(0.100150000000000))*(x147)))+(((IkReal(0.100150000000000))*(x148)))+(((IkReal(-0.331000000000000))*(x128)*(x145)))+(((x132)*(x137)))+(((IkReal(-0.405000000000000))*(x126)*(x129)))+(((IkReal(0.331000000000000))*(x151)))+(((r01)*(x137)*(x146))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x152=((cj1)*(sj2));
IkReal x153=((cj2)*(sj1));
IkReal x154=((cj0)*(py));
IkReal x155=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x153)*(x154)))+(((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x153)*(x155)))+(((x152)*(x154)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x152)*(x155)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x156=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((x153)*(x154)))+(((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x153)*(x155)))+(((x152)*(x154)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x152)*(x155)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x156));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x156))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x157=IKsin(j3);
IkReal x158=IKcos(j3);
IkReal x159=((cj2)*(sj1));
IkReal x160=((cj1)*(sj2));
IkReal x161=((sj2)*(x158));
IkReal x162=((px)*(sj0)*(x158));
IkReal x163=((IkReal(1.00000000000000))*(cj0)*(py)*(x158));
evalcond[0]=((((IkReal(-1.00000000000000))*(x159)*(x163)))+(((IkReal(-1.00000000000000))*(x160)*(x163)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x158)))+(((pz)*(sj1)*(x161)))+(((IkReal(0.100150000000000))*(x157)))+(((py)*(sj0)*(x157)))+(((x159)*(x162)))+(((x160)*(x162)))+(((cj0)*(px)*(x157)))+(((IkReal(0.405000000000000))*(x161))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x164=((IkReal(1.00000000000000))*(sj3));
IkReal x165=((cj2)*(cj3));
IkReal x166=((cj0)*(r01));
IkReal x167=((cj3)*(sj2));
IkReal x168=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((cj1)*(x166)*(x167)))+(((IkReal(-1.00000000000000))*(sj1)*(x165)*(x168)))+(((cj1)*(r02)*(x165)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x164)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x167)))+(((sj1)*(x165)*(x166)))+(((IkReal(-1.00000000000000))*(cj1)*(x167)*(x168)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x164))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x169=((sj1)*(sj3));
IkReal x170=((IkReal(1.00000000000000))*(cj0));
IkReal x171=((r02)*(sj2));
IkReal x172=((IkReal(1.00000000000000))*(cj1));
IkReal x173=((cj2)*(r02));
IkReal x174=((sj1)*(sj2));
IkReal x175=((cj2)*(r01));
IkReal x176=((r00)*(sj0));
IkReal x177=((cj2)*(x176));
IkReal x178=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((cj0)*(cj1)*(x175)))+(((IkReal(-1.00000000000000))*(sj1)*(x173)))+(((IkReal(-1.00000000000000))*(r01)*(x170)*(x174)))+(((x174)*(x176)))+(((IkReal(-1.00000000000000))*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(x172)*(x177))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(r00)*(x170)))+(((IkReal(-1.00000000000000))*(r01)*(x170)*(x178)))+(((IkReal(-1.00000000000000))*(x169)*(x170)*(x175)))+(((x176)*(x178)))+(((x169)*(x177)))+(((x169)*(x171)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x172)*(x173))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj0)*(cj1)*(x175)))+(((IkReal(-1.00000000000000))*(sj1)*(x173)))+(((IkReal(-1.00000000000000))*(r01)*(x170)*(x174)))+(((x174)*(x176)))+(((IkReal(-1.00000000000000))*(x171)*(x172)))+(((IkReal(-1.00000000000000))*(x172)*(x177)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(r00)*(x170)))+(((IkReal(-1.00000000000000))*(r01)*(x170)*(x178)))+(((IkReal(-1.00000000000000))*(x169)*(x170)*(x175)))+(((x176)*(x178)))+(((x169)*(x177)))+(((x169)*(x171)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x172)*(x173)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x179=((cj2)*(sj1));
IkReal x180=((cj1)*(sj2));
IkReal x181=((cj0)*(r01));
IkReal x182=((sj1)*(sj2));
IkReal x183=((r00)*(sj0));
IkReal x184=((cj1)*(cj2));
IkReal x185=((r02)*(sj3));
IkReal x186=((IkReal(1.00000000000000))*(sj3)*(x183));
evalcond[0]=((((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x181)*(x184)))+(((r02)*(x180)))+(((x183)*(x184)))+(((x181)*(x182)))+(((IkReal(-1.00000000000000))*(x182)*(x183)))+(((r02)*(x179))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x180)*(x186)))+(((x184)*(x185)))+(((sj5)*(IKcos(j4))))+(((sj3)*(x179)*(x181)))+(((IkReal(-1.00000000000000))*(x179)*(x186)))+(((sj3)*(x180)*(x181)))+(((cj0)*(cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x182)*(x185))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
IkReal x187=((IkReal(0.405000000000000))+(((pz)*(sj1))));
if( IKabs(x187) < IKFAST_ATAN2_MAGTHRESH && IKabs(((cj1)*(pz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x188=((IkReal(1.00000000000000))*(IKatan2(x187, ((cj1)*(pz)))));
if( ((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz)))))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.331000000000000))*(((IKabs(IKabs(IKsqrt((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.331000000000000))*(((IKabs(IKabs(IKsqrt((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x189=IKasin(((IkReal(0.331000000000000))*(((IKabs(IKabs(IKsqrt((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x187)*(x187))+((((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30))));
j2array[0]=((((IkReal(-1.00000000000000))*(x188)))+(((IkReal(-1.00000000000000))*(x189))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x188)))+(x189));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
IkReal x190=((IkReal(0.662000000000000))*(pz));
evalcond[0]=((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((cj1)*(x190)*(IKsin(j2))))+(pp)+(((((IkReal(0.268110000000000))+(((sj1)*(x190)))))*(IKcos(j2)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x191=(r00)*(r00);
IkReal x192=(cj1)*(cj1);
IkReal x193=(sj2)*(sj2);
IkReal x194=(cj2)*(cj2);
IkReal x195=(r01)*(r01);
IkReal x196=(sj1)*(sj1);
IkReal x197=((pz)*(sj1));
IkReal x198=((IkReal(0.331000000000000))*(pz));
IkReal x199=((IkReal(0.0100300225000000))*(cj2));
IkReal x200=((sj1)*(x191));
IkReal x201=((x192)*(x194));
IkReal x202=((x193)*(x195));
IkReal x203=((cj1)*(sj2)*(x191));
IkReal x204=((cj1)*(sj2)*(x195));
IkReal x205=((cj2)*(x204));
IkReal x206=((IkReal(0.405000000000000))*(cj2)*(pz)*(x192));
gconst3=IKsign(((((IkReal(-1.00000000000000))*(sj1)*(x195)*(x199)))+(((IkReal(-0.662000000000000))*(x197)*(x205)))+(((x195)*(x206)))+(((x191)*(x193)*(x196)*(x198)))+(((IkReal(0.134055000000000))*(sj1)*(x202)))+(((x191)*(x206)))+(((IkReal(-0.174055022500000))*(x203)))+(((IkReal(-0.174055022500000))*(x204)))+(((x195)*(x198)*(x201)))+(((IkReal(0.134055000000000))*(x193)*(x200)))+(((IkReal(-0.662000000000000))*(cj2)*(x197)*(x203)))+(((IkReal(-1.00000000000000))*(x199)*(x200)))+(((IkReal(-0.405000000000000))*(x197)*(x203)))+(((IkReal(-0.405000000000000))*(x197)*(x204)))+(((x191)*(x198)*(x201)))+(((IkReal(-0.134055000000000))*(x205)))+(((IkReal(-0.134055000000000))*(cj2)*(x203)))+(((x196)*(x198)*(x202)))));
IkReal x207=(r01)*(r01);
IkReal x208=(cj1)*(cj1);
IkReal x209=(r00)*(r00);
IkReal x210=(cj2)*(cj2);
IkReal x211=(sj1)*(sj1);
IkReal x212=(sj2)*(sj2);
IkReal x213=((IkReal(40.3787728292733))*(pz));
IkReal x214=((cj1)*(sj2));
IkReal x215=((IkReal(66.0018459579727))*(pz));
IkReal x216=((IkReal(33.0009229789863))*(pz));
IkReal x217=((sj1)*(x207));
IkReal x218=((IkReal(13.3653738064895))*(x212));
IkReal x219=((cj2)*(x209));
IkReal x220=((cj2)*(x207));
IkReal x221=((sj1)*(x209));
IkReal x222=((x211)*(x212));
IkReal x223=((x208)*(x210));
dummyeval[0]=((((IkReal(-13.3653738064895))*(x214)*(x219)))+(((IkReal(-17.3534029958557))*(x209)*(x214)))+(((IkReal(-1.00000000000000))*(sj1)*(x219)))+(((IkReal(-1.00000000000000))*(x213)*(x214)*(x221)))+(((x207)*(x216)*(x222)))+(((x207)*(x216)*(x223)))+(((IkReal(-13.3653738064895))*(x214)*(x220)))+(((IkReal(-1.00000000000000))*(sj1)*(x214)*(x215)*(x219)))+(((IkReal(-1.00000000000000))*(x213)*(x214)*(x217)))+(((x208)*(x213)*(x219)))+(((IkReal(-17.3534029958557))*(x207)*(x214)))+(((x218)*(x221)))+(((x209)*(x216)*(x223)))+(((x209)*(x216)*(x222)))+(((x217)*(x218)))+(((IkReal(-1.00000000000000))*(cj2)*(x214)*(x215)*(x217)))+(((x208)*(x213)*(x220)))+(((IkReal(-1.00000000000000))*(cj2)*(x217))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x224=(sj2)*(sj2);
IkReal x225=(sj1)*(sj1);
IkReal x226=(cj2)*(cj2);
IkReal x227=(cj1)*(cj1);
IkReal x228=(pz)*(pz);
IkReal x229=((r01)*(r02));
IkReal x230=((pz)*(sj1));
IkReal x231=((IkReal(0.100150000000000))*(cj1));
IkReal x232=((r00)*(r02));
IkReal x233=((IkReal(0.331000000000000))*(cj2));
IkReal x234=((sj1)*(sj2));
IkReal x235=((IkReal(0.0405607500000000))*(cj2));
IkReal x236=((pz)*(sj2));
IkReal x237=((cj1)*(cj2));
IkReal x238=((IkReal(0.100150000000000))*(cj2));
IkReal x239=((IkReal(1.00000000000000))*(x228));
IkReal x240=((IkReal(0.0331496500000000))*(x225));
IkReal x241=((IkReal(0.405000000000000))*(x225));
IkReal x242=((IkReal(0.0331496500000000))*(x227));
IkReal x243=((x232)*(x237));
IkReal x244=((x232)*(x236));
IkReal x245=((IkReal(0.134055000000000))*(cj2)*(x234));
IkReal x246=((IkReal(0.331000000000000))*(cj1)*(x224));
IkReal x247=((IkReal(0.134055000000000))*(cj1)*(x224));
IkReal x248=((IkReal(0.331000000000000))*(cj1)*(x226));
if( IKabs(((gconst3)*(((((x226)*(x229)*(x242)))+(((x226)*(x229)*(x240)))+(((IkReal(0.405000000000000))*(x244)))+(((x229)*(x231)*(x236)))+(((IkReal(-0.0100300225000000))*(x243)))+(((x225)*(x233)*(x244)))+(((x227)*(x229)*(x235)))+(((IkReal(-1.00000000000000))*(x230)*(x232)*(x248)))+(((x232)*(x247)))+(((x232)*(x245)))+(((x241)*(x244)))+(((IkReal(-0.405000000000000))*(x230)*(x243)))+(((x229)*(x230)*(x238)))+(((IkReal(-1.00000000000000))*(x239)*(x243)))+(((IkReal(0.174055022500000))*(x232)*(x234)))+(((x225)*(x229)*(x235)))+(((x230)*(x232)*(x246)))+(((x228)*(x232)*(x234)))+(((x224)*(x229)*(x242)))+(((x224)*(x229)*(x240)))+(((IkReal(-1.00000000000000))*(x227)*(x233)*(x244))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x225)*(x229)*(x233)*(x236)))+(((IkReal(0.0100300225000000))*(x229)*(x237)))+(((x227)*(x229)*(x233)*(x236)))+(((IkReal(0.405000000000000))*(x229)*(x230)*(x237)))+(((x228)*(x229)*(x237)))+(((IkReal(-1.00000000000000))*(x229)*(x245)))+(((IkReal(-1.00000000000000))*(x229)*(x247)))+(((IkReal(-0.174055022500000))*(x229)*(x234)))+(((x226)*(x232)*(x242)))+(((x226)*(x232)*(x240)))+(((x227)*(x232)*(x235)))+(((IkReal(-0.405000000000000))*(x229)*(x236)))+(((IkReal(-1.00000000000000))*(x229)*(x230)*(x246)))+(((x229)*(x230)*(x248)))+(((x230)*(x232)*(x238)))+(((x225)*(x232)*(x235)))+(((x231)*(x244)))+(((IkReal(-1.00000000000000))*(x229)*(x234)*(x239)))+(((IkReal(-1.00000000000000))*(x229)*(x236)*(x241)))+(((x224)*(x232)*(x240)))+(((x224)*(x232)*(x242))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst3)*(((((x226)*(x229)*(x242)))+(((x226)*(x229)*(x240)))+(((IkReal(0.405000000000000))*(x244)))+(((x229)*(x231)*(x236)))+(((IkReal(-0.0100300225000000))*(x243)))+(((x225)*(x233)*(x244)))+(((x227)*(x229)*(x235)))+(((IkReal(-1.00000000000000))*(x230)*(x232)*(x248)))+(((x232)*(x247)))+(((x232)*(x245)))+(((x241)*(x244)))+(((IkReal(-0.405000000000000))*(x230)*(x243)))+(((x229)*(x230)*(x238)))+(((IkReal(-1.00000000000000))*(x239)*(x243)))+(((IkReal(0.174055022500000))*(x232)*(x234)))+(((x225)*(x229)*(x235)))+(((x230)*(x232)*(x246)))+(((x228)*(x232)*(x234)))+(((x224)*(x229)*(x242)))+(((x224)*(x229)*(x240)))+(((IkReal(-1.00000000000000))*(x227)*(x233)*(x244)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x225)*(x229)*(x233)*(x236)))+(((IkReal(0.0100300225000000))*(x229)*(x237)))+(((x227)*(x229)*(x233)*(x236)))+(((IkReal(0.405000000000000))*(x229)*(x230)*(x237)))+(((x228)*(x229)*(x237)))+(((IkReal(-1.00000000000000))*(x229)*(x245)))+(((IkReal(-1.00000000000000))*(x229)*(x247)))+(((IkReal(-0.174055022500000))*(x229)*(x234)))+(((x226)*(x232)*(x242)))+(((x226)*(x232)*(x240)))+(((x227)*(x232)*(x235)))+(((IkReal(-0.405000000000000))*(x229)*(x236)))+(((IkReal(-1.00000000000000))*(x229)*(x230)*(x246)))+(((x229)*(x230)*(x248)))+(((x230)*(x232)*(x238)))+(((x225)*(x232)*(x235)))+(((x231)*(x244)))+(((IkReal(-1.00000000000000))*(x229)*(x234)*(x239)))+(((IkReal(-1.00000000000000))*(x229)*(x236)*(x241)))+(((x224)*(x232)*(x240)))+(((x224)*(x232)*(x242)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x249=IKsin(j0);
IkReal x250=IKcos(j0);
IkReal x251=(pz)*(pz);
IkReal x252=((r00)*(sj1));
IkReal x253=((cj1)*(r01));
IkReal x254=((IkReal(0.200300000000000))*(pz));
IkReal x255=((pp)*(sj1));
IkReal x256=((cj2)*(r02));
IkReal x257=((IkReal(1.00000000000000))*(pp));
IkReal x258=((IkReal(0.405000000000000))*(r00));
IkReal x259=((IkReal(0.0811215000000000))*(cj2));
IkReal x260=((cj1)*(r00));
IkReal x261=((IkReal(0.331000000000000))*(sj1));
IkReal x262=((pz)*(r02));
IkReal x263=((r02)*(sj2));
IkReal x264=((IkReal(0.100150000000000))*(sj1));
IkReal x265=((cj2)*(pz));
IkReal x266=((pz)*(r01));
IkReal x267=((r02)*(sj1));
IkReal x268=((IkReal(0.810000000000000))*(pz));
IkReal x269=((sj2)*(x249));
IkReal x270=((sj2)*(x250));
IkReal x271=((r00)*(x250));
IkReal x272=((r01)*(x249));
IkReal x273=((cj2)*(x250));
IkReal x274=((IkReal(2.00000000000000))*(x251));
IkReal x275=((cj2)*(x249)*(x260));
evalcond[0]=((((IkReal(0.100150000000000))*(x260)*(x269)))+(((IkReal(-1.00000000000000))*(x258)*(x270)))+(((x263)*(x264)))+(((x250)*(x260)*(x265)))+(((IkReal(0.100150000000000))*(cj2)*(x249)*(x252)))+(((IkReal(-0.100150000000000))*(cj1)*(x256)))+(((IkReal(-1.00000000000000))*(r01)*(x264)*(x273)))+(((IkReal(-0.100150000000000))*(x253)*(x270)))+(((IkReal(-0.405000000000000))*(r01)*(x269)))+(((IkReal(-1.00000000000000))*(pz)*(x252)*(x270)))+(((IkReal(-1.00000000000000))*(sj1)*(x266)*(x269)))+(((x249)*(x253)*(x265))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x263)*(x274)))+(((IkReal(-1.00000000000000))*(x253)*(x257)*(x273)))+(((IkReal(-1.00000000000000))*(x253)*(x254)*(x269)))+(((IkReal(-1.00000000000000))*(x259)*(x272)))+(((IkReal(-1.00000000000000))*(x259)*(x271)))+(((IkReal(-0.662000000000000))*(x262)))+(((IkReal(0.0644940225000000))*(cj1)*(x263)))+(((IkReal(-0.263555977500000))*(x275)))+(((IkReal(-0.268110000000000))*(x249)*(x260)))+(((cj1)*(pp)*(x263)))+(((IkReal(0.810000000000000))*(x266)*(x270)))+(((IkReal(-1.00000000000000))*(r00)*(x268)*(x269)))+(((IkReal(0.268110000000000))*(x250)*(x253)))+(((IkReal(-0.0662993000000000))*(x272)))+(((IkReal(-0.0662993000000000))*(x271)))+(((IkReal(-1.00000000000000))*(sj1)*(x256)*(x274)))+(((IkReal(0.0644940225000000))*(r01)*(sj1)*(x270)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x254)*(x272)))+(((IkReal(-0.268110000000000))*(x267)))+(((IkReal(-0.0644940225000000))*(x252)*(x269)))+(((IkReal(-0.263555977500000))*(sj1)*(x256)))+(((pp)*(x275)))+(((x255)*(x256)))+(((IkReal(-1.00000000000000))*(x256)*(x268)))+(((r01)*(x255)*(x270)))+(((IkReal(-1.00000000000000))*(x254)*(x260)*(x270)))+(((IkReal(-1.00000000000000))*(x252)*(x254)*(x273)))+(((IkReal(-1.00000000000000))*(x252)*(x257)*(x269)))+(((IkReal(0.263555977500000))*(x253)*(x273))));
evalcond[2]=((((IkReal(-0.405000000000000))*(x250)*(x253)))+(((IkReal(0.405000000000000))*(x267)))+(((IkReal(-0.331000000000000))*(x252)*(x269)))+(((IkReal(-0.331000000000000))*(x253)*(x273)))+(((IkReal(0.331000000000000))*(x275)))+(((cj1)*(x249)*(x258)))+(x262)+(((IkReal(0.100150000000000))*(x271)))+(((IkReal(0.100150000000000))*(x272)))+(((x256)*(x261)))+(((IkReal(0.331000000000000))*(cj1)*(x263)))+(((r01)*(x261)*(x270))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x276=((cj1)*(sj2));
IkReal x277=((cj2)*(sj1));
IkReal x278=((cj0)*(py));
IkReal x279=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x277)*(x279)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x277)*(x278)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x276)*(x278)))+(((IkReal(-1.00000000000000))*(x276)*(x279))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x280=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x277)*(x279)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x277)*(x278)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x276)*(x278)))+(((IkReal(-1.00000000000000))*(x276)*(x279)))));
j3array[0]=((IkReal(-1.00000000000000))*(x280));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x280))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x281=IKsin(j3);
IkReal x282=IKcos(j3);
IkReal x283=((cj2)*(sj1));
IkReal x284=((cj1)*(sj2));
IkReal x285=((sj2)*(x282));
IkReal x286=((px)*(sj0)*(x282));
IkReal x287=((IkReal(1.00000000000000))*(cj0)*(py)*(x282));
evalcond[0]=((((x284)*(x286)))+(((IkReal(0.405000000000000))*(x285)))+(((cj0)*(px)*(x281)))+(((IkReal(0.100150000000000))*(x281)))+(((IkReal(-1.00000000000000))*(x284)*(x287)))+(((pz)*(sj1)*(x285)))+(((py)*(sj0)*(x281)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x282)))+(((x283)*(x286)))+(((IkReal(-1.00000000000000))*(x283)*(x287))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x288=((IkReal(1.00000000000000))*(sj3));
IkReal x289=((cj2)*(cj3));
IkReal x290=((cj0)*(r01));
IkReal x291=((cj3)*(sj2));
IkReal x292=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x291)))+(((sj1)*(x289)*(x290)))+(((cj1)*(x290)*(x291)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x288)))+(((cj1)*(r02)*(x289)))+(((IkReal(-1.00000000000000))*(sj1)*(x289)*(x292)))+(((IkReal(-1.00000000000000))*(cj1)*(x291)*(x292)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x288))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x293=((sj1)*(sj3));
IkReal x294=((IkReal(1.00000000000000))*(cj0));
IkReal x295=((r02)*(sj2));
IkReal x296=((IkReal(1.00000000000000))*(cj1));
IkReal x297=((cj2)*(r02));
IkReal x298=((sj1)*(sj2));
IkReal x299=((cj2)*(r01));
IkReal x300=((r00)*(sj0));
IkReal x301=((cj2)*(x300));
IkReal x302=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x296)*(x301)))+(((IkReal(-1.00000000000000))*(sj1)*(x297)))+(((IkReal(-1.00000000000000))*(r01)*(x294)*(x298)))+(((x298)*(x300)))+(((IkReal(-1.00000000000000))*(x295)*(x296)))+(((cj0)*(cj1)*(x299))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(sj3)*(x296)*(x297)))+(((IkReal(-1.00000000000000))*(r01)*(x294)*(x302)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x293)*(x294)*(x299)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x294)))+(((x293)*(x301)))+(((x300)*(x302)))+(((x293)*(x295))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x296)*(x301)))+(((IkReal(-1.00000000000000))*(sj1)*(x297)))+(((IkReal(-1.00000000000000))*(r01)*(x294)*(x298)))+(((x298)*(x300)))+(((IkReal(-1.00000000000000))*(x295)*(x296)))+(((cj0)*(cj1)*(x299)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(sj3)*(x296)*(x297)))+(((IkReal(-1.00000000000000))*(r01)*(x294)*(x302)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x293)*(x294)*(x299)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x294)))+(((x293)*(x301)))+(((x300)*(x302)))+(((x293)*(x295)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x303=((cj2)*(sj1));
IkReal x304=((cj1)*(sj2));
IkReal x305=((cj0)*(r01));
IkReal x306=((sj1)*(sj2));
IkReal x307=((r00)*(sj0));
IkReal x308=((cj1)*(cj2));
IkReal x309=((r02)*(sj3));
IkReal x310=((IkReal(1.00000000000000))*(sj3)*(x307));
evalcond[0]=((((r02)*(x304)))+(((r02)*(x303)))+(((IkReal(-1.00000000000000))*(x305)*(x308)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x306)*(x307)))+(((x305)*(x306)))+(((x307)*(x308))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((sj3)*(x304)*(x305)))+(((IkReal(-1.00000000000000))*(x303)*(x310)))+(((sj3)*(x303)*(x305)))+(((sj5)*(IKcos(j4))))+(((x308)*(x309)))+(((IkReal(-1.00000000000000))*(x304)*(x310)))+(((IkReal(-1.00000000000000))*(x306)*(x309)))+(((cj0)*(cj3)*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
IkReal x311=((IkReal(0.405000000000000))+(((pz)*(sj1))));
if( IKabs(x311) < IKFAST_ATAN2_MAGTHRESH && IKabs(((cj1)*(pz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x312=((IkReal(1.00000000000000))*(IKatan2(x311, ((cj1)*(pz)))));
if( ((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz)))))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.331000000000000))*(((IKabs(IKabs(IKsqrt((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.331000000000000))*(((IKabs(IKabs(IKsqrt((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x313=IKasin(((IkReal(0.331000000000000))*(((IKabs(IKabs(IKsqrt((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x311)*(x311))+((((cj1)*(cj1))*((pz)*(pz))))))))):(IkReal)1.0e30))));
j2array[0]=((((IkReal(-1.00000000000000))*(x312)))+(((IkReal(-1.00000000000000))*(x313))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
j2array[1]=((IkReal(3.14159265358979))+(x313)+(((IkReal(-1.00000000000000))*(x312))));
sj2array[1]=IKsin(j2array[1]);
cj2array[1]=IKcos(j2array[1]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
if( j2array[1] > IKPI )
{
    j2array[1]-=IK2PI;
}
else if( j2array[1] < -IKPI )
{    j2array[1]+=IK2PI;
}
j2valid[1] = true;
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[1];
IkReal x314=IKcos(j2);
IkReal x315=((IkReal(0.662000000000000))*(pz));
evalcond[0]=((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.268110000000000))*(x314)))+(pp)+(((sj1)*(x314)*(x315)))+(((cj1)*(x315)*(IKsin(j2)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x316=(r00)*(r00);
IkReal x317=(cj1)*(cj1);
IkReal x318=(sj2)*(sj2);
IkReal x319=(cj2)*(cj2);
IkReal x320=(r01)*(r01);
IkReal x321=(sj1)*(sj1);
IkReal x322=((pz)*(sj1));
IkReal x323=((IkReal(0.331000000000000))*(pz));
IkReal x324=((IkReal(0.0100300225000000))*(cj2));
IkReal x325=((sj1)*(x316));
IkReal x326=((x317)*(x319));
IkReal x327=((x318)*(x320));
IkReal x328=((cj1)*(sj2)*(x316));
IkReal x329=((cj1)*(sj2)*(x320));
IkReal x330=((cj2)*(x329));
IkReal x331=((IkReal(0.405000000000000))*(cj2)*(pz)*(x317));
gconst3=IKsign(((((x316)*(x323)*(x326)))+(((IkReal(-0.405000000000000))*(x322)*(x328)))+(((IkReal(-0.405000000000000))*(x322)*(x329)))+(((IkReal(-0.662000000000000))*(x322)*(x330)))+(((IkReal(0.134055000000000))*(sj1)*(x327)))+(((IkReal(-0.134055000000000))*(cj2)*(x328)))+(((x320)*(x323)*(x326)))+(((x316)*(x318)*(x321)*(x323)))+(((x316)*(x331)))+(((IkReal(-0.662000000000000))*(cj2)*(x322)*(x328)))+(((x320)*(x331)))+(((x321)*(x323)*(x327)))+(((IkReal(-0.134055000000000))*(x330)))+(((IkReal(-1.00000000000000))*(x324)*(x325)))+(((IkReal(0.134055000000000))*(x318)*(x325)))+(((IkReal(-1.00000000000000))*(sj1)*(x320)*(x324)))+(((IkReal(-0.174055022500000))*(x329)))+(((IkReal(-0.174055022500000))*(x328)))));
IkReal x332=(r01)*(r01);
IkReal x333=(cj1)*(cj1);
IkReal x334=(r00)*(r00);
IkReal x335=(cj2)*(cj2);
IkReal x336=(sj1)*(sj1);
IkReal x337=(sj2)*(sj2);
IkReal x338=((IkReal(40.3787728292733))*(pz));
IkReal x339=((cj1)*(sj2));
IkReal x340=((IkReal(66.0018459579727))*(pz));
IkReal x341=((IkReal(33.0009229789863))*(pz));
IkReal x342=((sj1)*(x332));
IkReal x343=((IkReal(13.3653738064895))*(x337));
IkReal x344=((cj2)*(x334));
IkReal x345=((cj2)*(x332));
IkReal x346=((sj1)*(x334));
IkReal x347=((x336)*(x337));
IkReal x348=((x333)*(x335));
dummyeval[0]=((((IkReal(-1.00000000000000))*(sj1)*(x344)))+(((IkReal(-1.00000000000000))*(cj2)*(x342)))+(((IkReal(-17.3534029958557))*(x332)*(x339)))+(((IkReal(-13.3653738064895))*(x339)*(x344)))+(((IkReal(-13.3653738064895))*(x339)*(x345)))+(((IkReal(-1.00000000000000))*(x338)*(x339)*(x342)))+(((IkReal(-1.00000000000000))*(x338)*(x339)*(x346)))+(((IkReal(-1.00000000000000))*(cj2)*(x339)*(x340)*(x342)))+(((x334)*(x341)*(x348)))+(((x334)*(x341)*(x347)))+(((IkReal(-1.00000000000000))*(sj1)*(x339)*(x340)*(x344)))+(((x333)*(x338)*(x345)))+(((x333)*(x338)*(x344)))+(((x332)*(x341)*(x347)))+(((x332)*(x341)*(x348)))+(((x343)*(x346)))+(((IkReal(-17.3534029958557))*(x334)*(x339)))+(((x342)*(x343))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x349=(sj2)*(sj2);
IkReal x350=(sj1)*(sj1);
IkReal x351=(cj2)*(cj2);
IkReal x352=(cj1)*(cj1);
IkReal x353=(pz)*(pz);
IkReal x354=((r01)*(r02));
IkReal x355=((pz)*(sj1));
IkReal x356=((IkReal(0.100150000000000))*(cj1));
IkReal x357=((r00)*(r02));
IkReal x358=((IkReal(0.331000000000000))*(cj2));
IkReal x359=((sj1)*(sj2));
IkReal x360=((IkReal(0.0405607500000000))*(cj2));
IkReal x361=((pz)*(sj2));
IkReal x362=((cj1)*(cj2));
IkReal x363=((IkReal(0.100150000000000))*(cj2));
IkReal x364=((IkReal(1.00000000000000))*(x353));
IkReal x365=((IkReal(0.0331496500000000))*(x350));
IkReal x366=((IkReal(0.405000000000000))*(x350));
IkReal x367=((IkReal(0.0331496500000000))*(x352));
IkReal x368=((x357)*(x362));
IkReal x369=((x357)*(x361));
IkReal x370=((IkReal(0.134055000000000))*(cj2)*(x359));
IkReal x371=((IkReal(0.331000000000000))*(cj1)*(x349));
IkReal x372=((IkReal(0.134055000000000))*(cj1)*(x349));
IkReal x373=((IkReal(0.331000000000000))*(cj1)*(x351));
if( IKabs(((gconst3)*(((((x349)*(x354)*(x367)))+(((x349)*(x354)*(x365)))+(((IkReal(-1.00000000000000))*(x352)*(x358)*(x369)))+(((x350)*(x354)*(x360)))+(((IkReal(-0.405000000000000))*(x355)*(x368)))+(((x354)*(x355)*(x363)))+(((x353)*(x357)*(x359)))+(((x366)*(x369)))+(((IkReal(0.405000000000000))*(x369)))+(((x350)*(x358)*(x369)))+(((IkReal(0.174055022500000))*(x357)*(x359)))+(((IkReal(-1.00000000000000))*(x355)*(x357)*(x373)))+(((IkReal(-1.00000000000000))*(x364)*(x368)))+(((IkReal(-0.0100300225000000))*(x368)))+(((x351)*(x354)*(x365)))+(((x351)*(x354)*(x367)))+(((x355)*(x357)*(x371)))+(((x357)*(x370)))+(((x357)*(x372)))+(((x354)*(x356)*(x361)))+(((x352)*(x354)*(x360))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(0.405000000000000))*(x354)*(x355)*(x362)))+(((IkReal(-0.174055022500000))*(x354)*(x359)))+(((x354)*(x355)*(x373)))+(((IkReal(-1.00000000000000))*(x354)*(x372)))+(((IkReal(-1.00000000000000))*(x354)*(x370)))+(((x350)*(x357)*(x360)))+(((IkReal(-1.00000000000000))*(x354)*(x361)*(x366)))+(((IkReal(-0.405000000000000))*(x354)*(x361)))+(((x356)*(x369)))+(((x351)*(x357)*(x365)))+(((x351)*(x357)*(x367)))+(((IkReal(0.0100300225000000))*(x354)*(x362)))+(((IkReal(-1.00000000000000))*(x354)*(x359)*(x364)))+(((x352)*(x357)*(x360)))+(((IkReal(-1.00000000000000))*(x350)*(x354)*(x358)*(x361)))+(((x352)*(x354)*(x358)*(x361)))+(((x349)*(x357)*(x365)))+(((x349)*(x357)*(x367)))+(((IkReal(-1.00000000000000))*(x354)*(x355)*(x371)))+(((x355)*(x357)*(x363)))+(((x353)*(x354)*(x362))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst3)*(((((x349)*(x354)*(x367)))+(((x349)*(x354)*(x365)))+(((IkReal(-1.00000000000000))*(x352)*(x358)*(x369)))+(((x350)*(x354)*(x360)))+(((IkReal(-0.405000000000000))*(x355)*(x368)))+(((x354)*(x355)*(x363)))+(((x353)*(x357)*(x359)))+(((x366)*(x369)))+(((IkReal(0.405000000000000))*(x369)))+(((x350)*(x358)*(x369)))+(((IkReal(0.174055022500000))*(x357)*(x359)))+(((IkReal(-1.00000000000000))*(x355)*(x357)*(x373)))+(((IkReal(-1.00000000000000))*(x364)*(x368)))+(((IkReal(-0.0100300225000000))*(x368)))+(((x351)*(x354)*(x365)))+(((x351)*(x354)*(x367)))+(((x355)*(x357)*(x371)))+(((x357)*(x370)))+(((x357)*(x372)))+(((x354)*(x356)*(x361)))+(((x352)*(x354)*(x360)))))), ((gconst3)*(((((IkReal(0.405000000000000))*(x354)*(x355)*(x362)))+(((IkReal(-0.174055022500000))*(x354)*(x359)))+(((x354)*(x355)*(x373)))+(((IkReal(-1.00000000000000))*(x354)*(x372)))+(((IkReal(-1.00000000000000))*(x354)*(x370)))+(((x350)*(x357)*(x360)))+(((IkReal(-1.00000000000000))*(x354)*(x361)*(x366)))+(((IkReal(-0.405000000000000))*(x354)*(x361)))+(((x356)*(x369)))+(((x351)*(x357)*(x365)))+(((x351)*(x357)*(x367)))+(((IkReal(0.0100300225000000))*(x354)*(x362)))+(((IkReal(-1.00000000000000))*(x354)*(x359)*(x364)))+(((x352)*(x357)*(x360)))+(((IkReal(-1.00000000000000))*(x350)*(x354)*(x358)*(x361)))+(((x352)*(x354)*(x358)*(x361)))+(((x349)*(x357)*(x365)))+(((x349)*(x357)*(x367)))+(((IkReal(-1.00000000000000))*(x354)*(x355)*(x371)))+(((x355)*(x357)*(x363)))+(((x353)*(x354)*(x362)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x374=IKsin(j0);
IkReal x375=IKcos(j0);
IkReal x376=(pz)*(pz);
IkReal x377=((r00)*(sj1));
IkReal x378=((cj1)*(r01));
IkReal x379=((IkReal(0.200300000000000))*(pz));
IkReal x380=((pp)*(sj1));
IkReal x381=((cj2)*(r02));
IkReal x382=((IkReal(1.00000000000000))*(pp));
IkReal x383=((IkReal(0.405000000000000))*(r00));
IkReal x384=((IkReal(0.0811215000000000))*(cj2));
IkReal x385=((cj1)*(r00));
IkReal x386=((IkReal(0.331000000000000))*(sj1));
IkReal x387=((pz)*(r02));
IkReal x388=((r02)*(sj2));
IkReal x389=((IkReal(0.100150000000000))*(sj1));
IkReal x390=((cj2)*(pz));
IkReal x391=((pz)*(r01));
IkReal x392=((r02)*(sj1));
IkReal x393=((IkReal(0.810000000000000))*(pz));
IkReal x394=((sj2)*(x374));
IkReal x395=((sj2)*(x375));
IkReal x396=((r00)*(x375));
IkReal x397=((r01)*(x374));
IkReal x398=((cj2)*(x375));
IkReal x399=((IkReal(2.00000000000000))*(x376));
IkReal x400=((cj2)*(x374)*(x385));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj1)*(x391)*(x394)))+(((IkReal(-0.405000000000000))*(r01)*(x394)))+(((IkReal(-1.00000000000000))*(r01)*(x389)*(x398)))+(((x375)*(x385)*(x390)))+(((IkReal(0.100150000000000))*(x385)*(x394)))+(((IkReal(0.100150000000000))*(cj2)*(x374)*(x377)))+(((x388)*(x389)))+(((IkReal(-1.00000000000000))*(x383)*(x395)))+(((IkReal(-1.00000000000000))*(pz)*(x377)*(x395)))+(((IkReal(-0.100150000000000))*(cj1)*(x381)))+(((x374)*(x378)*(x390)))+(((IkReal(-0.100150000000000))*(x378)*(x395))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x384)*(x397)))+(((IkReal(-1.00000000000000))*(x384)*(x396)))+(((IkReal(-1.00000000000000))*(x377)*(x382)*(x394)))+(((IkReal(0.263555977500000))*(x378)*(x398)))+(((IkReal(0.0644940225000000))*(cj1)*(x388)))+(((IkReal(0.0644940225000000))*(r01)*(sj1)*(x395)))+(((IkReal(-1.00000000000000))*(x378)*(x382)*(x398)))+(((IkReal(-0.662000000000000))*(x387)))+(((IkReal(0.810000000000000))*(x391)*(x395)))+(((IkReal(-0.0662993000000000))*(x396)))+(((IkReal(-0.0662993000000000))*(x397)))+(((IkReal(-1.00000000000000))*(x377)*(x379)*(x398)))+(((r01)*(x380)*(x395)))+(((IkReal(0.268110000000000))*(x375)*(x378)))+(((IkReal(-1.00000000000000))*(cj1)*(x388)*(x399)))+(((IkReal(-0.268110000000000))*(x374)*(x385)))+(((IkReal(-1.00000000000000))*(x379)*(x385)*(x395)))+(((pp)*(x400)))+(((IkReal(-1.00000000000000))*(x378)*(x379)*(x394)))+(((cj1)*(pp)*(x388)))+(((IkReal(-0.263555977500000))*(x400)))+(((IkReal(-1.00000000000000))*(sj1)*(x381)*(x399)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x379)*(x397)))+(((IkReal(-1.00000000000000))*(x381)*(x393)))+(((IkReal(-1.00000000000000))*(r00)*(x393)*(x394)))+(((IkReal(-0.0644940225000000))*(x377)*(x394)))+(((IkReal(-0.263555977500000))*(sj1)*(x381)))+(((x380)*(x381)))+(((IkReal(-0.268110000000000))*(x392))));
evalcond[2]=((((IkReal(-0.405000000000000))*(x375)*(x378)))+(((IkReal(0.331000000000000))*(cj1)*(x388)))+(((cj1)*(x374)*(x383)))+(((IkReal(0.331000000000000))*(x400)))+(((IkReal(-0.331000000000000))*(x377)*(x394)))+(x387)+(((IkReal(0.100150000000000))*(x397)))+(((IkReal(0.100150000000000))*(x396)))+(((IkReal(-0.331000000000000))*(x378)*(x398)))+(((r01)*(x386)*(x395)))+(((IkReal(0.405000000000000))*(x392)))+(((x381)*(x386))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x401=((cj1)*(sj2));
IkReal x402=((cj2)*(sj1));
IkReal x403=((cj0)*(py));
IkReal x404=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x402)*(x404)))+(((IkReal(-1.00000000000000))*(x401)*(x404)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x402)*(x403)))+(((x401)*(x403))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x405=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x402)*(x404)))+(((IkReal(-1.00000000000000))*(x401)*(x404)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x402)*(x403)))+(((x401)*(x403)))));
j3array[0]=((IkReal(-1.00000000000000))*(x405));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x405))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x406=IKsin(j3);
IkReal x407=IKcos(j3);
IkReal x408=((cj2)*(sj1));
IkReal x409=((cj1)*(sj2));
IkReal x410=((sj2)*(x407));
IkReal x411=((px)*(sj0)*(x407));
IkReal x412=((IkReal(1.00000000000000))*(cj0)*(py)*(x407));
evalcond[0]=((((IkReal(0.405000000000000))*(x410)))+(((x408)*(x411)))+(((x409)*(x411)))+(((pz)*(sj1)*(x410)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x407)))+(((cj0)*(px)*(x406)))+(((IkReal(0.100150000000000))*(x406)))+(((py)*(sj0)*(x406)))+(((IkReal(-1.00000000000000))*(x408)*(x412)))+(((IkReal(-1.00000000000000))*(x409)*(x412))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x413=((IkReal(1.00000000000000))*(sj3));
IkReal x414=((cj2)*(cj3));
IkReal x415=((cj0)*(r01));
IkReal x416=((cj3)*(sj2));
IkReal x417=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((IkReal(-1.00000000000000))*(r01)*(sj0)*(x413)))+(((cj1)*(x415)*(x416)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x416)))+(((cj1)*(r02)*(x414)))+(((IkReal(-1.00000000000000))*(cj1)*(x416)*(x417)))+(((sj1)*(x414)*(x415)))+(((IkReal(-1.00000000000000))*(sj1)*(x414)*(x417)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x413))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x418=((sj1)*(sj3));
IkReal x419=((IkReal(1.00000000000000))*(cj0));
IkReal x420=((r02)*(sj2));
IkReal x421=((IkReal(1.00000000000000))*(cj1));
IkReal x422=((cj2)*(r02));
IkReal x423=((sj1)*(sj2));
IkReal x424=((cj2)*(r01));
IkReal x425=((r00)*(sj0));
IkReal x426=((cj2)*(x425));
IkReal x427=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x421)*(x426)))+(((IkReal(-1.00000000000000))*(x420)*(x421)))+(((cj0)*(cj1)*(x424)))+(((IkReal(-1.00000000000000))*(r01)*(x419)*(x423)))+(((IkReal(-1.00000000000000))*(sj1)*(x422)))+(((x423)*(x425))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x425)*(x427)))+(((IkReal(-1.00000000000000))*(sj3)*(x421)*(x422)))+(((IkReal(-1.00000000000000))*(x418)*(x419)*(x424)))+(((IkReal(-1.00000000000000))*(r01)*(x419)*(x427)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x418)*(x426)))+(((x418)*(x420)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x419))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x421)*(x426)))+(((IkReal(-1.00000000000000))*(x420)*(x421)))+(((cj0)*(cj1)*(x424)))+(((IkReal(-1.00000000000000))*(r01)*(x419)*(x423)))+(((IkReal(-1.00000000000000))*(sj1)*(x422)))+(((x423)*(x425)))))), ((gconst4)*(((((x425)*(x427)))+(((IkReal(-1.00000000000000))*(sj3)*(x421)*(x422)))+(((IkReal(-1.00000000000000))*(x418)*(x419)*(x424)))+(((IkReal(-1.00000000000000))*(r01)*(x419)*(x427)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x418)*(x426)))+(((x418)*(x420)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x419)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x428=((cj2)*(sj1));
IkReal x429=((cj1)*(sj2));
IkReal x430=((cj0)*(r01));
IkReal x431=((sj1)*(sj2));
IkReal x432=((r00)*(sj0));
IkReal x433=((cj1)*(cj2));
IkReal x434=((r02)*(sj3));
IkReal x435=((IkReal(1.00000000000000))*(sj3)*(x432));
evalcond[0]=((((IkReal(-1.00000000000000))*(x431)*(x432)))+(((r02)*(x429)))+(((r02)*(x428)))+(((sj5)*(IKsin(j4))))+(((x430)*(x431)))+(((x432)*(x433)))+(((IkReal(-1.00000000000000))*(x430)*(x433))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x431)*(x434)))+(((cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x429)*(x435)))+(((sj5)*(IKcos(j4))))+(((x433)*(x434)))+(((IkReal(-1.00000000000000))*(x428)*(x435)))+(((sj3)*(x428)*(x430)))+(((cj0)*(cj3)*(r00)))+(((sj3)*(x429)*(x430))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x436=(py)*(py)*(py)*(py);
IkReal x437=(sj1)*(sj1);
IkReal x438=(py)*(py);
IkReal x439=((py)*(sj1));
IkReal x440=((sj1)*(x438));
IkReal x441=((x436)*(x437));
if( IKabs(((IkReal(0.810000000000000))*(cj1)*(x440))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.200300000000000))*(x440))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x442=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.810000000000000))*(cj1)*(x440)), ((IkReal(-0.200300000000000))*(x440)))));
if( (((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441))))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441)))))))):(IkReal)1.0e30))*(((((IkReal(-0.810000000000000))*(py)*(pz)*(x437)))+(((IkReal(-1.00000000000000))*(pp)*(x439)))+(((IkReal(-0.0644940225000000))*(x439))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441)))))))):(IkReal)1.0e30))*(((((IkReal(-0.810000000000000))*(py)*(pz)*(x437)))+(((IkReal(-1.00000000000000))*(pp)*(x439)))+(((IkReal(-0.0644940225000000))*(x439))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x443=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.656100000000000))*(x441)*((cj1)*(cj1))))+(((IkReal(0.0401200900000000))*(x441)))))))):(IkReal)1.0e30))*(((((IkReal(-0.810000000000000))*(py)*(pz)*(x437)))+(((IkReal(-1.00000000000000))*(pp)*(x439)))+(((IkReal(-0.0644940225000000))*(x439)))))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x442)))+(((IkReal(-1.00000000000000))*(x443))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x442)))+(((IkReal(1.00000000000000))*(x443))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[1];
IkReal x444=(py)*(py);
IkReal x445=((cj1)*(py));
evalcond[0]=((((IkReal(0.200300000000000))*(cj1)*(x444)*(IKsin(j0))))+(((IkReal(-0.810000000000000))*(x444)*((cj1)*(cj1))*(IKcos(j0))))+(((pp)*(x445)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x445)))+(((IkReal(0.0644940225000000))*(x445))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x446=(sj1)*(sj1);
IkReal x447=(cj1)*(cj1);
IkReal x448=((cj0)*(r01));
IkReal x449=((IkReal(134.055000000000))*(sj1));
IkReal x450=((r00)*(sj0));
IkReal x451=((IkReal(331.000000000000))*(x446));
IkReal x452=((cj0)*(py)*(r02));
IkReal x453=((IkReal(331.000000000000))*(x447));
gconst2=IKsign(((((IkReal(-1.00000000000000))*(pz)*(x450)*(x453)))+(((IkReal(-1.00000000000000))*(pz)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(x452)*(x453)))+(((IkReal(-1.00000000000000))*(x451)*(x452)))+(((x448)*(x449)))+(((pz)*(x448)*(x453)))+(((pz)*(x448)*(x451)))+(((IkReal(134.055000000000))*(cj1)*(r02)))+(((IkReal(-1.00000000000000))*(x449)*(x450)))));
IkReal x454=(sj1)*(sj1);
IkReal x455=(cj1)*(cj1);
IkReal x456=((IkReal(2.46913580246914))*(pz));
IkReal x457=((r00)*(sj0));
IkReal x458=((cj0)*(r01));
IkReal x459=((IkReal(2.46913580246914))*(cj0)*(py)*(r02));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x455)*(x456)*(x457)))+(((x455)*(x456)*(x458)))+(((cj1)*(r02)))+(((x454)*(x456)*(x458)))+(((IkReal(-1.00000000000000))*(x454)*(x459)))+(((IkReal(-1.00000000000000))*(x454)*(x456)*(x457)))+(((sj1)*(x458)))+(((IkReal(-1.00000000000000))*(x455)*(x459)))+(((IkReal(-1.00000000000000))*(sj1)*(x457))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x460=(cj0)*(cj0);
IkReal x461=(cj1)*(cj1);
IkReal x462=(sj1)*(sj1);
IkReal x463=(py)*(py);
IkReal x464=(pz)*(pz);
IkReal x465=((IkReal(100.150000000000))*(cj0));
IkReal x466=((cj0)*(sj1));
IkReal x467=((py)*(r00));
IkReal x468=((IkReal(1000.00000000000))*(py));
IkReal x469=((r00)*(sj0));
IkReal x470=((pz)*(r02));
IkReal x471=((IkReal(100.150000000000))*(sj0));
IkReal x472=((py)*(r01));
IkReal x473=((cj1)*(r02));
IkReal x474=((IkReal(54.4640000000000))*(cj1));
IkReal x475=((r02)*(sj1));
IkReal x476=((cj0)*(cj1));
IkReal x477=((pz)*(r00)*(sj1));
IkReal x478=((IkReal(405.000000000000))*(cj1)*(sj0));
IkReal x479=((cj1)*(pz)*(r01));
IkReal x480=((py)*(x460));
IkReal x481=((IkReal(1000.00000000000))*(x464));
IkReal x482=((IkReal(100.150000000000))*(x460));
IkReal x483=((IkReal(405.000000000000))*(x462));
IkReal x484=((pz)*(r01)*(sj1));
IkReal x485=((IkReal(405.000000000000))*(x461));
IkReal x486=((IkReal(405.000000000000))*(cj1)*(sj1));
IkReal x487=((r01)*(x485));
IkReal x488=((IkReal(1000.00000000000))*(r01)*(x463));
if( IKabs(((gconst2)*(((((x468)*(x470)*(x476)))+(((IkReal(-405.000000000000))*(x472)))+(((IkReal(-405.000000000000))*(x470)))+(((cj0)*(sj0)*(x467)*(x485)))+(((IkReal(-1.00000000000000))*(x475)*(x481)))+(((x476)*(x488)))+(((IkReal(-1.00000000000000))*(x470)*(x483)))+(((IkReal(405.000000000000))*(x466)*(x479)))+(((cj0)*(r01)*(x474)))+(((IkReal(-1.00000000000000))*(x469)*(x474)))+(((cj1)*(sj0)*(x465)*(x472)))+(((IkReal(405.000000000000))*(py)*(x466)*(x473)))+(((IkReal(-1.00000000000000))*(pz)*(x469)*(x486)))+(((cj1)*(x467)*(x482)))+(((IkReal(-1.00000000000000))*(x468)*(x484)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-1.00000000000000))*(x471)*(x484)))+(((IkReal(-54.4640000000000))*(x475)))+(((IkReal(-1.00000000000000))*(x460)*(x472)*(x485)))+(((IkReal(-1.00000000000000))*(x465)*(x477)))+(((IkReal(-40.5607500000000))*(r01)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((x471)*(x479)))+(((IkReal(109.561000000000))*(sj1)*(x469)))+(((x466)*(x468)*(x470)))+(((IkReal(-1.00000000000000))*(cj0)*(pz)*(x487)))+(((x466)*(x467)*(x478)))+(((x470)*(x486)))+(((IkReal(-109.561000000000))*(x473)))+(((sj1)*(x467)*(x482)))+(((sj0)*(sj1)*(x465)*(x472)))+(((cj1)*(pz)*(r00)*(x465)))+(((pz)*(x469)*(x485)))+(((cj0)*(py)*(r02)*(x483)))+(((x466)*(x488)))+(((IkReal(-109.561000000000))*(r01)*(x466)))+(((IkReal(-1.00000000000000))*(x460)*(x472)*(x486)))+(((x468)*(x479)))+(((x473)*(x481))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((x468)*(x470)*(x476)))+(((IkReal(-405.000000000000))*(x472)))+(((IkReal(-405.000000000000))*(x470)))+(((cj0)*(sj0)*(x467)*(x485)))+(((IkReal(-1.00000000000000))*(x475)*(x481)))+(((x476)*(x488)))+(((IkReal(-1.00000000000000))*(x470)*(x483)))+(((IkReal(405.000000000000))*(x466)*(x479)))+(((cj0)*(r01)*(x474)))+(((IkReal(-1.00000000000000))*(x469)*(x474)))+(((cj1)*(sj0)*(x465)*(x472)))+(((IkReal(405.000000000000))*(py)*(x466)*(x473)))+(((IkReal(-1.00000000000000))*(pz)*(x469)*(x486)))+(((cj1)*(x467)*(x482)))+(((IkReal(-1.00000000000000))*(x468)*(x484)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-1.00000000000000))*(x471)*(x484)))+(((IkReal(-54.4640000000000))*(x475)))+(((IkReal(-1.00000000000000))*(x460)*(x472)*(x485)))+(((IkReal(-1.00000000000000))*(x465)*(x477)))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))))), ((gconst2)*(((((x471)*(x479)))+(((IkReal(109.561000000000))*(sj1)*(x469)))+(((x466)*(x468)*(x470)))+(((IkReal(-1.00000000000000))*(cj0)*(pz)*(x487)))+(((x466)*(x467)*(x478)))+(((x470)*(x486)))+(((IkReal(-109.561000000000))*(x473)))+(((sj1)*(x467)*(x482)))+(((sj0)*(sj1)*(x465)*(x472)))+(((cj1)*(pz)*(r00)*(x465)))+(((pz)*(x469)*(x485)))+(((cj0)*(py)*(r02)*(x483)))+(((x466)*(x488)))+(((IkReal(-109.561000000000))*(r01)*(x466)))+(((IkReal(-1.00000000000000))*(x460)*(x472)*(x486)))+(((x468)*(x479)))+(((x473)*(x481)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x489=IKsin(j2);
IkReal x490=IKcos(j2);
IkReal x491=(py)*(py);
IkReal x492=(pz)*(pz);
IkReal x493=((r00)*(sj0));
IkReal x494=((IkReal(0.405000000000000))*(cj1));
IkReal x495=((py)*(r02));
IkReal x496=((r02)*(sj1));
IkReal x497=((cj0)*(py));
IkReal x498=((IkReal(2.00000000000000))*(r01));
IkReal x499=((r01)*(sj0));
IkReal x500=((cj0)*(r00));
IkReal x501=((cj0)*(r01));
IkReal x502=((IkReal(0.268110000000000))*(cj1));
IkReal x503=((IkReal(0.200300000000000))*(sj1));
IkReal x504=((IkReal(0.662000000000000))*(pz));
IkReal x505=((IkReal(1.00000000000000))*(pz));
IkReal x506=((IkReal(0.100150000000000))*(r02));
IkReal x507=((IkReal(0.200300000000000))*(py));
IkReal x508=((IkReal(0.200300000000000))*(pz));
IkReal x509=((py)*(r01));
IkReal x510=((IkReal(2.00000000000000))*(cj0));
IkReal x511=((sj1)*(x489));
IkReal x512=((pp)*(x490));
IkReal x513=((pz)*(x490));
IkReal x514=((cj0)*(x491));
IkReal x515=((IkReal(0.331000000000000))*(x490));
IkReal x516=((cj1)*(x490));
IkReal x517=((IkReal(0.405000000000000))*(x489));
IkReal x518=((IkReal(0.0811215000000000))*(x490));
IkReal x519=((IkReal(0.810000000000000))*(x489));
IkReal x520=((cj1)*(x489));
IkReal x521=((IkReal(2.00000000000000))*(x492));
IkReal x522=((r02)*(x520));
IkReal x523=((py)*(sj1)*(x490));
IkReal x524=((IkReal(0.100150000000000))*(sj1)*(x490));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(0.405000000000000))*(x490)))+(((pz)*(x520)))+(((IkReal(-1.00000000000000))*(x497)*(x516)))+(((x497)*(x511)))+(((sj1)*(x513))));
evalcond[1]=((((cj1)*(x499)*(x513)))+(((py)*(r00)*(x520)))+(((sj0)*(x495)*(x511)))+(((IkReal(0.100150000000000))*(x489)*(x496)))+(((IkReal(-1.00000000000000))*(x500)*(x517)))+(((IkReal(-1.00000000000000))*(x501)*(x524)))+(((IkReal(-0.100150000000000))*(x501)*(x520)))+(((IkReal(-1.00000000000000))*(x499)*(x517)))+(((IkReal(-1.00000000000000))*(x500)*(x505)*(x511)))+(((x493)*(x524)))+(((IkReal(-1.00000000000000))*(x499)*(x505)*(x511)))+(((IkReal(-1.00000000000000))*(x506)*(x516)))+(((IkReal(-1.00000000000000))*(sj0)*(x495)*(x516)))+(((cj1)*(x500)*(x513)))+(((IkReal(0.100150000000000))*(x493)*(x520)))+(((r00)*(x523))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x499)*(x503)*(x513)))+(((IkReal(0.0644940225000000))*(x522)))+(((IkReal(-0.0644940225000000))*(x493)*(x511)))+(((x498)*(x514)*(x516)))+(((x496)*(x512)))+(((IkReal(-1.00000000000000))*(pp)*(x493)*(x511)))+(((pp)*(x522)))+(((IkReal(-0.263555977500000))*(x493)*(x516)))+(((IkReal(-0.662000000000000))*(x509)))+(((IkReal(-0.810000000000000))*(x490)*(x509)))+(((IkReal(-0.268110000000000))*(x496)))+(((IkReal(-1.00000000000000))*(x521)*(x522)))+(((r00)*(x507)*(x516)))+(((IkReal(-0.0662993000000000))*(x499)))+(((IkReal(-1.00000000000000))*(x499)*(x508)*(x520)))+(((IkReal(-1.00000000000000))*(x500)*(x503)*(x513)))+(((IkReal(-1.00000000000000))*(cj0)*(x495)*(x519)))+(((IkReal(-1.00000000000000))*(x500)*(x508)*(x520)))+(((IkReal(-1.00000000000000))*(x500)*(x518)))+(((IkReal(0.0644940225000000))*(x501)*(x511)))+(((IkReal(-1.00000000000000))*(py)*(r00)*(x489)*(x503)))+(((x501)*(x502)))+(((IkReal(-1.00000000000000))*(x498)*(x511)*(x514)))+(((pz)*(x501)*(x519)))+(((IkReal(-1.00000000000000))*(x499)*(x518)))+(((IkReal(-1.00000000000000))*(x493)*(x502)))+(((IkReal(-0.0662993000000000))*(x500)))+(((IkReal(-0.263555977500000))*(x490)*(x496)))+(((IkReal(-1.00000000000000))*(x490)*(x496)*(x521)))+(((IkReal(-0.810000000000000))*(r02)*(x513)))+(((IkReal(-1.00000000000000))*(cj1)*(x501)*(x512)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x498)*(x513)))+(((IkReal(-1.00000000000000))*(py)*(pz)*(x498)*(x520)))+(((IkReal(-1.00000000000000))*(pz)*(x495)*(x510)*(x511)))+(((sj0)*(x490)*(x495)*(x503)))+(((cj1)*(x493)*(x512)))+(((cj1)*(x495)*(x510)*(x513)))+(((IkReal(-1.00000000000000))*(pz)*(x493)*(x519)))+(((pp)*(x501)*(x511)))+(((IkReal(0.263555977500000))*(x501)*(x516)))+(((IkReal(0.200300000000000))*(sj0)*(x495)*(x520)))+(((IkReal(-1.00000000000000))*(r02)*(x504))));
evalcond[3]=((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((sj1)*(x490)*(x504)))+(((sj0)*(x507)))+(((IkReal(-0.810000000000000))*(cj1)*(x497)))+(((x504)*(x520)))+(((IkReal(0.662000000000000))*(x497)*(x511)))+(((IkReal(0.268110000000000))*(x490)))+(pp)+(((IkReal(-0.662000000000000))*(x497)*(x516))));
evalcond[4]=((((IkReal(0.405000000000000))*(x496)))+(((x496)*(x515)))+(((IkReal(0.331000000000000))*(x501)*(x511)))+(((IkReal(-0.331000000000000))*(x493)*(x511)))+(((x493)*(x494)))+(((IkReal(-1.00000000000000))*(cj1)*(x501)*(x515)))+(((IkReal(0.100150000000000))*(x500)))+(((IkReal(-1.00000000000000))*(x494)*(x501)))+(x509)+(((IkReal(0.331000000000000))*(x522)))+(((pz)*(r02)))+(((cj1)*(x493)*(x515)))+(((IkReal(0.100150000000000))*(x499))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x525=((cj1)*(sj2));
IkReal x526=((cj2)*(sj1));
IkReal x527=((cj0)*(py));
IkReal x528=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x525)*(x528)))+(((IkReal(-1.00000000000000))*(x526)*(x528)))+(((x526)*(x527)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x525)*(x527)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x529=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x525)*(x528)))+(((IkReal(-1.00000000000000))*(x526)*(x528)))+(((x526)*(x527)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x525)*(x527)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x529));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x529))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x530=IKsin(j3);
IkReal x531=IKcos(j3);
IkReal x532=((cj2)*(sj1));
IkReal x533=((cj1)*(sj2));
IkReal x534=((sj2)*(x531));
IkReal x535=((px)*(sj0)*(x531));
IkReal x536=((IkReal(1.00000000000000))*(cj0)*(py)*(x531));
evalcond[0]=((((IkReal(-1.00000000000000))*(x532)*(x536)))+(((IkReal(0.100150000000000))*(x530)))+(((IkReal(-1.00000000000000))*(x533)*(x536)))+(((pz)*(sj1)*(x534)))+(((cj0)*(px)*(x530)))+(((py)*(sj0)*(x530)))+(((x532)*(x535)))+(((x533)*(x535)))+(((IkReal(0.405000000000000))*(x534)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x531))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x537=((IkReal(1.00000000000000))*(sj3));
IkReal x538=((cj2)*(cj3));
IkReal x539=((cj0)*(r01));
IkReal x540=((cj3)*(sj2));
IkReal x541=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((cj1)*(x539)*(x540)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x537)))+(((IkReal(-1.00000000000000))*(cj1)*(x540)*(x541)))+(((sj1)*(x538)*(x539)))+(((IkReal(-1.00000000000000))*(sj1)*(x538)*(x541)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x540)))+(((cj1)*(r02)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x537))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x542=((sj1)*(sj3));
IkReal x543=((IkReal(1.00000000000000))*(cj0));
IkReal x544=((r02)*(sj2));
IkReal x545=((IkReal(1.00000000000000))*(cj1));
IkReal x546=((cj2)*(r02));
IkReal x547=((sj1)*(sj2));
IkReal x548=((cj2)*(r01));
IkReal x549=((r00)*(sj0));
IkReal x550=((cj2)*(x549));
IkReal x551=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x544)*(x545)))+(((IkReal(-1.00000000000000))*(x545)*(x550)))+(((x547)*(x549)))+(((IkReal(-1.00000000000000))*(sj1)*(x546)))+(((IkReal(-1.00000000000000))*(r01)*(x543)*(x547)))+(((cj0)*(cj1)*(x548))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x542)*(x550)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x543)))+(((IkReal(-1.00000000000000))*(x542)*(x543)*(x548)))+(((x542)*(x544)))+(((IkReal(-1.00000000000000))*(r01)*(x543)*(x551)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x549)*(x551)))+(((IkReal(-1.00000000000000))*(sj3)*(x545)*(x546))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x544)*(x545)))+(((IkReal(-1.00000000000000))*(x545)*(x550)))+(((x547)*(x549)))+(((IkReal(-1.00000000000000))*(sj1)*(x546)))+(((IkReal(-1.00000000000000))*(r01)*(x543)*(x547)))+(((cj0)*(cj1)*(x548)))))), ((gconst4)*(((((x542)*(x550)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x543)))+(((IkReal(-1.00000000000000))*(x542)*(x543)*(x548)))+(((x542)*(x544)))+(((IkReal(-1.00000000000000))*(r01)*(x543)*(x551)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x549)*(x551)))+(((IkReal(-1.00000000000000))*(sj3)*(x545)*(x546)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x552=((cj2)*(sj1));
IkReal x553=((cj1)*(sj2));
IkReal x554=((cj0)*(r01));
IkReal x555=((sj1)*(sj2));
IkReal x556=((r00)*(sj0));
IkReal x557=((cj1)*(cj2));
IkReal x558=((r02)*(sj3));
IkReal x559=((IkReal(1.00000000000000))*(sj3)*(x556));
evalcond[0]=((((r02)*(x553)))+(((r02)*(x552)))+(((IkReal(-1.00000000000000))*(x554)*(x557)))+(((x554)*(x555)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x555)*(x556)))+(((x556)*(x557))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((sj3)*(x552)*(x554)))+(((x557)*(x558)))+(((sj3)*(x553)*(x554)))+(((IkReal(-1.00000000000000))*(x553)*(x559)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x552)*(x559)))+(((IkReal(-1.00000000000000))*(x555)*(x558)))+(((cj0)*(cj3)*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x560=(py)*(py)*(py)*(py);
IkReal x561=(cj1)*(cj1);
IkReal x562=(py)*(py);
IkReal x563=((cj1)*(py));
if( IKabs(((IkReal(-0.810000000000000))*(x561)*(x562))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.200300000000000))*(cj1)*(x562))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x564=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-0.810000000000000))*(x561)*(x562)), ((IkReal(0.200300000000000))*(cj1)*(x562)))));
if( (((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561))))))))):(IkReal)1.0e30))*(((((IkReal(0.810000000000000))*(pz)*(sj1)*(x563)))+(((IkReal(0.0644940225000000))*(x563)))+(((pp)*(x563))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561))))))))):(IkReal)1.0e30))*(((((IkReal(0.810000000000000))*(pz)*(sj1)*(x563)))+(((IkReal(0.0644940225000000))*(x563)))+(((pp)*(x563))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x565=IKasin(((((IKabs(IKabs(IKsqrt(((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((IkReal(0.0401200900000000))*(x560)*(x561)))+(((IkReal(0.656100000000000))*(x560)*((x561)*(x561))))))))):(IkReal)1.0e30))*(((((IkReal(0.810000000000000))*(pz)*(sj1)*(x563)))+(((IkReal(0.0644940225000000))*(x563)))+(((pp)*(x563)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x565)))+(((IkReal(-1.00000000000000))*(x564))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x565)+(((IkReal(-1.00000000000000))*(x564))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[1];
IkReal x566=(py)*(py);
IkReal x567=((py)*(sj1));
IkReal x568=((sj1)*(x566));
evalcond[0]=((((IkReal(-0.200300000000000))*(x568)*(IKsin(j0))))+(((IkReal(0.810000000000000))*(cj1)*(x568)*(IKcos(j0))))+(((IkReal(-0.810000000000000))*(pz)*(sj1)*(x567)))+(((IkReal(-1.00000000000000))*(pp)*(x567)))+(((IkReal(-0.0644940225000000))*(x567))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x569=(sj1)*(sj1);
IkReal x570=(cj1)*(cj1);
IkReal x571=((cj0)*(r01));
IkReal x572=((IkReal(134.055000000000))*(sj1));
IkReal x573=((r00)*(sj0));
IkReal x574=((IkReal(331.000000000000))*(x569));
IkReal x575=((cj0)*(py)*(r02));
IkReal x576=((IkReal(331.000000000000))*(x570));
gconst2=IKsign(((((x571)*(x572)))+(((pz)*(x571)*(x576)))+(((pz)*(x571)*(x574)))+(((IkReal(-1.00000000000000))*(pz)*(x573)*(x574)))+(((IkReal(-1.00000000000000))*(pz)*(x573)*(x576)))+(((IkReal(-1.00000000000000))*(x574)*(x575)))+(((IkReal(-1.00000000000000))*(x575)*(x576)))+(((IkReal(134.055000000000))*(cj1)*(r02)))+(((IkReal(-1.00000000000000))*(x572)*(x573)))));
IkReal x577=(sj1)*(sj1);
IkReal x578=(cj1)*(cj1);
IkReal x579=((IkReal(2.46913580246914))*(pz));
IkReal x580=((r00)*(sj0));
IkReal x581=((cj0)*(r01));
IkReal x582=((IkReal(2.46913580246914))*(cj0)*(py)*(r02));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x578)*(x579)*(x580)))+(((sj1)*(x581)))+(((IkReal(-1.00000000000000))*(x578)*(x582)))+(((IkReal(-1.00000000000000))*(sj1)*(x580)))+(((cj1)*(r02)))+(((x577)*(x579)*(x581)))+(((IkReal(-1.00000000000000))*(x577)*(x582)))+(((IkReal(-1.00000000000000))*(x577)*(x579)*(x580)))+(((x578)*(x579)*(x581))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x583=(cj0)*(cj0);
IkReal x584=(cj1)*(cj1);
IkReal x585=(sj1)*(sj1);
IkReal x586=(py)*(py);
IkReal x587=(pz)*(pz);
IkReal x588=((IkReal(100.150000000000))*(cj0));
IkReal x589=((cj0)*(sj1));
IkReal x590=((py)*(r00));
IkReal x591=((IkReal(1000.00000000000))*(py));
IkReal x592=((r00)*(sj0));
IkReal x593=((pz)*(r02));
IkReal x594=((IkReal(100.150000000000))*(sj0));
IkReal x595=((py)*(r01));
IkReal x596=((cj1)*(r02));
IkReal x597=((IkReal(54.4640000000000))*(cj1));
IkReal x598=((r02)*(sj1));
IkReal x599=((cj0)*(cj1));
IkReal x600=((pz)*(r00)*(sj1));
IkReal x601=((IkReal(405.000000000000))*(cj1)*(sj0));
IkReal x602=((cj1)*(pz)*(r01));
IkReal x603=((py)*(x583));
IkReal x604=((IkReal(1000.00000000000))*(x587));
IkReal x605=((IkReal(100.150000000000))*(x583));
IkReal x606=((IkReal(405.000000000000))*(x585));
IkReal x607=((pz)*(r01)*(sj1));
IkReal x608=((IkReal(405.000000000000))*(x584));
IkReal x609=((IkReal(405.000000000000))*(cj1)*(sj1));
IkReal x610=((r01)*(x608));
IkReal x611=((IkReal(1000.00000000000))*(r01)*(x586));
if( IKabs(((gconst2)*(((((cj1)*(x590)*(x605)))+(((IkReal(-405.000000000000))*(x593)))+(((IkReal(-405.000000000000))*(x595)))+(((IkReal(405.000000000000))*(py)*(x589)*(x596)))+(((IkReal(-1.00000000000000))*(x593)*(x606)))+(((IkReal(-1.00000000000000))*(x583)*(x595)*(x608)))+(((IkReal(405.000000000000))*(x589)*(x602)))+(((cj0)*(r01)*(x597)))+(((IkReal(-1.00000000000000))*(x592)*(x597)))+(((IkReal(-1.00000000000000))*(x588)*(x600)))+(((IkReal(-1.00000000000000))*(x598)*(x604)))+(((IkReal(-1.00000000000000))*(x591)*(x607)))+(((cj1)*(sj0)*(x588)*(x595)))+(((x591)*(x593)*(x599)))+(((x599)*(x611)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-54.4640000000000))*(x598)))+(((IkReal(-1.00000000000000))*(pz)*(x592)*(x609)))+(((IkReal(-1.00000000000000))*(x594)*(x607)))+(((cj0)*(sj0)*(x590)*(x608)))+(((IkReal(-40.5607500000000))*(r01)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x583)*(x595)*(x609)))+(((x594)*(x602)))+(((x589)*(x611)))+(((IkReal(-1.00000000000000))*(cj0)*(pz)*(x610)))+(((IkReal(109.561000000000))*(sj1)*(x592)))+(((sj0)*(sj1)*(x588)*(x595)))+(((x593)*(x609)))+(((x589)*(x591)*(x593)))+(((x591)*(x602)))+(((cj0)*(py)*(r02)*(x606)))+(((IkReal(-109.561000000000))*(r01)*(x589)))+(((x596)*(x604)))+(((pz)*(x592)*(x608)))+(((IkReal(-109.561000000000))*(x596)))+(((sj1)*(x590)*(x605)))+(((cj1)*(pz)*(r00)*(x588)))+(((x589)*(x590)*(x601))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((cj1)*(x590)*(x605)))+(((IkReal(-405.000000000000))*(x593)))+(((IkReal(-405.000000000000))*(x595)))+(((IkReal(405.000000000000))*(py)*(x589)*(x596)))+(((IkReal(-1.00000000000000))*(x593)*(x606)))+(((IkReal(-1.00000000000000))*(x583)*(x595)*(x608)))+(((IkReal(405.000000000000))*(x589)*(x602)))+(((cj0)*(r01)*(x597)))+(((IkReal(-1.00000000000000))*(x592)*(x597)))+(((IkReal(-1.00000000000000))*(x588)*(x600)))+(((IkReal(-1.00000000000000))*(x598)*(x604)))+(((IkReal(-1.00000000000000))*(x591)*(x607)))+(((cj1)*(sj0)*(x588)*(x595)))+(((x591)*(x593)*(x599)))+(((x599)*(x611)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-54.4640000000000))*(x598)))+(((IkReal(-1.00000000000000))*(pz)*(x592)*(x609)))+(((IkReal(-1.00000000000000))*(x594)*(x607)))+(((cj0)*(sj0)*(x590)*(x608)))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x583)*(x595)*(x609)))+(((x594)*(x602)))+(((x589)*(x611)))+(((IkReal(-1.00000000000000))*(cj0)*(pz)*(x610)))+(((IkReal(109.561000000000))*(sj1)*(x592)))+(((sj0)*(sj1)*(x588)*(x595)))+(((x593)*(x609)))+(((x589)*(x591)*(x593)))+(((x591)*(x602)))+(((cj0)*(py)*(r02)*(x606)))+(((IkReal(-109.561000000000))*(r01)*(x589)))+(((x596)*(x604)))+(((pz)*(x592)*(x608)))+(((IkReal(-109.561000000000))*(x596)))+(((sj1)*(x590)*(x605)))+(((cj1)*(pz)*(r00)*(x588)))+(((x589)*(x590)*(x601)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x612=IKsin(j2);
IkReal x613=IKcos(j2);
IkReal x614=(py)*(py);
IkReal x615=(pz)*(pz);
IkReal x616=((r00)*(sj0));
IkReal x617=((IkReal(0.405000000000000))*(cj1));
IkReal x618=((py)*(r02));
IkReal x619=((r02)*(sj1));
IkReal x620=((cj0)*(py));
IkReal x621=((IkReal(2.00000000000000))*(r01));
IkReal x622=((r01)*(sj0));
IkReal x623=((cj0)*(r00));
IkReal x624=((cj0)*(r01));
IkReal x625=((IkReal(0.268110000000000))*(cj1));
IkReal x626=((IkReal(0.200300000000000))*(sj1));
IkReal x627=((IkReal(0.662000000000000))*(pz));
IkReal x628=((IkReal(1.00000000000000))*(pz));
IkReal x629=((IkReal(0.100150000000000))*(r02));
IkReal x630=((IkReal(0.200300000000000))*(py));
IkReal x631=((IkReal(0.200300000000000))*(pz));
IkReal x632=((py)*(r01));
IkReal x633=((IkReal(2.00000000000000))*(cj0));
IkReal x634=((sj1)*(x612));
IkReal x635=((pp)*(x613));
IkReal x636=((pz)*(x613));
IkReal x637=((cj0)*(x614));
IkReal x638=((IkReal(0.331000000000000))*(x613));
IkReal x639=((cj1)*(x613));
IkReal x640=((IkReal(0.405000000000000))*(x612));
IkReal x641=((IkReal(0.0811215000000000))*(x613));
IkReal x642=((IkReal(0.810000000000000))*(x612));
IkReal x643=((cj1)*(x612));
IkReal x644=((IkReal(2.00000000000000))*(x615));
IkReal x645=((r02)*(x643));
IkReal x646=((py)*(sj1)*(x613));
IkReal x647=((IkReal(0.100150000000000))*(sj1)*(x613));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(-1.00000000000000))*(x620)*(x639)))+(((IkReal(0.405000000000000))*(x613)))+(((pz)*(x643)))+(((sj1)*(x636)))+(((x620)*(x634))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x629)*(x639)))+(((IkReal(-1.00000000000000))*(x622)*(x628)*(x634)))+(((cj1)*(x622)*(x636)))+(((cj1)*(x623)*(x636)))+(((IkReal(-0.100150000000000))*(x624)*(x643)))+(((x616)*(x647)))+(((sj0)*(x618)*(x634)))+(((py)*(r00)*(x643)))+(((IkReal(-1.00000000000000))*(x624)*(x647)))+(((r00)*(x646)))+(((IkReal(-1.00000000000000))*(x622)*(x640)))+(((IkReal(-1.00000000000000))*(x623)*(x640)))+(((IkReal(0.100150000000000))*(x612)*(x619)))+(((IkReal(-1.00000000000000))*(x623)*(x628)*(x634)))+(((IkReal(0.100150000000000))*(x616)*(x643)))+(((IkReal(-1.00000000000000))*(sj0)*(x618)*(x639))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x613)*(x619)*(x644)))+(((pp)*(x624)*(x634)))+(((x621)*(x637)*(x639)))+(((IkReal(0.263555977500000))*(x624)*(x639)))+(((sj0)*(x613)*(x618)*(x626)))+(((IkReal(-1.00000000000000))*(pz)*(x616)*(x642)))+(((IkReal(-0.662000000000000))*(x632)))+(((IkReal(-1.00000000000000))*(cj0)*(x618)*(x642)))+(((pp)*(x645)))+(((IkReal(0.0644940225000000))*(x624)*(x634)))+(((cj1)*(x616)*(x635)))+(((x619)*(x635)))+(((IkReal(-1.00000000000000))*(r02)*(x627)))+(((IkReal(-1.00000000000000))*(x622)*(x631)*(x643)))+(((cj1)*(x618)*(x633)*(x636)))+(((IkReal(-1.00000000000000))*(py)*(r00)*(x612)*(x626)))+(((IkReal(-0.0644940225000000))*(x616)*(x634)))+(((IkReal(-0.268110000000000))*(x619)))+(((IkReal(0.200300000000000))*(sj0)*(x618)*(x643)))+(((IkReal(-1.00000000000000))*(x644)*(x645)))+(((IkReal(-1.00000000000000))*(x623)*(x626)*(x636)))+(((IkReal(-0.263555977500000))*(x616)*(x639)))+(((IkReal(-0.810000000000000))*(r02)*(x636)))+(((x624)*(x625)))+(((IkReal(-1.00000000000000))*(x622)*(x626)*(x636)))+(((IkReal(-0.0662993000000000))*(x622)))+(((IkReal(-0.0662993000000000))*(x623)))+(((IkReal(-0.263555977500000))*(x613)*(x619)))+(((IkReal(-1.00000000000000))*(cj1)*(x624)*(x635)))+(((IkReal(-1.00000000000000))*(x623)*(x631)*(x643)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x621)*(x636)))+(((IkReal(0.0644940225000000))*(x645)))+(((IkReal(-1.00000000000000))*(x622)*(x641)))+(((IkReal(-1.00000000000000))*(x616)*(x625)))+(((IkReal(-1.00000000000000))*(x623)*(x641)))+(((IkReal(-1.00000000000000))*(py)*(pz)*(x621)*(x643)))+(((pz)*(x624)*(x642)))+(((IkReal(-1.00000000000000))*(x621)*(x634)*(x637)))+(((r00)*(x630)*(x639)))+(((IkReal(-1.00000000000000))*(pz)*(x618)*(x633)*(x634)))+(((IkReal(-1.00000000000000))*(pp)*(x616)*(x634)))+(((IkReal(-0.810000000000000))*(x613)*(x632))));
evalcond[3]=((IkReal(0.283616022500000))+(((x627)*(x643)))+(((IkReal(-0.662000000000000))*(x620)*(x639)))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.662000000000000))*(x620)*(x634)))+(((sj1)*(x613)*(x627)))+(((IkReal(-0.810000000000000))*(cj1)*(x620)))+(((sj0)*(x630)))+(pp)+(((IkReal(0.268110000000000))*(x613))));
evalcond[4]=((((IkReal(0.100150000000000))*(x623)))+(((IkReal(0.100150000000000))*(x622)))+(((IkReal(0.331000000000000))*(x645)))+(((IkReal(-0.331000000000000))*(x616)*(x634)))+(((cj1)*(x616)*(x638)))+(((x619)*(x638)))+(((IkReal(-1.00000000000000))*(cj1)*(x624)*(x638)))+(((IkReal(0.405000000000000))*(x619)))+(((x616)*(x617)))+(((IkReal(-1.00000000000000))*(x617)*(x624)))+(x632)+(((IkReal(0.331000000000000))*(x624)*(x634)))+(((pz)*(r02))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x648=((cj1)*(sj2));
IkReal x649=((cj2)*(sj1));
IkReal x650=((cj0)*(py));
IkReal x651=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((x649)*(x650)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x648)*(x650)))+(((IkReal(-1.00000000000000))*(x649)*(x651)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x648)*(x651))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x652=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((x649)*(x650)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x648)*(x650)))+(((IkReal(-1.00000000000000))*(x649)*(x651)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x648)*(x651)))));
j3array[0]=((IkReal(-1.00000000000000))*(x652));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x652))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x653=IKsin(j3);
IkReal x654=IKcos(j3);
IkReal x655=((cj2)*(sj1));
IkReal x656=((cj1)*(sj2));
IkReal x657=((sj2)*(x654));
IkReal x658=((px)*(sj0)*(x654));
IkReal x659=((IkReal(1.00000000000000))*(cj0)*(py)*(x654));
evalcond[0]=((((IkReal(-1.00000000000000))*(x655)*(x659)))+(((IkReal(-1.00000000000000))*(x656)*(x659)))+(((cj0)*(px)*(x653)))+(((IkReal(0.405000000000000))*(x657)))+(((x656)*(x658)))+(((x655)*(x658)))+(((IkReal(0.100150000000000))*(x653)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x654)))+(((py)*(sj0)*(x653)))+(((pz)*(sj1)*(x657))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x660=((IkReal(1.00000000000000))*(sj3));
IkReal x661=((cj2)*(cj3));
IkReal x662=((cj0)*(r01));
IkReal x663=((cj3)*(sj2));
IkReal x664=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x663)))+(((IkReal(-1.00000000000000))*(sj1)*(x661)*(x664)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x660)))+(((IkReal(-1.00000000000000))*(cj1)*(x663)*(x664)))+(((sj1)*(x661)*(x662)))+(((cj1)*(r02)*(x661)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x660)))+(((cj1)*(x662)*(x663))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x665=((sj1)*(sj3));
IkReal x666=((IkReal(1.00000000000000))*(cj0));
IkReal x667=((r02)*(sj2));
IkReal x668=((IkReal(1.00000000000000))*(cj1));
IkReal x669=((cj2)*(r02));
IkReal x670=((sj1)*(sj2));
IkReal x671=((cj2)*(r01));
IkReal x672=((r00)*(sj0));
IkReal x673=((cj2)*(x672));
IkReal x674=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((x670)*(x672)))+(((IkReal(-1.00000000000000))*(x668)*(x673)))+(((IkReal(-1.00000000000000))*(r01)*(x666)*(x670)))+(((cj0)*(cj1)*(x671)))+(((IkReal(-1.00000000000000))*(x667)*(x668)))+(((IkReal(-1.00000000000000))*(sj1)*(x669))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x672)*(x674)))+(((x665)*(x673)))+(((IkReal(-1.00000000000000))*(x665)*(x666)*(x671)))+(((x665)*(x667)))+(((IkReal(-1.00000000000000))*(r01)*(x666)*(x674)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x668)*(x669)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x666))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((x670)*(x672)))+(((IkReal(-1.00000000000000))*(x668)*(x673)))+(((IkReal(-1.00000000000000))*(r01)*(x666)*(x670)))+(((cj0)*(cj1)*(x671)))+(((IkReal(-1.00000000000000))*(x667)*(x668)))+(((IkReal(-1.00000000000000))*(sj1)*(x669)))))), ((gconst4)*(((((x672)*(x674)))+(((x665)*(x673)))+(((IkReal(-1.00000000000000))*(x665)*(x666)*(x671)))+(((x665)*(x667)))+(((IkReal(-1.00000000000000))*(r01)*(x666)*(x674)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x668)*(x669)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x666)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x675=((cj2)*(sj1));
IkReal x676=((cj1)*(sj2));
IkReal x677=((cj0)*(r01));
IkReal x678=((sj1)*(sj2));
IkReal x679=((r00)*(sj0));
IkReal x680=((cj1)*(cj2));
IkReal x681=((r02)*(sj3));
IkReal x682=((IkReal(1.00000000000000))*(sj3)*(x679));
evalcond[0]=((((r02)*(x676)))+(((r02)*(x675)))+(((x677)*(x678)))+(((x679)*(x680)))+(((IkReal(-1.00000000000000))*(x678)*(x679)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x677)*(x680))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x678)*(x681)))+(((IkReal(-1.00000000000000))*(x675)*(x682)))+(((sj5)*(IKcos(j4))))+(((x680)*(x681)))+(((sj3)*(x675)*(x677)))+(((sj3)*(x676)*(x677)))+(((IkReal(-1.00000000000000))*(x676)*(x682)))+(((cj0)*(cj3)*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x683=(cj1)*(cj1);
IkReal x684=(py)*(py);
IkReal x685=((IkReal(0.200300000000000))*(cj1));
IkReal x686=((cj1)*(py));
IkReal x687=((px)*(py));
IkReal x688=((IkReal(0.810000000000000))*(x683));
IkReal x689=((((x685)*(x687)))+(((IkReal(-1.00000000000000))*(x684)*(x688))));
IkReal x690=((((x687)*(x688)))+(((x684)*(x685))));
if( IKabs(x689) < IKFAST_ATAN2_MAGTHRESH && IKabs(x690) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x691=((IkReal(1.00000000000000))*(IKatan2(x689, x690)));
if( ((((x690)*(x690))+((x689)*(x689)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x690)*(x690))+((x689)*(x689)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x690)*(x690))+((x689)*(x689))))))):(IkReal)1.0e30))*(((((IkReal(0.0644940225000000))*(x686)))+(((pp)*(x686)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x686))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x690)*(x690))+((x689)*(x689)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x690)*(x690))+((x689)*(x689))))))):(IkReal)1.0e30))*(((((IkReal(0.0644940225000000))*(x686)))+(((pp)*(x686)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x686))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x692=IKasin(((((IKabs(IKabs(IKsqrt((((x690)*(x690))+((x689)*(x689)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x690)*(x690))+((x689)*(x689))))))):(IkReal)1.0e30))*(((((IkReal(0.0644940225000000))*(x686)))+(((pp)*(x686)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x686)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x691)))+(((IkReal(-1.00000000000000))*(x692))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x692)+(((IkReal(-1.00000000000000))*(x691))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x693=(cj1)*(cj1);
IkReal x694=(px)*(px);
IkReal x695=IKcos(j0);
IkReal x696=IKsin(j0);
IkReal x697=(py)*(py);
IkReal x698=(sj1)*(sj1);
IkReal x699=((IkReal(0.200300000000000))*(sj1));
IkReal x700=((px)*(py));
IkReal x701=((cj1)*(sj1));
IkReal x702=((IkReal(0.200300000000000))*(cj1));
IkReal x703=((pp)*(px));
IkReal x704=((IkReal(0.0644940225000000))*(sj1));
IkReal x705=((IkReal(0.810000000000000))*(px)*(pz));
IkReal x706=((IkReal(0.810000000000000))*(x694));
IkReal x707=((x699)*(x700));
IkReal x708=((IkReal(0.810000000000000))*(x700)*(x701));
evalcond[0]=((((x695)*(((((IkReal(-1.00000000000000))*(x694)*(x702)))+(((IkReal(0.810000000000000))*(x693)*(x700)))))))+(((IkReal(-1.00000000000000))*(cj1)*(x703)))+(((IkReal(-0.0644940225000000))*(cj1)*(px)))+(((x696)*(((((IkReal(-1.00000000000000))*(x693)*(x706)))+(((IkReal(-1.00000000000000))*(x700)*(x702)))))))+(((IkReal(-1.00000000000000))*(x701)*(x705))));
evalcond[1]=((((x695)*(((((x694)*(x699)))+(((IkReal(-1.00000000000000))*(x708)))))))+(((px)*(x704)))+(((x696)*(((((x701)*(x706)))+(x707)))))+(((x698)*(x705)))+(((sj1)*(x703))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(x704)))+(((IkReal(-0.810000000000000))*(py)*(pz)*(x698)))+(((x696)*(((((IkReal(-1.00000000000000))*(x697)*(x699)))+(((IkReal(-1.00000000000000))*(x708)))))))+(((IkReal(-1.00000000000000))*(pp)*(py)*(sj1)))+(((x695)*(((((IkReal(0.810000000000000))*(x697)*(x701)))+(((IkReal(-1.00000000000000))*(x707))))))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x709=(cj1)*(cj1);
IkReal x710=(sj1)*(sj1);
IkReal x711=((cj0)*(r01));
IkReal x712=((IkReal(134.055000000000))*(sj1));
IkReal x713=((px)*(sj0));
IkReal x714=((cj0)*(py));
IkReal x715=((IkReal(331.000000000000))*(pz));
IkReal x716=((r00)*(sj0));
IkReal x717=((IkReal(331.000000000000))*(r02)*(x710));
IkReal x718=((IkReal(331.000000000000))*(r02)*(x709));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x714)*(x717)))+(((IkReal(-1.00000000000000))*(x714)*(x718)))+(((x709)*(x711)*(x715)))+(((IkReal(-1.00000000000000))*(x710)*(x715)*(x716)))+(((IkReal(-1.00000000000000))*(x709)*(x715)*(x716)))+(((x711)*(x712)))+(((IkReal(-1.00000000000000))*(x712)*(x716)))+(((IkReal(134.055000000000))*(cj1)*(r02)))+(((x710)*(x711)*(x715)))+(((x713)*(x718)))+(((x713)*(x717)))));
IkReal x719=(cj1)*(cj1);
IkReal x720=(sj1)*(sj1);
IkReal x721=((IkReal(2.46913580246914))*(sj0));
IkReal x722=((px)*(r02));
IkReal x723=((pz)*(r00));
IkReal x724=((IkReal(2.46913580246914))*(cj0));
IkReal x725=((pz)*(r01));
IkReal x726=((py)*(r02));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x719)*(x721)*(x723)))+(((IkReal(-1.00000000000000))*(x719)*(x724)*(x726)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj1)))+(((cj1)*(r02)))+(((x720)*(x724)*(x725)))+(((cj0)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(x720)*(x724)*(x726)))+(((x720)*(x721)*(x722)))+(((x719)*(x721)*(x722)))+(((IkReal(-1.00000000000000))*(x720)*(x721)*(x723)))+(((x719)*(x724)*(x725))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst1)*(((((IkReal(-1000.00000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(r00)*(sj0)*((px)*(px))))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(px)*(py)*(r00)))+(((IkReal(-405.000000000000))*(py)*(r01)*((cj0)*(cj0))*((cj1)*(cj1))))+(((IkReal(-54.4640000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-100.150000000000))*(cj1)*(px)*(r01)*((sj0)*(sj0))))+(((IkReal(100.150000000000))*(cj1)*(py)*(r00)*((cj0)*(cj0))))+(((IkReal(-54.4640000000000))*(r02)*(sj1)))+(((IkReal(-1000.00000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-100.150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(r01)*((py)*(py))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r02)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(py)*(r01)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(py)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-100.150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-405.000000000000))*(pz)*(r02)*((sj1)*(sj1))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(pz)*(r01)*(sj1)))+(((IkReal(-100.150000000000))*(cj0)*(cj1)*(px)*(r00)*(sj0)))+(((IkReal(-405.000000000000))*(px)*(r00)))+(((IkReal(54.4640000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-405.000000000000))*(py)*(r01)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(cj1)*(pz)*(r00)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r02)*(sj1)))+(((IkReal(-405.000000000000))*(px)*(r00)*((cj1)*(cj1))*((sj0)*(sj0))))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))+(((IkReal(405.000000000000))*(cj0)*(px)*(r01)*(sj0)*((cj1)*(cj1)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(109.561000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(py)*(r00)*(sj1)*((cj0)*(cj0))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r00)*(sj1)*((sj0)*(sj0))))+(((IkReal(-109.561000000000))*(cj1)*(r02)))+(((IkReal(-405.000000000000))*(cj0)*(pz)*(r01)*((cj1)*(cj1))))+(((IkReal(1000.00000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r02)*((sj1)*(sj1))))+(((IkReal(-100.150000000000))*(cj0)*(px)*(r00)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(px)*(py)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(r01)*(sj1)*((py)*(py))))+(((IkReal(100.150000000000))*(cj0)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1000.00000000000))*(r00)*(sj0)*(sj1)*((px)*(px))))+(((IkReal(405.000000000000))*(cj1)*(pz)*(r02)*(sj1)))+(((IkReal(-109.561000000000))*(cj0)*(r01)*(sj1)))+(((IkReal(405.000000000000))*(pz)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-405.000000000000))*(px)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-405.000000000000))*(cj1)*(py)*(r01)*(sj1)*((cj0)*(cj0))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(-100.150000000000))*(px)*(r01)*(sj1)*((sj0)*(sj0))))+(((IkReal(1000.00000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(px)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(py)*(pz)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1000.00000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(r00)*(sj0)*((px)*(px))))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(px)*(py)*(r00)))+(((IkReal(-405.000000000000))*(py)*(r01)*((cj0)*(cj0))*((cj1)*(cj1))))+(((IkReal(-54.4640000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-100.150000000000))*(cj1)*(px)*(r01)*((sj0)*(sj0))))+(((IkReal(100.150000000000))*(cj1)*(py)*(r00)*((cj0)*(cj0))))+(((IkReal(-54.4640000000000))*(r02)*(sj1)))+(((IkReal(-1000.00000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-100.150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(r01)*((py)*(py))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r02)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(py)*(r01)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(py)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-100.150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-405.000000000000))*(pz)*(r02)*((sj1)*(sj1))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(pz)*(r01)*(sj1)))+(((IkReal(-100.150000000000))*(cj0)*(cj1)*(px)*(r00)*(sj0)))+(((IkReal(-405.000000000000))*(px)*(r00)))+(((IkReal(54.4640000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-405.000000000000))*(py)*(r01)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(cj1)*(pz)*(r00)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r02)*(sj1)))+(((IkReal(-405.000000000000))*(px)*(r00)*((cj1)*(cj1))*((sj0)*(sj0))))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))+(((IkReal(405.000000000000))*(cj0)*(px)*(r01)*(sj0)*((cj1)*(cj1))))))), ((gconst1)*(((((IkReal(109.561000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(py)*(r00)*(sj1)*((cj0)*(cj0))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r00)*(sj1)*((sj0)*(sj0))))+(((IkReal(-109.561000000000))*(cj1)*(r02)))+(((IkReal(-405.000000000000))*(cj0)*(pz)*(r01)*((cj1)*(cj1))))+(((IkReal(1000.00000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r02)*((sj1)*(sj1))))+(((IkReal(-100.150000000000))*(cj0)*(px)*(r00)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(px)*(py)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(r01)*(sj1)*((py)*(py))))+(((IkReal(100.150000000000))*(cj0)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1000.00000000000))*(r00)*(sj0)*(sj1)*((px)*(px))))+(((IkReal(405.000000000000))*(cj1)*(pz)*(r02)*(sj1)))+(((IkReal(-109.561000000000))*(cj0)*(r01)*(sj1)))+(((IkReal(405.000000000000))*(pz)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-405.000000000000))*(px)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-405.000000000000))*(cj1)*(py)*(r01)*(sj1)*((cj0)*(cj0))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(-100.150000000000))*(px)*(r01)*(sj1)*((sj0)*(sj0))))+(((IkReal(1000.00000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(px)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(py)*(pz)*(r01)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x727=IKsin(j2);
IkReal x728=IKcos(j2);
IkReal x729=((cj1)*(sj0));
IkReal x730=((IkReal(0.405000000000000))*(r00));
IkReal x731=((py)*(sj1));
IkReal x732=((cj0)*(cj1));
IkReal x733=((IkReal(0.662000000000000))*(sj1));
IkReal x734=((IkReal(1.00000000000000))*(r02));
IkReal x735=((IkReal(0.405000000000000))*(r01));
IkReal x736=((IkReal(0.331000000000000))*(r02));
IkReal x737=((IkReal(0.331000000000000))*(sj1));
IkReal x738=((IkReal(1.00000000000000))*(r01));
IkReal x739=((pz)*(sj1));
IkReal x740=((IkReal(0.100150000000000))*(r01));
IkReal x741=((IkReal(0.100150000000000))*(r02));
IkReal x742=((IkReal(0.100150000000000))*(r00));
IkReal x743=((r02)*(sj1));
IkReal x744=((sj0)*(x727));
IkReal x745=((r00)*(x728));
IkReal x746=((cj0)*(x727));
IkReal x747=((px)*(x728));
IkReal x748=((pz)*(x728));
IkReal x749=((py)*(x728));
IkReal x750=((cj1)*(x727));
IkReal x751=((sj1)*(x728));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x744)))+(((x728)*(x739)))+(((pz)*(x750)))+(((x731)*(x746)))+(((IkReal(-1.00000000000000))*(x732)*(x749)))+(((IkReal(0.405000000000000))*(x728)))+(((x729)*(x747))));
evalcond[1]=((((py)*(r00)*(x750)))+(((IkReal(-1.00000000000000))*(cj0)*(x740)*(x751)))+(((IkReal(-1.00000000000000))*(x729)*(x734)*(x749)))+(((sj0)*(x742)*(x751)))+(((pz)*(x732)*(x745)))+(((IkReal(-1.00000000000000))*(cj1)*(x728)*(x741)))+(((IkReal(-1.00000000000000))*(x730)*(x746)))+(((x731)*(x745)))+(((px)*(x743)*(x746)))+(((IkReal(-1.00000000000000))*(x738)*(x739)*(x744)))+(((IkReal(-1.00000000000000))*(r00)*(x739)*(x746)))+(((sj1)*(x727)*(x741)))+(((IkReal(-1.00000000000000))*(px)*(x738)*(x750)))+(((x727)*(x729)*(x742)))+(((IkReal(-1.00000000000000))*(x732)*(x734)*(x747)))+(((r01)*(x729)*(x748)))+(((IkReal(-1.00000000000000))*(x727)*(x732)*(x740)))+(((r02)*(x731)*(x744)))+(((IkReal(-1.00000000000000))*(x735)*(x744)))+(((IkReal(-1.00000000000000))*(sj1)*(x738)*(x747))));
evalcond[2]=((((IkReal(-0.200300000000000))*(cj1)*(px)*(r01)*(IKcos(j2))))+(((IkReal(-0.662000000000000))*(pz)*(r02)))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(px)*(py)*(r00)*(IKcos(j2))))+(((cj1)*(pp)*(r00)*(sj0)*(IKcos(j2))))+(((IkReal(-0.0662993000000000))*(cj0)*(r00)))+(((IkReal(-0.200300000000000))*(py)*(r00)*(sj1)*(IKsin(j2))))+(((cj0)*(pp)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(-0.810000000000000))*(pz)*(r02)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(2.00000000000000))*(r00)*(sj0)*(sj1)*((px)*(px))*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(r00)*(sj0)*((px)*(px))*(IKcos(j2))))+(((IkReal(-0.200300000000000))*(cj0)*(pz)*(r00)*(sj1)*(IKcos(j2))))+(((IkReal(-0.0662993000000000))*(r01)*(sj0)))+(((IkReal(-0.810000000000000))*(px)*(r00)*(IKcos(j2))))+(((IkReal(-0.0644940225000000))*(r00)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(px)*(pz)*(r00)*(sj1)*(IKcos(j2))))+(((IkReal(0.200300000000000))*(cj0)*(px)*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(0.0644940225000000))*(cj0)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(px)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(-0.263555977500000))*(cj1)*(r00)*(sj0)*(IKcos(j2))))+(((IkReal(-0.810000000000000))*(pz)*(r00)*(sj0)*(IKsin(j2))))+(((IkReal(-0.0811215000000000))*(r01)*(sj0)*(IKcos(j2))))+(((IkReal(-0.810000000000000))*(cj0)*(py)*(r02)*(IKsin(j2))))+(((IkReal(-0.200300000000000))*(cj0)*(cj1)*(pz)*(r00)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r02)*(sj0)*(IKsin(j2))))+(((IkReal(-0.200300000000000))*(pz)*(r01)*(sj0)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(r00)*(sj1)*(IKsin(j2))))+(((IkReal(-0.662000000000000))*(py)*(r01)))+(((IkReal(-0.810000000000000))*(py)*(r01)*(IKcos(j2))))+(((IkReal(-1.00000000000000))*(cj0)*(cj1)*(pp)*(r01)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(r01)*(sj0)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(r01)*(sj1)*((py)*(py))*(IKsin(j2))))+(((IkReal(0.200300000000000))*(cj0)*(cj1)*(px)*(r02)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(py)*(r02)*(sj0)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(py)*(pz)*(r01)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(r02)*((pz)*(pz))*(IKsin(j2))))+(((IkReal(-0.268110000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-0.662000000000000))*(px)*(r00)))+(((IkReal(0.0644940225000000))*(cj1)*(r02)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(py)*(pz)*(r01)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)*(IKcos(j2))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r00)*(IKcos(j2))))+(((IkReal(0.263555977500000))*(cj0)*(cj1)*(r01)*(IKcos(j2))))+(((IkReal(0.810000000000000))*(cj0)*(pz)*(r01)*(IKsin(j2))))+(((IkReal(-0.268110000000000))*(r02)*(sj1)))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(r01)*((py)*(py))*(IKcos(j2))))+(((IkReal(0.268110000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-0.0811215000000000))*(cj0)*(r00)*(IKcos(j2))))+(((IkReal(-1.00000000000000))*(pp)*(r00)*(sj0)*(sj1)*(IKsin(j2))))+(((cj1)*(pp)*(r02)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(r02)*(sj1)*((pz)*(pz))*(IKcos(j2))))+(((IkReal(-0.200300000000000))*(cj1)*(pz)*(r01)*(sj0)*(IKsin(j2))))+(((IkReal(0.810000000000000))*(px)*(r02)*(sj0)*(IKsin(j2))))+(((pp)*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(-0.263555977500000))*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(px)*(py)*(r01)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r00)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)*(IKsin(j2)))));
evalcond[3]=((IkReal(0.283616022500000))+(((IkReal(0.662000000000000))*(pz)*(x750)))+(((IkReal(0.268110000000000))*(x728)))+(((IkReal(-1.00000000000000))*(px)*(x733)*(x744)))+(((IkReal(0.810000000000000))*(x739)))+(((IkReal(-0.662000000000000))*(x732)*(x749)))+(((IkReal(0.662000000000000))*(x729)*(x747)))+(((IkReal(0.810000000000000))*(px)*(x729)))+(pp)+(((IkReal(0.200300000000000))*(py)*(sj0)))+(((IkReal(0.200300000000000))*(cj0)*(px)))+(((IkReal(-0.810000000000000))*(py)*(x732)))+(((IkReal(0.662000000000000))*(x731)*(x746)))+(((x733)*(x748))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x732)*(x735)))+(((x736)*(x750)))+(((x736)*(x751)))+(((px)*(r00)))+(((IkReal(-1.00000000000000))*(r00)*(x737)*(x744)))+(((x729)*(x730)))+(((IkReal(0.405000000000000))*(x743)))+(((IkReal(-0.331000000000000))*(r01)*(x728)*(x732)))+(((sj0)*(x740)))+(((IkReal(0.331000000000000))*(x729)*(x745)))+(((r01)*(x737)*(x746)))+(((pz)*(r02)))+(((py)*(r01)))+(((cj0)*(x742))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x752=((cj1)*(sj2));
IkReal x753=((cj2)*(sj1));
IkReal x754=((cj0)*(py));
IkReal x755=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((x752)*(x754)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x753)*(x755)))+(((IkReal(-1.00000000000000))*(x752)*(x755)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x753)*(x754))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x756=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((x752)*(x754)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x753)*(x755)))+(((IkReal(-1.00000000000000))*(x752)*(x755)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x753)*(x754)))));
j3array[0]=((IkReal(-1.00000000000000))*(x756));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x756))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x757=IKsin(j3);
IkReal x758=IKcos(j3);
IkReal x759=((cj2)*(sj1));
IkReal x760=((cj1)*(sj2));
IkReal x761=((sj2)*(x758));
IkReal x762=((px)*(sj0)*(x758));
IkReal x763=((IkReal(1.00000000000000))*(cj0)*(py)*(x758));
evalcond[0]=((((pz)*(sj1)*(x761)))+(((cj0)*(px)*(x757)))+(((x760)*(x762)))+(((x759)*(x762)))+(((IkReal(0.100150000000000))*(x757)))+(((py)*(sj0)*(x757)))+(((IkReal(0.405000000000000))*(x761)))+(((IkReal(-1.00000000000000))*(x760)*(x763)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x758)))+(((IkReal(-1.00000000000000))*(x759)*(x763))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x764=((IkReal(1.00000000000000))*(sj3));
IkReal x765=((cj2)*(cj3));
IkReal x766=((cj0)*(r01));
IkReal x767=((cj3)*(sj2));
IkReal x768=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((sj1)*(x765)*(x766)))+(((cj1)*(x766)*(x767)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x767)))+(((cj1)*(r02)*(x765)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x764)))+(((IkReal(-1.00000000000000))*(cj1)*(x767)*(x768)))+(((IkReal(-1.00000000000000))*(sj1)*(x765)*(x768)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x764))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x769=((sj1)*(sj3));
IkReal x770=((IkReal(1.00000000000000))*(cj0));
IkReal x771=((r02)*(sj2));
IkReal x772=((IkReal(1.00000000000000))*(cj1));
IkReal x773=((cj2)*(r02));
IkReal x774=((sj1)*(sj2));
IkReal x775=((cj2)*(r01));
IkReal x776=((r00)*(sj0));
IkReal x777=((cj2)*(x776));
IkReal x778=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((x774)*(x776)))+(((IkReal(-1.00000000000000))*(sj1)*(x773)))+(((IkReal(-1.00000000000000))*(x771)*(x772)))+(((cj0)*(cj1)*(x775)))+(((IkReal(-1.00000000000000))*(x772)*(x777)))+(((IkReal(-1.00000000000000))*(r01)*(x770)*(x774))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x776)*(x778)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x769)*(x777)))+(((x769)*(x771)))+(((IkReal(-1.00000000000000))*(x769)*(x770)*(x775)))+(((IkReal(-1.00000000000000))*(sj3)*(x772)*(x773)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x770)))+(((IkReal(-1.00000000000000))*(r01)*(x770)*(x778))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((x774)*(x776)))+(((IkReal(-1.00000000000000))*(sj1)*(x773)))+(((IkReal(-1.00000000000000))*(x771)*(x772)))+(((cj0)*(cj1)*(x775)))+(((IkReal(-1.00000000000000))*(x772)*(x777)))+(((IkReal(-1.00000000000000))*(r01)*(x770)*(x774)))))), ((gconst4)*(((((x776)*(x778)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x769)*(x777)))+(((x769)*(x771)))+(((IkReal(-1.00000000000000))*(x769)*(x770)*(x775)))+(((IkReal(-1.00000000000000))*(sj3)*(x772)*(x773)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x770)))+(((IkReal(-1.00000000000000))*(r01)*(x770)*(x778)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x779=((cj2)*(sj1));
IkReal x780=((cj1)*(sj2));
IkReal x781=((cj0)*(r01));
IkReal x782=((sj1)*(sj2));
IkReal x783=((r00)*(sj0));
IkReal x784=((cj1)*(cj2));
IkReal x785=((r02)*(sj3));
IkReal x786=((IkReal(1.00000000000000))*(sj3)*(x783));
evalcond[0]=((((x781)*(x782)))+(((x783)*(x784)))+(((IkReal(-1.00000000000000))*(x781)*(x784)))+(((r02)*(x780)))+(((sj5)*(IKsin(j4))))+(((IkReal(-1.00000000000000))*(x782)*(x783)))+(((r02)*(x779))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((sj5)*(IKcos(j4))))+(((x784)*(x785)))+(((IkReal(-1.00000000000000))*(x782)*(x785)))+(((IkReal(-1.00000000000000))*(x780)*(x786)))+(((cj0)*(cj3)*(r00)))+(((sj3)*(x779)*(x781)))+(((sj3)*(x780)*(x781)))+(((IkReal(-1.00000000000000))*(x779)*(x786))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x787=(py)*(py);
IkReal x788=((IkReal(0.810000000000000))*(cj1));
IkReal x789=((py)*(sj1));
IkReal x790=((px)*(x789));
IkReal x791=((sj1)*(x787));
IkReal x792=((((IkReal(-0.200300000000000))*(x790)))+(((x788)*(x791))));
IkReal x793=((((IkReal(-1.00000000000000))*(x788)*(x790)))+(((IkReal(-0.200300000000000))*(x791))));
if( IKabs(x792) < IKFAST_ATAN2_MAGTHRESH && IKabs(x793) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x794=((IkReal(1.00000000000000))*(IKatan2(x792, x793)));
if( ((((x792)*(x792))+((x793)*(x793)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x792)*(x792))+((x793)*(x793)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x792)*(x792))+((x793)*(x793))))))):(IkReal)1.0e30))*(((((IkReal(-0.810000000000000))*(pz)*(sj1)*(x789)))+(((IkReal(-0.0644940225000000))*(x789)))+(((IkReal(-1.00000000000000))*(pp)*(x789))))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x792)*(x792))+((x793)*(x793)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x792)*(x792))+((x793)*(x793))))))):(IkReal)1.0e30))*(((((IkReal(-0.810000000000000))*(pz)*(sj1)*(x789)))+(((IkReal(-0.0644940225000000))*(x789)))+(((IkReal(-1.00000000000000))*(pp)*(x789))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x795=((IkReal(-1.00000000000000))*(IKasin(((IkReal(-1.00000000000000))*(((IKabs(IKabs(IKsqrt((((x792)*(x792))+((x793)*(x793)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x792)*(x792))+((x793)*(x793))))))):(IkReal)1.0e30))*(((((IkReal(-0.810000000000000))*(pz)*(sj1)*(x789)))+(((IkReal(-0.0644940225000000))*(x789)))+(((IkReal(-1.00000000000000))*(pp)*(x789)))))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x794)))+(((IkReal(-1.00000000000000))*(x795))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x794)))+(((IkReal(1.00000000000000))*(x795))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x796=(cj1)*(cj1);
IkReal x797=(px)*(px);
IkReal x798=IKcos(j0);
IkReal x799=IKsin(j0);
IkReal x800=(py)*(py);
IkReal x801=((cj1)*(py));
IkReal x802=((px)*(sj1));
IkReal x803=((IkReal(0.810000000000000))*(pz));
IkReal x804=((cj1)*(px));
IkReal x805=((IkReal(0.200300000000000))*(cj1));
IkReal x806=((IkReal(0.810000000000000))*(x797));
IkReal x807=((IkReal(0.810000000000000))*(x796));
IkReal x808=((IkReal(0.200300000000000))*(px)*(x801));
IkReal x809=((px)*(py)*(x807));
evalcond[0]=((((x798)*(((((IkReal(-1.00000000000000))*(x797)*(x805)))+(x809)))))+(((x799)*(((((IkReal(-1.00000000000000))*(x796)*(x806)))+(((IkReal(-1.00000000000000))*(x808)))))))+(((IkReal(-1.00000000000000))*(cj1)*(x802)*(x803)))+(((IkReal(-1.00000000000000))*(pp)*(x804)))+(((IkReal(-0.0644940225000000))*(x804))));
evalcond[1]=((((pp)*(x802)))+(((sj1)*(x802)*(x803)))+(((x798)*(((((IkReal(0.200300000000000))*(sj1)*(x797)))+(((IkReal(-0.810000000000000))*(x801)*(x802)))))))+(((IkReal(0.0644940225000000))*(x802)))+(((x799)*(((((IkReal(0.200300000000000))*(py)*(x802)))+(((cj1)*(sj1)*(x806))))))));
evalcond[2]=((((pp)*(x801)))+(((x798)*(((((IkReal(-1.00000000000000))*(x800)*(x807)))+(x808)))))+(((IkReal(0.0644940225000000))*(x801)))+(((sj1)*(x801)*(x803)))+(((x799)*(((((x800)*(x805)))+(x809))))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x810=(cj1)*(cj1);
IkReal x811=(sj1)*(sj1);
IkReal x812=((cj0)*(r01));
IkReal x813=((IkReal(134.055000000000))*(sj1));
IkReal x814=((px)*(sj0));
IkReal x815=((cj0)*(py));
IkReal x816=((IkReal(331.000000000000))*(pz));
IkReal x817=((r00)*(sj0));
IkReal x818=((IkReal(331.000000000000))*(r02)*(x811));
IkReal x819=((IkReal(331.000000000000))*(r02)*(x810));
gconst1=IKsign(((((x811)*(x812)*(x816)))+(((x812)*(x813)))+(((IkReal(-1.00000000000000))*(x815)*(x819)))+(((IkReal(-1.00000000000000))*(x815)*(x818)))+(((x810)*(x812)*(x816)))+(((IkReal(-1.00000000000000))*(x810)*(x816)*(x817)))+(((x814)*(x818)))+(((x814)*(x819)))+(((IkReal(-1.00000000000000))*(x811)*(x816)*(x817)))+(((IkReal(-1.00000000000000))*(x813)*(x817)))+(((IkReal(134.055000000000))*(cj1)*(r02)))));
IkReal x820=(cj1)*(cj1);
IkReal x821=(sj1)*(sj1);
IkReal x822=((IkReal(2.46913580246914))*(sj0));
IkReal x823=((px)*(r02));
IkReal x824=((pz)*(r00));
IkReal x825=((IkReal(2.46913580246914))*(cj0));
IkReal x826=((pz)*(r01));
IkReal x827=((py)*(r02));
dummyeval[0]=((((x820)*(x822)*(x823)))+(((x820)*(x825)*(x826)))+(((x821)*(x825)*(x826)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1.00000000000000))*(x821)*(x822)*(x824)))+(((cj1)*(r02)))+(((x821)*(x822)*(x823)))+(((cj0)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(x821)*(x825)*(x827)))+(((IkReal(-1.00000000000000))*(x820)*(x825)*(x827)))+(((IkReal(-1.00000000000000))*(x820)*(x822)*(x824))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst1)*(((((IkReal(-1000.00000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(r00)*(sj0)*((px)*(px))))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(px)*(py)*(r00)))+(((IkReal(-405.000000000000))*(py)*(r01)*((cj0)*(cj0))*((cj1)*(cj1))))+(((IkReal(-54.4640000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-100.150000000000))*(cj1)*(px)*(r01)*((sj0)*(sj0))))+(((IkReal(100.150000000000))*(cj1)*(py)*(r00)*((cj0)*(cj0))))+(((IkReal(-54.4640000000000))*(r02)*(sj1)))+(((IkReal(-1000.00000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-100.150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(r01)*((py)*(py))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r02)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(py)*(r01)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(py)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-100.150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-405.000000000000))*(pz)*(r02)*((sj1)*(sj1))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(pz)*(r01)*(sj1)))+(((IkReal(-100.150000000000))*(cj0)*(cj1)*(px)*(r00)*(sj0)))+(((IkReal(-405.000000000000))*(px)*(r00)))+(((IkReal(54.4640000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-405.000000000000))*(py)*(r01)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(cj1)*(pz)*(r00)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r02)*(sj1)))+(((IkReal(-405.000000000000))*(px)*(r00)*((cj1)*(cj1))*((sj0)*(sj0))))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))+(((IkReal(405.000000000000))*(cj0)*(px)*(r01)*(sj0)*((cj1)*(cj1)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(109.561000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(py)*(r00)*(sj1)*((cj0)*(cj0))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r00)*(sj1)*((sj0)*(sj0))))+(((IkReal(-109.561000000000))*(cj1)*(r02)))+(((IkReal(-405.000000000000))*(cj0)*(pz)*(r01)*((cj1)*(cj1))))+(((IkReal(1000.00000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r02)*((sj1)*(sj1))))+(((IkReal(-100.150000000000))*(cj0)*(px)*(r00)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(px)*(py)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(r01)*(sj1)*((py)*(py))))+(((IkReal(100.150000000000))*(cj0)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1000.00000000000))*(r00)*(sj0)*(sj1)*((px)*(px))))+(((IkReal(405.000000000000))*(cj1)*(pz)*(r02)*(sj1)))+(((IkReal(-109.561000000000))*(cj0)*(r01)*(sj1)))+(((IkReal(405.000000000000))*(pz)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-405.000000000000))*(px)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-405.000000000000))*(cj1)*(py)*(r01)*(sj1)*((cj0)*(cj0))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(-100.150000000000))*(px)*(r01)*(sj1)*((sj0)*(sj0))))+(((IkReal(1000.00000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(px)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(py)*(pz)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1000.00000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(r00)*(sj0)*((px)*(px))))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(px)*(py)*(r00)))+(((IkReal(-405.000000000000))*(py)*(r01)*((cj0)*(cj0))*((cj1)*(cj1))))+(((IkReal(-54.4640000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-100.150000000000))*(cj1)*(px)*(r01)*((sj0)*(sj0))))+(((IkReal(100.150000000000))*(cj1)*(py)*(r00)*((cj0)*(cj0))))+(((IkReal(-54.4640000000000))*(r02)*(sj1)))+(((IkReal(-1000.00000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-100.150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(r01)*((py)*(py))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r02)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(py)*(r01)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(py)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-100.150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-405.000000000000))*(pz)*(r02)*((sj1)*(sj1))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(pz)*(r01)*(sj1)))+(((IkReal(-100.150000000000))*(cj0)*(cj1)*(px)*(r00)*(sj0)))+(((IkReal(-405.000000000000))*(px)*(r00)))+(((IkReal(54.4640000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-405.000000000000))*(py)*(r01)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(cj1)*(pz)*(r00)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r02)*(sj1)))+(((IkReal(-405.000000000000))*(px)*(r00)*((cj1)*(cj1))*((sj0)*(sj0))))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))+(((IkReal(405.000000000000))*(cj0)*(px)*(r01)*(sj0)*((cj1)*(cj1))))))), ((gconst1)*(((((IkReal(109.561000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(py)*(r00)*(sj1)*((cj0)*(cj0))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r00)*(sj1)*((sj0)*(sj0))))+(((IkReal(-109.561000000000))*(cj1)*(r02)))+(((IkReal(-405.000000000000))*(cj0)*(pz)*(r01)*((cj1)*(cj1))))+(((IkReal(1000.00000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r02)*((sj1)*(sj1))))+(((IkReal(-100.150000000000))*(cj0)*(px)*(r00)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(px)*(py)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(r01)*(sj1)*((py)*(py))))+(((IkReal(100.150000000000))*(cj0)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1000.00000000000))*(r00)*(sj0)*(sj1)*((px)*(px))))+(((IkReal(405.000000000000))*(cj1)*(pz)*(r02)*(sj1)))+(((IkReal(-109.561000000000))*(cj0)*(r01)*(sj1)))+(((IkReal(405.000000000000))*(pz)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-405.000000000000))*(px)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-405.000000000000))*(cj1)*(py)*(r01)*(sj1)*((cj0)*(cj0))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(-100.150000000000))*(px)*(r01)*(sj1)*((sj0)*(sj0))))+(((IkReal(1000.00000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(px)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(py)*(pz)*(r01)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x828=IKsin(j2);
IkReal x829=IKcos(j2);
IkReal x830=((cj1)*(sj0));
IkReal x831=((IkReal(0.405000000000000))*(r00));
IkReal x832=((py)*(sj1));
IkReal x833=((cj0)*(cj1));
IkReal x834=((IkReal(0.662000000000000))*(sj1));
IkReal x835=((IkReal(1.00000000000000))*(r02));
IkReal x836=((IkReal(0.405000000000000))*(r01));
IkReal x837=((IkReal(0.331000000000000))*(r02));
IkReal x838=((IkReal(0.331000000000000))*(sj1));
IkReal x839=((IkReal(1.00000000000000))*(r01));
IkReal x840=((pz)*(sj1));
IkReal x841=((IkReal(0.100150000000000))*(r01));
IkReal x842=((IkReal(0.100150000000000))*(r02));
IkReal x843=((IkReal(0.100150000000000))*(r00));
IkReal x844=((r02)*(sj1));
IkReal x845=((sj0)*(x828));
IkReal x846=((r00)*(x829));
IkReal x847=((cj0)*(x828));
IkReal x848=((px)*(x829));
IkReal x849=((pz)*(x829));
IkReal x850=((py)*(x829));
IkReal x851=((cj1)*(x828));
IkReal x852=((sj1)*(x829));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(-1.00000000000000))*(x833)*(x850)))+(((x832)*(x847)))+(((x829)*(x840)))+(((x830)*(x848)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x845)))+(((IkReal(0.405000000000000))*(x829)))+(((pz)*(x851))));
evalcond[1]=((((r02)*(x832)*(x845)))+(((IkReal(-1.00000000000000))*(r00)*(x840)*(x847)))+(((IkReal(-1.00000000000000))*(x831)*(x847)))+(((x828)*(x830)*(x843)))+(((IkReal(-1.00000000000000))*(x830)*(x835)*(x850)))+(((IkReal(-1.00000000000000))*(x836)*(x845)))+(((pz)*(x833)*(x846)))+(((sj1)*(x828)*(x842)))+(((IkReal(-1.00000000000000))*(x833)*(x835)*(x848)))+(((x832)*(x846)))+(((IkReal(-1.00000000000000))*(sj1)*(x839)*(x848)))+(((IkReal(-1.00000000000000))*(px)*(x839)*(x851)))+(((IkReal(-1.00000000000000))*(x828)*(x833)*(x841)))+(((px)*(x844)*(x847)))+(((IkReal(-1.00000000000000))*(x839)*(x840)*(x845)))+(((py)*(r00)*(x851)))+(((IkReal(-1.00000000000000))*(cj0)*(x841)*(x852)))+(((IkReal(-1.00000000000000))*(cj1)*(x829)*(x842)))+(((r01)*(x830)*(x849)))+(((sj0)*(x843)*(x852))));
evalcond[2]=((((IkReal(-0.200300000000000))*(cj1)*(px)*(r01)*(IKcos(j2))))+(((IkReal(-0.662000000000000))*(pz)*(r02)))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(px)*(py)*(r00)*(IKcos(j2))))+(((cj1)*(pp)*(r00)*(sj0)*(IKcos(j2))))+(((IkReal(-0.0662993000000000))*(cj0)*(r00)))+(((IkReal(-0.200300000000000))*(py)*(r00)*(sj1)*(IKsin(j2))))+(((cj0)*(pp)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(-0.810000000000000))*(pz)*(r02)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(2.00000000000000))*(r00)*(sj0)*(sj1)*((px)*(px))*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(r00)*(sj0)*((px)*(px))*(IKcos(j2))))+(((IkReal(-0.200300000000000))*(cj0)*(pz)*(r00)*(sj1)*(IKcos(j2))))+(((IkReal(-0.0662993000000000))*(r01)*(sj0)))+(((IkReal(-0.810000000000000))*(px)*(r00)*(IKcos(j2))))+(((IkReal(-0.0644940225000000))*(r00)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(px)*(pz)*(r00)*(sj1)*(IKcos(j2))))+(((IkReal(0.200300000000000))*(cj0)*(px)*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(0.0644940225000000))*(cj0)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(px)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(-0.263555977500000))*(cj1)*(r00)*(sj0)*(IKcos(j2))))+(((IkReal(-0.810000000000000))*(pz)*(r00)*(sj0)*(IKsin(j2))))+(((IkReal(-0.0811215000000000))*(r01)*(sj0)*(IKcos(j2))))+(((IkReal(-0.810000000000000))*(cj0)*(py)*(r02)*(IKsin(j2))))+(((IkReal(-0.200300000000000))*(cj0)*(cj1)*(pz)*(r00)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r02)*(sj0)*(IKsin(j2))))+(((IkReal(-0.200300000000000))*(pz)*(r01)*(sj0)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(r00)*(sj1)*(IKsin(j2))))+(((IkReal(-0.662000000000000))*(py)*(r01)))+(((IkReal(-0.810000000000000))*(py)*(r01)*(IKcos(j2))))+(((IkReal(-1.00000000000000))*(cj0)*(cj1)*(pp)*(r01)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(r01)*(sj0)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(r01)*(sj1)*((py)*(py))*(IKsin(j2))))+(((IkReal(0.200300000000000))*(cj0)*(cj1)*(px)*(r02)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(py)*(r02)*(sj0)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(py)*(pz)*(r01)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(r02)*((pz)*(pz))*(IKsin(j2))))+(((IkReal(-0.268110000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-0.662000000000000))*(px)*(r00)))+(((IkReal(0.0644940225000000))*(cj1)*(r02)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(py)*(pz)*(r01)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)*(IKcos(j2))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r00)*(IKcos(j2))))+(((IkReal(0.263555977500000))*(cj0)*(cj1)*(r01)*(IKcos(j2))))+(((IkReal(0.810000000000000))*(cj0)*(pz)*(r01)*(IKsin(j2))))+(((IkReal(-0.268110000000000))*(r02)*(sj1)))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(r01)*((py)*(py))*(IKcos(j2))))+(((IkReal(0.268110000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-0.0811215000000000))*(cj0)*(r00)*(IKcos(j2))))+(((IkReal(-1.00000000000000))*(pp)*(r00)*(sj0)*(sj1)*(IKsin(j2))))+(((cj1)*(pp)*(r02)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(r02)*(sj1)*((pz)*(pz))*(IKcos(j2))))+(((IkReal(-0.200300000000000))*(cj1)*(pz)*(r01)*(sj0)*(IKsin(j2))))+(((IkReal(0.810000000000000))*(px)*(r02)*(sj0)*(IKsin(j2))))+(((pp)*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(-0.263555977500000))*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(px)*(py)*(r01)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r00)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)*(IKsin(j2)))));
evalcond[3]=((IkReal(0.283616022500000))+(((IkReal(0.662000000000000))*(x832)*(x847)))+(((IkReal(0.268110000000000))*(x829)))+(((IkReal(0.662000000000000))*(x830)*(x848)))+(((IkReal(0.810000000000000))*(px)*(x830)))+(((IkReal(-0.810000000000000))*(py)*(x833)))+(((IkReal(0.662000000000000))*(pz)*(x851)))+(((IkReal(0.810000000000000))*(x840)))+(((IkReal(-1.00000000000000))*(px)*(x834)*(x845)))+(pp)+(((IkReal(0.200300000000000))*(py)*(sj0)))+(((IkReal(0.200300000000000))*(cj0)*(px)))+(((x834)*(x849)))+(((IkReal(-0.662000000000000))*(x833)*(x850))));
evalcond[4]=((((IkReal(0.331000000000000))*(x830)*(x846)))+(((r01)*(x838)*(x847)))+(((px)*(r00)))+(((cj0)*(x843)))+(((IkReal(-1.00000000000000))*(x833)*(x836)))+(((IkReal(0.405000000000000))*(x844)))+(((x830)*(x831)))+(((IkReal(-0.331000000000000))*(r01)*(x829)*(x833)))+(((IkReal(-1.00000000000000))*(r00)*(x838)*(x845)))+(((pz)*(r02)))+(((py)*(r01)))+(((x837)*(x851)))+(((x837)*(x852)))+(((sj0)*(x841))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x853=((cj1)*(sj2));
IkReal x854=((cj2)*(sj1));
IkReal x855=((cj0)*(py));
IkReal x856=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x853)*(x856)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x854)*(x855)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x853)*(x855)))+(((IkReal(-1.00000000000000))*(x854)*(x856))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x857=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x853)*(x856)))+(((IkReal(-0.405000000000000))*(sj2)))+(((x854)*(x855)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))+(((x853)*(x855)))+(((IkReal(-1.00000000000000))*(x854)*(x856)))));
j3array[0]=((IkReal(-1.00000000000000))*(x857));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x857))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x858=IKsin(j3);
IkReal x859=IKcos(j3);
IkReal x860=((cj2)*(sj1));
IkReal x861=((cj1)*(sj2));
IkReal x862=((sj2)*(x859));
IkReal x863=((px)*(sj0)*(x859));
IkReal x864=((IkReal(1.00000000000000))*(cj0)*(py)*(x859));
evalcond[0]=((((IkReal(0.405000000000000))*(x862)))+(((pz)*(sj1)*(x862)))+(((IkReal(-1.00000000000000))*(x861)*(x864)))+(((cj0)*(px)*(x858)))+(((IkReal(-1.00000000000000))*(x860)*(x864)))+(((py)*(sj0)*(x858)))+(((x861)*(x863)))+(((x860)*(x863)))+(((IkReal(0.100150000000000))*(x858)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x859))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x865=((IkReal(1.00000000000000))*(sj3));
IkReal x866=((cj2)*(cj3));
IkReal x867=((cj0)*(r01));
IkReal x868=((cj3)*(sj2));
IkReal x869=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((IkReal(-1.00000000000000))*(cj0)*(r00)*(x865)))+(((IkReal(-1.00000000000000))*(sj1)*(x866)*(x869)))+(((cj1)*(r02)*(x866)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x865)))+(((cj1)*(x867)*(x868)))+(((sj1)*(x866)*(x867)))+(((IkReal(-1.00000000000000))*(cj1)*(x868)*(x869)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x868))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x870=((sj1)*(sj3));
IkReal x871=((IkReal(1.00000000000000))*(cj0));
IkReal x872=((r02)*(sj2));
IkReal x873=((IkReal(1.00000000000000))*(cj1));
IkReal x874=((cj2)*(r02));
IkReal x875=((sj1)*(sj2));
IkReal x876=((cj2)*(r01));
IkReal x877=((r00)*(sj0));
IkReal x878=((cj2)*(x877));
IkReal x879=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x872)*(x873)))+(((IkReal(-1.00000000000000))*(x873)*(x878)))+(((cj0)*(cj1)*(x876)))+(((x875)*(x877)))+(((IkReal(-1.00000000000000))*(r01)*(x871)*(x875)))+(((IkReal(-1.00000000000000))*(sj1)*(x874))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(sj3)*(x873)*(x874)))+(((x870)*(x872)))+(((x870)*(x878)))+(((IkReal(-1.00000000000000))*(x870)*(x871)*(x876)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x871)))+(((IkReal(-1.00000000000000))*(r01)*(x871)*(x879)))+(((x877)*(x879))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x872)*(x873)))+(((IkReal(-1.00000000000000))*(x873)*(x878)))+(((cj0)*(cj1)*(x876)))+(((x875)*(x877)))+(((IkReal(-1.00000000000000))*(r01)*(x871)*(x875)))+(((IkReal(-1.00000000000000))*(sj1)*(x874)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(sj3)*(x873)*(x874)))+(((x870)*(x872)))+(((x870)*(x878)))+(((IkReal(-1.00000000000000))*(x870)*(x871)*(x876)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x871)))+(((IkReal(-1.00000000000000))*(r01)*(x871)*(x879)))+(((x877)*(x879)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x880=((cj2)*(sj1));
IkReal x881=((cj1)*(sj2));
IkReal x882=((cj0)*(r01));
IkReal x883=((sj1)*(sj2));
IkReal x884=((r00)*(sj0));
IkReal x885=((cj1)*(cj2));
IkReal x886=((r02)*(sj3));
IkReal x887=((IkReal(1.00000000000000))*(sj3)*(x884));
evalcond[0]=((((r02)*(x881)))+(((r02)*(x880)))+(((IkReal(-1.00000000000000))*(x883)*(x884)))+(((IkReal(-1.00000000000000))*(x882)*(x885)))+(((x882)*(x883)))+(((sj5)*(IKsin(j4))))+(((x884)*(x885))));
evalcond[1]=((((cj3)*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(x883)*(x886)))+(((sj3)*(x880)*(x882)))+(((sj3)*(x881)*(x882)))+(((sj5)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(x881)*(x887)))+(((x885)*(x886)))+(((cj0)*(cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x880)*(x887))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x888=(px)*(px);
IkReal x889=((px)*(sj1));
IkReal x890=((IkReal(0.810000000000000))*(cj1));
IkReal x891=((sj1)*(x888));
IkReal x892=((((IkReal(0.200300000000000))*(x891)))+(((IkReal(-1.00000000000000))*(py)*(x889)*(x890))));
IkReal x893=((((IkReal(0.200300000000000))*(py)*(x889)))+(((x890)*(x891))));
if( IKabs(x892) < IKFAST_ATAN2_MAGTHRESH && IKabs(x893) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x894=((IkReal(1.00000000000000))*(IKatan2(x892, x893)));
if( ((((x892)*(x892))+((x893)*(x893)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x892)*(x892))+((x893)*(x893)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x892)*(x892))+((x893)*(x893))))))):(IkReal)1.0e30))*(((((IkReal(0.0644940225000000))*(x889)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x889)))+(((pp)*(x889))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x892)*(x892))+((x893)*(x893)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x892)*(x892))+((x893)*(x893))))))):(IkReal)1.0e30))*(((((IkReal(0.0644940225000000))*(x889)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x889)))+(((pp)*(x889))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x895=IKasin(((((IKabs(IKabs(IKsqrt((((x892)*(x892))+((x893)*(x893)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x892)*(x892))+((x893)*(x893))))))):(IkReal)1.0e30))*(((((IkReal(0.0644940225000000))*(x889)))+(((IkReal(0.810000000000000))*(pz)*(sj1)*(x889)))+(((pp)*(x889)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x894)))+(((IkReal(-1.00000000000000))*(x895))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x894)))+(x895));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x896=(cj1)*(cj1);
IkReal x897=(px)*(px);
IkReal x898=IKsin(j0);
IkReal x899=IKcos(j0);
IkReal x900=(py)*(py);
IkReal x901=((IkReal(0.200300000000000))*(sj1));
IkReal x902=((px)*(py));
IkReal x903=((cj1)*(py));
IkReal x904=((cj1)*(px));
IkReal x905=((IkReal(0.200300000000000))*(cj1));
IkReal x906=((py)*(sj1));
IkReal x907=((IkReal(1.00000000000000))*(pp));
IkReal x908=((IkReal(0.810000000000000))*(x896));
IkReal x909=((IkReal(0.810000000000000))*(pz)*(sj1));
IkReal x910=((IkReal(0.810000000000000))*(cj1)*(sj1));
IkReal x911=((x902)*(x905));
IkReal x912=((x902)*(x908));
evalcond[0]=((((x899)*(((((IkReal(-1.00000000000000))*(x897)*(x905)))+(x912)))))+(((x898)*(((((IkReal(-1.00000000000000))*(x897)*(x908)))+(((IkReal(-1.00000000000000))*(x911)))))))+(((IkReal(-1.00000000000000))*(x904)*(x907)))+(((IkReal(-1.00000000000000))*(x904)*(x909)))+(((IkReal(-0.0644940225000000))*(x904))));
evalcond[1]=((((x903)*(x909)))+(((pp)*(x903)))+(((x899)*(((((IkReal(-1.00000000000000))*(x900)*(x908)))+(x911)))))+(((x898)*(((((x900)*(x905)))+(x912)))))+(((IkReal(0.0644940225000000))*(x903))));
evalcond[2]=((((x899)*(((((x900)*(x910)))+(((IkReal(-1.00000000000000))*(x901)*(x902)))))))+(((x898)*(((((IkReal(-1.00000000000000))*(x900)*(x901)))+(((IkReal(-1.00000000000000))*(x902)*(x910)))))))+(((IkReal(-0.0644940225000000))*(x906)))+(((IkReal(-1.00000000000000))*(x906)*(x909)))+(((IkReal(-1.00000000000000))*(x906)*(x907))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x913=(cj1)*(cj1);
IkReal x914=(sj1)*(sj1);
IkReal x915=((cj0)*(r01));
IkReal x916=((IkReal(134.055000000000))*(sj1));
IkReal x917=((px)*(sj0));
IkReal x918=((cj0)*(py));
IkReal x919=((IkReal(331.000000000000))*(pz));
IkReal x920=((r00)*(sj0));
IkReal x921=((IkReal(331.000000000000))*(r02)*(x914));
IkReal x922=((IkReal(331.000000000000))*(r02)*(x913));
gconst1=IKsign(((((x913)*(x915)*(x919)))+(((IkReal(-1.00000000000000))*(x914)*(x919)*(x920)))+(((IkReal(-1.00000000000000))*(x918)*(x922)))+(((IkReal(-1.00000000000000))*(x918)*(x921)))+(((x917)*(x922)))+(((x917)*(x921)))+(((x915)*(x916)))+(((x914)*(x915)*(x919)))+(((IkReal(-1.00000000000000))*(x916)*(x920)))+(((IkReal(-1.00000000000000))*(x913)*(x919)*(x920)))+(((IkReal(134.055000000000))*(cj1)*(r02)))));
IkReal x923=(cj1)*(cj1);
IkReal x924=(sj1)*(sj1);
IkReal x925=((IkReal(2.46913580246914))*(sj0));
IkReal x926=((px)*(r02));
IkReal x927=((pz)*(r00));
IkReal x928=((IkReal(2.46913580246914))*(cj0));
IkReal x929=((pz)*(r01));
IkReal x930=((py)*(r02));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x923)*(x925)*(x927)))+(((x923)*(x928)*(x929)))+(((x924)*(x925)*(x926)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj1)))+(((cj1)*(r02)))+(((x924)*(x928)*(x929)))+(((cj0)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(x924)*(x925)*(x927)))+(((IkReal(-1.00000000000000))*(x923)*(x928)*(x930)))+(((IkReal(-1.00000000000000))*(x924)*(x928)*(x930)))+(((x923)*(x925)*(x926))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst1)*(((((IkReal(-1000.00000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(r00)*(sj0)*((px)*(px))))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(px)*(py)*(r00)))+(((IkReal(-405.000000000000))*(py)*(r01)*((cj0)*(cj0))*((cj1)*(cj1))))+(((IkReal(-54.4640000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-100.150000000000))*(cj1)*(px)*(r01)*((sj0)*(sj0))))+(((IkReal(100.150000000000))*(cj1)*(py)*(r00)*((cj0)*(cj0))))+(((IkReal(-54.4640000000000))*(r02)*(sj1)))+(((IkReal(-1000.00000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-100.150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(r01)*((py)*(py))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r02)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(py)*(r01)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(py)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-100.150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-405.000000000000))*(pz)*(r02)*((sj1)*(sj1))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(pz)*(r01)*(sj1)))+(((IkReal(-100.150000000000))*(cj0)*(cj1)*(px)*(r00)*(sj0)))+(((IkReal(-405.000000000000))*(px)*(r00)))+(((IkReal(54.4640000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-405.000000000000))*(py)*(r01)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(cj1)*(pz)*(r00)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r02)*(sj1)))+(((IkReal(-405.000000000000))*(px)*(r00)*((cj1)*(cj1))*((sj0)*(sj0))))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))+(((IkReal(405.000000000000))*(cj0)*(px)*(r01)*(sj0)*((cj1)*(cj1)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(109.561000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(py)*(r00)*(sj1)*((cj0)*(cj0))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r00)*(sj1)*((sj0)*(sj0))))+(((IkReal(-109.561000000000))*(cj1)*(r02)))+(((IkReal(-405.000000000000))*(cj0)*(pz)*(r01)*((cj1)*(cj1))))+(((IkReal(1000.00000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r02)*((sj1)*(sj1))))+(((IkReal(-100.150000000000))*(cj0)*(px)*(r00)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(px)*(py)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(r01)*(sj1)*((py)*(py))))+(((IkReal(100.150000000000))*(cj0)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1000.00000000000))*(r00)*(sj0)*(sj1)*((px)*(px))))+(((IkReal(405.000000000000))*(cj1)*(pz)*(r02)*(sj1)))+(((IkReal(-109.561000000000))*(cj0)*(r01)*(sj1)))+(((IkReal(405.000000000000))*(pz)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-405.000000000000))*(px)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-405.000000000000))*(cj1)*(py)*(r01)*(sj1)*((cj0)*(cj0))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(-100.150000000000))*(px)*(r01)*(sj1)*((sj0)*(sj0))))+(((IkReal(1000.00000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(px)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(py)*(pz)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1000.00000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(r00)*(sj0)*((px)*(px))))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(px)*(py)*(r00)))+(((IkReal(-405.000000000000))*(py)*(r01)*((cj0)*(cj0))*((cj1)*(cj1))))+(((IkReal(-54.4640000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-100.150000000000))*(cj1)*(px)*(r01)*((sj0)*(sj0))))+(((IkReal(100.150000000000))*(cj1)*(py)*(r00)*((cj0)*(cj0))))+(((IkReal(-54.4640000000000))*(r02)*(sj1)))+(((IkReal(-1000.00000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-100.150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(r01)*((py)*(py))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r02)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(py)*(r01)*(sj0)))+(((IkReal(-1000.00000000000))*(cj1)*(px)*(py)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-100.150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-405.000000000000))*(pz)*(r02)*((sj1)*(sj1))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(pz)*(r01)*(sj1)))+(((IkReal(-100.150000000000))*(cj0)*(cj1)*(px)*(r00)*(sj0)))+(((IkReal(-405.000000000000))*(px)*(r00)))+(((IkReal(54.4640000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-405.000000000000))*(py)*(r01)))+(((IkReal(-40.5607500000000))*(cj0)*(r00)))+(((IkReal(-405.000000000000))*(pz)*(r02)))+(((IkReal(-405.000000000000))*(cj1)*(pz)*(r00)*(sj0)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r02)*(sj1)))+(((IkReal(-405.000000000000))*(px)*(r00)*((cj1)*(cj1))*((sj0)*(sj0))))+(((IkReal(-40.5607500000000))*(r01)*(sj0)))+(((IkReal(405.000000000000))*(cj0)*(px)*(r01)*(sj0)*((cj1)*(cj1))))))), ((gconst1)*(((((IkReal(109.561000000000))*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(py)*(r00)*(sj1)*((cj0)*(cj0))))+(((IkReal(-405.000000000000))*(cj1)*(px)*(r00)*(sj1)*((sj0)*(sj0))))+(((IkReal(-109.561000000000))*(cj1)*(r02)))+(((IkReal(-405.000000000000))*(cj0)*(pz)*(r01)*((cj1)*(cj1))))+(((IkReal(1000.00000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)))+(((IkReal(405.000000000000))*(cj0)*(py)*(r02)*((sj1)*(sj1))))+(((IkReal(-100.150000000000))*(cj0)*(px)*(r00)*(sj0)*(sj1)))+(((IkReal(100.150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(1000.00000000000))*(cj0)*(px)*(py)*(r00)*(sj1)))+(((IkReal(1000.00000000000))*(cj0)*(r01)*(sj1)*((py)*(py))))+(((IkReal(100.150000000000))*(cj0)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1000.00000000000))*(r00)*(sj0)*(sj1)*((px)*(px))))+(((IkReal(405.000000000000))*(cj1)*(pz)*(r02)*(sj1)))+(((IkReal(-109.561000000000))*(cj0)*(r01)*(sj1)))+(((IkReal(405.000000000000))*(pz)*(r00)*(sj0)*((cj1)*(cj1))))+(((IkReal(100.150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-405.000000000000))*(px)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-405.000000000000))*(cj1)*(py)*(r01)*(sj1)*((cj0)*(cj0))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-1000.00000000000))*(px)*(py)*(r01)*(sj0)*(sj1)))+(((IkReal(-100.150000000000))*(px)*(r01)*(sj1)*((sj0)*(sj0))))+(((IkReal(1000.00000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(405.000000000000))*(cj0)*(cj1)*(px)*(r01)*(sj0)*(sj1)))+(((IkReal(1000.00000000000))*(cj1)*(py)*(pz)*(r01)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x931=IKsin(j2);
IkReal x932=IKcos(j2);
IkReal x933=((cj1)*(sj0));
IkReal x934=((IkReal(0.405000000000000))*(r00));
IkReal x935=((py)*(sj1));
IkReal x936=((cj0)*(cj1));
IkReal x937=((IkReal(0.662000000000000))*(sj1));
IkReal x938=((IkReal(1.00000000000000))*(r02));
IkReal x939=((IkReal(0.405000000000000))*(r01));
IkReal x940=((IkReal(0.331000000000000))*(r02));
IkReal x941=((IkReal(0.331000000000000))*(sj1));
IkReal x942=((IkReal(1.00000000000000))*(r01));
IkReal x943=((pz)*(sj1));
IkReal x944=((IkReal(0.100150000000000))*(r01));
IkReal x945=((IkReal(0.100150000000000))*(r02));
IkReal x946=((IkReal(0.100150000000000))*(r00));
IkReal x947=((r02)*(sj1));
IkReal x948=((sj0)*(x931));
IkReal x949=((r00)*(x932));
IkReal x950=((cj0)*(x931));
IkReal x951=((px)*(x932));
IkReal x952=((pz)*(x932));
IkReal x953=((py)*(x932));
IkReal x954=((cj1)*(x931));
IkReal x955=((sj1)*(x932));
evalcond[0]=((IkReal(0.331000000000000))+(((IkReal(0.405000000000000))*(x932)))+(((x932)*(x943)))+(((x933)*(x951)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x948)))+(((IkReal(-1.00000000000000))*(x936)*(x953)))+(((pz)*(x954)))+(((x935)*(x950))));
evalcond[1]=((((sj0)*(x946)*(x955)))+(((x931)*(x933)*(x946)))+(((pz)*(x936)*(x949)))+(((sj1)*(x931)*(x945)))+(((IkReal(-1.00000000000000))*(sj1)*(x942)*(x951)))+(((IkReal(-1.00000000000000))*(x931)*(x936)*(x944)))+(((IkReal(-1.00000000000000))*(x939)*(x948)))+(((px)*(x947)*(x950)))+(((IkReal(-1.00000000000000))*(x933)*(x938)*(x953)))+(((IkReal(-1.00000000000000))*(x942)*(x943)*(x948)))+(((IkReal(-1.00000000000000))*(x934)*(x950)))+(((py)*(r00)*(x954)))+(((IkReal(-1.00000000000000))*(cj0)*(x944)*(x955)))+(((IkReal(-1.00000000000000))*(r00)*(x943)*(x950)))+(((r01)*(x933)*(x952)))+(((x935)*(x949)))+(((IkReal(-1.00000000000000))*(cj1)*(x932)*(x945)))+(((r02)*(x935)*(x948)))+(((IkReal(-1.00000000000000))*(px)*(x942)*(x954)))+(((IkReal(-1.00000000000000))*(x936)*(x938)*(x951))));
evalcond[2]=((((IkReal(-0.200300000000000))*(cj1)*(px)*(r01)*(IKcos(j2))))+(((IkReal(-0.662000000000000))*(pz)*(r02)))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(px)*(py)*(r00)*(IKcos(j2))))+(((cj1)*(pp)*(r00)*(sj0)*(IKcos(j2))))+(((IkReal(-0.0662993000000000))*(cj0)*(r00)))+(((IkReal(-0.200300000000000))*(py)*(r00)*(sj1)*(IKsin(j2))))+(((cj0)*(pp)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(-0.810000000000000))*(pz)*(r02)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(px)*(pz)*(r02)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(2.00000000000000))*(r00)*(sj0)*(sj1)*((px)*(px))*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(r00)*(sj0)*((px)*(px))*(IKcos(j2))))+(((IkReal(-0.200300000000000))*(cj0)*(pz)*(r00)*(sj1)*(IKcos(j2))))+(((IkReal(-0.0662993000000000))*(r01)*(sj0)))+(((IkReal(-0.810000000000000))*(px)*(r00)*(IKcos(j2))))+(((IkReal(-0.0644940225000000))*(r00)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(px)*(pz)*(r00)*(sj1)*(IKcos(j2))))+(((IkReal(0.200300000000000))*(cj0)*(px)*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(0.0644940225000000))*(cj0)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(px)*(r01)*(sj1)*(IKsin(j2))))+(((IkReal(-0.263555977500000))*(cj1)*(r00)*(sj0)*(IKcos(j2))))+(((IkReal(-0.810000000000000))*(pz)*(r00)*(sj0)*(IKsin(j2))))+(((IkReal(-0.0811215000000000))*(r01)*(sj0)*(IKcos(j2))))+(((IkReal(-0.810000000000000))*(cj0)*(py)*(r02)*(IKsin(j2))))+(((IkReal(-0.200300000000000))*(cj0)*(cj1)*(pz)*(r00)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r02)*(sj0)*(IKsin(j2))))+(((IkReal(-0.200300000000000))*(pz)*(r01)*(sj0)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(r00)*(sj1)*(IKsin(j2))))+(((IkReal(-0.662000000000000))*(py)*(r01)))+(((IkReal(-0.810000000000000))*(py)*(r01)*(IKcos(j2))))+(((IkReal(-1.00000000000000))*(cj0)*(cj1)*(pp)*(r01)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(py)*(r01)*(sj0)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(py)*(pz)*(r02)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(r01)*(sj1)*((py)*(py))*(IKsin(j2))))+(((IkReal(0.200300000000000))*(cj0)*(cj1)*(px)*(r02)*(IKsin(j2))))+(((IkReal(0.200300000000000))*(py)*(r02)*(sj0)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(py)*(pz)*(r01)*(sj1)*(IKcos(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(r02)*((pz)*(pz))*(IKsin(j2))))+(((IkReal(-0.268110000000000))*(cj1)*(r00)*(sj0)))+(((IkReal(-0.662000000000000))*(px)*(r00)))+(((IkReal(0.0644940225000000))*(cj1)*(r02)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(py)*(pz)*(r01)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r02)*(sj0)*(IKcos(j2))))+(((IkReal(0.200300000000000))*(cj1)*(py)*(r00)*(IKcos(j2))))+(((IkReal(0.263555977500000))*(cj0)*(cj1)*(r01)*(IKcos(j2))))+(((IkReal(0.810000000000000))*(cj0)*(pz)*(r01)*(IKsin(j2))))+(((IkReal(-0.268110000000000))*(r02)*(sj1)))+(((IkReal(2.00000000000000))*(cj0)*(cj1)*(r01)*((py)*(py))*(IKcos(j2))))+(((IkReal(0.268110000000000))*(cj0)*(cj1)*(r01)))+(((IkReal(-0.0811215000000000))*(cj0)*(r00)*(IKcos(j2))))+(((IkReal(-1.00000000000000))*(pp)*(r00)*(sj0)*(sj1)*(IKsin(j2))))+(((cj1)*(pp)*(r02)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(r02)*(sj1)*((pz)*(pz))*(IKcos(j2))))+(((IkReal(-0.200300000000000))*(cj1)*(pz)*(r01)*(sj0)*(IKsin(j2))))+(((IkReal(0.810000000000000))*(px)*(r02)*(sj0)*(IKsin(j2))))+(((pp)*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(-0.263555977500000))*(r02)*(sj1)*(IKcos(j2))))+(((IkReal(2.00000000000000))*(px)*(py)*(r01)*(sj0)*(sj1)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj1)*(px)*(pz)*(r00)*(IKsin(j2))))+(((IkReal(-2.00000000000000))*(cj0)*(py)*(pz)*(r02)*(sj1)*(IKsin(j2)))));
evalcond[3]=((IkReal(0.283616022500000))+(((IkReal(0.810000000000000))*(x943)))+(((IkReal(0.662000000000000))*(x935)*(x950)))+(((IkReal(0.268110000000000))*(x932)))+(((IkReal(0.662000000000000))*(x933)*(x951)))+(((IkReal(-0.662000000000000))*(x936)*(x953)))+(((IkReal(0.810000000000000))*(px)*(x933)))+(pp)+(((IkReal(0.200300000000000))*(py)*(sj0)))+(((IkReal(0.200300000000000))*(cj0)*(px)))+(((IkReal(-0.810000000000000))*(py)*(x936)))+(((IkReal(-1.00000000000000))*(px)*(x937)*(x948)))+(((IkReal(0.662000000000000))*(pz)*(x954)))+(((x937)*(x952))));
evalcond[4]=((((cj0)*(x946)))+(((IkReal(0.405000000000000))*(x947)))+(((r01)*(x941)*(x950)))+(((px)*(r00)))+(((x940)*(x954)))+(((x940)*(x955)))+(((sj0)*(x944)))+(((IkReal(0.331000000000000))*(x933)*(x949)))+(((IkReal(-0.331000000000000))*(r01)*(x932)*(x936)))+(((IkReal(-1.00000000000000))*(r00)*(x941)*(x948)))+(((IkReal(-1.00000000000000))*(x936)*(x939)))+(((pz)*(r02)))+(((py)*(r01)))+(((x933)*(x934))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x956=((cj1)*(sj2));
IkReal x957=((cj2)*(sj1));
IkReal x958=((cj0)*(py));
IkReal x959=((IkReal(1.00000000000000))*(px)*(sj0));
if( IKabs(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(cj2)*(pz)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x956)*(x959)))+(((IkReal(-1.00000000000000))*(x957)*(x959)))+(((x956)*(x958)))+(((x957)*(x958)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x960=IKatan2(((IkReal(0.100150000000000))+(((cj0)*(px)))+(((py)*(sj0)))), ((((cj1)*(cj2)*(pz)))+(((IkReal(-0.405000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x956)*(x959)))+(((IkReal(-1.00000000000000))*(x957)*(x959)))+(((x956)*(x958)))+(((x957)*(x958)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)*(sj2)))));
j3array[0]=((IkReal(-1.00000000000000))*(x960));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x960))));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[1];
IkReal x961=IKsin(j3);
IkReal x962=IKcos(j3);
IkReal x963=((cj2)*(sj1));
IkReal x964=((cj1)*(sj2));
IkReal x965=((sj2)*(x962));
IkReal x966=((px)*(sj0)*(x962));
IkReal x967=((IkReal(1.00000000000000))*(cj0)*(py)*(x962));
evalcond[0]=((((py)*(sj0)*(x961)))+(((IkReal(0.100150000000000))*(x961)))+(((IkReal(-1.00000000000000))*(x963)*(x967)))+(((cj0)*(px)*(x961)))+(((pz)*(sj1)*(x965)))+(((IkReal(-1.00000000000000))*(x964)*(x967)))+(((IkReal(0.405000000000000))*(x965)))+(((x964)*(x966)))+(((x963)*(x966)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)*(x962))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x968=((IkReal(1.00000000000000))*(sj3));
IkReal x969=((cj2)*(cj3));
IkReal x970=((cj0)*(r01));
IkReal x971=((cj3)*(sj2));
IkReal x972=((IkReal(1.00000000000000))*(r00)*(sj0));
cj5array[0]=((((IkReal(-1.00000000000000))*(r01)*(sj0)*(x968)))+(((sj1)*(x969)*(x970)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x968)))+(((cj1)*(x970)*(x971)))+(((IkReal(-1.00000000000000))*(cj1)*(x971)*(x972)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x971)))+(((cj1)*(r02)*(x969)))+(((IkReal(-1.00000000000000))*(sj1)*(x969)*(x972))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x973=((sj1)*(sj3));
IkReal x974=((IkReal(1.00000000000000))*(cj0));
IkReal x975=((r02)*(sj2));
IkReal x976=((IkReal(1.00000000000000))*(cj1));
IkReal x977=((cj2)*(r02));
IkReal x978=((sj1)*(sj2));
IkReal x979=((cj2)*(r01));
IkReal x980=((r00)*(sj0));
IkReal x981=((cj2)*(x980));
IkReal x982=((cj1)*(sj2)*(sj3));
if( IKabs(((gconst4)*(((((cj0)*(cj1)*(x979)))+(((IkReal(-1.00000000000000))*(sj1)*(x977)))+(((IkReal(-1.00000000000000))*(r01)*(x974)*(x978)))+(((IkReal(-1.00000000000000))*(x976)*(x981)))+(((IkReal(-1.00000000000000))*(x975)*(x976)))+(((x978)*(x980))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(r00)*(x974)))+(((IkReal(-1.00000000000000))*(r01)*(x974)*(x982)))+(((IkReal(-1.00000000000000))*(sj3)*(x976)*(x977)))+(((IkReal(-1.00000000000000))*(x973)*(x974)*(x979)))+(((x973)*(x981)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x973)*(x975)))+(((x980)*(x982))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst4)*(((((cj0)*(cj1)*(x979)))+(((IkReal(-1.00000000000000))*(sj1)*(x977)))+(((IkReal(-1.00000000000000))*(r01)*(x974)*(x978)))+(((IkReal(-1.00000000000000))*(x976)*(x981)))+(((IkReal(-1.00000000000000))*(x975)*(x976)))+(((x978)*(x980)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(r00)*(x974)))+(((IkReal(-1.00000000000000))*(r01)*(x974)*(x982)))+(((IkReal(-1.00000000000000))*(sj3)*(x976)*(x977)))+(((IkReal(-1.00000000000000))*(x973)*(x974)*(x979)))+(((x973)*(x981)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(sj0)))+(((x973)*(x975)))+(((x980)*(x982)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[2];
IkReal x983=((cj2)*(sj1));
IkReal x984=((cj1)*(sj2));
IkReal x985=((cj0)*(r01));
IkReal x986=((sj1)*(sj2));
IkReal x987=((r00)*(sj0));
IkReal x988=((cj1)*(cj2));
IkReal x989=((r02)*(sj3));
IkReal x990=((IkReal(1.00000000000000))*(sj3)*(x987));
evalcond[0]=((((IkReal(-1.00000000000000))*(x986)*(x987)))+(((sj5)*(IKsin(j4))))+(((x987)*(x988)))+(((x985)*(x986)))+(((r02)*(x984)))+(((r02)*(x983)))+(((IkReal(-1.00000000000000))*(x985)*(x988))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x986)*(x989)))+(((cj3)*(r01)*(sj0)))+(((x988)*(x989)))+(((sj5)*(IKcos(j4))))+(((sj3)*(x983)*(x985)))+(((IkReal(-1.00000000000000))*(x984)*(x990)))+(((IkReal(-1.00000000000000))*(x983)*(x990)))+(((sj3)*(x984)*(x985)))+(((cj0)*(cj3)*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}
}
}
}
}

}

}
}
}

}

}

} else
{
continue;

}

}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
