/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:18:38.819724
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[2]);
x3=IKcos(j[2]);
x4=IKsin(j[1]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[0]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKcos(j[5]);
x11=IKsin(j[5]);
x12=((IkReal(1.54500000000000))*(x7));
x13=((IkReal(0.158000000000000))*(x6));
x14=((IkReal(0.158000000000000))*(x0));
x15=((IkReal(0.145000000000000))*(x0));
x16=((IkReal(1.00000000000000))*(x9));
x17=((IkReal(1.54500000000000))*(x0));
x18=((IkReal(2.16000000000000))*(x6));
x19=((IkReal(1.00000000000000))*(x7));
x20=((IkReal(1.00000000000000))*(x8));
x21=((IkReal(1.00000000000000))*(x0));
x22=((IkReal(2.16000000000000))*(x9));
x23=((IkReal(0.158000000000000))*(x7));
x24=((x0)*(x5));
x25=((x11)*(x5));
x26=((x2)*(x4));
x27=((x1)*(x3));
x28=((x4)*(x7));
x29=((x3)*(x4));
x30=((x1)*(x2));
x31=((x5)*(x7));
x32=((((IkReal(-1.00000000000000))*(x27)))+(x26));
x33=((((IkReal(1.00000000000000))*(x29)))+(((IkReal(1.00000000000000))*(x30))));
x34=((x21)*(((((IkReal(-1.00000000000000))*(x27)))+(x26))));
x35=((x19)*(((((IkReal(-1.00000000000000))*(x27)))+(x26))));
x36=((x21)*(((x30)+(x29))));
x37=((IkReal(-1.00000000000000))*(x36));
x38=((x19)*(((((IkReal(-1.00000000000000))*(x30)))+(((IkReal(-1.00000000000000))*(x29))))));
x39=((x19)*(((x30)+(x29))));
x40=((x34)*(x9));
x41=((x38)*(x6));
x42=((x39)*(x5));
x43=((x36)*(x5));
x44=((x31)+(((x37)*(x6))));
x45=((x44)*(x8));
IkReal x46=((IkReal(1.00000000000000))*(x15));
IkReal x47=((IkReal(1.00000000000000))*(x26));
eetrans[0]=((((IkReal(-1.00000000000000))*(x29)*(x46)))+(((IkReal(-1.20000000000000))*(x0)*(x4)))+(((IkReal(0.350000000000000))*(x0)))+(((x10)*(((((IkReal(-2.16000000000000))*(x45)))+(((IkReal(-2.16000000000000))*(x40)))))))+(((IkReal(-1.00000000000000))*(x30)*(x46)))+(((x8)*(((((x14)*(x27)))+(((IkReal(-1.00000000000000))*(x14)*(x47)))))))+(((x17)*(x27)))+(((x9)*(((((x23)*(x5)))+(((x13)*(x37)))))))+(((IkReal(-1.00000000000000))*(x17)*(x47)))+(((x11)*(((((IkReal(-1.00000000000000))*(x18)*(x7)))+(((IkReal(-2.16000000000000))*(x43))))))));
IkReal x48=((IkReal(1.00000000000000))*(x27));
eetrans[1]=((((x11)*(((((IkReal(2.16000000000000))*(x42)))+(((IkReal(-1.00000000000000))*(x0)*(x18)))))))+(((x10)*(((((x22)*(x35)))+(((IkReal(2.16000000000000))*(x8)*(((((IkReal(-1.00000000000000))*(x24)))+(x41)))))))))+(((x9)*(((((x14)*(x5)))+(((IkReal(-1.00000000000000))*(x13)*(x38)))))))+(((IkReal(-0.350000000000000))*(x7)))+(((IkReal(1.20000000000000))*(x28)))+(((IkReal(0.145000000000000))*(x28)*(x3)))+(((x8)*(((((IkReal(-1.00000000000000))*(x23)*(x48)))+(((x23)*(x26)))))))+(((x12)*(x26)))+(((IkReal(-1.00000000000000))*(x12)*(x48)))+(((IkReal(0.145000000000000))*(x30)*(x7))));
eetrans[2]=((IkReal(0.815000000000000))+(((x10)*(((((x18)*(x32)*(x8)))+(((x22)*(x33)))))))+(((x6)*(x9)*(((((IkReal(0.158000000000000))*(x27)))+(((IkReal(-0.158000000000000))*(x26)))))))+(((IkReal(1.54500000000000))*(x30)))+(((IkReal(1.20000000000000))*(x1)))+(((x8)*(((((IkReal(0.158000000000000))*(x29)))+(((IkReal(0.158000000000000))*(x30)))))))+(((IkReal(-0.145000000000000))*(x26)))+(((IkReal(1.54500000000000))*(x29)))+(((x25)*(((((IkReal(2.16000000000000))*(x27)))+(((IkReal(-2.16000000000000))*(x26)))))))+(((IkReal(0.145000000000000))*(x27))));
eerot[0]=((((x10)*(((x45)+(x40)))))+(((x11)*(((x43)+(((x6)*(x7))))))));
eerot[1]=((((IkReal(-1.00000000000000))*(x10)*(((((x16)*(x35)))+(((x20)*(((((IkReal(-1.00000000000000))*(x21)*(x5)))+(x41)))))))))+(((x11)*(((((IkReal(-1.00000000000000))*(x42)))+(((x21)*(x6))))))));
eerot[2]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x32)*(x6)))+(((IkReal(-1.00000000000000))*(x16)*(x33)))))))+(((x25)*(x32))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j1,cj1,sj1,htj1,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((((IkReal(-2.16000000000000))*(r00)))+(px));
new_r01=((IkReal(-1.00000000000000))*(r01));
new_py=((((IkReal(2.16000000000000))*(r01)))+(((IkReal(-1.00000000000000))*(py))));
new_r02=((IkReal(-1.00000000000000))*(r02));
new_pz=((IkReal(0.815000000000000))+(((IkReal(2.16000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(pz))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(1.54500000000000))*(r02)*(sj1)))+(((IkReal(1.34500000000000))*(r00)*(sj1)))+(((IkReal(-1.54500000000000))*(cj1)*(r00)))+(((IkReal(-0.350000000000000))*(r00)))+(((IkReal(1.34500000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst1;
gconst1=((((IkReal(-0.290000000000000))*(r02)*(sj1)))+(((IkReal(3.09000000000000))*(r00)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r02)))+(((IkReal(0.290000000000000))*(cj1)*(r00))));
IkReal gconst2;
gconst2=((IkReal(2.16000000000000))+(((IkReal(1.54500000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(1.05500000000000))*(cj1)*(r02)))+(((IkReal(-1.54500000000000))*(r02)*(sj1)))+(((IkReal(1.05500000000000))*(r00)*(sj1)))+(((IkReal(-0.350000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst3;
gconst3=((IkReal(-0.372014000000000))+(((IkReal(-0.700000000000000))*(px)))+(((IkReal(2.69000000000000))*(cj1)*(pz)))+(((IkReal(1.08150000000000))*(cj1)))+(((IkReal(-0.941500000000000))*(sj1)))+(((IkReal(2.69000000000000))*(px)*(sj1)))+(pp)+(((IkReal(-3.09000000000000))*(cj1)*(px)))+(((IkReal(3.09000000000000))*(pz)*(sj1))));
IkReal gconst4;
gconst4=((IkReal(7.41600000000000))+(((IkReal(-0.203000000000000))*(cj1)))+(((IkReal(-2.16300000000000))*(sj1)))+(((IkReal(-0.580000000000000))*(pz)*(sj1)))+(((IkReal(6.18000000000000))*(px)*(sj1)))+(((IkReal(0.580000000000000))*(cj1)*(px)))+(((IkReal(6.18000000000000))*(cj1)*(pz))));
IkReal gconst5;
gconst5=((IkReal(-1.06801400000000))+(((IkReal(-0.700000000000000))*(px)))+(((IkReal(2.11000000000000))*(px)*(sj1)))+(((IkReal(2.11000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(-3.09000000000000))*(pz)*(sj1)))+(((IkReal(-1.08150000000000))*(cj1)))+(((IkReal(-0.738500000000000))*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(px))));
IkReal gconst6;
gconst6=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(1.54500000000000))*(r02)*(sj1)))+(((IkReal(1.34500000000000))*(r00)*(sj1)))+(((IkReal(-1.54500000000000))*(cj1)*(r00)))+(((IkReal(-0.350000000000000))*(r00)))+(((IkReal(1.34500000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst7;
gconst7=((((IkReal(-0.290000000000000))*(r02)*(sj1)))+(((IkReal(3.09000000000000))*(r00)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r02)))+(((IkReal(0.290000000000000))*(cj1)*(r00))));
IkReal gconst8;
gconst8=((IkReal(2.16000000000000))+(((IkReal(1.54500000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(1.05500000000000))*(cj1)*(r02)))+(((IkReal(-1.54500000000000))*(r02)*(sj1)))+(((IkReal(1.05500000000000))*(r00)*(sj1)))+(((IkReal(-0.350000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst9;
gconst9=((IkReal(-0.372014000000000))+(((IkReal(-0.700000000000000))*(px)))+(((IkReal(2.69000000000000))*(cj1)*(pz)))+(((IkReal(1.08150000000000))*(cj1)))+(((IkReal(-0.941500000000000))*(sj1)))+(((IkReal(2.69000000000000))*(px)*(sj1)))+(pp)+(((IkReal(-3.09000000000000))*(cj1)*(px)))+(((IkReal(3.09000000000000))*(pz)*(sj1))));
IkReal gconst10;
gconst10=((IkReal(7.41600000000000))+(((IkReal(-0.203000000000000))*(cj1)))+(((IkReal(-2.16300000000000))*(sj1)))+(((IkReal(-0.580000000000000))*(pz)*(sj1)))+(((IkReal(6.18000000000000))*(px)*(sj1)))+(((IkReal(0.580000000000000))*(cj1)*(px)))+(((IkReal(6.18000000000000))*(cj1)*(pz))));
IkReal gconst11;
gconst11=((IkReal(-1.06801400000000))+(((IkReal(-0.700000000000000))*(px)))+(((IkReal(2.11000000000000))*(px)*(sj1)))+(((IkReal(2.11000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(-3.09000000000000))*(pz)*(sj1)))+(((IkReal(-1.08150000000000))*(cj1)))+(((IkReal(-0.738500000000000))*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(px))));
IkReal gconst12;
gconst12=((((IkReal(-0.700000000000000))*(r01)))+(((IkReal(2.69000000000000))*(r01)*(sj1)))+(((IkReal(-3.09000000000000))*(cj1)*(r01))));
IkReal gconst13;
gconst13=((((IkReal(0.580000000000000))*(cj1)*(r01)))+(((IkReal(6.18000000000000))*(r01)*(sj1))));
IkReal gconst14;
gconst14=((((IkReal(-0.700000000000000))*(r01)))+(((IkReal(2.11000000000000))*(r01)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r01))));
IkReal gconst15;
gconst15=((((IkReal(-1.40000000000000))*(py)))+(((IkReal(5.38000000000000))*(py)*(sj1)))+(((IkReal(-6.18000000000000))*(cj1)*(py))));
IkReal gconst16;
gconst16=((((IkReal(12.3600000000000))*(py)*(sj1)))+(((IkReal(1.16000000000000))*(cj1)*(py))));
IkReal gconst17;
gconst17=((((IkReal(-1.40000000000000))*(py)))+(((IkReal(6.18000000000000))*(cj1)*(py)))+(((IkReal(4.22000000000000))*(py)*(sj1))));
IkReal gconst18;
gconst18=((((IkReal(-0.700000000000000))*(r01)))+(((IkReal(2.69000000000000))*(r01)*(sj1)))+(((IkReal(-3.09000000000000))*(cj1)*(r01))));
IkReal gconst19;
gconst19=((((IkReal(0.580000000000000))*(cj1)*(r01)))+(((IkReal(6.18000000000000))*(r01)*(sj1))));
IkReal gconst20;
gconst20=((((IkReal(-0.700000000000000))*(r01)))+(((IkReal(2.11000000000000))*(r01)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r01))));
IkReal gconst21;
gconst21=((((IkReal(-1.40000000000000))*(py)))+(((IkReal(5.38000000000000))*(py)*(sj1)))+(((IkReal(-6.18000000000000))*(cj1)*(py))));
IkReal gconst22;
gconst22=((((IkReal(12.3600000000000))*(py)*(sj1)))+(((IkReal(1.16000000000000))*(cj1)*(py))));
IkReal gconst23;
gconst23=((((IkReal(-1.40000000000000))*(py)))+(((IkReal(6.18000000000000))*(cj1)*(py)))+(((IkReal(4.22000000000000))*(py)*(sj1))));
IkReal gconst24;
gconst24=((IkReal(2.16000000000000))+(((IkReal(1.54500000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(1.54500000000000))*(r02)*(sj1)))+(((IkReal(0.350000000000000))*(r00)))+(((IkReal(-1.34500000000000))*(r00)*(sj1)))+(((IkReal(1.34500000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst25;
gconst25=((((IkReal(-0.290000000000000))*(cj1)*(r00)))+(((IkReal(-0.290000000000000))*(r02)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r02)))+(((IkReal(-3.09000000000000))*(r00)*(sj1))));
IkReal gconst26;
gconst26=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(-1.54500000000000))*(cj1)*(r00)))+(((IkReal(1.05500000000000))*(cj1)*(r02)))+(((IkReal(-1.54500000000000))*(r02)*(sj1)))+(((IkReal(0.350000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-1.05500000000000))*(r00)*(sj1))));
IkReal gconst27;
gconst27=((IkReal(-0.372014000000000))+(((IkReal(0.700000000000000))*(px)))+(((IkReal(2.69000000000000))*(cj1)*(pz)))+(((IkReal(1.08150000000000))*(cj1)))+(((IkReal(-0.941500000000000))*(sj1)))+(pp)+(((IkReal(-2.69000000000000))*(px)*(sj1)))+(((IkReal(3.09000000000000))*(pz)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(px))));
IkReal gconst28;
gconst28=((IkReal(7.41600000000000))+(((IkReal(-0.203000000000000))*(cj1)))+(((IkReal(-0.580000000000000))*(cj1)*(px)))+(((IkReal(-6.18000000000000))*(px)*(sj1)))+(((IkReal(-2.16300000000000))*(sj1)))+(((IkReal(-0.580000000000000))*(pz)*(sj1)))+(((IkReal(6.18000000000000))*(cj1)*(pz))));
IkReal gconst29;
gconst29=((IkReal(-1.06801400000000))+(((IkReal(-2.11000000000000))*(px)*(sj1)))+(((IkReal(0.700000000000000))*(px)))+(((IkReal(2.11000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(-3.09000000000000))*(pz)*(sj1)))+(((IkReal(-3.09000000000000))*(cj1)*(px)))+(((IkReal(-1.08150000000000))*(cj1)))+(((IkReal(-0.738500000000000))*(sj1))));
IkReal gconst30;
gconst30=((IkReal(2.16000000000000))+(((IkReal(1.54500000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(1.54500000000000))*(r02)*(sj1)))+(((IkReal(0.350000000000000))*(r00)))+(((IkReal(-1.34500000000000))*(r00)*(sj1)))+(((IkReal(1.34500000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst31;
gconst31=((((IkReal(-0.290000000000000))*(cj1)*(r00)))+(((IkReal(-0.290000000000000))*(r02)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r02)))+(((IkReal(-3.09000000000000))*(r00)*(sj1))));
IkReal gconst32;
gconst32=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(-1.54500000000000))*(cj1)*(r00)))+(((IkReal(1.05500000000000))*(cj1)*(r02)))+(((IkReal(-1.54500000000000))*(r02)*(sj1)))+(((IkReal(0.350000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-1.05500000000000))*(r00)*(sj1))));
IkReal gconst33;
gconst33=((IkReal(-0.372014000000000))+(((IkReal(0.700000000000000))*(px)))+(((IkReal(2.69000000000000))*(cj1)*(pz)))+(((IkReal(1.08150000000000))*(cj1)))+(((IkReal(-0.941500000000000))*(sj1)))+(pp)+(((IkReal(-2.69000000000000))*(px)*(sj1)))+(((IkReal(3.09000000000000))*(pz)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(px))));
IkReal gconst34;
gconst34=((IkReal(7.41600000000000))+(((IkReal(-0.203000000000000))*(cj1)))+(((IkReal(-0.580000000000000))*(cj1)*(px)))+(((IkReal(-6.18000000000000))*(px)*(sj1)))+(((IkReal(-2.16300000000000))*(sj1)))+(((IkReal(-0.580000000000000))*(pz)*(sj1)))+(((IkReal(6.18000000000000))*(cj1)*(pz))));
IkReal gconst35;
gconst35=((IkReal(-1.06801400000000))+(((IkReal(-2.11000000000000))*(px)*(sj1)))+(((IkReal(0.700000000000000))*(px)))+(((IkReal(2.11000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(-3.09000000000000))*(pz)*(sj1)))+(((IkReal(-3.09000000000000))*(cj1)*(px)))+(((IkReal(-1.08150000000000))*(cj1)))+(((IkReal(-0.738500000000000))*(sj1))));
IkReal x49=((IkReal(1.00000000000000))*(gconst26));
IkReal x50=((gconst30)*(gconst34));
IkReal x51=((gconst31)*(gconst33));
IkReal x52=((IkReal(1.00000000000000))*(gconst17));
IkReal x53=((gconst2)*(gconst6));
IkReal x54=((gconst11)*(gconst3));
IkReal x55=((gconst25)*(gconst29));
IkReal x56=((gconst14)*(gconst28));
IkReal x57=((IkReal(1.00000000000000))*(gconst14));
IkReal x58=((gconst21)*(gconst32));
IkReal x59=((gconst10)*(gconst6));
IkReal x60=((gconst21)*(gconst5));
IkReal x61=((gconst0)*(gconst8));
IkReal x62=((gconst13)*(gconst5));
IkReal x63=((IkReal(1.00000000000000))*(gconst9));
IkReal x64=((gconst30)*(gconst35));
IkReal x65=((gconst26)*(gconst27));
IkReal x66=((gconst22)*(gconst30));
IkReal x67=((IkReal(1.00000000000000))*(gconst12));
IkReal x68=((gconst2)*(gconst3));
IkReal x69=((gconst1)*(gconst5));
IkReal x70=((gconst19)*(gconst33));
IkReal x71=((gconst26)*(gconst28));
IkReal x72=((gconst11)*(gconst6));
IkReal x73=((gconst1)*(gconst7));
IkReal x74=((gconst18)*(gconst2));
IkReal x75=((gconst18)*(gconst34));
IkReal x76=((gconst24)*(gconst29));
IkReal x77=((IkReal(1.00000000000000))*(gconst23));
IkReal x78=((gconst20)*(gconst33));
IkReal x79=((gconst32)*(gconst33));
IkReal x80=((gconst0)*(gconst5));
IkReal x81=((IkReal(1.00000000000000))*(gconst18));
IkReal x82=((gconst21)*(gconst31));
IkReal x83=((gconst5)*(gconst9));
IkReal x84=((gconst4)*(gconst9));
IkReal x85=((gconst13)*(gconst29));
IkReal x86=((gconst14)*(gconst6));
IkReal x87=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x88=((gconst27)*(x79));
IkReal x89=((gconst2)*(gconst4)*(gconst7));
op[0]=((((IkReal(-1.00000000000000))*(x51)*(x55)))+(((x50)*(x55)))+(((x76)*(x79)))+(((IkReal(-1.00000000000000))*(x64)*(x76)))+(((x51)*(x71)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x50)))+(((x64)*(x65)))+(((IkReal(-1.00000000000000))*(x49)*(x88))));
op[1]=((((gconst16)*(gconst26)*(x51)))+(((x50)*(x85)))+(((x58)*(x76)))+(((IkReal(-1.00000000000000))*(gconst24)*(x52)*(x64)))+(((IkReal(-1.00000000000000))*(gconst27)*(x49)*(x78)))+(((gconst15)*(gconst26)*(x64)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x75)))+(((IkReal(-1.00000000000000))*(x55)*(x82)))+(((IkReal(-1.00000000000000))*(gconst16)*(x49)*(x50)))+(((gconst17)*(gconst25)*(x50)))+(((IkReal(-1.00000000000000))*(x51)*(x85)))+(((IkReal(-1.00000000000000))*(gconst15)*(x49)*(x79)))+(((IkReal(-1.00000000000000))*(x57)*(x88)))+(((IkReal(-1.00000000000000))*(x55)*(x70)))+(((IkReal(-1.00000000000000))*(x50)*(x56)))+(((gconst12)*(gconst29)*(x79)))+(((x76)*(x78)))+(((gconst14)*(gconst27)*(x64)))+(((IkReal(-1.00000000000000))*(gconst27)*(x49)*(x58)))+(((gconst18)*(gconst35)*(x65)))+(((x55)*(x75)))+(((gconst17)*(gconst24)*(x79)))+(((x51)*(x56)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x66)))+(((x70)*(x71)))+(((IkReal(-1.00000000000000))*(gconst29)*(x64)*(x67)))+(((gconst23)*(gconst30)*(x65)))+(((IkReal(-1.00000000000000))*(gconst25)*(x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst30)*(x76)*(x77)))+(((IkReal(-1.00000000000000))*(gconst35)*(x76)*(x81)))+(((x55)*(x66)))+(((x71)*(x82))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((gconst22)*(gconst6)*(x69)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x53)))+(((gconst17)*(gconst9)*(x61)))+(((IkReal(-1.00000000000000))*(gconst19)*(x63)*(x69)))+(((IkReal(-1.00000000000000))*(gconst11)*(x80)*(x81)))+(((x59)*(x62)))+(((gconst1)*(gconst17)*(x59)))+(((IkReal(-1.00000000000000))*(gconst5)*(x67)*(x72)))+(((gconst10)*(gconst18)*(x69)))+(((IkReal(-1.00000000000000))*(gconst0)*(x52)*(x72)))+(((x60)*(x61)))+(((x54)*(x86)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x68)))+(((gconst11)*(gconst15)*(x53)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x53)))+(((IkReal(-1.00000000000000))*(gconst4)*(x57)*(x59)))+(((gconst20)*(gconst9)*(x80)))+(((IkReal(-1.00000000000000))*(gconst20)*(x63)*(x68)))+(((IkReal(-1.00000000000000))*(gconst7)*(x62)*(x63)))+(((gconst23)*(gconst3)*(x53)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x57)))+(((gconst14)*(gconst7)*(x84)))+(((IkReal(-1.00000000000000))*(x74)*(x87)))+(((x54)*(x74)))+(((IkReal(-1.00000000000000))*(x60)*(x73)))+(((IkReal(-1.00000000000000))*(gconst6)*(x77)*(x80)))+(((gconst21)*(x89)))+(((IkReal(-1.00000000000000))*(gconst9)*(x52)*(x73)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((gconst12)*(gconst8)*(x83)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x63)))+(((gconst19)*(gconst2)*(x84))));
op[8]=((((IkReal(-1.00000000000000))*(gconst8)*(x63)*(x68)))+(((x61)*(x83)))+(((x59)*(x69)))+(((IkReal(-1.00000000000000))*(gconst7)*(x63)*(x69)))+(((IkReal(-1.00000000000000))*(x53)*(x87)))+(((IkReal(-1.00000000000000))*(x72)*(x80)))+(((x53)*(x54)))+(((gconst2)*(gconst7)*(x84))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x90=(sj1)*(sj1);
IkReal x91=(cj1)*(cj1);
IkReal x92=((IkReal(4.81610000000000))*(cj0));
IkReal x93=((IkReal(5.77932000000000))*(cj1));
IkReal x94=((pz)*(r00));
IkReal x95=((IkReal(4.81610000000000))*(sj0));
IkReal x96=((pz)*(r01));
IkReal x97=((r02)*(x90));
IkReal x98=((r02)*(x91));
gconst52=IKsign(((((x90)*(x95)*(x96)))+(((x91)*(x95)*(x96)))+(((IkReal(-5.77932000000000))*(r02)*(sj1)))+(((cj0)*(r00)*(x93)))+(((IkReal(1.68563500000000))*(x98)))+(((IkReal(1.68563500000000))*(x97)))+(((r01)*(sj0)*(x93)))+(((x90)*(x92)*(x94)))+(((IkReal(-1.00000000000000))*(px)*(x92)*(x97)))+(((IkReal(-1.00000000000000))*(px)*(x92)*(x98)))+(((IkReal(-1.00000000000000))*(py)*(x95)*(x98)))+(((IkReal(-1.00000000000000))*(py)*(x95)*(x97)))+(((x91)*(x92)*(x94)))));
IkReal x99=(sj1)*(sj1);
IkReal x100=(cj1)*(cj1);
IkReal x101=((r01)*(sj0));
IkReal x102=((cj0)*(r00));
IkReal x103=((IkReal(3.42857142857143))*(cj1));
IkReal x104=((cj0)*(px));
IkReal x105=((r02)*(x100));
IkReal x106=((IkReal(2.85714285714286))*(py)*(sj0));
IkReal x107=((IkReal(2.85714285714286))*(x99));
IkReal x108=((r02)*(x99));
IkReal x109=((IkReal(2.85714285714286))*(pz)*(x100));
dummyeval[0]=((((pz)*(x101)*(x107)))+(((pz)*(x102)*(x107)))+(((IkReal(-1.00000000000000))*(x106)*(x108)))+(x108)+(x105)+(((x102)*(x103)))+(((x102)*(x109)))+(((IkReal(-2.85714285714286))*(x104)*(x105)))+(((IkReal(-3.42857142857143))*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r02)*(x104)*(x107)))+(((x101)*(x103)))+(((x101)*(x109)))+(((IkReal(-1.00000000000000))*(x105)*(x106))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst52)*(((IkReal(-0.751680000000000))+(((IkReal(0.101500000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.290000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-3.70800000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-1.08150000000000))*(cj1)*(px)*(r00)))+(((IkReal(-3.70800000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(3.70800000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(-0.626400000000000))*(cj0)*(px)*(sj1)))+(((IkReal(0.101500000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.290000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-2.33604000000000))*(cj1)))+(((IkReal(-3.70800000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(1.08150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.290000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.290000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1.29780000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.101500000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(0.219240000000000))*(sj1)))+(((IkReal(-0.290000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(-1.54500000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-3.09000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-6.67440000000000))*(pz)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(1.08150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.348000000000000))*(pz)*(r02)))+(((IkReal(0.145000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(0.145000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(3.09000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(1.49094663000000))*(cj0)*(cj1)*(r00)))+(((IkReal(1.54500000000000))*(pp)*(r02)*(sj1)))+(((IkReal(1.49094663000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.348000000000000))*(py)*(r01)))+(((IkReal(-0.101500000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(3.09000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.290000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(3.09000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(6.67440000000000))*(cj1)*(py)*(sj0)))+(((IkReal(6.67440000000000))*(cj0)*(cj1)*(px)))+(((IkReal(-0.290000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(-0.290000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-1.08150000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-1.54500000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-1.08150000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.626400000000000))*(cj1)*(pz)))+(((IkReal(-1.08150000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.290000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(0.121800000000000))*(r01)*(sj0)))+(((IkReal(-1.08150000000000))*(cj1)*(pz)*(r02)))+(((IkReal(0.121800000000000))*(cj0)*(r00)))+(((IkReal(-1.29780000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.557527030000000))*(cj0)*(r00)*(sj1)))+(((IkReal(0.101500000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(3.70800000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-3.09000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-0.522002030000000))*(cj1)*(r02)))+(((IkReal(-3.70800000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(3.09000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(3.70800000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.557527030000000))*(r01)*(sj0)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.101500000000000))*(py)*(r01)*(sj1)))+(((IkReal(-0.348000000000000))*(px)*(r00)))+(((IkReal(-1.11242163000000))*(r02)*(sj1)))+(((IkReal(3.70800000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.101500000000000))*(pz)*(r02)*(sj1)))+(((IkReal(-3.09000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(0.145000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.626400000000000))*(py)*(sj0)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((IkReal(8.00928000000000))+(((IkReal(3.70800000000000))*(py)*(r01)))+(((IkReal(3.09000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-0.290000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.101500000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(0.290000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.290000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-1.54500000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(0.348000000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-0.101500000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-1.29780000000000))*(cj0)*(r00)))+(((IkReal(0.145000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(0.348000000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(3.09000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(3.09000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(3.70800000000000))*(px)*(r00)))+(((IkReal(0.290000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(3.09000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(3.09000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(3.09000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(0.626400000000000))*(cj0)*(cj1)*(px)))+(((IkReal(-1.08150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-2.33604000000000))*(sj1)))+(((IkReal(0.348000000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(3.09000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-1.08150000000000))*(pz)*(r02)*(sj1)))+(((IkReal(-0.101500000000000))*(cj1)*(px)*(r00)))+(((IkReal(-0.348000000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.104402030000000))*(r02)*(sj1)))+(((IkReal(0.290000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.348000000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(3.09000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.101500000000000))*(cj1)*(py)*(r01)))+(((IkReal(3.09000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(-1.29780000000000))*(r01)*(sj0)))+(((IkReal(-0.101500000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(0.290000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(-0.219240000000000))*(cj1)))+(((IkReal(1.08150000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(-1.08150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.348000000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(5.94054663000000))*(r01)*(sj0)*(sj1)))+(((IkReal(5.56202163000000))*(cj1)*(r02)))+(((IkReal(-0.348000000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-1.08150000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.145000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(0.139927030000000))*(cj1)*(r01)*(sj0)))+(((IkReal(0.101500000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.101500000000000))*(cj1)*(pz)*(r02)))+(((IkReal(5.94054663000000))*(cj0)*(r00)*(sj1)))+(((IkReal(-0.121800000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.626400000000000))*(pz)*(sj1)))+(((IkReal(0.290000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-1.54500000000000))*(cj1)*(pp)*(r02)))+(((IkReal(6.67440000000000))*(cj1)*(pz)))+(((IkReal(0.139927030000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.290000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(3.70800000000000))*(pz)*(r02)))+(((IkReal(-0.121800000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(6.67440000000000))*(py)*(sj0)*(sj1)))+(((IkReal(-0.348000000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-1.08150000000000))*(py)*(r01)*(sj1)))+(((IkReal(6.67440000000000))*(cj0)*(px)*(sj1)))+(((IkReal(0.626400000000000))*(cj1)*(py)*(sj0)))+(((IkReal(1.08150000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(0.290000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.145000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-1.54500000000000))*(pp)*(r01)*(sj0)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst52)*(((IkReal(-0.751680000000000))+(((IkReal(0.101500000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.290000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-3.70800000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-1.08150000000000))*(cj1)*(px)*(r00)))+(((IkReal(-3.70800000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(3.70800000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(-0.626400000000000))*(cj0)*(px)*(sj1)))+(((IkReal(0.101500000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.290000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-2.33604000000000))*(cj1)))+(((IkReal(-3.70800000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(1.08150000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.290000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.290000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-1.29780000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.101500000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(0.219240000000000))*(sj1)))+(((IkReal(-0.290000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(-1.54500000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-3.09000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-6.67440000000000))*(pz)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(1.08150000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.348000000000000))*(pz)*(r02)))+(((IkReal(0.145000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(0.145000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(3.09000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(1.49094663000000))*(cj0)*(cj1)*(r00)))+(((IkReal(1.54500000000000))*(pp)*(r02)*(sj1)))+(((IkReal(1.49094663000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.348000000000000))*(py)*(r01)))+(((IkReal(-0.101500000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(3.09000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.290000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(3.09000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(6.67440000000000))*(cj1)*(py)*(sj0)))+(((IkReal(6.67440000000000))*(cj0)*(cj1)*(px)))+(((IkReal(-0.290000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(-0.290000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-1.08150000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-1.54500000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-1.08150000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.626400000000000))*(cj1)*(pz)))+(((IkReal(-1.08150000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.290000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(0.121800000000000))*(r01)*(sj0)))+(((IkReal(-1.08150000000000))*(cj1)*(pz)*(r02)))+(((IkReal(0.121800000000000))*(cj0)*(r00)))+(((IkReal(-1.29780000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.557527030000000))*(cj0)*(r00)*(sj1)))+(((IkReal(0.101500000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(3.70800000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-3.09000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(-0.522002030000000))*(cj1)*(r02)))+(((IkReal(-3.70800000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(3.09000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(3.70800000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.557527030000000))*(r01)*(sj0)*(sj1)))+(((IkReal(3.09000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.101500000000000))*(py)*(r01)*(sj1)))+(((IkReal(-0.348000000000000))*(px)*(r00)))+(((IkReal(-1.11242163000000))*(r02)*(sj1)))+(((IkReal(3.70800000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.101500000000000))*(pz)*(r02)*(sj1)))+(((IkReal(-3.09000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(0.145000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.626400000000000))*(py)*(sj0)*(sj1)))))), ((gconst52)*(((IkReal(8.00928000000000))+(((IkReal(3.70800000000000))*(py)*(r01)))+(((IkReal(3.09000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-0.290000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.101500000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(0.290000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.290000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-1.54500000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(0.348000000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-0.101500000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-1.29780000000000))*(cj0)*(r00)))+(((IkReal(0.145000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(0.348000000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(3.09000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(3.09000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(3.70800000000000))*(px)*(r00)))+(((IkReal(0.290000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(3.09000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(3.09000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(3.09000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(0.626400000000000))*(cj0)*(cj1)*(px)))+(((IkReal(-1.08150000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-2.33604000000000))*(sj1)))+(((IkReal(0.348000000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(3.09000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-1.08150000000000))*(pz)*(r02)*(sj1)))+(((IkReal(-0.101500000000000))*(cj1)*(px)*(r00)))+(((IkReal(-0.348000000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.104402030000000))*(r02)*(sj1)))+(((IkReal(0.290000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.348000000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(3.09000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.101500000000000))*(cj1)*(py)*(r01)))+(((IkReal(3.09000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(-1.29780000000000))*(r01)*(sj0)))+(((IkReal(-0.101500000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(0.290000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(-0.219240000000000))*(cj1)))+(((IkReal(1.08150000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(-1.08150000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.348000000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(5.94054663000000))*(r01)*(sj0)*(sj1)))+(((IkReal(5.56202163000000))*(cj1)*(r02)))+(((IkReal(-0.348000000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-1.08150000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.145000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(0.139927030000000))*(cj1)*(r01)*(sj0)))+(((IkReal(0.101500000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.101500000000000))*(cj1)*(pz)*(r02)))+(((IkReal(5.94054663000000))*(cj0)*(r00)*(sj1)))+(((IkReal(-0.121800000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.626400000000000))*(pz)*(sj1)))+(((IkReal(0.290000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-1.54500000000000))*(cj1)*(pp)*(r02)))+(((IkReal(6.67440000000000))*(cj1)*(pz)))+(((IkReal(0.139927030000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.290000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(3.70800000000000))*(pz)*(r02)))+(((IkReal(-0.121800000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(6.67440000000000))*(py)*(sj0)*(sj1)))+(((IkReal(-0.348000000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-1.08150000000000))*(py)*(r01)*(sj1)))+(((IkReal(6.67440000000000))*(cj0)*(px)*(sj1)))+(((IkReal(0.626400000000000))*(cj1)*(py)*(sj0)))+(((IkReal(1.08150000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(0.290000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.145000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-1.54500000000000))*(pp)*(r01)*(sj0)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
IkReal x110=IKcos(j2);
IkReal x111=IKsin(j2);
IkReal x112=((IkReal(0.290000000000000))*(cj1));
IkReal x113=((cj0)*(px));
IkReal x114=((py)*(sj0));
IkReal x115=((IkReal(2.40000000000000))*(sj1));
IkReal x116=((IkReal(3.09000000000000))*(sj1));
IkReal x117=((cj0)*(r00));
IkReal x118=((r01)*(sj0));
IkReal x119=((IkReal(1.54500000000000))*(r02));
IkReal x120=((IkReal(1.20000000000000))*(sj1));
IkReal x121=((IkReal(0.145000000000000))*(r02));
IkReal x122=((pz)*(x110));
IkReal x123=((sj1)*(x110));
IkReal x124=((cj1)*(x111));
IkReal x125=((cj1)*(x110));
IkReal x126=((sj1)*(x111));
evalcond[0]=((IkReal(-0.720014000000000))+(((IkReal(-0.840000000000000))*(sj1)))+(((x112)*(x122)))+(((IkReal(2.40000000000000))*(cj1)*(pz)))+(((x114)*(x115)))+(((IkReal(0.290000000000000))*(x113)*(x123)))+(((x111)*(x114)*(x116)))+(((x113)*(x115)))+(((IkReal(3.70800000000000))*(x111)))+(((IkReal(-0.290000000000000))*(pz)*(x126)))+(((x111)*(x113)*(x116)))+(((IkReal(-0.101500000000000))*(x124)))+(((IkReal(-0.101500000000000))*(x123)))+(((IkReal(-3.09000000000000))*(x113)*(x125)))+(((IkReal(1.08150000000000))*(x125)))+(((IkReal(0.348000000000000))*(x110)))+(((IkReal(-0.700000000000000))*(x114)))+(((IkReal(-0.700000000000000))*(x113)))+(pp)+(((IkReal(0.290000000000000))*(x114)*(x123)))+(((x116)*(x122)))+(((IkReal(-1.08150000000000))*(x126)))+(((IkReal(3.09000000000000))*(pz)*(x124)))+(((x111)*(x112)*(x113)))+(((x111)*(x112)*(x114)))+(((IkReal(-3.09000000000000))*(x114)*(x125))));
evalcond[1]=((IkReal(2.16000000000000))+(((x117)*(x120)))+(((x121)*(x125)))+(((px)*(r00)))+(((IkReal(-1.54500000000000))*(x118)*(x125)))+(((IkReal(1.20000000000000))*(cj1)*(r02)))+(((IkReal(1.54500000000000))*(x118)*(x126)))+(((IkReal(0.145000000000000))*(x118)*(x123)))+(((IkReal(0.145000000000000))*(x118)*(x124)))+(((IkReal(-1.00000000000000))*(x121)*(x126)))+(((IkReal(-0.350000000000000))*(x117)))+(((IkReal(-0.350000000000000))*(x118)))+(((IkReal(0.145000000000000))*(x117)*(x124)))+(((IkReal(0.145000000000000))*(x117)*(x123)))+(((pz)*(r02)))+(((IkReal(-1.54500000000000))*(x117)*(x125)))+(((py)*(r01)))+(((IkReal(1.54500000000000))*(x117)*(x126)))+(((x119)*(x123)))+(((x119)*(x124)))+(((x118)*(x120))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x127=((cj1)*(cj2));
IkReal x128=((IkReal(13.6708860759494))*(sj2));
IkReal x129=((sj1)*(sj2));
IkReal x130=((cj0)*(r00));
IkReal x131=((IkReal(6.32911392405063))*(pz));
IkReal x132=((cj2)*(sj1));
IkReal x133=((r01)*(sj0));
IkReal x134=((IkReal(6.32911392405063))*(py)*(sj0));
IkReal x135=((IkReal(6.32911392405063))*(cj0)*(px));
cj4array[0]=((IkReal(-9.77848101265823))+(((x127)*(x134)))+(((x127)*(x135)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x128)))+(((IkReal(13.6708860759494))*(x127)*(x130)))+(((IkReal(13.6708860759494))*(x127)*(x133)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x131)))+(((IkReal(-13.6708860759494))*(r02)*(x132)))+(((IkReal(-7.59493670886076))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x128)*(x130)))+(((IkReal(-1.00000000000000))*(sj1)*(x128)*(x133)))+(((IkReal(-1.00000000000000))*(x129)*(x135)))+(((IkReal(-1.00000000000000))*(x129)*(x134)))+(((IkReal(-2.21518987341772))*(x127)))+(((IkReal(2.21518987341772))*(x129)))+(((IkReal(-1.00000000000000))*(x131)*(x132))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x136=((cj2)*(sj1));
IkReal x137=((sj1)*(sj2));
IkReal x138=((cj1)*(sj2));
IkReal x139=((sj0)*(x137));
IkReal x140=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x141=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r02)*(x136)))+(((r02)*(x138)))+(((IkReal(-1.00000000000000))*(r01)*(x141)))+(((r01)*(x139)))+(((IkReal(-1.00000000000000))*(r00)*(x140)))+(((cj0)*(r00)*(x137))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(x141)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x136)))+(((pz)*(x138)))+(((py)*(x139)))+(((IkReal(-1.00000000000000))*(px)*(x140)))+(((IkReal(-0.350000000000000))*(x137)))+(((cj0)*(px)*(x137))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst64;
gconst64=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst65;
gconst65=IkReal(4.32000000000000);
IkReal gconst66;
gconst66=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst67;
gconst67=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst68;
gconst68=IkReal(-2.00000000000000);
IkReal gconst69;
gconst69=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst70;
gconst70=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst71;
gconst71=IkReal(4.32000000000000);
IkReal gconst72;
gconst72=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(4.32000000000000);
IkReal gconst76;
gconst76=IkReal(-4.32000000000000);
IkReal gconst77;
gconst77=IkReal(-2.00000000000000);
IkReal gconst78;
gconst78=IkReal(2.00000000000000);
IkReal gconst79;
gconst79=IkReal(4.32000000000000);
IkReal gconst80;
gconst80=IkReal(-4.32000000000000);
IkReal gconst81;
gconst81=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst82;
gconst82=IkReal(2.00000000000000);
IkReal gconst83;
gconst83=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst84;
gconst84=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst85;
gconst85=IkReal(-4.32000000000000);
IkReal gconst86;
gconst86=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst87;
gconst87=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst88;
gconst88=IkReal(2.00000000000000);
IkReal gconst89;
gconst89=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst90;
gconst90=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst91;
gconst91=IkReal(-4.32000000000000);
IkReal gconst92;
gconst92=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal x142=((gconst89)*(gconst90));
IkReal x143=((gconst83)*(gconst84));
IkReal x144=((gconst81)*(gconst86));
IkReal x145=((gconst87)*(gconst92));
IkReal x146=((gconst82)*(gconst86));
IkReal x147=((gconst87)*(gconst91));
IkReal x148=((gconst88)*(gconst90));
IkReal x149=((gconst83)*(gconst85));
dummyeval[0]=((((x142)*(x144)))+(((IkReal(-1.00000000000000))*(x142)*(x143)))+(((IkReal(-1.00000000000000))*(x144)*(x145)))+(((IkReal(-1.00000000000000))*(x146)*(x148)))+(((x148)*(x149)))+(((x146)*(x147)))+(((IkReal(-1.00000000000000))*(x147)*(x149)))+(((x143)*(x145))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x150=((gconst63)*(gconst67));
IkReal x151=((IkReal(1.00000000000000))*(gconst65));
IkReal x152=((IkReal(1.00000000000000))*(gconst86));
IkReal x153=((gconst80)*(gconst87));
IkReal x154=((gconst68)*(gconst82));
IkReal x155=((gconst72)*(gconst75));
IkReal x156=((gconst72)*(gconst77));
IkReal x157=((gconst61)*(gconst76));
IkReal x158=((gconst63)*(gconst87));
IkReal x159=((gconst61)*(gconst66));
IkReal x160=((gconst70)*(gconst78));
IkReal x161=((gconst86)*(gconst89));
IkReal x162=((gconst73)*(gconst90));
IkReal x163=((gconst64)*(gconst72));
IkReal x164=((gconst87)*(gconst92));
IkReal x165=((IkReal(1.00000000000000))*(gconst83));
IkReal x166=((gconst85)*(gconst91));
IkReal x167=((gconst69)*(gconst90));
IkReal x168=((gconst78)*(gconst90));
IkReal x169=((IkReal(1.00000000000000))*(gconst70));
IkReal x170=((gconst62)*(gconst66));
IkReal x171=((gconst62)*(gconst86));
IkReal x172=((gconst67)*(gconst71));
IkReal x173=((gconst72)*(gconst84));
IkReal x174=((gconst65)*(gconst68));
IkReal x175=((gconst74)*(gconst79));
IkReal x176=((gconst63)*(gconst69));
IkReal x177=((IkReal(1.00000000000000))*(gconst67));
IkReal x178=((gconst66)*(gconst73));
IkReal x179=((gconst70)*(gconst81));
IkReal x180=((gconst63)*(gconst70));
IkReal x181=((gconst79)*(gconst81));
IkReal x182=((gconst78)*(gconst86));
IkReal x183=((gconst84)*(gconst92));
IkReal x184=((IkReal(1.00000000000000))*(gconst79));
IkReal x185=((gconst83)*(gconst84));
IkReal x186=((gconst75)*(gconst83));
IkReal x187=((gconst64)*(gconst74));
IkReal x188=((gconst81)*(gconst92));
IkReal x189=((gconst63)*(gconst64));
IkReal x190=((gconst76)*(gconst89));
IkReal x191=((gconst65)*(gconst88));
IkReal x192=((gconst79)*(gconst84));
IkReal x193=((gconst87)*(gconst91));
IkReal x194=((gconst81)*(gconst86));
IkReal x195=((gconst69)*(gconst79));
IkReal x196=((gconst67)*(gconst83));
IkReal x197=((gconst82)*(gconst88));
IkReal x198=((gconst69)*(gconst70));
IkReal x199=((gconst67)*(gconst74));
IkReal x200=((gconst70)*(gconst89));
IkReal x201=((IkReal(1.00000000000000))*(gconst77));
IkReal x202=((IkReal(1.00000000000000))*(gconst81));
IkReal x203=((gconst68)*(gconst85));
IkReal x204=((gconst83)*(gconst90));
IkReal x205=((gconst62)*(gconst68));
IkReal x206=((gconst82)*(gconst86));
IkReal x207=((gconst67)*(gconst91));
IkReal x208=((IkReal(1.00000000000000))*(gconst63));
IkReal x209=((gconst85)*(gconst88));
IkReal x210=((gconst74)*(gconst84));
IkReal x211=((gconst89)*(gconst90));
IkReal x212=((gconst72)*(gconst87));
IkReal x213=((gconst73)*(gconst76));
IkReal x214=((gconst66)*(gconst81));
IkReal x215=((gconst63)*(gconst90));
IkReal x216=((gconst63)*(gconst77));
IkReal x217=((gconst74)*(gconst75));
IkReal x218=((gconst66)*(gconst82));
IkReal x219=((gconst71)*(gconst87));
IkReal x220=((IkReal(1.00000000000000))*(gconst90));
IkReal x221=((gconst71)*(gconst77));
IkReal x222=((gconst77)*(gconst80));
IkReal x223=((gconst76)*(gconst82));
IkReal x224=((gconst77)*(gconst91));
IkReal x225=((gconst70)*(gconst83));
IkReal x226=((gconst62)*(gconst76));
IkReal x227=((gconst81)*(gconst90));
IkReal x228=((gconst77)*(gconst92));
IkReal x229=((gconst74)*(x209));
op[0]=((((IkReal(-1.00000000000000))*(gconst84)*(x165)*(x211)))+(((IkReal(-1.00000000000000))*(gconst87)*(x165)*(x166)))+(((IkReal(-1.00000000000000))*(gconst81)*(x152)*(x164)))+(((x164)*(x185)))+(((x204)*(x209)))+(((x161)*(x227)))+(((x193)*(x206)))+(((IkReal(-1.00000000000000))*(gconst90)*(x152)*(x197))));
op[1]=((((x161)*(x162)))+(((IkReal(-1.00000000000000))*(gconst84)*(x165)*(x168)))+(((x153)*(x185)))+(((IkReal(-1.00000000000000))*(gconst79)*(x152)*(x197)))+(((x168)*(x194)))+(((x206)*(x224)))+(((x161)*(x181)))+(((IkReal(-1.00000000000000))*(gconst89)*(x165)*(x192)))+(((IkReal(-1.00000000000000))*(gconst75)*(x165)*(x211)))+(((x193)*(x223)))+(((x164)*(x210)))+(((x164)*(x186)))+(((IkReal(-1.00000000000000))*(gconst77)*(x152)*(x188)))+(((gconst77)*(gconst83)*(x183)))+(((IkReal(-1.00000000000000))*(gconst81)*(x152)*(x153)))+(((IkReal(-1.00000000000000))*(gconst73)*(x152)*(x164)))+(((gconst79)*(gconst83)*(x209)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst87)*(x166)))+(((gconst90)*(x229)))+(((IkReal(-1.00000000000000))*(gconst76)*(x197)*(x220)))+(((IkReal(-1.00000000000000))*(gconst76)*(x164)*(x202)))+(((x190)*(x227)))+(((IkReal(-1.00000000000000))*(x210)*(x211)))+(((IkReal(-1.00000000000000))*(gconst77)*(x165)*(x166))));
op[2]=((((IkReal(-1.00000000000000))*(gconst76)*(x184)*(x197)))+(((x175)*(x209)))+(((IkReal(-1.00000000000000))*(gconst84)*(x165)*(x167)))+(((IkReal(-1.00000000000000))*(x168)*(x210)))+(((x153)*(x186)))+(((x185)*(x222)))+(((x153)*(x210)))+(((x161)*(x179)))+(((IkReal(-1.00000000000000))*(gconst85)*(x165)*(x219)))+(((x162)*(x190)))+(((gconst83)*(gconst87)*(x173)))+(((IkReal(-1.00000000000000))*(gconst84)*(gconst89)*(x175)))+(((IkReal(-1.00000000000000))*(gconst74)*(x166)*(x201)))+(((x162)*(x182)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst88)*(gconst90)*(x152)))+(((x206)*(x219)))+(((x193)*(x218)))+(((IkReal(-1.00000000000000))*(gconst66)*(x197)*(x220)))+(((gconst73)*(gconst79)*(x161)))+(((x209)*(x225)))+(((IkReal(-1.00000000000000))*(gconst67)*(x152)*(x188)))+(((x167)*(x194)))+(((x211)*(x214)))+(((x164)*(x217)))+(((gconst61)*(gconst90)*(x161)))+(((x191)*(x204)))+(((IkReal(-1.00000000000000))*(gconst70)*(x152)*(x197)))+(((gconst74)*(gconst77)*(x183)))+(((x223)*(x224)))+(((x209)*(x215)))+(((x186)*(x228)))+(((IkReal(-1.00000000000000))*(gconst83)*(x151)*(x193)))+(((IkReal(-1.00000000000000))*(gconst81)*(x152)*(x222)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst89)*(x165)))+(((IkReal(-1.00000000000000))*(x158)*(x166)))+(((IkReal(-1.00000000000000))*(gconst81)*(x152)*(x212)))+(((IkReal(-1.00000000000000))*(gconst76)*(x153)*(x202)))+(((IkReal(-1.00000000000000))*(x164)*(x213)))+(((IkReal(-1.00000000000000))*(gconst67)*(x165)*(x166)))+(((x171)*(x193)))+(((IkReal(-1.00000000000000))*(gconst75)*(x165)*(x168)))+(((IkReal(-1.00000000000000))*(gconst64)*(x165)*(x211)))+(((IkReal(-1.00000000000000))*(gconst73)*(x152)*(x153)))+(((x203)*(x204)))+(((IkReal(-1.00000000000000))*(gconst73)*(x152)*(x228)))+(((IkReal(-1.00000000000000))*(gconst76)*(x188)*(x201)))+(((IkReal(-1.00000000000000))*(gconst84)*(x165)*(x200)))+(((IkReal(-1.00000000000000))*(gconst78)*(x165)*(x192)))+(((IkReal(-1.00000000000000))*(gconst90)*(x152)*(x154)))+(((x181)*(x190)))+(((IkReal(-1.00000000000000))*(x211)*(x217)))+(((IkReal(-1.00000000000000))*(gconst84)*(x208)*(x211)))+(((gconst64)*(gconst83)*(x164)))+(((IkReal(-1.00000000000000))*(gconst61)*(x152)*(x164)))+(((x158)*(x183)))+(((x206)*(x207)))+(((x181)*(x182)))+(((gconst76)*(gconst81)*(x168)))+(((x183)*(x196)))+(((IkReal(-1.00000000000000))*(gconst66)*(x164)*(x202))));
op[3]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst80)*(gconst87)))+(((gconst75)*(gconst77)*(gconst80)*(gconst83)))+(((gconst71)*(gconst77)*(gconst82)*(gconst86)))+(((gconst65)*(gconst74)*(gconst88)*(gconst90)))+(((gconst67)*(gconst80)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst89)*(gconst90)))+(((gconst61)*(gconst78)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst79)*(gconst82)*(gconst86)))+(((gconst61)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst77)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst81)*(gconst92)))+(((gconst63)*(gconst79)*(gconst85)*(gconst88)))+(((gconst62)*(gconst76)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst83)*(gconst90)))+(((gconst64)*(gconst74)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst80)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst79)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst88)*(gconst90)))+(((gconst71)*(gconst76)*(gconst82)*(gconst87)))+(((gconst70)*(gconst74)*(gconst85)*(gconst88)))+(((gconst74)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst82)*(gconst90)))+(((gconst74)*(gconst75)*(gconst80)*(gconst87)))+(((gconst62)*(gconst77)*(gconst86)*(gconst91)))+(((gconst69)*(gconst76)*(gconst81)*(gconst90)))+(((gconst66)*(gconst79)*(gconst81)*(gconst89)))+(((gconst69)*(gconst73)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst80)*(gconst86)*(gconst87)))+(((gconst66)*(gconst77)*(gconst82)*(gconst91)))+(((gconst70)*(gconst73)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst79)*(gconst86)*(gconst88)))+(((gconst74)*(gconst77)*(gconst80)*(gconst84)))+(((gconst70)*(gconst78)*(gconst81)*(gconst86)))+(((gconst63)*(gconst75)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst79)*(gconst83)*(gconst89)))+(((gconst68)*(gconst74)*(gconst85)*(gconst90)))+(((gconst64)*(gconst77)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst92)))+(((gconst65)*(gconst79)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst82)*(gconst88)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst86)*(gconst87)))+(((gconst72)*(gconst77)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst86)*(gconst92)))+(((gconst69)*(gconst79)*(gconst81)*(gconst86)))+(((gconst67)*(gconst76)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst83)*(gconst90)))+(((gconst61)*(gconst76)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst84)*(gconst89)))+(((gconst72)*(gconst75)*(gconst83)*(gconst87)))+(((gconst63)*(gconst80)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst78)*(gconst79)*(gconst83)))+(((gconst63)*(gconst77)*(gconst84)*(gconst92)))+(((gconst73)*(gconst78)*(gconst79)*(gconst86)))+(((gconst70)*(gconst76)*(gconst81)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst78)*(gconst83)*(gconst84)))+(((gconst66)*(gconst73)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst83)*(gconst89)))+(((gconst68)*(gconst79)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst74)*(gconst85)*(gconst91)))+(((gconst73)*(gconst76)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst77)*(gconst83)*(gconst91)))+(((gconst67)*(gconst74)*(gconst84)*(gconst92)))+(((gconst72)*(gconst74)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst84)*(gconst89)))+(((gconst76)*(gconst78)*(gconst79)*(gconst81)))+(((gconst66)*(gconst78)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst87)*(gconst92)))+(((gconst73)*(gconst76)*(gconst79)*(gconst89)))+(((gconst67)*(gconst75)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst80)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst78)*(gconst79)*(gconst84))));
op[4]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst83)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst87)*(gconst92)))+(((gconst64)*(gconst72)*(gconst83)*(gconst87)))+(((gconst64)*(gconst74)*(gconst80)*(gconst87)))+(((gconst63)*(gconst70)*(gconst85)*(gconst88)))+(((gconst68)*(gconst70)*(gconst83)*(gconst85)))+(((gconst67)*(gconst74)*(gconst80)*(gconst84)))+(((gconst73)*(gconst76)*(gconst78)*(gconst79)))+(((gconst64)*(gconst67)*(gconst83)*(gconst92)))+(((gconst67)*(gconst72)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst70)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst79)*(gconst83)))+(((gconst64)*(gconst74)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst90)))+(((gconst71)*(gconst76)*(gconst77)*(gconst82)))+(((gconst70)*(gconst73)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst84)*(gconst89)))+(((gconst61)*(gconst66)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst92)))+(((gconst62)*(gconst76)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst77)*(gconst81)))+(((gconst61)*(gconst69)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst80)))+(((gconst66)*(gconst70)*(gconst81)*(gconst89)))+(((gconst61)*(gconst70)*(gconst86)*(gconst89)))+(((gconst63)*(gconst64)*(gconst87)*(gconst92)))+(((gconst61)*(gconst78)*(gconst79)*(gconst86)))+(((gconst68)*(gconst74)*(gconst79)*(gconst85)))+(((gconst67)*(gconst75)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst86)*(gconst87)))+(((gconst66)*(gconst69)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst79)*(gconst88)))+(((gconst62)*(gconst66)*(gconst87)*(gconst91)))+(((gconst70)*(gconst76)*(gconst78)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst79)*(gconst81)))+(((gconst66)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst82)*(gconst86)))+(((gconst65)*(gconst70)*(gconst83)*(gconst88)))+(((gconst67)*(gconst71)*(gconst82)*(gconst86)))+(((gconst63)*(gconst65)*(gconst88)*(gconst90)))+(((gconst63)*(gconst75)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst87)*(gconst91)))+(((gconst63)*(gconst68)*(gconst85)*(gconst90)))+(((gconst70)*(gconst73)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst87)))+(((gconst61)*(gconst76)*(gconst78)*(gconst90)))+(((gconst72)*(gconst74)*(gconst75)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst84)*(gconst90)))+(((gconst62)*(gconst71)*(gconst86)*(gconst87)))+(((gconst69)*(gconst73)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst80)*(gconst81)))+(((gconst72)*(gconst75)*(gconst77)*(gconst83)))+(((gconst69)*(gconst70)*(gconst81)*(gconst86)))+(((gconst65)*(gconst68)*(gconst83)*(gconst90)))+(((gconst63)*(gconst67)*(gconst84)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst92)))+(((gconst74)*(gconst75)*(gconst77)*(gconst80)))+(((gconst63)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst80)*(gconst86)))+(((gconst66)*(gconst73)*(gconst78)*(gconst90)))+(((gconst66)*(gconst73)*(gconst79)*(gconst89)))+(((gconst61)*(gconst76)*(gconst79)*(gconst89)))+(((gconst72)*(gconst74)*(gconst77)*(gconst84)))+(((gconst66)*(gconst78)*(gconst79)*(gconst81)))+(((gconst65)*(gconst74)*(gconst79)*(gconst88)))+(((gconst64)*(gconst77)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst88)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst79)*(gconst89)))+(((gconst66)*(gconst67)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst79)*(gconst89)))+(((gconst62)*(gconst67)*(gconst86)*(gconst91)))+(((gconst69)*(gconst73)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst77)*(gconst86)))+(((gconst63)*(gconst77)*(gconst80)*(gconst84)))+(((gconst63)*(gconst72)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst83)*(gconst87))));
op[5]=((((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst79)*(gconst88)))+(((gconst64)*(gconst72)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst91)))+(((gconst61)*(gconst66)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst66)*(gconst70)*(gconst78)*(gconst81)))+(((gconst66)*(gconst73)*(gconst78)*(gconst79)))+(((gconst63)*(gconst75)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst79)))+(((gconst63)*(gconst72)*(gconst75)*(gconst87)))+(((gconst63)*(gconst67)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst92)))+(((gconst69)*(gconst70)*(gconst76)*(gconst81)))+(((gconst64)*(gconst74)*(gconst77)*(gconst80)))+(((gconst61)*(gconst70)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((gconst68)*(gconst70)*(gconst74)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst79)*(gconst84)))+(((gconst63)*(gconst64)*(gconst77)*(gconst92)))+(((gconst63)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst80)*(gconst86)))+(((gconst63)*(gconst72)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst76)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst80)))+(((gconst69)*(gconst70)*(gconst73)*(gconst86)))+(((gconst65)*(gconst68)*(gconst79)*(gconst83)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst65)*(gconst68)*(gconst74)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst76)*(gconst82)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((gconst66)*(gconst70)*(gconst73)*(gconst89)))+(((gconst72)*(gconst74)*(gconst75)*(gconst77)))+(((gconst69)*(gconst73)*(gconst76)*(gconst79)))+(((gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst62)*(gconst66)*(gconst77)*(gconst91)))+(((gconst62)*(gconst67)*(gconst76)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst74)*(gconst91)))+(((gconst65)*(gconst70)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((gconst61)*(gconst70)*(gconst78)*(gconst86)))+(((gconst61)*(gconst69)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst75)*(gconst89)))+(((gconst64)*(gconst67)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst79)*(gconst86)))+(((gconst61)*(gconst66)*(gconst79)*(gconst89)))+(((gconst67)*(gconst72)*(gconst74)*(gconst84)))+(((gconst63)*(gconst68)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst61)*(gconst69)*(gconst79)*(gconst86)))+(((gconst61)*(gconst76)*(gconst78)*(gconst79)))+(((gconst67)*(gconst72)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst80)))+(((gconst66)*(gconst69)*(gconst79)*(gconst81)))+(((gconst63)*(gconst67)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst74)*(gconst85)))+(((gconst70)*(gconst73)*(gconst76)*(gconst78)))+(((gconst63)*(gconst65)*(gconst79)*(gconst88)))+(((gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst64)*(gconst67)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst87)))+(((gconst64)*(gconst72)*(gconst74)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst90)))+(((gconst66)*(gconst69)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst74)*(gconst87))));
op[6]=((((IkReal(-1.00000000000000))*(gconst75)*(x160)*(x208)))+(((IkReal(-1.00000000000000))*(gconst78)*(x184)*(x189)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(x175)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(x152)))+(((IkReal(-1.00000000000000))*(gconst84)*(x169)*(x176)))+(((IkReal(-1.00000000000000))*(gconst70)*(x152)*(x205)))+(((x189)*(x222)))+(((x157)*(x160)))+(((IkReal(-1.00000000000000))*(gconst69)*(x169)*(x217)))+(((x180)*(x203)))+(((IkReal(-1.00000000000000))*(gconst80)*(x177)*(x178)))+(((gconst75)*(gconst80)*(x150)))+(((gconst64)*(gconst92)*(x150)))+(((IkReal(-1.00000000000000))*(gconst75)*(x176)*(x184)))+(((x174)*(x225)))+(((x178)*(x195)))+(((x180)*(x191)))+(((x155)*(x216)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst85)*(x150)))+(((IkReal(-1.00000000000000))*(gconst64)*(x165)*(x198)))+(((x170)*(x207)))+(((x156)*(x187)))+(((IkReal(-1.00000000000000))*(gconst92)*(x159)*(x177)))+(((gconst67)*(gconst80)*(x187)))+(((x198)*(x213)))+(((x150)*(x173)))+(((x174)*(x175)))+(((IkReal(-1.00000000000000))*(gconst66)*(x154)*(x169)))+(((x155)*(x199)))+(((x172)*(x218)))+(((IkReal(-1.00000000000000))*(x160)*(x187)))+(((IkReal(-1.00000000000000))*(gconst88)*(x169)*(x170)))+(((x170)*(x219)))+(((x171)*(x172)))+(((IkReal(-1.00000000000000))*(gconst68)*(x170)*(x220)))+(((IkReal(-1.00000000000000))*(x156)*(x157)))+(((IkReal(-1.00000000000000))*(gconst89)*(x169)*(x189)))+(((IkReal(-1.00000000000000))*(x159)*(x212)))+(((IkReal(-1.00000000000000))*(gconst91)*(x150)*(x151)))+(((IkReal(-1.00000000000000))*(gconst72)*(x177)*(x214)))+(((IkReal(-1.00000000000000))*(gconst72)*(x177)*(x213)))+(((x174)*(x215)))+(((IkReal(-1.00000000000000))*(gconst80)*(x159)*(x201)))+(((IkReal(-1.00000000000000))*(gconst83)*(x151)*(x172)))+(((x221)*(x226)))+(((IkReal(-1.00000000000000))*(x167)*(x189)))+(((gconst78)*(gconst79)*(x159)))+(((IkReal(-1.00000000000000))*(gconst74)*(x151)*(x221)))+(((IkReal(-1.00000000000000))*(gconst71)*(x151)*(x158)))+(((IkReal(-1.00000000000000))*(x156)*(x178)))+(((IkReal(-1.00000000000000))*(gconst76)*(x184)*(x205)))+(((x158)*(x163)))+(((gconst66)*(gconst69)*(x179)))+(((x159)*(x167)))+(((x157)*(x195)))+(((gconst61)*(gconst86)*(x198)))+(((x160)*(x178)))+(((IkReal(-1.00000000000000))*(gconst80)*(x157)*(x177)))+(((x163)*(x196)))+(((x159)*(x200))));
op[7]=((((IkReal(-1.00000000000000))*(gconst69)*(x169)*(x187)))+(((IkReal(-1.00000000000000))*(gconst75)*(x169)*(x176)))+(((IkReal(-1.00000000000000))*(gconst71)*(x151)*(x216)))+(((x178)*(x198)))+(((x170)*(x221)))+(((gconst70)*(gconst74)*(x174)))+(((x150)*(x155)))+(((gconst63)*(gconst79)*(x174)))+(((x156)*(x189)))+(((x172)*(x226)))+(((IkReal(-1.00000000000000))*(gconst74)*(x151)*(x172)))+(((IkReal(-1.00000000000000))*(gconst72)*(x157)*(x177)))+(((IkReal(-1.00000000000000))*(x160)*(x189)))+(((IkReal(-1.00000000000000))*(gconst68)*(x170)*(x184)))+(((IkReal(-1.00000000000000))*(x156)*(x159)))+(((IkReal(-1.00000000000000))*(gconst76)*(x169)*(x205)))+(((IkReal(-1.00000000000000))*(gconst80)*(x159)*(x177)))+(((IkReal(-1.00000000000000))*(gconst64)*(x176)*(x184)))+(((gconst64)*(gconst80)*(x150)))+(((x159)*(x160)))+(((IkReal(-1.00000000000000))*(gconst72)*(x177)*(x178)))+(((x159)*(x195)))+(((x157)*(x198)))+(((x163)*(x199))));
op[8]=((((IkReal(-1.00000000000000))*(gconst68)*(x169)*(x170)))+(((x174)*(x180)))+(((IkReal(-1.00000000000000))*(gconst71)*(x150)*(x151)))+(((x150)*(x163)))+(((IkReal(-1.00000000000000))*(gconst64)*(x169)*(x176)))+(((IkReal(-1.00000000000000))*(gconst72)*(x159)*(x177)))+(((x170)*(x172)))+(((x159)*(x198))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst109;
gconst109=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x230=((IkReal(1.00000000000000))*(cj3));
IkReal x231=((cj0)*(r01));
IkReal x232=((IkReal(1.00000000000000))*(sj3));
IkReal x233=((cj0)*(r00));
IkReal x234=((cj1)*(sj2));
IkReal x235=((cj2)*(sj1));
IkReal x236=((r00)*(sj0));
IkReal x237=((r02)*(sj1)*(sj2));
IkReal x238=((cj1)*(cj2)*(r02));
IkReal x239=((r01)*(sj0)*(x234));
IkReal x240=((r01)*(sj0)*(x235));
if( IKabs(((gconst109)*(((((IkReal(-1.00000000000000))*(x230)*(x231)))+(((cj3)*(x236)))+(((sj3)*(x240)))+(((sj3)*(x233)*(x235)))+(((sj3)*(x233)*(x234)))+(((IkReal(-1.00000000000000))*(x232)*(x237)))+(((sj3)*(x239)))+(((sj3)*(x238))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst109)*(((((IkReal(-1.00000000000000))*(x230)*(x238)))+(((IkReal(-1.00000000000000))*(x230)*(x239)))+(((cj3)*(x237)))+(((IkReal(-1.00000000000000))*(x230)*(x240)))+(((IkReal(-1.00000000000000))*(x230)*(x233)*(x235)))+(((IkReal(-1.00000000000000))*(x230)*(x233)*(x234)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((sj3)*(x236))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst109)*(((((IkReal(-1.00000000000000))*(x230)*(x231)))+(((cj3)*(x236)))+(((sj3)*(x240)))+(((sj3)*(x233)*(x235)))+(((sj3)*(x233)*(x234)))+(((IkReal(-1.00000000000000))*(x232)*(x237)))+(((sj3)*(x239)))+(((sj3)*(x238)))))), ((gconst109)*(((((IkReal(-1.00000000000000))*(x230)*(x238)))+(((IkReal(-1.00000000000000))*(x230)*(x239)))+(((cj3)*(x237)))+(((IkReal(-1.00000000000000))*(x230)*(x240)))+(((IkReal(-1.00000000000000))*(x230)*(x233)*(x235)))+(((IkReal(-1.00000000000000))*(x230)*(x233)*(x234)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((sj3)*(x236)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x241=IKsin(j5);
IkReal x242=IKcos(j5);
IkReal x243=((sj1)*(sj2));
IkReal x244=((cj2)*(sj1));
IkReal x245=((IkReal(1.00000000000000))*(cj1));
IkReal x246=((cj0)*(r00));
IkReal x247=((IkReal(1.00000000000000))*(sj0));
IkReal x248=((IkReal(1.00000000000000))*(cj0));
IkReal x249=((sj0)*(sj2));
IkReal x250=((sj3)*(x242));
IkReal x251=((sj3)*(x241));
IkReal x252=((cj3)*(x242));
IkReal x253=((cj3)*(x241));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x248)))+(((IkReal(-1.00000000000000))*(x253)))+(((IkReal(-1.00000000000000))*(x250))));
evalcond[1]=((((IkReal(2.16000000000000))*(x253)))+(((IkReal(2.16000000000000))*(x250)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x248))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x244)*(x246)))+(((r02)*(x243)))+(((IkReal(-1.00000000000000))*(sj2)*(x245)*(x246)))+(x251)+(((IkReal(-1.00000000000000))*(r01)*(x245)*(x249)))+(((IkReal(-1.00000000000000))*(r01)*(x244)*(x247)))+(((IkReal(-1.00000000000000))*(x252)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x245))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(2.16000000000000))*(x252)))+(((IkReal(-1.00000000000000))*(py)*(x245)*(x249)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(0.350000000000000))*(x244)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x245)))+(((IkReal(-1.00000000000000))*(px)*(x244)*(x248)))+(((IkReal(-2.16000000000000))*(x251)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x245)))+(((pz)*(x243)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(x244)*(x247))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x254=((cj2)*(sj1));
IkReal x255=((sj1)*(sj2));
IkReal x256=((cj1)*(sj2));
IkReal x257=((sj0)*(x255));
IkReal x258=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x259=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x255)))+(((r01)*(x257)))+(((IkReal(-1.00000000000000))*(r00)*(x258)))+(((IkReal(-1.00000000000000))*(r01)*(x259)))+(((r02)*(x256)))+(((r02)*(x254))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(-0.350000000000000))*(x255)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x256)))+(((pz)*(x254)))+(((py)*(x257)))+(((cj0)*(px)*(x255)))+(((IkReal(-1.00000000000000))*(py)*(x259)))+(((IkReal(-1.00000000000000))*(px)*(x258))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst111;
gconst111=IkReal(-2.00000000000000);
IkReal gconst112;
gconst112=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst113;
gconst113=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst114;
gconst114=IkReal(4.32000000000000);
IkReal gconst115;
gconst115=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst116;
gconst116=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst119;
gconst119=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst120;
gconst120=IkReal(4.32000000000000);
IkReal gconst121;
gconst121=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst122;
gconst122=IkReal(2.00000000000000);
IkReal gconst123;
gconst123=IkReal(-2.00000000000000);
IkReal gconst124;
gconst124=IkReal(-4.32000000000000);
IkReal gconst125;
gconst125=IkReal(4.32000000000000);
IkReal gconst126;
gconst126=IkReal(2.00000000000000);
IkReal gconst127;
gconst127=IkReal(-2.00000000000000);
IkReal gconst128;
gconst128=IkReal(-4.32000000000000);
IkReal gconst129;
gconst129=IkReal(4.32000000000000);
IkReal gconst130;
gconst130=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst131;
gconst131=IkReal(2.00000000000000);
IkReal gconst132;
gconst132=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst133;
gconst133=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst134;
gconst134=IkReal(-4.32000000000000);
IkReal gconst135;
gconst135=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst136;
gconst136=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst137;
gconst137=IkReal(2.00000000000000);
IkReal gconst138;
gconst138=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst139;
gconst139=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst140;
gconst140=IkReal(-4.32000000000000);
IkReal gconst141;
gconst141=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal x260=((gconst132)*(gconst134));
IkReal x261=((gconst137)*(gconst139));
IkReal x262=((IkReal(1.00000000000000))*(gconst136));
IkReal x263=((gconst130)*(gconst135));
IkReal x264=((gconst132)*(gconst133));
IkReal x265=((gconst138)*(gconst139));
IkReal x266=((gconst131)*(gconst135));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst140)*(x260)*(x262)))+(((x260)*(x261)))+(((x263)*(x265)))+(((gconst136)*(gconst141)*(x264)))+(((gconst136)*(gconst140)*(x266)))+(((IkReal(-1.00000000000000))*(gconst141)*(x262)*(x263)))+(((IkReal(-1.00000000000000))*(x261)*(x266)))+(((IkReal(-1.00000000000000))*(x264)*(x265))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x267=((gconst118)*(gconst119));
IkReal x268=((gconst110)*(gconst135));
IkReal x269=((gconst114)*(gconst117));
IkReal x270=((gconst123)*(gconst128));
IkReal x271=((gconst136)*(gconst141));
IkReal x272=((gconst113)*(gconst132));
IkReal x273=((IkReal(1.00000000000000))*(gconst124));
IkReal x274=((IkReal(1.00000000000000))*(gconst135));
IkReal x275=((gconst138)*(gconst139));
IkReal x276=((gconst131)*(gconst139));
IkReal x277=((gconst112)*(gconst116));
IkReal x278=((gconst124)*(gconst129));
IkReal x279=((gconst115)*(gconst122));
IkReal x280=((IkReal(1.00000000000000))*(gconst140));
IkReal x281=((gconst112)*(gconst139));
IkReal x282=((gconst121)*(gconst124));
IkReal x283=((gconst129)*(gconst136));
IkReal x284=((gconst132)*(gconst136));
IkReal x285=((gconst120)*(gconst126));
IkReal x286=((IkReal(1.00000000000000))*(gconst123));
IkReal x287=((gconst131)*(gconst137));
IkReal x288=((gconst116)*(gconst121));
IkReal x289=((IkReal(1.00000000000000))*(gconst116));
IkReal x290=((gconst131)*(gconst135));
IkReal x291=((gconst126)*(gconst140));
IkReal x292=((gconst111)*(gconst115));
IkReal x293=((IkReal(1.00000000000000))*(gconst125));
IkReal x294=((gconst116)*(gconst120));
IkReal x295=((gconst111)*(gconst135));
IkReal x296=((gconst113)*(gconst121));
IkReal x297=((gconst130)*(gconst138));
IkReal x298=((gconst126)*(gconst141));
IkReal x299=((IkReal(1.00000000000000))*(gconst119));
IkReal x300=((gconst132)*(gconst133));
IkReal x301=((gconst112)*(gconst133));
IkReal x302=((gconst110)*(gconst115));
IkReal x303=((gconst127)*(gconst139));
IkReal x304=((gconst123)*(gconst133));
IkReal x305=((IkReal(1.00000000000000))*(gconst120));
IkReal x306=((gconst132)*(gconst134));
IkReal x307=((gconst122)*(gconst125));
IkReal x308=((gconst125)*(gconst128));
IkReal x309=((gconst134)*(gconst137));
IkReal x310=((gconst116)*(gconst123));
IkReal x311=((gconst120)*(gconst136));
IkReal x312=((IkReal(1.00000000000000))*(gconst118));
IkReal x313=((gconst112)*(gconst113));
IkReal x314=((gconst111)*(gconst117));
IkReal x315=((IkReal(1.00000000000000))*(gconst121));
IkReal x316=((gconst124)*(gconst132));
IkReal x317=((gconst137)*(gconst139));
IkReal x318=((gconst114)*(gconst132));
IkReal x319=((gconst112)*(gconst119));
IkReal x320=((IkReal(1.00000000000000))*(gconst128));
IkReal x321=((gconst123)*(gconst134));
IkReal x322=((gconst111)*(gconst125));
IkReal x323=((gconst130)*(gconst135));
IkReal x324=((IkReal(1.00000000000000))*(gconst115));
IkReal x325=((gconst118)*(gconst139));
IkReal x326=((gconst116)*(gconst140));
IkReal x327=((gconst117)*(gconst139));
IkReal x328=((gconst112)*(gconst128));
IkReal x329=((gconst126)*(gconst129));
IkReal x330=((gconst127)*(gconst128));
IkReal x331=((gconst116)*(gconst141));
IkReal x332=((gconst115)*(gconst131));
IkReal x333=((gconst125)*(gconst131));
IkReal x334=((gconst136)*(gconst140));
IkReal x335=((gconst119)*(gconst127));
IkReal x336=((gconst113)*(gconst129));
IkReal x337=((gconst118)*(gconst128));
IkReal x338=((gconst128)*(gconst135));
IkReal x339=((gconst112)*(gconst136));
IkReal x340=((gconst110)*(gconst125));
IkReal x341=((gconst115)*(gconst130));
IkReal x342=((gconst121)*(gconst133));
IkReal x343=((gconst125)*(gconst130));
IkReal x344=((gconst122)*(gconst135));
IkReal x345=((gconst112)*(x296));
IkReal x346=((gconst126)*(gconst130)*(x274));
op[0]=((((x271)*(x300)))+(((IkReal(-1.00000000000000))*(gconst134)*(x280)*(x284)))+(((x290)*(x334)))+(((IkReal(-1.00000000000000))*(x275)*(x300)))+(((x275)*(x323)))+(((x306)*(x317)))+(((IkReal(-1.00000000000000))*(gconst130)*(x271)*(x274)))+(((IkReal(-1.00000000000000))*(gconst137)*(x274)*(x276))));
op[1]=((((IkReal(-1.00000000000000))*(gconst132)*(x273)*(x275)))+(((x290)*(x291)))+(((x271)*(x304)))+(((IkReal(-1.00000000000000))*(gconst137)*(x276)*(x293)))+(((gconst128)*(gconst137)*(x306)))+(((gconst123)*(gconst139)*(x309)))+(((IkReal(-1.00000000000000))*(gconst130)*(x271)*(x293)))+(((x283)*(x300)))+(((IkReal(-1.00000000000000))*(gconst130)*(x274)*(x298)))+(((x271)*(x316)))+(((IkReal(-1.00000000000000))*(gconst128)*(x274)*(x287)))+(((x303)*(x323)))+(((x275)*(x344)))+(((x275)*(x343)))+(((x298)*(x300)))+(((x333)*(x334)))+(((IkReal(-1.00000000000000))*(x300)*(x303)))+(((IkReal(-1.00000000000000))*(gconst138)*(x300)*(x320)))+(((IkReal(-1.00000000000000))*(gconst122)*(x271)*(x274)))+(((x297)*(x338)))+(((IkReal(-1.00000000000000))*(gconst133)*(x275)*(x286)))+(((IkReal(-1.00000000000000))*(gconst130)*(x274)*(x283)))+(((IkReal(-1.00000000000000))*(gconst126)*(x280)*(x306)))+(((IkReal(-1.00000000000000))*(gconst136)*(x280)*(x321))));
op[2]=((((IkReal(-1.00000000000000))*(gconst132)*(x273)*(x303)))+(((x317)*(x318)))+(((x275)*(x307)))+(((gconst123)*(gconst124)*(x271)))+(((IkReal(-1.00000000000000))*(gconst119)*(x274)*(x287)))+(((IkReal(-1.00000000000000))*(gconst133)*(x286)*(x303)))+(((x290)*(x326)))+(((x271)*(x301)))+(((x271)*(x272)))+(((IkReal(-1.00000000000000))*(gconst130)*(x274)*(x331)))+(((x323)*(x325)))+(((IkReal(-1.00000000000000))*(gconst117)*(x274)*(x276)))+(((x278)*(x284)))+(((IkReal(-1.00000000000000))*(gconst138)*(x299)*(x300)))+(((IkReal(-1.00000000000000))*(gconst130)*(x293)*(x298)))+(((IkReal(-1.00000000000000))*(gconst139)*(x300)*(x312)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst130)*(gconst136)*(x274)))+(((IkReal(-1.00000000000000))*(gconst128)*(x287)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x274)*(x329)))+(((x283)*(x304)))+(((IkReal(-1.00000000000000))*(gconst122)*(x274)*(x298)))+(((IkReal(-1.00000000000000))*(gconst114)*(x280)*(x284)))+(((x291)*(x333)))+(((x332)*(x334)))+(((gconst119)*(gconst137)*(x306)))+(((IkReal(-1.00000000000000))*(x275)*(x301)))+(((IkReal(-1.00000000000000))*(gconst127)*(x300)*(x320)))+(((IkReal(-1.00000000000000))*(gconst128)*(gconst132)*(gconst138)*(x273)))+(((IkReal(-1.00000000000000))*(gconst134)*(x280)*(x339)))+(((x298)*(x316)))+(((x295)*(x334)))+(((gconst119)*(gconst135)*(x297)))+(((x281)*(x309)))+(((x275)*(x341)))+(((x298)*(x304)))+(((x297)*(x308)))+(((x270)*(x309)))+(((IkReal(-1.00000000000000))*(x268)*(x271)))+(((IkReal(-1.00000000000000))*(gconst134)*(x284)*(x305)))+(((gconst122)*(gconst138)*(x338)))+(((IkReal(-1.00000000000000))*(gconst111)*(x274)*(x317)))+(((IkReal(-1.00000000000000))*(gconst122)*(x274)*(x283)))+(((IkReal(-1.00000000000000))*(gconst137)*(x276)*(x324)))+(((IkReal(-1.00000000000000))*(gconst130)*(x271)*(x324)))+(((x290)*(x311)))+(((IkReal(-1.00000000000000))*(gconst123)*(x273)*(x275)))+(((IkReal(-1.00000000000000))*(gconst126)*(x280)*(x321)))+(((x323)*(x330)))+(((IkReal(-1.00000000000000))*(gconst130)*(x283)*(x293)))+(((x303)*(x344)))+(((x303)*(x343)))+(((IkReal(-1.00000000000000))*(gconst116)*(x280)*(x306)))+(((IkReal(-1.00000000000000))*(gconst122)*(x271)*(x293)))+(((x306)*(x327)))+(((IkReal(-1.00000000000000))*(gconst133)*(gconst138)*(x270)))+(((x300)*(x329)))+(((IkReal(-1.00000000000000))*(x272)*(x275)))+(((x300)*(x331)))+(((x284)*(x342)))+(((x268)*(x275))));
op[3]=((((gconst121)*(gconst124)*(gconst132)*(gconst136)))+(((gconst115)*(gconst122)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst126)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst128)*(gconst131)*(gconst137)))+(((gconst122)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst129)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst126)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst134)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst132)*(gconst139)))+(((gconst120)*(gconst125)*(gconst131)*(gconst136)))+(((gconst113)*(gconst123)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst126)*(gconst130)*(gconst135)))+(((gconst117)*(gconst128)*(gconst132)*(gconst134)))+(((gconst112)*(gconst128)*(gconst134)*(gconst137)))+(((gconst119)*(gconst122)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst127)*(gconst128)*(gconst133)))+(((gconst119)*(gconst125)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst128)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst128)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst127)*(gconst132)*(gconst133)))+(((gconst114)*(gconst128)*(gconst132)*(gconst137)))+(((gconst122)*(gconst125)*(gconst128)*(gconst138)))+(((gconst116)*(gconst123)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst123)*(gconst134)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst138)*(gconst139)))+(((gconst115)*(gconst128)*(gconst130)*(gconst138)))+(((gconst112)*(gconst126)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst129)*(gconst135)*(gconst136)))+(((gconst123)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst129)*(gconst136)))+(((gconst116)*(gconst125)*(gconst131)*(gconst140)))+(((gconst119)*(gconst127)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst128)*(gconst132)*(gconst133)))+(((gconst124)*(gconst126)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst126)*(gconst129)*(gconst135)))+(((gconst115)*(gconst126)*(gconst131)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst132)*(gconst138)))+(((gconst118)*(gconst128)*(gconst130)*(gconst135)))+(((gconst121)*(gconst126)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst125)*(gconst131)*(gconst137)))+(((gconst121)*(gconst123)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst128)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst141)))+(((gconst119)*(gconst123)*(gconst134)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst130)*(gconst141)))+(((gconst120)*(gconst126)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst130)*(gconst141)))+(((gconst112)*(gconst124)*(gconst136)*(gconst141)))+(((gconst116)*(gconst129)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst124)*(gconst127)*(gconst128)*(gconst132)))+(((gconst110)*(gconst127)*(gconst135)*(gconst139)))+(((gconst110)*(gconst125)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst131)*(gconst139)))+(((gconst113)*(gconst126)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst133)*(gconst139)))+(((gconst115)*(gconst127)*(gconst130)*(gconst139)))+(((gconst117)*(gconst123)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst126)*(gconst134)*(gconst140)))+(((gconst122)*(gconst127)*(gconst128)*(gconst135)))+(((gconst112)*(gconst129)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst136)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst132)*(gconst139)))+(((gconst111)*(gconst125)*(gconst136)*(gconst140)))+(((gconst118)*(gconst125)*(gconst130)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst135)*(gconst136)))+(((gconst113)*(gconst129)*(gconst132)*(gconst136)))+(((gconst118)*(gconst122)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst136)*(gconst141)))+(((gconst123)*(gconst124)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst125)*(gconst126)*(gconst129)*(gconst130)))+(((gconst125)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst129)*(gconst130)*(gconst136)))+(((gconst116)*(gconst124)*(gconst132)*(gconst141)))+(((gconst111)*(gconst126)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst128)*(gconst131)*(gconst135)))+(((gconst110)*(gconst128)*(gconst135)*(gconst138)))+(((gconst123)*(gconst124)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst130)*(gconst136))));
op[4]=((((gconst117)*(gconst123)*(gconst128)*(gconst134)))+(((gconst112)*(gconst116)*(gconst133)*(gconst141)))+(((gconst119)*(gconst122)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst139)))+(((gconst111)*(gconst120)*(gconst135)*(gconst136)))+(((gconst116)*(gconst123)*(gconst124)*(gconst141)))+(((gconst116)*(gconst124)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst128)*(gconst138)))+(((gconst113)*(gconst121)*(gconst132)*(gconst136)))+(((gconst115)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst130)*(gconst135)))+(((gconst111)*(gconst125)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst126)*(gconst140)))+(((gconst123)*(gconst124)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst141)))+(((gconst110)*(gconst125)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst126)*(gconst134)))+(((gconst115)*(gconst116)*(gconst131)*(gconst140)))+(((gconst111)*(gconst116)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst141)))+(((gconst112)*(gconst117)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst129)*(gconst136)))+(((gconst113)*(gconst126)*(gconst129)*(gconst132)))+(((gconst120)*(gconst125)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst135)*(gconst141)))+(((gconst118)*(gconst122)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst128)*(gconst131)))+(((gconst110)*(gconst119)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst132)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst129)*(gconst135)))+(((gconst111)*(gconst115)*(gconst136)*(gconst140)))+(((gconst119)*(gconst125)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst134)*(gconst136)))+(((gconst112)*(gconst124)*(gconst129)*(gconst136)))+(((gconst112)*(gconst114)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst127)*(gconst132)))+(((gconst115)*(gconst122)*(gconst127)*(gconst139)))+(((gconst118)*(gconst119)*(gconst130)*(gconst135)))+(((gconst110)*(gconst118)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst126)*(gconst130)))+(((gconst121)*(gconst123)*(gconst126)*(gconst133)))+(((gconst116)*(gconst123)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst133)*(gconst138)))+(((gconst113)*(gconst123)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst132)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst129)*(gconst135)))+(((gconst110)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst135)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst116)*(gconst134)*(gconst140)))+(((gconst112)*(gconst119)*(gconst134)*(gconst137)))+(((gconst113)*(gconst123)*(gconst126)*(gconst141)))+(((gconst112)*(gconst121)*(gconst133)*(gconst136)))+(((gconst114)*(gconst119)*(gconst132)*(gconst137)))+(((gconst115)*(gconst118)*(gconst130)*(gconst139)))+(((gconst117)*(gconst119)*(gconst132)*(gconst134)))+(((gconst110)*(gconst115)*(gconst138)*(gconst139)))+(((gconst116)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst129)))+(((gconst121)*(gconst123)*(gconst124)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst131)*(gconst139)))+(((gconst112)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst130)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst128)*(gconst138)))+(((gconst112)*(gconst113)*(gconst136)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst138)))+(((gconst110)*(gconst127)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst135)*(gconst139)))+(((gconst121)*(gconst124)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst128)))+(((gconst118)*(gconst125)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst119)*(gconst131)*(gconst137)))+(((gconst115)*(gconst122)*(gconst128)*(gconst138)))+(((gconst114)*(gconst117)*(gconst132)*(gconst139)))+(((gconst116)*(gconst121)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst127)*(gconst133)))+(((gconst113)*(gconst116)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst137)*(gconst139)))+(((gconst112)*(gconst124)*(gconst126)*(gconst141)))+(((gconst115)*(gconst120)*(gconst131)*(gconst136)))+(((gconst115)*(gconst119)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst136)*(gconst140)))+(((gconst122)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst126)*(gconst135))));
op[5]=((((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst125)*(gconst137)))+(((gconst112)*(gconst121)*(gconst124)*(gconst136)))+(((gconst117)*(gconst119)*(gconst123)*(gconst134)))+(((gconst112)*(gconst124)*(gconst126)*(gconst129)))+(((gconst115)*(gconst119)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst122)*(gconst136)))+(((gconst110)*(gconst118)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst141)))+(((gconst110)*(gconst118)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((gconst112)*(gconst113)*(gconst129)*(gconst136)))+(((gconst114)*(gconst117)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst132)))+(((gconst112)*(gconst117)*(gconst128)*(gconst134)))+(((gconst115)*(gconst118)*(gconst122)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst126)))+(((gconst114)*(gconst119)*(gconst123)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst123)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst126)*(gconst141)))+(((gconst118)*(gconst119)*(gconst122)*(gconst135)))+(((gconst110)*(gconst115)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst122)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst129)))+(((gconst115)*(gconst118)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst125)*(gconst128)))+(((gconst112)*(gconst116)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst122)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst125)*(gconst131)))+(((gconst113)*(gconst123)*(gconst126)*(gconst129)))+(((gconst110)*(gconst119)*(gconst125)*(gconst138)))+(((gconst116)*(gconst121)*(gconst124)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst123)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst123)*(gconst134)))+(((gconst116)*(gconst121)*(gconst123)*(gconst133)))+(((gconst110)*(gconst115)*(gconst128)*(gconst138)))+(((gconst113)*(gconst116)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst128)*(gconst135)))+(((gconst115)*(gconst122)*(gconst127)*(gconst128)))+(((gconst110)*(gconst119)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst128)*(gconst138)))+(((gconst112)*(gconst114)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst128)*(gconst133)))+(((gconst110)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst129)))+(((gconst113)*(gconst121)*(gconst123)*(gconst136)))+(((gconst113)*(gconst116)*(gconst123)*(gconst141)))+(((gconst111)*(gconst115)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst129)*(gconst130)))+(((gconst115)*(gconst119)*(gconst122)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst124)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst128)*(gconst137)))+(((gconst112)*(gconst113)*(gconst126)*(gconst141)))+(((gconst114)*(gconst117)*(gconst123)*(gconst139)))+(((gconst118)*(gconst119)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst127)))+(((gconst121)*(gconst123)*(gconst124)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst124)*(gconst132)))+(((gconst113)*(gconst121)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((gconst112)*(gconst116)*(gconst124)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst127)))+(((gconst116)*(gconst123)*(gconst124)*(gconst129)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((gconst112)*(gconst121)*(gconst126)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst128)))+(((gconst111)*(gconst116)*(gconst125)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128))));
op[6]=((((x267)*(x268)))+(((x335)*(x340)))+(((IkReal(-1.00000000000000))*(gconst122)*(x288)*(x293)))+(((gconst110)*(gconst118)*(x308)))+(((IkReal(-1.00000000000000))*(gconst137)*(x292)*(x299)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(x293)))+(((IkReal(-1.00000000000000))*(x302)*(x329)))+(((IkReal(-1.00000000000000))*(gconst123)*(x267)*(x273)))+(((IkReal(-1.00000000000000))*(x267)*(x301)))+(((x269)*(x281)))+(((IkReal(-1.00000000000000))*(gconst118)*(x273)*(x328)))+(((IkReal(-1.00000000000000))*(gconst114)*(x285)*(x286)))+(((x292)*(x326)))+(((IkReal(-1.00000000000000))*(gconst141)*(x289)*(x302)))+(((IkReal(-1.00000000000000))*(gconst138)*(x299)*(x313)))+(((IkReal(-1.00000000000000))*(gconst134)*(x277)*(x305)))+(((IkReal(-1.00000000000000))*(gconst127)*(x273)*(x319)))+(((x313)*(x329)))+(((x302)*(x325)))+(((IkReal(-1.00000000000000))*(gconst114)*(x305)*(x339)))+(((IkReal(-1.00000000000000))*(gconst129)*(x279)*(x289)))+(((gconst114)*(gconst137)*(x319)))+(((IkReal(-1.00000000000000))*(gconst126)*(x279)*(x315)))+(((x279)*(x337)))+(((x279)*(x335)))+(((IkReal(-1.00000000000000))*(gconst129)*(x289)*(x340)))+(((x267)*(x341)))+(((x302)*(x330)))+(((x272)*(x288)))+(((IkReal(-1.00000000000000))*(gconst130)*(x288)*(x324)))+(((IkReal(-1.00000000000000))*(gconst119)*(x274)*(x314)))+(((x294)*(x332)))+(((x282)*(x310)))+(((IkReal(-1.00000000000000))*(gconst117)*(x299)*(x332)))+(((gconst112)*(gconst126)*(x282)))+(((IkReal(-1.00000000000000))*(x292)*(x327)))+(((gconst119)*(gconst132)*(x269)))+(((IkReal(-1.00000000000000))*(gconst114)*(x277)*(x280)))+(((IkReal(-1.00000000000000))*(gconst120)*(x289)*(x318)))+(((IkReal(-1.00000000000000))*(gconst128)*(x293)*(x314)))+(((IkReal(-1.00000000000000))*(gconst113)*(x286)*(x335)))+(((x285)*(x322)))+(((x277)*(x278)))+(((x277)*(x342)))+(((IkReal(-1.00000000000000))*(gconst113)*(x281)*(x312)))+(((gconst119)*(gconst138)*(x302)))+(((x294)*(x295)))+(((x267)*(x307)))+(((gconst113)*(gconst141)*(x277)))+(((gconst123)*(gconst126)*(x296)))+(((x296)*(x339)))+(((x269)*(x270)))+(((x292)*(x311)))+(((IkReal(-1.00000000000000))*(gconst113)*(x270)*(x312)))+(((IkReal(-1.00000000000000))*(x267)*(x272)))+(((gconst117)*(gconst134)*(x319)))+(((IkReal(-1.00000000000000))*(gconst127)*(x313)*(x320)))+(((IkReal(-1.00000000000000))*(x268)*(x288)))+(((IkReal(-1.00000000000000))*(gconst136)*(x302)*(x315)))+(((x310)*(x336))));
op[7]=((((IkReal(-1.00000000000000))*(gconst126)*(x302)*(x315)))+(((gconst126)*(x345)))+(((IkReal(-1.00000000000000))*(gconst128)*(x312)*(x313)))+(((IkReal(-1.00000000000000))*(gconst127)*(x299)*(x313)))+(((IkReal(-1.00000000000000))*(x279)*(x288)))+(((x267)*(x340)))+(((x302)*(x337)))+(((x302)*(x335)))+(((IkReal(-1.00000000000000))*(gconst110)*(x288)*(x293)))+(((x285)*(x292)))+(((gconst113)*(gconst123)*(x288)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(x285)))+(((x277)*(x336)))+(((IkReal(-1.00000000000000))*(gconst117)*(x292)*(x320)))+(((IkReal(-1.00000000000000))*(gconst113)*(x267)*(x286)))+(((IkReal(-1.00000000000000))*(gconst114)*(x286)*(x294)))+(((x294)*(x322)))+(((IkReal(-1.00000000000000))*(gconst129)*(x289)*(x302)))+(((IkReal(-1.00000000000000))*(gconst119)*(x293)*(x314)))+(((x277)*(x282)))+(((x269)*(x328)))+(((gconst119)*(gconst123)*(x269)))+(((x267)*(x279)))+(((IkReal(-1.00000000000000))*(gconst112)*(x267)*(x273))));
op[8]=((((IkReal(-1.00000000000000))*(gconst117)*(x292)*(x299)))+(((x269)*(x319)))+(((IkReal(-1.00000000000000))*(x288)*(x302)))+(((IkReal(-1.00000000000000))*(x267)*(x313)))+(((x277)*(x296)))+(((x267)*(x302)))+(((x292)*(x294)))+(((IkReal(-1.00000000000000))*(gconst114)*(x277)*(x305))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst158;
gconst158=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x347=((cj2)*(cj3));
IkReal x348=((cj0)*(r00));
IkReal x349=((sj2)*(sj3));
IkReal x350=((r01)*(sj0));
IkReal x351=((cj1)*(r02));
IkReal x352=((cj0)*(r01));
IkReal x353=((IkReal(1.00000000000000))*(cj3));
IkReal x354=((r02)*(sj1));
IkReal x355=((cj2)*(sj3));
IkReal x356=((r00)*(sj0));
IkReal x357=((cj1)*(cj3)*(sj2));
if( IKabs(((gconst158)*(((((cj3)*(x356)))+(((sj1)*(x350)*(x355)))+(((cj1)*(x349)*(x350)))+(((x351)*(x355)))+(((sj1)*(x348)*(x355)))+(((cj1)*(x348)*(x349)))+(((IkReal(-1.00000000000000))*(x349)*(x354)))+(((IkReal(-1.00000000000000))*(x352)*(x353))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(sj3)*(x356)))+(((IkReal(-1.00000000000000))*(sj2)*(x353)*(x354)))+(((sj1)*(x347)*(x350)))+(((sj3)*(x352)))+(((x347)*(x351)))+(((x350)*(x357)))+(((x348)*(x357)))+(((sj1)*(x347)*(x348))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst158)*(((((cj3)*(x356)))+(((sj1)*(x350)*(x355)))+(((cj1)*(x349)*(x350)))+(((x351)*(x355)))+(((sj1)*(x348)*(x355)))+(((cj1)*(x348)*(x349)))+(((IkReal(-1.00000000000000))*(x349)*(x354)))+(((IkReal(-1.00000000000000))*(x352)*(x353)))))), ((gconst158)*(((((IkReal(-1.00000000000000))*(sj3)*(x356)))+(((IkReal(-1.00000000000000))*(sj2)*(x353)*(x354)))+(((sj1)*(x347)*(x350)))+(((sj3)*(x352)))+(((x347)*(x351)))+(((x350)*(x357)))+(((x348)*(x357)))+(((sj1)*(x347)*(x348)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x358=IKsin(j5);
IkReal x359=IKcos(j5);
IkReal x360=((sj1)*(sj2));
IkReal x361=((cj2)*(sj1));
IkReal x362=((IkReal(2.16000000000000))*(sj3));
IkReal x363=((IkReal(1.00000000000000))*(cj1));
IkReal x364=((cj0)*(r00));
IkReal x365=((IkReal(1.00000000000000))*(sj0));
IkReal x366=((IkReal(1.00000000000000))*(cj0));
IkReal x367=((sj0)*(sj2));
IkReal x368=((cj3)*(x359));
IkReal x369=((cj3)*(x358));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x369)))+(((IkReal(-1.00000000000000))*(r01)*(x366)))+(((sj3)*(x359))));
evalcond[1]=((((IkReal(2.16000000000000))*(x369)))+(((IkReal(-1.00000000000000))*(x359)*(x362)))+(((IkReal(-1.00000000000000))*(py)*(x366)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x361)*(x364)))+(((IkReal(-1.00000000000000))*(r01)*(x363)*(x367)))+(x368)+(((sj3)*(x358)))+(((IkReal(-1.00000000000000))*(r01)*(x361)*(x365)))+(((IkReal(-1.00000000000000))*(sj2)*(x363)*(x364)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x363)))+(((r02)*(x360))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(px)*(x361)*(x366)))+(((IkReal(-1.00000000000000))*(py)*(x361)*(x365)))+(((IkReal(-1.00000000000000))*(x358)*(x362)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((pz)*(x360)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x363)))+(((IkReal(0.350000000000000))*(x361)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x363)))+(((IkReal(-2.16000000000000))*(x368)))+(((IkReal(-1.00000000000000))*(py)*(x363)*(x367)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x370=((cj0)*(px));
IkReal x371=((cj1)*(cj2));
IkReal x372=((sj1)*(sj2));
IkReal x373=((py)*(sj0));
cj5array[0]=((IkReal(-0.462962962962963))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((IkReal(1.54500000000000))+(((x370)*(x372)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-0.350000000000000))*(x372)))+(((IkReal(-1.00000000000000))*(x371)*(x373)))+(((x372)*(x373)))+(((cj2)*(pz)*(sj1)))+(((cj1)*(pz)*(sj2)))+(((IkReal(0.350000000000000))*(x371)))+(((IkReal(-1.00000000000000))*(x370)*(x371))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x374=((r01)*(sj0));
IkReal x375=((sj1)*(sj2));
IkReal x376=((cj0)*(r00));
IkReal x377=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x375)*(x376)))+(((IkReal(-1.00000000000000))*(sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x374)*(x377)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x376)*(x377)))+(((x374)*(x375)))+(((cj1)*(r02)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x378=((cj2)*(sj1));
IkReal x379=((sj1)*(sj2));
IkReal x380=((cj1)*(sj2));
IkReal x381=((sj0)*(x379));
IkReal x382=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x383=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r02)*(x378)))+(((r02)*(x380)))+(((cj0)*(r00)*(x379)))+(((IkReal(-1.00000000000000))*(r00)*(x382)))+(((IkReal(-1.00000000000000))*(r01)*(x383)))+(((r01)*(x381))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(-1.00000000000000))*(py)*(x383)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((cj0)*(px)*(x379)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-0.350000000000000))*(x379)))+(((IkReal(-1.00000000000000))*(px)*(x382)))+(((pz)*(x378)))+(((pz)*(x380)))+(((py)*(x381))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x384=((IkReal(1.00000000000000))*(cj5));
IkReal x385=((cj0)*(r01));
IkReal x386=((r00)*(sj0));
IkReal x387=((IkReal(1.00000000000000))*(sj5));
IkReal x388=((cj0)*(r00));
IkReal x389=((cj1)*(sj2));
IkReal x390=((cj2)*(sj1));
IkReal x391=((r01)*(sj0));
IkReal x392=((cj1)*(cj2)*(r02));
IkReal x393=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x387)*(x393)))+(((sj5)*(x388)*(x390)))+(((sj5)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(x384)*(x385)))+(((sj5)*(x388)*(x389)))+(((cj5)*(x386)))+(((sj5)*(x392)))+(((sj5)*(x389)*(x391))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x384)*(x392)))+(((sj5)*(x386)))+(((cj5)*(x393)))+(((IkReal(-1.00000000000000))*(x384)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(x384)*(x388)*(x390)))+(((IkReal(-1.00000000000000))*(x384)*(x389)*(x391)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(-1.00000000000000))*(x384)*(x388)*(x389))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(x387)*(x393)))+(((sj5)*(x388)*(x390)))+(((sj5)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(x384)*(x385)))+(((sj5)*(x388)*(x389)))+(((cj5)*(x386)))+(((sj5)*(x392)))+(((sj5)*(x389)*(x391)))))), ((gconst55)*(((((IkReal(-1.00000000000000))*(x384)*(x392)))+(((sj5)*(x386)))+(((cj5)*(x393)))+(((IkReal(-1.00000000000000))*(x384)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(x384)*(x388)*(x390)))+(((IkReal(-1.00000000000000))*(x384)*(x389)*(x391)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(-1.00000000000000))*(x384)*(x388)*(x389)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x394=IKcos(j3);
IkReal x395=IKsin(j3);
IkReal x396=((sj1)*(sj2));
IkReal x397=((cj2)*(sj1));
IkReal x398=((IkReal(2.16000000000000))*(cj5));
IkReal x399=((IkReal(1.00000000000000))*(cj1));
IkReal x400=((cj0)*(r00));
IkReal x401=((IkReal(1.00000000000000))*(sj0));
IkReal x402=((IkReal(1.00000000000000))*(cj0));
IkReal x403=((IkReal(1.00000000000000))*(cj5));
IkReal x404=((sj0)*(sj2));
IkReal x405=((sj5)*(x395));
IkReal x406=((sj5)*(x394));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x395)*(x403)))+(((IkReal(-1.00000000000000))*(r01)*(x402)))+(((IkReal(-1.00000000000000))*(x406))));
evalcond[1]=((((x395)*(x398)))+(((IkReal(2.16000000000000))*(x406)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x402))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x399)*(x404)))+(((IkReal(-1.00000000000000))*(x397)*(x400)))+(((IkReal(-1.00000000000000))*(x394)*(x403)))+(((IkReal(-1.00000000000000))*(r01)*(x397)*(x401)))+(x405)+(((IkReal(-1.00000000000000))*(sj2)*(x399)*(x400)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x399)))+(((r02)*(x396))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(py)*(x397)*(x401)))+(((IkReal(-1.00000000000000))*(py)*(x399)*(x404)))+(((x394)*(x398)))+(((pz)*(x396)))+(((IkReal(-1.00000000000000))*(px)*(x397)*(x402)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(0.350000000000000))*(x397)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x399)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x399)))+(((IkReal(-2.16000000000000))*(x405)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x407=((cj2)*(sj1));
IkReal x408=((sj1)*(sj2));
IkReal x409=((cj1)*(sj2));
IkReal x410=((sj0)*(x408));
IkReal x411=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x412=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r01)*(x410)))+(((IkReal(-1.00000000000000))*(r01)*(x412)))+(((IkReal(-1.00000000000000))*(r00)*(x411)))+(((cj0)*(r00)*(x408)))+(((r02)*(x407)))+(((r02)*(x409))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(-0.350000000000000))*(x408)))+(((py)*(x410)))+(((pz)*(x409)))+(((pz)*(x407)))+(((IkReal(-1.00000000000000))*(px)*(x411)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((cj0)*(px)*(x408)))+(((IkReal(-1.00000000000000))*(py)*(x412))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x413=((IkReal(1.00000000000000))*(cj1));
IkReal x414=((cj2)*(cj5));
IkReal x415=((cj0)*(sj5));
IkReal x416=((cj5)*(r00));
IkReal x417=((sj2)*(sj5));
IkReal x418=((r01)*(sj0));
IkReal x419=((r02)*(sj1));
IkReal x420=((IkReal(1.00000000000000))*(sj1));
IkReal x421=((cj2)*(sj5));
IkReal x422=((cj5)*(sj2));
if( IKabs(((gconst56)*(((((sj0)*(x416)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x415)*(x420)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((IkReal(-1.00000000000000))*(x418)*(x420)*(x421)))+(((IkReal(-1.00000000000000))*(r02)*(x413)*(x421)))+(((IkReal(-1.00000000000000))*(x413)*(x417)*(x418)))+(((x417)*(x419)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x413)*(x415))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(r02)*(x413)*(x414)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x414)*(x418)*(x420)))+(((x419)*(x422)))+(((r01)*(x415)))+(((IkReal(-1.00000000000000))*(x413)*(x418)*(x422)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x414)*(x420)))+(((IkReal(-1.00000000000000))*(cj0)*(sj2)*(x413)*(x416))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((sj0)*(x416)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x415)*(x420)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((IkReal(-1.00000000000000))*(x418)*(x420)*(x421)))+(((IkReal(-1.00000000000000))*(r02)*(x413)*(x421)))+(((IkReal(-1.00000000000000))*(x413)*(x417)*(x418)))+(((x417)*(x419)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x413)*(x415)))))), ((gconst56)*(((((IkReal(-1.00000000000000))*(r02)*(x413)*(x414)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x414)*(x418)*(x420)))+(((x419)*(x422)))+(((r01)*(x415)))+(((IkReal(-1.00000000000000))*(x413)*(x418)*(x422)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x414)*(x420)))+(((IkReal(-1.00000000000000))*(cj0)*(sj2)*(x413)*(x416)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x423=IKcos(j3);
IkReal x424=IKsin(j3);
IkReal x425=((sj1)*(sj2));
IkReal x426=((cj2)*(sj1));
IkReal x427=((IkReal(2.16000000000000))*(cj5));
IkReal x428=((IkReal(1.00000000000000))*(cj1));
IkReal x429=((cj0)*(r00));
IkReal x430=((IkReal(1.00000000000000))*(sj0));
IkReal x431=((IkReal(1.00000000000000))*(cj0));
IkReal x432=((sj0)*(sj2));
IkReal x433=((sj5)*(x424));
IkReal x434=((sj5)*(x423));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x434)))+(((cj5)*(x424)))+(((IkReal(-1.00000000000000))*(r01)*(x431))));
evalcond[1]=((((IkReal(2.16000000000000))*(x434)))+(((IkReal(-1.00000000000000))*(x424)*(x427)))+(((IkReal(-1.00000000000000))*(py)*(x431)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x426)*(x429)))+(((r02)*(x425)))+(((cj5)*(x423)))+(x433)+(((IkReal(-1.00000000000000))*(sj2)*(x428)*(x429)))+(((IkReal(-1.00000000000000))*(r01)*(x426)*(x430)))+(((IkReal(-1.00000000000000))*(r01)*(x428)*(x432)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x428))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(py)*(x426)*(x430)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x428)))+(((IkReal(0.350000000000000))*(x426)))+(((IkReal(-1.00000000000000))*(py)*(x428)*(x432)))+(((pz)*(x425)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x426)*(x431)))+(((IkReal(-2.16000000000000))*(x433)))+(((IkReal(-1.00000000000000))*(x423)*(x427)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x428)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x435=((cj2)*(sj1));
IkReal x436=((sj1)*(sj2));
IkReal x437=((cj1)*(sj2));
IkReal x438=((sj0)*(x436));
IkReal x439=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x440=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((cj0)*(r00)*(x436)))+(((r02)*(x437)))+(((r02)*(x435)))+(((IkReal(-1.00000000000000))*(r01)*(x440)))+(((IkReal(-1.00000000000000))*(r00)*(x439)))+(((r01)*(x438))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((pz)*(x437)))+(((pz)*(x435)))+(((IkReal(2.16000000000000))*(sj4)))+(((py)*(x438)))+(((cj0)*(px)*(x436)))+(((IkReal(-1.00000000000000))*(py)*(x440)))+(((IkReal(-0.350000000000000))*(x436)))+(((IkReal(-1.00000000000000))*(px)*(x439))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x441=((cj1)*(sj2));
IkReal x442=((sj1)*(sj2));
IkReal x443=((cj0)*(r00));
IkReal x444=((r01)*(sj0));
IkReal x445=((IkReal(1.00000000000000))*(cj2));
IkReal x446=((cj2)*(sj1));
IkReal x447=((cj0)*(px));
IkReal x448=((cj1)*(sj0)*(x445));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(-1.00000000000000))+(((r02)*(x446)))+(((r02)*(x441)))+(((x442)*(x444)))+(((x442)*(x443)))+(((IkReal(-1.00000000000000))*(cj1)*(x444)*(x445)))+(((IkReal(-1.00000000000000))*(cj1)*(x443)*(x445))));
evalcond[3]=((((r02)*(x442)))+(((IkReal(-1.00000000000000))*(sj1)*(x443)*(x445)))+(((IkReal(-1.00000000000000))*(sj1)*(x444)*(x445)))+(((IkReal(-1.00000000000000))*(x441)*(x444)))+(((IkReal(-1.00000000000000))*(x441)*(x443)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x445))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x441)))+(((pz)*(x446)))+(((IkReal(-0.350000000000000))*(x442)))+(((IkReal(-1.00000000000000))*(py)*(x448)))+(((x442)*(x447)))+(((py)*(sj0)*(x442)))+(((IkReal(-1.00000000000000))*(cj1)*(x445)*(x447))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x449=((cj2)*(sj1));
IkReal x450=((IkReal(6.32911392405063))*(sj2));
IkReal x451=((cj0)*(px));
IkReal x452=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((pz)*(sj1)*(x450)))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(cj1)*(x450)*(x452)))+(((IkReal(-6.32911392405063))*(x449)*(x452)))+(((IkReal(-6.32911392405063))*(x449)*(x451)))+(((IkReal(2.21518987341772))*(x449)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(2.21518987341772))*(cj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))))+IKsqr(((IkReal(-0.917721518987342))+(((pz)*(sj1)*(x450)))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(cj1)*(x450)*(x452)))+(((IkReal(-6.32911392405063))*(x449)*(x452)))+(((IkReal(-6.32911392405063))*(x449)*(x451)))+(((IkReal(2.21518987341772))*(x449)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))), ((IkReal(-0.917721518987342))+(((pz)*(sj1)*(x450)))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(cj1)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(cj1)*(x450)*(x452)))+(((IkReal(-6.32911392405063))*(x449)*(x452)))+(((IkReal(-6.32911392405063))*(x449)*(x451)))+(((IkReal(2.21518987341772))*(x449)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x453=((cj2)*(sj1));
IkReal x454=((cj1)*(sj2));
IkReal x455=((IkReal(1.00000000000000))*(py));
IkReal x456=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x455)))+(((IkReal(-0.158000000000000))*(IKsin(j3))))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(x454)*(x456)))+(((IkReal(0.350000000000000))*(x453)))+(((IkReal(0.350000000000000))*(x454)))+(((IkReal(-1.00000000000000))*(x453)*(x456)))+(((IkReal(-1.00000000000000))*(sj0)*(x454)*(x455)))+(((IkReal(-1.00000000000000))*(sj0)*(x453)*(x455)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x457=((cj1)*(sj2));
IkReal x458=((sj1)*(sj2));
IkReal x459=((cj0)*(r00));
IkReal x460=((r01)*(sj0));
IkReal x461=((IkReal(1.00000000000000))*(cj2));
IkReal x462=((cj2)*(sj1));
IkReal x463=((cj0)*(px));
IkReal x464=((cj1)*(sj0)*(x461));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(1.00000000000000))+(((r02)*(x462)))+(((r02)*(x457)))+(((x458)*(x459)))+(((x458)*(x460)))+(((IkReal(-1.00000000000000))*(cj1)*(x459)*(x461)))+(((IkReal(-1.00000000000000))*(cj1)*(x460)*(x461))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x457)*(x460)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x461)))+(((IkReal(-1.00000000000000))*(sj1)*(x459)*(x461)))+(((r02)*(x458)))+(((IkReal(-1.00000000000000))*(sj1)*(x460)*(x461)))+(((IkReal(-1.00000000000000))*(x457)*(x459))));
evalcond[4]=((IkReal(-0.615000000000000))+(((py)*(sj0)*(x458)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x461)*(x463)))+(((IkReal(-1.00000000000000))*(py)*(x464)))+(((IkReal(-0.350000000000000))*(x458)))+(((pz)*(x457)))+(((pz)*(x462)))+(((x458)*(x463))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x465=((cj2)*(sj1));
IkReal x466=((IkReal(6.32911392405063))*(cj1));
IkReal x467=((cj0)*(px));
IkReal x468=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((cj2)*(pz)*(x466)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(x465)))+(((sj2)*(x466)*(x467)))+(((sj2)*(x466)*(x468)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((IkReal(7.59493670886076))*(cj2)))+(((IkReal(6.32911392405063))*(x465)*(x468)))+(((IkReal(6.32911392405063))*(x465)*(x467))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))))+IKsqr(((IkReal(0.917721518987342))+(((cj2)*(pz)*(x466)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(x465)))+(((sj2)*(x466)*(x467)))+(((sj2)*(x466)*(x468)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((IkReal(7.59493670886076))*(cj2)))+(((IkReal(6.32911392405063))*(x465)*(x468)))+(((IkReal(6.32911392405063))*(x465)*(x467)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))), ((IkReal(0.917721518987342))+(((cj2)*(pz)*(x466)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(x465)))+(((sj2)*(x466)*(x467)))+(((sj2)*(x466)*(x468)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((IkReal(7.59493670886076))*(cj2)))+(((IkReal(6.32911392405063))*(x465)*(x468)))+(((IkReal(6.32911392405063))*(x465)*(x467)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x469=((cj2)*(sj1));
IkReal x470=((cj1)*(sj2));
IkReal x471=((IkReal(1.00000000000000))*(py));
IkReal x472=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x471)))+(((IkReal(0.158000000000000))*(IKsin(j3))))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x469)*(x472)))+(((pz)*(sj1)*(sj2)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(0.350000000000000))*(x470)))+(((IkReal(0.350000000000000))*(x469)))+(((IkReal(-1.00000000000000))*(sj0)*(x470)*(x471)))+(((IkReal(-1.00000000000000))*(sj0)*(x469)*(x471)))+(((IkReal(-1.00000000000000))*(x470)*(x472)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x473=((IkReal(1.00000000000000))*(r01));
IkReal x474=((cj2)*(sj1));
IkReal x475=((IkReal(1.00000000000000))*(cj1));
IkReal x476=((cj0)*(r00));
if( IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x473))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((IkReal(-1.00000000000000))*(x474)*(x476)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(cj1)*(sj0)*(sj2)*(x473)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x475)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj0)*(x473)*(x474))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x473)))))), ((gconst57)*(((((IkReal(-1.00000000000000))*(x474)*(x476)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(cj1)*(sj0)*(sj2)*(x473)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x475)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj0)*(x473)*(x474)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x477=IKsin(j3);
IkReal x478=IKcos(j3);
IkReal x479=((sj1)*(sj2));
IkReal x480=((cj2)*(sj1));
IkReal x481=((IkReal(0.158000000000000))*(sj4));
IkReal x482=((IkReal(1.00000000000000))*(cj1));
IkReal x483=((cj0)*(r00));
IkReal x484=((IkReal(1.00000000000000))*(sj0));
IkReal x485=((IkReal(1.00000000000000))*(cj0));
IkReal x486=((sj0)*(sj2));
IkReal x487=((cj4)*(x477));
IkReal x488=((cj4)*(x478));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x487)))+(((IkReal(-1.00000000000000))*(r01)*(x485))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x477)*(x481)))+(((px)*(sj0)))+(((IkReal(2.16000000000000))*(x487)))+(((IkReal(-1.00000000000000))*(py)*(x485))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x482)))+(((IkReal(-1.00000000000000))*(r01)*(x482)*(x486)))+(((IkReal(-1.00000000000000))*(r01)*(x480)*(x484)))+(((IkReal(-1.00000000000000))*(x488)))+(((r02)*(x479)))+(((IkReal(-1.00000000000000))*(x480)*(x483)))+(((IkReal(-1.00000000000000))*(sj2)*(x482)*(x483))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x480)))+(((IkReal(-1.00000000000000))*(px)*(x480)*(x485)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x482)))+(((IkReal(-1.00000000000000))*(py)*(x480)*(x484)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((pz)*(x479)))+(((IkReal(-1.00000000000000))*(py)*(x482)*(x486)))+(((IkReal(2.16000000000000))*(x488)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x482)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x478)*(x481))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x489=((cj2)*(sj1));
IkReal x490=((sj1)*(sj2));
IkReal x491=((cj1)*(sj2));
IkReal x492=((sj0)*(x490));
IkReal x493=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x494=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x493)))+(sj4)+(((IkReal(-1.00000000000000))*(r01)*(x494)))+(((r02)*(x491)))+(((r02)*(x489)))+(((cj0)*(r00)*(x490)))+(((r01)*(x492))));
evalcond[2]=((IkReal(1.54500000000000))+(((pz)*(x491)))+(((py)*(x492)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x489)))+(((IkReal(-2.16000000000000))*(sj4)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(px)*(x493)))+(((cj0)*(px)*(x490)))+(((IkReal(-0.350000000000000))*(x490)))+(((IkReal(-1.00000000000000))*(py)*(x494))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x495=((cj1)*(sj2));
IkReal x496=((sj1)*(sj2));
IkReal x497=((cj0)*(r00));
IkReal x498=((r01)*(sj0));
IkReal x499=((IkReal(1.00000000000000))*(cj2));
IkReal x500=((cj2)*(sj1));
IkReal x501=((cj0)*(px));
IkReal x502=((cj1)*(sj0)*(x499));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(1.00000000000000))+(((r02)*(x495)))+(((x496)*(x498)))+(((x496)*(x497)))+(((IkReal(-1.00000000000000))*(cj1)*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(cj1)*(x497)*(x499)))+(((r02)*(x500))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj1)*(r02)*(x499)))+(((r02)*(x496)))+(((IkReal(-1.00000000000000))*(sj1)*(x497)*(x499)))+(((IkReal(-1.00000000000000))*(sj1)*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(x495)*(x497)))+(((IkReal(-1.00000000000000))*(x495)*(x498))));
evalcond[4]=((IkReal(-0.615000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x499)*(x501)))+(((pz)*(x495)))+(((IkReal(-1.00000000000000))*(py)*(x502)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((py)*(sj0)*(x496)))+(((x496)*(x501)))+(((IkReal(-0.350000000000000))*(x496)))+(((pz)*(x500))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x503=((cj2)*(sj1));
IkReal x504=((IkReal(6.32911392405063))*(sj2));
IkReal x505=((cj0)*(px));
IkReal x506=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(2.21518987341772))*(x503)))+(((IkReal(-6.32911392405063))*(x503)*(x506)))+(((IkReal(-6.32911392405063))*(x503)*(x505)))+(((IkReal(-7.59493670886076))*(cj2)))+(((pz)*(sj1)*(x504)))+(((IkReal(-1.00000000000000))*(cj1)*(x504)*(x505)))+(((IkReal(-1.00000000000000))*(cj1)*(x504)*(x506)))+(((IkReal(2.21518987341772))*(cj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))))+IKsqr(((IkReal(-0.917721518987342))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(2.21518987341772))*(x503)))+(((IkReal(-6.32911392405063))*(x503)*(x506)))+(((IkReal(-6.32911392405063))*(x503)*(x505)))+(((IkReal(-7.59493670886076))*(cj2)))+(((pz)*(sj1)*(x504)))+(((IkReal(-1.00000000000000))*(cj1)*(x504)*(x505)))+(((IkReal(-1.00000000000000))*(cj1)*(x504)*(x506)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))), ((IkReal(-0.917721518987342))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(2.21518987341772))*(x503)))+(((IkReal(-6.32911392405063))*(x503)*(x506)))+(((IkReal(-6.32911392405063))*(x503)*(x505)))+(((IkReal(-7.59493670886076))*(cj2)))+(((pz)*(sj1)*(x504)))+(((IkReal(-1.00000000000000))*(cj1)*(x504)*(x505)))+(((IkReal(-1.00000000000000))*(cj1)*(x504)*(x506)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x507=((cj2)*(sj1));
IkReal x508=((cj1)*(sj2));
IkReal x509=((IkReal(1.00000000000000))*(py));
IkReal x510=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x509)))+(((IkReal(-0.158000000000000))*(IKsin(j3))))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(sj0)*(x507)*(x509)))+(((IkReal(0.350000000000000))*(x508)))+(((IkReal(0.350000000000000))*(x507)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(sj0)*(x508)*(x509)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x508)*(x510)))+(((IkReal(-1.00000000000000))*(x507)*(x510)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x511=((cj1)*(sj2));
IkReal x512=((sj1)*(sj2));
IkReal x513=((cj0)*(r00));
IkReal x514=((r01)*(sj0));
IkReal x515=((IkReal(1.00000000000000))*(cj2));
IkReal x516=((cj2)*(sj1));
IkReal x517=((cj0)*(px));
IkReal x518=((cj1)*(sj0)*(x515));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x514)*(x515)))+(((IkReal(-1.00000000000000))*(cj1)*(x513)*(x515)))+(((x512)*(x514)))+(((x512)*(x513)))+(((r02)*(x516)))+(((r02)*(x511))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj1)*(x514)*(x515)))+(((IkReal(-1.00000000000000))*(sj1)*(x513)*(x515)))+(((r02)*(x512)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x515)))+(((IkReal(-1.00000000000000))*(x511)*(x514)))+(((IkReal(-1.00000000000000))*(x511)*(x513))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x515)*(x517)))+(((pz)*(x511)))+(((pz)*(x516)))+(((x512)*(x517)))+(((py)*(sj0)*(x512)))+(((IkReal(-1.00000000000000))*(py)*(x518)))+(((IkReal(-0.350000000000000))*(x512))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x519=((cj2)*(sj1));
IkReal x520=((IkReal(6.32911392405063))*(cj1));
IkReal x521=((cj0)*(px));
IkReal x522=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((IkReal(6.32911392405063))*(x519)*(x521)))+(((IkReal(6.32911392405063))*(x519)*(x522)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((IkReal(7.59493670886076))*(cj2)))+(((cj2)*(pz)*(x520)))+(((IkReal(-2.21518987341772))*(x519)))+(((sj2)*(x520)*(x521)))+(((sj2)*(x520)*(x522))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))))+IKsqr(((IkReal(0.917721518987342))+(((IkReal(6.32911392405063))*(x519)*(x521)))+(((IkReal(6.32911392405063))*(x519)*(x522)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((IkReal(7.59493670886076))*(cj2)))+(((cj2)*(pz)*(x520)))+(((IkReal(-2.21518987341772))*(x519)))+(((sj2)*(x520)*(x521)))+(((sj2)*(x520)*(x522)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))), ((IkReal(0.917721518987342))+(((IkReal(6.32911392405063))*(x519)*(x521)))+(((IkReal(6.32911392405063))*(x519)*(x522)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((IkReal(7.59493670886076))*(cj2)))+(((cj2)*(pz)*(x520)))+(((IkReal(-2.21518987341772))*(x519)))+(((sj2)*(x520)*(x521)))+(((sj2)*(x520)*(x522)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x523=((cj2)*(sj1));
IkReal x524=((cj1)*(sj2));
IkReal x525=((IkReal(1.00000000000000))*(py));
IkReal x526=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(0.158000000000000))*(IKsin(j3))))+(((IkReal(-1.00000000000000))*(cj0)*(x525)))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(sj0)*(x524)*(x525)))+(((IkReal(-1.00000000000000))*(x524)*(x526)))+(((IkReal(-1.00000000000000))*(x523)*(x526)))+(((IkReal(-1.00000000000000))*(sj0)*(x523)*(x525)))+(((pz)*(sj1)*(sj2)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(0.350000000000000))*(x523)))+(((IkReal(0.350000000000000))*(x524)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x527=((cj1)*(sj2));
IkReal x528=((r01)*(sj0));
IkReal x529=((cj0)*(r00));
IkReal x530=((cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x527)*(x528)))+(((x527)*(x529)))+(((x528)*(x530)))+(((cj1)*(cj2)*(r02)))+(((x529)*(x530))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x527)*(x528)))+(((x527)*(x529)))+(((x528)*(x530)))+(((cj1)*(cj2)*(r02)))+(((x529)*(x530)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x531=IKsin(j3);
IkReal x532=IKcos(j3);
IkReal x533=((sj1)*(sj2));
IkReal x534=((cj2)*(sj1));
IkReal x535=((IkReal(0.158000000000000))*(sj4));
IkReal x536=((IkReal(1.00000000000000))*(sj0));
IkReal x537=((IkReal(1.00000000000000))*(cj1));
IkReal x538=((cj0)*(r00));
IkReal x539=((IkReal(1.00000000000000))*(cj0));
IkReal x540=((cj1)*(sj2));
IkReal x541=((cj4)*(x531));
IkReal x542=((cj4)*(x532));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x539)))+(x541));
evalcond[1]=((((IkReal(-1.00000000000000))*(x531)*(x535)))+(((IkReal(-2.16000000000000))*(x541)))+(((IkReal(-1.00000000000000))*(py)*(x539)))+(((px)*(sj0))));
evalcond[2]=((((r02)*(x533)))+(((IkReal(-1.00000000000000))*(sj2)*(x537)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(x536)*(x540)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x537)))+(x542)+(((IkReal(-1.00000000000000))*(r01)*(x534)*(x536)))+(((IkReal(-1.00000000000000))*(x534)*(x538))));
evalcond[3]=((IkReal(-0.145000000000000))+(((pz)*(x533)))+(((IkReal(-1.00000000000000))*(x532)*(x535)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x537)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x537)))+(((IkReal(-1.00000000000000))*(px)*(x534)*(x539)))+(((IkReal(-1.00000000000000))*(py)*(x534)*(x536)))+(((IkReal(0.350000000000000))*(x534)))+(((IkReal(-1.00000000000000))*(py)*(x536)*(x540)))+(((IkReal(-2.16000000000000))*(x542)))+(((IkReal(0.350000000000000))*(x540)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x543=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x544=((cj4)*(cj5));
IkReal x545=((cj0)*(r01));
IkReal x546=((IkReal(3.95000000000000))*(sj4));
IkReal x547=((IkReal(0.253164556962025))*(x543));
IkReal x548=((r00)*(sj0));
IkReal x549=((IkReal(25.0000000000000))*(cj0)*(py));
IkReal x550=((IkReal(25.0000000000000))*(px)*(sj0));
if( IKabs(((x547)*(((((IkReal(54.0000000000000))*(x548)))+(((IkReal(-1.00000000000000))*(x549)))+(x550)+(((IkReal(-54.0000000000000))*(x545))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x547)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x545)*(x546)))+(((x544)*(x549)))+(((IkReal(-1.00000000000000))*(x544)*(x550)))+(((IkReal(54.0000000000000))*(x544)*(x545)))+(((IkReal(-54.0000000000000))*(x544)*(x548)))+(((x546)*(x548))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x547)*(((((IkReal(54.0000000000000))*(x548)))+(((IkReal(-1.00000000000000))*(x549)))+(x550)+(((IkReal(-54.0000000000000))*(x545)))))))+IKsqr(((x547)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x545)*(x546)))+(((x544)*(x549)))+(((IkReal(-1.00000000000000))*(x544)*(x550)))+(((IkReal(54.0000000000000))*(x544)*(x545)))+(((IkReal(-54.0000000000000))*(x544)*(x548)))+(((x546)*(x548)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x547)*(((((IkReal(54.0000000000000))*(x548)))+(((IkReal(-1.00000000000000))*(x549)))+(x550)+(((IkReal(-54.0000000000000))*(x545)))))), ((x547)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x545)*(x546)))+(((x544)*(x549)))+(((IkReal(-1.00000000000000))*(x544)*(x550)))+(((IkReal(54.0000000000000))*(x544)*(x545)))+(((IkReal(-54.0000000000000))*(x544)*(x548)))+(((x546)*(x548)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x551=IKsin(j3);
IkReal x552=IKcos(j3);
IkReal x553=((sj1)*(sj2));
IkReal x554=((cj2)*(sj1));
IkReal x555=((IkReal(0.158000000000000))*(sj4));
IkReal x556=((IkReal(1.00000000000000))*(cj1));
IkReal x557=((cj0)*(r00));
IkReal x558=((IkReal(1.00000000000000))*(sj0));
IkReal x559=((IkReal(1.00000000000000))*(cj0));
IkReal x560=((sj0)*(sj2));
IkReal x561=((sj5)*(x551));
IkReal x562=((sj5)*(x552));
IkReal x563=((cj4)*(cj5)*(x552));
IkReal x564=((cj4)*(cj5)*(x551));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x559)))+(((IkReal(-1.00000000000000))*(x564)))+(((IkReal(-1.00000000000000))*(x562))));
evalcond[1]=((((IkReal(2.16000000000000))*(x562)))+(((IkReal(2.16000000000000))*(x564)))+(((IkReal(-1.00000000000000))*(x551)*(x555)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x559))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x556)*(x560)))+(((r02)*(x553)))+(((IkReal(-1.00000000000000))*(x554)*(x557)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x556)))+(((IkReal(-1.00000000000000))*(r01)*(x554)*(x558)))+(x561)+(((IkReal(-1.00000000000000))*(x563)))+(((IkReal(-1.00000000000000))*(sj2)*(x556)*(x557))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x556)))+(((IkReal(2.16000000000000))*(x563)))+(((IkReal(-2.16000000000000))*(x561)))+(((pz)*(x553)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x556)))+(((IkReal(-1.00000000000000))*(py)*(x554)*(x558)))+(((IkReal(-1.00000000000000))*(py)*(x556)*(x560)))+(((IkReal(-1.00000000000000))*(px)*(x554)*(x559)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x552)*(x555)))+(((IkReal(0.350000000000000))*(x554)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x565=((cj4)*(cj5));
IkReal x566=((r00)*(sj0));
IkReal x567=((IkReal(1.00000000000000))*(cj1));
IkReal x568=((cj0)*(r00));
IkReal x569=((r01)*(sj0));
IkReal x570=((cj2)*(r02));
IkReal x571=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x572=((IkReal(1.00000000000000))*(cj2)*(sj1));
IkReal x573=((r02)*(sj1)*(sj2));
IkReal x574=((cj1)*(sj2)*(sj5));
IkReal x575=((cj2)*(sj1)*(sj5));
if( IKabs(((gconst54)*(((((x565)*(x566)))+(((IkReal(-1.00000000000000))*(sj5)*(x573)))+(((cj1)*(sj5)*(x570)))+(((IkReal(-1.00000000000000))*(x565)*(x571)))+(((x569)*(x574)))+(((x569)*(x575)))+(((x568)*(x574)))+(((x568)*(x575))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x565)*(x568)*(x572)))+(((IkReal(-1.00000000000000))*(sj5)*(x571)))+(((x565)*(x573)))+(((IkReal(-1.00000000000000))*(x565)*(x567)*(x570)))+(((sj5)*(x566)))+(((IkReal(-1.00000000000000))*(sj2)*(x565)*(x567)*(x568)))+(((IkReal(-1.00000000000000))*(sj2)*(x565)*(x567)*(x569)))+(((IkReal(-1.00000000000000))*(x565)*(x569)*(x572))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((x565)*(x566)))+(((IkReal(-1.00000000000000))*(sj5)*(x573)))+(((cj1)*(sj5)*(x570)))+(((IkReal(-1.00000000000000))*(x565)*(x571)))+(((x569)*(x574)))+(((x569)*(x575)))+(((x568)*(x574)))+(((x568)*(x575)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(x565)*(x568)*(x572)))+(((IkReal(-1.00000000000000))*(sj5)*(x571)))+(((x565)*(x573)))+(((IkReal(-1.00000000000000))*(x565)*(x567)*(x570)))+(((sj5)*(x566)))+(((IkReal(-1.00000000000000))*(sj2)*(x565)*(x567)*(x568)))+(((IkReal(-1.00000000000000))*(sj2)*(x565)*(x567)*(x569)))+(((IkReal(-1.00000000000000))*(x565)*(x569)*(x572)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x576=IKsin(j3);
IkReal x577=IKcos(j3);
IkReal x578=((sj1)*(sj2));
IkReal x579=((cj2)*(sj1));
IkReal x580=((IkReal(0.158000000000000))*(sj4));
IkReal x581=((IkReal(1.00000000000000))*(cj1));
IkReal x582=((cj0)*(r00));
IkReal x583=((IkReal(1.00000000000000))*(sj0));
IkReal x584=((IkReal(1.00000000000000))*(cj0));
IkReal x585=((sj0)*(sj2));
IkReal x586=((sj5)*(x576));
IkReal x587=((sj5)*(x577));
IkReal x588=((cj4)*(cj5)*(x577));
IkReal x589=((cj4)*(cj5)*(x576));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x587)))+(((IkReal(-1.00000000000000))*(x589)))+(((IkReal(-1.00000000000000))*(r01)*(x584))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x584)))+(((IkReal(-1.00000000000000))*(x576)*(x580)))+(((px)*(sj0)))+(((IkReal(2.16000000000000))*(x587)))+(((IkReal(2.16000000000000))*(x589))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x588)))+(((IkReal(-1.00000000000000))*(sj2)*(x581)*(x582)))+(((r02)*(x578)))+(((IkReal(-1.00000000000000))*(r01)*(x579)*(x583)))+(((IkReal(-1.00000000000000))*(x579)*(x582)))+(x586)+(((IkReal(-1.00000000000000))*(r01)*(x581)*(x585)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x581))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x586)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x581)))+(((pz)*(x578)))+(((IkReal(-1.00000000000000))*(py)*(x581)*(x585)))+(((IkReal(-1.00000000000000))*(x577)*(x580)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x579)*(x584)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x581)))+(((IkReal(0.350000000000000))*(x579)))+(((IkReal(-1.00000000000000))*(py)*(x579)*(x583)))+(((IkReal(2.16000000000000))*(x588)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x590=((r01)*(sj0));
IkReal x591=((sj1)*(sj2));
IkReal x592=((cj0)*(r00));
IkReal x593=((IkReal(1.00000000000000))*(cj1)*(cj2));
cj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x592)*(x593)))+(((IkReal(-1.00000000000000))*(x590)*(x593)))+(((cj2)*(r02)*(sj1)))+(((x590)*(x591)))+(((x591)*(x592)))+(((cj1)*(r02)*(sj2))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x594=((cj0)*(px));
IkReal x595=((cj1)*(cj2));
IkReal x596=((sj1)*(sj2));
IkReal x597=((py)*(sj0));
evalcond[0]=((IkReal(1.54500000000000))+(((x594)*(x596)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x594)*(x595)))+(((IkReal(-1.00000000000000))*(x595)*(x597)))+(((IkReal(0.158000000000000))*(cj4)))+(((cj2)*(pz)*(sj1)))+(((cj1)*(pz)*(sj2)))+(((IkReal(2.16000000000000))*(sj4)*(IKcos(j5))))+(((x596)*(x597)))+(((IkReal(0.350000000000000))*(x595)))+(((IkReal(-0.350000000000000))*(x596))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x598=((cj2)*(sj1));
IkReal x599=((sj1)*(sj2));
IkReal x600=((cj1)*(sj2));
IkReal x601=((sj0)*(x599));
IkReal x602=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x603=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x603)))+(((IkReal(-1.00000000000000))*(r00)*(x602)))+(((cj0)*(r00)*(x599)))+(((r02)*(x598)))+(((r02)*(x600)))+(((r01)*(x601))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x598)))+(((cj0)*(px)*(x599)))+(((py)*(x601)))+(((IkReal(-1.00000000000000))*(py)*(x603)))+(((pz)*(x600)))+(((IkReal(-0.350000000000000))*(x599)))+(((IkReal(-1.00000000000000))*(px)*(x602))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x604=((IkReal(1.00000000000000))*(cj5));
IkReal x605=((cj0)*(r01));
IkReal x606=((r00)*(sj0));
IkReal x607=((IkReal(1.00000000000000))*(sj5));
IkReal x608=((cj0)*(r00));
IkReal x609=((cj1)*(sj2));
IkReal x610=((cj2)*(sj1));
IkReal x611=((r01)*(sj0));
IkReal x612=((cj1)*(cj2)*(r02));
IkReal x613=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((cj5)*(x606)))+(((IkReal(-1.00000000000000))*(x604)*(x605)))+(((sj5)*(x608)*(x610)))+(((sj5)*(x608)*(x609)))+(((sj5)*(x609)*(x611)))+(((IkReal(-1.00000000000000))*(x607)*(x613)))+(((sj5)*(x612)))+(((sj5)*(x610)*(x611))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x604)*(x610)*(x611)))+(((IkReal(-1.00000000000000))*(x605)*(x607)))+(((IkReal(-1.00000000000000))*(x604)*(x609)*(x611)))+(((IkReal(-1.00000000000000))*(x604)*(x608)*(x610)))+(((IkReal(-1.00000000000000))*(x604)*(x612)))+(((IkReal(-1.00000000000000))*(x604)*(x608)*(x609)))+(((sj5)*(x606)))+(((cj5)*(x613))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((cj5)*(x606)))+(((IkReal(-1.00000000000000))*(x604)*(x605)))+(((sj5)*(x608)*(x610)))+(((sj5)*(x608)*(x609)))+(((sj5)*(x609)*(x611)))+(((IkReal(-1.00000000000000))*(x607)*(x613)))+(((sj5)*(x612)))+(((sj5)*(x610)*(x611)))))), ((gconst55)*(((((IkReal(-1.00000000000000))*(x604)*(x610)*(x611)))+(((IkReal(-1.00000000000000))*(x605)*(x607)))+(((IkReal(-1.00000000000000))*(x604)*(x609)*(x611)))+(((IkReal(-1.00000000000000))*(x604)*(x608)*(x610)))+(((IkReal(-1.00000000000000))*(x604)*(x612)))+(((IkReal(-1.00000000000000))*(x604)*(x608)*(x609)))+(((sj5)*(x606)))+(((cj5)*(x613)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x614=IKcos(j3);
IkReal x615=IKsin(j3);
IkReal x616=((sj1)*(sj2));
IkReal x617=((cj2)*(sj1));
IkReal x618=((IkReal(2.16000000000000))*(cj5));
IkReal x619=((IkReal(1.00000000000000))*(cj1));
IkReal x620=((cj0)*(r00));
IkReal x621=((IkReal(1.00000000000000))*(sj0));
IkReal x622=((IkReal(1.00000000000000))*(cj0));
IkReal x623=((IkReal(1.00000000000000))*(cj5));
IkReal x624=((sj0)*(sj2));
IkReal x625=((sj5)*(x615));
IkReal x626=((sj5)*(x614));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x622)))+(((IkReal(-1.00000000000000))*(x626)))+(((IkReal(-1.00000000000000))*(x615)*(x623))));
evalcond[1]=((((IkReal(2.16000000000000))*(x626)))+(((x615)*(x618)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x622))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x617)*(x621)))+(((IkReal(-1.00000000000000))*(sj2)*(x619)*(x620)))+(((IkReal(-1.00000000000000))*(r01)*(x619)*(x624)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x619)))+(((r02)*(x616)))+(((IkReal(-1.00000000000000))*(x614)*(x623)))+(((IkReal(-1.00000000000000))*(x617)*(x620)))+(x625));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x619)))+(((IkReal(-1.00000000000000))*(py)*(x619)*(x624)))+(((IkReal(-1.00000000000000))*(py)*(x617)*(x621)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((pz)*(x616)))+(((x614)*(x618)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x619)))+(((IkReal(-2.16000000000000))*(x625)))+(((IkReal(0.350000000000000))*(x617)))+(((IkReal(-1.00000000000000))*(px)*(x617)*(x622)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x627=((cj2)*(sj1));
IkReal x628=((sj1)*(sj2));
IkReal x629=((cj1)*(sj2));
IkReal x630=((sj0)*(x628));
IkReal x631=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x632=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r02)*(x627)))+(((r02)*(x629)))+(((IkReal(-1.00000000000000))*(r01)*(x632)))+(((IkReal(-1.00000000000000))*(r00)*(x631)))+(((r01)*(x630)))+(((cj0)*(r00)*(x628))));
evalcond[2]=((IkReal(1.38700000000000))+(((py)*(x630)))+(((pz)*(x627)))+(((pz)*(x629)))+(((cj0)*(px)*(x628)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-0.350000000000000))*(x628)))+(((IkReal(-1.00000000000000))*(px)*(x631)))+(((IkReal(-1.00000000000000))*(py)*(x632))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x633=((IkReal(1.00000000000000))*(cj1));
IkReal x634=((cj2)*(cj5));
IkReal x635=((cj0)*(sj5));
IkReal x636=((cj5)*(r00));
IkReal x637=((sj2)*(sj5));
IkReal x638=((r01)*(sj0));
IkReal x639=((r02)*(sj1));
IkReal x640=((IkReal(1.00000000000000))*(sj1));
IkReal x641=((cj2)*(sj5));
IkReal x642=((cj5)*(sj2));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(x633)*(x637)*(x638)))+(((IkReal(-1.00000000000000))*(r02)*(x633)*(x641)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj0)*(x636)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x635)*(x640)))+(((IkReal(-1.00000000000000))*(x638)*(x640)*(x641)))+(((x637)*(x639)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x633)*(x635))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((x639)*(x642)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(cj0)*(sj2)*(x633)*(x636)))+(((IkReal(-1.00000000000000))*(r02)*(x633)*(x634)))+(((r01)*(x635)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x634)*(x640)))+(((IkReal(-1.00000000000000))*(x633)*(x638)*(x642)))+(((IkReal(-1.00000000000000))*(x634)*(x638)*(x640))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(x633)*(x637)*(x638)))+(((IkReal(-1.00000000000000))*(r02)*(x633)*(x641)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj0)*(x636)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x635)*(x640)))+(((IkReal(-1.00000000000000))*(x638)*(x640)*(x641)))+(((x637)*(x639)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x633)*(x635)))))), ((gconst56)*(((((x639)*(x642)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(cj0)*(sj2)*(x633)*(x636)))+(((IkReal(-1.00000000000000))*(r02)*(x633)*(x634)))+(((r01)*(x635)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x634)*(x640)))+(((IkReal(-1.00000000000000))*(x633)*(x638)*(x642)))+(((IkReal(-1.00000000000000))*(x634)*(x638)*(x640)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x643=IKcos(j3);
IkReal x644=IKsin(j3);
IkReal x645=((sj1)*(sj2));
IkReal x646=((cj2)*(sj1));
IkReal x647=((IkReal(2.16000000000000))*(cj5));
IkReal x648=((IkReal(1.00000000000000))*(cj1));
IkReal x649=((cj0)*(r00));
IkReal x650=((IkReal(1.00000000000000))*(sj0));
IkReal x651=((IkReal(1.00000000000000))*(cj0));
IkReal x652=((sj0)*(sj2));
IkReal x653=((sj5)*(x644));
IkReal x654=((sj5)*(x643));
evalcond[0]=((((r00)*(sj0)))+(((cj5)*(x644)))+(((IkReal(-1.00000000000000))*(x654)))+(((IkReal(-1.00000000000000))*(r01)*(x651))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x644)*(x647)))+(((IkReal(2.16000000000000))*(x654)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x651))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x646)*(x650)))+(((IkReal(-1.00000000000000))*(r01)*(x648)*(x652)))+(((cj5)*(x643)))+(((IkReal(-1.00000000000000))*(sj2)*(x648)*(x649)))+(((r02)*(x645)))+(x653)+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x648)))+(((IkReal(-1.00000000000000))*(x646)*(x649))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x653)))+(((IkReal(-1.00000000000000))*(py)*(x648)*(x652)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x648)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x648)))+(((pz)*(x645)))+(((IkReal(-1.00000000000000))*(px)*(x646)*(x651)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x646)*(x650)))+(((IkReal(0.350000000000000))*(x646)))+(((IkReal(-1.00000000000000))*(x643)*(x647)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x655=((cj2)*(sj1));
IkReal x656=((sj1)*(sj2));
IkReal x657=((cj1)*(sj2));
IkReal x658=((sj0)*(x656));
IkReal x659=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x660=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((cj0)*(r00)*(x656)))+(((IkReal(-1.00000000000000))*(r01)*(x660)))+(((IkReal(-1.00000000000000))*(r00)*(x659)))+(((r02)*(x655)))+(((r02)*(x657)))+(((r01)*(x658))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(px)*(x659)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(-0.350000000000000))*(x656)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((pz)*(x657)))+(((pz)*(x655)))+(((py)*(x658)))+(((IkReal(2.16000000000000))*(sj4)))+(((cj0)*(px)*(x656)))+(((IkReal(-1.00000000000000))*(py)*(x660))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x661=((cj1)*(sj2));
IkReal x662=((sj1)*(sj2));
IkReal x663=((cj0)*(r00));
IkReal x664=((r01)*(sj0));
IkReal x665=((IkReal(1.00000000000000))*(cj2));
IkReal x666=((cj2)*(sj1));
IkReal x667=((cj0)*(px));
IkReal x668=((cj1)*(sj0)*(x665));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x664)*(x665)))+(((x662)*(x663)))+(((x662)*(x664)))+(((r02)*(x661)))+(((r02)*(x666)))+(((IkReal(-1.00000000000000))*(cj1)*(x663)*(x665))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj1)*(x664)*(x665)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x665)))+(((r02)*(x662)))+(((IkReal(-1.00000000000000))*(sj1)*(x663)*(x665)))+(((IkReal(-1.00000000000000))*(x661)*(x664)))+(((IkReal(-1.00000000000000))*(x661)*(x663))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x665)*(x667)))+(((IkReal(1.20000000000000))*(sj2)))+(((x662)*(x667)))+(((IkReal(-1.00000000000000))*(py)*(x668)))+(((pz)*(x666)))+(((pz)*(x661)))+(((IkReal(-0.350000000000000))*(x662)))+(((py)*(sj0)*(x662))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x669=((cj2)*(sj1));
IkReal x670=((IkReal(6.32911392405063))*(sj2));
IkReal x671=((cj0)*(px));
IkReal x672=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(x670)*(x672)))+(((IkReal(-1.00000000000000))*(cj1)*(x670)*(x671)))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-6.32911392405063))*(x669)*(x671)))+(((IkReal(-6.32911392405063))*(x669)*(x672)))+(((pz)*(sj1)*(x670)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(2.21518987341772))*(x669)))+(((IkReal(2.21518987341772))*(cj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))))+IKsqr(((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(x670)*(x672)))+(((IkReal(-1.00000000000000))*(cj1)*(x670)*(x671)))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-6.32911392405063))*(x669)*(x671)))+(((IkReal(-6.32911392405063))*(x669)*(x672)))+(((pz)*(sj1)*(x670)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(2.21518987341772))*(x669)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))), ((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(x670)*(x672)))+(((IkReal(-1.00000000000000))*(cj1)*(x670)*(x671)))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-6.32911392405063))*(x669)*(x671)))+(((IkReal(-6.32911392405063))*(x669)*(x672)))+(((pz)*(sj1)*(x670)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(2.21518987341772))*(x669)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x673=((cj2)*(sj1));
IkReal x674=((cj1)*(sj2));
IkReal x675=((IkReal(1.00000000000000))*(py));
IkReal x676=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x675)))+(((IkReal(-0.158000000000000))*(IKsin(j3))))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x674)))+(((IkReal(0.350000000000000))*(x673)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(sj0)*(x673)*(x675)))+(((IkReal(-1.00000000000000))*(sj0)*(x674)*(x675)))+(((IkReal(-1.00000000000000))*(x674)*(x676)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x673)*(x676)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x677=((cj1)*(sj2));
IkReal x678=((sj1)*(sj2));
IkReal x679=((cj0)*(r00));
IkReal x680=((r01)*(sj0));
IkReal x681=((IkReal(1.00000000000000))*(cj2));
IkReal x682=((cj2)*(sj1));
IkReal x683=((cj0)*(px));
IkReal x684=((cj1)*(sj0)*(x681));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(1.00000000000000))+(((r02)*(x677)))+(((IkReal(-1.00000000000000))*(cj1)*(x680)*(x681)))+(((x678)*(x680)))+(((x678)*(x679)))+(((IkReal(-1.00000000000000))*(cj1)*(x679)*(x681)))+(((r02)*(x682))));
evalcond[3]=((((r02)*(x678)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x681)))+(((IkReal(-1.00000000000000))*(x677)*(x679)))+(((IkReal(-1.00000000000000))*(sj1)*(x679)*(x681)))+(((IkReal(-1.00000000000000))*(x677)*(x680)))+(((IkReal(-1.00000000000000))*(sj1)*(x680)*(x681))));
evalcond[4]=((IkReal(-0.615000000000000))+(((x678)*(x683)))+(((pz)*(x677)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((py)*(sj0)*(x678)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x681)*(x683)))+(((IkReal(-0.350000000000000))*(x678)))+(((IkReal(-1.00000000000000))*(py)*(x684)))+(((pz)*(x682))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x685=((cj2)*(sj1));
IkReal x686=((IkReal(6.32911392405063))*(cj1));
IkReal x687=((cj0)*(px));
IkReal x688=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((IkReal(6.32911392405063))*(x685)*(x687)))+(((IkReal(6.32911392405063))*(x685)*(x688)))+(((IkReal(-2.21518987341772))*(x685)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((cj2)*(pz)*(x686)))+(((IkReal(7.59493670886076))*(cj2)))+(((sj2)*(x686)*(x688)))+(((sj2)*(x686)*(x687))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))))+IKsqr(((IkReal(0.917721518987342))+(((IkReal(6.32911392405063))*(x685)*(x687)))+(((IkReal(6.32911392405063))*(x685)*(x688)))+(((IkReal(-2.21518987341772))*(x685)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((cj2)*(pz)*(x686)))+(((IkReal(7.59493670886076))*(cj2)))+(((sj2)*(x686)*(x688)))+(((sj2)*(x686)*(x687)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))), ((IkReal(0.917721518987342))+(((IkReal(6.32911392405063))*(x685)*(x687)))+(((IkReal(6.32911392405063))*(x685)*(x688)))+(((IkReal(-2.21518987341772))*(x685)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((cj2)*(pz)*(x686)))+(((IkReal(7.59493670886076))*(cj2)))+(((sj2)*(x686)*(x688)))+(((sj2)*(x686)*(x687)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x689=((cj2)*(sj1));
IkReal x690=((cj1)*(sj2));
IkReal x691=((IkReal(1.00000000000000))*(py));
IkReal x692=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(0.158000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x691))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(sj0)*(x689)*(x691)))+(((IkReal(0.350000000000000))*(x689)))+(((pz)*(sj1)*(sj2)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(0.350000000000000))*(x690)))+(((IkReal(-1.00000000000000))*(x690)*(x692)))+(((IkReal(-1.00000000000000))*(x689)*(x692)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(sj0)*(x690)*(x691)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x693=((IkReal(1.00000000000000))*(r01));
IkReal x694=((cj2)*(sj1));
IkReal x695=((IkReal(1.00000000000000))*(cj1));
IkReal x696=((cj0)*(r00));
if( IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x693))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((IkReal(-1.00000000000000))*(cj1)*(sj0)*(sj2)*(x693)))+(((IkReal(-1.00000000000000))*(x694)*(x696)))+(((IkReal(-1.00000000000000))*(sj0)*(x693)*(x694)))+(((IkReal(-1.00000000000000))*(sj2)*(x695)*(x696)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x695))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x693)))))), ((gconst57)*(((((IkReal(-1.00000000000000))*(cj1)*(sj0)*(sj2)*(x693)))+(((IkReal(-1.00000000000000))*(x694)*(x696)))+(((IkReal(-1.00000000000000))*(sj0)*(x693)*(x694)))+(((IkReal(-1.00000000000000))*(sj2)*(x695)*(x696)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x695)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x697=IKsin(j3);
IkReal x698=IKcos(j3);
IkReal x699=((sj1)*(sj2));
IkReal x700=((cj2)*(sj1));
IkReal x701=((IkReal(0.158000000000000))*(sj4));
IkReal x702=((IkReal(1.00000000000000))*(cj1));
IkReal x703=((cj0)*(r00));
IkReal x704=((IkReal(1.00000000000000))*(sj0));
IkReal x705=((IkReal(1.00000000000000))*(cj0));
IkReal x706=((sj0)*(sj2));
IkReal x707=((cj4)*(x697));
IkReal x708=((cj4)*(x698));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x707)))+(((IkReal(-1.00000000000000))*(r01)*(x705))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x705)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(x697)*(x701)))+(((IkReal(2.16000000000000))*(x707))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x700)*(x703)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x702)))+(((IkReal(-1.00000000000000))*(x708)))+(((r02)*(x699)))+(((IkReal(-1.00000000000000))*(r01)*(x700)*(x704)))+(((IkReal(-1.00000000000000))*(sj2)*(x702)*(x703)))+(((IkReal(-1.00000000000000))*(r01)*(x702)*(x706))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(py)*(x700)*(x704)))+(((IkReal(-1.00000000000000))*(px)*(x700)*(x705)))+(((pz)*(x699)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x702)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x702)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x698)*(x701)))+(((IkReal(-1.00000000000000))*(py)*(x702)*(x706)))+(((IkReal(0.350000000000000))*(x700)))+(((IkReal(2.16000000000000))*(x708)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x709=((cj2)*(sj1));
IkReal x710=((sj1)*(sj2));
IkReal x711=((cj1)*(sj2));
IkReal x712=((sj0)*(x710));
IkReal x713=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x714=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((r01)*(x712)))+(((cj0)*(r00)*(x710)))+(((IkReal(-1.00000000000000))*(r01)*(x714)))+(((r02)*(x709)))+(((r02)*(x711)))+(((IkReal(-1.00000000000000))*(r00)*(x713))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-2.16000000000000))*(sj4)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-0.350000000000000))*(x710)))+(((cj0)*(px)*(x710)))+(((IkReal(-1.00000000000000))*(px)*(x713)))+(((py)*(x712)))+(((pz)*(x711)))+(((pz)*(x709)))+(((IkReal(-1.00000000000000))*(py)*(x714))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x715=((cj1)*(sj2));
IkReal x716=((sj1)*(sj2));
IkReal x717=((cj0)*(r00));
IkReal x718=((r01)*(sj0));
IkReal x719=((IkReal(1.00000000000000))*(cj2));
IkReal x720=((cj2)*(sj1));
IkReal x721=((cj0)*(px));
IkReal x722=((cj1)*(sj0)*(x719));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x717)*(x719)))+(((IkReal(-1.00000000000000))*(cj1)*(x718)*(x719)))+(((r02)*(x720)))+(((x716)*(x717)))+(((x716)*(x718)))+(((r02)*(x715))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj1)*(x718)*(x719)))+(((IkReal(-1.00000000000000))*(sj1)*(x717)*(x719)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x719)))+(((IkReal(-1.00000000000000))*(x715)*(x717)))+(((IkReal(-1.00000000000000))*(x715)*(x718)))+(((r02)*(x716))));
evalcond[4]=((IkReal(-0.615000000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((py)*(sj0)*(x716)))+(((IkReal(-0.350000000000000))*(x716)))+(((IkReal(-1.00000000000000))*(cj1)*(x719)*(x721)))+(((pz)*(x720)))+(((IkReal(-1.00000000000000))*(py)*(x722)))+(((pz)*(x715)))+(((x716)*(x721))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x723=((cj2)*(sj1));
IkReal x724=((IkReal(6.32911392405063))*(sj2));
IkReal x725=((cj0)*(px));
IkReal x726=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-6.32911392405063))*(x723)*(x725)))+(((IkReal(-6.32911392405063))*(x723)*(x726)))+(((IkReal(-1.00000000000000))*(cj1)*(x724)*(x726)))+(((IkReal(-1.00000000000000))*(cj1)*(x724)*(x725)))+(((IkReal(-7.59493670886076))*(cj2)))+(((pz)*(sj1)*(x724)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))+(((IkReal(2.21518987341772))*(x723))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))))+IKsqr(((IkReal(-0.917721518987342))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-6.32911392405063))*(x723)*(x725)))+(((IkReal(-6.32911392405063))*(x723)*(x726)))+(((IkReal(-1.00000000000000))*(cj1)*(x724)*(x726)))+(((IkReal(-1.00000000000000))*(cj1)*(x724)*(x725)))+(((IkReal(-7.59493670886076))*(cj2)))+(((pz)*(sj1)*(x724)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))+(((IkReal(2.21518987341772))*(x723)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(cj0)*(py)))+(((IkReal(6.32911392405063))*(px)*(sj0)))), ((IkReal(-0.917721518987342))+(((IkReal(-6.32911392405063))*(cj1)*(cj2)*(pz)))+(((IkReal(-6.32911392405063))*(x723)*(x725)))+(((IkReal(-6.32911392405063))*(x723)*(x726)))+(((IkReal(-1.00000000000000))*(cj1)*(x724)*(x726)))+(((IkReal(-1.00000000000000))*(cj1)*(x724)*(x725)))+(((IkReal(-7.59493670886076))*(cj2)))+(((pz)*(sj1)*(x724)))+(((IkReal(2.21518987341772))*(cj1)*(sj2)))+(((IkReal(2.21518987341772))*(x723)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x727=((cj2)*(sj1));
IkReal x728=((cj1)*(sj2));
IkReal x729=((IkReal(1.00000000000000))*(py));
IkReal x730=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x729)))+(((IkReal(-0.158000000000000))*(IKsin(j3))))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x727)*(x730)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(sj0)*(x727)*(x729)))+(((IkReal(0.350000000000000))*(x728)))+(((IkReal(0.350000000000000))*(x727)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(sj0)*(x728)*(x729)))+(((IkReal(-1.00000000000000))*(x728)*(x730)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x731=((cj1)*(sj2));
IkReal x732=((sj1)*(sj2));
IkReal x733=((cj0)*(r00));
IkReal x734=((r01)*(sj0));
IkReal x735=((IkReal(1.00000000000000))*(cj2));
IkReal x736=((cj2)*(sj1));
IkReal x737=((cj0)*(px));
IkReal x738=((cj1)*(sj0)*(x735));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x734)*(x735)))+(((IkReal(-1.00000000000000))*(cj1)*(x733)*(x735)))+(((r02)*(x736)))+(((r02)*(x731)))+(((x732)*(x734)))+(((x732)*(x733))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj1)*(x733)*(x735)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x735)))+(((IkReal(-1.00000000000000))*(x731)*(x734)))+(((IkReal(-1.00000000000000))*(x731)*(x733)))+(((IkReal(-1.00000000000000))*(sj1)*(x734)*(x735)))+(((r02)*(x732))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x735)*(x737)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-0.350000000000000))*(x732)))+(((py)*(sj0)*(x732)))+(((pz)*(x736)))+(((pz)*(x731)))+(((IkReal(-1.00000000000000))*(py)*(x738)))+(((x732)*(x737))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x739=((cj2)*(sj1));
IkReal x740=((IkReal(6.32911392405063))*(cj1));
IkReal x741=((cj0)*(px));
IkReal x742=((py)*(sj0));
if( IKabs(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((sj2)*(x740)*(x742)))+(((sj2)*(x740)*(x741)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(6.32911392405063))*(x739)*(x742)))+(((IkReal(6.32911392405063))*(x739)*(x741)))+(((IkReal(-2.21518987341772))*(x739)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((cj2)*(pz)*(x740)))+(((IkReal(7.59493670886076))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))))+IKsqr(((IkReal(0.917721518987342))+(((sj2)*(x740)*(x742)))+(((sj2)*(x740)*(x741)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(6.32911392405063))*(x739)*(x742)))+(((IkReal(6.32911392405063))*(x739)*(x741)))+(((IkReal(-2.21518987341772))*(x739)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((cj2)*(pz)*(x740)))+(((IkReal(7.59493670886076))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-6.32911392405063))*(px)*(sj0)))+(((IkReal(6.32911392405063))*(cj0)*(py)))), ((IkReal(0.917721518987342))+(((sj2)*(x740)*(x742)))+(((sj2)*(x740)*(x741)))+(((IkReal(-6.32911392405063))*(pz)*(sj1)*(sj2)))+(((IkReal(6.32911392405063))*(x739)*(x742)))+(((IkReal(6.32911392405063))*(x739)*(x741)))+(((IkReal(-2.21518987341772))*(x739)))+(((IkReal(-2.21518987341772))*(cj1)*(sj2)))+(((cj2)*(pz)*(x740)))+(((IkReal(7.59493670886076))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x743=((cj2)*(sj1));
IkReal x744=((cj1)*(sj2));
IkReal x745=((IkReal(1.00000000000000))*(py));
IkReal x746=((IkReal(1.00000000000000))*(cj0)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x745)))+(((IkReal(0.158000000000000))*(IKsin(j3))))+(((px)*(sj0))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x743)))+(((IkReal(0.350000000000000))*(x744)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj0)*(x744)*(x745)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(x744)*(x746)))+(((IkReal(-1.00000000000000))*(sj0)*(x743)*(x745)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(x743)*(x746)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x747=((cj1)*(sj2));
IkReal x748=((r01)*(sj0));
IkReal x749=((cj0)*(r00));
IkReal x750=((cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x747)*(x748)))+(((x747)*(x749)))+(((cj1)*(cj2)*(r02)))+(((x749)*(x750)))+(((x748)*(x750))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x747)*(x748)))+(((x747)*(x749)))+(((cj1)*(cj2)*(r02)))+(((x749)*(x750)))+(((x748)*(x750)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x751=IKsin(j3);
IkReal x752=IKcos(j3);
IkReal x753=((sj1)*(sj2));
IkReal x754=((cj2)*(sj1));
IkReal x755=((IkReal(0.158000000000000))*(sj4));
IkReal x756=((IkReal(1.00000000000000))*(sj0));
IkReal x757=((IkReal(1.00000000000000))*(cj1));
IkReal x758=((cj0)*(r00));
IkReal x759=((IkReal(1.00000000000000))*(cj0));
IkReal x760=((cj1)*(sj2));
IkReal x761=((cj4)*(x751));
IkReal x762=((cj4)*(x752));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x759)))+(x761));
evalcond[1]=((((IkReal(-2.16000000000000))*(x761)))+(((IkReal(-1.00000000000000))*(x751)*(x755)))+(((IkReal(-1.00000000000000))*(py)*(x759)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x757)))+(((r02)*(x753)))+(((IkReal(-1.00000000000000))*(x754)*(x758)))+(x762)+(((IkReal(-1.00000000000000))*(r01)*(x754)*(x756)))+(((IkReal(-1.00000000000000))*(r01)*(x756)*(x760)))+(((IkReal(-1.00000000000000))*(sj2)*(x757)*(x758))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x762)))+(((IkReal(0.350000000000000))*(x754)))+(((pz)*(x753)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x757)))+(((IkReal(-1.00000000000000))*(px)*(x754)*(x759)))+(((IkReal(-1.00000000000000))*(py)*(x754)*(x756)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x757)))+(((IkReal(-1.00000000000000))*(x752)*(x755)))+(((IkReal(0.350000000000000))*(x760)))+(((IkReal(-1.00000000000000))*(py)*(x756)*(x760)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x763=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x764=((cj4)*(cj5));
IkReal x765=((cj0)*(r01));
IkReal x766=((IkReal(3.95000000000000))*(sj4));
IkReal x767=((IkReal(0.253164556962025))*(x763));
IkReal x768=((r00)*(sj0));
IkReal x769=((IkReal(25.0000000000000))*(cj0)*(py));
IkReal x770=((IkReal(25.0000000000000))*(px)*(sj0));
if( IKabs(((x767)*(((((IkReal(-1.00000000000000))*(x769)))+(((IkReal(-54.0000000000000))*(x765)))+(x770)+(((IkReal(54.0000000000000))*(x768))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x767)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-54.0000000000000))*(x764)*(x768)))+(((IkReal(54.0000000000000))*(x764)*(x765)))+(((IkReal(-1.00000000000000))*(x764)*(x770)))+(((x766)*(x768)))+(((x764)*(x769))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x767)*(((((IkReal(-1.00000000000000))*(x769)))+(((IkReal(-54.0000000000000))*(x765)))+(x770)+(((IkReal(54.0000000000000))*(x768)))))))+IKsqr(((x767)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-54.0000000000000))*(x764)*(x768)))+(((IkReal(54.0000000000000))*(x764)*(x765)))+(((IkReal(-1.00000000000000))*(x764)*(x770)))+(((x766)*(x768)))+(((x764)*(x769)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x767)*(((((IkReal(-1.00000000000000))*(x769)))+(((IkReal(-54.0000000000000))*(x765)))+(x770)+(((IkReal(54.0000000000000))*(x768)))))), ((x767)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-54.0000000000000))*(x764)*(x768)))+(((IkReal(54.0000000000000))*(x764)*(x765)))+(((IkReal(-1.00000000000000))*(x764)*(x770)))+(((x766)*(x768)))+(((x764)*(x769)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x771=IKsin(j3);
IkReal x772=IKcos(j3);
IkReal x773=((sj1)*(sj2));
IkReal x774=((cj2)*(sj1));
IkReal x775=((IkReal(0.158000000000000))*(sj4));
IkReal x776=((IkReal(1.00000000000000))*(cj1));
IkReal x777=((cj0)*(r00));
IkReal x778=((IkReal(1.00000000000000))*(sj0));
IkReal x779=((IkReal(1.00000000000000))*(cj0));
IkReal x780=((sj0)*(sj2));
IkReal x781=((sj5)*(x771));
IkReal x782=((sj5)*(x772));
IkReal x783=((cj4)*(cj5)*(x772));
IkReal x784=((cj4)*(cj5)*(x771));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x779)))+(((IkReal(-1.00000000000000))*(x784)))+(((IkReal(-1.00000000000000))*(x782))));
evalcond[1]=((((IkReal(2.16000000000000))*(x782)))+(((IkReal(2.16000000000000))*(x784)))+(((IkReal(-1.00000000000000))*(py)*(x779)))+(((IkReal(-1.00000000000000))*(x771)*(x775)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x776)))+(((IkReal(-1.00000000000000))*(x774)*(x777)))+(((IkReal(-1.00000000000000))*(r01)*(x774)*(x778)))+(((IkReal(-1.00000000000000))*(r01)*(x776)*(x780)))+(((IkReal(-1.00000000000000))*(sj2)*(x776)*(x777)))+(x781)+(((IkReal(-1.00000000000000))*(x783)))+(((r02)*(x773))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x781)))+(((IkReal(0.350000000000000))*(x774)))+(((pz)*(x773)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x776)))+(((IkReal(-1.00000000000000))*(px)*(x774)*(x779)))+(((IkReal(-1.00000000000000))*(py)*(x776)*(x780)))+(((IkReal(2.16000000000000))*(x783)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x772)*(x775)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x776)))+(((IkReal(-1.00000000000000))*(py)*(x774)*(x778)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x785=((cj4)*(cj5));
IkReal x786=((r00)*(sj0));
IkReal x787=((IkReal(1.00000000000000))*(cj1));
IkReal x788=((cj0)*(r00));
IkReal x789=((r01)*(sj0));
IkReal x790=((cj2)*(r02));
IkReal x791=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x792=((IkReal(1.00000000000000))*(cj2)*(sj1));
IkReal x793=((r02)*(sj1)*(sj2));
IkReal x794=((cj1)*(sj2)*(sj5));
IkReal x795=((cj2)*(sj1)*(sj5));
if( IKabs(((gconst54)*(((((x785)*(x786)))+(((x788)*(x795)))+(((x788)*(x794)))+(((IkReal(-1.00000000000000))*(sj5)*(x793)))+(((IkReal(-1.00000000000000))*(x785)*(x791)))+(((cj1)*(sj5)*(x790)))+(((x789)*(x794)))+(((x789)*(x795))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(sj2)*(x785)*(x787)*(x788)))+(((IkReal(-1.00000000000000))*(sj2)*(x785)*(x787)*(x789)))+(((IkReal(-1.00000000000000))*(sj5)*(x791)))+(((sj5)*(x786)))+(((x785)*(x793)))+(((IkReal(-1.00000000000000))*(x785)*(x789)*(x792)))+(((IkReal(-1.00000000000000))*(x785)*(x787)*(x790)))+(((IkReal(-1.00000000000000))*(x785)*(x788)*(x792))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((x785)*(x786)))+(((x788)*(x795)))+(((x788)*(x794)))+(((IkReal(-1.00000000000000))*(sj5)*(x793)))+(((IkReal(-1.00000000000000))*(x785)*(x791)))+(((cj1)*(sj5)*(x790)))+(((x789)*(x794)))+(((x789)*(x795)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(sj2)*(x785)*(x787)*(x788)))+(((IkReal(-1.00000000000000))*(sj2)*(x785)*(x787)*(x789)))+(((IkReal(-1.00000000000000))*(sj5)*(x791)))+(((sj5)*(x786)))+(((x785)*(x793)))+(((IkReal(-1.00000000000000))*(x785)*(x789)*(x792)))+(((IkReal(-1.00000000000000))*(x785)*(x787)*(x790)))+(((IkReal(-1.00000000000000))*(x785)*(x788)*(x792)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x796=IKsin(j3);
IkReal x797=IKcos(j3);
IkReal x798=((sj1)*(sj2));
IkReal x799=((cj2)*(sj1));
IkReal x800=((IkReal(0.158000000000000))*(sj4));
IkReal x801=((IkReal(1.00000000000000))*(cj1));
IkReal x802=((cj0)*(r00));
IkReal x803=((IkReal(1.00000000000000))*(sj0));
IkReal x804=((IkReal(1.00000000000000))*(cj0));
IkReal x805=((sj0)*(sj2));
IkReal x806=((sj5)*(x796));
IkReal x807=((sj5)*(x797));
IkReal x808=((cj4)*(cj5)*(x797));
IkReal x809=((cj4)*(cj5)*(x796));
evalcond[0]=((((IkReal(-1.00000000000000))*(x807)))+(((IkReal(-1.00000000000000))*(x809)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x804))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x796)*(x800)))+(((IkReal(2.16000000000000))*(x807)))+(((IkReal(2.16000000000000))*(x809)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x804))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x808)))+(((IkReal(-1.00000000000000))*(r01)*(x801)*(x805)))+(((IkReal(-1.00000000000000))*(sj2)*(x801)*(x802)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x801)))+(((IkReal(-1.00000000000000))*(x799)*(x802)))+(((r02)*(x798)))+(((IkReal(-1.00000000000000))*(r01)*(x799)*(x803)))+(x806));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x797)*(x800)))+(((IkReal(-1.00000000000000))*(py)*(x801)*(x805)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x801)))+(((IkReal(2.16000000000000))*(x808)))+(((pz)*(x798)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x799)*(x803)))+(((IkReal(-2.16000000000000))*(x806)))+(((IkReal(0.350000000000000))*(x799)))+(((IkReal(-1.00000000000000))*(px)*(x799)*(x804)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x801)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x810=((cj2)*(sj1));
IkReal x811=((IkReal(2.16000000000000))*(r01));
IkReal x812=((cj1)*(cj2));
IkReal x813=((sj1)*(sj2));
IkReal x814=((cj1)*(sj2));
IkReal x815=((py)*(sj0));
IkReal x816=((IkReal(2.16000000000000))*(r00));
IkReal x817=((IkReal(2.16000000000000))*(r02));
IkReal x818=((cj0)*(px));
if( IKabs(((IkReal(500.000000000000))*(gconst53)*(((((sj0)*(x816)))+(((IkReal(-1.00000000000000))*(cj0)*(x811)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-500.000000000000))*(gconst53)*(((IkReal(0.145000000000000))+(((sj0)*(x811)*(x814)))+(((pz)*(x812)))+(((x812)*(x817)))+(((x810)*(x818)))+(((x810)*(x815)))+(((cj0)*(x810)*(x816)))+(((x814)*(x815)))+(((x814)*(x818)))+(((IkReal(-0.350000000000000))*(x814)))+(((IkReal(-0.350000000000000))*(x810)))+(((sj0)*(x810)*(x811)))+(((IkReal(1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x813)*(x817)))+(((cj0)*(x814)*(x816)))+(((IkReal(-1.00000000000000))*(pz)*(x813))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(500.000000000000))*(gconst53)*(((((sj0)*(x816)))+(((IkReal(-1.00000000000000))*(cj0)*(x811)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py)))))), ((IkReal(-500.000000000000))*(gconst53)*(((IkReal(0.145000000000000))+(((sj0)*(x811)*(x814)))+(((pz)*(x812)))+(((x812)*(x817)))+(((x810)*(x818)))+(((x810)*(x815)))+(((cj0)*(x810)*(x816)))+(((x814)*(x815)))+(((x814)*(x818)))+(((IkReal(-0.350000000000000))*(x814)))+(((IkReal(-0.350000000000000))*(x810)))+(((sj0)*(x810)*(x811)))+(((IkReal(1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x813)*(x817)))+(((cj0)*(x814)*(x816)))+(((IkReal(-1.00000000000000))*(pz)*(x813)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x819=IKsin(j3);
IkReal x820=IKcos(j3);
IkReal x821=((IkReal(2.16000000000000))*(r01));
IkReal x822=((cj2)*(sj1));
IkReal x823=((cj4)*(sj2));
IkReal x824=((pz)*(sj1));
IkReal x825=((px)*(sj0));
IkReal x826=((cj0)*(px));
IkReal x827=((py)*(sj0));
IkReal x828=((IkReal(1.00000000000000))*(cj4));
IkReal x829=((IkReal(0.350000000000000))*(cj1));
IkReal x830=((cj0)*(cj4));
IkReal x831=((cj1)*(sj2));
IkReal x832=((IkReal(1.20000000000000))*(cj2));
IkReal x833=((IkReal(1.00000000000000))*(cj1));
IkReal x834=((cj0)*(py));
IkReal x835=((IkReal(2.16000000000000))*(r00)*(sj0));
IkReal x836=((IkReal(2.16000000000000))*(r02)*(sj1));
IkReal x837=((IkReal(2.16000000000000))*(cj0)*(r00));
IkReal x838=((cj1)*(cj2)*(pz));
IkReal x839=((IkReal(0.158000000000000))*(sj4)*(x819));
IkReal x840=((IkReal(2.16000000000000))*(cj1)*(cj2)*(r02));
IkReal x841=((IkReal(0.158000000000000))*(sj4)*(x820));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x821)))+(((IkReal(-1.00000000000000))*(x839)))+(((IkReal(-1.00000000000000))*(x834)))+(x825)+(x835));
evalcond[1]=((((IkReal(-1.00000000000000))*(x821)*(x830)))+(((IkReal(-1.00000000000000))*(x828)*(x834)))+(((IkReal(-1.00000000000000))*(cj4)*(x839)))+(((cj4)*(x825)))+(((cj4)*(x835))));
evalcond[2]=((IkReal(0.145000000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x824)))+(((x831)*(x837)))+(((sj0)*(x821)*(x831)))+(((x827)*(x831)))+(((x826)*(x831)))+(((IkReal(-0.350000000000000))*(x822)))+(((x822)*(x837)))+(((x822)*(x827)))+(((x822)*(x826)))+(((IkReal(-1.00000000000000))*(sj2)*(x829)))+(x832)+(x838)+(x841)+(x840)+(((sj0)*(x821)*(x822)))+(((IkReal(-1.00000000000000))*(sj2)*(x836))));
evalcond[3]=((((IkReal(-0.145000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj1)*(x823)*(x837)))+(((IkReal(-1.00000000000000))*(x828)*(x838)))+(((IkReal(-1.00000000000000))*(cj4)*(x841)))+(((IkReal(-1.00000000000000))*(cj4)*(x840)))+(((IkReal(-1.00000000000000))*(cj4)*(x832)))+(((IkReal(-1.00000000000000))*(cj4)*(sj0)*(x821)*(x822)))+(((IkReal(-2.16000000000000))*(r00)*(x822)*(x830)))+(((x823)*(x836)))+(((IkReal(-1.00000000000000))*(cj1)*(sj0)*(x821)*(x823)))+(((x823)*(x829)))+(((x823)*(x824)))+(((IkReal(-1.00000000000000))*(x822)*(x827)*(x828)))+(((IkReal(-1.00000000000000))*(x823)*(x827)*(x833)))+(((IkReal(0.350000000000000))*(cj4)*(x822)))+(((IkReal(-1.00000000000000))*(x822)*(x826)*(x828)))+(((IkReal(-1.00000000000000))*(x823)*(x826)*(x833))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x842=(sj3)*(sj3);
IkReal x843=(cj3)*(cj3);
dummyeval[0]=((x843)+(x842));
dummyeval[1]=((((cj4)*(x843)))+(((cj4)*(x842))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x844=(sj3)*(sj3);
IkReal x845=(cj3)*(cj3);
dummyeval[0]=((((IkReal(-1.00000000000000))*(x845)))+(((IkReal(-1.00000000000000))*(x844))));
dummyeval[1]=((((cj4)*(x844)))+(((cj4)*(x845))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x846=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x846))));
evalcond[2]=((IkReal(-0.158000000000000))+(((IkReal(-1.00000000000000))*(py)*(x846)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x847=((sj1)*(sj2));
IkReal x848=((r01)*(sj0));
IkReal x849=((cj1)*(sj2));
IkReal x850=((cj0)*(r00));
IkReal x851=((cj2)*(sj1));
IkReal x852=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((x848)*(x851)))+(((x850)*(x851)))+(((IkReal(-1.00000000000000))*(r02)*(x847)))+(((x848)*(x849)))+(((cj1)*(cj2)*(r02)))+(((x849)*(x850))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r02)*(x849)))+(((IkReal(-1.00000000000000))*(x850)*(x852)))+(((x847)*(x850)))+(((IkReal(-1.00000000000000))*(x848)*(x852)))+(((r02)*(x851)))+(((x847)*(x848))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x848)*(x851)))+(((x850)*(x851)))+(((IkReal(-1.00000000000000))*(r02)*(x847)))+(((x848)*(x849)))+(((cj1)*(cj2)*(r02)))+(((x849)*(x850)))))+IKsqr(((((r02)*(x849)))+(((IkReal(-1.00000000000000))*(x850)*(x852)))+(((x847)*(x850)))+(((IkReal(-1.00000000000000))*(x848)*(x852)))+(((r02)*(x851)))+(((x847)*(x848)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x848)*(x851)))+(((x850)*(x851)))+(((IkReal(-1.00000000000000))*(r02)*(x847)))+(((x848)*(x849)))+(((cj1)*(cj2)*(r02)))+(((x849)*(x850)))), ((((r02)*(x849)))+(((IkReal(-1.00000000000000))*(x850)*(x852)))+(((x847)*(x850)))+(((IkReal(-1.00000000000000))*(x848)*(x852)))+(((r02)*(x851)))+(((x847)*(x848)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x853=IKcos(j5);
IkReal x854=IKsin(j5);
IkReal x855=((pz)*(sj2));
IkReal x856=((cj2)*(sj1));
IkReal x857=((r02)*(sj2));
IkReal x858=((sj1)*(sj2));
IkReal x859=((cj0)*(r00));
IkReal x860=((r01)*(sj0));
IkReal x861=((IkReal(1.00000000000000))*(cj1));
IkReal x862=((cj0)*(px));
IkReal x863=((py)*(sj0));
IkReal x864=((IkReal(0.350000000000000))*(cj1));
IkReal x865=((cj2)*(sj0)*(x861));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(x860)*(x861)))+(((IkReal(-1.00000000000000))*(x853)))+(((x858)*(x859)))+(((cj1)*(x857)))+(((x858)*(x860)))+(((r02)*(x856)))+(((IkReal(-1.00000000000000))*(cj2)*(x859)*(x861))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x861)))+(((sj1)*(x857)))+(((IkReal(-1.00000000000000))*(sj2)*(x859)*(x861)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-1.00000000000000))*(x856)*(x860)))+(x854)+(((IkReal(-1.00000000000000))*(sj2)*(x860)*(x861))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(2.16000000000000))*(x853)))+(((cj2)*(x864)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x861)*(x863)))+(((IkReal(-1.00000000000000))*(cj2)*(x861)*(x862)))+(((cj1)*(x855)))+(((x858)*(x863)))+(((x858)*(x862)))+(((IkReal(-0.350000000000000))*(x858)))+(((pz)*(x856))));
evalcond[3]=((IkReal(-0.145000000000000))+(((sj2)*(x864)))+(((sj1)*(x855)))+(((IkReal(-2.16000000000000))*(x854)))+(((IkReal(-1.00000000000000))*(sj2)*(x861)*(x862)))+(((IkReal(-1.00000000000000))*(sj2)*(x861)*(x863)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x861)))+(((IkReal(-1.00000000000000))*(x856)*(x862)))+(((IkReal(-1.00000000000000))*(x856)*(x863)))+(((IkReal(0.350000000000000))*(x856)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x866=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x866))));
evalcond[2]=((IkReal(0.158000000000000))+(((IkReal(-1.00000000000000))*(py)*(x866)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x867=((r01)*(sj0));
IkReal x868=((cj1)*(sj2));
IkReal x869=((cj2)*(sj1));
IkReal x870=((cj0)*(r00));
IkReal x871=((IkReal(1.00000000000000))*(r02));
IkReal x872=((cj1)*(cj2));
IkReal x873=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x874=((cj1)*(x870));
if( IKabs(((((x867)*(x868)))+(((x867)*(x869)))+(((x868)*(x870)))+(((r02)*(x872)))+(((x869)*(x870)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x871))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x867)*(x872)))+(((x870)*(x872)))+(((IkReal(-1.00000000000000))*(x868)*(x871)))+(((IkReal(-1.00000000000000))*(x869)*(x871)))+(((IkReal(-1.00000000000000))*(x870)*(x873)))+(((IkReal(-1.00000000000000))*(x867)*(x873))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x867)*(x868)))+(((x867)*(x869)))+(((x868)*(x870)))+(((r02)*(x872)))+(((x869)*(x870)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x871)))))+IKsqr(((((x867)*(x872)))+(((x870)*(x872)))+(((IkReal(-1.00000000000000))*(x868)*(x871)))+(((IkReal(-1.00000000000000))*(x869)*(x871)))+(((IkReal(-1.00000000000000))*(x870)*(x873)))+(((IkReal(-1.00000000000000))*(x867)*(x873)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x867)*(x868)))+(((x867)*(x869)))+(((x868)*(x870)))+(((r02)*(x872)))+(((x869)*(x870)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x871)))), ((((x867)*(x872)))+(((x870)*(x872)))+(((IkReal(-1.00000000000000))*(x868)*(x871)))+(((IkReal(-1.00000000000000))*(x869)*(x871)))+(((IkReal(-1.00000000000000))*(x870)*(x873)))+(((IkReal(-1.00000000000000))*(x867)*(x873)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x875=IKcos(j5);
IkReal x876=IKsin(j5);
IkReal x877=((pz)*(sj2));
IkReal x878=((cj2)*(sj1));
IkReal x879=((r02)*(sj2));
IkReal x880=((sj1)*(sj2));
IkReal x881=((cj0)*(r00));
IkReal x882=((r01)*(sj0));
IkReal x883=((IkReal(1.00000000000000))*(cj1));
IkReal x884=((cj0)*(px));
IkReal x885=((py)*(sj0));
IkReal x886=((IkReal(0.350000000000000))*(cj1));
IkReal x887=((cj2)*(sj0)*(x883));
evalcond[0]=((((x880)*(x882)))+(((x880)*(x881)))+(((IkReal(-1.00000000000000))*(cj2)*(x882)*(x883)))+(((r02)*(x878)))+(((cj1)*(x879)))+(x875)+(((IkReal(-1.00000000000000))*(cj2)*(x881)*(x883))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x878)*(x882)))+(((IkReal(-1.00000000000000))*(x878)*(x881)))+(((sj1)*(x879)))+(((IkReal(-1.00000000000000))*(sj2)*(x882)*(x883)))+(x876)+(((IkReal(-1.00000000000000))*(sj2)*(x881)*(x883)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x883))));
evalcond[2]=((IkReal(1.54500000000000))+(((cj2)*(x886)))+(((x880)*(x885)))+(((x880)*(x884)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-2.16000000000000))*(x875)))+(((IkReal(-1.00000000000000))*(cj2)*(x883)*(x884)))+(((IkReal(-1.00000000000000))*(cj2)*(x883)*(x885)))+(((IkReal(-0.350000000000000))*(x880)))+(((cj1)*(x877)))+(((pz)*(x878))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x878)*(x885)))+(((IkReal(-1.00000000000000))*(x878)*(x884)))+(((IkReal(-2.16000000000000))*(x876)))+(((sj2)*(x886)))+(((sj1)*(x877)))+(((IkReal(0.350000000000000))*(x878)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x883)))+(((IkReal(-1.00000000000000))*(sj2)*(x883)*(x885)))+(((IkReal(-1.00000000000000))*(sj2)*(x883)*(x884)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x888=((IkReal(0.462962962962963))*(cj2));
IkReal x889=((cj0)*(px));
IkReal x890=((cj1)*(sj2));
IkReal x891=((py)*(sj0));
if( IKabs(((IkReal(-0.0671296296296296))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x888)))+(((IkReal(-0.462962962962963))*(x889)*(x890)))+(((IkReal(0.462962962962963))*(pz)*(sj1)*(sj2)))+(((IkReal(0.162037037037037))*(x890)))+(((IkReal(-0.462962962962963))*(x890)*(x891)))+(((IkReal(-0.555555555555556))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x888)*(x889)))+(((IkReal(0.162037037037037))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(sj1)*(x888)*(x891))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.0671296296296296))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x888)))+(((IkReal(-0.462962962962963))*(x889)*(x890)))+(((IkReal(0.462962962962963))*(pz)*(sj1)*(sj2)))+(((IkReal(0.162037037037037))*(x890)))+(((IkReal(-0.462962962962963))*(x890)*(x891)))+(((IkReal(-0.555555555555556))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x888)*(x889)))+(((IkReal(0.162037037037037))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(sj1)*(x888)*(x891)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-0.0671296296296296))+(((IkReal(-1.00000000000000))*(cj1)*(pz)*(x888)))+(((IkReal(-0.462962962962963))*(x889)*(x890)))+(((IkReal(0.462962962962963))*(pz)*(sj1)*(sj2)))+(((IkReal(0.162037037037037))*(x890)))+(((IkReal(-0.462962962962963))*(x890)*(x891)))+(((IkReal(-0.555555555555556))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x888)*(x889)))+(((IkReal(0.162037037037037))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(sj1)*(x888)*(x891)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x892=IKcos(j5);
IkReal x893=IKsin(j5);
IkReal x894=((cj0)*(px));
IkReal x895=((pz)*(sj2));
IkReal x896=((cj2)*(sj1));
IkReal x897=((r01)*(sj0));
IkReal x898=((cj1)*(sj2));
IkReal x899=((sj1)*(sj2));
IkReal x900=((cj0)*(r00));
IkReal x901=((py)*(sj0));
IkReal x902=((IkReal(1.00000000000000))*(cj0));
IkReal x903=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x904=((cj4)*(x892));
IkReal x905=((sj4)*(x892));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x902)))+(((IkReal(-1.00000000000000))*(x904))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x902)))+(((px)*(sj0)))+(((IkReal(2.16000000000000))*(x904))));
evalcond[2]=((((r02)*(x896)))+(((r02)*(x898)))+(((IkReal(-1.00000000000000))*(x900)*(x903)))+(((IkReal(-1.00000000000000))*(x897)*(x903)))+(((x897)*(x899)))+(((x899)*(x900)))+(((IkReal(-1.00000000000000))*(x905))));
evalcond[3]=((((r02)*(x899)))+(((IkReal(-1.00000000000000))*(x896)*(x897)))+(((IkReal(-1.00000000000000))*(x898)*(x900)))+(((IkReal(-1.00000000000000))*(r02)*(x903)))+(((IkReal(-1.00000000000000))*(x897)*(x898)))+(x893)+(((IkReal(-1.00000000000000))*(x896)*(x900))));
evalcond[4]=((IkReal(1.54500000000000))+(((x894)*(x899)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((pz)*(x896)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x894)*(x903)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-0.350000000000000))*(x899)))+(((x899)*(x901)))+(((IkReal(-1.00000000000000))*(x901)*(x903)))+(((cj1)*(x895)))+(((IkReal(2.16000000000000))*(x905))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x896)))+(((IkReal(0.350000000000000))*(x898)))+(((sj1)*(x895)))+(((IkReal(-2.16000000000000))*(x893)))+(((IkReal(-1.00000000000000))*(pz)*(x903)))+(((IkReal(-1.00000000000000))*(x898)*(x901)))+(((IkReal(-1.00000000000000))*(x894)*(x896)))+(((IkReal(-1.00000000000000))*(x894)*(x898)))+(((IkReal(-1.00000000000000))*(x896)*(x901)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x906=((cj1)*(sj2));
IkReal x907=((r01)*(sj0));
IkReal x908=((cj0)*(r00));
IkReal x909=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x907)*(x909)))+(((x906)*(x908)))+(((x906)*(x907)))+(((x908)*(x909)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj4))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x907)*(x909)))+(((x906)*(x908)))+(((x906)*(x907)))+(((x908)*(x909)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj4)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x907)*(x909)))+(((x906)*(x908)))+(((x906)*(x907)))+(((x908)*(x909)))+(((cj1)*(cj2)*(r02)))), ((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj4)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x910=IKcos(j5);
IkReal x911=IKsin(j5);
IkReal x912=((cj0)*(px));
IkReal x913=((pz)*(sj2));
IkReal x914=((cj2)*(sj1));
IkReal x915=((r01)*(sj0));
IkReal x916=((cj1)*(sj2));
IkReal x917=((sj1)*(sj2));
IkReal x918=((cj0)*(r00));
IkReal x919=((py)*(sj0));
IkReal x920=((IkReal(1.00000000000000))*(cj0));
IkReal x921=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x922=((cj4)*(x910));
IkReal x923=((sj4)*(x910));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x922)))+(((IkReal(-1.00000000000000))*(r01)*(x920))));
evalcond[1]=((((IkReal(2.16000000000000))*(x922)))+(((IkReal(-0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x920)))+(((px)*(sj0))));
evalcond[2]=((((x917)*(x918)))+(((r02)*(x916)))+(((r02)*(x914)))+(((IkReal(-1.00000000000000))*(x923)))+(((IkReal(-1.00000000000000))*(x918)*(x921)))+(((x915)*(x917)))+(((IkReal(-1.00000000000000))*(x915)*(x921))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x921)))+(((IkReal(-1.00000000000000))*(x915)*(x916)))+(((r02)*(x917)))+(((IkReal(-1.00000000000000))*(x916)*(x918)))+(x911)+(((IkReal(-1.00000000000000))*(x914)*(x915)))+(((IkReal(-1.00000000000000))*(x914)*(x918))));
evalcond[4]=((IkReal(1.54500000000000))+(((pz)*(x914)))+(((x917)*(x919)))+(((IkReal(2.16000000000000))*(x923)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(x912)*(x921)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-0.350000000000000))*(x917)))+(((IkReal(-1.00000000000000))*(x919)*(x921)))+(((x912)*(x917)))+(((cj1)*(x913))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x912)*(x916)))+(((IkReal(-1.00000000000000))*(x912)*(x914)))+(((IkReal(-1.00000000000000))*(x916)*(x919)))+(((IkReal(-2.16000000000000))*(x911)))+(((IkReal(0.350000000000000))*(x916)))+(((IkReal(0.350000000000000))*(x914)))+(((IkReal(-1.00000000000000))*(pz)*(x921)))+(((sj1)*(x913)))+(((IkReal(-1.00000000000000))*(x914)*(x919)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x924=((cj1)*(sj2));
IkReal x925=((r01)*(sj0));
IkReal x926=((cj0)*(r00));
IkReal x927=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x924)*(x926)))+(((x924)*(x925)))+(((x925)*(x927)))+(((x926)*(x927)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x924)*(x926)))+(((x924)*(x925)))+(((x925)*(x927)))+(((x926)*(x927)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(sj2)))+(((x924)*(x926)))+(((x924)*(x925)))+(((x925)*(x927)))+(((x926)*(x927)))+(((cj1)*(cj2)*(r02)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x928=IKcos(j5);
IkReal x929=IKsin(j5);
IkReal x930=((cj0)*(px));
IkReal x931=((pz)*(sj2));
IkReal x932=((cj2)*(sj1));
IkReal x933=((r01)*(sj0));
IkReal x934=((cj1)*(sj2));
IkReal x935=((sj1)*(sj2));
IkReal x936=((cj0)*(r00));
IkReal x937=((py)*(sj0));
IkReal x938=((IkReal(1.00000000000000))*(cj0));
IkReal x939=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x940=((cj4)*(x928));
IkReal x941=((sj4)*(x928));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x940)))+(((IkReal(-1.00000000000000))*(r01)*(x938))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj4)))+(((IkReal(2.16000000000000))*(x940)))+(((IkReal(-1.00000000000000))*(py)*(x938)))+(((px)*(sj0))));
evalcond[2]=((((x935)*(x936)))+(((IkReal(-1.00000000000000))*(x941)))+(((IkReal(-1.00000000000000))*(x936)*(x939)))+(((IkReal(-1.00000000000000))*(x933)*(x939)))+(((x933)*(x935)))+(((r02)*(x934)))+(((r02)*(x932))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x934)*(x936)))+(((IkReal(-1.00000000000000))*(r02)*(x939)))+(((IkReal(-1.00000000000000))*(x932)*(x933)))+(((IkReal(-1.00000000000000))*(x932)*(x936)))+(((IkReal(-1.00000000000000))*(x933)*(x934)))+(x929)+(((r02)*(x935))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x930)*(x939)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(2.16000000000000))*(x941)))+(((x935)*(x937)))+(((x930)*(x935)))+(((IkReal(-0.350000000000000))*(x935)))+(((pz)*(x932)))+(((cj1)*(x931)))+(((IkReal(-1.00000000000000))*(x937)*(x939))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x934)))+(((IkReal(0.350000000000000))*(x932)))+(((IkReal(-1.00000000000000))*(x934)*(x937)))+(((sj1)*(x931)))+(((IkReal(-2.16000000000000))*(x929)))+(((IkReal(-1.00000000000000))*(x930)*(x934)))+(((IkReal(-1.00000000000000))*(x930)*(x932)))+(((IkReal(-1.00000000000000))*(pz)*(x939)))+(((IkReal(-1.00000000000000))*(x932)*(x937)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x942=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x942)))+(((r00)*(sj0))));
evalcond[2]=((IkReal(0.158000000000000))+(((IkReal(-1.00000000000000))*(py)*(x942)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x943=((r01)*(sj0));
IkReal x944=((sj1)*(sj2));
IkReal x945=((IkReal(1.00000000000000))*(cj1));
IkReal x946=((cj0)*(r00));
IkReal x947=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x943)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x945)))+(((r02)*(x944)))+(((IkReal(-1.00000000000000))*(sj2)*(x945)*(x946)))+(((IkReal(-1.00000000000000))*(x946)*(x947)))+(((IkReal(-1.00000000000000))*(sj2)*(x943)*(x945))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r02)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(x943)*(x945)))+(((x943)*(x944)))+(((IkReal(-1.00000000000000))*(cj2)*(x945)*(x946)))+(((x944)*(x946)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x943)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x945)))+(((r02)*(x944)))+(((IkReal(-1.00000000000000))*(sj2)*(x945)*(x946)))+(((IkReal(-1.00000000000000))*(x946)*(x947)))+(((IkReal(-1.00000000000000))*(sj2)*(x943)*(x945)))))+IKsqr(((((r02)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(x943)*(x945)))+(((x943)*(x944)))+(((IkReal(-1.00000000000000))*(cj2)*(x945)*(x946)))+(((x944)*(x946)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x943)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x945)))+(((r02)*(x944)))+(((IkReal(-1.00000000000000))*(sj2)*(x945)*(x946)))+(((IkReal(-1.00000000000000))*(x946)*(x947)))+(((IkReal(-1.00000000000000))*(sj2)*(x943)*(x945)))), ((((r02)*(x947)))+(((IkReal(-1.00000000000000))*(cj2)*(x943)*(x945)))+(((x943)*(x944)))+(((IkReal(-1.00000000000000))*(cj2)*(x945)*(x946)))+(((x944)*(x946)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x948=IKcos(j5);
IkReal x949=IKsin(j5);
IkReal x950=((pz)*(sj2));
IkReal x951=((cj2)*(sj1));
IkReal x952=((cj1)*(sj2));
IkReal x953=((sj1)*(sj2));
IkReal x954=((cj0)*(r00));
IkReal x955=((r01)*(sj0));
IkReal x956=((cj0)*(px));
IkReal x957=((py)*(sj0));
IkReal x958=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x959=((sj0)*(x958));
evalcond[0]=((((IkReal(-1.00000000000000))*(x948)))+(((r02)*(x952)))+(((r02)*(x951)))+(((IkReal(-1.00000000000000))*(x954)*(x958)))+(((IkReal(-1.00000000000000))*(x955)*(x958)))+(((x953)*(x954)))+(((x953)*(x955))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x949)))+(((IkReal(-1.00000000000000))*(x951)*(x955)))+(((IkReal(-1.00000000000000))*(x951)*(x954)))+(((IkReal(-1.00000000000000))*(x952)*(x955)))+(((IkReal(-1.00000000000000))*(x952)*(x954)))+(((r02)*(x953)))+(((IkReal(-1.00000000000000))*(r02)*(x958))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((cj1)*(x950)))+(((IkReal(2.16000000000000))*(x948)))+(((IkReal(-1.00000000000000))*(x956)*(x958)))+(((IkReal(-1.00000000000000))*(x957)*(x958)))+(((pz)*(x951)))+(((IkReal(-0.350000000000000))*(x953)))+(((x953)*(x956)))+(((x953)*(x957))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x952)))+(((IkReal(0.350000000000000))*(x951)))+(((IkReal(-1.00000000000000))*(pz)*(x958)))+(((IkReal(2.16000000000000))*(x949)))+(((IkReal(-1.00000000000000))*(x951)*(x957)))+(((IkReal(-1.00000000000000))*(x951)*(x956)))+(((IkReal(-1.00000000000000))*(x952)*(x957)))+(((IkReal(-1.00000000000000))*(x952)*(x956)))+(((sj1)*(x950)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x960=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x960))));
evalcond[2]=((IkReal(-0.158000000000000))+(((IkReal(-1.00000000000000))*(py)*(x960)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x961=((sj1)*(sj2));
IkReal x962=((IkReal(1.00000000000000))*(sj2));
IkReal x963=((cj1)*(r02));
IkReal x964=((cj2)*(sj1));
IkReal x965=((cj0)*(cj1)*(r00));
IkReal x966=((IkReal(1.00000000000000))*(cj0)*(r00));
IkReal x967=((IkReal(1.00000000000000))*(r01)*(sj0));
IkReal x968=((cj1)*(r01)*(sj0));
if( IKabs(((((IkReal(-1.00000000000000))*(x962)*(x968)))+(((IkReal(-1.00000000000000))*(x962)*(x965)))+(((IkReal(-1.00000000000000))*(cj2)*(x963)))+(((IkReal(-1.00000000000000))*(x964)*(x967)))+(((IkReal(-1.00000000000000))*(x964)*(x966)))+(((r02)*(x961))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x962)*(x963)))+(((IkReal(-1.00000000000000))*(x961)*(x967)))+(((IkReal(-1.00000000000000))*(x961)*(x966)))+(((cj2)*(x965)))+(((cj2)*(x968)))+(((IkReal(-1.00000000000000))*(r02)*(x964))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x962)*(x968)))+(((IkReal(-1.00000000000000))*(x962)*(x965)))+(((IkReal(-1.00000000000000))*(cj2)*(x963)))+(((IkReal(-1.00000000000000))*(x964)*(x967)))+(((IkReal(-1.00000000000000))*(x964)*(x966)))+(((r02)*(x961)))))+IKsqr(((((IkReal(-1.00000000000000))*(x962)*(x963)))+(((IkReal(-1.00000000000000))*(x961)*(x967)))+(((IkReal(-1.00000000000000))*(x961)*(x966)))+(((cj2)*(x965)))+(((cj2)*(x968)))+(((IkReal(-1.00000000000000))*(r02)*(x964)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x962)*(x968)))+(((IkReal(-1.00000000000000))*(x962)*(x965)))+(((IkReal(-1.00000000000000))*(cj2)*(x963)))+(((IkReal(-1.00000000000000))*(x964)*(x967)))+(((IkReal(-1.00000000000000))*(x964)*(x966)))+(((r02)*(x961)))), ((((IkReal(-1.00000000000000))*(x962)*(x963)))+(((IkReal(-1.00000000000000))*(x961)*(x967)))+(((IkReal(-1.00000000000000))*(x961)*(x966)))+(((cj2)*(x965)))+(((cj2)*(x968)))+(((IkReal(-1.00000000000000))*(r02)*(x964)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x969=IKcos(j5);
IkReal x970=IKsin(j5);
IkReal x971=((pz)*(sj2));
IkReal x972=((cj2)*(sj1));
IkReal x973=((cj1)*(sj2));
IkReal x974=((sj1)*(sj2));
IkReal x975=((cj0)*(r00));
IkReal x976=((r01)*(sj0));
IkReal x977=((cj0)*(px));
IkReal x978=((py)*(sj0));
IkReal x979=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x980=((sj0)*(x979));
evalcond[0]=((((IkReal(-1.00000000000000))*(x976)*(x979)))+(((x974)*(x975)))+(((x974)*(x976)))+(((r02)*(x972)))+(((r02)*(x973)))+(x969)+(((IkReal(-1.00000000000000))*(x975)*(x979))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x970)))+(((IkReal(-1.00000000000000))*(x972)*(x975)))+(((IkReal(-1.00000000000000))*(x972)*(x976)))+(((IkReal(-1.00000000000000))*(r02)*(x979)))+(((IkReal(-1.00000000000000))*(x973)*(x976)))+(((IkReal(-1.00000000000000))*(x973)*(x975)))+(((r02)*(x974))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(-2.16000000000000))*(x969)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((cj1)*(x971)))+(((IkReal(-1.00000000000000))*(x978)*(x979)))+(((pz)*(x972)))+(((x974)*(x977)))+(((x974)*(x978)))+(((IkReal(-1.00000000000000))*(x977)*(x979)))+(((IkReal(-0.350000000000000))*(x974))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x979)))+(((IkReal(0.350000000000000))*(x972)))+(((IkReal(0.350000000000000))*(x973)))+(((sj1)*(x971)))+(((IkReal(-1.00000000000000))*(x972)*(x978)))+(((IkReal(-1.00000000000000))*(x972)*(x977)))+(((IkReal(2.16000000000000))*(x970)))+(((IkReal(-1.00000000000000))*(x973)*(x978)))+(((IkReal(-1.00000000000000))*(x973)*(x977)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x981=((IkReal(0.462962962962963))*(sj1));
IkReal x982=((cj0)*(px));
IkReal x983=((cj1)*(sj2));
IkReal x984=((py)*(sj0));
if( IKabs(((IkReal(0.0671296296296296))+(((IkReal(-0.162037037037037))*(cj2)*(sj1)))+(((IkReal(-0.162037037037037))*(x983)))+(((cj2)*(x981)*(x982)))+(((cj2)*(x981)*(x984)))+(((IkReal(0.462962962962963))*(x983)*(x984)))+(((IkReal(0.555555555555556))*(cj2)))+(((IkReal(0.462962962962963))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x981)))+(((IkReal(0.462962962962963))*(x982)*(x983))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0671296296296296))+(((IkReal(-0.162037037037037))*(cj2)*(sj1)))+(((IkReal(-0.162037037037037))*(x983)))+(((cj2)*(x981)*(x982)))+(((cj2)*(x981)*(x984)))+(((IkReal(0.462962962962963))*(x983)*(x984)))+(((IkReal(0.555555555555556))*(cj2)))+(((IkReal(0.462962962962963))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x981)))+(((IkReal(0.462962962962963))*(x982)*(x983)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0671296296296296))+(((IkReal(-0.162037037037037))*(cj2)*(sj1)))+(((IkReal(-0.162037037037037))*(x983)))+(((cj2)*(x981)*(x982)))+(((cj2)*(x981)*(x984)))+(((IkReal(0.462962962962963))*(x983)*(x984)))+(((IkReal(0.555555555555556))*(cj2)))+(((IkReal(0.462962962962963))*(cj1)*(cj2)*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x981)))+(((IkReal(0.462962962962963))*(x982)*(x983)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x985=IKcos(j5);
IkReal x986=IKsin(j5);
IkReal x987=((cj0)*(px));
IkReal x988=((pz)*(sj2));
IkReal x989=((cj2)*(sj1));
IkReal x990=((r01)*(sj0));
IkReal x991=((cj1)*(sj2));
IkReal x992=((sj1)*(sj2));
IkReal x993=((cj0)*(r00));
IkReal x994=((py)*(sj0));
IkReal x995=((IkReal(1.00000000000000))*(cj0));
IkReal x996=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x997=((cj4)*(x985));
IkReal x998=((sj4)*(x985));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x995)))+(x997));
evalcond[1]=((((IkReal(0.158000000000000))*(sj4)))+(((IkReal(-2.16000000000000))*(x997)))+(((IkReal(-1.00000000000000))*(py)*(x995)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x998)))+(((x992)*(x993)))+(((x990)*(x992)))+(((IkReal(-1.00000000000000))*(x990)*(x996)))+(((r02)*(x991)))+(((IkReal(-1.00000000000000))*(x993)*(x996)))+(((r02)*(x989))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x986)))+(((IkReal(-1.00000000000000))*(x991)*(x993)))+(((IkReal(-1.00000000000000))*(x989)*(x990)))+(((IkReal(-1.00000000000000))*(x989)*(x993)))+(((r02)*(x992)))+(((IkReal(-1.00000000000000))*(r02)*(x996)))+(((IkReal(-1.00000000000000))*(x990)*(x991))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-0.350000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(x994)*(x996)))+(((cj1)*(x988)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((x992)*(x994)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((pz)*(x989)))+(((x987)*(x992)))+(((IkReal(2.16000000000000))*(x998)))+(((IkReal(-1.00000000000000))*(x987)*(x996))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x991)*(x994)))+(((IkReal(-1.00000000000000))*(x989)*(x994)))+(((IkReal(0.350000000000000))*(x989)))+(((IkReal(-1.00000000000000))*(pz)*(x996)))+(((sj1)*(x988)))+(((IkReal(2.16000000000000))*(x986)))+(((IkReal(-1.00000000000000))*(x987)*(x989)))+(((IkReal(0.350000000000000))*(x991)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x987)*(x991))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x999=((IkReal(1.00000000000000))*(cj1));
IkReal x1000=((cj0)*(r00));
IkReal x1001=((r01)*(sj0));
IkReal x1002=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x1001)*(x999)))+(((IkReal(-1.00000000000000))*(sj2)*(x1000)*(x999)))+(((IkReal(-1.00000000000000))*(x1001)*(x1002)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x999))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj4)))+(((IkReal(25.0000000000000))*(px)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x1001)*(x999)))+(((IkReal(-1.00000000000000))*(sj2)*(x1000)*(x999)))+(((IkReal(-1.00000000000000))*(x1001)*(x1002)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x999)))))+IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj4)))+(((IkReal(25.0000000000000))*(px)*(sj0)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj2)*(x1001)*(x999)))+(((IkReal(-1.00000000000000))*(sj2)*(x1000)*(x999)))+(((IkReal(-1.00000000000000))*(x1001)*(x1002)))+(((IkReal(-1.00000000000000))*(x1000)*(x1002)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x999)))), ((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj4)))+(((IkReal(25.0000000000000))*(px)*(sj0)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1003=IKcos(j5);
IkReal x1004=IKsin(j5);
IkReal x1005=((cj0)*(px));
IkReal x1006=((pz)*(sj2));
IkReal x1007=((cj2)*(sj1));
IkReal x1008=((r01)*(sj0));
IkReal x1009=((cj1)*(sj2));
IkReal x1010=((sj1)*(sj2));
IkReal x1011=((cj0)*(r00));
IkReal x1012=((py)*(sj0));
IkReal x1013=((IkReal(1.00000000000000))*(cj0));
IkReal x1014=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1015=((cj4)*(x1003));
IkReal x1016=((sj4)*(x1003));
evalcond[0]=((((r00)*(sj0)))+(x1015)+(((IkReal(-1.00000000000000))*(r01)*(x1013))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x1013)))+(((IkReal(-2.16000000000000))*(x1015)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1011)*(x1014)))+(((r02)*(x1009)))+(((r02)*(x1007)))+(((IkReal(-1.00000000000000))*(x1008)*(x1014)))+(((x1008)*(x1010)))+(((x1010)*(x1011)))+(((IkReal(-1.00000000000000))*(x1016))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x1004)))+(((IkReal(-1.00000000000000))*(x1009)*(x1011)))+(((IkReal(-1.00000000000000))*(x1007)*(x1008)))+(((IkReal(-1.00000000000000))*(r02)*(x1014)))+(((r02)*(x1010)))+(((IkReal(-1.00000000000000))*(x1008)*(x1009)))+(((IkReal(-1.00000000000000))*(x1007)*(x1011))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x1012)*(x1014)))+(((IkReal(-0.350000000000000))*(x1010)))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1005)*(x1014)))+(((IkReal(0.158000000000000))*(cj4)))+(((x1005)*(x1010)))+(((pz)*(x1007)))+(((IkReal(2.16000000000000))*(x1016)))+(((cj1)*(x1006)))+(((x1010)*(x1012))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x1009)*(x1012)))+(((sj1)*(x1006)))+(((IkReal(-1.00000000000000))*(pz)*(x1014)))+(((IkReal(-1.00000000000000))*(x1005)*(x1007)))+(((IkReal(-1.00000000000000))*(x1005)*(x1009)))+(((IkReal(2.16000000000000))*(x1004)))+(((IkReal(0.350000000000000))*(x1009)))+(((IkReal(0.350000000000000))*(x1007)))+(((IkReal(-1.00000000000000))*(x1007)*(x1012)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1017=((IkReal(1.00000000000000))*(cj1));
IkReal x1018=((cj0)*(r00));
IkReal x1019=((r01)*(sj0));
IkReal x1020=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x1018)*(x1020)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1017)*(x1018)))+(((IkReal(-1.00000000000000))*(sj2)*(x1017)*(x1019)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1017))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x1018)*(x1020)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1017)*(x1018)))+(((IkReal(-1.00000000000000))*(sj2)*(x1017)*(x1019)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1017)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x1018)*(x1020)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)))+(((r02)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1017)*(x1018)))+(((IkReal(-1.00000000000000))*(sj2)*(x1017)*(x1019)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1017)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1021=IKcos(j5);
IkReal x1022=IKsin(j5);
IkReal x1023=((cj0)*(px));
IkReal x1024=((pz)*(sj2));
IkReal x1025=((cj2)*(sj1));
IkReal x1026=((r01)*(sj0));
IkReal x1027=((cj1)*(sj2));
IkReal x1028=((sj1)*(sj2));
IkReal x1029=((cj0)*(r00));
IkReal x1030=((py)*(sj0));
IkReal x1031=((IkReal(1.00000000000000))*(cj0));
IkReal x1032=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1033=((cj4)*(x1021));
IkReal x1034=((sj4)*(x1021));
evalcond[0]=((((r00)*(sj0)))+(x1033)+(((IkReal(-1.00000000000000))*(r01)*(x1031))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x1031)))+(((IkReal(-2.16000000000000))*(x1033)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1034)))+(((x1028)*(x1029)))+(((IkReal(-1.00000000000000))*(x1029)*(x1032)))+(((IkReal(-1.00000000000000))*(x1026)*(x1032)))+(((x1026)*(x1028)))+(((r02)*(x1025)))+(((r02)*(x1027))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x1026)*(x1027)))+(((IkReal(-1.00000000000000))*(x1022)))+(((IkReal(-1.00000000000000))*(r02)*(x1032)))+(((r02)*(x1028)))+(((IkReal(-1.00000000000000))*(x1027)*(x1029)))+(((IkReal(-1.00000000000000))*(x1025)*(x1026)))+(((IkReal(-1.00000000000000))*(x1025)*(x1029))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x1023)*(x1032)))+(((IkReal(2.16000000000000))*(x1034)))+(((x1023)*(x1028)))+(((x1028)*(x1030)))+(((IkReal(-1.00000000000000))*(x1030)*(x1032)))+(((IkReal(-0.350000000000000))*(x1028)))+(((pz)*(x1025)))+(((cj1)*(x1024))));
evalcond[5]=((IkReal(-0.145000000000000))+(((sj1)*(x1024)))+(((IkReal(-1.00000000000000))*(pz)*(x1032)))+(((IkReal(-1.00000000000000))*(x1025)*(x1030)))+(((IkReal(2.16000000000000))*(x1022)))+(((IkReal(-1.00000000000000))*(x1027)*(x1030)))+(((IkReal(-1.00000000000000))*(x1023)*(x1025)))+(((IkReal(-1.00000000000000))*(x1023)*(x1027)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(0.350000000000000))*(x1027)))+(((IkReal(0.350000000000000))*(x1025))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1035=((cj2)*(sj1));
IkReal x1036=((sj1)*(sj2));
IkReal x1037=((cj1)*(sj2));
IkReal x1038=((sj0)*(x1036));
IkReal x1039=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x1040=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x1040)))+(((r02)*(x1037)))+(((r02)*(x1035)))+(((IkReal(-1.00000000000000))*(r00)*(x1039)))+(((cj0)*(r00)*(x1036)))+(((r01)*(x1038))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((cj0)*(px)*(x1036)))+(((py)*(x1038)))+(((IkReal(-1.00000000000000))*(py)*(x1040)))+(((pz)*(x1035)))+(((pz)*(x1037)))+(((IkReal(-1.00000000000000))*(px)*(x1039)))+(((IkReal(-0.350000000000000))*(x1036))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1041=((IkReal(1.00000000000000))*(cj3));
IkReal x1042=((cj0)*(r01));
IkReal x1043=((IkReal(1.00000000000000))*(sj3));
IkReal x1044=((cj0)*(r00));
IkReal x1045=((cj1)*(sj2));
IkReal x1046=((cj2)*(sj1));
IkReal x1047=((r00)*(sj0));
IkReal x1048=((r02)*(sj1)*(sj2));
IkReal x1049=((cj1)*(cj2)*(r02));
IkReal x1050=((r01)*(sj0)*(x1045));
IkReal x1051=((r01)*(sj0)*(x1046));
if( IKabs(((gconst59)*(((((sj3)*(x1044)*(x1046)))+(((sj3)*(x1044)*(x1045)))+(((sj3)*(x1051)))+(((sj3)*(x1050)))+(((IkReal(-1.00000000000000))*(x1043)*(x1048)))+(((sj3)*(x1049)))+(((IkReal(-1.00000000000000))*(x1041)*(x1042)))+(((cj3)*(x1047))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(x1041)*(x1044)*(x1045)))+(((IkReal(-1.00000000000000))*(x1041)*(x1044)*(x1046)))+(((sj3)*(x1047)))+(((IkReal(-1.00000000000000))*(x1041)*(x1049)))+(((IkReal(-1.00000000000000))*(x1041)*(x1050)))+(((IkReal(-1.00000000000000))*(x1041)*(x1051)))+(((cj3)*(x1048)))+(((IkReal(-1.00000000000000))*(x1042)*(x1043))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst59)*(((((sj3)*(x1044)*(x1046)))+(((sj3)*(x1044)*(x1045)))+(((sj3)*(x1051)))+(((sj3)*(x1050)))+(((IkReal(-1.00000000000000))*(x1043)*(x1048)))+(((sj3)*(x1049)))+(((IkReal(-1.00000000000000))*(x1041)*(x1042)))+(((cj3)*(x1047)))))), ((gconst59)*(((((IkReal(-1.00000000000000))*(x1041)*(x1044)*(x1045)))+(((IkReal(-1.00000000000000))*(x1041)*(x1044)*(x1046)))+(((sj3)*(x1047)))+(((IkReal(-1.00000000000000))*(x1041)*(x1049)))+(((IkReal(-1.00000000000000))*(x1041)*(x1050)))+(((IkReal(-1.00000000000000))*(x1041)*(x1051)))+(((cj3)*(x1048)))+(((IkReal(-1.00000000000000))*(x1042)*(x1043)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1052=IKsin(j5);
IkReal x1053=IKcos(j5);
IkReal x1054=((sj1)*(sj2));
IkReal x1055=((cj2)*(sj1));
IkReal x1056=((IkReal(1.00000000000000))*(cj1));
IkReal x1057=((cj0)*(r00));
IkReal x1058=((IkReal(1.00000000000000))*(sj0));
IkReal x1059=((IkReal(1.00000000000000))*(cj0));
IkReal x1060=((sj0)*(sj2));
IkReal x1061=((sj3)*(x1053));
IkReal x1062=((sj3)*(x1052));
IkReal x1063=((cj3)*(x1053));
IkReal x1064=((cj3)*(x1052));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1059)))+(((IkReal(-1.00000000000000))*(x1064)))+(((IkReal(-1.00000000000000))*(x1061))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1059)))+(((IkReal(2.16000000000000))*(x1064)))+(((IkReal(2.16000000000000))*(x1061)))+(((px)*(sj0))));
evalcond[2]=((((r02)*(x1054)))+(((IkReal(-1.00000000000000))*(x1055)*(x1057)))+(x1062)+(((IkReal(-1.00000000000000))*(r01)*(x1056)*(x1060)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1056)))+(((IkReal(-1.00000000000000))*(r01)*(x1055)*(x1058)))+(((IkReal(-1.00000000000000))*(sj2)*(x1056)*(x1057)))+(((IkReal(-1.00000000000000))*(x1063))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x1056)))+(((pz)*(x1054)))+(((IkReal(0.350000000000000))*(x1055)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-2.16000000000000))*(x1062)))+(((IkReal(-1.00000000000000))*(px)*(x1055)*(x1059)))+(((IkReal(-1.00000000000000))*(py)*(x1055)*(x1058)))+(((IkReal(2.16000000000000))*(x1063)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1056)))+(((IkReal(-1.00000000000000))*(py)*(x1056)*(x1060)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1065=((cj2)*(sj1));
IkReal x1066=((sj1)*(sj2));
IkReal x1067=((cj1)*(sj2));
IkReal x1068=((sj0)*(x1066));
IkReal x1069=((IkReal(1.00000000000000))*(cj0)*(cj1)*(cj2));
IkReal x1070=((IkReal(1.00000000000000))*(cj1)*(cj2)*(sj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x1069)))+(((IkReal(-1.00000000000000))*(r01)*(x1070)))+(((cj0)*(r00)*(x1066)))+(((r01)*(x1068)))+(((r02)*(x1065)))+(((r02)*(x1067))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(x1069)))+(((IkReal(-0.350000000000000))*(x1066)))+(((IkReal(1.20000000000000))*(sj2)))+(((py)*(x1068)))+(((pz)*(x1067)))+(((pz)*(x1065)))+(((IkReal(-1.00000000000000))*(py)*(x1070)))+(((cj0)*(px)*(x1066))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1071=((cj2)*(cj3));
IkReal x1072=((cj0)*(r00));
IkReal x1073=((sj2)*(sj3));
IkReal x1074=((r01)*(sj0));
IkReal x1075=((cj1)*(r02));
IkReal x1076=((cj0)*(r01));
IkReal x1077=((IkReal(1.00000000000000))*(cj3));
IkReal x1078=((r02)*(sj1));
IkReal x1079=((cj2)*(sj3));
IkReal x1080=((r00)*(sj0));
IkReal x1081=((cj1)*(cj3)*(sj2));
if( IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(x1076)*(x1077)))+(((sj1)*(x1074)*(x1079)))+(((x1075)*(x1079)))+(((sj1)*(x1072)*(x1079)))+(((cj1)*(x1073)*(x1074)))+(((cj1)*(x1072)*(x1073)))+(((cj3)*(x1080)))+(((IkReal(-1.00000000000000))*(x1073)*(x1078))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((sj3)*(x1076)))+(((IkReal(-1.00000000000000))*(sj3)*(x1080)))+(((x1071)*(x1075)))+(((sj1)*(x1071)*(x1074)))+(((sj1)*(x1071)*(x1072)))+(((IkReal(-1.00000000000000))*(sj2)*(x1077)*(x1078)))+(((x1074)*(x1081)))+(((x1072)*(x1081))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst60)*(((((IkReal(-1.00000000000000))*(x1076)*(x1077)))+(((sj1)*(x1074)*(x1079)))+(((x1075)*(x1079)))+(((sj1)*(x1072)*(x1079)))+(((cj1)*(x1073)*(x1074)))+(((cj1)*(x1072)*(x1073)))+(((cj3)*(x1080)))+(((IkReal(-1.00000000000000))*(x1073)*(x1078)))))), ((gconst60)*(((((sj3)*(x1076)))+(((IkReal(-1.00000000000000))*(sj3)*(x1080)))+(((x1071)*(x1075)))+(((sj1)*(x1071)*(x1074)))+(((sj1)*(x1071)*(x1072)))+(((IkReal(-1.00000000000000))*(sj2)*(x1077)*(x1078)))+(((x1074)*(x1081)))+(((x1072)*(x1081)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1082=IKsin(j5);
IkReal x1083=IKcos(j5);
IkReal x1084=((sj1)*(sj2));
IkReal x1085=((cj2)*(sj1));
IkReal x1086=((IkReal(2.16000000000000))*(sj3));
IkReal x1087=((IkReal(1.00000000000000))*(cj1));
IkReal x1088=((cj0)*(r00));
IkReal x1089=((IkReal(1.00000000000000))*(sj0));
IkReal x1090=((IkReal(1.00000000000000))*(cj0));
IkReal x1091=((sj0)*(sj2));
IkReal x1092=((cj3)*(x1083));
IkReal x1093=((cj3)*(x1082));
evalcond[0]=((((r00)*(sj0)))+(((sj3)*(x1083)))+(((IkReal(-1.00000000000000))*(x1093)))+(((IkReal(-1.00000000000000))*(r01)*(x1090))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1090)))+(((IkReal(2.16000000000000))*(x1093)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(x1083)*(x1086))));
evalcond[2]=((x1092)+(((sj3)*(x1082)))+(((IkReal(-1.00000000000000))*(x1085)*(x1088)))+(((IkReal(-1.00000000000000))*(sj2)*(x1087)*(x1088)))+(((IkReal(-1.00000000000000))*(r01)*(x1087)*(x1091)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1087)))+(((IkReal(-1.00000000000000))*(r01)*(x1085)*(x1089)))+(((r02)*(x1084))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1087)*(x1091)))+(((IkReal(-2.16000000000000))*(x1092)))+(((IkReal(-1.00000000000000))*(cj0)*(px)*(sj2)*(x1087)))+(((pz)*(x1084)))+(((IkReal(0.350000000000000))*(x1085)))+(((IkReal(-1.00000000000000))*(px)*(x1085)*(x1090)))+(((IkReal(-1.00000000000000))*(py)*(x1085)*(x1089)))+(((IkReal(0.350000000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1082)*(x1086)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1087))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1094=((IkReal(0.462962962962963))*(cj1));
IkReal x1095=((cj0)*(px));
IkReal x1096=((py)*(sj0));
IkReal x1097=((IkReal(0.462962962962963))*(sj1));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.715277777777778))+(((IkReal(-1.00000000000000))*(sj2)*(x1095)*(x1097)))+(((cj2)*(x1094)*(x1096)))+(((cj2)*(x1094)*(x1095)))+(((IkReal(-1.00000000000000))*(sj2)*(x1096)*(x1097)))+(((IkReal(0.162037037037037))*(sj1)*(sj2)))+(((IkReal(-0.162037037037037))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1097)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x1094)))+(((IkReal(-0.555555555555556))*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))+IKsqr(((IkReal(-0.715277777777778))+(((IkReal(-1.00000000000000))*(sj2)*(x1095)*(x1097)))+(((cj2)*(x1094)*(x1096)))+(((cj2)*(x1094)*(x1095)))+(((IkReal(-1.00000000000000))*(sj2)*(x1096)*(x1097)))+(((IkReal(0.162037037037037))*(sj1)*(sj2)))+(((IkReal(-0.162037037037037))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1097)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x1094)))+(((IkReal(-0.555555555555556))*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((IkReal(-0.715277777777778))+(((IkReal(-1.00000000000000))*(sj2)*(x1095)*(x1097)))+(((cj2)*(x1094)*(x1096)))+(((cj2)*(x1094)*(x1095)))+(((IkReal(-1.00000000000000))*(sj2)*(x1096)*(x1097)))+(((IkReal(0.162037037037037))*(sj1)*(sj2)))+(((IkReal(-0.162037037037037))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1097)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x1094)))+(((IkReal(-0.555555555555556))*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1098=IKsin(j5);
IkReal x1099=IKcos(j5);
IkReal x1100=((pz)*(sj2));
IkReal x1101=((cj2)*(sj1));
IkReal x1102=((r02)*(sj2));
IkReal x1103=((sj1)*(sj2));
IkReal x1104=((cj0)*(r00));
IkReal x1105=((r01)*(sj0));
IkReal x1106=((IkReal(1.00000000000000))*(cj1));
IkReal x1107=((IkReal(1.00000000000000))*(cj0));
IkReal x1108=((py)*(sj0));
IkReal x1109=((IkReal(0.350000000000000))*(cj1));
IkReal x1110=((cj0)*(px));
IkReal x1111=((sj3)*(x1098));
IkReal x1112=((cj3)*(x1098));
IkReal x1113=((cj2)*(sj0)*(x1106));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1107)))+(((IkReal(-1.00000000000000))*(x1112))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(x1107)))+(((IkReal(2.16000000000000))*(x1112)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(x1104)*(x1106)))+(((r02)*(x1101)))+(((IkReal(-1.00000000000000))*(cj2)*(x1105)*(x1106)))+(((IkReal(-1.00000000000000))*(x1099)))+(((cj1)*(x1102)))+(((x1103)*(x1105)))+(((x1103)*(x1104))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj2)*(x1104)*(x1106)))+(x1111)+(((IkReal(-1.00000000000000))*(sj2)*(x1105)*(x1106)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1106)))+(((sj1)*(x1102)))+(((IkReal(-1.00000000000000))*(x1101)*(x1105)))+(((IkReal(-1.00000000000000))*(x1101)*(x1104))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(2.16000000000000))*(x1099)))+(((cj1)*(x1100)))+(((IkReal(-1.00000000000000))*(cj2)*(x1106)*(x1110)))+(((IkReal(-1.00000000000000))*(cj2)*(x1106)*(x1108)))+(((pz)*(x1101)))+(((IkReal(-0.350000000000000))*(x1103)))+(((x1103)*(x1110)))+(((cj2)*(x1109)))+(((x1103)*(x1108))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(0.350000000000000))*(x1101)))+(((sj2)*(x1109)))+(((IkReal(-0.158000000000000))*(cj3)))+(((IkReal(-2.16000000000000))*(x1111)))+(((IkReal(-1.00000000000000))*(sj2)*(x1106)*(x1110)))+(((IkReal(-1.00000000000000))*(sj2)*(x1106)*(x1108)))+(((sj1)*(x1100)))+(((IkReal(-1.00000000000000))*(x1101)*(x1108)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1106)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(x1101)*(x1107))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1114=((r01)*(sj0));
IkReal x1115=((sj1)*(sj2));
IkReal x1116=((cj0)*(r00));
IkReal x1117=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1114)*(x1115)))+(((IkReal(-1.00000000000000))*(x1114)*(x1117)))+(((IkReal(-1.00000000000000))*(x1116)*(x1117)))+(((cj2)*(r02)*(sj1)))+(((x1115)*(x1116)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj3)))))))+IKsqr(((((x1114)*(x1115)))+(((IkReal(-1.00000000000000))*(x1114)*(x1117)))+(((IkReal(-1.00000000000000))*(x1116)*(x1117)))+(((cj2)*(r02)*(sj1)))+(((x1115)*(x1116)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(3.95000000000000))*(sj3)))))), ((((x1114)*(x1115)))+(((IkReal(-1.00000000000000))*(x1114)*(x1117)))+(((IkReal(-1.00000000000000))*(x1116)*(x1117)))+(((cj2)*(r02)*(sj1)))+(((x1115)*(x1116)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1118=IKsin(j5);
IkReal x1119=IKcos(j5);
IkReal x1120=((pz)*(sj2));
IkReal x1121=((cj2)*(sj1));
IkReal x1122=((r02)*(sj2));
IkReal x1123=((sj1)*(sj2));
IkReal x1124=((cj0)*(r00));
IkReal x1125=((r01)*(sj0));
IkReal x1126=((IkReal(1.00000000000000))*(cj1));
IkReal x1127=((IkReal(1.00000000000000))*(cj0));
IkReal x1128=((py)*(sj0));
IkReal x1129=((IkReal(0.350000000000000))*(cj1));
IkReal x1130=((cj0)*(px));
IkReal x1131=((sj3)*(x1118));
IkReal x1132=((cj3)*(x1118));
IkReal x1133=((cj2)*(sj0)*(x1126));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1132)))+(((IkReal(-1.00000000000000))*(r01)*(x1127))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x1132)))+(((IkReal(-1.00000000000000))*(py)*(x1127)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(x1125)*(x1126)))+(((IkReal(-1.00000000000000))*(x1119)))+(((cj1)*(x1122)))+(((IkReal(-1.00000000000000))*(cj2)*(x1124)*(x1126)))+(((x1123)*(x1125)))+(((x1123)*(x1124)))+(((r02)*(x1121))));
evalcond[3]=((x1131)+(((sj1)*(x1122)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1126)))+(((IkReal(-1.00000000000000))*(x1121)*(x1125)))+(((IkReal(-1.00000000000000))*(x1121)*(x1124)))+(((IkReal(-1.00000000000000))*(sj2)*(x1125)*(x1126)))+(((IkReal(-1.00000000000000))*(sj2)*(x1124)*(x1126))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-0.350000000000000))*(x1123)))+(((cj2)*(x1129)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x1121)))+(((IkReal(-1.00000000000000))*(cj2)*(x1126)*(x1128)))+(((IkReal(-1.00000000000000))*(cj2)*(x1126)*(x1130)))+(((IkReal(2.16000000000000))*(x1119)))+(((cj1)*(x1120)))+(((x1123)*(x1130)))+(((x1123)*(x1128))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-0.158000000000000))*(cj3)))+(((sj1)*(x1120)))+(((IkReal(-1.00000000000000))*(px)*(x1121)*(x1127)))+(((IkReal(-1.00000000000000))*(x1121)*(x1128)))+(((IkReal(-1.00000000000000))*(sj2)*(x1126)*(x1128)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1126)))+(((IkReal(-1.00000000000000))*(sj2)*(x1126)*(x1130)))+(((sj2)*(x1129)))+(((IkReal(0.350000000000000))*(x1121)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-2.16000000000000))*(x1131))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1134=((r01)*(sj0));
IkReal x1135=((sj1)*(sj2));
IkReal x1136=((IkReal(1.00000000000000))*(cj0));
IkReal x1137=((cj1)*(cj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1136))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj0)*(r00)*(x1135)))+(((x1134)*(x1135)))+(((IkReal(-1.00000000000000))*(x1134)*(x1137)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(x1136)*(x1137)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1136)))))))+IKsqr(((((cj0)*(r00)*(x1135)))+(((x1134)*(x1135)))+(((IkReal(-1.00000000000000))*(x1134)*(x1137)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(x1136)*(x1137)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1136)))))), ((((cj0)*(r00)*(x1135)))+(((x1134)*(x1135)))+(((IkReal(-1.00000000000000))*(x1134)*(x1137)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(x1136)*(x1137)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1138=IKsin(j5);
IkReal x1139=IKcos(j5);
IkReal x1140=((pz)*(sj2));
IkReal x1141=((cj2)*(sj1));
IkReal x1142=((r02)*(sj2));
IkReal x1143=((sj1)*(sj2));
IkReal x1144=((cj0)*(r00));
IkReal x1145=((r01)*(sj0));
IkReal x1146=((IkReal(1.00000000000000))*(cj1));
IkReal x1147=((IkReal(1.00000000000000))*(cj0));
IkReal x1148=((py)*(sj0));
IkReal x1149=((IkReal(0.350000000000000))*(cj1));
IkReal x1150=((cj0)*(px));
IkReal x1151=((sj3)*(x1138));
IkReal x1152=((cj3)*(x1138));
IkReal x1153=((cj2)*(sj0)*(x1146));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1147)))+(((IkReal(-1.00000000000000))*(x1152))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x1152)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1147))));
evalcond[2]=((((x1143)*(x1144)))+(((x1143)*(x1145)))+(((IkReal(-1.00000000000000))*(cj2)*(x1145)*(x1146)))+(((r02)*(x1141)))+(((cj1)*(x1142)))+(((IkReal(-1.00000000000000))*(cj2)*(x1144)*(x1146)))+(((IkReal(-1.00000000000000))*(x1139))));
evalcond[3]=((x1151)+(((IkReal(-1.00000000000000))*(sj2)*(x1144)*(x1146)))+(((IkReal(-1.00000000000000))*(x1141)*(x1145)))+(((IkReal(-1.00000000000000))*(x1141)*(x1144)))+(((sj1)*(x1142)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1146)))+(((IkReal(-1.00000000000000))*(sj2)*(x1145)*(x1146))));
evalcond[4]=((IkReal(1.54500000000000))+(((x1143)*(x1148)))+(((pz)*(x1141)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1146)*(x1150)))+(((x1143)*(x1150)))+(((cj2)*(x1149)))+(((IkReal(-0.350000000000000))*(x1143)))+(((IkReal(-1.00000000000000))*(cj2)*(x1146)*(x1148)))+(((IkReal(2.16000000000000))*(x1139)))+(((cj1)*(x1140))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-0.158000000000000))*(cj3)))+(((IkReal(-2.16000000000000))*(x1151)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1146)))+(((sj2)*(x1149)))+(((IkReal(0.350000000000000))*(x1141)))+(((IkReal(-1.00000000000000))*(sj2)*(x1146)*(x1150)))+(((IkReal(-1.00000000000000))*(x1141)*(x1148)))+(((IkReal(-1.00000000000000))*(sj2)*(x1146)*(x1148)))+(((sj1)*(x1140)))+(((IkReal(-1.00000000000000))*(px)*(x1141)*(x1147)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1154=((IkReal(0.462962962962963))*(cj1));
IkReal x1155=((py)*(sj0));
IkReal x1156=((cj0)*(px));
IkReal x1157=((sj1)*(sj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.715277777777778))+(((pz)*(sj2)*(x1154)))+(((IkReal(-1.00000000000000))*(cj2)*(x1154)*(x1155)))+(((IkReal(-1.00000000000000))*(cj2)*(x1154)*(x1156)))+(((IkReal(0.162037037037037))*(cj1)*(cj2)))+(((IkReal(0.462962962962963))*(cj2)*(pz)*(sj1)))+(((IkReal(0.462962962962963))*(x1155)*(x1157)))+(((IkReal(0.462962962962963))*(x1156)*(x1157)))+(((IkReal(-0.162037037037037))*(x1157)))+(((IkReal(0.555555555555556))*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))+IKsqr(((IkReal(0.715277777777778))+(((pz)*(sj2)*(x1154)))+(((IkReal(-1.00000000000000))*(cj2)*(x1154)*(x1155)))+(((IkReal(-1.00000000000000))*(cj2)*(x1154)*(x1156)))+(((IkReal(0.162037037037037))*(cj1)*(cj2)))+(((IkReal(0.462962962962963))*(cj2)*(pz)*(sj1)))+(((IkReal(0.462962962962963))*(x1155)*(x1157)))+(((IkReal(0.462962962962963))*(x1156)*(x1157)))+(((IkReal(-0.162037037037037))*(x1157)))+(((IkReal(0.555555555555556))*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((IkReal(0.715277777777778))+(((pz)*(sj2)*(x1154)))+(((IkReal(-1.00000000000000))*(cj2)*(x1154)*(x1155)))+(((IkReal(-1.00000000000000))*(cj2)*(x1154)*(x1156)))+(((IkReal(0.162037037037037))*(cj1)*(cj2)))+(((IkReal(0.462962962962963))*(cj2)*(pz)*(sj1)))+(((IkReal(0.462962962962963))*(x1155)*(x1157)))+(((IkReal(0.462962962962963))*(x1156)*(x1157)))+(((IkReal(-0.162037037037037))*(x1157)))+(((IkReal(0.555555555555556))*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1158=IKsin(j5);
IkReal x1159=IKcos(j5);
IkReal x1160=((pz)*(sj2));
IkReal x1161=((cj2)*(sj1));
IkReal x1162=((r02)*(sj2));
IkReal x1163=((sj1)*(sj2));
IkReal x1164=((cj0)*(r00));
IkReal x1165=((r01)*(sj0));
IkReal x1166=((IkReal(1.00000000000000))*(cj1));
IkReal x1167=((IkReal(1.00000000000000))*(cj0));
IkReal x1168=((py)*(sj0));
IkReal x1169=((IkReal(0.350000000000000))*(cj1));
IkReal x1170=((cj0)*(px));
IkReal x1171=((sj3)*(x1158));
IkReal x1172=((cj3)*(x1158));
IkReal x1173=((cj2)*(sj0)*(x1166));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1167)))+(((IkReal(-1.00000000000000))*(x1172))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x1172)))+(((IkReal(-1.00000000000000))*(py)*(x1167)))+(((px)*(sj0))));
evalcond[2]=((((cj1)*(x1162)))+(x1159)+(((r02)*(x1161)))+(((x1163)*(x1164)))+(((x1163)*(x1165)))+(((IkReal(-1.00000000000000))*(cj2)*(x1165)*(x1166)))+(((IkReal(-1.00000000000000))*(cj2)*(x1164)*(x1166))));
evalcond[3]=((x1171)+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1166)))+(((sj1)*(x1162)))+(((IkReal(-1.00000000000000))*(sj2)*(x1165)*(x1166)))+(((IkReal(-1.00000000000000))*(x1161)*(x1164)))+(((IkReal(-1.00000000000000))*(x1161)*(x1165)))+(((IkReal(-1.00000000000000))*(sj2)*(x1164)*(x1166))));
evalcond[4]=((IkReal(1.54500000000000))+(((cj1)*(x1160)))+(((IkReal(-2.16000000000000))*(x1159)))+(((x1163)*(x1168)))+(((IkReal(1.20000000000000))*(sj2)))+(((x1163)*(x1170)))+(((IkReal(-1.00000000000000))*(cj2)*(x1166)*(x1170)))+(((IkReal(-1.00000000000000))*(cj2)*(x1166)*(x1168)))+(((pz)*(x1161)))+(((IkReal(-0.350000000000000))*(x1163)))+(((cj2)*(x1169))));
evalcond[5]=((IkReal(-0.145000000000000))+(((sj2)*(x1169)))+(((sj1)*(x1160)))+(((IkReal(0.350000000000000))*(x1161)))+(((IkReal(0.158000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)*(x1161)*(x1167)))+(((IkReal(-1.00000000000000))*(sj2)*(x1166)*(x1170)))+(((IkReal(-1.00000000000000))*(sj2)*(x1166)*(x1168)))+(((IkReal(-2.16000000000000))*(x1171)))+(((IkReal(-1.00000000000000))*(x1161)*(x1168)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1166)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1174=((cj0)*(r00));
IkReal x1175=((cj1)*(cj2));
IkReal x1176=((IkReal(1.00000000000000))*(sj2));
IkReal x1177=((r01)*(sj0));
if( IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(-3.95000000000000))*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj1)*(x1176)*(x1177)))+(((x1174)*(x1175)))+(((x1175)*(x1177)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1176)))+(((IkReal(-1.00000000000000))*(sj1)*(x1174)*(x1176))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(-3.95000000000000))*(sj3)))))))+IKsqr(((((IkReal(-1.00000000000000))*(sj1)*(x1176)*(x1177)))+(((x1174)*(x1175)))+(((x1175)*(x1177)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1176)))+(((IkReal(-1.00000000000000))*(sj1)*(x1174)*(x1176)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))+(((IkReal(-3.95000000000000))*(sj3)))))), ((((IkReal(-1.00000000000000))*(sj1)*(x1176)*(x1177)))+(((x1174)*(x1175)))+(((x1175)*(x1177)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1176)))+(((IkReal(-1.00000000000000))*(sj1)*(x1174)*(x1176)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1178=IKsin(j5);
IkReal x1179=IKcos(j5);
IkReal x1180=((pz)*(sj2));
IkReal x1181=((cj2)*(sj1));
IkReal x1182=((r02)*(sj2));
IkReal x1183=((sj1)*(sj2));
IkReal x1184=((cj0)*(r00));
IkReal x1185=((r01)*(sj0));
IkReal x1186=((IkReal(1.00000000000000))*(cj1));
IkReal x1187=((IkReal(1.00000000000000))*(cj0));
IkReal x1188=((py)*(sj0));
IkReal x1189=((IkReal(0.350000000000000))*(cj1));
IkReal x1190=((cj0)*(px));
IkReal x1191=((sj3)*(x1178));
IkReal x1192=((cj3)*(x1178));
IkReal x1193=((cj2)*(sj0)*(x1186));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1187)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1192))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x1192)))+(((IkReal(-1.00000000000000))*(py)*(x1187)))+(((px)*(sj0))));
evalcond[2]=((x1179)+(((x1183)*(x1184)))+(((x1183)*(x1185)))+(((IkReal(-1.00000000000000))*(cj2)*(x1184)*(x1186)))+(((r02)*(x1181)))+(((cj1)*(x1182)))+(((IkReal(-1.00000000000000))*(cj2)*(x1185)*(x1186))));
evalcond[3]=((x1191)+(((IkReal(-1.00000000000000))*(x1181)*(x1184)))+(((IkReal(-1.00000000000000))*(x1181)*(x1185)))+(((sj1)*(x1182)))+(((IkReal(-1.00000000000000))*(sj2)*(x1185)*(x1186)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1186)))+(((IkReal(-1.00000000000000))*(sj2)*(x1184)*(x1186))));
evalcond[4]=((IkReal(1.54500000000000))+(((x1183)*(x1188)))+(((IkReal(-1.00000000000000))*(cj2)*(x1186)*(x1188)))+(((IkReal(1.20000000000000))*(sj2)))+(((x1183)*(x1190)))+(((pz)*(x1181)))+(((IkReal(-1.00000000000000))*(cj2)*(x1186)*(x1190)))+(((IkReal(-2.16000000000000))*(x1179)))+(((cj1)*(x1180)))+(((IkReal(-0.350000000000000))*(x1183)))+(((cj2)*(x1189))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x1181)*(x1188)))+(((IkReal(0.158000000000000))*(cj3)))+(((sj1)*(x1180)))+(((sj2)*(x1189)))+(((IkReal(-1.00000000000000))*(px)*(x1181)*(x1187)))+(((IkReal(-1.00000000000000))*(sj2)*(x1186)*(x1190)))+(((IkReal(-1.00000000000000))*(sj2)*(x1186)*(x1188)))+(((IkReal(0.350000000000000))*(x1181)))+(((IkReal(-2.16000000000000))*(x1191)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1186)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1194=((IkReal(1.00000000000000))*(r01));
IkReal x1195=((cj0)*(r00));
IkReal x1196=((cj1)*(cj2));
IkReal x1197=((IkReal(1.00000000000000))*(r02));
IkReal x1198=((sj1)*(sj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1194))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x1194)*(x1198)))+(((IkReal(-1.00000000000000))*(x1195)*(x1198)))+(((x1195)*(x1196)))+(((r01)*(sj0)*(x1196)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1197)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1197))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1194)))))))+IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x1194)*(x1198)))+(((IkReal(-1.00000000000000))*(x1195)*(x1198)))+(((x1195)*(x1196)))+(((r01)*(sj0)*(x1196)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1197)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1197)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1194)))))), ((((IkReal(-1.00000000000000))*(sj0)*(x1194)*(x1198)))+(((IkReal(-1.00000000000000))*(x1195)*(x1198)))+(((x1195)*(x1196)))+(((r01)*(sj0)*(x1196)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1197)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1197)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1199=IKsin(j5);
IkReal x1200=IKcos(j5);
IkReal x1201=((pz)*(sj2));
IkReal x1202=((cj2)*(sj1));
IkReal x1203=((r02)*(sj2));
IkReal x1204=((sj1)*(sj2));
IkReal x1205=((cj0)*(r00));
IkReal x1206=((r01)*(sj0));
IkReal x1207=((IkReal(1.00000000000000))*(cj1));
IkReal x1208=((IkReal(1.00000000000000))*(cj0));
IkReal x1209=((py)*(sj0));
IkReal x1210=((IkReal(0.350000000000000))*(cj1));
IkReal x1211=((cj0)*(px));
IkReal x1212=((sj3)*(x1199));
IkReal x1213=((cj3)*(x1199));
IkReal x1214=((cj2)*(sj0)*(x1207));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1208)))+(((IkReal(-1.00000000000000))*(x1213))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x1213)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1208))));
evalcond[2]=((((x1204)*(x1205)))+(((x1204)*(x1206)))+(x1200)+(((cj1)*(x1203)))+(((IkReal(-1.00000000000000))*(cj2)*(x1206)*(x1207)))+(((r02)*(x1202)))+(((IkReal(-1.00000000000000))*(cj2)*(x1205)*(x1207))));
evalcond[3]=((x1212)+(((IkReal(-1.00000000000000))*(sj2)*(x1205)*(x1207)))+(((IkReal(-1.00000000000000))*(x1202)*(x1206)))+(((IkReal(-1.00000000000000))*(x1202)*(x1205)))+(((sj1)*(x1203)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1207)))+(((IkReal(-1.00000000000000))*(sj2)*(x1206)*(x1207))));
evalcond[4]=((IkReal(1.54500000000000))+(((x1204)*(x1209)))+(((IkReal(-0.350000000000000))*(x1204)))+(((IkReal(1.20000000000000))*(sj2)))+(((x1204)*(x1211)))+(((cj1)*(x1201)))+(((IkReal(-1.00000000000000))*(cj2)*(x1207)*(x1209)))+(((pz)*(x1202)))+(((IkReal(-2.16000000000000))*(x1200)))+(((cj2)*(x1210)))+(((IkReal(-1.00000000000000))*(cj2)*(x1207)*(x1211))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(0.158000000000000))*(cj3)))+(((IkReal(-2.16000000000000))*(x1212)))+(((IkReal(-1.00000000000000))*(px)*(x1202)*(x1208)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1207)))+(((IkReal(-1.00000000000000))*(x1202)*(x1209)))+(((IkReal(-1.00000000000000))*(sj2)*(x1207)*(x1209)))+(((sj1)*(x1201)))+(((IkReal(0.350000000000000))*(x1202)))+(((sj2)*(x1210)))+(((IkReal(-1.00000000000000))*(sj2)*(x1207)*(x1211)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1215=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1216=((cj4)*(sj3));
IkReal x1217=((r01)*(sj0));
IkReal x1218=((cj1)*(cj2));
IkReal x1219=((IkReal(1.00000000000000))*(sj1));
IkReal x1220=((cj0)*(r00));
IkReal x1221=((cj2)*(r02));
IkReal x1222=((sj1)*(sj2));
IkReal x1223=((cj1)*(r02)*(sj2));
if( IKabs(((x1215)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(sj2)*(x1216)*(x1217)*(x1219)))+(((IkReal(-1.00000000000000))*(x1216)*(x1219)*(x1221)))+(((IkReal(-1.00000000000000))*(x1216)*(x1223)))+(((x1216)*(x1217)*(x1218)))+(((IkReal(-1.00000000000000))*(sj2)*(x1216)*(x1219)*(x1220)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((x1216)*(x1218)*(x1220))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1215)*(((((sj1)*(x1221)))+(((IkReal(-1.00000000000000))*(x1218)*(x1220)))+(x1223)+(((x1217)*(x1222)))+(((IkReal(-1.00000000000000))*(x1217)*(x1218)))+(((x1220)*(x1222))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1215)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(sj2)*(x1216)*(x1217)*(x1219)))+(((IkReal(-1.00000000000000))*(x1216)*(x1219)*(x1221)))+(((IkReal(-1.00000000000000))*(x1216)*(x1223)))+(((x1216)*(x1217)*(x1218)))+(((IkReal(-1.00000000000000))*(sj2)*(x1216)*(x1219)*(x1220)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((x1216)*(x1218)*(x1220)))))))+IKsqr(((x1215)*(((((sj1)*(x1221)))+(((IkReal(-1.00000000000000))*(x1218)*(x1220)))+(x1223)+(((x1217)*(x1222)))+(((IkReal(-1.00000000000000))*(x1217)*(x1218)))+(((x1220)*(x1222)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1215)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(sj2)*(x1216)*(x1217)*(x1219)))+(((IkReal(-1.00000000000000))*(x1216)*(x1219)*(x1221)))+(((IkReal(-1.00000000000000))*(x1216)*(x1223)))+(((x1216)*(x1217)*(x1218)))+(((IkReal(-1.00000000000000))*(sj2)*(x1216)*(x1219)*(x1220)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((x1216)*(x1218)*(x1220)))))), ((x1215)*(((((sj1)*(x1221)))+(((IkReal(-1.00000000000000))*(x1218)*(x1220)))+(x1223)+(((x1217)*(x1222)))+(((IkReal(-1.00000000000000))*(x1217)*(x1218)))+(((x1220)*(x1222)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1224=IKcos(j5);
IkReal x1225=IKsin(j5);
IkReal x1226=((cj0)*(px));
IkReal x1227=((pz)*(sj2));
IkReal x1228=((cj2)*(sj1));
IkReal x1229=((r01)*(sj0));
IkReal x1230=((cj1)*(sj2));
IkReal x1231=((IkReal(0.158000000000000))*(sj4));
IkReal x1232=((py)*(sj0));
IkReal x1233=((sj1)*(sj2));
IkReal x1234=((cj0)*(r00));
IkReal x1235=((IkReal(1.00000000000000))*(cj0));
IkReal x1236=((IkReal(1.00000000000000))*(cj3));
IkReal x1237=((IkReal(2.16000000000000))*(cj3));
IkReal x1238=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1239=((sj3)*(x1225));
IkReal x1240=((cj4)*(x1224));
IkReal x1241=((sj4)*(x1224));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1225)*(x1236)))+(((IkReal(-1.00000000000000))*(r01)*(x1235)))+(((IkReal(-1.00000000000000))*(sj3)*(x1240))));
evalcond[1]=((((IkReal(2.16000000000000))*(sj3)*(x1240)))+(((x1225)*(x1237)))+(((IkReal(-1.00000000000000))*(py)*(x1235)))+(((IkReal(-1.00000000000000))*(sj3)*(x1231)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1241)))+(((r02)*(x1228)))+(((r02)*(x1230)))+(((x1233)*(x1234)))+(((IkReal(-1.00000000000000))*(x1229)*(x1238)))+(((IkReal(-1.00000000000000))*(x1234)*(x1238)))+(((x1229)*(x1233))));
evalcond[3]=((x1239)+(((IkReal(-1.00000000000000))*(x1236)*(x1240)))+(((IkReal(-1.00000000000000))*(x1230)*(x1234)))+(((r02)*(x1233)))+(((IkReal(-1.00000000000000))*(x1228)*(x1234)))+(((IkReal(-1.00000000000000))*(x1228)*(x1229)))+(((IkReal(-1.00000000000000))*(x1229)*(x1230)))+(((IkReal(-1.00000000000000))*(r02)*(x1238))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((pz)*(x1228)))+(((IkReal(-1.00000000000000))*(x1226)*(x1238)))+(((IkReal(0.158000000000000))*(cj4)))+(((x1226)*(x1233)))+(((IkReal(-0.350000000000000))*(x1233)))+(((IkReal(2.16000000000000))*(x1241)))+(((x1232)*(x1233)))+(((IkReal(-1.00000000000000))*(x1232)*(x1238)))+(((cj1)*(x1227))));
evalcond[5]=((IkReal(-0.145000000000000))+(((sj1)*(x1227)))+(((IkReal(-1.00000000000000))*(x1226)*(x1228)))+(((IkReal(0.350000000000000))*(x1228)))+(((IkReal(-1.00000000000000))*(x1226)*(x1230)))+(((IkReal(0.350000000000000))*(x1230)))+(((IkReal(-2.16000000000000))*(x1239)))+(((IkReal(-1.00000000000000))*(pz)*(x1238)))+(((IkReal(-1.00000000000000))*(x1230)*(x1232)))+(((IkReal(-1.00000000000000))*(x1228)*(x1232)))+(((IkReal(-1.00000000000000))*(cj3)*(x1231)))+(((IkReal(-1.20000000000000))*(cj2)))+(((x1237)*(x1240))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1242=(cj3)*(cj3);
IkReal x1243=(sj3)*(sj3);
IkReal x1244=((cj0)*(r00));
IkReal x1245=((IkReal(25.0000000000000))*(sj3));
IkReal x1246=((px)*(sj0));
IkReal x1247=((cj0)*(py));
IkReal x1248=((IkReal(25.0000000000000))*(cj3));
IkReal x1249=((r01)*(sj0));
IkReal x1250=((r02)*(sj2));
IkReal x1251=((IkReal(3.95000000000000))*(sj4));
IkReal x1252=((cj2)*(r02));
IkReal x1253=((IkReal(54.0000000000000))*(x1243));
IkReal x1254=((IkReal(54.0000000000000))*(cj3)*(sj1));
IkReal x1255=((IkReal(54.0000000000000))*(cj1)*(cj3));
IkReal x1256=((IkReal(54.0000000000000))*(x1242));
IkReal x1257=((IkReal(54.0000000000000))*(sj1)*(sj3));
IkReal x1258=((IkReal(54.0000000000000))*(cj1)*(sj2)*(sj3));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1256))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1256)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1249)*(x1258)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1257)))+(((x1250)*(x1257)))+(((IkReal(-1.00000000000000))*(x1244)*(x1258)))+(((x1246)*(x1248)))+(((IkReal(-54.0000000000000))*(cj1)*(sj3)*(x1252)))+(((IkReal(-1.00000000000000))*(cj3)*(sj3)*(x1251)))+(((IkReal(-1.00000000000000))*(x1247)*(x1248)))+(((IkReal(-1.00000000000000))*(cj2)*(x1244)*(x1257))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1256)))+(((cj4)*(x1253))))) != 0)?((IkReal)1/(((((cj4)*(x1256)))+(((cj4)*(x1253)))))):(IkReal)1.0e30))*(((((x1243)*(x1251)))+(((IkReal(-1.00000000000000))*(x1245)*(x1246)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1254)))+(((IkReal(-1.00000000000000))*(x1252)*(x1255)))+(((x1250)*(x1254)))+(((x1245)*(x1247)))+(((IkReal(-1.00000000000000))*(sj2)*(x1249)*(x1255)))+(((IkReal(-1.00000000000000))*(sj2)*(x1244)*(x1255)))+(((IkReal(-1.00000000000000))*(cj2)*(x1244)*(x1254))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1256))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1256)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1249)*(x1258)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1257)))+(((x1250)*(x1257)))+(((IkReal(-1.00000000000000))*(x1244)*(x1258)))+(((x1246)*(x1248)))+(((IkReal(-54.0000000000000))*(cj1)*(sj3)*(x1252)))+(((IkReal(-1.00000000000000))*(cj3)*(sj3)*(x1251)))+(((IkReal(-1.00000000000000))*(x1247)*(x1248)))+(((IkReal(-1.00000000000000))*(cj2)*(x1244)*(x1257)))))))+IKsqr(((((IKabs(((((cj4)*(x1256)))+(((cj4)*(x1253))))) != 0)?((IkReal)1/(((((cj4)*(x1256)))+(((cj4)*(x1253)))))):(IkReal)1.0e30))*(((((x1243)*(x1251)))+(((IkReal(-1.00000000000000))*(x1245)*(x1246)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1254)))+(((IkReal(-1.00000000000000))*(x1252)*(x1255)))+(((x1250)*(x1254)))+(((x1245)*(x1247)))+(((IkReal(-1.00000000000000))*(sj2)*(x1249)*(x1255)))+(((IkReal(-1.00000000000000))*(sj2)*(x1244)*(x1255)))+(((IkReal(-1.00000000000000))*(cj2)*(x1244)*(x1254)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1256))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1256)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1249)*(x1258)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1257)))+(((x1250)*(x1257)))+(((IkReal(-1.00000000000000))*(x1244)*(x1258)))+(((x1246)*(x1248)))+(((IkReal(-54.0000000000000))*(cj1)*(sj3)*(x1252)))+(((IkReal(-1.00000000000000))*(cj3)*(sj3)*(x1251)))+(((IkReal(-1.00000000000000))*(x1247)*(x1248)))+(((IkReal(-1.00000000000000))*(cj2)*(x1244)*(x1257)))))), ((((IKabs(((((cj4)*(x1256)))+(((cj4)*(x1253))))) != 0)?((IkReal)1/(((((cj4)*(x1256)))+(((cj4)*(x1253)))))):(IkReal)1.0e30))*(((((x1243)*(x1251)))+(((IkReal(-1.00000000000000))*(x1245)*(x1246)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1254)))+(((IkReal(-1.00000000000000))*(x1252)*(x1255)))+(((x1250)*(x1254)))+(((x1245)*(x1247)))+(((IkReal(-1.00000000000000))*(sj2)*(x1249)*(x1255)))+(((IkReal(-1.00000000000000))*(sj2)*(x1244)*(x1255)))+(((IkReal(-1.00000000000000))*(cj2)*(x1244)*(x1254)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1259=IKcos(j5);
IkReal x1260=IKsin(j5);
IkReal x1261=((cj0)*(px));
IkReal x1262=((pz)*(sj2));
IkReal x1263=((cj2)*(sj1));
IkReal x1264=((r01)*(sj0));
IkReal x1265=((cj1)*(sj2));
IkReal x1266=((IkReal(0.158000000000000))*(sj4));
IkReal x1267=((py)*(sj0));
IkReal x1268=((sj1)*(sj2));
IkReal x1269=((cj0)*(r00));
IkReal x1270=((IkReal(1.00000000000000))*(cj0));
IkReal x1271=((IkReal(1.00000000000000))*(cj3));
IkReal x1272=((IkReal(2.16000000000000))*(cj3));
IkReal x1273=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1274=((sj3)*(x1260));
IkReal x1275=((cj4)*(x1259));
IkReal x1276=((sj4)*(x1259));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1270)))+(((IkReal(-1.00000000000000))*(x1260)*(x1271)))+(((IkReal(-1.00000000000000))*(sj3)*(x1275))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1270)))+(((x1260)*(x1272)))+(((IkReal(-1.00000000000000))*(sj3)*(x1266)))+(((px)*(sj0)))+(((IkReal(2.16000000000000))*(sj3)*(x1275))));
evalcond[2]=((((x1264)*(x1268)))+(((r02)*(x1263)))+(((r02)*(x1265)))+(((IkReal(-1.00000000000000))*(x1269)*(x1273)))+(((x1268)*(x1269)))+(((IkReal(-1.00000000000000))*(x1276)))+(((IkReal(-1.00000000000000))*(x1264)*(x1273))));
evalcond[3]=((x1274)+(((r02)*(x1268)))+(((IkReal(-1.00000000000000))*(x1265)*(x1269)))+(((IkReal(-1.00000000000000))*(x1271)*(x1275)))+(((IkReal(-1.00000000000000))*(x1264)*(x1265)))+(((IkReal(-1.00000000000000))*(r02)*(x1273)))+(((IkReal(-1.00000000000000))*(x1263)*(x1264)))+(((IkReal(-1.00000000000000))*(x1263)*(x1269))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(-1.00000000000000))*(x1267)*(x1273)))+(((x1261)*(x1268)))+(((IkReal(-1.00000000000000))*(x1261)*(x1273)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(2.16000000000000))*(x1276)))+(((x1267)*(x1268)))+(((cj1)*(x1262)))+(((IkReal(-0.350000000000000))*(x1268)))+(((pz)*(x1263))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(cj3)*(x1266)))+(((x1272)*(x1275)))+(((sj1)*(x1262)))+(((IkReal(-1.00000000000000))*(x1265)*(x1267)))+(((IkReal(-2.16000000000000))*(x1274)))+(((IkReal(-1.00000000000000))*(x1263)*(x1267)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1273)))+(((IkReal(0.350000000000000))*(x1263)))+(((IkReal(0.350000000000000))*(x1265)))+(((IkReal(-1.00000000000000))*(x1261)*(x1263)))+(((IkReal(-1.00000000000000))*(x1261)*(x1265))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1277=(sj3)*(sj3);
IkReal x1278=(cj3)*(cj3);
IkReal x1279=((cj2)*(r02));
IkReal x1280=((cj1)*(sj3));
IkReal x1281=((cj0)*(r00));
IkReal x1282=((IkReal(1.00000000000000))*(cj3));
IkReal x1283=((cj2)*(sj1));
IkReal x1284=((r00)*(sj0));
IkReal x1285=((cj1)*(x1282));
IkReal x1286=((r01)*(sj0)*(sj2));
IkReal x1287=((r02)*(sj1)*(sj2));
IkReal x1288=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x1289=((r01)*(sj0)*(x1283));
if( IKabs(((((IKabs(((x1278)+(x1277))) != 0)?((IkReal)1/(((x1278)+(x1277)))):(IkReal)1.0e30))*(((((x1279)*(x1280)))+(((sj2)*(x1280)*(x1281)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x1282)))+(((sj3)*(x1281)*(x1283)))+(((sj3)*(x1289)))+(((cj3)*(x1284)))+(((x1280)*(x1286)))+(((IkReal(-1.00000000000000))*(sj3)*(x1287))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1277)))+(((cj4)*(x1278))))) != 0)?((IkReal)1/(((((cj4)*(x1277)))+(((cj4)*(x1278)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(x1281)*(x1285)))+(((IkReal(-1.00000000000000))*(x1285)*(x1286)))+(((IkReal(-1.00000000000000))*(x1281)*(x1282)*(x1283)))+(((sj3)*(x1284)))+(((cj3)*(x1287)))+(((IkReal(-1.00000000000000))*(sj3)*(x1288)))+(((IkReal(-1.00000000000000))*(x1279)*(x1285)))+(((IkReal(-1.00000000000000))*(x1282)*(x1289))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x1278)+(x1277))) != 0)?((IkReal)1/(((x1278)+(x1277)))):(IkReal)1.0e30))*(((((x1279)*(x1280)))+(((sj2)*(x1280)*(x1281)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x1282)))+(((sj3)*(x1281)*(x1283)))+(((sj3)*(x1289)))+(((cj3)*(x1284)))+(((x1280)*(x1286)))+(((IkReal(-1.00000000000000))*(sj3)*(x1287)))))))+IKsqr(((((IKabs(((((cj4)*(x1277)))+(((cj4)*(x1278))))) != 0)?((IkReal)1/(((((cj4)*(x1277)))+(((cj4)*(x1278)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(x1281)*(x1285)))+(((IkReal(-1.00000000000000))*(x1285)*(x1286)))+(((IkReal(-1.00000000000000))*(x1281)*(x1282)*(x1283)))+(((sj3)*(x1284)))+(((cj3)*(x1287)))+(((IkReal(-1.00000000000000))*(sj3)*(x1288)))+(((IkReal(-1.00000000000000))*(x1279)*(x1285)))+(((IkReal(-1.00000000000000))*(x1282)*(x1289)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x1278)+(x1277))) != 0)?((IkReal)1/(((x1278)+(x1277)))):(IkReal)1.0e30))*(((((x1279)*(x1280)))+(((sj2)*(x1280)*(x1281)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x1282)))+(((sj3)*(x1281)*(x1283)))+(((sj3)*(x1289)))+(((cj3)*(x1284)))+(((x1280)*(x1286)))+(((IkReal(-1.00000000000000))*(sj3)*(x1287)))))), ((((IKabs(((((cj4)*(x1277)))+(((cj4)*(x1278))))) != 0)?((IkReal)1/(((((cj4)*(x1277)))+(((cj4)*(x1278)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(x1281)*(x1285)))+(((IkReal(-1.00000000000000))*(x1285)*(x1286)))+(((IkReal(-1.00000000000000))*(x1281)*(x1282)*(x1283)))+(((sj3)*(x1284)))+(((cj3)*(x1287)))+(((IkReal(-1.00000000000000))*(sj3)*(x1288)))+(((IkReal(-1.00000000000000))*(x1279)*(x1285)))+(((IkReal(-1.00000000000000))*(x1282)*(x1289)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1290=IKcos(j5);
IkReal x1291=IKsin(j5);
IkReal x1292=((cj0)*(px));
IkReal x1293=((pz)*(sj2));
IkReal x1294=((cj2)*(sj1));
IkReal x1295=((r01)*(sj0));
IkReal x1296=((cj1)*(sj2));
IkReal x1297=((IkReal(0.158000000000000))*(sj4));
IkReal x1298=((py)*(sj0));
IkReal x1299=((sj1)*(sj2));
IkReal x1300=((cj0)*(r00));
IkReal x1301=((IkReal(1.00000000000000))*(cj0));
IkReal x1302=((IkReal(1.00000000000000))*(cj3));
IkReal x1303=((IkReal(2.16000000000000))*(cj3));
IkReal x1304=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1305=((sj3)*(x1291));
IkReal x1306=((cj4)*(x1290));
IkReal x1307=((sj4)*(x1290));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1301)))+(((IkReal(-1.00000000000000))*(x1291)*(x1302)))+(((IkReal(-1.00000000000000))*(sj3)*(x1306))));
evalcond[1]=((((x1291)*(x1303)))+(((IkReal(-1.00000000000000))*(sj3)*(x1297)))+(((IkReal(2.16000000000000))*(sj3)*(x1306)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1301))));
evalcond[2]=((((x1299)*(x1300)))+(((x1295)*(x1299)))+(((IkReal(-1.00000000000000))*(x1307)))+(((IkReal(-1.00000000000000))*(x1300)*(x1304)))+(((r02)*(x1296)))+(((r02)*(x1294)))+(((IkReal(-1.00000000000000))*(x1295)*(x1304))));
evalcond[3]=((x1305)+(((IkReal(-1.00000000000000))*(x1294)*(x1300)))+(((IkReal(-1.00000000000000))*(x1296)*(x1300)))+(((IkReal(-1.00000000000000))*(r02)*(x1304)))+(((IkReal(-1.00000000000000))*(x1294)*(x1295)))+(((r02)*(x1299)))+(((IkReal(-1.00000000000000))*(x1302)*(x1306)))+(((IkReal(-1.00000000000000))*(x1295)*(x1296))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(0.350000000000000))*(cj1)*(cj2)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((cj1)*(x1293)))+(((pz)*(x1294)))+(((IkReal(2.16000000000000))*(x1307)))+(((IkReal(-1.00000000000000))*(x1292)*(x1304)))+(((IkReal(-0.350000000000000))*(x1299)))+(((IkReal(-1.00000000000000))*(x1298)*(x1304)))+(((x1292)*(x1299)))+(((x1298)*(x1299))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x1305)))+(((sj1)*(x1293)))+(((IkReal(0.350000000000000))*(x1294)))+(((IkReal(0.350000000000000))*(x1296)))+(((x1303)*(x1306)))+(((IkReal(-1.00000000000000))*(x1294)*(x1298)))+(((IkReal(-1.00000000000000))*(x1292)*(x1296)))+(((IkReal(-1.00000000000000))*(x1292)*(x1294)))+(((IkReal(-1.00000000000000))*(x1296)*(x1298)))+(((IkReal(-1.00000000000000))*(pz)*(x1304)))+(((IkReal(-1.00000000000000))*(cj3)*(x1297)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "1999c6d2c0204e8fd3c6915845778c23"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
