/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:08:54.748604
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[1]);
x4=IKsin(j[2]);
x5=IKsin(j[0]);
x6=IKsin(j[4]);
x7=IKsin(j[3]);
x8=IKcos(j[3]);
x9=IKcos(j[4]);
x10=IKcos(j[5]);
x11=IKsin(j[5]);
x12=((IkReal(0.0900000000000000))*(x5));
x13=((IkReal(0.0900000000000000))*(x0));
x14=((IkReal(0.295000000000000))*(x5));
x15=((IkReal(0.0800000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x0));
x17=((IkReal(0.270000000000000))*(x3));
x18=((IkReal(1.00000000000000))*(x5));
x19=((IkReal(0.295000000000000))*(x0));
x20=((IkReal(0.0800000000000000))*(x0));
x21=((IkReal(0.0800000000000000))*(x8));
x22=((x2)*(x3));
x23=((x3)*(x4));
x24=((x1)*(x2));
x25=((IkReal(-1.00000000000000))*(x8));
x26=((x0)*(x7));
x27=((x1)*(x4));
x28=((IkReal(-1.00000000000000))*(x6));
x29=((((IkReal(-1.00000000000000))*(x24)))+(x23));
x30=((x16)*(((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x27))))));
x31=((x18)*(((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x27))))));
x32=((x30)*(x8));
IkReal x33=((IkReal(1.00000000000000))*(x23));
eetrans[0]=((((x0)*(x17)))+(((x19)*(x24)))+(((x6)*(((((x21)*(x30)))+(((IkReal(-1.00000000000000))*(x15)*(x7)))))))+(((IkReal(0.0750000000000000))*(x0)))+(((x9)*(((((IkReal(-1.00000000000000))*(x20)*(x33)))+(((x20)*(x24)))))))+(((IkReal(-1.00000000000000))*(x19)*(x33)))+(((x13)*(x22)))+(((x13)*(x27))));
IkReal x34=((IkReal(1.00000000000000))*(x12));
IkReal x35=((IkReal(1.00000000000000))*(x24));
eetrans[1]=((((IkReal(-1.00000000000000))*(x22)*(x34)))+(((x9)*(((((IkReal(-1.00000000000000))*(x15)*(x35)))+(((x15)*(x23)))))))+(((IkReal(-1.00000000000000))*(x27)*(x34)))+(((x14)*(x23)))+(((IkReal(-0.0750000000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x17)*(x5)))+(((x6)*(((((IkReal(-1.00000000000000))*(x21)*(x31)))+(((IkReal(-1.00000000000000))*(x20)*(x7)))))))+(((IkReal(-1.00000000000000))*(x14)*(x35))));
eetrans[2]=((IkReal(0.335000000000000))+(((x9)*(((((IkReal(-0.0800000000000000))*(x22)))+(((IkReal(-0.0800000000000000))*(x27)))))))+(((IkReal(0.270000000000000))*(x1)))+(((IkReal(-0.0900000000000000))*(x23)))+(((IkReal(-0.295000000000000))*(x22)))+(((IkReal(-0.295000000000000))*(x27)))+(((x6)*(x8)*(((((IkReal(-0.0800000000000000))*(x24)))+(((IkReal(0.0800000000000000))*(x23)))))))+(((IkReal(0.0900000000000000))*(x24))));
eerot[0]=((((x11)*(((((x30)*(x7)))+(((x5)*(x8)))))))+(((x10)*(((((x6)*(((((x0)*(x24)))+(((IkReal(-1.00000000000000))*(x16)*(x23)))))))+(((x9)*(((((x18)*(x7)))+(((IkReal(-1.00000000000000))*(x32))))))))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x9)*(((((IkReal(-1.00000000000000))*(x26)))+(((x25)*(x31)))))))+(((x28)*(((((IkReal(-1.00000000000000))*(x18)*(x23)))+(((x24)*(x5)))))))))))+(((x11)*(((((x16)*(x8)))+(((IkReal(-1.00000000000000))*(x31)*(x7))))))));
eerot[2]=((((x11)*(x29)*(x7)))+(((x10)*(((((x25)*(x29)*(x9)))+(((x28)*(((x27)+(x22))))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j1,cj1,sj1,htj1,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=px;
new_r01=((IkReal(-1.00000000000000))*(r01));
new_py=((IkReal(-1.00000000000000))*(py));
new_r02=((IkReal(-1.00000000000000))*(r02));
new_pz=((IkReal(0.335000000000000))+(((IkReal(-1.00000000000000))*(pz))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(-0.360000000000000))*(r00)*(sj1)))+(((IkReal(0.360000000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(r02)*(sj1)))+(((IkReal(-0.295000000000000))*(cj1)*(r00))));
IkReal gconst1;
gconst1=((((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.590000000000000))*(cj1)*(r02))));
IkReal gconst2;
gconst2=((((IkReal(0.295000000000000))*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(r00)*(sj1)))+(((IkReal(0.295000000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.180000000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst3;
gconst3=((IkReal(0.215850000000000))+(((IkReal(0.0540000000000000))*(sj1)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj1)))+(pp)+(((IkReal(0.0442500000000000))*(cj1)))+(((IkReal(-0.590000000000000))*(cj1)*(px)))+(((IkReal(-0.720000000000000))*(px)*(sj1)))+(((IkReal(0.720000000000000))*(cj1)*(pz))));
IkReal gconst4;
gconst4=((IkReal(-0.318600000000000))+(((IkReal(-0.0885000000000000))*(sj1)))+(((IkReal(1.18000000000000))*(px)*(sj1)))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-0.360000000000000))*(cj1)*(px)))+(((IkReal(-1.18000000000000))*(cj1)*(pz)))+(((IkReal(0.0270000000000000))*(cj1))));
IkReal gconst5;
gconst5=((IkReal(0.118650000000000))+(((IkReal(0.590000000000000))*(cj1)*(px)))+(((IkReal(-0.0442500000000000))*(cj1)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(0.360000000000000))*(cj1)*(pz)))+(((IkReal(-0.360000000000000))*(px)*(sj1)))+(pp)+(((IkReal(0.0270000000000000))*(sj1)))+(((IkReal(0.590000000000000))*(pz)*(sj1))));
IkReal gconst6;
gconst6=((((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(-0.360000000000000))*(r00)*(sj1)))+(((IkReal(0.360000000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(r02)*(sj1)))+(((IkReal(-0.295000000000000))*(cj1)*(r00))));
IkReal gconst7;
gconst7=((((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.590000000000000))*(cj1)*(r02))));
IkReal gconst8;
gconst8=((((IkReal(0.295000000000000))*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(r00)*(sj1)))+(((IkReal(0.295000000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.180000000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst9;
gconst9=((IkReal(0.215850000000000))+(((IkReal(0.0540000000000000))*(sj1)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj1)))+(pp)+(((IkReal(0.0442500000000000))*(cj1)))+(((IkReal(-0.590000000000000))*(cj1)*(px)))+(((IkReal(-0.720000000000000))*(px)*(sj1)))+(((IkReal(0.720000000000000))*(cj1)*(pz))));
IkReal gconst10;
gconst10=((IkReal(-0.318600000000000))+(((IkReal(-0.0885000000000000))*(sj1)))+(((IkReal(1.18000000000000))*(px)*(sj1)))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-0.360000000000000))*(cj1)*(px)))+(((IkReal(-1.18000000000000))*(cj1)*(pz)))+(((IkReal(0.0270000000000000))*(cj1))));
IkReal gconst11;
gconst11=((IkReal(0.118650000000000))+(((IkReal(0.590000000000000))*(cj1)*(px)))+(((IkReal(-0.0442500000000000))*(cj1)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(0.360000000000000))*(cj1)*(pz)))+(((IkReal(-0.360000000000000))*(px)*(sj1)))+(pp)+(((IkReal(0.0270000000000000))*(sj1)))+(((IkReal(0.590000000000000))*(pz)*(sj1))));
IkReal gconst12;
gconst12=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.720000000000000))*(r01)*(sj1)))+(((IkReal(-0.590000000000000))*(cj1)*(r01))));
IkReal gconst13;
gconst13=((((IkReal(-0.360000000000000))*(cj1)*(r01)))+(((IkReal(1.18000000000000))*(r01)*(sj1))));
IkReal gconst14;
gconst14=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(0.590000000000000))*(cj1)*(r01)))+(((IkReal(-0.360000000000000))*(r01)*(sj1))));
IkReal gconst15;
gconst15=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-1.44000000000000))*(py)*(sj1)))+(((IkReal(-1.18000000000000))*(cj1)*(py))));
IkReal gconst16;
gconst16=((((IkReal(2.36000000000000))*(py)*(sj1)))+(((IkReal(-0.720000000000000))*(cj1)*(py))));
IkReal gconst17;
gconst17=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.18000000000000))*(cj1)*(py)))+(((IkReal(-0.720000000000000))*(py)*(sj1))));
IkReal gconst18;
gconst18=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.720000000000000))*(r01)*(sj1)))+(((IkReal(-0.590000000000000))*(cj1)*(r01))));
IkReal gconst19;
gconst19=((((IkReal(-0.360000000000000))*(cj1)*(r01)))+(((IkReal(1.18000000000000))*(r01)*(sj1))));
IkReal gconst20;
gconst20=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(0.590000000000000))*(cj1)*(r01)))+(((IkReal(-0.360000000000000))*(r01)*(sj1))));
IkReal gconst21;
gconst21=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-1.44000000000000))*(py)*(sj1)))+(((IkReal(-1.18000000000000))*(cj1)*(py))));
IkReal gconst22;
gconst22=((((IkReal(2.36000000000000))*(py)*(sj1)))+(((IkReal(-0.720000000000000))*(cj1)*(py))));
IkReal gconst23;
gconst23=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.18000000000000))*(cj1)*(py)))+(((IkReal(-0.720000000000000))*(py)*(sj1))));
IkReal gconst24;
gconst24=((((IkReal(0.295000000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(0.360000000000000))*(r00)*(sj1)))+(((IkReal(0.0750000000000000))*(r00)))+(((IkReal(0.360000000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(r02)*(sj1))));
IkReal gconst25;
gconst25=((((IkReal(-0.590000000000000))*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.590000000000000))*(cj1)*(r02))));
IkReal gconst26;
gconst26=((((IkReal(0.295000000000000))*(r02)*(sj1)))+(((px)*(r00)))+(((IkReal(0.180000000000000))*(cj1)*(r02)))+(((IkReal(0.180000000000000))*(r00)*(sj1)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(cj1)*(r00))));
IkReal gconst27;
gconst27=((IkReal(0.215850000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.0540000000000000))*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj1)))+(((IkReal(0.720000000000000))*(px)*(sj1)))+(pp)+(((IkReal(0.0442500000000000))*(cj1)))+(((IkReal(0.720000000000000))*(cj1)*(pz))));
IkReal gconst28;
gconst28=((IkReal(-0.318600000000000))+(((IkReal(-0.0885000000000000))*(sj1)))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-1.18000000000000))*(px)*(sj1)))+(((IkReal(-1.18000000000000))*(cj1)*(pz)))+(((IkReal(0.360000000000000))*(cj1)*(px)))+(((IkReal(0.0270000000000000))*(cj1))));
IkReal gconst29;
gconst29=((IkReal(0.118650000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(-0.0442500000000000))*(cj1)))+(((IkReal(0.360000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(-0.590000000000000))*(cj1)*(px)))+(((IkReal(0.0270000000000000))*(sj1)))+(((IkReal(0.590000000000000))*(pz)*(sj1)))+(((IkReal(0.360000000000000))*(px)*(sj1))));
IkReal gconst30;
gconst30=((((IkReal(0.295000000000000))*(cj1)*(r00)))+(((px)*(r00)))+(((IkReal(0.360000000000000))*(r00)*(sj1)))+(((IkReal(0.0750000000000000))*(r00)))+(((IkReal(0.360000000000000))*(cj1)*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(r02)*(sj1))));
IkReal gconst31;
gconst31=((((IkReal(-0.590000000000000))*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.590000000000000))*(cj1)*(r02))));
IkReal gconst32;
gconst32=((((IkReal(0.295000000000000))*(r02)*(sj1)))+(((px)*(r00)))+(((IkReal(0.180000000000000))*(cj1)*(r02)))+(((IkReal(0.180000000000000))*(r00)*(sj1)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.295000000000000))*(cj1)*(r00))));
IkReal gconst33;
gconst33=((IkReal(0.215850000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.0540000000000000))*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(px)))+(((IkReal(-0.590000000000000))*(pz)*(sj1)))+(((IkReal(0.720000000000000))*(px)*(sj1)))+(pp)+(((IkReal(0.0442500000000000))*(cj1)))+(((IkReal(0.720000000000000))*(cj1)*(pz))));
IkReal gconst34;
gconst34=((IkReal(-0.318600000000000))+(((IkReal(-0.0885000000000000))*(sj1)))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-1.18000000000000))*(px)*(sj1)))+(((IkReal(-1.18000000000000))*(cj1)*(pz)))+(((IkReal(0.360000000000000))*(cj1)*(px)))+(((IkReal(0.0270000000000000))*(cj1))));
IkReal gconst35;
gconst35=((IkReal(0.118650000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(-0.0442500000000000))*(cj1)))+(((IkReal(0.360000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(-0.590000000000000))*(cj1)*(px)))+(((IkReal(0.0270000000000000))*(sj1)))+(((IkReal(0.590000000000000))*(pz)*(sj1)))+(((IkReal(0.360000000000000))*(px)*(sj1))));
IkReal x36=((IkReal(1.00000000000000))*(gconst26));
IkReal x37=((gconst30)*(gconst34));
IkReal x38=((gconst31)*(gconst33));
IkReal x39=((IkReal(1.00000000000000))*(gconst17));
IkReal x40=((gconst2)*(gconst6));
IkReal x41=((gconst11)*(gconst3));
IkReal x42=((gconst25)*(gconst29));
IkReal x43=((gconst14)*(gconst28));
IkReal x44=((IkReal(1.00000000000000))*(gconst14));
IkReal x45=((gconst21)*(gconst32));
IkReal x46=((gconst10)*(gconst6));
IkReal x47=((gconst21)*(gconst5));
IkReal x48=((gconst0)*(gconst8));
IkReal x49=((gconst13)*(gconst5));
IkReal x50=((IkReal(1.00000000000000))*(gconst9));
IkReal x51=((gconst30)*(gconst35));
IkReal x52=((gconst26)*(gconst27));
IkReal x53=((gconst22)*(gconst30));
IkReal x54=((IkReal(1.00000000000000))*(gconst12));
IkReal x55=((gconst2)*(gconst3));
IkReal x56=((gconst1)*(gconst5));
IkReal x57=((gconst19)*(gconst33));
IkReal x58=((gconst26)*(gconst28));
IkReal x59=((gconst11)*(gconst6));
IkReal x60=((gconst1)*(gconst7));
IkReal x61=((gconst18)*(gconst2));
IkReal x62=((gconst18)*(gconst34));
IkReal x63=((gconst24)*(gconst29));
IkReal x64=((IkReal(1.00000000000000))*(gconst23));
IkReal x65=((gconst20)*(gconst33));
IkReal x66=((gconst32)*(gconst33));
IkReal x67=((gconst0)*(gconst5));
IkReal x68=((IkReal(1.00000000000000))*(gconst18));
IkReal x69=((gconst21)*(gconst31));
IkReal x70=((gconst5)*(gconst9));
IkReal x71=((gconst4)*(gconst9));
IkReal x72=((gconst13)*(gconst29));
IkReal x73=((gconst14)*(gconst6));
IkReal x74=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x75=((gconst27)*(x66));
IkReal x76=((gconst2)*(gconst4)*(gconst7));
op[0]=((((x38)*(x58)))+(((IkReal(-1.00000000000000))*(x51)*(x63)))+(((x37)*(x42)))+(((IkReal(-1.00000000000000))*(x36)*(x75)))+(((x63)*(x66)))+(((x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst28)*(x36)*(x37)))+(((IkReal(-1.00000000000000))*(x38)*(x42))));
op[1]=((((gconst23)*(gconst30)*(x52)))+(((IkReal(-1.00000000000000))*(x44)*(x75)))+(((IkReal(-1.00000000000000))*(gconst15)*(x36)*(x66)))+(((IkReal(-1.00000000000000))*(gconst35)*(x63)*(x68)))+(((gconst16)*(gconst26)*(x38)))+(((gconst12)*(gconst29)*(x66)))+(((IkReal(-1.00000000000000))*(gconst28)*(x36)*(x62)))+(((IkReal(-1.00000000000000))*(x42)*(x69)))+(((x58)*(x69)))+(((IkReal(-1.00000000000000))*(gconst28)*(x36)*(x53)))+(((x45)*(x63)))+(((IkReal(-1.00000000000000))*(x42)*(x57)))+(((IkReal(-1.00000000000000))*(x38)*(x72)))+(((IkReal(-1.00000000000000))*(gconst16)*(x36)*(x37)))+(((x38)*(x43)))+(((gconst15)*(gconst26)*(x51)))+(((IkReal(-1.00000000000000))*(gconst25)*(x38)*(x39)))+(((gconst17)*(gconst25)*(x37)))+(((x42)*(x53)))+(((x57)*(x58)))+(((gconst14)*(gconst27)*(x51)))+(((gconst17)*(gconst24)*(x66)))+(((x42)*(x62)))+(((IkReal(-1.00000000000000))*(gconst27)*(x36)*(x65)))+(((IkReal(-1.00000000000000))*(x37)*(x43)))+(((x63)*(x65)))+(((gconst18)*(gconst35)*(x52)))+(((x37)*(x72)))+(((IkReal(-1.00000000000000))*(gconst27)*(x36)*(x45)))+(((IkReal(-1.00000000000000))*(gconst29)*(x51)*(x54)))+(((IkReal(-1.00000000000000))*(gconst30)*(x63)*(x64)))+(((IkReal(-1.00000000000000))*(gconst24)*(x39)*(x51))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((IkReal(-1.00000000000000))*(x47)*(x60)))+(((IkReal(-1.00000000000000))*(gconst19)*(x50)*(x56)))+(((IkReal(-1.00000000000000))*(gconst6)*(x64)*(x67)))+(((gconst10)*(gconst18)*(x56)))+(((gconst22)*(gconst6)*(x56)))+(((IkReal(-1.00000000000000))*(gconst20)*(x50)*(x55)))+(((IkReal(-1.00000000000000))*(gconst9)*(x39)*(x60)))+(((IkReal(-1.00000000000000))*(gconst0)*(x39)*(x59)))+(((x47)*(x48)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x40)))+(((x41)*(x73)))+(((gconst12)*(gconst8)*(x70)))+(((x46)*(x49)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x55)))+(((gconst19)*(gconst2)*(x71)))+(((gconst11)*(gconst15)*(x40)))+(((IkReal(-1.00000000000000))*(gconst11)*(x67)*(x68)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x44)))+(((IkReal(-1.00000000000000))*(x61)*(x74)))+(((gconst14)*(gconst7)*(x71)))+(((IkReal(-1.00000000000000))*(gconst7)*(x49)*(x50)))+(((gconst21)*(x76)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x50)))+(((gconst20)*(gconst9)*(x67)))+(((IkReal(-1.00000000000000))*(gconst4)*(x44)*(x46)))+(((x41)*(x61)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((gconst17)*(gconst9)*(x48)))+(((gconst23)*(gconst3)*(x40)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x40)))+(((gconst1)*(gconst17)*(x46)))+(((IkReal(-1.00000000000000))*(gconst5)*(x54)*(x59))));
op[8]=((((x40)*(x41)))+(((IkReal(-1.00000000000000))*(x59)*(x67)))+(((x48)*(x70)))+(((IkReal(-1.00000000000000))*(gconst8)*(x50)*(x55)))+(((IkReal(-1.00000000000000))*(gconst7)*(x50)*(x56)))+(((x46)*(x56)))+(((gconst2)*(gconst7)*(x71)))+(((IkReal(-1.00000000000000))*(x40)*(x74))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x77=(sj1)*(sj1);
IkReal x78=(cj1)*(cj1);
IkReal x79=((py)*(sj0));
IkReal x80=((cj0)*(px));
IkReal x81=((r01)*(sj0));
IkReal x82=((cj0)*(r00));
IkReal x83=((IkReal(0.0513675000000000))*(cj1));
IkReal x84=((r02)*(x77));
IkReal x85=((IkReal(0.190250000000000))*(r02)*(x78));
IkReal x86=((IkReal(0.190250000000000))*(pz)*(x77));
IkReal x87=((IkReal(0.190250000000000))*(pz)*(x78));
gconst52=IKsign(((((IkReal(-1.00000000000000))*(x81)*(x83)))+(((IkReal(-1.00000000000000))*(x81)*(x87)))+(((IkReal(-1.00000000000000))*(x81)*(x86)))+(((x79)*(x85)))+(((IkReal(-1.00000000000000))*(x82)*(x83)))+(((IkReal(-1.00000000000000))*(x82)*(x87)))+(((IkReal(-1.00000000000000))*(x82)*(x86)))+(((IkReal(-0.0513675000000000))*(r02)*(sj1)))+(((IkReal(0.190250000000000))*(x79)*(x84)))+(((IkReal(0.190250000000000))*(x80)*(x84)))+(((x80)*(x85)))+(((IkReal(-0.0142687500000000))*(r02)*(x78)))+(((IkReal(-0.0142687500000000))*(x84)))));
IkReal x88=(sj1)*(sj1);
IkReal x89=(cj1)*(cj1);
IkReal x90=((px)*(r02));
IkReal x91=((pz)*(r00));
IkReal x92=((IkReal(13.3333333333333))*(sj0));
IkReal x93=((py)*(r02));
IkReal x94=((pz)*(r01));
IkReal x95=((IkReal(1.00000000000000))*(r02));
IkReal x96=((IkReal(3.60000000000000))*(cj1));
IkReal x97=((IkReal(13.3333333333333))*(cj0)*(x88));
IkReal x98=((IkReal(13.3333333333333))*(cj0)*(x89));
dummyeval[0]=((((x88)*(x92)*(x93)))+(((IkReal(-1.00000000000000))*(x89)*(x92)*(x94)))+(((IkReal(-1.00000000000000))*(x91)*(x98)))+(((IkReal(-1.00000000000000))*(x91)*(x97)))+(((x89)*(x92)*(x93)))+(((x90)*(x98)))+(((x90)*(x97)))+(((IkReal(-3.60000000000000))*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x88)*(x92)*(x94)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x96)))+(((IkReal(-1.00000000000000))*(x88)*(x95)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x96)))+(((IkReal(-1.00000000000000))*(x89)*(x95))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst52)*(((((IkReal(0.590000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj1)))+(((IkReal(0.159300000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.590000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(0.159300000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.159300000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(-0.000918000000000000))*(r01)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.0119475000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.295000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(-0.000918000000000000))*(cj0)*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(0.590000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.159300000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.0135000000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(-0.0486000000000000))*(py)*(r01)))+(((IkReal(0.00193050000000000))*(cj1)*(r02)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj1)))+(((IkReal(0.00364500000000000))*(cj0)*(r00)))+(((IkReal(0.00364500000000000))*(r01)*(sj0)))+(((IkReal(-0.0119475000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.0442500000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.159300000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0486000000000000))*(pz)*(r02)))+(((IkReal(0.0442500000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(0.590000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(0.590000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(-0.295000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj1)*(px)*(r00)))+(((IkReal(-0.0460200000000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.0486000000000000))*(px)*(r00)))+(((IkReal(0.159300000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.0493387500000000))*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(-0.295000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-0.0442500000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.159300000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.0135000000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj1)*(pz)*(r02)))+(((IkReal(0.0442500000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.0135000000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(-0.159300000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(-0.0460200000000000))*(cj1)*(r01)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((IkReal(0.0486000000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(0.0900000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(-0.590000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(0.0150525000000000))*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.295000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(0.0140400000000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(0.00632775000000000))*(cj1)*(r02)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.295000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(0.00364500000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.590000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(cj1)*(py)*(r01)))+(((IkReal(0.0119475000000000))*(cj0)*(r00)))+(((IkReal(0.590000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-0.0486000000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.180000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.00300900000000000))*(cj0)*(r00)*(sj1)))+(((IkReal(0.0486000000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.590000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r02)))+(((IkReal(-0.0442500000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(0.0119475000000000))*(r01)*(sj0)))+(((IkReal(-0.159300000000000))*(pz)*(r02)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.0442500000000000))*(pz)*(r02)*(sj1)))+(((IkReal(-0.159300000000000))*(py)*(r01)))+(((IkReal(0.0135000000000000))*(cj1)*(px)*(r00)))+(((IkReal(0.0442500000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.295000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.0486000000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.0140400000000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.159300000000000))*(px)*(r00)))+(((IkReal(0.590000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.0486000000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.590000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.0486000000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(0.0442500000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.0442500000000000))*(py)*(r01)*(sj1)))+(((IkReal(0.0486000000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(0.0486000000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.0442500000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(0.00364500000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.00300900000000000))*(r01)*(sj0)*(sj1))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst52)*(((((IkReal(0.590000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj1)))+(((IkReal(0.159300000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.590000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(0.159300000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.159300000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(-0.000918000000000000))*(r01)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.0119475000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.295000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(-0.000918000000000000))*(cj0)*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(0.590000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.159300000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.0135000000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(-0.0486000000000000))*(py)*(r01)))+(((IkReal(0.00193050000000000))*(cj1)*(r02)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj1)))+(((IkReal(0.00364500000000000))*(cj0)*(r00)))+(((IkReal(0.00364500000000000))*(r01)*(sj0)))+(((IkReal(-0.0119475000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.0442500000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.159300000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0486000000000000))*(pz)*(r02)))+(((IkReal(0.0442500000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(0.590000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(0.590000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(-0.295000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj1)*(px)*(r00)))+(((IkReal(-0.0460200000000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.0486000000000000))*(px)*(r00)))+(((IkReal(0.159300000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.0493387500000000))*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(-0.295000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-0.0442500000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.159300000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.0135000000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj1)*(pz)*(r02)))+(((IkReal(0.0442500000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.0135000000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(-0.159300000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(-0.0460200000000000))*(cj1)*(r01)*(sj0)))))), ((gconst52)*(((((IkReal(0.0486000000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(0.0900000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(-0.590000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(0.0150525000000000))*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.295000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(0.0140400000000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(0.00632775000000000))*(cj1)*(r02)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.295000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(0.00364500000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(-0.590000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(cj1)*(py)*(r01)))+(((IkReal(0.0119475000000000))*(cj0)*(r00)))+(((IkReal(0.590000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-0.0486000000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.180000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.00300900000000000))*(cj0)*(r00)*(sj1)))+(((IkReal(0.0486000000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.590000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r02)))+(((IkReal(-0.0442500000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(0.0119475000000000))*(r01)*(sj0)))+(((IkReal(-0.159300000000000))*(pz)*(r02)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.0442500000000000))*(pz)*(r02)*(sj1)))+(((IkReal(-0.159300000000000))*(py)*(r01)))+(((IkReal(0.0135000000000000))*(cj1)*(px)*(r00)))+(((IkReal(0.0442500000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.295000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.0486000000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.0140400000000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.159300000000000))*(px)*(r00)))+(((IkReal(0.590000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.0486000000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.590000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.0486000000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(0.0442500000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.0442500000000000))*(py)*(r01)*(sj1)))+(((IkReal(0.0486000000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(0.0486000000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.0442500000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.0442500000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(0.00364500000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.00300900000000000))*(r01)*(sj0)*(sj1)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
IkReal x99=IKsin(j2);
IkReal x100=IKcos(j2);
IkReal x101=((cj0)*(px));
IkReal x102=((cj1)*(pz));
IkReal x103=((py)*(sj0));
IkReal x104=((r01)*(sj0));
IkReal x105=((IkReal(0.590000000000000))*(pz));
IkReal x106=((cj1)*(r02));
IkReal x107=((IkReal(0.295000000000000))*(r02));
IkReal x108=((IkReal(0.540000000000000))*(sj1));
IkReal x109=((cj0)*(r00));
IkReal x110=((sj1)*(x109));
IkReal x111=((IkReal(0.0900000000000000))*(x100));
IkReal x112=((cj1)*(x99));
IkReal x113=((sj1)*(x99));
IkReal x114=((sj1)*(x100));
IkReal x115=((cj1)*(x100));
evalcond[0]=((IkReal(0.167250000000000))+(((IkReal(-0.590000000000000))*(x102)*(x99)))+(((IkReal(0.0405000000000000))*(sj1)))+(((IkReal(-0.0442500000000000))*(x113)))+(((IkReal(0.590000000000000))*(x103)*(x113)))+(((IkReal(-1.00000000000000))*(x101)*(x108)))+(((IkReal(0.590000000000000))*(x101)*(x113)))+(((IkReal(-0.180000000000000))*(x103)*(x114)))+(((IkReal(-0.180000000000000))*(x103)*(x112)))+(((IkReal(0.180000000000000))*(x100)*(x102)))+(((IkReal(-0.180000000000000))*(x101)*(x112)))+(((IkReal(-0.180000000000000))*(x101)*(x114)))+(((IkReal(-0.590000000000000))*(x101)*(x115)))+(((IkReal(0.0442500000000000))*(x115)))+(((IkReal(-0.159300000000000))*(x99)))+(((IkReal(0.0135000000000000))*(x114)))+(((IkReal(0.0135000000000000))*(x112)))+(pp)+(((IkReal(0.0486000000000000))*(x100)))+(((IkReal(-0.150000000000000))*(x103)))+(((IkReal(-0.150000000000000))*(x101)))+(((IkReal(-0.590000000000000))*(x103)*(x115)))+(((IkReal(-1.00000000000000))*(x103)*(x108)))+(((IkReal(-1.00000000000000))*(x105)*(x114)))+(((IkReal(0.540000000000000))*(x102)))+(((IkReal(-0.180000000000000))*(pz)*(x113))));
evalcond[1]=((((IkReal(0.295000000000000))*(x110)*(x99)))+(((IkReal(-0.0900000000000000))*(x104)*(x112)))+(((IkReal(-0.295000000000000))*(x104)*(x115)))+(((IkReal(-0.0750000000000000))*(x109)))+(((IkReal(-0.0750000000000000))*(x104)))+(((x106)*(x111)))+(((px)*(r00)))+(((IkReal(-0.270000000000000))*(x110)))+(((IkReal(-1.00000000000000))*(x107)*(x114)))+(((IkReal(0.270000000000000))*(x106)))+(((IkReal(0.295000000000000))*(x104)*(x113)))+(((IkReal(-1.00000000000000))*(x110)*(x111)))+(((IkReal(-0.0900000000000000))*(r02)*(x113)))+(((IkReal(-1.00000000000000))*(sj1)*(x104)*(x111)))+(((IkReal(-0.295000000000000))*(x109)*(x115)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.270000000000000))*(sj1)*(x104)))+(((IkReal(-0.0900000000000000))*(x109)*(x112)))+(((IkReal(-0.295000000000000))*(x106)*(x99))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x116=((cj1)*(cj2));
IkReal x117=((sj1)*(sj2));
IkReal x118=((IkReal(12.5000000000000))*(pz));
IkReal x119=((IkReal(12.5000000000000))*(cj0)*(px));
IkReal x120=((IkReal(12.5000000000000))*(py)*(sj0));
cj4array[0]=((IkReal(-3.68750000000000))+(((IkReal(-0.937500000000000))*(x116)))+(((IkReal(-1.00000000000000))*(x117)*(x119)))+(((IkReal(3.37500000000000))*(sj2)))+(((IkReal(0.937500000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x117)*(x120)))+(((cj2)*(sj1)*(x118)))+(((cj1)*(sj2)*(x118)))+(((x116)*(x120)))+(((x116)*(x119))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x121=((pz)*(sj2));
IkReal x122=((IkReal(1.00000000000000))*(cj1));
IkReal x123=((cj2)*(sj0));
IkReal x124=((py)*(sj1));
IkReal x125=((cj0)*(sj1));
IkReal x126=((cj2)*(px));
IkReal x127=((sj0)*(sj2));
IkReal x128=((IkReal(0.0750000000000000))*(cj2));
IkReal x129=((px)*(sj2));
IkReal x130=((IkReal(0.0750000000000000))*(sj2));
IkReal x131=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x127)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x122)))+(((r00)*(sj2)*(x125)))+(((IkReal(-1.00000000000000))*(r02)*(x131)))+(((IkReal(-1.00000000000000))*(r01)*(x122)*(x123)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x122))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(x128)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x129)))+(((x125)*(x126)))+(((x123)*(x124)))+(((cj1)*(py)*(x127)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x122)))+(((IkReal(-1.00000000000000))*(cj1)*(x130)))+(((sj1)*(x121))));
evalcond[4]=((IkReal(0.375000000000000))+(((cj1)*(x128)))+(((IkReal(-1.00000000000000))*(pz)*(x131)))+(((IkReal(-1.00000000000000))*(sj1)*(x130)))+(((x125)*(x129)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x122)*(x126)))+(((x124)*(x127)))+(((IkReal(-1.00000000000000))*(x121)*(x122)))+(((IkReal(-1.00000000000000))*(py)*(x122)*(x123))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst63;
gconst63=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst64;
gconst64=IkReal(-2.00000000000000);
IkReal gconst65;
gconst65=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst66;
IkReal x132=((cj1)*(sj2));
IkReal x133=((r01)*(sj0));
IkReal x134=((cj0)*(r00));
IkReal x135=((cj2)*(sj1));
gconst66=((IkReal(-1.00000000000000))+(((x134)*(x135)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x133)*(x135)))+(((x132)*(x133)))+(((x132)*(x134)))+(((r02)*(sj1)*(sj2))));
IkReal gconst67;
IkReal x136=((cj1)*(sj2));
IkReal x137=((r01)*(sj0));
IkReal x138=((cj0)*(r00));
IkReal x139=((cj2)*(sj1));
gconst67=((IkReal(1.00000000000000))+(((x138)*(x139)))+(((x137)*(x139)))+(((x136)*(x138)))+(((x136)*(x137)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal gconst68;
gconst68=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst69;
gconst69=IkReal(-2.00000000000000);
IkReal gconst70;
gconst70=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst71;
IkReal x140=((cj1)*(sj2));
IkReal x141=((r01)*(sj0));
IkReal x142=((cj0)*(r00));
IkReal x143=((cj2)*(sj1));
gconst71=((IkReal(-1.00000000000000))+(((x140)*(x142)))+(((x140)*(x141)))+(((x142)*(x143)))+(((x141)*(x143)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal gconst72;
IkReal x144=((cj1)*(sj2));
IkReal x145=((r01)*(sj0));
IkReal x146=((cj0)*(r00));
IkReal x147=((cj2)*(sj1));
gconst72=((IkReal(1.00000000000000))+(((x144)*(x145)))+(((x144)*(x146)))+(((x145)*(x147)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x146)*(x147)))+(((r02)*(sj1)*(sj2))));
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(4.00000000000000);
IkReal gconst76;
gconst76=IkReal(-2.00000000000000);
IkReal gconst77;
gconst77=IkReal(2.00000000000000);
IkReal gconst78;
gconst78=IkReal(4.00000000000000);
IkReal gconst79;
gconst79=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst80;
gconst80=IkReal(2.00000000000000);
IkReal gconst81;
gconst81=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst82;
IkReal x148=((cj1)*(sj2));
IkReal x149=((r01)*(sj0));
IkReal x150=((cj0)*(r00));
IkReal x151=((cj2)*(sj1));
gconst82=((IkReal(1.00000000000000))+(((x149)*(x151)))+(((x150)*(x151)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x148)*(x149)))+(((x148)*(x150)))+(((r02)*(sj1)*(sj2))));
IkReal gconst83;
IkReal x152=((cj1)*(sj2));
IkReal x153=((r01)*(sj0));
IkReal x154=((cj0)*(r00));
IkReal x155=((cj2)*(sj1));
gconst83=((IkReal(-1.00000000000000))+(((x153)*(x155)))+(((x152)*(x154)))+(((x152)*(x153)))+(((x154)*(x155)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal gconst84;
gconst84=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst85;
gconst85=IkReal(2.00000000000000);
IkReal gconst86;
gconst86=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst87;
IkReal x156=((cj1)*(sj2));
IkReal x157=((r01)*(sj0));
IkReal x158=((cj0)*(r00));
IkReal x159=((cj2)*(sj1));
gconst87=((IkReal(1.00000000000000))+(((x157)*(x159)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x156)*(x158)))+(((x156)*(x157)))+(((x158)*(x159)))+(((r02)*(sj1)*(sj2))));
IkReal gconst88;
IkReal x160=((cj1)*(sj2));
IkReal x161=((r01)*(sj0));
IkReal x162=((cj0)*(r00));
IkReal x163=((cj2)*(sj1));
gconst88=((IkReal(-1.00000000000000))+(((x161)*(x163)))+(((x162)*(x163)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x160)*(x162)))+(((x160)*(x161))));
IkReal x164=((IkReal(1.00000000000000))*(gconst87));
IkReal x165=((gconst81)*(gconst82));
IkReal x166=((gconst84)*(gconst88));
IkReal x167=((gconst79)*(gconst83));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x166)*(x167)))+(((IkReal(-1.00000000000000))*(gconst86)*(x164)*(x165)))+(((IkReal(-1.00000000000000))*(gconst80)*(gconst83)*(gconst85)*(x164)))+(((gconst86)*(gconst87)*(x167)))+(((x165)*(x166))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x168=((IkReal(1.00000000000000))*(gconst87));
IkReal x169=((gconst66)*(gconst70));
IkReal x170=((gconst71)*(gconst83));
IkReal x171=((gconst80)*(gconst85));
IkReal x172=((gconst77)*(gconst79));
IkReal x173=((gconst67)*(gconst79));
IkReal x174=((gconst63)*(gconst83));
IkReal x175=((gconst72)*(gconst84));
IkReal x176=((gconst65)*(gconst66));
IkReal x177=((gconst73)*(gconst83));
IkReal x178=((gconst77)*(gconst87));
IkReal x179=((gconst74)*(gconst82));
IkReal x180=((gconst68)*(gconst88));
IkReal x181=((gconst63)*(gconst77));
IkReal x182=((gconst72)*(gconst76));
IkReal x183=((gconst78)*(gconst84));
IkReal x184=((gconst80)*(gconst83));
IkReal x185=((IkReal(1.00000000000000))*(gconst67));
IkReal x186=((gconst64)*(gconst85));
IkReal x187=((gconst64)*(gconst69));
IkReal x188=((gconst70)*(gconst87));
IkReal x189=((gconst65)*(gconst75));
IkReal x190=((gconst69)*(gconst87));
IkReal x191=((gconst71)*(gconst86));
IkReal x192=((gconst84)*(gconst88));
IkReal x193=((IkReal(1.00000000000000))*(gconst81));
IkReal x194=((gconst81)*(gconst82));
IkReal x195=((gconst76)*(gconst88));
IkReal x196=((gconst65)*(gconst82));
IkReal x197=((gconst70)*(gconst71));
IkReal x198=((gconst85)*(gconst87));
IkReal x199=((gconst79)*(gconst83));
IkReal x200=((gconst68)*(gconst78));
IkReal x201=((gconst64)*(gconst83));
IkReal x202=((gconst66)*(gconst74));
IkReal x203=((gconst76)*(gconst78));
IkReal x204=((gconst67)*(gconst80));
IkReal x205=((gconst67)*(gconst87));
IkReal x206=((gconst73)*(gconst86));
IkReal x207=((gconst75)*(gconst81));
IkReal x208=((gconst66)*(gconst81));
IkReal x209=((gconst71)*(gconst85));
IkReal x210=((gconst63)*(gconst86));
IkReal x211=((gconst67)*(gconst73));
IkReal x212=((gconst64)*(gconst67));
IkReal x213=((gconst69)*(gconst71));
IkReal x214=((gconst74)*(gconst75));
IkReal x215=((gconst63)*(gconst67));
IkReal x216=((gconst86)*(gconst87));
IkReal x217=((IkReal(1.00000000000000))*(gconst71));
IkReal x218=((gconst68)*(gconst72));
IkReal x219=((gconst71)*(gconst77));
IkReal x220=((gconst67)*(gconst71));
IkReal x221=((IkReal(1.00000000000000))*(x218));
IkReal x222=((IkReal(1.00000000000000))*(x199));
IkReal x223=((gconst82)*(x218));
IkReal x224=((gconst77)*(x217));
op[0]=((((IkReal(-1.00000000000000))*(x192)*(x222)))+(((x199)*(x216)))+(((IkReal(-1.00000000000000))*(gconst86)*(x168)*(x194)))+(((x192)*(x194)))+(((IkReal(-1.00000000000000))*(gconst83)*(x168)*(x171))));
op[1]=((((x198)*(x207)))+(((gconst83)*(gconst87)*(x172)))+(((x179)*(x192)))+(((x177)*(x216)))+(((IkReal(-1.00000000000000))*(x195)*(x222)))+(((IkReal(-1.00000000000000))*(gconst86)*(x168)*(x179)))+(((x194)*(x195)))+(((IkReal(-1.00000000000000))*(x177)*(x192)))+(((IkReal(-1.00000000000000))*(gconst77)*(x168)*(x194)))+(((x183)*(x184))));
op[2]=((((IkReal(-1.00000000000000))*(x170)*(x171)))+(((IkReal(-1.00000000000000))*(gconst70)*(x168)*(x194)))+(((IkReal(-1.00000000000000))*(gconst77)*(x168)*(x179)))+(((x179)*(x195)))+(((x180)*(x194)))+(((IkReal(-1.00000000000000))*(gconst86)*(x168)*(x196)))+(((IkReal(-1.00000000000000))*(x180)*(x222)))+(((x192)*(x208)))+(((x188)*(x199)))+(((x175)*(x194)))+(((IkReal(-1.00000000000000))*(x174)*(x192)))+(((IkReal(-1.00000000000000))*(gconst69)*(x168)*(x184)))+(((x198)*(x214)))+(((x184)*(x203)))+(((IkReal(-1.00000000000000))*(gconst82)*(x191)*(x193)))+(((x173)*(x216)))+(((x192)*(x196)))+(((IkReal(-1.00000000000000))*(x173)*(x192)))+(((gconst79)*(gconst86)*(x170)))+(((IkReal(-1.00000000000000))*(gconst75)*(x183)*(x193)))+(((IkReal(-1.00000000000000))*(x175)*(x222)))+(((x174)*(x216)))+(((IkReal(-1.00000000000000))*(gconst83)*(x168)*(x186)))+(((IkReal(-1.00000000000000))*(x177)*(x195)))+(((IkReal(-1.00000000000000))*(gconst67)*(x168)*(x171)))+(((IkReal(-1.00000000000000))*(gconst86)*(x168)*(x208)))+(((x177)*(x178))));
op[3]=((((x177)*(x188)))+(((x205)*(x206)))+(((IkReal(-1.00000000000000))*(gconst73)*(x185)*(x192)))+(((x192)*(x202)))+(((x195)*(x208)))+(((x179)*(x180)))+(((x170)*(x206)))+(((IkReal(-1.00000000000000))*(gconst77)*(x168)*(x208)))+(((x190)*(x207)))+(((IkReal(-1.00000000000000))*(x174)*(x195)))+(((x189)*(x198)))+(((x175)*(x179)))+(((x184)*(x200)))+(((x174)*(x178)))+(((x195)*(x196)))+(((IkReal(-1.00000000000000))*(x182)*(x222)))+(((IkReal(-1.00000000000000))*(x173)*(x195)))+(((IkReal(-1.00000000000000))*(gconst75)*(x193)*(x203)))+(((IkReal(-1.00000000000000))*(x183)*(x214)))+(((x170)*(x172)))+(((IkReal(-1.00000000000000))*(x175)*(x177)))+(((x183)*(x204)))+(((x183)*(x201)))+(((x172)*(x205)))+(((IkReal(-1.00000000000000))*(gconst77)*(x168)*(x196)))+(((x182)*(x194)))+(((IkReal(-1.00000000000000))*(x177)*(x180)))+(((IkReal(-1.00000000000000))*(gconst70)*(x168)*(x179)))+(((IkReal(-1.00000000000000))*(gconst86)*(x168)*(x202)))+(((x207)*(x209)))+(((IkReal(-1.00000000000000))*(x179)*(x191)))+(((IkReal(-1.00000000000000))*(gconst82)*(x193)*(x219))));
op[4]=((((x176)*(x192)))+(((x190)*(x214)))+(((x175)*(x208)))+(((IkReal(-1.00000000000000))*(x191)*(x196)))+(((IkReal(-1.00000000000000))*(gconst69)*(x168)*(x204)))+(((IkReal(-1.00000000000000))*(x170)*(x186)))+(((IkReal(-1.00000000000000))*(gconst67)*(x168)*(x186)))+(((x180)*(x208)))+(((IkReal(-1.00000000000000))*(gconst70)*(x168)*(x196)))+(((IkReal(-1.00000000000000))*(gconst73)*(x185)*(x195)))+(((x173)*(x188)))+(((x180)*(x196)))+(((IkReal(-1.00000000000000))*(gconst71)*(x171)*(x185)))+(((x173)*(x191)))+(((x195)*(x202)))+(((x179)*(x182)))+(((gconst70)*(gconst79)*(x170)))+(((x201)*(x203)))+(((IkReal(-1.00000000000000))*(gconst77)*(x168)*(x202)))+(((x174)*(x188)))+(((x175)*(x196)))+(((IkReal(-1.00000000000000))*(x179)*(x224)))+(((x205)*(x210)))+(((x209)*(x214)))+(((IkReal(-1.00000000000000))*(x174)*(x180)))+(((IkReal(-1.00000000000000))*(gconst63)*(x185)*(x192)))+(((gconst73)*(gconst77)*(x170)))+(((IkReal(-1.00000000000000))*(x199)*(x221)))+(((IkReal(-1.00000000000000))*(gconst86)*(x168)*(x176)))+(((IkReal(-1.00000000000000))*(gconst81)*(x168)*(x169)))+(((IkReal(-1.00000000000000))*(gconst75)*(x193)*(x200)))+(((IkReal(-1.00000000000000))*(gconst82)*(x193)*(x197)))+(((x170)*(x210)))+(((IkReal(-1.00000000000000))*(x174)*(x175)))+(((IkReal(-1.00000000000000))*(gconst66)*(x191)*(x193)))+(((IkReal(-1.00000000000000))*(x183)*(x189)))+(((IkReal(-1.00000000000000))*(x173)*(x180)))+(((IkReal(-1.00000000000000))*(x173)*(x175)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst80)*(x170)))+(((x203)*(x204)))+(((IkReal(-1.00000000000000))*(gconst83)*(x168)*(x187)))+(((x194)*(x218)))+(((IkReal(-1.00000000000000))*(x203)*(x214)))+(((IkReal(-1.00000000000000))*(x177)*(x182)))+(((x178)*(x211))));
op[5]=((((x176)*(x195)))+(((x175)*(x202)))+(((x179)*(x218)))+(((IkReal(-1.00000000000000))*(gconst66)*(x193)*(x219)))+(((x191)*(x211)))+(((x180)*(x202)))+(((IkReal(-1.00000000000000))*(gconst77)*(x168)*(x176)))+(((x200)*(x204)))+(((x200)*(x201)))+(((x183)*(x212)))+(((x207)*(x213)))+(((x189)*(x209)))+(((x172)*(x220)))+(((IkReal(-1.00000000000000))*(gconst73)*(x180)*(x185)))+(((x189)*(x190)))+(((gconst70)*(gconst73)*(x170)))+(((IkReal(-1.00000000000000))*(x196)*(x224)))+(((IkReal(-1.00000000000000))*(x174)*(x182)))+(((IkReal(-1.00000000000000))*(gconst63)*(x185)*(x195)))+(((IkReal(-1.00000000000000))*(x191)*(x202)))+(((IkReal(-1.00000000000000))*(gconst74)*(x168)*(x169)))+(((IkReal(-1.00000000000000))*(x173)*(x182)))+(((IkReal(-1.00000000000000))*(x189)*(x203)))+(((IkReal(-1.00000000000000))*(x177)*(x221)))+(((IkReal(-1.00000000000000))*(x200)*(x214)))+(((x188)*(x211)))+(((IkReal(-1.00000000000000))*(gconst73)*(x175)*(x185)))+(((x170)*(x181)))+(((x182)*(x208)))+(((x182)*(x196)))+(((x178)*(x215)))+(((IkReal(-1.00000000000000))*(x179)*(x197))));
op[6]=((((x191)*(x215)))+(((x176)*(x180)))+(((IkReal(-1.00000000000000))*(x170)*(x187)))+(((IkReal(-1.00000000000000))*(gconst67)*(x168)*(x187)))+(((x213)*(x214)))+(((IkReal(-1.00000000000000))*(gconst63)*(x180)*(x185)))+(((x173)*(x197)))+(((IkReal(-1.00000000000000))*(gconst71)*(x169)*(x193)))+(((x203)*(x212)))+(((IkReal(-1.00000000000000))*(x196)*(x197)))+(((x208)*(x218)))+(((x211)*(x219)))+(((IkReal(-1.00000000000000))*(gconst80)*(x185)*(x213)))+(((x175)*(x176)))+(((IkReal(-1.00000000000000))*(gconst65)*(x168)*(x169)))+(((IkReal(-1.00000000000000))*(gconst71)*(x185)*(x186)))+(((gconst63)*(gconst70)*(x170)))+(((x196)*(x218)))+(((IkReal(-1.00000000000000))*(gconst63)*(x175)*(x185)))+(((IkReal(-1.00000000000000))*(x202)*(x224)))+(((IkReal(-1.00000000000000))*(x174)*(x221)))+(((IkReal(-1.00000000000000))*(x189)*(x200)))+(((IkReal(-1.00000000000000))*(x176)*(x191)))+(((IkReal(-1.00000000000000))*(x173)*(x221)))+(((x188)*(x215)))+(((x182)*(x202)))+(((IkReal(-1.00000000000000))*(gconst73)*(x182)*(x185))));
op[7]=((((x176)*(x182)))+(((x197)*(x211)))+(((x202)*(x218)))+(((IkReal(-1.00000000000000))*(gconst74)*(x169)*(x217)))+(((IkReal(-1.00000000000000))*(gconst73)*(x185)*(x218)))+(((x189)*(x213)))+(((x181)*(x220)))+(((IkReal(-1.00000000000000))*(gconst63)*(x182)*(x185)))+(((IkReal(-1.00000000000000))*(x176)*(x224)))+(((x200)*(x212))));
op[8]=((((x197)*(x215)))+(((x176)*(x218)))+(((IkReal(-1.00000000000000))*(gconst71)*(x185)*(x187)))+(((IkReal(-1.00000000000000))*(gconst65)*(x169)*(x217)))+(((IkReal(-1.00000000000000))*(gconst63)*(x185)*(x218))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst105;
gconst105=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x225=((cj3)*(sj1));
IkReal x226=((r02)*(sj2));
IkReal x227=((r01)*(sj0));
IkReal x228=((IkReal(1.00000000000000))*(cj3));
IkReal x229=((cj1)*(sj2));
IkReal x230=((IkReal(1.00000000000000))*(sj3));
IkReal x231=((cj0)*(r00));
IkReal x232=((cj0)*(r01));
IkReal x233=((r00)*(sj0));
IkReal x234=((cj2)*(x231));
IkReal x235=((sj1)*(x230));
IkReal x236=((cj1)*(cj2)*(r02));
if( IKabs(((gconst105)*(((((IkReal(-1.00000000000000))*(x229)*(x230)*(x231)))+(((cj3)*(x233)))+(((IkReal(-1.00000000000000))*(x226)*(x235)))+(((IkReal(-1.00000000000000))*(x227)*(x229)*(x230)))+(((IkReal(-1.00000000000000))*(x234)*(x235)))+(((IkReal(-1.00000000000000))*(x228)*(x232)))+(((IkReal(-1.00000000000000))*(cj2)*(x227)*(x235)))+(((sj3)*(x236))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst105)*(((((x225)*(x226)))+(((IkReal(-1.00000000000000))*(x230)*(x232)))+(((cj2)*(x225)*(x227)))+(((x225)*(x234)))+(((IkReal(-1.00000000000000))*(x228)*(x236)))+(((cj3)*(x227)*(x229)))+(((cj3)*(x229)*(x231)))+(((sj3)*(x233))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst105)*(((((IkReal(-1.00000000000000))*(x229)*(x230)*(x231)))+(((cj3)*(x233)))+(((IkReal(-1.00000000000000))*(x226)*(x235)))+(((IkReal(-1.00000000000000))*(x227)*(x229)*(x230)))+(((IkReal(-1.00000000000000))*(x234)*(x235)))+(((IkReal(-1.00000000000000))*(x228)*(x232)))+(((IkReal(-1.00000000000000))*(cj2)*(x227)*(x235)))+(((sj3)*(x236)))))), ((gconst105)*(((((x225)*(x226)))+(((IkReal(-1.00000000000000))*(x230)*(x232)))+(((cj2)*(x225)*(x227)))+(((x225)*(x234)))+(((IkReal(-1.00000000000000))*(x228)*(x236)))+(((cj3)*(x227)*(x229)))+(((cj3)*(x229)*(x231)))+(((sj3)*(x233)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x237=IKsin(j5);
IkReal x238=IKcos(j5);
IkReal x239=((IkReal(1.00000000000000))*(cj3));
IkReal x240=((cj1)*(sj2));
IkReal x241=((r01)*(sj0));
IkReal x242=((cj0)*(r00));
IkReal x243=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(sj3)*(x238)))+(((IkReal(-1.00000000000000))*(x237)*(x239))));
evalcond[1]=((((x240)*(x242)))+(((x240)*(x241)))+(((IkReal(-1.00000000000000))*(x238)*(x239)))+(((x241)*(x243)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x242)*(x243)))+(((r02)*(sj1)*(sj2)))+(((sj3)*(x237))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x244=((pz)*(sj2));
IkReal x245=((IkReal(1.00000000000000))*(cj1));
IkReal x246=((cj2)*(sj0));
IkReal x247=((py)*(sj1));
IkReal x248=((cj0)*(sj1));
IkReal x249=((cj2)*(px));
IkReal x250=((sj0)*(sj2));
IkReal x251=((IkReal(0.0750000000000000))*(cj2));
IkReal x252=((px)*(sj2));
IkReal x253=((IkReal(0.0750000000000000))*(sj2));
IkReal x254=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x254)))+(((r00)*(sj2)*(x248)))+(((IkReal(-1.00000000000000))*(r01)*(x245)*(x246)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x245)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x245)))+(((r01)*(sj1)*(x250))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x248)*(x249)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj1)*(py)*(x250)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x245)))+(((IkReal(-1.00000000000000))*(cj1)*(x253)))+(((IkReal(-1.00000000000000))*(sj1)*(x251)))+(((cj0)*(cj1)*(x252)))+(((x246)*(x247)))+(((sj1)*(x244))));
evalcond[4]=((IkReal(0.215000000000000))+(((x248)*(x252)))+(((cj1)*(x251)))+(((IkReal(-1.00000000000000))*(py)*(x245)*(x246)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x245)*(x249)))+(((IkReal(-1.00000000000000))*(pz)*(x254)))+(((IkReal(-1.00000000000000))*(sj1)*(x253)))+(((IkReal(-1.00000000000000))*(x244)*(x245)))+(((x247)*(x250))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst106;
gconst106=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst107;
gconst107=IkReal(-2.00000000000000);
IkReal gconst108;
gconst108=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst109;
IkReal x255=((cj1)*(sj2));
IkReal x256=((r01)*(sj0));
IkReal x257=((cj0)*(r00));
IkReal x258=((cj2)*(sj1));
gconst109=((IkReal(1.00000000000000))+(((x256)*(x258)))+(((x257)*(x258)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x255)*(x256)))+(((x255)*(x257)))+(((r02)*(sj1)*(sj2))));
IkReal gconst110;
IkReal x259=((cj1)*(sj2));
IkReal x260=((r01)*(sj0));
IkReal x261=((cj0)*(r00));
IkReal x262=((cj2)*(sj1));
gconst110=((IkReal(-1.00000000000000))+(((x259)*(x260)))+(((x259)*(x261)))+(((x260)*(x262)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x261)*(x262)))+(((r02)*(sj1)*(sj2))));
IkReal gconst111;
gconst111=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst112;
gconst112=IkReal(-2.00000000000000);
IkReal gconst113;
gconst113=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst114;
IkReal x263=((cj1)*(sj2));
IkReal x264=((r01)*(sj0));
IkReal x265=((cj0)*(r00));
IkReal x266=((cj2)*(sj1));
gconst114=((IkReal(1.00000000000000))+(((x264)*(x266)))+(((x263)*(x265)))+(((x263)*(x264)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x265)*(x266))));
IkReal gconst115;
IkReal x267=((cj1)*(sj2));
IkReal x268=((r01)*(sj0));
IkReal x269=((cj0)*(r00));
IkReal x270=((cj2)*(sj1));
gconst115=((IkReal(-1.00000000000000))+(((x267)*(x268)))+(((x267)*(x269)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x269)*(x270)))+(((r02)*(sj1)*(sj2)))+(((x268)*(x270))));
IkReal gconst116;
gconst116=IkReal(2.00000000000000);
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=IkReal(4.00000000000000);
IkReal gconst119;
gconst119=IkReal(2.00000000000000);
IkReal gconst120;
gconst120=IkReal(-2.00000000000000);
IkReal gconst121;
gconst121=IkReal(4.00000000000000);
IkReal gconst122;
gconst122=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst123;
gconst123=IkReal(2.00000000000000);
IkReal gconst124;
gconst124=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst125;
IkReal x271=((cj1)*(sj2));
IkReal x272=((r01)*(sj0));
IkReal x273=((cj0)*(r00));
IkReal x274=((cj2)*(sj1));
gconst125=((IkReal(-1.00000000000000))+(((x272)*(x274)))+(((x273)*(x274)))+(((x271)*(x273)))+(((x271)*(x272)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
IkReal gconst126;
IkReal x275=((cj1)*(sj2));
IkReal x276=((r01)*(sj0));
IkReal x277=((cj0)*(r00));
IkReal x278=((cj2)*(sj1));
gconst126=((IkReal(1.00000000000000))+(((x275)*(x277)))+(((x275)*(x276)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x277)*(x278)))+(((x276)*(x278)))+(((r02)*(sj1)*(sj2))));
IkReal gconst127;
gconst127=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst128;
gconst128=IkReal(2.00000000000000);
IkReal gconst129;
gconst129=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst130;
IkReal x279=((cj1)*(sj2));
IkReal x280=((r01)*(sj0));
IkReal x281=((cj0)*(r00));
IkReal x282=((cj2)*(sj1));
gconst130=((IkReal(-1.00000000000000))+(((x279)*(x281)))+(((x279)*(x280)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x280)*(x282)))+(((r02)*(sj1)*(sj2)))+(((x281)*(x282))));
IkReal gconst131;
IkReal x283=((cj1)*(sj2));
IkReal x284=((r01)*(sj0));
IkReal x285=((cj0)*(r00));
IkReal x286=((cj2)*(sj1));
gconst131=((IkReal(1.00000000000000))+(((x284)*(x286)))+(((x285)*(x286)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x283)*(x284)))+(((x283)*(x285))));
IkReal x287=((gconst129)*(gconst130));
IkReal x288=((gconst122)*(gconst126));
IkReal x289=((gconst124)*(gconst125));
IkReal x290=((gconst127)*(gconst131));
dummyeval[0]=((((x289)*(x290)))+(((IkReal(-1.00000000000000))*(x288)*(x290)))+(((IkReal(-1.00000000000000))*(x287)*(x289)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst126)*(gconst128)*(gconst130)))+(((x287)*(x288))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x291=((IkReal(1.00000000000000))*(gconst127));
IkReal x292=((gconst106)*(gconst110));
IkReal x293=((gconst118)*(gconst124));
IkReal x294=((gconst109)*(gconst115));
IkReal x295=((gconst119)*(gconst124));
IkReal x296=((gconst110)*(gconst116));
IkReal x297=((gconst111)*(gconst131));
IkReal x298=((IkReal(1.00000000000000))*(gconst130));
IkReal x299=((gconst123)*(gconst126));
IkReal x300=((gconst111)*(gconst117));
IkReal x301=((gconst108)*(gconst119));
IkReal x302=((gconst114)*(gconst120));
IkReal x303=((IkReal(1.00000000000000))*(gconst109));
IkReal x304=((gconst125)*(gconst129));
IkReal x305=((gconst109)*(gconst113));
IkReal x306=((gconst110)*(gconst122));
IkReal x307=((gconst109)*(gconst131));
IkReal x308=((gconst108)*(gconst127));
IkReal x309=((gconst120)*(gconst130));
IkReal x310=((gconst106)*(gconst126));
IkReal x311=((gconst124)*(gconst125));
IkReal x312=((gconst110)*(gconst112));
IkReal x313=((gconst113)*(gconst130));
IkReal x314=((gconst122)*(gconst126));
IkReal x315=((gconst128)*(gconst130));
IkReal x316=((gconst111)*(gconst121));
IkReal x317=((gconst118)*(gconst121));
IkReal x318=((gconst129)*(gconst130));
IkReal x319=((gconst124)*(gconst127));
IkReal x320=((gconst108)*(gconst125));
IkReal x321=((gconst126)*(gconst128));
IkReal x322=((gconst108)*(gconst118));
IkReal x323=((gconst112)*(gconst130));
IkReal x324=((gconst114)*(gconst128));
IkReal x325=((gconst125)*(gconst131));
IkReal x326=((IkReal(1.00000000000000))*(gconst115));
IkReal x327=((gconst114)*(gconst129));
IkReal x328=((gconst113)*(gconst114));
IkReal x329=((gconst112)*(gconst114));
IkReal x330=((gconst110)*(gconst123));
IkReal x331=((gconst121)*(gconst127));
IkReal x332=((gconst115)*(gconst125));
IkReal x333=((gconst116)*(gconst126));
IkReal x334=((gconst117)*(gconst119));
IkReal x335=((gconst117)*(gconst125));
IkReal x336=((gconst109)*(gconst117));
IkReal x337=((gconst112)*(gconst126));
IkReal x338=((gconst109)*(gconst124));
IkReal x339=((IkReal(1.00000000000000))*(gconst114));
IkReal x340=((gconst117)*(gconst118));
IkReal x341=((gconst111)*(gconst115));
IkReal x342=((gconst107)*(gconst126));
IkReal x343=((gconst107)*(gconst110));
IkReal x344=((gconst117)*(gconst127));
IkReal x345=((gconst119)*(gconst121));
IkReal x346=((gconst108)*(gconst109));
IkReal x347=((gconst107)*(x339));
IkReal x348=((IkReal(1.00000000000000))*(gconst119)*(gconst131));
op[0]=((((gconst127)*(gconst131)*(x311)))+(((IkReal(-1.00000000000000))*(gconst128)*(x298)*(x299)))+(((IkReal(-1.00000000000000))*(gconst124)*(x298)*(x304)))+(((IkReal(-1.00000000000000))*(gconst131)*(x291)*(x314)))+(((x314)*(x318))));
op[1]=((((x293)*(x315)))+(((x325)*(x344)))+(((IkReal(-1.00000000000000))*(x314)*(x348)))+(((x318)*(x333)))+(((IkReal(-1.00000000000000))*(gconst131)*(x291)*(x333)))+(((x309)*(x314)))+(((x299)*(x331)))+(((IkReal(-1.00000000000000))*(gconst117)*(x298)*(x304)))+(((IkReal(-1.00000000000000))*(gconst120)*(x298)*(x311)))+(((x295)*(x325))));
op[2]=((((IkReal(-1.00000000000000))*(gconst115)*(x291)*(x314)))+(((x310)*(x318)))+(((IkReal(-1.00000000000000))*(x297)*(x314)))+(((gconst115)*(gconst127)*(x311)))+(((IkReal(-1.00000000000000))*(gconst124)*(x304)*(x339)))+(((IkReal(-1.00000000000000))*(x299)*(x324)))+(((IkReal(-1.00000000000000))*(gconst131)*(x291)*(x306)))+(((x308)*(x325)))+(((IkReal(-1.00000000000000))*(gconst121)*(x291)*(x293)))+(((x297)*(x311)))+(((IkReal(-1.00000000000000))*(gconst113)*(x298)*(x311)))+(((IkReal(-1.00000000000000))*(gconst129)*(x298)*(x338)))+(((x315)*(x340)))+(((x299)*(x345)))+(((x314)*(x327)))+(((x307)*(x319)))+(((x306)*(x318)))+(((IkReal(-1.00000000000000))*(gconst108)*(x298)*(x304)))+(((x325)*(x334)))+(((IkReal(-1.00000000000000))*(gconst131)*(x291)*(x310)))+(((IkReal(-1.00000000000000))*(x333)*(x348)))+(((IkReal(-1.00000000000000))*(gconst112)*(x298)*(x299)))+(((IkReal(-1.00000000000000))*(gconst107)*(x298)*(x321)))+(((IkReal(-1.00000000000000))*(gconst128)*(x298)*(x330)))+(((x309)*(x333)))+(((IkReal(-1.00000000000000))*(gconst120)*(x298)*(x335)))+(((x313)*(x314))));
op[3]=((((IkReal(-1.00000000000000))*(gconst119)*(x314)*(x326)))+(((IkReal(-1.00000000000000))*(gconst120)*(x298)*(x320)))+(((x301)*(x325)))+(((x302)*(x314)))+(((IkReal(-1.00000000000000))*(x293)*(x345)))+(((IkReal(-1.00000000000000))*(x297)*(x333)))+(((IkReal(-1.00000000000000))*(gconst117)*(x291)*(x317)))+(((IkReal(-1.00000000000000))*(gconst131)*(x291)*(x296)))+(((x330)*(x331)))+(((IkReal(-1.00000000000000))*(x306)*(x348)))+(((IkReal(-1.00000000000000))*(gconst115)*(x291)*(x333)))+(((IkReal(-1.00000000000000))*(gconst117)*(x304)*(x339)))+(((x296)*(x318)))+(((IkReal(-1.00000000000000))*(gconst129)*(x298)*(x336)))+(((x299)*(x316)))+(((x295)*(x332)))+(((x307)*(x344)))+(((x331)*(x342)))+(((x309)*(x310)))+(((x332)*(x344)))+(((IkReal(-1.00000000000000))*(x302)*(x311)))+(((x295)*(x307)))+(((x327)*(x333)))+(((x315)*(x322)))+(((x293)*(x323)))+(((x293)*(x324)))+(((x297)*(x335)))+(((IkReal(-1.00000000000000))*(gconst113)*(x298)*(x335)))+(((x313)*(x333)))+(((IkReal(-1.00000000000000))*(x310)*(x348)))+(((x306)*(x309)))+(((IkReal(-1.00000000000000))*(gconst120)*(x298)*(x338))));
op[4]=((((IkReal(-1.00000000000000))*(gconst115)*(x291)*(x310)))+(((IkReal(-1.00000000000000))*(gconst119)*(x326)*(x333)))+(((x294)*(x319)))+(((x310)*(x313)))+(((IkReal(-1.00000000000000))*(x297)*(x310)))+(((IkReal(-1.00000000000000))*(gconst131)*(x291)*(x292)))+(((IkReal(-1.00000000000000))*(gconst129)*(x298)*(x346)))+(((IkReal(-1.00000000000000))*(x299)*(x329)))+(((x324)*(x340)))+(((x310)*(x327)))+(((IkReal(-1.00000000000000))*(x324)*(x330)))+(((x332)*(x334)))+(((IkReal(-1.00000000000000))*(gconst115)*(x291)*(x306)))+(((IkReal(-1.00000000000000))*(gconst124)*(x298)*(x305)))+(((x311)*(x341)))+(((IkReal(-1.00000000000000))*(gconst111)*(x314)*(x326)))+(((x308)*(x332)))+(((IkReal(-1.00000000000000))*(gconst107)*(x298)*(x337)))+(((x302)*(x333)))+(((x314)*(x328)))+(((IkReal(-1.00000000000000))*(x321)*(x347)))+(((x306)*(x313)))+(((IkReal(-1.00000000000000))*(x293)*(x316)))+(((x297)*(x320)))+(((IkReal(-1.00000000000000))*(x317)*(x334)))+(((x323)*(x340)))+(((x296)*(x309)))+(((IkReal(-1.00000000000000))*(gconst108)*(x291)*(x317)))+(((x307)*(x334)))+(((IkReal(-1.00000000000000))*(gconst123)*(x298)*(x312)))+(((x297)*(x338)))+(((IkReal(-1.00000000000000))*(gconst124)*(x303)*(x327)))+(((IkReal(-1.00000000000000))*(gconst108)*(x304)*(x339)))+(((IkReal(-1.00000000000000))*(x311)*(x328)))+(((x306)*(x327)))+(((x307)*(x308)))+(((x292)*(x318)))+(((IkReal(-1.00000000000000))*(gconst113)*(x298)*(x320)))+(((IkReal(-1.00000000000000))*(x296)*(x348)))+(((x330)*(x345)))+(((IkReal(-1.00000000000000))*(x297)*(x306)))+(((IkReal(-1.00000000000000))*(x302)*(x335)))+(((IkReal(-1.00000000000000))*(gconst120)*(x298)*(x336)))+(((IkReal(-1.00000000000000))*(gconst128)*(x298)*(x343)))+(((x342)*(x345))));
op[5]=((((x301)*(x307)))+(((x302)*(x310)))+(((x322)*(x324)))+(((x322)*(x323)))+(((IkReal(-1.00000000000000))*(x302)*(x320)))+(((IkReal(-1.00000000000000))*(x301)*(x317)))+(((IkReal(-1.00000000000000))*(x328)*(x335)))+(((IkReal(-1.00000000000000))*(x300)*(x317)))+(((x296)*(x313)))+(((x292)*(x309)))+(((IkReal(-1.00000000000000))*(gconst111)*(x326)*(x333)))+(((x316)*(x330)))+(((x331)*(x343)))+(((IkReal(-1.00000000000000))*(gconst120)*(x298)*(x346)))+(((IkReal(-1.00000000000000))*(gconst117)*(x303)*(x327)))+(((x296)*(x327)))+(((IkReal(-1.00000000000000))*(gconst119)*(x306)*(x326)))+(((x294)*(x344)))+(((IkReal(-1.00000000000000))*(gconst117)*(x298)*(x305)))+(((x302)*(x306)))+(((x328)*(x333)))+(((IkReal(-1.00000000000000))*(gconst119)*(x310)*(x326)))+(((x293)*(x329)))+(((x294)*(x295)))+(((IkReal(-1.00000000000000))*(gconst124)*(x302)*(x303)))+(((x297)*(x336)))+(((IkReal(-1.00000000000000))*(x296)*(x297)))+(((x316)*(x342)))+(((x301)*(x332)))+(((x300)*(x332)))+(((IkReal(-1.00000000000000))*(x292)*(x348)))+(((IkReal(-1.00000000000000))*(gconst115)*(x291)*(x296))));
op[6]=((((IkReal(-1.00000000000000))*(x324)*(x343)))+(((IkReal(-1.00000000000000))*(gconst111)*(x306)*(x326)))+(((IkReal(-1.00000000000000))*(gconst111)*(x310)*(x326)))+(((IkReal(-1.00000000000000))*(x329)*(x342)))+(((IkReal(-1.00000000000000))*(gconst108)*(x303)*(x327)))+(((x292)*(x327)))+(((IkReal(-1.00000000000000))*(gconst123)*(x312)*(x339)))+(((x310)*(x328)))+(((gconst111)*(gconst124)*(x294)))+(((x329)*(x340)))+(((x297)*(x346)))+(((x294)*(x308)))+(((IkReal(-1.00000000000000))*(gconst117)*(x302)*(x303)))+(((x294)*(x334)))+(((IkReal(-1.00000000000000))*(gconst119)*(x296)*(x326)))+(((IkReal(-1.00000000000000))*(gconst107)*(x298)*(x312)))+(((IkReal(-1.00000000000000))*(x320)*(x328)))+(((x296)*(x302)))+(((IkReal(-1.00000000000000))*(gconst108)*(x298)*(x305)))+(((IkReal(-1.00000000000000))*(x292)*(x297)))+(((IkReal(-1.00000000000000))*(x316)*(x322)))+(((IkReal(-1.00000000000000))*(gconst124)*(x303)*(x328)))+(((x306)*(x328)))+(((x343)*(x345)))+(((x292)*(x313)))+(((x320)*(x341)))+(((IkReal(-1.00000000000000))*(gconst115)*(x291)*(x292))));
op[7]=((((x322)*(x329)))+(((x292)*(x302)))+(((x294)*(x301)))+(((x294)*(x300)))+(((IkReal(-1.00000000000000))*(gconst108)*(x302)*(x303)))+(((IkReal(-1.00000000000000))*(gconst119)*(x292)*(x326)))+(((IkReal(-1.00000000000000))*(gconst117)*(x303)*(x328)))+(((x296)*(x328)))+(((IkReal(-1.00000000000000))*(gconst111)*(x296)*(x326)))+(((x316)*(x343))));
op[8]=((((IkReal(-1.00000000000000))*(gconst111)*(x292)*(x326)))+(((IkReal(-1.00000000000000))*(gconst108)*(x303)*(x328)))+(((x292)*(x328)))+(((IkReal(-1.00000000000000))*(x312)*(x347)))+(((gconst108)*(gconst111)*(x294))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst148;
gconst148=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x349=((cj1)*(sj2));
IkReal x350=((cj2)*(sj1));
IkReal x351=((IkReal(1.00000000000000))*(sj3));
IkReal x352=((cj0)*(r00));
IkReal x353=((r01)*(sj0));
IkReal x354=((cj0)*(r01));
IkReal x355=((IkReal(1.00000000000000))*(cj3));
IkReal x356=((r00)*(sj0));
IkReal x357=((cj1)*(cj2)*(r02));
IkReal x358=((r02)*(sj1)*(sj2));
IkReal x359=((x353)*(x355));
if( IKabs(((gconst148)*(((((cj3)*(x356)))+(((sj3)*(x357)))+(((IkReal(-1.00000000000000))*(x350)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x350)*(x351)*(x353)))+(((IkReal(-1.00000000000000))*(x349)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x349)*(x351)*(x353)))+(((IkReal(-1.00000000000000))*(x351)*(x358)))+(((IkReal(-1.00000000000000))*(x354)*(x355))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst148)*(((((cj3)*(x357)))+(((IkReal(-1.00000000000000))*(x349)*(x352)*(x355)))+(((IkReal(-1.00000000000000))*(x349)*(x359)))+(((IkReal(-1.00000000000000))*(x350)*(x352)*(x355)))+(((sj3)*(x354)))+(((IkReal(-1.00000000000000))*(x351)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x358)))+(((IkReal(-1.00000000000000))*(x350)*(x359))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst148)*(((((cj3)*(x356)))+(((sj3)*(x357)))+(((IkReal(-1.00000000000000))*(x350)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x350)*(x351)*(x353)))+(((IkReal(-1.00000000000000))*(x349)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x349)*(x351)*(x353)))+(((IkReal(-1.00000000000000))*(x351)*(x358)))+(((IkReal(-1.00000000000000))*(x354)*(x355)))))), ((gconst148)*(((((cj3)*(x357)))+(((IkReal(-1.00000000000000))*(x349)*(x352)*(x355)))+(((IkReal(-1.00000000000000))*(x349)*(x359)))+(((IkReal(-1.00000000000000))*(x350)*(x352)*(x355)))+(((sj3)*(x354)))+(((IkReal(-1.00000000000000))*(x351)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x358)))+(((IkReal(-1.00000000000000))*(x350)*(x359)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x360=IKsin(j5);
IkReal x361=IKcos(j5);
IkReal x362=((cj1)*(sj2));
IkReal x363=((r01)*(sj0));
IkReal x364=((cj0)*(r00));
IkReal x365=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj3)*(x360)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((sj3)*(x361))));
evalcond[1]=((((x364)*(x365)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((sj3)*(x360)))+(((r02)*(sj1)*(sj2)))+(((cj3)*(x361)))+(((x363)*(x365)))+(((x362)*(x364)))+(((x362)*(x363))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x366=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x367=((cj1)*(x366));
IkReal x368=((cj0)*(r00));
IkReal x369=((sj1)*(x366));
IkReal x370=((IkReal(1.00000000000000))*(sj2));
IkReal x371=((r01)*(sj0));
cj5array[0]=((((cj2)*(r02)*(x369)))+(((cj2)*(x367)*(x371)))+(((cj2)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(x368)*(x369)*(x370)))+(((r02)*(sj2)*(x367)))+(((IkReal(-1.00000000000000))*(x369)*(x370)*(x371))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[3];
dummyeval[0]=sj4;
dummyeval[1]=cj4;
dummyeval[2]=cj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  || IKabs(dummyeval[2]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x372=((pz)*(sj2));
IkReal x373=((IkReal(1.00000000000000))*(cj1));
IkReal x374=((cj2)*(sj0));
IkReal x375=((py)*(sj1));
IkReal x376=((cj0)*(sj1));
IkReal x377=((cj2)*(px));
IkReal x378=((sj0)*(sj2));
IkReal x379=((IkReal(0.0750000000000000))*(cj2));
IkReal x380=((px)*(sj2));
IkReal x381=((IkReal(0.0750000000000000))*(sj2));
IkReal x382=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r00)*(sj2)*(x376)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x373)))+(((IkReal(-1.00000000000000))*(r01)*(x373)*(x374)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x373)))+(((r01)*(sj1)*(x378)))+(((IkReal(-1.00000000000000))*(r02)*(x382))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x381)))+(((sj1)*(x372)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x380)))+(((cj1)*(py)*(x378)))+(((IkReal(-1.00000000000000))*(sj1)*(x379)))+(((x376)*(x377)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x373)))+(((x374)*(x375))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(x373)*(x377)))+(((x376)*(x380)))+(((x375)*(x378)))+(((IkReal(-1.00000000000000))*(x372)*(x373)))+(((IkReal(-1.00000000000000))*(py)*(x373)*(x374)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj1)*(x379)))+(((IkReal(-1.00000000000000))*(sj1)*(x381)))+(((IkReal(-1.00000000000000))*(pz)*(x382))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x383=((cj1)*(cj5));
IkReal x384=((IkReal(1.00000000000000))*(cj2));
IkReal x385=((r00)*(sj0));
IkReal x386=((IkReal(1.00000000000000))*(sj5));
IkReal x387=((cj0)*(r01));
IkReal x388=((r01)*(sj0));
IkReal x389=((sj1)*(sj5));
IkReal x390=((cj0)*(r00));
IkReal x391=((sj2)*(x390));
IkReal x392=((cj2)*(cj5)*(sj1));
IkReal x393=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(cj5)*(x387)))+(((IkReal(-1.00000000000000))*(x386)*(x393)))+(((IkReal(-1.00000000000000))*(cj1)*(x386)*(x391)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x386)*(x388)))+(((cj5)*(x385)))+(((IkReal(-1.00000000000000))*(x384)*(x389)*(x390)))+(((IkReal(-1.00000000000000))*(x384)*(x388)*(x389)))+(((cj1)*(cj2)*(r02)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((sj5)*(x385)))+(((cj5)*(x393)))+(((IkReal(-1.00000000000000))*(r02)*(x383)*(x384)))+(((sj2)*(x383)*(x388)))+(((IkReal(-1.00000000000000))*(x386)*(x387)))+(((x390)*(x392)))+(((x383)*(x391)))+(((x388)*(x392))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(cj5)*(x387)))+(((IkReal(-1.00000000000000))*(x386)*(x393)))+(((IkReal(-1.00000000000000))*(cj1)*(x386)*(x391)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x386)*(x388)))+(((cj5)*(x385)))+(((IkReal(-1.00000000000000))*(x384)*(x389)*(x390)))+(((IkReal(-1.00000000000000))*(x384)*(x388)*(x389)))+(((cj1)*(cj2)*(r02)*(sj5)))))), ((gconst55)*(((((sj5)*(x385)))+(((cj5)*(x393)))+(((IkReal(-1.00000000000000))*(r02)*(x383)*(x384)))+(((sj2)*(x383)*(x388)))+(((IkReal(-1.00000000000000))*(x386)*(x387)))+(((x390)*(x392)))+(((x383)*(x391)))+(((x388)*(x392)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x394=IKsin(j3);
IkReal x395=IKcos(j3);
IkReal x396=((r01)*(sj0));
IkReal x397=((cj1)*(sj2));
IkReal x398=((cj2)*(sj1));
IkReal x399=((cj0)*(r00));
IkReal x400=((IkReal(1.00000000000000))*(x395));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x400)))+(((IkReal(-1.00000000000000))*(cj5)*(x394)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x398)*(x399)))+(((x396)*(x398)))+(((x396)*(x397)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x397)*(x399)))+(((sj5)*(x394)))+(((IkReal(-1.00000000000000))*(cj5)*(x400)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x401=((pz)*(sj2));
IkReal x402=((IkReal(1.00000000000000))*(cj1));
IkReal x403=((cj2)*(sj0));
IkReal x404=((py)*(sj1));
IkReal x405=((cj0)*(sj1));
IkReal x406=((cj2)*(px));
IkReal x407=((sj0)*(sj2));
IkReal x408=((IkReal(0.0750000000000000))*(cj2));
IkReal x409=((px)*(sj2));
IkReal x410=((IkReal(0.0750000000000000))*(sj2));
IkReal x411=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x407)))+(((IkReal(-1.00000000000000))*(r02)*(x411)))+(((r00)*(sj2)*(x405)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x402)))+(((IkReal(-1.00000000000000))*(r01)*(x402)*(x403)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x402))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x407)))+(((x405)*(x406)))+(((sj1)*(x401)))+(((IkReal(-1.00000000000000))*(cj1)*(x410)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x403)*(x404)))+(((IkReal(-1.00000000000000))*(sj1)*(x408)))+(((cj0)*(cj1)*(x409)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x402))));
evalcond[4]=((IkReal(0.215000000000000))+(((x405)*(x409)))+(((x404)*(x407)))+(((IkReal(-1.00000000000000))*(x401)*(x402)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x402)*(x403)))+(((cj1)*(x408)))+(((IkReal(-1.00000000000000))*(cj0)*(x402)*(x406)))+(((IkReal(-1.00000000000000))*(pz)*(x411)))+(((IkReal(-1.00000000000000))*(sj1)*(x410))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x412=((cj1)*(cj5));
IkReal x413=((cj0)*(r00));
IkReal x414=((cj5)*(sj0));
IkReal x415=((cj2)*(sj1));
IkReal x416=((r01)*(sj5));
IkReal x417=((cj1)*(sj5));
IkReal x418=((sj0)*(sj2));
IkReal x419=((IkReal(1.00000000000000))*(cj2)*(r02));
IkReal x420=((r02)*(sj1)*(sj2));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(x417)*(x419)))+(((sj5)*(x413)*(x415)))+(((sj2)*(x413)*(x417)))+(((cj1)*(x416)*(x418)))+(((sj0)*(x415)*(x416)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((r00)*(x414)))+(((sj5)*(x420))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((sj2)*(x412)*(x413)))+(((IkReal(-1.00000000000000))*(x412)*(x419)))+(((cj0)*(x416)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((r01)*(x414)*(x415)))+(((cj5)*(x420)))+(((cj5)*(x413)*(x415)))+(((r01)*(x412)*(x418))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(x417)*(x419)))+(((sj5)*(x413)*(x415)))+(((sj2)*(x413)*(x417)))+(((cj1)*(x416)*(x418)))+(((sj0)*(x415)*(x416)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((r00)*(x414)))+(((sj5)*(x420)))))), ((gconst56)*(((((sj2)*(x412)*(x413)))+(((IkReal(-1.00000000000000))*(x412)*(x419)))+(((cj0)*(x416)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((r01)*(x414)*(x415)))+(((cj5)*(x420)))+(((cj5)*(x413)*(x415)))+(((r01)*(x412)*(x418)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x421=IKsin(j3);
IkReal x422=IKcos(j3);
IkReal x423=((cj1)*(sj2));
IkReal x424=((r01)*(sj0));
IkReal x425=((cj0)*(r00));
IkReal x426=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((cj5)*(x421)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(sj5)*(x422))));
evalcond[1]=((((x425)*(x426)))+(((x424)*(x426)))+(((cj5)*(x422)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((sj5)*(x421)))+(((x423)*(x425)))+(((x423)*(x424)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x427=((cj1)*(cj2));
IkReal x428=((IkReal(1.00000000000000))*(cj0));
IkReal x429=((sj1)*(sj2));
IkReal x430=((IkReal(1.00000000000000))*(sj0));
IkReal x431=((sj0)*(x429));
IkReal x432=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x433=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x427)*(x428)))+(cj5)+(((cj0)*(r00)*(x429)))+(((IkReal(-1.00000000000000))*(r02)*(x433)))+(((IkReal(-1.00000000000000))*(r02)*(x432)))+(((r01)*(x431)))+(((IkReal(-1.00000000000000))*(r01)*(x427)*(x430))));
evalcond[2]=((IkReal(0.295000000000000))+(((py)*(x431)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x429)))+(((IkReal(-1.00000000000000))*(px)*(x427)*(x428)))+(((IkReal(0.0750000000000000))*(x427)))+(((IkReal(-1.00000000000000))*(pz)*(x432)))+(((IkReal(-1.00000000000000))*(pz)*(x433)))+(((cj0)*(px)*(x429)))+(((IkReal(-1.00000000000000))*(py)*(x427)*(x430))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x434=((cj1)*(sj2));
IkReal x435=((cj0)*(r00));
IkReal x436=((IkReal(1.00000000000000))*(pz));
IkReal x437=((sj1)*(sj2));
IkReal x438=((r01)*(sj0));
IkReal x439=((IkReal(1.00000000000000))*(r02));
IkReal x440=((cj1)*(cj2));
IkReal x441=((cj2)*(sj1));
IkReal x442=((IkReal(1.00000000000000))*(cj0));
IkReal x443=((IkReal(1.00000000000000))*(sj0)*(x440));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x442))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x435)*(x440)))+(((IkReal(-1.00000000000000))*(x434)*(x439)))+(((IkReal(-1.00000000000000))*(x439)*(x441)))+(((IkReal(-1.00000000000000))*(x438)*(x440)))+(((x437)*(x438)))+(((x435)*(x437))));
evalcond[3]=((((x435)*(x441)))+(((x434)*(x438)))+(((x434)*(x435)))+(((r02)*(x437)))+(((IkReal(-1.00000000000000))*(x439)*(x440)))+(((x438)*(x441))));
evalcond[4]=((IkReal(0.295000000000000))+(((py)*(sj0)*(x437)))+(((IkReal(-0.0750000000000000))*(x437)))+(((IkReal(-1.00000000000000))*(x434)*(x436)))+(((IkReal(-1.00000000000000))*(x436)*(x441)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x440)*(x442)))+(((IkReal(0.0750000000000000))*(x440)))+(((cj0)*(px)*(x437)))+(((IkReal(-1.00000000000000))*(py)*(x443))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x444=((IkReal(12.5000000000000))*(cj1));
IkReal x445=((py)*(sj0));
IkReal x446=((cj0)*(px));
IkReal x447=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x445)*(x447)))+(((IkReal(3.37500000000000))*(cj2)))+(((cj2)*(pz)*(x444)))+(((IkReal(-1.00000000000000))*(sj2)*(x444)*(x445)))+(((IkReal(-1.00000000000000))*(sj2)*(x444)*(x446)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(x446)*(x447))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x445)*(x447)))+(((IkReal(3.37500000000000))*(cj2)))+(((cj2)*(pz)*(x444)))+(((IkReal(-1.00000000000000))*(sj2)*(x444)*(x445)))+(((IkReal(-1.00000000000000))*(sj2)*(x444)*(x446)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(x446)*(x447)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x445)*(x447)))+(((IkReal(3.37500000000000))*(cj2)))+(((cj2)*(pz)*(x444)))+(((IkReal(-1.00000000000000))*(sj2)*(x444)*(x445)))+(((IkReal(-1.00000000000000))*(sj2)*(x444)*(x446)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(x446)*(x447)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x448=((cj1)*(sj2));
IkReal x449=((cj2)*(sj1));
IkReal x450=((cj0)*(px));
IkReal x451=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x449)))+(((IkReal(-0.0750000000000000))*(x448)))+(((pz)*(sj1)*(sj2)))+(((x449)*(x450)))+(((x449)*(x451)))+(((x448)*(x451)))+(((x448)*(x450)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x452=((cj1)*(sj2));
IkReal x453=((cj0)*(r00));
IkReal x454=((IkReal(1.00000000000000))*(pz));
IkReal x455=((sj1)*(sj2));
IkReal x456=((r01)*(sj0));
IkReal x457=((IkReal(1.00000000000000))*(r02));
IkReal x458=((cj1)*(cj2));
IkReal x459=((cj2)*(sj1));
IkReal x460=((IkReal(1.00000000000000))*(cj0));
IkReal x461=((IkReal(1.00000000000000))*(sj0)*(x458));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x460))));
evalcond[2]=((IkReal(-1.00000000000000))+(((x455)*(x456)))+(((x453)*(x455)))+(((IkReal(-1.00000000000000))*(x456)*(x458)))+(((IkReal(-1.00000000000000))*(x452)*(x457)))+(((IkReal(-1.00000000000000))*(x453)*(x458)))+(((IkReal(-1.00000000000000))*(x457)*(x459))));
evalcond[3]=((((x452)*(x453)))+(((x452)*(x456)))+(((x453)*(x459)))+(((r02)*(x455)))+(((IkReal(-1.00000000000000))*(x457)*(x458)))+(((x456)*(x459))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(px)*(x458)*(x460)))+(((py)*(sj0)*(x455)))+(((IkReal(0.0750000000000000))*(x458)))+(((cj0)*(px)*(x455)))+(((IkReal(-1.00000000000000))*(x454)*(x459)))+(((IkReal(-1.00000000000000))*(py)*(x461)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x452)*(x454)))+(((IkReal(-0.0750000000000000))*(x455))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x462=((IkReal(12.5000000000000))*(cj1));
IkReal x463=((py)*(sj0));
IkReal x464=((cj0)*(px));
IkReal x465=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x463)*(x465)))+(((cj2)*(pz)*(x462)))+(((IkReal(-1.00000000000000))*(sj2)*(x462)*(x463)))+(((IkReal(-1.00000000000000))*(sj2)*(x462)*(x464)))+(((IkReal(-1.00000000000000))*(x464)*(x465)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x463)*(x465)))+(((cj2)*(pz)*(x462)))+(((IkReal(-1.00000000000000))*(sj2)*(x462)*(x463)))+(((IkReal(-1.00000000000000))*(sj2)*(x462)*(x464)))+(((IkReal(-1.00000000000000))*(x464)*(x465)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x463)*(x465)))+(((cj2)*(pz)*(x462)))+(((IkReal(-1.00000000000000))*(sj2)*(x462)*(x463)))+(((IkReal(-1.00000000000000))*(sj2)*(x462)*(x464)))+(((IkReal(-1.00000000000000))*(x464)*(x465)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x466=((cj1)*(sj2));
IkReal x467=((cj2)*(sj1));
IkReal x468=((cj0)*(px));
IkReal x469=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x466)*(x469)))+(((x466)*(x468)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((x467)*(x469)))+(((x467)*(x468)))+(((IkReal(-0.0750000000000000))*(x467)))+(((IkReal(-0.0750000000000000))*(x466)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x470=IKsin(j3);
IkReal x471=IKcos(j3);
IkReal x472=((sj1)*(sj2));
IkReal x473=((cj2)*(sj1));
IkReal x474=((py)*(sj0));
IkReal x475=((IkReal(1.00000000000000))*(cj0));
IkReal x476=((r01)*(sj0));
IkReal x477=((cj1)*(sj2));
IkReal x478=((cj0)*(x477));
IkReal x479=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x475)))+(((IkReal(0.0800000000000000))*(x470)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x471)))+(((IkReal(-1.00000000000000))*(r01)*(x475))));
evalcond[2]=((((sj5)*(x470)))+(((x473)*(x476)))+(((cj0)*(r00)*(x473)))+(((r02)*(x472)))+(((x476)*(x477)))+(((r00)*(x478)))+(((IkReal(-1.00000000000000))*(r02)*(x479))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x473)*(x474)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x473)))+(((IkReal(-1.00000000000000))*(pz)*(x479)))+(((pz)*(x472)))+(((IkReal(-0.0750000000000000))*(x473)))+(((IkReal(-0.0750000000000000))*(x477)))+(((x474)*(x477)))+(((IkReal(0.0800000000000000))*(x471)))+(((px)*(x478))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x480=((IkReal(1.00000000000000))*(sj2));
IkReal x481=((IkReal(1.00000000000000))*(r01));
IkReal x482=((cj0)*(r00));
IkReal x483=((cj2)*(sj1));
if( IKabs(((gconst57)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x480)))+(((IkReal(-1.00000000000000))*(sj0)*(x481)*(x483)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x480)))+(((IkReal(-1.00000000000000))*(cj1)*(x480)*(x482)))+(((IkReal(-1.00000000000000))*(x482)*(x483)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x481))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x480)))+(((IkReal(-1.00000000000000))*(sj0)*(x481)*(x483)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x480)))+(((IkReal(-1.00000000000000))*(cj1)*(x480)*(x482)))+(((IkReal(-1.00000000000000))*(x482)*(x483)))+(((cj1)*(cj2)*(r02)))))), ((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x481)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x484=IKsin(j3);
IkReal x485=IKcos(j3);
IkReal x486=((sj1)*(sj2));
IkReal x487=((cj2)*(sj1));
IkReal x488=((py)*(sj0));
IkReal x489=((IkReal(1.00000000000000))*(cj0));
IkReal x490=((r01)*(sj0));
IkReal x491=((cj1)*(sj2));
IkReal x492=((cj0)*(x491));
IkReal x493=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(0.0800000000000000))*(x484)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x489))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x485)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x489))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x493)))+(((r02)*(x486)))+(((x490)*(x491)))+(((sj5)*(x484)))+(((r00)*(x492)))+(((x487)*(x490)))+(((cj0)*(r00)*(x487))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(0.0800000000000000))*(x485)))+(((x488)*(x491)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(x486)))+(((IkReal(-0.0750000000000000))*(x491)))+(((IkReal(-0.0750000000000000))*(x487)))+(((px)*(x492)))+(((IkReal(-1.00000000000000))*(pz)*(x493)))+(((cj0)*(px)*(x487)))+(((x487)*(x488))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x494=((sj1)*(sj2));
IkReal x495=((cj1)*(cj2));
IkReal x496=((IkReal(1.00000000000000))*(sj0));
IkReal x497=((sj0)*(x494));
IkReal x498=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x499=((IkReal(1.00000000000000))*(cj2)*(sj1));
IkReal x500=((IkReal(1.00000000000000))*(cj0)*(x495));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x499)))+(((IkReal(-1.00000000000000))*(r02)*(x498)))+(((IkReal(-1.00000000000000))*(r00)*(x500)))+(((IkReal(-1.00000000000000))*(r01)*(x495)*(x496)))+(((cj0)*(r00)*(x494)))+(((r01)*(x497)))+(((IkReal(-1.00000000000000))*(cj5))));
evalcond[2]=((IkReal(0.295000000000000))+(((py)*(x497)))+(((IkReal(-1.00000000000000))*(py)*(x495)*(x496)))+(((IkReal(-0.0750000000000000))*(x494)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj0)*(px)*(x494)))+(((IkReal(0.0750000000000000))*(x495)))+(((IkReal(-1.00000000000000))*(pz)*(x498)))+(((IkReal(-1.00000000000000))*(pz)*(x499)))+(((IkReal(-1.00000000000000))*(px)*(x500))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x501=((cj1)*(sj2));
IkReal x502=((cj0)*(r00));
IkReal x503=((IkReal(1.00000000000000))*(pz));
IkReal x504=((sj1)*(sj2));
IkReal x505=((r01)*(sj0));
IkReal x506=((IkReal(1.00000000000000))*(r02));
IkReal x507=((cj1)*(cj2));
IkReal x508=((cj2)*(sj1));
IkReal x509=((IkReal(1.00000000000000))*(cj0));
IkReal x510=((IkReal(1.00000000000000))*(sj0)*(x507));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x509))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x506)*(x508)))+(((IkReal(-1.00000000000000))*(x501)*(x506)))+(((x502)*(x504)))+(((IkReal(-1.00000000000000))*(x502)*(x507)))+(((x504)*(x505)))+(((IkReal(-1.00000000000000))*(x505)*(x507))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x506)*(x507)))+(((x505)*(x508)))+(((x501)*(x505)))+(((x501)*(x502)))+(((x502)*(x508)))+(((r02)*(x504))));
evalcond[4]=((IkReal(0.295000000000000))+(((py)*(sj0)*(x504)))+(((IkReal(-1.00000000000000))*(x501)*(x503)))+(((IkReal(-1.00000000000000))*(x503)*(x508)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x510)))+(((cj0)*(px)*(x504)))+(((IkReal(-0.0750000000000000))*(x504)))+(((IkReal(-1.00000000000000))*(px)*(x507)*(x509)))+(((IkReal(0.0750000000000000))*(x507))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x511=((IkReal(12.5000000000000))*(sj2));
IkReal x512=((py)*(sj0));
IkReal x513=((cj2)*(sj1));
IkReal x514=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x514))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x511)))+(((IkReal(-0.937500000000000))*(x513)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(12.5000000000000))*(x512)*(x513)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x511)))+(((cj1)*(x511)*(x512)))+(((px)*(x513)*(x514))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x514)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x511)))+(((IkReal(-0.937500000000000))*(x513)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(12.5000000000000))*(x512)*(x513)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x511)))+(((cj1)*(x511)*(x512)))+(((px)*(x513)*(x514)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x514)))), ((IkReal(-1.12500000000000))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x511)))+(((IkReal(-0.937500000000000))*(x513)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(12.5000000000000))*(x512)*(x513)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x511)))+(((cj1)*(x511)*(x512)))+(((px)*(x513)*(x514)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x515=((cj1)*(sj2));
IkReal x516=((cj2)*(sj1));
IkReal x517=((cj0)*(px));
IkReal x518=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0750000000000000))*(x515)))+(((IkReal(-0.0750000000000000))*(x516)))+(((x516)*(x517)))+(((x516)*(x518)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((x515)*(x518)))+(((x515)*(x517)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x519=((cj1)*(sj2));
IkReal x520=((cj0)*(r00));
IkReal x521=((IkReal(1.00000000000000))*(pz));
IkReal x522=((sj1)*(sj2));
IkReal x523=((r01)*(sj0));
IkReal x524=((IkReal(1.00000000000000))*(r02));
IkReal x525=((cj1)*(cj2));
IkReal x526=((cj2)*(sj1));
IkReal x527=((IkReal(1.00000000000000))*(cj0));
IkReal x528=((IkReal(1.00000000000000))*(sj0)*(x525));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x527))));
evalcond[2]=((IkReal(1.00000000000000))+(((x522)*(x523)))+(((IkReal(-1.00000000000000))*(x524)*(x526)))+(((x520)*(x522)))+(((IkReal(-1.00000000000000))*(x520)*(x525)))+(((IkReal(-1.00000000000000))*(x523)*(x525)))+(((IkReal(-1.00000000000000))*(x519)*(x524))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x524)*(x525)))+(((r02)*(x522)))+(((x519)*(x523)))+(((x519)*(x520)))+(((x520)*(x526)))+(((x523)*(x526))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x522)))+(((IkReal(-1.00000000000000))*(px)*(x525)*(x527)))+(((IkReal(-1.00000000000000))*(x521)*(x526)))+(((cj0)*(px)*(x522)))+(((IkReal(-0.270000000000000))*(sj2)))+(((py)*(sj0)*(x522)))+(((IkReal(0.0750000000000000))*(x525)))+(((IkReal(-1.00000000000000))*(py)*(x528)))+(((IkReal(-1.00000000000000))*(x519)*(x521))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x529=((IkReal(12.5000000000000))*(sj2));
IkReal x530=((py)*(sj0));
IkReal x531=((cj2)*(sj1));
IkReal x532=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(-1.00000000000000))*(py)*(x532)))+(((IkReal(12.5000000000000))*(px)*(sj0))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((cj1)*(x529)*(x530)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x531)*(x532)))+(((pz)*(sj1)*(x529)))+(((IkReal(-0.937500000000000))*(x531)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(12.5000000000000))*(x530)*(x531)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x529))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(py)*(x532)))+(((IkReal(12.5000000000000))*(px)*(sj0)))))+IKsqr(((IkReal(-1.12500000000000))+(((cj1)*(x529)*(x530)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x531)*(x532)))+(((pz)*(sj1)*(x529)))+(((IkReal(-0.937500000000000))*(x531)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(12.5000000000000))*(x530)*(x531)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x529)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(py)*(x532)))+(((IkReal(12.5000000000000))*(px)*(sj0)))), ((IkReal(-1.12500000000000))+(((cj1)*(x529)*(x530)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x531)*(x532)))+(((pz)*(sj1)*(x529)))+(((IkReal(-0.937500000000000))*(x531)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(12.5000000000000))*(x530)*(x531)))+(((IkReal(-3.37500000000000))*(cj2)))+(((cj0)*(cj1)*(px)*(x529)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x533=((cj1)*(sj2));
IkReal x534=((cj2)*(sj1));
IkReal x535=((cj0)*(px));
IkReal x536=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x533)))+(((IkReal(-0.0750000000000000))*(x534)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((x534)*(x535)))+(((x534)*(x536)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((x533)*(x535)))+(((x533)*(x536))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-12.5000000000000))*(cj0)*(py)))))+IKsqr(((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-12.5000000000000))*(cj0)*(py)))), ((((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x537=IKsin(j3);
IkReal x538=IKcos(j3);
IkReal x539=((sj1)*(sj2));
IkReal x540=((cj2)*(sj1));
IkReal x541=((py)*(sj0));
IkReal x542=((IkReal(1.00000000000000))*(cj0));
IkReal x543=((r01)*(sj0));
IkReal x544=((cj1)*(sj2));
IkReal x545=((cj0)*(x544));
IkReal x546=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-0.0800000000000000))*(x537)))+(((IkReal(-1.00000000000000))*(py)*(x542)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(x542))));
evalcond[2]=((((r02)*(x539)))+(((x540)*(x543)))+(((sj5)*(x537)))+(((x543)*(x544)))+(((IkReal(-1.00000000000000))*(r02)*(x546)))+(((cj0)*(r00)*(x540)))+(((r00)*(x545))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((pz)*(x539)))+(((IkReal(-0.0750000000000000))*(x540)))+(((IkReal(-0.0750000000000000))*(x544)))+(((x541)*(x544)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x540)))+(((IkReal(-0.0800000000000000))*(x538)))+(((x540)*(x541)))+(((px)*(x545)))+(((IkReal(-1.00000000000000))*(pz)*(x546))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x547=((IkReal(1.00000000000000))*(sj2));
IkReal x548=((IkReal(1.00000000000000))*(r01));
IkReal x549=((cj0)*(r00));
IkReal x550=((cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x547)))+(((IkReal(-1.00000000000000))*(sj0)*(x548)*(x550)))+(((IkReal(-1.00000000000000))*(cj1)*(x547)*(x549)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x547)))+(((IkReal(-1.00000000000000))*(x549)*(x550)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x548))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x547)))+(((IkReal(-1.00000000000000))*(sj0)*(x548)*(x550)))+(((IkReal(-1.00000000000000))*(cj1)*(x547)*(x549)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x547)))+(((IkReal(-1.00000000000000))*(x549)*(x550)))+(((cj1)*(cj2)*(r02)))))), ((gconst58)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x548)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x551=IKsin(j3);
IkReal x552=IKcos(j3);
IkReal x553=((sj1)*(sj2));
IkReal x554=((cj2)*(sj1));
IkReal x555=((py)*(sj0));
IkReal x556=((IkReal(1.00000000000000))*(cj0));
IkReal x557=((r01)*(sj0));
IkReal x558=((cj1)*(sj2));
IkReal x559=((cj0)*(x558));
IkReal x560=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-0.0800000000000000))*(x551)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x556))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x552)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x556))));
evalcond[2]=((((r02)*(x553)))+(((sj5)*(x551)))+(((x557)*(x558)))+(((x554)*(x557)))+(((cj0)*(r00)*(x554)))+(((r00)*(x559)))+(((IkReal(-1.00000000000000))*(r02)*(x560))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((pz)*(x553)))+(((IkReal(-0.0800000000000000))*(x552)))+(((IkReal(-0.0750000000000000))*(x558)))+(((IkReal(-0.0750000000000000))*(x554)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x554)*(x555)))+(((px)*(x559)))+(((IkReal(-1.00000000000000))*(pz)*(x560)))+(((x555)*(x558)))+(((cj0)*(px)*(x554))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x561=((cj1)*(cj2));
IkReal x562=((IkReal(1.00000000000000))*(cj0));
IkReal x563=((sj1)*(sj2));
IkReal x564=((IkReal(1.00000000000000))*(sj0));
IkReal x565=((sj0)*(x563));
IkReal x566=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x567=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x561)*(x564)))+(((r01)*(x565)))+(((cj0)*(r00)*(x563)))+(((IkReal(-1.00000000000000))*(r00)*(x561)*(x562)))+(((IkReal(-1.00000000000000))*(r02)*(x567)))+(((IkReal(-1.00000000000000))*(r02)*(x566))));
evalcond[2]=((IkReal(0.295000000000000))+(((cj0)*(px)*(x563)))+(((IkReal(-1.00000000000000))*(px)*(x561)*(x562)))+(((IkReal(0.0750000000000000))*(x561)))+(((IkReal(-1.00000000000000))*(pz)*(x567)))+(((IkReal(-1.00000000000000))*(pz)*(x566)))+(((IkReal(-1.00000000000000))*(py)*(x561)*(x564)))+(((IkReal(-0.270000000000000))*(sj2)))+(((py)*(x565)))+(((IkReal(-0.0750000000000000))*(x563)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x568=((IkReal(1.00000000000000))*(sj2));
IkReal x569=((IkReal(1.00000000000000))*(r01));
IkReal x570=((cj0)*(r00));
IkReal x571=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x568)))+(((IkReal(-1.00000000000000))*(sj0)*(x569)*(x571)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x568)))+(((IkReal(-1.00000000000000))*(cj1)*(x568)*(x570)))+(((IkReal(-1.00000000000000))*(x570)*(x571)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x569))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x568)))+(((IkReal(-1.00000000000000))*(sj0)*(x569)*(x571)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x568)))+(((IkReal(-1.00000000000000))*(cj1)*(x568)*(x570)))+(((IkReal(-1.00000000000000))*(x570)*(x571)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x569)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x568)))+(((IkReal(-1.00000000000000))*(sj0)*(x569)*(x571)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x568)))+(((IkReal(-1.00000000000000))*(cj1)*(x568)*(x570)))+(((IkReal(-1.00000000000000))*(x570)*(x571)))+(((cj1)*(cj2)*(r02)))), ((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x569)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x572=IKsin(j3);
IkReal x573=IKcos(j3);
IkReal x574=((sj1)*(sj2));
IkReal x575=((IkReal(0.0800000000000000))*(sj4));
IkReal x576=((cj2)*(sj1));
IkReal x577=((py)*(sj0));
IkReal x578=((IkReal(1.00000000000000))*(cj0));
IkReal x579=((r01)*(sj0));
IkReal x580=((cj1)*(sj2));
IkReal x581=((cj0)*(x580));
IkReal x582=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x578)))+(((px)*(sj0)))+(((x572)*(x575))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x578)))+(((IkReal(-1.00000000000000))*(x573))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x582)))+(((x579)*(x580)))+(((x576)*(x579)))+(((r02)*(x574)))+(x572)+(((r00)*(x581)))+(((cj0)*(r00)*(x576))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x576)*(x577)))+(((IkReal(-0.0750000000000000))*(x580)))+(((x577)*(x580)))+(((px)*(x581)))+(((IkReal(-1.00000000000000))*(pz)*(x582)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x576)))+(((IkReal(-0.0750000000000000))*(x576)))+(((pz)*(x574)))+(((x573)*(x575))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x583=((cj1)*(cj2));
IkReal x584=((IkReal(1.00000000000000))*(cj0));
IkReal x585=((sj1)*(sj2));
IkReal x586=((IkReal(1.00000000000000))*(sj0));
IkReal x587=((sj0)*(x585));
IkReal x588=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x589=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x585)))+(((IkReal(-1.00000000000000))*(r02)*(x588)))+(((IkReal(-1.00000000000000))*(r02)*(x589)))+(((IkReal(-1.00000000000000))*(r01)*(x583)*(x586)))+(((IkReal(-1.00000000000000))*(r00)*(x583)*(x584)))+(((r01)*(x587))));
evalcond[2]=((IkReal(0.295000000000000))+(((cj0)*(px)*(x585)))+(((IkReal(-1.00000000000000))*(px)*(x583)*(x584)))+(((IkReal(-0.0750000000000000))*(x585)))+(((IkReal(-1.00000000000000))*(pz)*(x588)))+(((IkReal(-1.00000000000000))*(pz)*(x589)))+(((IkReal(0.0750000000000000))*(x583)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(0.0800000000000000))*(cj4)))+(((py)*(x587)))+(((IkReal(-1.00000000000000))*(py)*(x583)*(x586))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x590=((cj1)*(sj2));
IkReal x591=((r01)*(sj0));
IkReal x592=((cj0)*(r00));
IkReal x593=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x590)*(x591)))+(((x590)*(x592)))+(((r02)*(sj1)*(sj2)))+(((x591)*(x593)))+(((x592)*(x593))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x590)*(x591)))+(((x590)*(x592)))+(((r02)*(sj1)*(sj2)))+(((x591)*(x593)))+(((x592)*(x593)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x590)*(x591)))+(((x590)*(x592)))+(((r02)*(sj1)*(sj2)))+(((x591)*(x593)))+(((x592)*(x593)))), ((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x594=IKsin(j3);
IkReal x595=IKcos(j3);
IkReal x596=((sj1)*(sj2));
IkReal x597=((IkReal(0.0800000000000000))*(sj4));
IkReal x598=((cj2)*(sj1));
IkReal x599=((py)*(sj0));
IkReal x600=((IkReal(1.00000000000000))*(cj0));
IkReal x601=((r01)*(sj0));
IkReal x602=((cj1)*(sj2));
IkReal x603=((cj0)*(x602));
IkReal x604=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x594)*(x597)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x600))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x600)))+(x595));
evalcond[2]=((((r00)*(x603)))+(((x601)*(x602)))+(((cj0)*(r00)*(x598)))+(((IkReal(-1.00000000000000))*(x594)))+(((r02)*(x596)))+(((IkReal(-1.00000000000000))*(r02)*(x604)))+(((x598)*(x601))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x598)*(x599)))+(((x595)*(x597)))+(((pz)*(x596)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x603)))+(((IkReal(-0.0750000000000000))*(x602)))+(((IkReal(-0.0750000000000000))*(x598)))+(((cj0)*(px)*(x598)))+(((x599)*(x602)))+(((IkReal(-1.00000000000000))*(pz)*(x604))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x605=((cj1)*(cj2));
IkReal x606=((IkReal(1.00000000000000))*(cj0));
IkReal x607=((sj1)*(sj2));
IkReal x608=((IkReal(1.00000000000000))*(sj0));
IkReal x609=((sj0)*(x607));
IkReal x610=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x611=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r01)*(x605)*(x608)))+(((IkReal(-1.00000000000000))*(r00)*(x605)*(x606)))+(((cj0)*(r00)*(x607)))+(((r01)*(x609)))+(((IkReal(-1.00000000000000))*(r02)*(x611)))+(((IkReal(-1.00000000000000))*(r02)*(x610))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(0.0750000000000000))*(x605)))+(((IkReal(-1.00000000000000))*(py)*(x605)*(x608)))+(((IkReal(-0.0750000000000000))*(x607)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x605)*(x606)))+(((cj0)*(px)*(x607)))+(((IkReal(-1.00000000000000))*(pz)*(x611)))+(((IkReal(-1.00000000000000))*(pz)*(x610)))+(((py)*(x609)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x612=((pz)*(sj2));
IkReal x613=((IkReal(1.00000000000000))*(cj1));
IkReal x614=((cj2)*(sj0));
IkReal x615=((py)*(sj1));
IkReal x616=((cj0)*(sj1));
IkReal x617=((cj2)*(px));
IkReal x618=((sj0)*(sj2));
IkReal x619=((IkReal(0.0750000000000000))*(cj2));
IkReal x620=((px)*(sj2));
IkReal x621=((IkReal(0.0750000000000000))*(sj2));
IkReal x622=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x613)))+(((IkReal(-1.00000000000000))*(r02)*(x622)))+(((r01)*(sj1)*(x618)))+(((IkReal(-1.00000000000000))*(r01)*(x613)*(x614)))+(((r00)*(sj2)*(x616)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x613))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x621)))+(((sj1)*(x612)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x619)))+(((cj1)*(py)*(x618)))+(((x616)*(x617)))+(((cj0)*(cj1)*(x620)))+(((x614)*(x615)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x613))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(x612)*(x613)))+(((IkReal(-1.00000000000000))*(pz)*(x622)))+(((IkReal(-1.00000000000000))*(cj0)*(x613)*(x617)))+(((x616)*(x620)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x613)*(x614)))+(((cj1)*(x619)))+(((x615)*(x618)))+(((IkReal(-1.00000000000000))*(sj1)*(x621))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x623=((cj1)*(sj2));
IkReal x624=((r01)*(sj0));
IkReal x625=((cj0)*(r00));
IkReal x626=((cj2)*(sj1));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x625)*(x626)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x624)*(x626)))+(((r02)*(sj1)*(sj2)))+(((x623)*(x624)))+(((x623)*(x625))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))+IKsqr(((((x625)*(x626)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x624)*(x626)))+(((r02)*(sj1)*(sj2)))+(((x623)*(x624)))+(((x623)*(x625)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))), ((((x625)*(x626)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x624)*(x626)))+(((r02)*(sj1)*(sj2)))+(((x623)*(x624)))+(((x623)*(x625)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x627=((cj1)*(sj2));
IkReal x628=((r01)*(sj0));
IkReal x629=((cj0)*(r00));
IkReal x630=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j3)))));
evalcond[1]=((((x628)*(x630)))+(((x629)*(x630)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(IKcos(j3))))+(((r02)*(sj1)*(sj2)))+(((x627)*(x629)))+(((x627)*(x628))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x631=((pz)*(sj2));
IkReal x632=((IkReal(1.00000000000000))*(cj1));
IkReal x633=((cj2)*(sj0));
IkReal x634=((py)*(sj1));
IkReal x635=((cj0)*(sj1));
IkReal x636=((cj2)*(px));
IkReal x637=((sj0)*(sj2));
IkReal x638=((IkReal(0.0750000000000000))*(cj2));
IkReal x639=((px)*(sj2));
IkReal x640=((IkReal(0.0750000000000000))*(sj2));
IkReal x641=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r00)*(sj2)*(x635)))+(((IkReal(-1.00000000000000))*(r01)*(x632)*(x633)))+(((r01)*(sj1)*(x637)))+(((IkReal(-1.00000000000000))*(r02)*(x641)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x632)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x632))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x637)))+(((IkReal(-1.00000000000000))*(cj1)*(x640)))+(((x635)*(x636)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x632)))+(((cj0)*(cj1)*(x639)))+(((sj1)*(x631)))+(((IkReal(-1.00000000000000))*(sj1)*(x638)))+(((x633)*(x634))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(x632)*(x636)))+(((IkReal(-1.00000000000000))*(x631)*(x632)))+(((x634)*(x637)))+(((x635)*(x639)))+(((IkReal(-1.00000000000000))*(sj1)*(x640)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x632)*(x633)))+(((IkReal(-1.00000000000000))*(pz)*(x641)))+(((cj1)*(x638))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x642=((IkReal(1.00000000000000))*(sj2));
IkReal x643=((cj0)*(r00));
IkReal x644=((r01)*(sj0));
IkReal x645=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x644)*(x645)))+(((IkReal(-1.00000000000000))*(x643)*(x645)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x642)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x642)*(x643)))+(((IkReal(-1.00000000000000))*(cj1)*(x642)*(x644))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(x644)*(x645)))+(((IkReal(-1.00000000000000))*(x643)*(x645)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x642)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x642)*(x643)))+(((IkReal(-1.00000000000000))*(cj1)*(x642)*(x644)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(x644)*(x645)))+(((IkReal(-1.00000000000000))*(x643)*(x645)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x642)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x642)*(x643)))+(((IkReal(-1.00000000000000))*(cj1)*(x642)*(x644)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x646=((cj1)*(sj2));
IkReal x647=((r01)*(sj0));
IkReal x648=((cj0)*(r00));
IkReal x649=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j3))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x648)*(x649)))+(IKcos(j3))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x646)*(x647)))+(((x646)*(x648)))+(((x647)*(x649))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x650=((cj1)*(sj2));
IkReal x651=((r01)*(sj0));
IkReal x652=((cj0)*(r00));
IkReal x653=((cj2)*(sj1));
if( IKabs(((IkReal(0.500000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x652)*(x653)))+(((x651)*(x653)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x650)*(x651)))+(((x650)*(x652))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.500000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x652)*(x653)))+(((x651)*(x653)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x650)*(x651)))+(((x650)*(x652)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(0.500000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(px)*(sj0)))+(((IkReal(25.0000000000000))*(cj0)*(py)))))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x652)*(x653)))+(((x651)*(x653)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x650)*(x651)))+(((x650)*(x652)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x654=IKsin(j3);
IkReal x655=IKcos(j3);
IkReal x656=((sj1)*(sj2));
IkReal x657=((IkReal(0.0800000000000000))*(sj4));
IkReal x658=((cj2)*(sj1));
IkReal x659=((py)*(sj0));
IkReal x660=((IkReal(1.00000000000000))*(cj0));
IkReal x661=((IkReal(1.00000000000000))*(cj4));
IkReal x662=((r01)*(sj0));
IkReal x663=((cj1)*(sj2));
IkReal x664=((cj0)*(x663));
IkReal x665=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x654)*(x657)))+(((IkReal(-1.00000000000000))*(py)*(x660)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x654)*(x661)))+(((IkReal(-1.00000000000000))*(r01)*(x660))));
evalcond[2]=((((r00)*(x664)))+(((cj0)*(r00)*(x658)))+(((x662)*(x663)))+(((r02)*(x656)))+(((IkReal(-1.00000000000000))*(x655)*(x661)))+(((x658)*(x662)))+(((IkReal(-1.00000000000000))*(r02)*(x665))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x658)))+(((IkReal(-0.0750000000000000))*(x663)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(x656)))+(((px)*(x664)))+(((cj0)*(px)*(x658)))+(((IkReal(-1.00000000000000))*(pz)*(x665)))+(((x655)*(x657)))+(((x659)*(x663)))+(((x658)*(x659))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x666=((cj1)*(sj2));
IkReal x667=((r01)*(sj0));
IkReal x668=((cj0)*(r00));
IkReal x669=((cj2)*(sj1));
if( IKabs(((gconst59)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((x666)*(x668)))+(((x666)*(x667)))+(((x667)*(x669)))+(((x668)*(x669)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst59)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((gconst59)*(((((x666)*(x668)))+(((x666)*(x667)))+(((x667)*(x669)))+(((x668)*(x669)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x670=IKsin(j3);
IkReal x671=IKcos(j3);
IkReal x672=((sj1)*(sj2));
IkReal x673=((IkReal(0.0800000000000000))*(sj4));
IkReal x674=((cj2)*(sj1));
IkReal x675=((py)*(sj0));
IkReal x676=((IkReal(1.00000000000000))*(cj0));
IkReal x677=((IkReal(1.00000000000000))*(cj4));
IkReal x678=((r01)*(sj0));
IkReal x679=((cj1)*(sj2));
IkReal x680=((cj0)*(x679));
IkReal x681=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x670)*(x673)))+(((IkReal(-1.00000000000000))*(py)*(x676)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x670)*(x677)))+(((IkReal(-1.00000000000000))*(r01)*(x676))));
evalcond[2]=((((r02)*(x672)))+(((x674)*(x678)))+(((IkReal(-1.00000000000000))*(r02)*(x681)))+(((x678)*(x679)))+(((IkReal(-1.00000000000000))*(x671)*(x677)))+(((r00)*(x680)))+(((cj0)*(r00)*(x674))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((px)*(x680)))+(((pz)*(x672)))+(((x671)*(x673)))+(((cj0)*(px)*(x674)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x674)*(x675)))+(((IkReal(-0.0750000000000000))*(x674)))+(((IkReal(-0.0750000000000000))*(x679)))+(((x675)*(x679)))+(((IkReal(-1.00000000000000))*(pz)*(x681))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x682=((cj1)*(cj2));
IkReal x683=((IkReal(1.00000000000000))*(cj0));
IkReal x684=((sj1)*(sj2));
IkReal x685=((IkReal(1.00000000000000))*(sj0));
IkReal x686=((sj0)*(x684));
IkReal x687=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x688=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r02)*(x688)))+(((IkReal(-1.00000000000000))*(r02)*(x687)))+(((IkReal(-1.00000000000000))*(r01)*(x682)*(x685)))+(((cj0)*(r00)*(x684)))+(((IkReal(-1.00000000000000))*(r00)*(x682)*(x683)))+(((r01)*(x686))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(px)*(x682)*(x683)))+(((IkReal(0.0750000000000000))*(x682)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x684)))+(((IkReal(-1.00000000000000))*(py)*(x682)*(x685)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(pz)*(x687)))+(((IkReal(-1.00000000000000))*(pz)*(x688)))+(((py)*(x686)))+(((cj0)*(px)*(x684))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x689=((IkReal(1.00000000000000))*(sj2));
IkReal x690=((cj0)*(r00));
IkReal x691=((r01)*(sj0));
IkReal x692=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(cj1)*(x689)*(x691)))+(((IkReal(-1.00000000000000))*(cj1)*(x689)*(x690)))+(((IkReal(-1.00000000000000))*(x690)*(x692)))+(((IkReal(-1.00000000000000))*(x691)*(x692)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x689))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst60)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst60)*(((((IkReal(-1.00000000000000))*(cj1)*(x689)*(x691)))+(((IkReal(-1.00000000000000))*(cj1)*(x689)*(x690)))+(((IkReal(-1.00000000000000))*(x690)*(x692)))+(((IkReal(-1.00000000000000))*(x691)*(x692)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x689)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x693=IKsin(j3);
IkReal x694=IKcos(j3);
IkReal x695=((sj1)*(sj2));
IkReal x696=((cj2)*(sj1));
IkReal x697=((IkReal(0.0800000000000000))*(sj4));
IkReal x698=((py)*(sj0));
IkReal x699=((IkReal(1.00000000000000))*(cj0));
IkReal x700=((r01)*(sj0));
IkReal x701=((cj1)*(sj2));
IkReal x702=((cj0)*(x701));
IkReal x703=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((x693)*(x697)))+(((IkReal(-1.00000000000000))*(py)*(x699)))+(((px)*(sj0))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x699)))+(((cj4)*(x693))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x703)))+(((cj4)*(x694)))+(((cj0)*(r00)*(x696)))+(((x700)*(x701)))+(((r00)*(x702)))+(((r02)*(x695)))+(((x696)*(x700))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x703)))+(((IkReal(-0.0750000000000000))*(x696)))+(((IkReal(-0.0750000000000000))*(x701)))+(((x694)*(x697)))+(((x696)*(x698)))+(((IkReal(-0.270000000000000))*(cj2)))+(((pz)*(x695)))+(((px)*(x702)))+(((x698)*(x701)))+(((cj0)*(px)*(x696))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x704=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x705=((r01)*(sj0));
IkReal x706=((IkReal(0.500000000000000))*(x704));
IkReal x707=((cj0)*(r00));
IkReal x708=((IkReal(25.0000000000000))*(cj0)*(py));
IkReal x709=((IkReal(25.0000000000000))*(px)*(sj0));
IkReal x710=((IkReal(2.00000000000000))*(cj2)*(sj4));
IkReal x711=((IkReal(2.00000000000000))*(cj1)*(sj2)*(sj4));
if( IKabs(((x706)*(((((IkReal(-1.00000000000000))*(x709)))+(x708))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x706)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((x707)*(x711)))+(((sj5)*(x708)))+(((sj1)*(x705)*(x710)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x710)))+(((IkReal(-1.00000000000000))*(sj5)*(x709)))+(((IkReal(2.00000000000000))*(r02)*(sj1)*(sj2)*(sj4)))+(((sj1)*(x707)*(x710)))+(((x705)*(x711))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x706)*(((((IkReal(-1.00000000000000))*(x709)))+(x708)))))+IKsqr(((x706)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((x707)*(x711)))+(((sj5)*(x708)))+(((sj1)*(x705)*(x710)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x710)))+(((IkReal(-1.00000000000000))*(sj5)*(x709)))+(((IkReal(2.00000000000000))*(r02)*(sj1)*(sj2)*(sj4)))+(((sj1)*(x707)*(x710)))+(((x705)*(x711)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x706)*(((((IkReal(-1.00000000000000))*(x709)))+(x708)))), ((x706)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((IKabs(cj5) != 0)?((IkReal)1/(cj5)):(IkReal)1.0e30))*(((((x707)*(x711)))+(((sj5)*(x708)))+(((sj1)*(x705)*(x710)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x710)))+(((IkReal(-1.00000000000000))*(sj5)*(x709)))+(((IkReal(2.00000000000000))*(r02)*(sj1)*(sj2)*(sj4)))+(((sj1)*(x707)*(x710)))+(((x705)*(x711)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x712=IKsin(j3);
IkReal x713=IKcos(j3);
IkReal x714=((sj1)*(sj2));
IkReal x715=((IkReal(0.0800000000000000))*(sj4));
IkReal x716=((cj2)*(sj1));
IkReal x717=((py)*(sj0));
IkReal x718=((IkReal(1.00000000000000))*(cj0));
IkReal x719=((r01)*(sj0));
IkReal x720=((cj1)*(sj2));
IkReal x721=((cj0)*(x720));
IkReal x722=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x723=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((px)*(sj0)))+(((x712)*(x715)))+(((IkReal(-1.00000000000000))*(py)*(x718))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x713)))+(((IkReal(-1.00000000000000))*(r01)*(x718)))+(((IkReal(-1.00000000000000))*(x712)*(x722))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x713)*(x722)))+(((IkReal(-1.00000000000000))*(r02)*(x723)))+(((sj5)*(x712)))+(((r00)*(x721)))+(((cj0)*(r00)*(x716)))+(((x719)*(x720)))+(((x716)*(x719)))+(((r02)*(x714))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x720)))+(((IkReal(-0.0750000000000000))*(x716)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x717)*(x720)))+(((cj0)*(px)*(x716)))+(((px)*(x721)))+(((x716)*(x717)))+(((pz)*(x714)))+(((IkReal(-1.00000000000000))*(pz)*(x723)))+(((x713)*(x715))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x724=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x725=((IkReal(2.00000000000000))*(sj4));
IkReal x726=((cj4)*(cj5));
IkReal x727=((IkReal(0.500000000000000))*(x724));
IkReal x728=((IkReal(25.0000000000000))*(cj0)*(py));
IkReal x729=((IkReal(25.0000000000000))*(px)*(sj0));
if( IKabs(((x727)*(((((IkReal(-1.00000000000000))*(x729)))+(x728))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x727)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x726)*(x729)))+(((r00)*(sj0)*(x725)))+(((IkReal(-1.00000000000000))*(x726)*(x728)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x725))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x727)*(((((IkReal(-1.00000000000000))*(x729)))+(x728)))))+IKsqr(((x727)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x726)*(x729)))+(((r00)*(sj0)*(x725)))+(((IkReal(-1.00000000000000))*(x726)*(x728)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x725)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x727)*(((((IkReal(-1.00000000000000))*(x729)))+(x728)))), ((x727)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x726)*(x729)))+(((r00)*(sj0)*(x725)))+(((IkReal(-1.00000000000000))*(x726)*(x728)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(x725)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x730=IKsin(j3);
IkReal x731=IKcos(j3);
IkReal x732=((sj1)*(sj2));
IkReal x733=((IkReal(0.0800000000000000))*(sj4));
IkReal x734=((cj2)*(sj1));
IkReal x735=((py)*(sj0));
IkReal x736=((IkReal(1.00000000000000))*(cj0));
IkReal x737=((r01)*(sj0));
IkReal x738=((cj1)*(sj2));
IkReal x739=((cj0)*(x738));
IkReal x740=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x741=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x736)))+(((x730)*(x733))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x736)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x730)*(x740)))+(((IkReal(-1.00000000000000))*(sj5)*(x731))));
evalcond[2]=((((x734)*(x737)))+(((r00)*(x739)))+(((IkReal(-1.00000000000000))*(x731)*(x740)))+(((sj5)*(x730)))+(((cj0)*(r00)*(x734)))+(((IkReal(-1.00000000000000))*(r02)*(x741)))+(((r02)*(x732)))+(((x737)*(x738))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x738)))+(((IkReal(-0.0750000000000000))*(x734)))+(((x731)*(x733)))+(((x735)*(x738)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x734)*(x735)))+(((cj0)*(px)*(x734)))+(((IkReal(-1.00000000000000))*(pz)*(x741)))+(((px)*(x739)))+(((pz)*(x732))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x742=((cj4)*(cj5));
IkReal x743=((r00)*(sj0));
IkReal x744=((cj0)*(r00));
IkReal x745=((r01)*(sj0));
IkReal x746=((IkReal(1.00000000000000))*(cj2));
IkReal x747=((cj1)*(r02));
IkReal x748=((sj1)*(sj5));
IkReal x749=((cj1)*(sj2));
IkReal x750=((cj2)*(sj1));
IkReal x751=((r02)*(sj2));
IkReal x752=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x753=((IkReal(1.00000000000000))*(sj5)*(x749));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x742)*(x752)))+(((x742)*(x743)))+(((IkReal(-1.00000000000000))*(x745)*(x753)))+(((IkReal(-1.00000000000000))*(x744)*(x746)*(x748)))+(((IkReal(-1.00000000000000))*(x744)*(x753)))+(((cj2)*(sj5)*(x747)))+(((IkReal(-1.00000000000000))*(x745)*(x746)*(x748)))+(((IkReal(-1.00000000000000))*(x748)*(x751))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(sj5)*(x752)))+(((sj1)*(x742)*(x751)))+(((IkReal(-1.00000000000000))*(x742)*(x746)*(x747)))+(((x742)*(x745)*(x750)))+(((x742)*(x744)*(x749)))+(((sj5)*(x743)))+(((x742)*(x745)*(x749)))+(((x742)*(x744)*(x750))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(x742)*(x752)))+(((x742)*(x743)))+(((IkReal(-1.00000000000000))*(x745)*(x753)))+(((IkReal(-1.00000000000000))*(x744)*(x746)*(x748)))+(((IkReal(-1.00000000000000))*(x744)*(x753)))+(((cj2)*(sj5)*(x747)))+(((IkReal(-1.00000000000000))*(x745)*(x746)*(x748)))+(((IkReal(-1.00000000000000))*(x748)*(x751)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(sj5)*(x752)))+(((sj1)*(x742)*(x751)))+(((IkReal(-1.00000000000000))*(x742)*(x746)*(x747)))+(((x742)*(x745)*(x750)))+(((x742)*(x744)*(x749)))+(((sj5)*(x743)))+(((x742)*(x745)*(x749)))+(((x742)*(x744)*(x750)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x754=IKsin(j3);
IkReal x755=IKcos(j3);
IkReal x756=((sj1)*(sj2));
IkReal x757=((IkReal(0.0800000000000000))*(sj4));
IkReal x758=((cj2)*(sj1));
IkReal x759=((py)*(sj0));
IkReal x760=((IkReal(1.00000000000000))*(cj0));
IkReal x761=((r01)*(sj0));
IkReal x762=((cj1)*(sj2));
IkReal x763=((cj0)*(x762));
IkReal x764=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x765=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x760)))+(((px)*(sj0)))+(((x754)*(x757))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x760)))+(((IkReal(-1.00000000000000))*(x754)*(x764)))+(((IkReal(-1.00000000000000))*(sj5)*(x755))));
evalcond[2]=((((r02)*(x756)))+(((x761)*(x762)))+(((IkReal(-1.00000000000000))*(x755)*(x764)))+(((x758)*(x761)))+(((sj5)*(x754)))+(((IkReal(-1.00000000000000))*(r02)*(x765)))+(((r00)*(x763)))+(((cj0)*(r00)*(x758))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x762)))+(((pz)*(x756)))+(((px)*(x763)))+(((x758)*(x759)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(px)*(x758)))+(((x759)*(x762)))+(((IkReal(-0.0750000000000000))*(x758)))+(((x755)*(x757)))+(((IkReal(-1.00000000000000))*(pz)*(x765))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x766=((IkReal(25.0000000000000))*(cj0));
IkReal x767=((IkReal(25.0000000000000))*(sj0));
IkReal x768=((cj1)*(sj2));
IkReal x769=((cj2)*(sj1));
IkReal x770=((IkReal(25.0000000000000))*(pz));
if( IKabs(((gconst53)*(((((IkReal(-1.00000000000000))*(px)*(x767)))+(((py)*(x766))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(px)*(x766)*(x769)))+(((IkReal(-1.00000000000000))*(px)*(x766)*(x768)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x770)))+(((IkReal(-1.00000000000000))*(py)*(x767)*(x769)))+(((IkReal(-1.00000000000000))*(py)*(x767)*(x768)))+(((IkReal(6.75000000000000))*(cj2)))+(((cj1)*(cj2)*(x770)))+(((IkReal(1.87500000000000))*(x769)))+(((IkReal(1.87500000000000))*(x768))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst53)*(((((IkReal(-1.00000000000000))*(px)*(x767)))+(((py)*(x766)))))), ((gconst53)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(px)*(x766)*(x769)))+(((IkReal(-1.00000000000000))*(px)*(x766)*(x768)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x770)))+(((IkReal(-1.00000000000000))*(py)*(x767)*(x769)))+(((IkReal(-1.00000000000000))*(py)*(x767)*(x768)))+(((IkReal(6.75000000000000))*(cj2)))+(((cj1)*(cj2)*(x770)))+(((IkReal(1.87500000000000))*(x769)))+(((IkReal(1.87500000000000))*(x768)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x771=((cj1)*(sj2));
IkReal x772=((IkReal(0.0800000000000000))*(sj4));
IkReal x773=((cj2)*(sj1));
IkReal x774=((cj0)*(px));
IkReal x775=((py)*(sj0));
evalcond[0]=((((px)*(sj0)))+(((x772)*(IKsin(j3))))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x771)*(x774)))+(((x771)*(x775)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x773)))+(((IkReal(-0.0750000000000000))*(x771)))+(((x773)*(x774)))+(((x773)*(x775)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((x772)*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x776=(sj3)*(sj3);
IkReal x777=(cj3)*(cj3);
dummyeval[0]=((x777)+(x776));
dummyeval[1]=((((cj4)*(x776)))+(((cj4)*(x777))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x778=((pz)*(sj2));
IkReal x779=((IkReal(1.00000000000000))*(cj1));
IkReal x780=((cj2)*(sj1));
IkReal x781=((py)*(sj0));
IkReal x782=((cj0)*(px));
IkReal x783=((sj1)*(sj2));
IkReal x784=((IkReal(0.0750000000000000))*(cj1));
IkReal x785=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((x781)*(x785)))+(((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x782)*(x785)))+(((sj1)*(x778)))+(((x780)*(x782)))+(((x780)*(x781)))+(((IkReal(-1.00000000000000))*(sj2)*(x784)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x779)))+(((IkReal(-0.0750000000000000))*(x780))));
evalcond[3]=((IkReal(0.295000000000000))+(((x781)*(x783)))+(((IkReal(-1.00000000000000))*(x778)*(x779)))+(((x782)*(x783)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x780)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x784)))+(((IkReal(-1.00000000000000))*(cj2)*(x779)*(x781)))+(((IkReal(-1.00000000000000))*(cj2)*(x779)*(x782)))+(((IkReal(-0.0750000000000000))*(x783))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x786=((pz)*(sj2));
IkReal x787=((IkReal(1.00000000000000))*(cj1));
IkReal x788=((cj2)*(sj0));
IkReal x789=((py)*(sj1));
IkReal x790=((cj0)*(sj1));
IkReal x791=((cj2)*(px));
IkReal x792=((sj0)*(sj2));
IkReal x793=((IkReal(0.0750000000000000))*(cj2));
IkReal x794=((px)*(sj2));
IkReal x795=((IkReal(0.0750000000000000))*(sj2));
IkReal x796=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x792)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x787)))+(((IkReal(-1.00000000000000))*(r01)*(x787)*(x788)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x787)))+(((r00)*(sj2)*(x790)))+(((IkReal(-1.00000000000000))*(r02)*(x796))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((x790)*(x791)))+(((sj1)*(x786)))+(((x788)*(x789)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x787)))+(((IkReal(-1.00000000000000))*(cj1)*(x795)))+(((cj0)*(cj1)*(x794)))+(((cj1)*(py)*(x792)))+(((IkReal(-1.00000000000000))*(sj1)*(x793))));
evalcond[4]=((IkReal(0.375000000000000))+(((x790)*(x794)))+(((cj1)*(x793)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x787)*(x788)))+(((IkReal(-1.00000000000000))*(x786)*(x787)))+(((IkReal(-1.00000000000000))*(pz)*(x796)))+(((x789)*(x792)))+(((IkReal(-1.00000000000000))*(sj1)*(x795)))+(((IkReal(-1.00000000000000))*(cj0)*(x787)*(x791))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x797=((cj1)*(sj2));
IkReal x798=((r01)*(sj0));
IkReal x799=((cj0)*(r00));
IkReal x800=((cj2)*(sj1));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x798)*(x800)))+(((x799)*(x800)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x797)*(x798)))+(((x797)*(x799))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))+IKsqr(((((x798)*(x800)))+(((x799)*(x800)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x797)*(x798)))+(((x797)*(x799)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))), ((((x798)*(x800)))+(((x799)*(x800)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x797)*(x798)))+(((x797)*(x799)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x801=((cj1)*(sj2));
IkReal x802=((r01)*(sj0));
IkReal x803=((cj0)*(r00));
IkReal x804=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((x803)*(x804)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x801)*(x803)))+(((x801)*(x802)))+(((r02)*(sj1)*(sj2)))+(((x802)*(x804))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x805=((pz)*(sj2));
IkReal x806=((IkReal(1.00000000000000))*(cj1));
IkReal x807=((cj2)*(sj0));
IkReal x808=((py)*(sj1));
IkReal x809=((cj0)*(sj1));
IkReal x810=((cj2)*(px));
IkReal x811=((sj0)*(sj2));
IkReal x812=((IkReal(0.0750000000000000))*(cj2));
IkReal x813=((px)*(sj2));
IkReal x814=((IkReal(0.0750000000000000))*(sj2));
IkReal x815=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x806)*(x807)))+(((r01)*(sj1)*(x811)))+(((r00)*(sj2)*(x809)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x806)))+(((IkReal(-1.00000000000000))*(r02)*(x815)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x806))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x809)*(x810)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj1)*(py)*(x811)))+(((cj0)*(cj1)*(x813)))+(((sj1)*(x805)))+(((IkReal(-1.00000000000000))*(cj1)*(x814)))+(((IkReal(-1.00000000000000))*(sj1)*(x812)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x806)))+(((x807)*(x808))));
evalcond[4]=((IkReal(0.215000000000000))+(((x809)*(x813)))+(((IkReal(-1.00000000000000))*(x805)*(x806)))+(((IkReal(-1.00000000000000))*(pz)*(x815)))+(((IkReal(-1.00000000000000))*(cj0)*(x806)*(x810)))+(((x808)*(x811)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj1)*(x812)))+(((IkReal(-1.00000000000000))*(sj1)*(x814)))+(((IkReal(-1.00000000000000))*(py)*(x806)*(x807))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x816=((IkReal(1.00000000000000))*(sj1));
IkReal x817=((IkReal(1.00000000000000))*(cj0));
IkReal x818=((r01)*(sj0));
IkReal x819=((cj1)*(sj2));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x817))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x818)*(x819)))+(((IkReal(-1.00000000000000))*(cj2)*(x816)*(x818)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x816)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r00)*(x817)*(x819)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x816))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x817)))))+IKsqr(((((IkReal(-1.00000000000000))*(x818)*(x819)))+(((IkReal(-1.00000000000000))*(cj2)*(x816)*(x818)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x816)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r00)*(x817)*(x819)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x816)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x817)))), ((((IkReal(-1.00000000000000))*(x818)*(x819)))+(((IkReal(-1.00000000000000))*(cj2)*(x816)*(x818)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x816)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r00)*(x817)*(x819)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x816)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x820=((cj1)*(sj2));
IkReal x821=((r01)*(sj0));
IkReal x822=((cj0)*(r00));
IkReal x823=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((x820)*(x821)))+(((x820)*(x822)))+(((x821)*(x823)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x822)*(x823)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x824=((pz)*(sj2));
IkReal x825=((IkReal(1.00000000000000))*(cj1));
IkReal x826=((cj2)*(sj1));
IkReal x827=((py)*(sj0));
IkReal x828=((cj0)*(px));
IkReal x829=((sj1)*(sj2));
IkReal x830=((IkReal(0.0750000000000000))*(cj1));
IkReal x831=((r01)*(sj0));
IkReal x832=((cj1)*(sj2));
IkReal x833=((cj0)*(x832));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x825)))+(((x831)*(x832)))+(((x826)*(x831)))+(((r00)*(x833)))+(((r02)*(x829)))+(((cj0)*(r00)*(x826))));
evalcond[3]=((IkReal(-0.0100000000000000))+(((IkReal(-0.0750000000000000))*(x826)))+(((sj1)*(x824)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x827)*(x832)))+(((x828)*(x832)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x825)))+(((x826)*(x827)))+(((x826)*(x828)))+(((IkReal(-1.00000000000000))*(sj2)*(x830))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x829)))+(((IkReal(-1.00000000000000))*(x824)*(x825)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj2)*(x830)))+(((x828)*(x829)))+(((IkReal(-1.00000000000000))*(pz)*(x826)))+(((IkReal(-1.00000000000000))*(cj2)*(x825)*(x828)))+(((IkReal(-1.00000000000000))*(cj2)*(x825)*(x827)))+(((x827)*(x829))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x834=((IkReal(1.00000000000000))*(cj0));
IkReal x835=((cj1)*(cj2));
IkReal x836=((sj1)*(sj2));
IkReal x837=((r01)*(sj0));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x834))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x835)*(x837)))+(((cj0)*(r00)*(x835)))+(((IkReal(-1.00000000000000))*(r00)*(x834)*(x836)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x836)*(x837)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x834)))))+IKsqr(((((x835)*(x837)))+(((cj0)*(r00)*(x835)))+(((IkReal(-1.00000000000000))*(r00)*(x834)*(x836)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x836)*(x837)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x834)))), ((((x835)*(x837)))+(((cj0)*(r00)*(x835)))+(((IkReal(-1.00000000000000))*(r00)*(x834)*(x836)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x836)*(x837)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x838=((r01)*(sj0));
IkReal x839=((sj1)*(sj2));
IkReal x840=((IkReal(1.00000000000000))*(cj0));
IkReal x841=((cj1)*(cj2));
IkReal x842=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x840)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((x838)*(x839)))+(((IkReal(-1.00000000000000))*(r00)*(x840)*(x841)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x842)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x842)))+(((cj0)*(r00)*(x839)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(x838)*(x841))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x843=((pz)*(sj2));
IkReal x844=((IkReal(1.00000000000000))*(cj1));
IkReal x845=((cj2)*(sj1));
IkReal x846=((py)*(sj0));
IkReal x847=((cj0)*(px));
IkReal x848=((sj1)*(sj2));
IkReal x849=((IkReal(0.0750000000000000))*(cj1));
IkReal x850=((r01)*(sj0));
IkReal x851=((cj1)*(sj2));
IkReal x852=((cj0)*(x851));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((x850)*(x851)))+(((r00)*(x852)))+(((x845)*(x850)))+(((r02)*(x848)))+(((cj0)*(r00)*(x845)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x844))));
evalcond[3]=((IkReal(-0.170000000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x849)))+(((IkReal(-0.0750000000000000))*(x845)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x847)*(x851)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x844)))+(((x845)*(x846)))+(((x845)*(x847)))+(((x846)*(x851)))+(((sj1)*(x843))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-0.0750000000000000))*(x848)))+(((x846)*(x848)))+(((IkReal(-1.00000000000000))*(x843)*(x844)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x844)*(x847)))+(((IkReal(-1.00000000000000))*(cj2)*(x844)*(x846)))+(((cj2)*(x849)))+(((x847)*(x848)))+(((IkReal(-1.00000000000000))*(pz)*(x845))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x853=((r01)*(sj0));
IkReal x854=((sj1)*(sj2));
IkReal x855=((IkReal(1.00000000000000))*(cj0));
IkReal x856=((cj1)*(cj2));
IkReal x857=((IkReal(1.00000000000000))*(r02));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x855))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(x855)*(x856)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x857)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x857)))+(((cj0)*(r00)*(x854)))+(((x853)*(x854)))+(((IkReal(-1.00000000000000))*(x853)*(x856))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x855)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(x855)*(x856)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x857)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x857)))+(((cj0)*(r00)*(x854)))+(((x853)*(x854)))+(((IkReal(-1.00000000000000))*(x853)*(x856)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x855)))), ((((IkReal(-1.00000000000000))*(r00)*(x855)*(x856)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x857)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x857)))+(((cj0)*(r00)*(x854)))+(((x853)*(x854)))+(((IkReal(-1.00000000000000))*(x853)*(x856)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x858=((r01)*(sj0));
IkReal x859=((sj1)*(sj2));
IkReal x860=((IkReal(1.00000000000000))*(cj0));
IkReal x861=((cj1)*(cj2));
IkReal x862=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x860)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(r00)*(x860)*(x861)))+(((IkReal(-1.00000000000000))*(x858)*(x861)))+(((x858)*(x859)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x862)))+(((cj0)*(r00)*(x859)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x862))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x863=((IkReal(1.00000000000000))*(cj0));
IkReal x864=((cj1)*(cj2));
IkReal x865=((sj1)*(sj2));
IkReal x866=((r01)*(sj0));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x863))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(x863)*(x865)))+(((cj2)*(r02)*(sj1)))+(((cj0)*(r00)*(x864)))+(((IkReal(-1.00000000000000))*(x865)*(x866)))+(((x864)*(x866)))+(((cj1)*(r02)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x863)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(x863)*(x865)))+(((cj2)*(r02)*(sj1)))+(((cj0)*(r00)*(x864)))+(((IkReal(-1.00000000000000))*(x865)*(x866)))+(((x864)*(x866)))+(((cj1)*(r02)*(sj2)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x863)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(x863)*(x865)))+(((cj2)*(r02)*(sj1)))+(((cj0)*(r00)*(x864)))+(((IkReal(-1.00000000000000))*(x865)*(x866)))+(((x864)*(x866)))+(((cj1)*(r02)*(sj2)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x867=IKcos(j5);
IkReal x868=((r01)*(sj0));
IkReal x869=((sj1)*(sj2));
IkReal x870=((IkReal(1.00000000000000))*(cj0));
IkReal x871=((cj1)*(cj2));
IkReal x872=((cj1)*(sj2));
IkReal x873=((cj0)*(r00));
IkReal x874=((cj2)*(sj1));
IkReal x875=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x870)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x872)*(x875)))+(((x868)*(x869)))+(((IkReal(-1.00000000000000))*(x868)*(x871)))+(((sj4)*(x867)))+(((IkReal(-1.00000000000000))*(r00)*(x870)*(x871)))+(((x869)*(x873)))+(((IkReal(-1.00000000000000))*(x874)*(x875))));
evalcond[2]=((((x872)*(x873)))+(((x868)*(x874)))+(((x868)*(x872)))+(((IkReal(-1.00000000000000))*(x871)*(x875)))+(((x873)*(x874)))+(((r02)*(x869)))+(((IkReal(-1.00000000000000))*(cj4)*(x867))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x876=((cj1)*(sj2));
IkReal x877=((r01)*(sj0));
IkReal x878=((cj0)*(r00));
IkReal x879=((cj2)*(sj1));
if( IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x876)*(x878)))+(((x876)*(x877)))+(((x878)*(x879)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x877)*(x879))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x876)*(x878)))+(((x876)*(x877)))+(((x878)*(x879)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x877)*(x879)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((x876)*(x878)))+(((x876)*(x877)))+(((x878)*(x879)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x877)*(x879)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x880=IKcos(j5);
IkReal x881=((r01)*(sj0));
IkReal x882=((sj1)*(sj2));
IkReal x883=((IkReal(1.00000000000000))*(cj0));
IkReal x884=((cj1)*(cj2));
IkReal x885=((cj1)*(sj2));
IkReal x886=((cj0)*(r00));
IkReal x887=((cj2)*(sj1));
IkReal x888=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((IkReal(-1.00000000000000))*(r01)*(x883))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x887)*(x888)))+(((sj4)*(x880)))+(((x881)*(x882)))+(((IkReal(-1.00000000000000))*(x881)*(x884)))+(((x882)*(x886)))+(((IkReal(-1.00000000000000))*(r00)*(x883)*(x884)))+(((IkReal(-1.00000000000000))*(x885)*(x888))));
evalcond[2]=((((x886)*(x887)))+(((r02)*(x882)))+(((x881)*(x885)))+(((x881)*(x887)))+(((IkReal(-1.00000000000000))*(cj4)*(x880)))+(((x885)*(x886)))+(((IkReal(-1.00000000000000))*(x884)*(x888))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x889=((pz)*(sj2));
IkReal x890=((IkReal(1.00000000000000))*(cj1));
IkReal x891=((cj2)*(sj1));
IkReal x892=((py)*(sj0));
IkReal x893=((cj0)*(px));
IkReal x894=((sj1)*(sj2));
IkReal x895=((IkReal(0.0750000000000000))*(cj1));
IkReal x896=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x891)))+(((sj1)*(x889)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x890)))+(((x893)*(x896)))+(((x892)*(x896)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((x891)*(x892)))+(((x891)*(x893)))+(((IkReal(-1.00000000000000))*(sj2)*(x895))));
evalcond[3]=((IkReal(0.295000000000000))+(((cj2)*(x895)))+(((IkReal(-0.0750000000000000))*(x894)))+(((IkReal(-1.00000000000000))*(cj2)*(x890)*(x892)))+(((IkReal(-1.00000000000000))*(cj2)*(x890)*(x893)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x889)*(x890)))+(((x893)*(x894)))+(((x892)*(x894)))+(((IkReal(-1.00000000000000))*(pz)*(x891)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x897=((pz)*(sj2));
IkReal x898=((IkReal(1.00000000000000))*(cj1));
IkReal x899=((cj2)*(sj1));
IkReal x900=((py)*(sj0));
IkReal x901=((cj0)*(px));
IkReal x902=((sj1)*(sj2));
IkReal x903=((IkReal(0.0750000000000000))*(cj1));
IkReal x904=((r01)*(sj0));
IkReal x905=((cj1)*(sj2));
IkReal x906=((cj0)*(x905));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((cj0)*(r00)*(x899)))+(((r00)*(x906)))+(((r02)*(x902)))+(((x899)*(x904)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x898)))+(((x904)*(x905))));
evalcond[3]=((IkReal(-0.170000000000000))+(((IkReal(-0.0750000000000000))*(x899)))+(((sj1)*(x897)))+(((x901)*(x905)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x903)))+(((x900)*(x905)))+(((x899)*(x901)))+(((x899)*(x900)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x898))));
evalcond[4]=((IkReal(0.295000000000000))+(((x901)*(x902)))+(((cj2)*(x903)))+(((x900)*(x902)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x898)*(x900)))+(((IkReal(-1.00000000000000))*(cj2)*(x898)*(x901)))+(((IkReal(-1.00000000000000))*(pz)*(x899)))+(((IkReal(-1.00000000000000))*(x897)*(x898)))+(((IkReal(-0.0750000000000000))*(x902))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x907=((cj0)*(r00));
IkReal x908=((cj1)*(cj2));
IkReal x909=((r01)*(sj0));
IkReal x910=((IkReal(1.00000000000000))*(sj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x907)*(x908)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x907)*(x910)))+(((x908)*(x909)))+(((IkReal(-1.00000000000000))*(x909)*(x910)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((x907)*(x908)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x907)*(x910)))+(((x908)*(x909)))+(((IkReal(-1.00000000000000))*(x909)*(x910)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((x907)*(x908)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x907)*(x910)))+(((x908)*(x909)))+(((IkReal(-1.00000000000000))*(x909)*(x910)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x911=((r01)*(sj0));
IkReal x912=((sj1)*(sj2));
IkReal x913=((IkReal(1.00000000000000))*(cj0));
IkReal x914=((cj1)*(cj2));
IkReal x915=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x913)))+(IKsin(j5)));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x915)))+(((cj0)*(r00)*(x912)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x915)))+(((x911)*(x912)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(r00)*(x913)*(x914)))+(((IkReal(-1.00000000000000))*(x911)*(x914))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x916=((pz)*(sj2));
IkReal x917=((IkReal(1.00000000000000))*(cj1));
IkReal x918=((cj2)*(sj1));
IkReal x919=((py)*(sj0));
IkReal x920=((cj0)*(px));
IkReal x921=((sj1)*(sj2));
IkReal x922=((IkReal(0.0750000000000000))*(cj1));
IkReal x923=((r01)*(sj0));
IkReal x924=((cj1)*(sj2));
IkReal x925=((cj0)*(x924));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((x918)*(x923)))+(((cj0)*(r00)*(x918)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x917)))+(((x923)*(x924)))+(((r00)*(x925)))+(((r02)*(x921))));
evalcond[3]=((IkReal(-0.0100000000000000))+(((x918)*(x920)))+(((x919)*(x924)))+(((IkReal(-1.00000000000000))*(sj2)*(x922)))+(((x920)*(x924)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x917)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x918)*(x919)))+(((IkReal(-0.0750000000000000))*(x918)))+(((sj1)*(x916))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x917)*(x920)))+(((x919)*(x921)))+(((x920)*(x921)))+(((IkReal(-0.0750000000000000))*(x921)))+(((IkReal(-0.270000000000000))*(sj2)))+(((cj2)*(x922)))+(((IkReal(-1.00000000000000))*(cj2)*(x917)*(x919)))+(((IkReal(-1.00000000000000))*(pz)*(x918)))+(((IkReal(-1.00000000000000))*(x916)*(x917))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x926=((r01)*(sj0));
IkReal x927=((sj1)*(sj2));
IkReal x928=((IkReal(1.00000000000000))*(cj1));
IkReal x929=((cj0)*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj2)*(x926)*(x928)))+(((x927)*(x929)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x926)*(x927)))+(((IkReal(-1.00000000000000))*(cj2)*(x928)*(x929)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x928))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(x926)*(x928)))+(((x927)*(x929)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x926)*(x927)))+(((IkReal(-1.00000000000000))*(cj2)*(x928)*(x929)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x928)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(cj2)*(x926)*(x928)))+(((x927)*(x929)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x926)*(x927)))+(((IkReal(-1.00000000000000))*(cj2)*(x928)*(x929)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x928)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x930=((r01)*(sj0));
IkReal x931=((sj1)*(sj2));
IkReal x932=((IkReal(1.00000000000000))*(cj0));
IkReal x933=((cj1)*(cj2));
IkReal x934=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(r01)*(x932))));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((x930)*(x931)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x934)))+(((cj0)*(r00)*(x931)))+(((IkReal(-1.00000000000000))*(x930)*(x933)))+(((IkReal(-1.00000000000000))*(r00)*(x932)*(x933)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x934))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x935=((pz)*(sj2));
IkReal x936=((IkReal(1.00000000000000))*(cj1));
IkReal x937=((cj2)*(sj0));
IkReal x938=((py)*(sj1));
IkReal x939=((cj0)*(sj1));
IkReal x940=((cj2)*(px));
IkReal x941=((sj0)*(sj2));
IkReal x942=((IkReal(0.0750000000000000))*(cj2));
IkReal x943=((px)*(sj2));
IkReal x944=((IkReal(0.0750000000000000))*(sj2));
IkReal x945=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x945)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x936)))+(((IkReal(-1.00000000000000))*(r01)*(x936)*(x937)))+(((r01)*(sj1)*(x941)))+(((r00)*(sj2)*(x939)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x936))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x941)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x936)))+(((x937)*(x938)))+(((IkReal(-1.00000000000000))*(cj1)*(x944)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x935)))+(((cj0)*(cj1)*(x943)))+(((x939)*(x940)))+(((IkReal(-1.00000000000000))*(sj1)*(x942))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(x935)*(x936)))+(((IkReal(-1.00000000000000))*(py)*(x936)*(x937)))+(((IkReal(-1.00000000000000))*(pz)*(x945)))+(((x938)*(x941)))+(((x939)*(x943)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x936)*(x940)))+(((cj1)*(x942)))+(((IkReal(-1.00000000000000))*(sj1)*(x944))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x946=((IkReal(1.00000000000000))*(sj2));
IkReal x947=((cj0)*(r00));
IkReal x948=((r01)*(sj0));
IkReal x949=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x947)*(x949)))+(((IkReal(-1.00000000000000))*(x948)*(x949)))+(((IkReal(-1.00000000000000))*(cj1)*(x946)*(x947)))+(((IkReal(-1.00000000000000))*(cj1)*(x946)*(x948)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x946))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(x947)*(x949)))+(((IkReal(-1.00000000000000))*(x948)*(x949)))+(((IkReal(-1.00000000000000))*(cj1)*(x946)*(x947)))+(((IkReal(-1.00000000000000))*(cj1)*(x946)*(x948)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x946)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IkReal(-1.00000000000000))*(x947)*(x949)))+(((IkReal(-1.00000000000000))*(x948)*(x949)))+(((IkReal(-1.00000000000000))*(cj1)*(x946)*(x947)))+(((IkReal(-1.00000000000000))*(cj1)*(x946)*(x948)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x946)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x950=((cj1)*(sj2));
IkReal x951=((r01)*(sj0));
IkReal x952=((cj0)*(r00));
IkReal x953=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x951)*(x953)))+(((x952)*(x953)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x950)*(x952)))+(((x950)*(x951))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x954=((pz)*(sj2));
IkReal x955=((IkReal(1.00000000000000))*(cj1));
IkReal x956=((cj2)*(sj0));
IkReal x957=((py)*(sj1));
IkReal x958=((cj0)*(sj1));
IkReal x959=((cj2)*(px));
IkReal x960=((sj0)*(sj2));
IkReal x961=((IkReal(0.0750000000000000))*(cj2));
IkReal x962=((px)*(sj2));
IkReal x963=((IkReal(0.0750000000000000))*(sj2));
IkReal x964=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x955)))+(((IkReal(-1.00000000000000))*(r01)*(x955)*(x956)))+(((r01)*(sj1)*(x960)))+(((r00)*(sj2)*(x958)))+(((IkReal(-1.00000000000000))*(r02)*(x964)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x955))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x960)))+(((IkReal(-1.00000000000000))*(cj1)*(x963)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x962)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x955)))+(((x956)*(x957)))+(((sj1)*(x954)))+(((IkReal(-1.00000000000000))*(sj1)*(x961)))+(((x958)*(x959))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x964)))+(((cj1)*(x961)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x957)*(x960)))+(((IkReal(-1.00000000000000))*(sj1)*(x963)))+(((x958)*(x962)))+(((IkReal(-1.00000000000000))*(x954)*(x955)))+(((IkReal(-1.00000000000000))*(cj0)*(x955)*(x959)))+(((IkReal(-1.00000000000000))*(py)*(x955)*(x956))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x965=((cj1)*(sj2));
IkReal x966=((r01)*(sj0));
IkReal x967=((cj0)*(r00));
IkReal x968=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x967)*(x968)))+(((x965)*(x966)))+(((x965)*(x967)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x966)*(x968)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((x967)*(x968)))+(((x965)*(x966)))+(((x965)*(x967)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x966)*(x968)))+(((r02)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((x967)*(x968)))+(((x965)*(x966)))+(((x965)*(x967)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x966)*(x968)))+(((r02)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x969=((cj1)*(sj2));
IkReal x970=((r01)*(sj0));
IkReal x971=((cj0)*(r00));
IkReal x972=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x969)*(x970)))+(((x969)*(x971)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x971)*(x972)))+(((x970)*(x972)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x973=((IkReal(1.00000000000000))*(sj2));
IkReal x974=((cj0)*(r00));
IkReal x975=((r01)*(sj0));
IkReal x976=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x974)*(x976)))+(((IkReal(-1.00000000000000))*(cj1)*(x973)*(x975)))+(((IkReal(-1.00000000000000))*(cj1)*(x973)*(x974)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x973)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x975)*(x976))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x974)*(x976)))+(((IkReal(-1.00000000000000))*(cj1)*(x973)*(x975)))+(((IkReal(-1.00000000000000))*(cj1)*(x973)*(x974)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x973)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x975)*(x976)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x974)*(x976)))+(((IkReal(-1.00000000000000))*(cj1)*(x973)*(x975)))+(((IkReal(-1.00000000000000))*(cj1)*(x973)*(x974)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x973)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x975)*(x976)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x977=IKcos(j5);
IkReal x978=((r01)*(sj0));
IkReal x979=((sj1)*(sj2));
IkReal x980=((IkReal(1.00000000000000))*(cj0));
IkReal x981=((cj1)*(cj2));
IkReal x982=((cj1)*(sj2));
IkReal x983=((cj0)*(r00));
IkReal x984=((cj2)*(sj1));
IkReal x985=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x980)))+(IKsin(j5)));
evalcond[1]=((((x978)*(x979)))+(((IkReal(-1.00000000000000))*(x978)*(x981)))+(((x979)*(x983)))+(((sj4)*(x977)))+(((IkReal(-1.00000000000000))*(x982)*(x985)))+(((IkReal(-1.00000000000000))*(x984)*(x985)))+(((IkReal(-1.00000000000000))*(r00)*(x980)*(x981))));
evalcond[2]=((((x983)*(x984)))+(((cj4)*(x977)))+(((x982)*(x983)))+(((IkReal(-1.00000000000000))*(x981)*(x985)))+(((r02)*(x979)))+(((x978)*(x984)))+(((x978)*(x982))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x986=((cj0)*(r00));
IkReal x987=((cj1)*(cj2));
IkReal x988=((r01)*(sj0));
IkReal x989=((IkReal(1.00000000000000))*(sj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x986)*(x987)))+(((IkReal(-1.00000000000000))*(x988)*(x989)))+(((IkReal(-1.00000000000000))*(x986)*(x989)))+(((cj2)*(r02)*(sj1)))+(((x987)*(x988)))+(((cj1)*(r02)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x986)*(x987)))+(((IkReal(-1.00000000000000))*(x988)*(x989)))+(((IkReal(-1.00000000000000))*(x986)*(x989)))+(((cj2)*(r02)*(sj1)))+(((x987)*(x988)))+(((cj1)*(r02)*(sj2)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x986)*(x987)))+(((IkReal(-1.00000000000000))*(x988)*(x989)))+(((IkReal(-1.00000000000000))*(x986)*(x989)))+(((cj2)*(r02)*(sj1)))+(((x987)*(x988)))+(((cj1)*(r02)*(sj2)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x990=IKcos(j5);
IkReal x991=((r01)*(sj0));
IkReal x992=((sj1)*(sj2));
IkReal x993=((IkReal(1.00000000000000))*(cj0));
IkReal x994=((cj1)*(cj2));
IkReal x995=((cj1)*(sj2));
IkReal x996=((cj0)*(r00));
IkReal x997=((cj2)*(sj1));
IkReal x998=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(r01)*(x993))));
evalcond[1]=((((sj4)*(x990)))+(((x991)*(x992)))+(((IkReal(-1.00000000000000))*(x991)*(x994)))+(((x992)*(x996)))+(((IkReal(-1.00000000000000))*(x995)*(x998)))+(((IkReal(-1.00000000000000))*(x997)*(x998)))+(((IkReal(-1.00000000000000))*(r00)*(x993)*(x994))));
evalcond[2]=((((x991)*(x995)))+(((x991)*(x997)))+(((IkReal(-1.00000000000000))*(x994)*(x998)))+(((cj4)*(x990)))+(((r02)*(x992)))+(((x995)*(x996)))+(((x996)*(x997))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x999=((pz)*(sj2));
IkReal x1000=((IkReal(1.00000000000000))*(cj1));
IkReal x1001=((cj2)*(sj0));
IkReal x1002=((py)*(sj1));
IkReal x1003=((cj0)*(sj1));
IkReal x1004=((cj2)*(px));
IkReal x1005=((sj0)*(sj2));
IkReal x1006=((IkReal(0.0750000000000000))*(cj2));
IkReal x1007=((px)*(sj2));
IkReal x1008=((IkReal(0.0750000000000000))*(sj2));
IkReal x1009=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1000)*(x1001)))+(((r00)*(sj2)*(x1003)))+(((IkReal(-1.00000000000000))*(r02)*(x1009)))+(((r01)*(sj1)*(x1005)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1000)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1000))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(cj1)*(x1007)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1000)))+(((IkReal(-1.00000000000000))*(cj1)*(x1008)))+(((IkReal(-1.00000000000000))*(sj1)*(x1006)))+(((cj1)*(py)*(x1005)))+(((sj1)*(x999)))+(((x1001)*(x1002)))+(((x1003)*(x1004))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1009)))+(((IkReal(-1.00000000000000))*(py)*(x1000)*(x1001)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1000)*(x999)))+(((IkReal(-1.00000000000000))*(sj1)*(x1008)))+(((IkReal(-1.00000000000000))*(cj0)*(x1000)*(x1004)))+(((cj1)*(x1006)))+(((x1002)*(x1005)))+(((x1003)*(x1007))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1010=((cj3)*(sj1));
IkReal x1011=((r02)*(sj2));
IkReal x1012=((r01)*(sj0));
IkReal x1013=((IkReal(1.00000000000000))*(cj3));
IkReal x1014=((cj1)*(sj2));
IkReal x1015=((IkReal(1.00000000000000))*(sj3));
IkReal x1016=((cj0)*(r00));
IkReal x1017=((cj0)*(r01));
IkReal x1018=((r00)*(sj0));
IkReal x1019=((cj2)*(x1016));
IkReal x1020=((sj1)*(x1015));
IkReal x1021=((cj1)*(cj2)*(r02));
if( IKabs(((gconst61)*(((((cj3)*(x1018)))+(((IkReal(-1.00000000000000))*(cj2)*(x1012)*(x1020)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)))+(((IkReal(-1.00000000000000))*(x1014)*(x1015)*(x1016)))+(((IkReal(-1.00000000000000))*(x1013)*(x1017)))+(((IkReal(-1.00000000000000))*(x1012)*(x1014)*(x1015)))+(((IkReal(-1.00000000000000))*(x1011)*(x1020)))+(((sj3)*(x1021))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst61)*(((((cj3)*(x1014)*(x1016)))+(((IkReal(-1.00000000000000))*(x1015)*(x1017)))+(((cj3)*(x1012)*(x1014)))+(((cj2)*(x1010)*(x1012)))+(((sj3)*(x1018)))+(((IkReal(-1.00000000000000))*(x1013)*(x1021)))+(((x1010)*(x1011)))+(((x1010)*(x1019))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst61)*(((((cj3)*(x1018)))+(((IkReal(-1.00000000000000))*(cj2)*(x1012)*(x1020)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)))+(((IkReal(-1.00000000000000))*(x1014)*(x1015)*(x1016)))+(((IkReal(-1.00000000000000))*(x1013)*(x1017)))+(((IkReal(-1.00000000000000))*(x1012)*(x1014)*(x1015)))+(((IkReal(-1.00000000000000))*(x1011)*(x1020)))+(((sj3)*(x1021)))))), ((gconst61)*(((((cj3)*(x1014)*(x1016)))+(((IkReal(-1.00000000000000))*(x1015)*(x1017)))+(((cj3)*(x1012)*(x1014)))+(((cj2)*(x1010)*(x1012)))+(((sj3)*(x1018)))+(((IkReal(-1.00000000000000))*(x1013)*(x1021)))+(((x1010)*(x1011)))+(((x1010)*(x1019)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1022=IKsin(j5);
IkReal x1023=IKcos(j5);
IkReal x1024=((IkReal(1.00000000000000))*(cj3));
IkReal x1025=((cj1)*(sj2));
IkReal x1026=((r01)*(sj0));
IkReal x1027=((cj0)*(r00));
IkReal x1028=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1022)*(x1024)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(sj3)*(x1023))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1023)*(x1024)))+(((x1026)*(x1028)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1025)*(x1027)))+(((x1025)*(x1026)))+(((x1027)*(x1028)))+(((r02)*(sj1)*(sj2)))+(((sj3)*(x1022))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1029=((pz)*(sj2));
IkReal x1030=((IkReal(1.00000000000000))*(cj1));
IkReal x1031=((cj2)*(sj0));
IkReal x1032=((py)*(sj1));
IkReal x1033=((cj0)*(sj1));
IkReal x1034=((cj2)*(px));
IkReal x1035=((sj0)*(sj2));
IkReal x1036=((IkReal(0.0750000000000000))*(cj2));
IkReal x1037=((px)*(sj2));
IkReal x1038=((IkReal(0.0750000000000000))*(sj2));
IkReal x1039=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1030)*(x1031)))+(((r00)*(sj2)*(x1033)))+(((IkReal(-1.00000000000000))*(r02)*(x1039)))+(((r01)*(sj1)*(x1035)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1030)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1030))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((sj1)*(x1029)))+(((x1031)*(x1032)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1030)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x1036)))+(((IkReal(-1.00000000000000))*(cj1)*(x1038)))+(((x1033)*(x1034)))+(((cj1)*(py)*(x1035)))+(((cj0)*(cj1)*(x1037))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(x1030)*(x1034)))+(((IkReal(-1.00000000000000))*(pz)*(x1039)))+(((IkReal(-1.00000000000000))*(x1029)*(x1030)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x1038)))+(((x1033)*(x1037)))+(((cj1)*(x1036)))+(((IkReal(-1.00000000000000))*(py)*(x1030)*(x1031)))+(((x1032)*(x1035))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst62;
gconst62=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1040=((cj1)*(sj2));
IkReal x1041=((cj2)*(sj1));
IkReal x1042=((r01)*(sj3));
IkReal x1043=((IkReal(1.00000000000000))*(sj0));
IkReal x1044=((cj1)*(cj2)*(r02));
IkReal x1045=((IkReal(1.00000000000000))*(cj0)*(cj3));
IkReal x1046=((cj3)*(r01)*(x1043));
IkReal x1047=((IkReal(1.00000000000000))*(cj0)*(r00)*(sj3));
IkReal x1048=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst62)*(((((IkReal(-1.00000000000000))*(r01)*(x1045)))+(((IkReal(-1.00000000000000))*(x1041)*(x1042)*(x1043)))+(((sj3)*(x1044)))+(((IkReal(-1.00000000000000))*(x1041)*(x1047)))+(((IkReal(-1.00000000000000))*(sj3)*(x1048)))+(((cj3)*(r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1040)*(x1047)))+(((IkReal(-1.00000000000000))*(x1040)*(x1042)*(x1043))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst62)*(((((IkReal(-1.00000000000000))*(cj3)*(x1048)))+(((cj0)*(x1042)))+(((IkReal(-1.00000000000000))*(x1041)*(x1046)))+(((cj3)*(x1044)))+(((IkReal(-1.00000000000000))*(r00)*(sj3)*(x1043)))+(((IkReal(-1.00000000000000))*(x1040)*(x1046)))+(((IkReal(-1.00000000000000))*(r00)*(x1041)*(x1045)))+(((IkReal(-1.00000000000000))*(r00)*(x1040)*(x1045))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst62)*(((((IkReal(-1.00000000000000))*(r01)*(x1045)))+(((IkReal(-1.00000000000000))*(x1041)*(x1042)*(x1043)))+(((sj3)*(x1044)))+(((IkReal(-1.00000000000000))*(x1041)*(x1047)))+(((IkReal(-1.00000000000000))*(sj3)*(x1048)))+(((cj3)*(r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1040)*(x1047)))+(((IkReal(-1.00000000000000))*(x1040)*(x1042)*(x1043)))))), ((gconst62)*(((((IkReal(-1.00000000000000))*(cj3)*(x1048)))+(((cj0)*(x1042)))+(((IkReal(-1.00000000000000))*(x1041)*(x1046)))+(((cj3)*(x1044)))+(((IkReal(-1.00000000000000))*(r00)*(sj3)*(x1043)))+(((IkReal(-1.00000000000000))*(x1040)*(x1046)))+(((IkReal(-1.00000000000000))*(r00)*(x1041)*(x1045)))+(((IkReal(-1.00000000000000))*(r00)*(x1040)*(x1045)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1049=IKsin(j5);
IkReal x1050=IKcos(j5);
IkReal x1051=((cj1)*(sj2));
IkReal x1052=((r01)*(sj0));
IkReal x1053=((cj0)*(r00));
IkReal x1054=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((sj3)*(x1050)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj3)*(x1049))));
evalcond[1]=((((x1052)*(x1054)))+(((sj3)*(x1049)))+(((x1053)*(x1054)))+(((cj3)*(x1050)))+(((x1051)*(x1053)))+(((x1051)*(x1052)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1055=((pz)*(sj2));
IkReal x1056=((IkReal(1.00000000000000))*(cj1));
IkReal x1057=((cj2)*(sj1));
IkReal x1058=((py)*(sj0));
IkReal x1059=((cj0)*(px));
IkReal x1060=((sj1)*(sj2));
IkReal x1061=((IkReal(0.0750000000000000))*(cj1));
IkReal x1062=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1057)*(x1059)))+(((x1057)*(x1058)))+(((IkReal(-1.00000000000000))*(sj2)*(x1061)))+(((sj1)*(x1055)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1056)))+(((x1059)*(x1062)))+(((IkReal(-0.0750000000000000))*(x1057)))+(((x1058)*(x1062))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1057)))+(((IkReal(-1.00000000000000))*(cj2)*(x1056)*(x1058)))+(((IkReal(-1.00000000000000))*(cj2)*(x1056)*(x1059)))+(((cj2)*(x1061)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1055)*(x1056)))+(((IkReal(-0.0750000000000000))*(x1060)))+(((IkReal(0.0800000000000000))*(cj4)))+(((x1059)*(x1060)))+(((x1058)*(x1060))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x1063=((pz)*(sj2));
IkReal x1064=((IkReal(1.00000000000000))*(cj1));
IkReal x1065=((cj2)*(sj0));
IkReal x1066=((py)*(sj1));
IkReal x1067=((cj0)*(sj1));
IkReal x1068=((cj2)*(px));
IkReal x1069=((sj0)*(sj2));
IkReal x1070=((IkReal(0.0750000000000000))*(cj2));
IkReal x1071=((px)*(sj2));
IkReal x1072=((IkReal(0.0750000000000000))*(sj2));
IkReal x1073=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1064)))+(((r01)*(sj1)*(x1069)))+(((IkReal(-1.00000000000000))*(r02)*(x1073)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1064)))+(((IkReal(-1.00000000000000))*(r01)*(x1064)*(x1065)))+(((r00)*(sj2)*(x1067))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x1072)))+(((x1067)*(x1068)))+(((IkReal(-1.00000000000000))*(sj1)*(x1070)))+(((cj1)*(py)*(x1069)))+(((sj1)*(x1063)))+(((cj0)*(cj1)*(x1071)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1064)))+(((x1065)*(x1066))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(x1063)*(x1064)))+(((x1067)*(x1071)))+(((x1066)*(x1069)))+(((IkReal(-1.00000000000000))*(pz)*(x1073)))+(((IkReal(-1.00000000000000))*(sj1)*(x1072)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x1064)*(x1068)))+(((IkReal(-1.00000000000000))*(py)*(x1064)*(x1065)))+(((cj1)*(x1070))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1074=((IkReal(1.00000000000000))*(sj2));
IkReal x1075=((IkReal(1.00000000000000))*(r01));
IkReal x1076=((cj0)*(r00));
IkReal x1077=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1074)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1074)))+(((IkReal(-1.00000000000000))*(sj0)*(x1075)*(x1077)))+(((IkReal(-1.00000000000000))*(cj1)*(x1074)*(x1076)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1076)*(x1077))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1075))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1074)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1074)))+(((IkReal(-1.00000000000000))*(sj0)*(x1075)*(x1077)))+(((IkReal(-1.00000000000000))*(cj1)*(x1074)*(x1076)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1076)*(x1077)))))+IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1075)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1074)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1074)))+(((IkReal(-1.00000000000000))*(sj0)*(x1075)*(x1077)))+(((IkReal(-1.00000000000000))*(cj1)*(x1074)*(x1076)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1076)*(x1077)))), ((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1075)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1078=((cj1)*(sj2));
IkReal x1079=((r01)*(sj0));
IkReal x1080=((cj0)*(r00));
IkReal x1081=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x1079)*(x1081)))+(((x1078)*(x1080)))+(((x1078)*(x1079)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1080)*(x1081)))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1082=((pz)*(sj2));
IkReal x1083=((IkReal(1.00000000000000))*(cj1));
IkReal x1084=((cj2)*(sj0));
IkReal x1085=((py)*(sj1));
IkReal x1086=((cj0)*(sj1));
IkReal x1087=((cj2)*(px));
IkReal x1088=((sj0)*(sj2));
IkReal x1089=((IkReal(0.0750000000000000))*(cj2));
IkReal x1090=((px)*(sj2));
IkReal x1091=((IkReal(0.0750000000000000))*(sj2));
IkReal x1092=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((r01)*(sj1)*(x1088)))+(((IkReal(-1.00000000000000))*(r02)*(x1092)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1083)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1083)))+(((IkReal(-1.00000000000000))*(r01)*(x1083)*(x1084)))+(((r00)*(sj2)*(x1086))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(x1089)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x1091)))+(((sj1)*(x1082)))+(((x1086)*(x1087)))+(((cj1)*(py)*(x1088)))+(((cj0)*(cj1)*(x1090)))+(((x1084)*(x1085)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1083))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1092)))+(((cj1)*(x1089)))+(((x1086)*(x1090)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x1091)))+(((IkReal(-1.00000000000000))*(cj0)*(x1083)*(x1087)))+(((IkReal(-1.00000000000000))*(x1082)*(x1083)))+(((IkReal(-1.00000000000000))*(py)*(x1083)*(x1084)))+(((x1085)*(x1088))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1093=((IkReal(1.00000000000000))*(sj2));
IkReal x1094=((cj0)*(r00));
IkReal x1095=((r01)*(sj0));
IkReal x1096=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(x1093)*(x1095)))+(((IkReal(-1.00000000000000))*(cj1)*(x1093)*(x1094)))+(((IkReal(-1.00000000000000))*(x1095)*(x1096)))+(((IkReal(-1.00000000000000))*(x1094)*(x1096)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1093)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(x1093)*(x1095)))+(((IkReal(-1.00000000000000))*(cj1)*(x1093)*(x1094)))+(((IkReal(-1.00000000000000))*(x1095)*(x1096)))+(((IkReal(-1.00000000000000))*(x1094)*(x1096)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1093)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(x1093)*(x1095)))+(((IkReal(-1.00000000000000))*(cj1)*(x1093)*(x1094)))+(((IkReal(-1.00000000000000))*(x1095)*(x1096)))+(((IkReal(-1.00000000000000))*(x1094)*(x1096)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1093)))+(((cj1)*(cj2)*(r02)))), ((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1097=((cj1)*(sj2));
IkReal x1098=((r01)*(sj0));
IkReal x1099=((cj0)*(r00));
IkReal x1100=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(IKcos(j5)));
evalcond[1]=((((x1098)*(x1100)))+(IKsin(j5))+(((x1099)*(x1100)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1097)*(x1098)))+(((x1097)*(x1099))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1101=((pz)*(sj2));
IkReal x1102=((IkReal(1.00000000000000))*(cj1));
IkReal x1103=((cj2)*(sj1));
IkReal x1104=((py)*(sj0));
IkReal x1105=((cj0)*(px));
IkReal x1106=((sj1)*(sj2));
IkReal x1107=((IkReal(1.00000000000000))*(cj0));
IkReal x1108=((IkReal(0.0750000000000000))*(cj1));
IkReal x1109=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1107)))+(((px)*(sj0))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1107))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1105)*(x1109)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1104)*(x1109)))+(((IkReal(-1.00000000000000))*(sj2)*(x1108)))+(((IkReal(-0.0750000000000000))*(x1103)))+(((sj1)*(x1101)))+(((x1103)*(x1105)))+(((x1103)*(x1104)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1102))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x1102)*(x1104)))+(((IkReal(-1.00000000000000))*(cj2)*(x1102)*(x1105)))+(((x1105)*(x1106)))+(((x1104)*(x1106)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x1106)))+(((cj2)*(x1108)))+(((IkReal(-1.00000000000000))*(pz)*(x1103)))+(((IkReal(-1.00000000000000))*(x1101)*(x1102))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1110=((r02)*(sj1));
IkReal x1111=((IkReal(1.00000000000000))*(sj2));
IkReal x1112=((IkReal(1.00000000000000))*(cj2));
IkReal x1113=((cj1)*(r02));
IkReal x1114=((cj0)*(cj1)*(r00));
IkReal x1115=((cj0)*(r00)*(sj1));
IkReal x1116=((r01)*(sj0)*(sj1));
IkReal x1117=((cj1)*(r01)*(sj0));
if( IKabs(((((IkReal(-1.00000000000000))*(x1112)*(x1116)))+(((IkReal(-1.00000000000000))*(x1112)*(x1115)))+(((cj2)*(x1113)))+(((IkReal(-1.00000000000000))*(x1110)*(x1111)))+(((IkReal(-1.00000000000000))*(x1111)*(x1114)))+(((IkReal(-1.00000000000000))*(x1111)*(x1117))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj2)*(x1113)))+(((cj2)*(x1114)))+(((cj2)*(x1117)))+(((cj2)*(x1110)))+(((IkReal(-1.00000000000000))*(x1111)*(x1115)))+(((IkReal(-1.00000000000000))*(x1111)*(x1116))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x1112)*(x1116)))+(((IkReal(-1.00000000000000))*(x1112)*(x1115)))+(((cj2)*(x1113)))+(((IkReal(-1.00000000000000))*(x1110)*(x1111)))+(((IkReal(-1.00000000000000))*(x1111)*(x1114)))+(((IkReal(-1.00000000000000))*(x1111)*(x1117)))))+IKsqr(((((sj2)*(x1113)))+(((cj2)*(x1114)))+(((cj2)*(x1117)))+(((cj2)*(x1110)))+(((IkReal(-1.00000000000000))*(x1111)*(x1115)))+(((IkReal(-1.00000000000000))*(x1111)*(x1116)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x1112)*(x1116)))+(((IkReal(-1.00000000000000))*(x1112)*(x1115)))+(((cj2)*(x1113)))+(((IkReal(-1.00000000000000))*(x1110)*(x1111)))+(((IkReal(-1.00000000000000))*(x1111)*(x1114)))+(((IkReal(-1.00000000000000))*(x1111)*(x1117)))), ((((sj2)*(x1113)))+(((cj2)*(x1114)))+(((cj2)*(x1117)))+(((cj2)*(x1110)))+(((IkReal(-1.00000000000000))*(x1111)*(x1115)))+(((IkReal(-1.00000000000000))*(x1111)*(x1116)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1118=((IkReal(1.00000000000000))*(cj2));
IkReal x1119=((r01)*(sj0));
IkReal x1120=((cj2)*(sj1));
IkReal x1121=((cj1)*(r02));
IkReal x1122=((r02)*(sj1));
IkReal x1123=((cj0)*(r00));
IkReal x1124=((sj2)*(x1119));
IkReal x1125=((cj1)*(x1123));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj1)*(x1118)*(x1119)))+(((sj1)*(sj2)*(x1123)))+(((sj1)*(x1124)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(sj2)*(x1121)))+(((IkReal(-1.00000000000000))*(x1118)*(x1122)))+(((IkReal(-1.00000000000000))*(x1118)*(x1125))));
evalcond[1]=((((x1120)*(x1123)))+(((x1119)*(x1120)))+(IKsin(j5))+(((cj1)*(x1124)))+(((sj2)*(x1125)))+(((sj2)*(x1122)))+(((IkReal(-1.00000000000000))*(x1118)*(x1121))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1126=((pz)*(sj2));
IkReal x1127=((IkReal(1.00000000000000))*(cj1));
IkReal x1128=((cj2)*(sj1));
IkReal x1129=((py)*(sj0));
IkReal x1130=((cj0)*(px));
IkReal x1131=((sj1)*(sj2));
IkReal x1132=((IkReal(1.00000000000000))*(cj0));
IkReal x1133=((IkReal(0.0750000000000000))*(cj1));
IkReal x1134=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1132)))+(((px)*(sj0))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1132))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1130)*(x1134)))+(((IkReal(-1.00000000000000))*(sj2)*(x1133)))+(((x1129)*(x1134)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x1126)))+(((IkReal(-0.0750000000000000))*(x1128)))+(((x1128)*(x1129)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1127)))+(((x1128)*(x1130))));
evalcond[4]=((IkReal(0.295000000000000))+(((cj2)*(x1133)))+(((x1130)*(x1131)))+(((x1129)*(x1131)))+(((IkReal(-1.00000000000000))*(x1126)*(x1127)))+(((IkReal(-1.00000000000000))*(pz)*(x1128)))+(((IkReal(-0.0750000000000000))*(x1131)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1127)*(x1130)))+(((IkReal(-1.00000000000000))*(cj2)*(x1127)*(x1129))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1135=((sj1)*(sj2));
IkReal x1136=((IkReal(1.00000000000000))*(r02));
IkReal x1137=((r01)*(sj0));
IkReal x1138=((cj0)*(r00));
IkReal x1139=((cj1)*(sj2));
IkReal x1140=((cj2)*(sj1));
IkReal x1141=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((IkReal(-1.00000000000000))*(x1138)*(x1140)))+(((IkReal(-1.00000000000000))*(x1137)*(x1140)))+(((IkReal(-1.00000000000000))*(x1135)*(x1136)))+(((IkReal(-1.00000000000000))*(x1138)*(x1139)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1137)*(x1139))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1136)*(x1140)))+(((IkReal(-1.00000000000000))*(x1137)*(x1141)))+(((x1135)*(x1137)))+(((x1135)*(x1138)))+(((IkReal(-1.00000000000000))*(x1136)*(x1139)))+(((IkReal(-1.00000000000000))*(x1138)*(x1141))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x1138)*(x1140)))+(((IkReal(-1.00000000000000))*(x1137)*(x1140)))+(((IkReal(-1.00000000000000))*(x1135)*(x1136)))+(((IkReal(-1.00000000000000))*(x1138)*(x1139)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1137)*(x1139)))))+IKsqr(((((IkReal(-1.00000000000000))*(x1136)*(x1140)))+(((IkReal(-1.00000000000000))*(x1137)*(x1141)))+(((x1135)*(x1137)))+(((x1135)*(x1138)))+(((IkReal(-1.00000000000000))*(x1136)*(x1139)))+(((IkReal(-1.00000000000000))*(x1138)*(x1141)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x1138)*(x1140)))+(((IkReal(-1.00000000000000))*(x1137)*(x1140)))+(((IkReal(-1.00000000000000))*(x1135)*(x1136)))+(((IkReal(-1.00000000000000))*(x1138)*(x1139)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1137)*(x1139)))), ((((IkReal(-1.00000000000000))*(x1136)*(x1140)))+(((IkReal(-1.00000000000000))*(x1137)*(x1141)))+(((x1135)*(x1137)))+(((x1135)*(x1138)))+(((IkReal(-1.00000000000000))*(x1136)*(x1139)))+(((IkReal(-1.00000000000000))*(x1138)*(x1141)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1142=((IkReal(1.00000000000000))*(cj2));
IkReal x1143=((r01)*(sj0));
IkReal x1144=((cj2)*(sj1));
IkReal x1145=((cj1)*(r02));
IkReal x1146=((r02)*(sj1));
IkReal x1147=((cj0)*(r00));
IkReal x1148=((sj2)*(x1143));
IkReal x1149=((cj1)*(x1147));
evalcond[0]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((sj1)*(sj2)*(x1147)))+(((IkReal(-1.00000000000000))*(cj1)*(x1142)*(x1143)))+(((sj1)*(x1148)))+(((IkReal(-1.00000000000000))*(x1142)*(x1146)))+(((IkReal(-1.00000000000000))*(x1142)*(x1149)))+(((IkReal(-1.00000000000000))*(sj2)*(x1145))));
evalcond[1]=((((x1143)*(x1144)))+(IKsin(j5))+(((sj2)*(x1146)))+(((sj2)*(x1149)))+(((cj1)*(x1148)))+(((x1144)*(x1147)))+(((IkReal(-1.00000000000000))*(x1142)*(x1145))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1150=((r02)*(sj1));
IkReal x1151=((IkReal(1.00000000000000))*(sj2));
IkReal x1152=((IkReal(1.00000000000000))*(cj2));
IkReal x1153=((cj1)*(r02));
IkReal x1154=((cj0)*(cj1)*(r00));
IkReal x1155=((cj0)*(r00)*(sj1));
IkReal x1156=((r01)*(sj0)*(sj1));
IkReal x1157=((cj1)*(r01)*(sj0));
if( IKabs(((((cj2)*(x1153)))+(((IkReal(-1.00000000000000))*(x1152)*(x1155)))+(((IkReal(-1.00000000000000))*(x1152)*(x1156)))+(((IkReal(-1.00000000000000))*(x1151)*(x1157)))+(((IkReal(-1.00000000000000))*(x1151)*(x1154)))+(((IkReal(-1.00000000000000))*(x1150)*(x1151))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj2)*(x1154)))+(((cj2)*(x1157)))+(((cj2)*(x1150)))+(((sj2)*(x1153)))+(((IkReal(-1.00000000000000))*(x1151)*(x1156)))+(((IkReal(-1.00000000000000))*(x1151)*(x1155))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(x1153)))+(((IkReal(-1.00000000000000))*(x1152)*(x1155)))+(((IkReal(-1.00000000000000))*(x1152)*(x1156)))+(((IkReal(-1.00000000000000))*(x1151)*(x1157)))+(((IkReal(-1.00000000000000))*(x1151)*(x1154)))+(((IkReal(-1.00000000000000))*(x1150)*(x1151)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj2)*(x1154)))+(((cj2)*(x1157)))+(((cj2)*(x1150)))+(((sj2)*(x1153)))+(((IkReal(-1.00000000000000))*(x1151)*(x1156)))+(((IkReal(-1.00000000000000))*(x1151)*(x1155)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(x1153)))+(((IkReal(-1.00000000000000))*(x1152)*(x1155)))+(((IkReal(-1.00000000000000))*(x1152)*(x1156)))+(((IkReal(-1.00000000000000))*(x1151)*(x1157)))+(((IkReal(-1.00000000000000))*(x1151)*(x1154)))+(((IkReal(-1.00000000000000))*(x1150)*(x1151)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj2)*(x1154)))+(((cj2)*(x1157)))+(((cj2)*(x1150)))+(((sj2)*(x1153)))+(((IkReal(-1.00000000000000))*(x1151)*(x1156)))+(((IkReal(-1.00000000000000))*(x1151)*(x1155)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1158=IKcos(j5);
IkReal x1159=((r01)*(sj0));
IkReal x1160=((sj1)*(sj2));
IkReal x1161=((IkReal(1.00000000000000))*(cj0));
IkReal x1162=((IkReal(1.00000000000000))*(cj1));
IkReal x1163=((cj1)*(sj2));
IkReal x1164=((cj0)*(r00));
IkReal x1165=((cj2)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj4)*(x1158)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1161))));
evalcond[1]=((((x1159)*(x1160)))+(((IkReal(-1.00000000000000))*(cj2)*(x1159)*(x1162)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)*(x1161)))+(((IkReal(-1.00000000000000))*(r02)*(x1165)))+(((x1160)*(x1164)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1162)))+(((sj4)*(x1158))));
evalcond[2]=((((x1159)*(x1165)))+(((x1159)*(x1163)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1162)))+(((x1164)*(x1165)))+(((r02)*(x1160)))+(((x1163)*(x1164)))+(IKsin(j5)));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1166=((IkReal(1.00000000000000))*(sj2));
IkReal x1167=((IkReal(1.00000000000000))*(r01));
IkReal x1168=((cj0)*(r00));
IkReal x1169=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x1167)*(x1169)))+(((IkReal(-1.00000000000000))*(cj1)*(x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1166)))+(((IkReal(-1.00000000000000))*(x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1166)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1167))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x1167)*(x1169)))+(((IkReal(-1.00000000000000))*(cj1)*(x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1166)))+(((IkReal(-1.00000000000000))*(x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1166)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1167)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj0)*(x1167)*(x1169)))+(((IkReal(-1.00000000000000))*(cj1)*(x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x1166)))+(((IkReal(-1.00000000000000))*(x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1166)))+(((cj1)*(cj2)*(r02)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1167)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1170=IKcos(j5);
IkReal x1171=((r01)*(sj0));
IkReal x1172=((sj1)*(sj2));
IkReal x1173=((IkReal(1.00000000000000))*(cj0));
IkReal x1174=((IkReal(1.00000000000000))*(cj1));
IkReal x1175=((cj1)*(sj2));
IkReal x1176=((cj0)*(r00));
IkReal x1177=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj4)*(x1170)))+(((IkReal(-1.00000000000000))*(r01)*(x1173))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)*(x1173)))+(((IkReal(-1.00000000000000))*(cj2)*(x1171)*(x1174)))+(((IkReal(-1.00000000000000))*(r02)*(x1177)))+(((sj4)*(x1170)))+(((x1172)*(x1176)))+(((x1171)*(x1172)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1174))));
evalcond[2]=((((r02)*(x1172)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1174)))+(IKsin(j5))+(((x1176)*(x1177)))+(((x1175)*(x1176)))+(((x1171)*(x1177)))+(((x1171)*(x1175))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1178=((pz)*(sj2));
IkReal x1179=((IkReal(1.00000000000000))*(cj1));
IkReal x1180=((cj2)*(sj1));
IkReal x1181=((py)*(sj0));
IkReal x1182=((cj0)*(px));
IkReal x1183=((sj1)*(sj2));
IkReal x1184=((IkReal(0.0750000000000000))*(cj1));
IkReal x1185=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((x1182)*(x1185)))+(((sj1)*(x1178)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1180)*(x1182)))+(((x1180)*(x1181)))+(((IkReal(-0.0750000000000000))*(x1180)))+(((x1181)*(x1185)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1179)))+(((IkReal(-1.00000000000000))*(sj2)*(x1184))));
evalcond[3]=((IkReal(0.295000000000000))+(((x1182)*(x1183)))+(((IkReal(-1.00000000000000))*(cj2)*(x1179)*(x1181)))+(((IkReal(-1.00000000000000))*(cj2)*(x1179)*(x1182)))+(((IkReal(-0.0750000000000000))*(x1183)))+(((IkReal(-1.00000000000000))*(x1178)*(x1179)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1180)))+(((x1181)*(x1183)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x1184))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x1186=((pz)*(sj2));
IkReal x1187=((IkReal(1.00000000000000))*(cj1));
IkReal x1188=((cj2)*(sj0));
IkReal x1189=((py)*(sj1));
IkReal x1190=((cj0)*(sj1));
IkReal x1191=((cj2)*(px));
IkReal x1192=((sj0)*(sj2));
IkReal x1193=((IkReal(0.0750000000000000))*(cj2));
IkReal x1194=((px)*(sj2));
IkReal x1195=((IkReal(0.0750000000000000))*(sj2));
IkReal x1196=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1187)))+(((r01)*(sj1)*(x1192)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1187)))+(((IkReal(-1.00000000000000))*(r01)*(x1187)*(x1188)))+(((r00)*(sj2)*(x1190)))+(((IkReal(-1.00000000000000))*(r02)*(x1196))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x1192)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj0)*(cj1)*(x1194)))+(((x1188)*(x1189)))+(((sj1)*(x1186)))+(((IkReal(-1.00000000000000))*(cj1)*(x1195)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1187)))+(((x1190)*(x1191)))+(((IkReal(-1.00000000000000))*(sj1)*(x1193))));
evalcond[4]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(x1186)*(x1187)))+(((IkReal(-1.00000000000000))*(pz)*(x1196)))+(((x1189)*(x1192)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj0)*(x1187)*(x1191)))+(((IkReal(-1.00000000000000))*(py)*(x1187)*(x1188)))+(((x1190)*(x1194)))+(((cj1)*(x1193)))+(((IkReal(-1.00000000000000))*(sj1)*(x1195))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1197=((cj1)*(sj2));
IkReal x1198=((r01)*(sj0));
IkReal x1199=((cj0)*(r00));
IkReal x1200=((cj2)*(sj1));
if( IKabs(((((x1197)*(x1198)))+(((x1197)*(x1199)))+(((x1199)*(x1200)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1198)*(x1200))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1197)*(x1198)))+(((x1197)*(x1199)))+(((x1199)*(x1200)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1198)*(x1200)))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1197)*(x1198)))+(((x1197)*(x1199)))+(((x1199)*(x1200)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1198)*(x1200)))), ((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1201=((cj1)*(sj2));
IkReal x1202=((r01)*(sj0));
IkReal x1203=((cj0)*(r00));
IkReal x1204=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(IKcos(j5)));
evalcond[1]=((((x1202)*(x1204)))+(((x1203)*(x1204)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1201)*(x1202)))+(((x1201)*(x1203)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1205=((pz)*(sj2));
IkReal x1206=((IkReal(1.00000000000000))*(cj1));
IkReal x1207=((cj2)*(sj0));
IkReal x1208=((py)*(sj1));
IkReal x1209=((cj0)*(sj1));
IkReal x1210=((cj2)*(px));
IkReal x1211=((sj0)*(sj2));
IkReal x1212=((IkReal(0.0750000000000000))*(cj2));
IkReal x1213=((px)*(sj2));
IkReal x1214=((IkReal(0.0750000000000000))*(sj2));
IkReal x1215=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1206)*(x1207)))+(((IkReal(-1.00000000000000))*(r02)*(x1215)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1206)))+(((r01)*(sj1)*(x1211)))+(((r00)*(sj2)*(x1209)))+(((IkReal(-1.00000000000000))*(cj0)*(cj2)*(r00)*(x1206))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(x1214)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1206)))+(((sj1)*(x1205)))+(((cj0)*(cj1)*(x1213)))+(((x1209)*(x1210)))+(((IkReal(-1.00000000000000))*(sj1)*(x1212)))+(((cj1)*(py)*(x1211)))+(((x1207)*(x1208))));
evalcond[4]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(x1206)*(x1210)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1205)*(x1206)))+(((x1209)*(x1213)))+(((x1208)*(x1211)))+(((IkReal(-1.00000000000000))*(sj1)*(x1214)))+(((cj1)*(x1212)))+(((IkReal(-1.00000000000000))*(pz)*(x1215)))+(((IkReal(-1.00000000000000))*(py)*(x1206)*(x1207))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1216=((cj1)*(sj2));
IkReal x1217=((r01)*(sj0));
IkReal x1218=((cj0)*(r00));
IkReal x1219=((cj2)*(sj1));
if( IKabs(((((x1218)*(x1219)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1216)*(x1217)))+(((x1216)*(x1218)))+(((x1217)*(x1219)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1218)*(x1219)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1216)*(x1217)))+(((x1216)*(x1218)))+(((x1217)*(x1219)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1218)*(x1219)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1216)*(x1217)))+(((x1216)*(x1218)))+(((x1217)*(x1219)))+(((r02)*(sj1)*(sj2)))), ((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1220=((cj1)*(sj2));
IkReal x1221=((r01)*(sj0));
IkReal x1222=((cj0)*(r00));
IkReal x1223=((cj2)*(sj1));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
evalcond[1]=((((x1220)*(x1221)))+(((x1220)*(x1222)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1221)*(x1223)))+(((x1222)*(x1223)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((r02)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1224=((pz)*(sj2));
IkReal x1225=((IkReal(1.00000000000000))*(cj1));
IkReal x1226=((cj2)*(sj1));
IkReal x1227=((py)*(sj0));
IkReal x1228=((cj0)*(px));
IkReal x1229=((sj1)*(sj2));
IkReal x1230=((IkReal(1.00000000000000))*(cj0));
IkReal x1231=((IkReal(0.0750000000000000))*(cj1));
IkReal x1232=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1230)))+(((px)*(sj0))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1230))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((sj1)*(x1224)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1226)*(x1228)))+(((x1226)*(x1227)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1225)))+(((IkReal(-1.00000000000000))*(sj2)*(x1231)))+(((x1228)*(x1232)))+(((x1227)*(x1232)))+(((IkReal(-0.0750000000000000))*(x1226))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x1224)*(x1225)))+(((cj2)*(x1231)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1226)))+(((x1227)*(x1229)))+(((x1228)*(x1229)))+(((IkReal(-1.00000000000000))*(cj2)*(x1225)*(x1228)))+(((IkReal(-1.00000000000000))*(cj2)*(x1225)*(x1227)))+(((IkReal(-0.0750000000000000))*(x1229))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1233=((r01)*(sj0));
IkReal x1234=((cj1)*(sj2));
IkReal x1235=((sj1)*(sj2));
IkReal x1236=((cj2)*(sj1));
IkReal x1237=((cj0)*(r00));
IkReal x1238=((cj1)*(cj2));
IkReal x1239=((cj1)*(x1237));
if( IKabs(((((x1234)*(x1237)))+(((r02)*(x1235)))+(((x1233)*(x1236)))+(((x1233)*(x1234)))+(((IkReal(-1.00000000000000))*(r02)*(x1238)))+(((x1236)*(x1237))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1233)*(x1235)))+(((r02)*(x1234)))+(((r02)*(x1236)))+(((x1233)*(x1238)))+(((IkReal(-1.00000000000000))*(x1235)*(x1237)))+(((x1237)*(x1238))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1234)*(x1237)))+(((r02)*(x1235)))+(((x1233)*(x1236)))+(((x1233)*(x1234)))+(((IkReal(-1.00000000000000))*(r02)*(x1238)))+(((x1236)*(x1237)))))+IKsqr(((((IkReal(-1.00000000000000))*(x1233)*(x1235)))+(((r02)*(x1234)))+(((r02)*(x1236)))+(((x1233)*(x1238)))+(((IkReal(-1.00000000000000))*(x1235)*(x1237)))+(((x1237)*(x1238)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1234)*(x1237)))+(((r02)*(x1235)))+(((x1233)*(x1236)))+(((x1233)*(x1234)))+(((IkReal(-1.00000000000000))*(r02)*(x1238)))+(((x1236)*(x1237)))), ((((IkReal(-1.00000000000000))*(x1233)*(x1235)))+(((r02)*(x1234)))+(((r02)*(x1236)))+(((x1233)*(x1238)))+(((IkReal(-1.00000000000000))*(x1235)*(x1237)))+(((x1237)*(x1238)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1240=((IkReal(1.00000000000000))*(cj2));
IkReal x1241=((r01)*(sj0));
IkReal x1242=((cj2)*(sj1));
IkReal x1243=((cj1)*(r02));
IkReal x1244=((r02)*(sj1));
IkReal x1245=((cj0)*(r00));
IkReal x1246=((sj2)*(x1241));
IkReal x1247=((cj1)*(x1245));
evalcond[0]=((((sj1)*(sj2)*(x1245)))+(((IkReal(-1.00000000000000))*(sj2)*(x1243)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(x1240)*(x1244)))+(((IkReal(-1.00000000000000))*(x1240)*(x1247)))+(((IkReal(-1.00000000000000))*(cj1)*(x1240)*(x1241)))+(((sj1)*(x1246))));
evalcond[1]=((((cj1)*(x1246)))+(((sj2)*(x1244)))+(((sj2)*(x1247)))+(((IkReal(-1.00000000000000))*(x1240)*(x1243)))+(((x1242)*(x1245)))+(((x1241)*(x1242)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1248=((pz)*(sj2));
IkReal x1249=((IkReal(1.00000000000000))*(cj1));
IkReal x1250=((cj2)*(sj1));
IkReal x1251=((py)*(sj0));
IkReal x1252=((cj0)*(px));
IkReal x1253=((sj1)*(sj2));
IkReal x1254=((IkReal(1.00000000000000))*(cj0));
IkReal x1255=((IkReal(0.0750000000000000))*(cj1));
IkReal x1256=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0800000000000000))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1254))));
evalcond[2]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1254))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1249)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1255)))+(((x1250)*(x1252)))+(((x1250)*(x1251)))+(((x1252)*(x1256)))+(((x1251)*(x1256)))+(((sj1)*(x1248)))+(((IkReal(-0.0750000000000000))*(x1250))));
evalcond[4]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1250)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1252)))+(((IkReal(-1.00000000000000))*(cj2)*(x1249)*(x1251)))+(((cj2)*(x1255)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x1252)*(x1253)))+(((x1251)*(x1253)))+(((IkReal(-0.0750000000000000))*(x1253)))+(((IkReal(-1.00000000000000))*(x1248)*(x1249))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1257=((IkReal(1.00000000000000))*(cj2));
IkReal x1258=((r02)*(sj2));
IkReal x1259=((r01)*(sj0));
IkReal x1260=((cj2)*(sj1));
IkReal x1261=((cj0)*(r00));
IkReal x1262=((sj2)*(x1259));
IkReal x1263=((cj1)*(x1261));
if( IKabs(((((sj1)*(x1258)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1257)))+(((x1259)*(x1260)))+(((x1260)*(x1261)))+(((cj1)*(x1262)))+(((sj2)*(x1263))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj1)*(x1262)))+(((IkReal(-1.00000000000000))*(cj1)*(x1258)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1257)))+(((IkReal(-1.00000000000000))*(x1257)*(x1263)))+(((IkReal(-1.00000000000000))*(cj1)*(x1257)*(x1259)))+(((sj1)*(sj2)*(x1261))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj1)*(x1258)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1257)))+(((x1259)*(x1260)))+(((x1260)*(x1261)))+(((cj1)*(x1262)))+(((sj2)*(x1263)))))+IKsqr(((((sj1)*(x1262)))+(((IkReal(-1.00000000000000))*(cj1)*(x1258)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1257)))+(((IkReal(-1.00000000000000))*(x1257)*(x1263)))+(((IkReal(-1.00000000000000))*(cj1)*(x1257)*(x1259)))+(((sj1)*(sj2)*(x1261)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((sj1)*(x1258)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1257)))+(((x1259)*(x1260)))+(((x1260)*(x1261)))+(((cj1)*(x1262)))+(((sj2)*(x1263)))), ((((sj1)*(x1262)))+(((IkReal(-1.00000000000000))*(cj1)*(x1258)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1257)))+(((IkReal(-1.00000000000000))*(x1257)*(x1263)))+(((IkReal(-1.00000000000000))*(cj1)*(x1257)*(x1259)))+(((sj1)*(sj2)*(x1261)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1264=((IkReal(1.00000000000000))*(cj2));
IkReal x1265=((r01)*(sj0));
IkReal x1266=((cj2)*(sj1));
IkReal x1267=((cj1)*(r02));
IkReal x1268=((r02)*(sj1));
IkReal x1269=((cj0)*(r00));
IkReal x1270=((sj2)*(x1265));
IkReal x1271=((cj1)*(x1269));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1264)*(x1268)))+(((sj1)*(x1270)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((sj1)*(sj2)*(x1269)))+(((IkReal(-1.00000000000000))*(sj2)*(x1267)))+(((IkReal(-1.00000000000000))*(x1264)*(x1271)))+(((IkReal(-1.00000000000000))*(cj1)*(x1264)*(x1265))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1264)*(x1267)))+(((sj2)*(x1271)))+(((cj1)*(x1270)))+(((x1266)*(x1269)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((x1265)*(x1266)))+(((sj2)*(x1268))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1272=((r01)*(sj0));
IkReal x1273=((cj1)*(sj2));
IkReal x1274=((sj1)*(sj2));
IkReal x1275=((cj2)*(sj1));
IkReal x1276=((cj0)*(r00));
IkReal x1277=((cj1)*(cj2));
IkReal x1278=((cj1)*(x1276));
if( IKabs(((((x1275)*(x1276)))+(((x1272)*(x1273)))+(((x1272)*(x1275)))+(((x1273)*(x1276)))+(((r02)*(x1274)))+(((IkReal(-1.00000000000000))*(r02)*(x1277))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x1276)*(x1277)))+(((x1272)*(x1277)))+(((IkReal(-1.00000000000000))*(x1274)*(x1276)))+(((r02)*(x1275)))+(((r02)*(x1273)))+(((IkReal(-1.00000000000000))*(x1272)*(x1274))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1275)*(x1276)))+(((x1272)*(x1273)))+(((x1272)*(x1275)))+(((x1273)*(x1276)))+(((r02)*(x1274)))+(((IkReal(-1.00000000000000))*(r02)*(x1277)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x1276)*(x1277)))+(((x1272)*(x1277)))+(((IkReal(-1.00000000000000))*(x1274)*(x1276)))+(((r02)*(x1275)))+(((r02)*(x1273)))+(((IkReal(-1.00000000000000))*(x1272)*(x1274)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1275)*(x1276)))+(((x1272)*(x1273)))+(((x1272)*(x1275)))+(((x1273)*(x1276)))+(((r02)*(x1274)))+(((IkReal(-1.00000000000000))*(r02)*(x1277)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((x1276)*(x1277)))+(((x1272)*(x1277)))+(((IkReal(-1.00000000000000))*(x1274)*(x1276)))+(((r02)*(x1275)))+(((r02)*(x1273)))+(((IkReal(-1.00000000000000))*(x1272)*(x1274)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1279=IKcos(j5);
IkReal x1280=((r01)*(sj0));
IkReal x1281=((sj1)*(sj2));
IkReal x1282=((IkReal(1.00000000000000))*(cj0));
IkReal x1283=((cj1)*(cj2));
IkReal x1284=((cj1)*(sj2));
IkReal x1285=((cj0)*(r00));
IkReal x1286=((cj2)*(sj1));
IkReal x1287=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1282)))+(((cj4)*(x1279))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1280)*(x1283)))+(((IkReal(-1.00000000000000))*(x1284)*(x1287)))+(((sj4)*(x1279)))+(((IkReal(-1.00000000000000))*(r00)*(x1282)*(x1283)))+(((IkReal(-1.00000000000000))*(x1286)*(x1287)))+(((x1280)*(x1281)))+(((x1281)*(x1285))));
evalcond[2]=((((x1285)*(x1286)))+(((r02)*(x1281)))+(((x1284)*(x1285)))+(((x1280)*(x1286)))+(((x1280)*(x1284)))+(((IkReal(-1.00000000000000))*(x1283)*(x1287)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1288=((cj1)*(sj2));
IkReal x1289=((r01)*(sj0));
IkReal x1290=((cj0)*(r00));
IkReal x1291=((cj2)*(sj1));
if( IKabs(((((x1288)*(x1289)))+(((x1290)*(x1291)))+(((x1288)*(x1290)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1289)*(x1291))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1288)*(x1289)))+(((x1290)*(x1291)))+(((x1288)*(x1290)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1289)*(x1291)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1288)*(x1289)))+(((x1290)*(x1291)))+(((x1288)*(x1290)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x1289)*(x1291)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1292=IKcos(j5);
IkReal x1293=((r01)*(sj0));
IkReal x1294=((sj1)*(sj2));
IkReal x1295=((IkReal(1.00000000000000))*(cj0));
IkReal x1296=((cj1)*(cj2));
IkReal x1297=((cj1)*(sj2));
IkReal x1298=((cj0)*(r00));
IkReal x1299=((cj2)*(sj1));
IkReal x1300=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1295)))+(((r00)*(sj0)))+(((cj4)*(x1292))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x1295)*(x1296)))+(((IkReal(-1.00000000000000))*(x1299)*(x1300)))+(((x1294)*(x1298)))+(((IkReal(-1.00000000000000))*(x1297)*(x1300)))+(((x1293)*(x1294)))+(((IkReal(-1.00000000000000))*(x1293)*(x1296)))+(((sj4)*(x1292))));
evalcond[2]=((((x1297)*(x1298)))+(((x1293)*(x1299)))+(((x1293)*(x1297)))+(((IkReal(-1.00000000000000))*(x1296)*(x1300)))+(((r02)*(x1294)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((x1298)*(x1299))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1301=((pz)*(sj2));
IkReal x1302=((IkReal(1.00000000000000))*(cj1));
IkReal x1303=((cj2)*(sj1));
IkReal x1304=((py)*(sj0));
IkReal x1305=((cj0)*(px));
IkReal x1306=((sj1)*(sj2));
IkReal x1307=((IkReal(0.0750000000000000))*(cj1));
IkReal x1308=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((x1305)*(x1308)))+(((IkReal(-0.270000000000000))*(cj2)))+(((x1303)*(x1304)))+(((x1303)*(x1305)))+(((IkReal(-0.0750000000000000))*(x1303)))+(((x1304)*(x1308)))+(((IkReal(-1.00000000000000))*(sj2)*(x1307)))+(((IkReal(0.0800000000000000))*(cj3)))+(((sj1)*(x1301)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1302))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x1302)*(x1304)))+(((IkReal(-1.00000000000000))*(cj2)*(x1302)*(x1305)))+(((x1305)*(x1306)))+(((IkReal(-1.00000000000000))*(pz)*(x1303)))+(((IkReal(-0.0750000000000000))*(x1306)))+(((IkReal(-0.270000000000000))*(sj2)))+(((x1304)*(x1306)))+(((cj2)*(x1307)))+(((IkReal(-1.00000000000000))*(x1301)*(x1302))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1309=((cj0)*(r00));
IkReal x1310=((cj1)*(cj2));
IkReal x1311=((cj2)*(sj1));
IkReal x1312=((r01)*(sj0));
IkReal x1313=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x1314=((IkReal(1.00000000000000))*(cj1)*(sj2));
if( IKabs(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r02)*(x1313)))+(((IkReal(-1.00000000000000))*(x1309)*(x1311)))+(((r02)*(x1310)))+(((IkReal(-1.00000000000000))*(x1311)*(x1312)))+(((IkReal(-1.00000000000000))*(x1312)*(x1314)))+(((IkReal(-1.00000000000000))*(x1309)*(x1314))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1310)*(x1312)))+(((x1309)*(x1310)))+(((r02)*(x1311)))+(((IkReal(-1.00000000000000))*(x1312)*(x1313)))+(((IkReal(-1.00000000000000))*(x1309)*(x1313)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r02)*(x1313)))+(((IkReal(-1.00000000000000))*(x1309)*(x1311)))+(((r02)*(x1310)))+(((IkReal(-1.00000000000000))*(x1311)*(x1312)))+(((IkReal(-1.00000000000000))*(x1312)*(x1314)))+(((IkReal(-1.00000000000000))*(x1309)*(x1314)))))))+IKsqr(((((x1310)*(x1312)))+(((x1309)*(x1310)))+(((r02)*(x1311)))+(((IkReal(-1.00000000000000))*(x1312)*(x1313)))+(((IkReal(-1.00000000000000))*(x1309)*(x1313)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r02)*(x1313)))+(((IkReal(-1.00000000000000))*(x1309)*(x1311)))+(((r02)*(x1310)))+(((IkReal(-1.00000000000000))*(x1311)*(x1312)))+(((IkReal(-1.00000000000000))*(x1312)*(x1314)))+(((IkReal(-1.00000000000000))*(x1309)*(x1314)))))), ((((x1310)*(x1312)))+(((x1309)*(x1310)))+(((r02)*(x1311)))+(((IkReal(-1.00000000000000))*(x1312)*(x1313)))+(((IkReal(-1.00000000000000))*(x1309)*(x1313)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1315=IKsin(j5);
IkReal x1316=((r01)*(sj0));
IkReal x1317=((sj1)*(sj2));
IkReal x1318=((IkReal(1.00000000000000))*(cj0));
IkReal x1319=((cj1)*(cj2));
IkReal x1320=((cj1)*(sj2));
IkReal x1321=((cj0)*(r00));
IkReal x1322=((cj2)*(sj1));
IkReal x1323=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1318)))+(((IkReal(-1.00000000000000))*(cj3)*(x1315))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x1318)*(x1319)))+(((x1316)*(x1317)))+(((IkReal(-1.00000000000000))*(x1322)*(x1323)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(x1316)*(x1319)))+(((IkReal(-1.00000000000000))*(x1320)*(x1323)))+(((x1317)*(x1321))));
evalcond[2]=((((x1316)*(x1320)))+(((x1316)*(x1322)))+(((sj3)*(x1315)))+(((IkReal(-1.00000000000000))*(x1319)*(x1323)))+(((x1320)*(x1321)))+(((r02)*(x1317)))+(((x1321)*(x1322))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1324=((IkReal(1.00000000000000))*(r01));
IkReal x1325=((cj0)*(r00));
IkReal x1326=((cj1)*(cj2));
IkReal x1327=((sj1)*(sj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1324))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r01)*(sj0)*(x1326)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(sj0)*(x1324)*(x1327)))+(((x1325)*(x1326)))+(((IkReal(-1.00000000000000))*(x1325)*(x1327)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1324)))))))+IKsqr(((((r01)*(sj0)*(x1326)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(sj0)*(x1324)*(x1327)))+(((x1325)*(x1326)))+(((IkReal(-1.00000000000000))*(x1325)*(x1327)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1324)))))), ((((r01)*(sj0)*(x1326)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(sj0)*(x1324)*(x1327)))+(((x1325)*(x1326)))+(((IkReal(-1.00000000000000))*(x1325)*(x1327)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1328=IKsin(j5);
IkReal x1329=((r01)*(sj0));
IkReal x1330=((sj1)*(sj2));
IkReal x1331=((IkReal(1.00000000000000))*(cj0));
IkReal x1332=((cj1)*(cj2));
IkReal x1333=((cj1)*(sj2));
IkReal x1334=((cj0)*(r00));
IkReal x1335=((cj2)*(sj1));
IkReal x1336=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj3)*(x1328)))+(((IkReal(-1.00000000000000))*(r01)*(x1331))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1335)*(x1336)))+(((x1329)*(x1330)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(x1329)*(x1332)))+(((IkReal(-1.00000000000000))*(x1333)*(x1336)))+(((x1330)*(x1334)))+(((IkReal(-1.00000000000000))*(r00)*(x1331)*(x1332))));
evalcond[2]=((((x1329)*(x1333)))+(((x1329)*(x1335)))+(((sj3)*(x1328)))+(((IkReal(-1.00000000000000))*(x1332)*(x1336)))+(((r02)*(x1330)))+(((x1333)*(x1334)))+(((x1334)*(x1335))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1337=((pz)*(sj2));
IkReal x1338=((IkReal(1.00000000000000))*(cj1));
IkReal x1339=((cj2)*(sj1));
IkReal x1340=((py)*(sj0));
IkReal x1341=((cj0)*(px));
IkReal x1342=((sj1)*(sj2));
IkReal x1343=((IkReal(0.0750000000000000))*(cj1));
IkReal x1344=((cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((x1339)*(x1340)))+(((x1339)*(x1341)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1343)))+(((x1340)*(x1344)))+(((x1341)*(x1344)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1338)))+(((sj1)*(x1337)))+(((IkReal(-0.0750000000000000))*(x1339)))+(((IkReal(-0.0800000000000000))*(cj3))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(x1338)*(x1341)))+(((IkReal(-1.00000000000000))*(cj2)*(x1338)*(x1340)))+(((IkReal(-1.00000000000000))*(x1337)*(x1338)))+(((IkReal(-0.0750000000000000))*(x1342)))+(((x1340)*(x1342)))+(((x1341)*(x1342)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1339)))+(((cj2)*(x1343))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1345=((sj1)*(sj2));
IkReal x1346=((IkReal(1.00000000000000))*(r02));
IkReal x1347=((r01)*(sj0));
IkReal x1348=((cj0)*(r00));
IkReal x1349=((cj1)*(sj2));
IkReal x1350=((cj2)*(sj1));
IkReal x1351=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1347)*(x1349)))+(((IkReal(-1.00000000000000))*(x1347)*(x1350)))+(((IkReal(-1.00000000000000))*(x1348)*(x1350)))+(((IkReal(-1.00000000000000))*(x1348)*(x1349)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1345)*(x1346))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1346)*(x1349)))+(((IkReal(-1.00000000000000))*(x1346)*(x1350)))+(((x1345)*(x1347)))+(((x1345)*(x1348)))+(((IkReal(-1.00000000000000))*(x1347)*(x1351)))+(((IkReal(-1.00000000000000))*(x1348)*(x1351))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1347)*(x1349)))+(((IkReal(-1.00000000000000))*(x1347)*(x1350)))+(((IkReal(-1.00000000000000))*(x1348)*(x1350)))+(((IkReal(-1.00000000000000))*(x1348)*(x1349)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1345)*(x1346)))))))+IKsqr(((((IkReal(-1.00000000000000))*(x1346)*(x1349)))+(((IkReal(-1.00000000000000))*(x1346)*(x1350)))+(((x1345)*(x1347)))+(((x1345)*(x1348)))+(((IkReal(-1.00000000000000))*(x1347)*(x1351)))+(((IkReal(-1.00000000000000))*(x1348)*(x1351)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1347)*(x1349)))+(((IkReal(-1.00000000000000))*(x1347)*(x1350)))+(((IkReal(-1.00000000000000))*(x1348)*(x1350)))+(((IkReal(-1.00000000000000))*(x1348)*(x1349)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x1345)*(x1346)))))), ((((IkReal(-1.00000000000000))*(x1346)*(x1349)))+(((IkReal(-1.00000000000000))*(x1346)*(x1350)))+(((x1345)*(x1347)))+(((x1345)*(x1348)))+(((IkReal(-1.00000000000000))*(x1347)*(x1351)))+(((IkReal(-1.00000000000000))*(x1348)*(x1351)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1352=IKsin(j5);
IkReal x1353=((r01)*(sj0));
IkReal x1354=((sj1)*(sj2));
IkReal x1355=((IkReal(1.00000000000000))*(cj0));
IkReal x1356=((cj1)*(cj2));
IkReal x1357=((cj1)*(sj2));
IkReal x1358=((cj0)*(r00));
IkReal x1359=((cj2)*(sj1));
IkReal x1360=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1355)))+(((IkReal(-1.00000000000000))*(cj3)*(x1352))));
evalcond[1]=((((x1354)*(x1358)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x1359)*(x1360)))+(((IkReal(-1.00000000000000))*(x1353)*(x1356)))+(((IkReal(-1.00000000000000))*(x1357)*(x1360)))+(((IkReal(-1.00000000000000))*(r00)*(x1355)*(x1356)))+(((x1353)*(x1354))));
evalcond[2]=((((x1357)*(x1358)))+(((x1358)*(x1359)))+(((r02)*(x1354)))+(((IkReal(-1.00000000000000))*(x1356)*(x1360)))+(((sj3)*(x1352)))+(((x1353)*(x1357)))+(((x1353)*(x1359))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1361=((r01)*(sj0));
IkReal x1362=((sj1)*(sj2));
IkReal x1363=((IkReal(1.00000000000000))*(cj0));
IkReal x1364=((cj1)*(cj2));
IkReal x1365=((IkReal(1.00000000000000))*(r02));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1363))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(x1363)*(x1364)))+(((cj0)*(r00)*(x1362)))+(((x1361)*(x1362)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1365)))+(((IkReal(-1.00000000000000))*(x1361)*(x1364)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1365))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1363)))))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(x1363)*(x1364)))+(((cj0)*(r00)*(x1362)))+(((x1361)*(x1362)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1365)))+(((IkReal(-1.00000000000000))*(x1361)*(x1364)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1365)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1363)))))), ((((IkReal(-1.00000000000000))*(r00)*(x1363)*(x1364)))+(((cj0)*(r00)*(x1362)))+(((x1361)*(x1362)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1365)))+(((IkReal(-1.00000000000000))*(x1361)*(x1364)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1365)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1366=IKsin(j5);
IkReal x1367=((r01)*(sj0));
IkReal x1368=((sj1)*(sj2));
IkReal x1369=((IkReal(1.00000000000000))*(cj0));
IkReal x1370=((cj1)*(cj2));
IkReal x1371=((cj1)*(sj2));
IkReal x1372=((cj0)*(r00));
IkReal x1373=((cj2)*(sj1));
IkReal x1374=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1369)))+(((IkReal(-1.00000000000000))*(cj3)*(x1366))));
evalcond[1]=((((x1367)*(x1368)))+(((IkReal(-1.00000000000000))*(x1371)*(x1374)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x1367)*(x1370)))+(((x1368)*(x1372)))+(((IkReal(-1.00000000000000))*(r00)*(x1369)*(x1370)))+(((IkReal(-1.00000000000000))*(x1373)*(x1374))));
evalcond[2]=((((x1372)*(x1373)))+(((IkReal(-1.00000000000000))*(x1370)*(x1374)))+(((r02)*(x1368)))+(((x1367)*(x1371)))+(((x1367)*(x1373)))+(((x1371)*(x1372)))+(((sj3)*(x1366))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1375=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1376=((r01)*(sj0));
IkReal x1377=((r02)*(sj1));
IkReal x1378=((cj1)*(cj2));
IkReal x1379=((cj3)*(cj4));
IkReal x1380=((cj0)*(r00));
IkReal x1381=((IkReal(1.00000000000000))*(sj2)*(sj4));
IkReal x1382=((cj1)*(x1380));
IkReal x1383=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x1384=((cj1)*(r02)*(sj2));
IkReal x1385=((IkReal(1.00000000000000))*(cj2)*(sj1)*(sj4));
if( IKabs(((x1375)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1379)*(x1380)*(x1383)))+(((IkReal(-1.00000000000000))*(x1376)*(x1379)*(x1383)))+(((x1376)*(x1378)*(x1379)))+(((IkReal(-1.00000000000000))*(cj1)*(x1376)*(x1381)))+(((IkReal(-1.00000000000000))*(x1377)*(x1381)))+(((IkReal(-1.00000000000000))*(x1376)*(x1385)))+(((cj2)*(x1377)*(x1379)))+(((IkReal(-1.00000000000000))*(x1380)*(x1385)))+(((x1378)*(x1379)*(x1380)))+(((IkReal(-1.00000000000000))*(x1381)*(x1382)))+(((x1379)*(x1384)))+(((r02)*(sj4)*(x1378))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1375)*(((x1384)+(((x1376)*(x1378)))+(((IkReal(-1.00000000000000))*(x1376)*(x1383)))+(((IkReal(-1.00000000000000))*(x1380)*(x1383)))+(((cj2)*(x1377)))+(((x1378)*(x1380))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1375)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1379)*(x1380)*(x1383)))+(((IkReal(-1.00000000000000))*(x1376)*(x1379)*(x1383)))+(((x1376)*(x1378)*(x1379)))+(((IkReal(-1.00000000000000))*(cj1)*(x1376)*(x1381)))+(((IkReal(-1.00000000000000))*(x1377)*(x1381)))+(((IkReal(-1.00000000000000))*(x1376)*(x1385)))+(((cj2)*(x1377)*(x1379)))+(((IkReal(-1.00000000000000))*(x1380)*(x1385)))+(((x1378)*(x1379)*(x1380)))+(((IkReal(-1.00000000000000))*(x1381)*(x1382)))+(((x1379)*(x1384)))+(((r02)*(sj4)*(x1378)))))))+IKsqr(((x1375)*(((x1384)+(((x1376)*(x1378)))+(((IkReal(-1.00000000000000))*(x1376)*(x1383)))+(((IkReal(-1.00000000000000))*(x1380)*(x1383)))+(((cj2)*(x1377)))+(((x1378)*(x1380)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1375)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1379)*(x1380)*(x1383)))+(((IkReal(-1.00000000000000))*(x1376)*(x1379)*(x1383)))+(((x1376)*(x1378)*(x1379)))+(((IkReal(-1.00000000000000))*(cj1)*(x1376)*(x1381)))+(((IkReal(-1.00000000000000))*(x1377)*(x1381)))+(((IkReal(-1.00000000000000))*(x1376)*(x1385)))+(((cj2)*(x1377)*(x1379)))+(((IkReal(-1.00000000000000))*(x1380)*(x1385)))+(((x1378)*(x1379)*(x1380)))+(((IkReal(-1.00000000000000))*(x1381)*(x1382)))+(((x1379)*(x1384)))+(((r02)*(sj4)*(x1378)))))), ((x1375)*(((x1384)+(((x1376)*(x1378)))+(((IkReal(-1.00000000000000))*(x1376)*(x1383)))+(((IkReal(-1.00000000000000))*(x1380)*(x1383)))+(((cj2)*(x1377)))+(((x1378)*(x1380)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1386=IKcos(j5);
IkReal x1387=IKsin(j5);
IkReal x1388=((r01)*(sj0));
IkReal x1389=((sj1)*(sj2));
IkReal x1390=((IkReal(1.00000000000000))*(cj0));
IkReal x1391=((cj1)*(cj2));
IkReal x1392=((cj1)*(sj2));
IkReal x1393=((cj0)*(r00));
IkReal x1394=((cj2)*(sj1));
IkReal x1395=((IkReal(1.00000000000000))*(cj3));
IkReal x1396=((IkReal(1.00000000000000))*(r02));
IkReal x1397=((cj4)*(x1386));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1397)))+(((IkReal(-1.00000000000000))*(r01)*(x1390)))+(((IkReal(-1.00000000000000))*(x1387)*(x1395))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1392)*(x1396)))+(((IkReal(-1.00000000000000))*(x1388)*(x1391)))+(((sj4)*(x1386)))+(((x1388)*(x1389)))+(((IkReal(-1.00000000000000))*(x1394)*(x1396)))+(((IkReal(-1.00000000000000))*(r00)*(x1390)*(x1391)))+(((x1389)*(x1393))));
evalcond[2]=((((x1388)*(x1392)))+(((x1388)*(x1394)))+(((x1393)*(x1394)))+(((IkReal(-1.00000000000000))*(x1391)*(x1396)))+(((IkReal(-1.00000000000000))*(x1395)*(x1397)))+(((sj3)*(x1387)))+(((x1392)*(x1393)))+(((r02)*(x1389))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1398=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1399=((r01)*(sj0));
IkReal x1400=((r02)*(sj1));
IkReal x1401=((IkReal(1.00000000000000))*(sj2));
IkReal x1402=((cj4)*(sj3));
IkReal x1403=((cj1)*(r02));
IkReal x1404=((sj1)*(sj2));
IkReal x1405=((cj0)*(r00));
IkReal x1406=((cj1)*(x1405));
IkReal x1407=((IkReal(1.00000000000000))*(cj2)*(x1402));
if( IKabs(((x1398)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1406)*(x1407)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1400)*(x1407)))+(((IkReal(-1.00000000000000))*(cj1)*(x1399)*(x1407)))+(((x1399)*(x1402)*(x1404)))+(((x1402)*(x1404)*(x1405)))+(((IkReal(-1.00000000000000))*(x1401)*(x1402)*(x1403))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1398)*(((((cj1)*(cj2)*(x1399)))+(((IkReal(-1.00000000000000))*(sj1)*(x1399)*(x1401)))+(((IkReal(-1.00000000000000))*(sj1)*(x1401)*(x1405)))+(((sj2)*(x1403)))+(((cj2)*(x1406)))+(((cj2)*(x1400))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1398)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1406)*(x1407)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1400)*(x1407)))+(((IkReal(-1.00000000000000))*(cj1)*(x1399)*(x1407)))+(((x1399)*(x1402)*(x1404)))+(((x1402)*(x1404)*(x1405)))+(((IkReal(-1.00000000000000))*(x1401)*(x1402)*(x1403)))))))+IKsqr(((x1398)*(((((cj1)*(cj2)*(x1399)))+(((IkReal(-1.00000000000000))*(sj1)*(x1399)*(x1401)))+(((IkReal(-1.00000000000000))*(sj1)*(x1401)*(x1405)))+(((sj2)*(x1403)))+(((cj2)*(x1406)))+(((cj2)*(x1400)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1398)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1406)*(x1407)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1400)*(x1407)))+(((IkReal(-1.00000000000000))*(cj1)*(x1399)*(x1407)))+(((x1399)*(x1402)*(x1404)))+(((x1402)*(x1404)*(x1405)))+(((IkReal(-1.00000000000000))*(x1401)*(x1402)*(x1403)))))), ((x1398)*(((((cj1)*(cj2)*(x1399)))+(((IkReal(-1.00000000000000))*(sj1)*(x1399)*(x1401)))+(((IkReal(-1.00000000000000))*(sj1)*(x1401)*(x1405)))+(((sj2)*(x1403)))+(((cj2)*(x1406)))+(((cj2)*(x1400)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1408=IKcos(j5);
IkReal x1409=IKsin(j5);
IkReal x1410=((r01)*(sj0));
IkReal x1411=((sj1)*(sj2));
IkReal x1412=((IkReal(1.00000000000000))*(cj0));
IkReal x1413=((cj1)*(cj2));
IkReal x1414=((cj1)*(sj2));
IkReal x1415=((cj0)*(r00));
IkReal x1416=((cj2)*(sj1));
IkReal x1417=((IkReal(1.00000000000000))*(cj3));
IkReal x1418=((IkReal(1.00000000000000))*(r02));
IkReal x1419=((cj4)*(x1408));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1419)))+(((IkReal(-1.00000000000000))*(r01)*(x1412)))+(((IkReal(-1.00000000000000))*(x1409)*(x1417))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1410)*(x1413)))+(((IkReal(-1.00000000000000))*(x1414)*(x1418)))+(((IkReal(-1.00000000000000))*(x1416)*(x1418)))+(((x1411)*(x1415)))+(((x1410)*(x1411)))+(((sj4)*(x1408)))+(((IkReal(-1.00000000000000))*(r00)*(x1412)*(x1413))));
evalcond[2]=((((x1415)*(x1416)))+(((IkReal(-1.00000000000000))*(x1413)*(x1418)))+(((IkReal(-1.00000000000000))*(x1417)*(x1419)))+(((sj3)*(x1409)))+(((x1410)*(x1414)))+(((x1410)*(x1416)))+(((r02)*(x1411)))+(((x1414)*(x1415))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1420=(sj3)*(sj3);
IkReal x1421=(cj3)*(cj3);
IkReal x1422=((cj2)*(cj3));
IkReal x1423=((cj1)*(r02));
IkReal x1424=((r01)*(sj0));
IkReal x1425=((cj3)*(sj2));
IkReal x1426=((r02)*(sj1));
IkReal x1427=((cj0)*(r00));
IkReal x1428=((r00)*(sj0));
IkReal x1429=((sj1)*(x1427));
IkReal x1430=((IkReal(1.00000000000000))*(cj2)*(sj3));
IkReal x1431=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x1432=((IkReal(1.00000000000000))*(cj1)*(sj2)*(sj3));
if( IKabs(((((IKabs(((x1421)+(x1420))) != 0)?((IkReal)1/(((x1421)+(x1420)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1426)))+(((IkReal(-1.00000000000000))*(x1429)*(x1430)))+(((IkReal(-1.00000000000000))*(cj3)*(x1431)))+(((cj2)*(sj3)*(x1423)))+(((cj3)*(x1428)))+(((IkReal(-1.00000000000000))*(sj1)*(x1424)*(x1430)))+(((IkReal(-1.00000000000000))*(x1424)*(x1432)))+(((IkReal(-1.00000000000000))*(x1427)*(x1432))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1420)))+(((cj4)*(x1421))))) != 0)?((IkReal)1/(((((cj4)*(x1420)))+(((cj4)*(x1421)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x1431)))+(((x1425)*(x1426)))+(((IkReal(-1.00000000000000))*(x1422)*(x1423)))+(((x1422)*(x1429)))+(((cj1)*(x1424)*(x1425)))+(((cj1)*(x1425)*(x1427)))+(((sj1)*(x1422)*(x1424)))+(((sj3)*(x1428))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x1421)+(x1420))) != 0)?((IkReal)1/(((x1421)+(x1420)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1426)))+(((IkReal(-1.00000000000000))*(x1429)*(x1430)))+(((IkReal(-1.00000000000000))*(cj3)*(x1431)))+(((cj2)*(sj3)*(x1423)))+(((cj3)*(x1428)))+(((IkReal(-1.00000000000000))*(sj1)*(x1424)*(x1430)))+(((IkReal(-1.00000000000000))*(x1424)*(x1432)))+(((IkReal(-1.00000000000000))*(x1427)*(x1432)))))))+IKsqr(((((IKabs(((((cj4)*(x1420)))+(((cj4)*(x1421))))) != 0)?((IkReal)1/(((((cj4)*(x1420)))+(((cj4)*(x1421)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x1431)))+(((x1425)*(x1426)))+(((IkReal(-1.00000000000000))*(x1422)*(x1423)))+(((x1422)*(x1429)))+(((cj1)*(x1424)*(x1425)))+(((cj1)*(x1425)*(x1427)))+(((sj1)*(x1422)*(x1424)))+(((sj3)*(x1428)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x1421)+(x1420))) != 0)?((IkReal)1/(((x1421)+(x1420)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1426)))+(((IkReal(-1.00000000000000))*(x1429)*(x1430)))+(((IkReal(-1.00000000000000))*(cj3)*(x1431)))+(((cj2)*(sj3)*(x1423)))+(((cj3)*(x1428)))+(((IkReal(-1.00000000000000))*(sj1)*(x1424)*(x1430)))+(((IkReal(-1.00000000000000))*(x1424)*(x1432)))+(((IkReal(-1.00000000000000))*(x1427)*(x1432)))))), ((((IKabs(((((cj4)*(x1420)))+(((cj4)*(x1421))))) != 0)?((IkReal)1/(((((cj4)*(x1420)))+(((cj4)*(x1421)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x1431)))+(((x1425)*(x1426)))+(((IkReal(-1.00000000000000))*(x1422)*(x1423)))+(((x1422)*(x1429)))+(((cj1)*(x1424)*(x1425)))+(((cj1)*(x1425)*(x1427)))+(((sj1)*(x1422)*(x1424)))+(((sj3)*(x1428)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x1433=IKcos(j5);
IkReal x1434=IKsin(j5);
IkReal x1435=((r01)*(sj0));
IkReal x1436=((sj1)*(sj2));
IkReal x1437=((IkReal(1.00000000000000))*(cj0));
IkReal x1438=((cj1)*(cj2));
IkReal x1439=((cj1)*(sj2));
IkReal x1440=((cj0)*(r00));
IkReal x1441=((cj2)*(sj1));
IkReal x1442=((IkReal(1.00000000000000))*(cj3));
IkReal x1443=((IkReal(1.00000000000000))*(r02));
IkReal x1444=((cj4)*(x1433));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1434)*(x1442)))+(((IkReal(-1.00000000000000))*(r01)*(x1437)))+(((IkReal(-1.00000000000000))*(sj3)*(x1444))));
evalcond[1]=((((x1435)*(x1436)))+(((IkReal(-1.00000000000000))*(x1435)*(x1438)))+(((IkReal(-1.00000000000000))*(x1439)*(x1443)))+(((IkReal(-1.00000000000000))*(x1441)*(x1443)))+(((x1436)*(x1440)))+(((sj4)*(x1433)))+(((IkReal(-1.00000000000000))*(r00)*(x1437)*(x1438))));
evalcond[2]=((((r02)*(x1436)))+(((x1439)*(x1440)))+(((x1435)*(x1441)))+(((x1440)*(x1441)))+(((IkReal(-1.00000000000000))*(x1438)*(x1443)))+(((x1435)*(x1439)))+(((sj3)*(x1434)))+(((IkReal(-1.00000000000000))*(x1442)*(x1444))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "f39e879836615ea489f5611d5593ce3c"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
