/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:27:03.542900
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[3]);
x3=IKcos(j[2]);
x4=IKsin(j[0]);
x5=IKsin(j[1]);
x6=IKsin(j[2]);
x7=IKsin(j[3]);
x8=IKsin(j[4]);
x9=IKcos(j[4]);
x10=((IkReal(0.0440000000000000))*(x3));
x11=((IkReal(0.845725967038483))*(x7));
x12=((IkReal(0.135000000000000))*(x0));
x13=((IkReal(0.533617455371189))*(x8));
x14=((IkReal(0.533617455371189))*(x9));
x15=((IkReal(0.0440000000000000))*(x5));
x16=((IkReal(0.0440000000000000))*(x2));
x17=((IkReal(0.0440000000000000))*(x1));
x18=((IkReal(1.00000000000000))*(x4));
x19=((IkReal(0.135000000000000))*(x4));
x20=((x1)*(x2));
x21=((x4)*(x6));
x22=((x5)*(x7));
x23=((x0)*(x6));
x24=((x1)*(x3));
x25=((x3)*(x5));
x26=((x2)*(x5));
x27=((x1)*(x4));
x28=((x0)*(x7));
x29=((((IkReal(-1.00000000000000))*(x18)*(x25)))+(x23));
x30=((((IkReal(-1.00000000000000))*(x18)*(x6)))+(((IkReal(-1.00000000000000))*(x0)*(x25))));
eetrans[0]=((((IkReal(0.100000000000000))*(x0)))+(((IkReal(0.400000000000000))*(x0)*(x1)))+(((x9)*(((((IkReal(-1.00000000000000))*(x16)*(x30)))+(((x17)*(x28)))))))+(((x7)*(((((IkReal(-1.00000000000000))*(x19)*(x6)))+(((IkReal(-1.00000000000000))*(x12)*(x25)))))))+(((x12)*(x20)))+(((x8)*(((((IkReal(-1.00000000000000))*(x15)*(x23)))+(((x10)*(x4))))))));
eetrans[1]=((IkReal(0.188000000000000))+(((x19)*(x20)))+(((IkReal(0.100000000000000))*(x4)))+(((x7)*(((((IkReal(-1.00000000000000))*(x19)*(x25)))+(((x12)*(x6)))))))+(((IkReal(0.400000000000000))*(x27)))+(((x9)*(((((x17)*(x4)*(x7)))+(((IkReal(-1.00000000000000))*(x16)*(x29)))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x15)*(x21)))+(((IkReal(-1.00000000000000))*(x0)*(x10))))))));
eetrans[2]=((((IkReal(-0.135000000000000))*(x24)*(x7)))+(((IkReal(-1.00000000000000))*(x17)*(x6)*(x8)))+(((IkReal(-0.135000000000000))*(x26)))+(((IkReal(-0.400000000000000))*(x5)))+(((x9)*(((((x10)*(x20)))+(((IkReal(-1.00000000000000))*(x15)*(x7))))))));
eerot[0]=((((IkReal(0.845725967038483))*(x0)*(x20)))+(((x11)*(x30)))+(((IkReal(-1.00000000000000))*(x13)*(((((IkReal(-1.00000000000000))*(x18)*(x3)))+(((x23)*(x5)))))))+(((IkReal(-1.00000000000000))*(x14)*(((((IkReal(-1.00000000000000))*(x1)*(x28)))+(((x2)*(x30))))))));
eerot[1]=((((x11)*(x29)))+(((IkReal(-1.00000000000000))*(x13)*(((((x0)*(x3)))+(((x21)*(x5)))))))+(((IkReal(0.845725967038483))*(x20)*(x4)))+(((IkReal(-1.00000000000000))*(x14)*(((((x2)*(x29)))+(((IkReal(-1.00000000000000))*(x27)*(x7))))))));
eerot[2]=((((IkReal(-0.845725967038483))*(x26)))+(((IkReal(-1.00000000000000))*(x1)*(x13)*(x6)))+(((IkReal(-1.00000000000000))*(x11)*(x24)))+(((IkReal(-1.00000000000000))*(x14)*(((x22)+(((IkReal(-1.00000000000000))*(x20)*(x3))))))));
}

IKFAST_API int GetNumFreeParameters() { return 0; }
IKFAST_API int* GetFreeParameters() { return NULL; }
IKFAST_API int GetNumJoints() { return 5; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j15,cj15,sj15,htj15,j16,cj16,sj16,htj16,j17,cj17,sj17,htj17,j18,cj18,sj18,htj18,j19,cj19,sj19,htj19,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij15[2], _nj15,_ij16[2], _nj16,_ij17[2], _nj17,_ij18[2], _nj18,_ij19[2], _nj19;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j15=numeric_limits<IkReal>::quiet_NaN(); _ij15[0] = -1; _ij15[1] = -1; _nj15 = -1; j16=numeric_limits<IkReal>::quiet_NaN(); _ij16[0] = -1; _ij16[1] = -1; _nj16 = -1; j17=numeric_limits<IkReal>::quiet_NaN(); _ij17[0] = -1; _ij17[1] = -1; _nj17 = -1; j18=numeric_limits<IkReal>::quiet_NaN(); _ij18[0] = -1; _ij18[1] = -1; _nj18 = -1; j19=numeric_limits<IkReal>::quiet_NaN(); _ij19[0] = -1; _ij19[1] = -1; _nj19 = -1; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=px;
new_r01=r01;
new_py=((IkReal(-0.188000000000000))+(py));
new_r02=r02;
new_pz=pz;
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(0.229839000000000))+(((IkReal(-1.00000000000000))*(px)))+(pp));
IkReal gconst1;
gconst1=((IkReal(1.60000000000000))*(pz));
IkReal gconst2;
gconst2=((IkReal(0.0698390000000000))+(((IkReal(0.600000000000000))*(px)))+(pp));
IkReal gconst3;
gconst3=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst4;
gconst4=((IkReal(0.800000000000000))*(r02));
IkReal gconst5;
gconst5=((IkReal(-0.137652173586528))+(((IkReal(0.300000000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst6;
gconst6=((IkReal(0.229839000000000))+(((IkReal(-1.00000000000000))*(px)))+(pp));
IkReal gconst7;
gconst7=((IkReal(1.60000000000000))*(pz));
IkReal gconst8;
gconst8=((IkReal(0.0698390000000000))+(((IkReal(0.600000000000000))*(px)))+(pp));
IkReal gconst9;
gconst9=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst10;
gconst10=((IkReal(0.800000000000000))*(r02));
IkReal gconst11;
gconst11=((IkReal(-0.137652173586528))+(((IkReal(0.300000000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst12;
gconst12=((IkReal(-2.00000000000000))*(py));
IkReal gconst13;
gconst13=((IkReal(1.20000000000000))*(py));
IkReal gconst14;
gconst14=((IkReal(-1.00000000000000))*(r01));
IkReal gconst15;
gconst15=((IkReal(0.600000000000000))*(r01));
IkReal gconst16;
gconst16=((IkReal(-2.00000000000000))*(py));
IkReal gconst17;
gconst17=((IkReal(1.20000000000000))*(py));
IkReal gconst18;
gconst18=((IkReal(-1.00000000000000))*(r01));
IkReal gconst19;
gconst19=((IkReal(0.600000000000000))*(r01));
IkReal gconst20;
gconst20=((IkReal(0.229839000000000))+(px)+(pp));
IkReal gconst21;
gconst21=((IkReal(1.60000000000000))*(pz));
IkReal gconst22;
gconst22=((IkReal(0.0698390000000000))+(((IkReal(-0.600000000000000))*(px)))+(pp));
IkReal gconst23;
gconst23=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst24;
gconst24=((IkReal(0.800000000000000))*(r02));
IkReal gconst25;
gconst25=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.300000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst26;
gconst26=((IkReal(0.229839000000000))+(px)+(pp));
IkReal gconst27;
gconst27=((IkReal(1.60000000000000))*(pz));
IkReal gconst28;
gconst28=((IkReal(0.0698390000000000))+(((IkReal(-0.600000000000000))*(px)))+(pp));
IkReal gconst29;
gconst29=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst30;
gconst30=((IkReal(0.800000000000000))*(r02));
IkReal gconst31;
gconst31=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.300000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal x31=((gconst27)*(gconst29));
IkReal x32=((gconst22)*(gconst4));
IkReal x33=((IkReal(1.00000000000000))*(gconst25));
IkReal x34=((gconst19)*(gconst26));
IkReal x35=((IkReal(1.00000000000000))*(gconst3));
IkReal x36=((gconst8)*(gconst9));
IkReal x37=((gconst17)*(gconst18));
IkReal x38=((gconst20)*(gconst25));
IkReal x39=((gconst0)*(gconst5));
IkReal x40=((IkReal(1.00000000000000))*(gconst11));
IkReal x41=((gconst22)*(gconst23));
IkReal x42=((gconst16)*(gconst19));
IkReal x43=((gconst27)*(gconst9));
IkReal x44=((gconst11)*(gconst6));
IkReal x45=((gconst13)*(gconst3));
IkReal x46=((gconst2)*(gconst8));
IkReal x47=((gconst14)*(gconst2));
IkReal x48=((gconst11)*(gconst16));
IkReal x49=((gconst7)*(gconst9));
IkReal x50=((gconst13)*(gconst4));
IkReal x51=((gconst15)*(gconst20));
IkReal x52=((gconst2)*(gconst23));
IkReal x53=((gconst28)*(gconst29));
IkReal x54=((gconst17)*(gconst9));
IkReal x55=((IkReal(1.00000000000000))*(gconst21));
IkReal x56=((gconst26)*(gconst31));
IkReal x57=((gconst22)*(gconst3));
IkReal x58=((gconst17)*(gconst29));
IkReal x59=((gconst12)*(gconst25));
IkReal x60=((IkReal(1.00000000000000))*(gconst5));
IkReal x61=((gconst19)*(gconst6));
IkReal x62=((gconst18)*(gconst28));
IkReal x63=((gconst29)*(gconst8));
IkReal x64=((gconst13)*(gconst23));
IkReal x65=((gconst10)*(gconst6));
IkReal x66=((gconst2)*(gconst24));
IkReal x67=((IkReal(1.00000000000000))*(gconst15));
IkReal x68=((gconst1)*(gconst25));
IkReal x69=((gconst22)*(gconst24));
IkReal x70=((IkReal(1.00000000000000))*(gconst14));
IkReal x71=((gconst0)*(gconst25));
IkReal x72=((gconst2)*(gconst3));
IkReal x73=((gconst26)*(gconst30));
IkReal x74=((gconst31)*(gconst6));
IkReal x75=((IkReal(1.00000000000000))*(gconst10));
IkReal x76=((gconst1)*(gconst7));
IkReal x77=((gconst15)*(gconst6));
IkReal x78=((gconst2)*(gconst4));
IkReal x79=((gconst1)*(gconst15));
IkReal x80=((gconst18)*(gconst7));
IkReal x81=((gconst12)*(gconst16));
IkReal x82=((gconst5)*(gconst6));
IkReal x83=((gconst18)*(gconst27));
IkReal x84=((gconst12)*(gconst5));
IkReal x85=((gconst28)*(gconst9));
IkReal x86=((gconst21)*(gconst5));
IkReal x87=((gconst1)*(gconst5));
IkReal x88=((gconst10)*(gconst26));
IkReal x89=((gconst16)*(gconst31));
IkReal x90=((gconst14)*(gconst22));
IkReal x91=((gconst13)*(gconst14));
IkReal x92=((gconst11)*(gconst26));
IkReal x93=((gconst12)*(gconst15));
IkReal x94=((gconst20)*(gconst5));
IkReal x95=((gconst0)*(gconst15));
IkReal x96=((gconst13)*(gconst24));
IkReal x97=((gconst18)*(gconst8));
IkReal x98=((gconst21)*(gconst25));
IkReal x99=((gconst29)*(gconst7));
IkReal x100=((gconst10)*(gconst16));
IkReal x101=((gconst20)*(x89));
IkReal x102=((gconst0)*(x97));
IkReal x103=((gconst15)*(gconst21)*(gconst30));
IkReal x104=((gconst18)*(x70));
IkReal x105=((IkReal(1.00000000000000))*(gconst30)*(gconst6));
IkReal x106=((gconst30)*(x98));
IkReal x107=((IkReal(1.00000000000000))*(gconst16)*(gconst30));
op[0]=((((x38)*(x53)))+(((IkReal(-1.00000000000000))*(x69)*(x73)))+(((x73)*(x98)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x56)))+(((IkReal(-1.00000000000000))*(x41)*(x53)))+(((x31)*(x69)))+(((IkReal(-1.00000000000000))*(gconst21)*(x31)*(x33)))+(((x41)*(x56))));
op[1]=((((x38)*(x58)))+(((x38)*(x62)))+(((x34)*(x41)))+(((IkReal(-1.00000000000000))*(x51)*(x56)))+(((x41)*(x89)))+(((gconst16)*(x106)))+(((IkReal(-1.00000000000000))*(x101)*(x33)))+(((IkReal(-1.00000000000000))*(gconst21)*(x33)*(x83)))+(((IkReal(-1.00000000000000))*(gconst15)*(x31)*(x55)))+(((x56)*(x90)))+(((IkReal(-1.00000000000000))*(gconst12)*(x33)*(x56)))+(((gconst15)*(gconst21)*(x73)))+(((x69)*(x83)))+(((IkReal(-1.00000000000000))*(x41)*(x58)))+(((IkReal(-1.00000000000000))*(x73)*(x96)))+(((x56)*(x64)))+(((x53)*(x59)))+(((IkReal(-1.00000000000000))*(x41)*(x62)))+(((IkReal(-1.00000000000000))*(x53)*(x64)))+(((IkReal(-1.00000000000000))*(gconst22)*(x53)*(x70)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x34)))+(((x51)*(x53)))+(((IkReal(-1.00000000000000))*(x107)*(x69)))+(((x31)*(x96))));
op[2]=((((IkReal(-1.00000000000000))*(x62)*(x64)))+(((IkReal(-1.00000000000000))*(x32)*(x73)))+(((IkReal(-1.00000000000000))*(x107)*(x96)))+(((x38)*(x63)))+(((x89)*(x90)))+(((x59)*(x62)))+(((gconst16)*(x103)))+(((x41)*(x42)))+(((IkReal(-1.00000000000000))*(gconst1)*(x31)*(x33)))+(((x34)*(x90)))+(((IkReal(-1.00000000000000))*(x105)*(x69)))+(((x64)*(x89)))+(((x53)*(x93)))+(((x53)*(x94)))+(((x58)*(x59)))+(((x37)*(x38)))+(((gconst6)*(x106)))+(((x41)*(x92)))+(((IkReal(-1.00000000000000))*(gconst21)*(x33)*(x99)))+(((x38)*(x85)))+(((x41)*(x74)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x74)))+(((x34)*(x64)))+(((IkReal(-1.00000000000000))*(gconst21)*(x33)*(x43)))+(((IkReal(-1.00000000000000))*(x52)*(x53)))+(((IkReal(-1.00000000000000))*(gconst0)*(x33)*(x56)))+(((x68)*(x73)))+(((IkReal(-1.00000000000000))*(gconst22)*(x58)*(x70)))+(((IkReal(-1.00000000000000))*(gconst20)*(x56)*(x60)))+(((x73)*(x86)))+(((x56)*(x91)))+(((x69)*(x99)))+(((IkReal(-1.00000000000000))*(x51)*(x89)))+(((IkReal(-1.00000000000000))*(x58)*(x64)))+(((IkReal(-1.00000000000000))*(gconst15)*(x55)*(x83)))+(((IkReal(-1.00000000000000))*(gconst5)*(x31)*(x55)))+(((IkReal(-1.00000000000000))*(gconst26)*(x69)*(x75)))+(((x31)*(x66)))+(((x88)*(x98)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x92)))+(((IkReal(-1.00000000000000))*(gconst31)*(x33)*(x81)))+(((IkReal(-1.00000000000000))*(gconst22)*(x62)*(x70)))+(((IkReal(-1.00000000000000))*(x41)*(x63)))+(((IkReal(-1.00000000000000))*(x37)*(x41)))+(((IkReal(-1.00000000000000))*(x66)*(x73)))+(((x56)*(x57)))+(((IkReal(-1.00000000000000))*(gconst12)*(x33)*(x34)))+(((IkReal(-1.00000000000000))*(x41)*(x85)))+(((x43)*(x69)))+(((x52)*(x56)))+(((x51)*(x58)))+(((x53)*(x71)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x42)))+(((x83)*(x96)))+(((x51)*(x62)))+(((x31)*(x32)))+(((IkReal(-1.00000000000000))*(x34)*(x51)))+(((IkReal(-1.00000000000000))*(gconst22)*(x35)*(x53)))+(((IkReal(-1.00000000000000))*(gconst12)*(x56)*(x67)))+(((IkReal(-1.00000000000000))*(gconst13)*(x53)*(x70))));
op[3]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst2)*(gconst28)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst20)*(gconst26)*(gconst5)))+(((gconst19)*(gconst2)*(gconst23)*(gconst26)))+(((gconst15)*(gconst21)*(gconst30)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst20)*(gconst25)))+(((gconst16)*(gconst21)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst20)*(gconst25)*(gconst6)))+(((gconst0)*(gconst15)*(gconst28)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst26)*(gconst31)*(gconst5)))+(((gconst15)*(gconst20)*(gconst29)*(gconst8)))+(((gconst14)*(gconst2)*(gconst26)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst23)*(gconst8)))+(((gconst13)*(gconst24)*(gconst27)*(gconst9)))+(((gconst13)*(gconst24)*(gconst29)*(gconst7)))+(((gconst18)*(gconst22)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst25)*(gconst26)))+(((gconst17)*(gconst20)*(gconst25)*(gconst9)))+(((gconst12)*(gconst25)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst21)*(gconst27)*(gconst5)))+(((gconst18)*(gconst22)*(gconst24)*(gconst7)))+(((gconst18)*(gconst20)*(gconst25)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst2)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst24)*(gconst30)*(gconst6)))+(((gconst11)*(gconst16)*(gconst22)*(gconst23)))+(((gconst13)*(gconst16)*(gconst19)*(gconst23)))+(((gconst12)*(gconst25)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst20)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst25)*(gconst31)*(gconst6)))+(((gconst17)*(gconst20)*(gconst29)*(gconst5)))+(((gconst15)*(gconst20)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst16)*(gconst19)*(gconst20)))+(((gconst13)*(gconst26)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst19)*(gconst25)*(gconst26)))+(((gconst18)*(gconst2)*(gconst24)*(gconst27)))+(((gconst10)*(gconst15)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst27)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst28)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst19)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst22)*(gconst24)))+(((gconst11)*(gconst13)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst23)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst18)*(gconst28)))+(((gconst1)*(gconst15)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst19)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst22)*(gconst23)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst18)*(gconst25)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst18)*(gconst22)))+(((gconst1)*(gconst16)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst21)*(gconst25)*(gconst7)))+(((gconst12)*(gconst15)*(gconst18)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst23)*(gconst28)*(gconst9)))+(((gconst13)*(gconst23)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst23)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst22)*(gconst29)*(gconst3)))+(((gconst13)*(gconst27)*(gconst29)*(gconst4)))+(((gconst16)*(gconst22)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst17)*(gconst29)))+(((gconst19)*(gconst22)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst16)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst26)*(gconst30)*(gconst4)))+(((gconst0)*(gconst17)*(gconst25)*(gconst29)))+(((gconst16)*(gconst2)*(gconst23)*(gconst31)))+(((gconst12)*(gconst28)*(gconst29)*(gconst5)))+(((gconst15)*(gconst17)*(gconst18)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst16)*(gconst25)*(gconst31)))+(((gconst13)*(gconst14)*(gconst16)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst24)*(gconst26)))+(((gconst13)*(gconst14)*(gconst19)*(gconst26)))+(((gconst18)*(gconst20)*(gconst28)*(gconst5)))+(((gconst11)*(gconst14)*(gconst22)*(gconst26)))+(((gconst12)*(gconst17)*(gconst18)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst28)*(gconst29)*(gconst3)))+(((gconst10)*(gconst16)*(gconst21)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst9)))+(((gconst12)*(gconst15)*(gconst17)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst24)*(gconst30)))+(((gconst14)*(gconst22)*(gconst31)*(gconst6)))+(((gconst19)*(gconst22)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst29)*(gconst8)))+(((gconst0)*(gconst18)*(gconst25)*(gconst28)))+(((gconst14)*(gconst16)*(gconst19)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst26)*(gconst31))));
op[4]=((((gconst0)*(gconst25)*(gconst28)*(gconst9)))+(((gconst11)*(gconst13)*(gconst16)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst16)*(gconst24)))+(((gconst11)*(gconst22)*(gconst26)*(gconst3)))+(((gconst1)*(gconst10)*(gconst25)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst18)*(gconst27)))+(((gconst16)*(gconst19)*(gconst2)*(gconst23)))+(((gconst2)*(gconst27)*(gconst29)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst2)*(gconst28)))+(((gconst11)*(gconst22)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst29)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst19)*(gconst20)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst30)*(gconst4)))+(((gconst1)*(gconst25)*(gconst30)*(gconst6)))+(((gconst0)*(gconst28)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst18)*(gconst28)*(gconst3)))+(((gconst20)*(gconst28)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst19)*(gconst26)*(gconst5)))+(((gconst21)*(gconst30)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst25)*(gconst26)))+(((gconst14)*(gconst19)*(gconst22)*(gconst6)))+(((gconst13)*(gconst19)*(gconst26)*(gconst3)))+(((gconst0)*(gconst15)*(gconst18)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst20)*(gconst25)*(gconst6)))+(((gconst0)*(gconst25)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst22)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst22)*(gconst9)))+(((gconst17)*(gconst18)*(gconst20)*(gconst5)))+(((gconst1)*(gconst26)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst18)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst19)*(gconst20)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst19)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst24)*(gconst30)*(gconst6)))+(((gconst15)*(gconst18)*(gconst20)*(gconst8)))+(((gconst13)*(gconst18)*(gconst27)*(gconst4)))+(((gconst22)*(gconst24)*(gconst7)*(gconst9)))+(((gconst22)*(gconst27)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst25)*(gconst31)*(gconst6)))+(((gconst1)*(gconst15)*(gconst16)*(gconst30)))+(((gconst2)*(gconst23)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst29)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst27)*(gconst29)*(gconst5)))+(((gconst14)*(gconst16)*(gconst2)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst31)*(gconst5)*(gconst6)))+(((gconst22)*(gconst29)*(gconst4)*(gconst7)))+(((gconst20)*(gconst25)*(gconst8)*(gconst9)))+(((gconst15)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst25)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst19)*(gconst26)))+(((gconst2)*(gconst24)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst8)))+(((gconst14)*(gconst19)*(gconst2)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst16)*(gconst19)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst23)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst26)*(gconst31)*(gconst5)))+(((gconst13)*(gconst16)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst15)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst22)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst22)*(gconst26)*(gconst4)))+(((gconst0)*(gconst17)*(gconst18)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst18)*(gconst23)*(gconst8)))+(((gconst11)*(gconst14)*(gconst16)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst28)*(gconst3)*(gconst9)))+(((gconst16)*(gconst19)*(gconst22)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst2)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst16)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst16)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst16)*(gconst25)))+(((gconst20)*(gconst29)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst23)*(gconst9)))+(((gconst2)*(gconst26)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst29)*(gconst3)*(gconst8)))+(((gconst11)*(gconst2)*(gconst23)*(gconst26)))+(((gconst10)*(gconst21)*(gconst25)*(gconst6)))+(((gconst10)*(gconst15)*(gconst16)*(gconst21)))+(((gconst10)*(gconst21)*(gconst26)*(gconst5)))+(((gconst13)*(gconst19)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst23)*(gconst29)*(gconst8)))+(((gconst2)*(gconst24)*(gconst27)*(gconst9)))+(((gconst13)*(gconst14)*(gconst31)*(gconst6)))+(((gconst12)*(gconst18)*(gconst25)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst16)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst27)*(gconst5)*(gconst9)))+(((gconst0)*(gconst15)*(gconst17)*(gconst29)))+(((gconst11)*(gconst13)*(gconst14)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst24)*(gconst26)))+(((gconst12)*(gconst15)*(gconst17)*(gconst18)))+(((gconst12)*(gconst18)*(gconst28)*(gconst5)))+(((gconst12)*(gconst17)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst23)*(gconst28)*(gconst9)))+(((gconst12)*(gconst15)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst30)*(gconst4)*(gconst6)))+(((gconst12)*(gconst15)*(gconst28)*(gconst9)))+(((gconst12)*(gconst17)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst20)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst29)*(gconst3)))+(((gconst22)*(gconst3)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst25)*(gconst29)*(gconst7)))+(((gconst13)*(gconst18)*(gconst24)*(gconst7)))+(((gconst13)*(gconst14)*(gconst16)*(gconst19))));
op[5]=((((IkReal(-1.00000000000000))*(gconst13)*(gconst30)*(gconst4)*(gconst6)))+(((gconst12)*(gconst25)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst18)*(gconst3)))+(((gconst13)*(gconst29)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst2)*(gconst23)*(gconst9)))+(((gconst1)*(gconst15)*(gconst30)*(gconst6)))+(((gconst11)*(gconst14)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst24)))+(((gconst11)*(gconst13)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst17)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst18)*(gconst25)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst26)*(gconst4)))+(((gconst18)*(gconst2)*(gconst24)*(gconst7)))+(((gconst14)*(gconst2)*(gconst31)*(gconst6)))+(((gconst19)*(gconst22)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst31)*(gconst5)*(gconst6)))+(((gconst12)*(gconst17)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst28)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst19)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst19)*(gconst25)*(gconst6)))+(((gconst16)*(gconst2)*(gconst3)*(gconst31)))+(((gconst1)*(gconst16)*(gconst30)*(gconst5)))+(((gconst13)*(gconst14)*(gconst19)*(gconst6)))+(((gconst10)*(gconst15)*(gconst21)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst19)*(gconst26)*(gconst5)))+(((gconst11)*(gconst16)*(gconst2)*(gconst23)))+(((gconst11)*(gconst14)*(gconst2)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst18)*(gconst8)))+(((gconst0)*(gconst18)*(gconst25)*(gconst8)))+(((gconst10)*(gconst16)*(gconst21)*(gconst5)))+(((gconst0)*(gconst17)*(gconst29)*(gconst5)))+(((gconst1)*(gconst10)*(gconst16)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst22)*(gconst3)*(gconst9)))+(((gconst19)*(gconst2)*(gconst23)*(gconst6)))+(((gconst17)*(gconst20)*(gconst5)*(gconst9)))+(((gconst0)*(gconst18)*(gconst28)*(gconst5)))+(((gconst18)*(gconst2)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst20)*(gconst6)))+(((gconst13)*(gconst24)*(gconst7)*(gconst9)))+(((gconst13)*(gconst16)*(gconst19)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst23)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst23)*(gconst8)))+(((gconst12)*(gconst15)*(gconst18)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst16)*(gconst25)))+(((gconst12)*(gconst29)*(gconst5)*(gconst8)))+(((gconst18)*(gconst20)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst21)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst2)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst20)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst15)*(gconst16)))+(((gconst13)*(gconst3)*(gconst31)*(gconst6)))+(((gconst11)*(gconst13)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst18)*(gconst27)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst2)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst16)*(gconst19)))+(((gconst12)*(gconst15)*(gconst17)*(gconst9)))+(((gconst15)*(gconst20)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst3)*(gconst8)))+(((gconst12)*(gconst28)*(gconst5)*(gconst9)))+(((gconst11)*(gconst16)*(gconst22)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst2)*(gconst29)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst15)*(gconst26)))+(((gconst13)*(gconst27)*(gconst4)*(gconst9)))+(((gconst19)*(gconst2)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst20)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst8)*(gconst9)))+(((gconst11)*(gconst13)*(gconst14)*(gconst16)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst7)*(gconst9)))+(((gconst18)*(gconst22)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst4)))+(((gconst0)*(gconst15)*(gconst28)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst2)))+(((gconst0)*(gconst15)*(gconst17)*(gconst18)))+(((gconst0)*(gconst15)*(gconst29)*(gconst8)))+(((gconst1)*(gconst10)*(gconst15)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst16)*(gconst31)*(gconst5)))+(((gconst0)*(gconst17)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst19)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst31)*(gconst6))));
op[6]=((((IkReal(-1.00000000000000))*(gconst12)*(x60)*(x61)))+(((IkReal(-1.00000000000000))*(gconst5)*(x40)*(x81)))+(((IkReal(-1.00000000000000))*(gconst22)*(x35)*(x36)))+(((IkReal(-1.00000000000000))*(x105)*(x78)))+(((IkReal(-1.00000000000000))*(x104)*(x46)))+(((IkReal(-1.00000000000000))*(gconst26)*(x39)*(x40)))+(((x39)*(x63)))+(((IkReal(-1.00000000000000))*(gconst1)*(x33)*(x49)))+(((IkReal(-1.00000000000000))*(gconst1)*(x43)*(x60)))+(((x42)*(x72)))+(((x50)*(x80)))+(((IkReal(-1.00000000000000))*(gconst13)*(x35)*(x97)))+(((IkReal(-1.00000000000000))*(x36)*(x52)))+(((IkReal(-1.00000000000000))*(gconst13)*(x35)*(x54)))+(((x54)*(x95)))+(((x43)*(x78)))+(((x37)*(x39)))+(((x100)*(x79)))+(((x54)*(x84)))+(((x47)*(x48)))+(((x44)*(x91)))+(((x78)*(x99)))+(((IkReal(-1.00000000000000))*(gconst0)*(x33)*(x44)))+(((x39)*(x85)))+(((IkReal(-1.00000000000000))*(x65)*(x66)))+(((x44)*(x52)))+(((x44)*(x57)))+(((x65)*(x86)))+(((x45)*(x61)))+(((x72)*(x92)))+(((IkReal(-1.00000000000000))*(gconst18)*(x67)*(x76)))+(((x36)*(x94)))+(((x36)*(x93)))+(((IkReal(-1.00000000000000))*(x39)*(x42)))+(((IkReal(-1.00000000000000))*(gconst16)*(x50)*(x75)))+(((IkReal(-1.00000000000000))*(gconst0)*(x61)*(x67)))+(((IkReal(-1.00000000000000))*(gconst20)*(x40)*(x82)))+(((x45)*(x48)))+(((x36)*(x71)))+(((IkReal(-1.00000000000000))*(gconst5)*(x49)*(x55)))+(((IkReal(-1.00000000000000))*(gconst16)*(x40)*(x95)))+(((IkReal(-1.00000000000000))*(gconst26)*(x75)*(x78)))+(((IkReal(-1.00000000000000))*(x39)*(x74)))+(((x49)*(x66)))+(((IkReal(-1.00000000000000))*(gconst13)*(x36)*(x70)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x85)))+(((x95)*(x97)))+(((IkReal(-1.00000000000000))*(gconst12)*(x40)*(x77)))+(((IkReal(-1.00000000000000))*(x32)*(x65)))+(((x72)*(x74)))+(((IkReal(-1.00000000000000))*(gconst29)*(x60)*(x76)))+(((x47)*(x61)))+(((x32)*(x49)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x37)))+(((gconst1)*(gconst30)*(x82)))+(((IkReal(-1.00000000000000))*(gconst29)*(x35)*(x46)))+(((x84)*(x97)))+(((IkReal(-1.00000000000000))*(x47)*(x54)))+(((x87)*(x88)))+(((x65)*(x68))));
op[7]=((((x39)*(x54)))+(((IkReal(-1.00000000000000))*(gconst13)*(x35)*(x36)))+(((IkReal(-1.00000000000000))*(gconst1)*(x49)*(x67)))+(((x100)*(x87)))+(((x78)*(x80)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x54)))+(((IkReal(-1.00000000000000))*(x39)*(x61)))+(((IkReal(-1.00000000000000))*(gconst18)*(x35)*(x46)))+(((x48)*(x72)))+(((x39)*(x97)))+(((IkReal(-1.00000000000000))*(gconst12)*(x40)*(x82)))+(((x36)*(x95)))+(((IkReal(-1.00000000000000))*(x50)*(x65)))+(((x36)*(x84)))+(((IkReal(-1.00000000000000))*(x36)*(x47)))+(((IkReal(-1.00000000000000))*(gconst16)*(x39)*(x40)))+(((x61)*(x72)))+(((x49)*(x50)))+(((IkReal(-1.00000000000000))*(gconst18)*(x60)*(x76)))+(((x44)*(x47)))+(((x44)*(x45)))+(((IkReal(-1.00000000000000))*(gconst16)*(x75)*(x78)))+(((IkReal(-1.00000000000000))*(gconst0)*(x40)*(x77)))+(((x65)*(x79))));
op[8]=((((IkReal(-1.00000000000000))*(gconst1)*(x49)*(x60)))+(((x36)*(x39)))+(((x49)*(x78)))+(((x65)*(x87)))+(((IkReal(-1.00000000000000))*(gconst6)*(x39)*(x40)))+(((IkReal(-1.00000000000000))*(x65)*(x78)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x36)))+(((x44)*(x72))));
polyroots8(op,zeror,numroots);
IkReal j15array[8], cj15array[8], sj15array[8], tempj15array[1];
int numsolutions = 0;
for(int ij15 = 0; ij15 < numroots; ++ij15)
{
IkReal htj15 = zeror[ij15];
tempj15array[0]=((IkReal(2.00000000000000))*(atan(htj15)));
for(int kj15 = 0; kj15 < 1; ++kj15)
{
j15array[numsolutions] = tempj15array[kj15];
if( j15array[numsolutions] > IKPI )
{
    j15array[numsolutions]-=IK2PI;
}
else if( j15array[numsolutions] < -IKPI )
{
    j15array[numsolutions]+=IK2PI;
}
sj15array[numsolutions] = IKsin(j15array[numsolutions]);
cj15array[numsolutions] = IKcos(j15array[numsolutions]);
numsolutions++;
}
}
bool j15valid[8]={true,true,true,true,true,true,true,true};
_nj15 = 8;
for(int ij15 = 0; ij15 < numsolutions; ++ij15)
    {
if( !j15valid[ij15] )
{
    continue;
}
    j15 = j15array[ij15]; cj15 = cj15array[ij15]; sj15 = sj15array[ij15];
htj15 = IKtan(j15/2);

_ij15[0] = ij15; _ij15[1] = -1;
for(int iij15 = ij15+1; iij15 < numsolutions; ++iij15)
{
if( j15valid[iij15] && IKabs(cj15array[ij15]-cj15array[iij15]) < IKFAST_SOLUTION_THRESH && IKabs(sj15array[ij15]-sj15array[iij15]) < IKFAST_SOLUTION_THRESH )
{
    j15valid[iij15]=false; _ij15[1] = iij15; break; 
}
}
{
IkReal dummyeval[1];
IkReal x108=((IkReal(1.00000000000000))*(r02));
dummyeval[0]=((((cj15)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x108)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x108)))+(((IkReal(0.100000000000000))*(r02)))+(((pz)*(r01)*(sj15))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j16array[1], cj16array[1], sj16array[1];
bool j16valid[1]={false};
_nj16 = 1;
IkReal x109=((r01)*(sj15));
IkReal x110=((IkReal(1.60000000000000))*(pz));
IkReal x111=((py)*(sj15));
IkReal x112=((IkReal(0.800000000000000))*(r02));
IkReal x113=((py)*(r01));
IkReal x114=((IkReal(4.00000000000000))*(px));
IkReal x115=((cj15)*(pz));
IkReal x116=((cj15)*(px));
IkReal x117=((pz)*(r02));
IkReal x118=((IkReal(2.00000000000000))*(pp));
IkReal x119=((cj15)*(r00));
IkReal x120=((IkReal(2.00000000000000))*(pz));
IkReal x121=((px)*(r00));
IkReal x122=((r02)*(x116));
if( IKabs(((((IKabs(((((x110)*(x119)))+(((IkReal(-1.60000000000000))*(x122)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x110)))+(((IkReal(-1.60000000000000))*(r02)*(x111))))) != 0)?((IkReal)1/(((((x110)*(x119)))+(((IkReal(-1.60000000000000))*(x122)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x110)))+(((IkReal(-1.60000000000000))*(r02)*(x111)))))):(IkReal)1.0e30))*(((IkReal(0.0550608694346110))+(((IkReal(-1.00000000000000))*(x118)*(x119)))+(((IkReal(-0.259678000000000))*(x109)))+(((IkReal(-0.259678000000000))*(x119)))+(((r00)*(x114)*(x116)))+(((IkReal(-0.550608694346110))*(x111)))+(((IkReal(-0.550608694346110))*(x116)))+(((IkReal(-0.400000000000000))*(x113)))+(((IkReal(-0.400000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x109)*(x118)))+(((IkReal(4.00000000000000))*(x111)*(x117)))+(((r02)*(x114)*(x115)))+(((IkReal(4.00000000000000))*(x109)*((py)*(py))))+(((IkReal(-0.400000000000000))*(x121)))+(((r00)*(x111)*(x114)))+(((cj15)*(x113)*(x114))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x112)*(x116)))+(((IkReal(0.800000000000000))*(pz)*(x109)))+(((IkReal(0.800000000000000))*(r00)*(x115)))+(((IkReal(0.0800000000000000))*(r02))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x112)*(x116)))+(((IkReal(0.800000000000000))*(pz)*(x109)))+(((IkReal(0.800000000000000))*(r00)*(x115)))+(((IkReal(0.0800000000000000))*(r02)))))):(IkReal)1.0e30))*(((((x117)*(x120)))+(((IkReal(-0.200000000000000))*(pz)*(x109)))+(((IkReal(-0.200000000000000))*(r00)*(x115)))+(((IkReal(0.200000000000000))*(r02)*(x111)))+(((x113)*(x120)))+(((IkReal(-1.00000000000000))*(pp)*(r02)))+(((x120)*(x121)))+(((IkReal(-0.149839000000000))*(r02)))+(((IkReal(0.200000000000000))*(x122)))+(((IkReal(-0.275304347173055))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((x110)*(x119)))+(((IkReal(-1.60000000000000))*(x122)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x110)))+(((IkReal(-1.60000000000000))*(r02)*(x111))))) != 0)?((IkReal)1/(((((x110)*(x119)))+(((IkReal(-1.60000000000000))*(x122)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x110)))+(((IkReal(-1.60000000000000))*(r02)*(x111)))))):(IkReal)1.0e30))*(((IkReal(0.0550608694346110))+(((IkReal(-1.00000000000000))*(x118)*(x119)))+(((IkReal(-0.259678000000000))*(x109)))+(((IkReal(-0.259678000000000))*(x119)))+(((r00)*(x114)*(x116)))+(((IkReal(-0.550608694346110))*(x111)))+(((IkReal(-0.550608694346110))*(x116)))+(((IkReal(-0.400000000000000))*(x113)))+(((IkReal(-0.400000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x109)*(x118)))+(((IkReal(4.00000000000000))*(x111)*(x117)))+(((r02)*(x114)*(x115)))+(((IkReal(4.00000000000000))*(x109)*((py)*(py))))+(((IkReal(-0.400000000000000))*(x121)))+(((r00)*(x111)*(x114)))+(((cj15)*(x113)*(x114)))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x112)*(x116)))+(((IkReal(0.800000000000000))*(pz)*(x109)))+(((IkReal(0.800000000000000))*(r00)*(x115)))+(((IkReal(0.0800000000000000))*(r02))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x112)*(x116)))+(((IkReal(0.800000000000000))*(pz)*(x109)))+(((IkReal(0.800000000000000))*(r00)*(x115)))+(((IkReal(0.0800000000000000))*(r02)))))):(IkReal)1.0e30))*(((((x117)*(x120)))+(((IkReal(-0.200000000000000))*(pz)*(x109)))+(((IkReal(-0.200000000000000))*(r00)*(x115)))+(((IkReal(0.200000000000000))*(r02)*(x111)))+(((x113)*(x120)))+(((IkReal(-1.00000000000000))*(pp)*(r02)))+(((x120)*(x121)))+(((IkReal(-0.149839000000000))*(r02)))+(((IkReal(0.200000000000000))*(x122)))+(((IkReal(-0.275304347173055))*(pz)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j16array[0]=IKatan2(((((IKabs(((((x110)*(x119)))+(((IkReal(-1.60000000000000))*(x122)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x110)))+(((IkReal(-1.60000000000000))*(r02)*(x111))))) != 0)?((IkReal)1/(((((x110)*(x119)))+(((IkReal(-1.60000000000000))*(x122)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x110)))+(((IkReal(-1.60000000000000))*(r02)*(x111)))))):(IkReal)1.0e30))*(((IkReal(0.0550608694346110))+(((IkReal(-1.00000000000000))*(x118)*(x119)))+(((IkReal(-0.259678000000000))*(x109)))+(((IkReal(-0.259678000000000))*(x119)))+(((r00)*(x114)*(x116)))+(((IkReal(-0.550608694346110))*(x111)))+(((IkReal(-0.550608694346110))*(x116)))+(((IkReal(-0.400000000000000))*(x113)))+(((IkReal(-0.400000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x109)*(x118)))+(((IkReal(4.00000000000000))*(x111)*(x117)))+(((r02)*(x114)*(x115)))+(((IkReal(4.00000000000000))*(x109)*((py)*(py))))+(((IkReal(-0.400000000000000))*(x121)))+(((r00)*(x111)*(x114)))+(((cj15)*(x113)*(x114)))))), ((((IKabs(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x112)*(x116)))+(((IkReal(0.800000000000000))*(pz)*(x109)))+(((IkReal(0.800000000000000))*(r00)*(x115)))+(((IkReal(0.0800000000000000))*(r02))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x111)*(x112)))+(((IkReal(-1.00000000000000))*(x112)*(x116)))+(((IkReal(0.800000000000000))*(pz)*(x109)))+(((IkReal(0.800000000000000))*(r00)*(x115)))+(((IkReal(0.0800000000000000))*(r02)))))):(IkReal)1.0e30))*(((((x117)*(x120)))+(((IkReal(-0.200000000000000))*(pz)*(x109)))+(((IkReal(-0.200000000000000))*(r00)*(x115)))+(((IkReal(0.200000000000000))*(r02)*(x111)))+(((x113)*(x120)))+(((IkReal(-1.00000000000000))*(pp)*(r02)))+(((x120)*(x121)))+(((IkReal(-0.149839000000000))*(r02)))+(((IkReal(0.200000000000000))*(x122)))+(((IkReal(-0.275304347173055))*(pz)))))));
sj16array[0]=IKsin(j16array[0]);
cj16array[0]=IKcos(j16array[0]);
if( j16array[0] > IKPI )
{
    j16array[0]-=IK2PI;
}
else if( j16array[0] < -IKPI )
{    j16array[0]+=IK2PI;
}
j16valid[0] = true;
for(int ij16 = 0; ij16 < 1; ++ij16)
{
if( !j16valid[ij16] )
{
    continue;
}
_ij16[0] = ij16; _ij16[1] = -1;
for(int iij16 = ij16+1; iij16 < 1; ++iij16)
{
if( j16valid[iij16] && IKabs(cj16array[ij16]-cj16array[iij16]) < IKFAST_SOLUTION_THRESH && IKabs(sj16array[ij16]-sj16array[iij16]) < IKFAST_SOLUTION_THRESH )
{
    j16valid[iij16]=false; _ij16[1] = iij16; break; 
}
}
j16 = j16array[ij16]; cj16 = cj16array[ij16]; sj16 = sj16array[ij16];
{
IkReal evalcond[2];
IkReal x123=IKcos(j16);
IkReal x124=IKsin(j16);
IkReal x125=((r01)*(sj15));
IkReal x126=((py)*(sj15));
IkReal x127=((cj15)*(px));
IkReal x128=((cj15)*(r00));
IkReal x129=((IkReal(0.400000000000000))*(x123));
IkReal x130=((IkReal(0.800000000000000))*(x123));
evalcond[0]=((IkReal(0.149839000000000))+(((IkReal(-0.200000000000000))*(x126)))+(((IkReal(-0.200000000000000))*(x127)))+(((IkReal(-1.00000000000000))*(x126)*(x130)))+(pp)+(((IkReal(0.0800000000000000))*(x123)))+(((IkReal(-1.00000000000000))*(x127)*(x130)))+(((IkReal(0.800000000000000))*(pz)*(x124))));
evalcond[1]=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-1.00000000000000))*(x128)*(x129)))+(((IkReal(-1.00000000000000))*(x125)*(x129)))+(((IkReal(-0.100000000000000))*(x125)))+(((IkReal(-0.100000000000000))*(x128)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.400000000000000))*(r02)*(x124))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j18array[2], cj18array[2], sj18array[2];
bool j18valid[2]={false};
_nj18 = 2;
IkReal x131=((cj15)*(cj16));
IkReal x132=((cj16)*(sj15));
cj18array[0]=((IkReal(-6.12888202057164))+(((IkReal(-1.26340886242921))*(r01)*(x132)))+(((IkReal(-1.26340886242921))*(r00)*(x131)))+(((IkReal(-1.53222050514291))*(cj16)))+(((IkReal(15.3222050514291))*(px)*(x131)))+(((IkReal(1.26340886242921))*(r02)*(sj16)))+(((IkReal(15.3222050514291))*(py)*(x132)))+(((IkReal(-15.3222050514291))*(pz)*(sj16))));
if( cj18array[0] >= -1-IKFAST_SINCOS_THRESH && cj18array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j18valid[0] = j18valid[1] = true;
    j18array[0] = IKacos(cj18array[0]);
    sj18array[0] = IKsin(j18array[0]);
    cj18array[1] = cj18array[0];
    j18array[1] = -j18array[0];
    sj18array[1] = -sj18array[0];
}
else if( isnan(cj18array[0]) )
{
    // probably any value will work
    j18valid[0] = true;
    cj18array[0] = 1; sj18array[0] = 0; j18array[0] = 0;
}
for(int ij18 = 0; ij18 < 2; ++ij18)
{
if( !j18valid[ij18] )
{
    continue;
}
_ij18[0] = ij18; _ij18[1] = -1;
for(int iij18 = ij18+1; iij18 < 2; ++iij18)
{
if( j18valid[iij18] && IKabs(cj18array[ij18]-cj18array[iij18]) < IKFAST_SOLUTION_THRESH && IKabs(sj18array[ij18]-sj18array[iij18]) < IKFAST_SOLUTION_THRESH )
{
    j18valid[iij18]=false; _ij18[1] = iij18; break; 
}
}
j18 = j18array[ij18]; cj18 = cj18array[ij18]; sj18 = sj18array[ij18];

{
IkReal dummyeval[1];
IkReal gconst48;
gconst48=IKsign(sj18);
dummyeval[0]=sj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x133=((cj16)*(sj15));
IkReal x134=((IkReal(1.00000000000000))*(sj16));
IkReal x135=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x134)))+(((px)*(x135)))+(((IkReal(-0.100000000000000))*(cj16)))+(((py)*(x133))));
evalcond[2]=((IkReal(-0.845725967038483))+(((r00)*(x135)))+(((r01)*(x133)))+(((IkReal(-1.00000000000000))*(r02)*(x134))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst57;
gconst57=IkReal(0.0880000000000000);
IkReal gconst58;
gconst58=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst59;
gconst59=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst60;
gconst60=IkReal(1.06723491074238);
IkReal gconst61;
gconst61=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst62;
gconst62=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst63;
gconst63=IkReal(0.0880000000000000);
IkReal gconst64;
gconst64=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst65;
gconst65=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst66;
gconst66=IkReal(1.06723491074238);
IkReal gconst67;
gconst67=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst68;
gconst68=IkReal(0.0880000000000000);
IkReal gconst69;
gconst69=IkReal(-0.0880000000000000);
IkReal gconst70;
gconst70=IkReal(1.06723491074238);
IkReal gconst71;
gconst71=IkReal(-1.06723491074238);
IkReal gconst72;
gconst72=IkReal(0.0880000000000000);
IkReal gconst73;
gconst73=IkReal(-0.0880000000000000);
IkReal gconst74;
gconst74=IkReal(1.06723491074238);
IkReal gconst75;
gconst75=IkReal(-1.06723491074238);
IkReal gconst76;
gconst76=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst77;
gconst77=IkReal(-0.0880000000000000);
IkReal gconst78;
gconst78=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst79;
gconst79=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst80;
gconst80=IkReal(-1.06723491074238);
IkReal gconst81;
gconst81=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst82;
gconst82=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst83;
gconst83=IkReal(-0.0880000000000000);
IkReal gconst84;
gconst84=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst85;
gconst85=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst86;
gconst86=IkReal(-1.06723491074238);
IkReal gconst87;
gconst87=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal x136=((gconst76)*(gconst81));
IkReal x137=((IkReal(1.00000000000000))*(gconst82));
IkReal x138=((gconst84)*(gconst85));
IkReal x139=((gconst78)*(gconst79));
IkReal x140=((gconst78)*(gconst80));
IkReal x141=((gconst83)*(gconst85));
IkReal x142=((gconst77)*(gconst81));
dummyeval[0]=((((x140)*(x141)))+(((IkReal(-1.00000000000000))*(gconst87)*(x136)*(x137)))+(((x136)*(x138)))+(((IkReal(-1.00000000000000))*(x138)*(x139)))+(((gconst82)*(gconst87)*(x139)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))+(((IkReal(-1.00000000000000))*(gconst86)*(x137)*(x140)))+(((gconst82)*(gconst86)*(x142))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x143=((gconst84)*(gconst85));
IkReal x144=((gconst68)*(gconst81));
IkReal x145=((IkReal(1.00000000000000))*(gconst58));
IkReal x146=((gconst59)*(gconst64));
IkReal x147=((gconst78)*(gconst80));
IkReal x148=((gconst83)*(gconst85));
IkReal x149=((gconst62)*(gconst67));
IkReal x150=((gconst59)*(gconst69));
IkReal x151=((gconst82)*(gconst86));
IkReal x152=((IkReal(1.00000000000000))*(gconst56));
IkReal x153=((gconst62)*(gconst71));
IkReal x154=((gconst69)*(gconst79));
IkReal x155=((gconst75)*(gconst82));
IkReal x156=((gconst60)*(gconst66));
IkReal x157=((IkReal(1.00000000000000))*(gconst61));
IkReal x158=((gconst63)*(gconst74));
IkReal x159=((gconst76)*(gconst81));
IkReal x160=((gconst60)*(gconst69));
IkReal x161=((gconst63)*(gconst65));
IkReal x162=((gconst73)*(gconst85));
IkReal x163=((gconst73)*(gconst74));
IkReal x164=((gconst58)*(gconst59));
IkReal x165=((gconst64)*(gconst85));
IkReal x166=((IkReal(1.00000000000000))*(gconst62));
IkReal x167=((gconst59)*(gconst78));
IkReal x168=((gconst82)*(gconst87));
IkReal x169=((gconst65)*(gconst84));
IkReal x170=((gconst67)*(gconst72));
IkReal x171=((gconst57)*(gconst66));
IkReal x172=((gconst62)*(gconst81));
IkReal x173=((gconst61)*(gconst77));
IkReal x174=((IkReal(1.00000000000000))*(gconst78));
IkReal x175=((gconst56)*(gconst61));
IkReal x176=((gconst78)*(gconst79));
IkReal x177=((IkReal(1.00000000000000))*(gconst69));
IkReal x178=((gconst61)*(gconst82));
IkReal x179=((gconst77)*(gconst81));
IkReal x180=((gconst65)*(gconst73));
IkReal x181=((gconst72)*(gconst86));
IkReal x182=((gconst58)*(gconst60));
IkReal x183=((gconst69)*(gconst70));
IkReal x184=((IkReal(1.00000000000000))*(gconst71));
IkReal x185=((gconst62)*(gconst87));
IkReal x186=((gconst58)*(gconst70));
IkReal x187=((gconst65)*(gconst83));
IkReal x188=((gconst70)*(gconst78));
IkReal x189=((gconst74)*(gconst84));
IkReal x190=((gconst72)*(gconst75));
IkReal x191=((gconst74)*(gconst83));
IkReal x192=((gconst58)*(gconst79));
IkReal x193=((gconst71)*(gconst77));
IkReal x194=((gconst60)*(gconst78));
IkReal x195=((gconst58)*(gconst80));
IkReal x196=((gconst71)*(gconst76));
IkReal x197=((gconst64)*(gconst74));
IkReal x198=((gconst63)*(gconst85));
IkReal x199=((gconst62)*(gconst86));
IkReal x200=((gconst57)*(gconst81));
IkReal x201=((gconst62)*(gconst75));
IkReal x202=((gconst67)*(gconst82));
IkReal x203=((gconst61)*(gconst68));
IkReal x204=((gconst64)*(gconst65));
IkReal x205=((gconst56)*(gconst71));
IkReal x206=((gconst61)*(gconst76));
IkReal x207=((gconst72)*(gconst87));
IkReal x208=((gconst69)*(gconst80));
IkReal x209=((gconst66)*(gconst82));
IkReal x210=((gconst62)*(gconst66));
IkReal x211=((gconst68)*(gconst71));
IkReal x212=((IkReal(1.00000000000000))*(x207));
IkReal x213=((IkReal(1.00000000000000))*(x189));
IkReal x214=((gconst56)*(x204));
IkReal x215=((IkReal(1.00000000000000))*(gconst81)*(x168));
op[0]=((((x168)*(x176)))+(((x151)*(x179)))+(((x147)*(x148)))+(((IkReal(-1.00000000000000))*(gconst79)*(x143)*(x174)))+(((IkReal(-1.00000000000000))*(x159)*(x168)))+(((IkReal(-1.00000000000000))*(x148)*(x179)))+(((IkReal(-1.00000000000000))*(x147)*(x151)))+(((x143)*(x159))));
op[1]=((((IkReal(-1.00000000000000))*(gconst76)*(x168)*(x184)))+(((x151)*(x193)))+(((x168)*(x188)))+(((IkReal(-1.00000000000000))*(x143)*(x154)))+(((IkReal(-1.00000000000000))*(x155)*(x159)))+(((IkReal(-1.00000000000000))*(gconst70)*(x143)*(x174)))+(((x179)*(x181)))+(((IkReal(-1.00000000000000))*(gconst79)*(x162)*(x174)))+(((IkReal(-1.00000000000000))*(x144)*(x168)))+(((IkReal(-1.00000000000000))*(gconst80)*(x151)*(x177)))+(((IkReal(-1.00000000000000))*(x147)*(x181)))+(((IkReal(-1.00000000000000))*(x159)*(x212)))+(((x154)*(x168)))+(((x155)*(x176)))+(((x147)*(x191)))+(((x148)*(x208)))+(((IkReal(-1.00000000000000))*(gconst77)*(x148)*(x184)))+(((IkReal(-1.00000000000000))*(gconst79)*(x174)*(x189)))+(((x176)*(x207)))+(((x159)*(x162)))+(((x143)*(x196)))+(((IkReal(-1.00000000000000))*(x179)*(x191)))+(((x143)*(x144)))+(((x159)*(x189))));
op[2]=((((x176)*(x190)))+(((x144)*(x189)))+(((IkReal(-1.00000000000000))*(gconst60)*(x151)*(x174)))+(((IkReal(-1.00000000000000))*(gconst76)*(x157)*(x168)))+(((x176)*(x185)))+(((x168)*(x192)))+(((x162)*(x196)))+(((IkReal(-1.00000000000000))*(gconst80)*(x145)*(x151)))+(((IkReal(-1.00000000000000))*(gconst87)*(x159)*(x166)))+(((x143)*(x206)))+(((x144)*(x162)))+(((IkReal(-1.00000000000000))*(x159)*(x202)))+(((x168)*(x183)))+(((IkReal(-1.00000000000000))*(x144)*(x155)))+(((IkReal(-1.00000000000000))*(gconst70)*(x174)*(x189)))+(((x151)*(x200)))+(((IkReal(-1.00000000000000))*(gconst70)*(x143)*(x177)))+(((IkReal(-1.00000000000000))*(x143)*(x167)))+(((IkReal(-1.00000000000000))*(gconst79)*(x169)*(x174)))+(((x151)*(x173)))+(((IkReal(-1.00000000000000))*(gconst79)*(x165)*(x174)))+(((IkReal(-1.00000000000000))*(gconst81)*(x152)*(x168)))+(((IkReal(-1.00000000000000))*(x159)*(x190)))+(((x189)*(x196)))+(((IkReal(-1.00000000000000))*(gconst77)*(x184)*(x191)))+(((IkReal(-1.00000000000000))*(x154)*(x213)))+(((IkReal(-1.00000000000000))*(gconst77)*(x148)*(x157)))+(((x188)*(x207)))+(((x191)*(x208)))+(((IkReal(-1.00000000000000))*(gconst79)*(x143)*(x145)))+(((IkReal(-1.00000000000000))*(gconst79)*(x163)*(x174)))+(((x154)*(x155)))+(((x167)*(x168)))+(((x179)*(x209)))+(((x155)*(x188)))+(((gconst56)*(gconst81)*(x143)))+(((x147)*(x187)))+(((IkReal(-1.00000000000000))*(x154)*(x162)))+(((IkReal(-1.00000000000000))*(gconst70)*(x162)*(x174)))+(((IkReal(-1.00000000000000))*(gconst68)*(x168)*(x184)))+(((IkReal(-1.00000000000000))*(x148)*(x200)))+(((IkReal(-1.00000000000000))*(gconst86)*(x147)*(x166)))+(((x154)*(x207)))+(((x147)*(x198)))+(((x148)*(x194)))+(((x148)*(x195)))+(((IkReal(-1.00000000000000))*(x144)*(x212)))+(((IkReal(-1.00000000000000))*(x147)*(x209)))+(((x143)*(x211)))+(((IkReal(-1.00000000000000))*(gconst76)*(x155)*(x184)))+(((IkReal(-1.00000000000000))*(gconst80)*(x177)*(x181)))+(((x176)*(x202)))+(((IkReal(-1.00000000000000))*(gconst76)*(x184)*(x207)))+(((x159)*(x163)))+(((x159)*(x165)))+(((x159)*(x169)))+(((IkReal(-1.00000000000000))*(x179)*(x187)))+(((x181)*(x193)))+(((gconst77)*(gconst86)*(x172)))+(((IkReal(-1.00000000000000))*(x179)*(x198))));
op[3]=((((gconst69)*(gconst72)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst71)*(gconst83)*(gconst85)))+(((gconst65)*(gconst71)*(gconst76)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst84)*(gconst85)))+(((gconst59)*(gconst72)*(gconst78)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst69)*(gconst82)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst64)*(gconst74)*(gconst76)*(gconst81)))+(((gconst58)*(gconst74)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst85)))+(((gconst57)*(gconst71)*(gconst82)*(gconst86)))+(((gconst64)*(gconst68)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst71)*(gconst72)*(gconst87)))+(((gconst71)*(gconst73)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst69)*(gconst80)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst70)*(gconst78)*(gconst84)))+(((gconst62)*(gconst69)*(gconst79)*(gconst87)))+(((gconst60)*(gconst74)*(gconst78)*(gconst83)))+(((gconst69)*(gconst70)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst75)*(gconst76)*(gconst82)))+(((gconst56)*(gconst71)*(gconst84)*(gconst85)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst67)*(gconst70)*(gconst78)*(gconst82)))+(((gconst63)*(gconst69)*(gconst80)*(gconst85)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst72)*(gconst78)*(gconst86)))+(((gconst58)*(gconst72)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst75)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst73)*(gconst74)*(gconst78)))+(((gconst62)*(gconst70)*(gconst78)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst74)*(gconst78)*(gconst84)))+(((gconst69)*(gconst70)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst73)*(gconst78)*(gconst79)))+(((gconst56)*(gconst73)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst74)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst82)*(gconst87)))+(((gconst61)*(gconst73)*(gconst76)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst74)*(gconst81)*(gconst83)))+(((gconst68)*(gconst71)*(gconst74)*(gconst84)))+(((gconst63)*(gconst74)*(gconst78)*(gconst80)))+(((gconst59)*(gconst69)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst74)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst75)*(gconst81)))+(((gconst67)*(gconst72)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst71)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst73)*(gconst78)*(gconst85)))+(((gconst65)*(gconst68)*(gconst81)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst75)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst74)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst64)*(gconst71)*(gconst76)*(gconst85)))+(((gconst59)*(gconst75)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst68)*(gconst71)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst73)*(gconst74)*(gconst79)))+(((gconst70)*(gconst72)*(gconst75)*(gconst78)))+(((gconst67)*(gconst69)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst78)*(gconst80)))+(((gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst69)*(gconst80)*(gconst86)))+(((gconst65)*(gconst73)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((gconst56)*(gconst74)*(gconst81)*(gconst84)))+(((gconst65)*(gconst69)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst73)*(gconst85)))+(((gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst61)*(gconst74)*(gconst76)*(gconst84)))+(((gconst58)*(gconst75)*(gconst79)*(gconst82)))+(((gconst61)*(gconst68)*(gconst84)*(gconst85)))+(((gconst68)*(gconst73)*(gconst74)*(gconst81)))+(((gconst58)*(gconst70)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst72)*(gconst75)*(gconst76)))+(((gconst62)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((gconst57)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst73)*(gconst79)*(gconst85)))+(((gconst60)*(gconst69)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst72)*(gconst80)*(gconst86))));
op[4]=((((gconst67)*(gconst69)*(gconst70)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst71)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst70)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst59)*(gconst84)*(gconst85)))+(((gconst64)*(gconst68)*(gconst71)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst63)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst82)))+(((gconst62)*(gconst69)*(gconst70)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst72)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst72)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst78)))+(((gconst58)*(gconst62)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst71)*(gconst75)*(gconst76)))+(((gconst60)*(gconst63)*(gconst78)*(gconst85)))+(((gconst57)*(gconst71)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst65)*(gconst77)*(gconst83)))+(((gconst61)*(gconst68)*(gconst73)*(gconst85)))+(((gconst56)*(gconst71)*(gconst73)*(gconst85)))+(((gconst57)*(gconst66)*(gconst81)*(gconst82)))+(((gconst58)*(gconst63)*(gconst80)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst70)*(gconst73)*(gconst78)))+(((gconst61)*(gconst66)*(gconst77)*(gconst82)))+(((gconst58)*(gconst60)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst66)*(gconst80)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst75)*(gconst82)))+(((gconst58)*(gconst65)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst75)*(gconst82)))+(((gconst62)*(gconst69)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst76)*(gconst87)))+(((gconst65)*(gconst71)*(gconst73)*(gconst76)))+(((gconst64)*(gconst68)*(gconst74)*(gconst81)))+(((gconst64)*(gconst71)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst71)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst62)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst73)*(gconst79)))+(((gconst61)*(gconst73)*(gconst74)*(gconst76)))+(((gconst64)*(gconst65)*(gconst76)*(gconst81)))+(((gconst58)*(gconst72)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst75)*(gconst76)))+(((gconst59)*(gconst69)*(gconst75)*(gconst82)))+(((gconst58)*(gconst70)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst73)*(gconst74)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst60)*(gconst82)*(gconst86)))+(((gconst58)*(gconst67)*(gconst79)*(gconst82)))+(((gconst57)*(gconst62)*(gconst81)*(gconst86)))+(((gconst67)*(gconst69)*(gconst72)*(gconst79)))+(((gconst59)*(gconst72)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst67)*(gconst76)*(gconst81)))+(((gconst68)*(gconst71)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst71)*(gconst87)))+(((gconst56)*(gconst64)*(gconst81)*(gconst85)))+(((gconst62)*(gconst66)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst74)*(gconst84)))+(((gconst56)*(gconst73)*(gconst74)*(gconst81)))+(((gconst58)*(gconst70)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst69)*(gconst72)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst78)*(gconst85)))+(((gconst58)*(gconst59)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst75)*(gconst81)))+(((gconst61)*(gconst68)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst67)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst61)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst65)*(gconst78)*(gconst79)))+(((gconst65)*(gconst68)*(gconst71)*(gconst84)))+(((gconst57)*(gconst61)*(gconst82)*(gconst86)))+(((gconst56)*(gconst71)*(gconst74)*(gconst84)))+(((gconst59)*(gconst62)*(gconst78)*(gconst87)))+(((gconst61)*(gconst62)*(gconst77)*(gconst86)))+(((gconst62)*(gconst70)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst81)))+(((gconst59)*(gconst67)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst65)*(gconst81)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst69)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst73)*(gconst74)*(gconst79)))+(((gconst56)*(gconst61)*(gconst84)*(gconst85)))+(((gconst61)*(gconst65)*(gconst76)*(gconst84)))+(((gconst69)*(gconst70)*(gconst72)*(gconst75)))+(((gconst66)*(gconst71)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst78)*(gconst80)))+(((gconst63)*(gconst69)*(gconst74)*(gconst80)))+(((gconst62)*(gconst67)*(gconst78)*(gconst79)))+(((gconst63)*(gconst65)*(gconst78)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst71)*(gconst74)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst73)*(gconst74)))+(((gconst65)*(gconst68)*(gconst73)*(gconst81)))+(((gconst60)*(gconst69)*(gconst74)*(gconst83)))+(((gconst67)*(gconst70)*(gconst72)*(gconst78)))+(((gconst61)*(gconst64)*(gconst76)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst72)*(gconst87)))+(((gconst59)*(gconst69)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst63)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst78)*(gconst84)))+(((gconst56)*(gconst65)*(gconst81)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst74)*(gconst77)))+(((gconst60)*(gconst65)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst62)*(gconst80)*(gconst86))));
op[5]=((((gconst58)*(gconst62)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst68)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst67)*(gconst71)*(gconst82)))+(((gconst61)*(gconst64)*(gconst68)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst69)*(gconst80)))+(((gconst61)*(gconst65)*(gconst73)*(gconst76)))+(((gconst58)*(gconst67)*(gconst72)*(gconst79)))+(((gconst56)*(gconst61)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst63)*(gconst71)*(gconst85)))+(((gconst56)*(gconst61)*(gconst73)*(gconst85)))+(((gconst60)*(gconst65)*(gconst69)*(gconst83)))+(((gconst58)*(gconst67)*(gconst70)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst60)*(gconst72)*(gconst86)))+(((gconst60)*(gconst63)*(gconst74)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst70)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst74)*(gconst79)))+(((gconst59)*(gconst62)*(gconst75)*(gconst78)))+(((gconst58)*(gconst63)*(gconst74)*(gconst80)))+(((gconst61)*(gconst68)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst73)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst63)*(gconst74)*(gconst77)))+(((gconst58)*(gconst59)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst68)*(gconst87)))+(((gconst63)*(gconst65)*(gconst69)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst74)*(gconst78)))+(((gconst56)*(gconst71)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst66)*(gconst72)*(gconst80)))+(((gconst59)*(gconst67)*(gconst72)*(gconst78)))+(((gconst62)*(gconst67)*(gconst69)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst63)*(gconst74)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst65)*(gconst70)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst67)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst71)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst62)*(gconst71)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst73)*(gconst79)))+(((gconst57)*(gconst61)*(gconst72)*(gconst86)))+(((gconst61)*(gconst64)*(gconst74)*(gconst76)))+(((gconst56)*(gconst65)*(gconst73)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst65)*(gconst71)*(gconst83)))+(((gconst56)*(gconst64)*(gconst71)*(gconst85)))+(((gconst61)*(gconst65)*(gconst68)*(gconst84)))+(((gconst62)*(gconst67)*(gconst70)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst69)*(gconst84)))+(((gconst57)*(gconst66)*(gconst72)*(gconst81)))+(((gconst58)*(gconst59)*(gconst72)*(gconst87)))+(((gconst62)*(gconst66)*(gconst71)*(gconst77)))+(((gconst59)*(gconst69)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst74)))+(((gconst57)*(gconst66)*(gconst71)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst69)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst72)*(gconst75)))+(((gconst56)*(gconst65)*(gconst71)*(gconst84)))+(((gconst58)*(gconst60)*(gconst74)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst61)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst67)*(gconst68)*(gconst81)))+(((gconst57)*(gconst62)*(gconst71)*(gconst86)))+(((gconst60)*(gconst63)*(gconst69)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst69)*(gconst85)))+(((gconst56)*(gconst64)*(gconst74)*(gconst81)))+(((gconst67)*(gconst69)*(gconst70)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst59)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst65)*(gconst69)*(gconst79)))+(((gconst64)*(gconst65)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst74)*(gconst83)))+(((gconst58)*(gconst62)*(gconst70)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst61)*(gconst72)*(gconst87)))+(((gconst64)*(gconst65)*(gconst68)*(gconst81)))+(((gconst59)*(gconst62)*(gconst69)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst70)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst71)*(gconst75)))+(((gconst65)*(gconst68)*(gconst71)*(gconst73)))+(((gconst64)*(gconst68)*(gconst71)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst70)*(gconst85)))+(((gconst62)*(gconst69)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst71)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst69)*(gconst82)))+(((gconst61)*(gconst66)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst59)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst62)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst67)*(gconst72)*(gconst81)))+(((gconst58)*(gconst70)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst72)*(gconst78)))+(((gconst59)*(gconst67)*(gconst69)*(gconst82))));
op[6]=((((x204)*(x211)))+(((IkReal(-1.00000000000000))*(gconst57)*(x157)*(x187)))+(((IkReal(-1.00000000000000))*(gconst78)*(x156)*(x166)))+(((IkReal(-1.00000000000000))*(gconst68)*(x157)*(x201)))+(((x180)*(x203)))+(((x180)*(x205)))+(((IkReal(-1.00000000000000))*(gconst57)*(x157)*(x198)))+(((IkReal(-1.00000000000000))*(gconst81)*(x149)*(x152)))+(((x161)*(x195)))+(((x161)*(x194)))+(((x149)*(x167)))+(((IkReal(-1.00000000000000))*(gconst57)*(x158)*(x184)))+(((IkReal(-1.00000000000000))*(gconst74)*(x146)*(x177)))+(((x169)*(x175)))+(((IkReal(-1.00000000000000))*(gconst71)*(x152)*(x170)))+(((x149)*(x192)))+(((IkReal(-1.00000000000000))*(gconst72)*(x156)*(x177)))+(((x150)*(x201)))+(((gconst81)*(x214)))+(((IkReal(-1.00000000000000))*(gconst82)*(x145)*(x156)))+(((x149)*(x183)))+(((x164)*(x185)))+(((IkReal(-1.00000000000000))*(gconst75)*(x152)*(x153)))+(((IkReal(-1.00000000000000))*(gconst77)*(x157)*(x161)))+(((x150)*(x170)))+(((x164)*(x190)))+(((IkReal(-1.00000000000000))*(gconst70)*(x145)*(x197)))+(((IkReal(-1.00000000000000))*(gconst68)*(x149)*(x184)))+(((x197)*(x203)))+(((x197)*(x205)))+(((x204)*(x206)))+(((x173)*(x210)))+(((IkReal(-1.00000000000000))*(gconst70)*(x177)*(x204)))+(((IkReal(-1.00000000000000))*(gconst70)*(x145)*(x180)))+(((x165)*(x175)))+(((x171)*(x178)))+(((x171)*(x172)))+(((IkReal(-1.00000000000000))*(gconst68)*(x157)*(x170)))+(((gconst71)*(gconst72)*(x171)))+(((IkReal(-1.00000000000000))*(gconst61)*(x152)*(x185)))+(((IkReal(-1.00000000000000))*(gconst65)*(x146)*(x174)))+(((IkReal(-1.00000000000000))*(x161)*(x200)))+(((IkReal(-1.00000000000000))*(gconst80)*(x145)*(x210)))+(((IkReal(-1.00000000000000))*(gconst61)*(x152)*(x190)))+(((IkReal(-1.00000000000000))*(gconst76)*(x149)*(x157)))+(((x164)*(x202)))+(((IkReal(-1.00000000000000))*(gconst85)*(x145)*(x146)))+(((x170)*(x186)))+(((x158)*(x160)))+(((x182)*(x198)))+(((IkReal(-1.00000000000000))*(gconst79)*(x145)*(x204)))+(((IkReal(-1.00000000000000))*(x150)*(x180)))+(((x163)*(x175)))+(((IkReal(-1.00000000000000))*(gconst60)*(x145)*(x199)))+(((x186)*(x201)))+(((x182)*(x187)))+(((gconst57)*(gconst61)*(x199)))+(((IkReal(-1.00000000000000))*(gconst59)*(x145)*(x163)))+(((IkReal(-1.00000000000000))*(gconst59)*(x145)*(x169)))+(((IkReal(-1.00000000000000))*(gconst67)*(x152)*(x178))));
op[7]=((((x149)*(x150)))+(((IkReal(-1.00000000000000))*(gconst74)*(x145)*(x146)))+(((IkReal(-1.00000000000000))*(gconst61)*(x152)*(x170)))+(((IkReal(-1.00000000000000))*(gconst61)*(x152)*(x201)))+(((IkReal(-1.00000000000000))*(gconst57)*(x157)*(x158)))+(((x175)*(x197)))+(((x149)*(x186)))+(((x175)*(x180)))+(((IkReal(-1.00000000000000))*(gconst70)*(x145)*(x204)))+(((x204)*(x205)))+(((x164)*(x170)))+(((IkReal(-1.00000000000000))*(gconst57)*(x161)*(x184)))+(((IkReal(-1.00000000000000))*(gconst69)*(x156)*(x166)))+(((IkReal(-1.00000000000000))*(gconst65)*(x146)*(x177)))+(((IkReal(-1.00000000000000))*(gconst68)*(x149)*(x157)))+(((IkReal(-1.00000000000000))*(gconst59)*(x145)*(x180)))+(((IkReal(-1.00000000000000))*(gconst71)*(x149)*(x152)))+(((x203)*(x204)))+(((IkReal(-1.00000000000000))*(gconst72)*(x145)*(x156)))+(((gconst61)*(gconst72)*(x171)))+(((x164)*(x201)))+(((x160)*(x161)))+(((x153)*(x171)))+(((x158)*(x182))));
op[8]=((((x175)*(x204)))+(((x161)*(x182)))+(((x149)*(x164)))+(((IkReal(-1.00000000000000))*(gconst62)*(x145)*(x156)))+(((IkReal(-1.00000000000000))*(gconst57)*(x157)*(x161)))+(((IkReal(-1.00000000000000))*(gconst65)*(x145)*(x146)))+(((IkReal(-1.00000000000000))*(gconst61)*(x149)*(x152)))+(((gconst61)*(gconst62)*(x171))));
polyroots8(op,zeror,numroots);
IkReal j17array[8], cj17array[8], sj17array[8], tempj17array[1];
int numsolutions = 0;
for(int ij17 = 0; ij17 < numroots; ++ij17)
{
IkReal htj17 = zeror[ij17];
tempj17array[0]=((IkReal(2.00000000000000))*(atan(htj17)));
for(int kj17 = 0; kj17 < 1; ++kj17)
{
j17array[numsolutions] = tempj17array[kj17];
if( j17array[numsolutions] > IKPI )
{
    j17array[numsolutions]-=IK2PI;
}
else if( j17array[numsolutions] < -IKPI )
{
    j17array[numsolutions]+=IK2PI;
}
sj17array[numsolutions] = IKsin(j17array[numsolutions]);
cj17array[numsolutions] = IKcos(j17array[numsolutions]);
numsolutions++;
}
}
bool j17valid[8]={true,true,true,true,true,true,true,true};
_nj17 = 8;
for(int ij17 = 0; ij17 < numsolutions; ++ij17)
    {
if( !j17valid[ij17] )
{
    continue;
}
    j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
htj17 = IKtan(j17/2);

_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < numsolutions; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst104;
gconst104=IKsign(((((IkReal(11.0000000000000))*((sj17)*(sj17))))+(((IkReal(11.0000000000000))*((cj17)*(cj17))))));
dummyeval[0]=(((sj17)*(sj17))+((cj17)*(cj17)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x216=((sj15)*(sj16));
IkReal x217=((IkReal(250.000000000000))*(px));
IkReal x218=((sj16)*(sj17));
IkReal x219=((cj17)*(sj16));
IkReal x220=((IkReal(250.000000000000))*(cj17)*(py));
IkReal x221=((IkReal(250.000000000000))*(py)*(sj17));
IkReal x222=((IkReal(250.000000000000))*(cj16)*(pz));
if( IKabs(((gconst104)*(((((IkReal(-1.00000000000000))*(x216)*(x221)))+(((IkReal(25.0000000000000))*(x218)))+(((IkReal(-1.00000000000000))*(cj15)*(x217)*(x218)))+(((cj17)*(sj15)*(x217)))+(((IkReal(-1.00000000000000))*(sj17)*(x222)))+(((IkReal(-1.00000000000000))*(cj15)*(x220))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst104)*(((((IkReal(-25.0000000000000))*(x219)))+(((x216)*(x220)))+(((cj17)*(x222)))+(((cj15)*(x217)*(x219)))+(((sj15)*(sj17)*(x217)))+(((IkReal(-1.00000000000000))*(cj15)*(x221))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j19array[0]=IKatan2(((gconst104)*(((((IkReal(-1.00000000000000))*(x216)*(x221)))+(((IkReal(25.0000000000000))*(x218)))+(((IkReal(-1.00000000000000))*(cj15)*(x217)*(x218)))+(((cj17)*(sj15)*(x217)))+(((IkReal(-1.00000000000000))*(sj17)*(x222)))+(((IkReal(-1.00000000000000))*(cj15)*(x220)))))), ((gconst104)*(((((IkReal(-25.0000000000000))*(x219)))+(((x216)*(x220)))+(((cj17)*(x222)))+(((cj15)*(x217)*(x219)))+(((sj15)*(sj17)*(x217)))+(((IkReal(-1.00000000000000))*(cj15)*(x221)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x223=IKsin(j19);
IkReal x224=IKcos(j19);
IkReal x225=((IkReal(1.00000000000000))*(sj16));
IkReal x226=((IkReal(1.00000000000000))*(cj16));
IkReal x227=((IkReal(1.00000000000000))*(sj15));
IkReal x228=((IkReal(0.533617455371189))*(x224));
IkReal x229=((IkReal(0.0440000000000000))*(x223));
IkReal x230=((IkReal(0.533617455371189))*(x223));
IkReal x231=((IkReal(0.0440000000000000))*(x224));
evalcond[0]=((((cj17)*(x229)))+(((sj17)*(x231)))+(((IkReal(-1.00000000000000))*(px)*(x227)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x227)))+(((cj17)*(x230)))+(((sj17)*(x228))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj15)*(x225)))+(((IkReal(-1.00000000000000))*(pz)*(x226)))+(((IkReal(-1.00000000000000))*(sj17)*(x229)))+(((cj17)*(x231)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x225)))+(((IkReal(0.100000000000000))*(sj16))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x226)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x225)))+(((IkReal(-1.00000000000000))*(sj17)*(x230)))+(((cj17)*(x228)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x225))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x232=((cj16)*(sj15));
IkReal x233=((IkReal(1.00000000000000))*(sj16));
IkReal x234=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j18, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x233)))+(((px)*(x234)))+(((IkReal(-0.100000000000000))*(cj16)))+(((py)*(x232))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x233)))+(((r00)*(x234)))+(((r01)*(x232))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst105;
gconst105=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst106;
gconst106=IkReal(0.0880000000000000);
IkReal gconst107;
gconst107=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst108;
gconst108=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst109;
gconst109=IkReal(1.06723491074238);
IkReal gconst110;
gconst110=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst111;
gconst111=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst112;
gconst112=IkReal(0.0880000000000000);
IkReal gconst113;
gconst113=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst114;
gconst114=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst115;
gconst115=IkReal(1.06723491074238);
IkReal gconst116;
gconst116=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst117;
gconst117=IkReal(-0.0880000000000000);
IkReal gconst118;
gconst118=IkReal(0.0880000000000000);
IkReal gconst119;
gconst119=IkReal(-1.06723491074238);
IkReal gconst120;
gconst120=IkReal(1.06723491074238);
IkReal gconst121;
gconst121=IkReal(-0.0880000000000000);
IkReal gconst122;
gconst122=IkReal(0.0880000000000000);
IkReal gconst123;
gconst123=IkReal(-1.06723491074238);
IkReal gconst124;
gconst124=IkReal(1.06723491074238);
IkReal gconst125;
gconst125=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst126;
gconst126=IkReal(-0.0880000000000000);
IkReal gconst127;
gconst127=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst128;
gconst128=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst129;
gconst129=IkReal(-1.06723491074238);
IkReal gconst130;
gconst130=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst131;
gconst131=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst132;
gconst132=IkReal(-0.0880000000000000);
IkReal gconst133;
gconst133=((((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj15))));
IkReal gconst134;
gconst134=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal gconst135;
gconst135=IkReal(-1.06723491074238);
IkReal gconst136;
gconst136=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj15))));
IkReal x235=((gconst125)*(gconst130));
IkReal x236=((gconst131)*(gconst136));
IkReal x237=((gconst133)*(gconst134));
IkReal x238=((gconst132)*(gconst134));
IkReal x239=((gconst127)*(gconst129));
IkReal x240=((gconst127)*(gconst128));
IkReal x241=((gconst126)*(gconst130));
IkReal x242=((gconst131)*(gconst135));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x237)*(x240)))+(((x236)*(x240)))+(((IkReal(-1.00000000000000))*(x238)*(x241)))+(((IkReal(-1.00000000000000))*(x235)*(x236)))+(((x235)*(x237)))+(((x241)*(x242)))+(((x238)*(x239)))+(((IkReal(-1.00000000000000))*(x239)*(x242))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x243=((gconst106)*(gconst110));
IkReal x244=((gconst115)*(gconst131));
IkReal x245=((gconst111)*(gconst116));
IkReal x246=((IkReal(1.00000000000000))*(gconst130));
IkReal x247=((IkReal(1.00000000000000))*(gconst127));
IkReal x248=((gconst133)*(gconst134));
IkReal x249=((gconst121)*(gconst136));
IkReal x250=((gconst118)*(gconst128));
IkReal x251=((gconst107)*(gconst119));
IkReal x252=((gconst124)*(gconst125));
IkReal x253=((gconst132)*(gconst134));
IkReal x254=((IkReal(1.00000000000000))*(gconst123));
IkReal x255=((gconst106)*(gconst120));
IkReal x256=((gconst127)*(gconst129));
IkReal x257=((gconst114)*(gconst132));
IkReal x258=((gconst108)*(gconst118));
IkReal x259=((gconst116)*(gconst121));
IkReal x260=((IkReal(1.00000000000000))*(gconst107));
IkReal x261=((gconst118)*(gconst129));
IkReal x262=((gconst114)*(gconst133));
IkReal x263=((gconst105)*(gconst110));
IkReal x264=((gconst111)*(gconst124));
IkReal x265=((gconst108)*(gconst127));
IkReal x266=((gconst108)*(gconst113));
IkReal x267=((gconst109)*(gconst118));
IkReal x268=((gconst112)*(gconst114));
IkReal x269=((gconst120)*(gconst125));
IkReal x270=((IkReal(1.00000000000000))*(gconst120));
IkReal x271=((gconst111)*(gconst115));
IkReal x272=((gconst115)*(gconst121));
IkReal x273=((gconst125)*(gconst130));
IkReal x274=((gconst114)*(gconst122));
IkReal x275=((gconst122)*(gconst134));
IkReal x276=((IkReal(1.00000000000000))*(gconst110));
IkReal x277=((gconst116)*(gconst131));
IkReal x278=((gconst129)*(gconst135));
IkReal x279=((gconst123)*(gconst133));
IkReal x280=((gconst122)*(gconst123));
IkReal x281=((gconst109)*(gconst127));
IkReal x282=((gconst131)*(gconst135));
IkReal x283=((gconst106)*(gconst130));
IkReal x284=((IkReal(1.00000000000000))*(gconst121));
IkReal x285=((gconst123)*(gconst132));
IkReal x286=((gconst107)*(gconst108));
IkReal x287=((gconst111)*(gconst136));
IkReal x288=((gconst113)*(gconst114));
IkReal x289=((gconst117)*(gconst120));
IkReal x290=((gconst113)*(gconst134));
IkReal x291=((gconst118)*(gconst119));
IkReal x292=((gconst111)*(gconst135));
IkReal x293=((gconst105)*(gconst130));
IkReal x294=((gconst105)*(gconst120));
IkReal x295=((gconst131)*(gconst136));
IkReal x296=((gconst110)*(gconst126));
IkReal x297=((gconst107)*(gconst109));
IkReal x298=((gconst112)*(gconst134));
IkReal x299=((gconst107)*(gconst129));
IkReal x300=((gconst112)*(gconst123));
IkReal x301=((gconst113)*(gconst123));
IkReal x302=((gconst120)*(gconst126));
IkReal x303=((gconst127)*(gconst128));
IkReal x304=((gconst117)*(gconst130));
IkReal x305=((gconst121)*(gconst124));
IkReal x306=((gconst124)*(gconst131));
IkReal x307=((gconst110)*(gconst117));
IkReal x308=((gconst107)*(gconst128));
IkReal x309=((gconst114)*(x266));
IkReal x310=((gconst131)*(x303));
IkReal x311=((gconst126)*(gconst130)*(gconst135));
IkReal x312=((gconst119)*(gconst127)*(gconst131));
IkReal x313=((gconst110)*(x288));
op[0]=((((gconst126)*(gconst130)*(x282)))+(((IkReal(-1.00000000000000))*(gconst126)*(x246)*(x253)))+(((x248)*(x273)))+(((IkReal(-1.00000000000000))*(gconst131)*(x247)*(x278)))+(((IkReal(-1.00000000000000))*(gconst125)*(x246)*(x295)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x248)))+(((x253)*(x256)))+(((x295)*(x303))));
op[1]=((((IkReal(-1.00000000000000))*(x261)*(x282)))+(((IkReal(-1.00000000000000))*(x269)*(x295)))+(((x273)*(x275)))+(((x273)*(x279)))+(((IkReal(-1.00000000000000))*(gconst125)*(x246)*(x249)))+(((x253)*(x261)))+(((x250)*(x295)))+(((IkReal(-1.00000000000000))*(gconst126)*(x246)*(x285)))+(((x248)*(x269)))+(((IkReal(-1.00000000000000))*(gconst131)*(x246)*(x252)))+(((x303)*(x306)))+(((IkReal(-1.00000000000000))*(gconst119)*(x247)*(x248)))+(((gconst119)*(gconst127)*(x295)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x275)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x279)))+(((x282)*(x302)))+(((IkReal(-1.00000000000000))*(x248)*(x250)))+(((x256)*(x285)))+(((x249)*(x303)))+(((gconst121)*(x311)))+(((x248)*(x304)))+(((IkReal(-1.00000000000000))*(gconst126)*(x253)*(x270)))+(((IkReal(-1.00000000000000))*(gconst121)*(x247)*(x278)))+(((IkReal(-1.00000000000000))*(gconst117)*(x246)*(x295))));
op[2]=((((IkReal(-1.00000000000000))*(gconst125)*(x246)*(x277)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x290)))+(((x275)*(x304)))+(((x253)*(x299)))+(((x248)*(x293)))+(((gconst110)*(gconst125)*(x248)))+(((x291)*(x295)))+(((x273)*(x290)))+(((IkReal(-1.00000000000000))*(gconst108)*(x247)*(x248)))+(((IkReal(-1.00000000000000))*(gconst133)*(x250)*(x254)))+(((IkReal(-1.00000000000000))*(gconst125)*(x246)*(x287)))+(((x261)*(x285)))+(((IkReal(-1.00000000000000))*(gconst109)*(x247)*(x282)))+(((IkReal(-1.00000000000000))*(gconst129)*(x244)*(x247)))+(((IkReal(-1.00000000000000))*(gconst126)*(x246)*(x257)))+(((x249)*(x250)))+(((x256)*(x298)))+(((x287)*(x303)))+(((x279)*(x304)))+(((IkReal(-1.00000000000000))*(gconst105)*(x246)*(x295)))+(((x282)*(x296)))+(((x303)*(x305)))+(((IkReal(-1.00000000000000))*(gconst125)*(x276)*(x295)))+(((x265)*(x295)))+(((IkReal(-1.00000000000000))*(gconst106)*(x246)*(x253)))+(((x256)*(x257)))+(((IkReal(-1.00000000000000))*(gconst117)*(x246)*(x249)))+(((IkReal(-1.00000000000000))*(gconst119)*(x247)*(x275)))+(((IkReal(-1.00000000000000))*(gconst119)*(x247)*(x279)))+(((IkReal(-1.00000000000000))*(gconst131)*(x260)*(x278)))+(((x262)*(x273)))+(((x253)*(x281)))+(((gconst126)*(gconst130)*(x244)))+(((IkReal(-1.00000000000000))*(x250)*(x275)))+(((x273)*(x280)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x262)))+(((x295)*(x308)))+(((IkReal(-1.00000000000000))*(gconst131)*(x252)*(x270)))+(((x248)*(x289)))+(((gconst126)*(gconst130)*(x292)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x280)))+(((IkReal(-1.00000000000000))*(gconst132)*(x254)*(x302)))+(((IkReal(-1.00000000000000))*(x248)*(x291)))+(((IkReal(-1.00000000000000))*(gconst117)*(x270)*(x295)))+(((IkReal(-1.00000000000000))*(gconst111)*(x247)*(x278)))+(((IkReal(-1.00000000000000))*(gconst128)*(x248)*(x260)))+(((IkReal(-1.00000000000000))*(gconst126)*(x246)*(x298)))+(((x277)*(x303)))+(((gconst121)*(gconst135)*(x302)))+(((x269)*(x275)))+(((x269)*(x279)))+(((gconst119)*(gconst127)*(x306)))+(((x250)*(x306)))+(((x282)*(x283)))+(((IkReal(-1.00000000000000))*(gconst126)*(x253)*(x276)))+(((gconst119)*(gconst127)*(x249)))+(((IkReal(-1.00000000000000))*(x249)*(x269)))+(((IkReal(-1.00000000000000))*(gconst135)*(x261)*(x284)))+(((IkReal(-1.00000000000000))*(gconst121)*(x246)*(x252)))+(((IkReal(-1.00000000000000))*(gconst117)*(x246)*(x306))));
op[3]=((((gconst111)*(gconst118)*(gconst128)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst122)*(gconst123)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst122)*(gconst123)*(gconst128)))+(((gconst114)*(gconst118)*(gconst129)*(gconst132)))+(((gconst119)*(gconst121)*(gconst124)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst131)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst118)*(gconst129)*(gconst135)))+(((gconst107)*(gconst123)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst120)*(gconst121)*(gconst136)))+(((gconst105)*(gconst120)*(gconst133)*(gconst134)))+(((gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst131)*(gconst136)))+(((gconst114)*(gconst122)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst123)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst122)*(gconst127)*(gconst134)))+(((gconst108)*(gconst118)*(gconst131)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst123)*(gconst126)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst121)*(gconst130)*(gconst136)))+(((gconst106)*(gconst121)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst123)*(gconst130)*(gconst132)))+(((gconst116)*(gconst121)*(gconst127)*(gconst128)))+(((gconst117)*(gconst120)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst128)*(gconst133)))+(((gconst113)*(gconst117)*(gconst130)*(gconst134)))+(((gconst106)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst124)*(gconst130)*(gconst131)))+(((gconst116)*(gconst119)*(gconst127)*(gconst131)))+(((gconst108)*(gconst124)*(gconst127)*(gconst131)))+(((gconst114)*(gconst120)*(gconst125)*(gconst133)))+(((gconst105)*(gconst122)*(gconst130)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst121)*(gconst127)*(gconst135)))+(((gconst118)*(gconst119)*(gconst121)*(gconst136)))+(((gconst113)*(gconst123)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst122)*(gconst127)*(gconst128)))+(((gconst118)*(gconst119)*(gconst124)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst134)))+(((gconst109)*(gconst118)*(gconst132)*(gconst134)))+(((gconst110)*(gconst122)*(gconst125)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst130)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst121)*(gconst129)*(gconst135)))+(((gconst109)*(gconst123)*(gconst127)*(gconst132)))+(((gconst113)*(gconst120)*(gconst125)*(gconst134)))+(((gconst118)*(gconst121)*(gconst124)*(gconst128)))+(((gconst111)*(gconst124)*(gconst127)*(gconst128)))+(((gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst127)*(gconst129)))+(((gconst110)*(gconst123)*(gconst125)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst118)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst124)*(gconst125)*(gconst130)))+(((gconst117)*(gconst120)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst118)*(gconst129)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst122)*(gconst134)))+(((gconst114)*(gconst117)*(gconst130)*(gconst133)))+(((gconst112)*(gconst123)*(gconst127)*(gconst129)))+(((gconst112)*(gconst118)*(gconst129)*(gconst134)))+(((gconst107)*(gconst119)*(gconst131)*(gconst136)))+(((gconst107)*(gconst121)*(gconst128)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst120)*(gconst124)*(gconst131)))+(((gconst107)*(gconst124)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst121)*(gconst124)*(gconst130)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst124)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst123)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((gconst108)*(gconst121)*(gconst127)*(gconst136)))+(((gconst110)*(gconst117)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((gconst120)*(gconst122)*(gconst123)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst121)*(gconst124)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst122)*(gconst128)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst133)*(gconst134)))+(((gconst105)*(gconst123)*(gconst130)*(gconst133)))+(((gconst117)*(gconst122)*(gconst123)*(gconst130)))+(((gconst111)*(gconst119)*(gconst127)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128)))+(((gconst116)*(gconst118)*(gconst128)*(gconst131))));
op[4]=((((IkReal(-1.00000000000000))*(gconst109)*(gconst111)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst112)*(gconst126)*(gconst134)))+(((gconst115)*(gconst120)*(gconst121)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst109)*(gconst131)*(gconst135)))+(((gconst110)*(gconst111)*(gconst126)*(gconst135)))+(((gconst111)*(gconst118)*(gconst124)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst112)*(gconst130)*(gconst134)))+(((gconst107)*(gconst121)*(gconst124)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst127)))+(((gconst114)*(gconst117)*(gconst122)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst121)*(gconst125)))+(((gconst111)*(gconst116)*(gconst127)*(gconst128)))+(((gconst112)*(gconst114)*(gconst127)*(gconst129)))+(((gconst110)*(gconst122)*(gconst123)*(gconst125)))+(((gconst107)*(gconst112)*(gconst129)*(gconst134)))+(((gconst110)*(gconst113)*(gconst125)*(gconst134)))+(((gconst110)*(gconst117)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst128)))+(((gconst114)*(gconst117)*(gconst120)*(gconst133)))+(((gconst108)*(gconst116)*(gconst127)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst127)*(gconst129)))+(((gconst110)*(gconst117)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst120)*(gconst121)*(gconst124)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst111)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst121)*(gconst136)))+(((gconst106)*(gconst110)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst120)*(gconst131)))+(((gconst116)*(gconst118)*(gconst119)*(gconst131)))+(((gconst105)*(gconst113)*(gconst130)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst118)*(gconst121)*(gconst129)))+(((gconst111)*(gconst115)*(gconst126)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst122)*(gconst123)*(gconst128)))+(((gconst107)*(gconst119)*(gconst124)*(gconst131)))+(((gconst113)*(gconst117)*(gconst123)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst120)*(gconst123)*(gconst132)))+(((gconst105)*(gconst120)*(gconst123)*(gconst133)))+(((gconst114)*(gconst120)*(gconst122)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst121)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst111)*(gconst125)*(gconst136)))+(((gconst118)*(gconst119)*(gconst121)*(gconst124)))+(((gconst105)*(gconst120)*(gconst122)*(gconst134)))+(((gconst107)*(gconst119)*(gconst121)*(gconst136)))+(((gconst107)*(gconst116)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst118)*(gconst121)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst121)*(gconst124)*(gconst130)))+(((gconst117)*(gconst120)*(gconst122)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst122)*(gconst123)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst114)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst120)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst119)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst115)*(gconst129)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst123)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst110)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst114)*(gconst130)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst122)*(gconst128)))+(((gconst108)*(gconst118)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst111)*(gconst130)*(gconst136)))+(((gconst111)*(gconst118)*(gconst119)*(gconst136)))+(((gconst112)*(gconst118)*(gconst123)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst115)*(gconst127)*(gconst131)))+(((gconst110)*(gconst115)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst110)*(gconst131)*(gconst136)))+(((gconst107)*(gconst111)*(gconst128)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst114)*(gconst128)*(gconst133)))+(((gconst107)*(gconst114)*(gconst129)*(gconst132)))+(((gconst105)*(gconst122)*(gconst123)*(gconst130)))+(((gconst105)*(gconst110)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst114)*(gconst127)*(gconst133)))+(((gconst109)*(gconst112)*(gconst127)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst124)*(gconst125)))+(((gconst116)*(gconst119)*(gconst121)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst116)*(gconst130)*(gconst131)))+(((gconst105)*(gconst114)*(gconst130)*(gconst133)))+(((gconst106)*(gconst111)*(gconst130)*(gconst135)))+(((gconst108)*(gconst121)*(gconst124)*(gconst127)))+(((gconst113)*(gconst117)*(gconst120)*(gconst134)))+(((gconst113)*(gconst120)*(gconst123)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst116)*(gconst125)*(gconst130)))+(((gconst106)*(gconst115)*(gconst130)*(gconst131)))+(((gconst116)*(gconst118)*(gconst121)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst119)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst124)*(gconst131)))+(((gconst107)*(gconst108)*(gconst131)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst122)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst108)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst124)*(gconst131)))+(((gconst108)*(gconst118)*(gconst124)*(gconst131)))+(((gconst109)*(gconst118)*(gconst123)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst130)))+(((gconst109)*(gconst114)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst113)*(gconst127)*(gconst134)))+(((gconst113)*(gconst114)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst113)*(gconst128)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst114)*(gconst127)*(gconst128)))+(((gconst110)*(gconst114)*(gconst125)*(gconst133)))+(((gconst111)*(gconst119)*(gconst124)*(gconst127)))+(((gconst108)*(gconst111)*(gconst127)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst124)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst120)*(gconst124)*(gconst125)))+(((gconst107)*(gconst109)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst119)*(gconst122)*(gconst127)))+(((gconst106)*(gconst120)*(gconst121)*(gconst135))));
op[5]=((((gconst107)*(gconst111)*(gconst119)*(gconst136)))+(((gconst110)*(gconst115)*(gconst121)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst116)*(gconst121)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst114)*(gconst119)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst112)*(gconst120)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst116)*(gconst117)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst115)*(gconst121)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst118)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst113)*(gconst119)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst120)*(gconst121)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst113)*(gconst123)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst110)*(gconst124)*(gconst131)))+(((gconst116)*(gconst118)*(gconst119)*(gconst121)))+(((gconst105)*(gconst110)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst111)*(gconst117)*(gconst136)))+(((gconst108)*(gconst116)*(gconst118)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst108)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst114)*(gconst122)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst111)*(gconst118)*(gconst135)))+(((gconst105)*(gconst114)*(gconst122)*(gconst130)))+(((gconst105)*(gconst120)*(gconst122)*(gconst123)))+(((gconst107)*(gconst119)*(gconst121)*(gconst124)))+(((gconst105)*(gconst110)*(gconst123)*(gconst133)))+(((gconst108)*(gconst116)*(gconst121)*(gconst127)))+(((gconst110)*(gconst117)*(gconst122)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst113)*(gconst123)*(gconst127)))+(((gconst114)*(gconst117)*(gconst120)*(gconst122)))+(((gconst106)*(gconst115)*(gconst121)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst110)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst122)*(gconst123)))+(((gconst111)*(gconst115)*(gconst120)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst121)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst121)*(gconst124)))+(((gconst106)*(gconst110)*(gconst121)*(gconst135)))+(((gconst105)*(gconst113)*(gconst120)*(gconst134)))+(((gconst113)*(gconst114)*(gconst120)*(gconst125)))+(((gconst109)*(gconst114)*(gconst118)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst117)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst108)*(gconst123)*(gconst133)))+(((gconst109)*(gconst112)*(gconst118)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst112)*(gconst123)*(gconst126)))+(((gconst105)*(gconst114)*(gconst120)*(gconst133)))+(((gconst107)*(gconst111)*(gconst124)*(gconst128)))+(((gconst110)*(gconst113)*(gconst117)*(gconst134)))+(((gconst110)*(gconst114)*(gconst122)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst112)*(gconst123)*(gconst130)))+(((gconst108)*(gconst118)*(gconst121)*(gconst124)))+(((gconst109)*(gconst112)*(gconst123)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst114)*(gconst122)*(gconst128)))+(((gconst107)*(gconst108)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst111)*(gconst124)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst114)*(gconst118)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst120)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst120)*(gconst124)))+(((gconst106)*(gconst111)*(gconst120)*(gconst135)))+(((gconst110)*(gconst114)*(gconst117)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst119)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst121)*(gconst124)))+(((gconst106)*(gconst115)*(gconst120)*(gconst131)))+(((gconst105)*(gconst113)*(gconst123)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst109)*(gconst121)*(gconst135)))+(((gconst107)*(gconst108)*(gconst124)*(gconst131)))+(((gconst111)*(gconst118)*(gconst119)*(gconst124)))+(((gconst111)*(gconst116)*(gconst119)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst113)*(gconst118)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst111)*(gconst120)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst110)*(gconst123)*(gconst132)))+(((gconst108)*(gconst111)*(gconst124)*(gconst127)))+(((gconst112)*(gconst114)*(gconst118)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst116)*(gconst120)*(gconst131)))+(((gconst107)*(gconst112)*(gconst123)*(gconst129)))+(((gconst108)*(gconst111)*(gconst118)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst115)*(gconst121)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst115)*(gconst118)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst119)*(gconst122)))+(((gconst110)*(gconst113)*(gconst123)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst114)*(gconst119)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst116)*(gconst120)*(gconst125)))+(((gconst107)*(gconst109)*(gconst123)*(gconst132)))+(((gconst107)*(gconst116)*(gconst119)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst114)*(gconst118)*(gconst133)))+(((gconst107)*(gconst116)*(gconst121)*(gconst128)))+(((gconst113)*(gconst117)*(gconst120)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst114)*(gconst120)*(gconst132)))+(((gconst113)*(gconst114)*(gconst117)*(gconst130)))+(((gconst111)*(gconst116)*(gconst118)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst111)*(gconst124)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst122)*(gconst123))));
op[6]=((((x268)*(x281)))+(((x301)*(x307)))+(((x263)*(x290)))+(((x271)*(x296)))+(((IkReal(-1.00000000000000))*(gconst113)*(x251)*(x254)))+(((IkReal(-1.00000000000000))*(x258)*(x274)))+(((IkReal(-1.00000000000000))*(gconst122)*(x254)*(x286)))+(((IkReal(-1.00000000000000))*(gconst125)*(x245)*(x276)))+(((IkReal(-1.00000000000000))*(gconst109)*(x244)*(x260)))+(((x286)*(x287)))+(((x258)*(x264)))+(((IkReal(-1.00000000000000))*(gconst105)*(x245)*(x246)))+(((x257)*(x297)))+(((IkReal(-1.00000000000000))*(x263)*(x287)))+(((IkReal(-1.00000000000000))*(x288)*(x291)))+(((IkReal(-1.00000000000000))*(gconst109)*(x260)*(x292)))+(((IkReal(-1.00000000000000))*(gconst108)*(x260)*(x262)))+(((x286)*(x305)))+(((x251)*(x259)))+(((IkReal(-1.00000000000000))*(gconst105)*(x259)*(x270)))+(((x243)*(x292)))+(((x262)*(x263)))+(((IkReal(-1.00000000000000))*(gconst126)*(x268)*(x276)))+(((x245)*(x291)))+(((x274)*(x307)))+(((IkReal(-1.00000000000000))*(gconst128)*(x260)*(x288)))+(((x271)*(x283)))+(((IkReal(-1.00000000000000))*(x247)*(x309)))+(((IkReal(-1.00000000000000))*(gconst112)*(x254)*(x255)))+(((IkReal(-1.00000000000000))*(x251)*(x274)))+(((IkReal(-1.00000000000000))*(gconst117)*(x245)*(x270)))+(((x294)*(x301)))+(((IkReal(-1.00000000000000))*(gconst113)*(x254)*(x258)))+(((x255)*(x272)))+(((x288)*(x289)))+(((IkReal(-1.00000000000000))*(x243)*(x257)))+(((x263)*(x280)))+(((IkReal(-1.00000000000000))*(x243)*(x298)))+(((x268)*(x299)))+(((IkReal(-1.00000000000000))*(x263)*(x277)))+(((gconst125)*(x313)))+(((x297)*(x298)))+(((IkReal(-1.00000000000000))*(gconst124)*(x263)*(x284)))+(((IkReal(-1.00000000000000))*(gconst117)*(x264)*(x276)))+(((IkReal(-1.00000000000000))*(gconst106)*(x246)*(x268)))+(((IkReal(-1.00000000000000))*(gconst129)*(x260)*(x271)))+(((x267)*(x300)))+(((x274)*(x294)))+(((IkReal(-1.00000000000000))*(gconst134)*(x260)*(x266)))+(((IkReal(-1.00000000000000))*(gconst117)*(x259)*(x276)))+(((IkReal(-1.00000000000000))*(gconst105)*(x264)*(x270)))+(((x277)*(x286)))+(((x245)*(x308)))+(((IkReal(-1.00000000000000))*(x267)*(x272)))+(((x243)*(x244)))+(((x245)*(x265)))+(((x251)*(x264)))+(((IkReal(-1.00000000000000))*(gconst109)*(x247)*(x271)))+(((x288)*(x293)))+(((x258)*(x259))));
op[7]=((((x267)*(x268)))+(((x245)*(x258)))+(((x245)*(x251)))+(((IkReal(-1.00000000000000))*(gconst109)*(x260)*(x272)))+(((x259)*(x286)))+(((IkReal(-1.00000000000000))*(gconst108)*(x260)*(x274)))+(((IkReal(-1.00000000000000))*(gconst105)*(x245)*(x270)))+(((IkReal(-1.00000000000000))*(gconst107)*(x254)*(x266)))+(((x263)*(x301)))+(((x264)*(x286)))+(((IkReal(-1.00000000000000))*(x251)*(x288)))+(((IkReal(-1.00000000000000))*(x259)*(x263)))+(((x263)*(x274)))+(((IkReal(-1.00000000000000))*(gconst117)*(x245)*(x276)))+(((x255)*(x271)))+(((IkReal(-1.00000000000000))*(gconst112)*(x243)*(x254)))+(((x297)*(x300)))+(((x288)*(x307)))+(((IkReal(-1.00000000000000))*(x263)*(x264)))+(((IkReal(-1.00000000000000))*(x258)*(x288)))+(((IkReal(-1.00000000000000))*(x255)*(x268)))+(((IkReal(-1.00000000000000))*(x267)*(x271)))+(((x243)*(x272)))+(((x288)*(x294))));
op[8]=((((IkReal(-1.00000000000000))*(gconst109)*(x260)*(x271)))+(((x263)*(x288)))+(((x268)*(x297)))+(((IkReal(-1.00000000000000))*(x243)*(x268)))+(((x245)*(x286)))+(((x243)*(x271)))+(((IkReal(-1.00000000000000))*(x260)*(x309)))+(((IkReal(-1.00000000000000))*(x245)*(x263))));
polyroots8(op,zeror,numroots);
IkReal j17array[8], cj17array[8], sj17array[8], tempj17array[1];
int numsolutions = 0;
for(int ij17 = 0; ij17 < numroots; ++ij17)
{
IkReal htj17 = zeror[ij17];
tempj17array[0]=((IkReal(2.00000000000000))*(atan(htj17)));
for(int kj17 = 0; kj17 < 1; ++kj17)
{
j17array[numsolutions] = tempj17array[kj17];
if( j17array[numsolutions] > IKPI )
{
    j17array[numsolutions]-=IK2PI;
}
else if( j17array[numsolutions] < -IKPI )
{
    j17array[numsolutions]+=IK2PI;
}
sj17array[numsolutions] = IKsin(j17array[numsolutions]);
cj17array[numsolutions] = IKcos(j17array[numsolutions]);
numsolutions++;
}
}
bool j17valid[8]={true,true,true,true,true,true,true,true};
_nj17 = 8;
for(int ij17 = 0; ij17 < numsolutions; ++ij17)
    {
if( !j17valid[ij17] )
{
    continue;
}
    j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
htj17 = IKtan(j17/2);

_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < numsolutions; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst153;
gconst153=IKsign(((((IkReal(11.0000000000000))*((sj17)*(sj17))))+(((IkReal(11.0000000000000))*((cj17)*(cj17))))));
dummyeval[0]=(((sj17)*(sj17))+((cj17)*(cj17)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x314=((sj15)*(sj16));
IkReal x315=((IkReal(250.000000000000))*(px));
IkReal x316=((sj16)*(sj17));
IkReal x317=((cj17)*(sj16));
IkReal x318=((IkReal(250.000000000000))*(cj17)*(py));
IkReal x319=((IkReal(250.000000000000))*(py)*(sj17));
IkReal x320=((IkReal(250.000000000000))*(cj16)*(pz));
if( IKabs(((gconst153)*(((((IkReal(-1.00000000000000))*(sj17)*(x320)))+(((IkReal(25.0000000000000))*(x316)))+(((cj17)*(sj15)*(x315)))+(((IkReal(-1.00000000000000))*(x314)*(x319)))+(((IkReal(-1.00000000000000))*(cj15)*(x318)))+(((IkReal(-1.00000000000000))*(cj15)*(x315)*(x316))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst153)*(((((cj15)*(x319)))+(((IkReal(-1.00000000000000))*(cj17)*(x320)))+(((IkReal(25.0000000000000))*(x317)))+(((IkReal(-1.00000000000000))*(x314)*(x318)))+(((IkReal(-1.00000000000000))*(sj15)*(sj17)*(x315)))+(((IkReal(-1.00000000000000))*(cj15)*(x315)*(x317))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j19array[0]=IKatan2(((gconst153)*(((((IkReal(-1.00000000000000))*(sj17)*(x320)))+(((IkReal(25.0000000000000))*(x316)))+(((cj17)*(sj15)*(x315)))+(((IkReal(-1.00000000000000))*(x314)*(x319)))+(((IkReal(-1.00000000000000))*(cj15)*(x318)))+(((IkReal(-1.00000000000000))*(cj15)*(x315)*(x316)))))), ((gconst153)*(((((cj15)*(x319)))+(((IkReal(-1.00000000000000))*(cj17)*(x320)))+(((IkReal(25.0000000000000))*(x317)))+(((IkReal(-1.00000000000000))*(x314)*(x318)))+(((IkReal(-1.00000000000000))*(sj15)*(sj17)*(x315)))+(((IkReal(-1.00000000000000))*(cj15)*(x315)*(x317)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x321=IKsin(j19);
IkReal x322=IKcos(j19);
IkReal x323=((IkReal(1.00000000000000))*(sj16));
IkReal x324=((IkReal(1.00000000000000))*(cj16));
IkReal x325=((IkReal(1.00000000000000))*(sj15));
IkReal x326=((IkReal(0.533617455371189))*(x322));
IkReal x327=((IkReal(0.0440000000000000))*(x321));
IkReal x328=((IkReal(0.533617455371189))*(x321));
IkReal x329=((IkReal(0.0440000000000000))*(x322));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj17)*(x329)))+(((IkReal(-1.00000000000000))*(px)*(x325)))+(((cj17)*(x327)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(sj17)*(x326)))+(((IkReal(-1.00000000000000))*(r00)*(x325)))+(((cj17)*(x328))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj15)*(x323)))+(((IkReal(-1.00000000000000))*(cj17)*(x329)))+(((IkReal(-1.00000000000000))*(sj17)*(x327)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x324)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x323))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj17)*(x326)))+(((IkReal(-1.00000000000000))*(sj17)*(x328)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x323)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x323)))+(((IkReal(-1.00000000000000))*(r02)*(x324))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j19array[2], cj19array[2], sj19array[2];
bool j19valid[2]={false};
_nj19 = 2;
cj19array[0]=((IkReal(1.87400166530233))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((((IkReal(-0.845725967038483))*(cj18)))+(((cj15)*(cj16)*(r00)))+(((IkReal(-1.00000000000000))*(r02)*(sj16)))+(((cj16)*(r01)*(sj15))))));
if( cj19array[0] >= -1-IKFAST_SINCOS_THRESH && cj19array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j19valid[0] = j19valid[1] = true;
    j19array[0] = IKacos(cj19array[0]);
    sj19array[0] = IKsin(j19array[0]);
    cj19array[1] = cj19array[0];
    j19array[1] = -j19array[0];
    sj19array[1] = -sj19array[0];
}
else if( isnan(cj19array[0]) )
{
    // probably any value will work
    j19valid[0] = true;
    cj19array[0] = 1; sj19array[0] = 0; j19array[0] = 0;
}
for(int ij19 = 0; ij19 < 2; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 2; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[1];
evalcond[0]=((IkReal(-0.400000000000000))+(((cj16)*(py)*(sj15)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(sj16)))+(((IkReal(-0.0440000000000000))*(sj18)*(IKcos(j19))))+(((cj15)*(cj16)*(px)))+(((IkReal(-0.135000000000000))*(cj18))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst49;
gconst49=IKsign(((((IkReal(121.000000000000))*((sj19)*(sj19))))+(((IkReal(-742.500000000000))*(cj18)*(cj19)*(sj18)))+(((IkReal(1139.06250000000))*((sj18)*(sj18))))+(((IkReal(121.000000000000))*((cj18)*(cj18))*((cj19)*(cj19))))));
dummyeval[0]=(((sj19)*(sj19))+((((cj18)*(cj18))*((cj19)*(cj19))))+(((IkReal(-6.13636363636364))*(cj18)*(cj19)*(sj18)))+(((IkReal(9.41373966942149))*((sj18)*(sj18)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[3];
dummyeval[0]=((((IkReal(-2.32653684440068))*((sj18)*(sj18))))+(((cj18)*(cj19)*(sj18))));
dummyeval[1]=sj18;
dummyeval[2]=sj19;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  || IKabs(dummyeval[2]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x330=((cj16)*(sj15));
IkReal x331=((IkReal(1.00000000000000))*(sj16));
IkReal x332=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((px)*(x332)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x331)))+(((py)*(x330))));
evalcond[2]=((IkReal(-0.845725967038483))+(((r01)*(x330)))+(((r00)*(x332)))+(((IkReal(-1.00000000000000))*(r02)*(x331))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst50;
gconst50=IKsign(((((IkReal(11.0000000000000))*((cj19)*(cj19))))+(((IkReal(11.0000000000000))*((sj19)*(sj19))))));
dummyeval[0]=(((sj19)*(sj19))+((cj19)*(cj19)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x333=((IkReal(250.000000000000))*(cj15));
IkReal x334=((cj19)*(py));
IkReal x335=((sj16)*(sj19));
IkReal x336=((cj19)*(sj16));
IkReal x337=((IkReal(250.000000000000))*(sj15));
IkReal x338=((IkReal(250.000000000000))*(cj16)*(pz));
IkReal x339=((px)*(x337));
if( IKabs(((gconst50)*(((((IkReal(-1.00000000000000))*(x333)*(x334)))+(((IkReal(-1.00000000000000))*(sj19)*(x338)))+(((cj19)*(x339)))+(((IkReal(-1.00000000000000))*(px)*(x333)*(x335)))+(((IkReal(25.0000000000000))*(x335)))+(((IkReal(-1.00000000000000))*(py)*(x335)*(x337))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst50)*(((((sj19)*(x339)))+(((IkReal(-1.00000000000000))*(py)*(sj19)*(x333)))+(((sj16)*(x334)*(x337)))+(((cj19)*(x338)))+(((px)*(x333)*(x336)))+(((IkReal(-25.0000000000000))*(x336))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst50)*(((((IkReal(-1.00000000000000))*(x333)*(x334)))+(((IkReal(-1.00000000000000))*(sj19)*(x338)))+(((cj19)*(x339)))+(((IkReal(-1.00000000000000))*(px)*(x333)*(x335)))+(((IkReal(25.0000000000000))*(x335)))+(((IkReal(-1.00000000000000))*(py)*(x335)*(x337)))))), ((gconst50)*(((((sj19)*(x339)))+(((IkReal(-1.00000000000000))*(py)*(sj19)*(x333)))+(((sj16)*(x334)*(x337)))+(((cj19)*(x338)))+(((px)*(x333)*(x336)))+(((IkReal(-25.0000000000000))*(x336)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x340=IKcos(j17);
IkReal x341=IKsin(j17);
IkReal x342=((IkReal(1.00000000000000))*(sj16));
IkReal x343=((IkReal(1.00000000000000))*(cj16));
IkReal x344=((IkReal(1.00000000000000))*(sj15));
IkReal x345=((IkReal(0.0440000000000000))*(x340));
IkReal x346=((sj19)*(x341));
IkReal x347=((cj19)*(x341));
IkReal x348=((IkReal(0.533617455371189))*(x340));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x344)))+(((sj19)*(x345)))+(((IkReal(0.0440000000000000))*(x347)))+(((cj15)*(py))));
evalcond[1]=((((sj19)*(x348)))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x344)))+(((IkReal(0.533617455371189))*(x347))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj15)*(px)*(x342)))+(((cj19)*(x345)))+(((IkReal(-0.0440000000000000))*(x346)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x342)))+(((IkReal(-1.00000000000000))*(pz)*(x343))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x343)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x342)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x342)))+(((IkReal(-0.533617455371189))*(x346)))+(((cj19)*(x348))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x349=((cj16)*(sj15));
IkReal x350=((IkReal(1.00000000000000))*(sj16));
IkReal x351=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j18, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((px)*(x351)))+(((py)*(x349)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x350))));
evalcond[2]=((IkReal(0.845725967038483))+(((r01)*(x349)))+(((r00)*(x351)))+(((IkReal(-1.00000000000000))*(r02)*(x350))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst51;
gconst51=IKsign(((((IkReal(11.0000000000000))*((cj19)*(cj19))))+(((IkReal(11.0000000000000))*((sj19)*(sj19))))));
dummyeval[0]=(((sj19)*(sj19))+((cj19)*(cj19)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x352=((IkReal(250.000000000000))*(cj15));
IkReal x353=((cj19)*(py));
IkReal x354=((sj16)*(sj19));
IkReal x355=((cj19)*(sj16));
IkReal x356=((IkReal(250.000000000000))*(sj15));
IkReal x357=((IkReal(250.000000000000))*(cj16)*(pz));
IkReal x358=((px)*(x356));
if( IKabs(((gconst51)*(((((IkReal(25.0000000000000))*(x354)))+(((IkReal(-1.00000000000000))*(cj19)*(x358)))+(((x352)*(x353)))+(((IkReal(-1.00000000000000))*(sj19)*(x357)))+(((IkReal(-1.00000000000000))*(px)*(x352)*(x354)))+(((IkReal(-1.00000000000000))*(py)*(x354)*(x356))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst51)*(((((IkReal(25.0000000000000))*(x355)))+(((sj19)*(x358)))+(((IkReal(-1.00000000000000))*(cj19)*(x357)))+(((IkReal(-1.00000000000000))*(py)*(sj19)*(x352)))+(((IkReal(-1.00000000000000))*(sj16)*(x353)*(x356)))+(((IkReal(-1.00000000000000))*(px)*(x352)*(x355))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst51)*(((((IkReal(25.0000000000000))*(x354)))+(((IkReal(-1.00000000000000))*(cj19)*(x358)))+(((x352)*(x353)))+(((IkReal(-1.00000000000000))*(sj19)*(x357)))+(((IkReal(-1.00000000000000))*(px)*(x352)*(x354)))+(((IkReal(-1.00000000000000))*(py)*(x354)*(x356)))))), ((gconst51)*(((((IkReal(25.0000000000000))*(x355)))+(((sj19)*(x358)))+(((IkReal(-1.00000000000000))*(cj19)*(x357)))+(((IkReal(-1.00000000000000))*(py)*(sj19)*(x352)))+(((IkReal(-1.00000000000000))*(sj16)*(x353)*(x356)))+(((IkReal(-1.00000000000000))*(px)*(x352)*(x355)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x359=IKcos(j17);
IkReal x360=IKsin(j17);
IkReal x361=((IkReal(0.0440000000000000))*(sj19));
IkReal x362=((IkReal(0.0440000000000000))*(cj19));
IkReal x363=((IkReal(1.00000000000000))*(cj16));
IkReal x364=((IkReal(1.00000000000000))*(sj15));
IkReal x365=((sj16)*(x364));
IkReal x366=((IkReal(0.533617455371189))*(x359));
IkReal x367=((IkReal(1.00000000000000))*(cj15)*(sj16));
IkReal x368=((IkReal(0.533617455371189))*(x360));
evalcond[0]=((((IkReal(-1.00000000000000))*(x360)*(x362)))+(((x359)*(x361)))+(((IkReal(-1.00000000000000))*(px)*(x364)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(cj19)*(x368)))+(((sj19)*(x366)))+(((IkReal(-1.00000000000000))*(r00)*(x364))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x360)*(x361)))+(((IkReal(-1.00000000000000))*(x359)*(x362)))+(((IkReal(-1.00000000000000))*(py)*(x365)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x363)))+(((IkReal(-1.00000000000000))*(px)*(x367))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x363)))+(((IkReal(-1.00000000000000))*(cj19)*(x366)))+(((IkReal(-1.00000000000000))*(r01)*(x365)))+(((IkReal(-1.00000000000000))*(r00)*(x367)))+(((IkReal(-1.00000000000000))*(sj19)*(x368))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x369=((cj16)*(sj15));
IkReal x370=((IkReal(1.00000000000000))*(sj16));
IkReal x371=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j19)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-0.0440000000000000))*(sj18)))+(((px)*(x371)))+(((py)*(x369)))+(((IkReal(-1.00000000000000))*(pz)*(x370)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((r01)*(x369)))+(((r00)*(x371)))+(((IkReal(-1.00000000000000))*(r02)*(x370)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-0.533617455371189))*(sj18))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst52;
gconst52=IKsign(((((IkReal(-0.135000000000000))*(sj18)))+(((IkReal(0.0440000000000000))*(cj18)))));
dummyeval[0]=((cj18)+(((IkReal(-3.06818181818182))*(sj18))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x372=((cj16)*(sj15));
IkReal x373=((IkReal(1.00000000000000))*(sj16));
IkReal x374=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(5.96811616783542))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x374)))+(((py)*(x372)))+(((IkReal(-1.00000000000000))*(pz)*(x373))));
evalcond[2]=((IkReal(0.969453625832208))+(((r00)*(x374)))+(((r01)*(x372)))+(((IkReal(-1.00000000000000))*(r02)*(x373))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x375=((IkReal(21213173356.8144))*(py));
IkReal x376=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(sj15)*(x376)))+(((cj15)*(x375))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x375)))+(((IkReal(2121317335.68144))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x376))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj15)*(x376)))+(((cj15)*(x375)))))+IKsqr(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x375)))+(((IkReal(2121317335.68144))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x376)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj15)*(x376)))+(((cj15)*(x375)))), ((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x375)))+(((IkReal(2121317335.68144))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x376)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x377=IKsin(j17);
IkReal x378=IKcos(j17);
IkReal x379=((IkReal(1.00000000000000))*(sj16));
IkReal x380=((IkReal(1.00000000000000))*(cj16));
IkReal x381=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x381)))+(((IkReal(-4.71405189209358e-11))*(x377)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-0.245274683444403))*(x377)))+(((IkReal(-1.00000000000000))*(r00)*(x381))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj15)*(x379)))+(((IkReal(-4.71405189209358e-11))*(x378)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x380)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x379))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj15)*(r00)*(x379)))+(((IkReal(-0.245274683444403))*(x378)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x379)))+(((IkReal(-1.00000000000000))*(r02)*(x380))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x382=((cj16)*(sj15));
IkReal x383=((IkReal(1.00000000000000))*(sj16));
IkReal x384=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(2.82652351424562))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((px)*(x384)))+(((IkReal(-0.100000000000000))*(cj16)))+(((py)*(x382)))+(((IkReal(-1.00000000000000))*(pz)*(x383))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x383)))+(((r00)*(x384)))+(((r01)*(x382))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x385=((IkReal(21213173356.8144))*(py));
IkReal x386=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(cj15)*(x385)))+(((sj15)*(x386))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj15)*(sj16)*(x385)))+(((cj15)*(sj16)*(x386)))+(((IkReal(-2121317335.68144))*(sj16)))+(((IkReal(21213173356.8144))*(cj16)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(x385)))+(((sj15)*(x386)))))+IKsqr(((((sj15)*(sj16)*(x385)))+(((cj15)*(sj16)*(x386)))+(((IkReal(-2121317335.68144))*(sj16)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj15)*(x385)))+(((sj15)*(x386)))), ((((sj15)*(sj16)*(x385)))+(((cj15)*(sj16)*(x386)))+(((IkReal(-2121317335.68144))*(sj16)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x387=IKsin(j17);
IkReal x388=IKcos(j17);
IkReal x389=((IkReal(1.00000000000000))*(sj16));
IkReal x390=((IkReal(1.00000000000000))*(cj16));
IkReal x391=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(4.71405189209358e-11))*(x387)))+(((IkReal(-1.00000000000000))*(px)*(x391)))+(((cj15)*(py))));
evalcond[1]=((((IkReal(0.245274683444403))*(x387)))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x391))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj15)*(px)*(x389)))+(((IkReal(-1.00000000000000))*(pz)*(x390)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x389)))+(((IkReal(4.71405189209358e-11))*(x388)))+(((IkReal(0.100000000000000))*(sj16))));
evalcond[3]=((((IkReal(0.245274683444403))*(x388)))+(((IkReal(-1.00000000000000))*(r02)*(x390)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x389)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x389))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
if( IKabs(((gconst52)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst52)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py)))))), ((gconst52)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x392=IKsin(j17);
IkReal x393=IKcos(j17);
IkReal x394=((IkReal(1.00000000000000))*(sj16));
IkReal x395=((IkReal(1.00000000000000))*(cj16));
IkReal x396=((IkReal(1.00000000000000))*(sj15));
IkReal x397=((cj18)*(x392));
IkReal x398=((sj18)*(x392));
IkReal x399=((sj18)*(x393));
IkReal x400=((cj18)*(x393));
evalcond[0]=((((IkReal(-0.135000000000000))*(x398)))+(((IkReal(0.0440000000000000))*(x397)))+(((IkReal(-1.00000000000000))*(px)*(x396)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x396)))+(((IkReal(-0.845725967038483))*(x398)))+(((IkReal(0.533617455371189))*(x397))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj15)*(x394)))+(((IkReal(-0.135000000000000))*(x399)))+(((IkReal(0.0440000000000000))*(x400)))+(((IkReal(-1.00000000000000))*(pz)*(x395)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x394)))+(((IkReal(0.100000000000000))*(sj16))));
evalcond[3]=((((IkReal(0.533617455371189))*(x400)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x394)))+(((IkReal(-1.00000000000000))*(r02)*(x395)))+(((IkReal(-0.845725967038483))*(x399)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x394))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x401=((cj16)*(sj15));
IkReal x402=((IkReal(1.00000000000000))*(sj16));
IkReal x403=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j19, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((py)*(x401)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x403)))+(((IkReal(0.0440000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(pz)*(x402)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(0.533617455371189))*(sj18)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x402)))+(((r00)*(x403)))+(((r01)*(x401))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(((((IkReal(-0.135000000000000))*(sj18)))+(((IkReal(-0.0440000000000000))*(cj18)))));
dummyeval[0]=((((IkReal(-3.06818181818182))*(sj18)))+(((IkReal(-1.00000000000000))*(cj18))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x404=((cj16)*(sj15));
IkReal x405=((IkReal(1.00000000000000))*(sj16));
IkReal x406=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(0.315069139344170))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((py)*(x404)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x406)))+(((IkReal(-1.00000000000000))*(pz)*(x405))));
evalcond[2]=((IkReal(0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x405)))+(((r00)*(x406)))+(((r01)*(x404))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x407=((IkReal(21213173356.8144))*(py));
IkReal x408=((IkReal(21213173356.8144))*(px));
if( IKabs(((((sj15)*(x408)))+(((IkReal(-1.00000000000000))*(cj15)*(x407))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj15)*(sj16)*(x408)))+(((sj15)*(sj16)*(x407)))+(((IkReal(-2121317335.68144))*(sj16)))+(((IkReal(21213173356.8144))*(cj16)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj15)*(x408)))+(((IkReal(-1.00000000000000))*(cj15)*(x407)))))+IKsqr(((((cj15)*(sj16)*(x408)))+(((sj15)*(sj16)*(x407)))+(((IkReal(-2121317335.68144))*(sj16)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((sj15)*(x408)))+(((IkReal(-1.00000000000000))*(cj15)*(x407)))), ((((cj15)*(sj16)*(x408)))+(((sj15)*(sj16)*(x407)))+(((IkReal(-2121317335.68144))*(sj16)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x409=IKsin(j17);
IkReal x410=IKcos(j17);
IkReal x411=((IkReal(1.00000000000000))*(sj16));
IkReal x412=((IkReal(1.00000000000000))*(cj16));
IkReal x413=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x413)))+(((IkReal(4.71405189209358e-11))*(x409)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(0.245274683444403))*(x409)))+(((IkReal(-1.00000000000000))*(r00)*(x413))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj15)*(px)*(x411)))+(((IkReal(4.71405189209358e-11))*(x410)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x411)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x412))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x412)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x411)))+(((IkReal(0.245274683444403))*(x410)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x411))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x414=((cj16)*(sj15));
IkReal x415=((IkReal(1.00000000000000))*(sj16));
IkReal x416=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-2.82652351424562))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((py)*(x414)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x416)))+(((IkReal(-1.00000000000000))*(pz)*(x415))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x415)))+(((r01)*(x414)))+(((r00)*(x416))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x417=((IkReal(21213173356.8144))*(py));
IkReal x418=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(sj15)*(x418)))+(((cj15)*(x417))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x418)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x417)))+(((IkReal(2121317335.68144))*(sj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj15)*(x418)))+(((cj15)*(x417)))))+IKsqr(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x418)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x417)))+(((IkReal(2121317335.68144))*(sj16)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj15)*(x418)))+(((cj15)*(x417)))), ((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x418)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x417)))+(((IkReal(2121317335.68144))*(sj16)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x419=IKsin(j17);
IkReal x420=IKcos(j17);
IkReal x421=((IkReal(1.00000000000000))*(sj16));
IkReal x422=((IkReal(1.00000000000000))*(cj16));
IkReal x423=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x423)))+(((IkReal(-4.71405189209358e-11))*(x419)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x423)))+(((IkReal(-0.245274683444403))*(x419))));
evalcond[2]=((((IkReal(-4.71405189209358e-11))*(x420)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x421)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x422)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x421))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj15)*(r00)*(x421)))+(((IkReal(-0.245274683444403))*(x420)))+(((IkReal(-1.00000000000000))*(r02)*(x422)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x421))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
if( IKabs(((gconst53)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst53)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py)))))), ((gconst53)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x424=IKsin(j17);
IkReal x425=IKcos(j17);
IkReal x426=((IkReal(1.00000000000000))*(sj16));
IkReal x427=((IkReal(1.00000000000000))*(cj16));
IkReal x428=((IkReal(1.00000000000000))*(sj15));
IkReal x429=((cj18)*(x424));
IkReal x430=((sj18)*(x424));
IkReal x431=((sj18)*(x425));
IkReal x432=((cj18)*(x425));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x428)))+(((IkReal(-0.0440000000000000))*(x429)))+(((cj15)*(py)))+(((IkReal(-0.135000000000000))*(x430))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-0.533617455371189))*(x429)))+(((IkReal(-1.00000000000000))*(r00)*(x428)))+(((IkReal(-0.845725967038483))*(x430))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(x432)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x426)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x427)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x426)))+(((IkReal(-0.135000000000000))*(x431))));
evalcond[3]=((((IkReal(-0.533617455371189))*(x432)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x426)))+(((IkReal(-1.00000000000000))*(r02)*(x427)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x426)))+(((IkReal(-0.845725967038483))*(x431))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x433=((cj15)*(sj18));
IkReal x434=((sj15)*(sj18));
IkReal x435=((cj18)*(cj19));
IkReal x436=((cj15)*(r01));
IkReal x437=((px)*(sj15));
IkReal x438=((r00)*(sj15));
IkReal x439=((cj15)*(py));
if( IKabs(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj18)*(x435)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj18)*(x435)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18))))))):(IkReal)1.0e30))*(((((IkReal(-3.64368616112160e+23))*(py)*(x433)))+(((IkReal(1.29137887418136e+22))*(x435)*(x438)))+(((IkReal(3.00444053086361e+22))*(r01)*(x433)))+(((IkReal(-3.00444053086361e+22))*(r00)*(x434)))+(((IkReal(3.64368616112160e+23))*(px)*(x434)))+(((IkReal(-1.29137887418136e+22))*(x435)*(x436)))+(((IkReal(1.56614161081993e+23))*(x435)*(x439)))+(((IkReal(-1.56614161081993e+23))*(x435)*(x437))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.14854255730435e-11))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IKabs(sj19) != 0)?((IkReal)1/(sj19)):(IkReal)1.0e30))*(((((IkReal(110001049103.000))*(x435)*(x438)))+(((IkReal(-1334056361602.36))*(x435)*(x437)))+(((IkReal(-110001049103.000))*(x435)*(x436)))+(((IkReal(2114335082451.87))*(px)*(x434)))+(((IkReal(-2114335082451.87))*(py)*(x433)))+(((IkReal(1334056361602.36))*(x435)*(x439)))+(((IkReal(337503218838.750))*(r01)*(x433)))+(((IkReal(-337503218838.750))*(r00)*(x434))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj18)*(x435)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj18)*(x435)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18))))))):(IkReal)1.0e30))*(((((IkReal(-3.64368616112160e+23))*(py)*(x433)))+(((IkReal(1.29137887418136e+22))*(x435)*(x438)))+(((IkReal(3.00444053086361e+22))*(r01)*(x433)))+(((IkReal(-3.00444053086361e+22))*(r00)*(x434)))+(((IkReal(3.64368616112160e+23))*(px)*(x434)))+(((IkReal(-1.29137887418136e+22))*(x435)*(x436)))+(((IkReal(1.56614161081993e+23))*(x435)*(x439)))+(((IkReal(-1.56614161081993e+23))*(x435)*(x437)))))))+IKsqr(((IkReal(-1.14854255730435e-11))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IKabs(sj19) != 0)?((IkReal)1/(sj19)):(IkReal)1.0e30))*(((((IkReal(110001049103.000))*(x435)*(x438)))+(((IkReal(-1334056361602.36))*(x435)*(x437)))+(((IkReal(-110001049103.000))*(x435)*(x436)))+(((IkReal(2114335082451.87))*(px)*(x434)))+(((IkReal(-2114335082451.87))*(py)*(x433)))+(((IkReal(1334056361602.36))*(x435)*(x439)))+(((IkReal(337503218838.750))*(r01)*(x433)))+(((IkReal(-337503218838.750))*(r00)*(x434)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj18)*(x435)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj18)*(x435)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18))))))):(IkReal)1.0e30))*(((((IkReal(-3.64368616112160e+23))*(py)*(x433)))+(((IkReal(1.29137887418136e+22))*(x435)*(x438)))+(((IkReal(3.00444053086361e+22))*(r01)*(x433)))+(((IkReal(-3.00444053086361e+22))*(r00)*(x434)))+(((IkReal(3.64368616112160e+23))*(px)*(x434)))+(((IkReal(-1.29137887418136e+22))*(x435)*(x436)))+(((IkReal(1.56614161081993e+23))*(x435)*(x439)))+(((IkReal(-1.56614161081993e+23))*(x435)*(x437)))))), ((IkReal(-1.14854255730435e-11))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IKabs(sj19) != 0)?((IkReal)1/(sj19)):(IkReal)1.0e30))*(((((IkReal(110001049103.000))*(x435)*(x438)))+(((IkReal(-1334056361602.36))*(x435)*(x437)))+(((IkReal(-110001049103.000))*(x435)*(x436)))+(((IkReal(2114335082451.87))*(px)*(x434)))+(((IkReal(-2114335082451.87))*(py)*(x433)))+(((IkReal(1334056361602.36))*(x435)*(x439)))+(((IkReal(337503218838.750))*(r01)*(x433)))+(((IkReal(-337503218838.750))*(r00)*(x434)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x440=IKsin(j17);
IkReal x441=IKcos(j17);
IkReal x442=((IkReal(1.00000000000000))*(sj15));
IkReal x443=((IkReal(1.00000000000000))*(cj16));
IkReal x444=((sj18)*(x440));
IkReal x445=((sj19)*(x441));
IkReal x446=((IkReal(1.00000000000000))*(cj15)*(sj16));
IkReal x447=((sj19)*(x440));
IkReal x448=((sj18)*(x441));
IkReal x449=((cj18)*(cj19)*(x440));
IkReal x450=((cj18)*(cj19)*(x441));
evalcond[0]=((((IkReal(0.0440000000000000))*(x449)))+(((IkReal(0.0440000000000000))*(x445)))+(((IkReal(-1.00000000000000))*(px)*(x442)))+(((cj15)*(py)))+(((IkReal(-0.135000000000000))*(x444))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-0.845725967038483))*(x444)))+(((IkReal(-1.00000000000000))*(r00)*(x442)))+(((IkReal(0.533617455371189))*(x445)))+(((IkReal(0.533617455371189))*(x449))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(x447)))+(((IkReal(-1.00000000000000))*(py)*(sj16)*(x442)))+(((IkReal(-1.00000000000000))*(pz)*(x443)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(0.0440000000000000))*(x450)))+(((IkReal(-1.00000000000000))*(px)*(x446)))+(((IkReal(-0.135000000000000))*(x448))));
evalcond[3]=((((IkReal(-0.533617455371189))*(x447)))+(((IkReal(-1.00000000000000))*(r01)*(sj16)*(x442)))+(((IkReal(0.533617455371189))*(x450)))+(((IkReal(-1.00000000000000))*(r02)*(x443)))+(((IkReal(-0.845725967038483))*(x448)))+(((IkReal(-1.00000000000000))*(r00)*(x446))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x451=((IkReal(2750.00000000000))*(sj19));
IkReal x452=((px)*(sj15));
IkReal x453=((cj16)*(pz));
IkReal x454=((sj15)*(sj16));
IkReal x455=((IkReal(8437.50000000000))*(sj18));
IkReal x456=((IkReal(275.000000000000))*(sj16));
IkReal x457=((cj18)*(cj19));
IkReal x458=((cj15)*(py));
IkReal x459=((cj15)*(px)*(sj16));
IkReal x460=((IkReal(2750.00000000000))*(py)*(x457));
if( IKabs(((gconst49)*(((((x455)*(x458)))+(((sj19)*(x456)))+(((IkReal(-1.00000000000000))*(x452)*(x455)))+(((IkReal(-1.00000000000000))*(py)*(x451)*(x454)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((IkReal(-1.00000000000000))*(x451)*(x459)))+(((IkReal(-2750.00000000000))*(x457)*(x458)))+(((IkReal(2750.00000000000))*(x452)*(x457))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst49)*(((((x454)*(x460)))+(((IkReal(843.750000000000))*(sj16)*(sj18)))+(((IkReal(-1.00000000000000))*(x456)*(x457)))+(((IkReal(-1.00000000000000))*(py)*(x454)*(x455)))+(((IkReal(2750.00000000000))*(x457)*(x459)))+(((IkReal(-1.00000000000000))*(x455)*(x459)))+(((IkReal(-1.00000000000000))*(x453)*(x455)))+(((IkReal(-1.00000000000000))*(x451)*(x458)))+(((IkReal(2750.00000000000))*(x453)*(x457)))+(((x451)*(x452))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst49)*(((((x455)*(x458)))+(((sj19)*(x456)))+(((IkReal(-1.00000000000000))*(x452)*(x455)))+(((IkReal(-1.00000000000000))*(py)*(x451)*(x454)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((IkReal(-1.00000000000000))*(x451)*(x459)))+(((IkReal(-2750.00000000000))*(x457)*(x458)))+(((IkReal(2750.00000000000))*(x452)*(x457)))))), ((gconst49)*(((((x454)*(x460)))+(((IkReal(843.750000000000))*(sj16)*(sj18)))+(((IkReal(-1.00000000000000))*(x456)*(x457)))+(((IkReal(-1.00000000000000))*(py)*(x454)*(x455)))+(((IkReal(2750.00000000000))*(x457)*(x459)))+(((IkReal(-1.00000000000000))*(x455)*(x459)))+(((IkReal(-1.00000000000000))*(x453)*(x455)))+(((IkReal(-1.00000000000000))*(x451)*(x458)))+(((IkReal(2750.00000000000))*(x453)*(x457)))+(((x451)*(x452)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x461=IKsin(j17);
IkReal x462=IKcos(j17);
IkReal x463=((IkReal(1.00000000000000))*(sj15));
IkReal x464=((IkReal(1.00000000000000))*(cj16));
IkReal x465=((sj18)*(x461));
IkReal x466=((sj19)*(x462));
IkReal x467=((IkReal(1.00000000000000))*(cj15)*(sj16));
IkReal x468=((sj19)*(x461));
IkReal x469=((sj18)*(x462));
IkReal x470=((cj18)*(cj19)*(x461));
IkReal x471=((cj18)*(cj19)*(x462));
evalcond[0]=((((IkReal(0.0440000000000000))*(x470)))+(((IkReal(-0.135000000000000))*(x465)))+(((IkReal(-1.00000000000000))*(px)*(x463)))+(((cj15)*(py)))+(((IkReal(0.0440000000000000))*(x466))));
evalcond[1]=((((IkReal(-0.845725967038483))*(x465)))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x463)))+(((IkReal(0.533617455371189))*(x470)))+(((IkReal(0.533617455371189))*(x466))));
evalcond[2]=((((IkReal(0.0440000000000000))*(x471)))+(((IkReal(-0.0440000000000000))*(x468)))+(((IkReal(-1.00000000000000))*(py)*(sj16)*(x463)))+(((IkReal(-1.00000000000000))*(pz)*(x464)))+(((IkReal(-0.135000000000000))*(x469)))+(((IkReal(-1.00000000000000))*(px)*(x467)))+(((IkReal(0.100000000000000))*(sj16))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj16)*(x463)))+(((IkReal(-0.845725967038483))*(x469)))+(((IkReal(-1.00000000000000))*(r00)*(x467)))+(((IkReal(0.533617455371189))*(x471)))+(((IkReal(-0.533617455371189))*(x468)))+(((IkReal(-1.00000000000000))*(r02)*(x464))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j19array[2], cj19array[2], sj19array[2];
bool j19valid[2]={false};
_nj19 = 2;
cj19array[0]=((IkReal(22.7272727272727))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IkReal(-0.400000000000000))+(((cj16)*(py)*(sj15)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(sj16)))+(((cj15)*(cj16)*(px)))+(((IkReal(-0.135000000000000))*(cj18))))));
if( cj19array[0] >= -1-IKFAST_SINCOS_THRESH && cj19array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j19valid[0] = j19valid[1] = true;
    j19array[0] = IKacos(cj19array[0]);
    sj19array[0] = IKsin(j19array[0]);
    cj19array[1] = cj19array[0];
    j19array[1] = -j19array[0];
    sj19array[1] = -sj19array[0];
}
else if( isnan(cj19array[0]) )
{
    // probably any value will work
    j19valid[0] = true;
    cj19array[0] = 1; sj19array[0] = 0; j19array[0] = 0;
}
for(int ij19 = 0; ij19 < 2; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 2; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-0.533617455371189))*(sj18)*(IKcos(j19))))+(((IkReal(-0.845725967038483))*(cj18)))+(((cj15)*(cj16)*(r00)))+(((IkReal(-1.00000000000000))*(r02)*(sj16)))+(((cj16)*(r01)*(sj15))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst49;
gconst49=IKsign(((((IkReal(121.000000000000))*((sj19)*(sj19))))+(((IkReal(-742.500000000000))*(cj18)*(cj19)*(sj18)))+(((IkReal(1139.06250000000))*((sj18)*(sj18))))+(((IkReal(121.000000000000))*((cj18)*(cj18))*((cj19)*(cj19))))));
dummyeval[0]=(((sj19)*(sj19))+((((cj18)*(cj18))*((cj19)*(cj19))))+(((IkReal(-6.13636363636364))*(cj18)*(cj19)*(sj18)))+(((IkReal(9.41373966942149))*((sj18)*(sj18)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[3];
dummyeval[0]=((((IkReal(-2.32653684440068))*((sj18)*(sj18))))+(((cj18)*(cj19)*(sj18))));
dummyeval[1]=sj18;
dummyeval[2]=sj19;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  || IKabs(dummyeval[2]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x472=((cj16)*(sj15));
IkReal x473=((IkReal(1.00000000000000))*(sj16));
IkReal x474=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x473)))+(((py)*(x472)))+(((px)*(x474))));
evalcond[2]=((IkReal(-0.845725967038483))+(((r00)*(x474)))+(((IkReal(-1.00000000000000))*(r02)*(x473)))+(((r01)*(x472))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst50;
gconst50=IKsign(((((IkReal(11.0000000000000))*((cj19)*(cj19))))+(((IkReal(11.0000000000000))*((sj19)*(sj19))))));
dummyeval[0]=(((sj19)*(sj19))+((cj19)*(cj19)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x475=((IkReal(250.000000000000))*(cj15));
IkReal x476=((cj19)*(py));
IkReal x477=((sj16)*(sj19));
IkReal x478=((cj19)*(sj16));
IkReal x479=((IkReal(250.000000000000))*(sj15));
IkReal x480=((IkReal(250.000000000000))*(cj16)*(pz));
IkReal x481=((px)*(x479));
if( IKabs(((gconst50)*(((((IkReal(25.0000000000000))*(x477)))+(((IkReal(-1.00000000000000))*(sj19)*(x480)))+(((IkReal(-1.00000000000000))*(px)*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(py)*(x477)*(x479)))+(((cj19)*(x481))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst50)*(((((IkReal(-25.0000000000000))*(x478)))+(((sj19)*(x481)))+(((IkReal(-1.00000000000000))*(py)*(sj19)*(x475)))+(((px)*(x475)*(x478)))+(((sj16)*(x476)*(x479)))+(((cj19)*(x480))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst50)*(((((IkReal(25.0000000000000))*(x477)))+(((IkReal(-1.00000000000000))*(sj19)*(x480)))+(((IkReal(-1.00000000000000))*(px)*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(py)*(x477)*(x479)))+(((cj19)*(x481)))))), ((gconst50)*(((((IkReal(-25.0000000000000))*(x478)))+(((sj19)*(x481)))+(((IkReal(-1.00000000000000))*(py)*(sj19)*(x475)))+(((px)*(x475)*(x478)))+(((sj16)*(x476)*(x479)))+(((cj19)*(x480)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x482=IKcos(j17);
IkReal x483=IKsin(j17);
IkReal x484=((IkReal(1.00000000000000))*(sj16));
IkReal x485=((IkReal(1.00000000000000))*(cj16));
IkReal x486=((IkReal(1.00000000000000))*(sj15));
IkReal x487=((IkReal(0.0440000000000000))*(x482));
IkReal x488=((sj19)*(x483));
IkReal x489=((cj19)*(x483));
IkReal x490=((IkReal(0.533617455371189))*(x482));
evalcond[0]=((((sj19)*(x487)))+(((IkReal(-1.00000000000000))*(px)*(x486)))+(((IkReal(0.0440000000000000))*(x489)))+(((cj15)*(py))));
evalcond[1]=((((IkReal(0.533617455371189))*(x489)))+(((cj15)*(r01)))+(((sj19)*(x490)))+(((IkReal(-1.00000000000000))*(r00)*(x486))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)*(x485)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x484)))+(((cj19)*(x487)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x484)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-0.0440000000000000))*(x488))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x484)))+(((IkReal(-0.533617455371189))*(x488)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x484)))+(((cj19)*(x490)))+(((IkReal(-1.00000000000000))*(r02)*(x485))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x491=((cj16)*(sj15));
IkReal x492=((IkReal(1.00000000000000))*(sj16));
IkReal x493=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j18, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((py)*(x491)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x493)))+(((IkReal(-1.00000000000000))*(pz)*(x492))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x492)))+(((r00)*(x493)))+(((r01)*(x491))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst51;
gconst51=IKsign(((((IkReal(11.0000000000000))*((cj19)*(cj19))))+(((IkReal(11.0000000000000))*((sj19)*(sj19))))));
dummyeval[0]=(((sj19)*(sj19))+((cj19)*(cj19)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x494=((IkReal(250.000000000000))*(cj15));
IkReal x495=((cj19)*(py));
IkReal x496=((sj16)*(sj19));
IkReal x497=((cj19)*(sj16));
IkReal x498=((IkReal(250.000000000000))*(sj15));
IkReal x499=((IkReal(250.000000000000))*(cj16)*(pz));
IkReal x500=((px)*(x498));
if( IKabs(((gconst51)*(((((IkReal(-1.00000000000000))*(py)*(x496)*(x498)))+(((x494)*(x495)))+(((IkReal(-1.00000000000000))*(cj19)*(x500)))+(((IkReal(-1.00000000000000))*(px)*(x494)*(x496)))+(((IkReal(25.0000000000000))*(x496)))+(((IkReal(-1.00000000000000))*(sj19)*(x499))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst51)*(((((IkReal(-1.00000000000000))*(py)*(sj19)*(x494)))+(((IkReal(-1.00000000000000))*(sj16)*(x495)*(x498)))+(((IkReal(-1.00000000000000))*(cj19)*(x499)))+(((sj19)*(x500)))+(((IkReal(-1.00000000000000))*(px)*(x494)*(x497)))+(((IkReal(25.0000000000000))*(x497))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst51)*(((((IkReal(-1.00000000000000))*(py)*(x496)*(x498)))+(((x494)*(x495)))+(((IkReal(-1.00000000000000))*(cj19)*(x500)))+(((IkReal(-1.00000000000000))*(px)*(x494)*(x496)))+(((IkReal(25.0000000000000))*(x496)))+(((IkReal(-1.00000000000000))*(sj19)*(x499)))))), ((gconst51)*(((((IkReal(-1.00000000000000))*(py)*(sj19)*(x494)))+(((IkReal(-1.00000000000000))*(sj16)*(x495)*(x498)))+(((IkReal(-1.00000000000000))*(cj19)*(x499)))+(((sj19)*(x500)))+(((IkReal(-1.00000000000000))*(px)*(x494)*(x497)))+(((IkReal(25.0000000000000))*(x497)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x501=IKcos(j17);
IkReal x502=IKsin(j17);
IkReal x503=((IkReal(0.0440000000000000))*(sj19));
IkReal x504=((IkReal(0.0440000000000000))*(cj19));
IkReal x505=((IkReal(1.00000000000000))*(cj16));
IkReal x506=((IkReal(1.00000000000000))*(sj15));
IkReal x507=((sj16)*(x506));
IkReal x508=((IkReal(0.533617455371189))*(x501));
IkReal x509=((IkReal(1.00000000000000))*(cj15)*(sj16));
IkReal x510=((IkReal(0.533617455371189))*(x502));
evalcond[0]=((((IkReal(-1.00000000000000))*(x502)*(x504)))+(((x501)*(x503)))+(((IkReal(-1.00000000000000))*(px)*(x506)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(cj19)*(x510)))+(((IkReal(-1.00000000000000))*(r00)*(x506)))+(((sj19)*(x508))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(x507)))+(((IkReal(-1.00000000000000))*(pz)*(x505)))+(((IkReal(-1.00000000000000))*(x502)*(x503)))+(((IkReal(-1.00000000000000))*(x501)*(x504)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(px)*(x509))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj19)*(x510)))+(((IkReal(-1.00000000000000))*(cj19)*(x508)))+(((IkReal(-1.00000000000000))*(r01)*(x507)))+(((IkReal(-1.00000000000000))*(r00)*(x509)))+(((IkReal(-1.00000000000000))*(r02)*(x505))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x511=((cj16)*(sj15));
IkReal x512=((IkReal(1.00000000000000))*(sj16));
IkReal x513=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j19)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((px)*(x513)))+(((IkReal(-0.100000000000000))*(cj16)))+(((py)*(x511)))+(((IkReal(-0.0440000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(pz)*(x512)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((r01)*(x511)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x512)))+(((IkReal(-0.533617455371189))*(sj18)))+(((r00)*(x513))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst52;
gconst52=IKsign(((((IkReal(-0.135000000000000))*(sj18)))+(((IkReal(0.0440000000000000))*(cj18)))));
dummyeval[0]=((cj18)+(((IkReal(-3.06818181818182))*(sj18))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x514=((cj16)*(sj15));
IkReal x515=((IkReal(1.00000000000000))*(sj16));
IkReal x516=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(5.96811616783542))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((px)*(x516)))+(((IkReal(-0.100000000000000))*(cj16)))+(((py)*(x514)))+(((IkReal(-1.00000000000000))*(pz)*(x515))));
evalcond[2]=((IkReal(0.969453625832208))+(((r01)*(x514)))+(((IkReal(-1.00000000000000))*(r02)*(x515)))+(((r00)*(x516))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x517=((IkReal(21213173356.8144))*(py));
IkReal x518=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(sj15)*(x518)))+(((cj15)*(x517))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x518)))+(((IkReal(2121317335.68144))*(sj16)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x517))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj15)*(x518)))+(((cj15)*(x517)))))+IKsqr(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x518)))+(((IkReal(2121317335.68144))*(sj16)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x517)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj15)*(x518)))+(((cj15)*(x517)))), ((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x518)))+(((IkReal(2121317335.68144))*(sj16)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x517)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x519=IKsin(j17);
IkReal x520=IKcos(j17);
IkReal x521=((IkReal(1.00000000000000))*(sj16));
IkReal x522=((IkReal(1.00000000000000))*(cj16));
IkReal x523=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(-4.71405189209358e-11))*(x519)))+(((IkReal(-1.00000000000000))*(px)*(x523)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x523)))+(((IkReal(-0.245274683444403))*(x519))));
evalcond[2]=((((IkReal(-4.71405189209358e-11))*(x520)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x521)))+(((IkReal(-1.00000000000000))*(pz)*(x522)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x521))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x521)))+(((IkReal(-0.245274683444403))*(x520)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x521)))+(((IkReal(-1.00000000000000))*(r02)*(x522))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x524=((cj16)*(sj15));
IkReal x525=((IkReal(1.00000000000000))*(sj16));
IkReal x526=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(2.82652351424562))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x526)))+(((IkReal(-1.00000000000000))*(pz)*(x525)))+(((py)*(x524))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x525)))+(((r01)*(x524)))+(((r00)*(x526))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x527=((IkReal(21213173356.8144))*(py));
IkReal x528=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(cj15)*(x527)))+(((sj15)*(x528))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-2121317335.68144))*(sj16)))+(((cj15)*(sj16)*(x528)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))+(((sj15)*(sj16)*(x527))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(x527)))+(((sj15)*(x528)))))+IKsqr(((((IkReal(-2121317335.68144))*(sj16)))+(((cj15)*(sj16)*(x528)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))+(((sj15)*(sj16)*(x527)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj15)*(x527)))+(((sj15)*(x528)))), ((((IkReal(-2121317335.68144))*(sj16)))+(((cj15)*(sj16)*(x528)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))+(((sj15)*(sj16)*(x527)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x529=IKsin(j17);
IkReal x530=IKcos(j17);
IkReal x531=((IkReal(1.00000000000000))*(sj16));
IkReal x532=((IkReal(1.00000000000000))*(cj16));
IkReal x533=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(4.71405189209358e-11))*(x529)))+(((IkReal(-1.00000000000000))*(px)*(x533)))+(((cj15)*(py))));
evalcond[1]=((((IkReal(0.245274683444403))*(x529)))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x533))));
evalcond[2]=((((IkReal(4.71405189209358e-11))*(x530)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x531)))+(((IkReal(-1.00000000000000))*(pz)*(x532)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x531))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x531)))+(((IkReal(0.245274683444403))*(x530)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x531)))+(((IkReal(-1.00000000000000))*(r02)*(x532))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
if( IKabs(((gconst52)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst52)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py)))))), ((gconst52)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x534=IKsin(j17);
IkReal x535=IKcos(j17);
IkReal x536=((IkReal(1.00000000000000))*(sj16));
IkReal x537=((IkReal(1.00000000000000))*(cj16));
IkReal x538=((IkReal(1.00000000000000))*(sj15));
IkReal x539=((cj18)*(x534));
IkReal x540=((sj18)*(x534));
IkReal x541=((sj18)*(x535));
IkReal x542=((cj18)*(x535));
evalcond[0]=((((IkReal(-0.135000000000000))*(x540)))+(((IkReal(-1.00000000000000))*(px)*(x538)))+(((IkReal(0.0440000000000000))*(x539)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x538)))+(((IkReal(-0.845725967038483))*(x540)))+(((IkReal(0.533617455371189))*(x539))));
evalcond[2]=((((IkReal(-0.135000000000000))*(x541)))+(((IkReal(0.0440000000000000))*(x542)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x536)))+(((IkReal(-1.00000000000000))*(pz)*(x537)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x536))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x536)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x536)))+(((IkReal(-1.00000000000000))*(r02)*(x537)))+(((IkReal(-0.845725967038483))*(x541)))+(((IkReal(0.533617455371189))*(x542))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x543=((cj16)*(sj15));
IkReal x544=((IkReal(1.00000000000000))*(sj16));
IkReal x545=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j19, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((py)*(x543)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x545)))+(((IkReal(0.0440000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(pz)*(x544)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(0.533617455371189))*(sj18)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x544)))+(((r00)*(x545)))+(((r01)*(x543))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(((((IkReal(-0.135000000000000))*(sj18)))+(((IkReal(-0.0440000000000000))*(cj18)))));
dummyeval[0]=((((IkReal(-3.06818181818182))*(sj18)))+(((IkReal(-1.00000000000000))*(cj18))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x546=((cj16)*(sj15));
IkReal x547=((IkReal(1.00000000000000))*(sj16));
IkReal x548=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(0.315069139344170))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((py)*(x546)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x548)))+(((IkReal(-1.00000000000000))*(pz)*(x547))));
evalcond[2]=((IkReal(0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x547)))+(((r00)*(x548)))+(((r01)*(x546))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x549=((IkReal(21213173356.8144))*(py));
IkReal x550=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(cj15)*(x549)))+(((sj15)*(x550))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-2121317335.68144))*(sj16)))+(((cj15)*(sj16)*(x550)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))+(((sj15)*(sj16)*(x549))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(x549)))+(((sj15)*(x550)))))+IKsqr(((((IkReal(-2121317335.68144))*(sj16)))+(((cj15)*(sj16)*(x550)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))+(((sj15)*(sj16)*(x549)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj15)*(x549)))+(((sj15)*(x550)))), ((((IkReal(-2121317335.68144))*(sj16)))+(((cj15)*(sj16)*(x550)))+(((IkReal(21213173356.8144))*(cj16)*(pz)))+(((sj15)*(sj16)*(x549)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x551=IKsin(j17);
IkReal x552=IKcos(j17);
IkReal x553=((IkReal(1.00000000000000))*(sj16));
IkReal x554=((IkReal(1.00000000000000))*(cj16));
IkReal x555=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(4.71405189209358e-11))*(x551)))+(((IkReal(-1.00000000000000))*(px)*(x555)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x555)))+(((IkReal(0.245274683444403))*(x551))));
evalcond[2]=((((IkReal(4.71405189209358e-11))*(x552)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x553)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x553)))+(((IkReal(-1.00000000000000))*(pz)*(x554))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x554)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x553)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x553)))+(((IkReal(0.245274683444403))*(x552))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x556=((cj16)*(sj15));
IkReal x557=((IkReal(1.00000000000000))*(sj16));
IkReal x558=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-2.82652351424562))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((py)*(x556)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x558)))+(((IkReal(-1.00000000000000))*(pz)*(x557))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x557)))+(((r01)*(x556)))+(((r00)*(x558))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x559=((IkReal(21213173356.8144))*(py));
IkReal x560=((IkReal(21213173356.8144))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(sj15)*(x560)))+(((cj15)*(x559))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x559)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x560)))+(((IkReal(2121317335.68144))*(sj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj15)*(x560)))+(((cj15)*(x559)))))+IKsqr(((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x559)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x560)))+(((IkReal(2121317335.68144))*(sj16)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj15)*(x560)))+(((cj15)*(x559)))), ((((IkReal(-21213173356.8144))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(sj15)*(sj16)*(x559)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x560)))+(((IkReal(2121317335.68144))*(sj16)))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x561=IKsin(j17);
IkReal x562=IKcos(j17);
IkReal x563=((IkReal(1.00000000000000))*(sj16));
IkReal x564=((IkReal(1.00000000000000))*(cj16));
IkReal x565=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((((IkReal(-4.71405189209358e-11))*(x561)))+(((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x565))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-0.245274683444403))*(x561)))+(((IkReal(-1.00000000000000))*(r00)*(x565))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj15)*(px)*(x563)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x563)))+(((IkReal(-1.00000000000000))*(pz)*(x564)))+(((IkReal(-4.71405189209358e-11))*(x562)))+(((IkReal(0.100000000000000))*(sj16))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x563)))+(((IkReal(-0.245274683444403))*(x562)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x563)))+(((IkReal(-1.00000000000000))*(r02)*(x564))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
if( IKabs(((gconst53)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst53)*(((((px)*(sj15)))+(((IkReal(-1.00000000000000))*(cj15)*(py)))))), ((gconst53)*(((((IkReal(-0.100000000000000))*(sj16)))+(((cj15)*(px)*(sj16)))+(((py)*(sj15)*(sj16)))+(((cj16)*(pz)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x566=IKsin(j17);
IkReal x567=IKcos(j17);
IkReal x568=((IkReal(1.00000000000000))*(sj16));
IkReal x569=((IkReal(1.00000000000000))*(cj16));
IkReal x570=((IkReal(1.00000000000000))*(sj15));
IkReal x571=((cj18)*(x566));
IkReal x572=((sj18)*(x566));
IkReal x573=((sj18)*(x567));
IkReal x574=((cj18)*(x567));
evalcond[0]=((((IkReal(-0.135000000000000))*(x572)))+(((IkReal(-1.00000000000000))*(px)*(x570)))+(((cj15)*(py)))+(((IkReal(-0.0440000000000000))*(x571))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x570)))+(((IkReal(-0.845725967038483))*(x572)))+(((IkReal(-0.533617455371189))*(x571))));
evalcond[2]=((((IkReal(-0.135000000000000))*(x573)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x568)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x568)))+(((IkReal(-1.00000000000000))*(pz)*(x569)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-0.0440000000000000))*(x574))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x568)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x568)))+(((IkReal(-0.845725967038483))*(x573)))+(((IkReal(-0.533617455371189))*(x574)))+(((IkReal(-1.00000000000000))*(r02)*(x569))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x575=((cj15)*(sj18));
IkReal x576=((sj15)*(sj18));
IkReal x577=((cj18)*(cj19));
IkReal x578=((cj15)*(r01));
IkReal x579=((px)*(sj15));
IkReal x580=((r00)*(sj15));
IkReal x581=((cj15)*(py));
if( IKabs(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj18)*(x577)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj18)*(x577)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18))))))):(IkReal)1.0e30))*(((((IkReal(-3.64368616112160e+23))*(py)*(x575)))+(((IkReal(3.00444053086361e+22))*(r01)*(x575)))+(((IkReal(1.29137887418136e+22))*(x577)*(x580)))+(((IkReal(3.64368616112160e+23))*(px)*(x576)))+(((IkReal(-1.29137887418136e+22))*(x577)*(x578)))+(((IkReal(-3.00444053086361e+22))*(r00)*(x576)))+(((IkReal(-1.56614161081993e+23))*(x577)*(x579)))+(((IkReal(1.56614161081993e+23))*(x577)*(x581))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.14854255730435e-11))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IKabs(sj19) != 0)?((IkReal)1/(sj19)):(IkReal)1.0e30))*(((((IkReal(2114335082451.87))*(px)*(x576)))+(((IkReal(337503218838.750))*(r01)*(x575)))+(((IkReal(-337503218838.750))*(r00)*(x576)))+(((IkReal(-110001049103.000))*(x577)*(x578)))+(((IkReal(1334056361602.36))*(x577)*(x581)))+(((IkReal(-1334056361602.36))*(x577)*(x579)))+(((IkReal(-2114335082451.87))*(py)*(x575)))+(((IkReal(110001049103.000))*(x577)*(x580))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj18)*(x577)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj18)*(x577)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18))))))):(IkReal)1.0e30))*(((((IkReal(-3.64368616112160e+23))*(py)*(x575)))+(((IkReal(3.00444053086361e+22))*(r01)*(x575)))+(((IkReal(1.29137887418136e+22))*(x577)*(x580)))+(((IkReal(3.64368616112160e+23))*(px)*(x576)))+(((IkReal(-1.29137887418136e+22))*(x577)*(x578)))+(((IkReal(-3.00444053086361e+22))*(r00)*(x576)))+(((IkReal(-1.56614161081993e+23))*(x577)*(x579)))+(((IkReal(1.56614161081993e+23))*(x577)*(x581)))))))+IKsqr(((IkReal(-1.14854255730435e-11))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IKabs(sj19) != 0)?((IkReal)1/(sj19)):(IkReal)1.0e30))*(((((IkReal(2114335082451.87))*(px)*(x576)))+(((IkReal(337503218838.750))*(r01)*(x575)))+(((IkReal(-337503218838.750))*(r00)*(x576)))+(((IkReal(-110001049103.000))*(x577)*(x578)))+(((IkReal(1334056361602.36))*(x577)*(x581)))+(((IkReal(-1334056361602.36))*(x577)*(x579)))+(((IkReal(-2114335082451.87))*(py)*(x575)))+(((IkReal(110001049103.000))*(x577)*(x580)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j17array[0]=IKatan2(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj18)*(x577)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj18)*(x577)))+(((IkReal(-2.37804294413992e+22))*((sj18)*(sj18))))))):(IkReal)1.0e30))*(((((IkReal(-3.64368616112160e+23))*(py)*(x575)))+(((IkReal(3.00444053086361e+22))*(r01)*(x575)))+(((IkReal(1.29137887418136e+22))*(x577)*(x580)))+(((IkReal(3.64368616112160e+23))*(px)*(x576)))+(((IkReal(-1.29137887418136e+22))*(x577)*(x578)))+(((IkReal(-3.00444053086361e+22))*(r00)*(x576)))+(((IkReal(-1.56614161081993e+23))*(x577)*(x579)))+(((IkReal(1.56614161081993e+23))*(x577)*(x581)))))), ((IkReal(-1.14854255730435e-11))*(((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30))*(((IKabs(sj19) != 0)?((IkReal)1/(sj19)):(IkReal)1.0e30))*(((((IkReal(2114335082451.87))*(px)*(x576)))+(((IkReal(337503218838.750))*(r01)*(x575)))+(((IkReal(-337503218838.750))*(r00)*(x576)))+(((IkReal(-110001049103.000))*(x577)*(x578)))+(((IkReal(1334056361602.36))*(x577)*(x581)))+(((IkReal(-1334056361602.36))*(x577)*(x579)))+(((IkReal(-2114335082451.87))*(py)*(x575)))+(((IkReal(110001049103.000))*(x577)*(x580)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x582=IKsin(j17);
IkReal x583=IKcos(j17);
IkReal x584=((IkReal(1.00000000000000))*(sj15));
IkReal x585=((IkReal(1.00000000000000))*(cj16));
IkReal x586=((sj18)*(x582));
IkReal x587=((sj19)*(x583));
IkReal x588=((IkReal(1.00000000000000))*(cj15)*(sj16));
IkReal x589=((sj19)*(x582));
IkReal x590=((sj18)*(x583));
IkReal x591=((cj18)*(cj19)*(x582));
IkReal x592=((cj18)*(cj19)*(x583));
evalcond[0]=((((IkReal(-0.135000000000000))*(x586)))+(((IkReal(0.0440000000000000))*(x587)))+(((IkReal(-1.00000000000000))*(px)*(x584)))+(((IkReal(0.0440000000000000))*(x591)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((IkReal(0.533617455371189))*(x591)))+(((IkReal(-0.845725967038483))*(x586)))+(((IkReal(-1.00000000000000))*(r00)*(x584)))+(((IkReal(0.533617455371189))*(x587))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)*(x585)))+(((IkReal(-1.00000000000000))*(px)*(x588)))+(((IkReal(0.0440000000000000))*(x592)))+(((IkReal(-0.135000000000000))*(x590)))+(((IkReal(-1.00000000000000))*(py)*(sj16)*(x584)))+(((IkReal(-0.0440000000000000))*(x589)))+(((IkReal(0.100000000000000))*(sj16))));
evalcond[3]=((((IkReal(-0.845725967038483))*(x590)))+(((IkReal(-1.00000000000000))*(r02)*(x585)))+(((IkReal(-1.00000000000000))*(r01)*(sj16)*(x584)))+(((IkReal(-0.533617455371189))*(x589)))+(((IkReal(0.533617455371189))*(x592)))+(((IkReal(-1.00000000000000))*(r00)*(x588))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x593=((IkReal(2750.00000000000))*(sj19));
IkReal x594=((px)*(sj15));
IkReal x595=((cj16)*(pz));
IkReal x596=((sj15)*(sj16));
IkReal x597=((IkReal(8437.50000000000))*(sj18));
IkReal x598=((IkReal(275.000000000000))*(sj16));
IkReal x599=((cj18)*(cj19));
IkReal x600=((cj15)*(py));
IkReal x601=((cj15)*(px)*(sj16));
IkReal x602=((IkReal(2750.00000000000))*(py)*(x599));
if( IKabs(((gconst49)*(((((IkReal(-2750.00000000000))*(x599)*(x600)))+(((IkReal(-1.00000000000000))*(x593)*(x601)))+(((sj19)*(x598)))+(((x597)*(x600)))+(((IkReal(-1.00000000000000))*(x593)*(x595)))+(((IkReal(2750.00000000000))*(x594)*(x599)))+(((IkReal(-1.00000000000000))*(x594)*(x597)))+(((IkReal(-1.00000000000000))*(py)*(x593)*(x596))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst49)*(((((IkReal(-1.00000000000000))*(x598)*(x599)))+(((IkReal(-1.00000000000000))*(x593)*(x600)))+(((IkReal(2750.00000000000))*(x599)*(x601)))+(((IkReal(843.750000000000))*(sj16)*(sj18)))+(((IkReal(-1.00000000000000))*(py)*(x596)*(x597)))+(((IkReal(-1.00000000000000))*(x597)*(x601)))+(((IkReal(2750.00000000000))*(x595)*(x599)))+(((x596)*(x602)))+(((IkReal(-1.00000000000000))*(x595)*(x597)))+(((x593)*(x594))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((gconst49)*(((((IkReal(-2750.00000000000))*(x599)*(x600)))+(((IkReal(-1.00000000000000))*(x593)*(x601)))+(((sj19)*(x598)))+(((x597)*(x600)))+(((IkReal(-1.00000000000000))*(x593)*(x595)))+(((IkReal(2750.00000000000))*(x594)*(x599)))+(((IkReal(-1.00000000000000))*(x594)*(x597)))+(((IkReal(-1.00000000000000))*(py)*(x593)*(x596)))))), ((gconst49)*(((((IkReal(-1.00000000000000))*(x598)*(x599)))+(((IkReal(-1.00000000000000))*(x593)*(x600)))+(((IkReal(2750.00000000000))*(x599)*(x601)))+(((IkReal(843.750000000000))*(sj16)*(sj18)))+(((IkReal(-1.00000000000000))*(py)*(x596)*(x597)))+(((IkReal(-1.00000000000000))*(x597)*(x601)))+(((IkReal(2750.00000000000))*(x595)*(x599)))+(((x596)*(x602)))+(((IkReal(-1.00000000000000))*(x595)*(x597)))+(((x593)*(x594)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x603=IKsin(j17);
IkReal x604=IKcos(j17);
IkReal x605=((IkReal(1.00000000000000))*(sj15));
IkReal x606=((IkReal(1.00000000000000))*(cj16));
IkReal x607=((sj18)*(x603));
IkReal x608=((sj19)*(x604));
IkReal x609=((IkReal(1.00000000000000))*(cj15)*(sj16));
IkReal x610=((sj19)*(x603));
IkReal x611=((sj18)*(x604));
IkReal x612=((cj18)*(cj19)*(x603));
IkReal x613=((cj18)*(cj19)*(x604));
evalcond[0]=((((IkReal(-0.135000000000000))*(x607)))+(((IkReal(0.0440000000000000))*(x608)))+(((IkReal(0.0440000000000000))*(x612)))+(((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x605))));
evalcond[1]=((((IkReal(0.533617455371189))*(x612)))+(((cj15)*(r01)))+(((IkReal(-0.845725967038483))*(x607)))+(((IkReal(-1.00000000000000))*(r00)*(x605)))+(((IkReal(0.533617455371189))*(x608))));
evalcond[2]=((((IkReal(-0.135000000000000))*(x611)))+(((IkReal(-1.00000000000000))*(py)*(sj16)*(x605)))+(((IkReal(0.0440000000000000))*(x613)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x606)))+(((IkReal(-0.0440000000000000))*(x610)))+(((IkReal(-1.00000000000000))*(px)*(x609))));
evalcond[3]=((((IkReal(0.533617455371189))*(x613)))+(((IkReal(-0.845725967038483))*(x611)))+(((IkReal(-1.00000000000000))*(r00)*(x609)))+(((IkReal(-0.533617455371189))*(x610)))+(((IkReal(-1.00000000000000))*(r02)*(x606)))+(((IkReal(-1.00000000000000))*(r01)*(sj16)*(x605))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j17array[1], cj17array[1], sj17array[1];
bool j17valid[1]={false};
_nj17 = 1;
IkReal x614=((cj15)*(sj16));
IkReal x615=((IkReal(0.533617455371189))*(px));
IkReal x616=((IkReal(0.0440000000000000))*(r00));
IkReal x617=((IkReal(0.533617455371189))*(py));
IkReal x618=((sj15)*(sj16));
IkReal x619=((IkReal(0.0440000000000000))*(r01));
if( IKabs(((IkReal(50000238.4319315))*(gconst48)*(((((IkReal(-1.00000000000000))*(sj15)*(x615)))+(((IkReal(-1.00000000000000))*(cj15)*(x619)))+(((sj15)*(x616)))+(((cj15)*(x617))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-50000238.4319315))*(gconst48)*(((((IkReal(-1.00000000000000))*(x614)*(x616)))+(((IkReal(-1.00000000000000))*(x618)*(x619)))+(((IkReal(-0.0440000000000000))*(cj16)*(r02)))+(((x617)*(x618)))+(((IkReal(0.533617455371189))*(cj16)*(pz)))+(((x614)*(x615)))+(((IkReal(-0.0533617455371189))*(sj16))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j17array[0]=IKatan2(((IkReal(50000238.4319315))*(gconst48)*(((((IkReal(-1.00000000000000))*(sj15)*(x615)))+(((IkReal(-1.00000000000000))*(cj15)*(x619)))+(((sj15)*(x616)))+(((cj15)*(x617)))))), ((IkReal(-50000238.4319315))*(gconst48)*(((((IkReal(-1.00000000000000))*(x614)*(x616)))+(((IkReal(-1.00000000000000))*(x618)*(x619)))+(((IkReal(-0.0440000000000000))*(cj16)*(r02)))+(((x617)*(x618)))+(((IkReal(0.533617455371189))*(cj16)*(pz)))+(((x614)*(x615)))+(((IkReal(-0.0533617455371189))*(sj16)))))));
sj17array[0]=IKsin(j17array[0]);
cj17array[0]=IKcos(j17array[0]);
if( j17array[0] > IKPI )
{
    j17array[0]-=IK2PI;
}
else if( j17array[0] < -IKPI )
{    j17array[0]+=IK2PI;
}
j17valid[0] = true;
for(int ij17 = 0; ij17 < 1; ++ij17)
{
if( !j17valid[ij17] )
{
    continue;
}
_ij17[0] = ij17; _ij17[1] = -1;
for(int iij17 = ij17+1; iij17 < 1; ++iij17)
{
if( j17valid[iij17] && IKabs(cj17array[ij17]-cj17array[iij17]) < IKFAST_SOLUTION_THRESH && IKabs(sj17array[ij17]-sj17array[iij17]) < IKFAST_SOLUTION_THRESH )
{
    j17valid[iij17]=false; _ij17[1] = iij17; break; 
}
}
j17 = j17array[ij17]; cj17 = cj17array[ij17]; sj17 = sj17array[ij17];
{
IkReal evalcond[4];
IkReal x620=IKsin(j17);
IkReal x621=IKcos(j17);
IkReal x622=((IkReal(0.0440000000000000))*(r00));
IkReal x623=((IkReal(0.0440000000000000))*(cj18));
IkReal x624=((cj15)*(r01));
IkReal x625=((IkReal(0.0533617455371189))*(sj16));
IkReal x626=((cj16)*(r02));
IkReal x627=((cj18)*(sj16));
IkReal x628=((IkReal(0.0348264139254172))*(sj18));
IkReal x629=((IkReal(0.533617455371189))*(py));
IkReal x630=((IkReal(0.533617455371189))*(px)*(sj15));
IkReal x631=((IkReal(0.533617455371189))*(cj16)*(pz));
IkReal x632=((r01)*(sj15)*(sj16));
IkReal x633=((IkReal(0.533617455371189))*(cj15)*(px));
evalcond[0]=((((IkReal(-0.0440000000000000))*(x624)))+(((sj15)*(x622)))+(((IkReal(-1.00000000000000))*(x620)*(x628)))+(((IkReal(-1.00000000000000))*(x630)))+(((cj15)*(x629))));
evalcond[1]=((((cj15)*(cj18)*(x629)))+(((IkReal(-1.00000000000000))*(x623)*(x624)))+(((IkReal(-1.00000000000000))*(cj18)*(x630)))+(((cj18)*(sj15)*(x622)))+(((IkReal(-1.00000000000000))*(cj18)*(x620)*(x628))));
evalcond[2]=((((sj16)*(x633)))+(((IkReal(-0.0440000000000000))*(x626)))+(((IkReal(-0.0440000000000000))*(x632)))+(((IkReal(-1.00000000000000))*(x625)))+(((IkReal(-1.00000000000000))*(cj15)*(sj16)*(x622)))+(x631)+(((x621)*(x628)))+(((sj15)*(sj16)*(x629))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj15)*(x627)*(x629)))+(((cj15)*(x622)*(x627)))+(((IkReal(-1.00000000000000))*(cj18)*(x631)))+(((IkReal(-1.00000000000000))*(x627)*(x633)))+(((cj18)*(x625)))+(((IkReal(-1.00000000000000))*(cj18)*(x621)*(x628)))+(((x623)*(x632)))+(((x623)*(x626))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x634=(cj17)*(cj17);
IkReal x635=(sj17)*(sj17);
dummyeval[0]=((x634)+(x635));
dummyeval[1]=((((cj18)*(x635)))+(((cj18)*(x634))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj17;
dummyeval[1]=sj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj17;
dummyeval[1]=sj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j17)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x636=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.135000000000000))+(((IkReal(-1.00000000000000))*(px)*(x636)))+(((cj15)*(py))));
evalcond[2]=((IkReal(-0.845725967038483))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x636))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x637=((IkReal(1.87400166530233))*(cj16));
IkReal x638=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x638)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x638)))+(((IkReal(-22.7272727272727))*(cj16)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj15)*(r00)*(x637)))+(((r01)*(sj15)*(x637)))+(((IkReal(-1.87400166530233))*(r02)*(sj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x638)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x638)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))))+IKsqr(((((cj15)*(r00)*(x637)))+(((r01)*(sj15)*(x637)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x638)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x638)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))), ((((cj15)*(r00)*(x637)))+(((r01)*(sj15)*(x637)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x639=IKcos(j19);
IkReal x640=IKsin(j19);
IkReal x641=((py)*(sj15));
IkReal x642=((IkReal(1.00000000000000))*(sj16));
IkReal x643=((cj15)*(px));
IkReal x644=((IkReal(1.00000000000000))*(cj16));
IkReal x645=((cj15)*(r00));
IkReal x646=((r01)*(sj15));
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(-0.0440000000000000))*(x639)))+(((cj16)*(x641)))+(((cj16)*(x643)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x642))));
evalcond[1]=((((IkReal(-0.533617455371189))*(x639)))+(((cj16)*(x645)))+(((cj16)*(x646)))+(((IkReal(-1.00000000000000))*(r02)*(x642))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x642)*(x643)))+(((IkReal(-1.00000000000000))*(x641)*(x642)))+(((IkReal(-0.0440000000000000))*(x640)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x644))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x642)*(x646)))+(((IkReal(-1.00000000000000))*(x642)*(x645)))+(((IkReal(-0.533617455371189))*(x640)))+(((IkReal(-1.00000000000000))*(r02)*(x644))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x647=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.135000000000000))+(((IkReal(-1.00000000000000))*(px)*(x647)))+(((cj15)*(py))));
evalcond[2]=((IkReal(0.845725967038483))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x647))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x648=((IkReal(1.87400166530233))*(cj16));
IkReal x649=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x649)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x649)))+(((IkReal(-22.7272727272727))*(cj16)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x648)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x648))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x649)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x649)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))))+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x648)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x648)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x649)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x649)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))), ((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x648)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x648)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x650=IKcos(j19);
IkReal x651=IKsin(j19);
IkReal x652=((py)*(sj15));
IkReal x653=((IkReal(1.00000000000000))*(sj16));
IkReal x654=((cj15)*(px));
IkReal x655=((IkReal(1.00000000000000))*(cj16));
IkReal x656=((cj15)*(r00));
IkReal x657=((r01)*(sj15));
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(0.0440000000000000))*(x650)))+(((IkReal(-0.100000000000000))*(cj16)))+(((cj16)*(x654)))+(((cj16)*(x652)))+(((IkReal(-1.00000000000000))*(pz)*(x653))));
evalcond[1]=((((IkReal(0.533617455371189))*(x650)))+(((cj16)*(x657)))+(((cj16)*(x656)))+(((IkReal(-1.00000000000000))*(r02)*(x653))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(x651)))+(((IkReal(-1.00000000000000))*(pz)*(x655)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(x653)*(x654)))+(((IkReal(-1.00000000000000))*(x652)*(x653))));
evalcond[3]=((((IkReal(-0.533617455371189))*(x651)))+(((IkReal(-1.00000000000000))*(r02)*(x655)))+(((IkReal(-1.00000000000000))*(x653)*(x656)))+(((IkReal(-1.00000000000000))*(x653)*(x657))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x658=((IkReal(1.87400166530233))*(sj16));
if( IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x658)))+(((IkReal(-1.87400166530233))*(cj16)*(r02)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x658))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x658)))+(((IkReal(-1.87400166530233))*(cj16)*(r02)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x658)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x658)))+(((IkReal(-1.87400166530233))*(cj16)*(r02)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x658)))), ((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x659=IKcos(j19);
IkReal x660=IKsin(j19);
IkReal x661=((IkReal(1.00000000000000))*(sj16));
IkReal x662=((r01)*(sj15));
IkReal x663=((IkReal(1.00000000000000))*(cj16));
IkReal x664=((IkReal(1.00000000000000))*(sj15));
IkReal x665=((cj15)*(px));
IkReal x666=((py)*(sj15));
IkReal x667=((cj15)*(r00));
IkReal x668=((IkReal(0.0440000000000000))*(x659));
IkReal x669=((IkReal(0.533617455371189))*(x659));
evalcond[0]=((((IkReal(-0.135000000000000))*(sj18)))+(((cj18)*(x668)))+(((IkReal(-1.00000000000000))*(px)*(x664)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(sj18)*(x668)))+(((cj16)*(x666)))+(((cj16)*(x665)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x661)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj18)*(x669)))+(((cj16)*(x667)))+(((cj16)*(x662)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x661))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x664)))+(((IkReal(-0.845725967038483))*(sj18)))+(((cj18)*(x669))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x661)*(x665)))+(((IkReal(-1.00000000000000))*(x661)*(x666)))+(((IkReal(-1.00000000000000))*(pz)*(x663)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-0.0440000000000000))*(x660))));
evalcond[5]=((((IkReal(-0.533617455371189))*(x660)))+(((IkReal(-1.00000000000000))*(x661)*(x662)))+(((IkReal(-1.00000000000000))*(x661)*(x667)))+(((IkReal(-1.00000000000000))*(r02)*(x663))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x670=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((IkReal(-1.00000000000000))*(py)*(sj15)*(x670)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x670)))+(((IkReal(-22.7272727272727))*(cj16)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))+(((IkReal(84573.0000000000))*(sj18))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(py)*(sj15)*(x670)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x670)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))))+IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))+(((IkReal(84573.0000000000))*(sj18)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IkReal(-1.00000000000000))*(py)*(sj15)*(x670)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x670)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))), ((IkReal(1.87399272890821e-5))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))+(((IkReal(84573.0000000000))*(sj18)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x671=IKcos(j19);
IkReal x672=IKsin(j19);
IkReal x673=((IkReal(1.00000000000000))*(sj16));
IkReal x674=((r01)*(sj15));
IkReal x675=((IkReal(1.00000000000000))*(cj16));
IkReal x676=((IkReal(1.00000000000000))*(sj15));
IkReal x677=((cj15)*(px));
IkReal x678=((py)*(sj15));
IkReal x679=((cj15)*(r00));
IkReal x680=((IkReal(0.0440000000000000))*(x671));
IkReal x681=((IkReal(0.533617455371189))*(x671));
evalcond[0]=((((IkReal(-0.135000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(px)*(x676)))+(((cj18)*(x680)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x673)))+(((IkReal(-1.00000000000000))*(sj18)*(x680)))+(((cj16)*(x677)))+(((cj16)*(x678)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x673)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(sj18)*(x681)))+(((cj16)*(x674)))+(((cj16)*(x679))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(-0.845725967038483))*(sj18)))+(((cj18)*(x681)))+(((IkReal(-1.00000000000000))*(r00)*(x676))));
evalcond[4]=((((IkReal(-1.00000000000000))*(pz)*(x675)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(x673)*(x678)))+(((IkReal(-1.00000000000000))*(x673)*(x677)))+(((IkReal(-0.0440000000000000))*(x672))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x675)))+(((IkReal(-0.533617455371189))*(x672)))+(((IkReal(-1.00000000000000))*(x673)*(x679)))+(((IkReal(-1.00000000000000))*(x673)*(x674))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x682=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x682)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x682))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x682)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x682)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj15)*(px)*(x682)))+(((IkReal(2.27272727272727))*(sj16)))+(((IkReal(-22.7272727272727))*(cj16)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x682)))), ((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x683=IKcos(j19);
IkReal x684=IKsin(j19);
IkReal x685=((IkReal(1.00000000000000))*(sj16));
IkReal x686=((r01)*(sj15));
IkReal x687=((IkReal(1.00000000000000))*(cj16));
IkReal x688=((IkReal(1.00000000000000))*(sj15));
IkReal x689=((cj15)*(px));
IkReal x690=((py)*(sj15));
IkReal x691=((cj15)*(r00));
IkReal x692=((IkReal(0.0440000000000000))*(x683));
IkReal x693=((IkReal(0.533617455371189))*(x683));
evalcond[0]=((((cj18)*(x692)))+(((IkReal(-0.135000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(px)*(x688)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(sj18)*(x692)))+(((cj16)*(x690)))+(((IkReal(-0.100000000000000))*(cj16)))+(((cj16)*(x689)))+(((IkReal(-1.00000000000000))*(pz)*(x685)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj18)*(x693)))+(((cj16)*(x691)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x685)))+(((cj16)*(x686))));
evalcond[3]=((((cj18)*(x693)))+(((cj15)*(r01)))+(((IkReal(-0.845725967038483))*(sj18)))+(((IkReal(-1.00000000000000))*(r00)*(x688))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x685)*(x689)))+(((IkReal(-0.0440000000000000))*(x684)))+(((IkReal(-1.00000000000000))*(x685)*(x690)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x687))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x685)*(x686)))+(((IkReal(-1.00000000000000))*(r02)*(x687)))+(((IkReal(-1.00000000000000))*(x685)*(x691)))+(((IkReal(-0.533617455371189))*(x684))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j17)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj18;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x694=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.135000000000000))+(((IkReal(-1.00000000000000))*(px)*(x694)))+(((cj15)*(py))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r00)*(x694)))+(((cj15)*(r01))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x695=((IkReal(1.87400166530233))*(cj16));
IkReal x696=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((py)*(sj15)*(x696)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((cj15)*(px)*(x696)))+(((IkReal(-2.27272727272727))*(sj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r01)*(sj15)*(x695)))+(((cj15)*(r00)*(x695)))+(((IkReal(-1.87400166530233))*(r02)*(sj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((py)*(sj15)*(x696)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((cj15)*(px)*(x696)))+(((IkReal(-2.27272727272727))*(sj16)))))+IKsqr(((((r01)*(sj15)*(x695)))+(((cj15)*(r00)*(x695)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((py)*(sj15)*(x696)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((cj15)*(px)*(x696)))+(((IkReal(-2.27272727272727))*(sj16)))), ((((r01)*(sj15)*(x695)))+(((cj15)*(r00)*(x695)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x697=IKcos(j19);
IkReal x698=IKsin(j19);
IkReal x699=((py)*(sj15));
IkReal x700=((IkReal(1.00000000000000))*(sj16));
IkReal x701=((cj15)*(px));
IkReal x702=((IkReal(1.00000000000000))*(cj16));
IkReal x703=((cj15)*(r00));
IkReal x704=((r01)*(sj15));
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x700)))+(((cj16)*(x701)))+(((cj16)*(x699)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-0.0440000000000000))*(x697))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x700)))+(((cj16)*(x704)))+(((cj16)*(x703)))+(((IkReal(-0.533617455371189))*(x697))));
evalcond[2]=((((IkReal(0.0440000000000000))*(x698)))+(((IkReal(-1.00000000000000))*(pz)*(x702)))+(((IkReal(-1.00000000000000))*(x699)*(x700)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(x700)*(x701))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x702)))+(((IkReal(0.533617455371189))*(x698)))+(((IkReal(-1.00000000000000))*(x700)*(x703)))+(((IkReal(-1.00000000000000))*(x700)*(x704))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x705=((IkReal(1.00000000000000))*(sj15));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.135000000000000))+(((IkReal(-1.00000000000000))*(px)*(x705)))+(((cj15)*(py))));
evalcond[2]=((IkReal(-0.845725967038483))+(((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x705))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x706=((IkReal(1.87400166530233))*(cj16));
IkReal x707=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((IkReal(22.7272727272727))*(cj16)*(pz)))+(((cj15)*(px)*(x707)))+(((IkReal(-2.27272727272727))*(sj16)))+(((py)*(sj15)*(x707))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj15)*(r00)*(x706)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x706))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(22.7272727272727))*(cj16)*(pz)))+(((cj15)*(px)*(x707)))+(((IkReal(-2.27272727272727))*(sj16)))+(((py)*(sj15)*(x707)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj15)*(r00)*(x706)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x706)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IkReal(22.7272727272727))*(cj16)*(pz)))+(((cj15)*(px)*(x707)))+(((IkReal(-2.27272727272727))*(sj16)))+(((py)*(sj15)*(x707)))), ((((IkReal(-1.00000000000000))*(cj15)*(r00)*(x706)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x706)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x708=IKcos(j19);
IkReal x709=IKsin(j19);
IkReal x710=((py)*(sj15));
IkReal x711=((IkReal(1.00000000000000))*(sj16));
IkReal x712=((cj15)*(px));
IkReal x713=((IkReal(1.00000000000000))*(cj16));
IkReal x714=((cj15)*(r00));
IkReal x715=((r01)*(sj15));
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x711)))+(((cj16)*(x712)))+(((cj16)*(x710)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(0.0440000000000000))*(x708))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x711)))+(((IkReal(0.533617455371189))*(x708)))+(((cj16)*(x714)))+(((cj16)*(x715))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)*(x713)))+(((IkReal(-1.00000000000000))*(x711)*(x712)))+(((IkReal(-1.00000000000000))*(x710)*(x711)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(0.0440000000000000))*(x709))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x713)))+(((IkReal(-1.00000000000000))*(x711)*(x715)))+(((IkReal(-1.00000000000000))*(x711)*(x714)))+(((IkReal(0.533617455371189))*(x709))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x716=((IkReal(1.87400166530233))*(sj16));
if( IKabs(((((cj15)*(r00)*(x716)))+(((r01)*(sj15)*(x716)))+(((IkReal(1.87400166530233))*(cj16)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(px)*(sj15)))+(((IkReal(250.000000000000))*(cj15)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj15)*(r00)*(x716)))+(((r01)*(sj15)*(x716)))+(((IkReal(1.87400166530233))*(cj16)*(r02)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(px)*(sj15)))+(((IkReal(250.000000000000))*(cj15)*(py)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((cj15)*(r00)*(x716)))+(((r01)*(sj15)*(x716)))+(((IkReal(1.87400166530233))*(cj16)*(r02)))), ((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(px)*(sj15)))+(((IkReal(250.000000000000))*(cj15)*(py)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x717=IKcos(j19);
IkReal x718=IKsin(j19);
IkReal x719=((IkReal(1.00000000000000))*(sj16));
IkReal x720=((r01)*(sj15));
IkReal x721=((IkReal(1.00000000000000))*(cj16));
IkReal x722=((IkReal(1.00000000000000))*(sj15));
IkReal x723=((cj15)*(px));
IkReal x724=((py)*(sj15));
IkReal x725=((cj15)*(r00));
IkReal x726=((IkReal(0.0440000000000000))*(x717));
IkReal x727=((IkReal(0.533617455371189))*(x717));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x722)))+(((IkReal(0.135000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(cj18)*(x726)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x719)))+(((IkReal(-1.00000000000000))*(sj18)*(x726)))+(((IkReal(-0.100000000000000))*(cj16)))+(((cj16)*(x724)))+(((cj16)*(x723)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x719)))+(((IkReal(-1.00000000000000))*(sj18)*(x727)))+(((IkReal(-0.845725967038483))*(cj18)))+(((cj16)*(x725)))+(((cj16)*(x720))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r00)*(x722)))+(((cj15)*(r01)))+(((IkReal(0.845725967038483))*(sj18)))+(((IkReal(-1.00000000000000))*(cj18)*(x727))));
evalcond[4]=((((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(x719)*(x723)))+(((IkReal(-1.00000000000000))*(x719)*(x724)))+(((IkReal(0.0440000000000000))*(x718)))+(((IkReal(-1.00000000000000))*(pz)*(x721))));
evalcond[5]=((((IkReal(0.533617455371189))*(x718)))+(((IkReal(-1.00000000000000))*(r02)*(x721)))+(((IkReal(-1.00000000000000))*(x719)*(x725)))+(((IkReal(-1.00000000000000))*(x719)*(x720))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x728=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((cj15)*(px)*(x728)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((IkReal(-2.27272727272727))*(sj16)))+(((py)*(sj15)*(x728))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(cj15)*(r01)))+(((IkReal(-100000.476863863))*(r00)*(sj15)))+(((IkReal(84573.0000000000))*(sj18))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj15)*(px)*(x728)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((IkReal(-2.27272727272727))*(sj16)))+(((py)*(sj15)*(x728)))))+IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(cj15)*(r01)))+(((IkReal(-100000.476863863))*(r00)*(sj15)))+(((IkReal(84573.0000000000))*(sj18)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((cj15)*(px)*(x728)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((IkReal(-2.27272727272727))*(sj16)))+(((py)*(sj15)*(x728)))), ((IkReal(1.87399272890821e-5))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(cj15)*(r01)))+(((IkReal(-100000.476863863))*(r00)*(sj15)))+(((IkReal(84573.0000000000))*(sj18)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x729=IKcos(j19);
IkReal x730=IKsin(j19);
IkReal x731=((IkReal(1.00000000000000))*(sj16));
IkReal x732=((r01)*(sj15));
IkReal x733=((IkReal(1.00000000000000))*(cj16));
IkReal x734=((IkReal(1.00000000000000))*(sj15));
IkReal x735=((cj15)*(px));
IkReal x736=((py)*(sj15));
IkReal x737=((cj15)*(r00));
IkReal x738=((IkReal(0.0440000000000000))*(x729));
IkReal x739=((IkReal(0.533617455371189))*(x729));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x734)))+(((IkReal(0.135000000000000))*(sj18)))+(((IkReal(-1.00000000000000))*(cj18)*(x738)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(sj18)*(x738)))+(((IkReal(-0.100000000000000))*(cj16)))+(((cj16)*(x735)))+(((cj16)*(x736)))+(((IkReal(-1.00000000000000))*(pz)*(x731)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj18)*(x739)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x731)))+(((cj16)*(x737)))+(((cj16)*(x732))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(0.845725967038483))*(sj18)))+(((IkReal(-1.00000000000000))*(cj18)*(x739)))+(((IkReal(-1.00000000000000))*(r00)*(x734))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x731)*(x735)))+(((IkReal(-1.00000000000000))*(x731)*(x736)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x733)))+(((IkReal(0.0440000000000000))*(x730))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x731)*(x732)))+(((IkReal(-1.00000000000000))*(x731)*(x737)))+(((IkReal(-1.00000000000000))*(r02)*(x733)))+(((IkReal(0.533617455371189))*(x730))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x740=((IkReal(22.7272727272727))*(sj16));
if( IKabs(((((py)*(sj15)*(x740)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((IkReal(-2.27272727272727))*(sj16)))+(((cj15)*(px)*(x740))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(px)*(sj15)))+(((IkReal(250.000000000000))*(cj15)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((py)*(sj15)*(x740)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((IkReal(-2.27272727272727))*(sj16)))+(((cj15)*(px)*(x740)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(px)*(sj15)))+(((IkReal(250.000000000000))*(cj15)*(py)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((py)*(sj15)*(x740)))+(((IkReal(22.7272727272727))*(cj16)*(pz)))+(((IkReal(-2.27272727272727))*(sj16)))+(((cj15)*(px)*(x740)))), ((IkReal(0.0909090909090909))*(((IKabs(cj18) != 0)?((IkReal)1/(cj18)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj18)))+(((IkReal(-250.000000000000))*(px)*(sj15)))+(((IkReal(250.000000000000))*(cj15)*(py)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x741=IKcos(j19);
IkReal x742=IKsin(j19);
IkReal x743=((IkReal(1.00000000000000))*(sj16));
IkReal x744=((r01)*(sj15));
IkReal x745=((IkReal(1.00000000000000))*(cj16));
IkReal x746=((IkReal(1.00000000000000))*(sj15));
IkReal x747=((cj15)*(px));
IkReal x748=((py)*(sj15));
IkReal x749=((cj15)*(r00));
IkReal x750=((IkReal(0.0440000000000000))*(x741));
IkReal x751=((IkReal(0.533617455371189))*(x741));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj18)*(x750)))+(((IkReal(-1.00000000000000))*(px)*(x746)))+(((IkReal(0.135000000000000))*(sj18)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj16)*(x747)))+(((cj16)*(x748)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x743)))+(((IkReal(-1.00000000000000))*(sj18)*(x750)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((cj16)*(x744)))+(((cj16)*(x749)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(sj18)*(x751)))+(((IkReal(-1.00000000000000))*(r02)*(x743))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj18)*(x751)))+(((cj15)*(r01)))+(((IkReal(0.845725967038483))*(sj18)))+(((IkReal(-1.00000000000000))*(r00)*(x746))));
evalcond[4]=((((IkReal(0.0440000000000000))*(x742)))+(((IkReal(-1.00000000000000))*(pz)*(x745)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(x743)*(x747)))+(((IkReal(-1.00000000000000))*(x743)*(x748))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x745)))+(((IkReal(0.533617455371189))*(x742)))+(((IkReal(-1.00000000000000))*(x743)*(x744)))+(((IkReal(-1.00000000000000))*(x743)*(x749))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x752=((cj16)*(sj15));
IkReal x753=((IkReal(1.00000000000000))*(sj16));
IkReal x754=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j18)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((py)*(x752)))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x754)))+(((IkReal(-1.00000000000000))*(pz)*(x753))));
evalcond[2]=((IkReal(-0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x753)))+(((r01)*(x752)))+(((r00)*(x754))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign(((((IkReal(11.0000000000000))*((sj17)*(sj17))))+(((IkReal(11.0000000000000))*((cj17)*(cj17))))));
dummyeval[0]=(((sj17)*(sj17))+((cj17)*(cj17)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x755=((sj15)*(sj16));
IkReal x756=((IkReal(250.000000000000))*(px));
IkReal x757=((sj16)*(sj17));
IkReal x758=((cj17)*(sj16));
IkReal x759=((IkReal(250.000000000000))*(cj17)*(py));
IkReal x760=((IkReal(250.000000000000))*(py)*(sj17));
IkReal x761=((IkReal(250.000000000000))*(cj16)*(pz));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(sj17)*(x761)))+(((IkReal(-1.00000000000000))*(cj15)*(x756)*(x757)))+(((IkReal(-1.00000000000000))*(x755)*(x760)))+(((cj17)*(sj15)*(x756)))+(((IkReal(25.0000000000000))*(x757)))+(((IkReal(-1.00000000000000))*(cj15)*(x759))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((cj17)*(x761)))+(((cj15)*(x756)*(x758)))+(((IkReal(-1.00000000000000))*(cj15)*(x760)))+(((sj15)*(sj17)*(x756)))+(((IkReal(-25.0000000000000))*(x758)))+(((x755)*(x759))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j19array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(sj17)*(x761)))+(((IkReal(-1.00000000000000))*(cj15)*(x756)*(x757)))+(((IkReal(-1.00000000000000))*(x755)*(x760)))+(((cj17)*(sj15)*(x756)))+(((IkReal(25.0000000000000))*(x757)))+(((IkReal(-1.00000000000000))*(cj15)*(x759)))))), ((gconst54)*(((((cj17)*(x761)))+(((cj15)*(x756)*(x758)))+(((IkReal(-1.00000000000000))*(cj15)*(x760)))+(((sj15)*(sj17)*(x756)))+(((IkReal(-25.0000000000000))*(x758)))+(((x755)*(x759)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x762=IKsin(j19);
IkReal x763=IKcos(j19);
IkReal x764=((IkReal(1.00000000000000))*(sj16));
IkReal x765=((IkReal(1.00000000000000))*(cj16));
IkReal x766=((IkReal(1.00000000000000))*(sj15));
IkReal x767=((IkReal(0.533617455371189))*(x763));
IkReal x768=((IkReal(0.0440000000000000))*(x762));
IkReal x769=((IkReal(0.533617455371189))*(x762));
IkReal x770=((IkReal(0.0440000000000000))*(x763));
evalcond[0]=((((cj17)*(x768)))+(((IkReal(-1.00000000000000))*(px)*(x766)))+(((sj17)*(x770)))+(((cj15)*(py))));
evalcond[1]=((((cj17)*(x769)))+(((IkReal(-1.00000000000000))*(r00)*(x766)))+(((cj15)*(r01)))+(((sj17)*(x767))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj15)*(x764)))+(((cj17)*(x770)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x764)))+(((IkReal(-1.00000000000000))*(sj17)*(x768)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x765))));
evalcond[3]=((((cj17)*(x767)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x764)))+(((IkReal(-1.00000000000000))*(sj17)*(x769)))+(((IkReal(-1.00000000000000))*(r02)*(x765)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x764))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x771=((cj16)*(sj15));
IkReal x772=((IkReal(1.00000000000000))*(sj16));
IkReal x773=((cj15)*(cj16));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j18, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((IkReal(-0.100000000000000))*(cj16)))+(((px)*(x773)))+(((IkReal(-1.00000000000000))*(pz)*(x772)))+(((py)*(x771))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x772)))+(((r01)*(x771)))+(((r00)*(x773))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign(((((IkReal(11.0000000000000))*((sj17)*(sj17))))+(((IkReal(11.0000000000000))*((cj17)*(cj17))))));
dummyeval[0]=(((sj17)*(sj17))+((cj17)*(cj17)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x774=((sj15)*(sj16));
IkReal x775=((IkReal(250.000000000000))*(px));
IkReal x776=((sj16)*(sj17));
IkReal x777=((cj17)*(sj16));
IkReal x778=((IkReal(250.000000000000))*(cj17)*(py));
IkReal x779=((IkReal(250.000000000000))*(py)*(sj17));
IkReal x780=((IkReal(250.000000000000))*(cj16)*(pz));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(cj15)*(x775)*(x776)))+(((IkReal(-1.00000000000000))*(x774)*(x779)))+(((IkReal(-1.00000000000000))*(sj17)*(x780)))+(((IkReal(-1.00000000000000))*(cj15)*(x778)))+(((cj17)*(sj15)*(x775)))+(((IkReal(25.0000000000000))*(x776))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(cj15)*(x775)*(x777)))+(((IkReal(-1.00000000000000))*(cj17)*(x780)))+(((cj15)*(x779)))+(((IkReal(-1.00000000000000))*(x774)*(x778)))+(((IkReal(-1.00000000000000))*(sj15)*(sj17)*(x775)))+(((IkReal(25.0000000000000))*(x777))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j19array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(cj15)*(x775)*(x776)))+(((IkReal(-1.00000000000000))*(x774)*(x779)))+(((IkReal(-1.00000000000000))*(sj17)*(x780)))+(((IkReal(-1.00000000000000))*(cj15)*(x778)))+(((cj17)*(sj15)*(x775)))+(((IkReal(25.0000000000000))*(x776)))))), ((gconst55)*(((((IkReal(-1.00000000000000))*(cj15)*(x775)*(x777)))+(((IkReal(-1.00000000000000))*(cj17)*(x780)))+(((cj15)*(x779)))+(((IkReal(-1.00000000000000))*(x774)*(x778)))+(((IkReal(-1.00000000000000))*(sj15)*(sj17)*(x775)))+(((IkReal(25.0000000000000))*(x777)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[4];
IkReal x781=IKsin(j19);
IkReal x782=IKcos(j19);
IkReal x783=((IkReal(1.00000000000000))*(sj16));
IkReal x784=((IkReal(1.00000000000000))*(cj16));
IkReal x785=((IkReal(1.00000000000000))*(sj15));
IkReal x786=((IkReal(0.533617455371189))*(x782));
IkReal x787=((IkReal(0.0440000000000000))*(x781));
IkReal x788=((IkReal(0.533617455371189))*(x781));
IkReal x789=((IkReal(0.0440000000000000))*(x782));
evalcond[0]=((((cj17)*(x787)))+(((IkReal(-1.00000000000000))*(sj17)*(x789)))+(((IkReal(-1.00000000000000))*(px)*(x785)))+(((cj15)*(py))));
evalcond[1]=((((cj15)*(r01)))+(((cj17)*(x788)))+(((IkReal(-1.00000000000000))*(r00)*(x785)))+(((IkReal(-1.00000000000000))*(sj17)*(x786))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj17)*(x789)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x783)))+(((IkReal(-1.00000000000000))*(sj17)*(x787)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x783)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(pz)*(x784))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj17)*(x786)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x783)))+(((IkReal(-1.00000000000000))*(sj17)*(x788)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x783)))+(((IkReal(-1.00000000000000))*(r02)*(x784))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j18)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj17;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj17;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj17;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x790=((IkReal(22.7272727272727))*(cj16));
if( IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))+(((IkReal(84573.0000000000))*(sj17))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-9.09090909090909))+(((IkReal(-22.7272727272727))*(pz)*(sj16)))+(((IkReal(-2.27272727272727))*(cj16)))+(((cj15)*(px)*(x790)))+(((py)*(sj15)*(x790))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))+(((IkReal(84573.0000000000))*(sj17)))))))+IKsqr(((IkReal(-9.09090909090909))+(((IkReal(-22.7272727272727))*(pz)*(sj16)))+(((IkReal(-2.27272727272727))*(cj16)))+(((cj15)*(px)*(x790)))+(((py)*(sj15)*(x790)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(1.87399272890821e-5))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))+(((IkReal(84573.0000000000))*(sj17)))))), ((IkReal(-9.09090909090909))+(((IkReal(-22.7272727272727))*(pz)*(sj16)))+(((IkReal(-2.27272727272727))*(cj16)))+(((cj15)*(px)*(x790)))+(((py)*(sj15)*(x790)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x791=IKcos(j19);
IkReal x792=IKsin(j19);
IkReal x793=((IkReal(1.00000000000000))*(sj16));
IkReal x794=((r01)*(sj15));
IkReal x795=((IkReal(1.00000000000000))*(cj16));
IkReal x796=((IkReal(1.00000000000000))*(sj15));
IkReal x797=((cj15)*(px));
IkReal x798=((py)*(sj15));
IkReal x799=((cj15)*(r00));
IkReal x800=((cj17)*(x792));
IkReal x801=((sj17)*(x792));
evalcond[0]=((((IkReal(-0.135000000000000))*(sj17)))+(((IkReal(0.0440000000000000))*(x800)))+(((IkReal(-1.00000000000000))*(px)*(x796)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj16)*(x798)))+(((cj16)*(x797)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x793)))+(((IkReal(-0.0440000000000000))*(x791))));
evalcond[2]=((((cj16)*(x799)))+(((cj16)*(x794)))+(((IkReal(-1.00000000000000))*(r02)*(x793)))+(((IkReal(-0.533617455371189))*(x791))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x796)))+(((IkReal(-0.845725967038483))*(sj17)))+(((IkReal(0.533617455371189))*(x800))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x793)*(x797)))+(((IkReal(-1.00000000000000))*(x793)*(x798)))+(((IkReal(-0.0440000000000000))*(x801)))+(((IkReal(-1.00000000000000))*(pz)*(x795)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-0.135000000000000))*(cj17))));
evalcond[5]=((((IkReal(-0.533617455371189))*(x801)))+(((IkReal(-0.845725967038483))*(cj17)))+(((IkReal(-1.00000000000000))*(x793)*(x794)))+(((IkReal(-1.00000000000000))*(x793)*(x799)))+(((IkReal(-1.00000000000000))*(r02)*(x795))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x802=((IkReal(1.87400166530233))*(cj16));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj17)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj15)*(r00)*(x802)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))+(((r01)*(sj15)*(x802))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj17)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))))+IKsqr(((((cj15)*(r00)*(x802)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))+(((r01)*(sj15)*(x802)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj17)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))), ((((cj15)*(r00)*(x802)))+(((IkReal(-1.87400166530233))*(r02)*(sj16)))+(((r01)*(sj15)*(x802)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x803=IKcos(j19);
IkReal x804=IKsin(j19);
IkReal x805=((IkReal(1.00000000000000))*(sj16));
IkReal x806=((r01)*(sj15));
IkReal x807=((IkReal(1.00000000000000))*(cj16));
IkReal x808=((IkReal(1.00000000000000))*(sj15));
IkReal x809=((cj15)*(px));
IkReal x810=((py)*(sj15));
IkReal x811=((cj15)*(r00));
IkReal x812=((cj17)*(x804));
IkReal x813=((sj17)*(x804));
evalcond[0]=((((IkReal(-0.135000000000000))*(sj17)))+(((IkReal(-1.00000000000000))*(px)*(x808)))+(((IkReal(0.0440000000000000))*(x812)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj16)*(x810)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-0.0440000000000000))*(x803)))+(((cj16)*(x809)))+(((IkReal(-1.00000000000000))*(pz)*(x805))));
evalcond[2]=((((IkReal(-0.533617455371189))*(x803)))+(((cj16)*(x811)))+(((IkReal(-1.00000000000000))*(r02)*(x805)))+(((cj16)*(x806))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(-0.845725967038483))*(sj17)))+(((IkReal(-1.00000000000000))*(r00)*(x808)))+(((IkReal(0.533617455371189))*(x812))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x805)*(x809)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(x805)*(x810)))+(((IkReal(-1.00000000000000))*(pz)*(x807)))+(((IkReal(-0.0440000000000000))*(x813)))+(((IkReal(-0.135000000000000))*(cj17))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x805)*(x806)))+(((IkReal(-0.845725967038483))*(cj17)))+(((IkReal(-1.00000000000000))*(r02)*(x807)))+(((IkReal(-0.533617455371189))*(x813)))+(((IkReal(-1.00000000000000))*(x805)*(x811))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x814=((IkReal(22.7272727272727))*(cj16));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj17)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-9.09090909090909))+(((IkReal(-22.7272727272727))*(pz)*(sj16)))+(((IkReal(-2.27272727272727))*(cj16)))+(((cj15)*(px)*(x814)))+(((py)*(sj15)*(x814))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj17)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))))+IKsqr(((IkReal(-9.09090909090909))+(((IkReal(-22.7272727272727))*(pz)*(sj16)))+(((IkReal(-2.27272727272727))*(cj16)))+(((cj15)*(px)*(x814)))+(((py)*(sj15)*(x814)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj17)))+(((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(250.000000000000))*(px)*(sj15)))))), ((IkReal(-9.09090909090909))+(((IkReal(-22.7272727272727))*(pz)*(sj16)))+(((IkReal(-2.27272727272727))*(cj16)))+(((cj15)*(px)*(x814)))+(((py)*(sj15)*(x814)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x815=IKcos(j19);
IkReal x816=IKsin(j19);
IkReal x817=((IkReal(1.00000000000000))*(sj16));
IkReal x818=((r01)*(sj15));
IkReal x819=((IkReal(1.00000000000000))*(cj16));
IkReal x820=((IkReal(1.00000000000000))*(sj15));
IkReal x821=((cj15)*(px));
IkReal x822=((py)*(sj15));
IkReal x823=((cj15)*(r00));
IkReal x824=((cj17)*(x816));
IkReal x825=((sj17)*(x816));
evalcond[0]=((((IkReal(-0.135000000000000))*(sj17)))+(((IkReal(-1.00000000000000))*(px)*(x820)))+(((IkReal(0.0440000000000000))*(x824)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x817)))+(((cj16)*(x821)))+(((cj16)*(x822)))+(((IkReal(-0.0440000000000000))*(x815))));
evalcond[2]=((((cj16)*(x818)))+(((cj16)*(x823)))+(((IkReal(-0.533617455371189))*(x815)))+(((IkReal(-1.00000000000000))*(r02)*(x817))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(0.533617455371189))*(x824)))+(((IkReal(-0.845725967038483))*(sj17)))+(((IkReal(-1.00000000000000))*(r00)*(x820))));
evalcond[4]=((((IkReal(-0.0440000000000000))*(x825)))+(((IkReal(-1.00000000000000))*(pz)*(x819)))+(((IkReal(-1.00000000000000))*(x817)*(x821)))+(((IkReal(-1.00000000000000))*(x817)*(x822)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-0.135000000000000))*(cj17))));
evalcond[5]=((((IkReal(-0.845725967038483))*(cj17)))+(((IkReal(-1.00000000000000))*(x817)*(x818)))+(((IkReal(-0.533617455371189))*(x825)))+(((IkReal(-1.00000000000000))*(x817)*(x823)))+(((IkReal(-1.00000000000000))*(r02)*(x819))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j18)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj17;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj17;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj17;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x826=((IkReal(22.7272727272727))*(cj16));
if( IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-84573.0000000000))*(sj17)))+(((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x826)))+(((IkReal(22.7272727272727))*(pz)*(sj16)))+(((IkReal(2.27272727272727))*(cj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x826))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-84573.0000000000))*(sj17)))+(((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))))))+IKsqr(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x826)))+(((IkReal(22.7272727272727))*(pz)*(sj16)))+(((IkReal(2.27272727272727))*(cj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x826)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(1.87399272890821e-5))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-84573.0000000000))*(sj17)))+(((IkReal(100000.476863863))*(r00)*(sj15)))+(((IkReal(-100000.476863863))*(cj15)*(r01)))))), ((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x826)))+(((IkReal(22.7272727272727))*(pz)*(sj16)))+(((IkReal(2.27272727272727))*(cj16)))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x826)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x827=IKcos(j19);
IkReal x828=IKsin(j19);
IkReal x829=((IkReal(1.00000000000000))*(sj16));
IkReal x830=((r01)*(sj15));
IkReal x831=((IkReal(1.00000000000000))*(cj16));
IkReal x832=((IkReal(1.00000000000000))*(sj15));
IkReal x833=((cj15)*(px));
IkReal x834=((py)*(sj15));
IkReal x835=((cj15)*(r00));
IkReal x836=((cj17)*(x828));
IkReal x837=((sj17)*(x828));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x832)))+(((IkReal(0.135000000000000))*(sj17)))+(((IkReal(0.0440000000000000))*(x836)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj16)*(x833)))+(((cj16)*(x834)))+(((IkReal(-1.00000000000000))*(pz)*(x829)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(0.0440000000000000))*(x827))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x829)))+(((cj16)*(x830)))+(((cj16)*(x835)))+(((IkReal(0.533617455371189))*(x827))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(0.533617455371189))*(x836)))+(((IkReal(0.845725967038483))*(sj17)))+(((IkReal(-1.00000000000000))*(r00)*(x832))));
evalcond[4]=((((IkReal(-1.00000000000000))*(pz)*(x831)))+(((IkReal(-0.0440000000000000))*(x837)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(0.135000000000000))*(cj17)))+(((IkReal(-1.00000000000000))*(x829)*(x834)))+(((IkReal(-1.00000000000000))*(x829)*(x833))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x831)))+(((IkReal(0.845725967038483))*(cj17)))+(((IkReal(-0.533617455371189))*(x837)))+(((IkReal(-1.00000000000000))*(x829)*(x835)))+(((IkReal(-1.00000000000000))*(x829)*(x830))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x838=((IkReal(1.87400166530233))*(cj16));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(-33.7500000000000))*(sj17)))+(((IkReal(250.000000000000))*(px)*(sj15))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x838)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x838)))+(((IkReal(1.87400166530233))*(r02)*(sj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(-33.7500000000000))*(sj17)))+(((IkReal(250.000000000000))*(px)*(sj15)))))))+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x838)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x838)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(-33.7500000000000))*(sj17)))+(((IkReal(250.000000000000))*(px)*(sj15)))))), ((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x838)))+(((IkReal(-1.00000000000000))*(cj15)*(r00)*(x838)))+(((IkReal(1.87400166530233))*(r02)*(sj16)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x839=IKcos(j19);
IkReal x840=IKsin(j19);
IkReal x841=((IkReal(1.00000000000000))*(sj16));
IkReal x842=((r01)*(sj15));
IkReal x843=((IkReal(1.00000000000000))*(cj16));
IkReal x844=((IkReal(1.00000000000000))*(sj15));
IkReal x845=((cj15)*(px));
IkReal x846=((py)*(sj15));
IkReal x847=((cj15)*(r00));
IkReal x848=((cj17)*(x840));
IkReal x849=((sj17)*(x840));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x844)))+(((IkReal(0.135000000000000))*(sj17)))+(((cj15)*(py)))+(((IkReal(0.0440000000000000))*(x848))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj16)*(x846)))+(((cj16)*(x845)))+(((IkReal(-1.00000000000000))*(pz)*(x841)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(0.0440000000000000))*(x839))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x841)))+(((cj16)*(x847)))+(((cj16)*(x842)))+(((IkReal(0.533617455371189))*(x839))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(0.533617455371189))*(x848)))+(((IkReal(0.845725967038483))*(sj17)))+(((IkReal(-1.00000000000000))*(r00)*(x844))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x841)*(x845)))+(((IkReal(-1.00000000000000))*(x841)*(x846)))+(((IkReal(-0.0440000000000000))*(x849)))+(((IkReal(-1.00000000000000))*(pz)*(x843)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(0.135000000000000))*(cj17))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x843)))+(((IkReal(-1.00000000000000))*(x841)*(x847)))+(((IkReal(-1.00000000000000))*(x841)*(x842)))+(((IkReal(0.845725967038483))*(cj17)))+(((IkReal(-0.533617455371189))*(x849))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x850=((IkReal(22.7272727272727))*(cj16));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(-33.7500000000000))*(sj17)))+(((IkReal(250.000000000000))*(px)*(sj15))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x850)))+(((IkReal(22.7272727272727))*(pz)*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x850)))+(((IkReal(2.27272727272727))*(cj16))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(-33.7500000000000))*(sj17)))+(((IkReal(250.000000000000))*(px)*(sj15)))))))+IKsqr(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x850)))+(((IkReal(22.7272727272727))*(pz)*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x850)))+(((IkReal(2.27272727272727))*(cj16)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-250.000000000000))*(cj15)*(py)))+(((IkReal(-33.7500000000000))*(sj17)))+(((IkReal(250.000000000000))*(px)*(sj15)))))), ((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(cj15)*(px)*(x850)))+(((IkReal(22.7272727272727))*(pz)*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x850)))+(((IkReal(2.27272727272727))*(cj16)))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x851=IKcos(j19);
IkReal x852=IKsin(j19);
IkReal x853=((IkReal(1.00000000000000))*(sj16));
IkReal x854=((r01)*(sj15));
IkReal x855=((IkReal(1.00000000000000))*(cj16));
IkReal x856=((IkReal(1.00000000000000))*(sj15));
IkReal x857=((cj15)*(px));
IkReal x858=((py)*(sj15));
IkReal x859=((cj15)*(r00));
IkReal x860=((cj17)*(x852));
IkReal x861=((sj17)*(x852));
evalcond[0]=((((IkReal(0.0440000000000000))*(x860)))+(((IkReal(0.135000000000000))*(sj17)))+(((cj15)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x856))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(0.0440000000000000))*(x851)))+(((cj16)*(x858)))+(((cj16)*(x857)))+(((IkReal(-1.00000000000000))*(pz)*(x853))));
evalcond[2]=((((cj16)*(x854)))+(((cj16)*(x859)))+(((IkReal(-1.00000000000000))*(r02)*(x853)))+(((IkReal(0.533617455371189))*(x851))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x856)))+(((IkReal(0.845725967038483))*(sj17)))+(((IkReal(0.533617455371189))*(x860))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x853)*(x857)))+(((IkReal(-1.00000000000000))*(x853)*(x858)))+(((IkReal(-1.00000000000000))*(pz)*(x855)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(0.135000000000000))*(cj17)))+(((IkReal(-0.0440000000000000))*(x861))));
evalcond[5]=((((IkReal(-0.533617455371189))*(x861)))+(((IkReal(-1.00000000000000))*(x853)*(x859)))+(((IkReal(-1.00000000000000))*(x853)*(x854)))+(((IkReal(0.845725967038483))*(cj17)))+(((IkReal(-1.00000000000000))*(r02)*(x855))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x862=((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30);
IkReal x863=((r02)*(sj16));
IkReal x864=((IkReal(13340500.0000000))*(sj18));
IkReal x865=((cj16)*(r01)*(sj15));
IkReal x866=((IkReal(1100005.24550249))*(cj18)*(sj17));
IkReal x867=((cj15)*(cj16)*(r00));
if( IKabs(((IkReal(1.70362975355292e-6))*(x862)*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x865)*(x866)))+(((IkReal(-1.00000000000000))*(cj15)*(py)*(x864)))+(((IkReal(-1.00000000000000))*(x866)*(x867)))+(((x863)*(x866)))+(((IkReal(1800967.50000000))*(sj17)*(((IKabs(x862) != 0)?((IkReal)1/((x862)*(x862))):(IkReal)1.0e30))))+(((px)*(sj15)*(x864)))+(((IkReal(930303.000000000))*(sj17)*((cj18)*(cj18)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.87399272890821e-5))*(x862)*(((((IkReal(100000.476863863))*(x865)))+(((IkReal(100000.476863863))*(x867)))+(((IkReal(-100000.476863863))*(x863)))+(((IkReal(-84573.0000000000))*(cj18))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1.70362975355292e-6))*(x862)*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x865)*(x866)))+(((IkReal(-1.00000000000000))*(cj15)*(py)*(x864)))+(((IkReal(-1.00000000000000))*(x866)*(x867)))+(((x863)*(x866)))+(((IkReal(1800967.50000000))*(sj17)*(((IKabs(x862) != 0)?((IkReal)1/((x862)*(x862))):(IkReal)1.0e30))))+(((px)*(sj15)*(x864)))+(((IkReal(930303.000000000))*(sj17)*((cj18)*(cj18))))))))+IKsqr(((IkReal(1.87399272890821e-5))*(x862)*(((((IkReal(100000.476863863))*(x865)))+(((IkReal(100000.476863863))*(x867)))+(((IkReal(-100000.476863863))*(x863)))+(((IkReal(-84573.0000000000))*(cj18)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((IkReal(1.70362975355292e-6))*(x862)*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x865)*(x866)))+(((IkReal(-1.00000000000000))*(cj15)*(py)*(x864)))+(((IkReal(-1.00000000000000))*(x866)*(x867)))+(((x863)*(x866)))+(((IkReal(1800967.50000000))*(sj17)*(((IKabs(x862) != 0)?((IkReal)1/((x862)*(x862))):(IkReal)1.0e30))))+(((px)*(sj15)*(x864)))+(((IkReal(930303.000000000))*(sj17)*((cj18)*(cj18))))))), ((IkReal(1.87399272890821e-5))*(x862)*(((((IkReal(100000.476863863))*(x865)))+(((IkReal(100000.476863863))*(x867)))+(((IkReal(-100000.476863863))*(x863)))+(((IkReal(-84573.0000000000))*(cj18)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x868=IKcos(j19);
IkReal x869=IKsin(j19);
IkReal x870=((cj16)*(sj15));
IkReal x871=((IkReal(1.00000000000000))*(sj16));
IkReal x872=((IkReal(1.00000000000000))*(cj16));
IkReal x873=((cj17)*(sj18));
IkReal x874=((IkReal(1.00000000000000))*(sj15));
IkReal x875=((cj15)*(px));
IkReal x876=((sj17)*(sj18));
IkReal x877=((cj15)*(r00));
IkReal x878=((IkReal(0.533617455371189))*(cj17));
IkReal x879=((IkReal(0.0440000000000000))*(x868));
IkReal x880=((IkReal(0.0440000000000000))*(x869));
IkReal x881=((cj18)*(sj17)*(x868));
evalcond[0]=((((cj17)*(x880)))+(((cj18)*(sj17)*(x879)))+(((IkReal(-1.00000000000000))*(px)*(x874)))+(((IkReal(-0.135000000000000))*(x876)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(sj18)*(x879)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x871)))+(((cj16)*(x875)))+(((py)*(x870)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((r01)*(x870)))+(((IkReal(-0.845725967038483))*(cj18)))+(((cj16)*(x877)))+(((IkReal(-1.00000000000000))*(r02)*(x871)))+(((IkReal(-0.533617455371189))*(sj18)*(x868))));
evalcond[3]=((((cj15)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x874)))+(((IkReal(0.533617455371189))*(x881)))+(((IkReal(-0.845725967038483))*(x876)))+(((x869)*(x878))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x871)*(x875)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x871)))+(((cj17)*(cj18)*(x879)))+(((IkReal(-1.00000000000000))*(pz)*(x872)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-0.135000000000000))*(x873)))+(((IkReal(-1.00000000000000))*(sj17)*(x880))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x871)*(x877)))+(((IkReal(-0.533617455371189))*(sj17)*(x869)))+(((IkReal(-0.845725967038483))*(x873)))+(((IkReal(-1.00000000000000))*(r02)*(x872)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x871)))+(((cj18)*(x868)*(x878))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x882=((IKabs(sj18) != 0)?((IkReal)1/(sj18)):(IkReal)1.0e30);
IkReal x883=((IkReal(25.0000000000000))*(cj16));
IkReal x884=((IkReal(33.7500000000000))*(sj17));
IkReal x885=((IkReal(250.000000000000))*(cj16));
IkReal x886=((py)*(sj15));
IkReal x887=((cj15)*(px));
IkReal x888=((cj18)*(sj17));
IkReal x889=((IkReal(250.000000000000))*(sj18));
IkReal x890=((IkReal(0.0909090909090909))*(x882));
IkReal x891=((IkReal(250.000000000000))*(pz)*(sj16));
if( IKabs(((x890)*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj15)*(py)*(x889)))+(((x884)*(((IKabs(x882) != 0)?((IkReal)1/((x882)*(x882))):(IkReal)1.0e30))))+(((px)*(sj15)*(x889)))+(((IkReal(100.000000000000))*(x888)))+(((x883)*(x888)))+(((IkReal(-1.00000000000000))*(x885)*(x886)*(x888)))+(((IkReal(-1.00000000000000))*(x885)*(x887)*(x888)))+(((x888)*(x891)))+(((x884)*((cj18)*(cj18)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x890)*(((IkReal(-100.000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(((IkReal(-1.00000000000000))*(x883)))+(((IkReal(-33.7500000000000))*(cj18)))+(((x885)*(x887)))+(((x885)*(x886))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x890)*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj15)*(py)*(x889)))+(((x884)*(((IKabs(x882) != 0)?((IkReal)1/((x882)*(x882))):(IkReal)1.0e30))))+(((px)*(sj15)*(x889)))+(((IkReal(100.000000000000))*(x888)))+(((x883)*(x888)))+(((IkReal(-1.00000000000000))*(x885)*(x886)*(x888)))+(((IkReal(-1.00000000000000))*(x885)*(x887)*(x888)))+(((x888)*(x891)))+(((x884)*((cj18)*(cj18))))))))+IKsqr(((x890)*(((IkReal(-100.000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(((IkReal(-1.00000000000000))*(x883)))+(((IkReal(-33.7500000000000))*(cj18)))+(((x885)*(x887)))+(((x885)*(x886)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((x890)*(((IKabs(cj17) != 0)?((IkReal)1/(cj17)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj15)*(py)*(x889)))+(((x884)*(((IKabs(x882) != 0)?((IkReal)1/((x882)*(x882))):(IkReal)1.0e30))))+(((px)*(sj15)*(x889)))+(((IkReal(100.000000000000))*(x888)))+(((x883)*(x888)))+(((IkReal(-1.00000000000000))*(x885)*(x886)*(x888)))+(((IkReal(-1.00000000000000))*(x885)*(x887)*(x888)))+(((x888)*(x891)))+(((x884)*((cj18)*(cj18))))))), ((x890)*(((IkReal(-100.000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(((IkReal(-1.00000000000000))*(x883)))+(((IkReal(-33.7500000000000))*(cj18)))+(((x885)*(x887)))+(((x885)*(x886)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x892=IKcos(j19);
IkReal x893=IKsin(j19);
IkReal x894=((cj16)*(sj15));
IkReal x895=((IkReal(1.00000000000000))*(sj16));
IkReal x896=((IkReal(1.00000000000000))*(cj16));
IkReal x897=((cj17)*(sj18));
IkReal x898=((IkReal(1.00000000000000))*(sj15));
IkReal x899=((cj15)*(px));
IkReal x900=((sj17)*(sj18));
IkReal x901=((cj15)*(r00));
IkReal x902=((IkReal(0.533617455371189))*(cj17));
IkReal x903=((IkReal(0.0440000000000000))*(x892));
IkReal x904=((IkReal(0.0440000000000000))*(x893));
IkReal x905=((cj18)*(sj17)*(x892));
evalcond[0]=((((IkReal(-0.135000000000000))*(x900)))+(((cj18)*(sj17)*(x903)))+(((IkReal(-1.00000000000000))*(px)*(x898)))+(((cj17)*(x904)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((py)*(x894)))+(((cj16)*(x899)))+(((IkReal(-0.100000000000000))*(cj16)))+(((IkReal(-1.00000000000000))*(pz)*(x895)))+(((IkReal(-1.00000000000000))*(sj18)*(x903)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((r01)*(x894)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x895)))+(((cj16)*(x901)))+(((IkReal(-0.533617455371189))*(sj18)*(x892))));
evalcond[3]=((((cj15)*(r01)))+(((x893)*(x902)))+(((IkReal(0.533617455371189))*(x905)))+(((IkReal(-0.845725967038483))*(x900)))+(((IkReal(-1.00000000000000))*(r00)*(x898))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x895)*(x899)))+(((IkReal(-1.00000000000000))*(pz)*(x896)))+(((IkReal(-1.00000000000000))*(sj17)*(x904)))+(((IkReal(-0.135000000000000))*(x897)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x895)))+(((cj17)*(cj18)*(x903))));
evalcond[5]=((((IkReal(-0.845725967038483))*(x897)))+(((IkReal(-1.00000000000000))*(x895)*(x901)))+(((IkReal(-0.533617455371189))*(sj17)*(x893)))+(((cj18)*(x892)*(x902)))+(((IkReal(-1.00000000000000))*(r02)*(x896)))+(((IkReal(-1.00000000000000))*(r01)*(sj15)*(x895))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j19array[1], cj19array[1], sj19array[1];
bool j19valid[1]={false};
_nj19 = 1;
IkReal x906=(cj17)*(cj17);
IkReal x907=(sj17)*(sj17);
IkReal x908=((sj15)*(sj16));
IkReal x909=((IkReal(33.7500000000000))*(sj18));
IkReal x910=((IkReal(250.000000000000))*(px));
IkReal x911=((sj16)*(sj17));
IkReal x912=((cj17)*(sj16));
IkReal x913=((IkReal(250.000000000000))*(cj17)*(py));
IkReal x914=((IkReal(11.0000000000000))*(x906));
IkReal x915=((IkReal(250.000000000000))*(py)*(sj17));
IkReal x916=((IkReal(11.0000000000000))*(x907));
IkReal x917=((IkReal(250.000000000000))*(cj16)*(pz));
if( IKabs(((((IKabs(((x916)+(x914))) != 0)?((IkReal)1/(((x916)+(x914)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x908)*(x915)))+(((IkReal(-1.00000000000000))*(cj15)*(x910)*(x911)))+(((IkReal(-1.00000000000000))*(cj15)*(x913)))+(((cj17)*(sj15)*(x910)))+(((IkReal(25.0000000000000))*(x911)))+(((IkReal(-1.00000000000000))*(sj17)*(x917))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj18)*(x914)))+(((cj18)*(x916))))) != 0)?((IkReal)1/(((((cj18)*(x914)))+(((cj18)*(x916)))))):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(x912)))+(((x907)*(x909)))+(((x906)*(x909)))+(((IkReal(-1.00000000000000))*(cj15)*(x915)))+(((cj17)*(x917)))+(((x908)*(x913)))+(((cj15)*(x910)*(x912)))+(((sj15)*(sj17)*(x910))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x916)+(x914))) != 0)?((IkReal)1/(((x916)+(x914)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x908)*(x915)))+(((IkReal(-1.00000000000000))*(cj15)*(x910)*(x911)))+(((IkReal(-1.00000000000000))*(cj15)*(x913)))+(((cj17)*(sj15)*(x910)))+(((IkReal(25.0000000000000))*(x911)))+(((IkReal(-1.00000000000000))*(sj17)*(x917)))))))+IKsqr(((((IKabs(((((cj18)*(x914)))+(((cj18)*(x916))))) != 0)?((IkReal)1/(((((cj18)*(x914)))+(((cj18)*(x916)))))):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(x912)))+(((x907)*(x909)))+(((x906)*(x909)))+(((IkReal(-1.00000000000000))*(cj15)*(x915)))+(((cj17)*(x917)))+(((x908)*(x913)))+(((cj15)*(x910)*(x912)))+(((sj15)*(sj17)*(x910)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j19array[0]=IKatan2(((((IKabs(((x916)+(x914))) != 0)?((IkReal)1/(((x916)+(x914)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x908)*(x915)))+(((IkReal(-1.00000000000000))*(cj15)*(x910)*(x911)))+(((IkReal(-1.00000000000000))*(cj15)*(x913)))+(((cj17)*(sj15)*(x910)))+(((IkReal(25.0000000000000))*(x911)))+(((IkReal(-1.00000000000000))*(sj17)*(x917)))))), ((((IKabs(((((cj18)*(x914)))+(((cj18)*(x916))))) != 0)?((IkReal)1/(((((cj18)*(x914)))+(((cj18)*(x916)))))):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(x912)))+(((x907)*(x909)))+(((x906)*(x909)))+(((IkReal(-1.00000000000000))*(cj15)*(x915)))+(((cj17)*(x917)))+(((x908)*(x913)))+(((cj15)*(x910)*(x912)))+(((sj15)*(sj17)*(x910)))))));
sj19array[0]=IKsin(j19array[0]);
cj19array[0]=IKcos(j19array[0]);
if( j19array[0] > IKPI )
{
    j19array[0]-=IK2PI;
}
else if( j19array[0] < -IKPI )
{    j19array[0]+=IK2PI;
}
j19valid[0] = true;
for(int ij19 = 0; ij19 < 1; ++ij19)
{
if( !j19valid[ij19] )
{
    continue;
}
_ij19[0] = ij19; _ij19[1] = -1;
for(int iij19 = ij19+1; iij19 < 1; ++iij19)
{
if( j19valid[iij19] && IKabs(cj19array[ij19]-cj19array[iij19]) < IKFAST_SOLUTION_THRESH && IKabs(sj19array[ij19]-sj19array[iij19]) < IKFAST_SOLUTION_THRESH )
{
    j19valid[iij19]=false; _ij19[1] = iij19; break; 
}
}
j19 = j19array[ij19]; cj19 = cj19array[ij19]; sj19 = sj19array[ij19];
{
IkReal evalcond[6];
IkReal x918=IKcos(j19);
IkReal x919=IKsin(j19);
IkReal x920=((cj16)*(sj15));
IkReal x921=((IkReal(1.00000000000000))*(sj16));
IkReal x922=((IkReal(1.00000000000000))*(cj16));
IkReal x923=((cj17)*(sj18));
IkReal x924=((IkReal(1.00000000000000))*(sj15));
IkReal x925=((cj15)*(px));
IkReal x926=((sj17)*(sj18));
IkReal x927=((cj15)*(r00));
IkReal x928=((IkReal(0.533617455371189))*(cj17));
IkReal x929=((IkReal(0.0440000000000000))*(x918));
IkReal x930=((IkReal(0.0440000000000000))*(x919));
IkReal x931=((cj18)*(sj17)*(x918));
evalcond[0]=((((cj17)*(x930)))+(((cj18)*(sj17)*(x929)))+(((IkReal(-1.00000000000000))*(px)*(x924)))+(((IkReal(-0.135000000000000))*(x926)))+(((cj15)*(py))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj16)*(x925)))+(((IkReal(-1.00000000000000))*(sj18)*(x929)))+(((IkReal(-0.100000000000000))*(cj16)))+(((py)*(x920)))+(((IkReal(-1.00000000000000))*(pz)*(x921)))+(((IkReal(-0.135000000000000))*(cj18))));
evalcond[2]=((((cj16)*(x927)))+(((IkReal(-0.845725967038483))*(cj18)))+(((IkReal(-1.00000000000000))*(r02)*(x921)))+(((r01)*(x920)))+(((IkReal(-0.533617455371189))*(sj18)*(x918))));
evalcond[3]=((((IkReal(0.533617455371189))*(x931)))+(((x919)*(x928)))+(((cj15)*(r01)))+(((IkReal(-0.845725967038483))*(x926)))+(((IkReal(-1.00000000000000))*(r00)*(x924))));
evalcond[4]=((((IkReal(-0.135000000000000))*(x923)))+(((cj17)*(cj18)*(x929)))+(((IkReal(-1.00000000000000))*(pz)*(x922)))+(((IkReal(-1.00000000000000))*(x921)*(x925)))+(((IkReal(0.100000000000000))*(sj16)))+(((IkReal(-1.00000000000000))*(py)*(sj15)*(x921)))+(((IkReal(-1.00000000000000))*(sj17)*(x930))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r01)*(sj15)*(x921)))+(((IkReal(-1.00000000000000))*(r02)*(x922)))+(((IkReal(-0.845725967038483))*(x923)))+(((IkReal(-0.533617455371189))*(sj17)*(x919)))+(((IkReal(-1.00000000000000))*(x921)*(x927)))+(((cj18)*(x918)*(x928))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j15;
vinfos[0].indices[0] = _ij15[0];
vinfos[0].indices[1] = _ij15[1];
vinfos[0].maxsolutions = _nj15;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j16;
vinfos[1].indices[0] = _ij16[0];
vinfos[1].indices[1] = _ij16[1];
vinfos[1].maxsolutions = _nj16;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j17;
vinfos[2].indices[0] = _ij17[0];
vinfos[2].indices[1] = _ij17[1];
vinfos[2].maxsolutions = _nj17;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j18;
vinfos[3].indices[0] = _ij18[0];
vinfos[3].indices[1] = _ij18[1];
vinfos[3].maxsolutions = _nj18;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j19;
vinfos[4].indices[0] = _ij19[0];
vinfos[4].indices[1] = _ij19[1];
vinfos[4].maxsolutions = _nj19;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "212b72a9a206f8708ae129b24e0f8340"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
