/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:06:26.731611
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[2]);
x4=IKsin(j[1]);
x5=IKsin(j[4]);
x6=IKsin(j[0]);
x7=IKcos(j[3]);
x8=IKcos(j[4]);
x9=IKsin(j[3]);
x10=IKcos(j[5]);
x11=IKsin(j[5]);
x12=((IkReal(0.270000000000000))*(x4));
x13=((IkReal(0.295000000000000))*(x1));
x14=((x5)*(x7));
x15=((x2)*(x4));
x16=((x11)*(x9));
x17=((x7)*(x8));
x18=((x3)*(x4));
x19=((x1)*(x2));
x20=((x1)*(x3));
x21=((IkReal(0.295000000000000))*(x18));
x22=((IkReal(0.0900000000000000))*(x20));
x23=((x13)*(x2));
x24=((IkReal(0.0900000000000000))*(x15));
x25=((IkReal(-0.0740000000000000))+(((IkReal(-0.0800000000000000))*(x5)*(x9))));
x26=((IkReal(0.0800000000000000))*(x18));
x27=((IkReal(0.0800000000000000))*(x19));
x28=((x15)+(x20));
x29=((((IkReal(-1.00000000000000))*(x19)))+(x18));
x30=((((IkReal(-1.00000000000000))*(x26)))+(x27));
x31=((((IkReal(-0.0800000000000000))*(x20)))+(((IkReal(-0.0800000000000000))*(x15))));
x32=((((x10)*(x8)*(x9)))+(((x11)*(x7))));
x33=((((IkReal(-1.00000000000000))*(x20)))+(((IkReal(-1.00000000000000))*(x15))));
x34=((x30)*(x8));
x35=((x12)+(x24)+(x22)+(x23));
x36=((((x17)*(x28)))+(((x5)*(((((IkReal(-1.00000000000000))*(x18)))+(x19))))));
x37=((x10)*(x36));
eetrans[0]=((((IkReal(0.0750000000000000))*(x0)))+(((x0)*(((((IkReal(-1.00000000000000))*(x21)))+(((x14)*(x31)))+(x35)+(x34)))))+(((x25)*(x6)))+(((IkReal(0.0740000000000000))*(x6))));
eetrans[1]=((((x0)*(x25)))+(((x6)*(((((IkReal(-1.00000000000000))*(x35)))+(((IkReal(-1.00000000000000))*(x34)))+(((IkReal(-1.00000000000000))*(x14)*(x31)))+(x21)))))+(((IkReal(-0.0750000000000000))*(x6)))+(((IkReal(0.0740000000000000))*(x0))));
eetrans[2]=((IkReal(0.335000000000000))+(((x14)*(((((IkReal(-1.00000000000000))*(x27)))+(x26)))))+(((x31)*(x8)))+(((IkReal(0.270000000000000))*(x1)))+(((IkReal(-0.0900000000000000))*(x18)))+(((IkReal(-0.295000000000000))*(x15)))+(((IkReal(-1.00000000000000))*(x13)*(x3)))+(((IkReal(0.0900000000000000))*(x19))));
eerot[0]=((((x0)*(((((x16)*(x33)))+(x37)))))+(((x32)*(x6))));
eerot[1]=((((x6)*(((((IkReal(-1.00000000000000))*(x37)))+(((IkReal(-1.00000000000000))*(x16)*(x33)))))))+(((x0)*(x32))));
eerot[2]=((((x16)*(x29)))+(((x10)*(((((IkReal(-1.00000000000000))*(x17)*(x29)))+(((IkReal(-1.00000000000000))*(x28)*(x5))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j0,cj0,sj0,htj0,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij0[2], _nj0;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij0[0] = -1; _ij0[1] = -1; _nj0 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=((((IkReal(-1.00000000000000))*(r01)*(IKsin(j0))))+(((r00)*(IKcos(j0)))));
new_px=((IkReal(-0.0750000000000000))+(((IkReal(-1.00000000000000))*(py)*(IKsin(j0))))+(((px)*(IKcos(j0)))));
new_r01=((IkReal(-1.00000000000000))*(r02));
new_py=((IkReal(0.335000000000000))+(((IkReal(-1.00000000000000))*(pz))));
new_r02=((((r00)*(IKsin(j0))))+(((r01)*(IKcos(j0)))));
new_pz=((IkReal(-0.0740000000000000))+(((px)*(IKsin(j0))))+(((py)*(IKcos(j0)))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(0.215701000000000))+(((IkReal(0.720000000000000))*(py)))+(((IkReal(-0.590000000000000))*(px)))+(pp)+(((IkReal(0.148000000000000))*(pz))));
IkReal gconst1;
gconst1=((IkReal(-0.318600000000000))+(((IkReal(-1.18000000000000))*(py)))+(((IkReal(-0.360000000000000))*(px))));
IkReal gconst2;
gconst2=((IkReal(0.118501000000000))+(((IkReal(0.590000000000000))*(px)))+(((IkReal(0.360000000000000))*(py)))+(pp)+(((IkReal(0.148000000000000))*(pz))));
IkReal gconst3;
gconst3=((((IkReal(0.0740000000000000))*(r02)))+(((px)*(r00)))+(((IkReal(-0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.360000000000000))*(r01))));
IkReal gconst4;
gconst4=((((IkReal(-0.180000000000000))*(r00)))+(((IkReal(-0.590000000000000))*(r01))));
IkReal gconst5;
gconst5=((((IkReal(0.0740000000000000))*(r02)))+(((px)*(r00)))+(((IkReal(0.180000000000000))*(r01)))+(((IkReal(0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst6;
gconst6=((IkReal(0.215701000000000))+(((IkReal(0.720000000000000))*(py)))+(((IkReal(-0.590000000000000))*(px)))+(pp)+(((IkReal(0.148000000000000))*(pz))));
IkReal gconst7;
gconst7=((IkReal(-0.318600000000000))+(((IkReal(-1.18000000000000))*(py)))+(((IkReal(-0.360000000000000))*(px))));
IkReal gconst8;
gconst8=((IkReal(0.118501000000000))+(((IkReal(0.590000000000000))*(px)))+(((IkReal(0.360000000000000))*(py)))+(pp)+(((IkReal(0.148000000000000))*(pz))));
IkReal gconst9;
gconst9=((((IkReal(0.0740000000000000))*(r02)))+(((px)*(r00)))+(((IkReal(-0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.360000000000000))*(r01))));
IkReal gconst10;
gconst10=((((IkReal(-0.180000000000000))*(r00)))+(((IkReal(-0.590000000000000))*(r01))));
IkReal gconst11;
gconst11=((((IkReal(0.0740000000000000))*(r02)))+(((px)*(r00)))+(((IkReal(0.180000000000000))*(r01)))+(((IkReal(0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst12;
gconst12=((((IkReal(-1.44000000000000))*(px)))+(((IkReal(-1.18000000000000))*(py))));
IkReal gconst13;
gconst13=((((IkReal(2.36000000000000))*(px)))+(((IkReal(-0.720000000000000))*(py))));
IkReal gconst14;
gconst14=((((IkReal(1.18000000000000))*(py)))+(((IkReal(-0.720000000000000))*(px))));
IkReal gconst15;
gconst15=((((IkReal(-0.720000000000000))*(r00)))+(((IkReal(-0.590000000000000))*(r01))));
IkReal gconst16;
gconst16=((((IkReal(-0.360000000000000))*(r01)))+(((IkReal(1.18000000000000))*(r00))));
IkReal gconst17;
gconst17=((((IkReal(-0.360000000000000))*(r00)))+(((IkReal(0.590000000000000))*(r01))));
IkReal gconst18;
gconst18=((((IkReal(-1.44000000000000))*(px)))+(((IkReal(-1.18000000000000))*(py))));
IkReal gconst19;
gconst19=((((IkReal(2.36000000000000))*(px)))+(((IkReal(-0.720000000000000))*(py))));
IkReal gconst20;
gconst20=((((IkReal(1.18000000000000))*(py)))+(((IkReal(-0.720000000000000))*(px))));
IkReal gconst21;
gconst21=((((IkReal(-0.720000000000000))*(r00)))+(((IkReal(-0.590000000000000))*(r01))));
IkReal gconst22;
gconst22=((((IkReal(-0.360000000000000))*(r01)))+(((IkReal(1.18000000000000))*(r00))));
IkReal gconst23;
gconst23=((((IkReal(-0.360000000000000))*(r00)))+(((IkReal(0.590000000000000))*(r01))));
IkReal gconst24;
gconst24=((IkReal(0.215701000000000))+(((IkReal(0.590000000000000))*(px)))+(pp)+(((IkReal(0.148000000000000))*(pz)))+(((IkReal(-0.720000000000000))*(py))));
IkReal gconst25;
gconst25=((IkReal(-0.318600000000000))+(((IkReal(0.360000000000000))*(px)))+(((IkReal(1.18000000000000))*(py))));
IkReal gconst26;
gconst26=((IkReal(0.118501000000000))+(((IkReal(-0.590000000000000))*(px)))+(pp)+(((IkReal(0.148000000000000))*(pz)))+(((IkReal(-0.360000000000000))*(py))));
IkReal gconst27;
gconst27=((((IkReal(0.0740000000000000))*(r02)))+(((IkReal(-0.360000000000000))*(r01)))+(((px)*(r00)))+(((IkReal(0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst28;
gconst28=((((IkReal(0.180000000000000))*(r00)))+(((IkReal(0.590000000000000))*(r01))));
IkReal gconst29;
gconst29=((((IkReal(0.0740000000000000))*(r02)))+(((px)*(r00)))+(((IkReal(-0.180000000000000))*(r01)))+(((IkReal(-0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst30;
gconst30=((IkReal(0.215701000000000))+(((IkReal(0.590000000000000))*(px)))+(pp)+(((IkReal(0.148000000000000))*(pz)))+(((IkReal(-0.720000000000000))*(py))));
IkReal gconst31;
gconst31=((IkReal(-0.318600000000000))+(((IkReal(0.360000000000000))*(px)))+(((IkReal(1.18000000000000))*(py))));
IkReal gconst32;
gconst32=((IkReal(0.118501000000000))+(((IkReal(-0.590000000000000))*(px)))+(pp)+(((IkReal(0.148000000000000))*(pz)))+(((IkReal(-0.360000000000000))*(py))));
IkReal gconst33;
gconst33=((((IkReal(0.0740000000000000))*(r02)))+(((IkReal(-0.360000000000000))*(r01)))+(((px)*(r00)))+(((IkReal(0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst34;
gconst34=((((IkReal(0.180000000000000))*(r00)))+(((IkReal(0.590000000000000))*(r01))));
IkReal gconst35;
gconst35=((((IkReal(0.0740000000000000))*(r02)))+(((px)*(r00)))+(((IkReal(-0.180000000000000))*(r01)))+(((IkReal(-0.295000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal x38=((IkReal(1.00000000000000))*(gconst26));
IkReal x39=((gconst30)*(gconst34));
IkReal x40=((gconst31)*(gconst33));
IkReal x41=((IkReal(1.00000000000000))*(gconst17));
IkReal x42=((gconst2)*(gconst6));
IkReal x43=((gconst11)*(gconst3));
IkReal x44=((gconst25)*(gconst29));
IkReal x45=((gconst14)*(gconst28));
IkReal x46=((IkReal(1.00000000000000))*(gconst14));
IkReal x47=((gconst21)*(gconst32));
IkReal x48=((gconst10)*(gconst6));
IkReal x49=((gconst21)*(gconst5));
IkReal x50=((gconst0)*(gconst8));
IkReal x51=((gconst13)*(gconst5));
IkReal x52=((IkReal(1.00000000000000))*(gconst9));
IkReal x53=((gconst30)*(gconst35));
IkReal x54=((gconst26)*(gconst27));
IkReal x55=((gconst22)*(gconst30));
IkReal x56=((IkReal(1.00000000000000))*(gconst12));
IkReal x57=((gconst2)*(gconst3));
IkReal x58=((gconst1)*(gconst5));
IkReal x59=((gconst19)*(gconst33));
IkReal x60=((gconst26)*(gconst28));
IkReal x61=((gconst11)*(gconst6));
IkReal x62=((gconst1)*(gconst7));
IkReal x63=((gconst18)*(gconst2));
IkReal x64=((gconst18)*(gconst34));
IkReal x65=((gconst24)*(gconst29));
IkReal x66=((IkReal(1.00000000000000))*(gconst23));
IkReal x67=((gconst20)*(gconst33));
IkReal x68=((gconst32)*(gconst33));
IkReal x69=((gconst0)*(gconst5));
IkReal x70=((IkReal(1.00000000000000))*(gconst18));
IkReal x71=((gconst21)*(gconst31));
IkReal x72=((gconst5)*(gconst9));
IkReal x73=((gconst4)*(gconst9));
IkReal x74=((gconst13)*(gconst29));
IkReal x75=((gconst14)*(gconst6));
IkReal x76=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x77=((gconst27)*(x68));
IkReal x78=((gconst2)*(gconst4)*(gconst7));
op[0]=((((IkReal(-1.00000000000000))*(x40)*(x44)))+(((x39)*(x44)))+(((IkReal(-1.00000000000000))*(x38)*(x77)))+(((IkReal(-1.00000000000000))*(gconst28)*(x38)*(x39)))+(((x53)*(x54)))+(((x40)*(x60)))+(((IkReal(-1.00000000000000))*(x53)*(x65)))+(((x65)*(x68))));
op[1]=((((gconst23)*(gconst30)*(x54)))+(((x59)*(x60)))+(((IkReal(-1.00000000000000))*(gconst35)*(x65)*(x70)))+(((x44)*(x64)))+(((IkReal(-1.00000000000000))*(gconst25)*(x40)*(x41)))+(((IkReal(-1.00000000000000))*(gconst29)*(x53)*(x56)))+(((gconst12)*(gconst29)*(x68)))+(((x40)*(x45)))+(((gconst16)*(gconst26)*(x40)))+(((IkReal(-1.00000000000000))*(x44)*(x71)))+(((x44)*(x55)))+(((IkReal(-1.00000000000000))*(x44)*(x59)))+(((IkReal(-1.00000000000000))*(gconst28)*(x38)*(x64)))+(((gconst15)*(gconst26)*(x53)))+(((gconst17)*(gconst25)*(x39)))+(((IkReal(-1.00000000000000))*(gconst28)*(x38)*(x55)))+(((IkReal(-1.00000000000000))*(x39)*(x45)))+(((x60)*(x71)))+(((gconst14)*(gconst27)*(x53)))+(((x39)*(x74)))+(((IkReal(-1.00000000000000))*(x46)*(x77)))+(((gconst17)*(gconst24)*(x68)))+(((IkReal(-1.00000000000000))*(gconst24)*(x41)*(x53)))+(((gconst18)*(gconst35)*(x54)))+(((IkReal(-1.00000000000000))*(x40)*(x74)))+(((IkReal(-1.00000000000000))*(gconst15)*(x38)*(x68)))+(((x47)*(x65)))+(((IkReal(-1.00000000000000))*(gconst27)*(x38)*(x47)))+(((IkReal(-1.00000000000000))*(gconst16)*(x38)*(x39)))+(((IkReal(-1.00000000000000))*(gconst27)*(x38)*(x67)))+(((IkReal(-1.00000000000000))*(gconst30)*(x65)*(x66)))+(((x65)*(x67))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((IkReal(-1.00000000000000))*(gconst6)*(x66)*(x69)))+(((IkReal(-1.00000000000000))*(gconst0)*(x41)*(x61)))+(((gconst10)*(gconst18)*(x58)))+(((gconst22)*(gconst6)*(x58)))+(((IkReal(-1.00000000000000))*(gconst9)*(x41)*(x62)))+(((x43)*(x75)))+(((IkReal(-1.00000000000000))*(gconst7)*(x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst5)*(x56)*(x61)))+(((IkReal(-1.00000000000000))*(gconst4)*(x46)*(x48)))+(((IkReal(-1.00000000000000))*(x49)*(x62)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x42)))+(((IkReal(-1.00000000000000))*(x63)*(x76)))+(((gconst12)*(gconst8)*(x72)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x57)))+(((gconst19)*(gconst2)*(x73)))+(((gconst11)*(gconst15)*(x42)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x46)))+(((IkReal(-1.00000000000000))*(gconst11)*(x69)*(x70)))+(((x49)*(x50)))+(((x48)*(x51)))+(((gconst14)*(gconst7)*(x73)))+(((x43)*(x63)))+(((gconst21)*(x78)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x52)))+(((gconst20)*(gconst9)*(x69)))+(((IkReal(-1.00000000000000))*(gconst19)*(x52)*(x58)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((gconst23)*(gconst3)*(x42)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x42)))+(((gconst1)*(gconst17)*(x48)))+(((IkReal(-1.00000000000000))*(gconst20)*(x52)*(x57)))+(((gconst17)*(gconst9)*(x50))));
op[8]=((((IkReal(-1.00000000000000))*(gconst8)*(x52)*(x57)))+(((IkReal(-1.00000000000000))*(gconst7)*(x52)*(x58)))+(((IkReal(-1.00000000000000))*(x61)*(x69)))+(((IkReal(-1.00000000000000))*(x42)*(x76)))+(((x42)*(x43)))+(((x48)*(x58)))+(((gconst2)*(gconst7)*(x73)))+(((x50)*(x72))));
polyroots8(op,zeror,numroots);
IkReal j1array[8], cj1array[8], sj1array[8], tempj1array[1];
int numsolutions = 0;
for(int ij1 = 0; ij1 < numroots; ++ij1)
{
IkReal htj1 = zeror[ij1];
tempj1array[0]=((IkReal(2.00000000000000))*(atan(htj1)));
for(int kj1 = 0; kj1 < 1; ++kj1)
{
j1array[numsolutions] = tempj1array[kj1];
if( j1array[numsolutions] > IKPI )
{
    j1array[numsolutions]-=IK2PI;
}
else if( j1array[numsolutions] < -IKPI )
{
    j1array[numsolutions]+=IK2PI;
}
sj1array[numsolutions] = IKsin(j1array[numsolutions]);
cj1array[numsolutions] = IKcos(j1array[numsolutions]);
numsolutions++;
}
}
bool j1valid[8]={true,true,true,true,true,true,true,true};
_nj1 = 8;
for(int ij1 = 0; ij1 < numsolutions; ++ij1)
    {
if( !j1valid[ij1] )
{
    continue;
}
    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
htj1 = IKtan(j1/2);

_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < numsolutions; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x79=(sj1)*(sj1);
IkReal x80=(cj1)*(cj1);
IkReal x81=((px)*(r01));
IkReal x82=((py)*(r00));
IkReal x83=((IkReal(0.190250000000000))*(x79));
IkReal x84=((IkReal(0.190250000000000))*(x80));
gconst52=IKsign(((((IkReal(-1.00000000000000))*(x82)*(x83)))+(((IkReal(-1.00000000000000))*(x82)*(x84)))+(((x81)*(x84)))+(((x81)*(x83)))+(((IkReal(-0.0513675000000000))*(r01)*(sj1)))+(((IkReal(-0.0513675000000000))*(cj1)*(r00)))));
IkReal x85=(cj1)*(cj1);
IkReal x86=(sj1)*(sj1);
IkReal x87=((px)*(r01));
IkReal x88=((py)*(r00));
IkReal x89=((IkReal(3.70370370370370))*(x85));
IkReal x90=((IkReal(3.70370370370370))*(x86));
dummyeval[0]=((((x87)*(x90)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(x88)*(x89)))+(((IkReal(-1.00000000000000))*(x88)*(x90)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)))+(((x87)*(x89))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst52)*(((((IkReal(-0.0133200000000000))*(cj1)*(py)*(r02)))+(((IkReal(-0.00359640000000000))*(r02)))+(((IkReal(0.590000000000000))*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(px)*(pz)*(r02)*(sj1)))+(((IkReal(0.159300000000000))*(px)*(r01)*((cj1)*(cj1))))+(((IkReal(0.180000000000000))*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.0486000000000000))*(py)*(r01)))+(((IkReal(0.590000000000000))*(r01)*(sj1)*((py)*(py))))+(((IkReal(-0.295000000000000))*(pp)*(r01)*(sj1)))+(((IkReal(0.0133200000000000))*(cj1)*(pz)*(r01)))+(((IkReal(0.590000000000000))*(cj1)*(r00)*((px)*(px))))+(((IkReal(-0.0133200000000000))*(pz)*(r00)*(sj1)))+(((IkReal(-0.0436600000000000))*(cj1)*(pz)*(r00)))+(((IkReal(-0.295000000000000))*(cj1)*(pp)*(r00)))+(((IkReal(-0.159300000000000))*(py)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.0486000000000000))*(pz)*(r02)))+(((IkReal(-0.00191709000000000))*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(py)*(r00)*(sj1)))+(((IkReal(-0.159300000000000))*(py)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0486000000000000))*(px)*(r00)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-0.0492947950000000))*(r01)*(sj1)))+(((IkReal(-0.0492947950000000))*(cj1)*(r00)))+(((IkReal(0.00191709000000000))*(cj1)*(r01)))+(((IkReal(0.0133200000000000))*(px)*(r02)*(sj1)))+(((IkReal(-0.0436600000000000))*(pz)*(r01)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r01)*((py)*(py))))+(((IkReal(0.180000000000000))*(px)*(py)*(r01)*(sj1)))+(((IkReal(0.159300000000000))*(px)*(r01)*((sj1)*(sj1))))+(((IkReal(-0.0900000000000000))*(pp)*(r00)*(sj1)))+(((IkReal(0.0436600000000000))*(cj1)*(px)*(r02)))+(((IkReal(0.0436600000000000))*(py)*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(px)*(py)*(r01)))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r01)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r00))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((IkReal(0.0436600000000000))*(px)*(r02)*(sj1)))+(((IkReal(-0.590000000000000))*(cj1)*(px)*(py)*(r00)))+(((IkReal(-0.0117882000000000))*(r02)))+(((IkReal(0.0486000000000000))*(py)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.180000000000000))*(px)*(py)*(r00)*(sj1)))+(((IkReal(0.0900000000000000))*(pp)*(r01)*(sj1)))+(((IkReal(0.0133200000000000))*(pz)*(r01)*(sj1)))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r02)*(sj1)))+(((IkReal(0.0150390900000000))*(cj1)*(r00)))+(((IkReal(-0.0486000000000000))*(px)*(r01)*((cj1)*(cj1))))+(((IkReal(0.295000000000000))*(cj1)*(pp)*(r01)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.0150390900000000))*(r01)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.0133200000000000))*(py)*(r02)*(sj1)))+(((IkReal(0.00628379500000000))*(cj1)*(r01)))+(((IkReal(0.0133200000000000))*(cj1)*(pz)*(r00)))+(((IkReal(-0.0133200000000000))*(cj1)*(px)*(r02)))+(((IkReal(0.0486000000000000))*(py)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.590000000000000))*(cj1)*(r01)*((py)*(py))))+(((IkReal(-0.180000000000000))*(cj1)*(r00)*((px)*(px))))+(((IkReal(-0.159300000000000))*(pz)*(r02)))+(((IkReal(-0.295000000000000))*(pp)*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.0486000000000000))*(px)*(r01)*((sj1)*(sj1))))+(((IkReal(-0.159300000000000))*(py)*(r01)))+(((IkReal(-0.0436600000000000))*(cj1)*(py)*(r02)))+(((IkReal(-0.0436600000000000))*(pz)*(r00)*(sj1)))+(((IkReal(0.0436600000000000))*(cj1)*(pz)*(r01)))+(((IkReal(-0.159300000000000))*(px)*(r00)))+(((IkReal(-0.00628379500000000))*(r00)*(sj1)))+(((IkReal(-0.590000000000000))*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-0.180000000000000))*(r01)*(sj1)*((py)*(py))))+(((IkReal(0.590000000000000))*(r00)*(sj1)*((px)*(px))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r00)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst52)*(((((IkReal(-0.0133200000000000))*(cj1)*(py)*(r02)))+(((IkReal(-0.00359640000000000))*(r02)))+(((IkReal(0.590000000000000))*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.590000000000000))*(py)*(pz)*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(px)*(pz)*(r02)*(sj1)))+(((IkReal(0.159300000000000))*(px)*(r01)*((cj1)*(cj1))))+(((IkReal(0.180000000000000))*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.0486000000000000))*(py)*(r01)))+(((IkReal(0.590000000000000))*(r01)*(sj1)*((py)*(py))))+(((IkReal(-0.295000000000000))*(pp)*(r01)*(sj1)))+(((IkReal(0.0133200000000000))*(cj1)*(pz)*(r01)))+(((IkReal(0.590000000000000))*(cj1)*(r00)*((px)*(px))))+(((IkReal(-0.0133200000000000))*(pz)*(r00)*(sj1)))+(((IkReal(-0.0436600000000000))*(cj1)*(pz)*(r00)))+(((IkReal(-0.295000000000000))*(cj1)*(pp)*(r00)))+(((IkReal(-0.159300000000000))*(py)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.0486000000000000))*(pz)*(r02)))+(((IkReal(-0.00191709000000000))*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(py)*(r00)*(sj1)))+(((IkReal(-0.159300000000000))*(py)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0486000000000000))*(px)*(r00)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-0.0492947950000000))*(r01)*(sj1)))+(((IkReal(-0.0492947950000000))*(cj1)*(r00)))+(((IkReal(0.00191709000000000))*(cj1)*(r01)))+(((IkReal(0.0133200000000000))*(px)*(r02)*(sj1)))+(((IkReal(-0.0436600000000000))*(pz)*(r01)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r01)*((py)*(py))))+(((IkReal(0.180000000000000))*(px)*(py)*(r01)*(sj1)))+(((IkReal(0.159300000000000))*(px)*(r01)*((sj1)*(sj1))))+(((IkReal(-0.0900000000000000))*(pp)*(r00)*(sj1)))+(((IkReal(0.0436600000000000))*(cj1)*(px)*(r02)))+(((IkReal(0.0436600000000000))*(py)*(r02)*(sj1)))+(((IkReal(0.590000000000000))*(cj1)*(px)*(py)*(r01)))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r01)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r00)))))), ((gconst52)*(((((IkReal(0.0436600000000000))*(px)*(r02)*(sj1)))+(((IkReal(-0.590000000000000))*(cj1)*(px)*(py)*(r00)))+(((IkReal(-0.0117882000000000))*(r02)))+(((IkReal(0.0486000000000000))*(py)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.180000000000000))*(px)*(py)*(r00)*(sj1)))+(((IkReal(0.0900000000000000))*(pp)*(r01)*(sj1)))+(((IkReal(0.0133200000000000))*(pz)*(r01)*(sj1)))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r02)*(sj1)))+(((IkReal(0.0150390900000000))*(cj1)*(r00)))+(((IkReal(-0.0486000000000000))*(px)*(r01)*((cj1)*(cj1))))+(((IkReal(0.295000000000000))*(cj1)*(pp)*(r01)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(pz)*(r02)))+(((IkReal(0.0150390900000000))*(r01)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.0133200000000000))*(py)*(r02)*(sj1)))+(((IkReal(0.00628379500000000))*(cj1)*(r01)))+(((IkReal(0.0133200000000000))*(cj1)*(pz)*(r00)))+(((IkReal(-0.0133200000000000))*(cj1)*(px)*(r02)))+(((IkReal(0.0486000000000000))*(py)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.590000000000000))*(cj1)*(r01)*((py)*(py))))+(((IkReal(-0.180000000000000))*(cj1)*(r00)*((px)*(px))))+(((IkReal(-0.159300000000000))*(pz)*(r02)))+(((IkReal(-0.295000000000000))*(pp)*(r00)*(sj1)))+(((IkReal(0.590000000000000))*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.0486000000000000))*(px)*(r01)*((sj1)*(sj1))))+(((IkReal(-0.159300000000000))*(py)*(r01)))+(((IkReal(-0.0436600000000000))*(cj1)*(py)*(r02)))+(((IkReal(-0.0436600000000000))*(pz)*(r00)*(sj1)))+(((IkReal(0.0436600000000000))*(cj1)*(pz)*(r01)))+(((IkReal(-0.159300000000000))*(px)*(r00)))+(((IkReal(-0.00628379500000000))*(r00)*(sj1)))+(((IkReal(-0.590000000000000))*(cj1)*(py)*(pz)*(r02)))+(((IkReal(-0.180000000000000))*(r01)*(sj1)*((py)*(py))))+(((IkReal(0.590000000000000))*(r00)*(sj1)*((px)*(px))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r00)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r01)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
IkReal x91=IKcos(j2);
IkReal x92=IKsin(j2);
IkReal x93=((cj1)*(py));
IkReal x94=((px)*(sj1));
IkReal x95=((r00)*(sj1));
IkReal x96=((cj1)*(r01));
IkReal x97=((cj1)*(r00));
IkReal x98=((cj1)*(px));
IkReal x99=((IkReal(0.180000000000000))*(x91));
IkReal x100=((IkReal(0.590000000000000))*(x92));
IkReal x101=((IkReal(0.295000000000000))*(x92));
IkReal x102=((IkReal(0.0900000000000000))*(x91));
IkReal x103=((sj1)*(x92));
IkReal x104=((IkReal(0.295000000000000))*(x91));
IkReal x105=((IkReal(0.590000000000000))*(x91));
evalcond[0]=((IkReal(0.167101000000000))+(((IkReal(0.0486000000000000))*(x91)))+(((IkReal(-0.180000000000000))*(x92)*(x98)))+(((x100)*(x94)))+(((IkReal(-1.00000000000000))*(x94)*(x99)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x105)))+(((IkReal(-1.00000000000000))*(x105)*(x98)))+(((IkReal(-0.159300000000000))*(x92)))+(pp)+(((x93)*(x99)))+(((IkReal(-1.00000000000000))*(x100)*(x93)))+(((IkReal(0.540000000000000))*(x93)))+(((IkReal(0.148000000000000))*(pz)))+(((IkReal(-0.540000000000000))*(x94)))+(((IkReal(-0.180000000000000))*(py)*(x103))));
evalcond[1]=((((IkReal(0.0740000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x102)*(x95)))+(((px)*(r00)))+(((x102)*(x96)))+(((x101)*(x95)))+(((IkReal(-0.0900000000000000))*(x92)*(x97)))+(((IkReal(-0.0900000000000000))*(r01)*(x103)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x104)))+(((IkReal(-1.00000000000000))*(x101)*(x96)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.270000000000000))*(x96)))+(((IkReal(-0.270000000000000))*(x95)))+(((IkReal(-1.00000000000000))*(x104)*(x97))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x106=((IkReal(12.5000000000000))*(cj2));
IkReal x107=((IkReal(12.5000000000000))*(sj2));
cj4array[0]=((IkReal(-3.68750000000000))+(((py)*(sj1)*(x106)))+(((IkReal(3.37500000000000))*(sj2)))+(((cj1)*(px)*(x106)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x107)))+(((cj1)*(py)*(x107))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x108=((sj1)*(sj2));
IkReal x109=((IkReal(1.00000000000000))*(cj1));
IkReal x110=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x109)))+(((IkReal(-1.00000000000000))*(r01)*(x110)))+(((r00)*(x108)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x109))));
evalcond[3]=((IkReal(0.375000000000000))+(((px)*(x108)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x109)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x109)))+(((IkReal(-1.00000000000000))*(py)*(x110))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((py)*(x108)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x110)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x109)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=r02;
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=r02;
IkReal gconst64;
gconst64=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst65;
gconst65=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst66;
gconst66=r02;
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
gconst68=r02;
IkReal gconst69;
gconst69=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst70;
gconst70=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst71;
gconst71=IkReal(-2.00000000000000);
IkReal gconst72;
gconst72=IkReal(2.00000000000000);
IkReal gconst73;
gconst73=IkReal(4.00000000000000);
IkReal gconst74;
gconst74=IkReal(-2.00000000000000);
IkReal gconst75;
gconst75=IkReal(2.00000000000000);
IkReal gconst76;
gconst76=IkReal(4.00000000000000);
IkReal gconst77;
gconst77=r02;
IkReal gconst78;
gconst78=IkReal(2.00000000000000);
IkReal gconst79;
gconst79=r02;
IkReal gconst80;
gconst80=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst81;
gconst81=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst82;
gconst82=r02;
IkReal gconst83;
gconst83=IkReal(2.00000000000000);
IkReal gconst84;
gconst84=r02;
IkReal gconst85;
gconst85=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst86;
gconst86=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal x111=((gconst84)*(gconst85));
IkReal x112=((gconst77)*(gconst81));
IkReal x113=((gconst79)*(gconst80));
IkReal x114=((gconst82)*(gconst86));
dummyeval[0]=((((x111)*(x112)))+(((x113)*(x114)))+(((IkReal(-1.00000000000000))*(x111)*(x113)))+(((IkReal(-1.00000000000000))*(gconst78)*(gconst81)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(x112)*(x114))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x115=((gconst79)*(gconst80));
IkReal x116=((gconst70)*(gconst74));
IkReal x117=((IkReal(1.00000000000000))*(gconst69));
IkReal x118=((gconst63)*(gconst64));
IkReal x119=((gconst66)*(gconst86));
IkReal x120=((gconst63)*(gconst80));
IkReal x121=((gconst82)*(gconst86));
IkReal x122=((IkReal(1.00000000000000))*(gconst61));
IkReal x123=((gconst84)*(gconst85));
IkReal x124=((gconst65)*(gconst77));
IkReal x125=((gconst62)*(gconst76));
IkReal x126=((gconst81)*(gconst82));
IkReal x127=((gconst68)*(gconst85));
IkReal x128=((gconst65)*(gconst74));
IkReal x129=((gconst61)*(gconst81));
IkReal x130=((gconst74)*(gconst86));
IkReal x131=((gconst65)*(gconst82));
IkReal x132=((gconst64)*(gconst72));
IkReal x133=((IkReal(1.00000000000000))*(gconst70));
IkReal x134=((gconst67)*(gconst85));
IkReal x135=((gconst72)*(gconst73));
IkReal x136=((IkReal(1.00000000000000))*(gconst81));
IkReal x137=((IkReal(1.00000000000000))*(gconst85));
IkReal x138=((gconst71)*(gconst81));
IkReal x139=((gconst75)*(gconst85));
IkReal x140=((gconst70)*(gconst82));
IkReal x141=((gconst72)*(gconst80));
IkReal x142=((gconst76)*(gconst78));
IkReal x143=((gconst66)*(gconst70));
IkReal x144=((gconst65)*(gconst66));
IkReal x145=((gconst64)*(gconst79));
IkReal x146=((gconst69)*(gconst84));
IkReal x147=((gconst65)*(gconst71));
IkReal x148=((gconst78)*(gconst81));
IkReal x149=((gconst73)*(gconst79));
IkReal x150=((gconst66)*(gconst81));
IkReal x151=((gconst69)*(gconst75));
IkReal x152=((gconst61)*(gconst65));
IkReal x153=((gconst63)*(gconst73));
IkReal x154=((gconst83)*(gconst85));
IkReal x155=((IkReal(1.00000000000000))*(gconst76));
IkReal x156=((gconst77)*(gconst81));
IkReal x157=((gconst68)*(gconst69));
IkReal x158=((gconst62)*(gconst83));
IkReal x159=((gconst74)*(gconst81));
IkReal x160=((gconst62)*(gconst67));
IkReal x161=((gconst69)*(gconst83));
IkReal x162=((gconst65)*(gconst67)*(gconst78));
IkReal x163=((gconst68)*(x147));
IkReal x164=((gconst72)*(x130));
IkReal x165=((IkReal(1.00000000000000))*(x145));
IkReal x166=((gconst67)*(gconst69)*(gconst73));
IkReal x167=((gconst83)*(x149));
IkReal x168=((gconst65)*(gconst78)*(gconst83));
IkReal x169=((x153)*(x155));
op[0]=((((x123)*(x156)))+(((IkReal(-1.00000000000000))*(x115)*(x123)))+(((IkReal(-1.00000000000000))*(gconst78)*(x136)*(x154)))+(((x115)*(x121)))+(((IkReal(-1.00000000000000))*(gconst77)*(x121)*(x136))));
op[1]=((((x139)*(x156)))+(((x149)*(x154)))+(((x126)*(x142)))+(((IkReal(-1.00000000000000))*(gconst77)*(x130)*(x136)))+(((x123)*(x138)))+(((x121)*(x141)))+(((IkReal(-1.00000000000000))*(gconst75)*(x115)*(x137)))+(((IkReal(-1.00000000000000))*(gconst71)*(x121)*(x136)))+(((IkReal(-1.00000000000000))*(x123)*(x141)))+(((x115)*(x130))));
op[2]=((((x138)*(x139)))+(((IkReal(-1.00000000000000))*(gconst78)*(x134)*(x136)))+(((IkReal(-1.00000000000000))*(gconst71)*(x130)*(x136)))+(((IkReal(-1.00000000000000))*(x115)*(x127)))+(((x142)*(x159)))+(((IkReal(-1.00000000000000))*(gconst77)*(x126)*(x133)))+(((IkReal(-1.00000000000000))*(gconst75)*(x137)*(x141)))+(((IkReal(-1.00000000000000))*(gconst82)*(x149)*(x155)))+(((x135)*(x154)))+(((IkReal(-1.00000000000000))*(x123)*(x165)))+(((IkReal(-1.00000000000000))*(gconst83)*(x117)*(x148)))+(((IkReal(-1.00000000000000))*(gconst77)*(x119)*(x136)))+(((x115)*(x140)))+(((x120)*(x121)))+(((x121)*(x145)))+(((x123)*(x124)))+(((x123)*(x129)))+(((x127)*(x156)))+(((IkReal(-1.00000000000000))*(gconst81)*(x121)*(x122)))+(((x146)*(x156)))+(((IkReal(-1.00000000000000))*(x121)*(x124)))+(((IkReal(-1.00000000000000))*(gconst84)*(x115)*(x117)))+(((IkReal(-1.00000000000000))*(x120)*(x123)))+(((IkReal(-1.00000000000000))*(gconst62)*(x136)*(x154)))+(((x115)*(x119)))+(((IkReal(-1.00000000000000))*(x137)*(x168)))+(((x130)*(x141))));
op[3]=((((x127)*(x138)))+(((x153)*(x154)))+(((IkReal(-1.00000000000000))*(gconst71)*(x119)*(x136)))+(((x140)*(x141)))+(((x121)*(x132)))+(((x131)*(x142)))+(((IkReal(-1.00000000000000))*(gconst81)*(x122)*(x130)))+(((IkReal(-1.00000000000000))*(gconst71)*(x126)*(x133)))+(((x142)*(x150)))+(((x123)*(x147)))+(((x129)*(x139)))+(((IkReal(-1.00000000000000))*(x121)*(x147)))+(((x149)*(x161)))+(((IkReal(-1.00000000000000))*(x123)*(x132)))+(((IkReal(-1.00000000000000))*(x124)*(x130)))+(((IkReal(-1.00000000000000))*(gconst75)*(x137)*(x145)))+(((x138)*(x146)))+(((IkReal(-1.00000000000000))*(gconst82)*(x135)*(x155)))+(((x151)*(x156)))+(((x119)*(x141)))+(((IkReal(-1.00000000000000))*(x127)*(x141)))+(((IkReal(-1.00000000000000))*(gconst75)*(x115)*(x117)))+(((x125)*(x126)))+(((x124)*(x139)))+(((x120)*(x130)))+(((x134)*(x149)))+(((IkReal(-1.00000000000000))*(gconst77)*(x116)*(x136)))+(((IkReal(-1.00000000000000))*(gconst84)*(x117)*(x141)))+(((x115)*(x116)))+(((IkReal(-1.00000000000000))*(gconst74)*(x149)*(x155)))+(((IkReal(-1.00000000000000))*(gconst75)*(x120)*(x137)))+(((x130)*(x145))));
op[4]=((((IkReal(-1.00000000000000))*(gconst77)*(x133)*(x150)))+(((x140)*(x145)))+(((x124)*(x146)))+(((IkReal(-1.00000000000000))*(gconst65)*(x137)*(x158)))+(((x123)*(x152)))+(((x120)*(x140)))+(((IkReal(-1.00000000000000))*(gconst66)*(x149)*(x155)))+(((IkReal(-1.00000000000000))*(gconst67)*(x117)*(x148)))+(((IkReal(-1.00000000000000))*(x127)*(x165)))+(((IkReal(-1.00000000000000))*(gconst70)*(x122)*(x126)))+(((IkReal(-1.00000000000000))*(x118)*(x123)))+(((IkReal(-1.00000000000000))*(gconst82)*(x124)*(x133)))+(((x128)*(x142)))+(((IkReal(-1.00000000000000))*(gconst75)*(x117)*(x141)))+(((IkReal(-1.00000000000000))*(gconst81)*(x117)*(x158)))+(((x119)*(x145)))+(((IkReal(-1.00000000000000))*(gconst71)*(x116)*(x136)))+(((IkReal(-1.00000000000000))*(gconst81)*(x119)*(x122)))+(((x138)*(x151)))+(((IkReal(-1.00000000000000))*(gconst82)*(x169)))+(((x115)*(x143)))+(((x129)*(x146)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst78)*(x134)))+(((x116)*(x141)))+(((x135)*(x161)))+(((x130)*(x132)))+(((x134)*(x135)))+(((IkReal(-1.00000000000000))*(gconst75)*(x132)*(x137)))+(((x124)*(x127)))+(((IkReal(-1.00000000000000))*(gconst74)*(x135)*(x155)))+(((IkReal(-1.00000000000000))*(gconst84)*(x117)*(x120)))+(((x156)*(x157)))+(((IkReal(-1.00000000000000))*(x117)*(x168)))+(((IkReal(-1.00000000000000))*(x120)*(x127)))+(((IkReal(-1.00000000000000))*(gconst65)*(x121)*(x122)))+(((IkReal(-1.00000000000000))*(gconst84)*(x117)*(x145)))+(((x119)*(x120)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst86)*(x128)))+(((x118)*(x121)))+(((IkReal(-1.00000000000000))*(gconst62)*(x134)*(x136)))+(((IkReal(-1.00000000000000))*(gconst68)*(x115)*(x117)))+(((x139)*(x147)))+(((x127)*(x129)))+(((IkReal(-1.00000000000000))*(x119)*(x124)))+(((x125)*(x159))));
op[5]=((((x139)*(x152)))+(((IkReal(-1.00000000000000))*(gconst66)*(x133)*(x138)))+(((IkReal(-1.00000000000000))*(gconst81)*(x116)*(x122)))+(((x142)*(x144)))+(((gconst67)*(gconst69)*(x149)))+(((IkReal(-1.00000000000000))*(x127)*(x132)))+(((IkReal(-1.00000000000000))*(gconst68)*(x117)*(x141)))+(((IkReal(-1.00000000000000))*(gconst75)*(x117)*(x145)))+(((x138)*(x157)))+(((IkReal(-1.00000000000000))*(x116)*(x124)))+(((x141)*(x143)))+(((x127)*(x147)))+(((IkReal(-1.00000000000000))*(gconst86)*(x122)*(x128)))+(((IkReal(-1.00000000000000))*(gconst75)*(x118)*(x137)))+(((IkReal(-1.00000000000000))*(gconst71)*(x131)*(x133)))+(((x116)*(x145)))+(((x125)*(x131)))+(((x129)*(x151)))+(((x146)*(x147)))+(((x116)*(x120)))+(((IkReal(-1.00000000000000))*(gconst84)*(x117)*(x132)))+(((x134)*(x153)))+(((x132)*(x140)))+(((IkReal(-1.00000000000000))*(x119)*(x147)))+(((IkReal(-1.00000000000000))*(gconst74)*(x169)))+(((IkReal(-1.00000000000000))*(gconst75)*(x117)*(x120)))+(((x118)*(x130)))+(((x119)*(x132)))+(((x153)*(x161)))+(((x124)*(x151)))+(((IkReal(-1.00000000000000))*(gconst66)*(x135)*(x155)))+(((x125)*(x150))));
op[6]=((((IkReal(-1.00000000000000))*(gconst62)*(gconst65)*(x134)))+(((IkReal(-1.00000000000000))*(gconst70)*(x122)*(x131)))+(((x120)*(x143)))+(((IkReal(-1.00000000000000))*(gconst81)*(x117)*(x160)))+(((gconst67)*(gconst69)*(x135)))+(((IkReal(-1.00000000000000))*(x118)*(x127)))+(((IkReal(-1.00000000000000))*(gconst65)*(x117)*(x158)))+(((IkReal(-1.00000000000000))*(gconst65)*(x119)*(x122)))+(((IkReal(-1.00000000000000))*(gconst68)*(x117)*(x145)))+(((IkReal(-1.00000000000000))*(gconst66)*(x169)))+(((x118)*(x140)))+(((IkReal(-1.00000000000000))*(gconst68)*(x117)*(x120)))+(((x125)*(x128)))+(((IkReal(-1.00000000000000))*(gconst66)*(x124)*(x133)))+(((x129)*(x157)))+(((IkReal(-1.00000000000000))*(gconst84)*(x117)*(x118)))+(((x127)*(x152)))+(((x146)*(x152)))+(((IkReal(-1.00000000000000))*(x116)*(x147)))+(((x116)*(x132)))+(((IkReal(-1.00000000000000))*(gconst81)*(x122)*(x143)))+(((x147)*(x151)))+(((IkReal(-1.00000000000000))*(x117)*(x162)))+(((x118)*(x119)))+(((IkReal(-1.00000000000000))*(gconst75)*(x117)*(x132)))+(((x143)*(x145)))+(((x124)*(x157))));
op[7]=((((IkReal(-1.00000000000000))*(gconst71)*(x133)*(x144)))+(((gconst67)*(gconst69)*(x153)))+(((IkReal(-1.00000000000000))*(gconst68)*(x117)*(x132)))+(((x151)*(x152)))+(((IkReal(-1.00000000000000))*(gconst65)*(x116)*(x122)))+(((IkReal(-1.00000000000000))*(gconst75)*(x117)*(x118)))+(((x147)*(x157)))+(((x116)*(x118)))+(((x132)*(x143)))+(((x125)*(x144))));
op[8]=((((x152)*(x157)))+(((x118)*(x143)))+(((IkReal(-1.00000000000000))*(gconst68)*(x117)*(x118)))+(((IkReal(-1.00000000000000))*(gconst65)*(x122)*(x143)))+(((IkReal(-1.00000000000000))*(gconst65)*(x117)*(x160))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst103;
gconst103=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x170=((r01)*(sj1));
IkReal x171=((cj1)*(r00));
IkReal x172=((IkReal(1.00000000000000))*(cj2));
IkReal x173=((cj1)*(r01));
IkReal x174=((r00)*(sj1));
IkReal x175=((cj3)*(sj2));
IkReal x176=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst103)*(((((IkReal(-1.00000000000000))*(x170)*(x176)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x171)*(x176)))+(((cj2)*(sj3)*(x173)))+(((IkReal(-1.00000000000000))*(sj3)*(x172)*(x174))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst103)*(((((IkReal(-1.00000000000000))*(cj3)*(x172)*(x173)))+(((cj2)*(cj3)*(x174)))+(((r02)*(sj3)))+(((x171)*(x175)))+(((x170)*(x175))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst103)*(((((IkReal(-1.00000000000000))*(x170)*(x176)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x171)*(x176)))+(((cj2)*(sj3)*(x173)))+(((IkReal(-1.00000000000000))*(sj3)*(x172)*(x174)))))), ((gconst103)*(((((IkReal(-1.00000000000000))*(cj3)*(x172)*(x173)))+(((cj2)*(cj3)*(x174)))+(((r02)*(sj3)))+(((x171)*(x175)))+(((x170)*(x175)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x177=IKsin(j5);
IkReal x178=IKcos(j5);
IkReal x179=((IkReal(1.00000000000000))*(cj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj3)*(x178)))+(((IkReal(-1.00000000000000))*(x177)*(x179)))+(r02));
evalcond[1]=((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x178)*(x179)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))+(((sj3)*(x177))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x180=((sj1)*(sj2));
IkReal x181=((IkReal(1.00000000000000))*(cj1));
IkReal x182=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((r00)*(x180)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x181)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x181)))+(((IkReal(-1.00000000000000))*(r01)*(x182))));
evalcond[3]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x181)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x181)))+(((IkReal(-1.00000000000000))*(py)*(x182)))+(((IkReal(-0.270000000000000))*(sj2)))+(((px)*(x180))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x182)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x181)))+(((cj1)*(px)*(sj2)))+(((py)*(x180))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst104;
gconst104=r02;
IkReal gconst105;
gconst105=IkReal(-2.00000000000000);
IkReal gconst106;
gconst106=r02;
IkReal gconst107;
gconst107=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst108;
gconst108=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst109;
gconst109=r02;
IkReal gconst110;
gconst110=IkReal(-2.00000000000000);
IkReal gconst111;
gconst111=r02;
IkReal gconst112;
gconst112=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst113;
gconst113=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst114;
gconst114=IkReal(2.00000000000000);
IkReal gconst115;
gconst115=IkReal(-2.00000000000000);
IkReal gconst116;
gconst116=IkReal(4.00000000000000);
IkReal gconst117;
gconst117=IkReal(2.00000000000000);
IkReal gconst118;
gconst118=IkReal(-2.00000000000000);
IkReal gconst119;
gconst119=IkReal(4.00000000000000);
IkReal gconst120;
gconst120=r02;
IkReal gconst121;
gconst121=IkReal(2.00000000000000);
IkReal gconst122;
gconst122=r02;
IkReal gconst123;
gconst123=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst124;
gconst124=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst125;
gconst125=r02;
IkReal gconst126;
gconst126=IkReal(2.00000000000000);
IkReal gconst127;
gconst127=r02;
IkReal gconst128;
gconst128=((IkReal(-1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal gconst129;
gconst129=((IkReal(1.00000000000000))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
IkReal x183=((IkReal(1.00000000000000))*(gconst128));
IkReal x184=((gconst122)*(gconst123));
IkReal x185=((gconst125)*(gconst129));
IkReal x186=((gconst120)*(gconst124));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x185)*(x186)))+(((IkReal(-1.00000000000000))*(gconst127)*(x183)*(x184)))+(((x184)*(x185)))+(((gconst127)*(gconst128)*(x186)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst124)*(gconst126)*(x183))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x187=((gconst121)*(gconst124));
IkReal x188=((gconst105)*(gconst108));
IkReal x189=((IkReal(1.00000000000000))*(gconst118));
IkReal x190=((gconst107)*(gconst128));
IkReal x191=((gconst106)*(gconst109));
IkReal x192=((gconst107)*(gconst113));
IkReal x193=((IkReal(1.00000000000000))*(gconst112));
IkReal x194=((gconst106)*(gconst112));
IkReal x195=((gconst122)*(gconst123));
IkReal x196=((gconst109)*(gconst113));
IkReal x197=((gconst120)*(gconst124));
IkReal x198=((gconst104)*(gconst108));
IkReal x199=((IkReal(1.00000000000000))*(gconst106));
IkReal x200=((IkReal(1.00000000000000))*(gconst128));
IkReal x201=((gconst110)*(gconst116));
IkReal x202=((IkReal(1.00000000000000))*(gconst125));
IkReal x203=((gconst108)*(gconst120));
IkReal x204=((gconst108)*(gconst114));
IkReal x205=((gconst112)*(gconst127));
IkReal x206=((gconst127)*(gconst128));
IkReal x207=((gconst108)*(gconst121));
IkReal x208=((gconst119)*(gconst125));
IkReal x209=((gconst111)*(gconst112));
IkReal x210=((gconst104)*(gconst124));
IkReal x211=((gconst112)*(gconst118));
IkReal x212=((gconst107)*(gconst129));
IkReal x213=((gconst112)*(gconst122));
IkReal x214=((gconst116)*(gconst126));
IkReal x215=((gconst106)*(gconst125));
IkReal x216=((gconst123)*(gconst129));
IkReal x217=((gconst109)*(gconst122));
IkReal x218=((gconst113)*(gconst117));
IkReal x219=((gconst118)*(gconst128));
IkReal x220=((gconst114)*(gconst124));
IkReal x221=((gconst122)*(gconst125));
IkReal x222=((gconst115)*(gconst123));
IkReal x223=((gconst107)*(gconst111));
IkReal x224=((gconst107)*(gconst127));
IkReal x225=((gconst105)*(gconst124));
IkReal x226=((gconst112)*(gconst115));
IkReal x227=((gconst117)*(gconst119));
IkReal x228=((IkReal(1.00000000000000))*(gconst111));
IkReal x229=((gconst109)*(gconst115));
IkReal x230=((gconst111)*(gconst128));
IkReal x231=((gconst115)*(gconst125));
IkReal x232=((gconst117)*(gconst122));
IkReal x233=((gconst106)*(gconst117));
IkReal x234=((gconst106)*(gconst128));
IkReal x235=((gconst113)*(gconst125));
IkReal x236=((gconst109)*(gconst119));
IkReal x237=((IkReal(1.00000000000000))*(gconst127));
IkReal x238=((gconst122)*(gconst128));
IkReal x239=((gconst106)*(gconst123));
IkReal x240=((gconst115)*(gconst128));
IkReal x241=((gconst115)*(gconst117));
IkReal x242=((gconst113)*(gconst123));
IkReal x243=((gconst110)*(x200));
IkReal x244=((gconst124)*(x230));
IkReal x245=((IkReal(1.00000000000000))*(gconst116)*(gconst119));
IkReal x246=((IkReal(1.00000000000000))*(gconst109)*(gconst129));
IkReal x247=((gconst115)*(x216));
IkReal x248=((IkReal(1.00000000000000))*(gconst117)*(gconst129));
op[0]=((((IkReal(-1.00000000000000))*(gconst127)*(x195)*(x200)))+(((IkReal(-1.00000000000000))*(gconst129)*(x197)*(x202)))+(((x197)*(x206)))+(((IkReal(-1.00000000000000))*(gconst126)*(x187)*(x200)))+(((gconst125)*(gconst129)*(x195))));
op[1]=((((gconst117)*(gconst129)*(x195)))+(((x206)*(x220)))+(((x187)*(x208)))+(((x197)*(x219)))+(((IkReal(-1.00000000000000))*(gconst129)*(x202)*(x220)))+(((IkReal(-1.00000000000000))*(gconst127)*(x200)*(x222)))+(((IkReal(-1.00000000000000))*(gconst128)*(x189)*(x195)))+(((IkReal(-1.00000000000000))*(x197)*(x248)))+(((x214)*(x238)))+(((x216)*(x231))));
op[2]=((((IkReal(-1.00000000000000))*(gconst129)*(x202)*(x210)))+(((gconst109)*(gconst129)*(x195)))+(((IkReal(-1.00000000000000))*(gconst127)*(x193)*(x195)))+(((x195)*(x235)))+(((IkReal(-1.00000000000000))*(gconst126)*(x200)*(x207)))+(((x206)*(x210)))+(((IkReal(-1.00000000000000))*(gconst122)*(x190)*(x237)))+(((IkReal(-1.00000000000000))*(gconst126)*(x200)*(x225)))+(((IkReal(-1.00000000000000))*(x220)*(x248)))+(((x215)*(x216)))+(((x187)*(x227)))+(((x216)*(x241)))+(((IkReal(-1.00000000000000))*(gconst128)*(x189)*(x222)))+(((x219)*(x220)))+(((IkReal(-1.00000000000000))*(x197)*(x246)))+(((x197)*(x205)))+(((x197)*(x230)))+(((IkReal(-1.00000000000000))*(gconst126)*(x187)*(x193)))+(((IkReal(-1.00000000000000))*(gconst113)*(x197)*(x202)))+(((IkReal(-1.00000000000000))*(gconst129)*(x202)*(x203)))+(((x203)*(x206)))+(((IkReal(-1.00000000000000))*(gconst111)*(x195)*(x200)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst119)*(gconst122)*(x202)))+(((x214)*(x240)))+(((IkReal(-1.00000000000000))*(gconst123)*(x199)*(x206)))+(((IkReal(-1.00000000000000))*(x187)*(x243)))+(((x212)*(x221))));
op[3]=((((IkReal(-1.00000000000000))*(gconst122)*(x189)*(x190)))+(((x205)*(x220)))+(((x187)*(x236)))+(((x216)*(x229)))+(((x210)*(x219)))+(((x197)*(x211)))+(((x213)*(x214)))+(((x201)*(x238)))+(((x208)*(x225)))+(((IkReal(-1.00000000000000))*(gconst112)*(x189)*(x195)))+(((IkReal(-1.00000000000000))*(x220)*(x246)))+(((IkReal(-1.00000000000000))*(gconst115)*(x190)*(x237)))+(((x203)*(x219)))+(((IkReal(-1.00000000000000))*(gconst127)*(x193)*(x222)))+(((IkReal(-1.00000000000000))*(x203)*(x248)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(x227)))+(((x222)*(x235)))+(((x204)*(x206)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst119)*(x202)))+(((IkReal(-1.00000000000000))*(gconst113)*(x202)*(x220)))+(((IkReal(-1.00000000000000))*(x197)*(x218)))+(((IkReal(-1.00000000000000))*(x210)*(x248)))+(((x214)*(x234)))+(((IkReal(-1.00000000000000))*(gconst129)*(x202)*(x204)))+(((IkReal(-1.00000000000000))*(gconst111)*(x200)*(x222)))+(((x216)*(x233)))+(((IkReal(-1.00000000000000))*(gconst123)*(x189)*(x234)))+(((x220)*(x230)))+(((x212)*(x231)))+(((x212)*(x232)))+(((x207)*(x208)))+(((x195)*(x218))));
op[4]=((((IkReal(-1.00000000000000))*(gconst122)*(x193)*(x224)))+(((IkReal(-1.00000000000000))*(x217)*(x245)))+(((x192)*(x221)))+(((x201)*(x240)))+(((IkReal(-1.00000000000000))*(gconst129)*(x198)*(x202)))+(((x204)*(x219)))+(((x191)*(x216)))+(((IkReal(-1.00000000000000))*(gconst113)*(x202)*(x210)))+(((x225)*(x227)))+(((x198)*(x206)))+(((IkReal(-1.00000000000000))*(gconst126)*(x188)*(x200)))+(((x207)*(x227)))+(((IkReal(-1.00000000000000))*(gconst111)*(x193)*(x195)))+(((IkReal(-1.00000000000000))*(gconst127)*(x190)*(x199)))+(((x212)*(x241)))+(((IkReal(-1.00000000000000))*(x225)*(x243)))+(((IkReal(-1.00000000000000))*(x207)*(x243)))+(((IkReal(-1.00000000000000))*(x204)*(x248)))+(((IkReal(-1.00000000000000))*(x196)*(x197)))+(((x214)*(x226)))+(((IkReal(-1.00000000000000))*(gconst112)*(x189)*(x222)))+(((x205)*(x210)))+(((IkReal(-1.00000000000000))*(gconst126)*(x193)*(x225)))+(((IkReal(-1.00000000000000))*(x203)*(x246)))+(((x197)*(x209)))+(((x215)*(x242)))+(((x195)*(x196)))+(((IkReal(-1.00000000000000))*(gconst122)*(x190)*(x228)))+(((IkReal(-1.00000000000000))*(gconst126)*(x193)*(x207)))+(((x212)*(x215)))+(((x212)*(x217)))+(((IkReal(-1.00000000000000))*(x210)*(x246)))+(((IkReal(-1.00000000000000))*(gconst115)*(x189)*(x190)))+(((x203)*(x230)))+(((x218)*(x222)))+(((IkReal(-1.00000000000000))*(x218)*(x220)))+(((IkReal(-1.00000000000000))*(gconst113)*(x202)*(x203)))+(((x203)*(x205)))+(((IkReal(-1.00000000000000))*(gconst116)*(x199)*(x208)))+(((IkReal(-1.00000000000000))*(gconst127)*(x193)*(x239)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(x227)))+(((IkReal(-1.00000000000000))*(gconst123)*(x199)*(x230)))+(((IkReal(-1.00000000000000))*(gconst110)*(x187)*(x193)))+(((x210)*(x230)))+(((x211)*(x220))));
op[5]=((((x196)*(x222)))+(((IkReal(-1.00000000000000))*(gconst115)*(x190)*(x228)))+(((x210)*(x211)))+(((IkReal(-1.00000000000000))*(x229)*(x245)))+(((x201)*(x234)))+(((IkReal(-1.00000000000000))*(gconst107)*(x189)*(x213)))+(((x209)*(x220)))+(((IkReal(-1.00000000000000))*(x204)*(x246)))+(((IkReal(-1.00000000000000))*(gconst123)*(x189)*(x194)))+(((x203)*(x211)))+(((x192)*(x231)))+(((x192)*(x232)))+(((x218)*(x239)))+(((IkReal(-1.00000000000000))*(x196)*(x220)))+(((x198)*(x219)))+(((x188)*(x208)))+(((IkReal(-1.00000000000000))*(gconst115)*(x193)*(x224)))+(((x204)*(x205)))+(((x204)*(x230)))+(((x225)*(x236)))+(((IkReal(-1.00000000000000))*(gconst113)*(x202)*(x204)))+(((IkReal(-1.00000000000000))*(gconst111)*(x193)*(x222)))+(((IkReal(-1.00000000000000))*(gconst106)*(x189)*(x190)))+(((IkReal(-1.00000000000000))*(x198)*(x248)))+(((x207)*(x236)))+(((x194)*(x214)))+(((IkReal(-1.00000000000000))*(x203)*(x218)))+(((x212)*(x233)))+(((IkReal(-1.00000000000000))*(gconst116)*(x199)*(x227)))+(((IkReal(-1.00000000000000))*(x210)*(x218)))+(((x212)*(x229)))+(((x201)*(x213))));
op[6]=((((IkReal(-1.00000000000000))*(gconst122)*(x193)*(x223)))+(((x198)*(x230)))+(((IkReal(-1.00000000000000))*(gconst111)*(x193)*(x239)))+(((IkReal(-1.00000000000000))*(gconst107)*(x189)*(x226)))+(((x204)*(x211)))+(((x191)*(x212)))+(((IkReal(-1.00000000000000))*(gconst110)*(x193)*(x225)))+(((x198)*(x205)))+(((x191)*(x242)))+(((x188)*(x227)))+(((IkReal(-1.00000000000000))*(x188)*(x243)))+(((IkReal(-1.00000000000000))*(gconst126)*(x188)*(x193)))+(((IkReal(-1.00000000000000))*(gconst110)*(x193)*(x207)))+(((IkReal(-1.00000000000000))*(x191)*(x245)))+(((IkReal(-1.00000000000000))*(x204)*(x218)))+(((x209)*(x210)))+(((x192)*(x241)))+(((IkReal(-1.00000000000000))*(gconst111)*(x190)*(x199)))+(((IkReal(-1.00000000000000))*(x196)*(x203)))+(((x203)*(x209)))+(((IkReal(-1.00000000000000))*(x196)*(x210)))+(((IkReal(-1.00000000000000))*(x198)*(x246)))+(((IkReal(-1.00000000000000))*(gconst113)*(x198)*(x202)))+(((x201)*(x226)))+(((x192)*(x215)))+(((x192)*(x217)))+(((IkReal(-1.00000000000000))*(gconst106)*(x193)*(x224))));
op[7]=((((x192)*(x229)))+(((IkReal(-1.00000000000000))*(x198)*(x218)))+(((x194)*(x201)))+(((x192)*(x233)))+(((x188)*(x236)))+(((x198)*(x211)))+(((IkReal(-1.00000000000000))*(gconst115)*(x193)*(x223)))+(((IkReal(-1.00000000000000))*(gconst107)*(x189)*(x194)))+(((x204)*(x209)))+(((IkReal(-1.00000000000000))*(x196)*(x204))));
op[8]=((((x198)*(x209)))+(((IkReal(-1.00000000000000))*(gconst110)*(x188)*(x193)))+(((IkReal(-1.00000000000000))*(x196)*(x198)))+(((x191)*(x192)))+(((IkReal(-1.00000000000000))*(gconst106)*(x193)*(x223))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst146;
gconst146=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x249=((r01)*(sj1));
IkReal x250=((cj1)*(r00));
IkReal x251=((IkReal(1.00000000000000))*(cj3));
IkReal x252=((IkReal(1.00000000000000))*(sj3));
IkReal x253=((sj2)*(x252));
IkReal x254=((cj2)*(r00)*(sj1));
IkReal x255=((cj1)*(cj2)*(r01));
if( IKabs(((gconst146)*(((((IkReal(-1.00000000000000))*(x252)*(x254)))+(((IkReal(-1.00000000000000))*(x250)*(x253)))+(((cj3)*(r02)))+(((sj3)*(x255)))+(((IkReal(-1.00000000000000))*(x249)*(x253))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst146)*(((((cj3)*(x255)))+(((IkReal(-1.00000000000000))*(x251)*(x254)))+(((IkReal(-1.00000000000000))*(r02)*(x252)))+(((IkReal(-1.00000000000000))*(sj2)*(x250)*(x251)))+(((IkReal(-1.00000000000000))*(sj2)*(x249)*(x251))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst146)*(((((IkReal(-1.00000000000000))*(x252)*(x254)))+(((IkReal(-1.00000000000000))*(x250)*(x253)))+(((cj3)*(r02)))+(((sj3)*(x255)))+(((IkReal(-1.00000000000000))*(x249)*(x253)))))), ((gconst146)*(((((cj3)*(x255)))+(((IkReal(-1.00000000000000))*(x251)*(x254)))+(((IkReal(-1.00000000000000))*(r02)*(x252)))+(((IkReal(-1.00000000000000))*(sj2)*(x250)*(x251)))+(((IkReal(-1.00000000000000))*(sj2)*(x249)*(x251)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x256=IKsin(j5);
IkReal x257=IKcos(j5);
evalcond[0]=((((sj3)*(x257)))+(r02)+(((IkReal(-1.00000000000000))*(cj3)*(x256))));
evalcond[1]=((((cj3)*(x257)))+(((cj2)*(r00)*(sj1)))+(((sj3)*(x256)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x258=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x259=((cj1)*(x258));
IkReal x260=((sj1)*(x258));
cj5array[0]=((((IkReal(-1.00000000000000))*(r00)*(sj2)*(x260)))+(((cj2)*(r00)*(x259)))+(((r01)*(sj2)*(x259)))+(((cj2)*(r01)*(x260))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x261=((sj1)*(sj2));
IkReal x262=((IkReal(1.00000000000000))*(cj1));
IkReal x263=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x262)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x262)))+(((IkReal(-1.00000000000000))*(r01)*(x263)))+(((r00)*(x261))));
evalcond[3]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(py)*(x263)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x262)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x262)))+(((px)*(x261))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x262)))+(((py)*(x261)))+(((cj1)*(px)*(sj2)))+(((px)*(x263))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x264=((IkReal(1.00000000000000))*(cj2));
IkReal x265=((cj1)*(cj5));
IkReal x266=((sj1)*(sj5));
IkReal x267=((IkReal(1.00000000000000))*(sj2));
IkReal x268=((cj1)*(sj5));
IkReal x269=((cj5)*(sj1));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(r01)*(x266)*(x267)))+(((IkReal(-1.00000000000000))*(r00)*(x264)*(x266)))+(((cj2)*(r01)*(x268)))+(((IkReal(-1.00000000000000))*(r00)*(x267)*(x268)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((r00)*(sj2)*(x265)))+(((r02)*(sj5)))+(((cj2)*(r00)*(x269)))+(((IkReal(-1.00000000000000))*(r01)*(x264)*(x265)))+(((r01)*(sj2)*(x269))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(r01)*(x266)*(x267)))+(((IkReal(-1.00000000000000))*(r00)*(x264)*(x266)))+(((cj2)*(r01)*(x268)))+(((IkReal(-1.00000000000000))*(r00)*(x267)*(x268)))+(((cj5)*(r02)))))), ((gconst55)*(((((r00)*(sj2)*(x265)))+(((r02)*(sj5)))+(((cj2)*(r00)*(x269)))+(((IkReal(-1.00000000000000))*(r01)*(x264)*(x265)))+(((r01)*(sj2)*(x269)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x270=IKsin(j3);
IkReal x271=IKcos(j3);
IkReal x272=((IkReal(1.00000000000000))*(x271));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x270)))+(r02)+(((IkReal(-1.00000000000000))*(sj5)*(x272))));
evalcond[1]=((((sj5)*(x270)))+(((IkReal(-1.00000000000000))*(cj5)*(x272)))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x273=((sj1)*(sj2));
IkReal x274=((IkReal(1.00000000000000))*(cj1));
IkReal x275=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x274)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x274)))+(((IkReal(-1.00000000000000))*(r01)*(x275)))+(((r00)*(x273))));
evalcond[3]=((IkReal(0.215000000000000))+(((px)*(x273)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x274)))+(((IkReal(-1.00000000000000))*(py)*(x275)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x274))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((px)*(x275)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x273)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x274)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x276=((r00)*(sj5));
IkReal x277=((cj2)*(sj1));
IkReal x278=((cj1)*(sj2));
IkReal x279=((cj5)*(r00));
IkReal x280=((r01)*(sj1)*(sj2));
IkReal x281=((IkReal(1.00000000000000))*(cj1)*(cj2)*(r01));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x281)))+(((sj5)*(x280)))+(((x276)*(x277)))+(((x276)*(x278)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((x277)*(x279)))+(((IkReal(-1.00000000000000))*(cj5)*(x281)))+(((x278)*(x279)))+(((cj5)*(x280))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x281)))+(((sj5)*(x280)))+(((x276)*(x277)))+(((x276)*(x278)))+(((cj5)*(r02)))))), ((gconst56)*(((((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((x277)*(x279)))+(((IkReal(-1.00000000000000))*(cj5)*(x281)))+(((x278)*(x279)))+(((cj5)*(x280)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x282=IKsin(j3);
IkReal x283=IKcos(j3);
evalcond[0]=((((IkReal(-1.00000000000000))*(sj5)*(x283)))+(r02)+(((cj5)*(x282))));
evalcond[1]=((((cj2)*(r00)*(sj1)))+(((sj5)*(x282)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))+(((cj5)*(x283))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x284=((IkReal(1.00000000000000))*(r01));
IkReal x285=((cj1)*(sj2));
IkReal x286=((sj1)*(sj2));
IkReal x287=((cj2)*(sj1));
IkReal x288=((IkReal(1.00000000000000))*(py));
IkReal x289=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x286)))+(sj4)+(((IkReal(-1.00000000000000))*(x284)*(x285)))+(((IkReal(-1.00000000000000))*(x284)*(x287)))+(((IkReal(-1.00000000000000))*(r00)*(x289))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x287)*(x288)))+(((IkReal(-1.00000000000000))*(px)*(x289)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x285)*(x288)))+(((IkReal(0.0800000000000000))*(cj4)))+(((px)*(x286))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x290=((r01)*(sj2));
IkReal x291=((IkReal(1.00000000000000))*(cj1));
IkReal x292=((r00)*(sj2));
IkReal x293=((cj2)*(r01));
IkReal x294=((IkReal(1.00000000000000))*(sj1));
IkReal x295=((cj2)*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x291)*(x295)))+(((IkReal(-1.00000000000000))*(x290)*(x291)))+(((sj1)*(x292)))+(((IkReal(-1.00000000000000))*(x293)*(x294))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x291)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x294)))+(((px)*(sj1)*(sj2)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x291))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x291)*(x293)))+(((cj1)*(x292)))+(((sj1)*(x295)))+(((sj1)*(x290))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x296=((IkReal(12.5000000000000))*(sj1));
IkReal x297=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(-0.925000000000000))+(((IkReal(-12.5000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x296)))+(((cj2)*(py)*(x297)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x296)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x297))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.925000000000000))+(((IkReal(-12.5000000000000))*(pz)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x296)))+(((cj2)*(py)*(x297)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x296)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x297)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-0.925000000000000))+(((IkReal(-12.5000000000000))*(pz)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x296)))+(((cj2)*(py)*(x297)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x296)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x297)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((IkReal(0.0740000000000000))+(((IkReal(0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x298=((r01)*(sj2));
IkReal x299=((IkReal(1.00000000000000))*(cj1));
IkReal x300=((r00)*(sj2));
IkReal x301=((cj2)*(r01));
IkReal x302=((IkReal(1.00000000000000))*(sj1));
IkReal x303=((cj2)*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x301)*(x302)))+(((IkReal(-1.00000000000000))*(x298)*(x299)))+(((IkReal(-1.00000000000000))*(x299)*(x303)))+(((sj1)*(x300))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x299)))+(((px)*(sj1)*(sj2)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x299)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x302))));
evalcond[4]=((((cj1)*(x300)))+(((sj1)*(x298)))+(((IkReal(-1.00000000000000))*(x299)*(x301)))+(((sj1)*(x303))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x304=((IkReal(12.5000000000000))*(sj1));
IkReal x305=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(0.925000000000000))+(((IkReal(12.5000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((px)*(sj2)*(x305)))+(((py)*(sj2)*(x304)))+(((cj2)*(px)*(x304)))+(((IkReal(-3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x305))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.925000000000000))+(((IkReal(12.5000000000000))*(pz)))))+IKsqr(((IkReal(-1.12500000000000))+(((px)*(sj2)*(x305)))+(((py)*(sj2)*(x304)))+(((cj2)*(px)*(x304)))+(((IkReal(-3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x305)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(0.925000000000000))+(((IkReal(12.5000000000000))*(pz)))), ((IkReal(-1.12500000000000))+(((px)*(sj2)*(x305)))+(((py)*(sj2)*(x304)))+(((cj2)*(px)*(x304)))+(((IkReal(-3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x305)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((IkReal(0.0740000000000000))+(((IkReal(-0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((gconst57)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(r02)), ((gconst57)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x306=IKsin(j3);
IkReal x307=IKcos(j3);
IkReal x308=((sj1)*(sj2));
IkReal x309=((IkReal(1.00000000000000))*(cj4));
IkReal x310=((IkReal(0.0800000000000000))*(sj4));
IkReal x311=((cj2)*(sj1));
IkReal x312=((cj1)*(sj2));
IkReal x313=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(0.0740000000000000))+(((x306)*(x310)))+(pz));
evalcond[1]=((((IkReal(-1.00000000000000))*(x306)*(x309)))+(r02));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((x307)*(x310)))+(((IkReal(-1.00000000000000))*(py)*(x313)))+(((px)*(x311)))+(((px)*(x312)))+(((py)*(x308))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(x313)))+(((r00)*(x311)))+(((r00)*(x312)))+(((IkReal(-1.00000000000000))*(x307)*(x309)))+(((r01)*(x308))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x314=((IkReal(1.00000000000000))*(r01));
IkReal x315=((cj1)*(sj2));
IkReal x316=((sj1)*(sj2));
IkReal x317=((cj2)*(sj1));
IkReal x318=((IkReal(1.00000000000000))*(py));
IkReal x319=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r00)*(x319)))+(((r00)*(x316)))+(((IkReal(-1.00000000000000))*(x314)*(x317)))+(((IkReal(-1.00000000000000))*(x314)*(x315))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x315)*(x318)))+(((px)*(x316)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(px)*(x319)))+(((IkReal(-1.00000000000000))*(x317)*(x318))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x320=((r01)*(sj2));
IkReal x321=((IkReal(1.00000000000000))*(cj1));
IkReal x322=((r00)*(sj2));
IkReal x323=((cj2)*(r01));
IkReal x324=((IkReal(1.00000000000000))*(sj1));
IkReal x325=((cj2)*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x321)*(x325)))+(((IkReal(-1.00000000000000))*(x323)*(x324)))+(((IkReal(-1.00000000000000))*(x320)*(x321)))+(((sj1)*(x322))));
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x324)))+(((px)*(sj1)*(sj2)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x321)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x321))));
evalcond[4]=((((cj1)*(x322)))+(((IkReal(-1.00000000000000))*(x321)*(x323)))+(((sj1)*(x320)))+(((sj1)*(x325))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x326=((IkReal(12.5000000000000))*(sj1));
IkReal x327=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(-0.925000000000000))+(((IkReal(-12.5000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x327)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x326)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x326)))+(((cj2)*(py)*(x327))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.925000000000000))+(((IkReal(-12.5000000000000))*(pz)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x327)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x326)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x326)))+(((cj2)*(py)*(x327)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-0.925000000000000))+(((IkReal(-12.5000000000000))*(pz)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x327)))+(((IkReal(3.37500000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x326)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x326)))+(((cj2)*(py)*(x327)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((IkReal(0.0740000000000000))+(((IkReal(0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x328=((r01)*(sj2));
IkReal x329=((IkReal(1.00000000000000))*(cj1));
IkReal x330=((r00)*(sj2));
IkReal x331=((cj2)*(r01));
IkReal x332=((IkReal(1.00000000000000))*(sj1));
IkReal x333=((cj2)*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x329)*(x333)))+(((IkReal(-1.00000000000000))*(x331)*(x332)))+(((IkReal(-1.00000000000000))*(x328)*(x329)))+(((sj1)*(x330))));
evalcond[3]=((IkReal(0.295000000000000))+(((px)*(sj1)*(sj2)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x329)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x332)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x329))));
evalcond[4]=((((cj1)*(x330)))+(((IkReal(-1.00000000000000))*(x329)*(x331)))+(((sj1)*(x328)))+(((sj1)*(x333))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x334=((IkReal(12.5000000000000))*(sj1));
IkReal x335=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(0.925000000000000))+(((IkReal(12.5000000000000))*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((px)*(sj2)*(x335)))+(((py)*(sj2)*(x334)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x335)))+(((cj2)*(px)*(x334)))+(((IkReal(-3.37500000000000))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.925000000000000))+(((IkReal(12.5000000000000))*(pz)))))+IKsqr(((IkReal(-1.12500000000000))+(((px)*(sj2)*(x335)))+(((py)*(sj2)*(x334)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x335)))+(((cj2)*(px)*(x334)))+(((IkReal(-3.37500000000000))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(0.925000000000000))+(((IkReal(12.5000000000000))*(pz)))), ((IkReal(-1.12500000000000))+(((px)*(sj2)*(x335)))+(((py)*(sj2)*(x334)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x335)))+(((cj2)*(px)*(x334)))+(((IkReal(-3.37500000000000))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((IkReal(0.0740000000000000))+(((IkReal(-0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x336=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(gconst58)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x336)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x336))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst58)*(r02)), ((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x336)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x336)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x337=IKsin(j3);
IkReal x338=IKcos(j3);
IkReal x339=((sj1)*(sj2));
IkReal x340=((IkReal(0.0800000000000000))*(sj4));
IkReal x341=((cj2)*(sj1));
IkReal x342=((cj1)*(sj2));
IkReal x343=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(0.0740000000000000))+(((x337)*(x340)))+(pz));
evalcond[1]=((r02)+(((cj4)*(x337))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x339)))+(((x338)*(x340)))+(((IkReal(-1.00000000000000))*(py)*(x343)))+(((px)*(x342)))+(((px)*(x341))));
evalcond[3]=((((r01)*(x339)))+(((r00)*(x341)))+(((r00)*(x342)))+(((IkReal(-1.00000000000000))*(r01)*(x343)))+(((cj4)*(x338))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x344=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x345=((cj4)*(cj5));
IkReal x346=((IkReal(25.0000000000000))*(pz));
IkReal x347=((IkReal(0.500000000000000))*(x344));
if( IKabs(((x347)*(((IkReal(-1.85000000000000))+(((IkReal(-1.00000000000000))*(x346))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x347)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x345)*(x346)))+(((IkReal(2.00000000000000))*(r02)*(sj4)))+(((IkReal(1.85000000000000))*(x345))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x347)*(((IkReal(-1.85000000000000))+(((IkReal(-1.00000000000000))*(x346)))))))+IKsqr(((x347)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x345)*(x346)))+(((IkReal(2.00000000000000))*(r02)*(sj4)))+(((IkReal(1.85000000000000))*(x345)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x347)*(((IkReal(-1.85000000000000))+(((IkReal(-1.00000000000000))*(x346)))))), ((x347)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x345)*(x346)))+(((IkReal(2.00000000000000))*(r02)*(sj4)))+(((IkReal(1.85000000000000))*(x345)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x348=IKsin(j3);
IkReal x349=IKcos(j3);
IkReal x350=((sj1)*(sj2));
IkReal x351=((IkReal(0.0800000000000000))*(sj4));
IkReal x352=((cj2)*(sj1));
IkReal x353=((cj1)*(sj2));
IkReal x354=((cj4)*(cj5));
IkReal x355=((IkReal(1.00000000000000))*(x349));
IkReal x356=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(0.0740000000000000))+(pz)+(((x348)*(x351))));
evalcond[1]=((r02)+(((IkReal(-1.00000000000000))*(sj5)*(x355)))+(((IkReal(-1.00000000000000))*(x348)*(x354))));
evalcond[2]=((IkReal(-0.0900000000000000))+(((px)*(x352)))+(((px)*(x353)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x350)))+(((x349)*(x351)))+(((IkReal(-1.00000000000000))*(py)*(x356))));
evalcond[3]=((((r01)*(x350)))+(((sj5)*(x348)))+(((r00)*(x352)))+(((r00)*(x353)))+(((IkReal(-1.00000000000000))*(r01)*(x356)))+(((IkReal(-1.00000000000000))*(x354)*(x355))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x357=((IkReal(25.0000000000000))*(sj1));
IkReal x358=((IkReal(25.0000000000000))*(cj1));
if( IKabs(((gconst54)*(((IkReal(-1.85000000000000))+(((IkReal(-25.0000000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x358)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x357)))+(((IkReal(6.75000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x357)))+(((cj2)*(py)*(x358))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((IkReal(-1.85000000000000))+(((IkReal(-25.0000000000000))*(pz)))))), ((gconst54)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x358)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x357)))+(((IkReal(6.75000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x357)))+(((cj2)*(py)*(x358)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x359=IKsin(j3);
IkReal x360=IKcos(j3);
IkReal x361=((sj1)*(sj2));
IkReal x362=((IkReal(0.0800000000000000))*(sj4));
IkReal x363=((cj2)*(sj1));
IkReal x364=((cj1)*(sj2));
IkReal x365=((cj4)*(cj5));
IkReal x366=((IkReal(1.00000000000000))*(x360));
IkReal x367=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(0.0740000000000000))+(((x359)*(x362)))+(pz));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(x366)))+(((IkReal(-1.00000000000000))*(x359)*(x365)))+(r02));
evalcond[2]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x361)))+(((px)*(x363)))+(((px)*(x364)))+(((IkReal(-1.00000000000000))*(py)*(x367)))+(((x360)*(x362))));
evalcond[3]=((((r01)*(x361)))+(((IkReal(-1.00000000000000))*(r01)*(x367)))+(((r00)*(x363)))+(((r00)*(x364)))+(((sj5)*(x359)))+(((IkReal(-1.00000000000000))*(x365)*(x366))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x368=((IkReal(25.0000000000000))*(px));
IkReal x369=((IkReal(25.0000000000000))*(py));
if( IKabs(((gconst53)*(((IkReal(-1.85000000000000))+(((IkReal(-25.0000000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x368)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x368)))+(((IkReal(6.75000000000000))*(cj2)))+(((cj1)*(cj2)*(x369)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x369))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst53)*(((IkReal(-1.85000000000000))+(((IkReal(-25.0000000000000))*(pz)))))), ((gconst53)*(((IkReal(2.25000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x368)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x368)))+(((IkReal(6.75000000000000))*(cj2)))+(((cj1)*(cj2)*(x369)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x369)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x370=((IkReal(0.0800000000000000))*(sj4));
evalcond[0]=((IkReal(0.0740000000000000))+(((x370)*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((x370)*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x371=(sj3)*(sj3);
IkReal x372=(cj3)*(cj3);
dummyeval[0]=((x372)+(x371));
dummyeval[1]=((((cj4)*(x371)))+(((cj4)*(x372))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x373=((py)*(sj2));
IkReal x374=((px)*(sj1));
IkReal x375=((IkReal(1.00000000000000))*(cj1));
IkReal x376=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x376)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x375)))+(((IkReal(-1.00000000000000))*(x373)*(x375)))+(((sj2)*(x374)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(0.0800000000000000))*(sj4)))+(((sj1)*(x373)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj2)*(x374)))+(((IkReal(-1.00000000000000))*(x375)*(x376)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x377=((sj1)*(sj2));
IkReal x378=((IkReal(1.00000000000000))*(cj1));
IkReal x379=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((r00)*(x377)))+(((IkReal(-1.00000000000000))*(r01)*(x379)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x378)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x378))));
evalcond[3]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(py)*(x379)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x378)))+(((px)*(x377)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x378))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x379)))+(((py)*(x377)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x378)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(r02)+IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(r02, ((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x380=((sj1)*(sj2));
IkReal x381=((IkReal(1.00000000000000))*(cj1));
IkReal x382=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x381)))+(((IkReal(-1.00000000000000))*(r01)*(x382)))+(((r00)*(x380)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x381))));
evalcond[3]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x381)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x381)))+(((px)*(x380)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x382))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((px)*(x382)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x381)))+(((cj1)*(px)*(sj2)))+(((py)*(x380))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x383=((IkReal(1.00000000000000))*(r00));
if( IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x383)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x383))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(r02)+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x383)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x383)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(r02, ((((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x383)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x383)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((IKcos(j5))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x384=((sj1)*(sj2));
IkReal x385=((cj2)*(sj1));
IkReal x386=((cj1)*(sj2));
IkReal x387=((IkReal(1.00000000000000))*(py));
IkReal x388=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((px)*(x384)))+(((IkReal(-1.00000000000000))*(px)*(x388)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x386)*(x387)))+(((IkReal(-1.00000000000000))*(x385)*(x387))));
evalcond[3]=((IkReal(-0.0100000000000000))+(((px)*(x386)))+(((px)*(x385)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x387)))+(((py)*(x384))));
evalcond[4]=((((IkReal(-1.00000000000000))*(r01)*(x388)))+(((r00)*(x386)))+(((r00)*(x385)))+(((r01)*(x384))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(r02)+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(r02, ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x389=((IkReal(1.00000000000000))*(cj2));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x389)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x389)))+(IKcos(j5))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x390=((sj1)*(sj2));
IkReal x391=((cj2)*(sj1));
IkReal x392=((cj1)*(sj2));
IkReal x393=((IkReal(1.00000000000000))*(py));
IkReal x394=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x391)*(x393)))+(((IkReal(-0.270000000000000))*(sj2)))+(((px)*(x390)))+(((IkReal(-1.00000000000000))*(px)*(x394)))+(((IkReal(-1.00000000000000))*(x392)*(x393))));
evalcond[3]=((IkReal(-0.170000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x393)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x390)))+(((px)*(x391)))+(((px)*(x392))));
evalcond[4]=((((r00)*(x391)))+(((r00)*(x392)))+(((r01)*(x390)))+(((IkReal(-1.00000000000000))*(r01)*(x394))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x395=((IkReal(1.00000000000000))*(cj2));
if( IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x395)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x395)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(r02)+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x395)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x395)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(r02, ((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x395)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x395)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x396=((IkReal(1.00000000000000))*(r01));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x396)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x396)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(r02)+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(r02, ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x397=IKcos(j5);
IkReal x398=((r01)*(sj2));
IkReal x399=((IkReal(1.00000000000000))*(cj1));
IkReal x400=((r00)*(sj2));
IkReal x401=((cj2)*(sj1));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x401)))+(((sj1)*(x400)))+(((IkReal(-1.00000000000000))*(x398)*(x399)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x399)))+(((sj4)*(x397))));
evalcond[2]=((((sj1)*(x398)))+(((r00)*(x401)))+(((IkReal(-1.00000000000000))*(cj4)*(x397)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x399)))+(((cj1)*(x400))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(r02)+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(r02, ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x402=IKcos(j5);
IkReal x403=((r01)*(sj2));
IkReal x404=((IkReal(1.00000000000000))*(cj1));
IkReal x405=((r00)*(sj2));
IkReal x406=((cj2)*(sj1));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x406)))+(((sj1)*(x405)))+(((IkReal(-1.00000000000000))*(x403)*(x404)))+(((sj4)*(x402)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x404))));
evalcond[2]=((((sj1)*(x403)))+(((r00)*(x406)))+(((cj1)*(x405)))+(((IkReal(-1.00000000000000))*(cj4)*(x402)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x404))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x407=((py)*(sj2));
IkReal x408=((px)*(sj1));
IkReal x409=((IkReal(1.00000000000000))*(cj1));
IkReal x410=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(sj1)*(x410)))+(((sj2)*(x408)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x409)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x407)*(x409)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((sj1)*(x407)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((cj1)*(px)*(sj2)))+(((cj2)*(x408)))+(((IkReal(-1.00000000000000))*(x409)*(x410))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x411=((sj1)*(sj2));
IkReal x412=((cj2)*(sj1));
IkReal x413=((cj1)*(sj2));
IkReal x414=((IkReal(1.00000000000000))*(py));
IkReal x415=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(px)*(x415)))+(((IkReal(-1.00000000000000))*(x412)*(x414)))+(((px)*(x411)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x413)*(x414))));
evalcond[3]=((IkReal(-0.170000000000000))+(((py)*(x411)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x412)))+(((px)*(x413)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x414))));
evalcond[4]=((((r01)*(x411)))+(((r00)*(x413)))+(((r00)*(x412)))+(((IkReal(-1.00000000000000))*(r01)*(x415))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)), ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x416=((IkReal(1.00000000000000))*(cj2));
evalcond[0]=((IKsin(j5))+(r02));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x416)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x416)))+(IKcos(j5))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x417=((sj1)*(sj2));
IkReal x418=((cj2)*(sj1));
IkReal x419=((cj1)*(sj2));
IkReal x420=((IkReal(1.00000000000000))*(py));
IkReal x421=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(px)*(x421)))+(((px)*(x417)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x419)*(x420)))+(((IkReal(-1.00000000000000))*(x418)*(x420))));
evalcond[3]=((IkReal(-0.0100000000000000))+(((py)*(x417)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x419)))+(((px)*(x418)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x420))));
evalcond[4]=((((r01)*(x417)))+(((r00)*(x419)))+(((r00)*(x418)))+(((IkReal(-1.00000000000000))*(r01)*(x421))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x422=((IkReal(1.00000000000000))*(r01));
if( IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x422)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x422))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x422)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x422)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)), ((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x422)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x422)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x423=((IkReal(1.00000000000000))*(cj2));
evalcond[0]=((IKsin(j5))+(r02));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x423)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x423)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x424=((sj1)*(sj2));
IkReal x425=((IkReal(1.00000000000000))*(cj1));
IkReal x426=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x425)))+(((IkReal(-1.00000000000000))*(r01)*(x426)))+(((r00)*(x424)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x425))));
evalcond[3]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(py)*(x426)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x425)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x425)))+(((px)*(x424))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x424)))+(((cj1)*(px)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x425)))+(((px)*(x426))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x427=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x427)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x427))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)))+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x427)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x427)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)), ((((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x427)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x427)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((IKsin(j5))+(r02));
evalcond[1]=((IKcos(j5))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x428=((sj1)*(sj2));
IkReal x429=((IkReal(1.00000000000000))*(cj1));
IkReal x430=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x430)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x429)))+(((r00)*(x428)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x429))));
evalcond[3]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(py)*(x430)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x429)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x429)))+(((px)*(x428))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((px)*(x430)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x428)))+(((cj1)*(px)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x429))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)))+IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)), ((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((IKsin(j5))+(r02));
evalcond[1]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x431=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x431)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x431))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x431)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x431)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x431)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x431)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x432=IKcos(j5);
IkReal x433=((r01)*(sj2));
IkReal x434=((IkReal(1.00000000000000))*(cj1));
IkReal x435=((r00)*(sj2));
IkReal x436=((cj2)*(sj1));
evalcond[0]=((IKsin(j5))+(r02));
evalcond[1]=((((sj4)*(x432)))+(((IkReal(-1.00000000000000))*(x433)*(x434)))+(((IkReal(-1.00000000000000))*(r01)*(x436)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x434)))+(((sj1)*(x435))));
evalcond[2]=((((cj1)*(x435)))+(((cj4)*(x432)))+(((r00)*(x436)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x434)))+(((sj1)*(x433))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x437=IKcos(j5);
IkReal x438=((r01)*(sj2));
IkReal x439=((IkReal(1.00000000000000))*(cj1));
IkReal x440=((r00)*(sj2));
IkReal x441=((cj2)*(sj1));
evalcond[0]=((IKsin(j5))+(r02));
evalcond[1]=((((sj4)*(x437)))+(((sj1)*(x440)))+(((IkReal(-1.00000000000000))*(x438)*(x439)))+(((IkReal(-1.00000000000000))*(r01)*(x441)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x439))));
evalcond[2]=((((cj1)*(x440)))+(((cj4)*(x437)))+(((r00)*(x441)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x439)))+(((sj1)*(x438))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x442=((sj1)*(sj2));
IkReal x443=((IkReal(1.00000000000000))*(cj1));
IkReal x444=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x443)))+(((IkReal(-1.00000000000000))*(r01)*(x444)))+(((r00)*(x442)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x443))));
evalcond[3]=((IkReal(0.375000000000000))+(((px)*(x442)))+(((IkReal(-1.00000000000000))*(py)*(x444)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x443)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x443))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((px)*(x444)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x442)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x443)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x445=((r01)*(sj1));
IkReal x446=((cj1)*(r00));
IkReal x447=((IkReal(1.00000000000000))*(cj2));
IkReal x448=((cj1)*(r01));
IkReal x449=((r00)*(sj1));
IkReal x450=((cj3)*(sj2));
IkReal x451=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(x446)*(x451)))+(((cj2)*(sj3)*(x448)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(sj3)*(x447)*(x449)))+(((IkReal(-1.00000000000000))*(x445)*(x451))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(cj3)*(x447)*(x448)))+(((r02)*(sj3)))+(((x446)*(x450)))+(((cj2)*(cj3)*(x449)))+(((x445)*(x450))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst59)*(((((IkReal(-1.00000000000000))*(x446)*(x451)))+(((cj2)*(sj3)*(x448)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(sj3)*(x447)*(x449)))+(((IkReal(-1.00000000000000))*(x445)*(x451)))))), ((gconst59)*(((((IkReal(-1.00000000000000))*(cj3)*(x447)*(x448)))+(((r02)*(sj3)))+(((x446)*(x450)))+(((cj2)*(cj3)*(x449)))+(((x445)*(x450)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x452=IKsin(j5);
IkReal x453=IKcos(j5);
IkReal x454=((IkReal(1.00000000000000))*(cj3));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj3)*(x453)))+(((IkReal(-1.00000000000000))*(x452)*(x454)))+(r02));
evalcond[1]=((((sj3)*(x452)))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x453)*(x454)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x455=((sj1)*(sj2));
IkReal x456=((IkReal(1.00000000000000))*(cj1));
IkReal x457=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x456)))+(((IkReal(-1.00000000000000))*(r01)*(x457)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x456)))+(((r00)*(x455))));
evalcond[3]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(py)*(x457)))+(((IkReal(-0.270000000000000))*(sj2)))+(((px)*(x455)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x456)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x456))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x456)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x457)))+(((py)*(x455)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x458=((r01)*(sj1));
IkReal x459=((cj1)*(r00));
IkReal x460=((cj2)*(cj3));
IkReal x461=((cj1)*(r01));
IkReal x462=((IkReal(1.00000000000000))*(sj3));
IkReal x463=((r00)*(sj1));
IkReal x464=((sj2)*(x462));
IkReal x465=((IkReal(1.00000000000000))*(cj3)*(sj2));
if( IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(cj2)*(x462)*(x463)))+(((IkReal(-1.00000000000000))*(x459)*(x464)))+(((cj2)*(sj3)*(x461)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x458)*(x464))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(x459)*(x465)))+(((x460)*(x461)))+(((IkReal(-1.00000000000000))*(r02)*(x462)))+(((IkReal(-1.00000000000000))*(x458)*(x465)))+(((IkReal(-1.00000000000000))*(x460)*(x463))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst60)*(((((IkReal(-1.00000000000000))*(cj2)*(x462)*(x463)))+(((IkReal(-1.00000000000000))*(x459)*(x464)))+(((cj2)*(sj3)*(x461)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x458)*(x464)))))), ((gconst60)*(((((IkReal(-1.00000000000000))*(x459)*(x465)))+(((x460)*(x461)))+(((IkReal(-1.00000000000000))*(r02)*(x462)))+(((IkReal(-1.00000000000000))*(x458)*(x465)))+(((IkReal(-1.00000000000000))*(x460)*(x463)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x466=IKsin(j5);
IkReal x467=IKcos(j5);
evalcond[0]=((((sj3)*(x467)))+(r02)+(((IkReal(-1.00000000000000))*(cj3)*(x466))));
evalcond[1]=((((sj3)*(x466)))+(((cj3)*(x467)))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x468=((py)*(sj2));
IkReal x469=((cj2)*(px));
IkReal x470=((IkReal(1.00000000000000))*(cj1));
IkReal x471=((cj2)*(py));
IkReal x472=((px)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(((IkReal(0.0800000000000000))*(sj4)))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x468)*(x470)))+(((IkReal(-1.00000000000000))*(x469)*(x470)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x471)))+(((sj1)*(x472)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj1)*(x472)))+(((sj1)*(x468)))+(((sj1)*(x469)))+(((IkReal(-1.00000000000000))*(x470)*(x471))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x473=((sj1)*(sj2));
IkReal x474=((IkReal(1.00000000000000))*(cj1));
IkReal x475=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x474)))+(((IkReal(-1.00000000000000))*(r01)*(x475)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x474)))+(((r00)*(x473))));
evalcond[3]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x474)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x474)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x475)))+(((px)*(x473))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x474)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x473)))+(((px)*(x475)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x476=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x476)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x476)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x476)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x476)))+(((cj1)*(cj2)*(r01)))))+IKsqr(r02)-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x476)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x476)))+(((cj1)*(cj2)*(r01)))), r02);
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(r02));
evalcond[1]=((IKsin(j5))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x477=((sj1)*(sj2));
IkReal x478=((IkReal(1.00000000000000))*(cj1));
IkReal x479=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x478)))+(((IkReal(-1.00000000000000))*(r01)*(x479)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x478)))+(((r00)*(x477))));
evalcond[3]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x478)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x478)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x479)))+(((px)*(x477))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x478)))+(((IkReal(-0.270000000000000))*(cj2)))+(((py)*(x477)))+(((px)*(x479)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x480=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x480)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x480)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x480)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x480)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x480)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x480)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))), ((IkReal(-1.00000000000000))*(r02)));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((IKcos(j5))+(r02));
evalcond[1]=((IKsin(j5))+(((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x481=((py)*(sj2));
IkReal x482=((px)*(sj1));
IkReal x483=((IkReal(1.00000000000000))*(cj1));
IkReal x484=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.154000000000000))+(pz));
evalcond[2]=r02;
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x483)))+(((IkReal(-1.00000000000000))*(x481)*(x483)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x484)))+(((sj2)*(x482))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((cj2)*(x482)))+(((IkReal(-1.00000000000000))*(x483)*(x484)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x481)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x485=((cj1)*(r00));
IkReal x486=((cj2)*(r01));
IkReal x487=((IkReal(1.00000000000000))*(sj2));
IkReal x488=((r00)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(x488)))+(((cj1)*(x486)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x487)))+(((IkReal(-1.00000000000000))*(x485)*(x487))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(x485)))+(((sj1)*(x486)))+(((IkReal(-1.00000000000000))*(x487)*(x488))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(x488)))+(((cj1)*(x486)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x487)))+(((IkReal(-1.00000000000000))*(x485)*(x487)))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(x485)))+(((sj1)*(x486)))+(((IkReal(-1.00000000000000))*(x487)*(x488)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(x488)))+(((cj1)*(x486)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x487)))+(((IkReal(-1.00000000000000))*(x485)*(x487)))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(x485)))+(((sj1)*(x486)))+(((IkReal(-1.00000000000000))*(x487)*(x488)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x489=((r01)*(sj2));
IkReal x490=((IkReal(1.00000000000000))*(cj1));
IkReal x491=((r00)*(sj2));
IkReal x492=((cj2)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(x489)*(x490)))+(((IkReal(-1.00000000000000))*(r01)*(x492)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x490)))+(((sj1)*(x491)))+(IKcos(j5)));
evalcond[1]=((((sj1)*(x489)))+(IKsin(j5))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x490)))+(((cj1)*(x491)))+(((r00)*(x492))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x493=((py)*(sj2));
IkReal x494=((px)*(sj1));
IkReal x495=((IkReal(1.00000000000000))*(cj1));
IkReal x496=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.00600000000000000))+(pz));
evalcond[2]=r02;
evalcond[3]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x493)*(x495)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x495)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x496)))+(((sj2)*(x494))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-0.270000000000000))*(cj2)))+(((sj1)*(x493)))+(((cj1)*(px)*(sj2)))+(((cj2)*(x494))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x497=((r00)*(sj1));
IkReal x498=((IkReal(1.00000000000000))*(cj2));
IkReal x499=((r01)*(sj1));
IkReal x500=((IkReal(1.00000000000000))*(cj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x499)))+(((IkReal(-1.00000000000000))*(x497)*(x498)))+(((IkReal(-1.00000000000000))*(r00)*(x500)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x498)))+(((IkReal(-1.00000000000000))*(r01)*(x500)))+(((sj2)*(x497))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x499)))+(((IkReal(-1.00000000000000))*(x497)*(x498)))+(((IkReal(-1.00000000000000))*(r00)*(x500)))+(((cj1)*(cj2)*(r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x498)))+(((IkReal(-1.00000000000000))*(r01)*(x500)))+(((sj2)*(x497)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj2)*(x499)))+(((IkReal(-1.00000000000000))*(x497)*(x498)))+(((IkReal(-1.00000000000000))*(r00)*(x500)))+(((cj1)*(cj2)*(r01)))), ((((IkReal(-1.00000000000000))*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x498)))+(((IkReal(-1.00000000000000))*(r01)*(x500)))+(((sj2)*(x497)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x501=((r01)*(sj2));
IkReal x502=((IkReal(1.00000000000000))*(cj1));
IkReal x503=((r00)*(sj2));
IkReal x504=((cj2)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x502)))+(((sj1)*(x503)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x501)*(x502)))+(((IkReal(-1.00000000000000))*(r01)*(x504))));
evalcond[1]=((((cj1)*(x503)))+(((sj1)*(x501)))+(IKsin(j5))+(((r00)*(x504)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x502))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x505=((cj1)*(r00));
IkReal x506=((cj2)*(r01));
IkReal x507=((IkReal(1.00000000000000))*(sj2));
IkReal x508=((r00)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x505)*(x507)))+(((cj1)*(x506)))+(((IkReal(-1.00000000000000))*(cj2)*(x508)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x507))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(x505)))+(((sj1)*(x506)))+(((IkReal(-1.00000000000000))*(x507)*(x508))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x505)*(x507)))+(((cj1)*(x506)))+(((IkReal(-1.00000000000000))*(cj2)*(x508)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x507)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(x505)))+(((sj1)*(x506)))+(((IkReal(-1.00000000000000))*(x507)*(x508)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x505)*(x507)))+(((cj1)*(x506)))+(((IkReal(-1.00000000000000))*(cj2)*(x508)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x507)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((cj1)*(r01)*(sj2)))+(((cj2)*(x505)))+(((sj1)*(x506)))+(((IkReal(-1.00000000000000))*(x507)*(x508)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x509=IKcos(j5);
IkReal x510=((r01)*(sj2));
IkReal x511=((IkReal(1.00000000000000))*(cj1));
IkReal x512=((r00)*(sj2));
IkReal x513=((cj2)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj4)*(x509)))+(r02));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x513)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x511)))+(((sj4)*(x509)))+(((sj1)*(x512)))+(((IkReal(-1.00000000000000))*(x510)*(x511))));
evalcond[2]=((IKsin(j5))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x511)))+(((cj1)*(x512)))+(((sj1)*(x510)))+(((r00)*(x513))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x514=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x514)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x514)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x514)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x514)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))+IKsqr(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x514)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x514)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))), ((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x515=IKcos(j5);
IkReal x516=((r01)*(sj2));
IkReal x517=((IkReal(1.00000000000000))*(cj1));
IkReal x518=((r00)*(sj2));
IkReal x519=((cj2)*(sj1));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(cj4)*(x515))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x519)))+(((sj4)*(x515)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x517)))+(((IkReal(-1.00000000000000))*(x516)*(x517)))+(((sj1)*(x518))));
evalcond[2]=((IKsin(j5))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x517)))+(((cj1)*(x518)))+(((sj1)*(x516)))+(((r00)*(x519))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x520=((py)*(sj2));
IkReal x521=((cj2)*(px));
IkReal x522=((IkReal(1.00000000000000))*(cj1));
IkReal x523=((cj2)*(py));
IkReal x524=((px)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz)+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x520)*(x522)))+(((sj1)*(x524)))+(((IkReal(-1.00000000000000))*(x521)*(x522)))+(((IkReal(-1.00000000000000))*(sj1)*(x523)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((sj1)*(x521)))+(((sj1)*(x520)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj1)*(x524)))+(((IkReal(-1.00000000000000))*(x522)*(x523))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x525=((sj1)*(sj2));
IkReal x526=((IkReal(1.00000000000000))*(cj1));
IkReal x527=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x526)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x526)))+(((IkReal(-1.00000000000000))*(r01)*(x527)))+(((r00)*(x525))));
evalcond[3]=((IkReal(0.375000000000000))+(((IkReal(-1.00000000000000))*(py)*(x527)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x526)))+(((IkReal(-0.270000000000000))*(sj2)))+(((px)*(x525)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x526))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x526)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x527)))+(((cj1)*(px)*(sj2)))+(((py)*(x525))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), ((IkReal(-1.00000000000000))*(r02)));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((IKcos(j5))+(r02));
evalcond[1]=((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x528=((sj1)*(sj2));
IkReal x529=((IkReal(1.00000000000000))*(cj1));
IkReal x530=((cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x529)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x529)))+(((IkReal(-1.00000000000000))*(r01)*(x530)))+(((r00)*(x528))));
evalcond[3]=((IkReal(0.215000000000000))+(((IkReal(-1.00000000000000))*(py)*(x530)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x529)))+(((IkReal(-0.270000000000000))*(sj2)))+(((px)*(x528)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x529))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x529)))+(((IkReal(-0.270000000000000))*(cj2)))+(((px)*(x530)))+(((cj1)*(px)*(sj2)))+(((py)*(x528))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(r02)-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), r02);
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(-1.00000000000000))*(IKcos(j5))))+(r02));
evalcond[1]=((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x531=((py)*(sj2));
IkReal x532=((px)*(sj1));
IkReal x533=((IkReal(1.00000000000000))*(cj1));
IkReal x534=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.00600000000000000))+(pz));
evalcond[2]=r02;
evalcond[3]=((IkReal(0.295000000000000))+(((sj2)*(x532)))+(((IkReal(-1.00000000000000))*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(sj1)*(x534)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x533)))+(((IkReal(-0.270000000000000))*(sj2))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x533)*(x534)))+(((sj1)*(x531)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj2)*(x532)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x535=((cj1)*(cj2));
IkReal x536=((r01)*(sj1));
IkReal x537=((cj1)*(sj2));
IkReal x538=((r00)*(sj1));
if( IKabs(((((sj2)*(x536)))+(((cj2)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(x535)))+(((r00)*(x537))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x538)))+(((cj2)*(x536)))+(((r01)*(x537)))+(((r00)*(x535))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj2)*(x536)))+(((cj2)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(x535)))+(((r00)*(x537)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x538)))+(((cj2)*(x536)))+(((r01)*(x537)))+(((r00)*(x535)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((sj2)*(x536)))+(((cj2)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(x535)))+(((r00)*(x537)))), ((((IkReal(-1.00000000000000))*(sj2)*(x538)))+(((cj2)*(x536)))+(((r01)*(x537)))+(((r00)*(x535)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x539=((r01)*(sj2));
IkReal x540=((IkReal(1.00000000000000))*(cj1));
IkReal x541=((r00)*(sj2));
IkReal x542=((cj2)*(sj1));
evalcond[0]=((((sj1)*(x541)))+(((IkReal(-1.00000000000000))*(x539)*(x540)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x540)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(r01)*(x542))));
evalcond[1]=((((sj1)*(x539)))+(((cj1)*(x541)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x540)))+(((IkReal(-1.00000000000000))*(IKsin(j5))))+(((r00)*(x542))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x543=((py)*(sj2));
IkReal x544=((px)*(sj1));
IkReal x545=((IkReal(1.00000000000000))*(cj1));
IkReal x546=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.154000000000000))+(pz));
evalcond[2]=r02;
evalcond[3]=((IkReal(0.295000000000000))+(((sj2)*(x544)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x545)))+(((IkReal(-1.00000000000000))*(sj1)*(x546)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x543)*(x545))));
evalcond[4]=((IkReal(-0.0900000000000000))+(((sj1)*(x543)))+(((IkReal(-1.00000000000000))*(x545)*(x546)))+(((IkReal(-0.270000000000000))*(cj2)))+(((cj2)*(x544)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x547=((r01)*(sj2));
IkReal x548=((IkReal(1.00000000000000))*(cj1));
IkReal x549=((r00)*(sj2));
IkReal x550=((cj2)*(sj1));
if( IKabs(((((sj1)*(x547)))+(((cj1)*(x549)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x548)))+(((r00)*(x550))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj1)*(x549)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x548)))+(((IkReal(-1.00000000000000))*(x547)*(x548)))+(((IkReal(-1.00000000000000))*(r01)*(x550))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj1)*(x547)))+(((cj1)*(x549)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x548)))+(((r00)*(x550)))))+IKsqr(((((sj1)*(x549)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x548)))+(((IkReal(-1.00000000000000))*(x547)*(x548)))+(((IkReal(-1.00000000000000))*(r01)*(x550)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((sj1)*(x547)))+(((cj1)*(x549)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x548)))+(((r00)*(x550)))), ((((sj1)*(x549)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x548)))+(((IkReal(-1.00000000000000))*(x547)*(x548)))+(((IkReal(-1.00000000000000))*(r01)*(x550)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x551=((r01)*(sj2));
IkReal x552=((IkReal(1.00000000000000))*(cj1));
IkReal x553=((r00)*(sj2));
IkReal x554=((cj2)*(sj1));
evalcond[0]=((((sj1)*(x553)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(x551)*(x552)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x552)))+(((IkReal(-1.00000000000000))*(r01)*(x554))));
evalcond[1]=((((sj1)*(x551)))+(((cj1)*(x553)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x552)))+(((r00)*(x554)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x555=((cj1)*(cj2));
IkReal x556=((r01)*(sj1));
IkReal x557=((cj1)*(sj2));
IkReal x558=((r00)*(sj1));
if( IKabs(((((sj2)*(x556)))+(((r00)*(x557)))+(((cj2)*(x558)))+(((IkReal(-1.00000000000000))*(r01)*(x555))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(x558)))+(((r01)*(x557)))+(((r00)*(x555)))+(((cj2)*(x556))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj2)*(x556)))+(((r00)*(x557)))+(((cj2)*(x558)))+(((IkReal(-1.00000000000000))*(r01)*(x555)))))+IKsqr(((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(x558)))+(((r01)*(x557)))+(((r00)*(x555)))+(((cj2)*(x556)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((sj2)*(x556)))+(((r00)*(x557)))+(((cj2)*(x558)))+(((IkReal(-1.00000000000000))*(r01)*(x555)))), ((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(x558)))+(((r01)*(x557)))+(((r00)*(x555)))+(((cj2)*(x556)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x559=IKcos(j5);
IkReal x560=((r01)*(sj2));
IkReal x561=((IkReal(1.00000000000000))*(cj1));
IkReal x562=((r00)*(sj2));
IkReal x563=((cj2)*(sj1));
evalcond[0]=((((cj4)*(x559)))+(r02));
evalcond[1]=((((sj4)*(x559)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x561)))+(((IkReal(-1.00000000000000))*(r01)*(x563)))+(((sj1)*(x562)))+(((IkReal(-1.00000000000000))*(x560)*(x561))));
evalcond[2]=((((cj1)*(x562)))+(((r00)*(x563)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x561)))+(((sj1)*(x560)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), ((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x564=IKcos(j5);
IkReal x565=((r01)*(sj2));
IkReal x566=((IkReal(1.00000000000000))*(cj1));
IkReal x567=((r00)*(sj2));
IkReal x568=((cj2)*(sj1));
evalcond[0]=((r02)+(((cj4)*(x564))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x566)))+(((IkReal(-1.00000000000000))*(r01)*(x568)))+(((sj4)*(x564)))+(((sj1)*(x567)))+(((IkReal(-1.00000000000000))*(x565)*(x566))));
evalcond[2]=((((cj1)*(x567)))+(((r00)*(x568)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x566)))+(((sj1)*(x565)))+(((IkReal(-1.00000000000000))*(IKsin(j5)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x569=((py)*(sj2));
IkReal x570=((px)*(sj1));
IkReal x571=((IkReal(1.00000000000000))*(cj1));
IkReal x572=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(((IkReal(0.0800000000000000))*(sj3)))+(pz));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(x569)*(x571)))+(((IkReal(-0.270000000000000))*(sj2)))+(((sj2)*(x570)))+(((IkReal(-1.00000000000000))*(sj1)*(x572)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x571))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x571)*(x572)))+(((cj2)*(x570)))+(((sj1)*(x569)))+(((IkReal(0.0800000000000000))*(cj3)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x573=((cj1)*(r00));
IkReal x574=((cj2)*(r01));
IkReal x575=((IkReal(1.00000000000000))*(sj2));
IkReal x576=((r00)*(sj1));
if( IKabs(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x575)))+(((IkReal(-1.00000000000000))*(x573)*(x575)))+(((cj1)*(x574)))+(((IkReal(-1.00000000000000))*(cj2)*(x576))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((sj1)*(x574)))+(((cj2)*(x573)))+(((IkReal(-1.00000000000000))*(x575)*(x576))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x575)))+(((IkReal(-1.00000000000000))*(x573)*(x575)))+(((cj1)*(x574)))+(((IkReal(-1.00000000000000))*(cj2)*(x576)))))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((sj1)*(x574)))+(((cj2)*(x573)))+(((IkReal(-1.00000000000000))*(x575)*(x576)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x575)))+(((IkReal(-1.00000000000000))*(x573)*(x575)))+(((cj1)*(x574)))+(((IkReal(-1.00000000000000))*(cj2)*(x576)))))), ((((cj1)*(r01)*(sj2)))+(((sj1)*(x574)))+(((cj2)*(x573)))+(((IkReal(-1.00000000000000))*(x575)*(x576)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x577=IKsin(j5);
IkReal x578=((r01)*(sj2));
IkReal x579=((IkReal(1.00000000000000))*(cj1));
IkReal x580=((r00)*(sj1));
IkReal x581=((cj2)*(r01));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj3)*(x577)))+(r02));
evalcond[1]=((((IkReal(-1.00000000000000))*(x578)*(x579)))+(((sj2)*(x580)))+(((IkReal(-1.00000000000000))*(sj1)*(x581)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x579)))+(IKcos(j5)));
evalcond[2]=((((sj1)*(x578)))+(((IkReal(-1.00000000000000))*(x579)*(x581)))+(((sj3)*(x577)))+(((cj1)*(r00)*(sj2)))+(((cj2)*(x580))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x582=IKsin(j5);
IkReal x583=((r01)*(sj2));
IkReal x584=((IkReal(1.00000000000000))*(cj1));
IkReal x585=((r00)*(sj1));
IkReal x586=((cj2)*(r01));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj3)*(x582)))+(r02));
evalcond[1]=((((sj2)*(x585)))+(((IkReal(-1.00000000000000))*(sj1)*(x586)))+(((IkReal(-1.00000000000000))*(x583)*(x584)))+(IKcos(j5))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x584))));
evalcond[2]=((((sj1)*(x583)))+(((IkReal(-1.00000000000000))*(x584)*(x586)))+(((sj3)*(x582)))+(((cj1)*(r00)*(sj2)))+(((cj2)*(x585))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x587=((py)*(sj2));
IkReal x588=((px)*(sj1));
IkReal x589=((IkReal(1.00000000000000))*(cj1));
IkReal x590=((cj2)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0740000000000000))+(pz)+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((IkReal(0.295000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x589)))+(((sj2)*(x588)))+(((IkReal(-1.00000000000000))*(x587)*(x589)))+(((IkReal(-0.270000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj1)*(x590))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x589)*(x590)))+(((sj1)*(x587)))+(((IkReal(-0.270000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(cj3)))+(((cj2)*(x588)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x591=((r00)*(sj1));
IkReal x592=((IkReal(1.00000000000000))*(cj2));
IkReal x593=((r01)*(sj1));
IkReal x594=((IkReal(1.00000000000000))*(cj1)*(sj2));
if( IKabs(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(x594)))+(((IkReal(-1.00000000000000))*(sj2)*(x593)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x591)*(x592))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(x594)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x592)))+(((IkReal(-1.00000000000000))*(x592)*(x593)))+(((sj2)*(x591))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(x594)))+(((IkReal(-1.00000000000000))*(sj2)*(x593)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x591)*(x592)))))))+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(x594)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x592)))+(((IkReal(-1.00000000000000))*(x592)*(x593)))+(((sj2)*(x591)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(x594)))+(((IkReal(-1.00000000000000))*(sj2)*(x593)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x591)*(x592)))))), ((((IkReal(-1.00000000000000))*(r01)*(x594)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x592)))+(((IkReal(-1.00000000000000))*(x592)*(x593)))+(((sj2)*(x591)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x595=IKsin(j5);
IkReal x596=((r01)*(sj2));
IkReal x597=((IkReal(1.00000000000000))*(cj1));
IkReal x598=((r00)*(sj1));
IkReal x599=((cj2)*(r01));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj3)*(x595)))+(r02));
evalcond[1]=((((IkReal(-1.00000000000000))*(x596)*(x597)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(sj1)*(x599)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x597)))+(((sj2)*(x598))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x597)*(x599)))+(((cj2)*(x598)))+(((sj1)*(x596)))+(((cj1)*(r00)*(sj2)))+(((sj3)*(x595))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x600=((IkReal(1.00000000000000))*(r01));
if( IKabs(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x600)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x600)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x600)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x600)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x600)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x600)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x601=IKsin(j5);
IkReal x602=((r01)*(sj2));
IkReal x603=((IkReal(1.00000000000000))*(cj1));
IkReal x604=((r00)*(sj1));
IkReal x605=((cj2)*(r01));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(cj3)*(x601))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj1)*(x605)))+(((IkReal(-1.00000000000000))*(x602)*(x603)))+(((IkReal(-1.00000000000000))*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x603)))+(((sj2)*(x604))));
evalcond[2]=((((sj1)*(x602)))+(((IkReal(-1.00000000000000))*(x603)*(x605)))+(((cj2)*(x604)))+(((cj1)*(r00)*(sj2)))+(((sj3)*(x601))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x606=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x607=((cj1)*(cj2));
IkReal x608=((cj3)*(cj4));
IkReal x609=((r01)*(sj4));
IkReal x610=((cj2)*(sj1));
IkReal x611=((IkReal(1.00000000000000))*(r00)*(sj2));
IkReal x612=((cj1)*(r01)*(sj2));
if( IKabs(((x606)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((x608)*(x612)))+(((IkReal(-1.00000000000000))*(sj1)*(x608)*(x611)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x609)))+(((r01)*(x608)*(x610)))+(((x607)*(x609)))+(((IkReal(-1.00000000000000))*(cj1)*(sj4)*(x611)))+(((IkReal(-1.00000000000000))*(r00)*(sj4)*(x610)))+(((r00)*(x607)*(x608))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x606)*(((((r00)*(x607)))+(((r01)*(x610)))+(((IkReal(-1.00000000000000))*(sj1)*(x611)))+(x612))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x606)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((x608)*(x612)))+(((IkReal(-1.00000000000000))*(sj1)*(x608)*(x611)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x609)))+(((r01)*(x608)*(x610)))+(((x607)*(x609)))+(((IkReal(-1.00000000000000))*(cj1)*(sj4)*(x611)))+(((IkReal(-1.00000000000000))*(r00)*(sj4)*(x610)))+(((r00)*(x607)*(x608)))))))+IKsqr(((x606)*(((((r00)*(x607)))+(((r01)*(x610)))+(((IkReal(-1.00000000000000))*(sj1)*(x611)))+(x612)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x606)*(((IKabs(sj3) != 0)?((IkReal)1/(sj3)):(IkReal)1.0e30))*(((((x608)*(x612)))+(((IkReal(-1.00000000000000))*(sj1)*(x608)*(x611)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x609)))+(((r01)*(x608)*(x610)))+(((x607)*(x609)))+(((IkReal(-1.00000000000000))*(cj1)*(sj4)*(x611)))+(((IkReal(-1.00000000000000))*(r00)*(sj4)*(x610)))+(((r00)*(x607)*(x608)))))), ((x606)*(((((r00)*(x607)))+(((r01)*(x610)))+(((IkReal(-1.00000000000000))*(sj1)*(x611)))+(x612)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x613=IKcos(j5);
IkReal x614=IKsin(j5);
IkReal x615=((r01)*(sj2));
IkReal x616=((IkReal(1.00000000000000))*(cj1));
IkReal x617=((r00)*(sj1));
IkReal x618=((IkReal(1.00000000000000))*(cj3));
IkReal x619=((cj2)*(r01));
IkReal x620=((cj4)*(x613));
evalcond[0]=((((IkReal(-1.00000000000000))*(x614)*(x618)))+(((IkReal(-1.00000000000000))*(sj3)*(x620)))+(r02));
evalcond[1]=((((sj4)*(x613)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x616)))+(((IkReal(-1.00000000000000))*(x615)*(x616)))+(((sj2)*(x617)))+(((IkReal(-1.00000000000000))*(sj1)*(x619))));
evalcond[2]=((((sj1)*(x615)))+(((cj2)*(x617)))+(((sj3)*(x614)))+(((IkReal(-1.00000000000000))*(x618)*(x620)))+(((cj1)*(r00)*(sj2)))+(((IkReal(-1.00000000000000))*(x616)*(x619))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x621=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x622=((IkReal(1.00000000000000))*(r01));
IkReal x623=((r00)*(sj1));
IkReal x624=((cj2)*(sj1));
IkReal x625=((cj4)*(sj3));
IkReal x626=((sj2)*(x625));
IkReal x627=((cj1)*(cj2)*(r00));
if( IKabs(((x621)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(x622)*(x626)))+(((IkReal(-1.00000000000000))*(x622)*(x624)*(x625)))+(((r02)*(sj4)))+(((x623)*(x626)))+(((IkReal(-1.00000000000000))*(x625)*(x627))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x621)*(((((cj1)*(r01)*(sj2)))+(((r01)*(x624)))+(((IkReal(-1.00000000000000))*(sj2)*(x623)))+(x627))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x621)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(x622)*(x626)))+(((IkReal(-1.00000000000000))*(x622)*(x624)*(x625)))+(((r02)*(sj4)))+(((x623)*(x626)))+(((IkReal(-1.00000000000000))*(x625)*(x627)))))))+IKsqr(((x621)*(((((cj1)*(r01)*(sj2)))+(((r01)*(x624)))+(((IkReal(-1.00000000000000))*(sj2)*(x623)))+(x627)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x621)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(x622)*(x626)))+(((IkReal(-1.00000000000000))*(x622)*(x624)*(x625)))+(((r02)*(sj4)))+(((x623)*(x626)))+(((IkReal(-1.00000000000000))*(x625)*(x627)))))), ((x621)*(((((cj1)*(r01)*(sj2)))+(((r01)*(x624)))+(((IkReal(-1.00000000000000))*(sj2)*(x623)))+(x627)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x628=IKcos(j5);
IkReal x629=IKsin(j5);
IkReal x630=((r01)*(sj2));
IkReal x631=((IkReal(1.00000000000000))*(cj1));
IkReal x632=((r00)*(sj1));
IkReal x633=((IkReal(1.00000000000000))*(cj3));
IkReal x634=((cj2)*(r01));
IkReal x635=((cj4)*(x628));
evalcond[0]=((((IkReal(-1.00000000000000))*(x629)*(x633)))+(((IkReal(-1.00000000000000))*(sj3)*(x635)))+(r02));
evalcond[1]=((((sj4)*(x628)))+(((IkReal(-1.00000000000000))*(x630)*(x631)))+(((IkReal(-1.00000000000000))*(sj1)*(x634)))+(((sj2)*(x632)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x631))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x631)*(x634)))+(((IkReal(-1.00000000000000))*(x633)*(x635)))+(((sj3)*(x629)))+(((sj1)*(x630)))+(((cj1)*(r00)*(sj2)))+(((cj2)*(x632))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x636=(sj3)*(sj3);
IkReal x637=(cj3)*(cj3);
IkReal x638=((r01)*(sj1));
IkReal x639=((cj1)*(r00));
IkReal x640=((IkReal(1.00000000000000))*(cj2));
IkReal x641=((cj1)*(r01));
IkReal x642=((r00)*(sj1));
IkReal x643=((cj3)*(sj2));
IkReal x644=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((((IKabs(((x636)+(x637))) != 0)?((IkReal)1/(((x636)+(x637)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x639)*(x644)))+(((cj2)*(sj3)*(x641)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(sj3)*(x640)*(x642)))+(((IkReal(-1.00000000000000))*(x638)*(x644))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x636)))+(((cj4)*(x637))))) != 0)?((IkReal)1/(((((cj4)*(x636)))+(((cj4)*(x637)))))):(IkReal)1.0e30))*(((((x638)*(x643)))+(((x639)*(x643)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x640)*(x641)))+(((cj2)*(cj3)*(x642))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x636)+(x637))) != 0)?((IkReal)1/(((x636)+(x637)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x639)*(x644)))+(((cj2)*(sj3)*(x641)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(sj3)*(x640)*(x642)))+(((IkReal(-1.00000000000000))*(x638)*(x644)))))))+IKsqr(((((IKabs(((((cj4)*(x636)))+(((cj4)*(x637))))) != 0)?((IkReal)1/(((((cj4)*(x636)))+(((cj4)*(x637)))))):(IkReal)1.0e30))*(((((x638)*(x643)))+(((x639)*(x643)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x640)*(x641)))+(((cj2)*(cj3)*(x642)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x636)+(x637))) != 0)?((IkReal)1/(((x636)+(x637)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x639)*(x644)))+(((cj2)*(sj3)*(x641)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(sj3)*(x640)*(x642)))+(((IkReal(-1.00000000000000))*(x638)*(x644)))))), ((((IKabs(((((cj4)*(x636)))+(((cj4)*(x637))))) != 0)?((IkReal)1/(((((cj4)*(x636)))+(((cj4)*(x637)))))):(IkReal)1.0e30))*(((((x638)*(x643)))+(((x639)*(x643)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x640)*(x641)))+(((cj2)*(cj3)*(x642)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[3];
IkReal x645=IKcos(j5);
IkReal x646=IKsin(j5);
IkReal x647=((r01)*(sj2));
IkReal x648=((IkReal(1.00000000000000))*(cj1));
IkReal x649=((r00)*(sj1));
IkReal x650=((IkReal(1.00000000000000))*(cj3));
IkReal x651=((cj2)*(r01));
IkReal x652=((cj4)*(x645));
evalcond[0]=((((IkReal(-1.00000000000000))*(x646)*(x650)))+(((IkReal(-1.00000000000000))*(sj3)*(x652)))+(r02));
evalcond[1]=((((sj4)*(x645)))+(((IkReal(-1.00000000000000))*(x647)*(x648)))+(((sj2)*(x649)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x648)))+(((IkReal(-1.00000000000000))*(sj1)*(x651))));
evalcond[2]=((((sj1)*(x647)))+(((cj2)*(x649)))+(((sj3)*(x646)))+(((IkReal(-1.00000000000000))*(x650)*(x652)))+(((IkReal(-1.00000000000000))*(x648)*(x651)))+(((cj1)*(r00)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "f39e879836615ea489f5611d5593ce3c"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
