/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:09:17.352891
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42;
x0=IKsin(j[3]);
x1=IKcos(j[0]);
x2=IKcos(j[4]);
x3=IKcos(j[1]);
x4=IKsin(j[2]);
x5=IKcos(j[3]);
x6=IKsin(j[4]);
x7=IKcos(j[2]);
x8=IKsin(j[1]);
x9=IKsin(j[5]);
x10=IKcos(j[5]);
x11=IKsin(j[0]);
x12=((IkReal(0.365000000000000))*(x8));
x13=((IkReal(1.00000000000000))*(x10));
x14=((x5)*(x6));
x15=((x7)*(x8));
x16=((x0)*(x9));
x17=((x2)*(x5));
x18=((x5)*(x9));
x19=((x3)*(x7));
x20=((x3)*(x4));
x21=((x4)*(x8));
x22=((IkReal(0.405000000000000))*(x21));
x23=((IkReal(0.0900000000000000))*(x20));
x24=((IkReal(0.405000000000000))*(x19));
x25=((IkReal(0.0900000000000000))*(x15));
x26=((IkReal(0.0800000000000000))*(x21));
x27=((IkReal(0.0800000000000000))*(x19));
x28=((x0)*(x10)*(x2));
x29=((x15)+(x20));
x30=((((IkReal(-1.00000000000000))*(x21)))+(x19));
x31=((((IkReal(-1.00000000000000))*(x19)))+(x21));
x32=((((IkReal(-1.00000000000000))*(x26)))+(x27));
x33=((((IkReal(-0.0800000000000000))*(x20)))+(((IkReal(-0.0800000000000000))*(x15))));
x34=((((IkReal(0.790000000000000))*(x20)))+(((IkReal(0.790000000000000))*(x15))));
x35=((x18)+(x28));
x36=((((IkReal(-1.00000000000000))*(x20)))+(((IkReal(-1.00000000000000))*(x15))));
x37=((x30)*(x6));
x38=((x2)*(x32));
x39=((((IkReal(-0.790000000000000))*(x18)))+(((IkReal(-0.790000000000000))*(x28)))+(((IkReal(-0.0800000000000000))*(x0)*(x6))));
x40=((x12)+(x24)+(x25)+(x23));
x41=((((x17)*(x29)))+(x37));
x42=((((IkReal(-0.790000000000000))*(x37)))+(((IkReal(-0.790000000000000))*(x17)*(x29))));
eetrans[0]=((((x11)*(x39)))+(((IkReal(0.0750000000000000))*(x1)))+(((x1)*(((((x10)*(x42)))+(((IkReal(-1.00000000000000))*(x22)))+(((x16)*(x34)))+(((x14)*(x33)))+(x38)+(x40))))));
eetrans[1]=((((x11)*(((((IkReal(-1.00000000000000))*(x38)))+(((IkReal(-1.00000000000000))*(x40)))+(((IkReal(-1.00000000000000))*(x14)*(x33)))+(((IkReal(-1.00000000000000))*(x16)*(x34)))+(x22)+(((IkReal(-1.00000000000000))*(x13)*(x42)))))))+(((IkReal(-0.0750000000000000))*(x11)))+(((x1)*(x39))));
eetrans[2]=((IkReal(0.335000000000000))+(((x14)*(((((IkReal(-1.00000000000000))*(x27)))+(x26)))))+(((x16)*(((((IkReal(0.790000000000000))*(x19)))+(((IkReal(-0.790000000000000))*(x21)))))))+(((IkReal(-0.405000000000000))*(x20)))+(((IkReal(-0.405000000000000))*(x15)))+(((x2)*(x33)))+(((IkReal(-0.0900000000000000))*(x21)))+(((IkReal(0.365000000000000))*(x3)))+(((x10)*(((((IkReal(0.790000000000000))*(x17)*(x31)))+(((IkReal(0.790000000000000))*(x29)*(x6)))))))+(((IkReal(0.0900000000000000))*(x19))));
eerot[0]=((((x1)*(((((x10)*(x41)))+(((x16)*(x36)))))))+(((x11)*(x35))));
eerot[1]=((((x11)*(((((IkReal(-1.00000000000000))*(x16)*(x36)))+(((IkReal(-1.00000000000000))*(x13)*(x41)))))))+(((x1)*(x35))));
eerot[2]=((((x16)*(x31)))+(((x10)*(((((IkReal(-1.00000000000000))*(x29)*(x6)))+(((IkReal(-1.00000000000000))*(x17)*(x31))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j0,cj0,sj0,htj0,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij0[2], _nj0;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij0[0] = -1; _ij0[1] = -1; _nj0 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=((((IkReal(-1.00000000000000))*(r01)*(IKsin(j0))))+(((r00)*(IKcos(j0)))));
IkReal x43=IKcos(j0);
IkReal x44=IKsin(j0);
new_px=((IkReal(-0.0750000000000000))+(((IkReal(-0.790000000000000))*(r00)*(x43)))+(((IkReal(-1.00000000000000))*(py)*(x44)))+(((px)*(x43)))+(((IkReal(0.790000000000000))*(r01)*(x44))));
new_r01=((IkReal(-1.00000000000000))*(r02));
new_py=((IkReal(0.335000000000000))+(((IkReal(0.790000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(pz))));
new_r02=((((r00)*(IKsin(j0))))+(((r01)*(IKcos(j0)))));
IkReal x45=IKsin(j0);
IkReal x46=IKcos(j0);
new_pz=((((IkReal(-0.790000000000000))*(r01)*(x46)))+(((py)*(x46)))+(((IkReal(-0.790000000000000))*(r00)*(x45)))+(((px)*(x45))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(-0.259450000000000))+(((IkReal(-0.810000000000000))*(px)))+(((IkReal(0.910000000000000))*(py)))+(pp));
IkReal gconst1;
gconst1=((IkReal(-0.591300000000000))+(((IkReal(-0.360000000000000))*(px)))+(((IkReal(-1.62000000000000))*(py))));
IkReal gconst2;
gconst2=((IkReal(-0.390850000000000))+(((IkReal(0.810000000000000))*(px)))+(pp)+(((IkReal(0.550000000000000))*(py))));
IkReal gconst3;
gconst3=((IkReal(0.790000000000000))+(((IkReal(-0.405000000000000))*(r00)))+(((IkReal(0.455000000000000))*(r01)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst4;
gconst4=((((IkReal(-0.810000000000000))*(r01)))+(((IkReal(-0.180000000000000))*(r00))));
IkReal gconst5;
gconst5=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(0.275000000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(r00))));
IkReal gconst6;
gconst6=((IkReal(-0.259450000000000))+(((IkReal(-0.810000000000000))*(px)))+(((IkReal(0.910000000000000))*(py)))+(pp));
IkReal gconst7;
gconst7=((IkReal(-0.591300000000000))+(((IkReal(-0.360000000000000))*(px)))+(((IkReal(-1.62000000000000))*(py))));
IkReal gconst8;
gconst8=((IkReal(-0.390850000000000))+(((IkReal(0.810000000000000))*(px)))+(pp)+(((IkReal(0.550000000000000))*(py))));
IkReal gconst9;
gconst9=((IkReal(0.790000000000000))+(((IkReal(-0.405000000000000))*(r00)))+(((IkReal(0.455000000000000))*(r01)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst10;
gconst10=((((IkReal(-0.810000000000000))*(r01)))+(((IkReal(-0.180000000000000))*(r00))));
IkReal gconst11;
gconst11=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(0.275000000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(r00))));
IkReal gconst12;
gconst12=((((IkReal(-1.82000000000000))*(px)))+(((IkReal(-1.62000000000000))*(py))));
IkReal gconst13;
gconst13=((((IkReal(-0.720000000000000))*(py)))+(((IkReal(3.24000000000000))*(px))));
IkReal gconst14;
gconst14=((((IkReal(-1.10000000000000))*(px)))+(((IkReal(1.62000000000000))*(py))));
IkReal gconst15;
gconst15=((((IkReal(-0.910000000000000))*(r00)))+(((IkReal(-0.810000000000000))*(r01))));
IkReal gconst16;
gconst16=((((IkReal(-0.360000000000000))*(r01)))+(((IkReal(1.62000000000000))*(r00))));
IkReal gconst17;
gconst17=((((IkReal(0.810000000000000))*(r01)))+(((IkReal(-0.550000000000000))*(r00))));
IkReal gconst18;
gconst18=((((IkReal(-1.82000000000000))*(px)))+(((IkReal(-1.62000000000000))*(py))));
IkReal gconst19;
gconst19=((((IkReal(-0.720000000000000))*(py)))+(((IkReal(3.24000000000000))*(px))));
IkReal gconst20;
gconst20=((((IkReal(-1.10000000000000))*(px)))+(((IkReal(1.62000000000000))*(py))));
IkReal gconst21;
gconst21=((((IkReal(-0.910000000000000))*(r00)))+(((IkReal(-0.810000000000000))*(r01))));
IkReal gconst22;
gconst22=((((IkReal(-0.360000000000000))*(r01)))+(((IkReal(1.62000000000000))*(r00))));
IkReal gconst23;
gconst23=((((IkReal(0.810000000000000))*(r01)))+(((IkReal(-0.550000000000000))*(r00))));
IkReal gconst24;
gconst24=((IkReal(-0.259450000000000))+(((IkReal(0.810000000000000))*(px)))+(pp)+(((IkReal(-0.910000000000000))*(py))));
IkReal gconst25;
gconst25=((IkReal(-0.591300000000000))+(((IkReal(0.360000000000000))*(px)))+(((IkReal(1.62000000000000))*(py))));
IkReal gconst26;
gconst26=((IkReal(-0.390850000000000))+(((IkReal(-0.550000000000000))*(py)))+(((IkReal(-0.810000000000000))*(px)))+(pp));
IkReal gconst27;
gconst27=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(-0.455000000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(r00))));
IkReal gconst28;
gconst28=((((IkReal(0.810000000000000))*(r01)))+(((IkReal(0.180000000000000))*(r00))));
IkReal gconst29;
gconst29=((IkReal(0.790000000000000))+(((IkReal(-0.405000000000000))*(r00)))+(((px)*(r00)))+(((IkReal(-0.275000000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst30;
gconst30=((IkReal(-0.259450000000000))+(((IkReal(0.810000000000000))*(px)))+(pp)+(((IkReal(-0.910000000000000))*(py))));
IkReal gconst31;
gconst31=((IkReal(-0.591300000000000))+(((IkReal(0.360000000000000))*(px)))+(((IkReal(1.62000000000000))*(py))));
IkReal gconst32;
gconst32=((IkReal(-0.390850000000000))+(((IkReal(-0.550000000000000))*(py)))+(((IkReal(-0.810000000000000))*(px)))+(pp));
IkReal gconst33;
gconst33=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(-0.455000000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(r00))));
IkReal gconst34;
gconst34=((((IkReal(0.810000000000000))*(r01)))+(((IkReal(0.180000000000000))*(r00))));
IkReal gconst35;
gconst35=((IkReal(0.790000000000000))+(((IkReal(-0.405000000000000))*(r00)))+(((px)*(r00)))+(((IkReal(-0.275000000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal x47=((IkReal(1.00000000000000))*(gconst26));
IkReal x48=((gconst30)*(gconst34));
IkReal x49=((gconst31)*(gconst33));
IkReal x50=((IkReal(1.00000000000000))*(gconst17));
IkReal x51=((gconst2)*(gconst6));
IkReal x52=((gconst11)*(gconst3));
IkReal x53=((gconst25)*(gconst29));
IkReal x54=((gconst14)*(gconst28));
IkReal x55=((IkReal(1.00000000000000))*(gconst14));
IkReal x56=((gconst21)*(gconst32));
IkReal x57=((gconst10)*(gconst6));
IkReal x58=((gconst21)*(gconst5));
IkReal x59=((gconst0)*(gconst8));
IkReal x60=((gconst13)*(gconst5));
IkReal x61=((IkReal(1.00000000000000))*(gconst9));
IkReal x62=((gconst30)*(gconst35));
IkReal x63=((gconst26)*(gconst27));
IkReal x64=((gconst22)*(gconst30));
IkReal x65=((IkReal(1.00000000000000))*(gconst12));
IkReal x66=((gconst2)*(gconst3));
IkReal x67=((gconst1)*(gconst5));
IkReal x68=((gconst19)*(gconst33));
IkReal x69=((gconst26)*(gconst28));
IkReal x70=((gconst11)*(gconst6));
IkReal x71=((gconst1)*(gconst7));
IkReal x72=((gconst18)*(gconst2));
IkReal x73=((gconst18)*(gconst34));
IkReal x74=((gconst24)*(gconst29));
IkReal x75=((IkReal(1.00000000000000))*(gconst23));
IkReal x76=((gconst20)*(gconst33));
IkReal x77=((gconst32)*(gconst33));
IkReal x78=((gconst0)*(gconst5));
IkReal x79=((IkReal(1.00000000000000))*(gconst18));
IkReal x80=((gconst21)*(gconst31));
IkReal x81=((gconst5)*(gconst9));
IkReal x82=((gconst4)*(gconst9));
IkReal x83=((gconst13)*(gconst29));
IkReal x84=((gconst14)*(gconst6));
IkReal x85=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x86=((gconst27)*(x77));
IkReal x87=((gconst2)*(gconst4)*(gconst7));
op[0]=((((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x48)))+(((IkReal(-1.00000000000000))*(x49)*(x53)))+(((IkReal(-1.00000000000000))*(x47)*(x86)))+(((x62)*(x63)))+(((x48)*(x53)))+(((IkReal(-1.00000000000000))*(x62)*(x74)))+(((x49)*(x69)))+(((x74)*(x77))));
op[1]=((((IkReal(-1.00000000000000))*(gconst25)*(x49)*(x50)))+(((IkReal(-1.00000000000000))*(gconst24)*(x50)*(x62)))+(((IkReal(-1.00000000000000))*(gconst15)*(x47)*(x77)))+(((IkReal(-1.00000000000000))*(x48)*(x54)))+(((IkReal(-1.00000000000000))*(gconst27)*(x47)*(x76)))+(((gconst16)*(gconst26)*(x49)))+(((x48)*(x83)))+(((IkReal(-1.00000000000000))*(x49)*(x83)))+(((IkReal(-1.00000000000000))*(gconst29)*(x62)*(x65)))+(((gconst15)*(gconst26)*(x62)))+(((IkReal(-1.00000000000000))*(x53)*(x80)))+(((IkReal(-1.00000000000000))*(gconst27)*(x47)*(x56)))+(((gconst17)*(gconst25)*(x48)))+(((x56)*(x74)))+(((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x73)))+(((IkReal(-1.00000000000000))*(gconst16)*(x47)*(x48)))+(((x68)*(x69)))+(((x69)*(x80)))+(((x53)*(x64)))+(((gconst12)*(gconst29)*(x77)))+(((x49)*(x54)))+(((gconst14)*(gconst27)*(x62)))+(((IkReal(-1.00000000000000))*(x53)*(x68)))+(((IkReal(-1.00000000000000))*(x55)*(x86)))+(((IkReal(-1.00000000000000))*(gconst35)*(x74)*(x79)))+(((gconst18)*(gconst35)*(x63)))+(((gconst17)*(gconst24)*(x77)))+(((x53)*(x73)))+(((IkReal(-1.00000000000000))*(gconst30)*(x74)*(x75)))+(((x74)*(x76)))+(((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x64)))+(((gconst23)*(gconst30)*(x63))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((IkReal(-1.00000000000000))*(gconst5)*(x65)*(x70)))+(((gconst22)*(gconst6)*(x67)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x51)))+(((gconst1)*(gconst17)*(x57)))+(((IkReal(-1.00000000000000))*(x72)*(x85)))+(((x58)*(x59)))+(((IkReal(-1.00000000000000))*(gconst6)*(x75)*(x78)))+(((gconst10)*(gconst18)*(x67)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x66)))+(((IkReal(-1.00000000000000))*(gconst19)*(x61)*(x67)))+(((gconst11)*(gconst15)*(x51)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x51)))+(((IkReal(-1.00000000000000))*(gconst0)*(x50)*(x70)))+(((IkReal(-1.00000000000000))*(gconst11)*(x78)*(x79)))+(((gconst23)*(gconst3)*(x51)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x55)))+(((x57)*(x60)))+(((gconst14)*(gconst7)*(x82)))+(((IkReal(-1.00000000000000))*(x58)*(x71)))+(((IkReal(-1.00000000000000))*(gconst7)*(x60)*(x61)))+(((x52)*(x84)))+(((IkReal(-1.00000000000000))*(gconst9)*(x50)*(x71)))+(((IkReal(-1.00000000000000))*(gconst20)*(x61)*(x66)))+(((gconst20)*(gconst9)*(x78)))+(((gconst21)*(x87)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst4)*(x55)*(x57)))+(((gconst12)*(gconst8)*(x81)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x61)))+(((x52)*(x72)))+(((gconst19)*(gconst2)*(x82)))+(((gconst17)*(gconst9)*(x59))));
op[8]=((((IkReal(-1.00000000000000))*(x51)*(x85)))+(((IkReal(-1.00000000000000))*(x70)*(x78)))+(((IkReal(-1.00000000000000))*(gconst7)*(x61)*(x67)))+(((IkReal(-1.00000000000000))*(gconst8)*(x61)*(x66)))+(((x57)*(x67)))+(((x51)*(x52)))+(((gconst2)*(gconst7)*(x82)))+(((x59)*(x81))));
polyroots8(op,zeror,numroots);
IkReal j1array[8], cj1array[8], sj1array[8], tempj1array[1];
int numsolutions = 0;
for(int ij1 = 0; ij1 < numroots; ++ij1)
{
IkReal htj1 = zeror[ij1];
tempj1array[0]=((IkReal(2.00000000000000))*(atan(htj1)));
for(int kj1 = 0; kj1 < 1; ++kj1)
{
j1array[numsolutions] = tempj1array[kj1];
if( j1array[numsolutions] > IKPI )
{
    j1array[numsolutions]-=IK2PI;
}
else if( j1array[numsolutions] < -IKPI )
{
    j1array[numsolutions]+=IK2PI;
}
sj1array[numsolutions] = IKsin(j1array[numsolutions]);
cj1array[numsolutions] = IKcos(j1array[numsolutions]);
numsolutions++;
}
}
bool j1valid[8]={true,true,true,true,true,true,true,true};
_nj1 = 8;
for(int ij1 = 0; ij1 < numsolutions; ++ij1)
    {
if( !j1valid[ij1] )
{
    continue;
}
    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
htj1 = IKtan(j1/2);

_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < numsolutions; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x88=(cj1)*(cj1);
IkReal x89=(sj1)*(sj1);
IkReal x90=((py)*(r00));
IkReal x91=((px)*(r01));
IkReal x92=((IkReal(0.344250000000000))*(x89));
IkReal x93=((IkReal(0.344250000000000))*(x88));
gconst52=IKsign(((((IkReal(-1.00000000000000))*(x90)*(x93)))+(((IkReal(-1.00000000000000))*(x90)*(x92)))+(((IkReal(-0.125651250000000))*(cj1)*(r00)))+(((IkReal(-0.125651250000000))*(r01)*(sj1)))+(((x91)*(x93)))+(((x91)*(x92)))));
IkReal x94=(cj1)*(cj1);
IkReal x95=(sj1)*(sj1);
IkReal x96=((IkReal(2.73972602739726))*(px)*(r01));
IkReal x97=((IkReal(2.73972602739726))*(py)*(r00));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x95)*(x97)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(x94)*(x97)))+(((x95)*(x96)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)))+(((x94)*(x96))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x98=(px)*(px);
IkReal x99=(sj1)*(sj1);
IkReal x100=(py)*(py);
IkReal x101=(cj1)*(cj1);
IkReal x102=((cj1)*(r01));
IkReal x103=((r01)*(sj1));
IkReal x104=((py)*(r00));
IkReal x105=((px)*(py));
IkReal x106=((cj1)*(py));
IkReal x107=((IkReal(0.810000000000000))*(sj1));
IkReal x108=((cj1)*(r00));
IkReal x109=((IkReal(0.295650000000000))*(r01));
IkReal x110=((IkReal(0.810000000000000))*(cj1));
IkReal x111=((IkReal(0.180000000000000))*(cj1));
IkReal x112=((px)*(r00));
IkReal x113=((r00)*(sj1));
IkReal x114=((IkReal(0.405000000000000))*(pp));
IkReal x115=((IkReal(0.0900000000000000))*(pp));
IkReal x116=((IkReal(0.180000000000000))*(sj1));
IkReal x117=((pz)*(r02));
IkReal x118=((IkReal(0.0657000000000000))*(r01));
IkReal x119=((cj1)*(px));
IkReal x120=((IkReal(0.142200000000000))*(sj1));
IkReal x121=((IkReal(0.639900000000000))*(sj1));
IkReal x122=((IkReal(0.295650000000000))*(x99));
IkReal x123=((IkReal(0.810000000000000))*(x100));
IkReal x124=((r00)*(x98));
IkReal x125=((px)*(x101));
IkReal x126=((px)*(x117));
IkReal x127=((IkReal(0.180000000000000))*(x100));
if( IKabs(((gconst52)*(((IkReal(-0.0519030000000000))+(((IkReal(0.639900000000000))*(x119)))+(((IkReal(-0.142200000000000))*(x106)))+(((IkReal(0.810000000000000))*(x102)*(x105)))+(((IkReal(0.0532440000000000))*(x113)))+(((IkReal(-1.00000000000000))*(x104)*(x122)))+(((IkReal(0.180000000000000))*(x103)*(x105)))+(((IkReal(-1.00000000000000))*(x113)*(x115)))+(((py)*(x121)))+(((x103)*(x123)))+(((IkReal(-0.295650000000000))*(x101)*(x104)))+(((px)*(x120)))+(((x102)*(x115)))+(((py)*(x107)*(x117)))+(((IkReal(-0.0532440000000000))*(x102)))+(((IkReal(-1.00000000000000))*(px)*(x104)*(x111)))+(((IkReal(-1.00000000000000))*(x108)*(x114)))+(((IkReal(-0.180000000000000))*(x106)*(x117)))+(((x109)*(x125)))+(((px)*(x104)*(x107)))+(((IkReal(-0.0657000000000000))*(x112)))+(((IkReal(-0.0657000000000000))*(x117)))+(((IkReal(0.180000000000000))*(x113)*(x98)))+(((x116)*(x126)))+(((IkReal(0.810000000000000))*(x108)*(x98)))+(((px)*(x109)*(x99)))+(((IkReal(-1.00000000000000))*(x103)*(x114)))+(((IkReal(-1.00000000000000))*(x102)*(x127)))+(((x110)*(x126)))+(((IkReal(0.131685750000000))*(x108)))+(((IkReal(0.131685750000000))*(x103)))+(((IkReal(-1.00000000000000))*(py)*(x118))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((IkReal(-0.233563500000000))+(((IkReal(-0.295650000000000))*(x117)))+(((IkReal(-0.295650000000000))*(x112)))+(((IkReal(-1.00000000000000))*(x118)*(x125)))+(((IkReal(-1.00000000000000))*(py)*(x120)))+(((IkReal(-0.180000000000000))*(x102)*(x105)))+(((IkReal(0.0657000000000000))*(x104)*(x99)))+(((IkReal(-1.00000000000000))*(px)*(x118)*(x99)))+(((IkReal(0.810000000000000))*(x103)*(x105)))+(((IkReal(-1.00000000000000))*(x113)*(x114)))+(((IkReal(-1.00000000000000))*(x111)*(x126)))+(((IkReal(0.239598000000000))*(x113)))+(((IkReal(-0.639900000000000))*(x106)))+(((px)*(x121)))+(((x102)*(x114)))+(((x103)*(x115)))+(((IkReal(-0.180000000000000))*(x108)*(x98)))+(((IkReal(-1.00000000000000))*(px)*(x104)*(x116)))+(((IkReal(-1.00000000000000))*(px)*(x104)*(x110)))+(((x108)*(x115)))+(((IkReal(-0.0292635000000000))*(x108)))+(((IkReal(-0.0292635000000000))*(x103)))+(((IkReal(-0.810000000000000))*(x106)*(x117)))+(((IkReal(-0.239598000000000))*(x102)))+(((IkReal(-1.00000000000000))*(py)*(x116)*(x117)))+(((IkReal(-1.00000000000000))*(x102)*(x123)))+(((IkReal(-1.00000000000000))*(x103)*(x127)))+(((x107)*(x124)))+(((x107)*(x126)))+(((IkReal(-1.00000000000000))*(py)*(x109)))+(((IkReal(0.0657000000000000))*(x101)*(x104)))+(((IkReal(-0.142200000000000))*(x119))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst52)*(((IkReal(-0.0519030000000000))+(((IkReal(0.639900000000000))*(x119)))+(((IkReal(-0.142200000000000))*(x106)))+(((IkReal(0.810000000000000))*(x102)*(x105)))+(((IkReal(0.0532440000000000))*(x113)))+(((IkReal(-1.00000000000000))*(x104)*(x122)))+(((IkReal(0.180000000000000))*(x103)*(x105)))+(((IkReal(-1.00000000000000))*(x113)*(x115)))+(((py)*(x121)))+(((x103)*(x123)))+(((IkReal(-0.295650000000000))*(x101)*(x104)))+(((px)*(x120)))+(((x102)*(x115)))+(((py)*(x107)*(x117)))+(((IkReal(-0.0532440000000000))*(x102)))+(((IkReal(-1.00000000000000))*(px)*(x104)*(x111)))+(((IkReal(-1.00000000000000))*(x108)*(x114)))+(((IkReal(-0.180000000000000))*(x106)*(x117)))+(((x109)*(x125)))+(((px)*(x104)*(x107)))+(((IkReal(-0.0657000000000000))*(x112)))+(((IkReal(-0.0657000000000000))*(x117)))+(((IkReal(0.180000000000000))*(x113)*(x98)))+(((x116)*(x126)))+(((IkReal(0.810000000000000))*(x108)*(x98)))+(((px)*(x109)*(x99)))+(((IkReal(-1.00000000000000))*(x103)*(x114)))+(((IkReal(-1.00000000000000))*(x102)*(x127)))+(((x110)*(x126)))+(((IkReal(0.131685750000000))*(x108)))+(((IkReal(0.131685750000000))*(x103)))+(((IkReal(-1.00000000000000))*(py)*(x118)))))), ((gconst52)*(((IkReal(-0.233563500000000))+(((IkReal(-0.295650000000000))*(x117)))+(((IkReal(-0.295650000000000))*(x112)))+(((IkReal(-1.00000000000000))*(x118)*(x125)))+(((IkReal(-1.00000000000000))*(py)*(x120)))+(((IkReal(-0.180000000000000))*(x102)*(x105)))+(((IkReal(0.0657000000000000))*(x104)*(x99)))+(((IkReal(-1.00000000000000))*(px)*(x118)*(x99)))+(((IkReal(0.810000000000000))*(x103)*(x105)))+(((IkReal(-1.00000000000000))*(x113)*(x114)))+(((IkReal(-1.00000000000000))*(x111)*(x126)))+(((IkReal(0.239598000000000))*(x113)))+(((IkReal(-0.639900000000000))*(x106)))+(((px)*(x121)))+(((x102)*(x114)))+(((x103)*(x115)))+(((IkReal(-0.180000000000000))*(x108)*(x98)))+(((IkReal(-1.00000000000000))*(px)*(x104)*(x116)))+(((IkReal(-1.00000000000000))*(px)*(x104)*(x110)))+(((x108)*(x115)))+(((IkReal(-0.0292635000000000))*(x108)))+(((IkReal(-0.0292635000000000))*(x103)))+(((IkReal(-0.810000000000000))*(x106)*(x117)))+(((IkReal(-0.239598000000000))*(x102)))+(((IkReal(-1.00000000000000))*(py)*(x116)*(x117)))+(((IkReal(-1.00000000000000))*(x102)*(x123)))+(((IkReal(-1.00000000000000))*(x103)*(x127)))+(((x107)*(x124)))+(((x107)*(x126)))+(((IkReal(-1.00000000000000))*(py)*(x109)))+(((IkReal(0.0657000000000000))*(x101)*(x104)))+(((IkReal(-0.142200000000000))*(x119)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
IkReal x128=IKcos(j2);
IkReal x129=IKsin(j2);
IkReal x130=((cj1)*(py));
IkReal x131=((px)*(sj1));
IkReal x132=((r00)*(sj1));
IkReal x133=((cj1)*(px));
IkReal x134=((py)*(sj1));
IkReal x135=((r01)*(sj1));
IkReal x136=((cj1)*(r01));
IkReal x137=((cj1)*(r00));
IkReal x138=((IkReal(0.180000000000000))*(x128));
IkReal x139=((IkReal(0.810000000000000))*(x129));
IkReal x140=((IkReal(0.0900000000000000))*(x128));
IkReal x141=((IkReal(0.810000000000000))*(x128));
IkReal x142=((IkReal(0.180000000000000))*(x129));
IkReal x143=((IkReal(0.405000000000000))*(x128));
IkReal x144=((IkReal(0.0900000000000000))*(x129));
IkReal x145=((IkReal(0.405000000000000))*(x129));
evalcond[0]=((IkReal(-0.325150000000000))+(((IkReal(-1.00000000000000))*(x134)*(x141)))+(((IkReal(-1.00000000000000))*(x134)*(x142)))+(((IkReal(0.730000000000000))*(x130)))+(((IkReal(-1.00000000000000))*(x133)*(x142)))+(((IkReal(-1.00000000000000))*(x133)*(x141)))+(((x131)*(x139)))+(((x130)*(x138)))+(pp)+(((IkReal(-0.295650000000000))*(x129)))+(((IkReal(-1.00000000000000))*(x131)*(x138)))+(((IkReal(-1.00000000000000))*(x130)*(x139)))+(((IkReal(-0.730000000000000))*(x131)))+(((IkReal(0.0657000000000000))*(x128))));
evalcond[1]=((IkReal(0.790000000000000))+(((IkReal(-1.00000000000000))*(x137)*(x144)))+(((IkReal(-1.00000000000000))*(x137)*(x143)))+(((IkReal(-1.00000000000000))*(x132)*(x140)))+(((px)*(r00)))+(((x136)*(x140)))+(((IkReal(-0.365000000000000))*(x132)))+(((x132)*(x145)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.365000000000000))*(x136)))+(((IkReal(-1.00000000000000))*(x136)*(x145)))+(((IkReal(-1.00000000000000))*(x135)*(x143)))+(((IkReal(-1.00000000000000))*(x135)*(x144))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x146=((IkReal(12.5000000000000))*(cj2));
IkReal x147=((IkReal(9.87500000000000))*(cj1));
IkReal x148=((IkReal(9.87500000000000))*(sj1));
IkReal x149=((IkReal(12.5000000000000))*(sj2));
cj4array[0]=((IkReal(-5.06250000000000))+(((cj2)*(r00)*(x147)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x148)))+(((cj2)*(r01)*(x148)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x149)))+(((cj1)*(px)*(x146)))+(((IkReal(4.56250000000000))*(sj2)))+(((cj1)*(py)*(x149)))+(((py)*(sj1)*(x146)))+(((r01)*(sj2)*(x147))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x150=((IkReal(1.00000000000000))*(r01));
IkReal x151=((cj1)*(sj2));
IkReal x152=((sj1)*(sj2));
IkReal x153=((cj2)*(sj1));
IkReal x154=((IkReal(1.00000000000000))*(py));
IkReal x155=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x152)))+(((IkReal(-1.00000000000000))*(x150)*(x153)))+(((IkReal(-1.00000000000000))*(x150)*(x151)))+(((IkReal(-1.00000000000000))*(r00)*(x155))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x153)*(x154)))+(((IkReal(-1.00000000000000))*(x151)*(x154)))+(((IkReal(-1.00000000000000))*(px)*(x155)))+(((px)*(x152))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=r02;
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=r02;
IkReal gconst64;
gconst64=pz;
IkReal gconst65;
gconst65=IkReal(1.58000000000000);
IkReal gconst66;
gconst66=pz;
IkReal gconst67;
gconst67=r02;
IkReal gconst68;
gconst68=IkReal(-2.00000000000000);
IkReal gconst69;
gconst69=r02;
IkReal gconst70;
gconst70=pz;
IkReal gconst71;
gconst71=IkReal(1.58000000000000);
IkReal gconst72;
gconst72=pz;
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(1.58000000000000);
IkReal gconst76;
gconst76=IkReal(-1.58000000000000);
IkReal gconst77;
gconst77=IkReal(-2.00000000000000);
IkReal gconst78;
gconst78=IkReal(2.00000000000000);
IkReal gconst79;
gconst79=IkReal(1.58000000000000);
IkReal gconst80;
gconst80=IkReal(-1.58000000000000);
IkReal gconst81;
gconst81=r02;
IkReal gconst82;
gconst82=IkReal(2.00000000000000);
IkReal gconst83;
gconst83=r02;
IkReal gconst84;
gconst84=pz;
IkReal gconst85;
gconst85=IkReal(-1.58000000000000);
IkReal gconst86;
gconst86=pz;
IkReal gconst87;
gconst87=r02;
IkReal gconst88;
gconst88=IkReal(2.00000000000000);
IkReal gconst89;
gconst89=r02;
IkReal gconst90;
gconst90=pz;
IkReal gconst91;
gconst91=IkReal(-1.58000000000000);
IkReal gconst92;
gconst92=pz;
IkReal x156=((gconst89)*(gconst90));
IkReal x157=((gconst83)*(gconst84));
IkReal x158=((gconst81)*(gconst86));
IkReal x159=((gconst87)*(gconst92));
IkReal x160=((gconst82)*(gconst86));
IkReal x161=((gconst87)*(gconst91));
IkReal x162=((gconst88)*(gconst90));
IkReal x163=((gconst83)*(gconst85));
dummyeval[0]=((((x157)*(x159)))+(((x162)*(x163)))+(((IkReal(-1.00000000000000))*(x158)*(x159)))+(((IkReal(-1.00000000000000))*(x156)*(x157)))+(((IkReal(-1.00000000000000))*(x161)*(x163)))+(((x156)*(x158)))+(((IkReal(-1.00000000000000))*(x160)*(x162)))+(((x160)*(x161))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x164=((gconst63)*(gconst67));
IkReal x165=((IkReal(1.00000000000000))*(gconst65));
IkReal x166=((IkReal(1.00000000000000))*(gconst86));
IkReal x167=((gconst80)*(gconst87));
IkReal x168=((gconst68)*(gconst82));
IkReal x169=((gconst72)*(gconst75));
IkReal x170=((gconst72)*(gconst77));
IkReal x171=((gconst61)*(gconst76));
IkReal x172=((gconst63)*(gconst87));
IkReal x173=((gconst61)*(gconst66));
IkReal x174=((gconst70)*(gconst78));
IkReal x175=((gconst86)*(gconst89));
IkReal x176=((gconst73)*(gconst90));
IkReal x177=((gconst64)*(gconst72));
IkReal x178=((gconst87)*(gconst92));
IkReal x179=((IkReal(1.00000000000000))*(gconst83));
IkReal x180=((gconst85)*(gconst91));
IkReal x181=((gconst69)*(gconst90));
IkReal x182=((gconst78)*(gconst90));
IkReal x183=((IkReal(1.00000000000000))*(gconst70));
IkReal x184=((gconst62)*(gconst66));
IkReal x185=((gconst62)*(gconst86));
IkReal x186=((gconst67)*(gconst71));
IkReal x187=((gconst72)*(gconst84));
IkReal x188=((gconst65)*(gconst68));
IkReal x189=((gconst74)*(gconst79));
IkReal x190=((gconst63)*(gconst69));
IkReal x191=((IkReal(1.00000000000000))*(gconst67));
IkReal x192=((gconst66)*(gconst73));
IkReal x193=((gconst70)*(gconst81));
IkReal x194=((gconst63)*(gconst70));
IkReal x195=((gconst79)*(gconst81));
IkReal x196=((gconst78)*(gconst86));
IkReal x197=((gconst84)*(gconst92));
IkReal x198=((IkReal(1.00000000000000))*(gconst79));
IkReal x199=((gconst83)*(gconst84));
IkReal x200=((gconst75)*(gconst83));
IkReal x201=((gconst64)*(gconst74));
IkReal x202=((gconst81)*(gconst92));
IkReal x203=((gconst63)*(gconst64));
IkReal x204=((gconst76)*(gconst89));
IkReal x205=((gconst65)*(gconst88));
IkReal x206=((gconst79)*(gconst84));
IkReal x207=((gconst87)*(gconst91));
IkReal x208=((gconst81)*(gconst86));
IkReal x209=((gconst69)*(gconst79));
IkReal x210=((gconst67)*(gconst83));
IkReal x211=((gconst82)*(gconst88));
IkReal x212=((gconst69)*(gconst70));
IkReal x213=((gconst67)*(gconst74));
IkReal x214=((gconst70)*(gconst89));
IkReal x215=((IkReal(1.00000000000000))*(gconst77));
IkReal x216=((IkReal(1.00000000000000))*(gconst81));
IkReal x217=((gconst68)*(gconst85));
IkReal x218=((gconst83)*(gconst90));
IkReal x219=((gconst62)*(gconst68));
IkReal x220=((gconst82)*(gconst86));
IkReal x221=((gconst67)*(gconst91));
IkReal x222=((IkReal(1.00000000000000))*(gconst63));
IkReal x223=((gconst85)*(gconst88));
IkReal x224=((gconst74)*(gconst84));
IkReal x225=((gconst89)*(gconst90));
IkReal x226=((gconst72)*(gconst87));
IkReal x227=((gconst73)*(gconst76));
IkReal x228=((gconst66)*(gconst81));
IkReal x229=((gconst63)*(gconst90));
IkReal x230=((gconst63)*(gconst77));
IkReal x231=((gconst74)*(gconst75));
IkReal x232=((gconst66)*(gconst82));
IkReal x233=((gconst71)*(gconst87));
IkReal x234=((IkReal(1.00000000000000))*(gconst90));
IkReal x235=((gconst71)*(gconst77));
IkReal x236=((gconst77)*(gconst80));
IkReal x237=((gconst76)*(gconst82));
IkReal x238=((gconst77)*(gconst91));
IkReal x239=((gconst70)*(gconst83));
IkReal x240=((gconst62)*(gconst76));
IkReal x241=((gconst81)*(gconst90));
IkReal x242=((gconst77)*(gconst92));
IkReal x243=((gconst74)*(x223));
op[0]=((((IkReal(-1.00000000000000))*(gconst87)*(x179)*(x180)))+(((x207)*(x220)))+(((IkReal(-1.00000000000000))*(gconst84)*(x179)*(x225)))+(((x178)*(x199)))+(((IkReal(-1.00000000000000))*(gconst81)*(x166)*(x178)))+(((x218)*(x223)))+(((IkReal(-1.00000000000000))*(gconst90)*(x166)*(x211)))+(((x175)*(x241))));
op[1]=((((IkReal(-1.00000000000000))*(gconst73)*(x166)*(x178)))+(((gconst79)*(gconst83)*(x223)))+(((IkReal(-1.00000000000000))*(gconst76)*(x211)*(x234)))+(((IkReal(-1.00000000000000))*(gconst77)*(x179)*(x180)))+(((IkReal(-1.00000000000000))*(gconst81)*(x166)*(x167)))+(((x175)*(x195)))+(((gconst90)*(x243)))+(((IkReal(-1.00000000000000))*(gconst79)*(x166)*(x211)))+(((x167)*(x199)))+(((x175)*(x176)))+(((IkReal(-1.00000000000000))*(gconst76)*(x178)*(x216)))+(((x178)*(x200)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst87)*(x180)))+(((gconst77)*(gconst83)*(x197)))+(((IkReal(-1.00000000000000))*(gconst75)*(x179)*(x225)))+(((IkReal(-1.00000000000000))*(gconst89)*(x179)*(x206)))+(((IkReal(-1.00000000000000))*(gconst77)*(x166)*(x202)))+(((x204)*(x241)))+(((IkReal(-1.00000000000000))*(x224)*(x225)))+(((x178)*(x224)))+(((x182)*(x208)))+(((x220)*(x238)))+(((x207)*(x237)))+(((IkReal(-1.00000000000000))*(gconst84)*(x179)*(x182))));
op[2]=((((IkReal(-1.00000000000000))*(gconst67)*(x179)*(x180)))+(((x176)*(x196)))+(((IkReal(-1.00000000000000))*(gconst73)*(x166)*(x167)))+(((IkReal(-1.00000000000000))*(gconst83)*(x165)*(x207)))+(((IkReal(-1.00000000000000))*(gconst84)*(x179)*(x214)))+(((x237)*(x238)))+(((IkReal(-1.00000000000000))*(gconst76)*(x167)*(x216)))+(((IkReal(-1.00000000000000))*(gconst81)*(x166)*(x226)))+(((IkReal(-1.00000000000000))*(gconst64)*(x179)*(x225)))+(((x225)*(x228)))+(((x189)*(x223)))+(((gconst76)*(gconst81)*(x182)))+(((x197)*(x210)))+(((IkReal(-1.00000000000000))*(x178)*(x227)))+(((IkReal(-1.00000000000000))*(gconst84)*(x222)*(x225)))+(((IkReal(-1.00000000000000))*(gconst78)*(x179)*(x206)))+(((gconst73)*(gconst79)*(x175)))+(((x172)*(x197)))+(((x195)*(x204)))+(((x167)*(x224)))+(((x175)*(x193)))+(((x185)*(x207)))+(((x205)*(x218)))+(((IkReal(-1.00000000000000))*(gconst61)*(x166)*(x178)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst88)*(gconst90)*(x166)))+(((x223)*(x229)))+(((IkReal(-1.00000000000000))*(gconst81)*(x166)*(x236)))+(((IkReal(-1.00000000000000))*(gconst75)*(x179)*(x182)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst89)*(x179)))+(((IkReal(-1.00000000000000))*(gconst76)*(x202)*(x215)))+(((gconst61)*(gconst90)*(x175)))+(((x220)*(x221)))+(((gconst74)*(gconst77)*(x197)))+(((x195)*(x196)))+(((IkReal(-1.00000000000000))*(x172)*(x180)))+(((IkReal(-1.00000000000000))*(gconst73)*(x166)*(x242)))+(((x178)*(x231)))+(((IkReal(-1.00000000000000))*(gconst74)*(x180)*(x215)))+(((IkReal(-1.00000000000000))*(gconst76)*(x198)*(x211)))+(((IkReal(-1.00000000000000))*(gconst66)*(x178)*(x216)))+(((IkReal(-1.00000000000000))*(gconst70)*(x166)*(x211)))+(((IkReal(-1.00000000000000))*(x182)*(x224)))+(((IkReal(-1.00000000000000))*(gconst66)*(x211)*(x234)))+(((x176)*(x204)))+(((gconst64)*(gconst83)*(x178)))+(((x217)*(x218)))+(((IkReal(-1.00000000000000))*(gconst84)*(gconst89)*(x189)))+(((gconst83)*(gconst87)*(x187)))+(((x181)*(x208)))+(((x167)*(x200)))+(((x220)*(x233)))+(((x207)*(x232)))+(((IkReal(-1.00000000000000))*(gconst90)*(x166)*(x168)))+(((IkReal(-1.00000000000000))*(gconst84)*(x179)*(x181)))+(((IkReal(-1.00000000000000))*(gconst85)*(x179)*(x233)))+(((x223)*(x239)))+(((IkReal(-1.00000000000000))*(gconst67)*(x166)*(x202)))+(((x199)*(x236)))+(((x200)*(x242)))+(((IkReal(-1.00000000000000))*(x225)*(x231))));
op[3]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst80)*(gconst87)))+(((gconst75)*(gconst77)*(gconst80)*(gconst83)))+(((gconst71)*(gconst77)*(gconst82)*(gconst86)))+(((gconst65)*(gconst74)*(gconst88)*(gconst90)))+(((gconst67)*(gconst80)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst89)*(gconst90)))+(((gconst61)*(gconst78)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst79)*(gconst82)*(gconst86)))+(((gconst61)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst77)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst81)*(gconst92)))+(((gconst63)*(gconst79)*(gconst85)*(gconst88)))+(((gconst62)*(gconst76)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst83)*(gconst90)))+(((gconst64)*(gconst74)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst80)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst79)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst88)*(gconst90)))+(((gconst71)*(gconst76)*(gconst82)*(gconst87)))+(((gconst70)*(gconst74)*(gconst85)*(gconst88)))+(((gconst74)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst82)*(gconst90)))+(((gconst74)*(gconst75)*(gconst80)*(gconst87)))+(((gconst62)*(gconst77)*(gconst86)*(gconst91)))+(((gconst69)*(gconst76)*(gconst81)*(gconst90)))+(((gconst66)*(gconst79)*(gconst81)*(gconst89)))+(((gconst69)*(gconst73)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst80)*(gconst86)*(gconst87)))+(((gconst66)*(gconst77)*(gconst82)*(gconst91)))+(((gconst70)*(gconst73)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst79)*(gconst86)*(gconst88)))+(((gconst74)*(gconst77)*(gconst80)*(gconst84)))+(((gconst70)*(gconst78)*(gconst81)*(gconst86)))+(((gconst63)*(gconst75)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst79)*(gconst83)*(gconst89)))+(((gconst68)*(gconst74)*(gconst85)*(gconst90)))+(((gconst64)*(gconst77)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst92)))+(((gconst65)*(gconst79)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst82)*(gconst88)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst86)*(gconst87)))+(((gconst72)*(gconst77)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst86)*(gconst92)))+(((gconst69)*(gconst79)*(gconst81)*(gconst86)))+(((gconst67)*(gconst76)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst83)*(gconst90)))+(((gconst61)*(gconst76)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst84)*(gconst89)))+(((gconst72)*(gconst75)*(gconst83)*(gconst87)))+(((gconst63)*(gconst80)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst78)*(gconst79)*(gconst83)))+(((gconst63)*(gconst77)*(gconst84)*(gconst92)))+(((gconst73)*(gconst78)*(gconst79)*(gconst86)))+(((gconst70)*(gconst76)*(gconst81)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst78)*(gconst83)*(gconst84)))+(((gconst66)*(gconst73)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst83)*(gconst89)))+(((gconst68)*(gconst79)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst74)*(gconst85)*(gconst91)))+(((gconst73)*(gconst76)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst77)*(gconst83)*(gconst91)))+(((gconst67)*(gconst74)*(gconst84)*(gconst92)))+(((gconst72)*(gconst74)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst84)*(gconst89)))+(((gconst76)*(gconst78)*(gconst79)*(gconst81)))+(((gconst66)*(gconst78)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst87)*(gconst92)))+(((gconst73)*(gconst76)*(gconst79)*(gconst89)))+(((gconst67)*(gconst75)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst80)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst78)*(gconst79)*(gconst84))));
op[4]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst83)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst87)*(gconst92)))+(((gconst64)*(gconst72)*(gconst83)*(gconst87)))+(((gconst64)*(gconst74)*(gconst80)*(gconst87)))+(((gconst63)*(gconst70)*(gconst85)*(gconst88)))+(((gconst68)*(gconst70)*(gconst83)*(gconst85)))+(((gconst67)*(gconst74)*(gconst80)*(gconst84)))+(((gconst73)*(gconst76)*(gconst78)*(gconst79)))+(((gconst64)*(gconst67)*(gconst83)*(gconst92)))+(((gconst67)*(gconst72)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst70)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst79)*(gconst83)))+(((gconst64)*(gconst74)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst90)))+(((gconst71)*(gconst76)*(gconst77)*(gconst82)))+(((gconst70)*(gconst73)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst84)*(gconst89)))+(((gconst61)*(gconst66)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst92)))+(((gconst62)*(gconst76)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst77)*(gconst81)))+(((gconst61)*(gconst69)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst80)))+(((gconst66)*(gconst70)*(gconst81)*(gconst89)))+(((gconst61)*(gconst70)*(gconst86)*(gconst89)))+(((gconst63)*(gconst64)*(gconst87)*(gconst92)))+(((gconst61)*(gconst78)*(gconst79)*(gconst86)))+(((gconst68)*(gconst74)*(gconst79)*(gconst85)))+(((gconst67)*(gconst75)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst86)*(gconst87)))+(((gconst66)*(gconst69)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst79)*(gconst88)))+(((gconst62)*(gconst66)*(gconst87)*(gconst91)))+(((gconst70)*(gconst76)*(gconst78)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst79)*(gconst81)))+(((gconst66)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst82)*(gconst86)))+(((gconst65)*(gconst70)*(gconst83)*(gconst88)))+(((gconst67)*(gconst71)*(gconst82)*(gconst86)))+(((gconst63)*(gconst65)*(gconst88)*(gconst90)))+(((gconst63)*(gconst75)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst87)*(gconst91)))+(((gconst63)*(gconst68)*(gconst85)*(gconst90)))+(((gconst70)*(gconst73)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst87)))+(((gconst61)*(gconst76)*(gconst78)*(gconst90)))+(((gconst72)*(gconst74)*(gconst75)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst84)*(gconst90)))+(((gconst62)*(gconst71)*(gconst86)*(gconst87)))+(((gconst69)*(gconst73)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst80)*(gconst81)))+(((gconst72)*(gconst75)*(gconst77)*(gconst83)))+(((gconst69)*(gconst70)*(gconst81)*(gconst86)))+(((gconst65)*(gconst68)*(gconst83)*(gconst90)))+(((gconst63)*(gconst67)*(gconst84)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst92)))+(((gconst74)*(gconst75)*(gconst77)*(gconst80)))+(((gconst63)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst80)*(gconst86)))+(((gconst66)*(gconst73)*(gconst78)*(gconst90)))+(((gconst66)*(gconst73)*(gconst79)*(gconst89)))+(((gconst61)*(gconst76)*(gconst79)*(gconst89)))+(((gconst72)*(gconst74)*(gconst77)*(gconst84)))+(((gconst66)*(gconst78)*(gconst79)*(gconst81)))+(((gconst65)*(gconst74)*(gconst79)*(gconst88)))+(((gconst64)*(gconst77)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst88)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst79)*(gconst89)))+(((gconst66)*(gconst67)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst79)*(gconst89)))+(((gconst62)*(gconst67)*(gconst86)*(gconst91)))+(((gconst69)*(gconst73)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst77)*(gconst86)))+(((gconst63)*(gconst77)*(gconst80)*(gconst84)))+(((gconst63)*(gconst72)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst83)*(gconst87))));
op[5]=((((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst79)*(gconst88)))+(((gconst64)*(gconst72)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst91)))+(((gconst61)*(gconst66)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst66)*(gconst70)*(gconst78)*(gconst81)))+(((gconst66)*(gconst73)*(gconst78)*(gconst79)))+(((gconst63)*(gconst75)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst79)))+(((gconst63)*(gconst72)*(gconst75)*(gconst87)))+(((gconst63)*(gconst67)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst92)))+(((gconst69)*(gconst70)*(gconst76)*(gconst81)))+(((gconst64)*(gconst74)*(gconst77)*(gconst80)))+(((gconst61)*(gconst70)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((gconst68)*(gconst70)*(gconst74)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst79)*(gconst84)))+(((gconst63)*(gconst64)*(gconst77)*(gconst92)))+(((gconst63)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst80)*(gconst86)))+(((gconst63)*(gconst72)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst76)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst80)))+(((gconst69)*(gconst70)*(gconst73)*(gconst86)))+(((gconst65)*(gconst68)*(gconst79)*(gconst83)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst65)*(gconst68)*(gconst74)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst76)*(gconst82)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((gconst66)*(gconst70)*(gconst73)*(gconst89)))+(((gconst72)*(gconst74)*(gconst75)*(gconst77)))+(((gconst69)*(gconst73)*(gconst76)*(gconst79)))+(((gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst62)*(gconst66)*(gconst77)*(gconst91)))+(((gconst62)*(gconst67)*(gconst76)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst74)*(gconst91)))+(((gconst65)*(gconst70)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((gconst61)*(gconst70)*(gconst78)*(gconst86)))+(((gconst61)*(gconst69)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst75)*(gconst89)))+(((gconst64)*(gconst67)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst79)*(gconst86)))+(((gconst61)*(gconst66)*(gconst79)*(gconst89)))+(((gconst67)*(gconst72)*(gconst74)*(gconst84)))+(((gconst63)*(gconst68)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst61)*(gconst69)*(gconst79)*(gconst86)))+(((gconst61)*(gconst76)*(gconst78)*(gconst79)))+(((gconst67)*(gconst72)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst80)))+(((gconst66)*(gconst69)*(gconst79)*(gconst81)))+(((gconst63)*(gconst67)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst74)*(gconst85)))+(((gconst70)*(gconst73)*(gconst76)*(gconst78)))+(((gconst63)*(gconst65)*(gconst79)*(gconst88)))+(((gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst64)*(gconst67)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst87)))+(((gconst64)*(gconst72)*(gconst74)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst90)))+(((gconst66)*(gconst69)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst74)*(gconst87))));
op[6]=((((IkReal(-1.00000000000000))*(x170)*(x171)))+(((IkReal(-1.00000000000000))*(gconst83)*(x165)*(x186)))+(((x186)*(x232)))+(((IkReal(-1.00000000000000))*(x173)*(x226)))+(((IkReal(-1.00000000000000))*(gconst80)*(x171)*(x191)))+(((IkReal(-1.00000000000000))*(gconst78)*(x198)*(x203)))+(((x169)*(x213)))+(((IkReal(-1.00000000000000))*(gconst69)*(x183)*(x231)))+(((x171)*(x209)))+(((IkReal(-1.00000000000000))*(gconst68)*(x184)*(x234)))+(((x184)*(x221)))+(((IkReal(-1.00000000000000))*(gconst92)*(x173)*(x191)))+(((x188)*(x229)))+(((IkReal(-1.00000000000000))*(gconst72)*(x191)*(x227)))+(((IkReal(-1.00000000000000))*(gconst72)*(x191)*(x228)))+(((x173)*(x181)))+(((IkReal(-1.00000000000000))*(x170)*(x192)))+(((IkReal(-1.00000000000000))*(gconst74)*(x165)*(x235)))+(((IkReal(-1.00000000000000))*(gconst84)*(x183)*(x190)))+(((x194)*(x205)))+(((IkReal(-1.00000000000000))*(x174)*(x201)))+(((x192)*(x209)))+(((gconst75)*(gconst80)*(x164)))+(((x170)*(x201)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(x189)))+(((x177)*(x210)))+(((x188)*(x239)))+(((x172)*(x177)))+(((x185)*(x186)))+(((x184)*(x233)))+(((x164)*(x187)))+(((x188)*(x189)))+(((gconst64)*(gconst92)*(x164)))+(((x174)*(x192)))+(((IkReal(-1.00000000000000))*(gconst88)*(x183)*(x184)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst85)*(x164)))+(((x173)*(x214)))+(((IkReal(-1.00000000000000))*(gconst89)*(x183)*(x203)))+(((IkReal(-1.00000000000000))*(gconst64)*(x179)*(x212)))+(((x171)*(x174)))+(((x203)*(x236)))+(((x169)*(x230)))+(((gconst66)*(gconst69)*(x193)))+(((gconst61)*(gconst86)*(x212)))+(((IkReal(-1.00000000000000))*(gconst76)*(x198)*(x219)))+(((IkReal(-1.00000000000000))*(gconst71)*(x165)*(x172)))+(((gconst67)*(gconst80)*(x201)))+(((IkReal(-1.00000000000000))*(gconst75)*(x174)*(x222)))+(((IkReal(-1.00000000000000))*(gconst66)*(x168)*(x183)))+(((IkReal(-1.00000000000000))*(gconst70)*(x166)*(x219)))+(((IkReal(-1.00000000000000))*(gconst80)*(x173)*(x215)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(x166)))+(((IkReal(-1.00000000000000))*(gconst91)*(x164)*(x165)))+(((gconst78)*(gconst79)*(x173)))+(((x194)*(x217)))+(((x235)*(x240)))+(((IkReal(-1.00000000000000))*(x181)*(x203)))+(((IkReal(-1.00000000000000))*(gconst80)*(x191)*(x192)))+(((IkReal(-1.00000000000000))*(gconst75)*(x190)*(x198)))+(((x212)*(x227))));
op[7]=((((IkReal(-1.00000000000000))*(gconst69)*(x183)*(x201)))+(((IkReal(-1.00000000000000))*(x170)*(x173)))+(((gconst63)*(gconst79)*(x188)))+(((gconst70)*(gconst74)*(x188)))+(((IkReal(-1.00000000000000))*(x174)*(x203)))+(((IkReal(-1.00000000000000))*(gconst72)*(x191)*(x192)))+(((x170)*(x203)))+(((x177)*(x213)))+(((IkReal(-1.00000000000000))*(gconst72)*(x171)*(x191)))+(((x173)*(x174)))+(((x171)*(x212)))+(((IkReal(-1.00000000000000))*(gconst64)*(x190)*(x198)))+(((x184)*(x235)))+(((IkReal(-1.00000000000000))*(gconst76)*(x183)*(x219)))+(((IkReal(-1.00000000000000))*(gconst71)*(x165)*(x230)))+(((x164)*(x169)))+(((IkReal(-1.00000000000000))*(gconst68)*(x184)*(x198)))+(((gconst64)*(gconst80)*(x164)))+(((IkReal(-1.00000000000000))*(gconst75)*(x183)*(x190)))+(((x186)*(x240)))+(((IkReal(-1.00000000000000))*(gconst74)*(x165)*(x186)))+(((x173)*(x209)))+(((x192)*(x212)))+(((IkReal(-1.00000000000000))*(gconst80)*(x173)*(x191))));
op[8]=((((IkReal(-1.00000000000000))*(gconst64)*(x183)*(x190)))+(((x188)*(x194)))+(((IkReal(-1.00000000000000))*(gconst68)*(x183)*(x184)))+(((x184)*(x186)))+(((x173)*(x212)))+(((x164)*(x177)))+(((IkReal(-1.00000000000000))*(gconst71)*(x164)*(x165)))+(((IkReal(-1.00000000000000))*(gconst72)*(x173)*(x191))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst109;
gconst109=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x244=((r01)*(sj1));
IkReal x245=((cj1)*(r00));
IkReal x246=((IkReal(1.00000000000000))*(cj2));
IkReal x247=((cj1)*(r01));
IkReal x248=((r00)*(sj1));
IkReal x249=((cj3)*(sj2));
IkReal x250=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst109)*(((((IkReal(-1.00000000000000))*(sj3)*(x246)*(x248)))+(((IkReal(-1.00000000000000))*(x244)*(x250)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x245)*(x250)))+(((cj2)*(sj3)*(x247))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst109)*(((((x244)*(x249)))+(((IkReal(-1.00000000000000))*(cj3)*(x246)*(x247)))+(((r02)*(sj3)))+(((cj2)*(cj3)*(x248)))+(((x245)*(x249))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst109)*(((((IkReal(-1.00000000000000))*(sj3)*(x246)*(x248)))+(((IkReal(-1.00000000000000))*(x244)*(x250)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x245)*(x250)))+(((cj2)*(sj3)*(x247)))))), ((gconst109)*(((((x244)*(x249)))+(((IkReal(-1.00000000000000))*(cj3)*(x246)*(x247)))+(((r02)*(sj3)))+(((cj2)*(cj3)*(x248)))+(((x245)*(x249)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x251=IKsin(j5);
IkReal x252=IKcos(j5);
IkReal x253=((sj1)*(sj2));
IkReal x254=((IkReal(0.790000000000000))*(cj3));
IkReal x255=((IkReal(1.00000000000000))*(cj3));
IkReal x256=((cj2)*(sj1));
IkReal x257=((cj1)*(sj2));
IkReal x258=((IkReal(0.790000000000000))*(sj3));
IkReal x259=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj3)*(x252)))+(((IkReal(-1.00000000000000))*(x251)*(x255)))+(r02));
evalcond[1]=((((x251)*(x254)))+(pz)+(((x252)*(x258))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x252)*(x255)))+(((r00)*(x257)))+(((r00)*(x256)))+(((r01)*(x253)))+(((IkReal(-1.00000000000000))*(r01)*(x259)))+(((sj3)*(x251))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x251)*(x258)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x253)))+(((px)*(x257)))+(((px)*(x256)))+(((IkReal(-1.00000000000000))*(py)*(x259)))+(((x252)*(x254))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x260=((IkReal(1.00000000000000))*(r01));
IkReal x261=((cj1)*(sj2));
IkReal x262=((sj1)*(sj2));
IkReal x263=((cj2)*(sj1));
IkReal x264=((IkReal(1.00000000000000))*(py));
IkReal x265=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x265)))+(((r00)*(x262)))+(((IkReal(-1.00000000000000))*(x260)*(x261)))+(((IkReal(-1.00000000000000))*(x260)*(x263))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x263)*(x264)))+(((IkReal(-1.00000000000000))*(x261)*(x264)))+(((IkReal(-1.00000000000000))*(px)*(x265)))+(((px)*(x262))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=r02;
IkReal gconst111;
gconst111=IkReal(-2.00000000000000);
IkReal gconst112;
gconst112=r02;
IkReal gconst113;
gconst113=pz;
IkReal gconst114;
gconst114=IkReal(1.58000000000000);
IkReal gconst115;
gconst115=pz;
IkReal gconst116;
gconst116=r02;
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=r02;
IkReal gconst119;
gconst119=pz;
IkReal gconst120;
gconst120=IkReal(1.58000000000000);
IkReal gconst121;
gconst121=pz;
IkReal gconst122;
gconst122=IkReal(2.00000000000000);
IkReal gconst123;
gconst123=IkReal(-2.00000000000000);
IkReal gconst124;
gconst124=IkReal(-1.58000000000000);
IkReal gconst125;
gconst125=IkReal(1.58000000000000);
IkReal gconst126;
gconst126=IkReal(2.00000000000000);
IkReal gconst127;
gconst127=IkReal(-2.00000000000000);
IkReal gconst128;
gconst128=IkReal(-1.58000000000000);
IkReal gconst129;
gconst129=IkReal(1.58000000000000);
IkReal gconst130;
gconst130=r02;
IkReal gconst131;
gconst131=IkReal(2.00000000000000);
IkReal gconst132;
gconst132=r02;
IkReal gconst133;
gconst133=pz;
IkReal gconst134;
gconst134=IkReal(-1.58000000000000);
IkReal gconst135;
gconst135=pz;
IkReal gconst136;
gconst136=r02;
IkReal gconst137;
gconst137=IkReal(2.00000000000000);
IkReal gconst138;
gconst138=r02;
IkReal gconst139;
gconst139=pz;
IkReal gconst140;
gconst140=IkReal(-1.58000000000000);
IkReal gconst141;
gconst141=pz;
IkReal x266=((gconst132)*(gconst134));
IkReal x267=((gconst137)*(gconst139));
IkReal x268=((IkReal(1.00000000000000))*(gconst136));
IkReal x269=((gconst130)*(gconst135));
IkReal x270=((gconst132)*(gconst133));
IkReal x271=((gconst138)*(gconst139));
IkReal x272=((gconst131)*(gconst135));
dummyeval[0]=((((x266)*(x267)))+(((gconst136)*(gconst141)*(x270)))+(((gconst136)*(gconst140)*(x272)))+(((IkReal(-1.00000000000000))*(gconst140)*(x266)*(x268)))+(((IkReal(-1.00000000000000))*(gconst141)*(x268)*(x269)))+(((x269)*(x271)))+(((IkReal(-1.00000000000000))*(x267)*(x272)))+(((IkReal(-1.00000000000000))*(x270)*(x271))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x273=((gconst118)*(gconst119));
IkReal x274=((gconst110)*(gconst135));
IkReal x275=((gconst114)*(gconst117));
IkReal x276=((gconst123)*(gconst128));
IkReal x277=((gconst136)*(gconst141));
IkReal x278=((gconst113)*(gconst132));
IkReal x279=((IkReal(1.00000000000000))*(gconst124));
IkReal x280=((IkReal(1.00000000000000))*(gconst135));
IkReal x281=((gconst138)*(gconst139));
IkReal x282=((gconst131)*(gconst139));
IkReal x283=((gconst112)*(gconst116));
IkReal x284=((gconst124)*(gconst129));
IkReal x285=((gconst115)*(gconst122));
IkReal x286=((IkReal(1.00000000000000))*(gconst140));
IkReal x287=((gconst112)*(gconst139));
IkReal x288=((gconst121)*(gconst124));
IkReal x289=((gconst129)*(gconst136));
IkReal x290=((gconst132)*(gconst136));
IkReal x291=((gconst120)*(gconst126));
IkReal x292=((IkReal(1.00000000000000))*(gconst123));
IkReal x293=((gconst131)*(gconst137));
IkReal x294=((gconst116)*(gconst121));
IkReal x295=((IkReal(1.00000000000000))*(gconst116));
IkReal x296=((gconst131)*(gconst135));
IkReal x297=((gconst126)*(gconst140));
IkReal x298=((gconst111)*(gconst115));
IkReal x299=((IkReal(1.00000000000000))*(gconst125));
IkReal x300=((gconst116)*(gconst120));
IkReal x301=((gconst111)*(gconst135));
IkReal x302=((gconst113)*(gconst121));
IkReal x303=((gconst130)*(gconst138));
IkReal x304=((gconst126)*(gconst141));
IkReal x305=((IkReal(1.00000000000000))*(gconst119));
IkReal x306=((gconst132)*(gconst133));
IkReal x307=((gconst112)*(gconst133));
IkReal x308=((gconst110)*(gconst115));
IkReal x309=((gconst127)*(gconst139));
IkReal x310=((gconst123)*(gconst133));
IkReal x311=((IkReal(1.00000000000000))*(gconst120));
IkReal x312=((gconst132)*(gconst134));
IkReal x313=((gconst122)*(gconst125));
IkReal x314=((gconst125)*(gconst128));
IkReal x315=((gconst134)*(gconst137));
IkReal x316=((gconst116)*(gconst123));
IkReal x317=((gconst120)*(gconst136));
IkReal x318=((IkReal(1.00000000000000))*(gconst118));
IkReal x319=((gconst112)*(gconst113));
IkReal x320=((gconst111)*(gconst117));
IkReal x321=((IkReal(1.00000000000000))*(gconst121));
IkReal x322=((gconst124)*(gconst132));
IkReal x323=((gconst137)*(gconst139));
IkReal x324=((gconst114)*(gconst132));
IkReal x325=((gconst112)*(gconst119));
IkReal x326=((IkReal(1.00000000000000))*(gconst128));
IkReal x327=((gconst123)*(gconst134));
IkReal x328=((gconst111)*(gconst125));
IkReal x329=((gconst130)*(gconst135));
IkReal x330=((IkReal(1.00000000000000))*(gconst115));
IkReal x331=((gconst118)*(gconst139));
IkReal x332=((gconst116)*(gconst140));
IkReal x333=((gconst117)*(gconst139));
IkReal x334=((gconst112)*(gconst128));
IkReal x335=((gconst126)*(gconst129));
IkReal x336=((gconst127)*(gconst128));
IkReal x337=((gconst116)*(gconst141));
IkReal x338=((gconst115)*(gconst131));
IkReal x339=((gconst125)*(gconst131));
IkReal x340=((gconst136)*(gconst140));
IkReal x341=((gconst119)*(gconst127));
IkReal x342=((gconst113)*(gconst129));
IkReal x343=((gconst118)*(gconst128));
IkReal x344=((gconst128)*(gconst135));
IkReal x345=((gconst112)*(gconst136));
IkReal x346=((gconst110)*(gconst125));
IkReal x347=((gconst115)*(gconst130));
IkReal x348=((gconst121)*(gconst133));
IkReal x349=((gconst125)*(gconst130));
IkReal x350=((gconst122)*(gconst135));
IkReal x351=((gconst112)*(x302));
IkReal x352=((gconst126)*(gconst130)*(x280));
op[0]=((((IkReal(-1.00000000000000))*(gconst130)*(x277)*(x280)))+(((IkReal(-1.00000000000000))*(gconst137)*(x280)*(x282)))+(((IkReal(-1.00000000000000))*(gconst134)*(x286)*(x290)))+(((x281)*(x329)))+(((x296)*(x340)))+(((IkReal(-1.00000000000000))*(x281)*(x306)))+(((x312)*(x323)))+(((x277)*(x306))));
op[1]=((((IkReal(-1.00000000000000))*(gconst138)*(x306)*(x326)))+(((IkReal(-1.00000000000000))*(gconst128)*(x280)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x304)))+(((IkReal(-1.00000000000000))*(gconst126)*(x286)*(x312)))+(((x281)*(x350)))+(((IkReal(-1.00000000000000))*(gconst137)*(x282)*(x299)))+(((IkReal(-1.00000000000000))*(gconst130)*(x277)*(x299)))+(((IkReal(-1.00000000000000))*(gconst132)*(x279)*(x281)))+(((IkReal(-1.00000000000000))*(gconst136)*(x286)*(x327)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x289)))+(((x309)*(x329)))+(((IkReal(-1.00000000000000))*(x306)*(x309)))+(((x296)*(x297)))+(((x289)*(x306)))+(((x277)*(x322)))+(((IkReal(-1.00000000000000))*(gconst122)*(x277)*(x280)))+(((gconst128)*(gconst137)*(x312)))+(((x304)*(x306)))+(((x303)*(x344)))+(((x281)*(x349)))+(((x339)*(x340)))+(((x277)*(x310)))+(((IkReal(-1.00000000000000))*(gconst133)*(x281)*(x292)))+(((gconst123)*(gconst139)*(x315))));
op[2]=((((x276)*(x315)))+(((x289)*(x310)))+(((gconst123)*(gconst124)*(x277)))+(((x304)*(x310)))+(((x323)*(x324)))+(((x301)*(x340)))+(((IkReal(-1.00000000000000))*(gconst119)*(x280)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x277)*(x330)))+(((IkReal(-1.00000000000000))*(x274)*(x277)))+(((IkReal(-1.00000000000000))*(gconst117)*(x280)*(x282)))+(((IkReal(-1.00000000000000))*(gconst133)*(x292)*(x309)))+(((x304)*(x322)))+(((IkReal(-1.00000000000000))*(gconst138)*(x305)*(x306)))+(((gconst122)*(gconst138)*(x344)))+(((x296)*(x317)))+(((x274)*(x281)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x335)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x337)))+(((IkReal(-1.00000000000000))*(gconst123)*(x279)*(x281)))+(((IkReal(-1.00000000000000))*(gconst122)*(x277)*(x299)))+(((IkReal(-1.00000000000000))*(gconst126)*(x286)*(x327)))+(((IkReal(-1.00000000000000))*(gconst128)*(gconst132)*(gconst138)*(x279)))+(((IkReal(-1.00000000000000))*(x281)*(x307)))+(((IkReal(-1.00000000000000))*(gconst134)*(x286)*(x345)))+(((gconst119)*(gconst135)*(x303)))+(((x284)*(x290)))+(((IkReal(-1.00000000000000))*(gconst134)*(x290)*(x311)))+(((IkReal(-1.00000000000000))*(gconst127)*(x306)*(x326)))+(((IkReal(-1.00000000000000))*(x278)*(x281)))+(((IkReal(-1.00000000000000))*(gconst116)*(x286)*(x312)))+(((IkReal(-1.00000000000000))*(gconst130)*(x299)*(x304)))+(((x329)*(x331)))+(((x329)*(x336)))+(((x281)*(x313)))+(((x277)*(x278)))+(((IkReal(-1.00000000000000))*(gconst130)*(x289)*(x299)))+(((IkReal(-1.00000000000000))*(gconst139)*(x306)*(x318)))+(((x309)*(x349)))+(((IkReal(-1.00000000000000))*(gconst122)*(x280)*(x304)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst130)*(gconst136)*(x280)))+(((IkReal(-1.00000000000000))*(gconst111)*(x280)*(x323)))+(((IkReal(-1.00000000000000))*(gconst114)*(x286)*(x290)))+(((gconst119)*(gconst137)*(x312)))+(((x338)*(x340)))+(((x287)*(x315)))+(((x297)*(x339)))+(((x309)*(x350)))+(((x277)*(x307)))+(((x303)*(x314)))+(((x281)*(x347)))+(((x312)*(x333)))+(((x306)*(x337)))+(((x306)*(x335)))+(((x296)*(x332)))+(((IkReal(-1.00000000000000))*(gconst133)*(gconst138)*(x276)))+(((x290)*(x348)))+(((IkReal(-1.00000000000000))*(gconst137)*(x282)*(x330)))+(((IkReal(-1.00000000000000))*(gconst132)*(x279)*(x309)))+(((IkReal(-1.00000000000000))*(gconst122)*(x280)*(x289)))+(((IkReal(-1.00000000000000))*(gconst128)*(x293)*(x299))));
op[3]=((((gconst121)*(gconst124)*(gconst132)*(gconst136)))+(((gconst115)*(gconst122)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst126)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst128)*(gconst131)*(gconst137)))+(((gconst122)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst129)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst126)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst134)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst132)*(gconst139)))+(((gconst120)*(gconst125)*(gconst131)*(gconst136)))+(((gconst113)*(gconst123)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst126)*(gconst130)*(gconst135)))+(((gconst117)*(gconst128)*(gconst132)*(gconst134)))+(((gconst112)*(gconst128)*(gconst134)*(gconst137)))+(((gconst119)*(gconst122)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst127)*(gconst128)*(gconst133)))+(((gconst119)*(gconst125)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst128)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst128)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst127)*(gconst132)*(gconst133)))+(((gconst114)*(gconst128)*(gconst132)*(gconst137)))+(((gconst122)*(gconst125)*(gconst128)*(gconst138)))+(((gconst116)*(gconst123)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst123)*(gconst134)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst138)*(gconst139)))+(((gconst115)*(gconst128)*(gconst130)*(gconst138)))+(((gconst112)*(gconst126)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst129)*(gconst135)*(gconst136)))+(((gconst123)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst129)*(gconst136)))+(((gconst116)*(gconst125)*(gconst131)*(gconst140)))+(((gconst119)*(gconst127)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst128)*(gconst132)*(gconst133)))+(((gconst124)*(gconst126)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst126)*(gconst129)*(gconst135)))+(((gconst115)*(gconst126)*(gconst131)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst132)*(gconst138)))+(((gconst118)*(gconst128)*(gconst130)*(gconst135)))+(((gconst121)*(gconst126)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst125)*(gconst131)*(gconst137)))+(((gconst121)*(gconst123)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst128)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst141)))+(((gconst119)*(gconst123)*(gconst134)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst130)*(gconst141)))+(((gconst120)*(gconst126)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst130)*(gconst141)))+(((gconst112)*(gconst124)*(gconst136)*(gconst141)))+(((gconst116)*(gconst129)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst124)*(gconst127)*(gconst128)*(gconst132)))+(((gconst110)*(gconst127)*(gconst135)*(gconst139)))+(((gconst110)*(gconst125)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst131)*(gconst139)))+(((gconst113)*(gconst126)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst133)*(gconst139)))+(((gconst115)*(gconst127)*(gconst130)*(gconst139)))+(((gconst117)*(gconst123)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst126)*(gconst134)*(gconst140)))+(((gconst122)*(gconst127)*(gconst128)*(gconst135)))+(((gconst112)*(gconst129)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst136)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst132)*(gconst139)))+(((gconst111)*(gconst125)*(gconst136)*(gconst140)))+(((gconst118)*(gconst125)*(gconst130)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst135)*(gconst136)))+(((gconst113)*(gconst129)*(gconst132)*(gconst136)))+(((gconst118)*(gconst122)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst136)*(gconst141)))+(((gconst123)*(gconst124)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst125)*(gconst126)*(gconst129)*(gconst130)))+(((gconst125)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst129)*(gconst130)*(gconst136)))+(((gconst116)*(gconst124)*(gconst132)*(gconst141)))+(((gconst111)*(gconst126)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst128)*(gconst131)*(gconst135)))+(((gconst110)*(gconst128)*(gconst135)*(gconst138)))+(((gconst123)*(gconst124)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst130)*(gconst136))));
op[4]=((((gconst117)*(gconst123)*(gconst128)*(gconst134)))+(((gconst112)*(gconst116)*(gconst133)*(gconst141)))+(((gconst119)*(gconst122)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst139)))+(((gconst111)*(gconst120)*(gconst135)*(gconst136)))+(((gconst116)*(gconst123)*(gconst124)*(gconst141)))+(((gconst116)*(gconst124)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst128)*(gconst138)))+(((gconst113)*(gconst121)*(gconst132)*(gconst136)))+(((gconst115)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst130)*(gconst135)))+(((gconst111)*(gconst125)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst126)*(gconst140)))+(((gconst123)*(gconst124)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst141)))+(((gconst110)*(gconst125)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst126)*(gconst134)))+(((gconst115)*(gconst116)*(gconst131)*(gconst140)))+(((gconst111)*(gconst116)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst141)))+(((gconst112)*(gconst117)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst129)*(gconst136)))+(((gconst113)*(gconst126)*(gconst129)*(gconst132)))+(((gconst120)*(gconst125)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst135)*(gconst141)))+(((gconst118)*(gconst122)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst128)*(gconst131)))+(((gconst110)*(gconst119)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst132)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst129)*(gconst135)))+(((gconst111)*(gconst115)*(gconst136)*(gconst140)))+(((gconst119)*(gconst125)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst134)*(gconst136)))+(((gconst112)*(gconst124)*(gconst129)*(gconst136)))+(((gconst112)*(gconst114)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst127)*(gconst132)))+(((gconst115)*(gconst122)*(gconst127)*(gconst139)))+(((gconst118)*(gconst119)*(gconst130)*(gconst135)))+(((gconst110)*(gconst118)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst126)*(gconst130)))+(((gconst121)*(gconst123)*(gconst126)*(gconst133)))+(((gconst116)*(gconst123)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst133)*(gconst138)))+(((gconst113)*(gconst123)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst132)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst129)*(gconst135)))+(((gconst110)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst135)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst116)*(gconst134)*(gconst140)))+(((gconst112)*(gconst119)*(gconst134)*(gconst137)))+(((gconst113)*(gconst123)*(gconst126)*(gconst141)))+(((gconst112)*(gconst121)*(gconst133)*(gconst136)))+(((gconst114)*(gconst119)*(gconst132)*(gconst137)))+(((gconst115)*(gconst118)*(gconst130)*(gconst139)))+(((gconst117)*(gconst119)*(gconst132)*(gconst134)))+(((gconst110)*(gconst115)*(gconst138)*(gconst139)))+(((gconst116)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst129)))+(((gconst121)*(gconst123)*(gconst124)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst131)*(gconst139)))+(((gconst112)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst130)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst128)*(gconst138)))+(((gconst112)*(gconst113)*(gconst136)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst138)))+(((gconst110)*(gconst127)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst135)*(gconst139)))+(((gconst121)*(gconst124)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst128)))+(((gconst118)*(gconst125)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst119)*(gconst131)*(gconst137)))+(((gconst115)*(gconst122)*(gconst128)*(gconst138)))+(((gconst114)*(gconst117)*(gconst132)*(gconst139)))+(((gconst116)*(gconst121)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst127)*(gconst133)))+(((gconst113)*(gconst116)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst137)*(gconst139)))+(((gconst112)*(gconst124)*(gconst126)*(gconst141)))+(((gconst115)*(gconst120)*(gconst131)*(gconst136)))+(((gconst115)*(gconst119)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst136)*(gconst140)))+(((gconst122)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst126)*(gconst135))));
op[5]=((((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst125)*(gconst137)))+(((gconst112)*(gconst121)*(gconst124)*(gconst136)))+(((gconst117)*(gconst119)*(gconst123)*(gconst134)))+(((gconst112)*(gconst124)*(gconst126)*(gconst129)))+(((gconst115)*(gconst119)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst122)*(gconst136)))+(((gconst110)*(gconst118)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst141)))+(((gconst110)*(gconst118)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((gconst112)*(gconst113)*(gconst129)*(gconst136)))+(((gconst114)*(gconst117)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst132)))+(((gconst112)*(gconst117)*(gconst128)*(gconst134)))+(((gconst115)*(gconst118)*(gconst122)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst126)))+(((gconst114)*(gconst119)*(gconst123)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst123)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst126)*(gconst141)))+(((gconst118)*(gconst119)*(gconst122)*(gconst135)))+(((gconst110)*(gconst115)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst122)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst129)))+(((gconst115)*(gconst118)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst125)*(gconst128)))+(((gconst112)*(gconst116)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst122)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst125)*(gconst131)))+(((gconst113)*(gconst123)*(gconst126)*(gconst129)))+(((gconst110)*(gconst119)*(gconst125)*(gconst138)))+(((gconst116)*(gconst121)*(gconst124)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst123)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst123)*(gconst134)))+(((gconst116)*(gconst121)*(gconst123)*(gconst133)))+(((gconst110)*(gconst115)*(gconst128)*(gconst138)))+(((gconst113)*(gconst116)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst128)*(gconst135)))+(((gconst115)*(gconst122)*(gconst127)*(gconst128)))+(((gconst110)*(gconst119)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst128)*(gconst138)))+(((gconst112)*(gconst114)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst128)*(gconst133)))+(((gconst110)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst129)))+(((gconst113)*(gconst121)*(gconst123)*(gconst136)))+(((gconst113)*(gconst116)*(gconst123)*(gconst141)))+(((gconst111)*(gconst115)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst129)*(gconst130)))+(((gconst115)*(gconst119)*(gconst122)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst124)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst128)*(gconst137)))+(((gconst112)*(gconst113)*(gconst126)*(gconst141)))+(((gconst114)*(gconst117)*(gconst123)*(gconst139)))+(((gconst118)*(gconst119)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst127)))+(((gconst121)*(gconst123)*(gconst124)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst124)*(gconst132)))+(((gconst113)*(gconst121)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((gconst112)*(gconst116)*(gconst124)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst127)))+(((gconst116)*(gconst123)*(gconst124)*(gconst129)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((gconst112)*(gconst121)*(gconst126)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst128)))+(((gconst111)*(gconst116)*(gconst125)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128))));
op[6]=((((IkReal(-1.00000000000000))*(x273)*(x278)))+(((gconst114)*(gconst137)*(x325)))+(((IkReal(-1.00000000000000))*(gconst113)*(x276)*(x318)))+(((x273)*(x274)))+(((IkReal(-1.00000000000000))*(gconst113)*(x287)*(x318)))+(((x285)*(x341)))+(((x285)*(x343)))+(((IkReal(-1.00000000000000))*(gconst113)*(x292)*(x341)))+(((IkReal(-1.00000000000000))*(gconst129)*(x285)*(x295)))+(((x291)*(x328)))+(((x275)*(x276)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(x299)))+(((IkReal(-1.00000000000000))*(gconst122)*(x294)*(x299)))+(((x273)*(x313)))+(((IkReal(-1.00000000000000))*(gconst114)*(x283)*(x286)))+(((IkReal(-1.00000000000000))*(gconst126)*(x285)*(x321)))+(((IkReal(-1.00000000000000))*(gconst114)*(x311)*(x345)))+(((IkReal(-1.00000000000000))*(gconst117)*(x305)*(x338)))+(((gconst123)*(gconst126)*(x302)))+(((IkReal(-1.00000000000000))*(gconst134)*(x283)*(x311)))+(((x319)*(x335)))+(((IkReal(-1.00000000000000))*(gconst137)*(x298)*(x305)))+(((IkReal(-1.00000000000000))*(gconst119)*(x280)*(x320)))+(((x302)*(x345)))+(((IkReal(-1.00000000000000))*(gconst120)*(x295)*(x324)))+(((x298)*(x317)))+(((IkReal(-1.00000000000000))*(gconst136)*(x308)*(x321)))+(((gconst117)*(gconst134)*(x325)))+(((x273)*(x347)))+(((x308)*(x331)))+(((x308)*(x336)))+(((IkReal(-1.00000000000000))*(x298)*(x333)))+(((IkReal(-1.00000000000000))*(gconst128)*(x299)*(x320)))+(((x278)*(x294)))+(((IkReal(-1.00000000000000))*(gconst127)*(x319)*(x326)))+(((gconst119)*(gconst132)*(x275)))+(((x275)*(x287)))+(((gconst112)*(gconst126)*(x288)))+(((x341)*(x346)))+(((gconst113)*(gconst141)*(x283)))+(((x298)*(x332)))+(((IkReal(-1.00000000000000))*(gconst130)*(x294)*(x330)))+(((IkReal(-1.00000000000000))*(gconst123)*(x273)*(x279)))+(((gconst119)*(gconst138)*(x308)))+(((IkReal(-1.00000000000000))*(gconst114)*(x291)*(x292)))+(((IkReal(-1.00000000000000))*(x308)*(x335)))+(((gconst110)*(gconst118)*(x314)))+(((x300)*(x301)))+(((IkReal(-1.00000000000000))*(x274)*(x294)))+(((IkReal(-1.00000000000000))*(gconst141)*(x295)*(x308)))+(((IkReal(-1.00000000000000))*(x273)*(x307)))+(((IkReal(-1.00000000000000))*(gconst138)*(x305)*(x319)))+(((x316)*(x342)))+(((x283)*(x348)))+(((IkReal(-1.00000000000000))*(gconst118)*(x279)*(x334)))+(((IkReal(-1.00000000000000))*(gconst129)*(x295)*(x346)))+(((IkReal(-1.00000000000000))*(gconst127)*(x279)*(x325)))+(((x300)*(x338)))+(((x283)*(x284)))+(((x288)*(x316))));
op[7]=((((gconst113)*(gconst123)*(x294)))+(((x291)*(x298)))+(((IkReal(-1.00000000000000))*(gconst113)*(x273)*(x292)))+(((IkReal(-1.00000000000000))*(gconst128)*(x318)*(x319)))+(((IkReal(-1.00000000000000))*(gconst126)*(x308)*(x321)))+(((IkReal(-1.00000000000000))*(gconst110)*(x294)*(x299)))+(((IkReal(-1.00000000000000))*(gconst119)*(x299)*(x320)))+(((x273)*(x346)))+(((IkReal(-1.00000000000000))*(gconst129)*(x295)*(x308)))+(((IkReal(-1.00000000000000))*(gconst112)*(x273)*(x279)))+(((x273)*(x285)))+(((IkReal(-1.00000000000000))*(gconst114)*(x292)*(x300)))+(((IkReal(-1.00000000000000))*(gconst127)*(x305)*(x319)))+(((IkReal(-1.00000000000000))*(x285)*(x294)))+(((gconst126)*(x351)))+(((IkReal(-1.00000000000000))*(gconst117)*(x298)*(x326)))+(((x308)*(x343)))+(((x308)*(x341)))+(((x275)*(x334)))+(((x283)*(x342)))+(((x300)*(x328)))+(((x283)*(x288)))+(((gconst119)*(gconst123)*(x275)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(x291))));
op[8]=((((IkReal(-1.00000000000000))*(x273)*(x319)))+(((IkReal(-1.00000000000000))*(gconst114)*(x283)*(x311)))+(((x283)*(x302)))+(((IkReal(-1.00000000000000))*(x294)*(x308)))+(((x275)*(x325)))+(((x298)*(x300)))+(((IkReal(-1.00000000000000))*(gconst117)*(x298)*(x305)))+(((x273)*(x308))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst158;
gconst158=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x353=((r01)*(sj1));
IkReal x354=((cj1)*(r00));
IkReal x355=((IkReal(1.00000000000000))*(cj3));
IkReal x356=((IkReal(1.00000000000000))*(sj3));
IkReal x357=((sj2)*(x356));
IkReal x358=((cj2)*(r00)*(sj1));
IkReal x359=((cj1)*(cj2)*(r01));
if( IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(x356)*(x358)))+(((sj3)*(x359)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x353)*(x357)))+(((IkReal(-1.00000000000000))*(x354)*(x357))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst158)*(((((cj3)*(x359)))+(((IkReal(-1.00000000000000))*(sj2)*(x354)*(x355)))+(((IkReal(-1.00000000000000))*(sj2)*(x353)*(x355)))+(((IkReal(-1.00000000000000))*(r02)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x358))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst158)*(((((IkReal(-1.00000000000000))*(x356)*(x358)))+(((sj3)*(x359)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x353)*(x357)))+(((IkReal(-1.00000000000000))*(x354)*(x357)))))), ((gconst158)*(((((cj3)*(x359)))+(((IkReal(-1.00000000000000))*(sj2)*(x354)*(x355)))+(((IkReal(-1.00000000000000))*(sj2)*(x353)*(x355)))+(((IkReal(-1.00000000000000))*(r02)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x358)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x360=IKsin(j5);
IkReal x361=IKcos(j5);
IkReal x362=((sj1)*(sj2));
IkReal x363=((IkReal(0.790000000000000))*(cj3));
IkReal x364=((cj2)*(sj1));
IkReal x365=((cj1)*(sj2));
IkReal x366=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x367=((sj3)*(x361));
IkReal x368=((sj3)*(x360));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj3)*(x360)))+(x367)+(r02));
evalcond[1]=((((IkReal(-0.790000000000000))*(x367)))+(pz)+(((x360)*(x363))));
evalcond[2]=((((r01)*(x362)))+(((IkReal(-1.00000000000000))*(r01)*(x366)))+(((r00)*(x365)))+(((r00)*(x364)))+(x368)+(((cj3)*(x361))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x368)))+(((IkReal(-1.00000000000000))*(x361)*(x363)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x362)))+(((px)*(x365)))+(((px)*(x364)))+(((IkReal(-1.00000000000000))*(py)*(x366))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x369=((IkReal(1.00000000000000))*(cj1));
cj5array[0]=((IkReal(1.26582278481013))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x369)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(sj1)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x369)))+(((IkReal(0.0800000000000000))*(cj4))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x370=((IkReal(1.00000000000000))*(cj2));
evalcond[0]=((((sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x370)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x370)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x371=((IkReal(1.00000000000000))*(r01));
IkReal x372=((cj1)*(sj2));
IkReal x373=((sj1)*(sj2));
IkReal x374=((cj2)*(sj1));
IkReal x375=((IkReal(1.00000000000000))*(py));
IkReal x376=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x376)))+(((r00)*(x373)))+(((IkReal(-1.00000000000000))*(x371)*(x374)))+(((IkReal(-1.00000000000000))*(x371)*(x372))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x372)*(x375)))+(((px)*(x373)))+(((IkReal(-1.00000000000000))*(x374)*(x375)))+(((IkReal(-1.00000000000000))*(px)*(x376))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x377=((IkReal(1.00000000000000))*(cj2));
IkReal x378=((cj1)*(cj5));
IkReal x379=((sj1)*(sj5));
IkReal x380=((IkReal(1.00000000000000))*(sj2));
IkReal x381=((cj1)*(sj5));
IkReal x382=((cj5)*(sj1));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(r00)*(x377)*(x379)))+(((IkReal(-1.00000000000000))*(r00)*(x380)*(x381)))+(((IkReal(-1.00000000000000))*(r01)*(x379)*(x380)))+(((cj2)*(r01)*(x381)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((r00)*(sj2)*(x378)))+(((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(r01)*(x377)*(x378)))+(((cj2)*(r00)*(x382)))+(((r01)*(sj2)*(x382))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(r00)*(x377)*(x379)))+(((IkReal(-1.00000000000000))*(r00)*(x380)*(x381)))+(((IkReal(-1.00000000000000))*(r01)*(x379)*(x380)))+(((cj2)*(r01)*(x381)))+(((cj5)*(r02)))))), ((gconst55)*(((((r00)*(sj2)*(x378)))+(((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(r01)*(x377)*(x378)))+(((cj2)*(r00)*(x382)))+(((r01)*(sj2)*(x382)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x383=IKsin(j3);
IkReal x384=IKcos(j3);
IkReal x385=((sj1)*(sj2));
IkReal x386=((cj2)*(sj1));
IkReal x387=((cj1)*(sj2));
IkReal x388=((sj5)*(x384));
IkReal x389=((cj5)*(x383));
IkReal x390=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x391=((sj5)*(x383));
IkReal x392=((cj5)*(x384));
evalcond[0]=((((IkReal(-1.00000000000000))*(x388)))+(((IkReal(-1.00000000000000))*(x389)))+(r02));
evalcond[1]=((((IkReal(0.790000000000000))*(x388)))+(((IkReal(0.790000000000000))*(x389)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(x392)))+(((IkReal(-1.00000000000000))*(r01)*(x390)))+(x391)+(((r00)*(x387)))+(((r00)*(x386)))+(((r01)*(x385))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(0.790000000000000))*(x392)))+(((px)*(x387)))+(((px)*(x386)))+(((IkReal(-0.790000000000000))*(x391)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(x390)))+(((py)*(x385))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x393=((IkReal(1.00000000000000))*(r01));
IkReal x394=((cj1)*(sj2));
IkReal x395=((sj1)*(sj2));
IkReal x396=((cj2)*(sj1));
IkReal x397=((IkReal(1.00000000000000))*(py));
IkReal x398=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x395)))+(((IkReal(-1.00000000000000))*(r00)*(x398)))+(((IkReal(-1.00000000000000))*(x393)*(x394)))+(((IkReal(-1.00000000000000))*(x393)*(x396))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x394)*(x397)))+(((px)*(x395)))+(((IkReal(-1.00000000000000))*(x396)*(x397)))+(((IkReal(-1.00000000000000))*(px)*(x398))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x399=((r00)*(sj5));
IkReal x400=((cj2)*(sj1));
IkReal x401=((cj1)*(sj2));
IkReal x402=((cj5)*(r00));
IkReal x403=((r01)*(sj1)*(sj2));
IkReal x404=((IkReal(1.00000000000000))*(cj1)*(cj2)*(r01));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x404)))+(((sj5)*(x403)))+(((x399)*(x401)))+(((x399)*(x400)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((cj5)*(x403)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x404)))+(((x400)*(x402)))+(((x401)*(x402))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x404)))+(((sj5)*(x403)))+(((x399)*(x401)))+(((x399)*(x400)))+(((cj5)*(r02)))))), ((gconst56)*(((((cj5)*(x403)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x404)))+(((x400)*(x402)))+(((x401)*(x402)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x405=IKcos(j3);
IkReal x406=IKsin(j3);
IkReal x407=((sj1)*(sj2));
IkReal x408=((cj2)*(sj1));
IkReal x409=((cj1)*(sj2));
IkReal x410=((cj5)*(x406));
IkReal x411=((sj5)*(x405));
IkReal x412=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x413=((sj5)*(x406));
IkReal x414=((cj5)*(x405));
evalcond[0]=((x410)+(r02)+(((IkReal(-1.00000000000000))*(x411))));
evalcond[1]=((((IkReal(-0.790000000000000))*(x410)))+(pz)+(((IkReal(0.790000000000000))*(x411))));
evalcond[2]=((((r00)*(x408)))+(((r00)*(x409)))+(((r01)*(x407)))+(x413)+(x414)+(((IkReal(-1.00000000000000))*(r01)*(x412))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x407)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x408)))+(((px)*(x409)))+(((IkReal(-0.790000000000000))*(x413)))+(((IkReal(-0.790000000000000))*(x414)))+(((IkReal(-1.00000000000000))*(py)*(x412))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x415=((IkReal(1.00000000000000))*(r01));
IkReal x416=((cj1)*(sj2));
IkReal x417=((sj1)*(sj2));
IkReal x418=((cj2)*(sj1));
IkReal x419=((IkReal(1.00000000000000))*(py));
IkReal x420=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r00)*(x420)))+(((r00)*(x417)))+(((IkReal(-1.00000000000000))*(x415)*(x418)))+(((IkReal(-1.00000000000000))*(x415)*(x416))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x418)*(x419)))+(((IkReal(-1.00000000000000))*(px)*(x420)))+(((px)*(x417)))+(((IkReal(-0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(x416)*(x419)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x421=((r01)*(sj2));
IkReal x422=((IkReal(1.00000000000000))*(cj1));
IkReal x423=((r00)*(sj2));
IkReal x424=((cj2)*(r00));
IkReal x425=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x422)*(x424)))+(((IkReal(-1.00000000000000))*(x421)*(x422)))+(((IkReal(-1.00000000000000))*(r01)*(x425)))+(((sj1)*(x423))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r01)*(x422)))+(((sj1)*(x421)))+(((sj1)*(x424)))+(((cj1)*(x423))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x422)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x422)))+(((IkReal(-1.00000000000000))*(py)*(x425))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x426=((IkReal(12.5000000000000))*(sj1));
IkReal x427=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(-12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((cj2)*(py)*(x427)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x426)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x426)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x427)))+(((IkReal(4.56250000000000))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-12.5000000000000))*(pz)))+IKsqr(((IkReal(1.12500000000000))+(((cj2)*(py)*(x427)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x426)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x426)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x427)))+(((IkReal(4.56250000000000))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-12.5000000000000))*(pz)), ((IkReal(1.12500000000000))+(((cj2)*(py)*(x427)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x426)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x426)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x427)))+(((IkReal(4.56250000000000))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x428=((r01)*(sj2));
IkReal x429=((IkReal(1.00000000000000))*(cj1));
IkReal x430=((r00)*(sj2));
IkReal x431=((cj2)*(r00));
IkReal x432=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x428)*(x429)))+(((IkReal(-1.00000000000000))*(r01)*(x432)))+(((sj1)*(x430))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r01)*(x429)))+(((cj1)*(x430)))+(((sj1)*(x428)))+(((sj1)*(x431))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x429)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x429)))+(((IkReal(-1.00000000000000))*(py)*(x432))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x433=((IkReal(12.5000000000000))*(sj1));
IkReal x434=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x434)))+(((cj2)*(px)*(x433)))+(((py)*(sj2)*(x433)))+(((px)*(sj2)*(x434))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(12.5000000000000))*(pz)))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x434)))+(((cj2)*(px)*(x433)))+(((py)*(sj2)*(x433)))+(((px)*(sj2)*(x434)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(12.5000000000000))*(pz)), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x434)))+(((cj2)*(px)*(x433)))+(((py)*(sj2)*(x433)))+(((px)*(sj2)*(x434)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((gconst57)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(r02)), ((gconst57)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x435=IKsin(j3);
IkReal x436=IKcos(j3);
IkReal x437=((sj1)*(sj2));
IkReal x438=((cj2)*(sj1));
IkReal x439=((cj1)*(sj2));
IkReal x440=((IkReal(0.0800000000000000))*(sj4));
IkReal x441=((cj4)*(x435));
IkReal x442=((cj4)*(x436));
IkReal x443=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x441)))+(r02));
evalcond[1]=((((x435)*(x440)))+(((IkReal(0.790000000000000))*(x441)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(x442)))+(((r00)*(x438)))+(((r00)*(x439)))+(((IkReal(-1.00000000000000))*(r01)*(x443)))+(((r01)*(x437))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((px)*(x438)))+(((px)*(x439)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x437)))+(((IkReal(0.790000000000000))*(x442)))+(((IkReal(-1.00000000000000))*(py)*(x443)))+(((x436)*(x440))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x444=((IkReal(1.00000000000000))*(r01));
IkReal x445=((cj1)*(sj2));
IkReal x446=((sj1)*(sj2));
IkReal x447=((cj2)*(sj1));
IkReal x448=((IkReal(1.00000000000000))*(py));
IkReal x449=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x444)*(x447)))+(((IkReal(-1.00000000000000))*(x444)*(x445)))+(((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r00)*(x449)))+(((r00)*(x446))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x445)*(x448)))+(((px)*(x446)))+(((IkReal(-1.00000000000000))*(x447)*(x448)))+(((IkReal(0.790000000000000))*(sj4)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(px)*(x449))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x450=((r01)*(sj2));
IkReal x451=((IkReal(1.00000000000000))*(cj1));
IkReal x452=((r00)*(sj2));
IkReal x453=((cj2)*(r00));
IkReal x454=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r01)*(x454)))+(((sj1)*(x452)))+(((IkReal(-1.00000000000000))*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(x451)*(x453))));
evalcond[3]=((((cj1)*(x452)))+(((sj1)*(x450)))+(((sj1)*(x453)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x451))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x454)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x451)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x451))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x455=((IkReal(12.5000000000000))*(sj1));
IkReal x456=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(-12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((cj2)*(py)*(x456)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x456)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x455)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x455))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-12.5000000000000))*(pz)))+IKsqr(((IkReal(1.12500000000000))+(((cj2)*(py)*(x456)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x456)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x455)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x455)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-12.5000000000000))*(pz)), ((IkReal(1.12500000000000))+(((cj2)*(py)*(x456)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x456)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x455)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x455)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x457=((r01)*(sj2));
IkReal x458=((IkReal(1.00000000000000))*(cj1));
IkReal x459=((r00)*(sj2));
IkReal x460=((cj2)*(r00));
IkReal x461=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r01)*(x461)))+(((sj1)*(x459)))+(((IkReal(-1.00000000000000))*(x458)*(x460)))+(((IkReal(-1.00000000000000))*(x457)*(x458))));
evalcond[3]=((((cj1)*(x459)))+(((sj1)*(x457)))+(((sj1)*(x460)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x458))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x461)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x458)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x458))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x462=((IkReal(12.5000000000000))*(sj1));
IkReal x463=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x463)))+(((px)*(sj2)*(x463)))+(((py)*(sj2)*(x462)))+(((cj2)*(px)*(x462))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(12.5000000000000))*(pz)))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x463)))+(((px)*(sj2)*(x463)))+(((py)*(sj2)*(x462)))+(((cj2)*(px)*(x462)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(12.5000000000000))*(pz)), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x463)))+(((px)*(sj2)*(x463)))+(((py)*(sj2)*(x462)))+(((cj2)*(px)*(x462)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x464=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(gconst58)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x464)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x464)))+(((cj1)*(cj2)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst58)*(r02)), ((gconst58)*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x464)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x464)))+(((cj1)*(cj2)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x465=IKsin(j3);
IkReal x466=IKcos(j3);
IkReal x467=((sj1)*(sj2));
IkReal x468=((cj2)*(sj1));
IkReal x469=((cj1)*(sj2));
IkReal x470=((IkReal(0.0800000000000000))*(sj4));
IkReal x471=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x472=((cj4)*(x465));
IkReal x473=((cj4)*(x466));
evalcond[0]=((x472)+(r02));
evalcond[1]=((((IkReal(-0.790000000000000))*(x472)))+(((x465)*(x470)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x471)))+(x473)+(((r01)*(x467)))+(((r00)*(x468)))+(((r00)*(x469))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x473)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x466)*(x470)))+(((IkReal(-1.00000000000000))*(py)*(x471)))+(((px)*(x468)))+(((px)*(x469)))+(((py)*(x467))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x474=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x475=((IkReal(100.000000000000))*(pz));
IkReal x476=((IkReal(79.0000000000000))*(r02));
IkReal x477=((cj4)*(cj5));
IkReal x478=((IkReal(-0.125000000000000))*(x474));
if( IKabs(((x478)*(((x475)+(x476))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x478)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-8.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(x476)*(x477))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x478)*(((x475)+(x476)))))+IKsqr(((x478)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-8.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(x476)*(x477)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x478)*(((x475)+(x476)))), ((x478)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-8.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(x476)*(x477)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x479=IKsin(j3);
IkReal x480=IKcos(j3);
IkReal x481=((sj1)*(sj2));
IkReal x482=((cj2)*(sj1));
IkReal x483=((IkReal(0.0800000000000000))*(sj4));
IkReal x484=((cj1)*(sj2));
IkReal x485=((sj5)*(x480));
IkReal x486=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x487=((sj5)*(x479));
IkReal x488=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x489=((IkReal(0.790000000000000))*(cj4)*(cj5));
evalcond[0]=((((IkReal(-1.00000000000000))*(x485)))+(((IkReal(-1.00000000000000))*(x479)*(x488)))+(r02));
evalcond[1]=((pz)+(((IkReal(0.790000000000000))*(x485)))+(((x479)*(x489)))+(((x479)*(x483))));
evalcond[2]=((x487)+(((IkReal(-1.00000000000000))*(x480)*(x488)))+(((r01)*(x481)))+(((r00)*(x484)))+(((r00)*(x482)))+(((IkReal(-1.00000000000000))*(r01)*(x486))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x481)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x484)))+(((px)*(x482)))+(((IkReal(-0.790000000000000))*(x487)))+(((x480)*(x483)))+(((x480)*(x489)))+(((IkReal(-1.00000000000000))*(py)*(x486))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x490=((cj4)*(cj5));
IkReal x491=((IkReal(1.00000000000000))*(r01));
IkReal x492=((cj1)*(cj2));
IkReal x493=((sj2)*(sj5));
IkReal x494=((cj1)*(r00));
IkReal x495=((cj2)*(r00)*(sj1));
if( IKabs(((gconst54)*(((((r01)*(sj5)*(x492)))+(((IkReal(-1.00000000000000))*(x493)*(x494)))+(((r02)*(x490)))+(((IkReal(-1.00000000000000))*(sj5)*(x495)))+(((IkReal(-1.00000000000000))*(sj1)*(x491)*(x493))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((r01)*(sj1)*(sj2)*(x490)))+(((x490)*(x495)))+(((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(x490)*(x491)*(x492)))+(((sj2)*(x490)*(x494))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((r01)*(sj5)*(x492)))+(((IkReal(-1.00000000000000))*(x493)*(x494)))+(((r02)*(x490)))+(((IkReal(-1.00000000000000))*(sj5)*(x495)))+(((IkReal(-1.00000000000000))*(sj1)*(x491)*(x493)))))), ((gconst54)*(((((r01)*(sj1)*(sj2)*(x490)))+(((x490)*(x495)))+(((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(x490)*(x491)*(x492)))+(((sj2)*(x490)*(x494)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x496=IKsin(j3);
IkReal x497=IKcos(j3);
IkReal x498=((sj1)*(sj2));
IkReal x499=((cj2)*(sj1));
IkReal x500=((IkReal(0.0800000000000000))*(sj4));
IkReal x501=((cj1)*(sj2));
IkReal x502=((sj5)*(x497));
IkReal x503=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x504=((sj5)*(x496));
IkReal x505=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x506=((IkReal(0.790000000000000))*(cj4)*(cj5));
evalcond[0]=((((IkReal(-1.00000000000000))*(x496)*(x505)))+(r02)+(((IkReal(-1.00000000000000))*(x502))));
evalcond[1]=((((x496)*(x500)))+(((x496)*(x506)))+(pz)+(((IkReal(0.790000000000000))*(x502))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x497)*(x505)))+(((IkReal(-1.00000000000000))*(r01)*(x503)))+(((r00)*(x501)))+(x504)+(((r00)*(x499)))+(((r01)*(x498))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x503)))+(((py)*(x498)))+(((x497)*(x500)))+(((x497)*(x506)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x504)))+(((px)*(x499)))+(((px)*(x501))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x507=((IkReal(1.00000000000000))*(r01));
cj5array[0]=((IkReal(-1.00000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x507)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x507)))+(((r00)*(sj1)*(sj2))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x508=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(sj4)*(IKcos(j5))))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x508)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(px)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x508))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x509=((IkReal(1.00000000000000))*(r01));
IkReal x510=((cj1)*(sj2));
IkReal x511=((sj1)*(sj2));
IkReal x512=((cj2)*(sj1));
IkReal x513=((IkReal(1.00000000000000))*(py));
IkReal x514=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x514)))+(((IkReal(-1.00000000000000))*(x509)*(x510)))+(((IkReal(-1.00000000000000))*(x509)*(x512)))+(((r00)*(x511))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x511)))+(((IkReal(-1.00000000000000))*(px)*(x514)))+(((IkReal(-1.00000000000000))*(x512)*(x513)))+(((IkReal(-1.00000000000000))*(x510)*(x513))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x515=((IkReal(1.00000000000000))*(cj2));
IkReal x516=((cj1)*(cj5));
IkReal x517=((sj1)*(sj5));
IkReal x518=((IkReal(1.00000000000000))*(sj2));
IkReal x519=((cj1)*(sj5));
IkReal x520=((cj5)*(sj1));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(r00)*(x518)*(x519)))+(((IkReal(-1.00000000000000))*(r01)*(x517)*(x518)))+(((IkReal(-1.00000000000000))*(r00)*(x515)*(x517)))+(((cj2)*(r01)*(x519)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((r02)*(sj5)))+(((cj2)*(r00)*(x520)))+(((r01)*(sj2)*(x520)))+(((r00)*(sj2)*(x516)))+(((IkReal(-1.00000000000000))*(r01)*(x515)*(x516))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(r00)*(x518)*(x519)))+(((IkReal(-1.00000000000000))*(r01)*(x517)*(x518)))+(((IkReal(-1.00000000000000))*(r00)*(x515)*(x517)))+(((cj2)*(r01)*(x519)))+(((cj5)*(r02)))))), ((gconst55)*(((((r02)*(sj5)))+(((cj2)*(r00)*(x520)))+(((r01)*(sj2)*(x520)))+(((r00)*(sj2)*(x516)))+(((IkReal(-1.00000000000000))*(r01)*(x515)*(x516)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x521=IKsin(j3);
IkReal x522=IKcos(j3);
IkReal x523=((sj1)*(sj2));
IkReal x524=((cj2)*(sj1));
IkReal x525=((cj1)*(sj2));
IkReal x526=((sj5)*(x522));
IkReal x527=((cj5)*(x521));
IkReal x528=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x529=((sj5)*(x521));
IkReal x530=((cj5)*(x522));
evalcond[0]=((((IkReal(-1.00000000000000))*(x527)))+(((IkReal(-1.00000000000000))*(x526)))+(r02));
evalcond[1]=((((IkReal(0.790000000000000))*(x526)))+(((IkReal(0.790000000000000))*(x527)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(x530)))+(((IkReal(-1.00000000000000))*(r01)*(x528)))+(x529)+(((r01)*(x523)))+(((r00)*(x525)))+(((r00)*(x524))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(0.790000000000000))*(x530)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x525)))+(((px)*(x524)))+(((IkReal(-1.00000000000000))*(py)*(x528)))+(((IkReal(-0.790000000000000))*(x529)))+(((py)*(x523))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x531=((IkReal(1.00000000000000))*(r01));
IkReal x532=((cj1)*(sj2));
IkReal x533=((sj1)*(sj2));
IkReal x534=((cj2)*(sj1));
IkReal x535=((IkReal(1.00000000000000))*(py));
IkReal x536=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x531)*(x532)))+(((IkReal(-1.00000000000000))*(x531)*(x534)))+(((IkReal(-1.00000000000000))*(r00)*(x536)))+(((r00)*(x533))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x532)*(x535)))+(((IkReal(-1.00000000000000))*(x534)*(x535)))+(((IkReal(-1.00000000000000))*(px)*(x536)))+(((px)*(x533))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x537=((r00)*(sj5));
IkReal x538=((cj2)*(sj1));
IkReal x539=((cj1)*(sj2));
IkReal x540=((cj5)*(r00));
IkReal x541=((r01)*(sj1)*(sj2));
IkReal x542=((IkReal(1.00000000000000))*(cj1)*(cj2)*(r01));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x542)))+(((sj5)*(x541)))+(((x537)*(x538)))+(((x537)*(x539)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((x539)*(x540)))+(((cj5)*(x541)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((x538)*(x540)))+(((IkReal(-1.00000000000000))*(cj5)*(x542))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x542)))+(((sj5)*(x541)))+(((x537)*(x538)))+(((x537)*(x539)))+(((cj5)*(r02)))))), ((gconst56)*(((((x539)*(x540)))+(((cj5)*(x541)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((x538)*(x540)))+(((IkReal(-1.00000000000000))*(cj5)*(x542)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x543=IKcos(j3);
IkReal x544=IKsin(j3);
IkReal x545=((sj1)*(sj2));
IkReal x546=((cj2)*(sj1));
IkReal x547=((cj1)*(sj2));
IkReal x548=((cj5)*(x544));
IkReal x549=((sj5)*(x543));
IkReal x550=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x551=((sj5)*(x544));
IkReal x552=((cj5)*(x543));
evalcond[0]=((((IkReal(-1.00000000000000))*(x549)))+(x548)+(r02));
evalcond[1]=((((IkReal(0.790000000000000))*(x549)))+(pz)+(((IkReal(-0.790000000000000))*(x548))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x550)))+(x552)+(x551)+(((r00)*(x547)))+(((r00)*(x546)))+(((r01)*(x545))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x545)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x546)))+(((px)*(x547)))+(((IkReal(-0.790000000000000))*(x552)))+(((IkReal(-0.790000000000000))*(x551)))+(((IkReal(-1.00000000000000))*(py)*(x550))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x553=((IkReal(1.00000000000000))*(r01));
IkReal x554=((cj1)*(sj2));
IkReal x555=((sj1)*(sj2));
IkReal x556=((cj2)*(sj1));
IkReal x557=((IkReal(1.00000000000000))*(py));
IkReal x558=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r00)*(x558)))+(((IkReal(-1.00000000000000))*(x553)*(x554)))+(((IkReal(-1.00000000000000))*(x553)*(x556)))+(((r00)*(x555))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x556)*(x557)))+(((IkReal(-1.00000000000000))*(x554)*(x557)))+(((px)*(x555)))+(((IkReal(-0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(px)*(x558)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x559=((r01)*(sj2));
IkReal x560=((IkReal(1.00000000000000))*(cj1));
IkReal x561=((r00)*(sj2));
IkReal x562=((cj2)*(r00));
IkReal x563=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x559)*(x560)))+(((sj1)*(x561)))+(((IkReal(-1.00000000000000))*(x560)*(x562)))+(((IkReal(-1.00000000000000))*(r01)*(x563))));
evalcond[3]=((((cj1)*(x561)))+(((sj1)*(x559)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x560)))+(((sj1)*(x562))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x563)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x560)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x560))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x564=((IkReal(12.5000000000000))*(sj1));
IkReal x565=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(-12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x564)))+(((cj2)*(py)*(x565)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x564)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x565))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-12.5000000000000))*(pz)))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x564)))+(((cj2)*(py)*(x565)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x564)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x565)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-12.5000000000000))*(pz)), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x564)))+(((cj2)*(py)*(x565)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x564)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x565)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x566=((r01)*(sj2));
IkReal x567=((IkReal(1.00000000000000))*(cj1));
IkReal x568=((r00)*(sj2));
IkReal x569=((cj2)*(r00));
IkReal x570=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x566)*(x567)))+(((IkReal(-1.00000000000000))*(r01)*(x570)))+(((sj1)*(x568)))+(((IkReal(-1.00000000000000))*(x567)*(x569))));
evalcond[3]=((((cj1)*(x568)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x567)))+(((sj1)*(x566)))+(((sj1)*(x569))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x567)))+(((IkReal(-1.00000000000000))*(py)*(x570)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x567))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x571=((IkReal(12.5000000000000))*(sj1));
IkReal x572=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x572)))+(((py)*(sj2)*(x571)))+(((px)*(sj2)*(x572)))+(((cj2)*(px)*(x571))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(12.5000000000000))*(pz)))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x572)))+(((py)*(sj2)*(x571)))+(((px)*(sj2)*(x572)))+(((cj2)*(px)*(x571)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(12.5000000000000))*(pz)), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x572)))+(((py)*(sj2)*(x571)))+(((px)*(sj2)*(x572)))+(((cj2)*(px)*(x571)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((gconst57)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(r02)), ((gconst57)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x573=IKsin(j3);
IkReal x574=IKcos(j3);
IkReal x575=((sj1)*(sj2));
IkReal x576=((cj2)*(sj1));
IkReal x577=((cj1)*(sj2));
IkReal x578=((IkReal(0.0800000000000000))*(sj4));
IkReal x579=((cj4)*(x573));
IkReal x580=((cj4)*(x574));
IkReal x581=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x579)))+(r02));
evalcond[1]=((((IkReal(0.790000000000000))*(x579)))+(((x573)*(x578)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(x580)))+(((r01)*(x575)))+(((r00)*(x577)))+(((r00)*(x576)))+(((IkReal(-1.00000000000000))*(r01)*(x581))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x581)))+(((py)*(x575)))+(((px)*(x577)))+(((px)*(x576)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x574)*(x578)))+(((IkReal(0.790000000000000))*(x580))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x582=((IkReal(1.00000000000000))*(r01));
IkReal x583=((cj1)*(sj2));
IkReal x584=((sj1)*(sj2));
IkReal x585=((cj2)*(sj1));
IkReal x586=((IkReal(1.00000000000000))*(py));
IkReal x587=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(x582)*(x583)))+(((IkReal(-1.00000000000000))*(x582)*(x585)))+(((r00)*(x584)))+(((IkReal(-1.00000000000000))*(r00)*(x587))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x584)))+(((IkReal(-1.00000000000000))*(px)*(x587)))+(((IkReal(-1.00000000000000))*(x583)*(x586)))+(((IkReal(-1.00000000000000))*(x585)*(x586)))+(((IkReal(0.790000000000000))*(sj4)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x588=((r01)*(sj2));
IkReal x589=((IkReal(1.00000000000000))*(cj1));
IkReal x590=((r00)*(sj2));
IkReal x591=((cj2)*(r00));
IkReal x592=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r01)*(x592)))+(((IkReal(-1.00000000000000))*(x589)*(x591)))+(((IkReal(-1.00000000000000))*(x588)*(x589)))+(((sj1)*(x590))));
evalcond[3]=((((sj1)*(x588)))+(((cj1)*(x590)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x589)))+(((sj1)*(x591))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x589)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x589)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x592))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x593=((IkReal(12.5000000000000))*(sj1));
IkReal x594=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(-12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x593)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x594)))+(((cj2)*(py)*(x594)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x593)))+(((IkReal(4.56250000000000))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-12.5000000000000))*(pz)))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x593)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x594)))+(((cj2)*(py)*(x594)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x593)))+(((IkReal(4.56250000000000))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-12.5000000000000))*(pz)), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x593)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x594)))+(((cj2)*(py)*(x594)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x593)))+(((IkReal(4.56250000000000))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x595=((r01)*(sj2));
IkReal x596=((IkReal(1.00000000000000))*(cj1));
IkReal x597=((r00)*(sj2));
IkReal x598=((cj2)*(r00));
IkReal x599=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r01)*(x599)))+(((IkReal(-1.00000000000000))*(x596)*(x598)))+(((IkReal(-1.00000000000000))*(x595)*(x596)))+(((sj1)*(x597))));
evalcond[3]=((((cj1)*(x597)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x596)))+(((sj1)*(x598)))+(((sj1)*(x595))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x596)))+(((IkReal(-1.00000000000000))*(py)*(x599)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x596))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x600=((IkReal(12.5000000000000))*(sj1));
IkReal x601=((IkReal(12.5000000000000))*(cj1));
if( IKabs(((IkReal(12.5000000000000))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((px)*(sj2)*(x601)))+(((cj2)*(px)*(x600)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x601)))+(((py)*(sj2)*(x600))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(12.5000000000000))*(pz)))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((px)*(sj2)*(x601)))+(((cj2)*(px)*(x600)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x601)))+(((py)*(sj2)*(x600)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(12.5000000000000))*(pz)), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((px)*(sj2)*(x601)))+(((cj2)*(px)*(x600)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x601)))+(((py)*(sj2)*(x600)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(pz));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(py)))+(((cj2)*(px)*(sj1)))+(((cj1)*(px)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x602=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(gconst58)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x602)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x602)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst58)*(r02)), ((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x602)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x602)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x603=IKsin(j3);
IkReal x604=IKcos(j3);
IkReal x605=((sj1)*(sj2));
IkReal x606=((cj2)*(sj1));
IkReal x607=((cj1)*(sj2));
IkReal x608=((IkReal(0.0800000000000000))*(sj4));
IkReal x609=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x610=((cj4)*(x603));
IkReal x611=((cj4)*(x604));
evalcond[0]=((r02)+(x610));
evalcond[1]=((((x603)*(x608)))+(pz)+(((IkReal(-0.790000000000000))*(x610))));
evalcond[2]=((((r00)*(x606)))+(((r00)*(x607)))+(((IkReal(-1.00000000000000))*(r01)*(x609)))+(x611)+(((r01)*(x605))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x604)*(x608)))+(((px)*(x606)))+(((px)*(x607)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x605)))+(((IkReal(-0.790000000000000))*(x611)))+(((IkReal(-1.00000000000000))*(py)*(x609))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x612=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x613=((IkReal(100.000000000000))*(pz));
IkReal x614=((IkReal(79.0000000000000))*(r02));
IkReal x615=((cj4)*(cj5));
IkReal x616=((IkReal(-0.125000000000000))*(x612));
if( IKabs(((x616)*(((x614)+(x613))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x616)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x614)*(x615)))+(((IkReal(-8.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x613)*(x615))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x616)*(((x614)+(x613)))))+IKsqr(((x616)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x614)*(x615)))+(((IkReal(-8.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x613)*(x615)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x616)*(((x614)+(x613)))), ((x616)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x614)*(x615)))+(((IkReal(-8.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x613)*(x615)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x617=IKsin(j3);
IkReal x618=IKcos(j3);
IkReal x619=((sj1)*(sj2));
IkReal x620=((cj2)*(sj1));
IkReal x621=((IkReal(0.0800000000000000))*(sj4));
IkReal x622=((cj1)*(sj2));
IkReal x623=((sj5)*(x618));
IkReal x624=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x625=((sj5)*(x617));
IkReal x626=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x627=((IkReal(0.790000000000000))*(cj4)*(cj5));
evalcond[0]=((((IkReal(-1.00000000000000))*(x617)*(x626)))+(r02)+(((IkReal(-1.00000000000000))*(x623))));
evalcond[1]=((((IkReal(0.790000000000000))*(x623)))+(((x617)*(x627)))+(((x617)*(x621)))+(pz));
evalcond[2]=((((r01)*(x619)))+(((IkReal(-1.00000000000000))*(r01)*(x624)))+(((r00)*(x622)))+(((r00)*(x620)))+(x625)+(((IkReal(-1.00000000000000))*(x618)*(x626))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x625)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x618)*(x627)))+(((x618)*(x621)))+(((py)*(x619)))+(((px)*(x622)))+(((px)*(x620)))+(((IkReal(-1.00000000000000))*(py)*(x624))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x628=((cj4)*(cj5));
IkReal x629=((IkReal(1.00000000000000))*(r01));
IkReal x630=((cj1)*(cj2));
IkReal x631=((sj2)*(sj5));
IkReal x632=((cj1)*(r00));
IkReal x633=((cj2)*(r00)*(sj1));
if( IKabs(((gconst54)*(((((r02)*(x628)))+(((IkReal(-1.00000000000000))*(sj1)*(x629)*(x631)))+(((IkReal(-1.00000000000000))*(sj5)*(x633)))+(((IkReal(-1.00000000000000))*(x631)*(x632)))+(((r01)*(sj5)*(x630))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x628)*(x629)*(x630)))+(((x628)*(x633)))+(((r01)*(sj1)*(sj2)*(x628)))+(((r02)*(sj5)))+(((sj2)*(x628)*(x632))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((r02)*(x628)))+(((IkReal(-1.00000000000000))*(sj1)*(x629)*(x631)))+(((IkReal(-1.00000000000000))*(sj5)*(x633)))+(((IkReal(-1.00000000000000))*(x631)*(x632)))+(((r01)*(sj5)*(x630)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(x628)*(x629)*(x630)))+(((x628)*(x633)))+(((r01)*(sj1)*(sj2)*(x628)))+(((r02)*(sj5)))+(((sj2)*(x628)*(x632)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x634=IKsin(j3);
IkReal x635=IKcos(j3);
IkReal x636=((sj1)*(sj2));
IkReal x637=((cj2)*(sj1));
IkReal x638=((IkReal(0.0800000000000000))*(sj4));
IkReal x639=((cj1)*(sj2));
IkReal x640=((sj5)*(x635));
IkReal x641=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x642=((sj5)*(x634));
IkReal x643=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x644=((IkReal(0.790000000000000))*(cj4)*(cj5));
evalcond[0]=((((IkReal(-1.00000000000000))*(x640)))+(r02)+(((IkReal(-1.00000000000000))*(x634)*(x643))));
evalcond[1]=((((x634)*(x638)))+(((x634)*(x644)))+(pz)+(((IkReal(0.790000000000000))*(x640))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x641)))+(((r01)*(x636)))+(((IkReal(-1.00000000000000))*(x635)*(x643)))+(((r00)*(x639)))+(((r00)*(x637)))+(x642));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x636)))+(((x635)*(x644)))+(((x635)*(x638)))+(((IkReal(-0.790000000000000))*(x642)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(x641)))+(((px)*(x637)))+(((px)*(x639))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x645=((IkReal(0.790000000000000))*(cj1));
IkReal x646=((IkReal(1.00000000000000))*(px));
IkReal x647=((cj2)*(sj1));
IkReal x648=((sj1)*(sj2));
if( IKabs(((IkReal(-25.0000000000000))*(gconst53)*(((((IkReal(0.790000000000000))*(r02)))+(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(25.0000000000000))*(gconst53)*(((IkReal(0.0900000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(py)*(x648)))+(((IkReal(0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x646)))+(((IkReal(-1.00000000000000))*(x646)*(x647)))+(((IkReal(-0.790000000000000))*(r01)*(x648)))+(((IkReal(-0.790000000000000))*(r00)*(x647)))+(((cj2)*(r01)*(x645)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x645))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-25.0000000000000))*(gconst53)*(((((IkReal(0.790000000000000))*(r02)))+(pz)))), ((IkReal(25.0000000000000))*(gconst53)*(((IkReal(0.0900000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(py)*(x648)))+(((IkReal(0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x646)))+(((IkReal(-1.00000000000000))*(x646)*(x647)))+(((IkReal(-0.790000000000000))*(r01)*(x648)))+(((IkReal(-0.790000000000000))*(r00)*(x647)))+(((cj2)*(r01)*(x645)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x645)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x649=IKsin(j3);
IkReal x650=IKcos(j3);
IkReal x651=((IkReal(0.365000000000000))*(cj2));
IkReal x652=((cj1)*(cj2));
IkReal x653=((cj4)*(py));
IkReal x654=((IkReal(0.790000000000000))*(sj2));
IkReal x655=((cj1)*(r00));
IkReal x656=((IkReal(0.790000000000000))*(r01));
IkReal x657=((r01)*(sj1));
IkReal x658=((IkReal(0.790000000000000))*(cj4));
IkReal x659=((sj1)*(sj2));
IkReal x660=((cj2)*(px)*(sj1));
IkReal x661=((cj2)*(r00)*(sj1));
IkReal x662=((cj1)*(px)*(sj2));
IkReal x663=((IkReal(0.0800000000000000))*(sj4)*(x649));
IkReal x664=((IkReal(0.0800000000000000))*(sj4)*(x650));
evalcond[0]=((((IkReal(0.790000000000000))*(r02)))+(pz)+(x663));
evalcond[1]=((((cj4)*(pz)))+(((r02)*(x658)))+(((cj4)*(x663))));
evalcond[2]=((IkReal(0.0900000000000000))+(((IkReal(-1.00000000000000))*(py)*(x659)))+(((py)*(x652)))+(((x652)*(x656)))+(((IkReal(-1.00000000000000))*(x662)))+(((IkReal(-1.00000000000000))*(x660)))+(((IkReal(-1.00000000000000))*(x664)))+(((IkReal(-0.790000000000000))*(x661)))+(x651)+(((IkReal(-1.00000000000000))*(x654)*(x657)))+(((IkReal(-1.00000000000000))*(x654)*(x655))));
evalcond[3]=((((x653)*(x659)))+(((IkReal(-1.00000000000000))*(cj4)*(x652)*(x656)))+(((cj4)*(x654)*(x655)))+(((cj4)*(x654)*(x657)))+(((x658)*(x661)))+(((cj4)*(x664)))+(((cj4)*(x660)))+(((cj4)*(x662)))+(((IkReal(-1.00000000000000))*(cj4)*(x651)))+(((IkReal(-1.00000000000000))*(x652)*(x653)))+(((IkReal(-0.0900000000000000))*(cj4))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x665=(sj3)*(sj3);
IkReal x666=(cj3)*(cj3);
dummyeval[0]=((x665)+(x666));
dummyeval[1]=((((cj4)*(x666)))+(((cj4)*(x665))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(0.0800000000000000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x667=((cj1)*(r00));
IkReal x668=((cj2)*(r01));
IkReal x669=((IkReal(1.00000000000000))*(sj2));
IkReal x670=((r00)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x669)))+(((cj1)*(x668)))+(((IkReal(-1.00000000000000))*(x667)*(x669)))+(((IkReal(-1.00000000000000))*(cj2)*(x670))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(x667)))+(((sj1)*(x668)))+(((IkReal(-1.00000000000000))*(x669)*(x670))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x669)))+(((cj1)*(x668)))+(((IkReal(-1.00000000000000))*(x667)*(x669)))+(((IkReal(-1.00000000000000))*(cj2)*(x670)))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(x667)))+(((sj1)*(x668)))+(((IkReal(-1.00000000000000))*(x669)*(x670)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x669)))+(((cj1)*(x668)))+(((IkReal(-1.00000000000000))*(x667)*(x669)))+(((IkReal(-1.00000000000000))*(cj2)*(x670)))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(x667)))+(((sj1)*(x668)))+(((IkReal(-1.00000000000000))*(x669)*(x670)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x671=IKcos(j5);
IkReal x672=IKsin(j5);
IkReal x673=((sj1)*(sj2));
IkReal x674=((IkReal(1.00000000000000))*(cj1));
IkReal x675=((cj2)*(sj1));
IkReal x676=((cj1)*(sj2));
evalcond[0]=((((r00)*(x673)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x674)))+(((IkReal(-1.00000000000000))*(r01)*(x675)))+(x671)+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x674))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(r01)*(x674)))+(((r01)*(x673)))+(((r00)*(x675)))+(((r00)*(x676)))+(x672));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x674)))+(((IkReal(-1.00000000000000))*(py)*(x675)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x674)))+(((IkReal(-0.790000000000000))*(x671)))+(((px)*(x673))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x673)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x674)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x672)))+(((px)*(x675)))+(((px)*(x676))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-0.0800000000000000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x677=((r00)*(sj1));
IkReal x678=((IkReal(1.00000000000000))*(cj2));
IkReal x679=((r01)*(sj1));
IkReal x680=((IkReal(1.00000000000000))*(cj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x679)))+(((IkReal(-1.00000000000000))*(r00)*(x680)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x677)*(x678))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj2)*(x677)))+(((IkReal(-1.00000000000000))*(x678)*(x679)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x678)))+(((IkReal(-1.00000000000000))*(r01)*(x680))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x679)))+(((IkReal(-1.00000000000000))*(r00)*(x680)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x677)*(x678)))))+IKsqr(((((sj2)*(x677)))+(((IkReal(-1.00000000000000))*(x678)*(x679)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x678)))+(((IkReal(-1.00000000000000))*(r01)*(x680)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj2)*(x679)))+(((IkReal(-1.00000000000000))*(r00)*(x680)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x677)*(x678)))), ((((sj2)*(x677)))+(((IkReal(-1.00000000000000))*(x678)*(x679)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x678)))+(((IkReal(-1.00000000000000))*(r01)*(x680)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x681=IKcos(j5);
IkReal x682=IKsin(j5);
IkReal x683=((sj1)*(sj2));
IkReal x684=((cj1)*(sj2));
IkReal x685=((IkReal(1.00000000000000))*(r01));
IkReal x686=((cj2)*(sj1));
IkReal x687=((IkReal(1.00000000000000))*(py));
IkReal x688=((cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x685)*(x686)))+(((IkReal(-1.00000000000000))*(x684)*(x685)))+(((IkReal(-1.00000000000000))*(r00)*(x688)))+(((r00)*(x683)))+(((IkReal(-1.00000000000000))*(x681))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x685)*(x688)))+(x682)+(((r01)*(x683)))+(((r00)*(x684)))+(((r00)*(x686))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x683)))+(((IkReal(-1.00000000000000))*(px)*(x688)))+(((IkReal(-1.00000000000000))*(x686)*(x687)))+(((IkReal(-1.00000000000000))*(x684)*(x687)))+(((IkReal(0.790000000000000))*(x681))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((px)*(x686)))+(((px)*(x684)))+(((IkReal(-1.00000000000000))*(x687)*(x688)))+(((IkReal(-0.790000000000000))*(x682)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x683))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x689=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x689)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x689)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj4))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x689)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x689)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))+IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj4)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x689)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x689)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))), ((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj4)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x690=IKcos(j5);
IkReal x691=IKsin(j5);
IkReal x692=((cj2)*(sj1));
IkReal x693=((r01)*(sj2));
IkReal x694=((IkReal(1.00000000000000))*(cj1));
IkReal x695=((r00)*(sj2));
IkReal x696=((py)*(sj2));
IkReal x697=((px)*(sj2));
IkReal x698=((sj4)*(x690));
IkReal x699=((cj4)*(x690));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(x699))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(pz)+(((IkReal(0.790000000000000))*(x699))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x694)))+(((sj1)*(x695)))+(((IkReal(-1.00000000000000))*(x693)*(x694)))+(x698)+(((IkReal(-1.00000000000000))*(r01)*(x692))));
evalcond[3]=((((cj1)*(x695)))+(((sj1)*(x693)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x694)))+(x691)+(((r00)*(x692))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((sj1)*(x697)))+(((IkReal(-1.00000000000000))*(py)*(x692)))+(((IkReal(-0.790000000000000))*(x698)))+(((IkReal(-1.00000000000000))*(x694)*(x696)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x694))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((cj1)*(x697)))+(((sj1)*(x696)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x691)))+(((px)*(x692)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x694))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x700=((IkReal(1.26582278481013))*(sj2));
IkReal x701=((IkReal(1.26582278481013))*(cj2));
if( IKabs(((IkReal(-0.113924050632911))+(((IkReal(-0.462025316455696))*(cj2)))+(((px)*(sj1)*(x701)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x701)))+(((cj1)*(px)*(x700)))+(((py)*(sj1)*(x700))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.113924050632911))+(((IkReal(-0.462025316455696))*(cj2)))+(((px)*(sj1)*(x701)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x701)))+(((cj1)*(px)*(x700)))+(((py)*(sj1)*(x700)))))+IKsqr(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-0.113924050632911))+(((IkReal(-0.462025316455696))*(cj2)))+(((px)*(sj1)*(x701)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x701)))+(((cj1)*(px)*(x700)))+(((py)*(sj1)*(x700)))), ((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x702=IKcos(j5);
IkReal x703=IKsin(j5);
IkReal x704=((cj2)*(sj1));
IkReal x705=((r01)*(sj2));
IkReal x706=((IkReal(1.00000000000000))*(cj1));
IkReal x707=((r00)*(sj2));
IkReal x708=((py)*(sj2));
IkReal x709=((px)*(sj2));
IkReal x710=((sj4)*(x702));
IkReal x711=((cj4)*(x702));
evalcond[0]=((((IkReal(-1.00000000000000))*(x711)))+(r02));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(0.790000000000000))*(x711)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x704)))+(((IkReal(-1.00000000000000))*(x705)*(x706)))+(((sj1)*(x707)))+(x710)+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x706))));
evalcond[3]=((((r00)*(x704)))+(((sj1)*(x705)))+(x703)+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x706)))+(((cj1)*(x707))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x706)*(x708)))+(((IkReal(-0.790000000000000))*(x710)))+(((IkReal(-1.00000000000000))*(py)*(x704)))+(((sj1)*(x709)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x706)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x703)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x706)))+(((px)*(x704)))+(((sj1)*(x708)))+(((cj1)*(x709))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x712=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x712)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x712)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x712)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x712)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))+IKsqr(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x712)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x712)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))), ((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x713=IKcos(j5);
IkReal x714=IKsin(j5);
IkReal x715=((cj2)*(sj1));
IkReal x716=((r01)*(sj2));
IkReal x717=((IkReal(1.00000000000000))*(cj1));
IkReal x718=((r00)*(sj2));
IkReal x719=((py)*(sj2));
IkReal x720=((px)*(sj2));
IkReal x721=((sj4)*(x713));
IkReal x722=((cj4)*(x713));
evalcond[0]=((((IkReal(-1.00000000000000))*(x722)))+(r02));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(0.790000000000000))*(x722)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x715)))+(((IkReal(-1.00000000000000))*(x716)*(x717)))+(((sj1)*(x718)))+(x721)+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x717))));
evalcond[3]=((((r00)*(x715)))+(((sj1)*(x716)))+(x714)+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x717)))+(((cj1)*(x718))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(x721)))+(((IkReal(-1.00000000000000))*(x717)*(x719)))+(((IkReal(-1.00000000000000))*(py)*(x715)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x717)))+(((IkReal(0.0800000000000000))*(cj4)))+(((sj1)*(x720))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((cj1)*(x720)))+(((IkReal(-0.790000000000000))*(x714)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x717)))+(((px)*(x715)))+(((sj1)*(x719))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(-0.0800000000000000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x723=((cj1)*(cj2));
IkReal x724=((r01)*(sj1));
IkReal x725=((cj1)*(sj2));
IkReal x726=((r00)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r01)*(x723)))+(((sj2)*(x724)))+(((r00)*(x725)))+(((cj2)*(x726))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj2)*(x726)))+(((r01)*(x725)))+(((r00)*(x723)))+(((cj2)*(x724))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r01)*(x723)))+(((sj2)*(x724)))+(((r00)*(x725)))+(((cj2)*(x726)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj2)*(x726)))+(((r01)*(x725)))+(((r00)*(x723)))+(((cj2)*(x724)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r01)*(x723)))+(((sj2)*(x724)))+(((r00)*(x725)))+(((cj2)*(x726)))), ((((IkReal(-1.00000000000000))*(sj2)*(x726)))+(((r01)*(x725)))+(((r00)*(x723)))+(((cj2)*(x724)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x727=IKcos(j5);
IkReal x728=IKsin(j5);
IkReal x729=((sj1)*(sj2));
IkReal x730=((IkReal(1.00000000000000))*(cj1));
IkReal x731=((cj2)*(sj1));
IkReal x732=((cj1)*(sj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(sj2)*(x730)))+(((IkReal(-1.00000000000000))*(r01)*(x731)))+(((r00)*(x729)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x730)))+(x727));
evalcond[1]=((((r01)*(x729)))+(((r00)*(x731)))+(((r00)*(x732)))+(((IkReal(-1.00000000000000))*(x728)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x730))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(x727)))+(((px)*(x729)))+(((IkReal(-1.00000000000000))*(py)*(x731)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x730)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x730))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(0.790000000000000))*(x728)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x730)))+(((px)*(x731)))+(((px)*(x732)))+(((py)*(x729))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=r02;
evalcond[2]=((IkReal(0.0800000000000000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x733=((r01)*(sj2));
IkReal x734=((IkReal(1.00000000000000))*(cj1));
IkReal x735=((r00)*(sj2));
IkReal x736=((cj2)*(sj1));
if( IKabs(((((r00)*(x736)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x734)))+(((sj1)*(x733)))+(((cj1)*(x735))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(x736)))+(((IkReal(-1.00000000000000))*(x733)*(x734)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x734)))+(((sj1)*(x735))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(x736)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x734)))+(((sj1)*(x733)))+(((cj1)*(x735)))))+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(x736)))+(((IkReal(-1.00000000000000))*(x733)*(x734)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x734)))+(((sj1)*(x735)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(x736)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x734)))+(((sj1)*(x733)))+(((cj1)*(x735)))), ((((IkReal(-1.00000000000000))*(r01)*(x736)))+(((IkReal(-1.00000000000000))*(x733)*(x734)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x734)))+(((sj1)*(x735)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x737=IKcos(j5);
IkReal x738=IKsin(j5);
IkReal x739=((cj2)*(sj1));
IkReal x740=((sj1)*(sj2));
IkReal x741=((cj1)*(r00));
IkReal x742=((IkReal(1.00000000000000))*(cj2));
IkReal x743=((cj1)*(r01));
IkReal x744=((cj1)*(py));
IkReal x745=((IkReal(1.00000000000000))*(sj2));
IkReal x746=((cj1)*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x739)))+(((IkReal(-1.00000000000000))*(x737)))+(((r00)*(x740)))+(((IkReal(-1.00000000000000))*(x741)*(x742)))+(((IkReal(-1.00000000000000))*(x743)*(x745))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x742)*(x743)))+(((r00)*(x739)))+(((IkReal(-1.00000000000000))*(x738)))+(((sj2)*(x741)))+(((r01)*(x740))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x742)*(x746)))+(((IkReal(0.790000000000000))*(x737)))+(((px)*(x740)))+(((IkReal(-1.00000000000000))*(x744)*(x745)))+(((IkReal(-1.00000000000000))*(py)*(x739))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x740)))+(((IkReal(-1.00000000000000))*(x742)*(x744)))+(((IkReal(0.790000000000000))*(x738)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x739)))+(((sj2)*(x746))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x747=((IkReal(1.26582278481013))*(sj2));
IkReal x748=((IkReal(1.26582278481013))*(cj2));
if( IKabs(((IkReal(0.113924050632911))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x747)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x747)))+(((cj1)*(py)*(x748)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x748))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.113924050632911))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x747)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x747)))+(((cj1)*(py)*(x748)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x748)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.113924050632911))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x747)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x747)))+(((cj1)*(py)*(x748)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x748)))), ((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x749=IKcos(j5);
IkReal x750=IKsin(j5);
IkReal x751=((cj2)*(sj1));
IkReal x752=((sj1)*(sj2));
IkReal x753=((cj1)*(r00));
IkReal x754=((IkReal(1.00000000000000))*(cj2));
IkReal x755=((cj1)*(r01));
IkReal x756=((cj1)*(py));
IkReal x757=((IkReal(1.00000000000000))*(sj2));
IkReal x758=((cj1)*(px));
IkReal x759=((sj4)*(x749));
IkReal x760=((cj4)*(x749));
evalcond[0]=((r02)+(x760));
evalcond[1]=((pz)+(((IkReal(-0.790000000000000))*(x760)))+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x755)*(x757)))+(((IkReal(-1.00000000000000))*(x753)*(x754)))+(((r00)*(x752)))+(x759)+(((IkReal(-1.00000000000000))*(r01)*(x751))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x754)*(x755)))+(((r01)*(x752)))+(((r00)*(x751)))+(((IkReal(-1.00000000000000))*(x750)))+(((sj2)*(x753))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x754)*(x758)))+(((IkReal(-0.790000000000000))*(x759)))+(((px)*(x752)))+(((IkReal(-1.00000000000000))*(x756)*(x757)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x751))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(0.790000000000000))*(x750)))+(((py)*(x752)))+(((IkReal(-1.00000000000000))*(x754)*(x756)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x751)))+(((sj2)*(x758))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj4))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj4)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), ((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj4)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x761=IKcos(j5);
IkReal x762=IKsin(j5);
IkReal x763=((cj2)*(sj1));
IkReal x764=((sj1)*(sj2));
IkReal x765=((cj1)*(r00));
IkReal x766=((IkReal(1.00000000000000))*(cj2));
IkReal x767=((cj1)*(r01));
IkReal x768=((cj1)*(py));
IkReal x769=((IkReal(1.00000000000000))*(sj2));
IkReal x770=((cj1)*(px));
IkReal x771=((sj4)*(x761));
IkReal x772=((cj4)*(x761));
evalcond[0]=((r02)+(x772));
evalcond[1]=((((IkReal(-0.790000000000000))*(x772)))+(pz)+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-1.00000000000000))*(r01)*(x763)))+(((IkReal(-1.00000000000000))*(x767)*(x769)))+(x771)+(((r00)*(x764))));
evalcond[3]=((((sj2)*(x765)))+(((IkReal(-1.00000000000000))*(x766)*(x767)))+(((IkReal(-1.00000000000000))*(x762)))+(((r00)*(x763)))+(((r01)*(x764))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x764)))+(((IkReal(-0.790000000000000))*(x771)))+(((IkReal(-1.00000000000000))*(x766)*(x770)))+(((IkReal(-1.00000000000000))*(py)*(x763)))+(((IkReal(-1.00000000000000))*(x768)*(x769)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((px)*(x763)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x766)*(x768)))+(((IkReal(0.790000000000000))*(x762)))+(((sj2)*(x770)))+(((py)*(x764))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), ((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x773=IKcos(j5);
IkReal x774=IKsin(j5);
IkReal x775=((cj2)*(sj1));
IkReal x776=((sj1)*(sj2));
IkReal x777=((cj1)*(r00));
IkReal x778=((IkReal(1.00000000000000))*(cj2));
IkReal x779=((cj1)*(r01));
IkReal x780=((cj1)*(py));
IkReal x781=((IkReal(1.00000000000000))*(sj2));
IkReal x782=((cj1)*(px));
IkReal x783=((sj4)*(x773));
IkReal x784=((cj4)*(x773));
evalcond[0]=((r02)+(x784));
evalcond[1]=((pz)+(((IkReal(-0.0800000000000000))*(sj4)))+(((IkReal(-0.790000000000000))*(x784))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x775)))+(((r00)*(x776)))+(x783)+(((IkReal(-1.00000000000000))*(x777)*(x778)))+(((IkReal(-1.00000000000000))*(x779)*(x781))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x778)*(x779)))+(((r01)*(x776)))+(((sj2)*(x777)))+(((r00)*(x775)))+(((IkReal(-1.00000000000000))*(x774))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x775)))+(((px)*(x776)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-0.790000000000000))*(x783)))+(((IkReal(-1.00000000000000))*(x780)*(x781)))+(((IkReal(-1.00000000000000))*(x778)*(x782))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((sj2)*(x782)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x775)))+(((IkReal(-1.00000000000000))*(x778)*(x780)))+(((IkReal(0.790000000000000))*(x774)))+(((py)*(x776))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x785=((IkReal(1.00000000000000))*(r01));
IkReal x786=((cj1)*(sj2));
IkReal x787=((sj1)*(sj2));
IkReal x788=((cj2)*(sj1));
IkReal x789=((IkReal(1.00000000000000))*(py));
IkReal x790=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x787)))+(((IkReal(-1.00000000000000))*(r00)*(x790)))+(((IkReal(-1.00000000000000))*(x785)*(x786)))+(((IkReal(-1.00000000000000))*(x785)*(x788))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x790)))+(((IkReal(-1.00000000000000))*(x788)*(x789)))+(((px)*(x787)))+(((IkReal(-1.00000000000000))*(x786)*(x789))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x791=((r01)*(sj1));
IkReal x792=((cj1)*(r00));
IkReal x793=((IkReal(1.00000000000000))*(cj2));
IkReal x794=((cj1)*(r01));
IkReal x795=((r00)*(sj1));
IkReal x796=((cj3)*(sj2));
IkReal x797=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(sj3)*(x793)*(x795)))+(((IkReal(-1.00000000000000))*(x792)*(x797)))+(((IkReal(-1.00000000000000))*(x791)*(x797)))+(((cj3)*(r02)))+(((cj2)*(sj3)*(x794))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((cj2)*(cj3)*(x795)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x793)*(x794)))+(((x792)*(x796)))+(((x791)*(x796))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst59)*(((((IkReal(-1.00000000000000))*(sj3)*(x793)*(x795)))+(((IkReal(-1.00000000000000))*(x792)*(x797)))+(((IkReal(-1.00000000000000))*(x791)*(x797)))+(((cj3)*(r02)))+(((cj2)*(sj3)*(x794)))))), ((gconst59)*(((((cj2)*(cj3)*(x795)))+(((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x793)*(x794)))+(((x792)*(x796)))+(((x791)*(x796)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x798=IKsin(j5);
IkReal x799=IKcos(j5);
IkReal x800=((sj1)*(sj2));
IkReal x801=((IkReal(0.790000000000000))*(cj3));
IkReal x802=((IkReal(1.00000000000000))*(cj3));
IkReal x803=((cj2)*(sj1));
IkReal x804=((cj1)*(sj2));
IkReal x805=((IkReal(0.790000000000000))*(sj3));
IkReal x806=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x798)*(x802)))+(((IkReal(-1.00000000000000))*(sj3)*(x799)))+(r02));
evalcond[1]=((((x798)*(x801)))+(((x799)*(x805)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(x799)*(x802)))+(((IkReal(-1.00000000000000))*(r01)*(x806)))+(((r01)*(x800)))+(((r00)*(x804)))+(((r00)*(x803)))+(((sj3)*(x798))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x798)*(x805)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x799)*(x801)))+(((px)*(x804)))+(((px)*(x803)))+(((py)*(x800)))+(((IkReal(-1.00000000000000))*(py)*(x806))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x807=((IkReal(1.00000000000000))*(r01));
IkReal x808=((cj1)*(sj2));
IkReal x809=((sj1)*(sj2));
IkReal x810=((cj2)*(sj1));
IkReal x811=((IkReal(1.00000000000000))*(py));
IkReal x812=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x807)*(x808)))+(((IkReal(-1.00000000000000))*(x807)*(x810)))+(((r00)*(x809)))+(((IkReal(-1.00000000000000))*(r00)*(x812))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x808)*(x811)))+(((IkReal(-1.00000000000000))*(x810)*(x811)))+(((px)*(x809)))+(((IkReal(-1.00000000000000))*(px)*(x812))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x813=((r01)*(sj1));
IkReal x814=((cj1)*(r00));
IkReal x815=((cj2)*(cj3));
IkReal x816=((cj1)*(r01));
IkReal x817=((IkReal(1.00000000000000))*(sj3));
IkReal x818=((r00)*(sj1));
IkReal x819=((sj2)*(x817));
IkReal x820=((IkReal(1.00000000000000))*(cj3)*(sj2));
if( IKabs(((gconst60)*(((((cj2)*(sj3)*(x816)))+(((IkReal(-1.00000000000000))*(cj2)*(x817)*(x818)))+(((IkReal(-1.00000000000000))*(x814)*(x819)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x813)*(x819))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((x815)*(x816)))+(((IkReal(-1.00000000000000))*(x814)*(x820)))+(((IkReal(-1.00000000000000))*(x815)*(x818)))+(((IkReal(-1.00000000000000))*(x813)*(x820)))+(((IkReal(-1.00000000000000))*(r02)*(x817))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst60)*(((((cj2)*(sj3)*(x816)))+(((IkReal(-1.00000000000000))*(cj2)*(x817)*(x818)))+(((IkReal(-1.00000000000000))*(x814)*(x819)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x813)*(x819)))))), ((gconst60)*(((((x815)*(x816)))+(((IkReal(-1.00000000000000))*(x814)*(x820)))+(((IkReal(-1.00000000000000))*(x815)*(x818)))+(((IkReal(-1.00000000000000))*(x813)*(x820)))+(((IkReal(-1.00000000000000))*(r02)*(x817)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x821=IKsin(j5);
IkReal x822=IKcos(j5);
IkReal x823=((sj1)*(sj2));
IkReal x824=((IkReal(0.790000000000000))*(cj3));
IkReal x825=((cj2)*(sj1));
IkReal x826=((cj1)*(sj2));
IkReal x827=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x828=((sj3)*(x822));
IkReal x829=((sj3)*(x821));
evalcond[0]=((r02)+(x828)+(((IkReal(-1.00000000000000))*(cj3)*(x821))));
evalcond[1]=((((x821)*(x824)))+(((IkReal(-0.790000000000000))*(x828)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x827)))+(((r00)*(x825)))+(((r00)*(x826)))+(((r01)*(x823)))+(x829)+(((cj3)*(x822))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((py)*(x823)))+(((IkReal(-1.00000000000000))*(x822)*(x824)))+(((IkReal(-1.00000000000000))*(py)*(x827)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x826)))+(((px)*(x825)))+(((IkReal(-0.790000000000000))*(x829))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x830=((IkReal(1.26582278481013))*(sj1));
IkReal x831=((IkReal(1.26582278481013))*(cj1));
if( IKabs(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.512658227848101))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x831)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x831)))+(((px)*(sj2)*(x830)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x830))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((IkReal(0.512658227848101))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x831)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x831)))+(((px)*(sj2)*(x830)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x830)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((IkReal(0.512658227848101))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x831)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x831)))+(((px)*(sj2)*(x830)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x830)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x832=IKsin(j5);
IkReal x833=IKcos(j5);
IkReal x834=((cj2)*(sj1));
IkReal x835=((sj1)*(sj2));
IkReal x836=((cj1)*(r00));
IkReal x837=((IkReal(1.00000000000000))*(cj2));
IkReal x838=((cj1)*(r01));
IkReal x839=((cj1)*(py));
IkReal x840=((IkReal(1.00000000000000))*(sj2));
IkReal x841=((cj1)*(px));
IkReal x842=((sj3)*(x832));
IkReal x843=((cj3)*(x832));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(x843))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((IkReal(0.790000000000000))*(x843)))+(pz));
evalcond[2]=((((IkReal(-1.00000000000000))*(x836)*(x837)))+(((r00)*(x835)))+(((IkReal(-1.00000000000000))*(r01)*(x834)))+(((IkReal(-1.00000000000000))*(x838)*(x840)))+(x833));
evalcond[3]=((((IkReal(-1.00000000000000))*(x837)*(x838)))+(((r00)*(x834)))+(((r01)*(x835)))+(x842)+(((sj2)*(x836))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x837)*(x841)))+(((IkReal(-1.00000000000000))*(x839)*(x840)))+(((IkReal(-0.790000000000000))*(x833)))+(((IkReal(-1.00000000000000))*(py)*(x834)))+(((px)*(x835))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x837)*(x839)))+(((py)*(x835)))+(((IkReal(-0.790000000000000))*(x842)))+(((sj2)*(x841)))+(((IkReal(0.0800000000000000))*(cj3)))+(((px)*(x834))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj3)))))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(-8.00000000000000))*(sj3)))))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x844=IKsin(j5);
IkReal x845=IKcos(j5);
IkReal x846=((cj2)*(sj1));
IkReal x847=((sj1)*(sj2));
IkReal x848=((cj1)*(r00));
IkReal x849=((IkReal(1.00000000000000))*(cj2));
IkReal x850=((cj1)*(r01));
IkReal x851=((cj1)*(py));
IkReal x852=((IkReal(1.00000000000000))*(sj2));
IkReal x853=((cj1)*(px));
IkReal x854=((sj3)*(x844));
IkReal x855=((cj3)*(x844));
evalcond[0]=((((IkReal(-1.00000000000000))*(x855)))+(r02));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(pz)+(((IkReal(0.790000000000000))*(x855))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x850)*(x852)))+(((r00)*(x847)))+(((IkReal(-1.00000000000000))*(r01)*(x846)))+(x845)+(((IkReal(-1.00000000000000))*(x848)*(x849))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x849)*(x850)))+(((r01)*(x847)))+(((r00)*(x846)))+(((sj2)*(x848)))+(x854));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x849)*(x853)))+(((IkReal(-0.790000000000000))*(x845)))+(((IkReal(-1.00000000000000))*(py)*(x846)))+(((IkReal(-1.00000000000000))*(x851)*(x852)))+(((px)*(x847))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x854)))+(((sj2)*(x853)))+(((IkReal(-1.00000000000000))*(x849)*(x851)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x847)))+(((IkReal(0.0800000000000000))*(cj3)))+(((px)*(x846))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x856=IKsin(j5);
IkReal x857=IKcos(j5);
IkReal x858=((cj2)*(sj1));
IkReal x859=((sj1)*(sj2));
IkReal x860=((cj1)*(r00));
IkReal x861=((IkReal(1.00000000000000))*(cj2));
IkReal x862=((cj1)*(r01));
IkReal x863=((cj1)*(py));
IkReal x864=((IkReal(1.00000000000000))*(sj2));
IkReal x865=((cj1)*(px));
IkReal x866=((sj3)*(x856));
IkReal x867=((cj3)*(x856));
evalcond[0]=((((IkReal(-1.00000000000000))*(x867)))+(r02));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(pz)+(((IkReal(0.790000000000000))*(x867))));
evalcond[2]=((((r00)*(x859)))+(((IkReal(-1.00000000000000))*(r01)*(x858)))+(((IkReal(-1.00000000000000))*(x862)*(x864)))+(((IkReal(-1.00000000000000))*(x860)*(x861)))+(x857));
evalcond[3]=((((r00)*(x858)))+(((sj2)*(x860)))+(((IkReal(-1.00000000000000))*(x861)*(x862)))+(x866)+(((r01)*(x859))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(x857)))+(((IkReal(-1.00000000000000))*(x861)*(x865)))+(((IkReal(-1.00000000000000))*(py)*(x858)))+(((px)*(x859)))+(((IkReal(-1.00000000000000))*(x863)*(x864))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((sj2)*(x865)))+(((IkReal(-0.790000000000000))*(x866)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x861)*(x863)))+(((px)*(x858)))+(((IkReal(0.0800000000000000))*(cj3)))+(((py)*(x859))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x868=((IkReal(1.00000000000000))*(r01));
if( IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(8.00000000000000))*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x868)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x868)))+(((r00)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(8.00000000000000))*(sj3)))))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x868)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x868)))+(((r00)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(pz)))+(((IkReal(8.00000000000000))*(sj3)))))), ((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x868)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x868)))+(((r00)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x869=IKsin(j5);
IkReal x870=IKcos(j5);
IkReal x871=((cj2)*(sj1));
IkReal x872=((sj1)*(sj2));
IkReal x873=((cj1)*(r00));
IkReal x874=((IkReal(1.00000000000000))*(cj2));
IkReal x875=((cj1)*(r01));
IkReal x876=((cj1)*(py));
IkReal x877=((IkReal(1.00000000000000))*(sj2));
IkReal x878=((cj1)*(px));
IkReal x879=((sj3)*(x869));
IkReal x880=((cj3)*(x869));
evalcond[0]=((((IkReal(-1.00000000000000))*(x880)))+(r02));
evalcond[1]=((pz)+(((IkReal(0.790000000000000))*(x880)))+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x871)))+(((r00)*(x872)))+(((IkReal(-1.00000000000000))*(x870)))+(((IkReal(-1.00000000000000))*(x873)*(x874)))+(((IkReal(-1.00000000000000))*(x875)*(x877))));
evalcond[3]=((((sj2)*(x873)))+(((r01)*(x872)))+(((r00)*(x871)))+(((IkReal(-1.00000000000000))*(x874)*(x875)))+(x879));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x876)*(x877)))+(((IkReal(-1.00000000000000))*(py)*(x871)))+(((px)*(x872)))+(((IkReal(0.790000000000000))*(x870)))+(((IkReal(-1.00000000000000))*(x874)*(x878))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((sj2)*(x878)))+(((IkReal(-0.790000000000000))*(x879)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x871)))+(((py)*(x872)))+(((IkReal(-1.00000000000000))*(x874)*(x876)))+(((IkReal(-0.0800000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x881=((IkReal(1.26582278481013))*(sj1));
IkReal x882=((IkReal(1.26582278481013))*(cj1));
if( IKabs(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.512658227848101))+(((py)*(sj2)*(x882)))+(((cj2)*(px)*(x882)))+(((cj2)*(py)*(x881)))+(((IkReal(0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x881))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((IkReal(-0.512658227848101))+(((py)*(sj2)*(x882)))+(((cj2)*(px)*(x882)))+(((cj2)*(py)*(x881)))+(((IkReal(0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x881)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((IkReal(-0.512658227848101))+(((py)*(sj2)*(x882)))+(((cj2)*(px)*(x882)))+(((cj2)*(py)*(x881)))+(((IkReal(0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x881)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x883=IKsin(j5);
IkReal x884=IKcos(j5);
IkReal x885=((cj2)*(sj1));
IkReal x886=((sj1)*(sj2));
IkReal x887=((cj1)*(r00));
IkReal x888=((IkReal(1.00000000000000))*(cj2));
IkReal x889=((cj1)*(r01));
IkReal x890=((cj1)*(py));
IkReal x891=((IkReal(1.00000000000000))*(sj2));
IkReal x892=((cj1)*(px));
IkReal x893=((sj3)*(x883));
IkReal x894=((cj3)*(x883));
evalcond[0]=((((IkReal(-1.00000000000000))*(x894)))+(r02));
evalcond[1]=((((IkReal(0.790000000000000))*(x894)))+(pz)+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x887)*(x888)))+(((r00)*(x886)))+(((IkReal(-1.00000000000000))*(x884)))+(((IkReal(-1.00000000000000))*(r01)*(x885)))+(((IkReal(-1.00000000000000))*(x889)*(x891))));
evalcond[3]=((((r00)*(x885)))+(((r01)*(x886)))+(((sj2)*(x887)))+(((IkReal(-1.00000000000000))*(x888)*(x889)))+(x893));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x886)))+(((IkReal(-1.00000000000000))*(x888)*(x892)))+(((IkReal(0.790000000000000))*(x884)))+(((IkReal(-1.00000000000000))*(py)*(x885)))+(((IkReal(-1.00000000000000))*(x890)*(x891))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((px)*(x885)))+(((py)*(x886)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x893)))+(((sj2)*(x892)))+(((IkReal(-1.00000000000000))*(x888)*(x890)))+(((IkReal(-0.0800000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x895=((IkReal(1.00000000000000))*(r01));
if( IKabs(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x895)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x895)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x895)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x895)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x895)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x895)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((r00)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x896=IKsin(j5);
IkReal x897=IKcos(j5);
IkReal x898=((cj2)*(sj1));
IkReal x899=((sj1)*(sj2));
IkReal x900=((cj1)*(r00));
IkReal x901=((IkReal(1.00000000000000))*(cj2));
IkReal x902=((cj1)*(r01));
IkReal x903=((cj1)*(py));
IkReal x904=((IkReal(1.00000000000000))*(sj2));
IkReal x905=((cj1)*(px));
IkReal x906=((sj3)*(x896));
IkReal x907=((cj3)*(x896));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(x907))));
evalcond[1]=((((IkReal(0.790000000000000))*(x907)))+(pz)+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x900)*(x901)))+(((IkReal(-1.00000000000000))*(x902)*(x904)))+(((r00)*(x899)))+(((IkReal(-1.00000000000000))*(x897)))+(((IkReal(-1.00000000000000))*(r01)*(x898))));
evalcond[3]=((((sj2)*(x900)))+(((r01)*(x899)))+(((r00)*(x898)))+(((IkReal(-1.00000000000000))*(x901)*(x902)))+(x906));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x903)*(x904)))+(((px)*(x899)))+(((IkReal(0.790000000000000))*(x897)))+(((IkReal(-1.00000000000000))*(py)*(x898)))+(((IkReal(-1.00000000000000))*(x901)*(x905))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((py)*(x899)))+(((sj2)*(x905)))+(((px)*(x898)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x906)))+(((IkReal(-1.00000000000000))*(x901)*(x903)))+(((IkReal(-0.0800000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x908=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x909=((cj2)*(r00));
IkReal x910=((r01)*(sj2));
IkReal x911=((cj2)*(r01)*(sj1));
IkReal x912=((IkReal(79.0000000000000))*(cj4)*(sj3));
IkReal x913=((r00)*(sj1)*(sj2));
IkReal x914=((cj1)*(x912));
if( IKabs(((IkReal(0.0126582278481013))*(x908)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-8.00000000000000))*(sj3)*(((IKabs(x908) != 0)?((IkReal)1/((x908)*(x908))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x910)*(x914)))+(((IkReal(-100.000000000000))*(pz)*(sj4)))+(((x912)*(x913)))+(((IkReal(-1.00000000000000))*(x911)*(x912)))+(((IkReal(-1.00000000000000))*(x909)*(x914))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x908)*(((((cj1)*(x909)))+(((IkReal(-1.00000000000000))*(x913)))+(x911)+(((cj1)*(x910))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(x908)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-8.00000000000000))*(sj3)*(((IKabs(x908) != 0)?((IkReal)1/((x908)*(x908))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x910)*(x914)))+(((IkReal(-100.000000000000))*(pz)*(sj4)))+(((x912)*(x913)))+(((IkReal(-1.00000000000000))*(x911)*(x912)))+(((IkReal(-1.00000000000000))*(x909)*(x914)))))))+IKsqr(((x908)*(((((cj1)*(x909)))+(((IkReal(-1.00000000000000))*(x913)))+(x911)+(((cj1)*(x910)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(x908)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-8.00000000000000))*(sj3)*(((IKabs(x908) != 0)?((IkReal)1/((x908)*(x908))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x910)*(x914)))+(((IkReal(-100.000000000000))*(pz)*(sj4)))+(((x912)*(x913)))+(((IkReal(-1.00000000000000))*(x911)*(x912)))+(((IkReal(-1.00000000000000))*(x909)*(x914)))))), ((x908)*(((((cj1)*(x909)))+(((IkReal(-1.00000000000000))*(x913)))+(x911)+(((cj1)*(x910)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x915=IKcos(j5);
IkReal x916=IKsin(j5);
IkReal x917=((cj2)*(sj1));
IkReal x918=((sj1)*(sj2));
IkReal x919=((cj1)*(r00));
IkReal x920=((IkReal(0.0800000000000000))*(sj4));
IkReal x921=((IkReal(1.00000000000000))*(cj2));
IkReal x922=((IkReal(0.790000000000000))*(cj3));
IkReal x923=((cj1)*(r01));
IkReal x924=((cj1)*(py));
IkReal x925=((IkReal(1.00000000000000))*(cj3));
IkReal x926=((IkReal(1.00000000000000))*(sj2));
IkReal x927=((cj1)*(px));
IkReal x928=((sj4)*(x915));
IkReal x929=((sj3)*(x916));
IkReal x930=((cj4)*(x915));
evalcond[0]=((r02)+(((IkReal(-1.00000000000000))*(x916)*(x925)))+(((IkReal(-1.00000000000000))*(sj3)*(x930))));
evalcond[1]=((((x916)*(x922)))+(pz)+(((sj3)*(x920)))+(((IkReal(0.790000000000000))*(sj3)*(x930))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x917)))+(((IkReal(-1.00000000000000))*(x919)*(x921)))+(x928)+(((IkReal(-1.00000000000000))*(x923)*(x926)))+(((r00)*(x918))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x925)*(x930)))+(((sj2)*(x919)))+(((IkReal(-1.00000000000000))*(x921)*(x923)))+(x929)+(((r01)*(x918)))+(((r00)*(x917))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x918)))+(((IkReal(-0.790000000000000))*(x928)))+(((IkReal(-1.00000000000000))*(x924)*(x926)))+(((IkReal(-1.00000000000000))*(x921)*(x927)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x917))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((px)*(x917)))+(((IkReal(-0.790000000000000))*(x929)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj3)*(x920)))+(((IkReal(-1.00000000000000))*(x921)*(x924)))+(((x922)*(x930)))+(((sj2)*(x927)))+(((py)*(x918))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x931=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x932=((IkReal(1.00000000000000))*(r01));
IkReal x933=((r00)*(sj1));
IkReal x934=((cj2)*(sj1));
IkReal x935=((cj4)*(sj3));
IkReal x936=((sj2)*(x935));
IkReal x937=((cj1)*(cj2)*(r00));
if( IKabs(((x931)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x932)*(x934)*(x935)))+(((IkReal(-1.00000000000000))*(cj1)*(x932)*(x936)))+(((r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x935)*(x937)))+(((x933)*(x936))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x931)*(((((cj1)*(r01)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x933)))+(((r01)*(x934)))+(x937))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x931)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x932)*(x934)*(x935)))+(((IkReal(-1.00000000000000))*(cj1)*(x932)*(x936)))+(((r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x935)*(x937)))+(((x933)*(x936)))))))+IKsqr(((x931)*(((((cj1)*(r01)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x933)))+(((r01)*(x934)))+(x937)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x931)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x932)*(x934)*(x935)))+(((IkReal(-1.00000000000000))*(cj1)*(x932)*(x936)))+(((r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x935)*(x937)))+(((x933)*(x936)))))), ((x931)*(((((cj1)*(r01)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x933)))+(((r01)*(x934)))+(x937)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x938=IKcos(j5);
IkReal x939=IKsin(j5);
IkReal x940=((cj2)*(sj1));
IkReal x941=((sj1)*(sj2));
IkReal x942=((cj1)*(r00));
IkReal x943=((IkReal(0.0800000000000000))*(sj4));
IkReal x944=((IkReal(1.00000000000000))*(cj2));
IkReal x945=((IkReal(0.790000000000000))*(cj3));
IkReal x946=((cj1)*(r01));
IkReal x947=((cj1)*(py));
IkReal x948=((IkReal(1.00000000000000))*(cj3));
IkReal x949=((IkReal(1.00000000000000))*(sj2));
IkReal x950=((cj1)*(px));
IkReal x951=((sj4)*(x938));
IkReal x952=((sj3)*(x939));
IkReal x953=((cj4)*(x938));
evalcond[0]=((((IkReal(-1.00000000000000))*(x939)*(x948)))+(r02)+(((IkReal(-1.00000000000000))*(sj3)*(x953))));
evalcond[1]=((((sj3)*(x943)))+(((x939)*(x945)))+(pz)+(((IkReal(0.790000000000000))*(sj3)*(x953))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x940)))+(((IkReal(-1.00000000000000))*(x946)*(x949)))+(((IkReal(-1.00000000000000))*(x942)*(x944)))+(((r00)*(x941)))+(x951));
evalcond[3]=((((sj2)*(x942)))+(((r01)*(x941)))+(((r00)*(x940)))+(((IkReal(-1.00000000000000))*(x944)*(x946)))+(x952)+(((IkReal(-1.00000000000000))*(x948)*(x953))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(x951)))+(((IkReal(-1.00000000000000))*(x944)*(x950)))+(((IkReal(-1.00000000000000))*(x947)*(x949)))+(((px)*(x941)))+(((IkReal(-1.00000000000000))*(py)*(x940)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x952)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj3)*(x943)))+(((IkReal(-1.00000000000000))*(x944)*(x947)))+(((px)*(x940)))+(((sj2)*(x950)))+(((x945)*(x953)))+(((py)*(x941))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x954=(sj3)*(sj3);
IkReal x955=(cj3)*(cj3);
IkReal x956=((r01)*(sj1));
IkReal x957=((cj1)*(r00));
IkReal x958=((IkReal(1.00000000000000))*(cj2));
IkReal x959=((cj1)*(r01));
IkReal x960=((r00)*(sj1));
IkReal x961=((cj3)*(sj2));
IkReal x962=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((((IKabs(((x955)+(x954))) != 0)?((IkReal)1/(((x955)+(x954)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x958)*(x960)))+(((IkReal(-1.00000000000000))*(x956)*(x962)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x957)*(x962)))+(((cj2)*(sj3)*(x959))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x954)))+(((cj4)*(x955))))) != 0)?((IkReal)1/(((((cj4)*(x954)))+(((cj4)*(x955)))))):(IkReal)1.0e30))*(((((r02)*(sj3)))+(((x957)*(x961)))+(((x956)*(x961)))+(((cj2)*(cj3)*(x960)))+(((IkReal(-1.00000000000000))*(cj3)*(x958)*(x959))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x955)+(x954))) != 0)?((IkReal)1/(((x955)+(x954)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x958)*(x960)))+(((IkReal(-1.00000000000000))*(x956)*(x962)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x957)*(x962)))+(((cj2)*(sj3)*(x959)))))))+IKsqr(((((IKabs(((((cj4)*(x954)))+(((cj4)*(x955))))) != 0)?((IkReal)1/(((((cj4)*(x954)))+(((cj4)*(x955)))))):(IkReal)1.0e30))*(((((r02)*(sj3)))+(((x957)*(x961)))+(((x956)*(x961)))+(((cj2)*(cj3)*(x960)))+(((IkReal(-1.00000000000000))*(cj3)*(x958)*(x959)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x955)+(x954))) != 0)?((IkReal)1/(((x955)+(x954)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj3)*(x958)*(x960)))+(((IkReal(-1.00000000000000))*(x956)*(x962)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x957)*(x962)))+(((cj2)*(sj3)*(x959)))))), ((((IKabs(((((cj4)*(x954)))+(((cj4)*(x955))))) != 0)?((IkReal)1/(((((cj4)*(x954)))+(((cj4)*(x955)))))):(IkReal)1.0e30))*(((((r02)*(sj3)))+(((x957)*(x961)))+(((x956)*(x961)))+(((cj2)*(cj3)*(x960)))+(((IkReal(-1.00000000000000))*(cj3)*(x958)*(x959)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x963=IKcos(j5);
IkReal x964=IKsin(j5);
IkReal x965=((cj2)*(sj1));
IkReal x966=((sj1)*(sj2));
IkReal x967=((cj1)*(r00));
IkReal x968=((IkReal(0.0800000000000000))*(sj4));
IkReal x969=((IkReal(1.00000000000000))*(cj2));
IkReal x970=((IkReal(0.790000000000000))*(cj3));
IkReal x971=((cj1)*(r01));
IkReal x972=((cj1)*(py));
IkReal x973=((IkReal(1.00000000000000))*(cj3));
IkReal x974=((IkReal(1.00000000000000))*(sj2));
IkReal x975=((cj1)*(px));
IkReal x976=((sj4)*(x963));
IkReal x977=((sj3)*(x964));
IkReal x978=((cj4)*(x963));
evalcond[0]=((((IkReal(-1.00000000000000))*(x964)*(x973)))+(((IkReal(-1.00000000000000))*(sj3)*(x978)))+(r02));
evalcond[1]=((((IkReal(0.790000000000000))*(sj3)*(x978)))+(pz)+(((sj3)*(x968)))+(((x964)*(x970))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x967)*(x969)))+(((IkReal(-1.00000000000000))*(x971)*(x974)))+(((r00)*(x966)))+(((IkReal(-1.00000000000000))*(r01)*(x965)))+(x976));
evalcond[3]=((((IkReal(-1.00000000000000))*(x969)*(x971)))+(((r00)*(x965)))+(((r01)*(x966)))+(((sj2)*(x967)))+(x977)+(((IkReal(-1.00000000000000))*(x973)*(x978))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((px)*(x966)))+(((IkReal(-0.790000000000000))*(x976)))+(((IkReal(-1.00000000000000))*(x969)*(x975)))+(((IkReal(-1.00000000000000))*(x972)*(x974)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x965))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((px)*(x965)))+(((IkReal(-0.790000000000000))*(x977)))+(((IkReal(-1.00000000000000))*(x969)*(x972)))+(((IkReal(-0.365000000000000))*(cj2)))+(((py)*(x966)))+(((cj3)*(x968)))+(((x970)*(x978)))+(((sj2)*(x975))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "8529cf64e1f47189a44788609179f24f"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
