/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:02:22.963508
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44,x45,x46,x47,x48,x49,x50,x51,x52,x53,x54,x55,x56,x57,x58,x59,x60,x61,x62,x63,x64,x65,x66,x67,x68,x69,x70,x71,x72,x73;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[0]);
x4=IKsin(j[2]);
x5=IKsin(j[3]);
x6=IKcos(j[3]);
x7=IKsin(j[1]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[6]);
x11=IKsin(j[5]);
x12=IKcos(j[5]);
x13=IKcos(j[6]);
x14=((IkReal(0.0600000000000000))*(x5));
x15=((IkReal(0.0450000000000000))*(x1));
x16=((IkReal(1.00000000000000))*(x5));
x17=((IkReal(0.0600000000000000))*(x9));
x18=((IkReal(0.300000000000000))*(x1));
x19=((IkReal(0.0600000000000000))*(x6));
x20=((IkReal(1.00000000000000))*(x11));
x21=((IkReal(0.0450000000000000))*(x5));
x22=((IkReal(1.00000000000000))*(x1));
x23=((IkReal(1.00000000000000))*(x12));
x24=((IkReal(0.0600000000000000))*(x8));
x25=((IkReal(1.00000000000000))*(x6));
x26=((x0)*(x4));
x27=((x0)*(x2));
x28=((x2)*(x7));
x29=((x3)*(x4));
x30=((x0)*(x7));
x31=((x2)*(x3));
x32=((x3)*(x7));
x33=((IkReal(1.00000000000000))*(x29));
x34=((IkReal(0.0450000000000000))*(x29));
x35=((IkReal(0.0450000000000000))*(x26));
x36=((x22)*(x6));
x37=((x32)*(x6));
x38=((x4)*(x7)*(x9));
x39=((x16)*(x30));
x40=((x25)*(x30));
x41=((x15)*(x27));
x42=((x16)*(x32));
x43=((x25)*(x32));
x44=((x15)*(x31));
x45=((((IkReal(-1.00000000000000))*(x33)))+(((x1)*(x27))));
x46=((x26)+(((x1)*(x31))));
x47=((x27)+(((IkReal(-1.00000000000000))*(x22)*(x29))));
x48=((((IkReal(-1.00000000000000))*(x36)))+(((x28)*(x5))));
x49=((x33)+(((IkReal(-1.00000000000000))*(x22)*(x27))));
x50=((((IkReal(-1.00000000000000))*(x31)))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x51=((((IkReal(-1.00000000000000))*(x26)))+(((IkReal(-1.00000000000000))*(x22)*(x31))));
x52=((((x1)*(x16)))+(((x25)*(x28))));
x53=((IkReal(-1.00000000000000))*(x52));
x54=((x35)+(x44));
x55=((x46)*(x6));
x56=((x11)*(x48));
x57=((x45)*(x6));
x58=((x5)*(x51));
x59=((x50)*(x9));
x60=((x53)*(x8));
x61=((((IkReal(-1.00000000000000))*(x39)))+(x57));
x62=((((IkReal(-1.00000000000000))*(x42)))+(x55));
x63=((((IkReal(-1.00000000000000))*(x40)))+(((x49)*(x5))));
x64=((((IkReal(-1.00000000000000))*(x43)))+(x58));
x65=((((x4)*(x7)*(x8)))+(((x52)*(x9))));
x66=((x38)+(x60));
x67=((x61)*(x8));
x68=((x11)*(x64));
x69=((((x47)*(x9)))+(((x62)*(x8))));
x70=((((x47)*(x8)))+(((x9)*(((((IkReal(-1.00000000000000))*(x25)*(x46)))+(x42))))));
x71=((x59)+(x67));
x72=((x12)*(x69));
x73=((((x50)*(x8)))+(((x9)*(((x39)+(((IkReal(-1.00000000000000))*(x57))))))));
eerot[0]=((((x10)*(x73)))+(((x13)*(((((x11)*(x63)))+(((x12)*(x71))))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x63)))+(((IkReal(-1.00000000000000))*(x23)*(x71)))))))+(((x13)*(x73))));
eerot[2]=((((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x49)))+(x40)))))+(((x11)*(x71))));
eetrans[0]=((IkReal(0.220000000000000))+(((IkReal(-1.00000000000000))*(x34)))+(((x12)*(((((x19)*(x30)))+(((IkReal(-1.00000000000000))*(x14)*(x49)))))))+(((x11)*(((((x17)*(x50)))+(((x24)*(x61)))))))+(((x21)*(x30)))+(((IkReal(0.300000000000000))*(x30)*(x6)))+(((IkReal(0.550000000000000))*(x30)))+(x41)+(((x5)*(((((x18)*(x27)))+(((IkReal(-0.300000000000000))*(x29)))))))+(((x6)*(((((IkReal(-1.00000000000000))*(x41)))+(x34))))));
eerot[3]=((((x10)*(x70)))+(((x13)*(((x72)+(x68))))));
eerot[4]=((((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x64)))+(((IkReal(-1.00000000000000))*(x23)*(x69)))))))+(((x13)*(x70))));
eerot[5]=((((x11)*(x69)))+(((x12)*(((x43)+(((IkReal(-1.00000000000000))*(x16)*(x51))))))));
eetrans[1]=((IkReal(0.140000000000000))+(((x5)*(((((x18)*(x31)))+(((IkReal(0.300000000000000))*(x26)))))))+(((IkReal(-1.00000000000000))*(x54)*(x6)))+(((x21)*(x32)))+(((IkReal(0.550000000000000))*(x32)))+(x54)+(((x12)*(((((x19)*(x32)))+(((IkReal(-1.00000000000000))*(x14)*(x51)))))))+(((x11)*(((((x17)*(x47)))+(((x24)*(x62)))))))+(((IkReal(0.300000000000000))*(x37))));
eerot[6]=((((x13)*(((((x12)*(x66)))+(x56)))))+(((x10)*(x65))));
eerot[7]=((((x13)*(x65)))+(((x10)*(((((IkReal(-1.00000000000000))*(x20)*(x48)))+(((IkReal(-1.00000000000000))*(x23)*(x66))))))));
eerot[8]=((((x11)*(x66)))+(((x12)*(((((IkReal(-1.00000000000000))*(x16)*(x28)))+(x36))))));
IkReal x74=((IkReal(0.0450000000000000))*(x28));
eetrans[2]=((IkReal(0.346000000000000))+(((x18)*(x6)))+(((x11)*(((((x24)*(x53)))+(((x17)*(x4)*(x7)))))))+(((IkReal(-0.300000000000000))*(x28)*(x5)))+(((IkReal(-1.00000000000000))*(x74)))+(((x12)*(((((x1)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x28)))))))+(((x6)*(x74)))+(((IkReal(0.550000000000000))*(x1)))+(((x15)*(x5))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j1,cj1,sj1,htj1,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=r00;
new_r01=r01;
new_r02=r02;
new_px=((IkReal(-0.220000000000000))+(((IkReal(-0.0600000000000000))*(r02)))+(px));
new_r10=r10;
new_r11=r11;
new_r12=r12;
new_py=((IkReal(-0.140000000000000))+(py)+(((IkReal(-0.0600000000000000))*(r12))));
new_r20=r20;
new_r21=r21;
new_r22=r22;
new_pz=((IkReal(-0.346000000000000))+(((IkReal(-0.0600000000000000))*(r22)))+(pz));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x75=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j3array[0]=((IkReal(-1.34027003705633))+(((IkReal(-1.00000000000000))*(x75))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(1.80132261653346))+(x75));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
IkReal x76=(sj1)*(sj1);
dummyeval[0]=((((x76)*((px)*(px))))+(((x76)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x77=(sj1)*(sj1);
dummyeval[0]=((((x77)*((px)*(px))))+(((x77)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x78=(sj1)*(sj1);
dummyeval[0]=((((x78)*((px)*(px)*(px)*(px))))+(((x78)*((px)*(px))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj3))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=((IkReal(-1.00000000000000))*(py));
IkReal gconst4;
gconst4=((IkReal(2.00000000000000))*(px));
IkReal gconst5;
gconst5=py;
IkReal gconst6;
gconst6=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst7;
gconst7=((IkReal(-2.00000000000000))*(py));
IkReal gconst8;
gconst8=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst9;
gconst9=((IkReal(-1.00000000000000))*(py));
IkReal gconst10;
gconst10=((IkReal(2.00000000000000))*(px));
IkReal gconst11;
gconst11=py;
IkReal gconst12;
gconst12=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst13;
gconst13=((IkReal(-2.00000000000000))*(py));
IkReal gconst14;
gconst14=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst15;
gconst15=((IkReal(2.00000000000000))*(px));
IkReal gconst16;
gconst16=((IkReal(4.00000000000000))*(py));
IkReal gconst17;
gconst17=((IkReal(-2.00000000000000))*(px));
IkReal gconst18;
gconst18=((IkReal(-2.00000000000000))*(py));
IkReal gconst19;
gconst19=((IkReal(4.00000000000000))*(px));
IkReal gconst20;
gconst20=((IkReal(2.00000000000000))*(py));
IkReal gconst21;
gconst21=((IkReal(2.00000000000000))*(px));
IkReal gconst22;
gconst22=((IkReal(4.00000000000000))*(py));
IkReal gconst23;
gconst23=((IkReal(-2.00000000000000))*(px));
IkReal gconst24;
gconst24=((IkReal(-2.00000000000000))*(py));
IkReal gconst25;
gconst25=((IkReal(4.00000000000000))*(px));
IkReal gconst26;
gconst26=((IkReal(2.00000000000000))*(py));
IkReal gconst27;
gconst27=py;
IkReal gconst28;
gconst28=((IkReal(-2.00000000000000))*(px));
IkReal gconst29;
gconst29=((IkReal(-1.00000000000000))*(py));
IkReal gconst30;
gconst30=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst31;
gconst31=((IkReal(2.00000000000000))*(py));
IkReal gconst32;
gconst32=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst33;
gconst33=py;
IkReal gconst34;
gconst34=((IkReal(-2.00000000000000))*(px));
IkReal gconst35;
gconst35=((IkReal(-1.00000000000000))*(py));
IkReal gconst36;
gconst36=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst37;
gconst37=((IkReal(2.00000000000000))*(py));
IkReal gconst38;
gconst38=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal x79=((gconst33)*(gconst37));
IkReal x80=((gconst28)*(gconst32));
IkReal x81=((gconst29)*(gconst31));
IkReal x82=((gconst34)*(gconst36));
IkReal x83=((gconst27)*(gconst32));
IkReal x84=((gconst33)*(gconst38));
IkReal x85=((gconst35)*(gconst36));
IkReal x86=((gconst29)*(gconst30));
dummyeval[0]=((((x84)*(x86)))+(((x79)*(x80)))+(((x81)*(x82)))+(((IkReal(-1.00000000000000))*(x80)*(x82)))+(((IkReal(-1.00000000000000))*(x85)*(x86)))+(((IkReal(-1.00000000000000))*(x79)*(x81)))+(((x83)*(x85)))+(((IkReal(-1.00000000000000))*(x83)*(x84))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x87=((gconst32)*(gconst36));
IkReal x88=((gconst32)*(gconst33));
IkReal x89=((gconst28)*(gconst37));
IkReal x90=((IkReal(1.00000000000000))*(gconst34));
IkReal x91=((gconst33)*(gconst38));
IkReal x92=((gconst17)*(gconst30));
IkReal x93=((gconst26)*(gconst9));
IkReal x94=((gconst5)*(gconst6));
IkReal x95=((IkReal(1.00000000000000))*(gconst8));
IkReal x96=((gconst14)*(gconst9));
IkReal x97=((gconst18)*(gconst5));
IkReal x98=((IkReal(1.00000000000000))*(gconst27));
IkReal x99=((gconst10)*(gconst12));
IkReal x100=((gconst17)*(gconst7));
IkReal x101=((gconst17)*(gconst6));
IkReal x102=((gconst5)*(gconst7));
IkReal x103=((gconst12)*(gconst3));
IkReal x104=((gconst13)*(gconst9));
IkReal x105=((gconst27)*(gconst35));
IkReal x106=((gconst14)*(gconst21));
IkReal x107=((gconst12)*(gconst22));
IkReal x108=((gconst4)*(gconst8));
IkReal x109=((gconst13)*(gconst21));
IkReal x110=((gconst31)*(gconst36));
IkReal x111=((gconst20)*(gconst36));
IkReal x112=((gconst21)*(gconst38));
IkReal x113=((IkReal(1.00000000000000))*(gconst20));
IkReal x114=((IkReal(1.00000000000000))*(gconst29));
IkReal x115=((gconst33)*(gconst37));
IkReal x116=((gconst30)*(gconst35));
IkReal x117=((gconst29)*(gconst30));
IkReal x118=((gconst25)*(gconst9));
IkReal x119=((gconst11)*(gconst24));
IkReal x120=((gconst24)*(gconst32));
IkReal x121=((gconst10)*(gconst24));
IkReal x122=((gconst11)*(gconst8));
IkReal x123=((gconst19)*(gconst5));
IkReal x124=((gconst29)*(gconst31)*(gconst34));
IkReal x125=((IkReal(1.00000000000000))*(gconst11)*(gconst12));
IkReal x126=((gconst36)*(x114));
IkReal x127=((gconst31)*(gconst33)*(x114));
op[0]=((((IkReal(-1.00000000000000))*(gconst28)*(x87)*(x90)))+(((x117)*(x91)))+(((IkReal(-1.00000000000000))*(gconst38)*(x88)*(x98)))+(((IkReal(-1.00000000000000))*(x116)*(x126)))+(((IkReal(-1.00000000000000))*(gconst31)*(x114)*(x115)))+(((gconst29)*(gconst34)*(x110)))+(((x88)*(x89)))+(((x105)*(x87))));
op[1]=((((IkReal(-1.00000000000000))*(gconst26)*(x88)*(x98)))+(((gconst23)*(gconst27)*(x87)))+(((gconst20)*(gconst33)*(x89)))+(((IkReal(-1.00000000000000))*(gconst32)*(x112)*(x98)))+(((x112)*(x117)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst28)*(x87)))+(((IkReal(-1.00000000000000))*(gconst35)*(gconst36)*(x92)))+(((IkReal(-1.00000000000000))*(gconst28)*(x111)*(x90)))+(((IkReal(-1.00000000000000))*(gconst19)*(x114)*(x115)))+(((IkReal(-1.00000000000000))*(gconst25)*(x127)))+(((gconst24)*(x124)))+(((gconst21)*(gconst32)*(x89)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst31)*(gconst37)*(x114)))+(((gconst16)*(gconst37)*(x88)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst30)*(x126)))+(((IkReal(-1.00000000000000))*(gconst16)*(x87)*(x90)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst35)*(x126)))+(((gconst15)*(gconst35)*(x87)))+(((gconst19)*(gconst29)*(gconst34)*(gconst36)))+(((x105)*(x120)))+(((gconst26)*(gconst33)*(x117)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst31)*(x115)))+(((gconst18)*(gconst29)*(x91)))+(((gconst22)*(gconst29)*(x110)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst38)*(x88)))+(((gconst25)*(gconst28)*(x88)))+(((x105)*(x111)))+(((IkReal(-1.00000000000000))*(gconst28)*(x120)*(x90)))+(((gconst17)*(gconst34)*(x110)))+(((IkReal(-1.00000000000000))*(gconst24)*(x114)*(x116)))+(((x91)*(x92)))+(((IkReal(-1.00000000000000))*(gconst20)*(x91)*(x98))));
op[2]=((((gconst16)*(gconst21)*(gconst32)*(gconst37)))+(((gconst27)*(gconst35)*(gconst36)*(gconst8)))+(((gconst10)*(gconst29)*(gconst31)*(gconst36)))+(((gconst21)*(gconst25)*(gconst28)*(gconst32)))+(((gconst18)*(gconst21)*(gconst29)*(gconst38)))+(((gconst16)*(gconst20)*(gconst33)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst24)*(gconst32)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst29)*(gconst35)))+(((gconst21)*(gconst26)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst30)*(gconst36)))+(((gconst20)*(gconst24)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst34)*(gconst36)))+(((gconst12)*(gconst27)*(gconst32)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst31)))+(((gconst30)*(gconst33)*(gconst38)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst30)*(gconst35)))+(((gconst31)*(gconst34)*(gconst36)*(gconst5)))+(((gconst17)*(gconst22)*(gconst31)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst33)*(gconst38)))+(((gconst28)*(gconst32)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst32)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst29)*(gconst30)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst28)*(gconst32)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst24)*(gconst28)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst33)))+(((gconst16)*(gconst25)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst31)*(gconst37)*(gconst9)))+(((gconst17)*(gconst18)*(gconst33)*(gconst38)))+(((gconst28)*(gconst33)*(gconst37)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst32)*(gconst34)*(gconst36)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst33)*(gconst37)))+(((gconst29)*(gconst30)*(gconst38)*(gconst9)))+(((gconst11)*(gconst27)*(gconst32)*(gconst36)))+(((gconst18)*(gconst26)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst30)*(gconst35)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst33)*(gconst37)*(gconst7)))+(((gconst29)*(gconst33)*(gconst38)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst31)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst32)*(gconst33)*(gconst38)))+(((IkReal(-1.00000000000000))*(gconst28)*(gconst34)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst28)*(gconst32)))+(((gconst14)*(gconst29)*(gconst30)*(gconst33)))+(((gconst17)*(gconst26)*(gconst30)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst31)*(gconst33)*(gconst37)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst32)*(gconst38)))+(((gconst19)*(gconst24)*(gconst29)*(gconst34)))+(((gconst15)*(gconst23)*(gconst32)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst29)*(gconst37)))+(((gconst15)*(gconst24)*(gconst32)*(gconst35)))+(((gconst13)*(gconst28)*(gconst32)*(gconst33)))+(((gconst20)*(gconst21)*(gconst28)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst28)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst33)*(gconst38)*(gconst8)))+(((gconst12)*(gconst29)*(gconst31)*(gconst34)))+(((gconst23)*(gconst24)*(gconst27)*(gconst32)))+(((gconst22)*(gconst24)*(gconst29)*(gconst31)))+(((gconst17)*(gconst19)*(gconst34)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst32)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst35)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst33)))+(((gconst19)*(gconst22)*(gconst29)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst35)*(gconst36)))+(((gconst29)*(gconst34)*(gconst36)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst28)*(gconst32)*(gconst34)))+(((gconst32)*(gconst33)*(gconst37)*(gconst4)))+(((gconst3)*(gconst32)*(gconst35)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst27)*(gconst38)))+(((gconst20)*(gconst25)*(gconst28)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst29)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst33)))+(((gconst15)*(gconst20)*(gconst35)*(gconst36)))+(((gconst20)*(gconst23)*(gconst27)*(gconst36)))+(((gconst17)*(gconst24)*(gconst31)*(gconst34)))+(((gconst17)*(gconst21)*(gconst30)*(gconst38))));
op[3]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst23)*(gconst27)*(gconst36)*(gconst8)))+(((gconst19)*(gconst22)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst34)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst34)*(gconst36)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((gconst17)*(gconst18)*(gconst21)*(gconst38)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst29)*(gconst37)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst32)*(gconst34)*(gconst4)))+(((gconst20)*(gconst28)*(gconst37)*(gconst9)))+(((gconst26)*(gconst29)*(gconst33)*(gconst6)))+(((gconst20)*(gconst3)*(gconst35)*(gconst36)))+(((gconst24)*(gconst27)*(gconst35)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst15)*(gconst20)*(gconst24)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst35)*(gconst36)*(gconst5)))+(((gconst12)*(gconst17)*(gconst31)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst29)*(gconst37)*(gconst9)))+(((gconst16)*(gconst20)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst24)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst33)*(gconst38)*(gconst8)))+(((gconst12)*(gconst22)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst24)*(gconst34)))+(((gconst15)*(gconst20)*(gconst23)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((gconst24)*(gconst31)*(gconst34)*(gconst5)))+(((gconst24)*(gconst29)*(gconst34)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst11)*(gconst15)*(gconst32)*(gconst36)))+(((gconst14)*(gconst18)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((gconst20)*(gconst33)*(gconst37)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst32)*(gconst36)))+(((gconst17)*(gconst33)*(gconst38)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst33)*(gconst37)*(gconst5)))+(((gconst21)*(gconst29)*(gconst38)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((gconst17)*(gconst19)*(gconst22)*(gconst36)))+(((gconst17)*(gconst21)*(gconst26)*(gconst30)))+(((gconst20)*(gconst23)*(gconst24)*(gconst27)))+(((gconst10)*(gconst17)*(gconst31)*(gconst36)))+(((gconst17)*(gconst19)*(gconst24)*(gconst34)))+(((gconst26)*(gconst29)*(gconst30)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst28)*(gconst34)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst29)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst29)*(gconst36)))+(((gconst11)*(gconst24)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst32)*(gconst36)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst32)*(gconst34)))+(((gconst14)*(gconst21)*(gconst29)*(gconst30)))+(((gconst15)*(gconst23)*(gconst24)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst17)*(gconst22)*(gconst24)*(gconst31)))+(((gconst16)*(gconst33)*(gconst37)*(gconst8)))+(((gconst16)*(gconst32)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst30)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst28)*(gconst36)*(gconst8)))+(((gconst17)*(gconst18)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((gconst22)*(gconst29)*(gconst36)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst12)*(gconst23)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst3)*(gconst33)*(gconst38)))+(((gconst16)*(gconst20)*(gconst21)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst35)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst31)*(gconst37)*(gconst5)))+(((gconst10)*(gconst24)*(gconst29)*(gconst31)))+(((gconst19)*(gconst34)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst28)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst27)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst38)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst33)*(gconst37)*(gconst7)))+(((gconst16)*(gconst21)*(gconst25)*(gconst32)))+(((gconst25)*(gconst28)*(gconst33)*(gconst8)))+(((gconst24)*(gconst3)*(gconst32)*(gconst35)))+(((gconst12)*(gconst15)*(gconst32)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst30)*(gconst36)*(gconst5)))+(((gconst13)*(gconst20)*(gconst28)*(gconst33)))+(((gconst21)*(gconst28)*(gconst37)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((gconst13)*(gconst16)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst10)*(gconst19)*(gconst29)*(gconst36)))+(((gconst20)*(gconst21)*(gconst25)*(gconst28)))+(((gconst22)*(gconst31)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst22)*(gconst36)))+(((gconst21)*(gconst32)*(gconst37)*(gconst4)))+(((gconst12)*(gconst19)*(gconst29)*(gconst34)))+(((gconst11)*(gconst20)*(gconst27)*(gconst36)))+(((gconst18)*(gconst21)*(gconst26)*(gconst29)))+(((gconst12)*(gconst20)*(gconst27)*(gconst35)))+(((gconst13)*(gconst21)*(gconst28)*(gconst32)))+(((gconst17)*(gconst30)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((gconst14)*(gconst17)*(gconst30)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst31)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst22)*(gconst28)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst3)*(gconst32)*(gconst38)))+(((gconst23)*(gconst3)*(gconst32)*(gconst36)))+(((gconst21)*(gconst30)*(gconst38)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst36)))+(((gconst18)*(gconst33)*(gconst38)*(gconst5)))+(((gconst25)*(gconst32)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst15)*(gconst35)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst28)*(gconst34)))+(((gconst17)*(gconst34)*(gconst36)*(gconst7)))+(((gconst18)*(gconst29)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst24)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst32)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst28)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst27)*(gconst38)*(gconst8)))+(((gconst26)*(gconst30)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst37)))+(((gconst25)*(gconst28)*(gconst32)*(gconst9))));
op[4]=((((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst29)*(gconst38)*(gconst6)*(gconst9)))+(((gconst15)*(gconst24)*(gconst35)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((gconst33)*(gconst38)*(gconst5)*(gconst6)))+(((gconst13)*(gconst16)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst28)*(gconst36)*(gconst8)))+(((gconst12)*(gconst29)*(gconst34)*(gconst7)))+(((gconst12)*(gconst15)*(gconst23)*(gconst32)))+(((gconst11)*(gconst15)*(gconst24)*(gconst32)))+(((gconst19)*(gconst22)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst11)*(gconst15)*(gconst20)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst30)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst33)*(gconst37)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((gconst10)*(gconst12)*(gconst29)*(gconst31)))+(((gconst12)*(gconst31)*(gconst34)*(gconst5)))+(((gconst22)*(gconst24)*(gconst31)*(gconst5)))+(((gconst17)*(gconst22)*(gconst36)*(gconst7)))+(((gconst17)*(gconst19)*(gconst22)*(gconst24)))+(((gconst10)*(gconst31)*(gconst36)*(gconst5)))+(((gconst23)*(gconst24)*(gconst3)*(gconst32)))+(((gconst12)*(gconst20)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst38)*(gconst8)))+(((gconst17)*(gconst18)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst16)*(gconst21)*(gconst37)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst29)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst29)*(gconst33)*(gconst6)))+(((gconst10)*(gconst29)*(gconst36)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst24)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst38)*(gconst8)*(gconst9)))+(((gconst11)*(gconst12)*(gconst27)*(gconst32)))+(((gconst13)*(gconst32)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst24)*(gconst32)))+(((gconst11)*(gconst20)*(gconst24)*(gconst27)))+(((gconst16)*(gconst25)*(gconst32)*(gconst9)))+(((gconst20)*(gconst23)*(gconst3)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst22)*(gconst28)))+(((gconst21)*(gconst25)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((gconst12)*(gconst15)*(gconst20)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst33)*(gconst38)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((gconst12)*(gconst27)*(gconst35)*(gconst8)))+(((gconst28)*(gconst37)*(gconst8)*(gconst9)))+(((gconst19)*(gconst24)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst37)*(gconst7)))+(((gconst18)*(gconst26)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst19)*(gconst36)))+(((gconst15)*(gconst20)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((gconst12)*(gconst19)*(gconst22)*(gconst29)))+(((gconst21)*(gconst26)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst17)*(gconst21)*(gconst38)*(gconst6)))+(((gconst12)*(gconst17)*(gconst22)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst36)*(gconst4)))+(((gconst11)*(gconst27)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst20)*(gconst25)*(gconst33)*(gconst4)))+(((gconst17)*(gconst18)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst31)*(gconst37)*(gconst5)*(gconst9)))+(((gconst21)*(gconst26)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((gconst17)*(gconst24)*(gconst34)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst20)*(gconst34)))+(((gconst13)*(gconst16)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst14)*(gconst30)*(gconst33)*(gconst5)))+(((gconst30)*(gconst38)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst24)*(gconst34)*(gconst8)))+(((gconst22)*(gconst24)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst24)*(gconst28)))+(((gconst14)*(gconst17)*(gconst18)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst20)*(gconst36)))+(((gconst13)*(gconst28)*(gconst32)*(gconst9)))+(((gconst15)*(gconst23)*(gconst36)*(gconst8)))+(((gconst16)*(gconst20)*(gconst21)*(gconst25)))+(((gconst14)*(gconst18)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst37)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst37)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst34)*(gconst36)*(gconst4)*(gconst8)))+(((gconst12)*(gconst3)*(gconst32)*(gconst35)))+(((gconst14)*(gconst17)*(gconst21)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst35)*(gconst36)*(gconst5)*(gconst6)))+(((gconst10)*(gconst19)*(gconst24)*(gconst29)))+(((gconst3)*(gconst35)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst32)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst28)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst22)*(gconst32)))+(((gconst20)*(gconst21)*(gconst37)*(gconst4)))+(((gconst23)*(gconst24)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst10)*(gconst17)*(gconst24)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst3)*(gconst38)))+(((gconst13)*(gconst20)*(gconst21)*(gconst28)))+(((gconst14)*(gconst29)*(gconst30)*(gconst9)))+(((gconst32)*(gconst37)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst16)*(gconst25)*(gconst33)*(gconst8)))+(((gconst34)*(gconst36)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((gconst18)*(gconst21)*(gconst38)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst28)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((gconst20)*(gconst25)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst12)*(gconst17)*(gconst19)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst22)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst28)*(gconst34)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst32)*(gconst36)*(gconst4)))+(((gconst20)*(gconst24)*(gconst3)*(gconst35)))+(((gconst33)*(gconst37)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst17)*(gconst26)*(gconst33)*(gconst6)))+(((gconst18)*(gconst26)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst3)*(gconst32)*(gconst36)))+(((gconst21)*(gconst25)*(gconst28)*(gconst8)))+(((gconst17)*(gconst26)*(gconst30)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst32)*(gconst38)*(gconst9)))+(((gconst16)*(gconst20)*(gconst37)*(gconst9)))+(((gconst13)*(gconst28)*(gconst33)*(gconst8))));
op[5]=((((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst24)*(gconst34)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((gconst14)*(gconst17)*(gconst30)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst11)*(gconst15)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst37)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((gconst13)*(gconst21)*(gconst32)*(gconst4)))+(((gconst10)*(gconst12)*(gconst19)*(gconst29)))+(((gconst12)*(gconst23)*(gconst27)*(gconst8)))+(((gconst12)*(gconst15)*(gconst20)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst24)*(gconst4)))+(((gconst16)*(gconst37)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst22)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((gconst17)*(gconst38)*(gconst6)*(gconst9)))+(((gconst15)*(gconst23)*(gconst24)*(gconst8)))+(((gconst14)*(gconst21)*(gconst29)*(gconst6)))+(((gconst12)*(gconst22)*(gconst31)*(gconst5)))+(((gconst12)*(gconst15)*(gconst35)*(gconst8)))+(((gconst21)*(gconst37)*(gconst4)*(gconst8)))+(((gconst23)*(gconst3)*(gconst36)*(gconst8)))+(((gconst26)*(gconst29)*(gconst6)*(gconst9)))+(((gconst25)*(gconst32)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst16)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst34)*(gconst4)))+(((gconst18)*(gconst21)*(gconst26)*(gconst5)))+(((gconst11)*(gconst24)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst36)*(gconst5)))+(((gconst25)*(gconst28)*(gconst8)*(gconst9)))+(((gconst10)*(gconst24)*(gconst29)*(gconst7)))+(((gconst22)*(gconst36)*(gconst5)*(gconst7)))+(((gconst16)*(gconst21)*(gconst25)*(gconst8)))+(((gconst10)*(gconst12)*(gconst17)*(gconst31)))+(((gconst10)*(gconst19)*(gconst36)*(gconst5)))+(((gconst12)*(gconst17)*(gconst34)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((gconst11)*(gconst24)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst36)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst3)*(gconst38)*(gconst8)))+(((gconst10)*(gconst17)*(gconst19)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst3)*(gconst38)*(gconst9)))+(((gconst11)*(gconst20)*(gconst3)*(gconst36)))+(((gconst10)*(gconst17)*(gconst36)*(gconst7)))+(((gconst20)*(gconst23)*(gconst24)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst20)*(gconst28)))+(((gconst13)*(gconst21)*(gconst28)*(gconst8)))+(((gconst12)*(gconst22)*(gconst29)*(gconst7)))+(((gconst17)*(gconst21)*(gconst26)*(gconst6)))+(((gconst26)*(gconst33)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst22)*(gconst28)*(gconst8)))+(((gconst14)*(gconst17)*(gconst18)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((gconst16)*(gconst20)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst28)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst17)*(gconst33)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst21)*(gconst38)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst20)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst34)*(gconst8)))+(((gconst12)*(gconst20)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst20)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst20)*(gconst37)*(gconst4)*(gconst9)))+(((gconst13)*(gconst16)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst37)*(gconst5)*(gconst9)))+(((gconst24)*(gconst3)*(gconst35)*(gconst8)))+(((gconst10)*(gconst24)*(gconst31)*(gconst5)))+(((gconst12)*(gconst19)*(gconst34)*(gconst5)))+(((gconst17)*(gconst18)*(gconst26)*(gconst9)))+(((gconst25)*(gconst33)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((gconst13)*(gconst20)*(gconst33)*(gconst4)))+(((gconst26)*(gconst30)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst37)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((gconst12)*(gconst23)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst24)*(gconst8)))+(((gconst20)*(gconst21)*(gconst25)*(gconst4)))+(((gconst14)*(gconst21)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst38)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst36)*(gconst4)))+(((gconst13)*(gconst16)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst19)*(gconst22)*(gconst24)*(gconst5)))+(((gconst11)*(gconst15)*(gconst20)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst36)*(gconst4)*(gconst8)))+(((gconst13)*(gconst20)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((gconst11)*(gconst12)*(gconst15)*(gconst32)))+(((gconst13)*(gconst16)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst34)*(gconst4)*(gconst8)))+(((gconst18)*(gconst38)*(gconst5)*(gconst9)))+(((gconst14)*(gconst18)*(gconst33)*(gconst5)))+(((gconst11)*(gconst12)*(gconst20)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((gconst17)*(gconst22)*(gconst24)*(gconst7)))+(((gconst12)*(gconst17)*(gconst19)*(gconst22)))+(((gconst14)*(gconst18)*(gconst29)*(gconst9))));
op[6]=((((gconst13)*(gconst32)*(gconst4)*(gconst9)))+(((gconst14)*(gconst33)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst13)*(gconst16)*(gconst20)*(gconst9)))+(((gconst13)*(gconst33)*(gconst4)*(gconst8)))+(((gconst16)*(gconst25)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst16)*(gconst20)))+(((gconst14)*(gconst29)*(gconst6)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst37)*(gconst4)*(gconst8)*(gconst9)))+(((gconst38)*(gconst5)*(gconst6)*(gconst9)))+(((gconst10)*(gconst36)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst36)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((gconst11)*(gconst3)*(gconst36)*(gconst8)))+(((gconst13)*(gconst20)*(gconst21)*(gconst4)))+(((gconst20)*(gconst25)*(gconst4)*(gconst9)))+(((gconst12)*(gconst15)*(gconst23)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((gconst12)*(gconst34)*(gconst5)*(gconst7)))+(((gconst11)*(gconst12)*(gconst27)*(gconst8)))+(((gconst17)*(gconst26)*(gconst6)*(gconst9)))+(((gconst11)*(gconst12)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((gconst12)*(gconst3)*(gconst35)*(gconst8)))+(((gconst10)*(gconst17)*(gconst24)*(gconst7)))+(((gconst18)*(gconst26)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst22)*(gconst8)))+(((gconst23)*(gconst24)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst13)*(gconst16)*(gconst21)*(gconst8)))+(((gconst12)*(gconst20)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst4)*(gconst8)))+(((gconst10)*(gconst12)*(gconst17)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst34)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst12)*(gconst15)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst28)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst38)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst17)*(gconst22)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((gconst12)*(gconst19)*(gconst22)*(gconst5)))+(((gconst10)*(gconst12)*(gconst29)*(gconst7)))+(((gconst14)*(gconst18)*(gconst21)*(gconst5)))+(((gconst22)*(gconst24)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((gconst21)*(gconst25)*(gconst4)*(gconst8)))+(((gconst21)*(gconst26)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((gconst14)*(gconst30)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst14)*(gconst17)*(gconst21)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst36)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst37)*(gconst5)*(gconst7)*(gconst9)))+(((gconst10)*(gconst19)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((gconst13)*(gconst28)*(gconst8)*(gconst9)))+(((gconst14)*(gconst17)*(gconst18)*(gconst9)))+(((gconst11)*(gconst15)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst24)*(gconst4)))+(((gconst10)*(gconst12)*(gconst31)*(gconst5)))+(((gconst11)*(gconst20)*(gconst24)*(gconst3))));
op[7]=((((IkReal(-1.00000000000000))*(x101)*(x125)))+(((IkReal(-1.00000000000000))*(x104)*(x123)))+(((gconst3)*(gconst8)*(x119)))+(((x96)*(x97)))+(((IkReal(-1.00000000000000))*(gconst4)*(x113)*(x99)))+(((x100)*(x99)))+(((IkReal(-1.00000000000000))*(x125)*(x97)))+(((IkReal(-1.00000000000000))*(gconst3)*(x113)*(x96)))+(((IkReal(-1.00000000000000))*(x102)*(x118)))+(((gconst20)*(gconst4)*(x104)))+(((gconst12)*(gconst15)*(x122)))+(((x101)*(x96)))+(((gconst23)*(gconst8)*(x103)))+(((gconst11)*(gconst20)*(x103)))+(((IkReal(-1.00000000000000))*(gconst4)*(x107)*(x95)))+(((IkReal(-1.00000000000000))*(x102)*(x109)))+(((x102)*(x121)))+(((IkReal(-1.00000000000000))*(gconst16)*(x95)*(x99)))+(((gconst16)*(gconst8)*(x104)))+(((IkReal(-1.00000000000000))*(gconst3)*(x106)*(x95)))+(((IkReal(-1.00000000000000))*(gconst15)*(x95)*(x96)))+(((x102)*(x107)))+(((x106)*(x94)))+(((x108)*(x109)))+(((x108)*(x118)))+(((x93)*(x94)))+(((IkReal(-1.00000000000000))*(x100)*(x104)))+(((IkReal(-1.00000000000000))*(x119)*(x94)))+(((IkReal(-1.00000000000000))*(gconst3)*(x93)*(x95)))+(((IkReal(-1.00000000000000))*(gconst4)*(x121)*(x95)))+(((x123)*(x99)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(x94))));
op[8]=((((IkReal(-1.00000000000000))*(x125)*(x94)))+(((IkReal(-1.00000000000000))*(gconst4)*(x95)*(x99)))+(((x102)*(x99)))+(((IkReal(-1.00000000000000))*(x102)*(x104)))+(((x103)*(x122)))+(((x104)*(x108)))+(((IkReal(-1.00000000000000))*(gconst3)*(x95)*(x96)))+(((x94)*(x96))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x128=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(pz));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj0)*(x128)))+(((px)*(sj0))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj0)*(x128)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
IkReal x129=(px)*(px);
IkReal x130=(sj0)*(sj0);
IkReal x131=(cj0)*(cj0);
IkReal x132=(py)*(py);
IkReal x133=((IkReal(0.0900000000000000))*(x130));
IkReal x134=((IkReal(0.0900000000000000))*(x131));
gconst56=IKsign(((((x129)*(x133)))+(((x129)*(x134)))+(((x132)*(x133)))+(((x132)*(x134)))));
IkReal x135=(sj0)*(sj0);
IkReal x136=(py)*(py);
IkReal x137=(cj0)*(cj0);
IkReal x138=(px)*(px);
dummyeval[0]=((((x135)*(x136)))+(((x135)*(x138)))+(((x137)*(x138)))+(((x136)*(x137))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x139=((pp)*(py));
IkReal x140=((IkReal(0.212500000000000))*(cj0));
IkReal x141=((py)*(sj0));
IkReal x142=((IkReal(1.10000000000000))*(pz));
IkReal x143=((px)*(sj0));
IkReal x144=((px)*(x142));
if( IKabs(((gconst56)*(((((x142)*(x143)))+(((IkReal(-0.212500000000000))*(x143)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x142)))+(((cj0)*(x139)))+(((IkReal(-1.00000000000000))*(pp)*(x143)))+(((py)*(x140))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(cj0)*(x144)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))+(((cj0)*(pp)*(px)))+(((px)*(x140)))+(((IkReal(0.212500000000000))*(x141)))+(((sj0)*(x139))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst56)*(((((x142)*(x143)))+(((IkReal(-0.212500000000000))*(x143)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x142)))+(((cj0)*(x139)))+(((IkReal(-1.00000000000000))*(pp)*(x143)))+(((py)*(x140)))))), ((gconst56)*(((((IkReal(-1.00000000000000))*(cj0)*(x144)))+(((IkReal(-1.00000000000000))*(x141)*(x142)))+(((cj0)*(pp)*(px)))+(((px)*(x140)))+(((IkReal(0.212500000000000))*(x141)))+(((sj0)*(x139)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x145=IKsin(j2);
IkReal x146=IKcos(j2);
IkReal x147=((IkReal(1.00000000000000))*(py));
IkReal x148=((IkReal(0.0900000000000000))*(py));
IkReal x149=((sj0)*(x146));
IkReal x150=((sj0)*(x145));
IkReal x151=((cj0)*(x145));
IkReal x152=((cj0)*(px)*(x146));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x146)*(x147)))+(((px)*(x151)))+(((px)*(x149)))+(((py)*(x150))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x147)*(x149)))+(((IkReal(-1.00000000000000))*(x152)))+(((IkReal(-1.00000000000000))*(x147)*(x151)))+(((px)*(x150))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-0.0900000000000000))*(px)*(x150)))+(((IkReal(0.0900000000000000))*(x152)))+(((IkReal(-1.00000000000000))*(pp)))+(((x148)*(x149)))+(((IkReal(1.10000000000000))*(pz)))+(((x148)*(x151))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x224=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(pz));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj0)*(x224)))+(((px)*(sj0))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj0)*(x224)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
IkReal x225=(px)*(px);
IkReal x226=(sj0)*(sj0);
IkReal x227=(cj0)*(cj0);
IkReal x228=(py)*(py);
IkReal x229=((IkReal(0.0900000000000000))*(x226));
IkReal x230=((IkReal(0.0900000000000000))*(x227));
gconst57=IKsign(((((x225)*(x229)))+(((x228)*(x230)))+(((x228)*(x229)))+(((x225)*(x230)))));
IkReal x231=(sj0)*(sj0);
IkReal x232=(py)*(py);
IkReal x233=(cj0)*(cj0);
IkReal x234=(px)*(px);
dummyeval[0]=((((x232)*(x233)))+(((x233)*(x234)))+(((x231)*(x234)))+(((x231)*(x232))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x235=((pp)*(py));
IkReal x236=((IkReal(0.212500000000000))*(cj0));
IkReal x237=((py)*(sj0));
IkReal x238=((IkReal(1.10000000000000))*(pz));
IkReal x239=((px)*(sj0));
IkReal x240=((px)*(x238));
if( IKabs(((gconst57)*(((((IkReal(-0.212500000000000))*(x239)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x238)))+(((py)*(x236)))+(((x238)*(x239)))+(((IkReal(-1.00000000000000))*(pp)*(x239)))+(((cj0)*(x235))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((sj0)*(x235)))+(((px)*(x236)))+(((IkReal(-1.00000000000000))*(cj0)*(x240)))+(((cj0)*(pp)*(px)))+(((IkReal(0.212500000000000))*(x237)))+(((IkReal(-1.00000000000000))*(x237)*(x238))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst57)*(((((IkReal(-0.212500000000000))*(x239)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x238)))+(((py)*(x236)))+(((x238)*(x239)))+(((IkReal(-1.00000000000000))*(pp)*(x239)))+(((cj0)*(x235)))))), ((gconst57)*(((((sj0)*(x235)))+(((px)*(x236)))+(((IkReal(-1.00000000000000))*(cj0)*(x240)))+(((cj0)*(pp)*(px)))+(((IkReal(0.212500000000000))*(x237)))+(((IkReal(-1.00000000000000))*(x237)*(x238)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x241=IKsin(j2);
IkReal x242=IKcos(j2);
IkReal x243=((IkReal(1.00000000000000))*(py));
IkReal x244=((IkReal(0.0900000000000000))*(py));
IkReal x245=((sj0)*(x242));
IkReal x246=((sj0)*(x241));
IkReal x247=((cj0)*(x241));
IkReal x248=((cj0)*(px)*(x242));
evalcond[0]=((((px)*(x245)))+(((px)*(x247)))+(((IkReal(-1.00000000000000))*(cj0)*(x242)*(x243)))+(((py)*(x246))));
evalcond[1]=((((px)*(x246)))+(((IkReal(-1.00000000000000))*(x248)))+(((IkReal(-1.00000000000000))*(x243)*(x247)))+(((IkReal(-1.00000000000000))*(x243)*(x245))));
evalcond[2]=((IkReal(-0.212500000000000))+(((x244)*(x245)))+(((x244)*(x247)))+(((IkReal(-0.0900000000000000))*(px)*(x246)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(0.0900000000000000))*(x248))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x249=((IkReal(1111111111.11111))*(px));
IkReal x250=((IkReal(1111111111.11111))*(py));
if( IKabs(((((cj0)*(x250)))+(((IkReal(-1.00000000000000))*(sj0)*(x249))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj0)*(x249)))+(((sj0)*(x250))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj0)*(x250)))+(((IkReal(-1.00000000000000))*(sj0)*(x249)))))+IKsqr(((((cj0)*(x249)))+(((sj0)*(x250)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((cj0)*(x250)))+(((IkReal(-1.00000000000000))*(sj0)*(x249)))), ((((cj0)*(x249)))+(((sj0)*(x250)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x251=IKsin(j2);
IkReal x252=IKcos(j2);
IkReal x253=((IkReal(0.0900000000000000))*(py));
IkReal x254=((IkReal(1.00000000000000))*(py));
IkReal x255=((px)*(sj0));
IkReal x256=((cj0)*(px));
IkReal x257=((sj0)*(x252));
IkReal x258=((cj0)*(x251));
IkReal x259=((x252)*(x256));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x254)))+(x255)+(((IkReal(9.00000000000000e-10))*(x251))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj0)*(x254)))+(((IkReal(-1.00000000000000))*(x256)))+(((IkReal(9.00000000000000e-10))*(x252))));
evalcond[2]=((((x251)*(x256)))+(((IkReal(-1.00000000000000))*(cj0)*(x252)*(x254)))+(((x252)*(x255)))+(((py)*(sj0)*(x251))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((x251)*(x255)))+(((IkReal(-1.00000000000000))*(x254)*(x258)))+(((IkReal(-1.00000000000000))*(x254)*(x257)))+(((IkReal(-1.00000000000000))*(x259))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-0.0900000000000000))*(x251)*(x255)))+(((IkReal(-1.00000000000000))*(pp)))+(((x253)*(x257)))+(((x253)*(x258)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(0.0900000000000000))*(x259))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((cj0)*(px)))+(((py)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst55)*(((((cj0)*(px)))+(((py)*(sj0)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x260=IKsin(j2);
IkReal x261=IKcos(j2);
IkReal x262=((IkReal(0.0450000000000000))*(cj3));
IkReal x263=((px)*(sj0));
IkReal x264=((IkReal(1.00000000000000))*(py));
IkReal x265=((IkReal(0.300000000000000))*(sj3));
IkReal x266=((IkReal(0.0900000000000000))*(py));
IkReal x267=((sj0)*(x261));
IkReal x268=((cj0)*(x260));
IkReal x269=((cj0)*(px)*(x261));
evalcond[0]=((((x260)*(x265)))+(x263)+(((IkReal(-1.00000000000000))*(cj0)*(x264)))+(((IkReal(0.0450000000000000))*(x260)))+(((IkReal(-1.00000000000000))*(x260)*(x262))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x261)*(x262)))+(((IkReal(-1.00000000000000))*(sj0)*(x264)))+(((x261)*(x265)))+(((IkReal(0.0450000000000000))*(x261)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj0)*(x261)*(x264)))+(((py)*(sj0)*(x260)))+(((x261)*(x263)))+(((px)*(x268))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x269)))+(((x260)*(x263)))+(x265)+(((IkReal(-1.00000000000000))*(x262)))+(((IkReal(-1.00000000000000))*(x264)*(x267)))+(((IkReal(-1.00000000000000))*(x264)*(x268))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x266)*(x268)))+(((x266)*(x267)))+(((IkReal(-0.0900000000000000))*(x260)*(x263)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(0.0900000000000000))*(x269))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj3))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=((IkReal(-1.00000000000000))*(py));
IkReal gconst59;
gconst59=((IkReal(-2.00000000000000))*(px));
IkReal gconst60;
gconst60=py;
IkReal gconst61;
gconst61=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst62;
gconst62=((IkReal(-2.00000000000000))*(py));
IkReal gconst63;
gconst63=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst64;
gconst64=((IkReal(-1.00000000000000))*(py));
IkReal gconst65;
gconst65=((IkReal(-2.00000000000000))*(px));
IkReal gconst66;
gconst66=py;
IkReal gconst67;
gconst67=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst68;
gconst68=((IkReal(-2.00000000000000))*(py));
IkReal gconst69;
gconst69=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst70;
gconst70=((IkReal(2.00000000000000))*(px));
IkReal gconst71;
gconst71=((IkReal(-4.00000000000000))*(py));
IkReal gconst72;
gconst72=((IkReal(-2.00000000000000))*(px));
IkReal gconst73;
gconst73=((IkReal(2.00000000000000))*(py));
IkReal gconst74;
gconst74=((IkReal(4.00000000000000))*(px));
IkReal gconst75;
gconst75=((IkReal(-2.00000000000000))*(py));
IkReal gconst76;
gconst76=((IkReal(2.00000000000000))*(px));
IkReal gconst77;
gconst77=((IkReal(-4.00000000000000))*(py));
IkReal gconst78;
gconst78=((IkReal(-2.00000000000000))*(px));
IkReal gconst79;
gconst79=((IkReal(2.00000000000000))*(py));
IkReal gconst80;
gconst80=((IkReal(4.00000000000000))*(px));
IkReal gconst81;
gconst81=((IkReal(-2.00000000000000))*(py));
IkReal gconst82;
gconst82=py;
IkReal gconst83;
gconst83=((IkReal(2.00000000000000))*(px));
IkReal gconst84;
gconst84=((IkReal(-1.00000000000000))*(py));
IkReal gconst85;
gconst85=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst86;
gconst86=((IkReal(2.00000000000000))*(py));
IkReal gconst87;
gconst87=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst88;
gconst88=py;
IkReal gconst89;
gconst89=((IkReal(2.00000000000000))*(px));
IkReal gconst90;
gconst90=((IkReal(-1.00000000000000))*(py));
IkReal gconst91;
gconst91=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px)))+(((IkReal(0.300000000000000))*(sj3))));
IkReal gconst92;
gconst92=((IkReal(2.00000000000000))*(py));
IkReal gconst93;
gconst93=((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(px)+(((IkReal(0.300000000000000))*(sj3))));
IkReal x270=((gconst83)*(gconst87));
IkReal x271=((gconst89)*(gconst91));
IkReal x272=((gconst88)*(gconst92));
IkReal x273=((gconst84)*(gconst86));
IkReal x274=((gconst88)*(gconst93));
IkReal x275=((gconst84)*(gconst85));
IkReal x276=((gconst82)*(gconst87));
IkReal x277=((gconst90)*(gconst91));
dummyeval[0]=((((x271)*(x273)))+(((IkReal(-1.00000000000000))*(x274)*(x276)))+(((x274)*(x275)))+(((x270)*(x272)))+(((IkReal(-1.00000000000000))*(x275)*(x277)))+(((x276)*(x277)))+(((IkReal(-1.00000000000000))*(x270)*(x271)))+(((IkReal(-1.00000000000000))*(x272)*(x273))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x278=((gconst61)*(gconst69));
IkReal x279=((gconst64)*(gconst72));
IkReal x280=((IkReal(1.00000000000000))*(gconst79));
IkReal x281=((gconst84)*(gconst85));
IkReal x282=((gconst65)*(gconst67));
IkReal x283=((gconst59)*(gconst63));
IkReal x284=((gconst64)*(gconst68));
IkReal x285=((IkReal(1.00000000000000))*(gconst88));
IkReal x286=((gconst82)*(gconst93));
IkReal x287=((gconst83)*(gconst87));
IkReal x288=((gconst72)*(gconst86));
IkReal x289=((gconst60)*(gconst76));
IkReal x290=((gconst58)*(gconst75));
IkReal x291=((gconst82)*(gconst87));
IkReal x292=((gconst90)*(gconst91));
IkReal x293=((IkReal(1.00000000000000))*(gconst87));
IkReal x294=((gconst60)*(gconst64));
IkReal x295=((gconst66)*(gconst67));
IkReal x296=((gconst58)*(gconst63));
IkReal x297=((gconst59)*(gconst75));
IkReal x298=((gconst67)*(gconst78));
IkReal x299=((gconst67)*(gconst77));
IkReal x300=((gconst60)*(gconst62));
IkReal x301=((gconst77)*(gconst91));
IkReal x302=((gconst88)*(gconst92));
IkReal x303=((gconst84)*(gconst86));
IkReal x304=((gconst88)*(gconst93));
IkReal x305=((gconst63)*(gconst70));
IkReal x306=((gconst89)*(gconst91));
IkReal x307=((gconst78)*(gconst91));
IkReal x308=((IkReal(1.00000000000000))*(gconst62));
IkReal x309=((gconst70)*(gconst87));
IkReal x310=((gconst60)*(gconst61));
IkReal x311=((IkReal(1.00000000000000))*(gconst60));
IkReal x312=((gconst74)*(gconst84));
IkReal x313=((gconst72)*(gconst85));
IkReal x314=((gconst73)*(gconst84));
IkReal x315=((gconst76)*(gconst92));
IkReal x316=((gconst63)*(gconst71));
IkReal x317=((gconst89)*(x303));
IkReal x318=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x319=((IkReal(1.00000000000000))*(gconst83)*(x306));
op[0]=((((IkReal(-1.00000000000000))*(x281)*(x292)))+(((x291)*(x292)))+(((x287)*(x302)))+(((x303)*(x306)))+(((x281)*(x304)))+(((IkReal(-1.00000000000000))*(x287)*(x306)))+(((IkReal(-1.00000000000000))*(gconst87)*(x285)*(x286)))+(((IkReal(-1.00000000000000))*(gconst92)*(x285)*(x303))));
op[1]=((((x301)*(x303)))+(((IkReal(-1.00000000000000))*(gconst92)*(x285)*(x312)))+(((IkReal(-1.00000000000000))*(x292)*(x313)))+(((IkReal(-1.00000000000000))*(x292)*(x314)))+(((x304)*(x313)))+(((x304)*(x314)))+(((x291)*(x307)))+(((gconst81)*(gconst88)*(x281)))+(((IkReal(-1.00000000000000))*(gconst75)*(x285)*(x286)))+(((IkReal(-1.00000000000000))*(gconst90)*(x280)*(x281)))+(((gconst79)*(x317)))+(((IkReal(-1.00000000000000))*(gconst92)*(x285)*(x288)))+(((IkReal(-1.00000000000000))*(gconst71)*(x293)*(x306)))+(((x292)*(x309)))+(((IkReal(-1.00000000000000))*(gconst76)*(x286)*(x293)))+(((IkReal(-1.00000000000000))*(x281)*(x307)))+(((gconst75)*(gconst82)*(x292)))+(((IkReal(-1.00000000000000))*(gconst75)*(x319)))+(((IkReal(-1.00000000000000))*(x303)*(x315)))+(((gconst79)*(gconst90)*(x291)))+(((IkReal(-1.00000000000000))*(gconst81)*(x285)*(x291)))+(((IkReal(-1.00000000000000))*(x287)*(x301)))+(((x306)*(x312)))+(((IkReal(-1.00000000000000))*(gconst80)*(x285)*(x303)))+(((x288)*(x306)))+(((gconst71)*(gconst87)*(x302)))+(((x287)*(x315)))+(((IkReal(-1.00000000000000))*(gconst89)*(x280)*(x287)))+(((IkReal(-1.00000000000000))*(gconst93)*(x285)*(x309)))+(((gconst75)*(gconst83)*(x302)))+(((gconst80)*(gconst88)*(x287)))+(((gconst76)*(gconst93)*(x281))));
op[2]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst78)*(gconst84)*(gconst91)))+(((gconst71)*(gconst80)*(gconst87)*(gconst88)))+(((gconst69)*(gconst84)*(gconst85)*(gconst88)))+(((gconst78)*(gconst79)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst83)*(gconst89)*(gconst91)))+(((gconst76)*(gconst80)*(gconst83)*(gconst87)))+(((gconst72)*(gconst76)*(gconst85)*(gconst93)))+(((gconst60)*(gconst86)*(gconst89)*(gconst91)))+(((gconst64)*(gconst83)*(gconst87)*(gconst92)))+(((gconst64)*(gconst84)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst86)*(gconst88)*(gconst92)))+(((gconst68)*(gconst83)*(gconst87)*(gconst88)))+(((gconst71)*(gconst75)*(gconst88)*(gconst92)))+(((gconst70)*(gconst79)*(gconst87)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst84)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst83)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst84)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst82)*(gconst87)*(gconst88)))+(((gconst72)*(gconst73)*(gconst88)*(gconst93)))+(((gconst73)*(gconst81)*(gconst84)*(gconst88)))+(((gconst75)*(gconst76)*(gconst83)*(gconst92)))+(((gconst74)*(gconst79)*(gconst84)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst81)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst79)*(gconst84)*(gconst90)))+(((gconst70)*(gconst75)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst84)*(gconst85)*(gconst91)))+(((gconst77)*(gconst79)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst76)*(gconst82)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst77)*(gconst79)*(gconst83)*(gconst87)))+(((gconst74)*(gconst77)*(gconst84)*(gconst91)))+(((gconst76)*(gconst81)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst77)*(gconst83)*(gconst91)))+(((gconst75)*(gconst80)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst87)*(gconst89)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst79)*(gconst85)*(gconst90)))+(((gconst67)*(gconst84)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst83)*(gconst89)))+(((gconst72)*(gconst81)*(gconst85)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst80)*(gconst86)*(gconst88)))+(((gconst66)*(gconst82)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst78)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst76)*(gconst84)*(gconst92)))+(((gconst75)*(gconst78)*(gconst82)*(gconst91)))+(((gconst72)*(gconst79)*(gconst86)*(gconst89)))+(((gconst59)*(gconst87)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst83)*(gconst87)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst81)*(gconst87)*(gconst88)))+(((gconst65)*(gconst84)*(gconst86)*(gconst91)))+(((gconst63)*(gconst83)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst78)*(gconst79)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst89)*(gconst91)))+(((gconst72)*(gconst77)*(gconst86)*(gconst91)))+(((gconst67)*(gconst82)*(gconst87)*(gconst90)))+(((gconst72)*(gconst74)*(gconst89)*(gconst91)))+(((gconst71)*(gconst76)*(gconst87)*(gconst92)))+(((gconst62)*(gconst84)*(gconst89)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst85)*(gconst90)*(gconst91)))+(((gconst63)*(gconst82)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst82)*(gconst88)*(gconst93)))+(((gconst58)*(gconst87)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst84)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst80)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst87)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst87)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst80)*(gconst84)*(gconst86)))+(((gconst75)*(gconst79)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst81)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst79)*(gconst87)*(gconst89)))+(((gconst73)*(gconst76)*(gconst84)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst84)*(gconst86)*(gconst88)))+(((gconst61)*(gconst84)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst82)*(gconst87)*(gconst93)))+(((gconst70)*(gconst78)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst84)*(gconst85)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst88)*(gconst92)))+(((gconst60)*(gconst85)*(gconst88)*(gconst93))));
op[3]=((((gconst65)*(gconst79)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst79)*(gconst85)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst78)*(gconst85)*(gconst91)))+(((gconst67)*(gconst75)*(gconst82)*(gconst90)))+(((gconst67)*(gconst70)*(gconst87)*(gconst90)))+(((gconst70)*(gconst78)*(gconst79)*(gconst87)))+(((gconst67)*(gconst74)*(gconst84)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst77)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst87)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst81)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst78)*(gconst84)*(gconst91)))+(((gconst68)*(gconst71)*(gconst87)*(gconst88)))+(((gconst58)*(gconst79)*(gconst87)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst76)*(gconst82)*(gconst87)))+(((gconst72)*(gconst77)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst78)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst84)*(gconst85)))+(((gconst71)*(gconst75)*(gconst76)*(gconst92)))+(((gconst63)*(gconst70)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst84)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst82)*(gconst88)))+(((gconst60)*(gconst73)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst88)*(gconst92)))+(((gconst67)*(gconst77)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst79)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst79)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst90)*(gconst91)))+(((gconst61)*(gconst81)*(gconst84)*(gconst88)))+(((gconst65)*(gconst74)*(gconst84)*(gconst91)))+(((gconst61)*(gconst72)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst79)*(gconst87)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst77)*(gconst79)*(gconst83)))+(((gconst72)*(gconst73)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst76)*(gconst93)))+(((gconst62)*(gconst77)*(gconst84)*(gconst91)))+(((gconst72)*(gconst74)*(gconst79)*(gconst89)))+(((gconst60)*(gconst77)*(gconst86)*(gconst91)))+(((gconst64)*(gconst75)*(gconst83)*(gconst92)))+(((gconst72)*(gconst73)*(gconst76)*(gconst93)))+(((gconst64)*(gconst72)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst74)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst78)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst80)*(gconst84)*(gconst88)))+(((gconst71)*(gconst76)*(gconst80)*(gconst87)))+(((gconst67)*(gconst78)*(gconst82)*(gconst87)))+(((gconst66)*(gconst75)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst80)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst81)*(gconst82)*(gconst87)))+(((gconst71)*(gconst75)*(gconst80)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst75)*(gconst82)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst87)*(gconst91)))+(((gconst63)*(gconst79)*(gconst82)*(gconst90)))+(((gconst60)*(gconst74)*(gconst89)*(gconst91)))+(((gconst67)*(gconst72)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst88)*(gconst93)))+(((gconst59)*(gconst75)*(gconst88)*(gconst92)))+(((gconst62)*(gconst79)*(gconst84)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst76)*(gconst92)))+(((gconst75)*(gconst78)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst75)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst79)*(gconst90)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst89)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst77)*(gconst87)*(gconst91)))+(((gconst59)*(gconst76)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst80)*(gconst88)))+(((gconst60)*(gconst81)*(gconst85)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst77)*(gconst91)))+(((gconst62)*(gconst72)*(gconst89)*(gconst91)))+(((gconst69)*(gconst73)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst89)*(gconst91)))+(((gconst65)*(gconst72)*(gconst86)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst76)*(gconst82)*(gconst93)))+(((gconst64)*(gconst71)*(gconst87)*(gconst92)))+(((gconst70)*(gconst75)*(gconst78)*(gconst91)))+(((gconst58)*(gconst75)*(gconst90)*(gconst91)))+(((gconst64)*(gconst73)*(gconst84)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst85)*(gconst91)))+(((gconst63)*(gconst80)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst83)*(gconst91)))+(((gconst75)*(gconst76)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst76)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst78)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst86)*(gconst92)))+(((gconst66)*(gconst79)*(gconst82)*(gconst87)))+(((gconst63)*(gconst78)*(gconst82)*(gconst91)))+(((gconst66)*(gconst70)*(gconst87)*(gconst91)))+(((gconst63)*(gconst71)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst80)*(gconst86)))+(((gconst59)*(gconst80)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst83)*(gconst89)))+(((gconst69)*(gconst72)*(gconst85)*(gconst88)))+(((gconst68)*(gconst75)*(gconst83)*(gconst88)))+(((gconst60)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst84)*(gconst90)))+(((gconst70)*(gconst75)*(gconst79)*(gconst90)))+(((gconst68)*(gconst76)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst80)*(gconst86)*(gconst88)))+(((gconst72)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst78)*(gconst79)*(gconst85)))+(((gconst74)*(gconst77)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst76)*(gconst87)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst87)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst76)*(gconst81)*(gconst82)))+(((gconst63)*(gconst76)*(gconst83)*(gconst92)))+(((gconst73)*(gconst76)*(gconst81)*(gconst84)))+(((gconst64)*(gconst81)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst75)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst84)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst84)*(gconst92)))+(((gconst72)*(gconst76)*(gconst81)*(gconst85)))+(((gconst61)*(gconst76)*(gconst84)*(gconst93)))+(((gconst60)*(gconst76)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst76)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst81)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst85)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst81)*(gconst87)))+(((gconst58)*(gconst78)*(gconst87)*(gconst91))));
op[4]=((((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst78)*(gconst79)))+(((gconst60)*(gconst74)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst76)*(gconst81)))+(((gconst58)*(gconst78)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst79)*(gconst87)))+(((gconst60)*(gconst65)*(gconst86)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst76)*(gconst93)))+(((gconst61)*(gconst64)*(gconst84)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst91)))+(((gconst63)*(gconst71)*(gconst80)*(gconst88)))+(((gconst68)*(gconst75)*(gconst76)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst86)*(gconst88)))+(((gconst65)*(gconst67)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst84)*(gconst85)))+(((gconst64)*(gconst68)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst88)*(gconst93)))+(((gconst60)*(gconst76)*(gconst81)*(gconst85)))+(((gconst65)*(gconst72)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst82)*(gconst87)))+(((gconst61)*(gconst72)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst76)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst87)*(gconst89)))+(((gconst58)*(gconst75)*(gconst79)*(gconst90)))+(((gconst58)*(gconst67)*(gconst87)*(gconst90)))+(((gconst69)*(gconst73)*(gconst76)*(gconst84)))+(((gconst61)*(gconst72)*(gconst76)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst78)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst78)*(gconst79)*(gconst84)))+(((gconst69)*(gconst72)*(gconst73)*(gconst88)))+(((gconst63)*(gconst70)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst80)*(gconst84)))+(((gconst58)*(gconst75)*(gconst78)*(gconst91)))+(((gconst62)*(gconst67)*(gconst84)*(gconst89)))+(((gconst67)*(gconst70)*(gconst75)*(gconst90)))+(((gconst63)*(gconst67)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst80)*(gconst88)))+(((gconst63)*(gconst68)*(gconst83)*(gconst88)))+(((gconst69)*(gconst72)*(gconst76)*(gconst85)))+(((gconst66)*(gconst70)*(gconst79)*(gconst87)))+(((gconst68)*(gconst71)*(gconst75)*(gconst88)))+(((gconst63)*(gconst70)*(gconst78)*(gconst91)))+(((gconst67)*(gconst70)*(gconst78)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst78)*(gconst91)))+(((gconst66)*(gconst70)*(gconst75)*(gconst91)))+(((gconst66)*(gconst67)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst85)*(gconst90)))+(((gconst64)*(gconst72)*(gconst73)*(gconst93)))+(((gconst68)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst78)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst75)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst81)*(gconst88)))+(((gconst63)*(gconst78)*(gconst79)*(gconst82)))+(((gconst59)*(gconst63)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst75)*(gconst91)))+(((gconst64)*(gconst71)*(gconst75)*(gconst92)))+(((gconst67)*(gconst72)*(gconst77)*(gconst86)))+(((gconst67)*(gconst74)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst78)*(gconst85)))+(((gconst61)*(gconst69)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst76)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst87)*(gconst91)))+(((gconst71)*(gconst75)*(gconst76)*(gconst80)))+(((gconst60)*(gconst67)*(gconst86)*(gconst89)))+(((gconst62)*(gconst72)*(gconst79)*(gconst89)))+(((gconst63)*(gconst71)*(gconst76)*(gconst92)))+(((gconst60)*(gconst73)*(gconst76)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst76)*(gconst87)))+(((gconst59)*(gconst75)*(gconst76)*(gconst92)))+(((gconst72)*(gconst74)*(gconst77)*(gconst79)))+(((gconst70)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst87)*(gconst93)))+(((gconst63)*(gconst66)*(gconst82)*(gconst91)))+(((gconst66)*(gconst75)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst84)*(gconst88)))+(((gconst60)*(gconst77)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst74)*(gconst92)))+(((gconst60)*(gconst64)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst90)*(gconst91)))+(((gconst59)*(gconst64)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst76)*(gconst92)))+(((gconst72)*(gconst73)*(gconst76)*(gconst81)))+(((gconst64)*(gconst71)*(gconst80)*(gconst87)))+(((gconst64)*(gconst72)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst84)*(gconst91)))+(((gconst67)*(gconst72)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst77)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst80)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst76)*(gconst92)))+(((gconst64)*(gconst73)*(gconst81)*(gconst84)))+(((gconst62)*(gconst77)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst76)*(gconst82)))+(((gconst58)*(gconst66)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst77)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst75)*(gconst81)*(gconst82)))+(((gconst61)*(gconst76)*(gconst81)*(gconst84)))+(((gconst64)*(gconst75)*(gconst80)*(gconst83)))+(((gconst63)*(gconst64)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst75)*(gconst89)))+(((gconst62)*(gconst72)*(gconst77)*(gconst91)))+(((gconst59)*(gconst76)*(gconst80)*(gconst87)))+(((gconst59)*(gconst68)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst89)*(gconst91)))+(((gconst67)*(gconst75)*(gconst78)*(gconst82)))+(((gconst59)*(gconst75)*(gconst80)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst76)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst84)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst82)*(gconst93)))+(((gconst62)*(gconst65)*(gconst84)*(gconst91)))+(((gconst60)*(gconst69)*(gconst85)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst76)*(gconst80)*(gconst86)))+(((gconst65)*(gconst74)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst79)*(gconst89)))+(((gconst60)*(gconst62)*(gconst89)*(gconst91)))+(((gconst60)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst74)*(gconst76)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst81)*(gconst88)))+(((gconst63)*(gconst76)*(gconst80)*(gconst83)))+(((gconst60)*(gconst73)*(gconst81)*(gconst88)))+(((gconst58)*(gconst63)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst75)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst77)*(gconst91)))+(((gconst64)*(gconst69)*(gconst84)*(gconst85)))+(((gconst60)*(gconst61)*(gconst88)*(gconst93)))+(((gconst65)*(gconst72)*(gconst74)*(gconst91))));
op[5]=((((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst74)*(gconst76)))+(((gconst61)*(gconst64)*(gconst72)*(gconst93)))+(((gconst64)*(gconst68)*(gconst71)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst74)*(gconst80)))+(((gconst63)*(gconst66)*(gconst79)*(gconst82)))+(((gconst60)*(gconst65)*(gconst79)*(gconst86)))+(((gconst60)*(gconst67)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst80)*(gconst88)))+(((gconst63)*(gconst67)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst75)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst70)*(gconst88)))+(((gconst69)*(gconst72)*(gconst73)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst75)*(gconst88)))+(((gconst63)*(gconst70)*(gconst78)*(gconst79)))+(((gconst58)*(gconst63)*(gconst78)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst84)))+(((gconst59)*(gconst64)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst71)*(gconst87)))+(((gconst58)*(gconst66)*(gconst75)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst72)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(gconst90)))+(((gconst64)*(gconst68)*(gconst75)*(gconst83)))+(((gconst58)*(gconst67)*(gconst78)*(gconst87)))+(((gconst68)*(gconst71)*(gconst75)*(gconst76)))+(((gconst62)*(gconst65)*(gconst72)*(gconst91)))+(((gconst59)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst77)*(gconst79)))+(((gconst60)*(gconst67)*(gconst77)*(gconst86)))+(((gconst63)*(gconst64)*(gconst80)*(gconst83)))+(((gconst58)*(gconst63)*(gconst79)*(gconst90)))+(((gconst61)*(gconst69)*(gconst76)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst75)*(gconst79)))+(((gconst59)*(gconst75)*(gconst76)*(gconst80)))+(((gconst66)*(gconst70)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst71)*(gconst89)))+(((gconst60)*(gconst69)*(gconst73)*(gconst88)))+(((gconst63)*(gconst68)*(gconst76)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst77)*(gconst83)))+(((gconst62)*(gconst72)*(gconst77)*(gconst79)))+(((gconst66)*(gconst67)*(gconst75)*(gconst82)))+(((gconst62)*(gconst67)*(gconst72)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst76)*(gconst80)))+(((gconst60)*(gconst62)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst76)*(gconst93)))+(((gconst60)*(gconst74)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst78)*(gconst91)))+(((gconst65)*(gconst67)*(gconst74)*(gconst84)))+(((gconst60)*(gconst61)*(gconst76)*(gconst93)))+(((gconst58)*(gconst75)*(gconst78)*(gconst79)))+(((gconst59)*(gconst68)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst81)*(gconst82)))+(((gconst63)*(gconst68)*(gconst71)*(gconst88)))+(((gconst61)*(gconst64)*(gconst81)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst72)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst73)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst80)*(gconst86)))+(((gconst60)*(gconst65)*(gconst74)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst75)*(gconst83)))+(((gconst59)*(gconst68)*(gconst75)*(gconst88)))+(((gconst60)*(gconst64)*(gconst81)*(gconst85)))+(((gconst58)*(gconst67)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst77)*(gconst91)))+(((gconst64)*(gconst71)*(gconst75)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst76)*(gconst82)))+(((gconst60)*(gconst69)*(gconst76)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst84)))+(((gconst60)*(gconst61)*(gconst81)*(gconst88)))+(((gconst60)*(gconst64)*(gconst73)*(gconst93)))+(((gconst61)*(gconst69)*(gconst72)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst70)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst71)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst72)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst78)*(gconst85)))+(((gconst64)*(gconst72)*(gconst73)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst75)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst76)*(gconst87)))+(((gconst65)*(gconst67)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst79)*(gconst85)))+(((gconst59)*(gconst63)*(gconst80)*(gconst88)))+(((gconst63)*(gconst64)*(gconst71)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst79)*(gconst83)))+(((gconst63)*(gconst66)*(gconst70)*(gconst91)))+(((gconst67)*(gconst72)*(gconst74)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst72)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst79)*(gconst84)))+(((gconst66)*(gconst67)*(gconst70)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst77)*(gconst87)))+(((gconst62)*(gconst65)*(gconst79)*(gconst84)))+(((gconst63)*(gconst71)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst78)*(gconst84)))+(((gconst65)*(gconst72)*(gconst74)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst79)*(gconst87)))+(((gconst60)*(gconst62)*(gconst79)*(gconst89)))+(((gconst59)*(gconst63)*(gconst76)*(gconst92)))+(((gconst63)*(gconst67)*(gconst70)*(gconst90)))+(((gconst64)*(gconst69)*(gconst73)*(gconst84)))+(((gconst61)*(gconst72)*(gconst76)*(gconst81)))+(((gconst62)*(gconst67)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst76)*(gconst86)))+(((gconst67)*(gconst70)*(gconst75)*(gconst78)))+(((gconst60)*(gconst73)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst75)*(gconst89)))+(((gconst64)*(gconst69)*(gconst72)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst87)))+(((gconst58)*(gconst66)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst75)*(gconst91))));
op[6]=((((gconst61)*(gconst64)*(gconst69)*(gconst84)))+(((gconst60)*(gconst62)*(gconst65)*(gconst91)))+(((gconst62)*(gconst65)*(gconst67)*(gconst84)))+(((gconst62)*(gconst67)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst71)*(gconst75)))+(((gconst61)*(gconst64)*(gconst72)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst69)*(gconst87)))+(((gconst58)*(gconst66)*(gconst67)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst69)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst67)*(gconst90)))+(((gconst60)*(gconst69)*(gconst73)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst66)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst64)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst69)*(gconst82)))+(((gconst58)*(gconst63)*(gconst78)*(gconst79)))+(((gconst63)*(gconst68)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(gconst78)))+(((gconst58)*(gconst67)*(gconst75)*(gconst78)))+(((gconst63)*(gconst67)*(gconst70)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst67)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst67)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst70)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst71)*(gconst79)))+(((gconst59)*(gconst68)*(gconst75)*(gconst76)))+(((gconst60)*(gconst61)*(gconst64)*(gconst93)))+(((gconst59)*(gconst63)*(gconst64)*(gconst92)))+(((gconst61)*(gconst69)*(gconst72)*(gconst76)))+(((gconst58)*(gconst63)*(gconst66)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst77)*(gconst79)))+(((gconst60)*(gconst61)*(gconst76)*(gconst81)))+(((gconst59)*(gconst64)*(gconst75)*(gconst80)))+(((gconst60)*(gconst64)*(gconst73)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst68)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst75)*(gconst77)))+(((gconst63)*(gconst66)*(gconst70)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst74)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst72)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst70)*(gconst76)))+(((gconst65)*(gconst67)*(gconst72)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst68)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst72)*(gconst80)))+(((gconst59)*(gconst63)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst73)*(gconst79)))+(((gconst64)*(gconst69)*(gconst72)*(gconst73)))+(((gconst60)*(gconst65)*(gconst67)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst67)*(gconst83)))+(((gconst66)*(gconst67)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst65)*(gconst91)))+(((gconst60)*(gconst61)*(gconst69)*(gconst88)))+(((gconst59)*(gconst63)*(gconst68)*(gconst88)))+(((gconst59)*(gconst64)*(gconst68)*(gconst87)))+(((gconst60)*(gconst62)*(gconst67)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst72)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst68)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst67)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst72)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst73)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst76)*(gconst81)))+(((gconst64)*(gconst68)*(gconst71)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst71)*(gconst77)))+(((gconst60)*(gconst65)*(gconst74)*(gconst79)))+(((gconst60)*(gconst67)*(gconst74)*(gconst77)))+(((gconst60)*(gconst64)*(gconst69)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst67)*(gconst85)))+(((gconst63)*(gconst66)*(gconst67)*(gconst82)))+(((gconst58)*(gconst63)*(gconst67)*(gconst90)))+(((gconst58)*(gconst66)*(gconst75)*(gconst79)))+(((gconst63)*(gconst64)*(gconst68)*(gconst83)))+(((gconst63)*(gconst64)*(gconst71)*(gconst80)))+(((gconst62)*(gconst65)*(gconst72)*(gconst79)))+(((gconst60)*(gconst62)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst64)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst75)*(gconst76))));
op[7]=((((IkReal(-1.00000000000000))*(gconst80)*(x294)*(x308)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(x295)))+(((x290)*(x295)))+(((IkReal(-1.00000000000000))*(gconst66)*(x280)*(x310)))+(((IkReal(-1.00000000000000))*(x290)*(x318)))+(((x278)*(x289)))+(((gconst65)*(gconst79)*(x300)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst81)*(x296)))+(((IkReal(-1.00000000000000))*(x283)*(x299)))+(((gconst64)*(gconst80)*(x283)))+(((IkReal(-1.00000000000000))*(gconst74)*(x284)*(x311)))+(((IkReal(-1.00000000000000))*(x298)*(x310)))+(((IkReal(-1.00000000000000))*(gconst68)*(x289)*(x308)))+(((x299)*(x300)))+(((x284)*(x316)))+(((x284)*(x297)))+(((IkReal(-1.00000000000000))*(x282)*(x297)))+(((gconst69)*(gconst73)*(x294)))+(((IkReal(-1.00000000000000))*(x305)*(x318)))+(((x295)*(x305)))+(((IkReal(-1.00000000000000))*(x282)*(x316)))+(((gconst68)*(gconst76)*(x283)))+(((IkReal(-1.00000000000000))*(gconst65)*(x280)*(x283)))+(((x296)*(x298)))+(((x278)*(x279)))+(((gconst60)*(gconst74)*(x282)))+(((IkReal(-1.00000000000000))*(gconst68)*(x279)*(x308)))+(((gconst61)*(gconst81)*(x294)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst76)*(x296)))+(((gconst62)*(gconst72)*(x282)))+(((gconst66)*(gconst79)*(x296)))+(((IkReal(-1.00000000000000))*(gconst73)*(x295)*(x311))));
op[8]=((((IkReal(-1.00000000000000))*(x282)*(x283)))+(((x282)*(x300)))+(((x278)*(x294)))+(((x295)*(x296)))+(((IkReal(-1.00000000000000))*(x295)*(x310)))+(((x283)*(x284)))+(((IkReal(-1.00000000000000))*(x284)*(x300)))+(((IkReal(-1.00000000000000))*(x296)*(x318))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((cj0)*(px)))+(((py)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst111;
IkReal x320=(px)*(px);
IkReal x321=(sj0)*(sj0);
IkReal x322=(cj0)*(cj0);
IkReal x323=(py)*(py);
IkReal x324=((IkReal(0.0900000000000000))*(x321));
IkReal x325=((IkReal(0.0900000000000000))*(x322));
gconst111=IKsign(((((x323)*(x325)))+(((x323)*(x324)))+(((x320)*(x324)))+(((x320)*(x325)))));
IkReal x326=(sj0)*(sj0);
IkReal x327=(py)*(py);
IkReal x328=(cj0)*(cj0);
IkReal x329=(px)*(px);
dummyeval[0]=((((x326)*(x329)))+(((x326)*(x327)))+(((x328)*(x329)))+(((x327)*(x328))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x330=((cj0)*(px));
IkReal x331=((IkReal(1.10000000000000))*(pz));
IkReal x332=((IkReal(1.00000000000000))*(pp));
IkReal x333=((px)*(sj0));
IkReal x334=((cj0)*(py));
IkReal x335=((py)*(sj0));
if( IKabs(((gconst111)*(((((IkReal(-0.212500000000000))*(x333)))+(((x331)*(x334)))+(((pp)*(x334)))+(((IkReal(-1.00000000000000))*(x331)*(x333)))+(((IkReal(-1.00000000000000))*(x332)*(x333)))+(((IkReal(0.212500000000000))*(x334))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst111)*(((((IkReal(-0.212500000000000))*(x330)))+(((IkReal(-0.212500000000000))*(x335)))+(((IkReal(-1.00000000000000))*(x330)*(x332)))+(((IkReal(-1.00000000000000))*(x330)*(x331)))+(((IkReal(-1.00000000000000))*(x331)*(x335)))+(((IkReal(-1.00000000000000))*(x332)*(x335))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst111)*(((((IkReal(-0.212500000000000))*(x333)))+(((x331)*(x334)))+(((pp)*(x334)))+(((IkReal(-1.00000000000000))*(x331)*(x333)))+(((IkReal(-1.00000000000000))*(x332)*(x333)))+(((IkReal(0.212500000000000))*(x334)))))), ((gconst111)*(((((IkReal(-0.212500000000000))*(x330)))+(((IkReal(-0.212500000000000))*(x335)))+(((IkReal(-1.00000000000000))*(x330)*(x332)))+(((IkReal(-1.00000000000000))*(x330)*(x331)))+(((IkReal(-1.00000000000000))*(x331)*(x335)))+(((IkReal(-1.00000000000000))*(x332)*(x335)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x336=IKcos(j2);
IkReal x337=IKsin(j2);
IkReal x338=((IkReal(0.0900000000000000))*(py));
IkReal x339=((IkReal(0.0900000000000000))*(px));
IkReal x340=((IkReal(1.00000000000000))*(py));
IkReal x341=((sj0)*(x336));
IkReal x342=((cj0)*(x336));
IkReal x343=((sj0)*(x337));
IkReal x344=((IkReal(1.00000000000000))*(cj0)*(x337));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x344)))+(((IkReal(-1.00000000000000))*(x340)*(x342)))+(((IkReal(-1.00000000000000))*(x340)*(x343)))+(((px)*(x341))));
evalcond[1]=((((py)*(x341)))+(((IkReal(-1.00000000000000))*(cj0)*(x337)*(x340)))+(((px)*(x343)))+(((px)*(x342))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x338)*(x341)))+(((IkReal(-1.00000000000000))*(x339)*(x342)))+(((IkReal(-1.00000000000000))*(x339)*(x343)))+(((cj0)*(x337)*(x338))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((cj0)*(px)))+(((py)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst112;
IkReal x345=(px)*(px);
IkReal x346=(sj0)*(sj0);
IkReal x347=(cj0)*(cj0);
IkReal x348=(py)*(py);
IkReal x349=((IkReal(0.0900000000000000))*(x346));
IkReal x350=((IkReal(0.0900000000000000))*(x347));
gconst112=IKsign(((((x345)*(x350)))+(((x348)*(x349)))+(((x345)*(x349)))+(((x348)*(x350)))));
IkReal x351=(sj0)*(sj0);
IkReal x352=(py)*(py);
IkReal x353=(cj0)*(cj0);
IkReal x354=(px)*(px);
dummyeval[0]=((((x352)*(x353)))+(((x353)*(x354)))+(((x351)*(x354)))+(((x351)*(x352))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x355=((cj0)*(px));
IkReal x356=((IkReal(1.10000000000000))*(pz));
IkReal x357=((IkReal(1.00000000000000))*(pp));
IkReal x358=((px)*(sj0));
IkReal x359=((cj0)*(py));
IkReal x360=((py)*(sj0));
if( IKabs(((gconst112)*(((((x356)*(x359)))+(((IkReal(-1.00000000000000))*(x357)*(x358)))+(((IkReal(-1.00000000000000))*(x356)*(x358)))+(((IkReal(-0.212500000000000))*(x358)))+(((pp)*(x359)))+(((IkReal(0.212500000000000))*(x359))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst112)*(((((IkReal(-1.00000000000000))*(x356)*(x360)))+(((IkReal(-0.212500000000000))*(x355)))+(((IkReal(-0.212500000000000))*(x360)))+(((IkReal(-1.00000000000000))*(x357)*(x360)))+(((IkReal(-1.00000000000000))*(x355)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x357))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst112)*(((((x356)*(x359)))+(((IkReal(-1.00000000000000))*(x357)*(x358)))+(((IkReal(-1.00000000000000))*(x356)*(x358)))+(((IkReal(-0.212500000000000))*(x358)))+(((pp)*(x359)))+(((IkReal(0.212500000000000))*(x359)))))), ((gconst112)*(((((IkReal(-1.00000000000000))*(x356)*(x360)))+(((IkReal(-0.212500000000000))*(x355)))+(((IkReal(-0.212500000000000))*(x360)))+(((IkReal(-1.00000000000000))*(x357)*(x360)))+(((IkReal(-1.00000000000000))*(x355)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x357)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x361=IKcos(j2);
IkReal x362=IKsin(j2);
IkReal x363=((IkReal(0.0900000000000000))*(py));
IkReal x364=((IkReal(0.0900000000000000))*(px));
IkReal x365=((IkReal(1.00000000000000))*(py));
IkReal x366=((sj0)*(x361));
IkReal x367=((cj0)*(x361));
IkReal x368=((sj0)*(x362));
IkReal x369=((IkReal(1.00000000000000))*(cj0)*(x362));
evalcond[0]=((((px)*(x366)))+(((IkReal(-1.00000000000000))*(px)*(x369)))+(((IkReal(-1.00000000000000))*(x365)*(x368)))+(((IkReal(-1.00000000000000))*(x365)*(x367))));
evalcond[1]=((((py)*(x366)))+(((px)*(x368)))+(((px)*(x367)))+(((IkReal(-1.00000000000000))*(cj0)*(x362)*(x365))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x363)*(x366)))+(((cj0)*(x362)*(x363)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x364)*(x368)))+(((IkReal(-1.00000000000000))*(x364)*(x367))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x370=((IkReal(1111111111.11111))*(px));
IkReal x371=((IkReal(1111111111.11111))*(py));
if( IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x370)))+(((cj0)*(x371))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x371)))+(((IkReal(-1.00000000000000))*(cj0)*(x370))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x370)))+(((cj0)*(x371)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x371)))+(((IkReal(-1.00000000000000))*(cj0)*(x370)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj0)*(x370)))+(((cj0)*(x371)))), ((((IkReal(-1.00000000000000))*(sj0)*(x371)))+(((IkReal(-1.00000000000000))*(cj0)*(x370)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x372=IKcos(j2);
IkReal x373=IKsin(j2);
IkReal x374=((cj0)*(px));
IkReal x375=((px)*(sj0));
IkReal x376=((py)*(sj0));
IkReal x377=((cj0)*(py));
IkReal x378=((IkReal(1.00000000000000))*(x373));
IkReal x379=((IkReal(0.0900000000000000))*(x373));
IkReal x380=((IkReal(0.0900000000000000))*(x372));
evalcond[0]=((((IkReal(-1.00000000000000))*(x377)))+(x375)+(((IkReal(9.00000000000000e-10))*(x373))));
evalcond[1]=((x376)+(x374)+(((IkReal(9.00000000000000e-10))*(x372))));
evalcond[2]=((((x372)*(x375)))+(((IkReal(-1.00000000000000))*(x374)*(x378)))+(((IkReal(-1.00000000000000))*(x372)*(x377)))+(((IkReal(-1.00000000000000))*(x376)*(x378))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x377)*(x378)))+(((x372)*(x376)))+(((x372)*(x374)))+(((x373)*(x375))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x376)*(x380)))+(((IkReal(-1.00000000000000))*(x374)*(x380)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((x377)*(x379)))+(((IkReal(-1.00000000000000))*(x375)*(x379))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x381=((IkReal(1.00000000000000))*(px));
if( IKabs(((gconst110)*(((((IkReal(-1.00000000000000))*(sj0)*(x381)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst110)*(((((IkReal(-1.00000000000000))*(cj0)*(x381)))+(((IkReal(-1.00000000000000))*(py)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst110)*(((((IkReal(-1.00000000000000))*(sj0)*(x381)))+(((cj0)*(py)))))), ((gconst110)*(((((IkReal(-1.00000000000000))*(cj0)*(x381)))+(((IkReal(-1.00000000000000))*(py)*(sj0)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x382=IKsin(j2);
IkReal x383=IKcos(j2);
IkReal x384=((cj0)*(px));
IkReal x385=((IkReal(0.0450000000000000))*(cj3));
IkReal x386=((cj0)*(py));
IkReal x387=((py)*(sj0));
IkReal x388=((px)*(sj0));
IkReal x389=((IkReal(0.300000000000000))*(sj3));
IkReal x390=((IkReal(0.0900000000000000))*(x383));
IkReal x391=((IkReal(1.00000000000000))*(x382));
IkReal x392=((IkReal(0.0900000000000000))*(x382));
evalcond[0]=((x388)+(((IkReal(0.0450000000000000))*(x382)))+(((IkReal(-1.00000000000000))*(x386)))+(((x382)*(x389)))+(((IkReal(-1.00000000000000))*(x382)*(x385))));
evalcond[1]=((((x383)*(x389)))+(x387)+(x384)+(((IkReal(0.0450000000000000))*(x383)))+(((IkReal(-1.00000000000000))*(x383)*(x385))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x387)*(x391)))+(((IkReal(-1.00000000000000))*(x384)*(x391)))+(((IkReal(-1.00000000000000))*(x383)*(x386)))+(((x383)*(x388))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x386)*(x391)))+(((x383)*(x387)))+(((x383)*(x384)))+(x389)+(((x382)*(x388)))+(((IkReal(-1.00000000000000))*(x385))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x387)*(x390)))+(((IkReal(-1.00000000000000))*(x384)*(x390)))+(((IkReal(-1.00000000000000))*(x388)*(x392)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((x386)*(x392))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x393=(px)*(px);
IkReal x394=(sj1)*(sj1);
IkReal x395=((IkReal(1.21000000000000))*(x394));
IkReal x396=((IkReal(-1.10000000000000))*(sj1));
if( IKabs(((x393)*(x396))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((px)*(py)*(x396))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x397=((IkReal(1.00000000000000))*(IKatan2(((x393)*(x396)), ((px)*(py)*(x396)))));
if( (((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py))))))))):(IkReal)1.0e30))*(((((IkReal(-1.10000000000000))*(cj1)*(px)*(pz)))+(((IkReal(-0.0270000000000000))*(px)*(sj3)))+(((IkReal(0.00405000000000000))*(cj3)*(px)))+(((IkReal(0.208450000000000))*(px)))+(((pp)*(px))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py))))))))):(IkReal)1.0e30))*(((((IkReal(-1.10000000000000))*(cj1)*(px)*(pz)))+(((IkReal(-0.0270000000000000))*(px)*(sj3)))+(((IkReal(0.00405000000000000))*(cj3)*(px)))+(((IkReal(0.208450000000000))*(px)))+(((pp)*(px))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x398=IKasin(((((IKabs(IKabs(IKsqrt(((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x395)*((x393)*(x393))))+(((x393)*(x395)*((py)*(py))))))))):(IkReal)1.0e30))*(((((IkReal(-1.10000000000000))*(cj1)*(px)*(pz)))+(((IkReal(-0.0270000000000000))*(px)*(sj3)))+(((IkReal(0.00405000000000000))*(cj3)*(px)))+(((IkReal(0.208450000000000))*(px)))+(((pp)*(px)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x397)))+(((IkReal(-1.00000000000000))*(x398))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x397)))+(x398));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x399=IKcos(j0);
IkReal x400=IKsin(j0);
IkReal x401=(py)*(py);
IkReal x402=(cj1)*(cj1);
IkReal x403=((IkReal(0.208450000000000))*(py));
IkReal x404=((cj1)*(py));
IkReal x405=((IkReal(0.0270000000000000))*(sj3));
IkReal x406=((cj1)*(px));
IkReal x407=((IkReal(0.00405000000000000))*(cj3));
IkReal x408=((IkReal(1.10000000000000))*(pz));
IkReal x409=((IkReal(1.00000000000000))*(pp));
IkReal x410=((sj1)*(x400));
IkReal x411=((IkReal(1.10000000000000))*(x401));
IkReal x412=((sj1)*(x399));
IkReal x413=((IkReal(1.10000000000000))*(px)*(x412));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((py)*(x410)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz)))+(((px)*(x412))));
evalcond[1]=((((py)*(x413)))+(((py)*(x405)))+(((x404)*(x408)))+(((x410)*(x411)))+(((IkReal(-1.00000000000000))*(py)*(x409)))+(((IkReal(-1.00000000000000))*(py)*(x407)))+(((IkReal(-1.00000000000000))*(x403))));
evalcond[2]=((((x404)*(x413)))+(((IkReal(-1.00000000000000))*(cj1)*(x403)))+(((x404)*(x405)))+(((cj1)*(x410)*(x411)))+(((py)*(x402)*(x408)))+(((IkReal(-1.00000000000000))*(x404)*(x409)))+(((IkReal(-1.00000000000000))*(x404)*(x407))));
evalcond[3]=((((px)*(x402)*(x408)))+(((x405)*(x406)))+(((IkReal(-1.00000000000000))*(x406)*(x407)))+(((IkReal(-1.00000000000000))*(x406)*(x409)))+(((IkReal(-0.208450000000000))*(x406)))+(((x406)*(x413)))+(((IkReal(1.10000000000000))*(px)*(x404)*(x410))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x414=((cj0)*(px));
IkReal x415=((py)*(sj0));
IkReal x416=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj1)*(x415)))+(((sj1)*(x414)))+(((cj1)*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x415)*(x416)))+(((pz)*(sj1)))+(((IkReal(-1.00000000000000))*(x414)*(x416))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x417=(cj1)*(cj1);
IkReal x418=(px)*(px);
IkReal x419=(sj0)*(sj0);
IkReal x420=(cj0)*(cj0);
IkReal x421=(py)*(py);
IkReal x422=((py)*(sj0));
IkReal x423=((IkReal(0.0900000000000000))*(x419));
IkReal x424=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x425=((cj1)*(pz)*(sj1));
IkReal x426=((IkReal(0.0900000000000000))*(x420));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x422)*(x424)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x417)*(x421)*(x423)))+(((x418)*(x423)))+(((x417)*(x418)*(x426)))+(((IkReal(-1.00000000000000))*(x424)*(x425)))+(((IkReal(-0.180000000000000))*(x422)*(x425)))+(((x417)*(x422)*(x424)))+(((x421)*(x426)))));
IkReal x427=(sj0)*(sj0);
IkReal x428=(py)*(py);
IkReal x429=(cj1)*(cj1);
IkReal x430=(cj0)*(cj0);
IkReal x431=(px)*(px);
IkReal x432=((cj0)*(px));
IkReal x433=((py)*(sj0));
IkReal x434=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x433)*(x434)))+(((x429)*(x430)*(x431)))+(((x428)*(x430)))+(((IkReal(-1.00000000000000))*(x432)*(x434)))+(((IkReal(-2.00000000000000))*(x432)*(x433)))+(((x427)*(x428)*(x429)))+((((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(2.00000000000000))*(x429)*(x432)*(x433)))+(((x427)*(x431))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x435=(sj1)*(sj1);
IkReal x436=(py)*(py);
IkReal x437=(sj0)*(sj0);
IkReal x438=(cj1)*(cj1);
IkReal x439=(cj0)*(cj0);
IkReal x440=(px)*(px);
IkReal x441=((IkReal(1.10000000000000))*(sj1));
IkReal x442=((px)*(py));
IkReal x443=((py)*(sj0));
IkReal x444=((cj1)*(pp));
IkReal x445=((px)*(sj0));
IkReal x446=((IkReal(1.00000000000000))*(pp));
IkReal x447=((cj0)*(sj0));
IkReal x448=((cj0)*(px));
IkReal x449=((cj0)*(py));
IkReal x450=((pz)*(sj1));
IkReal x451=((IkReal(0.212500000000000))*(cj1));
IkReal x452=((IkReal(1.10000000000000))*(pz));
IkReal x453=((x435)*(x452));
if( IKabs(((gconst1)*(((((x440)*(x441)*(x447)))+(((IkReal(-1.00000000000000))*(x445)*(x446)))+(((x437)*(x441)*(x442)))+(((IkReal(0.212500000000000))*(x449)))+(((IkReal(-0.212500000000000))*(x445)))+(((IkReal(-1.00000000000000))*(x439)*(x441)*(x442)))+(((pp)*(x449)))+(((cj1)*(x445)*(x452)))+(((IkReal(-1.00000000000000))*(cj1)*(x449)*(x452)))+(((IkReal(-1.00000000000000))*(x436)*(x441)*(x447))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(cj1)*(x436)*(x437)*(x441)))+(((IkReal(-1.00000000000000))*(x446)*(x450)))+(((IkReal(-1.00000000000000))*(x438)*(x448)*(x452)))+(((IkReal(-1.00000000000000))*(x438)*(x443)*(x452)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x442)*(x447)))+(((IkReal(-1.00000000000000))*(cj1)*(x439)*(x440)*(x441)))+(((cj1)*(x441)*((pz)*(pz))))+(((x444)*(x448)))+(((x443)*(x453)))+(((x443)*(x451)))+(((x448)*(x453)))+(((x448)*(x451)))+(((x443)*(x444)))+(((IkReal(-0.212500000000000))*(x450))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((x440)*(x441)*(x447)))+(((IkReal(-1.00000000000000))*(x445)*(x446)))+(((x437)*(x441)*(x442)))+(((IkReal(0.212500000000000))*(x449)))+(((IkReal(-0.212500000000000))*(x445)))+(((IkReal(-1.00000000000000))*(x439)*(x441)*(x442)))+(((pp)*(x449)))+(((cj1)*(x445)*(x452)))+(((IkReal(-1.00000000000000))*(cj1)*(x449)*(x452)))+(((IkReal(-1.00000000000000))*(x436)*(x441)*(x447)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(cj1)*(x436)*(x437)*(x441)))+(((IkReal(-1.00000000000000))*(x446)*(x450)))+(((IkReal(-1.00000000000000))*(x438)*(x448)*(x452)))+(((IkReal(-1.00000000000000))*(x438)*(x443)*(x452)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x442)*(x447)))+(((IkReal(-1.00000000000000))*(cj1)*(x439)*(x440)*(x441)))+(((cj1)*(x441)*((pz)*(pz))))+(((x444)*(x448)))+(((x443)*(x453)))+(((x443)*(x451)))+(((x448)*(x453)))+(((x448)*(x451)))+(((x443)*(x444)))+(((IkReal(-0.212500000000000))*(x450)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x454=IKsin(j2);
IkReal x455=IKcos(j2);
IkReal x456=((px)*(sj0));
IkReal x457=((IkReal(1.10000000000000))*(sj1));
IkReal x458=((py)*(sj0));
IkReal x459=((pz)*(sj1));
IkReal x460=((cj0)*(px));
IkReal x461=((IkReal(0.0900000000000000))*(x455));
IkReal x462=((cj1)*(x454));
IkReal x463=((IkReal(1.00000000000000))*(x455));
IkReal x464=((cj0)*(py)*(x454));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(x463)))+(((x455)*(x456)))+(((x460)*(x462)))+(((IkReal(-1.00000000000000))*(x454)*(x459)))+(((x458)*(x462))));
evalcond[1]=((((x454)*(x456)))+(((x455)*(x459)))+(((IkReal(-1.00000000000000))*(x464)))+(((IkReal(-1.00000000000000))*(cj1)*(x458)*(x463)))+(((IkReal(-1.00000000000000))*(cj1)*(x460)*(x463))));
evalcond[2]=((IkReal(-0.212500000000000))+(((cj1)*(x460)*(x461)))+(((IkReal(-0.0900000000000000))*(x454)*(x456)))+(((cj1)*(x458)*(x461)))+(((IkReal(-1.00000000000000))*(x459)*(x461)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(0.0900000000000000))*(x464)))+(((IkReal(-1.00000000000000))*(pp)))+(((x457)*(x460)))+(((x457)*(x458))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x465=((cj0)*(px));
IkReal x466=((py)*(sj0));
IkReal x467=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((cj1)*(pz)))+(((sj1)*(x465)))+(((sj1)*(x466))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x465)*(x467)))+(((IkReal(-1.00000000000000))*(x466)*(x467)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x468=(cj1)*(cj1);
IkReal x469=(px)*(px);
IkReal x470=(sj0)*(sj0);
IkReal x471=(cj0)*(cj0);
IkReal x472=(py)*(py);
IkReal x473=((py)*(sj0));
IkReal x474=((IkReal(0.0900000000000000))*(x470));
IkReal x475=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x476=((cj1)*(pz)*(sj1));
IkReal x477=((IkReal(0.0900000000000000))*(x471));
gconst2=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x468)*(x472)*(x474)))+(((x469)*(x474)))+(((x472)*(x477)))+(((x468)*(x473)*(x475)))+(((IkReal(-1.00000000000000))*(x475)*(x476)))+(((x468)*(x469)*(x477)))+(((IkReal(-1.00000000000000))*(x473)*(x475)))+(((IkReal(-0.180000000000000))*(x473)*(x476)))));
IkReal x478=(sj0)*(sj0);
IkReal x479=(py)*(py);
IkReal x480=(cj1)*(cj1);
IkReal x481=(cj0)*(cj0);
IkReal x482=(px)*(px);
IkReal x483=((cj0)*(px));
IkReal x484=((py)*(sj0));
IkReal x485=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x484)*(x485)))+(((IkReal(-1.00000000000000))*(x483)*(x485)))+(((IkReal(2.00000000000000))*(x480)*(x483)*(x484)))+(((IkReal(-2.00000000000000))*(x483)*(x484)))+(((x478)*(x479)*(x480)))+(((x478)*(x482)))+((((pz)*(pz))*((sj1)*(sj1))))+(((x479)*(x481)))+(((x480)*(x481)*(x482))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x486=(sj1)*(sj1);
IkReal x487=(py)*(py);
IkReal x488=(sj0)*(sj0);
IkReal x489=(cj1)*(cj1);
IkReal x490=(cj0)*(cj0);
IkReal x491=(px)*(px);
IkReal x492=((IkReal(1.10000000000000))*(sj1));
IkReal x493=((px)*(py));
IkReal x494=((py)*(sj0));
IkReal x495=((cj1)*(pp));
IkReal x496=((px)*(sj0));
IkReal x497=((IkReal(1.00000000000000))*(pp));
IkReal x498=((cj0)*(sj0));
IkReal x499=((cj0)*(px));
IkReal x500=((cj0)*(py));
IkReal x501=((pz)*(sj1));
IkReal x502=((IkReal(0.212500000000000))*(cj1));
IkReal x503=((IkReal(1.10000000000000))*(pz));
IkReal x504=((x486)*(x503));
if( IKabs(((gconst2)*(((((IkReal(0.212500000000000))*(x500)))+(((IkReal(-1.00000000000000))*(x490)*(x492)*(x493)))+(((IkReal(-1.00000000000000))*(cj1)*(x500)*(x503)))+(((IkReal(-0.212500000000000))*(x496)))+(((pp)*(x500)))+(((IkReal(-1.00000000000000))*(x496)*(x497)))+(((cj1)*(x496)*(x503)))+(((x491)*(x492)*(x498)))+(((IkReal(-1.00000000000000))*(x487)*(x492)*(x498)))+(((x488)*(x492)*(x493))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x493)*(x498)))+(((IkReal(-1.00000000000000))*(x489)*(x494)*(x503)))+(((x494)*(x502)))+(((x494)*(x504)))+(((x495)*(x499)))+(((IkReal(-1.00000000000000))*(x489)*(x499)*(x503)))+(((IkReal(-1.00000000000000))*(x497)*(x501)))+(((IkReal(-1.00000000000000))*(cj1)*(x487)*(x488)*(x492)))+(((x494)*(x495)))+(((cj1)*(x492)*((pz)*(pz))))+(((x499)*(x504)))+(((x499)*(x502)))+(((IkReal(-0.212500000000000))*(x501)))+(((IkReal(-1.00000000000000))*(cj1)*(x490)*(x491)*(x492))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(0.212500000000000))*(x500)))+(((IkReal(-1.00000000000000))*(x490)*(x492)*(x493)))+(((IkReal(-1.00000000000000))*(cj1)*(x500)*(x503)))+(((IkReal(-0.212500000000000))*(x496)))+(((pp)*(x500)))+(((IkReal(-1.00000000000000))*(x496)*(x497)))+(((cj1)*(x496)*(x503)))+(((x491)*(x492)*(x498)))+(((IkReal(-1.00000000000000))*(x487)*(x492)*(x498)))+(((x488)*(x492)*(x493)))))), ((gconst2)*(((((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x493)*(x498)))+(((IkReal(-1.00000000000000))*(x489)*(x494)*(x503)))+(((x494)*(x502)))+(((x494)*(x504)))+(((x495)*(x499)))+(((IkReal(-1.00000000000000))*(x489)*(x499)*(x503)))+(((IkReal(-1.00000000000000))*(x497)*(x501)))+(((IkReal(-1.00000000000000))*(cj1)*(x487)*(x488)*(x492)))+(((x494)*(x495)))+(((cj1)*(x492)*((pz)*(pz))))+(((x499)*(x504)))+(((x499)*(x502)))+(((IkReal(-0.212500000000000))*(x501)))+(((IkReal(-1.00000000000000))*(cj1)*(x490)*(x491)*(x492)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x505=IKsin(j2);
IkReal x506=IKcos(j2);
IkReal x507=((px)*(sj0));
IkReal x508=((IkReal(1.10000000000000))*(sj1));
IkReal x509=((py)*(sj0));
IkReal x510=((pz)*(sj1));
IkReal x511=((cj0)*(px));
IkReal x512=((IkReal(0.0900000000000000))*(x506));
IkReal x513=((cj1)*(x505));
IkReal x514=((IkReal(1.00000000000000))*(x506));
IkReal x515=((cj0)*(py)*(x505));
evalcond[0]=((((x511)*(x513)))+(((x506)*(x507)))+(((IkReal(-1.00000000000000))*(x505)*(x510)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x514)))+(((x509)*(x513))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x509)*(x514)))+(((IkReal(-1.00000000000000))*(x515)))+(((x505)*(x507)))+(((IkReal(-1.00000000000000))*(cj1)*(x511)*(x514)))+(((x506)*(x510))));
evalcond[2]=((IkReal(-0.212500000000000))+(((cj1)*(x509)*(x512)))+(((IkReal(-0.0900000000000000))*(x505)*(x507)))+(((x508)*(x509)))+(((IkReal(0.0900000000000000))*(x515)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x511)*(x512)))+(((x508)*(x511)))+(((IkReal(-1.00000000000000))*(x510)*(x512))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((cj1)*(pz)))+(((cj0)*(px)*(sj1)))+(((py)*(sj0)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x516=((IkReal(1111111111.11111))*(px));
IkReal x517=((IkReal(1111111111.11111))*(py));
if( IKabs(((((cj0)*(x517)))+(((IkReal(-1.00000000000000))*(sj0)*(x516))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(sj0)*(x517)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj0)*(cj1)*(x516))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj0)*(x517)))+(((IkReal(-1.00000000000000))*(sj0)*(x516)))))+IKsqr(((((cj1)*(sj0)*(x517)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj0)*(cj1)*(x516)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((cj0)*(x517)))+(((IkReal(-1.00000000000000))*(sj0)*(x516)))), ((((cj1)*(sj0)*(x517)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj0)*(cj1)*(x516)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x518=IKcos(j2);
IkReal x519=IKsin(j2);
IkReal x520=((px)*(sj0));
IkReal x521=((IkReal(1.10000000000000))*(sj1));
IkReal x522=((pz)*(sj1));
IkReal x523=((cj0)*(px));
IkReal x524=((IkReal(1.00000000000000))*(cj1));
IkReal x525=((cj1)*(py)*(sj0));
IkReal x526=((IkReal(0.0900000000000000))*(x518));
IkReal x527=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x528=((IkReal(0.0900000000000000))*(x519));
evalcond[0]=((((IkReal(-1.00000000000000))*(x527)))+(x520)+(((IkReal(9.00000000000000e-10))*(x519))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x523)*(x524)))+(x522)+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x524)))+(((IkReal(9.00000000000000e-10))*(x518))));
evalcond[2]=((((x519)*(x525)))+(((IkReal(-1.00000000000000))*(x519)*(x522)))+(((cj1)*(x519)*(x523)))+(((x518)*(x520)))+(((IkReal(-1.00000000000000))*(x518)*(x527))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((IkReal(-1.00000000000000))*(x518)*(x523)*(x524)))+(((x519)*(x520)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x518)*(x524)))+(((x518)*(x522)))+(((IkReal(-1.00000000000000))*(x519)*(x527))));
evalcond[4]=((IkReal(-0.212500000000000))+(((cj0)*(py)*(x528)))+(((IkReal(-1.00000000000000))*(x520)*(x528)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((x521)*(x523)))+(((py)*(sj0)*(x521)))+(((x525)*(x526)))+(((cj1)*(x523)*(x526)))+(((IkReal(-1.00000000000000))*(x522)*(x526))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x529=IKsin(j2);
IkReal x530=IKcos(j2);
IkReal x531=((IkReal(0.0450000000000000))*(cj3));
IkReal x532=((px)*(sj0));
IkReal x533=((py)*(sj0));
IkReal x534=((IkReal(1.00000000000000))*(cj0));
IkReal x535=((IkReal(1.10000000000000))*(sj1));
IkReal x536=((IkReal(0.300000000000000))*(sj3));
IkReal x537=((pz)*(sj1));
IkReal x538=((IkReal(0.0900000000000000))*(cj0));
IkReal x539=((cj0)*(px));
IkReal x540=((cj1)*(x530));
IkReal x541=((py)*(x529));
IkReal x542=((cj1)*(x529));
evalcond[0]=((((IkReal(0.0450000000000000))*(x529)))+(((IkReal(-1.00000000000000))*(x529)*(x531)))+(((IkReal(-1.00000000000000))*(py)*(x534)))+(x532)+(((x529)*(x536))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(px)*(x534)))+(((IkReal(0.0450000000000000))*(x530)))+(((IkReal(-1.00000000000000))*(x530)*(x531)))+(x537)+(((IkReal(-1.00000000000000))*(cj1)*(x533)))+(((x530)*(x536))));
evalcond[2]=((((x539)*(x542)))+(((IkReal(-1.00000000000000))*(x529)*(x537)))+(((IkReal(-1.00000000000000))*(py)*(x530)*(x534)))+(((x533)*(x542)))+(((x530)*(x532))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x534)*(x541)))+(((IkReal(-1.00000000000000))*(x531)))+(((IkReal(-1.00000000000000))*(px)*(x534)*(x540)))+(((IkReal(-1.00000000000000))*(x533)*(x540)))+(x536)+(((x530)*(x537)))+(((x529)*(x532))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(x533)*(x540)))+(((px)*(x538)*(x540)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x530)*(x537)))+(((x535)*(x539)))+(((x538)*(x541)))+(((x533)*(x535)))+(((IkReal(-0.0900000000000000))*(x529)*(x532))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x543=(sj1)*(sj1);
if( IKabs(((px)*(sj1))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((py)*(sj1))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x544=((IkReal(1.00000000000000))*(IKatan2(((px)*(sj1)), ((py)*(sj1)))));
if( (((((x543)*((py)*(py))))+(((x543)*((px)*(px)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x543)*((py)*(py))))+(((x543)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x543)*((py)*(py))))+(((x543)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x543)*((py)*(py))))+(((x543)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x543)*((py)*(py))))+(((x543)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x545=IKasin(((((IKabs(IKabs(IKsqrt(((((x543)*((py)*(py))))+(((x543)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x543)*((py)*(py))))+(((x543)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x545)))+(((IkReal(-1.00000000000000))*(x544))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x544)))+(x545));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x546=IKcos(j0);
IkReal x547=IKsin(j0);
IkReal x548=(py)*(py);
IkReal x549=(cj1)*(cj1);
IkReal x550=(px)*(px);
IkReal x551=((IkReal(0.208450000000000))*(py));
IkReal x552=((pp)*(px));
IkReal x553=((cj1)*(py));
IkReal x554=((IkReal(0.0270000000000000))*(sj3));
IkReal x555=((px)*(py));
IkReal x556=((cj1)*(px));
IkReal x557=((IkReal(1.10000000000000))*(pz));
IkReal x558=((IkReal(0.00405000000000000))*(cj3));
IkReal x559=((IkReal(1.00000000000000))*(pp));
IkReal x560=((px)*(x558));
IkReal x561=((IkReal(1.10000000000000))*(sj1)*(x546));
IkReal x562=((IkReal(1.10000000000000))*(sj1)*(x547));
evalcond[0]=((((IkReal(-1.00000000000000))*(x556)*(x557)))+(((IkReal(-1.00000000000000))*(x555)*(x562)))+(((IkReal(-1.00000000000000))*(px)*(x554)))+(x552)+(x560)+(((IkReal(-1.00000000000000))*(x550)*(x561)))+(((IkReal(0.208450000000000))*(px))));
evalcond[1]=((((py)*(x554)))+(((IkReal(-1.00000000000000))*(x551)))+(((x548)*(x562)))+(((IkReal(-1.00000000000000))*(py)*(x559)))+(((IkReal(-1.00000000000000))*(py)*(x558)))+(((x553)*(x557)))+(((x555)*(x561))));
evalcond[2]=((((px)*(x553)*(x561)))+(((IkReal(-1.00000000000000))*(cj1)*(x551)))+(((py)*(x549)*(x557)))+(((IkReal(-1.00000000000000))*(x553)*(x559)))+(((IkReal(-1.00000000000000))*(x553)*(x558)))+(((cj1)*(x548)*(x562)))+(((x553)*(x554))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x556)*(x558)))+(((px)*(x553)*(x562)))+(((cj1)*(x550)*(x561)))+(((x554)*(x556)))+(((px)*(x549)*(x557)))+(((IkReal(-0.208450000000000))*(x556)))+(((IkReal(-1.00000000000000))*(cj1)*(x552))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x563=((cj0)*(px));
IkReal x564=((py)*(sj0));
IkReal x565=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((cj1)*(pz)))+(((sj1)*(x563)))+(((sj1)*(x564))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x564)*(x565)))+(((IkReal(-1.00000000000000))*(x563)*(x565)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x566=(cj1)*(cj1);
IkReal x567=(px)*(px);
IkReal x568=(sj0)*(sj0);
IkReal x569=(cj0)*(cj0);
IkReal x570=(py)*(py);
IkReal x571=((py)*(sj0));
IkReal x572=((IkReal(0.0900000000000000))*(x568));
IkReal x573=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x574=((cj1)*(pz)*(sj1));
IkReal x575=((IkReal(0.0900000000000000))*(x569));
gconst1=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x566)*(x570)*(x572)))+(((x567)*(x572)))+(((IkReal(-1.00000000000000))*(x573)*(x574)))+(((IkReal(-1.00000000000000))*(x571)*(x573)))+(((IkReal(-0.180000000000000))*(x571)*(x574)))+(((x566)*(x567)*(x575)))+(((x570)*(x575)))+(((x566)*(x571)*(x573)))));
IkReal x576=(sj0)*(sj0);
IkReal x577=(py)*(py);
IkReal x578=(cj1)*(cj1);
IkReal x579=(cj0)*(cj0);
IkReal x580=(px)*(px);
IkReal x581=((cj0)*(px));
IkReal x582=((py)*(sj0));
IkReal x583=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x582)*(x583)))+((((pz)*(pz))*((sj1)*(sj1))))+(((x577)*(x579)))+(((IkReal(-2.00000000000000))*(x581)*(x582)))+(((x578)*(x579)*(x580)))+(((IkReal(2.00000000000000))*(x578)*(x581)*(x582)))+(((x576)*(x580)))+(((x576)*(x577)*(x578)))+(((IkReal(-1.00000000000000))*(x581)*(x583))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x584=(sj1)*(sj1);
IkReal x585=(py)*(py);
IkReal x586=(sj0)*(sj0);
IkReal x587=(cj1)*(cj1);
IkReal x588=(cj0)*(cj0);
IkReal x589=(px)*(px);
IkReal x590=((IkReal(1.10000000000000))*(sj1));
IkReal x591=((px)*(py));
IkReal x592=((py)*(sj0));
IkReal x593=((cj1)*(pp));
IkReal x594=((px)*(sj0));
IkReal x595=((IkReal(1.00000000000000))*(pp));
IkReal x596=((cj0)*(sj0));
IkReal x597=((cj0)*(px));
IkReal x598=((cj0)*(py));
IkReal x599=((pz)*(sj1));
IkReal x600=((IkReal(0.212500000000000))*(cj1));
IkReal x601=((IkReal(1.10000000000000))*(pz));
IkReal x602=((x584)*(x601));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x585)*(x590)*(x596)))+(((IkReal(-1.00000000000000))*(x588)*(x590)*(x591)))+(((x589)*(x590)*(x596)))+(((pp)*(x598)))+(((IkReal(0.212500000000000))*(x598)))+(((IkReal(-1.00000000000000))*(x594)*(x595)))+(((IkReal(-0.212500000000000))*(x594)))+(((cj1)*(x594)*(x601)))+(((x586)*(x590)*(x591)))+(((IkReal(-1.00000000000000))*(cj1)*(x598)*(x601))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x587)*(x597)*(x601)))+(((IkReal(-1.00000000000000))*(cj1)*(x585)*(x586)*(x590)))+(((x592)*(x602)))+(((x592)*(x600)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x591)*(x596)))+(((x597)*(x602)))+(((x597)*(x600)))+(((IkReal(-1.00000000000000))*(x587)*(x592)*(x601)))+(((cj1)*(x590)*((pz)*(pz))))+(((IkReal(-0.212500000000000))*(x599)))+(((IkReal(-1.00000000000000))*(x595)*(x599)))+(((IkReal(-1.00000000000000))*(cj1)*(x588)*(x589)*(x590)))+(((x593)*(x597)))+(((x592)*(x593))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x585)*(x590)*(x596)))+(((IkReal(-1.00000000000000))*(x588)*(x590)*(x591)))+(((x589)*(x590)*(x596)))+(((pp)*(x598)))+(((IkReal(0.212500000000000))*(x598)))+(((IkReal(-1.00000000000000))*(x594)*(x595)))+(((IkReal(-0.212500000000000))*(x594)))+(((cj1)*(x594)*(x601)))+(((x586)*(x590)*(x591)))+(((IkReal(-1.00000000000000))*(cj1)*(x598)*(x601)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x587)*(x597)*(x601)))+(((IkReal(-1.00000000000000))*(cj1)*(x585)*(x586)*(x590)))+(((x592)*(x602)))+(((x592)*(x600)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x591)*(x596)))+(((x597)*(x602)))+(((x597)*(x600)))+(((IkReal(-1.00000000000000))*(x587)*(x592)*(x601)))+(((cj1)*(x590)*((pz)*(pz))))+(((IkReal(-0.212500000000000))*(x599)))+(((IkReal(-1.00000000000000))*(x595)*(x599)))+(((IkReal(-1.00000000000000))*(cj1)*(x588)*(x589)*(x590)))+(((x593)*(x597)))+(((x592)*(x593)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x603=IKsin(j2);
IkReal x604=IKcos(j2);
IkReal x605=((px)*(sj0));
IkReal x606=((IkReal(1.10000000000000))*(sj1));
IkReal x607=((py)*(sj0));
IkReal x608=((pz)*(sj1));
IkReal x609=((cj0)*(px));
IkReal x610=((IkReal(0.0900000000000000))*(x604));
IkReal x611=((cj1)*(x603));
IkReal x612=((IkReal(1.00000000000000))*(x604));
IkReal x613=((cj0)*(py)*(x603));
evalcond[0]=((((x604)*(x605)))+(((IkReal(-1.00000000000000))*(x603)*(x608)))+(((x607)*(x611)))+(((x609)*(x611)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x612))));
evalcond[1]=((((x604)*(x608)))+(((IkReal(-1.00000000000000))*(cj1)*(x607)*(x612)))+(((IkReal(-1.00000000000000))*(x613)))+(((x603)*(x605)))+(((IkReal(-1.00000000000000))*(cj1)*(x609)*(x612))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(x613)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x607)*(x610)))+(((x606)*(x609)))+(((x606)*(x607)))+(((cj1)*(x609)*(x610)))+(((IkReal(-0.0900000000000000))*(x603)*(x605)))+(((IkReal(-1.00000000000000))*(x608)*(x610))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x614=((cj0)*(px));
IkReal x615=((py)*(sj0));
IkReal x616=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj1)*(x615)))+(((sj1)*(x614)))+(((cj1)*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x614)*(x616)))+(((pz)*(sj1)))+(((IkReal(-1.00000000000000))*(x615)*(x616))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x617=(cj1)*(cj1);
IkReal x618=(px)*(px);
IkReal x619=(sj0)*(sj0);
IkReal x620=(cj0)*(cj0);
IkReal x621=(py)*(py);
IkReal x622=((py)*(sj0));
IkReal x623=((IkReal(0.0900000000000000))*(x619));
IkReal x624=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x625=((cj1)*(pz)*(sj1));
IkReal x626=((IkReal(0.0900000000000000))*(x620));
gconst2=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-1.00000000000000))*(x622)*(x624)))+(((x617)*(x618)*(x626)))+(((x618)*(x623)))+(((IkReal(-1.00000000000000))*(x624)*(x625)))+(((x621)*(x626)))+(((IkReal(-0.180000000000000))*(x622)*(x625)))+(((x617)*(x621)*(x623)))+(((x617)*(x622)*(x624)))));
IkReal x627=(sj0)*(sj0);
IkReal x628=(py)*(py);
IkReal x629=(cj1)*(cj1);
IkReal x630=(cj0)*(cj0);
IkReal x631=(px)*(px);
IkReal x632=((cj0)*(px));
IkReal x633=((py)*(sj0));
IkReal x634=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x633)*(x634)))+(((x628)*(x630)))+(((x629)*(x630)*(x631)))+((((pz)*(pz))*((sj1)*(sj1))))+(((x627)*(x628)*(x629)))+(((IkReal(2.00000000000000))*(x629)*(x632)*(x633)))+(((x627)*(x631)))+(((IkReal(-1.00000000000000))*(x632)*(x634)))+(((IkReal(-2.00000000000000))*(x632)*(x633))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x635=(sj1)*(sj1);
IkReal x636=(py)*(py);
IkReal x637=(sj0)*(sj0);
IkReal x638=(cj1)*(cj1);
IkReal x639=(cj0)*(cj0);
IkReal x640=(px)*(px);
IkReal x641=((IkReal(1.10000000000000))*(sj1));
IkReal x642=((px)*(py));
IkReal x643=((py)*(sj0));
IkReal x644=((cj1)*(pp));
IkReal x645=((px)*(sj0));
IkReal x646=((IkReal(1.00000000000000))*(pp));
IkReal x647=((cj0)*(sj0));
IkReal x648=((cj0)*(px));
IkReal x649=((cj0)*(py));
IkReal x650=((pz)*(sj1));
IkReal x651=((IkReal(0.212500000000000))*(cj1));
IkReal x652=((IkReal(1.10000000000000))*(pz));
IkReal x653=((x635)*(x652));
if( IKabs(((gconst2)*(((((IkReal(0.212500000000000))*(x649)))+(((IkReal(-1.00000000000000))*(x636)*(x641)*(x647)))+(((IkReal(-0.212500000000000))*(x645)))+(((x640)*(x641)*(x647)))+(((IkReal(-1.00000000000000))*(x639)*(x641)*(x642)))+(((pp)*(x649)))+(((cj1)*(x645)*(x652)))+(((IkReal(-1.00000000000000))*(x645)*(x646)))+(((IkReal(-1.00000000000000))*(cj1)*(x649)*(x652)))+(((x637)*(x641)*(x642))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((x643)*(x644)))+(((x644)*(x648)))+(((IkReal(-0.212500000000000))*(x650)))+(((IkReal(-1.00000000000000))*(x646)*(x650)))+(((IkReal(-1.00000000000000))*(x638)*(x643)*(x652)))+(((x643)*(x653)))+(((x643)*(x651)))+(((cj1)*(x641)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x638)*(x648)*(x652)))+(((x648)*(x653)))+(((x648)*(x651)))+(((IkReal(-1.00000000000000))*(cj1)*(x639)*(x640)*(x641)))+(((IkReal(-1.00000000000000))*(cj1)*(x636)*(x637)*(x641)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x642)*(x647))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(0.212500000000000))*(x649)))+(((IkReal(-1.00000000000000))*(x636)*(x641)*(x647)))+(((IkReal(-0.212500000000000))*(x645)))+(((x640)*(x641)*(x647)))+(((IkReal(-1.00000000000000))*(x639)*(x641)*(x642)))+(((pp)*(x649)))+(((cj1)*(x645)*(x652)))+(((IkReal(-1.00000000000000))*(x645)*(x646)))+(((IkReal(-1.00000000000000))*(cj1)*(x649)*(x652)))+(((x637)*(x641)*(x642)))))), ((gconst2)*(((((x643)*(x644)))+(((x644)*(x648)))+(((IkReal(-0.212500000000000))*(x650)))+(((IkReal(-1.00000000000000))*(x646)*(x650)))+(((IkReal(-1.00000000000000))*(x638)*(x643)*(x652)))+(((x643)*(x653)))+(((x643)*(x651)))+(((cj1)*(x641)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x638)*(x648)*(x652)))+(((x648)*(x653)))+(((x648)*(x651)))+(((IkReal(-1.00000000000000))*(cj1)*(x639)*(x640)*(x641)))+(((IkReal(-1.00000000000000))*(cj1)*(x636)*(x637)*(x641)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x642)*(x647)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x654=IKsin(j2);
IkReal x655=IKcos(j2);
IkReal x656=((px)*(sj0));
IkReal x657=((IkReal(1.10000000000000))*(sj1));
IkReal x658=((py)*(sj0));
IkReal x659=((pz)*(sj1));
IkReal x660=((cj0)*(px));
IkReal x661=((IkReal(0.0900000000000000))*(x655));
IkReal x662=((cj1)*(x654));
IkReal x663=((IkReal(1.00000000000000))*(x655));
IkReal x664=((cj0)*(py)*(x654));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(x663)))+(((x660)*(x662)))+(((x655)*(x656)))+(((x658)*(x662)))+(((IkReal(-1.00000000000000))*(x654)*(x659))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x658)*(x663)))+(((x654)*(x656)))+(((IkReal(-1.00000000000000))*(x664)))+(((IkReal(-1.00000000000000))*(cj1)*(x660)*(x663)))+(((x655)*(x659))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x659)*(x661)))+(((IkReal(0.0900000000000000))*(x664)))+(((x657)*(x660)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x658)*(x661)))+(((x657)*(x658)))+(((cj1)*(x660)*(x661)))+(((IkReal(-0.0900000000000000))*(x654)*(x656))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((cj1)*(pz)))+(((cj0)*(px)*(sj1)))+(((py)*(sj0)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x665=((IkReal(1111111111.11111))*(px));
IkReal x666=((IkReal(1111111111.11111))*(py));
if( IKabs(((((cj0)*(x666)))+(((IkReal(-1.00000000000000))*(sj0)*(x665))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj0)*(cj1)*(x665)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj1)*(sj0)*(x666))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj0)*(x666)))+(((IkReal(-1.00000000000000))*(sj0)*(x665)))))+IKsqr(((((cj0)*(cj1)*(x665)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj1)*(sj0)*(x666)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((cj0)*(x666)))+(((IkReal(-1.00000000000000))*(sj0)*(x665)))), ((((cj0)*(cj1)*(x665)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj1)*(sj0)*(x666)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x667=IKcos(j2);
IkReal x668=IKsin(j2);
IkReal x669=((px)*(sj0));
IkReal x670=((IkReal(1.10000000000000))*(sj1));
IkReal x671=((pz)*(sj1));
IkReal x672=((cj0)*(px));
IkReal x673=((IkReal(1.00000000000000))*(cj1));
IkReal x674=((cj1)*(py)*(sj0));
IkReal x675=((IkReal(0.0900000000000000))*(x667));
IkReal x676=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x677=((IkReal(0.0900000000000000))*(x668));
evalcond[0]=((((IkReal(9.00000000000000e-10))*(x668)))+(x669)+(((IkReal(-1.00000000000000))*(x676))));
evalcond[1]=((((IkReal(9.00000000000000e-10))*(x667)))+(x671)+(((IkReal(-1.00000000000000))*(x672)*(x673)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x673))));
evalcond[2]=((((cj1)*(x668)*(x672)))+(((x667)*(x669)))+(((IkReal(-1.00000000000000))*(x667)*(x676)))+(((x668)*(x674)))+(((IkReal(-1.00000000000000))*(x668)*(x671))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((x668)*(x669)))+(((IkReal(-1.00000000000000))*(x668)*(x676)))+(((IkReal(-1.00000000000000))*(x667)*(x672)*(x673)))+(((x667)*(x671)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x667)*(x673))));
evalcond[4]=((IkReal(-0.212500000000000))+(((py)*(sj0)*(x670)))+(((x674)*(x675)))+(((x670)*(x672)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x669)*(x677)))+(((cj0)*(py)*(x677)))+(((IkReal(-1.00000000000000))*(x671)*(x675)))+(((cj1)*(x672)*(x675))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x678=IKsin(j2);
IkReal x679=IKcos(j2);
IkReal x680=((IkReal(0.0450000000000000))*(cj3));
IkReal x681=((px)*(sj0));
IkReal x682=((py)*(sj0));
IkReal x683=((IkReal(1.00000000000000))*(cj0));
IkReal x684=((IkReal(1.10000000000000))*(sj1));
IkReal x685=((IkReal(0.300000000000000))*(sj3));
IkReal x686=((pz)*(sj1));
IkReal x687=((IkReal(0.0900000000000000))*(cj0));
IkReal x688=((cj0)*(px));
IkReal x689=((cj1)*(x679));
IkReal x690=((py)*(x678));
IkReal x691=((cj1)*(x678));
evalcond[0]=((((x678)*(x685)))+(((IkReal(0.0450000000000000))*(x678)))+(((IkReal(-1.00000000000000))*(x678)*(x680)))+(x681)+(((IkReal(-1.00000000000000))*(py)*(x683))));
evalcond[1]=((((IkReal(0.0450000000000000))*(x679)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x683)))+(((IkReal(-1.00000000000000))*(x679)*(x680)))+(((x679)*(x685)))+(x686)+(((IkReal(-1.00000000000000))*(cj1)*(x682))));
evalcond[2]=((((x682)*(x691)))+(((IkReal(-1.00000000000000))*(py)*(x679)*(x683)))+(((x688)*(x691)))+(((IkReal(-1.00000000000000))*(x678)*(x686)))+(((x679)*(x681))));
evalcond[3]=((IkReal(0.0450000000000000))+(((x678)*(x681)))+(((IkReal(-1.00000000000000))*(x680)))+(((IkReal(-1.00000000000000))*(px)*(x683)*(x689)))+(((IkReal(-1.00000000000000))*(x683)*(x690)))+(((x679)*(x686)))+(x685)+(((IkReal(-1.00000000000000))*(x682)*(x689))));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(x682)*(x689)))+(((x687)*(x690)))+(((x682)*(x684)))+(((IkReal(-0.0900000000000000))*(x679)*(x686)))+(((IkReal(-0.0900000000000000))*(x678)*(x681)))+(((px)*(x687)*(x689)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((x684)*(x688))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x692=(sj1)*(sj1);
if( IKabs(((px)*(sj1))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((py)*(sj1))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x693=((IkReal(1.00000000000000))*(IKatan2(((px)*(sj1)), ((py)*(sj1)))));
if( (((((x692)*((px)*(px))))+(((x692)*((py)*(py)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x692)*((px)*(px))))+(((x692)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x692)*((px)*(px))))+(((x692)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x692)*((px)*(px))))+(((x692)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x692)*((px)*(px))))+(((x692)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x694=IKasin(((((IKabs(IKabs(IKsqrt(((((x692)*((px)*(px))))+(((x692)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x692)*((px)*(px))))+(((x692)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.300000000000000))*(cj3)))+(((cj1)*(pz)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x693)))+(((IkReal(-1.00000000000000))*(x694))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x694)+(((IkReal(-1.00000000000000))*(x693))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(0.300000000000000))*(sj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(6.66666666666667))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x695=((cj0)*(px));
IkReal x696=((py)*(sj0));
IkReal x697=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((sj1)*(x696)))+(((sj1)*(x695)))+(((cj1)*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x696)*(x697)))+(((IkReal(-1.00000000000000))*(x695)*(x697)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x698=(cj1)*(cj1);
IkReal x699=(px)*(px);
IkReal x700=(sj0)*(sj0);
IkReal x701=(cj0)*(cj0);
IkReal x702=(py)*(py);
IkReal x703=((py)*(sj0));
IkReal x704=((IkReal(0.0900000000000000))*(x700));
IkReal x705=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x706=((cj1)*(pz)*(sj1));
IkReal x707=((IkReal(0.0900000000000000))*(x701));
gconst1=IKsign(((((IkReal(-0.180000000000000))*(x703)*(x706)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x702)*(x707)))+(((x698)*(x699)*(x707)))+(((IkReal(-1.00000000000000))*(x703)*(x705)))+(((IkReal(-1.00000000000000))*(x705)*(x706)))+(((x698)*(x702)*(x704)))+(((x699)*(x704)))+(((x698)*(x703)*(x705)))));
IkReal x708=(sj0)*(sj0);
IkReal x709=(py)*(py);
IkReal x710=(cj1)*(cj1);
IkReal x711=(cj0)*(cj0);
IkReal x712=(px)*(px);
IkReal x713=((cj0)*(px));
IkReal x714=((py)*(sj0));
IkReal x715=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x714)*(x715)))+(((IkReal(2.00000000000000))*(x710)*(x713)*(x714)))+(((IkReal(-2.00000000000000))*(x713)*(x714)))+(((x708)*(x709)*(x710)))+(((x709)*(x711)))+((((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-1.00000000000000))*(x713)*(x715)))+(((x708)*(x712)))+(((x710)*(x711)*(x712))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x716=(sj1)*(sj1);
IkReal x717=(py)*(py);
IkReal x718=(sj0)*(sj0);
IkReal x719=(cj1)*(cj1);
IkReal x720=(cj0)*(cj0);
IkReal x721=(px)*(px);
IkReal x722=((IkReal(1.10000000000000))*(sj1));
IkReal x723=((px)*(py));
IkReal x724=((py)*(sj0));
IkReal x725=((cj1)*(pp));
IkReal x726=((px)*(sj0));
IkReal x727=((IkReal(1.00000000000000))*(pp));
IkReal x728=((cj0)*(sj0));
IkReal x729=((cj0)*(px));
IkReal x730=((cj0)*(py));
IkReal x731=((pz)*(sj1));
IkReal x732=((IkReal(0.212500000000000))*(cj1));
IkReal x733=((IkReal(1.10000000000000))*(pz));
IkReal x734=((x716)*(x733));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(cj1)*(x730)*(x733)))+(((IkReal(-0.212500000000000))*(x726)))+(((IkReal(0.212500000000000))*(x730)))+(((IkReal(-1.00000000000000))*(x717)*(x722)*(x728)))+(((x721)*(x722)*(x728)))+(((IkReal(-1.00000000000000))*(x720)*(x722)*(x723)))+(((x718)*(x722)*(x723)))+(((cj1)*(x726)*(x733)))+(((pp)*(x730)))+(((IkReal(-1.00000000000000))*(x726)*(x727))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x719)*(x724)*(x733)))+(((IkReal(-1.00000000000000))*(x727)*(x731)))+(((IkReal(-0.212500000000000))*(x731)))+(((IkReal(-1.00000000000000))*(cj1)*(x717)*(x718)*(x722)))+(((cj1)*(x722)*((pz)*(pz))))+(((x729)*(x734)))+(((x729)*(x732)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x723)*(x728)))+(((IkReal(-1.00000000000000))*(x719)*(x729)*(x733)))+(((x724)*(x734)))+(((x724)*(x732)))+(((x725)*(x729)))+(((x724)*(x725)))+(((IkReal(-1.00000000000000))*(cj1)*(x720)*(x721)*(x722))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(cj1)*(x730)*(x733)))+(((IkReal(-0.212500000000000))*(x726)))+(((IkReal(0.212500000000000))*(x730)))+(((IkReal(-1.00000000000000))*(x717)*(x722)*(x728)))+(((x721)*(x722)*(x728)))+(((IkReal(-1.00000000000000))*(x720)*(x722)*(x723)))+(((x718)*(x722)*(x723)))+(((cj1)*(x726)*(x733)))+(((pp)*(x730)))+(((IkReal(-1.00000000000000))*(x726)*(x727)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x719)*(x724)*(x733)))+(((IkReal(-1.00000000000000))*(x727)*(x731)))+(((IkReal(-0.212500000000000))*(x731)))+(((IkReal(-1.00000000000000))*(cj1)*(x717)*(x718)*(x722)))+(((cj1)*(x722)*((pz)*(pz))))+(((x729)*(x734)))+(((x729)*(x732)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x723)*(x728)))+(((IkReal(-1.00000000000000))*(x719)*(x729)*(x733)))+(((x724)*(x734)))+(((x724)*(x732)))+(((x725)*(x729)))+(((x724)*(x725)))+(((IkReal(-1.00000000000000))*(cj1)*(x720)*(x721)*(x722)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x735=IKsin(j2);
IkReal x736=IKcos(j2);
IkReal x737=((px)*(sj0));
IkReal x738=((IkReal(1.10000000000000))*(sj1));
IkReal x739=((py)*(sj0));
IkReal x740=((pz)*(sj1));
IkReal x741=((cj0)*(px));
IkReal x742=((IkReal(0.0900000000000000))*(x736));
IkReal x743=((cj1)*(x735));
IkReal x744=((IkReal(1.00000000000000))*(x736));
IkReal x745=((cj0)*(py)*(x735));
evalcond[0]=((((x736)*(x737)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x744)))+(((x739)*(x743)))+(((x741)*(x743)))+(((IkReal(-1.00000000000000))*(x735)*(x740))));
evalcond[1]=((((x735)*(x737)))+(((IkReal(-1.00000000000000))*(cj1)*(x739)*(x744)))+(((IkReal(-1.00000000000000))*(x745)))+(((IkReal(-1.00000000000000))*(cj1)*(x741)*(x744)))+(((x736)*(x740))));
evalcond[2]=((IkReal(-0.212500000000000))+(((x738)*(x741)))+(((IkReal(-0.0900000000000000))*(x735)*(x737)))+(((cj1)*(x741)*(x742)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(x745)))+(((cj1)*(x739)*(x742)))+(((IkReal(-1.00000000000000))*(x740)*(x742)))+(((x738)*(x739))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x746=((cj0)*(px));
IkReal x747=((py)*(sj0));
IkReal x748=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.722500000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.850000000000000))+(((cj1)*(pz)))+(((sj1)*(x747)))+(((sj1)*(x746))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x746)*(x748)))+(((IkReal(-1.00000000000000))*(x747)*(x748)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x749=(cj1)*(cj1);
IkReal x750=(px)*(px);
IkReal x751=(sj0)*(sj0);
IkReal x752=(cj0)*(cj0);
IkReal x753=(py)*(py);
IkReal x754=((py)*(sj0));
IkReal x755=((IkReal(0.0900000000000000))*(x751));
IkReal x756=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x757=((cj1)*(pz)*(sj1));
IkReal x758=((IkReal(0.0900000000000000))*(x752));
gconst2=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x749)*(x754)*(x756)))+(((IkReal(-1.00000000000000))*(x754)*(x756)))+(((x749)*(x750)*(x758)))+(((x749)*(x753)*(x755)))+(((IkReal(-1.00000000000000))*(x756)*(x757)))+(((IkReal(-0.180000000000000))*(x754)*(x757)))+(((x753)*(x758)))+(((x750)*(x755)))));
IkReal x759=(sj0)*(sj0);
IkReal x760=(py)*(py);
IkReal x761=(cj1)*(cj1);
IkReal x762=(cj0)*(cj0);
IkReal x763=(px)*(px);
IkReal x764=((cj0)*(px));
IkReal x765=((py)*(sj0));
IkReal x766=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((x761)*(x762)*(x763)))+(((IkReal(2.00000000000000))*(x761)*(x764)*(x765)))+(((IkReal(-1.00000000000000))*(x765)*(x766)))+(((x760)*(x762)))+(((x759)*(x763)))+(((IkReal(-1.00000000000000))*(x764)*(x766)))+(((IkReal(-2.00000000000000))*(x764)*(x765)))+(((x759)*(x760)*(x761)))+((((pz)*(pz))*((sj1)*(sj1)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x767=(sj1)*(sj1);
IkReal x768=(py)*(py);
IkReal x769=(sj0)*(sj0);
IkReal x770=(cj1)*(cj1);
IkReal x771=(cj0)*(cj0);
IkReal x772=(px)*(px);
IkReal x773=((IkReal(1.10000000000000))*(sj1));
IkReal x774=((px)*(py));
IkReal x775=((py)*(sj0));
IkReal x776=((cj1)*(pp));
IkReal x777=((px)*(sj0));
IkReal x778=((IkReal(1.00000000000000))*(pp));
IkReal x779=((cj0)*(sj0));
IkReal x780=((cj0)*(px));
IkReal x781=((cj0)*(py));
IkReal x782=((pz)*(sj1));
IkReal x783=((IkReal(0.212500000000000))*(cj1));
IkReal x784=((IkReal(1.10000000000000))*(pz));
IkReal x785=((x767)*(x784));
if( IKabs(((gconst2)*(((((pp)*(x781)))+(((x772)*(x773)*(x779)))+(((x769)*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(cj1)*(x781)*(x784)))+(((IkReal(0.212500000000000))*(x781)))+(((cj1)*(x777)*(x784)))+(((IkReal(-1.00000000000000))*(x768)*(x773)*(x779)))+(((IkReal(-1.00000000000000))*(x777)*(x778)))+(((IkReal(-0.212500000000000))*(x777)))+(((IkReal(-1.00000000000000))*(x771)*(x773)*(x774))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x770)*(x775)*(x784)))+(((x775)*(x785)))+(((x775)*(x783)))+(((IkReal(-1.00000000000000))*(cj1)*(x768)*(x769)*(x773)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x774)*(x779)))+(((IkReal(-0.212500000000000))*(x782)))+(((cj1)*(x773)*((pz)*(pz))))+(((x780)*(x785)))+(((x780)*(x783)))+(((IkReal(-1.00000000000000))*(cj1)*(x771)*(x772)*(x773)))+(((IkReal(-1.00000000000000))*(x770)*(x780)*(x784)))+(((IkReal(-1.00000000000000))*(x778)*(x782)))+(((x776)*(x780)))+(((x775)*(x776))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((pp)*(x781)))+(((x772)*(x773)*(x779)))+(((x769)*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(cj1)*(x781)*(x784)))+(((IkReal(0.212500000000000))*(x781)))+(((cj1)*(x777)*(x784)))+(((IkReal(-1.00000000000000))*(x768)*(x773)*(x779)))+(((IkReal(-1.00000000000000))*(x777)*(x778)))+(((IkReal(-0.212500000000000))*(x777)))+(((IkReal(-1.00000000000000))*(x771)*(x773)*(x774)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x770)*(x775)*(x784)))+(((x775)*(x785)))+(((x775)*(x783)))+(((IkReal(-1.00000000000000))*(cj1)*(x768)*(x769)*(x773)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x774)*(x779)))+(((IkReal(-0.212500000000000))*(x782)))+(((cj1)*(x773)*((pz)*(pz))))+(((x780)*(x785)))+(((x780)*(x783)))+(((IkReal(-1.00000000000000))*(cj1)*(x771)*(x772)*(x773)))+(((IkReal(-1.00000000000000))*(x770)*(x780)*(x784)))+(((IkReal(-1.00000000000000))*(x778)*(x782)))+(((x776)*(x780)))+(((x775)*(x776)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x786=IKsin(j2);
IkReal x787=IKcos(j2);
IkReal x788=((px)*(sj0));
IkReal x789=((IkReal(1.10000000000000))*(sj1));
IkReal x790=((py)*(sj0));
IkReal x791=((pz)*(sj1));
IkReal x792=((cj0)*(px));
IkReal x793=((IkReal(0.0900000000000000))*(x787));
IkReal x794=((cj1)*(x786));
IkReal x795=((IkReal(1.00000000000000))*(x787));
IkReal x796=((cj0)*(py)*(x786));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(x795)))+(((x790)*(x794)))+(((x787)*(x788)))+(((x792)*(x794)))+(((IkReal(-1.00000000000000))*(x786)*(x791))));
evalcond[1]=((((x786)*(x788)))+(((x787)*(x791)))+(((IkReal(-1.00000000000000))*(cj1)*(x790)*(x795)))+(((IkReal(-1.00000000000000))*(cj1)*(x792)*(x795)))+(((IkReal(-1.00000000000000))*(x796))));
evalcond[2]=((IkReal(-0.212500000000000))+(((IkReal(0.0900000000000000))*(x796)))+(((cj1)*(x790)*(x793)))+(((IkReal(-0.0900000000000000))*(x786)*(x788)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(x791)*(x793)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x792)*(x793)))+(((x789)*(x790)))+(((x789)*(x792))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.297779895218994))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0625000002460000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.250000000150000))+(((cj1)*(pz)))+(((cj0)*(px)*(sj1)))+(((py)*(sj0)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x797=((IkReal(1111111111.11111))*(px));
IkReal x798=((IkReal(1111111111.11111))*(py));
if( IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x797)))+(((cj0)*(x798))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(sj0)*(x798)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj0)*(cj1)*(x797))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x797)))+(((cj0)*(x798)))))+IKsqr(((((cj1)*(sj0)*(x798)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj0)*(cj1)*(x797)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj0)*(x797)))+(((cj0)*(x798)))), ((((cj1)*(sj0)*(x798)))+(((IkReal(-1111111111.11111))*(pz)*(sj1)))+(((cj0)*(cj1)*(x797)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x799=IKcos(j2);
IkReal x800=IKsin(j2);
IkReal x801=((px)*(sj0));
IkReal x802=((IkReal(1.10000000000000))*(sj1));
IkReal x803=((pz)*(sj1));
IkReal x804=((cj0)*(px));
IkReal x805=((IkReal(1.00000000000000))*(cj1));
IkReal x806=((cj1)*(py)*(sj0));
IkReal x807=((IkReal(0.0900000000000000))*(x799));
IkReal x808=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x809=((IkReal(0.0900000000000000))*(x800));
evalcond[0]=((x801)+(((IkReal(-1.00000000000000))*(x808)))+(((IkReal(9.00000000000000e-10))*(x800))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(sj0)*(x805)))+(((IkReal(-1.00000000000000))*(x804)*(x805)))+(((IkReal(9.00000000000000e-10))*(x799)))+(x803));
evalcond[2]=((((IkReal(-1.00000000000000))*(x799)*(x808)))+(((x800)*(x806)))+(((cj1)*(x800)*(x804)))+(((IkReal(-1.00000000000000))*(x800)*(x803)))+(((x799)*(x801))));
evalcond[3]=((IkReal(9.00000000000000e-10))+(((x800)*(x801)))+(((IkReal(-1.00000000000000))*(x800)*(x808)))+(((x799)*(x803)))+(((IkReal(-1.00000000000000))*(x799)*(x804)*(x805)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x799)*(x805))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x806)*(x807)))+(((py)*(sj0)*(x802)))+(((cj0)*(py)*(x809)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x803)*(x807)))+(((IkReal(-1.00000000000000))*(x801)*(x809)))+(((x802)*(x804)))+(((cj1)*(x804)*(x807))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x810=IKsin(j2);
IkReal x811=IKcos(j2);
IkReal x812=((IkReal(0.0450000000000000))*(cj3));
IkReal x813=((px)*(sj0));
IkReal x814=((py)*(sj0));
IkReal x815=((IkReal(1.00000000000000))*(cj0));
IkReal x816=((IkReal(1.10000000000000))*(sj1));
IkReal x817=((IkReal(0.300000000000000))*(sj3));
IkReal x818=((pz)*(sj1));
IkReal x819=((IkReal(0.0900000000000000))*(cj0));
IkReal x820=((cj0)*(px));
IkReal x821=((cj1)*(x811));
IkReal x822=((py)*(x810));
IkReal x823=((cj1)*(x810));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x815)))+(((x810)*(x817)))+(((IkReal(-1.00000000000000))*(x810)*(x812)))+(x813)+(((IkReal(0.0450000000000000))*(x810))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x814)))+(((IkReal(-1.00000000000000))*(x811)*(x812)))+(((x811)*(x817)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x815)))+(x818)+(((IkReal(0.0450000000000000))*(x811))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x810)*(x818)))+(((x820)*(x823)))+(((x814)*(x823)))+(((IkReal(-1.00000000000000))*(py)*(x811)*(x815)))+(((x811)*(x813))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x814)*(x821)))+(((x810)*(x813)))+(((IkReal(-1.00000000000000))*(x815)*(x822)))+(((IkReal(-1.00000000000000))*(x812)))+(((x811)*(x818)))+(x817)+(((IkReal(-1.00000000000000))*(px)*(x815)*(x821))));
evalcond[4]=((IkReal(-0.212500000000000))+(((px)*(x819)*(x821)))+(((IkReal(-0.0900000000000000))*(x811)*(x818)))+(((IkReal(-0.0900000000000000))*(x810)*(x813)))+(((x819)*(x822)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((x814)*(x816)))+(((x816)*(x820)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(x814)*(x821))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x153=((IkReal(1.00000000000000))*(cj3));
IkReal x154=((sj0)*(sj2));
IkReal x155=((cj2)*(sj1));
IkReal x156=((IkReal(1.00000000000000))*(sj3));
IkReal x157=((cj1)*(cj2));
IkReal x158=((sj1)*(sj2));
IkReal x159=((cj0)*(sj2));
IkReal x160=((IkReal(1.00000000000000))*(cj1));
IkReal x161=((((cj3)*(x157)))+(((IkReal(-1.00000000000000))*(sj1)*(x156))));
IkReal x162=((((IkReal(-1.00000000000000))*(x154)*(x160)))+(((cj0)*(cj2))));
IkReal x163=((((sj3)*(x157)))+(((cj3)*(sj1))));
IkReal x164=((((cj1)*(cj3)))+(((IkReal(-1.00000000000000))*(x155)*(x156))));
IkReal x165=((cj0)*(x161));
IkReal x166=((((IkReal(-1.00000000000000))*(x153)*(x155)))+(((IkReal(-1.00000000000000))*(cj1)*(x156))));
IkReal x167=((((IkReal(-1.00000000000000))*(x159)*(x160)))+(((IkReal(-1.00000000000000))*(cj2)*(sj0))));
IkReal x168=((((sj0)*(x161)))+(((cj3)*(x159))));
IkReal x169=((((cj0)*(x163)))+(((IkReal(-1.00000000000000))*(x154)*(x156))));
IkReal x170=((((sj0)*(x163)))+(((sj3)*(x159))));
IkReal x171=((x165)+(((IkReal(-1.00000000000000))*(cj3)*(x154))));
new_r00=((((r20)*(x166)))+(((r10)*(x168)))+(((r00)*(((((IkReal(-1.00000000000000))*(x153)*(x154)))+(x165))))));
new_r01=((((r01)*(x171)))+(((r21)*(x166)))+(((r11)*(x168))));
new_r02=((((r22)*(x166)))+(((r12)*(x168)))+(((r02)*(x171))));
new_r10=((((r00)*(x167)))+(((r20)*(x158)))+(((r10)*(x162))));
new_r11=((((r01)*(x167)))+(((r21)*(x158)))+(((r11)*(x162))));
new_r12=((((r12)*(x162)))+(((r22)*(x158)))+(((r02)*(x167))));
new_r20=((((r00)*(x169)))+(((r20)*(x164)))+(((r10)*(x170))));
new_r21=((((r01)*(x169)))+(((r21)*(x164)))+(((r11)*(x170))));
new_r22=((((r22)*(x164)))+(((r12)*(x170)))+(((r02)*(x169))));
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
cj5array[0]=new_r22;
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst114;
gconst114=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst113;
gconst113=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x172=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x172));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x172))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
evalcond[0]=((((new_r12)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj4)*(new_r00)))+(((new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))))+IKsqr(((((cj4)*(new_r00)))+(((new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))), ((((cj4)*(new_r00)))+(((new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x173=IKsin(j6);
IkReal x174=((IkReal(1.00000000000000))*(sj4));
IkReal x175=((IkReal(1.00000000000000))*(IKcos(j6)));
evalcond[0]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x173)))+(((IkReal(-1.00000000000000))*(new_r00)*(x174))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x175)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x174))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(x173));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x175))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x176=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x176;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x176;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x177=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x177));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x177))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[1];
evalcond[0]=((((new_r12)*(IKcos(j4))))+(((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j4)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((cj4)*(new_r01)))+(((new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))), ((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x178=IKcos(j6);
IkReal x179=((IkReal(1.00000000000000))*(sj4));
IkReal x180=((IkReal(1.00000000000000))*(IKsin(j6)));
evalcond[0]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x180)))+(((IkReal(-1.00000000000000))*(new_r00)*(x179))));
evalcond[1]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x178)))+(((IkReal(-1.00000000000000))*(new_r01)*(x179))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(((IkReal(-1.00000000000000))*(x180))));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(x178));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x181=((gconst113)*(sj5));
if( IKabs(((new_r12)*(x181))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x181))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((new_r12)*(x181)), ((new_r02)*(x181)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x182=IKcos(j4);
IkReal x183=IKsin(j4);
IkReal x184=((IkReal(1.00000000000000))*(sj5));
IkReal x185=((IkReal(1.00000000000000))*(cj5));
IkReal x186=((new_r12)*(x183));
IkReal x187=((new_r02)*(x182));
evalcond[0]=((((new_r12)*(x182)))+(((IkReal(-1.00000000000000))*(new_r02)*(x183))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x184)))+(x186)+(x187));
evalcond[2]=((((cj5)*(x186)))+(((cj5)*(x187)))+(((IkReal(-1.00000000000000))*(new_r22)*(x184))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r10)*(x183)*(x184)))+(((IkReal(-1.00000000000000))*(new_r00)*(x182)*(x184)))+(((IkReal(-1.00000000000000))*(new_r20)*(x185))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r11)*(x183)*(x184)))+(((IkReal(-1.00000000000000))*(new_r21)*(x185)))+(((IkReal(-1.00000000000000))*(new_r01)*(x182)*(x184))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x185)))+(((IkReal(-1.00000000000000))*(x184)*(x187)))+(((IkReal(-1.00000000000000))*(x184)*(x186))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst115;
gconst115=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x188=((((IkReal(-1.00000000000000))*(new_r02)*(sj4)))+(((cj4)*(new_r12))));
IkReal x189=((((new_r12)*(sj4)))+(((cj4)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x188;
evalcond[5]=x188;
evalcond[6]=x189;
evalcond[7]=x189;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj4)*(new_r00)))+(((new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))))+IKsqr(((((cj4)*(new_r00)))+(((new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj4)*(new_r01)))+(((IkReal(-1.00000000000000))*(new_r11)*(sj4)))), ((((cj4)*(new_r00)))+(((new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x190=IKsin(j6);
IkReal x191=((IkReal(1.00000000000000))*(sj4));
IkReal x192=((IkReal(1.00000000000000))*(IKcos(j6)));
evalcond[0]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x190)))+(((IkReal(-1.00000000000000))*(new_r00)*(x191))));
evalcond[1]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x192)))+(((IkReal(-1.00000000000000))*(new_r01)*(x191))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(x190));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x192))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x193=((IkReal(1.00000000000000))+(new_r22));
IkReal x194=((cj4)*(new_r02));
IkReal x195=((new_r12)*(sj4));
IkReal x196=((((IkReal(-1.00000000000000))*(new_r02)*(sj4)))+(((cj4)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x193;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x196;
evalcond[5]=x196;
evalcond[6]=((x195)+(x194));
evalcond[7]=((((IkReal(-1.00000000000000))*(x195)))+(((IkReal(-1.00000000000000))*(x194))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x193;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((((cj4)*(new_r01)))+(((new_r11)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((cj4)*(new_r01)))+(((new_r11)*(sj4)))), ((((IkReal(-1.00000000000000))*(cj4)*(new_r00)))+(((IkReal(-1.00000000000000))*(new_r10)*(sj4)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[4];
IkReal x197=IKcos(j6);
IkReal x198=((IkReal(1.00000000000000))*(sj4));
IkReal x199=((IkReal(1.00000000000000))*(IKsin(j6)));
evalcond[0]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x199)))+(((IkReal(-1.00000000000000))*(new_r00)*(x198))));
evalcond[1]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x197)))+(((IkReal(-1.00000000000000))*(new_r01)*(x198))));
evalcond[2]=((((cj4)*(new_r01)))+(((new_r11)*(sj4)))+(((IkReal(-1.00000000000000))*(x199))));
evalcond[3]=((((cj4)*(new_r00)))+(((new_r10)*(sj4)))+(x197));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst115)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst115)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst115)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst115)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x200=IKsin(j6);
IkReal x201=IKcos(j6);
IkReal x202=((cj4)*(new_r01));
IkReal x203=((IkReal(1.00000000000000))*(sj5));
IkReal x204=((new_r10)*(sj4));
IkReal x205=((new_r11)*(sj4));
IkReal x206=((cj4)*(new_r00));
IkReal x207=((IkReal(1.00000000000000))*(sj4));
IkReal x208=((IkReal(1.00000000000000))*(x201));
evalcond[0]=((new_r20)+(((sj5)*(x201))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x200)*(x203)))+(new_r21));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x207)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x200))));
evalcond[3]=((((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x208)))+(((IkReal(-1.00000000000000))*(new_r01)*(x207))));
evalcond[4]=((x205)+(x202)+(((cj5)*(x200))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj5)*(x208)))+(x206)+(x204));
evalcond[6]=((((IkReal(-1.00000000000000))*(new_r21)*(x203)))+(x200)+(((cj5)*(x202)))+(((cj5)*(x205))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x203)))+(((IkReal(-1.00000000000000))*(x208)))+(((cj5)*(x204)))+(((cj5)*(x206))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst114)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst114)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst114)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst114)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[2];
evalcond[0]=((new_r20)+(((sj5)*(IKcos(j6)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(IKsin(j6))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst116;
gconst116=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x209=((gconst116)*(sj5));
if( IKabs(((new_r12)*(x209))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x209))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((new_r12)*(x209)), ((new_r02)*(x209)));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[12];
IkReal x210=IKcos(j4);
IkReal x211=IKsin(j4);
IkReal x212=((IkReal(1.00000000000000))*(cj6));
IkReal x213=((IkReal(1.00000000000000))*(sj5));
IkReal x214=((IkReal(1.00000000000000))*(cj5));
IkReal x215=((new_r00)*(x210));
IkReal x216=((cj5)*(x211));
IkReal x217=((new_r11)*(x211));
IkReal x218=((cj5)*(x210));
IkReal x219=((new_r02)*(x210));
IkReal x220=((new_r12)*(x211));
IkReal x221=((new_r10)*(x211));
IkReal x222=((IkReal(1.00000000000000))*(x211));
IkReal x223=((new_r01)*(x210));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(x222)))+(((new_r12)*(x210))));
evalcond[1]=((x219)+(x220)+(((IkReal(-1.00000000000000))*(x213))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj6)))+(((new_r10)*(x210)))+(((IkReal(-1.00000000000000))*(new_r00)*(x222))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x222)))+(((IkReal(-1.00000000000000))*(x212)))+(((new_r11)*(x210))));
evalcond[4]=((((cj5)*(sj6)))+(x217)+(x223));
evalcond[5]=((x215)+(x221)+(((IkReal(-1.00000000000000))*(cj5)*(x212))));
evalcond[6]=((((new_r02)*(x218)))+(((IkReal(-1.00000000000000))*(new_r22)*(x213)))+(((new_r12)*(x216))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x213)*(x221)))+(((IkReal(-1.00000000000000))*(new_r20)*(x214)))+(((IkReal(-1.00000000000000))*(x213)*(x215))));
evalcond[8]=((((IkReal(-1.00000000000000))*(x213)*(x223)))+(((IkReal(-1.00000000000000))*(new_r21)*(x214)))+(((IkReal(-1.00000000000000))*(x213)*(x217))));
evalcond[9]=((sj6)+(((IkReal(-1.00000000000000))*(new_r21)*(x213)))+(((new_r01)*(x218)))+(((new_r11)*(x216))));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x213)*(x220)))+(((IkReal(-1.00000000000000))*(new_r22)*(x214)))+(((IkReal(-1.00000000000000))*(x213)*(x219))));
evalcond[11]=((((cj5)*(x215)))+(((IkReal(-1.00000000000000))*(new_r20)*(x213)))+(((IkReal(-1.00000000000000))*(x212)))+(((new_r10)*(x216))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j6;
vinfos[6].indices[0] = _ij6[0];
vinfos[6].indices[1] = _ij6[1];
vinfos[6].maxsolutions = _nj6;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3238550ee83defc84c543b45c2db9325"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
