/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:51:06.369733
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[2]);
x3=IKsin(j[3]);
x4=IKcos(j[3]);
x5=IKsin(j[1]);
x6=IKsin(j[5]);
x7=IKsin(j[4]);
x8=IKcos(j[4]);
x9=IKcos(j[5]);
x10=((IkReal(0.707106781186548))*(IKcos(j[0])));
x11=((IkReal(0.707106781186548))*(IKsin(j[0])));
x12=((IkReal(0.707106781186548))*(x6));
x13=((IkReal(0.707106781186548))*(x9));
x14=((IkReal(0.500001138018525))*(x9));
x15=((IkReal(0.499998861978885))*(x9));
x16=((IkReal(0.499998861978885))*(x6));
x17=((IkReal(1.00000000000000))*(x1));
x18=((IkReal(0.500001138018525))*(x6));
x19=((IkReal(1.00000000000000))*(x3));
x20=((x0)*(x1));
x21=((x0)*(x2));
x22=((x4)*(x5));
x23=((IkReal(-1.00000000000000))*(x2));
x24=((x3)*(x5));
x25=((IkReal(-1.00000000000000))*(x3));
x26=((((IkReal(-1.00000000000000))*(x10)))+(x11));
x27=((((IkReal(-1.00000000000000))*(x11)))+(x10));
x28=((((IkReal(-1.00000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(x10))));
x29=((x11)+(x10));
x30=((x26)*(x5));
x31=((IkReal(-1.00000000000000))*(x27));
x32=((x2)*(x27));
x33=((((IkReal(1.00000000000000))*(x0)*(x4)))+(((IkReal(-1.00000000000000))*(x17)*(x24))));
x34=((x33)*(x9));
x35=((((x8)*(((((IkReal(-1.00000000000000))*(x0)*(x19)))+(((IkReal(-1.00000000000000))*(x17)*(x22)))))))+(((x2)*(x5)*(x7))));
x36=((x35)*(x6));
x37=((((IkReal(-1.00000000000000))*(x22)*(x28)))+(((x19)*(((((x0)*(x17)*(x29)))+(((IkReal(1.00000000000000))*(x32))))))));
x38=((((x25)*(((((x20)*(x31)))+(((x23)*(x28)))))))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x39=((x37)*(x9));
x40=((((x8)*(((((x4)*(((((x2)*(x28)))+(((x20)*(x27)))))))+(((x24)*(x26)))))))+(((x7)*(((((x21)*(x26)))+(((x1)*(x28))))))));
x41=((((x8)*(((((x4)*(((x32)+(((x20)*(x29)))))))+(((x24)*(x28)))))))+(((x7)*(((((x21)*(x28)))+(((x1)*(x27))))))));
eerot[0]=((((((((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))+(((IkReal(0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))))*(IKcos(j[6])))));
eerot[1]=((((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707108390587779))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6])))));
eerot[2]=((((IkReal(-0.707108390587779))*(x36)))+(((IkReal(-0.707108390587779))*(x34)))+(((x16)*(x40)))+(((x15)*(x38)))+(((IkReal(-1.00000000000000))*(x16)*(x41)))+(((IkReal(-1.00000000000000))*(x15)*(x37))));
eetrans[0]=((IkReal(-0.350177000000000))+(((IkReal(0.212132517176334))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKcos(j[0]))))+(((IkReal(-0.106065775767248))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318198775764501))*(IKcos(j[2]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKsin(j[0]))))+(((IkReal(0.106065775767248))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(-0.388909614823279))*(IKcos(j[1]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((IkReal(-0.212132517176334))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(-0.125829938105095))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829938105095))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.125829938105095))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707108390587779))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1])))));
eerot[3]=((((((((IkReal(0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6])))));
eerot[4]=((((((((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6])))));
eerot[5]=((((IkReal(-0.707105171781653))*(x34)))+(((IkReal(-0.707105171781653))*(x36)))+(((IkReal(-1.00000000000000))*(x18)*(x40)))+(((x18)*(x41)))+(((IkReal(-1.00000000000000))*(x14)*(x38)))+(((x14)*(((((x25)*(((((x23)*(x27)))+(((IkReal(-1.00000000000000))*(x20)*(x29)))))))+(((IkReal(-1.00000000000000))*(x22)*(x28))))))));
eetrans[1]=((IkReal(-0.350177000000000))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318197327301744))*(IKcos(j[2]))*(IKsin(j[1]))))+(((IkReal(0.212131551534496))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829365318545))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.212131551534496))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKcos(j[0]))))+(((IkReal(-0.0159099387882250))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.388907844479909))*(IKcos(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(-0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3])))));
eerot[6]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6])))));
eerot[7]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6])))));
eerot[8]=((((x12)*(x40)))+(((x12)*(x41)))+(((x13)*(x37)))+(((x13)*(x38))));
eetrans[2]=((IkReal(0.378361000000000))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.275000000000000))*(IKcos(j[0]))))+(((IkReal(0.275000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.150000000000000))*(IKcos(j[0]))))+(((IkReal(0.150000000000000))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.275000000000000))*(IKsin(j[0]))))+(((IkReal(0.275000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.150000000000000))*(IKsin(j[0]))))+(((IkReal(0.150000000000000))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1])))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {5}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j6,cj6,sj6,htj6,j7,cj7,sj7,htj7,j8,cj8,sj8,htj8,j10,cj10,sj10,htj10,j9,cj9,sj9,htj9,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij4[2], _nj4,_ij5[2], _nj5,_ij6[2], _nj6,_ij7[2], _nj7,_ij8[2], _nj8,_ij10[2], _nj10,_ij9[2], _nj9;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j6=numeric_limits<IkReal>::quiet_NaN(); _ij6[0] = -1; _ij6[1] = -1; _nj6 = -1; j7=numeric_limits<IkReal>::quiet_NaN(); _ij7[0] = -1; _ij7[1] = -1; _nj7 = -1; j8=numeric_limits<IkReal>::quiet_NaN(); _ij8[0] = -1; _ij8[1] = -1; _nj8 = -1; j10=numeric_limits<IkReal>::quiet_NaN(); _ij10[0] = -1; _ij10[1] = -1; _nj10 = -1;  _ij9[0] = -1; _ij9[1] = -1; _nj9 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j9=pfree[0]; cj9=cos(pfree[0]); sj9=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=((((IkReal(-0.500001138018525))*(r11)))+(((IkReal(0.499998861978885))*(r01)))+(((IkReal(0.707106781186548))*(r21))));
new_r01=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(0.500001138018525))*(r10)))+(((IkReal(-0.499998861978885))*(r00))));
new_r02=((((IkReal(-0.500001138018525))*(r12)))+(((IkReal(0.499998861978885))*(r02)))+(((IkReal(0.707106781186548))*(r22))));
new_px=((IkReal(-0.267542425853256))+(((IkReal(0.499998861978885))*(px)))+(((IkReal(-0.500001138018525))*(py)))+(((IkReal(-0.0889747974891426))*(r02)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.0889752025103964))*(r12))));
new_r10=((((IkReal(0.500001138018525))*(r11)))+(((IkReal(0.707106781186548))*(r21)))+(((IkReal(-0.499998861978885))*(r01))));
new_r11=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(-0.500001138018525))*(r10)))+(((IkReal(0.499998861978885))*(r00))));
new_r12=((((IkReal(0.500001138018525))*(r12)))+(((IkReal(0.707106781186548))*(r22)))+(((IkReal(-0.499998861978885))*(r02))));
new_py=((IkReal(-0.267540831819791))+(((IkReal(-0.499998861978885))*(px)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.500001138018525))*(py)))+(((IkReal(0.0889747974891426))*(r02)))+(((IkReal(-0.0889752025103964))*(r12))));
new_r20=((((IkReal(-0.707108390587779))*(r01)))+(((IkReal(-0.707105171781653))*(r11))));
new_r21=((((IkReal(0.707108390587779))*(r00)))+(((IkReal(0.707105171781653))*(r10))));
new_r22=((((IkReal(-0.707108390587779))*(r02)))+(((IkReal(-0.707105171781653))*(r12))));
new_pz=((IkReal(-0.495225062629841))+(((IkReal(0.125829365318545))*(r12)))+(((IkReal(-0.707105171781653))*(py)))+(((IkReal(0.125829938105095))*(r02)))+(((IkReal(-0.707108390587779))*(px))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j7array[2], cj7array[2], sj7array[2];
bool j7valid[2]={false};
_nj7 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x42=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j7array[0]=((IkReal(1.80132261653346))+(((IkReal(1.00000000000000))*(x42))));
sj7array[0]=IKsin(j7array[0]);
cj7array[0]=IKcos(j7array[0]);
j7array[1]=((IkReal(4.94291527012326))+(((IkReal(-1.00000000000000))*(x42))));
sj7array[1]=IKsin(j7array[1]);
cj7array[1]=IKcos(j7array[1]);
if( j7array[0] > IKPI )
{
    j7array[0]-=IK2PI;
}
else if( j7array[0] < -IKPI )
{    j7array[0]+=IK2PI;
}
j7valid[0] = true;
if( j7array[1] > IKPI )
{
    j7array[1]-=IK2PI;
}
else if( j7array[1] < -IKPI )
{    j7array[1]+=IK2PI;
}
j7valid[1] = true;
for(int ij7 = 0; ij7 < 2; ++ij7)
{
if( !j7valid[ij7] )
{
    continue;
}
_ij7[0] = ij7; _ij7[1] = -1;
for(int iij7 = ij7+1; iij7 < 2; ++iij7)
{
if( j7valid[iij7] && IKabs(cj7array[ij7]-cj7array[iij7]) < IKFAST_SOLUTION_THRESH && IKabs(sj7array[ij7]-sj7array[iij7]) < IKFAST_SOLUTION_THRESH )
{
    j7valid[iij7]=false; _ij7[1] = iij7; break; 
}
}
j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];

{
IkReal dummyeval[1];
IkReal x43=(sj9)*(sj9);
dummyeval[0]=((((x43)*((npy)*(npy))))+(((x43)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x44=(sj9)*(sj9);
dummyeval[0]=((((x44)*((npy)*(npy))))+(((x44)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(sj9)))+(((IkReal(-12.2222222222222))*(sj7)*(sj9)))+(((cj7)*(sj9))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x45=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj7))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x45;
evalcond[3]=x45;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x46=((IkReal(1.00000000000000))*(npy));
IkReal x47=((IkReal(24.4444444444444))*(npy)*(sj7));
dummyeval[0]=((((IkReal(-149.382716049383))*(npy)*((sj7)*(sj7))))+(((IkReal(2.00000000000000))*(cj7)*(npy)))+(((IkReal(-1.00000000000000))*(x47)))+(((IkReal(-1.00000000000000))*(x46)))+(((cj7)*(x47)))+(((IkReal(493.827160493827))*(npy)*((npx)*(npx))))+(((IkReal(-1.00000000000000))*(x46)*((cj7)*(cj7))))+(((IkReal(493.827160493827))*((npy)*(npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x48=(cj7)*(cj7);
IkReal x49=(sj7)*(sj7);
IkReal x50=(npy)*(npy);
IkReal x51=(npx)*(npx)*(npx);
IkReal x52=((IkReal(0.00810000000000000))*(npy));
IkReal x53=((cj7)*(npy));
IkReal x54=((IkReal(0.198000000000000))*(sj7));
IkReal x55=((npx)*(sj7));
IkReal x56=((IkReal(0.0324000000000000))*(npx));
IkReal x57=((IkReal(0.0162000000000000))*(npx));
IkReal x58=((npx)*(x50));
IkReal x59=((npx)*(x49));
IkReal x60=((((npy)*(x54)))+(x52)+(((x48)*(x52)))+(((IkReal(1.21000000000000))*(npy)*(x49))));
IkReal x61=((((IkReal(4.00000000000000))*(npy)*((npx)*(npx))))+(((x53)*(x54)))+(((IkReal(4.00000000000000))*((npy)*(npy)*(npy))))+(((IkReal(0.0162000000000000))*(x53))));
IkReal x62=((x61)+(((IkReal(-1.00000000000000))*(x60))));
IkReal x63=((((IkReal(-8.00000000000000))*(x51)))+(((IkReal(-8.00000000000000))*(x58)))+(((IkReal(2.42000000000000))*(x59)))+(((IkReal(0.396000000000000))*(x55)))+(((IkReal(-0.396000000000000))*(cj7)*(x55)))+(((IkReal(-1.00000000000000))*(cj7)*(x56)))+(x57)+(((x48)*(x57))));
IkReal x64=((x60)+(((IkReal(-1.00000000000000))*(x61))));
op[0]=x62;
op[1]=x63;
op[2]=x62;
op[3]=((((IkReal(-16.0000000000000))*(x51)))+(((IkReal(-16.0000000000000))*(x58)))+(((IkReal(4.84000000000000))*(x59)))+(((IkReal(-0.0648000000000000))*(cj7)*(npx)))+(((IkReal(0.792000000000000))*(x55)))+(x56)+(((x48)*(x56)))+(((IkReal(-0.792000000000000))*(cj7)*(x55))));
op[4]=x64;
op[5]=x63;
op[6]=x64;
polyroots6(op,zeror,numroots);
IkReal j10array[6], cj10array[6], sj10array[6], tempj10array[1];
int numsolutions = 0;
for(int ij10 = 0; ij10 < numroots; ++ij10)
{
IkReal htj10 = zeror[ij10];
tempj10array[0]=((IkReal(2.00000000000000))*(atan(htj10)));
for(int kj10 = 0; kj10 < 1; ++kj10)
{
j10array[numsolutions] = tempj10array[kj10];
if( j10array[numsolutions] > IKPI )
{
    j10array[numsolutions]-=IK2PI;
}
else if( j10array[numsolutions] < -IKPI )
{
    j10array[numsolutions]+=IK2PI;
}
sj10array[numsolutions] = IKsin(j10array[numsolutions]);
cj10array[numsolutions] = IKcos(j10array[numsolutions]);
numsolutions++;
}
}
bool j10valid[6]={true,true,true,true,true,true};
_nj10 = 6;
for(int ij10 = 0; ij10 < numsolutions; ++ij10)
    {
if( !j10valid[ij10] )
{
    continue;
}
    j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
htj10 = IKtan(j10/2);

_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < numsolutions; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x65=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x66=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj10)*(x66)))+(((IkReal(-1.00000000000000))*(cj10)*(npy))));
evalcond[3]=x65;
evalcond[4]=x65;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x66)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x67=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x67));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x67))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x68=IKcos(j8);
IkReal x69=IKsin(j8);
IkReal x70=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x68)*(x70)))+(((IkReal(-1.00000000000000))*(npx)*(x69)*(x70)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x68)))+(((npy)*(sj10)*(x69))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x305=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x306=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x306))));
evalcond[3]=x305;
evalcond[4]=x305;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x306)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x307=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x307));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x307))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x308=IKcos(j8);
IkReal x309=IKsin(j8);
IkReal x310=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((((npy)*(sj10)*(x309)))+(((IkReal(-1.00000000000000))*(npy)*(x308)*(x310)))+(((IkReal(-1.00000000000000))*(npx)*(x309)*(x310)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x308))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x311=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x311;
evalcond[3]=x311;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x312=((IkReal(414405822679.079))*(sj10));
IkReal x313=((IkReal(414405822679.079))*(cj10));
if( IKabs(((((npy)*(x313)))+(((npx)*(x312))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npy)*(x312)))+(((IkReal(-1.00000000000000))*(npx)*(x313))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((npy)*(x313)))+(((npx)*(x312)))))+IKsqr(((((npy)*(x312)))+(((IkReal(-1.00000000000000))*(npx)*(x313)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((npy)*(x313)))+(((npx)*(x312)))), ((((npy)*(x312)))+(((IkReal(-1.00000000000000))*(npx)*(x313)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x314=IKsin(j8);
IkReal x315=IKcos(j8);
IkReal x316=((IkReal(1.00000000000000))*(cj10));
IkReal x317=((npy)*(sj10));
IkReal x318=((npx)*(sj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(x318)))+(((IkReal(-1.00000000000000))*(npy)*(x316)))+(((IkReal(2.41309350707268e-12))*(x314))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npx)*(x314)*(x316)))+(((IkReal(-1.00000000000000))*(npy)*(x315)*(x316)))+(((IkReal(-1.00000000000000))*(x315)*(x318)))+(((x314)*(x317))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(npx)*(x315)*(x316)))+(((x315)*(x317)))+(((cj10)*(npy)*(x314)))+(((x314)*(x318))));
evalcond[3]=((x317)+(((IkReal(-2.41309350707268e-12))*(x315)))+(((IkReal(-1.00000000000000))*(npx)*(x316))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst7)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst7)*(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst7)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst7)*(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x319=IKsin(j8);
IkReal x320=IKcos(j8);
IkReal x321=((IkReal(0.550000000000000))*(sj7));
IkReal x322=((npy)*(sj10));
IkReal x323=((IkReal(1.00000000000000))*(npx));
IkReal x324=((IkReal(0.0450000000000000))*(x320));
IkReal x325=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x326=((cj10)*(x319));
IkReal x327=((IkReal(0.0450000000000000))*(x319));
evalcond[0]=((((IkReal(-1.00000000000000))*(x325)))+(x327)+(((IkReal(-1.00000000000000))*(cj7)*(x327)))+(((IkReal(-1.00000000000000))*(sj10)*(x323)))+(((x319)*(x321))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x323)*(x326)))+(((IkReal(-1.00000000000000))*(x320)*(x325)))+(((IkReal(-1.00000000000000))*(sj10)*(x320)*(x323)))+(((x319)*(x322))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(cj10)*(x320)*(x323)))+(((IkReal(0.0450000000000000))*(cj7)))+(((x320)*(x322)))+(((IkReal(-1.00000000000000))*(x321)))+(((npx)*(sj10)*(x319)))+(((npy)*(x326))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x324)))+(((IkReal(-1.00000000000000))*(x320)*(x321)))+(x322)+(((cj7)*(x324)))+(((IkReal(-1.00000000000000))*(cj10)*(x323))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
IkReal x328=((IkReal(0.0450000000000000))*(sj7));
IkReal x329=((IkReal(0.550000000000000))*(cj7));
IkReal x330=((x328)+(x329));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x330));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x330))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x331=((IkReal(1.00000000000000))*(npy));
IkReal x332=((IkReal(24.4444444444444))*(npy)*(sj7));
dummyeval[0]=((((IkReal(-149.382716049383))*(npy)*((sj7)*(sj7))))+(((IkReal(2.00000000000000))*(cj7)*(npy)))+(((IkReal(-1.00000000000000))*(x331)*((cj7)*(cj7))))+(((cj7)*(x332)))+(((IkReal(493.827160493827))*(npy)*((npx)*(npx))))+(((IkReal(-1.00000000000000))*(x331)))+(((IkReal(-1.00000000000000))*(x332)))+(((IkReal(493.827160493827))*((npy)*(npy)*(npy)))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[6+1], zeror[6];
int numroots;
IkReal x333=(cj7)*(cj7);
IkReal x334=(sj7)*(sj7);
IkReal x335=(npy)*(npy);
IkReal x336=(npx)*(npx)*(npx);
IkReal x337=((IkReal(0.00810000000000000))*(npy));
IkReal x338=((cj7)*(npy));
IkReal x339=((IkReal(0.198000000000000))*(sj7));
IkReal x340=((npx)*(sj7));
IkReal x341=((IkReal(0.0324000000000000))*(npx));
IkReal x342=((IkReal(0.0162000000000000))*(npx));
IkReal x343=((npx)*(x335));
IkReal x344=((npx)*(x334));
IkReal x345=((((IkReal(1.21000000000000))*(npy)*(x334)))+(x337)+(((x333)*(x337)))+(((npy)*(x339))));
IkReal x346=((((x338)*(x339)))+(((IkReal(0.0162000000000000))*(x338)))+(((IkReal(4.00000000000000))*(npy)*((npx)*(npx))))+(((IkReal(4.00000000000000))*((npy)*(npy)*(npy)))));
IkReal x347=((x346)+(((IkReal(-1.00000000000000))*(x345))));
IkReal x348=((((IkReal(-0.396000000000000))*(cj7)*(x340)))+(((IkReal(2.42000000000000))*(x344)))+(x342)+(((x333)*(x342)))+(((IkReal(-8.00000000000000))*(x336)))+(((IkReal(-1.00000000000000))*(cj7)*(x341)))+(((IkReal(-8.00000000000000))*(x343)))+(((IkReal(0.396000000000000))*(x340))));
IkReal x349=((x345)+(((IkReal(-1.00000000000000))*(x346))));
op[0]=x347;
op[1]=x348;
op[2]=x347;
op[3]=((((IkReal(-0.792000000000000))*(cj7)*(x340)))+(((IkReal(-16.0000000000000))*(x336)))+(((IkReal(-16.0000000000000))*(x343)))+(((IkReal(-0.0648000000000000))*(cj7)*(npx)))+(((IkReal(4.84000000000000))*(x344)))+(x341)+(((x333)*(x341)))+(((IkReal(0.792000000000000))*(x340))));
op[4]=x349;
op[5]=x348;
op[6]=x349;
polyroots6(op,zeror,numroots);
IkReal j10array[6], cj10array[6], sj10array[6], tempj10array[1];
int numsolutions = 0;
for(int ij10 = 0; ij10 < numroots; ++ij10)
{
IkReal htj10 = zeror[ij10];
tempj10array[0]=((IkReal(2.00000000000000))*(atan(htj10)));
for(int kj10 = 0; kj10 < 1; ++kj10)
{
j10array[numsolutions] = tempj10array[kj10];
if( j10array[numsolutions] > IKPI )
{
    j10array[numsolutions]-=IK2PI;
}
else if( j10array[numsolutions] < -IKPI )
{
    j10array[numsolutions]+=IK2PI;
}
sj10array[numsolutions] = IKsin(j10array[numsolutions]);
cj10array[numsolutions] = IKcos(j10array[numsolutions]);
numsolutions++;
}
}
bool j10valid[6]={true,true,true,true,true,true};
_nj10 = 6;
for(int ij10 = 0; ij10 < numsolutions; ++ij10)
    {
if( !j10valid[ij10] )
{
    continue;
}
    j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
htj10 = IKtan(j10/2);

_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < numsolutions; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst8;
gconst8=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x350=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x350))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x350)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x351=IKatan2(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x351));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x351))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x352=IKsin(j8);
IkReal x353=IKcos(j8);
IkReal x354=((IkReal(1.00000000000000))*(npy));
evalcond[0]=((((cj10)*(npx)*(x352)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x353)))+(((IkReal(-1.00000000000000))*(sj10)*(x352)*(x354)))+(((IkReal(-1.00000000000000))*(cj10)*(x353)*(x354))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x355=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x355))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x355)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x356=IKatan2(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x356));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x356))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x357=IKsin(j8);
IkReal x358=IKcos(j8);
IkReal x359=((IkReal(1.00000000000000))*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj10)*(x357)*(x359)))+(((cj10)*(npx)*(x357)))+(((IkReal(-1.00000000000000))*(cj10)*(x358)*(x359)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x358))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(npz));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x360=((IkReal(414405822679.079))*(npx));
IkReal x361=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj10)*(x360)))+(((cj10)*(x361))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj10)*(x361)))+(((cj10)*(x360))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj10)*(x360)))+(((cj10)*(x361)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj10)*(x361)))+(((cj10)*(x360)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((sj10)*(x360)))+(((cj10)*(x361)))), ((((IkReal(-1.00000000000000))*(sj10)*(x361)))+(((cj10)*(x360)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x362=IKsin(j8);
IkReal x363=IKcos(j8);
IkReal x364=((IkReal(1.00000000000000))*(cj10));
IkReal x365=((npy)*(sj10));
IkReal x366=((IkReal(1.00000000000000))*(sj10));
IkReal x367=((npy)*(x363));
IkReal x368=((cj10)*(x362));
IkReal x369=((npx)*(x363));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x362)))+(((IkReal(-1.00000000000000))*(npy)*(x364)))+(((IkReal(-1.00000000000000))*(npx)*(x366))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x362)*(x365)))+(((IkReal(-1.00000000000000))*(x366)*(x369)))+(((npx)*(x368)))+(((IkReal(-1.00000000000000))*(x364)*(x367))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((npy)*(x368)))+(((IkReal(-1.00000000000000))*(x363)*(x365)))+(((cj10)*(x369)))+(((npx)*(sj10)*(x362))));
evalcond[3]=((((IkReal(2.41309350707268e-12))*(x363)))+(x365)+(((IkReal(-1.00000000000000))*(npx)*(x364))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst8)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst8)*(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst8)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst8)*(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x370=IKsin(j8);
IkReal x371=IKcos(j8);
IkReal x372=((IkReal(0.550000000000000))*(sj7));
IkReal x373=((cj10)*(npx));
IkReal x374=((npy)*(sj10));
IkReal x375=((IkReal(0.0450000000000000))*(x371));
IkReal x376=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x377=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x378=((IkReal(0.0450000000000000))*(x370));
evalcond[0]=((((x370)*(x372)))+(((IkReal(-1.00000000000000))*(cj7)*(x378)))+(((IkReal(-1.00000000000000))*(x376)))+(((IkReal(-1.00000000000000))*(x377)))+(x378));
evalcond[1]=((((x370)*(x373)))+(((IkReal(-1.00000000000000))*(x371)*(x377)))+(((IkReal(-1.00000000000000))*(x371)*(x376)))+(((IkReal(-1.00000000000000))*(x370)*(x374))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(x372)))+(((IkReal(-1.00000000000000))*(x371)*(x374)))+(((npx)*(sj10)*(x370)))+(((cj10)*(npy)*(x370)))+(((x371)*(x373))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x373)))+(((IkReal(-1.00000000000000))*(cj7)*(x375)))+(x375)+(x374)+(((x371)*(x372))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
    }

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.850000000000000))*(cj9))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst9;
gconst9=IKsign(((((IkReal(20.0000000000000))*((npy)*(npy))))+(((IkReal(20.0000000000000))*((npx)*(npx))))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x379=((gconst9)*(sj9));
if( IKabs(((IkReal(17.0000000000000))*(npy)*(x379))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-17.0000000000000))*(npx)*(x379))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((IkReal(17.0000000000000))*(npy)*(x379)), ((IkReal(-17.0000000000000))*(npx)*(x379)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[3];
IkReal x380=IKcos(j10);
IkReal x381=IKsin(j10);
IkReal x382=((IkReal(1.00000000000000))*(npy));
IkReal x383=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((((IkReal(-1.00000000000000))*(x380)*(x382)))+(((IkReal(-1.00000000000000))*(x381)*(x383))));
evalcond[1]=((IkReal(0.850000000000000))+(((npx)*(sj9)*(x380)))+(((IkReal(-1.00000000000000))*(sj9)*(x381)*(x382)))+(((IkReal(-1.00000000000000))*(cj9)*(npz))));
evalcond[2]=((((IkReal(-0.850000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(x380)*(x383)))+(((npy)*(x381))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x384=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x384));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x384))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x385=IKsin(j8);
IkReal x386=IKcos(j8);
IkReal x387=((IkReal(1.00000000000000))*(x386));
IkReal x388=((IkReal(1.00000000000000))*(x385));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj10)*(x387)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x388)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x387)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x388)))+(((cj9)*(npy)*(sj10)*(x385))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
IkReal x389=((IkReal(0.0450000000000000))*(sj9));
cj8array[0]=((((IKabs(((((IkReal(-0.550000000000000))*(sj7)*(sj9)))+(((cj7)*(x389)))+(((IkReal(-1.00000000000000))*(x389))))) != 0)?((IkReal)1/(((((IkReal(-0.550000000000000))*(sj7)*(sj9)))+(((cj7)*(x389)))+(((IkReal(-1.00000000000000))*(x389)))))):(IkReal)1.0e30))*(((((IkReal(1.00000000000000))*(npz)))+(((IkReal(-0.300000000000000))*(cj9)))+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9))))));
if( cj8array[0] >= -1-IKFAST_SINCOS_THRESH && cj8array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j8valid[0] = j8valid[1] = true;
    j8array[0] = IKacos(cj8array[0]);
    sj8array[0] = IKsin(j8array[0]);
    cj8array[1] = cj8array[0];
    j8array[1] = -j8array[0];
    sj8array[1] = -sj8array[0];
}
else if( isnan(cj8array[0]) )
{
    // probably any value will work
    j8valid[0] = true;
    cj8array[0] = 1; sj8array[0] = 0; j8array[0] = 0;
}
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((((sj9)*((npy)*(npy))))+(((sj9)*((npx)*(npx))))));
dummyeval[0]=((((sj9)*((npy)*(npy))))+(((sj9)*((npx)*(npx)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x390=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj7))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x390;
evalcond[3]=x390;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign((((npx)*(npx))+((npy)*(npy))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x391=((IkReal(0.0450000000000000))*(cj8));
IkReal x392=((IkReal(0.0450000000000000))*(sj8));
IkReal x393=((cj7)*(npy));
IkReal x394=((IkReal(0.550000000000000))*(sj7));
IkReal x395=((cj7)*(npx));
if( IKabs(((gconst1)*(((((cj8)*(npy)*(x394)))+(((npy)*(x391)))+(((IkReal(-1.00000000000000))*(x391)*(x393)))+(((npx)*(sj8)*(x394)))+(((npx)*(x392)))+(((IkReal(-1.00000000000000))*(x392)*(x395))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((npy)*(sj8)*(x394)))+(((npy)*(x392)))+(((x391)*(x395)))+(((IkReal(-1.00000000000000))*(cj8)*(npx)*(x394)))+(((IkReal(-1.00000000000000))*(npx)*(x391)))+(((IkReal(-1.00000000000000))*(x392)*(x393))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst1)*(((((cj8)*(npy)*(x394)))+(((npy)*(x391)))+(((IkReal(-1.00000000000000))*(x391)*(x393)))+(((npx)*(sj8)*(x394)))+(((npx)*(x392)))+(((IkReal(-1.00000000000000))*(x392)*(x395)))))), ((gconst1)*(((((npy)*(sj8)*(x394)))+(((npy)*(x392)))+(((x391)*(x395)))+(((IkReal(-1.00000000000000))*(cj8)*(npx)*(x394)))+(((IkReal(-1.00000000000000))*(npx)*(x391)))+(((IkReal(-1.00000000000000))*(x392)*(x393)))))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x396=IKsin(j10);
IkReal x397=IKcos(j10);
IkReal x398=((IkReal(0.0450000000000000))*(cj7));
IkReal x399=((IkReal(0.550000000000000))*(sj7));
IkReal x400=((npx)*(sj8));
IkReal x401=((IkReal(1.00000000000000))*(x397));
IkReal x402=((npy)*(x396));
IkReal x403=((IkReal(1.00000000000000))*(npx)*(x396));
evalcond[0]=((((sj8)*(x399)))+(((IkReal(0.0450000000000000))*(sj8)))+(((IkReal(-1.00000000000000))*(npy)*(x401)))+(((IkReal(-1.00000000000000))*(sj8)*(x398)))+(((IkReal(-1.00000000000000))*(x403))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj8)*(x403)))+(((sj8)*(x402)))+(((IkReal(-1.00000000000000))*(cj8)*(npy)*(x401)))+(((IkReal(-1.00000000000000))*(x400)*(x401))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((npy)*(sj8)*(x397)))+(((IkReal(-1.00000000000000))*(cj8)*(npx)*(x401)))+(((IkReal(-1.00000000000000))*(x399)))+(x398)+(((cj8)*(x402)))+(((x396)*(x400))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj8)*(x399)))+(((IkReal(-0.0450000000000000))*(cj8)))+(((IkReal(-1.00000000000000))*(npx)*(x401)))+(x402)+(((cj8)*(x398))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x404=((IkReal(0.0450000000000000))*(sj7));
IkReal x405=((IkReal(0.550000000000000))*(cj7));
IkReal x406=((x405)+(x404));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x406));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x406))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign((((npx)*(npx))+((npy)*(npy))));
dummyeval[0]=(((npx)*(npx))+((npy)*(npy)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x407=((IkReal(0.0450000000000000))*(npy));
IkReal x408=((cj7)*(cj8));
IkReal x409=((npx)*(sj8));
IkReal x410=((IkReal(0.550000000000000))*(sj7));
IkReal x411=((cj8)*(npx));
if( IKabs(((gconst2)*(((((x407)*(x408)))+(((IkReal(-1.00000000000000))*(cj8)*(x407)))+(((IkReal(0.0450000000000000))*(x409)))+(((IkReal(-0.0450000000000000))*(cj7)*(x409)))+(((IkReal(-1.00000000000000))*(cj8)*(npy)*(x410)))+(((x409)*(x410))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-0.0450000000000000))*(npx)*(x408)))+(((sj8)*(x407)))+(((IkReal(0.0450000000000000))*(x411)))+(((x410)*(x411)))+(((npy)*(sj8)*(x410)))+(((IkReal(-1.00000000000000))*(cj7)*(sj8)*(x407))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst2)*(((((x407)*(x408)))+(((IkReal(-1.00000000000000))*(cj8)*(x407)))+(((IkReal(0.0450000000000000))*(x409)))+(((IkReal(-0.0450000000000000))*(cj7)*(x409)))+(((IkReal(-1.00000000000000))*(cj8)*(npy)*(x410)))+(((x409)*(x410)))))), ((gconst2)*(((((IkReal(-0.0450000000000000))*(npx)*(x408)))+(((sj8)*(x407)))+(((IkReal(0.0450000000000000))*(x411)))+(((x410)*(x411)))+(((npy)*(sj8)*(x410)))+(((IkReal(-1.00000000000000))*(cj7)*(sj8)*(x407)))))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x412=IKcos(j10);
IkReal x413=IKsin(j10);
IkReal x414=((IkReal(1.00000000000000))*(npy));
IkReal x415=((IkReal(0.0450000000000000))*(cj8));
IkReal x416=((IkReal(0.0450000000000000))*(sj8));
IkReal x417=((IkReal(1.00000000000000))*(npx));
IkReal x418=((IkReal(0.550000000000000))*(sj7));
IkReal x419=((sj8)*(x413));
IkReal x420=((npx)*(x412));
IkReal x421=((cj8)*(x413));
evalcond[0]=((((sj8)*(x418)))+(((IkReal(-1.00000000000000))*(x412)*(x414)))+(((IkReal(-1.00000000000000))*(cj7)*(x416)))+(x416)+(((IkReal(-1.00000000000000))*(x413)*(x417))));
evalcond[1]=((((sj8)*(x420)))+(((IkReal(-1.00000000000000))*(cj8)*(x412)*(x414)))+(((IkReal(-1.00000000000000))*(x417)*(x421)))+(((IkReal(-1.00000000000000))*(x414)*(x419))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((cj8)*(x420)))+(((IkReal(0.0450000000000000))*(cj7)))+(((npy)*(sj8)*(x412)))+(((npx)*(x419)))+(((IkReal(-1.00000000000000))*(x414)*(x421)))+(((IkReal(-1.00000000000000))*(x418))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x412)*(x417)))+(((IkReal(-1.00000000000000))*(cj7)*(x415)))+(x415)+(((npy)*(x413)))+(((cj8)*(x418))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
IkReal x422=((IkReal(0.550000000000000))*(npy));
IkReal x423=((sj8)*(sj9));
IkReal x424=((cj9)*(npz));
IkReal x425=((IkReal(0.0450000000000000))*(cj7));
IkReal x426=((npx)*(sj7));
IkReal x427=((IkReal(0.0450000000000000))*(npy));
if( IKabs(((gconst0)*(((((IkReal(0.300000000000000))*(npy)))+(((IkReal(-1.00000000000000))*(npy)*(x424)))+(((sj7)*(x427)))+(((cj7)*(x422)))+(((IkReal(0.550000000000000))*(x423)*(x426)))+(((IkReal(0.0450000000000000))*(npx)*(x423)))+(((IkReal(-1.00000000000000))*(npx)*(x423)*(x425))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-0.550000000000000))*(cj7)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(x423)*(x425)))+(((npx)*(x424)))+(((IkReal(-0.0450000000000000))*(x426)))+(((x423)*(x427)))+(((IkReal(-0.300000000000000))*(npx)))+(((sj7)*(x422)*(x423))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst0)*(((((IkReal(0.300000000000000))*(npy)))+(((IkReal(-1.00000000000000))*(npy)*(x424)))+(((sj7)*(x427)))+(((cj7)*(x422)))+(((IkReal(0.550000000000000))*(x423)*(x426)))+(((IkReal(0.0450000000000000))*(npx)*(x423)))+(((IkReal(-1.00000000000000))*(npx)*(x423)*(x425)))))), ((gconst0)*(((((IkReal(-0.550000000000000))*(cj7)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(x423)*(x425)))+(((npx)*(x424)))+(((IkReal(-0.0450000000000000))*(x426)))+(((x423)*(x427)))+(((IkReal(-0.300000000000000))*(npx)))+(((sj7)*(x422)*(x423)))))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[5];
IkReal x428=IKcos(j10);
IkReal x429=IKsin(j10);
IkReal x430=((IkReal(0.0450000000000000))*(cj7));
IkReal x431=((IkReal(0.550000000000000))*(sj7));
IkReal x432=((cj9)*(sj8));
IkReal x433=((IkReal(1.00000000000000))*(npz));
IkReal x434=((cj8)*(cj9));
IkReal x435=((IkReal(0.0450000000000000))*(sj7));
IkReal x436=((IkReal(0.550000000000000))*(cj7));
IkReal x437=((IkReal(1.00000000000000))*(x428));
IkReal x438=((npy)*(x429));
IkReal x439=((IkReal(1.00000000000000))*(npx)*(x429));
evalcond[0]=((((IkReal(-1.00000000000000))*(x439)))+(((IkReal(-1.00000000000000))*(sj8)*(x430)))+(((IkReal(0.0450000000000000))*(sj8)))+(((IkReal(-1.00000000000000))*(npy)*(x437)))+(((sj8)*(x431))));
evalcond[1]=((IkReal(0.300000000000000))+(((npx)*(sj9)*(x428)))+(((IkReal(-1.00000000000000))*(sj9)*(x438)))+(((IkReal(-1.00000000000000))*(cj9)*(x433)))+(x435)+(x436));
evalcond[2]=((((IkReal(-1.00000000000000))*(npx)*(x432)*(x437)))+(((IkReal(-1.00000000000000))*(cj8)*(x439)))+(((IkReal(-1.00000000000000))*(sj8)*(sj9)*(x433)))+(((IkReal(-1.00000000000000))*(cj8)*(npy)*(x437)))+(((x432)*(x438))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((npy)*(sj8)*(x428)))+(((IkReal(-1.00000000000000))*(x431)))+(((x434)*(x438)))+(((IkReal(-1.00000000000000))*(npx)*(x434)*(x437)))+(x430)+(((IkReal(-1.00000000000000))*(cj8)*(sj9)*(x433)))+(((npx)*(sj8)*(x429))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x431)*(x434)))+(((IkReal(-1.00000000000000))*(sj9)*(x436)))+(((IkReal(-1.00000000000000))*(sj9)*(x435)))+(((IkReal(-0.0450000000000000))*(x434)))+(((IkReal(-1.00000000000000))*(npx)*(x437)))+(x438)+(((x430)*(x434)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[2], cj10array[2], sj10array[2];
bool j10valid[2]={false};
_nj10 = 2;
IkReal x440=(sj9)*(sj9);
if( IKabs(((npx)*(sj9))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npy)*(sj9))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x441=((IkReal(1.00000000000000))*(IKatan2(((npx)*(sj9)), ((IkReal(-1.00000000000000))*(npy)*(sj9)))));
if( (((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj9)*(npz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj9)*(npz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x442=IKasin(((((IKabs(IKabs(IKsqrt(((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x440)*((npy)*(npy))))+(((x440)*((npx)*(npx))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj9)*(npz)))))));
j10array[0]=((((IkReal(-1.00000000000000))*(x441)))+(((IkReal(-1.00000000000000))*(x442))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
j10array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x441)))+(x442));
sj10array[1]=IKsin(j10array[1]);
cj10array[1]=IKcos(j10array[1]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
if( j10array[1] > IKPI )
{
    j10array[1]-=IK2PI;
}
else if( j10array[1] < -IKPI )
{    j10array[1]+=IK2PI;
}
j10valid[1] = true;
for(int ij10 = 0; ij10 < 2; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 2; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];

{
IkReal dummyeval[2];
IkReal x443=((IkReal(12.2222222222222))*(sj7));
dummyeval[0]=((IkReal(1.00000000000000))+(x443)+(((IkReal(-1.00000000000000))*(cj7))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x443)))+(((cj7)*(sj9))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x444=((IkReal(12.2222222222222))*(sj7));
dummyeval[0]=((IkReal(1.00000000000000))+(x444)+(((IkReal(-1.00000000000000))*(cj7))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(cj9)*(x444)))+(((cj7)*(cj9)))+(((IkReal(-1.00000000000000))*(cj9))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x445=((IkReal(12.2222222222222))*(sj7));
IkReal x446=((cj10)*(npy));
IkReal x447=((npx)*(sj10));
dummyeval[0]=((IkReal(1.00000000000000))+(x445)+(((IkReal(-1.00000000000000))*(cj7))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(x445)*(x446)))+(((IkReal(-1.00000000000000))*(x445)*(x447)))+(((IkReal(-1.00000000000000))*(x447)))+(((IkReal(-1.00000000000000))*(x446)))+(((cj7)*(x446)))+(((cj7)*(x447))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x448=((IkReal(1.00000000000000))*(cj10));
IkReal x449=((npy)*(sj10));
IkReal x450=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x448)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(sj9)*(x449)))+(((IkReal(-1.00000000000000))*(cj9)*(x450)))+(((cj10)*(npx)*(sj9))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x450)))+(((IkReal(0.850000000000000))*(cj9))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(npx)*(x448)))+(x449));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x451=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x451));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x451))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x452=IKsin(j8);
IkReal x453=IKcos(j8);
IkReal x454=((IkReal(1.00000000000000))*(x453));
IkReal x455=((IkReal(1.00000000000000))*(x452));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj10)*(x454)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x454)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x455)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x455)))+(((cj9)*(npy)*(sj10)*(x452))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x456=((IkReal(1.00000000000000))*(cj10));
IkReal x457=((npy)*(sj10));
IkReal x458=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npx)*(sj10)))+(((IkReal(-1.00000000000000))*(npy)*(x456))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj9)*(x458)))+(((cj10)*(npx)*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x457))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x458)))+(((IkReal(0.850000000000000))*(cj9))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj9)))+(x457)+(((IkReal(-1.00000000000000))*(npx)*(x456))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x459=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x459));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x459))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x460=IKsin(j8);
IkReal x461=IKcos(j8);
IkReal x462=((IkReal(1.00000000000000))*(x461));
IkReal x463=((IkReal(1.00000000000000))*(x460));
evalcond[0]=((((cj9)*(npy)*(sj10)*(x460)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x462)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x462)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x463)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x463))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npy)*(sj10)*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(npz)))+(((cj10)*(npx)*(sj9))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x464=((npy)*(sj10));
IkReal x465=((cj10)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(x464)))+(x465));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(x465)))+(x464));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(-414405822679.079))*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x466=IKsin(j8);
IkReal x467=IKcos(j8);
IkReal x468=((IkReal(1.00000000000000))*(npz));
IkReal x469=((npx)*(sj10));
IkReal x470=((IkReal(1.00000000000000))*(cj10)*(npy));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x466)))+(((IkReal(-1.00000000000000))*(x469)))+(((IkReal(-1.00000000000000))*(x470))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x466)*(x468)))+(((IkReal(-1.00000000000000))*(x467)*(x470)))+(((IkReal(-1.00000000000000))*(x467)*(x469))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x467)))+(((IkReal(-1.00000000000000))*(x468))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x466)*(x469)))+(((cj10)*(npy)*(x466)))+(((IkReal(-1.00000000000000))*(x467)*(x468))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x471=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x471;
evalcond[3]=x471;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(414405822679.079))*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x472=IKsin(j8);
IkReal x473=IKcos(j8);
IkReal x474=((npx)*(sj10));
IkReal x475=((IkReal(1.00000000000000))*(cj10)*(npy));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x472)))+(((IkReal(-1.00000000000000))*(x475)))+(((IkReal(-1.00000000000000))*(x474))));
evalcond[1]=((((npz)*(x472)))+(((IkReal(-1.00000000000000))*(x473)*(x475)))+(((IkReal(-1.00000000000000))*(x473)*(x474))));
evalcond[2]=((((IkReal(2.41309350707268e-12))*(x473)))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((npz)*(x473)))+(((x472)*(x474)))+(((cj10)*(npy)*(x472))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x476=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x476;
evalcond[3]=x476;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x477=((IkReal(414405822679.079))*(sj10));
IkReal x478=((IkReal(414405822679.079))*(cj10));
if( IKabs(((((npy)*(x478)))+(((npx)*(x477))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(npx)*(x478)))+(((npy)*(x477))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((npy)*(x478)))+(((npx)*(x477)))))+IKsqr(((((IkReal(-1.00000000000000))*(npx)*(x478)))+(((npy)*(x477)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((npy)*(x478)))+(((npx)*(x477)))), ((((IkReal(-1.00000000000000))*(npx)*(x478)))+(((npy)*(x477)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x479=IKsin(j8);
IkReal x480=IKcos(j8);
IkReal x481=((IkReal(1.00000000000000))*(cj10));
IkReal x482=((npy)*(sj10));
IkReal x483=((npx)*(sj10));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x479)))+(((IkReal(-1.00000000000000))*(x483)))+(((IkReal(-1.00000000000000))*(npy)*(x481))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npx)*(x479)*(x481)))+(((IkReal(-1.00000000000000))*(npy)*(x480)*(x481)))+(((IkReal(-1.00000000000000))*(x480)*(x483)))+(((x479)*(x482))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(npx)*(x480)*(x481)))+(((cj10)*(npy)*(x479)))+(((x480)*(x482)))+(((x479)*(x483))));
evalcond[3]=((x482)+(((IkReal(-1.00000000000000))*(npx)*(x481)))+(((IkReal(-2.41309350707268e-12))*(x480))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(npz));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x484=((IkReal(414405822679.079))*(npx));
IkReal x485=((IkReal(414405822679.079))*(npy));
if( IKabs(((((sj10)*(x484)))+(((cj10)*(x485))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj10)*(x485)))+(((cj10)*(x484))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj10)*(x484)))+(((cj10)*(x485)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj10)*(x485)))+(((cj10)*(x484)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((sj10)*(x484)))+(((cj10)*(x485)))), ((((IkReal(-1.00000000000000))*(sj10)*(x485)))+(((cj10)*(x484)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x486=IKsin(j8);
IkReal x487=IKcos(j8);
IkReal x488=((IkReal(1.00000000000000))*(cj10));
IkReal x489=((npy)*(sj10));
IkReal x490=((IkReal(1.00000000000000))*(sj10));
IkReal x491=((npy)*(x487));
IkReal x492=((cj10)*(x486));
IkReal x493=((npx)*(x487));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x486)))+(((IkReal(-1.00000000000000))*(npy)*(x488)))+(((IkReal(-1.00000000000000))*(npx)*(x490))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x486)*(x489)))+(((IkReal(-1.00000000000000))*(x488)*(x491)))+(((npx)*(x492)))+(((IkReal(-1.00000000000000))*(x490)*(x493))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x487)*(x489)))+(((npx)*(sj10)*(x486)))+(((cj10)*(x493)))+(((npy)*(x492))));
evalcond[3]=((x489)+(((IkReal(2.41309350707268e-12))*(x487)))+(((IkReal(-1.00000000000000))*(npx)*(x488))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x494=((npy)*(sj10));
IkReal x495=((IkReal(2.28323001692700e+21))*(cj9));
IkReal x496=((cj10)*(npx));
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((x494)*(x495))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((x494)*(x495)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-1.89236762706040e+33))*(x494)*(x496)))+(((IkReal(-9.46183813530201e+32))*((cj10)*(cj10))*((npy)*(npy))))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj10)*(sj10)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((x494)*(x495))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((x494)*(x495)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-1.89236762706040e+33))*(x494)*(x496)))+(((IkReal(-9.46183813530201e+32))*((cj10)*(cj10))*((npy)*(npy))))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj10)*(sj10))))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((((IKabs(((((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((x494)*(x495))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((x494)*(x495)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-1.89236762706040e+33))*(x494)*(x496)))+(((IkReal(-9.46183813530201e+32))*((cj10)*(cj10))*((npy)*(npy))))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj10)*(sj10))))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x497=IKsin(j8);
IkReal x498=IKcos(j8);
IkReal x499=((cj9)*(sj10));
IkReal x500=((npz)*(sj9));
IkReal x501=((IkReal(1.00000000000000))*(cj10));
IkReal x502=((cj9)*(npx));
IkReal x503=((npx)*(sj10));
IkReal x504=((npy)*(x498));
IkReal x505=((IkReal(1.00000000000000))*(x498));
IkReal x506=((IkReal(2.41309350707268e-12))*(x498));
IkReal x507=((npy)*(x497));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x501)))+(((IkReal(2.41309350707268e-12))*(x497)))+(((IkReal(-1.00000000000000))*(x503))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x501)*(x504)))+(((IkReal(-1.00000000000000))*(x497)*(x501)*(x502)))+(((IkReal(-1.00000000000000))*(x503)*(x505)))+(((IkReal(-1.00000000000000))*(x497)*(x500)))+(((x499)*(x507))));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(sj9)*(x506)))+(((IkReal(-0.250000000390712))*(cj9))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x498)*(x501)*(x502)))+(((x497)*(x503)))+(((x499)*(x504)))+(((cj10)*(x507)))+(((IkReal(-1.00000000000000))*(x500)*(x505))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj9)))+(((IkReal(-1.00000000000000))*(npx)*(x501)))+(((IkReal(-1.00000000000000))*(cj9)*(x506)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(cj9) != 0)?((IkReal)1/(cj9)):(IkReal)1.0e30))*(((((IkReal(3.07601010000000e+16))*(npy)*(sj10)))+(((IkReal(7.69002526201833e+15))*(sj9)))+(((IkReal(-3.07601010000000e+16))*(cj10)*(npx))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(cj9) != 0)?((IkReal)1/(cj9)):(IkReal)1.0e30))*(((((IkReal(3.07601010000000e+16))*(npy)*(sj10)))+(((IkReal(7.69002526201833e+15))*(sj9)))+(((IkReal(-3.07601010000000e+16))*(cj10)*(npx)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(1.34721866706185e-5))*(((IKabs(cj9) != 0)?((IkReal)1/(cj9)):(IkReal)1.0e30))*(((((IkReal(3.07601010000000e+16))*(npy)*(sj10)))+(((IkReal(7.69002526201833e+15))*(sj9)))+(((IkReal(-3.07601010000000e+16))*(cj10)*(npx)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x508=IKsin(j8);
IkReal x509=IKcos(j8);
IkReal x510=((cj9)*(sj10));
IkReal x511=((npz)*(sj9));
IkReal x512=((IkReal(1.00000000000000))*(cj10));
IkReal x513=((cj9)*(npx));
IkReal x514=((npx)*(sj10));
IkReal x515=((npy)*(x509));
IkReal x516=((IkReal(1.00000000000000))*(x509));
IkReal x517=((IkReal(2.41309350707268e-12))*(x509));
IkReal x518=((npy)*(x508));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x512)))+(((IkReal(-1.00000000000000))*(x514)))+(((IkReal(2.41309350707268e-12))*(x508))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x508)*(x512)*(x513)))+(((x510)*(x518)))+(((IkReal(-1.00000000000000))*(x514)*(x516)))+(((IkReal(-1.00000000000000))*(x508)*(x511)))+(((IkReal(-1.00000000000000))*(x512)*(x515))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj9)*(x517)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-0.250000000390712))*(cj9))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x511)*(x516)))+(((x510)*(x515)))+(((cj10)*(x518)))+(((IkReal(-1.00000000000000))*(x509)*(x512)*(x513)))+(((x508)*(x514))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(x517)))+(((npy)*(sj10)))+(((IkReal(-1.00000000000000))*(npx)*(x512))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(sj9) != 0)?((IkReal)1/(sj9)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj9))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(sj9) != 0)?((IkReal)1/(sj9)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj9)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(1.34721866706185e-5))*(((IKabs(sj9) != 0)?((IkReal)1/(sj9)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj9)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x519=IKsin(j8);
IkReal x520=IKcos(j8);
IkReal x521=((cj9)*(sj10));
IkReal x522=((npz)*(sj9));
IkReal x523=((IkReal(1.00000000000000))*(cj10));
IkReal x524=((cj9)*(npx));
IkReal x525=((npx)*(sj10));
IkReal x526=((npy)*(x520));
IkReal x527=((IkReal(1.00000000000000))*(x520));
IkReal x528=((IkReal(2.41309350707268e-12))*(x520));
IkReal x529=((npy)*(x519));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x523)))+(((IkReal(-1.00000000000000))*(x525)))+(((IkReal(2.41309350707268e-12))*(x519))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x525)*(x527)))+(((IkReal(-1.00000000000000))*(x519)*(x522)))+(((IkReal(-1.00000000000000))*(x523)*(x526)))+(((IkReal(-1.00000000000000))*(x519)*(x523)*(x524)))+(((x521)*(x529))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj9)*(x528)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-0.250000000390712))*(cj9))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x519)*(x525)))+(((cj10)*(x529)))+(((IkReal(-1.00000000000000))*(x520)*(x523)*(x524)))+(((x521)*(x526)))+(((IkReal(-1.00000000000000))*(x522)*(x527))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj9)*(x528)))+(((IkReal(-1.00000000000000))*(npx)*(x523)))+(((IkReal(0.250000000390712))*(sj9)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x530=((IkReal(0.0450000000000000))*(sj7));
IkReal x531=((IkReal(0.550000000000000))*(cj7));
IkReal x532=((cj10)*(npx));
IkReal x533=((npy)*(sj10));
IkReal x534=((x531)+(x530));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(((IkReal(-1.00000000000000))*(x533)))+(x532)+(x534));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(x532)))+(((IkReal(-1.00000000000000))*(x534)))+(x533));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(-1.00000000000000))+(cj7)+(((IkReal(-12.2222222222222))*(sj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x535=((IkReal(1.00000000000000))*(cj10));
IkReal x536=((npy)*(sj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x535)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(x536)))+(((cj10)*(npx))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x535)))+(x536));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x537=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x537));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x537))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x538=IKcos(j8);
IkReal x539=((IkReal(1.00000000000000))*(x538));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x539)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x539))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x540=((npy)*(sj10));
IkReal x541=((cj10)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(6.11991262292145))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(x540)))+(x541));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(x541)))+(x540));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(-414405822679.079))*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x542=IKsin(j8);
IkReal x543=IKcos(j8);
IkReal x544=((IkReal(1.00000000000000))*(npz));
IkReal x545=((npx)*(sj10));
IkReal x546=((IkReal(1.00000000000000))*(cj10)*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(x545)))+(((IkReal(-1.00000000000000))*(x546)))+(((IkReal(2.41309350707268e-12))*(x542))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x542)*(x544)))+(((IkReal(-1.00000000000000))*(x543)*(x545)))+(((IkReal(-1.00000000000000))*(x543)*(x546))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x543)))+(((IkReal(-1.00000000000000))*(x544))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x542)*(x545)))+(((cj10)*(npy)*(x542)))+(((IkReal(-1.00000000000000))*(x543)*(x544))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x547=((IkReal(1.00000000000000))*(cj10));
IkReal x548=((npy)*(sj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x547)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(x548)))+(((cj10)*(npx))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x547)))+(x548));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x549=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x549));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x549))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x550=IKcos(j8);
IkReal x551=((IkReal(1.00000000000000))*(x550));
evalcond[0]=((((IkReal(-1.00000000000000))*(npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x551)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x551))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)))))), ((gconst3)*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x552=IKsin(j8);
IkReal x553=IKcos(j8);
IkReal x554=((IkReal(0.550000000000000))*(sj7));
IkReal x555=((cj10)*(npy));
IkReal x556=((IkReal(1.00000000000000))*(npz));
IkReal x557=((npx)*(sj10));
IkReal x558=((IkReal(1.00000000000000))*(x553));
IkReal x559=((IkReal(0.0450000000000000))*(x553));
IkReal x560=((IkReal(0.0450000000000000))*(x552));
evalcond[0]=((((IkReal(-1.00000000000000))*(x555)))+(((IkReal(-1.00000000000000))*(x557)))+(((IkReal(-1.00000000000000))*(cj7)*(x560)))+(x560)+(((x552)*(x554))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x555)*(x558)))+(((IkReal(-1.00000000000000))*(x552)*(x556)))+(((IkReal(-1.00000000000000))*(x557)*(x558))));
evalcond[2]=((((cj7)*(x559)))+(((IkReal(-1.00000000000000))*(x553)*(x554)))+(((IkReal(-1.00000000000000))*(x559)))+(((IkReal(-1.00000000000000))*(x556))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(x553)*(x556)))+(((IkReal(-1.00000000000000))*(x554)))+(((x552)*(x555)))+(((x552)*(x557))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x561=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((IkReal(0.550000000000000))*(cj7)))+(((npy)*(sj10))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x561;
evalcond[3]=x561;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x562=((IkReal(1.00000000000000))*(npx));
IkReal x563=((IkReal(0.850000000000000))+(((npy)*(sj10)))+(((IkReal(-1.00000000000000))*(cj10)*(x562))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x562))));
evalcond[3]=x563;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x563;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x564=IKatan2(npz, ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x564));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x564))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x565=IKcos(j8);
IkReal x566=((IkReal(1.00000000000000))*(x565));
evalcond[0]=((((npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x566)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x566))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x567=((IkReal(1.00000000000000))*(npx));
IkReal x568=((IkReal(0.850000000000000))+(((npy)*(sj10)))+(((IkReal(-1.00000000000000))*(cj10)*(x567))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x567))));
evalcond[3]=x568;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x568;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x569=IKatan2(npz, ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x569));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x569))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x570=IKcos(j8);
IkReal x571=((IkReal(1.00000000000000))*(x570));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(npy)*(x571)))+(((npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x571))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst4)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst4)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst4)*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x572=IKsin(j8);
IkReal x573=IKcos(j8);
IkReal x574=((IkReal(0.550000000000000))*(sj7));
IkReal x575=((IkReal(0.0450000000000000))*(x573));
IkReal x576=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x577=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x578=((IkReal(0.0450000000000000))*(x572));
evalcond[0]=((((IkReal(-1.00000000000000))*(x576)))+(((IkReal(-1.00000000000000))*(x577)))+(x578)+(((IkReal(-1.00000000000000))*(cj7)*(x578)))+(((x572)*(x574))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x573)*(x577)))+(((IkReal(-1.00000000000000))*(x573)*(x576)))+(((npz)*(x572))));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((x573)*(x574)))+(x575)+(((IkReal(-1.00000000000000))*(cj7)*(x575))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((npz)*(x573)))+(((cj10)*(npy)*(x572)))+(((IkReal(-1.00000000000000))*(x574)))+(((npx)*(sj10)*(x572))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x579=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj7))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x579;
evalcond[3]=x579;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x580=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x581=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x581))));
evalcond[3]=x580;
evalcond[4]=x580;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x581)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x582=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x582));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x582))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x583=IKcos(j8);
IkReal x584=IKsin(j8);
IkReal x585=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(x584)*(x585)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x583)))+(((npy)*(sj10)*(x584)))+(((IkReal(-1.00000000000000))*(npy)*(x583)*(x585))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x586=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x587=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x587))));
evalcond[3]=x586;
evalcond[4]=x586;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x587)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x588=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x588));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x588))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x589=IKcos(j8);
IkReal x590=IKsin(j8);
IkReal x591=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x589)*(x591)))+(((IkReal(-1.00000000000000))*(npx)*(x590)*(x591)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x589)))+(((npy)*(sj10)*(x590))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst5)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst5)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst5)*(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x592=IKsin(j8);
IkReal x593=IKcos(j8);
IkReal x594=((IkReal(0.550000000000000))*(sj7));
IkReal x595=((npy)*(sj10));
IkReal x596=((IkReal(1.00000000000000))*(npx));
IkReal x597=((IkReal(0.0450000000000000))*(x593));
IkReal x598=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x599=((cj10)*(x592));
IkReal x600=((IkReal(0.0450000000000000))*(x592));
evalcond[0]=((((IkReal(-1.00000000000000))*(x598)))+(x600)+(((IkReal(-1.00000000000000))*(sj10)*(x596)))+(((x592)*(x594)))+(((IkReal(-1.00000000000000))*(cj7)*(x600))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x596)*(x599)))+(((IkReal(-1.00000000000000))*(sj10)*(x593)*(x596)))+(((IkReal(-1.00000000000000))*(x593)*(x598)))+(((x592)*(x595))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj10)*(x593)*(x596)))+(((IkReal(-1.00000000000000))*(x594)))+(((npy)*(x599)))+(((x593)*(x595)))+(((npx)*(sj10)*(x592))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x597)))+(((IkReal(-1.00000000000000))*(x593)*(x594)))+(x595)+(((cj7)*(x597)))+(((IkReal(-1.00000000000000))*(cj10)*(x596))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x601=((IkReal(0.0450000000000000))*(sj7));
IkReal x602=((IkReal(0.550000000000000))*(cj7));
IkReal x603=((x602)+(x601));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x603));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x603))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x604=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x604))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x604)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x605=IKatan2(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x605));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x605))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x606=IKsin(j8);
IkReal x607=IKcos(j8);
IkReal x608=((IkReal(1.00000000000000))*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(x607)*(x608)))+(((IkReal(-1.00000000000000))*(sj10)*(x606)*(x608)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x607)))+(((cj10)*(npx)*(x606))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x609=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x609))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x609)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x610=IKatan2(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x610));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x610))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x611=IKsin(j8);
IkReal x612=IKcos(j8);
IkReal x613=((IkReal(1.00000000000000))*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj10)*(x611)*(x613)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x612)))+(((IkReal(-1.00000000000000))*(cj10)*(x612)*(x613)))+(((cj10)*(npx)*(x611))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst6)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst6)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst6)*(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x614=IKsin(j8);
IkReal x615=IKcos(j8);
IkReal x616=((IkReal(0.550000000000000))*(sj7));
IkReal x617=((cj10)*(npx));
IkReal x618=((npy)*(sj10));
IkReal x619=((IkReal(0.0450000000000000))*(x615));
IkReal x620=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x621=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x622=((IkReal(0.0450000000000000))*(x614));
evalcond[0]=((((IkReal(-1.00000000000000))*(x620)))+(((IkReal(-1.00000000000000))*(x621)))+(((IkReal(-1.00000000000000))*(cj7)*(x622)))+(((x614)*(x616)))+(x622));
evalcond[1]=((((IkReal(-1.00000000000000))*(x614)*(x618)))+(((x614)*(x617)))+(((IkReal(-1.00000000000000))*(x615)*(x620)))+(((IkReal(-1.00000000000000))*(x615)*(x621))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((cj10)*(npy)*(x614)))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(x615)*(x618)))+(((npx)*(sj10)*(x614)))+(((IkReal(-1.00000000000000))*(x616)))+(((x615)*(x617))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x617)))+(x618)+(x619)+(((x615)*(x616)))+(((IkReal(-1.00000000000000))*(cj7)*(x619))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x623=((cj10)*(npy));
IkReal x624=((npz)*(sj9));
IkReal x625=((IkReal(0.0450000000000000))*(cj7));
IkReal x626=((IkReal(0.550000000000000))*(sj7));
IkReal x627=((npx)*(sj10));
IkReal x628=((cj9)*(npx)*(npy));
IkReal x629=((cj10)*(cj9)*(sj10));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x625)))+(x626))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x625)))+(x626)))):(IkReal)1.0e30))*(((x627)+(x623))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x623)*(x626)))+(((x625)*(x627)))+(((IkReal(-0.0450000000000000))*(x623)))+(((IkReal(-0.0450000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(x626)*(x627)))+(((x623)*(x625))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x623)*(x626)))+(((x625)*(x627)))+(((IkReal(-0.0450000000000000))*(x623)))+(((IkReal(-0.0450000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(x626)*(x627)))+(((x623)*(x625)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x627)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x623)))+(((cj10)*(cj9)*(npx)*(x623)))+(((cj10)*(cj9)*(npx)*(x627)))+(((x624)*(x627)))+(((x623)*(x624))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x625)))+(x626))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x625)))+(x626)))):(IkReal)1.0e30))*(((x627)+(x623)))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x623)*(x626)))+(((x625)*(x627)))+(((IkReal(-0.0450000000000000))*(x623)))+(((IkReal(-0.0450000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(x626)*(x627)))+(((x623)*(x625))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x623)*(x626)))+(((x625)*(x627)))+(((IkReal(-0.0450000000000000))*(x623)))+(((IkReal(-0.0450000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(x626)*(x627)))+(((x623)*(x625)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x627)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x623)))+(((cj10)*(cj9)*(npx)*(x623)))+(((cj10)*(cj9)*(npx)*(x627)))+(((x624)*(x627)))+(((x623)*(x624)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x625)))+(x626))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x625)))+(x626)))):(IkReal)1.0e30))*(((x627)+(x623)))), ((((IKabs(((((IkReal(-1.00000000000000))*(x623)*(x626)))+(((x625)*(x627)))+(((IkReal(-0.0450000000000000))*(x623)))+(((IkReal(-0.0450000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(x626)*(x627)))+(((x623)*(x625))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x623)*(x626)))+(((x625)*(x627)))+(((IkReal(-0.0450000000000000))*(x623)))+(((IkReal(-0.0450000000000000))*(x627)))+(((IkReal(-1.00000000000000))*(x626)*(x627)))+(((x623)*(x625)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x627)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x623)))+(((cj10)*(cj9)*(npx)*(x623)))+(((cj10)*(cj9)*(npx)*(x627)))+(((x624)*(x627)))+(((x623)*(x624)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x630=IKsin(j8);
IkReal x631=IKcos(j8);
IkReal x632=((npy)*(sj10));
IkReal x633=((IkReal(1.00000000000000))*(cj10));
IkReal x634=((IkReal(0.550000000000000))*(sj7));
IkReal x635=((IkReal(1.00000000000000))*(npz));
IkReal x636=((IkReal(0.550000000000000))*(cj7));
IkReal x637=((IkReal(0.0450000000000000))*(cj7));
IkReal x638=((IkReal(0.0450000000000000))*(sj7));
IkReal x639=((cj9)*(x631));
IkReal x640=((sj9)*(x631));
IkReal x641=((cj9)*(x630));
IkReal x642=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(x630)*(x637)))+(((IkReal(-1.00000000000000))*(x642)))+(((x630)*(x634)))+(((IkReal(0.0450000000000000))*(x630)))+(((IkReal(-1.00000000000000))*(npy)*(x633))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x631)*(x642)))+(((IkReal(-1.00000000000000))*(sj9)*(x630)*(x635)))+(((x632)*(x641)))+(((IkReal(-1.00000000000000))*(npx)*(x633)*(x641)))+(((IkReal(-1.00000000000000))*(npy)*(x631)*(x633))));
evalcond[2]=((((x637)*(x640)))+(((IkReal(-1.00000000000000))*(x635)))+(((IkReal(-0.0450000000000000))*(x640)))+(((IkReal(0.300000000000000))*(cj9)))+(((cj9)*(x636)))+(((cj9)*(x638)))+(((IkReal(-1.00000000000000))*(x634)*(x640))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((cj10)*(npy)*(x630)))+(((IkReal(-1.00000000000000))*(x634)))+(((IkReal(-1.00000000000000))*(x635)*(x640)))+(x637)+(((x632)*(x639)))+(((npx)*(sj10)*(x630)))+(((IkReal(-1.00000000000000))*(npx)*(x633)*(x639))));
evalcond[4]=((((IkReal(-1.00000000000000))*(npx)*(x633)))+(((IkReal(-1.00000000000000))*(sj9)*(x636)))+(((IkReal(-1.00000000000000))*(sj9)*(x638)))+(((IkReal(-0.0450000000000000))*(x639)))+(((IkReal(-1.00000000000000))*(x634)*(x639)))+(x632)+(((x637)*(x639)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x643=((IkReal(0.0450000000000000))*(cj7));
IkReal x644=((IkReal(0.550000000000000))*(sj7));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x643)))+(x644))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x643)))+(x644)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(cj9)*(x644)))+(((cj9)*(x643)))+(((IkReal(-0.0450000000000000))*(cj9))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(cj9)*(x644)))+(((cj9)*(x643)))+(((IkReal(-0.0450000000000000))*(cj9)))))):(IkReal)1.0e30))*(((((cj10)*(npx)))+(((IkReal(0.0450000000000000))*(sj7)*(sj9)))+(((IkReal(0.300000000000000))*(sj9)))+(((IkReal(0.550000000000000))*(cj7)*(sj9)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x643)))+(x644))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x643)))+(x644)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(cj9)*(x644)))+(((cj9)*(x643)))+(((IkReal(-0.0450000000000000))*(cj9))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(cj9)*(x644)))+(((cj9)*(x643)))+(((IkReal(-0.0450000000000000))*(cj9)))))):(IkReal)1.0e30))*(((((cj10)*(npx)))+(((IkReal(0.0450000000000000))*(sj7)*(sj9)))+(((IkReal(0.300000000000000))*(sj9)))+(((IkReal(0.550000000000000))*(cj7)*(sj9)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x643)))+(x644))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x643)))+(x644)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((((IKabs(((((IkReal(-1.00000000000000))*(cj9)*(x644)))+(((cj9)*(x643)))+(((IkReal(-0.0450000000000000))*(cj9))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(cj9)*(x644)))+(((cj9)*(x643)))+(((IkReal(-0.0450000000000000))*(cj9)))))):(IkReal)1.0e30))*(((((cj10)*(npx)))+(((IkReal(0.0450000000000000))*(sj7)*(sj9)))+(((IkReal(0.300000000000000))*(sj9)))+(((IkReal(0.550000000000000))*(cj7)*(sj9)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x645=IKsin(j8);
IkReal x646=IKcos(j8);
IkReal x647=((npy)*(sj10));
IkReal x648=((IkReal(1.00000000000000))*(cj10));
IkReal x649=((IkReal(0.550000000000000))*(sj7));
IkReal x650=((IkReal(1.00000000000000))*(npz));
IkReal x651=((IkReal(0.550000000000000))*(cj7));
IkReal x652=((IkReal(0.0450000000000000))*(cj7));
IkReal x653=((IkReal(0.0450000000000000))*(sj7));
IkReal x654=((cj9)*(x646));
IkReal x655=((sj9)*(x646));
IkReal x656=((cj9)*(x645));
IkReal x657=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(0.0450000000000000))*(x645)))+(((IkReal(-1.00000000000000))*(x657)))+(((x645)*(x649)))+(((IkReal(-1.00000000000000))*(x645)*(x652)))+(((IkReal(-1.00000000000000))*(npy)*(x648))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj9)*(x645)*(x650)))+(((x647)*(x656)))+(((IkReal(-1.00000000000000))*(npy)*(x646)*(x648)))+(((IkReal(-1.00000000000000))*(x646)*(x657)))+(((IkReal(-1.00000000000000))*(npx)*(x648)*(x656))));
evalcond[2]=((((x652)*(x655)))+(((cj9)*(x653)))+(((cj9)*(x651)))+(((IkReal(-1.00000000000000))*(x650)))+(((IkReal(-0.0450000000000000))*(x655)))+(((IkReal(-1.00000000000000))*(x649)*(x655)))+(((IkReal(0.300000000000000))*(cj9))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((x647)*(x654)))+(((IkReal(-1.00000000000000))*(x649)))+(((npx)*(sj10)*(x645)))+(x652)+(((IkReal(-1.00000000000000))*(x650)*(x655)))+(((IkReal(-1.00000000000000))*(npx)*(x648)*(x654)))+(((cj10)*(npy)*(x645))));
evalcond[4]=((((x652)*(x654)))+(x647)+(((IkReal(-0.0450000000000000))*(x654)))+(((IkReal(-1.00000000000000))*(x649)*(x654)))+(((IkReal(-1.00000000000000))*(sj9)*(x653)))+(((IkReal(-1.00000000000000))*(sj9)*(x651)))+(((IkReal(-1.00000000000000))*(npx)*(x648)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x658=((IkReal(0.0450000000000000))*(cj7));
IkReal x659=((IkReal(0.550000000000000))*(sj7));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x658)))+(x659))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x658)))+(x659)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x659)))+(((sj9)*(x658))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x659)))+(((sj9)*(x658)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj9)))+(npz)+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x658)))+(x659))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x658)))+(x659)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))))+IKsqr(((((IKabs(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x659)))+(((sj9)*(x658))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x659)))+(((sj9)*(x658)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj9)))+(npz)+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x658)))+(x659))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x658)))+(x659)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((((IKabs(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x659)))+(((sj9)*(x658))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x659)))+(((sj9)*(x658)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj9)))+(npz)+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x660=IKsin(j8);
IkReal x661=IKcos(j8);
IkReal x662=((npy)*(sj10));
IkReal x663=((IkReal(1.00000000000000))*(cj10));
IkReal x664=((IkReal(0.550000000000000))*(sj7));
IkReal x665=((IkReal(1.00000000000000))*(npz));
IkReal x666=((IkReal(0.550000000000000))*(cj7));
IkReal x667=((IkReal(0.0450000000000000))*(cj7));
IkReal x668=((IkReal(0.0450000000000000))*(sj7));
IkReal x669=((cj9)*(x661));
IkReal x670=((sj9)*(x661));
IkReal x671=((cj9)*(x660));
IkReal x672=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(x660)*(x667)))+(((IkReal(0.0450000000000000))*(x660)))+(((x660)*(x664)))+(((IkReal(-1.00000000000000))*(npy)*(x663)))+(((IkReal(-1.00000000000000))*(x672))));
evalcond[1]=((((x662)*(x671)))+(((IkReal(-1.00000000000000))*(npy)*(x661)*(x663)))+(((IkReal(-1.00000000000000))*(sj9)*(x660)*(x665)))+(((IkReal(-1.00000000000000))*(npx)*(x663)*(x671)))+(((IkReal(-1.00000000000000))*(x661)*(x672))));
evalcond[2]=((((IkReal(-0.0450000000000000))*(x670)))+(((IkReal(-1.00000000000000))*(x664)*(x670)))+(((x667)*(x670)))+(((IkReal(-1.00000000000000))*(x665)))+(((cj9)*(x666)))+(((cj9)*(x668)))+(((IkReal(0.300000000000000))*(cj9))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((npx)*(sj10)*(x660)))+(((x662)*(x669)))+(((IkReal(-1.00000000000000))*(x665)*(x670)))+(((IkReal(-1.00000000000000))*(npx)*(x663)*(x669)))+(((IkReal(-1.00000000000000))*(x664)))+(x667)+(((cj10)*(npy)*(x660))));
evalcond[4]=((((x667)*(x669)))+(((IkReal(-1.00000000000000))*(sj9)*(x668)))+(((IkReal(-1.00000000000000))*(sj9)*(x666)))+(x662)+(((IkReal(-0.0450000000000000))*(x669)))+(((IkReal(-1.00000000000000))*(x664)*(x669)))+(((IkReal(-1.00000000000000))*(npx)*(x663)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[2], cj10array[2], sj10array[2];
bool j10valid[2]={false};
_nj10 = 2;
IkReal x673=(sj9)*(sj9);
if( IKabs(((npx)*(sj9))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(npy)*(sj9))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x674=((IkReal(1.00000000000000))*(IKatan2(((npx)*(sj9)), ((IkReal(-1.00000000000000))*(npy)*(sj9)))));
if( (((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj9)*(npz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj9)*(npz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x675=IKasin(((((IKabs(IKabs(IKsqrt(((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x673)*((npx)*(npx))))+(((x673)*((npy)*(npy))))))))):(IkReal)1.0e30))*(((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(0.550000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj9)*(npz)))))));
j10array[0]=((((IkReal(-1.00000000000000))*(x675)))+(((IkReal(-1.00000000000000))*(x674))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
j10array[1]=((IkReal(3.14159265358979))+(x675)+(((IkReal(-1.00000000000000))*(x674))));
sj10array[1]=IKsin(j10array[1]);
cj10array[1]=IKcos(j10array[1]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
if( j10array[1] > IKPI )
{
    j10array[1]-=IK2PI;
}
else if( j10array[1] < -IKPI )
{    j10array[1]+=IK2PI;
}
j10valid[1] = true;
for(int ij10 = 0; ij10 < 2; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 2; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];

{
IkReal dummyeval[2];
IkReal x676=((IkReal(12.2222222222222))*(sj7));
dummyeval[0]=((IkReal(1.00000000000000))+(x676)+(((IkReal(-1.00000000000000))*(cj7))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x676)))+(((cj7)*(sj9))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x677=((IkReal(12.2222222222222))*(sj7));
dummyeval[0]=((IkReal(1.00000000000000))+(x677)+(((IkReal(-1.00000000000000))*(cj7))));
dummyeval[1]=((((cj7)*(cj9)))+(((IkReal(-1.00000000000000))*(cj9)*(x677)))+(((IkReal(-1.00000000000000))*(cj9))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x678=((IkReal(12.2222222222222))*(sj7));
IkReal x679=((cj10)*(npy));
IkReal x680=((npx)*(sj10));
dummyeval[0]=((IkReal(1.00000000000000))+(x678)+(((IkReal(-1.00000000000000))*(cj7))));
dummyeval[1]=((((IkReal(-1.00000000000000))*(x678)*(x679)))+(((cj7)*(x679)))+(((IkReal(-1.00000000000000))*(x678)*(x680)))+(((cj7)*(x680)))+(((IkReal(-1.00000000000000))*(x679)))+(((IkReal(-1.00000000000000))*(x680))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x681=((IkReal(1.00000000000000))*(cj10));
IkReal x682=((npy)*(sj10));
IkReal x683=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x681)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(sj9)*(x682)))+(((IkReal(-1.00000000000000))*(cj9)*(x683)))+(((cj10)*(npx)*(sj9))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x683)))+(((IkReal(0.850000000000000))*(cj9))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(npx)*(x681)))+(x682));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x684=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x684));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x684))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x685=IKsin(j8);
IkReal x686=IKcos(j8);
IkReal x687=((IkReal(1.00000000000000))*(x686));
IkReal x688=((IkReal(1.00000000000000))*(x685));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(sj10)*(x687)))+(((cj9)*(npy)*(sj10)*(x685)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x687)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x688)))+(((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x688))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x689=((IkReal(1.00000000000000))*(cj10));
IkReal x690=((npy)*(sj10));
IkReal x691=((IkReal(1.00000000000000))*(npz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x689)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj9)*(x691)))+(((IkReal(-1.00000000000000))*(sj9)*(x690)))+(((cj10)*(npx)*(sj9))));
evalcond[4]=((((IkReal(0.850000000000000))*(cj9)))+(((IkReal(-1.00000000000000))*(x691))));
evalcond[5]=((((IkReal(-0.850000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(npx)*(x689)))+(x690));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x692=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x692));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x692))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x693=IKsin(j8);
IkReal x694=IKcos(j8);
IkReal x695=((IkReal(1.00000000000000))*(x694));
IkReal x696=((IkReal(1.00000000000000))*(x693));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)*(x696)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)*(x696)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x695)))+(((cj9)*(npy)*(sj10)*(x693)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x695))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.163272684258140))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npy)*(sj10)*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(npz)))+(((cj10)*(npx)*(sj9))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj10)*(cj9)*(npx)))+(((IkReal(-1.00000000000000))*(npz)*(sj9)))+(((cj9)*(npy)*(sj10))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x697=((npy)*(sj10));
IkReal x698=((cj10)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(x698)+(((IkReal(-1.00000000000000))*(x697))));
evalcond[3]=((IkReal(0.250000000390712))+(x697)+(((IkReal(-1.00000000000000))*(x698))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(-414405822679.079))*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x699=IKsin(j8);
IkReal x700=IKcos(j8);
IkReal x701=((IkReal(1.00000000000000))*(npz));
IkReal x702=((npx)*(sj10));
IkReal x703=((IkReal(1.00000000000000))*(cj10)*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(x702)))+(((IkReal(-1.00000000000000))*(x703)))+(((IkReal(2.41309350707268e-12))*(x699))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x700)*(x702)))+(((IkReal(-1.00000000000000))*(x699)*(x701)))+(((IkReal(-1.00000000000000))*(x700)*(x703))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x700)))+(((IkReal(-1.00000000000000))*(x701))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((cj10)*(npy)*(x699)))+(((x699)*(x702)))+(((IkReal(-1.00000000000000))*(x700)*(x701))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x704=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x704;
evalcond[3]=x704;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(414405822679.079))*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x705=IKsin(j8);
IkReal x706=IKcos(j8);
IkReal x707=((npx)*(sj10));
IkReal x708=((IkReal(1.00000000000000))*(cj10)*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(x707)))+(((IkReal(-1.00000000000000))*(x708)))+(((IkReal(2.41309350707268e-12))*(x705))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x706)*(x707)))+(((npz)*(x705))));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((IkReal(2.41309350707268e-12))*(x706))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((cj10)*(npy)*(x705)))+(((npz)*(x706)))+(((x705)*(x707))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x709=((IkReal(-0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=x709;
evalcond[3]=x709;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x710=((IkReal(414405822679.079))*(sj10));
IkReal x711=((IkReal(414405822679.079))*(cj10));
if( IKabs(((((npy)*(x711)))+(((npx)*(x710))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((npy)*(x710)))+(((IkReal(-1.00000000000000))*(npx)*(x711))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((npy)*(x711)))+(((npx)*(x710)))))+IKsqr(((((npy)*(x710)))+(((IkReal(-1.00000000000000))*(npx)*(x711)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((npy)*(x711)))+(((npx)*(x710)))), ((((npy)*(x710)))+(((IkReal(-1.00000000000000))*(npx)*(x711)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x712=IKsin(j8);
IkReal x713=IKcos(j8);
IkReal x714=((IkReal(1.00000000000000))*(cj10));
IkReal x715=((npy)*(sj10));
IkReal x716=((npx)*(sj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x714)))+(((IkReal(-1.00000000000000))*(x716)))+(((IkReal(2.41309350707268e-12))*(x712))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npx)*(x712)*(x714)))+(((IkReal(-1.00000000000000))*(npy)*(x713)*(x714)))+(((IkReal(-1.00000000000000))*(x713)*(x716)))+(((x712)*(x715))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((cj10)*(npy)*(x712)))+(((IkReal(-1.00000000000000))*(npx)*(x713)*(x714)))+(((x712)*(x716)))+(((x713)*(x715))));
evalcond[3]=((((IkReal(-1.00000000000000))*(npx)*(x714)))+(((IkReal(-2.41309350707268e-12))*(x713)))+(x715));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(npz));
evalcond[3]=((IkReal(0.250000000390712))+(((IkReal(-1.00000000000000))*(npz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x717=((IkReal(414405822679.079))*(npx));
IkReal x718=((IkReal(414405822679.079))*(npy));
if( IKabs(((((cj10)*(x718)))+(((sj10)*(x717))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(x717)))+(((IkReal(-1.00000000000000))*(sj10)*(x718))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj10)*(x718)))+(((sj10)*(x717)))))+IKsqr(((((cj10)*(x717)))+(((IkReal(-1.00000000000000))*(sj10)*(x718)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((cj10)*(x718)))+(((sj10)*(x717)))), ((((cj10)*(x717)))+(((IkReal(-1.00000000000000))*(sj10)*(x718)))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x719=IKsin(j8);
IkReal x720=IKcos(j8);
IkReal x721=((IkReal(1.00000000000000))*(cj10));
IkReal x722=((npy)*(sj10));
IkReal x723=((IkReal(1.00000000000000))*(sj10));
IkReal x724=((npy)*(x720));
IkReal x725=((cj10)*(x719));
IkReal x726=((npx)*(x720));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x721)))+(((IkReal(-1.00000000000000))*(npx)*(x723)))+(((IkReal(2.41309350707268e-12))*(x719))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x723)*(x726)))+(((IkReal(-1.00000000000000))*(x721)*(x724)))+(((npx)*(x725)))+(((IkReal(-1.00000000000000))*(x719)*(x722))));
evalcond[2]=((IkReal(-2.41309350707268e-12))+(((npx)*(sj10)*(x719)))+(((cj10)*(x726)))+(((npy)*(x725)))+(((IkReal(-1.00000000000000))*(x720)*(x722))));
evalcond[3]=((((IkReal(2.41309350707268e-12))*(x720)))+(((IkReal(-1.00000000000000))*(npx)*(x721)))+(x722));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x727=((npy)*(sj10));
IkReal x728=((IkReal(2.28323001692700e+21))*(cj9));
IkReal x729=((cj10)*(npx));
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((x727)*(x728)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(x728)*(x729))))) != 0)?((IkReal)1/(((((x727)*(x728)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(x728)*(x729)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-1.89236762706040e+33))*(x727)*(x729)))+(((IkReal(-9.46183813530201e+32))*((cj10)*(cj10))*((npy)*(npy))))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj10)*(sj10)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((((IKabs(((((x727)*(x728)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(x728)*(x729))))) != 0)?((IkReal)1/(((((x727)*(x728)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(x728)*(x729)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-1.89236762706040e+33))*(x727)*(x729)))+(((IkReal(-9.46183813530201e+32))*((cj10)*(cj10))*((npy)*(npy))))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj10)*(sj10))))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((((IKabs(((((x727)*(x728)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(x728)*(x729))))) != 0)?((IkReal)1/(((((x727)*(x728)))+(((IkReal(-2.28323001692700e+21))*(npz)*(sj9)))+(((IkReal(-1.00000000000000))*(x728)*(x729)))))):(IkReal)1.0e30))*(((IkReal(5509647529.00000))+(((IkReal(-1.89236762706040e+33))*(x727)*(x729)))+(((IkReal(-9.46183813530201e+32))*((cj10)*(cj10))*((npy)*(npy))))+(((IkReal(-9.46183813530201e+32))*((npx)*(npx))*((sj10)*(sj10))))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x730=IKsin(j8);
IkReal x731=IKcos(j8);
IkReal x732=((cj9)*(sj10));
IkReal x733=((npz)*(sj9));
IkReal x734=((IkReal(1.00000000000000))*(cj10));
IkReal x735=((cj9)*(npx));
IkReal x736=((npx)*(sj10));
IkReal x737=((npy)*(x731));
IkReal x738=((IkReal(1.00000000000000))*(x731));
IkReal x739=((IkReal(2.41309350707268e-12))*(x731));
IkReal x740=((npy)*(x730));
evalcond[0]=((((IkReal(-1.00000000000000))*(x736)))+(((IkReal(-1.00000000000000))*(npy)*(x734)))+(((IkReal(2.41309350707268e-12))*(x730))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x730)*(x734)*(x735)))+(((IkReal(-1.00000000000000))*(x736)*(x738)))+(((IkReal(-1.00000000000000))*(x730)*(x733)))+(((IkReal(-1.00000000000000))*(x734)*(x737)))+(((x732)*(x740))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj9)*(x739)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-0.250000000390712))*(cj9))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((cj10)*(x740)))+(((IkReal(-1.00000000000000))*(x733)*(x738)))+(((x732)*(x737)))+(((x730)*(x736)))+(((IkReal(-1.00000000000000))*(x731)*(x734)*(x735))));
evalcond[4]=((((IkReal(0.250000000390712))*(sj9)))+(((IkReal(-1.00000000000000))*(npx)*(x734)))+(((IkReal(-1.00000000000000))*(cj9)*(x739)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(cj9) != 0)?((IkReal)1/(cj9)):(IkReal)1.0e30))*(((((IkReal(3.07601010000000e+16))*(npy)*(sj10)))+(((IkReal(7.69002526201833e+15))*(sj9)))+(((IkReal(-3.07601010000000e+16))*(cj10)*(npx))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(cj9) != 0)?((IkReal)1/(cj9)):(IkReal)1.0e30))*(((((IkReal(3.07601010000000e+16))*(npy)*(sj10)))+(((IkReal(7.69002526201833e+15))*(sj9)))+(((IkReal(-3.07601010000000e+16))*(cj10)*(npx)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(1.34721866706185e-5))*(((IKabs(cj9) != 0)?((IkReal)1/(cj9)):(IkReal)1.0e30))*(((((IkReal(3.07601010000000e+16))*(npy)*(sj10)))+(((IkReal(7.69002526201833e+15))*(sj9)))+(((IkReal(-3.07601010000000e+16))*(cj10)*(npx)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x741=IKsin(j8);
IkReal x742=IKcos(j8);
IkReal x743=((cj9)*(sj10));
IkReal x744=((npz)*(sj9));
IkReal x745=((IkReal(1.00000000000000))*(cj10));
IkReal x746=((cj9)*(npx));
IkReal x747=((npx)*(sj10));
IkReal x748=((npy)*(x742));
IkReal x749=((IkReal(1.00000000000000))*(x742));
IkReal x750=((IkReal(2.41309350707268e-12))*(x742));
IkReal x751=((npy)*(x741));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x745)))+(((IkReal(2.41309350707268e-12))*(x741)))+(((IkReal(-1.00000000000000))*(x747))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x745)*(x748)))+(((IkReal(-1.00000000000000))*(x747)*(x749)))+(((x743)*(x751)))+(((IkReal(-1.00000000000000))*(x741)*(x745)*(x746)))+(((IkReal(-1.00000000000000))*(x741)*(x744))));
evalcond[2]=((((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-0.250000000390712))*(cj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x750))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x742)*(x745)*(x746)))+(((cj10)*(x751)))+(((IkReal(-1.00000000000000))*(x744)*(x749)))+(((x741)*(x747)))+(((x743)*(x748))));
evalcond[4]=((((IkReal(-1.00000000000000))*(npx)*(x745)))+(((IkReal(0.250000000390712))*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(x750)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.34721866706185e-5))*(((IKabs(sj9) != 0)?((IkReal)1/(sj9)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj9))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(1.34721866706185e-5))*(((IKabs(sj9) != 0)?((IkReal)1/(sj9)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj9)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(1.34721866706185e-5))*(((IKabs(sj9) != 0)?((IkReal)1/(sj9)):(IkReal)1.0e30))*(((((IkReal(-3.07601010000000e+16))*(npz)))+(((IkReal(-7.69002526201833e+15))*(cj9)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x752=IKsin(j8);
IkReal x753=IKcos(j8);
IkReal x754=((cj9)*(sj10));
IkReal x755=((npz)*(sj9));
IkReal x756=((IkReal(1.00000000000000))*(cj10));
IkReal x757=((cj9)*(npx));
IkReal x758=((npx)*(sj10));
IkReal x759=((npy)*(x753));
IkReal x760=((IkReal(1.00000000000000))*(x753));
IkReal x761=((IkReal(2.41309350707268e-12))*(x753));
IkReal x762=((npy)*(x752));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x752)))+(((IkReal(-1.00000000000000))*(x758)))+(((IkReal(-1.00000000000000))*(npy)*(x756))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x758)*(x760)))+(((IkReal(-1.00000000000000))*(x752)*(x755)))+(((IkReal(-1.00000000000000))*(x752)*(x756)*(x757)))+(((IkReal(-1.00000000000000))*(x756)*(x759)))+(((x754)*(x762))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj9)*(x761)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-0.250000000390712))*(cj9))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((x752)*(x758)))+(((IkReal(-1.00000000000000))*(x755)*(x760)))+(((cj10)*(x762)))+(((IkReal(-1.00000000000000))*(x753)*(x756)*(x757)))+(((x754)*(x759))));
evalcond[4]=((((IkReal(-1.00000000000000))*(npx)*(x756)))+(((IkReal(0.250000000390712))*(sj9)))+(((IkReal(-1.00000000000000))*(cj9)*(x761)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x763=((IkReal(0.0450000000000000))*(sj7));
IkReal x764=((IkReal(0.550000000000000))*(cj7));
IkReal x765=((cj10)*(npx));
IkReal x766=((npy)*(sj10));
IkReal x767=((x764)+(x763));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(((IkReal(-1.00000000000000))*(x766)))+(x765)+(x767));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(x767)))+(((IkReal(-1.00000000000000))*(x765)))+(x766));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign(((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(-1.00000000000000))+(cj7)+(((IkReal(-12.2222222222222))*(sj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x768=((IkReal(1.00000000000000))*(cj10));
IkReal x769=((npy)*(sj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x768)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(x769))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x768)))+(x769));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x770=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x770));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x770))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x771=IKcos(j8);
IkReal x772=((IkReal(1.00000000000000))*(x771));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(npy)*(x772)))+(((IkReal(-1.00000000000000))*(npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x772))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x773=((npy)*(sj10));
IkReal x774=((cj10)*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(6.11991262292145))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.0624999997657902))+(pp));
evalcond[2]=((IkReal(-0.250000000390712))+(x774)+(((IkReal(-1.00000000000000))*(x773))));
evalcond[3]=((IkReal(0.250000000390712))+(x773)+(((IkReal(-1.00000000000000))*(x774))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-414405822679.079))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))))+IKsqr(((IkReal(-414405822679.079))*(npz)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IkReal(414405822679.079))*(cj10)*(npy)))+(((IkReal(414405822679.079))*(npx)*(sj10)))), ((IkReal(-414405822679.079))*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x775=IKsin(j8);
IkReal x776=IKcos(j8);
IkReal x777=((IkReal(1.00000000000000))*(npz));
IkReal x778=((npx)*(sj10));
IkReal x779=((IkReal(1.00000000000000))*(cj10)*(npy));
evalcond[0]=((((IkReal(2.41309350707268e-12))*(x775)))+(((IkReal(-1.00000000000000))*(x779)))+(((IkReal(-1.00000000000000))*(x778))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x775)*(x777)))+(((IkReal(-1.00000000000000))*(x776)*(x778)))+(((IkReal(-1.00000000000000))*(x776)*(x779))));
evalcond[2]=((((IkReal(-2.41309350707268e-12))*(x776)))+(((IkReal(-1.00000000000000))*(x777))));
evalcond[3]=((IkReal(-2.41309350707268e-12))+(((IkReal(-1.00000000000000))*(x776)*(x777)))+(((x775)*(x778)))+(((cj10)*(npy)*(x775))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x780=((IkReal(1.00000000000000))*(cj10));
IkReal x781=((npy)*(sj10));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(npy)*(x780)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))));
evalcond[3]=((IkReal(0.850000000000000))+(((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(x781))));
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npx)*(x780)))+(x781));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((IkReal(-1.00000000000000))*(npz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x782=IKatan2(((IkReal(-1.00000000000000))*(npz)), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x782));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x782))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x783=IKcos(j8);
IkReal x784=((IkReal(1.00000000000000))*(x783));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(npy)*(x784)))+(((IkReal(-1.00000000000000))*(npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x784))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)))))), ((gconst3)*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x785=IKsin(j8);
IkReal x786=IKcos(j8);
IkReal x787=((IkReal(0.550000000000000))*(sj7));
IkReal x788=((cj10)*(npy));
IkReal x789=((IkReal(1.00000000000000))*(npz));
IkReal x790=((npx)*(sj10));
IkReal x791=((IkReal(1.00000000000000))*(x786));
IkReal x792=((IkReal(0.0450000000000000))*(x786));
IkReal x793=((IkReal(0.0450000000000000))*(x785));
evalcond[0]=((((x785)*(x787)))+(x793)+(((IkReal(-1.00000000000000))*(x790)))+(((IkReal(-1.00000000000000))*(x788)))+(((IkReal(-1.00000000000000))*(cj7)*(x793))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x788)*(x791)))+(((IkReal(-1.00000000000000))*(x790)*(x791)))+(((IkReal(-1.00000000000000))*(x785)*(x789))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x792)))+(((IkReal(-1.00000000000000))*(x789)))+(((cj7)*(x792)))+(((IkReal(-1.00000000000000))*(x786)*(x787))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((x785)*(x790)))+(((x785)*(x788)))+(((IkReal(-1.00000000000000))*(x787)))+(((IkReal(-1.00000000000000))*(x786)*(x789))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x794=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((IkReal(0.550000000000000))*(cj7)))+(((npy)*(sj10))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x794;
evalcond[3]=x794;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x795=((IkReal(1.00000000000000))*(npx));
IkReal x796=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj10)*(x795)))+(((npy)*(sj10))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj10)*(x795)))+(((IkReal(-1.00000000000000))*(cj10)*(npy))));
evalcond[3]=x796;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x796;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x797=IKatan2(npz, ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x797));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x797))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x798=IKcos(j8);
IkReal x799=((IkReal(1.00000000000000))*(x798));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(npy)*(x799)))+(((npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x799))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x800=((IkReal(1.00000000000000))*(npx));
IkReal x801=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(cj10)*(x800)))+(((npy)*(sj10))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x800))));
evalcond[3]=x801;
evalcond[4]=((IkReal(-1.00000000000000))*(npz));
evalcond[5]=x801;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(npz) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x802=IKatan2(npz, ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x802));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x802))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x803=IKcos(j8);
IkReal x804=((IkReal(1.00000000000000))*(x803));
evalcond[0]=((((npz)*(IKsin(j8))))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x804)))+(((IkReal(-1.00000000000000))*(cj10)*(npy)*(x804))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst4)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(npz))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst4)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst4)*(npz)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x805=IKsin(j8);
IkReal x806=IKcos(j8);
IkReal x807=((IkReal(0.550000000000000))*(sj7));
IkReal x808=((IkReal(0.0450000000000000))*(x806));
IkReal x809=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x810=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x811=((IkReal(0.0450000000000000))*(x805));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj7)*(x811)))+(((IkReal(-1.00000000000000))*(x810)))+(((x805)*(x807)))+(x811)+(((IkReal(-1.00000000000000))*(x809))));
evalcond[1]=((((npz)*(x805)))+(((IkReal(-1.00000000000000))*(x806)*(x809)))+(((IkReal(-1.00000000000000))*(x806)*(x810))));
evalcond[2]=((((x806)*(x807)))+(((IkReal(-1.00000000000000))*(npz)))+(x808)+(((IkReal(-1.00000000000000))*(cj7)*(x808))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((npx)*(sj10)*(x805)))+(((npz)*(x806)))+(((cj10)*(npy)*(x805)))+(((IkReal(-1.00000000000000))*(x807))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x812=((IkReal(0.300000000000000))+(((IkReal(0.0450000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(0.550000000000000))*(cj7))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=x812;
evalcond[3]=x812;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x813=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x814=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj10)*(npy)))+(((IkReal(-1.00000000000000))*(sj10)*(x814))));
evalcond[3]=x813;
evalcond[4]=x813;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x814)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x815=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x815));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x815))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x816=IKcos(j8);
IkReal x817=IKsin(j8);
IkReal x818=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npx)*(x817)*(x818)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x816)))+(((IkReal(-1.00000000000000))*(npy)*(x816)*(x818)))+(((npy)*(sj10)*(x817))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x819=((IkReal(0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
IkReal x820=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj10)*(x820)))+(((IkReal(-1.00000000000000))*(cj10)*(npy))));
evalcond[3]=x819;
evalcond[4]=x819;
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x820)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x821=IKatan2(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x821));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x821))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x822=IKcos(j8);
IkReal x823=IKsin(j8);
IkReal x824=((IkReal(1.00000000000000))*(cj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x822)*(x824)))+(((npy)*(sj10)*(x823)))+(((IkReal(-1.00000000000000))*(npx)*(x823)*(x824)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x822))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst5)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst5)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst5)*(((((IkReal(-1.00000000000000))*(cj10)*(npx)))+(((npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x825=IKsin(j8);
IkReal x826=IKcos(j8);
IkReal x827=((IkReal(0.550000000000000))*(sj7));
IkReal x828=((npy)*(sj10));
IkReal x829=((IkReal(1.00000000000000))*(npx));
IkReal x830=((IkReal(0.0450000000000000))*(x826));
IkReal x831=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x832=((cj10)*(x825));
IkReal x833=((IkReal(0.0450000000000000))*(x825));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj10)*(x829)))+(((x825)*(x827)))+(((IkReal(-1.00000000000000))*(x831)))+(((IkReal(-1.00000000000000))*(cj7)*(x833)))+(x833));
evalcond[1]=((((IkReal(-1.00000000000000))*(x826)*(x831)))+(((IkReal(-1.00000000000000))*(sj10)*(x826)*(x829)))+(((x825)*(x828)))+(((IkReal(-1.00000000000000))*(x829)*(x832))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj7)))+(((IkReal(-1.00000000000000))*(cj10)*(x826)*(x829)))+(((IkReal(-1.00000000000000))*(x827)))+(((npy)*(x832)))+(((npx)*(sj10)*(x825)))+(((x826)*(x828))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj10)*(x829)))+(((IkReal(-1.00000000000000))*(x826)*(x827)))+(((cj7)*(x830)))+(((IkReal(-1.00000000000000))*(x830)))+(x828));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x834=((IkReal(0.0450000000000000))*(sj7));
IkReal x835=((IkReal(0.550000000000000))*(cj7));
IkReal x836=((x834)+(x835));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.396550000000000))+(((IkReal(-0.0765000000000000))*(sj7)))+(((IkReal(-0.325950000000000))*(cj7)))+(pp));
evalcond[2]=((IkReal(0.300000000000000))+(npz)+(x836));
evalcond[3]=((IkReal(-0.300000000000000))+(((IkReal(-1.00000000000000))*(npz)))+(((IkReal(-1.00000000000000))*(x836))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(((IkReal(0.0450000000000000))+(((IkReal(-0.0450000000000000))*(cj7)))+(((IkReal(0.550000000000000))*(sj7)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(12.2222222222222))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[6];
IkReal x837=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj10)*(x837)))+(((IkReal(-1.00000000000000))*(cj10)*(npy))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x837)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x838=IKatan2(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x838));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x838))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x839=IKsin(j8);
IkReal x840=IKcos(j8);
IkReal x841=((IkReal(1.00000000000000))*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj10)*(x840)*(x841)))+(((IkReal(-1.00000000000000))*(sj10)*(x839)*(x841)))+(((cj10)*(npx)*(x839)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x840))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
IkReal x842=((IkReal(1.00000000000000))*(npx));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j7)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.722500000000000))+(pp));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj10)*(x842)))+(((IkReal(-1.00000000000000))*(cj10)*(npy))));
evalcond[3]=((IkReal(0.850000000000000))+(npz));
evalcond[4]=((IkReal(-0.850000000000000))+(((IkReal(-1.00000000000000))*(npz))));
evalcond[5]=((((IkReal(-1.00000000000000))*(cj10)*(x842)))+(((npy)*(sj10))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj10)*(npy)))+(((npx)*(sj10))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x843=IKatan2(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))), ((((cj10)*(npy)))+(((npx)*(sj10)))));
j8array[0]=((IkReal(-1.00000000000000))*(x843));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x843))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
IkReal x844=IKsin(j8);
IkReal x845=IKcos(j8);
IkReal x846=((IkReal(1.00000000000000))*(npy));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj10)*(x844)*(x846)))+(((IkReal(-1.00000000000000))*(cj10)*(x845)*(x846)))+(((cj10)*(npx)*(x844)))+(((IkReal(-1.00000000000000))*(npx)*(sj10)*(x845))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
if( IKabs(((gconst6)*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((gconst6)*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((gconst6)*(((((cj10)*(npx)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[4];
IkReal x847=IKsin(j8);
IkReal x848=IKcos(j8);
IkReal x849=((IkReal(0.550000000000000))*(sj7));
IkReal x850=((cj10)*(npx));
IkReal x851=((npy)*(sj10));
IkReal x852=((IkReal(0.0450000000000000))*(x848));
IkReal x853=((IkReal(1.00000000000000))*(cj10)*(npy));
IkReal x854=((IkReal(1.00000000000000))*(npx)*(sj10));
IkReal x855=((IkReal(0.0450000000000000))*(x847));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj7)*(x855)))+(((IkReal(-1.00000000000000))*(x853)))+(((IkReal(-1.00000000000000))*(x854)))+(((x847)*(x849)))+(x855));
evalcond[1]=((((IkReal(-1.00000000000000))*(x847)*(x851)))+(((x847)*(x850)))+(((IkReal(-1.00000000000000))*(x848)*(x854)))+(((IkReal(-1.00000000000000))*(x848)*(x853))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((x848)*(x850)))+(((IkReal(0.0450000000000000))*(cj7)))+(((npx)*(sj10)*(x847)))+(((IkReal(-1.00000000000000))*(x849)))+(((cj10)*(npy)*(x847)))+(((IkReal(-1.00000000000000))*(x848)*(x851))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj7)*(x852)))+(((IkReal(-1.00000000000000))*(x850)))+(((x848)*(x849)))+(x852)+(x851));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x856=((cj10)*(npy));
IkReal x857=((npz)*(sj9));
IkReal x858=((IkReal(0.0450000000000000))*(cj7));
IkReal x859=((IkReal(0.550000000000000))*(sj7));
IkReal x860=((npx)*(sj10));
IkReal x861=((cj9)*(npx)*(npy));
IkReal x862=((cj10)*(cj9)*(sj10));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x858)))+(x859))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x858)))+(x859)))):(IkReal)1.0e30))*(((x856)+(x860))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-0.0450000000000000))*(x856)))+(((x858)*(x860)))+(((x856)*(x858)))+(((IkReal(-1.00000000000000))*(x859)*(x860)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-0.0450000000000000))*(x860))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(x856)))+(((x858)*(x860)))+(((x856)*(x858)))+(((IkReal(-1.00000000000000))*(x859)*(x860)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-0.0450000000000000))*(x860)))))):(IkReal)1.0e30))*(((((cj10)*(cj9)*(npx)*(x860)))+(((cj10)*(cj9)*(npx)*(x856)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x856)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x860)))+(((x856)*(x857)))+(((x857)*(x860))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x858)))+(x859))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x858)))+(x859)))):(IkReal)1.0e30))*(((x856)+(x860)))))+IKsqr(((((IKabs(((((IkReal(-0.0450000000000000))*(x856)))+(((x858)*(x860)))+(((x856)*(x858)))+(((IkReal(-1.00000000000000))*(x859)*(x860)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-0.0450000000000000))*(x860))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(x856)))+(((x858)*(x860)))+(((x856)*(x858)))+(((IkReal(-1.00000000000000))*(x859)*(x860)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-0.0450000000000000))*(x860)))))):(IkReal)1.0e30))*(((((cj10)*(cj9)*(npx)*(x860)))+(((cj10)*(cj9)*(npx)*(x856)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x856)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x860)))+(((x856)*(x857)))+(((x857)*(x860)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x858)))+(x859))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x858)))+(x859)))):(IkReal)1.0e30))*(((x856)+(x860)))), ((((IKabs(((((IkReal(-0.0450000000000000))*(x856)))+(((x858)*(x860)))+(((x856)*(x858)))+(((IkReal(-1.00000000000000))*(x859)*(x860)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-0.0450000000000000))*(x860))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(x856)))+(((x858)*(x860)))+(((x856)*(x858)))+(((IkReal(-1.00000000000000))*(x859)*(x860)))+(((IkReal(-1.00000000000000))*(x856)*(x859)))+(((IkReal(-0.0450000000000000))*(x860)))))):(IkReal)1.0e30))*(((((cj10)*(cj9)*(npx)*(x860)))+(((cj10)*(cj9)*(npx)*(x856)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x856)))+(((IkReal(-1.00000000000000))*(cj9)*(npy)*(sj10)*(x860)))+(((x856)*(x857)))+(((x857)*(x860)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x863=IKsin(j8);
IkReal x864=IKcos(j8);
IkReal x865=((npy)*(sj10));
IkReal x866=((IkReal(1.00000000000000))*(cj10));
IkReal x867=((IkReal(0.550000000000000))*(sj7));
IkReal x868=((IkReal(1.00000000000000))*(npz));
IkReal x869=((IkReal(0.550000000000000))*(cj7));
IkReal x870=((IkReal(0.0450000000000000))*(cj7));
IkReal x871=((IkReal(0.0450000000000000))*(sj7));
IkReal x872=((cj9)*(x864));
IkReal x873=((sj9)*(x864));
IkReal x874=((cj9)*(x863));
IkReal x875=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(0.0450000000000000))*(x863)))+(((x863)*(x867)))+(((IkReal(-1.00000000000000))*(npy)*(x866)))+(((IkReal(-1.00000000000000))*(x863)*(x870)))+(((IkReal(-1.00000000000000))*(x875))));
evalcond[1]=((((IkReal(-1.00000000000000))*(npy)*(x864)*(x866)))+(((IkReal(-1.00000000000000))*(npx)*(x866)*(x874)))+(((IkReal(-1.00000000000000))*(x864)*(x875)))+(((IkReal(-1.00000000000000))*(sj9)*(x863)*(x868)))+(((x865)*(x874))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x868)))+(((x870)*(x873)))+(((cj9)*(x871)))+(((cj9)*(x869)))+(((IkReal(-1.00000000000000))*(x867)*(x873)))+(((IkReal(0.300000000000000))*(cj9)))+(((IkReal(-0.0450000000000000))*(x873))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(x867)))+(((npx)*(sj10)*(x863)))+(((cj10)*(npy)*(x863)))+(((IkReal(-1.00000000000000))*(x868)*(x873)))+(((IkReal(-1.00000000000000))*(npx)*(x866)*(x872)))+(x870)+(((x865)*(x872))));
evalcond[4]=((((x870)*(x872)))+(((IkReal(-1.00000000000000))*(sj9)*(x871)))+(((IkReal(-1.00000000000000))*(sj9)*(x869)))+(((IkReal(-1.00000000000000))*(npx)*(x866)))+(((IkReal(-1.00000000000000))*(x867)*(x872)))+(x865)+(((IkReal(-0.0450000000000000))*(x872)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x876=((IkReal(0.0450000000000000))*(cj7));
IkReal x877=((IkReal(0.550000000000000))*(sj7));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(x877)+(((IkReal(-1.00000000000000))*(x876))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(x877)+(((IkReal(-1.00000000000000))*(x876)))))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-0.0450000000000000))*(cj9)))+(((cj9)*(x876)))+(((IkReal(-1.00000000000000))*(cj9)*(x877))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(cj9)))+(((cj9)*(x876)))+(((IkReal(-1.00000000000000))*(cj9)*(x877)))))):(IkReal)1.0e30))*(((((cj10)*(npx)))+(((IkReal(0.0450000000000000))*(sj7)*(sj9)))+(((IkReal(0.300000000000000))*(sj9)))+(((IkReal(0.550000000000000))*(cj7)*(sj9)))+(((IkReal(-1.00000000000000))*(npy)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(x877)+(((IkReal(-1.00000000000000))*(x876))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(x877)+(((IkReal(-1.00000000000000))*(x876)))))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))))+IKsqr(((((IKabs(((((IkReal(-0.0450000000000000))*(cj9)))+(((cj9)*(x876)))+(((IkReal(-1.00000000000000))*(cj9)*(x877))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(cj9)))+(((cj9)*(x876)))+(((IkReal(-1.00000000000000))*(cj9)*(x877)))))):(IkReal)1.0e30))*(((((cj10)*(npx)))+(((IkReal(0.0450000000000000))*(sj7)*(sj9)))+(((IkReal(0.300000000000000))*(sj9)))+(((IkReal(0.550000000000000))*(cj7)*(sj9)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(x877)+(((IkReal(-1.00000000000000))*(x876))))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(x877)+(((IkReal(-1.00000000000000))*(x876)))))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((((IKabs(((((IkReal(-0.0450000000000000))*(cj9)))+(((cj9)*(x876)))+(((IkReal(-1.00000000000000))*(cj9)*(x877))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(cj9)))+(((cj9)*(x876)))+(((IkReal(-1.00000000000000))*(cj9)*(x877)))))):(IkReal)1.0e30))*(((((cj10)*(npx)))+(((IkReal(0.0450000000000000))*(sj7)*(sj9)))+(((IkReal(0.300000000000000))*(sj9)))+(((IkReal(0.550000000000000))*(cj7)*(sj9)))+(((IkReal(-1.00000000000000))*(npy)*(sj10)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x878=IKsin(j8);
IkReal x879=IKcos(j8);
IkReal x880=((npy)*(sj10));
IkReal x881=((IkReal(1.00000000000000))*(cj10));
IkReal x882=((IkReal(0.550000000000000))*(sj7));
IkReal x883=((IkReal(1.00000000000000))*(npz));
IkReal x884=((IkReal(0.550000000000000))*(cj7));
IkReal x885=((IkReal(0.0450000000000000))*(cj7));
IkReal x886=((IkReal(0.0450000000000000))*(sj7));
IkReal x887=((cj9)*(x879));
IkReal x888=((sj9)*(x879));
IkReal x889=((cj9)*(x878));
IkReal x890=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(0.0450000000000000))*(x878)))+(((IkReal(-1.00000000000000))*(npy)*(x881)))+(((IkReal(-1.00000000000000))*(x890)))+(((x878)*(x882)))+(((IkReal(-1.00000000000000))*(x878)*(x885))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x879)*(x890)))+(((x880)*(x889)))+(((IkReal(-1.00000000000000))*(npx)*(x881)*(x889)))+(((IkReal(-1.00000000000000))*(sj9)*(x878)*(x883)))+(((IkReal(-1.00000000000000))*(npy)*(x879)*(x881))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x882)*(x888)))+(((IkReal(-0.0450000000000000))*(x888)))+(((IkReal(-1.00000000000000))*(x883)))+(((IkReal(0.300000000000000))*(cj9)))+(((cj9)*(x886)))+(((cj9)*(x884)))+(((x885)*(x888))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((npx)*(sj10)*(x878)))+(((IkReal(-1.00000000000000))*(x883)*(x888)))+(((cj10)*(npy)*(x878)))+(((x880)*(x887)))+(((IkReal(-1.00000000000000))*(npx)*(x881)*(x887)))+(((IkReal(-1.00000000000000))*(x882)))+(x885));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj9)*(x886)))+(((IkReal(-1.00000000000000))*(sj9)*(x884)))+(((IkReal(-1.00000000000000))*(x882)*(x887)))+(((IkReal(-1.00000000000000))*(npx)*(x881)))+(((IkReal(-0.0450000000000000))*(x887)))+(x880)+(((x885)*(x887)))+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x891=((IkReal(0.0450000000000000))*(cj7));
IkReal x892=((IkReal(0.550000000000000))*(sj7));
if( IKabs(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(x892))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(x892)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x892)))+(((sj9)*(x891))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x892)))+(((sj9)*(x891)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj9)))+(npz)+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(x892))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(x892)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))))+IKsqr(((((IKabs(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x892)))+(((sj9)*(x891))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x892)))+(((sj9)*(x891)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj9)))+(npz)+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j8array[0]=IKatan2(((((IKabs(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(x892))) != 0)?((IkReal)1/(((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x891)))+(x892)))):(IkReal)1.0e30))*(((((cj10)*(npy)))+(((npx)*(sj10)))))), ((((IKabs(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x892)))+(((sj9)*(x891))))) != 0)?((IkReal)1/(((((IkReal(-0.0450000000000000))*(sj9)))+(((IkReal(-1.00000000000000))*(sj9)*(x892)))+(((sj9)*(x891)))))):(IkReal)1.0e30))*(((((IkReal(-0.300000000000000))*(cj9)))+(npz)+(((IkReal(-0.0450000000000000))*(cj9)*(sj7)))+(((IkReal(-0.550000000000000))*(cj7)*(cj9)))))));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[5];
IkReal x893=IKsin(j8);
IkReal x894=IKcos(j8);
IkReal x895=((npy)*(sj10));
IkReal x896=((IkReal(1.00000000000000))*(cj10));
IkReal x897=((IkReal(0.550000000000000))*(sj7));
IkReal x898=((IkReal(1.00000000000000))*(npz));
IkReal x899=((IkReal(0.550000000000000))*(cj7));
IkReal x900=((IkReal(0.0450000000000000))*(cj7));
IkReal x901=((IkReal(0.0450000000000000))*(sj7));
IkReal x902=((cj9)*(x894));
IkReal x903=((sj9)*(x894));
IkReal x904=((cj9)*(x893));
IkReal x905=((IkReal(1.00000000000000))*(npx)*(sj10));
evalcond[0]=((((IkReal(-1.00000000000000))*(npy)*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x900)))+(((IkReal(-1.00000000000000))*(x905)))+(((x893)*(x897)))+(((IkReal(0.0450000000000000))*(x893))));
evalcond[1]=((((x895)*(x904)))+(((IkReal(-1.00000000000000))*(npy)*(x894)*(x896)))+(((IkReal(-1.00000000000000))*(sj9)*(x893)*(x898)))+(((IkReal(-1.00000000000000))*(x894)*(x905)))+(((IkReal(-1.00000000000000))*(npx)*(x896)*(x904))));
evalcond[2]=((((cj9)*(x901)))+(((IkReal(-1.00000000000000))*(x897)*(x903)))+(((IkReal(-1.00000000000000))*(x898)))+(((x900)*(x903)))+(((IkReal(-0.0450000000000000))*(x903)))+(((cj9)*(x899)))+(((IkReal(0.300000000000000))*(cj9))));
evalcond[3]=((IkReal(-0.0450000000000000))+(((x895)*(x902)))+(((IkReal(-1.00000000000000))*(x897)))+(((IkReal(-1.00000000000000))*(x898)*(x903)))+(((cj10)*(npy)*(x893)))+(((npx)*(sj10)*(x893)))+(x900)+(((IkReal(-1.00000000000000))*(npx)*(x896)*(x902))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj9)*(x899)))+(((IkReal(-1.00000000000000))*(x897)*(x902)))+(((IkReal(-1.00000000000000))*(npx)*(x896)))+(((x900)*(x902)))+(((IkReal(-1.00000000000000))*(sj9)*(x901)))+(((IkReal(-0.0450000000000000))*(x902)))+(x895)+(((IkReal(-0.300000000000000))*(sj9))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x71=((IkReal(1.00000000000000))*(sj7));
IkReal x72=((IkReal(1.00000000000000))*(sj10));
IkReal x73=((IkReal(1.00000000000000))*(cj7));
IkReal x74=((IkReal(1.00000000000000))*(cj9));
IkReal x75=((IkReal(-1.00000000000000))*(cj9));
IkReal x76=((((IkReal(-1.00000000000000))*(r01)*(x72)))+(((cj10)*(r00))));
IkReal x77=((((r00)*(sj10)))+(((cj10)*(r01))));
IkReal x78=((((IkReal(-1.00000000000000))*(r11)*(x72)))+(((cj10)*(r10))));
IkReal x79=((((r10)*(sj10)))+(((cj10)*(r11))));
IkReal x80=((((IkReal(-1.00000000000000))*(r21)*(x72)))+(((cj10)*(r20))));
IkReal x81=((((r20)*(sj10)))+(((cj10)*(r21))));
IkReal x82=((sj9)*(x76));
IkReal x83=((sj9)*(x78));
IkReal x84=((sj9)*(x80));
IkReal x85=((sj8)*(x81));
IkReal x86=((sj8)*(x77));
IkReal x87=((sj8)*(x79));
IkReal x88=((((r02)*(sj9)))+(((cj9)*(x76))));
IkReal x89=((((r12)*(sj9)))+(((cj9)*(x78))));
IkReal x90=((((r22)*(sj9)))+(((cj9)*(x80))));
IkReal x91=((cj8)*(x88));
IkReal x92=((cj8)*(x89));
IkReal x93=((cj8)*(x90));
new_r00=((((cj7)*(((x91)+(((IkReal(-1.00000000000000))*(x86)))))))+(((IkReal(-1.00000000000000))*(x71)*(((x82)+(((IkReal(-1.00000000000000))*(r02)*(x74))))))));
new_r01=((((sj8)*(x88)))+(((cj8)*(x77))));
new_r02=((((IkReal(-1.00000000000000))*(x71)*(((x91)+(((IkReal(-1.00000000000000))*(x86)))))))+(((IkReal(-1.00000000000000))*(x73)*(((((r02)*(x75)))+(x82))))));
new_r10=((((IkReal(-1.00000000000000))*(x71)*(((x83)+(((IkReal(-1.00000000000000))*(r12)*(x74)))))))+(((cj7)*(((x92)+(((IkReal(-1.00000000000000))*(x87))))))));
new_r11=((((sj8)*(x89)))+(((cj8)*(x79))));
new_r12=((((IkReal(-1.00000000000000))*(x71)*(((x92)+(((IkReal(-1.00000000000000))*(x87)))))))+(((IkReal(-1.00000000000000))*(x73)*(((x83)+(((r12)*(x75))))))));
new_r20=((((IkReal(-1.00000000000000))*(x71)*(((((IkReal(-1.00000000000000))*(r22)*(x74)))+(x84)))))+(((cj7)*(((x93)+(((IkReal(-1.00000000000000))*(x85))))))));
new_r21=((((sj8)*(x90)))+(((cj8)*(x81))));
new_r22=((((IkReal(-1.00000000000000))*(x73)*(((((r22)*(x75)))+(x84)))))+(((IkReal(-1.00000000000000))*(x71)*(((x93)+(((IkReal(-1.00000000000000))*(x85))))))));
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
cj5array[0]=new_r22;
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst11;
gconst11=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst10;
gconst10=IKsign(((((IkReal(2.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(2.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x94=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x94));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x94))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x95=IKcos(j4);
IkReal x96=IKsin(j4);
IkReal x97=((IkReal(0.707106781186548))*(new_r02));
IkReal x98=((new_r12)*(x95));
IkReal x99=((IkReal(0.707106781186548))*(new_r12)*(x96));
IkReal x100=((x95)*(x97));
IkReal x101=((IkReal(0.707106781186548))*(x98));
IkReal x102=((x96)*(x97));
IkReal x103=((x100)+(x99));
evalcond[0]=((x98)+(((IkReal(-1.00000000000000))*(new_r02)*(x96))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x103)))+(((IkReal(-1.00000000000000))*(x102)))+(x101));
evalcond[2]=((((IkReal(-1.00000000000000))*(x102)))+(x101)+(x103));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x104=((IkReal(0.707106781186548))*(sj4));
IkReal x105=((IkReal(0.707106781186548))*(cj4));
IkReal x106=((new_r10)*(x104));
IkReal x107=((new_r00)*(x105));
IkReal x108=((new_r11)*(x104));
IkReal x109=((new_r01)*(x105));
IkReal x110=((x108)+(x109));
IkReal x111=((x106)+(x107));
if( IKabs(((((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x110))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x110)))+(x111))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x110)))))+IKsqr(((((IkReal(-1.00000000000000))*(x110)))+(x111)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x110)))), ((((IkReal(-1.00000000000000))*(x110)))+(x111)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x112=IKsin(j6);
IkReal x113=IKcos(j6);
IkReal x114=((cj4)*(new_r00));
IkReal x115=((cj4)*(new_r01));
IkReal x116=((IkReal(0.707106781186548))*(sj4));
IkReal x117=((IkReal(0.707106781186548))*(cj4));
IkReal x118=((IkReal(1.00000000000000))*(sj4));
IkReal x119=((IkReal(0.707106781186548))*(x113));
IkReal x120=((IkReal(0.707106781186548))*(x112));
IkReal x121=((new_r11)*(x116));
IkReal x122=((IkReal(0.707106781186548))*(x115));
IkReal x123=((new_r11)*(x117));
IkReal x124=((new_r01)*(x116));
IkReal x125=((new_r10)*(x116));
IkReal x126=((IkReal(0.707106781186548))*(x114));
IkReal x127=((new_r10)*(x117));
IkReal x128=((new_r00)*(x116));
IkReal x129=((IkReal(1.00000000000000))*(x113));
IkReal x130=((x120)+(x119));
IkReal x131=((x126)+(x125));
IkReal x132=((x122)+(x121));
evalcond[0]=((((new_r10)*(sj4)))+(((IkReal(-1.00000000000000))*(x119)))+(x120)+(x114));
evalcond[1]=((((new_r11)*(sj4)))+(x130)+(x115));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x118)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x130))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x118)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x119)))+(x120));
evalcond[4]=((((IkReal(-1.00000000000000))*(x128)))+(x127)+(((IkReal(-1.00000000000000))*(x131)))+(((IkReal(-1.00000000000000))*(x112))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x129)))+(((IkReal(-1.00000000000000))*(x124)))+(x123)+(((IkReal(-1.00000000000000))*(x132))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x124)))+(x132)+(x123)+(x112));
evalcond[7]=((((IkReal(-1.00000000000000))*(x129)))+(((IkReal(-1.00000000000000))*(x128)))+(x131)+(x127));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x133=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x133;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x133;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x134=IKatan2(new_r02, new_r12);
j4array[0]=((IkReal(-1.00000000000000))*(x134));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x134))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[3];
IkReal x135=IKcos(j4);
IkReal x136=IKsin(j4);
IkReal x137=((IkReal(0.707106781186548))*(new_r02));
IkReal x138=((new_r12)*(x135));
IkReal x139=((x136)*(x137));
IkReal x140=((IkReal(0.707106781186548))*(new_r12)*(x136));
IkReal x141=((x135)*(x137));
IkReal x142=((IkReal(0.707106781186548))*(x138));
IkReal x143=((x140)+(x141));
evalcond[0]=((x138)+(((IkReal(-1.00000000000000))*(new_r02)*(x136))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x143)))+(x142)+(((IkReal(-1.00000000000000))*(x139))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x143)))+(((IkReal(-1.00000000000000))*(x142)))+(x139));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x144=((IkReal(0.707106781186548))*(sj4));
IkReal x145=((IkReal(0.707106781186548))*(cj4));
IkReal x146=((new_r11)*(x144));
IkReal x147=((new_r10)*(x144));
IkReal x148=((new_r00)*(x145));
IkReal x149=((new_r01)*(x145));
IkReal x150=((x146)+(x149));
IkReal x151=((x147)+(x148));
if( IKabs(((((IkReal(-1.00000000000000))*(x151)))+(x150))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x150)))+(((IkReal(-1.00000000000000))*(x151))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x151)))+(x150)))+IKsqr(((((IkReal(-1.00000000000000))*(x150)))+(((IkReal(-1.00000000000000))*(x151)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x151)))+(x150)), ((((IkReal(-1.00000000000000))*(x150)))+(((IkReal(-1.00000000000000))*(x151)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x152=IKsin(j6);
IkReal x153=IKcos(j6);
IkReal x154=((cj4)*(new_r00));
IkReal x155=((cj4)*(new_r01));
IkReal x156=((IkReal(0.707106781186548))*(sj4));
IkReal x157=((IkReal(0.707106781186548))*(cj4));
IkReal x158=((IkReal(1.00000000000000))*(sj4));
IkReal x159=((IkReal(0.707106781186548))*(x153));
IkReal x160=((IkReal(0.707106781186548))*(x152));
IkReal x161=((new_r01)*(x156));
IkReal x162=((new_r11)*(x156));
IkReal x163=((IkReal(0.707106781186548))*(x155));
IkReal x164=((new_r11)*(x157));
IkReal x165=((new_r00)*(x156));
IkReal x166=((IkReal(1.00000000000000))*(x153));
IkReal x167=((new_r10)*(x156));
IkReal x168=((IkReal(0.707106781186548))*(x154));
IkReal x169=((new_r10)*(x157));
IkReal x170=((x168)+(x167));
IkReal x171=((x162)+(x163));
IkReal x172=((x160)+(x159));
evalcond[0]=((((new_r10)*(sj4)))+(x172)+(x154));
evalcond[1]=((((IkReal(-1.00000000000000))*(x160)))+(((new_r11)*(sj4)))+(x155)+(x159));
evalcond[2]=((((IkReal(-1.00000000000000))*(x160)))+(((cj4)*(new_r10)))+(x159)+(((IkReal(-1.00000000000000))*(new_r00)*(x158))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x172)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x158))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x165)))+(((IkReal(-1.00000000000000))*(x170)))+(x169)+(((IkReal(-1.00000000000000))*(x152))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x166)))+(((IkReal(-1.00000000000000))*(x161)))+(((IkReal(-1.00000000000000))*(x171)))+(x164));
evalcond[6]=((((IkReal(-1.00000000000000))*(x164)))+(((IkReal(-1.00000000000000))*(x171)))+(x161)+(x152));
evalcond[7]=((((IkReal(-1.00000000000000))*(x169)))+(((IkReal(-1.00000000000000))*(x166)))+(((IkReal(-1.00000000000000))*(x170)))+(x165));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x173=((IkReal(1.41421356237310))*(sj5));
IkReal x174=((new_r12)*(x173));
IkReal x175=((new_r02)*(x173));
if( IKabs(((gconst10)*(((((IkReal(-1.00000000000000))*(x175)))+(x174))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst10)*(((x175)+(x174))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst10)*(((((IkReal(-1.00000000000000))*(x175)))+(x174)))), ((gconst10)*(((x175)+(x174)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[7];
IkReal x176=IKcos(j4);
IkReal x177=((IkReal(0.707106781186548))*(sj5));
IkReal x178=IKsin(j4);
IkReal x179=((IkReal(1.00000000000000))*(cj5));
IkReal x180=((IkReal(0.707106781186548))*(cj5));
IkReal x181=((new_r02)*(x176));
IkReal x182=((new_r12)*(x176));
IkReal x183=((new_r12)*(x178));
IkReal x184=((new_r02)*(x178));
IkReal x185=((IkReal(0.707106781186548))*(x182));
evalcond[0]=((((IkReal(-1.00000000000000))*(x177)))+(x181)+(x183));
evalcond[1]=((((IkReal(-1.00000000000000))*(x177)))+(((IkReal(-1.00000000000000))*(x184)))+(x182));
evalcond[2]=((((IkReal(-0.707106781186548))*(x184)))+(((IkReal(-0.707106781186548))*(x181)))+(((IkReal(-0.707106781186548))*(x183)))+(x185));
evalcond[3]=((((x180)*(x183)))+(((x180)*(x181)))+(((x180)*(x182)))+(((IkReal(-1.00000000000000))*(x180)*(x184)))+(((IkReal(-1.00000000000000))*(new_r22)*(sj5))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r00)*(x176)*(x177)))+(((IkReal(-1.00000000000000))*(new_r20)*(x179)))+(((IkReal(-1.00000000000000))*(new_r10)*(x177)*(x178)))+(((IkReal(-1.00000000000000))*(new_r10)*(x176)*(x177)))+(((new_r00)*(x177)*(x178))));
evalcond[5]=((((IkReal(-1.00000000000000))*(new_r11)*(x177)*(x178)))+(((new_r01)*(x177)*(x178)))+(((IkReal(-1.00000000000000))*(new_r01)*(x176)*(x177)))+(((IkReal(-1.00000000000000))*(new_r21)*(x179)))+(((IkReal(-1.00000000000000))*(new_r11)*(x176)*(x177))));
evalcond[6]=((IkReal(1.00000000000000))+(((x177)*(x184)))+(((IkReal(-1.00000000000000))*(x177)*(x182)))+(((IkReal(-1.00000000000000))*(x177)*(x181)))+(((IkReal(-1.00000000000000))*(x177)*(x183)))+(((IkReal(-1.00000000000000))*(new_r22)*(x179))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst12;
gconst12=IKsign(sj5);
dummyeval[0]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x186=((cj4)*(new_r02));
IkReal x187=((cj4)*(new_r12));
IkReal x188=((new_r02)*(sj4));
IkReal x189=((new_r12)*(sj4));
IkReal x190=((IkReal(0.707106781186548))*(x186));
IkReal x191=((IkReal(0.707106781186548))*(x189));
IkReal x192=((IkReal(0.707106781186548))*(x187));
IkReal x193=((IkReal(0.707106781186548))*(x188));
IkReal x194=((x191)+(x190));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x186)+(x189));
evalcond[5]=((((IkReal(-1.00000000000000))*(x188)))+(x187));
evalcond[6]=((((IkReal(-1.00000000000000))*(x193)))+(((IkReal(-1.00000000000000))*(x194)))+(x192));
evalcond[7]=((((IkReal(-1.00000000000000))*(x193)))+(x192)+(x194));
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x195=((IkReal(0.707106781186548))*(sj4));
IkReal x196=((IkReal(0.707106781186548))*(cj4));
IkReal x197=((new_r10)*(x195));
IkReal x198=((new_r00)*(x196));
IkReal x199=((new_r11)*(x195));
IkReal x200=((new_r01)*(x196));
IkReal x201=((x199)+(x200));
IkReal x202=((x198)+(x197));
if( IKabs(((((IkReal(-1.00000000000000))*(x201)))+(((IkReal(-1.00000000000000))*(x202))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x201)))+(x202))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x201)))+(((IkReal(-1.00000000000000))*(x202)))))+IKsqr(((((IkReal(-1.00000000000000))*(x201)))+(x202)))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x201)))+(((IkReal(-1.00000000000000))*(x202)))), ((((IkReal(-1.00000000000000))*(x201)))+(x202)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x203=IKsin(j6);
IkReal x204=IKcos(j6);
IkReal x205=((cj4)*(new_r00));
IkReal x206=((cj4)*(new_r01));
IkReal x207=((IkReal(0.707106781186548))*(sj4));
IkReal x208=((IkReal(0.707106781186548))*(cj4));
IkReal x209=((IkReal(1.00000000000000))*(sj4));
IkReal x210=((IkReal(0.707106781186548))*(x204));
IkReal x211=((IkReal(0.707106781186548))*(x203));
IkReal x212=((new_r11)*(x207));
IkReal x213=((IkReal(0.707106781186548))*(x206));
IkReal x214=((new_r11)*(x208));
IkReal x215=((new_r01)*(x207));
IkReal x216=((new_r10)*(x207));
IkReal x217=((IkReal(0.707106781186548))*(x205));
IkReal x218=((new_r10)*(x208));
IkReal x219=((new_r00)*(x207));
IkReal x220=((IkReal(1.00000000000000))*(x204));
IkReal x221=((x210)+(x211));
IkReal x222=((x216)+(x217));
IkReal x223=((x212)+(x213));
evalcond[0]=((((new_r10)*(sj4)))+(x211)+(x205)+(((IkReal(-1.00000000000000))*(x210))));
evalcond[1]=((((new_r11)*(sj4)))+(x221)+(x206));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r00)*(x209)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x221))));
evalcond[3]=((((cj4)*(new_r11)))+(x211)+(((IkReal(-1.00000000000000))*(x210)))+(((IkReal(-1.00000000000000))*(new_r01)*(x209))));
evalcond[4]=((x218)+(((IkReal(-1.00000000000000))*(x219)))+(((IkReal(-1.00000000000000))*(x203)))+(((IkReal(-1.00000000000000))*(x222))));
evalcond[5]=((x214)+(((IkReal(-1.00000000000000))*(x215)))+(((IkReal(-1.00000000000000))*(x223)))+(((IkReal(-1.00000000000000))*(x220))));
evalcond[6]=((x214)+(x223)+(x203)+(((IkReal(-1.00000000000000))*(x215))));
evalcond[7]=((x218)+(x222)+(((IkReal(-1.00000000000000))*(x219)))+(((IkReal(-1.00000000000000))*(x220))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x224=((IkReal(1.00000000000000))+(new_r22));
IkReal x225=((cj4)*(new_r02));
IkReal x226=((cj4)*(new_r12));
IkReal x227=((new_r02)*(sj4));
IkReal x228=((new_r12)*(sj4));
IkReal x229=((IkReal(0.707106781186548))*(x227));
IkReal x230=((IkReal(0.707106781186548))*(x225));
IkReal x231=((IkReal(0.707106781186548))*(x228));
IkReal x232=((IkReal(0.707106781186548))*(x226));
IkReal x233=((x230)+(x231));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=x224;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((x228)+(x225));
evalcond[5]=((x226)+(((IkReal(-1.00000000000000))*(x227))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x233)))+(x232)+(((IkReal(-1.00000000000000))*(x229))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x233)))+(((IkReal(-1.00000000000000))*(x232)))+(x229));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x224;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
IkReal x234=((IkReal(0.707106781186548))*(sj4));
IkReal x235=((IkReal(0.707106781186548))*(cj4));
IkReal x236=((new_r11)*(x234));
IkReal x237=((new_r10)*(x234));
IkReal x238=((new_r00)*(x235));
IkReal x239=((new_r01)*(x235));
IkReal x240=((x239)+(x236));
IkReal x241=((x238)+(x237));
if( IKabs(((x240)+(((IkReal(-1.00000000000000))*(x241))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x240)))+(((IkReal(-1.00000000000000))*(x241))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x240)+(((IkReal(-1.00000000000000))*(x241)))))+IKsqr(((((IkReal(-1.00000000000000))*(x240)))+(((IkReal(-1.00000000000000))*(x241)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j6array[0]=IKatan2(((x240)+(((IkReal(-1.00000000000000))*(x241)))), ((((IkReal(-1.00000000000000))*(x240)))+(((IkReal(-1.00000000000000))*(x241)))));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[8];
IkReal x242=IKsin(j6);
IkReal x243=IKcos(j6);
IkReal x244=((cj4)*(new_r00));
IkReal x245=((cj4)*(new_r01));
IkReal x246=((IkReal(0.707106781186548))*(sj4));
IkReal x247=((IkReal(0.707106781186548))*(cj4));
IkReal x248=((IkReal(1.00000000000000))*(sj4));
IkReal x249=((IkReal(0.707106781186548))*(x243));
IkReal x250=((IkReal(0.707106781186548))*(x242));
IkReal x251=((new_r01)*(x246));
IkReal x252=((new_r11)*(x246));
IkReal x253=((IkReal(0.707106781186548))*(x245));
IkReal x254=((new_r11)*(x247));
IkReal x255=((new_r00)*(x246));
IkReal x256=((IkReal(1.00000000000000))*(x243));
IkReal x257=((new_r10)*(x246));
IkReal x258=((IkReal(0.707106781186548))*(x244));
IkReal x259=((new_r10)*(x247));
IkReal x260=((x258)+(x257));
IkReal x261=((x252)+(x253));
IkReal x262=((x250)+(x249));
evalcond[0]=((((new_r10)*(sj4)))+(x262)+(x244));
evalcond[1]=((((new_r11)*(sj4)))+(x245)+(x249)+(((IkReal(-1.00000000000000))*(x250))));
evalcond[2]=((((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x248)))+(x249)+(((IkReal(-1.00000000000000))*(x250))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r01)*(x248)))+(((cj4)*(new_r11)))+(((IkReal(-1.00000000000000))*(x262))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x242)))+(x259)+(((IkReal(-1.00000000000000))*(x260)))+(((IkReal(-1.00000000000000))*(x255))));
evalcond[5]=((x254)+(((IkReal(-1.00000000000000))*(x261)))+(((IkReal(-1.00000000000000))*(x251)))+(((IkReal(-1.00000000000000))*(x256))));
evalcond[6]=((x251)+(x242)+(((IkReal(-1.00000000000000))*(x261)))+(((IkReal(-1.00000000000000))*(x254))));
evalcond[7]=((x255)+(((IkReal(-1.00000000000000))*(x260)))+(((IkReal(-1.00000000000000))*(x259)))+(((IkReal(-1.00000000000000))*(x256))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst12)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst12)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst12)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst12)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[10];
IkReal x263=IKsin(j6);
IkReal x264=IKcos(j6);
IkReal x265=((IkReal(1.00000000000000))*(sj5));
IkReal x266=((IkReal(0.707106781186548))*(cj4));
IkReal x267=((IkReal(1.00000000000000))*(sj4));
IkReal x268=((cj5)*(new_r01));
IkReal x269=((IkReal(0.707106781186548))*(cj5));
IkReal x270=((new_r11)*(sj4));
IkReal x271=((IkReal(0.707106781186548))*(sj4));
IkReal x272=((new_r10)*(sj4));
IkReal x273=((IkReal(0.707106781186548))*(x263));
IkReal x274=((IkReal(0.707106781186548))*(x264));
IkReal x275=((IkReal(1.00000000000000))*(x264));
IkReal x276=((x264)*(x269));
IkReal x277=((x263)*(x269));
evalcond[0]=((((sj5)*(x264)))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(x263)*(x265)))+(new_r21));
evalcond[2]=((((cj4)*(new_r00)))+(x272)+(x273)+(((IkReal(-1.00000000000000))*(x276))));
evalcond[3]=((((cj4)*(new_r01)))+(x277)+(x274)+(x270));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r00)*(x267)))+(((cj4)*(new_r10)))+(((IkReal(-1.00000000000000))*(x276)))+(((IkReal(-1.00000000000000))*(x273))));
evalcond[5]=((((IkReal(-1.00000000000000))*(new_r01)*(x267)))+(((cj4)*(new_r11)))+(x277)+(((IkReal(-1.00000000000000))*(x274))));
evalcond[6]=((((IkReal(-1.00000000000000))*(x263)))+(((IkReal(-1.00000000000000))*(new_r00)*(x266)))+(((IkReal(-1.00000000000000))*(new_r00)*(x271)))+(((IkReal(-1.00000000000000))*(new_r10)*(x271)))+(((new_r10)*(x266))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r01)*(x266)))+(((IkReal(-0.707106781186548))*(x270)))+(((IkReal(-1.00000000000000))*(new_r01)*(x271)))+(((IkReal(-1.00000000000000))*(x275)))+(((new_r11)*(x266))));
evalcond[8]=((((x266)*(x268)))+(((cj5)*(new_r11)*(x266)))+(((IkReal(-1.00000000000000))*(new_r21)*(x265)))+(x263)+(((IkReal(-1.00000000000000))*(x268)*(x271)))+(((x269)*(x270))));
evalcond[9]=((((cj5)*(new_r10)*(x266)))+(((IkReal(-1.00000000000000))*(new_r00)*(sj4)*(x269)))+(((IkReal(-1.00000000000000))*(new_r20)*(x265)))+(((cj5)*(new_r00)*(x266)))+(((IkReal(-1.00000000000000))*(x275)))+(((x269)*(x272))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j6array[1], cj6array[1], sj6array[1];
bool j6valid[1]={false};
_nj6 = 1;
if( IKabs(((gconst11)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst11)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j6array[0]=IKatan2(((gconst11)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst11)*(new_r20)));
sj6array[0]=IKsin(j6array[0]);
cj6array[0]=IKcos(j6array[0]);
if( j6array[0] > IKPI )
{
    j6array[0]-=IK2PI;
}
else if( j6array[0] < -IKPI )
{    j6array[0]+=IK2PI;
}
j6valid[0] = true;
for(int ij6 = 0; ij6 < 1; ++ij6)
{
if( !j6valid[ij6] )
{
    continue;
}
_ij6[0] = ij6; _ij6[1] = -1;
for(int iij6 = ij6+1; iij6 < 1; ++iij6)
{
if( j6valid[iij6] && IKabs(cj6array[ij6]-cj6array[iij6]) < IKFAST_SOLUTION_THRESH && IKabs(sj6array[ij6]-sj6array[iij6]) < IKFAST_SOLUTION_THRESH )
{
    j6valid[iij6]=false; _ij6[1] = iij6; break; 
}
}
j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
{
IkReal evalcond[2];
evalcond[0]=((new_r20)+(((sj5)*(IKcos(j6)))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj5)*(IKsin(j6))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst13;
gconst13=IKsign(((((IkReal(2.00000000000000))*((new_r02)*(new_r02))))+(((IkReal(2.00000000000000))*((new_r12)*(new_r12))))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x278=((IkReal(1.41421356237310))*(sj5));
IkReal x279=((new_r12)*(x278));
IkReal x280=((new_r02)*(x278));
if( IKabs(((gconst13)*(((x279)+(((IkReal(-1.00000000000000))*(x280))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst13)*(((x279)+(x280))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst13)*(((x279)+(((IkReal(-1.00000000000000))*(x280)))))), ((gconst13)*(((x279)+(x280)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[15];
IkReal x281=IKcos(j4);
IkReal x282=((IkReal(0.707106781186548))*(sj5));
IkReal x283=IKsin(j4);
IkReal x284=((IkReal(0.707106781186548))*(sj6));
IkReal x285=((IkReal(0.707106781186548))*(cj6));
IkReal x286=((IkReal(1.00000000000000))*(cj6));
IkReal x287=((IkReal(0.707106781186548))*(cj5));
IkReal x288=((IkReal(1.00000000000000))*(cj5));
IkReal x289=((IkReal(1.00000000000000))*(sj5));
IkReal x290=((cj5)*(x285));
IkReal x291=((cj5)*(x284));
IkReal x292=((new_r10)*(x281));
IkReal x293=((new_r12)*(x281));
IkReal x294=((new_r11)*(x283));
IkReal x295=((new_r02)*(x283));
IkReal x296=((new_r00)*(x281));
IkReal x297=((IkReal(0.707106781186548))*(x283));
IkReal x298=((new_r02)*(x281));
IkReal x299=((new_r11)*(x281));
IkReal x300=((new_r00)*(x283));
IkReal x301=((new_r12)*(x283));
IkReal x302=((new_r01)*(x283));
IkReal x303=((new_r10)*(x283));
IkReal x304=((new_r01)*(x281));
evalcond[0]=((x301)+(x298)+(((IkReal(-1.00000000000000))*(x282))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x295)))+(x293)+(((IkReal(-1.00000000000000))*(x282))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x290)))+(x303)+(x296)+(x284));
evalcond[3]=((x304)+(x291)+(x294)+(x285));
evalcond[4]=((((IkReal(-1.00000000000000))*(x290)))+(((IkReal(-1.00000000000000))*(x300)))+(x292)+(((IkReal(-1.00000000000000))*(x284))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x302)))+(x299)+(x291)+(((IkReal(-1.00000000000000))*(x285))));
evalcond[6]=((((IkReal(-0.707106781186548))*(x298)))+(((IkReal(-0.707106781186548))*(x295)))+(((IkReal(-1.00000000000000))*(new_r12)*(x297)))+(((IkReal(0.707106781186548))*(x293))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r10)*(x297)))+(((IkReal(-1.00000000000000))*(sj6)))+(((IkReal(-0.707106781186548))*(x296)))+(((IkReal(-1.00000000000000))*(new_r00)*(x297)))+(((IkReal(0.707106781186548))*(x292))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r01)*(x297)))+(((IkReal(-0.707106781186548))*(x294)))+(((IkReal(-0.707106781186548))*(x304)))+(((IkReal(-1.00000000000000))*(x286)))+(((IkReal(0.707106781186548))*(x299))));
evalcond[9]=((((IkReal(-1.00000000000000))*(new_r22)*(x289)))+(((x287)*(x301)))+(((x287)*(x293)))+(((x287)*(x298)))+(((IkReal(-1.00000000000000))*(x287)*(x295))));
evalcond[10]=((((x282)*(x300)))+(((IkReal(-1.00000000000000))*(x282)*(x303)))+(((IkReal(-1.00000000000000))*(new_r20)*(x288)))+(((IkReal(-1.00000000000000))*(x282)*(x296)))+(((IkReal(-1.00000000000000))*(x282)*(x292))));
evalcond[11]=((((x282)*(x302)))+(((IkReal(-1.00000000000000))*(x282)*(x304)))+(((IkReal(-1.00000000000000))*(new_r21)*(x288)))+(((IkReal(-1.00000000000000))*(x282)*(x294)))+(((IkReal(-1.00000000000000))*(x282)*(x299))));
evalcond[12]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x288)))+(((x282)*(x295)))+(((IkReal(-1.00000000000000))*(x282)*(x301)))+(((IkReal(-1.00000000000000))*(x282)*(x293)))+(((IkReal(-1.00000000000000))*(x282)*(x298))));
evalcond[13]=((sj6)+(((x287)*(x304)))+(((IkReal(-1.00000000000000))*(x287)*(x302)))+(((x287)*(x294)))+(((x287)*(x299)))+(((IkReal(-1.00000000000000))*(new_r21)*(x289))));
evalcond[14]=((((x287)*(x303)))+(((IkReal(-1.00000000000000))*(x287)*(x300)))+(((x287)*(x296)))+(((x287)*(x292)))+(((IkReal(-1.00000000000000))*(x286)))+(((IkReal(-1.00000000000000))*(new_r20)*(x289))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  || IKabs(evalcond[12]) > 0.000001  || IKabs(evalcond[13]) > 0.000001  || IKabs(evalcond[14]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}static inline void polyroots6(IkReal rawcoeffs[6+1], IkReal rawroots[6], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[6];
    const int maxsteps = 110;
    for(int i = 0; i < 6; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[6];
    IkReal err[6];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 6; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 6; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 6; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 6; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[6] = {false};
    for(int i = 0; i < 6; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 6; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ba32f85ca3c87502739320e9e26a5905"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
