/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:47:53.799461
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41;
x0=IKcos(j[1]);
x1=IKcos(j[2]);
x2=IKsin(j[2]);
x3=IKsin(j[3]);
x4=IKcos(j[3]);
x5=IKsin(j[1]);
x6=IKsin(j[5]);
x7=IKsin(j[4]);
x8=IKcos(j[4]);
x9=IKcos(j[5]);
x10=((IkReal(0.707106781186548))*(IKcos(j[0])));
x11=((IkReal(0.707106781186548))*(IKsin(j[0])));
x12=((IkReal(0.707106781186548))*(x6));
x13=((IkReal(0.707106781186548))*(x9));
x14=((IkReal(0.500001138018525))*(x9));
x15=((IkReal(0.499998861978885))*(x9));
x16=((IkReal(0.499998861978885))*(x6));
x17=((IkReal(1.00000000000000))*(x1));
x18=((IkReal(0.500001138018525))*(x6));
x19=((IkReal(1.00000000000000))*(x3));
x20=((x0)*(x1));
x21=((x0)*(x2));
x22=((x4)*(x5));
x23=((IkReal(-1.00000000000000))*(x2));
x24=((x3)*(x5));
x25=((IkReal(-1.00000000000000))*(x3));
x26=((((IkReal(-1.00000000000000))*(x10)))+(x11));
x27=((((IkReal(-1.00000000000000))*(x11)))+(x10));
x28=((((IkReal(-1.00000000000000))*(x11)))+(((IkReal(-1.00000000000000))*(x10))));
x29=((x11)+(x10));
x30=((x26)*(x5));
x31=((IkReal(-1.00000000000000))*(x27));
x32=((x2)*(x27));
x33=((((IkReal(1.00000000000000))*(x0)*(x4)))+(((IkReal(-1.00000000000000))*(x17)*(x24))));
x34=((x33)*(x9));
x35=((((x8)*(((((IkReal(-1.00000000000000))*(x0)*(x19)))+(((IkReal(-1.00000000000000))*(x17)*(x22)))))))+(((x2)*(x5)*(x7))));
x36=((x35)*(x6));
x37=((((IkReal(-1.00000000000000))*(x22)*(x28)))+(((x19)*(((((x0)*(x17)*(x29)))+(((IkReal(1.00000000000000))*(x32))))))));
x38=((((x25)*(((((x20)*(x31)))+(((x23)*(x28)))))))+(((IkReal(-1.00000000000000))*(x22)*(x26))));
x39=((x37)*(x9));
x40=((((x8)*(((((x4)*(((((x2)*(x28)))+(((x20)*(x27)))))))+(((x24)*(x26)))))))+(((x7)*(((((x21)*(x26)))+(((x1)*(x28))))))));
x41=((((x8)*(((((x4)*(((x32)+(((x20)*(x29)))))))+(((x24)*(x28)))))))+(((x7)*(((((x21)*(x28)))+(((x1)*(x27))))))));
eerot[0]=((((((((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))+(((IkReal(0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))))*(IKcos(j[6])))));
eerot[1]=((((((((IkReal(-0.499998861978885))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707108390587779))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707108390587779))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.499998861978885))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707108390587779))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.707108390587779))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.499998861978885))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.499998861978885))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6])))));
eerot[2]=((((IkReal(-0.707108390587779))*(x36)))+(((IkReal(-0.707108390587779))*(x34)))+(((x16)*(x40)))+(((x15)*(x38)))+(((IkReal(-1.00000000000000))*(x16)*(x41)))+(((IkReal(-1.00000000000000))*(x15)*(x37))));
eetrans[0]=((IkReal(-0.350177000000000))+(((IkReal(0.212132517176334))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKcos(j[0]))))+(((IkReal(-0.106065775767248))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318198775764501))*(IKcos(j[2]))*(IKsin(j[1]))))+(((((((IkReal(-0.106065775767248))*(IKsin(j[0]))))+(((IkReal(0.106065775767248))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(-0.388909614823279))*(IKcos(j[1]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((IkReal(-0.212132517176334))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(-0.0318198775764501))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(-0.125829938105095))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829938105095))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.499998861978885))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(-0.499998861978885))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.125829938105095))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707108390587779))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.499998861978885))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.499998861978885))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(-0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.0159098663650872))*(IKcos(j[0]))))+(((IkReal(0.0159098663650872))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.194453922239955))*(IKcos(j[0]))))+(((IkReal(-0.194453922239955))*(IKsin(j[0]))))))*(IKsin(j[1])))));
eerot[3]=((((((((IkReal(0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6])))));
eerot[4]=((((((((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(IKcos(j[4]))*(IKsin(j[1]))*(IKsin(j[2]))))+(((IkReal(-0.707105171781653))*(((((IkReal(1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(-0.707105171781653))*(((((IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKcos(j[3]))))))*(IKsin(j[5]))))+(((IkReal(-0.707105171781653))*(((((IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((((((IkReal(-1.00000000000000))*(IKcos(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(-0.500001138018525))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(-0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.500001138018525))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.500001138018525))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6])))));
eerot[5]=((((IkReal(-0.707105171781653))*(x34)))+(((IkReal(-0.707105171781653))*(x36)))+(((IkReal(-1.00000000000000))*(x18)*(x40)))+(((x18)*(x41)))+(((IkReal(-1.00000000000000))*(x14)*(x38)))+(((x14)*(((((x25)*(((((x23)*(x27)))+(((IkReal(-1.00000000000000))*(x20)*(x29)))))))+(((IkReal(-1.00000000000000))*(x22)*(x28))))))));
eetrans[1]=((IkReal(-0.350177000000000))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((IkReal(0.0318197327301744))*(IKcos(j[2]))*(IKsin(j[1]))))+(((IkReal(0.212131551534496))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKsin(j[1]))*(IKsin(j[2]))*(IKsin(j[4]))))+(((IkReal(-0.707105171781653))*(((((IkReal(-0.177950000000000))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.177950000000000))*(IKcos(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-0.125829365318545))*(IKcos(j[1]))*(IKcos(j[3]))))+(((IkReal(0.125829365318545))*(IKcos(j[2]))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.212131551534496))*(IKcos(j[1]))*(IKcos(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(-0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKcos(j[0]))))+(((IkReal(-0.0159099387882250))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(-0.500001138018525))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((IkReal(-0.388907844479909))*(IKcos(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.194454807411639))*(IKcos(j[0]))))+(((IkReal(0.194454807411639))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((IkReal(-0.0318197327301744))*(IKcos(j[2]))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0159099387882250))*(IKsin(j[0]))))+(((IkReal(0.0159099387882250))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.106066258588167))*(IKsin(j[0]))))+(((IkReal(-0.106066258588167))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(-0.500001138018525))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-0.500001138018525))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3])))));
eerot[6]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKcos(j[6])))));
eerot[7]=((((((((IkReal(0.707106781186548))*(((((((((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKsin(j[5]))))+(((IkReal(0.707106781186548))*(((((((((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKcos(j[5]))))))*(IKcos(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))))*(IKcos(j[4]))))))*(IKsin(j[6]))))+(((((((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(-1.00000000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))))*(IKsin(j[4]))))))*(IKsin(j[6])))));
eerot[8]=((((x12)*(x40)))+(((x12)*(x41)))+(((x13)*(x37)))+(((x13)*(x38))));
eetrans[2]=((IkReal(0.378361000000000))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(-0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(-0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(0.275000000000000))*(IKcos(j[0]))))+(((IkReal(0.275000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(0.150000000000000))*(IKcos(j[0]))))+(((IkReal(0.150000000000000))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.0318198051533946))*(IKcos(j[0]))))+(((IkReal(0.0318198051533946))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.125829651712146))*(IKcos(j[0]))))+(((IkReal(0.125829651712146))*(IKsin(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[2]))))+(((IkReal(-0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKsin(j[3]))))))*(IKcos(j[5]))))+(((((((IkReal(0.0225000000000000))*(IKsin(j[0]))))+(((IkReal(0.0225000000000000))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(-0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(0.212132034355964))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.212132034355964))*(IKcos(j[0]))))+(((IkReal(-0.212132034355964))*(IKsin(j[0]))))))*(IKsin(j[2]))))))*(IKsin(j[3]))))+(((((((IkReal(0.0225000000000000))*(IKcos(j[0]))))+(((IkReal(-0.0225000000000000))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))+(((((((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[2]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKcos(j[1]))*(IKsin(j[2]))))))*(IKsin(j[4]))))+(((IkReal(0.707106781186548))*(((((IkReal(0.177950000000000))*(((((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(-0.707106781186548))*(IKcos(j[0]))))))*(IKsin(j[2]))))+(((((((IkReal(-0.707106781186548))*(IKsin(j[0]))))+(((IkReal(0.707106781186548))*(IKcos(j[0]))))))*(IKcos(j[1]))*(IKcos(j[2]))))))*(IKcos(j[3]))))+(((IkReal(0.177950000000000))*(((((IkReal(-0.707106781186548))*(IKcos(j[0]))))+(((IkReal(0.707106781186548))*(IKsin(j[0]))))))*(IKsin(j[1]))*(IKsin(j[3]))))))*(IKcos(j[4]))))))*(IKsin(j[5]))))+(((((((IkReal(-0.275000000000000))*(IKsin(j[0]))))+(((IkReal(0.275000000000000))*(IKcos(j[0]))))))*(IKsin(j[1]))))+(((((((IkReal(-0.150000000000000))*(IKsin(j[0]))))+(((IkReal(0.150000000000000))*(IKcos(j[0]))))))*(IKcos(j[3]))*(IKsin(j[1])))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int GetNumJoints() { return 7; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x67000001; }

class IKSolver {
public:
IkReal j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j7,cj7,sj7,htj7,j8,cj8,sj8,htj8,j9,cj9,sj9,htj9,j10,cj10,sj10,htj10,j6,cj6,sj6,htj6,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij4[2], _nj4,_ij5[2], _nj5,_ij7[2], _nj7,_ij8[2], _nj8,_ij9[2], _nj9,_ij10[2], _nj10,_ij6[2], _nj6;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1; j7=numeric_limits<IkReal>::quiet_NaN(); _ij7[0] = -1; _ij7[1] = -1; _nj7 = -1; j8=numeric_limits<IkReal>::quiet_NaN(); _ij8[0] = -1; _ij8[1] = -1; _nj8 = -1; j9=numeric_limits<IkReal>::quiet_NaN(); _ij9[0] = -1; _ij9[1] = -1; _nj9 = -1; j10=numeric_limits<IkReal>::quiet_NaN(); _ij10[0] = -1; _ij10[1] = -1; _nj10 = -1;  _ij6[0] = -1; _ij6[1] = -1; _nj6 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j6=pfree[0]; cj6=cos(pfree[0]); sj6=sin(pfree[0]);
r00 = eerot[0*3+0];
r01 = eerot[0*3+1];
r02 = eerot[0*3+2];
r10 = eerot[1*3+0];
r11 = eerot[1*3+1];
r12 = eerot[1*3+2];
r20 = eerot[2*3+0];
r21 = eerot[2*3+1];
r22 = eerot[2*3+2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_r00=((((IkReal(-0.500001138018525))*(r11)))+(((IkReal(0.499998861978885))*(r01)))+(((IkReal(0.707106781186548))*(r21))));
new_r01=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(0.500001138018525))*(r10)))+(((IkReal(-0.499998861978885))*(r00))));
new_r02=((((IkReal(-0.500001138018525))*(r12)))+(((IkReal(0.499998861978885))*(r02)))+(((IkReal(0.707106781186548))*(r22))));
new_px=((IkReal(-0.267542425853256))+(((IkReal(0.499998861978885))*(px)))+(((IkReal(-0.500001138018525))*(py)))+(((IkReal(-0.0889747974891426))*(r02)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.0889752025103964))*(r12))));
new_r10=((((IkReal(0.500001138018525))*(r11)))+(((IkReal(0.707106781186548))*(r21)))+(((IkReal(-0.499998861978885))*(r01))));
new_r11=((((IkReal(-0.707106781186548))*(r20)))+(((IkReal(-0.500001138018525))*(r10)))+(((IkReal(0.499998861978885))*(r00))));
new_r12=((((IkReal(0.500001138018525))*(r12)))+(((IkReal(0.707106781186548))*(r22)))+(((IkReal(-0.499998861978885))*(r02))));
new_py=((IkReal(-0.267540831819791))+(((IkReal(-0.499998861978885))*(px)))+(((IkReal(-0.125829651712146))*(r22)))+(((IkReal(0.707106781186548))*(pz)))+(((IkReal(0.500001138018525))*(py)))+(((IkReal(0.0889747974891426))*(r02)))+(((IkReal(-0.0889752025103964))*(r12))));
new_r20=((((IkReal(-0.707108390587779))*(r01)))+(((IkReal(-0.707105171781653))*(r11))));
new_r21=((((IkReal(0.707108390587779))*(r00)))+(((IkReal(0.707105171781653))*(r10))));
new_r22=((((IkReal(-0.707108390587779))*(r02)))+(((IkReal(-0.707105171781653))*(r12))));
new_pz=((IkReal(-0.495225062629841))+(((IkReal(0.125829365318545))*(r12)))+(((IkReal(-0.707105171781653))*(py)))+(((IkReal(0.125829938105095))*(r02)))+(((IkReal(-0.707108390587779))*(px))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
npx=((((px)*(r00)))+(((py)*(r10)))+(((pz)*(r20))));
npy=((((px)*(r01)))+(((py)*(r11)))+(((pz)*(r21))));
npz=((((px)*(r02)))+(((py)*(r12)))+(((pz)*(r22))));
rxp0_0=((((IkReal(-1.00000000000000))*(py)*(r20)))+(((pz)*(r10))));
rxp0_1=((((px)*(r20)))+(((IkReal(-1.00000000000000))*(pz)*(r00))));
rxp0_2=((((IkReal(-1.00000000000000))*(px)*(r10)))+(((py)*(r00))));
rxp1_0=((((IkReal(-1.00000000000000))*(py)*(r21)))+(((pz)*(r11))));
rxp1_1=((((px)*(r21)))+(((IkReal(-1.00000000000000))*(pz)*(r01))));
rxp1_2=((((IkReal(-1.00000000000000))*(px)*(r11)))+(((py)*(r01))));
rxp2_0=((((IkReal(-1.00000000000000))*(py)*(r22)))+(((pz)*(r12))));
rxp2_1=((((px)*(r22)))+(((IkReal(-1.00000000000000))*(pz)*(r02))));
rxp2_2=((((IkReal(-1.00000000000000))*(px)*(r12)))+(((py)*(r02))));
{
IkReal j7array[2], cj7array[2], sj7array[2];
bool j7valid[2]={false};
_nj7 = 2;
if( (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x42=IKasin(((IkReal(1.18441410190393))+(((IkReal(-2.98679637348110))*(pp)))));
j7array[0]=((IkReal(-1.34027003705633))+(((IkReal(-1.00000000000000))*(x42))));
sj7array[0]=IKsin(j7array[0]);
cj7array[0]=IKcos(j7array[0]);
j7array[1]=((IkReal(1.80132261653346))+(x42));
sj7array[1]=IKsin(j7array[1]);
cj7array[1]=IKcos(j7array[1]);
if( j7array[0] > IKPI )
{
    j7array[0]-=IK2PI;
}
else if( j7array[0] < -IKPI )
{    j7array[0]+=IK2PI;
}
j7valid[0] = true;
if( j7array[1] > IKPI )
{
    j7array[1]-=IK2PI;
}
else if( j7array[1] < -IKPI )
{    j7array[1]+=IK2PI;
}
j7valid[1] = true;
for(int ij7 = 0; ij7 < 2; ++ij7)
{
if( !j7valid[ij7] )
{
    continue;
}
_ij7[0] = ij7; _ij7[1] = -1;
for(int iij7 = ij7+1; iij7 < 2; ++iij7)
{
if( j7valid[iij7] && IKabs(cj7array[ij7]-cj7array[iij7]) < IKFAST_SOLUTION_THRESH && IKabs(sj7array[ij7]-sj7array[iij7]) < IKFAST_SOLUTION_THRESH )
{
    j7valid[iij7]=false; _ij7[1] = iij7; break; 
}
}
j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];

{
IkReal dummyeval[1];
IkReal x43=(cj6)*(cj6);
IkReal x44=(sj7)*(sj7);
IkReal x45=(cj7)*(cj7);
IkReal x46=((IkReal(13.3333333333333))*(sj7)*(x43));
dummyeval[0]=((IkReal(149.382716049383))+(((x43)*(x45)))+(((IkReal(44.4444444444444))*(x43)*(x44)))+(((IkReal(24.4444444444444))*(sj7)))+(((IkReal(44.4444444444444))*(x45)))+(x46)+(x44)+(x43)+(((IkReal(13.3333333333333))*(cj7)*(sj7)))+(((IkReal(-2.00000000000000))*(cj7)*(x43)))+(((IkReal(-1.00000000000000))*(cj7)*(x46)))+(((IkReal(162.962962962963))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x47=(cj6)*(cj6);
IkReal x48=(sj7)*(sj7);
IkReal x49=(cj7)*(cj7);
IkReal x50=((IkReal(13.3333333333333))*(sj7)*(x47));
dummyeval[0]=((IkReal(149.382716049383))+(((IkReal(44.4444444444444))*(x47)*(x48)))+(((x47)*(x49)))+(((IkReal(24.4444444444444))*(sj7)))+(((IkReal(-1.00000000000000))*(cj7)*(x50)))+(((IkReal(44.4444444444444))*(x49)))+(x48)+(x47)+(x50)+(((IkReal(13.3333333333333))*(cj7)*(sj7)))+(((IkReal(-2.00000000000000))*(cj7)*(x47)))+(((IkReal(162.962962962963))*(cj7))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x51=((IkReal(0.707106781186548))*(py));
IkReal x52=((IkReal(0.707106781186548))*(px));
IkReal x53=((IkReal(0.0450000000000000))*(sj6));
IkReal x54=((x52)+(((IkReal(-1.00000000000000))*(x51))));
IkReal x55=((x51)+(x52));
if( IKabs(x54) < IKFAST_ATAN2_MAGTHRESH && IKabs(x55) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x56=((IkReal(1.00000000000000))*(IKatan2(x54, x55)));
if( ((((x54)*(x54))+((x55)*(x55)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x54)*(x54))+((x55)*(x55)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x54)*(x54))+((x55)*(x55))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj7)*(x53)))+(x53)+(((IkReal(0.300000000000000))*(sj6)*(sj7))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x54)*(x54))+((x55)*(x55)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x54)*(x54))+((x55)*(x55))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj7)*(x53)))+(x53)+(((IkReal(0.300000000000000))*(sj6)*(sj7))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x57=IKasin(((((IKabs(IKabs(IKsqrt((((x54)*(x54))+((x55)*(x55)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x54)*(x54))+((x55)*(x55))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj7)*(x53)))+(x53)+(((IkReal(0.300000000000000))*(sj6)*(sj7)))))));
j4array[0]=((((IkReal(-1.00000000000000))*(x56)))+(((IkReal(-1.00000000000000))*(x57))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
j4array[1]=((IkReal(3.14159265358979))+(x57)+(((IkReal(-1.00000000000000))*(x56))));
sj4array[1]=IKsin(j4array[1]);
cj4array[1]=IKcos(j4array[1]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
if( j4array[1] > IKPI )
{
    j4array[1]-=IK2PI;
}
else if( j4array[1] < -IKPI )
{    j4array[1]+=IK2PI;
}
j4valid[1] = true;
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x58=((cj4)*(px));
IkReal x59=((IkReal(0.0318198051533946))*(sj7));
IkReal x60=((IkReal(0.212132034355964))*(cj7));
IkReal x61=((px)*(sj4));
IkReal x62=((cj4)*(py));
IkReal x63=((py)*(sj4));
IkReal x64=((IkReal(0.0450000000000000))*(cj6)*(pz));
gconst0=IKsign(((((x59)*(x61)))+(((x60)*(x61)))+(((IkReal(-0.388908729652601))*(x58)))+(((IkReal(-0.388908729652601))*(x62)))+(((IkReal(-0.388908729652601))*(x63)))+(((IkReal(0.388908729652601))*(x61)))+(((IkReal(-1.00000000000000))*(x59)*(x63)))+(((IkReal(-1.00000000000000))*(x59)*(x62)))+(((IkReal(-1.00000000000000))*(x58)*(x59)))+(((IkReal(-1.00000000000000))*(x58)*(x60)))+(((cj7)*(x64)))+(((IkReal(-1.00000000000000))*(x64)))+(((IkReal(-1.00000000000000))*(x60)*(x62)))+(((IkReal(-1.00000000000000))*(x60)*(x63)))+(((IkReal(-0.300000000000000))*(cj6)*(pz)*(sj7)))));
IkReal x65=((cj4)*(py));
IkReal x66=((IkReal(1.00000000000000))*(sj7));
IkReal x67=((IkReal(6.66666666666667))*(cj7));
IkReal x68=((cj4)*(px));
IkReal x69=((cj6)*(pz));
IkReal x70=((px)*(sj4));
IkReal x71=((py)*(sj4));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x66)*(x68)))+(((IkReal(1.41421356237310))*(cj7)*(x69)))+(((IkReal(-12.2222222222222))*(x71)))+(((IkReal(-1.00000000000000))*(x66)*(x71)))+(((IkReal(-9.42809041582063))*(sj7)*(x69)))+(((IkReal(-12.2222222222222))*(x68)))+(((IkReal(-12.2222222222222))*(x65)))+(((IkReal(-1.00000000000000))*(x67)*(x68)))+(((x67)*(x70)))+(((IkReal(12.2222222222222))*(x70)))+(((IkReal(-1.00000000000000))*(x67)*(x71)))+(((IkReal(-1.00000000000000))*(x65)*(x66)))+(((IkReal(-1.00000000000000))*(x65)*(x67)))+(((IkReal(-1.41421356237310))*(x69)))+(((sj7)*(x70))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x72=(cj7)*(cj7);
IkReal x73=(sj7)*(sj7);
IkReal x74=((IkReal(0.0135000000000000))*(cj6));
IkReal x75=((cj6)*(sj7));
IkReal x76=((IkReal(0.707106781186548))*(py)*(pz));
IkReal x77=((IkReal(0.707106781186548))*(px)*(pz));
if( IKabs(((gconst0)*(((IkReal(-0.302500000000000))+(((IkReal(-0.0270000000000000))*(cj7)*(sj7)))+(((IkReal(-0.0495000000000000))*(sj7)))+((pz)*(pz))+(((IkReal(-0.00202500000000000))*(x73)))+(((IkReal(-0.0900000000000000))*(x72)))+(((IkReal(-0.330000000000000))*(cj7))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-0.167025000000000))*(x75)))+(((IkReal(-0.0879750000000000))*(cj7)*(x75)))+(((IkReal(-1.00000000000000))*(cj4)*(x77)))+(((IkReal(-1.00000000000000))*(cj4)*(x76)))+(((IkReal(-0.0247500000000000))*(cj6)))+(((IkReal(0.0112500000000000))*(cj6)*(cj7)))+(((IkReal(-1.00000000000000))*(sj4)*(x76)))+(((IkReal(-1.00000000000000))*(x73)*(x74)))+(((x72)*(x74)))+(((sj4)*(x77))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst0)*(((IkReal(-0.302500000000000))+(((IkReal(-0.0270000000000000))*(cj7)*(sj7)))+(((IkReal(-0.0495000000000000))*(sj7)))+((pz)*(pz))+(((IkReal(-0.00202500000000000))*(x73)))+(((IkReal(-0.0900000000000000))*(x72)))+(((IkReal(-0.330000000000000))*(cj7)))))), ((gconst0)*(((((IkReal(-0.167025000000000))*(x75)))+(((IkReal(-0.0879750000000000))*(cj7)*(x75)))+(((IkReal(-1.00000000000000))*(cj4)*(x77)))+(((IkReal(-1.00000000000000))*(cj4)*(x76)))+(((IkReal(-0.0247500000000000))*(cj6)))+(((IkReal(0.0112500000000000))*(cj6)*(cj7)))+(((IkReal(-1.00000000000000))*(sj4)*(x76)))+(((IkReal(-1.00000000000000))*(x73)*(x74)))+(((x72)*(x74)))+(((sj4)*(x77)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x78=IKsin(j5);
IkReal x79=IKcos(j5);
IkReal x80=((IkReal(0.0450000000000000))*(cj7));
IkReal x81=((IkReal(0.707106781186548))*(cj4));
IkReal x82=((cj6)*(py));
IkReal x83=((py)*(sj6));
IkReal x84=((cj6)*(sj4));
IkReal x85=((IkReal(0.707106781186548))*(px));
IkReal x86=((sj4)*(sj6));
IkReal x87=((IkReal(0.300000000000000))*(cj7));
IkReal x88=((IkReal(0.300000000000000))*(sj7));
IkReal x89=((IkReal(0.0450000000000000))*(sj7));
IkReal x90=((IkReal(0.0636396103067893))*(px));
IkReal x91=((IkReal(0.707106781186548))*(sj4));
IkReal x92=((IkReal(0.0636396103067893))*(cj4));
IkReal x93=((cj6)*(x78));
IkReal x94=((px)*(x79));
IkReal x95=((cj6)*(x79));
IkReal x96=((sj4)*(x79));
IkReal x97=((pz)*(x79));
IkReal x98=((cj4)*(py)*(x78));
IkReal x99=((x82)*(x91));
IkReal x100=((py)*(sj4)*(x78));
IkReal x101=((IkReal(0.777817459305202))*(px)*(x78));
evalcond[0]=((((IkReal(-1.00000000000000))*(x80)*(x93)))+(((IkReal(-0.550000000000000))*(x79)))+(((IkReal(0.0450000000000000))*(x93)))+(((IkReal(-1.00000000000000))*(x79)*(x87)))+(((IkReal(-1.00000000000000))*(x79)*(x89)))+(((x88)*(x93)))+(pz));
evalcond[1]=((IkReal(-0.550000000000000))+(((py)*(x78)*(x91)))+(((py)*(x78)*(x81)))+(x97)+(((px)*(x78)*(x81)))+(((IkReal(-1.00000000000000))*(x89)))+(((IkReal(-1.00000000000000))*(x87)))+(((IkReal(-1.00000000000000))*(sj4)*(x78)*(x85))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(x81)))+(((IkReal(0.550000000000000))*(x78)))+(((x78)*(x89)))+(((x78)*(x87)))+(((IkReal(-1.00000000000000))*(x80)*(x95)))+(((IkReal(0.0450000000000000))*(x95)))+(((sj4)*(x85)))+(((x88)*(x95)))+(((IkReal(-1.00000000000000))*(px)*(x81)))+(((IkReal(-1.00000000000000))*(py)*(x91))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x81)*(x82)))+(((cj6)*(px)*(x81)))+(((x84)*(x85)))+(((x79)*(x81)*(x83)))+(((sj6)*(x81)*(x94)))+(((x79)*(x83)*(x91)))+(((IkReal(-1.00000000000000))*(pz)*(sj6)*(x78)))+(x99)+(((IkReal(-1.00000000000000))*(x79)*(x85)*(x86))));
evalcond[4]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x81)*(x83)))+(((px)*(sj6)*(x81)))+(((x85)*(x86)))+(((pz)*(x93)))+(((x79)*(x84)*(x85)))+(((IkReal(-1.00000000000000))*(x79)*(x99)))+(x88)+(((IkReal(-1.00000000000000))*(cj6)*(x81)*(x94)))+(((x83)*(x91)))+(((IkReal(-1.00000000000000))*(x80)))+(((IkReal(-1.00000000000000))*(x79)*(x81)*(x82))));
evalcond[5]=((IkReal(-0.212500000000000))+(((IkReal(0.0636396103067893))*(x82)*(x96)))+(((IkReal(1.10000000000000))*(x97)))+(((cj4)*(x90)*(x95)))+(((IkReal(-1.00000000000000))*(x79)*(x84)*(x90)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x86)*(x90)))+(((x79)*(x82)*(x92)))+(((cj4)*(x101)))+(((IkReal(0.777817459305202))*(x100)))+(((IkReal(-1.00000000000000))*(sj4)*(x101)))+(((x83)*(x92)))+(((IkReal(-1.00000000000000))*(cj4)*(sj6)*(x90)))+(((IkReal(0.777817459305202))*(x98)))+(((IkReal(-0.0636396103067893))*(sj4)*(x83)))+(((IkReal(-0.0900000000000000))*(pz)*(x93))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x187=((IkReal(0.0450000000000000))*(cj6));
IkReal x188=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7))));
IkReal x189=((((IkReal(0.300000000000000))*(cj6)*(sj7)))+(x187)+(((IkReal(-1.00000000000000))*(cj7)*(x187))));
if( IKabs(x188) < IKFAST_ATAN2_MAGTHRESH && IKabs(x189) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x190=((IkReal(1.00000000000000))*(IKatan2(x188, x189)));
if( ((((x189)*(x189))+((x188)*(x188)))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x189)*(x189))+((x188)*(x188)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x189)*(x189))+((x188)*(x188))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x189)*(x189))+((x188)*(x188)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x189)*(x189))+((x188)*(x188))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x191=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x189)*(x189))+((x188)*(x188)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x189)*(x189))+((x188)*(x188))))))):(IkReal)1.0e30))));
j5array[0]=((((IkReal(-1.00000000000000))*(x191)))+(((IkReal(-1.00000000000000))*(x190))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x190)))+(x191));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x192=((IkReal(1.00000000000000))*(sj5));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x192)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x192)*((px)*(px))))));
IkReal x193=((IkReal(1.00000000000000))*(sj5));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x193)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x193)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x194=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(pz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj7))));
evalcond[2]=x194;
evalcond[3]=x194;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x195=((cj6)*(px));
IkReal x196=((IkReal(0.0318198051533946))*(py));
IkReal x197=((IkReal(0.212132034355964))*(sj7));
IkReal x198=((IkReal(0.0318198051533946))*(cj7));
IkReal x199=((px)*(sj6));
IkReal x200=((sj6)*(x196));
IkReal x201=((IkReal(0.0318198051533946))*(x195));
IkReal x202=((cj6)*(x196));
IkReal x203=((IkReal(0.0318198051533946))*(x199));
IkReal x204=((x198)*(x199));
IkReal x205=((cj6)*(py)*(x197));
IkReal x206=((x195)*(x197));
IkReal x207=((py)*(sj6)*(x197));
IkReal x208=((cj7)*(x200));
IkReal x209=((cj7)*(x202));
IkReal x210=((x195)*(x198));
IkReal x211=((x197)*(x199));
IkReal x212=((x210)+(x208));
IkReal x213=((x205)+(x204)+(x202));
IkReal x214=((x211)+(x203)+(x209));
IkReal x215=((x207)+(x206)+(x201)+(x200));
if( IKabs(((gconst2)*(((x212)+(x213)+(((IkReal(-1.00000000000000))*(x215)))+(((IkReal(-1.00000000000000))*(x214))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((x213)+(x215)+(((IkReal(-1.00000000000000))*(x212)))+(((IkReal(-1.00000000000000))*(x214))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst2)*(((x212)+(x213)+(((IkReal(-1.00000000000000))*(x215)))+(((IkReal(-1.00000000000000))*(x214)))))), ((gconst2)*(((x213)+(x215)+(((IkReal(-1.00000000000000))*(x212)))+(((IkReal(-1.00000000000000))*(x214)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x216=IKsin(j4);
IkReal x217=IKcos(j4);
IkReal x218=((IkReal(0.0450000000000000))*(cj7));
IkReal x219=((px)*(sj6));
IkReal x220=((py)*(sj6));
IkReal x221=((IkReal(0.300000000000000))*(sj7));
IkReal x222=((cj6)*(py));
IkReal x223=((cj6)*(px));
IkReal x224=((IkReal(0.707106781186548))*(x217));
IkReal x225=((IkReal(0.0636396103067893))*(x217));
IkReal x226=((IkReal(0.707106781186548))*(x216));
IkReal x227=((IkReal(0.0636396103067893))*(x216));
IkReal x228=((px)*(x226));
IkReal x229=((px)*(x224));
IkReal x230=((py)*(x224));
IkReal x231=((py)*(x226));
IkReal x232=((x219)*(x224));
IkReal x233=((x223)*(x226));
IkReal x234=((x219)*(x226));
IkReal x235=((x220)*(x226));
IkReal x236=((x222)*(x224));
IkReal x237=((x220)*(x224));
IkReal x238=((x222)*(x226));
IkReal x239=((x223)*(x224));
IkReal x240=((x229)+(x231));
IkReal x241=((x232)+(x233)+(x235));
IkReal x242=((x238)+(x239)+(x237));
evalcond[0]=((((sj6)*(x221)))+(((IkReal(-1.00000000000000))*(sj6)*(x218)))+(((IkReal(0.0450000000000000))*(sj6)))+(((IkReal(-1.00000000000000))*(x230)))+(x228)+(x240));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj6)*(x218)))+(((IkReal(0.0450000000000000))*(cj6)))+(((cj6)*(x221)))+(((IkReal(-1.00000000000000))*(x230)))+(x228)+(((IkReal(-1.00000000000000))*(x240))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x234)))+(((IkReal(-1.00000000000000))*(x236)))+(x242)+(x241));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x236)))+(x221)+(x234)+(x241)+(((IkReal(-1.00000000000000))*(x242)))+(((IkReal(-1.00000000000000))*(x218))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x222)*(x225)))+(((x222)*(x227)))+(((x223)*(x225)))+(((IkReal(-1.00000000000000))*(x219)*(x227)))+(((IkReal(-1.00000000000000))*(x219)*(x225)))+(((IkReal(-1.00000000000000))*(pp)))+(((x220)*(x225)))+(((IkReal(-1.00000000000000))*(x223)*(x227)))+(((IkReal(-1.00000000000000))*(x220)*(x227)))+(((IkReal(1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x243=((IkReal(0.0450000000000000))*(sj7));
IkReal x244=((IkReal(0.300000000000000))*(cj7));
IkReal x245=((x243)+(x244));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj7))));
evalcond[2]=((IkReal(0.550000000000000))+(x245)+(pz));
evalcond[3]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x245))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x246=((cj6)*(px));
IkReal x247=((IkReal(0.0318198051533946))*(py));
IkReal x248=((IkReal(0.212132034355964))*(sj7));
IkReal x249=((IkReal(0.0318198051533946))*(cj7));
IkReal x250=((px)*(sj6));
IkReal x251=((sj6)*(x247));
IkReal x252=((IkReal(0.0318198051533946))*(x250));
IkReal x253=((IkReal(0.0318198051533946))*(x246));
IkReal x254=((cj6)*(x247));
IkReal x255=((cj7)*(x254));
IkReal x256=((x246)*(x249));
IkReal x257=((x249)*(x250));
IkReal x258=((py)*(sj6)*(x248));
IkReal x259=((cj7)*(x251));
IkReal x260=((cj6)*(py)*(x248));
IkReal x261=((x246)*(x248));
IkReal x262=((x248)*(x250));
IkReal x263=((x255)+(x257));
IkReal x264=((x259)+(x253)+(x261));
IkReal x265=((x258)+(x256)+(x251));
IkReal x266=((x254)+(x252)+(x260)+(x262));
if( IKabs(((gconst3)*(((x264)+(x263)+(((IkReal(-1.00000000000000))*(x265)))+(((IkReal(-1.00000000000000))*(x266))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((x265)+(x263)+(((IkReal(-1.00000000000000))*(x264)))+(((IkReal(-1.00000000000000))*(x266))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((x264)+(x263)+(((IkReal(-1.00000000000000))*(x265)))+(((IkReal(-1.00000000000000))*(x266)))))), ((gconst3)*(((x265)+(x263)+(((IkReal(-1.00000000000000))*(x264)))+(((IkReal(-1.00000000000000))*(x266)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x267=IKsin(j4);
IkReal x268=IKcos(j4);
IkReal x269=((IkReal(0.0450000000000000))*(cj7));
IkReal x270=((px)*(sj6));
IkReal x271=((cj6)*(px));
IkReal x272=((py)*(sj6));
IkReal x273=((IkReal(0.300000000000000))*(sj7));
IkReal x274=((cj6)*(py));
IkReal x275=((IkReal(0.707106781186548))*(x268));
IkReal x276=((IkReal(0.0636396103067893))*(x268));
IkReal x277=((IkReal(0.707106781186548))*(x267));
IkReal x278=((IkReal(0.0636396103067893))*(x267));
IkReal x279=((px)*(x277));
IkReal x280=((px)*(x275));
IkReal x281=((py)*(x275));
IkReal x282=((py)*(x277));
IkReal x283=((x270)*(x275));
IkReal x284=((x274)*(x275));
IkReal x285=((x274)*(x277));
IkReal x286=((x270)*(x277));
IkReal x287=((x272)*(x277));
IkReal x288=((x271)*(x275));
IkReal x289=((x272)*(x275));
IkReal x290=((x271)*(x277));
IkReal x291=((x282)+(x280));
IkReal x292=((x287)+(x284)+(x283));
IkReal x293=((x288)+(x286)+(x285));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj6)*(x269)))+(((IkReal(0.0450000000000000))*(sj6)))+(x291)+(x279)+(((sj6)*(x273)))+(((IkReal(-1.00000000000000))*(x281))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x291)))+(((IkReal(-1.00000000000000))*(cj6)*(x273)))+(((IkReal(-0.0450000000000000))*(cj6)))+(x279)+(((cj6)*(x269)))+(((IkReal(-1.00000000000000))*(x281))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x292)))+(x290)+(x293)+(((IkReal(-1.00000000000000))*(x289))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x290)))+(x292)+(x293)+(x273)+(((IkReal(-1.00000000000000))*(x289)))+(((IkReal(-1.00000000000000))*(x269))));
evalcond[4]=((IkReal(-0.212500000000000))+(((x272)*(x276)))+(((IkReal(-1.00000000000000))*(x272)*(x278)))+(((IkReal(-1.00000000000000))*(x274)*(x276)))+(((IkReal(-1.00000000000000))*(x274)*(x278)))+(((x271)*(x278)))+(((IkReal(-1.00000000000000))*(x271)*(x276)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x270)*(x278)))+(((IkReal(-1.00000000000000))*(x270)*(x276))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x294=((IkReal(0.388908729652601))*(py));
IkReal x295=((IkReal(0.388908729652601))*(px));
IkReal x296=((sj5)*(sj6));
IkReal x297=((IkReal(0.0318198051533946))*(py));
IkReal x298=((IkReal(0.212132034355964))*(sj7));
IkReal x299=((cj7)*(px));
IkReal x300=((IkReal(0.0318198051533946))*(px));
IkReal x301=((IkReal(0.212132034355964))*(x299));
IkReal x302=((IkReal(0.212132034355964))*(cj7)*(py));
IkReal x303=((sj7)*(x300));
IkReal x304=((sj7)*(x297));
IkReal x305=((IkReal(0.707106781186548))*(cj5)*(pz));
IkReal x306=((x296)*(x300));
IkReal x307=((px)*(x305));
IkReal x308=((py)*(x305));
IkReal x309=((x296)*(x297));
IkReal x310=((px)*(x296)*(x298));
IkReal x311=((cj7)*(x309));
IkReal x312=((IkReal(0.0318198051533946))*(x296)*(x299));
IkReal x313=((py)*(x296)*(x298));
IkReal x314=((x307)+(x311));
IkReal x315=((x308)+(x306)+(x310));
IkReal x316=((x304)+(x302)+(x312)+(x294));
IkReal x317=((x309)+(x303)+(x301)+(x313)+(x295));
if( IKabs(((gconst1)*(((x315)+(x317)+(((IkReal(-1.00000000000000))*(x316)))+(((IkReal(-1.00000000000000))*(x314))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((x315)+(x314)+(((IkReal(-1.00000000000000))*(x316)))+(((IkReal(-1.00000000000000))*(x317))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((x315)+(x317)+(((IkReal(-1.00000000000000))*(x316)))+(((IkReal(-1.00000000000000))*(x314)))))), ((gconst1)*(((x315)+(x314)+(((IkReal(-1.00000000000000))*(x316)))+(((IkReal(-1.00000000000000))*(x317)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x318=IKsin(j4);
IkReal x319=IKcos(j4);
IkReal x320=((IkReal(0.0450000000000000))*(cj7));
IkReal x321=((IkReal(0.707106781186548))*(sj6));
IkReal x322=((IkReal(0.777817459305202))*(sj5));
IkReal x323=((IkReal(0.300000000000000))*(sj7));
IkReal x324=((cj5)*(pz));
IkReal x325=((cj5)*(cj6));
IkReal x326=((IkReal(0.0450000000000000))*(sj7));
IkReal x327=((IkReal(0.707106781186548))*(sj5));
IkReal x328=((IkReal(0.707106781186548))*(cj6));
IkReal x329=((IkReal(0.300000000000000))*(cj7));
IkReal x330=((IkReal(0.0636396103067893))*(px)*(sj6));
IkReal x331=((px)*(x318));
IkReal x332=((cj6)*(pz)*(sj5));
IkReal x333=((px)*(x319));
IkReal x334=((py)*(x318));
IkReal x335=((IkReal(0.707106781186548))*(x331));
IkReal x336=((IkReal(0.707106781186548))*(x333));
IkReal x337=((IkReal(0.707106781186548))*(py)*(x319));
IkReal x338=((IkReal(0.707106781186548))*(x334));
IkReal x339=((IkReal(0.0636396103067893))*(py)*(x319));
IkReal x340=((x338)+(x336));
evalcond[0]=((((IkReal(0.0450000000000000))*(sj6)))+(x335)+(x340)+(((IkReal(-1.00000000000000))*(sj6)*(x320)))+(((sj6)*(x323)))+(((IkReal(-1.00000000000000))*(x337))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x326)))+(((IkReal(-1.00000000000000))*(x329)))+(x324)+(((x327)*(x334)))+(((x327)*(x333)))+(((py)*(x319)*(x327)))+(((IkReal(-1.00000000000000))*(x327)*(x331))));
evalcond[2]=((((x323)*(x325)))+(((sj5)*(x329)))+(((sj5)*(x326)))+(((IkReal(-1.00000000000000))*(x320)*(x325)))+(x335)+(((IkReal(-1.00000000000000))*(x337)))+(((IkReal(0.550000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(x340)))+(((IkReal(0.0450000000000000))*(x325))));
evalcond[3]=((((cj5)*(x321)*(x334)))+(((cj5)*(x321)*(x333)))+(((cj5)*(py)*(x319)*(x321)))+(((IkReal(-1.00000000000000))*(py)*(x319)*(x328)))+(((IkReal(-1.00000000000000))*(cj5)*(x321)*(x331)))+(((x328)*(x334)))+(((x328)*(x331)))+(((x328)*(x333)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)*(sj6))));
evalcond[4]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(py)*(x319)*(x321)))+(((IkReal(-1.00000000000000))*(x320)))+(x332)+(x323)+(((x325)*(x335)))+(((IkReal(-1.00000000000000))*(x325)*(x337)))+(((IkReal(-1.00000000000000))*(x325)*(x336)))+(((IkReal(-1.00000000000000))*(x325)*(x338)))+(((x321)*(x333)))+(((x321)*(x331)))+(((x321)*(x334))));
evalcond[5]=((IkReal(-0.212500000000000))+(((IkReal(1.10000000000000))*(x324)))+(((IkReal(-1.00000000000000))*(x318)*(x330)))+(((IkReal(-1.00000000000000))*(x319)*(x330)))+(((IkReal(-0.0636396103067893))*(x325)*(x331)))+(((IkReal(-0.0636396103067893))*(sj6)*(x334)))+(((IkReal(-1.00000000000000))*(pp)))+(((py)*(x319)*(x322)))+(((sj6)*(x339)))+(((IkReal(-1.00000000000000))*(x322)*(x331)))+(((x325)*(x339)))+(((x322)*(x333)))+(((x322)*(x334)))+(((IkReal(0.0636396103067893))*(x325)*(x334)))+(((IkReal(0.0636396103067893))*(x325)*(x333)))+(((IkReal(-0.0900000000000000))*(x332))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x341=((IkReal(0.0450000000000000))*(cj6));
IkReal x342=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7))));
IkReal x343=((((IkReal(0.300000000000000))*(cj6)*(sj7)))+(x341)+(((IkReal(-1.00000000000000))*(cj7)*(x341))));
if( IKabs(x342) < IKFAST_ATAN2_MAGTHRESH && IKabs(x343) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x344=((IkReal(1.00000000000000))*(IKatan2(x342, x343)));
if( ((((x343)*(x343))+((x342)*(x342)))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x343)*(x343))+((x342)*(x342)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x343)*(x343))+((x342)*(x342))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x343)*(x343))+((x342)*(x342)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x343)*(x343))+((x342)*(x342))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x345=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x343)*(x343))+((x342)*(x342)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x343)*(x343))+((x342)*(x342))))))):(IkReal)1.0e30))));
j5array[0]=((((IkReal(-1.00000000000000))*(x344)))+(((IkReal(-1.00000000000000))*(x345))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
j5array[1]=((IkReal(3.14159265358979))+(x345)+(((IkReal(-1.00000000000000))*(x344))));
sj5array[1]=IKsin(j5array[1]);
cj5array[1]=IKcos(j5array[1]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
if( j5array[1] > IKPI )
{
    j5array[1]-=IK2PI;
}
else if( j5array[1] < -IKPI )
{    j5array[1]+=IK2PI;
}
j5valid[1] = true;
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x346=((IkReal(1.00000000000000))*(sj5));
gconst1=IKsign(((((IkReal(-1.00000000000000))*(x346)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x346)*((px)*(px))))));
IkReal x347=((IkReal(1.00000000000000))*(sj5));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x347)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x347)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x348=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj7)))+(((IkReal(-0.300000000000000))*(cj7)))+(pz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj7))));
evalcond[2]=x348;
evalcond[3]=x348;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x349=((cj6)*(px));
IkReal x350=((IkReal(0.0318198051533946))*(py));
IkReal x351=((IkReal(0.212132034355964))*(sj7));
IkReal x352=((IkReal(0.0318198051533946))*(cj7));
IkReal x353=((px)*(sj6));
IkReal x354=((sj6)*(x350));
IkReal x355=((IkReal(0.0318198051533946))*(x349));
IkReal x356=((cj6)*(x350));
IkReal x357=((IkReal(0.0318198051533946))*(x353));
IkReal x358=((x352)*(x353));
IkReal x359=((cj6)*(py)*(x351));
IkReal x360=((x349)*(x351));
IkReal x361=((py)*(sj6)*(x351));
IkReal x362=((cj7)*(x354));
IkReal x363=((cj7)*(x356));
IkReal x364=((x349)*(x352));
IkReal x365=((x351)*(x353));
IkReal x366=((x362)+(x364));
IkReal x367=((x356)+(x359)+(x358));
IkReal x368=((x357)+(x363)+(x365));
IkReal x369=((x355)+(x354)+(x360)+(x361));
if( IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x369)))+(((IkReal(-1.00000000000000))*(x368)))+(x366)+(x367))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x366)))+(((IkReal(-1.00000000000000))*(x368)))+(x369)+(x367))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst2)*(((((IkReal(-1.00000000000000))*(x369)))+(((IkReal(-1.00000000000000))*(x368)))+(x366)+(x367)))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x366)))+(((IkReal(-1.00000000000000))*(x368)))+(x369)+(x367)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x370=IKsin(j4);
IkReal x371=IKcos(j4);
IkReal x372=((IkReal(0.0450000000000000))*(cj7));
IkReal x373=((px)*(sj6));
IkReal x374=((py)*(sj6));
IkReal x375=((IkReal(0.300000000000000))*(sj7));
IkReal x376=((cj6)*(py));
IkReal x377=((cj6)*(px));
IkReal x378=((IkReal(0.707106781186548))*(x371));
IkReal x379=((IkReal(0.0636396103067893))*(x371));
IkReal x380=((IkReal(0.707106781186548))*(x370));
IkReal x381=((IkReal(0.0636396103067893))*(x370));
IkReal x382=((px)*(x380));
IkReal x383=((px)*(x378));
IkReal x384=((py)*(x378));
IkReal x385=((py)*(x380));
IkReal x386=((x373)*(x378));
IkReal x387=((x377)*(x380));
IkReal x388=((x373)*(x380));
IkReal x389=((x374)*(x380));
IkReal x390=((x376)*(x378));
IkReal x391=((x374)*(x378));
IkReal x392=((x376)*(x380));
IkReal x393=((x377)*(x378));
IkReal x394=((x385)+(x383));
IkReal x395=((x386)+(x387)+(x389));
IkReal x396=((x391)+(x393)+(x392));
evalcond[0]=((((IkReal(0.0450000000000000))*(sj6)))+(x382)+(x394)+(((IkReal(-1.00000000000000))*(sj6)*(x372)))+(((sj6)*(x375)))+(((IkReal(-1.00000000000000))*(x384))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x394)))+(((IkReal(0.0450000000000000))*(cj6)))+(x382)+(((IkReal(-1.00000000000000))*(cj6)*(x372)))+(((cj6)*(x375)))+(((IkReal(-1.00000000000000))*(x384))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x390)))+(x395)+(x396)+(((IkReal(-1.00000000000000))*(x388))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x390)))+(((IkReal(-1.00000000000000))*(x396)))+(((IkReal(-1.00000000000000))*(x372)))+(x388)+(x375)+(x395));
evalcond[4]=((IkReal(-0.212500000000000))+(((IkReal(-1.00000000000000))*(x373)*(x381)))+(((x376)*(x381)))+(((IkReal(-1.00000000000000))*(x374)*(x381)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x373)*(x379)))+(((x376)*(x379)))+(((IkReal(1.10000000000000))*(pz)))+(((x377)*(x379)))+(((x374)*(x379)))+(((IkReal(-1.00000000000000))*(x377)*(x381))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
IkReal x397=((IkReal(0.0450000000000000))*(sj7));
IkReal x398=((IkReal(0.300000000000000))*(cj7));
IkReal x399=((x397)+(x398));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.396550000000000))+(((IkReal(0.0765000000000000))*(sj7)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.325950000000000))*(cj7))));
evalcond[2]=((IkReal(0.550000000000000))+(x399)+(pz));
evalcond[3]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x399)))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x400=((cj6)*(px));
IkReal x401=((IkReal(0.0318198051533946))*(py));
IkReal x402=((IkReal(0.212132034355964))*(sj7));
IkReal x403=((IkReal(0.0318198051533946))*(cj7));
IkReal x404=((px)*(sj6));
IkReal x405=((sj6)*(x401));
IkReal x406=((IkReal(0.0318198051533946))*(x404));
IkReal x407=((IkReal(0.0318198051533946))*(x400));
IkReal x408=((cj6)*(x401));
IkReal x409=((cj7)*(x408));
IkReal x410=((x400)*(x403));
IkReal x411=((x403)*(x404));
IkReal x412=((py)*(sj6)*(x402));
IkReal x413=((cj7)*(x405));
IkReal x414=((cj6)*(py)*(x402));
IkReal x415=((x400)*(x402));
IkReal x416=((x402)*(x404));
IkReal x417=((x409)+(x411));
IkReal x418=((x407)+(x413)+(x415));
IkReal x419=((x405)+(x412)+(x410));
IkReal x420=((x406)+(x408)+(x416)+(x414));
if( IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x420)))+(x417)+(x418)+(((IkReal(-1.00000000000000))*(x419))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-1.00000000000000))*(x420)))+(x417)+(x419)+(((IkReal(-1.00000000000000))*(x418))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst3)*(((((IkReal(-1.00000000000000))*(x420)))+(x417)+(x418)+(((IkReal(-1.00000000000000))*(x419)))))), ((gconst3)*(((((IkReal(-1.00000000000000))*(x420)))+(x417)+(x419)+(((IkReal(-1.00000000000000))*(x418)))))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[5];
IkReal x421=IKsin(j4);
IkReal x422=IKcos(j4);
IkReal x423=((IkReal(0.0450000000000000))*(cj7));
IkReal x424=((px)*(sj6));
IkReal x425=((cj6)*(px));
IkReal x426=((py)*(sj6));
IkReal x427=((IkReal(0.300000000000000))*(sj7));
IkReal x428=((cj6)*(py));
IkReal x429=((IkReal(0.707106781186548))*(x422));
IkReal x430=((IkReal(0.0636396103067893))*(x422));
IkReal x431=((IkReal(0.707106781186548))*(x421));
IkReal x432=((IkReal(0.0636396103067893))*(x421));
IkReal x433=((px)*(x431));
IkReal x434=((px)*(x429));
IkReal x435=((py)*(x429));
IkReal x436=((py)*(x431));
IkReal x437=((x424)*(x429));
IkReal x438=((x428)*(x429));
IkReal x439=((x428)*(x431));
IkReal x440=((x424)*(x431));
IkReal x441=((x426)*(x431));
IkReal x442=((x425)*(x429));
IkReal x443=((x426)*(x429));
IkReal x444=((x425)*(x431));
IkReal x445=((x434)+(x436));
IkReal x446=((x438)+(x437)+(x441));
IkReal x447=((x439)+(x440)+(x442));
evalcond[0]=((((IkReal(-1.00000000000000))*(x435)))+(((IkReal(0.0450000000000000))*(sj6)))+(((sj6)*(x427)))+(x433)+(x445)+(((IkReal(-1.00000000000000))*(sj6)*(x423))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj6)*(x427)))+(((IkReal(-1.00000000000000))*(x445)))+(((IkReal(-1.00000000000000))*(x435)))+(((IkReal(-0.0450000000000000))*(cj6)))+(((cj6)*(x423)))+(x433));
evalcond[2]=((((IkReal(-1.00000000000000))*(x446)))+(((IkReal(-1.00000000000000))*(x443)))+(x444)+(x447));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x444)))+(((IkReal(-1.00000000000000))*(x443)))+(((IkReal(-1.00000000000000))*(x423)))+(x427)+(x447)+(x446));
evalcond[4]=((IkReal(-0.212500000000000))+(((x426)*(x430)))+(((x425)*(x432)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x424)*(x430)))+(((IkReal(-1.00000000000000))*(x424)*(x432)))+(((IkReal(-1.00000000000000))*(x426)*(x432)))+(((IkReal(-1.00000000000000))*(x425)*(x430)))+(((IkReal(-1.00000000000000))*(x428)*(x432)))+(((IkReal(-1.00000000000000))*(x428)*(x430))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j4array[1], cj4array[1], sj4array[1];
bool j4valid[1]={false};
_nj4 = 1;
IkReal x448=((IkReal(0.388908729652601))*(py));
IkReal x449=((IkReal(0.388908729652601))*(px));
IkReal x450=((sj5)*(sj6));
IkReal x451=((IkReal(0.0318198051533946))*(py));
IkReal x452=((IkReal(0.212132034355964))*(sj7));
IkReal x453=((cj7)*(px));
IkReal x454=((IkReal(0.0318198051533946))*(px));
IkReal x455=((IkReal(0.212132034355964))*(x453));
IkReal x456=((IkReal(0.212132034355964))*(cj7)*(py));
IkReal x457=((sj7)*(x454));
IkReal x458=((sj7)*(x451));
IkReal x459=((IkReal(0.707106781186548))*(cj5)*(pz));
IkReal x460=((x450)*(x454));
IkReal x461=((px)*(x459));
IkReal x462=((py)*(x459));
IkReal x463=((x450)*(x451));
IkReal x464=((px)*(x450)*(x452));
IkReal x465=((cj7)*(x463));
IkReal x466=((IkReal(0.0318198051533946))*(x450)*(x453));
IkReal x467=((py)*(x450)*(x452));
IkReal x468=((x461)+(x465));
IkReal x469=((x462)+(x460)+(x464));
IkReal x470=((x448)+(x458)+(x456)+(x466));
IkReal x471=((x449)+(x457)+(x455)+(x463)+(x467));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x470)))+(((IkReal(-1.00000000000000))*(x468)))+(x469)+(x471))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x471)))+(((IkReal(-1.00000000000000))*(x470)))+(x469)+(x468))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j4array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x470)))+(((IkReal(-1.00000000000000))*(x468)))+(x469)+(x471)))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x471)))+(((IkReal(-1.00000000000000))*(x470)))+(x469)+(x468)))));
sj4array[0]=IKsin(j4array[0]);
cj4array[0]=IKcos(j4array[0]);
if( j4array[0] > IKPI )
{
    j4array[0]-=IK2PI;
}
else if( j4array[0] < -IKPI )
{    j4array[0]+=IK2PI;
}
j4valid[0] = true;
for(int ij4 = 0; ij4 < 1; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 1; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
{
IkReal evalcond[6];
IkReal x472=IKsin(j4);
IkReal x473=IKcos(j4);
IkReal x474=((IkReal(0.0450000000000000))*(cj7));
IkReal x475=((IkReal(0.707106781186548))*(sj6));
IkReal x476=((IkReal(0.777817459305202))*(sj5));
IkReal x477=((IkReal(0.300000000000000))*(sj7));
IkReal x478=((cj5)*(pz));
IkReal x479=((cj5)*(cj6));
IkReal x480=((IkReal(0.0450000000000000))*(sj7));
IkReal x481=((IkReal(0.707106781186548))*(sj5));
IkReal x482=((IkReal(0.707106781186548))*(cj6));
IkReal x483=((IkReal(0.300000000000000))*(cj7));
IkReal x484=((IkReal(0.0636396103067893))*(px)*(sj6));
IkReal x485=((px)*(x472));
IkReal x486=((cj6)*(pz)*(sj5));
IkReal x487=((px)*(x473));
IkReal x488=((py)*(x472));
IkReal x489=((IkReal(0.707106781186548))*(x485));
IkReal x490=((IkReal(0.707106781186548))*(x487));
IkReal x491=((IkReal(0.707106781186548))*(py)*(x473));
IkReal x492=((IkReal(0.707106781186548))*(x488));
IkReal x493=((IkReal(0.0636396103067893))*(py)*(x473));
IkReal x494=((x492)+(x490));
evalcond[0]=((((sj6)*(x477)))+(((IkReal(-1.00000000000000))*(x491)))+(((IkReal(0.0450000000000000))*(sj6)))+(((IkReal(-1.00000000000000))*(sj6)*(x474)))+(x489)+(x494));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x483)))+(((IkReal(-1.00000000000000))*(x480)))+(((IkReal(-1.00000000000000))*(x481)*(x485)))+(((x481)*(x487)))+(((x481)*(x488)))+(x478)+(((py)*(x473)*(x481))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x491)))+(((IkReal(-1.00000000000000))*(x494)))+(x489)+(((IkReal(0.0450000000000000))*(x479)))+(((sj5)*(x483)))+(((sj5)*(x480)))+(((x477)*(x479)))+(((IkReal(0.550000000000000))*(sj5)))+(((IkReal(-1.00000000000000))*(x474)*(x479))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj5)*(x475)*(x485)))+(((IkReal(-1.00000000000000))*(py)*(x473)*(x482)))+(((cj5)*(x475)*(x487)))+(((cj5)*(x475)*(x488)))+(((x482)*(x488)))+(((x482)*(x485)))+(((x482)*(x487)))+(((IkReal(-1.00000000000000))*(pz)*(sj5)*(sj6)))+(((cj5)*(py)*(x473)*(x475))));
evalcond[4]=((IkReal(0.0450000000000000))+(((x475)*(x487)))+(((x475)*(x485)))+(((x475)*(x488)))+(((IkReal(-1.00000000000000))*(x474)))+(x486)+(x477)+(((IkReal(-1.00000000000000))*(x479)*(x490)))+(((IkReal(-1.00000000000000))*(x479)*(x491)))+(((IkReal(-1.00000000000000))*(x479)*(x492)))+(((IkReal(-1.00000000000000))*(py)*(x473)*(x475)))+(((x479)*(x489))));
evalcond[5]=((IkReal(-0.212500000000000))+(((IkReal(0.0636396103067893))*(x479)*(x487)))+(((IkReal(0.0636396103067893))*(x479)*(x488)))+(((IkReal(-0.0636396103067893))*(x479)*(x485)))+(((IkReal(-1.00000000000000))*(x476)*(x485)))+(((IkReal(-0.0636396103067893))*(sj6)*(x488)))+(((x476)*(x488)))+(((x476)*(x487)))+(((IkReal(-1.00000000000000))*(pp)))+(((py)*(x473)*(x476)))+(((IkReal(-0.0900000000000000))*(x486)))+(((x479)*(x493)))+(((IkReal(1.10000000000000))*(x478)))+(((IkReal(-1.00000000000000))*(x473)*(x484)))+(((sj6)*(x493)))+(((IkReal(-1.00000000000000))*(x472)*(x484))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

rotationfunction0(solutions);
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
inline void rotationfunction0(IkSolutionListBase<IkReal>& solutions) {
for(int rotationiter = 0; rotationiter < 1; ++rotationiter) {
IkReal x102=((IkReal(0.707106781186548))*(cj6));
IkReal x103=((sj5)*(sj6));
IkReal x104=((IkReal(0.707106781186548))*(sj6));
IkReal x105=((cj5)*(cj7));
IkReal x106=((cj7)*(sj5));
IkReal x107=((IkReal(1.00000000000000))*(cj6));
IkReal x108=((IkReal(1.00000000000000))*(sj4));
IkReal x109=((cj5)*(sj7));
IkReal x110=((sj5)*(sj7));
IkReal x111=((cj7)*(x104));
IkReal x112=((IkReal(0.707106781186548))*(x110));
IkReal x113=((cj5)*(x104));
IkReal x114=((IkReal(0.707106781186548))*(x106));
IkReal x115=((sj7)*(x104));
IkReal x116=((IkReal(-1.00000000000000))*(sj4));
IkReal x117=((x102)*(x105));
IkReal x118=((((IkReal(-1.00000000000000))*(x113)))+(x102));
IkReal x119=((x102)*(x109));
IkReal x120=((((IkReal(-1.00000000000000))*(x102)))+(((IkReal(-1.00000000000000))*(x113))));
IkReal x121=((((IkReal(-1.00000000000000))*(x107)*(x110)))+(x105));
IkReal x122=((cj4)*(x120));
IkReal x123=((((IkReal(-1.00000000000000))*(x106)*(x107)))+(((IkReal(-1.00000000000000))*(x109))));
IkReal x124=((x114)+(x119));
IkReal x125=((((IkReal(-1.00000000000000))*(x112)))+(x111)+(x117));
IkReal x126=((((IkReal(-1.00000000000000))*(x111)))+(((IkReal(-1.00000000000000))*(x112)))+(x117));
IkReal x127=((((IkReal(-1.00000000000000))*(x115)))+(x124));
IkReal x128=((x124)+(x115));
IkReal x129=((cj4)*(x127));
IkReal x130=((((cj4)*(x118)))+(((sj4)*(x120))));
IkReal x131=((x122)+(((IkReal(-1.00000000000000))*(x108)*(x118))));
IkReal x132=((((sj4)*(x126)))+(((cj4)*(x125))));
IkReal x133=((((x116)*(x125)))+(((cj4)*(x126))));
IkReal x134=((((sj4)*(x127)))+(((cj4)*(x128))));
IkReal x135=((x129)+(((x116)*(x128))));
new_r00=((((r00)*(x133)))+(((r20)*(x123)))+(((r10)*(x132))));
new_r01=((((r21)*(x123)))+(((r01)*(x133)))+(((r11)*(x132))));
new_r02=((((r02)*(x133)))+(((r22)*(x123)))+(((r12)*(x132))));
new_r10=((((r00)*(x131)))+(((r20)*(x103)))+(((r10)*(x130))));
new_r11=((((r21)*(x103)))+(((r11)*(x130)))+(((r01)*(((x122)+(((x116)*(x118))))))));
new_r12=((((r22)*(x103)))+(((r02)*(x131)))+(((r12)*(x130))));
new_r20=((((r00)*(x135)))+(((r20)*(x121)))+(((r10)*(x134))));
new_r21=((((r01)*(((x129)+(((IkReal(-1.00000000000000))*(x108)*(x128)))))))+(((r21)*(x121)))+(((r11)*(x134))));
new_r22=((((r02)*(x135)))+(((r22)*(x121)))+(((r12)*(x134))));
{
IkReal j9array[2], cj9array[2], sj9array[2];
bool j9valid[2]={false};
_nj9 = 2;
cj9array[0]=new_r22;
if( cj9array[0] >= -1-IKFAST_SINCOS_THRESH && cj9array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j9valid[0] = j9valid[1] = true;
    j9array[0] = IKacos(cj9array[0]);
    sj9array[0] = IKsin(j9array[0]);
    cj9array[1] = cj9array[0];
    j9array[1] = -j9array[0];
    sj9array[1] = -sj9array[0];
}
else if( isnan(cj9array[0]) )
{
    // probably any value will work
    j9valid[0] = true;
    cj9array[0] = 1; sj9array[0] = 0; j9array[0] = 0;
}
for(int ij9 = 0; ij9 < 2; ++ij9)
{
if( !j9valid[ij9] )
{
    continue;
}
_ij9[0] = ij9; _ij9[1] = -1;
for(int iij9 = ij9+1; iij9 < 2; ++iij9)
{
if( j9valid[iij9] && IKabs(cj9array[ij9]-cj9array[iij9]) < IKFAST_SOLUTION_THRESH && IKabs(sj9array[ij9]-sj9array[iij9]) < IKFAST_SOLUTION_THRESH )
{
    j9valid[iij9]=false; _ij9[1] = iij9; break; 
}
}
j9 = j9array[ij9]; cj9 = cj9array[ij9]; sj9 = sj9array[ij9];

{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(sj9);
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[7];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[5]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[6]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x136=IKatan2(new_r02, new_r12);
j8array[0]=((IkReal(-1.00000000000000))*(x136));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x136))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j8))))+(((new_r12)*(IKcos(j8)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj8)))+(((cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))))+IKsqr(((((new_r10)*(sj8)))+(((cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))), ((((new_r10)*(sj8)))+(((cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x137=IKsin(j10);
IkReal x138=((IkReal(1.00000000000000))*(sj8));
IkReal x139=((IkReal(1.00000000000000))*(IKcos(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x138)))+(((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x137))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x138)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x139))));
evalcond[2]=((((new_r11)*(sj8)))+(x137)+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(x139)))+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
IkReal x140=((IkReal(1.00000000000000))+(new_r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=x140;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=new_r20;
evalcond[5]=new_r21;
evalcond[6]=x140;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  )
{
{
IkReal j8array[2], cj8array[2], sj8array[2];
bool j8valid[2]={false};
_nj8 = 2;
if( IKabs(new_r02) < IKFAST_ATAN2_MAGTHRESH && IKabs(new_r12) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x141=IKatan2(new_r02, new_r12);
j8array[0]=((IkReal(-1.00000000000000))*(x141));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
j8array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x141))));
sj8array[1]=IKsin(j8array[1]);
cj8array[1]=IKcos(j8array[1]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
if( j8array[1] > IKPI )
{
    j8array[1]-=IK2PI;
}
else if( j8array[1] < -IKPI )
{    j8array[1]+=IK2PI;
}
j8valid[1] = true;
for(int ij8 = 0; ij8 < 2; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 2; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[1];
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r02)*(IKsin(j8))))+(((new_r12)*(IKcos(j8)))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((new_r11)*(sj8)))+(((cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))), ((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x142=IKcos(j10);
IkReal x143=((IkReal(1.00000000000000))*(sj8));
IkReal x144=((IkReal(1.00000000000000))*(IKsin(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(x144)))+(((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x143))));
evalcond[1]=((((IkReal(-1.00000000000000))*(new_r01)*(x143)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x142))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x144)))+(((new_r11)*(sj8)))+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(x142)+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x145=((gconst4)*(sj9));
if( IKabs(((new_r12)*(x145))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x145))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((new_r12)*(x145)), ((new_r02)*(x145)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[6];
IkReal x146=IKsin(j8);
IkReal x147=IKcos(j8);
IkReal x148=((IkReal(1.00000000000000))*(cj9));
IkReal x149=((IkReal(1.00000000000000))*(sj9));
IkReal x150=((IkReal(1.00000000000000))*(x146));
IkReal x151=((new_r12)*(x146));
IkReal x152=((new_r02)*(x147));
evalcond[0]=((((new_r12)*(x147)))+(((IkReal(-1.00000000000000))*(new_r02)*(x150))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x149)))+(x152)+(x151));
evalcond[2]=((((IkReal(-1.00000000000000))*(new_r22)*(x149)))+(((cj9)*(x151)))+(((cj9)*(x152))));
evalcond[3]=((((IkReal(-1.00000000000000))*(new_r10)*(x146)*(x149)))+(((IkReal(-1.00000000000000))*(new_r20)*(x148)))+(((IkReal(-1.00000000000000))*(new_r00)*(x147)*(x149))));
evalcond[4]=((((IkReal(-1.00000000000000))*(new_r01)*(x147)*(x149)))+(((IkReal(-1.00000000000000))*(new_r21)*(x148)))+(((IkReal(-1.00000000000000))*(new_r11)*(x146)*(x149))));
evalcond[5]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22)*(x148)))+(((IkReal(-1.00000000000000))*(x149)*(x152)))+(((IkReal(-1.00000000000000))*(x149)*(x151))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign(sj9);
dummyeval[0]=sj9;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[11];
IkReal x153=((((IkReal(-1.00000000000000))*(new_r02)*(sj8)))+(((cj8)*(new_r12))));
IkReal x154=((((new_r12)*(sj8)))+(((cj8)*(new_r02))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))+(new_r22));
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x153;
evalcond[5]=x153;
evalcond[6]=x154;
evalcond[7]=x154;
evalcond[8]=((IkReal(-1.00000000000000))*(new_r20));
evalcond[9]=((IkReal(-1.00000000000000))*(new_r21));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(new_r22))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((new_r10)*(sj8)))+(((cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))))+IKsqr(((((new_r10)*(sj8)))+(((cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((IkReal(-1.00000000000000))*(new_r11)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r01)))), ((((new_r10)*(sj8)))+(((cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x155=IKsin(j10);
IkReal x156=((IkReal(1.00000000000000))*(sj8));
IkReal x157=((IkReal(1.00000000000000))*(IKcos(j10)));
evalcond[0]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(new_r00)*(x156))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x157)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x156))));
evalcond[2]=((((new_r11)*(sj8)))+(x155)+(((cj8)*(new_r01))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x157)))+(((new_r10)*(sj8)))+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x158=((IkReal(1.00000000000000))+(new_r22));
IkReal x159=((IkReal(1.00000000000000))*(new_r02));
IkReal x160=((new_r12)*(sj8));
IkReal x161=((((IkReal(-1.00000000000000))*(sj8)*(x159)))+(((cj8)*(new_r12))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j9)), IkReal(6.28318530717959))));
evalcond[1]=x158;
evalcond[2]=new_r20;
evalcond[3]=new_r21;
evalcond[4]=x161;
evalcond[5]=x161;
evalcond[6]=((x160)+(((cj8)*(new_r02))));
evalcond[7]=((((IkReal(-1.00000000000000))*(x160)))+(((IkReal(-1.00000000000000))*(cj8)*(x159))));
evalcond[8]=new_r20;
evalcond[9]=new_r21;
evalcond[10]=x158;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  && IKabs(evalcond[5]) < 0.0000010000000000  && IKabs(evalcond[6]) < 0.0000010000000000  && IKabs(evalcond[7]) < 0.0000010000000000  && IKabs(evalcond[8]) < 0.0000010000000000  && IKabs(evalcond[9]) < 0.0000010000000000  && IKabs(evalcond[10]) < 0.0000010000000000  )
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((((new_r11)*(sj8)))+(((cj8)*(new_r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))))+IKsqr(((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j10array[0]=IKatan2(((((new_r11)*(sj8)))+(((cj8)*(new_r01)))), ((((IkReal(-1.00000000000000))*(new_r10)*(sj8)))+(((IkReal(-1.00000000000000))*(cj8)*(new_r00)))));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[4];
IkReal x162=IKcos(j10);
IkReal x163=((IkReal(1.00000000000000))*(sj8));
IkReal x164=((IkReal(1.00000000000000))*(IKsin(j10)));
evalcond[0]=((((IkReal(-1.00000000000000))*(new_r00)*(x163)))+(((IkReal(-1.00000000000000))*(x164)))+(((cj8)*(new_r10))));
evalcond[1]=((((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(x162)))+(((IkReal(-1.00000000000000))*(new_r01)*(x163))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x164)))+(((new_r11)*(sj8)))+(((cj8)*(new_r01))));
evalcond[3]=((((new_r10)*(sj8)))+(x162)+(((cj8)*(new_r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((gconst6)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst6)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst6)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst6)*(new_r20)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[8];
IkReal x165=IKsin(j10);
IkReal x166=IKcos(j10);
IkReal x167=((IkReal(1.00000000000000))*(sj9));
IkReal x168=((new_r10)*(sj8));
IkReal x169=((cj8)*(cj9));
IkReal x170=((new_r11)*(sj8));
IkReal x171=((IkReal(1.00000000000000))*(sj8));
IkReal x172=((IkReal(1.00000000000000))*(x166));
evalcond[0]=((((sj9)*(x166)))+(new_r20));
evalcond[1]=((new_r21)+(((IkReal(-1.00000000000000))*(x165)*(x167))));
evalcond[2]=((((cj8)*(new_r10)))+(((IkReal(-1.00000000000000))*(x165)))+(((IkReal(-1.00000000000000))*(new_r00)*(x171))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x172)))+(((cj8)*(new_r11)))+(((IkReal(-1.00000000000000))*(new_r01)*(x171))));
evalcond[4]=((x170)+(((cj8)*(new_r01)))+(((cj9)*(x165))));
evalcond[5]=((x168)+(((cj8)*(new_r00)))+(((IkReal(-1.00000000000000))*(cj9)*(x172))));
evalcond[6]=((((new_r01)*(x169)))+(((IkReal(-1.00000000000000))*(new_r21)*(x167)))+(x165)+(((cj9)*(x170))));
evalcond[7]=((((new_r00)*(x169)))+(((IkReal(-1.00000000000000))*(x172)))+(((IkReal(-1.00000000000000))*(new_r20)*(x167)))+(((cj9)*(x168))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j10array[1], cj10array[1], sj10array[1];
bool j10valid[1]={false};
_nj10 = 1;
if( IKabs(((gconst5)*(new_r21))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(gconst5)*(new_r20))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j10array[0]=IKatan2(((gconst5)*(new_r21)), ((IkReal(-1.00000000000000))*(gconst5)*(new_r20)));
sj10array[0]=IKsin(j10array[0]);
cj10array[0]=IKcos(j10array[0]);
if( j10array[0] > IKPI )
{
    j10array[0]-=IK2PI;
}
else if( j10array[0] < -IKPI )
{    j10array[0]+=IK2PI;
}
j10valid[0] = true;
for(int ij10 = 0; ij10 < 1; ++ij10)
{
if( !j10valid[ij10] )
{
    continue;
}
_ij10[0] = ij10; _ij10[1] = -1;
for(int iij10 = ij10+1; iij10 < 1; ++iij10)
{
if( j10valid[iij10] && IKabs(cj10array[ij10]-cj10array[iij10]) < IKFAST_SOLUTION_THRESH && IKabs(sj10array[ij10]-sj10array[iij10]) < IKFAST_SOLUTION_THRESH )
{
    j10valid[iij10]=false; _ij10[1] = iij10; break; 
}
}
j10 = j10array[ij10]; cj10 = cj10array[ij10]; sj10 = sj10array[ij10];
{
IkReal evalcond[2];
evalcond[0]=((((sj9)*(IKcos(j10))))+(new_r20));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj9)*(IKsin(j10))))+(new_r21));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst7;
gconst7=IKsign((((new_r12)*(new_r12))+((new_r02)*(new_r02))));
dummyeval[0]=(((new_r12)*(new_r12))+((new_r02)*(new_r02)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j8array[1], cj8array[1], sj8array[1];
bool j8valid[1]={false};
_nj8 = 1;
IkReal x173=((gconst7)*(sj9));
if( IKabs(((new_r12)*(x173))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((new_r02)*(x173))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j8array[0]=IKatan2(((new_r12)*(x173)), ((new_r02)*(x173)));
sj8array[0]=IKsin(j8array[0]);
cj8array[0]=IKcos(j8array[0]);
if( j8array[0] > IKPI )
{
    j8array[0]-=IK2PI;
}
else if( j8array[0] < -IKPI )
{    j8array[0]+=IK2PI;
}
j8valid[0] = true;
for(int ij8 = 0; ij8 < 1; ++ij8)
{
if( !j8valid[ij8] )
{
    continue;
}
_ij8[0] = ij8; _ij8[1] = -1;
for(int iij8 = ij8+1; iij8 < 1; ++iij8)
{
if( j8valid[iij8] && IKabs(cj8array[ij8]-cj8array[iij8]) < IKFAST_SOLUTION_THRESH && IKabs(sj8array[ij8]-sj8array[iij8]) < IKFAST_SOLUTION_THRESH )
{
    j8valid[iij8]=false; _ij8[1] = iij8; break; 
}
}
j8 = j8array[ij8]; cj8 = cj8array[ij8]; sj8 = sj8array[ij8];
{
IkReal evalcond[12];
IkReal x174=IKsin(j8);
IkReal x175=IKcos(j8);
IkReal x176=((IkReal(1.00000000000000))*(cj10));
IkReal x177=((IkReal(1.00000000000000))*(sj9));
IkReal x178=((IkReal(1.00000000000000))*(cj9));
IkReal x179=((IkReal(1.00000000000000))*(x174));
IkReal x180=((new_r01)*(x175));
IkReal x181=((new_r11)*(x174));
IkReal x182=((cj9)*(x175));
IkReal x183=((new_r00)*(x175));
IkReal x184=((new_r02)*(x175));
IkReal x185=((new_r10)*(x174));
IkReal x186=((new_r12)*(x174));
evalcond[0]=((((new_r12)*(x175)))+(((IkReal(-1.00000000000000))*(new_r02)*(x179))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x177)))+(x184)+(x186));
evalcond[2]=((((new_r10)*(x175)))+(((IkReal(-1.00000000000000))*(sj10)))+(((IkReal(-1.00000000000000))*(new_r00)*(x179))));
evalcond[3]=((((new_r11)*(x175)))+(((IkReal(-1.00000000000000))*(x176)))+(((IkReal(-1.00000000000000))*(new_r01)*(x179))));
evalcond[4]=((x180)+(x181)+(((cj9)*(sj10))));
evalcond[5]=((x185)+(x183)+(((IkReal(-1.00000000000000))*(cj9)*(x176))));
evalcond[6]=((((new_r02)*(x182)))+(((IkReal(-1.00000000000000))*(new_r22)*(x177)))+(((cj9)*(x186))));
evalcond[7]=((((IkReal(-1.00000000000000))*(new_r20)*(x178)))+(((IkReal(-1.00000000000000))*(x177)*(x185)))+(((IkReal(-1.00000000000000))*(x177)*(x183))));
evalcond[8]=((((IkReal(-1.00000000000000))*(new_r21)*(x178)))+(((IkReal(-1.00000000000000))*(x177)*(x180)))+(((IkReal(-1.00000000000000))*(x177)*(x181))));
evalcond[9]=((((IkReal(-1.00000000000000))*(new_r21)*(x177)))+(sj10)+(((cj9)*(x180)))+(((cj9)*(x181))));
evalcond[10]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x177)*(x186)))+(((IkReal(-1.00000000000000))*(x177)*(x184)))+(((IkReal(-1.00000000000000))*(new_r22)*(x178))));
evalcond[11]=((((new_r00)*(x182)))+(((IkReal(-1.00000000000000))*(x176)))+(((IkReal(-1.00000000000000))*(new_r20)*(x177)))+(((cj9)*(x185))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  || IKabs(evalcond[6]) > 0.000001  || IKabs(evalcond[7]) > 0.000001  || IKabs(evalcond[8]) > 0.000001  || IKabs(evalcond[9]) > 0.000001  || IKabs(evalcond[10]) > 0.000001  || IKabs(evalcond[11]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(7);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j4;
vinfos[0].indices[0] = _ij4[0];
vinfos[0].indices[1] = _ij4[1];
vinfos[0].maxsolutions = _nj4;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j5;
vinfos[1].indices[0] = _ij5[0];
vinfos[1].indices[1] = _ij5[1];
vinfos[1].maxsolutions = _nj5;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j6;
vinfos[2].indices[0] = _ij6[0];
vinfos[2].indices[1] = _ij6[1];
vinfos[2].maxsolutions = _nj6;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j7;
vinfos[3].indices[0] = _ij7[0];
vinfos[3].indices[1] = _ij7[1];
vinfos[3].maxsolutions = _nj7;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j8;
vinfos[4].indices[0] = _ij8[0];
vinfos[4].indices[1] = _ij8[1];
vinfos[4].maxsolutions = _nj8;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j9;
vinfos[5].indices[0] = _ij9[0];
vinfos[5].indices[1] = _ij9[1];
vinfos[5].maxsolutions = _nj9;
vinfos[6].jointtype = 1;
vinfos[6].foffset = j10;
vinfos[6].indices[0] = _ij10[0];
vinfos[6].indices[1] = _ij10[1];
vinfos[6].maxsolutions = _nj10;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ba32f85ca3c87502739320e9e26a5905"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
