/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:23:09.435154
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21;
x0=IKcos(j[0]);
x1=IKsin(j[1]);
x2=IKcos(j[3]);
x3=IKsin(j[3]);
x4=IKcos(j[1]);
x5=IKcos(j[2]);
x6=IKsin(j[0]);
x7=IKsin(j[2]);
x8=((IkReal(0.0450000000000000))*(x3));
x9=((IkReal(0.350000000000000))*(x4));
x10=((IkReal(0.350000000000000))*(x7));
x11=((IkReal(0.0450000000000000))*(x6));
x12=((IkReal(0.350000000000000))*(x2));
x13=((x0)*(x1));
x14=((x1)*(x5));
x15=((x0)*(x5));
x16=((x1)*(x6));
x17=((x11)*(x7));
x18=((IkReal(0.0450000000000000))*(x0)*(x7));
x19=((IkReal(0.0450000000000000))*(x15)*(x4));
x20=((x11)*(x4)*(x5));
x21=((x18)+(x20));
eetrans[0]=((IkReal(0.220000000000000))+(((IkReal(0.550000000000000))*(x13)))+(((IkReal(-1.00000000000000))*(x17)))+(x19)+(((x12)*(x13)))+(((x2)*(((((IkReal(-1.00000000000000))*(x19)))+(x17)))))+(((x3)*(((((IkReal(-1.00000000000000))*(x10)*(x6)))+(((x15)*(x9)))))))+(((x13)*(x8))));
eetrans[1]=((IkReal(0.140000000000000))+(((IkReal(0.550000000000000))*(x16)))+(((IkReal(-1.00000000000000))*(x2)*(x21)))+(x21)+(((x3)*(((((x0)*(x10)))+(((x5)*(x6)*(x9)))))))+(((x16)*(x8)))+(((x12)*(x16))));
IkReal x22=((IkReal(0.0450000000000000))*(x14));
eetrans[2]=((IkReal(0.346000000000000))+(((IkReal(-1.00000000000000))*(x22)))+(((x4)*(x8)))+(((x2)*(x22)))+(((x2)*(x9)))+(((IkReal(-0.350000000000000))*(x14)*(x3)))+(((IkReal(0.550000000000000))*(x4))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 4; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x33000003; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j1,cj1,sj1,htj1,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_px=((IkReal(-0.220000000000000))+(px));
new_py=((IkReal(-0.140000000000000))+(py));
new_pz=((IkReal(-0.346000000000000))+(pz));
px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
if( (((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp))))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x23=IKasin(((IkReal(1.10163610111212))+(((IkReal(-2.56761706354067))*(pp)))));
j3array[0]=((IkReal(-1.36129006610583))+(((IkReal(-1.00000000000000))*(x23))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(1.78030258748397))+(x23));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
IkReal x24=(sj1)*(sj1);
dummyeval[0]=((((x24)*((px)*(px))))+(((x24)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x25=(sj1)*(sj1);
dummyeval[0]=((((x25)*((px)*(px))))+(((x25)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x26=(sj1)*(sj1);
dummyeval[0]=((((x26)*((px)*(px)*(px)*(px))))+(((x26)*((px)*(px))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=((IkReal(-1.00000000000000))*(py));
IkReal gconst4;
gconst4=((IkReal(2.00000000000000))*(px));
IkReal gconst5;
gconst5=py;
IkReal gconst6;
gconst6=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst7;
gconst7=((IkReal(-2.00000000000000))*(py));
IkReal gconst8;
gconst8=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst9;
gconst9=((IkReal(-1.00000000000000))*(py));
IkReal gconst10;
gconst10=((IkReal(2.00000000000000))*(px));
IkReal gconst11;
gconst11=py;
IkReal gconst12;
gconst12=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst13;
gconst13=((IkReal(-2.00000000000000))*(py));
IkReal gconst14;
gconst14=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst15;
gconst15=((IkReal(2.00000000000000))*(px));
IkReal gconst16;
gconst16=((IkReal(4.00000000000000))*(py));
IkReal gconst17;
gconst17=((IkReal(-2.00000000000000))*(px));
IkReal gconst18;
gconst18=((IkReal(-2.00000000000000))*(py));
IkReal gconst19;
gconst19=((IkReal(4.00000000000000))*(px));
IkReal gconst20;
gconst20=((IkReal(2.00000000000000))*(py));
IkReal gconst21;
gconst21=((IkReal(2.00000000000000))*(px));
IkReal gconst22;
gconst22=((IkReal(4.00000000000000))*(py));
IkReal gconst23;
gconst23=((IkReal(-2.00000000000000))*(px));
IkReal gconst24;
gconst24=((IkReal(-2.00000000000000))*(py));
IkReal gconst25;
gconst25=((IkReal(4.00000000000000))*(px));
IkReal gconst26;
gconst26=((IkReal(2.00000000000000))*(py));
IkReal gconst27;
gconst27=py;
IkReal gconst28;
gconst28=((IkReal(-2.00000000000000))*(px));
IkReal gconst29;
gconst29=((IkReal(-1.00000000000000))*(py));
IkReal gconst30;
gconst30=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst31;
gconst31=((IkReal(2.00000000000000))*(py));
IkReal gconst32;
gconst32=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst33;
gconst33=py;
IkReal gconst34;
gconst34=((IkReal(-2.00000000000000))*(px));
IkReal gconst35;
gconst35=((IkReal(-1.00000000000000))*(py));
IkReal gconst36;
gconst36=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst37;
gconst37=((IkReal(2.00000000000000))*(py));
IkReal gconst38;
gconst38=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal x27=((gconst33)*(gconst37));
IkReal x28=((gconst28)*(gconst32));
IkReal x29=((gconst29)*(gconst31));
IkReal x30=((gconst34)*(gconst36));
IkReal x31=((gconst27)*(gconst32));
IkReal x32=((gconst33)*(gconst38));
IkReal x33=((gconst35)*(gconst36));
IkReal x34=((gconst29)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x27)*(x29)))+(((x29)*(x30)))+(((IkReal(-1.00000000000000))*(x31)*(x32)))+(((IkReal(-1.00000000000000))*(x28)*(x30)))+(((x32)*(x34)))+(((IkReal(-1.00000000000000))*(x33)*(x34)))+(((x31)*(x33)))+(((x27)*(x28))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x35=((gconst32)*(gconst36));
IkReal x36=((gconst32)*(gconst33));
IkReal x37=((gconst28)*(gconst37));
IkReal x38=((IkReal(1.00000000000000))*(gconst34));
IkReal x39=((gconst33)*(gconst38));
IkReal x40=((gconst17)*(gconst30));
IkReal x41=((gconst26)*(gconst9));
IkReal x42=((gconst5)*(gconst6));
IkReal x43=((IkReal(1.00000000000000))*(gconst8));
IkReal x44=((gconst14)*(gconst9));
IkReal x45=((gconst18)*(gconst5));
IkReal x46=((IkReal(1.00000000000000))*(gconst27));
IkReal x47=((gconst10)*(gconst12));
IkReal x48=((gconst17)*(gconst7));
IkReal x49=((gconst17)*(gconst6));
IkReal x50=((gconst5)*(gconst7));
IkReal x51=((gconst12)*(gconst3));
IkReal x52=((gconst13)*(gconst9));
IkReal x53=((gconst27)*(gconst35));
IkReal x54=((gconst14)*(gconst21));
IkReal x55=((gconst12)*(gconst22));
IkReal x56=((gconst4)*(gconst8));
IkReal x57=((gconst13)*(gconst21));
IkReal x58=((gconst31)*(gconst36));
IkReal x59=((gconst20)*(gconst36));
IkReal x60=((gconst21)*(gconst38));
IkReal x61=((IkReal(1.00000000000000))*(gconst20));
IkReal x62=((IkReal(1.00000000000000))*(gconst29));
IkReal x63=((gconst33)*(gconst37));
IkReal x64=((gconst30)*(gconst35));
IkReal x65=((gconst29)*(gconst30));
IkReal x66=((gconst25)*(gconst9));
IkReal x67=((gconst11)*(gconst24));
IkReal x68=((gconst24)*(gconst32));
IkReal x69=((gconst10)*(gconst24));
IkReal x70=((gconst11)*(gconst8));
IkReal x71=((gconst19)*(gconst5));
IkReal x72=((gconst29)*(gconst31)*(gconst34));
IkReal x73=((IkReal(1.00000000000000))*(gconst11)*(gconst12));
IkReal x74=((gconst36)*(x62));
IkReal x75=((gconst31)*(gconst33)*(x62));
op[0]=((((x39)*(x65)))+(((x36)*(x37)))+(((x35)*(x53)))+(((IkReal(-1.00000000000000))*(gconst28)*(x35)*(x38)))+(((gconst29)*(gconst34)*(x58)))+(((IkReal(-1.00000000000000))*(gconst38)*(x36)*(x46)))+(((IkReal(-1.00000000000000))*(gconst31)*(x62)*(x63)))+(((IkReal(-1.00000000000000))*(x64)*(x74))));
op[1]=((((IkReal(-1.00000000000000))*(gconst35)*(gconst36)*(x40)))+(((IkReal(-1.00000000000000))*(gconst25)*(x75)))+(((gconst21)*(gconst32)*(x37)))+(((IkReal(-1.00000000000000))*(gconst16)*(x35)*(x38)))+(((x60)*(x65)))+(((gconst17)*(gconst34)*(x58)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst28)*(x35)))+(((IkReal(-1.00000000000000))*(gconst19)*(x62)*(x63)))+(((IkReal(-1.00000000000000))*(gconst26)*(x36)*(x46)))+(((x39)*(x40)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst30)*(x74)))+(((IkReal(-1.00000000000000))*(gconst28)*(x38)*(x68)))+(((gconst20)*(gconst33)*(x37)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst31)*(x63)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst35)*(x74)))+(((gconst23)*(gconst27)*(x35)))+(((gconst19)*(gconst29)*(gconst34)*(gconst36)))+(((gconst22)*(gconst29)*(x58)))+(((gconst16)*(gconst37)*(x36)))+(((IkReal(-1.00000000000000))*(gconst28)*(x38)*(x59)))+(((IkReal(-1.00000000000000))*(gconst32)*(x46)*(x60)))+(((IkReal(-1.00000000000000))*(gconst20)*(x39)*(x46)))+(((x53)*(x68)))+(((gconst24)*(x72)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst31)*(gconst37)*(x62)))+(((x53)*(x59)))+(((IkReal(-1.00000000000000))*(gconst24)*(x62)*(x64)))+(((gconst26)*(gconst33)*(x65)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst38)*(x36)))+(((gconst18)*(gconst29)*(x39)))+(((gconst15)*(gconst35)*(x35)))+(((gconst25)*(gconst28)*(x36))));
op[2]=((((gconst16)*(gconst21)*(gconst32)*(gconst37)))+(((gconst27)*(gconst35)*(gconst36)*(gconst8)))+(((gconst10)*(gconst29)*(gconst31)*(gconst36)))+(((gconst21)*(gconst25)*(gconst28)*(gconst32)))+(((gconst18)*(gconst21)*(gconst29)*(gconst38)))+(((gconst16)*(gconst20)*(gconst33)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst24)*(gconst32)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst29)*(gconst35)))+(((gconst21)*(gconst26)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst30)*(gconst36)))+(((gconst20)*(gconst24)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst34)*(gconst36)))+(((gconst12)*(gconst27)*(gconst32)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst31)))+(((gconst30)*(gconst33)*(gconst38)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst30)*(gconst35)))+(((gconst31)*(gconst34)*(gconst36)*(gconst5)))+(((gconst17)*(gconst22)*(gconst31)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst33)*(gconst38)))+(((gconst28)*(gconst32)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst32)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst29)*(gconst30)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst28)*(gconst32)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst24)*(gconst28)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst33)))+(((gconst16)*(gconst25)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst31)*(gconst37)*(gconst9)))+(((gconst17)*(gconst18)*(gconst33)*(gconst38)))+(((gconst28)*(gconst33)*(gconst37)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst32)*(gconst34)*(gconst36)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst33)*(gconst37)))+(((gconst29)*(gconst30)*(gconst38)*(gconst9)))+(((gconst11)*(gconst27)*(gconst32)*(gconst36)))+(((gconst18)*(gconst26)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst30)*(gconst35)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst33)*(gconst37)*(gconst7)))+(((gconst29)*(gconst33)*(gconst38)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst31)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst32)*(gconst33)*(gconst38)))+(((IkReal(-1.00000000000000))*(gconst28)*(gconst34)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst28)*(gconst32)))+(((gconst14)*(gconst29)*(gconst30)*(gconst33)))+(((gconst17)*(gconst26)*(gconst30)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst31)*(gconst33)*(gconst37)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst32)*(gconst38)))+(((gconst19)*(gconst24)*(gconst29)*(gconst34)))+(((gconst15)*(gconst23)*(gconst32)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst29)*(gconst37)))+(((gconst15)*(gconst24)*(gconst32)*(gconst35)))+(((gconst13)*(gconst28)*(gconst32)*(gconst33)))+(((gconst20)*(gconst21)*(gconst28)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst28)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst33)*(gconst38)*(gconst8)))+(((gconst12)*(gconst29)*(gconst31)*(gconst34)))+(((gconst23)*(gconst24)*(gconst27)*(gconst32)))+(((gconst22)*(gconst24)*(gconst29)*(gconst31)))+(((gconst17)*(gconst19)*(gconst34)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst32)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst35)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst33)))+(((gconst19)*(gconst22)*(gconst29)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst35)*(gconst36)))+(((gconst29)*(gconst34)*(gconst36)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst28)*(gconst32)*(gconst34)))+(((gconst32)*(gconst33)*(gconst37)*(gconst4)))+(((gconst3)*(gconst32)*(gconst35)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst27)*(gconst38)))+(((gconst20)*(gconst25)*(gconst28)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst29)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst33)))+(((gconst15)*(gconst20)*(gconst35)*(gconst36)))+(((gconst20)*(gconst23)*(gconst27)*(gconst36)))+(((gconst17)*(gconst24)*(gconst31)*(gconst34)))+(((gconst17)*(gconst21)*(gconst30)*(gconst38))));
op[3]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst23)*(gconst27)*(gconst36)*(gconst8)))+(((gconst19)*(gconst22)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst34)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst34)*(gconst36)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((gconst17)*(gconst18)*(gconst21)*(gconst38)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst29)*(gconst37)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst32)*(gconst34)*(gconst4)))+(((gconst20)*(gconst28)*(gconst37)*(gconst9)))+(((gconst26)*(gconst29)*(gconst33)*(gconst6)))+(((gconst20)*(gconst3)*(gconst35)*(gconst36)))+(((gconst24)*(gconst27)*(gconst35)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst15)*(gconst20)*(gconst24)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst35)*(gconst36)*(gconst5)))+(((gconst12)*(gconst17)*(gconst31)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst29)*(gconst37)*(gconst9)))+(((gconst16)*(gconst20)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst24)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst33)*(gconst38)*(gconst8)))+(((gconst12)*(gconst22)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst24)*(gconst34)))+(((gconst15)*(gconst20)*(gconst23)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((gconst24)*(gconst31)*(gconst34)*(gconst5)))+(((gconst24)*(gconst29)*(gconst34)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst11)*(gconst15)*(gconst32)*(gconst36)))+(((gconst14)*(gconst18)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((gconst20)*(gconst33)*(gconst37)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst32)*(gconst36)))+(((gconst17)*(gconst33)*(gconst38)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst33)*(gconst37)*(gconst5)))+(((gconst21)*(gconst29)*(gconst38)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((gconst17)*(gconst19)*(gconst22)*(gconst36)))+(((gconst17)*(gconst21)*(gconst26)*(gconst30)))+(((gconst20)*(gconst23)*(gconst24)*(gconst27)))+(((gconst10)*(gconst17)*(gconst31)*(gconst36)))+(((gconst17)*(gconst19)*(gconst24)*(gconst34)))+(((gconst26)*(gconst29)*(gconst30)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst28)*(gconst34)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst29)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst29)*(gconst36)))+(((gconst11)*(gconst24)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst32)*(gconst36)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst32)*(gconst34)))+(((gconst14)*(gconst21)*(gconst29)*(gconst30)))+(((gconst15)*(gconst23)*(gconst24)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst17)*(gconst22)*(gconst24)*(gconst31)))+(((gconst16)*(gconst33)*(gconst37)*(gconst8)))+(((gconst16)*(gconst32)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst30)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst28)*(gconst36)*(gconst8)))+(((gconst17)*(gconst18)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((gconst22)*(gconst29)*(gconst36)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst12)*(gconst23)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst3)*(gconst33)*(gconst38)))+(((gconst16)*(gconst20)*(gconst21)*(gconst37)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst35)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst31)*(gconst37)*(gconst5)))+(((gconst10)*(gconst24)*(gconst29)*(gconst31)))+(((gconst19)*(gconst34)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst28)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst27)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst38)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst33)*(gconst37)*(gconst7)))+(((gconst16)*(gconst21)*(gconst25)*(gconst32)))+(((gconst25)*(gconst28)*(gconst33)*(gconst8)))+(((gconst24)*(gconst3)*(gconst32)*(gconst35)))+(((gconst12)*(gconst15)*(gconst32)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst30)*(gconst36)*(gconst5)))+(((gconst13)*(gconst20)*(gconst28)*(gconst33)))+(((gconst21)*(gconst28)*(gconst37)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((gconst13)*(gconst16)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst10)*(gconst19)*(gconst29)*(gconst36)))+(((gconst20)*(gconst21)*(gconst25)*(gconst28)))+(((gconst22)*(gconst31)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst22)*(gconst36)))+(((gconst21)*(gconst32)*(gconst37)*(gconst4)))+(((gconst12)*(gconst19)*(gconst29)*(gconst34)))+(((gconst11)*(gconst20)*(gconst27)*(gconst36)))+(((gconst18)*(gconst21)*(gconst26)*(gconst29)))+(((gconst12)*(gconst20)*(gconst27)*(gconst35)))+(((gconst13)*(gconst21)*(gconst28)*(gconst32)))+(((gconst17)*(gconst30)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((gconst14)*(gconst17)*(gconst30)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst31)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst22)*(gconst28)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst3)*(gconst32)*(gconst38)))+(((gconst23)*(gconst3)*(gconst32)*(gconst36)))+(((gconst21)*(gconst30)*(gconst38)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst36)))+(((gconst18)*(gconst33)*(gconst38)*(gconst5)))+(((gconst25)*(gconst32)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst15)*(gconst35)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst28)*(gconst34)))+(((gconst17)*(gconst34)*(gconst36)*(gconst7)))+(((gconst18)*(gconst29)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst24)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst32)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst28)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst27)*(gconst38)*(gconst8)))+(((gconst26)*(gconst30)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst37)))+(((gconst25)*(gconst28)*(gconst32)*(gconst9))));
op[4]=((((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst29)*(gconst38)*(gconst6)*(gconst9)))+(((gconst15)*(gconst24)*(gconst35)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((gconst33)*(gconst38)*(gconst5)*(gconst6)))+(((gconst13)*(gconst16)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst28)*(gconst36)*(gconst8)))+(((gconst12)*(gconst29)*(gconst34)*(gconst7)))+(((gconst12)*(gconst15)*(gconst23)*(gconst32)))+(((gconst11)*(gconst15)*(gconst24)*(gconst32)))+(((gconst19)*(gconst22)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst11)*(gconst15)*(gconst20)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst30)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst33)*(gconst37)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((gconst10)*(gconst12)*(gconst29)*(gconst31)))+(((gconst12)*(gconst31)*(gconst34)*(gconst5)))+(((gconst22)*(gconst24)*(gconst31)*(gconst5)))+(((gconst17)*(gconst22)*(gconst36)*(gconst7)))+(((gconst17)*(gconst19)*(gconst22)*(gconst24)))+(((gconst10)*(gconst31)*(gconst36)*(gconst5)))+(((gconst23)*(gconst24)*(gconst3)*(gconst32)))+(((gconst12)*(gconst20)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst38)*(gconst8)))+(((gconst17)*(gconst18)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst16)*(gconst21)*(gconst37)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst29)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst29)*(gconst33)*(gconst6)))+(((gconst10)*(gconst29)*(gconst36)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst24)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst27)*(gconst38)*(gconst8)*(gconst9)))+(((gconst11)*(gconst12)*(gconst27)*(gconst32)))+(((gconst13)*(gconst32)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst24)*(gconst32)))+(((gconst11)*(gconst20)*(gconst24)*(gconst27)))+(((gconst16)*(gconst25)*(gconst32)*(gconst9)))+(((gconst20)*(gconst23)*(gconst3)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst22)*(gconst28)))+(((gconst21)*(gconst25)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((gconst12)*(gconst15)*(gconst20)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst33)*(gconst38)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((gconst12)*(gconst27)*(gconst35)*(gconst8)))+(((gconst28)*(gconst37)*(gconst8)*(gconst9)))+(((gconst19)*(gconst24)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst37)*(gconst7)))+(((gconst18)*(gconst26)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst19)*(gconst36)))+(((gconst15)*(gconst20)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((gconst12)*(gconst19)*(gconst22)*(gconst29)))+(((gconst21)*(gconst26)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst17)*(gconst21)*(gconst38)*(gconst6)))+(((gconst12)*(gconst17)*(gconst22)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst36)*(gconst4)))+(((gconst11)*(gconst27)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst20)*(gconst25)*(gconst33)*(gconst4)))+(((gconst17)*(gconst18)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst31)*(gconst37)*(gconst5)*(gconst9)))+(((gconst21)*(gconst26)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((gconst17)*(gconst24)*(gconst34)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst20)*(gconst34)))+(((gconst13)*(gconst16)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst14)*(gconst30)*(gconst33)*(gconst5)))+(((gconst30)*(gconst38)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst24)*(gconst34)*(gconst8)))+(((gconst22)*(gconst24)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst24)*(gconst28)))+(((gconst14)*(gconst17)*(gconst18)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst20)*(gconst36)))+(((gconst13)*(gconst28)*(gconst32)*(gconst9)))+(((gconst15)*(gconst23)*(gconst36)*(gconst8)))+(((gconst16)*(gconst20)*(gconst21)*(gconst25)))+(((gconst14)*(gconst18)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst37)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst29)*(gconst37)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst34)*(gconst36)*(gconst4)*(gconst8)))+(((gconst12)*(gconst3)*(gconst32)*(gconst35)))+(((gconst14)*(gconst17)*(gconst21)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst35)*(gconst36)*(gconst5)*(gconst6)))+(((gconst10)*(gconst19)*(gconst24)*(gconst29)))+(((gconst3)*(gconst35)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst32)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst28)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst22)*(gconst32)))+(((gconst20)*(gconst21)*(gconst37)*(gconst4)))+(((gconst23)*(gconst24)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst10)*(gconst17)*(gconst24)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst3)*(gconst38)))+(((gconst13)*(gconst20)*(gconst21)*(gconst28)))+(((gconst14)*(gconst29)*(gconst30)*(gconst9)))+(((gconst32)*(gconst37)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst16)*(gconst25)*(gconst33)*(gconst8)))+(((gconst34)*(gconst36)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((gconst18)*(gconst21)*(gconst38)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst28)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst36)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst36)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((gconst20)*(gconst25)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst12)*(gconst17)*(gconst19)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst22)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst38)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst28)*(gconst34)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst32)*(gconst36)*(gconst4)))+(((gconst20)*(gconst24)*(gconst3)*(gconst35)))+(((gconst33)*(gconst37)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst37)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst17)*(gconst26)*(gconst33)*(gconst6)))+(((gconst18)*(gconst26)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst3)*(gconst32)*(gconst36)))+(((gconst21)*(gconst25)*(gconst28)*(gconst8)))+(((gconst17)*(gconst26)*(gconst30)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst32)*(gconst38)*(gconst9)))+(((gconst16)*(gconst20)*(gconst37)*(gconst9)))+(((gconst13)*(gconst28)*(gconst33)*(gconst8))));
op[5]=((((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst24)*(gconst34)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((gconst14)*(gconst17)*(gconst30)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst36)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst11)*(gconst15)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst37)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((gconst13)*(gconst21)*(gconst32)*(gconst4)))+(((gconst10)*(gconst12)*(gconst19)*(gconst29)))+(((gconst12)*(gconst23)*(gconst27)*(gconst8)))+(((gconst12)*(gconst15)*(gconst20)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst22)*(gconst24)*(gconst4)))+(((gconst16)*(gconst37)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst22)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((gconst17)*(gconst38)*(gconst6)*(gconst9)))+(((gconst15)*(gconst23)*(gconst24)*(gconst8)))+(((gconst14)*(gconst21)*(gconst29)*(gconst6)))+(((gconst12)*(gconst22)*(gconst31)*(gconst5)))+(((gconst12)*(gconst15)*(gconst35)*(gconst8)))+(((gconst21)*(gconst37)*(gconst4)*(gconst8)))+(((gconst23)*(gconst3)*(gconst36)*(gconst8)))+(((gconst26)*(gconst29)*(gconst6)*(gconst9)))+(((gconst25)*(gconst32)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst16)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst34)*(gconst4)))+(((gconst18)*(gconst21)*(gconst26)*(gconst5)))+(((gconst11)*(gconst24)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst36)*(gconst5)))+(((gconst25)*(gconst28)*(gconst8)*(gconst9)))+(((gconst10)*(gconst24)*(gconst29)*(gconst7)))+(((gconst22)*(gconst36)*(gconst5)*(gconst7)))+(((gconst16)*(gconst21)*(gconst25)*(gconst8)))+(((gconst10)*(gconst12)*(gconst17)*(gconst31)))+(((gconst10)*(gconst19)*(gconst36)*(gconst5)))+(((gconst12)*(gconst17)*(gconst34)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((gconst11)*(gconst24)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst36)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst3)*(gconst38)*(gconst8)))+(((gconst10)*(gconst17)*(gconst19)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst3)*(gconst38)*(gconst9)))+(((gconst11)*(gconst20)*(gconst3)*(gconst36)))+(((gconst10)*(gconst17)*(gconst36)*(gconst7)))+(((gconst20)*(gconst23)*(gconst24)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst20)*(gconst28)))+(((gconst13)*(gconst21)*(gconst28)*(gconst8)))+(((gconst12)*(gconst22)*(gconst29)*(gconst7)))+(((gconst17)*(gconst21)*(gconst26)*(gconst6)))+(((gconst26)*(gconst33)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst22)*(gconst28)*(gconst8)))+(((gconst14)*(gconst17)*(gconst18)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((gconst16)*(gconst20)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst24)*(gconst28)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst36)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst17)*(gconst33)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst21)*(gconst38)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst20)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst34)*(gconst8)))+(((gconst12)*(gconst20)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst20)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst20)*(gconst37)*(gconst4)*(gconst9)))+(((gconst13)*(gconst16)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst37)*(gconst5)*(gconst9)))+(((gconst24)*(gconst3)*(gconst35)*(gconst8)))+(((gconst10)*(gconst24)*(gconst31)*(gconst5)))+(((gconst12)*(gconst19)*(gconst34)*(gconst5)))+(((gconst17)*(gconst18)*(gconst26)*(gconst9)))+(((gconst25)*(gconst33)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((gconst13)*(gconst20)*(gconst33)*(gconst4)))+(((gconst26)*(gconst30)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst37)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((gconst12)*(gconst23)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst24)*(gconst8)))+(((gconst20)*(gconst21)*(gconst25)*(gconst4)))+(((gconst14)*(gconst21)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst38)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst36)*(gconst4)))+(((gconst13)*(gconst16)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst19)*(gconst22)*(gconst24)*(gconst5)))+(((gconst11)*(gconst15)*(gconst20)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst36)*(gconst4)*(gconst8)))+(((gconst13)*(gconst20)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((gconst11)*(gconst12)*(gconst15)*(gconst32)))+(((gconst13)*(gconst16)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst34)*(gconst4)*(gconst8)))+(((gconst18)*(gconst38)*(gconst5)*(gconst9)))+(((gconst14)*(gconst18)*(gconst33)*(gconst5)))+(((gconst11)*(gconst12)*(gconst20)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((gconst17)*(gconst22)*(gconst24)*(gconst7)))+(((gconst12)*(gconst17)*(gconst19)*(gconst22)))+(((gconst14)*(gconst18)*(gconst29)*(gconst9))));
op[6]=((((gconst13)*(gconst32)*(gconst4)*(gconst9)))+(((gconst14)*(gconst33)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst13)*(gconst16)*(gconst20)*(gconst9)))+(((gconst13)*(gconst33)*(gconst4)*(gconst8)))+(((gconst16)*(gconst25)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst16)*(gconst20)))+(((gconst14)*(gconst29)*(gconst6)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst37)*(gconst4)*(gconst8)*(gconst9)))+(((gconst38)*(gconst5)*(gconst6)*(gconst9)))+(((gconst10)*(gconst36)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst36)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((gconst11)*(gconst3)*(gconst36)*(gconst8)))+(((gconst13)*(gconst20)*(gconst21)*(gconst4)))+(((gconst20)*(gconst25)*(gconst4)*(gconst9)))+(((gconst12)*(gconst15)*(gconst23)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((gconst12)*(gconst34)*(gconst5)*(gconst7)))+(((gconst11)*(gconst12)*(gconst27)*(gconst8)))+(((gconst17)*(gconst26)*(gconst6)*(gconst9)))+(((gconst11)*(gconst12)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((gconst12)*(gconst3)*(gconst35)*(gconst8)))+(((gconst10)*(gconst17)*(gconst24)*(gconst7)))+(((gconst18)*(gconst26)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst22)*(gconst8)))+(((gconst23)*(gconst24)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst13)*(gconst16)*(gconst21)*(gconst8)))+(((gconst12)*(gconst20)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst4)*(gconst8)))+(((gconst10)*(gconst12)*(gconst17)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst34)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst12)*(gconst15)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst20)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst32)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst12)*(gconst28)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst38)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst17)*(gconst22)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((gconst12)*(gconst19)*(gconst22)*(gconst5)))+(((gconst10)*(gconst12)*(gconst29)*(gconst7)))+(((gconst14)*(gconst18)*(gconst21)*(gconst5)))+(((gconst22)*(gconst24)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((gconst21)*(gconst25)*(gconst4)*(gconst8)))+(((gconst21)*(gconst26)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((gconst14)*(gconst30)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst14)*(gconst17)*(gconst21)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst36)*(gconst4)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst37)*(gconst5)*(gconst7)*(gconst9)))+(((gconst10)*(gconst19)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((gconst13)*(gconst28)*(gconst8)*(gconst9)))+(((gconst14)*(gconst17)*(gconst18)*(gconst9)))+(((gconst11)*(gconst15)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst20)*(gconst24)*(gconst4)))+(((gconst10)*(gconst12)*(gconst31)*(gconst5)))+(((gconst11)*(gconst20)*(gconst24)*(gconst3))));
op[7]=((((IkReal(-1.00000000000000))*(gconst4)*(x47)*(x61)))+(((IkReal(-1.00000000000000))*(x48)*(x52)))+(((IkReal(-1.00000000000000))*(x52)*(x71)))+(((x41)*(x42)))+(((gconst23)*(gconst8)*(x51)))+(((IkReal(-1.00000000000000))*(x45)*(x73)))+(((IkReal(-1.00000000000000))*(gconst15)*(x43)*(x44)))+(((IkReal(-1.00000000000000))*(gconst4)*(x43)*(x69)))+(((IkReal(-1.00000000000000))*(x42)*(x67)))+(((x47)*(x48)))+(((IkReal(-1.00000000000000))*(gconst4)*(x43)*(x55)))+(((gconst12)*(gconst15)*(x70)))+(((x47)*(x71)))+(((IkReal(-1.00000000000000))*(gconst16)*(x43)*(x47)))+(((IkReal(-1.00000000000000))*(x50)*(x66)))+(((IkReal(-1.00000000000000))*(gconst3)*(x41)*(x43)))+(((x42)*(x54)))+(((x50)*(x55)))+(((IkReal(-1.00000000000000))*(x50)*(x57)))+(((IkReal(-1.00000000000000))*(x49)*(x73)))+(((x50)*(x69)))+(((x56)*(x66)))+(((gconst16)*(gconst8)*(x52)))+(((x56)*(x57)))+(((gconst3)*(gconst8)*(x67)))+(((x44)*(x49)))+(((x44)*(x45)))+(((IkReal(-1.00000000000000))*(gconst3)*(x44)*(x61)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(x42)))+(((IkReal(-1.00000000000000))*(gconst3)*(x43)*(x54)))+(((gconst20)*(gconst4)*(x52)))+(((gconst11)*(gconst20)*(x51))));
op[8]=((((IkReal(-1.00000000000000))*(x42)*(x73)))+(((IkReal(-1.00000000000000))*(x50)*(x52)))+(((x42)*(x44)))+(((x51)*(x70)))+(((IkReal(-1.00000000000000))*(gconst3)*(x43)*(x44)))+(((IkReal(-1.00000000000000))*(gconst4)*(x43)*(x47)))+(((x52)*(x56)))+(((x47)*(x50))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x76=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(pz));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj0)*(x76)))+(((px)*(sj0))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj0)*(x76)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
IkReal x77=(px)*(px);
IkReal x78=(sj0)*(sj0);
IkReal x79=(cj0)*(cj0);
IkReal x80=(py)*(py);
IkReal x81=((IkReal(0.0900000000000000))*(x78));
IkReal x82=((IkReal(0.0900000000000000))*(x79));
gconst56=IKsign(((((x77)*(x82)))+(((x77)*(x81)))+(((x80)*(x82)))+(((x80)*(x81)))));
IkReal x83=(sj0)*(sj0);
IkReal x84=(py)*(py);
IkReal x85=(cj0)*(cj0);
IkReal x86=(px)*(px);
dummyeval[0]=((((x84)*(x85)))+(((x85)*(x86)))+(((x83)*(x84)))+(((x83)*(x86))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x87=((pp)*(py));
IkReal x88=((IkReal(0.180000000000000))*(cj0));
IkReal x89=((py)*(sj0));
IkReal x90=((IkReal(1.10000000000000))*(pz));
IkReal x91=((px)*(sj0));
IkReal x92=((px)*(x90));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(pp)*(x91)))+(((IkReal(-0.180000000000000))*(x91)))+(((py)*(x88)))+(((x90)*(x91)))+(((cj0)*(x87)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x90))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(cj0)*(x92)))+(((IkReal(0.180000000000000))*(x89)))+(((sj0)*(x87)))+(((px)*(x88)))+(((cj0)*(pp)*(px)))+(((IkReal(-1.00000000000000))*(x89)*(x90))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(pp)*(x91)))+(((IkReal(-0.180000000000000))*(x91)))+(((py)*(x88)))+(((x90)*(x91)))+(((cj0)*(x87)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x90)))))), ((gconst56)*(((((IkReal(-1.00000000000000))*(cj0)*(x92)))+(((IkReal(0.180000000000000))*(x89)))+(((sj0)*(x87)))+(((px)*(x88)))+(((cj0)*(pp)*(px)))+(((IkReal(-1.00000000000000))*(x89)*(x90)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x93=IKsin(j2);
IkReal x94=IKcos(j2);
IkReal x95=((IkReal(1.00000000000000))*(py));
IkReal x96=((IkReal(0.0900000000000000))*(py));
IkReal x97=((sj0)*(x94));
IkReal x98=((sj0)*(x93));
IkReal x99=((cj0)*(x93));
IkReal x100=((cj0)*(px)*(x94));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj0)*(x94)*(x95)))+(((py)*(x98)))+(((px)*(x97)))+(((px)*(x99))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x95)*(x97)))+(((IkReal(-1.00000000000000))*(x95)*(x99)))+(((px)*(x98)))+(((IkReal(-1.00000000000000))*(x100))));
evalcond[2]=((IkReal(-0.180000000000000))+(((x96)*(x97)))+(((x96)*(x99)))+(((IkReal(-0.0900000000000000))*(px)*(x98)))+(((IkReal(0.0900000000000000))*(x100)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x101=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(pz));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj0)*(x101)))+(((px)*(sj0))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj0)*(x101)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
IkReal x102=(px)*(px);
IkReal x103=(sj0)*(sj0);
IkReal x104=(cj0)*(cj0);
IkReal x105=(py)*(py);
IkReal x106=((IkReal(0.0900000000000000))*(x103));
IkReal x107=((IkReal(0.0900000000000000))*(x104));
gconst57=IKsign(((((x102)*(x107)))+(((x102)*(x106)))+(((x105)*(x106)))+(((x105)*(x107)))));
IkReal x108=(sj0)*(sj0);
IkReal x109=(py)*(py);
IkReal x110=(cj0)*(cj0);
IkReal x111=(px)*(px);
dummyeval[0]=((((x110)*(x111)))+(((x109)*(x110)))+(((x108)*(x109)))+(((x108)*(x111))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x112=((pp)*(py));
IkReal x113=((IkReal(0.180000000000000))*(cj0));
IkReal x114=((py)*(sj0));
IkReal x115=((IkReal(1.10000000000000))*(pz));
IkReal x116=((px)*(sj0));
IkReal x117=((px)*(x115));
if( IKabs(((gconst57)*(((((py)*(x113)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x115)))+(((cj0)*(x112)))+(((IkReal(-0.180000000000000))*(x116)))+(((x115)*(x116)))+(((IkReal(-1.00000000000000))*(pp)*(x116))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((sj0)*(x112)))+(((px)*(x113)))+(((IkReal(0.180000000000000))*(x114)))+(((IkReal(-1.00000000000000))*(x114)*(x115)))+(((cj0)*(pp)*(px)))+(((IkReal(-1.00000000000000))*(cj0)*(x117))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst57)*(((((py)*(x113)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x115)))+(((cj0)*(x112)))+(((IkReal(-0.180000000000000))*(x116)))+(((x115)*(x116)))+(((IkReal(-1.00000000000000))*(pp)*(x116)))))), ((gconst57)*(((((sj0)*(x112)))+(((px)*(x113)))+(((IkReal(0.180000000000000))*(x114)))+(((IkReal(-1.00000000000000))*(x114)*(x115)))+(((cj0)*(pp)*(px)))+(((IkReal(-1.00000000000000))*(cj0)*(x117)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x118=IKsin(j2);
IkReal x119=IKcos(j2);
IkReal x120=((IkReal(1.00000000000000))*(py));
IkReal x121=((IkReal(0.0900000000000000))*(py));
IkReal x122=((sj0)*(x119));
IkReal x123=((sj0)*(x118));
IkReal x124=((cj0)*(x118));
IkReal x125=((cj0)*(px)*(x119));
evalcond[0]=((((py)*(x123)))+(((px)*(x122)))+(((px)*(x124)))+(((IkReal(-1.00000000000000))*(cj0)*(x119)*(x120))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x120)*(x124)))+(((IkReal(-1.00000000000000))*(x120)*(x122)))+(((px)*(x123)))+(((IkReal(-1.00000000000000))*(x125))));
evalcond[2]=((IkReal(-0.180000000000000))+(((x121)*(x122)))+(((x121)*(x124)))+(((IkReal(0.0900000000000000))*(x125)))+(((IkReal(-0.0900000000000000))*(px)*(x123)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x126=((IkReal(1052631578.94737))*(py));
IkReal x127=((IkReal(1052631578.94737))*(px));
if( IKabs(((((cj0)*(x126)))+(((IkReal(-1.00000000000000))*(sj0)*(x127))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj0)*(x127)))+(((sj0)*(x126))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj0)*(x126)))+(((IkReal(-1.00000000000000))*(sj0)*(x127)))))+IKsqr(((((cj0)*(x127)))+(((sj0)*(x126)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((cj0)*(x126)))+(((IkReal(-1.00000000000000))*(sj0)*(x127)))), ((((cj0)*(x127)))+(((sj0)*(x126)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x128=IKsin(j2);
IkReal x129=IKcos(j2);
IkReal x130=((IkReal(0.0900000000000000))*(py));
IkReal x131=((IkReal(1.00000000000000))*(py));
IkReal x132=((px)*(sj0));
IkReal x133=((sj0)*(x129));
IkReal x134=((cj0)*(x128));
IkReal x135=((cj0)*(px)*(x129));
evalcond[0]=((((IkReal(9.50000000000000e-10))*(x128)))+(x132)+(((IkReal(-1.00000000000000))*(cj0)*(x131))));
evalcond[1]=((((IkReal(9.50000000000000e-10))*(x129)))+(((IkReal(-1.00000000000000))*(sj0)*(x131)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
evalcond[2]=((((x129)*(x132)))+(((px)*(x134)))+(((IkReal(-1.00000000000000))*(cj0)*(x129)*(x131)))+(((py)*(sj0)*(x128))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((IkReal(-1.00000000000000))*(x135)))+(((x128)*(x132)))+(((IkReal(-1.00000000000000))*(x131)*(x134)))+(((IkReal(-1.00000000000000))*(x131)*(x133))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x128)*(x132)))+(((IkReal(0.0900000000000000))*(x135)))+(((IkReal(-1.00000000000000))*(pp)))+(((x130)*(x134)))+(((x130)*(x133)))+(((IkReal(1.10000000000000))*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((cj0)*(px)))+(((py)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst55)*(((((cj0)*(px)))+(((py)*(sj0)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x136=IKsin(j2);
IkReal x137=IKcos(j2);
IkReal x138=((IkReal(0.0450000000000000))*(cj3));
IkReal x139=((px)*(sj0));
IkReal x140=((IkReal(1.00000000000000))*(py));
IkReal x141=((IkReal(0.350000000000000))*(sj3));
IkReal x142=((IkReal(0.0900000000000000))*(py));
IkReal x143=((sj0)*(x137));
IkReal x144=((cj0)*(x136));
IkReal x145=((cj0)*(px)*(x137));
evalcond[0]=((((x136)*(x141)))+(((IkReal(-1.00000000000000))*(cj0)*(x140)))+(((IkReal(0.0450000000000000))*(x136)))+(x139)+(((IkReal(-1.00000000000000))*(x136)*(x138))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj0)*(x140)))+(((x137)*(x141)))+(((IkReal(-1.00000000000000))*(x137)*(x138)))+(((IkReal(0.0450000000000000))*(x137)))+(((IkReal(-1.00000000000000))*(cj0)*(px))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj0)*(x137)*(x140)))+(((x137)*(x139)))+(((px)*(x144)))+(((py)*(sj0)*(x136))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x140)*(x143)))+(((IkReal(-1.00000000000000))*(x140)*(x144)))+(((x136)*(x139)))+(x141)+(((IkReal(-1.00000000000000))*(x138)))+(((IkReal(-1.00000000000000))*(x145))));
evalcond[4]=((IkReal(-0.180000000000000))+(((x142)*(x143)))+(((x142)*(x144)))+(((IkReal(0.0900000000000000))*(x145)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(x136)*(x139))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=((IkReal(-1.00000000000000))*(py));
IkReal gconst59;
gconst59=((IkReal(-2.00000000000000))*(px));
IkReal gconst60;
gconst60=py;
IkReal gconst61;
gconst61=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst62;
gconst62=((IkReal(-2.00000000000000))*(py));
IkReal gconst63;
gconst63=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst64;
gconst64=((IkReal(-1.00000000000000))*(py));
IkReal gconst65;
gconst65=((IkReal(-2.00000000000000))*(px));
IkReal gconst66;
gconst66=py;
IkReal gconst67;
gconst67=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst68;
gconst68=((IkReal(-2.00000000000000))*(py));
IkReal gconst69;
gconst69=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst70;
gconst70=((IkReal(2.00000000000000))*(px));
IkReal gconst71;
gconst71=((IkReal(-4.00000000000000))*(py));
IkReal gconst72;
gconst72=((IkReal(-2.00000000000000))*(px));
IkReal gconst73;
gconst73=((IkReal(2.00000000000000))*(py));
IkReal gconst74;
gconst74=((IkReal(4.00000000000000))*(px));
IkReal gconst75;
gconst75=((IkReal(-2.00000000000000))*(py));
IkReal gconst76;
gconst76=((IkReal(2.00000000000000))*(px));
IkReal gconst77;
gconst77=((IkReal(-4.00000000000000))*(py));
IkReal gconst78;
gconst78=((IkReal(-2.00000000000000))*(px));
IkReal gconst79;
gconst79=((IkReal(2.00000000000000))*(py));
IkReal gconst80;
gconst80=((IkReal(4.00000000000000))*(px));
IkReal gconst81;
gconst81=((IkReal(-2.00000000000000))*(py));
IkReal gconst82;
gconst82=py;
IkReal gconst83;
gconst83=((IkReal(2.00000000000000))*(px));
IkReal gconst84;
gconst84=((IkReal(-1.00000000000000))*(py));
IkReal gconst85;
gconst85=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst86;
gconst86=((IkReal(2.00000000000000))*(py));
IkReal gconst87;
gconst87=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal gconst88;
gconst88=py;
IkReal gconst89;
gconst89=((IkReal(2.00000000000000))*(px));
IkReal gconst90;
gconst90=((IkReal(-1.00000000000000))*(py));
IkReal gconst91;
gconst91=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(px))));
IkReal gconst92;
gconst92=((IkReal(2.00000000000000))*(py));
IkReal gconst93;
gconst93=((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(px));
IkReal x146=((gconst83)*(gconst87));
IkReal x147=((gconst89)*(gconst91));
IkReal x148=((gconst88)*(gconst92));
IkReal x149=((gconst84)*(gconst86));
IkReal x150=((gconst88)*(gconst93));
IkReal x151=((gconst84)*(gconst85));
IkReal x152=((gconst82)*(gconst87));
IkReal x153=((gconst90)*(gconst91));
dummyeval[0]=((((x150)*(x151)))+(((IkReal(-1.00000000000000))*(x150)*(x152)))+(((x152)*(x153)))+(((IkReal(-1.00000000000000))*(x151)*(x153)))+(((IkReal(-1.00000000000000))*(x146)*(x147)))+(((x147)*(x149)))+(((x146)*(x148)))+(((IkReal(-1.00000000000000))*(x148)*(x149))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x154=((gconst61)*(gconst69));
IkReal x155=((gconst64)*(gconst72));
IkReal x156=((IkReal(1.00000000000000))*(gconst79));
IkReal x157=((gconst84)*(gconst85));
IkReal x158=((gconst65)*(gconst67));
IkReal x159=((gconst59)*(gconst63));
IkReal x160=((gconst64)*(gconst68));
IkReal x161=((IkReal(1.00000000000000))*(gconst88));
IkReal x162=((gconst82)*(gconst93));
IkReal x163=((gconst83)*(gconst87));
IkReal x164=((gconst72)*(gconst86));
IkReal x165=((gconst60)*(gconst76));
IkReal x166=((gconst58)*(gconst75));
IkReal x167=((gconst82)*(gconst87));
IkReal x168=((gconst90)*(gconst91));
IkReal x169=((IkReal(1.00000000000000))*(gconst87));
IkReal x170=((gconst60)*(gconst64));
IkReal x171=((gconst66)*(gconst67));
IkReal x172=((gconst58)*(gconst63));
IkReal x173=((gconst59)*(gconst75));
IkReal x174=((gconst67)*(gconst78));
IkReal x175=((gconst67)*(gconst77));
IkReal x176=((gconst60)*(gconst62));
IkReal x177=((gconst77)*(gconst91));
IkReal x178=((gconst88)*(gconst92));
IkReal x179=((gconst84)*(gconst86));
IkReal x180=((gconst88)*(gconst93));
IkReal x181=((gconst63)*(gconst70));
IkReal x182=((gconst89)*(gconst91));
IkReal x183=((gconst78)*(gconst91));
IkReal x184=((IkReal(1.00000000000000))*(gconst62));
IkReal x185=((gconst70)*(gconst87));
IkReal x186=((gconst60)*(gconst61));
IkReal x187=((IkReal(1.00000000000000))*(gconst60));
IkReal x188=((gconst74)*(gconst84));
IkReal x189=((gconst72)*(gconst85));
IkReal x190=((gconst73)*(gconst84));
IkReal x191=((gconst76)*(gconst92));
IkReal x192=((gconst63)*(gconst71));
IkReal x193=((gconst89)*(x179));
IkReal x194=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x195=((IkReal(1.00000000000000))*(gconst83)*(x182));
op[0]=((((x157)*(x180)))+(((IkReal(-1.00000000000000))*(gconst92)*(x161)*(x179)))+(((x179)*(x182)))+(((IkReal(-1.00000000000000))*(x157)*(x168)))+(((x167)*(x168)))+(((IkReal(-1.00000000000000))*(gconst87)*(x161)*(x162)))+(((IkReal(-1.00000000000000))*(x163)*(x182)))+(((x163)*(x178))));
op[1]=((((IkReal(-1.00000000000000))*(gconst80)*(x161)*(x179)))+(((IkReal(-1.00000000000000))*(gconst89)*(x156)*(x163)))+(((gconst79)*(x193)))+(((x180)*(x189)))+(((x168)*(x185)))+(((gconst76)*(gconst93)*(x157)))+(((IkReal(-1.00000000000000))*(gconst71)*(x169)*(x182)))+(((IkReal(-1.00000000000000))*(gconst81)*(x161)*(x167)))+(((gconst75)*(gconst82)*(x168)))+(((x180)*(x190)))+(((gconst79)*(gconst90)*(x167)))+(((IkReal(-1.00000000000000))*(x168)*(x189)))+(((IkReal(-1.00000000000000))*(x168)*(x190)))+(((IkReal(-1.00000000000000))*(gconst92)*(x161)*(x164)))+(((x164)*(x182)))+(((gconst75)*(gconst83)*(x178)))+(((x167)*(x183)))+(((IkReal(-1.00000000000000))*(gconst90)*(x156)*(x157)))+(((IkReal(-1.00000000000000))*(gconst92)*(x161)*(x188)))+(((IkReal(-1.00000000000000))*(gconst75)*(x195)))+(((IkReal(-1.00000000000000))*(x157)*(x183)))+(((IkReal(-1.00000000000000))*(gconst76)*(x162)*(x169)))+(((IkReal(-1.00000000000000))*(gconst93)*(x161)*(x185)))+(((gconst81)*(gconst88)*(x157)))+(((gconst80)*(gconst88)*(x163)))+(((gconst71)*(gconst87)*(x178)))+(((x182)*(x188)))+(((IkReal(-1.00000000000000))*(x179)*(x191)))+(((IkReal(-1.00000000000000))*(x163)*(x177)))+(((x163)*(x191)))+(((IkReal(-1.00000000000000))*(gconst75)*(x161)*(x162)))+(((x177)*(x179))));
op[2]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst78)*(gconst84)*(gconst91)))+(((gconst71)*(gconst80)*(gconst87)*(gconst88)))+(((gconst69)*(gconst84)*(gconst85)*(gconst88)))+(((gconst78)*(gconst79)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst83)*(gconst89)*(gconst91)))+(((gconst76)*(gconst80)*(gconst83)*(gconst87)))+(((gconst72)*(gconst76)*(gconst85)*(gconst93)))+(((gconst60)*(gconst86)*(gconst89)*(gconst91)))+(((gconst64)*(gconst83)*(gconst87)*(gconst92)))+(((gconst64)*(gconst84)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst86)*(gconst88)*(gconst92)))+(((gconst68)*(gconst83)*(gconst87)*(gconst88)))+(((gconst71)*(gconst75)*(gconst88)*(gconst92)))+(((gconst70)*(gconst79)*(gconst87)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst84)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst83)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst84)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst82)*(gconst87)*(gconst88)))+(((gconst72)*(gconst73)*(gconst88)*(gconst93)))+(((gconst73)*(gconst81)*(gconst84)*(gconst88)))+(((gconst75)*(gconst76)*(gconst83)*(gconst92)))+(((gconst74)*(gconst79)*(gconst84)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst81)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst79)*(gconst84)*(gconst90)))+(((gconst70)*(gconst75)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst84)*(gconst85)*(gconst91)))+(((gconst77)*(gconst79)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst76)*(gconst82)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst77)*(gconst79)*(gconst83)*(gconst87)))+(((gconst74)*(gconst77)*(gconst84)*(gconst91)))+(((gconst76)*(gconst81)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst77)*(gconst83)*(gconst91)))+(((gconst75)*(gconst80)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst87)*(gconst89)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst79)*(gconst85)*(gconst90)))+(((gconst67)*(gconst84)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst83)*(gconst89)))+(((gconst72)*(gconst81)*(gconst85)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst80)*(gconst86)*(gconst88)))+(((gconst66)*(gconst82)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst78)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst76)*(gconst84)*(gconst92)))+(((gconst75)*(gconst78)*(gconst82)*(gconst91)))+(((gconst72)*(gconst79)*(gconst86)*(gconst89)))+(((gconst59)*(gconst87)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst83)*(gconst87)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst81)*(gconst87)*(gconst88)))+(((gconst65)*(gconst84)*(gconst86)*(gconst91)))+(((gconst63)*(gconst83)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst78)*(gconst79)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst89)*(gconst91)))+(((gconst72)*(gconst77)*(gconst86)*(gconst91)))+(((gconst67)*(gconst82)*(gconst87)*(gconst90)))+(((gconst72)*(gconst74)*(gconst89)*(gconst91)))+(((gconst71)*(gconst76)*(gconst87)*(gconst92)))+(((gconst62)*(gconst84)*(gconst89)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst85)*(gconst90)*(gconst91)))+(((gconst63)*(gconst82)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst82)*(gconst88)*(gconst93)))+(((gconst58)*(gconst87)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst84)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst80)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst87)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst87)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst80)*(gconst84)*(gconst86)))+(((gconst75)*(gconst79)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst81)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst79)*(gconst87)*(gconst89)))+(((gconst73)*(gconst76)*(gconst84)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst84)*(gconst86)*(gconst88)))+(((gconst61)*(gconst84)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst82)*(gconst87)*(gconst93)))+(((gconst70)*(gconst78)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst84)*(gconst85)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst88)*(gconst92)))+(((gconst60)*(gconst85)*(gconst88)*(gconst93))));
op[3]=((((gconst65)*(gconst79)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst79)*(gconst85)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst78)*(gconst85)*(gconst91)))+(((gconst67)*(gconst75)*(gconst82)*(gconst90)))+(((gconst67)*(gconst70)*(gconst87)*(gconst90)))+(((gconst70)*(gconst78)*(gconst79)*(gconst87)))+(((gconst67)*(gconst74)*(gconst84)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst77)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst87)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst81)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst78)*(gconst84)*(gconst91)))+(((gconst68)*(gconst71)*(gconst87)*(gconst88)))+(((gconst58)*(gconst79)*(gconst87)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst76)*(gconst82)*(gconst87)))+(((gconst72)*(gconst77)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst78)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst84)*(gconst85)))+(((gconst71)*(gconst75)*(gconst76)*(gconst92)))+(((gconst63)*(gconst70)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst84)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst82)*(gconst88)))+(((gconst60)*(gconst73)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst88)*(gconst92)))+(((gconst67)*(gconst77)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst79)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst79)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst90)*(gconst91)))+(((gconst61)*(gconst81)*(gconst84)*(gconst88)))+(((gconst65)*(gconst74)*(gconst84)*(gconst91)))+(((gconst61)*(gconst72)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst79)*(gconst87)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst77)*(gconst79)*(gconst83)))+(((gconst72)*(gconst73)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst76)*(gconst93)))+(((gconst62)*(gconst77)*(gconst84)*(gconst91)))+(((gconst72)*(gconst74)*(gconst79)*(gconst89)))+(((gconst60)*(gconst77)*(gconst86)*(gconst91)))+(((gconst64)*(gconst75)*(gconst83)*(gconst92)))+(((gconst72)*(gconst73)*(gconst76)*(gconst93)))+(((gconst64)*(gconst72)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst74)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst78)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst80)*(gconst84)*(gconst88)))+(((gconst71)*(gconst76)*(gconst80)*(gconst87)))+(((gconst67)*(gconst78)*(gconst82)*(gconst87)))+(((gconst66)*(gconst75)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst88)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst80)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst81)*(gconst82)*(gconst87)))+(((gconst71)*(gconst75)*(gconst80)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst75)*(gconst82)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst87)*(gconst91)))+(((gconst63)*(gconst79)*(gconst82)*(gconst90)))+(((gconst60)*(gconst74)*(gconst89)*(gconst91)))+(((gconst67)*(gconst72)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst88)*(gconst93)))+(((gconst59)*(gconst75)*(gconst88)*(gconst92)))+(((gconst62)*(gconst79)*(gconst84)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst76)*(gconst92)))+(((gconst75)*(gconst78)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst75)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst79)*(gconst90)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst89)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst77)*(gconst87)*(gconst91)))+(((gconst59)*(gconst76)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst80)*(gconst88)))+(((gconst60)*(gconst81)*(gconst85)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst77)*(gconst91)))+(((gconst62)*(gconst72)*(gconst89)*(gconst91)))+(((gconst69)*(gconst73)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst89)*(gconst91)))+(((gconst65)*(gconst72)*(gconst86)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst76)*(gconst82)*(gconst93)))+(((gconst64)*(gconst71)*(gconst87)*(gconst92)))+(((gconst70)*(gconst75)*(gconst78)*(gconst91)))+(((gconst58)*(gconst75)*(gconst90)*(gconst91)))+(((gconst64)*(gconst73)*(gconst84)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst85)*(gconst91)))+(((gconst63)*(gconst80)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst83)*(gconst91)))+(((gconst75)*(gconst76)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst76)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst78)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst86)*(gconst92)))+(((gconst66)*(gconst79)*(gconst82)*(gconst87)))+(((gconst63)*(gconst78)*(gconst82)*(gconst91)))+(((gconst66)*(gconst70)*(gconst87)*(gconst91)))+(((gconst63)*(gconst71)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst80)*(gconst86)))+(((gconst59)*(gconst80)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst83)*(gconst89)))+(((gconst69)*(gconst72)*(gconst85)*(gconst88)))+(((gconst68)*(gconst75)*(gconst83)*(gconst88)))+(((gconst60)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst84)*(gconst90)))+(((gconst70)*(gconst75)*(gconst79)*(gconst90)))+(((gconst68)*(gconst76)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst80)*(gconst86)*(gconst88)))+(((gconst72)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst78)*(gconst79)*(gconst85)))+(((gconst74)*(gconst77)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst76)*(gconst87)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst87)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst76)*(gconst81)*(gconst82)))+(((gconst63)*(gconst76)*(gconst83)*(gconst92)))+(((gconst73)*(gconst76)*(gconst81)*(gconst84)))+(((gconst64)*(gconst81)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst75)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst84)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst84)*(gconst92)))+(((gconst72)*(gconst76)*(gconst81)*(gconst85)))+(((gconst61)*(gconst76)*(gconst84)*(gconst93)))+(((gconst60)*(gconst76)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst76)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst81)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst85)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst81)*(gconst87)))+(((gconst58)*(gconst78)*(gconst87)*(gconst91))));
op[4]=((((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst78)*(gconst79)))+(((gconst60)*(gconst74)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst76)*(gconst81)))+(((gconst58)*(gconst78)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst74)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst79)*(gconst87)))+(((gconst60)*(gconst65)*(gconst86)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst76)*(gconst93)))+(((gconst61)*(gconst64)*(gconst84)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst91)))+(((gconst63)*(gconst71)*(gconst80)*(gconst88)))+(((gconst68)*(gconst75)*(gconst76)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst86)*(gconst88)))+(((gconst65)*(gconst67)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst84)*(gconst85)))+(((gconst64)*(gconst68)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst88)*(gconst93)))+(((gconst60)*(gconst76)*(gconst81)*(gconst85)))+(((gconst65)*(gconst72)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst82)*(gconst87)))+(((gconst61)*(gconst72)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst76)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst87)*(gconst89)))+(((gconst58)*(gconst75)*(gconst79)*(gconst90)))+(((gconst58)*(gconst67)*(gconst87)*(gconst90)))+(((gconst69)*(gconst73)*(gconst76)*(gconst84)))+(((gconst61)*(gconst72)*(gconst76)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst78)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst78)*(gconst79)*(gconst84)))+(((gconst69)*(gconst72)*(gconst73)*(gconst88)))+(((gconst63)*(gconst70)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst80)*(gconst84)))+(((gconst58)*(gconst75)*(gconst78)*(gconst91)))+(((gconst62)*(gconst67)*(gconst84)*(gconst89)))+(((gconst67)*(gconst70)*(gconst75)*(gconst90)))+(((gconst63)*(gconst67)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst84)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst80)*(gconst88)))+(((gconst63)*(gconst68)*(gconst83)*(gconst88)))+(((gconst69)*(gconst72)*(gconst76)*(gconst85)))+(((gconst66)*(gconst70)*(gconst79)*(gconst87)))+(((gconst68)*(gconst71)*(gconst75)*(gconst88)))+(((gconst63)*(gconst70)*(gconst78)*(gconst91)))+(((gconst67)*(gconst70)*(gconst78)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst78)*(gconst91)))+(((gconst66)*(gconst70)*(gconst75)*(gconst91)))+(((gconst66)*(gconst67)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst75)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst85)*(gconst90)))+(((gconst64)*(gconst72)*(gconst73)*(gconst93)))+(((gconst68)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst78)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst75)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst81)*(gconst88)))+(((gconst63)*(gconst78)*(gconst79)*(gconst82)))+(((gconst59)*(gconst63)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst75)*(gconst91)))+(((gconst64)*(gconst71)*(gconst75)*(gconst92)))+(((gconst67)*(gconst72)*(gconst77)*(gconst86)))+(((gconst67)*(gconst74)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst78)*(gconst85)))+(((gconst61)*(gconst69)*(gconst84)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst76)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst87)*(gconst91)))+(((gconst71)*(gconst75)*(gconst76)*(gconst80)))+(((gconst60)*(gconst67)*(gconst86)*(gconst89)))+(((gconst62)*(gconst72)*(gconst79)*(gconst89)))+(((gconst63)*(gconst71)*(gconst76)*(gconst92)))+(((gconst60)*(gconst73)*(gconst76)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst76)*(gconst87)))+(((gconst59)*(gconst75)*(gconst76)*(gconst92)))+(((gconst72)*(gconst74)*(gconst77)*(gconst79)))+(((gconst70)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst87)*(gconst93)))+(((gconst63)*(gconst66)*(gconst82)*(gconst91)))+(((gconst66)*(gconst75)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst84)*(gconst88)))+(((gconst60)*(gconst77)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst74)*(gconst92)))+(((gconst60)*(gconst64)*(gconst85)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst90)*(gconst91)))+(((gconst59)*(gconst64)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst76)*(gconst92)))+(((gconst72)*(gconst73)*(gconst76)*(gconst81)))+(((gconst64)*(gconst71)*(gconst80)*(gconst87)))+(((gconst64)*(gconst72)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst84)*(gconst91)))+(((gconst67)*(gconst72)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst77)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst80)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst76)*(gconst92)))+(((gconst64)*(gconst73)*(gconst81)*(gconst84)))+(((gconst62)*(gconst77)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst76)*(gconst82)))+(((gconst58)*(gconst66)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst77)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst75)*(gconst81)*(gconst82)))+(((gconst61)*(gconst76)*(gconst81)*(gconst84)))+(((gconst64)*(gconst75)*(gconst80)*(gconst83)))+(((gconst63)*(gconst64)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst75)*(gconst89)))+(((gconst62)*(gconst72)*(gconst77)*(gconst91)))+(((gconst59)*(gconst76)*(gconst80)*(gconst87)))+(((gconst59)*(gconst68)*(gconst87)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst89)*(gconst91)))+(((gconst67)*(gconst75)*(gconst78)*(gconst82)))+(((gconst59)*(gconst75)*(gconst80)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst76)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst84)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst82)*(gconst93)))+(((gconst62)*(gconst65)*(gconst84)*(gconst91)))+(((gconst60)*(gconst69)*(gconst85)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst76)*(gconst80)*(gconst86)))+(((gconst65)*(gconst74)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst79)*(gconst89)))+(((gconst60)*(gconst62)*(gconst89)*(gconst91)))+(((gconst60)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst74)*(gconst76)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst81)*(gconst88)))+(((gconst63)*(gconst76)*(gconst80)*(gconst83)))+(((gconst60)*(gconst73)*(gconst81)*(gconst88)))+(((gconst58)*(gconst63)*(gconst90)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst75)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst88)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst77)*(gconst91)))+(((gconst64)*(gconst69)*(gconst84)*(gconst85)))+(((gconst60)*(gconst61)*(gconst88)*(gconst93)))+(((gconst65)*(gconst72)*(gconst74)*(gconst91))));
op[5]=((((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst74)*(gconst76)))+(((gconst61)*(gconst64)*(gconst72)*(gconst93)))+(((gconst64)*(gconst68)*(gconst71)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst72)*(gconst74)*(gconst80)))+(((gconst63)*(gconst66)*(gconst79)*(gconst82)))+(((gconst60)*(gconst65)*(gconst79)*(gconst86)))+(((gconst60)*(gconst67)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst80)*(gconst88)))+(((gconst63)*(gconst67)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst75)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst70)*(gconst88)))+(((gconst69)*(gconst72)*(gconst73)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst75)*(gconst88)))+(((gconst63)*(gconst70)*(gconst78)*(gconst79)))+(((gconst58)*(gconst63)*(gconst78)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst84)))+(((gconst59)*(gconst64)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst71)*(gconst87)))+(((gconst58)*(gconst66)*(gconst75)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst72)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(gconst90)))+(((gconst64)*(gconst68)*(gconst75)*(gconst83)))+(((gconst58)*(gconst67)*(gconst78)*(gconst87)))+(((gconst68)*(gconst71)*(gconst75)*(gconst76)))+(((gconst62)*(gconst65)*(gconst72)*(gconst91)))+(((gconst59)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst75)*(gconst77)*(gconst79)))+(((gconst60)*(gconst67)*(gconst77)*(gconst86)))+(((gconst63)*(gconst64)*(gconst80)*(gconst83)))+(((gconst58)*(gconst63)*(gconst79)*(gconst90)))+(((gconst61)*(gconst69)*(gconst76)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst75)*(gconst79)))+(((gconst59)*(gconst75)*(gconst76)*(gconst80)))+(((gconst66)*(gconst70)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst71)*(gconst89)))+(((gconst60)*(gconst69)*(gconst73)*(gconst88)))+(((gconst63)*(gconst68)*(gconst76)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst75)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst79)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst73)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst77)*(gconst83)))+(((gconst62)*(gconst72)*(gconst77)*(gconst79)))+(((gconst66)*(gconst67)*(gconst75)*(gconst82)))+(((gconst62)*(gconst67)*(gconst72)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst81)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst72)*(gconst76)*(gconst80)))+(((gconst60)*(gconst62)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst76)*(gconst93)))+(((gconst60)*(gconst74)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst78)*(gconst91)))+(((gconst65)*(gconst67)*(gconst74)*(gconst84)))+(((gconst60)*(gconst61)*(gconst76)*(gconst93)))+(((gconst58)*(gconst75)*(gconst78)*(gconst79)))+(((gconst59)*(gconst68)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst81)*(gconst82)))+(((gconst63)*(gconst68)*(gconst71)*(gconst88)))+(((gconst61)*(gconst64)*(gconst81)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst72)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst73)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst80)*(gconst86)))+(((gconst60)*(gconst65)*(gconst74)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst75)*(gconst83)))+(((gconst59)*(gconst68)*(gconst75)*(gconst88)))+(((gconst60)*(gconst64)*(gconst81)*(gconst85)))+(((gconst58)*(gconst67)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst74)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst77)*(gconst91)))+(((gconst64)*(gconst71)*(gconst75)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst76)*(gconst82)))+(((gconst60)*(gconst69)*(gconst76)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst84)))+(((gconst60)*(gconst61)*(gconst81)*(gconst88)))+(((gconst60)*(gconst64)*(gconst73)*(gconst93)))+(((gconst61)*(gconst69)*(gconst72)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst70)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst71)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst72)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst78)*(gconst85)))+(((gconst64)*(gconst72)*(gconst73)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst75)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst76)*(gconst87)))+(((gconst65)*(gconst67)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst79)*(gconst85)))+(((gconst59)*(gconst63)*(gconst80)*(gconst88)))+(((gconst63)*(gconst64)*(gconst71)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst79)*(gconst83)))+(((gconst63)*(gconst66)*(gconst70)*(gconst91)))+(((gconst67)*(gconst72)*(gconst74)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst72)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst79)*(gconst84)))+(((gconst66)*(gconst67)*(gconst70)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst77)*(gconst87)))+(((gconst62)*(gconst65)*(gconst79)*(gconst84)))+(((gconst63)*(gconst71)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst78)*(gconst84)))+(((gconst65)*(gconst72)*(gconst74)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst79)*(gconst87)))+(((gconst60)*(gconst62)*(gconst79)*(gconst89)))+(((gconst59)*(gconst63)*(gconst76)*(gconst92)))+(((gconst63)*(gconst67)*(gconst70)*(gconst90)))+(((gconst64)*(gconst69)*(gconst73)*(gconst84)))+(((gconst61)*(gconst72)*(gconst76)*(gconst81)))+(((gconst62)*(gconst67)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst76)*(gconst86)))+(((gconst67)*(gconst70)*(gconst75)*(gconst78)))+(((gconst60)*(gconst73)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst75)*(gconst89)))+(((gconst64)*(gconst69)*(gconst72)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst87)))+(((gconst58)*(gconst66)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst75)*(gconst91))));
op[6]=((((gconst61)*(gconst64)*(gconst69)*(gconst84)))+(((gconst60)*(gconst62)*(gconst65)*(gconst91)))+(((gconst62)*(gconst65)*(gconst67)*(gconst84)))+(((gconst62)*(gconst67)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst71)*(gconst75)))+(((gconst61)*(gconst64)*(gconst72)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst69)*(gconst87)))+(((gconst58)*(gconst66)*(gconst67)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst69)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst67)*(gconst90)))+(((gconst60)*(gconst69)*(gconst73)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst66)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst64)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst69)*(gconst82)))+(((gconst58)*(gconst63)*(gconst78)*(gconst79)))+(((gconst63)*(gconst68)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(gconst78)))+(((gconst58)*(gconst67)*(gconst75)*(gconst78)))+(((gconst63)*(gconst67)*(gconst70)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst67)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst67)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst68)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst70)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst71)*(gconst79)))+(((gconst59)*(gconst68)*(gconst75)*(gconst76)))+(((gconst60)*(gconst61)*(gconst64)*(gconst93)))+(((gconst59)*(gconst63)*(gconst64)*(gconst92)))+(((gconst61)*(gconst69)*(gconst72)*(gconst76)))+(((gconst58)*(gconst63)*(gconst66)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst77)*(gconst79)))+(((gconst60)*(gconst61)*(gconst76)*(gconst81)))+(((gconst59)*(gconst64)*(gconst75)*(gconst80)))+(((gconst60)*(gconst64)*(gconst73)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst68)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst67)*(gconst75)*(gconst77)))+(((gconst63)*(gconst66)*(gconst70)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst74)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst68)*(gconst72)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst70)*(gconst76)))+(((gconst65)*(gconst67)*(gconst72)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst61)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst64)*(gconst68)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst72)*(gconst80)))+(((gconst59)*(gconst63)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst73)*(gconst79)))+(((gconst64)*(gconst69)*(gconst72)*(gconst73)))+(((gconst60)*(gconst65)*(gconst67)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst67)*(gconst83)))+(((gconst66)*(gconst67)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst65)*(gconst91)))+(((gconst60)*(gconst61)*(gconst69)*(gconst88)))+(((gconst59)*(gconst63)*(gconst68)*(gconst88)))+(((gconst59)*(gconst64)*(gconst68)*(gconst87)))+(((gconst60)*(gconst62)*(gconst67)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst72)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst64)*(gconst68)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst63)*(gconst67)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst72)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst67)*(gconst73)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst76)*(gconst81)))+(((gconst64)*(gconst68)*(gconst71)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst71)*(gconst77)))+(((gconst60)*(gconst65)*(gconst74)*(gconst79)))+(((gconst60)*(gconst67)*(gconst74)*(gconst77)))+(((gconst60)*(gconst64)*(gconst69)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst67)*(gconst85)))+(((gconst63)*(gconst66)*(gconst67)*(gconst82)))+(((gconst58)*(gconst63)*(gconst67)*(gconst90)))+(((gconst58)*(gconst66)*(gconst75)*(gconst79)))+(((gconst63)*(gconst64)*(gconst68)*(gconst83)))+(((gconst63)*(gconst64)*(gconst71)*(gconst80)))+(((gconst62)*(gconst65)*(gconst72)*(gconst79)))+(((gconst60)*(gconst62)*(gconst77)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst63)*(gconst64)*(gconst93)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst69)*(gconst75)*(gconst76))));
op[7]=((((IkReal(-1.00000000000000))*(gconst69)*(gconst76)*(x172)))+(((IkReal(-1.00000000000000))*(gconst74)*(x160)*(x187)))+(((x160)*(x192)))+(((gconst61)*(gconst81)*(x170)))+(((gconst69)*(gconst73)*(x170)))+(((IkReal(-1.00000000000000))*(gconst68)*(x165)*(x184)))+(((IkReal(-1.00000000000000))*(gconst80)*(x170)*(x184)))+(((gconst68)*(gconst76)*(x159)))+(((gconst65)*(gconst79)*(x176)))+(((gconst66)*(gconst79)*(x172)))+(((gconst60)*(gconst74)*(x158)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst81)*(x172)))+(((x172)*(x174)))+(((x175)*(x176)))+(((IkReal(-1.00000000000000))*(gconst73)*(x171)*(x187)))+(((IkReal(-1.00000000000000))*(x174)*(x186)))+(((gconst62)*(gconst72)*(x158)))+(((IkReal(-1.00000000000000))*(x158)*(x192)))+(((x166)*(x171)))+(((x154)*(x155)))+(((IkReal(-1.00000000000000))*(gconst65)*(x156)*(x159)))+(((x154)*(x165)))+(((x171)*(x181)))+(((IkReal(-1.00000000000000))*(x159)*(x175)))+(((IkReal(-1.00000000000000))*(x158)*(x173)))+(((IkReal(-1.00000000000000))*(gconst68)*(x155)*(x184)))+(((IkReal(-1.00000000000000))*(x166)*(x194)))+(((gconst64)*(gconst80)*(x159)))+(((IkReal(-1.00000000000000))*(gconst66)*(x156)*(x186)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(x171)))+(((IkReal(-1.00000000000000))*(x181)*(x194)))+(((x160)*(x173))));
op[8]=((((IkReal(-1.00000000000000))*(x171)*(x186)))+(((IkReal(-1.00000000000000))*(x158)*(x159)))+(((x171)*(x172)))+(((IkReal(-1.00000000000000))*(x160)*(x176)))+(((x154)*(x170)))+(((IkReal(-1.00000000000000))*(x172)*(x194)))+(((x158)*(x176)))+(((x159)*(x160))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((cj0)*(px)))+(((py)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst111;
IkReal x196=(px)*(px);
IkReal x197=(sj0)*(sj0);
IkReal x198=(cj0)*(cj0);
IkReal x199=(py)*(py);
IkReal x200=((IkReal(0.0900000000000000))*(x197));
IkReal x201=((IkReal(0.0900000000000000))*(x198));
gconst111=IKsign(((((x196)*(x200)))+(((x196)*(x201)))+(((x199)*(x201)))+(((x199)*(x200)))));
IkReal x202=(sj0)*(sj0);
IkReal x203=(py)*(py);
IkReal x204=(cj0)*(cj0);
IkReal x205=(px)*(px);
dummyeval[0]=((((x204)*(x205)))+(((x202)*(x203)))+(((x202)*(x205)))+(((x203)*(x204))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x206=((cj0)*(px));
IkReal x207=((IkReal(1.10000000000000))*(pz));
IkReal x208=((IkReal(1.00000000000000))*(pp));
IkReal x209=((px)*(sj0));
IkReal x210=((cj0)*(py));
IkReal x211=((py)*(sj0));
if( IKabs(((gconst111)*(((((IkReal(-0.180000000000000))*(x209)))+(((x207)*(x210)))+(((IkReal(-1.00000000000000))*(x208)*(x209)))+(((IkReal(-1.00000000000000))*(x207)*(x209)))+(((IkReal(0.180000000000000))*(x210)))+(((pp)*(x210))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst111)*(((((IkReal(-0.180000000000000))*(x211)))+(((IkReal(-1.00000000000000))*(x207)*(x211)))+(((IkReal(-0.180000000000000))*(x206)))+(((IkReal(-1.00000000000000))*(x206)*(x208)))+(((IkReal(-1.00000000000000))*(x206)*(x207)))+(((IkReal(-1.00000000000000))*(x208)*(x211))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst111)*(((((IkReal(-0.180000000000000))*(x209)))+(((x207)*(x210)))+(((IkReal(-1.00000000000000))*(x208)*(x209)))+(((IkReal(-1.00000000000000))*(x207)*(x209)))+(((IkReal(0.180000000000000))*(x210)))+(((pp)*(x210)))))), ((gconst111)*(((((IkReal(-0.180000000000000))*(x211)))+(((IkReal(-1.00000000000000))*(x207)*(x211)))+(((IkReal(-0.180000000000000))*(x206)))+(((IkReal(-1.00000000000000))*(x206)*(x208)))+(((IkReal(-1.00000000000000))*(x206)*(x207)))+(((IkReal(-1.00000000000000))*(x208)*(x211)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x212=IKcos(j2);
IkReal x213=IKsin(j2);
IkReal x214=((IkReal(0.0900000000000000))*(py));
IkReal x215=((IkReal(0.0900000000000000))*(px));
IkReal x216=((IkReal(1.00000000000000))*(py));
IkReal x217=((sj0)*(x212));
IkReal x218=((cj0)*(x212));
IkReal x219=((sj0)*(x213));
IkReal x220=((IkReal(1.00000000000000))*(cj0)*(x213));
evalcond[0]=((((px)*(x217)))+(((IkReal(-1.00000000000000))*(px)*(x220)))+(((IkReal(-1.00000000000000))*(x216)*(x219)))+(((IkReal(-1.00000000000000))*(x216)*(x218))));
evalcond[1]=((((py)*(x217)))+(((IkReal(-1.00000000000000))*(cj0)*(x213)*(x216)))+(((px)*(x219)))+(((px)*(x218))));
evalcond[2]=((IkReal(-0.180000000000000))+(((cj0)*(x213)*(x214)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x214)*(x217)))+(((IkReal(-1.00000000000000))*(x215)*(x218)))+(((IkReal(-1.00000000000000))*(x215)*(x219))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((cj0)*(px)))+(((py)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst112;
IkReal x221=(px)*(px);
IkReal x222=(sj0)*(sj0);
IkReal x223=(cj0)*(cj0);
IkReal x224=(py)*(py);
IkReal x225=((IkReal(0.0900000000000000))*(x222));
IkReal x226=((IkReal(0.0900000000000000))*(x223));
gconst112=IKsign(((((x224)*(x225)))+(((x224)*(x226)))+(((x221)*(x225)))+(((x221)*(x226)))));
IkReal x227=(sj0)*(sj0);
IkReal x228=(py)*(py);
IkReal x229=(cj0)*(cj0);
IkReal x230=(px)*(px);
dummyeval[0]=((((x227)*(x228)))+(((x229)*(x230)))+(((x228)*(x229)))+(((x227)*(x230))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x231=((cj0)*(px));
IkReal x232=((IkReal(1.10000000000000))*(pz));
IkReal x233=((IkReal(1.00000000000000))*(pp));
IkReal x234=((px)*(sj0));
IkReal x235=((cj0)*(py));
IkReal x236=((py)*(sj0));
if( IKabs(((gconst112)*(((((IkReal(-1.00000000000000))*(x233)*(x234)))+(((pp)*(x235)))+(((IkReal(0.180000000000000))*(x235)))+(((x232)*(x235)))+(((IkReal(-0.180000000000000))*(x234)))+(((IkReal(-1.00000000000000))*(x232)*(x234))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst112)*(((((IkReal(-1.00000000000000))*(x233)*(x236)))+(((IkReal(-0.180000000000000))*(x236)))+(((IkReal(-0.180000000000000))*(x231)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((IkReal(-1.00000000000000))*(x231)*(x233)))+(((IkReal(-1.00000000000000))*(x232)*(x236))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst112)*(((((IkReal(-1.00000000000000))*(x233)*(x234)))+(((pp)*(x235)))+(((IkReal(0.180000000000000))*(x235)))+(((x232)*(x235)))+(((IkReal(-0.180000000000000))*(x234)))+(((IkReal(-1.00000000000000))*(x232)*(x234)))))), ((gconst112)*(((((IkReal(-1.00000000000000))*(x233)*(x236)))+(((IkReal(-0.180000000000000))*(x236)))+(((IkReal(-0.180000000000000))*(x231)))+(((IkReal(-1.00000000000000))*(x231)*(x232)))+(((IkReal(-1.00000000000000))*(x231)*(x233)))+(((IkReal(-1.00000000000000))*(x232)*(x236)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x237=IKcos(j2);
IkReal x238=IKsin(j2);
IkReal x239=((IkReal(0.0900000000000000))*(py));
IkReal x240=((IkReal(0.0900000000000000))*(px));
IkReal x241=((IkReal(1.00000000000000))*(py));
IkReal x242=((sj0)*(x237));
IkReal x243=((cj0)*(x237));
IkReal x244=((sj0)*(x238));
IkReal x245=((IkReal(1.00000000000000))*(cj0)*(x238));
evalcond[0]=((((px)*(x242)))+(((IkReal(-1.00000000000000))*(x241)*(x243)))+(((IkReal(-1.00000000000000))*(x241)*(x244)))+(((IkReal(-1.00000000000000))*(px)*(x245))));
evalcond[1]=((((px)*(x243)))+(((px)*(x244)))+(((IkReal(-1.00000000000000))*(cj0)*(x238)*(x241)))+(((py)*(x242))));
evalcond[2]=((IkReal(-0.180000000000000))+(((cj0)*(x238)*(x239)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x240)*(x243)))+(((IkReal(-1.00000000000000))*(x240)*(x244)))+(((IkReal(-1.00000000000000))*(x239)*(x242))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x246=((IkReal(1052631578.94737))*(py));
IkReal x247=((IkReal(1052631578.94737))*(px));
if( IKabs(((((cj0)*(x246)))+(((IkReal(-1.00000000000000))*(sj0)*(x247))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x246)))+(((IkReal(-1.00000000000000))*(cj0)*(x247))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj0)*(x246)))+(((IkReal(-1.00000000000000))*(sj0)*(x247)))))+IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x246)))+(((IkReal(-1.00000000000000))*(cj0)*(x247)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((cj0)*(x246)))+(((IkReal(-1.00000000000000))*(sj0)*(x247)))), ((((IkReal(-1.00000000000000))*(sj0)*(x246)))+(((IkReal(-1.00000000000000))*(cj0)*(x247)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x248=IKcos(j2);
IkReal x249=IKsin(j2);
IkReal x250=((cj0)*(px));
IkReal x251=((IkReal(0.0900000000000000))*(py));
IkReal x252=((cj0)*(py));
IkReal x253=((px)*(sj0));
IkReal x254=((py)*(sj0));
IkReal x255=((IkReal(1.00000000000000))*(x249));
IkReal x256=((sj0)*(x248));
evalcond[0]=((((IkReal(9.50000000000000e-10))*(x249)))+(x253)+(((IkReal(-1.00000000000000))*(x252))));
evalcond[1]=((((IkReal(9.50000000000000e-10))*(x248)))+(x254)+(x250));
evalcond[2]=((((x248)*(x253)))+(((IkReal(-1.00000000000000))*(x250)*(x255)))+(((IkReal(-1.00000000000000))*(x254)*(x255)))+(((IkReal(-1.00000000000000))*(x248)*(x252))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((IkReal(-1.00000000000000))*(x252)*(x255)))+(((x248)*(x250)))+(((x248)*(x254)))+(((x249)*(x253))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x249)*(x253)))+(((IkReal(-0.0900000000000000))*(x248)*(x250)))+(((IkReal(-1.00000000000000))*(x251)*(x256)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((cj0)*(x249)*(x251))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x257=((IkReal(1.00000000000000))*(px));
if( IKabs(((gconst110)*(((((IkReal(-1.00000000000000))*(sj0)*(x257)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst110)*(((((IkReal(-1.00000000000000))*(cj0)*(x257)))+(((IkReal(-1.00000000000000))*(py)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst110)*(((((IkReal(-1.00000000000000))*(sj0)*(x257)))+(((cj0)*(py)))))), ((gconst110)*(((((IkReal(-1.00000000000000))*(cj0)*(x257)))+(((IkReal(-1.00000000000000))*(py)*(sj0)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x258=IKsin(j2);
IkReal x259=IKcos(j2);
IkReal x260=((cj0)*(px));
IkReal x261=((IkReal(0.0450000000000000))*(cj3));
IkReal x262=((cj0)*(py));
IkReal x263=((py)*(sj0));
IkReal x264=((IkReal(0.350000000000000))*(sj3));
IkReal x265=((px)*(sj0));
IkReal x266=((IkReal(0.0900000000000000))*(x259));
IkReal x267=((IkReal(1.00000000000000))*(x258));
IkReal x268=((IkReal(0.0900000000000000))*(x258));
evalcond[0]=((((IkReal(-1.00000000000000))*(x262)))+(((x258)*(x264)))+(((IkReal(-1.00000000000000))*(x258)*(x261)))+(((IkReal(0.0450000000000000))*(x258)))+(x265));
evalcond[1]=((((x259)*(x264)))+(((IkReal(-1.00000000000000))*(x259)*(x261)))+(((IkReal(0.0450000000000000))*(x259)))+(x260)+(x263));
evalcond[2]=((((IkReal(-1.00000000000000))*(x263)*(x267)))+(((x259)*(x265)))+(((IkReal(-1.00000000000000))*(x259)*(x262)))+(((IkReal(-1.00000000000000))*(x260)*(x267))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x262)*(x267)))+(((x258)*(x265)))+(((x259)*(x263)))+(((x259)*(x260)))+(x264)+(((IkReal(-1.00000000000000))*(x261))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(x263)*(x266)))+(((x262)*(x268)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x265)*(x268)))+(((IkReal(-1.00000000000000))*(x260)*(x266))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x269=(px)*(px);
IkReal x270=(sj1)*(sj1);
IkReal x271=((IkReal(1.21000000000000))*(x270));
IkReal x272=((IkReal(-1.10000000000000))*(sj1));
if( IKabs(((x269)*(x272))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((px)*(py)*(x272))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x273=((IkReal(1.00000000000000))*(IKatan2(((x269)*(x272)), ((px)*(py)*(x272)))));
if( (((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py))))))))):(IkReal)1.0e30))*(((((IkReal(-1.10000000000000))*(cj1)*(px)*(pz)))+(((IkReal(0.175950000000000))*(px)))+(((IkReal(0.00405000000000000))*(cj3)*(px)))+(((IkReal(-0.0315000000000000))*(px)*(sj3)))+(((pp)*(px))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py))))))))):(IkReal)1.0e30))*(((((IkReal(-1.10000000000000))*(cj1)*(px)*(pz)))+(((IkReal(0.175950000000000))*(px)))+(((IkReal(0.00405000000000000))*(cj3)*(px)))+(((IkReal(-0.0315000000000000))*(px)*(sj3)))+(((pp)*(px))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x274=IKasin(((((IKabs(IKabs(IKsqrt(((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x271)*((x269)*(x269))))+(((x269)*(x271)*((py)*(py))))))))):(IkReal)1.0e30))*(((((IkReal(-1.10000000000000))*(cj1)*(px)*(pz)))+(((IkReal(0.175950000000000))*(px)))+(((IkReal(0.00405000000000000))*(cj3)*(px)))+(((IkReal(-0.0315000000000000))*(px)*(sj3)))+(((pp)*(px)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x274)))+(((IkReal(-1.00000000000000))*(x273))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x274)+(((IkReal(-1.00000000000000))*(x273))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x275=IKcos(j0);
IkReal x276=IKsin(j0);
IkReal x277=(py)*(py);
IkReal x278=(cj1)*(cj1);
IkReal x279=((cj1)*(py));
IkReal x280=((IkReal(0.0315000000000000))*(sj3));
IkReal x281=((cj1)*(px));
IkReal x282=((IkReal(0.00405000000000000))*(cj3));
IkReal x283=((IkReal(1.00000000000000))*(pp));
IkReal x284=((sj1)*(x276));
IkReal x285=((IkReal(1.10000000000000))*(x277));
IkReal x286=((sj1)*(x275));
IkReal x287=((IkReal(1.10000000000000))*(pz)*(x278));
IkReal x288=((IkReal(1.10000000000000))*(px)*(x286));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz)))+(((py)*(x284)))+(((px)*(x286))));
evalcond[1]=((((x284)*(x285)))+(((IkReal(1.10000000000000))*(pz)*(x279)))+(((IkReal(-1.00000000000000))*(py)*(x283)))+(((IkReal(-1.00000000000000))*(py)*(x282)))+(((IkReal(-0.175950000000000))*(py)))+(((py)*(x288)))+(((py)*(x280))));
evalcond[2]=((((cj1)*(x284)*(x285)))+(((x279)*(x280)))+(((x279)*(x288)))+(((IkReal(-0.175950000000000))*(x279)))+(((IkReal(-1.00000000000000))*(x279)*(x282)))+(((IkReal(-1.00000000000000))*(x279)*(x283)))+(((py)*(x287))));
evalcond[3]=((((IkReal(-0.175950000000000))*(x281)))+(((IkReal(-1.00000000000000))*(x281)*(x283)))+(((IkReal(-1.00000000000000))*(x281)*(x282)))+(((x280)*(x281)))+(((x281)*(x288)))+(((px)*(x287)))+(((IkReal(1.10000000000000))*(px)*(x279)*(x284))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x289=((cj0)*(px));
IkReal x290=((py)*(sj0));
IkReal x291=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((sj1)*(x290)))+(((cj1)*(pz)))+(((sj1)*(x289))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x290)*(x291)))+(((IkReal(-1.00000000000000))*(x289)*(x291)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x292=(cj1)*(cj1);
IkReal x293=(px)*(px);
IkReal x294=(sj0)*(sj0);
IkReal x295=(cj0)*(cj0);
IkReal x296=(py)*(py);
IkReal x297=((py)*(sj0));
IkReal x298=((IkReal(0.0900000000000000))*(x294));
IkReal x299=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x300=((cj1)*(pz)*(sj1));
IkReal x301=((IkReal(0.0900000000000000))*(x295));
gconst1=IKsign(((((x292)*(x293)*(x301)))+(((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x292)*(x297)*(x299)))+(((x292)*(x296)*(x298)))+(((IkReal(-1.00000000000000))*(x297)*(x299)))+(((IkReal(-1.00000000000000))*(x299)*(x300)))+(((x296)*(x301)))+(((IkReal(-0.180000000000000))*(x297)*(x300)))+(((x293)*(x298)))));
IkReal x302=(sj0)*(sj0);
IkReal x303=(py)*(py);
IkReal x304=(cj1)*(cj1);
IkReal x305=(cj0)*(cj0);
IkReal x306=(px)*(px);
IkReal x307=((cj0)*(px));
IkReal x308=((py)*(sj0));
IkReal x309=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((x303)*(x305)))+(((IkReal(-1.00000000000000))*(x308)*(x309)))+(((IkReal(2.00000000000000))*(x304)*(x307)*(x308)))+((((pz)*(pz))*((sj1)*(sj1))))+(((x302)*(x306)))+(((x302)*(x303)*(x304)))+(((IkReal(-1.00000000000000))*(x307)*(x309)))+(((IkReal(-2.00000000000000))*(x307)*(x308)))+(((x304)*(x305)*(x306))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x310=(sj1)*(sj1);
IkReal x311=(py)*(py);
IkReal x312=(sj0)*(sj0);
IkReal x313=(cj1)*(cj1);
IkReal x314=(cj0)*(cj0);
IkReal x315=(px)*(px);
IkReal x316=((IkReal(1.10000000000000))*(sj1));
IkReal x317=((px)*(py));
IkReal x318=((py)*(sj0));
IkReal x319=((cj1)*(pp));
IkReal x320=((px)*(sj0));
IkReal x321=((IkReal(1.00000000000000))*(pp));
IkReal x322=((cj0)*(py));
IkReal x323=((cj0)*(px));
IkReal x324=((cj0)*(sj0));
IkReal x325=((pz)*(sj1));
IkReal x326=((IkReal(1.10000000000000))*(pz));
IkReal x327=((IkReal(0.180000000000000))*(cj1));
IkReal x328=((x310)*(x326));
if( IKabs(((gconst1)*(((((IkReal(-0.180000000000000))*(x320)))+(((IkReal(0.180000000000000))*(x322)))+(((IkReal(-1.00000000000000))*(x320)*(x321)))+(((IkReal(-1.00000000000000))*(x314)*(x316)*(x317)))+(((x315)*(x316)*(x324)))+(((IkReal(-1.00000000000000))*(cj1)*(x322)*(x326)))+(((pp)*(x322)))+(((x312)*(x316)*(x317)))+(((cj1)*(x320)*(x326)))+(((IkReal(-1.00000000000000))*(x311)*(x316)*(x324))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((x318)*(x327)))+(((x318)*(x328)))+(((IkReal(-1.00000000000000))*(x313)*(x323)*(x326)))+(((IkReal(-0.180000000000000))*(x325)))+(((x323)*(x328)))+(((x323)*(x327)))+(((IkReal(-1.00000000000000))*(x313)*(x318)*(x326)))+(((IkReal(-1.00000000000000))*(x321)*(x325)))+(((IkReal(-1.00000000000000))*(cj1)*(x314)*(x315)*(x316)))+(((cj1)*(x316)*((pz)*(pz))))+(((x318)*(x319)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x317)*(x324)))+(((IkReal(-1.00000000000000))*(cj1)*(x311)*(x312)*(x316)))+(((x319)*(x323))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((IkReal(-0.180000000000000))*(x320)))+(((IkReal(0.180000000000000))*(x322)))+(((IkReal(-1.00000000000000))*(x320)*(x321)))+(((IkReal(-1.00000000000000))*(x314)*(x316)*(x317)))+(((x315)*(x316)*(x324)))+(((IkReal(-1.00000000000000))*(cj1)*(x322)*(x326)))+(((pp)*(x322)))+(((x312)*(x316)*(x317)))+(((cj1)*(x320)*(x326)))+(((IkReal(-1.00000000000000))*(x311)*(x316)*(x324)))))), ((gconst1)*(((((x318)*(x327)))+(((x318)*(x328)))+(((IkReal(-1.00000000000000))*(x313)*(x323)*(x326)))+(((IkReal(-0.180000000000000))*(x325)))+(((x323)*(x328)))+(((x323)*(x327)))+(((IkReal(-1.00000000000000))*(x313)*(x318)*(x326)))+(((IkReal(-1.00000000000000))*(x321)*(x325)))+(((IkReal(-1.00000000000000))*(cj1)*(x314)*(x315)*(x316)))+(((cj1)*(x316)*((pz)*(pz))))+(((x318)*(x319)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x317)*(x324)))+(((IkReal(-1.00000000000000))*(cj1)*(x311)*(x312)*(x316)))+(((x319)*(x323)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x329=IKsin(j2);
IkReal x330=IKcos(j2);
IkReal x331=((px)*(sj0));
IkReal x332=((IkReal(1.10000000000000))*(sj1));
IkReal x333=((py)*(sj0));
IkReal x334=((pz)*(sj1));
IkReal x335=((cj0)*(px));
IkReal x336=((IkReal(0.0900000000000000))*(x330));
IkReal x337=((cj1)*(x329));
IkReal x338=((IkReal(1.00000000000000))*(x330));
IkReal x339=((cj0)*(py)*(x329));
evalcond[0]=((((x335)*(x337)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x338)))+(((IkReal(-1.00000000000000))*(x329)*(x334)))+(((x330)*(x331)))+(((x333)*(x337))));
evalcond[1]=((((x330)*(x334)))+(((IkReal(-1.00000000000000))*(cj1)*(x335)*(x338)))+(((IkReal(-1.00000000000000))*(x339)))+(((x329)*(x331)))+(((IkReal(-1.00000000000000))*(cj1)*(x333)*(x338))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x329)*(x331)))+(((IkReal(0.0900000000000000))*(x339)))+(((cj1)*(x335)*(x336)))+(((x332)*(x333)))+(((x332)*(x335)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x333)*(x336)))+(((IkReal(-1.00000000000000))*(x334)*(x336))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x340=((cj0)*(px));
IkReal x341=((py)*(sj0));
IkReal x342=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((cj1)*(pz)))+(((sj1)*(x341)))+(((sj1)*(x340))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x340)*(x342)))+(((IkReal(-1.00000000000000))*(x341)*(x342)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x343=(cj1)*(cj1);
IkReal x344=(px)*(px);
IkReal x345=(sj0)*(sj0);
IkReal x346=(cj0)*(cj0);
IkReal x347=(py)*(py);
IkReal x348=((py)*(sj0));
IkReal x349=((IkReal(0.0900000000000000))*(x345));
IkReal x350=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x351=((cj1)*(pz)*(sj1));
IkReal x352=((IkReal(0.0900000000000000))*(x346));
gconst2=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x343)*(x347)*(x349)))+(((IkReal(-0.180000000000000))*(x348)*(x351)))+(((IkReal(-1.00000000000000))*(x348)*(x350)))+(((x344)*(x349)))+(((x347)*(x352)))+(((x343)*(x344)*(x352)))+(((x343)*(x348)*(x350)))+(((IkReal(-1.00000000000000))*(x350)*(x351)))));
IkReal x353=(sj0)*(sj0);
IkReal x354=(py)*(py);
IkReal x355=(cj1)*(cj1);
IkReal x356=(cj0)*(cj0);
IkReal x357=(px)*(px);
IkReal x358=((cj0)*(px));
IkReal x359=((py)*(sj0));
IkReal x360=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((x355)*(x356)*(x357)))+(((IkReal(2.00000000000000))*(x355)*(x358)*(x359)))+(((IkReal(-1.00000000000000))*(x358)*(x360)))+(((x353)*(x357)))+(((x354)*(x356)))+((((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-1.00000000000000))*(x359)*(x360)))+(((x353)*(x354)*(x355)))+(((IkReal(-2.00000000000000))*(x358)*(x359))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x361=(sj1)*(sj1);
IkReal x362=(py)*(py);
IkReal x363=(sj0)*(sj0);
IkReal x364=(cj1)*(cj1);
IkReal x365=(cj0)*(cj0);
IkReal x366=(px)*(px);
IkReal x367=((IkReal(1.10000000000000))*(sj1));
IkReal x368=((px)*(py));
IkReal x369=((py)*(sj0));
IkReal x370=((cj1)*(pp));
IkReal x371=((px)*(sj0));
IkReal x372=((IkReal(1.00000000000000))*(pp));
IkReal x373=((cj0)*(sj0));
IkReal x374=((cj0)*(px));
IkReal x375=((cj0)*(py));
IkReal x376=((pz)*(sj1));
IkReal x377=((IkReal(0.180000000000000))*(cj1));
IkReal x378=((IkReal(1.10000000000000))*(pz));
IkReal x379=((x361)*(x378));
if( IKabs(((gconst2)*(((((IkReal(-0.180000000000000))*(x371)))+(((x366)*(x367)*(x373)))+(((IkReal(-1.00000000000000))*(x365)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(cj1)*(x375)*(x378)))+(((cj1)*(x371)*(x378)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((IkReal(-1.00000000000000))*(x362)*(x367)*(x373)))+(((IkReal(0.180000000000000))*(x375)))+(((pp)*(x375)))+(((x363)*(x367)*(x368))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x368)*(x373)))+(((IkReal(-0.180000000000000))*(x376)))+(((x370)*(x374)))+(((x369)*(x379)))+(((x369)*(x377)))+(((x369)*(x370)))+(((cj1)*(x367)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x372)*(x376)))+(((IkReal(-1.00000000000000))*(cj1)*(x362)*(x363)*(x367)))+(((IkReal(-1.00000000000000))*(x364)*(x374)*(x378)))+(((x374)*(x379)))+(((x374)*(x377)))+(((IkReal(-1.00000000000000))*(x364)*(x369)*(x378)))+(((IkReal(-1.00000000000000))*(cj1)*(x365)*(x366)*(x367))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((IkReal(-0.180000000000000))*(x371)))+(((x366)*(x367)*(x373)))+(((IkReal(-1.00000000000000))*(x365)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(cj1)*(x375)*(x378)))+(((cj1)*(x371)*(x378)))+(((IkReal(-1.00000000000000))*(x371)*(x372)))+(((IkReal(-1.00000000000000))*(x362)*(x367)*(x373)))+(((IkReal(0.180000000000000))*(x375)))+(((pp)*(x375)))+(((x363)*(x367)*(x368)))))), ((gconst2)*(((((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x368)*(x373)))+(((IkReal(-0.180000000000000))*(x376)))+(((x370)*(x374)))+(((x369)*(x379)))+(((x369)*(x377)))+(((x369)*(x370)))+(((cj1)*(x367)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x372)*(x376)))+(((IkReal(-1.00000000000000))*(cj1)*(x362)*(x363)*(x367)))+(((IkReal(-1.00000000000000))*(x364)*(x374)*(x378)))+(((x374)*(x379)))+(((x374)*(x377)))+(((IkReal(-1.00000000000000))*(x364)*(x369)*(x378)))+(((IkReal(-1.00000000000000))*(cj1)*(x365)*(x366)*(x367)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x380=IKsin(j2);
IkReal x381=IKcos(j2);
IkReal x382=((px)*(sj0));
IkReal x383=((IkReal(1.10000000000000))*(sj1));
IkReal x384=((py)*(sj0));
IkReal x385=((pz)*(sj1));
IkReal x386=((cj0)*(px));
IkReal x387=((IkReal(0.0900000000000000))*(x381));
IkReal x388=((cj1)*(x380));
IkReal x389=((IkReal(1.00000000000000))*(x381));
IkReal x390=((cj0)*(py)*(x380));
evalcond[0]=((((x384)*(x388)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x389)))+(((x386)*(x388)))+(((IkReal(-1.00000000000000))*(x380)*(x385)))+(((x381)*(x382))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x390)))+(((IkReal(-1.00000000000000))*(cj1)*(x384)*(x389)))+(((x380)*(x382)))+(((x381)*(x385)))+(((IkReal(-1.00000000000000))*(cj1)*(x386)*(x389))));
evalcond[2]=((IkReal(-0.180000000000000))+(((cj1)*(x384)*(x387)))+(((x383)*(x384)))+(((x383)*(x386)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x380)*(x382)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(0.0900000000000000))*(x390)))+(((cj1)*(x386)*(x387))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((cj1)*(pz)))+(((cj0)*(px)*(sj1)))+(((py)*(sj0)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x391=((IkReal(1052631578.94737))*(py));
IkReal x392=((IkReal(1052631578.94737))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x392)))+(((cj0)*(x391))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj1)*(sj0)*(x391)))+(((cj0)*(cj1)*(x392))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x392)))+(((cj0)*(x391)))))+IKsqr(((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj1)*(sj0)*(x391)))+(((cj0)*(cj1)*(x392)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj0)*(x392)))+(((cj0)*(x391)))), ((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj1)*(sj0)*(x391)))+(((cj0)*(cj1)*(x392)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x393=IKcos(j2);
IkReal x394=IKsin(j2);
IkReal x395=((px)*(sj0));
IkReal x396=((py)*(sj0));
IkReal x397=((IkReal(1.00000000000000))*(cj1));
IkReal x398=((IkReal(1.10000000000000))*(sj1));
IkReal x399=((pz)*(sj1));
IkReal x400=((cj0)*(px));
IkReal x401=((IkReal(0.0900000000000000))*(x394));
IkReal x402=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x403=((cj1)*(x394));
IkReal x404=((IkReal(0.0900000000000000))*(x393));
evalcond[0]=((x395)+(((IkReal(9.50000000000000e-10))*(x394)))+(((IkReal(-1.00000000000000))*(x402))));
evalcond[1]=((x399)+(((IkReal(9.50000000000000e-10))*(x393)))+(((IkReal(-1.00000000000000))*(x396)*(x397)))+(((IkReal(-1.00000000000000))*(x397)*(x400))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x393)*(x402)))+(((IkReal(-1.00000000000000))*(x394)*(x399)))+(((x400)*(x403)))+(((x393)*(x395)))+(((x396)*(x403))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((x394)*(x395)))+(((IkReal(-1.00000000000000))*(x394)*(x402)))+(((IkReal(-1.00000000000000))*(x393)*(x396)*(x397)))+(((x393)*(x399)))+(((IkReal(-1.00000000000000))*(x393)*(x397)*(x400))));
evalcond[4]=((IkReal(-0.180000000000000))+(((cj0)*(py)*(x401)))+(((cj1)*(x400)*(x404)))+(((IkReal(-1.00000000000000))*(x395)*(x401)))+(((x396)*(x398)))+(((cj1)*(x396)*(x404)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x399)*(x404)))+(((x398)*(x400))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x405=IKsin(j2);
IkReal x406=IKcos(j2);
IkReal x407=((IkReal(0.0450000000000000))*(cj3));
IkReal x408=((px)*(sj0));
IkReal x409=((py)*(sj0));
IkReal x410=((IkReal(1.00000000000000))*(cj0));
IkReal x411=((IkReal(1.10000000000000))*(sj1));
IkReal x412=((IkReal(0.350000000000000))*(sj3));
IkReal x413=((pz)*(sj1));
IkReal x414=((IkReal(0.0900000000000000))*(cj0));
IkReal x415=((cj0)*(px));
IkReal x416=((cj1)*(x406));
IkReal x417=((py)*(x405));
IkReal x418=((cj1)*(x405));
evalcond[0]=((((IkReal(0.0450000000000000))*(x405)))+(((x405)*(x412)))+(x408)+(((IkReal(-1.00000000000000))*(x405)*(x407)))+(((IkReal(-1.00000000000000))*(py)*(x410))));
evalcond[1]=((((IkReal(0.0450000000000000))*(x406)))+(((IkReal(-1.00000000000000))*(x406)*(x407)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x410)))+(((x406)*(x412)))+(((IkReal(-1.00000000000000))*(cj1)*(x409)))+(x413));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(x406)*(x410)))+(((x415)*(x418)))+(((x409)*(x418)))+(((x406)*(x408)))+(((IkReal(-1.00000000000000))*(x405)*(x413))));
evalcond[3]=((IkReal(0.0450000000000000))+(((x405)*(x408)))+(((IkReal(-1.00000000000000))*(px)*(x410)*(x416)))+(((IkReal(-1.00000000000000))*(x409)*(x416)))+(((x406)*(x413)))+(x412)+(((IkReal(-1.00000000000000))*(x410)*(x417)))+(((IkReal(-1.00000000000000))*(x407))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x405)*(x408)))+(((x411)*(x415)))+(((IkReal(0.0900000000000000))*(x409)*(x416)))+(((px)*(x414)*(x416)))+(((x414)*(x417)))+(((x409)*(x411)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x406)*(x413))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x419=(sj1)*(sj1);
if( IKabs(((px)*(sj1))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((py)*(sj1))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x420=((IkReal(1.00000000000000))*(IKatan2(((px)*(sj1)), ((py)*(sj1)))));
if( (((((x419)*((py)*(py))))+(((x419)*((px)*(px)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x419)*((py)*(py))))+(((x419)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x419)*((py)*(py))))+(((x419)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x419)*((py)*(py))))+(((x419)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x419)*((py)*(py))))+(((x419)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x421=IKasin(((((IKabs(IKabs(IKsqrt(((((x419)*((py)*(py))))+(((x419)*((px)*(px)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x419)*((py)*(py))))+(((x419)*((px)*(px))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x421)))+(((IkReal(-1.00000000000000))*(x420))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x420)))+(x421));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[4];
IkReal x422=IKcos(j0);
IkReal x423=IKsin(j0);
IkReal x424=(py)*(py);
IkReal x425=(cj1)*(cj1);
IkReal x426=(px)*(px);
IkReal x427=((cj1)*(py));
IkReal x428=((IkReal(0.0315000000000000))*(sj3));
IkReal x429=((pp)*(px));
IkReal x430=((px)*(py));
IkReal x431=((IkReal(1.10000000000000))*(pz));
IkReal x432=((cj1)*(px));
IkReal x433=((IkReal(0.00405000000000000))*(cj3));
IkReal x434=((IkReal(1.00000000000000))*(pp));
IkReal x435=((px)*(x433));
IkReal x436=((IkReal(1.10000000000000))*(sj1)*(x422));
IkReal x437=((IkReal(1.10000000000000))*(sj1)*(x423));
evalcond[0]=((((IkReal(-1.00000000000000))*(x430)*(x437)))+(((IkReal(-1.00000000000000))*(px)*(x428)))+(((IkReal(-1.00000000000000))*(x431)*(x432)))+(((IkReal(0.175950000000000))*(px)))+(x429)+(x435)+(((IkReal(-1.00000000000000))*(x426)*(x436))));
evalcond[1]=((((x424)*(x437)))+(((py)*(x428)))+(((x430)*(x436)))+(((IkReal(-1.00000000000000))*(py)*(x433)))+(((IkReal(-1.00000000000000))*(py)*(x434)))+(((IkReal(-0.175950000000000))*(py)))+(((x427)*(x431))));
evalcond[2]=((((x427)*(x428)))+(((IkReal(-1.00000000000000))*(x427)*(x433)))+(((IkReal(-1.00000000000000))*(x427)*(x434)))+(((IkReal(-0.175950000000000))*(x427)))+(((px)*(x427)*(x436)))+(((cj1)*(x424)*(x437)))+(((py)*(x425)*(x431))));
evalcond[3]=((((x428)*(x432)))+(((IkReal(-0.175950000000000))*(x432)))+(((IkReal(-1.00000000000000))*(x432)*(x433)))+(((px)*(x427)*(x437)))+(((px)*(x425)*(x431)))+(((cj1)*(x426)*(x436)))+(((IkReal(-1.00000000000000))*(cj1)*(x429))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x438=((cj0)*(px));
IkReal x439=((py)*(sj0));
IkReal x440=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((cj1)*(pz)))+(((sj1)*(x439)))+(((sj1)*(x438))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x438)*(x440)))+(((IkReal(-1.00000000000000))*(x439)*(x440)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x441=(cj1)*(cj1);
IkReal x442=(px)*(px);
IkReal x443=(sj0)*(sj0);
IkReal x444=(cj0)*(cj0);
IkReal x445=(py)*(py);
IkReal x446=((py)*(sj0));
IkReal x447=((IkReal(0.0900000000000000))*(x443));
IkReal x448=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x449=((cj1)*(pz)*(sj1));
IkReal x450=((IkReal(0.0900000000000000))*(x444));
gconst1=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x441)*(x442)*(x450)))+(((IkReal(-0.180000000000000))*(x446)*(x449)))+(((IkReal(-1.00000000000000))*(x448)*(x449)))+(((x441)*(x446)*(x448)))+(((x442)*(x447)))+(((x445)*(x450)))+(((x441)*(x445)*(x447)))+(((IkReal(-1.00000000000000))*(x446)*(x448)))));
IkReal x451=(sj0)*(sj0);
IkReal x452=(py)*(py);
IkReal x453=(cj1)*(cj1);
IkReal x454=(cj0)*(cj0);
IkReal x455=(px)*(px);
IkReal x456=((cj0)*(px));
IkReal x457=((py)*(sj0));
IkReal x458=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((x451)*(x452)*(x453)))+(((x452)*(x454)))+(((IkReal(-1.00000000000000))*(x456)*(x458)))+(((IkReal(-2.00000000000000))*(x456)*(x457)))+(((x453)*(x454)*(x455)))+((((pz)*(pz))*((sj1)*(sj1))))+(((x451)*(x455)))+(((IkReal(-1.00000000000000))*(x457)*(x458)))+(((IkReal(2.00000000000000))*(x453)*(x456)*(x457))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x459=(sj1)*(sj1);
IkReal x460=(py)*(py);
IkReal x461=(sj0)*(sj0);
IkReal x462=(cj1)*(cj1);
IkReal x463=(cj0)*(cj0);
IkReal x464=(px)*(px);
IkReal x465=((IkReal(1.10000000000000))*(sj1));
IkReal x466=((px)*(py));
IkReal x467=((py)*(sj0));
IkReal x468=((cj1)*(pp));
IkReal x469=((px)*(sj0));
IkReal x470=((IkReal(1.00000000000000))*(pp));
IkReal x471=((cj0)*(py));
IkReal x472=((cj0)*(px));
IkReal x473=((cj0)*(sj0));
IkReal x474=((pz)*(sj1));
IkReal x475=((IkReal(1.10000000000000))*(pz));
IkReal x476=((IkReal(0.180000000000000))*(cj1));
IkReal x477=((x459)*(x475));
if( IKabs(((gconst1)*(((((pp)*(x471)))+(((cj1)*(x469)*(x475)))+(((x464)*(x465)*(x473)))+(((x461)*(x465)*(x466)))+(((IkReal(-0.180000000000000))*(x469)))+(((IkReal(-1.00000000000000))*(x469)*(x470)))+(((IkReal(-1.00000000000000))*(x463)*(x465)*(x466)))+(((IkReal(-1.00000000000000))*(x460)*(x465)*(x473)))+(((IkReal(0.180000000000000))*(x471)))+(((IkReal(-1.00000000000000))*(cj1)*(x471)*(x475))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((x472)*(x476)))+(((x472)*(x477)))+(((cj1)*(x465)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x462)*(x467)*(x475)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x466)*(x473)))+(((IkReal(-1.00000000000000))*(x462)*(x472)*(x475)))+(((IkReal(-1.00000000000000))*(cj1)*(x460)*(x461)*(x465)))+(((IkReal(-0.180000000000000))*(x474)))+(((x467)*(x477)))+(((x467)*(x476)))+(((x467)*(x468)))+(((IkReal(-1.00000000000000))*(x470)*(x474)))+(((IkReal(-1.00000000000000))*(cj1)*(x463)*(x464)*(x465)))+(((x468)*(x472))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((pp)*(x471)))+(((cj1)*(x469)*(x475)))+(((x464)*(x465)*(x473)))+(((x461)*(x465)*(x466)))+(((IkReal(-0.180000000000000))*(x469)))+(((IkReal(-1.00000000000000))*(x469)*(x470)))+(((IkReal(-1.00000000000000))*(x463)*(x465)*(x466)))+(((IkReal(-1.00000000000000))*(x460)*(x465)*(x473)))+(((IkReal(0.180000000000000))*(x471)))+(((IkReal(-1.00000000000000))*(cj1)*(x471)*(x475)))))), ((gconst1)*(((((x472)*(x476)))+(((x472)*(x477)))+(((cj1)*(x465)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x462)*(x467)*(x475)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x466)*(x473)))+(((IkReal(-1.00000000000000))*(x462)*(x472)*(x475)))+(((IkReal(-1.00000000000000))*(cj1)*(x460)*(x461)*(x465)))+(((IkReal(-0.180000000000000))*(x474)))+(((x467)*(x477)))+(((x467)*(x476)))+(((x467)*(x468)))+(((IkReal(-1.00000000000000))*(x470)*(x474)))+(((IkReal(-1.00000000000000))*(cj1)*(x463)*(x464)*(x465)))+(((x468)*(x472)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x478=IKsin(j2);
IkReal x479=IKcos(j2);
IkReal x480=((px)*(sj0));
IkReal x481=((IkReal(1.10000000000000))*(sj1));
IkReal x482=((py)*(sj0));
IkReal x483=((pz)*(sj1));
IkReal x484=((cj0)*(px));
IkReal x485=((IkReal(0.0900000000000000))*(x479));
IkReal x486=((cj1)*(x478));
IkReal x487=((IkReal(1.00000000000000))*(x479));
IkReal x488=((cj0)*(py)*(x478));
evalcond[0]=((((x484)*(x486)))+(((IkReal(-1.00000000000000))*(x478)*(x483)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x487)))+(((x482)*(x486)))+(((x479)*(x480))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x488)))+(((x478)*(x480)))+(((IkReal(-1.00000000000000))*(cj1)*(x484)*(x487)))+(((x479)*(x483)))+(((IkReal(-1.00000000000000))*(cj1)*(x482)*(x487))));
evalcond[2]=((IkReal(-0.180000000000000))+(((cj1)*(x482)*(x485)))+(((IkReal(-1.00000000000000))*(x483)*(x485)))+(((IkReal(0.0900000000000000))*(x488)))+(((IkReal(-0.0900000000000000))*(x478)*(x480)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((x481)*(x482)))+(((x481)*(x484)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x484)*(x485))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x489=((cj0)*(px));
IkReal x490=((py)*(sj0));
IkReal x491=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((sj1)*(x489)))+(((cj1)*(pz)))+(((sj1)*(x490))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x489)*(x491)))+(((IkReal(-1.00000000000000))*(x490)*(x491)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x492=(cj1)*(cj1);
IkReal x493=(px)*(px);
IkReal x494=(sj0)*(sj0);
IkReal x495=(cj0)*(cj0);
IkReal x496=(py)*(py);
IkReal x497=((py)*(sj0));
IkReal x498=((IkReal(0.0900000000000000))*(x494));
IkReal x499=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x500=((cj1)*(pz)*(sj1));
IkReal x501=((IkReal(0.0900000000000000))*(x495));
gconst2=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-1.00000000000000))*(x499)*(x500)))+(((x492)*(x497)*(x499)))+(((x493)*(x498)))+(((IkReal(-1.00000000000000))*(x497)*(x499)))+(((x492)*(x493)*(x501)))+(((x496)*(x501)))+(((IkReal(-0.180000000000000))*(x497)*(x500)))+(((x492)*(x496)*(x498)))));
IkReal x502=(sj0)*(sj0);
IkReal x503=(py)*(py);
IkReal x504=(cj1)*(cj1);
IkReal x505=(cj0)*(cj0);
IkReal x506=(px)*(px);
IkReal x507=((cj0)*(px));
IkReal x508=((py)*(sj0));
IkReal x509=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((IkReal(-2.00000000000000))*(x507)*(x508)))+(((IkReal(-1.00000000000000))*(x508)*(x509)))+(((IkReal(-1.00000000000000))*(x507)*(x509)))+(((IkReal(2.00000000000000))*(x504)*(x507)*(x508)))+((((pz)*(pz))*((sj1)*(sj1))))+(((x503)*(x505)))+(((x502)*(x506)))+(((x502)*(x503)*(x504)))+(((x504)*(x505)*(x506))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x510=(sj1)*(sj1);
IkReal x511=(py)*(py);
IkReal x512=(sj0)*(sj0);
IkReal x513=(cj1)*(cj1);
IkReal x514=(cj0)*(cj0);
IkReal x515=(px)*(px);
IkReal x516=((IkReal(1.10000000000000))*(sj1));
IkReal x517=((px)*(py));
IkReal x518=((py)*(sj0));
IkReal x519=((cj1)*(pp));
IkReal x520=((px)*(sj0));
IkReal x521=((IkReal(1.00000000000000))*(pp));
IkReal x522=((cj0)*(sj0));
IkReal x523=((cj0)*(px));
IkReal x524=((cj0)*(py));
IkReal x525=((pz)*(sj1));
IkReal x526=((IkReal(0.180000000000000))*(cj1));
IkReal x527=((IkReal(1.10000000000000))*(pz));
IkReal x528=((x510)*(x527));
if( IKabs(((gconst2)*(((((pp)*(x524)))+(((IkReal(-1.00000000000000))*(x514)*(x516)*(x517)))+(((IkReal(-1.00000000000000))*(x520)*(x521)))+(((IkReal(0.180000000000000))*(x524)))+(((x512)*(x516)*(x517)))+(((IkReal(-1.00000000000000))*(cj1)*(x524)*(x527)))+(((IkReal(-1.00000000000000))*(x511)*(x516)*(x522)))+(((cj1)*(x520)*(x527)))+(((IkReal(-0.180000000000000))*(x520)))+(((x515)*(x516)*(x522))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x517)*(x522)))+(((x519)*(x523)))+(((IkReal(-1.00000000000000))*(x521)*(x525)))+(((x518)*(x528)))+(((x518)*(x526)))+(((x518)*(x519)))+(((x523)*(x528)))+(((x523)*(x526)))+(((cj1)*(x516)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x513)*(x523)*(x527)))+(((IkReal(-1.00000000000000))*(cj1)*(x514)*(x515)*(x516)))+(((IkReal(-1.00000000000000))*(cj1)*(x511)*(x512)*(x516)))+(((IkReal(-0.180000000000000))*(x525)))+(((IkReal(-1.00000000000000))*(x513)*(x518)*(x527))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((pp)*(x524)))+(((IkReal(-1.00000000000000))*(x514)*(x516)*(x517)))+(((IkReal(-1.00000000000000))*(x520)*(x521)))+(((IkReal(0.180000000000000))*(x524)))+(((x512)*(x516)*(x517)))+(((IkReal(-1.00000000000000))*(cj1)*(x524)*(x527)))+(((IkReal(-1.00000000000000))*(x511)*(x516)*(x522)))+(((cj1)*(x520)*(x527)))+(((IkReal(-0.180000000000000))*(x520)))+(((x515)*(x516)*(x522)))))), ((gconst2)*(((((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x517)*(x522)))+(((x519)*(x523)))+(((IkReal(-1.00000000000000))*(x521)*(x525)))+(((x518)*(x528)))+(((x518)*(x526)))+(((x518)*(x519)))+(((x523)*(x528)))+(((x523)*(x526)))+(((cj1)*(x516)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x513)*(x523)*(x527)))+(((IkReal(-1.00000000000000))*(cj1)*(x514)*(x515)*(x516)))+(((IkReal(-1.00000000000000))*(cj1)*(x511)*(x512)*(x516)))+(((IkReal(-0.180000000000000))*(x525)))+(((IkReal(-1.00000000000000))*(x513)*(x518)*(x527)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x529=IKsin(j2);
IkReal x530=IKcos(j2);
IkReal x531=((px)*(sj0));
IkReal x532=((IkReal(1.10000000000000))*(sj1));
IkReal x533=((py)*(sj0));
IkReal x534=((pz)*(sj1));
IkReal x535=((cj0)*(px));
IkReal x536=((IkReal(0.0900000000000000))*(x530));
IkReal x537=((cj1)*(x529));
IkReal x538=((IkReal(1.00000000000000))*(x530));
IkReal x539=((cj0)*(py)*(x529));
evalcond[0]=((((IkReal(-1.00000000000000))*(x529)*(x534)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x538)))+(((x535)*(x537)))+(((x530)*(x531)))+(((x533)*(x537))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x539)))+(((IkReal(-1.00000000000000))*(cj1)*(x535)*(x538)))+(((IkReal(-1.00000000000000))*(cj1)*(x533)*(x538)))+(((x530)*(x534)))+(((x529)*(x531))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(x534)*(x536)))+(((IkReal(0.0900000000000000))*(x539)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj1)*(x535)*(x536)))+(((x532)*(x535)))+(((x532)*(x533)))+(((cj1)*(x533)*(x536)))+(((IkReal(-0.0900000000000000))*(x529)*(x531))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((cj1)*(pz)))+(((cj0)*(px)*(sj1)))+(((py)*(sj0)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x540=((IkReal(1052631578.94737))*(py));
IkReal x541=((IkReal(1052631578.94737))*(px));
if( IKabs(((((IkReal(-1.00000000000000))*(sj0)*(x541)))+(((cj0)*(x540))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj1)*(sj0)*(x540)))+(((cj0)*(cj1)*(x541))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(sj0)*(x541)))+(((cj0)*(x540)))))+IKsqr(((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj1)*(sj0)*(x540)))+(((cj0)*(cj1)*(x541)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((IkReal(-1.00000000000000))*(sj0)*(x541)))+(((cj0)*(x540)))), ((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj1)*(sj0)*(x540)))+(((cj0)*(cj1)*(x541)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x542=IKcos(j2);
IkReal x543=IKsin(j2);
IkReal x544=((px)*(sj0));
IkReal x545=((py)*(sj0));
IkReal x546=((IkReal(1.00000000000000))*(cj1));
IkReal x547=((IkReal(1.10000000000000))*(sj1));
IkReal x548=((pz)*(sj1));
IkReal x549=((cj0)*(px));
IkReal x550=((IkReal(0.0900000000000000))*(x543));
IkReal x551=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x552=((cj1)*(x543));
IkReal x553=((IkReal(0.0900000000000000))*(x542));
evalcond[0]=((((IkReal(9.50000000000000e-10))*(x543)))+(((IkReal(-1.00000000000000))*(x551)))+(x544));
evalcond[1]=((((IkReal(-1.00000000000000))*(x545)*(x546)))+(((IkReal(9.50000000000000e-10))*(x542)))+(x548)+(((IkReal(-1.00000000000000))*(x546)*(x549))));
evalcond[2]=((((x542)*(x544)))+(((x549)*(x552)))+(((IkReal(-1.00000000000000))*(x543)*(x548)))+(((x545)*(x552)))+(((IkReal(-1.00000000000000))*(x542)*(x551))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((x542)*(x548)))+(((IkReal(-1.00000000000000))*(x542)*(x546)*(x549)))+(((x543)*(x544)))+(((IkReal(-1.00000000000000))*(x542)*(x545)*(x546)))+(((IkReal(-1.00000000000000))*(x543)*(x551))));
evalcond[4]=((IkReal(-0.180000000000000))+(((cj1)*(x545)*(x553)))+(((x547)*(x549)))+(((IkReal(-1.00000000000000))*(x544)*(x550)))+(((cj0)*(py)*(x550)))+(((cj1)*(x549)*(x553)))+(((IkReal(-1.00000000000000))*(x548)*(x553)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((x545)*(x547))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x554=IKsin(j2);
IkReal x555=IKcos(j2);
IkReal x556=((IkReal(0.0450000000000000))*(cj3));
IkReal x557=((px)*(sj0));
IkReal x558=((py)*(sj0));
IkReal x559=((IkReal(1.00000000000000))*(cj0));
IkReal x560=((IkReal(1.10000000000000))*(sj1));
IkReal x561=((IkReal(0.350000000000000))*(sj3));
IkReal x562=((pz)*(sj1));
IkReal x563=((IkReal(0.0900000000000000))*(cj0));
IkReal x564=((cj0)*(px));
IkReal x565=((cj1)*(x555));
IkReal x566=((py)*(x554));
IkReal x567=((cj1)*(x554));
evalcond[0]=((((IkReal(0.0450000000000000))*(x554)))+(((IkReal(-1.00000000000000))*(x554)*(x556)))+(x557)+(((IkReal(-1.00000000000000))*(py)*(x559)))+(((x554)*(x561))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(0.0450000000000000))*(x555)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x559)))+(x562)+(((IkReal(-1.00000000000000))*(cj1)*(x558)))+(((x555)*(x561))));
evalcond[2]=((((x558)*(x567)))+(((x564)*(x567)))+(((x555)*(x557)))+(((IkReal(-1.00000000000000))*(x554)*(x562)))+(((IkReal(-1.00000000000000))*(py)*(x555)*(x559))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x558)*(x565)))+(((IkReal(-1.00000000000000))*(px)*(x559)*(x565)))+(((x554)*(x557)))+(((IkReal(-1.00000000000000))*(x556)))+(((IkReal(-1.00000000000000))*(x559)*(x566)))+(x561)+(((x555)*(x562))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(0.0900000000000000))*(x558)*(x565)))+(((x558)*(x560)))+(((IkReal(-0.0900000000000000))*(x554)*(x557)))+(((x563)*(x566)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((x560)*(x564)))+(((IkReal(-1.00000000000000))*(pp)))+(((px)*(x563)*(x565)))+(((IkReal(-0.0900000000000000))*(x555)*(x562))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x568=(sj1)*(sj1);
if( IKabs(((px)*(sj1))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((py)*(sj1))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x569=((IkReal(1.00000000000000))*(IKatan2(((px)*(sj1)), ((py)*(sj1)))));
if( (((((x568)*((px)*(px))))+(((x568)*((py)*(py)))))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt(((((x568)*((px)*(px))))+(((x568)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x568)*((px)*(px))))+(((x568)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt(((((x568)*((px)*(px))))+(((x568)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x568)*((px)*(px))))+(((x568)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x570=IKasin(((((IKabs(IKabs(IKsqrt(((((x568)*((px)*(px))))+(((x568)*((py)*(py)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt(((((x568)*((px)*(px))))+(((x568)*((py)*(py))))))))):(IkReal)1.0e30))*(((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((cj1)*(pz)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x570)))+(((IkReal(-1.00000000000000))*(x569))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x570)+(((IkReal(-1.00000000000000))*(x569))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst0;
gconst0=IKsign(((IkReal(0.0450000000000000))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))));
dummyeval[0]=((IkReal(1.00000000000000))+(((IkReal(7.77777777777778))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x571=((cj0)*(px));
IkReal x572=((py)*(sj0));
IkReal x573=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((sj1)*(x572)))+(((sj1)*(x571)))+(((cj1)*(pz))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x571)*(x573)))+(((pz)*(sj1)))+(((IkReal(-1.00000000000000))*(x572)*(x573))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
IkReal x574=(cj1)*(cj1);
IkReal x575=(px)*(px);
IkReal x576=(sj0)*(sj0);
IkReal x577=(cj0)*(cj0);
IkReal x578=(py)*(py);
IkReal x579=((py)*(sj0));
IkReal x580=((IkReal(0.0900000000000000))*(x576));
IkReal x581=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x582=((cj1)*(pz)*(sj1));
IkReal x583=((IkReal(0.0900000000000000))*(x577));
gconst1=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((x578)*(x583)))+(((x574)*(x575)*(x583)))+(((IkReal(-0.180000000000000))*(x579)*(x582)))+(((IkReal(-1.00000000000000))*(x579)*(x581)))+(((x574)*(x579)*(x581)))+(((x574)*(x578)*(x580)))+(((x575)*(x580)))+(((IkReal(-1.00000000000000))*(x581)*(x582)))));
IkReal x584=(sj0)*(sj0);
IkReal x585=(py)*(py);
IkReal x586=(cj1)*(cj1);
IkReal x587=(cj0)*(cj0);
IkReal x588=(px)*(px);
IkReal x589=((cj0)*(px));
IkReal x590=((py)*(sj0));
IkReal x591=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((x584)*(x588)))+(((x586)*(x587)*(x588)))+(((IkReal(2.00000000000000))*(x586)*(x589)*(x590)))+(((IkReal(-1.00000000000000))*(x589)*(x591)))+(((IkReal(-1.00000000000000))*(x590)*(x591)))+(((x585)*(x587)))+(((x584)*(x585)*(x586)))+((((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-2.00000000000000))*(x589)*(x590))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x592=(sj1)*(sj1);
IkReal x593=(py)*(py);
IkReal x594=(sj0)*(sj0);
IkReal x595=(cj1)*(cj1);
IkReal x596=(cj0)*(cj0);
IkReal x597=(px)*(px);
IkReal x598=((IkReal(1.10000000000000))*(sj1));
IkReal x599=((px)*(py));
IkReal x600=((py)*(sj0));
IkReal x601=((cj1)*(pp));
IkReal x602=((px)*(sj0));
IkReal x603=((IkReal(1.00000000000000))*(pp));
IkReal x604=((cj0)*(py));
IkReal x605=((cj0)*(px));
IkReal x606=((cj0)*(sj0));
IkReal x607=((pz)*(sj1));
IkReal x608=((IkReal(1.10000000000000))*(pz));
IkReal x609=((IkReal(0.180000000000000))*(cj1));
IkReal x610=((x592)*(x608));
if( IKabs(((gconst1)*(((((pp)*(x604)))+(((IkReal(-1.00000000000000))*(x602)*(x603)))+(((IkReal(-1.00000000000000))*(cj1)*(x604)*(x608)))+(((IkReal(-0.180000000000000))*(x602)))+(((cj1)*(x602)*(x608)))+(((x594)*(x598)*(x599)))+(((IkReal(-1.00000000000000))*(x596)*(x598)*(x599)))+(((IkReal(0.180000000000000))*(x604)))+(((IkReal(-1.00000000000000))*(x593)*(x598)*(x606)))+(((x597)*(x598)*(x606))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((x605)*(x609)))+(((cj1)*(x598)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x603)*(x607)))+(((x600)*(x610)))+(((x601)*(x605)))+(((x605)*(x610)))+(((IkReal(-0.180000000000000))*(x607)))+(((IkReal(-1.00000000000000))*(cj1)*(x596)*(x597)*(x598)))+(((x600)*(x609)))+(((x600)*(x601)))+(((IkReal(-1.00000000000000))*(cj1)*(x593)*(x594)*(x598)))+(((IkReal(-1.00000000000000))*(x595)*(x605)*(x608)))+(((IkReal(-1.00000000000000))*(x595)*(x600)*(x608)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x599)*(x606))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst1)*(((((pp)*(x604)))+(((IkReal(-1.00000000000000))*(x602)*(x603)))+(((IkReal(-1.00000000000000))*(cj1)*(x604)*(x608)))+(((IkReal(-0.180000000000000))*(x602)))+(((cj1)*(x602)*(x608)))+(((x594)*(x598)*(x599)))+(((IkReal(-1.00000000000000))*(x596)*(x598)*(x599)))+(((IkReal(0.180000000000000))*(x604)))+(((IkReal(-1.00000000000000))*(x593)*(x598)*(x606)))+(((x597)*(x598)*(x606)))))), ((gconst1)*(((((x605)*(x609)))+(((cj1)*(x598)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x603)*(x607)))+(((x600)*(x610)))+(((x601)*(x605)))+(((x605)*(x610)))+(((IkReal(-0.180000000000000))*(x607)))+(((IkReal(-1.00000000000000))*(cj1)*(x596)*(x597)*(x598)))+(((x600)*(x609)))+(((x600)*(x601)))+(((IkReal(-1.00000000000000))*(cj1)*(x593)*(x594)*(x598)))+(((IkReal(-1.00000000000000))*(x595)*(x605)*(x608)))+(((IkReal(-1.00000000000000))*(x595)*(x600)*(x608)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x599)*(x606)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x611=IKsin(j2);
IkReal x612=IKcos(j2);
IkReal x613=((px)*(sj0));
IkReal x614=((IkReal(1.10000000000000))*(sj1));
IkReal x615=((py)*(sj0));
IkReal x616=((pz)*(sj1));
IkReal x617=((cj0)*(px));
IkReal x618=((IkReal(0.0900000000000000))*(x612));
IkReal x619=((cj1)*(x611));
IkReal x620=((IkReal(1.00000000000000))*(x612));
IkReal x621=((cj0)*(py)*(x611));
evalcond[0]=((((IkReal(-1.00000000000000))*(x611)*(x616)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x620)))+(((x612)*(x613)))+(((x617)*(x619)))+(((x615)*(x619))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x615)*(x620)))+(((x611)*(x613)))+(((IkReal(-1.00000000000000))*(cj1)*(x617)*(x620)))+(((x612)*(x616)))+(((IkReal(-1.00000000000000))*(x621))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x611)*(x613)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(x621)))+(((x614)*(x615)))+(((x614)*(x617)))+(((cj1)*(x615)*(x618)))+(((cj1)*(x617)*(x618)))+(((IkReal(-1.00000000000000))*(x616)*(x618))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x622=((cj0)*(px));
IkReal x623=((py)*(sj0));
IkReal x624=((IkReal(1.00000000000000))*(cj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.810000000000000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.900000000000000))+(((cj1)*(pz)))+(((sj1)*(x622)))+(((sj1)*(x623))));
evalcond[3]=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x623)*(x624)))+(((IkReal(-1.00000000000000))*(x622)*(x624)))+(((pz)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
IkReal x625=(cj1)*(cj1);
IkReal x626=(px)*(px);
IkReal x627=(sj0)*(sj0);
IkReal x628=(cj0)*(cj0);
IkReal x629=(py)*(py);
IkReal x630=((py)*(sj0));
IkReal x631=((IkReal(0.0900000000000000))*(x627));
IkReal x632=((IkReal(0.180000000000000))*(cj0)*(px));
IkReal x633=((cj1)*(pz)*(sj1));
IkReal x634=((IkReal(0.0900000000000000))*(x628));
gconst2=IKsign(((((IkReal(0.0900000000000000))*((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-1.00000000000000))*(x630)*(x632)))+(((x625)*(x630)*(x632)))+(((x629)*(x634)))+(((IkReal(-0.180000000000000))*(x630)*(x633)))+(((x625)*(x626)*(x634)))+(((x625)*(x629)*(x631)))+(((x626)*(x631)))+(((IkReal(-1.00000000000000))*(x632)*(x633)))));
IkReal x635=(sj0)*(sj0);
IkReal x636=(py)*(py);
IkReal x637=(cj1)*(cj1);
IkReal x638=(cj0)*(cj0);
IkReal x639=(px)*(px);
IkReal x640=((cj0)*(px));
IkReal x641=((py)*(sj0));
IkReal x642=((IkReal(2.00000000000000))*(cj1)*(pz)*(sj1));
dummyeval[0]=((((x637)*(x638)*(x639)))+(((x635)*(x636)*(x637)))+(((IkReal(2.00000000000000))*(x637)*(x640)*(x641)))+(((x635)*(x639)))+(((IkReal(-1.00000000000000))*(x640)*(x642)))+(((IkReal(-1.00000000000000))*(x641)*(x642)))+(((x636)*(x638)))+((((pz)*(pz))*((sj1)*(sj1))))+(((IkReal(-2.00000000000000))*(x640)*(x641))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x643=(sj1)*(sj1);
IkReal x644=(py)*(py);
IkReal x645=(sj0)*(sj0);
IkReal x646=(cj1)*(cj1);
IkReal x647=(cj0)*(cj0);
IkReal x648=(px)*(px);
IkReal x649=((IkReal(1.10000000000000))*(sj1));
IkReal x650=((px)*(py));
IkReal x651=((py)*(sj0));
IkReal x652=((cj1)*(pp));
IkReal x653=((px)*(sj0));
IkReal x654=((IkReal(1.00000000000000))*(pp));
IkReal x655=((cj0)*(sj0));
IkReal x656=((cj0)*(px));
IkReal x657=((cj0)*(py));
IkReal x658=((pz)*(sj1));
IkReal x659=((IkReal(0.180000000000000))*(cj1));
IkReal x660=((IkReal(1.10000000000000))*(pz));
IkReal x661=((x643)*(x660));
if( IKabs(((gconst2)*(((((pp)*(x657)))+(((IkReal(-1.00000000000000))*(x644)*(x649)*(x655)))+(((x645)*(x649)*(x650)))+(((x648)*(x649)*(x655)))+(((IkReal(-0.180000000000000))*(x653)))+(((IkReal(0.180000000000000))*(x657)))+(((IkReal(-1.00000000000000))*(cj1)*(x657)*(x660)))+(((IkReal(-1.00000000000000))*(x647)*(x649)*(x650)))+(((IkReal(-1.00000000000000))*(x653)*(x654)))+(((cj1)*(x653)*(x660))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x646)*(x651)*(x660)))+(((x651)*(x661)))+(((x652)*(x656)))+(((x656)*(x661)))+(((x651)*(x652)))+(((x651)*(x659)))+(((x656)*(x659)))+(((IkReal(-0.180000000000000))*(x658)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x650)*(x655)))+(((IkReal(-1.00000000000000))*(cj1)*(x644)*(x645)*(x649)))+(((IkReal(-1.00000000000000))*(cj1)*(x647)*(x648)*(x649)))+(((IkReal(-1.00000000000000))*(x646)*(x656)*(x660)))+(((cj1)*(x649)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x654)*(x658))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst2)*(((((pp)*(x657)))+(((IkReal(-1.00000000000000))*(x644)*(x649)*(x655)))+(((x645)*(x649)*(x650)))+(((x648)*(x649)*(x655)))+(((IkReal(-0.180000000000000))*(x653)))+(((IkReal(0.180000000000000))*(x657)))+(((IkReal(-1.00000000000000))*(cj1)*(x657)*(x660)))+(((IkReal(-1.00000000000000))*(x647)*(x649)*(x650)))+(((IkReal(-1.00000000000000))*(x653)*(x654)))+(((cj1)*(x653)*(x660)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x646)*(x651)*(x660)))+(((x651)*(x661)))+(((x652)*(x656)))+(((x656)*(x661)))+(((x651)*(x652)))+(((x651)*(x659)))+(((x656)*(x659)))+(((IkReal(-0.180000000000000))*(x658)))+(((IkReal(-2.20000000000000))*(cj1)*(sj1)*(x650)*(x655)))+(((IkReal(-1.00000000000000))*(cj1)*(x644)*(x645)*(x649)))+(((IkReal(-1.00000000000000))*(cj1)*(x647)*(x648)*(x649)))+(((IkReal(-1.00000000000000))*(x646)*(x656)*(x660)))+(((cj1)*(x649)*((pz)*(pz))))+(((IkReal(-1.00000000000000))*(x654)*(x658)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[3];
IkReal x662=IKsin(j2);
IkReal x663=IKcos(j2);
IkReal x664=((px)*(sj0));
IkReal x665=((IkReal(1.10000000000000))*(sj1));
IkReal x666=((py)*(sj0));
IkReal x667=((pz)*(sj1));
IkReal x668=((cj0)*(px));
IkReal x669=((IkReal(0.0900000000000000))*(x663));
IkReal x670=((cj1)*(x662));
IkReal x671=((IkReal(1.00000000000000))*(x663));
IkReal x672=((cj0)*(py)*(x662));
evalcond[0]=((((x663)*(x664)))+(((x666)*(x670)))+(((IkReal(-1.00000000000000))*(cj0)*(py)*(x671)))+(((x668)*(x670)))+(((IkReal(-1.00000000000000))*(x662)*(x667))));
evalcond[1]=((((x662)*(x664)))+(((x663)*(x667)))+(((IkReal(-1.00000000000000))*(cj1)*(x666)*(x671)))+(((IkReal(-1.00000000000000))*(x672)))+(((IkReal(-1.00000000000000))*(cj1)*(x668)*(x671))));
evalcond[2]=((IkReal(-0.180000000000000))+(((x665)*(x666)))+(((x665)*(x668)))+(((IkReal(-0.0900000000000000))*(x662)*(x664)))+(((IkReal(0.0900000000000000))*(x672)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((cj1)*(x668)*(x669)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x667)*(x669)))+(((cj1)*(x666)*(x669))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-0.255739837120000))+(j3)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.0400000003605000))+(((IkReal(-1.00000000000000))*(pp))));
evalcond[2]=((IkReal(-0.200000000250000))+(((cj1)*(pz)))+(((cj0)*(px)*(sj1)))+(((py)*(sj0)*(sj1))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x673=((IkReal(1052631578.94737))*(py));
IkReal x674=((IkReal(1052631578.94737))*(px));
if( IKabs(((((cj0)*(x673)))+(((IkReal(-1.00000000000000))*(sj0)*(x674))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj0)*(cj1)*(x674)))+(((cj1)*(sj0)*(x673))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj0)*(x673)))+(((IkReal(-1.00000000000000))*(sj0)*(x674)))))+IKsqr(((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj0)*(cj1)*(x674)))+(((cj1)*(sj0)*(x673)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j2array[0]=IKatan2(((((cj0)*(x673)))+(((IkReal(-1.00000000000000))*(sj0)*(x674)))), ((((IkReal(-1052631578.94737))*(pz)*(sj1)))+(((cj0)*(cj1)*(x674)))+(((cj1)*(sj0)*(x673)))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x675=IKcos(j2);
IkReal x676=IKsin(j2);
IkReal x677=((px)*(sj0));
IkReal x678=((py)*(sj0));
IkReal x679=((IkReal(1.00000000000000))*(cj1));
IkReal x680=((IkReal(1.10000000000000))*(sj1));
IkReal x681=((pz)*(sj1));
IkReal x682=((cj0)*(px));
IkReal x683=((IkReal(0.0900000000000000))*(x676));
IkReal x684=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x685=((cj1)*(x676));
IkReal x686=((IkReal(0.0900000000000000))*(x675));
evalcond[0]=((((IkReal(-1.00000000000000))*(x684)))+(((IkReal(9.50000000000000e-10))*(x676)))+(x677));
evalcond[1]=((((IkReal(-1.00000000000000))*(x678)*(x679)))+(((IkReal(-1.00000000000000))*(x679)*(x682)))+(((IkReal(9.50000000000000e-10))*(x675)))+(x681));
evalcond[2]=((((x678)*(x685)))+(((x682)*(x685)))+(((IkReal(-1.00000000000000))*(x675)*(x684)))+(((IkReal(-1.00000000000000))*(x676)*(x681)))+(((x675)*(x677))));
evalcond[3]=((IkReal(9.50000000000000e-10))+(((IkReal(-1.00000000000000))*(x675)*(x679)*(x682)))+(((x676)*(x677)))+(((IkReal(-1.00000000000000))*(x675)*(x678)*(x679)))+(((x675)*(x681)))+(((IkReal(-1.00000000000000))*(x676)*(x684))));
evalcond[4]=((IkReal(-0.180000000000000))+(((x678)*(x680)))+(((IkReal(-1.00000000000000))*(x681)*(x686)))+(((cj1)*(x678)*(x686)))+(((cj0)*(py)*(x683)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((x680)*(x682)))+(((IkReal(-1.00000000000000))*(x677)*(x683)))+(((cj1)*(x682)*(x686))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(sj0)))+(((cj0)*(py)))))), ((gconst0)*(((((cj1)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*(pz)*(sj1)))+(((cj0)*(cj1)*(px)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[5];
IkReal x687=IKsin(j2);
IkReal x688=IKcos(j2);
IkReal x689=((IkReal(0.0450000000000000))*(cj3));
IkReal x690=((px)*(sj0));
IkReal x691=((py)*(sj0));
IkReal x692=((IkReal(1.00000000000000))*(cj0));
IkReal x693=((IkReal(1.10000000000000))*(sj1));
IkReal x694=((IkReal(0.350000000000000))*(sj3));
IkReal x695=((pz)*(sj1));
IkReal x696=((IkReal(0.0900000000000000))*(cj0));
IkReal x697=((cj0)*(px));
IkReal x698=((cj1)*(x688));
IkReal x699=((py)*(x687));
IkReal x700=((cj1)*(x687));
evalcond[0]=((((IkReal(0.0450000000000000))*(x687)))+(((IkReal(-1.00000000000000))*(x687)*(x689)))+(((x687)*(x694)))+(x690)+(((IkReal(-1.00000000000000))*(py)*(x692))));
evalcond[1]=((((IkReal(0.0450000000000000))*(x688)))+(((IkReal(-1.00000000000000))*(x688)*(x689)))+(((x688)*(x694)))+(((IkReal(-1.00000000000000))*(cj1)*(x691)))+(x695)+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x692))));
evalcond[2]=((((x688)*(x690)))+(((x691)*(x700)))+(((IkReal(-1.00000000000000))*(x687)*(x695)))+(((x697)*(x700)))+(((IkReal(-1.00000000000000))*(py)*(x688)*(x692))));
evalcond[3]=((IkReal(0.0450000000000000))+(((x688)*(x695)))+(((IkReal(-1.00000000000000))*(x689)))+(((x687)*(x690)))+(((IkReal(-1.00000000000000))*(x691)*(x698)))+(((IkReal(-1.00000000000000))*(x692)*(x699)))+(x694)+(((IkReal(-1.00000000000000))*(px)*(x692)*(x698))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x688)*(x695)))+(((x696)*(x699)))+(((IkReal(0.0900000000000000))*(x691)*(x698)))+(((x691)*(x693)))+(((IkReal(-0.0900000000000000))*(x687)*(x690)))+(((IkReal(1.10000000000000))*(cj1)*(pz)))+(((IkReal(-1.00000000000000))*(pp)))+(((x693)*(x697)))+(((px)*(x696)*(x698))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "4382fadc0284794e13f46b1dd48b52ef"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
