/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:04:41.294078
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25;
x0=IKsin(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[1]);
x3=IKcos(j[2]);
x4=IKsin(j[2]);
x5=IKcos(j[0]);
x6=IKcos(j[3]);
x7=IKsin(j[3]);
x8=IKcos(j[5]);
x9=IKsin(j[5]);
x10=IKsin(j[4]);
x11=IKcos(j[4]);
x12=((IkReal(0.331000000000000))*(x0));
x13=((IkReal(1.00000000000000))*(x11));
x14=((IkReal(1.00000000000000))*(x0));
x15=((IkReal(1.00000000000000))*(x5));
x16=((IkReal(1.00000000000000))*(x10));
x17=((IkReal(0.405000000000000))*(x1));
x18=((IkReal(0.331000000000000))*(x5));
x19=((x1)*(x3));
x20=((x2)*(x4));
x21=((x2)*(x3));
x22=((x1)*(x4));
x23=((IkReal(1.00000000000000))*(x19));
x24=((x14)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x22))))));
x25=((x15)*(((x21)+(x22))));
eetrans[0]=((IkReal(0.0454160000000000))+(((IkReal(-0.100150000000000))*(x5)))+(((IkReal(-1.00000000000000))*(x12)*(x19)))+(((IkReal(-1.00000000000000))*(x0)*(x17)))+(((x12)*(x20))));
eetrans[1]=((IkReal(0.302408000000000))+(((IkReal(0.331000000000000))*(x21)))+(((IkReal(0.331000000000000))*(x22)))+(((IkReal(0.405000000000000))*(x2))));
eetrans[2]=((IkReal(0.467052000000000))+(((x18)*(x19)))+(((IkReal(-0.100150000000000))*(x0)))+(((IkReal(-1.00000000000000))*(x18)*(x20)))+(((x17)*(x5))));
eerot[0]=((((x8)*(((((IkReal(-1.00000000000000))*(x5)*(x7)))+(((x24)*(x6)))))))+(((IkReal(-1.00000000000000))*(x9)*(((((x13)*(((((x5)*(x6)))+(((x24)*(x7)))))))+(((x16)*(((((x0)*(x19)))+(((IkReal(-1.00000000000000))*(x14)*(x20))))))))))));
eerot[1]=((((x9)*(((((x11)*(x7)*(((((IkReal(-1.00000000000000))*(x20)))+(x23)))))+(((x10)*(((x21)+(x22)))))))))+(((x6)*(x8)*(((((IkReal(-1.00000000000000))*(x23)))+(x20))))));
eerot[2]=((((IkReal(-1.00000000000000))*(x9)*(((((x16)*(((((IkReal(-1.00000000000000))*(x15)*(x19)))+(((x20)*(x5)))))))+(((x13)*(((((x0)*(x6)))+(((x25)*(x7)))))))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x14)*(x7)))+(((x25)*(x6))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {4}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j5,cj5,sj5,htj5,j4,cj4,sj4,htj4,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij5[2], _nj5,_ij4[2], _nj4;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij4[0] = -1; _ij4[1] = -1; _nj4 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j4=pfree[0]; cj4=cos(pfree[0]); sj4=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((IkReal(-0.0454160000000000))+(px));
new_r01=r02;
new_py=((IkReal(-0.467052000000000))+(pz));
new_r02=((IkReal(-1.00000000000000))*(r01));
new_pz=((IkReal(0.302408000000000))+(((IkReal(-1.00000000000000))*(py))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal j2array[2], cj2array[2], sj2array[2];
bool j2valid[2]={false};
_nj2 = 2;
cj2array[0]=((IkReal(-1.05783455484689))+(((IkReal(3.72981239043676))*(pp))));
if( cj2array[0] >= -1-IKFAST_SINCOS_THRESH && cj2array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j2valid[0] = j2valid[1] = true;
    j2array[0] = IKacos(cj2array[0]);
    sj2array[0] = IKsin(j2array[0]);
    cj2array[1] = cj2array[0];
    j2array[1] = -j2array[0];
    sj2array[1] = -sj2array[0];
}
else if( isnan(cj2array[0]) )
{
    // probably any value will work
    j2valid[0] = true;
    cj2array[0] = 1; sj2array[0] = 0; j2array[0] = 0;
}
for(int ij2 = 0; ij2 < 2; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 2; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.49711119832787))+((sj2)*(sj2))+((cj2)*(cj2))+(((IkReal(2.44712990936556))*(cj2))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x26=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2))));
if( IKabs(((IkReal(0.331000000000000))*(sj2))) < IKFAST_ATAN2_MAGTHRESH && IKabs(x26) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x27=((IkReal(1.00000000000000))*(IKatan2(((IkReal(0.331000000000000))*(sj2)), x26)));
if( ((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x28=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2)))))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x26)*(x26))+(((IkReal(0.109561000000000))*((sj2)*(sj2))))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x27)))+(((IkReal(-1.00000000000000))*(x28))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x27)))+(x28));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x29=((IkReal(20000.0000000000))*(sj1));
gconst0=IKsign(((((x29)*((py)*(py))))+(((x29)*((px)*(px))))));
dummyeval[0]=((((sj1)*((py)*(py))))+(((sj1)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x30=((pz)+(((IkReal(0.331000000000000))*(sj2))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=x30;
evalcond[3]=x30;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((IkReal(-20000.0000000000))*((py)*(py))))+(((IkReal(-20000.0000000000))*((px)*(px))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x31=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x31)))+(((IkReal(2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x31)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x31)))+(((IkReal(2003.00000000000))*(py)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(py)*(x31)))+(((IkReal(-8100.00000000000))*(py)))+(((IkReal(2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x32=IKsin(j0);
IkReal x33=IKcos(j0);
IkReal x34=((px)*(x32));
IkReal x35=((py)*(x33));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x32)))+(((px)*(x33))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x35)))+(((IkReal(0.331000000000000))*(cj2)))+(x34));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(0.810000000000000))*(x35)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x34))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x36=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x36)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x36)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x37=((sj0)*(sj1));
IkReal x38=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x37)))+(((r02)*(sj2)*(x37)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x38)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x38))));
IkReal gconst6;
IkReal x39=((sj0)*(sj1));
IkReal x40=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x39)))+(((r02)*(sj2)*(x39)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x40)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x40))));
IkReal gconst7;
IkReal x41=((cj0)*(r02));
IkReal x42=((cj0)*(r00));
gconst7=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x42)))+(((cj1)*(sj2)*(x42)))+(((cj1)*(cj2)*(x41)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x41))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x43=((cj0)*(r02));
IkReal x44=((cj0)*(r00));
gconst9=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x44)))+(((cj1)*(sj2)*(x44)))+(((cj1)*(cj2)*(x43)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x43))));
IkReal gconst10;
IkReal x45=((sj0)*(sj1));
IkReal x46=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x45)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x45)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x46)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x46))));
IkReal gconst11;
IkReal x47=((sj0)*(sj1));
IkReal x48=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x47)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x47)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x48)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x48))));
IkReal gconst12;
IkReal x49=((cj0)*(r02));
IkReal x50=((cj0)*(r00));
gconst12=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x49)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x49)))+(((cj2)*(sj1)*(x50)))+(((cj1)*(sj2)*(x50))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x51=((cj0)*(r02));
IkReal x52=((cj0)*(r00));
gconst14=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x51)))+(((cj2)*(sj1)*(x52)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x51)))+(((cj1)*(sj2)*(x52))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x53=((sj0)*(sj1));
IkReal x54=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x54)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x53)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x53)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x54))));
IkReal gconst22;
IkReal x55=((sj0)*(sj1));
IkReal x56=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x56)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x55)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x55)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x56))));
IkReal gconst23;
IkReal x57=((cj0)*(r02));
IkReal x58=((cj0)*(r00));
gconst23=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x57)))+(((cj2)*(sj1)*(x58)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x57)))+(((cj1)*(sj2)*(x58))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x59=((cj0)*(r02));
IkReal x60=((cj0)*(r00));
gconst25=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x59)))+(((cj2)*(sj1)*(x60)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x59)))+(((cj1)*(sj2)*(x60))));
IkReal gconst26;
IkReal x61=((sj0)*(sj1));
IkReal x62=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x62)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x61)))+(((r02)*(sj2)*(x61)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x62))));
IkReal gconst27;
IkReal x63=((sj0)*(sj1));
IkReal x64=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x64)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x63)))+(((r02)*(sj2)*(x63)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x64))));
IkReal gconst28;
IkReal x65=((cj0)*(r02));
IkReal x66=((cj0)*(r00));
gconst28=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x65)))+(((cj2)*(sj1)*(x66)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x65)))+(((cj1)*(sj2)*(x66))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x67=((cj0)*(r02));
IkReal x68=((cj0)*(r00));
gconst30=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x67)))+(((cj2)*(sj1)*(x68)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x67)))+(((cj1)*(sj2)*(x68))));
IkReal x69=((gconst27)*(gconst28));
IkReal x70=((gconst22)*(gconst23));
IkReal x71=((gconst21)*(gconst25));
IkReal x72=((gconst26)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x69)*(x70)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((x69)*(x71)))+(((IkReal(-1.00000000000000))*(x71)*(x72)))+(((x70)*(x72))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x73=((IkReal(1.00000000000000))*(gconst19));
IkReal x74=((gconst23)*(gconst27));
IkReal x75=((gconst15)*(gconst18));
IkReal x76=((gconst11)*(gconst6));
IkReal x77=((gconst13)*(gconst26));
IkReal x78=((gconst15)*(gconst9));
IkReal x79=((gconst6)*(gconst8));
IkReal x80=((gconst18)*(gconst28));
IkReal x81=((gconst22)*(gconst30));
IkReal x82=((gconst16)*(gconst26));
IkReal x83=((IkReal(1.00000000000000))*(gconst10));
IkReal x84=((gconst14)*(gconst25));
IkReal x85=((IkReal(1.00000000000000))*(gconst26));
IkReal x86=((gconst12)*(gconst27));
IkReal x87=((gconst25)*(gconst5));
IkReal x88=((gconst17)*(gconst28));
IkReal x89=((gconst21)*(gconst27));
IkReal x90=((gconst27)*(gconst6));
IkReal x91=((IkReal(1.00000000000000))*(gconst28));
IkReal x92=((gconst11)*(gconst5));
IkReal x93=((gconst17)*(gconst21));
IkReal x94=((gconst18)*(gconst19));
IkReal x95=((gconst10)*(gconst7));
IkReal x96=((gconst30)*(gconst6));
IkReal x97=((gconst17)*(gconst5));
IkReal x98=((IkReal(1.00000000000000))*(gconst12));
IkReal x99=((gconst22)*(gconst7));
IkReal x100=((gconst24)*(gconst6));
IkReal x101=((gconst5)*(gconst9));
IkReal x102=((gconst15)*(gconst17));
IkReal x103=((gconst27)*(gconst28));
IkReal x104=((gconst23)*(gconst26));
IkReal x105=((gconst21)*(gconst9));
IkReal x106=((gconst22)*(gconst8));
IkReal x107=((gconst10)*(gconst23));
IkReal x108=((gconst12)*(gconst18));
IkReal x109=((gconst21)*(gconst25));
IkReal x110=((gconst11)*(gconst28));
IkReal x111=((gconst11)*(gconst19));
IkReal x112=((gconst19)*(gconst27));
IkReal x113=((gconst20)*(gconst6));
IkReal x114=((gconst10)*(gconst16));
IkReal x115=((gconst26)*(gconst29));
IkReal x116=((gconst26)*(gconst7));
IkReal x117=((gconst20)*(gconst22));
IkReal x118=((gconst22)*(gconst24));
IkReal x119=((gconst15)*(gconst25));
IkReal x120=((gconst10)*(gconst13));
IkReal x121=((gconst14)*(gconst6));
IkReal x122=((gconst16)*(gconst22));
IkReal x123=((gconst11)*(gconst12));
IkReal x124=((gconst14)*(gconst22));
IkReal x125=((gconst17)*(gconst19));
IkReal x126=((gconst10)*(gconst29));
IkReal x127=((gconst28)*(gconst9));
IkReal x128=((IkReal(1.00000000000000))*(gconst6));
IkReal x129=((gconst26)*(x113));
IkReal x130=((gconst9)*(x111));
IkReal x131=((gconst11)*(x122));
IkReal x132=((gconst14)*(x107));
IkReal x133=((gconst11)*(gconst22)*(gconst23));
op[0]=((((IkReal(-1.00000000000000))*(gconst29)*(x118)*(x85)))+(((gconst25)*(gconst28)*(x89)))+(((x104)*(x81)))+(((IkReal(-1.00000000000000))*(gconst30)*(x109)*(x85)))+(((IkReal(-1.00000000000000))*(gconst22)*(x74)*(x91))));
op[1]=((((IkReal(-1.00000000000000))*(gconst22)*(x73)*(x74)))+(((x81)*(x82)))+(((gconst19)*(gconst25)*(x89)))+(((IkReal(-1.00000000000000))*(gconst30)*(x85)*(x93)))+(((x88)*(x89)))+(((x118)*(x80)))+(((x104)*(x117)))+(((IkReal(-1.00000000000000))*(gconst27)*(x122)*(x91)))+(((x115)*(x119)))+(((IkReal(-1.00000000000000))*(gconst20)*(x109)*(x85))));
op[2]=((((x107)*(x81)))+(((IkReal(-1.00000000000000))*(gconst30)*(x105)*(x85)))+(((IkReal(-1.00000000000000))*(gconst6)*(x74)*(x91)))+(((x117)*(x82)))+(((x103)*(x87)))+(((IkReal(-1.00000000000000))*(gconst29)*(x106)*(x85)))+(((x125)*(x89)))+(((x102)*(x115)))+(((IkReal(-1.00000000000000))*(gconst25)*(x75)*(x91)))+(((x109)*(x110)))+(((IkReal(-1.00000000000000))*(x118)*(x77)))+(((IkReal(-1.00000000000000))*(gconst21)*(x84)*(x85)))+(((IkReal(-1.00000000000000))*(gconst29)*(x100)*(x85)))+(((IkReal(-1.00000000000000))*(gconst30)*(x85)*(x87)))+(((IkReal(-1.00000000000000))*(gconst27)*(x122)*(x73)))+(((x118)*(x94)))+(((IkReal(-1.00000000000000))*(x133)*(x91)))+(((x104)*(x124)))+(((x109)*(x86)))+(((x127)*(x89)))+(((IkReal(-1.00000000000000))*(gconst29)*(x118)*(x83)))+(((x116)*(x81)))+(((IkReal(-1.00000000000000))*(gconst27)*(x91)*(x99)))+(((IkReal(-1.00000000000000))*(gconst30)*(x109)*(x83)))+(((x104)*(x96)))+(((IkReal(-1.00000000000000))*(gconst20)*(x85)*(x93)))+(((IkReal(-1.00000000000000))*(gconst22)*(x74)*(x98))));
op[3]=((((gconst11)*(gconst21)*(x88)))+(((gconst19)*(gconst9)*(x89)))+(((gconst27)*(gconst5)*(x88)))+(((IkReal(-1.00000000000000))*(gconst20)*(x85)*(x87)))+(((x82)*(x96)))+(((x100)*(x80)))+(((gconst20)*(gconst26)*(x99)))+(((x124)*(x82)))+(((x86)*(x93)))+(((x115)*(x78)))+(((x109)*(x111)))+(((x114)*(x81)))+(((IkReal(-1.00000000000000))*(gconst16)*(x90)*(x91)))+(((x119)*(x77)))+(((x106)*(x80)))+(((IkReal(-1.00000000000000))*(x122)*(x86)))+(((IkReal(-1.00000000000000))*(gconst20)*(x105)*(x85)))+(((x112)*(x87)))+(((x108)*(x118)))+(((IkReal(-1.00000000000000))*(x75)*(x88)))+(((IkReal(-1.00000000000000))*(gconst30)*(x85)*(x97)))+(((IkReal(-1.00000000000000))*(gconst25)*(x73)*(x75)))+(((x104)*(x113)))+(((IkReal(-1.00000000000000))*(gconst6)*(x73)*(x74)))+(((IkReal(-1.00000000000000))*(gconst27)*(x73)*(x99)))+(((IkReal(-1.00000000000000))*(x133)*(x73)))+(((IkReal(-1.00000000000000))*(x131)*(x91)))+(((x119)*(x126)))+(((IkReal(-1.00000000000000))*(gconst30)*(x83)*(x93)))+(((IkReal(-1.00000000000000))*(gconst14)*(x85)*(x93)))+(((IkReal(-1.00000000000000))*(gconst20)*(x109)*(x83)))+(((x107)*(x117))));
op[4]=((((x110)*(x87)))+(((x113)*(x82)))+(((x100)*(x94)))+(((IkReal(-1.00000000000000))*(gconst30)*(x105)*(x83)))+(((IkReal(-1.00000000000000))*(gconst6)*(x74)*(x98)))+(((x107)*(x96)))+(((x81)*(x95)))+(((x114)*(x117)))+(((IkReal(-1.00000000000000))*(x86)*(x99)))+(((IkReal(-1.00000000000000))*(gconst29)*(x106)*(x83)))+(((x102)*(x126)))+(((IkReal(-1.00000000000000))*(gconst7)*(x90)*(x91)))+(((IkReal(-1.00000000000000))*(gconst17)*(x73)*(x75)))+(((x86)*(x87)))+(((IkReal(-1.00000000000000))*(gconst25)*(x75)*(x98)))+(((IkReal(-1.00000000000000))*(x100)*(x77)))+(((IkReal(-1.00000000000000))*(gconst9)*(x75)*(x91)))+(((x106)*(x94)))+(((IkReal(-1.00000000000000))*(gconst21)*(x83)*(x84)))+(((IkReal(-1.00000000000000))*(gconst29)*(x100)*(x83)))+(((IkReal(-1.00000000000000))*(gconst5)*(x84)*(x85)))+(((IkReal(-1.00000000000000))*(x106)*(x77)))+(((IkReal(-1.00000000000000))*(gconst29)*(x79)*(x85)))+(((x109)*(x123)))+(((IkReal(-1.00000000000000))*(gconst14)*(x105)*(x85)))+(((IkReal(-1.00000000000000))*(x133)*(x98)))+(((IkReal(-1.00000000000000))*(gconst20)*(x83)*(x93)))+(((gconst14)*(gconst26)*(x99)))+(((IkReal(-1.00000000000000))*(gconst23)*(x76)*(x91)))+(((x112)*(x97)))+(((x111)*(x93)))+(((x104)*(x121)))+(((IkReal(-1.00000000000000))*(gconst30)*(x101)*(x85)))+(((IkReal(-1.00000000000000))*(gconst11)*(x91)*(x99)))+(((x105)*(x110)))+(((x116)*(x96)))+(((IkReal(-1.00000000000000))*(gconst16)*(x73)*(x90)))+(((x101)*(x103)))+(((x107)*(x124)))+(((IkReal(-1.00000000000000))*(x131)*(x73)))+(((IkReal(-1.00000000000000))*(gconst13)*(x118)*(x83)))+(((x102)*(x77)))+(((IkReal(-1.00000000000000))*(gconst30)*(x83)*(x87)))+(((x105)*(x86)))+(((IkReal(-1.00000000000000))*(gconst20)*(x85)*(x97))));
op[5]=((((IkReal(-1.00000000000000))*(gconst20)*(x101)*(x85)))+(((x126)*(x78)))+(((x117)*(x95)))+(((x79)*(x80)))+(((IkReal(-1.00000000000000))*(gconst14)*(x83)*(x93)))+(((x106)*(x108)))+(((IkReal(-1.00000000000000))*(gconst23)*(x73)*(x76)))+(((x86)*(x97)))+(((x114)*(x124)))+(((x113)*(x116)))+(((x114)*(x96)))+(((IkReal(-1.00000000000000))*(gconst17)*(x75)*(x98)))+(((IkReal(-1.00000000000000))*(gconst7)*(x73)*(x90)))+(((IkReal(-1.00000000000000))*(gconst9)*(x73)*(x75)))+(((x121)*(x82)))+(((IkReal(-1.00000000000000))*(gconst11)*(x73)*(x99)))+(((IkReal(-1.00000000000000))*(gconst20)*(x83)*(x87)))+(((IkReal(-1.00000000000000))*(gconst16)*(x76)*(x91)))+(((x88)*(x92)))+(((IkReal(-1.00000000000000))*(gconst20)*(x105)*(x83)))+(((x111)*(x87)))+(((IkReal(-1.00000000000000))*(gconst16)*(x128)*(x86)))+(((x105)*(x111)))+(((x101)*(x112)))+(((IkReal(-1.00000000000000))*(x131)*(x98)))+(((x119)*(x120)))+(((IkReal(-1.00000000000000))*(gconst30)*(x83)*(x97)))+(((x77)*(x78)))+(((IkReal(-1.00000000000000))*(gconst14)*(x85)*(x97)))+(((x123)*(x93)))+(((x107)*(x113)))+(((x100)*(x108))));
op[6]=((((x79)*(x94)))+(((x101)*(x86)))+(((x125)*(x92)))+(((x124)*(x95)))+(((x102)*(x120)))+(((x113)*(x114)))+(((IkReal(-1.00000000000000))*(gconst9)*(x75)*(x98)))+(((IkReal(-1.00000000000000))*(gconst7)*(x76)*(x91)))+(((IkReal(-1.00000000000000))*(gconst7)*(x128)*(x86)))+(((IkReal(-1.00000000000000))*(gconst5)*(x83)*(x84)))+(((IkReal(-1.00000000000000))*(x77)*(x79)))+(((IkReal(-1.00000000000000))*(gconst29)*(x79)*(x83)))+(((IkReal(-1.00000000000000))*(gconst11)*(x98)*(x99)))+(((IkReal(-1.00000000000000))*(gconst14)*(x105)*(x83)))+(((x105)*(x123)))+(((IkReal(-1.00000000000000))*(gconst20)*(x83)*(x97)))+(((x127)*(x92)))+(((IkReal(-1.00000000000000))*(gconst23)*(x76)*(x98)))+(((IkReal(-1.00000000000000))*(gconst16)*(x73)*(x76)))+(((x116)*(x121)))+(((IkReal(-1.00000000000000))*(gconst13)*(x100)*(x83)))+(((IkReal(-1.00000000000000))*(gconst30)*(x101)*(x83)))+(((x95)*(x96)))+(((IkReal(-1.00000000000000))*(gconst13)*(x106)*(x83)))+(((x123)*(x87)))+(((x107)*(x121)))+(((IkReal(-1.00000000000000))*(gconst14)*(x101)*(x85))));
op[7]=((((x113)*(x95)))+(((IkReal(-1.00000000000000))*(gconst20)*(x101)*(x83)))+(((gconst19)*(gconst9)*(x92)))+(((IkReal(-1.00000000000000))*(gconst14)*(x83)*(x97)))+(((IkReal(-1.00000000000000))*(gconst7)*(x73)*(x76)))+(((x114)*(x121)))+(((x120)*(x78)))+(((IkReal(-1.00000000000000))*(gconst16)*(x76)*(x98)))+(((x108)*(x79)))+(((gconst12)*(gconst17)*(x92))));
op[8]=((((IkReal(-1.00000000000000))*(gconst13)*(x79)*(x83)))+(((IkReal(-1.00000000000000))*(gconst7)*(x76)*(x98)))+(((x121)*(x95)))+(((gconst12)*(gconst9)*(x92)))+(((IkReal(-1.00000000000000))*(gconst14)*(x101)*(x83))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x134=((r01)*(sj3));
IkReal x135=((IkReal(1.00000000000000))*(sj0));
IkReal x136=((sj1)*(sj3));
IkReal x137=((cj2)*(r00));
IkReal x138=((r02)*(sj2));
IkReal x139=((cj3)*(sj1));
IkReal x140=((IkReal(1.00000000000000))*(cj0));
IkReal x141=((cj3)*(r01));
IkReal x142=((cj1)*(r00)*(sj2));
IkReal x143=((cj1)*(cj2)*(r02)*(sj3));
IkReal x144=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x135)*(x136)*(x138)))+(((cj0)*(x144)))+(((sj0)*(x143)))+(((cj0)*(x134)))+(((sj0)*(sj3)*(x142)))+(((cj0)*(cj3)*(x142)))+(((cj0)*(x137)*(x139)))+(((IkReal(-1.00000000000000))*(x138)*(x139)*(x140)))+(((sj0)*(x136)*(x137)))+(((IkReal(-1.00000000000000))*(x135)*(x141))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x140)*(x141)))+(((cj0)*(x143)))+(((IkReal(-1.00000000000000))*(x135)*(x137)*(x139)))+(((IkReal(-1.00000000000000))*(x134)*(x135)))+(((IkReal(-1.00000000000000))*(x136)*(x138)*(x140)))+(((IkReal(-1.00000000000000))*(cj3)*(x135)*(x142)))+(((cj0)*(x136)*(x137)))+(((cj0)*(sj3)*(x142)))+(((sj0)*(x138)*(x139)))+(((IkReal(-1.00000000000000))*(x135)*(x144))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((IkReal(-1.00000000000000))*(x135)*(x136)*(x138)))+(((cj0)*(x144)))+(((sj0)*(x143)))+(((cj0)*(x134)))+(((sj0)*(sj3)*(x142)))+(((cj0)*(cj3)*(x142)))+(((cj0)*(x137)*(x139)))+(((IkReal(-1.00000000000000))*(x138)*(x139)*(x140)))+(((sj0)*(x136)*(x137)))+(((IkReal(-1.00000000000000))*(x135)*(x141)))))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x140)*(x141)))+(((cj0)*(x143)))+(((IkReal(-1.00000000000000))*(x135)*(x137)*(x139)))+(((IkReal(-1.00000000000000))*(x134)*(x135)))+(((IkReal(-1.00000000000000))*(x136)*(x138)*(x140)))+(((IkReal(-1.00000000000000))*(cj3)*(x135)*(x142)))+(((cj0)*(x136)*(x137)))+(((cj0)*(sj3)*(x142)))+(((sj0)*(x138)*(x139)))+(((IkReal(-1.00000000000000))*(x135)*(x144)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x145=IKsin(j5);
IkReal x146=IKcos(j5);
IkReal x147=((IkReal(1.00000000000000))*(cj0));
IkReal x148=((IkReal(1.00000000000000))*(cj3));
IkReal x149=((cj0)*(cj1));
IkReal x150=((cj2)*(r02));
IkReal x151=((r00)*(sj2));
IkReal x152=((IkReal(1.00000000000000))*(sj0));
IkReal x153=((r02)*(sj1)*(sj2));
IkReal x154=((cj2)*(r00)*(sj1));
evalcond[0]=((((x149)*(x150)))+(((x149)*(x151)))+(((cj0)*(x154)))+(((IkReal(-1.00000000000000))*(sj3)*(x146)))+(((IkReal(-1.00000000000000))*(r01)*(x152)))+(((IkReal(-1.00000000000000))*(x147)*(x153)))+(((IkReal(-1.00000000000000))*(x145)*(x148))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x151)*(x152)))+(((IkReal(-1.00000000000000))*(cj1)*(x150)*(x152)))+(((sj0)*(x153)))+(((IkReal(-1.00000000000000))*(r01)*(x147)))+(((IkReal(-1.00000000000000))*(x152)*(x154)))+(((IkReal(-1.00000000000000))*(x146)*(x148)))+(((sj3)*(x145))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x155=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x155)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x155))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x156=((sj0)*(sj1));
IkReal x157=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x157)))+(((r02)*(sj2)*(x156)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x156)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x157))));
IkReal gconst49;
IkReal x158=((sj0)*(sj1));
IkReal x159=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x159)))+(((r02)*(sj2)*(x158)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x158)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x159))));
IkReal gconst50;
IkReal x160=((cj0)*(r02));
IkReal x161=((cj0)*(r00));
gconst50=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x161)))+(((cj1)*(cj2)*(x160)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x160)))+(((cj2)*(sj1)*(x161))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x162=((cj0)*(r02));
IkReal x163=((cj0)*(r00));
gconst52=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x163)))+(((cj1)*(cj2)*(x162)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x162)))+(((cj2)*(sj1)*(x163))));
IkReal gconst53;
IkReal x164=((sj0)*(sj1));
IkReal x165=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x165)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x164)))+(((r02)*(sj2)*(x164)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x165))));
IkReal gconst54;
IkReal x166=((sj0)*(sj1));
IkReal x167=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x167)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x166)))+(((r02)*(sj2)*(x166)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x167))));
IkReal gconst55;
IkReal x168=((cj0)*(r02));
IkReal x169=((cj0)*(r00));
gconst55=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x169)))+(((cj1)*(cj2)*(x168)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x168)))+(((cj2)*(sj1)*(x169))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x170=((cj0)*(r02));
IkReal x171=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x171)))+(((cj1)*(cj2)*(x170)))+(((cj2)*(sj1)*(x171)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x170))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x172=((sj0)*(sj1));
IkReal x173=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x172)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x172)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x173)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x173))));
IkReal gconst65;
IkReal x174=((sj0)*(sj1));
IkReal x175=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x174)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x174)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x175)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x175))));
IkReal gconst66;
IkReal x176=((cj0)*(r02));
IkReal x177=((cj0)*(r00));
gconst66=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x177)))+(((cj1)*(cj2)*(x176)))+(((cj2)*(sj1)*(x177)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x176))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x178=((cj0)*(r02));
IkReal x179=((cj0)*(r00));
gconst68=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x179)))+(((cj1)*(cj2)*(x178)))+(((cj2)*(sj1)*(x179)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x178))));
IkReal gconst69;
IkReal x180=((sj0)*(sj1));
IkReal x181=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x180)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x181)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x180)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x181))));
IkReal gconst70;
IkReal x182=((sj0)*(sj1));
IkReal x183=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x182)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x183)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x182)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x183))));
IkReal gconst71;
IkReal x184=((cj0)*(r02));
IkReal x185=((cj0)*(r00));
gconst71=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x184)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x185)))+(((cj1)*(cj2)*(x184)))+(((cj1)*(sj2)*(x185))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x186=((cj0)*(r02));
IkReal x187=((cj0)*(r00));
gconst73=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x186)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x187)))+(((cj1)*(cj2)*(x186)))+(((cj1)*(sj2)*(x187))));
IkReal x188=((gconst70)*(gconst71));
IkReal x189=((IkReal(1.00000000000000))*(gconst65));
IkReal x190=((gconst64)*(gconst68));
IkReal x191=((gconst69)*(gconst73));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst66)*(x188)*(x189)))+(((x188)*(x190)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x189)))+(((gconst65)*(gconst66)*(x191)))+(((IkReal(-1.00000000000000))*(x190)*(x191))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x192=((gconst62)*(gconst70));
IkReal x193=((IkReal(1.00000000000000))*(gconst49));
IkReal x194=((gconst53)*(gconst73));
IkReal x195=((gconst48)*(gconst52));
IkReal x196=((gconst65)*(gconst66));
IkReal x197=((IkReal(1.00000000000000))*(gconst65));
IkReal x198=((gconst53)*(gconst56));
IkReal x199=((gconst49)*(gconst50));
IkReal x200=((gconst55)*(gconst70));
IkReal x201=((gconst54)*(gconst62));
IkReal x202=((gconst59)*(gconst69));
IkReal x203=((gconst49)*(gconst57));
IkReal x204=((gconst52)*(gconst63));
IkReal x205=((gconst60)*(gconst73));
IkReal x206=((gconst52)*(gconst58));
IkReal x207=((gconst56)*(gconst69));
IkReal x208=((gconst54)*(gconst55));
IkReal x209=((gconst63)*(gconst65));
IkReal x210=((gconst63)*(gconst69));
IkReal x211=((gconst49)*(gconst66));
IkReal x212=((IkReal(1.00000000000000))*(gconst48));
IkReal x213=((gconst52)*(gconst64));
IkReal x214=((gconst70)*(gconst71));
IkReal x215=((gconst68)*(gconst69));
IkReal x216=((IkReal(1.00000000000000))*(gconst68));
IkReal x217=((gconst54)*(gconst71));
IkReal x218=((gconst48)*(gconst60));
IkReal x219=((gconst53)*(gconst72));
IkReal x220=((gconst53)*(gconst59));
IkReal x221=((gconst58)*(gconst61));
IkReal x222=((gconst57)*(gconst65));
IkReal x223=((gconst53)*(gconst63));
IkReal x224=((gconst61)*(gconst71));
IkReal x225=((gconst65)*(gconst67));
IkReal x226=((gconst48)*(gconst68));
IkReal x227=((gconst53)*(gconst57));
IkReal x228=((gconst60)*(gconst64));
IkReal x229=((gconst69)*(gconst72));
IkReal x230=((gconst58)*(gconst68));
IkReal x231=((gconst50)*(gconst69));
IkReal x232=((IkReal(1.00000000000000))*(gconst60));
IkReal x233=((gconst69)*(gconst73));
IkReal x234=((gconst57)*(gconst69));
IkReal x235=((gconst49)*(gconst63));
IkReal x236=((gconst50)*(gconst53));
IkReal x237=((gconst55)*(gconst61));
IkReal x238=((gconst65)*(gconst73));
IkReal x239=((gconst61)*(gconst62));
IkReal x240=((gconst64)*(gconst68));
IkReal x241=((gconst58)*(gconst60));
IkReal x242=((gconst49)*(gconst67));
IkReal x243=((gconst60)*(x208));
IkReal x244=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x245=((gconst72)*(x241));
IkReal x246=((gconst49)*(gconst51)*(gconst61));
IkReal x247=((gconst51)*(gconst61)*(gconst65));
op[0]=((((x196)*(x233)))+(((IkReal(-1.00000000000000))*(gconst67)*(x197)*(x229)))+(((IkReal(-1.00000000000000))*(x196)*(x214)))+(((x214)*(x240)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x215))));
op[1]=((((x224)*(x225)))+(((IkReal(-1.00000000000000))*(gconst59)*(x197)*(x214)))+(((x214)*(x228)))+(((IkReal(-1.00000000000000))*(x205)*(x244)))+(((x192)*(x240)))+(((x196)*(x210)))+(((gconst58)*(gconst72)*(x215)))+(((IkReal(-1.00000000000000))*(gconst64)*(x210)*(x216)))+(((x202)*(x238)))+(((IkReal(-1.00000000000000))*(x192)*(x196))));
op[2]=((((x211)*(x233)))+(((x192)*(x228)))+(((IkReal(-1.00000000000000))*(gconst59)*(x192)*(x197)))+(((IkReal(-1.00000000000000))*(gconst51)*(x197)*(x229)))+(((IkReal(-1.00000000000000))*(gconst66)*(x193)*(x214)))+(((x213)*(x214)))+(((IkReal(-1.00000000000000))*(gconst64)*(x194)*(x216)))+(((x196)*(x234)))+(((x214)*(x226)))+(((IkReal(-1.00000000000000))*(x210)*(x228)))+(((x229)*(x241)))+(((IkReal(-1.00000000000000))*(x213)*(x233)))+(((x225)*(x239)))+(((x194)*(x196)))+(((IkReal(-1.00000000000000))*(x196)*(x200)))+(((x202)*(x209)))+(((IkReal(-1.00000000000000))*(gconst71)*(x216)*(x221)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x215)))+(((IkReal(-1.00000000000000))*(x196)*(x217)))+(((IkReal(-1.00000000000000))*(gconst67)*(x197)*(x207)))+(((x231)*(x238)))+(((x217)*(x240)))+(((IkReal(-1.00000000000000))*(gconst73)*(x212)*(x215)))+(((IkReal(-1.00000000000000))*(gconst50)*(x197)*(x214)))+(((IkReal(-1.00000000000000))*(gconst67)*(x197)*(x219)))+(((x200)*(x240)))+(((IkReal(-1.00000000000000))*(gconst67)*(x193)*(x229))));
op[3]=((((IkReal(-1.00000000000000))*(gconst50)*(x192)*(x197)))+(((x192)*(x226)))+(((x201)*(x240)))+(((gconst59)*(gconst65)*(x194)))+(((x196)*(x223)))+(((x206)*(x229)))+(((x214)*(x218)))+(((x210)*(x211)))+(((IkReal(-1.00000000000000))*(gconst59)*(x197)*(x200)))+(((IkReal(-1.00000000000000))*(gconst59)*(x197)*(x217)))+(((IkReal(-1.00000000000000))*(x204)*(x244)))+(((x202)*(x222)))+(((IkReal(-1.00000000000000))*(gconst64)*(x216)*(x223)))+(((IkReal(-1.00000000000000))*(x194)*(x228)))+(((x224)*(x242)))+(((gconst49)*(gconst73)*(x202)))+(((x219)*(x230)))+(((IkReal(-1.00000000000000))*(gconst66)*(x192)*(x193)))+(((x225)*(x237)))+(((IkReal(-1.00000000000000))*(x228)*(x234)))+(((IkReal(-1.00000000000000))*(gconst71)*(x221)*(x232)))+(((IkReal(-1.00000000000000))*(x196)*(x201)))+(((x217)*(x228)))+(((IkReal(-1.00000000000000))*(gconst62)*(x216)*(x221)))+(((x200)*(x228)))+(((x209)*(x231)))+(((gconst51)*(gconst65)*(x224)))+(((x192)*(x213)))+(((IkReal(-1.00000000000000))*(gconst69)*(x205)*(x212)))+(((x207)*(x230)))+(((IkReal(-1.00000000000000))*(gconst59)*(x193)*(x214)))+(((IkReal(-1.00000000000000))*(gconst68)*(x210)*(x212))));
op[4]=((((IkReal(-1.00000000000000))*(gconst59)*(x192)*(x193)))+(((IkReal(-1.00000000000000))*(gconst67)*(x193)*(x207)))+(((IkReal(-1.00000000000000))*(gconst66)*(x193)*(x217)))+(((x196)*(x227)))+(((IkReal(-1.00000000000000))*(gconst51)*(x197)*(x207)))+(((gconst66)*(gconst69)*(x203)))+(((IkReal(-1.00000000000000))*(gconst51)*(x193)*(x229)))+(((IkReal(-1.00000000000000))*(gconst59)*(x197)*(x201)))+(((IkReal(-1.00000000000000))*(x206)*(x224)))+(((x213)*(x217)))+(((IkReal(-1.00000000000000))*(gconst51)*(x197)*(x219)))+(((IkReal(-1.00000000000000))*(gconst67)*(x193)*(x219)))+(((IkReal(-1.00000000000000))*(gconst66)*(x193)*(x200)))+(((IkReal(-1.00000000000000))*(gconst57)*(x212)*(x215)))+(((x209)*(x220)))+(((IkReal(-1.00000000000000))*(gconst64)*(x216)*(x227)))+(((x239)*(x242)))+(((IkReal(-1.00000000000000))*(x213)*(x234)))+(((x222)*(x231)))+(((x208)*(x240)))+(((IkReal(-1.00000000000000))*(gconst55)*(x216)*(x221)))+(((IkReal(-1.00000000000000))*(x196)*(x208)))+(((IkReal(-1.00000000000000))*(x195)*(x233)))+(((gconst51)*(gconst65)*(x239)))+(((IkReal(-1.00000000000000))*(gconst62)*(x221)*(x232)))+(((x217)*(x226)))+(((x207)*(x241)))+(((x202)*(x235)))+(((x200)*(x226)))+(((IkReal(-1.00000000000000))*(gconst67)*(x197)*(x198)))+(((IkReal(-1.00000000000000))*(x194)*(x213)))+(((x219)*(x241)))+(((x201)*(x228)))+(((x192)*(x218)))+(((IkReal(-1.00000000000000))*(gconst68)*(x194)*(x212)))+(((IkReal(-1.00000000000000))*(gconst50)*(x193)*(x214)))+(((IkReal(-1.00000000000000))*(gconst50)*(x197)*(x200)))+(((x194)*(x211)))+(((gconst50)*(gconst65)*(x194)))+(((x200)*(x213)))+(((IkReal(-1.00000000000000))*(x223)*(x228)))+(((IkReal(-1.00000000000000))*(gconst50)*(x197)*(x217)))+(((x199)*(x233)))+(((x195)*(x214)))+(((IkReal(-1.00000000000000))*(gconst60)*(x210)*(x212))));
op[5]=((((IkReal(-1.00000000000000))*(gconst50)*(x192)*(x193)))+(((x198)*(x230)))+(((gconst49)*(gconst59)*(x194)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x204)))+(((IkReal(-1.00000000000000))*(gconst59)*(x197)*(x208)))+(((x199)*(x210)))+(((x206)*(x219)))+(((x208)*(x228)))+(((IkReal(-1.00000000000000))*(gconst66)*(x193)*(x201)))+(((x237)*(x242)))+(((IkReal(-1.00000000000000))*(x206)*(x239)))+(((IkReal(-1.00000000000000))*(gconst60)*(x194)*(x212)))+(((x220)*(x222)))+(((x192)*(x195)))+(((IkReal(-1.00000000000000))*(gconst68)*(x212)*(x223)))+(((x202)*(x203)))+(((IkReal(-1.00000000000000))*(gconst55)*(x221)*(x232)))+(((gconst51)*(gconst65)*(x237)))+(((IkReal(-1.00000000000000))*(x227)*(x228)))+(((x209)*(x236)))+(((x217)*(x218)))+(((x201)*(x226)))+(((gconst49)*(gconst51)*(x224)))+(((IkReal(-1.00000000000000))*(gconst59)*(x193)*(x200)))+(((IkReal(-1.00000000000000))*(x195)*(x210)))+(((IkReal(-1.00000000000000))*(gconst50)*(x197)*(x201)))+(((IkReal(-1.00000000000000))*(gconst59)*(x193)*(x217)))+(((x200)*(x218)))+(((x206)*(x207)))+(((x211)*(x223)))+(((IkReal(-1.00000000000000))*(gconst60)*(x212)*(x234)))+(((x201)*(x213))));
op[6]=((((IkReal(-1.00000000000000))*(x213)*(x227)))+(((IkReal(-1.00000000000000))*(gconst67)*(x193)*(x198)))+(((IkReal(-1.00000000000000))*(gconst50)*(x193)*(x200)))+(((x208)*(x226)))+(((IkReal(-1.00000000000000))*(gconst66)*(x193)*(x208)))+(((x195)*(x200)))+(((IkReal(-1.00000000000000))*(x194)*(x195)))+(((IkReal(-1.00000000000000))*(x206)*(x237)))+(((x208)*(x213)))+(((IkReal(-1.00000000000000))*(gconst51)*(x197)*(x198)))+(((gconst53)*(gconst66)*(x203)))+(((IkReal(-1.00000000000000))*(gconst60)*(x212)*(x223)))+(((x222)*(x236)))+(((gconst49)*(gconst51)*(x239)))+(((IkReal(-1.00000000000000))*(gconst68)*(x212)*(x227)))+(((x194)*(x199)))+(((IkReal(-1.00000000000000))*(x195)*(x234)))+(((IkReal(-1.00000000000000))*(gconst51)*(x193)*(x219)))+(((x198)*(x241)))+(((IkReal(-1.00000000000000))*(gconst51)*(x193)*(x207)))+(((IkReal(-1.00000000000000))*(gconst59)*(x193)*(x201)))+(((x220)*(x235)))+(((IkReal(-1.00000000000000))*(gconst50)*(x193)*(x217)))+(((IkReal(-1.00000000000000))*(gconst50)*(x197)*(x208)))+(((x199)*(x234)))+(((x195)*(x217)))+(((x201)*(x218))));
op[7]=((((x198)*(x206)))+(((IkReal(-1.00000000000000))*(gconst50)*(x193)*(x201)))+(((x195)*(x201)))+(((x208)*(x218)))+(((x199)*(x223)))+(((x203)*(x220)))+(((IkReal(-1.00000000000000))*(x195)*(x223)))+(((IkReal(-1.00000000000000))*(gconst60)*(x212)*(x227)))+(((gconst49)*(gconst51)*(x237)))+(((IkReal(-1.00000000000000))*(gconst59)*(x193)*(x208))));
op[8]=((((IkReal(-1.00000000000000))*(gconst50)*(x193)*(x208)))+(((x195)*(x208)))+(((x199)*(x227)))+(((IkReal(-1.00000000000000))*(x195)*(x227)))+(((IkReal(-1.00000000000000))*(gconst51)*(x193)*(x198))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x248=((sj1)*(sj3));
IkReal x249=((IkReal(1.00000000000000))*(cj0));
IkReal x250=((cj2)*(r00));
IkReal x251=((cj3)*(sj0));
IkReal x252=((cj0)*(sj3));
IkReal x253=((cj3)*(sj1));
IkReal x254=((r02)*(sj2));
IkReal x255=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x256=((cj1)*(r00)*(sj2));
IkReal x257=((sj0)*(x254));
IkReal x258=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(x249)*(x250)*(x253)))+(((x248)*(x257)))+(((IkReal(-1.00000000000000))*(sj0)*(x248)*(x250)))+(((cj0)*(x253)*(x254)))+(((IkReal(-1.00000000000000))*(x255)*(x256)))+(((IkReal(-1.00000000000000))*(x255)*(x258)))+(((r01)*(x251)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x249)))+(((IkReal(-1.00000000000000))*(cj3)*(x249)*(x258)))+(((IkReal(-1.00000000000000))*(cj3)*(x249)*(x256))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((sj1)*(x251)*(x254)))+(((IkReal(-1.00000000000000))*(x251)*(x256)))+(((IkReal(-1.00000000000000))*(x251)*(x258)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x249)))+(((IkReal(-1.00000000000000))*(r01)*(x255)))+(((IkReal(-1.00000000000000))*(x248)*(x249)*(x254)))+(((x252)*(x256)))+(((x252)*(x258)))+(((cj0)*(x248)*(x250)))+(((IkReal(-1.00000000000000))*(sj1)*(x250)*(x251))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((IkReal(-1.00000000000000))*(x249)*(x250)*(x253)))+(((x248)*(x257)))+(((IkReal(-1.00000000000000))*(sj0)*(x248)*(x250)))+(((cj0)*(x253)*(x254)))+(((IkReal(-1.00000000000000))*(x255)*(x256)))+(((IkReal(-1.00000000000000))*(x255)*(x258)))+(((r01)*(x251)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x249)))+(((IkReal(-1.00000000000000))*(cj3)*(x249)*(x258)))+(((IkReal(-1.00000000000000))*(cj3)*(x249)*(x256)))))), ((gconst90)*(((((sj1)*(x251)*(x254)))+(((IkReal(-1.00000000000000))*(x251)*(x256)))+(((IkReal(-1.00000000000000))*(x251)*(x258)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x249)))+(((IkReal(-1.00000000000000))*(r01)*(x255)))+(((IkReal(-1.00000000000000))*(x248)*(x249)*(x254)))+(((x252)*(x256)))+(((x252)*(x258)))+(((cj0)*(x248)*(x250)))+(((IkReal(-1.00000000000000))*(sj1)*(x250)*(x251)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x259=IKcos(j5);
IkReal x260=IKsin(j5);
IkReal x261=((IkReal(1.00000000000000))*(cj0));
IkReal x262=((IkReal(1.00000000000000))*(sj0));
IkReal x263=((r02)*(sj1)*(sj2));
IkReal x264=((IkReal(1.00000000000000))*(x259));
IkReal x265=((cj1)*(cj2)*(r02));
IkReal x266=((cj2)*(r00)*(sj1));
IkReal x267=((cj1)*(r00)*(sj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj3)*(x264)))+(((cj0)*(x265)))+(((cj0)*(x266)))+(((cj0)*(x267)))+(((IkReal(-1.00000000000000))*(x261)*(x263)))+(((IkReal(-1.00000000000000))*(r01)*(x262)))+(((cj3)*(x260))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x262)*(x265)))+(((IkReal(-1.00000000000000))*(x262)*(x266)))+(((IkReal(-1.00000000000000))*(x262)*(x267)))+(((IkReal(-1.00000000000000))*(sj3)*(x260)))+(((IkReal(-1.00000000000000))*(cj3)*(x264)))+(((IkReal(-1.00000000000000))*(r01)*(x261)))+(((sj0)*(x263))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x268=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x269=((cj1)*(x268));
IkReal x270=((IkReal(1.00000000000000))*(r02));
IkReal x271=((sj1)*(x268));
sj5array[0]=((((IkReal(-1.00000000000000))*(r00)*(sj2)*(x271)))+(((IkReal(-1.00000000000000))*(sj2)*(x269)*(x270)))+(((cj2)*(r00)*(x269)))+(((IkReal(-1.00000000000000))*(cj2)*(x270)*(x271))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x272=((cj0)*(cj5));
IkReal x273=((IkReal(1.00000000000000))*(r01));
IkReal x274=((cj4)*(sj0)*(sj5));
IkReal x275=((cj2)*(r00)*(sj1));
IkReal x276=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x277=((cj0)*(cj4)*(sj5));
IkReal x278=((cj1)*(r00)*(sj2));
IkReal x279=((cj1)*(cj2)*(r02));
IkReal x280=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x272)*(x275)))+(((x272)*(x279)))+(((x272)*(x278)))+(((x274)*(x279)))+(((x274)*(x278)))+(((x274)*(x275)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x273)))+(((r01)*(x277)))+(((IkReal(-1.00000000000000))*(x272)*(x280)))+(((IkReal(-1.00000000000000))*(x274)*(x280))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x272)*(x273)))+(((IkReal(-1.00000000000000))*(x273)*(x274)))+(((IkReal(-1.00000000000000))*(x277)*(x280)))+(((x275)*(x277)))+(((x277)*(x279)))+(((x277)*(x278)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x275)*(x276)))+(((IkReal(-1.00000000000000))*(x276)*(x279)))+(((IkReal(-1.00000000000000))*(x276)*(x278))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x272)*(x275)))+(((x272)*(x279)))+(((x272)*(x278)))+(((x274)*(x279)))+(((x274)*(x278)))+(((x274)*(x275)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x273)))+(((r01)*(x277)))+(((IkReal(-1.00000000000000))*(x272)*(x280)))+(((IkReal(-1.00000000000000))*(x274)*(x280)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x272)*(x273)))+(((IkReal(-1.00000000000000))*(x273)*(x274)))+(((IkReal(-1.00000000000000))*(x277)*(x280)))+(((x275)*(x277)))+(((x277)*(x279)))+(((x277)*(x278)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x275)*(x276)))+(((IkReal(-1.00000000000000))*(x276)*(x279)))+(((IkReal(-1.00000000000000))*(x276)*(x278)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x281=IKsin(j3);
IkReal x282=IKcos(j3);
IkReal x283=((IkReal(1.00000000000000))*(cj0));
IkReal x284=((cj0)*(cj1));
IkReal x285=((cj2)*(r02));
IkReal x286=((r00)*(sj2));
IkReal x287=((IkReal(1.00000000000000))*(sj0));
IkReal x288=((cj4)*(sj5));
IkReal x289=((r02)*(sj1)*(sj2));
IkReal x290=((cj2)*(r00)*(sj1));
IkReal x291=((IkReal(1.00000000000000))*(x282));
evalcond[0]=((((x284)*(x286)))+(((x284)*(x285)))+(((IkReal(-1.00000000000000))*(r01)*(x287)))+(((IkReal(-1.00000000000000))*(cj5)*(x281)))+(((IkReal(-1.00000000000000))*(x288)*(x291)))+(((cj0)*(x290)))+(((IkReal(-1.00000000000000))*(x283)*(x289))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x291)))+(((IkReal(-1.00000000000000))*(cj1)*(x285)*(x287)))+(((IkReal(-1.00000000000000))*(cj1)*(x286)*(x287)))+(((IkReal(-1.00000000000000))*(r01)*(x283)))+(((IkReal(-1.00000000000000))*(x287)*(x290)))+(((sj0)*(x289)))+(((x281)*(x288))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x292=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x292)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x292))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x293=((cj0)*(cj5));
IkReal x294=((IkReal(1.00000000000000))*(r01));
IkReal x295=((cj4)*(sj0)*(sj5));
IkReal x296=((cj2)*(r00)*(sj1));
IkReal x297=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x298=((cj0)*(cj4)*(sj5));
IkReal x299=((cj1)*(r00)*(sj2));
IkReal x300=((cj1)*(cj2)*(r02));
IkReal x301=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x295)*(x301)))+(((x293)*(x300)))+(((x295)*(x300)))+(((x295)*(x296)))+(((x295)*(x299)))+(((r01)*(x298)))+(((IkReal(-1.00000000000000))*(x293)*(x301)))+(((x293)*(x296)))+(((x293)*(x299)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x294))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x298)*(x301)))+(((IkReal(-1.00000000000000))*(x297)*(x299)))+(((IkReal(-1.00000000000000))*(x296)*(x297)))+(((IkReal(-1.00000000000000))*(x297)*(x300)))+(((IkReal(-1.00000000000000))*(x293)*(x294)))+(((x298)*(x300)))+(((x298)*(x299)))+(((x296)*(x298)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x294)*(x295))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x295)*(x301)))+(((x293)*(x300)))+(((x295)*(x300)))+(((x295)*(x296)))+(((x295)*(x299)))+(((r01)*(x298)))+(((IkReal(-1.00000000000000))*(x293)*(x301)))+(((x293)*(x296)))+(((x293)*(x299)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x294)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x298)*(x301)))+(((IkReal(-1.00000000000000))*(x297)*(x299)))+(((IkReal(-1.00000000000000))*(x296)*(x297)))+(((IkReal(-1.00000000000000))*(x297)*(x300)))+(((IkReal(-1.00000000000000))*(x293)*(x294)))+(((x298)*(x300)))+(((x298)*(x299)))+(((x296)*(x298)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x294)*(x295)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x302=IKsin(j3);
IkReal x303=IKcos(j3);
IkReal x304=((IkReal(1.00000000000000))*(cj0));
IkReal x305=((cj0)*(cj1));
IkReal x306=((cj2)*(r02));
IkReal x307=((r00)*(sj2));
IkReal x308=((IkReal(1.00000000000000))*(sj0));
IkReal x309=((cj4)*(sj5));
IkReal x310=((r02)*(sj1)*(sj2));
IkReal x311=((cj2)*(r00)*(sj1));
IkReal x312=((IkReal(1.00000000000000))*(x303));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x308)))+(((IkReal(-1.00000000000000))*(x304)*(x310)))+(((x305)*(x307)))+(((x305)*(x306)))+(((cj0)*(x311)))+(((IkReal(-1.00000000000000))*(x309)*(x312)))+(((IkReal(-1.00000000000000))*(cj5)*(x302))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x307)*(x308)))+(((IkReal(-1.00000000000000))*(r01)*(x304)))+(((sj0)*(x310)))+(((IkReal(-1.00000000000000))*(cj1)*(x306)*(x308)))+(((IkReal(-1.00000000000000))*(x308)*(x311)))+(((IkReal(-1.00000000000000))*(cj5)*(x312)))+(((x302)*(x309))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
IkReal x313=((IkReal(0.331000000000000))*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.283616022500000))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.268110000000000))*(cj2))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x313)))+(pz));
evalcond[3]=((x313)+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(20000.0000000000))*((px)*(px))))+(((IkReal(20000.0000000000))*((py)*(py))))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x314=((IkReal(6620.00000000000))*(cj2));
if( IKabs(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x314)))+(((IkReal(-2003.00000000000))*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-8100.00000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x314)))+(((IkReal(-2003.00000000000))*(px))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(8100.00000000000))*(px)))+(((px)*(x314)))+(((IkReal(-2003.00000000000))*(py)))))), ((gconst2)*(((((IkReal(-8100.00000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x314)))+(((IkReal(-2003.00000000000))*(px)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x315=IKcos(j0);
IkReal x316=IKsin(j0);
IkReal x317=((px)*(x316));
IkReal x318=((py)*(x315));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x316)))+(((px)*(x315))));
evalcond[1]=((IkReal(0.405000000000000))+(((IkReal(0.331000000000000))*(cj2)))+(x318)+(((IkReal(-1.00000000000000))*(x317))));
evalcond[2]=((IkReal(-0.0444339775000000))+(((IkReal(0.810000000000000))*(x317)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x318))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x319=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x319)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x319))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x320=((sj0)*(sj1));
IkReal x321=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x321)))+(((r02)*(sj2)*(x320)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x321)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x320))));
IkReal gconst6;
IkReal x322=((sj0)*(sj1));
IkReal x323=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x323)))+(((r02)*(sj2)*(x322)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x323)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x322))));
IkReal gconst7;
IkReal x324=((cj0)*(r02));
IkReal x325=((cj0)*(r00));
gconst7=((((cj1)*(sj2)*(x325)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x324)))+(((cj2)*(sj1)*(x325)))+(((cj1)*(cj2)*(x324))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x326=((cj0)*(r02));
IkReal x327=((cj0)*(r00));
gconst9=((((cj1)*(sj2)*(x327)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x326)))+(((cj2)*(sj1)*(x327)))+(((cj1)*(cj2)*(x326))));
IkReal gconst10;
IkReal x328=((sj0)*(sj1));
IkReal x329=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x329)))+(((r02)*(sj2)*(x328)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x329)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x328))));
IkReal gconst11;
IkReal x330=((sj0)*(sj1));
IkReal x331=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x331)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x330)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x331)))+(((r02)*(sj2)*(x330))));
IkReal gconst12;
IkReal x332=((cj0)*(r02));
IkReal x333=((cj0)*(r00));
gconst12=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x333)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x332)))+(((cj1)*(sj2)*(x333)))+(((cj1)*(cj2)*(x332))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x334=((cj0)*(r02));
IkReal x335=((cj0)*(r00));
gconst14=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x335)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x334)))+(((cj1)*(sj2)*(x335)))+(((cj1)*(cj2)*(x334))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x336=((sj0)*(sj1));
IkReal x337=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x337)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x336)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x337)))+(((r02)*(sj2)*(x336))));
IkReal gconst22;
IkReal x338=((sj0)*(sj1));
IkReal x339=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x339)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x338)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x339)))+(((r02)*(sj2)*(x338))));
IkReal gconst23;
IkReal x340=((cj0)*(r02));
IkReal x341=((cj0)*(r00));
gconst23=((((cj1)*(cj2)*(x340)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x340)))+(((cj1)*(sj2)*(x341)))+(((cj2)*(sj1)*(x341))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x342=((cj0)*(r02));
IkReal x343=((cj0)*(r00));
gconst25=((((cj1)*(cj2)*(x342)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x342)))+(((cj1)*(sj2)*(x343)))+(((cj2)*(sj1)*(x343))));
IkReal gconst26;
IkReal x344=((sj0)*(sj1));
IkReal x345=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x345)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x345)))+(((r02)*(sj2)*(x344)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x344))));
IkReal gconst27;
IkReal x346=((sj0)*(sj1));
IkReal x347=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x347)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x347)))+(((r02)*(sj2)*(x346)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x346))));
IkReal gconst28;
IkReal x348=((cj0)*(r02));
IkReal x349=((cj0)*(r00));
gconst28=((((cj1)*(cj2)*(x348)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x348)))+(((cj1)*(sj2)*(x349)))+(((cj2)*(sj1)*(x349))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x350=((cj0)*(r02));
IkReal x351=((cj0)*(r00));
gconst30=((((cj1)*(sj2)*(x351)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x350)))+(((cj2)*(sj1)*(x351)))+(((cj1)*(cj2)*(x350))));
IkReal x352=((gconst27)*(gconst28));
IkReal x353=((gconst22)*(gconst23));
IkReal x354=((gconst21)*(gconst25));
IkReal x355=((gconst26)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((IkReal(-1.00000000000000))*(x352)*(x353)))+(((x352)*(x354)))+(((x353)*(x355)))+(((IkReal(-1.00000000000000))*(x354)*(x355))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x356=((IkReal(1.00000000000000))*(gconst19));
IkReal x357=((gconst23)*(gconst27));
IkReal x358=((gconst15)*(gconst18));
IkReal x359=((gconst11)*(gconst6));
IkReal x360=((gconst13)*(gconst26));
IkReal x361=((gconst15)*(gconst9));
IkReal x362=((gconst6)*(gconst8));
IkReal x363=((gconst18)*(gconst28));
IkReal x364=((gconst22)*(gconst30));
IkReal x365=((gconst16)*(gconst26));
IkReal x366=((IkReal(1.00000000000000))*(gconst10));
IkReal x367=((gconst14)*(gconst25));
IkReal x368=((IkReal(1.00000000000000))*(gconst26));
IkReal x369=((gconst12)*(gconst27));
IkReal x370=((gconst25)*(gconst5));
IkReal x371=((gconst17)*(gconst28));
IkReal x372=((gconst21)*(gconst27));
IkReal x373=((gconst27)*(gconst6));
IkReal x374=((IkReal(1.00000000000000))*(gconst28));
IkReal x375=((gconst11)*(gconst5));
IkReal x376=((gconst17)*(gconst21));
IkReal x377=((gconst18)*(gconst19));
IkReal x378=((gconst10)*(gconst7));
IkReal x379=((gconst30)*(gconst6));
IkReal x380=((gconst17)*(gconst5));
IkReal x381=((IkReal(1.00000000000000))*(gconst12));
IkReal x382=((gconst22)*(gconst7));
IkReal x383=((gconst24)*(gconst6));
IkReal x384=((gconst5)*(gconst9));
IkReal x385=((gconst15)*(gconst17));
IkReal x386=((gconst27)*(gconst28));
IkReal x387=((gconst23)*(gconst26));
IkReal x388=((gconst21)*(gconst9));
IkReal x389=((gconst22)*(gconst8));
IkReal x390=((gconst10)*(gconst23));
IkReal x391=((gconst12)*(gconst18));
IkReal x392=((gconst21)*(gconst25));
IkReal x393=((gconst11)*(gconst28));
IkReal x394=((gconst11)*(gconst19));
IkReal x395=((gconst19)*(gconst27));
IkReal x396=((gconst20)*(gconst6));
IkReal x397=((gconst10)*(gconst16));
IkReal x398=((gconst26)*(gconst29));
IkReal x399=((gconst26)*(gconst7));
IkReal x400=((gconst20)*(gconst22));
IkReal x401=((gconst22)*(gconst24));
IkReal x402=((gconst15)*(gconst25));
IkReal x403=((gconst10)*(gconst13));
IkReal x404=((gconst14)*(gconst6));
IkReal x405=((gconst16)*(gconst22));
IkReal x406=((gconst11)*(gconst12));
IkReal x407=((gconst14)*(gconst22));
IkReal x408=((gconst17)*(gconst19));
IkReal x409=((gconst10)*(gconst29));
IkReal x410=((gconst28)*(gconst9));
IkReal x411=((IkReal(1.00000000000000))*(gconst6));
IkReal x412=((gconst26)*(x396));
IkReal x413=((gconst9)*(x394));
IkReal x414=((gconst11)*(x405));
IkReal x415=((gconst14)*(x390));
IkReal x416=((gconst11)*(gconst22)*(gconst23));
op[0]=((((IkReal(-1.00000000000000))*(gconst22)*(x357)*(x374)))+(((IkReal(-1.00000000000000))*(gconst29)*(x368)*(x401)))+(((IkReal(-1.00000000000000))*(gconst30)*(x368)*(x392)))+(((gconst25)*(gconst28)*(x372)))+(((x364)*(x387))));
op[1]=((((IkReal(-1.00000000000000))*(gconst30)*(x368)*(x376)))+(((x364)*(x365)))+(((x363)*(x401)))+(((x387)*(x400)))+(((gconst19)*(gconst25)*(x372)))+(((IkReal(-1.00000000000000))*(gconst27)*(x374)*(x405)))+(((IkReal(-1.00000000000000))*(gconst20)*(x368)*(x392)))+(((IkReal(-1.00000000000000))*(gconst22)*(x356)*(x357)))+(((x398)*(x402)))+(((x371)*(x372))));
op[2]=((((x377)*(x401)))+(((IkReal(-1.00000000000000))*(gconst27)*(x374)*(x382)))+(((x372)*(x408)))+(((x385)*(x398)))+(((IkReal(-1.00000000000000))*(gconst30)*(x368)*(x370)))+(((x372)*(x410)))+(((IkReal(-1.00000000000000))*(x374)*(x416)))+(((IkReal(-1.00000000000000))*(gconst29)*(x366)*(x401)))+(((x370)*(x386)))+(((x369)*(x392)))+(((x387)*(x407)))+(((x364)*(x399)))+(((x364)*(x390)))+(((IkReal(-1.00000000000000))*(gconst25)*(x358)*(x374)))+(((x365)*(x400)))+(((IkReal(-1.00000000000000))*(gconst22)*(x357)*(x381)))+(((x379)*(x387)))+(((IkReal(-1.00000000000000))*(gconst30)*(x368)*(x388)))+(((IkReal(-1.00000000000000))*(gconst29)*(x368)*(x389)))+(((IkReal(-1.00000000000000))*(gconst29)*(x368)*(x383)))+(((IkReal(-1.00000000000000))*(gconst20)*(x368)*(x376)))+(((IkReal(-1.00000000000000))*(gconst30)*(x366)*(x392)))+(((IkReal(-1.00000000000000))*(gconst27)*(x356)*(x405)))+(((IkReal(-1.00000000000000))*(gconst21)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(gconst6)*(x357)*(x374)))+(((IkReal(-1.00000000000000))*(x360)*(x401)))+(((x392)*(x393))));
op[3]=((((IkReal(-1.00000000000000))*(gconst20)*(x366)*(x392)))+(((IkReal(-1.00000000000000))*(x356)*(x416)))+(((gconst19)*(gconst9)*(x372)))+(((x390)*(x400)))+(((x369)*(x376)))+(((IkReal(-1.00000000000000))*(x374)*(x414)))+(((IkReal(-1.00000000000000))*(gconst27)*(x356)*(x382)))+(((IkReal(-1.00000000000000))*(x369)*(x405)))+(((IkReal(-1.00000000000000))*(x358)*(x371)))+(((x363)*(x383)))+(((x363)*(x389)))+(((x365)*(x379)))+(((x360)*(x402)))+(((x364)*(x397)))+(((x365)*(x407)))+(((IkReal(-1.00000000000000))*(gconst25)*(x356)*(x358)))+(((gconst27)*(gconst5)*(x371)))+(((IkReal(-1.00000000000000))*(gconst6)*(x356)*(x357)))+(((IkReal(-1.00000000000000))*(gconst14)*(x368)*(x376)))+(((IkReal(-1.00000000000000))*(gconst30)*(x368)*(x380)))+(((x387)*(x396)))+(((gconst20)*(gconst26)*(x382)))+(((IkReal(-1.00000000000000))*(gconst16)*(x373)*(x374)))+(((IkReal(-1.00000000000000))*(gconst20)*(x368)*(x370)))+(((x361)*(x398)))+(((gconst11)*(gconst21)*(x371)))+(((x370)*(x395)))+(((x402)*(x409)))+(((IkReal(-1.00000000000000))*(gconst20)*(x368)*(x388)))+(((x391)*(x401)))+(((x392)*(x394)))+(((IkReal(-1.00000000000000))*(gconst30)*(x366)*(x376))));
op[4]=((((x397)*(x400)))+(((IkReal(-1.00000000000000))*(x356)*(x414)))+(((x377)*(x383)))+(((x377)*(x389)))+(((IkReal(-1.00000000000000))*(gconst25)*(x358)*(x381)))+(((IkReal(-1.00000000000000))*(gconst6)*(x357)*(x381)))+(((x384)*(x386)))+(((x379)*(x390)))+(((x379)*(x399)))+(((x390)*(x407)))+(((IkReal(-1.00000000000000))*(gconst30)*(x366)*(x388)))+(((IkReal(-1.00000000000000))*(x381)*(x416)))+(((IkReal(-1.00000000000000))*(gconst13)*(x366)*(x401)))+(((IkReal(-1.00000000000000))*(gconst20)*(x366)*(x376)))+(((x369)*(x370)))+(((IkReal(-1.00000000000000))*(x369)*(x382)))+(((x392)*(x406)))+(((x365)*(x396)))+(((x380)*(x395)))+(((IkReal(-1.00000000000000))*(gconst9)*(x358)*(x374)))+(((x364)*(x378)))+(((x387)*(x404)))+(((IkReal(-1.00000000000000))*(gconst16)*(x356)*(x373)))+(((IkReal(-1.00000000000000))*(gconst17)*(x356)*(x358)))+(((IkReal(-1.00000000000000))*(gconst14)*(x368)*(x388)))+(((x360)*(x385)))+(((IkReal(-1.00000000000000))*(gconst11)*(x374)*(x382)))+(((x376)*(x394)))+(((IkReal(-1.00000000000000))*(gconst7)*(x373)*(x374)))+(((IkReal(-1.00000000000000))*(gconst5)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(gconst30)*(x368)*(x384)))+(((IkReal(-1.00000000000000))*(gconst29)*(x362)*(x368)))+(((x385)*(x409)))+(((IkReal(-1.00000000000000))*(gconst23)*(x359)*(x374)))+(((gconst14)*(gconst26)*(x382)))+(((x370)*(x393)))+(((x369)*(x388)))+(((IkReal(-1.00000000000000))*(gconst21)*(x366)*(x367)))+(((IkReal(-1.00000000000000))*(gconst20)*(x368)*(x380)))+(((x388)*(x393)))+(((IkReal(-1.00000000000000))*(gconst29)*(x366)*(x383)))+(((IkReal(-1.00000000000000))*(gconst29)*(x366)*(x389)))+(((IkReal(-1.00000000000000))*(x360)*(x383)))+(((IkReal(-1.00000000000000))*(x360)*(x389)))+(((IkReal(-1.00000000000000))*(gconst30)*(x366)*(x370))));
op[5]=((((x376)*(x406)))+(((x397)*(x407)))+(((IkReal(-1.00000000000000))*(gconst11)*(x356)*(x382)))+(((IkReal(-1.00000000000000))*(gconst9)*(x356)*(x358)))+(((x379)*(x397)))+(((IkReal(-1.00000000000000))*(gconst30)*(x366)*(x380)))+(((IkReal(-1.00000000000000))*(gconst23)*(x356)*(x359)))+(((IkReal(-1.00000000000000))*(x381)*(x414)))+(((IkReal(-1.00000000000000))*(gconst20)*(x366)*(x370)))+(((x361)*(x409)))+(((IkReal(-1.00000000000000))*(gconst16)*(x359)*(x374)))+(((IkReal(-1.00000000000000))*(gconst14)*(x368)*(x380)))+(((IkReal(-1.00000000000000))*(gconst20)*(x366)*(x388)))+(((IkReal(-1.00000000000000))*(gconst16)*(x369)*(x411)))+(((x396)*(x399)))+(((IkReal(-1.00000000000000))*(gconst7)*(x356)*(x373)))+(((x365)*(x404)))+(((IkReal(-1.00000000000000))*(gconst14)*(x366)*(x376)))+(((x384)*(x395)))+(((IkReal(-1.00000000000000))*(gconst17)*(x358)*(x381)))+(((x389)*(x391)))+(((x360)*(x361)))+(((x390)*(x396)))+(((x370)*(x394)))+(((x369)*(x380)))+(((x402)*(x403)))+(((x362)*(x363)))+(((IkReal(-1.00000000000000))*(gconst20)*(x368)*(x384)))+(((x383)*(x391)))+(((x388)*(x394)))+(((x378)*(x400)))+(((x371)*(x375))));
op[6]=((((IkReal(-1.00000000000000))*(gconst14)*(x366)*(x388)))+(((IkReal(-1.00000000000000))*(gconst7)*(x369)*(x411)))+(((IkReal(-1.00000000000000))*(gconst13)*(x366)*(x383)))+(((IkReal(-1.00000000000000))*(gconst13)*(x366)*(x389)))+(((x390)*(x404)))+(((IkReal(-1.00000000000000))*(gconst30)*(x366)*(x384)))+(((IkReal(-1.00000000000000))*(gconst16)*(x356)*(x359)))+(((x370)*(x406)))+(((IkReal(-1.00000000000000))*(gconst14)*(x368)*(x384)))+(((IkReal(-1.00000000000000))*(gconst7)*(x359)*(x374)))+(((IkReal(-1.00000000000000))*(gconst20)*(x366)*(x380)))+(((x396)*(x397)))+(((IkReal(-1.00000000000000))*(gconst9)*(x358)*(x381)))+(((x388)*(x406)))+(((x378)*(x379)))+(((IkReal(-1.00000000000000))*(x360)*(x362)))+(((IkReal(-1.00000000000000))*(gconst11)*(x381)*(x382)))+(((IkReal(-1.00000000000000))*(gconst29)*(x362)*(x366)))+(((IkReal(-1.00000000000000))*(gconst23)*(x359)*(x381)))+(((x399)*(x404)))+(((x375)*(x408)))+(((x385)*(x403)))+(((x375)*(x410)))+(((x369)*(x384)))+(((x362)*(x377)))+(((IkReal(-1.00000000000000))*(gconst5)*(x366)*(x367)))+(((x378)*(x407))));
op[7]=((((x397)*(x404)))+(((gconst19)*(gconst9)*(x375)))+(((IkReal(-1.00000000000000))*(gconst14)*(x366)*(x380)))+(((x361)*(x403)))+(((IkReal(-1.00000000000000))*(gconst20)*(x366)*(x384)))+(((x378)*(x396)))+(((IkReal(-1.00000000000000))*(gconst16)*(x359)*(x381)))+(((gconst12)*(gconst17)*(x375)))+(((IkReal(-1.00000000000000))*(gconst7)*(x356)*(x359)))+(((x362)*(x391))));
op[8]=((((IkReal(-1.00000000000000))*(gconst14)*(x366)*(x384)))+(((gconst12)*(gconst9)*(x375)))+(((IkReal(-1.00000000000000))*(gconst7)*(x359)*(x381)))+(((IkReal(-1.00000000000000))*(gconst13)*(x362)*(x366)))+(((x378)*(x404))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x417=((r01)*(sj3));
IkReal x418=((IkReal(1.00000000000000))*(sj0));
IkReal x419=((sj1)*(sj3));
IkReal x420=((cj2)*(r00));
IkReal x421=((r02)*(sj2));
IkReal x422=((cj3)*(sj1));
IkReal x423=((IkReal(1.00000000000000))*(cj0));
IkReal x424=((cj3)*(r01));
IkReal x425=((cj1)*(r00)*(sj2));
IkReal x426=((cj1)*(cj2)*(r02)*(sj3));
IkReal x427=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((cj0)*(x417)))+(((IkReal(-1.00000000000000))*(x418)*(x419)*(x421)))+(((cj0)*(cj3)*(x425)))+(((cj0)*(x427)))+(((sj0)*(x419)*(x420)))+(((IkReal(-1.00000000000000))*(x421)*(x422)*(x423)))+(((cj0)*(x420)*(x422)))+(((sj0)*(sj3)*(x425)))+(((IkReal(-1.00000000000000))*(x418)*(x424)))+(((sj0)*(x426))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x417)*(x418)))+(((IkReal(-1.00000000000000))*(cj3)*(x418)*(x425)))+(((cj0)*(x419)*(x420)))+(((cj0)*(sj3)*(x425)))+(((cj0)*(x426)))+(((IkReal(-1.00000000000000))*(x418)*(x420)*(x422)))+(((sj0)*(x421)*(x422)))+(((IkReal(-1.00000000000000))*(x418)*(x427)))+(((IkReal(-1.00000000000000))*(x423)*(x424)))+(((IkReal(-1.00000000000000))*(x419)*(x421)*(x423))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((cj0)*(x417)))+(((IkReal(-1.00000000000000))*(x418)*(x419)*(x421)))+(((cj0)*(cj3)*(x425)))+(((cj0)*(x427)))+(((sj0)*(x419)*(x420)))+(((IkReal(-1.00000000000000))*(x421)*(x422)*(x423)))+(((cj0)*(x420)*(x422)))+(((sj0)*(sj3)*(x425)))+(((IkReal(-1.00000000000000))*(x418)*(x424)))+(((sj0)*(x426)))))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x417)*(x418)))+(((IkReal(-1.00000000000000))*(cj3)*(x418)*(x425)))+(((cj0)*(x419)*(x420)))+(((cj0)*(sj3)*(x425)))+(((cj0)*(x426)))+(((IkReal(-1.00000000000000))*(x418)*(x420)*(x422)))+(((sj0)*(x421)*(x422)))+(((IkReal(-1.00000000000000))*(x418)*(x427)))+(((IkReal(-1.00000000000000))*(x423)*(x424)))+(((IkReal(-1.00000000000000))*(x419)*(x421)*(x423)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x428=IKsin(j5);
IkReal x429=IKcos(j5);
IkReal x430=((IkReal(1.00000000000000))*(cj0));
IkReal x431=((IkReal(1.00000000000000))*(cj3));
IkReal x432=((cj0)*(cj1));
IkReal x433=((cj2)*(r02));
IkReal x434=((r00)*(sj2));
IkReal x435=((IkReal(1.00000000000000))*(sj0));
IkReal x436=((r02)*(sj1)*(sj2));
IkReal x437=((cj2)*(r00)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(x430)*(x436)))+(((cj0)*(x437)))+(((IkReal(-1.00000000000000))*(sj3)*(x429)))+(((IkReal(-1.00000000000000))*(r01)*(x435)))+(((IkReal(-1.00000000000000))*(x428)*(x431)))+(((x432)*(x434)))+(((x432)*(x433))));
evalcond[1]=((((sj0)*(x436)))+(((IkReal(-1.00000000000000))*(x435)*(x437)))+(((IkReal(-1.00000000000000))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(cj1)*(x433)*(x435)))+(((IkReal(-1.00000000000000))*(cj1)*(x434)*(x435)))+(((sj3)*(x428)))+(((IkReal(-1.00000000000000))*(r01)*(x430))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x438=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x438)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x438))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x439=((sj0)*(sj1));
IkReal x440=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x440)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x439)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x440)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x439))));
IkReal gconst49;
IkReal x441=((sj0)*(sj1));
IkReal x442=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x442)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x441)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x442)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x441))));
IkReal gconst50;
IkReal x443=((cj0)*(r02));
IkReal x444=((cj0)*(r00));
gconst50=((((cj1)*(cj2)*(x443)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x443)))+(((cj2)*(sj1)*(x444)))+(((cj1)*(sj2)*(x444))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x445=((cj0)*(r02));
IkReal x446=((cj0)*(r00));
gconst52=((((cj1)*(cj2)*(x445)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x445)))+(((cj2)*(sj1)*(x446)))+(((cj1)*(sj2)*(x446))));
IkReal gconst53;
IkReal x447=((sj0)*(sj1));
IkReal x448=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x448)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x447)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x448)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x447))));
IkReal gconst54;
IkReal x449=((sj0)*(sj1));
IkReal x450=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x450)))+(((r02)*(sj2)*(x449)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x450)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x449))));
IkReal gconst55;
IkReal x451=((cj0)*(r02));
IkReal x452=((cj0)*(r00));
gconst55=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x451)))+(((cj1)*(cj2)*(x451)))+(((cj1)*(sj2)*(x452)))+(((cj2)*(sj1)*(x452))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x453=((cj0)*(r02));
IkReal x454=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x453)))+(((cj1)*(cj2)*(x453)))+(((cj1)*(sj2)*(x454)))+(((cj2)*(sj1)*(x454))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x455=((sj0)*(sj1));
IkReal x456=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x455)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x455)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x456)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x456))));
IkReal gconst65;
IkReal x457=((sj0)*(sj1));
IkReal x458=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x457)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x457)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x458)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x458))));
IkReal gconst66;
IkReal x459=((cj0)*(r02));
IkReal x460=((cj0)*(r00));
gconst66=((((cj2)*(sj1)*(x460)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x459)))+(((cj1)*(cj2)*(x459)))+(((cj1)*(sj2)*(x460))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x461=((cj0)*(r02));
IkReal x462=((cj0)*(r00));
gconst68=((((cj2)*(sj1)*(x462)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x461)))+(((cj1)*(cj2)*(x461)))+(((cj1)*(sj2)*(x462))));
IkReal gconst69;
IkReal x463=((sj0)*(sj1));
IkReal x464=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x464)))+(((r02)*(sj2)*(x463)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x463)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x464))));
IkReal gconst70;
IkReal x465=((sj0)*(sj1));
IkReal x466=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x466)))+(((r02)*(sj2)*(x465)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x465)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x466))));
IkReal gconst71;
IkReal x467=((cj0)*(r02));
IkReal x468=((cj0)*(r00));
gconst71=((((cj2)*(sj1)*(x468)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x467)))+(((cj1)*(cj2)*(x467)))+(((cj1)*(sj2)*(x468))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x469=((cj0)*(r02));
IkReal x470=((cj0)*(r00));
gconst73=((((cj1)*(sj2)*(x470)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x470)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x469)))+(((cj1)*(cj2)*(x469))));
IkReal x471=((gconst70)*(gconst71));
IkReal x472=((IkReal(1.00000000000000))*(gconst65));
IkReal x473=((gconst64)*(gconst68));
IkReal x474=((gconst69)*(gconst73));
dummyeval[0]=((((x471)*(x473)))+(((IkReal(-1.00000000000000))*(gconst66)*(x471)*(x472)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x472)))+(((gconst65)*(gconst66)*(x474)))+(((IkReal(-1.00000000000000))*(x473)*(x474))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x475=((gconst62)*(gconst70));
IkReal x476=((IkReal(1.00000000000000))*(gconst49));
IkReal x477=((gconst53)*(gconst73));
IkReal x478=((gconst48)*(gconst52));
IkReal x479=((gconst65)*(gconst66));
IkReal x480=((IkReal(1.00000000000000))*(gconst65));
IkReal x481=((gconst53)*(gconst56));
IkReal x482=((gconst49)*(gconst50));
IkReal x483=((gconst55)*(gconst70));
IkReal x484=((gconst54)*(gconst62));
IkReal x485=((gconst59)*(gconst69));
IkReal x486=((gconst49)*(gconst57));
IkReal x487=((gconst52)*(gconst63));
IkReal x488=((gconst60)*(gconst73));
IkReal x489=((gconst52)*(gconst58));
IkReal x490=((gconst56)*(gconst69));
IkReal x491=((gconst54)*(gconst55));
IkReal x492=((gconst63)*(gconst65));
IkReal x493=((gconst63)*(gconst69));
IkReal x494=((gconst49)*(gconst66));
IkReal x495=((IkReal(1.00000000000000))*(gconst48));
IkReal x496=((gconst52)*(gconst64));
IkReal x497=((gconst70)*(gconst71));
IkReal x498=((gconst68)*(gconst69));
IkReal x499=((IkReal(1.00000000000000))*(gconst68));
IkReal x500=((gconst54)*(gconst71));
IkReal x501=((gconst48)*(gconst60));
IkReal x502=((gconst53)*(gconst72));
IkReal x503=((gconst53)*(gconst59));
IkReal x504=((gconst58)*(gconst61));
IkReal x505=((gconst57)*(gconst65));
IkReal x506=((gconst53)*(gconst63));
IkReal x507=((gconst61)*(gconst71));
IkReal x508=((gconst65)*(gconst67));
IkReal x509=((gconst48)*(gconst68));
IkReal x510=((gconst53)*(gconst57));
IkReal x511=((gconst60)*(gconst64));
IkReal x512=((gconst69)*(gconst72));
IkReal x513=((gconst58)*(gconst68));
IkReal x514=((gconst50)*(gconst69));
IkReal x515=((IkReal(1.00000000000000))*(gconst60));
IkReal x516=((gconst69)*(gconst73));
IkReal x517=((gconst57)*(gconst69));
IkReal x518=((gconst49)*(gconst63));
IkReal x519=((gconst50)*(gconst53));
IkReal x520=((gconst55)*(gconst61));
IkReal x521=((gconst65)*(gconst73));
IkReal x522=((gconst61)*(gconst62));
IkReal x523=((gconst64)*(gconst68));
IkReal x524=((gconst58)*(gconst60));
IkReal x525=((gconst49)*(gconst67));
IkReal x526=((gconst60)*(x491));
IkReal x527=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x528=((gconst72)*(x524));
IkReal x529=((gconst49)*(gconst51)*(gconst61));
IkReal x530=((gconst51)*(gconst61)*(gconst65));
op[0]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x498)))+(((x479)*(x516)))+(((x497)*(x523)))+(((IkReal(-1.00000000000000))*(gconst67)*(x480)*(x512)))+(((IkReal(-1.00000000000000))*(x479)*(x497))));
op[1]=((((IkReal(-1.00000000000000))*(x475)*(x479)))+(((x485)*(x521)))+(((IkReal(-1.00000000000000))*(x488)*(x527)))+(((x507)*(x508)))+(((x479)*(x493)))+(((x497)*(x511)))+(((IkReal(-1.00000000000000))*(gconst59)*(x480)*(x497)))+(((IkReal(-1.00000000000000))*(gconst64)*(x493)*(x499)))+(((gconst58)*(gconst72)*(x498)))+(((x475)*(x523))));
op[2]=((((IkReal(-1.00000000000000))*(gconst51)*(x480)*(x512)))+(((x497)*(x509)))+(((IkReal(-1.00000000000000))*(gconst64)*(x477)*(x499)))+(((x500)*(x523)))+(((x496)*(x497)))+(((x479)*(x517)))+(((IkReal(-1.00000000000000))*(gconst67)*(x480)*(x502)))+(((IkReal(-1.00000000000000))*(gconst67)*(x476)*(x512)))+(((IkReal(-1.00000000000000))*(gconst59)*(x475)*(x480)))+(((IkReal(-1.00000000000000))*(gconst50)*(x480)*(x497)))+(((x508)*(x522)))+(((x512)*(x524)))+(((x475)*(x511)))+(((x514)*(x521)))+(((IkReal(-1.00000000000000))*(x496)*(x516)))+(((x494)*(x516)))+(((IkReal(-1.00000000000000))*(gconst66)*(x476)*(x497)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x498)))+(((IkReal(-1.00000000000000))*(x493)*(x511)))+(((x477)*(x479)))+(((IkReal(-1.00000000000000))*(x479)*(x483)))+(((IkReal(-1.00000000000000))*(gconst71)*(x499)*(x504)))+(((x485)*(x492)))+(((IkReal(-1.00000000000000))*(gconst73)*(x495)*(x498)))+(((x483)*(x523)))+(((IkReal(-1.00000000000000))*(gconst67)*(x480)*(x490)))+(((IkReal(-1.00000000000000))*(x479)*(x500))));
op[3]=((((IkReal(-1.00000000000000))*(x487)*(x527)))+(((x475)*(x496)))+(((x484)*(x523)))+(((x497)*(x501)))+(((x502)*(x513)))+(((IkReal(-1.00000000000000))*(gconst69)*(x488)*(x495)))+(((x492)*(x514)))+(((IkReal(-1.00000000000000))*(gconst64)*(x499)*(x506)))+(((IkReal(-1.00000000000000))*(x477)*(x511)))+(((x475)*(x509)))+(((x483)*(x511)))+(((x493)*(x494)))+(((IkReal(-1.00000000000000))*(gconst68)*(x493)*(x495)))+(((IkReal(-1.00000000000000))*(gconst59)*(x476)*(x497)))+(((gconst49)*(gconst73)*(x485)))+(((IkReal(-1.00000000000000))*(gconst59)*(x480)*(x483)))+(((IkReal(-1.00000000000000))*(gconst62)*(x499)*(x504)))+(((gconst51)*(gconst65)*(x507)))+(((x508)*(x520)))+(((x485)*(x505)))+(((x507)*(x525)))+(((x500)*(x511)))+(((IkReal(-1.00000000000000))*(gconst66)*(x475)*(x476)))+(((x489)*(x512)))+(((IkReal(-1.00000000000000))*(gconst50)*(x475)*(x480)))+(((gconst59)*(gconst65)*(x477)))+(((IkReal(-1.00000000000000))*(x479)*(x484)))+(((x479)*(x506)))+(((IkReal(-1.00000000000000))*(gconst59)*(x480)*(x500)))+(((x490)*(x513)))+(((IkReal(-1.00000000000000))*(gconst71)*(x504)*(x515)))+(((IkReal(-1.00000000000000))*(x511)*(x517))));
op[4]=((((IkReal(-1.00000000000000))*(x477)*(x496)))+(((x522)*(x525)))+(((gconst66)*(gconst69)*(x486)))+(((x500)*(x509)))+(((IkReal(-1.00000000000000))*(gconst55)*(x499)*(x504)))+(((IkReal(-1.00000000000000))*(gconst50)*(x480)*(x483)))+(((x475)*(x501)))+(((IkReal(-1.00000000000000))*(gconst66)*(x476)*(x500)))+(((IkReal(-1.00000000000000))*(gconst67)*(x476)*(x490)))+(((IkReal(-1.00000000000000))*(gconst57)*(x495)*(x498)))+(((x483)*(x496)))+(((x479)*(x510)))+(((IkReal(-1.00000000000000))*(gconst59)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(gconst59)*(x480)*(x484)))+(((x491)*(x523)))+(((IkReal(-1.00000000000000))*(gconst50)*(x480)*(x500)))+(((IkReal(-1.00000000000000))*(x489)*(x507)))+(((x490)*(x524)))+(((x496)*(x500)))+(((x492)*(x503)))+(((x502)*(x524)))+(((IkReal(-1.00000000000000))*(gconst51)*(x476)*(x512)))+(((IkReal(-1.00000000000000))*(gconst62)*(x504)*(x515)))+(((IkReal(-1.00000000000000))*(x478)*(x516)))+(((IkReal(-1.00000000000000))*(x506)*(x511)))+(((IkReal(-1.00000000000000))*(gconst60)*(x493)*(x495)))+(((IkReal(-1.00000000000000))*(gconst51)*(x480)*(x490)))+(((IkReal(-1.00000000000000))*(gconst68)*(x477)*(x495)))+(((IkReal(-1.00000000000000))*(x496)*(x517)))+(((IkReal(-1.00000000000000))*(gconst50)*(x476)*(x497)))+(((IkReal(-1.00000000000000))*(gconst51)*(x480)*(x502)))+(((x477)*(x494)))+(((IkReal(-1.00000000000000))*(gconst67)*(x480)*(x481)))+(((IkReal(-1.00000000000000))*(gconst67)*(x476)*(x502)))+(((x478)*(x497)))+(((IkReal(-1.00000000000000))*(gconst64)*(x499)*(x510)))+(((IkReal(-1.00000000000000))*(gconst66)*(x476)*(x483)))+(((x505)*(x514)))+(((gconst50)*(gconst65)*(x477)))+(((x483)*(x509)))+(((x482)*(x516)))+(((gconst51)*(gconst65)*(x522)))+(((x485)*(x518)))+(((IkReal(-1.00000000000000))*(x479)*(x491)))+(((x484)*(x511))));
op[5]=((((x489)*(x502)))+(((x485)*(x486)))+(((x491)*(x511)))+(((x500)*(x501)))+(((IkReal(-1.00000000000000))*(x510)*(x511)))+(((x481)*(x513)))+(((gconst49)*(gconst51)*(x507)))+(((x494)*(x506)))+(((IkReal(-1.00000000000000))*(gconst59)*(x476)*(x483)))+(((IkReal(-1.00000000000000))*(gconst50)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(gconst50)*(x480)*(x484)))+(((x492)*(x519)))+(((x489)*(x490)))+(((IkReal(-1.00000000000000))*(gconst59)*(x476)*(x500)))+(((IkReal(-1.00000000000000))*(gconst60)*(x477)*(x495)))+(((x484)*(x509)))+(((gconst49)*(gconst59)*(x477)))+(((x520)*(x525)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x487)))+(((IkReal(-1.00000000000000))*(x478)*(x493)))+(((IkReal(-1.00000000000000))*(gconst59)*(x480)*(x491)))+(((x503)*(x505)))+(((IkReal(-1.00000000000000))*(gconst55)*(x504)*(x515)))+(((IkReal(-1.00000000000000))*(x489)*(x522)))+(((IkReal(-1.00000000000000))*(gconst66)*(x476)*(x484)))+(((x484)*(x496)))+(((x475)*(x478)))+(((IkReal(-1.00000000000000))*(gconst68)*(x495)*(x506)))+(((x483)*(x501)))+(((gconst51)*(gconst65)*(x520)))+(((IkReal(-1.00000000000000))*(gconst60)*(x495)*(x517)))+(((x482)*(x493))));
op[6]=((((x481)*(x524)))+(((IkReal(-1.00000000000000))*(gconst68)*(x495)*(x510)))+(((IkReal(-1.00000000000000))*(x477)*(x478)))+(((IkReal(-1.00000000000000))*(gconst51)*(x476)*(x502)))+(((IkReal(-1.00000000000000))*(gconst59)*(x476)*(x484)))+(((x491)*(x496)))+(((IkReal(-1.00000000000000))*(gconst60)*(x495)*(x506)))+(((x503)*(x518)))+(((IkReal(-1.00000000000000))*(gconst50)*(x480)*(x491)))+(((IkReal(-1.00000000000000))*(gconst51)*(x480)*(x481)))+(((IkReal(-1.00000000000000))*(gconst50)*(x476)*(x483)))+(((IkReal(-1.00000000000000))*(gconst50)*(x476)*(x500)))+(((x484)*(x501)))+(((IkReal(-1.00000000000000))*(gconst67)*(x476)*(x481)))+(((x478)*(x483)))+(((x491)*(x509)))+(((gconst49)*(gconst51)*(x522)))+(((IkReal(-1.00000000000000))*(x478)*(x517)))+(((x478)*(x500)))+(((IkReal(-1.00000000000000))*(x496)*(x510)))+(((IkReal(-1.00000000000000))*(x489)*(x520)))+(((IkReal(-1.00000000000000))*(gconst66)*(x476)*(x491)))+(((gconst53)*(gconst66)*(x486)))+(((IkReal(-1.00000000000000))*(gconst51)*(x476)*(x490)))+(((x505)*(x519)))+(((x482)*(x517)))+(((x477)*(x482))));
op[7]=((((IkReal(-1.00000000000000))*(x478)*(x506)))+(((x482)*(x506)))+(((IkReal(-1.00000000000000))*(gconst59)*(x476)*(x491)))+(((IkReal(-1.00000000000000))*(gconst50)*(x476)*(x484)))+(((x481)*(x489)))+(((x478)*(x484)))+(((x491)*(x501)))+(((gconst49)*(gconst51)*(x520)))+(((x486)*(x503)))+(((IkReal(-1.00000000000000))*(gconst60)*(x495)*(x510))));
op[8]=((((IkReal(-1.00000000000000))*(gconst51)*(x476)*(x481)))+(((IkReal(-1.00000000000000))*(x478)*(x510)))+(((IkReal(-1.00000000000000))*(gconst50)*(x476)*(x491)))+(((x478)*(x491)))+(((x482)*(x510))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x531=((sj1)*(sj3));
IkReal x532=((IkReal(1.00000000000000))*(cj0));
IkReal x533=((cj2)*(r00));
IkReal x534=((cj3)*(sj0));
IkReal x535=((cj0)*(sj3));
IkReal x536=((cj3)*(sj1));
IkReal x537=((r02)*(sj2));
IkReal x538=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x539=((cj1)*(r00)*(sj2));
IkReal x540=((sj0)*(x537));
IkReal x541=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(sj0)*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(cj3)*(x532)*(x541)))+(((IkReal(-1.00000000000000))*(x538)*(x539)))+(((IkReal(-1.00000000000000))*(x538)*(x541)))+(((IkReal(-1.00000000000000))*(cj3)*(x532)*(x539)))+(((x531)*(x540)))+(((IkReal(-1.00000000000000))*(x532)*(x533)*(x536)))+(((cj0)*(x536)*(x537)))+(((r01)*(x534)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x532))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(r01)*(x532)))+(((cj0)*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(r01)*(x538)))+(((IkReal(-1.00000000000000))*(sj1)*(x533)*(x534)))+(((x535)*(x541)))+(((IkReal(-1.00000000000000))*(x534)*(x541)))+(((x535)*(x539)))+(((IkReal(-1.00000000000000))*(x534)*(x539)))+(((IkReal(-1.00000000000000))*(x531)*(x532)*(x537)))+(((sj1)*(x534)*(x537))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((IkReal(-1.00000000000000))*(sj0)*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(cj3)*(x532)*(x541)))+(((IkReal(-1.00000000000000))*(x538)*(x539)))+(((IkReal(-1.00000000000000))*(x538)*(x541)))+(((IkReal(-1.00000000000000))*(cj3)*(x532)*(x539)))+(((x531)*(x540)))+(((IkReal(-1.00000000000000))*(x532)*(x533)*(x536)))+(((cj0)*(x536)*(x537)))+(((r01)*(x534)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x532)))))), ((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(r01)*(x532)))+(((cj0)*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(r01)*(x538)))+(((IkReal(-1.00000000000000))*(sj1)*(x533)*(x534)))+(((x535)*(x541)))+(((IkReal(-1.00000000000000))*(x534)*(x541)))+(((x535)*(x539)))+(((IkReal(-1.00000000000000))*(x534)*(x539)))+(((IkReal(-1.00000000000000))*(x531)*(x532)*(x537)))+(((sj1)*(x534)*(x537)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x542=IKcos(j5);
IkReal x543=IKsin(j5);
IkReal x544=((IkReal(1.00000000000000))*(cj0));
IkReal x545=((IkReal(1.00000000000000))*(sj0));
IkReal x546=((r02)*(sj1)*(sj2));
IkReal x547=((IkReal(1.00000000000000))*(x542));
IkReal x548=((cj1)*(cj2)*(r02));
IkReal x549=((cj2)*(r00)*(sj1));
IkReal x550=((cj1)*(r00)*(sj2));
evalcond[0]=((((cj0)*(x550)))+(((IkReal(-1.00000000000000))*(x544)*(x546)))+(((IkReal(-1.00000000000000))*(sj3)*(x547)))+(((cj0)*(x549)))+(((cj0)*(x548)))+(((IkReal(-1.00000000000000))*(r01)*(x545)))+(((cj3)*(x543))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x545)*(x548)))+(((IkReal(-1.00000000000000))*(x545)*(x549)))+(((sj0)*(x546)))+(((IkReal(-1.00000000000000))*(x545)*(x550)))+(((IkReal(-1.00000000000000))*(cj3)*(x547)))+(((IkReal(-1.00000000000000))*(sj3)*(x543)))+(((IkReal(-1.00000000000000))*(r01)*(x544))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x551=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x552=((cj1)*(x551));
IkReal x553=((IkReal(1.00000000000000))*(r02));
IkReal x554=((sj1)*(x551));
sj5array[0]=((((IkReal(-1.00000000000000))*(sj2)*(x552)*(x553)))+(((IkReal(-1.00000000000000))*(cj2)*(x553)*(x554)))+(((cj2)*(r00)*(x552)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x554))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x555=((cj0)*(cj5));
IkReal x556=((IkReal(1.00000000000000))*(r01));
IkReal x557=((cj4)*(sj0)*(sj5));
IkReal x558=((cj2)*(r00)*(sj1));
IkReal x559=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x560=((cj0)*(cj4)*(sj5));
IkReal x561=((cj1)*(r00)*(sj2));
IkReal x562=((cj1)*(cj2)*(r02));
IkReal x563=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((r01)*(x560)))+(((x557)*(x558)))+(((IkReal(-1.00000000000000))*(x555)*(x563)))+(((x555)*(x558)))+(((IkReal(-1.00000000000000))*(x557)*(x563)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x556)))+(((x557)*(x562)))+(((x557)*(x561)))+(((x555)*(x561)))+(((x555)*(x562))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(-1.00000000000000))*(x556)*(x557)))+(((x558)*(x560)))+(((x560)*(x561)))+(((x560)*(x562)))+(((IkReal(-1.00000000000000))*(x559)*(x562)))+(((IkReal(-1.00000000000000))*(x559)*(x561)))+(((IkReal(-1.00000000000000))*(x560)*(x563)))+(((IkReal(-1.00000000000000))*(x558)*(x559)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((r01)*(x560)))+(((x557)*(x558)))+(((IkReal(-1.00000000000000))*(x555)*(x563)))+(((x555)*(x558)))+(((IkReal(-1.00000000000000))*(x557)*(x563)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x556)))+(((x557)*(x562)))+(((x557)*(x561)))+(((x555)*(x561)))+(((x555)*(x562)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(-1.00000000000000))*(x556)*(x557)))+(((x558)*(x560)))+(((x560)*(x561)))+(((x560)*(x562)))+(((IkReal(-1.00000000000000))*(x559)*(x562)))+(((IkReal(-1.00000000000000))*(x559)*(x561)))+(((IkReal(-1.00000000000000))*(x560)*(x563)))+(((IkReal(-1.00000000000000))*(x558)*(x559)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x564=IKsin(j3);
IkReal x565=IKcos(j3);
IkReal x566=((IkReal(1.00000000000000))*(cj0));
IkReal x567=((cj0)*(cj1));
IkReal x568=((cj2)*(r02));
IkReal x569=((r00)*(sj2));
IkReal x570=((IkReal(1.00000000000000))*(sj0));
IkReal x571=((cj4)*(sj5));
IkReal x572=((r02)*(sj1)*(sj2));
IkReal x573=((cj2)*(r00)*(sj1));
IkReal x574=((IkReal(1.00000000000000))*(x565));
evalcond[0]=((((IkReal(-1.00000000000000))*(x566)*(x572)))+(((IkReal(-1.00000000000000))*(r01)*(x570)))+(((cj0)*(x573)))+(((IkReal(-1.00000000000000))*(x571)*(x574)))+(((x567)*(x569)))+(((x567)*(x568)))+(((IkReal(-1.00000000000000))*(cj5)*(x564))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x570)*(x573)))+(((IkReal(-1.00000000000000))*(cj5)*(x574)))+(((x564)*(x571)))+(((IkReal(-1.00000000000000))*(cj1)*(x569)*(x570)))+(((IkReal(-1.00000000000000))*(cj1)*(x568)*(x570)))+(((IkReal(-1.00000000000000))*(r01)*(x566)))+(((sj0)*(x572))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x575=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x575)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x575)))+(((cj1)*(cj2)*(r00))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x576=((cj0)*(cj5));
IkReal x577=((IkReal(1.00000000000000))*(r01));
IkReal x578=((cj4)*(sj0)*(sj5));
IkReal x579=((cj2)*(r00)*(sj1));
IkReal x580=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x581=((cj0)*(cj4)*(sj5));
IkReal x582=((cj1)*(r00)*(sj2));
IkReal x583=((cj1)*(cj2)*(r02));
IkReal x584=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x578)*(x582)))+(((x578)*(x583)))+(((x576)*(x579)))+(((IkReal(-1.00000000000000))*(x578)*(x584)))+(((x578)*(x579)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x577)))+(((r01)*(x581)))+(((IkReal(-1.00000000000000))*(x576)*(x584)))+(((x576)*(x583)))+(((x576)*(x582))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x580)*(x582)))+(((IkReal(-1.00000000000000))*(x580)*(x583)))+(((x581)*(x583)))+(((x581)*(x582)))+(((x579)*(x581)))+(((IkReal(-1.00000000000000))*(x577)*(x578)))+(((IkReal(-1.00000000000000))*(x579)*(x580)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x581)*(x584))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x578)*(x582)))+(((x578)*(x583)))+(((x576)*(x579)))+(((IkReal(-1.00000000000000))*(x578)*(x584)))+(((x578)*(x579)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x577)))+(((r01)*(x581)))+(((IkReal(-1.00000000000000))*(x576)*(x584)))+(((x576)*(x583)))+(((x576)*(x582)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x580)*(x582)))+(((IkReal(-1.00000000000000))*(x580)*(x583)))+(((x581)*(x583)))+(((x581)*(x582)))+(((x579)*(x581)))+(((IkReal(-1.00000000000000))*(x577)*(x578)))+(((IkReal(-1.00000000000000))*(x579)*(x580)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x581)*(x584)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x585=IKsin(j3);
IkReal x586=IKcos(j3);
IkReal x587=((IkReal(1.00000000000000))*(cj0));
IkReal x588=((cj0)*(cj1));
IkReal x589=((cj2)*(r02));
IkReal x590=((r00)*(sj2));
IkReal x591=((IkReal(1.00000000000000))*(sj0));
IkReal x592=((cj4)*(sj5));
IkReal x593=((r02)*(sj1)*(sj2));
IkReal x594=((cj2)*(r00)*(sj1));
IkReal x595=((IkReal(1.00000000000000))*(x586));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x591)))+(((IkReal(-1.00000000000000))*(x587)*(x593)))+(((x588)*(x589)))+(((IkReal(-1.00000000000000))*(x592)*(x595)))+(((x588)*(x590)))+(((cj0)*(x594)))+(((IkReal(-1.00000000000000))*(cj5)*(x585))));
evalcond[1]=((((x585)*(x592)))+(((IkReal(-1.00000000000000))*(cj5)*(x595)))+(((IkReal(-1.00000000000000))*(cj1)*(x589)*(x591)))+(((IkReal(-1.00000000000000))*(cj1)*(x590)*(x591)))+(((sj0)*(x593)))+(((IkReal(-1.00000000000000))*(r01)*(x587)))+(((IkReal(-1.00000000000000))*(x591)*(x594))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x596=((IkReal(6620.00000000000))*(sj2));
IkReal x597=((IkReal(2003.00000000000))*(sj1));
IkReal x598=((IkReal(20000.0000000000))*(cj1)*(pz));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x597)))+(((px)*(x598)))+(((px)*(x596))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x597)))+(((IkReal(-1.00000000000000))*(py)*(x596)))+(((IkReal(-1.00000000000000))*(py)*(x598))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(py)*(x597)))+(((px)*(x598)))+(((px)*(x596)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(px)*(x597)))+(((IkReal(-1.00000000000000))*(py)*(x596)))+(((IkReal(-1.00000000000000))*(py)*(x598)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x599=IKcos(j0);
IkReal x600=IKsin(j0);
IkReal x601=((IkReal(0.810000000000000))*(cj1));
IkReal x602=((IkReal(0.331000000000000))*(sj2));
IkReal x603=((pz)*(sj1));
IkReal x604=((IkReal(0.331000000000000))*(cj2));
IkReal x605=((px)*(x600));
IkReal x606=((py)*(x599));
IkReal x607=((IkReal(1.00000000000000))*(x606));
evalcond[0]=((IkReal(0.100150000000000))+(((py)*(x600)))+(((px)*(x599))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj1)*(x605)))+(((sj1)*(x606)))+(((cj1)*(pz)))+(x602));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(x607)))+(((cj1)*(x605)))+(x603)+(x604));
evalcond[3]=((IkReal(-0.0444339775000000))+(((x601)*(x606)))+(((IkReal(-0.810000000000000))*(x603)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x601)*(x605))));
evalcond[4]=((((IkReal(0.405000000000000))*(cj1)))+(((IkReal(-1.00000000000000))*(sj1)*(x602)))+(((cj1)*(x604)))+(x605)+(((IkReal(-1.00000000000000))*(x607))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x608=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x608)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x608)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x609=((sj0)*(sj1));
IkReal x610=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x609)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x609)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x610)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x610))));
IkReal gconst6;
IkReal x611=((sj0)*(sj1));
IkReal x612=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x611)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x611)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x612)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x612))));
IkReal gconst7;
IkReal x613=((cj0)*(r02));
IkReal x614=((cj0)*(r00));
gconst7=((((cj2)*(sj1)*(x614)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x613)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x613)))+(((cj1)*(sj2)*(x614))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x615=((cj0)*(r02));
IkReal x616=((cj0)*(r00));
gconst9=((((cj2)*(sj1)*(x616)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x615)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x615)))+(((cj1)*(sj2)*(x616))));
IkReal gconst10;
IkReal x617=((sj0)*(sj1));
IkReal x618=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x617)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x617)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x618)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x618))));
IkReal gconst11;
IkReal x619=((sj0)*(sj1));
IkReal x620=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x619)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x620)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x619)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x620))));
IkReal gconst12;
IkReal x621=((cj0)*(r02));
IkReal x622=((cj0)*(r00));
gconst12=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x622)))+(((cj1)*(sj2)*(x622)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x621)))+(((cj1)*(cj2)*(x621))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x623=((cj0)*(r02));
IkReal x624=((cj0)*(r00));
gconst14=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x624)))+(((cj1)*(sj2)*(x624)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x623)))+(((cj1)*(cj2)*(x623))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x625=((sj0)*(sj1));
IkReal x626=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x626)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x625)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x626)))+(((r02)*(sj2)*(x625))));
IkReal gconst22;
IkReal x627=((sj0)*(sj1));
IkReal x628=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x628)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x627)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x628)))+(((r02)*(sj2)*(x627))));
IkReal gconst23;
IkReal x629=((cj0)*(r02));
IkReal x630=((cj0)*(r00));
gconst23=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x630)))+(((cj1)*(sj2)*(x630)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x629)))+(((cj1)*(cj2)*(x629))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x631=((cj0)*(r02));
IkReal x632=((cj0)*(r00));
gconst25=((((cj1)*(cj2)*(x631)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x632)))+(((cj1)*(sj2)*(x632)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x631))));
IkReal gconst26;
IkReal x633=((sj0)*(sj1));
IkReal x634=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x634)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x633)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x634)))+(((r02)*(sj2)*(x633))));
IkReal gconst27;
IkReal x635=((sj0)*(sj1));
IkReal x636=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x636)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x635)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x636)))+(((r02)*(sj2)*(x635))));
IkReal gconst28;
IkReal x637=((cj0)*(r02));
IkReal x638=((cj0)*(r00));
gconst28=((((cj1)*(cj2)*(x637)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x638)))+(((cj1)*(sj2)*(x638)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x637))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x639=((cj0)*(r02));
IkReal x640=((cj0)*(r00));
gconst30=((((cj1)*(sj2)*(x640)))+(((cj1)*(cj2)*(x639)))+(((cj2)*(sj1)*(x640)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x639))));
IkReal x641=((gconst27)*(gconst28));
IkReal x642=((gconst22)*(gconst23));
IkReal x643=((gconst21)*(gconst25));
IkReal x644=((gconst26)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x641)*(x642)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((IkReal(-1.00000000000000))*(x643)*(x644)))+(((x641)*(x643)))+(((x642)*(x644))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x645=((IkReal(1.00000000000000))*(gconst19));
IkReal x646=((gconst23)*(gconst27));
IkReal x647=((gconst15)*(gconst18));
IkReal x648=((gconst11)*(gconst6));
IkReal x649=((gconst13)*(gconst26));
IkReal x650=((gconst15)*(gconst9));
IkReal x651=((gconst6)*(gconst8));
IkReal x652=((gconst18)*(gconst28));
IkReal x653=((gconst22)*(gconst30));
IkReal x654=((gconst16)*(gconst26));
IkReal x655=((IkReal(1.00000000000000))*(gconst10));
IkReal x656=((gconst14)*(gconst25));
IkReal x657=((IkReal(1.00000000000000))*(gconst26));
IkReal x658=((gconst12)*(gconst27));
IkReal x659=((gconst25)*(gconst5));
IkReal x660=((gconst17)*(gconst28));
IkReal x661=((gconst21)*(gconst27));
IkReal x662=((gconst27)*(gconst6));
IkReal x663=((IkReal(1.00000000000000))*(gconst28));
IkReal x664=((gconst11)*(gconst5));
IkReal x665=((gconst17)*(gconst21));
IkReal x666=((gconst18)*(gconst19));
IkReal x667=((gconst10)*(gconst7));
IkReal x668=((gconst30)*(gconst6));
IkReal x669=((gconst17)*(gconst5));
IkReal x670=((IkReal(1.00000000000000))*(gconst12));
IkReal x671=((gconst22)*(gconst7));
IkReal x672=((gconst24)*(gconst6));
IkReal x673=((gconst5)*(gconst9));
IkReal x674=((gconst15)*(gconst17));
IkReal x675=((gconst27)*(gconst28));
IkReal x676=((gconst23)*(gconst26));
IkReal x677=((gconst21)*(gconst9));
IkReal x678=((gconst22)*(gconst8));
IkReal x679=((gconst10)*(gconst23));
IkReal x680=((gconst12)*(gconst18));
IkReal x681=((gconst21)*(gconst25));
IkReal x682=((gconst11)*(gconst28));
IkReal x683=((gconst11)*(gconst19));
IkReal x684=((gconst19)*(gconst27));
IkReal x685=((gconst20)*(gconst6));
IkReal x686=((gconst10)*(gconst16));
IkReal x687=((gconst26)*(gconst29));
IkReal x688=((gconst26)*(gconst7));
IkReal x689=((gconst20)*(gconst22));
IkReal x690=((gconst22)*(gconst24));
IkReal x691=((gconst15)*(gconst25));
IkReal x692=((gconst10)*(gconst13));
IkReal x693=((gconst14)*(gconst6));
IkReal x694=((gconst16)*(gconst22));
IkReal x695=((gconst11)*(gconst12));
IkReal x696=((gconst14)*(gconst22));
IkReal x697=((gconst17)*(gconst19));
IkReal x698=((gconst10)*(gconst29));
IkReal x699=((gconst28)*(gconst9));
IkReal x700=((IkReal(1.00000000000000))*(gconst6));
IkReal x701=((gconst26)*(x685));
IkReal x702=((gconst9)*(x683));
IkReal x703=((gconst11)*(x694));
IkReal x704=((gconst14)*(x679));
IkReal x705=((gconst11)*(gconst22)*(gconst23));
op[0]=((((gconst25)*(gconst28)*(x661)))+(((IkReal(-1.00000000000000))*(gconst22)*(x646)*(x663)))+(((IkReal(-1.00000000000000))*(gconst30)*(x657)*(x681)))+(((x653)*(x676)))+(((IkReal(-1.00000000000000))*(gconst29)*(x657)*(x690))));
op[1]=((((IkReal(-1.00000000000000))*(gconst30)*(x657)*(x665)))+(((x652)*(x690)))+(((x687)*(x691)))+(((IkReal(-1.00000000000000))*(gconst20)*(x657)*(x681)))+(((x653)*(x654)))+(((x676)*(x689)))+(((IkReal(-1.00000000000000))*(gconst22)*(x645)*(x646)))+(((x660)*(x661)))+(((gconst19)*(gconst25)*(x661)))+(((IkReal(-1.00000000000000))*(gconst27)*(x663)*(x694))));
op[2]=((((IkReal(-1.00000000000000))*(gconst25)*(x647)*(x663)))+(((x654)*(x689)))+(((x653)*(x688)))+(((IkReal(-1.00000000000000))*(gconst27)*(x645)*(x694)))+(((x658)*(x681)))+(((IkReal(-1.00000000000000))*(gconst6)*(x646)*(x663)))+(((x659)*(x675)))+(((IkReal(-1.00000000000000))*(gconst30)*(x657)*(x677)))+(((x674)*(x687)))+(((x661)*(x699)))+(((x661)*(x697)))+(((IkReal(-1.00000000000000))*(gconst20)*(x657)*(x665)))+(((IkReal(-1.00000000000000))*(x663)*(x705)))+(((IkReal(-1.00000000000000))*(gconst21)*(x656)*(x657)))+(((x681)*(x682)))+(((x668)*(x676)))+(((IkReal(-1.00000000000000))*(gconst22)*(x646)*(x670)))+(((x676)*(x696)))+(((IkReal(-1.00000000000000))*(gconst29)*(x655)*(x690)))+(((IkReal(-1.00000000000000))*(x649)*(x690)))+(((x653)*(x679)))+(((IkReal(-1.00000000000000))*(gconst30)*(x657)*(x659)))+(((x666)*(x690)))+(((IkReal(-1.00000000000000))*(gconst30)*(x655)*(x681)))+(((IkReal(-1.00000000000000))*(gconst27)*(x663)*(x671)))+(((IkReal(-1.00000000000000))*(gconst29)*(x657)*(x672)))+(((IkReal(-1.00000000000000))*(gconst29)*(x657)*(x678))));
op[3]=((((gconst20)*(gconst26)*(x671)))+(((IkReal(-1.00000000000000))*(x647)*(x660)))+(((x659)*(x684)))+(((x653)*(x686)))+(((x649)*(x691)))+(((IkReal(-1.00000000000000))*(gconst20)*(x655)*(x681)))+(((IkReal(-1.00000000000000))*(gconst30)*(x657)*(x669)))+(((x654)*(x696)))+(((IkReal(-1.00000000000000))*(gconst16)*(x662)*(x663)))+(((x650)*(x687)))+(((IkReal(-1.00000000000000))*(gconst25)*(x645)*(x647)))+(((x691)*(x698)))+(((x676)*(x685)))+(((IkReal(-1.00000000000000))*(x645)*(x705)))+(((IkReal(-1.00000000000000))*(gconst30)*(x655)*(x665)))+(((IkReal(-1.00000000000000))*(x663)*(x703)))+(((x679)*(x689)))+(((gconst19)*(gconst9)*(x661)))+(((x680)*(x690)))+(((x681)*(x683)))+(((x654)*(x668)))+(((gconst11)*(gconst21)*(x660)))+(((IkReal(-1.00000000000000))*(x658)*(x694)))+(((IkReal(-1.00000000000000))*(gconst14)*(x657)*(x665)))+(((gconst27)*(gconst5)*(x660)))+(((x658)*(x665)))+(((IkReal(-1.00000000000000))*(gconst20)*(x657)*(x659)))+(((IkReal(-1.00000000000000))*(gconst20)*(x657)*(x677)))+(((x652)*(x678)))+(((x652)*(x672)))+(((IkReal(-1.00000000000000))*(gconst27)*(x645)*(x671)))+(((IkReal(-1.00000000000000))*(gconst6)*(x645)*(x646))));
op[4]=((((x669)*(x684)))+(((x673)*(x675)))+(((x654)*(x685)))+(((x659)*(x682)))+(((x686)*(x689)))+(((IkReal(-1.00000000000000))*(gconst17)*(x645)*(x647)))+(((IkReal(-1.00000000000000))*(gconst20)*(x655)*(x665)))+(((IkReal(-1.00000000000000))*(gconst6)*(x646)*(x670)))+(((IkReal(-1.00000000000000))*(gconst29)*(x651)*(x657)))+(((IkReal(-1.00000000000000))*(gconst14)*(x657)*(x677)))+(((IkReal(-1.00000000000000))*(gconst30)*(x657)*(x673)))+(((x658)*(x677)))+(((x677)*(x682)))+(((IkReal(-1.00000000000000))*(gconst20)*(x657)*(x669)))+(((IkReal(-1.00000000000000))*(gconst30)*(x655)*(x677)))+(((gconst14)*(gconst26)*(x671)))+(((IkReal(-1.00000000000000))*(gconst9)*(x647)*(x663)))+(((IkReal(-1.00000000000000))*(gconst25)*(x647)*(x670)))+(((IkReal(-1.00000000000000))*(x645)*(x703)))+(((IkReal(-1.00000000000000))*(gconst11)*(x663)*(x671)))+(((x666)*(x672)))+(((x666)*(x678)))+(((x681)*(x695)))+(((IkReal(-1.00000000000000))*(x658)*(x671)))+(((IkReal(-1.00000000000000))*(gconst7)*(x662)*(x663)))+(((x679)*(x696)))+(((IkReal(-1.00000000000000))*(x649)*(x678)))+(((IkReal(-1.00000000000000))*(x649)*(x672)))+(((x668)*(x679)))+(((IkReal(-1.00000000000000))*(gconst23)*(x648)*(x663)))+(((IkReal(-1.00000000000000))*(gconst16)*(x645)*(x662)))+(((x649)*(x674)))+(((x653)*(x667)))+(((x676)*(x693)))+(((IkReal(-1.00000000000000))*(gconst29)*(x655)*(x678)))+(((IkReal(-1.00000000000000))*(gconst29)*(x655)*(x672)))+(((IkReal(-1.00000000000000))*(gconst21)*(x655)*(x656)))+(((IkReal(-1.00000000000000))*(gconst13)*(x655)*(x690)))+(((x674)*(x698)))+(((IkReal(-1.00000000000000))*(gconst5)*(x656)*(x657)))+(((x665)*(x683)))+(((x668)*(x688)))+(((x658)*(x659)))+(((IkReal(-1.00000000000000))*(gconst30)*(x655)*(x659)))+(((IkReal(-1.00000000000000))*(x670)*(x705))));
op[5]=((((x678)*(x680)))+(((IkReal(-1.00000000000000))*(gconst20)*(x655)*(x677)))+(((x659)*(x683)))+(((x650)*(x698)))+(((x654)*(x693)))+(((x686)*(x696)))+(((IkReal(-1.00000000000000))*(gconst16)*(x658)*(x700)))+(((x685)*(x688)))+(((IkReal(-1.00000000000000))*(gconst14)*(x655)*(x665)))+(((x649)*(x650)))+(((x691)*(x692)))+(((x677)*(x683)))+(((IkReal(-1.00000000000000))*(gconst23)*(x645)*(x648)))+(((x651)*(x652)))+(((IkReal(-1.00000000000000))*(gconst11)*(x645)*(x671)))+(((x667)*(x689)))+(((IkReal(-1.00000000000000))*(gconst30)*(x655)*(x669)))+(((x679)*(x685)))+(((x660)*(x664)))+(((x673)*(x684)))+(((IkReal(-1.00000000000000))*(gconst9)*(x645)*(x647)))+(((IkReal(-1.00000000000000))*(gconst20)*(x655)*(x659)))+(((x672)*(x680)))+(((IkReal(-1.00000000000000))*(gconst14)*(x657)*(x669)))+(((x658)*(x669)))+(((x665)*(x695)))+(((IkReal(-1.00000000000000))*(gconst7)*(x645)*(x662)))+(((IkReal(-1.00000000000000))*(gconst16)*(x648)*(x663)))+(((IkReal(-1.00000000000000))*(gconst20)*(x657)*(x673)))+(((x668)*(x686)))+(((IkReal(-1.00000000000000))*(gconst17)*(x647)*(x670)))+(((IkReal(-1.00000000000000))*(x670)*(x703))));
op[6]=((((IkReal(-1.00000000000000))*(gconst5)*(x655)*(x656)))+(((x688)*(x693)))+(((x667)*(x668)))+(((x651)*(x666)))+(((IkReal(-1.00000000000000))*(gconst20)*(x655)*(x669)))+(((IkReal(-1.00000000000000))*(gconst9)*(x647)*(x670)))+(((IkReal(-1.00000000000000))*(gconst29)*(x651)*(x655)))+(((IkReal(-1.00000000000000))*(gconst14)*(x657)*(x673)))+(((x685)*(x686)))+(((x658)*(x673)))+(((IkReal(-1.00000000000000))*(gconst30)*(x655)*(x673)))+(((IkReal(-1.00000000000000))*(gconst7)*(x648)*(x663)))+(((IkReal(-1.00000000000000))*(gconst13)*(x655)*(x672)))+(((IkReal(-1.00000000000000))*(gconst13)*(x655)*(x678)))+(((x679)*(x693)))+(((x659)*(x695)))+(((x677)*(x695)))+(((IkReal(-1.00000000000000))*(gconst14)*(x655)*(x677)))+(((IkReal(-1.00000000000000))*(gconst23)*(x648)*(x670)))+(((x664)*(x699)))+(((x664)*(x697)))+(((IkReal(-1.00000000000000))*(gconst16)*(x645)*(x648)))+(((x674)*(x692)))+(((IkReal(-1.00000000000000))*(x649)*(x651)))+(((x667)*(x696)))+(((IkReal(-1.00000000000000))*(gconst11)*(x670)*(x671)))+(((IkReal(-1.00000000000000))*(gconst7)*(x658)*(x700))));
op[7]=((((IkReal(-1.00000000000000))*(gconst20)*(x655)*(x673)))+(((x650)*(x692)))+(((x686)*(x693)))+(((IkReal(-1.00000000000000))*(gconst14)*(x655)*(x669)))+(((IkReal(-1.00000000000000))*(gconst16)*(x648)*(x670)))+(((x667)*(x685)))+(((gconst19)*(gconst9)*(x664)))+(((gconst12)*(gconst17)*(x664)))+(((IkReal(-1.00000000000000))*(gconst7)*(x645)*(x648)))+(((x651)*(x680))));
op[8]=((((IkReal(-1.00000000000000))*(gconst13)*(x651)*(x655)))+(((IkReal(-1.00000000000000))*(gconst7)*(x648)*(x670)))+(((IkReal(-1.00000000000000))*(gconst14)*(x655)*(x673)))+(((gconst12)*(gconst9)*(x664)))+(((x667)*(x693))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x706=((r01)*(sj3));
IkReal x707=((IkReal(1.00000000000000))*(sj0));
IkReal x708=((sj1)*(sj3));
IkReal x709=((cj2)*(r00));
IkReal x710=((r02)*(sj2));
IkReal x711=((cj3)*(sj1));
IkReal x712=((IkReal(1.00000000000000))*(cj0));
IkReal x713=((cj3)*(r01));
IkReal x714=((cj1)*(r00)*(sj2));
IkReal x715=((cj1)*(cj2)*(r02)*(sj3));
IkReal x716=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((sj0)*(x708)*(x709)))+(((IkReal(-1.00000000000000))*(x707)*(x713)))+(((IkReal(-1.00000000000000))*(x707)*(x708)*(x710)))+(((cj0)*(x706)))+(((cj0)*(x716)))+(((IkReal(-1.00000000000000))*(x710)*(x711)*(x712)))+(((cj0)*(cj3)*(x714)))+(((cj0)*(x709)*(x711)))+(((sj0)*(x715)))+(((sj0)*(sj3)*(x714))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x706)*(x707)))+(((IkReal(-1.00000000000000))*(x707)*(x716)))+(((cj0)*(x708)*(x709)))+(((cj0)*(x715)))+(((sj0)*(x710)*(x711)))+(((IkReal(-1.00000000000000))*(cj3)*(x707)*(x714)))+(((IkReal(-1.00000000000000))*(x708)*(x710)*(x712)))+(((IkReal(-1.00000000000000))*(x712)*(x713)))+(((cj0)*(sj3)*(x714)))+(((IkReal(-1.00000000000000))*(x707)*(x709)*(x711))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((sj0)*(x708)*(x709)))+(((IkReal(-1.00000000000000))*(x707)*(x713)))+(((IkReal(-1.00000000000000))*(x707)*(x708)*(x710)))+(((cj0)*(x706)))+(((cj0)*(x716)))+(((IkReal(-1.00000000000000))*(x710)*(x711)*(x712)))+(((cj0)*(cj3)*(x714)))+(((cj0)*(x709)*(x711)))+(((sj0)*(x715)))+(((sj0)*(sj3)*(x714)))))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x706)*(x707)))+(((IkReal(-1.00000000000000))*(x707)*(x716)))+(((cj0)*(x708)*(x709)))+(((cj0)*(x715)))+(((sj0)*(x710)*(x711)))+(((IkReal(-1.00000000000000))*(cj3)*(x707)*(x714)))+(((IkReal(-1.00000000000000))*(x708)*(x710)*(x712)))+(((IkReal(-1.00000000000000))*(x712)*(x713)))+(((cj0)*(sj3)*(x714)))+(((IkReal(-1.00000000000000))*(x707)*(x709)*(x711)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x717=IKsin(j5);
IkReal x718=IKcos(j5);
IkReal x719=((IkReal(1.00000000000000))*(cj0));
IkReal x720=((IkReal(1.00000000000000))*(cj3));
IkReal x721=((cj0)*(cj1));
IkReal x722=((cj2)*(r02));
IkReal x723=((r00)*(sj2));
IkReal x724=((IkReal(1.00000000000000))*(sj0));
IkReal x725=((r02)*(sj1)*(sj2));
IkReal x726=((cj2)*(r00)*(sj1));
evalcond[0]=((((x721)*(x723)))+(((x721)*(x722)))+(((IkReal(-1.00000000000000))*(sj3)*(x718)))+(((IkReal(-1.00000000000000))*(x717)*(x720)))+(((cj0)*(x726)))+(((IkReal(-1.00000000000000))*(x719)*(x725)))+(((IkReal(-1.00000000000000))*(r01)*(x724))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x724)*(x726)))+(((sj3)*(x717)))+(((sj0)*(x725)))+(((IkReal(-1.00000000000000))*(r01)*(x719)))+(((IkReal(-1.00000000000000))*(cj1)*(x723)*(x724)))+(((IkReal(-1.00000000000000))*(x718)*(x720)))+(((IkReal(-1.00000000000000))*(cj1)*(x722)*(x724))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x727=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x727)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x727))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x728=((sj0)*(sj1));
IkReal x729=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x728)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x729)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x729)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x728))));
IkReal gconst49;
IkReal x730=((sj0)*(sj1));
IkReal x731=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x731)))+(((r02)*(sj2)*(x730)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x731)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x730))));
IkReal gconst50;
IkReal x732=((cj0)*(r02));
IkReal x733=((cj0)*(r00));
gconst50=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x732)))+(((cj1)*(cj2)*(x732)))+(((cj2)*(sj1)*(x733)))+(((cj1)*(sj2)*(x733))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x734=((cj0)*(r02));
IkReal x735=((cj0)*(r00));
gconst52=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x734)))+(((cj1)*(cj2)*(x734)))+(((cj2)*(sj1)*(x735)))+(((cj1)*(sj2)*(x735))));
IkReal gconst53;
IkReal x736=((sj0)*(sj1));
IkReal x737=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x737)))+(((r02)*(sj2)*(x736)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x737)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x736))));
IkReal gconst54;
IkReal x738=((sj0)*(sj1));
IkReal x739=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x739)))+(((r02)*(sj2)*(x738)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x739)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x738))));
IkReal gconst55;
IkReal x740=((cj0)*(r02));
IkReal x741=((cj0)*(r00));
gconst55=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x740)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x741)))+(((cj1)*(cj2)*(x740)))+(((cj2)*(sj1)*(x741))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x742=((cj0)*(r02));
IkReal x743=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x742)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x743)))+(((cj1)*(cj2)*(x742)))+(((cj2)*(sj1)*(x743))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x744=((sj0)*(sj1));
IkReal x745=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x745)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x744)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x745)))+(((r02)*(sj2)*(x744))));
IkReal gconst65;
IkReal x746=((sj0)*(sj1));
IkReal x747=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x747)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x746)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x747)))+(((r02)*(sj2)*(x746))));
IkReal gconst66;
IkReal x748=((cj0)*(r02));
IkReal x749=((cj0)*(r00));
gconst66=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x748)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x749)))+(((cj1)*(cj2)*(x748)))+(((cj2)*(sj1)*(x749))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x750=((cj0)*(r02));
IkReal x751=((cj0)*(r00));
gconst68=((((cj1)*(sj2)*(x751)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x750)))+(((cj2)*(sj1)*(x751)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x750))));
IkReal gconst69;
IkReal x752=((sj0)*(sj1));
IkReal x753=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x753)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x752)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x753)))+(((r02)*(sj2)*(x752))));
IkReal gconst70;
IkReal x754=((sj0)*(sj1));
IkReal x755=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x755)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x754)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x755)))+(((r02)*(sj2)*(x754))));
IkReal gconst71;
IkReal x756=((cj0)*(r02));
IkReal x757=((cj0)*(r00));
gconst71=((((cj1)*(sj2)*(x757)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x756)))+(((cj2)*(sj1)*(x757)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x756))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x758=((cj0)*(r02));
IkReal x759=((cj0)*(r00));
gconst73=((((cj1)*(sj2)*(x759)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x758)))+(((cj2)*(sj1)*(x759)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x758))));
IkReal x760=((gconst70)*(gconst71));
IkReal x761=((IkReal(1.00000000000000))*(gconst65));
IkReal x762=((gconst64)*(gconst68));
IkReal x763=((gconst69)*(gconst73));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst66)*(x760)*(x761)))+(((gconst65)*(gconst66)*(x763)))+(((x760)*(x762)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x761)))+(((IkReal(-1.00000000000000))*(x762)*(x763))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x764=((gconst62)*(gconst70));
IkReal x765=((IkReal(1.00000000000000))*(gconst49));
IkReal x766=((gconst53)*(gconst73));
IkReal x767=((gconst48)*(gconst52));
IkReal x768=((gconst65)*(gconst66));
IkReal x769=((IkReal(1.00000000000000))*(gconst65));
IkReal x770=((gconst53)*(gconst56));
IkReal x771=((gconst49)*(gconst50));
IkReal x772=((gconst55)*(gconst70));
IkReal x773=((gconst54)*(gconst62));
IkReal x774=((gconst59)*(gconst69));
IkReal x775=((gconst49)*(gconst57));
IkReal x776=((gconst52)*(gconst63));
IkReal x777=((gconst60)*(gconst73));
IkReal x778=((gconst52)*(gconst58));
IkReal x779=((gconst56)*(gconst69));
IkReal x780=((gconst54)*(gconst55));
IkReal x781=((gconst63)*(gconst65));
IkReal x782=((gconst63)*(gconst69));
IkReal x783=((gconst49)*(gconst66));
IkReal x784=((IkReal(1.00000000000000))*(gconst48));
IkReal x785=((gconst52)*(gconst64));
IkReal x786=((gconst70)*(gconst71));
IkReal x787=((gconst68)*(gconst69));
IkReal x788=((IkReal(1.00000000000000))*(gconst68));
IkReal x789=((gconst54)*(gconst71));
IkReal x790=((gconst48)*(gconst60));
IkReal x791=((gconst53)*(gconst72));
IkReal x792=((gconst53)*(gconst59));
IkReal x793=((gconst58)*(gconst61));
IkReal x794=((gconst57)*(gconst65));
IkReal x795=((gconst53)*(gconst63));
IkReal x796=((gconst61)*(gconst71));
IkReal x797=((gconst65)*(gconst67));
IkReal x798=((gconst48)*(gconst68));
IkReal x799=((gconst53)*(gconst57));
IkReal x800=((gconst60)*(gconst64));
IkReal x801=((gconst69)*(gconst72));
IkReal x802=((gconst58)*(gconst68));
IkReal x803=((gconst50)*(gconst69));
IkReal x804=((IkReal(1.00000000000000))*(gconst60));
IkReal x805=((gconst69)*(gconst73));
IkReal x806=((gconst57)*(gconst69));
IkReal x807=((gconst49)*(gconst63));
IkReal x808=((gconst50)*(gconst53));
IkReal x809=((gconst55)*(gconst61));
IkReal x810=((gconst65)*(gconst73));
IkReal x811=((gconst61)*(gconst62));
IkReal x812=((gconst64)*(gconst68));
IkReal x813=((gconst58)*(gconst60));
IkReal x814=((gconst49)*(gconst67));
IkReal x815=((gconst60)*(x780));
IkReal x816=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x817=((gconst72)*(x813));
IkReal x818=((gconst49)*(gconst51)*(gconst61));
IkReal x819=((gconst51)*(gconst61)*(gconst65));
op[0]=((((x768)*(x805)))+(((x786)*(x812)))+(((IkReal(-1.00000000000000))*(x768)*(x786)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x787)))+(((IkReal(-1.00000000000000))*(gconst67)*(x769)*(x801))));
op[1]=((((x786)*(x800)))+(((x796)*(x797)))+(((x774)*(x810)))+(((x764)*(x812)))+(((IkReal(-1.00000000000000))*(gconst64)*(x782)*(x788)))+(((gconst58)*(gconst72)*(x787)))+(((x768)*(x782)))+(((IkReal(-1.00000000000000))*(x777)*(x816)))+(((IkReal(-1.00000000000000))*(gconst59)*(x769)*(x786)))+(((IkReal(-1.00000000000000))*(x764)*(x768))));
op[2]=((((x797)*(x811)))+(((x786)*(x798)))+(((x801)*(x813)))+(((x785)*(x786)))+(((IkReal(-1.00000000000000))*(x782)*(x800)))+(((IkReal(-1.00000000000000))*(gconst67)*(x765)*(x801)))+(((IkReal(-1.00000000000000))*(gconst59)*(x764)*(x769)))+(((x774)*(x781)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x787)))+(((IkReal(-1.00000000000000))*(gconst51)*(x769)*(x801)))+(((IkReal(-1.00000000000000))*(gconst67)*(x769)*(x791)))+(((x768)*(x806)))+(((IkReal(-1.00000000000000))*(gconst71)*(x788)*(x793)))+(((IkReal(-1.00000000000000))*(gconst73)*(x784)*(x787)))+(((x772)*(x812)))+(((IkReal(-1.00000000000000))*(gconst66)*(x765)*(x786)))+(((IkReal(-1.00000000000000))*(x768)*(x789)))+(((IkReal(-1.00000000000000))*(x785)*(x805)))+(((IkReal(-1.00000000000000))*(gconst67)*(x769)*(x779)))+(((x766)*(x768)))+(((IkReal(-1.00000000000000))*(x768)*(x772)))+(((IkReal(-1.00000000000000))*(gconst64)*(x766)*(x788)))+(((x789)*(x812)))+(((x764)*(x800)))+(((x803)*(x810)))+(((IkReal(-1.00000000000000))*(gconst50)*(x769)*(x786)))+(((x783)*(x805))));
op[3]=((((x764)*(x785)))+(((x764)*(x798)))+(((x786)*(x790)))+(((IkReal(-1.00000000000000))*(gconst69)*(x777)*(x784)))+(((x772)*(x800)))+(((x781)*(x803)))+(((x778)*(x801)))+(((x774)*(x794)))+(((gconst59)*(gconst65)*(x766)))+(((x779)*(x802)))+(((x782)*(x783)))+(((x796)*(x814)))+(((IkReal(-1.00000000000000))*(gconst64)*(x788)*(x795)))+(((IkReal(-1.00000000000000))*(gconst62)*(x788)*(x793)))+(((IkReal(-1.00000000000000))*(x776)*(x816)))+(((x768)*(x795)))+(((x773)*(x812)))+(((x797)*(x809)))+(((x789)*(x800)))+(((gconst49)*(gconst73)*(x774)))+(((x791)*(x802)))+(((IkReal(-1.00000000000000))*(x800)*(x806)))+(((gconst51)*(gconst65)*(x796)))+(((IkReal(-1.00000000000000))*(gconst59)*(x769)*(x789)))+(((IkReal(-1.00000000000000))*(x766)*(x800)))+(((IkReal(-1.00000000000000))*(gconst71)*(x793)*(x804)))+(((IkReal(-1.00000000000000))*(gconst59)*(x769)*(x772)))+(((IkReal(-1.00000000000000))*(x768)*(x773)))+(((IkReal(-1.00000000000000))*(gconst50)*(x764)*(x769)))+(((IkReal(-1.00000000000000))*(gconst66)*(x764)*(x765)))+(((IkReal(-1.00000000000000))*(gconst59)*(x765)*(x786)))+(((IkReal(-1.00000000000000))*(gconst68)*(x782)*(x784))));
op[4]=((((x791)*(x813)))+(((x781)*(x792)))+(((IkReal(-1.00000000000000))*(gconst67)*(x765)*(x791)))+(((x764)*(x790)))+(((x780)*(x812)))+(((x771)*(x805)))+(((IkReal(-1.00000000000000))*(gconst67)*(x765)*(x779)))+(((gconst51)*(gconst65)*(x811)))+(((IkReal(-1.00000000000000))*(gconst60)*(x782)*(x784)))+(((x785)*(x789)))+(((IkReal(-1.00000000000000))*(gconst59)*(x764)*(x765)))+(((gconst50)*(gconst65)*(x766)))+(((IkReal(-1.00000000000000))*(gconst64)*(x788)*(x799)))+(((IkReal(-1.00000000000000))*(gconst55)*(x788)*(x793)))+(((gconst66)*(gconst69)*(x775)))+(((x768)*(x799)))+(((IkReal(-1.00000000000000))*(gconst50)*(x765)*(x786)))+(((IkReal(-1.00000000000000))*(gconst51)*(x769)*(x791)))+(((IkReal(-1.00000000000000))*(gconst50)*(x769)*(x772)))+(((x794)*(x803)))+(((x766)*(x783)))+(((x811)*(x814)))+(((IkReal(-1.00000000000000))*(gconst62)*(x793)*(x804)))+(((IkReal(-1.00000000000000))*(gconst51)*(x765)*(x801)))+(((IkReal(-1.00000000000000))*(gconst59)*(x769)*(x773)))+(((IkReal(-1.00000000000000))*(x795)*(x800)))+(((IkReal(-1.00000000000000))*(gconst66)*(x765)*(x789)))+(((IkReal(-1.00000000000000))*(x768)*(x780)))+(((IkReal(-1.00000000000000))*(x785)*(x806)))+(((IkReal(-1.00000000000000))*(gconst67)*(x769)*(x770)))+(((x774)*(x807)))+(((IkReal(-1.00000000000000))*(gconst51)*(x769)*(x779)))+(((IkReal(-1.00000000000000))*(x778)*(x796)))+(((IkReal(-1.00000000000000))*(x766)*(x785)))+(((IkReal(-1.00000000000000))*(gconst57)*(x784)*(x787)))+(((x772)*(x798)))+(((x772)*(x785)))+(((x779)*(x813)))+(((IkReal(-1.00000000000000))*(gconst66)*(x765)*(x772)))+(((x773)*(x800)))+(((x789)*(x798)))+(((IkReal(-1.00000000000000))*(gconst50)*(x769)*(x789)))+(((x767)*(x786)))+(((IkReal(-1.00000000000000))*(gconst68)*(x766)*(x784)))+(((IkReal(-1.00000000000000))*(x767)*(x805))));
op[5]=((((x773)*(x798)))+(((x774)*(x775)))+(((x773)*(x785)))+(((IkReal(-1.00000000000000))*(x778)*(x811)))+(((x809)*(x814)))+(((x781)*(x808)))+(((IkReal(-1.00000000000000))*(gconst68)*(x784)*(x795)))+(((IkReal(-1.00000000000000))*(gconst60)*(x784)*(x806)))+(((IkReal(-1.00000000000000))*(gconst59)*(x765)*(x772)))+(((x783)*(x795)))+(((IkReal(-1.00000000000000))*(x799)*(x800)))+(((gconst49)*(gconst59)*(x766)))+(((IkReal(-1.00000000000000))*(x767)*(x782)))+(((x778)*(x791)))+(((IkReal(-1.00000000000000))*(gconst50)*(x769)*(x773)))+(((x778)*(x779)))+(((IkReal(-1.00000000000000))*(gconst60)*(x766)*(x784)))+(((IkReal(-1.00000000000000))*(gconst59)*(x769)*(x780)))+(((IkReal(-1.00000000000000))*(gconst55)*(x793)*(x804)))+(((x792)*(x794)))+(((x770)*(x802)))+(((gconst51)*(gconst65)*(x809)))+(((x780)*(x800)))+(((IkReal(-1.00000000000000))*(gconst50)*(x764)*(x765)))+(((x772)*(x790)))+(((gconst49)*(gconst51)*(x796)))+(((IkReal(-1.00000000000000))*(gconst59)*(x765)*(x789)))+(((x771)*(x782)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x776)))+(((IkReal(-1.00000000000000))*(gconst66)*(x765)*(x773)))+(((x764)*(x767)))+(((x789)*(x790))));
op[6]=((((x773)*(x790)))+(((IkReal(-1.00000000000000))*(x785)*(x799)))+(((IkReal(-1.00000000000000))*(gconst51)*(x765)*(x779)))+(((x770)*(x813)))+(((x771)*(x806)))+(((IkReal(-1.00000000000000))*(gconst67)*(x765)*(x770)))+(((IkReal(-1.00000000000000))*(gconst51)*(x765)*(x791)))+(((IkReal(-1.00000000000000))*(gconst50)*(x765)*(x772)))+(((IkReal(-1.00000000000000))*(gconst68)*(x784)*(x799)))+(((gconst53)*(gconst66)*(x775)))+(((IkReal(-1.00000000000000))*(gconst59)*(x765)*(x773)))+(((x767)*(x772)))+(((IkReal(-1.00000000000000))*(gconst50)*(x765)*(x789)))+(((x794)*(x808)))+(((x766)*(x771)))+(((IkReal(-1.00000000000000))*(x766)*(x767)))+(((IkReal(-1.00000000000000))*(gconst66)*(x765)*(x780)))+(((x780)*(x785)))+(((x780)*(x798)))+(((gconst49)*(gconst51)*(x811)))+(((x792)*(x807)))+(((IkReal(-1.00000000000000))*(x778)*(x809)))+(((IkReal(-1.00000000000000))*(gconst51)*(x769)*(x770)))+(((IkReal(-1.00000000000000))*(gconst60)*(x784)*(x795)))+(((IkReal(-1.00000000000000))*(gconst50)*(x769)*(x780)))+(((x767)*(x789)))+(((IkReal(-1.00000000000000))*(x767)*(x806))));
op[7]=((((IkReal(-1.00000000000000))*(gconst50)*(x765)*(x773)))+(((x770)*(x778)))+(((x775)*(x792)))+(((gconst49)*(gconst51)*(x809)))+(((IkReal(-1.00000000000000))*(x767)*(x795)))+(((x767)*(x773)))+(((x780)*(x790)))+(((IkReal(-1.00000000000000))*(gconst60)*(x784)*(x799)))+(((IkReal(-1.00000000000000))*(gconst59)*(x765)*(x780)))+(((x771)*(x795))));
op[8]=((((IkReal(-1.00000000000000))*(gconst51)*(x765)*(x770)))+(((IkReal(-1.00000000000000))*(x767)*(x799)))+(((IkReal(-1.00000000000000))*(gconst50)*(x765)*(x780)))+(((x771)*(x799)))+(((x767)*(x780))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x820=((sj1)*(sj3));
IkReal x821=((IkReal(1.00000000000000))*(cj0));
IkReal x822=((cj2)*(r00));
IkReal x823=((cj3)*(sj0));
IkReal x824=((cj0)*(sj3));
IkReal x825=((cj3)*(sj1));
IkReal x826=((r02)*(sj2));
IkReal x827=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x828=((cj1)*(r00)*(sj2));
IkReal x829=((sj0)*(x826));
IkReal x830=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(sj3)*(x821)))+(((x820)*(x829)))+(((IkReal(-1.00000000000000))*(x827)*(x830)))+(((cj0)*(x825)*(x826)))+(((IkReal(-1.00000000000000))*(cj3)*(x821)*(x828)))+(((IkReal(-1.00000000000000))*(x821)*(x822)*(x825)))+(((IkReal(-1.00000000000000))*(x827)*(x828)))+(((IkReal(-1.00000000000000))*(cj3)*(x821)*(x830)))+(((r01)*(x823)))+(((IkReal(-1.00000000000000))*(sj0)*(x820)*(x822))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(x827)))+(((IkReal(-1.00000000000000))*(x820)*(x821)*(x826)))+(((x824)*(x830)))+(((IkReal(-1.00000000000000))*(x823)*(x828)))+(((x824)*(x828)))+(((IkReal(-1.00000000000000))*(x823)*(x830)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x821)))+(((sj1)*(x823)*(x826)))+(((cj0)*(x820)*(x822)))+(((IkReal(-1.00000000000000))*(sj1)*(x822)*(x823))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(sj3)*(x821)))+(((x820)*(x829)))+(((IkReal(-1.00000000000000))*(x827)*(x830)))+(((cj0)*(x825)*(x826)))+(((IkReal(-1.00000000000000))*(cj3)*(x821)*(x828)))+(((IkReal(-1.00000000000000))*(x821)*(x822)*(x825)))+(((IkReal(-1.00000000000000))*(x827)*(x828)))+(((IkReal(-1.00000000000000))*(cj3)*(x821)*(x830)))+(((r01)*(x823)))+(((IkReal(-1.00000000000000))*(sj0)*(x820)*(x822)))))), ((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(x827)))+(((IkReal(-1.00000000000000))*(x820)*(x821)*(x826)))+(((x824)*(x830)))+(((IkReal(-1.00000000000000))*(x823)*(x828)))+(((x824)*(x828)))+(((IkReal(-1.00000000000000))*(x823)*(x830)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x821)))+(((sj1)*(x823)*(x826)))+(((cj0)*(x820)*(x822)))+(((IkReal(-1.00000000000000))*(sj1)*(x822)*(x823)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x831=IKcos(j5);
IkReal x832=IKsin(j5);
IkReal x833=((IkReal(1.00000000000000))*(cj0));
IkReal x834=((IkReal(1.00000000000000))*(sj0));
IkReal x835=((r02)*(sj1)*(sj2));
IkReal x836=((IkReal(1.00000000000000))*(x831));
IkReal x837=((cj1)*(cj2)*(r02));
IkReal x838=((cj2)*(r00)*(sj1));
IkReal x839=((cj1)*(r00)*(sj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x833)*(x835)))+(((cj0)*(x839)))+(((cj0)*(x838)))+(((cj0)*(x837)))+(((cj3)*(x832)))+(((IkReal(-1.00000000000000))*(r01)*(x834)))+(((IkReal(-1.00000000000000))*(sj3)*(x836))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj3)*(x832)))+(((IkReal(-1.00000000000000))*(x834)*(x839)))+(((IkReal(-1.00000000000000))*(x834)*(x838)))+(((IkReal(-1.00000000000000))*(x834)*(x837)))+(((IkReal(-1.00000000000000))*(r01)*(x833)))+(((IkReal(-1.00000000000000))*(cj3)*(x836)))+(((sj0)*(x835))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x840=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x841=((cj1)*(x840));
IkReal x842=((IkReal(1.00000000000000))*(r02));
IkReal x843=((sj1)*(x840));
sj5array[0]=((((IkReal(-1.00000000000000))*(sj2)*(x841)*(x842)))+(((IkReal(-1.00000000000000))*(cj2)*(x842)*(x843)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x843)))+(((cj2)*(r00)*(x841))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x844=((cj0)*(cj5));
IkReal x845=((IkReal(1.00000000000000))*(r01));
IkReal x846=((cj4)*(sj0)*(sj5));
IkReal x847=((cj2)*(r00)*(sj1));
IkReal x848=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x849=((cj0)*(cj4)*(sj5));
IkReal x850=((cj1)*(r00)*(sj2));
IkReal x851=((cj1)*(cj2)*(r02));
IkReal x852=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x846)*(x852)))+(((IkReal(-1.00000000000000))*(x844)*(x852)))+(((x846)*(x847)))+(((x844)*(x850)))+(((x844)*(x851)))+(((r01)*(x849)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x845)))+(((x846)*(x851)))+(((x846)*(x850)))+(((x844)*(x847))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x847)*(x848)))+(((IkReal(-1.00000000000000))*(x849)*(x852)))+(((IkReal(-1.00000000000000))*(x848)*(x851)))+(((IkReal(-1.00000000000000))*(x848)*(x850)))+(((IkReal(-1.00000000000000))*(x844)*(x845)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x845)*(x846)))+(((x847)*(x849)))+(((x849)*(x851)))+(((x849)*(x850))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x846)*(x852)))+(((IkReal(-1.00000000000000))*(x844)*(x852)))+(((x846)*(x847)))+(((x844)*(x850)))+(((x844)*(x851)))+(((r01)*(x849)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x845)))+(((x846)*(x851)))+(((x846)*(x850)))+(((x844)*(x847)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x847)*(x848)))+(((IkReal(-1.00000000000000))*(x849)*(x852)))+(((IkReal(-1.00000000000000))*(x848)*(x851)))+(((IkReal(-1.00000000000000))*(x848)*(x850)))+(((IkReal(-1.00000000000000))*(x844)*(x845)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x845)*(x846)))+(((x847)*(x849)))+(((x849)*(x851)))+(((x849)*(x850)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x853=IKsin(j3);
IkReal x854=IKcos(j3);
IkReal x855=((IkReal(1.00000000000000))*(cj0));
IkReal x856=((cj0)*(cj1));
IkReal x857=((cj2)*(r02));
IkReal x858=((r00)*(sj2));
IkReal x859=((IkReal(1.00000000000000))*(sj0));
IkReal x860=((cj4)*(sj5));
IkReal x861=((r02)*(sj1)*(sj2));
IkReal x862=((cj2)*(r00)*(sj1));
IkReal x863=((IkReal(1.00000000000000))*(x854));
evalcond[0]=((((cj0)*(x862)))+(((IkReal(-1.00000000000000))*(r01)*(x859)))+(((IkReal(-1.00000000000000))*(x860)*(x863)))+(((x856)*(x858)))+(((x856)*(x857)))+(((IkReal(-1.00000000000000))*(cj5)*(x853)))+(((IkReal(-1.00000000000000))*(x855)*(x861))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x855)))+(((IkReal(-1.00000000000000))*(cj1)*(x858)*(x859)))+(((sj0)*(x861)))+(((IkReal(-1.00000000000000))*(cj1)*(x857)*(x859)))+(((x853)*(x860)))+(((IkReal(-1.00000000000000))*(x859)*(x862)))+(((IkReal(-1.00000000000000))*(cj5)*(x863))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x864=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x864)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x864)))+(((cj1)*(cj2)*(r00))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x865=((cj0)*(cj5));
IkReal x866=((IkReal(1.00000000000000))*(r01));
IkReal x867=((cj4)*(sj0)*(sj5));
IkReal x868=((cj2)*(r00)*(sj1));
IkReal x869=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x870=((cj0)*(cj4)*(sj5));
IkReal x871=((cj1)*(r00)*(sj2));
IkReal x872=((cj1)*(cj2)*(r02));
IkReal x873=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x867)*(x872)))+(((x867)*(x871)))+(((r01)*(x870)))+(((x867)*(x868)))+(((IkReal(-1.00000000000000))*(x867)*(x873)))+(((x865)*(x868)))+(((x865)*(x872)))+(((x865)*(x871)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x866)))+(((IkReal(-1.00000000000000))*(x865)*(x873))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x865)*(x866)))+(((x868)*(x870)))+(((x870)*(x872)))+(((x870)*(x871)))+(((IkReal(-1.00000000000000))*(x866)*(x867)))+(((IkReal(-1.00000000000000))*(x869)*(x871)))+(((IkReal(-1.00000000000000))*(x869)*(x872)))+(((IkReal(-1.00000000000000))*(x868)*(x869)))+(((IkReal(-1.00000000000000))*(x870)*(x873)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x867)*(x872)))+(((x867)*(x871)))+(((r01)*(x870)))+(((x867)*(x868)))+(((IkReal(-1.00000000000000))*(x867)*(x873)))+(((x865)*(x868)))+(((x865)*(x872)))+(((x865)*(x871)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x866)))+(((IkReal(-1.00000000000000))*(x865)*(x873)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x865)*(x866)))+(((x868)*(x870)))+(((x870)*(x872)))+(((x870)*(x871)))+(((IkReal(-1.00000000000000))*(x866)*(x867)))+(((IkReal(-1.00000000000000))*(x869)*(x871)))+(((IkReal(-1.00000000000000))*(x869)*(x872)))+(((IkReal(-1.00000000000000))*(x868)*(x869)))+(((IkReal(-1.00000000000000))*(x870)*(x873)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x874=IKsin(j3);
IkReal x875=IKcos(j3);
IkReal x876=((IkReal(1.00000000000000))*(cj0));
IkReal x877=((cj0)*(cj1));
IkReal x878=((cj2)*(r02));
IkReal x879=((r00)*(sj2));
IkReal x880=((IkReal(1.00000000000000))*(sj0));
IkReal x881=((cj4)*(sj5));
IkReal x882=((r02)*(sj1)*(sj2));
IkReal x883=((cj2)*(r00)*(sj1));
IkReal x884=((IkReal(1.00000000000000))*(x875));
evalcond[0]=((((cj0)*(x883)))+(((IkReal(-1.00000000000000))*(x876)*(x882)))+(((IkReal(-1.00000000000000))*(x881)*(x884)))+(((IkReal(-1.00000000000000))*(cj5)*(x874)))+(((IkReal(-1.00000000000000))*(r01)*(x880)))+(((x877)*(x878)))+(((x877)*(x879))));
evalcond[1]=((((x874)*(x881)))+(((sj0)*(x882)))+(((IkReal(-1.00000000000000))*(r01)*(x876)))+(((IkReal(-1.00000000000000))*(cj1)*(x879)*(x880)))+(((IkReal(-1.00000000000000))*(cj1)*(x878)*(x880)))+(((IkReal(-1.00000000000000))*(cj5)*(x884)))+(((IkReal(-1.00000000000000))*(x880)*(x883))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x885=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x886=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x886)))+(((IkReal(-1.00000000000000))*(x885))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x885)))+(x886));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x887=((cj0)*(py));
IkReal x888=((px)*(sj0));
IkReal x889=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-1.00000000000000))*(x887)*(x889)))+(((IkReal(-331.000000000000))*(pz)*(sj2)))+(((IkReal(405.000000000000))*(x888)))+(((x888)*(x889)))+(((IkReal(-405.000000000000))*(x887)))));
IkReal x890=((px)*(sj0));
IkReal x891=((cj0)*(py));
dummyeval[0]=((((cj2)*(x890)))+(((IkReal(-1.00000000000000))*(cj2)*(x891)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(-1.22356495468278))*(x891)))+(((IkReal(1.22356495468278))*(x890))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x892=((IkReal(331.000000000000))*(sj2));
IkReal x893=((cj0)*(py));
IkReal x894=((px)*(sj0));
IkReal x895=((IkReal(1000.00000000000))*(pz));
IkReal x896=((cj2)*(pz));
IkReal x897=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((x894)*(x897)))+(((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x897))))) != 0)?((IkReal)1/(((((x894)*(x897)))+(((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x897)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x892)*(x893)))+(((IkReal(331.000000000000))*(x896)))+(((IkReal(405.000000000000))*(pz)))+(((x892)*(x894))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x892)*(x893)))+(((IkReal(331.000000000000))*(x896)))+(((IkReal(405.000000000000))*(pz)))+(((x892)*(x894)))))):(IkReal)1.0e30))*(((((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x893)*(x895)))+(((IkReal(-1.00000000000000))*(x894)*(x895))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((x894)*(x897)))+(((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x897))))) != 0)?((IkReal)1/(((((x894)*(x897)))+(((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x897)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x892)*(x893)))+(((IkReal(331.000000000000))*(x896)))+(((IkReal(405.000000000000))*(pz)))+(((x892)*(x894))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x892)*(x893)))+(((IkReal(331.000000000000))*(x896)))+(((IkReal(405.000000000000))*(pz)))+(((x892)*(x894)))))):(IkReal)1.0e30))*(((((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x893)*(x895)))+(((IkReal(-1.00000000000000))*(x894)*(x895)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((x894)*(x897)))+(((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x897))))) != 0)?((IkReal)1/(((((x894)*(x897)))+(((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x896)))+(((IkReal(-1.00000000000000))*(x893)*(x897)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(-1.00000000000000))*(x892)*(x893)))+(((IkReal(331.000000000000))*(x896)))+(((IkReal(405.000000000000))*(pz)))+(((x892)*(x894))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x892)*(x893)))+(((IkReal(331.000000000000))*(x896)))+(((IkReal(405.000000000000))*(pz)))+(((x892)*(x894)))))):(IkReal)1.0e30))*(((((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x893)*(x895)))+(((IkReal(-1.00000000000000))*(x894)*(x895)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x898=IKsin(j1);
IkReal x899=IKcos(j1);
IkReal x900=((IkReal(0.331000000000000))*(sj2));
IkReal x901=((cj0)*(py));
IkReal x902=((px)*(sj0));
IkReal x903=((IkReal(0.331000000000000))*(cj2));
IkReal x904=((IkReal(0.810000000000000))*(x899));
IkReal x905=((pz)*(x898));
evalcond[0]=((((x899)*(x900)))+(pz)+(((IkReal(0.405000000000000))*(x898)))+(((x898)*(x903))));
evalcond[1]=((((pz)*(x899)))+(((IkReal(-1.00000000000000))*(x898)*(x902)))+(((x898)*(x901)))+(x900));
evalcond[2]=((IkReal(0.405000000000000))+(((x899)*(x902)))+(((IkReal(-1.00000000000000))*(x899)*(x901)))+(x905)+(x903));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x902)*(x904)))+(((x901)*(x904)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x905))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x898)*(x900)))+(((x899)*(x903)))+(((IkReal(0.405000000000000))*(x899)))+(((IkReal(-1.00000000000000))*(x901)))+(x902));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x906=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x906)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x906)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x907=((sj0)*(sj1));
IkReal x908=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x907)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x908)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x908)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x907))));
IkReal gconst6;
IkReal x909=((sj0)*(sj1));
IkReal x910=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x909)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x910)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x910)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x909))));
IkReal gconst7;
IkReal x911=((cj0)*(r02));
IkReal x912=((cj0)*(r00));
gconst7=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x911)))+(((cj1)*(sj2)*(x912)))+(((cj1)*(cj2)*(x911)))+(((cj2)*(sj1)*(x912))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x913=((cj0)*(r02));
IkReal x914=((cj0)*(r00));
gconst9=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x913)))+(((cj1)*(sj2)*(x914)))+(((cj1)*(cj2)*(x913)))+(((cj2)*(sj1)*(x914))));
IkReal gconst10;
IkReal x915=((sj0)*(sj1));
IkReal x916=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x915)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x916)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x916)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x915))));
IkReal gconst11;
IkReal x917=((sj0)*(sj1));
IkReal x918=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x917)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x918)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x918)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x917))));
IkReal gconst12;
IkReal x919=((cj0)*(r02));
IkReal x920=((cj0)*(r00));
gconst12=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x920)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x919)))+(((cj1)*(sj2)*(x920)))+(((cj1)*(cj2)*(x919))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x921=((cj0)*(r02));
IkReal x922=((cj0)*(r00));
gconst14=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x921)))+(((cj2)*(sj1)*(x922)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x921)))+(((cj1)*(sj2)*(x922))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x923=((sj0)*(sj1));
IkReal x924=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x924)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x923)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x924)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x923))));
IkReal gconst22;
IkReal x925=((sj0)*(sj1));
IkReal x926=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x926)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x925)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x926)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x925))));
IkReal gconst23;
IkReal x927=((cj0)*(r02));
IkReal x928=((cj0)*(r00));
gconst23=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x927)))+(((cj2)*(sj1)*(x928)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x927)))+(((cj1)*(sj2)*(x928))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x929=((cj0)*(r02));
IkReal x930=((cj0)*(r00));
gconst25=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x929)))+(((cj2)*(sj1)*(x930)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x929)))+(((cj1)*(sj2)*(x930))));
IkReal gconst26;
IkReal x931=((sj0)*(sj1));
IkReal x932=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x932)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x931)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x932)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x931))));
IkReal gconst27;
IkReal x933=((sj0)*(sj1));
IkReal x934=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x934)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x933)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x934)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x933))));
IkReal gconst28;
IkReal x935=((cj0)*(r02));
IkReal x936=((cj0)*(r00));
gconst28=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x935)))+(((cj2)*(sj1)*(x936)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x935)))+(((cj1)*(sj2)*(x936))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x937=((cj0)*(r02));
IkReal x938=((cj0)*(r00));
gconst30=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x937)))+(((cj2)*(sj1)*(x938)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x937)))+(((cj1)*(sj2)*(x938))));
IkReal x939=((gconst27)*(gconst28));
IkReal x940=((gconst22)*(gconst23));
IkReal x941=((gconst21)*(gconst25));
IkReal x942=((gconst26)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x941)*(x942)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((IkReal(-1.00000000000000))*(x939)*(x940)))+(((x939)*(x941)))+(((x940)*(x942))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x943=((IkReal(1.00000000000000))*(gconst19));
IkReal x944=((gconst23)*(gconst27));
IkReal x945=((gconst15)*(gconst18));
IkReal x946=((gconst11)*(gconst6));
IkReal x947=((gconst13)*(gconst26));
IkReal x948=((gconst15)*(gconst9));
IkReal x949=((gconst6)*(gconst8));
IkReal x950=((gconst18)*(gconst28));
IkReal x951=((gconst22)*(gconst30));
IkReal x952=((gconst16)*(gconst26));
IkReal x953=((IkReal(1.00000000000000))*(gconst10));
IkReal x954=((gconst14)*(gconst25));
IkReal x955=((IkReal(1.00000000000000))*(gconst26));
IkReal x956=((gconst12)*(gconst27));
IkReal x957=((gconst25)*(gconst5));
IkReal x958=((gconst17)*(gconst28));
IkReal x959=((gconst21)*(gconst27));
IkReal x960=((gconst27)*(gconst6));
IkReal x961=((IkReal(1.00000000000000))*(gconst28));
IkReal x962=((gconst11)*(gconst5));
IkReal x963=((gconst17)*(gconst21));
IkReal x964=((gconst18)*(gconst19));
IkReal x965=((gconst10)*(gconst7));
IkReal x966=((gconst30)*(gconst6));
IkReal x967=((gconst17)*(gconst5));
IkReal x968=((IkReal(1.00000000000000))*(gconst12));
IkReal x969=((gconst22)*(gconst7));
IkReal x970=((gconst24)*(gconst6));
IkReal x971=((gconst5)*(gconst9));
IkReal x972=((gconst15)*(gconst17));
IkReal x973=((gconst27)*(gconst28));
IkReal x974=((gconst23)*(gconst26));
IkReal x975=((gconst21)*(gconst9));
IkReal x976=((gconst22)*(gconst8));
IkReal x977=((gconst10)*(gconst23));
IkReal x978=((gconst12)*(gconst18));
IkReal x979=((gconst21)*(gconst25));
IkReal x980=((gconst11)*(gconst28));
IkReal x981=((gconst11)*(gconst19));
IkReal x982=((gconst19)*(gconst27));
IkReal x983=((gconst20)*(gconst6));
IkReal x984=((gconst10)*(gconst16));
IkReal x985=((gconst26)*(gconst29));
IkReal x986=((gconst26)*(gconst7));
IkReal x987=((gconst20)*(gconst22));
IkReal x988=((gconst22)*(gconst24));
IkReal x989=((gconst15)*(gconst25));
IkReal x990=((gconst10)*(gconst13));
IkReal x991=((gconst14)*(gconst6));
IkReal x992=((gconst16)*(gconst22));
IkReal x993=((gconst11)*(gconst12));
IkReal x994=((gconst14)*(gconst22));
IkReal x995=((gconst17)*(gconst19));
IkReal x996=((gconst10)*(gconst29));
IkReal x997=((gconst28)*(gconst9));
IkReal x998=((IkReal(1.00000000000000))*(gconst6));
IkReal x999=((gconst26)*(x983));
IkReal x1000=((gconst9)*(x981));
IkReal x1001=((gconst11)*(x992));
IkReal x1002=((gconst14)*(x977));
IkReal x1003=((gconst11)*(gconst22)*(gconst23));
op[0]=((((IkReal(-1.00000000000000))*(gconst30)*(x955)*(x979)))+(((IkReal(-1.00000000000000))*(gconst22)*(x944)*(x961)))+(((gconst25)*(gconst28)*(x959)))+(((IkReal(-1.00000000000000))*(gconst29)*(x955)*(x988)))+(((x951)*(x974))));
op[1]=((((IkReal(-1.00000000000000))*(gconst22)*(x943)*(x944)))+(((x951)*(x952)))+(((x950)*(x988)))+(((IkReal(-1.00000000000000))*(gconst27)*(x961)*(x992)))+(((gconst19)*(gconst25)*(x959)))+(((IkReal(-1.00000000000000))*(gconst30)*(x955)*(x963)))+(((IkReal(-1.00000000000000))*(gconst20)*(x955)*(x979)))+(((x958)*(x959)))+(((x974)*(x987)))+(((x985)*(x989))));
op[2]=((((x972)*(x985)))+(((IkReal(-1.00000000000000))*(gconst30)*(x955)*(x975)))+(((x959)*(x997)))+(((x959)*(x995)))+(((IkReal(-1.00000000000000))*(gconst27)*(x943)*(x992)))+(((IkReal(-1.00000000000000))*(gconst30)*(x955)*(x957)))+(((IkReal(-1.00000000000000))*(x947)*(x988)))+(((IkReal(-1.00000000000000))*(gconst22)*(x944)*(x968)))+(((x979)*(x980)))+(((IkReal(-1.00000000000000))*(gconst6)*(x944)*(x961)))+(((IkReal(-1.00000000000000))*(gconst27)*(x961)*(x969)))+(((x974)*(x994)))+(((x966)*(x974)))+(((IkReal(-1.00000000000000))*(gconst21)*(x954)*(x955)))+(((IkReal(-1.00000000000000))*(gconst29)*(x953)*(x988)))+(((IkReal(-1.00000000000000))*(gconst29)*(x955)*(x976)))+(((IkReal(-1.00000000000000))*(gconst29)*(x955)*(x970)))+(((x956)*(x979)))+(((x964)*(x988)))+(((x957)*(x973)))+(((IkReal(-1.00000000000000))*(gconst30)*(x953)*(x979)))+(((IkReal(-1.00000000000000))*(gconst20)*(x955)*(x963)))+(((x952)*(x987)))+(((x951)*(x977)))+(((IkReal(-1.00000000000000))*(gconst25)*(x945)*(x961)))+(((IkReal(-1.00000000000000))*(x1003)*(x961)))+(((x951)*(x986))));
op[3]=((((x952)*(x966)))+(((x950)*(x976)))+(((x950)*(x970)))+(((x989)*(x996)))+(((x977)*(x987)))+(((gconst19)*(gconst9)*(x959)))+(((IkReal(-1.00000000000000))*(gconst30)*(x955)*(x967)))+(((x948)*(x985)))+(((x952)*(x994)))+(((IkReal(-1.00000000000000))*(x945)*(x958)))+(((IkReal(-1.00000000000000))*(gconst27)*(x943)*(x969)))+(((IkReal(-1.00000000000000))*(gconst14)*(x955)*(x963)))+(((x979)*(x981)))+(((IkReal(-1.00000000000000))*(gconst20)*(x955)*(x975)))+(((gconst20)*(gconst26)*(x969)))+(((x947)*(x989)))+(((IkReal(-1.00000000000000))*(gconst25)*(x943)*(x945)))+(((IkReal(-1.00000000000000))*(x956)*(x992)))+(((x956)*(x963)))+(((IkReal(-1.00000000000000))*(x1003)*(x943)))+(((x974)*(x983)))+(((x957)*(x982)))+(((IkReal(-1.00000000000000))*(gconst20)*(x955)*(x957)))+(((IkReal(-1.00000000000000))*(gconst6)*(x943)*(x944)))+(((gconst27)*(gconst5)*(x958)))+(((IkReal(-1.00000000000000))*(x1001)*(x961)))+(((IkReal(-1.00000000000000))*(gconst20)*(x953)*(x979)))+(((gconst11)*(gconst21)*(x958)))+(((IkReal(-1.00000000000000))*(gconst30)*(x953)*(x963)))+(((x951)*(x984)))+(((IkReal(-1.00000000000000))*(gconst16)*(x960)*(x961)))+(((x978)*(x988))));
op[4]=((((IkReal(-1.00000000000000))*(gconst11)*(x961)*(x969)))+(((IkReal(-1.00000000000000))*(gconst30)*(x953)*(x957)))+(((IkReal(-1.00000000000000))*(gconst30)*(x955)*(x971)))+(((IkReal(-1.00000000000000))*(gconst20)*(x953)*(x963)))+(((x977)*(x994)))+(((x951)*(x965)))+(((IkReal(-1.00000000000000))*(gconst9)*(x945)*(x961)))+(((x975)*(x980)))+(((IkReal(-1.00000000000000))*(gconst29)*(x949)*(x955)))+(((IkReal(-1.00000000000000))*(x947)*(x976)))+(((IkReal(-1.00000000000000))*(x947)*(x970)))+(((IkReal(-1.00000000000000))*(x956)*(x969)))+(((IkReal(-1.00000000000000))*(gconst13)*(x953)*(x988)))+(((IkReal(-1.00000000000000))*(gconst14)*(x955)*(x975)))+(((x972)*(x996)))+(((IkReal(-1.00000000000000))*(gconst17)*(x943)*(x945)))+(((x947)*(x972)))+(((x963)*(x981)))+(((IkReal(-1.00000000000000))*(gconst6)*(x944)*(x968)))+(((x956)*(x957)))+(((x974)*(x991)))+(((x966)*(x977)))+(((x966)*(x986)))+(((IkReal(-1.00000000000000))*(gconst29)*(x953)*(x976)))+(((IkReal(-1.00000000000000))*(gconst29)*(x953)*(x970)))+(((x967)*(x982)))+(((x971)*(x973)))+(((x956)*(x975)))+(((x957)*(x980)))+(((IkReal(-1.00000000000000))*(gconst7)*(x960)*(x961)))+(((x984)*(x987)))+(((IkReal(-1.00000000000000))*(gconst16)*(x943)*(x960)))+(((x964)*(x976)))+(((x964)*(x970)))+(((IkReal(-1.00000000000000))*(gconst30)*(x953)*(x975)))+(((IkReal(-1.00000000000000))*(gconst20)*(x955)*(x967)))+(((IkReal(-1.00000000000000))*(gconst23)*(x946)*(x961)))+(((IkReal(-1.00000000000000))*(x1001)*(x943)))+(((gconst14)*(gconst26)*(x969)))+(((IkReal(-1.00000000000000))*(gconst5)*(x954)*(x955)))+(((x952)*(x983)))+(((IkReal(-1.00000000000000))*(gconst25)*(x945)*(x968)))+(((x979)*(x993)))+(((IkReal(-1.00000000000000))*(x1003)*(x968)))+(((IkReal(-1.00000000000000))*(gconst21)*(x953)*(x954))));
op[5]=((((x983)*(x986)))+(((IkReal(-1.00000000000000))*(gconst9)*(x943)*(x945)))+(((x989)*(x990)))+(((IkReal(-1.00000000000000))*(gconst16)*(x956)*(x998)))+(((x975)*(x981)))+(((x963)*(x993)))+(((x947)*(x948)))+(((x977)*(x983)))+(((x949)*(x950)))+(((x952)*(x991)))+(((x965)*(x987)))+(((IkReal(-1.00000000000000))*(gconst14)*(x955)*(x967)))+(((x976)*(x978)))+(((IkReal(-1.00000000000000))*(gconst7)*(x943)*(x960)))+(((IkReal(-1.00000000000000))*(gconst20)*(x955)*(x971)))+(((IkReal(-1.00000000000000))*(gconst17)*(x945)*(x968)))+(((x966)*(x984)))+(((IkReal(-1.00000000000000))*(gconst23)*(x943)*(x946)))+(((x956)*(x967)))+(((x957)*(x981)))+(((x971)*(x982)))+(((x948)*(x996)))+(((x958)*(x962)))+(((IkReal(-1.00000000000000))*(gconst16)*(x946)*(x961)))+(((IkReal(-1.00000000000000))*(gconst11)*(x943)*(x969)))+(((x970)*(x978)))+(((IkReal(-1.00000000000000))*(x1001)*(x968)))+(((IkReal(-1.00000000000000))*(gconst14)*(x953)*(x963)))+(((IkReal(-1.00000000000000))*(gconst20)*(x953)*(x975)))+(((IkReal(-1.00000000000000))*(gconst30)*(x953)*(x967)))+(((x984)*(x994)))+(((IkReal(-1.00000000000000))*(gconst20)*(x953)*(x957))));
op[6]=((((IkReal(-1.00000000000000))*(gconst16)*(x943)*(x946)))+(((x983)*(x984)))+(((IkReal(-1.00000000000000))*(gconst20)*(x953)*(x967)))+(((IkReal(-1.00000000000000))*(gconst7)*(x956)*(x998)))+(((x977)*(x991)))+(((IkReal(-1.00000000000000))*(gconst9)*(x945)*(x968)))+(((x986)*(x991)))+(((IkReal(-1.00000000000000))*(x947)*(x949)))+(((x975)*(x993)))+(((IkReal(-1.00000000000000))*(gconst29)*(x949)*(x953)))+(((x965)*(x994)))+(((IkReal(-1.00000000000000))*(gconst7)*(x946)*(x961)))+(((IkReal(-1.00000000000000))*(gconst14)*(x955)*(x971)))+(((x972)*(x990)))+(((x949)*(x964)))+(((IkReal(-1.00000000000000))*(gconst13)*(x953)*(x970)))+(((IkReal(-1.00000000000000))*(gconst13)*(x953)*(x976)))+(((x962)*(x997)))+(((x962)*(x995)))+(((x965)*(x966)))+(((IkReal(-1.00000000000000))*(gconst11)*(x968)*(x969)))+(((x956)*(x971)))+(((IkReal(-1.00000000000000))*(gconst5)*(x953)*(x954)))+(((IkReal(-1.00000000000000))*(gconst30)*(x953)*(x971)))+(((IkReal(-1.00000000000000))*(gconst14)*(x953)*(x975)))+(((IkReal(-1.00000000000000))*(gconst23)*(x946)*(x968)))+(((x957)*(x993))));
op[7]=((((gconst19)*(gconst9)*(x962)))+(((x965)*(x983)))+(((x949)*(x978)))+(((gconst12)*(gconst17)*(x962)))+(((x948)*(x990)))+(((IkReal(-1.00000000000000))*(gconst16)*(x946)*(x968)))+(((IkReal(-1.00000000000000))*(gconst14)*(x953)*(x967)))+(((IkReal(-1.00000000000000))*(gconst7)*(x943)*(x946)))+(((IkReal(-1.00000000000000))*(gconst20)*(x953)*(x971)))+(((x984)*(x991))));
op[8]=((((gconst12)*(gconst9)*(x962)))+(((x965)*(x991)))+(((IkReal(-1.00000000000000))*(gconst7)*(x946)*(x968)))+(((IkReal(-1.00000000000000))*(gconst14)*(x953)*(x971)))+(((IkReal(-1.00000000000000))*(gconst13)*(x949)*(x953))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1004=((r01)*(sj3));
IkReal x1005=((IkReal(1.00000000000000))*(sj0));
IkReal x1006=((sj1)*(sj3));
IkReal x1007=((cj2)*(r00));
IkReal x1008=((r02)*(sj2));
IkReal x1009=((cj3)*(sj1));
IkReal x1010=((IkReal(1.00000000000000))*(cj0));
IkReal x1011=((cj3)*(r01));
IkReal x1012=((cj1)*(r00)*(sj2));
IkReal x1013=((cj1)*(cj2)*(r02)*(sj3));
IkReal x1014=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((sj0)*(x1013)))+(((IkReal(-1.00000000000000))*(x1005)*(x1011)))+(((cj0)*(x1014)))+(((cj0)*(x1007)*(x1009)))+(((cj0)*(x1004)))+(((cj0)*(cj3)*(x1012)))+(((IkReal(-1.00000000000000))*(x1008)*(x1009)*(x1010)))+(((IkReal(-1.00000000000000))*(x1005)*(x1006)*(x1008)))+(((sj0)*(x1006)*(x1007)))+(((sj0)*(sj3)*(x1012))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x1005)*(x1007)*(x1009)))+(((cj0)*(sj3)*(x1012)))+(((IkReal(-1.00000000000000))*(x1010)*(x1011)))+(((IkReal(-1.00000000000000))*(x1006)*(x1008)*(x1010)))+(((IkReal(-1.00000000000000))*(x1005)*(x1014)))+(((sj0)*(x1008)*(x1009)))+(((cj0)*(x1006)*(x1007)))+(((cj0)*(x1013)))+(((IkReal(-1.00000000000000))*(x1004)*(x1005)))+(((IkReal(-1.00000000000000))*(cj3)*(x1005)*(x1012))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((sj0)*(x1013)))+(((IkReal(-1.00000000000000))*(x1005)*(x1011)))+(((cj0)*(x1014)))+(((cj0)*(x1007)*(x1009)))+(((cj0)*(x1004)))+(((cj0)*(cj3)*(x1012)))+(((IkReal(-1.00000000000000))*(x1008)*(x1009)*(x1010)))+(((IkReal(-1.00000000000000))*(x1005)*(x1006)*(x1008)))+(((sj0)*(x1006)*(x1007)))+(((sj0)*(sj3)*(x1012)))))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x1005)*(x1007)*(x1009)))+(((cj0)*(sj3)*(x1012)))+(((IkReal(-1.00000000000000))*(x1010)*(x1011)))+(((IkReal(-1.00000000000000))*(x1006)*(x1008)*(x1010)))+(((IkReal(-1.00000000000000))*(x1005)*(x1014)))+(((sj0)*(x1008)*(x1009)))+(((cj0)*(x1006)*(x1007)))+(((cj0)*(x1013)))+(((IkReal(-1.00000000000000))*(x1004)*(x1005)))+(((IkReal(-1.00000000000000))*(cj3)*(x1005)*(x1012)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1015=IKsin(j5);
IkReal x1016=IKcos(j5);
IkReal x1017=((IkReal(1.00000000000000))*(cj0));
IkReal x1018=((IkReal(1.00000000000000))*(cj3));
IkReal x1019=((cj0)*(cj1));
IkReal x1020=((cj2)*(r02));
IkReal x1021=((r00)*(sj2));
IkReal x1022=((IkReal(1.00000000000000))*(sj0));
IkReal x1023=((r02)*(sj1)*(sj2));
IkReal x1024=((cj2)*(r00)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1015)*(x1018)))+(((cj0)*(x1024)))+(((x1019)*(x1021)))+(((x1019)*(x1020)))+(((IkReal(-1.00000000000000))*(r01)*(x1022)))+(((IkReal(-1.00000000000000))*(x1017)*(x1023)))+(((IkReal(-1.00000000000000))*(sj3)*(x1016))));
evalcond[1]=((((sj0)*(x1023)))+(((IkReal(-1.00000000000000))*(x1022)*(x1024)))+(((IkReal(-1.00000000000000))*(x1016)*(x1018)))+(((sj3)*(x1015)))+(((IkReal(-1.00000000000000))*(r01)*(x1017)))+(((IkReal(-1.00000000000000))*(cj1)*(x1021)*(x1022)))+(((IkReal(-1.00000000000000))*(cj1)*(x1020)*(x1022))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x1025=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1025)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1025)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x1026=((sj0)*(sj1));
IkReal x1027=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1026)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1027)))+(((r02)*(sj2)*(x1026)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1027))));
IkReal gconst49;
IkReal x1028=((sj0)*(sj1));
IkReal x1029=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1028)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1029)))+(((r02)*(sj2)*(x1028)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1029))));
IkReal gconst50;
IkReal x1030=((cj0)*(r02));
IkReal x1031=((cj0)*(r00));
gconst50=((((cj1)*(sj2)*(x1031)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1030)))+(((cj1)*(cj2)*(x1030)))+(((cj2)*(sj1)*(x1031))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x1032=((cj0)*(r02));
IkReal x1033=((cj0)*(r00));
gconst52=((((cj1)*(sj2)*(x1033)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1032)))+(((cj1)*(cj2)*(x1032)))+(((cj2)*(sj1)*(x1033))));
IkReal gconst53;
IkReal x1034=((sj0)*(sj1));
IkReal x1035=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1035)))+(((r02)*(sj2)*(x1034)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1034)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1035))));
IkReal gconst54;
IkReal x1036=((sj0)*(sj1));
IkReal x1037=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1037)))+(((r02)*(sj2)*(x1036)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1036)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1037))));
IkReal gconst55;
IkReal x1038=((cj0)*(r02));
IkReal x1039=((cj0)*(r00));
gconst55=((((cj1)*(sj2)*(x1039)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1038)))+(((cj1)*(cj2)*(x1038)))+(((cj2)*(sj1)*(x1039))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x1040=((cj0)*(r02));
IkReal x1041=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1040)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1041)))+(((cj1)*(cj2)*(x1040)))+(((cj2)*(sj1)*(x1041))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x1042=((sj0)*(sj1));
IkReal x1043=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x1042)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1043)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1043)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1042))));
IkReal gconst65;
IkReal x1044=((sj0)*(sj1));
IkReal x1045=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x1044)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1045)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1045)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1044))));
IkReal gconst66;
IkReal x1046=((cj0)*(r02));
IkReal x1047=((cj0)*(r00));
gconst66=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1046)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1047)))+(((cj1)*(cj2)*(x1046)))+(((cj2)*(sj1)*(x1047))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x1048=((cj0)*(r02));
IkReal x1049=((cj0)*(r00));
gconst68=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1048)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1049)))+(((cj1)*(cj2)*(x1048)))+(((cj2)*(sj1)*(x1049))));
IkReal gconst69;
IkReal x1050=((sj0)*(sj1));
IkReal x1051=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1051)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1051)))+(((r02)*(sj2)*(x1050)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1050))));
IkReal gconst70;
IkReal x1052=((sj0)*(sj1));
IkReal x1053=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1053)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1053)))+(((r02)*(sj2)*(x1052)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1052))));
IkReal gconst71;
IkReal x1054=((cj0)*(r02));
IkReal x1055=((cj0)*(r00));
gconst71=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1054)))+(((cj1)*(sj2)*(x1055)))+(((cj1)*(cj2)*(x1054)))+(((cj2)*(sj1)*(x1055))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x1056=((cj0)*(r02));
IkReal x1057=((cj0)*(r00));
gconst73=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1056)))+(((cj1)*(sj2)*(x1057)))+(((cj1)*(cj2)*(x1056)))+(((cj2)*(sj1)*(x1057))));
IkReal x1058=((gconst70)*(gconst71));
IkReal x1059=((IkReal(1.00000000000000))*(gconst65));
IkReal x1060=((gconst64)*(gconst68));
IkReal x1061=((gconst69)*(gconst73));
dummyeval[0]=((((gconst65)*(gconst66)*(x1061)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1058)*(x1059)))+(((IkReal(-1.00000000000000))*(x1060)*(x1061)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x1059)))+(((x1058)*(x1060))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1062=((gconst62)*(gconst70));
IkReal x1063=((IkReal(1.00000000000000))*(gconst49));
IkReal x1064=((gconst53)*(gconst73));
IkReal x1065=((gconst48)*(gconst52));
IkReal x1066=((gconst65)*(gconst66));
IkReal x1067=((IkReal(1.00000000000000))*(gconst65));
IkReal x1068=((gconst53)*(gconst56));
IkReal x1069=((gconst49)*(gconst50));
IkReal x1070=((gconst55)*(gconst70));
IkReal x1071=((gconst54)*(gconst62));
IkReal x1072=((gconst59)*(gconst69));
IkReal x1073=((gconst49)*(gconst57));
IkReal x1074=((gconst52)*(gconst63));
IkReal x1075=((gconst60)*(gconst73));
IkReal x1076=((gconst52)*(gconst58));
IkReal x1077=((gconst56)*(gconst69));
IkReal x1078=((gconst54)*(gconst55));
IkReal x1079=((gconst63)*(gconst65));
IkReal x1080=((gconst63)*(gconst69));
IkReal x1081=((gconst49)*(gconst66));
IkReal x1082=((IkReal(1.00000000000000))*(gconst48));
IkReal x1083=((gconst52)*(gconst64));
IkReal x1084=((gconst70)*(gconst71));
IkReal x1085=((gconst68)*(gconst69));
IkReal x1086=((IkReal(1.00000000000000))*(gconst68));
IkReal x1087=((gconst54)*(gconst71));
IkReal x1088=((gconst48)*(gconst60));
IkReal x1089=((gconst53)*(gconst72));
IkReal x1090=((gconst53)*(gconst59));
IkReal x1091=((gconst58)*(gconst61));
IkReal x1092=((gconst57)*(gconst65));
IkReal x1093=((gconst53)*(gconst63));
IkReal x1094=((gconst61)*(gconst71));
IkReal x1095=((gconst65)*(gconst67));
IkReal x1096=((gconst48)*(gconst68));
IkReal x1097=((gconst53)*(gconst57));
IkReal x1098=((gconst60)*(gconst64));
IkReal x1099=((gconst69)*(gconst72));
IkReal x1100=((gconst58)*(gconst68));
IkReal x1101=((gconst50)*(gconst69));
IkReal x1102=((IkReal(1.00000000000000))*(gconst60));
IkReal x1103=((gconst69)*(gconst73));
IkReal x1104=((gconst57)*(gconst69));
IkReal x1105=((gconst49)*(gconst63));
IkReal x1106=((gconst50)*(gconst53));
IkReal x1107=((gconst55)*(gconst61));
IkReal x1108=((gconst65)*(gconst73));
IkReal x1109=((gconst61)*(gconst62));
IkReal x1110=((gconst64)*(gconst68));
IkReal x1111=((gconst58)*(gconst60));
IkReal x1112=((gconst49)*(gconst67));
IkReal x1113=((gconst60)*(x1078));
IkReal x1114=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x1115=((gconst72)*(x1111));
IkReal x1116=((gconst49)*(gconst51)*(gconst61));
IkReal x1117=((gconst51)*(gconst61)*(gconst65));
op[0]=((((IkReal(-1.00000000000000))*(x1066)*(x1084)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x1085)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1067)*(x1099)))+(((x1084)*(x1110)))+(((x1066)*(x1103))));
op[1]=((((IkReal(-1.00000000000000))*(x1075)*(x1114)))+(((x1084)*(x1098)))+(((x1094)*(x1095)))+(((IkReal(-1.00000000000000))*(x1062)*(x1066)))+(((x1072)*(x1108)))+(((x1066)*(x1080)))+(((gconst58)*(gconst72)*(x1085)))+(((x1062)*(x1110)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1067)*(x1084)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1080)*(x1086))));
op[2]=((((x1062)*(x1098)))+(((IkReal(-1.00000000000000))*(x1080)*(x1098)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1067)*(x1084)))+(((IkReal(-1.00000000000000))*(x1066)*(x1087)))+(((x1084)*(x1096)))+(((IkReal(-1.00000000000000))*(x1066)*(x1070)))+(((x1087)*(x1110)))+(((x1101)*(x1108)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1063)*(x1099)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1067)*(x1089)))+(((x1099)*(x1111)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1067)*(x1077)))+(((IkReal(-1.00000000000000))*(x1083)*(x1103)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1086)*(x1091)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x1085)))+(((x1066)*(x1104)))+(((x1072)*(x1079)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1062)*(x1067)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1064)*(x1086)))+(((IkReal(-1.00000000000000))*(gconst73)*(x1082)*(x1085)))+(((x1081)*(x1103)))+(((x1064)*(x1066)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1063)*(x1084)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1067)*(x1099)))+(((x1070)*(x1110)))+(((x1095)*(x1109)))+(((x1083)*(x1084))));
op[3]=((((x1094)*(x1112)))+(((gconst49)*(gconst73)*(x1072)))+(((x1062)*(x1096)))+(((x1087)*(x1098)))+(((x1076)*(x1099)))+(((x1077)*(x1100)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1080)*(x1082)))+(((gconst51)*(gconst65)*(x1094)))+(((IkReal(-1.00000000000000))*(x1066)*(x1071)))+(((IkReal(-1.00000000000000))*(gconst69)*(x1075)*(x1082)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1086)*(x1093)))+(((x1072)*(x1092)))+(((IkReal(-1.00000000000000))*(x1098)*(x1104)))+(((x1062)*(x1083)))+(((gconst59)*(gconst65)*(x1064)))+(((x1089)*(x1100)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1086)*(x1091)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1062)*(x1063)))+(((x1079)*(x1101)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1067)*(x1070)))+(((IkReal(-1.00000000000000))*(x1074)*(x1114)))+(((x1080)*(x1081)))+(((x1070)*(x1098)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1067)*(x1087)))+(((x1066)*(x1093)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1063)*(x1084)))+(((IkReal(-1.00000000000000))*(x1064)*(x1098)))+(((x1084)*(x1088)))+(((x1071)*(x1110)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1091)*(x1102)))+(((x1095)*(x1107)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1062)*(x1067))));
op[4]=((((IkReal(-1.00000000000000))*(gconst51)*(x1067)*(x1077)))+(((x1109)*(x1112)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1063)*(x1084)))+(((gconst51)*(gconst65)*(x1109)))+(((x1092)*(x1101)))+(((x1087)*(x1096)))+(((x1077)*(x1111)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1067)*(x1087)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1067)*(x1070)))+(((x1079)*(x1090)))+(((IkReal(-1.00000000000000))*(x1066)*(x1078)))+(((IkReal(-1.00000000000000))*(x1076)*(x1094)))+(((IkReal(-1.00000000000000))*(x1064)*(x1083)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1063)*(x1099)))+(((IkReal(-1.00000000000000))*(gconst57)*(x1082)*(x1085)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1086)*(x1097)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1091)*(x1102)))+(((IkReal(-1.00000000000000))*(x1065)*(x1103)))+(((x1062)*(x1088)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1063)*(x1077)))+(((x1072)*(x1105)))+(((x1089)*(x1111)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1063)*(x1070)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1080)*(x1082)))+(((x1078)*(x1110)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1067)*(x1071)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1067)*(x1089)))+(((x1070)*(x1083)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1067)*(x1068)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1086)*(x1091)))+(((IkReal(-1.00000000000000))*(x1083)*(x1104)))+(((gconst50)*(gconst65)*(x1064)))+(((x1070)*(x1096)))+(((x1071)*(x1098)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1064)*(x1082)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1062)*(x1063)))+(((x1069)*(x1103)))+(((x1066)*(x1097)))+(((x1065)*(x1084)))+(((gconst66)*(gconst69)*(x1073)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1063)*(x1089)))+(((IkReal(-1.00000000000000))*(x1093)*(x1098)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1063)*(x1087)))+(((x1064)*(x1081)))+(((x1083)*(x1087))));
op[5]=((((gconst51)*(gconst65)*(x1107)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1082)*(x1093)))+(((x1076)*(x1089)))+(((IkReal(-1.00000000000000))*(x1065)*(x1080)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1067)*(x1071)))+(((gconst49)*(gconst51)*(x1094)))+(((gconst49)*(gconst59)*(x1064)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1063)*(x1070)))+(((IkReal(-1.00000000000000))*(x1076)*(x1109)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1082)*(x1104)))+(((x1069)*(x1080)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1063)*(x1071)))+(((x1062)*(x1065)))+(((x1079)*(x1106)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1067)*(x1078)))+(((x1087)*(x1088)))+(((x1070)*(x1088)))+(((x1068)*(x1100)))+(((IkReal(-1.00000000000000))*(x1097)*(x1098)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1064)*(x1082)))+(((x1071)*(x1083)))+(((x1081)*(x1093)))+(((x1076)*(x1077)))+(((x1071)*(x1096)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1091)*(x1102)))+(((x1072)*(x1073)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x1074)))+(((x1078)*(x1098)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1063)*(x1087)))+(((x1090)*(x1092)))+(((x1107)*(x1112)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1062)*(x1063))));
op[6]=((((IkReal(-1.00000000000000))*(gconst50)*(x1063)*(x1087)))+(((x1092)*(x1106)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1082)*(x1097)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1067)*(x1078)))+(((IkReal(-1.00000000000000))*(x1064)*(x1065)))+(((x1078)*(x1083)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1063)*(x1071)))+(((IkReal(-1.00000000000000))*(x1083)*(x1097)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1063)*(x1077)))+(((IkReal(-1.00000000000000))*(x1076)*(x1107)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1082)*(x1093)))+(((IkReal(-1.00000000000000))*(x1065)*(x1104)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1063)*(x1070)))+(((x1090)*(x1105)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1063)*(x1068)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1063)*(x1078)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1063)*(x1089)))+(((x1068)*(x1111)))+(((x1071)*(x1088)))+(((x1078)*(x1096)))+(((x1069)*(x1104)))+(((gconst49)*(gconst51)*(x1109)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1067)*(x1068)))+(((gconst53)*(gconst66)*(x1073)))+(((x1065)*(x1087)))+(((x1064)*(x1069)))+(((x1065)*(x1070))));
op[7]=((((x1068)*(x1076)))+(((x1073)*(x1090)))+(((x1078)*(x1088)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1063)*(x1078)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1082)*(x1097)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1063)*(x1071)))+(((x1069)*(x1093)))+(((gconst49)*(gconst51)*(x1107)))+(((x1065)*(x1071)))+(((IkReal(-1.00000000000000))*(x1065)*(x1093))));
op[8]=((((IkReal(-1.00000000000000))*(gconst51)*(x1063)*(x1068)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1063)*(x1078)))+(((x1069)*(x1097)))+(((x1065)*(x1078)))+(((IkReal(-1.00000000000000))*(x1065)*(x1097))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1118=((sj1)*(sj3));
IkReal x1119=((IkReal(1.00000000000000))*(cj0));
IkReal x1120=((cj2)*(r00));
IkReal x1121=((cj3)*(sj0));
IkReal x1122=((cj0)*(sj3));
IkReal x1123=((cj3)*(sj1));
IkReal x1124=((r02)*(sj2));
IkReal x1125=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x1126=((cj1)*(r00)*(sj2));
IkReal x1127=((sj0)*(x1124));
IkReal x1128=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(x1119)*(x1128)))+(((IkReal(-1.00000000000000))*(cj3)*(x1119)*(x1126)))+(((r01)*(x1121)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1119)))+(((cj0)*(x1123)*(x1124)))+(((IkReal(-1.00000000000000))*(sj0)*(x1118)*(x1120)))+(((IkReal(-1.00000000000000))*(x1125)*(x1128)))+(((IkReal(-1.00000000000000))*(x1125)*(x1126)))+(((x1118)*(x1127)))+(((IkReal(-1.00000000000000))*(x1119)*(x1120)*(x1123))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1119)))+(((IkReal(-1.00000000000000))*(x1118)*(x1119)*(x1124)))+(((IkReal(-1.00000000000000))*(x1121)*(x1126)))+(((IkReal(-1.00000000000000))*(x1121)*(x1128)))+(((sj1)*(x1121)*(x1124)))+(((cj0)*(x1118)*(x1120)))+(((x1122)*(x1126)))+(((x1122)*(x1128)))+(((IkReal(-1.00000000000000))*(r01)*(x1125)))+(((IkReal(-1.00000000000000))*(sj1)*(x1120)*(x1121))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(x1119)*(x1128)))+(((IkReal(-1.00000000000000))*(cj3)*(x1119)*(x1126)))+(((r01)*(x1121)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1119)))+(((cj0)*(x1123)*(x1124)))+(((IkReal(-1.00000000000000))*(sj0)*(x1118)*(x1120)))+(((IkReal(-1.00000000000000))*(x1125)*(x1128)))+(((IkReal(-1.00000000000000))*(x1125)*(x1126)))+(((x1118)*(x1127)))+(((IkReal(-1.00000000000000))*(x1119)*(x1120)*(x1123)))))), ((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1119)))+(((IkReal(-1.00000000000000))*(x1118)*(x1119)*(x1124)))+(((IkReal(-1.00000000000000))*(x1121)*(x1126)))+(((IkReal(-1.00000000000000))*(x1121)*(x1128)))+(((sj1)*(x1121)*(x1124)))+(((cj0)*(x1118)*(x1120)))+(((x1122)*(x1126)))+(((x1122)*(x1128)))+(((IkReal(-1.00000000000000))*(r01)*(x1125)))+(((IkReal(-1.00000000000000))*(sj1)*(x1120)*(x1121)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1129=IKcos(j5);
IkReal x1130=IKsin(j5);
IkReal x1131=((IkReal(1.00000000000000))*(cj0));
IkReal x1132=((IkReal(1.00000000000000))*(sj0));
IkReal x1133=((r02)*(sj1)*(sj2));
IkReal x1134=((IkReal(1.00000000000000))*(x1129));
IkReal x1135=((cj1)*(cj2)*(r02));
IkReal x1136=((cj2)*(r00)*(sj1));
IkReal x1137=((cj1)*(r00)*(sj2));
evalcond[0]=((((cj3)*(x1130)))+(((cj0)*(x1135)))+(((cj0)*(x1136)))+(((cj0)*(x1137)))+(((IkReal(-1.00000000000000))*(sj3)*(x1134)))+(((IkReal(-1.00000000000000))*(x1131)*(x1133)))+(((IkReal(-1.00000000000000))*(r01)*(x1132))));
evalcond[1]=((((sj0)*(x1133)))+(((IkReal(-1.00000000000000))*(sj3)*(x1130)))+(((IkReal(-1.00000000000000))*(x1132)*(x1136)))+(((IkReal(-1.00000000000000))*(x1132)*(x1137)))+(((IkReal(-1.00000000000000))*(x1132)*(x1135)))+(((IkReal(-1.00000000000000))*(cj3)*(x1134)))+(((IkReal(-1.00000000000000))*(r01)*(x1131))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x1138=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1139=((cj1)*(x1138));
IkReal x1140=((IkReal(1.00000000000000))*(r02));
IkReal x1141=((sj1)*(x1138));
sj5array[0]=((((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1141)))+(((IkReal(-1.00000000000000))*(sj2)*(x1139)*(x1140)))+(((IkReal(-1.00000000000000))*(cj2)*(x1140)*(x1141)))+(((cj2)*(r00)*(x1139))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x1142=((cj0)*(cj5));
IkReal x1143=((IkReal(1.00000000000000))*(r01));
IkReal x1144=((cj4)*(sj0)*(sj5));
IkReal x1145=((cj2)*(r00)*(sj1));
IkReal x1146=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x1147=((cj0)*(cj4)*(sj5));
IkReal x1148=((cj1)*(r00)*(sj2));
IkReal x1149=((cj1)*(cj2)*(r02));
IkReal x1150=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x1144)*(x1150)))+(((x1142)*(x1148)))+(((x1142)*(x1149)))+(((x1142)*(x1145)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1143)))+(((r01)*(x1147)))+(((x1144)*(x1149)))+(((x1144)*(x1148)))+(((x1144)*(x1145)))+(((IkReal(-1.00000000000000))*(x1142)*(x1150))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x1145)*(x1147)))+(((IkReal(-1.00000000000000))*(x1145)*(x1146)))+(((IkReal(-1.00000000000000))*(x1143)*(x1144)))+(((IkReal(-1.00000000000000))*(x1146)*(x1149)))+(((IkReal(-1.00000000000000))*(x1146)*(x1148)))+(((IkReal(-1.00000000000000))*(x1147)*(x1150)))+(((x1147)*(x1149)))+(((x1147)*(x1148)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1142)*(x1143))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x1144)*(x1150)))+(((x1142)*(x1148)))+(((x1142)*(x1149)))+(((x1142)*(x1145)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1143)))+(((r01)*(x1147)))+(((x1144)*(x1149)))+(((x1144)*(x1148)))+(((x1144)*(x1145)))+(((IkReal(-1.00000000000000))*(x1142)*(x1150)))))), ((gconst4)*(((((x1145)*(x1147)))+(((IkReal(-1.00000000000000))*(x1145)*(x1146)))+(((IkReal(-1.00000000000000))*(x1143)*(x1144)))+(((IkReal(-1.00000000000000))*(x1146)*(x1149)))+(((IkReal(-1.00000000000000))*(x1146)*(x1148)))+(((IkReal(-1.00000000000000))*(x1147)*(x1150)))+(((x1147)*(x1149)))+(((x1147)*(x1148)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1142)*(x1143)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x1151=IKsin(j3);
IkReal x1152=IKcos(j3);
IkReal x1153=((IkReal(1.00000000000000))*(cj0));
IkReal x1154=((cj0)*(cj1));
IkReal x1155=((cj2)*(r02));
IkReal x1156=((r00)*(sj2));
IkReal x1157=((IkReal(1.00000000000000))*(sj0));
IkReal x1158=((cj4)*(sj5));
IkReal x1159=((r02)*(sj1)*(sj2));
IkReal x1160=((cj2)*(r00)*(sj1));
IkReal x1161=((IkReal(1.00000000000000))*(x1152));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x1151)))+(((IkReal(-1.00000000000000))*(x1158)*(x1161)))+(((IkReal(-1.00000000000000))*(r01)*(x1157)))+(((cj0)*(x1160)))+(((IkReal(-1.00000000000000))*(x1153)*(x1159)))+(((x1154)*(x1156)))+(((x1154)*(x1155))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x1153)))+(((IkReal(-1.00000000000000))*(cj1)*(x1155)*(x1157)))+(((x1151)*(x1158)))+(((IkReal(-1.00000000000000))*(cj1)*(x1156)*(x1157)))+(((sj0)*(x1159)))+(((IkReal(-1.00000000000000))*(x1157)*(x1160)))+(((IkReal(-1.00000000000000))*(cj5)*(x1161))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x1162=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1162)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1162)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x1163=((cj0)*(cj5));
IkReal x1164=((IkReal(1.00000000000000))*(r01));
IkReal x1165=((cj4)*(sj0)*(sj5));
IkReal x1166=((cj2)*(r00)*(sj1));
IkReal x1167=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x1168=((cj0)*(cj4)*(sj5));
IkReal x1169=((cj1)*(r00)*(sj2));
IkReal x1170=((cj1)*(cj2)*(r02));
IkReal x1171=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x1165)*(x1171)))+(((x1165)*(x1169)))+(((x1165)*(x1166)))+(((x1165)*(x1170)))+(((x1163)*(x1166)))+(((x1163)*(x1169)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1164)))+(((x1163)*(x1170)))+(((IkReal(-1.00000000000000))*(x1163)*(x1171)))+(((r01)*(x1168))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x1166)*(x1167)))+(((x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(x1168)*(x1171)))+(((x1168)*(x1170)))+(((IkReal(-1.00000000000000))*(x1163)*(x1164)))+(((x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(x1167)*(x1170)))+(((IkReal(-1.00000000000000))*(x1167)*(x1169)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1164)*(x1165))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(x1165)*(x1171)))+(((x1165)*(x1169)))+(((x1165)*(x1166)))+(((x1165)*(x1170)))+(((x1163)*(x1166)))+(((x1163)*(x1169)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1164)))+(((x1163)*(x1170)))+(((IkReal(-1.00000000000000))*(x1163)*(x1171)))+(((r01)*(x1168)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x1166)*(x1167)))+(((x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(x1168)*(x1171)))+(((x1168)*(x1170)))+(((IkReal(-1.00000000000000))*(x1163)*(x1164)))+(((x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(x1167)*(x1170)))+(((IkReal(-1.00000000000000))*(x1167)*(x1169)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1164)*(x1165)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x1172=IKsin(j3);
IkReal x1173=IKcos(j3);
IkReal x1174=((IkReal(1.00000000000000))*(cj0));
IkReal x1175=((cj0)*(cj1));
IkReal x1176=((cj2)*(r02));
IkReal x1177=((r00)*(sj2));
IkReal x1178=((IkReal(1.00000000000000))*(sj0));
IkReal x1179=((cj4)*(sj5));
IkReal x1180=((r02)*(sj1)*(sj2));
IkReal x1181=((cj2)*(r00)*(sj1));
IkReal x1182=((IkReal(1.00000000000000))*(x1173));
evalcond[0]=((((cj0)*(x1181)))+(((IkReal(-1.00000000000000))*(x1179)*(x1182)))+(((x1175)*(x1176)))+(((x1175)*(x1177)))+(((IkReal(-1.00000000000000))*(cj5)*(x1172)))+(((IkReal(-1.00000000000000))*(r01)*(x1178)))+(((IkReal(-1.00000000000000))*(x1174)*(x1180))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x1182)))+(((IkReal(-1.00000000000000))*(x1178)*(x1181)))+(((sj0)*(x1180)))+(((IkReal(-1.00000000000000))*(cj1)*(x1177)*(x1178)))+(((x1172)*(x1179)))+(((IkReal(-1.00000000000000))*(r01)*(x1174)))+(((IkReal(-1.00000000000000))*(cj1)*(x1176)*(x1178))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x1183=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((IkReal(134.055000000000))*(sj2)))+(((cj0)*(py)*(x1183)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x1183)))+(((IkReal(109.561000000000))*(cj2)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x1183)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((IkReal(134.055000000000))*(sj2)))+(((cj0)*(py)*(x1183)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x1183)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x1183)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x1184=IKsin(j1);
IkReal x1185=IKcos(j1);
IkReal x1186=((IkReal(0.331000000000000))*(sj2));
IkReal x1187=((cj0)*(py));
IkReal x1188=((px)*(sj0));
IkReal x1189=((IkReal(0.331000000000000))*(cj2));
IkReal x1190=((IkReal(0.810000000000000))*(x1185));
IkReal x1191=((pz)*(x1184));
evalcond[0]=((((x1184)*(x1189)))+(((x1185)*(x1186)))+(pz)+(((IkReal(0.405000000000000))*(x1184))));
evalcond[1]=((x1186)+(((x1184)*(x1187)))+(((pz)*(x1185)))+(((IkReal(-1.00000000000000))*(x1184)*(x1188))));
evalcond[2]=((IkReal(0.405000000000000))+(x1189)+(x1191)+(((x1185)*(x1188)))+(((IkReal(-1.00000000000000))*(x1185)*(x1187))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((x1187)*(x1190)))+(((IkReal(-1.00000000000000))*(x1188)*(x1190)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x1191))));
evalcond[4]=((x1188)+(((x1185)*(x1189)))+(((IkReal(-1.00000000000000))*(x1184)*(x1186)))+(((IkReal(0.405000000000000))*(x1185)))+(((IkReal(-1.00000000000000))*(x1187))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x1192=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1192)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1192)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x1193=((sj0)*(sj1));
IkReal x1194=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x1193)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1194)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1194)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1193))));
IkReal gconst6;
IkReal x1195=((sj0)*(sj1));
IkReal x1196=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x1195)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1196)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1196)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1195))));
IkReal gconst7;
IkReal x1197=((cj0)*(r02));
IkReal x1198=((cj0)*(r00));
gconst7=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x1198)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1197)))+(((cj1)*(sj2)*(x1198)))+(((cj1)*(cj2)*(x1197))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x1199=((cj0)*(r02));
IkReal x1200=((cj0)*(r00));
gconst9=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1199)))+(((cj1)*(sj2)*(x1200)))+(((cj1)*(cj2)*(x1199)))+(((cj2)*(sj1)*(x1200))));
IkReal gconst10;
IkReal x1201=((sj0)*(sj1));
IkReal x1202=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x1201)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1201)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1202)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1202))));
IkReal gconst11;
IkReal x1203=((sj0)*(sj1));
IkReal x1204=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x1203)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1203)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1204)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1204))));
IkReal gconst12;
IkReal x1205=((cj0)*(r02));
IkReal x1206=((cj0)*(r00));
gconst12=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1205)))+(((cj1)*(cj2)*(x1205)))+(((cj1)*(sj2)*(x1206)))+(((cj2)*(sj1)*(x1206))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x1207=((cj0)*(r02));
IkReal x1208=((cj0)*(r00));
gconst14=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1207)))+(((cj1)*(cj2)*(x1207)))+(((cj1)*(sj2)*(x1208)))+(((cj2)*(sj1)*(x1208))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x1209=((sj0)*(sj1));
IkReal x1210=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1210)))+(((r02)*(sj2)*(x1209)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1210)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1209)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst22;
IkReal x1211=((sj0)*(sj1));
IkReal x1212=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1212)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1212)))+(((r02)*(sj2)*(x1211)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1211)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst23;
IkReal x1213=((cj0)*(r02));
IkReal x1214=((cj0)*(r00));
gconst23=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1213)))+(((cj1)*(cj2)*(x1213)))+(((cj2)*(sj1)*(x1214)))+(((cj1)*(sj2)*(x1214))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x1215=((cj0)*(r02));
IkReal x1216=((cj0)*(r00));
gconst25=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1215)))+(((cj1)*(cj2)*(x1215)))+(((cj2)*(sj1)*(x1216)))+(((cj1)*(sj2)*(x1216))));
IkReal gconst26;
IkReal x1217=((sj0)*(sj1));
IkReal x1218=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1218)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1218)))+(((r02)*(sj2)*(x1217)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1217)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst27;
IkReal x1219=((sj0)*(sj1));
IkReal x1220=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1220)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1220)))+(((r02)*(sj2)*(x1219)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1219)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst28;
IkReal x1221=((cj0)*(r02));
IkReal x1222=((cj0)*(r00));
gconst28=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1221)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1222)))+(((cj2)*(sj1)*(x1222)))+(((cj1)*(cj2)*(x1221))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x1223=((cj0)*(r02));
IkReal x1224=((cj0)*(r00));
gconst30=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1223)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1224)))+(((cj2)*(sj1)*(x1224)))+(((cj1)*(cj2)*(x1223))));
IkReal x1225=((gconst27)*(gconst28));
IkReal x1226=((gconst22)*(gconst23));
IkReal x1227=((gconst21)*(gconst25));
IkReal x1228=((gconst26)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x1225)*(x1226)))+(((IkReal(-1.00000000000000))*(x1227)*(x1228)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((x1225)*(x1227)))+(((x1226)*(x1228))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1229=((IkReal(1.00000000000000))*(gconst19));
IkReal x1230=((gconst23)*(gconst27));
IkReal x1231=((gconst15)*(gconst18));
IkReal x1232=((gconst11)*(gconst6));
IkReal x1233=((gconst13)*(gconst26));
IkReal x1234=((gconst15)*(gconst9));
IkReal x1235=((gconst6)*(gconst8));
IkReal x1236=((gconst18)*(gconst28));
IkReal x1237=((gconst22)*(gconst30));
IkReal x1238=((gconst16)*(gconst26));
IkReal x1239=((IkReal(1.00000000000000))*(gconst10));
IkReal x1240=((gconst14)*(gconst25));
IkReal x1241=((IkReal(1.00000000000000))*(gconst26));
IkReal x1242=((gconst12)*(gconst27));
IkReal x1243=((gconst25)*(gconst5));
IkReal x1244=((gconst17)*(gconst28));
IkReal x1245=((gconst21)*(gconst27));
IkReal x1246=((gconst27)*(gconst6));
IkReal x1247=((IkReal(1.00000000000000))*(gconst28));
IkReal x1248=((gconst11)*(gconst5));
IkReal x1249=((gconst17)*(gconst21));
IkReal x1250=((gconst18)*(gconst19));
IkReal x1251=((gconst10)*(gconst7));
IkReal x1252=((gconst30)*(gconst6));
IkReal x1253=((gconst17)*(gconst5));
IkReal x1254=((IkReal(1.00000000000000))*(gconst12));
IkReal x1255=((gconst22)*(gconst7));
IkReal x1256=((gconst24)*(gconst6));
IkReal x1257=((gconst5)*(gconst9));
IkReal x1258=((gconst15)*(gconst17));
IkReal x1259=((gconst27)*(gconst28));
IkReal x1260=((gconst23)*(gconst26));
IkReal x1261=((gconst21)*(gconst9));
IkReal x1262=((gconst22)*(gconst8));
IkReal x1263=((gconst10)*(gconst23));
IkReal x1264=((gconst12)*(gconst18));
IkReal x1265=((gconst21)*(gconst25));
IkReal x1266=((gconst11)*(gconst28));
IkReal x1267=((gconst11)*(gconst19));
IkReal x1268=((gconst19)*(gconst27));
IkReal x1269=((gconst20)*(gconst6));
IkReal x1270=((gconst10)*(gconst16));
IkReal x1271=((gconst26)*(gconst29));
IkReal x1272=((gconst26)*(gconst7));
IkReal x1273=((gconst20)*(gconst22));
IkReal x1274=((gconst22)*(gconst24));
IkReal x1275=((gconst15)*(gconst25));
IkReal x1276=((gconst10)*(gconst13));
IkReal x1277=((gconst14)*(gconst6));
IkReal x1278=((gconst16)*(gconst22));
IkReal x1279=((gconst11)*(gconst12));
IkReal x1280=((gconst14)*(gconst22));
IkReal x1281=((gconst17)*(gconst19));
IkReal x1282=((gconst10)*(gconst29));
IkReal x1283=((gconst28)*(gconst9));
IkReal x1284=((IkReal(1.00000000000000))*(gconst6));
IkReal x1285=((gconst26)*(x1269));
IkReal x1286=((gconst9)*(x1267));
IkReal x1287=((gconst11)*(x1278));
IkReal x1288=((gconst14)*(x1263));
IkReal x1289=((gconst11)*(gconst22)*(gconst23));
op[0]=((((IkReal(-1.00000000000000))*(gconst30)*(x1241)*(x1265)))+(((x1237)*(x1260)))+(((gconst25)*(gconst28)*(x1245)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1230)*(x1247)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1241)*(x1274))));
op[1]=((((IkReal(-1.00000000000000))*(gconst27)*(x1247)*(x1278)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1241)*(x1249)))+(((x1260)*(x1273)))+(((x1236)*(x1274)))+(((x1244)*(x1245)))+(((x1271)*(x1275)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1241)*(x1265)))+(((x1237)*(x1238)))+(((gconst19)*(gconst25)*(x1245)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1229)*(x1230))));
op[2]=((((x1242)*(x1265)))+(((IkReal(-1.00000000000000))*(gconst21)*(x1240)*(x1241)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1241)*(x1243)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1241)*(x1249)))+(((x1243)*(x1259)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1247)*(x1255)))+(((IkReal(-1.00000000000000))*(x1247)*(x1289)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1231)*(x1247)))+(((IkReal(-1.00000000000000))*(x1233)*(x1274)))+(((x1258)*(x1271)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1241)*(x1261)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1239)*(x1265)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1239)*(x1274)))+(((x1250)*(x1274)))+(((x1238)*(x1273)))+(((x1237)*(x1263)))+(((x1252)*(x1260)))+(((x1237)*(x1272)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1230)*(x1247)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1229)*(x1278)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1241)*(x1256)))+(((x1260)*(x1280)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1241)*(x1262)))+(((x1245)*(x1283)))+(((x1245)*(x1281)))+(((x1265)*(x1266)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1230)*(x1254))));
op[3]=((((IkReal(-1.00000000000000))*(gconst20)*(x1239)*(x1265)))+(((x1233)*(x1275)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1241)*(x1243)))+(((x1264)*(x1274)))+(((gconst27)*(gconst5)*(x1244)))+(((x1243)*(x1268)))+(((IkReal(-1.00000000000000))*(x1247)*(x1287)))+(((x1238)*(x1252)))+(((x1275)*(x1282)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1241)*(x1253)))+(((gconst20)*(gconst26)*(x1255)))+(((x1238)*(x1280)))+(((x1260)*(x1269)))+(((x1237)*(x1270)))+(((x1234)*(x1271)))+(((x1263)*(x1273)))+(((gconst11)*(gconst21)*(x1244)))+(((x1236)*(x1256)))+(((x1236)*(x1262)))+(((x1242)*(x1249)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1239)*(x1249)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1241)*(x1261)))+(((IkReal(-1.00000000000000))*(x1229)*(x1289)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1229)*(x1231)))+(((x1265)*(x1267)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1229)*(x1255)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1241)*(x1249)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1229)*(x1230)))+(((gconst19)*(gconst9)*(x1245)))+(((IkReal(-1.00000000000000))*(x1231)*(x1244)))+(((IkReal(-1.00000000000000))*(x1242)*(x1278)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1246)*(x1247))));
op[4]=((((x1242)*(x1261)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1246)*(x1247)))+(((x1263)*(x1280)))+(((x1265)*(x1279)))+(((IkReal(-1.00000000000000))*(x1242)*(x1255)))+(((IkReal(-1.00000000000000))*(x1254)*(x1289)))+(((x1261)*(x1266)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1235)*(x1241)))+(((x1243)*(x1266)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1239)*(x1256)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1239)*(x1262)))+(((IkReal(-1.00000000000000))*(gconst21)*(x1239)*(x1240)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1230)*(x1254)))+(((x1238)*(x1269)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1239)*(x1261)))+(((x1260)*(x1277)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1241)*(x1257)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1239)*(x1274)))+(((x1253)*(x1268)))+(((x1237)*(x1251)))+(((IkReal(-1.00000000000000))*(x1233)*(x1262)))+(((IkReal(-1.00000000000000))*(x1233)*(x1256)))+(((x1250)*(x1262)))+(((x1258)*(x1282)))+(((x1252)*(x1263)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1231)*(x1247)))+(((x1250)*(x1256)))+(((gconst14)*(gconst26)*(x1255)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1229)*(x1246)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1239)*(x1249)))+(((x1270)*(x1273)))+(((x1242)*(x1243)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1241)*(x1253)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1241)*(x1261)))+(((IkReal(-1.00000000000000))*(gconst17)*(x1229)*(x1231)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1239)*(x1243)))+(((x1257)*(x1259)))+(((x1252)*(x1272)))+(((IkReal(-1.00000000000000))*(x1229)*(x1287)))+(((IkReal(-1.00000000000000))*(gconst5)*(x1240)*(x1241)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1231)*(x1254)))+(((x1249)*(x1267)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1232)*(x1247)))+(((x1233)*(x1258)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1247)*(x1255))));
op[5]=((((x1269)*(x1272)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1239)*(x1261)))+(((x1270)*(x1280)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1239)*(x1249)))+(((x1275)*(x1276)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1242)*(x1284)))+(((x1235)*(x1236)))+(((IkReal(-1.00000000000000))*(x1254)*(x1287)))+(((x1261)*(x1267)))+(((x1243)*(x1267)))+(((x1256)*(x1264)))+(((x1238)*(x1277)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1239)*(x1253)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1232)*(x1247)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1229)*(x1231)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1239)*(x1243)))+(((x1262)*(x1264)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1241)*(x1253)))+(((x1233)*(x1234)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1241)*(x1257)))+(((x1244)*(x1248)))+(((x1263)*(x1269)))+(((IkReal(-1.00000000000000))*(gconst17)*(x1231)*(x1254)))+(((x1249)*(x1279)))+(((x1252)*(x1270)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1229)*(x1246)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1229)*(x1232)))+(((x1257)*(x1268)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1229)*(x1255)))+(((x1251)*(x1273)))+(((x1234)*(x1282)))+(((x1242)*(x1253))));
op[6]=((((x1269)*(x1270)))+(((x1243)*(x1279)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1242)*(x1284)))+(((IkReal(-1.00000000000000))*(x1233)*(x1235)))+(((x1272)*(x1277)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1232)*(x1254)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1239)*(x1261)))+(((x1261)*(x1279)))+(((x1258)*(x1276)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1229)*(x1232)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1239)*(x1257)))+(((x1235)*(x1250)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1239)*(x1256)))+(((x1263)*(x1277)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1241)*(x1257)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1239)*(x1262)))+(((x1248)*(x1281)))+(((x1248)*(x1283)))+(((x1251)*(x1280)))+(((IkReal(-1.00000000000000))*(gconst5)*(x1239)*(x1240)))+(((x1251)*(x1252)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1235)*(x1239)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1232)*(x1247)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1254)*(x1255)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1231)*(x1254)))+(((x1242)*(x1257)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1239)*(x1253))));
op[7]=((((gconst12)*(gconst17)*(x1248)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1239)*(x1253)))+(((x1235)*(x1264)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1229)*(x1232)))+(((x1234)*(x1276)))+(((x1270)*(x1277)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1232)*(x1254)))+(((x1251)*(x1269)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1239)*(x1257)))+(((gconst19)*(gconst9)*(x1248))));
op[8]=((((IkReal(-1.00000000000000))*(gconst7)*(x1232)*(x1254)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1239)*(x1257)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1235)*(x1239)))+(((gconst12)*(gconst9)*(x1248)))+(((x1251)*(x1277))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1290=((r01)*(sj3));
IkReal x1291=((IkReal(1.00000000000000))*(sj0));
IkReal x1292=((sj1)*(sj3));
IkReal x1293=((cj2)*(r00));
IkReal x1294=((r02)*(sj2));
IkReal x1295=((cj3)*(sj1));
IkReal x1296=((IkReal(1.00000000000000))*(cj0));
IkReal x1297=((cj3)*(r01));
IkReal x1298=((cj1)*(r00)*(sj2));
IkReal x1299=((cj1)*(cj2)*(r02)*(sj3));
IkReal x1300=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((cj0)*(x1293)*(x1295)))+(((sj0)*(sj3)*(x1298)))+(((sj0)*(x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(x1291)*(x1292)*(x1294)))+(((sj0)*(x1299)))+(((cj0)*(cj3)*(x1298)))+(((cj0)*(x1290)))+(((cj0)*(x1300)))+(((IkReal(-1.00000000000000))*(x1291)*(x1297)))+(((IkReal(-1.00000000000000))*(x1294)*(x1295)*(x1296))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((cj0)*(sj3)*(x1298)))+(((IkReal(-1.00000000000000))*(cj3)*(x1291)*(x1298)))+(((sj0)*(x1294)*(x1295)))+(((IkReal(-1.00000000000000))*(x1291)*(x1300)))+(((cj0)*(x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(x1291)*(x1293)*(x1295)))+(((IkReal(-1.00000000000000))*(x1290)*(x1291)))+(((cj0)*(x1299)))+(((IkReal(-1.00000000000000))*(x1296)*(x1297)))+(((IkReal(-1.00000000000000))*(x1292)*(x1294)*(x1296))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((cj0)*(x1293)*(x1295)))+(((sj0)*(sj3)*(x1298)))+(((sj0)*(x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(x1291)*(x1292)*(x1294)))+(((sj0)*(x1299)))+(((cj0)*(cj3)*(x1298)))+(((cj0)*(x1290)))+(((cj0)*(x1300)))+(((IkReal(-1.00000000000000))*(x1291)*(x1297)))+(((IkReal(-1.00000000000000))*(x1294)*(x1295)*(x1296)))))), ((gconst47)*(((((cj0)*(sj3)*(x1298)))+(((IkReal(-1.00000000000000))*(cj3)*(x1291)*(x1298)))+(((sj0)*(x1294)*(x1295)))+(((IkReal(-1.00000000000000))*(x1291)*(x1300)))+(((cj0)*(x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(x1291)*(x1293)*(x1295)))+(((IkReal(-1.00000000000000))*(x1290)*(x1291)))+(((cj0)*(x1299)))+(((IkReal(-1.00000000000000))*(x1296)*(x1297)))+(((IkReal(-1.00000000000000))*(x1292)*(x1294)*(x1296)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1301=IKsin(j5);
IkReal x1302=IKcos(j5);
IkReal x1303=((IkReal(1.00000000000000))*(cj0));
IkReal x1304=((IkReal(1.00000000000000))*(cj3));
IkReal x1305=((cj0)*(cj1));
IkReal x1306=((cj2)*(r02));
IkReal x1307=((r00)*(sj2));
IkReal x1308=((IkReal(1.00000000000000))*(sj0));
IkReal x1309=((r02)*(sj1)*(sj2));
IkReal x1310=((cj2)*(r00)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1308)))+(((x1305)*(x1307)))+(((x1305)*(x1306)))+(((IkReal(-1.00000000000000))*(x1303)*(x1309)))+(((IkReal(-1.00000000000000))*(x1301)*(x1304)))+(((IkReal(-1.00000000000000))*(sj3)*(x1302)))+(((cj0)*(x1310))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x1307)*(x1308)))+(((IkReal(-1.00000000000000))*(cj1)*(x1306)*(x1308)))+(((IkReal(-1.00000000000000))*(r01)*(x1303)))+(((IkReal(-1.00000000000000))*(x1308)*(x1310)))+(((sj3)*(x1301)))+(((sj0)*(x1309)))+(((IkReal(-1.00000000000000))*(x1302)*(x1304))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x1311=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1311)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1311)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x1312=((sj0)*(sj1));
IkReal x1313=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1313)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1312)))+(((r02)*(sj2)*(x1312)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1313))));
IkReal gconst49;
IkReal x1314=((sj0)*(sj1));
IkReal x1315=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1315)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1314)))+(((r02)*(sj2)*(x1314)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1315))));
IkReal gconst50;
IkReal x1316=((cj0)*(r02));
IkReal x1317=((cj0)*(r00));
gconst50=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x1316)))+(((cj1)*(sj2)*(x1317)))+(((cj2)*(sj1)*(x1317)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1316))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x1318=((cj0)*(r02));
IkReal x1319=((cj0)*(r00));
gconst52=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x1318)))+(((cj1)*(sj2)*(x1319)))+(((cj2)*(sj1)*(x1319)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1318))));
IkReal gconst53;
IkReal x1320=((sj0)*(sj1));
IkReal x1321=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1321)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1320)))+(((r02)*(sj2)*(x1320)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1321))));
IkReal gconst54;
IkReal x1322=((sj0)*(sj1));
IkReal x1323=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1323)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1322)))+(((r02)*(sj2)*(x1322)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1323))));
IkReal gconst55;
IkReal x1324=((cj0)*(r02));
IkReal x1325=((cj0)*(r00));
gconst55=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x1324)))+(((cj1)*(sj2)*(x1325)))+(((cj2)*(sj1)*(x1325)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1324))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x1326=((cj0)*(r02));
IkReal x1327=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x1326)))+(((cj1)*(sj2)*(x1327)))+(((cj2)*(sj1)*(x1327)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1326))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x1328=((sj0)*(sj1));
IkReal x1329=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1329)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1328)))+(((r02)*(sj2)*(x1328)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1329))));
IkReal gconst65;
IkReal x1330=((sj0)*(sj1));
IkReal x1331=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1331)))+(((r02)*(sj2)*(x1330)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1331)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1330))));
IkReal gconst66;
IkReal x1332=((cj0)*(r02));
IkReal x1333=((cj0)*(r00));
gconst66=((((cj1)*(cj2)*(x1332)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1333)))+(((cj2)*(sj1)*(x1333)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1332))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x1334=((cj0)*(r02));
IkReal x1335=((cj0)*(r00));
gconst68=((((cj1)*(cj2)*(x1334)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1335)))+(((cj2)*(sj1)*(x1335)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1334))));
IkReal gconst69;
IkReal x1336=((sj0)*(sj1));
IkReal x1337=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1337)))+(((r02)*(sj2)*(x1336)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1337)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1336))));
IkReal gconst70;
IkReal x1338=((sj0)*(sj1));
IkReal x1339=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1339)))+(((r02)*(sj2)*(x1338)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1339)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1338))));
IkReal gconst71;
IkReal x1340=((cj0)*(r02));
IkReal x1341=((cj0)*(r00));
gconst71=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1340)))+(((cj1)*(cj2)*(x1340)))+(((cj2)*(sj1)*(x1341)))+(((cj1)*(sj2)*(x1341))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x1342=((cj0)*(r02));
IkReal x1343=((cj0)*(r00));
gconst73=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1342)))+(((cj1)*(cj2)*(x1342)))+(((cj2)*(sj1)*(x1343)))+(((cj1)*(sj2)*(x1343))));
IkReal x1344=((gconst70)*(gconst71));
IkReal x1345=((IkReal(1.00000000000000))*(gconst65));
IkReal x1346=((gconst64)*(gconst68));
IkReal x1347=((gconst69)*(gconst73));
dummyeval[0]=((((gconst65)*(gconst66)*(x1347)))+(((x1344)*(x1346)))+(((IkReal(-1.00000000000000))*(x1346)*(x1347)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1344)*(x1345)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x1345))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1348=((gconst62)*(gconst70));
IkReal x1349=((IkReal(1.00000000000000))*(gconst49));
IkReal x1350=((gconst53)*(gconst73));
IkReal x1351=((gconst48)*(gconst52));
IkReal x1352=((gconst65)*(gconst66));
IkReal x1353=((IkReal(1.00000000000000))*(gconst65));
IkReal x1354=((gconst53)*(gconst56));
IkReal x1355=((gconst49)*(gconst50));
IkReal x1356=((gconst55)*(gconst70));
IkReal x1357=((gconst54)*(gconst62));
IkReal x1358=((gconst59)*(gconst69));
IkReal x1359=((gconst49)*(gconst57));
IkReal x1360=((gconst52)*(gconst63));
IkReal x1361=((gconst60)*(gconst73));
IkReal x1362=((gconst52)*(gconst58));
IkReal x1363=((gconst56)*(gconst69));
IkReal x1364=((gconst54)*(gconst55));
IkReal x1365=((gconst63)*(gconst65));
IkReal x1366=((gconst63)*(gconst69));
IkReal x1367=((gconst49)*(gconst66));
IkReal x1368=((IkReal(1.00000000000000))*(gconst48));
IkReal x1369=((gconst52)*(gconst64));
IkReal x1370=((gconst70)*(gconst71));
IkReal x1371=((gconst68)*(gconst69));
IkReal x1372=((IkReal(1.00000000000000))*(gconst68));
IkReal x1373=((gconst54)*(gconst71));
IkReal x1374=((gconst48)*(gconst60));
IkReal x1375=((gconst53)*(gconst72));
IkReal x1376=((gconst53)*(gconst59));
IkReal x1377=((gconst58)*(gconst61));
IkReal x1378=((gconst57)*(gconst65));
IkReal x1379=((gconst53)*(gconst63));
IkReal x1380=((gconst61)*(gconst71));
IkReal x1381=((gconst65)*(gconst67));
IkReal x1382=((gconst48)*(gconst68));
IkReal x1383=((gconst53)*(gconst57));
IkReal x1384=((gconst60)*(gconst64));
IkReal x1385=((gconst69)*(gconst72));
IkReal x1386=((gconst58)*(gconst68));
IkReal x1387=((gconst50)*(gconst69));
IkReal x1388=((IkReal(1.00000000000000))*(gconst60));
IkReal x1389=((gconst69)*(gconst73));
IkReal x1390=((gconst57)*(gconst69));
IkReal x1391=((gconst49)*(gconst63));
IkReal x1392=((gconst50)*(gconst53));
IkReal x1393=((gconst55)*(gconst61));
IkReal x1394=((gconst65)*(gconst73));
IkReal x1395=((gconst61)*(gconst62));
IkReal x1396=((gconst64)*(gconst68));
IkReal x1397=((gconst58)*(gconst60));
IkReal x1398=((gconst49)*(gconst67));
IkReal x1399=((gconst60)*(x1364));
IkReal x1400=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x1401=((gconst72)*(x1397));
IkReal x1402=((gconst49)*(gconst51)*(gconst61));
IkReal x1403=((gconst51)*(gconst61)*(gconst65));
op[0]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x1371)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1353)*(x1385)))+(((x1370)*(x1396)))+(((IkReal(-1.00000000000000))*(x1352)*(x1370)))+(((x1352)*(x1389))));
op[1]=((((x1380)*(x1381)))+(((x1370)*(x1384)))+(((gconst58)*(gconst72)*(x1371)))+(((x1358)*(x1394)))+(((x1348)*(x1396)))+(((IkReal(-1.00000000000000))*(x1348)*(x1352)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1353)*(x1370)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1366)*(x1372)))+(((IkReal(-1.00000000000000))*(x1361)*(x1400)))+(((x1352)*(x1366))));
op[2]=((((IkReal(-1.00000000000000))*(gconst64)*(x1350)*(x1372)))+(((x1370)*(x1382)))+(((x1356)*(x1396)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1353)*(x1385)))+(((IkReal(-1.00000000000000))*(x1369)*(x1389)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1353)*(x1363)))+(((IkReal(-1.00000000000000))*(x1352)*(x1356)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1372)*(x1377)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1349)*(x1385)))+(((x1358)*(x1365)))+(((IkReal(-1.00000000000000))*(gconst73)*(x1368)*(x1371)))+(((x1348)*(x1384)))+(((IkReal(-1.00000000000000))*(x1352)*(x1373)))+(((x1387)*(x1394)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1348)*(x1353)))+(((IkReal(-1.00000000000000))*(x1366)*(x1384)))+(((x1350)*(x1352)))+(((x1369)*(x1370)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x1371)))+(((x1352)*(x1390)))+(((x1373)*(x1396)))+(((x1385)*(x1397)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1349)*(x1370)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1353)*(x1375)))+(((x1367)*(x1389)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1353)*(x1370)))+(((x1381)*(x1395))));
op[3]=((((x1362)*(x1385)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1353)*(x1356)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1348)*(x1349)))+(((x1356)*(x1384)))+(((x1352)*(x1379)))+(((x1366)*(x1367)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1348)*(x1353)))+(((x1365)*(x1387)))+(((IkReal(-1.00000000000000))*(gconst69)*(x1361)*(x1368)))+(((x1357)*(x1396)))+(((IkReal(-1.00000000000000))*(x1352)*(x1357)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1366)*(x1368)))+(((gconst59)*(gconst65)*(x1350)))+(((IkReal(-1.00000000000000))*(x1350)*(x1384)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1377)*(x1388)))+(((IkReal(-1.00000000000000))*(x1384)*(x1390)))+(((x1348)*(x1382)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1353)*(x1373)))+(((x1348)*(x1369)))+(((x1370)*(x1374)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1372)*(x1379)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1349)*(x1370)))+(((x1375)*(x1386)))+(((gconst51)*(gconst65)*(x1380)))+(((x1363)*(x1386)))+(((gconst49)*(gconst73)*(x1358)))+(((IkReal(-1.00000000000000))*(x1360)*(x1400)))+(((x1373)*(x1384)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1372)*(x1377)))+(((x1358)*(x1378)))+(((x1381)*(x1393)))+(((x1380)*(x1398))));
op[4]=((((x1365)*(x1376)))+(((x1356)*(x1369)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1353)*(x1357)))+(((x1356)*(x1382)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1353)*(x1356)))+(((IkReal(-1.00000000000000))*(x1352)*(x1364)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1372)*(x1383)))+(((x1351)*(x1370)))+(((x1364)*(x1396)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1349)*(x1356)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1350)*(x1368)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1353)*(x1375)))+(((IkReal(-1.00000000000000))*(x1369)*(x1390)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1353)*(x1354)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1349)*(x1370)))+(((x1357)*(x1384)))+(((IkReal(-1.00000000000000))*(x1362)*(x1380)))+(((IkReal(-1.00000000000000))*(x1350)*(x1369)))+(((x1355)*(x1389)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1349)*(x1375)))+(((IkReal(-1.00000000000000))*(x1379)*(x1384)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1372)*(x1377)))+(((x1358)*(x1391)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1349)*(x1385)))+(((IkReal(-1.00000000000000))*(gconst57)*(x1368)*(x1371)))+(((x1395)*(x1398)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1348)*(x1349)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1349)*(x1363)))+(((IkReal(-1.00000000000000))*(x1351)*(x1389)))+(((x1375)*(x1397)))+(((x1350)*(x1367)))+(((x1363)*(x1397)))+(((gconst51)*(gconst65)*(x1395)))+(((x1369)*(x1373)))+(((x1373)*(x1382)))+(((x1352)*(x1383)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1349)*(x1373)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1377)*(x1388)))+(((x1348)*(x1374)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1353)*(x1363)))+(((gconst50)*(gconst65)*(x1350)))+(((gconst66)*(gconst69)*(x1359)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1353)*(x1373)))+(((x1378)*(x1387)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1366)*(x1368))));
op[5]=((((x1357)*(x1369)))+(((x1355)*(x1366)))+(((IkReal(-1.00000000000000))*(x1383)*(x1384)))+(((x1376)*(x1378)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1348)*(x1349)))+(((gconst49)*(gconst59)*(x1350)))+(((IkReal(-1.00000000000000))*(x1351)*(x1366)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1353)*(x1357)))+(((x1365)*(x1392)))+(((x1364)*(x1384)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1349)*(x1356)))+(((x1393)*(x1398)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1350)*(x1368)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1349)*(x1357)))+(((IkReal(-1.00000000000000))*(x1362)*(x1395)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1377)*(x1388)))+(((x1358)*(x1359)))+(((gconst49)*(gconst51)*(x1380)))+(((x1357)*(x1382)))+(((x1367)*(x1379)))+(((x1354)*(x1386)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x1360)))+(((x1348)*(x1351)))+(((x1362)*(x1375)))+(((gconst51)*(gconst65)*(x1393)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1349)*(x1373)))+(((x1356)*(x1374)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1368)*(x1390)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1368)*(x1379)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1353)*(x1364)))+(((x1362)*(x1363)))+(((x1373)*(x1374))));
op[6]=((((IkReal(-1.00000000000000))*(gconst50)*(x1353)*(x1364)))+(((gconst53)*(gconst66)*(x1359)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1368)*(x1383)))+(((x1364)*(x1382)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1349)*(x1357)))+(((x1351)*(x1373)))+(((IkReal(-1.00000000000000))*(x1369)*(x1383)))+(((IkReal(-1.00000000000000))*(x1362)*(x1393)))+(((x1355)*(x1390)))+(((x1354)*(x1397)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1349)*(x1373)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1349)*(x1364)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1349)*(x1375)))+(((gconst49)*(gconst51)*(x1395)))+(((IkReal(-1.00000000000000))*(x1350)*(x1351)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1349)*(x1354)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1368)*(x1379)))+(((x1364)*(x1369)))+(((IkReal(-1.00000000000000))*(x1351)*(x1390)))+(((x1350)*(x1355)))+(((x1376)*(x1391)))+(((x1351)*(x1356)))+(((x1357)*(x1374)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1349)*(x1356)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1353)*(x1354)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1349)*(x1363)))+(((x1378)*(x1392))));
op[7]=((((IkReal(-1.00000000000000))*(gconst59)*(x1349)*(x1364)))+(((x1354)*(x1362)))+(((gconst49)*(gconst51)*(x1393)))+(((IkReal(-1.00000000000000))*(x1351)*(x1379)))+(((x1351)*(x1357)))+(((x1359)*(x1376)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1349)*(x1357)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1368)*(x1383)))+(((x1364)*(x1374)))+(((x1355)*(x1379))));
op[8]=((((x1355)*(x1383)))+(((IkReal(-1.00000000000000))*(x1351)*(x1383)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1349)*(x1364)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1349)*(x1354)))+(((x1351)*(x1364))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1404=((sj1)*(sj3));
IkReal x1405=((IkReal(1.00000000000000))*(cj0));
IkReal x1406=((cj2)*(r00));
IkReal x1407=((cj3)*(sj0));
IkReal x1408=((cj0)*(sj3));
IkReal x1409=((cj3)*(sj1));
IkReal x1410=((r02)*(sj2));
IkReal x1411=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x1412=((cj1)*(r00)*(sj2));
IkReal x1413=((sj0)*(x1410));
IkReal x1414=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1405)))+(((IkReal(-1.00000000000000))*(sj0)*(x1404)*(x1406)))+(((IkReal(-1.00000000000000))*(x1411)*(x1414)))+(((IkReal(-1.00000000000000))*(x1411)*(x1412)))+(((cj0)*(x1409)*(x1410)))+(((x1404)*(x1413)))+(((r01)*(x1407)))+(((IkReal(-1.00000000000000))*(x1405)*(x1406)*(x1409)))+(((IkReal(-1.00000000000000))*(cj3)*(x1405)*(x1412)))+(((IkReal(-1.00000000000000))*(cj3)*(x1405)*(x1414))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1405)))+(((cj0)*(x1404)*(x1406)))+(((IkReal(-1.00000000000000))*(sj1)*(x1406)*(x1407)))+(((IkReal(-1.00000000000000))*(x1407)*(x1414)))+(((IkReal(-1.00000000000000))*(x1407)*(x1412)))+(((IkReal(-1.00000000000000))*(r01)*(x1411)))+(((sj1)*(x1407)*(x1410)))+(((x1408)*(x1414)))+(((x1408)*(x1412)))+(((IkReal(-1.00000000000000))*(x1404)*(x1405)*(x1410))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1405)))+(((IkReal(-1.00000000000000))*(sj0)*(x1404)*(x1406)))+(((IkReal(-1.00000000000000))*(x1411)*(x1414)))+(((IkReal(-1.00000000000000))*(x1411)*(x1412)))+(((cj0)*(x1409)*(x1410)))+(((x1404)*(x1413)))+(((r01)*(x1407)))+(((IkReal(-1.00000000000000))*(x1405)*(x1406)*(x1409)))+(((IkReal(-1.00000000000000))*(cj3)*(x1405)*(x1412)))+(((IkReal(-1.00000000000000))*(cj3)*(x1405)*(x1414)))))), ((gconst90)*(((((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1405)))+(((cj0)*(x1404)*(x1406)))+(((IkReal(-1.00000000000000))*(sj1)*(x1406)*(x1407)))+(((IkReal(-1.00000000000000))*(x1407)*(x1414)))+(((IkReal(-1.00000000000000))*(x1407)*(x1412)))+(((IkReal(-1.00000000000000))*(r01)*(x1411)))+(((sj1)*(x1407)*(x1410)))+(((x1408)*(x1414)))+(((x1408)*(x1412)))+(((IkReal(-1.00000000000000))*(x1404)*(x1405)*(x1410)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1415=IKcos(j5);
IkReal x1416=IKsin(j5);
IkReal x1417=((IkReal(1.00000000000000))*(cj0));
IkReal x1418=((IkReal(1.00000000000000))*(sj0));
IkReal x1419=((r02)*(sj1)*(sj2));
IkReal x1420=((IkReal(1.00000000000000))*(x1415));
IkReal x1421=((cj1)*(cj2)*(r02));
IkReal x1422=((cj2)*(r00)*(sj1));
IkReal x1423=((cj1)*(r00)*(sj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1417)*(x1419)))+(((IkReal(-1.00000000000000))*(sj3)*(x1420)))+(((IkReal(-1.00000000000000))*(r01)*(x1418)))+(((cj3)*(x1416)))+(((cj0)*(x1422)))+(((cj0)*(x1423)))+(((cj0)*(x1421))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj3)*(x1416)))+(((IkReal(-1.00000000000000))*(x1418)*(x1422)))+(((IkReal(-1.00000000000000))*(x1418)*(x1423)))+(((IkReal(-1.00000000000000))*(x1418)*(x1421)))+(((IkReal(-1.00000000000000))*(cj3)*(x1420)))+(((IkReal(-1.00000000000000))*(r01)*(x1417)))+(((sj0)*(x1419))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x1424=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1425=((cj1)*(x1424));
IkReal x1426=((IkReal(1.00000000000000))*(r02));
IkReal x1427=((sj1)*(x1424));
sj5array[0]=((((IkReal(-1.00000000000000))*(sj2)*(x1425)*(x1426)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1427)))+(((cj2)*(r00)*(x1425)))+(((IkReal(-1.00000000000000))*(cj2)*(x1426)*(x1427))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x1428=((cj0)*(cj5));
IkReal x1429=((IkReal(1.00000000000000))*(r01));
IkReal x1430=((cj4)*(sj0)*(sj5));
IkReal x1431=((cj2)*(r00)*(sj1));
IkReal x1432=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x1433=((cj0)*(cj4)*(sj5));
IkReal x1434=((cj1)*(r00)*(sj2));
IkReal x1435=((cj1)*(cj2)*(r02));
IkReal x1436=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x1428)*(x1431)))+(((x1428)*(x1434)))+(((x1428)*(x1435)))+(((x1430)*(x1434)))+(((x1430)*(x1435)))+(((x1430)*(x1431)))+(((IkReal(-1.00000000000000))*(x1428)*(x1436)))+(((r01)*(x1433)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1429)))+(((IkReal(-1.00000000000000))*(x1430)*(x1436))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x1429)*(x1430)))+(((x1433)*(x1435)))+(((x1433)*(x1434)))+(((IkReal(-1.00000000000000))*(x1431)*(x1432)))+(((IkReal(-1.00000000000000))*(x1432)*(x1434)))+(((IkReal(-1.00000000000000))*(x1432)*(x1435)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1433)*(x1436)))+(((x1431)*(x1433)))+(((IkReal(-1.00000000000000))*(x1428)*(x1429))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x1428)*(x1431)))+(((x1428)*(x1434)))+(((x1428)*(x1435)))+(((x1430)*(x1434)))+(((x1430)*(x1435)))+(((x1430)*(x1431)))+(((IkReal(-1.00000000000000))*(x1428)*(x1436)))+(((r01)*(x1433)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1429)))+(((IkReal(-1.00000000000000))*(x1430)*(x1436)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x1429)*(x1430)))+(((x1433)*(x1435)))+(((x1433)*(x1434)))+(((IkReal(-1.00000000000000))*(x1431)*(x1432)))+(((IkReal(-1.00000000000000))*(x1432)*(x1434)))+(((IkReal(-1.00000000000000))*(x1432)*(x1435)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1433)*(x1436)))+(((x1431)*(x1433)))+(((IkReal(-1.00000000000000))*(x1428)*(x1429)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x1437=IKsin(j3);
IkReal x1438=IKcos(j3);
IkReal x1439=((IkReal(1.00000000000000))*(cj0));
IkReal x1440=((cj0)*(cj1));
IkReal x1441=((cj2)*(r02));
IkReal x1442=((r00)*(sj2));
IkReal x1443=((IkReal(1.00000000000000))*(sj0));
IkReal x1444=((cj4)*(sj5));
IkReal x1445=((r02)*(sj1)*(sj2));
IkReal x1446=((cj2)*(r00)*(sj1));
IkReal x1447=((IkReal(1.00000000000000))*(x1438));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x1437)))+(((IkReal(-1.00000000000000))*(r01)*(x1443)))+(((x1440)*(x1442)))+(((x1440)*(x1441)))+(((IkReal(-1.00000000000000))*(x1439)*(x1445)))+(((cj0)*(x1446)))+(((IkReal(-1.00000000000000))*(x1444)*(x1447))));
evalcond[1]=((((x1437)*(x1444)))+(((IkReal(-1.00000000000000))*(cj1)*(x1442)*(x1443)))+(((IkReal(-1.00000000000000))*(cj1)*(x1441)*(x1443)))+(((sj0)*(x1445)))+(((IkReal(-1.00000000000000))*(cj5)*(x1447)))+(((IkReal(-1.00000000000000))*(r01)*(x1439)))+(((IkReal(-1.00000000000000))*(x1443)*(x1446))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x1448=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1448)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1448)))+(((cj1)*(cj2)*(r00))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x1449=((cj0)*(cj5));
IkReal x1450=((IkReal(1.00000000000000))*(r01));
IkReal x1451=((cj4)*(sj0)*(sj5));
IkReal x1452=((cj2)*(r00)*(sj1));
IkReal x1453=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x1454=((cj0)*(cj4)*(sj5));
IkReal x1455=((cj1)*(r00)*(sj2));
IkReal x1456=((cj1)*(cj2)*(r02));
IkReal x1457=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1450)))+(((r01)*(x1454)))+(((IkReal(-1.00000000000000))*(x1451)*(x1457)))+(((x1449)*(x1455)))+(((x1449)*(x1456)))+(((x1449)*(x1452)))+(((IkReal(-1.00000000000000))*(x1449)*(x1457)))+(((x1451)*(x1452)))+(((x1451)*(x1455)))+(((x1451)*(x1456))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x1454)*(x1456)))+(((x1454)*(x1455)))+(((IkReal(-1.00000000000000))*(x1454)*(x1457)))+(((IkReal(-1.00000000000000))*(x1453)*(x1455)))+(((IkReal(-1.00000000000000))*(x1453)*(x1456)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1452)*(x1453)))+(((IkReal(-1.00000000000000))*(x1449)*(x1450)))+(((x1452)*(x1454)))+(((IkReal(-1.00000000000000))*(x1450)*(x1451))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1450)))+(((r01)*(x1454)))+(((IkReal(-1.00000000000000))*(x1451)*(x1457)))+(((x1449)*(x1455)))+(((x1449)*(x1456)))+(((x1449)*(x1452)))+(((IkReal(-1.00000000000000))*(x1449)*(x1457)))+(((x1451)*(x1452)))+(((x1451)*(x1455)))+(((x1451)*(x1456)))))), ((gconst4)*(((((x1454)*(x1456)))+(((x1454)*(x1455)))+(((IkReal(-1.00000000000000))*(x1454)*(x1457)))+(((IkReal(-1.00000000000000))*(x1453)*(x1455)))+(((IkReal(-1.00000000000000))*(x1453)*(x1456)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1452)*(x1453)))+(((IkReal(-1.00000000000000))*(x1449)*(x1450)))+(((x1452)*(x1454)))+(((IkReal(-1.00000000000000))*(x1450)*(x1451)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x1458=IKsin(j3);
IkReal x1459=IKcos(j3);
IkReal x1460=((IkReal(1.00000000000000))*(cj0));
IkReal x1461=((cj0)*(cj1));
IkReal x1462=((cj2)*(r02));
IkReal x1463=((r00)*(sj2));
IkReal x1464=((IkReal(1.00000000000000))*(sj0));
IkReal x1465=((cj4)*(sj5));
IkReal x1466=((r02)*(sj1)*(sj2));
IkReal x1467=((cj2)*(r00)*(sj1));
IkReal x1468=((IkReal(1.00000000000000))*(x1459));
evalcond[0]=((((cj0)*(x1467)))+(((x1461)*(x1462)))+(((x1461)*(x1463)))+(((IkReal(-1.00000000000000))*(r01)*(x1464)))+(((IkReal(-1.00000000000000))*(x1460)*(x1466)))+(((IkReal(-1.00000000000000))*(x1465)*(x1468)))+(((IkReal(-1.00000000000000))*(cj5)*(x1458))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x1468)))+(((IkReal(-1.00000000000000))*(cj1)*(x1462)*(x1464)))+(((IkReal(-1.00000000000000))*(cj1)*(x1463)*(x1464)))+(((IkReal(-1.00000000000000))*(x1464)*(x1467)))+(((x1458)*(x1465)))+(((IkReal(-1.00000000000000))*(r01)*(x1460)))+(((sj0)*(x1466))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
if( IKabs(px) < IKFAST_ATAN2_MAGTHRESH && IKabs(py) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x1469=((IkReal(1.00000000000000))*(IKatan2(px, py)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x1470=IKasin(((IkReal(0.100150000000000))*(((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))));
j0array[0]=((((IkReal(-1.00000000000000))*(x1469)))+(((IkReal(-1.00000000000000))*(x1470))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x1469)))+(x1470));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst3;
IkReal x1471=((cj0)*(py));
IkReal x1472=((px)*(sj0));
IkReal x1473=((IkReal(331.000000000000))*(cj2));
gconst3=IKsign(((((IkReal(-1.00000000000000))*(x1471)*(x1473)))+(((IkReal(-331.000000000000))*(pz)*(sj2)))+(((x1472)*(x1473)))+(((IkReal(405.000000000000))*(x1472)))+(((IkReal(-405.000000000000))*(x1471)))));
IkReal x1474=((px)*(sj0));
IkReal x1475=((cj0)*(py));
dummyeval[0]=((((IkReal(-1.22356495468278))*(x1475)))+(((cj2)*(x1474)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1475)))+(((IkReal(1.22356495468278))*(x1474))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((((IkReal(-1.00000000000000))*(cj0)*(py)*(sj2)))+(((px)*(sj0)*(sj2)))+(((cj2)*(pz)))+(((IkReal(1.22356495468278))*(pz))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x1476=((IkReal(331.000000000000))*(sj2));
IkReal x1477=((cj0)*(py));
IkReal x1478=((px)*(sj0));
IkReal x1479=((IkReal(1000.00000000000))*(pz));
IkReal x1480=((cj2)*(pz));
IkReal x1481=((IkReal(331000.000000000))*(sj2));
if( IKabs(((((IKabs(((((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x1480)))+(((IkReal(-1.00000000000000))*(x1477)*(x1481)))+(((x1478)*(x1481))))) != 0)?((IkReal)1/(((((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x1480)))+(((IkReal(-1.00000000000000))*(x1477)*(x1481)))+(((x1478)*(x1481)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz)))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x1476)*(x1477)))+(((IkReal(331.000000000000))*(x1480)))+(((x1476)*(x1478)))+(((IkReal(405.000000000000))*(pz))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1476)*(x1477)))+(((IkReal(331.000000000000))*(x1480)))+(((x1476)*(x1478)))+(((IkReal(405.000000000000))*(pz)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1478)*(x1479)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x1477)*(x1479))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x1480)))+(((IkReal(-1.00000000000000))*(x1477)*(x1481)))+(((x1478)*(x1481))))) != 0)?((IkReal)1/(((((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x1480)))+(((IkReal(-1.00000000000000))*(x1477)*(x1481)))+(((x1478)*(x1481)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x1476)*(x1477)))+(((IkReal(331.000000000000))*(x1480)))+(((x1476)*(x1478)))+(((IkReal(405.000000000000))*(pz))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1476)*(x1477)))+(((IkReal(331.000000000000))*(x1480)))+(((x1476)*(x1478)))+(((IkReal(405.000000000000))*(pz)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1478)*(x1479)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x1477)*(x1479)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j1array[0]=IKatan2(((((IKabs(((((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x1480)))+(((IkReal(-1.00000000000000))*(x1477)*(x1481)))+(((x1478)*(x1481))))) != 0)?((IkReal)1/(((((IkReal(405000.000000000))*(pz)))+(((IkReal(331000.000000000))*(x1480)))+(((IkReal(-1.00000000000000))*(x1477)*(x1481)))+(((x1478)*(x1481)))))):(IkReal)1.0e30))*(((((IkReal(109561.000000000))*((sj2)*(sj2))))+(((IkReal(-1000000.00000000))*((pz)*(pz))))))), ((((IKabs(((((IkReal(-1.00000000000000))*(x1476)*(x1477)))+(((IkReal(331.000000000000))*(x1480)))+(((x1476)*(x1478)))+(((IkReal(405.000000000000))*(pz))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1476)*(x1477)))+(((IkReal(331.000000000000))*(x1480)))+(((x1476)*(x1478)))+(((IkReal(405.000000000000))*(pz)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1478)*(x1479)))+(((IkReal(-134.055000000000))*(sj2)))+(((IkReal(-109.561000000000))*(cj2)*(sj2)))+(((x1477)*(x1479)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x1482=IKsin(j1);
IkReal x1483=IKcos(j1);
IkReal x1484=((IkReal(0.331000000000000))*(sj2));
IkReal x1485=((cj0)*(py));
IkReal x1486=((px)*(sj0));
IkReal x1487=((IkReal(0.331000000000000))*(cj2));
IkReal x1488=((IkReal(0.810000000000000))*(x1483));
IkReal x1489=((pz)*(x1482));
evalcond[0]=((((x1482)*(x1487)))+(((IkReal(0.405000000000000))*(x1482)))+(((x1483)*(x1484)))+(pz));
evalcond[1]=((((x1482)*(x1485)))+(((pz)*(x1483)))+(((IkReal(-1.00000000000000))*(x1482)*(x1486)))+(x1484));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x1483)*(x1485)))+(((x1483)*(x1486)))+(x1487)+(x1489));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x1486)*(x1488)))+(((IkReal(-1.00000000000000))*(pp)))+(((x1485)*(x1488)))+(((IkReal(-0.810000000000000))*(x1489))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x1482)*(x1484)))+(((IkReal(0.405000000000000))*(x1483)))+(((IkReal(-1.00000000000000))*(x1485)))+(((x1483)*(x1487)))+(x1486));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x1490=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1490)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1490))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x1491=((sj0)*(sj1));
IkReal x1492=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1492)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1491)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1492)))+(((r02)*(sj2)*(x1491))));
IkReal gconst6;
IkReal x1493=((sj0)*(sj1));
IkReal x1494=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1494)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1493)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1494)))+(((r02)*(sj2)*(x1493))));
IkReal gconst7;
IkReal x1495=((cj0)*(r02));
IkReal x1496=((cj0)*(r00));
gconst7=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x1495)))+(((cj1)*(sj2)*(x1496)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1495)))+(((cj2)*(sj1)*(x1496))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x1497=((cj0)*(r02));
IkReal x1498=((cj0)*(r00));
gconst9=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(cj2)*(x1497)))+(((cj1)*(sj2)*(x1498)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1497)))+(((cj2)*(sj1)*(x1498))));
IkReal gconst10;
IkReal x1499=((sj0)*(sj1));
IkReal x1500=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1499)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1500)))+(((r02)*(sj2)*(x1499)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1500))));
IkReal gconst11;
IkReal x1501=((sj0)*(sj1));
IkReal x1502=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1502)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1501)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1502)))+(((r02)*(sj2)*(x1501))));
IkReal gconst12;
IkReal x1503=((cj0)*(r02));
IkReal x1504=((cj0)*(r00));
gconst12=((((cj2)*(sj1)*(x1504)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1503)))+(((cj1)*(cj2)*(x1503)))+(((cj1)*(sj2)*(x1504))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x1505=((cj0)*(r02));
IkReal x1506=((cj0)*(r00));
gconst14=((((cj2)*(sj1)*(x1506)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1505)))+(((cj1)*(cj2)*(x1505)))+(((cj1)*(sj2)*(x1506))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x1507=((sj0)*(sj1));
IkReal x1508=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1508)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1507)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1508)))+(((r02)*(sj2)*(x1507))));
IkReal gconst22;
IkReal x1509=((sj0)*(sj1));
IkReal x1510=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1510)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1509)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1510)))+(((r02)*(sj2)*(x1509))));
IkReal gconst23;
IkReal x1511=((cj0)*(r02));
IkReal x1512=((cj0)*(r00));
gconst23=((((cj1)*(cj2)*(x1511)))+(((cj1)*(sj2)*(x1512)))+(((cj2)*(sj1)*(x1512)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1511))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x1513=((cj0)*(r02));
IkReal x1514=((cj0)*(r00));
gconst25=((((cj1)*(cj2)*(x1513)))+(((cj1)*(sj2)*(x1514)))+(((cj2)*(sj1)*(x1514)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1513))));
IkReal gconst26;
IkReal x1515=((sj0)*(sj1));
IkReal x1516=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1516)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1515)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1516)))+(((r02)*(sj2)*(x1515))));
IkReal gconst27;
IkReal x1517=((sj0)*(sj1));
IkReal x1518=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1518)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1517)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1518)))+(((r02)*(sj2)*(x1517))));
IkReal gconst28;
IkReal x1519=((cj0)*(r02));
IkReal x1520=((cj0)*(r00));
gconst28=((((cj1)*(cj2)*(x1519)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1519)))+(((cj2)*(sj1)*(x1520)))+(((cj1)*(sj2)*(x1520))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x1521=((cj0)*(r02));
IkReal x1522=((cj0)*(r00));
gconst30=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1521)))+(((cj1)*(cj2)*(x1521)))+(((cj2)*(sj1)*(x1522)))+(((cj1)*(sj2)*(x1522))));
IkReal x1523=((gconst27)*(gconst28));
IkReal x1524=((gconst22)*(gconst23));
IkReal x1525=((gconst21)*(gconst25));
IkReal x1526=((gconst26)*(gconst30));
dummyeval[0]=((((x1524)*(x1526)))+(((IkReal(-1.00000000000000))*(x1525)*(x1526)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((x1523)*(x1525)))+(((IkReal(-1.00000000000000))*(x1523)*(x1524))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1527=((IkReal(1.00000000000000))*(gconst19));
IkReal x1528=((gconst23)*(gconst27));
IkReal x1529=((gconst15)*(gconst18));
IkReal x1530=((gconst11)*(gconst6));
IkReal x1531=((gconst13)*(gconst26));
IkReal x1532=((gconst15)*(gconst9));
IkReal x1533=((gconst6)*(gconst8));
IkReal x1534=((gconst18)*(gconst28));
IkReal x1535=((gconst22)*(gconst30));
IkReal x1536=((gconst16)*(gconst26));
IkReal x1537=((IkReal(1.00000000000000))*(gconst10));
IkReal x1538=((gconst14)*(gconst25));
IkReal x1539=((IkReal(1.00000000000000))*(gconst26));
IkReal x1540=((gconst12)*(gconst27));
IkReal x1541=((gconst25)*(gconst5));
IkReal x1542=((gconst17)*(gconst28));
IkReal x1543=((gconst21)*(gconst27));
IkReal x1544=((gconst27)*(gconst6));
IkReal x1545=((IkReal(1.00000000000000))*(gconst28));
IkReal x1546=((gconst11)*(gconst5));
IkReal x1547=((gconst17)*(gconst21));
IkReal x1548=((gconst18)*(gconst19));
IkReal x1549=((gconst10)*(gconst7));
IkReal x1550=((gconst30)*(gconst6));
IkReal x1551=((gconst17)*(gconst5));
IkReal x1552=((IkReal(1.00000000000000))*(gconst12));
IkReal x1553=((gconst22)*(gconst7));
IkReal x1554=((gconst24)*(gconst6));
IkReal x1555=((gconst5)*(gconst9));
IkReal x1556=((gconst15)*(gconst17));
IkReal x1557=((gconst27)*(gconst28));
IkReal x1558=((gconst23)*(gconst26));
IkReal x1559=((gconst21)*(gconst9));
IkReal x1560=((gconst22)*(gconst8));
IkReal x1561=((gconst10)*(gconst23));
IkReal x1562=((gconst12)*(gconst18));
IkReal x1563=((gconst21)*(gconst25));
IkReal x1564=((gconst11)*(gconst28));
IkReal x1565=((gconst11)*(gconst19));
IkReal x1566=((gconst19)*(gconst27));
IkReal x1567=((gconst20)*(gconst6));
IkReal x1568=((gconst10)*(gconst16));
IkReal x1569=((gconst26)*(gconst29));
IkReal x1570=((gconst26)*(gconst7));
IkReal x1571=((gconst20)*(gconst22));
IkReal x1572=((gconst22)*(gconst24));
IkReal x1573=((gconst15)*(gconst25));
IkReal x1574=((gconst10)*(gconst13));
IkReal x1575=((gconst14)*(gconst6));
IkReal x1576=((gconst16)*(gconst22));
IkReal x1577=((gconst11)*(gconst12));
IkReal x1578=((gconst14)*(gconst22));
IkReal x1579=((gconst17)*(gconst19));
IkReal x1580=((gconst10)*(gconst29));
IkReal x1581=((gconst28)*(gconst9));
IkReal x1582=((IkReal(1.00000000000000))*(gconst6));
IkReal x1583=((gconst26)*(x1567));
IkReal x1584=((gconst9)*(x1565));
IkReal x1585=((gconst11)*(x1576));
IkReal x1586=((gconst14)*(x1561));
IkReal x1587=((gconst11)*(gconst22)*(gconst23));
op[0]=((((IkReal(-1.00000000000000))*(gconst29)*(x1539)*(x1572)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1528)*(x1545)))+(((gconst25)*(gconst28)*(x1543)))+(((x1535)*(x1558)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1539)*(x1563))));
op[1]=((((IkReal(-1.00000000000000))*(gconst27)*(x1545)*(x1576)))+(((x1569)*(x1573)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1527)*(x1528)))+(((x1534)*(x1572)))+(((gconst19)*(gconst25)*(x1543)))+(((x1535)*(x1536)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1539)*(x1547)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1539)*(x1563)))+(((x1558)*(x1571)))+(((x1542)*(x1543))));
op[2]=((((IkReal(-1.00000000000000))*(x1531)*(x1572)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1528)*(x1545)))+(((IkReal(-1.00000000000000))*(gconst21)*(x1538)*(x1539)))+(((x1556)*(x1569)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1527)*(x1576)))+(((x1536)*(x1571)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1529)*(x1545)))+(((x1550)*(x1558)))+(((x1541)*(x1557)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1539)*(x1560)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1528)*(x1552)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1539)*(x1554)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1539)*(x1559)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1537)*(x1572)))+(((IkReal(-1.00000000000000))*(x1545)*(x1587)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1539)*(x1547)))+(((x1540)*(x1563)))+(((x1543)*(x1579)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1539)*(x1541)))+(((x1543)*(x1581)))+(((x1535)*(x1570)))+(((x1563)*(x1564)))+(((x1535)*(x1561)))+(((x1548)*(x1572)))+(((x1558)*(x1578)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1545)*(x1553)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1537)*(x1563))));
op[3]=((((x1532)*(x1569)))+(((x1531)*(x1573)))+(((x1536)*(x1550)))+(((IkReal(-1.00000000000000))*(x1529)*(x1542)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1527)*(x1529)))+(((x1541)*(x1566)))+(((x1536)*(x1578)))+(((gconst11)*(gconst21)*(x1542)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1527)*(x1553)))+(((gconst27)*(gconst5)*(x1542)))+(((x1573)*(x1580)))+(((x1534)*(x1560)))+(((x1534)*(x1554)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1539)*(x1547)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1539)*(x1559)))+(((gconst20)*(gconst26)*(x1553)))+(((x1561)*(x1571)))+(((x1540)*(x1547)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1539)*(x1551)))+(((IkReal(-1.00000000000000))*(x1545)*(x1585)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1539)*(x1541)))+(((IkReal(-1.00000000000000))*(x1527)*(x1587)))+(((IkReal(-1.00000000000000))*(x1540)*(x1576)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1544)*(x1545)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1527)*(x1528)))+(((x1562)*(x1572)))+(((x1563)*(x1565)))+(((x1535)*(x1568)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1537)*(x1563)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1537)*(x1547)))+(((x1558)*(x1567)))+(((gconst19)*(gconst9)*(x1543))));
op[4]=((((IkReal(-1.00000000000000))*(gconst25)*(x1529)*(x1552)))+(((IkReal(-1.00000000000000))*(gconst21)*(x1537)*(x1538)))+(((x1550)*(x1570)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1529)*(x1545)))+(((x1550)*(x1561)))+(((IkReal(-1.00000000000000))*(x1531)*(x1560)))+(((x1555)*(x1557)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1537)*(x1559)))+(((IkReal(-1.00000000000000))*(gconst17)*(x1527)*(x1529)))+(((x1541)*(x1564)))+(((x1556)*(x1580)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1544)*(x1545)))+(((x1547)*(x1565)))+(((x1536)*(x1567)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1537)*(x1572)))+(((IkReal(-1.00000000000000))*(x1552)*(x1587)))+(((IkReal(-1.00000000000000))*(x1531)*(x1554)))+(((gconst14)*(gconst26)*(x1553)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1527)*(x1544)))+(((x1568)*(x1571)))+(((x1540)*(x1559)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1530)*(x1545)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1539)*(x1551)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1537)*(x1554)))+(((x1561)*(x1578)))+(((x1540)*(x1541)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1539)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst5)*(x1538)*(x1539)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1539)*(x1559)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1533)*(x1539)))+(((IkReal(-1.00000000000000))*(x1527)*(x1585)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1545)*(x1553)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1537)*(x1560)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1537)*(x1547)))+(((x1548)*(x1554)))+(((x1563)*(x1577)))+(((x1535)*(x1549)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1528)*(x1552)))+(((x1558)*(x1575)))+(((x1559)*(x1564)))+(((x1548)*(x1560)))+(((x1551)*(x1566)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1537)*(x1541)))+(((IkReal(-1.00000000000000))*(x1540)*(x1553)))+(((x1531)*(x1556))));
op[5]=((((x1560)*(x1562)))+(((x1555)*(x1566)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1527)*(x1529)))+(((x1567)*(x1570)))+(((x1550)*(x1568)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1537)*(x1551)))+(((x1541)*(x1565)))+(((x1536)*(x1575)))+(((IkReal(-1.00000000000000))*(gconst17)*(x1529)*(x1552)))+(((x1532)*(x1580)))+(((x1547)*(x1577)))+(((x1573)*(x1574)))+(((IkReal(-1.00000000000000))*(x1552)*(x1585)))+(((x1531)*(x1532)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1527)*(x1530)))+(((x1554)*(x1562)))+(((x1549)*(x1571)))+(((x1561)*(x1567)))+(((x1568)*(x1578)))+(((x1540)*(x1551)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1539)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1539)*(x1551)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1537)*(x1541)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1530)*(x1545)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1540)*(x1582)))+(((x1533)*(x1534)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1527)*(x1553)))+(((x1559)*(x1565)))+(((x1542)*(x1546)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1527)*(x1544)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1537)*(x1547)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1537)*(x1559))));
op[6]=((((x1567)*(x1568)))+(((IkReal(-1.00000000000000))*(x1531)*(x1533)))+(((x1556)*(x1574)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1529)*(x1552)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1537)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1537)*(x1554)))+(((x1570)*(x1575)))+(((x1541)*(x1577)))+(((x1533)*(x1548)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1537)*(x1560)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1552)*(x1553)))+(((x1546)*(x1581)))+(((x1546)*(x1579)))+(((x1549)*(x1578)))+(((x1549)*(x1550)))+(((IkReal(-1.00000000000000))*(gconst5)*(x1537)*(x1538)))+(((x1540)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1527)*(x1530)))+(((x1561)*(x1575)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1539)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1533)*(x1537)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1530)*(x1545)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1540)*(x1582)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1530)*(x1552)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1537)*(x1559)))+(((x1559)*(x1577)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1537)*(x1551))));
op[7]=((((x1532)*(x1574)))+(((gconst12)*(gconst17)*(x1546)))+(((x1549)*(x1567)))+(((x1533)*(x1562)))+(((x1568)*(x1575)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1527)*(x1530)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1537)*(x1551)))+(((gconst19)*(gconst9)*(x1546)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1537)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1530)*(x1552))));
op[8]=((((x1549)*(x1575)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1530)*(x1552)))+(((gconst12)*(gconst9)*(x1546)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1537)*(x1555)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1533)*(x1537))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1588=((r01)*(sj3));
IkReal x1589=((IkReal(1.00000000000000))*(sj0));
IkReal x1590=((sj1)*(sj3));
IkReal x1591=((cj2)*(r00));
IkReal x1592=((r02)*(sj2));
IkReal x1593=((cj3)*(sj1));
IkReal x1594=((IkReal(1.00000000000000))*(cj0));
IkReal x1595=((cj3)*(r01));
IkReal x1596=((cj1)*(r00)*(sj2));
IkReal x1597=((cj1)*(cj2)*(r02)*(sj3));
IkReal x1598=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((sj0)*(sj3)*(x1596)))+(((cj0)*(x1591)*(x1593)))+(((IkReal(-1.00000000000000))*(x1592)*(x1593)*(x1594)))+(((IkReal(-1.00000000000000))*(x1589)*(x1595)))+(((sj0)*(x1590)*(x1591)))+(((sj0)*(x1597)))+(((cj0)*(x1598)))+(((cj0)*(x1588)))+(((IkReal(-1.00000000000000))*(x1589)*(x1590)*(x1592)))+(((cj0)*(cj3)*(x1596))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x1594)*(x1595)))+(((cj0)*(x1590)*(x1591)))+(((sj0)*(x1592)*(x1593)))+(((IkReal(-1.00000000000000))*(x1588)*(x1589)))+(((IkReal(-1.00000000000000))*(x1589)*(x1598)))+(((IkReal(-1.00000000000000))*(x1590)*(x1592)*(x1594)))+(((cj0)*(x1597)))+(((cj0)*(sj3)*(x1596)))+(((IkReal(-1.00000000000000))*(cj3)*(x1589)*(x1596)))+(((IkReal(-1.00000000000000))*(x1589)*(x1591)*(x1593))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((sj0)*(sj3)*(x1596)))+(((cj0)*(x1591)*(x1593)))+(((IkReal(-1.00000000000000))*(x1592)*(x1593)*(x1594)))+(((IkReal(-1.00000000000000))*(x1589)*(x1595)))+(((sj0)*(x1590)*(x1591)))+(((sj0)*(x1597)))+(((cj0)*(x1598)))+(((cj0)*(x1588)))+(((IkReal(-1.00000000000000))*(x1589)*(x1590)*(x1592)))+(((cj0)*(cj3)*(x1596)))))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x1594)*(x1595)))+(((cj0)*(x1590)*(x1591)))+(((sj0)*(x1592)*(x1593)))+(((IkReal(-1.00000000000000))*(x1588)*(x1589)))+(((IkReal(-1.00000000000000))*(x1589)*(x1598)))+(((IkReal(-1.00000000000000))*(x1590)*(x1592)*(x1594)))+(((cj0)*(x1597)))+(((cj0)*(sj3)*(x1596)))+(((IkReal(-1.00000000000000))*(cj3)*(x1589)*(x1596)))+(((IkReal(-1.00000000000000))*(x1589)*(x1591)*(x1593)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1599=IKsin(j5);
IkReal x1600=IKcos(j5);
IkReal x1601=((IkReal(1.00000000000000))*(cj0));
IkReal x1602=((IkReal(1.00000000000000))*(cj3));
IkReal x1603=((cj0)*(cj1));
IkReal x1604=((cj2)*(r02));
IkReal x1605=((r00)*(sj2));
IkReal x1606=((IkReal(1.00000000000000))*(sj0));
IkReal x1607=((r02)*(sj1)*(sj2));
IkReal x1608=((cj2)*(r00)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1601)*(x1607)))+(((cj0)*(x1608)))+(((IkReal(-1.00000000000000))*(x1599)*(x1602)))+(((IkReal(-1.00000000000000))*(r01)*(x1606)))+(((x1603)*(x1605)))+(((x1603)*(x1604)))+(((IkReal(-1.00000000000000))*(sj3)*(x1600))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x1604)*(x1606)))+(((sj0)*(x1607)))+(((IkReal(-1.00000000000000))*(x1600)*(x1602)))+(((IkReal(-1.00000000000000))*(r01)*(x1601)))+(((IkReal(-1.00000000000000))*(x1606)*(x1608)))+(((IkReal(-1.00000000000000))*(cj1)*(x1605)*(x1606)))+(((sj3)*(x1599))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x1609=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1609)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1609)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x1610=((sj0)*(sj1));
IkReal x1611=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1611)))+(((r02)*(sj2)*(x1610)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1610)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1611))));
IkReal gconst49;
IkReal x1612=((sj0)*(sj1));
IkReal x1613=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1613)))+(((r02)*(sj2)*(x1612)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1612)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1613))));
IkReal gconst50;
IkReal x1614=((cj0)*(r02));
IkReal x1615=((cj0)*(r00));
gconst50=((((cj1)*(cj2)*(x1614)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1614)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1615)))+(((cj2)*(sj1)*(x1615))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x1616=((cj0)*(r02));
IkReal x1617=((cj0)*(r00));
gconst52=((((cj1)*(cj2)*(x1616)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1616)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1617)))+(((cj2)*(sj1)*(x1617))));
IkReal gconst53;
IkReal x1618=((sj0)*(sj1));
IkReal x1619=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1619)))+(((r02)*(sj2)*(x1618)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1618)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1619))));
IkReal gconst54;
IkReal x1620=((sj0)*(sj1));
IkReal x1621=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1621)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1620)))+(((r02)*(sj2)*(x1620)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1621))));
IkReal gconst55;
IkReal x1622=((cj0)*(r02));
IkReal x1623=((cj0)*(r00));
gconst55=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1622)))+(((cj1)*(sj2)*(x1623)))+(((cj2)*(sj1)*(x1623)))+(((cj1)*(cj2)*(x1622))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x1624=((cj0)*(r02));
IkReal x1625=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1624)))+(((cj1)*(sj2)*(x1625)))+(((cj2)*(sj1)*(x1625)))+(((cj1)*(cj2)*(x1624))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x1626=((sj0)*(sj1));
IkReal x1627=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1627)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1626)))+(((r02)*(sj2)*(x1626)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1627))));
IkReal gconst65;
IkReal x1628=((sj0)*(sj1));
IkReal x1629=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1629)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1628)))+(((r02)*(sj2)*(x1628)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1629))));
IkReal gconst66;
IkReal x1630=((cj0)*(r02));
IkReal x1631=((cj0)*(r00));
gconst66=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1630)))+(((cj2)*(sj1)*(x1631)))+(((cj1)*(cj2)*(x1630)))+(((cj1)*(sj2)*(x1631))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x1632=((cj0)*(r02));
IkReal x1633=((cj0)*(r00));
gconst68=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1632)))+(((cj2)*(sj1)*(x1633)))+(((cj1)*(cj2)*(x1632)))+(((cj1)*(sj2)*(x1633))));
IkReal gconst69;
IkReal x1634=((sj0)*(sj1));
IkReal x1635=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1635)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1634)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1635)))+(((r02)*(sj2)*(x1634)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst70;
IkReal x1636=((sj0)*(sj1));
IkReal x1637=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1637)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1636)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1637)))+(((r02)*(sj2)*(x1636)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst71;
IkReal x1638=((cj0)*(r02));
IkReal x1639=((cj0)*(r00));
gconst71=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1638)))+(((cj2)*(sj1)*(x1639)))+(((cj1)*(cj2)*(x1638)))+(((cj1)*(sj2)*(x1639))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x1640=((cj0)*(r02));
IkReal x1641=((cj0)*(r00));
gconst73=((((cj1)*(sj2)*(x1641)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x1641)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1640)))+(((cj1)*(cj2)*(x1640))));
IkReal x1642=((gconst70)*(gconst71));
IkReal x1643=((IkReal(1.00000000000000))*(gconst65));
IkReal x1644=((gconst64)*(gconst68));
IkReal x1645=((gconst69)*(gconst73));
dummyeval[0]=((((gconst65)*(gconst66)*(x1645)))+(((IkReal(-1.00000000000000))*(x1644)*(x1645)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1642)*(x1643)))+(((x1642)*(x1644)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x1643))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1646=((gconst62)*(gconst70));
IkReal x1647=((IkReal(1.00000000000000))*(gconst49));
IkReal x1648=((gconst53)*(gconst73));
IkReal x1649=((gconst48)*(gconst52));
IkReal x1650=((gconst65)*(gconst66));
IkReal x1651=((IkReal(1.00000000000000))*(gconst65));
IkReal x1652=((gconst53)*(gconst56));
IkReal x1653=((gconst49)*(gconst50));
IkReal x1654=((gconst55)*(gconst70));
IkReal x1655=((gconst54)*(gconst62));
IkReal x1656=((gconst59)*(gconst69));
IkReal x1657=((gconst49)*(gconst57));
IkReal x1658=((gconst52)*(gconst63));
IkReal x1659=((gconst60)*(gconst73));
IkReal x1660=((gconst52)*(gconst58));
IkReal x1661=((gconst56)*(gconst69));
IkReal x1662=((gconst54)*(gconst55));
IkReal x1663=((gconst63)*(gconst65));
IkReal x1664=((gconst63)*(gconst69));
IkReal x1665=((gconst49)*(gconst66));
IkReal x1666=((IkReal(1.00000000000000))*(gconst48));
IkReal x1667=((gconst52)*(gconst64));
IkReal x1668=((gconst70)*(gconst71));
IkReal x1669=((gconst68)*(gconst69));
IkReal x1670=((IkReal(1.00000000000000))*(gconst68));
IkReal x1671=((gconst54)*(gconst71));
IkReal x1672=((gconst48)*(gconst60));
IkReal x1673=((gconst53)*(gconst72));
IkReal x1674=((gconst53)*(gconst59));
IkReal x1675=((gconst58)*(gconst61));
IkReal x1676=((gconst57)*(gconst65));
IkReal x1677=((gconst53)*(gconst63));
IkReal x1678=((gconst61)*(gconst71));
IkReal x1679=((gconst65)*(gconst67));
IkReal x1680=((gconst48)*(gconst68));
IkReal x1681=((gconst53)*(gconst57));
IkReal x1682=((gconst60)*(gconst64));
IkReal x1683=((gconst69)*(gconst72));
IkReal x1684=((gconst58)*(gconst68));
IkReal x1685=((gconst50)*(gconst69));
IkReal x1686=((IkReal(1.00000000000000))*(gconst60));
IkReal x1687=((gconst69)*(gconst73));
IkReal x1688=((gconst57)*(gconst69));
IkReal x1689=((gconst49)*(gconst63));
IkReal x1690=((gconst50)*(gconst53));
IkReal x1691=((gconst55)*(gconst61));
IkReal x1692=((gconst65)*(gconst73));
IkReal x1693=((gconst61)*(gconst62));
IkReal x1694=((gconst64)*(gconst68));
IkReal x1695=((gconst58)*(gconst60));
IkReal x1696=((gconst49)*(gconst67));
IkReal x1697=((gconst60)*(x1662));
IkReal x1698=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x1699=((gconst72)*(x1695));
IkReal x1700=((gconst49)*(gconst51)*(gconst61));
IkReal x1701=((gconst51)*(gconst61)*(gconst65));
op[0]=((((x1668)*(x1694)))+(((IkReal(-1.00000000000000))*(x1650)*(x1668)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x1669)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1651)*(x1683)))+(((x1650)*(x1687))));
op[1]=((((x1650)*(x1664)))+(((x1678)*(x1679)))+(((x1668)*(x1682)))+(((x1656)*(x1692)))+(((gconst58)*(gconst72)*(x1669)))+(((x1646)*(x1694)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1664)*(x1670)))+(((IkReal(-1.00000000000000))*(x1646)*(x1650)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1651)*(x1668)))+(((IkReal(-1.00000000000000))*(x1659)*(x1698))));
op[2]=((((x1671)*(x1694)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1670)*(x1675)))+(((x1683)*(x1695)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1647)*(x1668)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1651)*(x1683)))+(((x1648)*(x1650)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1651)*(x1668)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x1669)))+(((x1668)*(x1680)))+(((IkReal(-1.00000000000000))*(x1667)*(x1687)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1647)*(x1683)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1646)*(x1651)))+(((IkReal(-1.00000000000000))*(x1650)*(x1654)))+(((IkReal(-1.00000000000000))*(x1664)*(x1682)))+(((x1679)*(x1693)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1651)*(x1673)))+(((x1667)*(x1668)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1648)*(x1670)))+(((x1685)*(x1692)))+(((x1646)*(x1682)))+(((IkReal(-1.00000000000000))*(gconst73)*(x1666)*(x1669)))+(((x1665)*(x1687)))+(((x1654)*(x1694)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1651)*(x1661)))+(((IkReal(-1.00000000000000))*(x1650)*(x1671)))+(((x1650)*(x1688)))+(((x1656)*(x1663))));
op[3]=((((IkReal(-1.00000000000000))*(gconst69)*(x1659)*(x1666)))+(((x1654)*(x1682)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1651)*(x1654)))+(((x1655)*(x1694)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1675)*(x1686)))+(((gconst49)*(gconst73)*(x1656)))+(((x1673)*(x1684)))+(((x1650)*(x1677)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1646)*(x1647)))+(((x1671)*(x1682)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1651)*(x1671)))+(((x1668)*(x1672)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1664)*(x1666)))+(((IkReal(-1.00000000000000))*(x1650)*(x1655)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1670)*(x1677)))+(((x1646)*(x1667)))+(((x1679)*(x1691)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1670)*(x1675)))+(((x1660)*(x1683)))+(((x1663)*(x1685)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1646)*(x1651)))+(((x1646)*(x1680)))+(((gconst51)*(gconst65)*(x1678)))+(((x1661)*(x1684)))+(((IkReal(-1.00000000000000))*(x1682)*(x1688)))+(((x1656)*(x1676)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1647)*(x1668)))+(((IkReal(-1.00000000000000))*(x1658)*(x1698)))+(((x1664)*(x1665)))+(((IkReal(-1.00000000000000))*(x1648)*(x1682)))+(((x1678)*(x1696)))+(((gconst59)*(gconst65)*(x1648))));
op[4]=((((IkReal(-1.00000000000000))*(gconst67)*(x1647)*(x1661)))+(((x1654)*(x1680)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1651)*(x1655)))+(((IkReal(-1.00000000000000))*(x1660)*(x1678)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1670)*(x1675)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1647)*(x1654)))+(((x1656)*(x1689)))+(((x1653)*(x1687)))+(((x1648)*(x1665)))+(((x1673)*(x1695)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1647)*(x1673)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1646)*(x1647)))+(((x1654)*(x1667)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1651)*(x1654)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1647)*(x1671)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1675)*(x1686)))+(((x1671)*(x1680)))+(((x1667)*(x1671)))+(((IkReal(-1.00000000000000))*(x1667)*(x1688)))+(((IkReal(-1.00000000000000))*(x1650)*(x1662)))+(((x1676)*(x1685)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1647)*(x1668)))+(((IkReal(-1.00000000000000))*(x1648)*(x1667)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1651)*(x1671)))+(((x1662)*(x1694)))+(((IkReal(-1.00000000000000))*(x1649)*(x1687)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1670)*(x1681)))+(((x1649)*(x1668)))+(((x1693)*(x1696)))+(((x1661)*(x1695)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1648)*(x1666)))+(((IkReal(-1.00000000000000))*(x1677)*(x1682)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1651)*(x1652)))+(((IkReal(-1.00000000000000))*(gconst57)*(x1666)*(x1669)))+(((x1655)*(x1682)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1651)*(x1661)))+(((gconst51)*(gconst65)*(x1693)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1651)*(x1673)))+(((x1663)*(x1674)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1664)*(x1666)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1647)*(x1683)))+(((x1646)*(x1672)))+(((gconst50)*(gconst65)*(x1648)))+(((x1650)*(x1681)))+(((gconst66)*(gconst69)*(x1657))));
op[5]=((((x1656)*(x1657)))+(((x1660)*(x1661)))+(((x1652)*(x1684)))+(((x1665)*(x1677)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1647)*(x1655)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1666)*(x1688)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x1658)))+(((x1674)*(x1676)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1651)*(x1655)))+(((x1655)*(x1667)))+(((x1691)*(x1696)))+(((x1662)*(x1682)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1646)*(x1647)))+(((IkReal(-1.00000000000000))*(x1681)*(x1682)))+(((gconst49)*(gconst59)*(x1648)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1648)*(x1666)))+(((x1654)*(x1672)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1666)*(x1677)))+(((x1646)*(x1649)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1647)*(x1671)))+(((IkReal(-1.00000000000000))*(x1660)*(x1693)))+(((gconst49)*(gconst51)*(x1678)))+(((x1655)*(x1680)))+(((x1660)*(x1673)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1675)*(x1686)))+(((x1671)*(x1672)))+(((x1663)*(x1690)))+(((gconst51)*(gconst65)*(x1691)))+(((IkReal(-1.00000000000000))*(x1649)*(x1664)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1647)*(x1654)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1651)*(x1662)))+(((x1653)*(x1664))));
op[6]=((((x1652)*(x1695)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1647)*(x1673)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1647)*(x1662)))+(((x1653)*(x1688)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1666)*(x1677)))+(((x1648)*(x1653)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1651)*(x1662)))+(((IkReal(-1.00000000000000))*(x1667)*(x1681)))+(((x1662)*(x1667)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1647)*(x1661)))+(((gconst53)*(gconst66)*(x1657)))+(((x1662)*(x1680)))+(((x1676)*(x1690)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1647)*(x1654)))+(((x1674)*(x1689)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1666)*(x1681)))+(((IkReal(-1.00000000000000))*(x1648)*(x1649)))+(((x1655)*(x1672)))+(((x1649)*(x1654)))+(((IkReal(-1.00000000000000))*(x1649)*(x1688)))+(((IkReal(-1.00000000000000))*(x1660)*(x1691)))+(((x1649)*(x1671)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1651)*(x1652)))+(((gconst49)*(gconst51)*(x1693)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1647)*(x1671)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1647)*(x1652)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1647)*(x1655))));
op[7]=((((IkReal(-1.00000000000000))*(gconst60)*(x1666)*(x1681)))+(((IkReal(-1.00000000000000))*(x1649)*(x1677)))+(((x1653)*(x1677)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1647)*(x1655)))+(((x1662)*(x1672)))+(((x1649)*(x1655)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1647)*(x1662)))+(((gconst49)*(gconst51)*(x1691)))+(((x1652)*(x1660)))+(((x1657)*(x1674))));
op[8]=((((x1653)*(x1681)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1647)*(x1662)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1647)*(x1652)))+(((IkReal(-1.00000000000000))*(x1649)*(x1681)))+(((x1649)*(x1662))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1702=((sj1)*(sj3));
IkReal x1703=((IkReal(1.00000000000000))*(cj0));
IkReal x1704=((cj2)*(r00));
IkReal x1705=((cj3)*(sj0));
IkReal x1706=((cj0)*(sj3));
IkReal x1707=((cj3)*(sj1));
IkReal x1708=((r02)*(sj2));
IkReal x1709=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x1710=((cj1)*(r00)*(sj2));
IkReal x1711=((sj0)*(x1708));
IkReal x1712=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1703)))+(((IkReal(-1.00000000000000))*(cj3)*(x1703)*(x1712)))+(((IkReal(-1.00000000000000))*(cj3)*(x1703)*(x1710)))+(((IkReal(-1.00000000000000))*(x1709)*(x1712)))+(((IkReal(-1.00000000000000))*(x1709)*(x1710)))+(((r01)*(x1705)))+(((cj0)*(x1707)*(x1708)))+(((IkReal(-1.00000000000000))*(x1703)*(x1704)*(x1707)))+(((x1702)*(x1711)))+(((IkReal(-1.00000000000000))*(sj0)*(x1702)*(x1704))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((IkReal(-1.00000000000000))*(x1702)*(x1703)*(x1708)))+(((IkReal(-1.00000000000000))*(sj1)*(x1704)*(x1705)))+(((sj1)*(x1705)*(x1708)))+(((IkReal(-1.00000000000000))*(x1705)*(x1710)))+(((IkReal(-1.00000000000000))*(x1705)*(x1712)))+(((x1706)*(x1710)))+(((x1706)*(x1712)))+(((IkReal(-1.00000000000000))*(r01)*(x1709)))+(((cj0)*(x1702)*(x1704)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1703))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1703)))+(((IkReal(-1.00000000000000))*(cj3)*(x1703)*(x1712)))+(((IkReal(-1.00000000000000))*(cj3)*(x1703)*(x1710)))+(((IkReal(-1.00000000000000))*(x1709)*(x1712)))+(((IkReal(-1.00000000000000))*(x1709)*(x1710)))+(((r01)*(x1705)))+(((cj0)*(x1707)*(x1708)))+(((IkReal(-1.00000000000000))*(x1703)*(x1704)*(x1707)))+(((x1702)*(x1711)))+(((IkReal(-1.00000000000000))*(sj0)*(x1702)*(x1704)))))), ((gconst90)*(((((IkReal(-1.00000000000000))*(x1702)*(x1703)*(x1708)))+(((IkReal(-1.00000000000000))*(sj1)*(x1704)*(x1705)))+(((sj1)*(x1705)*(x1708)))+(((IkReal(-1.00000000000000))*(x1705)*(x1710)))+(((IkReal(-1.00000000000000))*(x1705)*(x1712)))+(((x1706)*(x1710)))+(((x1706)*(x1712)))+(((IkReal(-1.00000000000000))*(r01)*(x1709)))+(((cj0)*(x1702)*(x1704)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1703)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1713=IKcos(j5);
IkReal x1714=IKsin(j5);
IkReal x1715=((IkReal(1.00000000000000))*(cj0));
IkReal x1716=((IkReal(1.00000000000000))*(sj0));
IkReal x1717=((r02)*(sj1)*(sj2));
IkReal x1718=((IkReal(1.00000000000000))*(x1713));
IkReal x1719=((cj1)*(cj2)*(r02));
IkReal x1720=((cj2)*(r00)*(sj1));
IkReal x1721=((cj1)*(r00)*(sj2));
evalcond[0]=((((cj0)*(x1721)))+(((cj0)*(x1720)))+(((IkReal(-1.00000000000000))*(sj3)*(x1718)))+(((cj0)*(x1719)))+(((IkReal(-1.00000000000000))*(x1715)*(x1717)))+(((cj3)*(x1714)))+(((IkReal(-1.00000000000000))*(r01)*(x1716))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj3)*(x1718)))+(((IkReal(-1.00000000000000))*(x1716)*(x1719)))+(((IkReal(-1.00000000000000))*(sj3)*(x1714)))+(((IkReal(-1.00000000000000))*(x1716)*(x1721)))+(((IkReal(-1.00000000000000))*(x1716)*(x1720)))+(((sj0)*(x1717)))+(((IkReal(-1.00000000000000))*(r01)*(x1715))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x1722=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1723=((cj1)*(x1722));
IkReal x1724=((IkReal(1.00000000000000))*(r02));
IkReal x1725=((sj1)*(x1722));
sj5array[0]=((((cj2)*(r00)*(x1723)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1725)))+(((IkReal(-1.00000000000000))*(sj2)*(x1723)*(x1724)))+(((IkReal(-1.00000000000000))*(cj2)*(x1724)*(x1725))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x1726=((cj0)*(cj5));
IkReal x1727=((IkReal(1.00000000000000))*(r01));
IkReal x1728=((cj4)*(sj0)*(sj5));
IkReal x1729=((cj2)*(r00)*(sj1));
IkReal x1730=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x1731=((cj0)*(cj4)*(sj5));
IkReal x1732=((cj1)*(r00)*(sj2));
IkReal x1733=((cj1)*(cj2)*(r02));
IkReal x1734=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x1728)*(x1733)))+(((x1728)*(x1732)))+(((r01)*(x1731)))+(((x1728)*(x1729)))+(((IkReal(-1.00000000000000))*(x1728)*(x1734)))+(((IkReal(-1.00000000000000))*(x1726)*(x1734)))+(((x1726)*(x1729)))+(((x1726)*(x1732)))+(((x1726)*(x1733)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1727))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x1731)*(x1734)))+(((x1731)*(x1732)))+(((x1731)*(x1733)))+(((IkReal(-1.00000000000000))*(x1726)*(x1727)))+(((IkReal(-1.00000000000000))*(x1729)*(x1730)))+(((x1729)*(x1731)))+(((IkReal(-1.00000000000000))*(x1730)*(x1732)))+(((IkReal(-1.00000000000000))*(x1730)*(x1733)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1727)*(x1728))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x1728)*(x1733)))+(((x1728)*(x1732)))+(((r01)*(x1731)))+(((x1728)*(x1729)))+(((IkReal(-1.00000000000000))*(x1728)*(x1734)))+(((IkReal(-1.00000000000000))*(x1726)*(x1734)))+(((x1726)*(x1729)))+(((x1726)*(x1732)))+(((x1726)*(x1733)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1727)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x1731)*(x1734)))+(((x1731)*(x1732)))+(((x1731)*(x1733)))+(((IkReal(-1.00000000000000))*(x1726)*(x1727)))+(((IkReal(-1.00000000000000))*(x1729)*(x1730)))+(((x1729)*(x1731)))+(((IkReal(-1.00000000000000))*(x1730)*(x1732)))+(((IkReal(-1.00000000000000))*(x1730)*(x1733)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x1727)*(x1728)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x1735=IKsin(j3);
IkReal x1736=IKcos(j3);
IkReal x1737=((IkReal(1.00000000000000))*(cj0));
IkReal x1738=((cj0)*(cj1));
IkReal x1739=((cj2)*(r02));
IkReal x1740=((r00)*(sj2));
IkReal x1741=((IkReal(1.00000000000000))*(sj0));
IkReal x1742=((cj4)*(sj5));
IkReal x1743=((r02)*(sj1)*(sj2));
IkReal x1744=((cj2)*(r00)*(sj1));
IkReal x1745=((IkReal(1.00000000000000))*(x1736));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x1735)))+(((IkReal(-1.00000000000000))*(x1737)*(x1743)))+(((x1738)*(x1739)))+(((IkReal(-1.00000000000000))*(r01)*(x1741)))+(((cj0)*(x1744)))+(((IkReal(-1.00000000000000))*(x1742)*(x1745)))+(((x1738)*(x1740))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1741)*(x1744)))+(((IkReal(-1.00000000000000))*(cj5)*(x1745)))+(((IkReal(-1.00000000000000))*(r01)*(x1737)))+(((IkReal(-1.00000000000000))*(cj1)*(x1740)*(x1741)))+(((IkReal(-1.00000000000000))*(cj1)*(x1739)*(x1741)))+(((sj0)*(x1743)))+(((x1735)*(x1742))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x1746=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1746)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1746))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x1747=((cj0)*(cj5));
IkReal x1748=((IkReal(1.00000000000000))*(r01));
IkReal x1749=((cj4)*(sj0)*(sj5));
IkReal x1750=((cj2)*(r00)*(sj1));
IkReal x1751=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x1752=((cj0)*(cj4)*(sj5));
IkReal x1753=((cj1)*(r00)*(sj2));
IkReal x1754=((cj1)*(cj2)*(r02));
IkReal x1755=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x1749)*(x1754)))+(((x1749)*(x1753)))+(((x1749)*(x1750)))+(((x1747)*(x1754)))+(((x1747)*(x1753)))+(((x1747)*(x1750)))+(((IkReal(-1.00000000000000))*(x1749)*(x1755)))+(((IkReal(-1.00000000000000))*(x1747)*(x1755)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1748)))+(((r01)*(x1752))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x1752)*(x1754)))+(((x1752)*(x1753)))+(((IkReal(-1.00000000000000))*(x1748)*(x1749)))+(((x1750)*(x1752)))+(((IkReal(-1.00000000000000))*(x1750)*(x1751)))+(((IkReal(-1.00000000000000))*(x1751)*(x1754)))+(((IkReal(-1.00000000000000))*(x1751)*(x1753)))+(((IkReal(-1.00000000000000))*(x1747)*(x1748)))+(((IkReal(-1.00000000000000))*(x1752)*(x1755)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x1749)*(x1754)))+(((x1749)*(x1753)))+(((x1749)*(x1750)))+(((x1747)*(x1754)))+(((x1747)*(x1753)))+(((x1747)*(x1750)))+(((IkReal(-1.00000000000000))*(x1749)*(x1755)))+(((IkReal(-1.00000000000000))*(x1747)*(x1755)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x1748)))+(((r01)*(x1752)))))), ((gconst4)*(((((x1752)*(x1754)))+(((x1752)*(x1753)))+(((IkReal(-1.00000000000000))*(x1748)*(x1749)))+(((x1750)*(x1752)))+(((IkReal(-1.00000000000000))*(x1750)*(x1751)))+(((IkReal(-1.00000000000000))*(x1751)*(x1754)))+(((IkReal(-1.00000000000000))*(x1751)*(x1753)))+(((IkReal(-1.00000000000000))*(x1747)*(x1748)))+(((IkReal(-1.00000000000000))*(x1752)*(x1755)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x1756=IKsin(j3);
IkReal x1757=IKcos(j3);
IkReal x1758=((IkReal(1.00000000000000))*(cj0));
IkReal x1759=((cj0)*(cj1));
IkReal x1760=((cj2)*(r02));
IkReal x1761=((r00)*(sj2));
IkReal x1762=((IkReal(1.00000000000000))*(sj0));
IkReal x1763=((cj4)*(sj5));
IkReal x1764=((r02)*(sj1)*(sj2));
IkReal x1765=((cj2)*(r00)*(sj1));
IkReal x1766=((IkReal(1.00000000000000))*(x1757));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1762)))+(((x1759)*(x1760)))+(((x1759)*(x1761)))+(((IkReal(-1.00000000000000))*(cj5)*(x1756)))+(((cj0)*(x1765)))+(((IkReal(-1.00000000000000))*(x1763)*(x1766)))+(((IkReal(-1.00000000000000))*(x1758)*(x1764))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1762)*(x1765)))+(((sj0)*(x1764)))+(((x1756)*(x1763)))+(((IkReal(-1.00000000000000))*(r01)*(x1758)))+(((IkReal(-1.00000000000000))*(cj5)*(x1766)))+(((IkReal(-1.00000000000000))*(cj1)*(x1761)*(x1762)))+(((IkReal(-1.00000000000000))*(cj1)*(x1760)*(x1762))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x1767=((IkReal(1000.00000000000))*(pz));
if( IKabs(((gconst3)*(((((cj0)*(py)*(x1767)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x1767))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x1767)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst3)*(((((cj0)*(py)*(x1767)))+(((IkReal(134.055000000000))*(sj2)))+(((IkReal(109.561000000000))*(cj2)*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(sj0)*(x1767)))))), ((gconst3)*(((IkReal(-164.025000000000))+(((pz)*(x1767)))+(((IkReal(-268.110000000000))*(cj2)))+(((IkReal(-109.561000000000))*((cj2)*(cj2))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x1768=IKsin(j1);
IkReal x1769=IKcos(j1);
IkReal x1770=((IkReal(0.331000000000000))*(sj2));
IkReal x1771=((cj0)*(py));
IkReal x1772=((px)*(sj0));
IkReal x1773=((IkReal(0.331000000000000))*(cj2));
IkReal x1774=((IkReal(0.810000000000000))*(x1769));
IkReal x1775=((pz)*(x1768));
evalcond[0]=((((IkReal(0.405000000000000))*(x1768)))+(pz)+(((x1768)*(x1773)))+(((x1769)*(x1770))));
evalcond[1]=((x1770)+(((pz)*(x1769)))+(((IkReal(-1.00000000000000))*(x1768)*(x1772)))+(((x1768)*(x1771))));
evalcond[2]=((IkReal(0.405000000000000))+(x1775)+(x1773)+(((IkReal(-1.00000000000000))*(x1769)*(x1771)))+(((x1769)*(x1772))));
evalcond[3]=((IkReal(-0.0444339775000000))+(((IkReal(-1.00000000000000))*(x1772)*(x1774)))+(((x1771)*(x1774)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.810000000000000))*(x1775))));
evalcond[4]=((((IkReal(0.405000000000000))*(x1769)))+(((IkReal(-1.00000000000000))*(x1771)))+(x1772)+(((IkReal(-1.00000000000000))*(x1768)*(x1770)))+(((x1769)*(x1773))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[2];
IkReal x1776=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1776)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1776))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
IkReal x1777=((sj0)*(sj1));
IkReal x1778=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst5=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1777)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1778)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1778)))+(((r02)*(sj2)*(x1777))));
IkReal gconst6;
IkReal x1779=((sj0)*(sj1));
IkReal x1780=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst6=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1780)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1780)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1779)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((r02)*(sj2)*(x1779))));
IkReal gconst7;
IkReal x1781=((cj0)*(r02));
IkReal x1782=((cj0)*(r00));
gconst7=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1781)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1782)))+(((cj2)*(sj1)*(x1782)))+(((cj1)*(cj2)*(x1781))));
IkReal gconst8;
gconst8=IkReal(-2.00000000000000);
IkReal gconst9;
IkReal x1783=((cj0)*(r02));
IkReal x1784=((cj0)*(r00));
gconst9=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1783)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1784)))+(((cj2)*(sj1)*(x1784)))+(((cj1)*(cj2)*(x1783))));
IkReal gconst10;
IkReal x1785=((sj0)*(sj1));
IkReal x1786=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst10=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1786)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1786)))+(((r02)*(sj2)*(x1785)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1785))));
IkReal gconst11;
IkReal x1787=((sj0)*(sj1));
IkReal x1788=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst11=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1788)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1788)))+(((r02)*(sj2)*(x1787)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1787))));
IkReal gconst12;
IkReal x1789=((cj0)*(r02));
IkReal x1790=((cj0)*(r00));
gconst12=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1789)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1790)))+(((cj1)*(cj2)*(x1789)))+(((cj2)*(sj1)*(x1790))));
IkReal gconst13;
gconst13=IkReal(-2.00000000000000);
IkReal gconst14;
IkReal x1791=((cj0)*(r02));
IkReal x1792=((cj0)*(r00));
gconst14=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1792)))+(((cj2)*(sj1)*(x1792)))+(((cj1)*(cj2)*(x1791)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1791))));
IkReal gconst15;
gconst15=IkReal(4.00000000000000);
IkReal gconst16;
gconst16=IkReal(-2.00000000000000);
IkReal gconst17;
gconst17=IkReal(2.00000000000000);
IkReal gconst18;
gconst18=IkReal(4.00000000000000);
IkReal gconst19;
gconst19=IkReal(-2.00000000000000);
IkReal gconst20;
gconst20=IkReal(2.00000000000000);
IkReal gconst21;
IkReal x1793=((sj0)*(sj1));
IkReal x1794=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst21=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1794)))+(((r02)*(sj2)*(x1793)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1794)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1793))));
IkReal gconst22;
IkReal x1795=((sj0)*(sj1));
IkReal x1796=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst22=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1796)))+(((r02)*(sj2)*(x1795)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1796)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1795))));
IkReal gconst23;
IkReal x1797=((cj0)*(r02));
IkReal x1798=((cj0)*(r00));
gconst23=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1798)))+(((cj2)*(sj1)*(x1798)))+(((cj1)*(cj2)*(x1797)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1797))));
IkReal gconst24;
gconst24=IkReal(2.00000000000000);
IkReal gconst25;
IkReal x1799=((cj0)*(r02));
IkReal x1800=((cj0)*(r00));
gconst25=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x1800)))+(((cj1)*(sj2)*(x1800)))+(((cj1)*(cj2)*(x1799)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1799))));
IkReal gconst26;
IkReal x1801=((sj0)*(sj1));
IkReal x1802=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst26=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x1801)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1802)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1801)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1802))));
IkReal gconst27;
IkReal x1803=((sj0)*(sj1));
IkReal x1804=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst27=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x1803)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1804)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1803)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1804))));
IkReal gconst28;
IkReal x1805=((cj0)*(r02));
IkReal x1806=((cj0)*(r00));
gconst28=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x1806)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1805)))+(((cj1)*(cj2)*(x1805)))+(((cj1)*(sj2)*(x1806))));
IkReal gconst29;
gconst29=IkReal(2.00000000000000);
IkReal gconst30;
IkReal x1807=((cj0)*(r02));
IkReal x1808=((cj0)*(r00));
gconst30=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj2)*(sj1)*(x1808)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1807)))+(((cj1)*(cj2)*(x1807)))+(((cj1)*(sj2)*(x1808))));
IkReal x1809=((gconst27)*(gconst28));
IkReal x1810=((gconst22)*(gconst23));
IkReal x1811=((gconst21)*(gconst25));
IkReal x1812=((gconst26)*(gconst30));
dummyeval[0]=((((IkReal(-1.00000000000000))*(gconst22)*(gconst24)*(gconst26)*(gconst29)))+(((IkReal(-1.00000000000000))*(x1809)*(x1810)))+(((IkReal(-1.00000000000000))*(x1811)*(x1812)))+(((x1809)*(x1811)))+(((x1810)*(x1812))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1813=((IkReal(1.00000000000000))*(gconst19));
IkReal x1814=((gconst23)*(gconst27));
IkReal x1815=((gconst15)*(gconst18));
IkReal x1816=((gconst11)*(gconst6));
IkReal x1817=((gconst13)*(gconst26));
IkReal x1818=((gconst15)*(gconst9));
IkReal x1819=((gconst6)*(gconst8));
IkReal x1820=((gconst18)*(gconst28));
IkReal x1821=((gconst22)*(gconst30));
IkReal x1822=((gconst16)*(gconst26));
IkReal x1823=((IkReal(1.00000000000000))*(gconst10));
IkReal x1824=((gconst14)*(gconst25));
IkReal x1825=((IkReal(1.00000000000000))*(gconst26));
IkReal x1826=((gconst12)*(gconst27));
IkReal x1827=((gconst25)*(gconst5));
IkReal x1828=((gconst17)*(gconst28));
IkReal x1829=((gconst21)*(gconst27));
IkReal x1830=((gconst27)*(gconst6));
IkReal x1831=((IkReal(1.00000000000000))*(gconst28));
IkReal x1832=((gconst11)*(gconst5));
IkReal x1833=((gconst17)*(gconst21));
IkReal x1834=((gconst18)*(gconst19));
IkReal x1835=((gconst10)*(gconst7));
IkReal x1836=((gconst30)*(gconst6));
IkReal x1837=((gconst17)*(gconst5));
IkReal x1838=((IkReal(1.00000000000000))*(gconst12));
IkReal x1839=((gconst22)*(gconst7));
IkReal x1840=((gconst24)*(gconst6));
IkReal x1841=((gconst5)*(gconst9));
IkReal x1842=((gconst15)*(gconst17));
IkReal x1843=((gconst27)*(gconst28));
IkReal x1844=((gconst23)*(gconst26));
IkReal x1845=((gconst21)*(gconst9));
IkReal x1846=((gconst22)*(gconst8));
IkReal x1847=((gconst10)*(gconst23));
IkReal x1848=((gconst12)*(gconst18));
IkReal x1849=((gconst21)*(gconst25));
IkReal x1850=((gconst11)*(gconst28));
IkReal x1851=((gconst11)*(gconst19));
IkReal x1852=((gconst19)*(gconst27));
IkReal x1853=((gconst20)*(gconst6));
IkReal x1854=((gconst10)*(gconst16));
IkReal x1855=((gconst26)*(gconst29));
IkReal x1856=((gconst26)*(gconst7));
IkReal x1857=((gconst20)*(gconst22));
IkReal x1858=((gconst22)*(gconst24));
IkReal x1859=((gconst15)*(gconst25));
IkReal x1860=((gconst10)*(gconst13));
IkReal x1861=((gconst14)*(gconst6));
IkReal x1862=((gconst16)*(gconst22));
IkReal x1863=((gconst11)*(gconst12));
IkReal x1864=((gconst14)*(gconst22));
IkReal x1865=((gconst17)*(gconst19));
IkReal x1866=((gconst10)*(gconst29));
IkReal x1867=((gconst28)*(gconst9));
IkReal x1868=((IkReal(1.00000000000000))*(gconst6));
IkReal x1869=((gconst26)*(x1853));
IkReal x1870=((gconst9)*(x1851));
IkReal x1871=((gconst11)*(x1862));
IkReal x1872=((gconst14)*(x1847));
IkReal x1873=((gconst11)*(gconst22)*(gconst23));
op[0]=((((gconst25)*(gconst28)*(x1829)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1825)*(x1858)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1825)*(x1849)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1814)*(x1831)))+(((x1821)*(x1844))));
op[1]=((((IkReal(-1.00000000000000))*(gconst27)*(x1831)*(x1862)))+(((x1844)*(x1857)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1825)*(x1849)))+(((x1855)*(x1859)))+(((gconst19)*(gconst25)*(x1829)))+(((x1828)*(x1829)))+(((x1820)*(x1858)))+(((x1821)*(x1822)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1813)*(x1814)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1825)*(x1833))));
op[2]=((((IkReal(-1.00000000000000))*(gconst20)*(x1825)*(x1833)))+(((x1842)*(x1855)))+(((x1844)*(x1864)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1823)*(x1849)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1823)*(x1858)))+(((x1836)*(x1844)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1825)*(x1840)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1825)*(x1846)))+(((x1834)*(x1858)))+(((x1827)*(x1843)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1814)*(x1831)))+(((x1829)*(x1867)))+(((x1829)*(x1865)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1813)*(x1862)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1815)*(x1831)))+(((IkReal(-1.00000000000000))*(x1831)*(x1873)))+(((x1849)*(x1850)))+(((IkReal(-1.00000000000000))*(gconst21)*(x1824)*(x1825)))+(((x1826)*(x1849)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1831)*(x1839)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1825)*(x1827)))+(((IkReal(-1.00000000000000))*(x1817)*(x1858)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1825)*(x1845)))+(((x1822)*(x1857)))+(((x1821)*(x1856)))+(((IkReal(-1.00000000000000))*(gconst22)*(x1814)*(x1838)))+(((x1821)*(x1847))));
op[3]=((((IkReal(-1.00000000000000))*(x1813)*(x1873)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1830)*(x1831)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1825)*(x1827)))+(((x1822)*(x1864)))+(((x1826)*(x1833)))+(((x1848)*(x1858)))+(((IkReal(-1.00000000000000))*(x1831)*(x1871)))+(((x1818)*(x1855)))+(((x1849)*(x1851)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1813)*(x1815)))+(((x1847)*(x1857)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1825)*(x1833)))+(((x1827)*(x1852)))+(((x1822)*(x1836)))+(((x1859)*(x1866)))+(((IkReal(-1.00000000000000))*(x1826)*(x1862)))+(((x1844)*(x1853)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1825)*(x1845)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1823)*(x1833)))+(((gconst11)*(gconst21)*(x1828)))+(((gconst19)*(gconst9)*(x1829)))+(((gconst20)*(gconst26)*(x1839)))+(((IkReal(-1.00000000000000))*(gconst27)*(x1813)*(x1839)))+(((x1821)*(x1854)))+(((x1817)*(x1859)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1813)*(x1814)))+(((gconst27)*(gconst5)*(x1828)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1823)*(x1849)))+(((x1820)*(x1846)))+(((x1820)*(x1840)))+(((IkReal(-1.00000000000000))*(x1815)*(x1828)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1825)*(x1837))));
op[4]=((((IkReal(-1.00000000000000))*(gconst20)*(x1825)*(x1837)))+(((x1833)*(x1851)))+(((IkReal(-1.00000000000000))*(x1813)*(x1871)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1819)*(x1825)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1823)*(x1827)))+(((x1844)*(x1861)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1823)*(x1845)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1813)*(x1830)))+(((x1836)*(x1847)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1816)*(x1831)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1823)*(x1858)))+(((x1834)*(x1840)))+(((x1834)*(x1846)))+(((IkReal(-1.00000000000000))*(gconst6)*(x1814)*(x1838)))+(((IkReal(-1.00000000000000))*(gconst17)*(x1813)*(x1815)))+(((IkReal(-1.00000000000000))*(gconst21)*(x1823)*(x1824)))+(((x1826)*(x1827)))+(((x1837)*(x1852)))+(((IkReal(-1.00000000000000))*(gconst25)*(x1815)*(x1838)))+(((IkReal(-1.00000000000000))*(gconst5)*(x1824)*(x1825)))+(((x1836)*(x1856)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1823)*(x1833)))+(((x1826)*(x1845)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1830)*(x1831)))+(((x1821)*(x1835)))+(((x1827)*(x1850)))+(((IkReal(-1.00000000000000))*(x1826)*(x1839)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1823)*(x1840)))+(((IkReal(-1.00000000000000))*(gconst29)*(x1823)*(x1846)))+(((x1847)*(x1864)))+(((x1854)*(x1857)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1825)*(x1841)))+(((x1845)*(x1850)))+(((x1822)*(x1853)))+(((x1842)*(x1866)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1831)*(x1839)))+(((x1817)*(x1842)))+(((IkReal(-1.00000000000000))*(x1817)*(x1846)))+(((IkReal(-1.00000000000000))*(x1817)*(x1840)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1825)*(x1845)))+(((x1841)*(x1843)))+(((IkReal(-1.00000000000000))*(x1838)*(x1873)))+(((x1849)*(x1863)))+(((gconst14)*(gconst26)*(x1839)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1815)*(x1831))));
op[5]=((((IkReal(-1.00000000000000))*(gconst14)*(x1823)*(x1833)))+(((x1822)*(x1861)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1813)*(x1815)))+(((x1826)*(x1837)))+(((x1819)*(x1820)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1813)*(x1830)))+(((x1836)*(x1854)))+(((x1847)*(x1853)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1825)*(x1837)))+(((x1840)*(x1848)))+(((x1827)*(x1851)))+(((x1859)*(x1860)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1825)*(x1841)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1823)*(x1837)))+(((x1817)*(x1818)))+(((x1835)*(x1857)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1813)*(x1816)))+(((x1845)*(x1851)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1826)*(x1868)))+(((x1846)*(x1848)))+(((x1853)*(x1856)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1813)*(x1839)))+(((x1833)*(x1863)))+(((x1854)*(x1864)))+(((x1818)*(x1866)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1816)*(x1831)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1823)*(x1827)))+(((x1828)*(x1832)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1823)*(x1845)))+(((IkReal(-1.00000000000000))*(x1838)*(x1871)))+(((x1841)*(x1852)))+(((IkReal(-1.00000000000000))*(gconst17)*(x1815)*(x1838))));
op[6]=((((IkReal(-1.00000000000000))*(gconst29)*(x1819)*(x1823)))+(((IkReal(-1.00000000000000))*(gconst30)*(x1823)*(x1841)))+(((x1835)*(x1864)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1816)*(x1831)))+(((IkReal(-1.00000000000000))*(gconst23)*(x1816)*(x1838)))+(((IkReal(-1.00000000000000))*(gconst11)*(x1838)*(x1839)))+(((x1827)*(x1863)))+(((x1835)*(x1836)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1823)*(x1840)))+(((IkReal(-1.00000000000000))*(gconst13)*(x1823)*(x1846)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1823)*(x1837)))+(((x1826)*(x1841)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1813)*(x1816)))+(((x1847)*(x1861)))+(((x1856)*(x1861)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1826)*(x1868)))+(((x1853)*(x1854)))+(((x1842)*(x1860)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1823)*(x1845)))+(((x1819)*(x1834)))+(((x1832)*(x1867)))+(((x1832)*(x1865)))+(((IkReal(-1.00000000000000))*(x1817)*(x1819)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1825)*(x1841)))+(((IkReal(-1.00000000000000))*(gconst5)*(x1823)*(x1824)))+(((IkReal(-1.00000000000000))*(gconst9)*(x1815)*(x1838)))+(((x1845)*(x1863))));
op[7]=((((IkReal(-1.00000000000000))*(gconst14)*(x1823)*(x1837)))+(((gconst19)*(gconst9)*(x1832)))+(((x1819)*(x1848)))+(((x1835)*(x1853)))+(((gconst12)*(gconst17)*(x1832)))+(((x1854)*(x1861)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1813)*(x1816)))+(((x1818)*(x1860)))+(((IkReal(-1.00000000000000))*(gconst16)*(x1816)*(x1838)))+(((IkReal(-1.00000000000000))*(gconst20)*(x1823)*(x1841))));
op[8]=((((IkReal(-1.00000000000000))*(gconst13)*(x1819)*(x1823)))+(((x1835)*(x1861)))+(((IkReal(-1.00000000000000))*(gconst7)*(x1816)*(x1838)))+(((gconst12)*(gconst9)*(x1832)))+(((IkReal(-1.00000000000000))*(gconst14)*(x1823)*(x1841))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst47;
gconst47=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1874=((r01)*(sj3));
IkReal x1875=((IkReal(1.00000000000000))*(sj0));
IkReal x1876=((sj1)*(sj3));
IkReal x1877=((cj2)*(r00));
IkReal x1878=((r02)*(sj2));
IkReal x1879=((cj3)*(sj1));
IkReal x1880=((IkReal(1.00000000000000))*(cj0));
IkReal x1881=((cj3)*(r01));
IkReal x1882=((cj1)*(r00)*(sj2));
IkReal x1883=((cj1)*(cj2)*(r02)*(sj3));
IkReal x1884=((cj1)*(cj2)*(cj3)*(r02));
if( IKabs(((gconst47)*(((((sj0)*(x1876)*(x1877)))+(((IkReal(-1.00000000000000))*(x1875)*(x1876)*(x1878)))+(((IkReal(-1.00000000000000))*(x1875)*(x1881)))+(((cj0)*(x1877)*(x1879)))+(((IkReal(-1.00000000000000))*(x1878)*(x1879)*(x1880)))+(((cj0)*(cj3)*(x1882)))+(((sj0)*(sj3)*(x1882)))+(((cj0)*(x1884)))+(((sj0)*(x1883)))+(((cj0)*(x1874))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst47)*(((((IkReal(-1.00000000000000))*(x1880)*(x1881)))+(((IkReal(-1.00000000000000))*(x1876)*(x1878)*(x1880)))+(((IkReal(-1.00000000000000))*(x1874)*(x1875)))+(((IkReal(-1.00000000000000))*(cj3)*(x1875)*(x1882)))+(((IkReal(-1.00000000000000))*(x1875)*(x1884)))+(((cj0)*(sj3)*(x1882)))+(((sj0)*(x1878)*(x1879)))+(((cj0)*(x1876)*(x1877)))+(((IkReal(-1.00000000000000))*(x1875)*(x1877)*(x1879)))+(((cj0)*(x1883))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst47)*(((((sj0)*(x1876)*(x1877)))+(((IkReal(-1.00000000000000))*(x1875)*(x1876)*(x1878)))+(((IkReal(-1.00000000000000))*(x1875)*(x1881)))+(((cj0)*(x1877)*(x1879)))+(((IkReal(-1.00000000000000))*(x1878)*(x1879)*(x1880)))+(((cj0)*(cj3)*(x1882)))+(((sj0)*(sj3)*(x1882)))+(((cj0)*(x1884)))+(((sj0)*(x1883)))+(((cj0)*(x1874)))))), ((gconst47)*(((((IkReal(-1.00000000000000))*(x1880)*(x1881)))+(((IkReal(-1.00000000000000))*(x1876)*(x1878)*(x1880)))+(((IkReal(-1.00000000000000))*(x1874)*(x1875)))+(((IkReal(-1.00000000000000))*(cj3)*(x1875)*(x1882)))+(((IkReal(-1.00000000000000))*(x1875)*(x1884)))+(((cj0)*(sj3)*(x1882)))+(((sj0)*(x1878)*(x1879)))+(((cj0)*(x1876)*(x1877)))+(((IkReal(-1.00000000000000))*(x1875)*(x1877)*(x1879)))+(((cj0)*(x1883)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1885=IKsin(j5);
IkReal x1886=IKcos(j5);
IkReal x1887=((IkReal(1.00000000000000))*(cj0));
IkReal x1888=((IkReal(1.00000000000000))*(cj3));
IkReal x1889=((cj0)*(cj1));
IkReal x1890=((cj2)*(r02));
IkReal x1891=((r00)*(sj2));
IkReal x1892=((IkReal(1.00000000000000))*(sj0));
IkReal x1893=((r02)*(sj1)*(sj2));
IkReal x1894=((cj2)*(r00)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj3)*(x1886)))+(((IkReal(-1.00000000000000))*(r01)*(x1892)))+(((IkReal(-1.00000000000000))*(x1887)*(x1893)))+(((x1889)*(x1891)))+(((x1889)*(x1890)))+(((cj0)*(x1894)))+(((IkReal(-1.00000000000000))*(x1885)*(x1888))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x1892)*(x1894)))+(((IkReal(-1.00000000000000))*(cj1)*(x1891)*(x1892)))+(((sj0)*(x1893)))+(((IkReal(-1.00000000000000))*(r01)*(x1887)))+(((IkReal(-1.00000000000000))*(x1886)*(x1888)))+(((sj3)*(x1885)))+(((IkReal(-1.00000000000000))*(cj1)*(x1890)*(x1892))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x1895=((IkReal(1.00000000000000))*(r02));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1895)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1895)))+(((cj1)*(cj2)*(r00))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst48;
IkReal x1896=((sj0)*(sj1));
IkReal x1897=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst48=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1897)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1896)))+(((r02)*(sj2)*(x1896)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1897))));
IkReal gconst49;
IkReal x1898=((sj0)*(sj1));
IkReal x1899=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst49=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1899)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1898)))+(((r02)*(sj2)*(x1898)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1899))));
IkReal gconst50;
IkReal x1900=((cj0)*(r02));
IkReal x1901=((cj0)*(r00));
gconst50=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1900)))+(((cj2)*(sj1)*(x1901)))+(((cj1)*(cj2)*(x1900)))+(((cj1)*(sj2)*(x1901))));
IkReal gconst51;
gconst51=IkReal(2.00000000000000);
IkReal gconst52;
IkReal x1902=((cj0)*(r02));
IkReal x1903=((cj0)*(r00));
gconst52=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1902)))+(((cj2)*(sj1)*(x1903)))+(((cj1)*(cj2)*(x1902)))+(((cj1)*(sj2)*(x1903))));
IkReal gconst53;
IkReal x1904=((sj0)*(sj1));
IkReal x1905=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst53=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x1904)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1905)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1904)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1905))));
IkReal gconst54;
IkReal x1906=((sj0)*(sj1));
IkReal x1907=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst54=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x1906)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1907)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1906)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1907))));
IkReal gconst55;
IkReal x1908=((cj0)*(r02));
IkReal x1909=((cj0)*(r00));
gconst55=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1908)))+(((cj2)*(sj1)*(x1909)))+(((cj1)*(cj2)*(x1908)))+(((cj1)*(sj2)*(x1909))));
IkReal gconst56;
gconst56=IkReal(2.00000000000000);
IkReal gconst57;
IkReal x1910=((cj0)*(r02));
IkReal x1911=((cj0)*(r00));
gconst57=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1911)))+(((cj2)*(sj1)*(x1911)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1910)))+(((cj1)*(cj2)*(x1910))));
IkReal gconst58;
gconst58=IkReal(-4.00000000000000);
IkReal gconst59;
gconst59=IkReal(-2.00000000000000);
IkReal gconst60;
gconst60=IkReal(2.00000000000000);
IkReal gconst61;
gconst61=IkReal(-4.00000000000000);
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=IkReal(2.00000000000000);
IkReal gconst64;
IkReal x1912=((sj0)*(sj1));
IkReal x1913=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst64=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1913)))+(((r02)*(sj2)*(x1912)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1913)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1912))));
IkReal gconst65;
IkReal x1914=((sj0)*(sj1));
IkReal x1915=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst65=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1915)))+(((r02)*(sj2)*(x1914)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1915)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1914))));
IkReal gconst66;
IkReal x1916=((cj0)*(r02));
IkReal x1917=((cj0)*(r00));
gconst66=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1917)))+(((cj2)*(sj1)*(x1917)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1916)))+(((cj1)*(cj2)*(x1916))));
IkReal gconst67;
gconst67=IkReal(-2.00000000000000);
IkReal gconst68;
IkReal x1918=((cj0)*(r02));
IkReal x1919=((cj0)*(r00));
gconst68=((((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1919)))+(((cj2)*(sj1)*(x1919)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1918)))+(((cj1)*(cj2)*(x1918))));
IkReal gconst69;
IkReal x1920=((sj0)*(sj1));
IkReal x1921=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst69=((IkReal(1.00000000000000))+(((r02)*(sj2)*(x1920)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1920)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1921)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1921)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst70;
IkReal x1922=((sj0)*(sj1));
IkReal x1923=((IkReal(1.00000000000000))*(cj1)*(sj0));
gconst70=((IkReal(-1.00000000000000))+(((r02)*(sj2)*(x1922)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x1922)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x1923)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1923)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst71;
IkReal x1924=((cj0)*(r02));
IkReal x1925=((cj0)*(r00));
gconst71=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1924)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1925)))+(((cj2)*(sj1)*(x1925)))+(((cj1)*(cj2)*(x1924))));
IkReal gconst72;
gconst72=IkReal(-2.00000000000000);
IkReal gconst73;
IkReal x1926=((cj0)*(r02));
IkReal x1927=((cj0)*(r00));
gconst73=((((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1926)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)))+(((cj1)*(sj2)*(x1927)))+(((cj2)*(sj1)*(x1927)))+(((cj1)*(cj2)*(x1926))));
IkReal x1928=((gconst70)*(gconst71));
IkReal x1929=((IkReal(1.00000000000000))*(gconst65));
IkReal x1930=((gconst64)*(gconst68));
IkReal x1931=((gconst69)*(gconst73));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x1930)*(x1931)))+(((gconst65)*(gconst66)*(x1931)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst69)*(gconst72)*(x1929)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1928)*(x1929)))+(((x1928)*(x1930))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x1932=((gconst62)*(gconst70));
IkReal x1933=((IkReal(1.00000000000000))*(gconst49));
IkReal x1934=((gconst53)*(gconst73));
IkReal x1935=((gconst48)*(gconst52));
IkReal x1936=((gconst65)*(gconst66));
IkReal x1937=((IkReal(1.00000000000000))*(gconst65));
IkReal x1938=((gconst53)*(gconst56));
IkReal x1939=((gconst49)*(gconst50));
IkReal x1940=((gconst55)*(gconst70));
IkReal x1941=((gconst54)*(gconst62));
IkReal x1942=((gconst59)*(gconst69));
IkReal x1943=((gconst49)*(gconst57));
IkReal x1944=((gconst52)*(gconst63));
IkReal x1945=((gconst60)*(gconst73));
IkReal x1946=((gconst52)*(gconst58));
IkReal x1947=((gconst56)*(gconst69));
IkReal x1948=((gconst54)*(gconst55));
IkReal x1949=((gconst63)*(gconst65));
IkReal x1950=((gconst63)*(gconst69));
IkReal x1951=((gconst49)*(gconst66));
IkReal x1952=((IkReal(1.00000000000000))*(gconst48));
IkReal x1953=((gconst52)*(gconst64));
IkReal x1954=((gconst70)*(gconst71));
IkReal x1955=((gconst68)*(gconst69));
IkReal x1956=((IkReal(1.00000000000000))*(gconst68));
IkReal x1957=((gconst54)*(gconst71));
IkReal x1958=((gconst48)*(gconst60));
IkReal x1959=((gconst53)*(gconst72));
IkReal x1960=((gconst53)*(gconst59));
IkReal x1961=((gconst58)*(gconst61));
IkReal x1962=((gconst57)*(gconst65));
IkReal x1963=((gconst53)*(gconst63));
IkReal x1964=((gconst61)*(gconst71));
IkReal x1965=((gconst65)*(gconst67));
IkReal x1966=((gconst48)*(gconst68));
IkReal x1967=((gconst53)*(gconst57));
IkReal x1968=((gconst60)*(gconst64));
IkReal x1969=((gconst69)*(gconst72));
IkReal x1970=((gconst58)*(gconst68));
IkReal x1971=((gconst50)*(gconst69));
IkReal x1972=((IkReal(1.00000000000000))*(gconst60));
IkReal x1973=((gconst69)*(gconst73));
IkReal x1974=((gconst57)*(gconst69));
IkReal x1975=((gconst49)*(gconst63));
IkReal x1976=((gconst50)*(gconst53));
IkReal x1977=((gconst55)*(gconst61));
IkReal x1978=((gconst65)*(gconst73));
IkReal x1979=((gconst61)*(gconst62));
IkReal x1980=((gconst64)*(gconst68));
IkReal x1981=((gconst58)*(gconst60));
IkReal x1982=((gconst49)*(gconst67));
IkReal x1983=((gconst60)*(x1948));
IkReal x1984=((IkReal(1.00000000000000))*(gconst64)*(gconst69));
IkReal x1985=((gconst72)*(x1981));
IkReal x1986=((gconst49)*(gconst51)*(gconst61));
IkReal x1987=((gconst51)*(gconst61)*(gconst65));
op[0]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst73)*(x1955)))+(((IkReal(-1.00000000000000))*(x1936)*(x1954)))+(((x1936)*(x1973)))+(((x1954)*(x1980)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1937)*(x1969))));
op[1]=((((x1964)*(x1965)))+(((gconst58)*(gconst72)*(x1955)))+(((x1954)*(x1968)))+(((IkReal(-1.00000000000000))*(x1945)*(x1984)))+(((x1936)*(x1950)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1950)*(x1956)))+(((IkReal(-1.00000000000000))*(x1932)*(x1936)))+(((x1942)*(x1978)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1937)*(x1954)))+(((x1932)*(x1980))));
op[2]=((((IkReal(-1.00000000000000))*(gconst59)*(x1932)*(x1937)))+(((x1969)*(x1981)))+(((IkReal(-1.00000000000000))*(gconst73)*(x1952)*(x1955)))+(((x1934)*(x1936)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1934)*(x1956)))+(((x1951)*(x1973)))+(((x1971)*(x1978)))+(((x1954)*(x1966)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1937)*(x1959)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1933)*(x1954)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1933)*(x1969)))+(((IkReal(-1.00000000000000))*(x1950)*(x1968)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1937)*(x1954)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1937)*(x1947)))+(((x1957)*(x1980)))+(((IkReal(-1.00000000000000))*(x1936)*(x1957)))+(((x1936)*(x1974)))+(((x1940)*(x1980)))+(((x1965)*(x1979)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1937)*(x1969)))+(((x1953)*(x1954)))+(((IkReal(-1.00000000000000))*(x1936)*(x1940)))+(((x1942)*(x1949)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1956)*(x1961)))+(((x1932)*(x1968)))+(((IkReal(-1.00000000000000))*(x1953)*(x1973)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst64)*(x1955))));
op[3]=((((IkReal(-1.00000000000000))*(gconst62)*(x1956)*(x1961)))+(((x1959)*(x1970)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1932)*(x1937)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1956)*(x1963)))+(((x1940)*(x1968)))+(((x1950)*(x1951)))+(((IkReal(-1.00000000000000))*(gconst69)*(x1945)*(x1952)))+(((gconst49)*(gconst73)*(x1942)))+(((x1964)*(x1982)))+(((x1946)*(x1969)))+(((x1954)*(x1958)))+(((x1957)*(x1968)))+(((IkReal(-1.00000000000000))*(x1934)*(x1968)))+(((x1941)*(x1980)))+(((IkReal(-1.00000000000000))*(gconst71)*(x1961)*(x1972)))+(((IkReal(-1.00000000000000))*(x1944)*(x1984)))+(((x1936)*(x1963)))+(((x1965)*(x1977)))+(((gconst59)*(gconst65)*(x1934)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1950)*(x1952)))+(((IkReal(-1.00000000000000))*(x1936)*(x1941)))+(((x1949)*(x1971)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1933)*(x1954)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1937)*(x1940)))+(((IkReal(-1.00000000000000))*(x1968)*(x1974)))+(((x1932)*(x1953)))+(((x1942)*(x1962)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1937)*(x1957)))+(((gconst51)*(gconst65)*(x1964)))+(((x1932)*(x1966)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1932)*(x1933)))+(((x1947)*(x1970))));
op[4]=((((IkReal(-1.00000000000000))*(gconst59)*(x1932)*(x1933)))+(((IkReal(-1.00000000000000))*(x1963)*(x1968)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1933)*(x1954)))+(((IkReal(-1.00000000000000))*(x1946)*(x1964)))+(((IkReal(-1.00000000000000))*(gconst62)*(x1961)*(x1972)))+(((x1939)*(x1973)))+(((gconst50)*(gconst65)*(x1934)))+(((IkReal(-1.00000000000000))*(gconst64)*(x1956)*(x1967)))+(((x1940)*(x1966)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1956)*(x1961)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1934)*(x1952)))+(((IkReal(-1.00000000000000))*(gconst57)*(x1952)*(x1955)))+(((x1941)*(x1968)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1937)*(x1940)))+(((x1935)*(x1954)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1933)*(x1957)))+(((x1957)*(x1966)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1937)*(x1957)))+(((x1940)*(x1953)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1933)*(x1940)))+(((x1979)*(x1982)))+(((IkReal(-1.00000000000000))*(x1934)*(x1953)))+(((IkReal(-1.00000000000000))*(x1935)*(x1973)))+(((x1936)*(x1967)))+(((x1948)*(x1980)))+(((gconst66)*(gconst69)*(x1943)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1950)*(x1952)))+(((x1962)*(x1971)))+(((x1953)*(x1957)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1933)*(x1969)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1933)*(x1947)))+(((IkReal(-1.00000000000000))*(x1936)*(x1948)))+(((x1949)*(x1960)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1933)*(x1959)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1937)*(x1959)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1937)*(x1941)))+(((x1942)*(x1975)))+(((x1932)*(x1958)))+(((x1947)*(x1981)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1937)*(x1947)))+(((x1959)*(x1981)))+(((x1934)*(x1951)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1937)*(x1938)))+(((gconst51)*(gconst65)*(x1979)))+(((IkReal(-1.00000000000000))*(x1953)*(x1974))));
op[5]=((((IkReal(-1.00000000000000))*(x1946)*(x1979)))+(((gconst49)*(gconst59)*(x1934)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1932)*(x1933)))+(((x1939)*(x1950)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1934)*(x1952)))+(((x1951)*(x1963)))+(((x1932)*(x1935)))+(((x1938)*(x1970)))+(((x1946)*(x1947)))+(((x1941)*(x1966)))+(((IkReal(-1.00000000000000))*(gconst55)*(x1961)*(x1972)))+(((x1946)*(x1959)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1937)*(x1941)))+(((x1957)*(x1958)))+(((x1941)*(x1953)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1952)*(x1974)))+(((x1940)*(x1958)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1933)*(x1941)))+(((IkReal(-1.00000000000000))*(x1935)*(x1950)))+(((x1960)*(x1962)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1933)*(x1940)))+(((x1942)*(x1943)))+(((IkReal(-1.00000000000000))*(x1967)*(x1968)))+(((x1949)*(x1976)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1933)*(x1957)))+(((gconst49)*(gconst51)*(x1964)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1937)*(x1948)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1952)*(x1963)))+(((x1948)*(x1968)))+(((gconst51)*(gconst65)*(x1977)))+(((x1977)*(x1982)))+(((IkReal(-1.00000000000000))*(gconst53)*(gconst64)*(x1944))));
op[6]=((((IkReal(-1.00000000000000))*(x1946)*(x1977)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1933)*(x1957)))+(((x1938)*(x1981)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1933)*(x1940)))+(((x1934)*(x1939)))+(((x1939)*(x1974)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1937)*(x1938)))+(((IkReal(-1.00000000000000))*(x1934)*(x1935)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1937)*(x1948)))+(((x1935)*(x1957)))+(((x1941)*(x1958)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1952)*(x1963)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1933)*(x1959)))+(((x1935)*(x1940)))+(((IkReal(-1.00000000000000))*(gconst51)*(x1933)*(x1947)))+(((IkReal(-1.00000000000000))*(gconst66)*(x1933)*(x1948)))+(((IkReal(-1.00000000000000))*(gconst67)*(x1933)*(x1938)))+(((IkReal(-1.00000000000000))*(x1935)*(x1974)))+(((x1960)*(x1975)))+(((x1948)*(x1953)))+(((x1962)*(x1976)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1933)*(x1941)))+(((gconst53)*(gconst66)*(x1943)))+(((gconst49)*(gconst51)*(x1979)))+(((IkReal(-1.00000000000000))*(gconst68)*(x1952)*(x1967)))+(((x1948)*(x1966)))+(((IkReal(-1.00000000000000))*(x1953)*(x1967))));
op[7]=((((x1938)*(x1946)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1933)*(x1941)))+(((x1939)*(x1963)))+(((x1943)*(x1960)))+(((IkReal(-1.00000000000000))*(gconst60)*(x1952)*(x1967)))+(((x1935)*(x1941)))+(((IkReal(-1.00000000000000))*(x1935)*(x1963)))+(((x1948)*(x1958)))+(((IkReal(-1.00000000000000))*(gconst59)*(x1933)*(x1948)))+(((gconst49)*(gconst51)*(x1977))));
op[8]=((((IkReal(-1.00000000000000))*(gconst51)*(x1933)*(x1938)))+(((IkReal(-1.00000000000000))*(gconst50)*(x1933)*(x1948)))+(((x1939)*(x1967)))+(((x1935)*(x1948)))+(((IkReal(-1.00000000000000))*(x1935)*(x1967))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst90;
gconst90=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1988=((sj1)*(sj3));
IkReal x1989=((IkReal(1.00000000000000))*(cj0));
IkReal x1990=((cj2)*(r00));
IkReal x1991=((cj3)*(sj0));
IkReal x1992=((cj0)*(sj3));
IkReal x1993=((cj3)*(sj1));
IkReal x1994=((r02)*(sj2));
IkReal x1995=((IkReal(1.00000000000000))*(sj0)*(sj3));
IkReal x1996=((cj1)*(r00)*(sj2));
IkReal x1997=((sj0)*(x1994));
IkReal x1998=((cj1)*(cj2)*(r02));
if( IKabs(((gconst90)*(((((x1988)*(x1997)))+(((IkReal(-1.00000000000000))*(x1989)*(x1990)*(x1993)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1989)))+(((IkReal(-1.00000000000000))*(x1995)*(x1996)))+(((IkReal(-1.00000000000000))*(x1995)*(x1998)))+(((r01)*(x1991)))+(((cj0)*(x1993)*(x1994)))+(((IkReal(-1.00000000000000))*(sj0)*(x1988)*(x1990)))+(((IkReal(-1.00000000000000))*(cj3)*(x1989)*(x1996)))+(((IkReal(-1.00000000000000))*(cj3)*(x1989)*(x1998))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst90)*(((((sj1)*(x1991)*(x1994)))+(((cj0)*(x1988)*(x1990)))+(((IkReal(-1.00000000000000))*(r01)*(x1995)))+(((IkReal(-1.00000000000000))*(sj1)*(x1990)*(x1991)))+(((IkReal(-1.00000000000000))*(x1988)*(x1989)*(x1994)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1989)))+(((IkReal(-1.00000000000000))*(x1991)*(x1998)))+(((IkReal(-1.00000000000000))*(x1991)*(x1996)))+(((x1992)*(x1996)))+(((x1992)*(x1998))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst90)*(((((x1988)*(x1997)))+(((IkReal(-1.00000000000000))*(x1989)*(x1990)*(x1993)))+(((IkReal(-1.00000000000000))*(r01)*(sj3)*(x1989)))+(((IkReal(-1.00000000000000))*(x1995)*(x1996)))+(((IkReal(-1.00000000000000))*(x1995)*(x1998)))+(((r01)*(x1991)))+(((cj0)*(x1993)*(x1994)))+(((IkReal(-1.00000000000000))*(sj0)*(x1988)*(x1990)))+(((IkReal(-1.00000000000000))*(cj3)*(x1989)*(x1996)))+(((IkReal(-1.00000000000000))*(cj3)*(x1989)*(x1998)))))), ((gconst90)*(((((sj1)*(x1991)*(x1994)))+(((cj0)*(x1988)*(x1990)))+(((IkReal(-1.00000000000000))*(r01)*(x1995)))+(((IkReal(-1.00000000000000))*(sj1)*(x1990)*(x1991)))+(((IkReal(-1.00000000000000))*(x1988)*(x1989)*(x1994)))+(((IkReal(-1.00000000000000))*(cj3)*(r01)*(x1989)))+(((IkReal(-1.00000000000000))*(x1991)*(x1998)))+(((IkReal(-1.00000000000000))*(x1991)*(x1996)))+(((x1992)*(x1996)))+(((x1992)*(x1998)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[2];
IkReal x1999=IKcos(j5);
IkReal x2000=IKsin(j5);
IkReal x2001=((IkReal(1.00000000000000))*(cj0));
IkReal x2002=((IkReal(1.00000000000000))*(sj0));
IkReal x2003=((r02)*(sj1)*(sj2));
IkReal x2004=((IkReal(1.00000000000000))*(x1999));
IkReal x2005=((cj1)*(cj2)*(r02));
IkReal x2006=((cj2)*(r00)*(sj1));
IkReal x2007=((cj1)*(r00)*(sj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x2002)))+(((IkReal(-1.00000000000000))*(x2001)*(x2003)))+(((IkReal(-1.00000000000000))*(sj3)*(x2004)))+(((cj0)*(x2006)))+(((cj0)*(x2005)))+(((cj0)*(x2007)))+(((cj3)*(x2000))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x2001)))+(((IkReal(-1.00000000000000))*(x2002)*(x2006)))+(((IkReal(-1.00000000000000))*(x2002)*(x2005)))+(((IkReal(-1.00000000000000))*(x2002)*(x2007)))+(((IkReal(-1.00000000000000))*(sj3)*(x2000)))+(((sj0)*(x2003)))+(((IkReal(-1.00000000000000))*(cj3)*(x2004))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x2008=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x2009=((cj1)*(x2008));
IkReal x2010=((IkReal(1.00000000000000))*(r02));
IkReal x2011=((sj1)*(x2008));
sj5array[0]=((((IkReal(-1.00000000000000))*(cj2)*(x2010)*(x2011)))+(((IkReal(-1.00000000000000))*(sj2)*(x2009)*(x2010)))+(((cj2)*(r00)*(x2009)))+(((IkReal(-1.00000000000000))*(r00)*(sj2)*(x2011))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x2012=((cj0)*(cj5));
IkReal x2013=((IkReal(1.00000000000000))*(r01));
IkReal x2014=((cj4)*(sj0)*(sj5));
IkReal x2015=((cj2)*(r00)*(sj1));
IkReal x2016=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x2017=((cj0)*(cj4)*(sj5));
IkReal x2018=((cj1)*(r00)*(sj2));
IkReal x2019=((cj1)*(cj2)*(r02));
IkReal x2020=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x2014)*(x2019)))+(((x2014)*(x2018)))+(((x2014)*(x2015)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x2013)))+(((r01)*(x2017)))+(((x2012)*(x2015)))+(((x2012)*(x2018)))+(((x2012)*(x2019)))+(((IkReal(-1.00000000000000))*(x2014)*(x2020)))+(((IkReal(-1.00000000000000))*(x2012)*(x2020))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((x2015)*(x2017)))+(((IkReal(-1.00000000000000))*(x2013)*(x2014)))+(((IkReal(-1.00000000000000))*(x2015)*(x2016)))+(((IkReal(-1.00000000000000))*(x2017)*(x2020)))+(((IkReal(-1.00000000000000))*(x2016)*(x2019)))+(((IkReal(-1.00000000000000))*(x2016)*(x2018)))+(((IkReal(-1.00000000000000))*(x2012)*(x2013)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x2017)*(x2019)))+(((x2017)*(x2018))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x2014)*(x2019)))+(((x2014)*(x2018)))+(((x2014)*(x2015)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x2013)))+(((r01)*(x2017)))+(((x2012)*(x2015)))+(((x2012)*(x2018)))+(((x2012)*(x2019)))+(((IkReal(-1.00000000000000))*(x2014)*(x2020)))+(((IkReal(-1.00000000000000))*(x2012)*(x2020)))))), ((gconst4)*(((((x2015)*(x2017)))+(((IkReal(-1.00000000000000))*(x2013)*(x2014)))+(((IkReal(-1.00000000000000))*(x2015)*(x2016)))+(((IkReal(-1.00000000000000))*(x2017)*(x2020)))+(((IkReal(-1.00000000000000))*(x2016)*(x2019)))+(((IkReal(-1.00000000000000))*(x2016)*(x2018)))+(((IkReal(-1.00000000000000))*(x2012)*(x2013)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((x2017)*(x2019)))+(((x2017)*(x2018)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x2021=IKsin(j3);
IkReal x2022=IKcos(j3);
IkReal x2023=((IkReal(1.00000000000000))*(cj0));
IkReal x2024=((cj0)*(cj1));
IkReal x2025=((cj2)*(r02));
IkReal x2026=((r00)*(sj2));
IkReal x2027=((IkReal(1.00000000000000))*(sj0));
IkReal x2028=((cj4)*(sj5));
IkReal x2029=((r02)*(sj1)*(sj2));
IkReal x2030=((cj2)*(r00)*(sj1));
IkReal x2031=((IkReal(1.00000000000000))*(x2022));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj5)*(x2021)))+(((IkReal(-1.00000000000000))*(x2023)*(x2029)))+(((x2024)*(x2026)))+(((x2024)*(x2025)))+(((IkReal(-1.00000000000000))*(r01)*(x2027)))+(((cj0)*(x2030)))+(((IkReal(-1.00000000000000))*(x2028)*(x2031))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x2023)))+(((IkReal(-1.00000000000000))*(cj1)*(x2026)*(x2027)))+(((IkReal(-1.00000000000000))*(cj1)*(x2025)*(x2027)))+(((sj0)*(x2029)))+(((IkReal(-1.00000000000000))*(cj5)*(x2031)))+(((x2021)*(x2028)))+(((IkReal(-1.00000000000000))*(x2027)*(x2030))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x2032=((IkReal(1.00000000000000))*(r02));
sj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x2032)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x2032)))+(((cj1)*(cj2)*(r00))))));
if( sj5array[0] >= -1-IKFAST_SINCOS_THRESH && sj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKasin(sj5array[0]);
    cj5array[0] = IKcos(j5array[0]);
    sj5array[1] = sj5array[0];
    j5array[1] = j5array[0] > 0 ? (IKPI-j5array[0]) : (-IKPI-j5array[0]);
    cj5array[1] = -cj5array[0];
}
else if( isnan(sj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];

{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign((((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5))));
dummyeval[0]=(((((cj4)*(cj4))*((sj5)*(sj5))))+((cj5)*(cj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x2033=((cj0)*(cj5));
IkReal x2034=((IkReal(1.00000000000000))*(r01));
IkReal x2035=((cj4)*(sj0)*(sj5));
IkReal x2036=((cj2)*(r00)*(sj1));
IkReal x2037=((IkReal(1.00000000000000))*(cj5)*(sj0));
IkReal x2038=((cj0)*(cj4)*(sj5));
IkReal x2039=((cj1)*(r00)*(sj2));
IkReal x2040=((cj1)*(cj2)*(r02));
IkReal x2041=((IkReal(1.00000000000000))*(r02)*(sj1)*(sj2));
if( IKabs(((gconst4)*(((((x2033)*(x2039)))+(((x2033)*(x2036)))+(((r01)*(x2038)))+(((x2033)*(x2040)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x2034)))+(((IkReal(-1.00000000000000))*(x2035)*(x2041)))+(((x2035)*(x2036)))+(((x2035)*(x2039)))+(((IkReal(-1.00000000000000))*(x2033)*(x2041)))+(((x2035)*(x2040))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x2036)*(x2037)))+(((x2038)*(x2039)))+(((x2036)*(x2038)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x2037)*(x2040)))+(((IkReal(-1.00000000000000))*(x2034)*(x2035)))+(((IkReal(-1.00000000000000))*(x2038)*(x2041)))+(((IkReal(-1.00000000000000))*(x2033)*(x2034)))+(((x2038)*(x2040)))+(((IkReal(-1.00000000000000))*(x2037)*(x2039))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst4)*(((((x2033)*(x2039)))+(((x2033)*(x2036)))+(((r01)*(x2038)))+(((x2033)*(x2040)))+(((IkReal(-1.00000000000000))*(cj5)*(sj0)*(x2034)))+(((IkReal(-1.00000000000000))*(x2035)*(x2041)))+(((x2035)*(x2036)))+(((x2035)*(x2039)))+(((IkReal(-1.00000000000000))*(x2033)*(x2041)))+(((x2035)*(x2040)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x2036)*(x2037)))+(((x2038)*(x2039)))+(((x2036)*(x2038)))+(((cj5)*(r02)*(sj0)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x2037)*(x2040)))+(((IkReal(-1.00000000000000))*(x2034)*(x2035)))+(((IkReal(-1.00000000000000))*(x2038)*(x2041)))+(((IkReal(-1.00000000000000))*(x2033)*(x2034)))+(((x2038)*(x2040)))+(((IkReal(-1.00000000000000))*(x2037)*(x2039)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x2042=IKsin(j3);
IkReal x2043=IKcos(j3);
IkReal x2044=((IkReal(1.00000000000000))*(cj0));
IkReal x2045=((cj0)*(cj1));
IkReal x2046=((cj2)*(r02));
IkReal x2047=((r00)*(sj2));
IkReal x2048=((IkReal(1.00000000000000))*(sj0));
IkReal x2049=((cj4)*(sj5));
IkReal x2050=((r02)*(sj1)*(sj2));
IkReal x2051=((cj2)*(r00)*(sj1));
IkReal x2052=((IkReal(1.00000000000000))*(x2043));
evalcond[0]=((((IkReal(-1.00000000000000))*(x2044)*(x2050)))+(((cj0)*(x2051)))+(((IkReal(-1.00000000000000))*(x2049)*(x2052)))+(((x2045)*(x2047)))+(((x2045)*(x2046)))+(((IkReal(-1.00000000000000))*(cj5)*(x2042)))+(((IkReal(-1.00000000000000))*(r01)*(x2048))));
evalcond[1]=((((x2042)*(x2049)))+(((IkReal(-1.00000000000000))*(cj1)*(x2047)*(x2048)))+(((sj0)*(x2050)))+(((IkReal(-1.00000000000000))*(x2048)*(x2051)))+(((IkReal(-1.00000000000000))*(cj5)*(x2052)))+(((IkReal(-1.00000000000000))*(r01)*(x2044)))+(((IkReal(-1.00000000000000))*(cj1)*(x2046)*(x2048))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
}

}

}
}
}
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "3599d930e8a3e7e7ef95c62b7fb7bb55"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
