/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:12:55.747118
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44;
x0=IKcos(j[0]);
x1=IKcos(j[2]);
x2=IKsin(j[1]);
x3=IKsin(j[0]);
x4=IKsin(j[3]);
x5=IKcos(j[3]);
x6=IKcos(j[1]);
x7=IKsin(j[2]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[5]);
x11=IKcos(j[5]);
x12=((IkReal(0.790000000000000))*(x9));
x13=((IkReal(0.790000000000000))*(x5));
x14=((IkReal(0.365000000000000))*(x2));
x15=((IkReal(0.0800000000000000))*(x1));
x16=((IkReal(1.00000000000000))*(x3));
x17=((IkReal(1.00000000000000))*(x0));
x18=((IkReal(0.790000000000000))*(x8));
x19=((IkReal(-0.0800000000000000))*(x4));
x20=((x10)*(x4));
x21=((x1)*(x2));
x22=((x3)*(x6));
x23=((IkReal(-1.00000000000000))*(x8));
x24=((x2)*(x7));
x25=((x6)*(x7));
x26=((x1)*(x6));
x27=((IkReal(-1.00000000000000))*(x9));
x28=((x3)*(x5));
x29=((IkReal(0.0900000000000000))*(x21));
x30=((x0)*(x26));
x31=((((IkReal(-1.00000000000000))*(x26)))+(x24));
x32=((x25)+(x21));
x33=((((IkReal(-1.00000000000000))*(x17)*(x24)))+(x30));
x34=((((x1)*(x22)))+(((IkReal(-1.00000000000000))*(x16)*(x24))));
x35=((x17)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x25))))));
x36=((x16)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x25))))));
x37=((x33)*(x9));
x38=((x36)*(x5));
x39=((x35)*(x4));
x40=((x35)*(x5));
x41=((x36)*(x4));
x42=((((IkReal(-1.00000000000000))*(x40)))+(((x16)*(x4))));
x43=((((IkReal(-1.00000000000000))*(x38)))+(((IkReal(-1.00000000000000))*(x0)*(x4))));
x44=((x43)*(x8));
IkReal x45=((x0)*(x24));
eetrans[0]=((((IkReal(0.0900000000000000))*(x0)*(x25)))+(((x0)*(x29)))+(((x0)*(x14)))+(((IkReal(-0.405000000000000))*(x45)))+(((x10)*(((((IkReal(-0.790000000000000))*(x39)))+(((IkReal(-0.790000000000000))*(x28)))))))+(((IkReal(0.0750000000000000))*(x0)))+(((IkReal(0.405000000000000))*(x30)))+(((x11)*(((((IkReal(-1.00000000000000))*(x18)*(x42)))+(((IkReal(-1.00000000000000))*(x12)*(x33)))))))+(((x9)*(((((IkReal(0.0800000000000000))*(x40)))+(((x19)*(x3)))))))+(((x8)*(((((x0)*(x15)*(x6)))+(((IkReal(-0.0800000000000000))*(x45))))))));
IkReal x46=((x24)*(x3));
IkReal x47=((IkReal(1.00000000000000))*(x3));
eetrans[1]=((((IkReal(-1.00000000000000))*(x29)*(x47)))+(((IkReal(-0.0900000000000000))*(x22)*(x7)))+(((IkReal(-0.0750000000000000))*(x3)))+(((x10)*(((((IkReal(-1.00000000000000))*(x0)*(x13)))+(((IkReal(0.790000000000000))*(x41)))))))+(((x9)*(((((IkReal(-0.0800000000000000))*(x38)))+(((x0)*(x19)))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x15)*(x22)))+(((IkReal(0.0800000000000000))*(x46)))))))+(((IkReal(0.405000000000000))*(x46)))+(((IkReal(-0.405000000000000))*(x1)*(x22)))+(((IkReal(-1.00000000000000))*(x14)*(x47)))+(((x11)*(((((x18)*(x43)))+(((x12)*(x34))))))));
IkReal x48=((IkReal(1.00000000000000))*(x15));
eetrans[2]=((IkReal(0.335000000000000))+(((IkReal(-0.405000000000000))*(x25)))+(((IkReal(-0.405000000000000))*(x21)))+(((x20)*(((((IkReal(0.790000000000000))*(x26)))+(((IkReal(-0.790000000000000))*(x24)))))))+(((IkReal(-0.0900000000000000))*(x24)))+(((x11)*(((((x12)*(x32)))+(((x13)*(x31)*(x8)))))))+(((IkReal(0.365000000000000))*(x6)))+(((x5)*(x9)*(((((IkReal(-1.00000000000000))*(x48)*(x6)))+(((IkReal(0.0800000000000000))*(x24)))))))+(((x8)*(((((IkReal(-0.0800000000000000))*(x25)))+(((IkReal(-1.00000000000000))*(x2)*(x48)))))))+(((IkReal(0.0900000000000000))*(x26))));
eerot[0]=((((x11)*(((((x42)*(x8)))+(x37)))))+(((x10)*(((x39)+(x28))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x41)))+(((x17)*(x5)))))))+(((x11)*(((((x27)*(x34)))+(((x23)*(x43))))))));
eerot[2]=((((x20)*(x31)))+(((x11)*(((((x27)*(x32)))+(((x23)*(x31)*(x5))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j2,cj2,sj2,htj2,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij2[2], _nj2;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij2[0] = -1; _ij2[1] = -1; _nj2 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j2=pfree[0]; cj2=cos(pfree[0]); sj2=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((((IkReal(-0.790000000000000))*(r00)))+(px));
new_r01=((IkReal(-1.00000000000000))*(r01));
new_py=((((IkReal(0.790000000000000))*(r01)))+(((IkReal(-1.00000000000000))*(py))));
new_r02=((IkReal(-1.00000000000000))*(r02));
new_pz=((IkReal(0.335000000000000))+(((IkReal(0.790000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(pz))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(0.790000000000000))+(((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((IkReal(0.365000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(cj2)*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.405000000000000))*(r02)*(sj2))));
IkReal gconst1;
gconst1=((((IkReal(0.810000000000000))*(r00)*(sj2)))+(((IkReal(-0.730000000000000))*(r00)))+(((IkReal(-0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(-0.810000000000000))*(cj2)*(r02))));
IkReal gconst2;
gconst2=((IkReal(0.790000000000000))+(((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.405000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj2)))+(((IkReal(-0.365000000000000))*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02))));
IkReal gconst3;
gconst3=((IkReal(-0.319525000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj2)))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(-0.810000000000000))*(cj2)*(px)))+(((IkReal(0.126450000000000))*(cj2)))+(((IkReal(0.730000000000000))*(pz)))+(((IkReal(-0.282150000000000))*(sj2))));
IkReal gconst4;
gconst4=((IkReal(0.109500000000000))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(-1.46000000000000))*(px)))+(((IkReal(-0.360000000000000))*(cj2)*(px)))+(((IkReal(-0.121500000000000))*(sj2)))+(((IkReal(1.62000000000000))*(px)*(sj2)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.62000000000000))*(cj2)*(pz))));
IkReal gconst5;
gconst5=((IkReal(-0.319525000000000))+(((IkReal(0.810000000000000))*(cj2)*(px)))+(((IkReal(0.810000000000000))*(pz)*(sj2)))+(((IkReal(-0.730000000000000))*(pz)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(0.00495000000000000))*(cj2)))+(pp)+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(-0.309150000000000))*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz))));
IkReal gconst6;
gconst6=((IkReal(0.790000000000000))+(((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((IkReal(0.365000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(cj2)*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.405000000000000))*(r02)*(sj2))));
IkReal gconst7;
gconst7=((((IkReal(0.810000000000000))*(r00)*(sj2)))+(((IkReal(-0.730000000000000))*(r00)))+(((IkReal(-0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(-0.810000000000000))*(cj2)*(r02))));
IkReal gconst8;
gconst8=((IkReal(0.790000000000000))+(((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.405000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj2)))+(((IkReal(-0.365000000000000))*(r02)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02))));
IkReal gconst9;
gconst9=((IkReal(-0.319525000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj2)))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(-0.810000000000000))*(cj2)*(px)))+(((IkReal(0.126450000000000))*(cj2)))+(((IkReal(0.730000000000000))*(pz)))+(((IkReal(-0.282150000000000))*(sj2))));
IkReal gconst10;
gconst10=((IkReal(0.109500000000000))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(-1.46000000000000))*(px)))+(((IkReal(-0.360000000000000))*(cj2)*(px)))+(((IkReal(-0.121500000000000))*(sj2)))+(((IkReal(1.62000000000000))*(px)*(sj2)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.62000000000000))*(cj2)*(pz))));
IkReal gconst11;
gconst11=((IkReal(-0.319525000000000))+(((IkReal(0.810000000000000))*(cj2)*(px)))+(((IkReal(0.810000000000000))*(pz)*(sj2)))+(((IkReal(-0.730000000000000))*(pz)))+(((IkReal(-0.150000000000000))*(px)))+(((IkReal(0.00495000000000000))*(cj2)))+(pp)+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(-0.309150000000000))*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz))));
IkReal gconst12;
gconst12=((((IkReal(-0.180000000000000))*(r01)*(sj2)))+(((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.810000000000000))*(cj2)*(r01))));
IkReal gconst13;
gconst13=((((IkReal(1.62000000000000))*(r01)*(sj2)))+(((IkReal(-0.360000000000000))*(cj2)*(r01)))+(((IkReal(-1.46000000000000))*(r01))));
IkReal gconst14;
gconst14=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(0.810000000000000))*(cj2)*(r01)))+(((IkReal(0.180000000000000))*(r01)*(sj2))));
IkReal gconst15;
gconst15=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-0.360000000000000))*(py)*(sj2)))+(((IkReal(-1.62000000000000))*(cj2)*(py))));
IkReal gconst16;
gconst16=((((IkReal(-2.92000000000000))*(py)))+(((IkReal(3.24000000000000))*(py)*(sj2)))+(((IkReal(-0.720000000000000))*(cj2)*(py))));
IkReal gconst17;
gconst17=((((IkReal(0.360000000000000))*(py)*(sj2)))+(((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.62000000000000))*(cj2)*(py))));
IkReal gconst18;
gconst18=((((IkReal(-0.180000000000000))*(r01)*(sj2)))+(((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.810000000000000))*(cj2)*(r01))));
IkReal gconst19;
gconst19=((((IkReal(1.62000000000000))*(r01)*(sj2)))+(((IkReal(-0.360000000000000))*(cj2)*(r01)))+(((IkReal(-1.46000000000000))*(r01))));
IkReal gconst20;
gconst20=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(0.810000000000000))*(cj2)*(r01)))+(((IkReal(0.180000000000000))*(r01)*(sj2))));
IkReal gconst21;
gconst21=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-0.360000000000000))*(py)*(sj2)))+(((IkReal(-1.62000000000000))*(cj2)*(py))));
IkReal gconst22;
gconst22=((((IkReal(-2.92000000000000))*(py)))+(((IkReal(3.24000000000000))*(py)*(sj2)))+(((IkReal(-0.720000000000000))*(cj2)*(py))));
IkReal gconst23;
gconst23=((((IkReal(0.360000000000000))*(py)*(sj2)))+(((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.62000000000000))*(cj2)*(py))));
IkReal gconst24;
gconst24=((IkReal(0.790000000000000))+(((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.405000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((IkReal(0.365000000000000))*(r02)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.405000000000000))*(r02)*(sj2))));
IkReal gconst25;
gconst25=((((IkReal(0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(-0.810000000000000))*(cj2)*(r02)))+(((IkReal(-0.810000000000000))*(r00)*(sj2)))+(((IkReal(0.730000000000000))*(r00))));
IkReal gconst26;
gconst26=((IkReal(0.790000000000000))+(((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj2)))+(((IkReal(-0.365000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(cj2)*(r00)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02))));
IkReal gconst27;
gconst27=((IkReal(-0.319525000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj2)))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(0.810000000000000))*(cj2)*(px)))+(pp)+(((IkReal(0.126450000000000))*(cj2)))+(((IkReal(0.730000000000000))*(pz)))+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(-0.282150000000000))*(sj2))));
IkReal gconst28;
gconst28=((IkReal(0.109500000000000))+(((IkReal(-1.62000000000000))*(px)*(sj2)))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(0.360000000000000))*(cj2)*(px)))+(((IkReal(1.46000000000000))*(px)))+(((IkReal(-0.121500000000000))*(sj2)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.62000000000000))*(cj2)*(pz))));
IkReal gconst29;
gconst29=((IkReal(-0.319525000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.810000000000000))*(pz)*(sj2)))+(((IkReal(-0.730000000000000))*(pz)))+(((IkReal(0.00495000000000000))*(cj2)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(-0.810000000000000))*(cj2)*(px)))+(((IkReal(-0.309150000000000))*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz))));
IkReal gconst30;
gconst30=((IkReal(0.790000000000000))+(((IkReal(0.0900000000000000))*(r00)*(sj2)))+(((IkReal(0.405000000000000))*(cj2)*(r00)))+(((px)*(r00)))+(((IkReal(0.0900000000000000))*(cj2)*(r02)))+(((IkReal(0.365000000000000))*(r02)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.405000000000000))*(r02)*(sj2))));
IkReal gconst31;
gconst31=((((IkReal(0.180000000000000))*(cj2)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)))+(((IkReal(-0.810000000000000))*(cj2)*(r02)))+(((IkReal(-0.810000000000000))*(r00)*(sj2)))+(((IkReal(0.730000000000000))*(r00))));
IkReal gconst32;
gconst32=((IkReal(0.790000000000000))+(((IkReal(-0.0900000000000000))*(r00)*(sj2)))+(((px)*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj2)))+(((IkReal(-0.365000000000000))*(r02)))+(((IkReal(-0.405000000000000))*(cj2)*(r00)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj2)*(r02))));
IkReal gconst33;
gconst33=((IkReal(-0.319525000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj2)))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.180000000000000))*(cj2)*(pz)))+(((IkReal(0.810000000000000))*(cj2)*(px)))+(pp)+(((IkReal(0.126450000000000))*(cj2)))+(((IkReal(0.730000000000000))*(pz)))+(((IkReal(0.180000000000000))*(px)*(sj2)))+(((IkReal(-0.282150000000000))*(sj2))));
IkReal gconst34;
gconst34=((IkReal(0.109500000000000))+(((IkReal(-1.62000000000000))*(px)*(sj2)))+(((IkReal(-0.360000000000000))*(pz)*(sj2)))+(((IkReal(0.360000000000000))*(cj2)*(px)))+(((IkReal(1.46000000000000))*(px)))+(((IkReal(-0.121500000000000))*(sj2)))+(((IkReal(0.0270000000000000))*(cj2)))+(((IkReal(-1.62000000000000))*(cj2)*(pz))));
IkReal gconst35;
gconst35=((IkReal(-0.319525000000000))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.810000000000000))*(pz)*(sj2)))+(((IkReal(-0.730000000000000))*(pz)))+(((IkReal(0.00495000000000000))*(cj2)))+(((IkReal(-0.180000000000000))*(px)*(sj2)))+(pp)+(((IkReal(-0.810000000000000))*(cj2)*(px)))+(((IkReal(-0.309150000000000))*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(pz))));
IkReal x49=((IkReal(1.00000000000000))*(gconst26));
IkReal x50=((gconst30)*(gconst34));
IkReal x51=((gconst31)*(gconst33));
IkReal x52=((IkReal(1.00000000000000))*(gconst17));
IkReal x53=((gconst2)*(gconst6));
IkReal x54=((gconst11)*(gconst3));
IkReal x55=((gconst25)*(gconst29));
IkReal x56=((gconst14)*(gconst28));
IkReal x57=((IkReal(1.00000000000000))*(gconst14));
IkReal x58=((gconst21)*(gconst32));
IkReal x59=((gconst10)*(gconst6));
IkReal x60=((gconst21)*(gconst5));
IkReal x61=((gconst0)*(gconst8));
IkReal x62=((gconst13)*(gconst5));
IkReal x63=((IkReal(1.00000000000000))*(gconst9));
IkReal x64=((gconst30)*(gconst35));
IkReal x65=((gconst26)*(gconst27));
IkReal x66=((gconst22)*(gconst30));
IkReal x67=((IkReal(1.00000000000000))*(gconst12));
IkReal x68=((gconst2)*(gconst3));
IkReal x69=((gconst1)*(gconst5));
IkReal x70=((gconst19)*(gconst33));
IkReal x71=((gconst26)*(gconst28));
IkReal x72=((gconst11)*(gconst6));
IkReal x73=((gconst1)*(gconst7));
IkReal x74=((gconst18)*(gconst2));
IkReal x75=((gconst18)*(gconst34));
IkReal x76=((gconst24)*(gconst29));
IkReal x77=((IkReal(1.00000000000000))*(gconst23));
IkReal x78=((gconst20)*(gconst33));
IkReal x79=((gconst32)*(gconst33));
IkReal x80=((gconst0)*(gconst5));
IkReal x81=((IkReal(1.00000000000000))*(gconst18));
IkReal x82=((gconst21)*(gconst31));
IkReal x83=((gconst5)*(gconst9));
IkReal x84=((gconst4)*(gconst9));
IkReal x85=((gconst13)*(gconst29));
IkReal x86=((gconst14)*(gconst6));
IkReal x87=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x88=((gconst27)*(x79));
IkReal x89=((gconst2)*(gconst4)*(gconst7));
op[0]=((((IkReal(-1.00000000000000))*(x51)*(x55)))+(((x50)*(x55)))+(((x76)*(x79)))+(((IkReal(-1.00000000000000))*(x64)*(x76)))+(((x51)*(x71)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x50)))+(((x64)*(x65)))+(((IkReal(-1.00000000000000))*(x49)*(x88))));
op[1]=((((gconst16)*(gconst26)*(x51)))+(((x50)*(x85)))+(((x58)*(x76)))+(((IkReal(-1.00000000000000))*(gconst24)*(x52)*(x64)))+(((IkReal(-1.00000000000000))*(gconst27)*(x49)*(x78)))+(((gconst15)*(gconst26)*(x64)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x75)))+(((IkReal(-1.00000000000000))*(x55)*(x82)))+(((IkReal(-1.00000000000000))*(gconst16)*(x49)*(x50)))+(((gconst17)*(gconst25)*(x50)))+(((IkReal(-1.00000000000000))*(x51)*(x85)))+(((IkReal(-1.00000000000000))*(gconst15)*(x49)*(x79)))+(((IkReal(-1.00000000000000))*(x57)*(x88)))+(((IkReal(-1.00000000000000))*(x55)*(x70)))+(((IkReal(-1.00000000000000))*(x50)*(x56)))+(((gconst12)*(gconst29)*(x79)))+(((x76)*(x78)))+(((gconst14)*(gconst27)*(x64)))+(((IkReal(-1.00000000000000))*(gconst27)*(x49)*(x58)))+(((gconst18)*(gconst35)*(x65)))+(((x55)*(x75)))+(((gconst17)*(gconst24)*(x79)))+(((x51)*(x56)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x66)))+(((x70)*(x71)))+(((IkReal(-1.00000000000000))*(gconst29)*(x64)*(x67)))+(((gconst23)*(gconst30)*(x65)))+(((IkReal(-1.00000000000000))*(gconst25)*(x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst30)*(x76)*(x77)))+(((IkReal(-1.00000000000000))*(gconst35)*(x76)*(x81)))+(((x55)*(x66)))+(((x71)*(x82))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((gconst22)*(gconst6)*(x69)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x53)))+(((gconst17)*(gconst9)*(x61)))+(((IkReal(-1.00000000000000))*(gconst19)*(x63)*(x69)))+(((IkReal(-1.00000000000000))*(gconst11)*(x80)*(x81)))+(((x59)*(x62)))+(((gconst1)*(gconst17)*(x59)))+(((IkReal(-1.00000000000000))*(gconst5)*(x67)*(x72)))+(((gconst10)*(gconst18)*(x69)))+(((IkReal(-1.00000000000000))*(gconst0)*(x52)*(x72)))+(((x60)*(x61)))+(((x54)*(x86)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x68)))+(((gconst11)*(gconst15)*(x53)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x53)))+(((IkReal(-1.00000000000000))*(gconst4)*(x57)*(x59)))+(((gconst20)*(gconst9)*(x80)))+(((IkReal(-1.00000000000000))*(gconst20)*(x63)*(x68)))+(((IkReal(-1.00000000000000))*(gconst7)*(x62)*(x63)))+(((gconst23)*(gconst3)*(x53)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x57)))+(((gconst14)*(gconst7)*(x84)))+(((IkReal(-1.00000000000000))*(x74)*(x87)))+(((x54)*(x74)))+(((IkReal(-1.00000000000000))*(x60)*(x73)))+(((IkReal(-1.00000000000000))*(gconst6)*(x77)*(x80)))+(((gconst21)*(x89)))+(((IkReal(-1.00000000000000))*(gconst9)*(x52)*(x73)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((gconst12)*(gconst8)*(x83)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x63)))+(((gconst19)*(gconst2)*(x84))));
op[8]=((((IkReal(-1.00000000000000))*(gconst8)*(x63)*(x68)))+(((x61)*(x83)))+(((x59)*(x69)))+(((IkReal(-1.00000000000000))*(gconst7)*(x63)*(x69)))+(((IkReal(-1.00000000000000))*(x53)*(x87)))+(((IkReal(-1.00000000000000))*(x72)*(x80)))+(((x53)*(x54)))+(((gconst2)*(gconst7)*(x84))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x90=(cj2)*(cj2);
IkReal x91=(sj2)*(sj2);
IkReal x92=((cj0)*(r00));
IkReal x93=((py)*(sj0));
IkReal x94=((r01)*(sj0));
IkReal x95=((IkReal(0.266450000000000))*(pz));
IkReal x96=((cj0)*(px));
IkReal x97=((r02)*(sj2));
IkReal x98=((IkReal(0.266450000000000))*(r02));
IkReal x99=((IkReal(0.0258187500000000))*(r02));
IkReal x100=((IkReal(0.131400000000000))*(cj2)*(r02));
IkReal x101=((IkReal(0.344250000000000))*(x91));
IkReal x102=((IkReal(0.131400000000000))*(cj2)*(pz));
IkReal x103=((IkReal(0.591300000000000))*(pz)*(sj2));
IkReal x104=((IkReal(0.344250000000000))*(pz)*(x90));
IkReal x105=((IkReal(0.344250000000000))*(r02)*(x90));
gconst52=IKsign(((((r02)*(x101)*(x93)))+(((r02)*(x101)*(x96)))+(((x96)*(x98)))+(((IkReal(-1.00000000000000))*(x102)*(x92)))+(((IkReal(-1.00000000000000))*(x102)*(x94)))+(((x100)*(x96)))+(((x100)*(x93)))+(((IkReal(-1.00000000000000))*(x94)*(x95)))+(((IkReal(0.0443475000000000))*(x97)))+(((IkReal(-0.00985500000000000))*(cj2)*(r02)))+(((IkReal(-0.591300000000000))*(x96)*(x97)))+(((IkReal(-1.00000000000000))*(x90)*(x99)))+(((x103)*(x94)))+(((x103)*(x92)))+(((IkReal(-1.00000000000000))*(x91)*(x99)))+(((IkReal(-0.0199837500000000))*(r02)))+(((IkReal(-0.591300000000000))*(x93)*(x97)))+(((x93)*(x98)))+(((IkReal(-1.00000000000000))*(x92)*(x95)))+(((IkReal(-1.00000000000000))*(pz)*(x101)*(x94)))+(((IkReal(-1.00000000000000))*(pz)*(x101)*(x92)))+(((x105)*(x93)))+(((x105)*(x96)))+(((IkReal(-1.00000000000000))*(x104)*(x94)))+(((IkReal(-1.00000000000000))*(x104)*(x92)))));
IkReal x106=(sj2)*(sj2);
IkReal x107=(cj2)*(cj2);
IkReal x108=((IkReal(34.9315068493151))*(sj0));
IkReal x109=((cj2)*(r02));
IkReal x110=((IkReal(60.0000000000000))*(sj2));
IkReal x111=((IkReal(13.3333333333333))*(cj2));
IkReal x112=((cj0)*(px));
IkReal x113=((pz)*(r01));
IkReal x114=((pz)*(x107));
IkReal x115=((r02)*(x106));
IkReal x116=((IkReal(34.9315068493151))*(x112));
IkReal x117=((cj0)*(pz)*(r00));
IkReal x118=((r02)*(x107));
IkReal x119=((py)*(r02)*(sj0));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x111)*(x117)))+(((x110)*(x117)))+(((IkReal(13.3333333333333))*(py)*(sj0)*(x109)))+(((IkReal(-27.0370370370370))*(x117)))+(((IkReal(-1.00000000000000))*(sj0)*(x111)*(x113)))+(((IkReal(-1.00000000000000))*(x107)*(x108)*(x113)))+(((IkReal(-34.9315068493151))*(x106)*(x117)))+(((IkReal(4.50000000000000))*(r02)*(sj2)))+(((IkReal(-1.00000000000000))*(x110)*(x119)))+(((IkReal(-34.9315068493151))*(cj0)*(r00)*(x114)))+(((IkReal(-27.0370370370370))*(sj0)*(x113)))+(((py)*(x108)*(x118)))+(((py)*(x108)*(x115)))+(((IkReal(-2.61986301369863))*(x115)))+(((IkReal(-2.61986301369863))*(x118)))+(((IkReal(13.3333333333333))*(x109)*(x112)))+(((x116)*(x118)))+(((IkReal(-1.00000000000000))*(r02)*(x110)*(x112)))+(((IkReal(-2.02777777777778))*(r02)))+(((IkReal(-1.00000000000000))*(x106)*(x108)*(x113)))+(((x115)*(x116)))+(((sj0)*(x110)*(x113)))+(((IkReal(-1.00000000000000))*(x109)))+(((IkReal(27.0370370370370))*(r02)*(x112)))+(((IkReal(27.0370370370370))*(x119))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
if( IKabs(((gconst52)*(((((IkReal(-0.0479925000000000))*(cj2)))+(((IkReal(0.142200000000000))*(cj0)*(px)*(sj2)))+(((IkReal(0.0547500000000000))*(cj0)*(pz)*(r00)))+(((IkReal(-0.00477675000000000))*(cj2)*(r02)))+(((IkReal(-0.0607500000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.810000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(-0.116626625000000))*(r02)))+(((IkReal(0.810000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.0135000000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0532170000000000))*(cj2)*(r02)*(sj2)))+(((IkReal(0.133963875000000))*(cj0)*(cj2)*(r00)))+(((IkReal(0.639900000000000))*(cj2)*(py)*(sj0)))+(((IkReal(0.119738250000000))*(r02)*((sj2)*(sj2))))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(0.0297697500000000))*(cj0)*(r00)*(sj2)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(-0.730000000000000))*(r02)*((pz)*(pz))))+(((IkReal(-0.0547500000000000))*(py)*(r02)*(sj0)))+(((IkReal(0.133963875000000))*(cj2)*(r01)*(sj0)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(0.0214953750000000))*(r02)*(sj2)))+(((IkReal(-0.0266085000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(0.810000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.142200000000000))*(cj2)*(pz)))+(((IkReal(0.00591300000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(0.113825250000000))*(cj2)*(r01)*(sj0)*(sj2)))+(((IkReal(0.0266085000000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(0.113825250000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj2)))+(((IkReal(-0.405000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(-0.576700000000000))*(pz)))+(((IkReal(0.142200000000000))*(py)*(sj0)*(sj2)))+(((IkReal(-0.405000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(-0.0266085000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(0.0547500000000000))*(pz)*(r01)*(sj0)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(0.810000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(0.810000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.730000000000000))*(py)*(pz)*(r01)))+(((IkReal(-0.0106650000000000))*(sj2)))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj2)*(pz)*(r02)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(0.639900000000000))*(cj0)*(cj2)*(px)))+(((IkReal(0.0607500000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(0.0266085000000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(0.810000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(0.810000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(0.810000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.365000000000000))*(pp)*(r02)))+(((IkReal(0.810000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-0.730000000000000))*(px)*(pz)*(r00)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(0.0297697500000000))*(r01)*(sj0)*(sj2)))+(((IkReal(0.0607500000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.405000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.0135000000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(-0.0135000000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(0.639900000000000))*(pz)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj2)*(py)*(r01)))+(((IkReal(-0.0547500000000000))*(cj0)*(px)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((IkReal(0.0432525000000000))+(((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.129407625000000))*(cj2)*(r02)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(-0.730000000000000))*(py)*(pz)*(r02)*(sj0)))+(((IkReal(0.810000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(0.0547500000000000))*(py)*(r01)))+(((IkReal(-0.0287572500000000))*(r02)*(sj2)))+(((IkReal(0.119738250000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.576700000000000))*(py)*(sj0)))+(((IkReal(-0.120732875000000))*(r01)*(sj0)))+(((IkReal(0.0260516250000000))*(r01)*(sj0)*(sj2)))+(((IkReal(0.0607500000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0607500000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(-0.810000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(-0.00578925000000000))*(cj0)*(cj2)*(r00)))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(-0.730000000000000))*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.810000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-0.142200000000000))*(cj0)*(cj2)*(px)))+(((IkReal(-0.142200000000000))*(cj2)*(py)*(sj0)))+(((IkReal(-0.113825250000000))*(cj2)*(r02)*(sj2)))+(((IkReal(-0.0607500000000000))*(pz)*(r02)*(sj2)))+(((IkReal(0.0547500000000000))*(px)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.810000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(0.00591300000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.0900000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(0.0106650000000000))*(cj2)))+(((IkReal(-0.0532170000000000))*(cj2)*(r01)*(sj0)*(sj2)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(0.0607500000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.810000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(0.405000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(0.810000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(-0.405000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(-0.576700000000000))*(cj0)*(px)))+(((IkReal(0.0135000000000000))*(cj2)*(py)*(r01)))+(((IkReal(-0.0266085000000000))*(r02)*((sj2)*(sj2))))+(((IkReal(-0.0532170000000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.810000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(-0.0607500000000000))*(py)*(r01)*(sj2)))+(((IkReal(-0.0479925000000000))*(sj2)))+(((IkReal(0.810000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.405000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.120732875000000))*(cj0)*(r00)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(-0.639900000000000))*(cj2)*(pz)))+(((IkReal(-0.0607500000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(-0.810000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(0.639900000000000))*(cj0)*(px)*(sj2)))+(((IkReal(0.0266085000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.0607500000000000))*(px)*(r00)*(sj2)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r02)))+(((IkReal(-0.142200000000000))*(pz)*(sj2)))+(((IkReal(-0.730000000000000))*(cj0)*(px)*(py)*(r01)))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(-0.730000000000000))*(cj0)*(px)*(pz)*(r02)))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.0260516250000000))*(cj0)*(r00)*(sj2)))+(((IkReal(0.639900000000000))*(py)*(sj0)*(sj2)))+(((IkReal(-0.00578925000000000))*(cj2)*(r01)*(sj0)))+(((IkReal(0.00591300000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(0.365000000000000))*(cj0)*(pp)*(r00)))+(((IkReal(0.119738250000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(0.365000000000000))*(pp)*(r01)*(sj0)))+(((IkReal(0.0547500000000000))*(pz)*(r02)))+(((IkReal(-0.730000000000000))*(cj0)*(r00)*((px)*(px))))+(((IkReal(-0.730000000000000))*(r01)*(sj0)*((py)*(py)))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst52)*(((((IkReal(-0.0479925000000000))*(cj2)))+(((IkReal(0.142200000000000))*(cj0)*(px)*(sj2)))+(((IkReal(0.0547500000000000))*(cj0)*(pz)*(r00)))+(((IkReal(-0.00477675000000000))*(cj2)*(r02)))+(((IkReal(-0.0607500000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.810000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(-0.116626625000000))*(r02)))+(((IkReal(0.810000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.0135000000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0532170000000000))*(cj2)*(r02)*(sj2)))+(((IkReal(0.133963875000000))*(cj0)*(cj2)*(r00)))+(((IkReal(0.639900000000000))*(cj2)*(py)*(sj0)))+(((IkReal(0.119738250000000))*(r02)*((sj2)*(sj2))))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(0.0297697500000000))*(cj0)*(r00)*(sj2)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(-0.730000000000000))*(r02)*((pz)*(pz))))+(((IkReal(-0.0547500000000000))*(py)*(r02)*(sj0)))+(((IkReal(0.133963875000000))*(cj2)*(r01)*(sj0)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(0.0214953750000000))*(r02)*(sj2)))+(((IkReal(-0.0266085000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(0.810000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.142200000000000))*(cj2)*(pz)))+(((IkReal(0.00591300000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(0.113825250000000))*(cj2)*(r01)*(sj0)*(sj2)))+(((IkReal(0.0266085000000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(0.113825250000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj2)))+(((IkReal(-0.405000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(-0.576700000000000))*(pz)))+(((IkReal(0.142200000000000))*(py)*(sj0)*(sj2)))+(((IkReal(-0.405000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(-0.0266085000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(0.0547500000000000))*(pz)*(r01)*(sj0)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(0.810000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(0.810000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.730000000000000))*(py)*(pz)*(r01)))+(((IkReal(-0.0106650000000000))*(sj2)))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj2)*(pz)*(r02)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(0.639900000000000))*(cj0)*(cj2)*(px)))+(((IkReal(0.0607500000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(0.0266085000000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(0.810000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(0.810000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(0.810000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.365000000000000))*(pp)*(r02)))+(((IkReal(0.810000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(-0.730000000000000))*(px)*(pz)*(r00)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(0.0297697500000000))*(r01)*(sj0)*(sj2)))+(((IkReal(0.0607500000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.405000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(-0.0135000000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(-0.0135000000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(0.639900000000000))*(pz)*(sj2)))+(((IkReal(-0.0607500000000000))*(cj2)*(py)*(r01)))+(((IkReal(-0.0547500000000000))*(cj0)*(px)*(r02)))))), ((gconst52)*(((IkReal(0.0432525000000000))+(((IkReal(-0.180000000000000))*(cj2)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(-0.129407625000000))*(cj2)*(r02)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(py)*(r01)))+(((IkReal(-0.730000000000000))*(py)*(pz)*(r02)*(sj0)))+(((IkReal(0.810000000000000))*(cj0)*(px)*(pz)*(r02)*(sj2)))+(((IkReal(0.0547500000000000))*(py)*(r01)))+(((IkReal(-0.0287572500000000))*(r02)*(sj2)))+(((IkReal(0.119738250000000))*(cj0)*(r00)*((sj2)*(sj2))))+(((IkReal(-0.576700000000000))*(py)*(sj0)))+(((IkReal(-0.120732875000000))*(r01)*(sj0)))+(((IkReal(0.0260516250000000))*(r01)*(sj0)*(sj2)))+(((IkReal(0.0607500000000000))*(cj0)*(cj2)*(pz)*(r00)))+(((IkReal(-0.0607500000000000))*(cj2)*(py)*(r02)*(sj0)))+(((IkReal(-0.810000000000000))*(cj2)*(r02)*((pz)*(pz))))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(px)*(pz)*(r02)))+(((IkReal(-0.00578925000000000))*(cj0)*(cj2)*(r00)))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj2)))+(((IkReal(-0.730000000000000))*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.810000000000000))*(cj0)*(r00)*(sj2)*((px)*(px))))+(((IkReal(-0.142200000000000))*(cj0)*(cj2)*(px)))+(((IkReal(-0.142200000000000))*(cj2)*(py)*(sj0)))+(((IkReal(-0.113825250000000))*(cj2)*(r02)*(sj2)))+(((IkReal(-0.0607500000000000))*(pz)*(r02)*(sj2)))+(((IkReal(0.0547500000000000))*(px)*(r00)))+(((IkReal(-0.180000000000000))*(r02)*(sj2)*((pz)*(pz))))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj2)))+(((IkReal(0.810000000000000))*(px)*(py)*(r00)*(sj0)*(sj2)))+(((IkReal(0.00591300000000000))*(r01)*(sj0)*((cj2)*(cj2))))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj2)))+(((IkReal(0.0900000000000000))*(cj0)*(cj2)*(pp)*(r00)))+(((IkReal(0.0106650000000000))*(cj2)))+(((IkReal(-0.0532170000000000))*(cj2)*(r01)*(sj0)*(sj2)))+(((IkReal(-0.180000000000000))*(cj0)*(cj2)*(r00)*((px)*(px))))+(((IkReal(0.0607500000000000))*(cj2)*(pz)*(r01)*(sj0)))+(((IkReal(0.810000000000000))*(py)*(pz)*(r02)*(sj0)*(sj2)))+(((IkReal(0.405000000000000))*(cj2)*(pp)*(r02)))+(((IkReal(0.810000000000000))*(r01)*(sj0)*(sj2)*((py)*(py))))+(((IkReal(-0.405000000000000))*(pp)*(r01)*(sj0)*(sj2)))+(((IkReal(-0.576700000000000))*(cj0)*(px)))+(((IkReal(0.0135000000000000))*(cj2)*(py)*(r01)))+(((IkReal(-0.0266085000000000))*(r02)*((sj2)*(sj2))))+(((IkReal(-0.0532170000000000))*(cj0)*(cj2)*(r00)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(px)*(r00)))+(((IkReal(-0.810000000000000))*(cj2)*(py)*(pz)*(r01)))+(((IkReal(-0.0607500000000000))*(py)*(r01)*(sj2)))+(((IkReal(-0.0479925000000000))*(sj2)))+(((IkReal(0.810000000000000))*(cj0)*(px)*(py)*(r01)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(r01)*(sj0)*((py)*(py))))+(((IkReal(-0.405000000000000))*(cj0)*(pp)*(r00)*(sj2)))+(((IkReal(-0.180000000000000))*(cj2)*(px)*(py)*(r00)*(sj0)))+(((IkReal(-0.120732875000000))*(cj0)*(r00)))+(((IkReal(0.0900000000000000))*(cj2)*(pp)*(r01)*(sj0)))+(((IkReal(-0.639900000000000))*(cj2)*(pz)))+(((IkReal(-0.0607500000000000))*(cj0)*(cj2)*(px)*(r02)))+(((IkReal(-0.810000000000000))*(cj2)*(px)*(pz)*(r00)))+(((IkReal(0.639900000000000))*(cj0)*(px)*(sj2)))+(((IkReal(0.0266085000000000))*(r02)*((cj2)*(cj2))))+(((IkReal(-0.0607500000000000))*(px)*(r00)*(sj2)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj2)))+(((IkReal(0.0135000000000000))*(cj2)*(pz)*(r02)))+(((IkReal(-0.142200000000000))*(pz)*(sj2)))+(((IkReal(-0.730000000000000))*(cj0)*(px)*(py)*(r01)))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj2)))+(((IkReal(-0.730000000000000))*(cj0)*(px)*(pz)*(r02)))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj2)))+(((IkReal(0.0260516250000000))*(cj0)*(r00)*(sj2)))+(((IkReal(0.639900000000000))*(py)*(sj0)*(sj2)))+(((IkReal(-0.00578925000000000))*(cj2)*(r01)*(sj0)))+(((IkReal(0.00591300000000000))*(cj0)*(r00)*((cj2)*(cj2))))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj2)))+(((IkReal(0.365000000000000))*(cj0)*(pp)*(r00)))+(((IkReal(0.119738250000000))*(r01)*(sj0)*((sj2)*(sj2))))+(((IkReal(0.365000000000000))*(pp)*(r01)*(sj0)))+(((IkReal(0.0547500000000000))*(pz)*(r02)))+(((IkReal(-0.730000000000000))*(cj0)*(r00)*((px)*(px))))+(((IkReal(-0.730000000000000))*(r01)*(sj0)*((py)*(py))))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[2];
IkReal x120=IKcos(j1);
IkReal x121=IKsin(j1);
IkReal x122=((IkReal(0.0900000000000000))*(cj2));
IkReal x123=((cj0)*(px));
IkReal x124=((py)*(sj0));
IkReal x125=((cj0)*(r00));
IkReal x126=((r01)*(sj0));
IkReal x127=((IkReal(0.405000000000000))*(r02));
IkReal x128=((IkReal(0.405000000000000))*(sj2));
IkReal x129=((IkReal(0.810000000000000))*(pz));
IkReal x130=((sj2)*(x120));
IkReal x131=((IkReal(0.180000000000000))*(x121));
IkReal x132=((pz)*(x120));
IkReal x133=((r02)*(x120));
IkReal x134=((IkReal(0.730000000000000))*(x121));
IkReal x135=((cj2)*(x121));
IkReal x136=((sj2)*(x121));
IkReal x137=((cj2)*(x120));
IkReal x138=((x121)*(x125));
IkReal x139=((x121)*(x126));
evalcond[0]=((IkReal(-0.319525000000000))+(((IkReal(-1.00000000000000))*(x123)*(x134)))+(((IkReal(-0.295650000000000))*(sj2)))+(((IkReal(0.810000000000000))*(x124)*(x136)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x131)))+(((IkReal(-0.810000000000000))*(x123)*(x137)))+(((IkReal(0.0657000000000000))*(cj2)))+(((IkReal(0.730000000000000))*(x132)))+(((IkReal(-0.180000000000000))*(x123)*(x130)))+(((IkReal(-1.00000000000000))*(x124)*(x134)))+(((IkReal(-0.180000000000000))*(x124)*(x130)))+(((IkReal(-0.0607500000000000))*(x136)))+(((IkReal(0.0547500000000000))*(x121)))+(((IkReal(0.810000000000000))*(x123)*(x136)))+(((IkReal(0.0607500000000000))*(x137)))+(pp)+(((IkReal(-0.150000000000000))*(x123)))+(((IkReal(-0.150000000000000))*(x124)))+(((IkReal(-1.00000000000000))*(cj2)*(x123)*(x131)))+(((IkReal(0.0135000000000000))*(x130)))+(((IkReal(0.0135000000000000))*(x135)))+(((IkReal(-1.00000000000000))*(x129)*(x135)))+(((IkReal(-1.00000000000000))*(x129)*(x130)))+(((IkReal(-0.810000000000000))*(x124)*(x137)))+(((IkReal(0.180000000000000))*(cj2)*(x132)))+(((IkReal(-1.00000000000000))*(cj2)*(x124)*(x131))));
evalcond[1]=((IkReal(0.790000000000000))+(((IkReal(-1.00000000000000))*(x122)*(x138)))+(((IkReal(-1.00000000000000))*(x122)*(x139)))+(((IkReal(-0.0900000000000000))*(x125)*(x130)))+(((px)*(r00)))+(((IkReal(-0.0900000000000000))*(r02)*(x136)))+(((IkReal(-0.0750000000000000))*(x126)))+(((IkReal(-0.0750000000000000))*(x125)))+(((x122)*(x133)))+(((IkReal(-0.365000000000000))*(x139)))+(((IkReal(-0.365000000000000))*(x138)))+(((IkReal(-0.0900000000000000))*(x126)*(x130)))+(((IkReal(-0.405000000000000))*(x125)*(x137)))+(((IkReal(-0.405000000000000))*(x126)*(x137)))+(((x128)*(x139)))+(((x128)*(x138)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.365000000000000))*(x133)))+(((IkReal(-1.00000000000000))*(x127)*(x135)))+(((IkReal(-1.00000000000000))*(x127)*(x130))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x140=((sj1)*(sj2));
IkReal x141=((cj1)*(cj2));
IkReal x142=((cj2)*(sj1));
IkReal x143=((IkReal(9.87500000000000))*(r02));
IkReal x144=((IkReal(12.5000000000000))*(pz));
IkReal x145=((cj1)*(sj2));
IkReal x146=((IkReal(9.87500000000000))*(r01)*(sj0));
IkReal x147=((IkReal(12.5000000000000))*(cj0)*(px));
IkReal x148=((IkReal(12.5000000000000))*(py)*(sj0));
IkReal x149=((IkReal(9.87500000000000))*(cj0)*(r00));
cj4array[0]=((IkReal(-5.06250000000000))+(((IkReal(0.937500000000000))*(x140)))+(((x142)*(x143)))+(((x142)*(x144)))+(((IkReal(-1.00000000000000))*(x140)*(x149)))+(((IkReal(-1.00000000000000))*(x140)*(x146)))+(((IkReal(-1.00000000000000))*(x140)*(x148)))+(((IkReal(-1.00000000000000))*(x140)*(x147)))+(((x144)*(x145)))+(((IkReal(4.56250000000000))*(sj2)))+(((x141)*(x148)))+(((x141)*(x147)))+(((x141)*(x146)))+(((x141)*(x149)))+(((IkReal(-0.937500000000000))*(x141)))+(((x143)*(x145))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x150=((cj1)*(cj2));
IkReal x151=((IkReal(1.00000000000000))*(cj0));
IkReal x152=((sj1)*(sj2));
IkReal x153=((IkReal(1.00000000000000))*(sj0));
IkReal x154=((sj0)*(x152));
IkReal x155=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x156=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x152)))+(((r01)*(x154)))+(((IkReal(-1.00000000000000))*(r00)*(x150)*(x151)))+(((IkReal(-1.00000000000000))*(r02)*(x155)))+(((IkReal(-1.00000000000000))*(r02)*(x156)))+(((IkReal(-1.00000000000000))*(r01)*(x150)*(x153))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x150)))+(((IkReal(-1.00000000000000))*(pz)*(x156)))+(((IkReal(-1.00000000000000))*(pz)*(x155)))+(((IkReal(-1.00000000000000))*(px)*(x150)*(x151)))+(((IkReal(-1.00000000000000))*(py)*(x150)*(x153)))+(((cj0)*(px)*(x152)))+(((IkReal(-0.0750000000000000))*(x152)))+(((py)*(x154))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst64;
gconst64=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst65;
gconst65=IkReal(1.58000000000000);
IkReal gconst66;
gconst66=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst67;
gconst67=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst68;
gconst68=IkReal(-2.00000000000000);
IkReal gconst69;
gconst69=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst70;
gconst70=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst71;
gconst71=IkReal(1.58000000000000);
IkReal gconst72;
gconst72=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(1.58000000000000);
IkReal gconst76;
gconst76=IkReal(-1.58000000000000);
IkReal gconst77;
gconst77=IkReal(-2.00000000000000);
IkReal gconst78;
gconst78=IkReal(2.00000000000000);
IkReal gconst79;
gconst79=IkReal(1.58000000000000);
IkReal gconst80;
gconst80=IkReal(-1.58000000000000);
IkReal gconst81;
gconst81=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst82;
gconst82=IkReal(2.00000000000000);
IkReal gconst83;
gconst83=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst84;
gconst84=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst85;
gconst85=IkReal(-1.58000000000000);
IkReal gconst86;
gconst86=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst87;
gconst87=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst88;
gconst88=IkReal(2.00000000000000);
IkReal gconst89;
gconst89=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst90;
gconst90=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst91;
gconst91=IkReal(-1.58000000000000);
IkReal gconst92;
gconst92=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal x157=((gconst89)*(gconst90));
IkReal x158=((gconst83)*(gconst84));
IkReal x159=((gconst81)*(gconst86));
IkReal x160=((gconst87)*(gconst92));
IkReal x161=((gconst82)*(gconst86));
IkReal x162=((gconst87)*(gconst91));
IkReal x163=((gconst88)*(gconst90));
IkReal x164=((gconst83)*(gconst85));
dummyeval[0]=((((x161)*(x162)))+(((x157)*(x159)))+(((IkReal(-1.00000000000000))*(x157)*(x158)))+(((IkReal(-1.00000000000000))*(x162)*(x164)))+(((IkReal(-1.00000000000000))*(x161)*(x163)))+(((IkReal(-1.00000000000000))*(x159)*(x160)))+(((x158)*(x160)))+(((x163)*(x164))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x165=((gconst63)*(gconst67));
IkReal x166=((IkReal(1.00000000000000))*(gconst65));
IkReal x167=((IkReal(1.00000000000000))*(gconst86));
IkReal x168=((gconst80)*(gconst87));
IkReal x169=((gconst68)*(gconst82));
IkReal x170=((gconst72)*(gconst75));
IkReal x171=((gconst72)*(gconst77));
IkReal x172=((gconst61)*(gconst76));
IkReal x173=((gconst63)*(gconst87));
IkReal x174=((gconst61)*(gconst66));
IkReal x175=((gconst70)*(gconst78));
IkReal x176=((gconst86)*(gconst89));
IkReal x177=((gconst73)*(gconst90));
IkReal x178=((gconst64)*(gconst72));
IkReal x179=((gconst87)*(gconst92));
IkReal x180=((IkReal(1.00000000000000))*(gconst83));
IkReal x181=((gconst85)*(gconst91));
IkReal x182=((gconst69)*(gconst90));
IkReal x183=((gconst78)*(gconst90));
IkReal x184=((IkReal(1.00000000000000))*(gconst70));
IkReal x185=((gconst62)*(gconst66));
IkReal x186=((gconst62)*(gconst86));
IkReal x187=((gconst67)*(gconst71));
IkReal x188=((gconst72)*(gconst84));
IkReal x189=((gconst65)*(gconst68));
IkReal x190=((gconst74)*(gconst79));
IkReal x191=((gconst63)*(gconst69));
IkReal x192=((IkReal(1.00000000000000))*(gconst67));
IkReal x193=((gconst66)*(gconst73));
IkReal x194=((gconst70)*(gconst81));
IkReal x195=((gconst63)*(gconst70));
IkReal x196=((gconst79)*(gconst81));
IkReal x197=((gconst78)*(gconst86));
IkReal x198=((gconst84)*(gconst92));
IkReal x199=((IkReal(1.00000000000000))*(gconst79));
IkReal x200=((gconst83)*(gconst84));
IkReal x201=((gconst75)*(gconst83));
IkReal x202=((gconst64)*(gconst74));
IkReal x203=((gconst81)*(gconst92));
IkReal x204=((gconst63)*(gconst64));
IkReal x205=((gconst76)*(gconst89));
IkReal x206=((gconst65)*(gconst88));
IkReal x207=((gconst79)*(gconst84));
IkReal x208=((gconst87)*(gconst91));
IkReal x209=((gconst81)*(gconst86));
IkReal x210=((gconst69)*(gconst79));
IkReal x211=((gconst67)*(gconst83));
IkReal x212=((gconst82)*(gconst88));
IkReal x213=((gconst69)*(gconst70));
IkReal x214=((gconst67)*(gconst74));
IkReal x215=((gconst70)*(gconst89));
IkReal x216=((IkReal(1.00000000000000))*(gconst77));
IkReal x217=((IkReal(1.00000000000000))*(gconst81));
IkReal x218=((gconst68)*(gconst85));
IkReal x219=((gconst83)*(gconst90));
IkReal x220=((gconst62)*(gconst68));
IkReal x221=((gconst82)*(gconst86));
IkReal x222=((gconst67)*(gconst91));
IkReal x223=((IkReal(1.00000000000000))*(gconst63));
IkReal x224=((gconst85)*(gconst88));
IkReal x225=((gconst74)*(gconst84));
IkReal x226=((gconst89)*(gconst90));
IkReal x227=((gconst72)*(gconst87));
IkReal x228=((gconst73)*(gconst76));
IkReal x229=((gconst66)*(gconst81));
IkReal x230=((gconst63)*(gconst90));
IkReal x231=((gconst63)*(gconst77));
IkReal x232=((gconst74)*(gconst75));
IkReal x233=((gconst66)*(gconst82));
IkReal x234=((gconst71)*(gconst87));
IkReal x235=((IkReal(1.00000000000000))*(gconst90));
IkReal x236=((gconst71)*(gconst77));
IkReal x237=((gconst77)*(gconst80));
IkReal x238=((gconst76)*(gconst82));
IkReal x239=((gconst77)*(gconst91));
IkReal x240=((gconst70)*(gconst83));
IkReal x241=((gconst62)*(gconst76));
IkReal x242=((gconst81)*(gconst90));
IkReal x243=((gconst77)*(gconst92));
IkReal x244=((gconst74)*(x224));
op[0]=((((x176)*(x242)))+(((x208)*(x221)))+(((IkReal(-1.00000000000000))*(gconst87)*(x180)*(x181)))+(((IkReal(-1.00000000000000))*(gconst81)*(x167)*(x179)))+(((x219)*(x224)))+(((IkReal(-1.00000000000000))*(gconst90)*(x167)*(x212)))+(((x179)*(x200)))+(((IkReal(-1.00000000000000))*(gconst84)*(x180)*(x226))));
op[1]=((((x176)*(x196)))+(((IkReal(-1.00000000000000))*(gconst89)*(x180)*(x207)))+(((IkReal(-1.00000000000000))*(gconst84)*(x180)*(x183)))+(((gconst79)*(gconst83)*(x224)))+(((x176)*(x177)))+(((IkReal(-1.00000000000000))*(gconst76)*(x179)*(x217)))+(((IkReal(-1.00000000000000))*(gconst73)*(x167)*(x179)))+(((gconst90)*(x244)))+(((IkReal(-1.00000000000000))*(gconst79)*(x167)*(x212)))+(((IkReal(-1.00000000000000))*(gconst75)*(x180)*(x226)))+(((IkReal(-1.00000000000000))*(gconst81)*(x167)*(x168)))+(((IkReal(-1.00000000000000))*(gconst76)*(x212)*(x235)))+(((IkReal(-1.00000000000000))*(x225)*(x226)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst87)*(x181)))+(((x179)*(x201)))+(((gconst77)*(gconst83)*(x198)))+(((x168)*(x200)))+(((IkReal(-1.00000000000000))*(gconst77)*(x180)*(x181)))+(((x221)*(x239)))+(((x183)*(x209)))+(((x205)*(x242)))+(((x179)*(x225)))+(((x208)*(x238)))+(((IkReal(-1.00000000000000))*(gconst77)*(x167)*(x203))));
op[2]=((((x176)*(x194)))+(((x218)*(x219)))+(((x201)*(x243)))+(((IkReal(-1.00000000000000))*(x183)*(x225)))+(((IkReal(-1.00000000000000))*(gconst84)*(x180)*(x182)))+(((IkReal(-1.00000000000000))*(gconst84)*(x180)*(x215)))+(((gconst76)*(gconst81)*(x183)))+(((IkReal(-1.00000000000000))*(gconst84)*(x223)*(x226)))+(((x196)*(x205)))+(((IkReal(-1.00000000000000))*(gconst67)*(x167)*(x203)))+(((IkReal(-1.00000000000000))*(gconst74)*(x181)*(x216)))+(((IkReal(-1.00000000000000))*(gconst73)*(x167)*(x168)))+(((x206)*(x219)))+(((IkReal(-1.00000000000000))*(x226)*(x232)))+(((gconst73)*(gconst79)*(x176)))+(((x173)*(x198)))+(((IkReal(-1.00000000000000))*(gconst61)*(x167)*(x179)))+(((x200)*(x237)))+(((IkReal(-1.00000000000000))*(gconst75)*(x180)*(x183)))+(((IkReal(-1.00000000000000))*(gconst83)*(x166)*(x208)))+(((x198)*(x211)))+(((x196)*(x197)))+(((x224)*(x240)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst88)*(gconst90)*(x167)))+(((x179)*(x232)))+(((gconst61)*(gconst90)*(x176)))+(((gconst74)*(gconst77)*(x198)))+(((IkReal(-1.00000000000000))*(gconst76)*(x203)*(x216)))+(((x168)*(x225)))+(((x168)*(x201)))+(((x177)*(x205)))+(((IkReal(-1.00000000000000))*(gconst70)*(x167)*(x212)))+(((IkReal(-1.00000000000000))*(gconst81)*(x167)*(x237)))+(((IkReal(-1.00000000000000))*(x173)*(x181)))+(((IkReal(-1.00000000000000))*(gconst85)*(x180)*(x234)))+(((x238)*(x239)))+(((x224)*(x230)))+(((IkReal(-1.00000000000000))*(gconst73)*(x167)*(x243)))+(((x221)*(x234)))+(((x226)*(x229)))+(((IkReal(-1.00000000000000))*(gconst78)*(x180)*(x207)))+(((x221)*(x222)))+(((IkReal(-1.00000000000000))*(gconst64)*(x180)*(x226)))+(((IkReal(-1.00000000000000))*(gconst90)*(x167)*(x169)))+(((IkReal(-1.00000000000000))*(gconst76)*(x168)*(x217)))+(((IkReal(-1.00000000000000))*(gconst76)*(x199)*(x212)))+(((gconst64)*(gconst83)*(x179)))+(((IkReal(-1.00000000000000))*(gconst66)*(x179)*(x217)))+(((gconst83)*(gconst87)*(x188)))+(((IkReal(-1.00000000000000))*(gconst81)*(x167)*(x227)))+(((x182)*(x209)))+(((IkReal(-1.00000000000000))*(x179)*(x228)))+(((x208)*(x233)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst89)*(x180)))+(((x190)*(x224)))+(((IkReal(-1.00000000000000))*(gconst84)*(gconst89)*(x190)))+(((x177)*(x197)))+(((x186)*(x208)))+(((IkReal(-1.00000000000000))*(gconst67)*(x180)*(x181)))+(((IkReal(-1.00000000000000))*(gconst66)*(x212)*(x235))));
op[3]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst80)*(gconst87)))+(((gconst75)*(gconst77)*(gconst80)*(gconst83)))+(((gconst71)*(gconst77)*(gconst82)*(gconst86)))+(((gconst65)*(gconst74)*(gconst88)*(gconst90)))+(((gconst67)*(gconst80)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst89)*(gconst90)))+(((gconst61)*(gconst78)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst79)*(gconst82)*(gconst86)))+(((gconst61)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst77)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst81)*(gconst92)))+(((gconst63)*(gconst79)*(gconst85)*(gconst88)))+(((gconst62)*(gconst76)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst83)*(gconst90)))+(((gconst64)*(gconst74)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst80)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst79)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst88)*(gconst90)))+(((gconst71)*(gconst76)*(gconst82)*(gconst87)))+(((gconst70)*(gconst74)*(gconst85)*(gconst88)))+(((gconst74)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst82)*(gconst90)))+(((gconst74)*(gconst75)*(gconst80)*(gconst87)))+(((gconst62)*(gconst77)*(gconst86)*(gconst91)))+(((gconst69)*(gconst76)*(gconst81)*(gconst90)))+(((gconst66)*(gconst79)*(gconst81)*(gconst89)))+(((gconst69)*(gconst73)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst80)*(gconst86)*(gconst87)))+(((gconst66)*(gconst77)*(gconst82)*(gconst91)))+(((gconst70)*(gconst73)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst79)*(gconst86)*(gconst88)))+(((gconst74)*(gconst77)*(gconst80)*(gconst84)))+(((gconst70)*(gconst78)*(gconst81)*(gconst86)))+(((gconst63)*(gconst75)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst79)*(gconst83)*(gconst89)))+(((gconst68)*(gconst74)*(gconst85)*(gconst90)))+(((gconst64)*(gconst77)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst92)))+(((gconst65)*(gconst79)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst82)*(gconst88)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst86)*(gconst87)))+(((gconst72)*(gconst77)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst86)*(gconst92)))+(((gconst69)*(gconst79)*(gconst81)*(gconst86)))+(((gconst67)*(gconst76)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst83)*(gconst90)))+(((gconst61)*(gconst76)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst84)*(gconst89)))+(((gconst72)*(gconst75)*(gconst83)*(gconst87)))+(((gconst63)*(gconst80)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst78)*(gconst79)*(gconst83)))+(((gconst63)*(gconst77)*(gconst84)*(gconst92)))+(((gconst73)*(gconst78)*(gconst79)*(gconst86)))+(((gconst70)*(gconst76)*(gconst81)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst78)*(gconst83)*(gconst84)))+(((gconst66)*(gconst73)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst83)*(gconst89)))+(((gconst68)*(gconst79)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst74)*(gconst85)*(gconst91)))+(((gconst73)*(gconst76)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst77)*(gconst83)*(gconst91)))+(((gconst67)*(gconst74)*(gconst84)*(gconst92)))+(((gconst72)*(gconst74)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst84)*(gconst89)))+(((gconst76)*(gconst78)*(gconst79)*(gconst81)))+(((gconst66)*(gconst78)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst87)*(gconst92)))+(((gconst73)*(gconst76)*(gconst79)*(gconst89)))+(((gconst67)*(gconst75)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst80)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst78)*(gconst79)*(gconst84))));
op[4]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst83)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst87)*(gconst92)))+(((gconst64)*(gconst72)*(gconst83)*(gconst87)))+(((gconst64)*(gconst74)*(gconst80)*(gconst87)))+(((gconst63)*(gconst70)*(gconst85)*(gconst88)))+(((gconst68)*(gconst70)*(gconst83)*(gconst85)))+(((gconst67)*(gconst74)*(gconst80)*(gconst84)))+(((gconst73)*(gconst76)*(gconst78)*(gconst79)))+(((gconst64)*(gconst67)*(gconst83)*(gconst92)))+(((gconst67)*(gconst72)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst70)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst79)*(gconst83)))+(((gconst64)*(gconst74)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst90)))+(((gconst71)*(gconst76)*(gconst77)*(gconst82)))+(((gconst70)*(gconst73)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst84)*(gconst89)))+(((gconst61)*(gconst66)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst92)))+(((gconst62)*(gconst76)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst77)*(gconst81)))+(((gconst61)*(gconst69)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst80)))+(((gconst66)*(gconst70)*(gconst81)*(gconst89)))+(((gconst61)*(gconst70)*(gconst86)*(gconst89)))+(((gconst63)*(gconst64)*(gconst87)*(gconst92)))+(((gconst61)*(gconst78)*(gconst79)*(gconst86)))+(((gconst68)*(gconst74)*(gconst79)*(gconst85)))+(((gconst67)*(gconst75)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst86)*(gconst87)))+(((gconst66)*(gconst69)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst79)*(gconst88)))+(((gconst62)*(gconst66)*(gconst87)*(gconst91)))+(((gconst70)*(gconst76)*(gconst78)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst79)*(gconst81)))+(((gconst66)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst82)*(gconst86)))+(((gconst65)*(gconst70)*(gconst83)*(gconst88)))+(((gconst67)*(gconst71)*(gconst82)*(gconst86)))+(((gconst63)*(gconst65)*(gconst88)*(gconst90)))+(((gconst63)*(gconst75)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst87)*(gconst91)))+(((gconst63)*(gconst68)*(gconst85)*(gconst90)))+(((gconst70)*(gconst73)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst87)))+(((gconst61)*(gconst76)*(gconst78)*(gconst90)))+(((gconst72)*(gconst74)*(gconst75)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst84)*(gconst90)))+(((gconst62)*(gconst71)*(gconst86)*(gconst87)))+(((gconst69)*(gconst73)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst80)*(gconst81)))+(((gconst72)*(gconst75)*(gconst77)*(gconst83)))+(((gconst69)*(gconst70)*(gconst81)*(gconst86)))+(((gconst65)*(gconst68)*(gconst83)*(gconst90)))+(((gconst63)*(gconst67)*(gconst84)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst92)))+(((gconst74)*(gconst75)*(gconst77)*(gconst80)))+(((gconst63)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst80)*(gconst86)))+(((gconst66)*(gconst73)*(gconst78)*(gconst90)))+(((gconst66)*(gconst73)*(gconst79)*(gconst89)))+(((gconst61)*(gconst76)*(gconst79)*(gconst89)))+(((gconst72)*(gconst74)*(gconst77)*(gconst84)))+(((gconst66)*(gconst78)*(gconst79)*(gconst81)))+(((gconst65)*(gconst74)*(gconst79)*(gconst88)))+(((gconst64)*(gconst77)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst88)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst79)*(gconst89)))+(((gconst66)*(gconst67)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst79)*(gconst89)))+(((gconst62)*(gconst67)*(gconst86)*(gconst91)))+(((gconst69)*(gconst73)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst77)*(gconst86)))+(((gconst63)*(gconst77)*(gconst80)*(gconst84)))+(((gconst63)*(gconst72)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst83)*(gconst87))));
op[5]=((((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst79)*(gconst88)))+(((gconst64)*(gconst72)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst91)))+(((gconst61)*(gconst66)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst66)*(gconst70)*(gconst78)*(gconst81)))+(((gconst66)*(gconst73)*(gconst78)*(gconst79)))+(((gconst63)*(gconst75)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst79)))+(((gconst63)*(gconst72)*(gconst75)*(gconst87)))+(((gconst63)*(gconst67)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst92)))+(((gconst69)*(gconst70)*(gconst76)*(gconst81)))+(((gconst64)*(gconst74)*(gconst77)*(gconst80)))+(((gconst61)*(gconst70)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((gconst68)*(gconst70)*(gconst74)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst79)*(gconst84)))+(((gconst63)*(gconst64)*(gconst77)*(gconst92)))+(((gconst63)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst80)*(gconst86)))+(((gconst63)*(gconst72)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst76)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst80)))+(((gconst69)*(gconst70)*(gconst73)*(gconst86)))+(((gconst65)*(gconst68)*(gconst79)*(gconst83)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst65)*(gconst68)*(gconst74)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst76)*(gconst82)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((gconst66)*(gconst70)*(gconst73)*(gconst89)))+(((gconst72)*(gconst74)*(gconst75)*(gconst77)))+(((gconst69)*(gconst73)*(gconst76)*(gconst79)))+(((gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst62)*(gconst66)*(gconst77)*(gconst91)))+(((gconst62)*(gconst67)*(gconst76)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst74)*(gconst91)))+(((gconst65)*(gconst70)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((gconst61)*(gconst70)*(gconst78)*(gconst86)))+(((gconst61)*(gconst69)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst75)*(gconst89)))+(((gconst64)*(gconst67)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst79)*(gconst86)))+(((gconst61)*(gconst66)*(gconst79)*(gconst89)))+(((gconst67)*(gconst72)*(gconst74)*(gconst84)))+(((gconst63)*(gconst68)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst61)*(gconst69)*(gconst79)*(gconst86)))+(((gconst61)*(gconst76)*(gconst78)*(gconst79)))+(((gconst67)*(gconst72)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst80)))+(((gconst66)*(gconst69)*(gconst79)*(gconst81)))+(((gconst63)*(gconst67)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst74)*(gconst85)))+(((gconst70)*(gconst73)*(gconst76)*(gconst78)))+(((gconst63)*(gconst65)*(gconst79)*(gconst88)))+(((gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst64)*(gconst67)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst87)))+(((gconst64)*(gconst72)*(gconst74)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst90)))+(((gconst66)*(gconst69)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst74)*(gconst87))));
op[6]=((((IkReal(-1.00000000000000))*(gconst91)*(x165)*(x166)))+(((IkReal(-1.00000000000000))*(gconst75)*(x191)*(x199)))+(((IkReal(-1.00000000000000))*(gconst74)*(x166)*(x236)))+(((x186)*(x187)))+(((x185)*(x222)))+(((IkReal(-1.00000000000000))*(gconst92)*(x174)*(x192)))+(((x187)*(x233)))+(((IkReal(-1.00000000000000))*(gconst80)*(x172)*(x192)))+(((x171)*(x202)))+(((IkReal(-1.00000000000000))*(x171)*(x193)))+(((IkReal(-1.00000000000000))*(x182)*(x204)))+(((IkReal(-1.00000000000000))*(gconst72)*(x192)*(x228)))+(((IkReal(-1.00000000000000))*(gconst72)*(x192)*(x229)))+(((IkReal(-1.00000000000000))*(gconst68)*(x185)*(x235)))+(((x170)*(x231)))+(((x193)*(x210)))+(((IkReal(-1.00000000000000))*(x174)*(x227)))+(((gconst75)*(gconst80)*(x165)))+(((x195)*(x206)))+(((IkReal(-1.00000000000000))*(gconst76)*(x199)*(x220)))+(((IkReal(-1.00000000000000))*(gconst78)*(x199)*(x204)))+(((x236)*(x241)))+(((x174)*(x182)))+(((x175)*(x193)))+(((x172)*(x175)))+(((x173)*(x178)))+(((IkReal(-1.00000000000000))*(gconst69)*(x184)*(x232)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(x190)))+(((x213)*(x228)))+(((x189)*(x190)))+(((x189)*(x230)))+(((gconst64)*(gconst92)*(x165)))+(((x185)*(x234)))+(((IkReal(-1.00000000000000))*(gconst83)*(x166)*(x187)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst85)*(x165)))+(((IkReal(-1.00000000000000))*(gconst64)*(x180)*(x213)))+(((x172)*(x210)))+(((IkReal(-1.00000000000000))*(gconst75)*(x175)*(x223)))+(((x204)*(x237)))+(((x170)*(x214)))+(((x165)*(x188)))+(((gconst66)*(gconst69)*(x194)))+(((gconst61)*(gconst86)*(x213)))+(((IkReal(-1.00000000000000))*(gconst66)*(x169)*(x184)))+(((x189)*(x240)))+(((gconst67)*(gconst80)*(x202)))+(((x174)*(x215)))+(((IkReal(-1.00000000000000))*(gconst84)*(x184)*(x191)))+(((IkReal(-1.00000000000000))*(gconst71)*(x166)*(x173)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(x167)))+(((IkReal(-1.00000000000000))*(gconst80)*(x174)*(x216)))+(((IkReal(-1.00000000000000))*(gconst70)*(x167)*(x220)))+(((gconst78)*(gconst79)*(x174)))+(((IkReal(-1.00000000000000))*(gconst80)*(x192)*(x193)))+(((x178)*(x211)))+(((IkReal(-1.00000000000000))*(gconst89)*(x184)*(x204)))+(((IkReal(-1.00000000000000))*(gconst88)*(x184)*(x185)))+(((IkReal(-1.00000000000000))*(x171)*(x172)))+(((x195)*(x218)))+(((IkReal(-1.00000000000000))*(x175)*(x202))));
op[7]=((((gconst63)*(gconst79)*(x189)))+(((x171)*(x204)))+(((gconst70)*(gconst74)*(x189)))+(((x193)*(x213)))+(((IkReal(-1.00000000000000))*(gconst64)*(x191)*(x199)))+(((IkReal(-1.00000000000000))*(gconst69)*(x184)*(x202)))+(((IkReal(-1.00000000000000))*(gconst72)*(x192)*(x193)))+(((IkReal(-1.00000000000000))*(gconst75)*(x184)*(x191)))+(((x185)*(x236)))+(((x174)*(x175)))+(((x172)*(x213)))+(((gconst64)*(gconst80)*(x165)))+(((IkReal(-1.00000000000000))*(gconst68)*(x185)*(x199)))+(((IkReal(-1.00000000000000))*(gconst71)*(x166)*(x231)))+(((x165)*(x170)))+(((IkReal(-1.00000000000000))*(gconst76)*(x184)*(x220)))+(((IkReal(-1.00000000000000))*(gconst74)*(x166)*(x187)))+(((IkReal(-1.00000000000000))*(gconst72)*(x172)*(x192)))+(((x174)*(x210)))+(((x187)*(x241)))+(((x178)*(x214)))+(((IkReal(-1.00000000000000))*(gconst80)*(x174)*(x192)))+(((IkReal(-1.00000000000000))*(x171)*(x174)))+(((IkReal(-1.00000000000000))*(x175)*(x204))));
op[8]=((((x185)*(x187)))+(((x189)*(x195)))+(((IkReal(-1.00000000000000))*(gconst68)*(x184)*(x185)))+(((IkReal(-1.00000000000000))*(gconst72)*(x174)*(x192)))+(((IkReal(-1.00000000000000))*(gconst71)*(x165)*(x166)))+(((x165)*(x178)))+(((IkReal(-1.00000000000000))*(gconst64)*(x184)*(x191)))+(((x174)*(x213))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst109;
gconst109=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x245=((cj3)*(sj1));
IkReal x246=((r02)*(sj2));
IkReal x247=((r01)*(sj0));
IkReal x248=((IkReal(1.00000000000000))*(cj3));
IkReal x249=((cj1)*(sj2));
IkReal x250=((IkReal(1.00000000000000))*(sj3));
IkReal x251=((cj0)*(r00));
IkReal x252=((cj0)*(r01));
IkReal x253=((r00)*(sj0));
IkReal x254=((cj2)*(x251));
IkReal x255=((sj1)*(x250));
IkReal x256=((cj1)*(cj2)*(r02));
if( IKabs(((gconst109)*(((((IkReal(-1.00000000000000))*(x249)*(x250)*(x251)))+(((cj3)*(x253)))+(((IkReal(-1.00000000000000))*(x254)*(x255)))+(((IkReal(-1.00000000000000))*(x246)*(x255)))+(((IkReal(-1.00000000000000))*(x248)*(x252)))+(((sj3)*(x256)))+(((IkReal(-1.00000000000000))*(x247)*(x249)*(x250)))+(((IkReal(-1.00000000000000))*(cj2)*(x247)*(x255))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst109)*(((((x245)*(x254)))+(((IkReal(-1.00000000000000))*(x250)*(x252)))+(((IkReal(-1.00000000000000))*(x248)*(x256)))+(((sj3)*(x253)))+(((cj3)*(x247)*(x249)))+(((cj2)*(x245)*(x247)))+(((x245)*(x246)))+(((cj3)*(x249)*(x251))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst109)*(((((IkReal(-1.00000000000000))*(x249)*(x250)*(x251)))+(((cj3)*(x253)))+(((IkReal(-1.00000000000000))*(x254)*(x255)))+(((IkReal(-1.00000000000000))*(x246)*(x255)))+(((IkReal(-1.00000000000000))*(x248)*(x252)))+(((sj3)*(x256)))+(((IkReal(-1.00000000000000))*(x247)*(x249)*(x250)))+(((IkReal(-1.00000000000000))*(cj2)*(x247)*(x255)))))), ((gconst109)*(((((x245)*(x254)))+(((IkReal(-1.00000000000000))*(x250)*(x252)))+(((IkReal(-1.00000000000000))*(x248)*(x256)))+(((sj3)*(x253)))+(((cj3)*(x247)*(x249)))+(((cj2)*(x245)*(x247)))+(((x245)*(x246)))+(((cj3)*(x249)*(x251)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x257=IKsin(j5);
IkReal x258=IKcos(j5);
IkReal x259=((sj1)*(sj2));
IkReal x260=((cj2)*(sj1));
IkReal x261=((py)*(sj0));
IkReal x262=((IkReal(1.00000000000000))*(cj0));
IkReal x263=((r01)*(sj0));
IkReal x264=((cj1)*(sj2));
IkReal x265=((cj0)*(x264));
IkReal x266=((sj3)*(x257));
IkReal x267=((IkReal(1.00000000000000))*(x258));
IkReal x268=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x269=((IkReal(0.790000000000000))*(x258));
IkReal x270=((cj3)*(x257));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x267)))+(((IkReal(-1.00000000000000))*(x270)))+(((IkReal(-1.00000000000000))*(r01)*(x262))));
evalcond[1]=((((sj3)*(x269)))+(((IkReal(0.790000000000000))*(x270)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x262))));
evalcond[2]=((((x260)*(x263)))+(((x263)*(x264)))+(((IkReal(-1.00000000000000))*(cj3)*(x267)))+(((cj0)*(r00)*(x260)))+(x266)+(((IkReal(-1.00000000000000))*(r02)*(x268)))+(((r00)*(x265)))+(((r02)*(x259))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x260)*(x261)))+(((pz)*(x259)))+(((IkReal(-0.790000000000000))*(x266)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj0)*(px)*(x260)))+(((IkReal(-0.0750000000000000))*(x264)))+(((IkReal(-0.0750000000000000))*(x260)))+(((x261)*(x264)))+(((cj3)*(x269)))+(((IkReal(-1.00000000000000))*(pz)*(x268)))+(((px)*(x265))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x271=((cj1)*(cj2));
IkReal x272=((IkReal(1.00000000000000))*(cj0));
IkReal x273=((sj1)*(sj2));
IkReal x274=((IkReal(1.00000000000000))*(sj0));
IkReal x275=((sj0)*(x273));
IkReal x276=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x277=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x273)))+(((IkReal(-1.00000000000000))*(r01)*(x271)*(x274)))+(((IkReal(-1.00000000000000))*(r02)*(x277)))+(((IkReal(-1.00000000000000))*(r02)*(x276)))+(((r01)*(x275)))+(((IkReal(-1.00000000000000))*(r00)*(x271)*(x272))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x277)))+(((IkReal(-1.00000000000000))*(pz)*(x276)))+(((IkReal(-1.00000000000000))*(px)*(x271)*(x272)))+(((cj0)*(px)*(x273)))+(((IkReal(-1.00000000000000))*(py)*(x271)*(x274)))+(((IkReal(0.0750000000000000))*(x271)))+(((IkReal(-0.0750000000000000))*(x273)))+(((py)*(x275))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst111;
gconst111=IkReal(-2.00000000000000);
IkReal gconst112;
gconst112=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst113;
gconst113=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst114;
gconst114=IkReal(1.58000000000000);
IkReal gconst115;
gconst115=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst116;
gconst116=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst119;
gconst119=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst120;
gconst120=IkReal(1.58000000000000);
IkReal gconst121;
gconst121=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst122;
gconst122=IkReal(2.00000000000000);
IkReal gconst123;
gconst123=IkReal(-2.00000000000000);
IkReal gconst124;
gconst124=IkReal(-1.58000000000000);
IkReal gconst125;
gconst125=IkReal(1.58000000000000);
IkReal gconst126;
gconst126=IkReal(2.00000000000000);
IkReal gconst127;
gconst127=IkReal(-2.00000000000000);
IkReal gconst128;
gconst128=IkReal(-1.58000000000000);
IkReal gconst129;
gconst129=IkReal(1.58000000000000);
IkReal gconst130;
gconst130=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst131;
gconst131=IkReal(2.00000000000000);
IkReal gconst132;
gconst132=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst133;
gconst133=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst134;
gconst134=IkReal(-1.58000000000000);
IkReal gconst135;
gconst135=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst136;
gconst136=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst137;
gconst137=IkReal(2.00000000000000);
IkReal gconst138;
gconst138=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst139;
gconst139=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst140;
gconst140=IkReal(-1.58000000000000);
IkReal gconst141;
gconst141=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal x278=((gconst132)*(gconst134));
IkReal x279=((gconst137)*(gconst139));
IkReal x280=((IkReal(1.00000000000000))*(gconst136));
IkReal x281=((gconst130)*(gconst135));
IkReal x282=((gconst132)*(gconst133));
IkReal x283=((gconst138)*(gconst139));
IkReal x284=((gconst131)*(gconst135));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x282)*(x283)))+(((IkReal(-1.00000000000000))*(gconst140)*(x278)*(x280)))+(((gconst136)*(gconst140)*(x284)))+(((gconst136)*(gconst141)*(x282)))+(((IkReal(-1.00000000000000))*(x279)*(x284)))+(((IkReal(-1.00000000000000))*(gconst141)*(x280)*(x281)))+(((x278)*(x279)))+(((x281)*(x283))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x285=((gconst118)*(gconst119));
IkReal x286=((gconst110)*(gconst135));
IkReal x287=((gconst114)*(gconst117));
IkReal x288=((gconst123)*(gconst128));
IkReal x289=((gconst136)*(gconst141));
IkReal x290=((gconst113)*(gconst132));
IkReal x291=((IkReal(1.00000000000000))*(gconst124));
IkReal x292=((IkReal(1.00000000000000))*(gconst135));
IkReal x293=((gconst138)*(gconst139));
IkReal x294=((gconst131)*(gconst139));
IkReal x295=((gconst112)*(gconst116));
IkReal x296=((gconst124)*(gconst129));
IkReal x297=((gconst115)*(gconst122));
IkReal x298=((IkReal(1.00000000000000))*(gconst140));
IkReal x299=((gconst112)*(gconst139));
IkReal x300=((gconst121)*(gconst124));
IkReal x301=((gconst129)*(gconst136));
IkReal x302=((gconst132)*(gconst136));
IkReal x303=((gconst120)*(gconst126));
IkReal x304=((IkReal(1.00000000000000))*(gconst123));
IkReal x305=((gconst131)*(gconst137));
IkReal x306=((gconst116)*(gconst121));
IkReal x307=((IkReal(1.00000000000000))*(gconst116));
IkReal x308=((gconst131)*(gconst135));
IkReal x309=((gconst126)*(gconst140));
IkReal x310=((gconst111)*(gconst115));
IkReal x311=((IkReal(1.00000000000000))*(gconst125));
IkReal x312=((gconst116)*(gconst120));
IkReal x313=((gconst111)*(gconst135));
IkReal x314=((gconst113)*(gconst121));
IkReal x315=((gconst130)*(gconst138));
IkReal x316=((gconst126)*(gconst141));
IkReal x317=((IkReal(1.00000000000000))*(gconst119));
IkReal x318=((gconst132)*(gconst133));
IkReal x319=((gconst112)*(gconst133));
IkReal x320=((gconst110)*(gconst115));
IkReal x321=((gconst127)*(gconst139));
IkReal x322=((gconst123)*(gconst133));
IkReal x323=((IkReal(1.00000000000000))*(gconst120));
IkReal x324=((gconst132)*(gconst134));
IkReal x325=((gconst122)*(gconst125));
IkReal x326=((gconst125)*(gconst128));
IkReal x327=((gconst134)*(gconst137));
IkReal x328=((gconst116)*(gconst123));
IkReal x329=((gconst120)*(gconst136));
IkReal x330=((IkReal(1.00000000000000))*(gconst118));
IkReal x331=((gconst112)*(gconst113));
IkReal x332=((gconst111)*(gconst117));
IkReal x333=((IkReal(1.00000000000000))*(gconst121));
IkReal x334=((gconst124)*(gconst132));
IkReal x335=((gconst137)*(gconst139));
IkReal x336=((gconst114)*(gconst132));
IkReal x337=((gconst112)*(gconst119));
IkReal x338=((IkReal(1.00000000000000))*(gconst128));
IkReal x339=((gconst123)*(gconst134));
IkReal x340=((gconst111)*(gconst125));
IkReal x341=((gconst130)*(gconst135));
IkReal x342=((IkReal(1.00000000000000))*(gconst115));
IkReal x343=((gconst118)*(gconst139));
IkReal x344=((gconst116)*(gconst140));
IkReal x345=((gconst117)*(gconst139));
IkReal x346=((gconst112)*(gconst128));
IkReal x347=((gconst126)*(gconst129));
IkReal x348=((gconst127)*(gconst128));
IkReal x349=((gconst116)*(gconst141));
IkReal x350=((gconst115)*(gconst131));
IkReal x351=((gconst125)*(gconst131));
IkReal x352=((gconst136)*(gconst140));
IkReal x353=((gconst119)*(gconst127));
IkReal x354=((gconst113)*(gconst129));
IkReal x355=((gconst118)*(gconst128));
IkReal x356=((gconst128)*(gconst135));
IkReal x357=((gconst112)*(gconst136));
IkReal x358=((gconst110)*(gconst125));
IkReal x359=((gconst115)*(gconst130));
IkReal x360=((gconst121)*(gconst133));
IkReal x361=((gconst125)*(gconst130));
IkReal x362=((gconst122)*(gconst135));
IkReal x363=((gconst112)*(x314));
IkReal x364=((gconst126)*(gconst130)*(x292));
op[0]=((((x289)*(x318)))+(((x308)*(x352)))+(((IkReal(-1.00000000000000))*(gconst137)*(x292)*(x294)))+(((x324)*(x335)))+(((IkReal(-1.00000000000000))*(gconst134)*(x298)*(x302)))+(((IkReal(-1.00000000000000))*(x293)*(x318)))+(((IkReal(-1.00000000000000))*(gconst130)*(x289)*(x292)))+(((x293)*(x341))));
op[1]=((((IkReal(-1.00000000000000))*(gconst130)*(x292)*(x301)))+(((IkReal(-1.00000000000000))*(gconst122)*(x289)*(x292)))+(((gconst123)*(gconst139)*(x327)))+(((IkReal(-1.00000000000000))*(gconst133)*(x293)*(x304)))+(((x289)*(x322)))+(((x301)*(x318)))+(((IkReal(-1.00000000000000))*(gconst137)*(x294)*(x311)))+(((IkReal(-1.00000000000000))*(gconst138)*(x318)*(x338)))+(((IkReal(-1.00000000000000))*(gconst136)*(x298)*(x339)))+(((x351)*(x352)))+(((gconst128)*(gconst137)*(x324)))+(((x289)*(x334)))+(((x308)*(x309)))+(((x315)*(x356)))+(((IkReal(-1.00000000000000))*(gconst132)*(x291)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x289)*(x311)))+(((IkReal(-1.00000000000000))*(gconst128)*(x292)*(x305)))+(((IkReal(-1.00000000000000))*(gconst126)*(x298)*(x324)))+(((x293)*(x362)))+(((x293)*(x361)))+(((x321)*(x341)))+(((IkReal(-1.00000000000000))*(x318)*(x321)))+(((x316)*(x318)))+(((IkReal(-1.00000000000000))*(gconst130)*(x292)*(x316))));
op[2]=((((IkReal(-1.00000000000000))*(gconst133)*(x304)*(x321)))+(((IkReal(-1.00000000000000))*(gconst122)*(x289)*(x311)))+(((x289)*(x290)))+(((x301)*(x322)))+(((x289)*(x319)))+(((IkReal(-1.00000000000000))*(gconst134)*(x302)*(x323)))+(((x318)*(x347)))+(((x318)*(x349)))+(((x335)*(x336)))+(((gconst119)*(gconst137)*(x324)))+(((IkReal(-1.00000000000000))*(gconst111)*(x292)*(x335)))+(((IkReal(-1.00000000000000))*(gconst119)*(x292)*(x305)))+(((IkReal(-1.00000000000000))*(gconst114)*(x298)*(x302)))+(((IkReal(-1.00000000000000))*(gconst128)*(gconst132)*(gconst138)*(x291)))+(((IkReal(-1.00000000000000))*(gconst117)*(x292)*(x294)))+(((x308)*(x329)))+(((gconst123)*(gconst124)*(x289)))+(((x324)*(x345)))+(((IkReal(-1.00000000000000))*(x290)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x301)*(x311)))+(((x302)*(x360)))+(((x313)*(x352)))+(((gconst122)*(gconst138)*(x356)))+(((x288)*(x327)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst130)*(gconst136)*(x292)))+(((IkReal(-1.00000000000000))*(gconst130)*(x311)*(x316)))+(((x321)*(x361)))+(((x321)*(x362)))+(((x316)*(x334)))+(((IkReal(-1.00000000000000))*(gconst132)*(x291)*(x321)))+(((IkReal(-1.00000000000000))*(gconst128)*(x305)*(x311)))+(((x286)*(x293)))+(((IkReal(-1.00000000000000))*(gconst139)*(x318)*(x330)))+(((IkReal(-1.00000000000000))*(gconst116)*(x298)*(x324)))+(((IkReal(-1.00000000000000))*(gconst122)*(x292)*(x301)))+(((IkReal(-1.00000000000000))*(x293)*(x319)))+(((IkReal(-1.00000000000000))*(gconst126)*(x298)*(x339)))+(((IkReal(-1.00000000000000))*(gconst130)*(x289)*(x342)))+(((IkReal(-1.00000000000000))*(gconst137)*(x294)*(x342)))+(((x315)*(x326)))+(((gconst119)*(gconst135)*(x315)))+(((x296)*(x302)))+(((x341)*(x348)))+(((x341)*(x343)))+(((IkReal(-1.00000000000000))*(gconst122)*(x292)*(x316)))+(((x350)*(x352)))+(((IkReal(-1.00000000000000))*(gconst138)*(x317)*(x318)))+(((x293)*(x325)))+(((x316)*(x322)))+(((x299)*(x327)))+(((IkReal(-1.00000000000000))*(x286)*(x289)))+(((x309)*(x351)))+(((x293)*(x359)))+(((IkReal(-1.00000000000000))*(gconst133)*(gconst138)*(x288)))+(((x308)*(x344)))+(((IkReal(-1.00000000000000))*(gconst123)*(x291)*(x293)))+(((IkReal(-1.00000000000000))*(gconst134)*(x298)*(x357)))+(((IkReal(-1.00000000000000))*(gconst127)*(x318)*(x338)))+(((IkReal(-1.00000000000000))*(gconst130)*(x292)*(x347)))+(((IkReal(-1.00000000000000))*(gconst130)*(x292)*(x349))));
op[3]=((((gconst121)*(gconst124)*(gconst132)*(gconst136)))+(((gconst115)*(gconst122)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst126)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst128)*(gconst131)*(gconst137)))+(((gconst122)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst129)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst126)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst134)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst132)*(gconst139)))+(((gconst120)*(gconst125)*(gconst131)*(gconst136)))+(((gconst113)*(gconst123)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst126)*(gconst130)*(gconst135)))+(((gconst117)*(gconst128)*(gconst132)*(gconst134)))+(((gconst112)*(gconst128)*(gconst134)*(gconst137)))+(((gconst119)*(gconst122)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst127)*(gconst128)*(gconst133)))+(((gconst119)*(gconst125)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst128)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst128)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst127)*(gconst132)*(gconst133)))+(((gconst114)*(gconst128)*(gconst132)*(gconst137)))+(((gconst122)*(gconst125)*(gconst128)*(gconst138)))+(((gconst116)*(gconst123)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst123)*(gconst134)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst138)*(gconst139)))+(((gconst115)*(gconst128)*(gconst130)*(gconst138)))+(((gconst112)*(gconst126)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst129)*(gconst135)*(gconst136)))+(((gconst123)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst129)*(gconst136)))+(((gconst116)*(gconst125)*(gconst131)*(gconst140)))+(((gconst119)*(gconst127)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst128)*(gconst132)*(gconst133)))+(((gconst124)*(gconst126)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst126)*(gconst129)*(gconst135)))+(((gconst115)*(gconst126)*(gconst131)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst132)*(gconst138)))+(((gconst118)*(gconst128)*(gconst130)*(gconst135)))+(((gconst121)*(gconst126)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst125)*(gconst131)*(gconst137)))+(((gconst121)*(gconst123)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst128)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst141)))+(((gconst119)*(gconst123)*(gconst134)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst130)*(gconst141)))+(((gconst120)*(gconst126)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst130)*(gconst141)))+(((gconst112)*(gconst124)*(gconst136)*(gconst141)))+(((gconst116)*(gconst129)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst124)*(gconst127)*(gconst128)*(gconst132)))+(((gconst110)*(gconst127)*(gconst135)*(gconst139)))+(((gconst110)*(gconst125)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst131)*(gconst139)))+(((gconst113)*(gconst126)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst133)*(gconst139)))+(((gconst115)*(gconst127)*(gconst130)*(gconst139)))+(((gconst117)*(gconst123)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst126)*(gconst134)*(gconst140)))+(((gconst122)*(gconst127)*(gconst128)*(gconst135)))+(((gconst112)*(gconst129)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst136)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst132)*(gconst139)))+(((gconst111)*(gconst125)*(gconst136)*(gconst140)))+(((gconst118)*(gconst125)*(gconst130)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst135)*(gconst136)))+(((gconst113)*(gconst129)*(gconst132)*(gconst136)))+(((gconst118)*(gconst122)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst136)*(gconst141)))+(((gconst123)*(gconst124)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst125)*(gconst126)*(gconst129)*(gconst130)))+(((gconst125)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst129)*(gconst130)*(gconst136)))+(((gconst116)*(gconst124)*(gconst132)*(gconst141)))+(((gconst111)*(gconst126)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst128)*(gconst131)*(gconst135)))+(((gconst110)*(gconst128)*(gconst135)*(gconst138)))+(((gconst123)*(gconst124)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst130)*(gconst136))));
op[4]=((((gconst117)*(gconst123)*(gconst128)*(gconst134)))+(((gconst112)*(gconst116)*(gconst133)*(gconst141)))+(((gconst119)*(gconst122)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst139)))+(((gconst111)*(gconst120)*(gconst135)*(gconst136)))+(((gconst116)*(gconst123)*(gconst124)*(gconst141)))+(((gconst116)*(gconst124)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst128)*(gconst138)))+(((gconst113)*(gconst121)*(gconst132)*(gconst136)))+(((gconst115)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst130)*(gconst135)))+(((gconst111)*(gconst125)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst126)*(gconst140)))+(((gconst123)*(gconst124)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst141)))+(((gconst110)*(gconst125)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst126)*(gconst134)))+(((gconst115)*(gconst116)*(gconst131)*(gconst140)))+(((gconst111)*(gconst116)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst141)))+(((gconst112)*(gconst117)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst129)*(gconst136)))+(((gconst113)*(gconst126)*(gconst129)*(gconst132)))+(((gconst120)*(gconst125)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst135)*(gconst141)))+(((gconst118)*(gconst122)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst128)*(gconst131)))+(((gconst110)*(gconst119)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst132)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst129)*(gconst135)))+(((gconst111)*(gconst115)*(gconst136)*(gconst140)))+(((gconst119)*(gconst125)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst134)*(gconst136)))+(((gconst112)*(gconst124)*(gconst129)*(gconst136)))+(((gconst112)*(gconst114)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst127)*(gconst132)))+(((gconst115)*(gconst122)*(gconst127)*(gconst139)))+(((gconst118)*(gconst119)*(gconst130)*(gconst135)))+(((gconst110)*(gconst118)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst126)*(gconst130)))+(((gconst121)*(gconst123)*(gconst126)*(gconst133)))+(((gconst116)*(gconst123)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst133)*(gconst138)))+(((gconst113)*(gconst123)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst132)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst129)*(gconst135)))+(((gconst110)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst135)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst116)*(gconst134)*(gconst140)))+(((gconst112)*(gconst119)*(gconst134)*(gconst137)))+(((gconst113)*(gconst123)*(gconst126)*(gconst141)))+(((gconst112)*(gconst121)*(gconst133)*(gconst136)))+(((gconst114)*(gconst119)*(gconst132)*(gconst137)))+(((gconst115)*(gconst118)*(gconst130)*(gconst139)))+(((gconst117)*(gconst119)*(gconst132)*(gconst134)))+(((gconst110)*(gconst115)*(gconst138)*(gconst139)))+(((gconst116)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst129)))+(((gconst121)*(gconst123)*(gconst124)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst131)*(gconst139)))+(((gconst112)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst130)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst128)*(gconst138)))+(((gconst112)*(gconst113)*(gconst136)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst138)))+(((gconst110)*(gconst127)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst135)*(gconst139)))+(((gconst121)*(gconst124)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst128)))+(((gconst118)*(gconst125)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst119)*(gconst131)*(gconst137)))+(((gconst115)*(gconst122)*(gconst128)*(gconst138)))+(((gconst114)*(gconst117)*(gconst132)*(gconst139)))+(((gconst116)*(gconst121)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst127)*(gconst133)))+(((gconst113)*(gconst116)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst137)*(gconst139)))+(((gconst112)*(gconst124)*(gconst126)*(gconst141)))+(((gconst115)*(gconst120)*(gconst131)*(gconst136)))+(((gconst115)*(gconst119)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst136)*(gconst140)))+(((gconst122)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst126)*(gconst135))));
op[5]=((((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst125)*(gconst137)))+(((gconst112)*(gconst121)*(gconst124)*(gconst136)))+(((gconst117)*(gconst119)*(gconst123)*(gconst134)))+(((gconst112)*(gconst124)*(gconst126)*(gconst129)))+(((gconst115)*(gconst119)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst122)*(gconst136)))+(((gconst110)*(gconst118)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst141)))+(((gconst110)*(gconst118)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((gconst112)*(gconst113)*(gconst129)*(gconst136)))+(((gconst114)*(gconst117)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst132)))+(((gconst112)*(gconst117)*(gconst128)*(gconst134)))+(((gconst115)*(gconst118)*(gconst122)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst126)))+(((gconst114)*(gconst119)*(gconst123)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst123)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst126)*(gconst141)))+(((gconst118)*(gconst119)*(gconst122)*(gconst135)))+(((gconst110)*(gconst115)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst122)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst129)))+(((gconst115)*(gconst118)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst125)*(gconst128)))+(((gconst112)*(gconst116)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst122)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst125)*(gconst131)))+(((gconst113)*(gconst123)*(gconst126)*(gconst129)))+(((gconst110)*(gconst119)*(gconst125)*(gconst138)))+(((gconst116)*(gconst121)*(gconst124)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst123)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst123)*(gconst134)))+(((gconst116)*(gconst121)*(gconst123)*(gconst133)))+(((gconst110)*(gconst115)*(gconst128)*(gconst138)))+(((gconst113)*(gconst116)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst128)*(gconst135)))+(((gconst115)*(gconst122)*(gconst127)*(gconst128)))+(((gconst110)*(gconst119)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst128)*(gconst138)))+(((gconst112)*(gconst114)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst128)*(gconst133)))+(((gconst110)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst129)))+(((gconst113)*(gconst121)*(gconst123)*(gconst136)))+(((gconst113)*(gconst116)*(gconst123)*(gconst141)))+(((gconst111)*(gconst115)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst129)*(gconst130)))+(((gconst115)*(gconst119)*(gconst122)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst124)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst128)*(gconst137)))+(((gconst112)*(gconst113)*(gconst126)*(gconst141)))+(((gconst114)*(gconst117)*(gconst123)*(gconst139)))+(((gconst118)*(gconst119)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst127)))+(((gconst121)*(gconst123)*(gconst124)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst124)*(gconst132)))+(((gconst113)*(gconst121)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((gconst112)*(gconst116)*(gconst124)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst127)))+(((gconst116)*(gconst123)*(gconst124)*(gconst129)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((gconst112)*(gconst121)*(gconst126)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst128)))+(((gconst111)*(gconst116)*(gconst125)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128))));
op[6]=((((IkReal(-1.00000000000000))*(gconst114)*(x323)*(x357)))+(((gconst114)*(gconst137)*(x337)))+(((IkReal(-1.00000000000000))*(gconst114)*(x295)*(x298)))+(((x328)*(x354)))+(((IkReal(-1.00000000000000))*(x286)*(x306)))+(((IkReal(-1.00000000000000))*(gconst127)*(x331)*(x338)))+(((x285)*(x286)))+(((IkReal(-1.00000000000000))*(x310)*(x345)))+(((gconst119)*(gconst138)*(x320)))+(((x290)*(x306)))+(((IkReal(-1.00000000000000))*(gconst120)*(x307)*(x336)))+(((x312)*(x313)))+(((IkReal(-1.00000000000000))*(gconst128)*(x311)*(x332)))+(((IkReal(-1.00000000000000))*(gconst126)*(x297)*(x333)))+(((x310)*(x329)))+(((gconst113)*(gconst141)*(x295)))+(((IkReal(-1.00000000000000))*(gconst119)*(x292)*(x332)))+(((gconst119)*(gconst132)*(x287)))+(((gconst123)*(gconst126)*(x314)))+(((IkReal(-1.00000000000000))*(x320)*(x347)))+(((x353)*(x358)))+(((IkReal(-1.00000000000000))*(gconst122)*(x306)*(x311)))+(((x287)*(x299)))+(((x312)*(x350)))+(((IkReal(-1.00000000000000))*(gconst117)*(x317)*(x350)))+(((x331)*(x347)))+(((IkReal(-1.00000000000000))*(gconst129)*(x297)*(x307)))+(((IkReal(-1.00000000000000))*(gconst130)*(x306)*(x342)))+(((IkReal(-1.00000000000000))*(gconst129)*(x307)*(x358)))+(((x297)*(x353)))+(((x297)*(x355)))+(((IkReal(-1.00000000000000))*(x285)*(x319)))+(((x285)*(x359)))+(((gconst110)*(gconst118)*(x326)))+(((IkReal(-1.00000000000000))*(gconst123)*(x285)*(x291)))+(((x314)*(x357)))+(((x295)*(x296)))+(((x285)*(x325)))+(((x310)*(x344)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(x311)))+(((IkReal(-1.00000000000000))*(x285)*(x290)))+(((x303)*(x340)))+(((IkReal(-1.00000000000000))*(gconst113)*(x299)*(x330)))+(((IkReal(-1.00000000000000))*(gconst137)*(x310)*(x317)))+(((IkReal(-1.00000000000000))*(gconst114)*(x303)*(x304)))+(((IkReal(-1.00000000000000))*(gconst141)*(x307)*(x320)))+(((IkReal(-1.00000000000000))*(gconst113)*(x304)*(x353)))+(((IkReal(-1.00000000000000))*(gconst136)*(x320)*(x333)))+(((x287)*(x288)))+(((gconst112)*(gconst126)*(x300)))+(((IkReal(-1.00000000000000))*(gconst134)*(x295)*(x323)))+(((IkReal(-1.00000000000000))*(gconst127)*(x291)*(x337)))+(((x300)*(x328)))+(((IkReal(-1.00000000000000))*(gconst138)*(x317)*(x331)))+(((x295)*(x360)))+(((IkReal(-1.00000000000000))*(gconst113)*(x288)*(x330)))+(((x320)*(x348)))+(((x320)*(x343)))+(((gconst117)*(gconst134)*(x337)))+(((IkReal(-1.00000000000000))*(gconst118)*(x291)*(x346))));
op[7]=((((IkReal(-1.00000000000000))*(gconst128)*(x330)*(x331)))+(((IkReal(-1.00000000000000))*(gconst117)*(x310)*(x338)))+(((x320)*(x355)))+(((x320)*(x353)))+(((IkReal(-1.00000000000000))*(gconst119)*(x311)*(x332)))+(((IkReal(-1.00000000000000))*(gconst113)*(x285)*(x304)))+(((x312)*(x340)))+(((IkReal(-1.00000000000000))*(gconst129)*(x307)*(x320)))+(((x285)*(x358)))+(((x285)*(x297)))+(((IkReal(-1.00000000000000))*(gconst126)*(x320)*(x333)))+(((x295)*(x300)))+(((IkReal(-1.00000000000000))*(gconst114)*(x304)*(x312)))+(((gconst113)*(gconst123)*(x306)))+(((gconst126)*(x363)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(x303)))+(((IkReal(-1.00000000000000))*(gconst112)*(x285)*(x291)))+(((IkReal(-1.00000000000000))*(gconst110)*(x306)*(x311)))+(((x303)*(x310)))+(((x287)*(x346)))+(((IkReal(-1.00000000000000))*(gconst127)*(x317)*(x331)))+(((gconst119)*(gconst123)*(x287)))+(((x295)*(x354)))+(((IkReal(-1.00000000000000))*(x297)*(x306))));
op[8]=((((IkReal(-1.00000000000000))*(x306)*(x320)))+(((x310)*(x312)))+(((x295)*(x314)))+(((IkReal(-1.00000000000000))*(gconst114)*(x295)*(x323)))+(((IkReal(-1.00000000000000))*(gconst117)*(x310)*(x317)))+(((x287)*(x337)))+(((x285)*(x320)))+(((IkReal(-1.00000000000000))*(x285)*(x331))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst158;
gconst158=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x365=((cj1)*(sj2));
IkReal x366=((cj2)*(sj1));
IkReal x367=((IkReal(1.00000000000000))*(sj3));
IkReal x368=((cj0)*(r00));
IkReal x369=((r01)*(sj0));
IkReal x370=((cj0)*(r01));
IkReal x371=((IkReal(1.00000000000000))*(cj3));
IkReal x372=((r00)*(sj0));
IkReal x373=((cj1)*(cj2)*(r02));
IkReal x374=((r02)*(sj1)*(sj2));
IkReal x375=((x369)*(x371));
if( IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(x365)*(x367)*(x369)))+(((IkReal(-1.00000000000000))*(x365)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(x370)*(x371)))+(((IkReal(-1.00000000000000))*(x366)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(x366)*(x367)*(x369)))+(((IkReal(-1.00000000000000))*(x367)*(x374)))+(((sj3)*(x373)))+(((cj3)*(x372))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(x365)*(x368)*(x371)))+(((IkReal(-1.00000000000000))*(x366)*(x375)))+(((IkReal(-1.00000000000000))*(x367)*(x372)))+(((sj3)*(x370)))+(((IkReal(-1.00000000000000))*(x371)*(x374)))+(((cj3)*(x373)))+(((IkReal(-1.00000000000000))*(x366)*(x368)*(x371)))+(((IkReal(-1.00000000000000))*(x365)*(x375))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst158)*(((((IkReal(-1.00000000000000))*(x365)*(x367)*(x369)))+(((IkReal(-1.00000000000000))*(x365)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(x370)*(x371)))+(((IkReal(-1.00000000000000))*(x366)*(x367)*(x368)))+(((IkReal(-1.00000000000000))*(x366)*(x367)*(x369)))+(((IkReal(-1.00000000000000))*(x367)*(x374)))+(((sj3)*(x373)))+(((cj3)*(x372)))))), ((gconst158)*(((((IkReal(-1.00000000000000))*(x365)*(x368)*(x371)))+(((IkReal(-1.00000000000000))*(x366)*(x375)))+(((IkReal(-1.00000000000000))*(x367)*(x372)))+(((sj3)*(x370)))+(((IkReal(-1.00000000000000))*(x371)*(x374)))+(((cj3)*(x373)))+(((IkReal(-1.00000000000000))*(x366)*(x368)*(x371)))+(((IkReal(-1.00000000000000))*(x365)*(x375)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x376=IKsin(j5);
IkReal x377=IKcos(j5);
IkReal x378=((sj1)*(sj2));
IkReal x379=((cj2)*(sj1));
IkReal x380=((py)*(sj0));
IkReal x381=((IkReal(1.00000000000000))*(cj0));
IkReal x382=((r01)*(sj0));
IkReal x383=((cj1)*(sj2));
IkReal x384=((cj0)*(x383));
IkReal x385=((sj3)*(x376));
IkReal x386=((cj3)*(x377));
IkReal x387=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x388=((sj3)*(x377));
IkReal x389=((cj3)*(x376));
evalcond[0]=((((r00)*(sj0)))+(x388)+(((IkReal(-1.00000000000000))*(x389)))+(((IkReal(-1.00000000000000))*(r01)*(x381))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x381)))+(((IkReal(0.790000000000000))*(x389)))+(((px)*(sj0)))+(((IkReal(-0.790000000000000))*(x388))));
evalcond[2]=((((r02)*(x378)))+(x386)+(x385)+(((x382)*(x383)))+(((cj0)*(r00)*(x379)))+(((x379)*(x382)))+(((IkReal(-1.00000000000000))*(r02)*(x387)))+(((r00)*(x384))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((px)*(x384)))+(((cj0)*(px)*(x379)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x383)))+(((pz)*(x378)))+(((x379)*(x380)))+(((IkReal(-0.790000000000000))*(x386)))+(((IkReal(-0.790000000000000))*(x385)))+(((x380)*(x383)))+(((IkReal(-1.00000000000000))*(pz)*(x387)))+(((IkReal(-0.0750000000000000))*(x379))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x390=((cj0)*(px));
IkReal x391=((IkReal(1.00000000000000))*(cj1));
IkReal x392=((sj1)*(sj2));
IkReal x393=((py)*(sj0));
cj5array[0]=((IkReal(1.26582278481013))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((IkReal(0.405000000000000))+(((IkReal(0.0750000000000000))*(cj1)*(cj2)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(sj1)))+(((IkReal(-1.00000000000000))*(cj2)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x391)))+(((IkReal(-0.0750000000000000))*(x392)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(x391)*(x393)))+(((x390)*(x392)))+(((x392)*(x393))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x394=((r01)*(sj0));
IkReal x395=((sj1)*(sj2));
IkReal x396=((IkReal(1.00000000000000))*(cj1));
IkReal x397=((cj0)*(r00));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x396)))+(((x395)*(x397)))+(((x394)*(x395)))+(((sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(cj2)*(x396)*(x397)))+(((IkReal(-1.00000000000000))*(cj2)*(x394)*(x396))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x398=((cj1)*(cj2));
IkReal x399=((IkReal(1.00000000000000))*(cj0));
IkReal x400=((sj1)*(sj2));
IkReal x401=((IkReal(1.00000000000000))*(sj0));
IkReal x402=((sj0)*(x400));
IkReal x403=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x404=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x398)*(x401)))+(((IkReal(-1.00000000000000))*(r00)*(x398)*(x399)))+(((IkReal(-1.00000000000000))*(r02)*(x404)))+(((IkReal(-1.00000000000000))*(r02)*(x403)))+(((r01)*(x402)))+(((cj0)*(r00)*(x400))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(0.0750000000000000))*(x398)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x398)*(x399)))+(((py)*(x402)))+(((IkReal(-0.0750000000000000))*(x400)))+(((IkReal(-1.00000000000000))*(py)*(x398)*(x401)))+(((cj0)*(px)*(x400)))+(((IkReal(-1.00000000000000))*(pz)*(x404)))+(((IkReal(-1.00000000000000))*(pz)*(x403))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x405=((cj1)*(cj5));
IkReal x406=((IkReal(1.00000000000000))*(cj2));
IkReal x407=((r00)*(sj0));
IkReal x408=((IkReal(1.00000000000000))*(sj5));
IkReal x409=((cj0)*(r01));
IkReal x410=((r01)*(sj0));
IkReal x411=((sj1)*(sj5));
IkReal x412=((cj0)*(r00));
IkReal x413=((sj2)*(x412));
IkReal x414=((cj2)*(cj5)*(sj1));
IkReal x415=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x406)*(x411)*(x412)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x408)*(x410)))+(((IkReal(-1.00000000000000))*(x406)*(x410)*(x411)))+(((cj5)*(x407)))+(((IkReal(-1.00000000000000))*(cj1)*(x408)*(x413)))+(((IkReal(-1.00000000000000))*(cj5)*(x409)))+(((IkReal(-1.00000000000000))*(x408)*(x415)))+(((cj1)*(cj2)*(r02)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((x405)*(x413)))+(((x410)*(x414)))+(((sj5)*(x407)))+(((cj5)*(x415)))+(((IkReal(-1.00000000000000))*(r02)*(x405)*(x406)))+(((x412)*(x414)))+(((sj2)*(x405)*(x410)))+(((IkReal(-1.00000000000000))*(x408)*(x409))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(x406)*(x411)*(x412)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x408)*(x410)))+(((IkReal(-1.00000000000000))*(x406)*(x410)*(x411)))+(((cj5)*(x407)))+(((IkReal(-1.00000000000000))*(cj1)*(x408)*(x413)))+(((IkReal(-1.00000000000000))*(cj5)*(x409)))+(((IkReal(-1.00000000000000))*(x408)*(x415)))+(((cj1)*(cj2)*(r02)*(sj5)))))), ((gconst55)*(((((x405)*(x413)))+(((x410)*(x414)))+(((sj5)*(x407)))+(((cj5)*(x415)))+(((IkReal(-1.00000000000000))*(r02)*(x405)*(x406)))+(((x412)*(x414)))+(((sj2)*(x405)*(x410)))+(((IkReal(-1.00000000000000))*(x408)*(x409)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x416=IKsin(j3);
IkReal x417=IKcos(j3);
IkReal x418=((sj1)*(sj2));
IkReal x419=((cj2)*(sj1));
IkReal x420=((cj0)*(px));
IkReal x421=((py)*(sj0));
IkReal x422=((IkReal(0.790000000000000))*(sj5));
IkReal x423=((cj1)*(sj2));
IkReal x424=((cj0)*(r00));
IkReal x425=((IkReal(1.00000000000000))*(cj0));
IkReal x426=((r01)*(sj0));
IkReal x427=((cj5)*(x416));
IkReal x428=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x429=((cj5)*(x417));
evalcond[0]=((((IkReal(-1.00000000000000))*(x427)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x425)))+(((IkReal(-1.00000000000000))*(sj5)*(x417))));
evalcond[1]=((((px)*(sj0)))+(((IkReal(0.790000000000000))*(x427)))+(((IkReal(-1.00000000000000))*(py)*(x425)))+(((x417)*(x422))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x429)))+(((sj5)*(x416)))+(((x419)*(x426)))+(((x419)*(x424)))+(((IkReal(-1.00000000000000))*(r02)*(x428)))+(((x423)*(x426)))+(((x423)*(x424)))+(((r02)*(x418))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x420)*(x423)))+(((IkReal(-1.00000000000000))*(x416)*(x422)))+(((IkReal(-0.0750000000000000))*(x419)))+(((pz)*(x418)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x419)*(x421)))+(((x419)*(x420)))+(((IkReal(-0.0750000000000000))*(x423)))+(((IkReal(0.790000000000000))*(x429)))+(((IkReal(-1.00000000000000))*(pz)*(x428)))+(((x421)*(x423))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x430=((cj1)*(cj2));
IkReal x431=((IkReal(1.00000000000000))*(cj0));
IkReal x432=((sj1)*(sj2));
IkReal x433=((IkReal(1.00000000000000))*(sj0));
IkReal x434=((sj0)*(x432));
IkReal x435=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x436=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x432)))+(((IkReal(-1.00000000000000))*(r01)*(x430)*(x433)))+(((IkReal(-1.00000000000000))*(r00)*(x430)*(x431)))+(((IkReal(-1.00000000000000))*(r02)*(x436)))+(((IkReal(-1.00000000000000))*(r02)*(x435)))+(((r01)*(x434))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-1.00000000000000))*(px)*(x430)*(x431)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x430)*(x433)))+(((IkReal(-0.0750000000000000))*(x432)))+(((py)*(x434)))+(((IkReal(0.0750000000000000))*(x430)))+(((cj0)*(px)*(x432)))+(((IkReal(-1.00000000000000))*(pz)*(x435)))+(((IkReal(-1.00000000000000))*(pz)*(x436))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x437=((cj1)*(cj5));
IkReal x438=((cj0)*(r00));
IkReal x439=((cj5)*(sj0));
IkReal x440=((cj2)*(sj1));
IkReal x441=((r01)*(sj5));
IkReal x442=((cj1)*(sj5));
IkReal x443=((sj0)*(sj2));
IkReal x444=((IkReal(1.00000000000000))*(cj2)*(r02));
IkReal x445=((r02)*(sj1)*(sj2));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((cj1)*(x441)*(x443)))+(((sj0)*(x440)*(x441)))+(((r00)*(x439)))+(((sj5)*(x445)))+(((sj5)*(x438)*(x440)))+(((sj2)*(x438)*(x442)))+(((IkReal(-1.00000000000000))*(x442)*(x444))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(x437)*(x444)))+(((r01)*(x439)*(x440)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((cj5)*(x445)))+(((cj0)*(x441)))+(((sj2)*(x437)*(x438)))+(((r01)*(x437)*(x443)))+(((cj5)*(x438)*(x440))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((cj1)*(x441)*(x443)))+(((sj0)*(x440)*(x441)))+(((r00)*(x439)))+(((sj5)*(x445)))+(((sj5)*(x438)*(x440)))+(((sj2)*(x438)*(x442)))+(((IkReal(-1.00000000000000))*(x442)*(x444)))))), ((gconst56)*(((((IkReal(-1.00000000000000))*(x437)*(x444)))+(((r01)*(x439)*(x440)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((cj5)*(x445)))+(((cj0)*(x441)))+(((sj2)*(x437)*(x438)))+(((r01)*(x437)*(x443)))+(((cj5)*(x438)*(x440)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x446=IKcos(j3);
IkReal x447=IKsin(j3);
IkReal x448=((sj1)*(sj2));
IkReal x449=((cj2)*(sj1));
IkReal x450=((py)*(sj0));
IkReal x451=((IkReal(1.00000000000000))*(cj0));
IkReal x452=((r01)*(sj0));
IkReal x453=((cj1)*(sj2));
IkReal x454=((IkReal(0.790000000000000))*(x447));
IkReal x455=((cj0)*(x453));
IkReal x456=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x457=((IkReal(0.790000000000000))*(x446));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x451)))+(((cj5)*(x447)))+(((IkReal(-1.00000000000000))*(sj5)*(x446))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x454)))+(((IkReal(-1.00000000000000))*(py)*(x451)))+(((px)*(sj0)))+(((sj5)*(x457))));
evalcond[2]=((((cj0)*(r00)*(x449)))+(((r02)*(x448)))+(((x452)*(x453)))+(((cj5)*(x446)))+(((x449)*(x452)))+(((sj5)*(x447)))+(((r00)*(x455)))+(((IkReal(-1.00000000000000))*(r02)*(x456))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj5)*(x457)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj5)*(x454)))+(((IkReal(-0.0750000000000000))*(x449)))+(((pz)*(x448)))+(((IkReal(-1.00000000000000))*(pz)*(x456)))+(((x449)*(x450)))+(((px)*(x455)))+(((cj0)*(px)*(x449)))+(((x450)*(x453)))+(((IkReal(-0.0750000000000000))*(x453))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x458=((cj1)*(cj2));
IkReal x459=((IkReal(1.00000000000000))*(cj0));
IkReal x460=((sj1)*(sj2));
IkReal x461=((IkReal(1.00000000000000))*(sj0));
IkReal x462=((sj0)*(x460));
IkReal x463=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x464=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r01)*(x458)*(x461)))+(((IkReal(-1.00000000000000))*(r00)*(x458)*(x459)))+(((cj0)*(r00)*(x460)))+(((IkReal(-1.00000000000000))*(r02)*(x464)))+(((IkReal(-1.00000000000000))*(r02)*(x463)))+(((r01)*(x462))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x458)*(x461)))+(((IkReal(-1.00000000000000))*(px)*(x458)*(x459)))+(((IkReal(0.0750000000000000))*(x458)))+(((cj0)*(px)*(x460)))+(((IkReal(-0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)*(x464)))+(((IkReal(-1.00000000000000))*(pz)*(x463)))+(((IkReal(-0.0750000000000000))*(x460)))+(((py)*(x462)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x465=((cj1)*(sj2));
IkReal x466=((cj0)*(r00));
IkReal x467=((IkReal(1.00000000000000))*(pz));
IkReal x468=((sj1)*(sj2));
IkReal x469=((r01)*(sj0));
IkReal x470=((IkReal(1.00000000000000))*(r02));
IkReal x471=((cj1)*(cj2));
IkReal x472=((cj2)*(sj1));
IkReal x473=((IkReal(1.00000000000000))*(cj0));
IkReal x474=((IkReal(1.00000000000000))*(sj0)*(x471));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x473))));
evalcond[2]=((IkReal(1.00000000000000))+(((x468)*(x469)))+(((x466)*(x468)))+(((IkReal(-1.00000000000000))*(x469)*(x471)))+(((IkReal(-1.00000000000000))*(x470)*(x472)))+(((IkReal(-1.00000000000000))*(x466)*(x471)))+(((IkReal(-1.00000000000000))*(x465)*(x470))));
evalcond[3]=((((x469)*(x472)))+(((x465)*(x466)))+(((x465)*(x469)))+(((x466)*(x472)))+(((r02)*(x468)))+(((IkReal(-1.00000000000000))*(x470)*(x471))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x465)*(x467)))+(((IkReal(-1.00000000000000))*(x467)*(x472)))+(((py)*(sj0)*(x468)))+(((IkReal(0.0750000000000000))*(x471)))+(((IkReal(-1.00000000000000))*(py)*(x474)))+(((cj0)*(px)*(x468)))+(((IkReal(-0.0750000000000000))*(x468)))+(((IkReal(-1.00000000000000))*(px)*(x471)*(x473))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x475=((IkReal(12.5000000000000))*(cj1));
IkReal x476=((py)*(sj0));
IkReal x477=((cj0)*(px));
IkReal x478=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x477)*(x478)))+(((cj2)*(pz)*(x475)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(x476)*(x478)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x477)*(x478)))+(((cj2)*(pz)*(x475)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(x476)*(x478)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x477)*(x478)))+(((cj2)*(pz)*(x475)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(sj2)*(x475)*(x476)))+(((IkReal(-1.00000000000000))*(x476)*(x478)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x479=((cj1)*(sj2));
IkReal x480=((cj2)*(sj1));
IkReal x481=((cj0)*(px));
IkReal x482=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x480)))+(((pz)*(sj1)*(sj2)))+(((x480)*(x481)))+(((x480)*(x482)))+(((IkReal(-0.0750000000000000))*(x479)))+(((x479)*(x481)))+(((x479)*(x482)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x483=((cj1)*(sj2));
IkReal x484=((cj0)*(r00));
IkReal x485=((IkReal(1.00000000000000))*(pz));
IkReal x486=((sj1)*(sj2));
IkReal x487=((r01)*(sj0));
IkReal x488=((IkReal(1.00000000000000))*(r02));
IkReal x489=((cj1)*(cj2));
IkReal x490=((cj2)*(sj1));
IkReal x491=((IkReal(1.00000000000000))*(cj0));
IkReal x492=((IkReal(1.00000000000000))*(sj0)*(x489));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x491))));
evalcond[2]=((IkReal(-1.00000000000000))+(((x484)*(x486)))+(((IkReal(-1.00000000000000))*(x488)*(x490)))+(((IkReal(-1.00000000000000))*(x483)*(x488)))+(((IkReal(-1.00000000000000))*(x484)*(x489)))+(((IkReal(-1.00000000000000))*(x487)*(x489)))+(((x486)*(x487))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x488)*(x489)))+(((r02)*(x486)))+(((x483)*(x484)))+(((x483)*(x487)))+(((x484)*(x490)))+(((x487)*(x490))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x483)*(x485)))+(((py)*(sj0)*(x486)))+(((IkReal(-1.00000000000000))*(px)*(x489)*(x491)))+(((IkReal(-0.0750000000000000))*(x486)))+(((IkReal(-1.00000000000000))*(x485)*(x490)))+(((cj0)*(px)*(x486)))+(((IkReal(-1.00000000000000))*(py)*(x492)))+(((IkReal(0.0750000000000000))*(x489))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x493=((IkReal(12.5000000000000))*(sj2));
IkReal x494=((py)*(sj0));
IkReal x495=((cj2)*(sj1));
IkReal x496=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x496))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj0)*(cj1)*(px)*(x493)))+(((IkReal(12.5000000000000))*(x494)*(x495)))+(((pz)*(sj1)*(x493)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x495)))+(((cj1)*(x493)*(x494)))+(((px)*(x495)*(x496))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x496)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj0)*(cj1)*(px)*(x493)))+(((IkReal(12.5000000000000))*(x494)*(x495)))+(((pz)*(sj1)*(x493)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x495)))+(((cj1)*(x493)*(x494)))+(((px)*(x495)*(x496)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x496)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj0)*(cj1)*(px)*(x493)))+(((IkReal(12.5000000000000))*(x494)*(x495)))+(((pz)*(sj1)*(x493)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x495)))+(((cj1)*(x493)*(x494)))+(((px)*(x495)*(x496)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x497=((cj1)*(sj2));
IkReal x498=((cj2)*(sj1));
IkReal x499=((cj0)*(px));
IkReal x500=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x497)*(x500)))+(((IkReal(-0.0750000000000000))*(x498)))+(((IkReal(-0.0750000000000000))*(x497)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x497)*(x499)))+(((pz)*(sj1)*(sj2)))+(((x498)*(x500)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((x498)*(x499)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x501=((cj1)*(sj2));
IkReal x502=((r01)*(sj0));
IkReal x503=((cj0)*(r00));
IkReal x504=((cj2)*(sj1));
if( IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((x501)*(x503)))+(((x501)*(x502)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x503)*(x504)))+(((x502)*(x504)))+(((r02)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((gconst57)*(((((x501)*(x503)))+(((x501)*(x502)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x503)*(x504)))+(((x502)*(x504)))+(((r02)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x505=IKsin(j3);
IkReal x506=IKcos(j3);
IkReal x507=((sj1)*(sj2));
IkReal x508=((cj2)*(sj1));
IkReal x509=((IkReal(0.0800000000000000))*(sj4));
IkReal x510=((py)*(sj0));
IkReal x511=((IkReal(0.790000000000000))*(cj4));
IkReal x512=((IkReal(1.00000000000000))*(cj0));
IkReal x513=((IkReal(1.00000000000000))*(cj4));
IkReal x514=((r01)*(sj0));
IkReal x515=((cj1)*(sj2));
IkReal x516=((cj0)*(x515));
IkReal x517=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x512)))+(((IkReal(-1.00000000000000))*(x505)*(x513))));
evalcond[1]=((((x505)*(x509)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x512)))+(((x505)*(x511))));
evalcond[2]=((((x514)*(x515)))+(((cj0)*(r00)*(x508)))+(((IkReal(-1.00000000000000))*(x506)*(x513)))+(((IkReal(-1.00000000000000))*(r02)*(x517)))+(((x508)*(x514)))+(((r00)*(x516)))+(((r02)*(x507))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x510)*(x515)))+(((px)*(x516)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x506)*(x509)))+(((IkReal(-0.0750000000000000))*(x515)))+(((IkReal(-1.00000000000000))*(pz)*(x517)))+(((x506)*(x511)))+(((cj0)*(px)*(x508)))+(((x508)*(x510)))+(((pz)*(x507)))+(((IkReal(-0.0750000000000000))*(x508))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x518=((cj1)*(cj2));
IkReal x519=((IkReal(1.00000000000000))*(cj0));
IkReal x520=((sj1)*(sj2));
IkReal x521=((IkReal(1.00000000000000))*(sj0));
IkReal x522=((sj0)*(x520));
IkReal x523=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x524=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x518)*(x521)))+(((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r02)*(x524)))+(((IkReal(-1.00000000000000))*(r02)*(x523)))+(((IkReal(-1.00000000000000))*(r00)*(x518)*(x519)))+(((cj0)*(r00)*(x520)))+(((r01)*(x522))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x520)))+(((cj0)*(px)*(x520)))+(((IkReal(-1.00000000000000))*(py)*(x518)*(x521)))+(((IkReal(0.0750000000000000))*(x518)))+(((IkReal(-1.00000000000000))*(px)*(x518)*(x519)))+(((IkReal(-1.00000000000000))*(pz)*(x524)))+(((IkReal(-1.00000000000000))*(pz)*(x523)))+(((IkReal(0.790000000000000))*(sj4)))+(((IkReal(0.0800000000000000))*(cj4)))+(((py)*(x522))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x525=((cj1)*(sj2));
IkReal x526=((cj0)*(r00));
IkReal x527=((IkReal(1.00000000000000))*(pz));
IkReal x528=((sj1)*(sj2));
IkReal x529=((r01)*(sj0));
IkReal x530=((IkReal(1.00000000000000))*(r02));
IkReal x531=((cj1)*(cj2));
IkReal x532=((cj2)*(sj1));
IkReal x533=((IkReal(1.00000000000000))*(cj0));
IkReal x534=((IkReal(1.00000000000000))*(sj0)*(x531));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x533))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x525)*(x530)))+(((IkReal(-1.00000000000000))*(x529)*(x531)))+(((IkReal(-1.00000000000000))*(x530)*(x532)))+(((x526)*(x528)))+(((x528)*(x529)))+(((IkReal(-1.00000000000000))*(x526)*(x531))));
evalcond[3]=((((r02)*(x528)))+(((IkReal(-1.00000000000000))*(x530)*(x531)))+(((x526)*(x532)))+(((x525)*(x526)))+(((x525)*(x529)))+(((x529)*(x532))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x528)))+(((IkReal(-1.00000000000000))*(x525)*(x527)))+(((IkReal(-1.00000000000000))*(px)*(x531)*(x533)))+(((IkReal(-1.00000000000000))*(x527)*(x532)))+(((cj0)*(px)*(x528)))+(((IkReal(0.0750000000000000))*(x531)))+(((py)*(sj0)*(x528)))+(((IkReal(-1.00000000000000))*(py)*(x534))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x535=((IkReal(12.5000000000000))*(cj1));
IkReal x536=((py)*(sj0));
IkReal x537=((cj0)*(px));
IkReal x538=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x537)*(x538)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((cj2)*(pz)*(x535)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x535)*(x537)))+(((IkReal(-1.00000000000000))*(sj2)*(x535)*(x536)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x536)*(x538)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x537)*(x538)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((cj2)*(pz)*(x535)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x535)*(x537)))+(((IkReal(-1.00000000000000))*(sj2)*(x535)*(x536)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x536)*(x538)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x537)*(x538)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((cj2)*(pz)*(x535)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x535)*(x537)))+(((IkReal(-1.00000000000000))*(sj2)*(x535)*(x536)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x536)*(x538)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x539=((cj1)*(sj2));
IkReal x540=((cj2)*(sj1));
IkReal x541=((cj0)*(px));
IkReal x542=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x539)*(x542)))+(((x539)*(x541)))+(((IkReal(-0.0750000000000000))*(x539)))+(((IkReal(-0.0750000000000000))*(x540)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x540)*(x541)))+(((x540)*(x542)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x543=((cj1)*(sj2));
IkReal x544=((cj0)*(r00));
IkReal x545=((IkReal(1.00000000000000))*(pz));
IkReal x546=((sj1)*(sj2));
IkReal x547=((r01)*(sj0));
IkReal x548=((IkReal(1.00000000000000))*(r02));
IkReal x549=((cj1)*(cj2));
IkReal x550=((cj2)*(sj1));
IkReal x551=((IkReal(1.00000000000000))*(cj0));
IkReal x552=((IkReal(1.00000000000000))*(sj0)*(x549));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x551))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x547)*(x549)))+(((IkReal(-1.00000000000000))*(x544)*(x549)))+(((x544)*(x546)))+(((IkReal(-1.00000000000000))*(x548)*(x550)))+(((IkReal(-1.00000000000000))*(x543)*(x548)))+(((x546)*(x547))));
evalcond[3]=((((r02)*(x546)))+(((x544)*(x550)))+(((x543)*(x547)))+(((x543)*(x544)))+(((x547)*(x550)))+(((IkReal(-1.00000000000000))*(x548)*(x549))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x546)))+(((IkReal(-1.00000000000000))*(x545)*(x550)))+(((cj0)*(px)*(x546)))+(((IkReal(0.0750000000000000))*(x549)))+(((IkReal(-1.00000000000000))*(px)*(x549)*(x551)))+(((py)*(sj0)*(x546)))+(((IkReal(-1.00000000000000))*(py)*(x552)))+(((IkReal(-1.00000000000000))*(x543)*(x545))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x553=((IkReal(12.5000000000000))*(sj2));
IkReal x554=((py)*(sj0));
IkReal x555=((cj2)*(sj1));
IkReal x556=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x556))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-0.937500000000000))*(x555)))+(((pz)*(sj1)*(x553)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x555)*(x556)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((cj1)*(x553)*(x554)))+(((cj0)*(cj1)*(px)*(x553)))+(((IkReal(12.5000000000000))*(x554)*(x555))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x556)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-0.937500000000000))*(x555)))+(((pz)*(sj1)*(x553)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x555)*(x556)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((cj1)*(x553)*(x554)))+(((cj0)*(cj1)*(px)*(x553)))+(((IkReal(12.5000000000000))*(x554)*(x555)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x556)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-0.937500000000000))*(x555)))+(((pz)*(sj1)*(x553)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x555)*(x556)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((cj1)*(x553)*(x554)))+(((cj0)*(cj1)*(px)*(x553)))+(((IkReal(12.5000000000000))*(x554)*(x555)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x557=((cj1)*(sj2));
IkReal x558=((cj2)*(sj1));
IkReal x559=((cj0)*(px));
IkReal x560=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x558)*(x560)))+(((IkReal(-0.0750000000000000))*(x558)))+(((IkReal(-0.0750000000000000))*(x557)))+(((x557)*(x559)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((x558)*(x559)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((x557)*(x560))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x561=((IkReal(1.00000000000000))*(sj2));
IkReal x562=((cj0)*(r00));
IkReal x563=((r01)*(sj0));
IkReal x564=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x561)))+(((IkReal(-1.00000000000000))*(x562)*(x564)))+(((IkReal(-1.00000000000000))*(x563)*(x564)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x561)*(x562)))+(((IkReal(-1.00000000000000))*(cj1)*(x561)*(x563))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x561)))+(((IkReal(-1.00000000000000))*(x562)*(x564)))+(((IkReal(-1.00000000000000))*(x563)*(x564)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x561)*(x562)))+(((IkReal(-1.00000000000000))*(cj1)*(x561)*(x563)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x565=IKsin(j3);
IkReal x566=IKcos(j3);
IkReal x567=((sj1)*(sj2));
IkReal x568=((cj2)*(sj1));
IkReal x569=((IkReal(0.0800000000000000))*(sj4));
IkReal x570=((py)*(sj0));
IkReal x571=((IkReal(1.00000000000000))*(cj0));
IkReal x572=((r01)*(sj0));
IkReal x573=((cj1)*(sj2));
IkReal x574=((cj0)*(x573));
IkReal x575=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x576=((cj4)*(x565));
IkReal x577=((cj4)*(x566));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x571)))+(x576));
evalcond[1]=((((IkReal(-0.790000000000000))*(x576)))+(((x565)*(x569)))+(((IkReal(-1.00000000000000))*(py)*(x571)))+(((px)*(sj0))));
evalcond[2]=((((r00)*(x574)))+(((cj0)*(r00)*(x568)))+(x577)+(((x568)*(x572)))+(((IkReal(-1.00000000000000))*(r02)*(x575)))+(((r02)*(x567)))+(((x572)*(x573))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x566)*(x569)))+(((IkReal(-0.790000000000000))*(x577)))+(((cj0)*(px)*(x568)))+(((px)*(x574)))+(((IkReal(-0.0750000000000000))*(x573)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pz)*(x575)))+(((IkReal(-0.0750000000000000))*(x568)))+(((pz)*(x567)))+(((x568)*(x570)))+(((x570)*(x573))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x578=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x579=((r00)*(sj0));
IkReal x580=((cj4)*(cj5));
IkReal x581=((IkReal(8.00000000000000))*(sj4));
IkReal x582=((cj0)*(r01));
IkReal x583=((IkReal(-0.125000000000000))*(x578));
IkReal x584=((IkReal(100.000000000000))*(cj0)*(py));
IkReal x585=((IkReal(100.000000000000))*(px)*(sj0));
if( IKabs(((x583)*(((((IkReal(79.0000000000000))*(x579)))+(x585)+(((IkReal(-79.0000000000000))*(x582)))+(((IkReal(-1.00000000000000))*(x584))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x583)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(79.0000000000000))*(x580)*(x582)))+(((IkReal(-1.00000000000000))*(x580)*(x585)))+(((x581)*(x582)))+(((x580)*(x584)))+(((IkReal(-1.00000000000000))*(x579)*(x581)))+(((IkReal(-79.0000000000000))*(x579)*(x580))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x583)*(((((IkReal(79.0000000000000))*(x579)))+(x585)+(((IkReal(-79.0000000000000))*(x582)))+(((IkReal(-1.00000000000000))*(x584)))))))+IKsqr(((x583)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(79.0000000000000))*(x580)*(x582)))+(((IkReal(-1.00000000000000))*(x580)*(x585)))+(((x581)*(x582)))+(((x580)*(x584)))+(((IkReal(-1.00000000000000))*(x579)*(x581)))+(((IkReal(-79.0000000000000))*(x579)*(x580)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x583)*(((((IkReal(79.0000000000000))*(x579)))+(x585)+(((IkReal(-79.0000000000000))*(x582)))+(((IkReal(-1.00000000000000))*(x584)))))), ((x583)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(79.0000000000000))*(x580)*(x582)))+(((IkReal(-1.00000000000000))*(x580)*(x585)))+(((x581)*(x582)))+(((x580)*(x584)))+(((IkReal(-1.00000000000000))*(x579)*(x581)))+(((IkReal(-79.0000000000000))*(x579)*(x580)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x586=IKsin(j3);
IkReal x587=IKcos(j3);
IkReal x588=((sj1)*(sj2));
IkReal x589=((IkReal(0.0800000000000000))*(sj4));
IkReal x590=((cj2)*(sj1));
IkReal x591=((py)*(sj0));
IkReal x592=((IkReal(0.790000000000000))*(sj5));
IkReal x593=((IkReal(1.00000000000000))*(cj0));
IkReal x594=((r01)*(sj0));
IkReal x595=((cj1)*(sj2));
IkReal x596=((cj0)*(x595));
IkReal x597=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x598=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x599=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x593)))+(((IkReal(-1.00000000000000))*(x586)*(x597)))+(((IkReal(-1.00000000000000))*(sj5)*(x587))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x593)))+(((x586)*(x589)))+(((px)*(sj0)))+(((x586)*(x598)))+(((x587)*(x592))));
evalcond[2]=((((x594)*(x595)))+(((IkReal(-1.00000000000000))*(x587)*(x597)))+(((r02)*(x588)))+(((cj0)*(r00)*(x590)))+(((sj5)*(x586)))+(((r00)*(x596)))+(((IkReal(-1.00000000000000))*(r02)*(x599)))+(((x590)*(x594))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x586)*(x592)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x587)*(x589)))+(((IkReal(-1.00000000000000))*(pz)*(x599)))+(((IkReal(-0.0750000000000000))*(x595)))+(((IkReal(-0.0750000000000000))*(x590)))+(((cj0)*(px)*(x590)))+(((x590)*(x591)))+(((px)*(x596)))+(((x591)*(x595)))+(((pz)*(x588)))+(((x587)*(x598))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x600=((cj4)*(cj5));
IkReal x601=((r00)*(sj0));
IkReal x602=((cj0)*(r00));
IkReal x603=((r01)*(sj0));
IkReal x604=((IkReal(1.00000000000000))*(cj2));
IkReal x605=((cj1)*(r02));
IkReal x606=((sj1)*(sj5));
IkReal x607=((cj1)*(sj2));
IkReal x608=((cj2)*(sj1));
IkReal x609=((r02)*(sj2));
IkReal x610=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x611=((IkReal(1.00000000000000))*(sj5)*(x607));
if( IKabs(((gconst54)*(((((cj2)*(sj5)*(x605)))+(((IkReal(-1.00000000000000))*(x603)*(x604)*(x606)))+(((x600)*(x601)))+(((IkReal(-1.00000000000000))*(x602)*(x604)*(x606)))+(((IkReal(-1.00000000000000))*(x600)*(x610)))+(((IkReal(-1.00000000000000))*(x603)*(x611)))+(((IkReal(-1.00000000000000))*(x606)*(x609)))+(((IkReal(-1.00000000000000))*(x602)*(x611))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x600)*(x604)*(x605)))+(((IkReal(-1.00000000000000))*(sj5)*(x610)))+(((sj1)*(x600)*(x609)))+(((x600)*(x602)*(x607)))+(((x600)*(x602)*(x608)))+(((sj5)*(x601)))+(((x600)*(x603)*(x608)))+(((x600)*(x603)*(x607))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((cj2)*(sj5)*(x605)))+(((IkReal(-1.00000000000000))*(x603)*(x604)*(x606)))+(((x600)*(x601)))+(((IkReal(-1.00000000000000))*(x602)*(x604)*(x606)))+(((IkReal(-1.00000000000000))*(x600)*(x610)))+(((IkReal(-1.00000000000000))*(x603)*(x611)))+(((IkReal(-1.00000000000000))*(x606)*(x609)))+(((IkReal(-1.00000000000000))*(x602)*(x611)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(x600)*(x604)*(x605)))+(((IkReal(-1.00000000000000))*(sj5)*(x610)))+(((sj1)*(x600)*(x609)))+(((x600)*(x602)*(x607)))+(((x600)*(x602)*(x608)))+(((sj5)*(x601)))+(((x600)*(x603)*(x608)))+(((x600)*(x603)*(x607)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x612=IKsin(j3);
IkReal x613=IKcos(j3);
IkReal x614=((sj1)*(sj2));
IkReal x615=((IkReal(0.0800000000000000))*(sj4));
IkReal x616=((cj2)*(sj1));
IkReal x617=((py)*(sj0));
IkReal x618=((IkReal(0.790000000000000))*(sj5));
IkReal x619=((IkReal(1.00000000000000))*(cj0));
IkReal x620=((r01)*(sj0));
IkReal x621=((cj1)*(sj2));
IkReal x622=((cj0)*(x621));
IkReal x623=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x624=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x625=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x619)))+(((IkReal(-1.00000000000000))*(sj5)*(x613)))+(((IkReal(-1.00000000000000))*(x612)*(x623))));
evalcond[1]=((((x612)*(x615)))+(((x612)*(x624)))+(((x613)*(x618)))+(((IkReal(-1.00000000000000))*(py)*(x619)))+(((px)*(sj0))));
evalcond[2]=((((x616)*(x620)))+(((IkReal(-1.00000000000000))*(r02)*(x625)))+(((cj0)*(r00)*(x616)))+(((r02)*(x614)))+(((r00)*(x622)))+(((IkReal(-1.00000000000000))*(x613)*(x623)))+(((sj5)*(x612)))+(((x620)*(x621))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(x612)*(x618)))+(((IkReal(-1.00000000000000))*(pz)*(x625)))+(((x617)*(x621)))+(((IkReal(-0.0750000000000000))*(x616)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj0)*(px)*(x616)))+(((x613)*(x624)))+(((x616)*(x617)))+(((pz)*(x614)))+(((x613)*(x615)))+(((IkReal(-0.0750000000000000))*(x621)))+(((px)*(x622))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x626=((r01)*(sj0));
IkReal x627=((sj1)*(sj2));
IkReal x628=((IkReal(1.00000000000000))*(cj1));
IkReal x629=((cj0)*(r00));
cj5array[0]=((IkReal(-1.00000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj2)*(x628)*(x629)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x628)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(cj2)*(x626)*(x628)))+(((x627)*(x629)))+(((x626)*(x627))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x630=((cj0)*(px));
IkReal x631=((IkReal(1.00000000000000))*(cj1));
IkReal x632=((sj1)*(sj2));
IkReal x633=((py)*(sj0));
evalcond[0]=((IkReal(0.405000000000000))+(((IkReal(0.0750000000000000))*(cj1)*(cj2)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x631)*(x633)))+(((IkReal(-0.790000000000000))*(sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x631)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(sj1)))+(((x632)*(x633)))+(((IkReal(-0.0750000000000000))*(x632)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(x630)*(x631)))+(((x630)*(x632))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x634=((cj1)*(cj2));
IkReal x635=((IkReal(1.00000000000000))*(cj0));
IkReal x636=((sj1)*(sj2));
IkReal x637=((IkReal(1.00000000000000))*(sj0));
IkReal x638=((sj0)*(x636));
IkReal x639=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x640=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r01)*(x634)*(x637)))+(((IkReal(-1.00000000000000))*(r02)*(x639)))+(((IkReal(-1.00000000000000))*(r02)*(x640)))+(((r01)*(x638)))+(((IkReal(-1.00000000000000))*(r00)*(x634)*(x635)))+(((cj0)*(r00)*(x636))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((py)*(x638)))+(((IkReal(-1.00000000000000))*(pz)*(x639)))+(((cj0)*(px)*(x636)))+(((IkReal(-1.00000000000000))*(px)*(x634)*(x635)))+(((IkReal(-1.00000000000000))*(py)*(x634)*(x637)))+(((IkReal(0.0750000000000000))*(x634)))+(((IkReal(-0.0750000000000000))*(x636)))+(((IkReal(-1.00000000000000))*(pz)*(x640))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x641=((cj1)*(cj5));
IkReal x642=((IkReal(1.00000000000000))*(cj2));
IkReal x643=((r00)*(sj0));
IkReal x644=((IkReal(1.00000000000000))*(sj5));
IkReal x645=((cj0)*(r01));
IkReal x646=((r01)*(sj0));
IkReal x647=((sj1)*(sj5));
IkReal x648=((cj0)*(r00));
IkReal x649=((sj2)*(x648));
IkReal x650=((cj2)*(cj5)*(sj1));
IkReal x651=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x642)*(x647)*(x648)))+(((IkReal(-1.00000000000000))*(x642)*(x646)*(x647)))+(((cj5)*(x643)))+(((IkReal(-1.00000000000000))*(cj1)*(x644)*(x649)))+(((IkReal(-1.00000000000000))*(cj5)*(x645)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x644)*(x646)))+(((IkReal(-1.00000000000000))*(x644)*(x651)))+(((cj1)*(cj2)*(r02)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((cj5)*(x651)))+(((x646)*(x650)))+(((sj2)*(x641)*(x646)))+(((x648)*(x650)))+(((sj5)*(x643)))+(((x641)*(x649)))+(((IkReal(-1.00000000000000))*(x644)*(x645)))+(((IkReal(-1.00000000000000))*(r02)*(x641)*(x642))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(x642)*(x647)*(x648)))+(((IkReal(-1.00000000000000))*(x642)*(x646)*(x647)))+(((cj5)*(x643)))+(((IkReal(-1.00000000000000))*(cj1)*(x644)*(x649)))+(((IkReal(-1.00000000000000))*(cj5)*(x645)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x644)*(x646)))+(((IkReal(-1.00000000000000))*(x644)*(x651)))+(((cj1)*(cj2)*(r02)*(sj5)))))), ((gconst55)*(((((cj5)*(x651)))+(((x646)*(x650)))+(((sj2)*(x641)*(x646)))+(((x648)*(x650)))+(((sj5)*(x643)))+(((x641)*(x649)))+(((IkReal(-1.00000000000000))*(x644)*(x645)))+(((IkReal(-1.00000000000000))*(r02)*(x641)*(x642)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x652=IKsin(j3);
IkReal x653=IKcos(j3);
IkReal x654=((sj1)*(sj2));
IkReal x655=((cj2)*(sj1));
IkReal x656=((cj0)*(px));
IkReal x657=((py)*(sj0));
IkReal x658=((IkReal(0.790000000000000))*(sj5));
IkReal x659=((cj1)*(sj2));
IkReal x660=((cj0)*(r00));
IkReal x661=((IkReal(1.00000000000000))*(cj0));
IkReal x662=((r01)*(sj0));
IkReal x663=((cj5)*(x652));
IkReal x664=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x665=((cj5)*(x653));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x661)))+(((IkReal(-1.00000000000000))*(sj5)*(x653)))+(((IkReal(-1.00000000000000))*(x663))));
evalcond[1]=((((IkReal(0.790000000000000))*(x663)))+(((x653)*(x658)))+(((IkReal(-1.00000000000000))*(py)*(x661)))+(((px)*(sj0))));
evalcond[2]=((((r02)*(x654)))+(((IkReal(-1.00000000000000))*(x665)))+(((sj5)*(x652)))+(((x659)*(x660)))+(((x659)*(x662)))+(((IkReal(-1.00000000000000))*(r02)*(x664)))+(((x655)*(x660)))+(((x655)*(x662))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x659)))+(((IkReal(-0.0750000000000000))*(x655)))+(((IkReal(0.790000000000000))*(x665)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(x654)))+(((x656)*(x659)))+(((x657)*(x659)))+(((IkReal(-1.00000000000000))*(pz)*(x664)))+(((x655)*(x656)))+(((x655)*(x657)))+(((IkReal(-1.00000000000000))*(x652)*(x658))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x666=((cj1)*(cj2));
IkReal x667=((IkReal(1.00000000000000))*(cj0));
IkReal x668=((sj1)*(sj2));
IkReal x669=((IkReal(1.00000000000000))*(sj0));
IkReal x670=((sj0)*(x668));
IkReal x671=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x672=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x671)))+(((IkReal(-1.00000000000000))*(r02)*(x672)))+(((r01)*(x670)))+(((IkReal(-1.00000000000000))*(r00)*(x666)*(x667)))+(((cj0)*(r00)*(x668)))+(((IkReal(-1.00000000000000))*(r01)*(x666)*(x669))));
evalcond[2]=((IkReal(0.325000000000000))+(((py)*(x670)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x666)*(x667)))+(((IkReal(-1.00000000000000))*(py)*(x666)*(x669)))+(((IkReal(-0.0750000000000000))*(x668)))+(((cj0)*(px)*(x668)))+(((IkReal(-1.00000000000000))*(pz)*(x671)))+(((IkReal(-1.00000000000000))*(pz)*(x672)))+(((IkReal(0.0750000000000000))*(x666))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x673=((cj1)*(cj5));
IkReal x674=((cj0)*(r00));
IkReal x675=((cj5)*(sj0));
IkReal x676=((cj2)*(sj1));
IkReal x677=((r01)*(sj5));
IkReal x678=((cj1)*(sj5));
IkReal x679=((sj0)*(sj2));
IkReal x680=((IkReal(1.00000000000000))*(cj2)*(r02));
IkReal x681=((r02)*(sj1)*(sj2));
if( IKabs(((gconst56)*(((((sj5)*(x674)*(x676)))+(((r00)*(x675)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj5)*(x681)))+(((cj1)*(x677)*(x679)))+(((IkReal(-1.00000000000000))*(x678)*(x680)))+(((sj2)*(x674)*(x678)))+(((sj0)*(x676)*(x677))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((r01)*(x675)*(x676)))+(((r01)*(x673)*(x679)))+(((cj5)*(x674)*(x676)))+(((cj0)*(x677)))+(((cj5)*(x681)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x673)*(x680)))+(((sj2)*(x673)*(x674))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((sj5)*(x674)*(x676)))+(((r00)*(x675)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj5)*(x681)))+(((cj1)*(x677)*(x679)))+(((IkReal(-1.00000000000000))*(x678)*(x680)))+(((sj2)*(x674)*(x678)))+(((sj0)*(x676)*(x677)))))), ((gconst56)*(((((r01)*(x675)*(x676)))+(((r01)*(x673)*(x679)))+(((cj5)*(x674)*(x676)))+(((cj0)*(x677)))+(((cj5)*(x681)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x673)*(x680)))+(((sj2)*(x673)*(x674)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x682=IKcos(j3);
IkReal x683=IKsin(j3);
IkReal x684=((sj1)*(sj2));
IkReal x685=((cj2)*(sj1));
IkReal x686=((py)*(sj0));
IkReal x687=((IkReal(1.00000000000000))*(cj0));
IkReal x688=((r01)*(sj0));
IkReal x689=((cj1)*(sj2));
IkReal x690=((IkReal(0.790000000000000))*(x683));
IkReal x691=((cj0)*(x689));
IkReal x692=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x693=((IkReal(0.790000000000000))*(x682));
evalcond[0]=((((r00)*(sj0)))+(((cj5)*(x683)))+(((IkReal(-1.00000000000000))*(r01)*(x687)))+(((IkReal(-1.00000000000000))*(sj5)*(x682))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x690)))+(((sj5)*(x693)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x687))));
evalcond[2]=((((x688)*(x689)))+(((cj5)*(x682)))+(((x685)*(x688)))+(((sj5)*(x683)))+(((cj0)*(r00)*(x685)))+(((IkReal(-1.00000000000000))*(r02)*(x692)))+(((r00)*(x691)))+(((r02)*(x684))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj5)*(x690)))+(((x686)*(x689)))+(((IkReal(-1.00000000000000))*(cj5)*(x693)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x685)*(x686)))+(((px)*(x691)))+(((IkReal(-0.0750000000000000))*(x689)))+(((IkReal(-0.0750000000000000))*(x685)))+(((IkReal(-1.00000000000000))*(pz)*(x692)))+(((pz)*(x684)))+(((cj0)*(px)*(x685))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x694=((cj1)*(cj2));
IkReal x695=((IkReal(1.00000000000000))*(cj0));
IkReal x696=((sj1)*(sj2));
IkReal x697=((IkReal(1.00000000000000))*(sj0));
IkReal x698=((sj0)*(x696));
IkReal x699=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x700=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x700)))+(sj4)+(((cj0)*(r00)*(x696)))+(((IkReal(-1.00000000000000))*(r01)*(x694)*(x697)))+(((IkReal(-1.00000000000000))*(r00)*(x694)*(x695)))+(((IkReal(-1.00000000000000))*(r02)*(x699)))+(((r01)*(x698))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x700)))+(((IkReal(-0.0750000000000000))*(x696)))+(((IkReal(-1.00000000000000))*(px)*(x694)*(x695)))+(((IkReal(0.0750000000000000))*(x694)))+(((py)*(x698)))+(((IkReal(-0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)*(x699)))+(((cj0)*(px)*(x696)))+(((IkReal(-1.00000000000000))*(py)*(x694)*(x697)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x701=((cj1)*(sj2));
IkReal x702=((cj0)*(r00));
IkReal x703=((IkReal(1.00000000000000))*(pz));
IkReal x704=((sj1)*(sj2));
IkReal x705=((r01)*(sj0));
IkReal x706=((IkReal(1.00000000000000))*(r02));
IkReal x707=((cj1)*(cj2));
IkReal x708=((cj2)*(sj1));
IkReal x709=((IkReal(1.00000000000000))*(cj0));
IkReal x710=((IkReal(1.00000000000000))*(sj0)*(x707));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x709))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x702)*(x707)))+(((x702)*(x704)))+(((IkReal(-1.00000000000000))*(x705)*(x707)))+(((x704)*(x705)))+(((IkReal(-1.00000000000000))*(x701)*(x706))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x706)*(x707)))+(((x701)*(x705)))+(((x701)*(x702)))+(((x702)*(x708)))+(((r02)*(x704)))+(((x705)*(x708))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x707)))+(((IkReal(-0.0750000000000000))*(x704)))+(((IkReal(-1.00000000000000))*(px)*(x707)*(x709)))+(((py)*(sj0)*(x704)))+(((IkReal(-1.00000000000000))*(x703)*(x708)))+(((cj0)*(px)*(x704)))+(((IkReal(-1.00000000000000))*(x701)*(x703)))+(((IkReal(-1.00000000000000))*(py)*(x710))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x711=((IkReal(12.5000000000000))*(cj1));
IkReal x712=((py)*(sj0));
IkReal x713=((cj0)*(px));
IkReal x714=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x711)))+(((IkReal(-1.00000000000000))*(sj2)*(x711)*(x713)))+(((IkReal(-1.00000000000000))*(sj2)*(x711)*(x712)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x712)*(x714)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x713)*(x714)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x711)))+(((IkReal(-1.00000000000000))*(sj2)*(x711)*(x713)))+(((IkReal(-1.00000000000000))*(sj2)*(x711)*(x712)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x712)*(x714)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x713)*(x714)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((cj2)*(pz)*(x711)))+(((IkReal(-1.00000000000000))*(sj2)*(x711)*(x713)))+(((IkReal(-1.00000000000000))*(sj2)*(x711)*(x712)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x712)*(x714)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x713)*(x714)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x715=((cj1)*(sj2));
IkReal x716=((cj2)*(sj1));
IkReal x717=((cj0)*(px));
IkReal x718=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x715)*(x717)))+(((x715)*(x718)))+(((IkReal(-0.0750000000000000))*(x715)))+(((IkReal(-0.0750000000000000))*(x716)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((x716)*(x717)))+(((x716)*(x718)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x719=((cj1)*(sj2));
IkReal x720=((cj0)*(r00));
IkReal x721=((IkReal(1.00000000000000))*(pz));
IkReal x722=((sj1)*(sj2));
IkReal x723=((r01)*(sj0));
IkReal x724=((IkReal(1.00000000000000))*(r02));
IkReal x725=((cj1)*(cj2));
IkReal x726=((cj2)*(sj1));
IkReal x727=((IkReal(1.00000000000000))*(cj0));
IkReal x728=((IkReal(1.00000000000000))*(sj0)*(x725));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x727))));
evalcond[2]=((IkReal(-1.00000000000000))+(((x720)*(x722)))+(((IkReal(-1.00000000000000))*(x724)*(x726)))+(((x722)*(x723)))+(((IkReal(-1.00000000000000))*(x720)*(x725)))+(((IkReal(-1.00000000000000))*(x723)*(x725)))+(((IkReal(-1.00000000000000))*(x719)*(x724))));
evalcond[3]=((((x720)*(x726)))+(((IkReal(-1.00000000000000))*(x724)*(x725)))+(((x723)*(x726)))+(((r02)*(x722)))+(((x719)*(x723)))+(((x719)*(x720))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x722)))+(((cj0)*(px)*(x722)))+(((py)*(sj0)*(x722)))+(((IkReal(-1.00000000000000))*(x721)*(x726)))+(((IkReal(-1.00000000000000))*(px)*(x725)*(x727)))+(((IkReal(-1.00000000000000))*(x719)*(x721)))+(((IkReal(-1.00000000000000))*(py)*(x728)))+(((IkReal(0.0750000000000000))*(x725))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x729=((IkReal(12.5000000000000))*(sj2));
IkReal x730=((py)*(sj0));
IkReal x731=((cj2)*(sj1));
IkReal x732=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x732))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x729)*(x730)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x731)*(x732)))+(((pz)*(sj1)*(x729)))+(((IkReal(12.5000000000000))*(x730)*(x731)))+(((cj0)*(cj1)*(px)*(x729)))+(((IkReal(-0.937500000000000))*(x731))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x732)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x729)*(x730)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x731)*(x732)))+(((pz)*(sj1)*(x729)))+(((IkReal(12.5000000000000))*(x730)*(x731)))+(((cj0)*(cj1)*(px)*(x729)))+(((IkReal(-0.937500000000000))*(x731)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x732)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x729)*(x730)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x731)*(x732)))+(((pz)*(sj1)*(x729)))+(((IkReal(12.5000000000000))*(x730)*(x731)))+(((cj0)*(cj1)*(px)*(x729)))+(((IkReal(-0.937500000000000))*(x731)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x733=((cj1)*(sj2));
IkReal x734=((cj2)*(sj1));
IkReal x735=((cj0)*(px));
IkReal x736=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x733)))+(((IkReal(-0.0750000000000000))*(x734)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x734)*(x735)))+(((x734)*(x736)))+(((pz)*(sj1)*(sj2)))+(((x733)*(x736)))+(((x733)*(x735)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x737=((cj1)*(sj2));
IkReal x738=((r01)*(sj0));
IkReal x739=((cj0)*(r00));
IkReal x740=((cj2)*(sj1));
if( IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((x738)*(x740)))+(((x739)*(x740)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x737)*(x738)))+(((x737)*(x739))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((gconst57)*(((((x738)*(x740)))+(((x739)*(x740)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x737)*(x738)))+(((x737)*(x739)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x741=IKsin(j3);
IkReal x742=IKcos(j3);
IkReal x743=((sj1)*(sj2));
IkReal x744=((cj2)*(sj1));
IkReal x745=((IkReal(0.0800000000000000))*(sj4));
IkReal x746=((py)*(sj0));
IkReal x747=((IkReal(0.790000000000000))*(cj4));
IkReal x748=((IkReal(1.00000000000000))*(cj0));
IkReal x749=((IkReal(1.00000000000000))*(cj4));
IkReal x750=((r01)*(sj0));
IkReal x751=((cj1)*(sj2));
IkReal x752=((cj0)*(x751));
IkReal x753=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x748)))+(((IkReal(-1.00000000000000))*(x741)*(x749))));
evalcond[1]=((((x741)*(x747)))+(((x741)*(x745)))+(((IkReal(-1.00000000000000))*(py)*(x748)))+(((px)*(sj0))));
evalcond[2]=((((r02)*(x743)))+(((IkReal(-1.00000000000000))*(x742)*(x749)))+(((x744)*(x750)))+(((IkReal(-1.00000000000000))*(r02)*(x753)))+(((r00)*(x752)))+(((x750)*(x751)))+(((cj0)*(r00)*(x744))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(px)*(x744)))+(((x742)*(x745)))+(((x742)*(x747)))+(((x744)*(x746)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x752)))+(((IkReal(-0.0750000000000000))*(x744)))+(((IkReal(-0.0750000000000000))*(x751)))+(((x746)*(x751)))+(((IkReal(-1.00000000000000))*(pz)*(x753)))+(((pz)*(x743))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x754=((cj1)*(cj2));
IkReal x755=((IkReal(1.00000000000000))*(cj0));
IkReal x756=((sj1)*(sj2));
IkReal x757=((IkReal(1.00000000000000))*(sj0));
IkReal x758=((sj0)*(x756));
IkReal x759=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x760=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r02)*(x759)))+(((IkReal(-1.00000000000000))*(r00)*(x754)*(x755)))+(((r01)*(x758)))+(((IkReal(-1.00000000000000))*(r02)*(x760)))+(((IkReal(-1.00000000000000))*(r01)*(x754)*(x757)))+(((cj0)*(r00)*(x756))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((py)*(x758)))+(((cj0)*(px)*(x756)))+(((IkReal(-1.00000000000000))*(px)*(x754)*(x755)))+(((IkReal(-0.0750000000000000))*(x756)))+(((IkReal(-1.00000000000000))*(py)*(x754)*(x757)))+(((IkReal(0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)*(x759)))+(((IkReal(0.0750000000000000))*(x754)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(pz)*(x760))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x761=((cj1)*(sj2));
IkReal x762=((cj0)*(r00));
IkReal x763=((IkReal(1.00000000000000))*(pz));
IkReal x764=((sj1)*(sj2));
IkReal x765=((r01)*(sj0));
IkReal x766=((IkReal(1.00000000000000))*(r02));
IkReal x767=((cj1)*(cj2));
IkReal x768=((cj2)*(sj1));
IkReal x769=((IkReal(1.00000000000000))*(cj0));
IkReal x770=((IkReal(1.00000000000000))*(sj0)*(x767));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x769))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x766)*(x768)))+(((x762)*(x764)))+(((IkReal(-1.00000000000000))*(x765)*(x767)))+(((IkReal(-1.00000000000000))*(x762)*(x767)))+(((x764)*(x765)))+(((IkReal(-1.00000000000000))*(x761)*(x766))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x766)*(x767)))+(((x761)*(x762)))+(((x761)*(x765)))+(((x762)*(x768)))+(((r02)*(x764)))+(((x765)*(x768))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x767)))+(((cj0)*(px)*(x764)))+(((IkReal(-1.00000000000000))*(x763)*(x768)))+(((IkReal(-0.0750000000000000))*(x764)))+(((IkReal(-1.00000000000000))*(px)*(x767)*(x769)))+(((py)*(sj0)*(x764)))+(((IkReal(-1.00000000000000))*(py)*(x770)))+(((IkReal(-1.00000000000000))*(x761)*(x763))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x771=((IkReal(12.5000000000000))*(cj1));
IkReal x772=((py)*(sj0));
IkReal x773=((cj0)*(px));
IkReal x774=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((cj2)*(pz)*(x771)))+(((IkReal(-1.00000000000000))*(x772)*(x774)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(sj2)*(x771)*(x772)))+(((IkReal(-1.00000000000000))*(sj2)*(x771)*(x773)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((cj2)*(pz)*(x771)))+(((IkReal(-1.00000000000000))*(x772)*(x774)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(sj2)*(x771)*(x772)))+(((IkReal(-1.00000000000000))*(sj2)*(x771)*(x773)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((cj2)*(pz)*(x771)))+(((IkReal(-1.00000000000000))*(x772)*(x774)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x773)*(x774)))+(((IkReal(-1.00000000000000))*(sj2)*(x771)*(x772)))+(((IkReal(-1.00000000000000))*(sj2)*(x771)*(x773)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x775=((cj1)*(sj2));
IkReal x776=((cj2)*(sj1));
IkReal x777=((cj0)*(px));
IkReal x778=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x776)*(x778)))+(((x776)*(x777)))+(((IkReal(-0.0750000000000000))*(x775)))+(((IkReal(-0.0750000000000000))*(x776)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((x775)*(x777)))+(((x775)*(x778))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x779=((cj1)*(sj2));
IkReal x780=((cj0)*(r00));
IkReal x781=((IkReal(1.00000000000000))*(pz));
IkReal x782=((sj1)*(sj2));
IkReal x783=((r01)*(sj0));
IkReal x784=((IkReal(1.00000000000000))*(r02));
IkReal x785=((cj1)*(cj2));
IkReal x786=((cj2)*(sj1));
IkReal x787=((IkReal(1.00000000000000))*(cj0));
IkReal x788=((IkReal(1.00000000000000))*(sj0)*(x785));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x787))));
evalcond[2]=((IkReal(1.00000000000000))+(((x782)*(x783)))+(((IkReal(-1.00000000000000))*(x780)*(x785)))+(((IkReal(-1.00000000000000))*(x784)*(x786)))+(((x780)*(x782)))+(((IkReal(-1.00000000000000))*(x783)*(x785)))+(((IkReal(-1.00000000000000))*(x779)*(x784))));
evalcond[3]=((((x783)*(x786)))+(((x779)*(x780)))+(((x779)*(x783)))+(((r02)*(x782)))+(((IkReal(-1.00000000000000))*(x784)*(x785)))+(((x780)*(x786))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x781)*(x786)))+(((py)*(sj0)*(x782)))+(((IkReal(-1.00000000000000))*(py)*(x788)))+(((IkReal(-1.00000000000000))*(px)*(x785)*(x787)))+(((IkReal(0.0750000000000000))*(x785)))+(((cj0)*(px)*(x782)))+(((IkReal(-1.00000000000000))*(x779)*(x781)))+(((IkReal(-0.0750000000000000))*(x782))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x789=((IkReal(12.5000000000000))*(sj2));
IkReal x790=((py)*(sj0));
IkReal x791=((cj2)*(sj1));
IkReal x792=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x792))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x789)*(x790)))+(((pz)*(sj1)*(x789)))+(((IkReal(12.5000000000000))*(x790)*(x791)))+(((cj0)*(cj1)*(px)*(x789)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x791)*(x792)))+(((IkReal(-0.937500000000000))*(x791))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x792)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x789)*(x790)))+(((pz)*(sj1)*(x789)))+(((IkReal(12.5000000000000))*(x790)*(x791)))+(((cj0)*(cj1)*(px)*(x789)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x791)*(x792)))+(((IkReal(-0.937500000000000))*(x791)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x792)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((cj1)*(x789)*(x790)))+(((pz)*(sj1)*(x789)))+(((IkReal(12.5000000000000))*(x790)*(x791)))+(((cj0)*(cj1)*(px)*(x789)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x791)*(x792)))+(((IkReal(-0.937500000000000))*(x791)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x793=((cj1)*(sj2));
IkReal x794=((cj2)*(sj1));
IkReal x795=((cj0)*(px));
IkReal x796=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((x793)*(x795)))+(((x793)*(x796)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((x794)*(x796)))+(((x794)*(x795)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-0.0750000000000000))*(x793)))+(((IkReal(-0.0750000000000000))*(x794))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x797=((IkReal(1.00000000000000))*(sj2));
IkReal x798=((cj0)*(r00));
IkReal x799=((r01)*(sj0));
IkReal x800=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(x797)*(x798)))+(((IkReal(-1.00000000000000))*(cj1)*(x797)*(x799)))+(((IkReal(-1.00000000000000))*(x798)*(x800)))+(((IkReal(-1.00000000000000))*(x799)*(x800)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x797)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst58)*(((((IkReal(-1.00000000000000))*(cj1)*(x797)*(x798)))+(((IkReal(-1.00000000000000))*(cj1)*(x797)*(x799)))+(((IkReal(-1.00000000000000))*(x798)*(x800)))+(((IkReal(-1.00000000000000))*(x799)*(x800)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x797)))+(((cj1)*(cj2)*(r02)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x801=IKsin(j3);
IkReal x802=IKcos(j3);
IkReal x803=((sj1)*(sj2));
IkReal x804=((cj2)*(sj1));
IkReal x805=((IkReal(0.0800000000000000))*(sj4));
IkReal x806=((py)*(sj0));
IkReal x807=((IkReal(1.00000000000000))*(cj0));
IkReal x808=((r01)*(sj0));
IkReal x809=((cj1)*(sj2));
IkReal x810=((cj0)*(x809));
IkReal x811=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x812=((cj4)*(x801));
IkReal x813=((cj4)*(x802));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x807)))+(x812));
evalcond[1]=((((IkReal(-0.790000000000000))*(x812)))+(((x801)*(x805)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x807))));
evalcond[2]=((((x804)*(x808)))+(((r00)*(x810)))+(((r02)*(x803)))+(((cj0)*(r00)*(x804)))+(x813)+(((IkReal(-1.00000000000000))*(r02)*(x811)))+(((x808)*(x809))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x804)*(x806)))+(((x806)*(x809)))+(((px)*(x810)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x813)))+(((IkReal(-1.00000000000000))*(pz)*(x811)))+(((IkReal(-0.0750000000000000))*(x809)))+(((IkReal(-0.0750000000000000))*(x804)))+(((cj0)*(px)*(x804)))+(((pz)*(x803)))+(((x802)*(x805))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x814=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x815=((r00)*(sj0));
IkReal x816=((cj4)*(cj5));
IkReal x817=((IkReal(8.00000000000000))*(sj4));
IkReal x818=((cj0)*(r01));
IkReal x819=((IkReal(-0.125000000000000))*(x814));
IkReal x820=((IkReal(100.000000000000))*(cj0)*(py));
IkReal x821=((IkReal(100.000000000000))*(px)*(sj0));
if( IKabs(((x819)*(((((IkReal(-79.0000000000000))*(x818)))+(((IkReal(79.0000000000000))*(x815)))+(((IkReal(-1.00000000000000))*(x820)))+(x821))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x819)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x816)*(x821)))+(((IkReal(79.0000000000000))*(x816)*(x818)))+(((IkReal(-1.00000000000000))*(x815)*(x817)))+(((x816)*(x820)))+(((x817)*(x818)))+(((IkReal(-79.0000000000000))*(x815)*(x816))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x819)*(((((IkReal(-79.0000000000000))*(x818)))+(((IkReal(79.0000000000000))*(x815)))+(((IkReal(-1.00000000000000))*(x820)))+(x821)))))+IKsqr(((x819)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x816)*(x821)))+(((IkReal(79.0000000000000))*(x816)*(x818)))+(((IkReal(-1.00000000000000))*(x815)*(x817)))+(((x816)*(x820)))+(((x817)*(x818)))+(((IkReal(-79.0000000000000))*(x815)*(x816)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x819)*(((((IkReal(-79.0000000000000))*(x818)))+(((IkReal(79.0000000000000))*(x815)))+(((IkReal(-1.00000000000000))*(x820)))+(x821)))), ((x819)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x816)*(x821)))+(((IkReal(79.0000000000000))*(x816)*(x818)))+(((IkReal(-1.00000000000000))*(x815)*(x817)))+(((x816)*(x820)))+(((x817)*(x818)))+(((IkReal(-79.0000000000000))*(x815)*(x816)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x822=IKsin(j3);
IkReal x823=IKcos(j3);
IkReal x824=((sj1)*(sj2));
IkReal x825=((IkReal(0.0800000000000000))*(sj4));
IkReal x826=((cj2)*(sj1));
IkReal x827=((py)*(sj0));
IkReal x828=((IkReal(0.790000000000000))*(sj5));
IkReal x829=((IkReal(1.00000000000000))*(cj0));
IkReal x830=((r01)*(sj0));
IkReal x831=((cj1)*(sj2));
IkReal x832=((cj0)*(x831));
IkReal x833=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x834=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x835=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x829)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x823)))+(((IkReal(-1.00000000000000))*(x822)*(x833))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x829)))+(((x822)*(x834)))+(((x822)*(x825)))+(((x823)*(x828)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x835)))+(((IkReal(-1.00000000000000))*(x823)*(x833)))+(((x826)*(x830)))+(((r00)*(x832)))+(((sj5)*(x822)))+(((x830)*(x831)))+(((r02)*(x824)))+(((cj0)*(r00)*(x826))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x826)))+(((pz)*(x824)))+(((IkReal(-1.00000000000000))*(x822)*(x828)))+(((IkReal(-1.00000000000000))*(pz)*(x835)))+(((IkReal(-0.0750000000000000))*(x831)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x827)*(x831)))+(((x823)*(x834)))+(((x823)*(x825)))+(((cj0)*(px)*(x826)))+(((px)*(x832)))+(((x826)*(x827))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x836=((cj4)*(cj5));
IkReal x837=((r00)*(sj0));
IkReal x838=((cj0)*(r00));
IkReal x839=((r01)*(sj0));
IkReal x840=((IkReal(1.00000000000000))*(cj2));
IkReal x841=((cj1)*(r02));
IkReal x842=((sj1)*(sj5));
IkReal x843=((cj1)*(sj2));
IkReal x844=((cj2)*(sj1));
IkReal x845=((r02)*(sj2));
IkReal x846=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x847=((IkReal(1.00000000000000))*(sj5)*(x843));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x842)*(x845)))+(((IkReal(-1.00000000000000))*(x836)*(x846)))+(((IkReal(-1.00000000000000))*(x838)*(x840)*(x842)))+(((x836)*(x837)))+(((IkReal(-1.00000000000000))*(x839)*(x840)*(x842)))+(((cj2)*(sj5)*(x841)))+(((IkReal(-1.00000000000000))*(x839)*(x847)))+(((IkReal(-1.00000000000000))*(x838)*(x847))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x836)*(x840)*(x841)))+(((x836)*(x839)*(x843)))+(((x836)*(x839)*(x844)))+(((x836)*(x838)*(x844)))+(((x836)*(x838)*(x843)))+(((sj1)*(x836)*(x845)))+(((sj5)*(x837)))+(((IkReal(-1.00000000000000))*(sj5)*(x846))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(x842)*(x845)))+(((IkReal(-1.00000000000000))*(x836)*(x846)))+(((IkReal(-1.00000000000000))*(x838)*(x840)*(x842)))+(((x836)*(x837)))+(((IkReal(-1.00000000000000))*(x839)*(x840)*(x842)))+(((cj2)*(sj5)*(x841)))+(((IkReal(-1.00000000000000))*(x839)*(x847)))+(((IkReal(-1.00000000000000))*(x838)*(x847)))))), ((gconst54)*(((((IkReal(-1.00000000000000))*(x836)*(x840)*(x841)))+(((x836)*(x839)*(x843)))+(((x836)*(x839)*(x844)))+(((x836)*(x838)*(x844)))+(((x836)*(x838)*(x843)))+(((sj1)*(x836)*(x845)))+(((sj5)*(x837)))+(((IkReal(-1.00000000000000))*(sj5)*(x846)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x848=IKsin(j3);
IkReal x849=IKcos(j3);
IkReal x850=((sj1)*(sj2));
IkReal x851=((IkReal(0.0800000000000000))*(sj4));
IkReal x852=((cj2)*(sj1));
IkReal x853=((py)*(sj0));
IkReal x854=((IkReal(0.790000000000000))*(sj5));
IkReal x855=((IkReal(1.00000000000000))*(cj0));
IkReal x856=((r01)*(sj0));
IkReal x857=((cj1)*(sj2));
IkReal x858=((cj0)*(x857));
IkReal x859=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x860=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x861=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj5)*(x849)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x855)))+(((IkReal(-1.00000000000000))*(x848)*(x859))));
evalcond[1]=((((x848)*(x851)))+(((x848)*(x860)))+(((IkReal(-1.00000000000000))*(py)*(x855)))+(((px)*(sj0)))+(((x849)*(x854))));
evalcond[2]=((((r00)*(x858)))+(((IkReal(-1.00000000000000))*(x849)*(x859)))+(((x856)*(x857)))+(((sj5)*(x848)))+(((r02)*(x850)))+(((cj0)*(r00)*(x852)))+(((x852)*(x856)))+(((IkReal(-1.00000000000000))*(r02)*(x861))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj0)*(px)*(x852)))+(((IkReal(-1.00000000000000))*(pz)*(x861)))+(((IkReal(-1.00000000000000))*(x848)*(x854)))+(((px)*(x858)))+(((x852)*(x853)))+(((x849)*(x860)))+(((IkReal(-0.0750000000000000))*(x857)))+(((IkReal(-0.0750000000000000))*(x852)))+(((x849)*(x851)))+(((x853)*(x857)))+(((pz)*(x850))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x862=((IkReal(0.790000000000000))*(cj0));
IkReal x863=((cj2)*(sj1));
IkReal x864=((cj1)*(cj2));
IkReal x865=((IkReal(0.790000000000000))*(r02));
IkReal x866=((IkReal(0.790000000000000))*(sj0));
IkReal x867=((cj1)*(sj2));
IkReal x868=((sj1)*(sj2));
IkReal x869=((IkReal(1.00000000000000))*(cj0)*(px));
IkReal x870=((IkReal(1.00000000000000))*(py)*(sj0));
if( IKabs(((IkReal(-25.0000000000000))*(gconst53)*(((((r00)*(x866)))+(((IkReal(-1.00000000000000))*(r01)*(x862)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(25.0000000000000))*(gconst53)*(((IkReal(0.0900000000000000))+(((IkReal(-1.00000000000000))*(x865)*(x868)))+(((IkReal(-1.00000000000000))*(r01)*(x866)*(x867)))+(((IkReal(0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(r01)*(x863)*(x866)))+(((IkReal(-1.00000000000000))*(pz)*(x868)))+(((IkReal(0.0750000000000000))*(x863)))+(((IkReal(0.0750000000000000))*(x867)))+(((IkReal(-1.00000000000000))*(x867)*(x870)))+(((pz)*(x864)))+(((IkReal(-1.00000000000000))*(x867)*(x869)))+(((x864)*(x865)))+(((IkReal(-1.00000000000000))*(x863)*(x870)))+(((IkReal(-1.00000000000000))*(x863)*(x869)))+(((IkReal(-1.00000000000000))*(r00)*(x862)*(x863)))+(((IkReal(-1.00000000000000))*(r00)*(x862)*(x867))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-25.0000000000000))*(gconst53)*(((((r00)*(x866)))+(((IkReal(-1.00000000000000))*(r01)*(x862)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py)))))), ((IkReal(25.0000000000000))*(gconst53)*(((IkReal(0.0900000000000000))+(((IkReal(-1.00000000000000))*(x865)*(x868)))+(((IkReal(-1.00000000000000))*(r01)*(x866)*(x867)))+(((IkReal(0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(r01)*(x863)*(x866)))+(((IkReal(-1.00000000000000))*(pz)*(x868)))+(((IkReal(0.0750000000000000))*(x863)))+(((IkReal(0.0750000000000000))*(x867)))+(((IkReal(-1.00000000000000))*(x867)*(x870)))+(((pz)*(x864)))+(((IkReal(-1.00000000000000))*(x867)*(x869)))+(((x864)*(x865)))+(((IkReal(-1.00000000000000))*(x863)*(x870)))+(((IkReal(-1.00000000000000))*(x863)*(x869)))+(((IkReal(-1.00000000000000))*(r00)*(x862)*(x863)))+(((IkReal(-1.00000000000000))*(r00)*(x862)*(x867)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x871=IKsin(j3);
IkReal x872=IKcos(j3);
IkReal x873=((IkReal(0.790000000000000))*(cj0));
IkReal x874=((cj2)*(sj1));
IkReal x875=((IkReal(0.365000000000000))*(cj2));
IkReal x876=((sj1)*(sj2));
IkReal x877=((cj4)*(pz));
IkReal x878=((px)*(sj0));
IkReal x879=((cj4)*(r00));
IkReal x880=((IkReal(0.790000000000000))*(r02));
IkReal x881=((cj1)*(cj4));
IkReal x882=((cj0)*(px));
IkReal x883=((py)*(sj0));
IkReal x884=((cj1)*(cj2));
IkReal x885=((IkReal(0.0750000000000000))*(sj2));
IkReal x886=((IkReal(0.790000000000000))*(sj0));
IkReal x887=((cj1)*(sj2));
IkReal x888=((r01)*(x886));
IkReal x889=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x890=((IkReal(0.0800000000000000))*(sj4)*(x871));
IkReal x891=((IkReal(0.0800000000000000))*(sj4)*(x872));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x873)))+(((r00)*(x886)))+(((IkReal(-1.00000000000000))*(x889)))+(x878)+(x890));
evalcond[1]=((((x879)*(x886)))+(((IkReal(-1.00000000000000))*(cj4)*(r01)*(x873)))+(((cj4)*(x890)))+(((cj4)*(x878)))+(((IkReal(-1.00000000000000))*(cj4)*(x889))));
evalcond[2]=((IkReal(0.0900000000000000))+(((IkReal(-1.00000000000000))*(x887)*(x888)))+(((IkReal(-1.00000000000000))*(x883)*(x887)))+(((IkReal(-1.00000000000000))*(x882)*(x887)))+(((pz)*(x884)))+(((x880)*(x884)))+(((IkReal(-1.00000000000000))*(x891)))+(((IkReal(-1.00000000000000))*(x876)*(x880)))+(((IkReal(-1.00000000000000))*(pz)*(x876)))+(((IkReal(-1.00000000000000))*(r00)*(x873)*(x874)))+(((IkReal(-1.00000000000000))*(x874)*(x882)))+(((IkReal(-1.00000000000000))*(x874)*(x883)))+(((cj1)*(x885)))+(((IkReal(0.0750000000000000))*(x874)))+(x875)+(((IkReal(-1.00000000000000))*(r00)*(x873)*(x887)))+(((IkReal(-1.00000000000000))*(x874)*(x888))));
evalcond[3]=((((x873)*(x874)*(x879)))+(((x876)*(x877)))+(((IkReal(-1.00000000000000))*(x881)*(x885)))+(((cj4)*(x891)))+(((sj2)*(x881)*(x888)))+(((sj2)*(x881)*(x883)))+(((sj2)*(x881)*(x882)))+(((IkReal(-1.00000000000000))*(x877)*(x884)))+(((IkReal(-0.0750000000000000))*(cj4)*(x874)))+(((cj4)*(x876)*(x880)))+(((IkReal(-1.00000000000000))*(cj4)*(x875)))+(((cj4)*(x874)*(x883)))+(((cj4)*(x874)*(x882)))+(((cj4)*(x874)*(x888)))+(((x873)*(x879)*(x887)))+(((IkReal(-0.0900000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(x880)*(x881))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x892=(sj3)*(sj3);
IkReal x893=(cj3)*(cj3);
dummyeval[0]=((x892)+(x893));
dummyeval[1]=((((cj4)*(x893)))+(((cj4)*(x892))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x894=(sj3)*(sj3);
IkReal x895=(cj3)*(cj3);
dummyeval[0]=((((IkReal(-1.00000000000000))*(x894)))+(((IkReal(-1.00000000000000))*(x895))));
dummyeval[1]=((((cj4)*(x895)))+(((cj4)*(x894))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x896=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x896))));
evalcond[2]=((IkReal(0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x896)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x897=((r02)*(sj1));
IkReal x898=((IkReal(1.00000000000000))*(sj2));
IkReal x899=((IkReal(1.00000000000000))*(cj2));
IkReal x900=((cj1)*(r02));
IkReal x901=((cj0)*(cj1)*(r00));
IkReal x902=((cj0)*(r00)*(sj1));
IkReal x903=((r01)*(sj0)*(sj1));
IkReal x904=((cj1)*(r01)*(sj0));
if( IKabs(((((cj2)*(x900)))+(((IkReal(-1.00000000000000))*(x898)*(x904)))+(((IkReal(-1.00000000000000))*(x898)*(x901)))+(((IkReal(-1.00000000000000))*(x897)*(x898)))+(((IkReal(-1.00000000000000))*(x899)*(x902)))+(((IkReal(-1.00000000000000))*(x899)*(x903))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj2)*(x897)))+(((sj2)*(x900)))+(((cj2)*(x904)))+(((cj2)*(x901)))+(((IkReal(-1.00000000000000))*(x898)*(x903)))+(((IkReal(-1.00000000000000))*(x898)*(x902))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(x900)))+(((IkReal(-1.00000000000000))*(x898)*(x904)))+(((IkReal(-1.00000000000000))*(x898)*(x901)))+(((IkReal(-1.00000000000000))*(x897)*(x898)))+(((IkReal(-1.00000000000000))*(x899)*(x902)))+(((IkReal(-1.00000000000000))*(x899)*(x903)))))+IKsqr(((((cj2)*(x897)))+(((sj2)*(x900)))+(((cj2)*(x904)))+(((cj2)*(x901)))+(((IkReal(-1.00000000000000))*(x898)*(x903)))+(((IkReal(-1.00000000000000))*(x898)*(x902)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(x900)))+(((IkReal(-1.00000000000000))*(x898)*(x904)))+(((IkReal(-1.00000000000000))*(x898)*(x901)))+(((IkReal(-1.00000000000000))*(x897)*(x898)))+(((IkReal(-1.00000000000000))*(x899)*(x902)))+(((IkReal(-1.00000000000000))*(x899)*(x903)))), ((((cj2)*(x897)))+(((sj2)*(x900)))+(((cj2)*(x904)))+(((cj2)*(x901)))+(((IkReal(-1.00000000000000))*(x898)*(x903)))+(((IkReal(-1.00000000000000))*(x898)*(x902)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x905=IKcos(j5);
IkReal x906=IKsin(j5);
IkReal x907=((pz)*(sj2));
IkReal x908=((IkReal(1.00000000000000))*(cj1));
IkReal x909=((cj2)*(sj0));
IkReal x910=((cj2)*(sj1));
IkReal x911=((cj0)*(px));
IkReal x912=((py)*(sj1));
IkReal x913=((sj1)*(sj2));
IkReal x914=((r01)*(sj0));
IkReal x915=((sj0)*(sj2));
IkReal x916=((IkReal(0.0750000000000000))*(cj1));
IkReal x917=((cj1)*(sj2));
IkReal x918=((cj0)*(r00));
IkReal x919=((sj2)*(x918));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)*(x910)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x908)))+(((IkReal(-1.00000000000000))*(r01)*(x908)*(x909)))+(((x913)*(x914)))+(((x913)*(x918)))+(x905)+(((IkReal(-1.00000000000000))*(cj2)*(x908)*(x918))));
evalcond[1]=((((x917)*(x918)))+(((x910)*(x918)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x908)))+(((x914)*(x917)))+(((r02)*(x913)))+(((r01)*(sj1)*(x909)))+(x906));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((cj2)*(x916)))+(((x911)*(x913)))+(((IkReal(-1.00000000000000))*(py)*(x908)*(x909)))+(((IkReal(-0.790000000000000))*(x905)))+(((x912)*(x915)))+(((IkReal(-0.0750000000000000))*(x913)))+(((IkReal(-1.00000000000000))*(pz)*(x910)))+(((IkReal(-1.00000000000000))*(x907)*(x908)))+(((IkReal(-1.00000000000000))*(cj2)*(x908)*(x911))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x910)*(x911)))+(((x909)*(x912)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x908)))+(((x911)*(x917)))+(((IkReal(-1.00000000000000))*(sj2)*(x916)))+(((cj1)*(py)*(x915)))+(((IkReal(-0.790000000000000))*(x906)))+(((IkReal(-0.0750000000000000))*(x910)))+(((sj1)*(x907))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x920=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x920))));
evalcond[2]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x920)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x921=((sj1)*(sj2));
IkReal x922=((IkReal(1.00000000000000))*(r02));
IkReal x923=((r01)*(sj0));
IkReal x924=((cj0)*(r00));
IkReal x925=((cj1)*(sj2));
IkReal x926=((cj2)*(sj1));
IkReal x927=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((IkReal(-1.00000000000000))*(x923)*(x925)))+(((IkReal(-1.00000000000000))*(x923)*(x926)))+(((IkReal(-1.00000000000000))*(x921)*(x922)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x924)*(x926)))+(((IkReal(-1.00000000000000))*(x924)*(x925))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x924)*(x927)))+(((x921)*(x923)))+(((x921)*(x924)))+(((IkReal(-1.00000000000000))*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(x922)*(x925)))+(((IkReal(-1.00000000000000))*(x922)*(x926))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x923)*(x925)))+(((IkReal(-1.00000000000000))*(x923)*(x926)))+(((IkReal(-1.00000000000000))*(x921)*(x922)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x924)*(x926)))+(((IkReal(-1.00000000000000))*(x924)*(x925)))))+IKsqr(((((IkReal(-1.00000000000000))*(x924)*(x927)))+(((x921)*(x923)))+(((x921)*(x924)))+(((IkReal(-1.00000000000000))*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(x922)*(x925)))+(((IkReal(-1.00000000000000))*(x922)*(x926)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x923)*(x925)))+(((IkReal(-1.00000000000000))*(x923)*(x926)))+(((IkReal(-1.00000000000000))*(x921)*(x922)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(x924)*(x926)))+(((IkReal(-1.00000000000000))*(x924)*(x925)))), ((((IkReal(-1.00000000000000))*(x924)*(x927)))+(((x921)*(x923)))+(((x921)*(x924)))+(((IkReal(-1.00000000000000))*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(x922)*(x925)))+(((IkReal(-1.00000000000000))*(x922)*(x926)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x928=IKcos(j5);
IkReal x929=IKsin(j5);
IkReal x930=((pz)*(sj2));
IkReal x931=((IkReal(1.00000000000000))*(cj1));
IkReal x932=((cj2)*(sj0));
IkReal x933=((cj2)*(sj1));
IkReal x934=((cj0)*(px));
IkReal x935=((py)*(sj1));
IkReal x936=((sj1)*(sj2));
IkReal x937=((r01)*(sj0));
IkReal x938=((sj0)*(sj2));
IkReal x939=((IkReal(0.0750000000000000))*(cj1));
IkReal x940=((cj1)*(sj2));
IkReal x941=((cj0)*(r00));
IkReal x942=((sj2)*(x941));
evalcond[0]=((((x936)*(x937)))+(((x936)*(x941)))+(((IkReal(-1.00000000000000))*(cj2)*(x931)*(x941)))+(((IkReal(-1.00000000000000))*(r01)*(x931)*(x932)))+(((IkReal(-1.00000000000000))*(x928)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x931)))+(((IkReal(-1.00000000000000))*(r02)*(x933))));
evalcond[1]=((((x933)*(x941)))+(((x937)*(x940)))+(((r01)*(sj1)*(x932)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x931)))+(((x940)*(x941)))+(x929)+(((r02)*(x936))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x930)*(x931)))+(((IkReal(0.790000000000000))*(x928)))+(((x935)*(x938)))+(((IkReal(-0.0750000000000000))*(x936)))+(((cj2)*(x939)))+(((IkReal(-1.00000000000000))*(cj2)*(x931)*(x934)))+(((IkReal(-1.00000000000000))*(pz)*(x933)))+(((x934)*(x936)))+(((IkReal(-1.00000000000000))*(py)*(x931)*(x932))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x931)))+(((IkReal(-1.00000000000000))*(sj2)*(x939)))+(((IkReal(-0.790000000000000))*(x929)))+(((IkReal(-0.365000000000000))*(cj2)))+(((sj1)*(x930)))+(((IkReal(-0.0750000000000000))*(x933)))+(((cj1)*(py)*(x938)))+(((x934)*(x940)))+(((x933)*(x934)))+(((x932)*(x935))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x943=((cj0)*(px));
IkReal x944=((py)*(sj0));
IkReal x945=((cj1)*(sj2));
IkReal x946=((IkReal(1.26582278481013))*(pz));
IkReal x947=((IkReal(1.26582278481013))*(cj2)*(sj1));
if( IKabs(((IkReal(-0.113924050632911))+(((IkReal(1.26582278481013))*(x944)*(x945)))+(((IkReal(1.26582278481013))*(x943)*(x945)))+(((IkReal(-0.462025316455696))*(cj2)))+(((x943)*(x947)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x946)))+(((IkReal(-0.0949367088607595))*(x945)))+(((x944)*(x947)))+(((IkReal(-0.0949367088607595))*(cj2)*(sj1)))+(((sj1)*(sj2)*(x946))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.113924050632911))+(((IkReal(1.26582278481013))*(x944)*(x945)))+(((IkReal(1.26582278481013))*(x943)*(x945)))+(((IkReal(-0.462025316455696))*(cj2)))+(((x943)*(x947)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x946)))+(((IkReal(-0.0949367088607595))*(x945)))+(((x944)*(x947)))+(((IkReal(-0.0949367088607595))*(cj2)*(sj1)))+(((sj1)*(sj2)*(x946)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-0.113924050632911))+(((IkReal(1.26582278481013))*(x944)*(x945)))+(((IkReal(1.26582278481013))*(x943)*(x945)))+(((IkReal(-0.462025316455696))*(cj2)))+(((x943)*(x947)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x946)))+(((IkReal(-0.0949367088607595))*(x945)))+(((x944)*(x947)))+(((IkReal(-0.0949367088607595))*(cj2)*(sj1)))+(((sj1)*(sj2)*(x946)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x948=IKcos(j5);
IkReal x949=IKsin(j5);
IkReal x950=((pz)*(sj2));
IkReal x951=((IkReal(1.00000000000000))*(cj1));
IkReal x952=((cj2)*(sj0));
IkReal x953=((cj2)*(sj1));
IkReal x954=((cj0)*(px));
IkReal x955=((py)*(sj1));
IkReal x956=((sj1)*(sj2));
IkReal x957=((r01)*(sj0));
IkReal x958=((sj0)*(sj2));
IkReal x959=((IkReal(0.0750000000000000))*(cj1));
IkReal x960=((cj1)*(sj2));
IkReal x961=((IkReal(1.00000000000000))*(cj0));
IkReal x962=((cj0)*(r00));
IkReal x963=((sj2)*(x962));
IkReal x964=((sj4)*(x948));
IkReal x965=((cj4)*(x948));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x965)))+(((IkReal(-1.00000000000000))*(r01)*(x961))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x961)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x965))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(x951)*(x962)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x951)))+(((x956)*(x957)))+(((x956)*(x962)))+(((IkReal(-1.00000000000000))*(r01)*(x951)*(x952)))+(x964)+(((IkReal(-1.00000000000000))*(r02)*(x953))));
evalcond[3]=((((x960)*(x962)))+(((r01)*(sj1)*(x952)))+(((x957)*(x960)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x951)))+(((r02)*(x956)))+(x949)+(((x953)*(x962))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x955)*(x958)))+(((IkReal(-0.0750000000000000))*(x956)))+(((IkReal(-1.00000000000000))*(cj2)*(x951)*(x954)))+(((IkReal(-0.790000000000000))*(x964)))+(((cj2)*(x959)))+(((IkReal(-1.00000000000000))*(x950)*(x951)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(pz)*(x953)))+(((IkReal(-1.00000000000000))*(py)*(x951)*(x952)))+(((x954)*(x956))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x954)*(x960)))+(((cj1)*(py)*(x958)))+(((x952)*(x955)))+(((IkReal(-0.0750000000000000))*(x953)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x951)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x949)))+(((sj1)*(x950)))+(((IkReal(-1.00000000000000))*(sj2)*(x959)))+(((x953)*(x954))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x966=((IkReal(1.00000000000000))*(sj2));
IkReal x967=((cj0)*(r00));
IkReal x968=((r01)*(sj0));
IkReal x969=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x967)*(x969)))+(((IkReal(-1.00000000000000))*(cj1)*(x966)*(x967)))+(((IkReal(-1.00000000000000))*(cj1)*(x966)*(x968)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x966)))+(((IkReal(-1.00000000000000))*(x968)*(x969)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x967)*(x969)))+(((IkReal(-1.00000000000000))*(cj1)*(x966)*(x967)))+(((IkReal(-1.00000000000000))*(cj1)*(x966)*(x968)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x966)))+(((IkReal(-1.00000000000000))*(x968)*(x969)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(100.000000000000))*(cj0)*(py)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x967)*(x969)))+(((IkReal(-1.00000000000000))*(cj1)*(x966)*(x967)))+(((IkReal(-1.00000000000000))*(cj1)*(x966)*(x968)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x966)))+(((IkReal(-1.00000000000000))*(x968)*(x969)))+(((cj1)*(cj2)*(r02)))), ((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(100.000000000000))*(cj0)*(py)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x970=IKcos(j5);
IkReal x971=IKsin(j5);
IkReal x972=((pz)*(sj2));
IkReal x973=((IkReal(1.00000000000000))*(cj1));
IkReal x974=((cj2)*(sj0));
IkReal x975=((cj2)*(sj1));
IkReal x976=((cj0)*(px));
IkReal x977=((py)*(sj1));
IkReal x978=((sj1)*(sj2));
IkReal x979=((r01)*(sj0));
IkReal x980=((sj0)*(sj2));
IkReal x981=((IkReal(0.0750000000000000))*(cj1));
IkReal x982=((cj1)*(sj2));
IkReal x983=((IkReal(1.00000000000000))*(cj0));
IkReal x984=((cj0)*(r00));
IkReal x985=((sj2)*(x984));
IkReal x986=((sj4)*(x970));
IkReal x987=((cj4)*(x970));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x987)))+(((IkReal(-1.00000000000000))*(r01)*(x983))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x983)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x987))));
evalcond[2]=((((x978)*(x979)))+(((IkReal(-1.00000000000000))*(r02)*(x975)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x973)))+(x986)+(((IkReal(-1.00000000000000))*(r01)*(x973)*(x974)))+(((IkReal(-1.00000000000000))*(cj2)*(x973)*(x984)))+(((x978)*(x984))));
evalcond[3]=((((x975)*(x984)))+(((r01)*(sj1)*(x974)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x973)))+(((x979)*(x982)))+(((x982)*(x984)))+(((r02)*(x978)))+(x971));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x973)*(x976)))+(((IkReal(-0.790000000000000))*(x986)))+(((x977)*(x980)))+(((IkReal(-0.0750000000000000))*(x978)))+(((x976)*(x978)))+(((IkReal(-1.00000000000000))*(pz)*(x975)))+(((IkReal(-1.00000000000000))*(x972)*(x973)))+(((cj2)*(x981)))+(((IkReal(-1.00000000000000))*(py)*(x973)*(x974)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x973)))+(((x975)*(x976)))+(((IkReal(-0.790000000000000))*(x971)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x975)))+(((x976)*(x982)))+(((sj1)*(x972)))+(((x974)*(x977)))+(((IkReal(-1.00000000000000))*(sj2)*(x981)))+(((cj1)*(py)*(x980))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x988=((IkReal(1.00000000000000))*(sj2));
IkReal x989=((IkReal(1.00000000000000))*(r01));
IkReal x990=((cj0)*(r00));
IkReal x991=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x988)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x988)))+(((IkReal(-1.00000000000000))*(x990)*(x991)))+(((IkReal(-1.00000000000000))*(cj1)*(x988)*(x990)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(sj0)*(x989)*(x991))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x989))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x988)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x988)))+(((IkReal(-1.00000000000000))*(x990)*(x991)))+(((IkReal(-1.00000000000000))*(cj1)*(x988)*(x990)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(sj0)*(x989)*(x991)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x989)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x988)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x988)))+(((IkReal(-1.00000000000000))*(x990)*(x991)))+(((IkReal(-1.00000000000000))*(cj1)*(x988)*(x990)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(sj0)*(x989)*(x991)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x989)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x992=IKcos(j5);
IkReal x993=IKsin(j5);
IkReal x994=((pz)*(sj2));
IkReal x995=((IkReal(1.00000000000000))*(cj1));
IkReal x996=((cj2)*(sj0));
IkReal x997=((cj2)*(sj1));
IkReal x998=((cj0)*(px));
IkReal x999=((py)*(sj1));
IkReal x1000=((sj1)*(sj2));
IkReal x1001=((r01)*(sj0));
IkReal x1002=((sj0)*(sj2));
IkReal x1003=((IkReal(0.0750000000000000))*(cj1));
IkReal x1004=((cj1)*(sj2));
IkReal x1005=((IkReal(1.00000000000000))*(cj0));
IkReal x1006=((cj0)*(r00));
IkReal x1007=((sj2)*(x1006));
IkReal x1008=((sj4)*(x992));
IkReal x1009=((cj4)*(x992));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(r01)*(x1005))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x1005)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x1009))));
evalcond[2]=((x1008)+(((x1000)*(x1001)))+(((x1000)*(x1006)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x995)))+(((IkReal(-1.00000000000000))*(r02)*(x997)))+(((IkReal(-1.00000000000000))*(cj2)*(x1006)*(x995)))+(((IkReal(-1.00000000000000))*(r01)*(x995)*(x996))));
evalcond[3]=((((r01)*(sj1)*(x996)))+(((r02)*(x1000)))+(((x1004)*(x1006)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x995)))+(x993)+(((x1001)*(x1004)))+(((x1006)*(x997))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(x1008)))+(((x1000)*(x998)))+(((IkReal(-0.0750000000000000))*(x1000)))+(((cj2)*(x1003)))+(((IkReal(-1.00000000000000))*(x994)*(x995)))+(((IkReal(-1.00000000000000))*(pz)*(x997)))+(((x1002)*(x999)))+(((IkReal(-1.00000000000000))*(py)*(x995)*(x996)))+(((IkReal(-1.00000000000000))*(cj2)*(x995)*(x998)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x993)))+(((x997)*(x998)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1003)))+(((cj1)*(py)*(x1002)))+(((x1004)*(x998)))+(((x996)*(x999)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x995)))+(((sj1)*(x994)))+(((IkReal(-0.0750000000000000))*(x997))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x1010=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1010))));
evalcond[2]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1010)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1011=((r01)*(sj0));
IkReal x1012=((cj1)*(sj2));
IkReal x1013=((sj1)*(sj2));
IkReal x1014=((cj2)*(sj1));
IkReal x1015=((cj0)*(r00));
IkReal x1016=((cj1)*(cj2));
IkReal x1017=((cj1)*(x1015));
if( IKabs(((((x1014)*(x1015)))+(((r02)*(x1013)))+(((x1012)*(x1015)))+(((IkReal(-1.00000000000000))*(r02)*(x1016)))+(((x1011)*(x1014)))+(((x1011)*(x1012))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1011)*(x1013)))+(((r02)*(x1014)))+(((r02)*(x1012)))+(((IkReal(-1.00000000000000))*(x1013)*(x1015)))+(((x1015)*(x1016)))+(((x1011)*(x1016))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1014)*(x1015)))+(((r02)*(x1013)))+(((x1012)*(x1015)))+(((IkReal(-1.00000000000000))*(r02)*(x1016)))+(((x1011)*(x1014)))+(((x1011)*(x1012)))))+IKsqr(((((IkReal(-1.00000000000000))*(x1011)*(x1013)))+(((r02)*(x1014)))+(((r02)*(x1012)))+(((IkReal(-1.00000000000000))*(x1013)*(x1015)))+(((x1015)*(x1016)))+(((x1011)*(x1016)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1014)*(x1015)))+(((r02)*(x1013)))+(((x1012)*(x1015)))+(((IkReal(-1.00000000000000))*(r02)*(x1016)))+(((x1011)*(x1014)))+(((x1011)*(x1012)))), ((((IkReal(-1.00000000000000))*(x1011)*(x1013)))+(((r02)*(x1014)))+(((r02)*(x1012)))+(((IkReal(-1.00000000000000))*(x1013)*(x1015)))+(((x1015)*(x1016)))+(((x1011)*(x1016)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1018=IKcos(j5);
IkReal x1019=IKsin(j5);
IkReal x1020=((pz)*(sj2));
IkReal x1021=((IkReal(1.00000000000000))*(cj1));
IkReal x1022=((cj2)*(sj0));
IkReal x1023=((cj2)*(sj1));
IkReal x1024=((cj0)*(px));
IkReal x1025=((py)*(sj1));
IkReal x1026=((sj1)*(sj2));
IkReal x1027=((r01)*(sj0));
IkReal x1028=((sj0)*(sj2));
IkReal x1029=((IkReal(0.0750000000000000))*(cj1));
IkReal x1030=((cj1)*(sj2));
IkReal x1031=((cj0)*(r00));
IkReal x1032=((sj2)*(x1031));
evalcond[0]=((x1018)+(((x1026)*(x1031)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1021)))+(((x1026)*(x1027)))+(((IkReal(-1.00000000000000))*(r01)*(x1021)*(x1022)))+(((IkReal(-1.00000000000000))*(cj2)*(x1021)*(x1031)))+(((IkReal(-1.00000000000000))*(r02)*(x1023))));
evalcond[1]=((((x1030)*(x1031)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1021)))+(((x1023)*(x1031)))+(((x1027)*(x1030)))+(((r01)*(sj1)*(x1022)))+(((r02)*(x1026)))+(((IkReal(-1.00000000000000))*(x1019))));
evalcond[2]=((IkReal(0.405000000000000))+(((cj2)*(x1029)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x1021)*(x1022)))+(((IkReal(-0.0750000000000000))*(x1026)))+(((IkReal(-1.00000000000000))*(pz)*(x1023)))+(((IkReal(-1.00000000000000))*(x1020)*(x1021)))+(((IkReal(-1.00000000000000))*(cj2)*(x1021)*(x1024)))+(((x1025)*(x1028)))+(((x1024)*(x1026)))+(((IkReal(-0.790000000000000))*(x1018))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((sj1)*(x1020)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1021)))+(((IkReal(-0.0750000000000000))*(x1023)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1023)*(x1024)))+(((x1022)*(x1025)))+(((x1024)*(x1030)))+(((cj1)*(py)*(x1028)))+(((IkReal(0.790000000000000))*(x1019)))+(((IkReal(-1.00000000000000))*(sj2)*(x1029))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1033=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1033))));
evalcond[2]=((IkReal(0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1033)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1034=((IkReal(1.00000000000000))*(cj2));
IkReal x1035=((r02)*(sj2));
IkReal x1036=((r01)*(sj0));
IkReal x1037=((cj2)*(sj1));
IkReal x1038=((cj0)*(r00));
IkReal x1039=((sj2)*(x1036));
IkReal x1040=((cj1)*(x1038));
if( IKabs(((((x1036)*(x1037)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1034)))+(((sj2)*(x1040)))+(((sj1)*(x1035)))+(((x1037)*(x1038)))+(((cj1)*(x1039))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((sj1)*(sj2)*(x1038)))+(((sj1)*(x1039)))+(((IkReal(-1.00000000000000))*(cj1)*(x1035)))+(((IkReal(-1.00000000000000))*(cj1)*(x1034)*(x1036)))+(((IkReal(-1.00000000000000))*(x1034)*(x1040)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1034))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1036)*(x1037)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1034)))+(((sj2)*(x1040)))+(((sj1)*(x1035)))+(((x1037)*(x1038)))+(((cj1)*(x1039)))))+IKsqr(((((sj1)*(sj2)*(x1038)))+(((sj1)*(x1039)))+(((IkReal(-1.00000000000000))*(cj1)*(x1035)))+(((IkReal(-1.00000000000000))*(cj1)*(x1034)*(x1036)))+(((IkReal(-1.00000000000000))*(x1034)*(x1040)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1034)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1036)*(x1037)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1034)))+(((sj2)*(x1040)))+(((sj1)*(x1035)))+(((x1037)*(x1038)))+(((cj1)*(x1039)))), ((((sj1)*(sj2)*(x1038)))+(((sj1)*(x1039)))+(((IkReal(-1.00000000000000))*(cj1)*(x1035)))+(((IkReal(-1.00000000000000))*(cj1)*(x1034)*(x1036)))+(((IkReal(-1.00000000000000))*(x1034)*(x1040)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1034)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1041=IKcos(j5);
IkReal x1042=IKsin(j5);
IkReal x1043=((pz)*(sj2));
IkReal x1044=((IkReal(1.00000000000000))*(cj1));
IkReal x1045=((cj2)*(sj0));
IkReal x1046=((cj2)*(sj1));
IkReal x1047=((cj0)*(px));
IkReal x1048=((py)*(sj1));
IkReal x1049=((sj1)*(sj2));
IkReal x1050=((r01)*(sj0));
IkReal x1051=((sj0)*(sj2));
IkReal x1052=((IkReal(0.0750000000000000))*(cj1));
IkReal x1053=((cj1)*(sj2));
IkReal x1054=((cj0)*(r00));
IkReal x1055=((sj2)*(x1054));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(x1044)*(x1054)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1044)))+(((IkReal(-1.00000000000000))*(r02)*(x1046)))+(((x1049)*(x1054)))+(((x1049)*(x1050)))+(((IkReal(-1.00000000000000))*(x1041)))+(((IkReal(-1.00000000000000))*(r01)*(x1044)*(x1045))));
evalcond[1]=((((r02)*(x1049)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1044)))+(((x1050)*(x1053)))+(((x1053)*(x1054)))+(((IkReal(-1.00000000000000))*(x1042)))+(((x1046)*(x1054)))+(((r01)*(sj1)*(x1045))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.790000000000000))*(x1041)))+(((IkReal(-1.00000000000000))*(pz)*(x1046)))+(((IkReal(-1.00000000000000))*(cj2)*(x1044)*(x1047)))+(((IkReal(-1.00000000000000))*(x1043)*(x1044)))+(((x1048)*(x1051)))+(((IkReal(-1.00000000000000))*(py)*(x1044)*(x1045)))+(((cj2)*(x1052)))+(((x1047)*(x1049)))+(((IkReal(-0.0750000000000000))*(x1049))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(0.790000000000000))*(x1042)))+(((cj1)*(py)*(x1051)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1052)))+(((sj1)*(x1043)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1044)))+(((x1047)*(x1053)))+(((IkReal(-0.0750000000000000))*(x1046)))+(((x1045)*(x1048)))+(((x1046)*(x1047))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1056=((cj0)*(px));
IkReal x1057=((py)*(sj0));
IkReal x1058=((cj1)*(sj2));
IkReal x1059=((IkReal(1.26582278481013))*(pz));
IkReal x1060=((IkReal(1.26582278481013))*(cj2)*(sj1));
if( IKabs(((IkReal(0.113924050632911))+(((IkReal(0.0949367088607595))*(x1058)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(x1057)*(x1060)))+(((IkReal(-1.00000000000000))*(x1056)*(x1060)))+(((cj1)*(cj2)*(x1059)))+(((IkReal(-1.26582278481013))*(x1057)*(x1058)))+(((IkReal(0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(-1.26582278481013))*(x1056)*(x1058)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1059))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.113924050632911))+(((IkReal(0.0949367088607595))*(x1058)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(x1057)*(x1060)))+(((IkReal(-1.00000000000000))*(x1056)*(x1060)))+(((cj1)*(cj2)*(x1059)))+(((IkReal(-1.26582278481013))*(x1057)*(x1058)))+(((IkReal(0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(-1.26582278481013))*(x1056)*(x1058)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1059)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.113924050632911))+(((IkReal(0.0949367088607595))*(x1058)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(x1057)*(x1060)))+(((IkReal(-1.00000000000000))*(x1056)*(x1060)))+(((cj1)*(cj2)*(x1059)))+(((IkReal(-1.26582278481013))*(x1057)*(x1058)))+(((IkReal(0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(-1.26582278481013))*(x1056)*(x1058)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1059)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1061=IKcos(j5);
IkReal x1062=IKsin(j5);
IkReal x1063=((pz)*(sj2));
IkReal x1064=((IkReal(1.00000000000000))*(cj1));
IkReal x1065=((cj2)*(sj0));
IkReal x1066=((cj2)*(sj1));
IkReal x1067=((cj0)*(px));
IkReal x1068=((py)*(sj1));
IkReal x1069=((sj1)*(sj2));
IkReal x1070=((r01)*(sj0));
IkReal x1071=((sj0)*(sj2));
IkReal x1072=((IkReal(0.0750000000000000))*(cj1));
IkReal x1073=((cj1)*(sj2));
IkReal x1074=((IkReal(1.00000000000000))*(cj0));
IkReal x1075=((cj0)*(r00));
IkReal x1076=((sj2)*(x1075));
IkReal x1077=((sj4)*(x1061));
IkReal x1078=((cj4)*(x1061));
evalcond[0]=((((r00)*(sj0)))+(x1078)+(((IkReal(-1.00000000000000))*(r01)*(x1074))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1074)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((IkReal(-0.790000000000000))*(x1078))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1064)))+(((x1069)*(x1075)))+(((x1069)*(x1070)))+(x1077)+(((IkReal(-1.00000000000000))*(r02)*(x1066)))+(((IkReal(-1.00000000000000))*(cj2)*(x1064)*(x1075)))+(((IkReal(-1.00000000000000))*(r01)*(x1064)*(x1065))));
evalcond[3]=((((r01)*(sj1)*(x1065)))+(((x1070)*(x1073)))+(((x1066)*(x1075)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1064)))+(((x1073)*(x1075)))+(((IkReal(-1.00000000000000))*(x1062)))+(((r02)*(x1069))));
evalcond[4]=((IkReal(0.405000000000000))+(((x1068)*(x1071)))+(((IkReal(-1.00000000000000))*(x1063)*(x1064)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1066)))+(((x1067)*(x1069)))+(((cj2)*(x1072)))+(((IkReal(-1.00000000000000))*(cj2)*(x1064)*(x1067)))+(((IkReal(-1.00000000000000))*(py)*(x1064)*(x1065)))+(((IkReal(-0.0750000000000000))*(x1069)))+(((IkReal(-0.790000000000000))*(x1077)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1067)*(x1073)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1072)))+(((x1066)*(x1067)))+(((cj1)*(py)*(x1071)))+(((sj1)*(x1063)))+(((IkReal(-0.0750000000000000))*(x1066)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1064)))+(((IkReal(0.790000000000000))*(x1062)))+(((x1065)*(x1068))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1079=((cj1)*(sj2));
IkReal x1080=((r01)*(sj0));
IkReal x1081=((cj0)*(r00));
IkReal x1082=((cj2)*(sj1));
if( IKabs(((((x1079)*(x1081)))+(((x1079)*(x1080)))+(((x1081)*(x1082)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1080)*(x1082)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(-100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1079)*(x1081)))+(((x1079)*(x1080)))+(((x1081)*(x1082)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1080)*(x1082)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(-100.000000000000))*(cj0)*(py)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1079)*(x1081)))+(((x1079)*(x1080)))+(((x1081)*(x1082)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1080)*(x1082)))+(((r02)*(sj1)*(sj2)))), ((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(-100.000000000000))*(cj0)*(py)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1083=IKcos(j5);
IkReal x1084=IKsin(j5);
IkReal x1085=((pz)*(sj2));
IkReal x1086=((IkReal(1.00000000000000))*(cj1));
IkReal x1087=((cj2)*(sj0));
IkReal x1088=((cj2)*(sj1));
IkReal x1089=((cj0)*(px));
IkReal x1090=((py)*(sj1));
IkReal x1091=((sj1)*(sj2));
IkReal x1092=((r01)*(sj0));
IkReal x1093=((sj0)*(sj2));
IkReal x1094=((IkReal(0.0750000000000000))*(cj1));
IkReal x1095=((cj1)*(sj2));
IkReal x1096=((IkReal(1.00000000000000))*(cj0));
IkReal x1097=((cj0)*(r00));
IkReal x1098=((sj2)*(x1097));
IkReal x1099=((sj4)*(x1083));
IkReal x1100=((cj4)*(x1083));
evalcond[0]=((((r00)*(sj0)))+(x1100)+(((IkReal(-1.00000000000000))*(r01)*(x1096))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1096)))+(((IkReal(-0.790000000000000))*(x1100)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1086)*(x1087)))+(x1099)+(((IkReal(-1.00000000000000))*(cj2)*(x1086)*(x1097)))+(((IkReal(-1.00000000000000))*(r02)*(x1088)))+(((x1091)*(x1097)))+(((x1091)*(x1092)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1086))));
evalcond[3]=((((r01)*(sj1)*(x1087)))+(((x1095)*(x1097)))+(((r02)*(x1091)))+(((x1088)*(x1097)))+(((IkReal(-1.00000000000000))*(x1084)))+(((x1092)*(x1095)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1086))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1088)))+(((IkReal(-0.0750000000000000))*(x1091)))+(((IkReal(-0.790000000000000))*(x1099)))+(((x1089)*(x1091)))+(((IkReal(-1.00000000000000))*(x1085)*(x1086)))+(((IkReal(-1.00000000000000))*(cj2)*(x1086)*(x1089)))+(((IkReal(-1.00000000000000))*(py)*(x1086)*(x1087)))+(((IkReal(0.0800000000000000))*(cj4)))+(((x1090)*(x1093)))+(((cj2)*(x1094))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1087)*(x1090)))+(((x1089)*(x1095)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(0.790000000000000))*(x1084)))+(((IkReal(-0.0750000000000000))*(x1088)))+(((IkReal(-1.00000000000000))*(sj2)*(x1094)))+(((sj1)*(x1085)))+(((cj1)*(py)*(x1093)))+(((x1088)*(x1089)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1086))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1101=((cj1)*(sj2));
IkReal x1102=((r01)*(sj0));
IkReal x1103=((cj0)*(r00));
IkReal x1104=((cj2)*(sj1));
if( IKabs(((((x1102)*(x1104)))+(((x1101)*(x1102)))+(((x1101)*(x1103)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1103)*(x1104)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1102)*(x1104)))+(((x1101)*(x1102)))+(((x1101)*(x1103)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1103)*(x1104)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1102)*(x1104)))+(((x1101)*(x1102)))+(((x1101)*(x1103)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1103)*(x1104)))+(((r02)*(sj1)*(sj2)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1105=IKcos(j5);
IkReal x1106=IKsin(j5);
IkReal x1107=((pz)*(sj2));
IkReal x1108=((IkReal(1.00000000000000))*(cj1));
IkReal x1109=((cj2)*(sj0));
IkReal x1110=((cj2)*(sj1));
IkReal x1111=((cj0)*(px));
IkReal x1112=((py)*(sj1));
IkReal x1113=((sj1)*(sj2));
IkReal x1114=((r01)*(sj0));
IkReal x1115=((sj0)*(sj2));
IkReal x1116=((IkReal(0.0750000000000000))*(cj1));
IkReal x1117=((cj1)*(sj2));
IkReal x1118=((IkReal(1.00000000000000))*(cj0));
IkReal x1119=((cj0)*(r00));
IkReal x1120=((sj2)*(x1119));
IkReal x1121=((sj4)*(x1105));
IkReal x1122=((cj4)*(x1105));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1118)))+(((r00)*(sj0)))+(x1122));
evalcond[1]=((((IkReal(-0.790000000000000))*(x1122)))+(((IkReal(-1.00000000000000))*(py)*(x1118)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1108)*(x1109)))+(((IkReal(-1.00000000000000))*(cj2)*(x1108)*(x1119)))+(x1121)+(((IkReal(-1.00000000000000))*(r02)*(x1110)))+(((x1113)*(x1114)))+(((x1113)*(x1119)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1108))));
evalcond[3]=((((x1114)*(x1117)))+(((x1110)*(x1119)))+(((r01)*(sj1)*(x1109)))+(((IkReal(-1.00000000000000))*(x1106)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1108)))+(((x1117)*(x1119)))+(((r02)*(x1113))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1110)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1108)*(x1111)))+(((x1111)*(x1113)))+(((x1112)*(x1115)))+(((IkReal(-0.790000000000000))*(x1121)))+(((IkReal(-1.00000000000000))*(py)*(x1108)*(x1109)))+(((IkReal(-1.00000000000000))*(x1107)*(x1108)))+(((IkReal(-0.0750000000000000))*(x1113)))+(((cj2)*(x1116)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1109)*(x1112)))+(((x1111)*(x1117)))+(((IkReal(0.790000000000000))*(x1106)))+(((x1110)*(x1111)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1116)))+(((IkReal(-0.0750000000000000))*(x1110)))+(((sj1)*(x1107)))+(((cj1)*(py)*(x1115)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1108))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1123=((cj1)*(cj2));
IkReal x1124=((IkReal(1.00000000000000))*(cj0));
IkReal x1125=((sj1)*(sj2));
IkReal x1126=((IkReal(1.00000000000000))*(sj0));
IkReal x1127=((sj0)*(x1125));
IkReal x1128=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x1129=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r01)*(x1127)))+(((IkReal(-1.00000000000000))*(r00)*(x1123)*(x1124)))+(((cj0)*(r00)*(x1125)))+(((IkReal(-1.00000000000000))*(r01)*(x1123)*(x1126)))+(((IkReal(-1.00000000000000))*(r02)*(x1129)))+(((IkReal(-1.00000000000000))*(r02)*(x1128))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(0.0750000000000000))*(x1123)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x1123)*(x1126)))+(((IkReal(-1.00000000000000))*(pz)*(x1129)))+(((IkReal(-1.00000000000000))*(pz)*(x1128)))+(((py)*(x1127)))+(((IkReal(-1.00000000000000))*(px)*(x1123)*(x1124)))+(((IkReal(-0.0750000000000000))*(x1125)))+(((cj0)*(px)*(x1125))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1130=((cj2)*(cj3));
IkReal x1131=((cj1)*(r02));
IkReal x1132=((cj3)*(sj2));
IkReal x1133=((r02)*(sj1));
IkReal x1134=((cj2)*(sj3));
IkReal x1135=((cj0)*(r00));
IkReal x1136=((r01)*(sj0));
IkReal x1137=((r00)*(sj0));
IkReal x1138=((sj1)*(x1135));
IkReal x1139=((IkReal(1.00000000000000))*(x1136));
IkReal x1140=((cj1)*(sj2)*(sj3));
IkReal x1141=((IkReal(1.00000000000000))*(cj0)*(r01));
if( IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(cj3)*(x1141)))+(((cj3)*(x1137)))+(((x1131)*(x1134)))+(((IkReal(-1.00000000000000))*(x1134)*(x1138)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1133)))+(((IkReal(-1.00000000000000))*(sj1)*(x1134)*(x1139)))+(((IkReal(-1.00000000000000))*(x1139)*(x1140)))+(((IkReal(-1.00000000000000))*(x1135)*(x1140))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((x1130)*(x1138)))+(((cj1)*(x1132)*(x1135)))+(((cj1)*(x1132)*(x1136)))+(((IkReal(-1.00000000000000))*(x1130)*(x1131)))+(((sj1)*(x1130)*(x1136)))+(((sj3)*(x1137)))+(((IkReal(-1.00000000000000))*(sj3)*(x1141)))+(((x1132)*(x1133))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst59)*(((((IkReal(-1.00000000000000))*(cj3)*(x1141)))+(((cj3)*(x1137)))+(((x1131)*(x1134)))+(((IkReal(-1.00000000000000))*(x1134)*(x1138)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1133)))+(((IkReal(-1.00000000000000))*(sj1)*(x1134)*(x1139)))+(((IkReal(-1.00000000000000))*(x1139)*(x1140)))+(((IkReal(-1.00000000000000))*(x1135)*(x1140)))))), ((gconst59)*(((((x1130)*(x1138)))+(((cj1)*(x1132)*(x1135)))+(((cj1)*(x1132)*(x1136)))+(((IkReal(-1.00000000000000))*(x1130)*(x1131)))+(((sj1)*(x1130)*(x1136)))+(((sj3)*(x1137)))+(((IkReal(-1.00000000000000))*(sj3)*(x1141)))+(((x1132)*(x1133)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1142=IKsin(j5);
IkReal x1143=IKcos(j5);
IkReal x1144=((sj1)*(sj2));
IkReal x1145=((cj2)*(sj1));
IkReal x1146=((py)*(sj0));
IkReal x1147=((IkReal(1.00000000000000))*(cj0));
IkReal x1148=((r01)*(sj0));
IkReal x1149=((cj1)*(sj2));
IkReal x1150=((cj0)*(x1149));
IkReal x1151=((sj3)*(x1142));
IkReal x1152=((IkReal(1.00000000000000))*(x1143));
IkReal x1153=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1154=((IkReal(0.790000000000000))*(x1143));
IkReal x1155=((cj3)*(x1142));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1147)))+(((IkReal(-1.00000000000000))*(x1155)))+(((IkReal(-1.00000000000000))*(sj3)*(x1152))));
evalcond[1]=((((sj3)*(x1154)))+(((IkReal(0.790000000000000))*(x1155)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1147))));
evalcond[2]=((((r00)*(x1150)))+(((x1145)*(x1148)))+(x1151)+(((r02)*(x1144)))+(((IkReal(-1.00000000000000))*(r02)*(x1153)))+(((x1148)*(x1149)))+(((IkReal(-1.00000000000000))*(cj3)*(x1152)))+(((cj0)*(r00)*(x1145))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1145)*(x1146)))+(((IkReal(-0.790000000000000))*(x1151)))+(((x1146)*(x1149)))+(((pz)*(x1144)))+(((cj3)*(x1154)))+(((cj0)*(px)*(x1145)))+(((IkReal(-0.0750000000000000))*(x1149)))+(((IkReal(-0.0750000000000000))*(x1145)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1153)))+(((px)*(x1150))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1156=((cj1)*(cj2));
IkReal x1157=((IkReal(1.00000000000000))*(cj0));
IkReal x1158=((sj1)*(sj2));
IkReal x1159=((IkReal(1.00000000000000))*(sj0));
IkReal x1160=((sj0)*(x1158));
IkReal x1161=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x1162=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x1161)))+(((IkReal(-1.00000000000000))*(r02)*(x1162)))+(((r01)*(x1160)))+(((IkReal(-1.00000000000000))*(r00)*(x1156)*(x1157)))+(((cj0)*(r00)*(x1158)))+(((IkReal(-1.00000000000000))*(r01)*(x1156)*(x1159))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x1158)))+(((cj0)*(px)*(x1158)))+(((IkReal(0.0750000000000000))*(x1156)))+(((IkReal(-1.00000000000000))*(py)*(x1156)*(x1159)))+(((IkReal(-1.00000000000000))*(pz)*(x1162)))+(((IkReal(-1.00000000000000))*(pz)*(x1161)))+(((IkReal(-1.00000000000000))*(px)*(x1156)*(x1157)))+(((py)*(x1160))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1163=((r01)*(sj0));
IkReal x1164=((r02)*(sj1));
IkReal x1165=((cj2)*(cj3));
IkReal x1166=((IkReal(1.00000000000000))*(sj1));
IkReal x1167=((cj1)*(r02));
IkReal x1168=((cj2)*(sj3));
IkReal x1169=((cj0)*(r00));
IkReal x1170=((cj0)*(r01));
IkReal x1171=((r00)*(sj0));
IkReal x1172=((IkReal(1.00000000000000))*(cj3)*(sj2));
IkReal x1173=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(cj3)*(x1170)))+(((IkReal(-1.00000000000000))*(cj1)*(x1169)*(x1173)))+(((IkReal(-1.00000000000000))*(x1166)*(x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(x1164)*(x1173)))+(((cj3)*(x1171)))+(((x1167)*(x1168)))+(((IkReal(-1.00000000000000))*(x1163)*(x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(cj1)*(x1163)*(x1173))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((sj3)*(x1170)))+(((x1165)*(x1167)))+(((IkReal(-1.00000000000000))*(cj1)*(x1169)*(x1172)))+(((IkReal(-1.00000000000000))*(sj3)*(x1171)))+(((IkReal(-1.00000000000000))*(x1163)*(x1165)*(x1166)))+(((IkReal(-1.00000000000000))*(x1164)*(x1172)))+(((IkReal(-1.00000000000000))*(cj1)*(x1163)*(x1172)))+(((IkReal(-1.00000000000000))*(x1165)*(x1166)*(x1169))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst60)*(((((IkReal(-1.00000000000000))*(cj3)*(x1170)))+(((IkReal(-1.00000000000000))*(cj1)*(x1169)*(x1173)))+(((IkReal(-1.00000000000000))*(x1166)*(x1168)*(x1169)))+(((IkReal(-1.00000000000000))*(x1164)*(x1173)))+(((cj3)*(x1171)))+(((x1167)*(x1168)))+(((IkReal(-1.00000000000000))*(x1163)*(x1166)*(x1168)))+(((IkReal(-1.00000000000000))*(cj1)*(x1163)*(x1173)))))), ((gconst60)*(((((sj3)*(x1170)))+(((x1165)*(x1167)))+(((IkReal(-1.00000000000000))*(cj1)*(x1169)*(x1172)))+(((IkReal(-1.00000000000000))*(sj3)*(x1171)))+(((IkReal(-1.00000000000000))*(x1163)*(x1165)*(x1166)))+(((IkReal(-1.00000000000000))*(x1164)*(x1172)))+(((IkReal(-1.00000000000000))*(cj1)*(x1163)*(x1172)))+(((IkReal(-1.00000000000000))*(x1165)*(x1166)*(x1169)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1174=IKsin(j5);
IkReal x1175=IKcos(j5);
IkReal x1176=((sj1)*(sj2));
IkReal x1177=((cj2)*(sj1));
IkReal x1178=((py)*(sj0));
IkReal x1179=((IkReal(1.00000000000000))*(cj0));
IkReal x1180=((r01)*(sj0));
IkReal x1181=((cj1)*(sj2));
IkReal x1182=((cj0)*(x1181));
IkReal x1183=((sj3)*(x1174));
IkReal x1184=((cj3)*(x1175));
IkReal x1185=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1186=((sj3)*(x1175));
IkReal x1187=((cj3)*(x1174));
evalcond[0]=((((r00)*(sj0)))+(x1186)+(((IkReal(-1.00000000000000))*(r01)*(x1179)))+(((IkReal(-1.00000000000000))*(x1187))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1179)))+(((IkReal(0.790000000000000))*(x1187)))+(((IkReal(-0.790000000000000))*(x1186)))+(((px)*(sj0))));
evalcond[2]=((((r02)*(x1176)))+(x1184)+(x1183)+(((r00)*(x1182)))+(((x1180)*(x1181)))+(((x1177)*(x1180)))+(((cj0)*(r00)*(x1177)))+(((IkReal(-1.00000000000000))*(r02)*(x1185))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1178)*(x1181)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1185)))+(((IkReal(-0.790000000000000))*(x1184)))+(((IkReal(-0.790000000000000))*(x1183)))+(((IkReal(-0.0750000000000000))*(x1181)))+(((x1177)*(x1178)))+(((px)*(x1182)))+(((IkReal(-0.0750000000000000))*(x1177)))+(((pz)*(x1176)))+(((cj0)*(px)*(x1177))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1188=((cj1)*(cj2));
IkReal x1189=((sj1)*(sj2));
IkReal x1190=((IkReal(1.26582278481013))*(pz));
IkReal x1191=((IkReal(1.26582278481013))*(py)*(sj0));
IkReal x1192=((IkReal(1.26582278481013))*(cj0)*(px));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.512658227848101))+(((IkReal(0.0949367088607595))*(x1188)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(x1188)*(x1192)))+(((IkReal(-1.00000000000000))*(x1188)*(x1191)))+(((x1189)*(x1191)))+(((x1189)*(x1192)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1190)))+(((IkReal(-0.0949367088607595))*(x1189)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1190))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))+IKsqr(((IkReal(0.512658227848101))+(((IkReal(0.0949367088607595))*(x1188)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(x1188)*(x1192)))+(((IkReal(-1.00000000000000))*(x1188)*(x1191)))+(((x1189)*(x1191)))+(((x1189)*(x1192)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1190)))+(((IkReal(-0.0949367088607595))*(x1189)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1190)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((IkReal(0.512658227848101))+(((IkReal(0.0949367088607595))*(x1188)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(x1188)*(x1192)))+(((IkReal(-1.00000000000000))*(x1188)*(x1191)))+(((x1189)*(x1191)))+(((x1189)*(x1192)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1190)))+(((IkReal(-0.0949367088607595))*(x1189)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1190)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1193=IKsin(j5);
IkReal x1194=IKcos(j5);
IkReal x1195=((pz)*(sj2));
IkReal x1196=((IkReal(1.00000000000000))*(cj1));
IkReal x1197=((cj2)*(sj0));
IkReal x1198=((cj2)*(sj1));
IkReal x1199=((cj0)*(px));
IkReal x1200=((py)*(sj1));
IkReal x1201=((sj1)*(sj2));
IkReal x1202=((r01)*(sj0));
IkReal x1203=((IkReal(0.0750000000000000))*(cj1));
IkReal x1204=((sj0)*(sj2));
IkReal x1205=((cj1)*(sj2));
IkReal x1206=((IkReal(1.00000000000000))*(cj0));
IkReal x1207=((cj0)*(r00));
IkReal x1208=((sj2)*(x1207));
IkReal x1209=((sj3)*(x1193));
IkReal x1210=((cj3)*(x1193));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1206)))+(((IkReal(-1.00000000000000))*(x1210))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((IkReal(0.790000000000000))*(x1210)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1206))));
evalcond[2]=((x1194)+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1196)))+(((IkReal(-1.00000000000000))*(r01)*(x1196)*(x1197)))+(((IkReal(-1.00000000000000))*(cj2)*(x1196)*(x1207)))+(((x1201)*(x1202)))+(((x1201)*(x1207)))+(((IkReal(-1.00000000000000))*(r02)*(x1198))));
evalcond[3]=((x1209)+(((r01)*(sj1)*(x1197)))+(((x1202)*(x1205)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1196)))+(((x1205)*(x1207)))+(((r02)*(x1201)))+(((x1198)*(x1207))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1195)*(x1196)))+(((IkReal(-0.790000000000000))*(x1194)))+(((IkReal(-1.00000000000000))*(py)*(x1196)*(x1197)))+(((x1199)*(x1201)))+(((x1200)*(x1204)))+(((cj2)*(x1203)))+(((IkReal(-0.0750000000000000))*(x1201)))+(((IkReal(-1.00000000000000))*(cj2)*(x1196)*(x1199)))+(((IkReal(-1.00000000000000))*(pz)*(x1198))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x1209)))+(((cj1)*(py)*(x1204)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1203)))+(((IkReal(-0.0750000000000000))*(x1198)))+(((x1198)*(x1199)))+(((sj1)*(x1195)))+(((x1199)*(x1205)))+(((x1197)*(x1200)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1196)))+(((IkReal(0.0800000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1211=((cj0)*(r00));
IkReal x1212=((cj1)*(cj2));
IkReal x1213=((r01)*(sj0));
IkReal x1214=((IkReal(1.00000000000000))*(sj1)*(sj2));
if( IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj3)))+(((IkReal(100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1212)*(x1213)))+(((IkReal(-1.00000000000000))*(x1213)*(x1214)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x1211)*(x1214)))+(((x1211)*(x1212)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj3)))+(((IkReal(100.000000000000))*(cj0)*(py)))))))+IKsqr(((((x1212)*(x1213)))+(((IkReal(-1.00000000000000))*(x1213)*(x1214)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x1211)*(x1214)))+(((x1211)*(x1212)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj3)))+(((IkReal(100.000000000000))*(cj0)*(py)))))), ((((x1212)*(x1213)))+(((IkReal(-1.00000000000000))*(x1213)*(x1214)))+(((cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x1211)*(x1214)))+(((x1211)*(x1212)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1215=IKsin(j5);
IkReal x1216=IKcos(j5);
IkReal x1217=((pz)*(sj2));
IkReal x1218=((IkReal(1.00000000000000))*(cj1));
IkReal x1219=((cj2)*(sj0));
IkReal x1220=((cj2)*(sj1));
IkReal x1221=((cj0)*(px));
IkReal x1222=((py)*(sj1));
IkReal x1223=((sj1)*(sj2));
IkReal x1224=((r01)*(sj0));
IkReal x1225=((IkReal(0.0750000000000000))*(cj1));
IkReal x1226=((sj0)*(sj2));
IkReal x1227=((cj1)*(sj2));
IkReal x1228=((IkReal(1.00000000000000))*(cj0));
IkReal x1229=((cj0)*(r00));
IkReal x1230=((sj2)*(x1229));
IkReal x1231=((sj3)*(x1215));
IkReal x1232=((cj3)*(x1215));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1232)))+(((IkReal(-1.00000000000000))*(r01)*(x1228))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((IkReal(0.790000000000000))*(x1232)))+(((IkReal(-1.00000000000000))*(py)*(x1228)))+(((px)*(sj0))));
evalcond[2]=((x1216)+(((x1223)*(x1229)))+(((x1223)*(x1224)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1218)))+(((IkReal(-1.00000000000000))*(cj2)*(x1218)*(x1229)))+(((IkReal(-1.00000000000000))*(r02)*(x1220)))+(((IkReal(-1.00000000000000))*(r01)*(x1218)*(x1219))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1218)))+(x1231)+(((x1224)*(x1227)))+(((r01)*(sj1)*(x1219)))+(((r02)*(x1223)))+(((x1220)*(x1229)))+(((x1227)*(x1229))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((cj2)*(x1225)))+(((IkReal(-1.00000000000000))*(x1217)*(x1218)))+(((IkReal(-1.00000000000000))*(pz)*(x1220)))+(((x1221)*(x1223)))+(((IkReal(-1.00000000000000))*(cj2)*(x1218)*(x1221)))+(((x1222)*(x1226)))+(((IkReal(-1.00000000000000))*(py)*(x1218)*(x1219)))+(((IkReal(-0.790000000000000))*(x1216)))+(((IkReal(-0.0750000000000000))*(x1223))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1218)))+(((x1219)*(x1222)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1220)*(x1221)))+(((x1221)*(x1227)))+(((cj1)*(py)*(x1226)))+(((sj1)*(x1217)))+(((IkReal(-1.00000000000000))*(sj2)*(x1225)))+(((IkReal(0.0800000000000000))*(cj3)))+(((IkReal(-0.790000000000000))*(x1231)))+(((IkReal(-0.0750000000000000))*(x1220))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1233=((IkReal(1.00000000000000))*(r01));
IkReal x1234=((cj0)*(r00));
IkReal x1235=((cj1)*(cj2));
IkReal x1236=((sj1)*(sj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1233))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1234)*(x1235)))+(((cj2)*(r02)*(sj1)))+(((r01)*(sj0)*(x1235)))+(((IkReal(-1.00000000000000))*(sj0)*(x1233)*(x1236)))+(((IkReal(-1.00000000000000))*(x1234)*(x1236)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1233)))))))+IKsqr(((((x1234)*(x1235)))+(((cj2)*(r02)*(sj1)))+(((r01)*(sj0)*(x1235)))+(((IkReal(-1.00000000000000))*(sj0)*(x1233)*(x1236)))+(((IkReal(-1.00000000000000))*(x1234)*(x1236)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1233)))))), ((((x1234)*(x1235)))+(((cj2)*(r02)*(sj1)))+(((r01)*(sj0)*(x1235)))+(((IkReal(-1.00000000000000))*(sj0)*(x1233)*(x1236)))+(((IkReal(-1.00000000000000))*(x1234)*(x1236)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1237=IKsin(j5);
IkReal x1238=IKcos(j5);
IkReal x1239=((pz)*(sj2));
IkReal x1240=((IkReal(1.00000000000000))*(cj1));
IkReal x1241=((cj2)*(sj0));
IkReal x1242=((cj2)*(sj1));
IkReal x1243=((cj0)*(px));
IkReal x1244=((py)*(sj1));
IkReal x1245=((sj1)*(sj2));
IkReal x1246=((r01)*(sj0));
IkReal x1247=((IkReal(0.0750000000000000))*(cj1));
IkReal x1248=((sj0)*(sj2));
IkReal x1249=((cj1)*(sj2));
IkReal x1250=((IkReal(1.00000000000000))*(cj0));
IkReal x1251=((cj0)*(r00));
IkReal x1252=((sj2)*(x1251));
IkReal x1253=((sj3)*(x1237));
IkReal x1254=((cj3)*(x1237));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1250)))+(((IkReal(-1.00000000000000))*(x1254))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x1254)))+(((IkReal(-1.00000000000000))*(py)*(x1250))));
evalcond[2]=((x1238)+(((IkReal(-1.00000000000000))*(r01)*(x1240)*(x1241)))+(((IkReal(-1.00000000000000))*(cj2)*(x1240)*(x1251)))+(((x1245)*(x1251)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1240)))+(((x1245)*(x1246)))+(((IkReal(-1.00000000000000))*(r02)*(x1242))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1240)))+(x1253)+(((x1246)*(x1249)))+(((r02)*(x1245)))+(((r01)*(sj1)*(x1241)))+(((x1249)*(x1251)))+(((x1242)*(x1251))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1239)*(x1240)))+(((IkReal(-0.0750000000000000))*(x1245)))+(((x1244)*(x1248)))+(((x1243)*(x1245)))+(((IkReal(-0.790000000000000))*(x1238)))+(((IkReal(-1.00000000000000))*(cj2)*(x1240)*(x1243)))+(((cj2)*(x1247)))+(((IkReal(-1.00000000000000))*(pz)*(x1242)))+(((IkReal(-1.00000000000000))*(py)*(x1240)*(x1241))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((sj1)*(x1239)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1240)))+(((IkReal(-1.00000000000000))*(sj2)*(x1247)))+(((IkReal(-0.0750000000000000))*(x1242)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x1253)))+(((x1242)*(x1243)))+(((x1241)*(x1244)))+(((x1243)*(x1249)))+(((IkReal(0.0800000000000000))*(cj3)))+(((cj1)*(py)*(x1248))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1255=((cj1)*(cj2));
IkReal x1256=((sj1)*(sj2));
IkReal x1257=((IkReal(1.26582278481013))*(pz));
IkReal x1258=((IkReal(1.26582278481013))*(py)*(sj0));
IkReal x1259=((IkReal(1.26582278481013))*(cj0)*(px));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.512658227848101))+(((cj1)*(sj2)*(x1257)))+(((IkReal(-1.00000000000000))*(x1256)*(x1258)))+(((IkReal(-1.00000000000000))*(x1256)*(x1259)))+(((IkReal(0.462025316455696))*(sj2)))+(((x1255)*(x1258)))+(((x1255)*(x1259)))+(((IkReal(0.0949367088607595))*(x1256)))+(((IkReal(-0.0949367088607595))*(x1255)))+(((cj2)*(sj1)*(x1257))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))+IKsqr(((IkReal(-0.512658227848101))+(((cj1)*(sj2)*(x1257)))+(((IkReal(-1.00000000000000))*(x1256)*(x1258)))+(((IkReal(-1.00000000000000))*(x1256)*(x1259)))+(((IkReal(0.462025316455696))*(sj2)))+(((x1255)*(x1258)))+(((x1255)*(x1259)))+(((IkReal(0.0949367088607595))*(x1256)))+(((IkReal(-0.0949367088607595))*(x1255)))+(((cj2)*(sj1)*(x1257)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((IkReal(-0.512658227848101))+(((cj1)*(sj2)*(x1257)))+(((IkReal(-1.00000000000000))*(x1256)*(x1258)))+(((IkReal(-1.00000000000000))*(x1256)*(x1259)))+(((IkReal(0.462025316455696))*(sj2)))+(((x1255)*(x1258)))+(((x1255)*(x1259)))+(((IkReal(0.0949367088607595))*(x1256)))+(((IkReal(-0.0949367088607595))*(x1255)))+(((cj2)*(sj1)*(x1257)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1260=IKsin(j5);
IkReal x1261=IKcos(j5);
IkReal x1262=((pz)*(sj2));
IkReal x1263=((IkReal(1.00000000000000))*(cj1));
IkReal x1264=((cj2)*(sj0));
IkReal x1265=((cj2)*(sj1));
IkReal x1266=((cj0)*(px));
IkReal x1267=((py)*(sj1));
IkReal x1268=((sj1)*(sj2));
IkReal x1269=((r01)*(sj0));
IkReal x1270=((IkReal(0.0750000000000000))*(cj1));
IkReal x1271=((sj0)*(sj2));
IkReal x1272=((cj1)*(sj2));
IkReal x1273=((IkReal(1.00000000000000))*(cj0));
IkReal x1274=((cj0)*(r00));
IkReal x1275=((sj2)*(x1274));
IkReal x1276=((sj3)*(x1260));
IkReal x1277=((cj3)*(x1260));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1273)))+(((IkReal(-1.00000000000000))*(x1277))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1273)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3)))+(((IkReal(0.790000000000000))*(x1277))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x1265)))+(((IkReal(-1.00000000000000))*(r01)*(x1263)*(x1264)))+(((x1268)*(x1269)))+(((x1268)*(x1274)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1263)))+(((IkReal(-1.00000000000000))*(cj2)*(x1263)*(x1274)))+(((IkReal(-1.00000000000000))*(x1261))));
evalcond[3]=((((x1269)*(x1272)))+(x1276)+(((x1265)*(x1274)))+(((x1272)*(x1274)))+(((r02)*(x1268)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1263)))+(((r01)*(sj1)*(x1264))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1265)))+(((IkReal(-1.00000000000000))*(x1262)*(x1263)))+(((x1267)*(x1271)))+(((IkReal(-1.00000000000000))*(py)*(x1263)*(x1264)))+(((cj2)*(x1270)))+(((x1266)*(x1268)))+(((IkReal(-1.00000000000000))*(cj2)*(x1263)*(x1266)))+(((IkReal(0.790000000000000))*(x1261)))+(((IkReal(-0.0750000000000000))*(x1268))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x1271)))+(((x1264)*(x1267)))+(((sj1)*(x1262)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x1276)))+(((x1266)*(x1272)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1263)))+(((IkReal(-1.00000000000000))*(sj2)*(x1270)))+(((IkReal(-0.0750000000000000))*(x1265)))+(((IkReal(-0.0800000000000000))*(cj3)))+(((x1265)*(x1266))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1278=((r01)*(sj0));
IkReal x1279=((sj1)*(sj2));
IkReal x1280=((IkReal(1.00000000000000))*(cj1));
IkReal x1281=((cj0)*(r00));
if( IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(8.00000000000000))*(sj3)))+(((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1279)*(x1281)))+(((IkReal(-1.00000000000000))*(cj2)*(x1278)*(x1280)))+(((x1278)*(x1279)))+(((IkReal(-1.00000000000000))*(cj2)*(x1280)*(x1281)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1280)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(8.00000000000000))*(sj3)))+(((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(100.000000000000))*(cj0)*(py)))))))+IKsqr(((((x1279)*(x1281)))+(((IkReal(-1.00000000000000))*(cj2)*(x1278)*(x1280)))+(((x1278)*(x1279)))+(((IkReal(-1.00000000000000))*(cj2)*(x1280)*(x1281)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1280)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(8.00000000000000))*(sj3)))+(((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(100.000000000000))*(cj0)*(py)))))), ((((x1279)*(x1281)))+(((IkReal(-1.00000000000000))*(cj2)*(x1278)*(x1280)))+(((x1278)*(x1279)))+(((IkReal(-1.00000000000000))*(cj2)*(x1280)*(x1281)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1280)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1282=IKsin(j5);
IkReal x1283=IKcos(j5);
IkReal x1284=((pz)*(sj2));
IkReal x1285=((IkReal(1.00000000000000))*(cj1));
IkReal x1286=((cj2)*(sj0));
IkReal x1287=((cj2)*(sj1));
IkReal x1288=((cj0)*(px));
IkReal x1289=((py)*(sj1));
IkReal x1290=((sj1)*(sj2));
IkReal x1291=((r01)*(sj0));
IkReal x1292=((IkReal(0.0750000000000000))*(cj1));
IkReal x1293=((sj0)*(sj2));
IkReal x1294=((cj1)*(sj2));
IkReal x1295=((IkReal(1.00000000000000))*(cj0));
IkReal x1296=((cj0)*(r00));
IkReal x1297=((sj2)*(x1296));
IkReal x1298=((sj3)*(x1282));
IkReal x1299=((cj3)*(x1282));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1295)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1299))));
evalcond[1]=((((IkReal(0.790000000000000))*(x1299)))+(((IkReal(-1.00000000000000))*(py)*(x1295)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1283)))+(((IkReal(-1.00000000000000))*(r01)*(x1285)*(x1286)))+(((x1290)*(x1296)))+(((x1290)*(x1291)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1285)))+(((IkReal(-1.00000000000000))*(r02)*(x1287)))+(((IkReal(-1.00000000000000))*(cj2)*(x1285)*(x1296))));
evalcond[3]=((x1298)+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1285)))+(((x1287)*(x1296)))+(((x1294)*(x1296)))+(((r01)*(sj1)*(x1286)))+(((x1291)*(x1294)))+(((r02)*(x1290))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.790000000000000))*(x1283)))+(((IkReal(-1.00000000000000))*(py)*(x1285)*(x1286)))+(((IkReal(-1.00000000000000))*(cj2)*(x1285)*(x1288)))+(((cj2)*(x1292)))+(((IkReal(-0.0750000000000000))*(x1290)))+(((IkReal(-1.00000000000000))*(pz)*(x1287)))+(((IkReal(-1.00000000000000))*(x1284)*(x1285)))+(((x1288)*(x1290)))+(((x1289)*(x1293))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1285)))+(((IkReal(-1.00000000000000))*(sj2)*(x1292)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj1)*(py)*(x1293)))+(((IkReal(-0.790000000000000))*(x1298)))+(((x1288)*(x1294)))+(((x1287)*(x1288)))+(((x1286)*(x1289)))+(((IkReal(-0.0750000000000000))*(x1287)))+(((sj1)*(x1284)))+(((IkReal(-0.0800000000000000))*(cj3))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1300=((r01)*(sj0));
IkReal x1301=((sj1)*(sj2));
IkReal x1302=((IkReal(1.00000000000000))*(cj0));
IkReal x1303=((cj1)*(cj2));
IkReal x1304=((IkReal(1.00000000000000))*(r02));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1302))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1304)))+(((x1300)*(x1301)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1304)))+(((cj0)*(r00)*(x1301)))+(((IkReal(-1.00000000000000))*(x1300)*(x1303)))+(((IkReal(-1.00000000000000))*(r00)*(x1302)*(x1303))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1302)))))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1304)))+(((x1300)*(x1301)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1304)))+(((cj0)*(r00)*(x1301)))+(((IkReal(-1.00000000000000))*(x1300)*(x1303)))+(((IkReal(-1.00000000000000))*(r00)*(x1302)*(x1303)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1302)))))), ((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1304)))+(((x1300)*(x1301)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1304)))+(((cj0)*(r00)*(x1301)))+(((IkReal(-1.00000000000000))*(x1300)*(x1303)))+(((IkReal(-1.00000000000000))*(r00)*(x1302)*(x1303)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1305=IKsin(j5);
IkReal x1306=IKcos(j5);
IkReal x1307=((pz)*(sj2));
IkReal x1308=((IkReal(1.00000000000000))*(cj1));
IkReal x1309=((cj2)*(sj0));
IkReal x1310=((cj2)*(sj1));
IkReal x1311=((cj0)*(px));
IkReal x1312=((py)*(sj1));
IkReal x1313=((sj1)*(sj2));
IkReal x1314=((r01)*(sj0));
IkReal x1315=((IkReal(0.0750000000000000))*(cj1));
IkReal x1316=((sj0)*(sj2));
IkReal x1317=((cj1)*(sj2));
IkReal x1318=((IkReal(1.00000000000000))*(cj0));
IkReal x1319=((cj0)*(r00));
IkReal x1320=((sj2)*(x1319));
IkReal x1321=((sj3)*(x1305));
IkReal x1322=((cj3)*(x1305));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1318)))+(((IkReal(-1.00000000000000))*(x1322))));
evalcond[1]=((((IkReal(0.790000000000000))*(x1322)))+(((IkReal(-1.00000000000000))*(py)*(x1318)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1308)))+(((x1313)*(x1319)))+(((x1313)*(x1314)))+(((IkReal(-1.00000000000000))*(cj2)*(x1308)*(x1319)))+(((IkReal(-1.00000000000000))*(x1306)))+(((IkReal(-1.00000000000000))*(r01)*(x1308)*(x1309)))+(((IkReal(-1.00000000000000))*(r02)*(x1310))));
evalcond[3]=((x1321)+(((x1317)*(x1319)))+(((x1310)*(x1319)))+(((x1314)*(x1317)))+(((r01)*(sj1)*(x1309)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1308)))+(((r02)*(x1313))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x1311)*(x1313)))+(((IkReal(-1.00000000000000))*(x1307)*(x1308)))+(((IkReal(-1.00000000000000))*(cj2)*(x1308)*(x1311)))+(((IkReal(-1.00000000000000))*(pz)*(x1310)))+(((IkReal(-0.0750000000000000))*(x1313)))+(((cj2)*(x1315)))+(((IkReal(-1.00000000000000))*(py)*(x1308)*(x1309)))+(((IkReal(0.790000000000000))*(x1306)))+(((x1312)*(x1316))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1310)*(x1311)))+(((x1311)*(x1317)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj1)*(py)*(x1316)))+(((x1309)*(x1312)))+(((IkReal(-0.0750000000000000))*(x1310)))+(((IkReal(-1.00000000000000))*(sj2)*(x1315)))+(((IkReal(-0.790000000000000))*(x1321)))+(((IkReal(-0.0800000000000000))*(cj3)))+(((sj1)*(x1307)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1308))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1323=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1324=((r01)*(sj0));
IkReal x1325=((r02)*(sj1));
IkReal x1326=((IkReal(1.00000000000000))*(sj2));
IkReal x1327=((cj4)*(sj3));
IkReal x1328=((cj1)*(r02));
IkReal x1329=((sj1)*(sj2));
IkReal x1330=((cj0)*(r00));
IkReal x1331=((cj1)*(x1330));
IkReal x1332=((IkReal(1.00000000000000))*(cj2)*(x1327));
if( IKabs(((x1323)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1325)*(x1332)))+(((x1327)*(x1329)*(x1330)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1326)*(x1327)*(x1328)))+(((IkReal(-1.00000000000000))*(x1331)*(x1332)))+(((IkReal(-1.00000000000000))*(cj1)*(x1324)*(x1332)))+(((x1324)*(x1327)*(x1329))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1323)*(((((sj2)*(x1328)))+(((cj1)*(cj2)*(x1324)))+(((cj2)*(x1325)))+(((IkReal(-1.00000000000000))*(sj1)*(x1324)*(x1326)))+(((cj2)*(x1331)))+(((IkReal(-1.00000000000000))*(sj1)*(x1326)*(x1330))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1323)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1325)*(x1332)))+(((x1327)*(x1329)*(x1330)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1326)*(x1327)*(x1328)))+(((IkReal(-1.00000000000000))*(x1331)*(x1332)))+(((IkReal(-1.00000000000000))*(cj1)*(x1324)*(x1332)))+(((x1324)*(x1327)*(x1329)))))))+IKsqr(((x1323)*(((((sj2)*(x1328)))+(((cj1)*(cj2)*(x1324)))+(((cj2)*(x1325)))+(((IkReal(-1.00000000000000))*(sj1)*(x1324)*(x1326)))+(((cj2)*(x1331)))+(((IkReal(-1.00000000000000))*(sj1)*(x1326)*(x1330)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1323)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1325)*(x1332)))+(((x1327)*(x1329)*(x1330)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((IkReal(-1.00000000000000))*(x1326)*(x1327)*(x1328)))+(((IkReal(-1.00000000000000))*(x1331)*(x1332)))+(((IkReal(-1.00000000000000))*(cj1)*(x1324)*(x1332)))+(((x1324)*(x1327)*(x1329)))))), ((x1323)*(((((sj2)*(x1328)))+(((cj1)*(cj2)*(x1324)))+(((cj2)*(x1325)))+(((IkReal(-1.00000000000000))*(sj1)*(x1324)*(x1326)))+(((cj2)*(x1331)))+(((IkReal(-1.00000000000000))*(sj1)*(x1326)*(x1330)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1333=IKcos(j5);
IkReal x1334=IKsin(j5);
IkReal x1335=((pz)*(sj2));
IkReal x1336=((IkReal(1.00000000000000))*(cj1));
IkReal x1337=((cj2)*(sj0));
IkReal x1338=((cj2)*(sj1));
IkReal x1339=((cj0)*(px));
IkReal x1340=((py)*(sj1));
IkReal x1341=((sj1)*(sj2));
IkReal x1342=((r01)*(sj0));
IkReal x1343=((IkReal(0.0750000000000000))*(cj1));
IkReal x1344=((sj0)*(sj2));
IkReal x1345=((IkReal(0.0800000000000000))*(sj4));
IkReal x1346=((cj1)*(sj2));
IkReal x1347=((IkReal(0.790000000000000))*(cj3));
IkReal x1348=((IkReal(1.00000000000000))*(cj0));
IkReal x1349=((IkReal(1.00000000000000))*(cj3));
IkReal x1350=((cj0)*(r00));
IkReal x1351=((sj2)*(x1350));
IkReal x1352=((sj4)*(x1333));
IkReal x1353=((sj3)*(x1334));
IkReal x1354=((cj4)*(x1333));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1354)))+(((IkReal(-1.00000000000000))*(r01)*(x1348)))+(((IkReal(-1.00000000000000))*(x1334)*(x1349))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1348)))+(((x1334)*(x1347)))+(((IkReal(0.790000000000000))*(sj3)*(x1354)))+(((px)*(sj0)))+(((sj3)*(x1345))));
evalcond[2]=((x1352)+(((IkReal(-1.00000000000000))*(cj2)*(x1336)*(x1350)))+(((x1341)*(x1342)))+(((IkReal(-1.00000000000000))*(r01)*(x1336)*(x1337)))+(((x1341)*(x1350)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1336)))+(((IkReal(-1.00000000000000))*(r02)*(x1338))));
evalcond[3]=((x1353)+(((r01)*(sj1)*(x1337)))+(((r02)*(x1341)))+(((x1346)*(x1350)))+(((x1342)*(x1346)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1336)))+(((x1338)*(x1350)))+(((IkReal(-1.00000000000000))*(x1349)*(x1354))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x1335)*(x1336)))+(((IkReal(-0.365000000000000))*(sj2)))+(((x1339)*(x1341)))+(((IkReal(-1.00000000000000))*(cj2)*(x1336)*(x1339)))+(((IkReal(-0.0750000000000000))*(x1341)))+(((x1340)*(x1344)))+(((IkReal(-0.790000000000000))*(x1352)))+(((IkReal(-1.00000000000000))*(pz)*(x1338)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x1336)*(x1337)))+(((cj2)*(x1343))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1338)*(x1339)))+(((x1339)*(x1346)))+(((IkReal(-1.00000000000000))*(sj2)*(x1343)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x1353)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1336)))+(((x1337)*(x1340)))+(((sj1)*(x1335)))+(((x1347)*(x1354)))+(((IkReal(-0.0750000000000000))*(x1338)))+(((cj3)*(x1345)))+(((cj1)*(py)*(x1344))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1355=(sj3)*(sj3);
IkReal x1356=(cj3)*(cj3);
IkReal x1357=((cj3)*(sj0));
IkReal x1358=((IkReal(100.000000000000))*(px));
IkReal x1359=((cj2)*(sj1));
IkReal x1360=((cj1)*(sj2));
IkReal x1361=((IkReal(79.0000000000000))*(r01));
IkReal x1362=((IkReal(8.00000000000000))*(sj4));
IkReal x1363=((IkReal(79.0000000000000))*(cj0)*(r00));
IkReal x1364=((IkReal(79.0000000000000))*(x1356));
IkReal x1365=((IkReal(79.0000000000000))*(x1355));
IkReal x1366=((IkReal(100.000000000000))*(cj0)*(py));
IkReal x1367=((sj0)*(sj3)*(x1361));
IkReal x1368=((IkReal(79.0000000000000))*(r02)*(sj1)*(sj2));
IkReal x1369=((IkReal(79.0000000000000))*(cj1)*(cj2)*(r02));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x1365)))+(((IkReal(-1.00000000000000))*(x1364))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1365)))+(((IkReal(-1.00000000000000))*(x1364)))))):(IkReal)1.0e30))*(((((x1357)*(x1358)))+(((sj3)*(x1360)*(x1363)))+(((x1360)*(x1367)))+(((IkReal(-1.00000000000000))*(sj3)*(x1369)))+(((x1359)*(x1367)))+(((IkReal(-1.00000000000000))*(cj3)*(x1366)))+(((sj3)*(x1368)))+(((cj3)*(sj3)*(x1362)))+(((sj3)*(x1359)*(x1363))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1365)))+(((cj4)*(x1364))))) != 0)?((IkReal)1/(((((cj4)*(x1365)))+(((cj4)*(x1364)))))):(IkReal)1.0e30))*(((((x1357)*(x1360)*(x1361)))+(((IkReal(-1.00000000000000))*(sj0)*(sj3)*(x1358)))+(((IkReal(-1.00000000000000))*(cj3)*(x1369)))+(((cj3)*(x1359)*(x1363)))+(((IkReal(-1.00000000000000))*(x1355)*(x1362)))+(((cj3)*(x1360)*(x1363)))+(((cj3)*(x1368)))+(((x1357)*(x1359)*(x1361)))+(((sj3)*(x1366))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x1365)))+(((IkReal(-1.00000000000000))*(x1364))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1365)))+(((IkReal(-1.00000000000000))*(x1364)))))):(IkReal)1.0e30))*(((((x1357)*(x1358)))+(((sj3)*(x1360)*(x1363)))+(((x1360)*(x1367)))+(((IkReal(-1.00000000000000))*(sj3)*(x1369)))+(((x1359)*(x1367)))+(((IkReal(-1.00000000000000))*(cj3)*(x1366)))+(((sj3)*(x1368)))+(((cj3)*(sj3)*(x1362)))+(((sj3)*(x1359)*(x1363)))))))+IKsqr(((((IKabs(((((cj4)*(x1365)))+(((cj4)*(x1364))))) != 0)?((IkReal)1/(((((cj4)*(x1365)))+(((cj4)*(x1364)))))):(IkReal)1.0e30))*(((((x1357)*(x1360)*(x1361)))+(((IkReal(-1.00000000000000))*(sj0)*(sj3)*(x1358)))+(((IkReal(-1.00000000000000))*(cj3)*(x1369)))+(((cj3)*(x1359)*(x1363)))+(((IkReal(-1.00000000000000))*(x1355)*(x1362)))+(((cj3)*(x1360)*(x1363)))+(((cj3)*(x1368)))+(((x1357)*(x1359)*(x1361)))+(((sj3)*(x1366)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x1365)))+(((IkReal(-1.00000000000000))*(x1364))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1365)))+(((IkReal(-1.00000000000000))*(x1364)))))):(IkReal)1.0e30))*(((((x1357)*(x1358)))+(((sj3)*(x1360)*(x1363)))+(((x1360)*(x1367)))+(((IkReal(-1.00000000000000))*(sj3)*(x1369)))+(((x1359)*(x1367)))+(((IkReal(-1.00000000000000))*(cj3)*(x1366)))+(((sj3)*(x1368)))+(((cj3)*(sj3)*(x1362)))+(((sj3)*(x1359)*(x1363)))))), ((((IKabs(((((cj4)*(x1365)))+(((cj4)*(x1364))))) != 0)?((IkReal)1/(((((cj4)*(x1365)))+(((cj4)*(x1364)))))):(IkReal)1.0e30))*(((((x1357)*(x1360)*(x1361)))+(((IkReal(-1.00000000000000))*(sj0)*(sj3)*(x1358)))+(((IkReal(-1.00000000000000))*(cj3)*(x1369)))+(((cj3)*(x1359)*(x1363)))+(((IkReal(-1.00000000000000))*(x1355)*(x1362)))+(((cj3)*(x1360)*(x1363)))+(((cj3)*(x1368)))+(((x1357)*(x1359)*(x1361)))+(((sj3)*(x1366)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1370=IKcos(j5);
IkReal x1371=IKsin(j5);
IkReal x1372=((pz)*(sj2));
IkReal x1373=((IkReal(1.00000000000000))*(cj1));
IkReal x1374=((cj2)*(sj0));
IkReal x1375=((cj2)*(sj1));
IkReal x1376=((cj0)*(px));
IkReal x1377=((py)*(sj1));
IkReal x1378=((sj1)*(sj2));
IkReal x1379=((r01)*(sj0));
IkReal x1380=((IkReal(0.0750000000000000))*(cj1));
IkReal x1381=((sj0)*(sj2));
IkReal x1382=((IkReal(0.0800000000000000))*(sj4));
IkReal x1383=((cj1)*(sj2));
IkReal x1384=((IkReal(0.790000000000000))*(cj3));
IkReal x1385=((IkReal(1.00000000000000))*(cj0));
IkReal x1386=((IkReal(1.00000000000000))*(cj3));
IkReal x1387=((cj0)*(r00));
IkReal x1388=((sj2)*(x1387));
IkReal x1389=((sj4)*(x1370));
IkReal x1390=((sj3)*(x1371));
IkReal x1391=((cj4)*(x1370));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1391)))+(((IkReal(-1.00000000000000))*(r01)*(x1385)))+(((IkReal(-1.00000000000000))*(x1371)*(x1386))));
evalcond[1]=((((x1371)*(x1384)))+(((IkReal(0.790000000000000))*(sj3)*(x1391)))+(((sj3)*(x1382)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1385))));
evalcond[2]=((x1389)+(((IkReal(-1.00000000000000))*(r02)*(x1375)))+(((IkReal(-1.00000000000000))*(cj2)*(x1373)*(x1387)))+(((x1378)*(x1379)))+(((IkReal(-1.00000000000000))*(r01)*(x1373)*(x1374)))+(((x1378)*(x1387)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1373))));
evalcond[3]=((x1390)+(((r01)*(sj1)*(x1374)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1373)))+(((x1375)*(x1387)))+(((x1379)*(x1383)))+(((r02)*(x1378)))+(((IkReal(-1.00000000000000))*(x1386)*(x1391)))+(((x1383)*(x1387))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1373)*(x1374)))+(((IkReal(-0.365000000000000))*(sj2)))+(((x1376)*(x1378)))+(((IkReal(-1.00000000000000))*(cj2)*(x1373)*(x1376)))+(((IkReal(-0.790000000000000))*(x1389)))+(((cj2)*(x1380)))+(((IkReal(-1.00000000000000))*(pz)*(x1375)))+(((IkReal(-1.00000000000000))*(x1372)*(x1373)))+(((IkReal(-0.0750000000000000))*(x1378)))+(((x1377)*(x1381)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1373)))+(((IkReal(-0.790000000000000))*(x1390)))+(((x1375)*(x1376)))+(((cj1)*(py)*(x1381)))+(((cj3)*(x1382)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1374)*(x1377)))+(((IkReal(-0.0750000000000000))*(x1375)))+(((x1376)*(x1383)))+(((sj1)*(x1372)))+(((x1384)*(x1391)))+(((IkReal(-1.00000000000000))*(sj2)*(x1380))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1392=(sj3)*(sj3);
IkReal x1393=(cj3)*(cj3);
IkReal x1394=((cj2)*(cj3));
IkReal x1395=((cj1)*(r02));
IkReal x1396=((r01)*(sj0));
IkReal x1397=((cj3)*(sj2));
IkReal x1398=((r02)*(sj1));
IkReal x1399=((cj0)*(r00));
IkReal x1400=((r00)*(sj0));
IkReal x1401=((sj1)*(x1399));
IkReal x1402=((IkReal(1.00000000000000))*(cj2)*(sj3));
IkReal x1403=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x1404=((IkReal(1.00000000000000))*(cj1)*(sj2)*(sj3));
if( IKabs(((((IKabs(((x1393)+(x1392))) != 0)?((IkReal)1/(((x1393)+(x1392)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1398)))+(((IkReal(-1.00000000000000))*(x1399)*(x1404)))+(((IkReal(-1.00000000000000))*(cj3)*(x1403)))+(((IkReal(-1.00000000000000))*(sj1)*(x1396)*(x1402)))+(((IkReal(-1.00000000000000))*(x1401)*(x1402)))+(((IkReal(-1.00000000000000))*(x1396)*(x1404)))+(((cj3)*(x1400)))+(((cj2)*(sj3)*(x1395))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1392)))+(((cj4)*(x1393))))) != 0)?((IkReal)1/(((((cj4)*(x1392)))+(((cj4)*(x1393)))))):(IkReal)1.0e30))*(((((cj1)*(x1396)*(x1397)))+(((sj3)*(x1400)))+(((sj1)*(x1394)*(x1396)))+(((x1394)*(x1401)))+(((cj1)*(x1397)*(x1399)))+(((x1397)*(x1398)))+(((IkReal(-1.00000000000000))*(sj3)*(x1403)))+(((IkReal(-1.00000000000000))*(x1394)*(x1395))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x1393)+(x1392))) != 0)?((IkReal)1/(((x1393)+(x1392)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1398)))+(((IkReal(-1.00000000000000))*(x1399)*(x1404)))+(((IkReal(-1.00000000000000))*(cj3)*(x1403)))+(((IkReal(-1.00000000000000))*(sj1)*(x1396)*(x1402)))+(((IkReal(-1.00000000000000))*(x1401)*(x1402)))+(((IkReal(-1.00000000000000))*(x1396)*(x1404)))+(((cj3)*(x1400)))+(((cj2)*(sj3)*(x1395)))))))+IKsqr(((((IKabs(((((cj4)*(x1392)))+(((cj4)*(x1393))))) != 0)?((IkReal)1/(((((cj4)*(x1392)))+(((cj4)*(x1393)))))):(IkReal)1.0e30))*(((((cj1)*(x1396)*(x1397)))+(((sj3)*(x1400)))+(((sj1)*(x1394)*(x1396)))+(((x1394)*(x1401)))+(((cj1)*(x1397)*(x1399)))+(((x1397)*(x1398)))+(((IkReal(-1.00000000000000))*(sj3)*(x1403)))+(((IkReal(-1.00000000000000))*(x1394)*(x1395)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x1393)+(x1392))) != 0)?((IkReal)1/(((x1393)+(x1392)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1398)))+(((IkReal(-1.00000000000000))*(x1399)*(x1404)))+(((IkReal(-1.00000000000000))*(cj3)*(x1403)))+(((IkReal(-1.00000000000000))*(sj1)*(x1396)*(x1402)))+(((IkReal(-1.00000000000000))*(x1401)*(x1402)))+(((IkReal(-1.00000000000000))*(x1396)*(x1404)))+(((cj3)*(x1400)))+(((cj2)*(sj3)*(x1395)))))), ((((IKabs(((((cj4)*(x1392)))+(((cj4)*(x1393))))) != 0)?((IkReal)1/(((((cj4)*(x1392)))+(((cj4)*(x1393)))))):(IkReal)1.0e30))*(((((cj1)*(x1396)*(x1397)))+(((sj3)*(x1400)))+(((sj1)*(x1394)*(x1396)))+(((x1394)*(x1401)))+(((cj1)*(x1397)*(x1399)))+(((x1397)*(x1398)))+(((IkReal(-1.00000000000000))*(sj3)*(x1403)))+(((IkReal(-1.00000000000000))*(x1394)*(x1395)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1405=IKcos(j5);
IkReal x1406=IKsin(j5);
IkReal x1407=((pz)*(sj2));
IkReal x1408=((IkReal(1.00000000000000))*(cj1));
IkReal x1409=((cj2)*(sj0));
IkReal x1410=((cj2)*(sj1));
IkReal x1411=((cj0)*(px));
IkReal x1412=((py)*(sj1));
IkReal x1413=((sj1)*(sj2));
IkReal x1414=((r01)*(sj0));
IkReal x1415=((IkReal(0.0750000000000000))*(cj1));
IkReal x1416=((sj0)*(sj2));
IkReal x1417=((IkReal(0.0800000000000000))*(sj4));
IkReal x1418=((cj1)*(sj2));
IkReal x1419=((IkReal(0.790000000000000))*(cj3));
IkReal x1420=((IkReal(1.00000000000000))*(cj0));
IkReal x1421=((IkReal(1.00000000000000))*(cj3));
IkReal x1422=((cj0)*(r00));
IkReal x1423=((sj2)*(x1422));
IkReal x1424=((sj4)*(x1405));
IkReal x1425=((sj3)*(x1406));
IkReal x1426=((cj4)*(x1405));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1426)))+(((IkReal(-1.00000000000000))*(r01)*(x1420)))+(((IkReal(-1.00000000000000))*(x1406)*(x1421))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1420)))+(((IkReal(0.790000000000000))*(sj3)*(x1426)))+(((x1406)*(x1419)))+(((px)*(sj0)))+(((sj3)*(x1417))));
evalcond[2]=((((x1413)*(x1414)))+(((IkReal(-1.00000000000000))*(cj2)*(x1408)*(x1422)))+(((IkReal(-1.00000000000000))*(r01)*(x1408)*(x1409)))+(((IkReal(-1.00000000000000))*(r02)*(x1410)))+(x1424)+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1408)))+(((x1413)*(x1422))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x1421)*(x1426)))+(((x1410)*(x1422)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1408)))+(x1425)+(((x1418)*(x1422)))+(((r02)*(x1413)))+(((r01)*(sj1)*(x1409)))+(((x1414)*(x1418))));
evalcond[4]=((IkReal(0.405000000000000))+(((cj2)*(x1415)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1408)*(x1411)))+(((x1412)*(x1416)))+(((IkReal(-1.00000000000000))*(pz)*(x1410)))+(((x1411)*(x1413)))+(((IkReal(-0.790000000000000))*(x1424)))+(((IkReal(-1.00000000000000))*(x1407)*(x1408)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-0.0750000000000000))*(x1413)))+(((IkReal(-1.00000000000000))*(py)*(x1408)*(x1409))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1408)))+(((IkReal(-0.365000000000000))*(cj2)))+(((sj1)*(x1407)))+(((x1411)*(x1418)))+(((IkReal(-0.790000000000000))*(x1425)))+(((cj1)*(py)*(x1416)))+(((x1419)*(x1426)))+(((x1410)*(x1411)))+(((IkReal(-1.00000000000000))*(sj2)*(x1415)))+(((x1409)*(x1412)))+(((IkReal(-0.0750000000000000))*(x1410)))+(((cj3)*(x1417))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "8529cf64e1f47189a44788609179f24f"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
