/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:12:38.226918
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42,x43,x44;
x0=IKcos(j[0]);
x1=IKcos(j[2]);
x2=IKsin(j[1]);
x3=IKsin(j[0]);
x4=IKsin(j[3]);
x5=IKcos(j[3]);
x6=IKcos(j[1]);
x7=IKsin(j[2]);
x8=IKcos(j[4]);
x9=IKsin(j[4]);
x10=IKsin(j[5]);
x11=IKcos(j[5]);
x12=((IkReal(0.790000000000000))*(x9));
x13=((IkReal(0.790000000000000))*(x5));
x14=((IkReal(0.365000000000000))*(x2));
x15=((IkReal(0.0800000000000000))*(x1));
x16=((IkReal(1.00000000000000))*(x3));
x17=((IkReal(1.00000000000000))*(x0));
x18=((IkReal(0.790000000000000))*(x8));
x19=((IkReal(-0.0800000000000000))*(x4));
x20=((x10)*(x4));
x21=((x1)*(x2));
x22=((x3)*(x6));
x23=((IkReal(-1.00000000000000))*(x8));
x24=((x2)*(x7));
x25=((x6)*(x7));
x26=((x1)*(x6));
x27=((IkReal(-1.00000000000000))*(x9));
x28=((x3)*(x5));
x29=((IkReal(0.0900000000000000))*(x21));
x30=((x0)*(x26));
x31=((((IkReal(-1.00000000000000))*(x26)))+(x24));
x32=((x25)+(x21));
x33=((((IkReal(-1.00000000000000))*(x17)*(x24)))+(x30));
x34=((((x1)*(x22)))+(((IkReal(-1.00000000000000))*(x16)*(x24))));
x35=((x17)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x25))))));
x36=((x16)*(((((IkReal(-1.00000000000000))*(x21)))+(((IkReal(-1.00000000000000))*(x25))))));
x37=((x33)*(x9));
x38=((x36)*(x5));
x39=((x35)*(x4));
x40=((x35)*(x5));
x41=((x36)*(x4));
x42=((((IkReal(-1.00000000000000))*(x40)))+(((x16)*(x4))));
x43=((((IkReal(-1.00000000000000))*(x38)))+(((IkReal(-1.00000000000000))*(x0)*(x4))));
x44=((x43)*(x8));
IkReal x45=((x0)*(x24));
eetrans[0]=((((IkReal(0.0900000000000000))*(x0)*(x25)))+(((x0)*(x29)))+(((x0)*(x14)))+(((IkReal(-0.405000000000000))*(x45)))+(((x10)*(((((IkReal(-0.790000000000000))*(x39)))+(((IkReal(-0.790000000000000))*(x28)))))))+(((IkReal(0.0750000000000000))*(x0)))+(((IkReal(0.405000000000000))*(x30)))+(((x11)*(((((IkReal(-1.00000000000000))*(x18)*(x42)))+(((IkReal(-1.00000000000000))*(x12)*(x33)))))))+(((x9)*(((((IkReal(0.0800000000000000))*(x40)))+(((x19)*(x3)))))))+(((x8)*(((((x0)*(x15)*(x6)))+(((IkReal(-0.0800000000000000))*(x45))))))));
IkReal x46=((x24)*(x3));
IkReal x47=((IkReal(1.00000000000000))*(x3));
eetrans[1]=((((IkReal(-1.00000000000000))*(x29)*(x47)))+(((IkReal(-0.0900000000000000))*(x22)*(x7)))+(((IkReal(-0.0750000000000000))*(x3)))+(((x10)*(((((IkReal(-1.00000000000000))*(x0)*(x13)))+(((IkReal(0.790000000000000))*(x41)))))))+(((x9)*(((((IkReal(-0.0800000000000000))*(x38)))+(((x0)*(x19)))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x15)*(x22)))+(((IkReal(0.0800000000000000))*(x46)))))))+(((IkReal(0.405000000000000))*(x46)))+(((IkReal(-0.405000000000000))*(x1)*(x22)))+(((IkReal(-1.00000000000000))*(x14)*(x47)))+(((x11)*(((((x18)*(x43)))+(((x12)*(x34))))))));
IkReal x48=((IkReal(1.00000000000000))*(x15));
eetrans[2]=((IkReal(0.335000000000000))+(((IkReal(-0.405000000000000))*(x25)))+(((IkReal(-0.405000000000000))*(x21)))+(((x20)*(((((IkReal(0.790000000000000))*(x26)))+(((IkReal(-0.790000000000000))*(x24)))))))+(((IkReal(-0.0900000000000000))*(x24)))+(((x11)*(((((x12)*(x32)))+(((x13)*(x31)*(x8)))))))+(((IkReal(0.365000000000000))*(x6)))+(((x5)*(x9)*(((((IkReal(-1.00000000000000))*(x48)*(x6)))+(((IkReal(0.0800000000000000))*(x24)))))))+(((x8)*(((((IkReal(-0.0800000000000000))*(x25)))+(((IkReal(-1.00000000000000))*(x2)*(x48)))))))+(((IkReal(0.0900000000000000))*(x26))));
eerot[0]=((((x11)*(((((x42)*(x8)))+(x37)))))+(((x10)*(((x39)+(x28))))));
eerot[1]=((((x10)*(((((IkReal(-1.00000000000000))*(x41)))+(((x17)*(x5)))))))+(((x11)*(((((x27)*(x34)))+(((x23)*(x43))))))));
eerot[2]=((((x20)*(x31)))+(((x11)*(((((x27)*(x32)))+(((x23)*(x31)*(x5))))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {1}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j1,cj1,sj1,htj1,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij1[2], _nj1;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij1[0] = -1; _ij1[1] = -1; _nj1 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j1=pfree[0]; cj1=cos(pfree[0]); sj1=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=((((IkReal(-0.790000000000000))*(r00)))+(px));
new_r01=((IkReal(-1.00000000000000))*(r01));
new_py=((((IkReal(0.790000000000000))*(r01)))+(((IkReal(-1.00000000000000))*(py))));
new_r02=((IkReal(-1.00000000000000))*(r02));
new_pz=((IkReal(0.335000000000000))+(((IkReal(0.790000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(pz))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(0.790000000000000))+(((IkReal(0.455000000000000))*(cj1)*(r02)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(-0.455000000000000))*(r00)*(sj1)))+(((IkReal(-0.405000000000000))*(cj1)*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.405000000000000))*(r02)*(sj1))));
IkReal gconst1;
gconst1=((((IkReal(0.810000000000000))*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.810000000000000))*(cj1)*(r02))));
IkReal gconst2;
gconst2=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj1)))+(((IkReal(0.275000000000000))*(cj1)*(r02)))+(((IkReal(-0.275000000000000))*(r00)*(sj1)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(cj1)*(r00))));
IkReal gconst3;
gconst3=((IkReal(-0.253825000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.0607500000000000))*(cj1)))+(((IkReal(0.910000000000000))*(cj1)*(pz)))+(((IkReal(-0.810000000000000))*(cj1)*(px)))+(((IkReal(-0.150000000000000))*(px)))+(pp)+(((IkReal(0.0682500000000000))*(sj1)))+(((IkReal(-0.910000000000000))*(px)*(sj1))));
IkReal gconst4;
gconst4=((IkReal(-0.591300000000000))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-0.360000000000000))*(cj1)*(px)))+(((IkReal(-0.121500000000000))*(sj1)))+(((IkReal(1.62000000000000))*(px)*(sj1)))+(((IkReal(0.0270000000000000))*(cj1)))+(((IkReal(-1.62000000000000))*(cj1)*(pz))));
IkReal gconst5;
gconst5=((IkReal(-0.385225000000000))+(((IkReal(0.550000000000000))*(cj1)*(pz)))+(((IkReal(0.0412500000000000))*(sj1)))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(-0.150000000000000))*(px)))+(pp)+(((IkReal(-0.0607500000000000))*(cj1)))+(((IkReal(-0.550000000000000))*(px)*(sj1)))+(((IkReal(0.810000000000000))*(cj1)*(px))));
IkReal gconst6;
gconst6=((IkReal(0.790000000000000))+(((IkReal(0.455000000000000))*(cj1)*(r02)))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(-0.455000000000000))*(r00)*(sj1)))+(((IkReal(-0.405000000000000))*(cj1)*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-0.405000000000000))*(r02)*(sj1))));
IkReal gconst7;
gconst7=((((IkReal(0.810000000000000))*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.810000000000000))*(cj1)*(r02))));
IkReal gconst8;
gconst8=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(-0.0750000000000000))*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj1)))+(((IkReal(0.275000000000000))*(cj1)*(r02)))+(((IkReal(-0.275000000000000))*(r00)*(sj1)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(cj1)*(r00))));
IkReal gconst9;
gconst9=((IkReal(-0.253825000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.0607500000000000))*(cj1)))+(((IkReal(0.910000000000000))*(cj1)*(pz)))+(((IkReal(-0.810000000000000))*(cj1)*(px)))+(((IkReal(-0.150000000000000))*(px)))+(pp)+(((IkReal(0.0682500000000000))*(sj1)))+(((IkReal(-0.910000000000000))*(px)*(sj1))));
IkReal gconst10;
gconst10=((IkReal(-0.591300000000000))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-0.360000000000000))*(cj1)*(px)))+(((IkReal(-0.121500000000000))*(sj1)))+(((IkReal(1.62000000000000))*(px)*(sj1)))+(((IkReal(0.0270000000000000))*(cj1)))+(((IkReal(-1.62000000000000))*(cj1)*(pz))));
IkReal gconst11;
gconst11=((IkReal(-0.385225000000000))+(((IkReal(0.550000000000000))*(cj1)*(pz)))+(((IkReal(0.0412500000000000))*(sj1)))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(-0.150000000000000))*(px)))+(pp)+(((IkReal(-0.0607500000000000))*(cj1)))+(((IkReal(-0.550000000000000))*(px)*(sj1)))+(((IkReal(0.810000000000000))*(cj1)*(px))));
IkReal gconst12;
gconst12=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.910000000000000))*(r01)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(r01))));
IkReal gconst13;
gconst13=((((IkReal(1.62000000000000))*(r01)*(sj1)))+(((IkReal(-0.360000000000000))*(cj1)*(r01))));
IkReal gconst14;
gconst14=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.550000000000000))*(r01)*(sj1)))+(((IkReal(0.810000000000000))*(cj1)*(r01))));
IkReal gconst15;
gconst15=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-1.62000000000000))*(cj1)*(py)))+(((IkReal(-1.82000000000000))*(py)*(sj1))));
IkReal gconst16;
gconst16=((((IkReal(3.24000000000000))*(py)*(sj1)))+(((IkReal(-0.720000000000000))*(cj1)*(py))));
IkReal gconst17;
gconst17=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.62000000000000))*(cj1)*(py)))+(((IkReal(-1.10000000000000))*(py)*(sj1))));
IkReal gconst18;
gconst18=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.910000000000000))*(r01)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(r01))));
IkReal gconst19;
gconst19=((((IkReal(1.62000000000000))*(r01)*(sj1)))+(((IkReal(-0.360000000000000))*(cj1)*(r01))));
IkReal gconst20;
gconst20=((((IkReal(-0.150000000000000))*(r01)))+(((IkReal(-0.550000000000000))*(r01)*(sj1)))+(((IkReal(0.810000000000000))*(cj1)*(r01))));
IkReal gconst21;
gconst21=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(-1.62000000000000))*(cj1)*(py)))+(((IkReal(-1.82000000000000))*(py)*(sj1))));
IkReal gconst22;
gconst22=((((IkReal(3.24000000000000))*(py)*(sj1)))+(((IkReal(-0.720000000000000))*(cj1)*(py))));
IkReal gconst23;
gconst23=((((IkReal(-0.300000000000000))*(py)))+(((IkReal(1.62000000000000))*(cj1)*(py)))+(((IkReal(-1.10000000000000))*(py)*(sj1))));
IkReal gconst24;
gconst24=((IkReal(0.790000000000000))+(((IkReal(0.455000000000000))*(cj1)*(r02)))+(((px)*(r00)))+(((IkReal(0.455000000000000))*(r00)*(sj1)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(cj1)*(r00)))+(((IkReal(-0.405000000000000))*(r02)*(sj1))));
IkReal gconst25;
gconst25=((((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.810000000000000))*(r00)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(r02))));
IkReal gconst26;
gconst26=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj1)))+(((IkReal(0.275000000000000))*(cj1)*(r02)))+(((IkReal(-0.405000000000000))*(cj1)*(r00)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.275000000000000))*(r00)*(sj1))));
IkReal gconst27;
gconst27=((IkReal(-0.253825000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.0607500000000000))*(cj1)))+(((IkReal(0.910000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(0.0682500000000000))*(sj1)))+(((IkReal(0.910000000000000))*(px)*(sj1)))+(((IkReal(0.810000000000000))*(cj1)*(px))));
IkReal gconst28;
gconst28=((IkReal(-0.591300000000000))+(((IkReal(-1.62000000000000))*(px)*(sj1)))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-0.121500000000000))*(sj1)))+(((IkReal(0.360000000000000))*(cj1)*(px)))+(((IkReal(0.0270000000000000))*(cj1)))+(((IkReal(-1.62000000000000))*(cj1)*(pz))));
IkReal gconst29;
gconst29=((IkReal(-0.385225000000000))+(((IkReal(0.550000000000000))*(cj1)*(pz)))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.0412500000000000))*(sj1)))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(px)))+(pp)+(((IkReal(-0.0607500000000000))*(cj1)))+(((IkReal(0.550000000000000))*(px)*(sj1))));
IkReal gconst30;
gconst30=((IkReal(0.790000000000000))+(((IkReal(0.455000000000000))*(cj1)*(r02)))+(((px)*(r00)))+(((IkReal(0.455000000000000))*(r00)*(sj1)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.405000000000000))*(cj1)*(r00)))+(((IkReal(-0.405000000000000))*(r02)*(sj1))));
IkReal gconst31;
gconst31=((((IkReal(-0.180000000000000))*(r02)*(sj1)))+(((IkReal(0.180000000000000))*(cj1)*(r00)))+(((IkReal(-0.810000000000000))*(r00)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(r02))));
IkReal gconst32;
gconst32=((IkReal(0.790000000000000))+(((px)*(r00)))+(((IkReal(0.405000000000000))*(r02)*(sj1)))+(((IkReal(0.275000000000000))*(cj1)*(r02)))+(((IkReal(-0.405000000000000))*(cj1)*(r00)))+(((IkReal(0.0750000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.275000000000000))*(r00)*(sj1))));
IkReal gconst33;
gconst33=((IkReal(-0.253825000000000))+(((IkReal(-0.810000000000000))*(pz)*(sj1)))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.0607500000000000))*(cj1)))+(((IkReal(0.910000000000000))*(cj1)*(pz)))+(pp)+(((IkReal(0.0682500000000000))*(sj1)))+(((IkReal(0.910000000000000))*(px)*(sj1)))+(((IkReal(0.810000000000000))*(cj1)*(px))));
IkReal gconst34;
gconst34=((IkReal(-0.591300000000000))+(((IkReal(-1.62000000000000))*(px)*(sj1)))+(((IkReal(-0.360000000000000))*(pz)*(sj1)))+(((IkReal(-0.121500000000000))*(sj1)))+(((IkReal(0.360000000000000))*(cj1)*(px)))+(((IkReal(0.0270000000000000))*(cj1)))+(((IkReal(-1.62000000000000))*(cj1)*(pz))));
IkReal gconst35;
gconst35=((IkReal(-0.385225000000000))+(((IkReal(0.550000000000000))*(cj1)*(pz)))+(((IkReal(0.150000000000000))*(px)))+(((IkReal(0.0412500000000000))*(sj1)))+(((IkReal(0.810000000000000))*(pz)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(px)))+(pp)+(((IkReal(-0.0607500000000000))*(cj1)))+(((IkReal(0.550000000000000))*(px)*(sj1))));
IkReal x49=((IkReal(1.00000000000000))*(gconst26));
IkReal x50=((gconst30)*(gconst34));
IkReal x51=((gconst31)*(gconst33));
IkReal x52=((IkReal(1.00000000000000))*(gconst17));
IkReal x53=((gconst2)*(gconst6));
IkReal x54=((gconst11)*(gconst3));
IkReal x55=((gconst25)*(gconst29));
IkReal x56=((gconst14)*(gconst28));
IkReal x57=((IkReal(1.00000000000000))*(gconst14));
IkReal x58=((gconst21)*(gconst32));
IkReal x59=((gconst10)*(gconst6));
IkReal x60=((gconst21)*(gconst5));
IkReal x61=((gconst0)*(gconst8));
IkReal x62=((gconst13)*(gconst5));
IkReal x63=((IkReal(1.00000000000000))*(gconst9));
IkReal x64=((gconst30)*(gconst35));
IkReal x65=((gconst26)*(gconst27));
IkReal x66=((gconst22)*(gconst30));
IkReal x67=((IkReal(1.00000000000000))*(gconst12));
IkReal x68=((gconst2)*(gconst3));
IkReal x69=((gconst1)*(gconst5));
IkReal x70=((gconst19)*(gconst33));
IkReal x71=((gconst26)*(gconst28));
IkReal x72=((gconst11)*(gconst6));
IkReal x73=((gconst1)*(gconst7));
IkReal x74=((gconst18)*(gconst2));
IkReal x75=((gconst18)*(gconst34));
IkReal x76=((gconst24)*(gconst29));
IkReal x77=((IkReal(1.00000000000000))*(gconst23));
IkReal x78=((gconst20)*(gconst33));
IkReal x79=((gconst32)*(gconst33));
IkReal x80=((gconst0)*(gconst5));
IkReal x81=((IkReal(1.00000000000000))*(gconst18));
IkReal x82=((gconst21)*(gconst31));
IkReal x83=((gconst5)*(gconst9));
IkReal x84=((gconst4)*(gconst9));
IkReal x85=((gconst13)*(gconst29));
IkReal x86=((gconst14)*(gconst6));
IkReal x87=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x88=((gconst27)*(x79));
IkReal x89=((gconst2)*(gconst4)*(gconst7));
op[0]=((((IkReal(-1.00000000000000))*(x51)*(x55)))+(((x50)*(x55)))+(((x76)*(x79)))+(((IkReal(-1.00000000000000))*(x64)*(x76)))+(((x51)*(x71)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x50)))+(((x64)*(x65)))+(((IkReal(-1.00000000000000))*(x49)*(x88))));
op[1]=((((gconst16)*(gconst26)*(x51)))+(((x50)*(x85)))+(((x58)*(x76)))+(((IkReal(-1.00000000000000))*(gconst24)*(x52)*(x64)))+(((IkReal(-1.00000000000000))*(gconst27)*(x49)*(x78)))+(((gconst15)*(gconst26)*(x64)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x75)))+(((IkReal(-1.00000000000000))*(x55)*(x82)))+(((IkReal(-1.00000000000000))*(gconst16)*(x49)*(x50)))+(((gconst17)*(gconst25)*(x50)))+(((IkReal(-1.00000000000000))*(x51)*(x85)))+(((IkReal(-1.00000000000000))*(gconst15)*(x49)*(x79)))+(((IkReal(-1.00000000000000))*(x57)*(x88)))+(((IkReal(-1.00000000000000))*(x55)*(x70)))+(((IkReal(-1.00000000000000))*(x50)*(x56)))+(((gconst12)*(gconst29)*(x79)))+(((x76)*(x78)))+(((gconst14)*(gconst27)*(x64)))+(((IkReal(-1.00000000000000))*(gconst27)*(x49)*(x58)))+(((gconst18)*(gconst35)*(x65)))+(((x55)*(x75)))+(((gconst17)*(gconst24)*(x79)))+(((x51)*(x56)))+(((IkReal(-1.00000000000000))*(gconst28)*(x49)*(x66)))+(((x70)*(x71)))+(((IkReal(-1.00000000000000))*(gconst29)*(x64)*(x67)))+(((gconst23)*(gconst30)*(x65)))+(((IkReal(-1.00000000000000))*(gconst25)*(x51)*(x52)))+(((IkReal(-1.00000000000000))*(gconst30)*(x76)*(x77)))+(((IkReal(-1.00000000000000))*(gconst35)*(x76)*(x81)))+(((x55)*(x66)))+(((x71)*(x82))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((gconst22)*(gconst6)*(x69)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x53)))+(((gconst17)*(gconst9)*(x61)))+(((IkReal(-1.00000000000000))*(gconst19)*(x63)*(x69)))+(((IkReal(-1.00000000000000))*(gconst11)*(x80)*(x81)))+(((x59)*(x62)))+(((gconst1)*(gconst17)*(x59)))+(((IkReal(-1.00000000000000))*(gconst5)*(x67)*(x72)))+(((gconst10)*(gconst18)*(x69)))+(((IkReal(-1.00000000000000))*(gconst0)*(x52)*(x72)))+(((x60)*(x61)))+(((x54)*(x86)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x68)))+(((gconst11)*(gconst15)*(x53)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x53)))+(((IkReal(-1.00000000000000))*(gconst4)*(x57)*(x59)))+(((gconst20)*(gconst9)*(x80)))+(((IkReal(-1.00000000000000))*(gconst20)*(x63)*(x68)))+(((IkReal(-1.00000000000000))*(gconst7)*(x62)*(x63)))+(((gconst23)*(gconst3)*(x53)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x57)))+(((gconst14)*(gconst7)*(x84)))+(((IkReal(-1.00000000000000))*(x74)*(x87)))+(((x54)*(x74)))+(((IkReal(-1.00000000000000))*(x60)*(x73)))+(((IkReal(-1.00000000000000))*(gconst6)*(x77)*(x80)))+(((gconst21)*(x89)))+(((IkReal(-1.00000000000000))*(gconst9)*(x52)*(x73)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((gconst12)*(gconst8)*(x83)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x63)))+(((gconst19)*(gconst2)*(x84))));
op[8]=((((IkReal(-1.00000000000000))*(gconst8)*(x63)*(x68)))+(((x61)*(x83)))+(((x59)*(x69)))+(((IkReal(-1.00000000000000))*(gconst7)*(x63)*(x69)))+(((IkReal(-1.00000000000000))*(x53)*(x87)))+(((IkReal(-1.00000000000000))*(x72)*(x80)))+(((x53)*(x54)))+(((gconst2)*(gconst7)*(x84))));
polyroots8(op,zeror,numroots);
IkReal j0array[8], cj0array[8], sj0array[8], tempj0array[1];
int numsolutions = 0;
for(int ij0 = 0; ij0 < numroots; ++ij0)
{
IkReal htj0 = zeror[ij0];
tempj0array[0]=((IkReal(2.00000000000000))*(atan(htj0)));
for(int kj0 = 0; kj0 < 1; ++kj0)
{
j0array[numsolutions] = tempj0array[kj0];
if( j0array[numsolutions] > IKPI )
{
    j0array[numsolutions]-=IK2PI;
}
else if( j0array[numsolutions] < -IKPI )
{
    j0array[numsolutions]+=IK2PI;
}
sj0array[numsolutions] = IKsin(j0array[numsolutions]);
cj0array[numsolutions] = IKcos(j0array[numsolutions]);
numsolutions++;
}
}
bool j0valid[8]={true,true,true,true,true,true,true,true};
_nj0 = 8;
for(int ij0 = 0; ij0 < numsolutions; ++ij0)
    {
if( !j0valid[ij0] )
{
    continue;
}
    j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
htj0 = IKtan(j0/2);

_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < numsolutions; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x90=(sj1)*(sj1);
IkReal x91=(cj1)*(cj1);
IkReal x92=((cj0)*(r00));
IkReal x93=((r01)*(sj0));
IkReal x94=((IkReal(0.125651250000000))*(cj1));
IkReal x95=((r02)*(x90));
IkReal x96=((r02)*(x91));
IkReal x97=((IkReal(0.344250000000000))*(cj0)*(px));
IkReal x98=((IkReal(0.344250000000000))*(py)*(sj0));
IkReal x99=((IkReal(0.344250000000000))*(pz)*(x91));
IkReal x100=((IkReal(0.344250000000000))*(pz)*(x90));
gconst52=IKsign(((((x96)*(x97)))+(((x96)*(x98)))+(((IkReal(-0.0258187500000000))*(x96)))+(((IkReal(-0.0258187500000000))*(x95)))+(((IkReal(-1.00000000000000))*(x92)*(x99)))+(((IkReal(-1.00000000000000))*(x92)*(x94)))+(((x95)*(x97)))+(((x95)*(x98)))+(((IkReal(-1.00000000000000))*(x100)*(x92)))+(((IkReal(-1.00000000000000))*(x100)*(x93)))+(((IkReal(-0.125651250000000))*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(x93)*(x94)))+(((IkReal(-1.00000000000000))*(x93)*(x99)))));
IkReal x101=(sj1)*(sj1);
IkReal x102=(cj1)*(cj1);
IkReal x103=((px)*(r02));
IkReal x104=((pz)*(r00));
IkReal x105=((IkReal(13.3333333333333))*(sj0));
IkReal x106=((py)*(r02));
IkReal x107=((pz)*(r01));
IkReal x108=((IkReal(1.00000000000000))*(r02));
IkReal x109=((IkReal(4.86666666666667))*(cj1));
IkReal x110=((IkReal(13.3333333333333))*(cj0)*(x101));
IkReal x111=((IkReal(13.3333333333333))*(cj0)*(x102));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x101)*(x108)))+(((x103)*(x110)))+(((x103)*(x111)))+(((x102)*(x105)*(x106)))+(((IkReal(-1.00000000000000))*(cj0)*(r00)*(x109)))+(((x101)*(x105)*(x106)))+(((IkReal(-1.00000000000000))*(x101)*(x105)*(x107)))+(((IkReal(-1.00000000000000))*(r01)*(sj0)*(x109)))+(((IkReal(-1.00000000000000))*(x102)*(x105)*(x107)))+(((IkReal(-1.00000000000000))*(x102)*(x108)))+(((IkReal(-1.00000000000000))*(x104)*(x111)))+(((IkReal(-1.00000000000000))*(x104)*(x110)))+(((IkReal(-4.86666666666667))*(r02)*(sj1))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
if( IKabs(((gconst52)*(((IkReal(-0.0519030000000000))+(((IkReal(-0.295650000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.0479925000000000))*(cj1)))+(((IkReal(-0.0657000000000000))*(px)*(r00)))+(((IkReal(0.810000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.142200000000000))*(cj0)*(px)*(sj1)))+(((IkReal(-0.0607500000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj1)))+(((IkReal(-0.405000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.0607500000000000))*(cj1)*(pz)*(r02)))+(((IkReal(0.810000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(0.295650000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.0607500000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(-0.295650000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-0.0527377500000000))*(cj1)*(r02)))+(((IkReal(-0.0135000000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(0.0537502500000000))*(r01)*(sj0)*(sj1)))+(((IkReal(0.639900000000000))*(cj1)*(py)*(sj0)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj1)))+(((IkReal(0.810000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.0657000000000000))*(pz)*(r02)))+(((IkReal(0.0537502500000000))*(cj0)*(r00)*(sj1)))+(((IkReal(-0.295650000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.133963875000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(0.133963875000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.0607500000000000))*(cj1)*(px)*(r00)))+(((IkReal(0.129407625000000))*(r02)*(sj1)))+(((IkReal(0.295650000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.0221737500000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.180000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(0.142200000000000))*(py)*(sj0)*(sj1)))+(((IkReal(0.810000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(0.810000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(-0.0657000000000000))*(py)*(r01)))+(((IkReal(0.00492750000000000))*(cj0)*(r00)))+(((IkReal(0.810000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(0.295650000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-0.0106650000000000))*(sj1)))+(((IkReal(-0.142200000000000))*(cj1)*(pz)))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-0.405000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.295650000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0607500000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(0.0607500000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(0.0135000000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.810000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(0.639900000000000))*(cj0)*(cj1)*(px)))+(((IkReal(0.810000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(0.00492750000000000))*(r01)*(sj0)))+(((IkReal(0.810000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(0.0607500000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.0135000000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(-0.405000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(0.639900000000000))*(pz)*(sj1)))+(((IkReal(-0.0221737500000000))*(r02)*((cj1)*(cj1))))+(((IkReal(0.295650000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((IkReal(-0.233563500000000))+(((IkReal(0.0900000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(0.810000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.0607500000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(-0.0657000000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.0657000000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0287572500000000))*(r02)*(sj1)))+(((IkReal(0.241876125000000))*(cj0)*(r00)*(sj1)))+(((IkReal(-0.142200000000000))*(cj0)*(cj1)*(px)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-0.295650000000000))*(px)*(r00)))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(-0.0657000000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(0.0221737500000000))*(r01)*(sj0)))+(((IkReal(0.405000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(0.0657000000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.0607500000000000))*(pz)*(r02)*(sj1)))+(((IkReal(0.0135000000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.142200000000000))*(cj1)*(py)*(sj0)))+(((IkReal(-0.180000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.241876125000000))*(r01)*(sj0)*(sj1)))+(((IkReal(0.0607500000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.0221737500000000))*(cj0)*(r00)))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(0.810000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r02)))+(((IkReal(-0.0297697500000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.0607500000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(0.0106650000000000))*(cj1)))+(((IkReal(0.0607500000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.0657000000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-0.810000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(0.810000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(0.810000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(0.0657000000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.00492750000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(0.0135000000000000))*(cj1)*(px)*(r00)))+(((IkReal(-0.237319875000000))*(cj1)*(r02)))+(((IkReal(-0.0297697500000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.405000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(0.810000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.0607500000000000))*(py)*(r01)*(sj1)))+(((IkReal(-0.0479925000000000))*(sj1)))+(((IkReal(0.810000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.405000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(0.00492750000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.295650000000000))*(py)*(r01)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.639900000000000))*(cj0)*(px)*(sj1)))+(((IkReal(-0.0607500000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.142200000000000))*(pz)*(sj1)))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(-0.295650000000000))*(pz)*(r02)))+(((IkReal(0.0657000000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.639900000000000))*(cj1)*(pz)))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-0.0657000000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.639900000000000))*(py)*(sj0)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(px)*(pz)*(r00))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst52)*(((IkReal(-0.0519030000000000))+(((IkReal(-0.295650000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.0479925000000000))*(cj1)))+(((IkReal(-0.0657000000000000))*(px)*(r00)))+(((IkReal(0.810000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.142200000000000))*(cj0)*(px)*(sj1)))+(((IkReal(-0.0607500000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.0135000000000000))*(py)*(r01)*(sj1)))+(((IkReal(-0.405000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.0607500000000000))*(cj1)*(pz)*(r02)))+(((IkReal(0.810000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.180000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(0.295650000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(-0.0607500000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.0900000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(-0.295650000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(pz)*(r00)))+(((IkReal(-0.0527377500000000))*(cj1)*(r02)))+(((IkReal(-0.0135000000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(0.0537502500000000))*(r01)*(sj0)*(sj1)))+(((IkReal(0.639900000000000))*(cj1)*(py)*(sj0)))+(((IkReal(-0.0135000000000000))*(pz)*(r02)*(sj1)))+(((IkReal(0.810000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.0657000000000000))*(pz)*(r02)))+(((IkReal(0.0537502500000000))*(cj0)*(r00)*(sj1)))+(((IkReal(-0.295650000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.133963875000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.0900000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(0.133963875000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.0607500000000000))*(cj1)*(px)*(r00)))+(((IkReal(0.129407625000000))*(r02)*(sj1)))+(((IkReal(0.295650000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(-0.0135000000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.0221737500000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.180000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(0.142200000000000))*(py)*(sj0)*(sj1)))+(((IkReal(0.810000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(0.810000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(-0.0657000000000000))*(py)*(r01)))+(((IkReal(0.00492750000000000))*(cj0)*(r00)))+(((IkReal(0.810000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(0.295650000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-0.0106650000000000))*(sj1)))+(((IkReal(-0.142200000000000))*(cj1)*(pz)))+(((IkReal(0.180000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(-0.405000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(-0.295650000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0607500000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(0.180000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(0.0607500000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(0.0135000000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.810000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(0.639900000000000))*(cj0)*(cj1)*(px)))+(((IkReal(0.810000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.180000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(0.00492750000000000))*(r01)*(sj0)))+(((IkReal(0.810000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(0.0607500000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.0135000000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(-0.405000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(0.639900000000000))*(pz)*(sj1)))+(((IkReal(-0.0221737500000000))*(r02)*((cj1)*(cj1))))+(((IkReal(0.295650000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r02)))))), ((gconst52)*(((IkReal(-0.233563500000000))+(((IkReal(0.0900000000000000))*(cj0)*(cj1)*(pp)*(r00)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(r00)*((px)*(px))))+(((IkReal(0.810000000000000))*(cj0)*(px)*(pz)*(r02)*(sj1)))+(((IkReal(-0.0607500000000000))*(cj0)*(cj1)*(px)*(r02)))+(((IkReal(-0.0657000000000000))*(py)*(r02)*(sj0)*((sj1)*(sj1))))+(((IkReal(0.0657000000000000))*(cj0)*(pz)*(r00)*((sj1)*(sj1))))+(((IkReal(-0.0287572500000000))*(r02)*(sj1)))+(((IkReal(0.241876125000000))*(cj0)*(r00)*(sj1)))+(((IkReal(-0.142200000000000))*(cj0)*(cj1)*(px)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(py)*(r01)))+(((IkReal(-0.180000000000000))*(r02)*(sj1)*((pz)*(pz))))+(((IkReal(-0.295650000000000))*(px)*(r00)))+(((IkReal(0.0135000000000000))*(cj0)*(pz)*(r00)*(sj1)))+(((IkReal(-0.180000000000000))*(cj0)*(cj1)*(px)*(pz)*(r02)))+(((IkReal(-0.0657000000000000))*(cj0)*(px)*(r02)*((cj1)*(cj1))))+(((IkReal(0.0221737500000000))*(r01)*(sj0)))+(((IkReal(0.405000000000000))*(cj1)*(pp)*(r02)))+(((IkReal(0.0657000000000000))*(pz)*(r01)*(sj0)*((sj1)*(sj1))))+(((IkReal(-0.0607500000000000))*(pz)*(r02)*(sj1)))+(((IkReal(0.0135000000000000))*(cj1)*(py)*(r01)))+(((IkReal(-0.142200000000000))*(cj1)*(py)*(sj0)))+(((IkReal(-0.180000000000000))*(cj1)*(r01)*(sj0)*((py)*(py))))+(((IkReal(0.241876125000000))*(r01)*(sj0)*(sj1)))+(((IkReal(0.0607500000000000))*(cj0)*(cj1)*(pz)*(r00)))+(((IkReal(0.0221737500000000))*(cj0)*(r00)))+(((IkReal(0.0135000000000000))*(pz)*(r01)*(sj0)*(sj1)))+(((IkReal(0.810000000000000))*(px)*(py)*(r00)*(sj0)*(sj1)))+(((IkReal(0.0900000000000000))*(cj1)*(pp)*(r01)*(sj0)))+(((IkReal(-0.180000000000000))*(py)*(pz)*(r01)*(sj1)))+(((IkReal(0.0135000000000000))*(cj1)*(pz)*(r02)))+(((IkReal(-0.0297697500000000))*(cj1)*(r01)*(sj0)))+(((IkReal(-0.0607500000000000))*(cj1)*(py)*(r02)*(sj0)))+(((IkReal(0.0106650000000000))*(cj1)))+(((IkReal(0.0607500000000000))*(cj1)*(pz)*(r01)*(sj0)))+(((IkReal(-0.0657000000000000))*(cj0)*(px)*(r02)*((sj1)*(sj1))))+(((IkReal(-0.810000000000000))*(cj1)*(py)*(pz)*(r01)))+(((IkReal(0.810000000000000))*(r01)*(sj0)*(sj1)*((py)*(py))))+(((IkReal(0.810000000000000))*(py)*(pz)*(r02)*(sj0)*(sj1)))+(((IkReal(0.0657000000000000))*(pz)*(r01)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.00492750000000000))*(r02)*((cj1)*(cj1))))+(((IkReal(0.0135000000000000))*(cj1)*(px)*(r00)))+(((IkReal(-0.237319875000000))*(cj1)*(r02)))+(((IkReal(-0.0297697500000000))*(cj0)*(cj1)*(r00)))+(((IkReal(-0.405000000000000))*(pp)*(r01)*(sj0)*(sj1)))+(((IkReal(-0.180000000000000))*(cj1)*(py)*(pz)*(r02)*(sj0)))+(((IkReal(0.810000000000000))*(cj0)*(r00)*(sj1)*((px)*(px))))+(((IkReal(-0.0607500000000000))*(py)*(r01)*(sj1)))+(((IkReal(-0.0479925000000000))*(sj1)))+(((IkReal(0.810000000000000))*(cj0)*(px)*(py)*(r01)*(sj1)))+(((IkReal(-0.405000000000000))*(cj0)*(pp)*(r00)*(sj1)))+(((IkReal(0.00492750000000000))*(r02)*((sj1)*(sj1))))+(((IkReal(-0.295650000000000))*(py)*(r01)))+(((IkReal(-0.180000000000000))*(cj1)*(px)*(py)*(r00)*(sj0)))+(((IkReal(0.639900000000000))*(cj0)*(px)*(sj1)))+(((IkReal(-0.0607500000000000))*(px)*(r00)*(sj1)))+(((IkReal(-0.0135000000000000))*(py)*(r02)*(sj0)*(sj1)))+(((IkReal(-0.142200000000000))*(pz)*(sj1)))+(((IkReal(0.0900000000000000))*(pp)*(r02)*(sj1)))+(((IkReal(-0.295650000000000))*(pz)*(r02)))+(((IkReal(0.0657000000000000))*(cj0)*(pz)*(r00)*((cj1)*(cj1))))+(((IkReal(-0.639900000000000))*(cj1)*(pz)))+(((IkReal(-0.180000000000000))*(px)*(pz)*(r00)*(sj1)))+(((IkReal(-0.0657000000000000))*(py)*(r02)*(sj0)*((cj1)*(cj1))))+(((IkReal(0.639900000000000))*(py)*(sj0)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(r02)*((pz)*(pz))))+(((IkReal(-0.0135000000000000))*(cj0)*(px)*(r02)*(sj1)))+(((IkReal(-0.810000000000000))*(cj1)*(px)*(pz)*(r00)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
IkReal x112=IKsin(j2);
IkReal x113=IKcos(j2);
IkReal x114=((IkReal(0.810000000000000))*(sj1));
IkReal x115=((cj0)*(r00));
IkReal x116=((cj1)*(r01));
IkReal x117=((cj1)*(r02));
IkReal x118=((cj1)*(pz));
IkReal x119=((r01)*(sj0));
IkReal x120=((IkReal(0.365000000000000))*(sj1));
IkReal x121=((py)*(sj0));
IkReal x122=((cj0)*(px));
IkReal x123=((IkReal(0.730000000000000))*(sj1));
IkReal x124=((sj0)*(x112));
IkReal x125=((IkReal(0.405000000000000))*(x113));
IkReal x126=((IkReal(0.180000000000000))*(x122));
IkReal x127=((cj1)*(x112));
IkReal x128=((sj1)*(x112));
IkReal x129=((sj1)*(x113));
IkReal x130=((cj1)*(x113));
IkReal x131=((IkReal(0.0900000000000000))*(x129));
evalcond[0]=((IkReal(-0.319525000000000))+(((IkReal(-0.295650000000000))*(x112)))+(((IkReal(0.0657000000000000))*(x113)))+(((IkReal(-1.00000000000000))*(x126)*(x127)))+(((IkReal(-1.00000000000000))*(x126)*(x129)))+(((IkReal(-0.180000000000000))*(x121)*(x127)))+(((IkReal(-0.180000000000000))*(x121)*(x129)))+(((IkReal(0.730000000000000))*(x118)))+(((IkReal(-1.00000000000000))*(pz)*(x113)*(x114)))+(((IkReal(0.0547500000000000))*(sj1)))+(((IkReal(0.0607500000000000))*(x130)))+(((IkReal(-0.0607500000000000))*(x128)))+(pp)+(((IkReal(-0.150000000000000))*(x122)))+(((IkReal(-0.150000000000000))*(x121)))+(((IkReal(-1.00000000000000))*(x121)*(x123)))+(((IkReal(0.0135000000000000))*(x127)))+(((IkReal(0.0135000000000000))*(x129)))+(((IkReal(0.180000000000000))*(x113)*(x118)))+(((IkReal(-0.180000000000000))*(pz)*(x128)))+(((IkReal(-0.810000000000000))*(x121)*(x130)))+(((IkReal(-0.810000000000000))*(x112)*(x118)))+(((IkReal(-0.810000000000000))*(x122)*(x130)))+(((x112)*(x114)*(x122)))+(((x112)*(x114)*(x121)))+(((IkReal(-1.00000000000000))*(x122)*(x123))));
evalcond[1]=((IkReal(0.790000000000000))+(((IkReal(0.365000000000000))*(x117)))+(((IkReal(-1.00000000000000))*(x119)*(x120)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x125)))+(((IkReal(-0.405000000000000))*(x112)*(x117)))+(((px)*(r00)))+(((IkReal(-1.00000000000000))*(x115)*(x131)))+(((IkReal(-0.0750000000000000))*(x119)))+(((IkReal(-0.0750000000000000))*(x115)))+(((IkReal(-1.00000000000000))*(x115)*(x120)))+(((IkReal(-0.0900000000000000))*(r02)*(x128)))+(((IkReal(0.405000000000000))*(x115)*(x128)))+(((IkReal(0.0900000000000000))*(x113)*(x117)))+(((IkReal(-0.0900000000000000))*(x115)*(x127)))+(((IkReal(-1.00000000000000))*(cj1)*(x115)*(x125)))+(((IkReal(-0.0900000000000000))*(x116)*(x124)))+(((IkReal(-1.00000000000000))*(sj0)*(x116)*(x125)))+(((IkReal(0.405000000000000))*(x119)*(x128)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-1.00000000000000))*(x119)*(x131))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x132=((sj1)*(sj2));
IkReal x133=((cj1)*(cj2));
IkReal x134=((cj2)*(sj1));
IkReal x135=((IkReal(9.87500000000000))*(r02));
IkReal x136=((IkReal(12.5000000000000))*(pz));
IkReal x137=((cj1)*(sj2));
IkReal x138=((IkReal(9.87500000000000))*(r01)*(sj0));
IkReal x139=((IkReal(12.5000000000000))*(cj0)*(px));
IkReal x140=((IkReal(12.5000000000000))*(py)*(sj0));
IkReal x141=((IkReal(9.87500000000000))*(cj0)*(r00));
cj4array[0]=((IkReal(-5.06250000000000))+(((IkReal(-1.00000000000000))*(x132)*(x140)))+(((IkReal(-1.00000000000000))*(x132)*(x141)))+(((IkReal(0.937500000000000))*(x132)))+(((x135)*(x137)))+(((x136)*(x137)))+(((IkReal(4.56250000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x132)*(x138)))+(((IkReal(-1.00000000000000))*(x132)*(x139)))+(((x134)*(x135)))+(((x134)*(x136)))+(((x133)*(x138)))+(((x133)*(x139)))+(((x133)*(x141)))+(((x133)*(x140)))+(((IkReal(-0.937500000000000))*(x133))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x142=((cj1)*(cj2));
IkReal x143=((IkReal(1.00000000000000))*(cj0));
IkReal x144=((sj1)*(sj2));
IkReal x145=((IkReal(1.00000000000000))*(sj0));
IkReal x146=((sj0)*(x144));
IkReal x147=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x148=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x144)))+(((r01)*(x146)))+(((IkReal(-1.00000000000000))*(r01)*(x142)*(x145)))+(((IkReal(-1.00000000000000))*(r02)*(x148)))+(((IkReal(-1.00000000000000))*(r02)*(x147)))+(((IkReal(-1.00000000000000))*(r00)*(x142)*(x143))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((cj0)*(px)*(x144)))+(((IkReal(0.0750000000000000))*(x142)))+(((IkReal(-1.00000000000000))*(pz)*(x147)))+(((IkReal(-1.00000000000000))*(pz)*(x148)))+(((IkReal(-1.00000000000000))*(px)*(x142)*(x143)))+(((IkReal(-1.00000000000000))*(py)*(x142)*(x145)))+(((IkReal(-0.0750000000000000))*(x144)))+(((py)*(x146))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst64;
gconst64=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst65;
gconst65=IkReal(1.58000000000000);
IkReal gconst66;
gconst66=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst67;
gconst67=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst68;
gconst68=IkReal(-2.00000000000000);
IkReal gconst69;
gconst69=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst70;
gconst70=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst71;
gconst71=IkReal(1.58000000000000);
IkReal gconst72;
gconst72=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(1.58000000000000);
IkReal gconst76;
gconst76=IkReal(-1.58000000000000);
IkReal gconst77;
gconst77=IkReal(-2.00000000000000);
IkReal gconst78;
gconst78=IkReal(2.00000000000000);
IkReal gconst79;
gconst79=IkReal(1.58000000000000);
IkReal gconst80;
gconst80=IkReal(-1.58000000000000);
IkReal gconst81;
gconst81=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst82;
gconst82=IkReal(2.00000000000000);
IkReal gconst83;
gconst83=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst84;
gconst84=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst85;
gconst85=IkReal(-1.58000000000000);
IkReal gconst86;
gconst86=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst87;
gconst87=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst88;
gconst88=IkReal(2.00000000000000);
IkReal gconst89;
gconst89=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst90;
gconst90=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst91;
gconst91=IkReal(-1.58000000000000);
IkReal gconst92;
gconst92=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal x149=((gconst89)*(gconst90));
IkReal x150=((gconst83)*(gconst84));
IkReal x151=((gconst81)*(gconst86));
IkReal x152=((gconst87)*(gconst92));
IkReal x153=((gconst82)*(gconst86));
IkReal x154=((gconst87)*(gconst91));
IkReal x155=((gconst88)*(gconst90));
IkReal x156=((gconst83)*(gconst85));
dummyeval[0]=((((x153)*(x154)))+(((x149)*(x151)))+(((IkReal(-1.00000000000000))*(x149)*(x150)))+(((x150)*(x152)))+(((x155)*(x156)))+(((IkReal(-1.00000000000000))*(x151)*(x152)))+(((IkReal(-1.00000000000000))*(x153)*(x155)))+(((IkReal(-1.00000000000000))*(x154)*(x156))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x157=((gconst63)*(gconst67));
IkReal x158=((IkReal(1.00000000000000))*(gconst65));
IkReal x159=((IkReal(1.00000000000000))*(gconst86));
IkReal x160=((gconst80)*(gconst87));
IkReal x161=((gconst68)*(gconst82));
IkReal x162=((gconst72)*(gconst75));
IkReal x163=((gconst72)*(gconst77));
IkReal x164=((gconst61)*(gconst76));
IkReal x165=((gconst63)*(gconst87));
IkReal x166=((gconst61)*(gconst66));
IkReal x167=((gconst70)*(gconst78));
IkReal x168=((gconst86)*(gconst89));
IkReal x169=((gconst73)*(gconst90));
IkReal x170=((gconst64)*(gconst72));
IkReal x171=((gconst87)*(gconst92));
IkReal x172=((IkReal(1.00000000000000))*(gconst83));
IkReal x173=((gconst85)*(gconst91));
IkReal x174=((gconst69)*(gconst90));
IkReal x175=((gconst78)*(gconst90));
IkReal x176=((IkReal(1.00000000000000))*(gconst70));
IkReal x177=((gconst62)*(gconst66));
IkReal x178=((gconst62)*(gconst86));
IkReal x179=((gconst67)*(gconst71));
IkReal x180=((gconst72)*(gconst84));
IkReal x181=((gconst65)*(gconst68));
IkReal x182=((gconst74)*(gconst79));
IkReal x183=((gconst63)*(gconst69));
IkReal x184=((IkReal(1.00000000000000))*(gconst67));
IkReal x185=((gconst66)*(gconst73));
IkReal x186=((gconst70)*(gconst81));
IkReal x187=((gconst63)*(gconst70));
IkReal x188=((gconst79)*(gconst81));
IkReal x189=((gconst78)*(gconst86));
IkReal x190=((gconst84)*(gconst92));
IkReal x191=((IkReal(1.00000000000000))*(gconst79));
IkReal x192=((gconst83)*(gconst84));
IkReal x193=((gconst75)*(gconst83));
IkReal x194=((gconst64)*(gconst74));
IkReal x195=((gconst81)*(gconst92));
IkReal x196=((gconst63)*(gconst64));
IkReal x197=((gconst76)*(gconst89));
IkReal x198=((gconst65)*(gconst88));
IkReal x199=((gconst79)*(gconst84));
IkReal x200=((gconst87)*(gconst91));
IkReal x201=((gconst81)*(gconst86));
IkReal x202=((gconst69)*(gconst79));
IkReal x203=((gconst67)*(gconst83));
IkReal x204=((gconst82)*(gconst88));
IkReal x205=((gconst69)*(gconst70));
IkReal x206=((gconst67)*(gconst74));
IkReal x207=((gconst70)*(gconst89));
IkReal x208=((IkReal(1.00000000000000))*(gconst77));
IkReal x209=((IkReal(1.00000000000000))*(gconst81));
IkReal x210=((gconst68)*(gconst85));
IkReal x211=((gconst83)*(gconst90));
IkReal x212=((gconst62)*(gconst68));
IkReal x213=((gconst82)*(gconst86));
IkReal x214=((gconst67)*(gconst91));
IkReal x215=((IkReal(1.00000000000000))*(gconst63));
IkReal x216=((gconst85)*(gconst88));
IkReal x217=((gconst74)*(gconst84));
IkReal x218=((gconst89)*(gconst90));
IkReal x219=((gconst72)*(gconst87));
IkReal x220=((gconst73)*(gconst76));
IkReal x221=((gconst66)*(gconst81));
IkReal x222=((gconst63)*(gconst90));
IkReal x223=((gconst63)*(gconst77));
IkReal x224=((gconst74)*(gconst75));
IkReal x225=((gconst66)*(gconst82));
IkReal x226=((gconst71)*(gconst87));
IkReal x227=((IkReal(1.00000000000000))*(gconst90));
IkReal x228=((gconst71)*(gconst77));
IkReal x229=((gconst77)*(gconst80));
IkReal x230=((gconst76)*(gconst82));
IkReal x231=((gconst77)*(gconst91));
IkReal x232=((gconst70)*(gconst83));
IkReal x233=((gconst62)*(gconst76));
IkReal x234=((gconst81)*(gconst90));
IkReal x235=((gconst77)*(gconst92));
IkReal x236=((gconst74)*(x216));
op[0]=((((x211)*(x216)))+(((x168)*(x234)))+(((x171)*(x192)))+(((IkReal(-1.00000000000000))*(gconst90)*(x159)*(x204)))+(((IkReal(-1.00000000000000))*(gconst81)*(x159)*(x171)))+(((x200)*(x213)))+(((IkReal(-1.00000000000000))*(gconst84)*(x172)*(x218)))+(((IkReal(-1.00000000000000))*(gconst87)*(x172)*(x173))));
op[1]=((((x175)*(x201)))+(((IkReal(-1.00000000000000))*(gconst77)*(x172)*(x173)))+(((IkReal(-1.00000000000000))*(gconst81)*(x159)*(x160)))+(((x160)*(x192)))+(((x168)*(x188)))+(((gconst79)*(gconst83)*(x216)))+(((x168)*(x169)))+(((IkReal(-1.00000000000000))*(gconst77)*(x159)*(x195)))+(((IkReal(-1.00000000000000))*(gconst84)*(x172)*(x175)))+(((IkReal(-1.00000000000000))*(x217)*(x218)))+(((x200)*(x230)))+(((x171)*(x217)))+(((IkReal(-1.00000000000000))*(gconst79)*(x159)*(x204)))+(((IkReal(-1.00000000000000))*(gconst73)*(x159)*(x171)))+(((x197)*(x234)))+(((gconst77)*(gconst83)*(x190)))+(((x213)*(x231)))+(((IkReal(-1.00000000000000))*(gconst76)*(x204)*(x227)))+(((IkReal(-1.00000000000000))*(gconst76)*(x171)*(x209)))+(((x171)*(x193)))+(((gconst90)*(x236)))+(((IkReal(-1.00000000000000))*(gconst89)*(x172)*(x199)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst87)*(x173)))+(((IkReal(-1.00000000000000))*(gconst75)*(x172)*(x218))));
op[2]=((((IkReal(-1.00000000000000))*(gconst75)*(x172)*(x175)))+(((x192)*(x229)))+(((IkReal(-1.00000000000000))*(x165)*(x173)))+(((x160)*(x193)))+(((x169)*(x189)))+(((IkReal(-1.00000000000000))*(gconst61)*(x159)*(x171)))+(((IkReal(-1.00000000000000))*(gconst84)*(x215)*(x218)))+(((IkReal(-1.00000000000000))*(gconst66)*(x171)*(x209)))+(((IkReal(-1.00000000000000))*(gconst64)*(x172)*(x218)))+(((x216)*(x222)))+(((x168)*(x186)))+(((x210)*(x211)))+(((x169)*(x197)))+(((x213)*(x214)))+(((IkReal(-1.00000000000000))*(gconst81)*(x159)*(x219)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst88)*(gconst90)*(x159)))+(((gconst73)*(gconst79)*(x168)))+(((x171)*(x224)))+(((IkReal(-1.00000000000000))*(gconst84)*(x172)*(x174)))+(((x160)*(x217)))+(((x188)*(x197)))+(((IkReal(-1.00000000000000))*(gconst70)*(x159)*(x204)))+(((x182)*(x216)))+(((x190)*(x203)))+(((x213)*(x226)))+(((IkReal(-1.00000000000000))*(gconst76)*(x160)*(x209)))+(((IkReal(-1.00000000000000))*(gconst85)*(x172)*(x226)))+(((x188)*(x189)))+(((gconst61)*(gconst90)*(x168)))+(((x198)*(x211)))+(((x230)*(x231)))+(((IkReal(-1.00000000000000))*(gconst67)*(x159)*(x195)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst89)*(x172)))+(((IkReal(-1.00000000000000))*(x171)*(x220)))+(((gconst74)*(gconst77)*(x190)))+(((IkReal(-1.00000000000000))*(gconst66)*(x204)*(x227)))+(((x165)*(x190)))+(((x178)*(x200)))+(((IkReal(-1.00000000000000))*(gconst84)*(x172)*(x207)))+(((x218)*(x221)))+(((IkReal(-1.00000000000000))*(x218)*(x224)))+(((IkReal(-1.00000000000000))*(gconst73)*(x159)*(x160)))+(((x200)*(x225)))+(((IkReal(-1.00000000000000))*(x175)*(x217)))+(((IkReal(-1.00000000000000))*(gconst78)*(x172)*(x199)))+(((IkReal(-1.00000000000000))*(gconst76)*(x195)*(x208)))+(((IkReal(-1.00000000000000))*(gconst74)*(x173)*(x208)))+(((x216)*(x232)))+(((gconst64)*(gconst83)*(x171)))+(((IkReal(-1.00000000000000))*(gconst76)*(x191)*(x204)))+(((IkReal(-1.00000000000000))*(gconst84)*(gconst89)*(x182)))+(((gconst83)*(gconst87)*(x180)))+(((IkReal(-1.00000000000000))*(gconst73)*(x159)*(x235)))+(((x193)*(x235)))+(((IkReal(-1.00000000000000))*(gconst90)*(x159)*(x161)))+(((x174)*(x201)))+(((gconst76)*(gconst81)*(x175)))+(((IkReal(-1.00000000000000))*(gconst67)*(x172)*(x173)))+(((IkReal(-1.00000000000000))*(gconst81)*(x159)*(x229)))+(((IkReal(-1.00000000000000))*(gconst83)*(x158)*(x200))));
op[3]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst80)*(gconst87)))+(((gconst75)*(gconst77)*(gconst80)*(gconst83)))+(((gconst71)*(gconst77)*(gconst82)*(gconst86)))+(((gconst65)*(gconst74)*(gconst88)*(gconst90)))+(((gconst67)*(gconst80)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst89)*(gconst90)))+(((gconst61)*(gconst78)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst79)*(gconst82)*(gconst86)))+(((gconst61)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst77)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst81)*(gconst92)))+(((gconst63)*(gconst79)*(gconst85)*(gconst88)))+(((gconst62)*(gconst76)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst83)*(gconst90)))+(((gconst64)*(gconst74)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst80)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst79)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst88)*(gconst90)))+(((gconst71)*(gconst76)*(gconst82)*(gconst87)))+(((gconst70)*(gconst74)*(gconst85)*(gconst88)))+(((gconst74)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst82)*(gconst90)))+(((gconst74)*(gconst75)*(gconst80)*(gconst87)))+(((gconst62)*(gconst77)*(gconst86)*(gconst91)))+(((gconst69)*(gconst76)*(gconst81)*(gconst90)))+(((gconst66)*(gconst79)*(gconst81)*(gconst89)))+(((gconst69)*(gconst73)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst80)*(gconst86)*(gconst87)))+(((gconst66)*(gconst77)*(gconst82)*(gconst91)))+(((gconst70)*(gconst73)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst79)*(gconst86)*(gconst88)))+(((gconst74)*(gconst77)*(gconst80)*(gconst84)))+(((gconst70)*(gconst78)*(gconst81)*(gconst86)))+(((gconst63)*(gconst75)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst79)*(gconst83)*(gconst89)))+(((gconst68)*(gconst74)*(gconst85)*(gconst90)))+(((gconst64)*(gconst77)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst92)))+(((gconst65)*(gconst79)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst82)*(gconst88)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst86)*(gconst87)))+(((gconst72)*(gconst77)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst86)*(gconst92)))+(((gconst69)*(gconst79)*(gconst81)*(gconst86)))+(((gconst67)*(gconst76)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst83)*(gconst90)))+(((gconst61)*(gconst76)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst84)*(gconst89)))+(((gconst72)*(gconst75)*(gconst83)*(gconst87)))+(((gconst63)*(gconst80)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst78)*(gconst79)*(gconst83)))+(((gconst63)*(gconst77)*(gconst84)*(gconst92)))+(((gconst73)*(gconst78)*(gconst79)*(gconst86)))+(((gconst70)*(gconst76)*(gconst81)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst78)*(gconst83)*(gconst84)))+(((gconst66)*(gconst73)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst83)*(gconst89)))+(((gconst68)*(gconst79)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst74)*(gconst85)*(gconst91)))+(((gconst73)*(gconst76)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst77)*(gconst83)*(gconst91)))+(((gconst67)*(gconst74)*(gconst84)*(gconst92)))+(((gconst72)*(gconst74)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst84)*(gconst89)))+(((gconst76)*(gconst78)*(gconst79)*(gconst81)))+(((gconst66)*(gconst78)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst87)*(gconst92)))+(((gconst73)*(gconst76)*(gconst79)*(gconst89)))+(((gconst67)*(gconst75)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst80)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst78)*(gconst79)*(gconst84))));
op[4]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst83)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst87)*(gconst92)))+(((gconst64)*(gconst72)*(gconst83)*(gconst87)))+(((gconst64)*(gconst74)*(gconst80)*(gconst87)))+(((gconst63)*(gconst70)*(gconst85)*(gconst88)))+(((gconst68)*(gconst70)*(gconst83)*(gconst85)))+(((gconst67)*(gconst74)*(gconst80)*(gconst84)))+(((gconst73)*(gconst76)*(gconst78)*(gconst79)))+(((gconst64)*(gconst67)*(gconst83)*(gconst92)))+(((gconst67)*(gconst72)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst70)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst79)*(gconst83)))+(((gconst64)*(gconst74)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst90)))+(((gconst71)*(gconst76)*(gconst77)*(gconst82)))+(((gconst70)*(gconst73)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst84)*(gconst89)))+(((gconst61)*(gconst66)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst92)))+(((gconst62)*(gconst76)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst77)*(gconst81)))+(((gconst61)*(gconst69)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst80)))+(((gconst66)*(gconst70)*(gconst81)*(gconst89)))+(((gconst61)*(gconst70)*(gconst86)*(gconst89)))+(((gconst63)*(gconst64)*(gconst87)*(gconst92)))+(((gconst61)*(gconst78)*(gconst79)*(gconst86)))+(((gconst68)*(gconst74)*(gconst79)*(gconst85)))+(((gconst67)*(gconst75)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst86)*(gconst87)))+(((gconst66)*(gconst69)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst79)*(gconst88)))+(((gconst62)*(gconst66)*(gconst87)*(gconst91)))+(((gconst70)*(gconst76)*(gconst78)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst79)*(gconst81)))+(((gconst66)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst82)*(gconst86)))+(((gconst65)*(gconst70)*(gconst83)*(gconst88)))+(((gconst67)*(gconst71)*(gconst82)*(gconst86)))+(((gconst63)*(gconst65)*(gconst88)*(gconst90)))+(((gconst63)*(gconst75)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst87)*(gconst91)))+(((gconst63)*(gconst68)*(gconst85)*(gconst90)))+(((gconst70)*(gconst73)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst87)))+(((gconst61)*(gconst76)*(gconst78)*(gconst90)))+(((gconst72)*(gconst74)*(gconst75)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst84)*(gconst90)))+(((gconst62)*(gconst71)*(gconst86)*(gconst87)))+(((gconst69)*(gconst73)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst80)*(gconst81)))+(((gconst72)*(gconst75)*(gconst77)*(gconst83)))+(((gconst69)*(gconst70)*(gconst81)*(gconst86)))+(((gconst65)*(gconst68)*(gconst83)*(gconst90)))+(((gconst63)*(gconst67)*(gconst84)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst92)))+(((gconst74)*(gconst75)*(gconst77)*(gconst80)))+(((gconst63)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst80)*(gconst86)))+(((gconst66)*(gconst73)*(gconst78)*(gconst90)))+(((gconst66)*(gconst73)*(gconst79)*(gconst89)))+(((gconst61)*(gconst76)*(gconst79)*(gconst89)))+(((gconst72)*(gconst74)*(gconst77)*(gconst84)))+(((gconst66)*(gconst78)*(gconst79)*(gconst81)))+(((gconst65)*(gconst74)*(gconst79)*(gconst88)))+(((gconst64)*(gconst77)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst88)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst79)*(gconst89)))+(((gconst66)*(gconst67)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst79)*(gconst89)))+(((gconst62)*(gconst67)*(gconst86)*(gconst91)))+(((gconst69)*(gconst73)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst77)*(gconst86)))+(((gconst63)*(gconst77)*(gconst80)*(gconst84)))+(((gconst63)*(gconst72)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst83)*(gconst87))));
op[5]=((((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst79)*(gconst88)))+(((gconst64)*(gconst72)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst91)))+(((gconst61)*(gconst66)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst66)*(gconst70)*(gconst78)*(gconst81)))+(((gconst66)*(gconst73)*(gconst78)*(gconst79)))+(((gconst63)*(gconst75)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst79)))+(((gconst63)*(gconst72)*(gconst75)*(gconst87)))+(((gconst63)*(gconst67)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst92)))+(((gconst69)*(gconst70)*(gconst76)*(gconst81)))+(((gconst64)*(gconst74)*(gconst77)*(gconst80)))+(((gconst61)*(gconst70)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((gconst68)*(gconst70)*(gconst74)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst79)*(gconst84)))+(((gconst63)*(gconst64)*(gconst77)*(gconst92)))+(((gconst63)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst80)*(gconst86)))+(((gconst63)*(gconst72)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst76)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst80)))+(((gconst69)*(gconst70)*(gconst73)*(gconst86)))+(((gconst65)*(gconst68)*(gconst79)*(gconst83)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst65)*(gconst68)*(gconst74)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst76)*(gconst82)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((gconst66)*(gconst70)*(gconst73)*(gconst89)))+(((gconst72)*(gconst74)*(gconst75)*(gconst77)))+(((gconst69)*(gconst73)*(gconst76)*(gconst79)))+(((gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst62)*(gconst66)*(gconst77)*(gconst91)))+(((gconst62)*(gconst67)*(gconst76)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst74)*(gconst91)))+(((gconst65)*(gconst70)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((gconst61)*(gconst70)*(gconst78)*(gconst86)))+(((gconst61)*(gconst69)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst75)*(gconst89)))+(((gconst64)*(gconst67)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst79)*(gconst86)))+(((gconst61)*(gconst66)*(gconst79)*(gconst89)))+(((gconst67)*(gconst72)*(gconst74)*(gconst84)))+(((gconst63)*(gconst68)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst61)*(gconst69)*(gconst79)*(gconst86)))+(((gconst61)*(gconst76)*(gconst78)*(gconst79)))+(((gconst67)*(gconst72)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst80)))+(((gconst66)*(gconst69)*(gconst79)*(gconst81)))+(((gconst63)*(gconst67)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst74)*(gconst85)))+(((gconst70)*(gconst73)*(gconst76)*(gconst78)))+(((gconst63)*(gconst65)*(gconst79)*(gconst88)))+(((gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst64)*(gconst67)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst87)))+(((gconst64)*(gconst72)*(gconst74)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst90)))+(((gconst66)*(gconst69)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst74)*(gconst87))));
op[6]=((((x157)*(x180)))+(((gconst61)*(gconst86)*(x205)))+(((x205)*(x220)))+(((IkReal(-1.00000000000000))*(gconst89)*(x176)*(x196)))+(((IkReal(-1.00000000000000))*(x163)*(x164)))+(((IkReal(-1.00000000000000))*(gconst83)*(x158)*(x179)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(x159)))+(((x196)*(x229)))+(((IkReal(-1.00000000000000))*(gconst88)*(x176)*(x177)))+(((IkReal(-1.00000000000000))*(gconst71)*(x158)*(x165)))+(((IkReal(-1.00000000000000))*(gconst68)*(x177)*(x227)))+(((IkReal(-1.00000000000000))*(x166)*(x219)))+(((x178)*(x179)))+(((gconst75)*(gconst80)*(x157)))+(((gconst64)*(gconst92)*(x157)))+(((x187)*(x198)))+(((IkReal(-1.00000000000000))*(gconst92)*(x166)*(x184)))+(((x228)*(x233)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst85)*(x157)))+(((x170)*(x203)))+(((gconst67)*(gconst80)*(x194)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(x182)))+(((x177)*(x214)))+(((IkReal(-1.00000000000000))*(gconst91)*(x157)*(x158)))+(((x185)*(x202)))+(((IkReal(-1.00000000000000))*(x174)*(x196)))+(((x162)*(x206)))+(((IkReal(-1.00000000000000))*(gconst75)*(x167)*(x215)))+(((x167)*(x185)))+(((x166)*(x174)))+(((IkReal(-1.00000000000000))*(gconst84)*(x176)*(x183)))+(((x164)*(x167)))+(((IkReal(-1.00000000000000))*(gconst76)*(x191)*(x212)))+(((IkReal(-1.00000000000000))*(gconst64)*(x172)*(x205)))+(((gconst66)*(gconst69)*(x186)))+(((IkReal(-1.00000000000000))*(gconst80)*(x164)*(x184)))+(((IkReal(-1.00000000000000))*(gconst78)*(x191)*(x196)))+(((x165)*(x170)))+(((IkReal(-1.00000000000000))*(gconst75)*(x183)*(x191)))+(((IkReal(-1.00000000000000))*(x167)*(x194)))+(((x162)*(x223)))+(((x177)*(x226)))+(((IkReal(-1.00000000000000))*(gconst70)*(x159)*(x212)))+(((x181)*(x222)))+(((IkReal(-1.00000000000000))*(gconst72)*(x184)*(x220)))+(((IkReal(-1.00000000000000))*(gconst72)*(x184)*(x221)))+(((IkReal(-1.00000000000000))*(gconst69)*(x176)*(x224)))+(((x181)*(x232)))+(((x164)*(x202)))+(((IkReal(-1.00000000000000))*(gconst80)*(x166)*(x208)))+(((gconst78)*(gconst79)*(x166)))+(((IkReal(-1.00000000000000))*(x163)*(x185)))+(((x179)*(x225)))+(((x187)*(x210)))+(((x166)*(x207)))+(((IkReal(-1.00000000000000))*(gconst74)*(x158)*(x228)))+(((IkReal(-1.00000000000000))*(gconst66)*(x161)*(x176)))+(((x181)*(x182)))+(((x163)*(x194)))+(((IkReal(-1.00000000000000))*(gconst80)*(x184)*(x185))));
op[7]=((((IkReal(-1.00000000000000))*(gconst69)*(x176)*(x194)))+(((IkReal(-1.00000000000000))*(gconst64)*(x183)*(x191)))+(((gconst63)*(gconst79)*(x181)))+(((IkReal(-1.00000000000000))*(x163)*(x166)))+(((x157)*(x162)))+(((IkReal(-1.00000000000000))*(gconst71)*(x158)*(x223)))+(((gconst70)*(gconst74)*(x181)))+(((IkReal(-1.00000000000000))*(gconst75)*(x176)*(x183)))+(((IkReal(-1.00000000000000))*(gconst72)*(x184)*(x185)))+(((x170)*(x206)))+(((IkReal(-1.00000000000000))*(gconst74)*(x158)*(x179)))+(((x185)*(x205)))+(((x179)*(x233)))+(((x166)*(x167)))+(((IkReal(-1.00000000000000))*(gconst68)*(x177)*(x191)))+(((IkReal(-1.00000000000000))*(x167)*(x196)))+(((x177)*(x228)))+(((IkReal(-1.00000000000000))*(gconst72)*(x164)*(x184)))+(((IkReal(-1.00000000000000))*(gconst80)*(x166)*(x184)))+(((gconst64)*(gconst80)*(x157)))+(((x164)*(x205)))+(((x166)*(x202)))+(((IkReal(-1.00000000000000))*(gconst76)*(x176)*(x212)))+(((x163)*(x196))));
op[8]=((((x157)*(x170)))+(((IkReal(-1.00000000000000))*(gconst68)*(x176)*(x177)))+(((IkReal(-1.00000000000000))*(gconst72)*(x166)*(x184)))+(((IkReal(-1.00000000000000))*(gconst64)*(x176)*(x183)))+(((x166)*(x205)))+(((x181)*(x187)))+(((IkReal(-1.00000000000000))*(gconst71)*(x157)*(x158)))+(((x177)*(x179))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst109;
gconst109=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x237=((cj3)*(sj1));
IkReal x238=((r02)*(sj2));
IkReal x239=((r01)*(sj0));
IkReal x240=((IkReal(1.00000000000000))*(cj3));
IkReal x241=((cj1)*(sj2));
IkReal x242=((IkReal(1.00000000000000))*(sj3));
IkReal x243=((cj0)*(r00));
IkReal x244=((cj0)*(r01));
IkReal x245=((r00)*(sj0));
IkReal x246=((cj2)*(x243));
IkReal x247=((sj1)*(x242));
IkReal x248=((cj1)*(cj2)*(r02));
if( IKabs(((gconst109)*(((((sj3)*(x248)))+(((IkReal(-1.00000000000000))*(cj2)*(x239)*(x247)))+(((IkReal(-1.00000000000000))*(x239)*(x241)*(x242)))+(((IkReal(-1.00000000000000))*(x238)*(x247)))+(((IkReal(-1.00000000000000))*(x240)*(x244)))+(((IkReal(-1.00000000000000))*(x241)*(x242)*(x243)))+(((IkReal(-1.00000000000000))*(x246)*(x247)))+(((cj3)*(x245))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst109)*(((((x237)*(x238)))+(((cj3)*(x239)*(x241)))+(((IkReal(-1.00000000000000))*(x242)*(x244)))+(((x237)*(x246)))+(((sj3)*(x245)))+(((IkReal(-1.00000000000000))*(x240)*(x248)))+(((cj2)*(x237)*(x239)))+(((cj3)*(x241)*(x243))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst109)*(((((sj3)*(x248)))+(((IkReal(-1.00000000000000))*(cj2)*(x239)*(x247)))+(((IkReal(-1.00000000000000))*(x239)*(x241)*(x242)))+(((IkReal(-1.00000000000000))*(x238)*(x247)))+(((IkReal(-1.00000000000000))*(x240)*(x244)))+(((IkReal(-1.00000000000000))*(x241)*(x242)*(x243)))+(((IkReal(-1.00000000000000))*(x246)*(x247)))+(((cj3)*(x245)))))), ((gconst109)*(((((x237)*(x238)))+(((cj3)*(x239)*(x241)))+(((IkReal(-1.00000000000000))*(x242)*(x244)))+(((x237)*(x246)))+(((sj3)*(x245)))+(((IkReal(-1.00000000000000))*(x240)*(x248)))+(((cj2)*(x237)*(x239)))+(((cj3)*(x241)*(x243)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x249=IKsin(j5);
IkReal x250=IKcos(j5);
IkReal x251=((sj1)*(sj2));
IkReal x252=((cj2)*(sj1));
IkReal x253=((py)*(sj0));
IkReal x254=((IkReal(1.00000000000000))*(cj0));
IkReal x255=((r01)*(sj0));
IkReal x256=((cj1)*(sj2));
IkReal x257=((cj0)*(x256));
IkReal x258=((sj3)*(x249));
IkReal x259=((IkReal(1.00000000000000))*(x250));
IkReal x260=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x261=((IkReal(0.790000000000000))*(x250));
IkReal x262=((cj3)*(x249));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x262)))+(((IkReal(-1.00000000000000))*(sj3)*(x259)))+(((IkReal(-1.00000000000000))*(r01)*(x254))));
evalcond[1]=((((sj3)*(x261)))+(((IkReal(0.790000000000000))*(x262)))+(((IkReal(-1.00000000000000))*(py)*(x254)))+(((px)*(sj0))));
evalcond[2]=((((cj0)*(r00)*(x252)))+(((r00)*(x257)))+(x258)+(((IkReal(-1.00000000000000))*(r02)*(x260)))+(((IkReal(-1.00000000000000))*(cj3)*(x259)))+(((x252)*(x255)))+(((x255)*(x256)))+(((r02)*(x251))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x256)))+(((IkReal(-0.0750000000000000))*(x252)))+(((pz)*(x251)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x257)))+(((x253)*(x256)))+(((cj0)*(px)*(x252)))+(((x252)*(x253)))+(((cj3)*(x261)))+(((IkReal(-0.790000000000000))*(x258)))+(((IkReal(-1.00000000000000))*(pz)*(x260))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x263=((cj1)*(cj2));
IkReal x264=((IkReal(1.00000000000000))*(cj0));
IkReal x265=((sj1)*(sj2));
IkReal x266=((IkReal(1.00000000000000))*(sj0));
IkReal x267=((sj0)*(x265));
IkReal x268=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x269=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x263)*(x264)))+(((cj0)*(r00)*(x265)))+(((IkReal(-1.00000000000000))*(r02)*(x268)))+(((IkReal(-1.00000000000000))*(r02)*(x269)))+(((IkReal(-1.00000000000000))*(r01)*(x263)*(x266)))+(((r01)*(x267))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x263)*(x266)))+(((cj0)*(px)*(x265)))+(((IkReal(0.0750000000000000))*(x263)))+(((IkReal(-0.0750000000000000))*(x265)))+(((py)*(x267)))+(((IkReal(-1.00000000000000))*(px)*(x263)*(x264)))+(((IkReal(-1.00000000000000))*(pz)*(x269)))+(((IkReal(-1.00000000000000))*(pz)*(x268))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst111;
gconst111=IkReal(-2.00000000000000);
IkReal gconst112;
gconst112=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst113;
gconst113=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst114;
gconst114=IkReal(1.58000000000000);
IkReal gconst115;
gconst115=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst116;
gconst116=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst119;
gconst119=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst120;
gconst120=IkReal(1.58000000000000);
IkReal gconst121;
gconst121=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst122;
gconst122=IkReal(2.00000000000000);
IkReal gconst123;
gconst123=IkReal(-2.00000000000000);
IkReal gconst124;
gconst124=IkReal(-1.58000000000000);
IkReal gconst125;
gconst125=IkReal(1.58000000000000);
IkReal gconst126;
gconst126=IkReal(2.00000000000000);
IkReal gconst127;
gconst127=IkReal(-2.00000000000000);
IkReal gconst128;
gconst128=IkReal(-1.58000000000000);
IkReal gconst129;
gconst129=IkReal(1.58000000000000);
IkReal gconst130;
gconst130=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst131;
gconst131=IkReal(2.00000000000000);
IkReal gconst132;
gconst132=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst133;
gconst133=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst134;
gconst134=IkReal(-1.58000000000000);
IkReal gconst135;
gconst135=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst136;
gconst136=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst137;
gconst137=IkReal(2.00000000000000);
IkReal gconst138;
gconst138=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))));
IkReal gconst139;
gconst139=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal gconst140;
gconst140=IkReal(-1.58000000000000);
IkReal gconst141;
gconst141=((((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
IkReal x270=((gconst132)*(gconst134));
IkReal x271=((gconst137)*(gconst139));
IkReal x272=((IkReal(1.00000000000000))*(gconst136));
IkReal x273=((gconst130)*(gconst135));
IkReal x274=((gconst132)*(gconst133));
IkReal x275=((gconst138)*(gconst139));
IkReal x276=((gconst131)*(gconst135));
dummyeval[0]=((((x273)*(x275)))+(((IkReal(-1.00000000000000))*(gconst141)*(x272)*(x273)))+(((IkReal(-1.00000000000000))*(x274)*(x275)))+(((gconst136)*(gconst141)*(x274)))+(((IkReal(-1.00000000000000))*(x271)*(x276)))+(((x270)*(x271)))+(((gconst136)*(gconst140)*(x276)))+(((IkReal(-1.00000000000000))*(gconst140)*(x270)*(x272))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x277=((gconst118)*(gconst119));
IkReal x278=((gconst110)*(gconst135));
IkReal x279=((gconst114)*(gconst117));
IkReal x280=((gconst123)*(gconst128));
IkReal x281=((gconst136)*(gconst141));
IkReal x282=((gconst113)*(gconst132));
IkReal x283=((IkReal(1.00000000000000))*(gconst124));
IkReal x284=((IkReal(1.00000000000000))*(gconst135));
IkReal x285=((gconst138)*(gconst139));
IkReal x286=((gconst131)*(gconst139));
IkReal x287=((gconst112)*(gconst116));
IkReal x288=((gconst124)*(gconst129));
IkReal x289=((gconst115)*(gconst122));
IkReal x290=((IkReal(1.00000000000000))*(gconst140));
IkReal x291=((gconst112)*(gconst139));
IkReal x292=((gconst121)*(gconst124));
IkReal x293=((gconst129)*(gconst136));
IkReal x294=((gconst132)*(gconst136));
IkReal x295=((gconst120)*(gconst126));
IkReal x296=((IkReal(1.00000000000000))*(gconst123));
IkReal x297=((gconst131)*(gconst137));
IkReal x298=((gconst116)*(gconst121));
IkReal x299=((IkReal(1.00000000000000))*(gconst116));
IkReal x300=((gconst131)*(gconst135));
IkReal x301=((gconst126)*(gconst140));
IkReal x302=((gconst111)*(gconst115));
IkReal x303=((IkReal(1.00000000000000))*(gconst125));
IkReal x304=((gconst116)*(gconst120));
IkReal x305=((gconst111)*(gconst135));
IkReal x306=((gconst113)*(gconst121));
IkReal x307=((gconst130)*(gconst138));
IkReal x308=((gconst126)*(gconst141));
IkReal x309=((IkReal(1.00000000000000))*(gconst119));
IkReal x310=((gconst132)*(gconst133));
IkReal x311=((gconst112)*(gconst133));
IkReal x312=((gconst110)*(gconst115));
IkReal x313=((gconst127)*(gconst139));
IkReal x314=((gconst123)*(gconst133));
IkReal x315=((IkReal(1.00000000000000))*(gconst120));
IkReal x316=((gconst132)*(gconst134));
IkReal x317=((gconst122)*(gconst125));
IkReal x318=((gconst125)*(gconst128));
IkReal x319=((gconst134)*(gconst137));
IkReal x320=((gconst116)*(gconst123));
IkReal x321=((gconst120)*(gconst136));
IkReal x322=((IkReal(1.00000000000000))*(gconst118));
IkReal x323=((gconst112)*(gconst113));
IkReal x324=((gconst111)*(gconst117));
IkReal x325=((IkReal(1.00000000000000))*(gconst121));
IkReal x326=((gconst124)*(gconst132));
IkReal x327=((gconst137)*(gconst139));
IkReal x328=((gconst114)*(gconst132));
IkReal x329=((gconst112)*(gconst119));
IkReal x330=((IkReal(1.00000000000000))*(gconst128));
IkReal x331=((gconst123)*(gconst134));
IkReal x332=((gconst111)*(gconst125));
IkReal x333=((gconst130)*(gconst135));
IkReal x334=((IkReal(1.00000000000000))*(gconst115));
IkReal x335=((gconst118)*(gconst139));
IkReal x336=((gconst116)*(gconst140));
IkReal x337=((gconst117)*(gconst139));
IkReal x338=((gconst112)*(gconst128));
IkReal x339=((gconst126)*(gconst129));
IkReal x340=((gconst127)*(gconst128));
IkReal x341=((gconst116)*(gconst141));
IkReal x342=((gconst115)*(gconst131));
IkReal x343=((gconst125)*(gconst131));
IkReal x344=((gconst136)*(gconst140));
IkReal x345=((gconst119)*(gconst127));
IkReal x346=((gconst113)*(gconst129));
IkReal x347=((gconst118)*(gconst128));
IkReal x348=((gconst128)*(gconst135));
IkReal x349=((gconst112)*(gconst136));
IkReal x350=((gconst110)*(gconst125));
IkReal x351=((gconst115)*(gconst130));
IkReal x352=((gconst121)*(gconst133));
IkReal x353=((gconst125)*(gconst130));
IkReal x354=((gconst122)*(gconst135));
IkReal x355=((gconst112)*(x306));
IkReal x356=((gconst126)*(gconst130)*(x284));
op[0]=((((IkReal(-1.00000000000000))*(gconst137)*(x284)*(x286)))+(((IkReal(-1.00000000000000))*(gconst134)*(x290)*(x294)))+(((x300)*(x344)))+(((x285)*(x333)))+(((IkReal(-1.00000000000000))*(x285)*(x310)))+(((x281)*(x310)))+(((IkReal(-1.00000000000000))*(gconst130)*(x281)*(x284)))+(((x316)*(x327))));
op[1]=((((x293)*(x310)))+(((IkReal(-1.00000000000000))*(gconst122)*(x281)*(x284)))+(((IkReal(-1.00000000000000))*(gconst130)*(x281)*(x303)))+(((IkReal(-1.00000000000000))*(gconst133)*(x285)*(x296)))+(((x281)*(x326)))+(((IkReal(-1.00000000000000))*(gconst128)*(x284)*(x297)))+(((x308)*(x310)))+(((IkReal(-1.00000000000000))*(gconst132)*(x283)*(x285)))+(((IkReal(-1.00000000000000))*(x310)*(x313)))+(((x307)*(x348)))+(((IkReal(-1.00000000000000))*(gconst130)*(x284)*(x308)))+(((x285)*(x353)))+(((x285)*(x354)))+(((IkReal(-1.00000000000000))*(gconst138)*(x310)*(x330)))+(((x281)*(x314)))+(((IkReal(-1.00000000000000))*(gconst136)*(x290)*(x331)))+(((IkReal(-1.00000000000000))*(gconst137)*(x286)*(x303)))+(((gconst128)*(gconst137)*(x316)))+(((IkReal(-1.00000000000000))*(gconst126)*(x290)*(x316)))+(((x300)*(x301)))+(((x343)*(x344)))+(((IkReal(-1.00000000000000))*(gconst130)*(x284)*(x293)))+(((x313)*(x333)))+(((gconst123)*(gconst139)*(x319))));
op[2]=((((IkReal(-1.00000000000000))*(gconst111)*(x284)*(x327)))+(((x293)*(x314)))+(((IkReal(-1.00000000000000))*(x282)*(x285)))+(((IkReal(-1.00000000000000))*(gconst133)*(x296)*(x313)))+(((IkReal(-1.00000000000000))*(gconst134)*(x294)*(x315)))+(((x301)*(x343)))+(((IkReal(-1.00000000000000))*(gconst130)*(x284)*(x341)))+(((x278)*(x285)))+(((IkReal(-1.00000000000000))*(gconst128)*(x297)*(x303)))+(((x327)*(x328)))+(((x308)*(x326)))+(((gconst123)*(gconst124)*(x281)))+(((x285)*(x317)))+(((x313)*(x353)))+(((x313)*(x354)))+(((IkReal(-1.00000000000000))*(gconst130)*(x303)*(x308)))+(((gconst122)*(gconst138)*(x348)))+(((IkReal(-1.00000000000000))*(gconst119)*(x284)*(x297)))+(((x308)*(x314)))+(((IkReal(-1.00000000000000))*(gconst122)*(x284)*(x308)))+(((x305)*(x344)))+(((IkReal(-1.00000000000000))*(gconst139)*(x310)*(x322)))+(((IkReal(-1.00000000000000))*(gconst130)*(x281)*(x334)))+(((IkReal(-1.00000000000000))*(gconst127)*(x310)*(x330)))+(((x316)*(x337)))+(((gconst119)*(gconst135)*(x307)))+(((IkReal(-1.00000000000000))*(gconst126)*(x290)*(x331)))+(((IkReal(-1.00000000000000))*(gconst137)*(x286)*(x334)))+(((IkReal(-1.00000000000000))*(gconst122)*(x281)*(x303)))+(((x333)*(x340)))+(((IkReal(-1.00000000000000))*(gconst128)*(gconst132)*(gconst138)*(x283)))+(((IkReal(-1.00000000000000))*(x285)*(x311)))+(((x285)*(x351)))+(((x307)*(x318)))+(((x280)*(x319)))+(((IkReal(-1.00000000000000))*(x278)*(x281)))+(((IkReal(-1.00000000000000))*(gconst132)*(x283)*(x313)))+(((IkReal(-1.00000000000000))*(gconst138)*(x309)*(x310)))+(((IkReal(-1.00000000000000))*(gconst122)*(x284)*(x293)))+(((IkReal(-1.00000000000000))*(gconst114)*(x290)*(x294)))+(((x333)*(x335)))+(((IkReal(-1.00000000000000))*(gconst130)*(x284)*(x339)))+(((x281)*(x311)))+(((IkReal(-1.00000000000000))*(gconst134)*(x290)*(x349)))+(((x310)*(x341)))+(((x291)*(x319)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst130)*(gconst136)*(x284)))+(((x294)*(x352)))+(((gconst119)*(gconst137)*(x316)))+(((IkReal(-1.00000000000000))*(gconst116)*(x290)*(x316)))+(((IkReal(-1.00000000000000))*(gconst133)*(gconst138)*(x280)))+(((IkReal(-1.00000000000000))*(gconst123)*(x283)*(x285)))+(((x281)*(x282)))+(((x300)*(x321)))+(((x300)*(x336)))+(((x310)*(x339)))+(((x288)*(x294)))+(((IkReal(-1.00000000000000))*(gconst117)*(x284)*(x286)))+(((x342)*(x344)))+(((IkReal(-1.00000000000000))*(gconst130)*(x293)*(x303))));
op[3]=((((gconst121)*(gconst124)*(gconst132)*(gconst136)))+(((gconst115)*(gconst122)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst126)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst128)*(gconst131)*(gconst137)))+(((gconst122)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst129)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst126)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst134)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst132)*(gconst139)))+(((gconst120)*(gconst125)*(gconst131)*(gconst136)))+(((gconst113)*(gconst123)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst126)*(gconst130)*(gconst135)))+(((gconst117)*(gconst128)*(gconst132)*(gconst134)))+(((gconst112)*(gconst128)*(gconst134)*(gconst137)))+(((gconst119)*(gconst122)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst127)*(gconst128)*(gconst133)))+(((gconst119)*(gconst125)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst128)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst128)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst127)*(gconst132)*(gconst133)))+(((gconst114)*(gconst128)*(gconst132)*(gconst137)))+(((gconst122)*(gconst125)*(gconst128)*(gconst138)))+(((gconst116)*(gconst123)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst123)*(gconst134)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst138)*(gconst139)))+(((gconst115)*(gconst128)*(gconst130)*(gconst138)))+(((gconst112)*(gconst126)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst129)*(gconst135)*(gconst136)))+(((gconst123)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst129)*(gconst136)))+(((gconst116)*(gconst125)*(gconst131)*(gconst140)))+(((gconst119)*(gconst127)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst128)*(gconst132)*(gconst133)))+(((gconst124)*(gconst126)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst126)*(gconst129)*(gconst135)))+(((gconst115)*(gconst126)*(gconst131)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst132)*(gconst138)))+(((gconst118)*(gconst128)*(gconst130)*(gconst135)))+(((gconst121)*(gconst126)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst125)*(gconst131)*(gconst137)))+(((gconst121)*(gconst123)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst128)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst141)))+(((gconst119)*(gconst123)*(gconst134)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst130)*(gconst141)))+(((gconst120)*(gconst126)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst130)*(gconst141)))+(((gconst112)*(gconst124)*(gconst136)*(gconst141)))+(((gconst116)*(gconst129)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst124)*(gconst127)*(gconst128)*(gconst132)))+(((gconst110)*(gconst127)*(gconst135)*(gconst139)))+(((gconst110)*(gconst125)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst131)*(gconst139)))+(((gconst113)*(gconst126)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst133)*(gconst139)))+(((gconst115)*(gconst127)*(gconst130)*(gconst139)))+(((gconst117)*(gconst123)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst126)*(gconst134)*(gconst140)))+(((gconst122)*(gconst127)*(gconst128)*(gconst135)))+(((gconst112)*(gconst129)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst136)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst132)*(gconst139)))+(((gconst111)*(gconst125)*(gconst136)*(gconst140)))+(((gconst118)*(gconst125)*(gconst130)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst135)*(gconst136)))+(((gconst113)*(gconst129)*(gconst132)*(gconst136)))+(((gconst118)*(gconst122)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst136)*(gconst141)))+(((gconst123)*(gconst124)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst125)*(gconst126)*(gconst129)*(gconst130)))+(((gconst125)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst129)*(gconst130)*(gconst136)))+(((gconst116)*(gconst124)*(gconst132)*(gconst141)))+(((gconst111)*(gconst126)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst128)*(gconst131)*(gconst135)))+(((gconst110)*(gconst128)*(gconst135)*(gconst138)))+(((gconst123)*(gconst124)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst130)*(gconst136))));
op[4]=((((gconst117)*(gconst123)*(gconst128)*(gconst134)))+(((gconst112)*(gconst116)*(gconst133)*(gconst141)))+(((gconst119)*(gconst122)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst139)))+(((gconst111)*(gconst120)*(gconst135)*(gconst136)))+(((gconst116)*(gconst123)*(gconst124)*(gconst141)))+(((gconst116)*(gconst124)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst128)*(gconst138)))+(((gconst113)*(gconst121)*(gconst132)*(gconst136)))+(((gconst115)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst130)*(gconst135)))+(((gconst111)*(gconst125)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst126)*(gconst140)))+(((gconst123)*(gconst124)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst141)))+(((gconst110)*(gconst125)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst126)*(gconst134)))+(((gconst115)*(gconst116)*(gconst131)*(gconst140)))+(((gconst111)*(gconst116)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst141)))+(((gconst112)*(gconst117)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst129)*(gconst136)))+(((gconst113)*(gconst126)*(gconst129)*(gconst132)))+(((gconst120)*(gconst125)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst135)*(gconst141)))+(((gconst118)*(gconst122)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst128)*(gconst131)))+(((gconst110)*(gconst119)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst132)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst129)*(gconst135)))+(((gconst111)*(gconst115)*(gconst136)*(gconst140)))+(((gconst119)*(gconst125)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst134)*(gconst136)))+(((gconst112)*(gconst124)*(gconst129)*(gconst136)))+(((gconst112)*(gconst114)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst127)*(gconst132)))+(((gconst115)*(gconst122)*(gconst127)*(gconst139)))+(((gconst118)*(gconst119)*(gconst130)*(gconst135)))+(((gconst110)*(gconst118)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst126)*(gconst130)))+(((gconst121)*(gconst123)*(gconst126)*(gconst133)))+(((gconst116)*(gconst123)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst133)*(gconst138)))+(((gconst113)*(gconst123)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst132)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst129)*(gconst135)))+(((gconst110)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst135)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst116)*(gconst134)*(gconst140)))+(((gconst112)*(gconst119)*(gconst134)*(gconst137)))+(((gconst113)*(gconst123)*(gconst126)*(gconst141)))+(((gconst112)*(gconst121)*(gconst133)*(gconst136)))+(((gconst114)*(gconst119)*(gconst132)*(gconst137)))+(((gconst115)*(gconst118)*(gconst130)*(gconst139)))+(((gconst117)*(gconst119)*(gconst132)*(gconst134)))+(((gconst110)*(gconst115)*(gconst138)*(gconst139)))+(((gconst116)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst129)))+(((gconst121)*(gconst123)*(gconst124)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst131)*(gconst139)))+(((gconst112)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst130)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst128)*(gconst138)))+(((gconst112)*(gconst113)*(gconst136)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst138)))+(((gconst110)*(gconst127)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst135)*(gconst139)))+(((gconst121)*(gconst124)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst128)))+(((gconst118)*(gconst125)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst119)*(gconst131)*(gconst137)))+(((gconst115)*(gconst122)*(gconst128)*(gconst138)))+(((gconst114)*(gconst117)*(gconst132)*(gconst139)))+(((gconst116)*(gconst121)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst127)*(gconst133)))+(((gconst113)*(gconst116)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst137)*(gconst139)))+(((gconst112)*(gconst124)*(gconst126)*(gconst141)))+(((gconst115)*(gconst120)*(gconst131)*(gconst136)))+(((gconst115)*(gconst119)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst136)*(gconst140)))+(((gconst122)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst126)*(gconst135))));
op[5]=((((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst125)*(gconst137)))+(((gconst112)*(gconst121)*(gconst124)*(gconst136)))+(((gconst117)*(gconst119)*(gconst123)*(gconst134)))+(((gconst112)*(gconst124)*(gconst126)*(gconst129)))+(((gconst115)*(gconst119)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst122)*(gconst136)))+(((gconst110)*(gconst118)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst141)))+(((gconst110)*(gconst118)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((gconst112)*(gconst113)*(gconst129)*(gconst136)))+(((gconst114)*(gconst117)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst132)))+(((gconst112)*(gconst117)*(gconst128)*(gconst134)))+(((gconst115)*(gconst118)*(gconst122)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst126)))+(((gconst114)*(gconst119)*(gconst123)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst123)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst126)*(gconst141)))+(((gconst118)*(gconst119)*(gconst122)*(gconst135)))+(((gconst110)*(gconst115)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst122)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst129)))+(((gconst115)*(gconst118)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst125)*(gconst128)))+(((gconst112)*(gconst116)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst122)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst125)*(gconst131)))+(((gconst113)*(gconst123)*(gconst126)*(gconst129)))+(((gconst110)*(gconst119)*(gconst125)*(gconst138)))+(((gconst116)*(gconst121)*(gconst124)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst123)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst123)*(gconst134)))+(((gconst116)*(gconst121)*(gconst123)*(gconst133)))+(((gconst110)*(gconst115)*(gconst128)*(gconst138)))+(((gconst113)*(gconst116)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst128)*(gconst135)))+(((gconst115)*(gconst122)*(gconst127)*(gconst128)))+(((gconst110)*(gconst119)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst128)*(gconst138)))+(((gconst112)*(gconst114)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst128)*(gconst133)))+(((gconst110)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst129)))+(((gconst113)*(gconst121)*(gconst123)*(gconst136)))+(((gconst113)*(gconst116)*(gconst123)*(gconst141)))+(((gconst111)*(gconst115)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst129)*(gconst130)))+(((gconst115)*(gconst119)*(gconst122)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst124)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst128)*(gconst137)))+(((gconst112)*(gconst113)*(gconst126)*(gconst141)))+(((gconst114)*(gconst117)*(gconst123)*(gconst139)))+(((gconst118)*(gconst119)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst127)))+(((gconst121)*(gconst123)*(gconst124)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst124)*(gconst132)))+(((gconst113)*(gconst121)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((gconst112)*(gconst116)*(gconst124)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst127)))+(((gconst116)*(gconst123)*(gconst124)*(gconst129)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((gconst112)*(gconst121)*(gconst126)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst128)))+(((gconst111)*(gconst116)*(gconst125)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128))));
op[6]=((((gconst114)*(gconst137)*(x329)))+(((gconst112)*(gconst126)*(x292)))+(((IkReal(-1.00000000000000))*(gconst114)*(x295)*(x296)))+(((IkReal(-1.00000000000000))*(gconst120)*(x299)*(x328)))+(((x279)*(x280)))+(((x345)*(x350)))+(((IkReal(-1.00000000000000))*(gconst114)*(x287)*(x290)))+(((IkReal(-1.00000000000000))*(gconst113)*(x280)*(x322)))+(((IkReal(-1.00000000000000))*(gconst123)*(x277)*(x283)))+(((IkReal(-1.00000000000000))*(gconst137)*(x302)*(x309)))+(((x292)*(x320)))+(((IkReal(-1.00000000000000))*(gconst138)*(x309)*(x323)))+(((IkReal(-1.00000000000000))*(gconst128)*(x303)*(x324)))+(((IkReal(-1.00000000000000))*(gconst141)*(x299)*(x312)))+(((IkReal(-1.00000000000000))*(gconst136)*(x312)*(x325)))+(((gconst123)*(gconst126)*(x306)))+(((x302)*(x321)))+(((x282)*(x298)))+(((IkReal(-1.00000000000000))*(x277)*(x311)))+(((IkReal(-1.00000000000000))*(gconst129)*(x289)*(x299)))+(((IkReal(-1.00000000000000))*(gconst117)*(x309)*(x342)))+(((IkReal(-1.00000000000000))*(gconst134)*(x287)*(x315)))+(((x295)*(x332)))+(((x304)*(x342)))+(((IkReal(-1.00000000000000))*(gconst127)*(x283)*(x329)))+(((gconst117)*(gconst134)*(x329)))+(((x287)*(x352)))+(((IkReal(-1.00000000000000))*(gconst122)*(x298)*(x303)))+(((IkReal(-1.00000000000000))*(gconst113)*(x291)*(x322)))+(((IkReal(-1.00000000000000))*(x277)*(x282)))+(((x289)*(x345)))+(((x289)*(x347)))+(((IkReal(-1.00000000000000))*(gconst130)*(x298)*(x334)))+(((x312)*(x340)))+(((x306)*(x349)))+(((x302)*(x336)))+(((x279)*(x291)))+(((gconst119)*(gconst138)*(x312)))+(((gconst119)*(gconst132)*(x279)))+(((x277)*(x351)))+(((IkReal(-1.00000000000000))*(gconst118)*(x283)*(x338)))+(((IkReal(-1.00000000000000))*(gconst114)*(x315)*(x349)))+(((IkReal(-1.00000000000000))*(x312)*(x339)))+(((x277)*(x278)))+(((gconst113)*(gconst141)*(x287)))+(((IkReal(-1.00000000000000))*(gconst119)*(x284)*(x324)))+(((IkReal(-1.00000000000000))*(gconst126)*(x289)*(x325)))+(((IkReal(-1.00000000000000))*(gconst129)*(x299)*(x350)))+(((x323)*(x339)))+(((x304)*(x305)))+(((IkReal(-1.00000000000000))*(gconst113)*(x296)*(x345)))+(((gconst110)*(gconst118)*(x318)))+(((x312)*(x335)))+(((x287)*(x288)))+(((x277)*(x317)))+(((IkReal(-1.00000000000000))*(gconst127)*(x323)*(x330)))+(((IkReal(-1.00000000000000))*(x302)*(x337)))+(((IkReal(-1.00000000000000))*(x278)*(x298)))+(((x320)*(x346)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(x303))));
op[7]=((((gconst113)*(gconst123)*(x298)))+(((IkReal(-1.00000000000000))*(gconst129)*(x299)*(x312)))+(((IkReal(-1.00000000000000))*(x289)*(x298)))+(((IkReal(-1.00000000000000))*(gconst127)*(x309)*(x323)))+(((IkReal(-1.00000000000000))*(gconst117)*(x302)*(x330)))+(((x287)*(x292)))+(((IkReal(-1.00000000000000))*(gconst113)*(x277)*(x296)))+(((x279)*(x338)))+(((x312)*(x347)))+(((x312)*(x345)))+(((x304)*(x332)))+(((x295)*(x302)))+(((x277)*(x350)))+(((IkReal(-1.00000000000000))*(gconst110)*(x298)*(x303)))+(((gconst126)*(x355)))+(((IkReal(-1.00000000000000))*(gconst126)*(x312)*(x325)))+(((x287)*(x346)))+(((x277)*(x289)))+(((gconst119)*(gconst123)*(x279)))+(((IkReal(-1.00000000000000))*(gconst128)*(x322)*(x323)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(x295)))+(((IkReal(-1.00000000000000))*(gconst119)*(x303)*(x324)))+(((IkReal(-1.00000000000000))*(gconst114)*(x296)*(x304)))+(((IkReal(-1.00000000000000))*(gconst112)*(x277)*(x283))));
op[8]=((((IkReal(-1.00000000000000))*(x277)*(x323)))+(((x287)*(x306)))+(((IkReal(-1.00000000000000))*(x298)*(x312)))+(((IkReal(-1.00000000000000))*(gconst117)*(x302)*(x309)))+(((x302)*(x304)))+(((IkReal(-1.00000000000000))*(gconst114)*(x287)*(x315)))+(((x277)*(x312)))+(((x279)*(x329))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst158;
gconst158=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x357=((cj1)*(sj2));
IkReal x358=((cj2)*(sj1));
IkReal x359=((IkReal(1.00000000000000))*(sj3));
IkReal x360=((cj0)*(r00));
IkReal x361=((r01)*(sj0));
IkReal x362=((cj0)*(r01));
IkReal x363=((IkReal(1.00000000000000))*(cj3));
IkReal x364=((r00)*(sj0));
IkReal x365=((cj1)*(cj2)*(r02));
IkReal x366=((r02)*(sj1)*(sj2));
IkReal x367=((x361)*(x363));
if( IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(x362)*(x363)))+(((IkReal(-1.00000000000000))*(x358)*(x359)*(x360)))+(((IkReal(-1.00000000000000))*(x358)*(x359)*(x361)))+(((IkReal(-1.00000000000000))*(x357)*(x359)*(x361)))+(((IkReal(-1.00000000000000))*(x357)*(x359)*(x360)))+(((IkReal(-1.00000000000000))*(x359)*(x366)))+(((sj3)*(x365)))+(((cj3)*(x364))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(x363)*(x366)))+(((IkReal(-1.00000000000000))*(x357)*(x360)*(x363)))+(((IkReal(-1.00000000000000))*(x358)*(x367)))+(((IkReal(-1.00000000000000))*(x358)*(x360)*(x363)))+(((IkReal(-1.00000000000000))*(x359)*(x364)))+(((sj3)*(x362)))+(((IkReal(-1.00000000000000))*(x357)*(x367)))+(((cj3)*(x365))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst158)*(((((IkReal(-1.00000000000000))*(x362)*(x363)))+(((IkReal(-1.00000000000000))*(x358)*(x359)*(x360)))+(((IkReal(-1.00000000000000))*(x358)*(x359)*(x361)))+(((IkReal(-1.00000000000000))*(x357)*(x359)*(x361)))+(((IkReal(-1.00000000000000))*(x357)*(x359)*(x360)))+(((IkReal(-1.00000000000000))*(x359)*(x366)))+(((sj3)*(x365)))+(((cj3)*(x364)))))), ((gconst158)*(((((IkReal(-1.00000000000000))*(x363)*(x366)))+(((IkReal(-1.00000000000000))*(x357)*(x360)*(x363)))+(((IkReal(-1.00000000000000))*(x358)*(x367)))+(((IkReal(-1.00000000000000))*(x358)*(x360)*(x363)))+(((IkReal(-1.00000000000000))*(x359)*(x364)))+(((sj3)*(x362)))+(((IkReal(-1.00000000000000))*(x357)*(x367)))+(((cj3)*(x365)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x368=IKsin(j5);
IkReal x369=IKcos(j5);
IkReal x370=((sj1)*(sj2));
IkReal x371=((cj2)*(sj1));
IkReal x372=((py)*(sj0));
IkReal x373=((IkReal(1.00000000000000))*(cj0));
IkReal x374=((r01)*(sj0));
IkReal x375=((cj1)*(sj2));
IkReal x376=((cj0)*(x375));
IkReal x377=((sj3)*(x368));
IkReal x378=((cj3)*(x369));
IkReal x379=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x380=((sj3)*(x369));
IkReal x381=((cj3)*(x368));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x373)))+(x380)+(((IkReal(-1.00000000000000))*(x381))));
evalcond[1]=((((IkReal(0.790000000000000))*(x381)))+(((px)*(sj0)))+(((IkReal(-0.790000000000000))*(x380)))+(((IkReal(-1.00000000000000))*(py)*(x373))));
evalcond[2]=((((r02)*(x370)))+(((r00)*(x376)))+(((IkReal(-1.00000000000000))*(r02)*(x379)))+(x378)+(x377)+(((cj0)*(r00)*(x371)))+(((x374)*(x375)))+(((x371)*(x374))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.790000000000000))*(x378)))+(((IkReal(-0.790000000000000))*(x377)))+(((cj0)*(px)*(x371)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x372)*(x375)))+(((px)*(x376)))+(((pz)*(x370)))+(((IkReal(-1.00000000000000))*(pz)*(x379)))+(((IkReal(-0.0750000000000000))*(x375)))+(((IkReal(-0.0750000000000000))*(x371)))+(((x371)*(x372))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x382=((cj0)*(px));
IkReal x383=((IkReal(1.00000000000000))*(cj1));
IkReal x384=((sj1)*(sj2));
IkReal x385=((py)*(sj0));
cj5array[0]=((IkReal(1.26582278481013))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((IkReal(0.405000000000000))+(((IkReal(0.0750000000000000))*(cj1)*(cj2)))+(((IkReal(-0.365000000000000))*(sj2)))+(((x384)*(x385)))+(((IkReal(-1.00000000000000))*(cj2)*(x383)*(x385)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x383)))+(((IkReal(-0.0750000000000000))*(x384)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(sj1)))+(((x382)*(x384)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(x382)*(x383))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x386=((r01)*(sj0));
IkReal x387=((sj1)*(sj2));
IkReal x388=((IkReal(1.00000000000000))*(cj1));
IkReal x389=((cj0)*(r00));
evalcond[0]=((((x387)*(x389)))+(((IkReal(-1.00000000000000))*(cj2)*(x386)*(x388)))+(((x386)*(x387)))+(((sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((IkReal(-1.00000000000000))*(cj2)*(x388)*(x389)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x388))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x390=((cj1)*(cj2));
IkReal x391=((IkReal(1.00000000000000))*(cj0));
IkReal x392=((sj1)*(sj2));
IkReal x393=((IkReal(1.00000000000000))*(sj0));
IkReal x394=((sj0)*(x392));
IkReal x395=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x396=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(r02)*(x395)))+(((IkReal(-1.00000000000000))*(r02)*(x396)))+(((r01)*(x394)))+(((cj0)*(r00)*(x392)))+(((IkReal(-1.00000000000000))*(r01)*(x390)*(x393))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(0.0750000000000000))*(x390)))+(((IkReal(-0.365000000000000))*(sj2)))+(((cj0)*(px)*(x392)))+(((py)*(x394)))+(((IkReal(-1.00000000000000))*(px)*(x390)*(x391)))+(((IkReal(-1.00000000000000))*(pz)*(x396)))+(((IkReal(-1.00000000000000))*(pz)*(x395)))+(((IkReal(-1.00000000000000))*(py)*(x390)*(x393)))+(((IkReal(-0.0750000000000000))*(x392))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x397=((cj1)*(cj5));
IkReal x398=((IkReal(1.00000000000000))*(cj2));
IkReal x399=((r00)*(sj0));
IkReal x400=((IkReal(1.00000000000000))*(sj5));
IkReal x401=((cj0)*(r01));
IkReal x402=((r01)*(sj0));
IkReal x403=((sj1)*(sj5));
IkReal x404=((cj0)*(r00));
IkReal x405=((sj2)*(x404));
IkReal x406=((cj2)*(cj5)*(sj1));
IkReal x407=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((cj5)*(x399)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x400)*(x402)))+(((IkReal(-1.00000000000000))*(x400)*(x407)))+(((IkReal(-1.00000000000000))*(cj5)*(x401)))+(((IkReal(-1.00000000000000))*(cj1)*(x400)*(x405)))+(((cj1)*(cj2)*(r02)*(sj5)))+(((IkReal(-1.00000000000000))*(x398)*(x402)*(x403)))+(((IkReal(-1.00000000000000))*(x398)*(x403)*(x404))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((x397)*(x405)))+(((x404)*(x406)))+(((sj2)*(x397)*(x402)))+(((cj5)*(x407)))+(((IkReal(-1.00000000000000))*(x400)*(x401)))+(((sj5)*(x399)))+(((x402)*(x406)))+(((IkReal(-1.00000000000000))*(r02)*(x397)*(x398))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((cj5)*(x399)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x400)*(x402)))+(((IkReal(-1.00000000000000))*(x400)*(x407)))+(((IkReal(-1.00000000000000))*(cj5)*(x401)))+(((IkReal(-1.00000000000000))*(cj1)*(x400)*(x405)))+(((cj1)*(cj2)*(r02)*(sj5)))+(((IkReal(-1.00000000000000))*(x398)*(x402)*(x403)))+(((IkReal(-1.00000000000000))*(x398)*(x403)*(x404)))))), ((gconst55)*(((((x397)*(x405)))+(((x404)*(x406)))+(((sj2)*(x397)*(x402)))+(((cj5)*(x407)))+(((IkReal(-1.00000000000000))*(x400)*(x401)))+(((sj5)*(x399)))+(((x402)*(x406)))+(((IkReal(-1.00000000000000))*(r02)*(x397)*(x398)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x408=IKsin(j3);
IkReal x409=IKcos(j3);
IkReal x410=((sj1)*(sj2));
IkReal x411=((cj2)*(sj1));
IkReal x412=((cj0)*(px));
IkReal x413=((py)*(sj0));
IkReal x414=((IkReal(0.790000000000000))*(sj5));
IkReal x415=((cj1)*(sj2));
IkReal x416=((cj0)*(r00));
IkReal x417=((IkReal(1.00000000000000))*(cj0));
IkReal x418=((r01)*(sj0));
IkReal x419=((cj5)*(x408));
IkReal x420=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x421=((cj5)*(x409));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x417)))+(((IkReal(-1.00000000000000))*(sj5)*(x409)))+(((IkReal(-1.00000000000000))*(x419))));
evalcond[1]=((((x409)*(x414)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x417)))+(((IkReal(0.790000000000000))*(x419))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x421)))+(((x411)*(x418)))+(((x411)*(x416)))+(((x415)*(x416)))+(((x415)*(x418)))+(((sj5)*(x408)))+(((IkReal(-1.00000000000000))*(r02)*(x420)))+(((r02)*(x410))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x411)))+(((IkReal(-0.0750000000000000))*(x415)))+(((pz)*(x410)))+(((x411)*(x412)))+(((x411)*(x413)))+(((x413)*(x415)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x412)*(x415)))+(((IkReal(0.790000000000000))*(x421)))+(((IkReal(-1.00000000000000))*(pz)*(x420)))+(((IkReal(-1.00000000000000))*(x408)*(x414))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x422=((cj1)*(cj2));
IkReal x423=((IkReal(1.00000000000000))*(cj0));
IkReal x424=((sj1)*(sj2));
IkReal x425=((IkReal(1.00000000000000))*(sj0));
IkReal x426=((sj0)*(x424));
IkReal x427=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x428=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x424)))+(((IkReal(-1.00000000000000))*(r02)*(x427)))+(((IkReal(-1.00000000000000))*(r02)*(x428)))+(((r01)*(x426)))+(((IkReal(-1.00000000000000))*(r00)*(x422)*(x423)))+(((IkReal(-1.00000000000000))*(r01)*(x422)*(x425))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x424)))+(((py)*(x426)))+(((IkReal(0.0750000000000000))*(x422)))+(((IkReal(-1.00000000000000))*(py)*(x422)*(x425)))+(((cj0)*(px)*(x424)))+(((IkReal(-1.00000000000000))*(pz)*(x427)))+(((IkReal(-1.00000000000000))*(pz)*(x428)))+(((IkReal(-1.00000000000000))*(px)*(x422)*(x423))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x429=((cj1)*(cj5));
IkReal x430=((cj0)*(r00));
IkReal x431=((cj5)*(sj0));
IkReal x432=((cj2)*(sj1));
IkReal x433=((r01)*(sj5));
IkReal x434=((cj1)*(sj5));
IkReal x435=((sj0)*(sj2));
IkReal x436=((IkReal(1.00000000000000))*(cj2)*(r02));
IkReal x437=((r02)*(sj1)*(sj2));
if( IKabs(((gconst56)*(((((sj0)*(x432)*(x433)))+(((IkReal(-1.00000000000000))*(x434)*(x436)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj5)*(x430)*(x432)))+(((r00)*(x431)))+(((cj1)*(x433)*(x435)))+(((sj5)*(x437)))+(((sj2)*(x430)*(x434))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((cj5)*(x430)*(x432)))+(((cj5)*(x437)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x429)*(x436)))+(((cj0)*(x433)))+(((r01)*(x429)*(x435)))+(((sj2)*(x429)*(x430)))+(((r01)*(x431)*(x432))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((sj0)*(x432)*(x433)))+(((IkReal(-1.00000000000000))*(x434)*(x436)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj5)*(x430)*(x432)))+(((r00)*(x431)))+(((cj1)*(x433)*(x435)))+(((sj5)*(x437)))+(((sj2)*(x430)*(x434)))))), ((gconst56)*(((((cj5)*(x430)*(x432)))+(((cj5)*(x437)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x429)*(x436)))+(((cj0)*(x433)))+(((r01)*(x429)*(x435)))+(((sj2)*(x429)*(x430)))+(((r01)*(x431)*(x432)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x438=IKcos(j3);
IkReal x439=IKsin(j3);
IkReal x440=((sj1)*(sj2));
IkReal x441=((cj2)*(sj1));
IkReal x442=((py)*(sj0));
IkReal x443=((IkReal(1.00000000000000))*(cj0));
IkReal x444=((r01)*(sj0));
IkReal x445=((cj1)*(sj2));
IkReal x446=((IkReal(0.790000000000000))*(x439));
IkReal x447=((cj0)*(x445));
IkReal x448=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x449=((IkReal(0.790000000000000))*(x438));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj5)*(x438)))+(((r00)*(sj0)))+(((cj5)*(x439)))+(((IkReal(-1.00000000000000))*(r01)*(x443))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x446)))+(((IkReal(-1.00000000000000))*(py)*(x443)))+(((sj5)*(x449)))+(((px)*(sj0))));
evalcond[2]=((((cj0)*(r00)*(x441)))+(((r02)*(x440)))+(((cj5)*(x438)))+(((x441)*(x444)))+(((IkReal(-1.00000000000000))*(r02)*(x448)))+(((x444)*(x445)))+(((r00)*(x447)))+(((sj5)*(x439))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj5)*(x446)))+(((px)*(x447)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x445)))+(((IkReal(-0.0750000000000000))*(x441)))+(((pz)*(x440)))+(((IkReal(-1.00000000000000))*(cj5)*(x449)))+(((x441)*(x442)))+(((cj0)*(px)*(x441)))+(((x442)*(x445)))+(((IkReal(-1.00000000000000))*(pz)*(x448))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x450=((cj1)*(cj2));
IkReal x451=((IkReal(1.00000000000000))*(cj0));
IkReal x452=((sj1)*(sj2));
IkReal x453=((IkReal(1.00000000000000))*(sj0));
IkReal x454=((sj0)*(x452));
IkReal x455=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x456=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r00)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(r01)*(x450)*(x453)))+(((cj0)*(r00)*(x452)))+(((IkReal(-1.00000000000000))*(r02)*(x455)))+(((IkReal(-1.00000000000000))*(r02)*(x456)))+(((r01)*(x454))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x450)*(x453)))+(((IkReal(0.0750000000000000))*(x450)))+(((cj0)*(px)*(x452)))+(((IkReal(-1.00000000000000))*(pz)*(x455)))+(((IkReal(-1.00000000000000))*(pz)*(x456)))+(((IkReal(-0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(px)*(x450)*(x451)))+(((IkReal(-0.0750000000000000))*(x452)))+(((IkReal(0.0800000000000000))*(cj4)))+(((py)*(x454))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x457=((cj1)*(sj2));
IkReal x458=((cj0)*(r00));
IkReal x459=((IkReal(1.00000000000000))*(pz));
IkReal x460=((sj1)*(sj2));
IkReal x461=((r01)*(sj0));
IkReal x462=((IkReal(1.00000000000000))*(r02));
IkReal x463=((cj1)*(cj2));
IkReal x464=((cj2)*(sj1));
IkReal x465=((IkReal(1.00000000000000))*(cj0));
IkReal x466=((IkReal(1.00000000000000))*(sj0)*(x463));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x465))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x461)*(x463)))+(((IkReal(-1.00000000000000))*(x462)*(x464)))+(((x460)*(x461)))+(((IkReal(-1.00000000000000))*(x458)*(x463)))+(((x458)*(x460)))+(((IkReal(-1.00000000000000))*(x457)*(x462))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x462)*(x463)))+(((x461)*(x464)))+(((r02)*(x460)))+(((x458)*(x464)))+(((x457)*(x461)))+(((x457)*(x458))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((py)*(sj0)*(x460)))+(((IkReal(-1.00000000000000))*(px)*(x463)*(x465)))+(((IkReal(-1.00000000000000))*(x459)*(x464)))+(((IkReal(0.0750000000000000))*(x463)))+(((IkReal(-1.00000000000000))*(py)*(x466)))+(((cj0)*(px)*(x460)))+(((IkReal(-0.0750000000000000))*(x460)))+(((IkReal(-1.00000000000000))*(x457)*(x459))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x467=((IkReal(12.5000000000000))*(cj1));
IkReal x468=((py)*(sj0));
IkReal x469=((cj0)*(px));
IkReal x470=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x467)*(x468)))+(((IkReal(-1.00000000000000))*(sj2)*(x467)*(x469)))+(((IkReal(-1.00000000000000))*(x468)*(x470)))+(((cj2)*(pz)*(x467)))+(((IkReal(-1.00000000000000))*(x469)*(x470)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x467)*(x468)))+(((IkReal(-1.00000000000000))*(sj2)*(x467)*(x469)))+(((IkReal(-1.00000000000000))*(x468)*(x470)))+(((cj2)*(pz)*(x467)))+(((IkReal(-1.00000000000000))*(x469)*(x470)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x467)*(x468)))+(((IkReal(-1.00000000000000))*(sj2)*(x467)*(x469)))+(((IkReal(-1.00000000000000))*(x468)*(x470)))+(((cj2)*(pz)*(x467)))+(((IkReal(-1.00000000000000))*(x469)*(x470)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x471=((cj1)*(sj2));
IkReal x472=((cj2)*(sj1));
IkReal x473=((cj0)*(px));
IkReal x474=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x471)*(x473)))+(((x471)*(x474)))+(((x472)*(x474)))+(((x472)*(x473)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0750000000000000))*(x471)))+(((IkReal(-0.0750000000000000))*(x472)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3)))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x475=((cj1)*(sj2));
IkReal x476=((cj0)*(r00));
IkReal x477=((IkReal(1.00000000000000))*(pz));
IkReal x478=((sj1)*(sj2));
IkReal x479=((r01)*(sj0));
IkReal x480=((IkReal(1.00000000000000))*(r02));
IkReal x481=((cj1)*(cj2));
IkReal x482=((cj2)*(sj1));
IkReal x483=((IkReal(1.00000000000000))*(cj0));
IkReal x484=((IkReal(1.00000000000000))*(sj0)*(x481));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x483))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x476)*(x481)))+(((x478)*(x479)))+(((IkReal(-1.00000000000000))*(x475)*(x480)))+(((IkReal(-1.00000000000000))*(x480)*(x482)))+(((IkReal(-1.00000000000000))*(x479)*(x481)))+(((x476)*(x478))));
evalcond[3]=((((x476)*(x482)))+(((r02)*(x478)))+(((IkReal(-1.00000000000000))*(x480)*(x481)))+(((x479)*(x482)))+(((x475)*(x476)))+(((x475)*(x479))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(px)*(x481)*(x483)))+(((py)*(sj0)*(x478)))+(((cj0)*(px)*(x478)))+(((IkReal(-0.0750000000000000))*(x478)))+(((IkReal(-1.00000000000000))*(x477)*(x482)))+(((IkReal(-1.00000000000000))*(py)*(x484)))+(((IkReal(0.0750000000000000))*(x481))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x485=((IkReal(12.5000000000000))*(sj2));
IkReal x486=((py)*(sj0));
IkReal x487=((cj2)*(sj1));
IkReal x488=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x488))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(12.5000000000000))*(x486)*(x487)))+(((px)*(x487)*(x488)))+(((cj0)*(cj1)*(px)*(x485)))+(((pz)*(sj1)*(x485)))+(((cj1)*(x485)*(x486)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x487))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x488)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(12.5000000000000))*(x486)*(x487)))+(((px)*(x487)*(x488)))+(((cj0)*(cj1)*(px)*(x485)))+(((pz)*(sj1)*(x485)))+(((cj1)*(x485)*(x486)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x487)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x488)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(12.5000000000000))*(x486)*(x487)))+(((px)*(x487)*(x488)))+(((cj0)*(cj1)*(px)*(x485)))+(((pz)*(sj1)*(x485)))+(((cj1)*(x485)*(x486)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x487)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x489=((cj1)*(sj2));
IkReal x490=((cj2)*(sj1));
IkReal x491=((cj0)*(px));
IkReal x492=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x489)*(x492)))+(((x489)*(x491)))+(((IkReal(-0.0750000000000000))*(x490)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x490)*(x491)))+(((x490)*(x492)))+(((IkReal(-0.0750000000000000))*(x489)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x493=((cj1)*(sj2));
IkReal x494=((r01)*(sj0));
IkReal x495=((cj0)*(r00));
IkReal x496=((cj2)*(sj1));
if( IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((x495)*(x496)))+(((x493)*(x495)))+(((x493)*(x494)))+(((x494)*(x496)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((gconst57)*(((((x495)*(x496)))+(((x493)*(x495)))+(((x493)*(x494)))+(((x494)*(x496)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x497=IKsin(j3);
IkReal x498=IKcos(j3);
IkReal x499=((sj1)*(sj2));
IkReal x500=((cj2)*(sj1));
IkReal x501=((IkReal(0.0800000000000000))*(sj4));
IkReal x502=((py)*(sj0));
IkReal x503=((IkReal(0.790000000000000))*(cj4));
IkReal x504=((IkReal(1.00000000000000))*(cj0));
IkReal x505=((IkReal(1.00000000000000))*(cj4));
IkReal x506=((r01)*(sj0));
IkReal x507=((cj1)*(sj2));
IkReal x508=((cj0)*(x507));
IkReal x509=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x497)*(x505)))+(((IkReal(-1.00000000000000))*(r01)*(x504))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x504)))+(((x497)*(x501)))+(((x497)*(x503)))+(((px)*(sj0))));
evalcond[2]=((((x500)*(x506)))+(((r02)*(x499)))+(((x506)*(x507)))+(((cj0)*(r00)*(x500)))+(((r00)*(x508)))+(((IkReal(-1.00000000000000))*(x498)*(x505)))+(((IkReal(-1.00000000000000))*(r02)*(x509))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((pz)*(x499)))+(((x500)*(x502)))+(((IkReal(-1.00000000000000))*(pz)*(x509)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x498)*(x503)))+(((x498)*(x501)))+(((px)*(x508)))+(((x502)*(x507)))+(((cj0)*(px)*(x500)))+(((IkReal(-0.0750000000000000))*(x507)))+(((IkReal(-0.0750000000000000))*(x500))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x510=((cj1)*(cj2));
IkReal x511=((IkReal(1.00000000000000))*(cj0));
IkReal x512=((sj1)*(sj2));
IkReal x513=((IkReal(1.00000000000000))*(sj0));
IkReal x514=((sj0)*(x512));
IkReal x515=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x516=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((r01)*(x514)))+(((cj0)*(r00)*(x512)))+(((IkReal(-1.00000000000000))*(r01)*(x510)*(x513)))+(((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r00)*(x510)*(x511)))+(((IkReal(-1.00000000000000))*(r02)*(x515)))+(((IkReal(-1.00000000000000))*(r02)*(x516))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x510)*(x511)))+(((cj0)*(px)*(x512)))+(((py)*(x514)))+(((IkReal(0.0750000000000000))*(x510)))+(((IkReal(-0.0750000000000000))*(x512)))+(((IkReal(-1.00000000000000))*(pz)*(x516)))+(((IkReal(-1.00000000000000))*(pz)*(x515)))+(((IkReal(0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x510)*(x513)))+(((IkReal(0.0800000000000000))*(cj4))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x517=((cj1)*(sj2));
IkReal x518=((cj0)*(r00));
IkReal x519=((IkReal(1.00000000000000))*(pz));
IkReal x520=((sj1)*(sj2));
IkReal x521=((r01)*(sj0));
IkReal x522=((IkReal(1.00000000000000))*(r02));
IkReal x523=((cj1)*(cj2));
IkReal x524=((cj2)*(sj1));
IkReal x525=((IkReal(1.00000000000000))*(cj0));
IkReal x526=((IkReal(1.00000000000000))*(sj0)*(x523));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x525))));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x518)*(x523)))+(((x518)*(x520)))+(((IkReal(-1.00000000000000))*(x517)*(x522)))+(((x520)*(x521)))+(((IkReal(-1.00000000000000))*(x522)*(x524)))+(((IkReal(-1.00000000000000))*(x521)*(x523))));
evalcond[3]=((((r02)*(x520)))+(((x517)*(x521)))+(((x518)*(x524)))+(((x521)*(x524)))+(((x517)*(x518)))+(((IkReal(-1.00000000000000))*(x522)*(x523))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x520)))+(((cj0)*(px)*(x520)))+(((IkReal(-1.00000000000000))*(px)*(x523)*(x525)))+(((IkReal(-1.00000000000000))*(x517)*(x519)))+(((py)*(sj0)*(x520)))+(((IkReal(0.0750000000000000))*(x523)))+(((IkReal(-1.00000000000000))*(py)*(x526)))+(((IkReal(-1.00000000000000))*(x519)*(x524))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x527=((IkReal(12.5000000000000))*(cj1));
IkReal x528=((py)*(sj0));
IkReal x529=((cj0)*(px));
IkReal x530=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x529)*(x530)))+(((IkReal(-1.00000000000000))*(x528)*(x530)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x527)*(x528)))+(((IkReal(-1.00000000000000))*(sj2)*(x527)*(x529)))+(((IkReal(4.56250000000000))*(cj2)))+(((cj2)*(pz)*(x527)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x529)*(x530)))+(((IkReal(-1.00000000000000))*(x528)*(x530)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x527)*(x528)))+(((IkReal(-1.00000000000000))*(sj2)*(x527)*(x529)))+(((IkReal(4.56250000000000))*(cj2)))+(((cj2)*(pz)*(x527)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x529)*(x530)))+(((IkReal(-1.00000000000000))*(x528)*(x530)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x527)*(x528)))+(((IkReal(-1.00000000000000))*(sj2)*(x527)*(x529)))+(((IkReal(4.56250000000000))*(cj2)))+(((cj2)*(pz)*(x527)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x531=((cj1)*(sj2));
IkReal x532=((cj2)*(sj1));
IkReal x533=((cj0)*(px));
IkReal x534=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x531)))+(((IkReal(-0.0750000000000000))*(x532)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x531)*(x533)))+(((x531)*(x534)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((x532)*(x533)))+(((x532)*(x534))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x535=((cj1)*(sj2));
IkReal x536=((cj0)*(r00));
IkReal x537=((IkReal(1.00000000000000))*(pz));
IkReal x538=((sj1)*(sj2));
IkReal x539=((r01)*(sj0));
IkReal x540=((IkReal(1.00000000000000))*(r02));
IkReal x541=((cj1)*(cj2));
IkReal x542=((cj2)*(sj1));
IkReal x543=((IkReal(1.00000000000000))*(cj0));
IkReal x544=((IkReal(1.00000000000000))*(sj0)*(x541));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x543))));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x536)*(x541)))+(((IkReal(-1.00000000000000))*(x539)*(x541)))+(((IkReal(-1.00000000000000))*(x540)*(x542)))+(((x536)*(x538)))+(((IkReal(-1.00000000000000))*(x535)*(x540)))+(((x538)*(x539))));
evalcond[3]=((((x539)*(x542)))+(((r02)*(x538)))+(((IkReal(-1.00000000000000))*(x540)*(x541)))+(((x536)*(x542)))+(((x535)*(x536)))+(((x535)*(x539))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x537)*(x542)))+(((IkReal(-1.00000000000000))*(px)*(x541)*(x543)))+(((IkReal(-0.0750000000000000))*(x538)))+(((cj0)*(px)*(x538)))+(((IkReal(0.0750000000000000))*(x541)))+(((py)*(sj0)*(x538)))+(((IkReal(-1.00000000000000))*(py)*(x544)))+(((IkReal(-1.00000000000000))*(x535)*(x537))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x545=((IkReal(12.5000000000000))*(sj2));
IkReal x546=((py)*(sj0));
IkReal x547=((cj2)*(sj1));
IkReal x548=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x548))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x545)))+(((IkReal(-0.937500000000000))*(x547)))+(((IkReal(12.5000000000000))*(x546)*(x547)))+(((cj1)*(x545)*(x546)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x547)*(x548)))+(((cj0)*(cj1)*(px)*(x545))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x548)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x545)))+(((IkReal(-0.937500000000000))*(x547)))+(((IkReal(12.5000000000000))*(x546)*(x547)))+(((cj1)*(x545)*(x546)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x547)*(x548)))+(((cj0)*(cj1)*(px)*(x545)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x548)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x545)))+(((IkReal(-0.937500000000000))*(x547)))+(((IkReal(12.5000000000000))*(x546)*(x547)))+(((cj1)*(x545)*(x546)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((px)*(x547)*(x548)))+(((cj0)*(cj1)*(px)*(x545)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x549=((cj1)*(sj2));
IkReal x550=((cj2)*(sj1));
IkReal x551=((cj0)*(px));
IkReal x552=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x549)))+(((IkReal(-0.0750000000000000))*(x550)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((x550)*(x551)))+(((x550)*(x552)))+(((x549)*(x552)))+(((x549)*(x551)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x553=((IkReal(1.00000000000000))*(sj2));
IkReal x554=((cj0)*(r00));
IkReal x555=((r01)*(sj0));
IkReal x556=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(-1.00000000000000))*(x554)*(x556)))+(((IkReal(-1.00000000000000))*(cj1)*(x553)*(x554)))+(((IkReal(-1.00000000000000))*(cj1)*(x553)*(x555)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x553)))+(((cj1)*(cj2)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst58)*(((((IkReal(-1.00000000000000))*(x555)*(x556)))+(((IkReal(-1.00000000000000))*(x554)*(x556)))+(((IkReal(-1.00000000000000))*(cj1)*(x553)*(x554)))+(((IkReal(-1.00000000000000))*(cj1)*(x553)*(x555)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x553)))+(((cj1)*(cj2)*(r02)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x557=IKsin(j3);
IkReal x558=IKcos(j3);
IkReal x559=((sj1)*(sj2));
IkReal x560=((cj2)*(sj1));
IkReal x561=((IkReal(0.0800000000000000))*(sj4));
IkReal x562=((py)*(sj0));
IkReal x563=((IkReal(1.00000000000000))*(cj0));
IkReal x564=((r01)*(sj0));
IkReal x565=((cj1)*(sj2));
IkReal x566=((cj0)*(x565));
IkReal x567=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x568=((cj4)*(x557));
IkReal x569=((cj4)*(x558));
evalcond[0]=((((r00)*(sj0)))+(x568)+(((IkReal(-1.00000000000000))*(r01)*(x563))));
evalcond[1]=((((IkReal(-0.790000000000000))*(x568)))+(((IkReal(-1.00000000000000))*(py)*(x563)))+(((px)*(sj0)))+(((x557)*(x561))));
evalcond[2]=((((x564)*(x565)))+(((r02)*(x559)))+(((r00)*(x566)))+(((cj0)*(r00)*(x560)))+(((x560)*(x564)))+(x569)+(((IkReal(-1.00000000000000))*(r02)*(x567))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x558)*(x561)))+(((cj0)*(px)*(x560)))+(((pz)*(x559)))+(((IkReal(-0.790000000000000))*(x569)))+(((x562)*(x565)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pz)*(x567)))+(((x560)*(x562)))+(((IkReal(-0.0750000000000000))*(x560)))+(((IkReal(-0.0750000000000000))*(x565)))+(((px)*(x566))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x570=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x571=((r00)*(sj0));
IkReal x572=((cj4)*(cj5));
IkReal x573=((IkReal(8.00000000000000))*(sj4));
IkReal x574=((cj0)*(r01));
IkReal x575=((IkReal(-0.125000000000000))*(x570));
IkReal x576=((IkReal(100.000000000000))*(cj0)*(py));
IkReal x577=((IkReal(100.000000000000))*(px)*(sj0));
if( IKabs(((x575)*(((((IkReal(79.0000000000000))*(x571)))+(((IkReal(-79.0000000000000))*(x574)))+(((IkReal(-1.00000000000000))*(x576)))+(x577))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x575)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-79.0000000000000))*(x571)*(x572)))+(((IkReal(79.0000000000000))*(x572)*(x574)))+(((IkReal(-1.00000000000000))*(x571)*(x573)))+(((x573)*(x574)))+(((x572)*(x576)))+(((IkReal(-1.00000000000000))*(x572)*(x577))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x575)*(((((IkReal(79.0000000000000))*(x571)))+(((IkReal(-79.0000000000000))*(x574)))+(((IkReal(-1.00000000000000))*(x576)))+(x577)))))+IKsqr(((x575)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-79.0000000000000))*(x571)*(x572)))+(((IkReal(79.0000000000000))*(x572)*(x574)))+(((IkReal(-1.00000000000000))*(x571)*(x573)))+(((x573)*(x574)))+(((x572)*(x576)))+(((IkReal(-1.00000000000000))*(x572)*(x577)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x575)*(((((IkReal(79.0000000000000))*(x571)))+(((IkReal(-79.0000000000000))*(x574)))+(((IkReal(-1.00000000000000))*(x576)))+(x577)))), ((x575)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-79.0000000000000))*(x571)*(x572)))+(((IkReal(79.0000000000000))*(x572)*(x574)))+(((IkReal(-1.00000000000000))*(x571)*(x573)))+(((x573)*(x574)))+(((x572)*(x576)))+(((IkReal(-1.00000000000000))*(x572)*(x577)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x578=IKsin(j3);
IkReal x579=IKcos(j3);
IkReal x580=((sj1)*(sj2));
IkReal x581=((IkReal(0.0800000000000000))*(sj4));
IkReal x582=((cj2)*(sj1));
IkReal x583=((py)*(sj0));
IkReal x584=((IkReal(0.790000000000000))*(sj5));
IkReal x585=((IkReal(1.00000000000000))*(cj0));
IkReal x586=((r01)*(sj0));
IkReal x587=((cj1)*(sj2));
IkReal x588=((cj0)*(x587));
IkReal x589=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x590=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x591=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x579)))+(((IkReal(-1.00000000000000))*(x578)*(x589)))+(((IkReal(-1.00000000000000))*(r01)*(x585))));
evalcond[1]=((((x578)*(x581)))+(((x579)*(x584)))+(((IkReal(-1.00000000000000))*(py)*(x585)))+(((x578)*(x590)))+(((px)*(sj0))));
evalcond[2]=((((cj0)*(r00)*(x582)))+(((x582)*(x586)))+(((r02)*(x580)))+(((IkReal(-1.00000000000000))*(x579)*(x589)))+(((x586)*(x587)))+(((IkReal(-1.00000000000000))*(r02)*(x591)))+(((sj5)*(x578)))+(((r00)*(x588))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((cj0)*(px)*(x582)))+(((x583)*(x587)))+(((x579)*(x581)))+(((IkReal(-0.0750000000000000))*(x582)))+(((IkReal(-0.0750000000000000))*(x587)))+(((px)*(x588)))+(((x582)*(x583)))+(((IkReal(-1.00000000000000))*(x578)*(x584)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x579)*(x590)))+(((IkReal(-1.00000000000000))*(pz)*(x591)))+(((pz)*(x580))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x592=((cj4)*(cj5));
IkReal x593=((r00)*(sj0));
IkReal x594=((cj0)*(r00));
IkReal x595=((r01)*(sj0));
IkReal x596=((IkReal(1.00000000000000))*(cj2));
IkReal x597=((cj1)*(r02));
IkReal x598=((sj1)*(sj5));
IkReal x599=((cj1)*(sj2));
IkReal x600=((cj2)*(sj1));
IkReal x601=((r02)*(sj2));
IkReal x602=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x603=((IkReal(1.00000000000000))*(sj5)*(x599));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x592)*(x602)))+(((IkReal(-1.00000000000000))*(x595)*(x596)*(x598)))+(((cj2)*(sj5)*(x597)))+(((IkReal(-1.00000000000000))*(x594)*(x596)*(x598)))+(((IkReal(-1.00000000000000))*(x598)*(x601)))+(((IkReal(-1.00000000000000))*(x594)*(x603)))+(((x592)*(x593)))+(((IkReal(-1.00000000000000))*(x595)*(x603))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((sj1)*(x592)*(x601)))+(((IkReal(-1.00000000000000))*(x592)*(x596)*(x597)))+(((IkReal(-1.00000000000000))*(sj5)*(x602)))+(((x592)*(x595)*(x600)))+(((x592)*(x594)*(x600)))+(((x592)*(x595)*(x599)))+(((x592)*(x594)*(x599)))+(((sj5)*(x593))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(x592)*(x602)))+(((IkReal(-1.00000000000000))*(x595)*(x596)*(x598)))+(((cj2)*(sj5)*(x597)))+(((IkReal(-1.00000000000000))*(x594)*(x596)*(x598)))+(((IkReal(-1.00000000000000))*(x598)*(x601)))+(((IkReal(-1.00000000000000))*(x594)*(x603)))+(((x592)*(x593)))+(((IkReal(-1.00000000000000))*(x595)*(x603)))))), ((gconst54)*(((((sj1)*(x592)*(x601)))+(((IkReal(-1.00000000000000))*(x592)*(x596)*(x597)))+(((IkReal(-1.00000000000000))*(sj5)*(x602)))+(((x592)*(x595)*(x600)))+(((x592)*(x594)*(x600)))+(((x592)*(x595)*(x599)))+(((x592)*(x594)*(x599)))+(((sj5)*(x593)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x604=IKsin(j3);
IkReal x605=IKcos(j3);
IkReal x606=((sj1)*(sj2));
IkReal x607=((IkReal(0.0800000000000000))*(sj4));
IkReal x608=((cj2)*(sj1));
IkReal x609=((py)*(sj0));
IkReal x610=((IkReal(0.790000000000000))*(sj5));
IkReal x611=((IkReal(1.00000000000000))*(cj0));
IkReal x612=((r01)*(sj0));
IkReal x613=((cj1)*(sj2));
IkReal x614=((cj0)*(x613));
IkReal x615=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x616=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x617=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x611)))+(((IkReal(-1.00000000000000))*(sj5)*(x605)))+(((IkReal(-1.00000000000000))*(x604)*(x615))));
evalcond[1]=((((x604)*(x607)))+(((x605)*(x610)))+(((IkReal(-1.00000000000000))*(py)*(x611)))+(((px)*(sj0)))+(((x604)*(x616))));
evalcond[2]=((((r00)*(x614)))+(((x608)*(x612)))+(((IkReal(-1.00000000000000))*(x605)*(x615)))+(((x612)*(x613)))+(((cj0)*(r00)*(x608)))+(((r02)*(x606)))+(((sj5)*(x604)))+(((IkReal(-1.00000000000000))*(r02)*(x617))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x605)*(x607)))+(((x608)*(x609)))+(((x605)*(x616)))+(((IkReal(-0.0750000000000000))*(x613)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x614)))+(((IkReal(-0.0750000000000000))*(x608)))+(((IkReal(-1.00000000000000))*(x604)*(x610)))+(((cj0)*(px)*(x608)))+(((IkReal(-1.00000000000000))*(pz)*(x617)))+(((x609)*(x613)))+(((pz)*(x606))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x618=((r01)*(sj0));
IkReal x619=((sj1)*(sj2));
IkReal x620=((IkReal(1.00000000000000))*(cj1));
IkReal x621=((cj0)*(r00));
cj5array[0]=((IkReal(-1.00000000000000))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj2)*(x618)*(x620)))+(((x619)*(x621)))+(((IkReal(-1.00000000000000))*(cj2)*(x620)*(x621)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x620)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x618)*(x619))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x622=((cj0)*(px));
IkReal x623=((IkReal(1.00000000000000))*(cj1));
IkReal x624=((sj1)*(sj2));
IkReal x625=((py)*(sj0));
evalcond[0]=((IkReal(0.405000000000000))+(((IkReal(0.0750000000000000))*(cj1)*(cj2)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x623)*(x625)))+(((IkReal(-0.790000000000000))*(sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x623)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(sj1)))+(((x624)*(x625)))+(((IkReal(-0.0750000000000000))*(x624)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(x622)*(x623)))+(((x622)*(x624))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign((((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5))));
dummyeval[0]=(((((cj4)*(cj4))*((cj5)*(cj5))))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x626=((cj1)*(cj2));
IkReal x627=((IkReal(1.00000000000000))*(cj0));
IkReal x628=((sj1)*(sj2));
IkReal x629=((IkReal(1.00000000000000))*(sj0));
IkReal x630=((sj0)*(x628));
IkReal x631=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x632=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x631)))+(((IkReal(-1.00000000000000))*(r02)*(x632)))+(((IkReal(-1.00000000000000))*(r01)*(x626)*(x629)))+(((r01)*(x630)))+(((cj0)*(r00)*(x628)))+(((IkReal(-1.00000000000000))*(r00)*(x626)*(x627))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((py)*(x630)))+(((IkReal(-1.00000000000000))*(pz)*(x631)))+(((IkReal(-1.00000000000000))*(pz)*(x632)))+(((cj0)*(px)*(x628)))+(((IkReal(-1.00000000000000))*(px)*(x626)*(x627)))+(((IkReal(-1.00000000000000))*(py)*(x626)*(x629)))+(((IkReal(0.0750000000000000))*(x626)))+(((IkReal(-0.0750000000000000))*(x628))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x633=((cj1)*(cj5));
IkReal x634=((IkReal(1.00000000000000))*(cj2));
IkReal x635=((r00)*(sj0));
IkReal x636=((IkReal(1.00000000000000))*(sj5));
IkReal x637=((cj0)*(r01));
IkReal x638=((r01)*(sj0));
IkReal x639=((sj1)*(sj5));
IkReal x640=((cj0)*(r00));
IkReal x641=((sj2)*(x640));
IkReal x642=((cj2)*(cj5)*(sj1));
IkReal x643=((r02)*(sj1)*(sj2));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(cj5)*(x637)))+(((cj5)*(x635)))+(((IkReal(-1.00000000000000))*(x634)*(x639)*(x640)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x636)*(x638)))+(((IkReal(-1.00000000000000))*(cj1)*(x636)*(x641)))+(((IkReal(-1.00000000000000))*(x636)*(x643)))+(((IkReal(-1.00000000000000))*(x634)*(x638)*(x639)))+(((cj1)*(cj2)*(r02)*(sj5))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((x638)*(x642)))+(((cj5)*(x643)))+(((sj5)*(x635)))+(((IkReal(-1.00000000000000))*(r02)*(x633)*(x634)))+(((x640)*(x642)))+(((IkReal(-1.00000000000000))*(x636)*(x637)))+(((sj2)*(x633)*(x638)))+(((x633)*(x641))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(cj5)*(x637)))+(((cj5)*(x635)))+(((IkReal(-1.00000000000000))*(x634)*(x639)*(x640)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x636)*(x638)))+(((IkReal(-1.00000000000000))*(cj1)*(x636)*(x641)))+(((IkReal(-1.00000000000000))*(x636)*(x643)))+(((IkReal(-1.00000000000000))*(x634)*(x638)*(x639)))+(((cj1)*(cj2)*(r02)*(sj5)))))), ((gconst55)*(((((x638)*(x642)))+(((cj5)*(x643)))+(((sj5)*(x635)))+(((IkReal(-1.00000000000000))*(r02)*(x633)*(x634)))+(((x640)*(x642)))+(((IkReal(-1.00000000000000))*(x636)*(x637)))+(((sj2)*(x633)*(x638)))+(((x633)*(x641)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x644=IKsin(j3);
IkReal x645=IKcos(j3);
IkReal x646=((sj1)*(sj2));
IkReal x647=((cj2)*(sj1));
IkReal x648=((cj0)*(px));
IkReal x649=((py)*(sj0));
IkReal x650=((IkReal(0.790000000000000))*(sj5));
IkReal x651=((cj1)*(sj2));
IkReal x652=((cj0)*(r00));
IkReal x653=((IkReal(1.00000000000000))*(cj0));
IkReal x654=((r01)*(sj0));
IkReal x655=((cj5)*(x644));
IkReal x656=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x657=((cj5)*(x645));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x645)))+(((IkReal(-1.00000000000000))*(x655)))+(((IkReal(-1.00000000000000))*(r01)*(x653))));
evalcond[1]=((((x645)*(x650)))+(((IkReal(0.790000000000000))*(x655)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x653))));
evalcond[2]=((((x647)*(x654)))+(((x647)*(x652)))+(((IkReal(-1.00000000000000))*(x657)))+(((r02)*(x646)))+(((x651)*(x654)))+(((x651)*(x652)))+(((sj5)*(x644)))+(((IkReal(-1.00000000000000))*(r02)*(x656))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x651)))+(((IkReal(-0.0750000000000000))*(x647)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x649)*(x651)))+(((x648)*(x651)))+(((pz)*(x646)))+(((IkReal(0.790000000000000))*(x657)))+(((IkReal(-1.00000000000000))*(pz)*(x656)))+(((IkReal(-1.00000000000000))*(x644)*(x650)))+(((x647)*(x648)))+(((x647)*(x649))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x658=((cj1)*(cj2));
IkReal x659=((IkReal(1.00000000000000))*(cj0));
IkReal x660=((sj1)*(sj2));
IkReal x661=((IkReal(1.00000000000000))*(sj0));
IkReal x662=((sj0)*(x660));
IkReal x663=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x664=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r01)*(x662)))+(((IkReal(-1.00000000000000))*(r01)*(x658)*(x661)))+(((cj0)*(r00)*(x660)))+(((IkReal(-1.00000000000000))*(r00)*(x658)*(x659)))+(((IkReal(-1.00000000000000))*(r02)*(x664)))+(((IkReal(-1.00000000000000))*(r02)*(x663))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x658)))+(((IkReal(-1.00000000000000))*(py)*(x658)*(x661)))+(((IkReal(-0.0750000000000000))*(x660)))+(((cj0)*(px)*(x660)))+(((py)*(x662)))+(((IkReal(-1.00000000000000))*(px)*(x658)*(x659)))+(((IkReal(-1.00000000000000))*(pz)*(x664)))+(((IkReal(-1.00000000000000))*(pz)*(x663))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x665=((cj1)*(cj5));
IkReal x666=((cj0)*(r00));
IkReal x667=((cj5)*(sj0));
IkReal x668=((cj2)*(sj1));
IkReal x669=((r01)*(sj5));
IkReal x670=((cj1)*(sj5));
IkReal x671=((sj0)*(sj2));
IkReal x672=((IkReal(1.00000000000000))*(cj2)*(r02));
IkReal x673=((r02)*(sj1)*(sj2));
if( IKabs(((gconst56)*(((((sj5)*(x673)))+(((r00)*(x667)))+(((cj1)*(x669)*(x671)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj2)*(x666)*(x670)))+(((IkReal(-1.00000000000000))*(x670)*(x672)))+(((sj5)*(x666)*(x668)))+(((sj0)*(x668)*(x669))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((cj0)*(x669)))+(((r01)*(x665)*(x671)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x665)*(x672)))+(((sj2)*(x665)*(x666)))+(((r01)*(x667)*(x668)))+(((cj5)*(x666)*(x668)))+(((cj5)*(x673))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((sj5)*(x673)))+(((r00)*(x667)))+(((cj1)*(x669)*(x671)))+(((IkReal(-1.00000000000000))*(cj0)*(cj5)*(r01)))+(((sj2)*(x666)*(x670)))+(((IkReal(-1.00000000000000))*(x670)*(x672)))+(((sj5)*(x666)*(x668)))+(((sj0)*(x668)*(x669)))))), ((gconst56)*(((((cj0)*(x669)))+(((r01)*(x665)*(x671)))+(((IkReal(-1.00000000000000))*(r00)*(sj0)*(sj5)))+(((IkReal(-1.00000000000000))*(x665)*(x672)))+(((sj2)*(x665)*(x666)))+(((r01)*(x667)*(x668)))+(((cj5)*(x666)*(x668)))+(((cj5)*(x673)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x674=IKcos(j3);
IkReal x675=IKsin(j3);
IkReal x676=((sj1)*(sj2));
IkReal x677=((cj2)*(sj1));
IkReal x678=((py)*(sj0));
IkReal x679=((IkReal(1.00000000000000))*(cj0));
IkReal x680=((r01)*(sj0));
IkReal x681=((cj1)*(sj2));
IkReal x682=((IkReal(0.790000000000000))*(x675));
IkReal x683=((cj0)*(x681));
IkReal x684=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x685=((IkReal(0.790000000000000))*(x674));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj5)*(x674)))+(((cj5)*(x675)))+(((IkReal(-1.00000000000000))*(r01)*(x679))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj5)*(x682)))+(((IkReal(-1.00000000000000))*(py)*(x679)))+(((sj5)*(x685)))+(((px)*(sj0))));
evalcond[2]=((((r02)*(x676)))+(((sj5)*(x675)))+(((x677)*(x680)))+(((IkReal(-1.00000000000000))*(r02)*(x684)))+(((x680)*(x681)))+(((cj5)*(x674)))+(((r00)*(x683)))+(((cj0)*(r00)*(x677))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x678)*(x681)))+(((px)*(x683)))+(((pz)*(x676)))+(((cj0)*(px)*(x677)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x677)*(x678)))+(((IkReal(-1.00000000000000))*(cj5)*(x685)))+(((IkReal(-0.0750000000000000))*(x677)))+(((IkReal(-0.0750000000000000))*(x681)))+(((IkReal(-1.00000000000000))*(sj5)*(x682)))+(((IkReal(-1.00000000000000))*(pz)*(x684))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x686=((cj1)*(cj2));
IkReal x687=((IkReal(1.00000000000000))*(cj0));
IkReal x688=((sj1)*(sj2));
IkReal x689=((IkReal(1.00000000000000))*(sj0));
IkReal x690=((sj0)*(x688));
IkReal x691=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x692=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r01)*(x686)*(x689)))+(((IkReal(-1.00000000000000))*(r00)*(x686)*(x687)))+(((cj0)*(r00)*(x688)))+(((IkReal(-1.00000000000000))*(r02)*(x692)))+(((IkReal(-1.00000000000000))*(r02)*(x691)))+(((r01)*(x690))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((py)*(x690)))+(((IkReal(0.0750000000000000))*(x686)))+(((IkReal(-1.00000000000000))*(px)*(x686)*(x687)))+(((IkReal(-0.790000000000000))*(sj4)))+(((IkReal(-0.0750000000000000))*(x688)))+(((IkReal(-1.00000000000000))*(pz)*(x692)))+(((IkReal(-1.00000000000000))*(pz)*(x691)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x686)*(x689)))+(((cj0)*(px)*(x688))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x693=((cj1)*(sj2));
IkReal x694=((cj0)*(r00));
IkReal x695=((IkReal(1.00000000000000))*(pz));
IkReal x696=((sj1)*(sj2));
IkReal x697=((r01)*(sj0));
IkReal x698=((IkReal(1.00000000000000))*(r02));
IkReal x699=((cj1)*(cj2));
IkReal x700=((cj2)*(sj1));
IkReal x701=((IkReal(1.00000000000000))*(cj0));
IkReal x702=((IkReal(1.00000000000000))*(sj0)*(x699));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x701))));
evalcond[2]=((IkReal(1.00000000000000))+(((x694)*(x696)))+(((x696)*(x697)))+(((IkReal(-1.00000000000000))*(x697)*(x699)))+(((IkReal(-1.00000000000000))*(x693)*(x698)))+(((IkReal(-1.00000000000000))*(x694)*(x699)))+(((IkReal(-1.00000000000000))*(x698)*(x700))));
evalcond[3]=((((r02)*(x696)))+(((x694)*(x700)))+(((x693)*(x697)))+(((x693)*(x694)))+(((x697)*(x700)))+(((IkReal(-1.00000000000000))*(x698)*(x699))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-1.00000000000000))*(py)*(x702)))+(((IkReal(-0.365000000000000))*(sj2)))+(((py)*(sj0)*(x696)))+(((IkReal(-0.0750000000000000))*(x696)))+(((IkReal(0.0750000000000000))*(x699)))+(((IkReal(-1.00000000000000))*(x693)*(x695)))+(((IkReal(-1.00000000000000))*(x695)*(x700)))+(((IkReal(-1.00000000000000))*(px)*(x699)*(x701)))+(((cj0)*(px)*(x696))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x703=((IkReal(12.5000000000000))*(cj1));
IkReal x704=((py)*(sj0));
IkReal x705=((cj0)*(px));
IkReal x706=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x703)))+(((IkReal(-1.00000000000000))*(sj2)*(x703)*(x705)))+(((IkReal(-1.00000000000000))*(sj2)*(x703)*(x704)))+(((IkReal(-1.00000000000000))*(x704)*(x706)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x705)*(x706)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((cj2)*(pz)*(x703)))+(((IkReal(-1.00000000000000))*(sj2)*(x703)*(x705)))+(((IkReal(-1.00000000000000))*(sj2)*(x703)*(x704)))+(((IkReal(-1.00000000000000))*(x704)*(x706)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x705)*(x706)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((cj2)*(pz)*(x703)))+(((IkReal(-1.00000000000000))*(sj2)*(x703)*(x705)))+(((IkReal(-1.00000000000000))*(sj2)*(x703)*(x704)))+(((IkReal(-1.00000000000000))*(x704)*(x706)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(x705)*(x706)))+(((IkReal(4.56250000000000))*(cj2)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x707=((cj1)*(sj2));
IkReal x708=((cj2)*(sj1));
IkReal x709=((cj0)*(px));
IkReal x710=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x708)))+(((IkReal(-0.0750000000000000))*(x707)))+(((x707)*(x709)))+(((x707)*(x710)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((x708)*(x709)))+(((x708)*(x710))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x711=((cj1)*(sj2));
IkReal x712=((cj0)*(r00));
IkReal x713=((IkReal(1.00000000000000))*(pz));
IkReal x714=((sj1)*(sj2));
IkReal x715=((r01)*(sj0));
IkReal x716=((IkReal(1.00000000000000))*(r02));
IkReal x717=((cj1)*(cj2));
IkReal x718=((cj2)*(sj1));
IkReal x719=((IkReal(1.00000000000000))*(cj0));
IkReal x720=((IkReal(1.00000000000000))*(sj0)*(x717));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x719))));
evalcond[2]=((IkReal(-1.00000000000000))+(((x714)*(x715)))+(((IkReal(-1.00000000000000))*(x711)*(x716)))+(((IkReal(-1.00000000000000))*(x716)*(x718)))+(((IkReal(-1.00000000000000))*(x715)*(x717)))+(((IkReal(-1.00000000000000))*(x712)*(x717)))+(((x712)*(x714))));
evalcond[3]=((((x715)*(x718)))+(((x711)*(x712)))+(((x711)*(x715)))+(((IkReal(-1.00000000000000))*(x716)*(x717)))+(((r02)*(x714)))+(((x712)*(x718))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(0.0750000000000000))*(x717)))+(((IkReal(-1.00000000000000))*(x711)*(x713)))+(((IkReal(-0.0750000000000000))*(x714)))+(((py)*(sj0)*(x714)))+(((IkReal(-1.00000000000000))*(px)*(x717)*(x719)))+(((cj0)*(px)*(x714)))+(((IkReal(-1.00000000000000))*(x713)*(x718)))+(((IkReal(-1.00000000000000))*(py)*(x720))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x721=((IkReal(12.5000000000000))*(sj2));
IkReal x722=((py)*(sj0));
IkReal x723=((cj2)*(sj1));
IkReal x724=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x724))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((cj1)*(x721)*(x722)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x723)*(x724)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((pz)*(sj1)*(x721)))+(((IkReal(12.5000000000000))*(x722)*(x723)))+(((cj0)*(cj1)*(px)*(x721)))+(((IkReal(-0.937500000000000))*(x723))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x724)))))+IKsqr(((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((cj1)*(x721)*(x722)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x723)*(x724)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((pz)*(sj1)*(x721)))+(((IkReal(12.5000000000000))*(x722)*(x723)))+(((cj0)*(cj1)*(px)*(x721)))+(((IkReal(-0.937500000000000))*(x723)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x724)))), ((IkReal(-1.12500000000000))+(((IkReal(-4.56250000000000))*(cj2)))+(((cj1)*(x721)*(x722)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((px)*(x723)*(x724)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((pz)*(sj1)*(x721)))+(((IkReal(12.5000000000000))*(x722)*(x723)))+(((cj0)*(cj1)*(px)*(x721)))+(((IkReal(-0.937500000000000))*(x723)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x725=((cj1)*(sj2));
IkReal x726=((cj2)*(sj1));
IkReal x727=((cj0)*(px));
IkReal x728=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x725)))+(((IkReal(-0.0750000000000000))*(x726)))+(((x726)*(x727)))+(((x726)*(x728)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((x725)*(x728)))+(((x725)*(x727)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x729=((cj1)*(sj2));
IkReal x730=((r01)*(sj0));
IkReal x731=((cj0)*(r00));
IkReal x732=((cj2)*(sj1));
if( IKabs(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((x731)*(x732)))+(((x729)*(x730)))+(((x729)*(x731)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x730)*(x732))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst57)*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((gconst57)*(((((x731)*(x732)))+(((x729)*(x730)))+(((x729)*(x731)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((r02)*(sj1)*(sj2)))+(((x730)*(x732)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x733=IKsin(j3);
IkReal x734=IKcos(j3);
IkReal x735=((sj1)*(sj2));
IkReal x736=((cj2)*(sj1));
IkReal x737=((IkReal(0.0800000000000000))*(sj4));
IkReal x738=((py)*(sj0));
IkReal x739=((IkReal(0.790000000000000))*(cj4));
IkReal x740=((IkReal(1.00000000000000))*(cj0));
IkReal x741=((IkReal(1.00000000000000))*(cj4));
IkReal x742=((r01)*(sj0));
IkReal x743=((cj1)*(sj2));
IkReal x744=((cj0)*(x743));
IkReal x745=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x740)))+(((IkReal(-1.00000000000000))*(x733)*(x741))));
evalcond[1]=((((x733)*(x739)))+(((x733)*(x737)))+(((IkReal(-1.00000000000000))*(py)*(x740)))+(((px)*(sj0))));
evalcond[2]=((((x742)*(x743)))+(((cj0)*(r00)*(x736)))+(((IkReal(-1.00000000000000))*(r02)*(x745)))+(((IkReal(-1.00000000000000))*(x734)*(x741)))+(((r00)*(x744)))+(((r02)*(x735)))+(((x736)*(x742))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x736)))+(((x738)*(x743)))+(((x736)*(x738)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x734)*(x737)))+(((x734)*(x739)))+(((cj0)*(px)*(x736)))+(((IkReal(-0.0750000000000000))*(x743)))+(((px)*(x744)))+(((IkReal(-1.00000000000000))*(pz)*(x745)))+(((pz)*(x735))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x746=((cj1)*(cj2));
IkReal x747=((IkReal(1.00000000000000))*(cj0));
IkReal x748=((sj1)*(sj2));
IkReal x749=((IkReal(1.00000000000000))*(sj0));
IkReal x750=((sj0)*(x748));
IkReal x751=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x752=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(r01)*(x746)*(x749)))+(((IkReal(-1.00000000000000))*(r02)*(x752)))+(((IkReal(-1.00000000000000))*(r02)*(x751)))+(((r01)*(x750)))+(((IkReal(-1.00000000000000))*(r00)*(x746)*(x747)))+(((cj0)*(r00)*(x748))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((cj0)*(px)*(x748)))+(((py)*(x750)))+(((IkReal(-1.00000000000000))*(px)*(x746)*(x747)))+(((IkReal(-0.0750000000000000))*(x748)))+(((IkReal(0.0750000000000000))*(x746)))+(((IkReal(0.790000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)*(x752)))+(((IkReal(-1.00000000000000))*(pz)*(x751)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x746)*(x749))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x753=((cj1)*(sj2));
IkReal x754=((cj0)*(r00));
IkReal x755=((IkReal(1.00000000000000))*(pz));
IkReal x756=((sj1)*(sj2));
IkReal x757=((r01)*(sj0));
IkReal x758=((IkReal(1.00000000000000))*(r02));
IkReal x759=((cj1)*(cj2));
IkReal x760=((cj2)*(sj1));
IkReal x761=((IkReal(1.00000000000000))*(cj0));
IkReal x762=((IkReal(1.00000000000000))*(sj0)*(x759));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x761))));
evalcond[2]=((IkReal(-1.00000000000000))+(((x756)*(x757)))+(((IkReal(-1.00000000000000))*(x758)*(x760)))+(((IkReal(-1.00000000000000))*(x753)*(x758)))+(((IkReal(-1.00000000000000))*(x757)*(x759)))+(((IkReal(-1.00000000000000))*(x754)*(x759)))+(((x754)*(x756))));
evalcond[3]=((((x757)*(x760)))+(((r02)*(x756)))+(((IkReal(-1.00000000000000))*(x758)*(x759)))+(((x753)*(x754)))+(((x753)*(x757)))+(((x754)*(x760))));
evalcond[4]=((IkReal(1.19500000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x762)))+(((cj0)*(px)*(x756)))+(((IkReal(-1.00000000000000))*(x753)*(x755)))+(((IkReal(-1.00000000000000))*(x755)*(x760)))+(((IkReal(-0.0750000000000000))*(x756)))+(((IkReal(-1.00000000000000))*(px)*(x759)*(x761)))+(((py)*(sj0)*(x756)))+(((IkReal(0.0750000000000000))*(x759))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x763=((IkReal(12.5000000000000))*(cj1));
IkReal x764=((py)*(sj0));
IkReal x765=((cj0)*(px));
IkReal x766=((IkReal(12.5000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-1.00000000000000))*(x764)*(x766)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((cj2)*(pz)*(x763)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(sj2)*(x763)*(x764)))+(((IkReal(-1.00000000000000))*(sj2)*(x763)*(x765))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))))+IKsqr(((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-1.00000000000000))*(x764)*(x766)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((cj2)*(pz)*(x763)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(sj2)*(x763)*(x764)))+(((IkReal(-1.00000000000000))*(sj2)*(x763)*(x765)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(-12.5000000000000))*(px)*(sj0)))+(((IkReal(12.5000000000000))*(cj0)*(py)))), ((IkReal(1.12500000000000))+(((IkReal(-1.00000000000000))*(x765)*(x766)))+(((IkReal(-1.00000000000000))*(x764)*(x766)))+(((IkReal(-12.5000000000000))*(pz)*(sj1)*(sj2)))+(((IkReal(4.56250000000000))*(cj2)))+(((cj2)*(pz)*(x763)))+(((IkReal(0.937500000000000))*(cj1)*(sj2)))+(((IkReal(0.937500000000000))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(sj2)*(x763)*(x764)))+(((IkReal(-1.00000000000000))*(sj2)*(x763)*(x765)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x767=((cj1)*(sj2));
IkReal x768=((cj2)*(sj1));
IkReal x769=((cj0)*(px));
IkReal x770=((py)*(sj0));
evalcond[0]=((((IkReal(0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x767)))+(((IkReal(-0.0750000000000000))*(x768)))+(((x768)*(x769)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x767)*(x770)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(0.0800000000000000))*(IKcos(j3))))+(((x767)*(x769)))+(((x768)*(x770))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x771=((cj1)*(sj2));
IkReal x772=((cj0)*(r00));
IkReal x773=((IkReal(1.00000000000000))*(pz));
IkReal x774=((sj1)*(sj2));
IkReal x775=((r01)*(sj0));
IkReal x776=((IkReal(1.00000000000000))*(r02));
IkReal x777=((cj1)*(cj2));
IkReal x778=((cj2)*(sj1));
IkReal x779=((IkReal(1.00000000000000))*(cj0));
IkReal x780=((IkReal(1.00000000000000))*(sj0)*(x777));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x779))));
evalcond[2]=((IkReal(1.00000000000000))+(((x774)*(x775)))+(((IkReal(-1.00000000000000))*(x772)*(x777)))+(((x772)*(x774)))+(((IkReal(-1.00000000000000))*(x771)*(x776)))+(((IkReal(-1.00000000000000))*(x775)*(x777)))+(((IkReal(-1.00000000000000))*(x776)*(x778))));
evalcond[3]=((((x771)*(x772)))+(((x771)*(x775)))+(((x772)*(x778)))+(((IkReal(-1.00000000000000))*(x776)*(x777)))+(((r02)*(x774)))+(((x775)*(x778))));
evalcond[4]=((IkReal(-0.385000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((cj0)*(px)*(x774)))+(((IkReal(-0.0750000000000000))*(x774)))+(((IkReal(-1.00000000000000))*(px)*(x777)*(x779)))+(((IkReal(-1.00000000000000))*(x771)*(x773)))+(((IkReal(0.0750000000000000))*(x777)))+(((IkReal(-1.00000000000000))*(x773)*(x778)))+(((py)*(sj0)*(x774)))+(((IkReal(-1.00000000000000))*(py)*(x780))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x781=((IkReal(12.5000000000000))*(sj2));
IkReal x782=((py)*(sj0));
IkReal x783=((cj2)*(sj1));
IkReal x784=((IkReal(12.5000000000000))*(cj0));
if( IKabs(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x784))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.12500000000000))+(((px)*(x783)*(x784)))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x781)))+(((cj1)*(x781)*(x782)))+(((cj0)*(cj1)*(px)*(x781)))+(((IkReal(12.5000000000000))*(x782)*(x783)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x783))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x784)))))+IKsqr(((IkReal(-1.12500000000000))+(((px)*(x783)*(x784)))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x781)))+(((cj1)*(x781)*(x782)))+(((cj0)*(cj1)*(px)*(x781)))+(((IkReal(12.5000000000000))*(x782)*(x783)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x783)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((((IkReal(12.5000000000000))*(px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x784)))), ((IkReal(-1.12500000000000))+(((px)*(x783)*(x784)))+(((IkReal(-4.56250000000000))*(cj2)))+(((IkReal(-12.5000000000000))*(cj1)*(cj2)*(pz)))+(((pz)*(sj1)*(x781)))+(((cj1)*(x781)*(x782)))+(((cj0)*(cj1)*(px)*(x781)))+(((IkReal(12.5000000000000))*(x782)*(x783)))+(((IkReal(-0.937500000000000))*(cj1)*(sj2)))+(((IkReal(-0.937500000000000))*(x783)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x785=((cj1)*(sj2));
IkReal x786=((cj2)*(sj1));
IkReal x787=((cj0)*(px));
IkReal x788=((py)*(sj0));
evalcond[0]=((((IkReal(-0.0800000000000000))*(IKsin(j3))))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[1]=((IkReal(-0.0900000000000000))+(((x786)*(x788)))+(((x786)*(x787)))+(((x785)*(x787)))+(((x785)*(x788)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(sj1)*(sj2)))+(((IkReal(-0.0800000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(pz)))+(((IkReal(-0.0750000000000000))*(x785)))+(((IkReal(-0.0750000000000000))*(x786))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x789=((IkReal(1.00000000000000))*(sj2));
IkReal x790=((cj0)*(r00));
IkReal x791=((r01)*(sj0));
IkReal x792=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x789)))+(((IkReal(-1.00000000000000))*(x791)*(x792)))+(((IkReal(-1.00000000000000))*(x790)*(x792)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x789)*(x791)))+(((IkReal(-1.00000000000000))*(cj1)*(x789)*(x790))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))), ((gconst58)*(((((IkReal(-1.00000000000000))*(r02)*(sj1)*(x789)))+(((IkReal(-1.00000000000000))*(x791)*(x792)))+(((IkReal(-1.00000000000000))*(x790)*(x792)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x789)*(x791)))+(((IkReal(-1.00000000000000))*(cj1)*(x789)*(x790)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x793=IKsin(j3);
IkReal x794=IKcos(j3);
IkReal x795=((sj1)*(sj2));
IkReal x796=((cj2)*(sj1));
IkReal x797=((IkReal(0.0800000000000000))*(sj4));
IkReal x798=((py)*(sj0));
IkReal x799=((IkReal(1.00000000000000))*(cj0));
IkReal x800=((r01)*(sj0));
IkReal x801=((cj1)*(sj2));
IkReal x802=((cj0)*(x801));
IkReal x803=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x804=((cj4)*(x793));
IkReal x805=((cj4)*(x794));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x799)))+(x804));
evalcond[1]=((((x793)*(x797)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x799)))+(((IkReal(-0.790000000000000))*(x804))));
evalcond[2]=((((x800)*(x801)))+(((IkReal(-1.00000000000000))*(r02)*(x803)))+(((r02)*(x795)))+(((cj0)*(r00)*(x796)))+(((r00)*(x802)))+(x805)+(((x796)*(x800))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x796)*(x798)))+(((IkReal(-0.365000000000000))*(cj2)))+(((pz)*(x795)))+(((IkReal(-0.0750000000000000))*(x801)))+(((x798)*(x801)))+(((px)*(x802)))+(((x794)*(x797)))+(((IkReal(-0.0750000000000000))*(x796)))+(((cj0)*(px)*(x796)))+(((IkReal(-1.00000000000000))*(pz)*(x803)))+(((IkReal(-0.790000000000000))*(x805))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x806=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x807=((r00)*(sj0));
IkReal x808=((cj4)*(cj5));
IkReal x809=((IkReal(8.00000000000000))*(sj4));
IkReal x810=((cj0)*(r01));
IkReal x811=((IkReal(-0.125000000000000))*(x806));
IkReal x812=((IkReal(100.000000000000))*(cj0)*(py));
IkReal x813=((IkReal(100.000000000000))*(px)*(sj0));
if( IKabs(((x811)*(((((IkReal(-79.0000000000000))*(x810)))+(((IkReal(-1.00000000000000))*(x812)))+(((IkReal(79.0000000000000))*(x807)))+(x813))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x811)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x809)*(x810)))+(((IkReal(-1.00000000000000))*(x807)*(x809)))+(((IkReal(-1.00000000000000))*(x808)*(x813)))+(((IkReal(79.0000000000000))*(x808)*(x810)))+(((x808)*(x812)))+(((IkReal(-79.0000000000000))*(x807)*(x808))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x811)*(((((IkReal(-79.0000000000000))*(x810)))+(((IkReal(-1.00000000000000))*(x812)))+(((IkReal(79.0000000000000))*(x807)))+(x813)))))+IKsqr(((x811)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x809)*(x810)))+(((IkReal(-1.00000000000000))*(x807)*(x809)))+(((IkReal(-1.00000000000000))*(x808)*(x813)))+(((IkReal(79.0000000000000))*(x808)*(x810)))+(((x808)*(x812)))+(((IkReal(-79.0000000000000))*(x807)*(x808)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((x811)*(((((IkReal(-79.0000000000000))*(x810)))+(((IkReal(-1.00000000000000))*(x812)))+(((IkReal(79.0000000000000))*(x807)))+(x813)))), ((x811)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x809)*(x810)))+(((IkReal(-1.00000000000000))*(x807)*(x809)))+(((IkReal(-1.00000000000000))*(x808)*(x813)))+(((IkReal(79.0000000000000))*(x808)*(x810)))+(((x808)*(x812)))+(((IkReal(-79.0000000000000))*(x807)*(x808)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x814=IKsin(j3);
IkReal x815=IKcos(j3);
IkReal x816=((sj1)*(sj2));
IkReal x817=((IkReal(0.0800000000000000))*(sj4));
IkReal x818=((cj2)*(sj1));
IkReal x819=((py)*(sj0));
IkReal x820=((IkReal(0.790000000000000))*(sj5));
IkReal x821=((IkReal(1.00000000000000))*(cj0));
IkReal x822=((r01)*(sj0));
IkReal x823=((cj1)*(sj2));
IkReal x824=((cj0)*(x823));
IkReal x825=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x826=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x827=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x821)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x814)*(x825)))+(((IkReal(-1.00000000000000))*(sj5)*(x815))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x821)))+(((x815)*(x820)))+(((x814)*(x826)))+(((x814)*(x817)))+(((px)*(sj0))));
evalcond[2]=((((cj0)*(r00)*(x818)))+(((IkReal(-1.00000000000000))*(r02)*(x827)))+(((sj5)*(x814)))+(((IkReal(-1.00000000000000))*(x815)*(x825)))+(((r00)*(x824)))+(((x822)*(x823)))+(((x818)*(x822)))+(((r02)*(x816))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x823)))+(((IkReal(-0.0750000000000000))*(x818)))+(((x815)*(x817)))+(((cj0)*(px)*(x818)))+(((IkReal(-1.00000000000000))*(pz)*(x827)))+(((pz)*(x816)))+(((IkReal(-1.00000000000000))*(x814)*(x820)))+(((x815)*(x826)))+(((IkReal(-0.365000000000000))*(cj2)))+(((px)*(x824)))+(((x819)*(x823)))+(((x818)*(x819))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x828=((cj4)*(cj5));
IkReal x829=((r00)*(sj0));
IkReal x830=((cj0)*(r00));
IkReal x831=((r01)*(sj0));
IkReal x832=((IkReal(1.00000000000000))*(cj2));
IkReal x833=((cj1)*(r02));
IkReal x834=((sj1)*(sj5));
IkReal x835=((cj1)*(sj2));
IkReal x836=((cj2)*(sj1));
IkReal x837=((r02)*(sj2));
IkReal x838=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x839=((IkReal(1.00000000000000))*(sj5)*(x835));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(x834)*(x837)))+(((IkReal(-1.00000000000000))*(x828)*(x838)))+(((IkReal(-1.00000000000000))*(x830)*(x832)*(x834)))+(((IkReal(-1.00000000000000))*(x830)*(x839)))+(((IkReal(-1.00000000000000))*(x831)*(x832)*(x834)))+(((cj2)*(sj5)*(x833)))+(((x828)*(x829)))+(((IkReal(-1.00000000000000))*(x831)*(x839))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((x828)*(x830)*(x835)))+(((x828)*(x830)*(x836)))+(((sj1)*(x828)*(x837)))+(((IkReal(-1.00000000000000))*(x828)*(x832)*(x833)))+(((sj5)*(x829)))+(((x828)*(x831)*(x836)))+(((x828)*(x831)*(x835)))+(((IkReal(-1.00000000000000))*(sj5)*(x838))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(x834)*(x837)))+(((IkReal(-1.00000000000000))*(x828)*(x838)))+(((IkReal(-1.00000000000000))*(x830)*(x832)*(x834)))+(((IkReal(-1.00000000000000))*(x830)*(x839)))+(((IkReal(-1.00000000000000))*(x831)*(x832)*(x834)))+(((cj2)*(sj5)*(x833)))+(((x828)*(x829)))+(((IkReal(-1.00000000000000))*(x831)*(x839)))))), ((gconst54)*(((((x828)*(x830)*(x835)))+(((x828)*(x830)*(x836)))+(((sj1)*(x828)*(x837)))+(((IkReal(-1.00000000000000))*(x828)*(x832)*(x833)))+(((sj5)*(x829)))+(((x828)*(x831)*(x836)))+(((x828)*(x831)*(x835)))+(((IkReal(-1.00000000000000))*(sj5)*(x838)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x840=IKsin(j3);
IkReal x841=IKcos(j3);
IkReal x842=((sj1)*(sj2));
IkReal x843=((IkReal(0.0800000000000000))*(sj4));
IkReal x844=((cj2)*(sj1));
IkReal x845=((py)*(sj0));
IkReal x846=((IkReal(0.790000000000000))*(sj5));
IkReal x847=((IkReal(1.00000000000000))*(cj0));
IkReal x848=((r01)*(sj0));
IkReal x849=((cj1)*(sj2));
IkReal x850=((cj0)*(x849));
IkReal x851=((IkReal(1.00000000000000))*(cj4)*(cj5));
IkReal x852=((IkReal(0.790000000000000))*(cj4)*(cj5));
IkReal x853=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj5)*(x841)))+(((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x847)))+(((IkReal(-1.00000000000000))*(x840)*(x851))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x847)))+(((x840)*(x843)))+(((x840)*(x852)))+(((px)*(sj0)))+(((x841)*(x846))));
evalcond[2]=((((r00)*(x850)))+(((r02)*(x842)))+(((cj0)*(r00)*(x844)))+(((x848)*(x849)))+(((IkReal(-1.00000000000000))*(r02)*(x853)))+(((IkReal(-1.00000000000000))*(x841)*(x851)))+(((sj5)*(x840)))+(((x844)*(x848))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x844)))+(((IkReal(-0.0750000000000000))*(x849)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x840)*(x846)))+(((pz)*(x842)))+(((x841)*(x852)))+(((px)*(x850)))+(((x845)*(x849)))+(((IkReal(-1.00000000000000))*(pz)*(x853)))+(((x844)*(x845)))+(((cj0)*(px)*(x844)))+(((x841)*(x843))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x854=((IkReal(0.790000000000000))*(cj0));
IkReal x855=((cj2)*(sj1));
IkReal x856=((cj1)*(cj2));
IkReal x857=((IkReal(0.790000000000000))*(r02));
IkReal x858=((IkReal(0.790000000000000))*(sj0));
IkReal x859=((cj1)*(sj2));
IkReal x860=((sj1)*(sj2));
IkReal x861=((IkReal(1.00000000000000))*(cj0)*(px));
IkReal x862=((IkReal(1.00000000000000))*(py)*(sj0));
if( IKabs(((IkReal(-25.0000000000000))*(gconst53)*(((((r00)*(x858)))+(((IkReal(-1.00000000000000))*(r01)*(x854)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(25.0000000000000))*(gconst53)*(((IkReal(0.0900000000000000))+(((IkReal(0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(r00)*(x854)*(x855)))+(((IkReal(-1.00000000000000))*(r00)*(x854)*(x859)))+(((IkReal(-1.00000000000000))*(r01)*(x855)*(x858)))+(((IkReal(-1.00000000000000))*(pz)*(x860)))+(((x856)*(x857)))+(((IkReal(-1.00000000000000))*(x859)*(x862)))+(((IkReal(-1.00000000000000))*(x859)*(x861)))+(((IkReal(-1.00000000000000))*(r01)*(x858)*(x859)))+(((IkReal(0.0750000000000000))*(x859)))+(((IkReal(0.0750000000000000))*(x855)))+(((IkReal(-1.00000000000000))*(x857)*(x860)))+(((pz)*(x856)))+(((IkReal(-1.00000000000000))*(x855)*(x862)))+(((IkReal(-1.00000000000000))*(x855)*(x861))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-25.0000000000000))*(gconst53)*(((((r00)*(x858)))+(((IkReal(-1.00000000000000))*(r01)*(x854)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py)))))), ((IkReal(25.0000000000000))*(gconst53)*(((IkReal(0.0900000000000000))+(((IkReal(0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(r00)*(x854)*(x855)))+(((IkReal(-1.00000000000000))*(r00)*(x854)*(x859)))+(((IkReal(-1.00000000000000))*(r01)*(x855)*(x858)))+(((IkReal(-1.00000000000000))*(pz)*(x860)))+(((x856)*(x857)))+(((IkReal(-1.00000000000000))*(x859)*(x862)))+(((IkReal(-1.00000000000000))*(x859)*(x861)))+(((IkReal(-1.00000000000000))*(r01)*(x858)*(x859)))+(((IkReal(0.0750000000000000))*(x859)))+(((IkReal(0.0750000000000000))*(x855)))+(((IkReal(-1.00000000000000))*(x857)*(x860)))+(((pz)*(x856)))+(((IkReal(-1.00000000000000))*(x855)*(x862)))+(((IkReal(-1.00000000000000))*(x855)*(x861)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x863=IKsin(j3);
IkReal x864=IKcos(j3);
IkReal x865=((IkReal(0.790000000000000))*(cj0));
IkReal x866=((cj2)*(sj1));
IkReal x867=((IkReal(0.365000000000000))*(cj2));
IkReal x868=((sj1)*(sj2));
IkReal x869=((cj4)*(pz));
IkReal x870=((px)*(sj0));
IkReal x871=((cj4)*(r00));
IkReal x872=((IkReal(0.790000000000000))*(r02));
IkReal x873=((cj1)*(cj4));
IkReal x874=((cj0)*(px));
IkReal x875=((py)*(sj0));
IkReal x876=((cj1)*(cj2));
IkReal x877=((IkReal(0.0750000000000000))*(sj2));
IkReal x878=((IkReal(0.790000000000000))*(sj0));
IkReal x879=((cj1)*(sj2));
IkReal x880=((r01)*(x878));
IkReal x881=((IkReal(1.00000000000000))*(cj0)*(py));
IkReal x882=((IkReal(0.0800000000000000))*(sj4)*(x863));
IkReal x883=((IkReal(0.0800000000000000))*(sj4)*(x864));
evalcond[0]=((((r00)*(x878)))+(((IkReal(-1.00000000000000))*(r01)*(x865)))+(((IkReal(-1.00000000000000))*(x881)))+(x870)+(x882));
evalcond[1]=((((x871)*(x878)))+(((IkReal(-1.00000000000000))*(cj4)*(r01)*(x865)))+(((cj4)*(x882)))+(((cj4)*(x870)))+(((IkReal(-1.00000000000000))*(cj4)*(x881))));
evalcond[2]=((IkReal(0.0900000000000000))+(((x872)*(x876)))+(((IkReal(-1.00000000000000))*(x879)*(x880)))+(((IkReal(-1.00000000000000))*(r00)*(x865)*(x879)))+(((IkReal(-1.00000000000000))*(r00)*(x865)*(x866)))+(((IkReal(-1.00000000000000))*(x868)*(x872)))+(((IkReal(-1.00000000000000))*(x883)))+(((IkReal(-1.00000000000000))*(x875)*(x879)))+(((IkReal(-1.00000000000000))*(pz)*(x868)))+(((IkReal(-1.00000000000000))*(x874)*(x879)))+(((cj1)*(x877)))+(((IkReal(-1.00000000000000))*(x866)*(x874)))+(((IkReal(-1.00000000000000))*(x866)*(x875)))+(((IkReal(0.0750000000000000))*(x866)))+(((IkReal(-1.00000000000000))*(x866)*(x880)))+(x867)+(((pz)*(x876))));
evalcond[3]=((((x865)*(x866)*(x871)))+(((sj2)*(x873)*(x880)))+(((x868)*(x869)))+(((IkReal(-1.00000000000000))*(x869)*(x876)))+(((IkReal(-1.00000000000000))*(x873)*(x877)))+(((cj4)*(x866)*(x874)))+(((cj4)*(x866)*(x875)))+(((IkReal(-0.0750000000000000))*(cj4)*(x866)))+(((IkReal(-1.00000000000000))*(cj2)*(x872)*(x873)))+(((cj4)*(x883)))+(((cj4)*(x866)*(x880)))+(((sj2)*(x873)*(x875)))+(((sj2)*(x873)*(x874)))+(((cj4)*(x868)*(x872)))+(((IkReal(-1.00000000000000))*(cj4)*(x867)))+(((x865)*(x871)*(x879)))+(((IkReal(-0.0900000000000000))*(cj4))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x884=(sj3)*(sj3);
IkReal x885=(cj3)*(cj3);
dummyeval[0]=((x885)+(x884));
dummyeval[1]=((((cj4)*(x884)))+(((cj4)*(x885))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
IkReal x886=(sj3)*(sj3);
IkReal x887=(cj3)*(cj3);
dummyeval[0]=((((IkReal(-1.00000000000000))*(x887)))+(((IkReal(-1.00000000000000))*(x886))));
dummyeval[1]=((((cj4)*(x886)))+(((cj4)*(x887))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x888=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x888))));
evalcond[2]=((IkReal(0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x888)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x889=((r02)*(sj1));
IkReal x890=((IkReal(1.00000000000000))*(sj2));
IkReal x891=((IkReal(1.00000000000000))*(cj2));
IkReal x892=((cj1)*(r02));
IkReal x893=((cj0)*(cj1)*(r00));
IkReal x894=((cj0)*(r00)*(sj1));
IkReal x895=((r01)*(sj0)*(sj1));
IkReal x896=((cj1)*(r01)*(sj0));
if( IKabs(((((cj2)*(x892)))+(((IkReal(-1.00000000000000))*(x889)*(x890)))+(((IkReal(-1.00000000000000))*(x890)*(x893)))+(((IkReal(-1.00000000000000))*(x890)*(x896)))+(((IkReal(-1.00000000000000))*(x891)*(x894)))+(((IkReal(-1.00000000000000))*(x891)*(x895))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj2)*(x893)))+(((cj2)*(x896)))+(((cj2)*(x889)))+(((sj2)*(x892)))+(((IkReal(-1.00000000000000))*(x890)*(x894)))+(((IkReal(-1.00000000000000))*(x890)*(x895))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(x892)))+(((IkReal(-1.00000000000000))*(x889)*(x890)))+(((IkReal(-1.00000000000000))*(x890)*(x893)))+(((IkReal(-1.00000000000000))*(x890)*(x896)))+(((IkReal(-1.00000000000000))*(x891)*(x894)))+(((IkReal(-1.00000000000000))*(x891)*(x895)))))+IKsqr(((((cj2)*(x893)))+(((cj2)*(x896)))+(((cj2)*(x889)))+(((sj2)*(x892)))+(((IkReal(-1.00000000000000))*(x890)*(x894)))+(((IkReal(-1.00000000000000))*(x890)*(x895)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(x892)))+(((IkReal(-1.00000000000000))*(x889)*(x890)))+(((IkReal(-1.00000000000000))*(x890)*(x893)))+(((IkReal(-1.00000000000000))*(x890)*(x896)))+(((IkReal(-1.00000000000000))*(x891)*(x894)))+(((IkReal(-1.00000000000000))*(x891)*(x895)))), ((((cj2)*(x893)))+(((cj2)*(x896)))+(((cj2)*(x889)))+(((sj2)*(x892)))+(((IkReal(-1.00000000000000))*(x890)*(x894)))+(((IkReal(-1.00000000000000))*(x890)*(x895)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x897=IKcos(j5);
IkReal x898=IKsin(j5);
IkReal x899=((pz)*(sj2));
IkReal x900=((IkReal(1.00000000000000))*(cj1));
IkReal x901=((cj2)*(sj0));
IkReal x902=((cj2)*(sj1));
IkReal x903=((cj0)*(px));
IkReal x904=((py)*(sj1));
IkReal x905=((sj1)*(sj2));
IkReal x906=((r01)*(sj0));
IkReal x907=((sj0)*(sj2));
IkReal x908=((IkReal(0.0750000000000000))*(cj1));
IkReal x909=((cj1)*(sj2));
IkReal x910=((cj0)*(r00));
IkReal x911=((sj2)*(x910));
evalcond[0]=((((x905)*(x906)))+(((IkReal(-1.00000000000000))*(r02)*(x902)))+(((IkReal(-1.00000000000000))*(r01)*(x900)*(x901)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x900)))+(((IkReal(-1.00000000000000))*(cj2)*(x900)*(x910)))+(x897)+(((x905)*(x910))));
evalcond[1]=((((x902)*(x910)))+(((x909)*(x910)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x900)))+(((r02)*(x905)))+(((x906)*(x909)))+(x898)+(((r01)*(sj1)*(x901))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x903)*(x905)))+(((cj2)*(x908)))+(((IkReal(-0.790000000000000))*(x897)))+(((IkReal(-0.0750000000000000))*(x905)))+(((IkReal(-1.00000000000000))*(cj2)*(x900)*(x903)))+(((IkReal(-1.00000000000000))*(x899)*(x900)))+(((x904)*(x907)))+(((IkReal(-1.00000000000000))*(pz)*(x902)))+(((IkReal(-1.00000000000000))*(py)*(x900)*(x901))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x903)*(x909)))+(((sj1)*(x899)))+(((x901)*(x904)))+(((x902)*(x903)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x908)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x900)))+(((IkReal(-0.790000000000000))*(x898)))+(((cj1)*(py)*(x907)))+(((IkReal(-0.0750000000000000))*(x902))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x912=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x912))));
evalcond[2]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x912)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x913=((sj1)*(sj2));
IkReal x914=((IkReal(1.00000000000000))*(r02));
IkReal x915=((r01)*(sj0));
IkReal x916=((cj0)*(r00));
IkReal x917=((cj1)*(sj2));
IkReal x918=((cj2)*(sj1));
IkReal x919=((IkReal(1.00000000000000))*(cj1)*(cj2));
if( IKabs(((((IkReal(-1.00000000000000))*(x913)*(x914)))+(((IkReal(-1.00000000000000))*(x915)*(x917)))+(((IkReal(-1.00000000000000))*(x915)*(x918)))+(((IkReal(-1.00000000000000))*(x916)*(x917)))+(((IkReal(-1.00000000000000))*(x916)*(x918)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x914)*(x918)))+(((IkReal(-1.00000000000000))*(x914)*(x917)))+(((x913)*(x916)))+(((x913)*(x915)))+(((IkReal(-1.00000000000000))*(x916)*(x919)))+(((IkReal(-1.00000000000000))*(x915)*(x919))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x913)*(x914)))+(((IkReal(-1.00000000000000))*(x915)*(x917)))+(((IkReal(-1.00000000000000))*(x915)*(x918)))+(((IkReal(-1.00000000000000))*(x916)*(x917)))+(((IkReal(-1.00000000000000))*(x916)*(x918)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((((IkReal(-1.00000000000000))*(x914)*(x918)))+(((IkReal(-1.00000000000000))*(x914)*(x917)))+(((x913)*(x916)))+(((x913)*(x915)))+(((IkReal(-1.00000000000000))*(x916)*(x919)))+(((IkReal(-1.00000000000000))*(x915)*(x919)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x913)*(x914)))+(((IkReal(-1.00000000000000))*(x915)*(x917)))+(((IkReal(-1.00000000000000))*(x915)*(x918)))+(((IkReal(-1.00000000000000))*(x916)*(x917)))+(((IkReal(-1.00000000000000))*(x916)*(x918)))+(((cj1)*(cj2)*(r02)))), ((((IkReal(-1.00000000000000))*(x914)*(x918)))+(((IkReal(-1.00000000000000))*(x914)*(x917)))+(((x913)*(x916)))+(((x913)*(x915)))+(((IkReal(-1.00000000000000))*(x916)*(x919)))+(((IkReal(-1.00000000000000))*(x915)*(x919)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x920=IKcos(j5);
IkReal x921=IKsin(j5);
IkReal x922=((pz)*(sj2));
IkReal x923=((IkReal(1.00000000000000))*(cj1));
IkReal x924=((cj2)*(sj0));
IkReal x925=((cj2)*(sj1));
IkReal x926=((cj0)*(px));
IkReal x927=((py)*(sj1));
IkReal x928=((sj1)*(sj2));
IkReal x929=((r01)*(sj0));
IkReal x930=((sj0)*(sj2));
IkReal x931=((IkReal(0.0750000000000000))*(cj1));
IkReal x932=((cj1)*(sj2));
IkReal x933=((cj0)*(r00));
IkReal x934=((sj2)*(x933));
evalcond[0]=((((IkReal(-1.00000000000000))*(cj2)*(x923)*(x933)))+(((x928)*(x929)))+(((x928)*(x933)))+(((IkReal(-1.00000000000000))*(x920)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x923)))+(((IkReal(-1.00000000000000))*(r02)*(x925)))+(((IkReal(-1.00000000000000))*(r01)*(x923)*(x924))));
evalcond[1]=((((r01)*(sj1)*(x924)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x923)))+(((x925)*(x933)))+(x921)+(((x929)*(x932)))+(((r02)*(x928)))+(((x932)*(x933))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x923)*(x926)))+(((x927)*(x930)))+(((IkReal(0.790000000000000))*(x920)))+(((cj2)*(x931)))+(((IkReal(-0.0750000000000000))*(x928)))+(((IkReal(-1.00000000000000))*(pz)*(x925)))+(((x926)*(x928)))+(((IkReal(-1.00000000000000))*(py)*(x923)*(x924)))+(((IkReal(-1.00000000000000))*(x922)*(x923))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x923)))+(((IkReal(-1.00000000000000))*(sj2)*(x931)))+(((x924)*(x927)))+(((IkReal(-0.790000000000000))*(x921)))+(((IkReal(-0.365000000000000))*(cj2)))+(((sj1)*(x922)))+(((IkReal(-0.0750000000000000))*(x925)))+(((cj1)*(py)*(x930)))+(((x925)*(x926)))+(((x926)*(x932))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x935=((cj0)*(px));
IkReal x936=((py)*(sj0));
IkReal x937=((cj1)*(sj2));
IkReal x938=((IkReal(1.26582278481013))*(pz));
IkReal x939=((IkReal(1.26582278481013))*(cj2)*(sj1));
if( IKabs(((IkReal(-0.113924050632911))+(((x936)*(x939)))+(((IkReal(-0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x938)))+(((IkReal(1.26582278481013))*(x936)*(x937)))+(((x935)*(x939)))+(((IkReal(-0.0949367088607595))*(x937)))+(((sj1)*(sj2)*(x938)))+(((IkReal(-0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(1.26582278481013))*(x935)*(x937))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.113924050632911))+(((x936)*(x939)))+(((IkReal(-0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x938)))+(((IkReal(1.26582278481013))*(x936)*(x937)))+(((x935)*(x939)))+(((IkReal(-0.0949367088607595))*(x937)))+(((sj1)*(sj2)*(x938)))+(((IkReal(-0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(1.26582278481013))*(x935)*(x937)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-0.113924050632911))+(((x936)*(x939)))+(((IkReal(-0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x938)))+(((IkReal(1.26582278481013))*(x936)*(x937)))+(((x935)*(x939)))+(((IkReal(-0.0949367088607595))*(x937)))+(((sj1)*(sj2)*(x938)))+(((IkReal(-0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(1.26582278481013))*(x935)*(x937)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x940=IKcos(j5);
IkReal x941=IKsin(j5);
IkReal x942=((pz)*(sj2));
IkReal x943=((IkReal(1.00000000000000))*(cj1));
IkReal x944=((cj2)*(sj0));
IkReal x945=((cj2)*(sj1));
IkReal x946=((cj0)*(px));
IkReal x947=((py)*(sj1));
IkReal x948=((sj1)*(sj2));
IkReal x949=((r01)*(sj0));
IkReal x950=((sj0)*(sj2));
IkReal x951=((IkReal(0.0750000000000000))*(cj1));
IkReal x952=((cj1)*(sj2));
IkReal x953=((IkReal(1.00000000000000))*(cj0));
IkReal x954=((cj0)*(r00));
IkReal x955=((sj2)*(x954));
IkReal x956=((sj4)*(x940));
IkReal x957=((cj4)*(x940));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x953)))+(((IkReal(-1.00000000000000))*(x957))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x953)))+(((IkReal(0.790000000000000))*(x957)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x943)*(x944)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x943)))+(((x948)*(x949)))+(((IkReal(-1.00000000000000))*(cj2)*(x943)*(x954)))+(((IkReal(-1.00000000000000))*(r02)*(x945)))+(((x948)*(x954)))+(x956));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x943)))+(((x952)*(x954)))+(((r02)*(x948)))+(((x949)*(x952)))+(((r01)*(sj1)*(x944)))+(((x945)*(x954)))+(x941));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x946)*(x948)))+(((IkReal(-0.790000000000000))*(x956)))+(((IkReal(-1.00000000000000))*(py)*(x943)*(x944)))+(((IkReal(-1.00000000000000))*(x942)*(x943)))+(((IkReal(-1.00000000000000))*(cj2)*(x943)*(x946)))+(((IkReal(-0.0750000000000000))*(x948)))+(((cj2)*(x951)))+(((x947)*(x950)))+(((IkReal(-1.00000000000000))*(pz)*(x945)))+(((IkReal(0.0800000000000000))*(cj4))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x950)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x943)))+(((x945)*(x946)))+(((sj1)*(x942)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.790000000000000))*(x941)))+(((IkReal(-0.0750000000000000))*(x945)))+(((x944)*(x947)))+(((IkReal(-1.00000000000000))*(sj2)*(x951)))+(((x946)*(x952))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x958=((IkReal(1.00000000000000))*(sj2));
IkReal x959=((cj0)*(r00));
IkReal x960=((r01)*(sj0));
IkReal x961=((IkReal(1.00000000000000))*(cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x960)*(x961)))+(((IkReal(-1.00000000000000))*(x959)*(x961)))+(((IkReal(-1.00000000000000))*(cj1)*(x958)*(x960)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x958)))+(((IkReal(-1.00000000000000))*(cj1)*(x958)*(x959)))+(((cj1)*(cj2)*(r02))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x960)*(x961)))+(((IkReal(-1.00000000000000))*(x959)*(x961)))+(((IkReal(-1.00000000000000))*(cj1)*(x958)*(x960)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x958)))+(((IkReal(-1.00000000000000))*(cj1)*(x958)*(x959)))+(((cj1)*(cj2)*(r02)))))+IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(100.000000000000))*(cj0)*(py)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x960)*(x961)))+(((IkReal(-1.00000000000000))*(x959)*(x961)))+(((IkReal(-1.00000000000000))*(cj1)*(x958)*(x960)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x958)))+(((IkReal(-1.00000000000000))*(cj1)*(x958)*(x959)))+(((cj1)*(cj2)*(r02)))), ((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(100.000000000000))*(cj0)*(py)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x962=IKcos(j5);
IkReal x963=IKsin(j5);
IkReal x964=((pz)*(sj2));
IkReal x965=((IkReal(1.00000000000000))*(cj1));
IkReal x966=((cj2)*(sj0));
IkReal x967=((cj2)*(sj1));
IkReal x968=((cj0)*(px));
IkReal x969=((py)*(sj1));
IkReal x970=((sj1)*(sj2));
IkReal x971=((r01)*(sj0));
IkReal x972=((sj0)*(sj2));
IkReal x973=((IkReal(0.0750000000000000))*(cj1));
IkReal x974=((cj1)*(sj2));
IkReal x975=((IkReal(1.00000000000000))*(cj0));
IkReal x976=((cj0)*(r00));
IkReal x977=((sj2)*(x976));
IkReal x978=((sj4)*(x962));
IkReal x979=((cj4)*(x962));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x975)))+(((IkReal(-1.00000000000000))*(x979))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x975)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x979))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(x965)*(x976)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x965)))+(((IkReal(-1.00000000000000))*(r01)*(x965)*(x966)))+(((x970)*(x976)))+(((x970)*(x971)))+(x978)+(((IkReal(-1.00000000000000))*(r02)*(x967))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x965)))+(((r01)*(sj1)*(x966)))+(((x971)*(x974)))+(((x974)*(x976)))+(((x967)*(x976)))+(((r02)*(x970)))+(x963));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x969)*(x972)))+(((IkReal(-0.790000000000000))*(x978)))+(((IkReal(-0.0750000000000000))*(x970)))+(((IkReal(-1.00000000000000))*(x964)*(x965)))+(((x968)*(x970)))+(((IkReal(-1.00000000000000))*(py)*(x965)*(x966)))+(((cj2)*(x973)))+(((IkReal(-1.00000000000000))*(pz)*(x967)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(x965)*(x968))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x967)))+(((IkReal(-0.790000000000000))*(x963)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x965)))+(((x968)*(x974)))+(((x967)*(x968)))+(((sj1)*(x964)))+(((x966)*(x969)))+(((IkReal(-1.00000000000000))*(sj2)*(x973)))+(((cj1)*(py)*(x972))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x980=((IkReal(1.00000000000000))*(sj2));
IkReal x981=((IkReal(1.00000000000000))*(r01));
IkReal x982=((cj0)*(r00));
IkReal x983=((cj2)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(x982)*(x983)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x980)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x980)))+(((IkReal(-1.00000000000000))*(sj0)*(x981)*(x983)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x980)*(x982))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x981))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(x982)*(x983)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x980)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x980)))+(((IkReal(-1.00000000000000))*(sj0)*(x981)*(x983)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x980)*(x982)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x981)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(x982)*(x983)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj0)*(x980)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x980)))+(((IkReal(-1.00000000000000))*(sj0)*(x981)*(x983)))+(((cj1)*(cj2)*(r02)))+(((IkReal(-1.00000000000000))*(cj1)*(x980)*(x982)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x981)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x984=IKcos(j5);
IkReal x985=IKsin(j5);
IkReal x986=((pz)*(sj2));
IkReal x987=((IkReal(1.00000000000000))*(cj1));
IkReal x988=((cj2)*(sj0));
IkReal x989=((cj2)*(sj1));
IkReal x990=((cj0)*(px));
IkReal x991=((py)*(sj1));
IkReal x992=((sj1)*(sj2));
IkReal x993=((r01)*(sj0));
IkReal x994=((sj0)*(sj2));
IkReal x995=((IkReal(0.0750000000000000))*(cj1));
IkReal x996=((cj1)*(sj2));
IkReal x997=((IkReal(1.00000000000000))*(cj0));
IkReal x998=((cj0)*(r00));
IkReal x999=((sj2)*(x998));
IkReal x1000=((sj4)*(x984));
IkReal x1001=((cj4)*(x984));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1001)))+(((IkReal(-1.00000000000000))*(r01)*(x997))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(py)*(x997)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x1001))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x989)))+(x1000)+(((x992)*(x998)))+(((x992)*(x993)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x987)))+(((IkReal(-1.00000000000000))*(r01)*(x987)*(x988)))+(((IkReal(-1.00000000000000))*(cj2)*(x987)*(x998))));
evalcond[3]=((((x993)*(x996)))+(((x989)*(x998)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x987)))+(((r02)*(x992)))+(((r01)*(sj1)*(x988)))+(((x996)*(x998)))+(x985));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.790000000000000))*(x1000)))+(((x991)*(x994)))+(((x990)*(x992)))+(((IkReal(-1.00000000000000))*(x986)*(x987)))+(((IkReal(-1.00000000000000))*(cj2)*(x987)*(x990)))+(((IkReal(-1.00000000000000))*(pz)*(x989)))+(((IkReal(-1.00000000000000))*(py)*(x987)*(x988)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x995)))+(((IkReal(-0.0750000000000000))*(x992))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x989)*(x990)))+(((IkReal(-0.790000000000000))*(x985)))+(((x988)*(x991)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x987)))+(((x990)*(x996)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x989)))+(((IkReal(-1.00000000000000))*(sj2)*(x995)))+(((sj1)*(x986)))+(((cj1)*(py)*(x994))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x1002=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1002))));
evalcond[2]=((IkReal(-0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1002)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1003=((r01)*(sj0));
IkReal x1004=((cj1)*(sj2));
IkReal x1005=((sj1)*(sj2));
IkReal x1006=((cj2)*(sj1));
IkReal x1007=((cj0)*(r00));
IkReal x1008=((cj1)*(cj2));
IkReal x1009=((cj1)*(x1007));
if( IKabs(((((r02)*(x1005)))+(((IkReal(-1.00000000000000))*(r02)*(x1008)))+(((x1004)*(x1007)))+(((x1006)*(x1007)))+(((x1003)*(x1006)))+(((x1003)*(x1004))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1003)*(x1005)))+(((r02)*(x1006)))+(((r02)*(x1004)))+(((IkReal(-1.00000000000000))*(x1005)*(x1007)))+(((x1007)*(x1008)))+(((x1003)*(x1008))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r02)*(x1005)))+(((IkReal(-1.00000000000000))*(r02)*(x1008)))+(((x1004)*(x1007)))+(((x1006)*(x1007)))+(((x1003)*(x1006)))+(((x1003)*(x1004)))))+IKsqr(((((IkReal(-1.00000000000000))*(x1003)*(x1005)))+(((r02)*(x1006)))+(((r02)*(x1004)))+(((IkReal(-1.00000000000000))*(x1005)*(x1007)))+(((x1007)*(x1008)))+(((x1003)*(x1008)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r02)*(x1005)))+(((IkReal(-1.00000000000000))*(r02)*(x1008)))+(((x1004)*(x1007)))+(((x1006)*(x1007)))+(((x1003)*(x1006)))+(((x1003)*(x1004)))), ((((IkReal(-1.00000000000000))*(x1003)*(x1005)))+(((r02)*(x1006)))+(((r02)*(x1004)))+(((IkReal(-1.00000000000000))*(x1005)*(x1007)))+(((x1007)*(x1008)))+(((x1003)*(x1008)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1010=IKcos(j5);
IkReal x1011=IKsin(j5);
IkReal x1012=((pz)*(sj2));
IkReal x1013=((IkReal(1.00000000000000))*(cj1));
IkReal x1014=((cj2)*(sj0));
IkReal x1015=((cj2)*(sj1));
IkReal x1016=((cj0)*(px));
IkReal x1017=((py)*(sj1));
IkReal x1018=((sj1)*(sj2));
IkReal x1019=((r01)*(sj0));
IkReal x1020=((sj0)*(sj2));
IkReal x1021=((IkReal(0.0750000000000000))*(cj1));
IkReal x1022=((cj1)*(sj2));
IkReal x1023=((cj0)*(r00));
IkReal x1024=((sj2)*(x1023));
evalcond[0]=((x1010)+(((IkReal(-1.00000000000000))*(cj2)*(x1013)*(x1023)))+(((x1018)*(x1023)))+(((IkReal(-1.00000000000000))*(r02)*(x1015)))+(((IkReal(-1.00000000000000))*(r01)*(x1013)*(x1014)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1013)))+(((x1018)*(x1019))));
evalcond[1]=((((r01)*(sj1)*(x1014)))+(((r02)*(x1018)))+(((x1015)*(x1023)))+(((x1022)*(x1023)))+(((x1019)*(x1022)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1013)))+(((IkReal(-1.00000000000000))*(x1011))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(x1012)*(x1013)))+(((cj2)*(x1021)))+(((IkReal(-0.365000000000000))*(sj2)))+(((x1017)*(x1020)))+(((x1016)*(x1018)))+(((IkReal(-0.0750000000000000))*(x1018)))+(((IkReal(-1.00000000000000))*(cj2)*(x1013)*(x1016)))+(((IkReal(-1.00000000000000))*(pz)*(x1015)))+(((IkReal(-1.00000000000000))*(py)*(x1013)*(x1014)))+(((IkReal(-0.790000000000000))*(x1010))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1014)*(x1017)))+(((IkReal(-0.0750000000000000))*(x1015)))+(((sj1)*(x1012)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1015)*(x1016)))+(((cj1)*(py)*(x1020)))+(((IkReal(0.790000000000000))*(x1011)))+(((IkReal(-1.00000000000000))*(sj2)*(x1021)))+(((x1016)*(x1022)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1013))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x1025=((IkReal(1.00000000000000))*(cj0));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1025))));
evalcond[2]=((IkReal(0.0800000000000000))+(((IkReal(-1.00000000000000))*(py)*(x1025)))+(((px)*(sj0))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1026=((IkReal(1.00000000000000))*(cj2));
IkReal x1027=((r02)*(sj2));
IkReal x1028=((r01)*(sj0));
IkReal x1029=((cj2)*(sj1));
IkReal x1030=((cj0)*(r00));
IkReal x1031=((sj2)*(x1028));
IkReal x1032=((cj1)*(x1030));
if( IKabs(((((sj1)*(x1027)))+(((x1028)*(x1029)))+(((sj2)*(x1032)))+(((x1029)*(x1030)))+(((cj1)*(x1031)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1026))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(x1027)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1026)))+(((sj1)*(sj2)*(x1030)))+(((IkReal(-1.00000000000000))*(x1026)*(x1032)))+(((sj1)*(x1031)))+(((IkReal(-1.00000000000000))*(cj1)*(x1026)*(x1028))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((sj1)*(x1027)))+(((x1028)*(x1029)))+(((sj2)*(x1032)))+(((x1029)*(x1030)))+(((cj1)*(x1031)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1026)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(x1027)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1026)))+(((sj1)*(sj2)*(x1030)))+(((IkReal(-1.00000000000000))*(x1026)*(x1032)))+(((sj1)*(x1031)))+(((IkReal(-1.00000000000000))*(cj1)*(x1026)*(x1028)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((sj1)*(x1027)))+(((x1028)*(x1029)))+(((sj2)*(x1032)))+(((x1029)*(x1030)))+(((cj1)*(x1031)))+(((IkReal(-1.00000000000000))*(cj1)*(r02)*(x1026)))), ((((IkReal(-1.00000000000000))*(cj1)*(x1027)))+(((IkReal(-1.00000000000000))*(r02)*(sj1)*(x1026)))+(((sj1)*(sj2)*(x1030)))+(((IkReal(-1.00000000000000))*(x1026)*(x1032)))+(((sj1)*(x1031)))+(((IkReal(-1.00000000000000))*(cj1)*(x1026)*(x1028)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1033=IKcos(j5);
IkReal x1034=IKsin(j5);
IkReal x1035=((pz)*(sj2));
IkReal x1036=((IkReal(1.00000000000000))*(cj1));
IkReal x1037=((cj2)*(sj0));
IkReal x1038=((cj2)*(sj1));
IkReal x1039=((cj0)*(px));
IkReal x1040=((py)*(sj1));
IkReal x1041=((sj1)*(sj2));
IkReal x1042=((r01)*(sj0));
IkReal x1043=((sj0)*(sj2));
IkReal x1044=((IkReal(0.0750000000000000))*(cj1));
IkReal x1045=((cj1)*(sj2));
IkReal x1046=((cj0)*(r00));
IkReal x1047=((sj2)*(x1046));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1033)))+(((x1041)*(x1042)))+(((x1041)*(x1046)))+(((IkReal(-1.00000000000000))*(r02)*(x1038)))+(((IkReal(-1.00000000000000))*(cj2)*(x1036)*(x1046)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1036)))+(((IkReal(-1.00000000000000))*(r01)*(x1036)*(x1037))));
evalcond[1]=((((r02)*(x1041)))+(((x1038)*(x1046)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1036)))+(((x1042)*(x1045)))+(((IkReal(-1.00000000000000))*(x1034)))+(((r01)*(sj1)*(x1037)))+(((x1045)*(x1046))));
evalcond[2]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1036)*(x1039)))+(((IkReal(-1.00000000000000))*(x1035)*(x1036)))+(((x1040)*(x1043)))+(((x1039)*(x1041)))+(((cj2)*(x1044)))+(((IkReal(-1.00000000000000))*(pz)*(x1038)))+(((IkReal(0.790000000000000))*(x1033)))+(((IkReal(-1.00000000000000))*(py)*(x1036)*(x1037)))+(((IkReal(-0.0750000000000000))*(x1041))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((x1038)*(x1039)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1036)))+(((x1037)*(x1040)))+(((x1039)*(x1045)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj1)*(py)*(x1043)))+(((sj1)*(x1035)))+(((IkReal(-1.00000000000000))*(sj2)*(x1044)))+(((IkReal(0.790000000000000))*(x1034)))+(((IkReal(-0.0750000000000000))*(x1038))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1048=((cj0)*(px));
IkReal x1049=((py)*(sj0));
IkReal x1050=((cj1)*(sj2));
IkReal x1051=((IkReal(1.26582278481013))*(pz));
IkReal x1052=((IkReal(1.26582278481013))*(cj2)*(sj1));
if( IKabs(((IkReal(0.113924050632911))+(((IkReal(-1.26582278481013))*(x1048)*(x1050)))+(((IkReal(-1.26582278481013))*(x1049)*(x1050)))+(((IkReal(0.0949367088607595))*(x1050)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(x1048)*(x1052)))+(((cj1)*(cj2)*(x1051)))+(((IkReal(0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(x1049)*(x1052)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1051))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.113924050632911))+(((IkReal(-1.26582278481013))*(x1048)*(x1050)))+(((IkReal(-1.26582278481013))*(x1049)*(x1050)))+(((IkReal(0.0949367088607595))*(x1050)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(x1048)*(x1052)))+(((cj1)*(cj2)*(x1051)))+(((IkReal(0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(x1049)*(x1052)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1051)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.113924050632911))+(((IkReal(-1.26582278481013))*(x1048)*(x1050)))+(((IkReal(-1.26582278481013))*(x1049)*(x1050)))+(((IkReal(0.0949367088607595))*(x1050)))+(((IkReal(0.462025316455696))*(cj2)))+(((IkReal(-1.00000000000000))*(x1048)*(x1052)))+(((cj1)*(cj2)*(x1051)))+(((IkReal(0.0949367088607595))*(cj2)*(sj1)))+(((IkReal(-1.00000000000000))*(x1049)*(x1052)))+(((IkReal(-1.00000000000000))*(sj1)*(sj2)*(x1051)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1053=IKcos(j5);
IkReal x1054=IKsin(j5);
IkReal x1055=((pz)*(sj2));
IkReal x1056=((IkReal(1.00000000000000))*(cj1));
IkReal x1057=((cj2)*(sj0));
IkReal x1058=((cj2)*(sj1));
IkReal x1059=((cj0)*(px));
IkReal x1060=((py)*(sj1));
IkReal x1061=((sj1)*(sj2));
IkReal x1062=((r01)*(sj0));
IkReal x1063=((sj0)*(sj2));
IkReal x1064=((IkReal(0.0750000000000000))*(cj1));
IkReal x1065=((cj1)*(sj2));
IkReal x1066=((IkReal(1.00000000000000))*(cj0));
IkReal x1067=((cj0)*(r00));
IkReal x1068=((sj2)*(x1067));
IkReal x1069=((sj4)*(x1053));
IkReal x1070=((cj4)*(x1053));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1066)))+(x1070));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1066)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4)))+(((IkReal(-0.790000000000000))*(x1070))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1056)))+(((IkReal(-1.00000000000000))*(cj2)*(x1056)*(x1067)))+(((x1061)*(x1062)))+(((x1061)*(x1067)))+(x1069)+(((IkReal(-1.00000000000000))*(r02)*(x1058)))+(((IkReal(-1.00000000000000))*(r01)*(x1056)*(x1057))));
evalcond[3]=((((r01)*(sj1)*(x1057)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1056)))+(((x1062)*(x1065)))+(((IkReal(-1.00000000000000))*(x1054)))+(((r02)*(x1061)))+(((x1058)*(x1067)))+(((x1065)*(x1067))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x1058)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1056)*(x1059)))+(((x1060)*(x1063)))+(((cj2)*(x1064)))+(((IkReal(-1.00000000000000))*(x1055)*(x1056)))+(((IkReal(-0.790000000000000))*(x1069)))+(((IkReal(-0.0750000000000000))*(x1061)))+(((IkReal(0.0800000000000000))*(cj4)))+(((x1059)*(x1061)))+(((IkReal(-1.00000000000000))*(py)*(x1056)*(x1057))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(0.790000000000000))*(x1054)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1064)))+(((sj1)*(x1055)))+(((x1057)*(x1060)))+(((cj1)*(py)*(x1063)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1056)))+(((x1059)*(x1065)))+(((IkReal(-0.0750000000000000))*(x1058)))+(((x1058)*(x1059))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1071=((cj1)*(sj2));
IkReal x1072=((r01)*(sj0));
IkReal x1073=((cj0)*(r00));
IkReal x1074=((cj2)*(sj1));
if( IKabs(((((x1071)*(x1073)))+(((x1071)*(x1072)))+(((x1073)*(x1074)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1072)*(x1074)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(-100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1071)*(x1073)))+(((x1071)*(x1072)))+(((x1073)*(x1074)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1072)*(x1074)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(-100.000000000000))*(cj0)*(py)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1071)*(x1073)))+(((x1071)*(x1072)))+(((x1073)*(x1074)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1072)*(x1074)))+(((r02)*(sj1)*(sj2)))), ((IkReal(0.0126582278481013))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj4)))+(((IkReal(-100.000000000000))*(cj0)*(py)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1075=IKcos(j5);
IkReal x1076=IKsin(j5);
IkReal x1077=((pz)*(sj2));
IkReal x1078=((IkReal(1.00000000000000))*(cj1));
IkReal x1079=((cj2)*(sj0));
IkReal x1080=((cj2)*(sj1));
IkReal x1081=((cj0)*(px));
IkReal x1082=((py)*(sj1));
IkReal x1083=((sj1)*(sj2));
IkReal x1084=((r01)*(sj0));
IkReal x1085=((sj0)*(sj2));
IkReal x1086=((IkReal(0.0750000000000000))*(cj1));
IkReal x1087=((cj1)*(sj2));
IkReal x1088=((IkReal(1.00000000000000))*(cj0));
IkReal x1089=((cj0)*(r00));
IkReal x1090=((sj2)*(x1089));
IkReal x1091=((sj4)*(x1075));
IkReal x1092=((cj4)*(x1075));
evalcond[0]=((((r00)*(sj0)))+(x1092)+(((IkReal(-1.00000000000000))*(r01)*(x1088))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1088)))+(((IkReal(-0.790000000000000))*(x1092)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((x1091)+(((IkReal(-1.00000000000000))*(cj2)*(x1078)*(x1089)))+(((IkReal(-1.00000000000000))*(r02)*(x1080)))+(((IkReal(-1.00000000000000))*(r01)*(x1078)*(x1079)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1078)))+(((x1083)*(x1089)))+(((x1083)*(x1084))));
evalcond[3]=((((r01)*(sj1)*(x1079)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1078)))+(((x1087)*(x1089)))+(((x1080)*(x1089)))+(((IkReal(-1.00000000000000))*(x1076)))+(((r02)*(x1083)))+(((x1084)*(x1087))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1080)))+(((x1082)*(x1085)))+(((IkReal(-1.00000000000000))*(x1077)*(x1078)))+(((IkReal(-0.790000000000000))*(x1091)))+(((x1081)*(x1083)))+(((IkReal(-1.00000000000000))*(cj2)*(x1078)*(x1081)))+(((IkReal(-1.00000000000000))*(py)*(x1078)*(x1079)))+(((IkReal(-0.0750000000000000))*(x1083)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x1086))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1079)*(x1082)))+(((IkReal(-1.00000000000000))*(sj2)*(x1086)))+(((x1081)*(x1087)))+(((IkReal(-0.365000000000000))*(cj2)))+(((sj1)*(x1077)))+(((IkReal(-0.0750000000000000))*(x1080)))+(((x1080)*(x1081)))+(((cj1)*(py)*(x1085)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1078)))+(((IkReal(0.790000000000000))*(x1076))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1093=((cj1)*(sj2));
IkReal x1094=((r01)*(sj0));
IkReal x1095=((cj0)*(r00));
IkReal x1096=((cj2)*(sj1));
if( IKabs(((((x1095)*(x1096)))+(((x1094)*(x1096)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1093)*(x1095)))+(((x1093)*(x1094)))+(((r02)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((x1095)*(x1096)))+(((x1094)*(x1096)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1093)*(x1095)))+(((x1093)*(x1094)))+(((r02)*(sj1)*(sj2)))))+IKsqr(((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((x1095)*(x1096)))+(((x1094)*(x1096)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r02)))+(((x1093)*(x1095)))+(((x1093)*(x1094)))+(((r02)*(sj1)*(sj2)))), ((((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r00)*(sj0)))+(((cj0)*(r01)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1097=IKcos(j5);
IkReal x1098=IKsin(j5);
IkReal x1099=((pz)*(sj2));
IkReal x1100=((IkReal(1.00000000000000))*(cj1));
IkReal x1101=((cj2)*(sj0));
IkReal x1102=((cj2)*(sj1));
IkReal x1103=((cj0)*(px));
IkReal x1104=((py)*(sj1));
IkReal x1105=((sj1)*(sj2));
IkReal x1106=((r01)*(sj0));
IkReal x1107=((sj0)*(sj2));
IkReal x1108=((IkReal(0.0750000000000000))*(cj1));
IkReal x1109=((cj1)*(sj2));
IkReal x1110=((IkReal(1.00000000000000))*(cj0));
IkReal x1111=((cj0)*(r00));
IkReal x1112=((sj2)*(x1111));
IkReal x1113=((sj4)*(x1097));
IkReal x1114=((cj4)*(x1097));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x1110)))+(((r00)*(sj0)))+(x1114));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1110)))+(((IkReal(-0.790000000000000))*(x1114)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj4))));
evalcond[2]=((x1113)+(((IkReal(-1.00000000000000))*(r02)*(x1102)))+(((x1105)*(x1106)))+(((x1105)*(x1111)))+(((IkReal(-1.00000000000000))*(cj2)*(x1100)*(x1111)))+(((IkReal(-1.00000000000000))*(r01)*(x1100)*(x1101)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1100))));
evalcond[3]=((((r02)*(x1105)))+(((x1109)*(x1111)))+(((x1102)*(x1111)))+(((r01)*(sj1)*(x1101)))+(((IkReal(-1.00000000000000))*(x1098)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1100)))+(((x1106)*(x1109))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(x1100)*(x1103)))+(((IkReal(-1.00000000000000))*(py)*(x1100)*(x1101)))+(((x1104)*(x1107)))+(((IkReal(-0.790000000000000))*(x1113)))+(((IkReal(-1.00000000000000))*(x1099)*(x1100)))+(((IkReal(-0.0750000000000000))*(x1105)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x1108)))+(((x1103)*(x1105)))+(((IkReal(-1.00000000000000))*(pz)*(x1102))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1102)*(x1103)))+(((sj1)*(x1099)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1101)*(x1104)))+(((IkReal(0.790000000000000))*(x1098)))+(((IkReal(-1.00000000000000))*(sj2)*(x1108)))+(((IkReal(-0.0750000000000000))*(x1102)))+(((cj1)*(py)*(x1107)))+(((x1103)*(x1109)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1100))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1115=((cj1)*(cj2));
IkReal x1116=((IkReal(1.00000000000000))*(cj0));
IkReal x1117=((sj1)*(sj2));
IkReal x1118=((IkReal(1.00000000000000))*(sj0));
IkReal x1119=((sj0)*(x1117));
IkReal x1120=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x1121=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((cj0)*(r00)*(x1117)))+(((r01)*(x1119)))+(((IkReal(-1.00000000000000))*(r00)*(x1115)*(x1116)))+(((IkReal(-1.00000000000000))*(r01)*(x1115)*(x1118)))+(((IkReal(-1.00000000000000))*(r02)*(x1120)))+(((IkReal(-1.00000000000000))*(r02)*(x1121))));
evalcond[2]=((IkReal(0.485000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x1115)*(x1118)))+(((IkReal(-1.00000000000000))*(pz)*(x1120)))+(((IkReal(-1.00000000000000))*(pz)*(x1121)))+(((py)*(x1119)))+(((IkReal(-0.0750000000000000))*(x1117)))+(((IkReal(-1.00000000000000))*(px)*(x1115)*(x1116)))+(((cj0)*(px)*(x1117)))+(((IkReal(0.0750000000000000))*(x1115))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1122=((cj2)*(cj3));
IkReal x1123=((cj1)*(r02));
IkReal x1124=((cj3)*(sj2));
IkReal x1125=((r02)*(sj1));
IkReal x1126=((cj2)*(sj3));
IkReal x1127=((cj0)*(r00));
IkReal x1128=((r01)*(sj0));
IkReal x1129=((r00)*(sj0));
IkReal x1130=((sj1)*(x1127));
IkReal x1131=((IkReal(1.00000000000000))*(x1128));
IkReal x1132=((cj1)*(sj2)*(sj3));
IkReal x1133=((IkReal(1.00000000000000))*(cj0)*(r01));
if( IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(x1131)*(x1132)))+(((IkReal(-1.00000000000000))*(sj1)*(x1126)*(x1131)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1125)))+(((IkReal(-1.00000000000000))*(x1127)*(x1132)))+(((IkReal(-1.00000000000000))*(x1126)*(x1130)))+(((IkReal(-1.00000000000000))*(cj3)*(x1133)))+(((x1123)*(x1126)))+(((cj3)*(x1129))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(sj3)*(x1133)))+(((sj1)*(x1122)*(x1128)))+(((x1124)*(x1125)))+(((cj1)*(x1124)*(x1127)))+(((cj1)*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(x1122)*(x1123)))+(((sj3)*(x1129)))+(((x1122)*(x1130))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst59)*(((((IkReal(-1.00000000000000))*(x1131)*(x1132)))+(((IkReal(-1.00000000000000))*(sj1)*(x1126)*(x1131)))+(((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1125)))+(((IkReal(-1.00000000000000))*(x1127)*(x1132)))+(((IkReal(-1.00000000000000))*(x1126)*(x1130)))+(((IkReal(-1.00000000000000))*(cj3)*(x1133)))+(((x1123)*(x1126)))+(((cj3)*(x1129)))))), ((gconst59)*(((((IkReal(-1.00000000000000))*(sj3)*(x1133)))+(((sj1)*(x1122)*(x1128)))+(((x1124)*(x1125)))+(((cj1)*(x1124)*(x1127)))+(((cj1)*(x1124)*(x1128)))+(((IkReal(-1.00000000000000))*(x1122)*(x1123)))+(((sj3)*(x1129)))+(((x1122)*(x1130)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1134=IKsin(j5);
IkReal x1135=IKcos(j5);
IkReal x1136=((sj1)*(sj2));
IkReal x1137=((cj2)*(sj1));
IkReal x1138=((py)*(sj0));
IkReal x1139=((IkReal(1.00000000000000))*(cj0));
IkReal x1140=((r01)*(sj0));
IkReal x1141=((cj1)*(sj2));
IkReal x1142=((cj0)*(x1141));
IkReal x1143=((sj3)*(x1134));
IkReal x1144=((IkReal(1.00000000000000))*(x1135));
IkReal x1145=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1146=((IkReal(0.790000000000000))*(x1135));
IkReal x1147=((cj3)*(x1134));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1147)))+(((IkReal(-1.00000000000000))*(r01)*(x1139)))+(((IkReal(-1.00000000000000))*(sj3)*(x1144))));
evalcond[1]=((((sj3)*(x1146)))+(((IkReal(-1.00000000000000))*(py)*(x1139)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x1147))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj3)*(x1144)))+(((x1140)*(x1141)))+(((x1137)*(x1140)))+(x1143)+(((cj0)*(r00)*(x1137)))+(((IkReal(-1.00000000000000))*(r02)*(x1145)))+(((r02)*(x1136)))+(((r00)*(x1142))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x1141)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj3)*(x1146)))+(((x1137)*(x1138)))+(((pz)*(x1136)))+(((IkReal(-0.0750000000000000))*(x1137)))+(((px)*(x1142)))+(((IkReal(-1.00000000000000))*(pz)*(x1145)))+(((IkReal(-0.790000000000000))*(x1143)))+(((x1138)*(x1141)))+(((cj0)*(px)*(x1137))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x1148=((cj1)*(cj2));
IkReal x1149=((IkReal(1.00000000000000))*(cj0));
IkReal x1150=((sj1)*(sj2));
IkReal x1151=((IkReal(1.00000000000000))*(sj0));
IkReal x1152=((sj0)*(x1150));
IkReal x1153=((IkReal(1.00000000000000))*(cj1)*(sj2));
IkReal x1154=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x1148)*(x1149)))+(((IkReal(-1.00000000000000))*(r01)*(x1148)*(x1151)))+(((IkReal(-1.00000000000000))*(r02)*(x1154)))+(((IkReal(-1.00000000000000))*(r02)*(x1153)))+(((cj0)*(r00)*(x1150)))+(((r01)*(x1152))));
evalcond[2]=((IkReal(0.325000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x1150)))+(((cj0)*(px)*(x1150)))+(((IkReal(-1.00000000000000))*(py)*(x1148)*(x1151)))+(((IkReal(0.0750000000000000))*(x1148)))+(((IkReal(-1.00000000000000))*(pz)*(x1153)))+(((IkReal(-1.00000000000000))*(pz)*(x1154)))+(((IkReal(-1.00000000000000))*(px)*(x1148)*(x1149)))+(((py)*(x1152))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign((((sj3)*(sj3))+((cj3)*(cj3))));
dummyeval[0]=(((sj3)*(sj3))+((cj3)*(cj3)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1155=((r01)*(sj0));
IkReal x1156=((r02)*(sj1));
IkReal x1157=((cj2)*(cj3));
IkReal x1158=((IkReal(1.00000000000000))*(sj1));
IkReal x1159=((cj1)*(r02));
IkReal x1160=((cj2)*(sj3));
IkReal x1161=((cj0)*(r00));
IkReal x1162=((cj0)*(r01));
IkReal x1163=((r00)*(sj0));
IkReal x1164=((IkReal(1.00000000000000))*(cj3)*(sj2));
IkReal x1165=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(x1156)*(x1165)))+(((IkReal(-1.00000000000000))*(cj1)*(x1155)*(x1165)))+(((x1159)*(x1160)))+(((IkReal(-1.00000000000000))*(cj3)*(x1162)))+(((IkReal(-1.00000000000000))*(x1158)*(x1160)*(x1161)))+(((IkReal(-1.00000000000000))*(cj1)*(x1161)*(x1165)))+(((cj3)*(x1163)))+(((IkReal(-1.00000000000000))*(x1155)*(x1158)*(x1160))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(x1156)*(x1164)))+(((IkReal(-1.00000000000000))*(cj1)*(x1155)*(x1164)))+(((IkReal(-1.00000000000000))*(x1155)*(x1157)*(x1158)))+(((IkReal(-1.00000000000000))*(sj3)*(x1163)))+(((IkReal(-1.00000000000000))*(cj1)*(x1161)*(x1164)))+(((IkReal(-1.00000000000000))*(x1157)*(x1158)*(x1161)))+(((x1157)*(x1159)))+(((sj3)*(x1162))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst60)*(((((IkReal(-1.00000000000000))*(x1156)*(x1165)))+(((IkReal(-1.00000000000000))*(cj1)*(x1155)*(x1165)))+(((x1159)*(x1160)))+(((IkReal(-1.00000000000000))*(cj3)*(x1162)))+(((IkReal(-1.00000000000000))*(x1158)*(x1160)*(x1161)))+(((IkReal(-1.00000000000000))*(cj1)*(x1161)*(x1165)))+(((cj3)*(x1163)))+(((IkReal(-1.00000000000000))*(x1155)*(x1158)*(x1160)))))), ((gconst60)*(((((IkReal(-1.00000000000000))*(x1156)*(x1164)))+(((IkReal(-1.00000000000000))*(cj1)*(x1155)*(x1164)))+(((IkReal(-1.00000000000000))*(x1155)*(x1157)*(x1158)))+(((IkReal(-1.00000000000000))*(sj3)*(x1163)))+(((IkReal(-1.00000000000000))*(cj1)*(x1161)*(x1164)))+(((IkReal(-1.00000000000000))*(x1157)*(x1158)*(x1161)))+(((x1157)*(x1159)))+(((sj3)*(x1162)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x1166=IKsin(j5);
IkReal x1167=IKcos(j5);
IkReal x1168=((sj1)*(sj2));
IkReal x1169=((cj2)*(sj1));
IkReal x1170=((py)*(sj0));
IkReal x1171=((IkReal(1.00000000000000))*(cj0));
IkReal x1172=((r01)*(sj0));
IkReal x1173=((cj1)*(sj2));
IkReal x1174=((cj0)*(x1173));
IkReal x1175=((sj3)*(x1166));
IkReal x1176=((cj3)*(x1167));
IkReal x1177=((IkReal(1.00000000000000))*(cj1)*(cj2));
IkReal x1178=((sj3)*(x1167));
IkReal x1179=((cj3)*(x1166));
evalcond[0]=((((r00)*(sj0)))+(x1178)+(((IkReal(-1.00000000000000))*(r01)*(x1171)))+(((IkReal(-1.00000000000000))*(x1179))));
evalcond[1]=((((IkReal(0.790000000000000))*(x1179)))+(((IkReal(-1.00000000000000))*(py)*(x1171)))+(((IkReal(-0.790000000000000))*(x1178)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x1177)))+(x1176)+(x1175)+(((r02)*(x1168)))+(((r00)*(x1174)))+(((x1172)*(x1173)))+(((cj0)*(r00)*(x1169)))+(((x1169)*(x1172))));
evalcond[3]=((IkReal(-0.0900000000000000))+(((px)*(x1174)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1177)))+(((IkReal(-0.790000000000000))*(x1176)))+(((IkReal(-0.790000000000000))*(x1175)))+(((pz)*(x1168)))+(((IkReal(-0.0750000000000000))*(x1173)))+(((cj0)*(px)*(x1169)))+(((IkReal(-0.0750000000000000))*(x1169)))+(((x1170)*(x1173)))+(((x1169)*(x1170))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1180=((cj1)*(cj2));
IkReal x1181=((sj1)*(sj2));
IkReal x1182=((IkReal(1.26582278481013))*(pz));
IkReal x1183=((IkReal(1.26582278481013))*(py)*(sj0));
IkReal x1184=((IkReal(1.26582278481013))*(cj0)*(px));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.512658227848101))+(((IkReal(0.0949367088607595))*(x1180)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1182)))+(((IkReal(-1.00000000000000))*(x1180)*(x1184)))+(((IkReal(-1.00000000000000))*(x1180)*(x1183)))+(((x1181)*(x1184)))+(((x1181)*(x1183)))+(((IkReal(-0.0949367088607595))*(x1181)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1182))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))+IKsqr(((IkReal(0.512658227848101))+(((IkReal(0.0949367088607595))*(x1180)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1182)))+(((IkReal(-1.00000000000000))*(x1180)*(x1184)))+(((IkReal(-1.00000000000000))*(x1180)*(x1183)))+(((x1181)*(x1184)))+(((x1181)*(x1183)))+(((IkReal(-0.0949367088607595))*(x1181)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1182)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((IkReal(0.512658227848101))+(((IkReal(0.0949367088607595))*(x1180)))+(((IkReal(-0.462025316455696))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1182)))+(((IkReal(-1.00000000000000))*(x1180)*(x1184)))+(((IkReal(-1.00000000000000))*(x1180)*(x1183)))+(((x1181)*(x1184)))+(((x1181)*(x1183)))+(((IkReal(-0.0949367088607595))*(x1181)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1182)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1185=IKsin(j5);
IkReal x1186=IKcos(j5);
IkReal x1187=((pz)*(sj2));
IkReal x1188=((IkReal(1.00000000000000))*(cj1));
IkReal x1189=((cj2)*(sj0));
IkReal x1190=((cj2)*(sj1));
IkReal x1191=((cj0)*(px));
IkReal x1192=((py)*(sj1));
IkReal x1193=((sj1)*(sj2));
IkReal x1194=((r01)*(sj0));
IkReal x1195=((IkReal(0.0750000000000000))*(cj1));
IkReal x1196=((sj0)*(sj2));
IkReal x1197=((cj1)*(sj2));
IkReal x1198=((IkReal(1.00000000000000))*(cj0));
IkReal x1199=((cj0)*(r00));
IkReal x1200=((sj2)*(x1199));
IkReal x1201=((sj3)*(x1185));
IkReal x1202=((cj3)*(x1185));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1202)))+(((IkReal(-1.00000000000000))*(r01)*(x1198))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(x1198)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(x1202))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1188)))+(x1186)+(((IkReal(-1.00000000000000))*(r01)*(x1188)*(x1189)))+(((x1193)*(x1194)))+(((x1193)*(x1199)))+(((IkReal(-1.00000000000000))*(r02)*(x1190)))+(((IkReal(-1.00000000000000))*(cj2)*(x1188)*(x1199))));
evalcond[3]=((x1201)+(((x1197)*(x1199)))+(((x1194)*(x1197)))+(((r01)*(sj1)*(x1189)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1188)))+(((r02)*(x1193)))+(((x1190)*(x1199))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x1193)))+(((IkReal(-0.790000000000000))*(x1186)))+(((IkReal(-1.00000000000000))*(x1187)*(x1188)))+(((IkReal(-1.00000000000000))*(py)*(x1188)*(x1189)))+(((x1192)*(x1196)))+(((IkReal(-1.00000000000000))*(pz)*(x1190)))+(((cj2)*(x1195)))+(((x1191)*(x1193)))+(((IkReal(-1.00000000000000))*(cj2)*(x1188)*(x1191))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(sj2)*(x1195)))+(((cj1)*(py)*(x1196)))+(((IkReal(-0.790000000000000))*(x1201)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-0.0750000000000000))*(x1190)))+(((x1189)*(x1192)))+(((sj1)*(x1187)))+(((IkReal(0.0800000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1188)))+(((x1191)*(x1197)))+(((x1190)*(x1191))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1203=((cj0)*(r00));
IkReal x1204=((cj1)*(cj2));
IkReal x1205=((r01)*(sj0));
IkReal x1206=((IkReal(1.00000000000000))*(sj1)*(sj2));
if( IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj3)))+(((IkReal(100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((x1204)*(x1205)))+(((x1203)*(x1204)))+(((IkReal(-1.00000000000000))*(x1205)*(x1206)))+(((IkReal(-1.00000000000000))*(x1203)*(x1206)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj3)))+(((IkReal(100.000000000000))*(cj0)*(py)))))))+IKsqr(((((x1204)*(x1205)))+(((x1203)*(x1204)))+(((IkReal(-1.00000000000000))*(x1205)*(x1206)))+(((IkReal(-1.00000000000000))*(x1203)*(x1206)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(-8.00000000000000))*(sj3)))+(((IkReal(100.000000000000))*(cj0)*(py)))))), ((((x1204)*(x1205)))+(((x1203)*(x1204)))+(((IkReal(-1.00000000000000))*(x1205)*(x1206)))+(((IkReal(-1.00000000000000))*(x1203)*(x1206)))+(((cj2)*(r02)*(sj1)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1207=IKsin(j5);
IkReal x1208=IKcos(j5);
IkReal x1209=((pz)*(sj2));
IkReal x1210=((IkReal(1.00000000000000))*(cj1));
IkReal x1211=((cj2)*(sj0));
IkReal x1212=((cj2)*(sj1));
IkReal x1213=((cj0)*(px));
IkReal x1214=((py)*(sj1));
IkReal x1215=((sj1)*(sj2));
IkReal x1216=((r01)*(sj0));
IkReal x1217=((IkReal(0.0750000000000000))*(cj1));
IkReal x1218=((sj0)*(sj2));
IkReal x1219=((cj1)*(sj2));
IkReal x1220=((IkReal(1.00000000000000))*(cj0));
IkReal x1221=((cj0)*(r00));
IkReal x1222=((sj2)*(x1221));
IkReal x1223=((sj3)*(x1207));
IkReal x1224=((cj3)*(x1207));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1220)))+(((IkReal(-1.00000000000000))*(x1224))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((IkReal(0.790000000000000))*(x1224)))+(((IkReal(-1.00000000000000))*(py)*(x1220)))+(((px)*(sj0))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1210)*(x1211)))+(x1208)+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1210)))+(((IkReal(-1.00000000000000))*(r02)*(x1212)))+(((x1215)*(x1221)))+(((IkReal(-1.00000000000000))*(cj2)*(x1210)*(x1221)))+(((x1215)*(x1216))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1210)))+(x1223)+(((r02)*(x1215)))+(((x1219)*(x1221)))+(((r01)*(sj1)*(x1211)))+(((x1216)*(x1219)))+(((x1212)*(x1221))));
evalcond[4]=((IkReal(0.405000000000000))+(((x1214)*(x1218)))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1209)*(x1210)))+(((IkReal(-1.00000000000000))*(pz)*(x1212)))+(((x1213)*(x1215)))+(((IkReal(-0.790000000000000))*(x1208)))+(((IkReal(-1.00000000000000))*(cj2)*(x1210)*(x1213)))+(((cj2)*(x1217)))+(((IkReal(-0.0750000000000000))*(x1215)))+(((IkReal(-1.00000000000000))*(py)*(x1210)*(x1211))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1212)*(x1213)))+(((x1213)*(x1219)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1210)))+(((IkReal(-0.365000000000000))*(cj2)))+(((sj1)*(x1209)))+(((IkReal(-1.00000000000000))*(sj2)*(x1217)))+(((IkReal(-0.790000000000000))*(x1223)))+(((x1211)*(x1214)))+(((IkReal(0.0800000000000000))*(cj3)))+(((IkReal(-0.0750000000000000))*(x1212)))+(((cj1)*(py)*(x1218))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1225=((IkReal(1.00000000000000))*(r01));
IkReal x1226=((cj0)*(r00));
IkReal x1227=((cj1)*(cj2));
IkReal x1228=((sj1)*(sj2));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1225))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x1226)*(x1228)))+(((IkReal(-1.00000000000000))*(sj0)*(x1225)*(x1228)))+(((cj2)*(r02)*(sj1)))+(((x1226)*(x1227)))+(((r01)*(sj0)*(x1227)))+(((cj1)*(r02)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1225)))))))+IKsqr(((((IkReal(-1.00000000000000))*(x1226)*(x1228)))+(((IkReal(-1.00000000000000))*(sj0)*(x1225)*(x1228)))+(((cj2)*(r02)*(sj1)))+(((x1226)*(x1227)))+(((r01)*(sj0)*(x1227)))+(((cj1)*(r02)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(x1225)))))), ((((IkReal(-1.00000000000000))*(x1226)*(x1228)))+(((IkReal(-1.00000000000000))*(sj0)*(x1225)*(x1228)))+(((cj2)*(r02)*(sj1)))+(((x1226)*(x1227)))+(((r01)*(sj0)*(x1227)))+(((cj1)*(r02)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1229=IKsin(j5);
IkReal x1230=IKcos(j5);
IkReal x1231=((pz)*(sj2));
IkReal x1232=((IkReal(1.00000000000000))*(cj1));
IkReal x1233=((cj2)*(sj0));
IkReal x1234=((cj2)*(sj1));
IkReal x1235=((cj0)*(px));
IkReal x1236=((py)*(sj1));
IkReal x1237=((sj1)*(sj2));
IkReal x1238=((r01)*(sj0));
IkReal x1239=((IkReal(0.0750000000000000))*(cj1));
IkReal x1240=((sj0)*(sj2));
IkReal x1241=((cj1)*(sj2));
IkReal x1242=((IkReal(1.00000000000000))*(cj0));
IkReal x1243=((cj0)*(r00));
IkReal x1244=((sj2)*(x1243));
IkReal x1245=((sj3)*(x1229));
IkReal x1246=((cj3)*(x1229));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1246)))+(((IkReal(-1.00000000000000))*(r01)*(x1242))));
evalcond[1]=((((IkReal(0.0800000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(x1242)))+(((IkReal(0.790000000000000))*(x1246)))+(((px)*(sj0))));
evalcond[2]=((x1230)+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1232)))+(((IkReal(-1.00000000000000))*(r01)*(x1232)*(x1233)))+(((IkReal(-1.00000000000000))*(cj2)*(x1232)*(x1243)))+(((IkReal(-1.00000000000000))*(r02)*(x1234)))+(((x1237)*(x1238)))+(((x1237)*(x1243))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1232)))+(x1245)+(((r02)*(x1237)))+(((r01)*(sj1)*(x1233)))+(((x1238)*(x1241)))+(((x1241)*(x1243)))+(((x1234)*(x1243))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x1237)))+(((cj2)*(x1239)))+(((x1236)*(x1240)))+(((x1235)*(x1237)))+(((IkReal(-1.00000000000000))*(py)*(x1232)*(x1233)))+(((IkReal(-1.00000000000000))*(pz)*(x1234)))+(((IkReal(-1.00000000000000))*(cj2)*(x1232)*(x1235)))+(((IkReal(-0.790000000000000))*(x1230)))+(((IkReal(-1.00000000000000))*(x1231)*(x1232))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((sj1)*(x1231)))+(((IkReal(-0.0750000000000000))*(x1234)))+(((x1234)*(x1235)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1233)*(x1236)))+(((IkReal(-1.00000000000000))*(sj2)*(x1239)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1232)))+(((IkReal(0.0800000000000000))*(cj3)))+(((x1235)*(x1241)))+(((cj1)*(py)*(x1240)))+(((IkReal(-0.790000000000000))*(x1245))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1247=((cj1)*(cj2));
IkReal x1248=((sj1)*(sj2));
IkReal x1249=((IkReal(1.26582278481013))*(pz));
IkReal x1250=((IkReal(1.26582278481013))*(py)*(sj0));
IkReal x1251=((IkReal(1.26582278481013))*(cj0)*(px));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.512658227848101))+(((x1247)*(x1251)))+(((x1247)*(x1250)))+(((cj2)*(sj1)*(x1249)))+(((IkReal(-0.0949367088607595))*(x1247)))+(((IkReal(0.0949367088607595))*(x1248)))+(((IkReal(-1.00000000000000))*(x1248)*(x1251)))+(((IkReal(-1.00000000000000))*(x1248)*(x1250)))+(((cj1)*(sj2)*(x1249)))+(((IkReal(0.462025316455696))*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))))+IKsqr(((IkReal(-0.512658227848101))+(((x1247)*(x1251)))+(((x1247)*(x1250)))+(((cj2)*(sj1)*(x1249)))+(((IkReal(-0.0949367088607595))*(x1247)))+(((IkReal(0.0949367088607595))*(x1248)))+(((IkReal(-1.00000000000000))*(x1248)*(x1251)))+(((IkReal(-1.00000000000000))*(x1248)*(x1250)))+(((cj1)*(sj2)*(x1249)))+(((IkReal(0.462025316455696))*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)))))), ((IkReal(-0.512658227848101))+(((x1247)*(x1251)))+(((x1247)*(x1250)))+(((cj2)*(sj1)*(x1249)))+(((IkReal(-0.0949367088607595))*(x1247)))+(((IkReal(0.0949367088607595))*(x1248)))+(((IkReal(-1.00000000000000))*(x1248)*(x1251)))+(((IkReal(-1.00000000000000))*(x1248)*(x1250)))+(((cj1)*(sj2)*(x1249)))+(((IkReal(0.462025316455696))*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1252=IKsin(j5);
IkReal x1253=IKcos(j5);
IkReal x1254=((pz)*(sj2));
IkReal x1255=((IkReal(1.00000000000000))*(cj1));
IkReal x1256=((cj2)*(sj0));
IkReal x1257=((cj2)*(sj1));
IkReal x1258=((cj0)*(px));
IkReal x1259=((py)*(sj1));
IkReal x1260=((sj1)*(sj2));
IkReal x1261=((r01)*(sj0));
IkReal x1262=((IkReal(0.0750000000000000))*(cj1));
IkReal x1263=((sj0)*(sj2));
IkReal x1264=((cj1)*(sj2));
IkReal x1265=((IkReal(1.00000000000000))*(cj0));
IkReal x1266=((cj0)*(r00));
IkReal x1267=((sj2)*(x1266));
IkReal x1268=((sj3)*(x1252));
IkReal x1269=((cj3)*(x1252));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1265)))+(((IkReal(-1.00000000000000))*(x1269))));
evalcond[1]=((((IkReal(0.790000000000000))*(x1269)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(x1265))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x1255)*(x1256)))+(((IkReal(-1.00000000000000))*(r02)*(x1257)))+(((IkReal(-1.00000000000000))*(cj2)*(x1255)*(x1266)))+(((x1260)*(x1266)))+(((x1260)*(x1261)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1255)))+(((IkReal(-1.00000000000000))*(x1253))));
evalcond[3]=((x1268)+(((x1261)*(x1264)))+(((x1264)*(x1266)))+(((r02)*(x1260)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1255)))+(((r01)*(sj1)*(x1256)))+(((x1257)*(x1266))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x1258)*(x1260)))+(((IkReal(-1.00000000000000))*(pz)*(x1257)))+(((IkReal(-1.00000000000000))*(py)*(x1255)*(x1256)))+(((cj2)*(x1262)))+(((IkReal(-1.00000000000000))*(cj2)*(x1255)*(x1258)))+(((x1259)*(x1263)))+(((IkReal(-1.00000000000000))*(x1254)*(x1255)))+(((IkReal(0.790000000000000))*(x1253)))+(((IkReal(-0.0750000000000000))*(x1260))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1258)*(x1264)))+(((cj1)*(py)*(x1263)))+(((IkReal(-0.365000000000000))*(cj2)))+(((sj1)*(x1254)))+(((x1256)*(x1259)))+(((IkReal(-1.00000000000000))*(sj2)*(x1262)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1255)))+(((IkReal(-0.790000000000000))*(x1268)))+(((x1257)*(x1258)))+(((IkReal(-0.0800000000000000))*(cj3)))+(((IkReal(-0.0750000000000000))*(x1257))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1270=((r01)*(sj0));
IkReal x1271=((sj1)*(sj2));
IkReal x1272=((IkReal(1.00000000000000))*(cj1));
IkReal x1273=((cj0)*(r00));
if( IKabs(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(8.00000000000000))*(sj3)))+(((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(100.000000000000))*(cj0)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj2)*(x1270)*(x1272)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x1270)*(x1271)))+(((x1271)*(x1273)))+(((IkReal(-1.00000000000000))*(cj2)*(x1272)*(x1273)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1272))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(8.00000000000000))*(sj3)))+(((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(100.000000000000))*(cj0)*(py)))))))+IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(x1270)*(x1272)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x1270)*(x1271)))+(((x1271)*(x1273)))+(((IkReal(-1.00000000000000))*(cj2)*(x1272)*(x1273)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1272)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0126582278481013))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(8.00000000000000))*(sj3)))+(((IkReal(-100.000000000000))*(px)*(sj0)))+(((IkReal(100.000000000000))*(cj0)*(py)))))), ((((IkReal(-1.00000000000000))*(cj2)*(x1270)*(x1272)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(sj1)))+(((x1270)*(x1271)))+(((x1271)*(x1273)))+(((IkReal(-1.00000000000000))*(cj2)*(x1272)*(x1273)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1272)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1274=IKsin(j5);
IkReal x1275=IKcos(j5);
IkReal x1276=((pz)*(sj2));
IkReal x1277=((IkReal(1.00000000000000))*(cj1));
IkReal x1278=((cj2)*(sj0));
IkReal x1279=((cj2)*(sj1));
IkReal x1280=((cj0)*(px));
IkReal x1281=((py)*(sj1));
IkReal x1282=((sj1)*(sj2));
IkReal x1283=((r01)*(sj0));
IkReal x1284=((IkReal(0.0750000000000000))*(cj1));
IkReal x1285=((sj0)*(sj2));
IkReal x1286=((cj1)*(sj2));
IkReal x1287=((IkReal(1.00000000000000))*(cj0));
IkReal x1288=((cj0)*(r00));
IkReal x1289=((sj2)*(x1288));
IkReal x1290=((sj3)*(x1274));
IkReal x1291=((cj3)*(x1274));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1287)))+(((IkReal(-1.00000000000000))*(x1291))));
evalcond[1]=((((IkReal(0.790000000000000))*(x1291)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(x1287))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(x1277)*(x1288)))+(((IkReal(-1.00000000000000))*(r02)*(x1279)))+(((IkReal(-1.00000000000000))*(r01)*(x1277)*(x1278)))+(((IkReal(-1.00000000000000))*(x1275)))+(((x1282)*(x1283)))+(((x1282)*(x1288)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1277))));
evalcond[3]=((((x1279)*(x1288)))+(x1290)+(((r02)*(x1282)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1277)))+(((r01)*(sj1)*(x1278)))+(((x1283)*(x1286)))+(((x1286)*(x1288))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(pz)*(x1279)))+(((IkReal(-1.00000000000000))*(x1276)*(x1277)))+(((IkReal(-1.00000000000000))*(cj2)*(x1277)*(x1280)))+(((cj2)*(x1284)))+(((IkReal(-0.0750000000000000))*(x1282)))+(((x1280)*(x1282)))+(((IkReal(-1.00000000000000))*(py)*(x1277)*(x1278)))+(((x1281)*(x1285)))+(((IkReal(0.790000000000000))*(x1275))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1279)*(x1280)))+(((x1278)*(x1281)))+(((sj1)*(x1276)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1277)))+(((IkReal(-0.790000000000000))*(x1290)))+(((x1280)*(x1286)))+(((IkReal(-1.00000000000000))*(sj2)*(x1284)))+(((IkReal(-0.0800000000000000))*(cj3)))+(((IkReal(-0.0750000000000000))*(x1279)))+(((cj1)*(py)*(x1285))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1292=((r01)*(sj0));
IkReal x1293=((sj1)*(sj2));
IkReal x1294=((IkReal(1.00000000000000))*(cj0));
IkReal x1295=((cj1)*(cj2));
IkReal x1296=((IkReal(1.00000000000000))*(r02));
if( IKabs(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r01)*(x1294)))+(((r00)*(sj0))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r00)*(x1294)*(x1295)))+(((cj0)*(r00)*(x1293)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1296)))+(((IkReal(-1.00000000000000))*(x1292)*(x1295)))+(((x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1296))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r01)*(x1294)))+(((r00)*(sj0)))))))+IKsqr(((((IkReal(-1.00000000000000))*(r00)*(x1294)*(x1295)))+(((cj0)*(r00)*(x1293)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1296)))+(((IkReal(-1.00000000000000))*(x1292)*(x1295)))+(((x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1296)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(r01)*(x1294)))+(((r00)*(sj0)))))), ((((IkReal(-1.00000000000000))*(r00)*(x1294)*(x1295)))+(((cj0)*(r00)*(x1293)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x1296)))+(((IkReal(-1.00000000000000))*(x1292)*(x1295)))+(((x1292)*(x1293)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x1296)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1297=IKsin(j5);
IkReal x1298=IKcos(j5);
IkReal x1299=((pz)*(sj2));
IkReal x1300=((IkReal(1.00000000000000))*(cj1));
IkReal x1301=((cj2)*(sj0));
IkReal x1302=((cj2)*(sj1));
IkReal x1303=((cj0)*(px));
IkReal x1304=((py)*(sj1));
IkReal x1305=((sj1)*(sj2));
IkReal x1306=((r01)*(sj0));
IkReal x1307=((IkReal(0.0750000000000000))*(cj1));
IkReal x1308=((sj0)*(sj2));
IkReal x1309=((cj1)*(sj2));
IkReal x1310=((IkReal(1.00000000000000))*(cj0));
IkReal x1311=((cj0)*(r00));
IkReal x1312=((sj2)*(x1311));
IkReal x1313=((sj3)*(x1297));
IkReal x1314=((cj3)*(x1297));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(x1314)))+(((IkReal(-1.00000000000000))*(r01)*(x1310))));
evalcond[1]=((((IkReal(0.790000000000000))*(x1314)))+(((IkReal(-1.00000000000000))*(py)*(x1310)))+(((px)*(sj0)))+(((IkReal(-0.0800000000000000))*(sj3))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1300)))+(((IkReal(-1.00000000000000))*(r02)*(x1302)))+(((x1305)*(x1306)))+(((x1305)*(x1311)))+(((IkReal(-1.00000000000000))*(cj2)*(x1300)*(x1311)))+(((IkReal(-1.00000000000000))*(x1298)))+(((IkReal(-1.00000000000000))*(r01)*(x1300)*(x1301))));
evalcond[3]=((x1313)+(((x1306)*(x1309)))+(((r01)*(sj1)*(x1301)))+(((x1309)*(x1311)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1300)))+(((x1302)*(x1311)))+(((r02)*(x1305))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1299)*(x1300)))+(((IkReal(-1.00000000000000))*(pz)*(x1302)))+(((IkReal(-1.00000000000000))*(cj2)*(x1300)*(x1303)))+(((IkReal(0.790000000000000))*(x1298)))+(((x1303)*(x1305)))+(((IkReal(-0.0750000000000000))*(x1305)))+(((x1304)*(x1308)))+(((cj2)*(x1307)))+(((IkReal(-1.00000000000000))*(py)*(x1300)*(x1301))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((cj1)*(py)*(x1308)))+(((sj1)*(x1299)))+(((IkReal(-0.365000000000000))*(cj2)))+(((x1303)*(x1309)))+(((IkReal(-0.0750000000000000))*(x1302)))+(((x1302)*(x1303)))+(((IkReal(-0.790000000000000))*(x1313)))+(((IkReal(-1.00000000000000))*(sj2)*(x1307)))+(((IkReal(-0.0800000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1300)))+(((x1301)*(x1304))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1315=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x1316=((r01)*(sj0));
IkReal x1317=((r02)*(sj1));
IkReal x1318=((IkReal(1.00000000000000))*(sj2));
IkReal x1319=((cj4)*(sj3));
IkReal x1320=((cj1)*(r02));
IkReal x1321=((sj1)*(sj2));
IkReal x1322=((cj0)*(r00));
IkReal x1323=((cj1)*(x1322));
IkReal x1324=((IkReal(1.00000000000000))*(cj2)*(x1319));
if( IKabs(((x1315)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1323)*(x1324)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((x1316)*(x1319)*(x1321)))+(((IkReal(-1.00000000000000))*(x1318)*(x1319)*(x1320)))+(((x1319)*(x1321)*(x1322)))+(((IkReal(-1.00000000000000))*(cj1)*(x1316)*(x1324)))+(((IkReal(-1.00000000000000))*(x1317)*(x1324))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x1315)*(((((sj2)*(x1320)))+(((cj1)*(cj2)*(x1316)))+(((IkReal(-1.00000000000000))*(sj1)*(x1318)*(x1322)))+(((IkReal(-1.00000000000000))*(sj1)*(x1316)*(x1318)))+(((cj2)*(x1323)))+(((cj2)*(x1317))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x1315)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1323)*(x1324)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((x1316)*(x1319)*(x1321)))+(((IkReal(-1.00000000000000))*(x1318)*(x1319)*(x1320)))+(((x1319)*(x1321)*(x1322)))+(((IkReal(-1.00000000000000))*(cj1)*(x1316)*(x1324)))+(((IkReal(-1.00000000000000))*(x1317)*(x1324)))))))+IKsqr(((x1315)*(((((sj2)*(x1320)))+(((cj1)*(cj2)*(x1316)))+(((IkReal(-1.00000000000000))*(sj1)*(x1318)*(x1322)))+(((IkReal(-1.00000000000000))*(sj1)*(x1316)*(x1318)))+(((cj2)*(x1323)))+(((cj2)*(x1317)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x1315)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((r00)*(sj0)*(sj4)))+(((IkReal(-1.00000000000000))*(x1323)*(x1324)))+(((IkReal(-1.00000000000000))*(cj0)*(r01)*(sj4)))+(((x1316)*(x1319)*(x1321)))+(((IkReal(-1.00000000000000))*(x1318)*(x1319)*(x1320)))+(((x1319)*(x1321)*(x1322)))+(((IkReal(-1.00000000000000))*(cj1)*(x1316)*(x1324)))+(((IkReal(-1.00000000000000))*(x1317)*(x1324)))))), ((x1315)*(((((sj2)*(x1320)))+(((cj1)*(cj2)*(x1316)))+(((IkReal(-1.00000000000000))*(sj1)*(x1318)*(x1322)))+(((IkReal(-1.00000000000000))*(sj1)*(x1316)*(x1318)))+(((cj2)*(x1323)))+(((cj2)*(x1317)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1325=IKcos(j5);
IkReal x1326=IKsin(j5);
IkReal x1327=((pz)*(sj2));
IkReal x1328=((IkReal(1.00000000000000))*(cj1));
IkReal x1329=((cj2)*(sj0));
IkReal x1330=((cj2)*(sj1));
IkReal x1331=((cj0)*(px));
IkReal x1332=((py)*(sj1));
IkReal x1333=((sj1)*(sj2));
IkReal x1334=((r01)*(sj0));
IkReal x1335=((IkReal(0.0750000000000000))*(cj1));
IkReal x1336=((sj0)*(sj2));
IkReal x1337=((IkReal(0.0800000000000000))*(sj4));
IkReal x1338=((cj1)*(sj2));
IkReal x1339=((IkReal(0.790000000000000))*(cj3));
IkReal x1340=((IkReal(1.00000000000000))*(cj0));
IkReal x1341=((IkReal(1.00000000000000))*(cj3));
IkReal x1342=((cj0)*(r00));
IkReal x1343=((sj2)*(x1342));
IkReal x1344=((sj4)*(x1325));
IkReal x1345=((sj3)*(x1326));
IkReal x1346=((cj4)*(x1325));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1346)))+(((IkReal(-1.00000000000000))*(r01)*(x1340)))+(((IkReal(-1.00000000000000))*(x1326)*(x1341))));
evalcond[1]=((((IkReal(-1.00000000000000))*(py)*(x1340)))+(((sj3)*(x1337)))+(((x1326)*(x1339)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(sj3)*(x1346))));
evalcond[2]=((x1344)+(((x1333)*(x1342)))+(((IkReal(-1.00000000000000))*(cj2)*(x1328)*(x1342)))+(((IkReal(-1.00000000000000))*(r01)*(x1328)*(x1329)))+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1328)))+(((x1333)*(x1334)))+(((IkReal(-1.00000000000000))*(r02)*(x1330))));
evalcond[3]=((x1345)+(((IkReal(-1.00000000000000))*(x1341)*(x1346)))+(((x1330)*(x1342)))+(((r01)*(sj1)*(x1329)))+(((x1338)*(x1342)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1328)))+(((r02)*(x1333)))+(((x1334)*(x1338))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((x1331)*(x1333)))+(((IkReal(-0.790000000000000))*(x1344)))+(((IkReal(-1.00000000000000))*(x1327)*(x1328)))+(((IkReal(-1.00000000000000))*(cj2)*(x1328)*(x1331)))+(((IkReal(-1.00000000000000))*(py)*(x1328)*(x1329)))+(((cj2)*(x1335)))+(((IkReal(-1.00000000000000))*(pz)*(x1330)))+(((IkReal(-0.0750000000000000))*(x1333)))+(((IkReal(0.0800000000000000))*(cj4)))+(((x1332)*(x1336))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1328)))+(((x1329)*(x1332)))+(((x1331)*(x1338)))+(((x1339)*(x1346)))+(((IkReal(-0.790000000000000))*(x1345)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj3)*(x1337)))+(((cj1)*(py)*(x1336)))+(((IkReal(-0.0750000000000000))*(x1330)))+(((IkReal(-1.00000000000000))*(sj2)*(x1335)))+(((x1330)*(x1331)))+(((sj1)*(x1327))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1347=(sj3)*(sj3);
IkReal x1348=(cj3)*(cj3);
IkReal x1349=((cj3)*(sj0));
IkReal x1350=((IkReal(100.000000000000))*(px));
IkReal x1351=((cj2)*(sj1));
IkReal x1352=((cj1)*(sj2));
IkReal x1353=((IkReal(79.0000000000000))*(r01));
IkReal x1354=((IkReal(8.00000000000000))*(sj4));
IkReal x1355=((IkReal(79.0000000000000))*(cj0)*(r00));
IkReal x1356=((IkReal(79.0000000000000))*(x1348));
IkReal x1357=((IkReal(79.0000000000000))*(x1347));
IkReal x1358=((IkReal(100.000000000000))*(cj0)*(py));
IkReal x1359=((sj0)*(sj3)*(x1353));
IkReal x1360=((IkReal(79.0000000000000))*(r02)*(sj1)*(sj2));
IkReal x1361=((IkReal(79.0000000000000))*(cj1)*(cj2)*(r02));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x1356)))+(((IkReal(-1.00000000000000))*(x1357))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1356)))+(((IkReal(-1.00000000000000))*(x1357)))))):(IkReal)1.0e30))*(((((sj3)*(x1351)*(x1355)))+(((cj3)*(sj3)*(x1354)))+(((IkReal(-1.00000000000000))*(cj3)*(x1358)))+(((IkReal(-1.00000000000000))*(sj3)*(x1361)))+(((sj3)*(x1352)*(x1355)))+(((x1349)*(x1350)))+(((x1351)*(x1359)))+(((sj3)*(x1360)))+(((x1352)*(x1359))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1356)))+(((cj4)*(x1357))))) != 0)?((IkReal)1/(((((cj4)*(x1356)))+(((cj4)*(x1357)))))):(IkReal)1.0e30))*(((((cj3)*(x1352)*(x1355)))+(((IkReal(-1.00000000000000))*(sj0)*(sj3)*(x1350)))+(((x1349)*(x1351)*(x1353)))+(((IkReal(-1.00000000000000))*(cj3)*(x1361)))+(((IkReal(-1.00000000000000))*(x1347)*(x1354)))+(((x1349)*(x1352)*(x1353)))+(((sj3)*(x1358)))+(((cj3)*(x1360)))+(((cj3)*(x1351)*(x1355))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x1356)))+(((IkReal(-1.00000000000000))*(x1357))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1356)))+(((IkReal(-1.00000000000000))*(x1357)))))):(IkReal)1.0e30))*(((((sj3)*(x1351)*(x1355)))+(((cj3)*(sj3)*(x1354)))+(((IkReal(-1.00000000000000))*(cj3)*(x1358)))+(((IkReal(-1.00000000000000))*(sj3)*(x1361)))+(((sj3)*(x1352)*(x1355)))+(((x1349)*(x1350)))+(((x1351)*(x1359)))+(((sj3)*(x1360)))+(((x1352)*(x1359)))))))+IKsqr(((((IKabs(((((cj4)*(x1356)))+(((cj4)*(x1357))))) != 0)?((IkReal)1/(((((cj4)*(x1356)))+(((cj4)*(x1357)))))):(IkReal)1.0e30))*(((((cj3)*(x1352)*(x1355)))+(((IkReal(-1.00000000000000))*(sj0)*(sj3)*(x1350)))+(((x1349)*(x1351)*(x1353)))+(((IkReal(-1.00000000000000))*(cj3)*(x1361)))+(((IkReal(-1.00000000000000))*(x1347)*(x1354)))+(((x1349)*(x1352)*(x1353)))+(((sj3)*(x1358)))+(((cj3)*(x1360)))+(((cj3)*(x1351)*(x1355)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x1356)))+(((IkReal(-1.00000000000000))*(x1357))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1356)))+(((IkReal(-1.00000000000000))*(x1357)))))):(IkReal)1.0e30))*(((((sj3)*(x1351)*(x1355)))+(((cj3)*(sj3)*(x1354)))+(((IkReal(-1.00000000000000))*(cj3)*(x1358)))+(((IkReal(-1.00000000000000))*(sj3)*(x1361)))+(((sj3)*(x1352)*(x1355)))+(((x1349)*(x1350)))+(((x1351)*(x1359)))+(((sj3)*(x1360)))+(((x1352)*(x1359)))))), ((((IKabs(((((cj4)*(x1356)))+(((cj4)*(x1357))))) != 0)?((IkReal)1/(((((cj4)*(x1356)))+(((cj4)*(x1357)))))):(IkReal)1.0e30))*(((((cj3)*(x1352)*(x1355)))+(((IkReal(-1.00000000000000))*(sj0)*(sj3)*(x1350)))+(((x1349)*(x1351)*(x1353)))+(((IkReal(-1.00000000000000))*(cj3)*(x1361)))+(((IkReal(-1.00000000000000))*(x1347)*(x1354)))+(((x1349)*(x1352)*(x1353)))+(((sj3)*(x1358)))+(((cj3)*(x1360)))+(((cj3)*(x1351)*(x1355)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1362=IKcos(j5);
IkReal x1363=IKsin(j5);
IkReal x1364=((pz)*(sj2));
IkReal x1365=((IkReal(1.00000000000000))*(cj1));
IkReal x1366=((cj2)*(sj0));
IkReal x1367=((cj2)*(sj1));
IkReal x1368=((cj0)*(px));
IkReal x1369=((py)*(sj1));
IkReal x1370=((sj1)*(sj2));
IkReal x1371=((r01)*(sj0));
IkReal x1372=((IkReal(0.0750000000000000))*(cj1));
IkReal x1373=((sj0)*(sj2));
IkReal x1374=((IkReal(0.0800000000000000))*(sj4));
IkReal x1375=((cj1)*(sj2));
IkReal x1376=((IkReal(0.790000000000000))*(cj3));
IkReal x1377=((IkReal(1.00000000000000))*(cj0));
IkReal x1378=((IkReal(1.00000000000000))*(cj3));
IkReal x1379=((cj0)*(r00));
IkReal x1380=((sj2)*(x1379));
IkReal x1381=((sj4)*(x1362));
IkReal x1382=((sj3)*(x1363));
IkReal x1383=((cj4)*(x1362));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(r01)*(x1377)))+(((IkReal(-1.00000000000000))*(sj3)*(x1383)))+(((IkReal(-1.00000000000000))*(x1363)*(x1378))));
evalcond[1]=((((IkReal(0.790000000000000))*(sj3)*(x1383)))+(((sj3)*(x1374)))+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(py)*(x1377)))+(((x1363)*(x1376))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(x1365)*(x1379)))+(x1381)+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1365)))+(((x1370)*(x1371)))+(((x1370)*(x1379)))+(((IkReal(-1.00000000000000))*(r02)*(x1367)))+(((IkReal(-1.00000000000000))*(r01)*(x1365)*(x1366))));
evalcond[3]=((x1382)+(((r01)*(sj1)*(x1366)))+(((x1375)*(x1379)))+(((x1367)*(x1379)))+(((x1371)*(x1375)))+(((r02)*(x1370)))+(((IkReal(-1.00000000000000))*(x1378)*(x1383)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1365))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x1364)*(x1365)))+(((IkReal(-1.00000000000000))*(pz)*(x1367)))+(((IkReal(-0.790000000000000))*(x1381)))+(((IkReal(-1.00000000000000))*(cj2)*(x1365)*(x1368)))+(((IkReal(-0.0750000000000000))*(x1370)))+(((cj2)*(x1372)))+(((x1368)*(x1370)))+(((x1369)*(x1373)))+(((IkReal(0.0800000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(py)*(x1365)*(x1366))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((x1367)*(x1368)))+(((IkReal(-0.0750000000000000))*(x1367)))+(((x1366)*(x1369)))+(((IkReal(-0.790000000000000))*(x1382)))+(((cj1)*(py)*(x1373)))+(((sj1)*(x1364)))+(((IkReal(-0.365000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x1372)))+(((x1376)*(x1383)))+(((cj3)*(x1374)))+(((x1368)*(x1375)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1365))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1384=(sj3)*(sj3);
IkReal x1385=(cj3)*(cj3);
IkReal x1386=((cj2)*(cj3));
IkReal x1387=((cj1)*(r02));
IkReal x1388=((r01)*(sj0));
IkReal x1389=((cj3)*(sj2));
IkReal x1390=((r02)*(sj1));
IkReal x1391=((cj0)*(r00));
IkReal x1392=((r00)*(sj0));
IkReal x1393=((sj1)*(x1391));
IkReal x1394=((IkReal(1.00000000000000))*(cj2)*(sj3));
IkReal x1395=((IkReal(1.00000000000000))*(cj0)*(r01));
IkReal x1396=((IkReal(1.00000000000000))*(cj1)*(sj2)*(sj3));
if( IKabs(((((IKabs(((x1384)+(x1385))) != 0)?((IkReal)1/(((x1384)+(x1385)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1390)))+(((IkReal(-1.00000000000000))*(x1391)*(x1396)))+(((cj3)*(x1392)))+(((IkReal(-1.00000000000000))*(x1393)*(x1394)))+(((IkReal(-1.00000000000000))*(sj1)*(x1388)*(x1394)))+(((IkReal(-1.00000000000000))*(cj3)*(x1395)))+(((IkReal(-1.00000000000000))*(x1388)*(x1396)))+(((cj2)*(sj3)*(x1387))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1385)))+(((cj4)*(x1384))))) != 0)?((IkReal)1/(((((cj4)*(x1385)))+(((cj4)*(x1384)))))):(IkReal)1.0e30))*(((((cj1)*(x1389)*(x1391)))+(((IkReal(-1.00000000000000))*(x1386)*(x1387)))+(((sj1)*(x1386)*(x1388)))+(((cj1)*(x1388)*(x1389)))+(((sj3)*(x1392)))+(((x1386)*(x1393)))+(((x1389)*(x1390)))+(((IkReal(-1.00000000000000))*(sj3)*(x1395))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x1384)+(x1385))) != 0)?((IkReal)1/(((x1384)+(x1385)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1390)))+(((IkReal(-1.00000000000000))*(x1391)*(x1396)))+(((cj3)*(x1392)))+(((IkReal(-1.00000000000000))*(x1393)*(x1394)))+(((IkReal(-1.00000000000000))*(sj1)*(x1388)*(x1394)))+(((IkReal(-1.00000000000000))*(cj3)*(x1395)))+(((IkReal(-1.00000000000000))*(x1388)*(x1396)))+(((cj2)*(sj3)*(x1387)))))))+IKsqr(((((IKabs(((((cj4)*(x1385)))+(((cj4)*(x1384))))) != 0)?((IkReal)1/(((((cj4)*(x1385)))+(((cj4)*(x1384)))))):(IkReal)1.0e30))*(((((cj1)*(x1389)*(x1391)))+(((IkReal(-1.00000000000000))*(x1386)*(x1387)))+(((sj1)*(x1386)*(x1388)))+(((cj1)*(x1388)*(x1389)))+(((sj3)*(x1392)))+(((x1386)*(x1393)))+(((x1389)*(x1390)))+(((IkReal(-1.00000000000000))*(sj3)*(x1395)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((x1384)+(x1385))) != 0)?((IkReal)1/(((x1384)+(x1385)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(sj2)*(sj3)*(x1390)))+(((IkReal(-1.00000000000000))*(x1391)*(x1396)))+(((cj3)*(x1392)))+(((IkReal(-1.00000000000000))*(x1393)*(x1394)))+(((IkReal(-1.00000000000000))*(sj1)*(x1388)*(x1394)))+(((IkReal(-1.00000000000000))*(cj3)*(x1395)))+(((IkReal(-1.00000000000000))*(x1388)*(x1396)))+(((cj2)*(sj3)*(x1387)))))), ((((IKabs(((((cj4)*(x1385)))+(((cj4)*(x1384))))) != 0)?((IkReal)1/(((((cj4)*(x1385)))+(((cj4)*(x1384)))))):(IkReal)1.0e30))*(((((cj1)*(x1389)*(x1391)))+(((IkReal(-1.00000000000000))*(x1386)*(x1387)))+(((sj1)*(x1386)*(x1388)))+(((cj1)*(x1388)*(x1389)))+(((sj3)*(x1392)))+(((x1386)*(x1393)))+(((x1389)*(x1390)))+(((IkReal(-1.00000000000000))*(sj3)*(x1395)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1397=IKcos(j5);
IkReal x1398=IKsin(j5);
IkReal x1399=((pz)*(sj2));
IkReal x1400=((IkReal(1.00000000000000))*(cj1));
IkReal x1401=((cj2)*(sj0));
IkReal x1402=((cj2)*(sj1));
IkReal x1403=((cj0)*(px));
IkReal x1404=((py)*(sj1));
IkReal x1405=((sj1)*(sj2));
IkReal x1406=((r01)*(sj0));
IkReal x1407=((IkReal(0.0750000000000000))*(cj1));
IkReal x1408=((sj0)*(sj2));
IkReal x1409=((IkReal(0.0800000000000000))*(sj4));
IkReal x1410=((cj1)*(sj2));
IkReal x1411=((IkReal(0.790000000000000))*(cj3));
IkReal x1412=((IkReal(1.00000000000000))*(cj0));
IkReal x1413=((IkReal(1.00000000000000))*(cj3));
IkReal x1414=((cj0)*(r00));
IkReal x1415=((sj2)*(x1414));
IkReal x1416=((sj4)*(x1397));
IkReal x1417=((sj3)*(x1398));
IkReal x1418=((cj4)*(x1397));
evalcond[0]=((((r00)*(sj0)))+(((IkReal(-1.00000000000000))*(sj3)*(x1418)))+(((IkReal(-1.00000000000000))*(x1398)*(x1413)))+(((IkReal(-1.00000000000000))*(r01)*(x1412))));
evalcond[1]=((((x1398)*(x1411)))+(((sj3)*(x1409)))+(((px)*(sj0)))+(((IkReal(0.790000000000000))*(sj3)*(x1418)))+(((IkReal(-1.00000000000000))*(py)*(x1412))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x1402)))+(((IkReal(-1.00000000000000))*(r01)*(x1400)*(x1401)))+(((IkReal(-1.00000000000000))*(cj2)*(x1400)*(x1414)))+(((x1405)*(x1414)))+(((x1405)*(x1406)))+(x1416)+(((IkReal(-1.00000000000000))*(r02)*(sj2)*(x1400))));
evalcond[3]=((((r02)*(x1405)))+(((IkReal(-1.00000000000000))*(x1413)*(x1418)))+(((x1402)*(x1414)))+(((IkReal(-1.00000000000000))*(cj2)*(r02)*(x1400)))+(((x1406)*(x1410)))+(((x1410)*(x1414)))+(x1417)+(((r01)*(sj1)*(x1401))));
evalcond[4]=((IkReal(0.405000000000000))+(((IkReal(-0.365000000000000))*(sj2)))+(((IkReal(-0.0750000000000000))*(x1405)))+(((IkReal(-1.00000000000000))*(cj2)*(x1400)*(x1403)))+(((IkReal(-1.00000000000000))*(x1399)*(x1400)))+(((IkReal(-1.00000000000000))*(pz)*(x1402)))+(((x1403)*(x1405)))+(((x1404)*(x1408)))+(((IkReal(-1.00000000000000))*(py)*(x1400)*(x1401)))+(((IkReal(0.0800000000000000))*(cj4)))+(((cj2)*(x1407)))+(((IkReal(-0.790000000000000))*(x1416))));
evalcond[5]=((IkReal(-0.0900000000000000))+(((IkReal(-0.0750000000000000))*(x1402)))+(((IkReal(-1.00000000000000))*(cj2)*(pz)*(x1400)))+(((IkReal(-0.365000000000000))*(cj2)))+(((cj1)*(py)*(x1408)))+(((x1411)*(x1418)))+(((sj1)*(x1399)))+(((cj3)*(x1409)))+(((x1402)*(x1403)))+(((x1401)*(x1404)))+(((IkReal(-1.00000000000000))*(sj2)*(x1407)))+(((IkReal(-0.790000000000000))*(x1417)))+(((x1403)*(x1410))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "8529cf64e1f47189a44788609179f24f"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
