/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:15:40.831635
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30,x31,x32,x33,x34,x35,x36,x37,x38,x39,x40,x41,x42;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[2]);
x3=IKsin(j[1]);
x4=IKsin(j[2]);
x5=IKcos(j[4]);
x6=IKcos(j[3]);
x7=IKsin(j[4]);
x8=IKsin(j[3]);
x9=IKsin(j[5]);
x10=IKcos(j[5]);
x11=IKsin(j[0]);
x12=((IkReal(1.00000000000000))*(x5));
x13=((IkReal(1.00000000000000))*(x10));
x14=((IkReal(2.16000000000000))*(x6));
x15=((IkReal(2.16000000000000))*(x7));
x16=((x10)*(x8));
x17=((x1)*(x2));
x18=((x1)*(x4));
x19=((x3)*(x4));
x20=((x6)*(x7));
x21=((x5)*(x6));
x22=((x2)*(x3));
x23=((x8)*(x9));
x24=((IkReal(1.54500000000000))*(x17));
x25=((IkReal(1.00000000000000))*(x19));
x26=((IkReal(1.00000000000000))*(x23));
x27=((((IkReal(-1.00000000000000))*(x25)))+(x17));
x28=((((IkReal(0.158000000000000))*(x17)))+(((IkReal(-0.158000000000000))*(x19))));
x29=((((IkReal(-1.00000000000000))*(x17)))+(x25));
x30=((((IkReal(-0.158000000000000))*(x22)))+(((IkReal(-0.158000000000000))*(x18))));
x31=((((IkReal(-2.16000000000000))*(x22)))+(((IkReal(-2.16000000000000))*(x18))));
x32=((((IkReal(0.158000000000000))*(x18)))+(((IkReal(0.158000000000000))*(x22))));
x33=((((IkReal(-1.00000000000000))*(x22)))+(((IkReal(-1.00000000000000))*(x18))));
x34=((((IkReal(1.00000000000000))*(x18)))+(((IkReal(1.00000000000000))*(x22))));
x35=((x28)*(x5));
x36=((((IkReal(1.00000000000000))*(x6)*(x9)))+(((x12)*(x16))));
x37=((x33)*(x7));
x38=((x21)*(x33));
x39=((((IkReal(-1.00000000000000))*(x14)*(x9)))+(((IkReal(-2.16000000000000))*(x16)*(x5)))+(((IkReal(0.158000000000000))*(x7)*(x8))));
x40=((((IkReal(1.20000000000000))*(x3)))+(((IkReal(1.54500000000000))*(x19)))+(((IkReal(0.145000000000000))*(x18)))+(((IkReal(0.145000000000000))*(x22))));
x41=((((x29)*(x7)))+(x38));
x42=((((IkReal(-1.00000000000000))*(x15)*(x29)))+(((IkReal(-1.00000000000000))*(x14)*(x33)*(x5))));
eetrans[0]=((((IkReal(0.350000000000000))*(x0)))+(((x11)*(x39)))+(((x0)*(((((IkReal(-1.00000000000000))*(x40)))+(((x20)*(x30)))+(((x23)*(x31)))+(((x10)*(x42)))+(x35)+(x24))))));
eetrans[1]=((((IkReal(-0.350000000000000))*(x11)))+(((x11)*(((((IkReal(-1.00000000000000))*(x20)*(x30)))+(((IkReal(-1.00000000000000))*(x12)*(x28)))+(((IkReal(-1.00000000000000))*(x24)))+(x40)+(((IkReal(-1.00000000000000))*(x26)*(x31)))+(((IkReal(-1.00000000000000))*(x13)*(x42)))))))+(((x0)*(x39))));
eetrans[2]=((IkReal(0.815000000000000))+(((x32)*(x5)))+(((x10)*(((((IkReal(-1.00000000000000))*(x14)*(x27)*(x5)))+(((IkReal(-1.00000000000000))*(x15)*(x33)))))))+(((IkReal(1.20000000000000))*(x1)))+(((IkReal(-0.145000000000000))*(x19)))+(((IkReal(1.54500000000000))*(x22)))+(((IkReal(1.54500000000000))*(x18)))+(((x23)*(((((IkReal(2.16000000000000))*(x17)))+(((IkReal(-2.16000000000000))*(x19)))))))+(((x20)*(x28)))+(((IkReal(0.145000000000000))*(x17))));
eerot[0]=((((x11)*(x36)))+(((x0)*(((((x23)*(x34)))+(((x10)*(x41))))))));
eerot[1]=((((x11)*(((((IkReal(-1.00000000000000))*(x26)*(x34)))+(((IkReal(-1.00000000000000))*(x13)*(x41)))))))+(((x0)*(x36))));
eerot[2]=((((x23)*(x29)))+(((x10)*(((((x21)*(x27)))+(x37))))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {0}; return freeparams; }
IKFAST_API int GetNumJoints() { return 6; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j1,cj1,sj1,htj1,j2,cj2,sj2,htj2,j3,cj3,sj3,htj3,j4,cj4,sj4,htj4,j5,cj5,sj5,htj5,j0,cj0,sj0,htj0,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij1[2], _nj1,_ij2[2], _nj2,_ij3[2], _nj3,_ij4[2], _nj4,_ij5[2], _nj5,_ij0[2], _nj0;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j2=numeric_limits<IkReal>::quiet_NaN(); _ij2[0] = -1; _ij2[1] = -1; _nj2 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1; j4=numeric_limits<IkReal>::quiet_NaN(); _ij4[0] = -1; _ij4[1] = -1; _nj4 = -1; j5=numeric_limits<IkReal>::quiet_NaN(); _ij5[0] = -1; _ij5[1] = -1; _nj5 = -1;  _ij0[0] = -1; _ij0[1] = -1; _nj0 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=((((IkReal(-1.00000000000000))*(r01)*(IKsin(j0))))+(((r00)*(IKcos(j0)))));
IkReal x43=IKcos(j0);
IkReal x44=IKsin(j0);
new_px=((IkReal(-0.350000000000000))+(((IkReal(-1.00000000000000))*(py)*(x44)))+(((IkReal(2.16000000000000))*(r01)*(x44)))+(((px)*(x43)))+(((IkReal(-2.16000000000000))*(r00)*(x43))));
new_r01=r02;
new_py=((IkReal(-0.815000000000000))+(((IkReal(-2.16000000000000))*(r02)))+(pz));
new_r02=((((IkReal(-1.00000000000000))*(r00)*(IKsin(j0))))+(((IkReal(-1.00000000000000))*(r01)*(IKcos(j0)))));
IkReal x45=IKcos(j0);
IkReal x46=IKsin(j0);
new_pz=((((IkReal(-1.00000000000000))*(px)*(x46)))+(((IkReal(-1.00000000000000))*(py)*(x45)))+(((IkReal(2.16000000000000))*(r01)*(x45)))+(((IkReal(2.16000000000000))*(r00)*(x46))));
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(-0.494514000000000))+(((IkReal(-2.69000000000000))*(py)))+(pp)+(((IkReal(-3.09000000000000))*(px))));
IkReal gconst1;
gconst1=((IkReal(7.41600000000000))+(((IkReal(0.580000000000000))*(px)))+(((IkReal(-6.18000000000000))*(py))));
IkReal gconst2;
gconst2=((IkReal(-1.19051400000000))+(((IkReal(-2.11000000000000))*(py)))+(((IkReal(3.09000000000000))*(px)))+(pp));
IkReal gconst3;
gconst3=((IkReal(2.16000000000000))+(((IkReal(-1.54500000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-1.34500000000000))*(r01))));
IkReal gconst4;
gconst4=((((IkReal(0.290000000000000))*(r00)))+(((IkReal(-3.09000000000000))*(r01))));
IkReal gconst5;
gconst5=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(-1.05500000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(1.54500000000000))*(r00))));
IkReal gconst6;
gconst6=((IkReal(-0.494514000000000))+(((IkReal(-2.69000000000000))*(py)))+(pp)+(((IkReal(-3.09000000000000))*(px))));
IkReal gconst7;
gconst7=((IkReal(7.41600000000000))+(((IkReal(0.580000000000000))*(px)))+(((IkReal(-6.18000000000000))*(py))));
IkReal gconst8;
gconst8=((IkReal(-1.19051400000000))+(((IkReal(-2.11000000000000))*(py)))+(((IkReal(3.09000000000000))*(px)))+(pp));
IkReal gconst9;
gconst9=((IkReal(2.16000000000000))+(((IkReal(-1.54500000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(-1.34500000000000))*(r01))));
IkReal gconst10;
gconst10=((((IkReal(0.290000000000000))*(r00)))+(((IkReal(-3.09000000000000))*(r01))));
IkReal gconst11;
gconst11=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(-1.05500000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(1.54500000000000))*(r00))));
IkReal gconst12;
gconst12=((((IkReal(5.38000000000000))*(px)))+(((IkReal(-6.18000000000000))*(py))));
IkReal gconst13;
gconst13=((((IkReal(1.16000000000000))*(py)))+(((IkReal(12.3600000000000))*(px))));
IkReal gconst14;
gconst14=((((IkReal(4.22000000000000))*(px)))+(((IkReal(6.18000000000000))*(py))));
IkReal gconst15;
gconst15=((((IkReal(2.69000000000000))*(r00)))+(((IkReal(-3.09000000000000))*(r01))));
IkReal gconst16;
gconst16=((((IkReal(6.18000000000000))*(r00)))+(((IkReal(0.580000000000000))*(r01))));
IkReal gconst17;
gconst17=((((IkReal(2.11000000000000))*(r00)))+(((IkReal(3.09000000000000))*(r01))));
IkReal gconst18;
gconst18=((((IkReal(5.38000000000000))*(px)))+(((IkReal(-6.18000000000000))*(py))));
IkReal gconst19;
gconst19=((((IkReal(1.16000000000000))*(py)))+(((IkReal(12.3600000000000))*(px))));
IkReal gconst20;
gconst20=((((IkReal(4.22000000000000))*(px)))+(((IkReal(6.18000000000000))*(py))));
IkReal gconst21;
gconst21=((((IkReal(2.69000000000000))*(r00)))+(((IkReal(-3.09000000000000))*(r01))));
IkReal gconst22;
gconst22=((((IkReal(6.18000000000000))*(r00)))+(((IkReal(0.580000000000000))*(r01))));
IkReal gconst23;
gconst23=((((IkReal(2.11000000000000))*(r00)))+(((IkReal(3.09000000000000))*(r01))));
IkReal gconst24;
gconst24=((IkReal(-0.494514000000000))+(((IkReal(3.09000000000000))*(px)))+(((IkReal(2.69000000000000))*(py)))+(pp));
IkReal gconst25;
gconst25=((IkReal(7.41600000000000))+(((IkReal(-0.580000000000000))*(px)))+(((IkReal(6.18000000000000))*(py))));
IkReal gconst26;
gconst26=((IkReal(-1.19051400000000))+(pp)+(((IkReal(2.11000000000000))*(py)))+(((IkReal(-3.09000000000000))*(px))));
IkReal gconst27;
gconst27=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(1.34500000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(1.54500000000000))*(r00))));
IkReal gconst28;
gconst28=((((IkReal(3.09000000000000))*(r01)))+(((IkReal(-0.290000000000000))*(r00))));
IkReal gconst29;
gconst29=((IkReal(2.16000000000000))+(((IkReal(1.05500000000000))*(r01)))+(((IkReal(-1.54500000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst30;
gconst30=((IkReal(-0.494514000000000))+(((IkReal(3.09000000000000))*(px)))+(((IkReal(2.69000000000000))*(py)))+(pp));
IkReal gconst31;
gconst31=((IkReal(7.41600000000000))+(((IkReal(-0.580000000000000))*(px)))+(((IkReal(6.18000000000000))*(py))));
IkReal gconst32;
gconst32=((IkReal(-1.19051400000000))+(pp)+(((IkReal(2.11000000000000))*(py)))+(((IkReal(-3.09000000000000))*(px))));
IkReal gconst33;
gconst33=((IkReal(2.16000000000000))+(((px)*(r00)))+(((IkReal(1.34500000000000))*(r01)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(1.54500000000000))*(r00))));
IkReal gconst34;
gconst34=((((IkReal(3.09000000000000))*(r01)))+(((IkReal(-0.290000000000000))*(r00))));
IkReal gconst35;
gconst35=((IkReal(2.16000000000000))+(((IkReal(1.05500000000000))*(r01)))+(((IkReal(-1.54500000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal x47=((IkReal(1.00000000000000))*(gconst26));
IkReal x48=((gconst30)*(gconst34));
IkReal x49=((gconst31)*(gconst33));
IkReal x50=((IkReal(1.00000000000000))*(gconst17));
IkReal x51=((gconst2)*(gconst6));
IkReal x52=((gconst11)*(gconst3));
IkReal x53=((gconst25)*(gconst29));
IkReal x54=((gconst14)*(gconst28));
IkReal x55=((IkReal(1.00000000000000))*(gconst14));
IkReal x56=((gconst21)*(gconst32));
IkReal x57=((gconst10)*(gconst6));
IkReal x58=((gconst21)*(gconst5));
IkReal x59=((gconst0)*(gconst8));
IkReal x60=((gconst13)*(gconst5));
IkReal x61=((IkReal(1.00000000000000))*(gconst9));
IkReal x62=((gconst30)*(gconst35));
IkReal x63=((gconst26)*(gconst27));
IkReal x64=((gconst22)*(gconst30));
IkReal x65=((IkReal(1.00000000000000))*(gconst12));
IkReal x66=((gconst2)*(gconst3));
IkReal x67=((gconst1)*(gconst5));
IkReal x68=((gconst19)*(gconst33));
IkReal x69=((gconst26)*(gconst28));
IkReal x70=((gconst11)*(gconst6));
IkReal x71=((gconst1)*(gconst7));
IkReal x72=((gconst18)*(gconst2));
IkReal x73=((gconst18)*(gconst34));
IkReal x74=((gconst24)*(gconst29));
IkReal x75=((IkReal(1.00000000000000))*(gconst23));
IkReal x76=((gconst20)*(gconst33));
IkReal x77=((gconst32)*(gconst33));
IkReal x78=((gconst0)*(gconst5));
IkReal x79=((IkReal(1.00000000000000))*(gconst18));
IkReal x80=((gconst21)*(gconst31));
IkReal x81=((gconst5)*(gconst9));
IkReal x82=((gconst4)*(gconst9));
IkReal x83=((gconst13)*(gconst29));
IkReal x84=((gconst14)*(gconst6));
IkReal x85=((IkReal(1.00000000000000))*(gconst10)*(gconst4));
IkReal x86=((gconst27)*(x77));
IkReal x87=((gconst2)*(gconst4)*(gconst7));
op[0]=((((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x48)))+(((IkReal(-1.00000000000000))*(x49)*(x53)))+(((IkReal(-1.00000000000000))*(x47)*(x86)))+(((x62)*(x63)))+(((x48)*(x53)))+(((IkReal(-1.00000000000000))*(x62)*(x74)))+(((x49)*(x69)))+(((x74)*(x77))));
op[1]=((((IkReal(-1.00000000000000))*(gconst25)*(x49)*(x50)))+(((IkReal(-1.00000000000000))*(gconst24)*(x50)*(x62)))+(((IkReal(-1.00000000000000))*(gconst15)*(x47)*(x77)))+(((IkReal(-1.00000000000000))*(x48)*(x54)))+(((IkReal(-1.00000000000000))*(gconst27)*(x47)*(x76)))+(((gconst16)*(gconst26)*(x49)))+(((x48)*(x83)))+(((IkReal(-1.00000000000000))*(x49)*(x83)))+(((IkReal(-1.00000000000000))*(gconst29)*(x62)*(x65)))+(((gconst15)*(gconst26)*(x62)))+(((IkReal(-1.00000000000000))*(x53)*(x80)))+(((IkReal(-1.00000000000000))*(gconst27)*(x47)*(x56)))+(((gconst17)*(gconst25)*(x48)))+(((x56)*(x74)))+(((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x73)))+(((IkReal(-1.00000000000000))*(gconst16)*(x47)*(x48)))+(((x68)*(x69)))+(((x69)*(x80)))+(((x53)*(x64)))+(((gconst12)*(gconst29)*(x77)))+(((x49)*(x54)))+(((gconst14)*(gconst27)*(x62)))+(((IkReal(-1.00000000000000))*(x53)*(x68)))+(((IkReal(-1.00000000000000))*(x55)*(x86)))+(((IkReal(-1.00000000000000))*(gconst35)*(x74)*(x79)))+(((gconst18)*(gconst35)*(x63)))+(((gconst17)*(gconst24)*(x77)))+(((x53)*(x73)))+(((IkReal(-1.00000000000000))*(gconst30)*(x74)*(x75)))+(((x74)*(x76)))+(((IkReal(-1.00000000000000))*(gconst28)*(x47)*(x64)))+(((gconst23)*(gconst30)*(x63))));
op[2]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst32)))+(((gconst14)*(gconst19)*(gconst28)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst30)))+(((gconst24)*(gconst29)*(gconst33)*(gconst8)))+(((gconst16)*(gconst19)*(gconst26)*(gconst33)))+(((gconst24)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst29)*(gconst35)))+(((gconst26)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst28)*(gconst34)))+(((gconst12)*(gconst17)*(gconst32)*(gconst33)))+(((gconst26)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst29)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst24)*(gconst35)))+(((gconst26)*(gconst28)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst30)))+(((gconst15)*(gconst18)*(gconst26)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst30)))+(((gconst12)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst30)))+(((gconst13)*(gconst18)*(gconst29)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst33)))+(((gconst16)*(gconst21)*(gconst26)*(gconst31)))+(((gconst17)*(gconst18)*(gconst25)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst30)*(gconst35)))+(((gconst24)*(gconst29)*(gconst32)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst27)))+(((gconst14)*(gconst21)*(gconst28)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst32)*(gconst9)))+(((gconst17)*(gconst22)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst30)*(gconst35)*(gconst5)))+(((gconst14)*(gconst16)*(gconst31)*(gconst33)))+(((gconst18)*(gconst23)*(gconst26)*(gconst27)))+(((gconst14)*(gconst23)*(gconst27)*(gconst30)))+(((gconst14)*(gconst18)*(gconst27)*(gconst35)))+(((gconst1)*(gconst29)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst33)))+(((gconst2)*(gconst28)*(gconst31)*(gconst33)))+(((gconst13)*(gconst17)*(gconst30)*(gconst34)))+(((gconst2)*(gconst27)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst33)))+(((gconst20)*(gconst21)*(gconst24)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst29)))+(((gconst25)*(gconst30)*(gconst34)*(gconst5)))+(((gconst19)*(gconst21)*(gconst26)*(gconst28)))+(((gconst25)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst29)))+(((gconst10)*(gconst25)*(gconst29)*(gconst30)))+(((gconst17)*(gconst20)*(gconst24)*(gconst33)))+(((gconst26)*(gconst31)*(gconst33)*(gconst4)))+(((gconst26)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst26)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst28)*(gconst34)*(gconst6)))+(((gconst15)*(gconst23)*(gconst26)*(gconst30)))+(((gconst17)*(gconst21)*(gconst24)*(gconst32)))+(((gconst14)*(gconst15)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst33)))+(((gconst0)*(gconst29)*(gconst32)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst30)*(gconst34)))+(((gconst11)*(gconst26)*(gconst27)*(gconst30))));
op[3]=((((gconst12)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst30)))+(((gconst12)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst28)*(gconst34)*(gconst6)))+(((gconst12)*(gconst29)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst23)*(gconst24)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst34)))+(((gconst16)*(gconst26)*(gconst33)*(gconst7)))+(((gconst15)*(gconst18)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst29)*(gconst35)*(gconst6)))+(((gconst14)*(gconst28)*(gconst33)*(gconst7)))+(((gconst20)*(gconst24)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst15)*(gconst23)*(gconst30)))+(((gconst13)*(gconst30)*(gconst34)*(gconst5)))+(((gconst17)*(gconst24)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst32)))+(((gconst14)*(gconst16)*(gconst21)*(gconst31)))+(((gconst2)*(gconst21)*(gconst28)*(gconst31)))+(((gconst15)*(gconst26)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst27)*(gconst8)))+(((gconst18)*(gconst25)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst33)))+(((gconst10)*(gconst18)*(gconst25)*(gconst29)))+(((gconst14)*(gconst19)*(gconst21)*(gconst28)))+(((gconst11)*(gconst14)*(gconst27)*(gconst30)))+(((gconst19)*(gconst26)*(gconst33)*(gconst4)))+(((gconst16)*(gconst19)*(gconst21)*(gconst26)))+(((gconst17)*(gconst18)*(gconst22)*(gconst25)))+(((gconst20)*(gconst24)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst30)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst30)))+(((gconst12)*(gconst17)*(gconst20)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst30)))+(((gconst22)*(gconst25)*(gconst30)*(gconst5)))+(((gconst16)*(gconst2)*(gconst31)*(gconst33)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst33)*(gconst5)))+(((gconst14)*(gconst27)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst24)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst30)*(gconst35)*(gconst5)))+(((gconst12)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst27)*(gconst9)))+(((gconst17)*(gconst25)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst29)*(gconst6)))+(((gconst11)*(gconst15)*(gconst26)*(gconst30)))+(((gconst18)*(gconst26)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst28)))+(((gconst21)*(gconst26)*(gconst28)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst29)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst22)*(gconst26)))+(((gconst21)*(gconst24)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst35)))+(((gconst19)*(gconst2)*(gconst28)*(gconst33)))+(((gconst22)*(gconst25)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst33)))+(((gconst1)*(gconst17)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst26)*(gconst34)*(gconst6)))+(((gconst18)*(gconst2)*(gconst27)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst33)))+(((gconst23)*(gconst26)*(gconst3)*(gconst30)))+(((gconst0)*(gconst17)*(gconst32)*(gconst33)))+(((gconst0)*(gconst20)*(gconst29)*(gconst33)))+(((gconst12)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst33)*(gconst7)))+(((gconst2)*(gconst23)*(gconst27)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst30)*(gconst35)))+(((gconst14)*(gconst28)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst28)))+(((gconst14)*(gconst3)*(gconst30)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst31)*(gconst9)))+(((gconst1)*(gconst18)*(gconst29)*(gconst34)))+(((gconst14)*(gconst18)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst33)))+(((gconst14)*(gconst15)*(gconst18)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst32)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst33)))+(((gconst17)*(gconst24)*(gconst33)*(gconst8)))+(((gconst13)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst34)))+(((gconst0)*(gconst21)*(gconst29)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst33)))+(((gconst10)*(gconst17)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst24)*(gconst35)*(gconst5)))+(((gconst14)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst27)))+(((gconst13)*(gconst18)*(gconst22)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst26)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst21)*(gconst25)))+(((gconst17)*(gconst20)*(gconst21)*(gconst24)))+(((gconst23)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst33)))+(((gconst19)*(gconst26)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst33)))+(((gconst21)*(gconst26)*(gconst31)*(gconst4)))+(((gconst13)*(gconst17)*(gconst18)*(gconst34)))+(((gconst15)*(gconst2)*(gconst30)*(gconst35)))+(((gconst16)*(gconst26)*(gconst31)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst29)))+(((gconst21)*(gconst24)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst26)*(gconst27)))+(((gconst13)*(gconst17)*(gconst22)*(gconst30)))+(((gconst1)*(gconst22)*(gconst29)*(gconst30))));
op[4]=((((gconst14)*(gconst15)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst31)*(gconst5)*(gconst9)))+(((gconst12)*(gconst20)*(gconst29)*(gconst9)))+(((gconst12)*(gconst21)*(gconst32)*(gconst5)))+(((gconst10)*(gconst13)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst21)*(gconst25)*(gconst7)))+(((gconst0)*(gconst20)*(gconst21)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst29)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst33)*(gconst5)))+(((gconst14)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst33)*(gconst8)))+(((gconst14)*(gconst16)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst27)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst21)*(gconst25)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst30)))+(((gconst15)*(gconst23)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst29)*(gconst35)*(gconst6)))+(((gconst13)*(gconst17)*(gconst18)*(gconst22)))+(((gconst26)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst23)*(gconst24)*(gconst6)))+(((gconst1)*(gconst29)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst26)*(gconst4)))+(((gconst2)*(gconst28)*(gconst31)*(gconst9)))+(((gconst0)*(gconst17)*(gconst20)*(gconst33)))+(((gconst12)*(gconst21)*(gconst29)*(gconst8)))+(((gconst16)*(gconst2)*(gconst21)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst26)))+(((gconst25)*(gconst34)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst30)*(gconst34)*(gconst4)))+(((gconst0)*(gconst17)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst35)*(gconst5)))+(((gconst12)*(gconst17)*(gconst33)*(gconst8)))+(((gconst1)*(gconst17)*(gconst18)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst18)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst35)))+(((gconst11)*(gconst2)*(gconst27)*(gconst30)))+(((gconst13)*(gconst17)*(gconst34)*(gconst6)))+(((gconst0)*(gconst29)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst34)*(gconst6)))+(((gconst14)*(gconst16)*(gconst31)*(gconst9)))+(((gconst2)*(gconst27)*(gconst35)*(gconst6)))+(((gconst18)*(gconst23)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst33)*(gconst8)))+(((gconst14)*(gconst21)*(gconst31)*(gconst4)))+(((gconst2)*(gconst3)*(gconst30)*(gconst35)))+(((gconst19)*(gconst2)*(gconst21)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst19)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst32)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst31)*(gconst9)))+(((gconst19)*(gconst21)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst35)*(gconst6)))+(((gconst10)*(gconst17)*(gconst18)*(gconst25)))+(((gconst26)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst26)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst30)))+(((gconst0)*(gconst32)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst30)))+(((gconst11)*(gconst14)*(gconst18)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst21)*(gconst26)*(gconst3)))+(((gconst12)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst34)*(gconst4)))+(((gconst12)*(gconst20)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst29)))+(((gconst18)*(gconst22)*(gconst25)*(gconst5)))+(((gconst2)*(gconst28)*(gconst33)*(gconst7)))+(((gconst26)*(gconst33)*(gconst4)*(gconst7)))+(((gconst16)*(gconst21)*(gconst26)*(gconst7)))+(((gconst16)*(gconst19)*(gconst26)*(gconst9)))+(((gconst17)*(gconst21)*(gconst24)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst28)))+(((gconst14)*(gconst18)*(gconst3)*(gconst35)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst33)))+(((gconst18)*(gconst2)*(gconst23)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst33)))+(((gconst26)*(gconst31)*(gconst4)*(gconst9)))+(((gconst24)*(gconst32)*(gconst5)*(gconst9)))+(((gconst10)*(gconst25)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst26)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst33)))+(((gconst13)*(gconst22)*(gconst29)*(gconst6)))+(((gconst1)*(gconst30)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst21)))+(((gconst14)*(gconst15)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst18)*(gconst24)))+(((gconst24)*(gconst33)*(gconst5)*(gconst8)))+(((gconst15)*(gconst2)*(gconst23)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst21)))+(((gconst14)*(gconst19)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst30)*(gconst35)*(gconst5)))+(((gconst10)*(gconst25)*(gconst29)*(gconst6)))+(((gconst15)*(gconst18)*(gconst2)*(gconst35)))+(((gconst17)*(gconst22)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst34)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst31)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst30)))+(((gconst24)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst24)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst23)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst28)))+(((gconst2)*(gconst31)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst29)))+(((gconst1)*(gconst18)*(gconst22)*(gconst29)))+(((gconst17)*(gconst20)*(gconst24)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst26)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst30)*(gconst5)))+(((gconst14)*(gconst19)*(gconst33)*(gconst4)))+(((gconst20)*(gconst21)*(gconst24)*(gconst5)))+(((gconst11)*(gconst26)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst29)))+(((gconst11)*(gconst15)*(gconst18)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst31)))+(((gconst1)*(gconst17)*(gconst22)*(gconst30)))+(((gconst13)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst30)))+(((gconst1)*(gconst10)*(gconst29)*(gconst30)))+(((gconst16)*(gconst19)*(gconst2)*(gconst33)))+(((gconst14)*(gconst16)*(gconst19)*(gconst21)))+(((gconst12)*(gconst17)*(gconst32)*(gconst9)))+(((gconst0)*(gconst29)*(gconst32)*(gconst9)))+(((gconst13)*(gconst22)*(gconst30)*(gconst5)))+(((gconst11)*(gconst26)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst33)))+(((gconst14)*(gconst21)*(gconst28)*(gconst7))));
op[5]=((((gconst13)*(gconst18)*(gconst22)*(gconst5)))+(((gconst14)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst19)*(gconst9)))+(((gconst2)*(gconst23)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst27)*(gconst8)))+(((gconst12)*(gconst20)*(gconst21)*(gconst5)))+(((gconst16)*(gconst19)*(gconst2)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst28)*(gconst6)))+(((gconst14)*(gconst15)*(gconst23)*(gconst6)))+(((gconst1)*(gconst17)*(gconst18)*(gconst22)))+(((gconst2)*(gconst23)*(gconst27)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst16)*(gconst22)*(gconst6)))+(((gconst0)*(gconst21)*(gconst32)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst29)*(gconst9)))+(((gconst14)*(gconst19)*(gconst21)*(gconst4)))+(((gconst21)*(gconst26)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst18)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst25)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst31)*(gconst5)*(gconst9)))+(((gconst10)*(gconst17)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst18)*(gconst24)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst21)*(gconst8)))+(((gconst2)*(gconst21)*(gconst28)*(gconst7)))+(((gconst14)*(gconst16)*(gconst21)*(gconst7)))+(((gconst14)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst18)*(gconst23)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst26)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst3)*(gconst33)))+(((gconst16)*(gconst2)*(gconst33)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst25)*(gconst7)*(gconst9)))+(((gconst11)*(gconst14)*(gconst27)*(gconst6)))+(((gconst1)*(gconst18)*(gconst34)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst26)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst23)*(gconst24)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst31)*(gconst9)))+(((gconst17)*(gconst24)*(gconst8)*(gconst9)))+(((gconst13)*(gconst17)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst30)))+(((gconst10)*(gconst13)*(gconst29)*(gconst6)))+(((gconst1)*(gconst22)*(gconst29)*(gconst6)))+(((gconst11)*(gconst14)*(gconst15)*(gconst18)))+(((gconst19)*(gconst2)*(gconst33)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst26)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst27)*(gconst9)))+(((gconst0)*(gconst21)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst29)*(gconst6)))+(((gconst15)*(gconst2)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst26)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst34)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst28)*(gconst6)))+(((gconst16)*(gconst2)*(gconst31)*(gconst9)))+(((gconst10)*(gconst13)*(gconst30)*(gconst5)))+(((gconst0)*(gconst20)*(gconst33)*(gconst5)))+(((gconst19)*(gconst26)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst33)*(gconst5)*(gconst7)))+(((gconst11)*(gconst14)*(gconst3)*(gconst30)))+(((gconst22)*(gconst25)*(gconst5)*(gconst6)))+(((gconst2)*(gconst21)*(gconst31)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst2)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst26)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst21)*(gconst29)*(gconst7)))+(((gconst0)*(gconst17)*(gconst32)*(gconst9)))+(((gconst13)*(gconst34)*(gconst5)*(gconst6)))+(((gconst16)*(gconst26)*(gconst7)*(gconst9)))+(((gconst14)*(gconst31)*(gconst4)*(gconst9)))+(((gconst10)*(gconst18)*(gconst25)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst27)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst21)*(gconst5)))+(((gconst12)*(gconst17)*(gconst21)*(gconst8)))+(((gconst11)*(gconst15)*(gconst2)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst33)*(gconst8)))+(((gconst11)*(gconst18)*(gconst2)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst34)*(gconst6)))+(((gconst0)*(gconst20)*(gconst29)*(gconst9)))+(((gconst0)*(gconst17)*(gconst33)*(gconst8)))+(((gconst20)*(gconst24)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst21)*(gconst7)))+(((gconst23)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst17)*(gconst23)*(gconst6)))+(((gconst12)*(gconst33)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst23)*(gconst30)*(gconst5)))+(((gconst12)*(gconst32)*(gconst5)*(gconst9)))+(((gconst11)*(gconst18)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst18)*(gconst26)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst33)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst17)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst21)))+(((gconst1)*(gconst22)*(gconst30)*(gconst5)))+(((gconst15)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst18)*(gconst2)*(gconst22)))+(((gconst14)*(gconst16)*(gconst19)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst20)*(gconst9)))+(((gconst1)*(gconst17)*(gconst34)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst35)*(gconst5)))+(((gconst10)*(gconst13)*(gconst17)*(gconst18)))+(((gconst0)*(gconst17)*(gconst20)*(gconst21)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst33)*(gconst7)))+(((gconst12)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst3)*(gconst32)*(gconst9)))+(((gconst12)*(gconst29)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst22)*(gconst30)*(gconst4)))+(((gconst19)*(gconst2)*(gconst28)*(gconst9)))+(((gconst18)*(gconst2)*(gconst3)*(gconst35)))+(((gconst14)*(gconst33)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst21)*(gconst3)*(gconst32)))+(((gconst11)*(gconst15)*(gconst26)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst30)))+(((gconst21)*(gconst24)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst18)*(gconst23)))+(((gconst1)*(gconst10)*(gconst18)*(gconst29)))+(((gconst14)*(gconst18)*(gconst23)*(gconst3))));
op[6]=((((IkReal(-1.00000000000000))*(gconst26)*(gconst3)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst22)*(gconst6)))+(((gconst14)*(gconst21)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst25)*(gconst5)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst23)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst19)*(gconst9)))+(((gconst0)*(gconst33)*(gconst5)*(gconst8)))+(((gconst1)*(gconst10)*(gconst29)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst27)*(gconst8)*(gconst9)))+(((gconst14)*(gconst23)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst34)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst17)*(gconst23)*(gconst6)))+(((gconst1)*(gconst10)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst17)*(gconst6)))+(((gconst13)*(gconst22)*(gconst5)*(gconst6)))+(((gconst12)*(gconst20)*(gconst5)*(gconst9)))+(((gconst24)*(gconst5)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst24)*(gconst5)*(gconst6)))+(((gconst2)*(gconst3)*(gconst35)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst21)*(gconst8)))+(((gconst0)*(gconst20)*(gconst21)*(gconst5)))+(((gconst15)*(gconst2)*(gconst23)*(gconst6)))+(((gconst1)*(gconst18)*(gconst22)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst31)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst26)*(gconst4)*(gconst6)))+(((gconst10)*(gconst25)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst7)*(gconst9)))+(((gconst14)*(gconst16)*(gconst7)*(gconst9)))+(((gconst11)*(gconst2)*(gconst27)*(gconst6)))+(((gconst0)*(gconst32)*(gconst5)*(gconst9)))+(((gconst19)*(gconst2)*(gconst21)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst17)*(gconst18)))+(((gconst11)*(gconst26)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst29)*(gconst7)*(gconst9)))+(((gconst12)*(gconst21)*(gconst5)*(gconst8)))+(((gconst11)*(gconst15)*(gconst18)*(gconst2)))+(((gconst1)*(gconst17)*(gconst22)*(gconst6)))+(((gconst10)*(gconst13)*(gconst17)*(gconst6)))+(((gconst16)*(gconst19)*(gconst2)*(gconst9)))+(((gconst18)*(gconst2)*(gconst23)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst32)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst19)*(gconst21)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst30)*(gconst4)))+(((gconst16)*(gconst2)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst15)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst18)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst20)*(gconst21)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst21)*(gconst5)*(gconst7)))+(((gconst1)*(gconst34)*(gconst5)*(gconst6)))+(((gconst10)*(gconst13)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst4)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst19)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst35)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst14)*(gconst16)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst29)*(gconst6)))+(((gconst26)*(gconst4)*(gconst7)*(gconst9)))+(((gconst1)*(gconst10)*(gconst30)*(gconst5)))+(((gconst2)*(gconst28)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst3)*(gconst33)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst22)*(gconst4)))+(((gconst11)*(gconst14)*(gconst15)*(gconst6)))+(((gconst12)*(gconst17)*(gconst8)*(gconst9)))+(((gconst11)*(gconst2)*(gconst3)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst18)*(gconst23)*(gconst5)))+(((gconst0)*(gconst29)*(gconst8)*(gconst9)))+(((gconst14)*(gconst19)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst28)*(gconst6)))+(((gconst11)*(gconst14)*(gconst18)*(gconst3)))+(((gconst0)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst33)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst21)*(gconst3)*(gconst8)))+(((gconst2)*(gconst31)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst17)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst20)*(gconst3)*(gconst9)))+(((gconst2)*(gconst33)*(gconst4)*(gconst7)))+(((gconst0)*(gconst17)*(gconst21)*(gconst8))));
op[7]=((((IkReal(-1.00000000000000))*(gconst5)*(x65)*(x70)))+(((gconst22)*(gconst6)*(x67)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(x51)))+(((gconst1)*(gconst17)*(x57)))+(((IkReal(-1.00000000000000))*(x72)*(x85)))+(((x58)*(x59)))+(((IkReal(-1.00000000000000))*(gconst6)*(x75)*(x78)))+(((gconst10)*(gconst18)*(x67)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst8)*(x66)))+(((IkReal(-1.00000000000000))*(gconst19)*(x61)*(x67)))+(((gconst11)*(gconst15)*(x51)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst4)*(x51)))+(((IkReal(-1.00000000000000))*(gconst0)*(x50)*(x70)))+(((IkReal(-1.00000000000000))*(gconst11)*(x78)*(x79)))+(((gconst23)*(gconst3)*(x51)))+(((IkReal(-1.00000000000000))*(gconst3)*(gconst8)*(gconst9)*(x55)))+(((x57)*(x60)))+(((gconst14)*(gconst7)*(x82)))+(((IkReal(-1.00000000000000))*(x58)*(x71)))+(((IkReal(-1.00000000000000))*(gconst7)*(x60)*(x61)))+(((x52)*(x84)))+(((IkReal(-1.00000000000000))*(gconst9)*(x50)*(x71)))+(((IkReal(-1.00000000000000))*(gconst20)*(x61)*(x66)))+(((gconst20)*(gconst9)*(x78)))+(((gconst21)*(x87)))+(((gconst16)*(gconst2)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst4)*(x55)*(x57)))+(((gconst12)*(gconst8)*(x81)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst2)*(gconst8)*(x61)))+(((x52)*(x72)))+(((gconst19)*(gconst2)*(x82)))+(((gconst17)*(gconst9)*(x59))));
op[8]=((((IkReal(-1.00000000000000))*(x51)*(x85)))+(((IkReal(-1.00000000000000))*(x70)*(x78)))+(((IkReal(-1.00000000000000))*(gconst7)*(x61)*(x67)))+(((IkReal(-1.00000000000000))*(gconst8)*(x61)*(x66)))+(((x57)*(x67)))+(((x51)*(x52)))+(((gconst2)*(gconst7)*(x82)))+(((x59)*(x81))));
polyroots8(op,zeror,numroots);
IkReal j1array[8], cj1array[8], sj1array[8], tempj1array[1];
int numsolutions = 0;
for(int ij1 = 0; ij1 < numroots; ++ij1)
{
IkReal htj1 = zeror[ij1];
tempj1array[0]=((IkReal(2.00000000000000))*(atan(htj1)));
for(int kj1 = 0; kj1 < 1; ++kj1)
{
j1array[numsolutions] = tempj1array[kj1];
if( j1array[numsolutions] > IKPI )
{
    j1array[numsolutions]-=IK2PI;
}
else if( j1array[numsolutions] < -IKPI )
{
    j1array[numsolutions]+=IK2PI;
}
sj1array[numsolutions] = IKsin(j1array[numsolutions]);
cj1array[numsolutions] = IKcos(j1array[numsolutions]);
numsolutions++;
}
}
bool j1valid[8]={true,true,true,true,true,true,true,true};
_nj1 = 8;
for(int ij1 = 0; ij1 < numsolutions; ++ij1)
    {
if( !j1valid[ij1] )
{
    continue;
}
    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
htj1 = IKtan(j1/2);

_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < numsolutions; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst52;
IkReal x88=(cj1)*(cj1);
IkReal x89=(sj1)*(sj1);
IkReal x90=((py)*(r00));
IkReal x91=((px)*(r01));
IkReal x92=((IkReal(4.81610000000000))*(x89));
IkReal x93=((IkReal(4.81610000000000))*(x88));
gconst52=IKsign(((((IkReal(5.77932000000000))*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(x90)*(x93)))+(((IkReal(-1.00000000000000))*(x90)*(x92)))+(((IkReal(5.77932000000000))*(cj1)*(r00)))+(((x91)*(x93)))+(((x91)*(x92)))));
IkReal x94=(sj1)*(sj1);
IkReal x95=(cj1)*(cj1);
IkReal x96=((px)*(r01));
IkReal x97=((IkReal(1.00000000000000))*(py)*(r00));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x95)*(x97)))+(((IkReal(-1.00000000000000))*(x94)*(x97)))+(((IkReal(1.20000000000000))*(cj1)*(r00)))+(((IkReal(1.20000000000000))*(r01)*(sj1)))+(((x95)*(x96)))+(((x94)*(x96))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j2array[1], cj2array[1], sj2array[1];
bool j2valid[1]={false};
_nj2 = 1;
IkReal x98=(py)*(py);
IkReal x99=(px)*(px);
IkReal x100=(sj1)*(sj1);
IkReal x101=(cj1)*(cj1);
IkReal x102=((py)*(r00));
IkReal x103=((px)*(r01));
IkReal x104=((IkReal(1.54500000000000))*(pp));
IkReal x105=((cj1)*(r01));
IkReal x106=((IkReal(3.09000000000000))*(sj1));
IkReal x107=((cj1)*(py));
IkReal x108=((r00)*(sj1));
IkReal x109=((cj1)*(px));
IkReal x110=((py)*(sj1));
IkReal x111=((px)*(r00));
IkReal x112=((py)*(r01));
IkReal x113=((cj1)*(r00));
IkReal x114=((pz)*(r02));
IkReal x115=((r01)*(sj1));
IkReal x116=((IkReal(0.145000000000000))*(pp));
IkReal x117=((px)*(sj1));
IkReal x118=((IkReal(0.348000000000000))*(x100));
IkReal x119=((IkReal(3.70800000000000))*(x101));
IkReal x120=((IkReal(3.09000000000000))*(x114));
IkReal x121=((IkReal(0.348000000000000))*(x101));
IkReal x122=((IkReal(0.290000000000000))*(x117));
IkReal x123=((r01)*(x98));
IkReal x124=((IkReal(0.290000000000000))*(x99));
IkReal x125=((IkReal(3.70800000000000))*(x100));
if( IKabs(((gconst52)*(((IkReal(-0.751680000000000))+(((IkReal(0.539764530000000))*(x105)))+(((IkReal(6.67440000000000))*(x109)))+(((IkReal(-0.539764530000000))*(x108)))+(((px)*(x102)*(x106)))+(((x106)*(x123)))+(((x102)*(x125)))+(((IkReal(1.30168413000000))*(x115)))+(((IkReal(1.30168413000000))*(x113)))+(((x102)*(x119)))+(((IkReal(3.09000000000000))*(x113)*(x99)))+(((IkReal(0.290000000000000))*(x107)*(x114)))+(((IkReal(3.09000000000000))*(x103)*(x107)))+(((IkReal(-0.348000000000000))*(x112)))+(((IkReal(-0.348000000000000))*(x111)))+(((IkReal(-0.348000000000000))*(x114)))+(((x109)*(x120)))+(((IkReal(-0.626400000000000))*(x117)))+(((x108)*(x116)))+(((IkReal(0.290000000000000))*(x102)*(x109)))+(((py)*(x106)*(x114)))+(((IkReal(-1.00000000000000))*(x114)*(x122)))+(((IkReal(-0.290000000000000))*(x103)*(x110)))+(((IkReal(-1.00000000000000))*(x108)*(x124)))+(((IkReal(0.290000000000000))*(x105)*(x98)))+(((IkReal(0.626400000000000))*(x107)))+(((IkReal(-1.00000000000000))*(x103)*(x119)))+(((IkReal(-1.00000000000000))*(x105)*(x116)))+(((IkReal(-1.00000000000000))*(x103)*(x125)))+(((IkReal(-1.00000000000000))*(x104)*(x113)))+(((IkReal(-1.00000000000000))*(x104)*(x115)))+(((IkReal(6.67440000000000))*(x110))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((IkReal(8.00928000000000))+(((IkReal(-6.67440000000000))*(x107)))+(((IkReal(-1.00000000000000))*(x104)*(x108)))+(((IkReal(0.290000000000000))*(x115)*(x98)))+(((IkReal(-3.09000000000000))*(x102)*(x109)))+(((r00)*(x106)*(x99)))+(((IkReal(0.290000000000000))*(x109)*(x114)))+(((IkReal(-1.00000000000000))*(x113)*(x116)))+(((x102)*(x121)))+(((x102)*(x122)))+(((IkReal(-5.75128413000000))*(x105)))+(((x102)*(x118)))+(((IkReal(0.122164530000000))*(x115)))+(((IkReal(0.122164530000000))*(x113)))+(((IkReal(-1.00000000000000))*(x115)*(x116)))+(((x113)*(x124)))+(((IkReal(3.70800000000000))*(x114)))+(((IkReal(3.70800000000000))*(x111)))+(((IkReal(3.70800000000000))*(x112)))+(((IkReal(-3.09000000000000))*(x105)*(x98)))+(((IkReal(-1.00000000000000))*(x107)*(x120)))+(((IkReal(0.290000000000000))*(x103)*(x107)))+(((IkReal(0.290000000000000))*(x110)*(x114)))+(((IkReal(5.75128413000000))*(x108)))+(((IkReal(0.626400000000000))*(x110)))+(((px)*(x106)*(x114)))+(((x104)*(x105)))+(((IkReal(0.626400000000000))*(x109)))+(((IkReal(-1.00000000000000))*(x103)*(x118)))+(((IkReal(-1.00000000000000))*(x103)*(x121)))+(((py)*(x103)*(x106)))+(((IkReal(6.67440000000000))*(x117))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j2array[0]=IKatan2(((gconst52)*(((IkReal(-0.751680000000000))+(((IkReal(0.539764530000000))*(x105)))+(((IkReal(6.67440000000000))*(x109)))+(((IkReal(-0.539764530000000))*(x108)))+(((px)*(x102)*(x106)))+(((x106)*(x123)))+(((x102)*(x125)))+(((IkReal(1.30168413000000))*(x115)))+(((IkReal(1.30168413000000))*(x113)))+(((x102)*(x119)))+(((IkReal(3.09000000000000))*(x113)*(x99)))+(((IkReal(0.290000000000000))*(x107)*(x114)))+(((IkReal(3.09000000000000))*(x103)*(x107)))+(((IkReal(-0.348000000000000))*(x112)))+(((IkReal(-0.348000000000000))*(x111)))+(((IkReal(-0.348000000000000))*(x114)))+(((x109)*(x120)))+(((IkReal(-0.626400000000000))*(x117)))+(((x108)*(x116)))+(((IkReal(0.290000000000000))*(x102)*(x109)))+(((py)*(x106)*(x114)))+(((IkReal(-1.00000000000000))*(x114)*(x122)))+(((IkReal(-0.290000000000000))*(x103)*(x110)))+(((IkReal(-1.00000000000000))*(x108)*(x124)))+(((IkReal(0.290000000000000))*(x105)*(x98)))+(((IkReal(0.626400000000000))*(x107)))+(((IkReal(-1.00000000000000))*(x103)*(x119)))+(((IkReal(-1.00000000000000))*(x105)*(x116)))+(((IkReal(-1.00000000000000))*(x103)*(x125)))+(((IkReal(-1.00000000000000))*(x104)*(x113)))+(((IkReal(-1.00000000000000))*(x104)*(x115)))+(((IkReal(6.67440000000000))*(x110)))))), ((gconst52)*(((IkReal(8.00928000000000))+(((IkReal(-6.67440000000000))*(x107)))+(((IkReal(-1.00000000000000))*(x104)*(x108)))+(((IkReal(0.290000000000000))*(x115)*(x98)))+(((IkReal(-3.09000000000000))*(x102)*(x109)))+(((r00)*(x106)*(x99)))+(((IkReal(0.290000000000000))*(x109)*(x114)))+(((IkReal(-1.00000000000000))*(x113)*(x116)))+(((x102)*(x121)))+(((x102)*(x122)))+(((IkReal(-5.75128413000000))*(x105)))+(((x102)*(x118)))+(((IkReal(0.122164530000000))*(x115)))+(((IkReal(0.122164530000000))*(x113)))+(((IkReal(-1.00000000000000))*(x115)*(x116)))+(((x113)*(x124)))+(((IkReal(3.70800000000000))*(x114)))+(((IkReal(3.70800000000000))*(x111)))+(((IkReal(3.70800000000000))*(x112)))+(((IkReal(-3.09000000000000))*(x105)*(x98)))+(((IkReal(-1.00000000000000))*(x107)*(x120)))+(((IkReal(0.290000000000000))*(x103)*(x107)))+(((IkReal(0.290000000000000))*(x110)*(x114)))+(((IkReal(5.75128413000000))*(x108)))+(((IkReal(0.626400000000000))*(x110)))+(((px)*(x106)*(x114)))+(((x104)*(x105)))+(((IkReal(0.626400000000000))*(x109)))+(((IkReal(-1.00000000000000))*(x103)*(x118)))+(((IkReal(-1.00000000000000))*(x103)*(x121)))+(((py)*(x103)*(x106)))+(((IkReal(6.67440000000000))*(x117)))))));
sj2array[0]=IKsin(j2array[0]);
cj2array[0]=IKcos(j2array[0]);
if( j2array[0] > IKPI )
{
    j2array[0]-=IK2PI;
}
else if( j2array[0] < -IKPI )
{    j2array[0]+=IK2PI;
}
j2valid[0] = true;
for(int ij2 = 0; ij2 < 1; ++ij2)
{
if( !j2valid[ij2] )
{
    continue;
}
_ij2[0] = ij2; _ij2[1] = -1;
for(int iij2 = ij2+1; iij2 < 1; ++iij2)
{
if( j2valid[iij2] && IKabs(cj2array[ij2]-cj2array[iij2]) < IKFAST_SOLUTION_THRESH && IKabs(sj2array[ij2]-sj2array[iij2]) < IKFAST_SOLUTION_THRESH )
{
    j2valid[iij2]=false; _ij2[1] = iij2; break; 
}
}
j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
{
IkReal evalcond[2];
IkReal x126=IKsin(j2);
IkReal x127=IKcos(j2);
IkReal x128=((cj1)*(r00));
IkReal x129=((px)*(sj1));
IkReal x130=((r01)*(sj1));
IkReal x131=((cj1)*(py));
IkReal x132=((cj1)*(px));
IkReal x133=((cj1)*(r01));
IkReal x134=((py)*(sj1));
IkReal x135=((r00)*(sj1));
IkReal x136=((IkReal(0.145000000000000))*(x126));
IkReal x137=((IkReal(3.09000000000000))*(x126));
IkReal x138=((IkReal(0.290000000000000))*(x126));
IkReal x139=((IkReal(0.290000000000000))*(x127));
IkReal x140=((IkReal(1.54500000000000))*(x127));
IkReal x141=((IkReal(1.54500000000000))*(x126));
IkReal x142=((IkReal(3.09000000000000))*(x127));
IkReal x143=((IkReal(0.145000000000000))*(x127));
evalcond[0]=((IkReal(-0.842514000000000))+(((IkReal(-1.00000000000000))*(x134)*(x142)))+(((x129)*(x137)))+(((x129)*(x139)))+(((IkReal(-1.00000000000000))*(x132)*(x142)))+(((IkReal(3.70800000000000))*(x126)))+(((IkReal(0.348000000000000))*(x127)))+(((IkReal(2.40000000000000))*(x129)))+(((x134)*(x138)))+(pp)+(((IkReal(-2.40000000000000))*(x131)))+(((x132)*(x138)))+(((IkReal(-1.00000000000000))*(x131)*(x139)))+(((IkReal(-1.00000000000000))*(x131)*(x137))));
evalcond[1]=((IkReal(2.16000000000000))+(((IkReal(-1.00000000000000))*(x128)*(x140)))+(((px)*(r00)))+(((IkReal(-1.00000000000000))*(x133)*(x143)))+(((IkReal(-1.00000000000000))*(x133)*(x141)))+(((x135)*(x141)))+(((x135)*(x143)))+(((IkReal(-1.20000000000000))*(x133)))+(((x130)*(x136)))+(((x128)*(x136)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(1.20000000000000))*(x135)))+(((IkReal(-1.00000000000000))*(x130)*(x140))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j4array[2], cj4array[2], sj4array[2];
bool j4valid[2]={false};
_nj4 = 2;
IkReal x144=((IkReal(13.6708860759494))*(cj2));
IkReal x145=((IkReal(6.32911392405063))*(cj1));
IkReal x146=((IkReal(6.32911392405063))*(sj1));
IkReal x147=((IkReal(13.6708860759494))*(sj2));
cj4array[0]=((IkReal(-9.77848101265823))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(x147)))+(((cj1)*(r00)*(x144)))+(((cj1)*(r01)*(x147)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x146)))+(((cj2)*(py)*(x146)))+(((py)*(sj2)*(x145)))+(((cj2)*(px)*(x145)))+(((IkReal(-7.59493670886076))*(sj2)))+(((r01)*(sj1)*(x144))));
if( cj4array[0] >= -1-IKFAST_SINCOS_THRESH && cj4array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j4valid[0] = j4valid[1] = true;
    j4array[0] = IKacos(cj4array[0]);
    sj4array[0] = IKsin(j4array[0]);
    cj4array[1] = cj4array[0];
    j4array[1] = -j4array[0];
    sj4array[1] = -sj4array[0];
}
else if( isnan(cj4array[0]) )
{
    // probably any value will work
    j4valid[0] = true;
    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
}
for(int ij4 = 0; ij4 < 2; ++ij4)
{
if( !j4valid[ij4] )
{
    continue;
}
_ij4[0] = ij4; _ij4[1] = -1;
for(int iij4 = ij4+1; iij4 < 2; ++iij4)
{
if( j4valid[iij4] && IKabs(cj4array[ij4]-cj4array[iij4]) < IKFAST_SOLUTION_THRESH && IKabs(sj4array[ij4]-sj4array[iij4]) < IKFAST_SOLUTION_THRESH )
{
    j4valid[iij4]=false; _ij4[1] = iij4; break; 
}
}
j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];

{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(sj4);
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x148=((IkReal(1.00000000000000))*(r01));
IkReal x149=((cj1)*(sj2));
IkReal x150=((sj1)*(sj2));
IkReal x151=((cj2)*(sj1));
IkReal x152=((IkReal(1.00000000000000))*(py));
IkReal x153=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x150)))+(((IkReal(-1.00000000000000))*(r00)*(x153)))+(((IkReal(-1.00000000000000))*(x148)*(x149)))+(((IkReal(-1.00000000000000))*(x148)*(x151))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x151)*(x152)))+(((IkReal(-1.00000000000000))*(px)*(x153)))+(((px)*(x150)))+(((IkReal(-1.00000000000000))*(x149)*(x152))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst61;
gconst61=((IkReal(-1.00000000000000))*(r02));
IkReal gconst62;
gconst62=IkReal(-2.00000000000000);
IkReal gconst63;
gconst63=((IkReal(-1.00000000000000))*(r02));
IkReal gconst64;
gconst64=((IkReal(-1.00000000000000))*(pz));
IkReal gconst65;
gconst65=IkReal(4.32000000000000);
IkReal gconst66;
gconst66=((IkReal(-1.00000000000000))*(pz));
IkReal gconst67;
gconst67=((IkReal(-1.00000000000000))*(r02));
IkReal gconst68;
gconst68=IkReal(-2.00000000000000);
IkReal gconst69;
gconst69=((IkReal(-1.00000000000000))*(r02));
IkReal gconst70;
gconst70=((IkReal(-1.00000000000000))*(pz));
IkReal gconst71;
gconst71=IkReal(4.32000000000000);
IkReal gconst72;
gconst72=((IkReal(-1.00000000000000))*(pz));
IkReal gconst73;
gconst73=IkReal(-2.00000000000000);
IkReal gconst74;
gconst74=IkReal(2.00000000000000);
IkReal gconst75;
gconst75=IkReal(4.32000000000000);
IkReal gconst76;
gconst76=IkReal(-4.32000000000000);
IkReal gconst77;
gconst77=IkReal(-2.00000000000000);
IkReal gconst78;
gconst78=IkReal(2.00000000000000);
IkReal gconst79;
gconst79=IkReal(4.32000000000000);
IkReal gconst80;
gconst80=IkReal(-4.32000000000000);
IkReal gconst81;
gconst81=((IkReal(-1.00000000000000))*(r02));
IkReal gconst82;
gconst82=IkReal(2.00000000000000);
IkReal gconst83;
gconst83=((IkReal(-1.00000000000000))*(r02));
IkReal gconst84;
gconst84=((IkReal(-1.00000000000000))*(pz));
IkReal gconst85;
gconst85=IkReal(-4.32000000000000);
IkReal gconst86;
gconst86=((IkReal(-1.00000000000000))*(pz));
IkReal gconst87;
gconst87=((IkReal(-1.00000000000000))*(r02));
IkReal gconst88;
gconst88=IkReal(2.00000000000000);
IkReal gconst89;
gconst89=((IkReal(-1.00000000000000))*(r02));
IkReal gconst90;
gconst90=((IkReal(-1.00000000000000))*(pz));
IkReal gconst91;
gconst91=IkReal(-4.32000000000000);
IkReal gconst92;
gconst92=((IkReal(-1.00000000000000))*(pz));
IkReal x154=((gconst89)*(gconst90));
IkReal x155=((gconst83)*(gconst84));
IkReal x156=((gconst81)*(gconst86));
IkReal x157=((gconst87)*(gconst92));
IkReal x158=((gconst82)*(gconst86));
IkReal x159=((gconst87)*(gconst91));
IkReal x160=((gconst88)*(gconst90));
IkReal x161=((gconst83)*(gconst85));
dummyeval[0]=((((x155)*(x157)))+(((x154)*(x156)))+(((IkReal(-1.00000000000000))*(x158)*(x160)))+(((IkReal(-1.00000000000000))*(x156)*(x157)))+(((IkReal(-1.00000000000000))*(x159)*(x161)))+(((x158)*(x159)))+(((IkReal(-1.00000000000000))*(x154)*(x155)))+(((x160)*(x161))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x162=((gconst63)*(gconst67));
IkReal x163=((IkReal(1.00000000000000))*(gconst65));
IkReal x164=((IkReal(1.00000000000000))*(gconst86));
IkReal x165=((gconst80)*(gconst87));
IkReal x166=((gconst68)*(gconst82));
IkReal x167=((gconst72)*(gconst75));
IkReal x168=((gconst72)*(gconst77));
IkReal x169=((gconst61)*(gconst76));
IkReal x170=((gconst63)*(gconst87));
IkReal x171=((gconst61)*(gconst66));
IkReal x172=((gconst70)*(gconst78));
IkReal x173=((gconst86)*(gconst89));
IkReal x174=((gconst73)*(gconst90));
IkReal x175=((gconst64)*(gconst72));
IkReal x176=((gconst87)*(gconst92));
IkReal x177=((IkReal(1.00000000000000))*(gconst83));
IkReal x178=((gconst85)*(gconst91));
IkReal x179=((gconst69)*(gconst90));
IkReal x180=((gconst78)*(gconst90));
IkReal x181=((IkReal(1.00000000000000))*(gconst70));
IkReal x182=((gconst62)*(gconst66));
IkReal x183=((gconst62)*(gconst86));
IkReal x184=((gconst67)*(gconst71));
IkReal x185=((gconst72)*(gconst84));
IkReal x186=((gconst65)*(gconst68));
IkReal x187=((gconst74)*(gconst79));
IkReal x188=((gconst63)*(gconst69));
IkReal x189=((IkReal(1.00000000000000))*(gconst67));
IkReal x190=((gconst66)*(gconst73));
IkReal x191=((gconst70)*(gconst81));
IkReal x192=((gconst63)*(gconst70));
IkReal x193=((gconst79)*(gconst81));
IkReal x194=((gconst78)*(gconst86));
IkReal x195=((gconst84)*(gconst92));
IkReal x196=((IkReal(1.00000000000000))*(gconst79));
IkReal x197=((gconst83)*(gconst84));
IkReal x198=((gconst75)*(gconst83));
IkReal x199=((gconst64)*(gconst74));
IkReal x200=((gconst81)*(gconst92));
IkReal x201=((gconst63)*(gconst64));
IkReal x202=((gconst76)*(gconst89));
IkReal x203=((gconst65)*(gconst88));
IkReal x204=((gconst79)*(gconst84));
IkReal x205=((gconst87)*(gconst91));
IkReal x206=((gconst81)*(gconst86));
IkReal x207=((gconst69)*(gconst79));
IkReal x208=((gconst67)*(gconst83));
IkReal x209=((gconst82)*(gconst88));
IkReal x210=((gconst69)*(gconst70));
IkReal x211=((gconst67)*(gconst74));
IkReal x212=((gconst70)*(gconst89));
IkReal x213=((IkReal(1.00000000000000))*(gconst77));
IkReal x214=((IkReal(1.00000000000000))*(gconst81));
IkReal x215=((gconst68)*(gconst85));
IkReal x216=((gconst83)*(gconst90));
IkReal x217=((gconst62)*(gconst68));
IkReal x218=((gconst82)*(gconst86));
IkReal x219=((gconst67)*(gconst91));
IkReal x220=((IkReal(1.00000000000000))*(gconst63));
IkReal x221=((gconst85)*(gconst88));
IkReal x222=((gconst74)*(gconst84));
IkReal x223=((gconst89)*(gconst90));
IkReal x224=((gconst72)*(gconst87));
IkReal x225=((gconst73)*(gconst76));
IkReal x226=((gconst66)*(gconst81));
IkReal x227=((gconst63)*(gconst90));
IkReal x228=((gconst63)*(gconst77));
IkReal x229=((gconst74)*(gconst75));
IkReal x230=((gconst66)*(gconst82));
IkReal x231=((gconst71)*(gconst87));
IkReal x232=((IkReal(1.00000000000000))*(gconst90));
IkReal x233=((gconst71)*(gconst77));
IkReal x234=((gconst77)*(gconst80));
IkReal x235=((gconst76)*(gconst82));
IkReal x236=((gconst77)*(gconst91));
IkReal x237=((gconst70)*(gconst83));
IkReal x238=((gconst62)*(gconst76));
IkReal x239=((gconst81)*(gconst90));
IkReal x240=((gconst77)*(gconst92));
IkReal x241=((gconst74)*(x221));
op[0]=((((x176)*(x197)))+(((x216)*(x221)))+(((IkReal(-1.00000000000000))*(gconst84)*(x177)*(x223)))+(((IkReal(-1.00000000000000))*(gconst87)*(x177)*(x178)))+(((x205)*(x218)))+(((IkReal(-1.00000000000000))*(gconst90)*(x164)*(x209)))+(((IkReal(-1.00000000000000))*(gconst81)*(x164)*(x176)))+(((x173)*(x239))));
op[1]=((((x176)*(x198)))+(((x205)*(x235)))+(((IkReal(-1.00000000000000))*(gconst77)*(x177)*(x178)))+(((IkReal(-1.00000000000000))*(gconst73)*(x164)*(x176)))+(((IkReal(-1.00000000000000))*(gconst77)*(x164)*(x200)))+(((x180)*(x206)))+(((gconst79)*(gconst83)*(x221)))+(((x173)*(x193)))+(((x218)*(x236)))+(((x173)*(x174)))+(((gconst90)*(x241)))+(((x165)*(x197)))+(((IkReal(-1.00000000000000))*(gconst79)*(x164)*(x209)))+(((IkReal(-1.00000000000000))*(x222)*(x223)))+(((gconst77)*(gconst83)*(x195)))+(((x176)*(x222)))+(((IkReal(-1.00000000000000))*(gconst84)*(x177)*(x180)))+(((IkReal(-1.00000000000000))*(gconst81)*(x164)*(x165)))+(((IkReal(-1.00000000000000))*(gconst75)*(x177)*(x223)))+(((x202)*(x239)))+(((IkReal(-1.00000000000000))*(gconst76)*(x209)*(x232)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst87)*(x178)))+(((IkReal(-1.00000000000000))*(gconst89)*(x177)*(x204)))+(((IkReal(-1.00000000000000))*(gconst76)*(x176)*(x214))));
op[2]=((((IkReal(-1.00000000000000))*(x170)*(x178)))+(((IkReal(-1.00000000000000))*(gconst66)*(x176)*(x214)))+(((IkReal(-1.00000000000000))*(gconst84)*(x177)*(x212)))+(((x218)*(x219)))+(((x205)*(x230)))+(((IkReal(-1.00000000000000))*(gconst73)*(x164)*(x165)))+(((x165)*(x222)))+(((gconst76)*(gconst81)*(x180)))+(((x193)*(x194)))+(((IkReal(-1.00000000000000))*(gconst61)*(x164)*(x176)))+(((IkReal(-1.00000000000000))*(gconst84)*(x220)*(x223)))+(((IkReal(-1.00000000000000))*(gconst76)*(x165)*(x214)))+(((gconst73)*(gconst79)*(x173)))+(((x173)*(x191)))+(((x195)*(x208)))+(((x203)*(x216)))+(((IkReal(-1.00000000000000))*(gconst84)*(x177)*(x179)))+(((x218)*(x231)))+(((IkReal(-1.00000000000000))*(gconst78)*(x177)*(x204)))+(((x215)*(x216)))+(((IkReal(-1.00000000000000))*(gconst76)*(x196)*(x209)))+(((IkReal(-1.00000000000000))*(gconst85)*(x177)*(x231)))+(((IkReal(-1.00000000000000))*(gconst64)*(x177)*(x223)))+(((x187)*(x221)))+(((x174)*(x194)))+(((IkReal(-1.00000000000000))*(gconst67)*(x177)*(x178)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst88)*(gconst90)*(x164)))+(((x223)*(x226)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst79)*(gconst89)*(x177)))+(((IkReal(-1.00000000000000))*(gconst74)*(x178)*(x213)))+(((gconst61)*(gconst90)*(x173)))+(((IkReal(-1.00000000000000))*(gconst83)*(x163)*(x205)))+(((gconst74)*(gconst77)*(x195)))+(((x235)*(x236)))+(((IkReal(-1.00000000000000))*(gconst76)*(x200)*(x213)))+(((x165)*(x198)))+(((x197)*(x234)))+(((x179)*(x206)))+(((IkReal(-1.00000000000000))*(gconst75)*(x177)*(x180)))+(((x176)*(x229)))+(((IkReal(-1.00000000000000))*(gconst73)*(x164)*(x240)))+(((x221)*(x237)))+(((IkReal(-1.00000000000000))*(gconst81)*(x164)*(x224)))+(((x221)*(x227)))+(((x170)*(x195)))+(((IkReal(-1.00000000000000))*(gconst67)*(x164)*(x200)))+(((IkReal(-1.00000000000000))*(gconst81)*(x164)*(x234)))+(((IkReal(-1.00000000000000))*(gconst66)*(x209)*(x232)))+(((x183)*(x205)))+(((x198)*(x240)))+(((gconst64)*(gconst83)*(x176)))+(((IkReal(-1.00000000000000))*(gconst90)*(x164)*(x166)))+(((IkReal(-1.00000000000000))*(gconst84)*(gconst89)*(x187)))+(((gconst83)*(gconst87)*(x185)))+(((x193)*(x202)))+(((IkReal(-1.00000000000000))*(x223)*(x229)))+(((IkReal(-1.00000000000000))*(gconst70)*(x164)*(x209)))+(((IkReal(-1.00000000000000))*(x176)*(x225)))+(((x174)*(x202)))+(((IkReal(-1.00000000000000))*(x180)*(x222))));
op[3]=((((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst80)*(gconst87)))+(((gconst75)*(gconst77)*(gconst80)*(gconst83)))+(((gconst71)*(gconst77)*(gconst82)*(gconst86)))+(((gconst65)*(gconst74)*(gconst88)*(gconst90)))+(((gconst67)*(gconst80)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst89)*(gconst90)))+(((gconst61)*(gconst78)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst79)*(gconst82)*(gconst86)))+(((gconst61)*(gconst79)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst77)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst81)*(gconst92)))+(((gconst63)*(gconst79)*(gconst85)*(gconst88)))+(((gconst62)*(gconst76)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst77)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst83)*(gconst90)))+(((gconst64)*(gconst74)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst76)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst80)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst79)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst88)*(gconst90)))+(((gconst71)*(gconst76)*(gconst82)*(gconst87)))+(((gconst70)*(gconst74)*(gconst85)*(gconst88)))+(((gconst74)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst82)*(gconst90)))+(((gconst74)*(gconst75)*(gconst80)*(gconst87)))+(((gconst62)*(gconst77)*(gconst86)*(gconst91)))+(((gconst69)*(gconst76)*(gconst81)*(gconst90)))+(((gconst66)*(gconst79)*(gconst81)*(gconst89)))+(((gconst69)*(gconst73)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst80)*(gconst86)*(gconst87)))+(((gconst66)*(gconst77)*(gconst82)*(gconst91)))+(((gconst70)*(gconst73)*(gconst86)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst76)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst79)*(gconst86)*(gconst88)))+(((gconst74)*(gconst77)*(gconst80)*(gconst84)))+(((gconst70)*(gconst78)*(gconst81)*(gconst86)))+(((gconst63)*(gconst75)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst79)*(gconst83)*(gconst89)))+(((gconst68)*(gconst74)*(gconst85)*(gconst90)))+(((gconst64)*(gconst77)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst92)))+(((gconst65)*(gconst79)*(gconst83)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst87)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst79)*(gconst82)*(gconst88)))+(((gconst64)*(gconst80)*(gconst83)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst84)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst86)*(gconst87)))+(((gconst72)*(gconst77)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst86)*(gconst92)))+(((gconst69)*(gconst79)*(gconst81)*(gconst86)))+(((gconst67)*(gconst76)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst83)*(gconst90)))+(((gconst61)*(gconst76)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst79)*(gconst84)*(gconst89)))+(((gconst72)*(gconst75)*(gconst83)*(gconst87)))+(((gconst63)*(gconst80)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst75)*(gconst78)*(gconst79)*(gconst83)))+(((gconst63)*(gconst77)*(gconst84)*(gconst92)))+(((gconst73)*(gconst78)*(gconst79)*(gconst86)))+(((gconst70)*(gconst76)*(gconst81)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst78)*(gconst83)*(gconst84)))+(((gconst66)*(gconst73)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst83)*(gconst89)))+(((gconst68)*(gconst79)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst74)*(gconst85)*(gconst91)))+(((gconst73)*(gconst76)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst77)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst77)*(gconst83)*(gconst91)))+(((gconst67)*(gconst74)*(gconst84)*(gconst92)))+(((gconst72)*(gconst74)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst84)*(gconst89)))+(((gconst76)*(gconst78)*(gconst79)*(gconst81)))+(((gconst66)*(gconst78)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst87)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst87)*(gconst92)))+(((gconst73)*(gconst76)*(gconst79)*(gconst89)))+(((gconst67)*(gconst75)*(gconst83)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst80)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst78)*(gconst79)*(gconst84))));
op[4]=((((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst83)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst83)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst77)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst87)*(gconst92)))+(((gconst64)*(gconst72)*(gconst83)*(gconst87)))+(((gconst64)*(gconst74)*(gconst80)*(gconst87)))+(((gconst63)*(gconst70)*(gconst85)*(gconst88)))+(((gconst68)*(gconst70)*(gconst83)*(gconst85)))+(((gconst67)*(gconst74)*(gconst80)*(gconst84)))+(((gconst73)*(gconst76)*(gconst78)*(gconst79)))+(((gconst64)*(gconst67)*(gconst83)*(gconst92)))+(((gconst67)*(gconst72)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst86)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst70)*(gconst82)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst78)*(gconst79)*(gconst83)))+(((gconst64)*(gconst74)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst75)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst67)*(gconst85)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst90)))+(((gconst71)*(gconst76)*(gconst77)*(gconst82)))+(((gconst70)*(gconst73)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst84)*(gconst89)))+(((gconst61)*(gconst66)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst75)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst76)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst92)))+(((gconst62)*(gconst76)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst78)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst76)*(gconst77)*(gconst81)))+(((gconst61)*(gconst69)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst73)*(gconst76)*(gconst77)*(gconst80)))+(((gconst66)*(gconst70)*(gconst81)*(gconst89)))+(((gconst61)*(gconst70)*(gconst86)*(gconst89)))+(((gconst63)*(gconst64)*(gconst87)*(gconst92)))+(((gconst61)*(gconst78)*(gconst79)*(gconst86)))+(((gconst68)*(gconst74)*(gconst79)*(gconst85)))+(((gconst67)*(gconst75)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst86)*(gconst87)))+(((gconst66)*(gconst69)*(gconst81)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst77)*(gconst80)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst74)*(gconst77)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst74)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst74)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst86)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst83)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst76)*(gconst79)*(gconst88)))+(((gconst62)*(gconst66)*(gconst87)*(gconst91)))+(((gconst70)*(gconst76)*(gconst78)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst82)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst79)*(gconst84)))+(((gconst69)*(gconst76)*(gconst79)*(gconst81)))+(((gconst66)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst82)*(gconst86)))+(((gconst65)*(gconst70)*(gconst83)*(gconst88)))+(((gconst67)*(gconst71)*(gconst82)*(gconst86)))+(((gconst63)*(gconst65)*(gconst88)*(gconst90)))+(((gconst63)*(gconst75)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst87)*(gconst91)))+(((gconst63)*(gconst68)*(gconst85)*(gconst90)))+(((gconst70)*(gconst73)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst89)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst87)))+(((gconst61)*(gconst76)*(gconst78)*(gconst90)))+(((gconst72)*(gconst74)*(gconst75)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst81)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst84)*(gconst90)))+(((gconst62)*(gconst71)*(gconst86)*(gconst87)))+(((gconst69)*(gconst73)*(gconst79)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst76)*(gconst80)*(gconst81)))+(((gconst72)*(gconst75)*(gconst77)*(gconst83)))+(((gconst69)*(gconst70)*(gconst81)*(gconst86)))+(((gconst65)*(gconst68)*(gconst83)*(gconst90)))+(((gconst63)*(gconst67)*(gconst84)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst92)))+(((gconst74)*(gconst75)*(gconst77)*(gconst80)))+(((gconst63)*(gconst75)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst80)*(gconst86)))+(((gconst66)*(gconst73)*(gconst78)*(gconst90)))+(((gconst66)*(gconst73)*(gconst79)*(gconst89)))+(((gconst61)*(gconst76)*(gconst79)*(gconst89)))+(((gconst72)*(gconst74)*(gconst77)*(gconst84)))+(((gconst66)*(gconst78)*(gconst79)*(gconst81)))+(((gconst65)*(gconst74)*(gconst79)*(gconst88)))+(((gconst64)*(gconst77)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst88)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst83)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst79)*(gconst89)))+(((gconst66)*(gconst67)*(gconst82)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst79)*(gconst89)))+(((gconst62)*(gconst67)*(gconst86)*(gconst91)))+(((gconst69)*(gconst73)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst77)*(gconst86)))+(((gconst63)*(gconst77)*(gconst80)*(gconst84)))+(((gconst63)*(gconst72)*(gconst84)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst85)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst86)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst83)*(gconst87))));
op[5]=((((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst79)*(gconst88)))+(((gconst64)*(gconst72)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst91)))+(((gconst61)*(gconst66)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst66)*(gconst70)*(gconst78)*(gconst81)))+(((gconst66)*(gconst73)*(gconst78)*(gconst79)))+(((gconst63)*(gconst75)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst75)*(gconst78)*(gconst79)))+(((gconst63)*(gconst72)*(gconst75)*(gconst87)))+(((gconst63)*(gconst67)*(gconst80)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst80)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst92)))+(((gconst69)*(gconst70)*(gconst76)*(gconst81)))+(((gconst64)*(gconst74)*(gconst77)*(gconst80)))+(((gconst61)*(gconst70)*(gconst76)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((gconst68)*(gconst70)*(gconst74)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst79)*(gconst84)))+(((gconst63)*(gconst64)*(gconst77)*(gconst92)))+(((gconst63)*(gconst64)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst80)*(gconst86)))+(((gconst63)*(gconst72)*(gconst77)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst70)*(gconst76)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst73)*(gconst77)*(gconst80)))+(((gconst69)*(gconst70)*(gconst73)*(gconst86)))+(((gconst65)*(gconst68)*(gconst79)*(gconst83)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst65)*(gconst68)*(gconst74)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst77)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst70)*(gconst76)*(gconst82)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((gconst66)*(gconst70)*(gconst73)*(gconst89)))+(((gconst72)*(gconst74)*(gconst75)*(gconst77)))+(((gconst69)*(gconst73)*(gconst76)*(gconst79)))+(((gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst68)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst74)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst78)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst62)*(gconst66)*(gconst77)*(gconst91)))+(((gconst62)*(gconst67)*(gconst76)*(gconst91)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst79)*(gconst89)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst74)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst72)*(gconst73)*(gconst76)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst64)*(gconst78)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst66)*(gconst80)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst67)*(gconst74)*(gconst91)))+(((gconst65)*(gconst70)*(gconst74)*(gconst88)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst73)*(gconst76)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((gconst61)*(gconst70)*(gconst78)*(gconst86)))+(((gconst61)*(gconst69)*(gconst76)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst70)*(gconst75)*(gconst89)))+(((gconst64)*(gconst67)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst79)*(gconst86)))+(((gconst61)*(gconst66)*(gconst79)*(gconst89)))+(((gconst67)*(gconst72)*(gconst74)*(gconst84)))+(((gconst63)*(gconst68)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst61)*(gconst69)*(gconst79)*(gconst86)))+(((gconst61)*(gconst76)*(gconst78)*(gconst79)))+(((gconst67)*(gconst72)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst67)*(gconst73)*(gconst92)))+(((gconst67)*(gconst74)*(gconst75)*(gconst80)))+(((gconst66)*(gconst69)*(gconst79)*(gconst81)))+(((gconst63)*(gconst67)*(gconst75)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst73)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst75)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst74)*(gconst85)))+(((gconst70)*(gconst73)*(gconst76)*(gconst78)))+(((gconst63)*(gconst65)*(gconst79)*(gconst88)))+(((gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst64)*(gconst67)*(gconst74)*(gconst92)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst73)*(gconst87)))+(((gconst64)*(gconst72)*(gconst74)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst69)*(gconst75)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst90)))+(((gconst66)*(gconst69)*(gconst73)*(gconst90)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst76)*(gconst77)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst74)*(gconst87))));
op[6]=((((x186)*(x237)))+(((x175)*(x208)))+(((x210)*(x225)))+(((x186)*(x187)))+(((x168)*(x199)))+(((IkReal(-1.00000000000000))*(gconst66)*(x166)*(x181)))+(((IkReal(-1.00000000000000))*(gconst80)*(x189)*(x190)))+(((x162)*(x185)))+(((IkReal(-1.00000000000000))*(gconst80)*(x169)*(x189)))+(((x201)*(x234)))+(((x192)*(x203)))+(((gconst75)*(gconst80)*(x162)))+(((x172)*(x190)))+(((x167)*(x228)))+(((x169)*(x172)))+(((gconst67)*(gconst80)*(x199)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(x187)))+(((IkReal(-1.00000000000000))*(x168)*(x190)))+(((IkReal(-1.00000000000000))*(gconst76)*(x196)*(x217)))+(((IkReal(-1.00000000000000))*(gconst75)*(x188)*(x196)))+(((x182)*(x219)))+(((x233)*(x238)))+(((IkReal(-1.00000000000000))*(gconst88)*(x181)*(x182)))+(((IkReal(-1.00000000000000))*(gconst74)*(x163)*(x233)))+(((x190)*(x207)))+(((x171)*(x212)))+(((x184)*(x230)))+(((IkReal(-1.00000000000000))*(gconst84)*(x181)*(x188)))+(((gconst64)*(gconst92)*(x162)))+(((IkReal(-1.00000000000000))*(x168)*(x169)))+(((x167)*(x211)))+(((IkReal(-1.00000000000000))*(gconst89)*(x181)*(x201)))+(((IkReal(-1.00000000000000))*(gconst69)*(x181)*(x229)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst85)*(x162)))+(((x186)*(x227)))+(((IkReal(-1.00000000000000))*(gconst83)*(x163)*(x184)))+(((IkReal(-1.00000000000000))*(gconst71)*(x163)*(x170)))+(((IkReal(-1.00000000000000))*(gconst75)*(x172)*(x220)))+(((IkReal(-1.00000000000000))*(x171)*(x224)))+(((x169)*(x207)))+(((IkReal(-1.00000000000000))*(gconst70)*(x164)*(x217)))+(((x171)*(x179)))+(((gconst66)*(gconst69)*(x191)))+(((gconst61)*(gconst86)*(x210)))+(((IkReal(-1.00000000000000))*(x172)*(x199)))+(((x170)*(x175)))+(((IkReal(-1.00000000000000))*(gconst72)*(x189)*(x226)))+(((IkReal(-1.00000000000000))*(gconst72)*(x189)*(x225)))+(((IkReal(-1.00000000000000))*(gconst92)*(x171)*(x189)))+(((x182)*(x231)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(x164)))+(((x192)*(x215)))+(((gconst78)*(gconst79)*(x171)))+(((IkReal(-1.00000000000000))*(x179)*(x201)))+(((x183)*(x184)))+(((IkReal(-1.00000000000000))*(gconst68)*(x182)*(x232)))+(((IkReal(-1.00000000000000))*(gconst64)*(x177)*(x210)))+(((IkReal(-1.00000000000000))*(gconst78)*(x196)*(x201)))+(((IkReal(-1.00000000000000))*(gconst80)*(x171)*(x213)))+(((IkReal(-1.00000000000000))*(gconst91)*(x162)*(x163))));
op[7]=((((x190)*(x210)))+(((IkReal(-1.00000000000000))*(gconst72)*(x169)*(x189)))+(((gconst63)*(gconst79)*(x186)))+(((x169)*(x210)))+(((x171)*(x207)))+(((gconst70)*(gconst74)*(x186)))+(((IkReal(-1.00000000000000))*(x172)*(x201)))+(((IkReal(-1.00000000000000))*(gconst76)*(x181)*(x217)))+(((x162)*(x167)))+(((IkReal(-1.00000000000000))*(gconst72)*(x189)*(x190)))+(((x184)*(x238)))+(((IkReal(-1.00000000000000))*(gconst64)*(x188)*(x196)))+(((IkReal(-1.00000000000000))*(x168)*(x171)))+(((gconst64)*(gconst80)*(x162)))+(((IkReal(-1.00000000000000))*(gconst68)*(x182)*(x196)))+(((x171)*(x172)))+(((x168)*(x201)))+(((IkReal(-1.00000000000000))*(gconst74)*(x163)*(x184)))+(((x175)*(x211)))+(((IkReal(-1.00000000000000))*(gconst71)*(x163)*(x228)))+(((x182)*(x233)))+(((IkReal(-1.00000000000000))*(gconst69)*(x181)*(x199)))+(((IkReal(-1.00000000000000))*(gconst75)*(x181)*(x188)))+(((IkReal(-1.00000000000000))*(gconst80)*(x171)*(x189))));
op[8]=((((x186)*(x192)))+(((x162)*(x175)))+(((IkReal(-1.00000000000000))*(gconst71)*(x162)*(x163)))+(((x171)*(x210)))+(((IkReal(-1.00000000000000))*(gconst68)*(x181)*(x182)))+(((IkReal(-1.00000000000000))*(gconst72)*(x171)*(x189)))+(((IkReal(-1.00000000000000))*(gconst64)*(x181)*(x188)))+(((x182)*(x184))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst109;
gconst109=IKsign(((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x242=((r01)*(sj1));
IkReal x243=((cj1)*(r00));
IkReal x244=((IkReal(1.00000000000000))*(cj2));
IkReal x245=((cj1)*(r01));
IkReal x246=((r00)*(sj1));
IkReal x247=((cj3)*(sj2));
IkReal x248=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst109)*(((((IkReal(-1.00000000000000))*(x242)*(x248)))+(((IkReal(-1.00000000000000))*(x243)*(x248)))+(((cj3)*(r02)))+(((cj2)*(sj3)*(x245)))+(((IkReal(-1.00000000000000))*(sj3)*(x244)*(x246))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst109)*(((((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x244)*(x245)))+(((x242)*(x247)))+(((cj2)*(cj3)*(x246)))+(((x243)*(x247))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst109)*(((((IkReal(-1.00000000000000))*(x242)*(x248)))+(((IkReal(-1.00000000000000))*(x243)*(x248)))+(((cj3)*(r02)))+(((cj2)*(sj3)*(x245)))+(((IkReal(-1.00000000000000))*(sj3)*(x244)*(x246)))))), ((gconst109)*(((((r02)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x244)*(x245)))+(((x242)*(x247)))+(((cj2)*(cj3)*(x246)))+(((x243)*(x247)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x249=IKsin(j5);
IkReal x250=IKcos(j5);
IkReal x251=((IkReal(2.16000000000000))*(sj3));
IkReal x252=((IkReal(1.00000000000000))*(px));
IkReal x253=((cj2)*(sj1));
IkReal x254=((cj1)*(cj2));
IkReal x255=((cj1)*(sj2));
IkReal x256=((IkReal(1.00000000000000))*(r00));
IkReal x257=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x258=((cj3)*(x249));
IkReal x259=((IkReal(1.00000000000000))*(x250));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(sj3)*(x259)))+(((IkReal(-1.00000000000000))*(x258))));
evalcond[1]=((((IkReal(2.16000000000000))*(x258)))+(((x250)*(x251)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x255)*(x256)))+(((sj3)*(x249)))+(((r01)*(x254)))+(((IkReal(-1.00000000000000))*(r01)*(x257)))+(((IkReal(-1.00000000000000))*(cj3)*(x259)))+(((IkReal(-1.00000000000000))*(x253)*(x256))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(2.16000000000000))*(cj3)*(x250)))+(((IkReal(-1.00000000000000))*(x252)*(x255)))+(((IkReal(-1.00000000000000))*(x252)*(x253)))+(((py)*(x254)))+(((IkReal(-1.00000000000000))*(py)*(x257)))+(((IkReal(-1.00000000000000))*(x249)*(x251)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x260=((IkReal(1.00000000000000))*(r01));
IkReal x261=((cj1)*(sj2));
IkReal x262=((sj1)*(sj2));
IkReal x263=((cj2)*(sj1));
IkReal x264=((IkReal(1.00000000000000))*(py));
IkReal x265=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x265)))+(((r00)*(x262)))+(((IkReal(-1.00000000000000))*(x260)*(x261)))+(((IkReal(-1.00000000000000))*(x260)*(x263))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(-1.00000000000000))*(x263)*(x264)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x261)*(x264)))+(((IkReal(-1.00000000000000))*(px)*(x265)))+(((px)*(x262))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst110;
gconst110=((IkReal(-1.00000000000000))*(r02));
IkReal gconst111;
gconst111=IkReal(-2.00000000000000);
IkReal gconst112;
gconst112=((IkReal(-1.00000000000000))*(r02));
IkReal gconst113;
gconst113=((IkReal(-1.00000000000000))*(pz));
IkReal gconst114;
gconst114=IkReal(4.32000000000000);
IkReal gconst115;
gconst115=((IkReal(-1.00000000000000))*(pz));
IkReal gconst116;
gconst116=((IkReal(-1.00000000000000))*(r02));
IkReal gconst117;
gconst117=IkReal(-2.00000000000000);
IkReal gconst118;
gconst118=((IkReal(-1.00000000000000))*(r02));
IkReal gconst119;
gconst119=((IkReal(-1.00000000000000))*(pz));
IkReal gconst120;
gconst120=IkReal(4.32000000000000);
IkReal gconst121;
gconst121=((IkReal(-1.00000000000000))*(pz));
IkReal gconst122;
gconst122=IkReal(2.00000000000000);
IkReal gconst123;
gconst123=IkReal(-2.00000000000000);
IkReal gconst124;
gconst124=IkReal(-4.32000000000000);
IkReal gconst125;
gconst125=IkReal(4.32000000000000);
IkReal gconst126;
gconst126=IkReal(2.00000000000000);
IkReal gconst127;
gconst127=IkReal(-2.00000000000000);
IkReal gconst128;
gconst128=IkReal(-4.32000000000000);
IkReal gconst129;
gconst129=IkReal(4.32000000000000);
IkReal gconst130;
gconst130=((IkReal(-1.00000000000000))*(r02));
IkReal gconst131;
gconst131=IkReal(2.00000000000000);
IkReal gconst132;
gconst132=((IkReal(-1.00000000000000))*(r02));
IkReal gconst133;
gconst133=((IkReal(-1.00000000000000))*(pz));
IkReal gconst134;
gconst134=IkReal(-4.32000000000000);
IkReal gconst135;
gconst135=((IkReal(-1.00000000000000))*(pz));
IkReal gconst136;
gconst136=((IkReal(-1.00000000000000))*(r02));
IkReal gconst137;
gconst137=IkReal(2.00000000000000);
IkReal gconst138;
gconst138=((IkReal(-1.00000000000000))*(r02));
IkReal gconst139;
gconst139=((IkReal(-1.00000000000000))*(pz));
IkReal gconst140;
gconst140=IkReal(-4.32000000000000);
IkReal gconst141;
gconst141=((IkReal(-1.00000000000000))*(pz));
IkReal x266=((gconst132)*(gconst134));
IkReal x267=((gconst137)*(gconst139));
IkReal x268=((IkReal(1.00000000000000))*(gconst136));
IkReal x269=((gconst130)*(gconst135));
IkReal x270=((gconst132)*(gconst133));
IkReal x271=((gconst138)*(gconst139));
IkReal x272=((gconst131)*(gconst135));
dummyeval[0]=((((x266)*(x267)))+(((gconst136)*(gconst141)*(x270)))+(((gconst136)*(gconst140)*(x272)))+(((IkReal(-1.00000000000000))*(gconst140)*(x266)*(x268)))+(((IkReal(-1.00000000000000))*(gconst141)*(x268)*(x269)))+(((x269)*(x271)))+(((IkReal(-1.00000000000000))*(x267)*(x272)))+(((IkReal(-1.00000000000000))*(x270)*(x271))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x273=((gconst118)*(gconst119));
IkReal x274=((gconst110)*(gconst135));
IkReal x275=((gconst114)*(gconst117));
IkReal x276=((gconst123)*(gconst128));
IkReal x277=((gconst136)*(gconst141));
IkReal x278=((gconst113)*(gconst132));
IkReal x279=((IkReal(1.00000000000000))*(gconst124));
IkReal x280=((IkReal(1.00000000000000))*(gconst135));
IkReal x281=((gconst138)*(gconst139));
IkReal x282=((gconst131)*(gconst139));
IkReal x283=((gconst112)*(gconst116));
IkReal x284=((gconst124)*(gconst129));
IkReal x285=((gconst115)*(gconst122));
IkReal x286=((IkReal(1.00000000000000))*(gconst140));
IkReal x287=((gconst112)*(gconst139));
IkReal x288=((gconst121)*(gconst124));
IkReal x289=((gconst129)*(gconst136));
IkReal x290=((gconst132)*(gconst136));
IkReal x291=((gconst120)*(gconst126));
IkReal x292=((IkReal(1.00000000000000))*(gconst123));
IkReal x293=((gconst131)*(gconst137));
IkReal x294=((gconst116)*(gconst121));
IkReal x295=((IkReal(1.00000000000000))*(gconst116));
IkReal x296=((gconst131)*(gconst135));
IkReal x297=((gconst126)*(gconst140));
IkReal x298=((gconst111)*(gconst115));
IkReal x299=((IkReal(1.00000000000000))*(gconst125));
IkReal x300=((gconst116)*(gconst120));
IkReal x301=((gconst111)*(gconst135));
IkReal x302=((gconst113)*(gconst121));
IkReal x303=((gconst130)*(gconst138));
IkReal x304=((gconst126)*(gconst141));
IkReal x305=((IkReal(1.00000000000000))*(gconst119));
IkReal x306=((gconst132)*(gconst133));
IkReal x307=((gconst112)*(gconst133));
IkReal x308=((gconst110)*(gconst115));
IkReal x309=((gconst127)*(gconst139));
IkReal x310=((gconst123)*(gconst133));
IkReal x311=((IkReal(1.00000000000000))*(gconst120));
IkReal x312=((gconst132)*(gconst134));
IkReal x313=((gconst122)*(gconst125));
IkReal x314=((gconst125)*(gconst128));
IkReal x315=((gconst134)*(gconst137));
IkReal x316=((gconst116)*(gconst123));
IkReal x317=((gconst120)*(gconst136));
IkReal x318=((IkReal(1.00000000000000))*(gconst118));
IkReal x319=((gconst112)*(gconst113));
IkReal x320=((gconst111)*(gconst117));
IkReal x321=((IkReal(1.00000000000000))*(gconst121));
IkReal x322=((gconst124)*(gconst132));
IkReal x323=((gconst137)*(gconst139));
IkReal x324=((gconst114)*(gconst132));
IkReal x325=((gconst112)*(gconst119));
IkReal x326=((IkReal(1.00000000000000))*(gconst128));
IkReal x327=((gconst123)*(gconst134));
IkReal x328=((gconst111)*(gconst125));
IkReal x329=((gconst130)*(gconst135));
IkReal x330=((IkReal(1.00000000000000))*(gconst115));
IkReal x331=((gconst118)*(gconst139));
IkReal x332=((gconst116)*(gconst140));
IkReal x333=((gconst117)*(gconst139));
IkReal x334=((gconst112)*(gconst128));
IkReal x335=((gconst126)*(gconst129));
IkReal x336=((gconst127)*(gconst128));
IkReal x337=((gconst116)*(gconst141));
IkReal x338=((gconst115)*(gconst131));
IkReal x339=((gconst125)*(gconst131));
IkReal x340=((gconst136)*(gconst140));
IkReal x341=((gconst119)*(gconst127));
IkReal x342=((gconst113)*(gconst129));
IkReal x343=((gconst118)*(gconst128));
IkReal x344=((gconst128)*(gconst135));
IkReal x345=((gconst112)*(gconst136));
IkReal x346=((gconst110)*(gconst125));
IkReal x347=((gconst115)*(gconst130));
IkReal x348=((gconst121)*(gconst133));
IkReal x349=((gconst125)*(gconst130));
IkReal x350=((gconst122)*(gconst135));
IkReal x351=((gconst112)*(x302));
IkReal x352=((gconst126)*(gconst130)*(x280));
op[0]=((((IkReal(-1.00000000000000))*(gconst130)*(x277)*(x280)))+(((IkReal(-1.00000000000000))*(gconst137)*(x280)*(x282)))+(((IkReal(-1.00000000000000))*(gconst134)*(x286)*(x290)))+(((x281)*(x329)))+(((x296)*(x340)))+(((IkReal(-1.00000000000000))*(x281)*(x306)))+(((x312)*(x323)))+(((x277)*(x306))));
op[1]=((((IkReal(-1.00000000000000))*(gconst138)*(x306)*(x326)))+(((IkReal(-1.00000000000000))*(gconst128)*(x280)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x304)))+(((IkReal(-1.00000000000000))*(gconst126)*(x286)*(x312)))+(((x281)*(x350)))+(((IkReal(-1.00000000000000))*(gconst137)*(x282)*(x299)))+(((IkReal(-1.00000000000000))*(gconst130)*(x277)*(x299)))+(((IkReal(-1.00000000000000))*(gconst132)*(x279)*(x281)))+(((IkReal(-1.00000000000000))*(gconst136)*(x286)*(x327)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x289)))+(((x309)*(x329)))+(((IkReal(-1.00000000000000))*(x306)*(x309)))+(((x296)*(x297)))+(((x289)*(x306)))+(((x277)*(x322)))+(((IkReal(-1.00000000000000))*(gconst122)*(x277)*(x280)))+(((gconst128)*(gconst137)*(x312)))+(((x304)*(x306)))+(((x303)*(x344)))+(((x281)*(x349)))+(((x339)*(x340)))+(((x277)*(x310)))+(((IkReal(-1.00000000000000))*(gconst133)*(x281)*(x292)))+(((gconst123)*(gconst139)*(x315))));
op[2]=((((x276)*(x315)))+(((x289)*(x310)))+(((gconst123)*(gconst124)*(x277)))+(((x304)*(x310)))+(((x323)*(x324)))+(((x301)*(x340)))+(((IkReal(-1.00000000000000))*(gconst119)*(x280)*(x293)))+(((IkReal(-1.00000000000000))*(gconst130)*(x277)*(x330)))+(((IkReal(-1.00000000000000))*(x274)*(x277)))+(((IkReal(-1.00000000000000))*(gconst117)*(x280)*(x282)))+(((IkReal(-1.00000000000000))*(gconst133)*(x292)*(x309)))+(((x304)*(x322)))+(((IkReal(-1.00000000000000))*(gconst138)*(x305)*(x306)))+(((gconst122)*(gconst138)*(x344)))+(((x296)*(x317)))+(((x274)*(x281)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x335)))+(((IkReal(-1.00000000000000))*(gconst130)*(x280)*(x337)))+(((IkReal(-1.00000000000000))*(gconst123)*(x279)*(x281)))+(((IkReal(-1.00000000000000))*(gconst122)*(x277)*(x299)))+(((IkReal(-1.00000000000000))*(gconst126)*(x286)*(x327)))+(((IkReal(-1.00000000000000))*(gconst128)*(gconst132)*(gconst138)*(x279)))+(((IkReal(-1.00000000000000))*(x281)*(x307)))+(((IkReal(-1.00000000000000))*(gconst134)*(x286)*(x345)))+(((gconst119)*(gconst135)*(x303)))+(((x284)*(x290)))+(((IkReal(-1.00000000000000))*(gconst134)*(x290)*(x311)))+(((IkReal(-1.00000000000000))*(gconst127)*(x306)*(x326)))+(((IkReal(-1.00000000000000))*(x278)*(x281)))+(((IkReal(-1.00000000000000))*(gconst116)*(x286)*(x312)))+(((IkReal(-1.00000000000000))*(gconst130)*(x299)*(x304)))+(((x329)*(x331)))+(((x329)*(x336)))+(((x281)*(x313)))+(((x277)*(x278)))+(((IkReal(-1.00000000000000))*(gconst130)*(x289)*(x299)))+(((IkReal(-1.00000000000000))*(gconst139)*(x306)*(x318)))+(((x309)*(x349)))+(((IkReal(-1.00000000000000))*(gconst122)*(x280)*(x304)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst130)*(gconst136)*(x280)))+(((IkReal(-1.00000000000000))*(gconst111)*(x280)*(x323)))+(((IkReal(-1.00000000000000))*(gconst114)*(x286)*(x290)))+(((gconst119)*(gconst137)*(x312)))+(((x338)*(x340)))+(((x287)*(x315)))+(((x297)*(x339)))+(((x309)*(x350)))+(((x277)*(x307)))+(((x303)*(x314)))+(((x281)*(x347)))+(((x312)*(x333)))+(((x306)*(x337)))+(((x306)*(x335)))+(((x296)*(x332)))+(((IkReal(-1.00000000000000))*(gconst133)*(gconst138)*(x276)))+(((x290)*(x348)))+(((IkReal(-1.00000000000000))*(gconst137)*(x282)*(x330)))+(((IkReal(-1.00000000000000))*(gconst132)*(x279)*(x309)))+(((IkReal(-1.00000000000000))*(gconst122)*(x280)*(x289)))+(((IkReal(-1.00000000000000))*(gconst128)*(x293)*(x299))));
op[3]=((((gconst121)*(gconst124)*(gconst132)*(gconst136)))+(((gconst115)*(gconst122)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst126)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst128)*(gconst131)*(gconst137)))+(((gconst122)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst129)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst126)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst134)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst132)*(gconst139)))+(((gconst120)*(gconst125)*(gconst131)*(gconst136)))+(((gconst113)*(gconst123)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst126)*(gconst130)*(gconst135)))+(((gconst117)*(gconst128)*(gconst132)*(gconst134)))+(((gconst112)*(gconst128)*(gconst134)*(gconst137)))+(((gconst119)*(gconst122)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst127)*(gconst128)*(gconst133)))+(((gconst119)*(gconst125)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst128)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst128)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst127)*(gconst132)*(gconst133)))+(((gconst114)*(gconst128)*(gconst132)*(gconst137)))+(((gconst122)*(gconst125)*(gconst128)*(gconst138)))+(((gconst116)*(gconst123)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst123)*(gconst134)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst138)*(gconst139)))+(((gconst115)*(gconst128)*(gconst130)*(gconst138)))+(((gconst112)*(gconst126)*(gconst133)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst129)*(gconst135)*(gconst136)))+(((gconst123)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst129)*(gconst136)))+(((gconst116)*(gconst125)*(gconst131)*(gconst140)))+(((gconst119)*(gconst127)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst128)*(gconst132)*(gconst133)))+(((gconst124)*(gconst126)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst126)*(gconst129)*(gconst135)))+(((gconst115)*(gconst126)*(gconst131)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst132)*(gconst138)))+(((gconst118)*(gconst128)*(gconst130)*(gconst135)))+(((gconst121)*(gconst126)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst125)*(gconst131)*(gconst137)))+(((gconst121)*(gconst123)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst128)*(gconst133)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst141)))+(((gconst119)*(gconst123)*(gconst134)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst130)*(gconst141)))+(((gconst120)*(gconst126)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst130)*(gconst141)))+(((gconst112)*(gconst124)*(gconst136)*(gconst141)))+(((gconst116)*(gconst129)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst124)*(gconst127)*(gconst128)*(gconst132)))+(((gconst110)*(gconst127)*(gconst135)*(gconst139)))+(((gconst110)*(gconst125)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst131)*(gconst139)))+(((gconst113)*(gconst126)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst133)*(gconst139)))+(((gconst115)*(gconst127)*(gconst130)*(gconst139)))+(((gconst117)*(gconst123)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst135)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst126)*(gconst134)*(gconst140)))+(((gconst122)*(gconst127)*(gconst128)*(gconst135)))+(((gconst112)*(gconst129)*(gconst133)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst136)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst132)*(gconst139)))+(((gconst111)*(gconst125)*(gconst136)*(gconst140)))+(((gconst118)*(gconst125)*(gconst130)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst135)*(gconst136)))+(((gconst113)*(gconst129)*(gconst132)*(gconst136)))+(((gconst118)*(gconst122)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst136)*(gconst141)))+(((gconst123)*(gconst124)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst125)*(gconst126)*(gconst129)*(gconst130)))+(((gconst125)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst129)*(gconst130)*(gconst136)))+(((gconst116)*(gconst124)*(gconst132)*(gconst141)))+(((gconst111)*(gconst126)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst128)*(gconst131)*(gconst135)))+(((gconst110)*(gconst128)*(gconst135)*(gconst138)))+(((gconst123)*(gconst124)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst130)*(gconst136))));
op[4]=((((gconst117)*(gconst123)*(gconst128)*(gconst134)))+(((gconst112)*(gconst116)*(gconst133)*(gconst141)))+(((gconst119)*(gconst122)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst139)))+(((gconst111)*(gconst120)*(gconst135)*(gconst136)))+(((gconst116)*(gconst123)*(gconst124)*(gconst141)))+(((gconst116)*(gconst124)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst128)*(gconst138)))+(((gconst113)*(gconst121)*(gconst132)*(gconst136)))+(((gconst115)*(gconst127)*(gconst128)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst130)*(gconst135)))+(((gconst111)*(gconst125)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst132)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst133)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst123)*(gconst126)*(gconst140)))+(((gconst123)*(gconst124)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst141)))+(((gconst110)*(gconst125)*(gconst128)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst123)*(gconst126)*(gconst134)))+(((gconst115)*(gconst116)*(gconst131)*(gconst140)))+(((gconst111)*(gconst116)*(gconst135)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst132)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst127)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst127)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst136)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst141)))+(((gconst112)*(gconst117)*(gconst134)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst126)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst129)*(gconst136)))+(((gconst113)*(gconst126)*(gconst129)*(gconst132)))+(((gconst120)*(gconst125)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst125)*(gconst129)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst135)*(gconst141)))+(((gconst118)*(gconst122)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst125)*(gconst128)*(gconst131)))+(((gconst110)*(gconst119)*(gconst135)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst132)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst129)*(gconst135)))+(((gconst111)*(gconst115)*(gconst136)*(gconst140)))+(((gconst119)*(gconst125)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst135)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst134)*(gconst136)))+(((gconst112)*(gconst124)*(gconst129)*(gconst136)))+(((gconst112)*(gconst114)*(gconst137)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst124)*(gconst127)*(gconst132)))+(((gconst115)*(gconst122)*(gconst127)*(gconst139)))+(((gconst118)*(gconst119)*(gconst130)*(gconst135)))+(((gconst110)*(gconst118)*(gconst135)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst138)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst125)*(gconst126)*(gconst130)))+(((gconst121)*(gconst123)*(gconst126)*(gconst133)))+(((gconst116)*(gconst123)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst125)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst133)*(gconst138)))+(((gconst113)*(gconst123)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst132)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst126)*(gconst129)*(gconst135)))+(((gconst110)*(gconst125)*(gconst127)*(gconst139)))+(((gconst114)*(gconst123)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst135)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst116)*(gconst134)*(gconst140)))+(((gconst112)*(gconst119)*(gconst134)*(gconst137)))+(((gconst113)*(gconst123)*(gconst126)*(gconst141)))+(((gconst112)*(gconst121)*(gconst133)*(gconst136)))+(((gconst114)*(gconst119)*(gconst132)*(gconst137)))+(((gconst115)*(gconst118)*(gconst130)*(gconst139)))+(((gconst117)*(gconst119)*(gconst132)*(gconst134)))+(((gconst110)*(gconst115)*(gconst138)*(gconst139)))+(((gconst116)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst122)*(gconst125)*(gconst126)*(gconst129)))+(((gconst121)*(gconst123)*(gconst124)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst131)*(gconst139)))+(((gconst112)*(gconst126)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst130)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst128)*(gconst138)))+(((gconst112)*(gconst113)*(gconst136)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst138)))+(((gconst110)*(gconst127)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst135)*(gconst139)))+(((gconst121)*(gconst124)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst123)*(gconst124)*(gconst127)*(gconst128)))+(((gconst118)*(gconst125)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst119)*(gconst131)*(gconst137)))+(((gconst115)*(gconst122)*(gconst128)*(gconst138)))+(((gconst114)*(gconst117)*(gconst132)*(gconst139)))+(((gconst116)*(gconst121)*(gconst132)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst127)*(gconst133)))+(((gconst113)*(gconst116)*(gconst132)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst124)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst137)*(gconst139)))+(((gconst112)*(gconst124)*(gconst126)*(gconst141)))+(((gconst115)*(gconst120)*(gconst131)*(gconst136)))+(((gconst115)*(gconst119)*(gconst130)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst136)*(gconst140)))+(((gconst122)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst126)*(gconst135))));
op[5]=((((IkReal(-1.00000000000000))*(gconst111)*(gconst119)*(gconst125)*(gconst137)))+(((gconst112)*(gconst121)*(gconst124)*(gconst136)))+(((gconst117)*(gconst119)*(gconst123)*(gconst134)))+(((gconst112)*(gconst124)*(gconst126)*(gconst129)))+(((gconst115)*(gconst119)*(gconst127)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst122)*(gconst136)))+(((gconst110)*(gconst118)*(gconst128)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst124)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst141)))+(((gconst110)*(gconst118)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst117)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((gconst112)*(gconst113)*(gconst129)*(gconst136)))+(((gconst114)*(gconst117)*(gconst128)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst132)))+(((gconst112)*(gconst117)*(gconst128)*(gconst134)))+(((gconst115)*(gconst118)*(gconst122)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst121)*(gconst122)*(gconst125)*(gconst126)))+(((gconst114)*(gconst119)*(gconst123)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst116)*(gconst123)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst126)*(gconst141)))+(((gconst118)*(gconst119)*(gconst122)*(gconst135)))+(((gconst110)*(gconst115)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst122)*(gconst141)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst124)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst122)*(gconst125)*(gconst129)))+(((gconst115)*(gconst118)*(gconst128)*(gconst130)))+(((gconst118)*(gconst122)*(gconst125)*(gconst128)))+(((gconst112)*(gconst116)*(gconst129)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst122)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst119)*(gconst125)*(gconst131)))+(((gconst113)*(gconst123)*(gconst126)*(gconst129)))+(((gconst110)*(gconst119)*(gconst125)*(gconst138)))+(((gconst116)*(gconst121)*(gconst124)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst127)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst123)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst123)*(gconst134)))+(((gconst116)*(gconst121)*(gconst123)*(gconst133)))+(((gconst110)*(gconst115)*(gconst128)*(gconst138)))+(((gconst113)*(gconst116)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst128)*(gconst135)))+(((gconst115)*(gconst122)*(gconst127)*(gconst128)))+(((gconst110)*(gconst119)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst113)*(gconst128)*(gconst138)))+(((gconst112)*(gconst114)*(gconst128)*(gconst137)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst128)*(gconst133)))+(((gconst110)*(gconst125)*(gconst127)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst122)*(gconst126)*(gconst129)))+(((gconst113)*(gconst121)*(gconst123)*(gconst136)))+(((gconst113)*(gconst116)*(gconst123)*(gconst141)))+(((gconst111)*(gconst115)*(gconst126)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst116)*(gconst129)*(gconst130)))+(((gconst115)*(gconst119)*(gconst122)*(gconst138)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst118)*(gconst124)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst115)*(gconst129)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst128)*(gconst137)))+(((gconst112)*(gconst113)*(gconst126)*(gconst141)))+(((gconst114)*(gconst117)*(gconst123)*(gconst139)))+(((gconst118)*(gconst119)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst123)*(gconst124)*(gconst127)))+(((gconst121)*(gconst123)*(gconst124)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst124)*(gconst132)))+(((gconst113)*(gconst121)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((gconst112)*(gconst116)*(gconst124)*(gconst141)))+(((gconst119)*(gconst122)*(gconst125)*(gconst127)))+(((gconst116)*(gconst123)*(gconst124)*(gconst129)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst125)*(gconst139)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((gconst112)*(gconst121)*(gconst126)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst123)*(gconst124)*(gconst128)))+(((gconst111)*(gconst116)*(gconst125)*(gconst140)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst125)*(gconst126)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128))));
op[6]=((((IkReal(-1.00000000000000))*(x273)*(x278)))+(((gconst114)*(gconst137)*(x325)))+(((IkReal(-1.00000000000000))*(gconst113)*(x276)*(x318)))+(((x273)*(x274)))+(((IkReal(-1.00000000000000))*(gconst113)*(x287)*(x318)))+(((x285)*(x341)))+(((x285)*(x343)))+(((IkReal(-1.00000000000000))*(gconst113)*(x292)*(x341)))+(((IkReal(-1.00000000000000))*(gconst129)*(x285)*(x295)))+(((x291)*(x328)))+(((x275)*(x276)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst126)*(x299)))+(((IkReal(-1.00000000000000))*(gconst122)*(x294)*(x299)))+(((x273)*(x313)))+(((IkReal(-1.00000000000000))*(gconst114)*(x283)*(x286)))+(((IkReal(-1.00000000000000))*(gconst126)*(x285)*(x321)))+(((IkReal(-1.00000000000000))*(gconst114)*(x311)*(x345)))+(((IkReal(-1.00000000000000))*(gconst117)*(x305)*(x338)))+(((gconst123)*(gconst126)*(x302)))+(((IkReal(-1.00000000000000))*(gconst134)*(x283)*(x311)))+(((x319)*(x335)))+(((IkReal(-1.00000000000000))*(gconst137)*(x298)*(x305)))+(((IkReal(-1.00000000000000))*(gconst119)*(x280)*(x320)))+(((x302)*(x345)))+(((IkReal(-1.00000000000000))*(gconst120)*(x295)*(x324)))+(((x298)*(x317)))+(((IkReal(-1.00000000000000))*(gconst136)*(x308)*(x321)))+(((gconst117)*(gconst134)*(x325)))+(((x273)*(x347)))+(((x308)*(x331)))+(((x308)*(x336)))+(((IkReal(-1.00000000000000))*(x298)*(x333)))+(((IkReal(-1.00000000000000))*(gconst128)*(x299)*(x320)))+(((x278)*(x294)))+(((IkReal(-1.00000000000000))*(gconst127)*(x319)*(x326)))+(((gconst119)*(gconst132)*(x275)))+(((x275)*(x287)))+(((gconst112)*(gconst126)*(x288)))+(((x341)*(x346)))+(((gconst113)*(gconst141)*(x283)))+(((x298)*(x332)))+(((IkReal(-1.00000000000000))*(gconst130)*(x294)*(x330)))+(((IkReal(-1.00000000000000))*(gconst123)*(x273)*(x279)))+(((gconst119)*(gconst138)*(x308)))+(((IkReal(-1.00000000000000))*(gconst114)*(x291)*(x292)))+(((IkReal(-1.00000000000000))*(x308)*(x335)))+(((gconst110)*(gconst118)*(x314)))+(((x300)*(x301)))+(((IkReal(-1.00000000000000))*(x274)*(x294)))+(((IkReal(-1.00000000000000))*(gconst141)*(x295)*(x308)))+(((IkReal(-1.00000000000000))*(x273)*(x307)))+(((IkReal(-1.00000000000000))*(gconst138)*(x305)*(x319)))+(((x316)*(x342)))+(((x283)*(x348)))+(((IkReal(-1.00000000000000))*(gconst118)*(x279)*(x334)))+(((IkReal(-1.00000000000000))*(gconst129)*(x295)*(x346)))+(((IkReal(-1.00000000000000))*(gconst127)*(x279)*(x325)))+(((x300)*(x338)))+(((x283)*(x284)))+(((x288)*(x316))));
op[7]=((((gconst113)*(gconst123)*(x294)))+(((x291)*(x298)))+(((IkReal(-1.00000000000000))*(gconst113)*(x273)*(x292)))+(((IkReal(-1.00000000000000))*(gconst128)*(x318)*(x319)))+(((IkReal(-1.00000000000000))*(gconst126)*(x308)*(x321)))+(((IkReal(-1.00000000000000))*(gconst110)*(x294)*(x299)))+(((IkReal(-1.00000000000000))*(gconst119)*(x299)*(x320)))+(((x273)*(x346)))+(((IkReal(-1.00000000000000))*(gconst129)*(x295)*(x308)))+(((IkReal(-1.00000000000000))*(gconst112)*(x273)*(x279)))+(((x273)*(x285)))+(((IkReal(-1.00000000000000))*(gconst114)*(x292)*(x300)))+(((IkReal(-1.00000000000000))*(gconst127)*(x305)*(x319)))+(((IkReal(-1.00000000000000))*(x285)*(x294)))+(((gconst126)*(x351)))+(((IkReal(-1.00000000000000))*(gconst117)*(x298)*(x326)))+(((x308)*(x343)))+(((x308)*(x341)))+(((x275)*(x334)))+(((x283)*(x342)))+(((x300)*(x328)))+(((x283)*(x288)))+(((gconst119)*(gconst123)*(x275)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(x291))));
op[8]=((((IkReal(-1.00000000000000))*(x273)*(x319)))+(((IkReal(-1.00000000000000))*(gconst114)*(x283)*(x311)))+(((x283)*(x302)))+(((IkReal(-1.00000000000000))*(x294)*(x308)))+(((x275)*(x325)))+(((x298)*(x300)))+(((IkReal(-1.00000000000000))*(gconst117)*(x298)*(x305)))+(((x273)*(x308))));
polyroots8(op,zeror,numroots);
IkReal j3array[8], cj3array[8], sj3array[8], tempj3array[1];
int numsolutions = 0;
for(int ij3 = 0; ij3 < numroots; ++ij3)
{
IkReal htj3 = zeror[ij3];
tempj3array[0]=((IkReal(2.00000000000000))*(atan(htj3)));
for(int kj3 = 0; kj3 < 1; ++kj3)
{
j3array[numsolutions] = tempj3array[kj3];
if( j3array[numsolutions] > IKPI )
{
    j3array[numsolutions]-=IK2PI;
}
else if( j3array[numsolutions] < -IKPI )
{
    j3array[numsolutions]+=IK2PI;
}
sj3array[numsolutions] = IKsin(j3array[numsolutions]);
cj3array[numsolutions] = IKcos(j3array[numsolutions]);
numsolutions++;
}
}
bool j3valid[8]={true,true,true,true,true,true,true,true};
_nj3 = 8;
for(int ij3 = 0; ij3 < numsolutions; ++ij3)
    {
if( !j3valid[ij3] )
{
    continue;
}
    j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
htj3 = IKtan(j3/2);

_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < numsolutions; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst158;
gconst158=IKsign(((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x353=((r01)*(sj1));
IkReal x354=((cj1)*(r00));
IkReal x355=((IkReal(1.00000000000000))*(cj3));
IkReal x356=((IkReal(1.00000000000000))*(sj3));
IkReal x357=((sj2)*(x356));
IkReal x358=((cj2)*(r00)*(sj1));
IkReal x359=((cj1)*(cj2)*(r01));
if( IKabs(((gconst158)*(((((IkReal(-1.00000000000000))*(x356)*(x358)))+(((sj3)*(x359)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x353)*(x357)))+(((IkReal(-1.00000000000000))*(x354)*(x357))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst158)*(((((cj3)*(x359)))+(((IkReal(-1.00000000000000))*(sj2)*(x354)*(x355)))+(((IkReal(-1.00000000000000))*(sj2)*(x353)*(x355)))+(((IkReal(-1.00000000000000))*(r02)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x358))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst158)*(((((IkReal(-1.00000000000000))*(x356)*(x358)))+(((sj3)*(x359)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x353)*(x357)))+(((IkReal(-1.00000000000000))*(x354)*(x357)))))), ((gconst158)*(((((cj3)*(x359)))+(((IkReal(-1.00000000000000))*(sj2)*(x354)*(x355)))+(((IkReal(-1.00000000000000))*(sj2)*(x353)*(x355)))+(((IkReal(-1.00000000000000))*(r02)*(x356)))+(((IkReal(-1.00000000000000))*(x355)*(x358)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x360=IKsin(j5);
IkReal x361=IKcos(j5);
IkReal x362=((IkReal(2.16000000000000))*(sj3));
IkReal x363=((IkReal(1.00000000000000))*(px));
IkReal x364=((cj2)*(sj1));
IkReal x365=((cj1)*(cj2));
IkReal x366=((cj1)*(sj2));
IkReal x367=((IkReal(1.00000000000000))*(r00));
IkReal x368=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x369=((cj3)*(x360));
IkReal x370=((cj3)*(x361));
evalcond[0]=((((IkReal(-1.00000000000000))*(x369)))+(((IkReal(-1.00000000000000))*(r02)))+(((sj3)*(x361))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x361)*(x362)))+(((IkReal(2.16000000000000))*(x369)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((r01)*(x365)))+(((IkReal(-1.00000000000000))*(r01)*(x368)))+(x370)+(((IkReal(-1.00000000000000))*(x366)*(x367)))+(((sj3)*(x360)))+(((IkReal(-1.00000000000000))*(x364)*(x367))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x363)*(x366)))+(((IkReal(-1.00000000000000))*(x363)*(x364)))+(((IkReal(-1.00000000000000))*(x360)*(x362)))+(((py)*(x365)))+(((IkReal(-1.00000000000000))*(py)*(x368)))+(((IkReal(-2.16000000000000))*(x370)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x371=((IkReal(1.00000000000000))*(py));
cj5array[0]=((IkReal(-0.462962962962963))*(((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x371)))+(((IkReal(0.158000000000000))*(cj4)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x371)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(px))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x372=((IkReal(1.00000000000000))*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x372)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x372)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign(((((IkReal(-1.00000000000000))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*((cj4)*(cj4))*((cj5)*(cj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*((cj4)*(cj4))*((cj5)*(cj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x373=((IkReal(1.00000000000000))*(r01));
IkReal x374=((cj1)*(sj2));
IkReal x375=((sj1)*(sj2));
IkReal x376=((cj2)*(sj1));
IkReal x377=((IkReal(1.00000000000000))*(py));
IkReal x378=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x378)))+(((r00)*(x375)))+(((IkReal(-1.00000000000000))*(x373)*(x374)))+(((IkReal(-1.00000000000000))*(x373)*(x376))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(x375)))+(((IkReal(-1.00000000000000))*(x374)*(x377)))+(((IkReal(-1.00000000000000))*(x376)*(x377)))+(((IkReal(-1.00000000000000))*(px)*(x378))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x379=((IkReal(1.00000000000000))*(cj2));
IkReal x380=((cj1)*(cj5));
IkReal x381=((sj1)*(sj5));
IkReal x382=((IkReal(1.00000000000000))*(sj2));
IkReal x383=((cj1)*(sj5));
IkReal x384=((cj5)*(sj1));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(r00)*(x379)*(x381)))+(((IkReal(-1.00000000000000))*(r01)*(x381)*(x382)))+(((IkReal(-1.00000000000000))*(r00)*(x382)*(x383)))+(((cj2)*(r01)*(x383)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(r01)*(x379)*(x380)))+(((r00)*(sj2)*(x380)))+(((cj2)*(r00)*(x384)))+(((r01)*(sj2)*(x384))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(r00)*(x379)*(x381)))+(((IkReal(-1.00000000000000))*(r01)*(x381)*(x382)))+(((IkReal(-1.00000000000000))*(r00)*(x382)*(x383)))+(((cj2)*(r01)*(x383)))+(((cj5)*(r02)))))), ((gconst55)*(((((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(r01)*(x379)*(x380)))+(((r00)*(sj2)*(x380)))+(((cj2)*(r00)*(x384)))+(((r01)*(sj2)*(x384)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x385=IKcos(j3);
IkReal x386=IKsin(j3);
IkReal x387=((IkReal(2.16000000000000))*(cj5));
IkReal x388=((IkReal(1.00000000000000))*(px));
IkReal x389=((cj2)*(sj1));
IkReal x390=((cj1)*(cj2));
IkReal x391=((cj1)*(sj2));
IkReal x392=((IkReal(1.00000000000000))*(cj5));
IkReal x393=((IkReal(1.00000000000000))*(r00));
IkReal x394=((sj5)*(x385));
IkReal x395=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x396=((sj5)*(x386));
evalcond[0]=((((IkReal(-1.00000000000000))*(x394)))+(((IkReal(-1.00000000000000))*(x386)*(x392)))+(((IkReal(-1.00000000000000))*(r02))));
evalcond[1]=((((x386)*(x387)))+(((IkReal(2.16000000000000))*(x394)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x385)*(x392)))+(((IkReal(-1.00000000000000))*(x389)*(x393)))+(((r01)*(x390)))+(((IkReal(-1.00000000000000))*(r01)*(x395)))+(x396)+(((IkReal(-1.00000000000000))*(x391)*(x393))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x388)*(x391)))+(((py)*(x390)))+(((IkReal(-1.00000000000000))*(py)*(x395)))+(((x385)*(x387)))+(((IkReal(-2.16000000000000))*(x396)))+(((IkReal(-1.00000000000000))*(x388)*(x389)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x397=((IkReal(1.00000000000000))*(r01));
IkReal x398=((cj1)*(sj2));
IkReal x399=((sj1)*(sj2));
IkReal x400=((cj2)*(sj1));
IkReal x401=((IkReal(1.00000000000000))*(py));
IkReal x402=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x399)))+(((IkReal(-1.00000000000000))*(x397)*(x398)))+(((IkReal(-1.00000000000000))*(x397)*(x400)))+(((IkReal(-1.00000000000000))*(r00)*(x402))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x402)))+(((IkReal(-1.00000000000000))*(x400)*(x401)))+(((IkReal(-1.00000000000000))*(x398)*(x401)))+(((px)*(x399))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x403=((r00)*(sj5));
IkReal x404=((cj2)*(sj1));
IkReal x405=((cj1)*(sj2));
IkReal x406=((cj5)*(r00));
IkReal x407=((r01)*(sj1)*(sj2));
IkReal x408=((IkReal(1.00000000000000))*(cj1)*(cj2)*(r01));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x408)))+(((sj5)*(x407)))+(((x403)*(x405)))+(((x403)*(x404)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((x405)*(x406)))+(((x404)*(x406)))+(((cj5)*(x407)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x408))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x408)))+(((sj5)*(x407)))+(((x403)*(x405)))+(((x403)*(x404)))+(((cj5)*(r02)))))), ((gconst56)*(((((x405)*(x406)))+(((x404)*(x406)))+(((cj5)*(x407)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((IkReal(-1.00000000000000))*(cj5)*(x408)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x409=IKcos(j3);
IkReal x410=IKsin(j3);
IkReal x411=((IkReal(1.00000000000000))*(px));
IkReal x412=((cj2)*(sj1));
IkReal x413=((cj1)*(cj2));
IkReal x414=((cj1)*(sj2));
IkReal x415=((IkReal(1.00000000000000))*(r00));
IkReal x416=((cj5)*(x410));
IkReal x417=((sj5)*(x409));
IkReal x418=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x419=((cj5)*(x409));
IkReal x420=((sj5)*(x410));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x416)+(((IkReal(-1.00000000000000))*(x417))));
evalcond[1]=((((IkReal(2.16000000000000))*(x417)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-2.16000000000000))*(x416))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x412)*(x415)))+(((r01)*(x413)))+(x419)+(x420)+(((IkReal(-1.00000000000000))*(r01)*(x418)))+(((IkReal(-1.00000000000000))*(x414)*(x415))));
evalcond[3]=((IkReal(-0.145000000000000))+(((py)*(x413)))+(((IkReal(-1.00000000000000))*(x411)*(x414)))+(((IkReal(-1.00000000000000))*(x411)*(x412)))+(((IkReal(-2.16000000000000))*(x420)))+(((IkReal(-1.00000000000000))*(py)*(x418)))+(((IkReal(-2.16000000000000))*(x419)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x421=((IkReal(1.00000000000000))*(r01));
IkReal x422=((cj1)*(sj2));
IkReal x423=((sj1)*(sj2));
IkReal x424=((cj2)*(sj1));
IkReal x425=((IkReal(1.00000000000000))*(py));
IkReal x426=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x421)*(x424)))+(((IkReal(-1.00000000000000))*(x421)*(x422)))+(((IkReal(-1.00000000000000))*(sj4)))+(((r00)*(x423)))+(((IkReal(-1.00000000000000))*(r00)*(x426))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x422)*(x425)))+(((IkReal(-1.00000000000000))*(px)*(x426)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(2.16000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(x424)*(x425)))+(((px)*(x423))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x427=((r00)*(sj1));
IkReal x428=((IkReal(1.00000000000000))*(cj2));
IkReal x429=((r01)*(sj1));
IkReal x430=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x428)*(x429)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x428)))+(((IkReal(-1.00000000000000))*(r01)*(x430)))+(((sj2)*(x427))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x427)*(x428)))+(((IkReal(-1.00000000000000))*(r00)*(x430)))+(((IkReal(-1.00000000000000))*(sj2)*(x429)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x428)))+(((IkReal(-1.00000000000000))*(py)*(x430)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x428))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x431=((IkReal(6.32911392405063))*(cj2));
IkReal x432=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(-6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((cj1)*(py)*(x431)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x432)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x432)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x431)))+(((IkReal(-7.59493670886076))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-6.32911392405063))*(pz)))+IKsqr(((IkReal(-0.917721518987342))+(((cj1)*(py)*(x431)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x432)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x432)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x431)))+(((IkReal(-7.59493670886076))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-6.32911392405063))*(pz)), ((IkReal(-0.917721518987342))+(((cj1)*(py)*(x431)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x432)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x432)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x431)))+(((IkReal(-7.59493670886076))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x433=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x433)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x433)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x434=((r00)*(sj1));
IkReal x435=((IkReal(1.00000000000000))*(cj2));
IkReal x436=((r01)*(sj1));
IkReal x437=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(x435)*(x436)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x435)))+(((sj2)*(x434)))+(((IkReal(-1.00000000000000))*(r01)*(x437))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x434)*(x435)))+(((IkReal(-1.00000000000000))*(sj2)*(x436)))+(((IkReal(-1.00000000000000))*(r00)*(x437)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(-0.615000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x435)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x435)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x437))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x438=((IkReal(6.32911392405063))*(cj2));
IkReal x439=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x438)))+(((px)*(sj1)*(x438)))+(((IkReal(7.59493670886076))*(cj2)))+(((cj1)*(px)*(x439)))+(((py)*(sj1)*(x439))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(6.32911392405063))*(pz)))+IKsqr(((IkReal(0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x438)))+(((px)*(sj1)*(x438)))+(((IkReal(7.59493670886076))*(cj2)))+(((cj1)*(px)*(x439)))+(((py)*(sj1)*(x439)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(6.32911392405063))*(pz)), ((IkReal(0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x438)))+(((px)*(sj1)*(x438)))+(((IkReal(7.59493670886076))*(cj2)))+(((cj1)*(px)*(x439)))+(((py)*(sj1)*(x439)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x440=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x440)))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x440)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x441=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(gconst57)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x441)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x441)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst57)*(r02)), ((gconst57)*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x441)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x441)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x442=IKsin(j3);
IkReal x443=IKcos(j3);
IkReal x444=((cj1)*(cj2));
IkReal x445=((IkReal(1.00000000000000))*(sj2));
IkReal x446=((IkReal(0.158000000000000))*(sj4));
IkReal x447=((cj4)*(x443));
IkReal x448=((cj4)*(x442));
IkReal x449=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(x448)))+(((IkReal(-1.00000000000000))*(r02))));
evalcond[1]=((((IkReal(2.16000000000000))*(x448)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x442)*(x446))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x445)))+(((IkReal(-1.00000000000000))*(x447)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x445)))+(((r01)*(x444)))+(((IkReal(-1.00000000000000))*(r00)*(x449))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x445)))+(((IkReal(2.16000000000000))*(x447)))+(((py)*(x444)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x445)))+(((IkReal(-1.00000000000000))*(x443)*(x446)))+(((IkReal(-1.00000000000000))*(px)*(x449)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x450=((IkReal(1.00000000000000))*(r01));
IkReal x451=((cj1)*(sj2));
IkReal x452=((sj1)*(sj2));
IkReal x453=((cj2)*(sj1));
IkReal x454=((IkReal(1.00000000000000))*(py));
IkReal x455=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r00)*(x455)))+(((IkReal(-1.00000000000000))*(x450)*(x453)))+(((IkReal(-1.00000000000000))*(x450)*(x451)))+(((r00)*(x452))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-2.16000000000000))*(sj4)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x453)*(x454)))+(((px)*(x452)))+(((IkReal(-1.00000000000000))*(x451)*(x454)))+(((IkReal(-1.00000000000000))*(px)*(x455))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x456=((r00)*(sj1));
IkReal x457=((IkReal(1.00000000000000))*(cj2));
IkReal x458=((r01)*(sj1));
IkReal x459=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(1.00000000000000))+(((sj2)*(x456)))+(((IkReal(-1.00000000000000))*(r01)*(x459)))+(((IkReal(-1.00000000000000))*(x457)*(x458)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x457))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r00)*(x459)))+(((IkReal(-1.00000000000000))*(x456)*(x457)))+(((IkReal(-1.00000000000000))*(sj2)*(x458)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(-0.615000000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x459)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x457)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x457))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x460=((IkReal(6.32911392405063))*(cj2));
IkReal x461=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(-6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((cj1)*(py)*(x460)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x461)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x461)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x460))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-6.32911392405063))*(pz)))+IKsqr(((IkReal(-0.917721518987342))+(((cj1)*(py)*(x460)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x461)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x461)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x460)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-6.32911392405063))*(pz)), ((IkReal(-0.917721518987342))+(((cj1)*(py)*(x460)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x461)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x461)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x460)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x462=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x462)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x462)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x463=((r00)*(sj1));
IkReal x464=((IkReal(1.00000000000000))*(cj2));
IkReal x465=((r01)*(sj1));
IkReal x466=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r01)*(x466)))+(((IkReal(-1.00000000000000))*(x464)*(x465)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x464)))+(((sj2)*(x463))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x463)*(x464)))+(((IkReal(-1.00000000000000))*(r00)*(x466)))+(((IkReal(-1.00000000000000))*(sj2)*(x465)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x466)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x464)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x464))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x467=((IkReal(6.32911392405063))*(cj2));
IkReal x468=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((cj1)*(px)*(x468)))+(((py)*(sj1)*(x468)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x467)))+(((IkReal(7.59493670886076))*(cj2)))+(((px)*(sj1)*(x467))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(6.32911392405063))*(pz)))+IKsqr(((IkReal(0.917721518987342))+(((cj1)*(px)*(x468)))+(((py)*(sj1)*(x468)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x467)))+(((IkReal(7.59493670886076))*(cj2)))+(((px)*(sj1)*(x467)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(6.32911392405063))*(pz)), ((IkReal(0.917721518987342))+(((cj1)*(px)*(x468)))+(((py)*(sj1)*(x468)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x467)))+(((IkReal(7.59493670886076))*(cj2)))+(((px)*(sj1)*(x467)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x469=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x469)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x469)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((gconst58)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(r02)), ((gconst58)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x470=IKsin(j3);
IkReal x471=IKcos(j3);
IkReal x472=((IkReal(1.00000000000000))*(px));
IkReal x473=((cj2)*(sj1));
IkReal x474=((cj1)*(cj2));
IkReal x475=((cj1)*(sj2));
IkReal x476=((IkReal(0.158000000000000))*(sj4));
IkReal x477=((IkReal(1.00000000000000))*(r00));
IkReal x478=((cj4)*(x471));
IkReal x479=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x480=((cj4)*(x470));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x480));
evalcond[1]=((((IkReal(-2.16000000000000))*(x480)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x470)*(x476))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x475)*(x477)))+(((IkReal(-1.00000000000000))*(r01)*(x479)))+(((IkReal(-1.00000000000000))*(x473)*(x477)))+(x478)+(((r01)*(x474))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x478)))+(((IkReal(-1.00000000000000))*(x472)*(x475)))+(((IkReal(-1.00000000000000))*(x472)*(x473)))+(((IkReal(-1.00000000000000))*(py)*(x479)))+(((IkReal(-1.00000000000000))*(x471)*(x476)))+(((py)*(x474)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x481=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x482=((cj4)*(cj5));
IkReal x483=((IkReal(25.0000000000000))*(pz));
IkReal x484=((IkReal(54.0000000000000))*(r02));
if( IKabs(((IkReal(-0.253164556962025))*(x481)*(((x483)+(x484))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.253164556962025))*(x481)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.95000000000000))*(r02)*(sj4)))+(((x482)*(x483)))+(((x482)*(x484))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.253164556962025))*(x481)*(((x483)+(x484)))))+IKsqr(((IkReal(0.253164556962025))*(x481)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.95000000000000))*(r02)*(sj4)))+(((x482)*(x483)))+(((x482)*(x484)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-0.253164556962025))*(x481)*(((x483)+(x484)))), ((IkReal(0.253164556962025))*(x481)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((IkReal(-3.95000000000000))*(r02)*(sj4)))+(((x482)*(x483)))+(((x482)*(x484)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x485=IKsin(j3);
IkReal x486=IKcos(j3);
IkReal x487=((IkReal(1.00000000000000))*(px));
IkReal x488=((cj2)*(sj1));
IkReal x489=((cj1)*(sj2));
IkReal x490=((IkReal(0.158000000000000))*(sj4));
IkReal x491=((cj4)*(cj5));
IkReal x492=((IkReal(1.00000000000000))*(r00));
IkReal x493=((cj1)*(cj2));
IkReal x494=((sj5)*(x485));
IkReal x495=((IkReal(1.00000000000000))*(x486));
IkReal x496=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x497=((IkReal(2.16000000000000))*(x486));
evalcond[0]=((((IkReal(-1.00000000000000))*(x485)*(x491)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(sj5)*(x495))));
evalcond[1]=((((IkReal(2.16000000000000))*(x485)*(x491)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x485)*(x490)))+(((sj5)*(x497))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x488)*(x492)))+(((IkReal(-1.00000000000000))*(r01)*(x496)))+(((IkReal(-1.00000000000000))*(x489)*(x492)))+(((IkReal(-1.00000000000000))*(x491)*(x495)))+(x494)+(((r01)*(x493))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x494)))+(((py)*(x493)))+(((x491)*(x497)))+(((IkReal(-1.00000000000000))*(x486)*(x490)))+(((IkReal(-1.00000000000000))*(x487)*(x489)))+(((IkReal(-1.00000000000000))*(x487)*(x488)))+(((IkReal(-1.00000000000000))*(py)*(x496)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x498=((cj4)*(cj5));
IkReal x499=((IkReal(1.00000000000000))*(r01));
IkReal x500=((cj1)*(cj2));
IkReal x501=((sj2)*(sj5));
IkReal x502=((cj1)*(r00));
IkReal x503=((cj2)*(r00)*(sj1));
if( IKabs(((gconst54)*(((((r02)*(x498)))+(((r01)*(sj5)*(x500)))+(((IkReal(-1.00000000000000))*(x501)*(x502)))+(((IkReal(-1.00000000000000))*(sj5)*(x503)))+(((IkReal(-1.00000000000000))*(sj1)*(x499)*(x501))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((sj2)*(x498)*(x502)))+(((r01)*(sj1)*(sj2)*(x498)))+(((r02)*(sj5)))+(((x498)*(x503)))+(((IkReal(-1.00000000000000))*(x498)*(x499)*(x500))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((r02)*(x498)))+(((r01)*(sj5)*(x500)))+(((IkReal(-1.00000000000000))*(x501)*(x502)))+(((IkReal(-1.00000000000000))*(sj5)*(x503)))+(((IkReal(-1.00000000000000))*(sj1)*(x499)*(x501)))))), ((gconst54)*(((((sj2)*(x498)*(x502)))+(((r01)*(sj1)*(sj2)*(x498)))+(((r02)*(sj5)))+(((x498)*(x503)))+(((IkReal(-1.00000000000000))*(x498)*(x499)*(x500)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x504=IKsin(j3);
IkReal x505=IKcos(j3);
IkReal x506=((IkReal(1.00000000000000))*(px));
IkReal x507=((cj2)*(sj1));
IkReal x508=((cj1)*(sj2));
IkReal x509=((IkReal(0.158000000000000))*(sj4));
IkReal x510=((cj4)*(cj5));
IkReal x511=((IkReal(1.00000000000000))*(r00));
IkReal x512=((cj1)*(cj2));
IkReal x513=((sj5)*(x504));
IkReal x514=((IkReal(1.00000000000000))*(x505));
IkReal x515=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x516=((IkReal(2.16000000000000))*(x505));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x504)*(x510)))+(((IkReal(-1.00000000000000))*(sj5)*(x514))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x504)*(x510)))+(((sj5)*(x516)))+(((IkReal(-1.00000000000000))*(x504)*(x509))));
evalcond[2]=((((r01)*(x512)))+(((IkReal(-1.00000000000000))*(r01)*(x515)))+(x513)+(((IkReal(-1.00000000000000))*(x508)*(x511)))+(((IkReal(-1.00000000000000))*(x510)*(x514)))+(((IkReal(-1.00000000000000))*(x507)*(x511))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x506)*(x508)))+(((IkReal(-1.00000000000000))*(x506)*(x507)))+(((x510)*(x516)))+(((IkReal(-1.00000000000000))*(x505)*(x509)))+(((IkReal(-2.16000000000000))*(x513)))+(((py)*(x512)))+(((IkReal(-1.00000000000000))*(py)*(x515)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j5array[2], cj5array[2], sj5array[2];
bool j5valid[2]={false};
_nj5 = 2;
IkReal x517=((IkReal(1.00000000000000))*(cj2));
cj5array[0]=((((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x517)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x517)))+(((r00)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(r01)*(sj2))))));
if( cj5array[0] >= -1-IKFAST_SINCOS_THRESH && cj5array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j5valid[0] = j5valid[1] = true;
    j5array[0] = IKacos(cj5array[0]);
    sj5array[0] = IKsin(j5array[0]);
    cj5array[1] = cj5array[0];
    j5array[1] = -j5array[0];
    sj5array[1] = -sj5array[0];
}
else if( isnan(cj5array[0]) )
{
    // probably any value will work
    j5valid[0] = true;
    cj5array[0] = 1; sj5array[0] = 0; j5array[0] = 0;
}
for(int ij5 = 0; ij5 < 2; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 2; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[1];
IkReal x518=((IkReal(1.00000000000000))*(py));
evalcond[0]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x518)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x518)))+(((IkReal(2.16000000000000))*(sj4)*(IKcos(j5))))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(px))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign(((((IkReal(-1.00000000000000))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*((cj4)*(cj4))*((cj5)*(cj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((sj5)*(sj5))))+(((IkReal(-1.00000000000000))*((cj4)*(cj4))*((cj5)*(cj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=sj4;
dummyeval[1]=sj5;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x519=((IkReal(1.00000000000000))*(r01));
IkReal x520=((cj1)*(sj2));
IkReal x521=((sj1)*(sj2));
IkReal x522=((cj2)*(sj1));
IkReal x523=((IkReal(1.00000000000000))*(py));
IkReal x524=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x524)))+(((IkReal(-1.00000000000000))*(x519)*(x522)))+(((IkReal(-1.00000000000000))*(x519)*(x520)))+(((r00)*(x521))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(-1.00000000000000))*(x520)*(x523)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(px)*(x524)))+(((px)*(x521)))+(((IkReal(-1.00000000000000))*(x522)*(x523))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign(((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj5)*(cj5))))+(((IkReal(-1.00000000000000))*((sj5)*(sj5)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x525=((IkReal(1.00000000000000))*(cj2));
IkReal x526=((cj1)*(cj5));
IkReal x527=((sj1)*(sj5));
IkReal x528=((IkReal(1.00000000000000))*(sj2));
IkReal x529=((cj1)*(sj5));
IkReal x530=((cj5)*(sj1));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(r01)*(x527)*(x528)))+(((IkReal(-1.00000000000000))*(r00)*(x528)*(x529)))+(((cj2)*(r01)*(x529)))+(((IkReal(-1.00000000000000))*(r00)*(x525)*(x527)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((r02)*(sj5)))+(((r01)*(sj2)*(x530)))+(((cj2)*(r00)*(x530)))+(((r00)*(sj2)*(x526)))+(((IkReal(-1.00000000000000))*(r01)*(x525)*(x526))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(r01)*(x527)*(x528)))+(((IkReal(-1.00000000000000))*(r00)*(x528)*(x529)))+(((cj2)*(r01)*(x529)))+(((IkReal(-1.00000000000000))*(r00)*(x525)*(x527)))+(((cj5)*(r02)))))), ((gconst55)*(((((r02)*(sj5)))+(((r01)*(sj2)*(x530)))+(((cj2)*(r00)*(x530)))+(((r00)*(sj2)*(x526)))+(((IkReal(-1.00000000000000))*(r01)*(x525)*(x526)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x531=IKcos(j3);
IkReal x532=IKsin(j3);
IkReal x533=((IkReal(2.16000000000000))*(cj5));
IkReal x534=((IkReal(1.00000000000000))*(px));
IkReal x535=((cj2)*(sj1));
IkReal x536=((cj1)*(cj2));
IkReal x537=((cj1)*(sj2));
IkReal x538=((IkReal(1.00000000000000))*(cj5));
IkReal x539=((IkReal(1.00000000000000))*(r00));
IkReal x540=((sj5)*(x531));
IkReal x541=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x542=((sj5)*(x532));
evalcond[0]=((((IkReal(-1.00000000000000))*(x532)*(x538)))+(((IkReal(-1.00000000000000))*(x540)))+(((IkReal(-1.00000000000000))*(r02))));
evalcond[1]=((((IkReal(2.16000000000000))*(x540)))+(((IkReal(-1.00000000000000))*(pz)))+(((x532)*(x533))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x537)*(x539)))+(((IkReal(-1.00000000000000))*(x531)*(x538)))+(((IkReal(-1.00000000000000))*(r01)*(x541)))+(x542)+(((IkReal(-1.00000000000000))*(x535)*(x539)))+(((r01)*(x536))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x534)*(x537)))+(((IkReal(-1.00000000000000))*(x534)*(x535)))+(((x531)*(x533)))+(((IkReal(-2.16000000000000))*(x542)))+(((IkReal(-1.00000000000000))*(py)*(x541)))+(((IkReal(-1.20000000000000))*(cj2)))+(((py)*(x536))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x543=((IkReal(1.00000000000000))*(r01));
IkReal x544=((cj1)*(sj2));
IkReal x545=((sj1)*(sj2));
IkReal x546=((cj2)*(sj1));
IkReal x547=((IkReal(1.00000000000000))*(py));
IkReal x548=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r00)*(x548)))+(((IkReal(-1.00000000000000))*(x543)*(x546)))+(((IkReal(-1.00000000000000))*(x543)*(x544)))+(((r00)*(x545))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(-1.00000000000000))*(x544)*(x547)))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(x545)))+(((IkReal(-1.00000000000000))*(px)*(x548)))+(((IkReal(-1.00000000000000))*(x546)*(x547))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=IKsign((((cj5)*(cj5))+((sj5)*(sj5))));
dummyeval[0]=(((cj5)*(cj5))+((sj5)*(sj5)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x549=((r00)*(sj5));
IkReal x550=((cj2)*(sj1));
IkReal x551=((cj1)*(sj2));
IkReal x552=((cj5)*(r00));
IkReal x553=((r01)*(sj1)*(sj2));
IkReal x554=((IkReal(1.00000000000000))*(cj1)*(cj2)*(r01));
if( IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x554)))+(((sj5)*(x553)))+(((x549)*(x550)))+(((x549)*(x551)))+(((cj5)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst56)*(((((IkReal(-1.00000000000000))*(cj5)*(x554)))+(((cj5)*(x553)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((x550)*(x552)))+(((x551)*(x552))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst56)*(((((IkReal(-1.00000000000000))*(sj5)*(x554)))+(((sj5)*(x553)))+(((x549)*(x550)))+(((x549)*(x551)))+(((cj5)*(r02)))))), ((gconst56)*(((((IkReal(-1.00000000000000))*(cj5)*(x554)))+(((cj5)*(x553)))+(((IkReal(-1.00000000000000))*(r02)*(sj5)))+(((x550)*(x552)))+(((x551)*(x552)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x555=IKcos(j3);
IkReal x556=IKsin(j3);
IkReal x557=((IkReal(1.00000000000000))*(px));
IkReal x558=((cj2)*(sj1));
IkReal x559=((cj1)*(cj2));
IkReal x560=((cj1)*(sj2));
IkReal x561=((IkReal(1.00000000000000))*(r00));
IkReal x562=((cj5)*(x556));
IkReal x563=((sj5)*(x555));
IkReal x564=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x565=((cj5)*(x555));
IkReal x566=((sj5)*(x556));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x562)+(((IkReal(-1.00000000000000))*(x563))));
evalcond[1]=((((IkReal(2.16000000000000))*(x563)))+(((IkReal(-2.16000000000000))*(x562)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x558)*(x561)))+(((IkReal(-1.00000000000000))*(x560)*(x561)))+(x566)+(x565)+(((r01)*(x559)))+(((IkReal(-1.00000000000000))*(r01)*(x564))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x565)))+(((IkReal(-2.16000000000000))*(x566)))+(((py)*(x559)))+(((IkReal(-1.00000000000000))*(py)*(x564)))+(((IkReal(-1.00000000000000))*(x557)*(x560)))+(((IkReal(-1.00000000000000))*(x557)*(x558)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x567=((IkReal(1.00000000000000))*(r01));
IkReal x568=((cj1)*(sj2));
IkReal x569=((sj1)*(sj2));
IkReal x570=((cj2)*(sj1));
IkReal x571=((IkReal(1.00000000000000))*(py));
IkReal x572=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj4)))+(((r00)*(x569)))+(((IkReal(-1.00000000000000))*(r00)*(x572)))+(((IkReal(-1.00000000000000))*(x567)*(x570)))+(((IkReal(-1.00000000000000))*(x567)*(x568))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(px)*(x572)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x570)*(x571)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(2.16000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(x568)*(x571)))+(((px)*(x569))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst57;
gconst57=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x573=((r00)*(sj1));
IkReal x574=((IkReal(1.00000000000000))*(cj2));
IkReal x575=((r01)*(sj1));
IkReal x576=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(r01)*(x576)))+(((IkReal(-1.00000000000000))*(x574)*(x575)))+(((sj2)*(x573)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x574))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj2)*(x575)))+(((IkReal(-1.00000000000000))*(x573)*(x574)))+(((IkReal(-1.00000000000000))*(r00)*(x576)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x574)))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x574)))+(((IkReal(-1.00000000000000))*(py)*(x576))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x577=((IkReal(6.32911392405063))*(cj2));
IkReal x578=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(-6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x578)))+(((cj1)*(py)*(x577)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x577)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x578)))+(((IkReal(-7.59493670886076))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-6.32911392405063))*(pz)))+IKsqr(((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x578)))+(((cj1)*(py)*(x577)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x577)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x578)))+(((IkReal(-7.59493670886076))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-6.32911392405063))*(pz)), ((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x578)))+(((cj1)*(py)*(x577)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x577)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x578)))+(((IkReal(-7.59493670886076))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x579=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x579)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x579)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x580=((r00)*(sj1));
IkReal x581=((IkReal(1.00000000000000))*(cj2));
IkReal x582=((r01)*(sj1));
IkReal x583=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(1.00000000000000))+(((sj2)*(x580)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x581)))+(((IkReal(-1.00000000000000))*(r01)*(x583)))+(((IkReal(-1.00000000000000))*(x581)*(x582))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x580)*(x581)))+(((IkReal(-1.00000000000000))*(sj2)*(x582)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x583))));
evalcond[4]=((IkReal(-0.615000000000000))+(((IkReal(-1.00000000000000))*(py)*(x583)))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x581)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x581))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x584=((IkReal(6.32911392405063))*(cj2));
IkReal x585=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((cj1)*(px)*(x585)))+(((py)*(sj1)*(x585)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x584)))+(((IkReal(7.59493670886076))*(cj2)))+(((px)*(sj1)*(x584))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(6.32911392405063))*(pz)))+IKsqr(((IkReal(0.917721518987342))+(((cj1)*(px)*(x585)))+(((py)*(sj1)*(x585)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x584)))+(((IkReal(7.59493670886076))*(cj2)))+(((px)*(sj1)*(x584)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(6.32911392405063))*(pz)), ((IkReal(0.917721518987342))+(((cj1)*(px)*(x585)))+(((py)*(sj1)*(x585)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x584)))+(((IkReal(7.59493670886076))*(cj2)))+(((px)*(sj1)*(x584)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x586=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x586)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x586)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x587=((IkReal(1.00000000000000))*(r00));
if( IKabs(((IkReal(-1.00000000000000))*(gconst57)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst57)*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x587)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x587)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-1.00000000000000))*(gconst57)*(r02)), ((gconst57)*(((((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x587)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x587)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x588=IKsin(j3);
IkReal x589=IKcos(j3);
IkReal x590=((cj1)*(cj2));
IkReal x591=((IkReal(1.00000000000000))*(sj2));
IkReal x592=((IkReal(0.158000000000000))*(sj4));
IkReal x593=((cj4)*(x589));
IkReal x594=((cj4)*(x588));
IkReal x595=((IkReal(1.00000000000000))*(cj2)*(sj1));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x594))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x588)*(x592)))+(((IkReal(2.16000000000000))*(x594)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r00)*(x595)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x591)))+(((IkReal(-1.00000000000000))*(x593)))+(((r01)*(x590)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(x591))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(px)*(x595)))+(((py)*(x590)))+(((IkReal(-1.00000000000000))*(x589)*(x592)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x591)))+(((IkReal(2.16000000000000))*(x593)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x591)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x596=((IkReal(1.00000000000000))*(r01));
IkReal x597=((cj1)*(sj2));
IkReal x598=((sj1)*(sj2));
IkReal x599=((cj2)*(sj1));
IkReal x600=((IkReal(1.00000000000000))*(py));
IkReal x601=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j5)), IkReal(6.28318530717959))));
evalcond[1]=((sj4)+(((IkReal(-1.00000000000000))*(r00)*(x601)))+(((IkReal(-1.00000000000000))*(x596)*(x599)))+(((IkReal(-1.00000000000000))*(x596)*(x597)))+(((r00)*(x598))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-2.16000000000000))*(sj4)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x597)*(x600)))+(((IkReal(-1.00000000000000))*(x599)*(x600)))+(((px)*(x598)))+(((IkReal(-1.00000000000000))*(px)*(x601))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst58;
gconst58=IKsign(cj4);
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[5];
IkReal x602=((r00)*(sj1));
IkReal x603=((IkReal(1.00000000000000))*(cj2));
IkReal x604=((r01)*(sj1));
IkReal x605=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(1.00000000000000))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x603)))+(((IkReal(-1.00000000000000))*(r01)*(x605)))+(((IkReal(-1.00000000000000))*(x603)*(x604)))+(((sj2)*(x602))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x602)*(x603)))+(((IkReal(-1.00000000000000))*(r00)*(x605)))+(((IkReal(-1.00000000000000))*(sj2)*(x604)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(-0.615000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x603)))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x603)))+(((IkReal(-1.00000000000000))*(py)*(x605))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x606=((IkReal(6.32911392405063))*(cj2));
IkReal x607=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(-6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x607)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x606)))+(((cj1)*(py)*(x606)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x607))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-6.32911392405063))*(pz)))+IKsqr(((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x607)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x606)))+(((cj1)*(py)*(x606)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x607)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-6.32911392405063))*(pz)), ((IkReal(-0.917721518987342))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x607)))+(((IkReal(-1.00000000000000))*(px)*(sj1)*(x606)))+(((cj1)*(py)*(x606)))+(((IkReal(-7.59493670886076))*(cj2)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x607)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x608=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x608)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x608)))+(((IkReal(-0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x609=((r00)*(sj1));
IkReal x610=((IkReal(1.00000000000000))*(cj2));
IkReal x611=((r01)*(sj1));
IkReal x612=((IkReal(1.00000000000000))*(cj1)*(sj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(-1.00000000000000))+(((IkReal(-1.00000000000000))*(x610)*(x611)))+(((IkReal(-1.00000000000000))*(r01)*(x612)))+(((sj2)*(x609)))+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x610))));
evalcond[3]=((((IkReal(-1.00000000000000))*(sj2)*(x611)))+(((IkReal(-1.00000000000000))*(x609)*(x610)))+(((IkReal(-1.00000000000000))*(r00)*(x612)))+(((cj1)*(cj2)*(r01))));
evalcond[4]=((IkReal(3.70500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(sj1)*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(x610)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x610)))+(((IkReal(-1.00000000000000))*(py)*(x612))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  && IKabs(evalcond[4]) < 0.0000010000000000  )
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x613=((IkReal(6.32911392405063))*(cj2));
IkReal x614=((IkReal(6.32911392405063))*(sj2));
if( IKabs(((IkReal(6.32911392405063))*(pz))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.917721518987342))+(((py)*(sj1)*(x614)))+(((cj1)*(px)*(x614)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x613)))+(((px)*(sj1)*(x613)))+(((IkReal(7.59493670886076))*(cj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(6.32911392405063))*(pz)))+IKsqr(((IkReal(0.917721518987342))+(((py)*(sj1)*(x614)))+(((cj1)*(px)*(x614)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x613)))+(((px)*(sj1)*(x613)))+(((IkReal(7.59493670886076))*(cj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(6.32911392405063))*(pz)), ((IkReal(0.917721518987342))+(((py)*(sj1)*(x614)))+(((cj1)*(px)*(x614)))+(((IkReal(-1.00000000000000))*(cj1)*(py)*(x613)))+(((px)*(sj1)*(x613)))+(((IkReal(7.59493670886076))*(cj2)))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[2];
IkReal x615=((IkReal(1.00000000000000))*(px));
evalcond[0]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(0.158000000000000))*(IKsin(j3)))));
evalcond[1]=((IkReal(-0.145000000000000))+(((cj1)*(cj2)*(py)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x615)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x615)))+(((IkReal(0.158000000000000))*(IKcos(j3))))+(((IkReal(-1.00000000000000))*(py)*(sj1)*(sj2)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
if( IKabs(((gconst58)*(r02))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst58)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst58)*(r02)), ((gconst58)*(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x616=IKsin(j3);
IkReal x617=IKcos(j3);
IkReal x618=((IkReal(1.00000000000000))*(px));
IkReal x619=((cj2)*(sj1));
IkReal x620=((cj1)*(cj2));
IkReal x621=((cj1)*(sj2));
IkReal x622=((IkReal(0.158000000000000))*(sj4));
IkReal x623=((IkReal(1.00000000000000))*(r00));
IkReal x624=((cj4)*(x617));
IkReal x625=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x626=((cj4)*(x616));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x626));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-2.16000000000000))*(x626)))+(((IkReal(-1.00000000000000))*(x616)*(x622))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x621)*(x623)))+(((IkReal(-1.00000000000000))*(r01)*(x625)))+(((r01)*(x620)))+(x624)+(((IkReal(-1.00000000000000))*(x619)*(x623))));
evalcond[3]=((IkReal(-0.145000000000000))+(((py)*(x620)))+(((IkReal(-1.00000000000000))*(x618)*(x619)))+(((IkReal(-1.00000000000000))*(x617)*(x622)))+(((IkReal(-2.16000000000000))*(x624)))+(((IkReal(-1.00000000000000))*(x618)*(x621)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(x625))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x627=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x628=((cj4)*(cj5));
IkReal x629=((IkReal(25.0000000000000))*(pz));
IkReal x630=((IkReal(54.0000000000000))*(r02));
if( IKabs(((IkReal(-0.253164556962025))*(x627)*(((x629)+(x630))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.253164556962025))*(x627)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x628)*(x630)))+(((x628)*(x629)))+(((IkReal(-3.95000000000000))*(r02)*(sj4))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.253164556962025))*(x627)*(((x629)+(x630)))))+IKsqr(((IkReal(0.253164556962025))*(x627)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x628)*(x630)))+(((x628)*(x629)))+(((IkReal(-3.95000000000000))*(r02)*(sj4)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j3array[0]=IKatan2(((IkReal(-0.253164556962025))*(x627)*(((x629)+(x630)))), ((IkReal(0.253164556962025))*(x627)*(((IKabs(sj5) != 0)?((IkReal)1/(sj5)):(IkReal)1.0e30))*(((((x628)*(x630)))+(((x628)*(x629)))+(((IkReal(-3.95000000000000))*(r02)*(sj4)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x631=IKsin(j3);
IkReal x632=IKcos(j3);
IkReal x633=((IkReal(1.00000000000000))*(px));
IkReal x634=((cj2)*(sj1));
IkReal x635=((cj1)*(sj2));
IkReal x636=((IkReal(0.158000000000000))*(sj4));
IkReal x637=((cj4)*(cj5));
IkReal x638=((IkReal(1.00000000000000))*(r00));
IkReal x639=((cj1)*(cj2));
IkReal x640=((sj5)*(x631));
IkReal x641=((IkReal(1.00000000000000))*(x632));
IkReal x642=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x643=((IkReal(2.16000000000000))*(x632));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj5)*(x641)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x631)*(x637))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x631)*(x636)))+(((sj5)*(x643)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x631)*(x637))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x642)))+(((IkReal(-1.00000000000000))*(x634)*(x638)))+(((r01)*(x639)))+(((IkReal(-1.00000000000000))*(x635)*(x638)))+(x640)+(((IkReal(-1.00000000000000))*(x637)*(x641))));
evalcond[3]=((IkReal(-0.145000000000000))+(((py)*(x639)))+(((IkReal(-1.00000000000000))*(x633)*(x634)))+(((IkReal(-1.00000000000000))*(x633)*(x635)))+(((IkReal(-2.16000000000000))*(x640)))+(((x637)*(x643)))+(((IkReal(-1.00000000000000))*(py)*(x642)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x632)*(x636))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x644=((cj4)*(cj5));
IkReal x645=((IkReal(1.00000000000000))*(r01));
IkReal x646=((cj1)*(cj2));
IkReal x647=((sj2)*(sj5));
IkReal x648=((cj1)*(r00));
IkReal x649=((cj2)*(r00)*(sj1));
if( IKabs(((gconst54)*(((((IkReal(-1.00000000000000))*(sj5)*(x649)))+(((r02)*(x644)))+(((r01)*(sj5)*(x646)))+(((IkReal(-1.00000000000000))*(x647)*(x648)))+(((IkReal(-1.00000000000000))*(sj1)*(x645)*(x647))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((x644)*(x649)))+(((r01)*(sj1)*(sj2)*(x644)))+(((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(x644)*(x645)*(x646)))+(((sj2)*(x644)*(x648))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((gconst54)*(((((IkReal(-1.00000000000000))*(sj5)*(x649)))+(((r02)*(x644)))+(((r01)*(sj5)*(x646)))+(((IkReal(-1.00000000000000))*(x647)*(x648)))+(((IkReal(-1.00000000000000))*(sj1)*(x645)*(x647)))))), ((gconst54)*(((((x644)*(x649)))+(((r01)*(sj1)*(sj2)*(x644)))+(((r02)*(sj5)))+(((IkReal(-1.00000000000000))*(x644)*(x645)*(x646)))+(((sj2)*(x644)*(x648)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x650=IKsin(j3);
IkReal x651=IKcos(j3);
IkReal x652=((IkReal(1.00000000000000))*(px));
IkReal x653=((cj2)*(sj1));
IkReal x654=((cj1)*(sj2));
IkReal x655=((IkReal(0.158000000000000))*(sj4));
IkReal x656=((cj4)*(cj5));
IkReal x657=((IkReal(1.00000000000000))*(r00));
IkReal x658=((cj1)*(cj2));
IkReal x659=((sj5)*(x650));
IkReal x660=((IkReal(1.00000000000000))*(x651));
IkReal x661=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x662=((IkReal(2.16000000000000))*(x651));
evalcond[0]=((((IkReal(-1.00000000000000))*(x650)*(x656)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(sj5)*(x660))));
evalcond[1]=((((IkReal(2.16000000000000))*(x650)*(x656)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x650)*(x655)))+(((sj5)*(x662))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r01)*(x661)))+(((IkReal(-1.00000000000000))*(x656)*(x660)))+(x659)+(((IkReal(-1.00000000000000))*(x653)*(x657)))+(((r01)*(x658)))+(((IkReal(-1.00000000000000))*(x654)*(x657))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x651)*(x655)))+(((IkReal(-2.16000000000000))*(x659)))+(((py)*(x658)))+(((x656)*(x662)))+(((IkReal(-1.00000000000000))*(py)*(x661)))+(((IkReal(-1.00000000000000))*(x652)*(x653)))+(((IkReal(-1.00000000000000))*(x652)*(x654)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[1], cj3array[1], sj3array[1];
bool j3valid[1]={false};
_nj3 = 1;
IkReal x663=((sj1)*(sj2));
IkReal x664=((cj2)*(sj1));
IkReal x665=((cj1)*(cj2));
IkReal x666=((IkReal(2.16000000000000))*(r01));
IkReal x667=((IkReal(2.16000000000000))*(r00));
IkReal x668=((cj1)*(sj2));
if( IKabs(((IkReal(500.000000000000))*(gconst53)*(((((IkReal(-2.16000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-500.000000000000))*(gconst53)*(((IkReal(0.145000000000000))+(((IkReal(-1.00000000000000))*(x665)*(x666)))+(((x667)*(x668)))+(((x664)*(x667)))+(((x663)*(x666)))+(((IkReal(-1.00000000000000))*(py)*(x665)))+(((px)*(x664)))+(((px)*(x668)))+(((py)*(x663)))+(((IkReal(1.20000000000000))*(cj2))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j3array[0]=IKatan2(((IkReal(500.000000000000))*(gconst53)*(((((IkReal(-2.16000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(pz)))))), ((IkReal(-500.000000000000))*(gconst53)*(((IkReal(0.145000000000000))+(((IkReal(-1.00000000000000))*(x665)*(x666)))+(((x667)*(x668)))+(((x664)*(x667)))+(((x663)*(x666)))+(((IkReal(-1.00000000000000))*(py)*(x665)))+(((px)*(x664)))+(((px)*(x668)))+(((py)*(x663)))+(((IkReal(1.20000000000000))*(cj2)))))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
for(int ij3 = 0; ij3 < 1; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 1; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
{
IkReal evalcond[4];
IkReal x669=IKsin(j3);
IkReal x670=IKcos(j3);
IkReal x671=((IkReal(2.16000000000000))*(r02));
IkReal x672=((cj2)*(sj1));
IkReal x673=((IkReal(1.00000000000000))*(pz));
IkReal x674=((IkReal(2.16000000000000))*(r00));
IkReal x675=((cj4)*(sj2));
IkReal x676=((cj2)*(cj4));
IkReal x677=((IkReal(1.00000000000000))*(px));
IkReal x678=((IkReal(1.00000000000000))*(py));
IkReal x679=((cj1)*(sj2));
IkReal x680=((IkReal(2.16000000000000))*(r01)*(sj1));
IkReal x681=((IkReal(2.16000000000000))*(cj1)*(cj2)*(r01));
IkReal x682=((IkReal(0.158000000000000))*(sj4)*(x669));
IkReal x683=((IkReal(0.158000000000000))*(sj4)*(x670));
evalcond[0]=((((IkReal(-1.00000000000000))*(x682)))+(((IkReal(-1.00000000000000))*(x671)))+(((IkReal(-1.00000000000000))*(x673))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj4)*(x682)))+(((IkReal(-1.00000000000000))*(cj4)*(x671)))+(((IkReal(-1.00000000000000))*(cj4)*(x673))));
evalcond[2]=((IkReal(0.145000000000000))+(((x672)*(x674)))+(((IkReal(-1.00000000000000))*(x681)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(x678)))+(((sj2)*(x680)))+(((x674)*(x679)))+(((IkReal(1.20000000000000))*(cj2)))+(x683)+(((py)*(sj1)*(sj2)))+(((px)*(x679)))+(((px)*(x672))));
evalcond[3]=((((IkReal(-0.145000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj1)*(x674)*(x675)))+(((IkReal(-1.20000000000000))*(x676)))+(((IkReal(-1.00000000000000))*(cj1)*(x675)*(x677)))+(((IkReal(-1.00000000000000))*(cj4)*(x683)))+(((IkReal(-1.00000000000000))*(x675)*(x680)))+(((IkReal(-1.00000000000000))*(sj1)*(x675)*(x678)))+(((IkReal(-1.00000000000000))*(cj4)*(x672)*(x674)))+(((IkReal(-1.00000000000000))*(cj4)*(x672)*(x677)))+(((cj1)*(py)*(x676)))+(((IkReal(2.16000000000000))*(cj1)*(r01)*(x676))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x684=(sj3)*(sj3);
IkReal x685=(cj3)*(cj3);
dummyeval[0]=((((IkReal(-1.00000000000000))*(x685)))+(((IkReal(-1.00000000000000))*(x684))));
dummyeval[1]=((((cj4)*(x684)))+(((cj4)*(x685))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj3;
dummyeval[1]=sj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(-0.158000000000000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x686=((r01)*(sj2));
IkReal x687=((IkReal(1.00000000000000))*(cj1));
IkReal x688=((r00)*(sj2));
IkReal x689=((cj2)*(sj1));
if( IKabs(((((cj1)*(x688)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x687)))+(((sj1)*(x686)))+(((r00)*(x689))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(x686)*(x687)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x687)))+(((IkReal(-1.00000000000000))*(r01)*(x689)))+(((sj1)*(x688))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj1)*(x688)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x687)))+(((sj1)*(x686)))+(((r00)*(x689)))))+IKsqr(((((IkReal(-1.00000000000000))*(x686)*(x687)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x687)))+(((IkReal(-1.00000000000000))*(r01)*(x689)))+(((sj1)*(x688)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj1)*(x688)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x687)))+(((sj1)*(x686)))+(((r00)*(x689)))), ((((IkReal(-1.00000000000000))*(x686)*(x687)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x687)))+(((IkReal(-1.00000000000000))*(r01)*(x689)))+(((sj1)*(x688)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x690=IKcos(j5);
IkReal x691=IKsin(j5);
IkReal x692=((IkReal(1.00000000000000))*(px));
IkReal x693=((cj2)*(sj1));
IkReal x694=((cj1)*(py));
IkReal x695=((cj1)*(sj2));
IkReal x696=((IkReal(1.00000000000000))*(r01));
IkReal x697=((sj1)*(sj2));
IkReal x698=((IkReal(1.00000000000000))*(r00));
IkReal x699=((IkReal(1.00000000000000))*(py));
IkReal x700=((cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x693)*(x696)))+(((IkReal(-1.00000000000000))*(x695)*(x696)))+(((IkReal(-1.00000000000000))*(x698)*(x700)))+(((r00)*(x697)))+(((IkReal(-1.00000000000000))*(x690))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x696)*(x697)))+(((r01)*(x700)))+(((IkReal(-1.00000000000000))*(x693)*(x698)))+(((IkReal(-1.00000000000000))*(x695)*(x698)))+(x691));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(sj2)*(x694)))+(((IkReal(-1.00000000000000))*(x693)*(x699)))+(((IkReal(-1.00000000000000))*(x692)*(x700)))+(((px)*(x697)))+(((IkReal(2.16000000000000))*(x690))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x697)*(x699)))+(((IkReal(-2.16000000000000))*(x691)))+(((IkReal(-1.00000000000000))*(x692)*(x693)))+(((IkReal(-1.00000000000000))*(x692)*(x695)))+(((cj2)*(x694)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(0.158000000000000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x701=((cj1)*(cj2));
IkReal x702=((r01)*(sj1));
IkReal x703=((cj1)*(sj2));
IkReal x704=((r00)*(sj1));
if( IKabs(((((r00)*(x703)))+(((sj2)*(x702)))+(((IkReal(-1.00000000000000))*(r01)*(x701)))+(((cj2)*(x704))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r01)*(x703)))+(((r00)*(x701)))+(((IkReal(-1.00000000000000))*(sj2)*(x704)))+(((cj2)*(x702))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((r00)*(x703)))+(((sj2)*(x702)))+(((IkReal(-1.00000000000000))*(r01)*(x701)))+(((cj2)*(x704)))))+IKsqr(((((r01)*(x703)))+(((r00)*(x701)))+(((IkReal(-1.00000000000000))*(sj2)*(x704)))+(((cj2)*(x702)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((r00)*(x703)))+(((sj2)*(x702)))+(((IkReal(-1.00000000000000))*(r01)*(x701)))+(((cj2)*(x704)))), ((((r01)*(x703)))+(((r00)*(x701)))+(((IkReal(-1.00000000000000))*(sj2)*(x704)))+(((cj2)*(x702)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x705=IKcos(j5);
IkReal x706=IKsin(j5);
IkReal x707=((IkReal(1.00000000000000))*(px));
IkReal x708=((cj2)*(sj1));
IkReal x709=((cj1)*(py));
IkReal x710=((cj1)*(sj2));
IkReal x711=((IkReal(1.00000000000000))*(r01));
IkReal x712=((sj1)*(sj2));
IkReal x713=((IkReal(1.00000000000000))*(r00));
IkReal x714=((IkReal(1.00000000000000))*(py));
IkReal x715=((cj1)*(cj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x710)*(x711)))+(((r00)*(x712)))+(((IkReal(-1.00000000000000))*(x708)*(x711)))+(x705)+(((IkReal(-1.00000000000000))*(x713)*(x715))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x711)*(x712)))+(((r01)*(x715)))+(((IkReal(-1.00000000000000))*(x710)*(x713)))+(((IkReal(-1.00000000000000))*(x708)*(x713)))+(x706));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x707)*(x715)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-2.16000000000000))*(x705)))+(((IkReal(-1.00000000000000))*(sj2)*(x709)))+(((px)*(x712)))+(((IkReal(-1.00000000000000))*(x708)*(x714))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x707)*(x708)))+(((IkReal(-1.00000000000000))*(x707)*(x710)))+(((IkReal(-2.16000000000000))*(x706)))+(((IkReal(-1.00000000000000))*(x712)*(x714)))+(((cj2)*(x709)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x716=((IkReal(0.462962962962963))*(cj1));
IkReal x717=((IkReal(0.462962962962963))*(sj1));
if( IKabs(((IkReal(-0.0671296296296296))+(((cj2)*(py)*(x716)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x716)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x717)))+(((IkReal(-0.555555555555556))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x717))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-0.0671296296296296))+(((cj2)*(py)*(x716)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x716)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x717)))+(((IkReal(-0.555555555555556))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x717)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-0.0671296296296296))+(((cj2)*(py)*(x716)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x716)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x717)))+(((IkReal(-0.555555555555556))*(cj2)))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x717)))), ((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x718=IKcos(j5);
IkReal x719=IKsin(j5);
IkReal x720=((IkReal(1.00000000000000))*(sj1));
IkReal x721=((py)*(sj2));
IkReal x722=((cj2)*(px));
IkReal x723=((IkReal(1.00000000000000))*(cj1));
IkReal x724=((cj2)*(r00));
IkReal x725=((cj1)*(cj2));
IkReal x726=((r01)*(sj2));
IkReal x727=((r00)*(sj2));
IkReal x728=((px)*(sj2));
IkReal x729=((sj4)*(x718));
IkReal x730=((cj4)*(x718));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x730))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x730))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x723)*(x724)))+(((IkReal(-1.00000000000000))*(x723)*(x726)))+(((IkReal(-1.00000000000000))*(x729)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x720)))+(((sj1)*(x727))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x723)*(x727)))+(((r01)*(x725)))+(((IkReal(-1.00000000000000))*(x720)*(x724)))+(((IkReal(-1.00000000000000))*(x720)*(x726)))+(x719));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x722)*(x723)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x720)))+(((IkReal(-1.00000000000000))*(x721)*(x723)))+(((IkReal(2.16000000000000))*(x729)))+(((sj1)*(x728))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x723)*(x728)))+(((IkReal(-2.16000000000000))*(x719)))+(((py)*(x725)))+(((IkReal(-1.00000000000000))*(x720)*(x722)))+(((IkReal(-1.00000000000000))*(x720)*(x721)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj4))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj4)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), ((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj4)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x731=IKcos(j5);
IkReal x732=IKsin(j5);
IkReal x733=((IkReal(1.00000000000000))*(sj1));
IkReal x734=((py)*(sj2));
IkReal x735=((cj2)*(px));
IkReal x736=((IkReal(1.00000000000000))*(cj1));
IkReal x737=((cj2)*(r00));
IkReal x738=((cj1)*(cj2));
IkReal x739=((r01)*(sj2));
IkReal x740=((r00)*(sj2));
IkReal x741=((px)*(sj2));
IkReal x742=((sj4)*(x731));
IkReal x743=((cj4)*(x731));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x743))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x743))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x736)*(x737)))+(((IkReal(-1.00000000000000))*(x736)*(x739)))+(((sj1)*(x740)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x733)))+(((IkReal(-1.00000000000000))*(x742))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x733)*(x739)))+(((IkReal(-1.00000000000000))*(x733)*(x737)))+(((r01)*(x738)))+(x732)+(((IkReal(-1.00000000000000))*(x736)*(x740))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x735)*(x736)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x733)))+(((IkReal(-1.00000000000000))*(x734)*(x736)))+(((sj1)*(x741)))+(((IkReal(2.16000000000000))*(x742))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x732)))+(((IkReal(-1.00000000000000))*(x733)*(x734)))+(((IkReal(-1.00000000000000))*(x733)*(x735)))+(((py)*(x738)))+(((IkReal(-1.00000000000000))*(x736)*(x741)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))))+IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((cj2)*(r00)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r01)))+(((cj1)*(r00)*(sj2)))+(((r01)*(sj1)*(sj2)))), ((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x744=IKcos(j5);
IkReal x745=IKsin(j5);
IkReal x746=((IkReal(1.00000000000000))*(sj1));
IkReal x747=((py)*(sj2));
IkReal x748=((cj2)*(px));
IkReal x749=((IkReal(1.00000000000000))*(cj1));
IkReal x750=((cj2)*(r00));
IkReal x751=((cj1)*(cj2));
IkReal x752=((r01)*(sj2));
IkReal x753=((r00)*(sj2));
IkReal x754=((px)*(sj2));
IkReal x755=((sj4)*(x744));
IkReal x756=((cj4)*(x744));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x756))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x756))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r01)*(x746)))+(((sj1)*(x753)))+(((IkReal(-1.00000000000000))*(x755)))+(((IkReal(-1.00000000000000))*(x749)*(x752)))+(((IkReal(-1.00000000000000))*(x749)*(x750))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x746)*(x752)))+(((IkReal(-1.00000000000000))*(x746)*(x750)))+(((r01)*(x751)))+(x745)+(((IkReal(-1.00000000000000))*(x749)*(x753))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x747)*(x749)))+(((sj1)*(x754)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x746)))+(((IkReal(2.16000000000000))*(x755)))+(((IkReal(-1.00000000000000))*(x748)*(x749))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x745)))+(((py)*(x751)))+(((IkReal(-1.00000000000000))*(x746)*(x747)))+(((IkReal(-1.00000000000000))*(x746)*(x748)))+(((IkReal(-1.00000000000000))*(x749)*(x754)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j3)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj4;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(0.158000000000000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x757=((r00)*(sj1));
IkReal x758=((IkReal(1.00000000000000))*(cj2));
IkReal x759=((r01)*(sj1));
IkReal x760=((IkReal(1.00000000000000))*(cj1)*(sj2));
if( IKabs(((((IkReal(-1.00000000000000))*(r00)*(x760)))+(((IkReal(-1.00000000000000))*(sj2)*(x759)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x757)*(x758))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x758)))+(((IkReal(-1.00000000000000))*(r01)*(x760)))+(((IkReal(-1.00000000000000))*(x758)*(x759)))+(((sj2)*(x757))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r00)*(x760)))+(((IkReal(-1.00000000000000))*(sj2)*(x759)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x757)*(x758)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x758)))+(((IkReal(-1.00000000000000))*(r01)*(x760)))+(((IkReal(-1.00000000000000))*(x758)*(x759)))+(((sj2)*(x757)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r00)*(x760)))+(((IkReal(-1.00000000000000))*(sj2)*(x759)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x757)*(x758)))), ((((IkReal(-1.00000000000000))*(cj1)*(r00)*(x758)))+(((IkReal(-1.00000000000000))*(r01)*(x760)))+(((IkReal(-1.00000000000000))*(x758)*(x759)))+(((sj2)*(x757)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x761=IKcos(j5);
IkReal x762=IKsin(j5);
IkReal x763=((IkReal(1.00000000000000))*(sj1));
IkReal x764=((py)*(sj2));
IkReal x765=((cj2)*(px));
IkReal x766=((cj1)*(cj2));
IkReal x767=((IkReal(1.00000000000000))*(r00));
IkReal x768=((r01)*(sj2));
IkReal x769=((IkReal(1.00000000000000))*(cj1));
IkReal x770=((sj1)*(sj2));
evalcond[0]=((((IkReal(-1.00000000000000))*(x766)*(x767)))+(((IkReal(-1.00000000000000))*(x761)))+(((r00)*(x770)))+(((IkReal(-1.00000000000000))*(x768)*(x769)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x763))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x763)*(x768)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x767)))+(((IkReal(-1.00000000000000))*(x762)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x763)))+(((r01)*(x766))));
evalcond[2]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x765)*(x769)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x763)))+(((IkReal(-1.00000000000000))*(x764)*(x769)))+(((px)*(x770)))+(((IkReal(2.16000000000000))*(x761))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x763)*(x764)))+(((IkReal(-1.00000000000000))*(x763)*(x765)))+(((IkReal(2.16000000000000))*(x762)))+(((py)*(x766)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x769))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-1.00000000000000))*(r02));
evalcond[2]=((IkReal(-0.158000000000000))+(((IkReal(-1.00000000000000))*(pz))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x771=((cj1)*(r00));
IkReal x772=((cj2)*(r01));
IkReal x773=((IkReal(1.00000000000000))*(sj2));
IkReal x774=((r00)*(sj1));
if( IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x773)))+(((cj1)*(x772)))+(((IkReal(-1.00000000000000))*(cj2)*(x774)))+(((IkReal(-1.00000000000000))*(x771)*(x773))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(x771)))+(((sj1)*(x772)))+(((IkReal(-1.00000000000000))*(x773)*(x774))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x773)))+(((cj1)*(x772)))+(((IkReal(-1.00000000000000))*(cj2)*(x774)))+(((IkReal(-1.00000000000000))*(x771)*(x773)))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(x771)))+(((sj1)*(x772)))+(((IkReal(-1.00000000000000))*(x773)*(x774)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(r01)*(sj1)*(x773)))+(((cj1)*(x772)))+(((IkReal(-1.00000000000000))*(cj2)*(x774)))+(((IkReal(-1.00000000000000))*(x771)*(x773)))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(x771)))+(((sj1)*(x772)))+(((IkReal(-1.00000000000000))*(x773)*(x774)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x775=IKcos(j5);
IkReal x776=IKsin(j5);
IkReal x777=((cj1)*(py));
IkReal x778=((r00)*(sj1));
IkReal x779=((IkReal(1.00000000000000))*(cj2));
IkReal x780=((r01)*(sj1));
IkReal x781=((px)*(sj1));
IkReal x782=((IkReal(1.00000000000000))*(sj2));
IkReal x783=((py)*(sj1));
IkReal x784=((cj1)*(x782));
evalcond[0]=((((IkReal(-1.00000000000000))*(r01)*(x784)))+(((sj2)*(x778)))+(x775)+(((IkReal(-1.00000000000000))*(cj1)*(r00)*(x779)))+(((IkReal(-1.00000000000000))*(x779)*(x780))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x778)*(x779)))+(((IkReal(-1.00000000000000))*(r00)*(x784)))+(((IkReal(-1.00000000000000))*(x776)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(x780)*(x782))));
evalcond[2]=((IkReal(1.54500000000000))+(((sj2)*(x781)))+(((IkReal(-1.00000000000000))*(cj1)*(px)*(x779)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x777)*(x782)))+(((IkReal(-1.00000000000000))*(x779)*(x783)))+(((IkReal(-2.16000000000000))*(x775))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(px)*(x784)))+(((cj2)*(x777)))+(((IkReal(2.16000000000000))*(x776)))+(((IkReal(-1.00000000000000))*(x782)*(x783)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x779)*(x781))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x785=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x785)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x785))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj4))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x785)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x785)))))+IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj4)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x785)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x785)))), ((IkReal(0.0185185185185185))*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))*(((((IkReal(-25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj4)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x786=IKcos(j5);
IkReal x787=IKsin(j5);
IkReal x788=((IkReal(1.00000000000000))*(sj1));
IkReal x789=((py)*(sj2));
IkReal x790=((cj2)*(px));
IkReal x791=((cj1)*(cj2));
IkReal x792=((IkReal(1.00000000000000))*(r00));
IkReal x793=((r01)*(sj2));
IkReal x794=((IkReal(1.00000000000000))*(cj1));
IkReal x795=((sj1)*(sj2));
IkReal x796=((sj4)*(x786));
IkReal x797=((cj4)*(x786));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x797));
evalcond[1]=((((IkReal(-2.16000000000000))*(x797)))+(((IkReal(0.158000000000000))*(sj4)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((r00)*(x795)))+(((IkReal(-1.00000000000000))*(x793)*(x794)))+(((IkReal(-1.00000000000000))*(x791)*(x792)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x788)))+(((IkReal(-1.00000000000000))*(x796))));
evalcond[3]=((((r01)*(x791)))+(((IkReal(-1.00000000000000))*(x788)*(x793)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x792)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x788)))+(((IkReal(-1.00000000000000))*(x787))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x788)))+(((IkReal(0.158000000000000))*(cj4)))+(((px)*(x795)))+(((IkReal(2.16000000000000))*(x796)))+(((IkReal(-1.00000000000000))*(x790)*(x794)))+(((IkReal(-1.00000000000000))*(x789)*(x794))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x788)*(x789)))+(((IkReal(-1.00000000000000))*(x788)*(x790)))+(((IkReal(2.16000000000000))*(x787)))+(((py)*(x791)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x794)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x798=((IkReal(0.462962962962963))*(cj1));
IkReal x799=((IkReal(0.462962962962963))*(sj1));
if( IKabs(((IkReal(0.0671296296296296))+(((cj2)*(px)*(x799)))+(((py)*(sj2)*(x799)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x798)))+(((IkReal(0.555555555555556))*(cj2)))+(((px)*(sj2)*(x798))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0671296296296296))+(((cj2)*(px)*(x799)))+(((py)*(sj2)*(x799)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x798)))+(((IkReal(0.555555555555556))*(cj2)))+(((px)*(sj2)*(x798)))))+IKsqr(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0671296296296296))+(((cj2)*(px)*(x799)))+(((py)*(sj2)*(x799)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x798)))+(((IkReal(0.555555555555556))*(cj2)))+(((px)*(sj2)*(x798)))), ((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x800=IKcos(j5);
IkReal x801=IKsin(j5);
IkReal x802=((IkReal(1.00000000000000))*(sj1));
IkReal x803=((py)*(sj2));
IkReal x804=((cj2)*(px));
IkReal x805=((cj1)*(cj2));
IkReal x806=((IkReal(1.00000000000000))*(r00));
IkReal x807=((r01)*(sj2));
IkReal x808=((IkReal(1.00000000000000))*(cj1));
IkReal x809=((sj1)*(sj2));
IkReal x810=((sj4)*(x800));
IkReal x811=((cj4)*(x800));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x811));
evalcond[1]=((((IkReal(0.158000000000000))*(sj4)))+(((IkReal(-2.16000000000000))*(x811)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x807)*(x808)))+(((IkReal(-1.00000000000000))*(x805)*(x806)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x802)))+(((IkReal(-1.00000000000000))*(x810)))+(((r00)*(x809))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x806)))+(((IkReal(-1.00000000000000))*(x801)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x802)))+(((r01)*(x805)))+(((IkReal(-1.00000000000000))*(x802)*(x807))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x804)*(x808)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x802)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x803)*(x808)))+(((px)*(x809)))+(((IkReal(2.16000000000000))*(x810))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x808)))+(((IkReal(2.16000000000000))*(x801)))+(((py)*(x805)))+(((IkReal(-1.00000000000000))*(x802)*(x803)))+(((IkReal(-1.00000000000000))*(x802)*(x804)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x812=((IkReal(1.00000000000000))*(r00));
if( IKabs(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x812)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x812)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x812)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x812)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))))+IKsqr(((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x812)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x812)))+(((IkReal(-1.00000000000000))*(r01)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r01)))), ((r02)*(((IKabs(cj4) != 0)?((IkReal)1/(cj4)):(IkReal)1.0e30))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x813=IKcos(j5);
IkReal x814=IKsin(j5);
IkReal x815=((IkReal(1.00000000000000))*(sj1));
IkReal x816=((py)*(sj2));
IkReal x817=((cj2)*(px));
IkReal x818=((cj1)*(cj2));
IkReal x819=((IkReal(1.00000000000000))*(r00));
IkReal x820=((r01)*(sj2));
IkReal x821=((IkReal(1.00000000000000))*(cj1));
IkReal x822=((sj1)*(sj2));
IkReal x823=((sj4)*(x813));
IkReal x824=((cj4)*(x813));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(x824));
evalcond[1]=((((IkReal(0.158000000000000))*(sj4)))+(((IkReal(-2.16000000000000))*(x824)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x818)*(x819)))+(((r00)*(x822)))+(((IkReal(-1.00000000000000))*(x823)))+(((IkReal(-1.00000000000000))*(x820)*(x821)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x815))));
evalcond[3]=((((r01)*(x818)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x819)))+(((IkReal(-1.00000000000000))*(x815)*(x820)))+(((IkReal(-1.00000000000000))*(x814)))+(((IkReal(-1.00000000000000))*(cj2)*(r00)*(x815))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x816)*(x821)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((px)*(x822)))+(((IkReal(-1.00000000000000))*(x817)*(x821)))+(((IkReal(2.16000000000000))*(x823)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x815))));
evalcond[5]=((IkReal(-0.145000000000000))+(((py)*(x818)))+(((IkReal(-1.00000000000000))*(x815)*(x817)))+(((IkReal(-1.00000000000000))*(x815)*(x816)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x821)))+(((IkReal(2.16000000000000))*(x814)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x825=((IkReal(1.00000000000000))*(r01));
IkReal x826=((cj1)*(sj2));
IkReal x827=((sj1)*(sj2));
IkReal x828=((cj2)*(sj1));
IkReal x829=((IkReal(1.00000000000000))*(py));
IkReal x830=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x827)))+(((IkReal(-1.00000000000000))*(r00)*(x830)))+(((IkReal(-1.00000000000000))*(x825)*(x828)))+(((IkReal(-1.00000000000000))*(x825)*(x826))));
evalcond[2]=((IkReal(1.70300000000000))+(((IkReal(-1.00000000000000))*(x828)*(x829)))+(((IkReal(1.20000000000000))*(sj2)))+(((px)*(x827)))+(((IkReal(-1.00000000000000))*(x826)*(x829)))+(((IkReal(-1.00000000000000))*(px)*(x830))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst59;
gconst59=IKsign(((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x831=((r01)*(sj1));
IkReal x832=((cj1)*(r00));
IkReal x833=((IkReal(1.00000000000000))*(cj2));
IkReal x834=((cj1)*(r01));
IkReal x835=((r00)*(sj1));
IkReal x836=((cj3)*(sj2));
IkReal x837=((IkReal(1.00000000000000))*(sj2)*(sj3));
if( IKabs(((gconst59)*(((((IkReal(-1.00000000000000))*(x832)*(x837)))+(((IkReal(-1.00000000000000))*(sj3)*(x833)*(x835)))+(((cj2)*(sj3)*(x834)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x831)*(x837))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst59)*(((((x831)*(x836)))+(((IkReal(-1.00000000000000))*(cj3)*(x833)*(x834)))+(((cj2)*(cj3)*(x835)))+(((x832)*(x836)))+(((r02)*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst59)*(((((IkReal(-1.00000000000000))*(x832)*(x837)))+(((IkReal(-1.00000000000000))*(sj3)*(x833)*(x835)))+(((cj2)*(sj3)*(x834)))+(((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x831)*(x837)))))), ((gconst59)*(((((x831)*(x836)))+(((IkReal(-1.00000000000000))*(cj3)*(x833)*(x834)))+(((cj2)*(cj3)*(x835)))+(((x832)*(x836)))+(((r02)*(sj3)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x838=IKsin(j5);
IkReal x839=IKcos(j5);
IkReal x840=((IkReal(2.16000000000000))*(sj3));
IkReal x841=((IkReal(1.00000000000000))*(px));
IkReal x842=((cj2)*(sj1));
IkReal x843=((cj1)*(cj2));
IkReal x844=((cj1)*(sj2));
IkReal x845=((IkReal(1.00000000000000))*(r00));
IkReal x846=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x847=((cj3)*(x838));
IkReal x848=((IkReal(1.00000000000000))*(x839));
evalcond[0]=((((IkReal(-1.00000000000000))*(sj3)*(x848)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x847))));
evalcond[1]=((((x839)*(x840)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x847))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x842)*(x845)))+(((r01)*(x843)))+(((IkReal(-1.00000000000000))*(x844)*(x845)))+(((IkReal(-1.00000000000000))*(r01)*(x846)))+(((sj3)*(x838)))+(((IkReal(-1.00000000000000))*(cj3)*(x848))));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x841)*(x844)))+(((IkReal(-1.00000000000000))*(x841)*(x842)))+(((IkReal(-1.00000000000000))*(py)*(x846)))+(((py)*(x843)))+(((IkReal(-1.00000000000000))*(x838)*(x840)))+(((IkReal(2.16000000000000))*(cj3)*(x839)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x849=((IkReal(1.00000000000000))*(r01));
IkReal x850=((cj1)*(sj2));
IkReal x851=((sj1)*(sj2));
IkReal x852=((cj2)*(sj1));
IkReal x853=((IkReal(1.00000000000000))*(py));
IkReal x854=((IkReal(1.00000000000000))*(cj1)*(cj2));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j4)), IkReal(6.28318530717959))));
evalcond[1]=((((r00)*(x851)))+(((IkReal(-1.00000000000000))*(r00)*(x854)))+(((IkReal(-1.00000000000000))*(x849)*(x852)))+(((IkReal(-1.00000000000000))*(x849)*(x850))));
evalcond[2]=((IkReal(1.38700000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x852)*(x853)))+(((IkReal(-1.00000000000000))*(x850)*(x853)))+(((px)*(x851)))+(((IkReal(-1.00000000000000))*(px)*(x854))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst60;
gconst60=IKsign(((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj3)*(cj3))))+(((IkReal(-1.00000000000000))*((sj3)*(sj3)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x855=((r01)*(sj1));
IkReal x856=((cj1)*(r00));
IkReal x857=((cj2)*(cj3));
IkReal x858=((cj1)*(r01));
IkReal x859=((IkReal(1.00000000000000))*(sj3));
IkReal x860=((r00)*(sj1));
IkReal x861=((sj2)*(x859));
IkReal x862=((IkReal(1.00000000000000))*(cj3)*(sj2));
if( IKabs(((gconst60)*(((((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x856)*(x861)))+(((IkReal(-1.00000000000000))*(cj2)*(x859)*(x860)))+(((cj2)*(sj3)*(x858)))+(((IkReal(-1.00000000000000))*(x855)*(x861))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst60)*(((((IkReal(-1.00000000000000))*(x857)*(x860)))+(((IkReal(-1.00000000000000))*(r02)*(x859)))+(((IkReal(-1.00000000000000))*(x856)*(x862)))+(((x857)*(x858)))+(((IkReal(-1.00000000000000))*(x855)*(x862))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j5array[0]=IKatan2(((gconst60)*(((((cj3)*(r02)))+(((IkReal(-1.00000000000000))*(x856)*(x861)))+(((IkReal(-1.00000000000000))*(cj2)*(x859)*(x860)))+(((cj2)*(sj3)*(x858)))+(((IkReal(-1.00000000000000))*(x855)*(x861)))))), ((gconst60)*(((((IkReal(-1.00000000000000))*(x857)*(x860)))+(((IkReal(-1.00000000000000))*(r02)*(x859)))+(((IkReal(-1.00000000000000))*(x856)*(x862)))+(((x857)*(x858)))+(((IkReal(-1.00000000000000))*(x855)*(x862)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[4];
IkReal x863=IKsin(j5);
IkReal x864=IKcos(j5);
IkReal x865=((IkReal(2.16000000000000))*(sj3));
IkReal x866=((IkReal(1.00000000000000))*(px));
IkReal x867=((cj2)*(sj1));
IkReal x868=((cj1)*(cj2));
IkReal x869=((cj1)*(sj2));
IkReal x870=((IkReal(1.00000000000000))*(r00));
IkReal x871=((IkReal(1.00000000000000))*(sj1)*(sj2));
IkReal x872=((cj3)*(x863));
IkReal x873=((cj3)*(x864));
evalcond[0]=((((sj3)*(x864)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x872))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x864)*(x865)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x872))));
evalcond[2]=((((sj3)*(x863)))+(((r01)*(x868)))+(((IkReal(-1.00000000000000))*(r01)*(x871)))+(((IkReal(-1.00000000000000))*(x869)*(x870)))+(((IkReal(-1.00000000000000))*(x867)*(x870)))+(x873));
evalcond[3]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x866)*(x869)))+(((IkReal(-1.00000000000000))*(x866)*(x867)))+(((IkReal(-2.16000000000000))*(x873)))+(((IkReal(-1.00000000000000))*(py)*(x871)))+(((py)*(x868)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x863)*(x865))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x874=((IkReal(1.00000000000000))*(r01));
if( IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x874)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x874)))+(((r00)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj3)))))))+IKsqr(((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x874)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x874)))+(((r00)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(3.95000000000000))*(sj3)))))), ((((IkReal(-1.00000000000000))*(cj1)*(cj2)*(r00)))+(((IkReal(-1.00000000000000))*(cj1)*(sj2)*(x874)))+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x874)))+(((r00)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x875=IKsin(j5);
IkReal x876=IKcos(j5);
IkReal x877=((IkReal(1.00000000000000))*(sj2));
IkReal x878=((cj1)*(r00));
IkReal x879=((py)*(sj1));
IkReal x880=((IkReal(1.00000000000000))*(cj2));
IkReal x881=((px)*(sj1));
IkReal x882=((cj1)*(py));
IkReal x883=((r00)*(sj1));
IkReal x884=((r01)*(sj1));
IkReal x885=((cj1)*(r01));
IkReal x886=((cj1)*(px));
IkReal x887=((sj3)*(x875));
IkReal x888=((cj3)*(x875));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x888))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x888))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x876)))+(((IkReal(-1.00000000000000))*(x877)*(x885)))+(((sj2)*(x883)))+(((IkReal(-1.00000000000000))*(x880)*(x884)))+(((IkReal(-1.00000000000000))*(x878)*(x880))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x877)*(x878)))+(((cj2)*(x885)))+(((IkReal(-1.00000000000000))*(x877)*(x884)))+(x887)+(((IkReal(-1.00000000000000))*(x880)*(x883))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x879)*(x880)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x877)*(x882)))+(((sj2)*(x881)))+(((IkReal(2.16000000000000))*(x876)))+(((IkReal(-1.00000000000000))*(x880)*(x886))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(x877)*(x879)))+(((IkReal(-0.158000000000000))*(cj3)))+(((cj2)*(x882)))+(((IkReal(-2.16000000000000))*(x887)))+(((IkReal(-1.00000000000000))*(x877)*(x886)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(x880)*(x881))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x889=((IkReal(0.462962962962963))*(sj1));
IkReal x890=((IkReal(0.462962962962963))*(cj1));
if( IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-0.715277777777778))+(((py)*(sj2)*(x890)))+(((cj2)*(px)*(x890)))+(((cj2)*(py)*(x889)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x889)))+(((IkReal(-0.555555555555556))*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((IkReal(-0.715277777777778))+(((py)*(sj2)*(x890)))+(((cj2)*(px)*(x890)))+(((cj2)*(py)*(x889)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x889)))+(((IkReal(-0.555555555555556))*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((IkReal(-0.715277777777778))+(((py)*(sj2)*(x890)))+(((cj2)*(px)*(x890)))+(((cj2)*(py)*(x889)))+(((IkReal(-1.00000000000000))*(px)*(sj2)*(x889)))+(((IkReal(-0.555555555555556))*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x891=IKsin(j5);
IkReal x892=IKcos(j5);
IkReal x893=((IkReal(1.00000000000000))*(sj2));
IkReal x894=((cj1)*(r00));
IkReal x895=((py)*(sj1));
IkReal x896=((IkReal(1.00000000000000))*(cj2));
IkReal x897=((px)*(sj1));
IkReal x898=((cj1)*(py));
IkReal x899=((r00)*(sj1));
IkReal x900=((r01)*(sj1));
IkReal x901=((cj1)*(r01));
IkReal x902=((cj1)*(px));
IkReal x903=((sj3)*(x891));
IkReal x904=((cj3)*(x891));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x904))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x904))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x896)*(x900)))+(((IkReal(-1.00000000000000))*(x893)*(x901)))+(((IkReal(-1.00000000000000))*(x892)))+(((sj2)*(x899)))+(((IkReal(-1.00000000000000))*(x894)*(x896))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x896)*(x899)))+(((IkReal(-1.00000000000000))*(x893)*(x900)))+(((cj2)*(x901)))+(((IkReal(-1.00000000000000))*(x893)*(x894)))+(x903));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x896)*(x902)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x895)*(x896)))+(((sj2)*(x897)))+(((IkReal(-1.00000000000000))*(x893)*(x898)))+(((IkReal(2.16000000000000))*(x892))));
evalcond[5]=((IkReal(-0.145000000000000))+(((cj2)*(x898)))+(((IkReal(-0.158000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x896)*(x897)))+(((IkReal(-1.00000000000000))*(x893)*(x902)))+(((IkReal(-2.16000000000000))*(x903)))+(((IkReal(-1.00000000000000))*(x893)*(x895)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x905=((IkReal(1.00000000000000))*(cj1));
if( IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x905)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x905)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(sj1)))+(((r00)*(sj1)*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x905)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x905)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(sj1)))+(((r00)*(sj1)*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((((IkReal(-1.00000000000000))*(cj2)*(r00)*(x905)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x905)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(sj1)))+(((r00)*(sj1)*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x906=IKsin(j5);
IkReal x907=IKcos(j5);
IkReal x908=((IkReal(1.00000000000000))*(sj2));
IkReal x909=((cj1)*(r00));
IkReal x910=((py)*(sj1));
IkReal x911=((IkReal(1.00000000000000))*(cj2));
IkReal x912=((px)*(sj1));
IkReal x913=((cj1)*(py));
IkReal x914=((r00)*(sj1));
IkReal x915=((r01)*(sj1));
IkReal x916=((cj1)*(r01));
IkReal x917=((cj1)*(px));
IkReal x918=((sj3)*(x906));
IkReal x919=((cj3)*(x906));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x919))));
evalcond[1]=((((IkReal(-0.158000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x919))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x908)*(x916)))+(((sj2)*(x914)))+(((IkReal(-1.00000000000000))*(x907)))+(((IkReal(-1.00000000000000))*(x911)*(x915)))+(((IkReal(-1.00000000000000))*(x909)*(x911))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x908)*(x909)))+(((IkReal(-1.00000000000000))*(x908)*(x915)))+(((cj2)*(x916)))+(x918)+(((IkReal(-1.00000000000000))*(x911)*(x914))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-1.00000000000000))*(x908)*(x913)))+(((IkReal(1.20000000000000))*(sj2)))+(((sj2)*(x912)))+(((IkReal(-1.00000000000000))*(x910)*(x911)))+(((IkReal(-1.00000000000000))*(x911)*(x917)))+(((IkReal(2.16000000000000))*(x907))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-0.158000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x908)*(x910)))+(((IkReal(-1.00000000000000))*(x908)*(x917)))+(((cj2)*(x913)))+(((IkReal(-2.16000000000000))*(x918)))+(((IkReal(-1.00000000000000))*(x911)*(x912)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j4)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj3;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x920=((IkReal(0.462962962962963))*(sj1));
IkReal x921=((IkReal(0.462962962962963))*(cj1));
if( IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.715277777777778))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x921)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x921)))+(((px)*(sj2)*(x920)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x920)))+(((IkReal(0.555555555555556))*(sj2))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((IkReal(0.715277777777778))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x921)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x921)))+(((px)*(sj2)*(x920)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x920)))+(((IkReal(0.555555555555556))*(sj2)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((IkReal(0.715277777777778))+(((IkReal(-1.00000000000000))*(cj2)*(px)*(x921)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x921)))+(((px)*(sj2)*(x920)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x920)))+(((IkReal(0.555555555555556))*(sj2)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x922=IKsin(j5);
IkReal x923=IKcos(j5);
IkReal x924=((IkReal(1.00000000000000))*(sj2));
IkReal x925=((cj1)*(r00));
IkReal x926=((py)*(sj1));
IkReal x927=((IkReal(1.00000000000000))*(cj2));
IkReal x928=((px)*(sj1));
IkReal x929=((cj1)*(py));
IkReal x930=((r00)*(sj1));
IkReal x931=((r01)*(sj1));
IkReal x932=((cj1)*(r01));
IkReal x933=((cj1)*(px));
IkReal x934=((sj3)*(x922));
IkReal x935=((cj3)*(x922));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x935))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x935)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x925)*(x927)))+(((IkReal(-1.00000000000000))*(x924)*(x932)))+(((IkReal(-1.00000000000000))*(x927)*(x931)))+(((sj2)*(x930)))+(x923));
evalcond[3]=((((IkReal(-1.00000000000000))*(x924)*(x931)))+(((IkReal(-1.00000000000000))*(x924)*(x925)))+(((IkReal(-1.00000000000000))*(x927)*(x930)))+(((cj2)*(x932)))+(x934));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x924)*(x929)))+(((IkReal(-1.00000000000000))*(x927)*(x933)))+(((IkReal(-1.00000000000000))*(x926)*(x927)))+(((IkReal(-2.16000000000000))*(x923)))+(((sj2)*(x928))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(0.158000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x924)*(x933)))+(((IkReal(-1.00000000000000))*(x924)*(x926)))+(((IkReal(-1.00000000000000))*(x927)*(x928)))+(((cj2)*(x929)))+(((IkReal(-2.16000000000000))*(x934)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(-3.95000000000000))*(sj3))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(-3.95000000000000))*(sj3)))))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0185185185185185))*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)))+(((IkReal(-3.95000000000000))*(sj3)))))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x936=IKsin(j5);
IkReal x937=IKcos(j5);
IkReal x938=((IkReal(1.00000000000000))*(sj2));
IkReal x939=((cj1)*(r00));
IkReal x940=((py)*(sj1));
IkReal x941=((IkReal(1.00000000000000))*(cj2));
IkReal x942=((px)*(sj1));
IkReal x943=((cj1)*(py));
IkReal x944=((r00)*(sj1));
IkReal x945=((r01)*(sj1));
IkReal x946=((cj1)*(r01));
IkReal x947=((cj1)*(px));
IkReal x948=((sj3)*(x936));
IkReal x949=((cj3)*(x936));
evalcond[0]=((((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x949))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x949)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((sj2)*(x944)))+(((IkReal(-1.00000000000000))*(x939)*(x941)))+(((IkReal(-1.00000000000000))*(x938)*(x946)))+(((IkReal(-1.00000000000000))*(x941)*(x945)))+(x937));
evalcond[3]=((((IkReal(-1.00000000000000))*(x938)*(x945)))+(((cj2)*(x946)))+(((IkReal(-1.00000000000000))*(x941)*(x944)))+(((IkReal(-1.00000000000000))*(x938)*(x939)))+(x948));
evalcond[4]=((IkReal(1.54500000000000))+(((sj2)*(x942)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x938)*(x943)))+(((IkReal(-1.00000000000000))*(x941)*(x947)))+(((IkReal(-2.16000000000000))*(x937)))+(((IkReal(-1.00000000000000))*(x940)*(x941))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(0.158000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x938)*(x940)))+(((IkReal(-1.00000000000000))*(x938)*(x947)))+(((cj2)*(x943)))+(((IkReal(-1.00000000000000))*(x941)*(x942)))+(((IkReal(-2.16000000000000))*(x948)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
if( IKabs(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30)))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))))+IKsqr(((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(-1.00000000000000))*(r02)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))), ((((cj1)*(r01)*(sj2)))+(((cj2)*(r01)*(sj1)))+(((IkReal(-1.00000000000000))*(r00)*(sj1)*(sj2)))+(((cj1)*(cj2)*(r00)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x950=IKsin(j5);
IkReal x951=IKcos(j5);
IkReal x952=((IkReal(1.00000000000000))*(sj2));
IkReal x953=((cj1)*(r00));
IkReal x954=((py)*(sj1));
IkReal x955=((IkReal(1.00000000000000))*(cj2));
IkReal x956=((px)*(sj1));
IkReal x957=((cj1)*(py));
IkReal x958=((r00)*(sj1));
IkReal x959=((r01)*(sj1));
IkReal x960=((cj1)*(r01));
IkReal x961=((cj1)*(px));
IkReal x962=((sj3)*(x950));
IkReal x963=((cj3)*(x950));
evalcond[0]=((((IkReal(-1.00000000000000))*(x963)))+(((IkReal(-1.00000000000000))*(r02))));
evalcond[1]=((((IkReal(0.158000000000000))*(sj3)))+(((IkReal(2.16000000000000))*(x963)))+(((IkReal(-1.00000000000000))*(pz))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x953)*(x955)))+(((sj2)*(x958)))+(((IkReal(-1.00000000000000))*(x952)*(x960)))+(((IkReal(-1.00000000000000))*(x955)*(x959)))+(x951));
evalcond[3]=((((cj2)*(x960)))+(((IkReal(-1.00000000000000))*(x955)*(x958)))+(((IkReal(-1.00000000000000))*(x952)*(x959)))+(((IkReal(-1.00000000000000))*(x952)*(x953)))+(x962));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(-2.16000000000000))*(x951)))+(((IkReal(1.20000000000000))*(sj2)))+(((sj2)*(x956)))+(((IkReal(-1.00000000000000))*(x955)*(x961)))+(((IkReal(-1.00000000000000))*(x954)*(x955)))+(((IkReal(-1.00000000000000))*(x952)*(x957))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-2.16000000000000))*(x962)))+(((IkReal(0.158000000000000))*(cj3)))+(((cj2)*(x957)))+(((IkReal(-1.00000000000000))*(x952)*(x961)))+(((IkReal(-1.00000000000000))*(x955)*(x956)))+(((IkReal(-1.00000000000000))*(x952)*(x954)))+(((IkReal(-1.20000000000000))*(cj2))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x964=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x965=((cj1)*(r00));
IkReal x966=((IkReal(1.00000000000000))*(r01));
IkReal x967=((cj1)*(sj2));
IkReal x968=((r00)*(sj1)*(sj2));
IkReal x969=((IkReal(54.0000000000000))*(cj4)*(sj3));
IkReal x970=((cj2)*(x969));
if( IKabs(((IkReal(0.0185185185185185))*(x964)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)*(sj4)))+(((r01)*(x967)*(x969)))+(((r01)*(sj1)*(x970)))+(((x965)*(x970)))+(((IkReal(3.95000000000000))*(sj3)*(((IKabs(x964) != 0)?((IkReal)1/((x964)*(x964))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x968)*(x969))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x964)*(((((IkReal(-1.00000000000000))*(cj2)*(x965)))+(((IkReal(-1.00000000000000))*(x966)*(x967)))+(x968)+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x966))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0185185185185185))*(x964)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)*(sj4)))+(((r01)*(x967)*(x969)))+(((r01)*(sj1)*(x970)))+(((x965)*(x970)))+(((IkReal(3.95000000000000))*(sj3)*(((IKabs(x964) != 0)?((IkReal)1/((x964)*(x964))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x968)*(x969)))))))+IKsqr(((x964)*(((((IkReal(-1.00000000000000))*(cj2)*(x965)))+(((IkReal(-1.00000000000000))*(x966)*(x967)))+(x968)+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x966)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((IkReal(0.0185185185185185))*(x964)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((IkReal(25.0000000000000))*(pz)*(sj4)))+(((r01)*(x967)*(x969)))+(((r01)*(sj1)*(x970)))+(((x965)*(x970)))+(((IkReal(3.95000000000000))*(sj3)*(((IKabs(x964) != 0)?((IkReal)1/((x964)*(x964))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x968)*(x969)))))), ((x964)*(((((IkReal(-1.00000000000000))*(cj2)*(x965)))+(((IkReal(-1.00000000000000))*(x966)*(x967)))+(x968)+(((IkReal(-1.00000000000000))*(cj2)*(sj1)*(x966)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x971=IKcos(j5);
IkReal x972=IKsin(j5);
IkReal x973=((IkReal(0.158000000000000))*(sj4));
IkReal x974=((IkReal(1.00000000000000))*(sj1));
IkReal x975=((cj2)*(px));
IkReal x976=((IkReal(1.00000000000000))*(cj1));
IkReal x977=((cj2)*(r00));
IkReal x978=((cj3)*(cj4));
IkReal x979=((cj1)*(cj2));
IkReal x980=((cj4)*(sj3));
IkReal x981=((sj1)*(sj2));
IkReal x982=((sj2)*(x976));
IkReal x983=((sj3)*(x972));
IkReal x984=((IkReal(2.16000000000000))*(x971));
IkReal x985=((IkReal(1.00000000000000))*(x971));
IkReal x986=((cj3)*(x972));
evalcond[0]=((((IkReal(-1.00000000000000))*(x986)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x980)*(x985))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(((IkReal(2.16000000000000))*(x986)))+(((IkReal(-1.00000000000000))*(sj3)*(x973)))+(((x980)*(x984))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj4)*(x985)))+(((IkReal(-1.00000000000000))*(r01)*(x982)))+(((r00)*(x981)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x974)))+(((IkReal(-1.00000000000000))*(x976)*(x977))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r00)*(x982)))+(((r01)*(x979)))+(((IkReal(-1.00000000000000))*(x974)*(x977)))+(((IkReal(-1.00000000000000))*(x978)*(x985)))+(x983)+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x974))));
evalcond[4]=((IkReal(1.54500000000000))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x982)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x974)))+(((IkReal(0.158000000000000))*(cj4)))+(((sj4)*(x984)))+(((px)*(x981)))+(((IkReal(-1.00000000000000))*(x975)*(x976))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(px)*(x982)))+(((IkReal(-2.16000000000000))*(x983)))+(((IkReal(-1.00000000000000))*(cj3)*(x973)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x974)))+(((IkReal(-1.00000000000000))*(x974)*(x975)))+(((py)*(x979)))+(((IkReal(-1.20000000000000))*(cj2)))+(((x978)*(x984))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x987=((IKabs(sj4) != 0)?((IkReal)1/(sj4)):(IkReal)1.0e30);
IkReal x988=((cj4)*(sj3));
IkReal x989=((r00)*(sj1)*(sj2));
IkReal x990=((cj2)*(r01)*(sj1));
IkReal x991=((cj1)*(cj2)*(r00));
IkReal x992=((cj1)*(r01)*(sj2));
if( IKabs(((x987)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((x988)*(x991)))+(((x988)*(x992)))+(((x988)*(x990)))+(((IkReal(-1.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x988)*(x989))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x987)*(((((IkReal(-1.00000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(x991)))+(((IkReal(-1.00000000000000))*(x990)))+(x989))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x987)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((x988)*(x991)))+(((x988)*(x992)))+(((x988)*(x990)))+(((IkReal(-1.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x988)*(x989)))))))+IKsqr(((x987)*(((((IkReal(-1.00000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(x991)))+(((IkReal(-1.00000000000000))*(x990)))+(x989)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((x987)*(((IKabs(cj3) != 0)?((IkReal)1/(cj3)):(IkReal)1.0e30))*(((((x988)*(x991)))+(((x988)*(x992)))+(((x988)*(x990)))+(((IkReal(-1.00000000000000))*(r02)*(sj4)))+(((IkReal(-1.00000000000000))*(x988)*(x989)))))), ((x987)*(((((IkReal(-1.00000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(x991)))+(((IkReal(-1.00000000000000))*(x990)))+(x989)))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x993=IKcos(j5);
IkReal x994=IKsin(j5);
IkReal x995=((IkReal(0.158000000000000))*(sj4));
IkReal x996=((IkReal(1.00000000000000))*(sj1));
IkReal x997=((cj2)*(px));
IkReal x998=((IkReal(1.00000000000000))*(cj1));
IkReal x999=((cj2)*(r00));
IkReal x1000=((cj3)*(cj4));
IkReal x1001=((cj1)*(cj2));
IkReal x1002=((cj4)*(sj3));
IkReal x1003=((sj1)*(sj2));
IkReal x1004=((sj2)*(x998));
IkReal x1005=((sj3)*(x994));
IkReal x1006=((IkReal(2.16000000000000))*(x993));
IkReal x1007=((IkReal(1.00000000000000))*(x993));
IkReal x1008=((cj3)*(x994));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1008)))+(((IkReal(-1.00000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(x1002)*(x1007))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj3)*(x995)))+(((IkReal(2.16000000000000))*(x1008)))+(((IkReal(-1.00000000000000))*(pz)))+(((x1002)*(x1006))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj2)*(r01)*(x996)))+(((IkReal(-1.00000000000000))*(x998)*(x999)))+(((IkReal(-1.00000000000000))*(r01)*(x1004)))+(((IkReal(-1.00000000000000))*(sj4)*(x1007)))+(((r00)*(x1003))));
evalcond[3]=((((r01)*(x1001)))+(x1005)+(((IkReal(-1.00000000000000))*(x996)*(x999)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x996)))+(((IkReal(-1.00000000000000))*(r00)*(x1004)))+(((IkReal(-1.00000000000000))*(x1000)*(x1007))));
evalcond[4]=((IkReal(1.54500000000000))+(((sj4)*(x1006)))+(((px)*(x1003)))+(((IkReal(-1.00000000000000))*(py)*(x1004)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(0.158000000000000))*(cj4)))+(((IkReal(-1.00000000000000))*(x997)*(x998)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x996))));
evalcond[5]=((IkReal(-0.145000000000000))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x996)))+(((x1000)*(x1006)))+(((IkReal(-1.00000000000000))*(x996)*(x997)))+(((IkReal(-2.16000000000000))*(x1005)))+(((IkReal(-1.00000000000000))*(cj3)*(x995)))+(((py)*(x1001)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(px)*(x1004))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j5array[1], cj5array[1], sj5array[1];
bool j5valid[1]={false};
_nj5 = 1;
IkReal x1009=(sj3)*(sj3);
IkReal x1010=(cj3)*(cj3);
IkReal x1011=((r01)*(sj1));
IkReal x1012=((cj1)*(r00));
IkReal x1013=((IkReal(1.00000000000000))*(sj3));
IkReal x1014=((IkReal(1.00000000000000))*(cj3));
IkReal x1015=((sj2)*(x1013));
IkReal x1016=((cj2)*(r00)*(sj1));
IkReal x1017=((cj1)*(cj2)*(r01));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1010))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1010)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1012)*(x1015)))+(((IkReal(-1.00000000000000))*(x1011)*(x1015)))+(((cj3)*(r02)))+(((sj3)*(x1017)))+(((IkReal(-1.00000000000000))*(x1013)*(x1016))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj4)*(x1009)))+(((cj4)*(x1010))))) != 0)?((IkReal)1/(((((cj4)*(x1009)))+(((cj4)*(x1010)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1014)*(x1016)))+(((cj3)*(x1017)))+(((IkReal(-1.00000000000000))*(r02)*(x1013)))+(((IkReal(-1.00000000000000))*(sj2)*(x1011)*(x1014)))+(((IkReal(-1.00000000000000))*(sj2)*(x1012)*(x1014))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1010))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1010)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1012)*(x1015)))+(((IkReal(-1.00000000000000))*(x1011)*(x1015)))+(((cj3)*(r02)))+(((sj3)*(x1017)))+(((IkReal(-1.00000000000000))*(x1013)*(x1016)))))))+IKsqr(((((IKabs(((((cj4)*(x1009)))+(((cj4)*(x1010))))) != 0)?((IkReal)1/(((((cj4)*(x1009)))+(((cj4)*(x1010)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1014)*(x1016)))+(((cj3)*(x1017)))+(((IkReal(-1.00000000000000))*(r02)*(x1013)))+(((IkReal(-1.00000000000000))*(sj2)*(x1011)*(x1014)))+(((IkReal(-1.00000000000000))*(sj2)*(x1012)*(x1014)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j5array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1010))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1010)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1012)*(x1015)))+(((IkReal(-1.00000000000000))*(x1011)*(x1015)))+(((cj3)*(r02)))+(((sj3)*(x1017)))+(((IkReal(-1.00000000000000))*(x1013)*(x1016)))))), ((((IKabs(((((cj4)*(x1009)))+(((cj4)*(x1010))))) != 0)?((IkReal)1/(((((cj4)*(x1009)))+(((cj4)*(x1010)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x1014)*(x1016)))+(((cj3)*(x1017)))+(((IkReal(-1.00000000000000))*(r02)*(x1013)))+(((IkReal(-1.00000000000000))*(sj2)*(x1011)*(x1014)))+(((IkReal(-1.00000000000000))*(sj2)*(x1012)*(x1014)))))));
sj5array[0]=IKsin(j5array[0]);
cj5array[0]=IKcos(j5array[0]);
if( j5array[0] > IKPI )
{
    j5array[0]-=IK2PI;
}
else if( j5array[0] < -IKPI )
{    j5array[0]+=IK2PI;
}
j5valid[0] = true;
for(int ij5 = 0; ij5 < 1; ++ij5)
{
if( !j5valid[ij5] )
{
    continue;
}
_ij5[0] = ij5; _ij5[1] = -1;
for(int iij5 = ij5+1; iij5 < 1; ++iij5)
{
if( j5valid[iij5] && IKabs(cj5array[ij5]-cj5array[iij5]) < IKFAST_SOLUTION_THRESH && IKabs(sj5array[ij5]-sj5array[iij5]) < IKFAST_SOLUTION_THRESH )
{
    j5valid[iij5]=false; _ij5[1] = iij5; break; 
}
}
j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
{
IkReal evalcond[6];
IkReal x1018=IKcos(j5);
IkReal x1019=IKsin(j5);
IkReal x1020=((IkReal(0.158000000000000))*(sj4));
IkReal x1021=((IkReal(1.00000000000000))*(sj1));
IkReal x1022=((cj2)*(px));
IkReal x1023=((IkReal(1.00000000000000))*(cj1));
IkReal x1024=((cj2)*(r00));
IkReal x1025=((cj3)*(cj4));
IkReal x1026=((cj1)*(cj2));
IkReal x1027=((cj4)*(sj3));
IkReal x1028=((sj1)*(sj2));
IkReal x1029=((sj2)*(x1023));
IkReal x1030=((sj3)*(x1019));
IkReal x1031=((IkReal(2.16000000000000))*(x1018));
IkReal x1032=((IkReal(1.00000000000000))*(x1018));
IkReal x1033=((cj3)*(x1019));
evalcond[0]=((((IkReal(-1.00000000000000))*(x1027)*(x1032)))+(((IkReal(-1.00000000000000))*(x1033)))+(((IkReal(-1.00000000000000))*(r02))));
evalcond[1]=((((IkReal(2.16000000000000))*(x1033)))+(((IkReal(-1.00000000000000))*(pz)))+(((x1027)*(x1031)))+(((IkReal(-1.00000000000000))*(sj3)*(x1020))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x1023)*(x1024)))+(((IkReal(-1.00000000000000))*(cj2)*(r01)*(x1021)))+(((IkReal(-1.00000000000000))*(r01)*(x1029)))+(((IkReal(-1.00000000000000))*(sj4)*(x1032)))+(((r00)*(x1028))));
evalcond[3]=((((r01)*(x1026)))+(((IkReal(-1.00000000000000))*(r01)*(sj2)*(x1021)))+(((IkReal(-1.00000000000000))*(x1025)*(x1032)))+(x1030)+(((IkReal(-1.00000000000000))*(x1021)*(x1024)))+(((IkReal(-1.00000000000000))*(r00)*(x1029))));
evalcond[4]=((IkReal(1.54500000000000))+(((sj4)*(x1031)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x1021)))+(((IkReal(1.20000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(py)*(x1029)))+(((IkReal(-1.00000000000000))*(x1022)*(x1023)))+(((IkReal(0.158000000000000))*(cj4)))+(((px)*(x1028))));
evalcond[5]=((IkReal(-0.145000000000000))+(((py)*(x1026)))+(((IkReal(-2.16000000000000))*(x1030)))+(((IkReal(-1.00000000000000))*(x1021)*(x1022)))+(((IkReal(-1.00000000000000))*(cj3)*(x1020)))+(((x1025)*(x1031)))+(((IkReal(-1.00000000000000))*(px)*(x1029)))+(((IkReal(-1.20000000000000))*(cj2)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x1021))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(6);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j4;
vinfos[4].indices[0] = _ij4[0];
vinfos[4].indices[1] = _ij4[1];
vinfos[4].maxsolutions = _nj4;
vinfos[5].jointtype = 1;
vinfos[5].foffset = j5;
vinfos[5].indices[0] = _ij5[0];
vinfos[5].indices[1] = _ij5[1];
vinfos[5].maxsolutions = _nj5;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "1999c6d2c0204e8fd3c6915845778c23"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
