/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 07:27:13.798891
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26,x27,x28,x29,x30;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKcos(j[3]);
x3=IKcos(j[2]);
x4=IKsin(j[0]);
x5=IKsin(j[1]);
x6=IKsin(j[2]);
x7=IKsin(j[3]);
x8=IKsin(j[4]);
x9=IKcos(j[4]);
x10=((IkReal(0.0440000000000000))*(x3));
x11=((IkReal(0.845725967038483))*(x7));
x12=((IkReal(0.135000000000000))*(x0));
x13=((IkReal(0.533617455371189))*(x8));
x14=((IkReal(0.533617455371189))*(x9));
x15=((IkReal(0.0440000000000000))*(x5));
x16=((IkReal(0.0440000000000000))*(x2));
x17=((IkReal(0.0440000000000000))*(x1));
x18=((IkReal(1.00000000000000))*(x4));
x19=((IkReal(0.135000000000000))*(x4));
x20=((x1)*(x2));
x21=((x4)*(x6));
x22=((x5)*(x7));
x23=((x0)*(x6));
x24=((x1)*(x3));
x25=((x3)*(x5));
x26=((x2)*(x5));
x27=((x1)*(x4));
x28=((x0)*(x7));
x29=((((IkReal(-1.00000000000000))*(x18)*(x25)))+(x23));
x30=((((IkReal(-1.00000000000000))*(x18)*(x6)))+(((IkReal(-1.00000000000000))*(x0)*(x25))));
eetrans[0]=((((IkReal(0.100000000000000))*(x0)))+(((IkReal(0.400000000000000))*(x0)*(x1)))+(((x9)*(((((IkReal(-1.00000000000000))*(x16)*(x30)))+(((x17)*(x28)))))))+(((x7)*(((((IkReal(-1.00000000000000))*(x19)*(x6)))+(((IkReal(-1.00000000000000))*(x12)*(x25)))))))+(((x12)*(x20)))+(((x8)*(((((IkReal(-1.00000000000000))*(x15)*(x23)))+(((x10)*(x4))))))));
eetrans[1]=((IkReal(-0.188000000000000))+(((x19)*(x20)))+(((IkReal(0.100000000000000))*(x4)))+(((x7)*(((((IkReal(-1.00000000000000))*(x19)*(x25)))+(((x12)*(x6)))))))+(((IkReal(0.400000000000000))*(x27)))+(((x9)*(((((x17)*(x4)*(x7)))+(((IkReal(-1.00000000000000))*(x16)*(x29)))))))+(((x8)*(((((IkReal(-1.00000000000000))*(x15)*(x21)))+(((IkReal(-1.00000000000000))*(x0)*(x10))))))));
eetrans[2]=((((IkReal(-0.135000000000000))*(x24)*(x7)))+(((IkReal(-1.00000000000000))*(x17)*(x6)*(x8)))+(((IkReal(-0.135000000000000))*(x26)))+(((IkReal(-0.400000000000000))*(x5)))+(((x9)*(((((x10)*(x20)))+(((IkReal(-1.00000000000000))*(x15)*(x7))))))));
eerot[0]=((((IkReal(0.845725967038483))*(x0)*(x20)))+(((x11)*(x30)))+(((IkReal(-1.00000000000000))*(x13)*(((((IkReal(-1.00000000000000))*(x18)*(x3)))+(((x23)*(x5)))))))+(((IkReal(-1.00000000000000))*(x14)*(((((IkReal(-1.00000000000000))*(x1)*(x28)))+(((x2)*(x30))))))));
eerot[1]=((((x11)*(x29)))+(((IkReal(-1.00000000000000))*(x13)*(((((x0)*(x3)))+(((x21)*(x5)))))))+(((IkReal(0.845725967038483))*(x20)*(x4)))+(((IkReal(-1.00000000000000))*(x14)*(((((x2)*(x29)))+(((IkReal(-1.00000000000000))*(x27)*(x7))))))));
eerot[2]=((((IkReal(-0.845725967038483))*(x26)))+(((IkReal(-1.00000000000000))*(x1)*(x13)*(x6)))+(((IkReal(-1.00000000000000))*(x11)*(x24)))+(((IkReal(-1.00000000000000))*(x14)*(((x22)+(((IkReal(-1.00000000000000))*(x20)*(x3))))))));
}

IKFAST_API int GetNumFreeParameters() { return 0; }
IKFAST_API int* GetFreeParameters() { return NULL; }
IKFAST_API int GetNumJoints() { return 5; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x56000007; }

class IKSolver {
public:
IkReal j27,cj27,sj27,htj27,j28,cj28,sj28,htj28,j29,cj29,sj29,htj29,j30,cj30,sj30,htj30,j31,cj31,sj31,htj31,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij27[2], _nj27,_ij28[2], _nj28,_ij29[2], _nj29,_ij30[2], _nj30,_ij31[2], _nj31;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j27=numeric_limits<IkReal>::quiet_NaN(); _ij27[0] = -1; _ij27[1] = -1; _nj27 = -1; j28=numeric_limits<IkReal>::quiet_NaN(); _ij28[0] = -1; _ij28[1] = -1; _nj28 = -1; j29=numeric_limits<IkReal>::quiet_NaN(); _ij29[0] = -1; _ij29[1] = -1; _nj29 = -1; j30=numeric_limits<IkReal>::quiet_NaN(); _ij30[0] = -1; _ij30[1] = -1; _nj30 = -1; j31=numeric_limits<IkReal>::quiet_NaN(); _ij31[0] = -1; _ij31[1] = -1; _nj31 = -1; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

r00 = eerot[0];
r01 = eerot[1];
r02 = eerot[2];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
new_r00=r00;
new_px=px;
new_r01=r01;
new_py=((IkReal(0.188000000000000))+(py));
new_r02=r02;
new_pz=pz;
r00 = new_r00; r01 = new_r01; r02 = new_r02; px = new_px; py = new_py; pz = new_pz;

pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
IkReal op[8+1], zeror[8];
int numroots;
IkReal gconst0;
gconst0=((IkReal(0.229839000000000))+(((IkReal(-1.00000000000000))*(px)))+(pp));
IkReal gconst1;
gconst1=((IkReal(1.60000000000000))*(pz));
IkReal gconst2;
gconst2=((IkReal(0.0698390000000000))+(((IkReal(0.600000000000000))*(px)))+(pp));
IkReal gconst3;
gconst3=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst4;
gconst4=((IkReal(0.800000000000000))*(r02));
IkReal gconst5;
gconst5=((IkReal(-0.137652173586528))+(((IkReal(0.300000000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst6;
gconst6=((IkReal(0.229839000000000))+(((IkReal(-1.00000000000000))*(px)))+(pp));
IkReal gconst7;
gconst7=((IkReal(1.60000000000000))*(pz));
IkReal gconst8;
gconst8=((IkReal(0.0698390000000000))+(((IkReal(0.600000000000000))*(px)))+(pp));
IkReal gconst9;
gconst9=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst10;
gconst10=((IkReal(0.800000000000000))*(r02));
IkReal gconst11;
gconst11=((IkReal(-0.137652173586528))+(((IkReal(0.300000000000000))*(r00)))+(((px)*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst12;
gconst12=((IkReal(-2.00000000000000))*(py));
IkReal gconst13;
gconst13=((IkReal(1.20000000000000))*(py));
IkReal gconst14;
gconst14=((IkReal(-1.00000000000000))*(r01));
IkReal gconst15;
gconst15=((IkReal(0.600000000000000))*(r01));
IkReal gconst16;
gconst16=((IkReal(-2.00000000000000))*(py));
IkReal gconst17;
gconst17=((IkReal(1.20000000000000))*(py));
IkReal gconst18;
gconst18=((IkReal(-1.00000000000000))*(r01));
IkReal gconst19;
gconst19=((IkReal(0.600000000000000))*(r01));
IkReal gconst20;
gconst20=((IkReal(0.229839000000000))+(px)+(pp));
IkReal gconst21;
gconst21=((IkReal(1.60000000000000))*(pz));
IkReal gconst22;
gconst22=((IkReal(0.0698390000000000))+(((IkReal(-0.600000000000000))*(px)))+(pp));
IkReal gconst23;
gconst23=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst24;
gconst24=((IkReal(0.800000000000000))*(r02));
IkReal gconst25;
gconst25=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.300000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst26;
gconst26=((IkReal(0.229839000000000))+(px)+(pp));
IkReal gconst27;
gconst27=((IkReal(1.60000000000000))*(pz));
IkReal gconst28;
gconst28=((IkReal(0.0698390000000000))+(((IkReal(-0.600000000000000))*(px)))+(pp));
IkReal gconst29;
gconst29=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(0.500000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal gconst30;
gconst30=((IkReal(0.800000000000000))*(r02));
IkReal gconst31;
gconst31=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.300000000000000))*(r00)))+(((pz)*(r02)))+(((py)*(r01))));
IkReal x31=((gconst27)*(gconst29));
IkReal x32=((gconst22)*(gconst4));
IkReal x33=((IkReal(1.00000000000000))*(gconst25));
IkReal x34=((gconst19)*(gconst26));
IkReal x35=((IkReal(1.00000000000000))*(gconst3));
IkReal x36=((gconst8)*(gconst9));
IkReal x37=((gconst17)*(gconst18));
IkReal x38=((gconst20)*(gconst25));
IkReal x39=((gconst0)*(gconst5));
IkReal x40=((IkReal(1.00000000000000))*(gconst11));
IkReal x41=((gconst22)*(gconst23));
IkReal x42=((gconst16)*(gconst19));
IkReal x43=((gconst27)*(gconst9));
IkReal x44=((gconst11)*(gconst6));
IkReal x45=((gconst13)*(gconst3));
IkReal x46=((gconst2)*(gconst8));
IkReal x47=((gconst14)*(gconst2));
IkReal x48=((gconst11)*(gconst16));
IkReal x49=((gconst7)*(gconst9));
IkReal x50=((gconst13)*(gconst4));
IkReal x51=((gconst15)*(gconst20));
IkReal x52=((gconst2)*(gconst23));
IkReal x53=((gconst28)*(gconst29));
IkReal x54=((gconst17)*(gconst9));
IkReal x55=((IkReal(1.00000000000000))*(gconst21));
IkReal x56=((gconst26)*(gconst31));
IkReal x57=((gconst22)*(gconst3));
IkReal x58=((gconst17)*(gconst29));
IkReal x59=((gconst12)*(gconst25));
IkReal x60=((IkReal(1.00000000000000))*(gconst5));
IkReal x61=((gconst19)*(gconst6));
IkReal x62=((gconst18)*(gconst28));
IkReal x63=((gconst29)*(gconst8));
IkReal x64=((gconst13)*(gconst23));
IkReal x65=((gconst10)*(gconst6));
IkReal x66=((gconst2)*(gconst24));
IkReal x67=((IkReal(1.00000000000000))*(gconst15));
IkReal x68=((gconst1)*(gconst25));
IkReal x69=((gconst22)*(gconst24));
IkReal x70=((IkReal(1.00000000000000))*(gconst14));
IkReal x71=((gconst0)*(gconst25));
IkReal x72=((gconst2)*(gconst3));
IkReal x73=((gconst26)*(gconst30));
IkReal x74=((gconst31)*(gconst6));
IkReal x75=((IkReal(1.00000000000000))*(gconst10));
IkReal x76=((gconst1)*(gconst7));
IkReal x77=((gconst15)*(gconst6));
IkReal x78=((gconst2)*(gconst4));
IkReal x79=((gconst1)*(gconst15));
IkReal x80=((gconst18)*(gconst7));
IkReal x81=((gconst12)*(gconst16));
IkReal x82=((gconst5)*(gconst6));
IkReal x83=((gconst18)*(gconst27));
IkReal x84=((gconst12)*(gconst5));
IkReal x85=((gconst28)*(gconst9));
IkReal x86=((gconst21)*(gconst5));
IkReal x87=((gconst1)*(gconst5));
IkReal x88=((gconst10)*(gconst26));
IkReal x89=((gconst16)*(gconst31));
IkReal x90=((gconst14)*(gconst22));
IkReal x91=((gconst13)*(gconst14));
IkReal x92=((gconst11)*(gconst26));
IkReal x93=((gconst12)*(gconst15));
IkReal x94=((gconst20)*(gconst5));
IkReal x95=((gconst0)*(gconst15));
IkReal x96=((gconst13)*(gconst24));
IkReal x97=((gconst18)*(gconst8));
IkReal x98=((gconst21)*(gconst25));
IkReal x99=((gconst29)*(gconst7));
IkReal x100=((gconst10)*(gconst16));
IkReal x101=((gconst20)*(x89));
IkReal x102=((gconst0)*(x97));
IkReal x103=((gconst15)*(gconst21)*(gconst30));
IkReal x104=((gconst18)*(x70));
IkReal x105=((IkReal(1.00000000000000))*(gconst30)*(gconst6));
IkReal x106=((gconst30)*(x98));
IkReal x107=((IkReal(1.00000000000000))*(gconst16)*(gconst30));
op[0]=((((x38)*(x53)))+(((IkReal(-1.00000000000000))*(x69)*(x73)))+(((x73)*(x98)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x56)))+(((IkReal(-1.00000000000000))*(x41)*(x53)))+(((x31)*(x69)))+(((IkReal(-1.00000000000000))*(gconst21)*(x31)*(x33)))+(((x41)*(x56))));
op[1]=((((x38)*(x58)))+(((x38)*(x62)))+(((x34)*(x41)))+(((IkReal(-1.00000000000000))*(x51)*(x56)))+(((x41)*(x89)))+(((gconst16)*(x106)))+(((IkReal(-1.00000000000000))*(x101)*(x33)))+(((IkReal(-1.00000000000000))*(gconst21)*(x33)*(x83)))+(((IkReal(-1.00000000000000))*(gconst15)*(x31)*(x55)))+(((x56)*(x90)))+(((IkReal(-1.00000000000000))*(gconst12)*(x33)*(x56)))+(((gconst15)*(gconst21)*(x73)))+(((x69)*(x83)))+(((IkReal(-1.00000000000000))*(x41)*(x58)))+(((IkReal(-1.00000000000000))*(x73)*(x96)))+(((x56)*(x64)))+(((x53)*(x59)))+(((IkReal(-1.00000000000000))*(x41)*(x62)))+(((IkReal(-1.00000000000000))*(x53)*(x64)))+(((IkReal(-1.00000000000000))*(gconst22)*(x53)*(x70)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x34)))+(((x51)*(x53)))+(((IkReal(-1.00000000000000))*(x107)*(x69)))+(((x31)*(x96))));
op[2]=((((IkReal(-1.00000000000000))*(x62)*(x64)))+(((IkReal(-1.00000000000000))*(x32)*(x73)))+(((IkReal(-1.00000000000000))*(x107)*(x96)))+(((x38)*(x63)))+(((x89)*(x90)))+(((x59)*(x62)))+(((gconst16)*(x103)))+(((x41)*(x42)))+(((IkReal(-1.00000000000000))*(gconst1)*(x31)*(x33)))+(((x34)*(x90)))+(((IkReal(-1.00000000000000))*(x105)*(x69)))+(((x64)*(x89)))+(((x53)*(x93)))+(((x53)*(x94)))+(((x58)*(x59)))+(((x37)*(x38)))+(((gconst6)*(x106)))+(((x41)*(x92)))+(((IkReal(-1.00000000000000))*(gconst21)*(x33)*(x99)))+(((x38)*(x85)))+(((x41)*(x74)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x74)))+(((x34)*(x64)))+(((IkReal(-1.00000000000000))*(gconst21)*(x33)*(x43)))+(((IkReal(-1.00000000000000))*(x52)*(x53)))+(((IkReal(-1.00000000000000))*(gconst0)*(x33)*(x56)))+(((x68)*(x73)))+(((IkReal(-1.00000000000000))*(gconst22)*(x58)*(x70)))+(((IkReal(-1.00000000000000))*(gconst20)*(x56)*(x60)))+(((x73)*(x86)))+(((x56)*(x91)))+(((x69)*(x99)))+(((IkReal(-1.00000000000000))*(x51)*(x89)))+(((IkReal(-1.00000000000000))*(x58)*(x64)))+(((IkReal(-1.00000000000000))*(gconst15)*(x55)*(x83)))+(((IkReal(-1.00000000000000))*(gconst5)*(x31)*(x55)))+(((IkReal(-1.00000000000000))*(gconst26)*(x69)*(x75)))+(((x31)*(x66)))+(((x88)*(x98)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x92)))+(((IkReal(-1.00000000000000))*(gconst31)*(x33)*(x81)))+(((IkReal(-1.00000000000000))*(gconst22)*(x62)*(x70)))+(((IkReal(-1.00000000000000))*(x41)*(x63)))+(((IkReal(-1.00000000000000))*(x37)*(x41)))+(((IkReal(-1.00000000000000))*(x66)*(x73)))+(((x56)*(x57)))+(((IkReal(-1.00000000000000))*(gconst12)*(x33)*(x34)))+(((IkReal(-1.00000000000000))*(x41)*(x85)))+(((x43)*(x69)))+(((x52)*(x56)))+(((x51)*(x58)))+(((x53)*(x71)))+(((IkReal(-1.00000000000000))*(gconst20)*(x33)*(x42)))+(((x83)*(x96)))+(((x51)*(x62)))+(((x31)*(x32)))+(((IkReal(-1.00000000000000))*(x34)*(x51)))+(((IkReal(-1.00000000000000))*(gconst22)*(x35)*(x53)))+(((IkReal(-1.00000000000000))*(gconst12)*(x56)*(x67)))+(((IkReal(-1.00000000000000))*(gconst13)*(x53)*(x70))));
op[3]=((((IkReal(-1.00000000000000))*(gconst14)*(gconst2)*(gconst28)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst20)*(gconst26)*(gconst5)))+(((gconst19)*(gconst2)*(gconst23)*(gconst26)))+(((gconst15)*(gconst21)*(gconst30)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst20)*(gconst25)))+(((gconst16)*(gconst21)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst20)*(gconst25)*(gconst6)))+(((gconst0)*(gconst15)*(gconst28)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst26)*(gconst31)*(gconst5)))+(((gconst15)*(gconst20)*(gconst29)*(gconst8)))+(((gconst14)*(gconst2)*(gconst26)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst23)*(gconst8)))+(((gconst13)*(gconst24)*(gconst27)*(gconst9)))+(((gconst13)*(gconst24)*(gconst29)*(gconst7)))+(((gconst18)*(gconst22)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst25)*(gconst26)))+(((gconst17)*(gconst20)*(gconst25)*(gconst9)))+(((gconst12)*(gconst25)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst21)*(gconst27)*(gconst5)))+(((gconst18)*(gconst22)*(gconst24)*(gconst7)))+(((gconst18)*(gconst20)*(gconst25)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst2)*(gconst23)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst24)*(gconst30)*(gconst6)))+(((gconst11)*(gconst16)*(gconst22)*(gconst23)))+(((gconst13)*(gconst16)*(gconst19)*(gconst23)))+(((gconst12)*(gconst25)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst20)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst25)*(gconst31)*(gconst6)))+(((gconst17)*(gconst20)*(gconst29)*(gconst5)))+(((gconst15)*(gconst20)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst16)*(gconst19)*(gconst20)))+(((gconst13)*(gconst26)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst19)*(gconst25)*(gconst26)))+(((gconst18)*(gconst2)*(gconst24)*(gconst27)))+(((gconst10)*(gconst15)*(gconst21)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst27)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst22)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst28)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst19)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst22)*(gconst24)))+(((gconst11)*(gconst13)*(gconst23)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst23)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst18)*(gconst28)))+(((gconst1)*(gconst15)*(gconst26)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst19)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst22)*(gconst23)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst20)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst18)*(gconst25)*(gconst27)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst18)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst18)*(gconst22)))+(((gconst1)*(gconst16)*(gconst25)*(gconst30)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst21)*(gconst25)*(gconst7)))+(((gconst12)*(gconst15)*(gconst18)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst23)*(gconst28)*(gconst9)))+(((gconst13)*(gconst23)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst23)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst22)*(gconst29)*(gconst3)))+(((gconst13)*(gconst27)*(gconst29)*(gconst4)))+(((gconst16)*(gconst22)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst20)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst17)*(gconst29)))+(((gconst19)*(gconst22)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst16)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst26)*(gconst30)*(gconst4)))+(((gconst0)*(gconst17)*(gconst25)*(gconst29)))+(((gconst16)*(gconst2)*(gconst23)*(gconst31)))+(((gconst12)*(gconst28)*(gconst29)*(gconst5)))+(((gconst15)*(gconst17)*(gconst18)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst16)*(gconst25)*(gconst31)))+(((gconst13)*(gconst14)*(gconst16)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst24)*(gconst26)))+(((gconst13)*(gconst14)*(gconst19)*(gconst26)))+(((gconst18)*(gconst20)*(gconst28)*(gconst5)))+(((gconst11)*(gconst14)*(gconst22)*(gconst26)))+(((gconst12)*(gconst17)*(gconst18)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst28)*(gconst29)*(gconst3)))+(((gconst10)*(gconst16)*(gconst21)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst28)*(gconst9)))+(((gconst12)*(gconst15)*(gconst17)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst24)*(gconst30)))+(((gconst14)*(gconst22)*(gconst31)*(gconst6)))+(((gconst19)*(gconst22)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst29)*(gconst8)))+(((gconst0)*(gconst18)*(gconst25)*(gconst28)))+(((gconst14)*(gconst16)*(gconst19)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst26)*(gconst31))));
op[4]=((((gconst0)*(gconst25)*(gconst28)*(gconst9)))+(((gconst11)*(gconst13)*(gconst16)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst16)*(gconst24)))+(((gconst11)*(gconst22)*(gconst26)*(gconst3)))+(((gconst1)*(gconst10)*(gconst25)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst18)*(gconst27)))+(((gconst16)*(gconst19)*(gconst2)*(gconst23)))+(((gconst2)*(gconst27)*(gconst29)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst2)*(gconst28)))+(((gconst11)*(gconst22)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst28)*(gconst29)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst19)*(gconst20)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst16)*(gconst30)*(gconst4)))+(((gconst1)*(gconst25)*(gconst30)*(gconst6)))+(((gconst0)*(gconst28)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst18)*(gconst28)*(gconst3)))+(((gconst20)*(gconst28)*(gconst5)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst19)*(gconst26)*(gconst5)))+(((gconst21)*(gconst30)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst25)*(gconst26)))+(((gconst14)*(gconst19)*(gconst22)*(gconst6)))+(((gconst13)*(gconst19)*(gconst26)*(gconst3)))+(((gconst0)*(gconst15)*(gconst18)*(gconst28)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst20)*(gconst25)*(gconst6)))+(((gconst0)*(gconst25)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst26)*(gconst30)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst22)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst22)*(gconst9)))+(((gconst17)*(gconst18)*(gconst20)*(gconst5)))+(((gconst1)*(gconst26)*(gconst30)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst18)*(gconst21)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst2)*(gconst23)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst19)*(gconst20)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst19)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst24)*(gconst30)*(gconst6)))+(((gconst15)*(gconst18)*(gconst20)*(gconst8)))+(((gconst13)*(gconst18)*(gconst27)*(gconst4)))+(((gconst22)*(gconst24)*(gconst7)*(gconst9)))+(((gconst22)*(gconst27)*(gconst4)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst25)*(gconst31)*(gconst6)))+(((gconst1)*(gconst15)*(gconst16)*(gconst30)))+(((gconst2)*(gconst23)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst29)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst27)*(gconst29)*(gconst5)))+(((gconst14)*(gconst16)*(gconst2)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst20)*(gconst31)*(gconst5)*(gconst6)))+(((gconst22)*(gconst29)*(gconst4)*(gconst7)))+(((gconst20)*(gconst25)*(gconst8)*(gconst9)))+(((gconst15)*(gconst17)*(gconst20)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst25)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst19)*(gconst26)))+(((gconst2)*(gconst24)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst18)*(gconst22)*(gconst8)))+(((gconst14)*(gconst19)*(gconst2)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst16)*(gconst19)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst23)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst26)*(gconst31)*(gconst5)))+(((gconst13)*(gconst16)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst15)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst18)*(gconst22)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst25)*(gconst7)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst22)*(gconst26)*(gconst4)))+(((gconst0)*(gconst17)*(gconst18)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst18)*(gconst23)*(gconst8)))+(((gconst11)*(gconst14)*(gconst16)*(gconst22)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst28)*(gconst3)*(gconst9)))+(((gconst16)*(gconst19)*(gconst22)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst2)*(gconst29)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst16)*(gconst19)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst16)*(gconst20)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst16)*(gconst25)))+(((gconst20)*(gconst29)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst23)*(gconst9)))+(((gconst2)*(gconst26)*(gconst3)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst17)*(gconst18)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst29)*(gconst3)*(gconst8)))+(((gconst11)*(gconst2)*(gconst23)*(gconst26)))+(((gconst10)*(gconst21)*(gconst25)*(gconst6)))+(((gconst10)*(gconst15)*(gconst16)*(gconst21)))+(((gconst10)*(gconst21)*(gconst26)*(gconst5)))+(((gconst13)*(gconst19)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst23)*(gconst29)*(gconst8)))+(((gconst2)*(gconst24)*(gconst27)*(gconst9)))+(((gconst13)*(gconst14)*(gconst31)*(gconst6)))+(((gconst12)*(gconst18)*(gconst25)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst16)*(gconst31)))+(((IkReal(-1.00000000000000))*(gconst21)*(gconst27)*(gconst5)*(gconst9)))+(((gconst0)*(gconst15)*(gconst17)*(gconst29)))+(((gconst11)*(gconst13)*(gconst14)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst2)*(gconst24)*(gconst26)))+(((gconst12)*(gconst15)*(gconst17)*(gconst18)))+(((gconst12)*(gconst18)*(gconst28)*(gconst5)))+(((gconst12)*(gconst17)*(gconst29)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst2)*(gconst23)*(gconst28)*(gconst9)))+(((gconst12)*(gconst15)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst22)*(gconst30)*(gconst4)*(gconst6)))+(((gconst12)*(gconst15)*(gconst28)*(gconst9)))+(((gconst12)*(gconst17)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst20)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst31)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst29)*(gconst3)))+(((gconst22)*(gconst3)*(gconst31)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst25)*(gconst29)*(gconst7)))+(((gconst13)*(gconst18)*(gconst24)*(gconst7)))+(((gconst13)*(gconst14)*(gconst16)*(gconst19))));
op[5]=((((IkReal(-1.00000000000000))*(gconst13)*(gconst30)*(gconst4)*(gconst6)))+(((gconst12)*(gconst25)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst17)*(gconst18)*(gconst3)))+(((gconst13)*(gconst29)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst2)*(gconst23)*(gconst9)))+(((gconst1)*(gconst15)*(gconst30)*(gconst6)))+(((gconst11)*(gconst14)*(gconst22)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst2)*(gconst24)))+(((gconst11)*(gconst13)*(gconst23)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst17)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst18)*(gconst25)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst26)*(gconst4)))+(((gconst18)*(gconst2)*(gconst24)*(gconst7)))+(((gconst14)*(gconst2)*(gconst31)*(gconst6)))+(((gconst19)*(gconst22)*(gconst3)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst31)*(gconst5)*(gconst6)))+(((gconst12)*(gconst17)*(gconst18)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst28)*(gconst3)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst16)*(gconst19)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst19)*(gconst25)*(gconst6)))+(((gconst16)*(gconst2)*(gconst3)*(gconst31)))+(((gconst1)*(gconst16)*(gconst30)*(gconst5)))+(((gconst13)*(gconst14)*(gconst19)*(gconst6)))+(((gconst10)*(gconst15)*(gconst21)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst19)*(gconst26)*(gconst5)))+(((gconst11)*(gconst16)*(gconst2)*(gconst23)))+(((gconst11)*(gconst14)*(gconst2)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst14)*(gconst18)*(gconst8)))+(((gconst0)*(gconst18)*(gconst25)*(gconst8)))+(((gconst10)*(gconst16)*(gconst21)*(gconst5)))+(((gconst0)*(gconst17)*(gconst29)*(gconst5)))+(((gconst1)*(gconst10)*(gconst16)*(gconst25)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst22)*(gconst3)*(gconst9)))+(((gconst19)*(gconst2)*(gconst23)*(gconst6)))+(((gconst17)*(gconst20)*(gconst5)*(gconst9)))+(((gconst0)*(gconst18)*(gconst28)*(gconst5)))+(((gconst18)*(gconst2)*(gconst27)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst29)*(gconst3)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst15)*(gconst20)*(gconst6)))+(((gconst13)*(gconst24)*(gconst7)*(gconst9)))+(((gconst13)*(gconst16)*(gconst19)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst13)*(gconst23)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst23)*(gconst8)))+(((gconst12)*(gconst15)*(gconst18)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst16)*(gconst25)))+(((gconst12)*(gconst29)*(gconst5)*(gconst8)))+(((gconst18)*(gconst20)*(gconst5)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst21)*(gconst5)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst2)*(gconst29)*(gconst8)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst16)*(gconst20)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst15)*(gconst16)))+(((gconst13)*(gconst3)*(gconst31)*(gconst6)))+(((gconst11)*(gconst13)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst26)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst2)*(gconst28)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst18)*(gconst27)*(gconst5)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst27)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst2)*(gconst28)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst17)*(gconst18)*(gconst2)))+(((IkReal(-1.00000000000000))*(gconst11)*(gconst12)*(gconst25)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst16)*(gconst19)))+(((gconst12)*(gconst15)*(gconst17)*(gconst9)))+(((gconst15)*(gconst20)*(gconst8)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst18)*(gconst22)*(gconst3)*(gconst8)))+(((gconst12)*(gconst28)*(gconst5)*(gconst9)))+(((gconst11)*(gconst16)*(gconst22)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst13)*(gconst24)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst17)*(gconst2)*(gconst29)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst11)*(gconst15)*(gconst26)))+(((gconst13)*(gconst27)*(gconst4)*(gconst9)))+(((gconst19)*(gconst2)*(gconst26)*(gconst3)))+(((IkReal(-1.00000000000000))*(gconst19)*(gconst20)*(gconst5)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst1)*(gconst15)*(gconst29)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst10)*(gconst16)*(gconst22)*(gconst4)))+(((IkReal(-1.00000000000000))*(gconst14)*(gconst22)*(gconst8)*(gconst9)))+(((gconst11)*(gconst13)*(gconst14)*(gconst16)))+(((IkReal(-1.00000000000000))*(gconst15)*(gconst21)*(gconst7)*(gconst9)))+(((gconst18)*(gconst22)*(gconst4)*(gconst7)))+(((IkReal(-1.00000000000000))*(gconst16)*(gconst2)*(gconst30)*(gconst4)))+(((gconst0)*(gconst15)*(gconst28)*(gconst9)))+(((gconst14)*(gconst16)*(gconst19)*(gconst2)))+(((gconst0)*(gconst15)*(gconst17)*(gconst18)))+(((gconst0)*(gconst15)*(gconst29)*(gconst8)))+(((gconst1)*(gconst10)*(gconst15)*(gconst26)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst16)*(gconst31)*(gconst5)))+(((gconst0)*(gconst17)*(gconst25)*(gconst9)))+(((IkReal(-1.00000000000000))*(gconst12)*(gconst15)*(gconst19)*(gconst6)))+(((IkReal(-1.00000000000000))*(gconst0)*(gconst15)*(gconst31)*(gconst6))));
op[6]=((((IkReal(-1.00000000000000))*(gconst12)*(x60)*(x61)))+(((IkReal(-1.00000000000000))*(gconst5)*(x40)*(x81)))+(((IkReal(-1.00000000000000))*(gconst22)*(x35)*(x36)))+(((IkReal(-1.00000000000000))*(x105)*(x78)))+(((IkReal(-1.00000000000000))*(x104)*(x46)))+(((IkReal(-1.00000000000000))*(gconst26)*(x39)*(x40)))+(((x39)*(x63)))+(((IkReal(-1.00000000000000))*(gconst1)*(x33)*(x49)))+(((IkReal(-1.00000000000000))*(gconst1)*(x43)*(x60)))+(((x42)*(x72)))+(((x50)*(x80)))+(((IkReal(-1.00000000000000))*(gconst13)*(x35)*(x97)))+(((IkReal(-1.00000000000000))*(x36)*(x52)))+(((IkReal(-1.00000000000000))*(gconst13)*(x35)*(x54)))+(((x54)*(x95)))+(((x43)*(x78)))+(((x37)*(x39)))+(((x100)*(x79)))+(((x54)*(x84)))+(((x47)*(x48)))+(((x44)*(x91)))+(((x78)*(x99)))+(((IkReal(-1.00000000000000))*(gconst0)*(x33)*(x44)))+(((x39)*(x85)))+(((IkReal(-1.00000000000000))*(x65)*(x66)))+(((x44)*(x52)))+(((x44)*(x57)))+(((x65)*(x86)))+(((x45)*(x61)))+(((x72)*(x92)))+(((IkReal(-1.00000000000000))*(gconst18)*(x67)*(x76)))+(((x36)*(x94)))+(((x36)*(x93)))+(((IkReal(-1.00000000000000))*(x39)*(x42)))+(((IkReal(-1.00000000000000))*(gconst16)*(x50)*(x75)))+(((IkReal(-1.00000000000000))*(gconst0)*(x61)*(x67)))+(((IkReal(-1.00000000000000))*(gconst20)*(x40)*(x82)))+(((x45)*(x48)))+(((x36)*(x71)))+(((IkReal(-1.00000000000000))*(gconst5)*(x49)*(x55)))+(((IkReal(-1.00000000000000))*(gconst16)*(x40)*(x95)))+(((IkReal(-1.00000000000000))*(gconst26)*(x75)*(x78)))+(((IkReal(-1.00000000000000))*(x39)*(x74)))+(((x49)*(x66)))+(((IkReal(-1.00000000000000))*(gconst13)*(x36)*(x70)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x85)))+(((x95)*(x97)))+(((IkReal(-1.00000000000000))*(gconst12)*(x40)*(x77)))+(((IkReal(-1.00000000000000))*(x32)*(x65)))+(((x72)*(x74)))+(((IkReal(-1.00000000000000))*(gconst29)*(x60)*(x76)))+(((x47)*(x61)))+(((x32)*(x49)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x37)))+(((gconst1)*(gconst30)*(x82)))+(((IkReal(-1.00000000000000))*(gconst29)*(x35)*(x46)))+(((x84)*(x97)))+(((IkReal(-1.00000000000000))*(x47)*(x54)))+(((x87)*(x88)))+(((x65)*(x68))));
op[7]=((((x39)*(x54)))+(((IkReal(-1.00000000000000))*(gconst13)*(x35)*(x36)))+(((IkReal(-1.00000000000000))*(gconst1)*(x49)*(x67)))+(((x100)*(x87)))+(((x78)*(x80)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x54)))+(((IkReal(-1.00000000000000))*(x39)*(x61)))+(((IkReal(-1.00000000000000))*(gconst18)*(x35)*(x46)))+(((x48)*(x72)))+(((x39)*(x97)))+(((IkReal(-1.00000000000000))*(gconst12)*(x40)*(x82)))+(((x36)*(x95)))+(((IkReal(-1.00000000000000))*(x50)*(x65)))+(((x36)*(x84)))+(((IkReal(-1.00000000000000))*(x36)*(x47)))+(((IkReal(-1.00000000000000))*(gconst16)*(x39)*(x40)))+(((x61)*(x72)))+(((x49)*(x50)))+(((IkReal(-1.00000000000000))*(gconst18)*(x60)*(x76)))+(((x44)*(x47)))+(((x44)*(x45)))+(((IkReal(-1.00000000000000))*(gconst16)*(x75)*(x78)))+(((IkReal(-1.00000000000000))*(gconst0)*(x40)*(x77)))+(((x65)*(x79))));
op[8]=((((IkReal(-1.00000000000000))*(gconst1)*(x49)*(x60)))+(((x36)*(x39)))+(((x49)*(x78)))+(((x65)*(x87)))+(((IkReal(-1.00000000000000))*(gconst6)*(x39)*(x40)))+(((IkReal(-1.00000000000000))*(x65)*(x78)))+(((IkReal(-1.00000000000000))*(gconst2)*(x35)*(x36)))+(((x44)*(x72))));
polyroots8(op,zeror,numroots);
IkReal j27array[8], cj27array[8], sj27array[8], tempj27array[1];
int numsolutions = 0;
for(int ij27 = 0; ij27 < numroots; ++ij27)
{
IkReal htj27 = zeror[ij27];
tempj27array[0]=((IkReal(2.00000000000000))*(atan(htj27)));
for(int kj27 = 0; kj27 < 1; ++kj27)
{
j27array[numsolutions] = tempj27array[kj27];
if( j27array[numsolutions] > IKPI )
{
    j27array[numsolutions]-=IK2PI;
}
else if( j27array[numsolutions] < -IKPI )
{
    j27array[numsolutions]+=IK2PI;
}
sj27array[numsolutions] = IKsin(j27array[numsolutions]);
cj27array[numsolutions] = IKcos(j27array[numsolutions]);
numsolutions++;
}
}
bool j27valid[8]={true,true,true,true,true,true,true,true};
_nj27 = 8;
for(int ij27 = 0; ij27 < numsolutions; ++ij27)
    {
if( !j27valid[ij27] )
{
    continue;
}
    j27 = j27array[ij27]; cj27 = cj27array[ij27]; sj27 = sj27array[ij27];
htj27 = IKtan(j27/2);

_ij27[0] = ij27; _ij27[1] = -1;
for(int iij27 = ij27+1; iij27 < numsolutions; ++iij27)
{
if( j27valid[iij27] && IKabs(cj27array[ij27]-cj27array[iij27]) < IKFAST_SOLUTION_THRESH && IKabs(sj27array[ij27]-sj27array[iij27]) < IKFAST_SOLUTION_THRESH )
{
    j27valid[iij27]=false; _ij27[1] = iij27; break; 
}
}
{
IkReal dummyeval[1];
IkReal x108=((IkReal(1.00000000000000))*(r02));
dummyeval[0]=((((pz)*(r01)*(sj27)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x108)))+(((IkReal(0.100000000000000))*(r02)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x108)))+(((cj27)*(pz)*(r00))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j28array[1], cj28array[1], sj28array[1];
bool j28valid[1]={false};
_nj28 = 1;
IkReal x109=((cj27)*(r00));
IkReal x110=((py)*(r01));
IkReal x111=((r01)*(sj27));
IkReal x112=((IkReal(0.800000000000000))*(r02));
IkReal x113=((py)*(sj27));
IkReal x114=((cj27)*(px));
IkReal x115=((IkReal(2.00000000000000))*(pp));
IkReal x116=((IkReal(0.800000000000000))*(pz));
IkReal x117=((IkReal(0.200000000000000))*(r02));
IkReal x118=((IkReal(1.60000000000000))*(pz));
IkReal x119=((IkReal(0.200000000000000))*(pz));
IkReal x120=((pz)*(r02));
IkReal x121=((IkReal(1.60000000000000))*(r02));
IkReal x122=((IkReal(2.00000000000000))*(pz));
IkReal x123=((px)*(r00));
if( IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x113)*(x121)))+(((x111)*(x118)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x118)))+(((IkReal(-1.00000000000000))*(x114)*(x121))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x113)*(x121)))+(((x111)*(x118)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x118)))+(((IkReal(-1.00000000000000))*(x114)*(x121)))))):(IkReal)1.0e30))*(((IkReal(0.0550608694346110))+(((IkReal(4.00000000000000))*(x114)*(x120)))+(((IkReal(-1.00000000000000))*(x111)*(x115)))+(((IkReal(4.00000000000000))*(x109)*((px)*(px))))+(((IkReal(-0.259678000000000))*(x109)))+(((IkReal(-0.259678000000000))*(x111)))+(((IkReal(4.00000000000000))*(x110)*(x113)))+(((IkReal(4.00000000000000))*(x110)*(x114)))+(((IkReal(-0.550608694346110))*(x113)))+(((IkReal(-0.550608694346110))*(x114)))+(((IkReal(-0.400000000000000))*(x110)))+(((IkReal(-1.00000000000000))*(x109)*(x115)))+(((IkReal(-0.400000000000000))*(x123)))+(((IkReal(-0.400000000000000))*(x120)))+(((IkReal(4.00000000000000))*(x113)*(x120)))+(((IkReal(4.00000000000000))*(x113)*(x123))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((IkReal(-1.00000000000000))*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)))+(((x111)*(x116)))+(((x109)*(x116)))+(((IkReal(0.0800000000000000))*(r02))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)))+(((x111)*(x116)))+(((x109)*(x116)))+(((IkReal(0.0800000000000000))*(r02)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x111)*(x119)))+(((x114)*(x117)))+(((x113)*(x117)))+(((x122)*(x123)))+(((IkReal(-1.00000000000000))*(pp)*(r02)))+(((x120)*(x122)))+(((IkReal(-1.00000000000000))*(x109)*(x119)))+(((IkReal(-0.149839000000000))*(r02)))+(((IkReal(-0.275304347173055))*(pz)))+(((x110)*(x122))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x113)*(x121)))+(((x111)*(x118)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x118)))+(((IkReal(-1.00000000000000))*(x114)*(x121))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x113)*(x121)))+(((x111)*(x118)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x118)))+(((IkReal(-1.00000000000000))*(x114)*(x121)))))):(IkReal)1.0e30))*(((IkReal(0.0550608694346110))+(((IkReal(4.00000000000000))*(x114)*(x120)))+(((IkReal(-1.00000000000000))*(x111)*(x115)))+(((IkReal(4.00000000000000))*(x109)*((px)*(px))))+(((IkReal(-0.259678000000000))*(x109)))+(((IkReal(-0.259678000000000))*(x111)))+(((IkReal(4.00000000000000))*(x110)*(x113)))+(((IkReal(4.00000000000000))*(x110)*(x114)))+(((IkReal(-0.550608694346110))*(x113)))+(((IkReal(-0.550608694346110))*(x114)))+(((IkReal(-0.400000000000000))*(x110)))+(((IkReal(-1.00000000000000))*(x109)*(x115)))+(((IkReal(-0.400000000000000))*(x123)))+(((IkReal(-0.400000000000000))*(x120)))+(((IkReal(4.00000000000000))*(x113)*(x120)))+(((IkReal(4.00000000000000))*(x113)*(x123)))))))+IKsqr(((((IKabs(((((IkReal(-1.00000000000000))*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)))+(((x111)*(x116)))+(((x109)*(x116)))+(((IkReal(0.0800000000000000))*(r02))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)))+(((x111)*(x116)))+(((x109)*(x116)))+(((IkReal(0.0800000000000000))*(r02)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x111)*(x119)))+(((x114)*(x117)))+(((x113)*(x117)))+(((x122)*(x123)))+(((IkReal(-1.00000000000000))*(pp)*(r02)))+(((x120)*(x122)))+(((IkReal(-1.00000000000000))*(x109)*(x119)))+(((IkReal(-0.149839000000000))*(r02)))+(((IkReal(-0.275304347173055))*(pz)))+(((x110)*(x122)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j28array[0]=IKatan2(((((IKabs(((((IkReal(-1.00000000000000))*(x113)*(x121)))+(((x111)*(x118)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x118)))+(((IkReal(-1.00000000000000))*(x114)*(x121))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x113)*(x121)))+(((x111)*(x118)))+(((IkReal(0.160000000000000))*(r02)))+(((x109)*(x118)))+(((IkReal(-1.00000000000000))*(x114)*(x121)))))):(IkReal)1.0e30))*(((IkReal(0.0550608694346110))+(((IkReal(4.00000000000000))*(x114)*(x120)))+(((IkReal(-1.00000000000000))*(x111)*(x115)))+(((IkReal(4.00000000000000))*(x109)*((px)*(px))))+(((IkReal(-0.259678000000000))*(x109)))+(((IkReal(-0.259678000000000))*(x111)))+(((IkReal(4.00000000000000))*(x110)*(x113)))+(((IkReal(4.00000000000000))*(x110)*(x114)))+(((IkReal(-0.550608694346110))*(x113)))+(((IkReal(-0.550608694346110))*(x114)))+(((IkReal(-0.400000000000000))*(x110)))+(((IkReal(-1.00000000000000))*(x109)*(x115)))+(((IkReal(-0.400000000000000))*(x123)))+(((IkReal(-0.400000000000000))*(x120)))+(((IkReal(4.00000000000000))*(x113)*(x120)))+(((IkReal(4.00000000000000))*(x113)*(x123)))))), ((((IKabs(((((IkReal(-1.00000000000000))*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)))+(((x111)*(x116)))+(((x109)*(x116)))+(((IkReal(0.0800000000000000))*(r02))))) != 0)?((IkReal)1/(((((IkReal(-1.00000000000000))*(x112)*(x113)))+(((IkReal(-1.00000000000000))*(x112)*(x114)))+(((x111)*(x116)))+(((x109)*(x116)))+(((IkReal(0.0800000000000000))*(r02)))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x111)*(x119)))+(((x114)*(x117)))+(((x113)*(x117)))+(((x122)*(x123)))+(((IkReal(-1.00000000000000))*(pp)*(r02)))+(((x120)*(x122)))+(((IkReal(-1.00000000000000))*(x109)*(x119)))+(((IkReal(-0.149839000000000))*(r02)))+(((IkReal(-0.275304347173055))*(pz)))+(((x110)*(x122)))))));
sj28array[0]=IKsin(j28array[0]);
cj28array[0]=IKcos(j28array[0]);
if( j28array[0] > IKPI )
{
    j28array[0]-=IK2PI;
}
else if( j28array[0] < -IKPI )
{    j28array[0]+=IK2PI;
}
j28valid[0] = true;
for(int ij28 = 0; ij28 < 1; ++ij28)
{
if( !j28valid[ij28] )
{
    continue;
}
_ij28[0] = ij28; _ij28[1] = -1;
for(int iij28 = ij28+1; iij28 < 1; ++iij28)
{
if( j28valid[iij28] && IKabs(cj28array[ij28]-cj28array[iij28]) < IKFAST_SOLUTION_THRESH && IKabs(sj28array[ij28]-sj28array[iij28]) < IKFAST_SOLUTION_THRESH )
{
    j28valid[iij28]=false; _ij28[1] = iij28; break; 
}
}
j28 = j28array[ij28]; cj28 = cj28array[ij28]; sj28 = sj28array[ij28];
{
IkReal evalcond[2];
IkReal x124=IKcos(j28);
IkReal x125=IKsin(j28);
IkReal x126=((cj27)*(px));
IkReal x127=((cj27)*(r00));
IkReal x128=((sj27)*(x124));
evalcond[0]=((IkReal(0.149839000000000))+(((IkReal(-0.200000000000000))*(x126)))+(((IkReal(-0.800000000000000))*(py)*(x128)))+(((IkReal(-0.800000000000000))*(x124)*(x126)))+(pp)+(((IkReal(0.0800000000000000))*(x124)))+(((IkReal(0.800000000000000))*(pz)*(x125)))+(((IkReal(-0.200000000000000))*(py)*(sj27))));
evalcond[1]=((IkReal(-0.137652173586528))+(((px)*(r00)))+(((IkReal(-0.100000000000000))*(r01)*(sj27)))+(((IkReal(-0.100000000000000))*(x127)))+(((IkReal(-0.400000000000000))*(r01)*(x128)))+(((pz)*(r02)))+(((py)*(r01)))+(((IkReal(0.400000000000000))*(r02)*(x125)))+(((IkReal(-0.400000000000000))*(x124)*(x127))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal j30array[2], cj30array[2], sj30array[2];
bool j30valid[2]={false};
_nj30 = 2;
IkReal x129=((IkReal(15.3222050514291))*(cj28));
IkReal x130=((IkReal(1.26340886242921))*(cj28));
cj30array[0]=((IkReal(-6.12888202057164))+(((IkReal(-15.3222050514291))*(pz)*(sj28)))+(((py)*(sj27)*(x129)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x130)))+(((IkReal(1.26340886242921))*(r02)*(sj28)))+(((cj27)*(px)*(x129)))+(((IkReal(-1.53222050514291))*(cj28)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x130))));
if( cj30array[0] >= -1-IKFAST_SINCOS_THRESH && cj30array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j30valid[0] = j30valid[1] = true;
    j30array[0] = IKacos(cj30array[0]);
    sj30array[0] = IKsin(j30array[0]);
    cj30array[1] = cj30array[0];
    j30array[1] = -j30array[0];
    sj30array[1] = -sj30array[0];
}
else if( isnan(cj30array[0]) )
{
    // probably any value will work
    j30valid[0] = true;
    cj30array[0] = 1; sj30array[0] = 0; j30array[0] = 0;
}
for(int ij30 = 0; ij30 < 2; ++ij30)
{
if( !j30valid[ij30] )
{
    continue;
}
_ij30[0] = ij30; _ij30[1] = -1;
for(int iij30 = ij30+1; iij30 < 2; ++iij30)
{
if( j30valid[iij30] && IKabs(cj30array[ij30]-cj30array[iij30]) < IKFAST_SOLUTION_THRESH && IKabs(sj30array[ij30]-sj30array[iij30]) < IKFAST_SOLUTION_THRESH )
{
    j30valid[iij30]=false; _ij30[1] = iij30; break; 
}
}
j30 = j30array[ij30]; cj30 = cj30array[ij30]; sj30 = sj30array[ij30];

{
IkReal dummyeval[1];
IkReal gconst48;
gconst48=IKsign(sj30);
dummyeval[0]=sj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=sj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x131=((IkReal(1.00000000000000))*(sj28));
IkReal x132=((cj27)*(cj28));
IkReal x133=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x131)))+(((px)*(x132)))+(((py)*(x133)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(-0.845725967038483))+(((r00)*(x132)))+(((r01)*(x133)))+(((IkReal(-1.00000000000000))*(r02)*(x131))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst56;
gconst56=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst57;
gconst57=IkReal(0.0880000000000000);
IkReal gconst58;
gconst58=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst59;
gconst59=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst60;
gconst60=IkReal(1.06723491074238);
IkReal gconst61;
gconst61=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst62;
gconst62=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst63;
gconst63=IkReal(0.0880000000000000);
IkReal gconst64;
gconst64=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst65;
gconst65=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst66;
gconst66=IkReal(1.06723491074238);
IkReal gconst67;
gconst67=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst68;
gconst68=IkReal(0.0880000000000000);
IkReal gconst69;
gconst69=IkReal(-0.0880000000000000);
IkReal gconst70;
gconst70=IkReal(1.06723491074238);
IkReal gconst71;
gconst71=IkReal(-1.06723491074238);
IkReal gconst72;
gconst72=IkReal(0.0880000000000000);
IkReal gconst73;
gconst73=IkReal(-0.0880000000000000);
IkReal gconst74;
gconst74=IkReal(1.06723491074238);
IkReal gconst75;
gconst75=IkReal(-1.06723491074238);
IkReal gconst76;
gconst76=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst77;
gconst77=IkReal(-0.0880000000000000);
IkReal gconst78;
gconst78=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst79;
gconst79=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst80;
gconst80=IkReal(-1.06723491074238);
IkReal gconst81;
gconst81=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst82;
gconst82=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst83;
gconst83=IkReal(-0.0880000000000000);
IkReal gconst84;
gconst84=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst85;
gconst85=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst86;
gconst86=IkReal(-1.06723491074238);
IkReal gconst87;
gconst87=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal x134=((gconst76)*(gconst81));
IkReal x135=((IkReal(1.00000000000000))*(gconst82));
IkReal x136=((gconst84)*(gconst85));
IkReal x137=((gconst78)*(gconst79));
IkReal x138=((gconst78)*(gconst80));
IkReal x139=((gconst83)*(gconst85));
IkReal x140=((gconst77)*(gconst81));
dummyeval[0]=((((x138)*(x139)))+(((IkReal(-1.00000000000000))*(gconst87)*(x134)*(x135)))+(((IkReal(-1.00000000000000))*(x139)*(x140)))+(((x134)*(x136)))+(((IkReal(-1.00000000000000))*(gconst86)*(x135)*(x138)))+(((IkReal(-1.00000000000000))*(x136)*(x137)))+(((gconst82)*(gconst87)*(x137)))+(((gconst82)*(gconst86)*(x140))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x141=((gconst84)*(gconst85));
IkReal x142=((gconst68)*(gconst81));
IkReal x143=((IkReal(1.00000000000000))*(gconst58));
IkReal x144=((gconst59)*(gconst64));
IkReal x145=((gconst78)*(gconst80));
IkReal x146=((gconst83)*(gconst85));
IkReal x147=((gconst62)*(gconst67));
IkReal x148=((gconst59)*(gconst69));
IkReal x149=((gconst82)*(gconst86));
IkReal x150=((IkReal(1.00000000000000))*(gconst56));
IkReal x151=((gconst62)*(gconst71));
IkReal x152=((gconst69)*(gconst79));
IkReal x153=((gconst75)*(gconst82));
IkReal x154=((gconst60)*(gconst66));
IkReal x155=((IkReal(1.00000000000000))*(gconst61));
IkReal x156=((gconst63)*(gconst74));
IkReal x157=((gconst76)*(gconst81));
IkReal x158=((gconst60)*(gconst69));
IkReal x159=((gconst63)*(gconst65));
IkReal x160=((gconst73)*(gconst85));
IkReal x161=((gconst73)*(gconst74));
IkReal x162=((gconst58)*(gconst59));
IkReal x163=((gconst64)*(gconst85));
IkReal x164=((IkReal(1.00000000000000))*(gconst62));
IkReal x165=((gconst59)*(gconst78));
IkReal x166=((gconst82)*(gconst87));
IkReal x167=((gconst65)*(gconst84));
IkReal x168=((gconst67)*(gconst72));
IkReal x169=((gconst57)*(gconst66));
IkReal x170=((gconst62)*(gconst81));
IkReal x171=((gconst61)*(gconst77));
IkReal x172=((IkReal(1.00000000000000))*(gconst78));
IkReal x173=((gconst56)*(gconst61));
IkReal x174=((gconst78)*(gconst79));
IkReal x175=((IkReal(1.00000000000000))*(gconst69));
IkReal x176=((gconst61)*(gconst82));
IkReal x177=((gconst77)*(gconst81));
IkReal x178=((gconst65)*(gconst73));
IkReal x179=((gconst72)*(gconst86));
IkReal x180=((gconst58)*(gconst60));
IkReal x181=((gconst69)*(gconst70));
IkReal x182=((IkReal(1.00000000000000))*(gconst71));
IkReal x183=((gconst62)*(gconst87));
IkReal x184=((gconst58)*(gconst70));
IkReal x185=((gconst65)*(gconst83));
IkReal x186=((gconst70)*(gconst78));
IkReal x187=((gconst74)*(gconst84));
IkReal x188=((gconst72)*(gconst75));
IkReal x189=((gconst74)*(gconst83));
IkReal x190=((gconst58)*(gconst79));
IkReal x191=((gconst71)*(gconst77));
IkReal x192=((gconst60)*(gconst78));
IkReal x193=((gconst58)*(gconst80));
IkReal x194=((gconst71)*(gconst76));
IkReal x195=((gconst64)*(gconst74));
IkReal x196=((gconst63)*(gconst85));
IkReal x197=((gconst62)*(gconst86));
IkReal x198=((gconst57)*(gconst81));
IkReal x199=((gconst62)*(gconst75));
IkReal x200=((gconst67)*(gconst82));
IkReal x201=((gconst61)*(gconst68));
IkReal x202=((gconst64)*(gconst65));
IkReal x203=((gconst56)*(gconst71));
IkReal x204=((gconst61)*(gconst76));
IkReal x205=((gconst72)*(gconst87));
IkReal x206=((gconst69)*(gconst80));
IkReal x207=((gconst66)*(gconst82));
IkReal x208=((gconst62)*(gconst66));
IkReal x209=((gconst68)*(gconst71));
IkReal x210=((IkReal(1.00000000000000))*(x205));
IkReal x211=((IkReal(1.00000000000000))*(x187));
IkReal x212=((gconst56)*(x202));
IkReal x213=((IkReal(1.00000000000000))*(gconst81)*(x166));
op[0]=((((x149)*(x177)))+(((x145)*(x146)))+(((x141)*(x157)))+(((IkReal(-1.00000000000000))*(gconst79)*(x141)*(x172)))+(((x166)*(x174)))+(((IkReal(-1.00000000000000))*(x157)*(x166)))+(((IkReal(-1.00000000000000))*(x145)*(x149)))+(((IkReal(-1.00000000000000))*(x146)*(x177))));
op[1]=((((x157)*(x187)))+(((x157)*(x160)))+(((IkReal(-1.00000000000000))*(x157)*(x210)))+(((x141)*(x194)))+(((x152)*(x166)))+(((x149)*(x191)))+(((IkReal(-1.00000000000000))*(x142)*(x166)))+(((IkReal(-1.00000000000000))*(gconst70)*(x141)*(x172)))+(((IkReal(-1.00000000000000))*(gconst76)*(x166)*(x182)))+(((x166)*(x186)))+(((x141)*(x142)))+(((IkReal(-1.00000000000000))*(x153)*(x157)))+(((x145)*(x189)))+(((IkReal(-1.00000000000000))*(gconst79)*(x172)*(x187)))+(((IkReal(-1.00000000000000))*(gconst77)*(x146)*(x182)))+(((IkReal(-1.00000000000000))*(gconst79)*(x160)*(x172)))+(((IkReal(-1.00000000000000))*(x145)*(x179)))+(((x146)*(x206)))+(((IkReal(-1.00000000000000))*(gconst80)*(x149)*(x175)))+(((x153)*(x174)))+(((IkReal(-1.00000000000000))*(x141)*(x152)))+(((IkReal(-1.00000000000000))*(x177)*(x189)))+(((x174)*(x205)))+(((x177)*(x179))));
op[2]=((((IkReal(-1.00000000000000))*(gconst76)*(x155)*(x166)))+(((IkReal(-1.00000000000000))*(gconst80)*(x175)*(x179)))+(((x153)*(x186)))+(((x160)*(x194)))+(((x157)*(x167)))+(((x157)*(x161)))+(((x157)*(x163)))+(((IkReal(-1.00000000000000))*(x142)*(x153)))+(((IkReal(-1.00000000000000))*(gconst70)*(x160)*(x172)))+(((x187)*(x194)))+(((x179)*(x191)))+(((IkReal(-1.00000000000000))*(gconst79)*(x167)*(x172)))+(((IkReal(-1.00000000000000))*(x152)*(x160)))+(((x149)*(x171)))+(((IkReal(-1.00000000000000))*(x157)*(x200)))+(((IkReal(-1.00000000000000))*(gconst68)*(x166)*(x182)))+(((x152)*(x205)))+(((x149)*(x198)))+(((x152)*(x153)))+(((x174)*(x183)))+(((x174)*(x188)))+(((x189)*(x206)))+(((IkReal(-1.00000000000000))*(gconst70)*(x172)*(x187)))+(((IkReal(-1.00000000000000))*(x152)*(x211)))+(((x166)*(x190)))+(((IkReal(-1.00000000000000))*(gconst70)*(x141)*(x175)))+(((x166)*(x181)))+(((IkReal(-1.00000000000000))*(gconst77)*(x146)*(x155)))+(((IkReal(-1.00000000000000))*(gconst79)*(x163)*(x172)))+(((x141)*(x209)))+(((x141)*(x204)))+(((IkReal(-1.00000000000000))*(gconst81)*(x150)*(x166)))+(((IkReal(-1.00000000000000))*(x142)*(x210)))+(((gconst56)*(gconst81)*(x141)))+(((x146)*(x192)))+(((x146)*(x193)))+(((x145)*(x185)))+(((x177)*(x207)))+(((IkReal(-1.00000000000000))*(x146)*(x198)))+(((IkReal(-1.00000000000000))*(gconst79)*(x141)*(x143)))+(((IkReal(-1.00000000000000))*(gconst79)*(x161)*(x172)))+(((IkReal(-1.00000000000000))*(x141)*(x165)))+(((x145)*(x196)))+(((IkReal(-1.00000000000000))*(x157)*(x188)))+(((x165)*(x166)))+(((IkReal(-1.00000000000000))*(gconst87)*(x157)*(x164)))+(((IkReal(-1.00000000000000))*(gconst76)*(x182)*(x205)))+(((x142)*(x187)))+(((IkReal(-1.00000000000000))*(gconst60)*(x149)*(x172)))+(((IkReal(-1.00000000000000))*(x177)*(x196)))+(((IkReal(-1.00000000000000))*(x145)*(x207)))+(((IkReal(-1.00000000000000))*(gconst76)*(x153)*(x182)))+(((IkReal(-1.00000000000000))*(x177)*(x185)))+(((gconst77)*(gconst86)*(x170)))+(((x142)*(x160)))+(((x186)*(x205)))+(((x174)*(x200)))+(((IkReal(-1.00000000000000))*(gconst86)*(x145)*(x164)))+(((IkReal(-1.00000000000000))*(gconst80)*(x143)*(x149)))+(((IkReal(-1.00000000000000))*(gconst77)*(x182)*(x189))));
op[3]=((((gconst69)*(gconst72)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst71)*(gconst83)*(gconst85)))+(((gconst65)*(gconst71)*(gconst76)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst84)*(gconst85)))+(((gconst59)*(gconst72)*(gconst78)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst69)*(gconst82)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst74)*(gconst78)*(gconst79)))+(((gconst64)*(gconst74)*(gconst76)*(gconst81)))+(((gconst58)*(gconst74)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst78)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst84)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst79)*(gconst85)))+(((gconst57)*(gconst71)*(gconst82)*(gconst86)))+(((gconst64)*(gconst68)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst71)*(gconst72)*(gconst87)))+(((gconst71)*(gconst73)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst69)*(gconst80)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst71)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst70)*(gconst78)*(gconst84)))+(((gconst62)*(gconst69)*(gconst79)*(gconst87)))+(((gconst60)*(gconst74)*(gconst78)*(gconst83)))+(((gconst69)*(gconst70)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst75)*(gconst76)*(gconst82)))+(((gconst56)*(gconst71)*(gconst84)*(gconst85)))+(((gconst62)*(gconst71)*(gconst77)*(gconst86)))+(((gconst67)*(gconst70)*(gconst78)*(gconst82)))+(((gconst63)*(gconst69)*(gconst80)*(gconst85)))+(((gconst66)*(gconst71)*(gconst77)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst72)*(gconst78)*(gconst86)))+(((gconst58)*(gconst72)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst75)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst70)*(gconst73)*(gconst74)*(gconst78)))+(((gconst62)*(gconst70)*(gconst78)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst74)*(gconst78)*(gconst84)))+(((gconst69)*(gconst70)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst73)*(gconst78)*(gconst79)))+(((gconst56)*(gconst73)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst74)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst82)*(gconst87)))+(((gconst61)*(gconst73)*(gconst76)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst72)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst74)*(gconst81)*(gconst83)))+(((gconst68)*(gconst71)*(gconst74)*(gconst84)))+(((gconst63)*(gconst74)*(gconst78)*(gconst80)))+(((gconst59)*(gconst69)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst74)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst72)*(gconst75)*(gconst81)))+(((gconst67)*(gconst72)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst72)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst71)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst73)*(gconst78)*(gconst85)))+(((gconst65)*(gconst68)*(gconst81)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst75)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst74)*(gconst77)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst76)*(gconst87)))+(((gconst64)*(gconst71)*(gconst76)*(gconst85)))+(((gconst59)*(gconst75)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst76)*(gconst82)))+(((gconst68)*(gconst71)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst73)*(gconst74)*(gconst79)))+(((gconst70)*(gconst72)*(gconst75)*(gconst78)))+(((gconst67)*(gconst69)*(gconst79)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst72)*(gconst78)*(gconst80)))+(((gconst66)*(gconst72)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst69)*(gconst80)*(gconst86)))+(((gconst65)*(gconst73)*(gconst76)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst74)*(gconst84)))+(((gconst56)*(gconst74)*(gconst81)*(gconst84)))+(((gconst65)*(gconst69)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst73)*(gconst85)))+(((gconst61)*(gconst72)*(gconst77)*(gconst86)))+(((gconst61)*(gconst74)*(gconst76)*(gconst84)))+(((gconst58)*(gconst75)*(gconst79)*(gconst82)))+(((gconst61)*(gconst68)*(gconst84)*(gconst85)))+(((gconst68)*(gconst73)*(gconst74)*(gconst81)))+(((gconst58)*(gconst70)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst71)*(gconst72)*(gconst75)*(gconst76)))+(((gconst62)*(gconst75)*(gconst78)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst71)*(gconst76)*(gconst87)))+(((gconst57)*(gconst72)*(gconst81)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst73)*(gconst79)*(gconst85)))+(((gconst60)*(gconst69)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst72)*(gconst80)*(gconst86))));
op[4]=((((gconst67)*(gconst69)*(gconst70)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst71)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst70)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst59)*(gconst84)*(gconst85)))+(((gconst64)*(gconst68)*(gconst71)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst63)*(gconst77)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst76)*(gconst82)))+(((gconst62)*(gconst69)*(gconst70)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst72)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst72)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst70)*(gconst74)*(gconst78)))+(((gconst58)*(gconst62)*(gconst79)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst71)*(gconst75)*(gconst76)))+(((gconst60)*(gconst63)*(gconst78)*(gconst85)))+(((gconst57)*(gconst71)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst65)*(gconst77)*(gconst83)))+(((gconst61)*(gconst68)*(gconst73)*(gconst85)))+(((gconst56)*(gconst71)*(gconst73)*(gconst85)))+(((gconst57)*(gconst66)*(gconst81)*(gconst82)))+(((gconst58)*(gconst63)*(gconst80)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst70)*(gconst73)*(gconst78)))+(((gconst61)*(gconst66)*(gconst77)*(gconst82)))+(((gconst58)*(gconst60)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst66)*(gconst80)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst75)*(gconst82)))+(((gconst58)*(gconst65)*(gconst80)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst75)*(gconst82)))+(((gconst62)*(gconst69)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst76)*(gconst87)))+(((gconst65)*(gconst71)*(gconst73)*(gconst76)))+(((gconst64)*(gconst68)*(gconst74)*(gconst81)))+(((gconst64)*(gconst71)*(gconst74)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst68)*(gconst71)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst78)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst62)*(gconst81)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst79)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst73)*(gconst79)))+(((gconst61)*(gconst73)*(gconst74)*(gconst76)))+(((gconst64)*(gconst65)*(gconst76)*(gconst81)))+(((gconst58)*(gconst72)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst72)*(gconst75)*(gconst76)))+(((gconst59)*(gconst69)*(gconst75)*(gconst82)))+(((gconst58)*(gconst70)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst73)*(gconst74)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst60)*(gconst82)*(gconst86)))+(((gconst58)*(gconst67)*(gconst79)*(gconst82)))+(((gconst57)*(gconst62)*(gconst81)*(gconst86)))+(((gconst67)*(gconst69)*(gconst72)*(gconst79)))+(((gconst59)*(gconst72)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst67)*(gconst76)*(gconst81)))+(((gconst68)*(gconst71)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst71)*(gconst87)))+(((gconst56)*(gconst64)*(gconst81)*(gconst85)))+(((gconst62)*(gconst66)*(gconst77)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst74)*(gconst84)))+(((gconst56)*(gconst73)*(gconst74)*(gconst81)))+(((gconst58)*(gconst70)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst66)*(gconst69)*(gconst72)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst78)*(gconst85)))+(((gconst58)*(gconst59)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst75)*(gconst81)))+(((gconst61)*(gconst68)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst67)*(gconst81)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst61)*(gconst82)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst71)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst65)*(gconst78)*(gconst79)))+(((gconst65)*(gconst68)*(gconst71)*(gconst84)))+(((gconst57)*(gconst61)*(gconst82)*(gconst86)))+(((gconst56)*(gconst71)*(gconst74)*(gconst84)))+(((gconst59)*(gconst62)*(gconst78)*(gconst87)))+(((gconst61)*(gconst62)*(gconst77)*(gconst86)))+(((gconst62)*(gconst70)*(gconst75)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst77)*(gconst81)))+(((gconst59)*(gconst67)*(gconst78)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst65)*(gconst81)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst69)*(gconst72)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst73)*(gconst74)*(gconst79)))+(((gconst56)*(gconst61)*(gconst84)*(gconst85)))+(((gconst61)*(gconst65)*(gconst76)*(gconst84)))+(((gconst69)*(gconst70)*(gconst72)*(gconst75)))+(((gconst66)*(gconst71)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst78)*(gconst80)))+(((gconst63)*(gconst69)*(gconst74)*(gconst80)))+(((gconst62)*(gconst67)*(gconst78)*(gconst79)))+(((gconst63)*(gconst65)*(gconst78)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst74)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst71)*(gconst74)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst69)*(gconst70)*(gconst73)*(gconst74)))+(((gconst65)*(gconst68)*(gconst73)*(gconst81)))+(((gconst60)*(gconst69)*(gconst74)*(gconst83)))+(((gconst67)*(gconst70)*(gconst72)*(gconst78)))+(((gconst61)*(gconst64)*(gconst76)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst72)*(gconst87)))+(((gconst59)*(gconst69)*(gconst72)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst79)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst63)*(gconst81)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst78)*(gconst84)))+(((gconst56)*(gconst65)*(gconst81)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst83)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst71)*(gconst74)*(gconst77)))+(((gconst60)*(gconst65)*(gconst78)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst62)*(gconst80)*(gconst86))));
op[5]=((((gconst58)*(gconst62)*(gconst75)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst68)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst67)*(gconst71)*(gconst82)))+(((gconst61)*(gconst64)*(gconst68)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst66)*(gconst69)*(gconst80)))+(((gconst61)*(gconst65)*(gconst73)*(gconst76)))+(((gconst58)*(gconst67)*(gconst72)*(gconst79)))+(((gconst56)*(gconst61)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst63)*(gconst71)*(gconst85)))+(((gconst56)*(gconst61)*(gconst73)*(gconst85)))+(((gconst60)*(gconst65)*(gconst69)*(gconst83)))+(((gconst58)*(gconst67)*(gconst70)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst60)*(gconst72)*(gconst86)))+(((gconst60)*(gconst63)*(gconst74)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst70)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst74)*(gconst79)))+(((gconst59)*(gconst62)*(gconst75)*(gconst78)))+(((gconst58)*(gconst63)*(gconst74)*(gconst80)))+(((gconst61)*(gconst68)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst73)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst63)*(gconst74)*(gconst77)))+(((gconst58)*(gconst59)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst67)*(gconst72)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst71)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst68)*(gconst87)))+(((gconst63)*(gconst65)*(gconst69)*(gconst80)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst74)*(gconst78)))+(((gconst56)*(gconst71)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst66)*(gconst72)*(gconst80)))+(((gconst59)*(gconst67)*(gconst72)*(gconst78)))+(((gconst62)*(gconst67)*(gconst69)*(gconst79)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst63)*(gconst74)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst65)*(gconst70)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst67)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst63)*(gconst65)*(gconst71)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst62)*(gconst71)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst65)*(gconst73)*(gconst79)))+(((gconst57)*(gconst61)*(gconst72)*(gconst86)))+(((gconst61)*(gconst64)*(gconst74)*(gconst76)))+(((gconst56)*(gconst65)*(gconst73)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst65)*(gconst71)*(gconst83)))+(((gconst56)*(gconst64)*(gconst71)*(gconst85)))+(((gconst61)*(gconst65)*(gconst68)*(gconst84)))+(((gconst62)*(gconst67)*(gconst70)*(gconst78)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst65)*(gconst69)*(gconst84)))+(((gconst57)*(gconst66)*(gconst72)*(gconst81)))+(((gconst58)*(gconst59)*(gconst72)*(gconst87)))+(((gconst62)*(gconst66)*(gconst71)*(gconst77)))+(((gconst59)*(gconst69)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst69)*(gconst70)*(gconst74)))+(((gconst57)*(gconst66)*(gconst71)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst62)*(gconst69)*(gconst86)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst68)*(gconst72)*(gconst75)))+(((gconst56)*(gconst65)*(gconst71)*(gconst84)))+(((gconst58)*(gconst60)*(gconst74)*(gconst83)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst61)*(gconst75)*(gconst82)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst70)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst67)*(gconst68)*(gconst81)))+(((gconst57)*(gconst62)*(gconst71)*(gconst86)))+(((gconst60)*(gconst63)*(gconst69)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst64)*(gconst69)*(gconst85)))+(((gconst56)*(gconst64)*(gconst74)*(gconst81)))+(((gconst67)*(gconst69)*(gconst70)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst59)*(gconst73)*(gconst85)))+(((IkReal(-1.00000000000000))*(gconst64)*(gconst65)*(gconst69)*(gconst79)))+(((gconst64)*(gconst65)*(gconst71)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst57)*(gconst61)*(gconst74)*(gconst83)))+(((gconst58)*(gconst62)*(gconst70)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst61)*(gconst72)*(gconst87)))+(((gconst64)*(gconst65)*(gconst68)*(gconst81)))+(((gconst59)*(gconst62)*(gconst69)*(gconst87)))+(((IkReal(-1.00000000000000))*(gconst65)*(gconst69)*(gconst70)*(gconst73)))+(((IkReal(-1.00000000000000))*(gconst62)*(gconst68)*(gconst71)*(gconst75)))+(((gconst65)*(gconst68)*(gconst71)*(gconst73)))+(((gconst64)*(gconst68)*(gconst71)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst61)*(gconst62)*(gconst75)*(gconst76)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst64)*(gconst70)*(gconst85)))+(((gconst62)*(gconst69)*(gconst70)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst59)*(gconst69)*(gconst73)*(gconst74)))+(((IkReal(-1.00000000000000))*(gconst67)*(gconst68)*(gconst71)*(gconst72)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst69)*(gconst82)))+(((gconst61)*(gconst66)*(gconst72)*(gconst77)))+(((IkReal(-1.00000000000000))*(gconst58)*(gconst59)*(gconst74)*(gconst84)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst62)*(gconst75)*(gconst81)))+(((IkReal(-1.00000000000000))*(gconst56)*(gconst67)*(gconst72)*(gconst81)))+(((gconst58)*(gconst70)*(gconst72)*(gconst75)))+(((IkReal(-1.00000000000000))*(gconst60)*(gconst66)*(gconst72)*(gconst78)))+(((gconst59)*(gconst67)*(gconst69)*(gconst82))));
op[6]=((((IkReal(-1.00000000000000))*(gconst85)*(x143)*(x144)))+(((x161)*(x173)))+(((x171)*(x208)))+(((IkReal(-1.00000000000000))*(gconst71)*(x150)*(x168)))+(((IkReal(-1.00000000000000))*(gconst72)*(x154)*(x175)))+(((x180)*(x185)))+(((x168)*(x184)))+(((IkReal(-1.00000000000000))*(gconst57)*(x156)*(x182)))+(((x162)*(x188)))+(((x162)*(x183)))+(((IkReal(-1.00000000000000))*(gconst60)*(x143)*(x197)))+(((x180)*(x196)))+(((IkReal(-1.00000000000000))*(gconst70)*(x143)*(x178)))+(((IkReal(-1.00000000000000))*(gconst59)*(x143)*(x161)))+(((IkReal(-1.00000000000000))*(gconst59)*(x143)*(x167)))+(((x195)*(x201)))+(((x195)*(x203)))+(((x169)*(x176)))+(((x169)*(x170)))+(((IkReal(-1.00000000000000))*(gconst68)*(x147)*(x182)))+(((gconst81)*(x212)))+(((x184)*(x199)))+(((IkReal(-1.00000000000000))*(x159)*(x198)))+(((IkReal(-1.00000000000000))*(gconst77)*(x155)*(x159)))+(((x162)*(x200)))+(((x147)*(x165)))+(((x167)*(x173)))+(((x148)*(x168)))+(((IkReal(-1.00000000000000))*(gconst61)*(x150)*(x183)))+(((IkReal(-1.00000000000000))*(gconst61)*(x150)*(x188)))+(((IkReal(-1.00000000000000))*(gconst79)*(x143)*(x202)))+(((IkReal(-1.00000000000000))*(gconst81)*(x147)*(x150)))+(((x178)*(x203)))+(((x178)*(x201)))+(((IkReal(-1.00000000000000))*(gconst65)*(x144)*(x172)))+(((IkReal(-1.00000000000000))*(gconst75)*(x150)*(x151)))+(((IkReal(-1.00000000000000))*(gconst74)*(x144)*(x175)))+(((x147)*(x181)))+(((x202)*(x204)))+(((x202)*(x209)))+(((IkReal(-1.00000000000000))*(gconst82)*(x143)*(x154)))+(((IkReal(-1.00000000000000))*(gconst68)*(x155)*(x199)))+(((x156)*(x158)))+(((x147)*(x190)))+(((IkReal(-1.00000000000000))*(x148)*(x178)))+(((x148)*(x199)))+(((IkReal(-1.00000000000000))*(gconst70)*(x175)*(x202)))+(((gconst71)*(gconst72)*(x169)))+(((IkReal(-1.00000000000000))*(gconst67)*(x150)*(x176)))+(((IkReal(-1.00000000000000))*(gconst80)*(x143)*(x208)))+(((IkReal(-1.00000000000000))*(gconst57)*(x155)*(x196)))+(((x163)*(x173)))+(((x159)*(x192)))+(((x159)*(x193)))+(((IkReal(-1.00000000000000))*(gconst78)*(x154)*(x164)))+(((IkReal(-1.00000000000000))*(gconst76)*(x147)*(x155)))+(((gconst57)*(gconst61)*(x197)))+(((IkReal(-1.00000000000000))*(gconst57)*(x155)*(x185)))+(((IkReal(-1.00000000000000))*(gconst70)*(x143)*(x195)))+(((IkReal(-1.00000000000000))*(gconst68)*(x155)*(x168))));
op[7]=((((IkReal(-1.00000000000000))*(gconst70)*(x143)*(x202)))+(((IkReal(-1.00000000000000))*(gconst71)*(x147)*(x150)))+(((x162)*(x199)))+(((x151)*(x169)))+(((IkReal(-1.00000000000000))*(gconst57)*(x159)*(x182)))+(((x173)*(x195)))+(((x162)*(x168)))+(((x201)*(x202)))+(((IkReal(-1.00000000000000))*(gconst59)*(x143)*(x178)))+(((IkReal(-1.00000000000000))*(gconst57)*(x155)*(x156)))+(((IkReal(-1.00000000000000))*(gconst74)*(x143)*(x144)))+(((x156)*(x180)))+(((x173)*(x178)))+(((IkReal(-1.00000000000000))*(gconst65)*(x144)*(x175)))+(((IkReal(-1.00000000000000))*(gconst69)*(x154)*(x164)))+(((IkReal(-1.00000000000000))*(gconst61)*(x150)*(x199)))+(((x147)*(x184)))+(((x147)*(x148)))+(((x202)*(x203)))+(((IkReal(-1.00000000000000))*(gconst68)*(x147)*(x155)))+(((gconst61)*(gconst72)*(x169)))+(((x158)*(x159)))+(((IkReal(-1.00000000000000))*(gconst61)*(x150)*(x168)))+(((IkReal(-1.00000000000000))*(gconst72)*(x143)*(x154))));
op[8]=((((IkReal(-1.00000000000000))*(gconst57)*(x155)*(x159)))+(((IkReal(-1.00000000000000))*(gconst61)*(x147)*(x150)))+(((x147)*(x162)))+(((IkReal(-1.00000000000000))*(gconst65)*(x143)*(x144)))+(((gconst61)*(gconst62)*(x169)))+(((x173)*(x202)))+(((IkReal(-1.00000000000000))*(gconst62)*(x143)*(x154)))+(((x159)*(x180))));
polyroots8(op,zeror,numroots);
IkReal j29array[8], cj29array[8], sj29array[8], tempj29array[1];
int numsolutions = 0;
for(int ij29 = 0; ij29 < numroots; ++ij29)
{
IkReal htj29 = zeror[ij29];
tempj29array[0]=((IkReal(2.00000000000000))*(atan(htj29)));
for(int kj29 = 0; kj29 < 1; ++kj29)
{
j29array[numsolutions] = tempj29array[kj29];
if( j29array[numsolutions] > IKPI )
{
    j29array[numsolutions]-=IK2PI;
}
else if( j29array[numsolutions] < -IKPI )
{
    j29array[numsolutions]+=IK2PI;
}
sj29array[numsolutions] = IKsin(j29array[numsolutions]);
cj29array[numsolutions] = IKcos(j29array[numsolutions]);
numsolutions++;
}
}
bool j29valid[8]={true,true,true,true,true,true,true,true};
_nj29 = 8;
for(int ij29 = 0; ij29 < numsolutions; ++ij29)
    {
if( !j29valid[ij29] )
{
    continue;
}
    j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
htj29 = IKtan(j29/2);

_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < numsolutions; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst104;
gconst104=IKsign(((((IkReal(11.0000000000000))*((sj29)*(sj29))))+(((IkReal(11.0000000000000))*((cj29)*(cj29))))));
dummyeval[0]=(((sj29)*(sj29))+((cj29)*(cj29)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x214=((IkReal(250.000000000000))*(cj29));
IkReal x215=((px)*(sj27));
IkReal x216=((cj27)*(py));
IkReal x217=((IkReal(250.000000000000))*(sj29));
IkReal x218=((IkReal(25.0000000000000))*(sj28));
IkReal x219=((cj28)*(pz));
IkReal x220=((cj27)*(px)*(sj28));
IkReal x221=((py)*(sj27)*(sj28));
if( IKabs(((gconst104)*(((((x214)*(x215)))+(((IkReal(-1.00000000000000))*(x217)*(x219)))+(((sj29)*(x218)))+(((IkReal(-1.00000000000000))*(x217)*(x220)))+(((IkReal(-1.00000000000000))*(x217)*(x221)))+(((IkReal(-1.00000000000000))*(x214)*(x216))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst104)*(((((IkReal(-1.00000000000000))*(cj29)*(x218)))+(((x214)*(x219)))+(((x214)*(x220)))+(((x214)*(x221)))+(((x215)*(x217)))+(((IkReal(-1.00000000000000))*(x216)*(x217))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j31array[0]=IKatan2(((gconst104)*(((((x214)*(x215)))+(((IkReal(-1.00000000000000))*(x217)*(x219)))+(((sj29)*(x218)))+(((IkReal(-1.00000000000000))*(x217)*(x220)))+(((IkReal(-1.00000000000000))*(x217)*(x221)))+(((IkReal(-1.00000000000000))*(x214)*(x216)))))), ((gconst104)*(((((IkReal(-1.00000000000000))*(cj29)*(x218)))+(((x214)*(x219)))+(((x214)*(x220)))+(((x214)*(x221)))+(((x215)*(x217)))+(((IkReal(-1.00000000000000))*(x216)*(x217)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x222=IKcos(j31);
IkReal x223=IKsin(j31);
IkReal x224=((IkReal(1.00000000000000))*(cj28));
IkReal x225=((IkReal(1.00000000000000))*(sj28));
IkReal x226=((IkReal(1.00000000000000))*(sj27));
IkReal x227=((IkReal(0.533617455371189))*(cj29));
IkReal x228=((IkReal(0.533617455371189))*(sj29));
IkReal x229=((IkReal(0.0440000000000000))*(x222));
IkReal x230=((IkReal(0.0440000000000000))*(x223));
evalcond[0]=((((cj27)*(py)))+(((sj29)*(x229)))+(((cj29)*(x230)))+(((IkReal(-1.00000000000000))*(px)*(x226))));
evalcond[1]=((((cj27)*(r01)))+(((x222)*(x228)))+(((x223)*(x227)))+(((IkReal(-1.00000000000000))*(r00)*(x226))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x225)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x225)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x224)))+(((cj29)*(x229)))+(((IkReal(-1.00000000000000))*(sj29)*(x230))));
evalcond[3]=((((x222)*(x227)))+(((IkReal(-1.00000000000000))*(r02)*(x224)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x225)))+(((IkReal(-1.00000000000000))*(x223)*(x228)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x225))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
IkReal x231=((IkReal(1.00000000000000))*(sj28));
IkReal x232=((cj27)*(cj28));
IkReal x233=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j30, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x231)))+(((px)*(x232)))+(((py)*(x233)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x231)))+(((r00)*(x232)))+(((r01)*(x233))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst105;
gconst105=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst106;
gconst106=IkReal(0.0880000000000000);
IkReal gconst107;
gconst107=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst108;
gconst108=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst109;
gconst109=IkReal(1.06723491074238);
IkReal gconst110;
gconst110=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst111;
gconst111=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst112;
gconst112=IkReal(0.0880000000000000);
IkReal gconst113;
gconst113=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst114;
gconst114=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst115;
gconst115=IkReal(1.06723491074238);
IkReal gconst116;
gconst116=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst117;
gconst117=IkReal(-0.0880000000000000);
IkReal gconst118;
gconst118=IkReal(0.0880000000000000);
IkReal gconst119;
gconst119=IkReal(-1.06723491074238);
IkReal gconst120;
gconst120=IkReal(1.06723491074238);
IkReal gconst121;
gconst121=IkReal(-0.0880000000000000);
IkReal gconst122;
gconst122=IkReal(0.0880000000000000);
IkReal gconst123;
gconst123=IkReal(-1.06723491074238);
IkReal gconst124;
gconst124=IkReal(1.06723491074238);
IkReal gconst125;
gconst125=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst126;
gconst126=IkReal(-0.0880000000000000);
IkReal gconst127;
gconst127=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst128;
gconst128=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst129;
gconst129=IkReal(-1.06723491074238);
IkReal gconst130;
gconst130=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst131;
gconst131=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst132;
gconst132=IkReal(-0.0880000000000000);
IkReal gconst133;
gconst133=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(sj27))));
IkReal gconst134;
gconst134=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal gconst135;
gconst135=IkReal(-1.06723491074238);
IkReal gconst136;
gconst136=((((IkReal(-1.00000000000000))*(r00)*(sj27)))+(((cj27)*(r01))));
IkReal x234=((gconst125)*(gconst130));
IkReal x235=((gconst131)*(gconst136));
IkReal x236=((gconst133)*(gconst134));
IkReal x237=((gconst132)*(gconst134));
IkReal x238=((gconst127)*(gconst129));
IkReal x239=((gconst127)*(gconst128));
IkReal x240=((gconst126)*(gconst130));
IkReal x241=((gconst131)*(gconst135));
dummyeval[0]=((((x234)*(x236)))+(((x237)*(x238)))+(((x240)*(x241)))+(((IkReal(-1.00000000000000))*(x237)*(x240)))+(((IkReal(-1.00000000000000))*(x238)*(x241)))+(((IkReal(-1.00000000000000))*(x234)*(x235)))+(((x235)*(x239)))+(((IkReal(-1.00000000000000))*(x236)*(x239))));
if( IKabs(dummyeval[0]) < 0.0000001000000000  )
{
continue;

} else
{
IkReal op[8+1], zeror[8];
int numroots;
IkReal x242=((gconst106)*(gconst110));
IkReal x243=((gconst115)*(gconst131));
IkReal x244=((gconst111)*(gconst116));
IkReal x245=((IkReal(1.00000000000000))*(gconst130));
IkReal x246=((IkReal(1.00000000000000))*(gconst127));
IkReal x247=((gconst133)*(gconst134));
IkReal x248=((gconst121)*(gconst136));
IkReal x249=((gconst118)*(gconst128));
IkReal x250=((gconst107)*(gconst119));
IkReal x251=((gconst124)*(gconst125));
IkReal x252=((gconst132)*(gconst134));
IkReal x253=((IkReal(1.00000000000000))*(gconst123));
IkReal x254=((gconst106)*(gconst120));
IkReal x255=((gconst127)*(gconst129));
IkReal x256=((gconst114)*(gconst132));
IkReal x257=((gconst108)*(gconst118));
IkReal x258=((gconst116)*(gconst121));
IkReal x259=((IkReal(1.00000000000000))*(gconst107));
IkReal x260=((gconst118)*(gconst129));
IkReal x261=((gconst114)*(gconst133));
IkReal x262=((gconst105)*(gconst110));
IkReal x263=((gconst111)*(gconst124));
IkReal x264=((gconst108)*(gconst127));
IkReal x265=((gconst108)*(gconst113));
IkReal x266=((gconst109)*(gconst118));
IkReal x267=((gconst112)*(gconst114));
IkReal x268=((gconst120)*(gconst125));
IkReal x269=((IkReal(1.00000000000000))*(gconst120));
IkReal x270=((gconst111)*(gconst115));
IkReal x271=((gconst115)*(gconst121));
IkReal x272=((gconst125)*(gconst130));
IkReal x273=((gconst114)*(gconst122));
IkReal x274=((gconst122)*(gconst134));
IkReal x275=((IkReal(1.00000000000000))*(gconst110));
IkReal x276=((gconst116)*(gconst131));
IkReal x277=((gconst129)*(gconst135));
IkReal x278=((gconst123)*(gconst133));
IkReal x279=((gconst122)*(gconst123));
IkReal x280=((gconst109)*(gconst127));
IkReal x281=((gconst131)*(gconst135));
IkReal x282=((gconst106)*(gconst130));
IkReal x283=((IkReal(1.00000000000000))*(gconst121));
IkReal x284=((gconst123)*(gconst132));
IkReal x285=((gconst107)*(gconst108));
IkReal x286=((gconst111)*(gconst136));
IkReal x287=((gconst113)*(gconst114));
IkReal x288=((gconst117)*(gconst120));
IkReal x289=((gconst113)*(gconst134));
IkReal x290=((gconst118)*(gconst119));
IkReal x291=((gconst111)*(gconst135));
IkReal x292=((gconst105)*(gconst130));
IkReal x293=((gconst105)*(gconst120));
IkReal x294=((gconst131)*(gconst136));
IkReal x295=((gconst110)*(gconst126));
IkReal x296=((gconst107)*(gconst109));
IkReal x297=((gconst112)*(gconst134));
IkReal x298=((gconst107)*(gconst129));
IkReal x299=((gconst112)*(gconst123));
IkReal x300=((gconst113)*(gconst123));
IkReal x301=((gconst120)*(gconst126));
IkReal x302=((gconst127)*(gconst128));
IkReal x303=((gconst117)*(gconst130));
IkReal x304=((gconst121)*(gconst124));
IkReal x305=((gconst124)*(gconst131));
IkReal x306=((gconst110)*(gconst117));
IkReal x307=((gconst107)*(gconst128));
IkReal x308=((gconst114)*(x265));
IkReal x309=((gconst131)*(x302));
IkReal x310=((gconst126)*(gconst130)*(gconst135));
IkReal x311=((gconst119)*(gconst127)*(gconst131));
IkReal x312=((gconst110)*(x287));
op[0]=((((gconst126)*(gconst130)*(x281)))+(((IkReal(-1.00000000000000))*(gconst126)*(x245)*(x252)))+(((IkReal(-1.00000000000000))*(gconst128)*(x246)*(x247)))+(((IkReal(-1.00000000000000))*(gconst131)*(x246)*(x277)))+(((x294)*(x302)))+(((IkReal(-1.00000000000000))*(gconst125)*(x245)*(x294)))+(((x252)*(x255)))+(((x247)*(x272))));
op[1]=((((x272)*(x274)))+(((x272)*(x278)))+(((IkReal(-1.00000000000000))*(gconst117)*(x245)*(x294)))+(((IkReal(-1.00000000000000))*(x247)*(x249)))+(((IkReal(-1.00000000000000))*(gconst131)*(x245)*(x251)))+(((IkReal(-1.00000000000000))*(gconst125)*(x245)*(x248)))+(((IkReal(-1.00000000000000))*(gconst128)*(x246)*(x278)))+(((IkReal(-1.00000000000000))*(gconst128)*(x246)*(x274)))+(((IkReal(-1.00000000000000))*(gconst126)*(x245)*(x284)))+(((IkReal(-1.00000000000000))*(x260)*(x281)))+(((x252)*(x260)))+(((IkReal(-1.00000000000000))*(gconst119)*(x246)*(x247)))+(((gconst119)*(gconst127)*(x294)))+(((x281)*(x301)))+(((x249)*(x294)))+(((IkReal(-1.00000000000000))*(x268)*(x294)))+(((x255)*(x284)))+(((x302)*(x305)))+(((x247)*(x268)))+(((gconst121)*(x310)))+(((IkReal(-1.00000000000000))*(gconst121)*(x246)*(x277)))+(((x248)*(x302)))+(((x247)*(x303)))+(((IkReal(-1.00000000000000))*(gconst126)*(x252)*(x269))));
op[2]=((((x272)*(x279)))+(((IkReal(-1.00000000000000))*(gconst128)*(x246)*(x261)))+(((IkReal(-1.00000000000000))*(gconst105)*(x245)*(x294)))+(((x255)*(x297)))+(((gconst110)*(gconst125)*(x247)))+(((x290)*(x294)))+(((IkReal(-1.00000000000000))*(gconst117)*(x245)*(x305)))+(((IkReal(-1.00000000000000))*(gconst125)*(x245)*(x276)))+(((IkReal(-1.00000000000000))*(gconst129)*(x243)*(x246)))+(((IkReal(-1.00000000000000))*(gconst126)*(x245)*(x256)))+(((IkReal(-1.00000000000000))*(gconst126)*(x252)*(x275)))+(((IkReal(-1.00000000000000))*(gconst109)*(x246)*(x281)))+(((IkReal(-1.00000000000000))*(gconst132)*(x253)*(x301)))+(((IkReal(-1.00000000000000))*(gconst128)*(x246)*(x279)))+(((IkReal(-1.00000000000000))*(gconst117)*(x269)*(x294)))+(((IkReal(-1.00000000000000))*(gconst131)*(x251)*(x269)))+(((IkReal(-1.00000000000000))*(gconst131)*(x259)*(x277)))+(((x252)*(x280)))+(((IkReal(-1.00000000000000))*(gconst128)*(x246)*(x289)))+(((IkReal(-1.00000000000000))*(gconst125)*(x245)*(x286)))+(((x248)*(x249)))+(((IkReal(-1.00000000000000))*(gconst106)*(x245)*(x252)))+(((x286)*(x302)))+(((IkReal(-1.00000000000000))*(gconst128)*(x247)*(x259)))+(((IkReal(-1.00000000000000))*(x248)*(x268)))+(((IkReal(-1.00000000000000))*(gconst121)*(x245)*(x251)))+(((x281)*(x295)))+(((IkReal(-1.00000000000000))*(gconst108)*(x246)*(x247)))+(((x278)*(x303)))+(((x274)*(x303)))+(((x247)*(x292)))+(((IkReal(-1.00000000000000))*(gconst119)*(x246)*(x274)))+(((IkReal(-1.00000000000000))*(gconst119)*(x246)*(x278)))+(((x294)*(x307)))+(((IkReal(-1.00000000000000))*(gconst117)*(x245)*(x248)))+(((gconst126)*(gconst130)*(x243)))+(((x272)*(x289)))+(((IkReal(-1.00000000000000))*(gconst135)*(x260)*(x283)))+(((IkReal(-1.00000000000000))*(gconst133)*(x249)*(x253)))+(((IkReal(-1.00000000000000))*(gconst125)*(x275)*(x294)))+(((gconst126)*(gconst130)*(x291)))+(((x252)*(x298)))+(((IkReal(-1.00000000000000))*(x247)*(x290)))+(((x302)*(x304)))+(((x260)*(x284)))+(((IkReal(-1.00000000000000))*(x249)*(x274)))+(((x249)*(x305)))+(((gconst121)*(gconst135)*(x301)))+(((IkReal(-1.00000000000000))*(gconst111)*(x246)*(x277)))+(((x255)*(x256)))+(((x264)*(x294)))+(((x261)*(x272)))+(((IkReal(-1.00000000000000))*(gconst126)*(x245)*(x297)))+(((x281)*(x282)))+(((x276)*(x302)))+(((x247)*(x288)))+(((gconst119)*(gconst127)*(x305)))+(((gconst119)*(gconst127)*(x248)))+(((x268)*(x278)))+(((x268)*(x274))));
op[3]=((((gconst111)*(gconst118)*(gconst128)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst119)*(gconst122)*(gconst123)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst119)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst122)*(gconst123)*(gconst128)))+(((gconst114)*(gconst118)*(gconst129)*(gconst132)))+(((gconst119)*(gconst121)*(gconst124)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst120)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst131)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst118)*(gconst129)*(gconst135)))+(((gconst107)*(gconst123)*(gconst129)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst120)*(gconst121)*(gconst136)))+(((gconst105)*(gconst120)*(gconst133)*(gconst134)))+(((gconst110)*(gconst121)*(gconst126)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst131)*(gconst136)))+(((gconst114)*(gconst122)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst123)*(gconst127)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst128)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst122)*(gconst127)*(gconst134)))+(((gconst108)*(gconst118)*(gconst131)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst123)*(gconst126)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst130)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst121)*(gconst130)*(gconst136)))+(((gconst106)*(gconst121)*(gconst130)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst123)*(gconst130)*(gconst132)))+(((gconst116)*(gconst121)*(gconst127)*(gconst128)))+(((gconst117)*(gconst120)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst128)*(gconst133)))+(((gconst113)*(gconst117)*(gconst130)*(gconst134)))+(((gconst106)*(gconst120)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst124)*(gconst130)*(gconst131)))+(((gconst116)*(gconst119)*(gconst127)*(gconst131)))+(((gconst108)*(gconst124)*(gconst127)*(gconst131)))+(((gconst114)*(gconst120)*(gconst125)*(gconst133)))+(((gconst105)*(gconst122)*(gconst130)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst121)*(gconst127)*(gconst135)))+(((gconst118)*(gconst119)*(gconst121)*(gconst136)))+(((gconst113)*(gconst123)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst122)*(gconst127)*(gconst128)))+(((gconst118)*(gconst119)*(gconst124)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst127)*(gconst134)))+(((gconst109)*(gconst118)*(gconst132)*(gconst134)))+(((gconst110)*(gconst122)*(gconst125)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst120)*(gconst125)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst120)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst130)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst121)*(gconst129)*(gconst135)))+(((gconst109)*(gconst123)*(gconst127)*(gconst132)))+(((gconst113)*(gconst120)*(gconst125)*(gconst134)))+(((gconst118)*(gconst121)*(gconst124)*(gconst128)))+(((gconst111)*(gconst124)*(gconst127)*(gconst128)))+(((gconst115)*(gconst121)*(gconst126)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst121)*(gconst127)*(gconst129)))+(((gconst110)*(gconst123)*(gconst125)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst118)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst123)*(gconst128)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst124)*(gconst125)*(gconst130)))+(((gconst117)*(gconst120)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst118)*(gconst129)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst122)*(gconst134)))+(((gconst114)*(gconst117)*(gconst130)*(gconst133)))+(((gconst112)*(gconst123)*(gconst127)*(gconst129)))+(((gconst112)*(gconst118)*(gconst129)*(gconst134)))+(((gconst107)*(gconst119)*(gconst131)*(gconst136)))+(((gconst107)*(gconst121)*(gconst128)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst120)*(gconst124)*(gconst131)))+(((gconst107)*(gconst124)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst121)*(gconst124)*(gconst130)))+(((gconst115)*(gconst120)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst124)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst121)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst126)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst123)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst125)*(gconst136)))+(((gconst108)*(gconst121)*(gconst127)*(gconst136)))+(((gconst110)*(gconst117)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst123)*(gconst133)))+(((gconst111)*(gconst120)*(gconst126)*(gconst135)))+(((gconst120)*(gconst122)*(gconst123)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst120)*(gconst121)*(gconst124)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst122)*(gconst128)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst133)*(gconst134)))+(((gconst105)*(gconst123)*(gconst130)*(gconst133)))+(((gconst117)*(gconst122)*(gconst123)*(gconst130)))+(((gconst111)*(gconst119)*(gconst127)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst123)*(gconst127)*(gconst128)))+(((gconst116)*(gconst118)*(gconst128)*(gconst131))));
op[4]=((((IkReal(-1.00000000000000))*(gconst109)*(gconst111)*(gconst127)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst112)*(gconst126)*(gconst134)))+(((gconst115)*(gconst120)*(gconst121)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst109)*(gconst131)*(gconst135)))+(((gconst110)*(gconst111)*(gconst126)*(gconst135)))+(((gconst111)*(gconst118)*(gconst124)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst112)*(gconst130)*(gconst134)))+(((gconst107)*(gconst121)*(gconst124)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst119)*(gconst123)*(gconst127)))+(((gconst114)*(gconst117)*(gconst122)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst120)*(gconst121)*(gconst125)))+(((gconst111)*(gconst116)*(gconst127)*(gconst128)))+(((gconst112)*(gconst114)*(gconst127)*(gconst129)))+(((gconst110)*(gconst122)*(gconst123)*(gconst125)))+(((gconst107)*(gconst112)*(gconst129)*(gconst134)))+(((gconst110)*(gconst113)*(gconst125)*(gconst134)))+(((gconst110)*(gconst117)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst123)*(gconst128)))+(((gconst114)*(gconst117)*(gconst120)*(gconst133)))+(((gconst108)*(gconst116)*(gconst127)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst127)*(gconst129)))+(((gconst110)*(gconst117)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst125)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst117)*(gconst120)*(gconst121)*(gconst124)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst111)*(gconst129)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst121)*(gconst136)))+(((gconst106)*(gconst110)*(gconst131)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst120)*(gconst131)))+(((gconst116)*(gconst118)*(gconst119)*(gconst131)))+(((gconst105)*(gconst113)*(gconst130)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst115)*(gconst118)*(gconst121)*(gconst129)))+(((gconst111)*(gconst115)*(gconst126)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst122)*(gconst123)*(gconst128)))+(((gconst107)*(gconst119)*(gconst124)*(gconst131)))+(((gconst113)*(gconst117)*(gconst123)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst120)*(gconst123)*(gconst132)))+(((gconst105)*(gconst120)*(gconst123)*(gconst133)))+(((gconst114)*(gconst120)*(gconst122)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst121)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst111)*(gconst125)*(gconst136)))+(((gconst118)*(gconst119)*(gconst121)*(gconst124)))+(((gconst105)*(gconst120)*(gconst122)*(gconst134)))+(((gconst107)*(gconst119)*(gconst121)*(gconst136)))+(((gconst107)*(gconst116)*(gconst128)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst118)*(gconst121)*(gconst135)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst121)*(gconst124)*(gconst130)))+(((gconst117)*(gconst120)*(gconst122)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst122)*(gconst123)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst114)*(gconst126)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst120)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst119)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst115)*(gconst129)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst120)*(gconst123)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst110)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst114)*(gconst130)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst122)*(gconst128)))+(((gconst108)*(gconst118)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst111)*(gconst130)*(gconst136)))+(((gconst111)*(gconst118)*(gconst119)*(gconst136)))+(((gconst112)*(gconst118)*(gconst123)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst115)*(gconst127)*(gconst131)))+(((gconst110)*(gconst115)*(gconst126)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst110)*(gconst131)*(gconst136)))+(((gconst107)*(gconst111)*(gconst128)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst114)*(gconst128)*(gconst133)))+(((gconst107)*(gconst114)*(gconst129)*(gconst132)))+(((gconst105)*(gconst122)*(gconst123)*(gconst130)))+(((gconst105)*(gconst110)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst114)*(gconst127)*(gconst133)))+(((gconst109)*(gconst112)*(gconst127)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst121)*(gconst124)*(gconst125)))+(((gconst116)*(gconst119)*(gconst121)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst116)*(gconst130)*(gconst131)))+(((gconst105)*(gconst114)*(gconst130)*(gconst133)))+(((gconst106)*(gconst111)*(gconst130)*(gconst135)))+(((gconst108)*(gconst121)*(gconst124)*(gconst127)))+(((gconst113)*(gconst117)*(gconst120)*(gconst134)))+(((gconst113)*(gconst120)*(gconst123)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst116)*(gconst125)*(gconst130)))+(((gconst106)*(gconst115)*(gconst130)*(gconst131)))+(((gconst116)*(gconst118)*(gconst121)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst119)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst124)*(gconst131)))+(((gconst107)*(gconst108)*(gconst131)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst118)*(gconst119)*(gconst122)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst123)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst108)*(gconst133)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst124)*(gconst131)))+(((gconst108)*(gconst118)*(gconst124)*(gconst131)))+(((gconst109)*(gconst118)*(gconst123)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst126)*(gconst130)))+(((gconst109)*(gconst114)*(gconst127)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst113)*(gconst127)*(gconst134)))+(((gconst113)*(gconst114)*(gconst125)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst113)*(gconst128)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst114)*(gconst127)*(gconst128)))+(((gconst110)*(gconst114)*(gconst125)*(gconst133)))+(((gconst111)*(gconst119)*(gconst124)*(gconst127)))+(((gconst108)*(gconst111)*(gconst127)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst124)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst120)*(gconst124)*(gconst125)))+(((gconst107)*(gconst109)*(gconst132)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst119)*(gconst122)*(gconst127)))+(((gconst106)*(gconst120)*(gconst121)*(gconst135))));
op[5]=((((gconst107)*(gconst111)*(gconst119)*(gconst136)))+(((gconst110)*(gconst115)*(gconst121)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst116)*(gconst121)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst114)*(gconst119)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst112)*(gconst120)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst116)*(gconst117)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst115)*(gconst121)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst115)*(gconst118)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst113)*(gconst119)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst116)*(gconst117)*(gconst120)*(gconst121)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst113)*(gconst123)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst110)*(gconst124)*(gconst131)))+(((gconst116)*(gconst118)*(gconst119)*(gconst121)))+(((gconst105)*(gconst110)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst111)*(gconst117)*(gconst136)))+(((gconst108)*(gconst116)*(gconst118)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst108)*(gconst122)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst114)*(gconst122)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst111)*(gconst118)*(gconst135)))+(((gconst105)*(gconst114)*(gconst122)*(gconst130)))+(((gconst105)*(gconst120)*(gconst122)*(gconst123)))+(((gconst107)*(gconst119)*(gconst121)*(gconst124)))+(((gconst105)*(gconst110)*(gconst123)*(gconst133)))+(((gconst108)*(gconst116)*(gconst121)*(gconst127)))+(((gconst110)*(gconst117)*(gconst122)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst113)*(gconst123)*(gconst127)))+(((gconst114)*(gconst117)*(gconst120)*(gconst122)))+(((gconst106)*(gconst115)*(gconst121)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst110)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst118)*(gconst122)*(gconst123)))+(((gconst111)*(gconst115)*(gconst120)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst121)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst117)*(gconst121)*(gconst124)))+(((gconst106)*(gconst110)*(gconst121)*(gconst135)))+(((gconst105)*(gconst113)*(gconst120)*(gconst134)))+(((gconst113)*(gconst114)*(gconst120)*(gconst125)))+(((gconst109)*(gconst114)*(gconst118)*(gconst132)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst116)*(gconst117)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst108)*(gconst123)*(gconst133)))+(((gconst109)*(gconst112)*(gconst118)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst112)*(gconst123)*(gconst126)))+(((gconst105)*(gconst114)*(gconst120)*(gconst133)))+(((gconst107)*(gconst111)*(gconst124)*(gconst128)))+(((gconst110)*(gconst113)*(gconst117)*(gconst134)))+(((gconst110)*(gconst114)*(gconst122)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst112)*(gconst123)*(gconst130)))+(((gconst108)*(gconst118)*(gconst121)*(gconst124)))+(((gconst109)*(gconst112)*(gconst123)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst114)*(gconst122)*(gconst128)))+(((gconst107)*(gconst108)*(gconst121)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst110)*(gconst111)*(gconst124)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst114)*(gconst118)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst112)*(gconst114)*(gconst120)*(gconst126)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst117)*(gconst120)*(gconst124)))+(((gconst106)*(gconst111)*(gconst120)*(gconst135)))+(((gconst110)*(gconst114)*(gconst117)*(gconst133)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst118)*(gconst119)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst120)*(gconst121)*(gconst124)))+(((gconst106)*(gconst115)*(gconst120)*(gconst131)))+(((gconst105)*(gconst113)*(gconst123)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst109)*(gconst121)*(gconst135)))+(((gconst107)*(gconst108)*(gconst124)*(gconst131)))+(((gconst111)*(gconst118)*(gconst119)*(gconst124)))+(((gconst111)*(gconst116)*(gconst119)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst113)*(gconst118)*(gconst134)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst111)*(gconst120)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst110)*(gconst123)*(gconst132)))+(((gconst108)*(gconst111)*(gconst124)*(gconst127)))+(((gconst112)*(gconst114)*(gconst118)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst116)*(gconst120)*(gconst131)))+(((gconst107)*(gconst112)*(gconst123)*(gconst129)))+(((gconst108)*(gconst111)*(gconst118)*(gconst136)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst115)*(gconst121)*(gconst129)))+(((IkReal(-1.00000000000000))*(gconst109)*(gconst115)*(gconst118)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst114)*(gconst118)*(gconst119)*(gconst122)))+(((gconst110)*(gconst113)*(gconst123)*(gconst125)))+(((IkReal(-1.00000000000000))*(gconst113)*(gconst114)*(gconst119)*(gconst127)))+(((IkReal(-1.00000000000000))*(gconst111)*(gconst116)*(gconst120)*(gconst125)))+(((gconst107)*(gconst109)*(gconst123)*(gconst132)))+(((gconst107)*(gconst116)*(gconst119)*(gconst131)))+(((IkReal(-1.00000000000000))*(gconst108)*(gconst114)*(gconst118)*(gconst133)))+(((gconst107)*(gconst116)*(gconst121)*(gconst128)))+(((gconst113)*(gconst117)*(gconst120)*(gconst123)))+(((IkReal(-1.00000000000000))*(gconst106)*(gconst114)*(gconst120)*(gconst132)))+(((gconst113)*(gconst114)*(gconst117)*(gconst130)))+(((gconst111)*(gconst116)*(gconst118)*(gconst128)))+(((IkReal(-1.00000000000000))*(gconst105)*(gconst111)*(gconst124)*(gconst130)))+(((IkReal(-1.00000000000000))*(gconst107)*(gconst119)*(gconst122)*(gconst123))));
op[6]=((((IkReal(-1.00000000000000))*(gconst117)*(x244)*(x269)))+(((IkReal(-1.00000000000000))*(gconst117)*(x263)*(x275)))+(((IkReal(-1.00000000000000))*(gconst129)*(x259)*(x270)))+(((IkReal(-1.00000000000000))*(x257)*(x273)))+(((x273)*(x293)))+(((x285)*(x286)))+(((x285)*(x304)))+(((IkReal(-1.00000000000000))*(gconst105)*(x263)*(x269)))+(((IkReal(-1.00000000000000))*(gconst122)*(x253)*(x285)))+(((x256)*(x296)))+(((IkReal(-1.00000000000000))*(gconst109)*(x259)*(x291)))+(((x266)*(x299)))+(((IkReal(-1.00000000000000))*(gconst113)*(x250)*(x253)))+(((IkReal(-1.00000000000000))*(gconst134)*(x259)*(x265)))+(((IkReal(-1.00000000000000))*(gconst113)*(x253)*(x257)))+(((IkReal(-1.00000000000000))*(gconst105)*(x244)*(x245)))+(((x267)*(x298)))+(((IkReal(-1.00000000000000))*(x262)*(x286)))+(((x244)*(x264)))+(((x257)*(x258)))+(((x270)*(x295)))+(((x287)*(x292)))+(((IkReal(-1.00000000000000))*(gconst108)*(x259)*(x261)))+(((x262)*(x279)))+(((IkReal(-1.00000000000000))*(x246)*(x308)))+(((x250)*(x258)))+(((IkReal(-1.00000000000000))*(gconst106)*(x245)*(x267)))+(((x244)*(x307)))+(((IkReal(-1.00000000000000))*(x242)*(x256)))+(((x293)*(x300)))+(((IkReal(-1.00000000000000))*(x250)*(x273)))+(((IkReal(-1.00000000000000))*(gconst124)*(x262)*(x283)))+(((IkReal(-1.00000000000000))*(gconst109)*(x243)*(x259)))+(((x262)*(x289)))+(((IkReal(-1.00000000000000))*(x262)*(x276)))+(((x254)*(x271)))+(((gconst125)*(x312)))+(((IkReal(-1.00000000000000))*(gconst112)*(x253)*(x254)))+(((x242)*(x243)))+(((x296)*(x297)))+(((x257)*(x263)))+(((x261)*(x262)))+(((x250)*(x263)))+(((IkReal(-1.00000000000000))*(gconst105)*(x258)*(x269)))+(((IkReal(-1.00000000000000))*(gconst117)*(x258)*(x275)))+(((IkReal(-1.00000000000000))*(x287)*(x290)))+(((x300)*(x306)))+(((IkReal(-1.00000000000000))*(gconst109)*(x246)*(x270)))+(((IkReal(-1.00000000000000))*(x266)*(x271)))+(((x242)*(x291)))+(((IkReal(-1.00000000000000))*(x242)*(x297)))+(((x287)*(x288)))+(((x244)*(x290)))+(((x276)*(x285)))+(((IkReal(-1.00000000000000))*(gconst126)*(x267)*(x275)))+(((IkReal(-1.00000000000000))*(gconst125)*(x244)*(x275)))+(((IkReal(-1.00000000000000))*(gconst128)*(x259)*(x287)))+(((x267)*(x280)))+(((x270)*(x282)))+(((x273)*(x306))));
op[7]=((((x266)*(x267)))+(((IkReal(-1.00000000000000))*(x258)*(x262)))+(((IkReal(-1.00000000000000))*(gconst112)*(x242)*(x253)))+(((x258)*(x285)))+(((x287)*(x306)))+(((x262)*(x300)))+(((IkReal(-1.00000000000000))*(x250)*(x287)))+(((IkReal(-1.00000000000000))*(gconst105)*(x244)*(x269)))+(((x287)*(x293)))+(((IkReal(-1.00000000000000))*(x257)*(x287)))+(((x262)*(x273)))+(((IkReal(-1.00000000000000))*(gconst108)*(x259)*(x273)))+(((IkReal(-1.00000000000000))*(x262)*(x263)))+(((x263)*(x285)))+(((x254)*(x270)))+(((x242)*(x271)))+(((x296)*(x299)))+(((IkReal(-1.00000000000000))*(gconst109)*(x259)*(x271)))+(((IkReal(-1.00000000000000))*(gconst107)*(x253)*(x265)))+(((IkReal(-1.00000000000000))*(x254)*(x267)))+(((IkReal(-1.00000000000000))*(x266)*(x270)))+(((IkReal(-1.00000000000000))*(gconst117)*(x244)*(x275)))+(((x244)*(x250)))+(((x244)*(x257))));
op[8]=((((IkReal(-1.00000000000000))*(x244)*(x262)))+(((x244)*(x285)))+(((x267)*(x296)))+(((x262)*(x287)))+(((x242)*(x270)))+(((IkReal(-1.00000000000000))*(gconst109)*(x259)*(x270)))+(((IkReal(-1.00000000000000))*(x259)*(x308)))+(((IkReal(-1.00000000000000))*(x242)*(x267))));
polyroots8(op,zeror,numroots);
IkReal j29array[8], cj29array[8], sj29array[8], tempj29array[1];
int numsolutions = 0;
for(int ij29 = 0; ij29 < numroots; ++ij29)
{
IkReal htj29 = zeror[ij29];
tempj29array[0]=((IkReal(2.00000000000000))*(atan(htj29)));
for(int kj29 = 0; kj29 < 1; ++kj29)
{
j29array[numsolutions] = tempj29array[kj29];
if( j29array[numsolutions] > IKPI )
{
    j29array[numsolutions]-=IK2PI;
}
else if( j29array[numsolutions] < -IKPI )
{
    j29array[numsolutions]+=IK2PI;
}
sj29array[numsolutions] = IKsin(j29array[numsolutions]);
cj29array[numsolutions] = IKcos(j29array[numsolutions]);
numsolutions++;
}
}
bool j29valid[8]={true,true,true,true,true,true,true,true};
_nj29 = 8;
for(int ij29 = 0; ij29 < numsolutions; ++ij29)
    {
if( !j29valid[ij29] )
{
    continue;
}
    j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
htj29 = IKtan(j29/2);

_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < numsolutions; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
{
IkReal dummyeval[1];
IkReal gconst153;
gconst153=IKsign(((((IkReal(11.0000000000000))*((sj29)*(sj29))))+(((IkReal(11.0000000000000))*((cj29)*(cj29))))));
dummyeval[0]=(((sj29)*(sj29))+((cj29)*(cj29)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x313=((IkReal(25.0000000000000))*(sj28));
IkReal x314=((IkReal(250.000000000000))*(cj27));
IkReal x315=((cj29)*(py));
IkReal x316=((px)*(sj28));
IkReal x317=((IkReal(250.000000000000))*(sj27));
IkReal x318=((py)*(sj29));
IkReal x319=((IkReal(250.000000000000))*(cj28)*(pz));
if( IKabs(((gconst153)*(((((IkReal(-1.00000000000000))*(sj28)*(x317)*(x318)))+(((IkReal(-1.00000000000000))*(sj29)*(x314)*(x316)))+(((sj29)*(x313)))+(((IkReal(-1.00000000000000))*(sj29)*(x319)))+(((IkReal(-1.00000000000000))*(x314)*(x315)))+(((cj29)*(px)*(x317))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst153)*(((((IkReal(-1.00000000000000))*(cj29)*(x314)*(x316)))+(((IkReal(-1.00000000000000))*(cj29)*(x319)))+(((cj29)*(x313)))+(((IkReal(-1.00000000000000))*(px)*(sj29)*(x317)))+(((IkReal(-1.00000000000000))*(sj28)*(x315)*(x317)))+(((x314)*(x318))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j31array[0]=IKatan2(((gconst153)*(((((IkReal(-1.00000000000000))*(sj28)*(x317)*(x318)))+(((IkReal(-1.00000000000000))*(sj29)*(x314)*(x316)))+(((sj29)*(x313)))+(((IkReal(-1.00000000000000))*(sj29)*(x319)))+(((IkReal(-1.00000000000000))*(x314)*(x315)))+(((cj29)*(px)*(x317)))))), ((gconst153)*(((((IkReal(-1.00000000000000))*(cj29)*(x314)*(x316)))+(((IkReal(-1.00000000000000))*(cj29)*(x319)))+(((cj29)*(x313)))+(((IkReal(-1.00000000000000))*(px)*(sj29)*(x317)))+(((IkReal(-1.00000000000000))*(sj28)*(x315)*(x317)))+(((x314)*(x318)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x320=IKsin(j31);
IkReal x321=IKcos(j31);
IkReal x322=((IkReal(1.00000000000000))*(cj28));
IkReal x323=((IkReal(1.00000000000000))*(sj28));
IkReal x324=((IkReal(1.00000000000000))*(sj27));
IkReal x325=((IkReal(0.533617455371189))*(cj29));
IkReal x326=((IkReal(0.533617455371189))*(sj29));
IkReal x327=((IkReal(0.0440000000000000))*(x321));
IkReal x328=((IkReal(0.0440000000000000))*(x320));
evalcond[0]=((((cj27)*(py)))+(((cj29)*(x328)))+(((IkReal(-1.00000000000000))*(px)*(x324)))+(((IkReal(-1.00000000000000))*(sj29)*(x327))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(x321)*(x326)))+(((x320)*(x325)))+(((IkReal(-1.00000000000000))*(r00)*(x324))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x323)))+(((IkReal(-1.00000000000000))*(cj29)*(x327)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x323)))+(((IkReal(-1.00000000000000))*(pz)*(x322)))+(((IkReal(-1.00000000000000))*(sj29)*(x328))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x321)*(x325)))+(((IkReal(-1.00000000000000))*(x320)*(x326)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x323)))+(((IkReal(-1.00000000000000))*(r02)*(x322)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x323))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
    }

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j31array[2], cj31array[2], sj31array[2];
bool j31valid[2]={false};
_nj31 = 2;
cj31array[0]=((IkReal(1.87400166530233))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((((cj27)*(cj28)*(r00)))+(((cj28)*(r01)*(sj27)))+(((IkReal(-1.00000000000000))*(r02)*(sj28)))+(((IkReal(-0.845725967038483))*(cj30))))));
if( cj31array[0] >= -1-IKFAST_SINCOS_THRESH && cj31array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j31valid[0] = j31valid[1] = true;
    j31array[0] = IKacos(cj31array[0]);
    sj31array[0] = IKsin(j31array[0]);
    cj31array[1] = cj31array[0];
    j31array[1] = -j31array[0];
    sj31array[1] = -sj31array[0];
}
else if( isnan(cj31array[0]) )
{
    // probably any value will work
    j31valid[0] = true;
    cj31array[0] = 1; sj31array[0] = 0; j31array[0] = 0;
}
for(int ij31 = 0; ij31 < 2; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 2; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[1];
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(-0.0440000000000000))*(sj30)*(IKcos(j31))))+(((cj28)*(py)*(sj27)))+(((IkReal(-0.135000000000000))*(cj30)))+(((cj27)*(cj28)*(px)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-1.00000000000000))*(pz)*(sj28))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst49;
gconst49=IKsign(((((IkReal(1139.06250000000))*((sj30)*(sj30))))+(((IkReal(121.000000000000))*((cj30)*(cj30))*((cj31)*(cj31))))+(((IkReal(-742.500000000000))*(cj30)*(cj31)*(sj30)))+(((IkReal(121.000000000000))*((sj31)*(sj31))))));
dummyeval[0]=((((IkReal(9.41373966942149))*((sj30)*(sj30))))+((((cj30)*(cj30))*((cj31)*(cj31))))+((sj31)*(sj31))+(((IkReal(-6.13636363636364))*(cj30)*(cj31)*(sj30))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[3];
dummyeval[0]=((((cj30)*(cj31)*(sj30)))+(((IkReal(-2.32653684440068))*((sj30)*(sj30)))));
dummyeval[1]=sj30;
dummyeval[2]=sj31;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  || IKabs(dummyeval[2]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x329=((IkReal(1.00000000000000))*(sj28));
IkReal x330=((cj27)*(cj28));
IkReal x331=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((px)*(x330)))+(((py)*(x331)))+(((IkReal(-1.00000000000000))*(pz)*(x329)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(-0.845725967038483))+(((r01)*(x331)))+(((r00)*(x330)))+(((IkReal(-1.00000000000000))*(r02)*(x329))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst50;
gconst50=IKsign(((((IkReal(11.0000000000000))*((sj31)*(sj31))))+(((IkReal(11.0000000000000))*((cj31)*(cj31))))));
dummyeval[0]=(((sj31)*(sj31))+((cj31)*(cj31)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x332=((py)*(sj28));
IkReal x333=((IkReal(250.000000000000))*(sj31));
IkReal x334=((cj27)*(py));
IkReal x335=((cj31)*(sj28));
IkReal x336=((cj27)*(px));
IkReal x337=((IkReal(250.000000000000))*(cj31));
IkReal x338=((cj28)*(pz));
IkReal x339=((sj27)*(x337));
if( IKabs(((gconst50)*(((((px)*(x339)))+(((IkReal(-1.00000000000000))*(x333)*(x338)))+(((IkReal(-1.00000000000000))*(sj28)*(x333)*(x336)))+(((IkReal(25.0000000000000))*(sj28)*(sj31)))+(((IkReal(-1.00000000000000))*(sj27)*(x332)*(x333)))+(((IkReal(-1.00000000000000))*(x334)*(x337))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst50)*(((((px)*(sj27)*(x333)))+(((x337)*(x338)))+(((IkReal(-1.00000000000000))*(x333)*(x334)))+(((x332)*(x339)))+(((IkReal(-25.0000000000000))*(x335)))+(((IkReal(250.000000000000))*(x335)*(x336))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst50)*(((((px)*(x339)))+(((IkReal(-1.00000000000000))*(x333)*(x338)))+(((IkReal(-1.00000000000000))*(sj28)*(x333)*(x336)))+(((IkReal(25.0000000000000))*(sj28)*(sj31)))+(((IkReal(-1.00000000000000))*(sj27)*(x332)*(x333)))+(((IkReal(-1.00000000000000))*(x334)*(x337)))))), ((gconst50)*(((((px)*(sj27)*(x333)))+(((x337)*(x338)))+(((IkReal(-1.00000000000000))*(x333)*(x334)))+(((x332)*(x339)))+(((IkReal(-25.0000000000000))*(x335)))+(((IkReal(250.000000000000))*(x335)*(x336)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x340=IKcos(j29);
IkReal x341=IKsin(j29);
IkReal x342=((IkReal(1.00000000000000))*(cj28));
IkReal x343=((IkReal(0.533617455371189))*(cj31));
IkReal x344=((IkReal(1.00000000000000))*(sj27));
IkReal x345=((IkReal(0.0440000000000000))*(x341));
IkReal x346=((IkReal(1.00000000000000))*(cj27)*(sj28));
IkReal x347=((sj31)*(x340));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x344)))+(((IkReal(0.0440000000000000))*(x347)))+(((cj31)*(x345))));
evalcond[1]=((((cj27)*(r01)))+(((x341)*(x343)))+(((IkReal(-1.00000000000000))*(r00)*(x344)))+(((IkReal(0.533617455371189))*(x347))));
evalcond[2]=((((IkReal(0.0440000000000000))*(cj31)*(x340)))+(((IkReal(-1.00000000000000))*(px)*(x346)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x344)))+(((IkReal(-1.00000000000000))*(sj31)*(x345)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x342))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x342)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x344)))+(((IkReal(-0.533617455371189))*(sj31)*(x341)))+(((IkReal(-1.00000000000000))*(r00)*(x346)))+(((x340)*(x343))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x348=((IkReal(1.00000000000000))*(sj28));
IkReal x349=((cj27)*(cj28));
IkReal x350=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j30, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((py)*(x350)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-1.00000000000000))*(pz)*(x348)))+(((px)*(x349))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x348)))+(((r00)*(x349)))+(((r01)*(x350))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst51;
gconst51=IKsign(((((IkReal(11.0000000000000))*((sj31)*(sj31))))+(((IkReal(11.0000000000000))*((cj31)*(cj31))))));
dummyeval[0]=(((sj31)*(sj31))+((cj31)*(cj31)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x351=((py)*(sj28));
IkReal x352=((cj31)*(sj28));
IkReal x353=((IkReal(250.000000000000))*(cj31));
IkReal x354=((cj28)*(pz));
IkReal x355=((sj28)*(sj31));
IkReal x356=((cj27)*(py));
IkReal x357=((IkReal(250.000000000000))*(sj31));
IkReal x358=((sj27)*(x353));
IkReal x359=((sj27)*(x357));
IkReal x360=((IkReal(250.000000000000))*(cj27)*(px));
if( IKabs(((gconst51)*(((((IkReal(25.0000000000000))*(x355)))+(((IkReal(-1.00000000000000))*(px)*(x358)))+(((IkReal(-1.00000000000000))*(x355)*(x360)))+(((x353)*(x356)))+(((IkReal(-1.00000000000000))*(x351)*(x359)))+(((IkReal(-1.00000000000000))*(x354)*(x357))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst51)*(((((px)*(x359)))+(((IkReal(25.0000000000000))*(x352)))+(((IkReal(-1.00000000000000))*(x352)*(x360)))+(((IkReal(-1.00000000000000))*(x356)*(x357)))+(((IkReal(-1.00000000000000))*(x353)*(x354)))+(((IkReal(-1.00000000000000))*(x351)*(x358))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst51)*(((((IkReal(25.0000000000000))*(x355)))+(((IkReal(-1.00000000000000))*(px)*(x358)))+(((IkReal(-1.00000000000000))*(x355)*(x360)))+(((x353)*(x356)))+(((IkReal(-1.00000000000000))*(x351)*(x359)))+(((IkReal(-1.00000000000000))*(x354)*(x357)))))), ((gconst51)*(((((px)*(x359)))+(((IkReal(25.0000000000000))*(x352)))+(((IkReal(-1.00000000000000))*(x352)*(x360)))+(((IkReal(-1.00000000000000))*(x356)*(x357)))+(((IkReal(-1.00000000000000))*(x353)*(x354)))+(((IkReal(-1.00000000000000))*(x351)*(x358)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x361=IKcos(j29);
IkReal x362=IKsin(j29);
IkReal x363=((IkReal(1.00000000000000))*(cj28));
IkReal x364=((IkReal(0.533617455371189))*(cj31));
IkReal x365=((IkReal(1.00000000000000))*(sj27));
IkReal x366=((IkReal(0.0440000000000000))*(x362));
IkReal x367=((IkReal(1.00000000000000))*(cj27)*(sj28));
IkReal x368=((sj31)*(x361));
evalcond[0]=((((cj27)*(py)))+(((IkReal(0.0440000000000000))*(x368)))+(((IkReal(-1.00000000000000))*(cj31)*(x366)))+(((IkReal(-1.00000000000000))*(px)*(x365))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(x362)*(x364)))+(((IkReal(-1.00000000000000))*(r00)*(x365)))+(((IkReal(0.533617455371189))*(x368))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(cj31)*(x361)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(sj31)*(x366)))+(((IkReal(-1.00000000000000))*(pz)*(x363)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x365)))+(((IkReal(-1.00000000000000))*(px)*(x367))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x361)*(x364)))+(((IkReal(-1.00000000000000))*(r02)*(x363)))+(((IkReal(-1.00000000000000))*(r00)*(x367)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x365)))+(((IkReal(-0.533617455371189))*(sj31)*(x362))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x369=((IkReal(1.00000000000000))*(sj28));
IkReal x370=((cj27)*(cj28));
IkReal x371=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j31)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((px)*(x370)))+(((IkReal(-0.135000000000000))*(cj30)))+(((py)*(x371)))+(((IkReal(-1.00000000000000))*(pz)*(x369)))+(((IkReal(-0.0440000000000000))*(sj30)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x369)))+(((IkReal(-0.533617455371189))*(sj30)))+(((r00)*(x370)))+(((r01)*(x371)))+(((IkReal(-0.845725967038483))*(cj30))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst52;
gconst52=IKsign(((((IkReal(-0.135000000000000))*(sj30)))+(((IkReal(0.0440000000000000))*(cj30)))));
dummyeval[0]=((((IkReal(-3.06818181818182))*(sj30)))+(cj30));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x372=((IkReal(1.00000000000000))*(sj28));
IkReal x373=((cj27)*(cj28));
IkReal x374=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(5.96811616783542))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((px)*(x373)))+(((py)*(x374)))+(((IkReal(-1.00000000000000))*(pz)*(x372)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(0.969453625832208))+(((r00)*(x373)))+(((r01)*(x374)))+(((IkReal(-1.00000000000000))*(r02)*(x372))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x375=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(py)*(sj27)*(x375)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x375))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))))+IKsqr(((((IkReal(-1.00000000000000))*(py)*(sj27)*(x375)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x375)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))), ((((IkReal(-1.00000000000000))*(py)*(sj27)*(x375)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x375)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x376=IKsin(j29);
IkReal x377=IKcos(j29);
IkReal x378=((IkReal(1.00000000000000))*(cj28));
IkReal x379=((IkReal(1.00000000000000))*(sj28));
IkReal x380=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x380)))+(((IkReal(-4.71405189209358e-11))*(x376))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-0.245274683444403))*(x376)))+(((IkReal(-1.00000000000000))*(r00)*(x380))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x379)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-4.71405189209358e-11))*(x377)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x379)))+(((IkReal(-1.00000000000000))*(pz)*(x378))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x379)))+(((IkReal(-1.00000000000000))*(r02)*(x378)))+(((IkReal(-0.245274683444403))*(x377)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x379))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x381=((IkReal(1.00000000000000))*(sj28));
IkReal x382=((cj27)*(cj28));
IkReal x383=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(2.82652351424562))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((px)*(x382)))+(((IkReal(-0.100000000000000))*(cj28)))+(((py)*(x383)))+(((IkReal(-1.00000000000000))*(pz)*(x381))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x381)))+(((r00)*(x382)))+(((r01)*(x383))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x384=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x384)))+(((cj27)*(px)*(x384))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))))+IKsqr(((((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x384)))+(((cj27)*(px)*(x384)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))), ((((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x384)))+(((cj27)*(px)*(x384)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x385=IKsin(j29);
IkReal x386=IKcos(j29);
IkReal x387=((IkReal(1.00000000000000))*(cj28));
IkReal x388=((IkReal(1.00000000000000))*(sj28));
IkReal x389=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x389)))+(((IkReal(4.71405189209358e-11))*(x385))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(0.245274683444403))*(x385)))+(((IkReal(-1.00000000000000))*(r00)*(x389))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj27)*(px)*(x388)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x388)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(4.71405189209358e-11))*(x386)))+(((IkReal(-1.00000000000000))*(pz)*(x387))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj27)*(x388)))+(((IkReal(0.245274683444403))*(x386)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x388)))+(((IkReal(-1.00000000000000))*(r02)*(x387))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
if( IKabs(((gconst52)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst52)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27)))))), ((gconst52)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x390=IKsin(j29);
IkReal x391=IKcos(j29);
IkReal x392=((IkReal(1.00000000000000))*(cj28));
IkReal x393=((IkReal(1.00000000000000))*(sj28));
IkReal x394=((IkReal(1.00000000000000))*(sj27));
IkReal x395=((IkReal(0.135000000000000))*(sj30));
IkReal x396=((IkReal(0.845725967038483))*(sj30));
IkReal x397=((cj30)*(x391));
IkReal x398=((cj30)*(x390));
evalcond[0]=((((cj27)*(py)))+(((IkReal(0.0440000000000000))*(x398)))+(((IkReal(-1.00000000000000))*(x390)*(x395)))+(((IkReal(-1.00000000000000))*(px)*(x394))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x394)))+(((IkReal(0.533617455371189))*(x398)))+(((IkReal(-1.00000000000000))*(x390)*(x396))));
evalcond[2]=((((IkReal(0.0440000000000000))*(x397)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x391)*(x395)))+(((IkReal(-1.00000000000000))*(pz)*(x392)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x393)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x393))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x393)))+(((IkReal(-1.00000000000000))*(r02)*(x392)))+(((IkReal(0.533617455371189))*(x397)))+(((IkReal(-1.00000000000000))*(x391)*(x396)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x393))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x399=((IkReal(1.00000000000000))*(sj28));
IkReal x400=((cj27)*(cj28));
IkReal x401=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j31, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((py)*(x401)))+(((IkReal(0.0440000000000000))*(sj30)))+(((px)*(x400)))+(((IkReal(-1.00000000000000))*(pz)*(x399)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x399)))+(((r00)*(x400)))+(((r01)*(x401)))+(((IkReal(0.533617455371189))*(sj30)))+(((IkReal(-0.845725967038483))*(cj30))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(((((IkReal(-0.0440000000000000))*(cj30)))+(((IkReal(-0.135000000000000))*(sj30)))));
dummyeval[0]=((((IkReal(-3.06818181818182))*(sj30)))+(((IkReal(-1.00000000000000))*(cj30))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x402=((IkReal(1.00000000000000))*(sj28));
IkReal x403=((cj27)*(cj28));
IkReal x404=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(0.315069139344170))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((py)*(x404)))+(((px)*(x403)))+(((IkReal(-1.00000000000000))*(pz)*(x402)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x402)))+(((r00)*(x403)))+(((r01)*(x404))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x405=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((cj27)*(px)*(x405)))+(((py)*(sj27)*(x405))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))))+IKsqr(((((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((cj27)*(px)*(x405)))+(((py)*(sj27)*(x405)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))), ((((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((cj27)*(px)*(x405)))+(((py)*(sj27)*(x405)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x406=IKsin(j29);
IkReal x407=IKcos(j29);
IkReal x408=((IkReal(1.00000000000000))*(cj28));
IkReal x409=((IkReal(1.00000000000000))*(sj28));
IkReal x410=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x410)))+(((IkReal(4.71405189209358e-11))*(x406))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(0.245274683444403))*(x406)))+(((IkReal(-1.00000000000000))*(r00)*(x410))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x409)))+(((IkReal(4.71405189209358e-11))*(x407)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x408)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x409))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x409)))+(((IkReal(0.245274683444403))*(x407)))+(((IkReal(-1.00000000000000))*(r02)*(x408)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x409))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x411=((IkReal(1.00000000000000))*(sj28));
IkReal x412=((cj27)*(cj28));
IkReal x413=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-2.82652351424562))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((py)*(x413)))+(((px)*(x412)))+(((IkReal(-1.00000000000000))*(pz)*(x411)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x411)))+(((r01)*(x413)))+(((r00)*(x412))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x414=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(py)*(sj27)*(x414)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x414))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))))+IKsqr(((((IkReal(-1.00000000000000))*(py)*(sj27)*(x414)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x414)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))), ((((IkReal(-1.00000000000000))*(py)*(sj27)*(x414)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x414)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x415=IKsin(j29);
IkReal x416=IKcos(j29);
IkReal x417=((IkReal(1.00000000000000))*(cj28));
IkReal x418=((IkReal(1.00000000000000))*(sj28));
IkReal x419=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x419)))+(((IkReal(-4.71405189209358e-11))*(x415))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-0.245274683444403))*(x415)))+(((IkReal(-1.00000000000000))*(r00)*(x419))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x418)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x418)))+(((IkReal(-4.71405189209358e-11))*(x416)))+(((IkReal(-1.00000000000000))*(pz)*(x417))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x418)))+(((IkReal(-1.00000000000000))*(r02)*(x417)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x418)))+(((IkReal(-0.245274683444403))*(x416))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
if( IKabs(((gconst53)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst53)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27)))))), ((gconst53)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x420=IKsin(j29);
IkReal x421=IKcos(j29);
IkReal x422=((IkReal(1.00000000000000))*(cj28));
IkReal x423=((IkReal(1.00000000000000))*(sj28));
IkReal x424=((IkReal(1.00000000000000))*(sj27));
IkReal x425=((IkReal(0.135000000000000))*(sj30));
IkReal x426=((IkReal(0.845725967038483))*(sj30));
IkReal x427=((cj30)*(x421));
IkReal x428=((cj30)*(x420));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(x420)*(x425)))+(((IkReal(-1.00000000000000))*(px)*(x424)))+(((IkReal(-0.0440000000000000))*(x428))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(x420)*(x426)))+(((IkReal(-0.533617455371189))*(x428)))+(((IkReal(-1.00000000000000))*(r00)*(x424))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x421)*(x425)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x423)))+(((IkReal(-0.0440000000000000))*(x427)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x422)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x423))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x421)*(x426)))+(((IkReal(-0.533617455371189))*(x427)))+(((IkReal(-1.00000000000000))*(r02)*(x422)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x423)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x423))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x429=((cj30)*(cj31));
IkReal x430=((cj27)*(r01));
IkReal x431=((px)*(sj27));
IkReal x432=((IkReal(3.64368616112160e+23))*(sj30));
IkReal x433=((cj27)*(py));
IkReal x434=((r00)*(sj27));
IkReal x435=((IkReal(3.00444053086361e+22))*(sj30));
IkReal x436=((IkReal(337503218838.750))*(sj30));
IkReal x437=((IkReal(2114335082451.87))*(sj30));
if( IKabs(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj30)*(x429)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj30)*(x429)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x432)*(x433)))+(((IkReal(-1.56614161081993e+23))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x434)*(x435)))+(((IkReal(1.56614161081993e+23))*(x429)*(x433)))+(((x430)*(x435)))+(((IkReal(-1.29137887418136e+22))*(x429)*(x430)))+(((x431)*(x432)))+(((IkReal(1.29137887418136e+22))*(x429)*(x434))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.14854255730435e-11))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IKabs(sj31) != 0)?((IkReal)1/(sj31)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x433)*(x437)))+(((IkReal(-1334056361602.36))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x434)*(x436)))+(((IkReal(1334056361602.36))*(x429)*(x433)))+(((IkReal(110001049103.000))*(x429)*(x434)))+(((IkReal(-110001049103.000))*(x429)*(x430)))+(((x430)*(x436)))+(((x431)*(x437))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj30)*(x429)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj30)*(x429)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x432)*(x433)))+(((IkReal(-1.56614161081993e+23))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x434)*(x435)))+(((IkReal(1.56614161081993e+23))*(x429)*(x433)))+(((x430)*(x435)))+(((IkReal(-1.29137887418136e+22))*(x429)*(x430)))+(((x431)*(x432)))+(((IkReal(1.29137887418136e+22))*(x429)*(x434)))))))+IKsqr(((IkReal(-1.14854255730435e-11))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IKabs(sj31) != 0)?((IkReal)1/(sj31)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x433)*(x437)))+(((IkReal(-1334056361602.36))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x434)*(x436)))+(((IkReal(1334056361602.36))*(x429)*(x433)))+(((IkReal(110001049103.000))*(x429)*(x434)))+(((IkReal(-110001049103.000))*(x429)*(x430)))+(((x430)*(x436)))+(((x431)*(x437)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj30)*(x429)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj30)*(x429)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x432)*(x433)))+(((IkReal(-1.56614161081993e+23))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x434)*(x435)))+(((IkReal(1.56614161081993e+23))*(x429)*(x433)))+(((x430)*(x435)))+(((IkReal(-1.29137887418136e+22))*(x429)*(x430)))+(((x431)*(x432)))+(((IkReal(1.29137887418136e+22))*(x429)*(x434)))))), ((IkReal(-1.14854255730435e-11))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IKabs(sj31) != 0)?((IkReal)1/(sj31)):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(x433)*(x437)))+(((IkReal(-1334056361602.36))*(x429)*(x431)))+(((IkReal(-1.00000000000000))*(x434)*(x436)))+(((IkReal(1334056361602.36))*(x429)*(x433)))+(((IkReal(110001049103.000))*(x429)*(x434)))+(((IkReal(-110001049103.000))*(x429)*(x430)))+(((x430)*(x436)))+(((x431)*(x437)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x438=IKsin(j29);
IkReal x439=IKcos(j29);
IkReal x440=((IkReal(1.00000000000000))*(r00));
IkReal x441=((cj27)*(sj28));
IkReal x442=((IkReal(1.00000000000000))*(cj28));
IkReal x443=((IkReal(0.0440000000000000))*(sj31));
IkReal x444=((IkReal(1.00000000000000))*(sj27));
IkReal x445=((IkReal(0.533617455371189))*(sj31));
IkReal x446=((sj30)*(x438));
IkReal x447=((sj30)*(x439));
IkReal x448=((cj30)*(cj31)*(x439));
IkReal x449=((cj30)*(cj31)*(x438));
evalcond[0]=((((x439)*(x443)))+(((cj27)*(py)))+(((IkReal(0.0440000000000000))*(x449)))+(((IkReal(-1.00000000000000))*(px)*(x444)))+(((IkReal(-0.135000000000000))*(x446))));
evalcond[1]=((((x439)*(x445)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(sj27)*(x440)))+(((IkReal(-0.845725967038483))*(x446)))+(((IkReal(0.533617455371189))*(x449))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x438)*(x443)))+(((IkReal(-1.00000000000000))*(px)*(x441)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x442)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x444)))+(((IkReal(0.0440000000000000))*(x448)))+(((IkReal(-0.135000000000000))*(x447))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x438)*(x445)))+(((IkReal(-1.00000000000000))*(r02)*(x442)))+(((IkReal(-0.845725967038483))*(x447)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x444)))+(((IkReal(0.533617455371189))*(x448)))+(((IkReal(-1.00000000000000))*(x440)*(x441))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x450=((cj27)*(py));
IkReal x451=((IkReal(2750.00000000000))*(sj31));
IkReal x452=((cj28)*(pz));
IkReal x453=((IkReal(8437.50000000000))*(sj30));
IkReal x454=((px)*(sj27));
IkReal x455=((IkReal(275.000000000000))*(sj28));
IkReal x456=((IkReal(2750.00000000000))*(cj30)*(cj31));
IkReal x457=((py)*(sj27)*(sj28));
IkReal x458=((cj27)*(px)*(sj28));
if( IKabs(((gconst49)*(((((x454)*(x456)))+(((IkReal(-1.00000000000000))*(x453)*(x454)))+(((IkReal(-1.00000000000000))*(x450)*(x456)))+(((IkReal(-1.00000000000000))*(x451)*(x452)))+(((IkReal(-1.00000000000000))*(x451)*(x457)))+(((IkReal(-1.00000000000000))*(x451)*(x458)))+(((x450)*(x453)))+(((sj31)*(x455))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst49)*(((((x452)*(x456)))+(((IkReal(-1.00000000000000))*(x452)*(x453)))+(((IkReal(-1.00000000000000))*(x453)*(x458)))+(((IkReal(-1.00000000000000))*(x453)*(x457)))+(((IkReal(-1.00000000000000))*(x450)*(x451)))+(((IkReal(843.750000000000))*(sj28)*(sj30)))+(((IkReal(-1.00000000000000))*(cj30)*(cj31)*(x455)))+(((x451)*(x454)))+(((x456)*(x457)))+(((x456)*(x458))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst49)*(((((x454)*(x456)))+(((IkReal(-1.00000000000000))*(x453)*(x454)))+(((IkReal(-1.00000000000000))*(x450)*(x456)))+(((IkReal(-1.00000000000000))*(x451)*(x452)))+(((IkReal(-1.00000000000000))*(x451)*(x457)))+(((IkReal(-1.00000000000000))*(x451)*(x458)))+(((x450)*(x453)))+(((sj31)*(x455)))))), ((gconst49)*(((((x452)*(x456)))+(((IkReal(-1.00000000000000))*(x452)*(x453)))+(((IkReal(-1.00000000000000))*(x453)*(x458)))+(((IkReal(-1.00000000000000))*(x453)*(x457)))+(((IkReal(-1.00000000000000))*(x450)*(x451)))+(((IkReal(843.750000000000))*(sj28)*(sj30)))+(((IkReal(-1.00000000000000))*(cj30)*(cj31)*(x455)))+(((x451)*(x454)))+(((x456)*(x457)))+(((x456)*(x458)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x459=IKsin(j29);
IkReal x460=IKcos(j29);
IkReal x461=((IkReal(1.00000000000000))*(r00));
IkReal x462=((cj27)*(sj28));
IkReal x463=((IkReal(1.00000000000000))*(cj28));
IkReal x464=((IkReal(0.0440000000000000))*(sj31));
IkReal x465=((IkReal(1.00000000000000))*(sj27));
IkReal x466=((IkReal(0.533617455371189))*(sj31));
IkReal x467=((sj30)*(x459));
IkReal x468=((sj30)*(x460));
IkReal x469=((cj30)*(cj31)*(x460));
IkReal x470=((cj30)*(cj31)*(x459));
evalcond[0]=((((IkReal(0.0440000000000000))*(x470)))+(((cj27)*(py)))+(((x460)*(x464)))+(((IkReal(-0.135000000000000))*(x467)))+(((IkReal(-1.00000000000000))*(px)*(x465))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj27)*(x461)))+(((IkReal(-0.845725967038483))*(x467)))+(((cj27)*(r01)))+(((IkReal(0.533617455371189))*(x470)))+(((x460)*(x466))));
evalcond[2]=((((IkReal(-1.00000000000000))*(px)*(x462)))+(((IkReal(-1.00000000000000))*(x459)*(x464)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x463)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x465)))+(((IkReal(-0.135000000000000))*(x468)))+(((IkReal(0.0440000000000000))*(x469))));
evalcond[3]=((((IkReal(-0.845725967038483))*(x468)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x465)))+(((IkReal(-1.00000000000000))*(x459)*(x466)))+(((IkReal(0.533617455371189))*(x469)))+(((IkReal(-1.00000000000000))*(r02)*(x463)))+(((IkReal(-1.00000000000000))*(x461)*(x462))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j31array[2], cj31array[2], sj31array[2];
bool j31valid[2]={false};
_nj31 = 2;
cj31array[0]=((IkReal(22.7272727272727))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IkReal(-0.400000000000000))+(((cj28)*(py)*(sj27)))+(((IkReal(-0.135000000000000))*(cj30)))+(((cj27)*(cj28)*(px)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-1.00000000000000))*(pz)*(sj28))))));
if( cj31array[0] >= -1-IKFAST_SINCOS_THRESH && cj31array[0] <= 1+IKFAST_SINCOS_THRESH )
{
    j31valid[0] = j31valid[1] = true;
    j31array[0] = IKacos(cj31array[0]);
    sj31array[0] = IKsin(j31array[0]);
    cj31array[1] = cj31array[0];
    j31array[1] = -j31array[0];
    sj31array[1] = -sj31array[0];
}
else if( isnan(cj31array[0]) )
{
    // probably any value will work
    j31valid[0] = true;
    cj31array[0] = 1; sj31array[0] = 0; j31array[0] = 0;
}
for(int ij31 = 0; ij31 < 2; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 2; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[1];
evalcond[0]=((((cj27)*(cj28)*(r00)))+(((cj28)*(r01)*(sj27)))+(((IkReal(-0.533617455371189))*(sj30)*(IKcos(j31))))+(((IkReal(-1.00000000000000))*(r02)*(sj28)))+(((IkReal(-0.845725967038483))*(cj30))));
if( IKabs(evalcond[0]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst49;
gconst49=IKsign(((((IkReal(1139.06250000000))*((sj30)*(sj30))))+(((IkReal(121.000000000000))*((cj30)*(cj30))*((cj31)*(cj31))))+(((IkReal(-742.500000000000))*(cj30)*(cj31)*(sj30)))+(((IkReal(121.000000000000))*((sj31)*(sj31))))));
dummyeval[0]=((((IkReal(9.41373966942149))*((sj30)*(sj30))))+((((cj30)*(cj30))*((cj31)*(cj31))))+((sj31)*(sj31))+(((IkReal(-6.13636363636364))*(cj30)*(cj31)*(sj30))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[3];
dummyeval[0]=((((cj30)*(cj31)*(sj30)))+(((IkReal(-2.32653684440068))*((sj30)*(sj30)))));
dummyeval[1]=sj30;
dummyeval[2]=sj31;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  || IKabs(dummyeval[2]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x471=((IkReal(1.00000000000000))*(sj28));
IkReal x472=((cj27)*(cj28));
IkReal x473=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x471)))+(((py)*(x473)))+(((px)*(x472)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(-0.845725967038483))+(((r00)*(x472)))+(((IkReal(-1.00000000000000))*(r02)*(x471)))+(((r01)*(x473))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst50;
gconst50=IKsign(((((IkReal(11.0000000000000))*((sj31)*(sj31))))+(((IkReal(11.0000000000000))*((cj31)*(cj31))))));
dummyeval[0]=(((sj31)*(sj31))+((cj31)*(cj31)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x474=((py)*(sj28));
IkReal x475=((IkReal(250.000000000000))*(sj31));
IkReal x476=((cj27)*(py));
IkReal x477=((cj31)*(sj28));
IkReal x478=((cj27)*(px));
IkReal x479=((IkReal(250.000000000000))*(cj31));
IkReal x480=((cj28)*(pz));
IkReal x481=((sj27)*(x479));
if( IKabs(((gconst50)*(((((IkReal(-1.00000000000000))*(x475)*(x480)))+(((IkReal(-1.00000000000000))*(sj28)*(x475)*(x478)))+(((IkReal(-1.00000000000000))*(x476)*(x479)))+(((px)*(x481)))+(((IkReal(-1.00000000000000))*(sj27)*(x474)*(x475)))+(((IkReal(25.0000000000000))*(sj28)*(sj31))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst50)*(((((IkReal(-25.0000000000000))*(x477)))+(((IkReal(-1.00000000000000))*(x475)*(x476)))+(((x474)*(x481)))+(((IkReal(250.000000000000))*(x477)*(x478)))+(((px)*(sj27)*(x475)))+(((x479)*(x480))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst50)*(((((IkReal(-1.00000000000000))*(x475)*(x480)))+(((IkReal(-1.00000000000000))*(sj28)*(x475)*(x478)))+(((IkReal(-1.00000000000000))*(x476)*(x479)))+(((px)*(x481)))+(((IkReal(-1.00000000000000))*(sj27)*(x474)*(x475)))+(((IkReal(25.0000000000000))*(sj28)*(sj31)))))), ((gconst50)*(((((IkReal(-25.0000000000000))*(x477)))+(((IkReal(-1.00000000000000))*(x475)*(x476)))+(((x474)*(x481)))+(((IkReal(250.000000000000))*(x477)*(x478)))+(((px)*(sj27)*(x475)))+(((x479)*(x480)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x482=IKcos(j29);
IkReal x483=IKsin(j29);
IkReal x484=((IkReal(1.00000000000000))*(cj28));
IkReal x485=((IkReal(0.533617455371189))*(cj31));
IkReal x486=((IkReal(1.00000000000000))*(sj27));
IkReal x487=((IkReal(0.0440000000000000))*(x483));
IkReal x488=((IkReal(1.00000000000000))*(cj27)*(sj28));
IkReal x489=((sj31)*(x482));
evalcond[0]=((((cj27)*(py)))+(((cj31)*(x487)))+(((IkReal(-1.00000000000000))*(px)*(x486)))+(((IkReal(0.0440000000000000))*(x489))));
evalcond[1]=((((IkReal(0.533617455371189))*(x489)))+(((cj27)*(r01)))+(((x483)*(x485)))+(((IkReal(-1.00000000000000))*(r00)*(x486))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)*(x484)))+(((IkReal(0.0440000000000000))*(cj31)*(x482)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x486)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(px)*(x488)))+(((IkReal(-1.00000000000000))*(sj31)*(x487))));
evalcond[3]=((((x482)*(x485)))+(((IkReal(-1.00000000000000))*(r02)*(x484)))+(((IkReal(-0.533617455371189))*(sj31)*(x483)))+(((IkReal(-1.00000000000000))*(r00)*(x488)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x486))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x490=((IkReal(1.00000000000000))*(sj28));
IkReal x491=((cj27)*(cj28));
IkReal x492=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j30, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((py)*(x492)))+(((px)*(x491)))+(((IkReal(-1.00000000000000))*(pz)*(x490)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x490)))+(((r00)*(x491)))+(((r01)*(x492))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst51;
gconst51=IKsign(((((IkReal(11.0000000000000))*((sj31)*(sj31))))+(((IkReal(11.0000000000000))*((cj31)*(cj31))))));
dummyeval[0]=(((sj31)*(sj31))+((cj31)*(cj31)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x493=((py)*(sj28));
IkReal x494=((cj31)*(sj28));
IkReal x495=((IkReal(250.000000000000))*(cj31));
IkReal x496=((cj28)*(pz));
IkReal x497=((sj28)*(sj31));
IkReal x498=((cj27)*(py));
IkReal x499=((IkReal(250.000000000000))*(sj31));
IkReal x500=((sj27)*(x495));
IkReal x501=((sj27)*(x499));
IkReal x502=((IkReal(250.000000000000))*(cj27)*(px));
if( IKabs(((gconst51)*(((((x495)*(x498)))+(((IkReal(-1.00000000000000))*(x497)*(x502)))+(((IkReal(-1.00000000000000))*(x496)*(x499)))+(((IkReal(-1.00000000000000))*(x493)*(x501)))+(((IkReal(25.0000000000000))*(x497)))+(((IkReal(-1.00000000000000))*(px)*(x500))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst51)*(((((IkReal(-1.00000000000000))*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-1.00000000000000))*(x494)*(x502)))+(((IkReal(-1.00000000000000))*(x493)*(x500)))+(((px)*(x501)))+(((IkReal(25.0000000000000))*(x494))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst51)*(((((x495)*(x498)))+(((IkReal(-1.00000000000000))*(x497)*(x502)))+(((IkReal(-1.00000000000000))*(x496)*(x499)))+(((IkReal(-1.00000000000000))*(x493)*(x501)))+(((IkReal(25.0000000000000))*(x497)))+(((IkReal(-1.00000000000000))*(px)*(x500)))))), ((gconst51)*(((((IkReal(-1.00000000000000))*(x498)*(x499)))+(((IkReal(-1.00000000000000))*(x495)*(x496)))+(((IkReal(-1.00000000000000))*(x494)*(x502)))+(((IkReal(-1.00000000000000))*(x493)*(x500)))+(((px)*(x501)))+(((IkReal(25.0000000000000))*(x494)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x503=IKcos(j29);
IkReal x504=IKsin(j29);
IkReal x505=((IkReal(1.00000000000000))*(cj28));
IkReal x506=((IkReal(0.533617455371189))*(cj31));
IkReal x507=((IkReal(1.00000000000000))*(sj27));
IkReal x508=((IkReal(0.0440000000000000))*(x504));
IkReal x509=((IkReal(1.00000000000000))*(cj27)*(sj28));
IkReal x510=((sj31)*(x503));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(cj31)*(x508)))+(((IkReal(0.0440000000000000))*(x510)))+(((IkReal(-1.00000000000000))*(px)*(x507))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(0.533617455371189))*(x510)))+(((IkReal(-1.00000000000000))*(r00)*(x507)))+(((IkReal(-1.00000000000000))*(x504)*(x506))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(cj31)*(x503)))+(((IkReal(-1.00000000000000))*(pz)*(x505)))+(((IkReal(-1.00000000000000))*(sj31)*(x508)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x507)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(px)*(x509))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj28)*(x507)))+(((IkReal(-1.00000000000000))*(x503)*(x506)))+(((IkReal(-1.00000000000000))*(r00)*(x509)))+(((IkReal(-0.533617455371189))*(sj31)*(x504)))+(((IkReal(-1.00000000000000))*(r02)*(x505))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x511=((IkReal(1.00000000000000))*(sj28));
IkReal x512=((cj27)*(cj28));
IkReal x513=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j31)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((px)*(x512)))+(((py)*(x513)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-1.00000000000000))*(pz)*(x511)))+(((IkReal(-0.0440000000000000))*(sj30)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((r01)*(x513)))+(((IkReal(-0.533617455371189))*(sj30)))+(((IkReal(-1.00000000000000))*(r02)*(x511)))+(((IkReal(-0.845725967038483))*(cj30)))+(((r00)*(x512))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst52;
gconst52=IKsign(((((IkReal(-0.135000000000000))*(sj30)))+(((IkReal(0.0440000000000000))*(cj30)))));
dummyeval[0]=((((IkReal(-3.06818181818182))*(sj30)))+(cj30));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x514=((IkReal(1.00000000000000))*(sj28));
IkReal x515=((cj27)*(cj28));
IkReal x516=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(5.96811616783542))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((px)*(x515)))+(((py)*(x516)))+(((IkReal(-1.00000000000000))*(pz)*(x514)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(0.969453625832208))+(((r01)*(x516)))+(((IkReal(-1.00000000000000))*(r02)*(x514)))+(((r00)*(x515))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x517=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x517)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x517))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))))+IKsqr(((((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x517)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x517)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))), ((((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x517)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x517)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x518=IKsin(j29);
IkReal x519=IKcos(j29);
IkReal x520=((IkReal(1.00000000000000))*(cj28));
IkReal x521=((IkReal(1.00000000000000))*(sj28));
IkReal x522=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-4.71405189209358e-11))*(x518)))+(((IkReal(-1.00000000000000))*(px)*(x522))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x522)))+(((IkReal(-0.245274683444403))*(x518))));
evalcond[2]=((((IkReal(-4.71405189209358e-11))*(x519)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x521)))+(((IkReal(-1.00000000000000))*(pz)*(x520)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x521))));
evalcond[3]=((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x521)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x521)))+(((IkReal(-1.00000000000000))*(r02)*(x520)))+(((IkReal(-0.245274683444403))*(x519))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x523=((IkReal(1.00000000000000))*(sj28));
IkReal x524=((cj27)*(cj28));
IkReal x525=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(2.82652351424562))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((px)*(x524)))+(((IkReal(-1.00000000000000))*(pz)*(x523)))+(((IkReal(-0.100000000000000))*(cj28)))+(((py)*(x525))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x523)))+(((r01)*(x525)))+(((r00)*(x524))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x526=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj27)*(px)*(x526)))+(((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x526))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))))+IKsqr(((((cj27)*(px)*(x526)))+(((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x526)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))), ((((cj27)*(px)*(x526)))+(((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x526)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x527=IKsin(j29);
IkReal x528=IKcos(j29);
IkReal x529=((IkReal(1.00000000000000))*(cj28));
IkReal x530=((IkReal(1.00000000000000))*(sj28));
IkReal x531=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(4.71405189209358e-11))*(x527)))+(((IkReal(-1.00000000000000))*(px)*(x531))));
evalcond[1]=((((IkReal(0.245274683444403))*(x527)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x531))));
evalcond[2]=((((IkReal(4.71405189209358e-11))*(x528)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x530)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x529)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x530))));
evalcond[3]=((((IkReal(0.245274683444403))*(x528)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x530)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x530)))+(((IkReal(-1.00000000000000))*(r02)*(x529))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
if( IKabs(((gconst52)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst52)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst52)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27)))))), ((gconst52)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x532=IKsin(j29);
IkReal x533=IKcos(j29);
IkReal x534=((IkReal(1.00000000000000))*(cj28));
IkReal x535=((IkReal(1.00000000000000))*(sj28));
IkReal x536=((IkReal(1.00000000000000))*(sj27));
IkReal x537=((IkReal(0.135000000000000))*(sj30));
IkReal x538=((IkReal(0.845725967038483))*(sj30));
IkReal x539=((cj30)*(x533));
IkReal x540=((cj30)*(x532));
evalcond[0]=((((IkReal(-1.00000000000000))*(x532)*(x537)))+(((cj27)*(py)))+(((IkReal(0.0440000000000000))*(x540)))+(((IkReal(-1.00000000000000))*(px)*(x536))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x532)*(x538)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x536)))+(((IkReal(0.533617455371189))*(x540))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x533)*(x537)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x535)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(0.0440000000000000))*(x539)))+(((IkReal(-1.00000000000000))*(pz)*(x534)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x535))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r01)*(sj27)*(x535)))+(((IkReal(-1.00000000000000))*(x533)*(x538)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x535)))+(((IkReal(-1.00000000000000))*(r02)*(x534)))+(((IkReal(0.533617455371189))*(x539))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x541=((IkReal(1.00000000000000))*(sj28));
IkReal x542=((cj27)*(cj28));
IkReal x543=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j31, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.400000000000000))+(((py)*(x543)))+(((IkReal(0.0440000000000000))*(sj30)))+(((px)*(x542)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-1.00000000000000))*(pz)*(x541)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x541)))+(((IkReal(0.533617455371189))*(sj30)))+(((r00)*(x542)))+(((IkReal(-0.845725967038483))*(cj30)))+(((r01)*(x543))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst53;
gconst53=IKsign(((((IkReal(-0.0440000000000000))*(cj30)))+(((IkReal(-0.135000000000000))*(sj30)))));
dummyeval[0]=((((IkReal(-3.06818181818182))*(sj30)))+(((IkReal(-1.00000000000000))*(cj30))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x544=((IkReal(1.00000000000000))*(sj28));
IkReal x545=((cj27)*(cj28));
IkReal x546=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(0.315069139344170))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.258010563561796))+(((py)*(x546)))+(((px)*(x545)))+(((IkReal(-1.00000000000000))*(pz)*(x544)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x544)))+(((r00)*(x545)))+(((r01)*(x546))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x547=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj27)*(px)*(x547)))+(((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x547))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))))+IKsqr(((((cj27)*(px)*(x547)))+(((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x547)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(px)*(sj27)))+(((IkReal(-21213173356.8144))*(cj27)*(py)))), ((((cj27)*(px)*(x547)))+(((IkReal(-2121317335.68144))*(sj28)))+(((IkReal(21213173356.8144))*(cj28)*(pz)))+(((py)*(sj27)*(x547)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x548=IKsin(j29);
IkReal x549=IKcos(j29);
IkReal x550=((IkReal(1.00000000000000))*(cj28));
IkReal x551=((IkReal(1.00000000000000))*(sj28));
IkReal x552=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(4.71405189209358e-11))*(x548)))+(((IkReal(-1.00000000000000))*(px)*(x552))));
evalcond[1]=((((IkReal(0.245274683444403))*(x548)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x552))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x551)))+(((IkReal(4.71405189209358e-11))*(x549)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x551)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x550))));
evalcond[3]=((((IkReal(0.245274683444403))*(x549)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x551)))+(((IkReal(-1.00000000000000))*(r02)*(x550)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x551))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x553=((IkReal(1.00000000000000))*(sj28));
IkReal x554=((cj27)*(cj28));
IkReal x555=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(-2.82652351424562))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.541989436438204))+(((py)*(x555)))+(((px)*(x554)))+(((IkReal(-1.00000000000000))*(pz)*(x553)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(-0.969453625832208))+(((IkReal(-1.00000000000000))*(r02)*(x553)))+(((r01)*(x555)))+(((r00)*(x554))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x556=((IkReal(21213173356.8144))*(sj28));
if( IKabs(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(py)*(sj27)*(x556)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x556)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))))+IKsqr(((((IkReal(-1.00000000000000))*(py)*(sj27)*(x556)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x556)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IkReal(21213173356.8144))*(cj27)*(py)))+(((IkReal(-21213173356.8144))*(px)*(sj27)))), ((((IkReal(-1.00000000000000))*(py)*(sj27)*(x556)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x556)))+(((IkReal(2121317335.68144))*(sj28)))+(((IkReal(-21213173356.8144))*(cj28)*(pz)))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x557=IKsin(j29);
IkReal x558=IKcos(j29);
IkReal x559=((IkReal(1.00000000000000))*(cj28));
IkReal x560=((IkReal(1.00000000000000))*(sj28));
IkReal x561=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-4.71405189209358e-11))*(x557)))+(((IkReal(-1.00000000000000))*(px)*(x561))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x561)))+(((IkReal(-0.245274683444403))*(x557))));
evalcond[2]=((((IkReal(-4.71405189209358e-11))*(x558)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x560)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x559)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x560))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x559)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x560)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x560)))+(((IkReal(-0.245274683444403))*(x558))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
if( IKabs(((gconst53)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst53)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst53)*(((((IkReal(-1.00000000000000))*(cj27)*(py)))+(((px)*(sj27)))))), ((gconst53)*(((((cj28)*(pz)))+(((cj27)*(px)*(sj28)))+(((py)*(sj27)*(sj28)))+(((IkReal(-0.100000000000000))*(sj28)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x562=IKsin(j29);
IkReal x563=IKcos(j29);
IkReal x564=((IkReal(1.00000000000000))*(cj28));
IkReal x565=((IkReal(1.00000000000000))*(sj28));
IkReal x566=((IkReal(1.00000000000000))*(sj27));
IkReal x567=((IkReal(0.135000000000000))*(sj30));
IkReal x568=((IkReal(0.845725967038483))*(sj30));
IkReal x569=((cj30)*(x563));
IkReal x570=((cj30)*(x562));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(x562)*(x567)))+(((IkReal(-0.0440000000000000))*(x570)))+(((IkReal(-1.00000000000000))*(px)*(x566))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(x562)*(x568)))+(((IkReal(-0.533617455371189))*(x570)))+(((IkReal(-1.00000000000000))*(r00)*(x566))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(x569)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x565)))+(((IkReal(-1.00000000000000))*(pz)*(x564)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x563)*(x567)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x565))));
evalcond[3]=((((IkReal(-0.533617455371189))*(x569)))+(((IkReal(-1.00000000000000))*(x563)*(x568)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x565)))+(((IkReal(-1.00000000000000))*(r02)*(x564)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x565))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x571=((cj30)*(cj31));
IkReal x572=((cj27)*(r01));
IkReal x573=((px)*(sj27));
IkReal x574=((IkReal(3.64368616112160e+23))*(sj30));
IkReal x575=((cj27)*(py));
IkReal x576=((r00)*(sj27));
IkReal x577=((IkReal(3.00444053086361e+22))*(sj30));
IkReal x578=((IkReal(337503218838.750))*(sj30));
IkReal x579=((IkReal(2114335082451.87))*(sj30));
if( IKabs(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj30)*(x571)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj30)*(x571)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30))))))):(IkReal)1.0e30))*(((((IkReal(-1.29137887418136e+22))*(x571)*(x572)))+(((x573)*(x574)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((IkReal(-1.00000000000000))*(x574)*(x575)))+(((IkReal(-1.56614161081993e+23))*(x571)*(x573)))+(((x572)*(x577)))+(((IkReal(1.56614161081993e+23))*(x571)*(x575)))+(((IkReal(1.29137887418136e+22))*(x571)*(x576))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-1.14854255730435e-11))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IKabs(sj31) != 0)?((IkReal)1/(sj31)):(IkReal)1.0e30))*(((((IkReal(110001049103.000))*(x571)*(x576)))+(((IkReal(-110001049103.000))*(x571)*(x572)))+(((x573)*(x579)))+(((IkReal(-1.00000000000000))*(x576)*(x578)))+(((IkReal(-1.00000000000000))*(x575)*(x579)))+(((IkReal(-1334056361602.36))*(x571)*(x573)))+(((x572)*(x578)))+(((IkReal(1334056361602.36))*(x571)*(x575))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj30)*(x571)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj30)*(x571)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30))))))):(IkReal)1.0e30))*(((((IkReal(-1.29137887418136e+22))*(x571)*(x572)))+(((x573)*(x574)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((IkReal(-1.00000000000000))*(x574)*(x575)))+(((IkReal(-1.56614161081993e+23))*(x571)*(x573)))+(((x572)*(x577)))+(((IkReal(1.56614161081993e+23))*(x571)*(x575)))+(((IkReal(1.29137887418136e+22))*(x571)*(x576)))))))+IKsqr(((IkReal(-1.14854255730435e-11))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IKabs(sj31) != 0)?((IkReal)1/(sj31)):(IkReal)1.0e30))*(((((IkReal(110001049103.000))*(x571)*(x576)))+(((IkReal(-110001049103.000))*(x571)*(x572)))+(((x573)*(x579)))+(((IkReal(-1.00000000000000))*(x576)*(x578)))+(((IkReal(-1.00000000000000))*(x575)*(x579)))+(((IkReal(-1334056361602.36))*(x571)*(x573)))+(((x572)*(x578)))+(((IkReal(1334056361602.36))*(x571)*(x575)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j29array[0]=IKatan2(((((IKabs(((((IkReal(1.02213852742680e+22))*(sj30)*(x571)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30)))))) != 0)?((IkReal)1/(((((IkReal(1.02213852742680e+22))*(sj30)*(x571)))+(((IkReal(-2.37804294413992e+22))*((sj30)*(sj30))))))):(IkReal)1.0e30))*(((((IkReal(-1.29137887418136e+22))*(x571)*(x572)))+(((x573)*(x574)))+(((IkReal(-1.00000000000000))*(x576)*(x577)))+(((IkReal(-1.00000000000000))*(x574)*(x575)))+(((IkReal(-1.56614161081993e+23))*(x571)*(x573)))+(((x572)*(x577)))+(((IkReal(1.56614161081993e+23))*(x571)*(x575)))+(((IkReal(1.29137887418136e+22))*(x571)*(x576)))))), ((IkReal(-1.14854255730435e-11))*(((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30))*(((IKabs(sj31) != 0)?((IkReal)1/(sj31)):(IkReal)1.0e30))*(((((IkReal(110001049103.000))*(x571)*(x576)))+(((IkReal(-110001049103.000))*(x571)*(x572)))+(((x573)*(x579)))+(((IkReal(-1.00000000000000))*(x576)*(x578)))+(((IkReal(-1.00000000000000))*(x575)*(x579)))+(((IkReal(-1334056361602.36))*(x571)*(x573)))+(((x572)*(x578)))+(((IkReal(1334056361602.36))*(x571)*(x575)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x580=IKsin(j29);
IkReal x581=IKcos(j29);
IkReal x582=((IkReal(1.00000000000000))*(r00));
IkReal x583=((cj27)*(sj28));
IkReal x584=((IkReal(1.00000000000000))*(cj28));
IkReal x585=((IkReal(0.0440000000000000))*(sj31));
IkReal x586=((IkReal(1.00000000000000))*(sj27));
IkReal x587=((IkReal(0.533617455371189))*(sj31));
IkReal x588=((sj30)*(x580));
IkReal x589=((sj30)*(x581));
IkReal x590=((cj30)*(cj31)*(x581));
IkReal x591=((cj30)*(cj31)*(x580));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-0.135000000000000))*(x588)))+(((x581)*(x585)))+(((IkReal(-1.00000000000000))*(px)*(x586)))+(((IkReal(0.0440000000000000))*(x591))));
evalcond[1]=((((x581)*(x587)))+(((cj27)*(r01)))+(((IkReal(0.533617455371189))*(x591)))+(((IkReal(-0.845725967038483))*(x588)))+(((IkReal(-1.00000000000000))*(sj27)*(x582))));
evalcond[2]=((((IkReal(-0.135000000000000))*(x589)))+(((IkReal(-1.00000000000000))*(x580)*(x585)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x586)))+(((IkReal(-1.00000000000000))*(pz)*(x584)))+(((IkReal(0.0440000000000000))*(x590)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(px)*(x583))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x580)*(x587)))+(((IkReal(-1.00000000000000))*(r02)*(x584)))+(((IkReal(-1.00000000000000))*(x582)*(x583)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x586)))+(((IkReal(0.533617455371189))*(x590)))+(((IkReal(-0.845725967038483))*(x589))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x592=((cj27)*(py));
IkReal x593=((IkReal(2750.00000000000))*(sj31));
IkReal x594=((cj28)*(pz));
IkReal x595=((IkReal(8437.50000000000))*(sj30));
IkReal x596=((px)*(sj27));
IkReal x597=((IkReal(275.000000000000))*(sj28));
IkReal x598=((IkReal(2750.00000000000))*(cj30)*(cj31));
IkReal x599=((py)*(sj27)*(sj28));
IkReal x600=((cj27)*(px)*(sj28));
if( IKabs(((gconst49)*(((((IkReal(-1.00000000000000))*(x593)*(x600)))+(((sj31)*(x597)))+(((IkReal(-1.00000000000000))*(x592)*(x598)))+(((IkReal(-1.00000000000000))*(x593)*(x599)))+(((IkReal(-1.00000000000000))*(x593)*(x594)))+(((IkReal(-1.00000000000000))*(x595)*(x596)))+(((x596)*(x598)))+(((x592)*(x595))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst49)*(((((x594)*(x598)))+(((IkReal(-1.00000000000000))*(cj30)*(cj31)*(x597)))+(((x598)*(x599)))+(((IkReal(-1.00000000000000))*(x592)*(x593)))+(((IkReal(-1.00000000000000))*(x594)*(x595)))+(((IkReal(-1.00000000000000))*(x595)*(x599)))+(((IkReal(843.750000000000))*(sj28)*(sj30)))+(((x593)*(x596)))+(((x598)*(x600)))+(((IkReal(-1.00000000000000))*(x595)*(x600))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((gconst49)*(((((IkReal(-1.00000000000000))*(x593)*(x600)))+(((sj31)*(x597)))+(((IkReal(-1.00000000000000))*(x592)*(x598)))+(((IkReal(-1.00000000000000))*(x593)*(x599)))+(((IkReal(-1.00000000000000))*(x593)*(x594)))+(((IkReal(-1.00000000000000))*(x595)*(x596)))+(((x596)*(x598)))+(((x592)*(x595)))))), ((gconst49)*(((((x594)*(x598)))+(((IkReal(-1.00000000000000))*(cj30)*(cj31)*(x597)))+(((x598)*(x599)))+(((IkReal(-1.00000000000000))*(x592)*(x593)))+(((IkReal(-1.00000000000000))*(x594)*(x595)))+(((IkReal(-1.00000000000000))*(x595)*(x599)))+(((IkReal(843.750000000000))*(sj28)*(sj30)))+(((x593)*(x596)))+(((x598)*(x600)))+(((IkReal(-1.00000000000000))*(x595)*(x600)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x601=IKsin(j29);
IkReal x602=IKcos(j29);
IkReal x603=((IkReal(1.00000000000000))*(r00));
IkReal x604=((cj27)*(sj28));
IkReal x605=((IkReal(1.00000000000000))*(cj28));
IkReal x606=((IkReal(0.0440000000000000))*(sj31));
IkReal x607=((IkReal(1.00000000000000))*(sj27));
IkReal x608=((IkReal(0.533617455371189))*(sj31));
IkReal x609=((sj30)*(x601));
IkReal x610=((sj30)*(x602));
IkReal x611=((cj30)*(cj31)*(x602));
IkReal x612=((cj30)*(cj31)*(x601));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-0.135000000000000))*(x609)))+(((IkReal(0.0440000000000000))*(x612)))+(((x602)*(x606)))+(((IkReal(-1.00000000000000))*(px)*(x607))));
evalcond[1]=((((IkReal(0.533617455371189))*(x612)))+(((cj27)*(r01)))+(((IkReal(-0.845725967038483))*(x609)))+(((x602)*(x608)))+(((IkReal(-1.00000000000000))*(sj27)*(x603))));
evalcond[2]=((((IkReal(-0.135000000000000))*(x610)))+(((IkReal(-1.00000000000000))*(py)*(sj28)*(x607)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(0.0440000000000000))*(x611)))+(((IkReal(-1.00000000000000))*(px)*(x604)))+(((IkReal(-1.00000000000000))*(pz)*(x605)))+(((IkReal(-1.00000000000000))*(x601)*(x606))));
evalcond[3]=((((IkReal(0.533617455371189))*(x611)))+(((IkReal(-0.845725967038483))*(x610)))+(((IkReal(-1.00000000000000))*(x603)*(x604)))+(((IkReal(-1.00000000000000))*(r01)*(sj28)*(x607)))+(((IkReal(-1.00000000000000))*(r02)*(x605)))+(((IkReal(-1.00000000000000))*(x601)*(x608))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j29array[1], cj29array[1], sj29array[1];
bool j29valid[1]={false};
_nj29 = 1;
IkReal x613=((IkReal(0.533617455371189))*(sj28));
IkReal x614=((IkReal(0.0440000000000000))*(sj28));
if( IKabs(((IkReal(50000238.4319315))*(gconst48)*(((((IkReal(-0.0440000000000000))*(cj27)*(r01)))+(((IkReal(-0.533617455371189))*(px)*(sj27)))+(((IkReal(0.0440000000000000))*(r00)*(sj27)))+(((IkReal(0.533617455371189))*(cj27)*(py))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-50000238.4319315))*(gconst48)*(((((IkReal(-0.0533617455371189))*(sj28)))+(((cj27)*(px)*(x613)))+(((IkReal(0.533617455371189))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x614)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x614)))+(((py)*(sj27)*(x613)))+(((IkReal(-0.0440000000000000))*(cj28)*(r02))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j29array[0]=IKatan2(((IkReal(50000238.4319315))*(gconst48)*(((((IkReal(-0.0440000000000000))*(cj27)*(r01)))+(((IkReal(-0.533617455371189))*(px)*(sj27)))+(((IkReal(0.0440000000000000))*(r00)*(sj27)))+(((IkReal(0.533617455371189))*(cj27)*(py)))))), ((IkReal(-50000238.4319315))*(gconst48)*(((((IkReal(-0.0533617455371189))*(sj28)))+(((cj27)*(px)*(x613)))+(((IkReal(0.533617455371189))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x614)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x614)))+(((py)*(sj27)*(x613)))+(((IkReal(-0.0440000000000000))*(cj28)*(r02)))))));
sj29array[0]=IKsin(j29array[0]);
cj29array[0]=IKcos(j29array[0]);
if( j29array[0] > IKPI )
{
    j29array[0]-=IK2PI;
}
else if( j29array[0] < -IKPI )
{    j29array[0]+=IK2PI;
}
j29valid[0] = true;
for(int ij29 = 0; ij29 < 1; ++ij29)
{
if( !j29valid[ij29] )
{
    continue;
}
_ij29[0] = ij29; _ij29[1] = -1;
for(int iij29 = ij29+1; iij29 < 1; ++iij29)
{
if( j29valid[iij29] && IKabs(cj29array[ij29]-cj29array[iij29]) < IKFAST_SOLUTION_THRESH && IKabs(sj29array[ij29]-sj29array[iij29]) < IKFAST_SOLUTION_THRESH )
{
    j29valid[iij29]=false; _ij29[1] = iij29; break; 
}
}
j29 = j29array[ij29]; cj29 = cj29array[ij29]; sj29 = sj29array[ij29];
{
IkReal evalcond[4];
IkReal x615=IKsin(j29);
IkReal x616=IKcos(j29);
IkReal x617=((IkReal(0.0348264139254172))*(sj30));
IkReal x618=((IkReal(0.0440000000000000))*(r01));
IkReal x619=((sj27)*(sj28));
IkReal x620=((cj27)*(cj30));
IkReal x621=((IkReal(0.533617455371189))*(py));
IkReal x622=((IkReal(0.0440000000000000))*(r00));
IkReal x623=((IkReal(0.0533617455371189))*(sj28));
IkReal x624=((IkReal(0.533617455371189))*(px)*(sj27));
IkReal x625=((IkReal(0.0440000000000000))*(cj28)*(r02));
IkReal x626=((IkReal(0.533617455371189))*(cj28)*(pz));
IkReal x627=((IkReal(0.533617455371189))*(cj27)*(px)*(sj28));
evalcond[0]=((((cj27)*(x621)))+(((sj27)*(x622)))+(((IkReal(-1.00000000000000))*(x624)))+(((IkReal(-1.00000000000000))*(cj27)*(x618)))+(((IkReal(-1.00000000000000))*(x615)*(x617))));
evalcond[1]=((((cj30)*(sj27)*(x622)))+(((IkReal(-1.00000000000000))*(cj30)*(x624)))+(((IkReal(-1.00000000000000))*(x618)*(x620)))+(((IkReal(-1.00000000000000))*(cj30)*(x615)*(x617)))+(((x620)*(x621))));
evalcond[2]=((((IkReal(-1.00000000000000))*(x618)*(x619)))+(((x619)*(x621)))+(((IkReal(-1.00000000000000))*(x625)))+(((IkReal(-1.00000000000000))*(x623)))+(((IkReal(-1.00000000000000))*(cj27)*(sj28)*(x622)))+(((x616)*(x617)))+(x627)+(x626));
evalcond[3]=((((IkReal(-0.533617455371189))*(px)*(sj28)*(x620)))+(((cj30)*(x618)*(x619)))+(((cj30)*(x623)))+(((cj30)*(x625)))+(((IkReal(-1.00000000000000))*(cj30)*(x626)))+(((sj28)*(x620)*(x622)))+(((IkReal(-1.00000000000000))*(cj30)*(x619)*(x621)))+(((IkReal(-1.00000000000000))*(cj30)*(x616)*(x617))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[2];
IkReal x628=(cj29)*(cj29);
IkReal x629=(sj29)*(sj29);
dummyeval[0]=((x629)+(x628));
dummyeval[1]=((((cj30)*(x628)))+(((cj30)*(x629))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj29;
dummyeval[1]=sj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal dummyeval[2];
dummyeval[0]=cj29;
dummyeval[1]=sj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  || IKabs(dummyeval[1]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j29)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x630=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.135000000000000))+(((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x630))));
evalcond[2]=((IkReal(-0.845725967038483))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x630))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x631=((IkReal(1.87400166530233))*(cj28));
IkReal x632=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((IkReal(-1.00000000000000))*(cj27)*(px)*(x632)))+(((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(2.27272727272727))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x632))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((r01)*(sj27)*(x631)))+(((IkReal(-1.87400166530233))*(r02)*(sj28)))+(((cj27)*(r00)*(x631))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj27)*(px)*(x632)))+(((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(2.27272727272727))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x632)))))+IKsqr(((((r01)*(sj27)*(x631)))+(((IkReal(-1.87400166530233))*(r02)*(sj28)))+(((cj27)*(r00)*(x631)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj27)*(px)*(x632)))+(((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(2.27272727272727))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x632)))), ((((r01)*(sj27)*(x631)))+(((IkReal(-1.87400166530233))*(r02)*(sj28)))+(((cj27)*(r00)*(x631)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x633=IKcos(j31);
IkReal x634=IKsin(j31);
IkReal x635=((IkReal(1.00000000000000))*(sj28));
IkReal x636=((IkReal(1.00000000000000))*(cj28));
IkReal x637=((cj27)*(r00));
IkReal x638=((cj28)*(sj27));
IkReal x639=((cj27)*(px));
evalcond[0]=((IkReal(-0.400000000000000))+(((py)*(x638)))+(((IkReal(-1.00000000000000))*(pz)*(x635)))+(((IkReal(-0.0440000000000000))*(x633)))+(((cj28)*(x639)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[1]=((((IkReal(-0.533617455371189))*(x633)))+(((IkReal(-1.00000000000000))*(r02)*(x635)))+(((r01)*(x638)))+(((cj28)*(x637))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)*(x636)))+(((IkReal(-0.0440000000000000))*(x634)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x635)*(x639)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x635))));
evalcond[3]=((((IkReal(-0.533617455371189))*(x634)))+(((IkReal(-1.00000000000000))*(r02)*(x636)))+(((IkReal(-1.00000000000000))*(x635)*(x637)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x635))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x640=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.135000000000000))+(((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x640))));
evalcond[2]=((IkReal(0.845725967038483))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x640))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x641=((IkReal(1.87400166530233))*(cj28));
IkReal x642=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x642)))+(((IkReal(2.27272727272727))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x642))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x641)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x641)))+(((IkReal(1.87400166530233))*(r02)*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x642)))+(((IkReal(2.27272727272727))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x642)))))+IKsqr(((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x641)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x641)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x642)))+(((IkReal(2.27272727272727))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x642)))), ((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x641)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x641)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x643=IKcos(j31);
IkReal x644=IKsin(j31);
IkReal x645=((IkReal(1.00000000000000))*(sj28));
IkReal x646=((IkReal(1.00000000000000))*(cj28));
IkReal x647=((cj27)*(r00));
IkReal x648=((cj28)*(sj27));
IkReal x649=((cj27)*(px));
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(0.0440000000000000))*(x643)))+(((py)*(x648)))+(((IkReal(-1.00000000000000))*(pz)*(x645)))+(((IkReal(-0.100000000000000))*(cj28)))+(((cj28)*(x649))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x645)))+(((IkReal(0.533617455371189))*(x643)))+(((r01)*(x648)))+(((cj28)*(x647))));
evalcond[2]=((((IkReal(-0.0440000000000000))*(x644)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x645)*(x649)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x645)))+(((IkReal(-1.00000000000000))*(pz)*(x646))));
evalcond[3]=((((IkReal(-0.533617455371189))*(x644)))+(((IkReal(-1.00000000000000))*(r02)*(x646)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x645)))+(((IkReal(-1.00000000000000))*(x645)*(x647))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x650=((IkReal(1.87400166530233))*(sj28));
if( IKabs(((((IkReal(-1.87400166530233))*(cj28)*(r02)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x650)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x650))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj30))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.87400166530233))*(cj28)*(r02)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x650)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x650)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj30)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-1.87400166530233))*(cj28)*(r02)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x650)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x650)))), ((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj30)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x651=IKcos(j31);
IkReal x652=IKsin(j31);
IkReal x653=((IkReal(1.00000000000000))*(sj28));
IkReal x654=((IkReal(1.00000000000000))*(sj27));
IkReal x655=((IkReal(1.00000000000000))*(cj28));
IkReal x656=((cj27)*(r00));
IkReal x657=((r01)*(sj27));
IkReal x658=((cj27)*(px));
IkReal x659=((py)*(sj27));
IkReal x660=((IkReal(0.0440000000000000))*(x651));
IkReal x661=((IkReal(0.533617455371189))*(x651));
evalcond[0]=((((cj27)*(py)))+(((cj30)*(x660)))+(((IkReal(-1.00000000000000))*(px)*(x654)))+(((IkReal(-0.135000000000000))*(sj30))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(sj30)*(x660)))+(((IkReal(-0.135000000000000))*(cj30)))+(((cj28)*(x659)))+(((cj28)*(x658)))+(((IkReal(-1.00000000000000))*(pz)*(x653)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj30)*(x661)))+(((IkReal(-1.00000000000000))*(r02)*(x653)))+(((cj28)*(x657)))+(((cj28)*(x656)))+(((IkReal(-0.845725967038483))*(cj30))));
evalcond[3]=((((cj27)*(r01)))+(((cj30)*(x661)))+(((IkReal(-1.00000000000000))*(r00)*(x654)))+(((IkReal(-0.845725967038483))*(sj30))));
evalcond[4]=((((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-0.0440000000000000))*(x652)))+(((IkReal(-1.00000000000000))*(pz)*(x655)))+(((IkReal(-1.00000000000000))*(x653)*(x659)))+(((IkReal(-1.00000000000000))*(x653)*(x658))));
evalcond[5]=((((IkReal(-0.533617455371189))*(x652)))+(((IkReal(-1.00000000000000))*(r02)*(x655)))+(((IkReal(-1.00000000000000))*(x653)*(x656)))+(((IkReal(-1.00000000000000))*(x653)*(x657))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x662=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((IkReal(-1.00000000000000))*(cj27)*(px)*(x662)))+(((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x662)))+(((IkReal(2.27272727272727))*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))+(((IkReal(84573.0000000000))*(sj30))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-1.00000000000000))*(cj27)*(px)*(x662)))+(((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x662)))+(((IkReal(2.27272727272727))*(sj28)))))+IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))+(((IkReal(84573.0000000000))*(sj30)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-1.00000000000000))*(cj27)*(px)*(x662)))+(((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x662)))+(((IkReal(2.27272727272727))*(sj28)))), ((IkReal(1.87399272890821e-5))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))+(((IkReal(84573.0000000000))*(sj30)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x663=IKcos(j31);
IkReal x664=IKsin(j31);
IkReal x665=((IkReal(1.00000000000000))*(sj28));
IkReal x666=((IkReal(1.00000000000000))*(sj27));
IkReal x667=((IkReal(1.00000000000000))*(cj28));
IkReal x668=((cj27)*(r00));
IkReal x669=((r01)*(sj27));
IkReal x670=((cj27)*(px));
IkReal x671=((py)*(sj27));
IkReal x672=((IkReal(0.0440000000000000))*(x663));
IkReal x673=((IkReal(0.533617455371189))*(x663));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-0.135000000000000))*(sj30)))+(((cj30)*(x672)))+(((IkReal(-1.00000000000000))*(px)*(x666))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj28)*(x671)))+(((cj28)*(x670)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-1.00000000000000))*(pz)*(x665)))+(((IkReal(-1.00000000000000))*(sj30)*(x672)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(sj30)*(x673)))+(((IkReal(-1.00000000000000))*(r02)*(x665)))+(((IkReal(-0.845725967038483))*(cj30)))+(((cj28)*(x668)))+(((cj28)*(x669))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x666)))+(((cj30)*(x673)))+(((IkReal(-0.845725967038483))*(sj30))));
evalcond[4]=((((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x665)*(x671)))+(((IkReal(-1.00000000000000))*(x665)*(x670)))+(((IkReal(-1.00000000000000))*(pz)*(x667)))+(((IkReal(-0.0440000000000000))*(x664))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x665)*(x668)))+(((IkReal(-1.00000000000000))*(x665)*(x669)))+(((IkReal(-0.533617455371189))*(x664)))+(((IkReal(-1.00000000000000))*(r02)*(x667))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x674=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x674)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x674)))+(((IkReal(2.27272727272727))*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj30))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x674)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x674)))+(((IkReal(2.27272727272727))*(sj28)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj30)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-22.7272727272727))*(cj28)*(pz)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x674)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x674)))+(((IkReal(2.27272727272727))*(sj28)))), ((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj30)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x675=IKcos(j31);
IkReal x676=IKsin(j31);
IkReal x677=((IkReal(1.00000000000000))*(sj28));
IkReal x678=((IkReal(1.00000000000000))*(sj27));
IkReal x679=((IkReal(1.00000000000000))*(cj28));
IkReal x680=((cj27)*(r00));
IkReal x681=((r01)*(sj27));
IkReal x682=((cj27)*(px));
IkReal x683=((py)*(sj27));
IkReal x684=((IkReal(0.0440000000000000))*(x675));
IkReal x685=((IkReal(0.533617455371189))*(x675));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-0.135000000000000))*(sj30)))+(((IkReal(-1.00000000000000))*(px)*(x678)))+(((cj30)*(x684))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x677)))+(((IkReal(-0.135000000000000))*(cj30)))+(((cj28)*(x682)))+(((cj28)*(x683)))+(((IkReal(-1.00000000000000))*(sj30)*(x684)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x677)))+(((cj28)*(x680)))+(((cj28)*(x681)))+(((IkReal(-1.00000000000000))*(sj30)*(x685)))+(((IkReal(-0.845725967038483))*(cj30))));
evalcond[3]=((((cj27)*(r01)))+(((cj30)*(x685)))+(((IkReal(-1.00000000000000))*(r00)*(x678)))+(((IkReal(-0.845725967038483))*(sj30))));
evalcond[4]=((((IkReal(-1.00000000000000))*(pz)*(x679)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x677)*(x682)))+(((IkReal(-1.00000000000000))*(x677)*(x683)))+(((IkReal(-0.0440000000000000))*(x676))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x679)))+(((IkReal(-0.533617455371189))*(x676)))+(((IkReal(-1.00000000000000))*(x677)*(x680)))+(((IkReal(-1.00000000000000))*(x677)*(x681))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j29)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj30;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[3];
IkReal x686=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.135000000000000))+(((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x686))));
evalcond[2]=((IkReal(0.845725967038483))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x686))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x687=((IkReal(1.87400166530233))*(cj28));
IkReal x688=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x688)))+(((cj27)*(px)*(x688)))+(((IkReal(22.7272727272727))*(cj28)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((cj27)*(r00)*(x687)))+(((r01)*(sj27)*(x687)))+(((IkReal(-1.87400166530233))*(r02)*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x688)))+(((cj27)*(px)*(x688)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))))+IKsqr(((((cj27)*(r00)*(x687)))+(((r01)*(sj27)*(x687)))+(((IkReal(-1.87400166530233))*(r02)*(sj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x688)))+(((cj27)*(px)*(x688)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))), ((((cj27)*(r00)*(x687)))+(((r01)*(sj27)*(x687)))+(((IkReal(-1.87400166530233))*(r02)*(sj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x689=IKcos(j31);
IkReal x690=IKsin(j31);
IkReal x691=((IkReal(1.00000000000000))*(sj28));
IkReal x692=((IkReal(1.00000000000000))*(cj28));
IkReal x693=((cj27)*(r00));
IkReal x694=((cj28)*(sj27));
IkReal x695=((cj27)*(px));
evalcond[0]=((IkReal(-0.400000000000000))+(((cj28)*(x695)))+(((IkReal(-0.0440000000000000))*(x689)))+(((py)*(x694)))+(((IkReal(-1.00000000000000))*(pz)*(x691)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[1]=((((cj28)*(x693)))+(((IkReal(-1.00000000000000))*(r02)*(x691)))+(((r01)*(x694)))+(((IkReal(-0.533617455371189))*(x689))));
evalcond[2]=((((IkReal(0.0440000000000000))*(x690)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x691)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x692)))+(((IkReal(-1.00000000000000))*(x691)*(x695))));
evalcond[3]=((((IkReal(0.533617455371189))*(x690)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x691)))+(((IkReal(-1.00000000000000))*(r02)*(x692)))+(((IkReal(-1.00000000000000))*(x691)*(x693))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
IkReal x696=((IkReal(1.00000000000000))*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.135000000000000))+(((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x696))));
evalcond[2]=((IkReal(-0.845725967038483))+(((IkReal(-1.00000000000000))*(r00)*(x696)))+(((cj27)*(r01))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x697=((IkReal(1.87400166530233))*(cj28));
IkReal x698=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((cj27)*(px)*(x698)))+(((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x698)))+(((IkReal(22.7272727272727))*(cj28)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(r01)*(sj27)*(x697)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x697))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj27)*(px)*(x698)))+(((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x698)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))))+IKsqr(((((IkReal(-1.00000000000000))*(r01)*(sj27)*(x697)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x697)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((cj27)*(px)*(x698)))+(((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x698)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))), ((((IkReal(-1.00000000000000))*(r01)*(sj27)*(x697)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x697)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x699=IKcos(j31);
IkReal x700=IKsin(j31);
IkReal x701=((IkReal(1.00000000000000))*(sj28));
IkReal x702=((IkReal(1.00000000000000))*(cj28));
IkReal x703=((cj27)*(r00));
IkReal x704=((cj28)*(sj27));
IkReal x705=((cj27)*(px));
evalcond[0]=((IkReal(-0.400000000000000))+(((IkReal(0.0440000000000000))*(x699)))+(((IkReal(-1.00000000000000))*(pz)*(x701)))+(((cj28)*(x705)))+(((py)*(x704)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[1]=((((IkReal(-1.00000000000000))*(r02)*(x701)))+(((cj28)*(x703)))+(((r01)*(x704)))+(((IkReal(0.533617455371189))*(x699))));
evalcond[2]=((((IkReal(-1.00000000000000))*(pz)*(x702)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x701)))+(((IkReal(0.0440000000000000))*(x700)))+(((IkReal(-1.00000000000000))*(x701)*(x705))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r02)*(x702)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x701)))+(((IkReal(0.533617455371189))*(x700)))+(((IkReal(-1.00000000000000))*(x701)*(x703))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x706=((IkReal(1.87400166530233))*(sj28));
if( IKabs(((((IkReal(1.87400166530233))*(cj28)*(r02)))+(((cj27)*(r00)*(x706)))+(((r01)*(sj27)*(x706))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj30)))+(((IkReal(250.000000000000))*(cj27)*(py)))+(((IkReal(-250.000000000000))*(px)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(1.87400166530233))*(cj28)*(r02)))+(((cj27)*(r00)*(x706)))+(((r01)*(sj27)*(x706)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj30)))+(((IkReal(250.000000000000))*(cj27)*(py)))+(((IkReal(-250.000000000000))*(px)*(sj27)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(1.87400166530233))*(cj28)*(r02)))+(((cj27)*(r00)*(x706)))+(((r01)*(sj27)*(x706)))), ((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj30)))+(((IkReal(250.000000000000))*(cj27)*(py)))+(((IkReal(-250.000000000000))*(px)*(sj27)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x707=IKcos(j31);
IkReal x708=IKsin(j31);
IkReal x709=((IkReal(1.00000000000000))*(sj28));
IkReal x710=((IkReal(1.00000000000000))*(sj27));
IkReal x711=((IkReal(1.00000000000000))*(cj28));
IkReal x712=((cj27)*(r00));
IkReal x713=((r01)*(sj27));
IkReal x714=((cj27)*(px));
IkReal x715=((py)*(sj27));
IkReal x716=((IkReal(0.0440000000000000))*(x707));
IkReal x717=((IkReal(0.533617455371189))*(x707));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x710)))+(((IkReal(-1.00000000000000))*(cj30)*(x716)))+(((IkReal(0.135000000000000))*(sj30))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x709)))+(((cj28)*(x714)))+(((cj28)*(x715)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-1.00000000000000))*(sj30)*(x716)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x709)))+(((cj28)*(x712)))+(((cj28)*(x713)))+(((IkReal(-1.00000000000000))*(sj30)*(x717)))+(((IkReal(-0.845725967038483))*(cj30))));
evalcond[3]=((((IkReal(0.845725967038483))*(sj30)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(cj30)*(x717)))+(((IkReal(-1.00000000000000))*(r00)*(x710))));
evalcond[4]=((((IkReal(-1.00000000000000))*(pz)*(x711)))+(((IkReal(-1.00000000000000))*(x709)*(x715)))+(((IkReal(-1.00000000000000))*(x709)*(x714)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(0.0440000000000000))*(x708))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x711)))+(((IkReal(0.533617455371189))*(x708)))+(((IkReal(-1.00000000000000))*(x709)*(x713)))+(((IkReal(-1.00000000000000))*(x709)*(x712))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x718=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x718)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))+(((cj27)*(px)*(x718))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(cj27)*(r01)))+(((IkReal(-100000.476863863))*(r00)*(sj27)))+(((IkReal(84573.0000000000))*(sj30))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x718)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))+(((cj27)*(px)*(x718)))))+IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(cj27)*(r01)))+(((IkReal(-100000.476863863))*(r00)*(sj27)))+(((IkReal(84573.0000000000))*(sj30)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x718)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))+(((cj27)*(px)*(x718)))), ((IkReal(1.87399272890821e-5))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(100000.476863863))*(cj27)*(r01)))+(((IkReal(-100000.476863863))*(r00)*(sj27)))+(((IkReal(84573.0000000000))*(sj30)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x719=IKcos(j31);
IkReal x720=IKsin(j31);
IkReal x721=((IkReal(1.00000000000000))*(sj28));
IkReal x722=((IkReal(1.00000000000000))*(sj27));
IkReal x723=((IkReal(1.00000000000000))*(cj28));
IkReal x724=((cj27)*(r00));
IkReal x725=((r01)*(sj27));
IkReal x726=((cj27)*(px));
IkReal x727=((py)*(sj27));
IkReal x728=((IkReal(0.0440000000000000))*(x719));
IkReal x729=((IkReal(0.533617455371189))*(x719));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x722)))+(((IkReal(-1.00000000000000))*(cj30)*(x728)))+(((IkReal(0.135000000000000))*(sj30))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-1.00000000000000))*(pz)*(x721)))+(((cj28)*(x727)))+(((cj28)*(x726)))+(((IkReal(-1.00000000000000))*(sj30)*(x728))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x721)))+(((IkReal(-0.845725967038483))*(cj30)))+(((cj28)*(x725)))+(((cj28)*(x724)))+(((IkReal(-1.00000000000000))*(sj30)*(x729))));
evalcond[3]=((((IkReal(0.845725967038483))*(sj30)))+(((IkReal(-1.00000000000000))*(r00)*(x722)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(cj30)*(x729))));
evalcond[4]=((((IkReal(0.0440000000000000))*(x720)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x721)*(x727)))+(((IkReal(-1.00000000000000))*(x721)*(x726)))+(((IkReal(-1.00000000000000))*(pz)*(x723))));
evalcond[5]=((((IkReal(0.533617455371189))*(x720)))+(((IkReal(-1.00000000000000))*(r02)*(x723)))+(((IkReal(-1.00000000000000))*(x721)*(x725)))+(((IkReal(-1.00000000000000))*(x721)*(x724))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x730=((IkReal(22.7272727272727))*(sj28));
if( IKabs(((((cj27)*(px)*(x730)))+(((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x730)))+(((IkReal(22.7272727272727))*(cj28)*(pz))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj30)))+(((IkReal(250.000000000000))*(cj27)*(py)))+(((IkReal(-250.000000000000))*(px)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((cj27)*(px)*(x730)))+(((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x730)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))))+IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj30)))+(((IkReal(250.000000000000))*(cj27)*(py)))+(((IkReal(-250.000000000000))*(px)*(sj27)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((cj27)*(px)*(x730)))+(((IkReal(-2.27272727272727))*(sj28)))+(((py)*(sj27)*(x730)))+(((IkReal(22.7272727272727))*(cj28)*(pz)))), ((IkReal(0.0909090909090909))*(((IKabs(cj30) != 0)?((IkReal)1/(cj30)):(IkReal)1.0e30))*(((((IkReal(33.7500000000000))*(sj30)))+(((IkReal(250.000000000000))*(cj27)*(py)))+(((IkReal(-250.000000000000))*(px)*(sj27)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x731=IKcos(j31);
IkReal x732=IKsin(j31);
IkReal x733=((IkReal(1.00000000000000))*(sj28));
IkReal x734=((IkReal(1.00000000000000))*(sj27));
IkReal x735=((IkReal(1.00000000000000))*(cj28));
IkReal x736=((cj27)*(r00));
IkReal x737=((r01)*(sj27));
IkReal x738=((cj27)*(px));
IkReal x739=((py)*(sj27));
IkReal x740=((IkReal(0.0440000000000000))*(x731));
IkReal x741=((IkReal(0.533617455371189))*(x731));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x734)))+(((IkReal(-1.00000000000000))*(cj30)*(x740)))+(((IkReal(0.135000000000000))*(sj30))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-1.00000000000000))*(sj30)*(x740)))+(((IkReal(-1.00000000000000))*(pz)*(x733)))+(((cj28)*(x738)))+(((cj28)*(x739)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x733)))+(((IkReal(-1.00000000000000))*(sj30)*(x741)))+(((cj28)*(x736)))+(((cj28)*(x737)))+(((IkReal(-0.845725967038483))*(cj30))));
evalcond[3]=((((IkReal(0.845725967038483))*(sj30)))+(((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(cj30)*(x741)))+(((IkReal(-1.00000000000000))*(r00)*(x734))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x733)*(x738)))+(((IkReal(-1.00000000000000))*(x733)*(x739)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x735)))+(((IkReal(0.0440000000000000))*(x732))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x735)))+(((IkReal(-1.00000000000000))*(x733)*(x736)))+(((IkReal(-1.00000000000000))*(x733)*(x737)))+(((IkReal(0.533617455371189))*(x732))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x742=((IkReal(1.00000000000000))*(sj28));
IkReal x743=((cj27)*(cj28));
IkReal x744=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j30)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.535000000000000))+(((py)*(x744)))+(((px)*(x743)))+(((IkReal(-1.00000000000000))*(pz)*(x742)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((IkReal(-0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x742)))+(((r00)*(x743)))+(((r01)*(x744))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst54;
gconst54=IKsign(((((IkReal(11.0000000000000))*((sj29)*(sj29))))+(((IkReal(11.0000000000000))*((cj29)*(cj29))))));
dummyeval[0]=(((sj29)*(sj29))+((cj29)*(cj29)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x745=((IkReal(250.000000000000))*(cj29));
IkReal x746=((px)*(sj27));
IkReal x747=((cj27)*(py));
IkReal x748=((IkReal(250.000000000000))*(sj29));
IkReal x749=((IkReal(25.0000000000000))*(sj28));
IkReal x750=((cj28)*(pz));
IkReal x751=((cj27)*(px)*(sj28));
IkReal x752=((py)*(sj27)*(sj28));
if( IKabs(((gconst54)*(((((x745)*(x746)))+(((IkReal(-1.00000000000000))*(x745)*(x747)))+(((sj29)*(x749)))+(((IkReal(-1.00000000000000))*(x748)*(x751)))+(((IkReal(-1.00000000000000))*(x748)*(x750)))+(((IkReal(-1.00000000000000))*(x748)*(x752))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst54)*(((((x745)*(x752)))+(((x745)*(x750)))+(((x745)*(x751)))+(((IkReal(-1.00000000000000))*(x747)*(x748)))+(((x746)*(x748)))+(((IkReal(-1.00000000000000))*(cj29)*(x749))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j31array[0]=IKatan2(((gconst54)*(((((x745)*(x746)))+(((IkReal(-1.00000000000000))*(x745)*(x747)))+(((sj29)*(x749)))+(((IkReal(-1.00000000000000))*(x748)*(x751)))+(((IkReal(-1.00000000000000))*(x748)*(x750)))+(((IkReal(-1.00000000000000))*(x748)*(x752)))))), ((gconst54)*(((((x745)*(x752)))+(((x745)*(x750)))+(((x745)*(x751)))+(((IkReal(-1.00000000000000))*(x747)*(x748)))+(((x746)*(x748)))+(((IkReal(-1.00000000000000))*(cj29)*(x749)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x753=IKcos(j31);
IkReal x754=IKsin(j31);
IkReal x755=((IkReal(1.00000000000000))*(cj28));
IkReal x756=((IkReal(1.00000000000000))*(sj28));
IkReal x757=((IkReal(1.00000000000000))*(sj27));
IkReal x758=((IkReal(0.533617455371189))*(cj29));
IkReal x759=((IkReal(0.533617455371189))*(sj29));
IkReal x760=((IkReal(0.0440000000000000))*(x753));
IkReal x761=((IkReal(0.0440000000000000))*(x754));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x757)))+(((cj29)*(x761)))+(((sj29)*(x760))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x757)))+(((x753)*(x759)))+(((x754)*(x758))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x756)))+(((IkReal(-1.00000000000000))*(sj29)*(x761)))+(((IkReal(0.100000000000000))*(sj28)))+(((cj29)*(x760)))+(((IkReal(-1.00000000000000))*(pz)*(x755)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x756))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x754)*(x759)))+(((IkReal(-1.00000000000000))*(r02)*(x755)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x756)))+(((x753)*(x758)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x756))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x762=((IkReal(1.00000000000000))*(sj28));
IkReal x763=((cj27)*(cj28));
IkReal x764=((cj28)*(sj27));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(j30, IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.265000000000000))+(((px)*(x763)))+(((py)*(x764)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-1.00000000000000))*(pz)*(x762))));
evalcond[2]=((IkReal(0.845725967038483))+(((IkReal(-1.00000000000000))*(r02)*(x762)))+(((r00)*(x763)))+(((r01)*(x764))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst55;
gconst55=IKsign(((((IkReal(11.0000000000000))*((sj29)*(sj29))))+(((IkReal(11.0000000000000))*((cj29)*(cj29))))));
dummyeval[0]=(((sj29)*(sj29))+((cj29)*(cj29)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x765=((IkReal(250.000000000000))*(cj29));
IkReal x766=((px)*(sj27));
IkReal x767=((cj27)*(py));
IkReal x768=((IkReal(25.0000000000000))*(sj28));
IkReal x769=((IkReal(250.000000000000))*(sj29));
IkReal x770=((cj28)*(pz));
IkReal x771=((cj27)*(px)*(sj28));
IkReal x772=((py)*(sj27)*(sj28));
if( IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x765)*(x767)))+(((IkReal(-1.00000000000000))*(x769)*(x770)))+(((IkReal(-1.00000000000000))*(x769)*(x771)))+(((IkReal(-1.00000000000000))*(x769)*(x772)))+(((x765)*(x766)))+(((sj29)*(x768))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst55)*(((((IkReal(-1.00000000000000))*(x766)*(x769)))+(((IkReal(-1.00000000000000))*(x765)*(x771)))+(((IkReal(-1.00000000000000))*(x765)*(x772)))+(((IkReal(-1.00000000000000))*(x765)*(x770)))+(((cj29)*(x768)))+(((x767)*(x769))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j31array[0]=IKatan2(((gconst55)*(((((IkReal(-1.00000000000000))*(x765)*(x767)))+(((IkReal(-1.00000000000000))*(x769)*(x770)))+(((IkReal(-1.00000000000000))*(x769)*(x771)))+(((IkReal(-1.00000000000000))*(x769)*(x772)))+(((x765)*(x766)))+(((sj29)*(x768)))))), ((gconst55)*(((((IkReal(-1.00000000000000))*(x766)*(x769)))+(((IkReal(-1.00000000000000))*(x765)*(x771)))+(((IkReal(-1.00000000000000))*(x765)*(x772)))+(((IkReal(-1.00000000000000))*(x765)*(x770)))+(((cj29)*(x768)))+(((x767)*(x769)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[4];
IkReal x773=IKsin(j31);
IkReal x774=IKcos(j31);
IkReal x775=((IkReal(1.00000000000000))*(cj28));
IkReal x776=((IkReal(1.00000000000000))*(sj28));
IkReal x777=((IkReal(1.00000000000000))*(sj27));
IkReal x778=((IkReal(0.533617455371189))*(cj29));
IkReal x779=((IkReal(0.533617455371189))*(sj29));
IkReal x780=((IkReal(0.0440000000000000))*(x774));
IkReal x781=((IkReal(0.0440000000000000))*(x773));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x777)))+(((IkReal(-1.00000000000000))*(sj29)*(x780)))+(((cj29)*(x781))));
evalcond[1]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(x774)*(x779)))+(((x773)*(x778)))+(((IkReal(-1.00000000000000))*(r00)*(x777))));
evalcond[2]=((((IkReal(-1.00000000000000))*(py)*(sj27)*(x776)))+(((IkReal(-1.00000000000000))*(sj29)*(x781)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(pz)*(x775)))+(((IkReal(-1.00000000000000))*(cj29)*(x780)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x776))));
evalcond[3]=((((IkReal(-1.00000000000000))*(x774)*(x778)))+(((IkReal(-1.00000000000000))*(r02)*(x775)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x776)))+(((IkReal(-1.00000000000000))*(x773)*(x779)))+(((IkReal(-1.00000000000000))*(cj27)*(r00)*(x776))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j30)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj29;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj29;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj29;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x782=((IkReal(22.7272727272727))*(cj28));
if( IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(84573.0000000000))*(sj29)))+(((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-9.09090909090909))+(((IkReal(-2.27272727272727))*(cj28)))+(((py)*(sj27)*(x782)))+(((cj27)*(px)*(x782)))+(((IkReal(-22.7272727272727))*(pz)*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(84573.0000000000))*(sj29)))+(((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))))))+IKsqr(((IkReal(-9.09090909090909))+(((IkReal(-2.27272727272727))*(cj28)))+(((py)*(sj27)*(x782)))+(((cj27)*(px)*(x782)))+(((IkReal(-22.7272727272727))*(pz)*(sj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(1.87399272890821e-5))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(84573.0000000000))*(sj29)))+(((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))))), ((IkReal(-9.09090909090909))+(((IkReal(-2.27272727272727))*(cj28)))+(((py)*(sj27)*(x782)))+(((cj27)*(px)*(x782)))+(((IkReal(-22.7272727272727))*(pz)*(sj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x783=IKcos(j31);
IkReal x784=IKsin(j31);
IkReal x785=((IkReal(1.00000000000000))*(sj28));
IkReal x786=((IkReal(1.00000000000000))*(sj27));
IkReal x787=((r01)*(sj27));
IkReal x788=((IkReal(1.00000000000000))*(cj28));
IkReal x789=((cj27)*(r00));
IkReal x790=((cj27)*(px));
IkReal x791=((py)*(sj27));
IkReal x792=((IkReal(0.533617455371189))*(x784));
IkReal x793=((IkReal(0.0440000000000000))*(x784));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x786)))+(((IkReal(-0.135000000000000))*(sj29)))+(((cj29)*(x793))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj28)*(x790)))+(((cj28)*(x791)))+(((IkReal(-1.00000000000000))*(pz)*(x785)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-0.0440000000000000))*(x783))));
evalcond[2]=((((cj28)*(x789)))+(((cj28)*(x787)))+(((IkReal(-0.533617455371189))*(x783)))+(((IkReal(-1.00000000000000))*(r02)*(x785))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x786)))+(((cj29)*(x792)))+(((IkReal(-0.845725967038483))*(sj29))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj29)*(x793)))+(((IkReal(-0.135000000000000))*(cj29)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x785)*(x791)))+(((IkReal(-1.00000000000000))*(x785)*(x790)))+(((IkReal(-1.00000000000000))*(pz)*(x788))));
evalcond[5]=((((IkReal(-1.00000000000000))*(sj29)*(x792)))+(((IkReal(-1.00000000000000))*(x785)*(x787)))+(((IkReal(-1.00000000000000))*(x785)*(x789)))+(((IkReal(-0.845725967038483))*(cj29)))+(((IkReal(-1.00000000000000))*(r02)*(x788))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x794=((IkReal(1.87400166530233))*(cj28));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj29))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.87400166530233))*(r02)*(sj28)))+(((r01)*(sj27)*(x794)))+(((cj27)*(r00)*(x794))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj29)))))))+IKsqr(((((IkReal(-1.87400166530233))*(r02)*(sj28)))+(((r01)*(sj27)*(x794)))+(((cj27)*(r00)*(x794)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj29)))))), ((((IkReal(-1.87400166530233))*(r02)*(sj28)))+(((r01)*(sj27)*(x794)))+(((cj27)*(r00)*(x794)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x795=IKcos(j31);
IkReal x796=IKsin(j31);
IkReal x797=((IkReal(1.00000000000000))*(sj28));
IkReal x798=((IkReal(1.00000000000000))*(sj27));
IkReal x799=((r01)*(sj27));
IkReal x800=((IkReal(1.00000000000000))*(cj28));
IkReal x801=((cj27)*(r00));
IkReal x802=((cj27)*(px));
IkReal x803=((py)*(sj27));
IkReal x804=((IkReal(0.533617455371189))*(x796));
IkReal x805=((IkReal(0.0440000000000000))*(x796));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x798)))+(((IkReal(-0.135000000000000))*(sj29)))+(((cj29)*(x805))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x797)))+(((cj28)*(x802)))+(((cj28)*(x803)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-0.0440000000000000))*(x795))));
evalcond[2]=((((cj28)*(x799)))+(((cj28)*(x801)))+(((IkReal(-1.00000000000000))*(r02)*(x797)))+(((IkReal(-0.533617455371189))*(x795))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x798)))+(((cj29)*(x804)))+(((IkReal(-0.845725967038483))*(sj29))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x797)*(x803)))+(((IkReal(-1.00000000000000))*(x797)*(x802)))+(((IkReal(-0.135000000000000))*(cj29)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(sj29)*(x805)))+(((IkReal(-1.00000000000000))*(pz)*(x800))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x797)*(x801)))+(((IkReal(-1.00000000000000))*(r02)*(x800)))+(((IkReal(-1.00000000000000))*(x797)*(x799)))+(((IkReal(-1.00000000000000))*(sj29)*(x804)))+(((IkReal(-0.845725967038483))*(cj29))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x806=((IkReal(22.7272727272727))*(cj28));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj29))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(-9.09090909090909))+(((cj27)*(px)*(x806)))+(((IkReal(-2.27272727272727))*(cj28)))+(((py)*(sj27)*(x806)))+(((IkReal(-22.7272727272727))*(pz)*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj29)))))))+IKsqr(((IkReal(-9.09090909090909))+(((cj27)*(px)*(x806)))+(((IkReal(-2.27272727272727))*(cj28)))+(((py)*(sj27)*(x806)))+(((IkReal(-22.7272727272727))*(pz)*(sj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(33.7500000000000))*(sj29)))))), ((IkReal(-9.09090909090909))+(((cj27)*(px)*(x806)))+(((IkReal(-2.27272727272727))*(cj28)))+(((py)*(sj27)*(x806)))+(((IkReal(-22.7272727272727))*(pz)*(sj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x807=IKcos(j31);
IkReal x808=IKsin(j31);
IkReal x809=((IkReal(1.00000000000000))*(sj28));
IkReal x810=((IkReal(1.00000000000000))*(sj27));
IkReal x811=((r01)*(sj27));
IkReal x812=((IkReal(1.00000000000000))*(cj28));
IkReal x813=((cj27)*(r00));
IkReal x814=((cj27)*(px));
IkReal x815=((py)*(sj27));
IkReal x816=((IkReal(0.533617455371189))*(x808));
IkReal x817=((IkReal(0.0440000000000000))*(x808));
evalcond[0]=((((cj27)*(py)))+(((cj29)*(x817)))+(((IkReal(-0.135000000000000))*(sj29)))+(((IkReal(-1.00000000000000))*(px)*(x810))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj28)*(x815)))+(((cj28)*(x814)))+(((IkReal(-0.0440000000000000))*(x807)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-1.00000000000000))*(pz)*(x809))));
evalcond[2]=((((IkReal(-0.533617455371189))*(x807)))+(((cj28)*(x811)))+(((cj28)*(x813)))+(((IkReal(-1.00000000000000))*(r02)*(x809))));
evalcond[3]=((((cj27)*(r01)))+(((cj29)*(x816)))+(((IkReal(-0.845725967038483))*(sj29)))+(((IkReal(-1.00000000000000))*(r00)*(x810))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x809)*(x814)))+(((IkReal(-1.00000000000000))*(x809)*(x815)))+(((IkReal(-1.00000000000000))*(pz)*(x812)))+(((IkReal(-1.00000000000000))*(sj29)*(x817)))+(((IkReal(-0.135000000000000))*(cj29)))+(((IkReal(0.100000000000000))*(sj28))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x809)*(x813)))+(((IkReal(-1.00000000000000))*(x809)*(x811)))+(((IkReal(-1.00000000000000))*(sj29)*(x816)))+(((IkReal(-1.00000000000000))*(r02)*(x812)))+(((IkReal(-0.845725967038483))*(cj29))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j30)), IkReal(6.28318530717959))));
if( IKabs(evalcond[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj29;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj29;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=cj29;
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x818=((IkReal(22.7272727272727))*(cj28));
if( IKabs(((IkReal(1.87399272890821e-5))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(-84573.0000000000))*(sj29)))+(((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x818)))+(((IkReal(22.7272727272727))*(pz)*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x818)))+(((IkReal(2.27272727272727))*(cj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1.87399272890821e-5))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(-84573.0000000000))*(sj29)))+(((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))))))+IKsqr(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x818)))+(((IkReal(22.7272727272727))*(pz)*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x818)))+(((IkReal(2.27272727272727))*(cj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(1.87399272890821e-5))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(-84573.0000000000))*(sj29)))+(((IkReal(-100000.476863863))*(cj27)*(r01)))+(((IkReal(100000.476863863))*(r00)*(sj27)))))), ((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x818)))+(((IkReal(22.7272727272727))*(pz)*(sj28)))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x818)))+(((IkReal(2.27272727272727))*(cj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x819=IKcos(j31);
IkReal x820=IKsin(j31);
IkReal x821=((IkReal(1.00000000000000))*(sj28));
IkReal x822=((IkReal(1.00000000000000))*(sj27));
IkReal x823=((IkReal(1.00000000000000))*(cj28));
IkReal x824=((cj27)*(r00));
IkReal x825=((r01)*(sj27));
IkReal x826=((cj27)*(px));
IkReal x827=((py)*(sj27));
IkReal x828=((IkReal(0.533617455371189))*(x820));
IkReal x829=((IkReal(0.0440000000000000))*(x820));
evalcond[0]=((((cj27)*(py)))+(((cj29)*(x829)))+(((IkReal(-1.00000000000000))*(px)*(x822)))+(((IkReal(0.135000000000000))*(sj29))));
evalcond[1]=((IkReal(-0.400000000000000))+(((cj28)*(x826)))+(((cj28)*(x827)))+(((IkReal(-1.00000000000000))*(pz)*(x821)))+(((IkReal(0.0440000000000000))*(x819)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x821)))+(((cj28)*(x824)))+(((cj28)*(x825)))+(((IkReal(0.533617455371189))*(x819))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(0.845725967038483))*(sj29)))+(((cj29)*(x828)))+(((IkReal(-1.00000000000000))*(r00)*(x822))));
evalcond[4]=((((IkReal(-1.00000000000000))*(pz)*(x823)))+(((IkReal(-1.00000000000000))*(sj29)*(x829)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x821)*(x827)))+(((IkReal(-1.00000000000000))*(x821)*(x826)))+(((IkReal(0.135000000000000))*(cj29))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x823)))+(((IkReal(-1.00000000000000))*(sj29)*(x828)))+(((IkReal(-1.00000000000000))*(x821)*(x825)))+(((IkReal(-1.00000000000000))*(x821)*(x824)))+(((IkReal(0.845725967038483))*(cj29))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x830=((IkReal(1.87400166530233))*(cj28));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(-33.7500000000000))*(sj29))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x830)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x830)))+(((IkReal(1.87400166530233))*(r02)*(sj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(-33.7500000000000))*(sj29)))))))+IKsqr(((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x830)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x830)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(-33.7500000000000))*(sj29)))))), ((((IkReal(-1.00000000000000))*(cj27)*(r00)*(x830)))+(((IkReal(-1.00000000000000))*(r01)*(sj27)*(x830)))+(((IkReal(1.87400166530233))*(r02)*(sj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x831=IKcos(j31);
IkReal x832=IKsin(j31);
IkReal x833=((IkReal(1.00000000000000))*(sj28));
IkReal x834=((IkReal(1.00000000000000))*(sj27));
IkReal x835=((IkReal(1.00000000000000))*(cj28));
IkReal x836=((cj27)*(r00));
IkReal x837=((r01)*(sj27));
IkReal x838=((cj27)*(px));
IkReal x839=((py)*(sj27));
IkReal x840=((IkReal(0.533617455371189))*(x832));
IkReal x841=((IkReal(0.0440000000000000))*(x832));
evalcond[0]=((((cj27)*(py)))+(((cj29)*(x841)))+(((IkReal(-1.00000000000000))*(px)*(x834)))+(((IkReal(0.135000000000000))*(sj29))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x833)))+(((cj28)*(x839)))+(((cj28)*(x838)))+(((IkReal(0.0440000000000000))*(x831)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x833)))+(((IkReal(0.533617455371189))*(x831)))+(((cj28)*(x837)))+(((cj28)*(x836))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(0.845725967038483))*(sj29)))+(((cj29)*(x840)))+(((IkReal(-1.00000000000000))*(r00)*(x834))));
evalcond[4]=((((IkReal(-1.00000000000000))*(sj29)*(x841)))+(((IkReal(-1.00000000000000))*(pz)*(x835)))+(((IkReal(-1.00000000000000))*(x833)*(x838)))+(((IkReal(-1.00000000000000))*(x833)*(x839)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(0.135000000000000))*(cj29))));
evalcond[5]=((((IkReal(-1.00000000000000))*(sj29)*(x840)))+(((IkReal(-1.00000000000000))*(r02)*(x835)))+(((IkReal(-1.00000000000000))*(x833)*(x837)))+(((IkReal(-1.00000000000000))*(x833)*(x836)))+(((IkReal(0.845725967038483))*(cj29))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x842=((IkReal(22.7272727272727))*(cj28));
if( IKabs(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(-33.7500000000000))*(sj29))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x842)))+(((IkReal(22.7272727272727))*(pz)*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x842)))+(((IkReal(2.27272727272727))*(cj28))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(-33.7500000000000))*(sj29)))))))+IKsqr(((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x842)))+(((IkReal(22.7272727272727))*(pz)*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x842)))+(((IkReal(2.27272727272727))*(cj28)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(0.0909090909090909))*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(250.000000000000))*(px)*(sj27)))+(((IkReal(-250.000000000000))*(cj27)*(py)))+(((IkReal(-33.7500000000000))*(sj29)))))), ((IkReal(9.09090909090909))+(((IkReal(-1.00000000000000))*(py)*(sj27)*(x842)))+(((IkReal(22.7272727272727))*(pz)*(sj28)))+(((IkReal(-1.00000000000000))*(cj27)*(px)*(x842)))+(((IkReal(2.27272727272727))*(cj28)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x843=IKcos(j31);
IkReal x844=IKsin(j31);
IkReal x845=((IkReal(1.00000000000000))*(sj28));
IkReal x846=((IkReal(1.00000000000000))*(sj27));
IkReal x847=((IkReal(1.00000000000000))*(cj28));
IkReal x848=((cj27)*(r00));
IkReal x849=((r01)*(sj27));
IkReal x850=((cj27)*(px));
IkReal x851=((py)*(sj27));
IkReal x852=((IkReal(0.533617455371189))*(x844));
IkReal x853=((IkReal(0.0440000000000000))*(x844));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x846)))+(((IkReal(0.135000000000000))*(sj29)))+(((cj29)*(x853))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x845)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(0.0440000000000000))*(x843)))+(((cj28)*(x851)))+(((cj28)*(x850))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x845)))+(((IkReal(0.533617455371189))*(x843)))+(((cj28)*(x848)))+(((cj28)*(x849))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(0.845725967038483))*(sj29)))+(((IkReal(-1.00000000000000))*(r00)*(x846)))+(((cj29)*(x852))));
evalcond[4]=((((IkReal(-1.00000000000000))*(pz)*(x847)))+(((IkReal(-1.00000000000000))*(x845)*(x850)))+(((IkReal(-1.00000000000000))*(x845)*(x851)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(sj29)*(x853)))+(((IkReal(0.135000000000000))*(cj29))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r02)*(x847)))+(((IkReal(-1.00000000000000))*(sj29)*(x852)))+(((IkReal(-1.00000000000000))*(x845)*(x849)))+(((IkReal(-1.00000000000000))*(x845)*(x848)))+(((IkReal(0.845725967038483))*(cj29))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}
}
}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x854=((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30);
IkReal x855=((r02)*(sj28));
IkReal x856=((IkReal(13340500.0000000))*(sj30));
IkReal x857=((r01)*(sj27));
IkReal x858=((cj27)*(r00));
IkReal x859=((IkReal(100000.476863863))*(cj28));
IkReal x860=((IkReal(1100005.24550249))*(cj28)*(cj30)*(sj29));
if( IKabs(((IkReal(1.70362975355292e-6))*(x854)*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(930303.000000000))*(sj29)*((cj30)*(cj30))))+(((IkReal(-1.00000000000000))*(cj27)*(py)*(x856)))+(((px)*(sj27)*(x856)))+(((IkReal(-1.00000000000000))*(x858)*(x860)))+(((IkReal(1800967.50000000))*(sj29)*(((IKabs(x854) != 0)?((IkReal)1/((x854)*(x854))):(IkReal)1.0e30))))+(((IkReal(1100005.24550249))*(cj30)*(sj29)*(x855)))+(((IkReal(-1.00000000000000))*(x857)*(x860))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((IkReal(1.87399272890821e-5))*(x854)*(((((IkReal(-84573.0000000000))*(cj30)))+(((IkReal(-100000.476863863))*(x855)))+(((x858)*(x859)))+(((x857)*(x859))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((IkReal(1.70362975355292e-6))*(x854)*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(930303.000000000))*(sj29)*((cj30)*(cj30))))+(((IkReal(-1.00000000000000))*(cj27)*(py)*(x856)))+(((px)*(sj27)*(x856)))+(((IkReal(-1.00000000000000))*(x858)*(x860)))+(((IkReal(1800967.50000000))*(sj29)*(((IKabs(x854) != 0)?((IkReal)1/((x854)*(x854))):(IkReal)1.0e30))))+(((IkReal(1100005.24550249))*(cj30)*(sj29)*(x855)))+(((IkReal(-1.00000000000000))*(x857)*(x860)))))))+IKsqr(((IkReal(1.87399272890821e-5))*(x854)*(((((IkReal(-84573.0000000000))*(cj30)))+(((IkReal(-100000.476863863))*(x855)))+(((x858)*(x859)))+(((x857)*(x859)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((IkReal(1.70362975355292e-6))*(x854)*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((IkReal(930303.000000000))*(sj29)*((cj30)*(cj30))))+(((IkReal(-1.00000000000000))*(cj27)*(py)*(x856)))+(((px)*(sj27)*(x856)))+(((IkReal(-1.00000000000000))*(x858)*(x860)))+(((IkReal(1800967.50000000))*(sj29)*(((IKabs(x854) != 0)?((IkReal)1/((x854)*(x854))):(IkReal)1.0e30))))+(((IkReal(1100005.24550249))*(cj30)*(sj29)*(x855)))+(((IkReal(-1.00000000000000))*(x857)*(x860)))))), ((IkReal(1.87399272890821e-5))*(x854)*(((((IkReal(-84573.0000000000))*(cj30)))+(((IkReal(-100000.476863863))*(x855)))+(((x858)*(x859)))+(((x857)*(x859)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x861=IKcos(j31);
IkReal x862=IKsin(j31);
IkReal x863=((IkReal(1.00000000000000))*(sj28));
IkReal x864=((IkReal(0.533617455371189))*(cj29));
IkReal x865=((IkReal(0.533617455371189))*(sj29));
IkReal x866=((IkReal(1.00000000000000))*(sj27));
IkReal x867=((IkReal(1.00000000000000))*(cj28));
IkReal x868=((IkReal(0.0440000000000000))*(sj29));
IkReal x869=((cj27)*(r00));
IkReal x870=((IkReal(0.0440000000000000))*(cj29));
IkReal x871=((r01)*(sj27));
IkReal x872=((cj27)*(px));
IkReal x873=((sj29)*(sj30));
IkReal x874=((cj29)*(sj30));
IkReal x875=((py)*(sj27));
IkReal x876=((cj30)*(x861));
IkReal x877=((sj30)*(x861));
evalcond[0]=((((cj27)*(py)))+(((x868)*(x876)))+(((x862)*(x870)))+(((IkReal(-0.135000000000000))*(x873)))+(((IkReal(-1.00000000000000))*(px)*(x866))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x863)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-0.0440000000000000))*(x877)))+(((cj28)*(x875)))+(((cj28)*(x872)))+(((IkReal(-0.100000000000000))*(cj28))));
evalcond[2]=((((IkReal(-0.533617455371189))*(x877)))+(((IkReal(-1.00000000000000))*(r02)*(x863)))+(((cj28)*(x869)))+(((cj28)*(x871)))+(((IkReal(-0.845725967038483))*(cj30))));
evalcond[3]=((((cj27)*(r01)))+(((IkReal(-1.00000000000000))*(r00)*(x866)))+(((IkReal(-0.845725967038483))*(x873)))+(((x862)*(x864)))+(((x865)*(x876))));
evalcond[4]=((((x870)*(x876)))+(((IkReal(-1.00000000000000))*(x862)*(x868)))+(((IkReal(-1.00000000000000))*(pz)*(x867)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-0.135000000000000))*(x874)))+(((IkReal(-1.00000000000000))*(x863)*(x875)))+(((IkReal(-1.00000000000000))*(x863)*(x872))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x862)*(x865)))+(((IkReal(-0.845725967038483))*(x874)))+(((IkReal(-1.00000000000000))*(r02)*(x867)))+(((x864)*(x876)))+(((IkReal(-1.00000000000000))*(x863)*(x871)))+(((IkReal(-1.00000000000000))*(x863)*(x869))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x878=((IKabs(sj30) != 0)?((IkReal)1/(sj30)):(IkReal)1.0e30);
IkReal x879=((cj30)*(sj29));
IkReal x880=((IkReal(25.0000000000000))*(cj28));
IkReal x881=((IkReal(0.0909090909090909))*(x878));
IkReal x882=((IkReal(33.7500000000000))*(sj29));
IkReal x883=((IkReal(250.000000000000))*(py));
IkReal x884=((cj28)*(sj27));
IkReal x885=((IkReal(250.000000000000))*(pz)*(sj28));
IkReal x886=((IkReal(250.000000000000))*(cj27)*(cj28)*(px));
if( IKabs(((x881)*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((x882)*(((IKabs(x878) != 0)?((IkReal)1/((x878)*(x878))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x879)*(x886)))+(((x879)*(x885)))+(((x879)*(x880)))+(((IkReal(-1.00000000000000))*(cj27)*(sj30)*(x883)))+(((IkReal(33.7500000000000))*(cj30)*(x879)))+(((IkReal(-1.00000000000000))*(x879)*(x883)*(x884)))+(((IkReal(250.000000000000))*(px)*(sj27)*(sj30)))+(((IkReal(100.000000000000))*(x879))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((x881)*(((IkReal(-100.000000000000))+(((IkReal(-33.7500000000000))*(cj30)))+(((x883)*(x884)))+(((IkReal(-1.00000000000000))*(x880)))+(((IkReal(-1.00000000000000))*(x885)))+(x886))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((x881)*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((x882)*(((IKabs(x878) != 0)?((IkReal)1/((x878)*(x878))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x879)*(x886)))+(((x879)*(x885)))+(((x879)*(x880)))+(((IkReal(-1.00000000000000))*(cj27)*(sj30)*(x883)))+(((IkReal(33.7500000000000))*(cj30)*(x879)))+(((IkReal(-1.00000000000000))*(x879)*(x883)*(x884)))+(((IkReal(250.000000000000))*(px)*(sj27)*(sj30)))+(((IkReal(100.000000000000))*(x879)))))))+IKsqr(((x881)*(((IkReal(-100.000000000000))+(((IkReal(-33.7500000000000))*(cj30)))+(((x883)*(x884)))+(((IkReal(-1.00000000000000))*(x880)))+(((IkReal(-1.00000000000000))*(x885)))+(x886)))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((x881)*(((IKabs(cj29) != 0)?((IkReal)1/(cj29)):(IkReal)1.0e30))*(((((x882)*(((IKabs(x878) != 0)?((IkReal)1/((x878)*(x878))):(IkReal)1.0e30))))+(((IkReal(-1.00000000000000))*(x879)*(x886)))+(((x879)*(x885)))+(((x879)*(x880)))+(((IkReal(-1.00000000000000))*(cj27)*(sj30)*(x883)))+(((IkReal(33.7500000000000))*(cj30)*(x879)))+(((IkReal(-1.00000000000000))*(x879)*(x883)*(x884)))+(((IkReal(250.000000000000))*(px)*(sj27)*(sj30)))+(((IkReal(100.000000000000))*(x879)))))), ((x881)*(((IkReal(-100.000000000000))+(((IkReal(-33.7500000000000))*(cj30)))+(((x883)*(x884)))+(((IkReal(-1.00000000000000))*(x880)))+(((IkReal(-1.00000000000000))*(x885)))+(x886)))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x887=IKcos(j31);
IkReal x888=IKsin(j31);
IkReal x889=((IkReal(1.00000000000000))*(sj28));
IkReal x890=((IkReal(0.533617455371189))*(cj29));
IkReal x891=((IkReal(0.533617455371189))*(sj29));
IkReal x892=((IkReal(1.00000000000000))*(sj27));
IkReal x893=((IkReal(1.00000000000000))*(cj28));
IkReal x894=((IkReal(0.0440000000000000))*(sj29));
IkReal x895=((cj27)*(r00));
IkReal x896=((IkReal(0.0440000000000000))*(cj29));
IkReal x897=((r01)*(sj27));
IkReal x898=((cj27)*(px));
IkReal x899=((sj29)*(sj30));
IkReal x900=((cj29)*(sj30));
IkReal x901=((py)*(sj27));
IkReal x902=((cj30)*(x887));
IkReal x903=((sj30)*(x887));
evalcond[0]=((((cj27)*(py)))+(((x894)*(x902)))+(((IkReal(-0.135000000000000))*(x899)))+(((IkReal(-1.00000000000000))*(px)*(x892)))+(((x888)*(x896))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-0.0440000000000000))*(x903)))+(((IkReal(-1.00000000000000))*(pz)*(x889)))+(((cj28)*(x898)))+(((IkReal(-0.135000000000000))*(cj30)))+(((IkReal(-0.100000000000000))*(cj28)))+(((cj28)*(x901))));
evalcond[2]=((((cj28)*(x895)))+(((cj28)*(x897)))+(((IkReal(-1.00000000000000))*(r02)*(x889)))+(((IkReal(-0.845725967038483))*(cj30)))+(((IkReal(-0.533617455371189))*(x903))));
evalcond[3]=((((IkReal(-0.845725967038483))*(x899)))+(((cj27)*(r01)))+(((x891)*(x902)))+(((x888)*(x890)))+(((IkReal(-1.00000000000000))*(r00)*(x892))));
evalcond[4]=((((IkReal(-0.135000000000000))*(x900)))+(((IkReal(-1.00000000000000))*(x889)*(x901)))+(((IkReal(-1.00000000000000))*(pz)*(x893)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x889)*(x898)))+(((IkReal(-1.00000000000000))*(x888)*(x894)))+(((x896)*(x902))));
evalcond[5]=((((x890)*(x902)))+(((IkReal(-1.00000000000000))*(x889)*(x897)))+(((IkReal(-1.00000000000000))*(x889)*(x895)))+(((IkReal(-1.00000000000000))*(x888)*(x891)))+(((IkReal(-1.00000000000000))*(r02)*(x893)))+(((IkReal(-0.845725967038483))*(x900))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
{
IkReal j31array[1], cj31array[1], sj31array[1];
bool j31valid[1]={false};
_nj31 = 1;
IkReal x904=(sj29)*(sj29);
IkReal x905=(cj29)*(cj29);
IkReal x906=((sj28)*(sj29));
IkReal x907=((IkReal(33.7500000000000))*(sj30));
IkReal x908=((IkReal(250.000000000000))*(px));
IkReal x909=((cj29)*(sj27));
IkReal x910=((IkReal(250.000000000000))*(py));
IkReal x911=((cj29)*(sj28));
IkReal x912=((cj27)*(x910));
IkReal x913=((IkReal(11.0000000000000))*(x904));
IkReal x914=((IkReal(250.000000000000))*(cj28)*(pz));
IkReal x915=((IkReal(11.0000000000000))*(x905));
if( IKabs(((((IKabs(((x913)+(x915))) != 0)?((IkReal)1/(((x913)+(x915)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj27)*(x906)*(x908)))+(((IkReal(-1.00000000000000))*(sj27)*(x906)*(x910)))+(((IkReal(-1.00000000000000))*(cj29)*(x912)))+(((x908)*(x909)))+(((IkReal(25.0000000000000))*(x906)))+(((IkReal(-1.00000000000000))*(sj29)*(x914))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((((IKabs(((((cj30)*(x915)))+(((cj30)*(x913))))) != 0)?((IkReal)1/(((((cj30)*(x915)))+(((cj30)*(x913)))))):(IkReal)1.0e30))*(((((x905)*(x907)))+(((cj29)*(x914)))+(((sj27)*(sj29)*(x908)))+(((IkReal(-25.0000000000000))*(x911)))+(((cj27)*(x908)*(x911)))+(((sj28)*(x909)*(x910)))+(((IkReal(-1.00000000000000))*(sj29)*(x912)))+(((x904)*(x907))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(IKsqr(((((IKabs(((x913)+(x915))) != 0)?((IkReal)1/(((x913)+(x915)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj27)*(x906)*(x908)))+(((IkReal(-1.00000000000000))*(sj27)*(x906)*(x910)))+(((IkReal(-1.00000000000000))*(cj29)*(x912)))+(((x908)*(x909)))+(((IkReal(25.0000000000000))*(x906)))+(((IkReal(-1.00000000000000))*(sj29)*(x914)))))))+IKsqr(((((IKabs(((((cj30)*(x915)))+(((cj30)*(x913))))) != 0)?((IkReal)1/(((((cj30)*(x915)))+(((cj30)*(x913)))))):(IkReal)1.0e30))*(((((x905)*(x907)))+(((cj29)*(x914)))+(((sj27)*(sj29)*(x908)))+(((IkReal(-25.0000000000000))*(x911)))+(((cj27)*(x908)*(x911)))+(((sj28)*(x909)*(x910)))+(((IkReal(-1.00000000000000))*(sj29)*(x912)))+(((x904)*(x907)))))))-1) <= IKFAST_SINCOS_THRESH )
    continue;
j31array[0]=IKatan2(((((IKabs(((x913)+(x915))) != 0)?((IkReal)1/(((x913)+(x915)))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj27)*(x906)*(x908)))+(((IkReal(-1.00000000000000))*(sj27)*(x906)*(x910)))+(((IkReal(-1.00000000000000))*(cj29)*(x912)))+(((x908)*(x909)))+(((IkReal(25.0000000000000))*(x906)))+(((IkReal(-1.00000000000000))*(sj29)*(x914)))))), ((((IKabs(((((cj30)*(x915)))+(((cj30)*(x913))))) != 0)?((IkReal)1/(((((cj30)*(x915)))+(((cj30)*(x913)))))):(IkReal)1.0e30))*(((((x905)*(x907)))+(((cj29)*(x914)))+(((sj27)*(sj29)*(x908)))+(((IkReal(-25.0000000000000))*(x911)))+(((cj27)*(x908)*(x911)))+(((sj28)*(x909)*(x910)))+(((IkReal(-1.00000000000000))*(sj29)*(x912)))+(((x904)*(x907)))))));
sj31array[0]=IKsin(j31array[0]);
cj31array[0]=IKcos(j31array[0]);
if( j31array[0] > IKPI )
{
    j31array[0]-=IK2PI;
}
else if( j31array[0] < -IKPI )
{    j31array[0]+=IK2PI;
}
j31valid[0] = true;
for(int ij31 = 0; ij31 < 1; ++ij31)
{
if( !j31valid[ij31] )
{
    continue;
}
_ij31[0] = ij31; _ij31[1] = -1;
for(int iij31 = ij31+1; iij31 < 1; ++iij31)
{
if( j31valid[iij31] && IKabs(cj31array[ij31]-cj31array[iij31]) < IKFAST_SOLUTION_THRESH && IKabs(sj31array[ij31]-sj31array[iij31]) < IKFAST_SOLUTION_THRESH )
{
    j31valid[iij31]=false; _ij31[1] = iij31; break; 
}
}
j31 = j31array[ij31]; cj31 = cj31array[ij31]; sj31 = sj31array[ij31];
{
IkReal evalcond[6];
IkReal x916=IKcos(j31);
IkReal x917=IKsin(j31);
IkReal x918=((IkReal(1.00000000000000))*(sj28));
IkReal x919=((IkReal(0.533617455371189))*(cj29));
IkReal x920=((IkReal(0.533617455371189))*(sj29));
IkReal x921=((IkReal(1.00000000000000))*(sj27));
IkReal x922=((IkReal(1.00000000000000))*(cj28));
IkReal x923=((IkReal(0.0440000000000000))*(sj29));
IkReal x924=((cj27)*(r00));
IkReal x925=((IkReal(0.0440000000000000))*(cj29));
IkReal x926=((r01)*(sj27));
IkReal x927=((cj27)*(px));
IkReal x928=((sj29)*(sj30));
IkReal x929=((cj29)*(sj30));
IkReal x930=((py)*(sj27));
IkReal x931=((cj30)*(x916));
IkReal x932=((sj30)*(x916));
evalcond[0]=((((cj27)*(py)))+(((IkReal(-1.00000000000000))*(px)*(x921)))+(((IkReal(-0.135000000000000))*(x928)))+(((x917)*(x925)))+(((x923)*(x931))));
evalcond[1]=((IkReal(-0.400000000000000))+(((IkReal(-1.00000000000000))*(pz)*(x918)))+(((IkReal(-0.135000000000000))*(cj30)))+(((cj28)*(x930)))+(((cj28)*(x927)))+(((IkReal(-0.100000000000000))*(cj28)))+(((IkReal(-0.0440000000000000))*(x932))));
evalcond[2]=((((IkReal(-1.00000000000000))*(r02)*(x918)))+(((cj28)*(x924)))+(((cj28)*(x926)))+(((IkReal(-0.533617455371189))*(x932)))+(((IkReal(-0.845725967038483))*(cj30))));
evalcond[3]=((((x917)*(x919)))+(((cj27)*(r01)))+(((x920)*(x931)))+(((IkReal(-0.845725967038483))*(x928)))+(((IkReal(-1.00000000000000))*(r00)*(x921))));
evalcond[4]=((((IkReal(-0.135000000000000))*(x929)))+(((IkReal(0.100000000000000))*(sj28)))+(((IkReal(-1.00000000000000))*(x918)*(x927)))+(((IkReal(-1.00000000000000))*(x918)*(x930)))+(((IkReal(-1.00000000000000))*(pz)*(x922)))+(((x925)*(x931)))+(((IkReal(-1.00000000000000))*(x917)*(x923))));
evalcond[5]=((((x919)*(x931)))+(((IkReal(-1.00000000000000))*(r02)*(x922)))+(((IkReal(-0.845725967038483))*(x929)))+(((IkReal(-1.00000000000000))*(x918)*(x926)))+(((IkReal(-1.00000000000000))*(x918)*(x924)))+(((IkReal(-1.00000000000000))*(x917)*(x920))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(5);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j27;
vinfos[0].indices[0] = _ij27[0];
vinfos[0].indices[1] = _ij27[1];
vinfos[0].maxsolutions = _nj27;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j28;
vinfos[1].indices[0] = _ij28[0];
vinfos[1].indices[1] = _ij28[1];
vinfos[1].maxsolutions = _nj28;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j29;
vinfos[2].indices[0] = _ij29[0];
vinfos[2].indices[1] = _ij29[1];
vinfos[2].maxsolutions = _nj29;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j30;
vinfos[3].indices[0] = _ij30[0];
vinfos[3].indices[1] = _ij30[1];
vinfos[3].maxsolutions = _nj30;
vinfos[4].jointtype = 1;
vinfos[4].foffset = j31;
vinfos[4].indices[0] = _ij31[0];
vinfos[4].indices[1] = _ij31[1];
vinfos[4].maxsolutions = _nj31;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}
}
}

}

}
    }
}
return solutions.GetNumSolutions()>0;
}
static inline void polyroots8(IkReal rawcoeffs[8+1], IkReal rawroots[8], int& numroots)
{
    using std::complex;
    IKFAST_ASSERT(rawcoeffs[0] != 0);
    const IkReal tol = 128.0*std::numeric_limits<IkReal>::epsilon();
    const IkReal tolsqrt = sqrt(std::numeric_limits<IkReal>::epsilon());
    complex<IkReal> coeffs[8];
    const int maxsteps = 110;
    for(int i = 0; i < 8; ++i) {
        coeffs[i] = complex<IkReal>(rawcoeffs[i+1]/rawcoeffs[0]);
    }
    complex<IkReal> roots[8];
    IkReal err[8];
    roots[0] = complex<IkReal>(1,0);
    roots[1] = complex<IkReal>(0.4,0.9); // any complex number not a root of unity works
    err[0] = 1.0;
    err[1] = 1.0;
    for(int i = 2; i < 8; ++i) {
        roots[i] = roots[i-1]*roots[1];
        err[i] = 1.0;
    }
    for(int step = 0; step < maxsteps; ++step) {
        bool changed = false;
        for(int i = 0; i < 8; ++i) {
            if ( err[i] >= tol ) {
                changed = true;
                // evaluate
                complex<IkReal> x = roots[i] + coeffs[0];
                for(int j = 1; j < 8; ++j) {
                    x = roots[i] * x + coeffs[j];
                }
                for(int j = 0; j < 8; ++j) {
                    if( i != j ) {
                        if( roots[i] != roots[j] ) {
                            x /= (roots[i] - roots[j]);
                        }
                    }
                }
                roots[i] -= x;
                err[i] = abs(x);
            }
        }
        if( !changed ) {
            break;
        }
    }

    numroots = 0;
    bool visited[8] = {false};
    for(int i = 0; i < 8; ++i) {
        if( !visited[i] ) {
            // might be a multiple root, in which case it will have more error than the other roots
            // find any neighboring roots, and take the average
            complex<IkReal> newroot=roots[i];
            int n = 1;
            for(int j = i+1; j < 8; ++j) {
                if( abs(roots[i]-roots[j]) < 8*tolsqrt ) {
                    newroot += roots[j];
                    n += 1;
                    visited[j] = true;
                }
            }
            if( n > 1 ) {
                newroot /= n;
            }
            // there are still cases where even the mean is not accurate enough, until a better multi-root algorithm is used, need to use the sqrt
            if( IKabs(imag(newroot)) < tolsqrt ) {
                rawroots[numroots++] = real(newroot);
            }
        }
    }
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "ceb6be51bd14f345e22997cc0bca9f2f"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
