/// autogenerated analytical inverse kinematics code from ikfast program part of OpenRAVE
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 60 generated on 2012-09-02 09:24:11.712556
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object (might need -llapack):
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -DIKFAST_CLIBRARY -shared -Wl,-soname,libik.so -o libik.so ik.cpp
#define IKFAST_HAS_LIBRARY
#include "ikfast.h" // found inside share/openrave-X.Y/python/ikfast.h
using namespace ikfast;

// check if the included ikfast version matches what this file was compiled with
#define IKFAST_COMPILE_ASSERT(x) extern int __dummy[(int)x]
IKFAST_COMPILE_ASSERT(IKFAST_VERSION==60);

#include <cmath>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IKFAST_STRINGIZE2(s) #s
#define IKFAST_STRINGIZE(s) IKFAST_STRINGIZE2(s)

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>
#include <iostream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }

#endif

#if defined(_MSC_VER)
#define IKFAST_ALIGNED16(x) __declspec(align(16)) x
#else
#define IKFAST_ALIGNED16(x) x __attribute((aligned(16)))
#endif

#define IK2PI  ((IkReal)6.28318530717959)
#define IKPI  ((IkReal)3.14159265358979)
#define IKPI_2  ((IkReal)1.57079632679490)

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// lapack routines
extern "C" {
  void dgetrf_ (const int* m, const int* n, double* a, const int* lda, int* ipiv, int* info);
  void zgetrf_ (const int* m, const int* n, std::complex<double>* a, const int* lda, int* ipiv, int* info);
  void dgetri_(const int* n, const double* a, const int* lda, int* ipiv, double* work, const int* lwork, int* info);
  void dgesv_ (const int* n, const int* nrhs, double* a, const int* lda, int* ipiv, double* b, const int* ldb, int* info);
  void dgetrs_(const char *trans, const int *n, const int *nrhs, double *a, const int *lda, int *ipiv, double *b, const int *ldb, int *info);
  void dgeev_(const char *jobvl, const char *jobvr, const int *n, double *a, const int *lda, double *wr, double *wi,double *vl, const int *ldvl, double *vr, const int *ldvr, double *work, const int *lwork, int *info);
}

using namespace std; // necessary to get std math routines

#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE {
#endif

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKsqr(float f) { return f*f; }
inline double IKsqr(double f) { return f*f; }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

// allows asin and acos to exceed 1
#ifndef IKFAST_SINCOS_THRESH
#define IKFAST_SINCOS_THRESH ((IkReal)0.000001)
#endif

// used to check input to atan2 for degenerate cases
#ifndef IKFAST_ATAN2_MAGTHRESH
#define IKFAST_ATAN2_MAGTHRESH ((IkReal)2e-6)
#endif

// minimum distance of separate solutions
#ifndef IKFAST_SOLUTION_THRESH
#define IKFAST_SOLUTION_THRESH ((IkReal)1e-6)
#endif

inline float IKasin(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(-IKPI_2);
else if( f >= 1 ) return float(IKPI_2);
return asinf(f);
}
inline double IKasin(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline double IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return float(IKPI);
else if( f >= 1 ) return float(0);
return acosf(f);
}
inline double IKacos(double f)
{
IKFAST_ASSERT( f > -1-IKFAST_SINCOS_THRESH && f < 1+IKFAST_SINCOS_THRESH ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKtan(float f) { return tanf(f); }
inline double IKtan(double f) { return tan(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return float(IKPI_2);
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        IKFAST_ASSERT(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) ) {
        return 0;
    }
    return atan2(fy,fx);
}

inline float IKsign(float f) {
    if( f > 0 ) {
        return float(1);
    }
    else if( f < 0 ) {
        return float(-1);
    }
    return 0;
}

inline double IKsign(double f) {
    if( f > 0 ) {
        return 1.0;
    }
    else if( f < 0 ) {
        return -1.0;
    }
    return 0;
}

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void ComputeFk(const IkReal* j, IkReal* eetrans, IkReal* eerot) {

// necessary for local/global translation3d
eerot[0] = eerot[4] = eerot[8] = 0;
IkReal r00 = 0, r11 = 0, r22 = 0;
IkReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22,x23,x24,x25,x26;
x0=IKcos(j[0]);
x1=IKcos(j[1]);
x2=IKsin(j[2]);
x3=IKcos(j[2]);
x4=IKsin(j[0]);
x5=IKsin(j[1]);
x6=IKsin(j[3]);
x7=IKcos(j[3]);
x8=((IkReal(0.350000000000000))+(r22));
x9=((IkReal(0.0450000000000000))*(x6));
x10=((IkReal(1.00000000000000))*(x6));
x11=((IkReal(0.0450000000000000))*(x4));
x12=((IkReal(1.00000000000000))*(x2));
x13=((IkReal(1.00000000000000))*(x3));
x14=((x0)*(x5));
x15=((x0)*(x1));
x16=((x0)*(x2));
x17=((x4)*(x5));
x18=((x1)*(x3));
x19=((x11)*(x2));
x20=((IkReal(0.0450000000000000))*(x16));
x21=((IkReal(0.0450000000000000))*(x3)*(x5));
x22=((IkReal(0.0450000000000000))*(x15)*(x3));
x23=((x11)*(x18));
x24=((((IkReal(-1.00000000000000))*(x12)*(x4)))+(((x15)*(x3))));
x25=((((x18)*(x4)))+(x16));
x26=((x20)+(x23));
eetrans[0]=((IkReal(0.220000000000000))+(((r11)*(((((IkReal(-1.00000000000000))*(x12)*(x15)))+(((IkReal(-1.00000000000000))*(x13)*(x4)))))))+(((IkReal(0.550000000000000))*(x14)))+(((r00)*(((((IkReal(-1.00000000000000))*(x14)*(x6)))+(((x24)*(x7)))))))+(((IkReal(-1.00000000000000))*(x19)))+(((x14)*(x9)))+(((x7)*(((((IkReal(-1.00000000000000))*(x22)))+(x19)))))+(((x8)*(((((x14)*(x7)))+(((x24)*(x6)))))))+(x22));
eetrans[1]=((IkReal(0.140000000000000))+(((IkReal(0.550000000000000))*(x17)))+(((x8)*(((((x17)*(x7)))+(((x25)*(x6)))))))+(((r00)*(((((IkReal(-1.00000000000000))*(x10)*(x17)))+(((x25)*(x7)))))))+(x26)+(((x17)*(x9)))+(((r11)*(((((x0)*(x3)))+(((IkReal(-1.00000000000000))*(x1)*(x12)*(x4)))))))+(((IkReal(-1.00000000000000))*(x26)*(x7))));
IkReal x27=((IkReal(1.00000000000000))*(x5));
eetrans[2]=((IkReal(0.346000000000000))+(((x8)*(((((x1)*(x7)))+(((IkReal(-1.00000000000000))*(x10)*(x27)*(x3)))))))+(((x1)*(x9)))+(((r11)*(x2)*(x5)))+(((x21)*(x7)))+(((IkReal(-1.00000000000000))*(x21)))+(((r00)*(((((IkReal(-1.00000000000000))*(x1)*(x10)))+(((IkReal(-1.00000000000000))*(x13)*(x27)*(x7)))))))+(((IkReal(0.550000000000000))*(x1))));
}

IKFAST_API int GetNumFreeParameters() { return 1; }
IKFAST_API int* GetFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int GetNumJoints() { return 4; }

IKFAST_API int GetIkRealSize() { return sizeof(IkReal); }

IKFAST_API int GetIkType() { return 0x3600000a; }

class IKSolver {
public:
IkReal j0,cj0,sj0,htj0,j1,cj1,sj1,htj1,j3,cj3,sj3,htj3,j2,cj2,sj2,htj2,new_r00,r00,rxp0_0,new_r01,r01,rxp0_1,new_r02,r02,rxp0_2,new_r10,r10,rxp1_0,new_r11,r11,rxp1_1,new_r12,r12,rxp1_2,new_r20,r20,rxp2_0,new_r21,r21,rxp2_1,new_r22,r22,rxp2_2,new_px,px,npx,new_py,py,npy,new_pz,pz,npz,pp;
unsigned char _ij0[2], _nj0,_ij1[2], _nj1,_ij3[2], _nj3,_ij2[2], _nj2;

bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
j0=numeric_limits<IkReal>::quiet_NaN(); _ij0[0] = -1; _ij0[1] = -1; _nj0 = -1; j1=numeric_limits<IkReal>::quiet_NaN(); _ij1[0] = -1; _ij1[1] = -1; _nj1 = -1; j3=numeric_limits<IkReal>::quiet_NaN(); _ij3[0] = -1; _ij3[1] = -1; _nj3 = -1;  _ij2[0] = -1; _ij2[1] = -1; _nj2 = 0; 
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    solutions.Clear();
j2=pfree[0]; cj2=cos(pfree[0]); sj2=sin(pfree[0]);
r00 = eerot[0];
r11 = eerot[4];
r22 = eerot[8];
px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];

new_px=((IkReal(-0.220000000000000))+(px));
new_py=((IkReal(-0.140000000000000))+(py));
new_pz=((IkReal(-0.346000000000000))+(pz));
px = new_px; py = new_py; pz = new_pz;
pp=(((px)*(px))+((py)*(py))+((pz)*(pz)));
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.38361111111111))+(((IkReal(7.77777777777778))*(r22)))+(((IkReal(-1.00000000000000))*(r00)))+(((IkReal(11.1111111111111))*((r00)*(r00))))+(((IkReal(11.1111111111111))*((r22)*(r22)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=((IkReal(1.38361111111111))+(((IkReal(7.77777777777778))*(r22)))+(((IkReal(-1.00000000000000))*(r00)))+(((IkReal(11.1111111111111))*((r00)*(r00))))+(((IkReal(11.1111111111111))*((r22)*(r22)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x28=((IkReal(0.0900000000000000))*(r00));
IkReal x29=((IkReal(0.0810000000000000))+(((IkReal(0.0900000000000000))*(r22)))+(((IkReal(-1.10000000000000))*(r00))));
IkReal x30=((IkReal(0.380950000000000))+(x28)+(((IkReal(1.10000000000000))*(r22))));
if( IKabs(x30) < IKFAST_ATAN2_MAGTHRESH && IKabs(x29) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x31=((IkReal(1.00000000000000))*(IKatan2(x30, x29)));
if( ((((x30)*(x30))+((x29)*(x29)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x30)*(x30))+((x29)*(x29)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x30)*(x30))+((x29)*(x29))))))):(IkReal)1.0e30))*(((IkReal(0.429050000000000))+(((IkReal(-1.00000000000000))*(x28)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x30)*(x30))+((x29)*(x29)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x30)*(x30))+((x29)*(x29))))))):(IkReal)1.0e30))*(((IkReal(0.429050000000000))+(((IkReal(-1.00000000000000))*(x28)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x32=IKasin(((((IKabs(IKabs(IKsqrt((((x30)*(x30))+((x29)*(x29)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x30)*(x30))+((x29)*(x29))))))):(IkReal)1.0e30))*(((IkReal(0.429050000000000))+(((IkReal(-1.00000000000000))*(x28)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))))));
j3array[0]=((((IkReal(-1.00000000000000))*(x32)))+(((IkReal(-1.00000000000000))*(x31))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x31)))+(x32));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x33=(cj2)*(cj2);
IkReal x34=(sj3)*(sj3);
IkReal x35=(cj3)*(cj3);
IkReal x36=(r00)*(r00);
IkReal x37=(r22)*(r22);
IkReal x38=((IkReal(345.679012345679))*(sj3));
IkReal x39=((IkReal(44.4444444444444))*(r00));
IkReal x40=((IkReal(44.4444444444444))*(cj3));
IkReal x41=((r22)*(sj3));
IkReal x42=((IkReal(15.5555555555556))*(sj3));
IkReal x43=((IkReal(345.679012345679))*(r22));
IkReal x44=((cj3)*(r00));
IkReal x45=((cj2)*(r11)*(sj2));
IkReal x46=((IkReal(493.827160493827))*(x37));
IkReal x47=((IkReal(493.827160493827))*(x36));
IkReal x48=((cj3)*(x33));
IkReal x49=((x33)*(x34));
IkReal x50=((x33)*(x35));
dummyeval[0]=((IkReal(149.382716049383))+(((IkReal(543.209876543210))*(cj3)*(r22)))+(((x34)*(x47)))+(((x35)*(x46)))+(((x35)*(x43)))+(((IkReal(-1.00000000000000))*(x42)*(x48)))+(((IkReal(-1.00000000000000))*(x38)*(x45)))+(((IkReal(-1.00000000000000))*(x38)*(x44)))+(((x40)*(x45)))+(((x40)*(x41)))+(((IkReal(-543.209876543210))*(r00)*(sj3)))+(((x43)*(x49)))+(((IkReal(60.4938271604938))*(x49)))+(((x33)*(x38)*(x44)))+(((x46)*(x49)))+(((cj3)*(x42)))+(((IkReal(60.4938271604938))*(x35)))+(((x39)*(x48)))+(((IkReal(24.4444444444444))*(sj3)))+(((IkReal(-1.00000000000000))*(x39)*(x50)))+(((IkReal(44.4444444444444))*(x33)*(x41)))+(((IkReal(190.123456790123))*(cj3)))+(((IkReal(-2.00000000000000))*(x48)))+(x33)+(x34)+(x50)+(((IkReal(493.827160493827))*((r11)*(r11))*((sj2)*(sj2))))+(((IkReal(-1.00000000000000))*(x34)*(x39)))+(((IkReal(-1.00000000000000))*(x33)*(x40)*(x41)))+(((x47)*(x50)))+(((x33)*(x42)))+(((IkReal(-44.4444444444444))*(x45)))+(((IkReal(987.654320987654))*(x33)*(x41)*(x44)))+(((IkReal(-987.654320987654))*(x41)*(x45)))+(((IkReal(-987.654320987654))*(x41)*(x44)))+(((IkReal(-987.654320987654))*(x44)*(x45))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x51=((IkReal(0.0450000000000000))*(cj2));
IkReal x52=((cj2)*(sj3));
IkReal x53=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r22))));
IkReal x54=((((r22)*(x52)))+(x51)+(((IkReal(-1.00000000000000))*(r11)*(sj2)))+(((IkReal(-1.00000000000000))*(cj3)*(x51)))+(((IkReal(0.350000000000000))*(x52)))+(((cj2)*(cj3)*(r00))));
if( IKabs(x53) < IKFAST_ATAN2_MAGTHRESH && IKabs(x54) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x55=((IkReal(1.00000000000000))*(IKatan2(x53, x54)));
if( ((((x54)*(x54))+((x53)*(x53)))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x54)*(x54))+((x53)*(x53)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x54)*(x54))+((x53)*(x53))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x54)*(x54))+((x53)*(x53)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x54)*(x54))+((x53)*(x53))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x56=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x54)*(x54))+((x53)*(x53)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x54)*(x54))+((x53)*(x53))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x55)))+(((IkReal(-1.00000000000000))*(x56))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(x56)+(((IkReal(-1.00000000000000))*(x55))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x57=((IkReal(1.00000000000000))*(cj2)*(sj1));
gconst0=IKsign(((((IkReal(-1.00000000000000))*(x57)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x57)*((py)*(py))))));
IkReal x58=((IkReal(1.00000000000000))*(cj2)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x58)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x58)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x59=((r00)*(sj3));
IkReal x60=((IkReal(0.0900000000000000))*(r00));
IkReal x61=((cj3)*(r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x59)))+(((IkReal(-1.00000000000000))*(x60)))+(((IkReal(1.10000000000000))*(x61)))+(((cj3)*(x60))));
evalcond[2]=((((cj1)*(x59)))+(((IkReal(-0.0450000000000000))*(cj1)*(sj3)))+(((IkReal(-0.550000000000000))*(cj1)))+(((IkReal(-0.350000000000000))*(cj1)*(cj3)))+(pz)+(((IkReal(-1.00000000000000))*(r11)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(x61))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((cj1)*((px)*(px))))+(((cj1)*((py)*(py))))));
dummyeval[0]=((((cj1)*((px)*(px))))+(((cj1)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x62=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(r11));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.00000000000000))*(x62)))+(((IkReal(-1.10000000000000))*(r00)*(sj3)))+(((cj3)*(x62))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r11)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x63=((IkReal(0.0450000000000000))*(px));
IkReal x64=((px)*(sj3));
IkReal x65=((cj3)*(py));
IkReal x66=((IkReal(1.00000000000000))*(r22));
IkReal x67=((py)*(sj3));
IkReal x68=((cj3)*(px));
if( IKabs(((gconst2)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-0.0450000000000000))*(x67)))+(((r22)*(x64)))+(((IkReal(-0.350000000000000))*(x65)))+(((r00)*(x68)))+(((r00)*(x67)))+(x63)+(((IkReal(0.350000000000000))*(x64)))+(((IkReal(-1.00000000000000))*(x65)*(x66)))+(((IkReal(-1.00000000000000))*(cj3)*(x63))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-0.550000000000000))*(px)))+(((IkReal(-1.00000000000000))*(x66)*(x68)))+(((IkReal(-1.00000000000000))*(x66)*(x67)))+(((IkReal(-0.0450000000000000))*(py)))+(((IkReal(0.0450000000000000))*(x65)))+(((IkReal(-1.00000000000000))*(r00)*(x65)))+(((IkReal(-0.350000000000000))*(x67)))+(((IkReal(-0.350000000000000))*(x68)))+(((r00)*(x64)))+(((IkReal(-1.00000000000000))*(sj3)*(x63))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-0.0450000000000000))*(x67)))+(((r22)*(x64)))+(((IkReal(-0.350000000000000))*(x65)))+(((r00)*(x68)))+(((r00)*(x67)))+(x63)+(((IkReal(0.350000000000000))*(x64)))+(((IkReal(-1.00000000000000))*(x65)*(x66)))+(((IkReal(-1.00000000000000))*(cj3)*(x63)))))), ((gconst2)*(((((IkReal(-0.550000000000000))*(px)))+(((IkReal(-1.00000000000000))*(x66)*(x68)))+(((IkReal(-1.00000000000000))*(x66)*(x67)))+(((IkReal(-0.0450000000000000))*(py)))+(((IkReal(0.0450000000000000))*(x65)))+(((IkReal(-1.00000000000000))*(r00)*(x65)))+(((IkReal(-0.350000000000000))*(x67)))+(((IkReal(-0.350000000000000))*(x68)))+(((r00)*(x64)))+(((IkReal(-1.00000000000000))*(sj3)*(x63)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x69=IKsin(j0);
IkReal x70=IKcos(j0);
IkReal x71=((px)*(x69));
IkReal x72=((py)*(x70));
IkReal x73=((px)*(x70));
IkReal x74=((py)*(x69));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)))+(x74)+(x73));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x71)+(((IkReal(-1.00000000000000))*(x72))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x74)))+(((IkReal(1.10000000000000))*(x73)))+(((IkReal(0.0900000000000000))*(x72)))+(((IkReal(-0.0900000000000000))*(x71)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x75=((pz)+(r11));
IkReal x76=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x75;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(-1.00000000000000))*(x76)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((cj3)*(x76)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
evalcond[3]=x75;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x77=((py)*(sj3));
IkReal x78=((IkReal(0.0450000000000000))*(px));
IkReal x79=((cj3)*(r00));
IkReal x80=((IkReal(1.00000000000000))*(px));
IkReal x81=((px)*(sj3));
IkReal x82=((cj3)*(py));
if( IKabs(((gconst3)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(x78)))+(((IkReal(-0.0450000000000000))*(x77)))+(((IkReal(-1.00000000000000))*(r22)*(x82)))+(((IkReal(-1.00000000000000))*(x79)*(x80)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x80)))+(((IkReal(-0.350000000000000))*(x82)))+(((IkReal(-0.350000000000000))*(x81)))+(((cj3)*(x78)))+(((r00)*(x77))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-0.550000000000000))*(px)))+(((py)*(x79)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x80)))+(((r00)*(x81)))+(((IkReal(0.0450000000000000))*(py)))+(((r22)*(x77)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((IkReal(-1.00000000000000))*(sj3)*(x78)))+(((IkReal(-0.0450000000000000))*(x82)))+(((IkReal(0.350000000000000))*(x77))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst3)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(x78)))+(((IkReal(-0.0450000000000000))*(x77)))+(((IkReal(-1.00000000000000))*(r22)*(x82)))+(((IkReal(-1.00000000000000))*(x79)*(x80)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x80)))+(((IkReal(-0.350000000000000))*(x82)))+(((IkReal(-0.350000000000000))*(x81)))+(((cj3)*(x78)))+(((r00)*(x77)))))), ((gconst3)*(((((IkReal(-0.550000000000000))*(px)))+(((py)*(x79)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x80)))+(((r00)*(x81)))+(((IkReal(0.0450000000000000))*(py)))+(((r22)*(x77)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((IkReal(-1.00000000000000))*(sj3)*(x78)))+(((IkReal(-0.0450000000000000))*(x82)))+(((IkReal(0.350000000000000))*(x77)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x83=IKsin(j0);
IkReal x84=IKcos(j0);
IkReal x85=((px)*(x83));
IkReal x86=((IkReal(1.00000000000000))*(x84));
IkReal x87=((py)*(x83));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)))+(((IkReal(-1.00000000000000))*(x87)))+(((IkReal(-1.00000000000000))*(px)*(x86))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(py)*(x86)))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x85));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-1.10000000000000))*(x87)))+(((IkReal(-0.0900000000000000))*(x85)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(px)*(x84)))+(((IkReal(0.0900000000000000))*(py)*(x84)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x88=((cj1)*(px));
IkReal x89=((r22)*(sj3));
IkReal x90=((cj1)*(py));
IkReal x91=((pz)*(sj1));
IkReal x92=((IkReal(1.00000000000000))*(r11));
IkReal x93=((IkReal(0.350000000000000))*(sj3));
IkReal x94=((cj3)*(r00));
IkReal x95=((IkReal(0.0450000000000000))*(cj3));
if( IKabs(((gconst1)*(((((py)*(x91)))+(((IkReal(-1.00000000000000))*(x88)*(x89)))+(((IkReal(-1.00000000000000))*(x88)*(x94)))+(((x88)*(x95)))+(((IkReal(-1.00000000000000))*(x88)*(x93)))+(((IkReal(-0.0450000000000000))*(x88)))+(((IkReal(-1.00000000000000))*(py)*(x92))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((x89)*(x90)))+(((IkReal(-1.00000000000000))*(x90)*(x95)))+(((IkReal(0.0450000000000000))*(x90)))+(((x90)*(x93)))+(((x90)*(x94)))+(((px)*(x91)))+(((IkReal(-1.00000000000000))*(px)*(x92))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((py)*(x91)))+(((IkReal(-1.00000000000000))*(x88)*(x89)))+(((IkReal(-1.00000000000000))*(x88)*(x94)))+(((x88)*(x95)))+(((IkReal(-1.00000000000000))*(x88)*(x93)))+(((IkReal(-0.0450000000000000))*(x88)))+(((IkReal(-1.00000000000000))*(py)*(x92)))))), ((gconst1)*(((((x89)*(x90)))+(((IkReal(-1.00000000000000))*(x90)*(x95)))+(((IkReal(0.0450000000000000))*(x90)))+(((x90)*(x93)))+(((x90)*(x94)))+(((px)*(x91)))+(((IkReal(-1.00000000000000))*(px)*(x92)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x96=IKcos(j0);
IkReal x97=IKsin(j0);
IkReal x98=((r00)*(sj3));
IkReal x99=((IkReal(1.00000000000000))*(sj1));
IkReal x100=((IkReal(0.350000000000000))*(cj3));
IkReal x101=((IkReal(0.0450000000000000))*(sj3));
IkReal x102=((cj3)*(r22));
IkReal x103=((cj1)*(pz));
IkReal x104=((px)*(x97));
IkReal x105=((px)*(x96));
IkReal x106=((py)*(x97));
IkReal x107=((py)*(x96));
IkReal x108=((sj1)*(x106));
evalcond[0]=((IkReal(-0.550000000000000))+(((sj1)*(x105)))+(((IkReal(-1.00000000000000))*(x101)))+(((IkReal(-1.00000000000000))*(x100)))+(x108)+(x103)+(x98)+(((IkReal(-1.00000000000000))*(x102))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)*(x99)))+(((cj1)*(x106)))+(((cj1)*(x105)))+(r11));
evalcond[2]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(x104)+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x107))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x104)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(1.10000000000000))*(sj1)*(x105)))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(x107)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x103)))+(((IkReal(1.10000000000000))*(x108)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((sj1)*(x102)))+(((sj1)*(x101)))+(((sj1)*(x100)))+(((IkReal(-1.00000000000000))*(x98)*(x99)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(r11)))+(((IkReal(-1.00000000000000))*(x106)))+(((IkReal(-1.00000000000000))*(x105))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x109=((r00)*(sj3));
IkReal x110=((IkReal(0.0900000000000000))*(r00));
IkReal x111=((cj3)*(r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((cj3)*(x110)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x110)))+(((IkReal(-1.10000000000000))*(x109)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x111))));
evalcond[2]=((((cj1)*(x109)))+(((IkReal(-0.0450000000000000))*(cj1)*(sj3)))+(((IkReal(-0.550000000000000))*(cj1)))+(((r11)*(sj1)))+(((IkReal(-0.350000000000000))*(cj1)*(cj3)))+(pz)+(((IkReal(-1.00000000000000))*(cj1)*(x111))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((cj1)*((px)*(px))))+(((cj1)*((py)*(py))))));
dummyeval[0]=((((cj1)*((px)*(px))))+(((cj1)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x112=((pz)+(r11));
IkReal x113=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x112;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((cj3)*(x113)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x113)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
evalcond[3]=x112;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x114=((py)*(sj3));
IkReal x115=((IkReal(0.0450000000000000))*(px));
IkReal x116=((cj3)*(r00));
IkReal x117=((IkReal(1.00000000000000))*(px));
IkReal x118=((px)*(sj3));
IkReal x119=((cj3)*(py));
if( IKabs(((gconst5)*(((((IkReal(-0.550000000000000))*(py)))+(((cj3)*(x115)))+(((IkReal(-1.00000000000000))*(x116)*(x117)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x117)))+(((IkReal(-1.00000000000000))*(x115)))+(((r00)*(x114)))+(((IkReal(-0.350000000000000))*(x119)))+(((IkReal(-0.350000000000000))*(x118)))+(((IkReal(-0.0450000000000000))*(x114)))+(((IkReal(-1.00000000000000))*(r22)*(x119))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(-0.550000000000000))*(px)))+(((r22)*(x114)))+(((IkReal(0.350000000000000))*(x114)))+(((py)*(x116)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x117)))+(((r00)*(x118)))+(((IkReal(-0.0450000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(sj3)*(x115))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst5)*(((((IkReal(-0.550000000000000))*(py)))+(((cj3)*(x115)))+(((IkReal(-1.00000000000000))*(x116)*(x117)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x117)))+(((IkReal(-1.00000000000000))*(x115)))+(((r00)*(x114)))+(((IkReal(-0.350000000000000))*(x119)))+(((IkReal(-0.350000000000000))*(x118)))+(((IkReal(-0.0450000000000000))*(x114)))+(((IkReal(-1.00000000000000))*(r22)*(x119)))))), ((gconst5)*(((((IkReal(-0.550000000000000))*(px)))+(((r22)*(x114)))+(((IkReal(0.350000000000000))*(x114)))+(((py)*(x116)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x117)))+(((r00)*(x118)))+(((IkReal(-0.0450000000000000))*(x119)))+(((IkReal(-1.00000000000000))*(sj3)*(x115)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x120=IKsin(j0);
IkReal x121=IKcos(j0);
IkReal x122=((IkReal(1.00000000000000))*(r22));
IkReal x123=((px)*(x120));
IkReal x124=((py)*(x121));
IkReal x125=((px)*(x121));
IkReal x126=((py)*(x120));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(x126)+(x125)+(((IkReal(-1.00000000000000))*(cj3)*(x122))));
evalcond[1]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(sj3)*(x122)))+(x123)+(((IkReal(-0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x124))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(0.0900000000000000))*(x123)))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x124)))+(((IkReal(-0.0900000000000000))*(r00)))+(((IkReal(1.10000000000000))*(x126)))+(((IkReal(1.10000000000000))*(x125))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x127=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(r11));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((cj3)*(x127)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x127)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r11)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x128=((IkReal(0.0450000000000000))*(px));
IkReal x129=((px)*(sj3));
IkReal x130=((cj3)*(py));
IkReal x131=((IkReal(1.00000000000000))*(r22));
IkReal x132=((py)*(sj3));
IkReal x133=((cj3)*(px));
if( IKabs(((gconst6)*(((((IkReal(-0.0450000000000000))*(x132)))+(((IkReal(-0.550000000000000))*(py)))+(((IkReal(0.350000000000000))*(x129)))+(((r22)*(x129)))+(((r00)*(x132)))+(((r00)*(x133)))+(x128)+(((IkReal(-1.00000000000000))*(cj3)*(x128)))+(((IkReal(-0.350000000000000))*(x130)))+(((IkReal(-1.00000000000000))*(x130)*(x131))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(((((IkReal(-0.550000000000000))*(px)))+(((IkReal(-0.0450000000000000))*(py)))+(((IkReal(-1.00000000000000))*(sj3)*(x128)))+(((IkReal(0.0450000000000000))*(x130)))+(((r00)*(x129)))+(((IkReal(-1.00000000000000))*(r00)*(x130)))+(((IkReal(-0.350000000000000))*(x133)))+(((IkReal(-0.350000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x131)*(x132)))+(((IkReal(-1.00000000000000))*(x131)*(x133))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst6)*(((((IkReal(-0.0450000000000000))*(x132)))+(((IkReal(-0.550000000000000))*(py)))+(((IkReal(0.350000000000000))*(x129)))+(((r22)*(x129)))+(((r00)*(x132)))+(((r00)*(x133)))+(x128)+(((IkReal(-1.00000000000000))*(cj3)*(x128)))+(((IkReal(-0.350000000000000))*(x130)))+(((IkReal(-1.00000000000000))*(x130)*(x131)))))), ((gconst6)*(((((IkReal(-0.550000000000000))*(px)))+(((IkReal(-0.0450000000000000))*(py)))+(((IkReal(-1.00000000000000))*(sj3)*(x128)))+(((IkReal(0.0450000000000000))*(x130)))+(((r00)*(x129)))+(((IkReal(-1.00000000000000))*(r00)*(x130)))+(((IkReal(-0.350000000000000))*(x133)))+(((IkReal(-0.350000000000000))*(x132)))+(((IkReal(-1.00000000000000))*(x131)*(x132)))+(((IkReal(-1.00000000000000))*(x131)*(x133)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x134=IKsin(j0);
IkReal x135=IKcos(j0);
IkReal x136=((IkReal(1.00000000000000))*(r22));
IkReal x137=((px)*(x134));
IkReal x138=((IkReal(1.00000000000000))*(x135));
IkReal x139=((py)*(x134));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(px)*(x138)))+(((IkReal(-1.00000000000000))*(cj3)*(x136)))+(((IkReal(-1.00000000000000))*(x139))));
evalcond[1]=((IkReal(-0.0450000000000000))+(((IkReal(-1.00000000000000))*(sj3)*(x136)))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(py)*(x138)))+(x137)+(((IkReal(-0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r00))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(-1.10000000000000))*(px)*(x135)))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(x137)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(py)*(x135)))+(((IkReal(-1.10000000000000))*(x139)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x140=((IkReal(0.0450000000000000))*(cj1));
IkReal x141=((pz)*(sj1));
IkReal x142=((IkReal(1.00000000000000))*(py));
IkReal x143=((IkReal(0.350000000000000))*(cj1)*(sj3));
IkReal x144=((cj1)*(cj3)*(r00));
IkReal x145=((cj1)*(r22)*(sj3));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(px)*(x140)))+(((py)*(r11)))+(((px)*(x140)))+(((px)*(x144)))+(((px)*(x143)))+(((px)*(x145)))+(((py)*(x141))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(x142)*(x144)))+(((IkReal(-1.00000000000000))*(x142)*(x145)))+(((IkReal(-1.00000000000000))*(py)*(x140)))+(((IkReal(-1.00000000000000))*(py)*(x143)))+(((px)*(r11)))+(((cj3)*(py)*(x140)))+(((px)*(x141))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(px)*(x140)))+(((py)*(r11)))+(((px)*(x140)))+(((px)*(x144)))+(((px)*(x143)))+(((px)*(x145)))+(((py)*(x141)))))), ((gconst4)*(((((IkReal(-1.00000000000000))*(x142)*(x144)))+(((IkReal(-1.00000000000000))*(x142)*(x145)))+(((IkReal(-1.00000000000000))*(py)*(x140)))+(((IkReal(-1.00000000000000))*(py)*(x143)))+(((px)*(r11)))+(((cj3)*(py)*(x140)))+(((px)*(x141)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x146=IKcos(j0);
IkReal x147=IKsin(j0);
IkReal x148=((r00)*(sj3));
IkReal x149=((IkReal(0.350000000000000))*(cj3));
IkReal x150=((IkReal(0.0450000000000000))*(sj3));
IkReal x151=((IkReal(1.00000000000000))*(r22));
IkReal x152=((IkReal(1.00000000000000))*(py));
IkReal x153=((cj1)*(pz));
IkReal x154=((px)*(x147));
IkReal x155=((px)*(x146));
IkReal x156=((py)*(sj1)*(x147));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x149)))+(((IkReal(-1.00000000000000))*(x150)))+(((sj1)*(x155)))+(x156)+(x153)+(x148)+(((IkReal(-1.00000000000000))*(cj3)*(x151))));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x155)))+(r11)+(((pz)*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(x147)*(x152))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(x154)+(((IkReal(-0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)))+(((IkReal(-1.00000000000000))*(sj3)*(x151)))+(((IkReal(-1.00000000000000))*(x146)*(x152))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(sj1)*(x155)))+(((IkReal(1.10000000000000))*(x156)))+(((IkReal(1.10000000000000))*(x153)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(0.0900000000000000))*(x154)))+(((IkReal(-0.0900000000000000))*(py)*(x146)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((cj3)*(r22)*(sj1)))+(((sj1)*(x150)))+(((sj1)*(x149)))+(((cj1)*(r11)))+(((IkReal(-1.00000000000000))*(x155)))+(((IkReal(-1.00000000000000))*(x147)*(x152)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(sj1)*(x148))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x157=((r00)*(sj3));
IkReal x158=((IkReal(0.0900000000000000))*(r00));
IkReal x159=((cj3)*(r22));
IkReal x160=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(x157)+(((IkReal(-1.00000000000000))*(x159)))+(pz));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x160;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(1.10000000000000))*(x159)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(x158)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((cj3)*(x158)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x157))));
evalcond[3]=x160;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst7;
IkReal x161=((IkReal(1.00000000000000))*(sj2));
gconst7=IKsign(((((IkReal(-1.00000000000000))*(x161)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x161)*((px)*(px))))));
IkReal x162=((IkReal(1.00000000000000))*(sj2));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x162)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x162)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x163=((r00)*(sj3));
IkReal x164=((IkReal(0.0900000000000000))*(r00));
IkReal x165=((cj3)*(r22));
IkReal x166=x160;
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x166;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(-1.00000000000000))*(x164)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(1.10000000000000))*(x165)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((cj3)*(x164)))+(((IkReal(-1.10000000000000))*(x163))));
evalcond[3]=x166;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst8;
gconst8=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x167=((py)*(sj3));
IkReal x168=((IkReal(0.0450000000000000))*(cj3));
IkReal x169=((px)*(sj3));
IkReal x170=((cj3)*(r00));
if( IKabs(((gconst8)*(((((r22)*(x167)))+(((py)*(x170)))+(((IkReal(-1.00000000000000))*(px)*(r11)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x168)))+(((IkReal(0.350000000000000))*(x167))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst8)*(((((r22)*(x169)))+(((IkReal(0.0450000000000000))*(px)))+(((px)*(x170)))+(((IkReal(-1.00000000000000))*(px)*(x168)))+(((py)*(r11)))+(((IkReal(0.350000000000000))*(x169))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst8)*(((((r22)*(x167)))+(((py)*(x170)))+(((IkReal(-1.00000000000000))*(px)*(r11)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x168)))+(((IkReal(0.350000000000000))*(x167)))))), ((gconst8)*(((((r22)*(x169)))+(((IkReal(0.0450000000000000))*(px)))+(((px)*(x170)))+(((IkReal(-1.00000000000000))*(px)*(x168)))+(((py)*(r11)))+(((IkReal(0.350000000000000))*(x169)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x171=IKcos(j0);
IkReal x172=IKsin(j0);
IkReal x173=((IkReal(1.00000000000000))*(py));
IkReal x174=((px)*(x171));
evalcond[0]=((((px)*(x172)))+(((IkReal(-1.00000000000000))*(x171)*(x173)))+(r11));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x174)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x172)*(x173))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.0900000000000000))*(py)*(x172)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(r00)))+(((IkReal(0.0900000000000000))*(x174))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x175=((r00)*(sj3));
IkReal x176=((IkReal(0.0900000000000000))*(r00));
IkReal x177=((cj3)*(r22));
IkReal x178=x160;
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x178;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(-1.00000000000000))*(x176)))+(((IkReal(1.10000000000000))*(x177)))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((cj3)*(x176)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x175))));
evalcond[3]=x178;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst9;
gconst9=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x179=((IkReal(1.00000000000000))*(py));
IkReal x180=((cj3)*(r00));
IkReal x181=((r22)*(sj3));
IkReal x182=((IkReal(1.00000000000000))*(px));
IkReal x183=((IkReal(0.350000000000000))*(sj3));
IkReal x184=((IkReal(0.0450000000000000))*(py));
IkReal x185=((IkReal(0.0450000000000000))*(px));
if( IKabs(((gconst9)*(((((IkReal(-1.00000000000000))*(py)*(x183)))+(((IkReal(-1.00000000000000))*(x184)))+(((px)*(r11)))+(((cj3)*(x184)))+(((IkReal(-1.00000000000000))*(x179)*(x181)))+(((IkReal(-1.00000000000000))*(x179)*(x180))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst9)*(((((IkReal(-1.00000000000000))*(r11)*(x179)))+(((IkReal(-1.00000000000000))*(x185)))+(((IkReal(-1.00000000000000))*(x180)*(x182)))+(((cj3)*(x185)))+(((IkReal(-1.00000000000000))*(px)*(x183)))+(((IkReal(-1.00000000000000))*(x181)*(x182))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst9)*(((((IkReal(-1.00000000000000))*(py)*(x183)))+(((IkReal(-1.00000000000000))*(x184)))+(((px)*(r11)))+(((cj3)*(x184)))+(((IkReal(-1.00000000000000))*(x179)*(x181)))+(((IkReal(-1.00000000000000))*(x179)*(x180)))))), ((gconst9)*(((((IkReal(-1.00000000000000))*(r11)*(x179)))+(((IkReal(-1.00000000000000))*(x185)))+(((IkReal(-1.00000000000000))*(x180)*(x182)))+(((cj3)*(x185)))+(((IkReal(-1.00000000000000))*(px)*(x183)))+(((IkReal(-1.00000000000000))*(x181)*(x182)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x186=IKsin(j0);
IkReal x187=IKcos(j0);
IkReal x188=((px)*(x187));
IkReal x189=((py)*(x186));
evalcond[0]=((r11)+(((py)*(x187)))+(((IkReal(-1.00000000000000))*(px)*(x186))));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(x188)+(x189)+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x189)))+(((IkReal(-0.0900000000000000))*(x188)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x190=(sj2)*(sj2);
IkReal x191=(cj2)*(cj2);
IkReal x192=((cj3)*(px));
IkReal x193=((py)*(r11));
IkReal x194=((cj2)*(sj2));
IkReal x195=((px)*(r11));
IkReal x196=((IkReal(0.0450000000000000))*(px));
IkReal x197=((IkReal(1.00000000000000))*(x194));
IkReal x198=((py)*(r22)*(sj3));
IkReal x199=((IkReal(0.350000000000000))*(px)*(sj3));
IkReal x200=((cj3)*(py)*(r00));
IkReal x201=((IkReal(1.00000000000000))*(x190));
IkReal x202=((IkReal(0.350000000000000))*(py)*(sj3));
IkReal x203=((px)*(r22)*(sj3));
IkReal x204=((IkReal(1.00000000000000))*(x191));
IkReal x205=((IkReal(0.0450000000000000))*(py)*(x194));
IkReal x206=((IkReal(0.0450000000000000))*(py)*(x190));
if( IKabs(((gconst7)*(((((IkReal(-1.00000000000000))*(x194)*(x202)))+(((cj3)*(x205)))+(((IkReal(-1.00000000000000))*(x197)*(x200)))+(((r00)*(x190)*(x192)))+(((IkReal(-0.0450000000000000))*(x190)*(x192)))+(((x190)*(x203)))+(((IkReal(-1.00000000000000))*(x197)*(x198)))+(x193)+(((IkReal(-1.00000000000000))*(x205)))+(((x194)*(x195)))+(((IkReal(-1.00000000000000))*(x193)*(x204)))+(((x190)*(x199)))+(((x190)*(x196))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst7)*(((((IkReal(-1.00000000000000))*(x195)*(x204)))+(((IkReal(-1.00000000000000))*(x200)*(x201)))+(((IkReal(-1.00000000000000))*(x194)*(x199)))+(((IkReal(-1.00000000000000))*(x194)*(x196)))+(((IkReal(0.0450000000000000))*(x192)*(x194)))+(((cj3)*(x206)))+(((IkReal(-1.00000000000000))*(x197)*(x203)))+(((IkReal(-1.00000000000000))*(r00)*(x192)*(x197)))+(((IkReal(-1.00000000000000))*(x190)*(x202)))+(x195)+(((IkReal(-1.00000000000000))*(x206)))+(((IkReal(-1.00000000000000))*(x198)*(x201)))+(((IkReal(-1.00000000000000))*(x193)*(x197))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst7)*(((((IkReal(-1.00000000000000))*(x194)*(x202)))+(((cj3)*(x205)))+(((IkReal(-1.00000000000000))*(x197)*(x200)))+(((r00)*(x190)*(x192)))+(((IkReal(-0.0450000000000000))*(x190)*(x192)))+(((x190)*(x203)))+(((IkReal(-1.00000000000000))*(x197)*(x198)))+(x193)+(((IkReal(-1.00000000000000))*(x205)))+(((x194)*(x195)))+(((IkReal(-1.00000000000000))*(x193)*(x204)))+(((x190)*(x199)))+(((x190)*(x196)))))), ((gconst7)*(((((IkReal(-1.00000000000000))*(x195)*(x204)))+(((IkReal(-1.00000000000000))*(x200)*(x201)))+(((IkReal(-1.00000000000000))*(x194)*(x199)))+(((IkReal(-1.00000000000000))*(x194)*(x196)))+(((IkReal(0.0450000000000000))*(x192)*(x194)))+(((cj3)*(x206)))+(((IkReal(-1.00000000000000))*(x197)*(x203)))+(((IkReal(-1.00000000000000))*(r00)*(x192)*(x197)))+(((IkReal(-1.00000000000000))*(x190)*(x202)))+(x195)+(((IkReal(-1.00000000000000))*(x206)))+(((IkReal(-1.00000000000000))*(x198)*(x201)))+(((IkReal(-1.00000000000000))*(x193)*(x197)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x207=IKsin(j0);
IkReal x208=IKcos(j0);
IkReal x209=((sj2)*(sj3));
IkReal x210=((IkReal(0.0450000000000000))*(cj2));
IkReal x211=((IkReal(0.0900000000000000))*(py));
IkReal x212=((IkReal(0.0450000000000000))*(cj3));
IkReal x213=((cj3)*(r00));
IkReal x214=((cj2)*(sj3));
IkReal x215=((IkReal(1.00000000000000))*(py));
IkReal x216=((px)*(x207));
IkReal x217=((sj2)*(x208));
IkReal x218=((cj2)*(x207));
IkReal x219=((cj2)*(px)*(x208));
evalcond[0]=((((cj2)*(x216)))+(r11)+(((px)*(x217)))+(((IkReal(-1.00000000000000))*(cj2)*(x208)*(x215)))+(((py)*(sj2)*(x207))));
evalcond[1]=((((r22)*(x209)))+(((IkReal(0.350000000000000))*(x209)))+(((IkReal(0.0450000000000000))*(sj2)))+(((sj2)*(x213)))+(((IkReal(-1.00000000000000))*(sj2)*(x212)))+(((cj2)*(r11)))+(x216)+(((IkReal(-1.00000000000000))*(x208)*(x215))));
evalcond[2]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((sj2)*(x216)))+(((IkReal(0.350000000000000))*(sj3)))+(x213)+(((IkReal(-1.00000000000000))*(x219)))+(((IkReal(-1.00000000000000))*(x212)))+(((IkReal(-1.00000000000000))*(x215)*(x218)))+(((IkReal(-1.00000000000000))*(x215)*(x217))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(sj2)*(x216)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(0.0900000000000000))*(x219)))+((r11)*(r11))+(((x211)*(x218)))+(((x211)*(x217)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj3)*(x210)))+(((IkReal(-1.00000000000000))*(x207)*(x215)))+(((cj2)*(x213)))+(((IkReal(-1.00000000000000))*(px)*(x208)))+(x210)+(((IkReal(-1.00000000000000))*(r11)*(sj2)))+(((r22)*(x214)))+(((IkReal(0.350000000000000))*(x214))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x220=((IkReal(0.0450000000000000))*(sj3));
IkReal x221=((IkReal(0.350000000000000))*(cj3));
IkReal x222=((r00)*(sj3));
IkReal x223=((IkReal(0.0900000000000000))*(r00));
IkReal x224=((cj3)*(r22));
IkReal x225=((x221)+(x220));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(pz)))+(x222)+(((IkReal(-1.00000000000000))*(x225)))+(((IkReal(-1.00000000000000))*(x224))));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((cj3)*(x223)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x224)))+(((IkReal(-1.10000000000000))*(x222)))+(((IkReal(-1.00000000000000))*(x223))));
evalcond[3]=((IkReal(0.550000000000000))+(x225)+(x224)+(pz)+(((IkReal(-1.00000000000000))*(x222))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst10;
gconst10=IKsign(((((sj2)*((px)*(px))))+(((sj2)*((py)*(py))))));
dummyeval[0]=((((sj2)*((px)*(px))))+(((sj2)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x226=((IkReal(0.0450000000000000))*(sj3));
IkReal x227=((IkReal(0.350000000000000))*(cj3));
IkReal x228=((r00)*(sj3));
IkReal x229=((IkReal(0.0900000000000000))*(r00));
IkReal x230=((cj3)*(r22));
IkReal x231=((x227)+(x226));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x231)))+(x228)+(((IkReal(-1.00000000000000))*(x230))));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((cj3)*(x229)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x230)))+(((IkReal(-1.10000000000000))*(x228)))+(((IkReal(-1.00000000000000))*(x229))));
evalcond[3]=((IkReal(0.550000000000000))+(x230)+(x231)+(pz)+(((IkReal(-1.00000000000000))*(x228))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst11;
gconst11=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x232=((py)*(sj3));
IkReal x233=((IkReal(0.0450000000000000))*(px));
IkReal x234=((px)*(sj3));
IkReal x235=((IkReal(0.0450000000000000))*(py));
IkReal x236=((cj3)*(r00));
if( IKabs(((gconst11)*(((((px)*(r11)))+(((IkReal(0.350000000000000))*(x232)))+(x235)+(((py)*(x236)))+(((IkReal(-1.00000000000000))*(cj3)*(x235)))+(((r22)*(x232))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst11)*(((((px)*(x236)))+(((IkReal(0.350000000000000))*(x234)))+(x233)+(((IkReal(-1.00000000000000))*(py)*(r11)))+(((IkReal(-1.00000000000000))*(cj3)*(x233)))+(((r22)*(x234))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst11)*(((((px)*(r11)))+(((IkReal(0.350000000000000))*(x232)))+(x235)+(((py)*(x236)))+(((IkReal(-1.00000000000000))*(cj3)*(x235)))+(((r22)*(x232)))))), ((gconst11)*(((((px)*(x236)))+(((IkReal(0.350000000000000))*(x234)))+(x233)+(((IkReal(-1.00000000000000))*(py)*(r11)))+(((IkReal(-1.00000000000000))*(cj3)*(x233)))+(((r22)*(x234)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x237=IKsin(j0);
IkReal x238=IKcos(j0);
IkReal x239=((px)*(x238));
IkReal x240=((py)*(x237));
evalcond[0]=((((IkReal(-1.00000000000000))*(py)*(x238)))+(((px)*(x237)))+(r11));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(x239)+(x240)+(((cj3)*(r00))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-0.0900000000000000))*(x240)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(x239)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x241=((IkReal(0.0450000000000000))*(sj3));
IkReal x242=((IkReal(0.350000000000000))*(cj3));
IkReal x243=((r00)*(sj3));
IkReal x244=((IkReal(0.0900000000000000))*(r00));
IkReal x245=((cj3)*(r22));
IkReal x246=((x242)+(x241));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x245)))+(((IkReal(-1.00000000000000))*(pz)))+(x243)+(((IkReal(-1.00000000000000))*(x246))));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x245)))+(((IkReal(-1.10000000000000))*(x243)))+(((IkReal(-1.00000000000000))*(x244)))+(((cj3)*(x244))));
evalcond[3]=((IkReal(0.550000000000000))+(((IkReal(-1.00000000000000))*(x243)))+(x246)+(x245)+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst12;
gconst12=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x247=((py)*(sj3));
IkReal x248=((IkReal(0.0450000000000000))*(px));
IkReal x249=((px)*(sj3));
IkReal x250=((IkReal(0.0450000000000000))*(py));
IkReal x251=((cj3)*(r00));
if( IKabs(((gconst12)*(((((px)*(r11)))+(((r22)*(x247)))+(((py)*(x251)))+(x250)+(((IkReal(0.350000000000000))*(x247)))+(((IkReal(-1.00000000000000))*(cj3)*(x250))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst12)*(((((r22)*(x249)))+(((px)*(x251)))+(x248)+(((IkReal(-1.00000000000000))*(cj3)*(x248)))+(((IkReal(0.350000000000000))*(x249)))+(((IkReal(-1.00000000000000))*(py)*(r11))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst12)*(((((px)*(r11)))+(((r22)*(x247)))+(((py)*(x251)))+(x250)+(((IkReal(0.350000000000000))*(x247)))+(((IkReal(-1.00000000000000))*(cj3)*(x250)))))), ((gconst12)*(((((r22)*(x249)))+(((px)*(x251)))+(x248)+(((IkReal(-1.00000000000000))*(cj3)*(x248)))+(((IkReal(0.350000000000000))*(x249)))+(((IkReal(-1.00000000000000))*(py)*(r11)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x252=IKcos(j0);
IkReal x253=IKsin(j0);
IkReal x254=((px)*(x252));
IkReal x255=((IkReal(1.00000000000000))*(x253));
evalcond[0]=((((py)*(x252)))+(r11)+(((IkReal(-1.00000000000000))*(px)*(x255))));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(py)*(x255)))+(((IkReal(-1.00000000000000))*(x254))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(py)*(x253)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(0.0900000000000000))*(x254)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x256=(sj2)*(sj2);
IkReal x257=(cj2)*(cj2);
IkReal x258=((cj2)*(sj2));
IkReal x259=((py)*(r11));
IkReal x260=((IkReal(0.0450000000000000))*(py));
IkReal x261=((IkReal(0.0450000000000000))*(px));
IkReal x262=((IkReal(0.350000000000000))*(sj3));
IkReal x263=((px)*(r11));
IkReal x264=((IkReal(1.00000000000000))*(r22)*(sj3));
IkReal x265=((px)*(x256));
IkReal x266=((cj3)*(py)*(r00));
IkReal x267=((py)*(x256));
IkReal x268=((IkReal(1.00000000000000))*(x257));
IkReal x269=((IkReal(1.00000000000000))*(cj3)*(px)*(r00));
if( IKabs(((gconst10)*(((((IkReal(-1.00000000000000))*(x262)*(x265)))+(((IkReal(-1.00000000000000))*(x258)*(x263)))+(((IkReal(-1.00000000000000))*(x258)*(x266)))+(((IkReal(-1.00000000000000))*(x258)*(x260)))+(((IkReal(-1.00000000000000))*(x259)*(x268)))+(((cj3)*(x258)*(x260)))+(((cj3)*(x256)*(x261)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x265)))+(x259)+(((IkReal(-1.00000000000000))*(x264)*(x265)))+(((IkReal(-1.00000000000000))*(x256)*(x261)))+(((IkReal(-1.00000000000000))*(py)*(x258)*(x262)))+(((IkReal(-1.00000000000000))*(py)*(x258)*(x264))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst10)*(((((IkReal(-1.00000000000000))*(x263)*(x268)))+(((IkReal(-1.00000000000000))*(x258)*(x269)))+(((IkReal(-1.00000000000000))*(x258)*(x261)))+(((x262)*(x267)))+(((cj3)*(x258)*(x261)))+(x263)+(((x256)*(x260)))+(((x256)*(x266)))+(((r22)*(sj3)*(x267)))+(((IkReal(-1.00000000000000))*(px)*(x258)*(x264)))+(((IkReal(-1.00000000000000))*(px)*(x258)*(x262)))+(((x258)*(x259)))+(((IkReal(-1.00000000000000))*(cj3)*(x256)*(x260))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst10)*(((((IkReal(-1.00000000000000))*(x262)*(x265)))+(((IkReal(-1.00000000000000))*(x258)*(x263)))+(((IkReal(-1.00000000000000))*(x258)*(x266)))+(((IkReal(-1.00000000000000))*(x258)*(x260)))+(((IkReal(-1.00000000000000))*(x259)*(x268)))+(((cj3)*(x258)*(x260)))+(((cj3)*(x256)*(x261)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x265)))+(x259)+(((IkReal(-1.00000000000000))*(x264)*(x265)))+(((IkReal(-1.00000000000000))*(x256)*(x261)))+(((IkReal(-1.00000000000000))*(py)*(x258)*(x262)))+(((IkReal(-1.00000000000000))*(py)*(x258)*(x264)))))), ((gconst10)*(((((IkReal(-1.00000000000000))*(x263)*(x268)))+(((IkReal(-1.00000000000000))*(x258)*(x269)))+(((IkReal(-1.00000000000000))*(x258)*(x261)))+(((x262)*(x267)))+(((cj3)*(x258)*(x261)))+(x263)+(((x256)*(x260)))+(((x256)*(x266)))+(((r22)*(sj3)*(x267)))+(((IkReal(-1.00000000000000))*(px)*(x258)*(x264)))+(((IkReal(-1.00000000000000))*(px)*(x258)*(x262)))+(((x258)*(x259)))+(((IkReal(-1.00000000000000))*(cj3)*(x256)*(x260)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x270=IKcos(j0);
IkReal x271=IKsin(j0);
IkReal x272=((sj2)*(sj3));
IkReal x273=((IkReal(0.0450000000000000))*(cj2));
IkReal x274=((IkReal(0.0900000000000000))*(sj2));
IkReal x275=((IkReal(0.0450000000000000))*(cj3));
IkReal x276=((IkReal(1.00000000000000))*(sj2));
IkReal x277=((IkReal(0.0900000000000000))*(cj2));
IkReal x278=((cj3)*(r00));
IkReal x279=((IkReal(1.00000000000000))*(cj2));
IkReal x280=((r22)*(sj3));
IkReal x281=((IkReal(0.350000000000000))*(sj3));
IkReal x282=((px)*(x271));
IkReal x283=((py)*(x270));
IkReal x284=((px)*(x270));
IkReal x285=((py)*(x271));
evalcond[0]=((((IkReal(-1.00000000000000))*(x276)*(x285)))+(((IkReal(-1.00000000000000))*(x276)*(x284)))+(((cj2)*(x282)))+(r11)+(((IkReal(-1.00000000000000))*(x279)*(x283))));
evalcond[1]=((((IkReal(-1.00000000000000))*(sj2)*(x275)))+(((sj2)*(x278)))+(((r22)*(x272)))+(((IkReal(0.0450000000000000))*(sj2)))+(((cj2)*(r11)))+(x282)+(((IkReal(0.350000000000000))*(x272)))+(((IkReal(-1.00000000000000))*(x283))));
evalcond[2]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x276)*(x283)))+(((sj2)*(x282)))+(((cj2)*(x285)))+(((cj2)*(x284)))+(x278)+(x281)+(x280)+(((IkReal(-1.00000000000000))*(x275))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(x277)*(x285)))+(((IkReal(-1.00000000000000))*(x277)*(x284)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((x274)*(x283)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(r00)))+(((IkReal(-1.00000000000000))*(x274)*(x282))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj2)*(x281)))+(((r11)*(sj2)))+(((IkReal(-1.00000000000000))*(x285)))+(((IkReal(-1.00000000000000))*(x284)))+(((IkReal(-1.00000000000000))*(x278)*(x279)))+(((IkReal(-1.00000000000000))*(x273)))+(((IkReal(-1.00000000000000))*(x279)*(x280)))+(((cj3)*(x273))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x286=(sj1)*(sj1);
IkReal x287=(cj1)*(cj1);
IkReal x288=((IkReal(0.550000000000000))*(cj2));
IkReal x289=((IkReal(1.00000000000000))*(px));
IkReal x290=((cj2)*(cj3));
IkReal x291=((cj1)*(sj2));
IkReal x292=((IkReal(0.350000000000000))*(py));
IkReal x293=((r11)*(sj1));
IkReal x294=((IkReal(1.00000000000000))*(py));
IkReal x295=((r00)*(sj3));
IkReal x296=((IkReal(0.350000000000000))*(px));
IkReal x297=((cj3)*(r22));
IkReal x298=((IkReal(0.0450000000000000))*(px)*(sj3));
IkReal x299=((cj1)*(cj2)*(pz));
IkReal x300=((IkReal(0.0450000000000000))*(py)*(sj3));
IkReal x301=((pz)*(sj2)*(x287));
IkReal x302=((pz)*(sj2)*(x286));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(x289)*(x291)*(x295)))+(((IkReal(-1.00000000000000))*(cj2)*(x300)))+(((x291)*(x298)))+(((cj2)*(py)*(x295)))+(((IkReal(-1.00000000000000))*(x290)*(x292)))+(((px)*(x291)*(x297)))+(((py)*(x299)))+(((px)*(x293)))+(((cj3)*(x291)*(x296)))+(((IkReal(0.550000000000000))*(px)*(x291)))+(((IkReal(-1.00000000000000))*(py)*(x288)))+(((IkReal(-1.00000000000000))*(x289)*(x301)))+(((IkReal(-1.00000000000000))*(x289)*(x302)))+(((IkReal(-1.00000000000000))*(r22)*(x290)*(x294))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((py)*(x291)*(x295)))+(((IkReal(-1.00000000000000))*(x290)*(x296)))+(((IkReal(-1.00000000000000))*(px)*(x288)))+(((IkReal(-1.00000000000000))*(r22)*(x289)*(x290)))+(((IkReal(-1.00000000000000))*(x291)*(x294)*(x297)))+(((IkReal(-1.00000000000000))*(x293)*(x294)))+(((px)*(x299)))+(((cj2)*(px)*(x295)))+(((IkReal(-1.00000000000000))*(cj2)*(x298)))+(((IkReal(-1.00000000000000))*(cj3)*(x291)*(x292)))+(((IkReal(-0.550000000000000))*(py)*(x291)))+(((py)*(x301)))+(((py)*(x302)))+(((IkReal(-1.00000000000000))*(x291)*(x300))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(x289)*(x291)*(x295)))+(((IkReal(-1.00000000000000))*(cj2)*(x300)))+(((x291)*(x298)))+(((cj2)*(py)*(x295)))+(((IkReal(-1.00000000000000))*(x290)*(x292)))+(((px)*(x291)*(x297)))+(((py)*(x299)))+(((px)*(x293)))+(((cj3)*(x291)*(x296)))+(((IkReal(0.550000000000000))*(px)*(x291)))+(((IkReal(-1.00000000000000))*(py)*(x288)))+(((IkReal(-1.00000000000000))*(x289)*(x301)))+(((IkReal(-1.00000000000000))*(x289)*(x302)))+(((IkReal(-1.00000000000000))*(r22)*(x290)*(x294)))))), ((gconst0)*(((((py)*(x291)*(x295)))+(((IkReal(-1.00000000000000))*(x290)*(x296)))+(((IkReal(-1.00000000000000))*(px)*(x288)))+(((IkReal(-1.00000000000000))*(r22)*(x289)*(x290)))+(((IkReal(-1.00000000000000))*(x291)*(x294)*(x297)))+(((IkReal(-1.00000000000000))*(x293)*(x294)))+(((px)*(x299)))+(((cj2)*(px)*(x295)))+(((IkReal(-1.00000000000000))*(cj2)*(x298)))+(((IkReal(-1.00000000000000))*(cj3)*(x291)*(x292)))+(((IkReal(-0.550000000000000))*(py)*(x291)))+(((py)*(x301)))+(((py)*(x302)))+(((IkReal(-1.00000000000000))*(x291)*(x300)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[6];
IkReal x303=IKcos(j0);
IkReal x304=IKsin(j0);
IkReal x305=((sj2)*(sj3));
IkReal x306=((cj1)*(px));
IkReal x307=((IkReal(0.0900000000000000))*(py));
IkReal x308=((IkReal(0.0450000000000000))*(cj3));
IkReal x309=((IkReal(0.350000000000000))*(cj3));
IkReal x310=((IkReal(1.10000000000000))*(sj1));
IkReal x311=((r00)*(sj3));
IkReal x312=((IkReal(1.00000000000000))*(sj1));
IkReal x313=((cj1)*(sj2));
IkReal x314=((cj3)*(r00));
IkReal x315=((cj1)*(cj2));
IkReal x316=((IkReal(1.00000000000000))*(py));
IkReal x317=((IkReal(0.0450000000000000))*(sj3));
IkReal x318=((cj3)*(r22));
IkReal x319=((r22)*(sj3));
IkReal x320=((IkReal(0.350000000000000))*(sj3));
IkReal x321=((cj1)*(pz));
IkReal x322=((px)*(x304));
IkReal x323=((sj2)*(x303));
IkReal x324=((px)*(x303));
IkReal x325=((py)*(x304));
IkReal x326=((cj2)*(x303));
IkReal x327=((cj2)*(pz)*(sj1));
evalcond[0]=((IkReal(-0.550000000000000))+(x321)+(x311)+(((IkReal(-1.00000000000000))*(x317)))+(((sj1)*(x325)))+(((sj1)*(x324)))+(((IkReal(-1.00000000000000))*(x318)))+(((IkReal(-1.00000000000000))*(x309))));
evalcond[1]=((((cj2)*(x322)))+(((x313)*(x325)))+(((IkReal(-1.00000000000000))*(x316)*(x326)))+(r11)+(((x306)*(x323)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x312))));
evalcond[2]=((((IkReal(0.350000000000000))*(x305)))+(((r22)*(x305)))+(((IkReal(-1.00000000000000))*(sj2)*(x308)))+(((IkReal(-1.00000000000000))*(x303)*(x316)))+(((IkReal(0.0450000000000000))*(sj2)))+(x322)+(((cj2)*(r11)))+(((sj2)*(x314))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x306)*(x326)))+(x327)+(x320)+(x319)+(x314)+(((sj2)*(x322)))+(((IkReal(-1.00000000000000))*(x316)*(x323)))+(((IkReal(-1.00000000000000))*(x304)*(x315)*(x316)))+(((IkReal(-1.00000000000000))*(x308))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x327)))+(((IkReal(1.10000000000000))*(x321)))+(((x304)*(x307)*(x315)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(0.0900000000000000))*(x306)*(x326)))+((r22)*(r22))+(((x310)*(x325)))+(((x310)*(x324)))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(sj2)*(x322)))+(((x307)*(x323)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[5]=((((IkReal(-1.00000000000000))*(r11)*(x313)))+(((IkReal(-1.00000000000000))*(x324)))+(((x315)*(x319)))+(((sj1)*(x317)))+(((sj1)*(x318)))+(((IkReal(-1.00000000000000))*(x308)*(x315)))+(((IkReal(-1.00000000000000))*(x304)*(x316)))+(((x315)*(x320)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(0.0450000000000000))*(x315)))+(((x314)*(x315)))+(((sj1)*(x309)))+(((IkReal(-1.00000000000000))*(x311)*(x312))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x328=((sj2)*(sj3));
IkReal x329=((IkReal(0.0450000000000000))*(sj2));
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x330=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x329)))+(x329)+(((cj2)*(r11)))+(((r22)*(x328)))+(((cj3)*(r00)*(sj2)))+(((IkReal(0.350000000000000))*(x328))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x329)))+(x329)+(((cj2)*(r11)))+(((r22)*(x328)))+(((cj3)*(r00)*(sj2)))+(((IkReal(0.350000000000000))*(x328))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x331=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x329)))+(x329)+(((cj2)*(r11)))+(((r22)*(x328)))+(((cj3)*(r00)*(sj2)))+(((IkReal(0.350000000000000))*(x328)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x331)))+(((IkReal(-1.00000000000000))*(x330))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(x331)+(((IkReal(-1.00000000000000))*(x330))));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
IkReal x332=(py)*(py);
IkReal x333=(cj2)*(cj2);
IkReal x334=(sj2)*(sj2);
IkReal x335=IKcos(j0);
IkReal x336=IKsin(j0);
IkReal x337=(px)*(px);
IkReal x338=((r22)*(sj3));
IkReal x339=((cj3)*(r00));
IkReal x340=((IkReal(0.0450000000000000))*(sj2));
IkReal x341=((px)*(sj2));
IkReal x342=((IkReal(0.350000000000000))*(sj3));
IkReal x343=((IkReal(1.00000000000000))*(cj2)*(r11));
IkReal x344=((IkReal(1.00000000000000))*(py)*(sj2));
IkReal x345=((IkReal(1.00000000000000))*(x337));
IkReal x346=((px)*(py)*(x334));
IkReal x347=((px)*(py)*(x333));
evalcond[0]=((((cj3)*(py)*(x340)))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x342)))+(((x336)*(((((IkReal(-1.00000000000000))*(x346)))+(((IkReal(-1.00000000000000))*(x347)))))))+(((x335)*(((((x332)*(x333)))+(((x332)*(x334)))))))+(((IkReal(-1.00000000000000))*(x338)*(x344)))+(((IkReal(-1.00000000000000))*(py)*(x343)))+(((IkReal(-1.00000000000000))*(py)*(x340)))+(((IkReal(-1.00000000000000))*(x339)*(x344))));
evalcond[1]=((((IkReal(-1.00000000000000))*(px)*(x340)))+(((IkReal(-1.00000000000000))*(px)*(x343)))+(((x335)*(((x346)+(x347)))))+(((x336)*(((((IkReal(-1.00000000000000))*(x334)*(x345)))+(((IkReal(-1.00000000000000))*(x333)*(x345)))))))+(((IkReal(-1.00000000000000))*(x338)*(x341)))+(((IkReal(-1.00000000000000))*(x339)*(x341)))+(((IkReal(-1.00000000000000))*(x341)*(x342)))+(((cj3)*(px)*(x340))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst13;
gconst13=IKsign(((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x348=((IkReal(0.0900000000000000))*(r00));
IkReal x349=((r11)+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x349;
evalcond[2]=x349;
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.00000000000000))*(x348)))+(((IkReal(-1.10000000000000))*(r00)*(sj3)))+(((cj3)*(x348))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst14;
gconst14=IKsign(((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x350=((pz)*(sj3));
IkReal x351=((py)*(sj0));
IkReal x352=((IkReal(1.00000000000000))*(cj3));
IkReal x353=((cj0)*(px));
IkReal x354=((r00)*(sj3));
IkReal x355=((IkReal(0.350000000000000))*(cj3));
IkReal x356=((IkReal(0.350000000000000))*(sj3));
IkReal x357=((IkReal(0.0450000000000000))*(sj3));
IkReal x358=((IkReal(0.0450000000000000))*(pz));
IkReal x359=((IkReal(0.0450000000000000))*(cj3));
IkReal x360=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst14)*(((((cj3)*(pz)*(r00)))+(((IkReal(-0.550000000000000))*(x351)))+(((IkReal(-0.550000000000000))*(x353)))+(((IkReal(-1.00000000000000))*(cj3)*(x358)))+(((IkReal(-1.00000000000000))*(r22)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(r22)*(x352)*(x353)))+(((x353)*(x354)))+(((x351)*(x354)))+(x358)+(((IkReal(0.350000000000000))*(x350)))+(((r22)*(x350)))+(((IkReal(-1.00000000000000))*(x353)*(x357)))+(((IkReal(-1.00000000000000))*(x353)*(x355)))+(((IkReal(-1.00000000000000))*(x351)*(x357)))+(((IkReal(-1.00000000000000))*(x351)*(x355))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x353)*(x360)))+(((IkReal(-0.0450000000000000))*(x351)))+(((IkReal(-0.0450000000000000))*(x350)))+(((IkReal(-0.0450000000000000))*(x353)))+(((r00)*(x350)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x352)))+(((x353)*(x359)))+(((x351)*(x359)))+(((IkReal(-1.00000000000000))*(r00)*(x352)*(x353)))+(((IkReal(-1.00000000000000))*(r00)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x353)*(x356)))+(((IkReal(-1.00000000000000))*(x351)*(x356)))+(((IkReal(-1.00000000000000))*(pz)*(x355)))+(((IkReal(-1.00000000000000))*(x351)*(x360))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst14)*(((((cj3)*(pz)*(r00)))+(((IkReal(-0.550000000000000))*(x351)))+(((IkReal(-0.550000000000000))*(x353)))+(((IkReal(-1.00000000000000))*(cj3)*(x358)))+(((IkReal(-1.00000000000000))*(r22)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(r22)*(x352)*(x353)))+(((x353)*(x354)))+(((x351)*(x354)))+(x358)+(((IkReal(0.350000000000000))*(x350)))+(((r22)*(x350)))+(((IkReal(-1.00000000000000))*(x353)*(x357)))+(((IkReal(-1.00000000000000))*(x353)*(x355)))+(((IkReal(-1.00000000000000))*(x351)*(x357)))+(((IkReal(-1.00000000000000))*(x351)*(x355)))))), ((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x353)*(x360)))+(((IkReal(-0.0450000000000000))*(x351)))+(((IkReal(-0.0450000000000000))*(x350)))+(((IkReal(-0.0450000000000000))*(x353)))+(((r00)*(x350)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x352)))+(((x353)*(x359)))+(((x351)*(x359)))+(((IkReal(-1.00000000000000))*(r00)*(x352)*(x353)))+(((IkReal(-1.00000000000000))*(r00)*(x351)*(x352)))+(((IkReal(-1.00000000000000))*(x353)*(x356)))+(((IkReal(-1.00000000000000))*(x351)*(x356)))+(((IkReal(-1.00000000000000))*(pz)*(x355)))+(((IkReal(-1.00000000000000))*(x351)*(x360)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x361=IKsin(j1);
IkReal x362=IKcos(j1);
IkReal x363=((r22)*(sj3));
IkReal x364=((IkReal(0.0450000000000000))*(cj3));
IkReal x365=((cj0)*(px));
IkReal x366=((cj3)*(r22));
IkReal x367=((r00)*(sj3));
IkReal x368=((py)*(sj0));
IkReal x369=((IkReal(0.350000000000000))*(sj3));
IkReal x370=((IkReal(0.350000000000000))*(cj3));
IkReal x371=((IkReal(0.0450000000000000))*(sj3));
IkReal x372=((cj3)*(r00));
IkReal x373=((IkReal(1.00000000000000))*(x362));
IkReal x374=((sj3)*(x362));
IkReal x375=((IkReal(0.0900000000000000))*(x362));
IkReal x376=((pz)*(x361));
IkReal x377=((pz)*(x362));
IkReal x378=((IkReal(1.10000000000000))*(x361));
evalcond[0]=((IkReal(-0.550000000000000))+(((x361)*(x368)))+(((x361)*(x365)))+(((IkReal(-1.00000000000000))*(x366)))+(((IkReal(-1.00000000000000))*(x370)))+(((IkReal(-1.00000000000000))*(x371)))+(x377)+(x367));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x368)*(x373)))+(((IkReal(-1.00000000000000))*(x364)))+(x376)+(x372)+(x369)+(x363)+(((IkReal(-1.00000000000000))*(x365)*(x373))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((x365)*(x375)))+(((x365)*(x378)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x376)))+(((IkReal(1.10000000000000))*(x377)))+(((x368)*(x375)))+(((x368)*(x378)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((x361)*(x363)))+(((x361)*(x369)))+(((IkReal(-1.00000000000000))*(x362)*(x371)))+(((IkReal(-1.00000000000000))*(x362)*(x370)))+(((IkReal(0.0450000000000000))*(x361)))+(((IkReal(-1.00000000000000))*(x361)*(x364)))+(((IkReal(-1.00000000000000))*(x366)*(x373)))+(((IkReal(-0.550000000000000))*(x362)))+(pz)+(((x362)*(x367)))+(((x361)*(x372))));
evalcond[4]=((((x361)*(x366)))+(((IkReal(0.0450000000000000))*(x362)))+(((IkReal(-1.00000000000000))*(x368)))+(((IkReal(-1.00000000000000))*(x365)))+(((IkReal(-1.00000000000000))*(x362)*(x364)))+(((IkReal(-1.00000000000000))*(x361)*(x367)))+(((IkReal(0.550000000000000))*(x361)))+(((x362)*(x369)))+(((x362)*(x363)))+(((x361)*(x370)))+(((x361)*(x371)))+(((x362)*(x372))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x379=((IkReal(0.0900000000000000))*(r00));
IkReal x380=((px)*(sj0));
IkReal x381=((cj0)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((x381)+(((IkReal(-1.00000000000000))*(x380)))+(r11));
evalcond[2]=((((IkReal(-1.00000000000000))*(r11)))+(x380)+(((IkReal(-1.00000000000000))*(x381))));
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x379)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((cj3)*(x379)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst15;
gconst15=IKsign((((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)))));
dummyeval[0]=(((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x382=((pz)*(sj3));
IkReal x383=((py)*(sj0));
IkReal x384=((IkReal(1.00000000000000))*(r00));
IkReal x385=((cj0)*(px));
IkReal x386=((IkReal(0.350000000000000))*(cj3));
IkReal x387=((IkReal(0.350000000000000))*(sj3));
IkReal x388=((IkReal(0.0450000000000000))*(sj3));
IkReal x389=((IkReal(0.0450000000000000))*(cj3));
IkReal x390=((cj3)*(r22));
IkReal x391=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst15)*(((((r22)*(x382)))+(((IkReal(0.550000000000000))*(x385)))+(((IkReal(0.550000000000000))*(x383)))+(((x385)*(x390)))+(((IkReal(0.350000000000000))*(x382)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(sj3)*(x383)*(x384)))+(((IkReal(0.0450000000000000))*(pz)))+(((x383)*(x386)))+(((x383)*(x388)))+(((x385)*(x388)))+(((x385)*(x386)))+(((IkReal(-1.00000000000000))*(sj3)*(x384)*(x385)))+(((x383)*(x390)))+(((IkReal(-1.00000000000000))*(pz)*(x389))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst15)*(((((IkReal(-1.00000000000000))*(cj3)*(x383)*(x384)))+(((IkReal(-1.00000000000000))*(x385)*(x391)))+(((IkReal(-1.00000000000000))*(cj3)*(x384)*(x385)))+(((pz)*(x390)))+(((x383)*(x389)))+(((IkReal(0.0450000000000000))*(x382)))+(((IkReal(-1.00000000000000))*(x383)*(x391)))+(((x385)*(x389)))+(((IkReal(-0.0450000000000000))*(x383)))+(((IkReal(-0.0450000000000000))*(x385)))+(((pz)*(x386)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x382)*(x384)))+(((IkReal(-1.00000000000000))*(x383)*(x387))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst15)*(((((r22)*(x382)))+(((IkReal(0.550000000000000))*(x385)))+(((IkReal(0.550000000000000))*(x383)))+(((x385)*(x390)))+(((IkReal(0.350000000000000))*(x382)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(sj3)*(x383)*(x384)))+(((IkReal(0.0450000000000000))*(pz)))+(((x383)*(x386)))+(((x383)*(x388)))+(((x385)*(x388)))+(((x385)*(x386)))+(((IkReal(-1.00000000000000))*(sj3)*(x384)*(x385)))+(((x383)*(x390)))+(((IkReal(-1.00000000000000))*(pz)*(x389)))))), ((gconst15)*(((((IkReal(-1.00000000000000))*(cj3)*(x383)*(x384)))+(((IkReal(-1.00000000000000))*(x385)*(x391)))+(((IkReal(-1.00000000000000))*(cj3)*(x384)*(x385)))+(((pz)*(x390)))+(((x383)*(x389)))+(((IkReal(0.0450000000000000))*(x382)))+(((IkReal(-1.00000000000000))*(x383)*(x391)))+(((x385)*(x389)))+(((IkReal(-0.0450000000000000))*(x383)))+(((IkReal(-0.0450000000000000))*(x385)))+(((pz)*(x386)))+(((IkReal(-1.00000000000000))*(x385)*(x387)))+(((IkReal(0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x382)*(x384)))+(((IkReal(-1.00000000000000))*(x383)*(x387)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x392=IKsin(j1);
IkReal x393=IKcos(j1);
IkReal x394=((IkReal(1.00000000000000))*(r22));
IkReal x395=((IkReal(0.0450000000000000))*(cj3));
IkReal x396=((cj0)*(px));
IkReal x397=((r00)*(sj3));
IkReal x398=((IkReal(1.00000000000000))*(r00));
IkReal x399=((py)*(sj0));
IkReal x400=((sj3)*(x392));
IkReal x401=((cj3)*(x393));
IkReal x402=((cj3)*(x392));
IkReal x403=((sj3)*(x393));
IkReal x404=((IkReal(0.0900000000000000))*(x393));
IkReal x405=((pz)*(x392));
IkReal x406=((pz)*(x393));
IkReal x407=((IkReal(1.10000000000000))*(x392));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(x397)+(x406)+(((IkReal(-1.00000000000000))*(cj3)*(x394)))+(((x392)*(x396)))+(((x392)*(x399))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x395)))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x405)))+(((x393)*(x399)))+(((x393)*(x396))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(x405)))+(((IkReal(1.10000000000000))*(x406)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x399)*(x404)))+(((x399)*(x407)))+(((IkReal(-1.00000000000000))*(x396)*(x404)))+(((x396)*(x407)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((IkReal(-0.350000000000000))*(x401)))+(((IkReal(-0.350000000000000))*(x400)))+(((IkReal(-0.0450000000000000))*(x392)))+(((IkReal(-0.0450000000000000))*(x403)))+(((IkReal(-1.00000000000000))*(x394)*(x400)))+(((IkReal(-1.00000000000000))*(x394)*(x401)))+(((IkReal(-1.00000000000000))*(x398)*(x402)))+(pz)+(((IkReal(-0.550000000000000))*(x393)))+(((x393)*(x397)))+(((x392)*(x395))));
evalcond[4]=((((IkReal(-0.350000000000000))*(x403)))+(((IkReal(0.0450000000000000))*(x400)))+(((IkReal(-0.0450000000000000))*(x393)))+(((IkReal(-1.00000000000000))*(x399)))+(((IkReal(-1.00000000000000))*(x396)))+(((r22)*(x402)))+(((IkReal(-1.00000000000000))*(x394)*(x403)))+(((IkReal(0.550000000000000))*(x392)))+(((IkReal(-1.00000000000000))*(x398)*(x401)))+(((IkReal(-1.00000000000000))*(x392)*(x397)))+(((IkReal(0.350000000000000))*(x402)))+(((x393)*(x395))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x408=((cj3)*(r22));
IkReal x409=((cj2)*(sj0));
IkReal x410=((IkReal(1.00000000000000))*(cj0));
IkReal x411=((sj2)*(sj3));
IkReal x412=((cj0)*(px));
IkReal x413=((pz)*(sj2));
IkReal x414=((IkReal(0.350000000000000))*(cj3));
IkReal x415=((IkReal(1.00000000000000))*(r00));
IkReal x416=((IkReal(1.00000000000000))*(cj2)*(py));
IkReal x417=((py)*(sj0)*(sj2));
if( IKabs(((gconst13)*(((((IkReal(0.0450000000000000))*(x411)*(x412)))+(((sj2)*(x412)*(x414)))+(((sj2)*(x408)*(x412)))+(((x414)*(x417)))+(((px)*(pz)*(x409)))+(((x408)*(x417)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x410)*(x411)))+(((IkReal(0.550000000000000))*(sj2)*(x412)))+(((IkReal(0.550000000000000))*(x417)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x411)*(x415)))+(((pz)*(r11)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x410)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x411))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst13)*(((((cj0)*(x409)*((py)*(py))))+(((IkReal(-1.00000000000000))*(pz)*(x411)*(x415)))+(((IkReal(0.0450000000000000))*(pz)*(x411)))+(((x413)*(x414)))+(((IkReal(-1.00000000000000))*(x409)*(x410)*((px)*(px))))+(((x408)*(x413)))+(((cj0)*(cj2)*(py)*(x412)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x410)))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((IkReal(0.550000000000000))*(x413)))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x409))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst13)*(((((IkReal(0.0450000000000000))*(x411)*(x412)))+(((sj2)*(x412)*(x414)))+(((sj2)*(x408)*(x412)))+(((x414)*(x417)))+(((px)*(pz)*(x409)))+(((x408)*(x417)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x410)*(x411)))+(((IkReal(0.550000000000000))*(sj2)*(x412)))+(((IkReal(0.550000000000000))*(x417)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x411)*(x415)))+(((pz)*(r11)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x410)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x411)))))), ((gconst13)*(((((cj0)*(x409)*((py)*(py))))+(((IkReal(-1.00000000000000))*(pz)*(x411)*(x415)))+(((IkReal(0.0450000000000000))*(pz)*(x411)))+(((x413)*(x414)))+(((IkReal(-1.00000000000000))*(x409)*(x410)*((px)*(px))))+(((x408)*(x413)))+(((cj0)*(cj2)*(py)*(x412)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x410)))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((IkReal(0.550000000000000))*(x413)))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x409)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[6];
IkReal x418=IKsin(j1);
IkReal x419=IKcos(j1);
IkReal x420=((px)*(sj2));
IkReal x421=((IkReal(0.0900000000000000))*(sj0));
IkReal x422=((cj2)*(px));
IkReal x423=((IkReal(0.0450000000000000))*(cj3));
IkReal x424=((r11)*(sj2));
IkReal x425=((cj3)*(r22));
IkReal x426=((r00)*(sj3));
IkReal x427=((py)*(sj0));
IkReal x428=((IkReal(0.350000000000000))*(sj3));
IkReal x429=((cj3)*(r00));
IkReal x430=((IkReal(1.00000000000000))*(cj0));
IkReal x431=((r22)*(sj3));
IkReal x432=((IkReal(0.0450000000000000))*(sj3));
IkReal x433=((IkReal(0.350000000000000))*(cj3));
IkReal x434=((py)*(sj2));
IkReal x435=((cj0)*(x419));
IkReal x436=((cj2)*(x418));
IkReal x437=((cj2)*(x419));
IkReal x438=((IkReal(1.00000000000000))*(x419));
IkReal x439=((IkReal(1.00000000000000))*(x418));
IkReal x440=((pz)*(x419));
IkReal x441=((cj0)*(px)*(x418));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x425)))+(((IkReal(-1.00000000000000))*(x433)))+(((IkReal(-1.00000000000000))*(x432)))+(((x418)*(x427)))+(x426)+(x441)+(x440));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)*(sj2)*(x439)))+(((sj2)*(x419)*(x427)))+(r11)+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x430)))+(((x420)*(x435)))+(((sj0)*(x422))));
evalcond[2]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x430)*(x434)))+(((IkReal(-1.00000000000000))*(x423)))+(((pz)*(x436)))+(((IkReal(-1.00000000000000))*(x427)*(x437)))+(((IkReal(-1.00000000000000))*(x419)*(x422)*(x430)))+(x429)+(x428)+(x431)+(((sj0)*(x420))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(x420)*(x421)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(1.10000000000000))*(x441)))+(((IkReal(1.10000000000000))*(x440)))+((r22)*(r22))+(((IkReal(-0.0900000000000000))*(pz)*(x436)))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x418)*(x427)))+(((py)*(x421)*(x437)))+(((IkReal(0.0900000000000000))*(cj0)*(x434)))+(((IkReal(0.0900000000000000))*(x422)*(x435)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((x429)*(x436)))+(((IkReal(-0.550000000000000))*(x419)))+(((x428)*(x436)))+(((x419)*(x426)))+(((IkReal(-1.00000000000000))*(x424)*(x439)))+(pz)+(((x431)*(x436)))+(((IkReal(-1.00000000000000))*(x425)*(x438)))+(((IkReal(0.0450000000000000))*(x436)))+(((IkReal(-1.00000000000000))*(x419)*(x432)))+(((IkReal(-1.00000000000000))*(x419)*(x433)))+(((IkReal(-1.00000000000000))*(x423)*(x436))));
evalcond[5]=((((x429)*(x437)))+(((IkReal(-1.00000000000000))*(x427)))+(((x428)*(x437)))+(((x418)*(x425)))+(((x418)*(x433)))+(((x418)*(x432)))+(((IkReal(-1.00000000000000))*(x424)*(x438)))+(((IkReal(-1.00000000000000))*(x426)*(x439)))+(((x431)*(x437)))+(((IkReal(0.0450000000000000))*(x437)))+(((IkReal(0.550000000000000))*(x418)))+(((IkReal(-1.00000000000000))*(x423)*(x437)))+(((IkReal(-1.00000000000000))*(px)*(x430))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x442=((sj2)*(sj3));
IkReal x443=((IkReal(0.0450000000000000))*(sj2));
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x444=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x443)))+(((IkReal(0.350000000000000))*(x442)))+(((cj2)*(r11)))+(x443)+(((r22)*(x442)))+(((cj3)*(r00)*(sj2))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x443)))+(((IkReal(0.350000000000000))*(x442)))+(((cj2)*(r11)))+(x443)+(((r22)*(x442)))+(((cj3)*(r00)*(sj2))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x445=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((IkReal(-1.00000000000000))*(cj3)*(x443)))+(((IkReal(0.350000000000000))*(x442)))+(((cj2)*(r11)))+(x443)+(((r22)*(x442)))+(((cj3)*(r00)*(sj2)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x444)))+(((IkReal(-1.00000000000000))*(x445))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x444)))+(x445));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst13;
gconst13=IKsign(((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x446=((IkReal(0.0900000000000000))*(r00));
IkReal x447=((r11)+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x447;
evalcond[2]=x447;
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x446)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((cj3)*(x446)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst14;
gconst14=IKsign(((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x448=((pz)*(sj3));
IkReal x449=((py)*(sj0));
IkReal x450=((IkReal(1.00000000000000))*(cj3));
IkReal x451=((cj0)*(px));
IkReal x452=((r00)*(sj3));
IkReal x453=((IkReal(0.350000000000000))*(cj3));
IkReal x454=((IkReal(0.350000000000000))*(sj3));
IkReal x455=((IkReal(0.0450000000000000))*(sj3));
IkReal x456=((IkReal(0.0450000000000000))*(pz));
IkReal x457=((IkReal(0.0450000000000000))*(cj3));
IkReal x458=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst14)*(((((IkReal(0.350000000000000))*(x448)))+(((cj3)*(pz)*(r00)))+(((IkReal(-0.550000000000000))*(x449)))+(((IkReal(-0.550000000000000))*(x451)))+(((IkReal(-1.00000000000000))*(r22)*(x450)*(x451)))+(x456)+(((r22)*(x448)))+(((x449)*(x452)))+(((IkReal(-1.00000000000000))*(x451)*(x455)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((x451)*(x452)))+(((IkReal(-1.00000000000000))*(cj3)*(x456)))+(((IkReal(-1.00000000000000))*(x449)*(x453)))+(((IkReal(-1.00000000000000))*(x449)*(x455)))+(((IkReal(-1.00000000000000))*(r22)*(x449)*(x450))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x450)))+(((IkReal(-0.0450000000000000))*(x448)))+(((IkReal(-0.0450000000000000))*(x449)))+(((IkReal(-1.00000000000000))*(r00)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(pz)*(x453)))+(((x449)*(x457)))+(((IkReal(-1.00000000000000))*(r00)*(x449)*(x450)))+(((IkReal(-1.00000000000000))*(x451)*(x454)))+(((IkReal(-1.00000000000000))*(x451)*(x458)))+(((r00)*(x448)))+(((IkReal(-0.0450000000000000))*(x451)))+(((x451)*(x457)))+(((IkReal(-1.00000000000000))*(x449)*(x454)))+(((IkReal(-1.00000000000000))*(x449)*(x458))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst14)*(((((IkReal(0.350000000000000))*(x448)))+(((cj3)*(pz)*(r00)))+(((IkReal(-0.550000000000000))*(x449)))+(((IkReal(-0.550000000000000))*(x451)))+(((IkReal(-1.00000000000000))*(r22)*(x450)*(x451)))+(x456)+(((r22)*(x448)))+(((x449)*(x452)))+(((IkReal(-1.00000000000000))*(x451)*(x455)))+(((IkReal(-1.00000000000000))*(x451)*(x453)))+(((x451)*(x452)))+(((IkReal(-1.00000000000000))*(cj3)*(x456)))+(((IkReal(-1.00000000000000))*(x449)*(x453)))+(((IkReal(-1.00000000000000))*(x449)*(x455)))+(((IkReal(-1.00000000000000))*(r22)*(x449)*(x450)))))), ((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x450)))+(((IkReal(-0.0450000000000000))*(x448)))+(((IkReal(-0.0450000000000000))*(x449)))+(((IkReal(-1.00000000000000))*(r00)*(x450)*(x451)))+(((IkReal(-1.00000000000000))*(pz)*(x453)))+(((x449)*(x457)))+(((IkReal(-1.00000000000000))*(r00)*(x449)*(x450)))+(((IkReal(-1.00000000000000))*(x451)*(x454)))+(((IkReal(-1.00000000000000))*(x451)*(x458)))+(((r00)*(x448)))+(((IkReal(-0.0450000000000000))*(x451)))+(((x451)*(x457)))+(((IkReal(-1.00000000000000))*(x449)*(x454)))+(((IkReal(-1.00000000000000))*(x449)*(x458)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x459=IKsin(j1);
IkReal x460=IKcos(j1);
IkReal x461=((r22)*(sj3));
IkReal x462=((IkReal(0.0450000000000000))*(cj3));
IkReal x463=((cj0)*(px));
IkReal x464=((cj3)*(r22));
IkReal x465=((r00)*(sj3));
IkReal x466=((py)*(sj0));
IkReal x467=((IkReal(0.350000000000000))*(sj3));
IkReal x468=((IkReal(0.350000000000000))*(cj3));
IkReal x469=((IkReal(0.0450000000000000))*(sj3));
IkReal x470=((cj3)*(r00));
IkReal x471=((IkReal(1.00000000000000))*(x460));
IkReal x472=((sj3)*(x460));
IkReal x473=((IkReal(0.0900000000000000))*(x460));
IkReal x474=((pz)*(x459));
IkReal x475=((pz)*(x460));
IkReal x476=((IkReal(1.10000000000000))*(x459));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x464)))+(((IkReal(-1.00000000000000))*(x468)))+(((IkReal(-1.00000000000000))*(x469)))+(x465)+(x475)+(((x459)*(x463)))+(((x459)*(x466))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x462)))+(x461)+(x467)+(x474)+(x470)+(((IkReal(-1.00000000000000))*(x466)*(x471)))+(((IkReal(-1.00000000000000))*(x463)*(x471))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(-0.0900000000000000))*(x474)))+(((x463)*(x473)))+(((x463)*(x476)))+((r11)*(r11))+(((x466)*(x476)))+(((x466)*(x473)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x475)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((x459)*(x470)))+(((IkReal(-1.00000000000000))*(x460)*(x468)))+(((IkReal(-1.00000000000000))*(x460)*(x469)))+(((IkReal(-1.00000000000000))*(x459)*(x462)))+(((IkReal(-0.550000000000000))*(x460)))+(((IkReal(0.0450000000000000))*(x459)))+(((IkReal(-1.00000000000000))*(x464)*(x471)))+(pz)+(((x460)*(x465)))+(((x459)*(x461)))+(((x459)*(x467))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x459)*(x465)))+(((IkReal(0.550000000000000))*(x459)))+(((IkReal(-1.00000000000000))*(x460)*(x462)))+(((IkReal(-1.00000000000000))*(x466)))+(((IkReal(-1.00000000000000))*(x463)))+(((IkReal(0.0450000000000000))*(x460)))+(((x460)*(x470)))+(((x460)*(x461)))+(((x460)*(x467)))+(((x459)*(x464)))+(((x459)*(x469)))+(((x459)*(x468))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x477=((IkReal(0.0900000000000000))*(r00));
IkReal x478=((px)*(sj0));
IkReal x479=((cj0)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x478)))+(x479)+(r11));
evalcond[2]=((((IkReal(-1.00000000000000))*(r11)))+(((IkReal(-1.00000000000000))*(x479)))+(x478));
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((cj3)*(x477)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x477)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst15;
gconst15=IKsign((((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)))));
dummyeval[0]=(((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x480=((pz)*(sj3));
IkReal x481=((py)*(sj0));
IkReal x482=((IkReal(1.00000000000000))*(r00));
IkReal x483=((cj0)*(px));
IkReal x484=((IkReal(0.350000000000000))*(cj3));
IkReal x485=((IkReal(0.350000000000000))*(sj3));
IkReal x486=((IkReal(0.0450000000000000))*(sj3));
IkReal x487=((IkReal(0.0450000000000000))*(cj3));
IkReal x488=((cj3)*(r22));
IkReal x489=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst15)*(((((IkReal(-1.00000000000000))*(pz)*(x487)))+(((IkReal(0.350000000000000))*(x480)))+(((cj3)*(pz)*(r00)))+(((IkReal(0.550000000000000))*(x481)))+(((IkReal(0.550000000000000))*(x483)))+(((x483)*(x486)))+(((x483)*(x488)))+(((x483)*(x484)))+(((IkReal(-1.00000000000000))*(sj3)*(x482)*(x483)))+(((IkReal(0.0450000000000000))*(pz)))+(((x481)*(x484)))+(((x481)*(x488)))+(((x481)*(x486)))+(((r22)*(x480)))+(((IkReal(-1.00000000000000))*(sj3)*(x481)*(x482))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst15)*(((((IkReal(-1.00000000000000))*(cj3)*(x482)*(x483)))+(((IkReal(-1.00000000000000))*(x483)*(x485)))+(((IkReal(-1.00000000000000))*(x483)*(x489)))+(((IkReal(-0.0450000000000000))*(x483)))+(((IkReal(-0.0450000000000000))*(x481)))+(((pz)*(x484)))+(((pz)*(x488)))+(((x483)*(x487)))+(((IkReal(-1.00000000000000))*(cj3)*(x481)*(x482)))+(((IkReal(-1.00000000000000))*(x481)*(x489)))+(((IkReal(-1.00000000000000))*(x481)*(x485)))+(((x481)*(x487)))+(((IkReal(-1.00000000000000))*(x480)*(x482)))+(((IkReal(0.0450000000000000))*(x480)))+(((IkReal(0.550000000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst15)*(((((IkReal(-1.00000000000000))*(pz)*(x487)))+(((IkReal(0.350000000000000))*(x480)))+(((cj3)*(pz)*(r00)))+(((IkReal(0.550000000000000))*(x481)))+(((IkReal(0.550000000000000))*(x483)))+(((x483)*(x486)))+(((x483)*(x488)))+(((x483)*(x484)))+(((IkReal(-1.00000000000000))*(sj3)*(x482)*(x483)))+(((IkReal(0.0450000000000000))*(pz)))+(((x481)*(x484)))+(((x481)*(x488)))+(((x481)*(x486)))+(((r22)*(x480)))+(((IkReal(-1.00000000000000))*(sj3)*(x481)*(x482)))))), ((gconst15)*(((((IkReal(-1.00000000000000))*(cj3)*(x482)*(x483)))+(((IkReal(-1.00000000000000))*(x483)*(x485)))+(((IkReal(-1.00000000000000))*(x483)*(x489)))+(((IkReal(-0.0450000000000000))*(x483)))+(((IkReal(-0.0450000000000000))*(x481)))+(((pz)*(x484)))+(((pz)*(x488)))+(((x483)*(x487)))+(((IkReal(-1.00000000000000))*(cj3)*(x481)*(x482)))+(((IkReal(-1.00000000000000))*(x481)*(x489)))+(((IkReal(-1.00000000000000))*(x481)*(x485)))+(((x481)*(x487)))+(((IkReal(-1.00000000000000))*(x480)*(x482)))+(((IkReal(0.0450000000000000))*(x480)))+(((IkReal(0.550000000000000))*(pz)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x490=IKsin(j1);
IkReal x491=IKcos(j1);
IkReal x492=((IkReal(1.00000000000000))*(r22));
IkReal x493=((IkReal(0.0450000000000000))*(cj3));
IkReal x494=((cj0)*(px));
IkReal x495=((r00)*(sj3));
IkReal x496=((IkReal(1.00000000000000))*(r00));
IkReal x497=((py)*(sj0));
IkReal x498=((sj3)*(x490));
IkReal x499=((cj3)*(x491));
IkReal x500=((cj3)*(x490));
IkReal x501=((sj3)*(x491));
IkReal x502=((IkReal(0.0900000000000000))*(x491));
IkReal x503=((pz)*(x490));
IkReal x504=((pz)*(x491));
IkReal x505=((IkReal(1.10000000000000))*(x490));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x492)))+(((x490)*(x494)))+(((x490)*(x497)))+(x495)+(x504));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x493)))+(((x491)*(x497)))+(((x491)*(x494)))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x503))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+(((x494)*(x505)))+((r22)*(r22))+(((x497)*(x505)))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x497)*(x502)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(x494)*(x502)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x504)))+(((IkReal(0.0900000000000000))*(x503)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((IkReal(-0.0450000000000000))*(x490)))+(((x491)*(x495)))+(((x490)*(x493)))+(((IkReal(-1.00000000000000))*(x496)*(x500)))+(((IkReal(-0.550000000000000))*(x491)))+(pz)+(((IkReal(-0.350000000000000))*(x498)))+(((IkReal(-0.350000000000000))*(x499)))+(((IkReal(-0.0450000000000000))*(x501)))+(((IkReal(-1.00000000000000))*(x492)*(x499)))+(((IkReal(-1.00000000000000))*(x492)*(x498))));
evalcond[4]=((((IkReal(-0.0450000000000000))*(x491)))+(((x491)*(x493)))+(((IkReal(-0.350000000000000))*(x501)))+(((IkReal(0.550000000000000))*(x490)))+(((IkReal(-1.00000000000000))*(x496)*(x499)))+(((IkReal(0.350000000000000))*(x500)))+(((r22)*(x500)))+(((IkReal(-1.00000000000000))*(x497)))+(((IkReal(-1.00000000000000))*(x494)))+(((IkReal(-1.00000000000000))*(x492)*(x501)))+(((IkReal(0.0450000000000000))*(x498)))+(((IkReal(-1.00000000000000))*(x490)*(x495))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x506=((cj3)*(r22));
IkReal x507=((cj2)*(sj0));
IkReal x508=((IkReal(1.00000000000000))*(cj0));
IkReal x509=((sj2)*(sj3));
IkReal x510=((cj0)*(px));
IkReal x511=((pz)*(sj2));
IkReal x512=((IkReal(0.350000000000000))*(cj3));
IkReal x513=((IkReal(1.00000000000000))*(r00));
IkReal x514=((IkReal(1.00000000000000))*(cj2)*(py));
IkReal x515=((py)*(sj0)*(sj2));
if( IKabs(((gconst13)*(((((IkReal(-1.00000000000000))*(py)*(sj0)*(x509)*(x513)))+(((sj2)*(x510)*(x512)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x508)))+(((sj2)*(x506)*(x510)))+(((px)*(pz)*(x507)))+(((x512)*(x515)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x509)))+(((IkReal(0.550000000000000))*(sj2)*(x510)))+(((IkReal(0.550000000000000))*(x515)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x508)*(x509)))+(((x506)*(x515)))+(((pz)*(r11)))+(((IkReal(0.0450000000000000))*(x509)*(x510))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst13)*(((((x511)*(x512)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x508)))+(((IkReal(-1.00000000000000))*(pz)*(x509)*(x513)))+(((IkReal(-1.00000000000000))*(x507)*(x508)*((px)*(px))))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x507)))+(((cj0)*(x507)*((py)*(py))))+(((IkReal(0.0450000000000000))*(pz)*(x509)))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((IkReal(0.550000000000000))*(x511)))+(((x506)*(x511)))+(((cj0)*(cj2)*(py)*(x510))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst13)*(((((IkReal(-1.00000000000000))*(py)*(sj0)*(x509)*(x513)))+(((sj2)*(x510)*(x512)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x508)))+(((sj2)*(x506)*(x510)))+(((px)*(pz)*(x507)))+(((x512)*(x515)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x509)))+(((IkReal(0.550000000000000))*(sj2)*(x510)))+(((IkReal(0.550000000000000))*(x515)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x508)*(x509)))+(((x506)*(x515)))+(((pz)*(r11)))+(((IkReal(0.0450000000000000))*(x509)*(x510)))))), ((gconst13)*(((((x511)*(x512)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x508)))+(((IkReal(-1.00000000000000))*(pz)*(x509)*(x513)))+(((IkReal(-1.00000000000000))*(x507)*(x508)*((px)*(px))))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x507)))+(((cj0)*(x507)*((py)*(py))))+(((IkReal(0.0450000000000000))*(pz)*(x509)))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((IkReal(0.550000000000000))*(x511)))+(((x506)*(x511)))+(((cj0)*(cj2)*(py)*(x510)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[6];
IkReal x516=IKsin(j1);
IkReal x517=IKcos(j1);
IkReal x518=((px)*(sj2));
IkReal x519=((IkReal(0.0900000000000000))*(sj0));
IkReal x520=((cj2)*(px));
IkReal x521=((IkReal(0.0450000000000000))*(cj3));
IkReal x522=((r11)*(sj2));
IkReal x523=((cj3)*(r22));
IkReal x524=((r00)*(sj3));
IkReal x525=((py)*(sj0));
IkReal x526=((IkReal(0.350000000000000))*(sj3));
IkReal x527=((cj3)*(r00));
IkReal x528=((IkReal(1.00000000000000))*(cj0));
IkReal x529=((r22)*(sj3));
IkReal x530=((IkReal(0.0450000000000000))*(sj3));
IkReal x531=((IkReal(0.350000000000000))*(cj3));
IkReal x532=((py)*(sj2));
IkReal x533=((cj0)*(x517));
IkReal x534=((cj2)*(x516));
IkReal x535=((cj2)*(x517));
IkReal x536=((IkReal(1.00000000000000))*(x517));
IkReal x537=((IkReal(1.00000000000000))*(x516));
IkReal x538=((pz)*(x517));
IkReal x539=((cj0)*(px)*(x516));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x523)))+(((x516)*(x525)))+(((IkReal(-1.00000000000000))*(x530)))+(((IkReal(-1.00000000000000))*(x531)))+(x539)+(x538)+(x524));
evalcond[1]=((((sj0)*(x520)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x528)))+(((x518)*(x533)))+(r11)+(((sj2)*(x517)*(x525)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x537))));
evalcond[2]=((IkReal(0.0450000000000000))+(((pz)*(x534)))+(((IkReal(-1.00000000000000))*(x517)*(x520)*(x528)))+(((IkReal(-1.00000000000000))*(x521)))+(((IkReal(-1.00000000000000))*(x528)*(x532)))+(((IkReal(-1.00000000000000))*(x525)*(x535)))+(x526)+(x527)+(x529)+(((sj0)*(x518))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(1.10000000000000))*(x539)))+(((IkReal(1.10000000000000))*(x538)))+((r11)*(r11))+(((py)*(x519)*(x535)))+(((IkReal(-0.0900000000000000))*(pz)*(x534)))+(((IkReal(1.10000000000000))*(x516)*(x525)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x518)*(x519)))+(((IkReal(0.0900000000000000))*(x520)*(x533)))+(((IkReal(0.0900000000000000))*(cj0)*(x532)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((IkReal(0.0450000000000000))*(x534)))+(((IkReal(-1.00000000000000))*(x521)*(x534)))+(((IkReal(-1.00000000000000))*(x523)*(x536)))+(((x526)*(x534)))+(((x517)*(x524)))+(((IkReal(-1.00000000000000))*(x517)*(x531)))+(((IkReal(-1.00000000000000))*(x517)*(x530)))+(pz)+(((IkReal(-0.550000000000000))*(x517)))+(((IkReal(-1.00000000000000))*(x522)*(x537)))+(((x527)*(x534)))+(((x529)*(x534))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x524)*(x537)))+(((IkReal(0.0450000000000000))*(x535)))+(((IkReal(-1.00000000000000))*(x521)*(x535)))+(((IkReal(-1.00000000000000))*(x525)))+(((x526)*(x535)))+(((x516)*(x523)))+(((IkReal(-1.00000000000000))*(px)*(x528)))+(((x516)*(x531)))+(((x516)*(x530)))+(((IkReal(0.550000000000000))*(x516)))+(((IkReal(-1.00000000000000))*(x522)*(x536)))+(((x527)*(x535)))+(((x529)*(x535))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j3array[2], cj3array[2], sj3array[2];
bool j3valid[2]={false};
_nj3 = 2;
IkReal x540=((IkReal(0.0900000000000000))*(r00));
IkReal x541=((IkReal(0.0810000000000000))+(((IkReal(0.0900000000000000))*(r22)))+(((IkReal(-1.10000000000000))*(r00))));
IkReal x542=((IkReal(0.380950000000000))+(x540)+(((IkReal(1.10000000000000))*(r22))));
if( IKabs(x542) < IKFAST_ATAN2_MAGTHRESH && IKabs(x541) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x543=((IkReal(1.00000000000000))*(IKatan2(x542, x541)));
if( ((((x541)*(x541))+((x542)*(x542)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((x541)*(x541))+((x542)*(x542)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x541)*(x541))+((x542)*(x542))))))):(IkReal)1.0e30))*(((IkReal(0.429050000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x540)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((x541)*(x541))+((x542)*(x542)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x541)*(x541))+((x542)*(x542))))))):(IkReal)1.0e30))*(((IkReal(0.429050000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x540)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x544=IKasin(((((IKabs(IKabs(IKsqrt((((x541)*(x541))+((x542)*(x542)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x541)*(x541))+((x542)*(x542))))))):(IkReal)1.0e30))*(((IkReal(0.429050000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x540)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))))));
j3array[0]=((((IkReal(-1.00000000000000))*(x544)))+(((IkReal(-1.00000000000000))*(x543))));
sj3array[0]=IKsin(j3array[0]);
cj3array[0]=IKcos(j3array[0]);
j3array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x543)))+(x544));
sj3array[1]=IKsin(j3array[1]);
cj3array[1]=IKcos(j3array[1]);
if( j3array[0] > IKPI )
{
    j3array[0]-=IK2PI;
}
else if( j3array[0] < -IKPI )
{    j3array[0]+=IK2PI;
}
j3valid[0] = true;
if( j3array[1] > IKPI )
{
    j3array[1]-=IK2PI;
}
else if( j3array[1] < -IKPI )
{    j3array[1]+=IK2PI;
}
j3valid[1] = true;
for(int ij3 = 0; ij3 < 2; ++ij3)
{
if( !j3valid[ij3] )
{
    continue;
}
_ij3[0] = ij3; _ij3[1] = -1;
for(int iij3 = ij3+1; iij3 < 2; ++iij3)
{
if( j3valid[iij3] && IKabs(cj3array[ij3]-cj3array[iij3]) < IKFAST_SOLUTION_THRESH && IKabs(sj3array[ij3]-sj3array[iij3]) < IKFAST_SOLUTION_THRESH )
{
    j3valid[iij3]=false; _ij3[1] = iij3; break; 
}
}
j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];

{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal x545=(cj2)*(cj2);
IkReal x546=(sj3)*(sj3);
IkReal x547=(cj3)*(cj3);
IkReal x548=(r00)*(r00);
IkReal x549=(r22)*(r22);
IkReal x550=((IkReal(345.679012345679))*(sj3));
IkReal x551=((IkReal(44.4444444444444))*(r00));
IkReal x552=((IkReal(44.4444444444444))*(cj3));
IkReal x553=((r22)*(sj3));
IkReal x554=((IkReal(15.5555555555556))*(sj3));
IkReal x555=((IkReal(345.679012345679))*(r22));
IkReal x556=((cj3)*(r00));
IkReal x557=((cj2)*(r11)*(sj2));
IkReal x558=((IkReal(493.827160493827))*(x549));
IkReal x559=((IkReal(493.827160493827))*(x548));
IkReal x560=((cj3)*(x545));
IkReal x561=((x545)*(x546));
IkReal x562=((x545)*(x547));
dummyeval[0]=((IkReal(149.382716049383))+(((IkReal(543.209876543210))*(cj3)*(r22)))+(((x558)*(x561)))+(((IkReal(-44.4444444444444))*(x557)))+(((x551)*(x560)))+(((IkReal(-543.209876543210))*(r00)*(sj3)))+(((IkReal(-1.00000000000000))*(x550)*(x557)))+(((IkReal(-1.00000000000000))*(x550)*(x556)))+(((IkReal(60.4938271604938))*(x547)))+(((IkReal(24.4444444444444))*(sj3)))+(((IkReal(190.123456790123))*(cj3)))+(((IkReal(-2.00000000000000))*(x560)))+(((x547)*(x555)))+(((x547)*(x558)))+(((IkReal(-1.00000000000000))*(x545)*(x552)*(x553)))+(x545)+(x546)+(x562)+(((IkReal(-1.00000000000000))*(x554)*(x560)))+(((IkReal(493.827160493827))*((r11)*(r11))*((sj2)*(sj2))))+(((IkReal(-987.654320987654))*(x556)*(x557)))+(((x545)*(x550)*(x556)))+(((IkReal(-1.00000000000000))*(x551)*(x562)))+(((x546)*(x559)))+(((IkReal(44.4444444444444))*(x545)*(x553)))+(((IkReal(-987.654320987654))*(x553)*(x557)))+(((IkReal(-987.654320987654))*(x553)*(x556)))+(((IkReal(987.654320987654))*(x545)*(x553)*(x556)))+(((x552)*(x553)))+(((x552)*(x557)))+(((IkReal(-1.00000000000000))*(x546)*(x551)))+(((x545)*(x554)))+(((x555)*(x561)))+(((cj3)*(x554)))+(((x559)*(x562)))+(((IkReal(60.4938271604938))*(x561))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[2], cj1array[2], sj1array[2];
bool j1valid[2]={false};
_nj1 = 2;
IkReal x563=((IkReal(0.0450000000000000))*(cj2));
IkReal x564=((cj2)*(sj3));
IkReal x565=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r22))));
IkReal x566=((((IkReal(-1.00000000000000))*(cj3)*(x563)))+(((IkReal(0.350000000000000))*(x564)))+(((r22)*(x564)))+(x563)+(((IkReal(-1.00000000000000))*(r11)*(sj2)))+(((cj2)*(cj3)*(r00))));
if( IKabs(x565) < IKFAST_ATAN2_MAGTHRESH && IKabs(x566) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x567=((IkReal(1.00000000000000))*(IKatan2(x565, x566)));
if( ((((x566)*(x566))+((x565)*(x565)))) < (IkReal)-0.00001 )
    continue;
if( (((pz)*(((IKabs(IKabs(IKsqrt((((x566)*(x566))+((x565)*(x565)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x566)*(x566))+((x565)*(x565))))))):(IkReal)1.0e30)))) < -1-IKFAST_SINCOS_THRESH || (((pz)*(((IKabs(IKabs(IKsqrt((((x566)*(x566))+((x565)*(x565)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x566)*(x566))+((x565)*(x565))))))):(IkReal)1.0e30)))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x568=IKasin(((pz)*(((IKabs(IKabs(IKsqrt((((x566)*(x566))+((x565)*(x565)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((x566)*(x566))+((x565)*(x565))))))):(IkReal)1.0e30))));
j1array[0]=((((IkReal(-1.00000000000000))*(x568)))+(((IkReal(-1.00000000000000))*(x567))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
j1array[1]=((IkReal(3.14159265358979))+(x568)+(((IkReal(-1.00000000000000))*(x567))));
sj1array[1]=IKsin(j1array[1]);
cj1array[1]=IKcos(j1array[1]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
if( j1array[1] > IKPI )
{
    j1array[1]-=IK2PI;
}
else if( j1array[1] < -IKPI )
{    j1array[1]+=IK2PI;
}
j1valid[1] = true;
for(int ij1 = 0; ij1 < 2; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 2; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];

{
IkReal dummyeval[1];
IkReal gconst0;
IkReal x569=((IkReal(1.00000000000000))*(cj2)*(sj1));
gconst0=IKsign(((((IkReal(-1.00000000000000))*(x569)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x569)*((py)*(py))))));
IkReal x570=((IkReal(1.00000000000000))*(cj2)*(sj1));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x570)*((py)*(py))))+(((IkReal(-1.00000000000000))*(x570)*((px)*(px)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x571=((r00)*(sj3));
IkReal x572=((IkReal(0.0900000000000000))*(r00));
IkReal x573=((cj3)*(r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.10000000000000))*(x571)))+(((IkReal(-1.00000000000000))*(pp)))+(((cj3)*(x572)))+(((IkReal(-1.00000000000000))*(x572)))+(((IkReal(1.10000000000000))*(x573))));
evalcond[2]=((((IkReal(-0.0450000000000000))*(cj1)*(sj3)))+(((IkReal(-0.550000000000000))*(cj1)))+(((IkReal(-0.350000000000000))*(cj1)*(cj3)))+(pz)+(((IkReal(-1.00000000000000))*(r11)*(sj1)))+(((cj1)*(x571)))+(((IkReal(-1.00000000000000))*(cj1)*(x573))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst1;
gconst1=IKsign(((((cj1)*((px)*(px))))+(((cj1)*((py)*(py))))));
dummyeval[0]=((((cj1)*((px)*(px))))+(((cj1)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x574=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(r11));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((cj3)*(x574)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.00000000000000))*(x574)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r11)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst2;
gconst2=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x575=((IkReal(0.0450000000000000))*(px));
IkReal x576=((px)*(sj3));
IkReal x577=((cj3)*(py));
IkReal x578=((IkReal(1.00000000000000))*(r22));
IkReal x579=((py)*(sj3));
IkReal x580=((cj3)*(px));
if( IKabs(((gconst2)*(((((IkReal(-0.550000000000000))*(py)))+(((r00)*(x579)))+(((IkReal(-1.00000000000000))*(cj3)*(x575)))+(((IkReal(-0.0450000000000000))*(x579)))+(((IkReal(-1.00000000000000))*(x577)*(x578)))+(((IkReal(-0.350000000000000))*(x577)))+(x575)+(((IkReal(0.350000000000000))*(x576)))+(((r00)*(x580)))+(((r22)*(x576))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst2)*(((((IkReal(-1.00000000000000))*(x578)*(x579)))+(((IkReal(-0.550000000000000))*(px)))+(((IkReal(0.0450000000000000))*(x577)))+(((IkReal(-1.00000000000000))*(r00)*(x577)))+(((IkReal(-0.0450000000000000))*(py)))+(((r00)*(x576)))+(((IkReal(-1.00000000000000))*(x578)*(x580)))+(((IkReal(-0.350000000000000))*(x580)))+(((IkReal(-1.00000000000000))*(sj3)*(x575)))+(((IkReal(-0.350000000000000))*(x579))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst2)*(((((IkReal(-0.550000000000000))*(py)))+(((r00)*(x579)))+(((IkReal(-1.00000000000000))*(cj3)*(x575)))+(((IkReal(-0.0450000000000000))*(x579)))+(((IkReal(-1.00000000000000))*(x577)*(x578)))+(((IkReal(-0.350000000000000))*(x577)))+(x575)+(((IkReal(0.350000000000000))*(x576)))+(((r00)*(x580)))+(((r22)*(x576)))))), ((gconst2)*(((((IkReal(-1.00000000000000))*(x578)*(x579)))+(((IkReal(-0.550000000000000))*(px)))+(((IkReal(0.0450000000000000))*(x577)))+(((IkReal(-1.00000000000000))*(r00)*(x577)))+(((IkReal(-0.0450000000000000))*(py)))+(((r00)*(x576)))+(((IkReal(-1.00000000000000))*(x578)*(x580)))+(((IkReal(-0.350000000000000))*(x580)))+(((IkReal(-1.00000000000000))*(sj3)*(x575)))+(((IkReal(-0.350000000000000))*(x579)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x581=IKsin(j0);
IkReal x582=IKcos(j0);
IkReal x583=((px)*(x581));
IkReal x584=((py)*(x582));
IkReal x585=((px)*(x582));
IkReal x586=((py)*(x581));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)))+(x585)+(x586));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x584)))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x583));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x583)))+(((IkReal(0.0900000000000000))*(x584)))+(((IkReal(1.10000000000000))*(x585)))+(((IkReal(1.10000000000000))*(x586)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x587=((pz)+(r11));
IkReal x588=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x587;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.00000000000000))*(x588)))+(((IkReal(-1.10000000000000))*(r00)*(sj3)))+(((cj3)*(x588))));
evalcond[3]=x587;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst3;
gconst3=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x589=((py)*(sj3));
IkReal x590=((IkReal(0.0450000000000000))*(px));
IkReal x591=((cj3)*(r00));
IkReal x592=((IkReal(1.00000000000000))*(px));
IkReal x593=((px)*(sj3));
IkReal x594=((cj3)*(py));
if( IKabs(((gconst3)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(r22)*(x594)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x592)))+(((IkReal(-1.00000000000000))*(x590)))+(((cj3)*(x590)))+(((r00)*(x589)))+(((IkReal(-0.0450000000000000))*(x589)))+(((IkReal(-0.350000000000000))*(x593)))+(((IkReal(-0.350000000000000))*(x594)))+(((IkReal(-1.00000000000000))*(x591)*(x592))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst3)*(((((IkReal(-0.550000000000000))*(px)))+(((py)*(x591)))+(((IkReal(-0.0450000000000000))*(x594)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x592)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(0.350000000000000))*(x589)))+(((r00)*(x593)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((IkReal(-1.00000000000000))*(sj3)*(x590)))+(((r22)*(x589))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst3)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(r22)*(x594)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x592)))+(((IkReal(-1.00000000000000))*(x590)))+(((cj3)*(x590)))+(((r00)*(x589)))+(((IkReal(-0.0450000000000000))*(x589)))+(((IkReal(-0.350000000000000))*(x593)))+(((IkReal(-0.350000000000000))*(x594)))+(((IkReal(-1.00000000000000))*(x591)*(x592)))))), ((gconst3)*(((((IkReal(-0.550000000000000))*(px)))+(((py)*(x591)))+(((IkReal(-0.0450000000000000))*(x594)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x592)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(0.350000000000000))*(x589)))+(((r00)*(x593)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((IkReal(-1.00000000000000))*(sj3)*(x590)))+(((r22)*(x589)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x595=IKsin(j0);
IkReal x596=IKcos(j0);
IkReal x597=((px)*(x595));
IkReal x598=((IkReal(1.00000000000000))*(x596));
IkReal x599=((py)*(x595));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)))+(((IkReal(-1.00000000000000))*(px)*(x598)))+(((IkReal(-1.00000000000000))*(x599))));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(-1.00000000000000))*(py)*(x598)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x597));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x597)))+(((IkReal(-1.10000000000000))*(x599)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(px)*(x596)))+(((IkReal(0.0900000000000000))*(py)*(x596)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x600=((cj1)*(px));
IkReal x601=((r22)*(sj3));
IkReal x602=((cj1)*(py));
IkReal x603=((pz)*(sj1));
IkReal x604=((IkReal(1.00000000000000))*(r11));
IkReal x605=((IkReal(0.350000000000000))*(sj3));
IkReal x606=((cj3)*(r00));
IkReal x607=((IkReal(0.0450000000000000))*(cj3));
if( IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x600)*(x606)))+(((IkReal(-1.00000000000000))*(x600)*(x601)))+(((x600)*(x607)))+(((IkReal(-1.00000000000000))*(x600)*(x605)))+(((py)*(x603)))+(((IkReal(-1.00000000000000))*(py)*(x604)))+(((IkReal(-0.0450000000000000))*(x600))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst1)*(((((IkReal(-1.00000000000000))*(x602)*(x607)))+(((x601)*(x602)))+(((px)*(x603)))+(((IkReal(0.0450000000000000))*(x602)))+(((x602)*(x606)))+(((x602)*(x605)))+(((IkReal(-1.00000000000000))*(px)*(x604))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst1)*(((((IkReal(-1.00000000000000))*(x600)*(x606)))+(((IkReal(-1.00000000000000))*(x600)*(x601)))+(((x600)*(x607)))+(((IkReal(-1.00000000000000))*(x600)*(x605)))+(((py)*(x603)))+(((IkReal(-1.00000000000000))*(py)*(x604)))+(((IkReal(-0.0450000000000000))*(x600)))))), ((gconst1)*(((((IkReal(-1.00000000000000))*(x602)*(x607)))+(((x601)*(x602)))+(((px)*(x603)))+(((IkReal(0.0450000000000000))*(x602)))+(((x602)*(x606)))+(((x602)*(x605)))+(((IkReal(-1.00000000000000))*(px)*(x604)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x608=IKcos(j0);
IkReal x609=IKsin(j0);
IkReal x610=((r00)*(sj3));
IkReal x611=((IkReal(1.00000000000000))*(sj1));
IkReal x612=((IkReal(0.350000000000000))*(cj3));
IkReal x613=((IkReal(0.0450000000000000))*(sj3));
IkReal x614=((cj3)*(r22));
IkReal x615=((cj1)*(pz));
IkReal x616=((px)*(x609));
IkReal x617=((px)*(x608));
IkReal x618=((py)*(x609));
IkReal x619=((py)*(x608));
IkReal x620=((sj1)*(x618));
evalcond[0]=((IkReal(-0.550000000000000))+(((sj1)*(x617)))+(((IkReal(-1.00000000000000))*(x614)))+(((IkReal(-1.00000000000000))*(x613)))+(((IkReal(-1.00000000000000))*(x612)))+(x615)+(x610)+(x620));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)*(x611)))+(((cj1)*(x617)))+(((cj1)*(x618)))+(r11));
evalcond[2]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(x619)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x616));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(x616)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(1.10000000000000))*(sj1)*(x617)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(x619)))+(((IkReal(1.10000000000000))*(x620)))+(((IkReal(1.10000000000000))*(x615)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((IkReal(-1.00000000000000))*(x610)*(x611)))+(((sj1)*(x613)))+(((sj1)*(x612)))+(((sj1)*(x614)))+(((IkReal(-1.00000000000000))*(x617)))+(((IkReal(-1.00000000000000))*(x618)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(cj1)*(r11))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x621=((r00)*(sj3));
IkReal x622=((IkReal(0.0900000000000000))*(r00));
IkReal x623=((cj3)*(r22));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((cj3)*(x622)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(1.10000000000000))*(x623)))+(((IkReal(-1.00000000000000))*(x622)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x621))));
evalcond[2]=((((IkReal(-1.00000000000000))*(cj1)*(x623)))+(((IkReal(-0.0450000000000000))*(cj1)*(sj3)))+(((IkReal(-0.550000000000000))*(cj1)))+(((r11)*(sj1)))+(((IkReal(-0.350000000000000))*(cj1)*(cj3)))+(pz)+(((cj1)*(x621))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst4;
gconst4=IKsign(((((cj1)*((px)*(px))))+(((cj1)*((py)*(py))))));
dummyeval[0]=((((cj1)*((px)*(px))))+(((cj1)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x624=((pz)+(r11));
IkReal x625=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.57079632679490))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x624;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((cj3)*(x625)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x625)))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
evalcond[3]=x624;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst5;
gconst5=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x626=((py)*(sj3));
IkReal x627=((IkReal(0.0450000000000000))*(px));
IkReal x628=((cj3)*(r00));
IkReal x629=((IkReal(1.00000000000000))*(px));
IkReal x630=((px)*(sj3));
IkReal x631=((cj3)*(py));
if( IKabs(((gconst5)*(((((IkReal(-0.550000000000000))*(py)))+(((cj3)*(x627)))+(((IkReal(-1.00000000000000))*(x628)*(x629)))+(((IkReal(-1.00000000000000))*(x627)))+(((IkReal(-0.0450000000000000))*(x626)))+(((IkReal(-1.00000000000000))*(r22)*(x631)))+(((IkReal(-0.350000000000000))*(x630)))+(((IkReal(-0.350000000000000))*(x631)))+(((r00)*(x626)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x629))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst5)*(((((IkReal(-0.550000000000000))*(px)))+(((py)*(x628)))+(((IkReal(0.350000000000000))*(x626)))+(((r22)*(x626)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-0.0450000000000000))*(x631)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((r00)*(x630)))+(((IkReal(-1.00000000000000))*(sj3)*(x627)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x629))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst5)*(((((IkReal(-0.550000000000000))*(py)))+(((cj3)*(x627)))+(((IkReal(-1.00000000000000))*(x628)*(x629)))+(((IkReal(-1.00000000000000))*(x627)))+(((IkReal(-0.0450000000000000))*(x626)))+(((IkReal(-1.00000000000000))*(r22)*(x631)))+(((IkReal(-0.350000000000000))*(x630)))+(((IkReal(-0.350000000000000))*(x631)))+(((r00)*(x626)))+(((IkReal(-1.00000000000000))*(r22)*(sj3)*(x629)))))), ((gconst5)*(((((IkReal(-0.550000000000000))*(px)))+(((py)*(x628)))+(((IkReal(0.350000000000000))*(x626)))+(((r22)*(x626)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-0.0450000000000000))*(x631)))+(((IkReal(-0.350000000000000))*(cj3)*(px)))+(((r00)*(x630)))+(((IkReal(-1.00000000000000))*(sj3)*(x627)))+(((IkReal(-1.00000000000000))*(cj3)*(r22)*(x629)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x632=IKsin(j0);
IkReal x633=IKcos(j0);
IkReal x634=((IkReal(1.00000000000000))*(r22));
IkReal x635=((px)*(x632));
IkReal x636=((py)*(x633));
IkReal x637=((px)*(x633));
IkReal x638=((py)*(x632));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(x637)+(x638)+(((IkReal(-1.00000000000000))*(cj3)*(x634))));
evalcond[1]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)))+(x635)+(((IkReal(-1.00000000000000))*(x636)))+(((IkReal(-1.00000000000000))*(sj3)*(x634))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(1.10000000000000))*(x638)))+(((IkReal(1.10000000000000))*(x637)))+(((IkReal(-0.0900000000000000))*(x636)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(x635)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x639=((IkReal(0.0900000000000000))*(r00));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(4.71238898038469))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(pz)))+(r11));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((cj3)*(x639)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x639)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
evalcond[3]=((((IkReal(-1.00000000000000))*(r11)))+(pz));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst6;
gconst6=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x640=((IkReal(0.0450000000000000))*(px));
IkReal x641=((px)*(sj3));
IkReal x642=((cj3)*(py));
IkReal x643=((IkReal(1.00000000000000))*(r22));
IkReal x644=((py)*(sj3));
IkReal x645=((cj3)*(px));
if( IKabs(((gconst6)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(x642)*(x643)))+(((IkReal(-0.350000000000000))*(x642)))+(((IkReal(-1.00000000000000))*(cj3)*(x640)))+(((IkReal(0.350000000000000))*(x641)))+(x640)+(((r22)*(x641)))+(((IkReal(-0.0450000000000000))*(x644)))+(((r00)*(x644)))+(((r00)*(x645))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst6)*(((((IkReal(-0.550000000000000))*(px)))+(((IkReal(-0.0450000000000000))*(py)))+(((IkReal(-0.350000000000000))*(x644)))+(((IkReal(-0.350000000000000))*(x645)))+(((IkReal(-1.00000000000000))*(sj3)*(x640)))+(((IkReal(0.0450000000000000))*(x642)))+(((IkReal(-1.00000000000000))*(r00)*(x642)))+(((IkReal(-1.00000000000000))*(x643)*(x645)))+(((IkReal(-1.00000000000000))*(x643)*(x644)))+(((r00)*(x641))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst6)*(((((IkReal(-0.550000000000000))*(py)))+(((IkReal(-1.00000000000000))*(x642)*(x643)))+(((IkReal(-0.350000000000000))*(x642)))+(((IkReal(-1.00000000000000))*(cj3)*(x640)))+(((IkReal(0.350000000000000))*(x641)))+(x640)+(((r22)*(x641)))+(((IkReal(-0.0450000000000000))*(x644)))+(((r00)*(x644)))+(((r00)*(x645)))))), ((gconst6)*(((((IkReal(-0.550000000000000))*(px)))+(((IkReal(-0.0450000000000000))*(py)))+(((IkReal(-0.350000000000000))*(x644)))+(((IkReal(-0.350000000000000))*(x645)))+(((IkReal(-1.00000000000000))*(sj3)*(x640)))+(((IkReal(0.0450000000000000))*(x642)))+(((IkReal(-1.00000000000000))*(r00)*(x642)))+(((IkReal(-1.00000000000000))*(x643)*(x645)))+(((IkReal(-1.00000000000000))*(x643)*(x644)))+(((r00)*(x641)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x646=IKsin(j0);
IkReal x647=IKcos(j0);
IkReal x648=((IkReal(1.00000000000000))*(r22));
IkReal x649=((px)*(x646));
IkReal x650=((IkReal(1.00000000000000))*(x647));
IkReal x651=((py)*(x646));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((r00)*(sj3)))+(((IkReal(-1.00000000000000))*(px)*(x650)))+(((IkReal(-1.00000000000000))*(x651)))+(((IkReal(-1.00000000000000))*(cj3)*(x648))));
evalcond[1]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(sj3)*(x648)))+(((IkReal(-0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)))+(x649)+(((IkReal(-1.00000000000000))*(py)*(x650))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-0.0900000000000000))*(py)*(x647)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(-1.10000000000000))*(x651)))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(px)*(x647)))+(((IkReal(0.0900000000000000))*(x649)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x652=((IkReal(0.0450000000000000))*(cj1));
IkReal x653=((pz)*(sj1));
IkReal x654=((IkReal(1.00000000000000))*(py));
IkReal x655=((IkReal(0.350000000000000))*(cj1)*(sj3));
IkReal x656=((cj1)*(cj3)*(r00));
IkReal x657=((cj1)*(r22)*(sj3));
if( IKabs(((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(px)*(x652)))+(((py)*(x653)))+(((px)*(x652)))+(((px)*(x657)))+(((px)*(x656)))+(((px)*(x655)))+(((py)*(r11))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst4)*(((((px)*(r11)))+(((cj3)*(py)*(x652)))+(((px)*(x653)))+(((IkReal(-1.00000000000000))*(py)*(x652)))+(((IkReal(-1.00000000000000))*(py)*(x655)))+(((IkReal(-1.00000000000000))*(x654)*(x657)))+(((IkReal(-1.00000000000000))*(x654)*(x656))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst4)*(((((IkReal(-1.00000000000000))*(cj3)*(px)*(x652)))+(((py)*(x653)))+(((px)*(x652)))+(((px)*(x657)))+(((px)*(x656)))+(((px)*(x655)))+(((py)*(r11)))))), ((gconst4)*(((((px)*(r11)))+(((cj3)*(py)*(x652)))+(((px)*(x653)))+(((IkReal(-1.00000000000000))*(py)*(x652)))+(((IkReal(-1.00000000000000))*(py)*(x655)))+(((IkReal(-1.00000000000000))*(x654)*(x657)))+(((IkReal(-1.00000000000000))*(x654)*(x656)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x658=IKcos(j0);
IkReal x659=IKsin(j0);
IkReal x660=((r00)*(sj3));
IkReal x661=((IkReal(0.350000000000000))*(cj3));
IkReal x662=((IkReal(0.0450000000000000))*(sj3));
IkReal x663=((IkReal(1.00000000000000))*(r22));
IkReal x664=((IkReal(1.00000000000000))*(py));
IkReal x665=((cj1)*(pz));
IkReal x666=((px)*(x659));
IkReal x667=((px)*(x658));
IkReal x668=((py)*(sj1)*(x659));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(cj3)*(x663)))+(((sj1)*(x667)))+(((IkReal(-1.00000000000000))*(x662)))+(((IkReal(-1.00000000000000))*(x661)))+(x668)+(x660)+(x665));
evalcond[1]=((((IkReal(-1.00000000000000))*(cj1)*(x659)*(x664)))+(((IkReal(-1.00000000000000))*(cj1)*(x667)))+(r11)+(((pz)*(sj1))));
evalcond[2]=((IkReal(-0.0450000000000000))+(((IkReal(0.0450000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(sj3)*(x663)))+(((IkReal(-0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(x658)*(x664)))+(((IkReal(-1.00000000000000))*(cj3)*(r00)))+(x666));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(sj1)*(x667)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(0.0900000000000000))*(x666)))+((r11)*(r11))+(((IkReal(1.10000000000000))*(x665)))+(((IkReal(1.10000000000000))*(x668)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(py)*(x658)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((cj3)*(r22)*(sj1)))+(((IkReal(-1.00000000000000))*(x659)*(x664)))+(((sj1)*(x662)))+(((sj1)*(x661)))+(((cj1)*(r11)))+(((IkReal(-1.00000000000000))*(x667)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(sj1)*(x660))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x669=((r00)*(sj3));
IkReal x670=((IkReal(0.0900000000000000))*(r00));
IkReal x671=((cj3)*(r22));
IkReal x672=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(pz)+(x669)+(((IkReal(-1.00000000000000))*(x671))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=x672;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((cj3)*(x670)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(1.10000000000000))*(x671)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x669)))+(((IkReal(-1.00000000000000))*(x670))));
evalcond[3]=x672;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst7;
IkReal x673=((IkReal(1.00000000000000))*(sj2));
gconst7=IKsign(((((IkReal(-1.00000000000000))*(x673)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x673)*((py)*(py))))));
IkReal x674=((IkReal(1.00000000000000))*(sj2));
dummyeval[0]=((((IkReal(-1.00000000000000))*(x674)*((px)*(px))))+(((IkReal(-1.00000000000000))*(x674)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x675=((r00)*(sj3));
IkReal x676=((IkReal(0.0900000000000000))*(r00));
IkReal x677=((cj3)*(r22));
IkReal x678=x672;
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x678;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((cj3)*(x676)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(1.10000000000000))*(x677)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x675)))+(((IkReal(-1.00000000000000))*(x676))));
evalcond[3]=x678;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst8;
gconst8=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x679=((py)*(sj3));
IkReal x680=((IkReal(0.0450000000000000))*(cj3));
IkReal x681=((px)*(sj3));
IkReal x682=((cj3)*(r00));
if( IKabs(((gconst8)*(((((IkReal(0.350000000000000))*(x679)))+(((IkReal(-1.00000000000000))*(px)*(r11)))+(((r22)*(x679)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x680)))+(((py)*(x682))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst8)*(((((px)*(x682)))+(((IkReal(0.350000000000000))*(x681)))+(((r22)*(x681)))+(((IkReal(0.0450000000000000))*(px)))+(((IkReal(-1.00000000000000))*(px)*(x680)))+(((py)*(r11))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst8)*(((((IkReal(0.350000000000000))*(x679)))+(((IkReal(-1.00000000000000))*(px)*(r11)))+(((r22)*(x679)))+(((IkReal(0.0450000000000000))*(py)))+(((IkReal(-1.00000000000000))*(py)*(x680)))+(((py)*(x682)))))), ((gconst8)*(((((px)*(x682)))+(((IkReal(0.350000000000000))*(x681)))+(((r22)*(x681)))+(((IkReal(0.0450000000000000))*(px)))+(((IkReal(-1.00000000000000))*(px)*(x680)))+(((py)*(r11)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x683=IKcos(j0);
IkReal x684=IKsin(j0);
IkReal x685=((IkReal(1.00000000000000))*(py));
IkReal x686=((px)*(x683));
evalcond[0]=((((px)*(x684)))+(r11)+(((IkReal(-1.00000000000000))*(x683)*(x685))));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x684)*(x685)))+(((IkReal(-1.00000000000000))*(x686))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(py)*(x684)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(0.0900000000000000))*(x686)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x687=((r00)*(sj3));
IkReal x688=((IkReal(0.0900000000000000))*(r00));
IkReal x689=((cj3)*(r22));
IkReal x690=x672;
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x690;
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(1.10000000000000))*(x689)))+((r22)*(r22))+(((IkReal(-1.00000000000000))*(x688)))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x687)))+(((cj3)*(x688))));
evalcond[3]=x690;
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst9;
gconst9=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x691=((IkReal(1.00000000000000))*(py));
IkReal x692=((cj3)*(r00));
IkReal x693=((r22)*(sj3));
IkReal x694=((IkReal(1.00000000000000))*(px));
IkReal x695=((IkReal(0.350000000000000))*(sj3));
IkReal x696=((IkReal(0.0450000000000000))*(py));
IkReal x697=((IkReal(0.0450000000000000))*(px));
if( IKabs(((gconst9)*(((((px)*(r11)))+(((cj3)*(x696)))+(((IkReal(-1.00000000000000))*(x696)))+(((IkReal(-1.00000000000000))*(py)*(x695)))+(((IkReal(-1.00000000000000))*(x691)*(x692)))+(((IkReal(-1.00000000000000))*(x691)*(x693))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst9)*(((((IkReal(-1.00000000000000))*(x693)*(x694)))+(((IkReal(-1.00000000000000))*(r11)*(x691)))+(((IkReal(-1.00000000000000))*(px)*(x695)))+(((IkReal(-1.00000000000000))*(x692)*(x694)))+(((cj3)*(x697)))+(((IkReal(-1.00000000000000))*(x697))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst9)*(((((px)*(r11)))+(((cj3)*(x696)))+(((IkReal(-1.00000000000000))*(x696)))+(((IkReal(-1.00000000000000))*(py)*(x695)))+(((IkReal(-1.00000000000000))*(x691)*(x692)))+(((IkReal(-1.00000000000000))*(x691)*(x693)))))), ((gconst9)*(((((IkReal(-1.00000000000000))*(x693)*(x694)))+(((IkReal(-1.00000000000000))*(r11)*(x691)))+(((IkReal(-1.00000000000000))*(px)*(x695)))+(((IkReal(-1.00000000000000))*(x692)*(x694)))+(((cj3)*(x697)))+(((IkReal(-1.00000000000000))*(x697)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x698=IKsin(j0);
IkReal x699=IKcos(j0);
IkReal x700=((px)*(x699));
IkReal x701=((py)*(x698));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x698)))+(((py)*(x699)))+(r11));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x700)+(x701));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(x701)))+(((IkReal(-0.0900000000000000))*(x700)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x702=(sj2)*(sj2);
IkReal x703=(cj2)*(cj2);
IkReal x704=((cj3)*(px));
IkReal x705=((py)*(r11));
IkReal x706=((cj2)*(sj2));
IkReal x707=((px)*(r11));
IkReal x708=((IkReal(0.0450000000000000))*(px));
IkReal x709=((IkReal(1.00000000000000))*(x706));
IkReal x710=((py)*(r22)*(sj3));
IkReal x711=((IkReal(0.350000000000000))*(px)*(sj3));
IkReal x712=((cj3)*(py)*(r00));
IkReal x713=((IkReal(1.00000000000000))*(x702));
IkReal x714=((IkReal(0.350000000000000))*(py)*(sj3));
IkReal x715=((px)*(r22)*(sj3));
IkReal x716=((IkReal(1.00000000000000))*(x703));
IkReal x717=((IkReal(0.0450000000000000))*(py)*(x706));
IkReal x718=((IkReal(0.0450000000000000))*(py)*(x702));
if( IKabs(((gconst7)*(((((IkReal(-1.00000000000000))*(x706)*(x714)))+(((IkReal(-0.0450000000000000))*(x702)*(x704)))+(((x702)*(x708)))+(((x702)*(x715)))+(((x702)*(x711)))+(((x706)*(x707)))+(((IkReal(-1.00000000000000))*(x709)*(x710)))+(((IkReal(-1.00000000000000))*(x709)*(x712)))+(((IkReal(-1.00000000000000))*(x717)))+(((IkReal(-1.00000000000000))*(x705)*(x716)))+(((r00)*(x702)*(x704)))+(x705)+(((cj3)*(x717))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst7)*(((((IkReal(-1.00000000000000))*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x707)*(x716)))+(((IkReal(-1.00000000000000))*(x706)*(x711)))+(((IkReal(-1.00000000000000))*(x702)*(x714)))+(((IkReal(-1.00000000000000))*(x710)*(x713)))+(((IkReal(-1.00000000000000))*(x709)*(x715)))+(((IkReal(-1.00000000000000))*(x718)))+(((IkReal(-1.00000000000000))*(x705)*(x709)))+(((IkReal(0.0450000000000000))*(x704)*(x706)))+(((IkReal(-1.00000000000000))*(x712)*(x713)))+(x707)+(((cj3)*(x718)))+(((IkReal(-1.00000000000000))*(r00)*(x704)*(x709))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst7)*(((((IkReal(-1.00000000000000))*(x706)*(x714)))+(((IkReal(-0.0450000000000000))*(x702)*(x704)))+(((x702)*(x708)))+(((x702)*(x715)))+(((x702)*(x711)))+(((x706)*(x707)))+(((IkReal(-1.00000000000000))*(x709)*(x710)))+(((IkReal(-1.00000000000000))*(x709)*(x712)))+(((IkReal(-1.00000000000000))*(x717)))+(((IkReal(-1.00000000000000))*(x705)*(x716)))+(((r00)*(x702)*(x704)))+(x705)+(((cj3)*(x717)))))), ((gconst7)*(((((IkReal(-1.00000000000000))*(x706)*(x708)))+(((IkReal(-1.00000000000000))*(x707)*(x716)))+(((IkReal(-1.00000000000000))*(x706)*(x711)))+(((IkReal(-1.00000000000000))*(x702)*(x714)))+(((IkReal(-1.00000000000000))*(x710)*(x713)))+(((IkReal(-1.00000000000000))*(x709)*(x715)))+(((IkReal(-1.00000000000000))*(x718)))+(((IkReal(-1.00000000000000))*(x705)*(x709)))+(((IkReal(0.0450000000000000))*(x704)*(x706)))+(((IkReal(-1.00000000000000))*(x712)*(x713)))+(x707)+(((cj3)*(x718)))+(((IkReal(-1.00000000000000))*(r00)*(x704)*(x709)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x719=IKsin(j0);
IkReal x720=IKcos(j0);
IkReal x721=((sj2)*(sj3));
IkReal x722=((IkReal(0.0450000000000000))*(cj2));
IkReal x723=((IkReal(0.0900000000000000))*(py));
IkReal x724=((IkReal(0.0450000000000000))*(cj3));
IkReal x725=((cj3)*(r00));
IkReal x726=((cj2)*(sj3));
IkReal x727=((IkReal(1.00000000000000))*(py));
IkReal x728=((px)*(x719));
IkReal x729=((sj2)*(x720));
IkReal x730=((cj2)*(x719));
IkReal x731=((cj2)*(px)*(x720));
evalcond[0]=((((py)*(sj2)*(x719)))+(((IkReal(-1.00000000000000))*(cj2)*(x720)*(x727)))+(((cj2)*(x728)))+(r11)+(((px)*(x729))));
evalcond[1]=((((IkReal(0.0450000000000000))*(sj2)))+(((sj2)*(x725)))+(((cj2)*(r11)))+(((IkReal(-1.00000000000000))*(x720)*(x727)))+(((IkReal(-1.00000000000000))*(sj2)*(x724)))+(((IkReal(0.350000000000000))*(x721)))+(x728)+(((r22)*(x721))));
evalcond[2]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x727)*(x729)))+(((IkReal(-1.00000000000000))*(x727)*(x730)))+(((r22)*(sj3)))+(((sj2)*(x728)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-1.00000000000000))*(x731)))+(((IkReal(-1.00000000000000))*(x724)))+(x725));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((x723)*(x730)))+(((IkReal(0.0900000000000000))*(x731)))+(((x723)*(x729)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(sj2)*(x728)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj3)*(x722)))+(((cj2)*(x725)))+(((IkReal(-1.00000000000000))*(r11)*(sj2)))+(((IkReal(0.350000000000000))*(x726)))+(x722)+(((IkReal(-1.00000000000000))*(x719)*(x727)))+(((r22)*(x726)))+(((IkReal(-1.00000000000000))*(px)*(x720))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x732=((IkReal(0.0450000000000000))*(sj3));
IkReal x733=((IkReal(0.350000000000000))*(cj3));
IkReal x734=((r00)*(sj3));
IkReal x735=((IkReal(0.0900000000000000))*(r00));
IkReal x736=((cj3)*(r22));
IkReal x737=((x733)+(x732));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j1)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x736)))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x737)))+(x734));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(1.10000000000000))*(x736)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x735)))+(((cj3)*(x735)))+(((IkReal(-1.10000000000000))*(x734))));
evalcond[3]=((IkReal(0.550000000000000))+(((IkReal(-1.00000000000000))*(x734)))+(pz)+(x737)+(x736));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst10;
gconst10=IKsign(((((sj2)*((px)*(px))))+(((sj2)*((py)*(py))))));
dummyeval[0]=((((sj2)*((px)*(px))))+(((sj2)*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x738=((IkReal(0.0450000000000000))*(sj3));
IkReal x739=((IkReal(0.350000000000000))*(cj3));
IkReal x740=((r00)*(sj3));
IkReal x741=((IkReal(0.0900000000000000))*(r00));
IkReal x742=((cj3)*(r22));
IkReal x743=((x739)+(x738));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x742)))+(x740)+(((IkReal(-1.00000000000000))*(x743))));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((cj3)*(x741)))+((r00)*(r00))+(((IkReal(-1.10000000000000))*(x740)))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x742)))+(((IkReal(-1.00000000000000))*(x741))));
evalcond[3]=((IkReal(0.550000000000000))+(pz)+(((IkReal(-1.00000000000000))*(x740)))+(x742)+(x743));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst11;
gconst11=IKsign(((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((px)*(px))))+(((IkReal(-1.00000000000000))*((py)*(py)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x744=((py)*(sj3));
IkReal x745=((IkReal(0.0450000000000000))*(px));
IkReal x746=((px)*(sj3));
IkReal x747=((IkReal(0.0450000000000000))*(py));
IkReal x748=((cj3)*(r00));
if( IKabs(((gconst11)*(((((IkReal(-1.00000000000000))*(cj3)*(x747)))+(((py)*(x748)))+(((r22)*(x744)))+(((IkReal(0.350000000000000))*(x744)))+(((px)*(r11)))+(x747))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst11)*(((((IkReal(-1.00000000000000))*(cj3)*(x745)))+(((r22)*(x746)))+(((IkReal(0.350000000000000))*(x746)))+(((px)*(x748)))+(((IkReal(-1.00000000000000))*(py)*(r11)))+(x745))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst11)*(((((IkReal(-1.00000000000000))*(cj3)*(x747)))+(((py)*(x748)))+(((r22)*(x744)))+(((IkReal(0.350000000000000))*(x744)))+(((px)*(r11)))+(x747)))), ((gconst11)*(((((IkReal(-1.00000000000000))*(cj3)*(x745)))+(((r22)*(x746)))+(((IkReal(0.350000000000000))*(x746)))+(((px)*(x748)))+(((IkReal(-1.00000000000000))*(py)*(r11)))+(x745)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x749=IKsin(j0);
IkReal x750=IKcos(j0);
IkReal x751=((px)*(x750));
IkReal x752=((py)*(x749));
evalcond[0]=((((px)*(x749)))+(r11)+(((IkReal(-1.00000000000000))*(py)*(x750))));
evalcond[1]=((IkReal(0.0450000000000000))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(x751)+(x752));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-0.0900000000000000))*(x751)))+(((IkReal(-0.0900000000000000))*(x752)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x753=((IkReal(0.0450000000000000))*(sj3));
IkReal x754=((IkReal(0.350000000000000))*(cj3));
IkReal x755=((r00)*(sj3));
IkReal x756=((IkReal(0.0900000000000000))*(r00));
IkReal x757=((cj3)*(r22));
IkReal x758=((x754)+(x753));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x757)))+(x755)+(((IkReal(-1.00000000000000))*(x758))));
evalcond[2]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(x755)))+(((cj3)*(x756)))+(((IkReal(-1.00000000000000))*(x756)))+(((IkReal(1.10000000000000))*(x757))));
evalcond[3]=((IkReal(0.550000000000000))+(pz)+(((IkReal(-1.00000000000000))*(x755)))+(x757)+(x758));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst12;
gconst12=IKsign((((px)*(px))+((py)*(py))));
dummyeval[0]=(((px)*(px))+((py)*(py)));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x759=((py)*(sj3));
IkReal x760=((IkReal(0.0450000000000000))*(px));
IkReal x761=((px)*(sj3));
IkReal x762=((IkReal(0.0450000000000000))*(py));
IkReal x763=((cj3)*(r00));
if( IKabs(((gconst12)*(((((IkReal(0.350000000000000))*(x759)))+(((px)*(r11)))+(((r22)*(x759)))+(((IkReal(-1.00000000000000))*(cj3)*(x762)))+(((py)*(x763)))+(x762))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst12)*(((((px)*(x763)))+(((IkReal(-1.00000000000000))*(cj3)*(x760)))+(((IkReal(-1.00000000000000))*(py)*(r11)))+(((r22)*(x761)))+(((IkReal(0.350000000000000))*(x761)))+(x760))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst12)*(((((IkReal(0.350000000000000))*(x759)))+(((px)*(r11)))+(((r22)*(x759)))+(((IkReal(-1.00000000000000))*(cj3)*(x762)))+(((py)*(x763)))+(x762)))), ((gconst12)*(((((px)*(x763)))+(((IkReal(-1.00000000000000))*(cj3)*(x760)))+(((IkReal(-1.00000000000000))*(py)*(r11)))+(((r22)*(x761)))+(((IkReal(0.350000000000000))*(x761)))+(x760)))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[3];
IkReal x764=IKcos(j0);
IkReal x765=IKsin(j0);
IkReal x766=((px)*(x764));
IkReal x767=((IkReal(1.00000000000000))*(x765));
evalcond[0]=((((IkReal(-1.00000000000000))*(px)*(x767)))+(r11)+(((py)*(x764))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(py)*(x767)))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((IkReal(-0.0450000000000000))*(cj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x766))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.0900000000000000))*(x766)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(0.0900000000000000))*(py)*(x765)))+(((IkReal(-0.0900000000000000))*(r00))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x768=(sj2)*(sj2);
IkReal x769=(cj2)*(cj2);
IkReal x770=((cj2)*(sj2));
IkReal x771=((py)*(r11));
IkReal x772=((IkReal(0.0450000000000000))*(py));
IkReal x773=((IkReal(0.0450000000000000))*(px));
IkReal x774=((IkReal(0.350000000000000))*(sj3));
IkReal x775=((px)*(r11));
IkReal x776=((IkReal(1.00000000000000))*(r22)*(sj3));
IkReal x777=((px)*(x768));
IkReal x778=((cj3)*(py)*(r00));
IkReal x779=((py)*(x768));
IkReal x780=((IkReal(1.00000000000000))*(x769));
IkReal x781=((IkReal(1.00000000000000))*(cj3)*(px)*(r00));
if( IKabs(((gconst10)*(((((IkReal(-1.00000000000000))*(x768)*(x773)))+(((IkReal(-1.00000000000000))*(x774)*(x777)))+(((cj3)*(x768)*(x773)))+(((IkReal(-1.00000000000000))*(py)*(x770)*(x776)))+(((IkReal(-1.00000000000000))*(py)*(x770)*(x774)))+(((IkReal(-1.00000000000000))*(x771)*(x780)))+(((cj3)*(x770)*(x772)))+(((IkReal(-1.00000000000000))*(x770)*(x778)))+(((IkReal(-1.00000000000000))*(x770)*(x775)))+(((IkReal(-1.00000000000000))*(x776)*(x777)))+(x771)+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x777)))+(((IkReal(-1.00000000000000))*(x770)*(x772))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst10)*(((((x774)*(x779)))+(((IkReal(-1.00000000000000))*(x770)*(x781)))+(((x770)*(x771)))+(((IkReal(-1.00000000000000))*(x775)*(x780)))+(((r22)*(sj3)*(x779)))+(((cj3)*(x770)*(x773)))+(((IkReal(-1.00000000000000))*(px)*(x770)*(x776)))+(((IkReal(-1.00000000000000))*(px)*(x770)*(x774)))+(((IkReal(-1.00000000000000))*(cj3)*(x768)*(x772)))+(x775)+(((IkReal(-1.00000000000000))*(x770)*(x773)))+(((x768)*(x778)))+(((x768)*(x772))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst10)*(((((IkReal(-1.00000000000000))*(x768)*(x773)))+(((IkReal(-1.00000000000000))*(x774)*(x777)))+(((cj3)*(x768)*(x773)))+(((IkReal(-1.00000000000000))*(py)*(x770)*(x776)))+(((IkReal(-1.00000000000000))*(py)*(x770)*(x774)))+(((IkReal(-1.00000000000000))*(x771)*(x780)))+(((cj3)*(x770)*(x772)))+(((IkReal(-1.00000000000000))*(x770)*(x778)))+(((IkReal(-1.00000000000000))*(x770)*(x775)))+(((IkReal(-1.00000000000000))*(x776)*(x777)))+(x771)+(((IkReal(-1.00000000000000))*(cj3)*(r00)*(x777)))+(((IkReal(-1.00000000000000))*(x770)*(x772)))))), ((gconst10)*(((((x774)*(x779)))+(((IkReal(-1.00000000000000))*(x770)*(x781)))+(((x770)*(x771)))+(((IkReal(-1.00000000000000))*(x775)*(x780)))+(((r22)*(sj3)*(x779)))+(((cj3)*(x770)*(x773)))+(((IkReal(-1.00000000000000))*(px)*(x770)*(x776)))+(((IkReal(-1.00000000000000))*(px)*(x770)*(x774)))+(((IkReal(-1.00000000000000))*(cj3)*(x768)*(x772)))+(x775)+(((IkReal(-1.00000000000000))*(x770)*(x773)))+(((x768)*(x778)))+(((x768)*(x772)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[5];
IkReal x782=IKcos(j0);
IkReal x783=IKsin(j0);
IkReal x784=((sj2)*(sj3));
IkReal x785=((IkReal(0.0450000000000000))*(cj2));
IkReal x786=((IkReal(0.0900000000000000))*(sj2));
IkReal x787=((IkReal(0.0450000000000000))*(cj3));
IkReal x788=((IkReal(1.00000000000000))*(sj2));
IkReal x789=((IkReal(0.0900000000000000))*(cj2));
IkReal x790=((cj3)*(r00));
IkReal x791=((IkReal(1.00000000000000))*(cj2));
IkReal x792=((r22)*(sj3));
IkReal x793=((IkReal(0.350000000000000))*(sj3));
IkReal x794=((px)*(x783));
IkReal x795=((py)*(x782));
IkReal x796=((px)*(x782));
IkReal x797=((py)*(x783));
evalcond[0]=((((IkReal(-1.00000000000000))*(x788)*(x797)))+(((IkReal(-1.00000000000000))*(x788)*(x796)))+(((IkReal(-1.00000000000000))*(x791)*(x795)))+(r11)+(((cj2)*(x794))));
evalcond[1]=((((IkReal(0.0450000000000000))*(sj2)))+(((cj2)*(r11)))+(((r22)*(x784)))+(((IkReal(0.350000000000000))*(x784)))+(x794)+(((IkReal(-1.00000000000000))*(x795)))+(((IkReal(-1.00000000000000))*(sj2)*(x787)))+(((sj2)*(x790))));
evalcond[2]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x787)))+(((IkReal(-1.00000000000000))*(x788)*(x795)))+(x790)+(x793)+(x792)+(((cj2)*(x796)))+(((cj2)*(x797)))+(((sj2)*(x794))));
evalcond[3]=((IkReal(-0.180000000000000))+(((x786)*(x795)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.10000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x786)*(x794)))+(((IkReal(-1.00000000000000))*(x789)*(x796)))+(((IkReal(-1.00000000000000))*(x789)*(x797)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((IkReal(-1.00000000000000))*(cj2)*(x793)))+(((IkReal(-1.00000000000000))*(x785)))+(((r11)*(sj2)))+(((IkReal(-1.00000000000000))*(x791)*(x792)))+(((IkReal(-1.00000000000000))*(x790)*(x791)))+(((IkReal(-1.00000000000000))*(x796)))+(((IkReal(-1.00000000000000))*(x797)))+(((cj3)*(x785))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}
}
}

} else
{
{
IkReal j0array[1], cj0array[1], sj0array[1];
bool j0valid[1]={false};
_nj0 = 1;
IkReal x798=(sj1)*(sj1);
IkReal x799=(cj1)*(cj1);
IkReal x800=((IkReal(0.550000000000000))*(cj2));
IkReal x801=((IkReal(1.00000000000000))*(px));
IkReal x802=((cj2)*(cj3));
IkReal x803=((cj1)*(sj2));
IkReal x804=((IkReal(0.350000000000000))*(py));
IkReal x805=((r11)*(sj1));
IkReal x806=((IkReal(1.00000000000000))*(py));
IkReal x807=((r00)*(sj3));
IkReal x808=((IkReal(0.350000000000000))*(px));
IkReal x809=((cj3)*(r22));
IkReal x810=((IkReal(0.0450000000000000))*(px)*(sj3));
IkReal x811=((cj1)*(cj2)*(pz));
IkReal x812=((IkReal(0.0450000000000000))*(py)*(sj3));
IkReal x813=((pz)*(sj2)*(x799));
IkReal x814=((pz)*(sj2)*(x798));
if( IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(cj2)*(x812)))+(((py)*(x811)))+(((cj2)*(py)*(x807)))+(((IkReal(-1.00000000000000))*(x801)*(x813)))+(((IkReal(-1.00000000000000))*(x801)*(x814)))+(((IkReal(-1.00000000000000))*(x801)*(x803)*(x807)))+(((cj3)*(x803)*(x808)))+(((px)*(x805)))+(((px)*(x803)*(x809)))+(((IkReal(-1.00000000000000))*(r22)*(x802)*(x806)))+(((IkReal(-1.00000000000000))*(py)*(x800)))+(((IkReal(-1.00000000000000))*(x802)*(x804)))+(((x803)*(x810)))+(((IkReal(0.550000000000000))*(px)*(x803))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst0)*(((((IkReal(-1.00000000000000))*(r22)*(x801)*(x802)))+(((IkReal(-1.00000000000000))*(x803)*(x812)))+(((IkReal(-1.00000000000000))*(cj2)*(x810)))+(((py)*(x813)))+(((py)*(x814)))+(((IkReal(-1.00000000000000))*(px)*(x800)))+(((IkReal(-1.00000000000000))*(x805)*(x806)))+(((IkReal(-1.00000000000000))*(x803)*(x806)*(x809)))+(((px)*(x811)))+(((py)*(x803)*(x807)))+(((IkReal(-0.550000000000000))*(py)*(x803)))+(((IkReal(-1.00000000000000))*(cj3)*(x803)*(x804)))+(((cj2)*(px)*(x807)))+(((IkReal(-1.00000000000000))*(x802)*(x808))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j0array[0]=IKatan2(((gconst0)*(((((IkReal(-1.00000000000000))*(cj2)*(x812)))+(((py)*(x811)))+(((cj2)*(py)*(x807)))+(((IkReal(-1.00000000000000))*(x801)*(x813)))+(((IkReal(-1.00000000000000))*(x801)*(x814)))+(((IkReal(-1.00000000000000))*(x801)*(x803)*(x807)))+(((cj3)*(x803)*(x808)))+(((px)*(x805)))+(((px)*(x803)*(x809)))+(((IkReal(-1.00000000000000))*(r22)*(x802)*(x806)))+(((IkReal(-1.00000000000000))*(py)*(x800)))+(((IkReal(-1.00000000000000))*(x802)*(x804)))+(((x803)*(x810)))+(((IkReal(0.550000000000000))*(px)*(x803)))))), ((gconst0)*(((((IkReal(-1.00000000000000))*(r22)*(x801)*(x802)))+(((IkReal(-1.00000000000000))*(x803)*(x812)))+(((IkReal(-1.00000000000000))*(cj2)*(x810)))+(((py)*(x813)))+(((py)*(x814)))+(((IkReal(-1.00000000000000))*(px)*(x800)))+(((IkReal(-1.00000000000000))*(x805)*(x806)))+(((IkReal(-1.00000000000000))*(x803)*(x806)*(x809)))+(((px)*(x811)))+(((py)*(x803)*(x807)))+(((IkReal(-0.550000000000000))*(py)*(x803)))+(((IkReal(-1.00000000000000))*(cj3)*(x803)*(x804)))+(((cj2)*(px)*(x807)))+(((IkReal(-1.00000000000000))*(x802)*(x808)))))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
for(int ij0 = 0; ij0 < 1; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 1; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[6];
IkReal x815=IKcos(j0);
IkReal x816=IKsin(j0);
IkReal x817=((sj2)*(sj3));
IkReal x818=((cj1)*(px));
IkReal x819=((IkReal(0.0900000000000000))*(py));
IkReal x820=((IkReal(0.0450000000000000))*(cj3));
IkReal x821=((IkReal(0.350000000000000))*(cj3));
IkReal x822=((IkReal(1.10000000000000))*(sj1));
IkReal x823=((r00)*(sj3));
IkReal x824=((IkReal(1.00000000000000))*(sj1));
IkReal x825=((cj1)*(sj2));
IkReal x826=((cj3)*(r00));
IkReal x827=((cj1)*(cj2));
IkReal x828=((IkReal(1.00000000000000))*(py));
IkReal x829=((IkReal(0.0450000000000000))*(sj3));
IkReal x830=((cj3)*(r22));
IkReal x831=((r22)*(sj3));
IkReal x832=((IkReal(0.350000000000000))*(sj3));
IkReal x833=((cj1)*(pz));
IkReal x834=((px)*(x816));
IkReal x835=((sj2)*(x815));
IkReal x836=((px)*(x815));
IkReal x837=((py)*(x816));
IkReal x838=((cj2)*(x815));
IkReal x839=((cj2)*(pz)*(sj1));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x829)))+(((IkReal(-1.00000000000000))*(x821)))+(((IkReal(-1.00000000000000))*(x830)))+(x823)+(x833)+(((sj1)*(x837)))+(((sj1)*(x836))));
evalcond[1]=((((x818)*(x835)))+(((IkReal(-1.00000000000000))*(x828)*(x838)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x824)))+(r11)+(((cj2)*(x834)))+(((x825)*(x837))));
evalcond[2]=((((IkReal(0.350000000000000))*(x817)))+(((IkReal(0.0450000000000000))*(sj2)))+(((IkReal(-1.00000000000000))*(x815)*(x828)))+(((sj2)*(x826)))+(((cj2)*(r11)))+(((IkReal(-1.00000000000000))*(sj2)*(x820)))+(x834)+(((r22)*(x817))));
evalcond[3]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x818)*(x838)))+(((IkReal(-1.00000000000000))*(x828)*(x835)))+(((IkReal(-1.00000000000000))*(x820)))+(((IkReal(-1.00000000000000))*(x816)*(x827)*(x828)))+(x826)+(x831)+(x832)+(x839)+(((sj2)*(x834))));
evalcond[4]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+(((x819)*(x835)))+(((IkReal(0.0900000000000000))*(x818)*(x838)))+((r22)*(r22))+((r11)*(r11))+(((x816)*(x819)*(x827)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(sj2)*(x834)))+(((IkReal(-0.0900000000000000))*(x839)))+(((x822)*(x837)))+(((x822)*(x836)))+(((IkReal(1.10000000000000))*(x833)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x816)*(x828)))+(((sj1)*(x821)))+(((sj1)*(x829)))+(((IkReal(-1.00000000000000))*(x823)*(x824)))+(((x827)*(x832)))+(((x827)*(x831)))+(((IkReal(-1.00000000000000))*(x820)*(x827)))+(((IkReal(-1.00000000000000))*(x836)))+(((IkReal(0.550000000000000))*(sj1)))+(((IkReal(-1.00000000000000))*(r11)*(x825)))+(((sj1)*(x830)))+(((IkReal(0.0450000000000000))*(x827)))+(((x826)*(x827))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x840=((sj2)*(sj3));
IkReal x841=((IkReal(0.0450000000000000))*(sj2));
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x842=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((r22)*(x840)))+(((IkReal(0.350000000000000))*(x840)))+(((cj2)*(r11)))+(((cj3)*(r00)*(sj2)))+(x841)+(((IkReal(-1.00000000000000))*(cj3)*(x841))))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((r22)*(x840)))+(((IkReal(0.350000000000000))*(x840)))+(((cj2)*(r11)))+(((cj3)*(r00)*(sj2)))+(x841)+(((IkReal(-1.00000000000000))*(cj3)*(x841))))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x843=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((r22)*(x840)))+(((IkReal(0.350000000000000))*(x840)))+(((cj2)*(r11)))+(((cj3)*(r00)*(sj2)))+(x841)+(((IkReal(-1.00000000000000))*(cj3)*(x841)))))));
j0array[0]=((((IkReal(-1.00000000000000))*(x842)))+(((IkReal(-1.00000000000000))*(x843))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x842)))+(x843));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
{
IkReal evalcond[2];
IkReal x844=(py)*(py);
IkReal x845=(cj2)*(cj2);
IkReal x846=(sj2)*(sj2);
IkReal x847=IKcos(j0);
IkReal x848=IKsin(j0);
IkReal x849=(px)*(px);
IkReal x850=((r22)*(sj3));
IkReal x851=((cj3)*(r00));
IkReal x852=((IkReal(0.0450000000000000))*(sj2));
IkReal x853=((px)*(sj2));
IkReal x854=((IkReal(0.350000000000000))*(sj3));
IkReal x855=((IkReal(1.00000000000000))*(cj2)*(r11));
IkReal x856=((IkReal(1.00000000000000))*(py)*(sj2));
IkReal x857=((IkReal(1.00000000000000))*(x849));
IkReal x858=((px)*(py)*(x846));
IkReal x859=((px)*(py)*(x845));
evalcond[0]=((((x847)*(((((x844)*(x846)))+(((x844)*(x845)))))))+(((x848)*(((((IkReal(-1.00000000000000))*(x858)))+(((IkReal(-1.00000000000000))*(x859)))))))+(((IkReal(-1.00000000000000))*(py)*(sj2)*(x854)))+(((IkReal(-1.00000000000000))*(x850)*(x856)))+(((IkReal(-1.00000000000000))*(py)*(x855)))+(((IkReal(-1.00000000000000))*(py)*(x852)))+(((cj3)*(py)*(x852)))+(((IkReal(-1.00000000000000))*(x851)*(x856))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x850)*(x853)))+(((IkReal(-1.00000000000000))*(x853)*(x854)))+(((IkReal(-1.00000000000000))*(x851)*(x853)))+(((x847)*(((x858)+(x859)))))+(((cj3)*(px)*(x852)))+(((IkReal(-1.00000000000000))*(px)*(x855)))+(((IkReal(-1.00000000000000))*(px)*(x852)))+(((x848)*(((((IkReal(-1.00000000000000))*(x846)*(x857)))+(((IkReal(-1.00000000000000))*(x845)*(x857))))))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  )
{
continue;
}
}

{
IkReal dummyeval[1];
IkReal gconst13;
gconst13=IKsign(((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x860=((IkReal(0.0900000000000000))*(r00));
IkReal x861=((r11)+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x861;
evalcond[2]=x861;
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x860)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+(((cj3)*(x860)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst14;
gconst14=IKsign(((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x862=((pz)*(sj3));
IkReal x863=((py)*(sj0));
IkReal x864=((IkReal(1.00000000000000))*(cj3));
IkReal x865=((cj0)*(px));
IkReal x866=((r00)*(sj3));
IkReal x867=((IkReal(0.350000000000000))*(cj3));
IkReal x868=((IkReal(0.350000000000000))*(sj3));
IkReal x869=((IkReal(0.0450000000000000))*(sj3));
IkReal x870=((IkReal(0.0450000000000000))*(pz));
IkReal x871=((IkReal(0.0450000000000000))*(cj3));
IkReal x872=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst14)*(((((IkReal(-1.00000000000000))*(x865)*(x869)))+(((IkReal(-1.00000000000000))*(x865)*(x867)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(r22)*(x863)*(x864)))+(((IkReal(-1.00000000000000))*(cj3)*(x870)))+(((x863)*(x866)))+(((r22)*(x862)))+(((IkReal(-0.550000000000000))*(x865)))+(((IkReal(-0.550000000000000))*(x863)))+(((IkReal(0.350000000000000))*(x862)))+(x870)+(((x865)*(x866)))+(((IkReal(-1.00000000000000))*(x863)*(x869)))+(((IkReal(-1.00000000000000))*(x863)*(x867)))+(((IkReal(-1.00000000000000))*(r22)*(x864)*(x865))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(r00)*(x864)*(x865)))+(((IkReal(-1.00000000000000))*(x865)*(x868)))+(((r00)*(x862)))+(((IkReal(-1.00000000000000))*(r00)*(x863)*(x864)))+(((x863)*(x871)))+(((IkReal(-1.00000000000000))*(pz)*(x867)))+(((x865)*(x871)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x864)))+(((IkReal(-1.00000000000000))*(x863)*(x872)))+(((IkReal(-0.0450000000000000))*(x865)))+(((IkReal(-0.0450000000000000))*(x863)))+(((IkReal(-0.0450000000000000))*(x862)))+(((IkReal(-1.00000000000000))*(x865)*(x872)))+(((IkReal(-1.00000000000000))*(x863)*(x868))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst14)*(((((IkReal(-1.00000000000000))*(x865)*(x869)))+(((IkReal(-1.00000000000000))*(x865)*(x867)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(r22)*(x863)*(x864)))+(((IkReal(-1.00000000000000))*(cj3)*(x870)))+(((x863)*(x866)))+(((r22)*(x862)))+(((IkReal(-0.550000000000000))*(x865)))+(((IkReal(-0.550000000000000))*(x863)))+(((IkReal(0.350000000000000))*(x862)))+(x870)+(((x865)*(x866)))+(((IkReal(-1.00000000000000))*(x863)*(x869)))+(((IkReal(-1.00000000000000))*(x863)*(x867)))+(((IkReal(-1.00000000000000))*(r22)*(x864)*(x865)))))), ((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(r00)*(x864)*(x865)))+(((IkReal(-1.00000000000000))*(x865)*(x868)))+(((r00)*(x862)))+(((IkReal(-1.00000000000000))*(r00)*(x863)*(x864)))+(((x863)*(x871)))+(((IkReal(-1.00000000000000))*(pz)*(x867)))+(((x865)*(x871)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x864)))+(((IkReal(-1.00000000000000))*(x863)*(x872)))+(((IkReal(-0.0450000000000000))*(x865)))+(((IkReal(-0.0450000000000000))*(x863)))+(((IkReal(-0.0450000000000000))*(x862)))+(((IkReal(-1.00000000000000))*(x865)*(x872)))+(((IkReal(-1.00000000000000))*(x863)*(x868)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x873=IKsin(j1);
IkReal x874=IKcos(j1);
IkReal x875=((r22)*(sj3));
IkReal x876=((IkReal(0.0450000000000000))*(cj3));
IkReal x877=((cj0)*(px));
IkReal x878=((cj3)*(r22));
IkReal x879=((r00)*(sj3));
IkReal x880=((py)*(sj0));
IkReal x881=((IkReal(0.350000000000000))*(sj3));
IkReal x882=((IkReal(0.350000000000000))*(cj3));
IkReal x883=((IkReal(0.0450000000000000))*(sj3));
IkReal x884=((cj3)*(r00));
IkReal x885=((IkReal(1.00000000000000))*(x874));
IkReal x886=((sj3)*(x874));
IkReal x887=((IkReal(0.0900000000000000))*(x874));
IkReal x888=((pz)*(x873));
IkReal x889=((pz)*(x874));
IkReal x890=((IkReal(1.10000000000000))*(x873));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x878)))+(((x873)*(x877)))+(((IkReal(-1.00000000000000))*(x882)))+(((IkReal(-1.00000000000000))*(x883)))+(x879)+(x889)+(((x873)*(x880))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x877)*(x885)))+(x875)+(x884)+(x881)+(x888)+(((IkReal(-1.00000000000000))*(x880)*(x885)))+(((IkReal(-1.00000000000000))*(x876))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(1.10000000000000))*(x889)))+(((IkReal(0.700000000000000))*(r22)))+(((x880)*(x890)))+(((x880)*(x887)))+((r22)*(r22))+((r11)*(r11))+(((x877)*(x887)))+(((x877)*(x890)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-0.0900000000000000))*(x888)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((x874)*(x879)))+(((IkReal(0.0450000000000000))*(x873)))+(((x873)*(x875)))+(((IkReal(-1.00000000000000))*(x873)*(x876)))+(pz)+(((IkReal(-0.550000000000000))*(x874)))+(((IkReal(-1.00000000000000))*(x874)*(x882)))+(((IkReal(-1.00000000000000))*(x874)*(x883)))+(((x873)*(x881)))+(((x873)*(x884)))+(((IkReal(-1.00000000000000))*(x878)*(x885))));
evalcond[4]=((((x874)*(x881)))+(((x874)*(x884)))+(((x874)*(x875)))+(((IkReal(0.0450000000000000))*(x874)))+(((IkReal(-1.00000000000000))*(x877)))+(((IkReal(0.550000000000000))*(x873)))+(((x873)*(x878)))+(((IkReal(-1.00000000000000))*(x880)))+(((IkReal(-1.00000000000000))*(x873)*(x879)))+(((IkReal(-1.00000000000000))*(x874)*(x876)))+(((x873)*(x883)))+(((x873)*(x882))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x891=((IkReal(0.0900000000000000))*(r00));
IkReal x892=((px)*(sj0));
IkReal x893=((cj0)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x892)))+(r11)+(x893));
evalcond[2]=((((IkReal(-1.00000000000000))*(r11)))+(((IkReal(-1.00000000000000))*(x893)))+(x892));
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((cj3)*(x891)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x891)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst15;
gconst15=IKsign((((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)))));
dummyeval[0]=(((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x894=((pz)*(sj3));
IkReal x895=((py)*(sj0));
IkReal x896=((IkReal(1.00000000000000))*(r00));
IkReal x897=((cj0)*(px));
IkReal x898=((IkReal(0.350000000000000))*(cj3));
IkReal x899=((IkReal(0.350000000000000))*(sj3));
IkReal x900=((IkReal(0.0450000000000000))*(sj3));
IkReal x901=((IkReal(0.0450000000000000))*(cj3));
IkReal x902=((cj3)*(r22));
IkReal x903=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst15)*(((((x895)*(x900)))+(((x895)*(x902)))+(((IkReal(0.350000000000000))*(x894)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(sj3)*(x896)*(x897)))+(((IkReal(-1.00000000000000))*(pz)*(x901)))+(((x895)*(x898)))+(((IkReal(0.0450000000000000))*(pz)))+(((x897)*(x900)))+(((x897)*(x902)))+(((IkReal(-1.00000000000000))*(sj3)*(x895)*(x896)))+(((x897)*(x898)))+(((IkReal(0.550000000000000))*(x897)))+(((IkReal(0.550000000000000))*(x895)))+(((r22)*(x894))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst15)*(((((x895)*(x901)))+(((IkReal(-1.00000000000000))*(cj3)*(x895)*(x896)))+(((IkReal(-1.00000000000000))*(x895)*(x903)))+(((IkReal(-1.00000000000000))*(x897)*(x903)))+(((pz)*(x898)))+(((IkReal(-1.00000000000000))*(x895)*(x899)))+(((IkReal(-0.0450000000000000))*(x895)))+(((IkReal(-0.0450000000000000))*(x897)))+(((x897)*(x901)))+(((IkReal(-1.00000000000000))*(x894)*(x896)))+(((IkReal(-1.00000000000000))*(x897)*(x899)))+(((IkReal(0.0450000000000000))*(x894)))+(((pz)*(x902)))+(((IkReal(0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj3)*(x896)*(x897))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst15)*(((((x895)*(x900)))+(((x895)*(x902)))+(((IkReal(0.350000000000000))*(x894)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(sj3)*(x896)*(x897)))+(((IkReal(-1.00000000000000))*(pz)*(x901)))+(((x895)*(x898)))+(((IkReal(0.0450000000000000))*(pz)))+(((x897)*(x900)))+(((x897)*(x902)))+(((IkReal(-1.00000000000000))*(sj3)*(x895)*(x896)))+(((x897)*(x898)))+(((IkReal(0.550000000000000))*(x897)))+(((IkReal(0.550000000000000))*(x895)))+(((r22)*(x894)))))), ((gconst15)*(((((x895)*(x901)))+(((IkReal(-1.00000000000000))*(cj3)*(x895)*(x896)))+(((IkReal(-1.00000000000000))*(x895)*(x903)))+(((IkReal(-1.00000000000000))*(x897)*(x903)))+(((pz)*(x898)))+(((IkReal(-1.00000000000000))*(x895)*(x899)))+(((IkReal(-0.0450000000000000))*(x895)))+(((IkReal(-0.0450000000000000))*(x897)))+(((x897)*(x901)))+(((IkReal(-1.00000000000000))*(x894)*(x896)))+(((IkReal(-1.00000000000000))*(x897)*(x899)))+(((IkReal(0.0450000000000000))*(x894)))+(((pz)*(x902)))+(((IkReal(0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(cj3)*(x896)*(x897)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x904=IKsin(j1);
IkReal x905=IKcos(j1);
IkReal x906=((IkReal(1.00000000000000))*(r22));
IkReal x907=((IkReal(0.0450000000000000))*(cj3));
IkReal x908=((cj0)*(px));
IkReal x909=((r00)*(sj3));
IkReal x910=((IkReal(1.00000000000000))*(r00));
IkReal x911=((py)*(sj0));
IkReal x912=((sj3)*(x904));
IkReal x913=((cj3)*(x905));
IkReal x914=((cj3)*(x904));
IkReal x915=((sj3)*(x905));
IkReal x916=((IkReal(0.0900000000000000))*(x905));
IkReal x917=((pz)*(x904));
IkReal x918=((pz)*(x905));
IkReal x919=((IkReal(1.10000000000000))*(x904));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(cj3)*(x906)))+(x918)+(x909)+(((x904)*(x911)))+(((x904)*(x908))));
evalcond[1]=((IkReal(0.0450000000000000))+(((x905)*(x908)))+(((r22)*(sj3)))+(((IkReal(-1.00000000000000))*(x907)))+(((IkReal(0.350000000000000))*(sj3)))+(((cj3)*(r00)))+(((IkReal(-1.00000000000000))*(x917)))+(((x905)*(x911))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(-1.00000000000000))*(x908)*(x916)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+(((IkReal(1.10000000000000))*(x918)))+((r11)*(r11))+(((x911)*(x919)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((x908)*(x919)))+(((IkReal(-1.00000000000000))*(x911)*(x916)))+(((IkReal(0.0900000000000000))*(x917)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((x905)*(x909)))+(((IkReal(-1.00000000000000))*(x906)*(x912)))+(((IkReal(-1.00000000000000))*(x906)*(x913)))+(((IkReal(-0.350000000000000))*(x913)))+(((IkReal(-0.350000000000000))*(x912)))+(((IkReal(-0.550000000000000))*(x905)))+(((IkReal(-1.00000000000000))*(x910)*(x914)))+(pz)+(((IkReal(-0.0450000000000000))*(x904)))+(((IkReal(-0.0450000000000000))*(x915)))+(((x904)*(x907))));
evalcond[4]=((((x905)*(x907)))+(((IkReal(-1.00000000000000))*(x906)*(x915)))+(((IkReal(-0.350000000000000))*(x915)))+(((IkReal(0.0450000000000000))*(x912)))+(((IkReal(-1.00000000000000))*(x904)*(x909)))+(((IkReal(-1.00000000000000))*(x910)*(x913)))+(((IkReal(-0.0450000000000000))*(x905)))+(((IkReal(0.350000000000000))*(x914)))+(((IkReal(-1.00000000000000))*(x911)))+(((r22)*(x914)))+(((IkReal(-1.00000000000000))*(x908)))+(((IkReal(0.550000000000000))*(x904))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x920=((cj3)*(r22));
IkReal x921=((cj2)*(sj0));
IkReal x922=((IkReal(1.00000000000000))*(cj0));
IkReal x923=((sj2)*(sj3));
IkReal x924=((cj0)*(px));
IkReal x925=((pz)*(sj2));
IkReal x926=((IkReal(0.350000000000000))*(cj3));
IkReal x927=((IkReal(1.00000000000000))*(r00));
IkReal x928=((IkReal(1.00000000000000))*(cj2)*(py));
IkReal x929=((py)*(sj0)*(sj2));
if( IKabs(((gconst13)*(((((px)*(pz)*(x921)))+(((x920)*(x929)))+(((sj2)*(x924)*(x926)))+(((IkReal(0.0450000000000000))*(x923)*(x924)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x923)))+(((IkReal(0.550000000000000))*(x929)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x922)))+(((x926)*(x929)))+(((pz)*(r11)))+(((IkReal(0.550000000000000))*(sj2)*(x924)))+(((sj2)*(x920)*(x924)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x922)*(x923))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst13)*(((((IkReal(0.0450000000000000))*(pz)*(x923)))+(((x920)*(x925)))+(((IkReal(-1.00000000000000))*(x921)*(x922)*((px)*(px))))+(((cj0)*(cj2)*(py)*(x924)))+(((IkReal(0.550000000000000))*(x925)))+(((cj0)*(x921)*((py)*(py))))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((x925)*(x926)))+(((IkReal(-1.00000000000000))*(pz)*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x922)))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x921))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst13)*(((((px)*(pz)*(x921)))+(((x920)*(x929)))+(((sj2)*(x924)*(x926)))+(((IkReal(0.0450000000000000))*(x923)*(x924)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x923)))+(((IkReal(0.550000000000000))*(x929)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x922)))+(((x926)*(x929)))+(((pz)*(r11)))+(((IkReal(0.550000000000000))*(sj2)*(x924)))+(((sj2)*(x920)*(x924)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x922)*(x923)))))), ((gconst13)*(((((IkReal(0.0450000000000000))*(pz)*(x923)))+(((x920)*(x925)))+(((IkReal(-1.00000000000000))*(x921)*(x922)*((px)*(px))))+(((cj0)*(cj2)*(py)*(x924)))+(((IkReal(0.550000000000000))*(x925)))+(((cj0)*(x921)*((py)*(py))))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((x925)*(x926)))+(((IkReal(-1.00000000000000))*(pz)*(x923)*(x927)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x922)))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x921)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[6];
IkReal x930=IKsin(j1);
IkReal x931=IKcos(j1);
IkReal x932=((px)*(sj2));
IkReal x933=((IkReal(0.0900000000000000))*(sj0));
IkReal x934=((cj2)*(px));
IkReal x935=((IkReal(0.0450000000000000))*(cj3));
IkReal x936=((r11)*(sj2));
IkReal x937=((cj3)*(r22));
IkReal x938=((r00)*(sj3));
IkReal x939=((py)*(sj0));
IkReal x940=((IkReal(0.350000000000000))*(sj3));
IkReal x941=((cj3)*(r00));
IkReal x942=((IkReal(1.00000000000000))*(cj0));
IkReal x943=((r22)*(sj3));
IkReal x944=((IkReal(0.0450000000000000))*(sj3));
IkReal x945=((IkReal(0.350000000000000))*(cj3));
IkReal x946=((py)*(sj2));
IkReal x947=((cj0)*(x931));
IkReal x948=((cj2)*(x930));
IkReal x949=((cj2)*(x931));
IkReal x950=((IkReal(1.00000000000000))*(x931));
IkReal x951=((IkReal(1.00000000000000))*(x930));
IkReal x952=((pz)*(x931));
IkReal x953=((cj0)*(px)*(x930));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x945)))+(((IkReal(-1.00000000000000))*(x944)))+(((x930)*(x939)))+(((IkReal(-1.00000000000000))*(x937)))+(x938)+(x953)+(x952));
evalcond[1]=((((x932)*(x947)))+(((sj0)*(x934)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x942)))+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x951)))+(((sj2)*(x931)*(x939)))+(r11));
evalcond[2]=((IkReal(0.0450000000000000))+(((sj0)*(x932)))+(((IkReal(-1.00000000000000))*(x939)*(x949)))+(((IkReal(-1.00000000000000))*(x942)*(x946)))+(((IkReal(-1.00000000000000))*(x931)*(x934)*(x942)))+(x940)+(x941)+(x943)+(((pz)*(x948)))+(((IkReal(-1.00000000000000))*(x935))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((py)*(x933)*(x949)))+(((IkReal(-0.0900000000000000))*(pz)*(x948)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(0.0900000000000000))*(cj0)*(x946)))+(((IkReal(1.10000000000000))*(x930)*(x939)))+(((IkReal(0.0900000000000000))*(x934)*(x947)))+(((IkReal(1.10000000000000))*(x952)))+(((IkReal(1.10000000000000))*(x953)))+(((IkReal(-1.00000000000000))*(x932)*(x933)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((IkReal(-0.550000000000000))*(x931)))+(((x941)*(x948)))+(((x943)*(x948)))+(((x931)*(x938)))+(pz)+(((IkReal(-1.00000000000000))*(x936)*(x951)))+(((IkReal(-1.00000000000000))*(x937)*(x950)))+(((IkReal(-1.00000000000000))*(x931)*(x944)))+(((IkReal(-1.00000000000000))*(x931)*(x945)))+(((x940)*(x948)))+(((IkReal(-1.00000000000000))*(x935)*(x948)))+(((IkReal(0.0450000000000000))*(x948))));
evalcond[5]=((((IkReal(-1.00000000000000))*(x938)*(x951)))+(((x941)*(x949)))+(((x930)*(x945)))+(((x930)*(x944)))+(((x943)*(x949)))+(((x930)*(x937)))+(((IkReal(0.550000000000000))*(x930)))+(((IkReal(-1.00000000000000))*(x939)))+(((IkReal(-1.00000000000000))*(x936)*(x950)))+(((IkReal(-1.00000000000000))*(px)*(x942)))+(((x940)*(x949)))+(((IkReal(-1.00000000000000))*(x935)*(x949)))+(((IkReal(0.0450000000000000))*(x949))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}

} else
{
{
IkReal j0array[2], cj0array[2], sj0array[2];
bool j0valid[2]={false};
_nj0 = 2;
IkReal x954=((sj2)*(sj3));
IkReal x955=((IkReal(0.0450000000000000))*(sj2));
if( IKabs(((IkReal(-1.00000000000000))*(py))) < IKFAST_ATAN2_MAGTHRESH && IKabs(px) < IKFAST_ATAN2_MAGTHRESH )
    continue;
IkReal x956=((IkReal(1.00000000000000))*(IKatan2(((IkReal(-1.00000000000000))*(py)), px)));
if( ((((px)*(px))+((py)*(py)))) < (IkReal)-0.00001 )
    continue;
if( (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((r22)*(x954)))+(((IkReal(0.350000000000000))*(x954)))+(((IkReal(-1.00000000000000))*(cj3)*(x955)))+(((cj2)*(r11)))+(((cj3)*(r00)*(sj2)))+(x955))))) < -1-IKFAST_SINCOS_THRESH || (((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((r22)*(x954)))+(((IkReal(0.350000000000000))*(x954)))+(((IkReal(-1.00000000000000))*(cj3)*(x955)))+(((cj2)*(r11)))+(((cj3)*(r00)*(sj2)))+(x955))))) > 1+IKFAST_SINCOS_THRESH )
    continue;
IkReal x957=IKasin(((((IKabs(IKabs(IKsqrt((((px)*(px))+((py)*(py)))))) != 0)?((IkReal)1/(IKabs(IKsqrt((((px)*(px))+((py)*(py))))))):(IkReal)1.0e30))*(((((r22)*(x954)))+(((IkReal(0.350000000000000))*(x954)))+(((IkReal(-1.00000000000000))*(cj3)*(x955)))+(((cj2)*(r11)))+(((cj3)*(r00)*(sj2)))+(x955)))));
j0array[0]=((((IkReal(-1.00000000000000))*(x957)))+(((IkReal(-1.00000000000000))*(x956))));
sj0array[0]=IKsin(j0array[0]);
cj0array[0]=IKcos(j0array[0]);
j0array[1]=((IkReal(3.14159265358979))+(((IkReal(-1.00000000000000))*(x956)))+(x957));
sj0array[1]=IKsin(j0array[1]);
cj0array[1]=IKcos(j0array[1]);
if( j0array[0] > IKPI )
{
    j0array[0]-=IK2PI;
}
else if( j0array[0] < -IKPI )
{    j0array[0]+=IK2PI;
}
j0valid[0] = true;
if( j0array[1] > IKPI )
{
    j0array[1]-=IK2PI;
}
else if( j0array[1] < -IKPI )
{    j0array[1]+=IK2PI;
}
j0valid[1] = true;
for(int ij0 = 0; ij0 < 2; ++ij0)
{
if( !j0valid[ij0] )
{
    continue;
}
_ij0[0] = ij0; _ij0[1] = -1;
for(int iij0 = ij0+1; iij0 < 2; ++iij0)
{
if( j0valid[iij0] && IKabs(cj0array[ij0]-cj0array[iij0]) < IKFAST_SOLUTION_THRESH && IKabs(sj0array[ij0]-sj0array[iij0]) < IKFAST_SOLUTION_THRESH )
{
    j0valid[iij0]=false; _ij0[1] = iij0; break; 
}
}
j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];

{
IkReal dummyeval[1];
IkReal gconst13;
gconst13=IKsign(((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
{
IkReal evalcond[4];
IkReal x958=((IkReal(0.0900000000000000))*(r00));
IkReal x959=((r11)+(((px)*(sj0)))+(((IkReal(-1.00000000000000))*(cj0)*(py))));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(3.14159265358979))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=x959;
evalcond[2]=x959;
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((cj3)*(x958)))+(((IkReal(-1.00000000000000))*(x958)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst14;
gconst14=IKsign(((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0))))));
dummyeval[0]=((((IkReal(-1.00000000000000))*((cj0)*(cj0))*((px)*(px))))+(((IkReal(-1.00000000000000))*((pz)*(pz))))+(((IkReal(-2.00000000000000))*(cj0)*(px)*(py)*(sj0)))+(((IkReal(-1.00000000000000))*((py)*(py))*((sj0)*(sj0)))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x960=((pz)*(sj3));
IkReal x961=((py)*(sj0));
IkReal x962=((IkReal(1.00000000000000))*(cj3));
IkReal x963=((cj0)*(px));
IkReal x964=((r00)*(sj3));
IkReal x965=((IkReal(0.350000000000000))*(cj3));
IkReal x966=((IkReal(0.350000000000000))*(sj3));
IkReal x967=((IkReal(0.0450000000000000))*(sj3));
IkReal x968=((IkReal(0.0450000000000000))*(pz));
IkReal x969=((IkReal(0.0450000000000000))*(cj3));
IkReal x970=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst14)*(((((IkReal(-1.00000000000000))*(r22)*(x961)*(x962)))+(((IkReal(-0.550000000000000))*(x963)))+(((IkReal(-0.550000000000000))*(x961)))+(((IkReal(-1.00000000000000))*(cj3)*(x968)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(x961)*(x967)))+(((IkReal(-1.00000000000000))*(x961)*(x965)))+(((IkReal(0.350000000000000))*(x960)))+(((IkReal(-1.00000000000000))*(x963)*(x965)))+(((IkReal(-1.00000000000000))*(x963)*(x967)))+(((x961)*(x964)))+(((x963)*(x964)))+(((IkReal(-1.00000000000000))*(r22)*(x962)*(x963)))+(x968)+(((r22)*(x960))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x963)*(x970)))+(((IkReal(-1.00000000000000))*(pz)*(x965)))+(((IkReal(-1.00000000000000))*(x961)*(x966)))+(((IkReal(-0.0450000000000000))*(x960)))+(((IkReal(-0.0450000000000000))*(x961)))+(((IkReal(-0.0450000000000000))*(x963)))+(((IkReal(-1.00000000000000))*(x963)*(x966)))+(((x961)*(x969)))+(((r00)*(x960)))+(((IkReal(-1.00000000000000))*(r00)*(x961)*(x962)))+(((x963)*(x969)))+(((IkReal(-1.00000000000000))*(r00)*(x962)*(x963)))+(((IkReal(-1.00000000000000))*(x961)*(x970)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x962))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst14)*(((((IkReal(-1.00000000000000))*(r22)*(x961)*(x962)))+(((IkReal(-0.550000000000000))*(x963)))+(((IkReal(-0.550000000000000))*(x961)))+(((IkReal(-1.00000000000000))*(cj3)*(x968)))+(((cj3)*(pz)*(r00)))+(((IkReal(-1.00000000000000))*(x961)*(x967)))+(((IkReal(-1.00000000000000))*(x961)*(x965)))+(((IkReal(0.350000000000000))*(x960)))+(((IkReal(-1.00000000000000))*(x963)*(x965)))+(((IkReal(-1.00000000000000))*(x963)*(x967)))+(((x961)*(x964)))+(((x963)*(x964)))+(((IkReal(-1.00000000000000))*(r22)*(x962)*(x963)))+(x968)+(((r22)*(x960)))))), ((gconst14)*(((((IkReal(-0.550000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(x963)*(x970)))+(((IkReal(-1.00000000000000))*(pz)*(x965)))+(((IkReal(-1.00000000000000))*(x961)*(x966)))+(((IkReal(-0.0450000000000000))*(x960)))+(((IkReal(-0.0450000000000000))*(x961)))+(((IkReal(-0.0450000000000000))*(x963)))+(((IkReal(-1.00000000000000))*(x963)*(x966)))+(((x961)*(x969)))+(((r00)*(x960)))+(((IkReal(-1.00000000000000))*(r00)*(x961)*(x962)))+(((x963)*(x969)))+(((IkReal(-1.00000000000000))*(r00)*(x962)*(x963)))+(((IkReal(-1.00000000000000))*(x961)*(x970)))+(((IkReal(-1.00000000000000))*(pz)*(r22)*(x962)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x971=IKsin(j1);
IkReal x972=IKcos(j1);
IkReal x973=((r22)*(sj3));
IkReal x974=((IkReal(0.0450000000000000))*(cj3));
IkReal x975=((cj0)*(px));
IkReal x976=((cj3)*(r22));
IkReal x977=((r00)*(sj3));
IkReal x978=((py)*(sj0));
IkReal x979=((IkReal(0.350000000000000))*(sj3));
IkReal x980=((IkReal(0.350000000000000))*(cj3));
IkReal x981=((IkReal(0.0450000000000000))*(sj3));
IkReal x982=((cj3)*(r00));
IkReal x983=((IkReal(1.00000000000000))*(x972));
IkReal x984=((sj3)*(x972));
IkReal x985=((IkReal(0.0900000000000000))*(x972));
IkReal x986=((pz)*(x971));
IkReal x987=((pz)*(x972));
IkReal x988=((IkReal(1.10000000000000))*(x971));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-1.00000000000000))*(x981)))+(((IkReal(-1.00000000000000))*(x980)))+(((IkReal(-1.00000000000000))*(x976)))+(((x971)*(x978)))+(((x971)*(x975)))+(x987)+(x977));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x978)*(x983)))+(((IkReal(-1.00000000000000))*(x975)*(x983)))+(x986)+(x982)+(x973)+(x979)+(((IkReal(-1.00000000000000))*(x974))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+(((x975)*(x988)))+(((x975)*(x985)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(-0.0900000000000000))*(x986)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(x987)))+(((IkReal(-0.0900000000000000))*(r00)))+(((x978)*(x985)))+(((x978)*(x988))));
evalcond[3]=((((IkReal(-0.550000000000000))*(x972)))+(((IkReal(0.0450000000000000))*(x971)))+(((IkReal(-1.00000000000000))*(x971)*(x974)))+(pz)+(((x971)*(x979)))+(((x971)*(x973)))+(((IkReal(-1.00000000000000))*(x972)*(x981)))+(((IkReal(-1.00000000000000))*(x972)*(x980)))+(((x971)*(x982)))+(((IkReal(-1.00000000000000))*(x976)*(x983)))+(((x972)*(x977))));
evalcond[4]=((((IkReal(0.550000000000000))*(x971)))+(((x972)*(x982)))+(((IkReal(0.0450000000000000))*(x972)))+(((IkReal(-1.00000000000000))*(x978)))+(((IkReal(-1.00000000000000))*(x975)))+(((IkReal(-1.00000000000000))*(x971)*(x977)))+(((x971)*(x976)))+(((IkReal(-1.00000000000000))*(x972)*(x974)))+(((x971)*(x981)))+(((x971)*(x980)))+(((x972)*(x973)))+(((x972)*(x979))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
IkReal x989=((IkReal(0.0900000000000000))*(r00));
IkReal x990=((px)*(sj0));
IkReal x991=((cj0)*(py));
evalcond[0]=((IkReal(-3.14159265358979))+(IKfmod(((IkReal(1.11022302462516e-16))+(j2)), IkReal(6.28318530717959))));
evalcond[1]=((((IkReal(-1.00000000000000))*(x990)))+(r11)+(x991));
evalcond[2]=((((IkReal(-1.00000000000000))*(x991)))+(((IkReal(-1.00000000000000))*(r11)))+(x990));
evalcond[3]=((IkReal(0.429050000000000))+(((IkReal(0.380950000000000))*(cj3)))+(((IkReal(-1.00000000000000))*(x989)))+(((IkReal(0.0810000000000000))*(sj3)))+(((IkReal(0.700000000000000))*(r22)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(0.0900000000000000))*(r22)*(sj3)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(1.10000000000000))*(cj3)*(r22)))+(((cj3)*(x989)))+(((IkReal(-1.10000000000000))*(r00)*(sj3))));
if( IKabs(evalcond[0]) < 0.0000010000000000  && IKabs(evalcond[1]) < 0.0000010000000000  && IKabs(evalcond[2]) < 0.0000010000000000  && IKabs(evalcond[3]) < 0.0000010000000000  )
{
{
IkReal dummyeval[1];
IkReal gconst15;
gconst15=IKsign((((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0)))));
dummyeval[0]=(((((py)*(py))*((sj0)*(sj0))))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px))))+(((IkReal(2.00000000000000))*(cj0)*(px)*(py)*(sj0))));
if( IKabs(dummyeval[0]) < 0.0000010000000000  )
{
continue;

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x992=((pz)*(sj3));
IkReal x993=((py)*(sj0));
IkReal x994=((IkReal(1.00000000000000))*(r00));
IkReal x995=((cj0)*(px));
IkReal x996=((IkReal(0.350000000000000))*(cj3));
IkReal x997=((IkReal(0.350000000000000))*(sj3));
IkReal x998=((IkReal(0.0450000000000000))*(sj3));
IkReal x999=((IkReal(0.0450000000000000))*(cj3));
IkReal x1000=((cj3)*(r22));
IkReal x1001=((IkReal(1.00000000000000))*(r22)*(sj3));
if( IKabs(((gconst15)*(((((x993)*(x998)))+(((x993)*(x996)))+(((x1000)*(x995)))+(((x1000)*(x993)))+(((cj3)*(pz)*(r00)))+(((IkReal(0.0450000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x999)))+(((IkReal(-1.00000000000000))*(sj3)*(x993)*(x994)))+(((x995)*(x996)))+(((x995)*(x998)))+(((r22)*(x992)))+(((IkReal(0.350000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(sj3)*(x994)*(x995)))+(((IkReal(0.550000000000000))*(x993)))+(((IkReal(0.550000000000000))*(x995))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst15)*(((((x993)*(x999)))+(((pz)*(x996)))+(((IkReal(-1.00000000000000))*(x992)*(x994)))+(((IkReal(-1.00000000000000))*(cj3)*(x994)*(x995)))+(((IkReal(-1.00000000000000))*(x995)*(x997)))+(((IkReal(0.0450000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(x1001)*(x995)))+(((IkReal(-1.00000000000000))*(x1001)*(x993)))+(((pz)*(x1000)))+(((x995)*(x999)))+(((IkReal(-0.0450000000000000))*(x993)))+(((IkReal(-0.0450000000000000))*(x995)))+(((IkReal(-1.00000000000000))*(x993)*(x997)))+(((IkReal(-1.00000000000000))*(cj3)*(x993)*(x994)))+(((IkReal(0.550000000000000))*(pz))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst15)*(((((x993)*(x998)))+(((x993)*(x996)))+(((x1000)*(x995)))+(((x1000)*(x993)))+(((cj3)*(pz)*(r00)))+(((IkReal(0.0450000000000000))*(pz)))+(((IkReal(-1.00000000000000))*(pz)*(x999)))+(((IkReal(-1.00000000000000))*(sj3)*(x993)*(x994)))+(((x995)*(x996)))+(((x995)*(x998)))+(((r22)*(x992)))+(((IkReal(0.350000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(sj3)*(x994)*(x995)))+(((IkReal(0.550000000000000))*(x993)))+(((IkReal(0.550000000000000))*(x995)))))), ((gconst15)*(((((x993)*(x999)))+(((pz)*(x996)))+(((IkReal(-1.00000000000000))*(x992)*(x994)))+(((IkReal(-1.00000000000000))*(cj3)*(x994)*(x995)))+(((IkReal(-1.00000000000000))*(x995)*(x997)))+(((IkReal(0.0450000000000000))*(x992)))+(((IkReal(-1.00000000000000))*(x1001)*(x995)))+(((IkReal(-1.00000000000000))*(x1001)*(x993)))+(((pz)*(x1000)))+(((x995)*(x999)))+(((IkReal(-0.0450000000000000))*(x993)))+(((IkReal(-0.0450000000000000))*(x995)))+(((IkReal(-1.00000000000000))*(x993)*(x997)))+(((IkReal(-1.00000000000000))*(cj3)*(x993)*(x994)))+(((IkReal(0.550000000000000))*(pz)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[5];
IkReal x1002=IKsin(j1);
IkReal x1003=IKcos(j1);
IkReal x1004=((IkReal(1.00000000000000))*(r22));
IkReal x1005=((IkReal(0.0450000000000000))*(cj3));
IkReal x1006=((cj0)*(px));
IkReal x1007=((r00)*(sj3));
IkReal x1008=((IkReal(1.00000000000000))*(r00));
IkReal x1009=((py)*(sj0));
IkReal x1010=((sj3)*(x1002));
IkReal x1011=((cj3)*(x1003));
IkReal x1012=((cj3)*(x1002));
IkReal x1013=((sj3)*(x1003));
IkReal x1014=((IkReal(0.0900000000000000))*(x1003));
IkReal x1015=((pz)*(x1002));
IkReal x1016=((pz)*(x1003));
IkReal x1017=((IkReal(1.10000000000000))*(x1002));
evalcond[0]=((IkReal(-0.550000000000000))+(((IkReal(-0.0450000000000000))*(sj3)))+(((IkReal(-0.350000000000000))*(cj3)))+(x1016)+(x1007)+(((IkReal(-1.00000000000000))*(cj3)*(x1004)))+(((x1002)*(x1006)))+(((x1002)*(x1009))));
evalcond[1]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x1005)))+(((r22)*(sj3)))+(((IkReal(0.350000000000000))*(sj3)))+(((cj3)*(r00)))+(((x1003)*(x1006)))+(((x1003)*(x1009)))+(((IkReal(-1.00000000000000))*(x1015))));
evalcond[2]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+(((x1009)*(x1017)))+(((IkReal(-1.00000000000000))*(x1009)*(x1014)))+(((IkReal(0.0900000000000000))*(x1015)))+((r22)*(r22))+(((IkReal(1.10000000000000))*(x1016)))+((r11)*(r11))+(((IkReal(-1.00000000000000))*(x1006)*(x1014)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((x1006)*(x1017)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[3]=((((IkReal(-0.550000000000000))*(x1003)))+(((IkReal(-0.350000000000000))*(x1010)))+(((IkReal(-0.350000000000000))*(x1011)))+(((IkReal(-0.0450000000000000))*(x1002)))+(((IkReal(-0.0450000000000000))*(x1013)))+(((IkReal(-1.00000000000000))*(x1004)*(x1010)))+(((IkReal(-1.00000000000000))*(x1004)*(x1011)))+(((IkReal(-1.00000000000000))*(x1008)*(x1012)))+(pz)+(((x1002)*(x1005)))+(((x1003)*(x1007))));
evalcond[4]=((((IkReal(-0.350000000000000))*(x1013)))+(((IkReal(-1.00000000000000))*(x1009)))+(((IkReal(-1.00000000000000))*(x1006)))+(((IkReal(-1.00000000000000))*(x1002)*(x1007)))+(((IkReal(-0.0450000000000000))*(x1003)))+(((r22)*(x1012)))+(((IkReal(-1.00000000000000))*(x1004)*(x1013)))+(((IkReal(-1.00000000000000))*(x1008)*(x1011)))+(((IkReal(0.550000000000000))*(x1002)))+(((IkReal(0.350000000000000))*(x1012)))+(((IkReal(0.0450000000000000))*(x1010)))+(((x1003)*(x1005))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}

} else
{
if( 1 )
{
continue;

} else
{
}
}
}
}

} else
{
{
IkReal j1array[1], cj1array[1], sj1array[1];
bool j1valid[1]={false};
_nj1 = 1;
IkReal x1018=((cj3)*(r22));
IkReal x1019=((cj2)*(sj0));
IkReal x1020=((IkReal(1.00000000000000))*(cj0));
IkReal x1021=((sj2)*(sj3));
IkReal x1022=((cj0)*(px));
IkReal x1023=((pz)*(sj2));
IkReal x1024=((IkReal(0.350000000000000))*(cj3));
IkReal x1025=((IkReal(1.00000000000000))*(r00));
IkReal x1026=((IkReal(1.00000000000000))*(cj2)*(py));
IkReal x1027=((py)*(sj0)*(sj2));
if( IKabs(((gconst13)*(((((px)*(pz)*(x1019)))+(((sj2)*(x1022)*(x1024)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x1021)*(x1025)))+(((IkReal(0.550000000000000))*(sj2)*(x1022)))+(((x1018)*(x1027)))+(((IkReal(0.550000000000000))*(x1027)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x1020)*(x1021)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x1020)))+(((x1024)*(x1027)))+(((pz)*(r11)))+(((sj2)*(x1018)*(x1022)))+(((IkReal(0.0450000000000000))*(x1021)*(x1022)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x1021))))))) < IKFAST_ATAN2_MAGTHRESH && IKabs(((gconst13)*(((((cj0)*(x1019)*((py)*(py))))+(((IkReal(0.0450000000000000))*(pz)*(x1021)))+(((x1023)*(x1024)))+(((cj0)*(cj2)*(py)*(x1022)))+(((x1018)*(x1023)))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x1019)))+(((IkReal(0.550000000000000))*(x1023)))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)*((px)*(px))))+(((IkReal(-1.00000000000000))*(pz)*(x1021)*(x1025)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x1020))))))) < IKFAST_ATAN2_MAGTHRESH )
    continue;
j1array[0]=IKatan2(((gconst13)*(((((px)*(pz)*(x1019)))+(((sj2)*(x1022)*(x1024)))+(((IkReal(-1.00000000000000))*(py)*(sj0)*(x1021)*(x1025)))+(((IkReal(0.550000000000000))*(sj2)*(x1022)))+(((x1018)*(x1027)))+(((IkReal(0.550000000000000))*(x1027)))+(((IkReal(-1.00000000000000))*(px)*(r00)*(x1020)*(x1021)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(pz)*(x1020)))+(((x1024)*(x1027)))+(((pz)*(r11)))+(((sj2)*(x1018)*(x1022)))+(((IkReal(0.0450000000000000))*(x1021)*(x1022)))+(((IkReal(0.0450000000000000))*(py)*(sj0)*(x1021)))))), ((gconst13)*(((((cj0)*(x1019)*((py)*(py))))+(((IkReal(0.0450000000000000))*(pz)*(x1021)))+(((x1023)*(x1024)))+(((cj0)*(cj2)*(py)*(x1022)))+(((x1018)*(x1023)))+(((IkReal(-1.00000000000000))*(px)*(py)*(sj0)*(x1019)))+(((IkReal(0.550000000000000))*(x1023)))+(((IkReal(-1.00000000000000))*(py)*(r11)*(sj0)))+(((IkReal(-1.00000000000000))*(x1019)*(x1020)*((px)*(px))))+(((IkReal(-1.00000000000000))*(pz)*(x1021)*(x1025)))+(((IkReal(-1.00000000000000))*(px)*(r11)*(x1020)))))));
sj1array[0]=IKsin(j1array[0]);
cj1array[0]=IKcos(j1array[0]);
if( j1array[0] > IKPI )
{
    j1array[0]-=IK2PI;
}
else if( j1array[0] < -IKPI )
{    j1array[0]+=IK2PI;
}
j1valid[0] = true;
for(int ij1 = 0; ij1 < 1; ++ij1)
{
if( !j1valid[ij1] )
{
    continue;
}
_ij1[0] = ij1; _ij1[1] = -1;
for(int iij1 = ij1+1; iij1 < 1; ++iij1)
{
if( j1valid[iij1] && IKabs(cj1array[ij1]-cj1array[iij1]) < IKFAST_SOLUTION_THRESH && IKabs(sj1array[ij1]-sj1array[iij1]) < IKFAST_SOLUTION_THRESH )
{
    j1valid[iij1]=false; _ij1[1] = iij1; break; 
}
}
j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
{
IkReal evalcond[6];
IkReal x1028=IKsin(j1);
IkReal x1029=IKcos(j1);
IkReal x1030=((px)*(sj2));
IkReal x1031=((IkReal(0.0900000000000000))*(sj0));
IkReal x1032=((cj2)*(px));
IkReal x1033=((IkReal(0.0450000000000000))*(cj3));
IkReal x1034=((r11)*(sj2));
IkReal x1035=((cj3)*(r22));
IkReal x1036=((r00)*(sj3));
IkReal x1037=((py)*(sj0));
IkReal x1038=((IkReal(0.350000000000000))*(sj3));
IkReal x1039=((cj3)*(r00));
IkReal x1040=((IkReal(1.00000000000000))*(cj0));
IkReal x1041=((r22)*(sj3));
IkReal x1042=((IkReal(0.0450000000000000))*(sj3));
IkReal x1043=((IkReal(0.350000000000000))*(cj3));
IkReal x1044=((py)*(sj2));
IkReal x1045=((cj0)*(x1029));
IkReal x1046=((cj2)*(x1028));
IkReal x1047=((cj2)*(x1029));
IkReal x1048=((IkReal(1.00000000000000))*(x1029));
IkReal x1049=((IkReal(1.00000000000000))*(x1028));
IkReal x1050=((pz)*(x1029));
IkReal x1051=((cj0)*(px)*(x1028));
evalcond[0]=((IkReal(-0.550000000000000))+(x1050)+(x1051)+(x1036)+(((IkReal(-1.00000000000000))*(x1035)))+(((x1028)*(x1037)))+(((IkReal(-1.00000000000000))*(x1042)))+(((IkReal(-1.00000000000000))*(x1043))));
evalcond[1]=((((sj0)*(x1032)))+(((x1030)*(x1045)))+(((sj2)*(x1029)*(x1037)))+(r11)+(((IkReal(-1.00000000000000))*(pz)*(sj2)*(x1049)))+(((IkReal(-1.00000000000000))*(cj2)*(py)*(x1040))));
evalcond[2]=((IkReal(0.0450000000000000))+(((IkReal(-1.00000000000000))*(x1033)))+(x1041)+(x1038)+(x1039)+(((IkReal(-1.00000000000000))*(x1029)*(x1032)*(x1040)))+(((IkReal(-1.00000000000000))*(x1037)*(x1047)))+(((sj0)*(x1030)))+(((pz)*(x1046)))+(((IkReal(-1.00000000000000))*(x1040)*(x1044))));
evalcond[3]=((IkReal(-0.180000000000000))+(((IkReal(0.700000000000000))*(r22)))+(((IkReal(-0.0900000000000000))*(pz)*(x1046)))+(((IkReal(0.0900000000000000))*(x1032)*(x1045)))+((r22)*(r22))+((r11)*(r11))+(((IkReal(1.10000000000000))*(x1050)))+(((IkReal(1.10000000000000))*(x1051)))+((r00)*(r00))+(((IkReal(-1.00000000000000))*(pp)))+(((IkReal(-1.00000000000000))*(x1030)*(x1031)))+(((IkReal(0.0900000000000000))*(cj0)*(x1044)))+(((IkReal(1.10000000000000))*(x1028)*(x1037)))+(((py)*(x1031)*(x1047)))+(((IkReal(-0.0900000000000000))*(r00))));
evalcond[4]=((((x1038)*(x1046)))+(((IkReal(-0.550000000000000))*(x1029)))+(((IkReal(-1.00000000000000))*(x1033)*(x1046)))+(((x1039)*(x1046)))+(((x1041)*(x1046)))+(((IkReal(-1.00000000000000))*(x1029)*(x1042)))+(((IkReal(-1.00000000000000))*(x1029)*(x1043)))+(((x1029)*(x1036)))+(pz)+(((IkReal(-1.00000000000000))*(x1035)*(x1048)))+(((IkReal(-1.00000000000000))*(x1034)*(x1049)))+(((IkReal(0.0450000000000000))*(x1046))));
evalcond[5]=((((x1038)*(x1047)))+(((IkReal(-1.00000000000000))*(x1037)))+(((IkReal(-1.00000000000000))*(x1033)*(x1047)))+(((x1039)*(x1047)))+(((x1041)*(x1047)))+(((IkReal(-1.00000000000000))*(px)*(x1040)))+(((x1028)*(x1035)))+(((IkReal(0.550000000000000))*(x1028)))+(((x1028)*(x1043)))+(((x1028)*(x1042)))+(((IkReal(-1.00000000000000))*(x1034)*(x1048)))+(((IkReal(-1.00000000000000))*(x1036)*(x1049)))+(((IkReal(0.0450000000000000))*(x1047))));
if( IKabs(evalcond[0]) > 0.000001  || IKabs(evalcond[1]) > 0.000001  || IKabs(evalcond[2]) > 0.000001  || IKabs(evalcond[3]) > 0.000001  || IKabs(evalcond[4]) > 0.000001  || IKabs(evalcond[5]) > 0.000001  )
{
continue;
}
}

{
std::vector<IkSingleDOFSolutionBase<IkReal> > vinfos(4);
vinfos[0].jointtype = 1;
vinfos[0].foffset = j0;
vinfos[0].indices[0] = _ij0[0];
vinfos[0].indices[1] = _ij0[1];
vinfos[0].maxsolutions = _nj0;
vinfos[1].jointtype = 1;
vinfos[1].foffset = j1;
vinfos[1].indices[0] = _ij1[0];
vinfos[1].indices[1] = _ij1[1];
vinfos[1].maxsolutions = _nj1;
vinfos[2].jointtype = 1;
vinfos[2].foffset = j2;
vinfos[2].indices[0] = _ij2[0];
vinfos[2].indices[1] = _ij2[1];
vinfos[2].maxsolutions = _nj2;
vinfos[3].jointtype = 1;
vinfos[3].foffset = j3;
vinfos[3].indices[0] = _ij3[0];
vinfos[3].indices[1] = _ij3[1];
vinfos[3].maxsolutions = _nj3;
std::vector<int> vfree(0);
solutions.AddSolution(vinfos,vfree);
}
}
}

}

}
}
}

}

}
}
}

}

}
}
return solutions.GetNumSolutions()>0;
}
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ComputeIk(const IkReal* eetrans, const IkReal* eerot, const IkReal* pfree, IkSolutionListBase<IkReal>& solutions) {
IKSolver solver;
return solver.ComputeIk(eetrans,eerot,pfree,solutions);
}

IKFAST_API const char* GetKinematicsHash() { return "4382fadc0284794e13f46b1dd48b52ef"; }

IKFAST_API const char* GetIkFastVersion() { return IKFAST_STRINGIZE(IKFAST_VERSION); }

#ifdef IKFAST_NAMESPACE
} // end namespace
#endif

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>
#ifdef IKFAST_NAMESPACE
using namespace IKFAST_NAMESPACE;
#endif
int main(int argc, char** argv)
{
    if( argc != 12+GetNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",GetNumFreeParameters());
        return 1;
    }

    IkSolutionList<IkReal> solutions;
    std::vector<IkReal> vfree(GetNumFreeParameters());
    IkReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(std::size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ComputeIk(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, solutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)solutions.GetNumSolutions());
    std::vector<IkReal> solvalues(GetNumJoints());
    for(std::size_t i = 0; i < solutions.GetNumSolutions(); ++i) {
        const IkSolutionBase<IkReal>& sol = solutions.GetSolution(i);
        printf("sol%d (free=%d): ", (int)i, (int)sol.GetFree().size());
        std::vector<IkReal> vsolfree(sol.GetFree().size());
        sol.GetSolution(&solvalues[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( std::size_t j = 0; j < solvalues.size(); ++j)
            printf("%.15f, ", solvalues[j]);
        printf("\n");
    }
    return 0;
}

#endif
